note

	description:

		"Various externals for dealing with C structures ... predates macro mechanism in eiffel"

	author: "Mark Howard"
	last_editor: "$Author: rj $"
	date: "$Date: 2014/06/19 $"
	revision: "$Revision: #2 $"
	path: "$File: //rose/source/library/external/src/access_externals.e $"

class ACCESS_EXTERNALS

inherit

	MAKESTR

	ROSE_LEGACY_CONSTANTS

feature -- Miscellaneous

	copy_to_pointer (a_target, a_source: POINTER; a_length: INTEGER)
   		-- Copy from `a_source' to `a_target' for `a_length' bytes.
		do
			a_target.memory_copy (a_source, a_length)
		end

	integer_to_short_pointer (a_target: POINTER; a_item: INTEGER)
			-- Convert `a_item' to INTEGER_16 then copy it to `a_target'.
		local
			l_short: INTEGER_16
		do
			l_short := a_item.to_integer_16
			a_target.memory_copy ($l_short, 2)
		end

   short_pointer_to_integer (a_source: POINTER): INTEGER
			-- Extract INTEGER_16 from `a_source'
		local
			l_int16: INTEGER_16
		do
			($l_int16).memory_copy (a_source, 2)
			Result := l_int16.to_integer
		end

   dereference_integer (a_source: POINTER): INTEGER
			-- 32-bit integer at `a_source'
		do
			($Result).memory_copy (a_source, 4)
		end

   dereference_natural_32(a_source: POINTER): NATURAL_32
			-- 32-bit unsigned integer at `a_source'
		do
			($Result).memory_copy(a_source, 4)
		end

   dereference_integer_64 (a_source: POINTER): INTEGER_64
			-- 64-bit integer at `a_source'
		do
			($Result).memory_copy (a_source, 8)
		end

   dereference_short (a_source: POINTER): INTEGER
			-- 16-bit integer at `a_source'
		do
			($Result).memory_copy (a_source, 2)
		end

	dereference_boolean (a_source: POINTER): BOOLEAN
			-- Byte at `a_source' interpreted as `False' if zero, `True' otherwise
		local
   	  	l_character: CHARACTER
		do
			($l_character).memory_copy (a_source, 1)
			Result := l_character.code /= 0
		end

   dereference_real (a_source: POINTER): REAL
			-- 32-bit floating-point value at `a_source'
		do
			($Result).memory_copy (a_source, 4)
		end

   dereference_double (a_source: POINTER): DOUBLE
			-- 64-bit floating-point value at `a_source'
		do
			($Result).memory_copy (a_source, 8)
		end

   dereference_character (a_source: POINTER): CHARACTER
			-- Byte at `a_source'
		do
			($Result).memory_copy (a_source, 1)
		end

   dereference_pointer (a_source: POINTER): POINTER
			-- Pointer stored at `a_source'
		do
			($Result).memory_copy (a_source, {PLATFORM}.pointer_bytes)
		end

   decode_ri (a_iv: POINTER): REAL
			-- Byte at `a_iv' decoded;
			-- Inverse of `encode_ri' (?);
			-- N.B. Only a redefined version is actually called, so an empty body would do just as well.
		local
			l_ar: REAL
			l_ibit7, l_sign, l_code: INTEGER
			l_iv: CHARACTER
		do
			($l_iv).memory_copy (a_iv, 1)
			l_ibit7 := 128
			l_sign := 1
			l_code := l_iv.code
			if l_code = 0 then
				Result := Legacy_missing_real_32
			elseif (l_code = 1) then
				Result := 0
			else
				l_ar := l_code & 127
				if l_code & 128 /= 0 then
					l_sign := -1
				end
				if l_ar < 4 then
					if l_ar = 1.0 and l_sign = -1 then
						Result := l_sign * ({REAL_32} 0.0 + (l_ar) / {REAL_32} 120.0)
					else
						Result := l_sign * ( {REAL_32} 0.0 + (l_ar  - {REAL_32} 1.0) / {REAL_32} 30.0)
					end
				elseif l_ar < 40 then
					Result := l_sign * ({REAL_32} 0.0 + (l_ar) / {REAL_32} 40.0)
				elseif l_ar < 80 then
					Result := l_sign * ({REAL_32} 1.0 + (l_ar - {REAL_32} 40.0) / {REAL_32} 20.0)
				elseif l_ar < 100 then
					Result := l_sign * ({REAL_32} 3.0 + (l_ar - {REAL_32} 80.0) / {REAL_32} 10.0)
				elseif l_ar < 110 then
					Result := l_sign * ({REAL_32} 5.0 + (l_ar - {REAL_32} 100.0) / {REAL_32} 5.0)
				elseif l_ar <= 126 then
					Result := l_sign * ({REAL_32} 7.0 + (l_ar - {REAL_32} 110.0))
				else
					Result := l_sign * {REAL_32} 23.0
				end
			end
		end

	encode_ri (a_ri: POINTER; a_y: REAL)
			-- Encode `a_y' into a byte and store at `a_ri'.
			-- No callers in integration.
		local
			a: REAL
			l_ri: INTEGER
			l_ri_byte: CHARACTER
		do
			if a_y = Legacy_missing_real_32 then
				l_ri := 0
			elseif a_y = 0 then
				l_ri := 1
			else
				a := a_y
				if a < 0 then
					a := -a
				end
				if (a < 0.10) then
					l_ri := (1 + (0.5 + 30.0 * (a))).truncated_to_integer
				elseif a < 1.0 then
					l_ri := ((0.5 + 40.0 * (a))).truncated_to_integer
				elseif (a < 3.0) then
					l_ri := (40 + (0.5 + 20.0 * (a - 1.0))).truncated_to_integer
				elseif (a < 5.0) then
					l_ri := (80 + (0.5 + 10.0 * (a - 3.0))).truncated_to_integer
				elseif (a < 7.0) then
					l_ri := (100 + (0.5 + 5.0 * (a - 5.0))).truncated_to_integer
				elseif (a <= 23.0) then
					l_ri := (110 + (0.5 + (a - 7.0))).truncated_to_integer
				else
					l_ri := 127
				end
				if a_y < 0 then
					l_ri := l_ri | 128
				end
			end
			l_ri_byte := l_ri.to_character_8
			a_ri.memory_copy ($l_ri_byte, 1)
		end

	decode_rtrn (a_real_pointer: POINTER): REAL
			--	16-bits at `a_real_pointer' decoded as a 32-bit floating-point value
		local
			l_sign: INTEGER
			l_short: INTEGER_16
			l_ar: REAL
			x: REAL
		do
			($l_short).memory_copy (a_real_pointer, 2)
			x := l_short.to_integer
			if x = Legacy_missing_real_32 then
				Result := x
			else
				l_ar := x
				if x < 0 then
					l_sign := -1
					l_ar := -x
				else
					l_sign := 1
				end
				if (l_ar < 20000.0) then
					Result := l_sign * ({REAL_32} 0.0 + (l_ar) / {REAL_32} 10000.0)
				elseif (l_ar < 30000.0) then
					Result := l_sign * ({REAL_32} 2.0 + (l_ar - {REAL_32} 20000.0) / {REAL_32} 1000.0)
				elseif (l_ar < 31000.0) then
					Result := l_sign * ({REAL_32} 12.0 + (l_ar - {REAL_32} 30000.0) / {REAL_32} 10.0)
				elseif (l_ar <= 32000.0) then
					Result := l_sign * ({REAL_32} 112.0 + (l_ar - {REAL_32} 31000.0))
				elseif (l_ar >= 32001.0) then
					Result := l_sign * {REAL_32} 1112.0
				end
			end
		end

	encode_rtrn (a_pointer: POINTER; a_y: REAL)
			-- Encode `a_y' as 16-bit integer and store at `a_pointer'.
		local
			a, l_sign: REAL
			l_return: REAL
			l_short: INTEGER_16
		do
			l_sign := 1.0
			l_return := Legacy_missing_real_32
			if (a_y = Legacy_missing_real_32) then
				l_return := a_y
			else
				a := a_y
				if (a_y < 0.0) then
					l_sign := -1.0
					a := -a_y
				end
				if (a < 2.0) then
					l_return := l_sign * (({REAL_32} 0.5 + {REAL_32} 10000.0 * (a)))
					if l_return  = Legacy_missing_real_32 then
						l_return := -998.0
					end
				elseif (a < 12.0) then
					l_return := l_sign * ({REAL_32}20000.+ ({REAL_32} 0.5 + {REAL_32} 1000.0 * (a - {REAL_32} 2.0)))
				elseif (a < 112.0) then
					l_return := l_sign * ({REAL_32} 30000.0 + ({REAL_32} 0.5 + {REAL_32} 10.0 * (a - {REAL_32} 12.0)))
				elseif (a <= 1112.0) then
					l_return := l_sign * ({REAL_32} 31000.0 + ({REAL_32} 0.5 + (a - {REAL_32} 112.0)))
				end
			end
			l_short := l_return.truncated_to_integer.to_integer_16
			a_pointer.memory_copy ($l_short, 2)
		end

   pointer_plus (a_pointer: POINTER; a_offset: INTEGER): POINTER
			-- `a_pointer' incremented by `a_offset'
		do
			Result := a_pointer + a_offset
		end

feature -- Real externals

   all_nulls (a_pointer: POINTER; a_length: INTEGER): BOOLEAN
			-- Does `a_pointer' point to all NULLs for `a_length'?
		external
	  		"C inline use %"rose_clib.h%""
		alias
			"[
				int i;
				char * ptr = (char *) $a_pointer;

 				for (i = 0; i < $a_length; i++)
				{
					if (ptr[i] != 0) return(0);
				}
				return (1);
			]"
		end

	malloc (a_length: INTEGER): POINTER
			-- Newly allocated `a_length' bytes of memory
		local
			p: POINTER
		do
			Result := p.memory_alloc (a_length)
		end

end


