note

	description:

		"AXA Rosenberg inline C routines"

	author: "Colin Adams"
	last_editor: "$Author: it $"
	date: "$Date: 2014/07/30 $"
	revision: "$Revision: #2 $"
	path: "$File: //rose/source/library/external/src/rose_c_library.e $"

class ROSE_C_LIBRARY

feature {NONE} -- Access

	cpu_ticks: INTEGER
			-- CPU ticks since process started
		external
			"C inline use %"rose_clib.h%""
		alias
			"[
				#ifdef _WIN32
					#include <winbase.h>
					#include <windows.h>

					_int64 ticks,*ticks_ptr;
					FILETIME creation,exit,kernel,user;
					BOOL status;

					status = GetProcessTimes(GetCurrentProcess(),&creation,&exit,&kernel,&user);
					ticks = *(_int64 *)&user+*(_int64 *)&kernel;
					ticks = ticks / (10000000/CLOCKS_PER_SEC);
					return(ticks);

				#else /* (not) _WIN32 */
					#include <time.h>

					return clock();

				#endif /* _WIN32 */
			]"
		end

	cast_as_pointer (a_value: ANY): POINTER
			-- `a_value' cast as pointer to char
		external
			"C inline  use %"rose_clib.h%""
		alias
			"[
				return (char *) $a_value;
			]"
		end

	c_vms_short (a_item: POINTER): INTEGER
			-- Two-byte VMS integer value pointed at by `a_item'
		external
			"C inline use %"rose_clib.h%""
		alias
			"[
				char *reverse;
				unsigned short buffer;
				unsigned int item;
				char *ptr;

				ptr = (char *) $a_item;
				reverse = (char *) &buffer;
				reverse [0] = ptr [1];
				reverse [1] = ptr [0];
				item = buffer;
				return (item);
			]"
		end

	c_vms_integer (a_item: POINTER): INTEGER
			-- Four-byte VMS integer value pointed at by `a_item'
		external
			"C inline use %"rose_clib.h%""
		alias
			"[
				char *reverse;
				unsigned int buffer;
				char *ptr;

				ptr = (char *) $a_item;
				reverse = (char *) &buffer;
				reverse [0] = ptr [1];
				reverse [1] = ptr [0];
				reverse [0] = ptr [1];
				reverse [1] = ptr [0];
				return (buffer);
			]"
		end

	integer_to_short_pointer (a_target: POINTER; a_item: INTEGER)
			-- Copy two bytes of `a_item' to location at `a_target'.
		external
			"C inline use %"rose_clib.h%""
		alias
			"[		
				char *target;
				short source;

				target = (char *) $a_target;
				source = $a_item;
				memcpy (target, &source, 2);
			]"
		end

	short_pointer_to_integer (a_source: POINTER): INTEGER
			-- Two-byte integer value at `a_source'
		external
			"C inline use %"rose_clib.h%""
		alias
			"[		
				int value;

				value = *((short *)$a_source);
				return (value);
			]"
		end

	dereference_integer (a_source: POINTER): INTEGER
			-- Four-byte integer value at `a_source'
		external
		"C inline use %"rose_clib.h%""
		alias
			"[		
				return (*((int *)$a_source));
			]"
		end

	dereference_boolean (a_source: POINTER): BOOLEAN
			-- Boolean value at `a_source'
		external
			"C inline use  %"rose_clib.h%",%"eif_eiffel.h%""
		alias
			"[		
				return (*((EIF_BOOLEAN *)$a_source));
			]"
		end

	dereference_real (a_source: POINTER): REAL
			-- Thirty-two bit IEEE 754 floating point value at `a_source'
		external
			"C inline use %"rose_clib.h%""
		alias
			"[		
				return (*((float *)$a_source));
			]"
		end

	dereference_double (a_source: POINTER): DOUBLE
			-- Sixty-four bit IEEE 754 floating point value at `a_source' (??)
		external
			"C inline use %"rose_clib.h%""
		alias
			"[
				#ifdef _WIN32
				#define DSbit        0x0000000000008000
				#define Sbit_2bytes  0x8000
				#define DExp         0x7F80
				#define DFracHi      0x007F
				#define IEEEExp      0x7FF0
				#define NotIEEEExp   0x800F
				#define IEEEFracHi   0x000F
					unsigned short use, look[4];
					double out=0, mantissa=0, in;
					unsigned __int64 use2, temp, look2;

					in = *((double *)$a_source);
					if (in == 0) return (in);

					/* do the mantissa first */
					((unsigned short *)&out)[0] = ((unsigned short *)&in)[3];
					((unsigned short *)&out)[1] = ((unsigned short *)&in)[2];
					((unsigned short *)&out)[2] = ((unsigned short *)&in)[1];
					((unsigned short *)&out)[3] = (((unsigned short *)&in)[0])&DFracHi;
					temp = (*((unsigned __int64 *)&out))>>3;
					*((unsigned __int64 *)&out) = temp;

					/* do the exponent and sign bit */
					use = ((unsigned short *)&in)[0];
					use = use&DExp;
					use = use>>7;

					/*
					 * For the exponent, 894 is the magic number.
					 * It's actually the difference of the 11- and 8-bit biases
					 * 1023 - 129 = 894
					*/
					  use = use+894;
					  use = use<<4;

  					/* handle the sign bit */
					if (look2 = (*((unsigned __int64 *)&in))&DSbit) {
						use=use|Sbit_2bytes;
					}

					/* OK write the most significant byte (contaning exp and sign bit) into place */
					((unsigned short *)&out)[3] = ((unsigned short *)&out)[3]|use;
					return (out);
				#else
					return (*((double *)$a_source));
				#endif
			]"
		end

	dereference_character (a_source: POINTER): CHARACTER
			-- Eight-bit character value at `a_source'
		external
			"C inline use %"rose_clib.h%""
		alias
			"[		
				int r = 0;
				char c;

				c = *((char *)$a_source) & 255UL;
				r = c;

				return (r);
			]"
		end

	dereference_pointer (a_source: POINTER): POINTER
			-- Pointer to 8-bit character  at `a_source'
		external
			"C inline use %"rose_clib.h%""
		alias
			"[		
				return (*(( char **)$a_source));
			]"
		end

end
