note

	description:

		"32 bit routines -- may have to be ported"

	project: "http://www.nenie.org/eiffel/"
	license: "http://www.opensource.org/licenses/mit-license.html"
	exportable: "bit_or, bit_xor, bit_and, bit_not, shift_left, shift_right, add, to_crypt32"
	author: "Franck Arnaud (from pre-AXA Rosenberg days)"
	last_editor: "$Author: c4 $"
	date: "$Date: 2014/02/10 $"
	revision: "$Revision: #2 $"
	path: "$File: //rose/source/third_party/crypto/src/crypt_bit32_private_routines.e $"

class CRYPT_BIT32_PRIVATE_ROUTINES

inherit

	CRYPT_HEX_PRIVATE_ROUTINES
	
feature {NONE} -- 32 bit type with mostly boolean operands

	crypt32: INTEGER_32
			-- Type anchor
		do end 
	
	bit_or (a, b: like crypt32): like crypt32
			-- Bitwise or of `a' and `b'
		do
			Result := a.bit_or (b)
		end
		
	bit_xor (a, b: like crypt32): like crypt32
			-- Bitwise exclusive or of `a' and `b'
		do
			Result := a.bit_xor (b)
		end
		
	bit_and (a, b: like crypt32): like crypt32
			-- Bitwise and of `a' and `b'
		do
			Result := a.bit_and (b)
		end

	bit_not (a: like crypt32): like crypt32
			-- Bitwise not of `a'
		do
			Result := a.bit_not
		end
		
	shift_left (a: like crypt32; n: INTEGER): like crypt32
			-- Shift `a' by `n' bits to the left
		do
			Result := a.bit_shift_left (n)
		end

	shift_right (a: like crypt32; n: INTEGER): like crypt32
			-- Shift `a' by `n' bits to the right
		do
			if a < 0 and n > 0 then
				-- on Smarteiffel bit_shift_right does sign extension
				-- so shift by one, clear top bit, and do remainder
				-- bits on positive result.
				Result := a.bit_shift_right (1)
				Result := Result.bit_and ((1).bit_shift_left (31).bit_not)
				if n > 1 then
					Result := shift_right (Result, n -  1)
				end
			else
				check sign_bit_clear: a >= 0 or n = 0 end
				Result := a.bit_shift_right (n)
			end
		end

	add (a, b: like crypt32): like crypt32
			-- Modulo 32-bit unsigned addition of `a' and `b'
		do
			Result := a + b
		end

	to_crypt32 (a_int: INTEGER): like crypt32
			-- Md5 word from `a_int'
		do
			Result := a_int
		end
	
	crypt32_byte_1 (a: like crypt32): NATURAL_8
			-- First byte of crypt32
		do
			Result := shift_right (shift_left (a,24), 24).as_natural_8
		ensure
			byte: is_byte (Result)
		end
		

	crypt32_byte_2 (a: like crypt32): NATURAL_8
			-- 2nd byte of crypt32
		do
			Result := crypt32_byte_1 (shift_right (a, 8))
		ensure
			byte: is_byte (Result)
		end
		
	crypt32_byte_3 (a: like crypt32): NATURAL_8
			-- 3rd byte of crypt32
		do
			Result := crypt32_byte_1 (shift_right (a, 16))
		ensure
			byte: is_byte (Result)
		end
		
	crypt32_byte_4 (a: like crypt32): NATURAL_8
			-- 4th byte of crypt32
		do
			Result := crypt32_byte_1 (shift_right (a, 24))
		ensure
			byte: is_byte (Result)
		end

	hex (a_str: STRING): like crypt32
			-- For constants
		require
			a_str_attached: a_str /= Void
			count: a_str.count <= 32
		local
			an_index: INTEGER
		do
			from
				an_index := 1
			until
				an_index > a_str.count
			loop
				Result := shift_left (Result, 4)
				Result := bit_or (Result, to_crypt32 (hex_to_int (a_str.item (an_index))))
				an_index := an_index + 1
			end
		end

feature {NONE} -- Byte routines

	is_byte (a: INTEGER): BOOLEAN
			-- Is `a' a byte value?
		do
			Result := a >= 0 and a < 256
		ensure
			define: Result = (a >= 0 and a < 256)
		end
		
	byte_xor (a, b: NATURAL_8): NATURAL_8
			-- Bitwise XOR of bytes `a' and `b'
		do
			Result := a.bit_xor (b)
		end
		
	byte_or (a, b: INTEGER): INTEGER
			-- Bitwise XR of bytes `a' and `b'
		require
			a: is_byte (a)
			b: is_byte (b)
		do
			Result := crypt32_byte_1 (bit_or (to_crypt32 (a), to_crypt32 (b)))
		ensure
			byte: is_byte (Result)
		end
		
	byte_and (a, b: INTEGER): INTEGER
			-- Bitwise AND of bytes `a' and `b'
		require
			a: is_byte (a)
			b: is_byte (b)
		do
			Result := crypt32_byte_1 (bit_and (to_crypt32 (a), to_crypt32 (b)))
		ensure
			byte: is_byte (Result)
		end
		
	byte_not (a: INTEGER): INTEGER
			-- Bitwise NOT of byte `a'
		require
			a: is_byte (a)
		do
			Result := crypt32_byte_1 (bit_not (to_crypt32 (a)))
		ensure
			byte: is_byte (Result)
		end
		
end
