note

	description: "Byte consumer with init/finish"


	author: "Franck Arnaud (from pre-AXA Rosenberg days)"
	last_editor: "$Author: c4 $"
	date: "$Date: 2014/02/10 $"
	revision: "$Revision: #1 $"
	path: "$File: //rose/source/third_party/crypto/src/crypt_byte_consumer.e $"
	project: "http://www.nenie.org/eiffel/"
	license: "http://www.opensource.org/licenses/mit-license.html"

deferred class CRYPT_BYTE_CONSUMER

feature -- Status report

	initialized: BOOLEAN
			-- Has `init' been run?

feature -- Basic specifiers

	init
			-- Reset processing.
		deferred
		ensure
			consumption_started: initialized
		end
		
	update_byte (a_byte: NATURAL_8)
			-- Add `a_byte' to stream.
		require
			consumption_started: initialized
		deferred
		end
	
	final
			-- Terminate processing.
		require
			consumption_started: initialized
		deferred
		end

feature -- Convenience routines

	update_string (a_in_string: STRING)
			-- Call `update_byte' for each character treated a byte.
			-- This assumes character codes fit in BYTE!
		require
			a_in_string: a_in_string /= Void
			all_chars_below_256: across a_in_string as i_string all i_string.item.code < 256 end
			consumption_started: initialized
		local
			j,cnt: INTEGER
		do
			from
				j := 1
				cnt := a_in_string.count 
			invariant
				j <= cnt implies a_in_string.item_code (j) < 256
			until
				j > cnt 
			loop
				update_byte (a_in_string.item_code (j).as_natural_8)
				j := j + 1
			end
		end
	
	update_bytes (a_in_bytes: CRYPT_BYTES)
			-- Call `update_byte' for each byte in this block.
		require
			a_in_bytes_attached: a_in_bytes /= Void
			consumption_started: initialized
		local
			i: INTEGER
		do
			from
				i := 1
			until
				i > a_in_bytes.count
			loop
				update_byte (a_in_bytes.item (i))
				i := i + 1
			end
		end

end
