note

	description: "Any size byte block as an array of NATURAL_8"

	author: "Franck Arnaud (from pre-AXA Rosenberg days)"
	last_editor: "$Author: c4 $"
	date: "$Date: 2014/02/10 $"
	revision: "$Revision: #1 $"
	path: "$File: //rose/source/third_party/crypto/src/crypt_bytes_array.e $"

class CRYPT_BYTES_ARRAY

inherit

	CRYPT_BYTES

create

	make,
	make_from_hex,
	make_from_string
		
feature {NONE} -- Creation

	make (a: INTEGER)
			-- Make as fixed size (`a') block.
		do
			create array.make (1, a)
		ensure
			count: count = a
		end
		
	make_from_hex (a: STRING)
			-- Make from hexadecimal string `a'
		require
			a_attached: a /= Void
			size: a.count \\ 2 = 0
			hex: STRING_.is_hexadecimal (a)
		do
			make (a.count // 2)
			from_hex (a)
		end
		
	make_from_string (a: STRING)
		-- Create from bytes of `a'
		require
			a_attached: a /= Void
		do
			make (a.count)
			from_string (a)
		end

feature -- Basic specifiers

	item (i: INTEGER): NATURAL_8
			-- `i'_th byte.
		do
			Result := array.item (i)
		end
		
	put (a_byte: NATURAL_8; i: INTEGER)
			-- Put `a_byte' at index `i'.
		do
			array.put (a_byte, i)
		end
		
	count: INTEGER
			-- Number of bytes
		do
			Result := array.count
		end
	
feature {NONE} -- Implementation

	array: ARRAY [NATURAL_8]
			-- Raw bytes

invariant
	
	array_attached: array /= Void
	count: array.count = count
	lower: array.lower = 1
	
end
