note

	description:

		"[
        Intervals which are compared on lower bound (half open).

        In some descendants, two different intervals can be present.
        No invariants have been given on the upper bound (sensible intervals will have it
         not less then the lower bound). But it is not clear to me if this is always the
         case with all existing implementations/clients.
       ]"

	keywords: "comparable,interval"
	author: "Mark Howard"
	last_editor: "$Author: c4 $"
	date: "$Date: 2014/05/13 $"
	revision: "$Revision: #1 $"
	path: "$File: //rose/source/library/containers_base/src/interval/comparable_interval.e $"

deferred class COMPARABLE_INTERVAL [G -> COMPARABLE]

inherit
	
	COMPARABLE
	
feature -- Access
	
	infix "<" (a_other: like Current): BOOLEAN
			-- <Precursor>
		do
			Result := lower < a_other.lower
		end

   lower: attached G
			-- Inclusive lower bound A
		deferred
		ensure
			lower_attached: attached Result
		end
	 
   upper: attached G
			-- Exclusive upper bound A
		deferred
		ensure
			upper_attached: attached Result			
		end
	
	upper_bound: attached G
			-- Exclusive upper bound B
	
   lower_bound: attached G
			-- Inclusive lower bound B
	
	brackets (x: attached G): BOOLEAN
			-- Is `x' within `Current' (B bounds)?
		require
			x_attached: attached x
		do
		 Result := lower_bound <= x and x < upper_bound
	  end

feature -- Setting
	
   restrict (a_lower, a_upper: attached G)
			-- Narrow `Current' to [`a_lower', `a_upper').
			-- Narrowed view is for B bounds, but A bounds
			--  govern permissible arguments.
		require
			a_lower_attached: attached a_lower
			a_upper_attached: attached a_upper
			reasonable_bound: lower <= a_lower and a_upper <=  upper
	  do
		 lower_bound := a_lower
		 upper_bound := a_upper
	  end

invariant
	
	upper_bound_attached: attached upper_bound
	lower_bound_attached: attached lower_bound
	
end

