note

	description: "Routines operating on hash tables"
	author: "Eric Bezault"
	last_editor: "$Author: c4 $"
	date: "$Date: 2014/05/13 $"
	revision: "$Revision: #1 $"
	path: "$File: //rose/source/library/containers_base/src/table/hash_table_routines.e $"

class HASH_TABLE_ROUTINES [G, H -> HASHABLE]

feature -- Basic actions

	reorder_compatible (a_hash_table: HASH_TABLE [G, H])
			-- Reorder the items in `a_hash_table' so that they will be traversed
			-- in the same order as if they had been inserted using the version
			-- of HASH_TABLE provided in EiffelBase compatible mode.
			--
			-- Note: the ordering will not match the one of compatible mode
			-- if items have been removed from the hash-table.
		require
			a_hash_table_not_void: a_hash_table /= Void
		local
			l_items: SPECIAL [detachable TUPLE [value: G; key: H]]
			l_capacity: INTEGER
			i, nb: INTEGER
			l_item: G
			l_key: H
			l_default_key: detachable H
			l_hash_value, l_increment, l_pos: INTEGER
			l_stop: INTEGER
		do
			if internal.has_field_by_name ("indexes_map", a_hash_table) then
					-- New (void-safe) implementation of HASH_TABLE.
				l_capacity := a_hash_table.capacity
				create l_items.make_filled (Void, l_capacity + 1)
					-- Compute the compatible ordering.
				from
					a_hash_table.start
				until
					a_hash_table.after
				loop
					l_item := a_hash_table.item_for_iteration
					l_key := a_hash_table.key_for_iteration
					if l_key = l_default_key or l_key = Void then
						l_items.put ([l_item, l_key], l_capacity)
					else
						from
							l_stop := l_capacity
							l_hash_value := l_key.hash_code
							l_increment := 1 + l_hash_value \\ (l_capacity - 1)
							l_pos := (l_hash_value \\ l_capacity) - l_increment
						until
							l_stop = 0
						loop
								-- Go to next increment.
							l_pos := (l_pos + l_increment) \\ l_capacity
							if not attached l_items.item (l_pos) as l_key_value then
								l_stop := 1
							elseif not attached l_key_value.key as l_other_key then
								l_stop := 1
							else
								debug ("detect_hash_table_catcall")
									check
										catcall_detected: l_other_key.same_type (l_key)
									end
								end
								if a_hash_table.same_keys (l_other_key, l_key) then
									l_stop := 1
								end
							end
							l_stop := l_stop - 1
						end
						l_items.put ([l_item, l_key], l_pos)
					end
					a_hash_table.forth
				end
					-- Reinsert the items using the compatible ordering.
				a_hash_table.wipe_out
				from
					i := 0
					nb := l_items.count - 1
				until
					i > nb
				loop
					if attached l_items.item (i) as l_key_value then
						a_hash_table.put (l_key_value.value, l_key_value.key)
					end
					i := i + 1
				end
			end
		end

feature {NONE} -- Implementation

	internal: ENHANCED_INTERNAL
			-- Internal routines
		do
			create Result
		ensure
			internal_not_void: Result /= Void
		end

end
