note

	description:

		"Ordering over a sortable list established by an agent"

	author: "Mark Howard"
	last_editor: "$Author: c4 $"
	date: "$Date: 2014/05/13 $"
	revision: "$Revision: #1 $"
	path: "$File: //rose/source/library/containers_base/src/sort/ordering.e $"

class ORDERING[G]

inherit

	SORTABLE [G]
		rename
			put as put_i_th,
			item as i_th alias "[]"
		undefine
			is_equal,
			copy
		end

	ARRAYED_LIST [G]
		rename
			make as arrayed_list_make,
			make_filled as arrayed_list_make_filled
		redefine
			new_filled_list
		end

create
	
	make

create {ORDERING}

	make_filled

feature {NONE} -- Initialization

	make_filled (n: INTEGER; a_comparison: like comparison)
			-- Allocate list with `n' items.
			-- (`n' may be zero for empty list.)
			-- This list will be full, ordered by `a_comparison'.
		require
			valid_number_of_items: n >= 0
			a_comparison_attached: a_comparison /= Void
			has_default: ({G}).has_default
		do
			arrayed_list_make_filled (n)
			comparison := a_comparison
		ensure
			correct_position: before
			filled: full
			comparison_set: comparison = a_comparison
		end

	make (a_array: ARRAY [G]; a_comparison: like comparison)
			-- Create from items of `a_array', ordered by `a_comparison'.
		require
			a_array_attached: a_array /= Void
			a_comparison_attached: a_comparison /= Void
		local
			i,j: INTEGER
		do
			comparison := a_comparison
			arrayed_list_make_filled (a_array.count)
			from i := a_array.lower until i > a_array.upper loop
				j := j + 1
				if j <= a_array.count then
					put_i_th (a_array.item (i), j)
				end
				i := i + 1
			end
			sort
		end
	
feature -- Access

	comparison: FUNCTION [ANY, TUPLE [G, G], BOOLEAN]
			-- Ordering function

feature -- Basic operation

	sort_by (a_comparison: FUNCTION [ANY, TUPLE [G, G], BOOLEAN])
			-- Sort this array using `a_comparison'.
		require
			a_comparison_attached: a_comparison /= Void
		local
			l_old_comparison: like comparison
		do
			l_old_comparison := comparison
			comparison := a_comparison
			sort
			comparison := l_old_comparison
		ensure
			comparison_restored: comparison = old comparison
		end
	
feature -- Comparison

	less_than (a_1, a_2: G): BOOLEAN
			-- Is `a_1' less than `a_2' according to `comparison'?
		do
			if attached comparison as l_comparison then
				Result := l_comparison.item ([a_1,a_2])
			end
		end

feature {NONE} -- Implementation

	new_filled_list (n: INTEGER): like Current
			-- New list with `n' elements
		do
			create Result.make_filled (n, comparison)
		ensure then
			comparison_set: Result.comparison = comparison
		end
	
invariant

	comparison_attached: comparison /= Void

end
