note

	description:

		"Rose Arrayed Tree."

	keywords: "Rose Arrayed Tree"
	author: "David Stevens"
	last_editor: "$Author: c4 $"
	date: "$Date: 2014/05/13 $"
	revision: "$Revision: #1 $"
	path: "$File: //rose/source/library/containers_base/src/rose_arrayed_tree.e $"

class ROSE_ARRAYED_TREE [G]

inherit

	ARRAYED_TREE [G]
		rename
			do_all as arrayed_tree_do_all
		redefine
			tree_is_equal, tree_copy
		end

create

	make

feature -- Basic Operations

	do_all (a_agent: PROCEDURE [ANY, TUPLE [G]])
			-- Apply `a_agent' to every node in tree.
		require
			a_agent_attached: a_agent /= Void
		do
			rose_do_all_internal (a_agent, Current)
		end

feature {NONE} -- Implementation

	rose_do_all_internal (a_agent: PROCEDURE [ANY, TUPLE [G]]; a_tree: ARRAYED_TREE [G])
			-- Apply `a_agent' to every node in `a_tree'.
		require
			a_agent_attached: a_agent /= Void
			a_tree_attached: a_tree /= Void
		do
			a_agent.call ([a_tree.item])
			from a_tree.child_start until a_tree.child_off loop
				rose_do_all_internal (a_agent, a_tree.child)
				a_tree.child_forth
			end
		end

feature {NONE} -- Flat Degree 3

	tree_is_equal (t1, t2: like Current): BOOLEAN 
			-- Are `t1' and `t2' recursively equal?
		local
			p1, p2: detachable TREE [G]
			t1_stack, t2_stack: LINKED_STACK [TREE [G]]
			orgidx1_stack, orgidx2_stack: LINKED_STACK [INTEGER]
			l_current_cursor, l_other_cursor: like child_cursor
		do
			l_current_cursor := t1.child_cursor
			l_other_cursor := t2.child_cursor

			if t1.is_leaf and t2.is_leaf then
				Result := equal (t1.item, t2.item)
			elseif t1.is_leaf xor t2.is_leaf then
				Result := False
			else
				create t1_stack.make
				create t2_stack.make
				create orgidx1_stack.make
				create orgidx2_stack.make
				orgidx1_stack.put (t1.child_index)
				orgidx2_stack.put (t2.child_index)
				from
					Result := True
					p1 := t1
					p2 := t2
					p1.child_start
					p2.child_start
				invariant
					same_count: t1_stack.count = t2_stack.count
				until
					not Result or else
						p1.child_after and t1_stack.is_empty
				loop
					if p1.child_readable and p2.child_readable and
						p1.child_capacity = p2.child_capacity then
						Result := p1.node_is_equal (p2)
						check attached p1.child as l_child1 and attached p2.child as l_child2 then
								-- Missing ISE postconditions of {TREE}.child_readable
							if not (l_child1.is_leaf or l_child2.is_leaf) then
								t1_stack.put (p1)
								t2_stack.put (p2)
								p1 := l_child1
								p2 := l_child2
								Result := p1.node_is_equal (p2)
								orgidx1_stack.put (p1.child_index)
								orgidx2_stack.put (p2.child_index)
								p1.child_start
								p2.child_start
							elseif l_child1.is_leaf xor l_child2.is_leaf then
								Result := False
							else
								Result := l_child1.node_is_equal (l_child2)
							end
						end
					elseif p1.child_capacity /= p2.child_capacity or else
						(p1.child_readable xor p2.child_readable) then
						Result := False
					end
					if not p1.child_after then
						p1.child_forth
						p2.child_forth
					else
						from
						invariant
							same_count: t1_stack.count = t2_stack.count
						until
							t1_stack.is_empty or else not p1.child_after --or else i <= p1.arity
						loop
							p1 := t1_stack.item
							p2 := t2_stack.item
							p1.child_forth
							p2.child_forth
							t1_stack.remove
							t2_stack.remove
							orgidx1_stack.remove
							orgidx2_stack.remove
						end
					end
				end
				if not Result then
					from
					invariant
						same_count: t1_stack.count = t2_stack.count and
								orgidx1_stack.count = orgidx2_stack.count
					until
						orgidx1_stack.count = 1
					loop
						p1.child_go_i_th (orgidx1_stack.item)
						p2.child_go_i_th (orgidx2_stack.item)
						p1 := t1_stack.item
						p2 := t2_stack.item
							check
								p1_not_void: p1 /= Void
								p2_not_void: p2 /= Void
									-- Because we never put Void references on the
									-- stack.
							end
						t1_stack.remove
						t2_stack.remove
						orgidx1_stack.remove
						orgidx2_stack.remove
					end
						check
							tree_stacks_empty: t1_stack.is_empty and
									t2_stack.is_empty
								-- Because we removed all items.
							at_root: p1 = t1 and p2 = t2
								-- Because the root nodes where the last item we
								-- removed.
							p1_not_void: p1 /= Void
							p2_not_void: p2 /= Void
								-- Because the root nodes cannot be Void.
						end
					p1.child_go_i_th (orgidx1_stack.item)
					p2.child_go_i_th (orgidx2_stack.item)
					orgidx1_stack.remove
					orgidx2_stack.remove
						check
							index_stacks_empty: orgidx1_stack.is_empty and
										orgidx2_stack.is_empty
								-- Because we also removed the roots from the index
								-- stacks now.
						end
				end
			end
			t1.child_go_to (l_current_cursor)
			t2.child_go_to (l_other_cursor)
		end

 	tree_copy (other, tmp_tree: like Current)
			-- Generic implementation of `copy'. `other' is copied onto
			-- `Current'. `tmp_tree' is used as temporary storage during
			-- copying. Since it cannot be created locally because of the
			-- generic implementation, it has to be passed in.
		local
			i: INTEGER
			p1, p2, node: TREE [G]
			other_stack, tmp_stack: LINKED_STACK [TREE [G]]
			idx_stack, orgidx_stack: LINKED_STACK [INTEGER]
		do
			create other_stack.make
			create tmp_stack.make
			create idx_stack.make
			create orgidx_stack.make
			if other.object_comparison then
				tmp_tree.compare_objects
			end
			orgidx_stack.put (other.child_index)
			from
				i := 1
				p1 := other
				p2 := tmp_tree
			invariant
				same_count: other_stack.count = tmp_stack.count and
							tmp_stack.count = idx_stack.count
			until
				i > p1.child_capacity and other_stack.is_empty
			loop
				p1.child_go_i_th (i)
				p2.child_go_i_th (i)
				if p1.child_readable then
					check attached p1.child as l_child1 then
							-- Because we only get here when the child is
							-- readable.
							-- target_child_void: p2.child = Void
							-- Because the target child has not been copied
							-- yet.
						-- Inline `clone_node'.
						node := l_child1.clone_node (l_child1)
						-- End inlining.
						check
							equal_but_not_the_same: standard_equal (node, p1.child) and node /= p1.child
								-- Because `node' has been cloned.
						end
						if attached {FIXED_TREE [G]} p1 then
							p2.replace_child (node)
						else
							p2.put_child (node)
						end
						check
							node_is_child: node = p2.child
								-- Because we inserted `node' as child.
								-- remaining assertions commented out to avoid too much void-safety repair
							--comparison_mode_ok: p2.child.object_comparison =
							--			p1.child.object_comparison
								-- Because the comparson mode flag must be copied
								-- correctly, too.
							--p1_consistent: p1.child.parent = p1
							--p2_consistent: p2.child.parent = p2
								-- Because the tree has to be consistent.
						end
						if not l_child1.is_leaf then
							other_stack.put (p1)
							tmp_stack.put (p2)
							idx_stack.put (i + 1)
							p1 := l_child1
							p2 := node
							orgidx_stack.put (p1.child_index)
							i := 0
						end
					end
				end
				if i <= p1.child_capacity then
					i := i + 1
				else
					from
					invariant
						same_count: other_stack.count = tmp_stack.count and
									tmp_stack.count = idx_stack.count
					until
						other_stack.is_empty or else i <= p1.child_capacity
					loop
						p1.child_go_i_th (orgidx_stack.item)
						p2.child_go_i_th (orgidx_stack.item)
							check
								child_indices_equal:
									p1.child_index = p2.child_index
										-- Because we have set them equal before.
							end
						p1 := other_stack.item
						p2 := tmp_stack.item
							check
								p1_not_void: p1 /= Void
								p2_not_void: p2 /= Void
									-- Because we never put Void references on the
									-- stack.
							end
						i := idx_stack.item
						other_stack.remove
						tmp_stack.remove
						idx_stack.remove
						orgidx_stack.remove
					end
				end
			end
			other.child_go_i_th (orgidx_stack.item)
			tmp_tree.child_go_i_th (orgidx_stack.item)
			orgidx_stack.remove
				check
					tree_stacks_empty: other_stack.is_empty and tmp_stack.is_empty
						-- Because we removed all items.
					at_root: p1 = other and p2 = tmp_tree
						-- Because the root nodes where the last item we removed.
					copy_correct: equal (other, tmp_tree)
						-- Because `other' has been copied to `tmp_tree'.

					index_stack_empty: orgidx_stack.is_empty
						-- Because we also removed the root from the index
						-- stack now.
				end
		end

end
