note

	description:

		"Lockable hashmap"

	author: "Sami Ben-Grine"
	last_editor: "$Author: c4 $"
	date: "$Date: 2014/05/13 $"
	revision: "$Revision: #1 $"
	path: "$File: //rose/source/library/containers_base/src/concurrent/rose_concurrent_hash_map.e $"

class	ROSE_CONCURRENT_HASH_MAP [G, H -> HASHABLE]

inherit
	
	HASH_TABLE [G, H]
		rename
			make as make_table
		redefine
			empty_duplicate
		end

create
	
	make

feature {NONE} -- Initialization

	make (n: INTEGER)
			-- Initialize with `n' slots'
		require
			n_positive_or_null: n >= 0
		do
			create mutex.make
			make_table (n)
		end

feature -- Basic Operations

	put_thread_safe (a_item: G; a_key: H)
			-- Thread safe `put'.
		do
			mutex.lock
			put (a_item, a_key)
			mutex.unlock
		end

	item_thread_safe (a_key: H): G
			-- Thread safe `item'.
		local
			l_default: detachable G
		do
			mutex.lock
			if attached item (a_key) as l_item then 
				Result := l_item
			elseif ({G}).has_default then
				Result := ({G}).default
			else
				check attached l_default as l_default_2 then
						-- G must be an attached type as `has_default' is `False'
					Result := l_default_2
				end
			end
			mutex.unlock
		end

	remove_thread_safe (a_key: H)
			-- Thread safe `remove'.
		do
			mutex.lock
			remove (a_key)
			mutex.unlock
		end

	has_thread_safe (a_key: H): BOOLEAN
			-- Thread safe `has'.
		do
			mutex.lock
			Result := has (a_key)
			mutex.unlock
		end

	lock
			-- Lock this hashmap.
		do
			mutex.lock
		end

	unlock
			-- Unlock this hashmap.
		do
			mutex.unlock
		end
	
feature {NONE} -- Duplication

	empty_duplicate (n: INTEGER): like Current
			-- Create an empty copy of `Current. that can accommodate `n' items
		do
			create Result.make (n)
			if object_comparison then
				Result.compare_objects
			end
		end

feature {NONE} -- Implementation

	mutex: CRITICAL_SECTION
			-- exclusive locker
	
invariant
	
	mutex_not_void: mutex /= Void

end
