note

	description:

		"[
        Simple implementation of an ordered triple of items.

        None of the generics may represent a self-initializing type.
       ]"

	keywords: "triple, ordered"
	author: "David Stevens"
	last_editor: "$Author: c4 $"
	date: "$Date: 2014/05/13 $"
	revision: "$Revision: #1 $"
	path: "$File: //rose/source/library/containers_base/src/tuple/triple.e $"

class
   
   TRIPLE [G, H, I]

inherit
	
	ANY
		redefine
			default_create
		end

create
   
   make, default_create
	
feature {NONE} -- Initialization

	default_create
			-- <Precursor>
		do
			check ({G}).has_default then
					-- programmer obligation, as expressed in class description
				first := ({G}).default
			end
			check ({H}).has_default then
					-- programmer obligation, as expressed in class description
				second := ({H}).default
			end			
			check ({I}).has_default then
					-- programmer obligation, as expressed in class description
				third := ({I}).default
			end			
		end
	
feature -- Access
   
   first: G
			-- Projection 1
	
   second: H
			-- Projection 2
	
   third: I
			-- Projection 3

feature -- Initialization
	
   make, set (a_first: G; a_second: H; a_third: I)
			-- Re-initialize fields.
      do
		 first := a_first
		 second := a_second
		 third := a_third
		ensure
			first_aliased: (a_first = a_first) implies first = a_first -- for non-reflexive types e.g. REAL_nn
			second_aliased: (a_second = a_second) implies second = a_second
			third_aliased: (a_third = a_third) implies third = a_third
      end
	
feature -- Setting
	
	put_first (a_first: G)
			-- Set `first' from `a_first'.
		do
			first := a_first
		ensure
			first_aliased: (a_first = a_first) implies first = a_first
		end
    
	put_second (a_second: H)
			-- Set `second' from `a_second'.
		do
			second := a_second
		ensure
			second_aliased: (a_second = a_second) implies second = a_second
		end
    
	put_third (a_third: I)
			-- Set `third' from `a_third'.
		do
			third := a_third
		ensure
			third_aliased: (a_third = a_third) implies third = a_third
		end
	
end
