note
	description: "Dynamically_Sortable_Array"
	status: "Development"
	author: "David Stevens"
	last_editor: "$Author: c4 $"
	date: "$Date: 2014/05/13 $"
	revision: "$Revision: #1 $"
	path: "$File: //rose/source/library/containers_core/src/sort/dynamically_sortable_array.e $"

class
	DYNAMICALLY_SORTABLE_ARRAY [G]

inherit
	RP_SERIALIZABLE
	ROSE_SORTABLE_ARRAY [G]
		rename
			-- These should really be redefines but due to a bug in Precursor.
			-- This is safer at the moment.
			make as sa_make,
			make_empty as sa_make_empty,
			make_from_array as sa_make_from_array,
			make_from_linear as sa_make_from_linear
		end

create
	make,
	make_empty,
	make_from_array,
	make_from_linear

feature {NONE} -- Initialization

	make (a_lower, a_upper: INTEGER)
			-- Create with 'a_lower' and 'a_upper' bounds.
		do
			create pair
			sa_make (a_lower, a_upper)
		end

	make_empty
			-- Create an empty array.
		do
			create pair
			sa_make_empty
		end

	make_from_array (a_array: ARRAY [G])
			-- Make from contents in 'a_array'.
		do
			create pair
			sa_make_from_array (a_array)
		end

	make_from_linear (a_linear: LINEAR [G])
			-- Make from contents in 'a_linear'.
		do
			create pair
			sa_make_from_linear (a_linear)
		end

feature -- Basic Operations

	sort_by (a_routine: FUNCTION [ANY, TUPLE [G, G], BOOLEAN])
			-- Sort this array using 'a_routine' as comparison.
		do
			comparison := a_routine
			sort
			comparison := Void
		end

feature {NONE} -- Implementation

	pair: TUPLE [G, G]
			-- Useless attribute, trying to speed things up a bit in 'less_than' by avoiding a creation of a temporary tuple every time 'less_than' is called
			-- This could be rewritten to not need any extra attribute

	comparison: detachable FUNCTION [ANY, TUPLE [G, G], BOOLEAN]
			-- Comparison agent to use in `sort';
			-- Ideally, this shouldn't be an attribute

	less_than (a, b: G): BOOLEAN
			-- Is `a' < `b'?
		do
			if a /= Void and b /= Void then
				pair.put (a, 1)
				pair.put (b, 2)
				if attached comparison as l_comparison then
					Result := l_comparison.item (pair)
				else
					-- BUG? G isn't COMPARABLE
				end
			else
				Result := a /= Void and then b = Void
			end
		end

end
