note

	description:

		"Boolean array"

	author: "Micahel Zhang"
	last_editor: "$Author: c4 $"
	date: "$Date: 2014/05/13 $"
	revision: "$Revision: #1 $"
	path: "$File: //rose/source/library/containers_core/src/array/rose_boolean_array.e $"

class	ROSE_BOOLEAN_ARRAY

inherit
	
	ROSE_LINEAR_ARRAY[BOOLEAN]

create
	
	make, make_empty, make_from_array, make_from_linear, make_from_array_reversed, make_with_default_value

feature -- Access

	complement_array: like Current
			-- Copy of `Current' with all values negated
		local
			i : INTEGER
		do
			create Result.make (1, count)
			from
				i := lower
			until
				i > upper
			loop
				Result [i - lower + 1] := not Current [i]
				i := i + 1
			end
		ensure
			complement_array_attached: Result /= Void
			same_count: Result.count = count
		end
	
feature -- Element change
	
	and_with_boolean_array (a_other_array: ARRAY [BOOLEAN])
			-- Logical AND `a_other_array' into `Current'
		require
			same_length : a_other_array /= Void and then a_other_array.count = count
		local
			i, l_other_lower: INTEGER
		do
			l_other_lower := a_other_array.lower
			from
				i := lower
			until
				i > upper
			loop
				Current [i] := Current [i] and a_other_array [i - lower + l_other_lower]
				i := i + 1
			end
		end

	or_with_boolean_array (a_other_array: ARRAY [BOOLEAN])
			-- Logical OR `a_other_array' into `Current'
		require
			same_length : a_other_array /= Void and then a_other_array.count = count
		local
			i, l_other_lower: INTEGER
		do
			l_other_lower := a_other_array.lower
			from
				i := lower
			until
				i > upper
			loop
				Current [i] := Current [i] or a_other_array [i - lower + l_other_lower]
				i := i + 1
			end
		end
	
feature -- Access

	valid_count: INTEGER
			-- Number of `True' items
		local
			i: INTEGER
		do
			from
				i := lower
			until
				i > upper
			loop
				if Current[i] then
					Result := Result + 1
				end
				i := i + 1
			end
		ensure
			non_negative: Result >= 0
			not_more_than_count: Result <= count
		end

	valid_indice: ROSE_LINEAR_ARRAY [INTEGER]
			-- Indices of `True' items
		local
			i: INTEGER
			l_count, l_valid_count: INTEGER
		do
			l_valid_count := valid_count
			if l_valid_count > 0 then
				create Result.make (1, l_valid_count)
				from
					i := lower
				until
					i > upper
				loop
					if Current [i] then
						l_count := l_count + 1
						Result [l_count] := i
					end
					i := i + 1
				end
			else
				create Result.make_empty
			end
		ensure
			valid_indice_attached: Result /= Void
			lower_one: Result.lower = 1
			valid_count: Result.count = valid_count
		end

	invalid_indice: ROSE_LINEAR_ARRAY [INTEGER]
			-- Indices of `False' items
		local
			i: INTEGER
			l_count, l_valid_count: INTEGER
		do
			l_valid_count := valid_count
			if l_valid_count < count then
				create Result.make (1, count - l_valid_count)
				from
					i := lower
				until
					i > upper
				loop
					if not Current [i] then
						l_count := l_count + 1
						Result [l_count] := i
					end
					i := i + 1
				end
			else
				create Result.make_empty
			end
		ensure
			invalid_indice_attached: Result /= Void
			lower_one: Result.lower = 1
			invalid_count: Result.count = count - valid_count
		end

end
