note
	
	description: "[
                  Linear structures with elements directly addressable in O(1)

                  The position within the structure can also be looked up efficiently.
                 ]"

	author: "Mark Howard"
	last_editor: "$Author: c4 $"
	date: "$Date: 2014/05/13 $"
	revision: "$Revision: #1 $"
	path: "$File: //rose/source/library/containers_core/src/array/rose_hashed_linear_array.e $"
	storable_version: ""

class	ROSE_HASHED_LINEAR_ARRAY [G -> HASHABLE]

obsolete "Don't use this badly designed class"

inherit

	ROSE_LINEAR_ARRAY [G]
		redefine
			make,
			enter,
			put,
			extend,
			force,
			has_value,
			remove,
			new_index_of,
			union_values_is_equal,
			intersection_values,
			make_from_array,
			make_empty,
			make_from_linear
		end

create
	
	make, make_distinct_from_array, make_from_array,
	make_empty, make_from_linear

feature {NONE} -- Initialization
	
	make (a_lower, a_upper: INTEGER)
			-- Create with indices from `a_lower' to `a_upper'.
		local
			l_count: INTEGER
		do
			l_count := a_upper - a_lower + 1
			if l_count <= 0 then
				l_count := 100
			end
			create inverse.make (l_count)
			Precursor (a_lower, a_upper)
		end

	make_distinct_from_array (a_array: ARRAY [G])
			-- Create from distinct elements of `a_arrary'.
		require
			a_array_attached: a_array /= Void
		local
			i: INTEGER
			l_item: G
		do
			make_empty
			create inverse.make (a_array.count)
			from
				i := a_array.lower
			until i > a_array.upper
			loop
				l_item := a_array.item (i)
				if not inverse.has (l_item) then
					extend (l_item)
				end
				i := i + 1
			end
		end
	
	make_empty
				-- <Precursor>
		do
			create inverse.make (0)
			Precursor
		end

	make_from_linear (a_linear: LINEAR [like item])
			-- <Precursor>
		do
			Precursor (a_linear)
			create inverse.make (count)
		end
	
feature -- Initialization
	
	make_from_array (a_array: ARRAY [like item])
			-- <Precursor>
		do
			create inverse.make (a_array.count)
			Precursor (a_array)
		end
			
feature -- Element change
	
	put (a_g: G; i: INTEGER)
			-- <Precursor>
		do
			Precursor (a_g,i)
			inverse.force (i, a_g)
		end

	enter (a_g: G; i: INTEGER)
			-- <Precursor>
		do
			Precursor (a_g, i)
			inverse.force (i, a_g)
		end

	extend (a_g: G)
			-- <Precursor>
		do
			Precursor (a_g)
			inverse.force (upper, a_g)
		end

	force (v: like item; i: INTEGER)
			-- <Precursor>
		do
			Precursor (v, i)
			inverse.force (i, v)
		end
	
feature -- Removal
	
	remove
			-- <Precursor>
		local
			i: INTEGER
		do
			if not off then
				inverse.remove (current_item)
				from
					i := index
				until
					i > upper
				loop
					inverse.force (i, item (i))
					i := i + 1
				end
			end
			Precursor
		end

feature -- Access
	
	new_index_of (a_g: G): INTEGER
			-- <Precursor>
		do
			Result := inverse.item (a_g)
		end

	indices_of (a_array: ARRAY [G]): ROSE_LINEAR_ARRAY [INTEGER]
			-- Indices of elements of `a_array' within `Current'
		require
			a_array_attached: a_array /= Void			
		local
			i, l_ix: INTEGER
		do
			create Result.make (1, a_array.count)
			from
				i := a_array.lower
			until
				i > a_array.upper
			loop
				l_ix := new_index_of (a_array.item(i))
				Result.put (l_ix, i - a_array.lower + 1)
				i := i + 1
			end
		ensure
			indices_of_attached: Result /= Void
			same_count: Result.count = a_array.count
			one_based: Result.lower = 1
		end

	flags_of (a_array: ARRAY [G]): ROSE_BOOLEAN_ARRAY
			-- Valid elements of `a_array'
		require
			a_array_attached: a_array /= Void			
		local
			i, l_ix: INTEGER
		do
			create Result.make (1, count)
			from
				i := a_array.lower
			until
				i > a_array.upper
			loop
				l_ix := new_index_of (a_array.item (i))
				if valid_index (l_ix) then
					Result.put (True, l_ix)
				end
				i := i + 1
			end
		ensure
			flags_of_attached: Result /= Void
			same_count: Result.count = a_array.count
			one_based: Result.lower = 1
		end

	has_value (a_g: G): BOOLEAN
			-- <Precursor>
		do
			Result := inverse.has (a_g)
		end
	
feature -- Element change
	
	merge (a_items: ARRAY [G])
			-- Add elements of `a_items' to end of `Current', if not already present
		local
			i: INTEGER
			l_item: G
		do
			if a_items /= Void and then a_items.count > 0 then
				from
					i := a_items.lower
				until
					i > a_items.upper
				loop
					l_item := a_items.item (i)
					if not has_value (l_item) then
						extend (l_item)
					end
					i := i + 1
				end
			end
		ensure
			all_items_present: attached a_items as l_items implies across l_items as i_item all has_value (i_item.item) end
		end
	
feature -- Access
	
	union_values_is_equal (a_linear: LINEAR [G]): like Current
			-- <Precursor>
		do
			Result := twin
			from
				a_linear.start
			until
				a_linear.off
			loop
				if not has_value (a_linear.item) then
					Result.extend (a_linear.item)
				end
				a_linear.forth
			end
		end

	intersection_values (a_linear: LINEAR [G]): like Current
			-- Intersection of current and argument array values, uses is_equal
		do
			create Result.make_empty
			from
				a_linear.start
			until
				a_linear.off
			loop
				if has_value (a_linear.item) then
					Result.extend (a_linear.item)
				end
				a_linear.forth
			end
		end

	items_not_have (a_linear: LINEAR [G]): like Current
			-- Elelments of `a_linear' not present in `Current'
		require
			a_linear_attached: a_linear /= Void
		do
			create Result.make_empty
			from
				a_linear.start
			until
				a_linear.off
			loop
				if not has_value (a_linear.item) then
					Result.extend (a_linear.item)
				end
				a_linear.forth
			end
		ensure
			items_not_have_attached: Result /= Void
		end
	
feature -- Implementation

	inverse: HASH_TABLE [INTEGER, G]
			-- Positions indexed by items

invariant

	inverse_attached: inverse /= Void

end



