note

	description:

		"linear, sortable, indexed by hashable keys"

	author: "Michael Zhang"
	last_editor: "$Author: c4 $"
	date: "$Date: 2014/05/13 $"
	revision: "$Revision: #1 $"
	path: "$File: //rose/source/library/containers_core/src/array/rose_indexed_array.e $"

class	ROSE_INDEXED_ARRAY [G, H -> HASHABLE]

obsolete "Don't use this badly designed class."

inherit

	ROSE_LINEAR_ARRAY [G]
		rename
			put as linear_put,
			extend as linear_extend
		redefine
			make_empty,
			make,
			append_array,
			internal_swap,
			subset,
			subarray,
			remove,
			make_from_array
		end

create
	
	make, make_empty, make_all, make_from_array

feature -- Initialization

	make_empty
			-- <Precursor>
		do
			create keys.make_empty
			Precursor
		end

	make (a_lower, a_upper: INTEGER)
			-- <Precursor>
		do
			create keys.make (a_lower, a_upper)
			Precursor (a_lower, a_upper)
		end

	make_all (a_array: ARRAY [G]; a_keys: ARRAY [H])
			-- Initialize with elements of `a_array' and keys from `a_keys'.
		require
			valid_size: (a_array /= Void and a_keys /= Void) implies a_array.count = a_keys.count
		do
			if a_array /= Void and a_keys /= Void then
				create keys.make_from_array (a_keys)
				make_from_array (a_array)
			else
				make_empty
			end
		end
	
	make_from_array (a_array: ARRAY [G])
			-- <Precursor>
		do
			create keys.make (a_array.lower, a_array.upper)
			Precursor (a_array)
		end
	
feature -- Access

	item_at (a_key: H): detachable G
			-- Item indexed by `a_key'
		local
			l_ix: INTEGER
		do
			l_ix := index_at (a_key)
			if valid_index (l_ix) then
				Result := item (l_ix)
			end
		end

	item_before (a_key: H): detachable G
			-- Item at previous index to that corresponding to `a_key'
		local
			l_ix: INTEGER
		do
			l_ix := keys.new_index_of (a_key)
			if valid_index (l_ix - 1) then
				Result := item (l_ix - 1)
			end
		end

	item_after (a_key: H): detachable G
			-- Item at next index to that corresponding to `a_key'
		local
			l_ix: INTEGER
		do
			l_ix := keys.new_index_of (a_key)
			if valid_index (l_ix + 1) then
				Result := item (l_ix + 1)
			end
		end

	index_at (a_key: H): INTEGER
			-- Index corresponding to `a_key'
		do
			Result := keys.new_index_of (a_key)
		end

	has_key (a_key: H): BOOLEAN
			-- Is `a_key' present?
		local
			l_ix: INTEGER
		do
			l_ix := index_at (a_key)
			Result := valid_index (l_ix)
		end

	subarray (a_start_pos, a_end_pos: INTEGER): like Current
			-- <Precursor>
		local
			i: INTEGER
			l_key: H
		do
			Result := Precursor (a_start_pos, a_end_pos)
			from
				i := a_start_pos
			until
				i > a_end_pos
			loop
				l_key := keys.item (i)
				Result.put_key (l_key, i)
				i := i + 1
			end
		end

	subset (a_indices: ARRAY [INTEGER]): like Current
			-- <Precursor>
		local
			i, j, k: INTEGER
			l_key: H
		do
			Result := Precursor (a_indices)
			k := a_indices.count
			from
				i := 1
			until
				i > k
			loop
				j := a_indices.item (i)
				l_key := keys.item (j)
				Result.put_key (l_key, i)
				i := i + 1
			end
		end

	subset_by_keys (a_keys: ARRAY [H]): like Current
			--	Items of `Current' indexed by `a_keys'
		require
			a_keys_attached: a_keys /= Void
		local
			i: INTEGER
		do
			create Result.make_empty
			from
				i := a_keys.lower
			until
				i > a_keys.upper
			loop
				if attached a_keys.item (i) as l_key and then has_key (l_key) and then
					attached item_at (l_key) as l_item then
					Result.extend (l_item, l_key)
				end
				i := i + 1
			end
		end

	sorted_array_by_key (a_order: FUNCTION [ANY, TUPLE [H, H], BOOLEAN]): like Current
			-- Copy of `Current' sorted by `a_order'
		require
			a_order_attached: a_order /= Void
		local
			l_keys: like keys
		do
			create l_keys.make_from_array (keys) 
			l_keys.sort_by (a_order)
			Result := subset_by_keys (l_keys)
		end
		
feature -- Access
	
	key (i: INTEGER): detachable H
			-- Key for item at position `i'
		do
			if valid_index (i) then
				Result := keys.item (i)
			end
		ensure
			key_attached: valid_index (i) implies attached Result
		end

	current_key: H
			-- Key of item at `index'
		require
			i_valid: valid_index (index)
		do
			Result := keys.item (index)
		end

	first_key: detachable H
			-- First (if any) in `keys'
		do
			Result := keys.first
		end

	last_key: detachable H
			-- Last (if any) in `keys'
		do
			Result := keys.last
		end

	keys: ROSE_HASHED_LINEAR_ARRAY [H]
			-- Keys of corresponding items 
	
feature -- Element change

	put (a_value: G; a_key: H; i: INTEGER)
			-- Replace `i'-th entry by `a_value'.
			-- Index it by `a_key'.
		require
			i_valid: valid_index (i)
		do
			linear_put (a_value, i)
			keys.put (a_key, i)
		ensure
			inserted: (a_value = a_value) implies item (i) = a_value
			key_inserted: (a_key = a_key) implies key (i) = a_key
		end

	put_key (a_key: H; i: INTEGER)
			-- Replace key at `i' by `a_key'.
		require
			i_valid: valid_index (i)
		do
			keys.put (a_key, i)
		ensure
			key_inserted: (a_key = a_key) implies key (i) = a_key
		end

	extend (a_value: G; a_key: H)
			-- Add `a_value' to end of `Current'. Index it by `a_key'.
		do
			linear_extend (a_value)
			keys.extend (a_key)
		end

	replace_at (a_value: G; a_key: H)
			-- Replace item indexed by `a_key' (if any) with `a_value'.
		local
			l_ix: INTEGER
		do
			l_ix := index_at (a_key)
			if valid_index (l_ix) then
				linear_put (a_value, l_ix)
				keys.put (a_key, l_ix)
			end
		end
	
feature -- Removal
	
	remove
			-- <Precursor>
		do
			keys.go_i_th (index)
			keys.remove
			Precursor
		end
	
feature -- Element change
	
	append_array (a_array: like Current)
			-- <Precursor>
		local
			i: INTEGER
		do
			if a_array /= Void then
				from
					i := a_array.lower
				until
					i > a_array.upper
				loop
					check attached a_array.key(i) as l_key then
							-- postcondition "key_attached" of `key'
						extend (a_array.item (i), l_key)
					end
					i := i + 1
				end
			end
		end

feature {NONE} -- Implementation

   internal_swap (i, j: INTEGER)
			-- <Precursor>
		local
			l_temp_key: H
		do
			l_temp_key := keys.item (i)
			keys.put (keys.item (j), i)
			keys.put (l_temp_key, j)
			Precursor (i, j)
		end

feature -- Access
	
	name: detachable STRING
			-- Optional dataset label
	
feature -- Setting
	
	set_name (a_name: STRING)
			-- Set `name' to `a_name'.
		require
			a_name_attached: a_name /= Void
		do
			name := a_name
		ensure
			name_aliased: name = a_name
		end

invariant

	keys_attached: keys /= Void
	same_count: keys.count = count

end
