note
	
	description: "Multiple purpose array which is linear, extendable, sortable and iterable"

	attention: "[
		You must have an EXCELLENT reason to modify this class, especially if:
		- you are adding/removing an attribute
		- adding an ancestor
		Keep in mind that a non negligeable amount of instances of this object will
		typically be present in any system from the RC
	]"
	author: "Mark Howard"
	last_editor: "$Author: c4 $"
	date: "$Date: 2014/05/13 $"
	revision: "$Revision: #1 $"
	path: "$File: //rose/source/library/containers_core/src/array/rose_linear_array.e $"
	storable_version: ""

class ROSE_LINEAR_ARRAY [G]

inherit
	
	RP_SERIALIZABLE

	ROSE_SORTABLE_ARRAY [G]
		redefine
			binary_search,
			linear_search
		end

	MISMATCH_CORRECTOR
			-- Remove inheritance to this once all storables have been regenerated
			-- (ROSE_STORABLE removed from ancestors on 04/30/2004)
		export
			{NONE} all
		undefine
			is_equal, copy
		redefine
			correct_mismatch
		end

	DEBUG_OUTPUT
		undefine
			is_equal, copy
		end

create

	make, make_empty, make_from_array, make_from_linear, make_from_array_reversed,
	make_with_default_value

convert

	make_from_array ({ARRAY [G]})

feature {NONE} -- Correction

	correct_mismatch
			-- Attempt to correct object mismatch using `mismatch_information'.
		local
			l_mismatch: MISMATCH_INFORMATION
		do
			l_mismatch := mismatch_information
		end

feature -- Access

	infix "+" (a_other: LINEAR [G]): like Current
			-- Union of 'Current' and 'a_other'
		require
			valid_other: a_other /= Void
		do
			Result := twin
			if a_other /= Void then
				from a_other.start until a_other.off loop
					Result.extend (a_other.item)
					a_other.forth
				end
			end
		ensure
			infix_plus_attached: Result /= Void
		end

	subset (a_indices: ARRAY [INTEGER]): like Current
			-- Subset composed of items at `a_indices' from this array
		require
			a_indices_attached: a_indices /= Void
		local
			i, j, k, l_lower: INTEGER
		do
			k := a_indices.count
			l_lower := a_indices.lower
			create Result.make (1, k)
			from i := 1 until i > k loop
				j := a_indices.item (l_lower + i - 1)
				if not valid_index (j) then
					(create {EXCEPTIONS}).raise ("Invalid index given to ROSE_LINEAR_ARRAY.subset")
				end
				Result.put (item (j), i)
				i := i + 1
			end
		ensure
			subset_attached: Result /= Void
		end

	subset_loose (a_indices: ARRAY [INTEGER]): like Current
			-- Subset composed of items at `a_indices' from this array;
			-- Compared to `subset', this has loose requirement. If a_indices [i] is invalid,
			--  put null in position i.
		require
			a_indices_attached: a_indices /= Void	
		local
			i, j, k, l_lower: INTEGER
		do
			k := a_indices.count
			l_lower := a_indices.lower
			create Result.make (1, k)
			from i := 1 until i > k loop
				j := a_indices.item (l_lower + i - 1)
				if not valid_index (j) then
					Print ("%NInvalid index given to ROSE_LINEAR_ARRAY.subset_loose%N")
				else
					Result.put (item (j), i)
				end
				i := i + 1
			end
		ensure
			subset_loose_attached: Result /= Void
		end

	subarray_by_flags (a_flags: ARRAY [BOOLEAN]): like Current
			-- Sub-array by a valid flag array
		do
			create Result.make_empty
			if a_flags /= Void then
				from start until off loop
					if a_flags.valid_index (index) and then a_flags [index] then
						Result.extend (current_item)
					end
					forth
				end
			end
		ensure
			subarray_by_flags_attached: Result /= Void
		end

	random_subarray (a_count: INTEGER): like Current
			-- Random subarray of size `a_count';
			-- Empty if `a_count' <= 0, the entire array if `a_count' >= `count'
		local
			l_needed, l_remaining, l_found, l_seed, i: INTEGER
			l_date: ROSE_DATE
			l_random: RANDOM
			l_random_double: DOUBLE
		do
			if a_count <= 0 then
				l_needed := 0
			elseif a_count >= count then
				l_needed := count
			else
				l_needed := a_count
			end

			if l_needed = 0 then
				create Result.make_empty
			else
				l_remaining := count
				l_found := 0
				create Result.make (1, l_needed)
				create l_random.make
				create l_date.make_now
				l_seed := l_date.second
				l_random.set_seed (l_seed)
				from i := 1 until i > count or l_needed = 0 loop
					l_random.forth
					l_random_double := l_random.double_item
					if l_random_double <= l_needed/l_remaining then
						-- add this to the array
						l_found := l_found + 1
						l_needed := l_needed - 1
						Result [l_found] := item (i)
					end
					i := i + 1
					l_remaining := l_remaining - 1
				end
			end
		ensure
			random_subarray_attached: Result /= Void
		end

	valid_subarray (a_valid: detachable FUNCTION [ANY, TUPLE [G], BOOLEAN]): like Current
			-- Sub-array by agent `a_valid';
			-- Empty if no agent is provided
		local
			i: INTEGER
		do
			create Result.make_empty
			if a_valid /= Void then
				from i := lower until i > upper loop
					if a_valid.item ([item (i)]) then
						Result.extend (item (i))
					end
					i := i + 1
				end
			end
		ensure
			valid_subarray_attached: Result /= Void
		end
	
	first_valid (a_valid: FUNCTION [ANY, TUPLE [G], BOOLEAN]): detachable G 
			-- The first item in that meets `a_valid'
			-- `default_value' if none found
		require
			valid_function: a_valid /= Void
		local
			i: INTEGER
			l_found: BOOLEAN
		do
			from
				i := lower
			until
				i > upper or else l_found
			loop
				if a_valid.item ([item (i)]) then
					Result := item (i)
					l_found := True
				end
				i := i + 1
			end
		end
	
	valid_flags (a_valid: FUNCTION [ANY, TUPLE [G], BOOLEAN]): ROSE_LINEAR_ARRAY [BOOLEAN]
			-- Flags indicating if corresponding data item is valid
		local
			i: INTEGER
		do
			create Result.make (lower, upper)
			if a_valid /= Void then
				from i := lower until i > upper loop
					Result.put (a_valid.item ([item (i)]), i)
					i := i + 1
				end
			else
				from i := lower until i > upper loop
					Result.put (True, i)
					i := i + 1
				end
			end
		ensure
			valid_flags_attached: Result /= Void
			same_lower: Result.lower = lower
			same_upper: Result.upper = upper
		end

	values_intersect (a_linear: LINEAR [G]): BOOLEAN
			-- Do the elements of a_linear intersect our elements?
			-- Violates CQS by trashing the internal cursor.
		require
			a_linear_attached: a_linear /= Void	
		do
			from
				a_linear.start
			until
				Result or else a_linear.off
			loop
				Result := has_value (a_linear.item)
				a_linear.forth
			end
		end

	references_intersect (a_linear: LINEAR [G]): BOOLEAN
			-- Does `Current' intersect with `a_linear'?
			-- Violates CQS by trashing the internal cursor.
		require
			valid_linear: a_linear /= Void
		do
			from
				a_linear.start
			until
				Result or else a_linear.off
			loop
				Result := has_reference (a_linear.item)
				a_linear.forth
			end
		end

	min_max (a_comparison: FUNCTION [ANY, TUPLE [G, G], BOOLEAN]): detachable PAIR [G, G]
			-- Smallest and largest items (as determined by `a_comparison') in this array;
			-- Void if `Current' is empty;
			-- Violates CQS by trashing the internal cursor.
		require
			a_comparison_attached: a_comparison /= Void
		do
			if not is_empty then
				from
					start
					create Result.make (item (lower), item (lower))
				until
					off
				loop
					if a_comparison.item ([current_item, Result.first]) then
						Result.set_first (current_item)
					end
					if a_comparison.item ([Result.second, current_item]) then
						Result.set_second (current_item)
					end
					forth
				end
			end
		end

	equivalent_from_order (a_order: FUNCTION [ANY, TUPLE [G, G], BOOLEAN]; a_1: G; a_2: G): BOOLEAN
			-- Are `a_1' and `a_2' equivalent according to `a_order' (a "less than" function)?
		require
			a_order_attached: a_order /= Void
		do
			Result := (not a_order.item ([a_1, a_2])) and (not a_order.item ([a_2, a_1]))
		end

	quotient_with_less_than_function (a_order: FUNCTION [ANY, TUPLE [G, G], BOOLEAN]): ROSE_LINEAR_ARRAY [ROSE_LINEAR_ARRAY [G]]
			-- Quotient where equivalence is determined by `a_order' (a "less than" function);
			-- Violates CQS by trashing the internal cursor.
		require
			a_order_attached: a_order /= Void
		do
			Result := ordered_quotient (a_order, agent equivalent_from_order (a_order, ?, ?))
		ensure
			quotient_with_less_than_function_attached: Result /= Void
		end

	ordered_quotient (a_order: FUNCTION [ANY, TUPLE [G, G], BOOLEAN]; a_equivalent: FUNCTION [ANY, TUPLE [G, G], BOOLEAN]): ROSE_LINEAR_ARRAY [ROSE_LINEAR_ARRAY [G]]
			-- Quotient using `a_equivalent' obtained by sorting copy of `Current' with `a_order';
			-- Violates CQS by trashing the internal cursor.
		require
			a_order_attached: a_order /= Void
			a_equivalent_attached: a_equivalent /= Void
		local
			l_array: ROSE_LINEAR_ARRAY [G]
		do
			create l_array.make_from_array (Current)
			l_array.sort_by (a_order)
			Result := l_array.quotient (a_equivalent)
		ensure
			ordered_quotient_attached: Result /= Void
		end

	quotient (a_equivalent: FUNCTION [ANY, TUPLE [G, G], BOOLEAN]): ROSE_LINEAR_ARRAY [ROSE_LINEAR_ARRAY [G]]
			-- Quotient using `a_equivalent';
			-- assumes you have sorted `Current';
			-- Violates CQS by trashing the internal cursor.	
		local
			l_array: ROSE_LINEAR_ARRAY [G]
			l_last: G
			l_default: detachable G
		do
			create Result.make_empty
			from
				start
				if ({G}).has_default then
					l_last := ({G}).default
				else
					check attached l_default as l_last_2 then
						-- G must be a self-initializing type
						l_last := l_last_2
					end
				end
				create l_array.make_empty
			until
				off
			loop
				if index = lower then
					l_array.extend (current_item)
					Result.extend (l_array)
				else
					if a_equivalent.item ([current_item, l_last]) then
						l_array.extend (current_item)
					else
						create l_array.make_empty
						Result.extend (l_array)
						l_array.extend (current_item)
					end
				end
				l_last := current_item
				forth
			end
		ensure
			quotient_attached: Result /= Void
		end

	is_sorted: BOOLEAN
			-- Is `Current' considered sorted according to `comparison'?
		require
			comparison_not_void: comparison /= Void
		local
			i: INTEGER
			l_last_item, l_current_item: like item
		do
			Result := True
			if count > 1 then
				check attached comparison as l_comparison then
						 -- precondition "comparison_not_void"
					from
						i := lower + 1
						l_last_item := item (lower)
					until
						i > upper or not Result
					loop
						l_current_item := item (i)
						Result := l_comparison.item ([l_last_item, l_current_item])
						l_last_item := l_current_item
						i := i + 1
					end
				end
			end
		end

	debug_output: STRING
			-- Display of `count' and possibly `upper' and `lower'.
		do
			Result := "count=" + count.out
			if lower /= 1 then
				Result.append (", lower=" + lower.out + ", upper=" + upper.out)
			end
		end

	filtered_by (a_predicate: PREDICATE [ANY, TUPLE [G]]): like Current
			-- Pruned version of `Current' whose elements pass `a_predicate'
		require
			a_predicate_attached: a_predicate /= Void
		local
			i: INTEGER
		do
			create Result.make_empty
			from
				i := lower
			until
				i > upper
			loop
				if a_predicate.item ([item (i)]) then
					Result.extend (item (i))
				end
				i := i + 1
			end
		ensure
			meaningful_result: Result /= Void and then Result.count <= count
		end

	subarray (a_start_pos, a_end_pos: INTEGER): like Current
			-- Array made of items of current array within
			-- bounds `a_start_pos' and `a_end_pos'.
		require
			valid_start_pos: valid_index (a_start_pos)
			valid_end_pos: a_end_pos <= upper
			valid_bounds: (a_start_pos <= a_end_pos) or else (a_start_pos = a_end_pos + 1)
		do
			create Result.make_from_array (array_subarray (a_start_pos, a_end_pos))
		ensure
			subarray_attached: Result /= Void
			lower: Result.lower = 1
			upper: Result.count = a_end_pos - a_start_pos + 1
		end

	reversed: like Current
			-- Reversed array from current (last swapped with first etc.)
		local
			i: INTEGER
		do
			Result := twin
			from
				i := upper
			until
				i < lower
			loop
				Result.put (item (i), lower + upper - i)
				i := i - 1
			end
		ensure
			reversed_attached: Result /= Void
		end

	sorted (a_comparison: like comparison): like Current
			-- Sorted version of `Current'
		require
			a_comparison_not_void: a_comparison /= Void
		do
			Result := twin
			Result.sort_by_and_remember (a_comparison)
		ensure
			sorted_attached: Result /= Void
			same_count: Result.count = count
			same_items: Result.for_all (agent has)
			is_sorted_by_a_comparison: a_comparison = Result.comparison and then Result.is_sorted
		end

	comparison: detachable FUNCTION [ANY, TUPLE [G, G], BOOLEAN]
			-- Agent used in `sort'
	
feature -- Basic operations

	sort_by (a_comparison: FUNCTION [ANY, TUPLE [G, G], BOOLEAN])
			-- Sort this array using `a_comparison'.
			-- `Comparison' becomes detached.
		require
			a_comparison_attached: a_comparison /= Void
		do
			comparison := a_comparison
			sort
			comparison := Void
		ensure
			comparison_detached: comparison = Void
		end

	sort_by_and_remember (a_routine: detachable FUNCTION [ANY, TUPLE [G, G], BOOLEAN])
			-- Sort this array using `a_routine' if non-Void, remembering it for further searching
		require
			need_comparison: a_routine /= Void or comparison /= Void
		do
			if a_routine /= Void then
				comparison := a_routine
			end
			sort
		ensure
			comparison_attached: comparison /= Void
		end

	set_comparison (a_comparison: like comparison)
			-- Assign `a_comparison' to `comparison'.
		do
			comparison := a_comparison
		ensure
			comparison_aliased: comparison = a_comparison
		end

	binary_search (a_g: G)
			-- Binary search for item `a_g';
			-- `Current' better be sorted ...
		do
			Precursor (a_g)
			index := found_index
		end

	linear_search (a_g: G)
			-- Linear search (look at every item) for item `a_g'
		do
			Precursor (a_g)
			index := found_index
		end

	concatenate (a_linear: detachable LINEAR [G])
			-- Appends each element of `a_linear' to the end of this array.
			-- Trashes internal cursor of `a_linear'.
		do
			if a_linear /= Void then
				from
					a_linear.start
				until
					a_linear.off
				loop
					extend (a_linear.item)
					a_linear.forth
				end
			end
		end

	merge_values (a_linear: LINEAR [G])
			-- Merge all values from `a_linear' into `Current' if not already contained.
			-- Trashes internal cursor of `a_linear'.
		require
			a_linear_attached: a_linear /= Void
		do
			from
				a_linear.start
			until
				a_linear.off
			loop
				if not has_value (a_linear.item) then
					extend (a_linear.item)
				end
				a_linear.forth
			end
		end

	merge_references (a_linear: LINEAR [G])
			-- Add items from `a_linear' not present in `Current' to end of `Current'.
			-- Trashes internal cursor of `a_linear'.
		require
			a_linear_attached: a_linear /= Void
		do
			from
				a_linear.start
			until
				a_linear.off
			loop
				if not has_reference (a_linear.item) then
					extend (a_linear.item)
				end
				a_linear.forth
			end
		end
	
feature -- Set functions
	
	union_values (a_linear: LINEAR [G]): like Current
			-- `Current' merged with `a_linear' using value comparison;
				-- Violates CQS by trashing internal cursor of `a_linear'.
		require
			a_linear_attached: a_linear /= Void
		do
			Result := twin
			Result.merge_values (a_linear)
		ensure
			union_values_attached: Result /= Void
		end

	union_references (a_linear: LINEAR [G]): like Current
			-- `Current' merged with `a_linear' using reference comparison;
			-- Violates CQS by trashing internal cursor of `a_linear'.
		require
			a_linear_attached: a_linear /= Void
		do
			Result := twin
			Result.merge_references (a_linear)
		ensure
			union_references_attached: Result /= Void
		end

	union_values_is_equal (a_linear: LINEAR [G]): like Current
			-- `Current' merged with `a_linear' using `~' if `object_comparison' and `=' otherwise;
			-- Violates CQS by trashing internal cursor of `a_linear'.
		do
			Result := twin
			from
				a_linear.start
			until
				a_linear.off
			loop
				if not has (a_linear.item) then
					Result.extend (a_linear.item)
				end
				a_linear.forth
			end
		ensure
			union_values_is_equal_attached: Result /= Void			
		end

	intersection_values (a_linear: LINEAR [G]): like Current
			-- Intersection of `Current' and values of `a_linear', according to `object_comparison'
			-- Violates CQS by trashing internal cursor of `a_linear'.
		do
			create Result.make_empty
			from
				a_linear.start
			until
				a_linear.off
			loop
				if has (a_linear.item) then
					Result.extend (a_linear.item)
				end
				a_linear.forth
			end
		ensure
			intersection_values_attached: Result /= Void
		end

	ordered_union_values (a_other: like Current; a_comparison: attached like comparison; a_distinct: BOOLEAN): like Current
			-- Full linear merge yielding ordered union of `Current' and `a_other';
			-- `Current' and `a_other' must already be in the order specified by `a_comparison'.
			-- Equal elements are dropped if `a_distinct' is `True'
		require
			a_other_attached: a_other /= Void
			a_comparison_attached: a_comparison /= Void
		local
			i, j: INTEGER
		do
			if count = 0 then
				Result := a_other.twin
			elseif a_other.count = 0 then
				Result := twin
			else
				create Result.make_empty
				from i := lower; j := a_other.lower
				until i > upper and j > a_other.upper
				loop
					if j > a_other.upper or else (i <= upper and a_comparison.item ([item (i), a_other.item (j)])) then
						Result.extend (item (i))
						i := i + 1
					elseif i > upper or else a_comparison.item ([a_other.item (j), item (i)]) then
						Result.extend (a_other.item (j))
						j := j + 1
					else
						Result.extend (item (i))
						if not a_distinct then
							Result.extend (a_other.item (j))
						end
						i := i + 1
						j := j + 1
					end
				end
			end
		ensure
			ordered_union_values_attached: Result /= Void
		end

	selected_items (a_ixs: detachable ARRAY [INTEGER]): detachable like Current
			-- Items from indices in `a_ixs'
		local
			i, l_ix: INTEGER
		do
			if a_ixs /= Void then
				create Result.make (1, a_ixs.count)
				from
					i := 1
				until
					i > a_ixs.count
				loop
					l_ix := a_ixs.item (i)
					if valid_index (l_ix) then
						Result.put (item (l_ix), i)
					end
					i := i + 1
				end
			end
		end

feature -- Iteration

	do_while (a_action: PROCEDURE [ANY, TUPLE [G]]; a_test: FUNCTION [ANY, TUPLE [], BOOLEAN])
			-- Apply `a_action' to every item of target up to
			-- first one not satisfying `a_test'
		require
			a_action_attached: a_action /= Void
			a_test_attached: a_test /= Void
		local
			i, nb: INTEGER
		do
			from
				i := lower
				nb := upper
			until
				i > nb or not a_test.item ([])
			loop
				a_action.call ([item (i)])
				i := i + 1
			end
		end

feature {NONE} -- Comparison

	less_than (a, b: G): BOOLEAN
			-- Is `a' less than `b'?
		do
			if a /= Void and b /= Void then
				if attached comparison as l_comparison then
					Result := l_comparison.item ([a, b])
				else
					(create {EXCEPTIONS}).raise ("Comparison is Void in `less_than'")
				end
			elseif a /= Void and b = Void then
				Result := True
			else
				Result := False
			end
		end

end
