note

	description:

		"Routines for representing an object as a string"

	author: "Colin Adams (code refactored - originally written by Zoran Simic)"
	last_editor: "$Author: c4 $"
	date: "$Date: 2014/05/13 $"
	revision: "$Revision: #1 $"
	path: "$File: //rose/source/library/containers_core/src/rose_object_representation.e $"
	remark: "[
             This class belongs in a new cluster.
             For the moment it is located in rose_containers_core, as this has the correct
              dependencies. Feel free to move it to a more approriate library
              as soon as a client needs it but does/should not depend upon rose_containers_core.
           ]"
	
class ROSE_OBJECT_REPRESENTATION

feature -- Access

	represented (a_object: detachable ANY): STRING
			-- `a_object' represented in a STRING
			-- "" if object is Void
		local
			l_retried: BOOLEAN
		do
			if l_retried then
				Result := "<Crashed>"
			else
				Result := object_representation (a_object, "")
			end
		ensure
			represented_attached: Result /= Void
		rescue
			if not l_retried then
				l_retried := True
				retry
			end
		end
	
	frozen object_representation (a_object: detachable ANY; a_void: detachable STRING): STRING
			-- Textual representation of `a_object', `a_void' if object is Void;
			-- If both arguments are Void, then the empty string is returned
		local
			i: INTEGER
		do
			if attached {DEBUG_OUTPUT} a_object as l_debug then
				Result := l_debug.debug_output.as_string_8
			elseif attached {DATE_TIME_DURATION} a_object as l_date_time_duration then
				Result := duration_representation (l_date_time_duration.seconds_count, False)
			elseif attached {TIME_DURATION} a_object as l_time_duration then				
				Result := duration_representation (l_time_duration.seconds_count, True)
			elseif attached {DATE_DURATION} a_object as l_duration then
				Result := l_duration.days_count.out + s_days				
			elseif attached {TRAVERSABLE [detachable ANY]} a_object as l_traversable then
				create Result.make_empty
				l_traversable.do_all (agent append_object_representation (Result, ?, a_void))
			elseif attached {ARRAY [detachable ANY]} a_object as l_array then
				create Result.make_empty
				l_array.do_all (agent append_object_representation (Result, ?, a_void))
			elseif attached a_object as l_object then
				Result := l_object.out
			else
				if attached a_void as l_void then
					Result := l_void
				else
					Result := ""
				end				
			end
		ensure
			object_representation_attached: Result /= Void
		end
	
	frozen duration_representation (a_seconds: INTEGER_64; a_show_seconds: BOOLEAN): STRING
			-- `a_seconds' represented as "Xh Xm Xs"
		local
			l_days, l_hrs, l_min, l_sec: INTEGER_64
			l_show_seconds: BOOLEAN
		do
			create Result.make (10)
				-- Show seconds part only if requested and duration is < 3 hours
			l_show_seconds := a_show_seconds and then a_seconds < 10800
			l_sec := a_seconds \\ 60
			if not l_show_seconds and then l_sec > 30 then
				l_min := (a_seconds + 30) // 60
			else
				l_min := a_seconds // 60
			end
			l_hrs := l_min // 60
			l_min := l_min \\ 60
			l_days := l_hrs // 24
			l_hrs := l_hrs \\ 24
			if l_days > 0 then
				Result.append_string (l_days.out)
				Result.append_character ('d')
				Result.append_character (' ')
				if l_hrs < 10 then
					Result.append_character ('0')
				end
			end
			Result.append_string (l_hrs.out)
			Result.append_character ('h')
			Result.append_character (' ')
			if l_min < 10 then
				Result.append_character ('0')
			end
			Result.append_string (l_min.out)
			Result.append_character ('m')
			Result.append_character (' ')
			if l_show_seconds then
				if l_sec < 10 then
					Result.append_character ('0')
				end
				Result.append_string (l_sec.out)
				Result.append_character ('s')
				Result.append_character (' ')
			end
		ensure
			duration_representation_attached: Result /= Void
		end

feature {NONE} -- Implementation
	
	frozen append_object_representation (a_target: STRING; a_object: detachable ANY; a_void: detachable STRING)
			-- Append object representation of `a_object' to `a_target'
			-- `a_void' is used to represent Void objects. If this is itself `Void',
			--  then the empty string is used.
		require
			target_ok: a_target /= Void
		do
			a_target.append_string (object_representation (a_object, a_void))
			a_target.append_character ('%N')
		end
	
feature {NONE} -- Constants

	s_days: STRING = "d "
			-- Indicator that numeric literal is a number of days

end
