note

	description:

		"Rosenberg Date Utility"

	author: "Mark Howard"
	last_editor: "$Author: it $"
	date: "$Date: 2014/11/25 $"
	revision: "$Revision: #11 $"
	path: "$File: //rose/source/library/datetime/src/rose_date.e $"
	storable_version: ""

class	ROSE_DATE

inherit RP_SERIALIZABLE

	ROSE_CHRONOLOGICAL
		rename
			add as date_add,
			duration as rose_date_duration,
			relative_duration as rose_date_relative_duration,
			make_from_string_default as make_from_date_string_default,
			make_from_string as make_from_date_string
		undefine
			correct_mismatch
		redefine
			default_create,
			debug_output,
			formatted_out,
			hash_code,
			make_ccyymmdd,
			make_from_rose_julian_day,
			make_from_date_string_default,
			make_now,
			make_now_utc,
			make_today,
			out,
			persecs,
			to_date_time
		end

	MISMATCH_CORRECTOR
		undefine
			correct_mismatch,
			is_equal,
			out,
			default_create
		end

create

	default_create,
	make,
	make_undetermined,
	make_unavailable,
	make_fine,
	make_now,
	make_today,
	make_today_utc,
	make_ccyymmdd,
	make_ccyymmdd_frac,
	make_mmddccyy,
	make_yymmdd,
	make_by_date_time,
	make_by_datetime,
	make_by_date,
	make_max_date,
	make_min_date,
	make_from_string_default,
	make_from_string,
	make_from_code,
	make_date_from_code,
	make_by_double,
	make_from_date_string_default,
	make_by_unix_time,
	make_from_ccyymmddhhmmss_string,
	make_from_ccyymmddhhmmss_frac_string,
	make_from_julian_quarter,
	make_from_julian_month,
	make_from_rose_julian_day,
	make_w_delimited_dd_mmm_yy_string,
	make_w_delimited_dd_mmm_ccyy_string,
	make_w_delimited_mmddyy_string,
	make_w_delimited_mmddccyy_string,
	make_unchecked,
	make_by_days,
	make_day_month_year,
	make_date_time,
	make_month_day_year,
	make_w_string,
	make_ccyymm_end,
	make_by_compact_date,
	make_by_ordered_compact_date,
	make_ccyymmdd_forgiving,
	make_now_utc,
	make_by_ordered_compact_date_and_persecs,
	make_from_iso8601_date_time_string,
	make_from_iso8601_date_time

convert

	date_time: {DATE_TIME},
	make_by_date({DATE}),
	make_by_datetime({DATE_TIME})

feature {NONE} -- Initialization

	default_create
			-- <Precursor>
		do
			make_undetermined
		end

	make_date_time (a_year, a_month, a_day, a_hour, a_minute, a_second: INTEGER)
			-- Create with `a_year', `a_month' `a_day',
			-- `a_hour', `a_minute', and `a_second'.
		require
			correct_date: is_correct_date (a_year, a_month, a_day)
		do
			make (a_year, a_month, a_day)
			persecs := persecs_in (a_hour, a_minute, a_second)
		end

	make_now
			-- Get the date and the time from the system in local time zone.
		local
			l_date: C_DATE
			l_seconds: DOUBLE
			l_dummy: ROSE_DATE_TIME
		do
			create l_date
			make (l_date.year_now, l_date.month_now, l_date.day_now)
			l_seconds := l_date.hour_now * 3_600.0 + l_date.minute_now * 60.0 + l_date.second_now + l_date.millisecond_now / 1_000.0
			persecs := (l_seconds * 10_000).truncated_to_integer

			-- keep ROSE_DATE_TIME (obsolete placeholder class)
			-- alive while it is still in some storables (these will be converted soon)
			if 2 = 4 then
				l_dummy := to_rose_date_time
			end
		end

	make_now_utc
			-- Get the UTC date and the time from the system.
		local
			l_date: C_DATE
			l_seconds: DOUBLE
		do
			create l_date.make_utc
			make (l_date.year_now, l_date.month_now, l_date.day_now)
			l_seconds := l_date.hour_now * 3_600.0 + l_date.minute_now * 60.0 + l_date.second_now + l_date.millisecond_now / 1_000.0
			persecs := (l_seconds * 10_000).truncated_to_integer
		end

	make_today
			-- Get the current date in local time zone but set the time to midnight.
		local
			l_date: C_DATE
		do
			create l_date
			make (l_date.year_now, l_date.month_now, l_date.day_now)
			persecs := 0
		end

	make_today_utc
			-- Get the current date in UTC time zone but set the time to midnight.
		local
			l_date: C_DATE
		do
			create l_date.make_utc
			make (l_date.year_now, l_date.month_now, l_date.day_now)
			persecs := 0
		end

	make_fine (a_year, a_month, a_day, a_hour, a_minute: INTEGER; a_second: DOUBLE)
			-- Create with `a_year', `a_month' `a_day',
			-- `a_hour', `a_minute', and `a_second'.
		require
			correct_date: is_correct_date (a_year, a_month, a_day)
			correct_time: True -- {TIME_VALIDITY_CHECKER}.is_correct_time (h, m, s, False)
		local
			l_rose_time: ROSE_TIME
		do
			make (a_year, a_month, a_day)
			create l_rose_time.make_fine (a_hour, a_minute, a_second)
			persecs := l_rose_time.persecs
		end

	make_max_date
			-- Make `max date' (bigger than all other date).
		do
			make (year_for_max_date, month_for_max_date, day_for_max_date)
			persecs := persecs_in (hour_for_max_date, minute_for_max_date, second_for_max_date)
		end

	make_min_date
			-- Make `min_date'.
		do
			make (year_for_min_date, month_for_min_date, day_for_min_date)
			persecs := persecs_in (hour_for_min_date, minute_for_min_date, second_for_min_date)
		end

	make_from_string_default (a_string: STRING)
			-- Make from a_string using default_format_string (from TIME_UTILITY).
		require
			a_string_attached: a_string /= Void
			valid_string: date_time_validity_checker.date_time_valid (a_string, default_format_string)
		local
			l_date_time: DATE_TIME
		do
			create l_date_time.make_from_string (a_string, default_format_string)
			make_by_date_time (l_date_time.date, l_date_time.time)
		end

	make_from_date_string_default (a_string: STRING_8)
			-- Make from `a_string' using default date format string.
		do
			make_by_date (create {DATE}.make_from_string_default (a_string))
		end

	make_from_string (a_string: STRING; a_code: STRING)
			-- Make with `a_string' and representation `a_code'.
		local
			l_date_time: DATE_TIME
			l_date: DATE
		do
			if date_time_validity_checker.date_time_valid (a_string, a_code) then
				create l_date_time.make_from_string (a_string, a_code)
				make_by_date_time (l_date_time.date, l_date_time.time)
			elseif date_validity_checker.date_valid (a_string, a_code) then
				create l_date.make_from_string (a_string, a_code)
				make_by_date (l_date)
			else
				make_undetermined
			end
		end

	make_from_code (a_string: STRING; a_code_string: DATE_TIME_CODE_STRING)
			-- Make with `a_string' as representation `a_code_string'.
		require
			a_string_not_void: a_string /= Void
			a_code_string_not_void: a_code_string /= Void
			a_code_string_precise: a_code_string.precise
			a_string_corresponds_to_a_code_string: a_code_string.correspond (a_string)
			a_string_valid: a_code_string.is_value_valid (a_string)
		local
			l_date_time: DATE_TIME
		do
			l_date_time := a_code_string.create_date_time (a_string)
			make_by_date_time (l_date_time.date, l_date_time.time)
		end

	make_by_unix_time (a_unix_time: INTEGER)
			-- UNIX time is the integer number of seconds since midnight 1/1/1970.
		local
			l_days_since_unix_origin: INTEGER
			l_seconds: INTEGER
		do
			l_days_since_unix_origin := a_unix_time // seconds_in_day
			l_seconds := a_unix_time \\ seconds_in_day
			make_by_days (l_days_since_unix_origin + unix_origin.days)
			persecs := l_seconds * rose_ticks_per_second
		end

	make_date_from_code (a_string: STRING; a_code_string: DATE_TIME_CODE_STRING)
			-- Make with `a_string' and representation `a_code_string'.
		require
			a_string_attached: a_string /= Void
			a_code_string_attached: a_code_string /= Void
			a_code_string_precise: a_code_string.precise_date
			a_string_valid_for_a_code_string: a_code_string.correspond (a_string)
		local
			l_date: DATE
		do
			l_date := a_code_string.create_date (a_string)
			make_by_date (l_date)
		end

	make_by_double (a_double: DOUBLE)
			-- Make with `a_double, which is expected of the same form as `to_double'
		local
			i: INTEGER
			h, m, s: INTEGER
		do
			make_by_days (a_double.floor)
			i := ((a_double - a_double.floor) * 86400).rounded
			h := i // 3600
			i := i - h * 3600
			m := i // 60
			i := i - m * 60
			s := i
			persecs := persecs_in (h, m, s)
		end

	make_from_ccyymmddhhmmss_string (a_string: STRING)
			-- The string is of the form CCYYMMDDHHMMSS (extra characters are ignored).
		require
			a_string_not_void: a_string /= Void -- and as documented in the header comment
		do
			make_from_ccyymmddhhmmss_frac_string(a_string.substring(1, a_string.count.min(14)))
		end

	make_from_ccyymmddhhmmss_frac_string (a_string: STRING)
			-- Like make_from_ccyymmddhhmmss_string but it adds the fractional seconds.
			-- The string is of the form CCYYMMDDHHMMSSFFF (from 0 to 4 F's).
			-- There is no decimal point before the F's.
		require
			a_string_not_void: a_string /= Void  -- and as documented in the header comment
		local
			l_string: STRING
			h: INTEGER
			m: INTEGER
			s: DOUBLE
		do
			l_string := a_string.substring(1, 8)
			if l_string.is_integer then
				make_ccyymmdd(l_string.to_integer)
				if a_string.count >= 14 then
					l_string := a_string.substring(9, a_string.count) + "0000"
					l_string := l_string.substring(1, 10)
					if l_string.is_double then
						h := l_string.substring(1, 2).to_integer
						m := l_string.substring(3, 4).to_integer
						s := l_string.substring(5, 10).to_double
						persecs := persecs_in(h, m, s / rose_ticks_per_second)
					end
				end
			end
		end

	make_from_rose_julian_day (a_julian_day: INTEGER)
			--make from julian day ... i.e. number of days since 1/1/1950
		do
			Precursor (a_julian_day)
			persecs := 0
		end

	make_unchecked (a_year, a_month, a_day, a_hour, a_minute: INTEGER; a_second: DOUBLE)
			-- Create with `a_year', `a_month' `a_day',
			-- `a_hour', `a_minute', and `a_second'.
		do
			if is_correct_date (a_year, a_month, a_day) then
				set_date (a_year, a_month, a_day)
				persecs := persecs_in (a_hour, a_minute, a_second)
			else
				make_unavailable
			end
		end

	make_ccyymm_end (a_ccyymm: INTEGER)
			-- Make at month end from 'a_ccyymm'
		do
			make_ccyymmdd (a_ccyymm * 100 + 1)
			set_to_end_of_month
		end

	make_by_datetime (a_datetime: DATE_TIME)
			-- Create with 'a_datetime'.
		require
			nonvoid : a_datetime /= void
		do
			make_by_ordered_compact_date (a_datetime.date.ordered_compact_date)
			persecs := (a_datetime.time.fine_seconds * rose_ticks_per_second).truncated_to_integer
		end

	make_by_date_time (a_date: DATE; a_time: TIME)
			-- Create with 'a_date' and 'a_time'.
		require
			nonvoid : a_date /= void and then a_time /= void
		do
			make_by_ordered_compact_date (a_date.ordered_compact_date)
			persecs := (a_time.fine_seconds * rose_ticks_per_second).truncated_to_integer
		end

	make_ccyymmdd (a_ccyymmdd: INTEGER)
			-- Make with 'a_ccyymmdd'.
		do
			Precursor (a_ccyymmdd)
			persecs := 0
		end

	make_ccyymmdd_frac(a_ccmmyydd_frac: DOUBLE)
			-- The argument is a double in the form CCYYMMDD.frac where 'frac' is the fraction of a day.
			-- You can obtain such a date with the following code:
			--		l_date.ccyymmdd + l_date.fractional_day
			-- Useful if you need to store a date (down to the fractional seconds) in a DOUBLE.
		local
			l_ccyymmdd: INTEGER
		do
			l_ccyymmdd := a_ccmmyydd_frac.truncated_to_integer
			make_ccyymmdd(l_ccyymmdd)
			if not is_undetermined
			then
				persecs := ((a_ccmmyydd_frac - l_ccyymmdd) * Seconds_in_day * Rose_ticks_per_second).truncated_to_integer
			end
		end

	make_by_date (a_date: DATE)
			-- Create with `a_date' and time to origin of time.
		do
			make_by_ordered_compact_date (a_date.ordered_compact_date)
			persecs := 0
		end

	make_by_ordered_compact_date_and_persecs (a_ordered_compact_date, a_persecs: INTEGER)
			-- Create a new date from `a_ordered_compact_date' and `a_persecs'.
		require
			ordered_comapct_date_valid: ordered_compact_date_valid (a_ordered_compact_date)
		do
			ordered_compact_date := a_ordered_compact_date
			persecs := a_persecs
		ensure
			ordered_compact_date_set: ordered_compact_date = a_ordered_compact_date
			persecs_set: persecs = a_persecs
		end

	make_from_iso8601_date_time_string (a_iso: STRING)
			-- Make from `a_iso'.
			-- `a_iso' must be of form "YYYY-MM-DDThh:mm:ss[,sss]",
			-- with optional timezone in form:
			--	Z
			--	+hhmm
			--	-hhmm
			-- Date-time will be converted to UTC.
		require
			a_iso_attached: a_iso /= Void
			a_sio_valid: iso_8601_factory.valid_iso8601_date_time (a_iso)
		local
			l_iso: ISO8601_DATE_TIME
		do
			l_iso := iso_8601_factory.create_iso8601_date_time (a_iso)
			make_from_iso8601_date_time (l_iso)
		end

	make_from_iso8601_date_time (a_iso: ISO8601_DATE_TIME)
			-- Make from `a_iso'.
			-- Date-time will be converted to UTC.
		require
			a_iso_attached: a_iso /= Void
		do
			make_fine (a_iso.year, a_iso.month, a_iso.day, a_iso.hour, a_iso.minute, a_iso.second.to_double + a_iso.fractional_second)
			if attached a_iso.timezone as l_tz and then not l_tz.is_gmt then
				second_add (l_tz.to_seconds)
			end
		end

feature -- Access

	iso_8601_factory: ISO8601_FACTORY
			-- For IS 8601 date formats
		once
			create Result
		ensure
			iso_8601_factory_attached: Result /= Void
		end

	date_time_validity_checker: DATE_TIME_VALIDITY_CHECKER
			-- Shared instance of validity checker
		once
			create Result
		ensure
			date_time_validity_checker_attached: Result /= Void
		end

	date_validity_checker: DATE_VALIDITY_CHECKER
			-- Shared instance of validity checker
		once
			create Result
		ensure
			date_validity_checker_attached: Result /= Void
		end

	unix_seconds: INTEGER_64
			-- Number of seconds elapsed since `unix_origin'.
		local
			l_time_duration: DATE_TIME_DURATION
			l_date: like Current
		do
			create l_date.make (unix_origin.year, unix_origin.month, unix_origin.day)
			l_time_duration := relative_duration (l_date)
			Result := l_time_duration.seconds_count
		end

	date: ROSE_DATE
			-- Date equal to `Current', but without any time part
		do
			create Result.make_by_ordered_compact_date (ordered_compact_date)
		ensure
			date_attached: Result /= Void
		end

	time: ROSE_TIME
			-- Workaround for bug in EiffelTime {TIME}.make_by_fine_seconds
		local
			h, m: INTEGER
			s: DOUBLE
		do
			h := persecs // rose_ticks_per_second // 3600
			m := persecs // rose_ticks_per_second // 60 \\ 60
			s := persecs // rose_ticks_per_second \\ 60 + persecs \\ rose_ticks_per_second / rose_ticks_per_second
			create Result.make_fine (h, m, s)
		ensure
			time_attached: Result /= Void
		end

	date_time: DATE_TIME
			-- Corresponding DATE_TIME object
		do
			create Result.make_fine(year, month, day, hour, minute, second.to_double + fractional_second)
		ensure
			date_time_attached: Result /= Void
		end

	hour: INTEGER
			-- Hour within day
		do
			Result := time.hour
		end

	minute: INTEGER
			-- Minute within hour
		do
			Result := time.minute
		end

	second: INTEGER
			-- Second within minute
		do
			Result := time.second
		end

	fractional_second: DOUBLE
			-- Fractional second within second
		do
			Result := time.fractional_second
		end

	persecs: INTEGER
			-- 10000ths of seconds elapsed since midnight (see rose_ticks_per_second)

	fractional_day: DOUBLE
			-- The time part as a fraction;  For instance, 18:00 would return 0.75.
		do
			Result := persecs / Rose_ticks_per_second / Seconds_in_day
		end

	duration: DATE_TIME_DURATION
			-- Duration since `origin'
		do
			create Result.make_by_date_time (rose_date_duration, time.duration)
		ensure
			duration_attached: Result /= Void
		end

	unix_origin: ROSE_DATE
			-- January 1, 1970
		once
			create Result.make (1970, 1, 1)
		ensure
			unix_oriigin_attached: Result /= Void
			correct_year: Result.year = 1970
			correct_month: Result.month = January
			first_day_of_month: Result.day = 1
		end

	max_date: ROSE_DATE
			-- Date treated as indistinguishable from infinity
		once
			create Result.make_max_date
		ensure
			max_date_attached: Result /= Void
		end

	min_date: ROSE_DATE
			-- January 1st 1900
		once
			create Result.make_min_date
		ensure
			min_date_attached: Result /= Void
		end

	relative_duration (a_other: like Current): DATE_TIME_DURATION
			-- Duration from `a_other' to `Current'
		do
			Result := date_time.relative_duration (a_other.date_time)
		ensure
			relative_duration_attached: Result /= Void
		end

	relative_duration_definite (a_other: like Current): DATE_TIME_DURATION
			-- Definite duration from `a_other' to `Current'
			-- (definite duration is independent of dates; uses days, not years or months)
		do
			Result := relative_duration (a_other)
			if not Result.definite then
				Result.date.to_definite (a_other)
			end
		ensure
			result_is_definite: Result /= Void and then Result.definite
		end

	relative_duration_in_seconds (a_other: like Current): DOUBLE
			-- Duration from `a_other' to `Current'
		local
			l_time_duration: DATE_TIME_DURATION
		do
			l_time_duration := relative_duration_definite (a_other)
			Result := ((l_time_duration.day * 24 + l_time_duration.hour) * 60 + l_time_duration.minute) * 60 + l_time_duration.fine_second
		end

	not_max: like Current
			-- Same date as `Current', unless `Current' is `max_date', in which case the day before `max_date'
		do
			if date.year = year_for_max_date then
				create Result.make (year_for_max_date, month_for_max_date, day_for_max_date - 1)
			else
				Result := Current
			end
		ensure
			not_max_attached: Result /= Void
			not_max_date: not Result.is_max_date
		end

feature -- Comparison

	hash_code: INTEGER
			-- <Precursor>
		do
			Result := (compact_date + persecs).hash_code
		end

feature -- Status Report

	is_max_date: BOOLEAN
			-- Is `Current' equal to `max_date'?
		do
			Result := same_date_time (max_date)
		end

	is_min_date: BOOLEAN
			-- Is `Current' equal to `min_date'?
		do
			Result := same_date_time (min_date)
		end

feature -- Conversion

	to_date_time: DATE_TIME
			-- Corresponding DATE_TIME object
		do
			Result := date_time
		ensure then
			to_date_time_attached: Result /= Void
		end

	to_double: DOUBLE
			-- Date represented as a DOUBLE, Excel style (but with origin being 1600/1/1)
		do
			Result := days + fractional_day
		end

	to_quarter: ROSE_QUARTER
			-- End of quarter (31st March, 30th June, 30th September or 31st December)
		do
			create Result.make (year, quarter)
		ensure
			result_not_void: Result /= Void
			same_year_and_quarter: (Result.year = year) and (Result.quarter = quarter)
		end

	to_rose_date_time: ROSE_DATE_TIME
			-- Convert to a rose date time
		do
			if attached {ROSE_DATE_TIME} Current as l_result then
				Result := l_result
			else
				create Result.make_by_ordered_compact_date_and_persecs (ordered_compact_date, persecs)
			end
		end

feature -- Basic operations

	correct_mismatch
			-- Ignore errors when restoring storables.
		do
		end

feature -- Change

	add (a_duration: DATE_TIME_DURATION)
			-- Add `a_duration' to `Current'
		do
			add_n_years (a_duration.year)
			add_n_months (a_duration.month)
			add_n_days (a_duration.day)
			hour_add (a_duration.hour)
			minute_add (a_duration.minute)
			second_add (a_duration.fine_second)
		end

	set_to_midnight
			-- Set time part to 0
		do
			persecs := 0
		ensure
			midnight: persecs = 0
		end

	set_time (a_time: TIME)
			-- Set `time' to `a_time'.
		require
			a_time_attached: a_time /= Void
		do
			persecs := (a_time.fine_seconds * rose_ticks_per_second).truncated_to_integer
		end

	set_time_from_delimited_string (a_string: detachable STRING)
			-- Set `time' from `a_string'.
			-- Does nothing if `a_string' is not of form hh:mm:ss (where ':' is a wildcard)
		local
			l_h, l_m, l_s: INTEGER
		do
			if a_string /= Void and then a_string.count = 8 then
				l_h := a_string.substring (1, 2).to_integer
				l_m := a_string.substring (4, 5).to_integer
				l_s := a_string.substring (7, 8).to_integer
				persecs := persecs_in (l_h, l_m, l_s)
			end
		end

	hour_add (a_hours: INTEGER)
			-- Add `a_hours' to `Current'.
		do
			second_add (a_hours * 3600)
		end

	minute_add (a_mins: INTEGER)
			-- Add `a_mins' to `Current'.
		do
			second_add (a_mins * 60)
		end

	second_add (a_secs: DOUBLE)
			-- Add `a_secs' to `Current'.
		local
			l_persecs: INTEGER_64
			l_days: INTEGER
		do
			l_persecs := persecs.to_integer_64 + (a_secs * rose_ticks_per_second).truncated_to_integer_64
			l_days := (l_persecs // seconds_in_day // rose_ticks_per_second).to_integer
			persecs := (l_persecs \\ (seconds_in_day * rose_ticks_per_second)).to_integer
			if persecs < 0 then
				l_days := l_days - 1
				persecs := (seconds_in_day * rose_ticks_per_second) + persecs
			end

			if l_days /= 0 then
				add_n_days (l_days)
			end
		end

feature -- Output

	iso_8601_string: STRING
			-- Output in ISO8601 format;
			-- Assumes time zone is UTC
		local
			l_iso: ISO8601_DATE_TIME
		do
			create l_iso.make_date_time (to_date_time)
			Result := l_iso.as_string + "Z"
		ensure
			iso_8601_string_attached: Result /= Void
			valid_iso_8601_date_time_string: iso_8601_factory.valid_iso8601_date_time (Result)
		end

	out: STRING
			-- Textual representation
		local
			l_time: STRING
			l_rose_time: ROSE_TIME
			l_string: STRING
		do
			l_rose_time := time
			l_string := (1000000 + l_rose_time.hour * 10000 + l_rose_time.minute * 100 + l_rose_time.second).out
			l_time := l_string.substring (2, 7)
			Result:= ccyymmdd.out + l_time
		end

 	xslt_formatted_date (a_zone: detachable DT_FIXED_OFFSET_TIME_ZONE; a_picture: STRING;
		a_language, a_calendar, a_country: detachable STRING): ST_FORMAT_DATE_TIME_RESULT
 			-- `Current' formatted according to XSLT's format-date() function;
 			-- See http://www.w3.org/TR/xslt20/#format-date for a detailed description.
 			-- Passing `Void' for `a_language' defaults to "en".
 			-- Passing `Void' for `a_calendar' defaults to "CE".
 			-- Passing `Void' for `a_country' defaults to "US".
 		require
 			a_picture_not_void: a_picture /= Void
 		local
 			l_date: DT_DATE
 			l_routines: ST_DATE_TIME_ROUTINES
 		do
 			create l_routines
 			create l_date.make (year, month, day)
 			Result := l_routines.xslt_formatted_date (l_date, a_zone, a_picture, a_language, a_calendar, a_country)
 		ensure
 			xslt_formatted_date_not_void: Result /= Void
 		end

 	xslt_formatted_time (a_zone: detachable DT_FIXED_OFFSET_TIME_ZONE; a_picture: STRING;
		a_language, a_calendar, a_country: detachable STRING): ST_FORMAT_DATE_TIME_RESULT
 			-- `Current' formatted according to XSLT's format-time() function;
 			-- See http://www.w3.org/TR/xslt20/#format-date for a detailed description.
 			-- Passing `Void' for `a_language' defaults to "en".
 			-- Passing `Void' for `a_calendar' defaults to "CE".
 			-- Passing `Void' for `a_country' defaults to "US".
 		require
 			a_picture_not_void: a_picture /= Void
 		local
 			l_time: DT_TIME
 			l_milliseconds: INTEGER
 			l_routines: ST_DATE_TIME_ROUTINES
 		do
 			create l_routines
 			l_milliseconds := (persecs \\ 10000) // 10
 			create l_time.make_precise (hour, minute, second, l_milliseconds)
 			Result := l_routines.xslt_formatted_time (l_time, a_zone, a_picture, a_language, a_calendar, a_country)
 		ensure
 			xslt_formatted_time_not_void: Result /= Void
 		end

 	xslt_formatted_date_time (a_zone: detachable DT_FIXED_OFFSET_TIME_ZONE; a_picture: STRING;
		a_language, a_calendar, a_country: detachable STRING): ST_FORMAT_DATE_TIME_RESULT
 			-- `Current' formatted according to XSLT's format-date-time() function;
 			-- See http://www.w3.org/TR/xslt20/#format-date for a detailed description.
 			-- Passing `Void' for `a_language' defaults to "en".
 			-- Passing `Void' for `a_calendar' defaults to "CE".
			-- Passing `Void' for `a_country' defaults to "US".
 		require
 			a_picture_not_void: a_picture /= Void
 		local
 			l_date: DT_DATE_TIME
 			l_milliseconds: INTEGER
 			l_routines: ST_DATE_TIME_ROUTINES
 		do
 			create l_routines
 			l_milliseconds := (persecs \\ 10000) // 10
 			create l_date.make_precise (year, month, day, hour, minute, second, l_milliseconds)
 			Result := l_routines.xslt_formatted_date_time (l_date, a_zone, a_picture, a_language, a_calendar, a_country)
 		ensure
 			xslt_formatted_date_time_not_void: Result /= Void
 		end

	formatted_out (a_string: STRING): STRING
			-- Formatted representation
		do
			Result := formatter (a_string).create_string (date_time)
		ensure then
			formatted_out_attached: Result /= Void
		end

	formatted_time: STRING
			-- Formatted time as hh:mm:ss:fff with zero padded fractional seconds
		do
			Result := time.formatted_time
		ensure
			formatted_time_attached: Result /= Void
		end

	formatted_time_short: STRING
			-- Formatted time as hh:mm:ss
		do
			Result := time.formatted_out ("[0]hh:[0]mi:[0]ss")
		ensure
			formatted_time_short_attached: Result /= Void
		end

	debug_output: STRING
			-- Debug display
		do
			if persecs = 0 then
				Result := formatter ("yyyy/[0]mm/[0]dd").create_string (date_time)
			else
				Result := formatter ("yyyy/[0]mm/[0]dd hh:[0]mi:[0]ss.ff3").create_string (date_time)
			end
		end

	display: STRING
			-- Default representation
		do
			Result := ccyymmdd.out
		ensure
			display_attached: Result /= Void
		end

feature {NONE} -- Implementation

	formatter (a_string: STRING): DATE_TIME_CODE_STRING
			-- Formatter corresponding to represention in `a_string'
		require
			a_string_attached: a_string /= Void
		do
			formatters.search (a_string)
			if not formatters.found then
				create Result.make (a_string)
				formatters.force (Result, a_string)
			else
				check attached formatters.found_item as l_item then
						-- only this routine inserts into formatters, and it doesn't insert `Void'.
					Result := l_item
				end
			end
		ensure
			formatter_attached: Result /= Void
		end

	formatters: HASH_TABLE [DATE_TIME_CODE_STRING, STRING]
			-- Formatters cache
		once
			create Result.make (10)
		ensure
			formatters_attached: Result /= Void
		end

	year_for_max_date: INTEGER = 9999
			-- Year for `max_date'

	month_for_max_date: INTEGER = 12
			-- Month for `max_date'

	day_for_max_date: INTEGER = 31
			-- Day for `max_date'

	year_for_min_date: INTEGER = 1900
			-- Year for `min_date'

	month_for_min_date: INTEGER = 1
			-- Month for `min_date'

	day_for_min_date: INTEGER = 1
			-- Day for `min_date'

	hour_for_max_date: INTEGER = 0
			-- Hour for `max_date'

	minute_for_max_date: INTEGER = 0
			-- Minute for `max_date'

	second_for_max_date: INTEGER = 0
			-- Second for `max_date'

	hour_for_min_date: INTEGER = 0
			-- Hour for `min_date'

	minute_for_min_date: INTEGER = 0
			-- Minute for `min_date'

	second_for_min_date: INTEGER = 0
			-- Second for `min_date'

	seconds_in_day: INTEGER = 86400
			-- Number of seconds in one day (ignores leap seconds)

	persecs_in (a_hour, a_minute: INTEGER; a_second: DOUBLE): INTEGER
			-- Parsecs for `a_hour', `a_minute' and `a_second'
		do
			Result := (rose_ticks_per_second * (a_hour * 3600 + a_minute * 60 + a_second)).truncated_to_integer
		end

	rose_ticks_per_second: INTEGER = 10000
			-- Number of ticks per second

end
