note

	description:

		"Object representing a date where the day is the end of the month ... misguided"

	storable_version: ""
	author: "Mark Howard"
	last_editor: "$Author: c4 $"
	date: "$Date: 2014/01/08 $"
	revision: "$Revision: #2 $"
	path: "$File: //rose/source/library/datetime/src/rose_month.e $"

class	ROSE_MONTH

inherit

	RP_SERIALIZABLE

	ROSE_DATE
		rename
			make as rose_date_make,
			julian_month as rose_date_julian_month
		redefine
			make_from_julian_month,
			month_forth,
			month_back,
			first,last,
			month_add,
			year_add
		end

create

	make, make_ccyymm, make_from_julian_month, make_unavailable,make_from_julian_quarter

create {ROSE_MONTH}

	rose_date_make

feature -- Initialization

	make (a_ccyy, a_mm: INTEGER)
			-- Create with year `a_ccyy' and month `a_mm'.
		require
			correct_date: is_correct_date (a_ccyy, a_mm, 1)
		do
			rose_date_make (a_ccyy, a_mm, 1)
			set_to_end_of_month
			julian_month := rose_date_julian_month
		end

	make_ccyymm (a_ccyymm: INTEGER)
			-- Create with year `a_ccyymm' // 100 and month `a_ccyymm' \\ 100.
		require
			correct_date: is_correct_date (a_ccyymm // 100, a_ccyymm \\ 100, 1)
		do
			make (a_ccyymm // 100, a_ccyymm \\ 100)
		end

	make_from_julian_month (a_month: INTEGER)
			-- <Precursor>
		do
			copy_month (from_julian_month (a_month))
		end

	month_forth
			-- <Precursor>
		do
			Precursor
			julian_month := julian_month + 1
		ensure then
			julian_month_advanced: julian_month =  old julian_month + 1
		end

	month_back
			-- <Precursor>
		do
			Precursor
			julian_month := julian_month - 1
		ensure then
			julian_month_retreated: julian_month =  old julian_month - 1
		end

	month_add (m: INTEGER)
			-- Add `m' months to `Current'. Can move days backward.
		do
			make_from_julian_month (m + julian_month)
		end

	year_add (y: INTEGER)
			-- Add `y' years to `Current'.
		do
			make_from_julian_month (y * 12 + julian_month)
		end

feature -- Access

	next_month: ROSE_MONTH
			-- Month immediately after `Current'.
		do
			Result := from_julian_month (julian_month + 1)
		ensure
			next_month_attached: Result /= Void
		end

	previous_month: ROSE_MONTH
			-- Month immediately before `Current'.
		do
			if julian_month > 0 then
				Result := from_julian_month (julian_month - 1)
			else
				create Result.make (year,month)
				Result.month_back
				Result.set_to_end_of_month
			end
		ensure
			previous_month_attached: Result /= Void
		end

	future_month (a_count: INTEGER): ROSE_MONTH
			-- Month `a_count' months later than `Current'
		do
			Result := from_julian_month (julian_month + a_count)
		ensure
			future_month_attached: Result /= Void
		end

	from_julian_month (a_julian: INTEGER): ROSE_MONTH
			-- Rose month corresponding to `a_julian'
		require
			legal_month: a_julian > 0 and a_julian < 2000
		do
			if attached {ROSE_MONTH} julian_months.item (a_julian) as j then
				Result := j
			else
				create Result.make (
					1950 + ((a_julian - 1) // 12),
					(a_julian-1) \\ 12 + 1)
				julian_months.put (Result,a_julian)
			end
		ensure
			from_julian_month_attached: Result /= Void
		end

	julian_month: INTEGER
			-- Number of months since December 1949

	last: ROSE_DATE
			-- Last day of `Current'
		do
			Result := month_end
		end

	first: ROSE_DATE
		-- First day of `Current'
		do
			create Result.make (year, month, 1)
		end

	months_to (a_month: like Current): ARRAY [ROSE_MONTH]
			-- Inclusive range of months from `Current' to `a_month'
		require
			a_month_not_void: a_month /= Void
			a_month_not_earlier: a_month >= Current
		local
			i, l_count: INTEGER
			l_month: ROSE_MONTH
		do
			l_count := diff_in_months (a_month).abs + 1
			create Result.make (1, l_count)
			from
				i := 1; l_month := Current
			until
				i > l_count
			loop
				Result.put (l_month, i)
				i := i + 1; l_month := l_month.next_month
			end
		ensure
			result_not_void: Result /= Void
			correct_count: Result.count = diff_in_months (a_month).abs + 1
		end

	months_from (a_month: like Current): ARRAY [ROSE_MONTH]
			-- Inclusive range of months from `a_month' to `Current'.
		require
			a_month_not_void: a_month /= Void
			a_month_not_later: a_month <= Current
		local
			i, l_count: INTEGER
			l_month: ROSE_MONTH
		do
			l_count := diff_in_months (a_month) + 1
			create Result.make (1, l_count)
			from
				i := 1; l_month := a_month
			until
				i > l_count
			loop
				Result.put (l_month, i)
				i := i + 1; l_month := l_month.next_month
			end
		ensure
			result_not_void: Result /= Void
			correct_count: Result.count = diff_in_months (a_month) + 1
		end

	months_from_descending (a_month: like Current): ARRAY [ROSE_MONTH]
			-- Inclusive range of months from `a_month' back to `Current'.
		require
			a_month_not_void: a_month /= Void
			a_month_not_later: a_month <= Current
		local
			i, l_count: INTEGER
			l_month: ROSE_MONTH
		do
			l_count := diff_in_months (a_month) + 1
			create Result.make (1, l_count)
			from
				i := 1; l_month := Current
			until
				i > l_count
			loop
				Result.put (l_month, i)
				i := i + 1; l_month := l_month.previous_month
			end
		ensure
			result_not_void: Result /= Void
			correct_count: Result.count = diff_in_months (a_month) + 1
		end

feature -- Measurement

	diff_in_months (a_other: like Current): INTEGER
			-- Difference from `a_other' in months
		require
			a_other_not_void: a_other /= Void
		do
			Result := julian_month - a_other.julian_month
		end

feature -- Duplication

	copy_month (a_month: ROSE_MONTH)
			-- Copy `a_month' into `Current'.
		require
			a_month_not_void: a_month /= Void
		do
			ordered_compact_date := a_month.ordered_compact_date
			persecs := a_month.persecs
			julian_month := a_month.julian_month
		ensure
			same_ordered_compact_date: ordered_compact_date = a_month.ordered_compact_date
			same_persecs: persecs = a_month.persecs
			same_julian_month: julian_month = a_month.julian_month
		end
	
feature {NONE} -- Implementation

	julian_months: ARRAY [detachable ROSE_MONTH]
			-- Cache for `from_julian_month'
		once
			create Result.make_filled (Void, 1, 2000)
		ensure
			julian_months_attached: Result /= Void
		end

end
