note

	description:

		"Date object representing (usually) the end of a quarter"

	author: "Mark Howard"
	last_editor: "$Author: c4 $"
	date: "$Date: 2014/01/08 $"
	revision: "$Revision: #2 $"
	path: "$File: //rose/source/library/datetime/src/rose_quarter.e $"

class	ROSE_QUARTER

inherit

	ROSE_MONTH
		rename
			make as rose_month_make,
			make_now as rose_month_make_now
		redefine
			first,
			out
		end

create

	make,
	make_ccyyq,
	make_current,
	make_previous,
	make_from_julian_quarter

create {ROSE_QUARTER}

	rose_date_make

feature {NONE} -- Initialization

	make_ccyyq (a_ccyyq: INTEGER)
			-- Create as year `a_ccyyq' // 10 and quarter `a_ccyyq' \\ 10.
		require
			sensible_ccyyq : a_ccyyq >= 19001 and then a_ccyyq <= 29994
		do
			make (a_ccyyq // 10, a_ccyyq \\ 10)
		ensure
			correct_year: year = a_ccyyq // 10
			correct_month: month = (a_ccyyq \\ 10) * 3
		end

	make_current
			-- Create as today's quarter.
		do
			make_today
			make (year, quarter)
		end

	make_previous
			-- Create as quarter prior to today's quarter.
		local
			l_prev: ROSE_DATE
		do
			make_today
			l_prev := previous_quarter_end
			make (l_prev.year, l_prev.quarter)
		end

	make (a_year: INTEGER; a_quarter: INTEGER)
			-- Create with `a_year' and month 3 * `a_quarter'.
		do
			rose_month_make (a_year, a_quarter * 3)
		ensure
			correct_year: year = a_year
			correct_month: month = (a_quarter) * 3
		end

feature -- Access

	start_date: ROSE_DATE
			-- First day of quarter represented by `Current' (if `Current' hasn't been mutated by, e.g., a call to `month_forth')
		do
			create Result.make (year, month - 2, 1)
		ensure
			start_date_attached: Result /= Void
			correct_year: Result.year = year
			correct_month: Result.month = month - 2
			first_of_month: Result.month = 1
		end

	first: ROSE_MONTH
			-- First month of `Current'
		do
			create Result.make (year, month - (month \\ 3) + 1)
		ensure then
			same_year: Result.year = year
			less_than_three_months: (Result.month - month) <= 2
		end

	last_month: ROSE_MONTH
			-- Last month of `Current'
		do
			Result := last.rose_month
		ensure
			last_month_attached: Result /= Void
		end

	next_quarter: ROSE_QUARTER
			-- Quarter after current
		do
			create Result.make (year, quarter)
			Result.add_n_days (1)
			create Result.make (Result.year, Result.quarter)
		end

	out: STRING
			 -- <Precursor>
		do
			Result := debug_output
		end

feature -- Status Report

	is_q1: BOOLEAN
			-- Is first calendar quarter?
		do
			Result := (month \\ 4) = 0
		ensure
			between_january_and_march: Result = (month >= January and month <= march)
		end

	is_q2: BOOLEAN
			-- Is second calendar quarter?
		do
			Result := (month \\ 4) = 1
		ensure
			between_april_and_june: Result = (month >= April and month <= June)
		end

	is_q3: BOOLEAN
			-- Is third calendar quarter?
		do
			Result := (month \\ 4) = 2
		ensure
			between_july_and_september: Result = (month >= July and month <= September)
		end

	is_q4: BOOLEAN
			-- Is fourth calendar quarter?
		do
			Result := (month \\ 4) = 3
		ensure
			between_october_and_december: Result = (month >= October and month <= December)
		end

end
