note

	description:

		"Represents time of day"

	author: "Pierre Nallet"
	last_editor: "$Author: c4 $"
	date: "$Date: 2014/01/08 $"
	revision: "$Revision: #2 $"
	path: "$File: //rose/source/library/datetime/src/rose_time.e $"

class	ROSE_TIME

inherit

	TIME
		redefine
			origin
		end

	HASHABLE
		undefine
			is_equal, out
		end

create

	make,
	make_fine,
	make_now,
	make_by_seconds,
	make_by_fine_seconds,
	make_from_string_default,
	make_from_integer

feature {NONE} -- Initialization

	make_from_integer (a_time: INTEGER)
			-- Create from `a_time' assuming a format of hhmmss.
			-- Bad arguments (e.g 258781) will result in setting to midnight.
		local
			l_hour: INTEGER
			l_minute: INTEGER
			l_second: INTEGER
		do
			l_hour := a_time // 10000
			l_minute := (a_time - l_hour * 10000) // 100
			l_second := a_time - l_hour * 10000 - l_minute * 100
			if is_correct_time (l_hour, l_minute, l_second, False) then
				make (l_hour, l_minute, l_second)
			else
				make (0, 0, 0)
			end
		end

feature -- Access

	hhmm: STRING
			-- Represented as HHMM
		local
			i: INTEGER
		do
			create Result.make (4)
			i := hour
			if i < 10 then
				Result.append_character ('0')
			end
			Result.append_integer (i)
			i := minute
			if i < 10 then
				Result.append_character ('0')
			end
			Result.append_integer (i)
		ensure
			hhmm_not_void: Result /= Void
		end

	hhmmss: STRING
			-- Represented as HHMMSS
		local
			i: INTEGER
		do
			create Result.make (6)
			i := hour
			if i < 10 then
				Result.append_character ('0')
			end
			Result.append_integer (i)
			i := minute
			if i < 10 then
				Result.append_character ('0')
			end
			Result.append_integer (i)
			i := second
			if i < 10 then
				Result.append_character ('0')
			end
			Result.append_integer (i)
		ensure
			hhmmss_not_void: Result /= Void
		end

	hhmmssfff: INTEGER
			-- `Current' as number of milliseconds since midnight
		do
			Result := (hour * 10000000) + (minute * 100000) + (second * 1000) + milli_second
		end

	hash_code: INTEGER
			-- <Precursor>
		do
			Result := hhmmssfff.hash_code
		end

	persecs: INTEGER
			-- 10000ths of seconds elapsed since midnight
		do
			Result := (fine_seconds * 10000).truncated_to_integer
		end

	padded_time_count: INTEGER
			-- Length of padded time as hh:mm:ss.fff
		once
			Result := ("hh.mm.ss.fff").count
		end

	formatted_time: STRING
			-- Formatted time as hh:mm:ss:fff with zero padded fractional seconds
			-- {TIME}.formatted_out does not always pad the fractional seconds
		do
			Result := formatted_out ("[0]hh:[0]mi:[0]ss.ff3")
			if Result.count < padded_time_count then
				Result.append (create {STRING}.make_filled ('0', padded_time_count - Result.count))
			end
		ensure
			formatted_time_attached: Result /= Void
			correct_length: Result.count = padded_time_count
		end

	formatted_time_short: STRING
			-- Formatted time as hh:mm:ss
		do
			Result := formatted_out ("[0]hh:[0]mi:[0]ss")
		ensure
			formatted_time_short_attached: Result /= Void
		end

	origin: ROSE_TIME
			-- Midnight
		once
			create Result.make (0, 0, 0)
		end

end
