note

	description:

		"Atomic partitioning criteria"

	author: "Zoran Simic"
	last_editor: "$Author: c4 $"
	date: "$Date: 2014/05/13 $"
	revision: "$Revision: #1 $"
	path: "$File: //rose/source/library/quotient/src/rose_quotient_atomic_criteria.e $"

class	ROSE_QUOTIENT_ATOMIC_CRITERIA [G]

inherit

	ROSE_QUOTIENT_CRITERIA [G]

create

	make

feature {NONE} -- Initialization

	make (a_index: INTEGER; a_name: STRING; a_getter: like getter)
			-- Create an atomic criteria associated to `a_index', `a_name' and `a_getter'.
		require
			valid_index: a_index > 0
			valid_name: a_name /= Void
			valid_getter: a_getter /= Void
		do
			index := a_index
			name := a_name.as_lower
			getter := a_getter
		ensure
			index_set: index = a_index
			name_set: name.is_equal (a_name.as_lower)
			getter_aliased: getter = a_getter
		end

feature -- Access

	index: INTEGER
			-- Internal index of this criteria

	name: STRING
			-- Name of this criteria

	getter: FUNCTION [ANY, TUPLE [G], detachable COMPARABLE]
			-- Associated getter, function that fetches the associated value from within an object of type G

	property (a_object: detachable G): detachable COMPARABLE
			-- First property for `a_object'
		do
			if a_object /= Void then
				Result := getter.item ([a_object])
			end
		end

	properties (a_object: detachable G): ARRAY [detachable COMPARABLE]
			-- All properties in this criteria for `a_object'
		do
			Result := << property (a_object) >>
		end

feature -- Comparison

	compared (a, b: G): INTEGER
			-- Compare `a' and `b', return -1 if `a' < `b', 0 if equal, 1 if `a' > `b'
		do
			Result := property_compared (property (a), property (b))
		end

	invariant_compared (a_object: detachable G; a_invariant: ROSE_QUOTIENT_INVARIANT [G]): INTEGER
			-- Compare `a_object' to `a_invariant' accordingly to this criteria
			-- Return -1 if `a_object' < Current, 0 if equal, 1 otherwise
		local
			l_property: detachable COMPARABLE
		do
			if attached a_invariant.properties as l_properties then
				l_property := l_properties.item (index)
			end
			Result := property_compared (property (a_object), l_property)
		end

	property_compared (a_property1, a_property2: detachable COMPARABLE): INTEGER
			-- Comparison of `a_property1' with `a_property2' using <
		do
			if equal (a_property1, a_property2) then
				Result := 0
			elseif a_property1 = Void then
				Result := -1
			elseif a_property2 = Void then
				Result := 1
			elseif a_property1 < a_property2 then
				Result := -1
			else
				Result := 1
			end
		end
	
feature -- Access
	
	names: ARRAY [STRING]
			-- Atomic criteria names composing this criteria
		do
			Result := <<name>>
		end

	has (a_criteria: ROSE_QUOTIENT_CRITERIA [G]): BOOLEAN
			-- Is `a_criteria' part of this criteria?
		do
			Result := a_criteria = Current
		end

	includes (a_other: ROSE_QUOTIENT_CRITERIA [G]): BOOLEAN
			-- Does this criteria include `a_other'?
		do
			Result := a_other = Current
		end

	is_empty: BOOLEAN = False
			-- Is this criteria empty?

	string_representation (a_object: detachable G; a_separator: STRING): STRING
			-- Representation of `a_object' for this criteria
			-- `a_separator' is used to separated multiple criterias
		do
			if attached property (a_object) as l_property then
				Result := l_property.out
			else
				Result := "?"
			end
		end

	out, debug_output: STRING
			-- New string containing terse printable representation of current object
		do
			Result := name
		end
	
invariant

	name_attached: name /= Void
	getter_attached: getter /= Void

end
