note

	description:

		"Quotient invariant criteria"

	author: "Zoran Simic"
	last_editor: "$Author: c4 $"
	date: "$Date: 2014/05/13 $"
	revision: "$Revision: #1 $"
	path: "$File: //rose/source/library/quotient/src/rose_quotient_invariant.e $"

class	ROSE_QUOTIENT_INVARIANT [G]

inherit
	
	DEBUG_OUTPUT
		redefine
			out
		end
	
	ROSE_OBJECT_REPRESENTATION
		export
			{NONE} all
		redefine
			out
		end	
		
create

	make

feature {NONE} -- Initialization

	make (a_quotient: ROSE_QUOTIENT [G]; a_criteria: like parent_criteria)
			-- Initialize from `a_quotient'.
		require
			valid_quotient: a_quotient /= Void
		do
			quotient := a_quotient
			parent_criteria := a_criteria
			wipe_out
		ensure
			quotient_aliased: quotient = a_quotient
			parent_criteria_aliased: parent_criteria = a_criteria
		end
	
feature -- Access
	
	quotient: ROSE_QUOTIENT [G]
			-- Associated quotient
	
	criteria: detachable ROSE_QUOTIENT_CRITERIA [G]
			-- Criteria attached to this invariant
			-- (properties in this 'criteria' are invariant by definition)
			-- Void if this `quotient' is root
	
	properties: detachable ARRAY [detachable COMPARABLE]
			-- Invariant properties so far
			-- Variant properties are set to Void
	
	criteria_properties (a_criteria: detachable ROSE_QUOTIENT_CRITERIA [G]): detachable ARRAY [detachable COMPARABLE]
			-- Properties corresponding to `a_criteria', or all properties if Void
		local
			i: INTEGER
		do
			if a_criteria = Void then
				Result := properties
			else
				if attached {ROSE_QUOTIENT_ATOMIC_CRITERIA [G]} a_criteria as l_atomic then
					create Result.make (1, 1)
					if attached properties as l_properties then
						Result.put (l_properties.item (l_atomic.index), 1)
					end
				elseif attached {ROSE_QUOTIENT_COMPOSED_CRITERIA [G]} a_criteria as l_composed then
					from
						create Result.make (l_composed.criteria.lower, l_composed.criteria.upper)
						i := l_composed.criteria.lower
					until
						i > l_composed.criteria.upper
					loop
						if attached properties as l_properties then
							Result.put (l_properties.item (l_composed.criteria.item (i).index), i)
						end
						i := i + 1
					end
				end
			end
		end
	
	compared (a_object: detachable G): INTEGER
			-- Compare `a_object' to this invariant
			-- Return -1 if `a_object' precedes this invariant, 0 if equal, 1 otherwise
		do
			if attached criteria as l_criteria then
				Result := l_criteria.invariant_compared (a_object, Current)
			end
		end
	
	out, debug_output: STRING
			-- New string containing terse printable representation of current object
		do
			if criteria /= Void then
				Result := criteria_representation (criteria)
			else
				Result := criteria_representation (quotient.definition.complete_criteria)
			end
		end
	
feature {ROSE_QUOTIENT} -- Implementation
	
	synchronize (a_object: detachable G)
			-- Synchronize this invariant to `a_object'.
		local
			i: INTEGER
			l_property, l_object_property: detachable COMPARABLE
		do
			if attached properties as l_properties then
				from
					i := l_properties.lower
				until
					i > l_properties.upper
				loop
					l_property := l_properties.item (i)
					if l_property /= Void then
						l_object_property := quotient.definition.all_criterias.item (i).property (a_object)
						if not equal (l_property, l_object_property) then
							l_properties.put (Void, i)
						end
					end
					i := i + 1
				end
			else
				properties := quotient.definition.complete_criteria.properties (a_object)
			end
		end
	
feature -- Removal
	
	wipe_out
			-- Clear this invariant.
		do
			properties := Void
			if attached parent_criteria as l_first then
					criteria := l_first
			else
				criteria := Void
			end
		end
	
feature {NONE} -- Implementation

	parent_criteria: detachable ROSE_QUOTIENT_CRITERIA [G]
			-- To avoid fake void-safety errors

	no_property: STRING = "?"
			-- No property defined representation
	
	criteria_representation (a_criteria: detachable ROSE_QUOTIENT_CRITERIA [G]): STRING
			-- Properties in this invariant represented for `a_criteria'
		local
			i: INTEGER
		do
			if attached {ROSE_QUOTIENT_ATOMIC_CRITERIA [G]} a_criteria as l_atomic then
				Result := represented_property (l_atomic.index)
			elseif attached {ROSE_QUOTIENT_COMPOSED_CRITERIA [G]} a_criteria as l_composed then
				from
					create Result.make (l_composed.criteria.count * 10)
					i := l_composed.criteria.lower
				until
					i > l_composed.criteria.upper
				loop
					Result.append (represented_property (l_composed.criteria.item (i).index))
					i := i + 1
					if i < l_composed.criteria.upper then
						Result.append (", ")
					end
				end
			else
				Result := "?"
			end
		end
	
	represented_property (a_index: INTEGER): STRING 
			-- Represented property at `a_index'
		local
			l_property: detachable COMPARABLE
		do
			if attached properties as l_properties then
				l_property := l_properties.item (a_index)
			end
			if l_property = Void then
				Result := no_property
			else
				Result := represented (l_property)
			end
		end

invariant

	attached_quotient: quotient /= Void

end
