note

	description:

		"Quotient scale (sequence of criteria)"

	author: "Zoran Simic"
	last_editor: "$Author: c4 $"
	date: "$Date: 2014/05/13 $"
	revision: "$Revision: #1 $"
	path: "$File: //rose/source/library/quotient/src/rose_quotient_scale.e $"

class	ROSE_QUOTIENT_SCALE [G]

inherit

	DEBUG_OUTPUT
		redefine
			out
		end

create

	make, make_atomic

feature {NONE} -- Initialization

	make (a_criteria: detachable ARRAY [ROSE_QUOTIENT_CRITERIA [G]])
			-- Create scale with `a_criteria' sequence.
		local
			i: INTEGER
		do
			first := Void
			following := Void
			if attached a_criteria as l_criteria then
				from
					i := l_criteria.lower
				until
					i > l_criteria.upper
				loop
					add (l_criteria.item (i))
					i := i + 1
				end
			end
		end

	make_atomic (a_criteria: detachable ROSE_QUOTIENT_CRITERIA [G])
			-- Create `Current' with a single critereon.
		do
			first := a_criteria
			following := Void
		ensure
			first_aliased: first = a_criteria
		end

feature -- Access

	first: detachable ROSE_QUOTIENT_CRITERIA [G]
			-- First critereon of this scale (structured a la Lisp...)

	following: detachable ROSE_QUOTIENT_SCALE [G]
			-- Following criteria

	depth: INTEGER
			-- Depth of this scale: number of criteria in its sequence
		do
			if is_empty then
				Result := 0
			elseif is_atomic then
				Result := 1
			else
				check attached following as l_following then
						-- postcondition "definition" of `is_atomic'				
					Result := 1 + l_following.depth
				end
			end
		end

feature -- Status report

	is_empty: BOOLEAN
			-- Is this scale empty (no criteria)?
		do
			Result := first = Void
		end

	is_atomic: BOOLEAN
			-- Is this scale atomic (no `following' scales)?
		do
			Result := following = Void
		ensure
			definition: Result = (following = Void)
		end

	has (a_criteria: ROSE_QUOTIENT_CRITERIA [G]): BOOLEAN
			-- Is `a_criteria' part of this scale?
		do
			Result := equal (first, a_criteria)
				or else (attached following as l_following and then l_following.has (a_criteria))
		end

feature -- Basic operations

	add_first (a_criteria: ROSE_QUOTIENT_CRITERIA [G])
			-- Add `a_criteria' as first critereon of this scale.
		require
			valid_criteria: a_criteria /= Void
		do
			if is_empty then
				--| Do nothing
			elseif is_atomic then
				create following.make_atomic (first)
			else
				check attached first as l_first and attached following as l_following then
						-- not `is_empty'
					l_following.add_first (l_first)
				end
			end
			first:= a_criteria
		end

	add (a_criteria: ROSE_QUOTIENT_CRITERIA [G])
			-- Add `a_criteria' to this scale (as last criteria).
		require
			valid_criteria: a_criteria /= Void
		do
			if is_empty then
				first := a_criteria
			elseif is_atomic then
				create following.make_atomic (a_criteria)
			else
				check attached following as l_following then
						-- postcondition "definition" of `is_atomic'
					l_following.add (a_criteria)
				end
			end
		end

feature -- Output

	out, debug_output: STRING
			-- New string containing terse printable representation of current object
		do
			create Result.make (100)
			if attached first as l_first then
				Result.append (l_first.out)
			end
			if attached following as l_following then
				Result.append ("; ")
				Result.append (l_following.out)
			end
		end

end
