note
	description: "Event type implementation"
	author: "Zoran Simic"
	last_editor: "$Author: c4 $"
	date: "$Date: 2014/05/13 $"
	revision: "$Revision: #1 $"
	path: "$File: //rose/source/library/event/src/rose_event_type.e $"

class
	ROSE_EVENT_TYPE [G -> TUPLE create default_create end]

inherit
	ANY
		redefine
			default_create
		end

create
	default_create

feature {NONE} -- Initialization

	default_create
			-- Default creation procedure
		do
			create subscribers.make (5)
			subscribers.compare_objects
		ensure then
			call_counter_initialized: call_counter = 0
			not_paused: not is_paused
		end

feature -- Access

	is_paused: BOOLEAN
			-- Is this event currently paused? (ie, calls to 'call' will not fire the event)

	call_counter: INTEGER
			-- Number of times 'call' was called so far

	subscriber_count: INTEGER
			-- Number of subscribers to this event
		do
			Result := subscribers.count
		ensure
			non_negative_subscriber_count: Result >= 0
		end

	i_th (a_index: INTEGER): PROCEDURE [ANY, G]
			-- Subscriber number `a_index'
		require
			index_ok: a_index >= 1 and then a_index <= subscriber_count
		do
			Result := subscribers.i_th (a_index)
		ensure
			i_th_attached: Result /= Void
		end

	has (a_subscriber: detachable PROCEDURE [ANY, G]): BOOLEAN
			-- Is `a_subscriber' subscribed to this event?
		local
			i: INTEGER
		do
			if a_subscriber /= Void then
				from
					i := 1
				until
					Result or else i > subscribers.count
				loop
					Result := subscribers.i_th (i) ~ a_subscriber
					i := i+1
				end
			end
		end

feature -- Basic operations

	reset
			-- Reset this event.
		do
			call_counter := 0
		ensure
			call_counter_reset: call_counter = 0
		end

	call (a_arguments: G)
			-- Call all 'subscribers' with 'a_arguments', if not currently paused.
		require
			a_arguments_not_void: a_arguments /= Void
		local
			i: INTEGER
		do
			if not is_paused then
				call_counter := call_counter + 1
				from
					i := 1
				until
					i > subscribers.count
				loop
					subscribers.i_th (i).call (a_arguments)
					i := i + 1
				end
			end
		ensure
			not_paused_increments_calls: not is_paused implies (call_counter = old call_counter + 1)
			paused_does_not_increment_calls: is_paused implies (call_counter = old call_counter)
		end

	extend (a_subscriber: PROCEDURE [ANY,G])
			-- Add `a_subscriber' to this event.
		require
			a_subscriber_attached: a_subscriber /= Void
		do
			if not has (a_subscriber) then
				subscribers.extend (a_subscriber)
			end
		ensure
			has_subscriber: has (a_subscriber)
		end

	remove (a_subscriber: PROCEDURE [ANY, G])
			-- Remove 1a_subscriber' from this event.
		require
			a_subscriber_attached: a_subscriber /= Void
		do
			subscribers.prune_all (a_subscriber)
		ensure
			not_has: not has (a_subscriber)
		end

	pause (a_pause: BOOLEAN)
			-- Pause triggering of this event
		do
			is_paused := a_pause
		ensure
			is_paused_set: is_paused = a_pause
		end

feature {NONE} -- Implementation

	subscribers: ARRAYED_LIST [attached PROCEDURE [ANY, G]]
			-- Subscribers to this event type

invariant

	subscribers_attached: subscribers /= Void
	subscribers_compare_objects: subscribers.object_comparison
	call_counter_non_negative: call_counter >= 0

end
