note

	description:

		"Ancestor of MATRIX without the implementation"

	author: "pn"
	last_editor: "$Author: myeager $"
	date: "$Date: 2015/03/19 $"
	revision: "$Revision: #20 $"
	path: "$File: //rose/source/kernel/infrastructure/math/abstract_matrix.e $"

deferred class ABSTRACT_MATRIX

inherit

	ABSTRACT_READABLE_MATRIX
		redefine
			item,
			set_column_names,
			set_row_names
		end

	ROSE_FORMATTING

feature {NONE} -- Initialization

	make (h, w: INTEGER) is
			-- Create with height `h' and width `w'.
		require
			h_non_negative: h >= 0
			w_non_negative: w >= 0
		deferred
		ensure
			height_set: height = h
			width_set: width = w
		end

	make_from_matrix(a_matrix:MATRIX) is
		-- make only with a matrix, trying to infer the
		-- dates from row names if any.
		require
			a_matrix_not_void: a_matrix /= Void
		local
			i,j:INTEGER
			l_other : SPECIAL[DOUBLE]
		do
			make(a_matrix.height,a_matrix.width)
			from i:=1 until i>height loop
				l_other := a_matrix.data[i]
				from j:=1 until j>width loop
					put(l_other[j],i,j)
					j:=j+1
				end
				i:=i+1
			end
			column_names := a_matrix.column_names
			row_names := a_matrix.row_names
			name := a_matrix.name
		end

feature -- Creation from files

	make_from_tab_delimited_text_file(a_file: ROSE_TEXT_FILE; a_vars: ROSE_LINEAR_ARRAY [STRING]) is
			-- The file can have an optional header lines, else columns won't have any names
		do
			make_from_delimited_text_file(a_file, a_vars ,"%T ,")
		end

	make_from_delimited_text_file(a_file: ROSE_TEXT_FILE; a_vars: ROSE_LINEAR_ARRAY [STRING]; a_delimiter : STRING) is
			-- The file can have an optional header lines, else columns won't have any names
		require
			valid_text_file: a_file /= Void
		local
			l_width, l_height: INTEGER
			l_lines: ROSE_LINEAR_ARRAY [STRING]
			l_line: STRING
			l_cols: ROSE_LINEAR_ARRAY [STRING]
			l_cols_index_table: ROSE_HASH_TABLE [INTEGER, STRING]
			i, j, k: INTEGER
		do
			-- Find the height and the width of the matrix
			if a_vars /= Void and then a_vars.count > 0 then
				l_width := a_vars.count
			end
			if a_file.is_readable then
				l_lines := a_file.non_empty_lines_array
				-- Check if first line is a header
				l_line := l_lines.item(1)
				create l_cols.make_from_linear(formatter.split(l_line, a_delimiter, "\").linear_representation)
				if l_cols.count > 0 then
					create l_cols_index_table.make(50)
					if a_vars = Void or else a_vars.count < 1 then
						l_width := l_cols.count
					end
					if not l_cols.item(1).is_double then
						-- Header line
						l_cols_index_table.put (1, l_cols.item(1))
						l_cols.forth
						from i := 2 until i > l_cols.count loop
							l_cols_index_table.put(i, l_cols.item(i))
							i := i + 1
						end
						i := 2 -- Reset iterator for use in going over the lines
						l_height := l_lines.count - 1
					else
						i := 1
						l_height := l_lines.count
					end

				end

				make(l_height, l_width)
				j := 1
				from until i > l_lines.count loop
					l_line := l_lines.item(i)
					create l_cols.make_from_linear(formatter.split(l_line, a_delimiter, "\").linear_representation)
					if a_vars /= Void and then a_vars.count > 0 then
						set_column_names(a_vars)
						from k := 1 until k > a_vars.count loop
							l_cols_index_table.search(a_vars.item(k))
							if l_cols_index_table.found and then l_cols.item(l_cols_index_table.found_item).is_double then
								put(l_cols.item(l_cols_index_table.found_item).to_double, j, k)
							else
								put(undefined_double, j, k)
							end
							k := k + 1
						end
					else
						from k := 1 until k > l_cols.count loop
							if l_cols.item(k).is_double then
								put(l_cols.item(k).to_double, j, k)
							else
								put(undefined_double, j, k)
							end
							k := k + 1
						end
					end
					j := j + 1
					i := i + 1
					l_lines.forth
				end
			end
		end

feature -- Access

	item alias "[]" (i, j: INTEGER): DOUBLE assign put is deferred end

	identifiers: ROSE_ARRAY2 [STRING]

	set_identifiers (a_identifiers: like identifiers)
			-- Set 'identifiers'.
		do
			identifiers := a_identifiers
		end
	duplicated : like Current
			-- Matrix with data duplicated
		do
			Result := deep_twin
		end
	put (a_value : DOUBLE;i,j : INTEGER) is
			--set the ith row, jth column to a_value
		require
			valid_i: i >= 1 and i <= height
			valid_j: j >= 1 and j <= width
		deferred
		end
	make_as_inverse(a_other : ABSTRACT_READABLE_MATRIX) is
			--create as inverse of a_other
		require
			square : width = height
		local
			m: MATRIX
			i: INTEGER
			lu: LU_DECOMPOSITION
		do
			create m.make (a_other.width, a_other.width)
			from i := 1 until i > a_other.width loop
			--	m.put (1, i, i)
				m[i, i] := 1
			i := i + 1
			end
			create lu.make (a_other)
			duplicate(lu.solve (m))
		ensure
			is_inverse : (a_other * Current).is_identity
		end
	block (a_upper, a_left, a_row_count, a_column_count: INTEGER): ABSTRACT_MATRIX_BLOCK is
		-- defines a matrix block starting with row of a_upper and column of a_left
		-- the block has a_row_count rows and a_column_count columns.
		do
			create Result.make_block (Current, a_upper, a_left, a_row_count, a_column_count)
		end

	duplicate (a_matrix : ABSTRACT_READABLE_MATRIX)
			--Duplicate
		local
			i,j : INTEGER
		do
			make(a_matrix.height,a_matrix.width)
			from i := 1 until i > height loop
				from j := 1 until j > width loop
					Current[i,j] := a_matrix[i,j]
					j := j + 1
				end
				i := i + 1
			end
		end

	scale (a_scale : DOUBLE) is
			-- Scale all items in the matrix by `a_scale'
		local
			i,j : INTEGER
		do
			from i := 1 until i > height loop
				from j := 1 until j > width loop
					Current[i, j] := Current [i, j] * a_scale
					j := j + 1
				end
				i := i + 1
			end
		end
	scalar_multiply,scaled (a_value : DOUBLE) : like Current is
			-- returns a matrix like current, but with every element multiplied by `a_value'
		local
			i,j : INTEGER
		do
			Result := duplicated
			from i := 1 until i > height loop
				from j := 1 until j > width loop
					Result[i, j] := Current[i, j] * a_value
					j := j + 1
				end
				i := i + 1
			end
			if column_names /= Void then
				Result.set_column_names(column_names)
			end
			if row_names /= Void then
				Result.set_row_names(row_names)
			end
		end
	add (a_other: ABSTRACT_READABLE_MATRIX) is
		-- Perfom addition in-place. (Compare to feature infix "+".)
		require
			dimensions_match : width = a_other.width and height = a_other.height
		local
			i,j : INTEGER
		do
			from i := 1 until i > height loop
				from j := 1 until j > width loop
				--	put (item (i,j) + a_other.item (i,j), i,j)
					Current [i, j] := Current [i, j] + a_other[i, j]
					j := j + 1
				end
				i := i + 1
			end
		end
	subtract (a_other: ABSTRACT_READABLE_MATRIX) is
		-- Perfom subtraction in-place. (Compare to feature infix "-".)
		require
			dimensions_match : width = a_other.width and height = a_other.height
		local
			i,j : INTEGER
		do
			from i := 1 until i > height loop
				from j := 1 until j > width loop
--					put (item (i,j) - a_other.item (i,j), i,j)
					Current [i, j] := Current[i, j] - a_other[i, j]
					j := j + 1
				end
				i := i + 1
			end
		end

	subtract_column_vector(a_vec : ABSTRACT_READABLE_VECTOR)
			--Subtract a_vec from each column
		require
			dimensions_match : height = a_vec.count
		local
			i, j : INTEGER
		do
			from i := 1 until i > height loop
				from j := 1 until j > width loop
					Current [i, j] := Current [i, j] - a_vec[i]
					j := j + 1
				end
				i := i + 1
			end
		end

	subtract_row_vector, demean(a_col_means : ABSTRACT_READABLE_VECTOR) is
		require
			dimensions_match : width = a_col_means.count
		local
			i, j : INTEGER
		do
			from i := 1 until i > height loop
				from j := 1 until j > width loop
				--	put (item (i, j) - a_col_means.item(j), i, j)
					Current [i, j] := Current [i, j] - a_col_means[j]
					j := j + 1
				end
				i := i + 1
			end
		end

	statistics_of_the_columns (a_weights: MATRIX): MATRIX is
			-- moments statistics of column data series
			-- do not use column_moments feature to save memory
		require
			dimension_matched: a_weights /= Void implies (a_weights.height = height and then a_weights.width = width)
		local
			l_row_vector: VECTOR
			l_names: ROSE_LINEAR_ARRAY [STRING]
			l_moments: ROSE_MOMENTS
			i: INTEGER
			l_weights: MATRIX
		do
			l_weights := a_weights
			if l_weights = Void then
				create l_weights.make_unit (height, width)
			end
			from
				i := 1 until i > width
			loop
				create l_moments.make (column (i).to_array, l_weights.column (i).to_array)
				if Result = Void then
					l_names := l_moments.access.column_names
					create Result.make (width, l_names.count)
					if column_names /= Void then
						Result.set_row_names (column_names.twin)
					end
					Result.set_column_names (l_names)
				end
				l_row_vector := l_moments.to_vector
				Result.put_row_vector (l_row_vector, i)
				i := i + 1
			end
		end

	statistics_of_the_rows (a_weights: MATRIX): MATRIX is
			-- moments statistics of row data series
			-- do not use row_moments feature to save memory
		require
			dimension_matched: a_weights /= Void implies (a_weights.height = height and then a_weights.width = width)
		local
			l_row_vector: VECTOR
			l_names: ROSE_LINEAR_ARRAY [STRING]
			l_moments: ROSE_MOMENTS
			i: INTEGER
			l_weights: MATRIX
		do
			l_weights := a_weights
			if l_weights = Void then
				create l_weights.make_unit (height, width)
			end
			from
				i := 1 until i > height
			loop
				create l_moments.make (row (i).to_array, l_weights.row (i).to_array)
				if Result = Void then
					l_names := l_moments.access.column_names
					create Result.make (height, l_names.count)
					if row_names /= Void then
						Result.set_row_names (row_names.twin)
					end
					Result.set_column_names (l_names)
				end
				l_row_vector := l_moments.to_vector
				Result.put_row_vector (l_row_vector, i)
				i := i + 1
			end
		end

	replace_column_null(a_c: INTEGER) is
			-- Replace undefined doubles and null values with zero in column with index `a_c'.
		local
			i: INTEGER
			l_v: DOUBLE
		do
			if valid_column_index(a_c) then
				from i := 1 until i > height loop
					l_v := Current[i, a_c]
					if not is_defined_double(l_v) then
						Current[i, a_c] := 0.0
					end
					i := i + 1
				end
			end
		end

	replace_row_null(a_r: INTEGER) is
			-- Replace undefined doubles and null values with zero in row with index `a_r'.
		local
			j: INTEGER
			l_v: DOUBLE
		do
			if valid_row_index(a_r) then
				from j := 1 until j > width loop
					l_v := Current[a_r, j]
					if not is_defined_double(l_v) then
						Current[a_r, j] := 0.0
					end
					j := j + 1
				end
			end
		end

feature -- Modification

	item_add (a_value: DOUBLE; i,j: INTEGER)
		require else
			valid_indices: 1 <= i and i <= height and 1 <= j and j <= width
		do
			Current[i, j] := Current[i, j] + a_value
		end

	row_add (a_row: ABSTRACT_READABLE_VECTOR; a_row_ix: INTEGER)
		require
			dimensions_match: width = a_row.count
		local
			i: INTEGER
		do
			from i := 1 until i > width loop
				Current[a_row_ix, i] := Current[a_row_ix, i] + a_row[i]
				i := i + 1
			end
		end

	column_add (a_column: ABSTRACT_READABLE_VECTOR; a_column_ix: INTEGER)
		require
			dimensions_match: height = a_column.count
		local
			i: INTEGER
		do
			from i := 1 until i > height loop
				Current[i, a_column_ix] := Current[i, a_column_ix] + a_column[i]
				i := i + 1
			end
		end

	set_row_names (a_names: ARRAY[STRING])
			-- <Precursor>
		do
			Precursor (a_names)
			row_table_saved := Void
		end

	set_column_names (a_names: ARRAY[STRING])
			-- <Precursor>
		do
			Precursor (a_names)
			column_table_saved := Void
		end

feature  --Update
	put_row_array, update_row(a_array : attached ARRAY[DOUBLE]; a_row_ix : INTEGER) is
		local
			l_vector : VECTOR
		do
			if  valid_row_index(a_row_ix) then
				create l_vector.make_from_array(a_array)
				put_row_vector(l_vector, a_row_ix)
			end
		end


	put_column_array(a_array : attached ARRAY[DOUBLE]; a_col_ix : INTEGER) is
		local
			l_vector : VECTOR
		do
			if  valid_column_index(a_col_ix) then
				create l_vector.make_from_array(a_array)
				put_column_vector(l_vector, a_col_ix)
			end
		end

	put_row_vector(a_vector : attached ABSTRACT_READABLE_VECTOR; a_row_ix : INTEGER) is
		local
			i : INTEGER
			l_double : DOUBLE
		do
			if  valid_row_index(a_row_ix) then
				from i := 1
				until i > a_vector.count or i > width
				loop
					l_double := a_vector.item(i)
				--	put(l_double, a_row_ix, i)
					Current [a_row_ix, i] := l_double
					i := i + 1
				end
			end
		end

	put_column_vector, update_column(a_vector: attached ABSTRACT_READABLE_VECTOR; a_col_ix : INTEGER) is
		local
			i : INTEGER
			l_double : DOUBLE
		do
			if valid_column_index(a_col_ix) then
				if a_vector.count > height then
					print ("WARNING: "+(a_vector.count-height).out +" items in vector will be trancated.%N")
				end
				from i := 1
				until i > a_vector.count or i > height
				loop
					l_double := a_vector.item(i)
				--	put(l_double, i, a_col_ix)
					Current [i, a_col_ix] := l_double
					i := i + 1
				end
			end
		end

	put_column_sub_vector(a_vector:ABSTRACT_READABLE_VECTOR; a_col_ix, a_beg_ix : INTEGER) is
		local
			i : INTEGER
			l_double : DOUBLE
		do
			if a_vector /= Void and then valid_column_index(a_col_ix) and then (a_beg_ix > 0 and a_beg_ix + height -1 <= a_vector.count) then
				from i := 1
				until i > height
				loop
					l_double := a_vector.item(a_beg_ix + i - 1)
				--	put(l_double, i, a_col_ix)
					Current [i, a_col_ix] := l_double
					i := i + 1
				end
			end
		end

	put_matrix(a_matrix : ABSTRACT_READABLE_MATRIX; a_row_ix,a_col_ix:INTEGER) is
			local
				i,j:INTEGER
			do
				from i:=1 until i>a_matrix.height or (i+a_row_ix-1)>height loop
					from j:=1 until j>a_matrix.width or (j+a_col_ix-1)>width loop
					--	put(a_matrix.item(i,j),i+a_row_ix-1,j+a_col_ix-1)
						Current [i+a_row_ix-1,j+a_col_ix-1] := a_matrix[i, j]
						j:=j+1
					end
					i:=i+1
				end
			end

feature  --Winsorize
	winsorize_row(a_row_ix : INTEGER; a_weights : ARRAY[DOUBLE];  a_lower_winsor_z, a_upper_winsor_z: DOUBLE) is
		require
			valid_weights_size : a_weights = Void or else a_weights.count = width
		local
			l_moments : ROSE_MOMENTS
		do
			if valid_row_index (a_row_ix) and then {l_row : ARRAY[DOUBLE]} row_array(a_row_ix) then
				create l_moments.make_winsorized_pass_limited(l_row, a_weights, a_lower_winsor_z, a_upper_winsor_z, 10, 0, 0)
				put_row_array (l_moments.data, a_row_ix)
			end
		end

	winsorize_column(a_column_ix : INTEGER; a_weights : ARRAY[DOUBLE];  a_lower_winsor_z, a_upper_winsor_z: DOUBLE) is
		require
			valid_weights_size : a_weights = Void or else a_weights.count = height
		local
			l_moments : ROSE_MOMENTS
		do
			if valid_column_index (a_column_ix) and then {l_column : ARRAY[DOUBLE]} column_array(a_column_ix) then
				create l_moments.make_winsorized_pass_limited(l_column, a_weights, a_lower_winsor_z, a_upper_winsor_z, 10, 0, 0)
				put_column_array (l_moments.data, a_column_ix)
			end
		end

	winsorize_rowise(a_weights : ARRAY[DOUBLE];  a_lower_winsor_z, a_upper_winsor_z: DOUBLE) is
		require
			valid_weights_size : a_weights = Void or else a_weights.count = width
		local
			i : INTEGER
		do
			from i := 1 until i > height loop
				winsorize_row (i, a_weights, a_lower_winsor_z, a_upper_winsor_z)
				i := i + 1
			end
		end

	winsorize_columnwise(a_weights : ARRAY[DOUBLE];  a_lower_winsor_z, a_upper_winsor_z: DOUBLE) is
		require
			valid_weights_size : a_weights = Void or else a_weights.count = height
		local
			i : INTEGER
		do
			from i := 1 until i > width loop
				winsorize_column (i, a_weights, a_lower_winsor_z, a_upper_winsor_z)
				i := i + 1
			end
		end
	winsorized_normalized(a_weights : ARRAY[DOUBLE];  a_lower_winsor_z, a_upper_winsor_z: DOUBLE) : MATRIX
			--Result of winsorizing each column and normalizing the result wrt to mean and standard deviation of the result
		require
			valid_weights_size : a_weights = Void or else a_weights.count = height
		local
			i,j : INTEGER
			l_defined : ROSE_LINEAR_ARRAY[INTEGER]
			l_column,l_weights : ROSE_LINEAR_ARRAY[DOUBLE]
			l_moments : ROSE_MOMENTS
		do
			create Result.make (height, width)
			from j := 1 until j > width loop
				create l_defined.make_empty
				create l_column.make_empty
				if a_weights /= Void then
					create l_weights.make_empty
				end
				from i := 1 until i > height loop
					if is_defined_double (item(i,j)) then
						l_defined.extend (i)
						l_column.extend (item(i,j))
						if a_weights /= Void then
							l_weights.extend (a_weights[i])
						end
					else
						Result[i,j] := undefined_double
					end
					i := i + 1
				end
				create l_moments.make_winsorized(l_column, l_weights, a_lower_winsor_z, a_upper_winsor_z)
				from i := 1 until i > l_moments.data.count loop
					Result[l_defined[i],j] := (l_moments.data[i] - l_moments.mean) / l_moments.standard_deviation
					i := i + 1
				end
				j := j + 1
			end
		end

feature -- Access

	column_table: HASH_TABLE [INTEGER, STRING]
			-- Hash table of column names
		require
			column_names_exist: column_names /= Void
		local
			l_name: STRING
		do
			if column_table_saved = Void then
				create Result.make (column_names.count)
				from
					column_names.start
				until
					column_names.off
				loop
					if column_names.current_item /= Void then
						l_name := column_names.current_item.twin
					else
						l_name := Void
					end
					Result.put (column_names.index, l_name)
					column_names.forth
				end
				column_table_saved := Result
			else
				Result := column_table_saved
			end
		end

	row_table: HASH_TABLE [INTEGER, STRING]
			-- Hash table associating row names with their indices
		require
			row_names_exists: row_names /= Void
		local
			l_name:STRING
		do
			if row_table_saved = Void then
				create Result.make (row_names.count)
				from
					row_names.start
				until
					row_names.off
				loop
					if row_names.current_item /= Void then
						l_name := row_names.current_item.twin
					else
						l_name := Void
					end
					Result.put (row_names.index, l_name)
					row_names.forth
				end
				row_table_saved := Result
			else
				Result := row_table_saved
			end
		end

feature {NONE} -- Implementation

	row_table_saved:like row_table
		-- HASH_TABLE version of `row_names' for faster lookup by name.

	column_table_saved: like column_table
		-- HASH_TABLE version of `column_names' for faster lookup by name.

end
