note

	description:

		"Get/put a block (sub_matrix) of data from/to a mother matrix."

	author: "xc, mk"
	last_editor: "$Author: mk $"
	date: "$Date: 2012/08/20 $"
	revision: "$Revision: #9 $"
	path: "$File: //rose/source/kernel/infrastructure/math/abstract_matrix_block.e $"

class
	ABSTRACT_MATRIX_BLOCK

inherit
	ABSTRACT_MATRIX
		redefine
			make_as_inverse
		end
create
	make_block, test

feature -- creation
	make (h, w: INTEGER) is
			--Creation
		do
		end
	make_as_inverse(a_matrix : ABSTRACT_READABLE_MATRIX) is
		do
			make_block(a_matrix.inverse,1,1,a_matrix.width,a_matrix.height)
		end

	make_block (a_matrix: ABSTRACT_MATRIX; a_upper, a_left, a_block_row_count, a_block_column_count: INTEGER) is
		-- a_upper of 2 and a_left 1 mean matrix element (2,1) in a_matrix is (1,1) of the matrix block
		-- a_blockr_row_count of 3 and a_block_column_count of 2 mean the block has 3 rows and 2 columns
		require
			valid_start_row: a_upper >= 1 and then a_upper <= a_matrix.height
			valid_stard_column: a_left >= 1 and then a_left <= a_matrix.width
		do
			matrix := a_matrix
			virtual_top := a_upper
			virtual_left := a_left
			width := a_block_column_count
			height := a_block_row_count
		ensure
			valid_row: virtual_left + width <= matrix.width
			valid_column : virtual_top + height <= matrix.height
		end

	test is
		local
			l_matrix: MATRIX
		do
			create l_matrix.make(3,3)
		--	l_matrix.put(11,1,1); l_matrix.put(12,1,2); l_matrix.put(13, 1,3)
		--	l_matrix.put(21,2,1); l_matrix.put(22, 2,2); l_matrix.put(23, 2,3)
		--	l_matrix.put(31,3,1); l_matrix.put(32,3,2); l_matrix.put(33, 3,3)
			make_block (l_matrix, 2, 1, 2,2)
	--		print ("Original value at block(1,2) = "+ item(1,2).out+"%N")
	--		print ("Original value at mother matrix (2,2) = "+ l_matrix.item(2,2).out+", as expected is block (1,2)%N")
	--		print ("%NNow change block (1,2) to 2004.%N%N")
	--		put (2004.0, 1,2)
	--		print ("New value at block(1,2) = "+ item(1,2).out+"%N")
	--		print ("New value at mother matrix (2,2) = "+ l_matrix.item(2,2).out+", as expected is block (1,2)%N")

		end


feature -- access
	item alias "[]" (i, j: INTEGER): DOUBLE is
		require else
			valid_i: i >= 1 and then i <= height
			valid_j: j >= 1 and then j <= width
		do
			Result := matrix [i+virtual_top-1, j+virtual_left-1]
		end

	put (a_value: DOUBLE; i,j: INTEGER) is
		require else
			valid_i: i >= 1 and then i <= height
			valid_j: j >= 1 and then j <= width
		do
		--	matrix.put (a_value, i+virtual_top-1, j+virtual_left-1)
			matrix [i+virtual_top-1, j+virtual_left-1] := a_value
		end

	put_row_names (a_names: ARRAY [STRING]) is
		require else
			valid_size: a_names.count >= 1 and then a_names.count+virtual_top-1 <= matrix.row_count
		local
			i: INTEGER
		do
			if a_names /= Void then
				if matrix.row_names = Void then
					matrix.set_row_names (default_strings(matrix.row_count))
				end
				from i := 1 until i > a_names.count
				loop
					matrix.row_names.put (a_names.item (i), virtual_top+i-1)
					i := i + 1
				end
			end
		end
	put_column_names (a_names: ARRAY [STRING]) is
		require else
			valid_size: a_names.count >= 1 and then a_names.count+virtual_left-1 <= matrix.column_count
		local
			i: INTEGER
		do
			if a_names /= Void then
				if matrix.column_names = Void then
					matrix.set_column_names (default_strings(matrix.column_count))
				end
				from i := 1 until i > a_names.count
				loop
					matrix.column_names.put (a_names.item (i), virtual_left+i-1)
					i := i + 1
				end
			end
		end
	default_strings (a_size: INTEGER) : ROSE_LINEAR_ARRAY [STRING] is
		do
			create Result.make (1, a_size)
		end
feature -- attributes
	matrix: ABSTRACT_MATRIX

	virtual_left: INTEGER -- virtual_left: column index of block in (mother) matrix
	virtual_top: INTEGER  -- virtual_top: row index of block in (mother) matrix
	height: INTEGER       -- row count of matrix block
	width: INTEGER        -- column count of matrix block

end -- class ABSTRACT_MATRIX_BLOCK
