indexing
	description: "Encapsulation of the ADS_SEARCHPREF_INFO AD structure"
	author: "Pierre Nallet"
	date: "$Date: 2009/01/05 $"
	revision: "$Revision: #1 $"
	archive: "$File: //rose/source/Kernel/infrastructure/windows/activedirectory/ad_ads_searchpref_info.e $"

class
	AD_ADS_SEARCHPREF_INFO

inherit
	COM_STRUCTURE

create
	make

feature {NONE} -- Initialization

	make is
			-- Initializes the current instance
		do
			item := calloc (structure_size, 1)
		end

feature -- Basic operations

	set_search_pref (a_id: INTEGER_32) is
			-- Set the type of preference
		do
			set_integer_value (a_id, offsetofdwsearchpref)
		end

	set_page_size (a_value: INTEGER_32) is
			-- Set the page size. If the page size is not set, the maximum number of items (rows) is 1000
		local
			l_ptr: POINTER
			l_value: AD_ADSVALUE
		do
			set_search_pref (ads_searchpref_pagesize)
			l_ptr := item + offsetofvvalue
			create l_value
			l_value.set (l_ptr)
			l_value.set_type (adstype_integer)
			l_value.set_integer (a_value)
		end

	dispose is
			-- Frees the underlying memory - Could be called by the garbage collector in future versions
		do
			free (item)
			item := default_pointer
		end

feature {NONE} -- Implementation

	structure_size: INTEGER is
			-- The size of the structure pointed by 'item'
		do
			Result := sizeof_ads_searchpref_info
		end

	ads_searchpref_pagesize: INTEGER_32 is
			-- Encapsulation of AD constant
		external
			"C blocking macro use <Iads.h>"
		alias
			"ADS_SEARCHPREF_PAGESIZE"
		end

	adstype_integer: INTEGER_32 is
			-- Encapsulation of AD constant
		external
			"C blocking macro use <Iads.h>"
		alias
			"ADSTYPE_INTEGER"
		end

	offsetofdwsearchpref: INTEGER is
			-- Offset od the dwSearchPref field in the ADS_SEARCHPREF_INFO structure
		external
			"C blocking macro use <Iads.h>"
		alias
			"offsetof(ADS_SEARCHPREF_INFO, dwSearchPref)"
		end

	offsetofvvalue: INTEGER_32 is
			-- Offset od the vValue field in the ADS_SEARCHPREF_INFO structure
		external
			"C blocking macro use <Iads.h>"
		alias
			"offsetof (ADS_SEARCHPREF_INFO, vValue)"
		end

	sizeof_ads_searchpref_info: INTEGER_32 is
			-- Size (in bytes of AD structure
		external
			"C blocking macro use <Iads.h>"
		alias
			"sizeof (ADS_SEARCHPREF_INFO)"
		end

end
