indexing
	description: "Represents an Active Directory entry"
	author: "Pierre Nallet"
	last_editor: "$Author: sm $"
	date: "$Date: 2009/11/02 $"
	revision: "$Revision: #3 $"
	archive: "$File: //rose/source/Kernel/infrastructure/windows/activedirectory/ad_directory_entry.e $"

class
	AD_DIRECTORY_ENTRY

inherit
	COM_SMART_UNKNOWN

create
	make_root,
	make_with_path

feature {NONE} -- Initialization

	make_root is
			-- Create as the root entry
		do
			bind
		end

	make_with_path (a_path: STRING) is
				-- Create with a path
		do
			path := a_path
			bind
		end

feature -- Access

	path: STRING
			-- Active Directory path to this entry


	parent: AD_DIRECTORY_ENTRY is
			-- Parent entry, if any
		local
			l_path: STRING
		do
			l_path := parent_path
			if (l_path /= Void) then
				create Result.make_with_path(l_path)
			end
		end
	parent_path: STRING
			-- Parent path, if any
		local
			l_bstr: POINTER
			hr: INTEGER
		do
			hr := iads_get_parent (p, $l_bstr)
			if (l_bstr /= default_pointer) then
				Result :=widechar_to_string (l_bstr)

			end
		end

	property (a_name: STRING): ANY is
			-- Value of property with 'a_name'
		require
			a_name_not_void: a_name /= Void
		local
			l_array: ARRAY [ANY]
		do
			l_array ?= get_ex (a_name)
			if l_array /= Void and then l_array.count > 0 then
				Result := l_array.item (l_array.lower)
			end
		end

	properties (a_name: STRING): ARRAY [ANY] is
			-- All values of property with 'a_name'
		require
			a_name_not_void: a_name /= Void
		local
			l_array: ARRAY [ANY]
		do
			l_array ?= get_ex (a_name)
			Result := l_array
		end

feature {NONE} -- Implementation

	get_ex (a_name: STRING): ANY is
			-- Get attribute value with 'a_name'
		local
			l_bstr: POINTER
			l_variant: COM_VARIANT
			hr: INTEGER
		do
			l_bstr := string_to_bstr (a_name)
			create l_variant
			l_variant.init
			hr := iads_getex (p, l_bstr, l_variant.item)
			debug ("AD")
				if hr < 0 then
					print ("ActiveDirectory: Unable to get " + a_name + " for " + path)
				end
			 end
			Result := l_variant.to_any
			l_variant.release
			sysfreestring (l_bstr)
		end

	bind is
			-- Bind 'path' to Active Directory COM object.
		local
			l_entry: AD_DIRECTORY_ENTRY
			l_text: STRING
			l_text_p: POINTER
			l_ptr: POINTER
			hr: INTEGER
		do
			co_initialize
			if p = default_pointer then
				l_text := path
				if l_text = Void then
					create l_entry.make_with_path ("LDAP://RootDSE")
					l_text := "LDAP://" + l_entry.property ("defaultNamingContext").out
				end
				l_text_p := string_to_widechar (l_text)
				hr := adsopenobject (l_text_p, default_pointer, default_pointer, 0, iid_iunknown, $l_ptr)
				p := l_ptr
				free (l_text_p)
			end
		end

	clone_browsable: AD_DIRECTORY_ENTRY is
		do
			Result := Current
		ensure
			result_not_void: Result /= Void
		end

	co_initialize is
		local
			l_runtime: COM_RUN_TIME
			i: INTEGER
		once
			create l_runtime
			i := l_runtime.coinitialize (default_pointer)
		end

	adsopenobject (lpszPathName: POINTER; lpszUserName: POINTER; lpszPassword: POINTER; dwReserved: INTEGER; riid: POINTER; ppObject: POINTER): INTEGER is
			-- The ADsOpenObject function binds to an ADSI object using explicit username and password credentials
		external
			"C [dllwin32 %"Activeds.dll%"] (LPWSTR ,  LPWSTR ,  LPWSTR ,  DWORD ,  REFIID ,  VOID**): EIF_INTEGER"
		alias
			"ADsOpenObject"
		end

	iads_getex (this, bstrname: POINTER; pvprop: POINTER): INTEGER is
			-- This method can be used to retrieve a range of attribute values
		require
			exists: this /= default_pointer
		external
			"C blocking macro signature (IADs*, BSTR, VARIANT *): EIF_INTEGER use <Iads.h>"
		alias
			"IADs_GetEx"
		end

	iads_get_parent (this, retval: POINTER): INTEGER is
			-- Get the parent entry
		require
			exists: this /= default_pointer
		external
			"C blocking macro signature (IADs*, BSTR *): EIF_INTEGER use <Iads.h>"
		alias
			"IADs_get_Parent"
		end

end
