indexing
	description: "Objects that search active directory."
	author: "pn"
	date: "$Date: 2009/01/05 $"
	revision: "$Revision: #1 $"
	archive: "$File: //rose/source/Kernel/infrastructure/windows/activedirectory/ad_directory_searcher.e $"

class
	AD_DIRECTORY_SEARCHER

inherit
	COM_SMART_UNKNOWN

create
	make

feature -- Creation

	make (a_search_root: AD_DIRECTORY_ENTRY; a_filter: STRING_8) is
			-- Initialize the current searcher so that it will search entries below a_search_root that match a_filter
		do
			search_root := a_search_root
			filter := a_filter
		end

feature -- Access

	filter: STRING_8
			-- The text for the current filter

	search_root: AD_DIRECTORY_ENTRY
			-- The root entry for the search

	set_preferences (a_preferences: AD_ADS_SEARCHPREF_INFO) is
			-- Set the preferences of the current search. The C API allows for an array of preferences. However, since we use only one preference, a pointer to one instance is OK
		local
			hr: INTEGER_32
		do
			hr := idirectorysearch_setsearchpreference (Current.p, a_preferences.item, 1)
		end

	find_all_with_name (a_column_name: STRING_8): LINEAR [ANY] is
			-- Get all items with the a given name
		local
			l_list: ARRAYED_LIST [ANY]
			hr: INTEGER_32
			l_ptr: POINTER
			l_filter_ptr: POINTER
			l_handle: POINTER
			l_column: AD_SEARCH_COLUMN
			l_stop: BOOLEAN
			l_preferences: AD_ADS_SEARCHPREF_INFO
		do
			create l_list.make (0)
			hr := search_root.query_interface (iid_idirectorysearch, $l_ptr)
			p := l_ptr
			l_filter_ptr := string_to_widechar (filter)
			create l_preferences.make
			l_preferences.set_page_size (2000)
			set_preferences (l_preferences)
			hr := idirectorysearch_executesearch (l_ptr, l_filter_ptr, default_pointer, -1, $l_handle)
			free (l_filter_ptr)
			hr := idirectorysearch_getfirstrow (p, l_handle)
			l_stop := hr /= 0
			from
			until
				l_stop
			loop
				l_column := get_column (l_handle, a_column_name)
				l_list.extend (l_column.value)
				hr := idirectorysearch_freecolumn (p, l_column.item)
				l_column.free (l_column.item)
				hr := idirectorysearch_getnextrow (p, l_handle)
				if (hr /= 0) then
					if (hr = s_ads_nomore_rows) then
						l_stop := True
					else
						l_stop := True
					end
				end
			end
			hr := idirectorysearch_closesearchhandle (p, l_handle)
			Result := l_list
			l_preferences.dispose
		end

feature {NONE} -- Implementation

	get_column_names (a_handle: POINTER): ARRAYED_LIST [STRING_8] is
			-- Get all the column names for the items for this search
		local
			str: POINTER
			l_name: STRING_8
			hr: INTEGER_32
		do
			create Result.make (0)
			from
				hr := idirectorysearch_getnextcolumnname (p, a_handle, $str)
			until
				hr /= 0
			loop
				l_name := widechar_to_string (str)
				print (l_name)
				print (" ")
				Result.extend (l_name)
				print (get_column (a_handle, l_name).type)
				hr := idirectorysearch_getnextcolumnname (p, a_handle, $str)
				print ("%N")
			end
		end

	get_column (a_handle: POINTER; a_name: STRING_8): AD_SEARCH_COLUMN is
			-- Get the column object for a given name
		local
			hr: INTEGER_32
			l_name: POINTER
		do
			create Result.make
			l_name := string_to_widechar (a_name)
			hr := idirectorysearch_getcolumn (p, a_handle, l_name, Result.item)
			free (l_name)
		end

	print_list (a: ARRAYED_LIST [STRING_8]) is
			-- debug stuff, to remove
		do
			from
				a.start
			until
				a.after
			loop
				print (a.item)
				print (", ")
				a.forth
			end
		end

	iid_idirectorysearchcache: COM_GUID [RUN_TIME_MEMORY_MODEL] is
			-- GUID of the COM interface
		once
			create Result.make
			Result.set_from_string ("{109BA8EC-92F0-11D0-A790-00C04FD8D5A8}")
		end

	iid_idirectorysearch: POINTER is
			-- pointer to the GUID for the COM interface
		do
			Result := iid_idirectorysearchcache.item
		end

	idirectorysearch_executesearch (this: POINTER; pszsearchfilter: POINTER; pattributenames: POINTER; dwnumberattributes: INTEGER_32; phsearchresult: POINTER): INTEGER_32 is
			-- Encapsulation of the ExecuteSearch method
		external
			"C macro signature (IDirectorySearch *, LPWSTR, LPWSTR, DWORD,  PADS_SEARCH_HANDLE): EIF_INTEGER use <Iads.h>"
		alias
			"IDirectorySearch_ExecuteSearch"
		end

	idirectorysearch_getfirstrow (this: POINTER; hsearchresult: POINTER): INTEGER_32 is
			-- Encapsulation of the GetFirstRow method
		external
			"C macro signature (IDirectorySearch *, ADS_SEARCH_HANDLE): EIF_INTEGER use <Iads.h>"
		alias
			"IDirectorySearch_GetFirstRow"
		end

	idirectorysearch_getnextrow (this: POINTER; hsearchresult: POINTER): INTEGER_32 is
			-- Encapsulation of the GetNextRow method
		external
			"C macro signature (IDirectorySearch *, ADS_SEARCH_HANDLE): EIF_INTEGER use <Iads.h>"
		alias
			"IDirectorySearch_GetNextRow"
		end

	s_ads_nomore_rows: INTEGER_32 is
			-- Encapsulation of the corresponding constant
		external
			"C macro  use <AdsErr.h>"
		alias
			"S_ADS_NOMORE_ROWS"
		end

	idirectorysearch_getnextcolumnname (this, hsearchhandle, ppszcolumnname: POINTER): INTEGER_32 is
			-- Encapsulation of the GetNextColumnName method
		external
			"C macro signature (IDirectorySearch *, ADS_SEARCH_HANDLE, LPWSTR *): EIF_INTEGER use <Iads.h>"
		alias
			"IDirectorySearch_GetNextColumnName"
		end

	idirectorysearch_getcolumn (this, hsearchhandle, szcolumnname, psearchcolumn: POINTER): INTEGER_32 is
			-- Encapsulation of the GetColumn method
		external
			"C macro signature (IDirectorySearch *, ADS_SEARCH_HANDLE, LPWSTR, PADS_SEARCH_COLUMN): EIF_INTEGER use <Iads.h>"
		alias
			"IDirectorySearch_GetColumn"
		end

	idirectorysearch_setsearchpreference (this, psearchprefs: POINTER; dwnumprefs: INTEGER_32): INTEGER_32 is
			-- Encapsulation of the SetSearchPreference method
		external
			"C macro signature (IDirectorySearch *, PADS_SEARCHPREF_INFO, DWORD): EIF_INTEGER use <Iads.h>"
		alias
			"IDirectorySearch_SetSearchPreference"
		end

	idirectorysearch_closesearchhandle (this, hsearchhandle: POINTER): INTEGER_32 is
			-- Encapsulation of the CloseSearchHandle method
		external
			"C macro signature (IDirectorySearch *, ADS_SEARCH_HANDLE): EIF_INTEGER use <Iads.h>"
		alias
			"IDirectorySearch_CloseSearchHandle"
		end

	idirectorysearch_freecolumn (this, psearchcolumn: POINTER): INTEGER_32 is
			-- Encapsulation of the FreeColumn method
		external
			"C macro signature (IDirectorySearch *, PADS_SEARCH_COLUMN): EIF_INTEGER use <Iads.h>"
		alias
			"IDirectorySearch_FreeColumn"
		end

end
