indexing
	description: "Represents root Active Directory domain"
	author: "Pierre Nallet"
	date: "$Date: 2009/01/05 $"
	revision: "$Revision: #1 $"
	archive: "$File: //rose/source/Kernel/infrastructure/windows/activedirectory/ad_domain.e $"

class
	AD_DOMAIN

create
	make

feature {NONE} -- Initialization

	make is
			-- Create as root.
		do
			create root.make_root
		end

feature -- Access

	groups: ARRAYED_LIST [AD_GROUP] is
			-- Groups in current domain
		local
			l_searcher: AD_DIRECTORY_SEARCHER
			l_found: LINEAR [ANY]
			l_string: STRING
		do
			create l_searcher.make (root, "(objectCategory=group)")
			l_found := l_searcher.find_all_with_name ("adsPath")
			from
				create Result.make (0)
				l_found.start
			until
				l_found.after
			loop
				l_string ?= l_found.item
				Result.extend (create {AD_GROUP}.make (l_string))
				l_found.forth
			end
		ensure
			Result /= Void
		end

	group (a_name: STRING): AD_GROUP is
			-- Return group with name matching `a_name'
			-- or `Void' if none.
		require
			a_name_not_void: a_name /= Void
		local
			l_searcher: AD_DIRECTORY_SEARCHER
			l_found: LINEAR [ANY]
			l_string: STRING
		do
			create l_searcher.make (root, "(&(objectCategory=group)(CN=" + a_name + "))")
			l_found := l_searcher.find_all_with_name ("adsPath")
			if not l_found.is_empty then
				l_found.start
				l_string ?= l_found.item
				create Result.make (l_string)
			end
		end

	users: ROSE_LINEAR_ARRAY [AD_USER] is
			-- Return all users in current domain.
		local
			l_searcher: AD_DIRECTORY_SEARCHER
			l_found: LINEAR [ANY]
			l_string: STRING
			l_user: AD_USER
		do
			create l_searcher.make (root, "(objectCategory=user)")
			l_found := l_searcher.find_all_with_name ("adsPath")
			create Result.make_empty
			from
				l_found.start
			until
				l_found.off
			loop
				l_string ?= l_found.item
				create l_user.make_with_path (l_string)
				Result.extend (l_user)
				l_found.forth
			end
		ensure
			result_not_void: Result /= Void
		end

	user (a_name: STRING): AD_USER is
			-- Return user with name matching `a_name'
			-- or `Void' if none.
		require
			a_name_not_void: a_name /= Void
		do
			Result := get_user_by_field ("cn", a_name)
		end

	user_by_login (a_id: STRING): AD_USER is
			-- Return user with name matching `a_id' (example: "ZS")
			-- or `Void' if none.
		require
			a_id /= Void
		do
			Result := get_user_by_field ("samAccountName", a_id)
		end

feature {NONE} -- Implementation

	root: AD_DIRECTORY_ENTRY
			-- Root active directory entry

	get_user_by_field (a_key, a_value: STRING): AD_USER is
			-- Return user with property with 'a_key' matching `a_value'
			-- or `Void' if none.
		require
			a_key /= Void
			a_value /= Void
		local
			l_searcher: AD_DIRECTORY_SEARCHER
			l_found: LINEAR [ANY]
			l_string: STRING
		do
			create l_searcher.make (root, "(&(objectCategory=user)(" + a_key+ "=" + a_value + "))")
			l_found := l_searcher.find_all_with_name ("adsPath")
			if not l_found.is_empty then
				l_found.start
				l_string ?= l_found.item
				create Result.make_with_path (l_string)
			end
		end

end
