indexing
	description: "Represents an Active Directory group"
	author: "Pierre Nallet"
	date: "$Date: 2009/04/02 $"
	revision: "$Revision: #2 $"
	archive: "$File: //rose/source/Kernel/infrastructure/windows/activedirectory/ad_group.e $"

class
	AD_GROUP

inherit
	DEBUG_OUTPUT

create {AD_DOMAIN}
	make

create
	make_with_entry

feature {NONE} -- Initialization

	make_with_entry (a_entry: AD_DIRECTORY_ENTRY) is
			-- Create `Current' with active directory entry `a_entry'.
		require
			a_entry_not_void: a_entry /= Void
		do
			entry_saved := a_entry
		ensure
			entry_set: entry = a_entry
		end

	make (a_path: STRING) is
			-- Create `Current' with path `a_path'.
		require
			a_path_not_void: a_path /= Void
			-- valid path: valid_path
		do
			path := a_path
		ensure
			entry_set: entry.path = a_path
		end

feature -- Access

	path: STRING
			-- Full path to this Active Directory group

	name: STRING is
			-- Group name
		do
			if name_saved = Void then
				name_saved := entry.property ("name").out
			end
			Result := name_saved
		end

	users: ARRAYED_LIST [AD_USER] is
			-- Members of this group
		local
			l_properties: ARRAY [ANY]
			l_entry: AD_DIRECTORY_ENTRY
			l_class_array: ARRAY [ANY]
			l_user: AD_USER
			i: INTEGER
		do
			l_properties := entry.properties ("member")
			create Result.make (0)
			if l_properties /= Void then
				from
					i := l_properties.lower
				until
					i > l_properties.upper
				loop
					create l_entry.make_with_path ("LDAP://" + l_properties.item (i).out)
					l_class_array := l_entry.properties ("objectClass")
					l_class_array.compare_objects
					if l_class_array.has ("user") then
						create l_user.make_with_entry (l_entry)
						Result.extend (l_user)
					end
					i := i + 1
				end
			end
		ensure
			result_not_void: Result /= Void
		end

	member_of_groups: ARRAYED_LIST [AD_GROUP] is
			-- Groups to which this group is a member.
		local
			l_results: ARRAY [ANY]
			l_entry: AD_DIRECTORY_ENTRY
			i: INTEGER
			l_class_array: ARRAY [ANY]
			l_group: AD_GROUP
		do
			l_results := entry.properties ("memberOf")
			create Result.make (0)
			if l_results /= Void then
				from
					i := l_results.lower
				until
					i > l_results.upper
				loop
					create l_entry.make_with_path ("LDAP://" + l_results.item (i).out)
					if l_entry.p /= default_pointer then
						l_class_array := l_entry.properties ("objectClass")
						l_class_array.compare_objects
						if l_class_array.has ("group") then
							create l_group.make_with_entry (l_entry)
							Result.extend (l_group)
						end
					end
					i := i + 1
				end
			end
		ensure
			result_not_void: Result /= Void
		end

	member_groups: ARRAYED_LIST [AD_GROUP] is
			-- Members of this group that themselves are groups.
		local
			l_results: ARRAY [ANY]
			l_entry: AD_DIRECTORY_ENTRY
			i: INTEGER
			l_class_array: ARRAY [ANY]
			l_group: AD_GROUP
		do
			l_results := entry.properties ("member")
			create Result.make (0)
			if l_results /= Void then
				from
					i := l_results.lower
				until
					i > l_results.upper
				loop
					create l_entry.make_with_path ("LDAP://" + l_results.item (i).out)
					if l_entry.p /= default_pointer then
						l_class_array := l_entry.properties ("objectClass")
						l_class_array.compare_objects
						if l_class_array.has ("group") then
							create l_group.make_with_entry (l_entry)
							Result.extend (l_group)
						end
					end
					i := i + 1
				end
			end
		ensure
			result_not_void: Result /= Void
		end

	entry: AD_DIRECTORY_ENTRY is
			-- Entry corresponding to this group
		do
			if entry_saved = Void then
				create entry_saved.make_with_path (path)
			end
			Result := entry_saved
		ensure
			result_not_void: result /= Void
		end

feature {NONE} -- Implementation

	name_saved: STRING
		-- Saved implementation for `name'.

	entry_saved: AD_DIRECTORY_ENTRY
		-- Saved implementation for `entry'.

	debug_output: STRING is
			-- String that should be displayed in debugger to represent `Current'.
		do
			Result := name
		end

invariant
	entry_not_void: entry /= Void

end
