indexing
	description: "Represents an Active Directory user"
	author: "Pierre Nallet"
	last_editor: "$Author: sm $"
	date: "$Date: 2009/11/02 $"
	revision: "$Revision: #6 $"
	path: "$File: //rose/source/Kernel/infrastructure/windows/activedirectory/ad_user.e $"

class AD_USER

inherit
	AD_ANY

	DEBUG_OUTPUT

create

	make_with_path, make_with_entry

feature -- Access

	organizational_unit: AD_ORGANIZATIONAL_UNIT is
			-- 	Organizational unit
		local
			l_entry: AD_DIRECTORY_ENTRY
		do
			l_entry := entry.parent
			if (l_entry /= Void) then
				create result.make_with_entry (l_entry)
			end
		end

	account_name: STRING is
			-- AD Property "sAMAccountName", holds login id just as "jq".
		local
			l_account_name_saved: STRING
			l_property: ANY
		do
			if account_name_saved = Void then
				l_property := entry.property ("sAMAccountName")
				if l_property /= Void then
					l_account_name_saved:= entry.property ("sAMAccountName").out
				end
				if l_account_name_saved /= Void then
					account_name_saved := l_account_name_saved
				else
					account_name_saved := ""
				end
			end
			Result := account_name_saved
		end

	office: STRING is
			-- AD Property "physicalDeliveryOfficeName"
		local
			l_office_saved: STRING
			l_property: ANY
		do
			if office_saved = Void then
				l_property := entry.property ("physicalDeliveryOfficeName")
				if l_property /= Void then
					l_office_saved:= entry.property ("physicalDeliveryOfficeName").out
				end
				if l_office_saved /= Void then
					office_saved := l_office_saved
				else
					office_saved := ""
				end
			end
			Result := office_saved
		end

	department: STRING is
			-- AD Property "department"
		local
			l_department_saved: STRING
			l_property: ANY
		do
			if department_saved = Void then
				l_property := entry.property ("department")
				if l_property /= Void then
					l_department_saved:= entry.property ("department").out
				end
				if l_department_saved /= Void then
					department_saved := l_department_saved
				else
					department_saved := ""
				end
			end
			Result := department_saved
		end

	title: STRING is
			-- AD Property "title"
		local
			l_title_saved: STRING
			l_property: ANY
		do
			if title_saved = Void then
				l_property := entry.property ("title")
				if l_property /= Void then
					l_title_saved:= entry.property ("title").out
				end
				if l_title_saved /= Void then
					title_saved := l_title_saved
				else
					title_saved := ""
				end
			end
			Result := title_saved
		end

	groups: ARRAYED_LIST [AD_GROUP] is
			-- Groups to which this user belongs
		local
			l_results: ARRAY [ANY]
			l_entry: AD_DIRECTORY_ENTRY
			i: INTEGER
			l_class_array: ARRAY [ANY]
			l_group: AD_GROUP
		do
			l_results := entry.properties ("memberOf")
			create Result.make (0)
			if l_results /= Void then
				from
					i := l_results.lower
				until
					i > l_results.upper
				loop
					create l_entry.make_with_path ("LDAP://" + l_results.item (i).out)
					if l_entry.p /= default_pointer then
						l_class_array := l_entry.properties ("objectClass")
						l_class_array.compare_objects
						if l_class_array.has ("group") then
							create l_group.make_with_entry (l_entry)
							Result.extend (l_group)
						end
					end
					i := i + 1
				end
			end
		ensure
			groups_not_void: Result /= Void
		end

feature {NONE} -- Implementation

	account_name_saved: STRING
			-- Saved version of `account_name'.

	office_saved: STRING
			-- Saved version of `office'.

	department_saved: STRING
			-- Saved version of `department'.

	title_saved: STRING
			-- Saved version of `title'.

	debug_output: STRING is
			-- String that should be displayed in debugger to represent `Current'.
		do
			Result := name
		end

invariant

	entry_not_void: entry /= Void

end
