indexing

	description: "DOUBLE equality with tolerance for rounding errors"

	author: "Franck Arnaud"
	last_editor: "$Author: yc $"
	date: "$Date: 2013/06/11 $"
	revision: "$Revision: #8 $"
	path: "$File: //rose/source/kernel/infrastructure/math/approximate_equality.e $"

class APPROXIMATE_EQUALITY

inherit

	ANY

	ROSE_SNAPSHOT_AWARE
		export
			{NONE} all
			{ANY} equal_double_with_tolerance, equal_double
		end

	DOUBLE_MATH
		export {NONE} all end

	ROSE_CONSTANTS
		export {NONE} all end

feature -- Basic operation

	approx_equal (a, b: DOUBLE): BOOLEAN is
			-- Equality with default order of magnitude tolerance.
		do
			Result := approx_equal_with_magnitude (a, b, Approx_equal_default_magnitude)
		ensure
			one_zero_implies_false: ((a = 0.0 and b /= 0.0) or (b = 0.0 and a /= 0.0)) implies Result = False
			any_undefined_impies_false: (not is_defined_double (a) or not is_defined_double (b)) implies Result = False
		end

	one_zero_or_approx_equal (a, b: DOUBLE): BOOLEAN is
			-- Optimistic approximate equality that is true when one parameter is zero,
			-- otherwise is the same as the regular `approx_equal'.
		do
			if a = 0.0 xor b = 0.0 then
				Result := True
			else
				Result := approx_equal (a, b)
			end
		ensure
			definition: Result = ((a = 0.0 xor b = 0.0) or approx_equal (a, b))
		end


	approx_equal_with_magnitude (a, b: DOUBLE; a_magnitude: INTEGER): BOOLEAN is
			-- Equality with order of magnitude (base 10) tolerance.
		do
			if is_defined_double (a) and is_defined_double (b) then
				Result := approx_equal_with_magnitude_or_undefined (a, b, a_magnitude)
			end
		ensure
			any_undefined_implies_false: (not is_defined_double (a) or not is_defined_double (b)) implies Result = False
		end

	approx_equal_with_magnitude_or_undefined (a, b: DOUBLE; a_magnitude: INTEGER): BOOLEAN is
			-- Equality with order of magnitude (base 10) tolerance.
		local
			l_tolerance: DOUBLE
		do
			Result := a = b
			if not Result then
				l_tolerance := 10 ^ (log10 (a.abs.max (b.abs)) - a_magnitude - 1)
				check positive_tolerance: l_tolerance > 0 end
				Result := equal_double_with_tolerance (a, b, l_tolerance)
			end
		ensure
			one_zero_implies_false: ((a = 0.0 and b /= 0.0) or (b = 0.0 and a /= 0.0)) implies Result = False
		end

	Approx_equal_default_magnitude: INTEGER is 6
			-- Default used for 'approx_equal'.

end
