note

	description:

	"[
		Split an array of items G by keys K derived from G
	]"

	author: "Ivan Tarnavsky"
	last_editor: "$Author: it $"
	date: "$Date: 2012/10/23 $"
	revision: "$Revision: #1 $"
	path: "$File: //rose/source/kernel/capital_actions/array_splitter_by_embedded_key.e $"

class ARRAY_SPLITTER_BY_EMBEDDED_KEY [G, K -> HASHABLE]

create

	make

feature {NONE} -- Initialization

	make (a_key_extractor: like key_extractor) is
			-- Initialize `Current'
		require
			a_key_extractor: a_key_extractor /= Void
		do
			key_extractor := a_key_extractor
		ensure
			key_extractor_set: key_extractor = a_key_extractor
		end

feature -- Access

	void_item: detachable G
			-- Void item template

	key_extractor: FUNCTION [ANY, TUPLE [G], K]
			-- Function to extract key K from item G

	split (a_array: ARRAY [G]): DS_HASH_TABLE [ARRAY [G], K] is
			-- Split items in `a_array' by using `key_extractor'
		require
			a_array_not_void: a_array /= Void
		local
			l_lists: DS_HASH_TABLE [DS_LINKED_LIST [G], K]
		do
			create l_lists.make_default
			a_array.do_all (agent split_to_lists (l_lists, ?))
			create Result.make (l_lists.count)
			l_lists.do_all_with_key (agent convert_lists_to_arrays (Result, ?, ?))
		ensure
			not_void: Result /= Void
			no_void_key: not Result.has_void
			no_void_item: not Result.has_void_item
			no_void_items_in_arrays: not Result.there_exists (agent {ARRAY [G]}.has (void_item))
		end

feature {NONE} -- Implementation

	split_to_lists (a_lists: DS_HASH_TABLE [DS_LINKED_LIST [G], K]; a_item: G) is
			-- Add `a_item' to `a_lists'
		require
			a_lists_not_void: a_lists /= Void
			no_void_key: not a_lists.has_void
			no_void_items: not a_lists.has_void_item
			no_void_item_in_lists: not a_lists.there_exists (agent {DS_LINKED_LIST [G]}.has_void)
		local
			l_key: K
			l_list: DS_LINKED_LIST [G]
		do
			if a_item /= Void then
				l_key := key_extractor.item ([a_item])
				if l_key /= Void then
					a_lists.search (l_key)
					if a_lists.found then
						l_list := a_lists.found_item
					else
						create l_list.make_default
						a_lists.force_new (l_list, l_key)
					end
					l_list.force_last (a_item)
				end
			end
		ensure
			a_lists_not_void: a_lists /= Void
			no_void_key: not a_lists.has_void
			no_void_items: not a_lists.has_void_item
			no_void_item_in_lists: not a_lists.there_exists (agent {DS_LINKED_LIST [G]}.has_void)
		end

	convert_lists_to_arrays (a_hash: like split; a_list: DS_LINKED_LIST [G]; a_key: K) is
			-- Convert `a_list' to array then add `a_hash' with `a_key'
		require
			a_hash_not_void: a_hash /= Void
			a_list_not_void: a_list /= Void
			a_key_not_void: a_key /= Void
			no_a_key_in_a_hash: not a_hash.has (a_key)
		do
			a_hash.force_new (a_list.to_array, a_key)
		ensure
			added: a_hash.has (a_key)
		end

invariant

	key_extractor_not_void: key_extractor /= Void

end
