indexing
	description: "Class that contains externals for atomic operations"
	author: "Jann Roeder"
	date: "$Date: 2008/12/01 $"
	revision: "$Revision: #3 $"

deferred class
	ATOMIC_BASE

feature {NONE} -- Initialization

	make_integer_32_value is
			-- Create a INTEGER_32 variable pointed to by `value'
		do
			create value.make (sizeof_long)
		end

feature {NONE} -- Implementation

	value: MANAGED_POINTER

	sizeof_long: INTEGER_32 is
			-- Size of a LONG variable
		external
			"C macro use <Windows.h>"
		alias
			"sizeof(LONG)"
		end

feature {NONE} -- Microsoft Interlocked externals

	get_external (a_ptr: POINTER): INTEGER_32 is
			-- Get the value pointed to by `a_ptr'
		external
			"C inline use <Windows.h>"
		alias
			"[
				LONG volatile* ptr = (LONG volatile*)$a_ptr;
				return (EIF_INTEGER_32)(*ptr);
			]"
		end

	set_external (a_ptr: POINTER; a_value: INTEGER_32) is
			-- Set the value at `a_ptr' to `a_value'
		external
			"C inline use <Windows.h>"
		alias
			"[
				LONG volatile* ptr = (LONG volatile*)$a_ptr;
				InterlockedExchange( ptr, (LONG)$a_value );
			]"
		end

	interlocked_exchange_add_external (a_ptr: POINTER; a_value: INTEGER_32): INTEGER_32 is
			-- Alias for InterlockedExchangeAdd
		external
			"C inline use <Windows.h>"
		alias
			"[
				LONG volatile* ptr = (LONG volatile*)$a_ptr;
				return (EIF_INTEGER_32)InterlockedExchangeAdd( ptr, (LONG)$a_value );
			]"
		end

	interlocked_exchange_external (a_ptr: POINTER; a_value: INTEGER_32): INTEGER_32 is
			-- Alias for InterlockedExchange
		external
			"C inline use <Windows.h>"
		alias
			"[
				LONG volatile* ptr = (LONG volatile*)$a_ptr;
				return (EIF_INTEGER_32)InterlockedExchange( ptr, (LONG)$a_value );
			]"
		end

	interlocked_compare_exchange_external (a_ptr: POINTER; a_exchange, a_comparand: INTEGER_32): INTEGER_32 is
			-- Alias for InterlockedCompareExchange
		external
			"C inline use <Windows.h>"
		alias
			"[
				LONG volatile* ptr = (LONG volatile*)$a_ptr;
				return (EIF_INTEGER_32)InterlockedCompareExchange( ptr, (LONG)$a_exchange, (LONG)$a_comparand );
			]"
		end

	interlocked_decrement_external (a_ptr: POINTER): INTEGER_32 is
			-- Alias for InterlockedDecrement
		external
			"C inline use <Windows.h>"
		alias
			"[
				LONG volatile* ptr = (LONG volatile*)$a_ptr;
				return (EIF_INTEGER_32)InterlockedDecrement( ptr );
			]"
		end

	interlocked_increment_external (a_ptr: POINTER): INTEGER_32 is
			-- Alias for InterlockedIncrement
		external
			"C inline use <Windows.h>"
		alias
			"[
				LONG volatile* ptr = (LONG volatile*)$a_ptr;
				return (EIF_INTEGER_32)InterlockedIncrement( ptr );
			]"
		end

invariant
	value_not_void: value /= Void

end
