indexing
	description: "Windows implementation of ATOMIC_BOOLEAN"
	author: "Jann Roeder"
	date: "$Date: 2008/12/01 $"
	revision: "$Revision: #2 $"

class
	ATOMIC_BOOLEAN

inherit
	ATOMIC_BASE

create
	make, make_with_value

feature {NONE} -- Initialization

	make is
			-- Creates a new ATOMIC_BOOLEAN variable with initial value `False'
		do
			make_with_value (False)
		end

	make_with_value (a_initial_value: BOOLEAN) is
			-- Creates a new ATOMIC_BOOLEAN variable with initial value `initial_value'
		do
			make_integer_32_value
			set_item (a_initial_value)
		end

feature -- Basic Operations

	set_item (a_value: BOOLEAN) is
			-- Atomically sets `Current' to `a_value
		do
			set_external (value.item, a_value.to_integer)
		end

	get_and_set (a_value: BOOLEAN): BOOLEAN is
			-- Atomically sets the given value.
			-- Returns the previous value
		do
			Result := interlocked_exchange_external (value.item, a_value.to_integer).to_boolean
		end

	compare_and_set (a_expect: BOOLEAN; a_update: BOOLEAN): BOOLEAN is
			-- Atomically sets the value to the given updated value if the current value = the expected value.
			-- Returns True if successful, False if not
		do
			Result := interlocked_compare_exchange_external (value.item, a_update.to_integer, a_expect.to_integer).to_boolean = a_expect
		end

feature -- Access

	item: BOOLEAN is
			-- Return value of `Current'
		do
			Result := get_external (value.item).to_boolean
		end
		
end

