indexing

   description: "Bloomberg Currency Access"
   status: "Development"
   author: "Jun Zhu"
   date: "$Date: 2014/02/26 $"
   revision: "$Revision: #7 $"

class
	BLOOMBERG_CURRENCY_ACCESS

inherit
	DATABASE_SERVERS

feature -- Access

	get_forward ( a_iso: STRING; a_base_date, a_future_date: ROSE_DATE ): BLOOMBERG_FORWARD_RECORD is
	-- get forward
	local
		l_fxx_period, l_lower_fxx_period, l_higher_fxx_period: INTEGER
		l_forward, l_lower_forward, l_higher_forward, l_weight_lower, l_weight_higher, l_quote_factor: DOUBLE
		l_roll_back_days: INTEGER
		l_data_date: ROSE_DATE
		l_data_date_time: ROSE_DATE
		l_currency_data: ROSE_LINEAR_ARRAY[ TEMPLATE_CURRENCY_DATA_SIGNATURES ]
		l_flag: BOOLEAN
	do
		create Result.make
		Result.set_currency_iso( a_iso )
		Result.set_base_date( a_base_date )
		Result.set_future_date( a_future_date )
		--- check bounds
		if a_base_date.ccyymmdd < 20031015 or a_future_date < a_base_date then
			Result.set_forward_rate( -999 )
		else
			l_fxx_period := a_future_date.diff_in_days( a_base_date )
			Result.set_forward_period( l_fxx_period )
			--
			l_data_date := clone( a_base_date )
			if l_data_date.is_weekend then
				l_data_date.add_n_days(-1)
			end
			if l_data_date.is_weekend then
				l_data_date.add_n_days(-1)
			end
			--
			l_flag := FALSE
			from l_roll_back_days := 5
			until l_flag = TRUE or l_roll_back_days < 0
			loop
				create l_data_date_time.make_by_date( l_data_date )
				l_currency_data := get_currency_data( l_data_date_time, a_iso )
				if l_currency_data /= void and l_currency_data.count > 10 then
					l_flag := TRUE
					Result.set_data_date( l_data_date )
				else
					l_data_date.add_n_days(-1)
					l_roll_back_days := l_roll_back_days - 1
				end
			end

			if l_flag = FALSE then  -- try spot only
				l_data_date := clone( a_base_date )
				if l_data_date.is_weekend then
					l_data_date.add_n_days(-1)
				end
				if l_data_date.is_weekend then
					l_data_date.add_n_days(-1)
				end

				from l_roll_back_days := 5
				until l_flag = TRUE or l_roll_back_days < 0
				loop
					create l_data_date_time.make_by_date( l_data_date )
					l_currency_data := get_currency_data( l_data_date_time, a_iso )
					if l_currency_data /= void and l_currency_data.count > 0 then
						l_flag := TRUE
						Result.set_data_date( l_data_date )
					else
						l_data_date.add_n_days(-1)
						l_roll_back_days := l_roll_back_days - 1
					end
				end
			end

			if l_flag then --- find lower and upper period and forward calcualte
				l_currency_data.sort_by( agent compare_forward_period( 0, ?, ?, TRUE ))
				l_lower_fxx_period := 0
				l_higher_fxx_period := 0
				l_lower_forward := 0.0
				l_higher_forward := 0.0
				l_quote_factor := 0.0
				l_flag := FALSE
				from l_currency_data.start
				until l_currency_data.off or l_flag = TRUE
				loop
					l_lower_fxx_period := l_higher_fxx_period
					l_lower_forward := l_higher_forward
					l_higher_fxx_period := l_currency_data.current_item.forward_period
					l_higher_forward := l_currency_data.current_item.px_last
					l_quote_factor := l_currency_data.current_item.quote_factor
					if l_fxx_period >= l_lower_fxx_period and l_fxx_period <= l_higher_fxx_period then
						l_flag := TRUE  --- stop here
						l_weight_lower := (l_fxx_period-l_lower_fxx_period)/(l_higher_fxx_period-l_lower_fxx_period)
						l_weight_higher := (l_higher_fxx_period-l_fxx_period)/(l_higher_fxx_period-l_lower_fxx_period)
						l_forward := l_weight_lower*l_lower_forward + l_weight_higher*l_higher_forward
					end
					l_currency_data.forth
				end
				if l_fxx_period = 0 and l_lower_fxx_period = 0 and l_higher_fxx_period = 0 then  -- spot
					l_lower_forward := l_higher_forward
					l_forward := l_higher_forward
				end
			--	l_quote_factor := l_currency_data.current_item.quote_factor
				--- assign value
				Result.set_lower_forward_period( l_lower_fxx_period )
				Result.set_higher_forward_period( l_higher_fxx_period )
				Result.set_lower_forward( l_lower_forward )
				Result.set_higher_forward( l_higher_forward )
				Result.set_forward_rate( l_forward * l_quote_factor )
			--	Result.set_quote_factor( l_quote_factor )
			--- get largest period if bigger than that, set -999
				l_flag := TRUE --- reset to original value
			else
				Result.set_forward_rate( -999 )
			end

			if l_flag = TRUE and l_fxx_period > 0 and l_currency_data.count = 1 then
				-- only one spot rate available, then no forward should exist
				Result.set_forward_rate(-999)
			end
		end
	end

	get_currency_data( a_date: ROSE_DATE; a_iso: STRING ): ROSE_LINEAR_ARRAY[ TEMPLATE_CURRENCY_DATA_SIGNATURES ] is
	local
		l_record: TEMPLATE_CURRENCY_DATA_SIGNATURES
		l_short_iso, l_ticker: STRING
		i, l_period: INTEGER
		l_start_date, l_end_date: ROSE_DATE
	do
		create Result.make_empty
		if currency_map.has( a_iso ) then
			l_short_iso := currency_map.name_from_ukey( a_iso )
		else
			l_short_iso := ""
		end
		l_start_date := a_date.date


		--- get spot rate first
		l_record := get_currency_ticker_data( a_date, a_iso )
		if l_record /= void then
			l_record.set_forward_period( 0 )
			Result.extend( l_record )
		end

		--- get d record
		if a_date.ccyymmdd < 20040726 then
			from i := 1
			until i > 8
			loop
				l_ticker := l_short_iso + i.out + "D"
				l_record := get_currency_ticker_data( a_date, l_ticker )
				if l_record /= void then
					l_record.set_forward_period(i)
					Result.extend( l_record )
				end
				i := i + 1
			end
		else
			l_ticker := a_iso + "ON"
			l_record := get_currency_ticker_data( a_date, l_ticker )
			if l_record /= void then
				l_record.set_forward_period(1)
				Result.extend( l_record )
			end
			l_ticker := a_iso + "TN"
			l_record := get_currency_ticker_data( a_date, l_ticker )
			if l_record /= void then
				l_record.set_forward_period(2)
				Result.extend( l_record )
			end
			l_ticker := a_iso + "SN"
			l_record := get_currency_ticker_data( a_date, l_ticker )
			if l_record /= void then
				l_record.set_forward_period(3)
				Result.extend( l_record )
			end
		end

		--- get w record
		from i := 1
		until i > 50
		loop
			if a_date.ccyymmdd < 20040726 then
				l_ticker := l_short_iso + i.out + "W"
			else
				l_ticker := a_iso + i.out + "W"
			end
			l_record := get_currency_ticker_data( a_date, l_ticker )
			if l_record /= void then
				l_period := i * 7
				l_record.set_forward_period( l_period )
				Result.extend( l_record )
			end
			i := i + 1
		end


		--- get m record
		from i := 1
		until i > 50
		loop
			if a_date.ccyymmdd < 20040726 then
				l_ticker := l_short_iso + i.out + "M"
			else
				l_ticker := a_iso + i.out + "M"
			end
			l_record := get_currency_ticker_data( a_date, l_ticker )
			if l_record /= void then
				l_end_date := clone( l_start_date )
				l_end_date.add_n_months( i )
				l_period := l_end_date.diff_in_days( l_start_date )
				l_record.set_forward_period( l_period )
				Result.extend( l_record )
			end
			i := i + 1
		end


		--- get y record

		from i := 1
		until i > 50
		loop
			if a_date.ccyymmdd < 20040726 then
				l_ticker := l_short_iso + i.out + "Y"
			else
				l_ticker := a_iso + i.out + "Y"
			end
			l_record := get_currency_ticker_data( a_date, l_ticker )
			if l_record /= void then
				l_end_date := clone( l_start_date )
				l_end_date.add_n_years( i )
				l_period := l_end_date.diff_in_days( l_start_date )
				l_record.set_forward_period( l_period )
				Result.extend( l_record )
			end
			i := i + 1
		end

		if Result /= void and Result.count > 1 then
			Result.sort_by( agent compare_forward_period( 0, ?, ?, TRUE ))
		end
	end

	get_currency_ticker_data( a_date: ROSE_DATE; a_ticker: STRING ):TEMPLATE_CURRENCY_DATA_SIGNATURES is
	local
		l_array: ROSE_LINEAR_ARRAY[ TEMPLATE_CURRENCY_DATA_SIGNATURES ]
	do
		select_currency_signature_predicate.retrieve( void, << a_date, a_ticker >> )
		if not select_currency_signature_predicate.success then
			fail ("get_currency_data", "Could not make predicate for select_currency_signature_predicate date '$1' ticker '$2'",<<a_date,a_ticker>>)
		end
		l_array := select_currency_signature_predicate.array
		if l_array /= void and l_array.count > 0 then
			Result := l_array.item(1)
		else
			Result := void
		end
	end


	get_currency_history_from_ticker( a_ticker: STRING ):ROSE_LINEAR_ARRAY [TEMPLATE_CURRENCY_DATA_SIGNATURES] is
	local
		l_array: ROSE_LINEAR_ARRAY[ TEMPLATE_CURRENCY_DATA_SIGNATURES ]
	do
		select_currency_history_predicate.retrieve( void, << a_ticker >> )
		if not select_currency_history_predicate.success then
			fail ("get_currency_history_from_ticker","Could not make predicate for select_currency_history_predicate ticker '$1'",<<a_ticker>>)
		end
		l_array := select_currency_history_predicate.array
		if l_array /= void and l_array.count > 0 then
			Result := l_array
		else
			Result := void
		end
	end

	get_currency_history_map_from_ticker( a_ticker: STRING ):ROSE_HASH_TABLE [DOUBLE, ROSE_DATE] is
	local
		l_array: ROSE_LINEAR_ARRAY[ TEMPLATE_CURRENCY_DATA_SIGNATURES ]
		l_short_iso, l_first, l_second, l_short_first, l_short_second: STRING
		l_quote_factor, l_final_price: DOUBLE
	do
		l_array := get_currency_history_from_ticker( a_ticker )
		if l_array /= void then
			create Result.make(1)
			from l_array.start
			until l_array.off
			loop
				l_quote_factor := l_array.current_item.quote_factor
				l_final_price := l_array.current_item.px_last / l_quote_factor
				Result.put( l_final_price, l_array.current_item.price_date.date )
				l_array.forth
			end
			-- if cross rate, get short iso
			if a_ticker.count = 6 then
				if currency_map.has( a_ticker ) then
					l_short_iso := currency_map.name_from_ukey( a_ticker )
				else
					l_first := a_ticker.substring(1,3)
					l_second := a_ticker.substring(4,6)
					if currency_map.has( l_first ) and currency_map.has( l_second ) then
						l_short_first := currency_map.name_from_ukey( l_first )
						l_short_second := currency_map.name_from_ukey( l_second )
						l_short_iso := l_short_second + l_short_first
					end
				end
				if l_short_iso /= void and l_short_iso.count > 0 then
					l_array := get_currency_history_from_ticker( l_short_iso )
					from l_array.start
					until l_array.off
					loop
						l_quote_factor := l_array.current_item.quote_factor
						l_final_price := l_array.current_item.px_last / l_quote_factor
						Result.put( l_final_price, l_array.current_item.price_date.date )
						l_array.forth
					end
				end
			end
		end
	end

feature {NONE} -- Implementation

	compare_forward_period( i: INTEGER; a,b: TEMPLATE_CURRENCY_DATA_SIGNATURES; j: BOOLEAN ): BOOLEAN is
	do
		if a.forward_period < b.forward_period then
			Result := TRUE
		else
			Result := FALSE
		end
	end

	select_currency_signature_predicate: COLLECTING_PREDICATE[ TEMPLATE_CURRENCY_DATA_SIGNATURES ] is
	local
		l_statement: STRING
	once
		l_statement := " select * from currency_data_signatures where price_date = ? and ticker = ?"
		create Result.make( bloomberg_currency_server, l_statement)
	end

	select_currency_history_predicate: COLLECTING_PREDICATE[ TEMPLATE_CURRENCY_DATA_SIGNATURES ] is
	local
		l_statement: STRING
	once
		l_statement := " select * from currency_data_signatures where ticker = ?"
		create Result.make( bloomberg_currency_server, l_statement)
	end

	currency_map: ROSE_LOOKUP_TABLE[ STRING, STRING ] is
	once
		create Result.make_table( bloomberg_currency_server, "currency_map", "iso", "short_iso" )
	end

end -- class BLOOMBERG_CURRENCY_ACCESS
