indexing
	description: "ROSE_CONTEXT version of PERSISTENT_BOND"
	author: "Mark Howard"
	date: "$Date: 2012/06/22 $"
	revision: "$Revision: #30 $"
	path: "$File: //rose/source/kernel/investment_entities/bond.e $"
	last_editor: "$Author: mk $"
	last_review: "Review #612: @150501 - last_currency for bond with past maturation"

class
	BOND
inherit
   INSTRUMENT
	undefine
		is_equal
	  end
   PRICEABLE

   	INVESTMENT_ENTITY
		undefine
			peer_anchor,
			ur_targets
		end
	PERSISTENT_BOND
		undefine
			context,
			make,
			make_context,
			ur_targets,
			debug_output
		end
	LEGACY_INSTRUMENT
create
	make,make_anonymous
feature -- Basic operation

	price_record (a_date : ROSE_DATE) : PRICE_RECORD is
			-- Price Record as of a date
		do
			Result := last_price_record (a_date)
			if Result /= Void and then Result.date /= Void and then a_date /= Void
			and then Result.date.ccyymmdd /= a_date.ccyymmdd then
				Result := Void
			end
		end

	last_price_record (a_date : ROSE_DATE) : PRICE_RECORD is
		-- Last price record as of 'a_date' (most recent price record available on or before 'a_date')
		local
			l_record : ROSE_BOND_RECORD
		do
			if a_date.ccyymmdd < start_date.ccyymmdd  then
				l_record := bond_record.last_record(bxrf_key (start_date).to_integer,a_date)
			else
			l_record := bond_record.last_record(bxrf_key (a_date).to_integer,a_date)
			end

			if l_record /= void then
				Result := l_record.record
			end
		end

	last_price_record_160 (a_date : ROSE_DATE) : PRICE_RECORD is
			-- Last price record (note: mode 160 not implemented for BONDs)
		do
			Result := last_price_record (a_date)
		end

	register is
			-- Register this entity.
		do
		end
	ur_targets : ROSE_LINEAR_ARRAY[ROSE_ENTITY_DATE[BOND]] is
		-- target
		once
			create Result.make_from_array(<<bond_record>>)
		end
	bond_record : ROSE_BOND_RECORD is
		-- bond record
		once
			create Result
		end
	last_close (a_date : ROSE_DATE) : DOUBLE is
		-- last close price
		require
			date_not_void: a_date /= void
		local
			l_record : ROSE_BOND_RECORD
		do
			l_record := bond_record.last_record(bxrf_key (a_date).to_integer,a_date)
			if l_record/=void then
				Result := l_record.record.close
			end
		end
	last_currency (a_date : ROSE_DATE) : CURRENCY is
		-- currency
		require
			date_not_void: a_date /= void
		local
			l_record : ROSE_BOND_RECORD
		do
			l_record := bond_record.last_record(bxrf_key (a_date).to_integer,a_date)
			if l_record/=void then
				Result := l_record.record.currency
--				Result := context.currency_from_iso_code(l_record.record.currency,a_date)
			end
		end
	last_date (a_date : ROSE_DATE) : ROSE_DATE is
		-- last date
		require
			date_not_void: a_date /= void
		local
			l_record : ROSE_BOND_RECORD
		do
			l_record := bond_record.last_record(bxrf_key (a_date).to_integer,a_date)
			if l_record/=void then
				Result := l_record.date
			end
		end
	adjusted_last_price (a_as_of_date,a_date : ROSE_DATE) : REAL is
		-- adjusted last price
		do
		end
	return (a_start,a_end : ROSE_DATE) : REAL is
		-- return
		do
		end
	name (a_date : ROSE_DATE) : STRING is
			-- Name as of `a_date'.
		do
			Result := name_at (a_date)
		end

	asset_class_code (a_date: ROSE_DATE) : STRING is
			-- asset class code
		do
			Result := "F" -- Fixed income
		end

	risk_region_code (a_date: ROSE_DATE) : STRING is
			-- region
		local
			s: STRING
		do
			s := brim_code(a_date)
			if s /= void then
				Result := s.substring(1,2)
			end
		end

end
