note

	description:

		"Encapsulation of one calendar for one country (bank or market)."

	author: "Randy John"
	last_editor: "$Author: rj $"
	date: "$Date: 2012/06/27 $"
	revision: "$Revision: #27 $"
	path: "$File: //rose/source/kernel/datetime/business_date_bitmap.e $"

class BUSINESS_DATE_BITMAP

inherit
	EXCEPTIONS
		undefine
			copy,
			is_equal
		end
	ROSE_BITMAP_BASE
		rename
			make as rose_bitmap_make
		end
	ROSE_ID_USER
		undefine
			copy,
			is_equal
		end
	DATABASE_SERVER
		undefine
			copy,
			is_equal
		end

create

	make,
	retrieve

create {CALENDAR_UPDATE_BASE}

	retrieve_from_db,
	make_use_brim,
	make_use_rose

feature {NONE} -- Creation

	retrieve (a_calendar_mnemonic: STRING) is
			--retrieve by mnemonic
		require
			valid_mnemonic: a_calendar_mnemonic /= Void
		do
			mnemonic := a_calendar_mnemonic
			make (mnemonic)
		end

	make (a_calendar_mnemonic: STRING) is
		require
			valid_mnemonic: a_calendar_mnemonic /= Void
		local
			i: INTEGER
			j: INTEGER
			l_day_of_the_week: INTEGER
			l_exceptional_holidays: ROSE_LINEAR_ARRAY [ROSE_DATE]
			l_regular_holiday: BRIM_REGULAR_HOLIDAY
			l_regular_holidays: ROSE_LINEAR_ARRAY [BRIM_REGULAR_HOLIDAY]
			l_date: ROSE_DATE
			l_saturday_found: BOOLEAN
			l_sunday_found: BOOLEAN
			l_min_date: ROSE_DATE
			l_max_date: ROSE_DATE
			l_last_rjd: INTEGER
			l_last_valid_date: ROSE_DATE
			l_last_full_byte: INTEGER
			l_today: ROSE_DATE
		do
			mnemonic := a_calendar_mnemonic
			exceptional_holiday_predicate.reset_array
			exceptional_holiday_predicate.retrieve(Void, <<calendar_id>>)
			if not exceptional_holiday_predicate.success
			then
				raise ("BUSINESS_DATE_BITMAP - exceptional_holiday_predicate failed.")
			end

			l_exceptional_holidays := exceptional_holiday_predicate.array
			if not l_exceptional_holidays.is_empty then
				first_valid_rjd := l_exceptional_holidays.first.rose_julian_day
				last_valid_rjd := l_exceptional_holidays.last.rose_julian_day
				create l_last_valid_date.make_from_rose_julian_day (last_valid_rjd)

				if l_last_valid_date<vendor_cutoff_date
				then
					if l_last_valid_date.month>4
					then
						-- If someone has partially filled in the exceptional holidays, we assume that
						-- they want the calendar to be valid through the end of the year.  In the past
						-- the calendar would stop at the last exceptional holiday, for instance, 12/25,
						-- and that was where the calendar stopped.  We are assuming here that calendars
						-- are updated for an entire year at a time.
						l_last_valid_date.set_month(12)
						l_last_valid_date.set_day(31)
						last_valid_rjd := l_last_valid_date.rose_julian_day
					end
				else
					-- Since our vendor always sends data 13 months in advance, we assume that our
					-- calendar should end 13 months greater than today or 'l_last_valid_date' -
					-- whichever is smaller.  If the vendor stops sending data, we won't set the last
					-- date more than 13 months past the last date that the vendor provided.
					create l_today.make_today
					if l_last_valid_date<l_today
					then
						l_last_valid_date.add_n_months(13)
					else
						l_last_valid_date := l_today.twin
						l_last_valid_date.add_n_months(13)
					end

					last_valid_rjd := l_last_valid_date.rose_julian_day
				end

				-- Set all days to be working days.
				rose_bitmap_make (last_valid_rjd)

				-- Clear all dates that are before the first valid day.
				l_last_full_byte := first_valid_rjd // 8
				from
					i := l_last_full_byte
				until
					i < 1
				loop
					array_put (0, i)
					i := i - 1
				end
				set_cardinality
				from
					i := l_last_full_byte * 8 + 1
				until
					i > first_valid_rjd
				loop
					set (False, i)
					i := i + 1
				end

				-- Clear all exceptional holidays.
				from
					i := 1
				until
					i > l_exceptional_holidays.count
				loop
--					j := l_exceptional_holidays.item(i).rose_julian_day
					set (False, l_exceptional_holidays.item (i).rose_julian_day)
					i := i + 1
				end

				-- Retrieve all regular holidays (Saturday, Sunday, whatever).
				create l_regular_holidays.make_empty
				regular_holiday_predicate.set_container (l_regular_holidays)
				regular_holiday_predicate.retrieve (Void, <<calendar_id>>)
				if not regular_holiday_predicate.success then
					raise ("BUSINESS_DATE_BITMAP - regular_holiday_predicate failed.")
				end

				-- Check for Saturdays and Sundays.
				from
					i := l_regular_holidays.lower
				until
					i > l_regular_holidays.upper or (l_saturday_found and l_sunday_found)
				loop
					if l_regular_holidays.item (i).offset_from_sunday = 0 then
						l_sunday_found := True
					end
					if l_regular_holidays.item (i).offset_from_sunday = 6 then
						l_saturday_found := True
					end
					i := i + 1
				end
				create l_min_date.make_min_date
				create l_max_date.make_max_date

				-- We assume that Saturday and Sunday are regular holidays even if not specifically mentioned.
				if not l_sunday_found then
					create l_regular_holiday.make (calendar_id, 0, l_min_date, l_max_date)
					l_regular_holidays.extend (l_regular_holiday)
				end
				if not l_saturday_found then
					create l_regular_holiday.make (calendar_id, 6, l_min_date, l_max_date)
					l_regular_holidays.extend (l_regular_holiday)
				end

				-- Clear all regular holidays.
				from
					i := l_regular_holidays.lower
				until
					i > l_regular_holidays.upper
				loop
					-- Change offset_from_sunday to Sun=1, Mon=2, ..., Sat=7.
					l_regular_holidays.item (i).set_offset_from_sunday (l_regular_holidays.item (i).offset_from_sunday + 1)
					from
						j := l_regular_holidays.item (i).start_date.rose_julian_day
						if j < 1 then
							j := 1
						end
						create l_date.make_from_rose_julian_day (j)
						l_day_of_the_week := l_date.day_of_the_week

						-- l_last_rjd is a speed improvement.
						l_last_rjd := l_regular_holidays.item (i).end_date.rose_julian_day
					until
						j > l_last_rjd or j > last_valid_rjd
					loop
						if l_day_of_the_week = l_regular_holidays.item (i).offset_from_sunday then
							set (False, j)
							j := j + 7
						else
							j := j + 1
							l_day_of_the_week := l_day_of_the_week + 1
							if l_day_of_the_week > 7 then
								l_day_of_the_week := 1
							end
						end
					end
					i := i + 1
				end
			end
		ensure
			invalid_first_and_last_boundary: not get (first_valid_rjd - 1) and not get (last_valid_rjd + 1)
		end

	make_use_brim is
			-- Use this for testing to force us to use brim_entities instead of the default.
		do
			use_brim.set_item (True)
		end

	make_use_rose is
			-- Use this for testing to force us to use rose instead of the default.
		do
			use_brim.set_item (False)
		end

feature -- Access

	first_valid_rjd: INTEGER

	last_valid_rjd: INTEGER

	mnemonic: STRING

	calendar_id: INTEGER is
			-- A calendar_id is really an entity_id but in the calendar tables it is called calendar_id.
		local
			l_research_calendar: RESEARCH_SUPPLIER_ENTITY
			l_brim_calendar: BRIM_ENTITIES_ENTITY
			l_now: ROSE_DATE
			l_brim_use: BRIM_ENTITIES_ATTRIBUTE_TYPE_ID_USE
			l_rose_ids: ROSE_ATTRIBUTE_IDS
		do
			if calendar_id_ref = Void then
				create calendar_id_ref
				create l_now.make_now
				if can_make_new_entities.item then
					if use_brim.item then
						create l_brim_use
						create l_brim_calendar.make_from_attribute_calling (ids.calendar_category_id, l_brim_use.mnemonic_type_id, mnemonic, l_now, Void, Void)
						calendar_id_ref.set_item (l_brim_calendar.entity_id)
					else
						create l_rose_ids
						create l_research_calendar.make_from_attribute_calling (ids.calendar_category_id, l_rose_ids.mnemonic_type_id, mnemonic, l_now, Void, Void)
						calendar_id_ref.set_item (l_research_calendar.entity_id)
					end
				else
					if use_brim.item then
						create l_brim_use
						create l_brim_calendar.attempt_make_from_attribute (ids.calendar_category_id, l_brim_use.mnemonic_type_id, mnemonic, l_now)
						if l_brim_calendar.valid then
							calendar_id_ref.set_item (l_brim_calendar.entity_id)
						else
							calendar_id_ref.set_item (0)
						end
					else
						create l_rose_ids
						create l_research_calendar.attempt_make_from_attribute (ids.calendar_category_id, l_rose_ids.mnemonic_type_id, mnemonic, l_now)
						if l_research_calendar.valid then
							calendar_id_ref.set_item (l_research_calendar.entity_id)
						else
							calendar_id_ref.set_item (0)
						end
					end
				end
			end
			Result := calendar_id_ref.item
		end

	valid_day (i: INTEGER): BOOLEAN is
		do
			Result := first_valid_rjd <= i and i <= last_valid_rjd
		end

	use_brim: BOOLEAN_REF is
		once
			create Result
			Result.set_item (False)
			-- Set this to False when we switch to Rose.
		end

	can_make_new_entities: BOOLEAN_REF is
		once
			create Result
			Result.set_item (False)
		end

feature {CALENDAR_UPDATE} -- Creation

	retrieve_from_db (a_calendar_mnemonic: STRING) is
		do
			mnemonic := a_calendar_mnemonic
			can_make_new_entities.set_item (True)
			make (mnemonic)
			can_make_new_entities.set_item (False)
		end

feature {NONE} -- Implementation

	exceptional_holiday_statement: STRING is "select calendar_date from exceptional_holiday where calendar_id = ? order by calendar_date"

	exceptional_holiday_predicate: DATE_PREDICATE is
		do
			if use_brim.item then
				Result := brim_exceptional_holiday_predicate
			else
				Result := rose_exceptional_holiday_predicate
			end
		end

	brim_exceptional_holiday_predicate: DATE_PREDICATE is
		once
			create Result.make (servers.brim_entities_server, exceptional_holiday_statement)
		end

	rose_exceptional_holiday_predicate: DATE_PREDICATE is
		once
			create Result.make (servers.rose_entity_foreground_server, exceptional_holiday_statement)
		end

	regular_holiday_statement: STRING is "select * from regular_holiday where calendar_id = ?"

	regular_holiday_predicate: COLLECTING_PREDICATE [BRIM_REGULAR_HOLIDAY] is
		do
			if use_brim.item then
				Result := brim_regular_holiday_predicate
			else
				Result := rose_regular_holiday_predicate
			end
		end

	brim_regular_holiday_predicate: COLLECTING_PREDICATE [BRIM_REGULAR_HOLIDAY] is
		once
			create Result.make (servers.brim_entities_server, regular_holiday_statement)
		end

	rose_regular_holiday_predicate: COLLECTING_PREDICATE [BRIM_REGULAR_HOLIDAY] is
		once
			create Result.make (servers.rose_entity_foreground_server, regular_holiday_statement)
		end

	date (a_year, a_month, a_day: INTEGER): ROSE_DATE is
		do
			create Result.make (a_year, a_month, a_day)
		end

feature {NONE} -- Once/constants.
	vendor_cutoff_date: ROSE_DATE
	-- We assume that calendars whose last valid date is on or after this date must be
	-- getting updated by the new calendar vendor.
	once
		create Result.make_ccyymmdd(20100101)
	end

feature {NONE} -- Attributes

	calendar_id_ref: INTEGER_REF

end
