indexing
	description: "Wrapper of an external C structure"
	author: "Zoran Simic"
	date: "$Date: 2009/09/11 $"
	revision: "$Revision: #4 $"
	archive: "$File: //rose/source/kernel/infrastructure/windows/c_struct.e $"

deferred class
	C_STRUCT

inherit
	DEBUG_OUTPUT

feature {NONE} -- Initialization

	make is
			-- Process instances of classes with no creation clause.
		do
			create pointer.make (struct_size)
		end

feature -- Access

	pointer: MANAGED_POINTER
			-- Managed pointer to structure

	struct_size: INTEGER is
			-- Size of this external C structure
		deferred
		end

	item: POINTER is
			-- Actual pointer to structure
		do
			if pointer /= Void then
				Result := pointer.item
			end
		end

	frozen debug_output: STRING is
			-- String that should be displayed in debugger to represent `Current'.
		do
			Result := generating_type
		end

feature -- Basic operations

	set_from_managed_pointer (a_pointer: MANAGED_POINTER) is
			-- Make from 'a_pointer', takes a copy of the content from 'a_pointer'
		require
			valid_pointer: a_pointer /= Void
		do
			create pointer.make_from_pointer (a_pointer.item, a_pointer.count)
		ensure
			valid_pointer: pointer /= Void
		end

	set_from_pointer (a_pointer: POINTER) is
			-- Make from 'a_pointer', takes a copy of the content from 'a_pointer'
		require
			valid_pointer: a_pointer /= default_pointer
		do
			create pointer.make_from_pointer (a_pointer, struct_size)
		ensure
			valid_pointer: pointer /= Void
		end

	share_from_pointer (a_pointer: POINTER) is
			-- Share from 'a_pointer', simply access contents from 'a_pointer'.
			-- Warning: memory pointed to by 'a_pointer' must remain allocated while this object is in use
			-- Do this only if you have to avoid creating a copy of the underlying structure,
			-- and need temporary access to its data
		require
			valid_pointer: a_pointer /= default_pointer
		do
			create pointer.share_from_pointer (a_pointer, struct_size)
		ensure
			valid_pointer: pointer /= Void and then pointer.item = a_pointer
		end

	clear is
			-- Clear 'pointer'
		do
			pointer := Void
		ensure
			valid_pointer: pointer = Void
		end

feature {NONE} -- Implementation

	to_string (a_pointer: POINTER): STRING is
			-- Convert widechar string pointed to by 'a_pointer' into a STRING
		local
			l_c_string: C_STRING
		do
			if a_pointer /= default_pointer then
				create l_c_string.make_by_pointer (a_pointer)
				Result := l_c_string.string
			end
		end

	to_string_32 (a_pointer: POINTER): STRING_32 is
			-- Convert widechar string pointed to by 'a_pointer' into a STRING
		local
			l_c_string_32: C_STRING_32
		do
			if a_pointer /= default_pointer then
				create l_c_string_32.make_by_pointer (a_pointer)
				Result := l_c_string_32.string
			end
		end

	system_codes: WINDOWS_SYSTEM_CODES is
			-- Windows system codes and GetLastError
		once
			create Result
		end

end
