note
	description: "The TIME_ZONE_INFORMATION represents the settings for a time zone"
	author: "Emmanuel Bouyer"
	last_editor: "$Author: e2 $"
	date: "$Date: 2012/12/06 $"
	revision: "$Revision: #1 $"
	path: "$File: //rose/source/kernel/infrastructure/windows/c_struct_time_zone_information.e $"
	document: "http://msdn.microsoft.com/en-us/library/windows/desktop/ms725481(v=vs.85).aspx"
	original: "[
		typedef struct _TIME_ZONE_INFORMATION {
		  LONG       Bias;
		  WCHAR      StandardName[32];
		  SYSTEMTIME StandardDate;
		  LONG       StandardBias;
		  WCHAR      DaylightName[32];
		  SYSTEMTIME DaylightDate;
		  LONG       DaylightBias;
		} TIME_ZONE_INFORMATION, *PTIME_ZONE_INFORMATION;
	]"
class
	C_STRUCT_TIME_ZONE_INFORMATION

inherit
	C_STRUCT

create
	make,
	make_from_reg_tzi_format

feature {NONE} -- Initialization

	make_from_reg_tzi_format (a_reg_tzi_format: MANAGED_POINTER)
			-- Initializes Current with (partial) information from the registry
		require
			a_reg_tzi_format_not_void: a_reg_tzi_format /= Void
			a_reg_tzi_format_item_not_void: a_reg_tzi_format.item /= default_pointer
			correct_structure_length: a_reg_tzi_format.count = 44
		do
			make
			c_set_from_format (pointer.item, a_reg_tzi_format.item)
		end

feature -- Access

	struct_size: INTEGER
			-- Size of this external C structure
		external
			"C blocking inline use <windows.h>"
		alias
			"sizeof(TIME_ZONE_INFORMATION)"
		end

feature {NONE} -- Implementation

	c_set_from_format (a_pointer: POINTER; a_value: POINTER)
			-- Set this structure from the REG_TZI_FORMAT structure `a_value'
		external
			"C blocking inline use <windows.h>"
		alias
			"[
			typedef struct _REG_TZI_FORMAT
			{
			    LONG Bias;
			    LONG StandardBias;
			    LONG DaylightBias;
			    SYSTEMTIME StandardDate;
			    SYSTEMTIME DaylightDate;
			} REG_TZI_FORMAT;
			((TIME_ZONE_INFORMATION *) $a_pointer)->Bias = ((REG_TZI_FORMAT *)$a_value)->Bias;
			((TIME_ZONE_INFORMATION *) $a_pointer)->StandardBias = ((REG_TZI_FORMAT *)$a_value)->StandardBias;
			((TIME_ZONE_INFORMATION *) $a_pointer)->DaylightBias = ((REG_TZI_FORMAT *)$a_value)->DaylightBias;
			((TIME_ZONE_INFORMATION *) $a_pointer)->StandardDate = ((REG_TZI_FORMAT *)$a_value)->StandardDate;
			((TIME_ZONE_INFORMATION *) $a_pointer)->DaylightDate = ((REG_TZI_FORMAT *)$a_value)->DaylightDate;
			((TIME_ZONE_INFORMATION *) $a_pointer)->StandardName[0] = 0;
			((TIME_ZONE_INFORMATION *) $a_pointer)->DaylightName[0] = 0;
			]"
		end

end
