indexing

	description:

		"Compute adjustments, returns, cash_flows, etc. from simultaneous rose capital actions"

	author: "Srilatha Singh/JS"
	date: "$Date: 2009/11/06 $"
	sccid: "$Id: //rose/source/kernel/capital_actions/expressions/capact_additive_expression.e#18 $"

class CAPACT_ADDITIVE_EXPRESSION

inherit

	CAPACT_EXPRESSION
		redefine
			terminal_distribution
		end

	CAPACT_COMMON
		export
			{NONE} all
		undefine
			default_create
		end

create

	make_with_expressions

feature -- creation

	make_with_expressions (a_left, a_right: CAPACT_EXPRESSION) is
			-- compute result by adding results from input expressions
		do
			set_expression_index
			left_expression := a_left
			right_expression := a_right
			if a_left.rmdat_adjustment_factor_available and a_right.rmdat_adjustment_factor_available then
				rmdat_adjustment_factor := 1 + (a_left.rmdat_adjustment_factor - 1) + (a_right.rmdat_adjustment_factor - 1)
			else
				rmdat_adjustment_factor := -999
			end
--		cash_value_for_price_adjustment := left_expression.cash_value_for_price_adjustment + a_right.cash_value_for_price_adjustment
		end

feature -- Access

	processing_instruction (a_basis: RP_CORPORATE_ACTION_PROCESSING_SHARE_BASIS_TYPE; a_corporate_action_processing_system: RP_CORPORATE_ACTION_PROCESSING_SYSTEM): RP_CORPORATE_ACTION_PROCESSING_INSTRUCTIONS is
			-- processing instructions for the corp action - no processing
		do
		end

	left_expression: CAPACT_EXPRESSION
			-- Left expression

	right_expression: CAPACT_EXPRESSION
			-- Right expression

	name: STRING is
			-- name
		do
			Result := left_expression.name + " and " + right_expression.name
		end

	terminal_distribution: ENTITY_VALUES [LISTING] is
			-- Sum of terminal distributions of leaves.
		local
			l_left, l_right: like terminal_distribution
		do
			l_left := left_expression.terminal_distribution
			l_right := right_expression.terminal_distribution
			if l_left = Void then
				Result := l_right
			elseif l_right = Void then
				Result := l_left
			else
				Result := l_left + l_right
			end
		end

	cash_outflow (a_currency: CURRENCY): DOUBLE is
			-- compute cash outflow from component expressions
		local
			l_right, l_left: DOUBLE
		do
			l_right := right_expression.cash_outflow (a_currency)
			l_left := left_expression.cash_outflow (a_currency)
			if is_defined_double (l_left) and then is_defined_double (l_right) then
				Result := l_left + l_right
			elseif is_defined_double (l_left) then
				Result := l_left
			elseif is_defined_double (l_right) then
				Result := l_right
			end
		end

	cash_inflow (a_currency: CURRENCY): DOUBLE is
			-- compute cash inflow from component expressions
		local
			l_right, l_left: DOUBLE
		do
			l_right := right_expression.cash_inflow (a_currency)
			l_left := left_expression.cash_inflow (a_currency)
			if is_defined_double (l_left) and then is_defined_double (l_right) then
				Result := l_left + l_right
			elseif is_defined_double (l_left) then
				Result := l_left
			elseif is_defined_double (l_right) then
				Result := l_right
			end
		end

	cash_dividend (a_currency: CURRENCY): DOUBLE is
			-- compute cash dividend from component expressions
		local
			l_right, l_left: DOUBLE
		do
			l_right := right_expression.cash_dividend (a_currency)
			l_left := left_expression.cash_dividend (a_currency)
			if is_defined_double (l_left) and then is_defined_double (l_right) then
				Result := l_left + l_right
			elseif is_defined_double (l_left) then
				Result := l_left
			elseif is_defined_double (l_right) then
				Result := l_right
			end
		end

feature --capact_capital_action

	instantiate_data is
			-- to be filled in
		do
		end

	template_name: STRING is
			-- name of template
		once
			Result := additive_expression_string
		end

	set_data_items is
      -- Set data items
		do
		end

	update_data_items is
		do
		end

end -- class CAPACT_ADDITIVE_EXPRESSION
