note

	description:

		"Rose capital action: Atomic cost flow"

	author: "Janet Small"
	last_editor: "$Author: it $"
	date: "$Date: 2014/02/24 $"
	revision: "$Revision: #41 $"
	path: "$File: //rose/source/kernel/capital_actions/capact_capital_action.e $"

deferred class CAPACT_CAPITAL_ACTION

inherit

	ROSE_AWARE
		rename
			context as rose_context
		export
			{NONE} all
		end

	CAPACT_DOUBLE_FORMATS
		export {NONE} all end

	CAPACT_COMMON
		export {NONE} all end

	ANY

feature -- Access

	string_items: HASH_TABLE [STRING, INTEGER]
			-- string items for virtual record

	integer_items: HASH_TABLE [INTEGER, INTEGER]
			-- integer items for virtual record

	item_count_estimate: INTEGER is 100

	ur_target: CAPACT_SIGNATURE_RECORD is
			-- Target
		once
			create Result
		end

	virtual_record: ROSE_VIRTUAL_RECORD [like ur_target]
			-- virtual record in rose capital actions database

	set_virtual_record (a_value: like virtual_record) is
			-- set virtual record to a_value
		do
			virtual_record := a_value
		end

	name: STRING is
			-- name of specific descendent of capital action
		deferred
		end

	signature: CAPACT_SIGNATURE_RECORD
			-- signature of capital action

	set_signature (a_value: like signature) is
			-- set signature of capital action
		do
			signature := a_value
		end

	zone: STRING

	set_zone (a_zone: like zone) is
			-- set zone of capital action
		do
			zone := a_zone
		end

	security: SECURITY

	set_security (a_value: like security) is
			--set security
		do
			security := a_value
		end

	parent_instrument: LISTING

	set_parent_instrument (a_value: like parent_instrument) is
			--
		do
			parent_instrument := a_value
		end

	reason: STRING
			-- irregular special,arrears payment,merger,merger defense,class action proceeds,
			-- in lieu of rights, overprovision of tax, Final, Interim, anniversary, special, regular intrest,
			--cummulative interest, Proceeds fromLiquidation - ongoing, Proceeds from Liquidation , etc.

	set_reason (a_value: like reason) is
			--
		do
			reason := a_value
		end

	periodicity: INTEGER
			-- periodicity id

	set_periodicity (a_value: like periodicity) is
			--
		do
			periodicity := a_value
		end

	announcement_date: INTEGER
			-- Announcement date

	set_announcement_date (a_value: like announcement_date) is
			--
		do
			announcement_date := a_value
		end

	payment_date: INTEGER

	set_payment_date (a_value: like payment_date) is
			-- paydate (aka settlement date)
		do
			payment_date := a_value
		end

	is_paydate_estimate: BOOLEAN

	set_is_paydate_estimate (a_value: like is_paydate_estimate) is
			-- for some countries + capact types we want to us unknown paydate until date is confirmed by data integrity
		do
			is_paydate_estimate := a_value
		end

	share_multiplier: DOUBLE

	set_share_multiplier (a_value: like share_multiplier) is
			--
		do
			share_multiplier := a_value
		end

	legacy_vendor_key: STRING

	set_legacy_vendor_key (a_value: like legacy_vendor_key) is
			--
		do
			legacy_vendor_key := a_value
		end

	legacy_txbas: STRING

	set_legacy_txbas (a_value: like legacy_txbas) is
			-- Set `legacy_txbas' to `a_value'
		do
			legacy_txbas := a_value
		ensure
			legacy_txbas_set: legacy_txbas = a_value
		end

	comment: STRING

	set_comment (a_value: like comment) is
			--
		do
			comment := a_value
		end

feature -- vendor specific data
	-- can't put in vendor interpretation, because it can have multiple capacts

	brimsym: STRING

	set_brimsym (a_value: like brimsym) is
			--
		do
			brimsym := a_value
		end

	change_date: INTEGER

	set_change_date (a_value: like change_date) is
			--
		do
			change_date := a_value
		end

	transmission_date: INTEGER

	set_transmission_date (a_value: like transmission_date) is
			--
		do
			transmission_date := a_value
		end

	pay_order: STRING

	set_pay_order (a_value: like pay_order) is
			--
		do
			pay_order := a_value
		end

	on_factor: DOUBLE

	set_on_factor (a_value: like on_factor) is
			--
		do
			on_factor := a_value
		end

	vendor_key: STRING

	set_vendor_key (a_value: like vendor_key) is
			--
		do
			vendor_key := a_value
		end

	dl_taxflag: STRING
			-- tmp: pass tax status to legacy

	set_dl_taxflag (a_value: like dl_taxflag) is
			--
		do
			dl_taxflag := a_value
		end

feature -- complex capital actions (aka. options)

	expression_string: STRING
			-- string indicating ordering of capital actions

	set_expression_string (a_value: like expression_string) is
			-- set string indicating ordering of capital actions
		do
			expression_string := a_value
		end

feature -- instantiate

	instantiate_data is
			-- retrieve data items from storage
		require
			has_signature: signature /= Void
		deferred
		end

	instantiate_vendor_specific_data is
			-- retrieve data specific to various vendors from storage
		local
			s: STRING
		do
			s := string_value (vendor_key_expression, signature)
			if s /= Void then
				set_vendor_key (s)
			end
			instantiate_legacy_items
		end

	instantiate_all is
			-- retrieve all capital action data from storage
		do
			instantiate_attributes
			instantiate_data
			instantiate_vendor_specific_data
			instantiate_legacy_items
		end

	instantiate_attributes is
			-- retrieve data common to all capital action types from storage
		local
			s: STRING
			i: INTEGER
			d: ROSE_DATE
		do
				--signature.data_manager.begin_transaction
			i := announcement_date_expression.integer_value (signature)
			if i > 0 then
				set_announcement_date (i)
			end
			i := payment_date_expression.integer_value (signature)
			if i >= 0 then
				set_payment_date (i)
			end
			s := string_value (is_paydate_estimate_expression, signature)
			if s /= Void and then s.is_equal ("T") then
				set_is_paydate_estimate (True)
			end
			s := string_value (reason_expression, signature)
			if s /= Void then
				set_reason (s)
			end
			i := periodicity_expression.integer_value (signature)
			if i > 0 then
				set_periodicity (i)
			end
			s := string_value (expression_string_expression, signature)
			if s /= Void then
				set_expression_string (s)
			end
			if signature.first_process_id /= 0 then
				d := signature.first_process_date_for_signature
				if d /= Void then
					set_transmission_date (d.ccyymmdd)
						-- this may be superceded by legacy_transmission_date if present
				end
			end
			if signature.last_process_id /= 0 then
				d := signature.last_process_date_for_signature
				if d /= Void then
					set_change_date (d.ccyymmdd)
						-- this may be superceded by legacy_change_date if present
				end
			end
			s := string_value (comment_expression, signature)
			set_comment (s)
				--signature.data_manager.data_server_commit
		end

	instantiate_entity_attributes is
			-- 	removed from instantiate_attributes, in case we need it, can be called separately	
		local
			i: INTEGER
			l_security: SECURITY
			l_exchange: EXCHANGE
			l_listing: LISTING
		do
			i := signature.listing_id
				--i := parent_instrument_expression.integer_value(signature)
			if i > 0 then
				l_listing := rose_context.listing_accessor.entity (i)
			else
				l_security := rose_context.security_accessor.entity (signature.instrument_id)
				if l_security /= Void then
					l_exchange := rose_context.exchange_accessor.entity (signature.market_id)
					if l_exchange /= Void then
						from
							i := 1
						until
							i > l_security.listing_count or l_listing /= Void
						loop
							l_listing := l_security.listing (i)
							if l_listing.exchange /= Void and then l_listing.exchange.handle /= l_exchange.handle then
								l_listing := Void
							end
							i := i + 1
						end
					end
				end
			end
			if l_listing /= Void then
				set_parent_instrument (l_listing)
				set_brimsym (l_listing.brim_code_most_recent_key (constants.now))
			end
		end

	instantiate_legacy_items is
			-- legacy backdiv and distribution specific fields
		local
			i: INTEGER
			s: STRING
		do
			i := legacy_change_date_expression.integer_value (signature)
			if i >= 0 then
				set_change_date (i)
			end
			i := legacy_transmission_date_expression.integer_value (signature)
			if i > 0 then
				set_transmission_date (i)
			end
			s := string_value (legacy_on_factor_expression, signature)
			if s /= Void and then s.is_double then
				set_on_factor (s.to_double)
			end
			s := string_value (legacy_payod_expression, signature)
			if s /= Void then
				set_pay_order (s)
			end
			s := string_value (share_multiplier_expression, signature)
			if s /= Void and then s.is_double then
				set_share_multiplier (s.to_double)
			end
			s := string_value (legacy_vendor_key_expression, signature)
			if s /= Void then
				set_legacy_vendor_key (s)
			end
		end

feature -- Set string and integer items from named features

	create_item_tables is
			-- create tables of items for virtual arecords
		do
			create string_items.make (item_count_estimate)
			create integer_items.make (item_count_estimate)
		end

	set_capital_action_items is
			-- enter data values into item tables for virtual records
		require
			valid_signature: signature /= Void
		do
			create_item_tables
			set_attribute_items
			set_data_items
		end

	set_attribute_items is
			-- for use in daily updates, with virtual records and override records
		require
			has_signature: signature /= Void
		local
			l_variable: CAPACT_VARIABLE_DESCRIPTION_ROW
			s: STRING
		do
			if parent_instrument /= Void then
				l_variable := variable_table.item ("PARENT_INSTRUMENT")
				integer_items.put (parent_instrument.handle, l_variable.item_id)
			end
			if reason /= Void then
				l_variable := variable_table.item ("REASON")
				string_items.put (reason, l_variable.item_id)
			end
			if periodicity > 0 then
				l_variable := variable_table.item ("PERIODICITY")
				integer_items.put (periodicity, l_variable.item_id)
			end
			if announcement_date > 0 and announcement_date < 90000000 then
				l_variable := variable_table.item ("ANNOUNCEMENT_DATE")
				integer_items.put (announcement_date, l_variable.item_id)
			end
			if payment_date > 0 then
					-- unknown paydate is now 99991231
				l_variable := variable_table.item ("PAYMENT_DATE")
				integer_items.put (payment_date, l_variable.item_id)
			end
			l_variable := variable_table.item ("IS_PAYDATE_ESTIMATE")
			if is_paydate_estimate = True then
				string_items.put ("T", l_variable.item_id)
			else
				string_items.put ("F", l_variable.item_id)
			end
			if expression_string /= Void then
				l_variable := variable_table.item ("EXPRESSION_STRING")
				string_items.put (reason, l_variable.item_id)
			end
				-- vendor specific itens
			if transmission_date > 0 and transmission_date < 90000000 then
				l_variable := variable_table.item ("LEGACY_TRANSMISSION_DATE")
				integer_items.put (transmission_date, l_variable.item_id)
			end
			if change_date > 0 and change_date < 90000000 then
				l_variable := variable_table.item ("LEGACY_CHANGE_DATE")
				integer_items.put (change_date, l_variable.item_id)
			end
			if pay_order /= Void and then not pay_order.is_equal ("0") then
				l_variable := variable_table.item ("LEGACY_PAYOD")
				string_items.put (pay_order, l_variable.item_id)
			end
			if on_factor /= 0 then
				l_variable := variable_table.item ("LEGACY_ONFACTOR")
				s := format_onfactor.formatted (on_factor)
				string_items.put (s, l_variable.item_id)
			end
			if share_multiplier /= 0 then
				l_variable := variable_table.item ("SHARE_MULTIPLIER")
				s := format_onfactor.formatted (share_multiplier)
				string_items.put (s, l_variable.item_id)
			end
			if legacy_vendor_key /= Void then
				l_variable := variable_table.item ("LEGACY_VENDORKEY")
				string_items.put (legacy_vendor_key, l_variable.item_id)
			end
			if legacy_txbas /= Void then
				l_variable := variable_table.item ("LEGACY_TXBAS")
				string_items.put (legacy_txbas, l_variable.item_id)
			end
			if vendor_key /= Void then
				l_variable := variable_table.item ("VENDORKEY")
				string_items.put (vendor_key, l_variable.item_id)
			end
			if comment /= Void then
				l_variable := variable_table.item ("COMMENT")
				string_items.put (comment, l_variable.item_id)
			end
		end

	set_data_items is
			--
		deferred
		end

feature -- convenience

	exdate: INTEGER is
			--
		do
			Result := signature.exdate
		end

	exdate_rose: ROSE_DATE

	option_member_id: INTEGER is
		do
--			Result := signature.option_member_id
		end

feature -- variables	

	initialize_variable_table is
			-- use variable description in common database shared by zones to retrieve variables
		local
			l_variable_retriever: SELECTION_PREDICATE [CAPACT_VARIABLE_DESCRIPTION_ROW]
			l_row: CAPACT_VARIABLE_DESCRIPTION_ROW
		once
			create l_row
			create l_variable_retriever.make (servers.rose_capital_action_common_server, l_row, "select * from variable_description")
			l_variable_retriever.apply (agent populate_variable_table (?), Void, <<>>)
			print ("variable description rows fetched: " + l_variable_retriever.records_fetched.out + "%N")
				-- don't commit; this is called from the once, might happen in the middle of a transaction
				-- should separate access from updating
		end

	display_capact_string: STRING is
			-- display capact string
		do
			if signature /= Void then
				Result := signature.display_signature
			end
		end

feature -- attribute expressions

	parent_instrument_expression: ROSE_EXPRESSION [ROSE_ENTITY_DATE [ENTITY]] is
			--
		once
			Result := ur_target.expression_from_name ("PARENT_INSTRUMENT")
		end

	announcement_date_expression: ROSE_EXPRESSION [ROSE_ENTITY_DATE [ENTITY]] is
			--
		once
			Result := ur_target.expression_from_name ("ANNOUNCEMENT_DATE")
		end

	payment_date_expression: ROSE_EXPRESSION [ROSE_ENTITY_DATE [ENTITY]] is
			--
		once
			Result := ur_target.expression_from_name ("PAYMENT_DATE")
		end

	is_paydate_estimate_expression: ROSE_EXPRESSION [ROSE_ENTITY_DATE [ENTITY]] is
			--
		once
			Result := ur_target.expression_from_name ("IS_PAYDATE_ESTIMATE")
		end

	reason_expression: ROSE_EXPRESSION [ROSE_ENTITY_DATE [ENTITY]] is
			--
		once
				--Result := ur_target.expression("CAPACT_REASON")
			Result := ur_target.expression_from_name ("REASON")
		end

	periodicity_expression: ROSE_EXPRESSION [ROSE_ENTITY_DATE [ENTITY]] is
			--
		once
			Result := ur_target.expression_from_name ("PERIODICITY")
		end

	expression_string_expression: ROSE_EXPRESSION [ROSE_ENTITY_DATE [ENTITY]] is
			--
		once
			Result := ur_target.expression_from_name ("EXPRESSION_STRING")
		end

	instrument_expression: ROSE_EXPRESSION [ROSE_ENTITY_DATE [ENTITY]] is
			-- do we need this? the instrument is represented in the signature
			-- we'll have to define multiple names where we have multiple instruments
		once
			Result := ur_target.expression_from_name ("INSTRUMENT")
		end

	share_multiplier_expression: ROSE_EXPRESSION [ROSE_ENTITY_DATE [ENTITY]] is
			--
		once
			Result := ur_target.expression_from_name ("SHARE_MULTIPLIER")
		end

	comment_expression: ROSE_EXPRESSION [ROSE_ENTITY_DATE [ENTITY]] is
			-- Comment expression
		once
			Result := ur_target.expression_from_name ("COMMENT")
		end

feature -- vendor specific

	legacy_payod_expression: ROSE_EXPRESSION [ROSE_ENTITY_DATE [ENTITY]] is
			--
		once
			Result := ur_target.expression_from_name ("LEGACY_PAYOD")
		end

	legacy_on_factor_expression: ROSE_EXPRESSION [ROSE_ENTITY_DATE [ENTITY]] is
			--
		once
			Result := ur_target.expression_from_name ("LEGACY_ONFACTOR")
		end

	legacy_transmission_date_expression: ROSE_EXPRESSION [ROSE_ENTITY_DATE [ENTITY]] is
			--
		once
			Result := ur_target.expression_from_name ("LEGACY_TRANSMISSION_DATE")
		end

	legacy_change_date_expression: ROSE_EXPRESSION [ROSE_ENTITY_DATE [ENTITY]] is
			--
		once
			Result := ur_target.expression_from_name ("LEGACY_CHANGE_DATE")
		end

	legacy_basis_currency_expression: ROSE_EXPRESSION [ROSE_ENTITY_DATE [ENTITY]] is
			--
		once
			Result := ur_target.expression_from_name ("LEGACY_BASIS_CURRENCY")
		end

	legacy_basis_prop_expression: ROSE_EXPRESSION [ROSE_ENTITY_DATE [ENTITY]] is
			--
		once
			Result := ur_target.expression_from_name ("LEGACY_BASIS_PROP")
		end

	legacy_basis_price_expression: ROSE_EXPRESSION [ROSE_ENTITY_DATE [ENTITY]] is
			--
		once
			Result := ur_target.expression_from_name ("LEGACY_BASIS_PRICE")
		end

	legacy_order_expression: ROSE_EXPRESSION [ROSE_ENTITY_DATE [ENTITY]] is
			--
		once
			Result := ur_target.expression_from_name ("LEGACY_ORDER")
		end

	legacy_user_id_expression: ROSE_EXPRESSION [ROSE_ENTITY_DATE [ENTITY]] is
			--
		once
			Result := ur_target.expression_from_name ("LEGACY_USER_ID")
		end

	legacy_comment_expression: ROSE_EXPRESSION [ROSE_ENTITY_DATE [ENTITY]] is
			--
		once
			Result := ur_target.expression_from_name ("LEGACY_COMMENT")
		end

	legacy_vendor_key_expression: ROSE_EXPRESSION [ROSE_ENTITY_DATE [ENTITY]] is
			--
		once
			Result := ur_target.expression_from_name ("LEGACY_VENDORKEY")
		end

	legacy_txbas_expression: ROSE_EXPRESSION [ROSE_ENTITY_DATE [ENTITY]] is
			--legacy tax base expression
		once
			Result := ur_target.expression_from_name ("LEGACY_TXBAS")
		end

	vendor_key_expression: ROSE_EXPRESSION [ROSE_ENTITY_DATE [ENTITY]] is
			--
		once
			Result := ur_target.expression_from_name ("VENDORKEY")
		end

	format_onfactor: FORMAT_DOUBLE is
			--
		once
			create Result.make (14, 8)
		end

feature {NONE} -- Implementation

	brim_sources: ROSE_LINEAR_ARRAY [VENDOR_CAPITAL_ACTION]

	variable_table: HASH_TABLE [CAPACT_VARIABLE_DESCRIPTION_ROW, STRING] is
			--
		once
			create Result.make (100)
			initialize_variable_table
		end

	populate_variable_table (a_row: CAPACT_VARIABLE_DESCRIPTION_ROW) is
		local
			l_row: CAPACT_VARIABLE_DESCRIPTION_ROW
		do
			l_row := a_row.twin
			l_row.name.right_adjust
			l_row.type.right_adjust
			variable_table.put (l_row, l_row.name)
		end

	string_value (a_expression: ROSE_EXPRESSION [ANY]; a_target: ANY): STRING is
			-- String value for a_expression
		do
			Result := a_expression.string_value (a_target)
		end


end -- class CAPACT_CAPITAL_ACTION
