note

	description:

		"rose capital action expression: cash dividend"

	author: "Srilatha Singh"
	last_editor: "$Author: it $"
	date: "$Date: 2013/08/13 $"
	revision: "$Revision: #33 $"
	path: "$File: //rose/source/kernel/capital_actions/expressions/capact_cash_dividend_expression.e $"

class CAPACT_CASH_DIVIDEND_EXPRESSION

inherit

	CAPACT_EXPRESSION

	CAPACT_CASH_DIVIDEND
		rename
			signature as target,
			exdate as integer_exdate,
			name as template_name
		undefine
			default_create
		end

create

	make,
	instantiate,
	make_empty

feature {NONE} -- creation

	instantiate (a_target: CAPACT_SIGNATURE_RECORD) is
			-- populate from storate
		do
			set_expression_index
			target := a_target
			set_signature (a_target)
			instantiate_all
			rmdat_adjustment_factor := 1
		end

feature -- Access

	processing_instruction (a_basis: RP_CORPORATE_ACTION_PROCESSING_SHARE_BASIS_TYPE; a_processing_system: RP_CORPORATE_ACTION_PROCESSING_SYSTEM): RP_CORPORATE_ACTION_PROCESSING_INSTRUCTIONS is
			-- processing corp action instruction
		do
			create Result.make (a_basis, create {RP_SIMPLE_CORPORATE_ACTION}.make (exdate, a_processing_system.cash_dividend_instructions (Current)))
		end

feature -- Access

	dividend_cash_received: DOUBLE
			-- cash received

	cash_outflow (a_currency: CURRENCY): DOUBLE is
			-- no cash outflow for this type of capital action
		do
			Result := 0
		end

	cash_inflow, cash_dividend (a_currency: CURRENCY): DOUBLE is
			-- cash inflow or cash_dividends computed from capital action expressed in input currency
		local
			l_xrate: DOUBLE
		do
			if cash_currency = Void and then target.listing /= Void then
				if target.listing.zone_mnemonic.as_upper = 'J' then
					cash_currency := rose_context.currency_by_iso_code ("JPY", exdate)
				end
			end
			if (a_currency /= Void and cash_currency /= Void) and then a_currency.handle = cash_currency.handle then
				Result := cash_rate
			else
				if cash_currency = Void or exdate = Void then
					Result := undefined_double
				else
					if (a_currency /= Void and cash_currency /= Void) then
						l_xrate := a_currency.exchange_rate (cash_currency, exdate.min (today_rdt))
						if legacy_undefined.is_defined_double (l_xrate) then
							Result := cash_rate * l_xrate
						else
							Result := undefined_double
						end
					else
						Result := undefined_double
					end
				end
			end

--			In U zone, dividends may be affected by splits that are several days or weeks in the future.
--			Such capacts may then not be "sufficiently coupled" by their exdate. We should be able to couple them by their record date.
--			With such coupling, we should then be able to apply the capact split factor to the coupled dividend, independently of the exdate.
--			The uncoupling/coupling of capact by exdate or record dates should be specific to Uzone and requires new design
--			as "share_multiplier" would then be computed from this coupling.
--			In the meantime, we are just applying a manually entered share_multiplier
			if is_defined_double (share_multiplier) and then share_multiplier /= 0 then
				Result := Result * share_multiplier -- In U zone can have dividend affected by split several days or weeks away
			end

			Result := cash_dividend_value (Current, Result)
		end

	name: STRING is
			-- Name
		do
			Result := template_name + " F" + target.member_number.out
		end

	adjustment_factor: DOUBLE is
			-- This may be redundant
		do
			Result := rmdat_adjustment_factor
		end

feature -- invalid from CAPACT_EXPRESSION

	legacy_basis_currency: STRING is
			-- basis currency
		do
			Result := Void
		end

	share_factor: DOUBLE is
			-- share factor
		do
			Result := undefined_double
		end

	legacy_basis_price: DOUBLE is
			-- Legacy basis price
		do
			Result := undefined_double
		end

	subscription_price: DOUBLE is
			-- Subscription price
		do
			Result := undefined_double
		end

	child_units: DOUBLE is
			-- child units not relevant to cash dividend
		do
			Result := undefined_double
		end

	legacy_basis_prop: DOUBLE is
			-- basis prop not relevant to cash distribution
		do
			Result := undefined_double
		end

	split_rate: DOUBLE is
			-- split rate not relevant to cash distribution
		do
			Result := undefined_double
		end

	bonus_rate: DOUBLE is
			-- Bonus rate
		do
			Result := undefined_double
		end

	dividend_rate: DOUBLE is
			-- dividend rate
		do
			Result := undefined_double
		end

end -- class CAPACT_CASH_DIVIDEND_EXPRESSION
