note

	description:

	"[
		Cash dividend value calculation control
	]"

	author: "Ivan Tarnavsky"
	last_editor: "$Author: it $"
	date: "$Date: 2014/02/05 $"
	revision: "$Revision: #1 $"
	path: "$File: //rose/source/kernel/capital_actions/expressions/capact_cash_dividend_value_control.e $"

class CAPACT_CASH_DIVIDEND_VALUE_CONTROL

create {CAPACT_SHARED_CASH_DIVIDEND_VALUE_CONTROL}

	default_create

feature -- Query

	is_special_dividend (a_capact: CAPACT_CAPITAL_ACTION): BOOLEAN is
			-- Is `a_capact' signifying special dividend?
		local
			l_reason: STRING
			l_listing: LISTING
			l_use_reason: BOOLEAN
			l_divtype: STRING
			l_source: STRING
		do
			if a_capact /= Void then
				if a_capact.signature /= Void and then a_capact.signature.source /= Void then
					l_source := a_capact.signature.source.as_upper
						-- Use REASON for manual and IDSI entries
					if l_source.has_substring ("OVERRIDE") or l_source.has_substring ("IDSI") then
						l_use_reason := True
					else
						l_listing := a_capact.signature.listing
						if l_listing /= Void and then l_listing.zone /= Void then
							if l_listing.zone.is_c_zone then
									-- C zone is all IDSI even the legacy BRIM records
								l_use_reason := True
							elseif l_listing.zone.is_u_zone and not l_source.has_substring ("DATALINE") then
									-- use divtype in U zone only for DATALINE records (minority)
								l_use_reason := True
							else
									-- Use DATALINE DIVTYPE
									-- in E, J, O, S zones and U zone dataline records
								l_divtype := a_capact.legacy_vendor_key
								if l_divtype /= Void and then l_divtype.count >= 4 then
									l_divtype := l_divtype.substring (3, 4).as_upper
									Result := not dataline_regular_divtypes.has (l_divtype)
								else
									l_divtype := Void
									l_use_reason := True
								end
							end
						else
							l_use_reason := True
						end
					end
				else
					l_use_reason := True
				end
				if l_use_reason and then a_capact.reason /= Void then
					l_reason := a_capact.reason.as_upper
					Result := l_reason.has_substring ("SPECIAL")
							or l_reason.has_substring ("MERGER")
							or l_reason.has_substring ("ANNIVERSARY")
							or l_reason.has_substring ("RIGHTS")
							or l_reason.has_substring ("ARREARS")
							or l_reason.has_substring ("GAINS")
							or l_reason.has_substring ("MEMORIAL")
				end
				if is_treating_interim_dividends_as_special and then a_capact.reason /= Void then
					l_reason := a_capact.reason.as_upper
					Result := Result or l_reason.has_substring ("INTERIM")
				end
				if is_treating_non_quarterly_dividends_as_special and l_divtype /= Void then
					Result := Result or (l_divtype /~ quarterly
										and l_divtype /~ quarterly_installment
										and l_divtype /~ quarterly_with_alternatives
										and l_divtype /~ quarterly_with_alternatives_installment)
				end
				if is_treating_non_halfyearly_dividends_as_special and l_divtype /= Void then
					Result := Result or (l_divtype /~ half_yearly
										and l_divtype /~ half_yearly_installment
										and l_divtype /~ half_yearly_with_alternatives)
				end
			end
		end

feature -- Status

	state: TUPLE [special, interim, halfyearly, quarterly : BOOLEAN] is
			-- Current state
		do
			Result := [is_ignoring_special_dividends, is_treating_interim_dividends_as_special, is_treating_non_halfyearly_dividends_as_special, is_treating_non_quarterly_dividends_as_special]
		ensure
			non_void: Result /= Void
			correct_special: Result.special = is_ignoring_special_dividends
			correct_interim: Result.interim = is_treating_interim_dividends_as_special
			correct_halfyearly: Result.halfyearly = is_treating_non_halfyearly_dividends_as_special
			correct_quarterly: Result.quarterly = is_treating_non_quarterly_dividends_as_special
		end

	is_ignoring_special_dividends: BOOLEAN is
			-- Are special dividends ignored?
		do
			Result := ignore_special.item
		end

	is_treating_interim_dividends_as_special: BOOLEAN is
			-- Are interim dividends treated as special?
		do
			Result := interim_as_special.item
		end

	is_treating_non_quarterly_dividends_as_special: BOOLEAN is
			-- Are non quarterly dividends treated as special?
		do
			Result := non_quarterly_as_special.item
		end

	is_treating_non_halfyearly_dividends_as_special: BOOLEAN is
			-- Are non half-yearly dividends treated as special?
		do
			Result := non_halfyearly_as_special.item
		end

feature -- Change

	restore_state (a_state: like state) is
			-- Restore state from `a_state'
		require
			a_state_not_void: a_state /= Void
			correct_count: a_state.count = 4
		do
			ignore_special.put (a_state.special)
			interim_as_special.put (a_state.interim)
			non_quarterly_as_special.put (a_state.quarterly)
			non_halfyearly_as_special.put (a_state.halfyearly)
		ensure
			special_set: a_state.special = is_ignoring_special_dividends
			interim_set: a_state.interim = is_treating_interim_dividends_as_special
			halfyearly_set: a_state.halfyearly = is_treating_non_halfyearly_dividends_as_special
			quarterly_set: a_state.quarterly = is_treating_non_quarterly_dividends_as_special
		end

	enable_ignore_special_dividends is
			-- Ignore special dividends
		do
			ignore_special.put (True)
		ensure
			definition: is_ignoring_special_dividends
		end

	disable_ignore_special_dividends is
			-- Do not ignore special dividends
		do
			ignore_special.put (False)
		ensure
			definition: not is_ignoring_special_dividends
		end

	enable_treat_interim_dividends_as_special is
			-- Treat interim dividends as special
		do
			interim_as_special.put (True)
		ensure
			definition: is_treating_interim_dividends_as_special
		end

	disable_treat_interim_dividends_as_special is
			-- Do not treat interim dividends as special
		do
			interim_as_special.put (False)
		ensure
			definition: not is_treating_interim_dividends_as_special
		end

	enable_treat_non_quarterly_dividends_as_special is
			-- Treat non quarterly dividends as special
		do
			non_quarterly_as_special.put (True)
		ensure
			definition: is_treating_non_quarterly_dividends_as_special
		end

	disable_treat_non_quarterly_dividends_as_special is
			-- Do not treat non quarterly dividends as special
		do
			non_quarterly_as_special.put (False)
		ensure
			definition: not is_treating_non_quarterly_dividends_as_special
		end

	enable_treat_non_halfyearly_dividends_as_special is
			-- Treat non half-yearly dividends as special
		do
			non_halfyearly_as_special.put (True)
		ensure
			definition: is_treating_non_halfyearly_dividends_as_special
		end

	disable_treat_non_halfyearly_dividends_as_special is
			-- Do not treat non half-yearly dividends as special
		do
			non_halfyearly_as_special.put (False)
		ensure
			definition: not is_treating_non_halfyearly_dividends_as_special
		end

feature {NONE} -- Implementation

	ignore_special: CELL [BOOLEAN] is
			-- Boolean singleton
		once ("PROCESS")
			create Result.put (False)
		ensure
			not_void: Result /= Void
		end

	interim_as_special: CELL [BOOLEAN] is
			-- Boolean singleton
		once ("PROCESS")
			create Result.put (False)
		ensure
			not_void: Result /= Void
		end

	non_quarterly_as_special: CELL [BOOLEAN] is
			-- Boolean singleton
		once ("PROCESS")
			create Result.put (False)
		ensure
			not_void: Result /= Void
		end

	non_halfyearly_as_special: CELL [BOOLEAN] is
			-- Boolean singleton
		once ("PROCESS")
			create Result.put (False)
		ensure
			not_void: Result /= Void
		end

	quarterly: STRING = "03"
			-- QUARTERLY DIVIDEND

	quarterly_with_alternatives: STRING = "A3"
			-- QUARTERLY DIVIDEND CARRYING OPTIONAL ALTERNATIVES

	quarterly_installment: STRING = "B4"
			-- QUARTERLY DIVIDEND PAID IN INSTALLMENTS

	quarterly_with_alternatives_installment: STRING = "B5"
			-- QUARTERLY DIV PAID IN INSTALLMENTS WITH OPTIONAL ALTER.

	half_yearly: STRING = "04"
			-- HALF-YEARLY PAYMENT

	half_yearly_with_alternatives: STRING = "B2"
			-- HALF-YEARLY DIVIDEND CARRYING OPTIONAL ALTERNATIVES

	half_yearly_installment: STRING = "B7"
			-- HALF-YEARLY PAYMENT MADE IN INSTALLMENTS

	dataline_regular_divtypes: DS_HASH_SET [STRING] is
			-- Collection of DATALINE regular dividend types
		once
			create Result.make_equal (25)
			Result.force_new ("01") --	INTERIM DIVIDEND
			Result.force_new ("02") --	FINAL DIVIDEND
			Result.force_new ("03") --	QUARTERLY DIVIDEND
			Result.force_new ("04") --	HALF-YEARLY PAYMENT
			Result.force_new ("05") --	YEARLY PAYMENT
			Result.force_new ("14") --	FIRST INTEREST PAYMENT ON NEW FIXED-INTEREST SECURITY
			Result.force_new ("15") --	FINAL INTEREST PAYMENT
			Result.force_new ("17") --	FINAL DIVIDEND PAID IN INSTALLMENTS
			Result.force_new ("18") --	INTERIM DIVIDEND PAID IN INSTALLMENTS
			Result.force_new ("A1") --	INTERIM DIVIDEND CARRYING OPTIONAL ALTERNATIVES
			Result.force_new ("A2") --	FINAL DIVIDEND CARRYING OPTIONAL ALTERNATIVES
			Result.force_new ("A3") --	QUARTERLY DIVIDEND CARRYING OPTIONAL ALTERNATIVES
			Result.force_new ("A4") --	YEARLY DIVIDEND CARRYING OPTIONAL ALTERNATIVES
			Result.force_new ("A5") --	INTERIM DIV PAID IN INSTALL CARRYING OPTL ALTERNATIVES
			Result.force_new ("A6") --	FINAL DIV PAID IN INSTALL CARRYING OPTL ALTERNATIVES
			Result.force_new ("A9") --	YEARLY DIVIDEND PAID IN INSTALLMENTS
			Result.force_new ("B1") --	YEARLY DIV PAID IN INSTALLMENTS WITH OPTL ALTERNATIVE
			Result.force_new ("B2") --	HALF-YEARLY DIVIDEND CARRYING OPTIONAL ALTERNATIVES
			Result.force_new ("B4") --	QUARTERLY DIVIDEND PAID IN INSTALLMENTS
			Result.force_new ("B5") --	QUARTERLY DIV PAID IN INSTALLMENTS WITH OPTIONAL ALTER.
			Result.force_new ("B6") --	MONTHLY DIVIDEND
			Result.force_new ("B7") --	HALF-YEARLY PAYMENT MADE IN INSTALLMENTS
			Result.force_new ("C6") --	MONTHLY DIVIDEND CARRYING OPTIONAL ALTERNATIVES
			Result.force_new ("C7") --	MONTHLY DIVIDNED PAID IN INSTALLMENTS
			Result.force_new ("C8") --	MONTHLY DIVIDEND PAID IN INSTALLMENTS WITH OPTIONAL ALT.    		
		ensure
			not_void: Result /= Void
			no_void_item: not Result.has_void
		end



end
