note

	description:
	"[
		CAPACT_SIGNATURE_RECORD and CAPACT_OPTION_SIGNATURE_RECORD common fields
	]"
	author: "Ivan Tarnavsky"
	last_editor: "$Author: it $"
	date: "$Date: 2012/10/23 $"
	revision: "$Revision: #1 $"
	path: "$File: //rose/source/kernel/capital_actions/capact_common_signature_fields.e $"

deferred class CAPACT_COMMON_SIGNATURE_FIELDS

inherit

	ANY
		undefine
			default_create,
			is_equal
		end

feature -- Access

	zone: CHARACTER assign set_zone
			-- Zone mnemonic

	listing_id: INTEGER assign set_listing_id
			-- Listing entity id (handle)

	instrument_id: INTEGER assign set_instrument_id
			-- Security entity id (handle)

	exdate: INTEGER assign set_exdate
			-- First date following dividend declaration on which
			-- stock buyer is not entitled to receive next dividend payment

	market_id: INTEGER assign set_market_id
			-- Exchange entity id (handle)

	status_code: STRING assign set_status_code
			-- Record status indicating invalid, cancelled, etc.

	source: STRING assign set_source
			-- Vendor identifier

	entity_id: INTEGER assign set_entity_id
			-- Entity id (handle). Not used but required for STID records

feature -- Status report

	has_listing_id (a_value: like listing_id): BOOLEAN is
			-- Is `listing_id' equal to `a_value'?
		do
			Result := listing_id = a_value
		ensure
			definition: Result = (listing_id = a_value)
		end

	has_instrument_id (a_value: like instrument_id): BOOLEAN is
			-- Is `instrument_id' equal to `a_value'?
		do
			Result := instrument_id = a_value
		ensure
			definition: Result = (instrument_id = a_value)
		end

	is_exdate_equal (a_exdate: INTEGER): BOOLEAN is
			-- Is `exdate' equal to `a_exdate'?
		do
			Result := exdate = a_exdate
		ensure
			definition: Result = (exdate = a_exdate)
		end

	is_exdate_greater_than (a_exdate: INTEGER): BOOLEAN is
			-- Is `exdate' greater than `a_exdate'?
		do
			Result := exdate > a_exdate
		ensure
			definition: Result = (exdate > a_exdate)
		end

	is_exdate_between_open_closed (a_start_exdate, a_end_exdate: INTEGER): BOOLEAN is
			-- Is `exdate' > `a_start_exdate' and <= `a_end_exdate'?
		do
			Result := exdate > a_start_exdate and exdate <= a_end_exdate
		ensure
			definition: Result = (exdate > a_start_exdate and exdate <= a_end_exdate)
		end

	is_exdate_between_closed_open (a_start_exdate, a_end_exdate: INTEGER): BOOLEAN is
			-- Is `exdate' >= `a_start_exdate' and < `a_end_exdate'?
		do
			Result := exdate >= a_start_exdate and exdate < a_end_exdate
		ensure
			definition: Result = (exdate >= a_start_exdate and exdate < a_end_exdate)
		end

feature -- Setting

	set_zone (a_value: like zone) is
			-- Set `zone' to `a_value'
		do
			zone := a_value
		ensure
			value_set: zone = a_value
		end

	set_listing_id (a_value: like listing_id) is
			-- Set `listing_id' to `a_value'
		do
			listing_id := a_value
		ensure
			value_set: listing_id = a_value
		end

	set_instrument_id (a_value: like instrument_id) is
			-- Set `instrument_id' to `a_value'
		do
			instrument_id := a_value
		ensure
			value_set: instrument_id = a_value
		end

	set_exdate (a_value: like exdate) is
			-- Set `exdate' to `a_value'
		do
			exdate := a_value
		ensure
			value_set: exdate = a_value
		end

	set_market_id (a_value: like market_id) is
			-- Set `market_id' to `a_value'
		do
			market_id := a_value
		ensure
			value_set: market_id = a_value
		end

	set_status_code (a_value: like status_code) is
			-- Set `status_code' to `a_value'
		do
			status_code := a_value
		ensure
			value_set: status_code = a_value
		end

	set_source (a_value: like source) is
			-- Set `source' to `a_value'
		do
			source := a_value
		ensure
			value_set: source = a_value
		end

	set_entity_id (a_value: like entity_id) is
			-- Set `entity_id' to `a_value'
		do
			entity_id := a_value
		ensure
			value_set: entity_id = a_value
		end

end
