indexing
	description: "Rose capital actions conversion, for bonuses"
	author: "Janet Small"
	date: "$Date: 2010/04/01 $"
	revision: "$Revision: #6 $"
	archive: "$File: //rose/source/kernel/capital_actions/capact_conversion.e $"

deferred class
	CAPACT_CONVERSION
inherit
	CAPACT_CAPITAL_ACTION
	CAPACT_CONVERSION_TEMPLATE

feature -- Access

	parent_price_saved: DOUBLE --  may be rose best price or a future estimated price
	set_parent_price_saved(a_value: like parent_price_saved) is
			--set override rose price used for calculating basis prop
		do
			parent_price_saved := a_value
		end

	parent_price_iso: STRING -- currency of override rose price.
	set_parent_price_iso(a_value: like parent_price_iso) is
			--set currency of override rose price
		do
			parent_price_iso := a_value
		end

	parent_basis_prop: DOUBLE -- for override of basis_prop if necessary, stored in item legacy_basis_prop
	set_parent_basis_prop(a_value: like parent_basis_prop) is
			--set parent basis prop for override of basis_prop if necessary, stored in item legacy_basis_prop
		do
			parent_basis_prop := a_value
		end

	dealing_date: INTEGER --  dealing date, used to calculated estimated future price
	set_dealing_date(a_value: like dealing_date) is
			--set dealing date, used to calculated estimated future price
		do
			dealing_date := a_value
		end

feature -- Basic operations

	instantiate_conversion_data is
		-- retrieve data items from storage
		local
			s: STRING
		do
			s := string_value(conversion_rate_expression,signature)
			if s /= Void and then s.is_double then
				set_conversion_rate(s.to_double)
			end
			s := string_value(new_shares_conversion_expression, signature)
			if s /= Void and then s.is_double then
				set_new_shares_conversion(s.to_double)
			end
			s := string_value(old_shares_conversion_expression,signature)
			if s /= Void and then s.is_double then
				set_old_shares_conversion(s.to_double)
			end
			set_last_trading_date(last_trading_date_expression.integer_value (signature))
			set_first_trading_date(first_trading_date_expression.integer_value (signature))
			set_issue_date(issue_date_expression.integer_value (signature))
			set_dealing_date(dealing_date_expression.integer_value (signature))
			set_last_date_to_convert(last_date_to_convert_expression.integer_value (signature))
			set_call_due_date(call_due_date_expression.integer_value (signature))

			s := string_value(parent_price_saved_expression,signature)
			if s /= Void and then s.is_double then
				set_parent_price_saved(s.to_double)
			end

			s := string_value(parent_price_iso_expression,signature)
			if s /= Void then
				set_parent_price_iso(s)
			end

			s := string_value(parent_basis_prop_expression,signature)
			if s /= Void and then s.is_double then
				set_parent_basis_prop(s.to_double)
			end


		end

	set_conversion_data_items is
			-- put data items relevant to conversions into item hash tables
		local
			l_variable: CAPACT_VARIABLE_DESCRIPTION_ROW
			s: STRING
		do
			if conversion_rate > 0 then
				l_variable := variable_table.item("RIGHTS_CONVERSION_RATE")
				s := format_rate.formatted (conversion_rate)
				string_items.put(s, l_variable.item_id)
			end
			if new_shares_conversion > 0 then
				l_variable := variable_table.item("NEW_SHARES_CONVERSION")
				s := format_rate.formatted (new_shares_conversion)
				string_items.put(s, l_variable.item_id)
			end
			if conversion_rate > 0 then
				l_variable := variable_table.item("OLD_SHARES_CONVERSION")
				s := format_rate.formatted (old_shares_conversion)
				string_items.put(s, l_variable.item_id)
			end
			l_variable := variable_table.item("LAST_TRADING_DATE")
			integer_items.put(last_trading_date, l_variable.item_id)
			l_variable := variable_table.item("FIRST_TRADING_DATE")
			integer_items.put(first_trading_date, l_variable.item_id)
			l_variable := variable_table.item("ISSUE_DATE")
			integer_items.put(issue_date, l_variable.item_id)
			l_variable := variable_table.item("DEALING_DATE")
			integer_items.put(dealing_date, l_variable.item_id)
			l_variable := variable_table.item("LAST_DATE_TO_CONVERT")
			integer_items.put(last_date_to_convert, l_variable.item_id)
			l_variable := variable_table.item("CALL_DUE_DATE")
			integer_items.put(call_due_date, l_variable.item_id)

			l_variable := variable_table.item("PARENT_PRICE")
			s := format_price.formatted (parent_price_saved)
			string_items.put(s, l_variable.item_id)

			if parent_price_iso /= Void then
				l_variable := variable_table.item("PARENT_PRICE_ISO")
				string_items.put(parent_price_iso, l_variable.item_id)
			end

			if parent_basis_prop > 0 then
				l_variable := variable_table.item("LEGACY_BASIS_PROP")
				string_items.put(parent_basis_prop.out, l_variable.item_id)
			end

		end

feature  -- Implementation

	conversion_rate_expression:ROSE_EXPRESSION [ROSE_ENTITY_DATE [ENTITY]] is
			--rights conversion rate
		once
			Result := ur_target.expression_from_name("RIGHTS_CONVERSION_RATE")
		end
	new_shares_conversion_expression:ROSE_EXPRESSION [ROSE_ENTITY_DATE [ENTITY]] is
			-- new conversion shares expression
		once
			Result := ur_target.expression_from_name("NEW_SHARES_CONVERSION")
		end
	old_shares_conversion_expression:ROSE_EXPRESSION [ROSE_ENTITY_DATE [ENTITY]] is
			-- old conversion shares expression
		once
			Result := ur_target.expression_from_name("OLD_SHARES_CONVERSION")
		end
	last_trading_date_expression:ROSE_EXPRESSION [ROSE_ENTITY_DATE [ENTITY]] is
			-- last date to trade expression
		once
			Result := ur_target.expression_from_name("LAST_TRADING_DATE")
		end
	first_trading_date_expression:ROSE_EXPRESSION [ROSE_ENTITY_DATE [ENTITY]] is
			-- first date to trade expression
		once
			Result := ur_target.expression_from_name("FIRST_TRADING_DATE")
		end
	issue_date_expression:ROSE_EXPRESSION [ROSE_ENTITY_DATE [ENTITY]] is
			-- issue date expression
		once
			Result := ur_target.expression_from_name("ISSUE_DATE")
		end

	dealing_date_expression:ROSE_EXPRESSION [ROSE_ENTITY_DATE [ENTITY]] is
			-- dealing date expression
		once
			Result := ur_target.expression_from_name("DEALING_DATE")
		end

	last_date_to_convert_expression:ROSE_EXPRESSION [ROSE_ENTITY_DATE [ENTITY]] is
			-- last date to convert expression
		once
			Result := ur_target.expression_from_name("LAST_DATE_TO_CONVERT")
		end
	call_due_date_expression:ROSE_EXPRESSION [ROSE_ENTITY_DATE [ENTITY]] is
			-- call due date expression
		once
			Result := ur_target.expression_from_name("CALL_DUE_DATE")
		end

	parent_price_saved_expression:ROSE_EXPRESSION [ROSE_ENTITY_DATE [ENTITY]] is
			--expression for retrieving parent price (save appended to name to avoid conflicts)
		once
			Result := ur_target.expression_from_name("PARENT_PRICE")
		end

	parent_price_iso_expression:ROSE_EXPRESSION [ROSE_ENTITY_DATE [ENTITY]] is
			-- expression for retrieving iso code of currency of parent price
		once
			Result := ur_target.expression_from_name("PARENT_PRICE_ISO")
		end

	parent_basis_prop_expression:ROSE_EXPRESSION [ROSE_ENTITY_DATE [ENTITY]] is
			-- expression for retrieving iso code of currency of parent price
		once
			Result := ur_target.expression_from_name("LEGACY_BASIS_PROP")
		end


end
