note

	description:

		"capital action Expression in which outer expression is applied only to the change in shares from the inner expression"

	author: "Janet Small"
	last_editor: "$Author: it $"
	date: "$Date: 2012/05/22 $"
	revision: "$Revision: #9 $"
	path: "$File: //rose/source/kernel/capital_actions/expressions/capact_delta_expression.e $"

class

	CAPACT_DELTA_EXPRESSION
inherit
	CAPACT_EXPRESSION
	CAPACT_COMMON
			undefine default_create end
create
	make_with_expressions

feature {NONE} -- creation

	make_with_expressions(a_inner,a_outer:CAPACT_EXPRESSION) is
			-- initialize component expressions and compute rmdat_adjustment_factor
		require
			has_component_expressions: a_inner /= Void and a_outer /= Void
		do
			set_expression_index
			inner_expression := a_inner
			outer_expression := a_outer
			if inner_expression.rmdat_adjustment_factor_available and outer_expression.rmdat_adjustment_factor_available then
				if inner_expression.delta_new_shares = 0 then
					rmdat_adjustment_factor := (outer_expression.rmdat_adjustment_factor * inner_expression.rmdat_adjustment_factor)
				else
					rmdat_adjustment_factor := (outer_expression.rmdat_adjustment_factor * inner_expression.delta_new_shares) +1
				end
			else
				rmdat_adjustment_factor := undefined_double
			end
		ensure
			has_expressions : inner_expression /= Void and outer_expression /= Void
		end

feature -- Access

	processing_instruction (
		a_basis: RP_CORPORATE_ACTION_PROCESSING_SHARE_BASIS_TYPE;
		a_corporate_action_processing_system: RP_CORPORATE_ACTION_PROCESSING_SYSTEM): RP_CORPORATE_ACTION_PROCESSING_INSTRUCTIONS is
			-- processing instructions for the corp action - no processing
		do
		end

	inner_expression:CAPACT_EXPRESSION -- first expression to be evaluated

	outer_expression :CAPACT_EXPRESSION -- following expression to be applied only to additional shares from first expression

	name : STRING is
			-- description of expression
		do
			Result := outer_expression.name + " following on new from " + 	inner_expression.name
		end

	cash_outflow(a_currency:CURRENCY):DOUBLE is
			--behaves like a compound expression of new shares of inner = 0
		do
			if is_defined_double (outer_expression.cash_outflow (a_currency)) and
			   is_defined_double (inner_expression.cash_outflow(a_currency)) and
			   is_defined_double (inner_expression.delta_new_shares) then
			   	if inner_expression.delta_new_shares = 0 then
			   		Result := outer_expression.cash_outflow(a_currency)*inner_expression.rmdat_adjustment_factor + inner_expression.cash_outflow(a_currency)
			   	else
					Result := outer_expression.cash_outflow(a_currency)*inner_expression.delta_new_shares + inner_expression.cash_outflow(a_currency)
				end
			end
		end

	cash_inflow(a_currency:CURRENCY):DOUBLE is
			--behaves like a compound expression of new shares of inner = 0
		do
			if is_defined_double (outer_expression.cash_inflow (a_currency)) and
			   is_defined_double (inner_expression.cash_inflow(a_currency)) and
			   is_defined_double (inner_expression.delta_new_shares) then
			   	if inner_expression.delta_new_shares = 0 then
			   		Result := outer_expression.cash_inflow(a_currency)*inner_expression.rmdat_adjustment_factor + inner_expression.cash_inflow(a_currency)
			   	else
					Result := outer_expression.cash_inflow(a_currency)*inner_expression.delta_new_shares + inner_expression.cash_inflow(a_currency)
				end
			end
		end

	cash_dividend(a_currency:CURRENCY):DOUBLE is
			--behaves like a compound expression of new shares of inner = 0
		do
			if is_defined_double (outer_expression.cash_inflow (a_currency)) and
			   is_defined_double (inner_expression.cash_inflow(a_currency)) and
			   is_defined_double (inner_expression.delta_new_shares) then
			   	if inner_expression.delta_new_shares = 0 then
			   		Result := outer_expression.cash_inflow(a_currency)*inner_expression.rmdat_adjustment_factor + inner_expression.cash_inflow(a_currency)
			   	else
					Result := outer_expression.cash_inflow(a_currency)*inner_expression.delta_new_shares + inner_expression.cash_inflow(a_currency)
				end
			end
		end

feature--capact_capital_action

	instantiate_data is
		-- not implemented, deferred from capact_capital_action
		do
		end

	template_name:STRING  is "DELTA EXPRESSION"

	set_data_items is
		-- not implemented, deferred from capact_capital_action
		do
		end

	update_data_items is
		-- not implemented, deferred from capact_capital_action
		do
		end

end -- class CAPACT_DELTA_EXPRESSION
