note

	description:

		"Compute adjustments, returns, cash_flows, etc. from rose capital actions"

	author: "Srilatha Singh/JS"
	last_editor: "$Author: it $"
	date: "$Date: 2014/03/14 $"
	revision: "$Revision: #38 $"
	path: "$File: //rose/source/kernel/capital_actions/expressions/capact_expression.e $"

deferred class CAPACT_EXPRESSION

inherit

	RX_EXPRESSION
		undefine
			default_create
		end

	ROSE_EXPRESSION [CAPACT_SIGNATURE_RECORD]
		rename
			name as expression_name,
			string_value as rose_expression_string_value
		redefine
			infix "+",
			date_value
		end

	CAPACT_CAPITAL_ACTION
		rename
			signature as target,
			exdate as integer_exdate,
			name as template_name
		undefine
			default_create
		end

	CAPACT_CASH_DIVIDEND_VALUE

	ROSE_CAPACT_DATE_USE
		undefine
			default_create
		end

feature {NONE} -- Initialization

	make_empty is
			-- initialize to empty
		do
			set_expression_index
			is_new := True
			create target
			target.set_status_code ("")
			target.set_source ("")
			target.set_vendor_key ("")
		end

feature -- Access

	processing_instruction (a_basis: RP_CORPORATE_ACTION_PROCESSING_SHARE_BASIS_TYPE; a_processing_system: RP_CORPORATE_ACTION_PROCESSING_SYSTEM): RP_CORPORATE_ACTION_PROCESSING_INSTRUCTIONS is
			-- processing instructions for the corp action
		deferred
		end

	name: STRING is
			-- names defined in specific descendents
		deferred
		end

	primary_capact_option: ROSE_COMPLEX_CAPITAL_ACTION
			--parent of which Current is a member of capact_option.capact_expressions

	source: STRING is
			-- Supplier of data
		do
			Result := target.source
		end

	exdate: ROSE_DATE is
			-- from target
		do
			if save_exdate = Void then
				if target /= Void then
					save_exdate := safe_date_from_ccyymmdd (target.exdate)
				end
			end
			Result := save_exdate
		end

	set_exdate (a_date: ROSE_DATE) is
			-- set exdate
		do
			save_exdate := a_date
		ensure
			save_exdate_is_equal_to_a_date: save_exdate.is_equal (a_date)
		end


	record_date: ROSE_DATE is
			-- From target
		do
			if saved_record_date = Void then
				if target /= Void then
					saved_record_date := safe_date_from_ccyymmdd (target.record_date)
				end
			end
			Result := saved_record_date
		end

	terminal_distribution: ENTITY_VALUES [LISTING] is
			-- Shares given in exchange for current security.
			-- (void if none or not a terminal distribution)
		do
		end

	rmdat_adjustment_factor: DOUBLE
			-- adjustment factor used for adjustment in rmdat

	set_rmdat_adjustment_factor (a_value: like rmdat_adjustment_factor) is
			-- Set value of `rmdat_adjustment_factor' to `a_value'
		do
			rmdat_adjustment_factor := a_value
		ensure
			rmdat_adjustment_factor_set: rmdat_adjustment_factor = a_value
		end

	delta_new_shares: DOUBLE
			-- new shares to be used for evaluating delta expressions

	round_lot_shares (a_shares: DOUBLE; a_lotsize: INTEGER): INTEGER is
			-- convert a_shares to multiply of a_lotsize
		do
			Result := (a_shares.floor // a_lotsize) * a_lotsize
		end

feature -- Status Report

	is_terminal_distribution: BOOLEAN
			-- indicate whether expression is terminal distribution

	set_is_terminal_distribution (a_value: like is_terminal_distribution) is
			-- Set value of `is_terminal_distribution' to `a_value'
		do
			is_terminal_distribution := a_value
		ensure
			is_terminal_distribution_set: is_terminal_distribution = a_value
		end

	is_override: BOOLEAN is
			-- Is this expression the result of an override?
		local
			l_source: STRING
		do
			Result := false
			if source /= Void then
				l_source := source.twin
				l_source.prune_all (' ')
				Result := l_source.is_case_insensitive_equal ("Override")
			end
		end

feature -- Basic Operations

	infix "@" (a_expression: CAPACT_EXPRESSION): CAPACT_COMPOUND_EXPRESSION is
			-- outer expression applied to shares resulting after inner expression
		do
			create Result.make_with_expressions (a_expression, Current)
		end

	infix "+" (a_expression: CAPACT_EXPRESSION): CAPACT_ADDITIVE_EXPRESSION is
			-- right and left expressions both applied to incoming shares
		do
			create Result.make_with_expressions (a_expression, Current)
		end

	infix "&" (a_expression: CAPACT_EXPRESSION): CAPACT_DELTA_EXPRESSION is
			-- outer expression applied only to new shares from inner expression
		do
			create Result.make_with_expressions (a_expression, Current)
		end

	set_primary_capact_option (a_option: ROSE_COMPLEX_CAPITAL_ACTION) is
			-- set the primary capital action
		do
			primary_capact_option := a_option
		end

	set_target (a_value: like target) is
			-- set target to a_value
		do
			target := a_value
		end

	rmdat_adjustment_factor_available: BOOLEAN is
			-- rmdat_adjustment_factor /= -999.0
		do
			Result := is_defined_double (rmdat_adjustment_factor)
		end

	cash_outflow (a_currency: CURRENCY): DOUBLE is
			-- cash outflow computed from capital action expressed in specified currency
		deferred
		end

	cash_inflow (a_currency: CURRENCY): DOUBLE is
			-- cash inflow computed from capital action expressed in specified currency
		deferred
		end

feature -- inherited from rose_expression

	currency (a_target: like expression_target): CURRENCY is
			-- currency
		do
		end

	get_real_value (a_target: like expression_target): REAL is
			-- Real value
		do
			Result := rmdat_adjustment_factor.truncated_to_real
		end

	get_double_value (a_target: like expression_target): DOUBLE is
			-- Double value
		do
			Result := rmdat_adjustment_factor
		end

	get_integer_value (a_target: like expression_target): INTEGER is
			-- integer value
		do
		end

	character_value (a_target: like expression_target): CHARACTER is
			-- Character value, used?
		do
		end

	date_value (a_target: like expression_target): ROSE_DATE is
			--DATE value
		do
			Result := exdate
		end


feature -- status

	is_new: BOOLEAN
			-- Is new record?

feature {NONE} -- Implementation

	save_exdate: ROSE_DATE
			-- Exdate

	saved_record_date: ROSE_DATE
			-- Record date

	legacy_undefined: ROSE_LEGACY_UNDEFINED is
			-- Legacy constants
		once
			create Result
		end

end
