indexing
	description: "Integration operations for rose capital actions"
	author: ""
	date: "$Date: 2008/12/01 $"
	revision: "$Revision: #8 $"
	archive: "$File: //rose/source/kernel/capital_actions/capact_integrate_one.e $"

class
	CAPACT_INTEGRATE_ONE
inherit
	CAPITAL_ACTIONS_CONTEXT_SHARED


feature -- Basic operations


	populate_rose_best(a_aware:CAPACT_OVERRIDE_AWARE) is
			-- populate rose capital action from override updates
		local
			l_option_override_sig: ROSE_CAPACT_OPTION_OVERRIDE_SIGNATURE
			l_override_capact_recs: ROSE_LINEAR_ARRAY[ROSE_VIRTUAL_RECORD[ROSE_CAPACT_OVERRIDE_SIGNATURE]]
			l_override_capact_rec: ROSE_VIRTUAL_RECORD[ROSE_CAPACT_OVERRIDE_SIGNATURE]
			l_rose_capact_sig: CAPACT_SIGNATURE_RECORD
			l_rose_option_sig: CAPACT_OPTION_SIGNATURE_RECORD
			l_rose_best_capacts: ROSE_LINEAR_ARRAY[ROSE_VIRTUAL_RECORD[CAPACT_SIGNATURE_RECORD]]
			l_update_capact_rec: ROSE_VIRTUAL_RECORD[CAPACT_SIGNATURE_RECORD]
			l_count: INTEGER
			l_rose_best_options:ROSE_LINEAR_ARRAY[CAPACT_OPTION_SIGNATURE_RECORD]
			l_all_rose_best_capacts: ROSE_LINEAR_ARRAY[ROSE_VIRTUAL_RECORD[CAPACT_SIGNATURE_RECORD]]
		do
			create l_rose_best_options.make_empty
			create l_all_rose_best_capacts.make_empty
			from a_aware.override_options.start
			until a_aware.override_options.off
			loop
				l_option_override_sig := a_aware.override_options.item_for_iteration
				--initialise the rose_best_capacts to the ones in the database, if any
				l_rose_best_capacts := rose_capacts_from_option_override(l_option_override_sig)
				if l_rose_best_capacts /= Void and then l_rose_best_capacts.count > 0 then
					from l_rose_best_capacts.start
					until l_rose_best_capacts.off
					loop
						l_rose_best_capacts.replace (l_rose_best_capacts.current_item.terminating_record)
--						l_rose_best_capacts.current_item.signature.set_status_code("C")						
						l_rose_best_capacts.forth
					end
				else
					-- nothing to cancel, make new virtual record, all override records will be appended
					create l_rose_best_capacts.make_empty
				end

				---get the corresponding option signature for rose
				create l_rose_option_sig
				l_rose_option_sig.set_from_override(l_option_override_sig)
				l_rose_best_options.extend(l_rose_option_sig)

				-- get all the override capital actions associated with this option
				l_override_capact_recs := a_aware.override_capital_actions(l_option_override_sig.key_tuple)
				l_override_capact_recs.sort_by(agent sort_overrides_by_member_number)

				--- copy the override values into the initialized records

				from l_override_capact_recs.start; l_rose_best_capacts.start; l_count := l_rose_best_capacts.count
				until l_override_capact_recs.off
				loop
					l_override_capact_rec := l_override_capact_recs.current_item
					--match the records on member number
					if l_rose_best_capacts.index <= l_count and (l_override_capact_rec.signature.member_number=l_rose_best_capacts.current_item.signature.member_number) then
						--update the terminated existing record
						l_update_capact_rec := l_rose_best_capacts.current_item
						create l_rose_capact_sig
						l_rose_capact_sig.set_from_override(l_override_capact_rec.signature)
						l_update_capact_rec.set_signature(l_rose_capact_sig)
						copy_string_items(l_update_capact_rec.string_items,l_override_capact_rec.string_items)
						copy_integer_items(l_update_capact_rec.integer_items,l_override_capact_rec.integer_items)
						l_rose_best_capacts.extend(l_update_capact_rec)
					elseif	l_rose_best_capacts.index <= l_count and then (l_override_capact_rec.signature.member_number /= l_rose_best_capacts.current_item.signature.member_number) then
						exceptions.raise("Member number Mismatch: This should never happen given our override scheme")
					else
						--make new virtual record, append new capact to end of updates	
						create l_rose_capact_sig
						create l_update_capact_rec
						l_rose_capact_sig.set_from_override(l_override_capact_rec.signature)
						l_update_capact_rec.set_signature(l_rose_capact_sig)
						copy_string_items(l_update_capact_rec.string_items,l_override_capact_rec.string_items)
						copy_integer_items(l_update_capact_rec.integer_items,l_override_capact_rec.integer_items)
						l_rose_best_capacts.extend(l_update_capact_rec)
					end

					--move forward both the arrays. the override capacts and the rose best capacts
					l_override_capact_recs.forth
					if not l_rose_best_capacts.off then
						l_rose_best_capacts.forth
					end

				end
				l_all_rose_best_capacts.append_array(l_rose_best_capacts)
				a_aware.override_options.forth
			end

			extend_capact_override_aware (a_aware, l_all_rose_best_capacts, l_rose_best_options)
		end


	copy_string_items(a_to, a_from: HASH_TABLE[STRING,INTEGER]) is
			-- copy string items for virtual records
		require
			has_to : a_to /= Void
			has_from : a_from /= Void
		local
			s: STRING
			i: INTEGER
		do
			from a_from.start
			until a_from.off
			loop
				s := a_from.item_for_iteration -- debug
				i := a_from.key_for_iteration -- debug
				a_to.force (a_from.item_for_iteration,a_from.key_for_iteration)
				a_from.forth
			end

		end

	copy_integer_items(a_to, a_from: HASH_TABLE[INTEGER,INTEGER]) is
			-- copy integer items for virtual records
		require
			has_to : a_to /= Void
			has_from : a_from /= Void
		do
			from a_from.start
			until a_from.off
			loop
				a_to.force (a_from.item_for_iteration,a_from.key_for_iteration)
				a_from.forth
			end
		end


	rose_capacts_from_option_override (a_capact_option_override: ROSE_CAPACT_OPTION_OVERRIDE_SIGNATURE): ROSE_LINEAR_ARRAY[ROSE_VIRTUAL_RECORD[CAPACT_SIGNATURE_RECORD]] is
			-- retrieve rose capital actions matching option override
		local
			l_target: CAPACT_SIGNATURE_RECORD
			l_option_target:CAPACT_OPTION_SIGNATURE_RECORD
		do
			create l_option_target
			l_option_target.set_listing_id (a_capact_option_override.listing_id)
			l_option_target.set_instrument_id (a_capact_option_override.instrument_id)
			l_option_target.set_market_id (a_capact_option_override.market_id)

			create l_target
			l_target.set_listing_id (l_option_target.listing.handle)
			l_target.set_instrument_id(a_capact_option_override.instrument_id)
			l_target.set_market_id(a_capact_option_override.market_id)

			l_target.set_exdate(a_capact_option_override.exdate)
			Result := l_target.data_manager.retrieve_signature (l_target,l_target.option_restriction_statement,Void)
			Result.sort_by (agent sort_by_member_number)
		end

	extend_capact_override_aware (a_aware:CAPACT_OVERRIDE_AWARE ;
		a_capacts: ROSE_LINEAR_ARRAY [ROSE_VIRTUAL_RECORD[CAPACT_SIGNATURE_RECORD]];
		a_options: ROSE_LINEAR_ARRAY[CAPACT_OPTION_SIGNATURE_RECORD]) is
				--
		do
			a_aware.rose_best_capital_actions.wipe_out
			from a_capacts.start
			until a_capacts.after
			loop
				a_aware.rose_best_capital_actions.put (a_capacts.current_item, a_capacts.current_item.signature.key_tuple)
				a_capacts.forth
			end
			a_aware.rose_best_options.wipe_out
			from a_options.start
			until a_options.after
			loop
				a_aware.rose_best_options.put (a_options.current_item, a_options.current_item.key_tuple)
				a_options.forth
			end
		end

	sort_by_member_number(a,b:ROSE_VIRTUAL_RECORD[CAPACT_SIGNATURE_RECORD]):BOOLEAN	is
		do
			if a.signature.member_number < b.signature.member_number then
				Result := True
			end
		end

	sort_overrides_by_member_number(a,b:ROSE_VIRTUAL_RECORD[ROSE_CAPACT_OVERRIDE_SIGNATURE]):BOOLEAN	is
		do
			if a.signature.member_number < b.signature.member_number then
				Result := True
			end
		end

feature {NONE} -- Implementation

	exceptions:EXCEPTIONS is
		once
			create Result
		end

end
