note
	description: "Rose capital action generic distribution (legacy)"
	author: "Janet Small"
	last_editor: "$Author: it $"
	date: "$Date: 2014/03/14 $"
	revision: "$Revision: #30 $"
	path: "$File: //rose/source/kernel/capital_actions/capact_legacy_distribution.e $"

class CAPACT_LEGACY_DISTRIBUTION

inherit

	CAPACT_CAPITAL_ACTION
		rename
			exdate as capact_exdate
		redefine
			instantiate_vendor_specific_data
		end

	CAPACT_LEGACY_DISTRIBUTION_TEMPLATE

	ROSE_CAPACT_DATE_USE

feature --Modifiers

	set_data_items is
			--set data items
		local
			l_variable: CAPACT_VARIABLE_DESCRIPTION_ROW
			s: STRING
			l_exdate: ROSE_DATE
		do
			l_exdate := safe_date_from_ccyymmdd (signature.exdate)
				-- child instrument, listing or currency
			if child_instrument /= Void then
				l_variable := variable_table.item ("CHILD_INSTRUMENT")
				integer_items.put (child_instrument.handle, l_variable.item_id)
			end
			if child_cash /= Void then
				l_variable := variable_table.item ("CHILD_CASH")
				s := child_cash.iso_code_key (l_exdate)
				string_items.put (s, l_variable.item_id)
			end
			l_variable := variable_table.item ("CHILD_UNIT_FACTOR")
			s := format_share_factor.formatted (child_unit_factor)
			string_items.put (s, l_variable.item_id)
				-- additional contributions required
			if additional_instrument /= Void then
				l_variable := variable_table.item ("ADDITIONAL_INSTRUMENT")
				integer_items.put (additional_instrument.handle, l_variable.item_id)
			end
			if additional_cash /= Void then
				l_variable := variable_table.item ("ADDITIONAL_CASH")
				s := additional_cash.iso_code_key (l_exdate)
				string_items.put (s, l_variable.item_id)
			end
			l_variable := variable_table.item ("ADDITIONAL_UNIT_FACTOR")
			s := format_share_factor.formatted (additional_unit_factor)
			string_items.put (s, l_variable.item_id)
				-- legacy
			if legacy_basis_currency /= Void then
				l_variable := variable_table.item ("LEGACY_BASIS_CURRENCY")
				string_items.put (legacy_basis_currency, l_variable.item_id)
			end
			if legacy_basis_prop /= 0 then
				l_variable := variable_table.item ("LEGACY_BASIS_PROP")
				s := format_share_factor.formatted (legacy_basis_prop)
				string_items.put (s, l_variable.item_id)
			end
			if legacy_basis_price > 0 then
				l_variable := variable_table.item ("LEGACY_BASIS_PRICE")
				s := format_price.formatted (legacy_basis_price)
				string_items.put (s, l_variable.item_id)
			end
			if legacy_order > 0 then
				l_variable := variable_table.item ("LEGACY_ORDER")
				integer_items.put (legacy_order, l_variable.item_id)
			end
			if legacy_user_id /= Void then
				l_variable := variable_table.item ("LEGACY_USER_ID")
				string_items.put (legacy_user_id, l_variable.item_id)
			end
			if legacy_comment /= Void then
				l_variable := variable_table.item ("LEGACY_COMMENT")
				string_items.put (legacy_comment, l_variable.item_id)
			end
		end

	instantiate_data is
			-- retrieve data items from storage
		local
			l_instrument: LISTING
			l_iso: STRING
			l_string: STRING
		do
			child_instrument_id := child_instrument_expression.integer_value (signature)
			parent_instrument_id := parent_instrument_expression.integer_value (signature)
			l_string := string_value (child_unit_factor_expression, signature)
			if l_string /= Void and then l_string.is_double then
				child_units := l_string.to_double
			end
--			if signature.vendor_key /= Void and then signature.vendor_key.item(1).out.is_integer then
--				legacy_order:= signature.vendor_key.item(1).out.to_integer
--			end
			additional_instrument_id := additional_instrument_expression.integer_value (signature)
			if additional_instrument_id > 0 then
				additional_instrument := rose_context.listing_accessor.entity (additional_instrument_id)
			end
				--legacy_comment:= legacy_comment_expression.string_value(signature) -- done below
			if parent_instrument_id > 0 and parent_instrument = Void then
				parent_instrument := rose_context.listing_accessor.entity (parent_instrument_id)
			end
			child_instrument := rose_context.listing_accessor.entity (child_instrument_id)
			if child_instrument_id > 0 then
				l_instrument := rose_context.listing_accessor.entity (child_instrument_id)
					--create {RP_STOCK} rp_child_instrument.make(l_instrument)
			else
				l_iso := string_value (child_cash_expression, signature)
				if l_iso /= Void then
					if l_iso.is_equal ("JP") then
						l_iso := "JPY"
					end
					child_cash := rose_context.currency_by_iso_code (l_iso, constants.now)
						--create {RP_CURRENCY} rp_child_instrument.make(child_cash)
				end
			end
			l_iso := string_value (additional_cash_expression, signature)
			if l_iso /= Void then
				additional_cash := rose_context.currency_by_iso_code (l_iso, constants.now)
					--do we need an rp_additional_cash_instrument ?
			end
			payment_date := payment_date_expression.integer_value (signature)
			l_string := string_value (additional_unit_factor_expression, signature)
			if l_string /= Void then
				additional_unit_factor := l_string.to_double
			end
			instantiate_vendor_specific_data
		end

	additional_instrument_id: INTEGER
			--rp_child_instrument : RP_INSTRUMENT

	child_instrument_id: INTEGER

	parent_instrument_id: INTEGER

	child_units: DOUBLE
		-- Child units

	instantiate_vendor_specific_data is
			--
		local
			s: STRING
			i: INTEGER
		do
			instantiate_legacy_items
				-- legacy distribution specific
			legacy_basis_currency := string_value (legacy_basis_currency_expression, signature)
			s := string_value (legacy_basis_prop_expression, signature)
			if s /= Void and then s.is_double then
				legacy_basis_prop := s.to_double
			end
			s := string_value (legacy_basis_price_expression, signature)
			if s /= Void and then s.is_double then
				legacy_basis_price := s.to_double
			end
			s := string_value (legacy_vendor_key_expression, signature)
			if s /= Void and then s.item (1).out.is_integer then
				legacy_order := s.item (1).out.to_integer
			else
				i := legacy_order_expression.integer_value (signature)
				if i > 0 then
					legacy_order := i
				end
			end
			if legacy_order = 0 then
				(create {EXCEPTIONS}).raise ("legacy order is 0 for " + signature.display_signature)
			end
			legacy_comment := string_value (legacy_comment_expression, signature)
			legacy_user_id :=string_value( legacy_user_id_expression, signature)
		end

feature -- expressions
	--children, may be listing or currency

	child_instrument_expression: ROSE_EXPRESSION [ROSE_ENTITY_DATE [ENTITY]] is
			-- Child instrucment expression
		once
			Result := ur_target.expression_from_name ("CHILD_INSTRUMENT")
		end

	child_cash_expression: ROSE_EXPRESSION [ROSE_ENTITY_DATE [ENTITY]] is
			--
		once
			Result := ur_target.expression_from_name ("CHILD_CASH")
		end

	child_unit_factor_expression: ROSE_EXPRESSION [ROSE_ENTITY_DATE [ENTITY]] is
			--
		once
			Result := ur_target.expression_from_name ("CHILD_UNIT_FACTOR")
		end

	additional_instrument_expression: ROSE_EXPRESSION [ROSE_ENTITY_DATE [ENTITY]] is
			--additional required input
			--
			--
		once
			Result := ur_target.expression_from_name ("ADDITIONAL_INSTRUMENT")
		end

	additional_cash_expression: ROSE_EXPRESSION [ROSE_ENTITY_DATE [ENTITY]] is
			-- Expression for additional cash item.
		once
			Result := ur_target.expression_from_name ("ADDITIONAL_CASH")
		end

	additional_unit_factor_expression: ROSE_EXPRESSION [ROSE_ENTITY_DATE [ENTITY]] is
			-- additional unit factor expression
		once
			Result := ur_target.expression_from_name ("ADDITIONAL_UNIT_FACTOR")
		end

end -- class CAPACT_LEGACY_DISTRIBUTION
