note

	description:

		"Legacy distribution expression"

	author: "MK"
	last_editor: "$Author: it $"
	date: "$Date: 2014/03/14 $"
	revision: "$Revision: #34 $"
	path: "$File: //rose/source/kernel/capital_actions/expressions/capact_legacy_distribution_expression.e $"

class CAPACT_LEGACY_DISTRIBUTION_EXPRESSION

inherit

	CAPACT_LEGACY_DISTRIBUTION
		rename
			signature as target,
			name as template_name,
			capact_exdate as integer_exdate
		undefine
			default_create
		end

	CAPACT_EXPRESSION
		undefine
			instantiate_vendor_specific_data
		redefine
			terminal_distribution
		end

	ATTRIBUTES
		rename
			name as attributes_name,
			mnemonic as attributes_mnemonic,
			variable_index as attributes_variable_index
		undefine
			default_create
		end

	CAPACT_UTILS
		export
			{NONE} all
		undefine
			default_create
		end

create

	make,
	instantiate,
	make_empty

create {CAPACT_LEGACY_DISTRIBUTION_EXPRESSION}

	default_create

feature {NONE} -- Initialization

	instantiate (a_target: CAPACT_SIGNATURE_RECORD) is
			-- instantiate capact
		do
			set_expression_index
			target := a_target
			set_signature (a_target)
			instantiate_all
			if parent_instrument_id = child_instrument_id then
				child_is_parent := True
				share_factor := child_units
				if child_units /= 0 then
					if legacy_order = 1 then
						rmdat_adjustment_factor := share_factor
					else
						rmdat_adjustment_factor := share_factor + 1
					end
				else
						--its a terminal distribution
					is_terminal_distribution := True
					rmdat_adjustment_factor := qnan
				end
			else
					--- parent <> child  if child is cash cash :=
					--- if child is equity then get cash  := unadjusted closing price for that instrument
				share_factor := 0
				rmdat_adjustment_factor := 1
			end
		end

feature -- Access

	processing_instruction (a_basis: RP_CORPORATE_ACTION_PROCESSING_SHARE_BASIS_TYPE; a_processing_system: RP_CORPORATE_ACTION_PROCESSING_SYSTEM): RP_CORPORATE_ACTION_PROCESSING_INSTRUCTIONS is
			-- processing corp action instruction
		local
			l_election_cash_dist: RP_CORPORATE_ACTION_ELECTION_CASH_DISTRIBUTION
			l_election_units_dist: RP_CORPORATE_ACTION_ELECTION_UNITS_DISTRIBUTION
			l_election_delivery: RP_CORPORATE_ACTION_ELECTION_DELIVERY
			l_election_delivery_distribution: RP_CORPORATE_ACTION_ELECTION_DELIVERY_DISTRIBUTION
			l_sim_corp_action: RP_SIMPLE_CORPORATE_ACTION
			l_found_error: BOOLEAN
		do
				-- This is much more simple that the code implies
				-- We get a pair with (parent instrument, child instrument) that explains how many child units / parent unit we get and how the cost gets distributed
				-- We simply create the correct
				-- The parent will always been delivered (eg. removed from the portfolio) and possibly redistributed (eg. added to the portfolio)
				-- The child can be the same security, another security	or cash
				-- In other words, this describes transformations of (1 security, N cash) to (M security, X cash) positions
				-- This is basically replaced with the expressions in the new system, which are much more flexible than fixed records
				-- 3 cases - cash distribution, units distribution, delivery
				-- DISTRIBUTION OF CASH
			if child_is_cash then
					-- cash distribution
				create l_election_cash_dist.make (a_processing_system, Current)
				create Result.make (a_basis, create {RP_SIMPLE_CORPORATE_ACTION}.make (exdate, l_election_cash_dist))
			end
				-- DISTRIBUTION OF ANOTHER INSTRUMENT
			if (not child_is_cash) and (not child_is_parent) and not child_units.is_equal (0) then
					-- we got units of a new instrument
				if child_instrument = Void then
					l_found_error := True
				elseif brimsym_from_listing (child_instrument, exdate) = Void then
					l_found_error := True
				else
					create l_election_units_dist.make (a_processing_system, Current)
					create l_sim_corp_action.make (exdate, l_election_units_dist)
					if Result = Void then
						create Result.make (a_basis, l_sim_corp_action)
					else
							-- add to exisitng instructions
						Result.add_to_corporate_actions (l_sim_corp_action)
					end
				end
			end
				-- DELIVERY OF THE SAME INSTRUMENT + optional DISTRIBUTION OF THE SAME INSTRUMENT
			if child_is_parent then
					-- handle delivery case
				if child_units > 0 then
						-- If the shares persist, then we need to create a distribution of the same instrument
					create l_election_delivery_distribution.make (a_processing_system, Current)
					create l_sim_corp_action.make (exdate, l_election_delivery_distribution)
				elseif child_units = 0 then
						-- Create a simple delivery
					create l_election_delivery.make (a_processing_system, Current)
					create l_sim_corp_action.make (exdate, l_election_delivery)
				end
				if Result = Void then
					create Result.make (a_basis, l_sim_corp_action)
				else
						-- add to exisitng instructions
					Result.add_to_corporate_actions (l_sim_corp_action)
				end
			end
		end

	cash_inflow (a_currency: CURRENCY): DOUBLE is
			-- Cash inflow
		local
			l_xrate: DOUBLE
			l_currency: CURRENCY
			l_country: COUNTRY
		do
			if child_is_cash and child_instrument = Void and not child_is_parent then
				if (a_currency /= Void and child_cash /= Void) and then a_currency.is_equal (child_cash) then
					Result := child_units
				else
					if (a_currency /= Void and child_cash /= Void) then
						l_xrate := a_currency.exchange_rate (child_cash, exdate)
						if legacy_undefined.is_defined_double (l_xrate) then
							Result := child_units * l_xrate
						else
							Result := undefined_double
						end
					else
						Result := undefined_double
					end
				end
			elseif legacy_basis_currency /= Void and not child_is_parent then
				l_currency := rose_context.currency_by_iso_code (legacy_basis_currency, now)
				if l_currency = Void then
					if legacy_basis_currency.is_equal ("EU") then
						l_currency := rose_context.eur
					else
						l_country := rose_context.country_at (iso_code (legacy_basis_currency), now)
						if l_country /= Void then
							l_currency := l_country.currency_history.witness (exdate)
						end
					end
				end
				if l_currency = Void then
					(create {EXCEPTIONS}).raise ("No currrency for given code " + legacy_basis_currency + "%N")
				end
				if a_currency.is_equal (l_currency) then
					Result := child_units * legacy_basis_price
				else
					l_xrate := a_currency.exchange_rate (l_currency, exdate)
					if legacy_undefined.is_defined_double (l_xrate) then
						Result := child_units * l_xrate * legacy_basis_price
					else
						Result := undefined_double
					end
				end
			elseif child_is_parent then
				Result := 0
			elseif legacy_basis_currency = Void and child_instrument /= Void then
				Result := undefined_double
			else
					-- when would we come here?
				Result := 0
			end
		end

	cash_outflow (a_currency: CURRENCY): DOUBLE is
			-- Cash outflow
		local
			l_xrate: DOUBLE
		do
			if additional_cash /= Void then
				if additional_cash.is_equal (a_currency) then
					Result := additional_unit_factor
				else
					l_xrate := a_currency.exchange_rate (additional_cash, exdate)
					if legacy_undefined.is_defined_double (l_xrate) then
						Result := additional_unit_factor * l_xrate
					else
						Result := undefined_double
					end
				end
			else
				Result := 0
			end
		end

	cash_dividend (a_currency: CURRENCY): DOUBLE is
			-- Cash dividend
		do
			Result := 0
		end

	terminal_distribution: ENTITY_VALUES [LISTING] is
			-- Terminal distribution
		do
			create Result.make_empty
			if not child_is_cash and child_instrument /= Void and child_units /= 0 then
				Result.pair_add (child_instrument, child_units)
			end
		ensure then
			not_void: Result /= Void
		end

	contract (a_corporate_action_processing_system: RP_CORPORATE_ACTION_PROCESSING_SYSTEM; a_portfolio: RP_PORTFOLIO; a_holding: RP_QUANTITY): RP_PORTFOLIO_CORPORATE_ACTION_CONTRACT is
			-- contract
		do
		end

	name: STRING is
			-- constructed name
		do
			Result := template_name + " F" + target.member_number.out
		end

	cash_rate: DOUBLE is
			-- does it make sense to define this here?
		do
			Result := undefined_double
		end

	cash_currency_iso: STRING is
			-- Cash currency iso code
		do
			Result := Void
		end

	adjustment_factor: DOUBLE is
			-- This may be redundant
		do
			Result := rmdat_adjustment_factor
		end

	subscription_price: DOUBLE is
			-- Subscription price
		do
			Result := undefined_double
		end

	legacy_basis_curr: STRING is
			-- Legacy basis curr
		do
		ensure then
			result_is_void: Result = Void
		end

	split_rate: DOUBLE is
			-- Split rate
		do
			Result := undefined_double
		end

	bonus_rate: DOUBLE is
			--Bonus rate
		do
			Result := undefined_double
		end

	dividend_rate: DOUBLE is
			-- dividend rate
		do
			Result := undefined_double
		end

feature -- Status Report

	child_is_parent: BOOLEAN
			-- flag checking if child is parent

end
