note

	description:

		"Rose capital action complex action (aka option) signature"

	author: "Janet Small"
	last_editor: "$Author: mb $"
	date: "$Date: 2014/07/11 $"
	revision: "$Revision: #24 $"
	path: "$File: //rose/source/kernel/capital_actions/capact_option_signature_record.e $"

class
	CAPACT_OPTION_SIGNATURE_RECORD
inherit

	CAPACT_OPTION_SIGNATURE_RECORD_FIELDS

	ROSE_SIGNATURE_RECORD [ENTITY]
		redefine
			data_manager,
			date_name,
			date,
			context,
			data_fields_string_,
			set_entity,
			signatures_fields_clause,
			signature_and_data_fields_clause
		end

	CAPITAL_ACTIONS_CONTEXT_SHARED
		undefine
			servers,
			is_equal
		end

	SHARED_CAPACT_RETRIEVAL
		export {NONE} all end

create
	default_create

create {CAPACT_OPTION_SIGNATURE_RECORD}
	rose_entity_date_make


feature-- Access

	listing:LISTING is
		-- parent listing that is generating the capital actions on the exdate
		require
			valid_listing:listing_id > 0
		do
			if listing_saved = Void then
				listing_saved := rose_context.listing_accessor.entity(listing_id)
			end
			Result := listing_saved
		end

	listing_saved:LISTING

	statement_: STRING is "select * from capact_option_data_signatures"

	fields_: STRING is "listing_id,exdate,option_number"

	data_fields_string_ : STRING is "instrument_id,market_id,option_expression,status_code,source"

	expression_tag : STRING is "capact_option_signature_record"

	date_name: STRING is	once Result := "end_current_date" end
	
	signatures_fields_clause: STRING is
			-- <Precursor>
		once
			Result := Precursor
		end
	
	signature_and_data_fields_clause: STRING is
			-- <Precursor>
		once
			Result := Precursor
		end

	date: ROSE_DATE is
			-- Record date
		do
			Result := end_current_date
		end

	data_manager : SIGNATURE_TAGGED_ITEM_DATABASE[like Current] is
			-- access to zonal signature-tagged data that finds zone if it's not already set
		local
			l_zone: STRING
			l_investment_entity: INVESTMENT_ENTITY
		do
			if zone /= '%U' then
				l_zone := zone.out
			elseif entity /= Void then
				l_investment_entity ?= entity
				if l_investment_entity /= Void then
					l_zone := l_investment_entity.zone_mnemonic.out
				end
			elseif instrument_id > 0 then
				if attached {INVESTMENT_ENTITY} rose_context.security_accessor.entity (instrument_id) as l_security then
					l_zone := l_security.zone_mnemonic.out
				end
			elseif listing_id > 0 and then listing /= Void then
				l_zone := listing.zone_mnemonic.out
			end
			if l_zone /= Void then
				Result := data_manager_by_zone (l_zone)
			end
		end

	data_manager_by_zone(a_zone: STRING): SIGNATURE_TAGGED_ITEM_DATABASE[like Current] is
			-- access to zonal signature-tagged data by zone
		require
			has_zone: a_zone /= Void
		local
			l_zone: STRING
		do
			l_zone := a_zone.as_upper
			if l_zone.is_equal("U") then
				Result := data_manager_U
			elseif  l_zone.is_equal("C") then
				Result := data_manager_C
			elseif  l_zone.is_equal("J") then
				Result := data_manager_J
			elseif  l_zone.is_equal("E") then
				Result := data_manager_E
			elseif  l_zone.is_equal("O") then
				Result := data_manager_O
			elseif  l_zone.is_equal("S") then
				Result := data_manager_S
			end

		end

	data_manager_U : SIGNATURE_TAGGED_ITEM_DATABASE[CAPACT_OPTION_SIGNATURE_RECORD] is
		once
			Result := private_data_manager(servers.rose_capital_action_U_server)
		end
	data_manager_C : SIGNATURE_TAGGED_ITEM_DATABASE[CAPACT_OPTION_SIGNATURE_RECORD] is
		once
			Result := private_data_manager(servers.rose_capital_action_C_server)
		end
	data_manager_J : SIGNATURE_TAGGED_ITEM_DATABASE[CAPACT_OPTION_SIGNATURE_RECORD] is
		once
			Result := private_data_manager(servers.rose_capital_action_J_server)
		end
	data_manager_E : SIGNATURE_TAGGED_ITEM_DATABASE[CAPACT_OPTION_SIGNATURE_RECORD] is
		once
			Result := private_data_manager(servers.rose_capital_action_E_server)
		end
	data_manager_O : SIGNATURE_TAGGED_ITEM_DATABASE[CAPACT_OPTION_SIGNATURE_RECORD] is
		once
			Result := private_data_manager(servers.rose_capital_action_O_server)
		end
	data_manager_S : SIGNATURE_TAGGED_ITEM_DATABASE[CAPACT_OPTION_SIGNATURE_RECORD] is
		once
			Result := private_data_manager(servers.rose_capital_action_S_server)
		end

	private_data_manager(a_server: SQL_DATA_SERVER): SIGNATURE_TAGGED_ITEM_DATABASE[like Current] is
		do
			if a_server /= Void then
				create Result.make_named("capact_option","browse", a_server)
			end
		end


	key_tuple:TUPLE is
		do
			Result:= [instrument_id,market_id,exdate,option_number]
		end

feature -- display

	display_signature: STRING is
		do
			Result := instrument_id.out+" "+exdate.out+" "+option_number.out+" "+option_expression+" "+instrument_id.out+" "+market_id.out
		end

feature--Restriction Statements

	restriction_statement : STRING is "#listing_id.id = listing_id  and #exdate.id = exdate"


feature -- Basic operations

	wipe_out is
			-- Clear caches
		do
			if data_manager_U /= Void then
				data_manager_U.purge
			end
			if data_manager_C /= Void then
				data_manager_C.purge
			end
			if data_manager_J /= Void then
				data_manager_J.purge
			end
			if data_manager_E /= Void then
				data_manager_E.purge
			end
			if data_manager_O /= Void then
				data_manager_O.purge
			end
			if data_manager_S /= Void then
				data_manager_S.purge
			end
			capact_retrieval.wipe_out
		end

	signatures_by_listing_exdate(a_listing_id : INTEGER; a_exdate: INTEGER): ROSE_LINEAR_ARRAY [CAPACT_OPTION_SIGNATURE_RECORD] is
		local
			l_targets: ROSE_LINEAR_ARRAY [CAPACT_OPTION_SIGNATURE_RECORD]
		do
			l_targets := signatures_by_listing(a_listing_id)
			if l_targets /= Void then
				create Result.make_empty
				l_targets.do_if (agent Result.extend, agent {like Current}.is_exdate_equal (a_exdate))
				if Result.is_empty then
					Result := Void
				end
			end
		ensure
			non_void_implies_non_empty: Result /= Void implies not Result.is_empty
		end

	signatures_by_listing_after(a_listing_id : INTEGER;  a_exdate: INTEGER): ROSE_LINEAR_ARRAY [CAPACT_OPTION_SIGNATURE_RECORD] is
		local
			l_targets: ROSE_LINEAR_ARRAY [CAPACT_OPTION_SIGNATURE_RECORD]
		do
			l_targets := signatures_by_listing(a_listing_id)
			if l_targets /= Void then
				create Result.make_empty
				l_targets.do_if (agent Result.extend, agent {like Current}.is_exdate_greater_than (a_exdate))
				if Result.is_empty then
					Result := Void
				end
			end
		ensure
			non_void_implies_non_empty: Result /= Void implies not Result.is_empty
		end

	signatures_by_listing(a_listing_id : INTEGER): ROSE_LINEAR_ARRAY [CAPACT_OPTION_SIGNATURE_RECORD] is
		do
			Result := capact_retrieval.capact_option_signatures_by_listing_id (a_listing_id)
		end

	signatures_by_instrument_market_and_exdate (a_instrument_id, a_market_id, a_exdate: INTEGER): ROSE_LINEAR_ARRAY [CAPACT_OPTION_SIGNATURE_RECORD] is
		local
			l_targets: ROSE_LINEAR_ARRAY [CAPACT_OPTION_SIGNATURE_RECORD]
		do
			create Result.make_empty
			l_targets := capact_retrieval.capact_option_signatures_by_instrument_id (a_instrument_id)
			if l_targets /= Void then
				l_targets.do_if (agent Result.extend, agent {like Current}.is_market_id_and_exdate_eq (a_market_id, a_exdate))
			end
		ensure
			not_void: Result /= Void
			no_void_item: not Result.has (Void)
		end

	is_market_id_and_exdate_eq (a_market_id, a_exdate: INTEGER): BOOLEAN is
			-- Is `exdate' = 'a_exdate' and `market_id' = `a_market_id'?
		do
			Result := is_exdate_equal (a_exdate) and (market_id = a_market_id)
		ensure
			definition: Result = (is_exdate_equal (a_exdate) and (market_id = a_market_id))
		end




	set_from_override (a_sig: ROSE_CAPACT_OPTION_OVERRIDE_SIGNATURE) is
		do
			set_listing_id (a_sig.listing_id)
			set_instrument_id(a_sig.instrument_id)
			set_market_id(a_sig.market_id)
			set_exdate(a_sig.exdate)
			set_option_number(a_sig.option_number)
			set_option_expression(a_sig.option_expression)

			if a_sig.status_code.is_equal("M") or a_sig.status_code.is_equal("A") then
				set_status_code(" ")
			elseif a_sig.status_code.is_equal("D") then
				set_status_code("Y")
			else
				set_status_code(a_sig.status_code)
			end
			set_source ("Override")
		end

feature  --Access

	context: CAPITAL_ACTIONS_CONTEXT is
			--
		once
			Result := capact_context
		end

	context_name: STRING is
		once
			Result := capact_context_name
		end

	set_entity (a_entity: like entity) is
		do
			Precursor (a_entity)
			if a_entity /= Void and then a_entity.category_id = context.ids.security_category_id then
				set_instrument_id(a_entity.handle)
			end
		end

end
