indexing
	description: "rose capital actions: rights instruments are given out"
	author: "Janet Small"
	last_editor: "$Author: bn $"
	date: "$Date: 2014/02/19 $"
	revision: "$Revision: #11 $"
	path: "$File: //rose/source/kernel/capital_actions/capact_renounceable.e $"

deferred class
	CAPACT_RENOUNCEABLE
inherit
	CAPACT_DISTRIBUTION
	ATTRIBUTES
		rename
			variable_index as attributes_variable_index,
			name as attributes_name,
			mnemonic as attributes_mnemonic
		end

	ROSE_CAPACT_DATE_USE

feature -- Access

	is_tradeable: BOOLEAN is
			-- tradeable?
		once
			Result := True
		end

	is_transferable: BOOLEAN is
			--transferable?
		once
			Result := True
		end

	last_trading_date: INTEGER
	set_last_trading_date (a_value: like last_trading_date) is
			-- last trading date
		do
			last_trading_date := a_value
		end

	first_trading_date: INTEGER
	set_first_trading_date (a_value: like first_trading_date) is
			-- first trading date
		do
			first_trading_date := a_value
		end

	rights_delivery_date: INTEGER
	set_rights_delivery_date (a_value: like rights_delivery_date) is
			-- rights delivery date
		do
			rights_delivery_date := a_value
		end
	rights_sedol: STRING
	set_rights_sedol (a_value: like rights_sedol) is
			-- rights sedol
		do
			rights_sedol := a_value
		end

	rights_entity: LISTING
	set_rights_entity (a_value: like rights_entity) is
			-- rights entity
		do
			rights_entity := a_value
		end

	rights_xrfzone: STRING
	set_rights_xrfzone (a_value: like rights_xrfzone) is
			-- rights xrf zone
		do
			rights_xrfzone := a_value
		end

	temporary_sedol: STRING
	set_temporary_sedol (a_value: like temporary_sedol) is
			-- temp. sedol
		do
			temporary_sedol := a_value
		end

	temporary_entity: LISTING
	set_temporary_entity (a_value: like temporary_entity) is
			-- temp. entity
		do
			temporary_entity := a_value
		end

	temporary_xrfzone: STRING
	set_temporary_xrfzone (a_value: like temporary_xrfzone) is
			-- temp. xrf zone
		do
			temporary_xrfzone := a_value
		end

	participation_rate: DOUBLE
	set_participation_rate (a_value: like participation_rate) is
			-- participation rate
		do
			participation_rate := a_value
		end

	in_lieu_cash_currency: STRING
	set_in_lieu_cash_currency (a_value: like in_lieu_cash_currency) is
			-- iso of in lieu cash currency (needed also to support legacy backdiv with an iso value)
		do
			in_lieu_cash_currency := a_value
		end



feature -- Basic operations

	instantiate_renounceable_data is
		-- retrieve data items from storage
		local
			s: STRING
			i: INTEGER
		do
			s := string_value (new_shares_rate_expression, signature)
			if s /= Void and then s.is_double then
				set_new_shares_rate(s.to_double)
			end

			i := child_instrument_expression.integer_value(signature)
			if i > 0 then
				rights_entity := rose_context.listing_accessor.entity (i)
			end

			set_rights_sedol(string_value (rights_sedol_expression, signature))

			if rights_entity = Void and then rights_sedol /= Void then
				rights_entity := rose_context.listing_at (sedol (rights_sedol), today)
				if rights_entity = Void or else rights_entity.handle = 0 then
					rights_entity := rose_context.listing_at (sedol (rights_sedol), safe_date_from_ccyymmdd (exdate))
				end
			end

			if rights_entity /= Void and then rights_entity.handle > 0 then
				rights_xrfzone := rights_entity.xrfzone_key (today)
			end

			set_temporary_sedol(string_value (temporary_sedol_expression, signature))

			if temporary_sedol /= Void then
				temporary_entity := rose_context.listing_at (sedol (temporary_sedol), today)
				if temporary_entity = Void or else temporary_entity.handle = 0 then
					temporary_entity := rose_context.listing_at (sedol (temporary_sedol), safe_date_from_ccyymmdd (exdate))
				end
			end

			if temporary_entity /= Void and then temporary_entity.handle > 0 then
				temporary_xrfzone := temporary_entity.xrf_code_key (today)
			end

			set_rights_delivery_date (rights_delivery_date_expression.integer_value (signature))

			set_last_trading_date (last_trading_date_expression.integer_value(signature))
			set_first_trading_date (first_trading_date_expression.integer_value(signature))

			set_in_lieu_cash_currency(string_value (in_lieu_cash_currency_expression, signature))
		end

	set_renounceable_data_items is
			-- set renounceable data items
		local
			l_variable: CAPACT_VARIABLE_DESCRIPTION_ROW
			s: STRING
		do
			if new_shares_rate > 0  then
				l_variable := variable_table.item("NEW_SHARES_RATE")
				s := format_rate.formatted (new_shares_rate)
				string_items.put(s, l_variable.item_id)
			end

			if rights_entity /= Void then
				l_variable := variable_table.item("CHILD_INSTRUMENT")
				integer_items.put(rights_entity.handle, l_variable.item_id)
			end

			if rights_sedol /= Void then
				l_variable := variable_table.item("RIGHTS_SEDOL")
				string_items.put(rights_sedol, l_variable.item_id)
			end

			if temporary_sedol /= Void then
				l_variable := variable_table.item("TEMPORARY_SEDOL")
				string_items.put(temporary_sedol, l_variable.item_id)
			end

			if last_trading_date > 19000101 then
				l_variable := variable_table.item("LAST_TRADING_DATE")
				integer_items.put(last_trading_date, l_variable.item_id)
			end

			if first_trading_date > 19000101 then
				l_variable := variable_table.item("FIRST_TRADING_DATE")
				integer_items.put(first_trading_date, l_variable.item_id)
			end

			if in_lieu_cash_currency /= Void  then
				l_variable := variable_table.item("IN_LIEU_CASH_CURRENCY")
				string_items.put(in_lieu_cash_currency, l_variable.item_id)
			end

		end

	child_instrument_expression:ROSE_EXPRESSION [ROSE_ENTITY_DATE [ENTITY]] is
			-- child instrument item in virtual record
		once
			Result := ur_target.expression_from_name("CHILD_INSTRUMENT")
		end


	rights_sedol_expression:ROSE_EXPRESSION [ROSE_ENTITY_DATE [ENTITY]] is
			-- rights sedol item in virtual record
		once
			Result := ur_target.expression_from_name("RIGHTS_SEDOL")
		end

	temporary_sedol_expression:ROSE_EXPRESSION [ROSE_ENTITY_DATE [ENTITY]] is
			-- temporary sedol itemin virtual record
		once
			Result := ur_target.expression_from_name("TEMPORARY_SEDOL")
		end

	last_trading_date_expression:ROSE_EXPRESSION [ROSE_ENTITY_DATE [ENTITY]] is
			-- last trading date virtual record
		once
			Result := ur_target.expression_from_name("LAST_TRADING_DATE")
		end
	first_trading_date_expression:ROSE_EXPRESSION [ROSE_ENTITY_DATE [ENTITY]] is
			-- first trading date virtual record
		once
			Result := ur_target.expression_from_name("FIRST_TRADING_DATE")
		end

	rights_delivery_date_expression:ROSE_EXPRESSION [ROSE_ENTITY_DATE [ENTITY]] is
			-- rights delivery date virtual record
		once
			Result := ur_target.expression_from_name("RIGHTS_DELIVERY_DATE")
		end

	 in_lieu_cash_currency_expression:ROSE_EXPRESSION [ROSE_ENTITY_DATE [ENTITY]] is
			-- in lieu cash currency iso code
		once
			Result := ur_target.expression_from_name("IN_LIEU_CASH_CURRENCY")
		end



feature {NONE} -- Implementation



end
