note

	description:

		"CAPACT signature caches"

	author: "Ivan Tarnavsky"
	last_editor: "$Author: it $"
	date: "$Date: 2012/10/26 $"
	revision: "$Revision: #2 $"
	path: "$File: //rose/source/kernel/capital_actions/capact_retrieval.e $"

class CAPACT_RETRIEVAL

inherit

	ANY

	DATABASE_SERVERS
		export {NONE} all end

create {SHARED_CAPACT_RETRIEVAL}

	make

feature {NONE} -- Initialization

	make is
			-- Initialize `Current'
		do
			create capacts_by_listing.make (agent {CAPACT_SIGNATURE_RECORD}.listing_id)
			create options_by_listing.make (agent {CAPACT_OPTION_SIGNATURE_RECORD}.listing_id)
			wipe_out
		end

feature -- Access

	capact_signatures_by_company_id (a_company_id: INTEGER): ARRAY [CAPACT_SIGNATURE_RECORD] is
			-- CAPACT_SIGNATURE_RECORDs for `a_company_id'
		local
			l_by_listing: DS_HASH_TABLE [ARRAY [CAPACT_SIGNATURE_RECORD], INTEGER]
		do
			capact_signatures_by_company.search (a_company_id)
			if not capact_signatures_by_company.found then
				capact_signatures_by_company.load (a_company_id)
				l_by_listing := capacts_by_listing.split (capact_signatures_by_company.signatures)
				l_by_listing.do_all_with_key (agent capact_signatures_by_listing.store (?, ?))
			end
			Result := capact_signatures_by_company.signatures
		ensure
			not_void: Result /= Void
			no_void_item: not Result.has (Void)
			for_company: Result.for_all (agent {CAPACT_SIGNATURE_RECORD}.has_company_id (a_company_id))
		end

	capact_signatures_by_listing_id (a_listing_id: INTEGER): ARRAY [CAPACT_SIGNATURE_RECORD] is
			-- CAPACT_SIGNATURE_RECORD for `a_listing_is'
		do
			capact_signatures_by_listing.search (a_listing_id)
			if not capact_signatures_by_listing.found then
				capact_signatures_by_listing.load (a_listing_id)
			end
			Result := capact_signatures_by_listing.signatures
		ensure
			not_void: Result /= Void
			no_void_item: not Result.has (Void)
			for_listing: Result.for_all (agent {CAPACT_SIGNATURE_RECORD}.has_listing_id (a_listing_id))
		end

	capact_signatures_by_listing_id_and_exdate_between (a_listing_id: INTEGER; a_start_exdate, a_end_exdate: INTEGER): ROSE_LINEAR_ARRAY [CAPACT_SIGNATURE_RECORD] is
			-- Signatures for `a_listing_id' and exdate > `a_start_date' and exdate <= `a_end_exdate'
		do
			create Result.make_empty
			capact_signatures_by_listing_id (a_listing_id).do_if (agent Result.extend, agent {CAPACT_SIGNATURE_RECORD}.is_exdate_between_open_closed (a_start_exdate, a_end_exdate))
		ensure
			not_void: Result /= Void
			no_void_item: not Result.has (Void)
			for_listing: Result.for_all (agent {CAPACT_SIGNATURE_RECORD}.has_listing_id (a_listing_id))
			exdate_between_open_closed: Result.for_all (agent {CAPACT_SIGNATURE_RECORD}.is_exdate_between_open_closed (a_start_exdate, a_end_exdate))
		end

	capact_option_signatures_by_instrument_id (a_instrument_id: INTEGER): ARRAY [CAPACT_OPTION_SIGNATURE_RECORD] is
			-- CAPACT_OPTION_SIGNATURE_RECORDs for `a_instrument_id'
		local
			l_by_listing: DS_HASH_TABLE [ARRAY [CAPACT_OPTION_SIGNATURE_RECORD], INTEGER]
		do
			capact_option_signatures_by_instrument.search (a_instrument_id)
			if not capact_option_signatures_by_instrument.found  then
				capact_option_signatures_by_instrument.load (a_instrument_id)
				l_by_listing := options_by_listing.split (capact_option_signatures_by_instrument.signatures)
				l_by_listing.do_all_with_key (agent capact_option_signatures_by_instrument.store (?, ?))
			end
			Result := capact_option_signatures_by_instrument.signatures
		ensure
			not_void: Result /= Void
			no_void_item: not Result.has (Void)
			for_listing: Result.for_all (agent {CAPACT_OPTION_SIGNATURE_RECORD}.has_instrument_id (a_instrument_id))
		end

	capact_option_signatures_by_listing_id (a_listing_id: INTEGER): ARRAY [CAPACT_OPTION_SIGNATURE_RECORD] is
			-- CAPACT_OPTION_SIGNATURE_RECORDs for `a_listing_id'
		do
			capact_option_signatures_by_listing.search (a_listing_id)
			if not capact_option_signatures_by_listing.found then
				capact_option_signatures_by_listing.load (a_listing_id)
			end
			Result := capact_option_signatures_by_listing.signatures
		ensure
			not_void: Result /= Void
			no_void_item: not Result.has (Void)
			for_listing: Result.for_all (agent {CAPACT_OPTION_SIGNATURE_RECORD}.has_listing_id (a_listing_id))
		end

feature -- Basic Operations

	wipe_out is
			-- Wipe out cache content
		do
			create capact_signatures_by_company.make (company_id_query, agent capact_signature_template_with_company_id_set)
			create capact_signatures_by_listing.make_with_order_by (listing_id_query, agent capact_signature_template_with_listing_id_set, agent order_capact_signatures_by_exdate_member_number)
			create capact_option_signatures_by_instrument.make (instrument_id_query, agent capact_option_signature_template_with_instrument_id_set)
			create capact_option_signatures_by_listing.make_with_order_by (listing_id_query, agent capact_option_signature_template_with_listing_id_set, agent order_option_signatures_by_exdate_option_number)
		end

feature {NONE} -- Implementation

	company_id_query: STRING is "#company_id.id = company_id"
			-- Company filter

	capact_signatures_by_company: STID_RECORD_CACHE [CAPACT_SIGNATURE_RECORD, INTEGER]
			-- Cached CAPACT_SIGNATURE_RECORDs by company

	capact_signature_template_with_company_id_set (a_company_id: INTEGER): CAPACT_SIGNATURE_RECORD is
			-- Template for retrieval by `a_company_id'
		do
			create Result
			Result.set_company_id (a_company_id)
		ensure
			not_void: Result /= Void
			company_id_set: Result.company_id = a_company_id
		end

	capacts_by_listing: ARRAY_SPLITTER_BY_EMBEDDED_KEY [CAPACT_SIGNATURE_RECORD, INTEGER]
			-- Array splitter by listing id

	listing_id_query: STRING is "#listing_id.id = listing_id"
			-- Listing filter

	capact_signatures_by_listing: STID_RECORD_CACHE [CAPACT_SIGNATURE_RECORD, INTEGER]
			-- Cached CAPACT_SIGNATURE_RECORDs by listing

	capact_signature_template_with_listing_id_set (a_listing_id: INTEGER): CAPACT_SIGNATURE_RECORD is
			-- Template for retrieval by `a_listing_id'
		do
			create Result
			Result.set_listing_id (a_listing_id)
		ensure
			not_void: Result /= Void
			listing_id_set: Result.listing_id = a_listing_id
		end

	capact_option_signatures_by_listing: STID_RECORD_CACHE [CAPACT_OPTION_SIGNATURE_RECORD, INTEGER]
			-- Cached CAPACT_OPTION_SIGNATURE_RECORDs by listing

	capact_option_signature_template_with_listing_id_set (a_listing_id: INTEGER): CAPACT_OPTION_SIGNATURE_RECORD is
			-- Template for retrieval by `a_listing_id'
		do
			create Result
			Result.set_listing_id (a_listing_id)
		ensure
			not_void: Result /= Void
			listing_id_set: Result.listing_id = a_listing_id
		end

	instrument_id_query: STRING is "#instrument_id.id = instrument_id"
			-- Listing filter

	options_by_listing: ARRAY_SPLITTER_BY_EMBEDDED_KEY [CAPACT_OPTION_SIGNATURE_RECORD, INTEGER]
			-- CAPACT_OPTION_SIGNATURE_RECORDs array splitter by listing

	capact_option_signatures_by_instrument: STID_RECORD_CACHE [CAPACT_OPTION_SIGNATURE_RECORD, INTEGER]
			-- Cached CAPACT_OPTION_SIGNATURE_RECORDs by instrument

	capact_option_signature_template_with_instrument_id_set (a_instrument_id: INTEGER): CAPACT_OPTION_SIGNATURE_RECORD is
			-- Template for retrieval by `a_instrument_id'
		do
			create Result
			Result.set_instrument_id (a_instrument_id)
		ensure
			not_void: Result /= Void
			instrument_id_set: Result.instrument_id = a_instrument_id
		end

	order_capact_signatures_by_exdate_member_number (a, b: CAPACT_SIGNATURE_RECORD): BOOLEAN is
			-- Order `a' and `b' by exdate then option number
		require
			a_not_void: a /= Void
			b_not_void: b /= Void
		do
			if a.exdate = b.exdate then
				Result := a.member_number < b.member_number
			else
				Result := a.exdate < b.exdate
			end
		ensure
			definition: Result = (a.exdate < b.exdate or else (a.exdate = b.exdate and a.member_number < b.member_number))
		end

	order_option_signatures_by_exdate_option_number (a, b: CAPACT_OPTION_SIGNATURE_RECORD): BOOLEAN is
			-- Order `a' and `b' by exdate then option number
		require
			a_not_void: a /= Void
			b_not_void: b /= Void
		do
			if a.exdate = b.exdate then
				Result := a.option_number < b.option_number
			else
				Result := a.exdate < b.exdate
			end
		ensure
			definition: Result = (a.exdate < b.exdate or else (a.exdate = b.exdate and a.option_number < b.option_number))
		end

end
