indexing
	description: "rose capital action expression: non-renounceable rights in same instrument given out"
	author: "Janet Small"
	date: "$Date: 2009/11/06 $"
	revision: "$Revision: #9 $"
	archive: "$File: //rose/source/kernel/capital_actions/expressions/capact_rights_in_same_expression.e $"

class
	CAPACT_RIGHTS_IN_SAME_EXPRESSION

inherit

	CAPACT_DEFAULT_EXPRESSION
--		undefine
--			cash_rate
		redefine
			cash_inflow, cash_outflow, cash_dividend
		end

	CAPACT_RIGHTS_IN_SAME
		rename
			signature as target, exdate as integer_exdate, name as template_name
		undefine
			default_create
		end

create
	make,instantiate,make_empty

feature {NONE} -- Initialization

	instantiate (a_target: CAPACT_SIGNATURE_RECORD) is
			-- Populate expression from storage
		do
			target := a_target
			set_signature(a_target)
			instantiate_all
			delta_new_shares := new_shares_rate
			if new_shares_rate > 0 and cash_rate <= 0 then
				rmdat_adjustment_factor := 1 + new_shares_rate
			else
				rmdat_adjustment_factor := 1
			end
			participation_rate := 1
		end

feature -- Access

	processing_instruction (
		a_basis: RP_CORPORATE_ACTION_PROCESSING_SHARE_BASIS_TYPE;
		a_processing_system: RP_CORPORATE_ACTION_PROCESSING_SYSTEM): RP_CORPORATE_ACTION_PROCESSING_INSTRUCTIONS is
			-- Processing corp action instruction
		do
			create Result.make (a_basis, create {RP_SIMPLE_CORPORATE_ACTION}.make (exdate, a_processing_system.rights_in_same_instructions (Current)))
		end

	cash_outflow (a_currency:CURRENCY):DOUBLE is
			-- Cash leaving portfolio as a result of capital actions
		local
			l_xrate : DOUBLE
			l_price: RMDAT_PRICE_RECORD
			l_curr: CURRENCY
		do
			if cash_currency = Void and then target.listing/= Void then
				if target.listing.zone_mnemonic.as_upper = 'J' then
					cash_currency := rose_context.currency_by_iso_code ("JPY",exdate)
				end
			end
			if (a_currency /= Void and cash_currency /= Void) and then a_currency.handle = cash_currency.handle then
				Result :=  -cash_rate
			else
				if cash_currency = Void or exdate = Void then
					Result := undefined_double
				else
					if (a_currency /= Void and cash_currency /= Void) then
						l_xrate := a_currency.exchange_rate (cash_currency, exdate)
						if legacy_undefined.is_defined_double(l_xrate) then
							Result := - cash_rate*l_xrate
						else
							Result := undefined_double
						end
					else
						Result := undefined_double
					end
				end
			end
			-- check for subscription price greater than market price
			l_price := target.listing.price_record (exdate)
			if l_price /= Void  then
				l_curr := l_price.currency
				if l_curr /= void and subscription_currency /= Void and then l_curr.handle = subscription_currency.handle then
					if subscription_price > l_price.close then
						participation_rate := 0
					end
				end
			end
		end

	cash_inflow(a_currency:CURRENCY):DOUBLE is
			-- Cash entering portfolio as a result of capital action
		do
			Result := 0
		end

	cash_dividend (a_currency:CURRENCY):DOUBLE is
			-- Cash dividend to porfolio as a result of capital action
		do
			Result := 0
		end

feature -- Output

	name: STRING is
			-- Name plus member number for display in capact_browser
		do
			Result := "RIGHTS IN SAME" + ": F" + target.member_number.out
		end

end
