note

	description:

		"Rose capital action signature"

	author: "Janet Small"
	last_editor: "$Author: mb $"
	date: "$Date: 2014/07/11 $"
	revision: "$Revision: #55 $"
	path: "$File: //rose/source/kernel/capital_actions/capact_signature_record.e $"

class
	CAPACT_SIGNATURE_RECORD

inherit

	CAPACT_SIGNATURE_RECORD_FIELDS

	ROSE_SIGNATURE_RECORD [ENTITY]
		redefine
			data_manager,
			date_name,
			date,
			context,
			data_fields_string_,
			set_entity,
			targets,
			new_variable,
			expression_cache_size,
			signature_and_data_fields_clause
		end

	CAPITAL_ACTIONS_CONTEXT_SHARED
		undefine
			servers,
			is_equal
		end

	ROSE_AWARE
		rename
			context as rose_context
		end

	MISMATCH_CORRECTOR
		undefine
			is_equal
		redefine
			correct_mismatch
		end

	SHARED_CAPACT_RETRIEVAL
		export {NONE} all end

create
	default_create

create {CAPACT_SIGNATURE_RECORD}
	rose_entity_date_make

feature -- Cluster Ids

	key_0 : INTEGER is
			-- key 0
		do
			Result := instrument_id
		end

	key_0_name : STRING is
			-- key 0 name
		do
			Result := "instrument_id"
		end

	key_1 : INTEGER is
			-- key 1
		do
			Result := exdate
		end

	key_1_name : STRING is
			-- key 0 name
		do
			Result := "exdate"
		end

feature -- signature fields not in rose_signature

	correct_mismatch is
			-- correct mismatch
		do  end

feature -- Access

	signature_and_data_fields_clause: STRING is
			-- <Precursor>
		once
			Result := Precursor
		end

  listing:LISTING is
  	-- listing from listing_id or else from instrument_id and market_id
	require
		valid_instrument:instrument_id > 0
		valid_market: market_id > 0
	local
		l_listing:LISTING
		l_security:SECURITY
		l_exchange :EXCHANGE
		n,i:INTEGER
	do
		if listing_saved = Void then
			if listing_id > 0 then
				listing_saved := rose_context.listing_accessor.entity (listing_id)
			end
			if listing_saved = Void then
				l_security := rose_context.security_accessor.entity(instrument_id)
				if l_security = void or else l_security.handle <= 0 then
					fail("listing","No security for handle $1",<<instrument_id>>)
				end

				l_exchange := rose_context.exchange_accessor.entity(market_id)
				if l_exchange = void or else l_exchange.handle <= 0 then
					fail("listing","No exchange for handle $1",<<market_id>>)
				end
				n := l_security.listing_count
				from i := 1 until i > n or listing_saved /= Void and listing_saved.handle > 0 loop
					l_listing := l_security.listing(i)
					if l_listing.in_investment_universe and then l_listing.exchange.handle = l_exchange.handle then
						listing_saved := l_listing
						listing_id := listing_saved.handle
					end
					i:= i+1
				end
				if listing_saved = Void or else listing_saved.handle <= 0 then
				-- exchange change?! how to deal with these? need MARKETS!...in the meantime
				-- if listing exchange is same country as market exchange country, accept it
				--example:
				--114141	Void	Void	Singapore Stock Exchange - alternate	
				--3378690	SSE	XSES	SINGAPORE EXCHANGE
					from i := 1 until i > n or listing_saved /= Void loop
						l_listing := l_security.listing(i)
						if l_listing.exchange.country.handle = l_exchange.country.handle then
							listing_saved := l_listing
							listing_id := listing_saved.handle
						end
						i:= i+1
					end
				end
				if listing_saved = Void  then
					fail("listing" , "NO listing found for security: $1, market: $2 ", <<instrument_id,market_id>>)
				end
			end
		end
		Result := listing_saved
	end

	listing_saved:LISTING -- utility to avoid multiple retrieval

	set_entity (a_entity: like entity) is
			--set instrument_id from handle of security entity
		do
			Precursor (a_entity)
			if a_entity /= Void and then a_entity.category_id = context.ids.security_category_id then
				set_instrument_id(a_entity.handle)
			end
		end

	context: CAPITAL_ACTIONS_CONTEXT is
			-- context required by rose_signature_record
			-- (entity handles refer to rose_entity context)
		once
			Result := capact_context
		end

	context_name : STRING is
			-- name for context required by rose_signature_record
		once
			Result := capact_context_name
		end

	key_tuple:TUPLE is
		-- tuple of signature key fields
		do
			Result:= [instrument_id,market_id,exdate,member_number]
		end

	expression_tag : STRING is "capact_signature_record"
	date_name: STRING is	once Result := "end_current_date" end

	date: ROSE_DATE is
			-- Record date
		do
			Result := end_current_date
		end

feature -- status

	is_valid_status: BOOLEAN is
			-- status indicating whether capital action is valid
		do
			Result := status_code = Void or else status_code ~ " " or status_code ~ "R" or status_code ~ ""
		end

feature -- data managers

	data_manager: SIGNATURE_TAGGED_ITEM_DATABASE [like Current] is
		-- data manager that gets zone either from zone or from zone of listing
		local
			l_listing: LISTING
			l_zone: STRING
			l_investment_entity: INVESTMENT_ENTITY
		do
			if zone /= '%U' then
				l_zone := zone.out
				Result := data_manager_by_zone(l_zone)
			elseif entity /= Void then
				l_investment_entity ?= entity
				if l_investment_entity /= Void then
					Result := data_manager_by_zone (l_investment_entity.zone_mnemonic.out)
				end
			else
				l_listing := rose_context.listing_accessor.entity (listing_id)
				if l_listing /= void and then l_listing.handle > 0 then
					l_zone := l_listing.zone_mnemonic.out
					Result := data_manager_by_zone(l_zone)
			 	else
					if company_id > 0 then
						if attached {INVESTMENT_ENTITY} rose_context.company_accessor.entity (company_id) as l_company then
							Result := data_manager_by_zone (l_company.zone_mnemonic.out)
						else
					 		fail("data_manager","No company for company_id $1", <<company_id>>)
						end
					else
					 	fail("data_manager","No listing for listing_id $1", <<listing_id>>)
				 	end
				end
			end
		end

	wipe_out is
			-- Purge caches
		do
			if data_manager_U /= Void then
				data_manager_U.purge
			end
			if data_manager_C /= Void then
				data_manager_C.purge
			end
			if data_manager_J /= Void then
				data_manager_J.purge
			end
			if data_manager_E /= Void then
				data_manager_E.purge
			end
			if data_manager_O /= Void then
				data_manager_O.purge
			end
			if data_manager_S /= Void then
				data_manager_S.purge
			end
			capact_retrieval.wipe_out
		end

	data_manager_by_zone(a_zone: STRING): SIGNATURE_TAGGED_ITEM_DATABASE[like Current] is
			-- return zonal data_manager given a zone
		require
			has_zone: a_zone /= Void
		do
			inspect a_zone.item (1)
			when 'U' then
				Result := data_manager_U
			when 'C' then
				Result := data_manager_C
			when 'J' then
				Result := data_manager_J
			when 'E' then
				Result := data_manager_E
			when 'O' then
				Result := data_manager_O
			when 'S' then
				Result := data_manager_S
			end

		end

	data_manager_U : SIGNATURE_TAGGED_ITEM_DATABASE[like Current] is
		-- data manager accessing U zone rose capital actions
		do
			Result ?= data_managers_U[generating_type]
			if Result = void then
				Result := private_data_manager(servers.rose_capital_action_U_server)
				data_managers_U[generating_type] := Result
			end
		end
	data_manager_C : SIGNATURE_TAGGED_ITEM_DATABASE[like Current] is
			-- data manager accessing C zone rose capital actions
		do
			Result ?= data_managers_C[generating_type]
			if Result = void then
				Result := private_data_manager(servers.rose_capital_action_C_server)
				data_managers_C[generating_type] := Result
			end
		end
	data_manager_J : SIGNATURE_TAGGED_ITEM_DATABASE[like Current] is
			-- data manager accessing J zone rose capital actions
		do
			Result ?= data_managers_J[generating_type]
			if Result = void then
				Result := private_data_manager(servers.rose_capital_action_J_server)
				data_managers_J[generating_type] := Result
			end
		end
	data_manager_E : SIGNATURE_TAGGED_ITEM_DATABASE[like Current] is
			-- data manager accessing E zone rose capital actions
		do
			Result ?= data_managers_E[generating_type]
			if Result = void then
				Result := private_data_manager(servers.rose_capital_action_E_server)
				data_managers_E[generating_type] := Result
			end
		end
	data_manager_O : SIGNATURE_TAGGED_ITEM_DATABASE[like Current] is
			-- data manager accessing O zone rose capital actions
		do
			Result ?= data_managers_O[generating_type]
			if Result = void then
				Result := private_data_manager(servers.rose_capital_action_O_server)
				data_managers_O[generating_type] := Result
			end
		end
	data_manager_S : SIGNATURE_TAGGED_ITEM_DATABASE[like Current] is
			-- data manager accessing S zone rose capital actions
		do
			Result ?= data_managers_S[generating_type]
			if Result = void then
				Result := private_data_manager(servers.rose_capital_action_S_server)
				data_managers_S[generating_type] := Result
			end
		end

	data_managers_U : HASH_TABLE[SIGNATURE_TAGGED_ITEM_DATABASE[CAPACT_SIGNATURE_RECORD],STRING] is
		-- data managers accessing U zone rose capital actions
		once
			create Result.make(10)
		end
	data_managers_C : HASH_TABLE[SIGNATURE_TAGGED_ITEM_DATABASE[CAPACT_SIGNATURE_RECORD],STRING] is
			-- data managers accessing C zone rose capital actions
		once
			create Result.make(10)
		end
	data_managers_J : HASH_TABLE[SIGNATURE_TAGGED_ITEM_DATABASE[CAPACT_SIGNATURE_RECORD],STRING] is
			-- data managers accessing J zone rose capital actions
		once
			create Result.make(10)
		end
	data_managers_E : HASH_TABLE[SIGNATURE_TAGGED_ITEM_DATABASE[CAPACT_SIGNATURE_RECORD],STRING] is
			-- data managers accessing E zone rose capital actions
		once
			create Result.make(10)
		end
	data_managers_O : HASH_TABLE[SIGNATURE_TAGGED_ITEM_DATABASE[CAPACT_SIGNATURE_RECORD],STRING] is
			-- data managers accessing O zone rose capital actions
		once
			create Result.make(10)
		end
	data_managers_S : HASH_TABLE[SIGNATURE_TAGGED_ITEM_DATABASE[CAPACT_SIGNATURE_RECORD],STRING] is
			-- data managers accessing S zone rose capital actions
		once
			create Result.make(10)
		end
	private_data_manager(a_server: SQL_DATA_SERVER): SIGNATURE_TAGGED_ITEM_DATABASE[like Current] is
		--create data_manager other than the default from capact_signature_record
		do
			if a_server /= Void then
				create Result.make_named("capact","browse", a_server)
				Result.set_cache_pool_on (False)
			end
		end

	new_variable (a_description: VARIABLE_DESCRIPTION): SIGNATURE_TAGGED_NEIGHBORHOOD_EXPRESSION [like Current]
		do
			create Result.make (a_description)
		end

	expression_cache_size: PAIR  [INTEGER, INTEGER] is
			-- PAIR  [CACHE SIZE, CHUNK SIZE]
		do
			create Result.make (10000, 1000)
		end

	target_neighborhood: ROSE_LINEAR_ARRAY [like Current]
	set_target_neighborhood (a_neighborhood: ROSE_LINEAR_ARRAY [like Current]) is
		do
			target_neighborhood := a_neighborhood
		end
	targets(a_targets: ARRAY [like Current]; a_statement: STRING): ROSE_LINEAR_ARRAY[like Current] is
		do
			if target_neighborhood /= Void then
				Result := target_neighborhood
			else
				Result := precursor (a_targets, a_statement)
			end
		end

feature -- signatures by various selection criteria

	signature_statement: STRING is "#listing_id.id = listing_id  and #exdate.id = exdate and #member_number.id = member_number"
	restriction_statement : STRING is once Result := option_restriction_statement end
			-- restrict on (instrument_id, market_id, exdate)

	option_restriction_statement: STRING is "#listing_id.id = listing_id and #exdate.id = exdate"

	statement_ : STRING is "select * from capact_data_signatures"
	fields_: STRING is "listing_id,exdate,member_number"
	data_fields_string_ : STRING is "instrument_id,market_id,record_date,capact_type_id,company_id,status_code,source"

	signatures_by_company_id_type_interval (a_company_id, a_type_id: INTEGER; a_start_date, a_end_date: ROSE_DATE): ROSE_LINEAR_ARRAY [CAPACT_SIGNATURE_RECORD] is
			-- Signatures for company, type, and xdates with restriction statement
		require
			valid_start_date: a_start_date /= Void
			valid_end_date: a_end_date /= Void
			date_ordering: a_start_date <= a_end_date
			valid_type: a_type_id >= 0
		do
			create Result.make_empty
			capact_retrieval.capact_signatures_by_company_id (a_company_id).do_if (agent Result.extend, agent {like Current}.filter_by_type_status_ne_and_exdate_between_closed_open (a_type_id, "Y", a_start_date.ccyymmdd, a_end_date.ccyymmdd))
		ensure
			not_void: Result /= Void
		end

	signatures_by_listing_id(a_listing_id : INTEGER): ARRAY [CAPACT_SIGNATURE_RECORD] is
			-- Signature records for `a_listing_id'
		do
			Result := capact_retrieval.capact_signatures_by_listing_id (a_listing_id)
		end

	signatures_by_listing_exdate(a_listing_id : INTEGER; a_exdate: INTEGER): ROSE_LINEAR_ARRAY [CAPACT_SIGNATURE_RECORD] is
			-- Signature records for `a_listing_id' and `a_exdate'
		local
			l_targets: ROSE_LINEAR_ARRAY [CAPACT_SIGNATURE_RECORD]
		do
			l_targets := signatures_by_listing_id(a_listing_id)
			if l_targets /= Void then
				create Result.make_empty
				l_targets.do_if (agent Result.extend, agent {like Current}.is_exdate_equal (a_exdate))
				if Result.count < 1 then
					Result := Void
				end
			end
		end

	signatures_by_listing_id_after(a_listing_id : INTEGER; a_exdate: INTEGER): ROSE_LINEAR_ARRAY [CAPACT_SIGNATURE_RECORD] is
			-- Signature records for `a_listing_id' after `a_exdate'
		local
			l_target: like Current
		do
			create l_target
			l_target.set_listing_id (a_listing_id)
			l_target.set_exdate (a_exdate)
			Result := data_manager.retrieve_current_signatures (<<l_target>>, "#listing_id.id = listing_id and #exdate.id <= exdate")
		end

	signatures_by_listing_id_and_exdate_between (a_listing_id: INTEGER; a_start_exdate, a_end_exdate: INTEGER): ROSE_LINEAR_ARRAY [CAPACT_SIGNATURE_RECORD] is
			-- Signatures for `a_listing_id' and exdate > `a_start_date' and exdate <= `a_end_exdate'
		do
			Result := capact_retrieval.capact_signatures_by_listing_id_and_exdate_between (a_listing_id, a_start_exdate, a_end_exdate)
		ensure
			not_void: Result /= Void
			no_void_item: not Result.has (Void)
		end

	filter_by_type_status_ne_and_exdate_between_closed_open (a_type: like capact_type_id; a_status_code: like status_code; a_start_exdate, a_end_exdate: INTEGER): BOOLEAN is
			-- Is type = `a_type' and status /= `a_status_code' and exdate between `a_start_date' and `a_end_date'
		do
			Result := capact_type_id = a_type and status_code /~ a_status_code and is_exdate_between_closed_open (a_start_exdate, a_end_exdate)
		ensure
			definition: Result = (capact_type_id = a_type and status_code /~ a_status_code and is_exdate_between_closed_open (a_start_exdate, a_end_exdate))
		end

feature -- Implementation		

	typed_capital_action_from_signature: CAPACT_CAPITAL_ACTION is
		--return empty instance of appropriate type for Current, with signature set
		do
			inspect capact_type_id
			when 1 then
				create {CAPACT_CASH_DIVIDEND} Result
			when 2 then
				create {CAPACT_FORWARD_SPLIT} Result
			when 3 then
				create {CAPACT_REVERSE_SPLIT} Result
			when 4 then
				create {CAPACT_BONUS_IN_SAME} Result
			when 5 then
				create {CAPACT_STOCK_DIVIDEND} Result
			when 6 then
				create {CAPACT_EXERCISE_RIGHTS} Result
			when 7 then
				create {CAPACT_CAPITAL_RETURN_OF_CASH} Result
			when 8 then
				create {CAPACT_RETURN_ON_SHARE_PREMIUM} Result
			when 9 then
				create {CAPACT_LEGACY_DISTRIBUTION} Result
			when 10 then
				create {CAPACT_BONUS_IN_OTHER} Result
			when 11 then
				create {CAPACT_RENOUNCEABLE_BONUS_IN_SAME} Result
			when 12 then
				create {CAPACT_RENOUNCEABLE_BONUS_IN_OTHER} Result
			when 13 then
				create {CAPACT_RIGHTS_IN_SAME} Result
			when 14 then
				create {CAPACT_RIGHTS_IN_OTHER} Result
			when 15 then
				create {CAPACT_RENOUNCEABLE_RIGHTS_IN_SAME} Result
			when 16 then
				create {CAPACT_RENOUNCEABLE_RIGHTS_IN_OTHER} Result
			when 17 then
				create {CAPACT_WRITEUP_OF_CAPITAL} Result
			when 18 then
				create {CAPACT_WRITEDOWN_OF_CAPITAL} Result
			when 19 then
				create {CAPACT_PUBLIC_OFFERING} Result
			when 20 then
				create {CAPACT_PRELIMINARY_SUBSCRIPTION} Result
			when 21 then
				create {CAPACT_REDENOMINATION} Result
			when 22 then
				create {CAPACT_RENOMINALIZATION} Result
			when 23 then
				create {CAPACT_CALL_PAYMENT} Result
			when 24 then
				create {CAPACT_STOCK_DIVIDEND_IN_OTHER} Result
			end
			Result.set_signature(Current)
		end

	set_from_override (a_signature: ROSE_CAPACT_OVERRIDE_SIGNATURE) is
			-- populate current from an override signature
		do
			instrument_id := a_signature.instrument_id
			market_id := a_signature.market_id
			exdate := a_signature.exdate
			member_number := a_signature.member_number
			capact_type_id := a_signature.capact_type_id
			record_date := a_signature.record_date
			company_id := a_signature.company_id
			listing_id := a_signature.listing_id
			if listing_id <= 0 then
				listing_id := listing.handle
			end
			if a_signature.status_code.is_equal("M") or a_signature.status_code.is_equal("A") then
				set_status_code(" ")
			elseif a_signature.status_code.is_equal("D") then
				set_status_code("Y")
			else
				set_status_code(a_signature.status_code)
			end
			source := "Override"
		end

	title_strings : ROSE_LINEAR_ARRAY[STRING] is
			--array of titles
		once
			create Result.make_from_array(<<"Company", "Market", "Vendor", "Option Number", "Capact Id", "Capact Type Id","Vendor Key", "Exdate","Record Date","Status Code">>)
		end

	signature_strings : ROSE_LINEAR_ARRAY[STRING] is
			-- signature strings
		do
			create Result.make_from_array(<<instrument_id.out,market_id.out,exdate.out,record_date.out,capact_type_id.out,company_id.out,listing_id.out, member_number.out, status_code>>)
		end

	ur_expression:SIGNATURE_TAGGED_EXPRESSION[CAPACT_SIGNATURE_RECORD] is
		once
			create result.make(Void)
		end
feature -- display

	display_signature: STRING is
			-- Readable output version of `Current'
		do
			Result :=  listing_id.out+" "+ exdate.out + " " + member_number.out  + " " + instrument_id.out + " " + market_id.out
		end

end

