indexing
	description: "rose capital actions subscriptions: Holder has to give up cash"
	author: "Janet Small"
	date: "$Date: 2014/02/26 $"
	revision: "$Revision: #8 $"
	archive: "$File: //rose/source/kernel/capital_actions/capact_subscription.e $"

deferred class
	CAPACT_SUBSCRIPTION
inherit
	CAPACT_CAPITAL_ACTION
	CAPACT_SUBSCRIPTION_TEMPLATE


feature -- Access

	cash_currency: CURRENCY
			-- Cash currency

	set_cash_currency(a_value: like cash_currency) is
			--
		do
			cash_currency := a_value
		end

	parent_price_saved: DOUBLE -- override rose price used for calculating basis prop if is_imc_subscribed, may equal or differ from rose best price
	set_parent_price_saved(a_value: like parent_price_saved) is
			--set override rose price used for calculating basis prop
		do
			parent_price_saved := a_value
		end

	parent_price_iso: STRING -- currency of override rose price.
	set_parent_price_iso(a_value: like parent_price_iso) is
			--set currency of override rose price
		do
			parent_price_iso := a_value
		end

	parent_basis_prop: DOUBLE -- for override of basis_prop if necessary, stored in item legacy_basis_prop
	set_parent_basis_prop(a_value: like parent_basis_prop) is
			--set parent basis prop for override of basis_prop if necessary, stored in item legacy_basis_prop
		do
			parent_basis_prop := a_value
		end



	is_imc_subscribe: BOOLEAN -- indicates whether IMC (Invest.Mgt.Co) has determined that portfolios will subscribe to this action
							-- used for rights in same (non-renounceable). any others?
	set_is_imc_subscribe(a_value: like is_imc_subscribe) is
			-- set value indicating whether IMC has determed portfolios will subscribe to this action
		do
			is_imc_subscribe := a_value
		end


feature -- Basic operations


	instantiate_subscription_data is
		-- retrieve data items from storage
		local
			s: STRING
			i: INTEGER
		do

			s := string_value(new_shares_subscription_expression,signature)
			if s /= Void and then s.is_double then
				set_new_shares_subscription(s.to_double)
			end

			s := string_value(old_shares_subscription_expression,signature)
			if s /= Void and then s.is_double then
				set_old_shares_subscription(s.to_double)
			end

			s := string_value(subscription_price_expression,signature)
			if s /= Void and then s.is_double then
				set_subscription_price(s.to_double)
			end

			s := string_value(subscription_currency_iso_expression,signature)
			if s /= Void then
				set_subscription_currency_iso(s)
				cash_currency := rose_context.currency_by_iso_code (s, constants.today_rdt)
				subscription_currency := cash_currency
			end


			s := string_value(subscription_rate_expression,signature)
			if s /= Void and then s.is_double then
				set_subscription_rate(s.to_double)
			end

			i := call_due_date_expression.integer_value(signature)
			if i > 0  then
				set_call_due_date(i)
			end

			i := last_date_to_subscribe_expression.integer_value(signature)
			if i > 0  then
				set_last_date_to_subscribe(i)
			end


			i := first_date_to_subscribe_expression.integer_value(signature)
			if i > 0  then
				set_first_date_to_subscribe(i)
			end
				--
			i := last_date_for_splitting_expression.integer_value(signature)
			if i > 0  then
				set_last_date_for_splitting(i)
			end

			i := issue_date_expression.integer_value(signature)
			if i > 0  then
				set_issue_date(i)
			end

			i := books_closed_date_expression.integer_value(signature)
			if i > 0  then
				set_books_closed_date(i)
			end


			s := string_value(is_oversubscription_allowed_expression,signature)
			if s /= Void and then s.as_upper.is_equal("T") then
				set_is_oversubscription_allowed(True)
			end
			s := string_value(is_underwritten_expression,signature)
			if s /= Void and then s.as_upper.is_equal("T") then
				set_is_underwritten(True)
			end

			s := string_value (parent_price_saved_expression,signature)
			if s /= Void and then s.is_double then
				set_parent_price_saved(s.to_double)
			end

			s := string_value(parent_price_iso_expression,signature)
			if s /= Void then
				set_parent_price_iso(s)
			end

			s := string_value(parent_basis_prop_expression,signature)
			if s /= Void and then s.is_double then
				set_parent_basis_prop(s.to_double)
			end

			s := string_value(is_imc_subscribe_expression,signature)
			if s /= Void and then s.as_upper.is_equal("T") then
				set_is_imc_subscribe (True)
			end
		end

	set_subscription_data_items is
			-- populate item tables for virtual record with subscription data
		local
			l_variable: CAPACT_VARIABLE_DESCRIPTION_ROW
			s: STRING
		do

			if new_shares_subscription > 0 then
				l_variable := variable_table.item("NEW_SHARES_SUBSCRIPTION")
				s := format_rate.formatted (new_shares_subscription)
				string_items.put(s, l_variable.item_id)
			end

			if old_shares_subscription > 0 then
				l_variable := variable_table.item("OLD_SHARES_SUBSCRIPTION")
				s := format_rate.formatted (old_shares_subscription)
				string_items.put(s, l_variable.item_id)
			end

			l_variable := variable_table.item("SUBSCRIPTION_PRICE")
			s := format_price.formatted (subscription_price)
			string_items.put(s, l_variable.item_id)

			if subscription_currency_iso /= Void then
				l_variable := variable_table.item("SUBSCRIPTION_CURRENCY_ISO")
				string_items.put(subscription_currency_iso, l_variable.item_id)
			end

			l_variable := variable_table.item("SUBSCRIPTION_RATE")
			s := format_price.formatted (subscription_rate)
			string_items.put(s, l_variable.item_id)


			l_variable := variable_table.item("CALL_DUE_DATE")
			integer_items.put(call_due_date, l_variable.item_id)

			l_variable := variable_table.item("LAST_DATE_TO_SUBSCRIBE")
			integer_items.put(last_date_to_subscribe, l_variable.item_id)

			l_variable := variable_table.item("FIRST_DATE_TO_SUBSCRIBE")

			l_variable := variable_table.item("LAST_DATE_FOR_SPLITTING")
			integer_items.put(last_date_for_splitting, l_variable.item_id)

			l_variable := variable_table.item("ISSUE_DATE")
			integer_items.put(issue_date, l_variable.item_id)

			l_variable := variable_table.item("BOOKS_CLOSED_DATE")
			integer_items.put(books_closed_date, l_variable.item_id)

			l_variable := variable_table.item("IS_OVERSUBSCRIPTION_ALLOWED")
			if  is_oversubscription_allowed = True then
				string_items.put ("T", l_variable.item_id)
			else
				string_items.put ("F", l_variable.item_id)
			end

			l_variable := variable_table.item("IS_UNDERWRITTEN")
			if  is_underwritten = True then
				string_items.put ("T", l_variable.item_id)
			else
				string_items.put ("F", l_variable.item_id)
			end

			l_variable := variable_table.item("PARENT_PRICE")
			s := format_price.formatted (parent_price_saved)
			string_items.put(s, l_variable.item_id)

			if parent_price_iso /= Void then
				l_variable := variable_table.item("PARENT_PRICE_ISO")
				string_items.put(parent_price_iso, l_variable.item_id)
			end

			if parent_basis_prop > 0 then
				l_variable := variable_table.item("LEGACY_BASIS_PROP")
				string_items.put(parent_basis_prop.out, l_variable.item_id)
			end

			l_variable := variable_table.item("IS_IMC_SUBSCRIBE")
			if  is_imc_subscribe = True then
				string_items.put ("T", l_variable.item_id)
			else
				string_items.put ("F", l_variable.item_id)
			end


----## where does this go -- it's in renouceable
--			l_variable := variable_table.item("DEALING_DATE")
--			integer_items.put(dealing_date, l_variable.item_id)			


		end

feature -- expressions

	old_shares_subscription_expression:ROSE_EXPRESSION [ROSE_ENTITY_DATE [ENTITY]] is
			--
		once
			Result := ur_target.expression_from_name("OLD_SHARES_SUBSCRIPTION")
		end

	new_shares_subscription_expression:ROSE_EXPRESSION [ROSE_ENTITY_DATE [ENTITY]] is
			--
		once
			Result := ur_target.expression_from_name("NEW_SHARES_SUBSCRIPTION")
		end


	subscription_price_expression:ROSE_EXPRESSION [ROSE_ENTITY_DATE [ENTITY]] is
			--
		once
			Result := ur_target.expression_from_name("SUBSCRIPTION_PRICE")
		end

	subscription_currency_iso_expression:ROSE_EXPRESSION [ROSE_ENTITY_DATE [ENTITY]] is
			--
		once
			Result := ur_target.expression_from_name("SUBSCRIPTION_CURRENCY_ISO")
		end

	subscription_rate_expression:ROSE_EXPRESSION [ROSE_ENTITY_DATE [ENTITY]] is
			--
		once
			Result := ur_target.expression_from_name("SUBSCRIPTION_RATE")
		end

	call_due_date_expression:ROSE_EXPRESSION [ROSE_ENTITY_DATE [ENTITY]] is
			--
		once
			Result := ur_target.expression_from_name("CALL_DUE_DATE")
		end

	last_date_to_subscribe_expression:ROSE_EXPRESSION [ROSE_ENTITY_DATE [ENTITY]] is
			--
		once
			Result := ur_target.expression_from_name("LAST_DATE_TO_SUBSCRIBE")
		end

	first_date_to_subscribe_expression:ROSE_EXPRESSION [ROSE_ENTITY_DATE [ENTITY]] is
			--
		once
			Result := ur_target.expression_from_name("FIRST_DATE_TO_SUBSCRIBE")
		end
	last_date_for_splitting_expression:ROSE_EXPRESSION [ROSE_ENTITY_DATE [ENTITY]] is
			--
		once
			Result := ur_target.expression_from_name("LAST_DATE_FOR_SPLITTING")
		end

	issue_date_expression:ROSE_EXPRESSION [ROSE_ENTITY_DATE [ENTITY]] is
			--issue date expression
		once
			Result := ur_target.expression_from_name("ISSUE_DATE")
		end
	books_closed_date_expression:ROSE_EXPRESSION [ROSE_ENTITY_DATE [ENTITY]] is
			-- Books closed date expression
		once
			Result := ur_target.expression_from_name("BOOKS_CLOSED_DATE")
		end


	is_oversubscription_allowed_expression:ROSE_EXPRESSION [ROSE_ENTITY_DATE [ENTITY]] is
			--
		once
			Result := ur_target.expression_from_name("IS_OVERSUBSCRIPTION_ALLOWED")
		end

	is_underwritten_expression:ROSE_EXPRESSION [ROSE_ENTITY_DATE [ENTITY]] is
			-- expression to retrieve string boolean indicating whether is underwritten
		once
			Result := ur_target.expression_from_name("IS_UNDERWRITTEN")
		end


	dealing_date_expression:ROSE_EXPRESSION [ROSE_ENTITY_DATE [ENTITY]] is
			-- expression to retrieve dealing date
		once
			Result := ur_target.expression_from_name("DEALING_DATE")
		end

	parent_price_saved_expression:ROSE_EXPRESSION [ROSE_ENTITY_DATE [ENTITY]] is
			--expression for retrieving parent price (save appended to name to avoid conflicts)
		once
			Result := ur_target.expression_from_name("PARENT_PRICE")
		end

	parent_price_iso_expression:ROSE_EXPRESSION [ROSE_ENTITY_DATE [ENTITY]] is
			-- expression for retrieving iso code of currency of parent price
		once
			Result := ur_target.expression_from_name("PARENT_PRICE_ISO")
		end

	parent_basis_prop_expression:ROSE_EXPRESSION [ROSE_ENTITY_DATE [ENTITY]] is
			-- expression for retrieving iso code of currency of parent price
		once
			Result := ur_target.expression_from_name("LEGACY_BASIS_PROP")
		end

	is_imc_subscribe_expression:ROSE_EXPRESSION [ROSE_ENTITY_DATE [ENTITY]] is
			-- expression for retrieving string boolean indicating whether IMC has chosen to subscribe
		once
			Result := ur_target.expression_from_name("IS_IMC_SUBSCRIBE")
		end

feature {NONE} -- Implementation


end
