indexing
	description: "capital action Expression in which ordering of component expressions is not known"
	author: "Janet Small"
	date: "$Date: 2009/11/06 $"
	revision: "$Revision: #8 $"
	archive: "$File: //rose/source/kernel/capital_actions/expressions/capact_unknown_expression.e $"

class

	CAPACT_UNKNOWN_EXPRESSION
inherit
	CAPACT_EXPRESSION

	CAPACT_COMMON
			undefine default_create end
create
	make_with_expressions

feature -- creation

	make_with_expressions(a_inner,a_outer:CAPACT_EXPRESSION) is
			-- initialize component expressions and compute rmdat_adjustment_factor
			-- unknown expression should have an option_status=E for exception
			-- but if not, it should have no effect on returns or adjustments
		require
			has_component_expressions: a_inner /= Void and a_outer /= Void
		do
			set_expression_index
			inner_expression := a_inner
			outer_expression := a_outer
			rmdat_adjustment_factor := 1 --has no adjustment effect
		ensure
			has_expressions : inner_expression /= Void and outer_expression /= Void
		end

feature -- Access

	processing_instruction (
		a_basis: RP_CORPORATE_ACTION_PROCESSING_SHARE_BASIS_TYPE;
		a_processing_system: RP_CORPORATE_ACTION_PROCESSING_SYSTEM): RP_CORPORATE_ACTION_PROCESSING_INSTRUCTIONS is
			-- <Precursor>
		do
		    create Result.make (a_basis, create {RP_SIMPLE_CORPORATE_ACTION}.make (exdate, a_processing_system.unknown_instructions (Current)))
		end

	contract(a_corporate_action_processing_system: RP_CORPORATE_ACTION_PROCESSING_SYSTEM; a_portfolio:RP_PORTFOLIO; a_holding: RP_QUANTITY):RP_PORTFOLIO_CORPORATE_ACTION_COMPOUND_CONTRACT is
			-- flows for corporate action contract
		do
			create Result.make(inner_expression,outer_expression,a_corporate_action_processing_system, a_portfolio, a_holding)
		end

	inner_expression:CAPACT_EXPRESSION -- first expression to be evaluated

	outer_expression :CAPACT_EXPRESSION -- following expression to be applied only to additional shares from first expression

--	computed_portfolio(a_portfolio_date:PORTFOLIO_DATE):PORTFOLIO_DATE is
--	do
--		if inner_expression /=Void then
--			Result := outer_expression.computed_portfolio(inner_expression.computed_portfolio(a_portfolio_date))
--		end	
--	end
--	
--	computed_delta_portfolio(a_portfolio_date:PORTFOLIO_DATE):PORTFOLIO_DATE is
--	do
--		Result := a_portfolio_date - computed_portfolio(inner_expression.computed_portfolio(a_portfolio_date))
--	end

	name : STRING is
			-- description of expression
		do
			Result := outer_expression.name + " unknown " + 	inner_expression.name
		end

	cash_outflow(a_currency:CURRENCY):DOUBLE is
			--unknown cash outflow -> none
		do
--			if is_defined_double (outer_expression.cash_outflow (a_currency)) and
--			   is_defined_double (inner_expression.cash_outflow(a_currency)) and
--			   is_defined_double (inner_expression.delta_new_shares) then
--			   	if inner_expression.delta_new_shares = 0 then
--			   		Result := outer_expression.cash_outflow(a_currency)*inner_expression.rmdat_adjustment_factor + inner_expression.cash_outflow(a_currency)
--			   	else
--					Result := outer_expression.cash_outflow(a_currency)*inner_expression.delta_new_shares + inner_expression.cash_outflow(a_currency)
--				end
--			end
		end

	cash_inflow(a_currency:CURRENCY):DOUBLE is
			--unknown cash inflow -> none
		do
--			if is_defined_double (outer_expression.cash_inflow (a_currency)) and
--			   is_defined_double (inner_expression.cash_inflow(a_currency)) and
--			   is_defined_double (inner_expression.delta_new_shares) then
--			   	if inner_expression.delta_new_shares = 0 then
--			   		Result := outer_expression.cash_inflow(a_currency)*inner_expression.rmdat_adjustment_factor + inner_expression.cash_inflow(a_currency)
--			   	else
--					Result := outer_expression.cash_inflow(a_currency)*inner_expression.delta_new_shares + inner_expression.cash_inflow(a_currency)
--				end
--			end
		end

	cash_dividend(a_currency:CURRENCY):DOUBLE is
			--unknown cash dividend -> none
		do
--			if is_defined_double (outer_expression.cash_inflow (a_currency)) and
--			   is_defined_double (inner_expression.cash_inflow(a_currency)) and
--			   is_defined_double (inner_expression.delta_new_shares) then
--			   	if inner_expression.delta_new_shares = 0 then
--			   		Result := outer_expression.cash_inflow(a_currency)*inner_expression.rmdat_adjustment_factor + inner_expression.cash_inflow(a_currency)
--			   	else
--					Result := outer_expression.cash_inflow(a_currency)*inner_expression.delta_new_shares + inner_expression.cash_inflow(a_currency)
--				end
--			end
		end

feature -- Status

--	invalid_delta_should_be_compound: BOOLEAN is
--			--
--		do
--			Result := inner_expression.
--		end


feature--capact_capital_action

	instantiate_data is
		-- not implemented, deferred from capact_capital_action
		do
		end

	template_name:STRING  is "DELTA EXPRESSION"

	set_data_items is
		-- not implemented, deferred from capact_capital_action
		do
		end

	update_data_items is
		-- not implemented, deferred from capact_capital_action
		do
		end

feature -- invalid from CAPACT_EXPRESSION

--	cash_rate : DOUBLE is
--			-- does it make sense to define this here?
--		do
--			Result := undefined_double
--		end

--	cash_currency_iso : STRING is
--			-- not defined
--		do
--			Result := Void
--		end

--	adjustment_factor : DOUBLE is
--			-- This may be redundant
--		do
--			Result := rmdat_adjustment_factor
--		end
--	share_factor : DOUBLE is
--				-- not defined
--		do
--			Result := undefined_double
--		end

--	subscription_price : DOUBLE is
--				-- not defined
--		do
--			Result := undefined_double
--		end

--	child_units : DOUBLE is
--			-- not defined
--		do
--			Result := undefined_double
--		end

--	legacy_basis_prop : DOUBLE is
--			-- not defined	
--		do
--			Result := undefined_double
--		end

--	legacy_basis_price : DOUBLE is
--			-- not defined
--		do
--			Result := undefined_double
--		end

--	bonus_rate : DOUBLE is
--			-- not defined
--		do
--			Result := undefined_double
--		end

--	dividend_rate : DOUBLE is
--			-- not defined
--		do
--			Result := undefined_double
--		end

--	split_rate : DOUBLE is
--			-- not defined
--		do
--			Result := undefined_double
--		end

----	comment : STRING is
----		do
----			Result := Void
----		end
----		
--	legacy_basis_currency : STRING is
--			-- not defined	
--		do
--			Result := Void
--		end

end -- class CAPACT_UNKNOWN_EXPRESSION
