indexing
	description: "Implements the check ratio used in FORTRAN routines in the legacy VMS code"
	author: "Mark Howard"
	date: "$Date: 2008/12/01 $"
	revision: "$Revision: #8 $"

class
	CHECK_RATIO

inherit
	ROSE_IEEE_UNDEFINED

create
	make

feature -- Settings

	set_precision (a_precision: DOUBLE)
			-- sets precision
		require
			precision_not_zero: a_precision /= 0
		do
			precision := a_precision
		end

feature -- Basic Operations

	make (a_precision: REAL_64)
			-- make setting tolerance for detecting a difference
		require
			precision_not_zero: a_precision /= 0
		do
			precision := a_precision
		end

	fit (a_1, a_2: REAL_64): REAL_64
			-- compute the absolute ratio difference between a_1,a_2
		local
			l_denom: REAL_64
			l_diff: REAL_64
			l_1, l_2: REAL_64
		do
			if is_defined_double (a_1) then
				l_1 := a_1
			end
			if is_defined_double (a_2) then
				l_2 := a_2
			end
			l_denom := (l_1.abs + l_2.abs) / 2
			l_diff := (l_1 - l_2).abs
			if l_denom > 0 then
				Result := l_diff / l_denom
				if l_diff < .02 * precision then
					Result := Result.min (.001)
				end
			end
		end

	signed_diff (a_1, a_2: REAL_64): REAL_64
			-- signed difference ratio (percentage) between a_1, a_2
		local
			l_denom: REAL_64
			l_diff: REAL_64
			l_1, l_2: REAL_64
		do
			if is_defined_double (a_1) then
				l_1 := a_1
			end
			if is_defined_double (a_2) then
				l_2 := a_2
			end
			l_denom := (l_1.abs + l_2.abs) / 2
			l_diff := (l_1 - l_2)
			if l_denom > 0 then
				Result := l_diff / l_denom
			end
		end

	precision: REAL_64
			-- tolerance, absolute difference below precision is returned as 0

end -- class CHECK_RATIO

