note

	description:

		"Required fields for a clustered signature tagged item manager"

	author: "Mauricio Bustos"
	last_editor: "$Author: rj $"
	date: "$Date: 2015/05/20 $"
	revision: "$Revision: #20 $"
	path: "$File: //rose/source/kernel/persistence/databases/signature_tagged/reference/clustered_signature.e $"

deferred class
   CLUSTERED_SIGNATURE[G -> COMPARABLE, H -> COMPARABLE, reference I -> ENTITY]

inherit

	ROSE_SIGNATURE_RECORD[I]
		redefine
			records,
			historical_records
		end

feature -- Status report

	is_clustered: BOOLEAN
			-- Is the signature tagged item manager clustered?
		do
			Result := True
		end

feature  --Keys

	key_0 : G is
		deferred
		end

	key_0_name : STRING is
			--name of the first key
		deferred
		ensure
			valid_result : Result /= Void and not Result.is_empty
			in_signature : fields_.as_lower.has_substring(Result.as_lower) or data_fields_string_.as_lower.has_substring(Result.as_lower)
		end

	key_1 : H is
		deferred
		end

	key_1_name : STRING is
		deferred
		ensure
			valid_result : Result /= Void and not Result.is_empty
			in_signature : fields_.as_lower.has_substring(Result.as_lower) or data_fields_string_.as_lower.has_substring(Result.as_lower)
		end

feature -- Retrieval

	records(a_targets : ARRAY[like Current]; a_statement : STRING) : ROSE_LINEAR_ARRAY[ROSE_VIRTUAL_RECORD[like signature_template]] is
			-- <Precursor>
		local
			l_record : ROSE_VIRTUAL_RECORD[like signature_template]
		do
			if is_clustered then
				create Result.make_empty
				targets_ ?= a_targets
				if targets_ = Void then
					create targets_.make_from_array(a_targets)
				end
				from targets_.start until targets_.after loop
					l_record := targets_.current_item.data_manager.record_by_signature(targets_.current_item)
					if l_record /= Void then
						Result.extend(l_record)
					end
					targets_.forth
				end
				if Result /= Void then
					Result.do_all(agent reconcile_record)
				end
			else
				Result := Precursor (a_targets, a_statement)
			end
		end

	historical_records (a_targets: ARRAY [like Current]; a_statement: STRING): ROSE_LINEAR_ARRAY [ROSE_VIRTUAL_RECORD [like signature_template]] is
			-- <Precursor>
		local
			l_record : ROSE_VIRTUAL_RECORD[like signature_template]
			l_signature : like Current
			l_clustered_manager : CLUSTERED_TAGGED_ITEM_DATABASE[G,H,like signature_template]
		do
			if is_clustered then
				if not a_targets.is_empty and then a_targets.item(1) /= Void then
					l_clustered_manager ?= a_targets.item(1).data_manager
				end
				if l_clustered_manager = Void then
					Result ?= Precursor(a_targets,a_statement)
				else
					create Result.make_empty
					targets_ ?= a_targets
					if targets_ = Void then
						create targets_.make_from_array(a_targets)
					end
					from targets_.start until targets_.after loop
						l_signature ?= targets_.current_item
						l_record := l_clustered_manager.historical_record_by_signature(l_signature)
						if l_record /= Void then
							Result.extend(l_record)
						end
						targets_.forth
					end
					if Result /= Void then
						Result.do_all(agent reconcile_record)
					end
				end
			else
				Result := Precursor (a_targets, a_statement)
			end
		end

feature {CLUSTERED_TAGGED_SERVER_CLIENT} -- Implementation

	signature_for_persistent_server: CLUSTERED_SIGNATURE[G, H, I]
	-- Returns a signature with only the minimum attributes needed by the persistent server.  This is because
	-- some descendants add a lot of junk.  In one case as much as 8 MB.  Your implementation should be in the
	-- class (or descendant) that defines 'fields_'.  The return type should be the same as the class that
	-- defines 'fields_'; not a descendant and not "like Current".  Failure to use the proper type will result
	-- in a run-time error in CLUSTERED_TAGGED_SERVER_CLIENT.set_query_signatures (make sure to test this).
	-- The attributes to be copied are those mentioned in 'fields_'.  To be safe, I also copy 'signature_id' and
	-- 'path'.  References should not be twinned since Result is merely sent to the server and then discarded.
	-- No other attributes should be set since the server would not know what to do with them.
	do
		Result := Current
	end

end
