note

	description:

		"Manager for a clustered signature tagged item database"

	keywords: "database,signature,item,tagged,manager"
	last_updater: "$Author: rj $"
	author: "Mauricio Bustos"
	last_editor: "$Author: rj $"
	date: "$Date: 2015/03/25 $"
	revision: "$Revision: #67 $"
	path: "$File: //rose/source/kernel/persistence/databases/signature_tagged/clustered_tagged_item_database.e $"

class
	CLUSTERED_TAGGED_ITEM_DATABASE[G -> COMPARABLE,
								   H -> COMPARABLE,
								   reference I -> CLUSTERED_SIGNATURE[G,H,ENTITY] create default_create end]

inherit

	SIGNATURE_TAGGED_ITEM_DATABASE[I]
		export
			{INTEGER_INTEGER_CLUSTER_VIEW,INTEGER_DATE_CLUSTER_VIEW,GLOBAL_MODEL_UNIVERSE} manager_cache
		redefine
			path,
			signature_id,
			cube_from_signatures,
			record_by_signature,
			historical_records_by_signature,
			historical_record_by_signature,
			signature_tagged_server,
			retrieve_current_signatures,
			retrieve_records_by_signatures,
			retrieve_historical_signatures,
			retrieve_virtual_records,
			initial_cache_size,
			set_server_for_writing,
			enable_testenv_overlay,
			disable_testenv_overlay
		end
	DATABASE_SERVER
		export
			{NONE} all
		end

create
	make,
	make_named,
	make_remote,
	make_persistent_remote,
	make_with_process_manager,
	make_named_with_process_manager

feature -- Initialization

	make_remote (a_data_name, a_process_name, a_network_server_name: STRING; a_args: ARRAY[STRING]; a_executable_name: STRING) is
			-- Create the database connecting to a dedicated server
		require
			valid_network_server_name: a_network_server_name /= Void
			valid_executable_name: a_executable_name /= Void
		local
			l_remote_client: CLUSTERED_TAGGED_SERVER_CLIENT[G, H, I]
		do
			max_cube_result_size := 20_000
			data_name := a_data_name
			create l_remote_client.make (a_network_server_name, a_args, a_executable_name, version)
			signature_tagged_server := l_remote_client
			initialize_manager (a_process_name, Void)
		end

	make_persistent_remote (a_data_name, a_process_name, a_network_server_name: STRING; a_args: ARRAY[STRING]; a_executable_name: STRING) is
			-- Create the database connecting to a persistent server
		require
			valid_network_server_name: a_network_server_name /= Void
			valid_executable_name: a_executable_name /= Void
		local
			l_remote_client: CLUSTERED_TAGGED_SERVER_CLIENT[G, H, I]
			l_env: EXECUTION_ENVIRONMENT
			l_server_list, l_persistent_server_list: STRING
			l_server_list_predicate: STRING_PREDICATE
			l_executable_name, l_server_delivery: STRING
			l_network_server_name: STRING
		do
			create l_env

			if attached l_env.get(a_network_server_name + "_NAME") as al_network_server_name
			then
				-- For testing you can supply your own name here.  Enter that name in the persistent_server table in the repository
				-- to use a private port_server, and therefore, a private executable.
				l_network_server_name := al_network_server_name
			else
				l_network_server_name := a_network_server_name
			end

			l_server_list := l_env.get ("NT_SERVER_LIST")
			l_persistent_server_list := l_env.get ("PERSISTENT_SERVER_LIST")
			if l_persistent_server_list /= Void then
				l_env.put (l_persistent_server_list,"NT_SERVER_LIST")
			else
				create l_server_list_predicate.make (servers.repository_server,"select machine from persistent_servers where name = ? and enabled = 1")
				if not l_server_list_predicate.success then
					raise ("Could not create l_server_list_predicate")
				end
				l_server_list_predicate.retrieve (Void, <<l_network_server_name>>)
				if not l_server_list_predicate.success then
					raise ("Could not retrieve with l_server_list_predicate")
				end
				from l_server_list_predicate.array.start until l_server_list_predicate.array.off loop
					if l_persistent_server_list = Void then
						l_persistent_server_list := l_server_list_predicate.array.current_item
					else
						l_persistent_server_list := l_persistent_server_list + "," + l_server_list_predicate.array.current_item
					end
					l_server_list_predicate.array.forth
				end
				if l_persistent_server_list = Void then
					raise ("Empty PERSISTENT_SERVER_LIST variable, check repository")
				end
				l_env.put (l_persistent_server_list,"NT_SERVER_LIST")
			end
			l_executable_name := a_executable_name.twin
			l_server_delivery := server_delivery (l_network_server_name)
			if l_server_delivery /= Void and not l_server_delivery.is_empty then
				output (global.interpreted ("($now) Using delivery $1 for $2%N"), <<l_server_delivery.as_upper, l_network_server_name>>)
				if not l_server_delivery.item (l_server_delivery.count).is_equal ('\') THEN
					l_server_delivery := l_server_delivery.twin + "\"
				end
				l_executable_name.prepend (l_server_delivery)
			end
			max_cube_result_size := 20_000
			data_name := a_data_name
			create l_remote_client.make_persistent (l_network_server_name, a_args, l_executable_name, version)
			signature_tagged_server := l_remote_client
			initialize_manager (a_process_name, Void)
			initialize_manager (a_process_name + "[" + Global.app.platform.name + "]", Void)
			if l_server_list/=void then
				l_env.put (l_server_list,"NT_SERVER_LIST")
			end
		end

feature	-- Testenv

	set_server_for_writing (a_server: SQL_DATA_SERVER) is
			-- This specifies the server that will be used to write any deltas
			-- This is intended to be used only for remote clustered tagged item database instances
		require else
			valid_server: a_server /= Void
		local
			l_table_structure: SIGNATURE_TAGGED_TABLE_STRUCTURE
			l_overlay_enabled: BOOLEAN
		do
			create l_table_structure.make (a_server, data_name, Void)
			if l_table_structure.data_available then
				signature_tagged_server.make (l_table_structure)
				signature_tagged_server.set_signature_item_server (a_server)
				signature_tagged_server.check_full_update_mode
				l_overlay_enabled := config.boolean (data_server_logical_name + "_enable_overlay",False)
				if l_overlay_enabled then
					enable_testenv_overlay
				end
			end
		end

	enable_testenv_overlay is
			--enable overlay for persistent remote database
		local
			l_remote_client: CLUSTERED_TAGGED_SERVER_CLIENT[G, H, I]
		do
			l_remote_client ?= signature_tagged_server
			if l_remote_client /= Void then
				signature_tagged_server.enable_testenv_overlay
			else
				raise ("Testenv Overlay is only allowed with a persistent remote STID")
			end
		end

	disable_testenv_overlay is
			--disable overlay for persistent remote database
		do
			signature_tagged_server.disable_testenv_overlay
		end

feature -- Access

	path_tree: PARTITION_TREE[G,H] is
			-- The path tree used to find the cluster id for a signature
		do
			Result := signature_tagged_server.path_tree
		end

	ad_hoc_record_by_signature (a_signature: I): ROSE_VIRTUAL_RECORD[I] is
			-- Retrieve a record bypassing the clustering, cache results
		require
			valid_signature: a_signature /= Void
		local
			l_records: ROSE_LINEAR_ARRAY[ROSE_VIRTUAL_RECORD[I]]
			l_state: BOOLEAN
		do
			if last_ad_hoc_rose_virtual_records /= Void and then
				last_ad_hoc_rose_virtual_records.has (a_signature) then
				Result := last_ad_hoc_rose_virtual_records.item (a_signature)
			else
				l_state := signature_tagged_server.ad_hoc_query
				signature_tagged_server.set_ad_hoc_query (True)
				l_records := retrieve_signature (a_signature,a_signature.selective_restriction_statement,Void)
				if l_records.count = 1 then
					Result := l_records.first
				end
				if last_ad_hoc_rose_virtual_records = Void then
					create last_ad_hoc_rose_virtual_records.make (100)
				end
				last_ad_hoc_rose_virtual_records.force (Result, a_signature)
				signature_tagged_server.set_ad_hoc_query (l_state)
			end
		end

	set_ad_hoc_query (a_value: BOOLEAN) is
			-- Set `ad_hoc_query' to `a_value'
		do
			signature_tagged_server.set_ad_hoc_query (a_value)
		end

	retrieve_current_signatures (a_signatures: ROSE_LINEAR_ARRAY [I]; a_statement: STRING): ROSE_LINEAR_ARRAY[I] is
		do
			signature_tagged_server.set_ad_hoc_query (True)
			Result := Precursor (a_signatures,a_statement)
			signature_tagged_server.set_ad_hoc_query (False)
		end

	retrieve_historical_signatures (a_signatures: ROSE_LINEAR_ARRAY [I]; a_statement: STRING): ROSE_LINEAR_ARRAY[I] is
		do
			signature_tagged_server.set_ad_hoc_query (True)
			Result := Precursor (a_signatures,a_statement)
			signature_tagged_server.set_ad_hoc_query (False)
		end

	retrieve_virtual_records (a_historical_mode: BOOLEAN; a_signatures: ROSE_LINEAR_ARRAY [I]; a_items: ROSE_LINEAR_ARRAY [INTEGER]; a_template: ROSE_VIRTUAL_RECORD [I]): ROSE_LINEAR_ARRAY [ROSE_VIRTUAL_RECORD [I]]
			-- This routine retrieve virtual records without changing the existing settings and caches
		do
			signature_tagged_server.set_ad_hoc_query (True)
			Result := Precursor (a_historical_mode, a_signatures, a_items, a_template)
			signature_tagged_server.set_ad_hoc_query (False)
		end

	record_by_signature (a_signature: I): ROSE_VIRTUAL_RECORD[I] is
		local
			l_records: ROSE_LINEAR_ARRAY[ROSE_VIRTUAL_RECORD[I]]
		do
			if signature_tagged_server.ad_hoc_query then
				Result := ad_hoc_record_by_signature (a_signature)
			else
				if not signature_tagged_server.historical_mode and
				   last_rose_virtual_record /= Void and then
				   last_rose_virtual_record.is_signature_equal(a_signature)
				then
					Result := last_rose_virtual_record
				else
					set_results_from_cache (a_signature)
					if cube_result_table /= Void then
						Result := record_from_cube_result_table_and_overlay (a_signature)
					else
						l_records := retrieve_signature (a_signature,Void,Void)
						if l_records.count=0 then
							signature_tagged_server.remake_virtual_record_set
							signature_tagged_server.set_virtual_record_set_cube_result_temp (l_records)
							signature_tagged_server.set_cache_from_results (new_record(a_signature), Current)
							signature_tagged_server.set_virtual_record_set_cube_result_temp (Void)
						else
							set_results_from_cache (a_signature)
							if cube_result_table /= Void then
								Result := record_from_cube_result_table_and_overlay (a_signature)
							end
						end
					end
					last_rose_virtual_record := Result
				end
			end
		end

	new_record(a_signature : I) : ROSE_VIRTUAL_RECORD[I]
			-- Vritual record for `a_signature'
		do
			create Result.make(a_signature)
		ensure
			not_void: Result /= Void
		end

	retrieve_records_by_signatures (a_signatures: ROSE_LINEAR_ARRAY [I]; a_historical_mode: BOOLEAN; a_items: ROSE_LINEAR_ARRAY [INTEGER]): ROSE_LINEAR_ARRAY [ROSE_VIRTUAL_RECORD [I]] is
		require else
			signature_not_void: a_signatures /= Void
		local
			l_record: ROSE_VIRTUAL_RECORD [I]
		do
			if a_historical_mode then
				signature_tagged_server.set_historical_mode
			end
			create Result.make_empty
			from a_signatures.start until a_signatures.off loop
				l_record := record_by_signature (a_signatures.current_item)
				if l_record /= Void then
					Result.extend (l_record)
				end
				a_signatures.forth
			end
			if a_historical_mode then
				signature_tagged_server.clear_historical_mode
			end
		end

	historical_record_by_signature (a_signature: I): ROSE_VIRTUAL_RECORD[I] is
			-- Retrieve the historical record keyed by a_signature using as_of_date to get the requested version
		do
			signature_tagged_server.set_historical_mode
			Result := record_by_signature (a_signature)
			signature_tagged_server.clear_historical_mode
		end

	historical_records_by_signature (a_signature: I; a_restriction_statement: STRING; a_items: ROSE_LINEAR_ARRAY [INTEGER]): ROSE_LINEAR_ARRAY[ROSE_VIRTUAL_RECORD[I]] is
		local
			l_records: ROSE_LINEAR_ARRAY[ROSE_VIRTUAL_RECORD[I]]
			--l_timer: ROSE_PROCESS_TIMER
			l_signature_id: INTEGER
			l_complete: BOOLEAN
			l_start, l_end: INTEGER
		do
			set_results_from_cache (a_signature)
			if cube_result_table /= Void and not cube_result_table.is_empty and
				cube_result /= Void and not cube_result.is_empty then
				from cube_result.start until cube_result.after or l_complete loop
					if (l_signature_id /= 0 and
						cube_result.current_item.signature.signature_id = l_signature_id) or else
						cube_result.current_item.signature.multi_key.is_equal (a_signature.multi_key) then
						if l_start = 0 then
							l_signature_id := cube_result.current_item.signature.signature_id
							l_start := cube_result.index
						end
						l_end := cube_result.index
					elseif l_end > 0 then
						l_complete := True
					end
					cube_result.forth
				end
				if l_start > 0 and l_end >= l_start then
					l_records := cube_result.subarray (l_start,l_end)
					create Result.make_empty
					from l_records.start until l_records.off loop
						Result.extend (l_records.current_item.twin)
						l_records.forth
					end
				end
			else
				--create l_timer
				if a_restriction_statement /= Void then
					signature_tagged_server.set_ad_hoc_query (True)
				end
				signature_tagged_server.set_historical_mode
				l_records := retrieve_signature (a_signature,a_restriction_statement,a_items)
				signature_tagged_server.clear_historical_mode
				if a_restriction_statement /= Void then
					signature_tagged_server.set_ad_hoc_query (False)
					Result := l_records
				else
					if l_records.count = 0 then
						signature_tagged_server.remake_virtual_record_set
						signature_tagged_server.set_virtual_record_set_cube_result_temp (l_records)
						signature_tagged_server.set_cache_from_results (new_record(a_signature), Current)
						signature_tagged_server.set_virtual_record_set_cube_result_temp (Void)
					else
						Result := historical_records_by_signature (a_signature,a_restriction_statement, a_items)
					end
				end
				--create_cache_variables
				--l_timer.display (path (a_signature).out)
			end
		end

feature {NONE} -- Implementation

	record_from_cube_result_table_and_overlay (a_signature: I): like record_by_signature is
			-- Record for `a_signature' from `cube_result_table' and database overlay
		require
			cube_result_table_not_void: a_signature /= Void implies cube_result_table /= Void
		local
			l_key: MULTI_KEY
		do
			if a_signature /= Void then
				if signature_tagged_server.historical_mode and then a_signature.as_of_date /= Void then
					l_key := a_signature.multi_key.deep_twin
					l_key.array_force (a_signature.as_of_date, l_key.key_count + 1)
				else
					l_key := a_signature.multi_key
				end
				cube_result_table.search (l_key)
				if cube_result_table.found then
					Result := cube_result_table.found_item
				end
				if signature_tagged_server.testenv_overlay_enabled and then Result /= Void then
					Result := signature_tagged_server.testenv_database.record_overlayed_with_testenv_data (Result)
				end
			end
		end

	cube_from_signatures (a_signatures: ROSE_LINEAR_ARRAY[I]; a_columns: ROSE_LINEAR_ARRAY[STRING]): COLUMN_DESCRIPTORS[I] is
			-- <Precursor>
		do
			if signature_tagged_server.ad_hoc_query then
				Result := Precursor (a_signatures,a_columns)
			else
				signature_tagged_server.set_query_signatures (a_signatures)
			end
		end

	version: STRING is	-- The descendant can append additional version info.
		do
			Result := "1"	-- This is the version of the database.
		end

feature {STID_TESTENV_VIEW,STID_TESTENV_DATABASE} -- Signature and Path

	path (a_signature: I): INTEGER is
		local
			l_signature: I
			l_last_leaf: PARTITION_TREE_RECORD[G, H]
		do
			l_signature := a_signature
			l_last_leaf := signature_tagged_server.path_leaf (l_signature)
			Result := l_last_leaf.path
			if last_path /= Result then
				last_path := Result
				if not signature_tagged_server.ad_hoc_query	then
					adjust_cache_parameters (l_signature)
				end
			end
		end

feature {NONE} -- Signature and Path

	signature_id (a_signature: I; a_can_read_old_signature, a_can_write_new_signature: BOOLEAN): INTEGER
			-- <Precursor>
		do
			if last_signature /= Void and then
			   (last_signature.key_1 = a_signature.key_1 and last_signature.key_0 = a_signature.key_0) and then
			   last_signature.multi_key.is_equal (a_signature.multi_key) then
				Result := last_signature_id
			else
				Result := signature_tagged_server.signature_id (a_signature, a_can_read_old_signature, a_can_write_new_signature)
				if Result /= 0
				then
					last_signature_id := Result
					last_signature := a_signature
				else
					last_signature := Void
				end
			end
		end

	adjust_cache_parameters (a_signature: CLUSTERED_SIGNATURE[G,H,ENTITY]) is
			-- Set how much the manager cache should be keeping around
		require
			valid_signature: a_signature /= Void
		local
			l_cache_size: INTEGER
			l_new_count: INTEGER
		do
			if ready_for_adjust or paths_attempted_for_direction > Direction_determination_count
			then
				if not ready_for_adjust
				then
					paths_attempted_for_direction := 0
					ready_for_adjust := True
				end

				if not direction_determined then
					if paths_attempted_for_direction = 0 then
						query_0 := a_signature.key_0
						query_1 := a_signature.key_1
					elseif paths_attempted_for_direction > direction_determination_count then
						direction_is_key_0 := key_1_changes > key_0_changes
						direction_determined := True
					else
						if query_0 /= a_signature.key_0 then
							key_0_changes := key_0_changes + 1
						end
						if query_1 /= a_signature.key_1 then
							key_1_changes := key_1_changes + 1
						end
					end
					paths_attempted_for_direction := paths_attempted_for_direction + 1
				else
					if direction_is_key_0 then
						l_cache_size := path_tree.key_0_leaves_count (a_signature.key_0)
					else
						l_cache_size := path_tree.key_1_leaves_count (a_signature.key_1)
					end

					if l_cache_size = 0 then
						l_new_count := manager_cache.maximum_count
					else
						l_new_count := l_cache_size * cache_multiplier
					end

					if max_allowable_count /= 0 and then l_new_count > max_allowable_count
					then
						l_new_count := max_allowable_count
					end
					if l_new_count <= 0 then
						output (global.interpreted ("($now) Refuse to set manager_cache.maximum_count to $1.%N"), <<l_new_count>>)
					else
						if l_new_count > manager_cache_count_lower_limit then
							manager_cache.set_maximum_count (l_new_count)
						end
					end
					direction_determined := False
					key_0_changes := 0
					key_1_changes := 0
					paths_attempted_for_direction := 0
					ready_for_adjust := False
				end
			else
				if (direction_is_key_0 and query_0 /= a_signature.key_0) or
					(not direction_is_key_0 and query_1 /= a_signature.key_1) then
					paths_attempted_for_direction := paths_attempted_for_direction + 1
				end
			end
		end

	manager_cache_count_lower_limit: INTEGER = 50

	max_allowable_count: INTEGER is
			-- By setting the config value, logical or environment variable MAX_STID_CACHE_COUNT you can prevent the cache from growing too large.
		local
			l_max_string: STRING
		once
			l_max_string := config.value("MAX_STID_CACHE_COUNT")
			if l_max_string=Void or else l_max_string.count=0
			then
				l_max_string := global.env.get("MAX_STID_CACHE_COUNT")
			end

			if l_max_string/=Void and then l_max_string.is_integer
			then
				Result := l_max_string.to_integer
			end

			if Result<0
			then
				Result := 0
			end
		ensure
			result_is_non_negative: Result>=0
		end

feature -- Basic operations

	lock_cacheing is
		do
		end

	unlock_cacheing is
		do
		end

	reset_direction is
			-- Set up cache reading to determine direction anew.
		do
			direction_determined := False
			direction_is_key_0 := False
			paths_attempted_for_direction := 0
			key_0_changes := 0
			key_1_changes := 0
		end

feature {NONE} -- Implementation

	--direction_reset_count: INTEGER is 500
	direction_determination_count: INTEGER is 5

	initial_cache_size: INTEGER is
			-- Default size of cache (minimum of 100)
		do
			if max_allowable_count>0
			then
				Result := max_allowable_count.max(100)	-- Here we have decided that the initial size will be the max size.
			else
				Result := cache_multiplier.max (100)
			end
		end

	cache_multiplier: INTEGER is
			-- The multiplier value to set how many cross sections should be kept in the volatile cache
		local
			l_multiplier: STRING
		once
			l_multiplier := config.value("STID_CACHE_MULTIPLIER")
			if l_multiplier=Void or else l_multiplier.count=0
			then
				l_multiplier := global.env.get ("STID_CACHE_MULTIPLIER")
			end

			if l_multiplier /= Void then
				if l_multiplier.is_integer then
					Result := l_multiplier.to_integer
				elseif l_multiplier.as_lower.has_substring ("max") then
					Result := 10		-- Note that this used to be MAX_INT!
				end
			end
			if Result <= 0 then
				Result := 2
			end
		ensure
			valid_multiplier: Result > 0
		end

feature {NONE} -- Clustering

	last_rose_virtual_record: ROSE_VIRTUAL_RECORD[I]
	last_ad_hoc_rose_virtual_records: HASH_TABLE [ROSE_VIRTUAL_RECORD[I], I]
	last_path: INTEGER
	direction_determined: BOOLEAN
	direction_is_key_0: BOOLEAN
	paths_attempted_for_direction: INTEGER
	key_0_changes: INTEGER
	key_1_changes: INTEGER
	query_0: G
	query_1: H

feature {CLUSTERING_TOOL, PERSISTENT_SERVER_COMMAND} -- Implementation

	reset_path_tree is
		do
			signature_tagged_server.reset_path_tree
		end

feature {NONE} -- Implementation

	signature_tagged_server: CLUSTERED_TAGGED_SERVER[G, H, I]	-- The database portion of this class.

	ready_for_adjust: BOOLEAN

	server_delivery (a_network_server_name: STRING): STRING is
			-- The delivery configured for this server
		require
			valid_name: a_network_server_name /= Void
		do
			Result := Config.value (a_network_server_name + "_delivery")
			if Result = Void then
				Result := Global.app.delivery.value (a_network_server_name + "_delivery")
				if Result = Void then
					Result := Global.app.site.value (a_network_server_name + "_delivery")
				end
			end
		end

end
