note

	description:
	"[
		The client object for remote signature tagged databases.
		This object becomes the server object in the signature tagged database object
		when the database is created remote.
		SERVED_CLUSTERED_TAGGED_ITEM_DATABASE is the corresponding client class.
	]"
	author: "Randy John"
	last_editor: "$Author: rj $"
	date: "$Date: 2015/05/20 $"
	revision: "$Revision: #64 $"
	path: "$File: //rose/source/kernel/persistence/databases/signature_tagged/clustered_tagged_server_client.e $"

class
	CLUSTERED_TAGGED_SERVER_CLIENT [G -> COMPARABLE, H -> COMPARABLE,
									reference I -> CLUSTERED_SIGNATURE [G, H, ENTITY] create default_create end]

inherit
	CLUSTERED_TAGGED_SERVER [G, H, I]
		rename
			make as cts_make
		export
			{CLUSTERED_TAGGED_ITEM_DATABASE} all
		redefine
			path_tree,
			set_current_cache_pool,
			ordered_cube_of_items,
			remake_virtual_record_set,
			set_query_signatures,
			set_new_query,
			set_process_name,
			create_data_items,
			create_selected_column_ids,
			set_record_limit,
			set_restriction_clause,
			set_historical_mode,
			clear_historical_mode,
			columns_from_restriction_statement,
			clear_column_comparison,
			set_cache_from_results,
			last_rose_signature,
			set_virtual_record_set_cube_result_temp,
			stats_string,
			set_ad_hoc_query,
			prepare_to_retrieve_signatures,
			select_signature_ids,
			last_cube_signature_result,
			create_trigger,
			retrieve_signature_data_history,
			process_id_map,
			refresh_process_id_map,
			signature_id,
			process_id,
			records_count,
			update_end_time,
			start_process,
			terminate_process_id
		select
			exceptions_class_name
		end
	SECURE_ROSE_DEDICATED_NETWORK_CLIENT
		rename
			servers as rdnc_servers
		export
			{NONE} all
		redefine
			remote_call
		end
	ROSE_EXECUTION_ENVIRONMENT
		export
			{NONE} all
		end
	PLATFORM
		export
			{NONE} all
		end
	ANY

	ROSE_PROCESS_TIMER_COLLECTION_AWARE

	DEBUG_OUTPUT

	PM_AWARE
		rename
			trace as global_trace
		end

create
	make,
	make_persistent

create {MARKET_DATA_CLUSTERED_TAGGED_ITEM_DATABASE}
	cts_make

feature {NONE} -- Creation

	make (a_network_server_name: STRING; a_args: ARRAY[STRING]; a_executable_name, a_version: STRING) is
			-- Creation procedure for a single use dedicated server
		require
			valid_network_server_name: a_network_server_name /= Void
			valid_executble_name: a_executable_name /= Void
		do
			arguments := a_args
			server_name := a_network_server_name
			timer_metric_name := "Compute timers/" + server_name
			a_args.do_all (agent (a: STRING)
		                       do
							      timer_metric_name.append (" ")
							      timer_metric_name.append (a)
							   end)
			--server_name := "-1"
			create port_server_message.make
			--port_server_message.set_rose_command (a_network_server_name)
			port_server_message.set_server_arguments (a_args)
			port_server_message.set_executable (server_directory_root + "\" + a_executable_name)
			get_server
			in_out.set_nodelay
			remote_call (compare_version_id, <<a_version>>)
		end

	make_persistent (a_network_server_name: STRING; a_args: ARRAY[STRING]; a_executable_name, a_version: STRING) is
			-- Create the client for a persistent remote instance of the database
		require
			valid_network_server_name: a_network_server_name /= Void
			valid_executble_name: a_executable_name /= Void
		do
			arguments := a_args
			server_name := a_network_server_name
			timer_metric_name := "Compute timers/" + server_name
			a_args.do_all (agent (a: STRING)
		                       do
							      timer_metric_name.append (" ")
							      timer_metric_name.append (a)
							   end)
			--server_name := "-1"
			create port_server_message.make
			port_server_message.set_rose_command (a_network_server_name)
			port_server_message.set_server_arguments (a_args)
			port_server_message.set_executable (server_directory_root + "\" + a_executable_name)

			-- Once we call 'get_server', the clock is running.  The server can timeout waiting for our first message ('remote_call').
			get_server
			in_out.set_nodelay
			global.file_access_log.record_persistent_server_read_access (a_network_server_name + args_string (a_args),
																	  	 port_server_message.machine + ":" + port_server_message.port.out + ":" + a_executable_name)
			port_server_message.set_user_id (Global.user.name)
			port_server_message.set_machine (Global.app.host_name)
			port_server_message.set_command_line (Global.app.program_path + " (" + global.env.current_process_id.out + ")")

			if allowed_security_system_names = Void
			then
				-- The server does not support security.
				remote_call (set_data_client_id, <<True, port_server_message>>)
			else
				secure_connect
			end

			remote_call (compare_version_id, <<a_version>>)
		end

feature {CLUSTERED_TAGGED_ITEM_DATABASE} -- Redefined

	path_tree: PARTITION_TREE [G,H] is
			-- Precomputed binary tree used to find cluster id for a signature retrieved from DB.
		local
			l_any: ANY
			l_collecting: BOOLEAN
			l_stream: ROSE_STREAM
			l_file: ROSE_FILE
			l_folder: STRING
			l_use_file: BOOLEAN
			l_file_name: STRING
			l_factory: PARTITION_TREE_FACTORY [G, H]
		do
			if path_tree_saved = Void then
				debug("timers_clustered_tagged_server_client")
					timers.resume(generating_type, "path_tree")
				end
				l_use_file := is_vms
				debug ("windows_path_tree")
					l_use_file := True; print ("Not for production use! CLUSTERED_TAGGED_SERVER_CLIENT:path_tree%N")
				end
				if unique_signature_server_name.count /= 0 and l_use_file then
					if is_vms then
						l_folder := "signature_tagged_data:"
					else
						debug("windows_path_tree")
							l_folder := "c:\temp\"
						end
					end

					l_file_name := l_folder + unique_signature_server_name + "_path_tree_70.storable"	-- Adjust version number as appropriate.
					create l_file.make (l_file_name)
					if l_file.exists then
						if should_print_gc_stats then
							print_gc_stats
						end
						path_tree_saved ?= safe_read_storable_from_file (l_file)
						if should_print_gc_stats then
							print_gc_stats
						end
						if path_tree_saved = Void then
							warn ("path_tree","Unable to read path_tree from $1%N",<<l_file_name>>)
						end
						--create l_file.make_open_write ("c:\temp\k.dat")
						--l_file.independent_store (path_tree_saved)
						--l_file.close
					end
				end
				if path_tree_saved = Void then
					if should_print_gc_stats then
						print_gc_stats
					end
					l_any := remote_function (path_tree_id, Void)
					if should_print_gc_stats then
						print_gc_stats
					end
					if attached {STRING} l_any as l_string then
						if collecting then
							l_collecting := True
							collection_off
						end
						create l_stream
						path_tree_saved ?= l_stream.retrieve_from_string (l_string)
						if l_collecting then
							collection_on
						end
					else
						path_tree_saved ?= l_any
					end
					create l_factory.make (table_structure)
					l_factory.reconcile (path_tree_saved)
					if path_tree_saved = Void then
						warn ("path_tree","Unable to retrieve path_tree from server for $1%N",<<unique_signature_server_name>>)
					end
					if l_use_file and l_file /= Void then
						inform ("path_tree","Saving the path_tree for $1 in $2%N",<<unique_signature_server_name,l_file.name>>)
						if l_file.exists then
							l_file.delete
						end
						Global.persist.store (path_tree_saved, l_file.name)
						inform ("path_tree","Saved the path_tree for $1 in $2%N",<<unique_signature_server_name,l_file.name>>)
					end
				end
			debug("timers_clustered_tagged_server_client")
				timers.interrupt(generating_type, "path_tree")
			end
		end
			Result := path_tree_saved
		rescue
			if l_collecting then
				collection_on
			end
		end

	remake_virtual_record_set is
		do
			remote_call (remake_virtual_record_set_id, Void)
		end

	start_process (a_process_name: STRING; a_file_date, a_market_date, a_rose_date: ROSE_DATE) is
			-- Start a process named 'a_process_name'
		do
			remote_call (start_process_id, <<a_process_name, a_file_date, a_market_date, a_rose_date>>)
		end

	terminate_process_id
	-- Terminates the process started by 'start_process'.
	do
		-- Not yet supported by the server.
--		remote_call(terminate_process_id_id, Void)
	end

	update_end_time is
			-- Update the end time for the currently active supplier process
		do
			remote_call (update_end_time_id, Void)
		end

	set_current_cache_pool (a_signature: I) is
			-- Call `set_current_cache_pool' on the server
		do
			remote_call (set_current_cache_pool_id, <<a_signature.signature_for_persistent_server>>)
			cache_pool_id := path (a_signature)
		end

	prepare_to_retrieve_signatures is
			-- <Precursor>
		do
			remote_call (prepare_to_retrieve_signatures_id, Void)
		end

	select_signature_ids (a_columns: COLUMN_DESCRIPTORS [I]) is
			-- <Precursor>
		do
			remote_call (select_signature_ids_id, <<a_columns>>)
		end

	retrieve_signature_data_history (a_signature: I): ROSE_LINEAR_ARRAY [I] is
			-- <Precursor>
		do
			Result ?= remote_function (retrieve_signature_data_history_id, <<a_signature.signature_for_persistent_server>>)
		end

	ordered_cube_of_items (a_columns: COLUMN_DESCRIPTORS[I];
						   a_items: ROSE_LINEAR_ARRAY [INTEGER]): ROSE_LINEAR_ARRAY [ROSE_VIRTUAL_RECORD[I]] is
			-- Array of virtual records that meet query criteria specified by `a_columns'.
			-- Results will contain only items specified in `a_items' unless `a_items' is `Void' which then forces all items to be retrieved.
		local
			l_any: ANY
			l_string: STRING
			l_stream: ROSE_STREAM
			l_collecting: BOOLEAN
		do
			l_collecting := (create {MEMORY}).collecting;
			(create {MEMORY}).collection_off
  			l_any := remote_function (ordered_cube_of_items_id, <<a_columns, a_items>>)
  			l_string ?= l_any
  			if l_string /= Void then
  				create l_stream
  				Result ?= l_stream.retrieve_from_string (l_string)
				if Result = Void then
					print (Global.interpreted ("%N($now) 'CLUSTERED_TAGGED_SERVER_CLIENT.ordered_cube_of_items' did not receive a string representation of 'ROSE_LINEAR_ARRAY [ROSE_VIRTUAL_RECORD[I]]'%N"))
				end
  			else
				Result ?= l_any
				if Result = Void then
					print (Global.interpreted ("%N($now) 'CLUSTERED_TAGGED_SERVER_CLIENT.ordered_cube_of_items' did not receive 'ROSE_LINEAR_ARRAY [ROSE_VIRTUAL_RECORD[I]]'%N"))
				end
  			end
			if Result = Void then
				if l_any /= Void then
					print (Global.interpreted ("%N($now) 'CLUSTERED_TAGGED_SERVER_CLIENT.ordered_cube_of_items' received: %N"))
					print (l_any.out)
				else
					print (Global.interpreted ("%N($now) 'CLUSTERED_TAGGED_SERVER_CLIENT.ordered_cube_of_items' received nothing%N"))
				end
			end
			if l_collecting then
				(create {MEMORY}).collection_on
			end
		rescue
			if l_collecting then
				(create {MEMORY}).collection_on
			end
		end

	set_query_signatures (a_signatures: ROSE_LINEAR_ARRAY[I])
			-- <Precursor>
		local
			i: INTEGER
			l_signatures: ROSE_LINEAR_ARRAY[I]
		do
			if a_signatures /= Void
			then
				create l_signatures.make(a_signatures.lower, a_signatures.upper)

				from
					i := a_signatures.lower
				until
					i > a_signatures.upper
				loop
					check attached {I} a_signatures[i].signature_for_persistent_server as al_signature
					then
						-- If you fail this test then your implementation of 'signature_for_persistent_server'
						-- is returning the wrong type or Current was built with an I that does not match the
						-- persistent server.  The latter problem starts with a generic argument to your
						-- descendant of CLUSTERED_TAGGED_ITEM_DATABASE.
						l_signatures[i] := al_signature
					end

					i := i + 1
				end
			end

			remote_call (set_query_signatures_id, <<l_signatures>>)
		end

	set_new_query is
		do
			remote_call (set_new_query_id, Void)
		end

	set_process_name (a_name: STRING) is
			-- Set process name to `a_name'.
		do
			remote_call (set_process_name_id, <<a_name>>)
		end

	create_data_items is
			-- <Precursor>
		do
			remote_call (create_data_items_id, Void)
			create data_items.make_empty
		end

	create_selected_column_ids is
		do
			remote_call (create_selected_column_ids_id, Void)
		end

	set_record_limit (a_record_limit: INTEGER) is
		do
			remote_call (set_record_limit_id, <<a_record_limit>>)
			set_ad_hoc_query (a_record_limit/=0)
		end

	set_historical_mode is
		do
			remote_call (set_historical_mode_id,Void)
		end

	clear_historical_mode is
			-- <Precursor>
		do
			remote_call (clear_historical_mode_id,Void)
		end

	set_ad_hoc_query (a_value: BOOLEAN) is
			-- <Precursor>
		do
			remote_call (set_ad_hoc_query_id, <<a_value>>)
			ad_hoc_query := a_value
		end

	create_trigger_called: BOOLEAN		-- Prevents us from calling 'create_trigger' twice.

	create_trigger (a_data_name: STRING) is
			-- <Precursor>
		do
			if not create_trigger_called then
				remote_call (create_trigger_id, <<a_data_name>>)
				create_trigger_called := True
			end
		end

	last_cube_signature_result: ROSE_LINEAR_ARRAY [I] is
		do
			Result ?= remote_function (last_cube_signature_result_id,Void)
		end

	set_restriction_clause (a_restriction_clause: STRING) is
		do
			remote_call (set_restriction_clause_id, <<a_restriction_clause>>)
		end

	columns_from_restriction_statement (a_statement: STRING): ROSE_LINEAR_ARRAY [STRING] is
		do
			Result ?= remote_function (columns_from_restriction_statement_id, <<a_statement>>)
		end

	clear_column_comparison
			-- <Precursor>
		do
			remote_call (clear_column_comparison_id, Void)
		end

	signature_id (a_signature: I; a_can_read_old_signature, a_can_write_new_signature: BOOLEAN): INTEGER
			-- Given 'a_signature' with certain fields set (e.g. company, date), return the signature_id.
			-- See SIGNATURE_TAGGED_SERVER.signature_id for documentation about the 'a_can_*' arguments.
		local
			l_integer_ref: INTEGER_REF
		do
			l_integer_ref ?= remote_function (signature_id_id, <<a_signature.signature_for_persistent_server, a_can_read_old_signature, a_can_write_new_signature>>)
			Result := l_integer_ref.item
		end

	process_id: INTEGER is
			-- The process_id (in the process table) that identifies this particular run.
		local
			l_integer_ref: INTEGER_REF
			l_server_name: STRING
		do
			if process_id_set then
				Result := set_process_id_value
			elseif process_id_manager_created then
				Result := process_id_manager.process_id
			else
				if server_name/=Void
				then
					l_server_name := server_name
				else
					l_server_name := "<Void>"
				end

				l_integer_ref ?= remote_function (process_id_id, Void)
				Result := l_integer_ref.item
				output (Global.interpreted ("($now) ") +
						"CLUSTERED_TAGGED_SERVER_CLIENT: Created process_id " + Result.out + " via " + l_server_name + " (" + debug_output + ").%N",
						Void)
				set_process_id_value := Result
				process_id_set := True
			end
		end

	records_count (a_name: STRING; a_value, a_item: INTEGER): INTEGER is
		local
			l_integer_ref: INTEGER_REF
		do
			l_integer_ref ?= remote_function (records_count_id, <<a_name, a_value, a_item>>)
			Result := l_integer_ref.item
		end

	set_cache_from_results (a_virtual_record: ROSE_VIRTUAL_RECORD[I]; a_client: SIGNATURE_TAGGED_ITEM_DATABASE[I]) is
		local
			l_path: INTEGER
			l_record: ROSE_VIRTUAL_RECORD[I]
			l_cube_result_table: HASH_TABLE[ROSE_VIRTUAL_RECORD[I], MULTI_KEY]
			l_cube_signature_result: ROSE_LINEAR_ARRAY[I]
			l_cube_position_result: HASH_TABLE[I, INTEGER]
			l_multi_key: MULTI_KEY
			l_signature: I
			l_date: ROSE_DATE
			l_virtual_record_set: TRIPLE[ROSE_LINEAR_ARRAY[ROSE_VIRTUAL_RECORD[I]],
										 HASH_TABLE[ROSE_VIRTUAL_RECORD[I], MULTI_KEY],
										 INTEGER]
		do
			if (a_virtual_record /= Void or cache_pool_id /= 0) and not ad_hoc_query then
				if a_virtual_record /= Void and then a_virtual_record.signature /= Void then
					l_path := a_client.path (a_virtual_record.signature)
				else
					l_path := cache_pool_id
				end
				create l_cube_result_table.make (1_000)
				from
					virtual_record_set_cube_result_temp.start
				until
					virtual_record_set_cube_result_temp.off
				loop
					l_record := virtual_record_set_cube_result_temp.current_item
					if l_record = Void then
						fail ("set_cache_from_results", "-Void- record found in `virtual_record_set_cube_result_temp'", Void)
					end

					l_signature := l_record.signature
					if l_signature/=Void
					then
						-- The signature will be Void if 'l_record' is a ROSE_RMDAT_LEGACY_VIRTUAL_RECORD.
						l_signature.clear_saved_multi_key
					end

					if l_signature/=Void and then l_signature.as_of_date /= Void then
						l_date := process_id_map.item (l_signature.last_process_id.max (l_signature.signature_data_process_id))
						if l_date = Void then
							refresh_process_id_map
							l_date := process_id_map.item (l_signature.last_process_id.max (l_signature.signature_data_process_id))
							if l_date = Void then
								fail ("set_cache_from_results", "Invalid last_process_id $1 in signature $2 from the $3",<<l_signature.last_process_id,l_signature.signature_id,server_name>>)
							end
						end
						if not equal (l_date,l_signature.as_of_date) then
							if not historical_mode then
								--warn ("set_cache_from_results", "Historical signature version when historical mode is false for signature $1 from the $2", <<l_signature.signature_id,server_name>>)
								l_cube_result_table.extend (l_record, l_record.multi_key)
							end
							create l_multi_key.make_from_multi_key(l_signature.multi_key)
							l_multi_key.array_force (l_signature.as_of_date, l_multi_key.key_count + 1)
							l_cube_result_table.extend (l_record, l_multi_key)
						else
							l_cube_result_table.extend (l_record, l_record.multi_key)
						end
					else
						l_cube_result_table.extend (l_record, l_record.multi_key)
					end
					virtual_record_set_cube_result_temp.forth
				end
				create l_virtual_record_set.make (virtual_record_set_cube_result_temp, l_cube_result_table, l_path)
				a_client.force_manager_cache (l_virtual_record_set, l_path)
				l_cube_result_table := l_virtual_record_set.second
				create l_cube_signature_result.make_empty
				if should_build_cube_position_result then
					create l_cube_position_result.make (l_cube_result_table.count)
				end
				from
					l_cube_result_table.start
				until
					l_cube_result_table.off
				loop
					l_record := l_cube_result_table.item_for_iteration
					l_cube_signature_result.extend (l_record.signature)
					if should_build_cube_position_result then
						l_cube_position_result.extend (l_record.signature, l_record.signature.signature_id)
					end
					l_cube_result_table.forth
				end
				a_client.set_cube_result_table (l_cube_result_table)
				a_client.set_cube_signature_result (l_cube_signature_result)
				if should_build_cube_position_result then
					a_client.set_cube_position_result (l_cube_position_result)
				end
			end
		end

	last_rose_signature: ROSE_VIRTUAL_RECORD[I] is
		do
			Result ?= remote_function (last_rose_signature_id, Void)
		end

	set_virtual_record_set_cube_result_temp (a_cube_result: ROSE_LINEAR_ARRAY[ROSE_VIRTUAL_RECORD[I]]) is
		do
			virtual_record_set_cube_result_temp := a_cube_result
		end

	virtual_record_set_cube_result_temp: ROSE_LINEAR_ARRAY[ROSE_VIRTUAL_RECORD[I]]

	process_id_map: HASH_TABLE [ROSE_DATE, INTEGER] is
		do
			if process_id_map_saved = Void then
				process_id_map_saved ?= remote_function (process_id_map_id, Void)
			end
			Result := process_id_map_saved
		end
	process_id_map_saved: HASH_TABLE[ROSE_DATE,INTEGER]

	refresh_process_id_map is
		do
			process_id_map_saved := Void
			remote_call (refresh_process_id_map_id,Void)
		end

feature -- Message statistics

	stats_string: STRING is
			-- Query for and display general statistics from the server
		do
			Result ?= remote_function (stats_string_id, Void)
			Result.append (
						   "Messages sent     = " + messages_sent.out + "%N" +
						   "Messages received = " + messages_received.out + "%N"
						   )
		end

	messages_sent: INTEGER
			-- Count of messages sent to the server
	messages_received: INTEGER
			-- Count of messages received from the server

feature {CLUSTERED_TAGGED_ITEM_DATABASE} -- Messaging

	timer_metric_name: STRING

	remote_call (a_command_id: INTEGER; a_args: ARRAY[ANY]) is
			-- Invoke 'a_command_id' with 'a_args' on the remote server
		local
			l_cmd: PAIR[INTEGER, ARRAY[ANY]]
			l_timer: ROSE_PROCESS_TIMER
		do
			create l_timer
			create l_cmd.make (a_command_id, a_args)
			send (l_cmd)
			messages_sent := messages_sent + 1
			l_timer.interrupt
			add_metric (timer_metric_name + "/call_" + a_command_id.out, l_timer.elapsed)
		rescue
			print_failure_message
		end

	remote_function (a_command_id: INTEGER; a_args: ARRAY[ANY]): ANY is
			-- Invoke 'a_command_id' with 'a_args' on the remote server and return the result of the function
		local
			l_cell: CELL[ANY]
			l_string: STRING
			l_in_remote_call: BOOLEAN
			l_timer: ROSE_PROCESS_TIMER
		do
			l_in_remote_call := True
			remote_call (a_command_id, a_args)
			l_in_remote_call := False

			create l_timer
			receive
			l_cell ?= received
			l_timer.interrupt
			add_metric (timer_metric_name + "/function_" + a_command_id.out, l_timer.elapsed)

			messages_received := messages_received + 1
			if l_cell /= Void then
				Result := l_cell.item
			else
				l_string ?= received
				if l_string /= Void and l_string.count > 0 then
					if l_string.substring_index ("SERVER_EXCEPTION", 1) > 0 then
						print (server_name + " - " + l_string)
						print ("%NEND_OF_SERVER_EXCEPTION%NEND_OF_SERVER_EXCEPTION%NEND_OF_SERVER_EXCEPTION%NEND_OF_SERVER_EXCEPTION%N")
						server_failure_count := max_server_failures
						raise ("SERVER_EXCEPTION")
					elseif l_string.substring_index ("SERVER_MESSAGE", 1) > 0 then
						print ("%N%NSTART_OF_SERVER_MESSAGE%N%N" + server_name + " - " + l_string + "%N%NEND_OF_SERVER_MESSAGE%N%N")
					end
				end
			end
			if a_command_id = ordered_cube_of_items_id and Result = Void then
				print (Global.interpreted ("%N($now) 'CLUSTERED_TAGGED_SERVER_CLIENT.remote_function' for 'ordered_cube_of_items_id' did not receive 'CELL [ANY]'%N"))
			end
		rescue
			-- No need to print anything if we were calling 'remote_call' since it already did this.
			if not l_in_remote_call
			then
				print_failure_message
			end
		end

	print_failure_message
	-- Prints details about this object for use during an exception.  Since this is run inside a 'rescue',
	-- it is very important that no failures can happen.
	local
		l_server_name: STRING
	do
		if rdnc_servers/=Void and then not rdnc_servers.off
		then
			l_server_name := rdnc_servers.current_item
		else
			l_server_name := "?"
		end

		output("%N" + Global.interpreted("($now) ") + "Error in $1 ($2) on $3:$4%N", <<server_name, debug_output, l_server_name, port_number>>)
	end

feature {NONE} -- GC Statistics

	should_print_gc_stats: BOOLEAN
			-- If True then garbage collection statistics are printed in the log.
		once
			Result := get ("PRINT_EIFFEL_GC_INFO") /= Void
			--		Result := True; print ("should_print_gc_stats forced to True%N")
		end

	print_gc_stats is
			-- Dump the Garbage Collection statistics to the console.
		local
			l_mem_constants: MEM_CONST
		do
			create l_mem_constants
			print ("Full GC stats:%N")
			print_gc_stats_by_type (l_mem_constants.Full_collector)
			print ("%N")
			print ("Incremental GC stats:%N")
			print_gc_stats_by_type (l_mem_constants.Incremental_collector)
			print ("%N")
		end

	print_gc_stats_by_type (a_gc_type: INTEGER) is
		local
			l_gc_info: GC_INFO
		do
			l_gc_info := gc_statistics (a_gc_type)
			print ("cycle_count = " + l_gc_info.cycle_count.out + "%N")
			print ("memory_used = " + l_gc_info.memory_used.out + "%N")
			print ("collected = " + l_gc_info.collected .out + "%N")
			print ("collected_average = " + l_gc_info.collected_average.out + "%N")
			print ("real_time = " + l_gc_info.real_time.out + "%N")
			print ("real_time_average = " + l_gc_info.real_time_average.out + "%N")
			print ("real_interval_time = " + l_gc_info.real_interval_time.out + "%N")
			print ("real_interval_time_average = " + l_gc_info.real_interval_time_average.out + "%N")
			print ("cpu_time = " + l_gc_info.cpu_time.out + "%N")
			print ("cpu_time_average = " + l_gc_info.cpu_time_average.out + "%N")
			print ("cpu_interval_time = " + l_gc_info.cpu_interval_time.out + "%N")
			print ("cpu_interval_time_average = " + l_gc_info.cpu_interval_time_average.out + "%N")
			print ("sys_time = " + l_gc_info.sys_time.out + "%N")
			print ("sys_time_average = " + l_gc_info.sys_time_average.out + "%N")
			print ("sys_interval_time = " + l_gc_info.sys_interval_time.out + "%N")
			print ("sys_interval_time_average = " + l_gc_info.sys_interval_time_average.out + "%N")
		end

feature {NONE} -- Implementation.
	unique_signature_server_name: STRING is
			-- Application specific method of identifying the server.
		do
			if unique_signature_server_name_saved=Void
			 then
				unique_signature_server_name_saved ?= remote_function (Unique_signature_server_name_id, Void)
				if unique_signature_server_name_saved=Void
				 then
					unique_signature_server_name_saved := ""
				end
			end

			Result := unique_signature_server_name_saved
		end

	unique_signature_server_name_saved: STRING
			-- once per object

	safe_read_storable_from_file (a_file: FILE): ANY is
			-- Retrieve a storable from a_file but catch the exception if there is a retrieval error
		require
			valid_file: a_file /= Void
		local
			l_retry: BOOLEAN
			l_collecting: BOOLEAN
		do
			if not l_retry then
				l_collecting := collecting
				Result ?= Global.persist.retrieved (a_file.name)
			end
		rescue
			l_retry := True
			if l_collecting then
				collection_on
			else
				collection_off
			end
			retry
		end

	args_string (a_args: ARRAY [STRING]): STRING is
			-- A string representing 'a_args' used for resource access tracking
		local
			i: INTEGER
		do
			if a_args = Void then
				Result := ""
			else
				from
					Result := ""
					i := a_args.lower
				until
					i > a_args.upper
				loop
					if a_args.item (i) /= Void and then not a_args.item (i).is_empty then
						Result.append (" " + a_args.item (i).as_upper)
					end
					i := i + 1
				end
			end
		ensure
			attached_result: Result /= Void
		end

feature -- Access.
	debug_output: STRING
	-- The usual.
	local
		l_arg: STRING
	do
		if server_name /= Void
		then
			Result := server_name.twin
		else
			Result := "Void"
		end

		Result.append("/")

		l_arg := args_string(arguments)
		l_arg.prune_all_leading(' ')
		Result.append(l_arg)

		Result.append("/")

		Result.append(port_number.out)

		Result.append("/")

		if in_out /= Void
		then
			Result.append(in_out.debug_output)
		else
			Result.append("Void")
		end
	end

feature {NONE} -- Attributes (alpha order).
	arguments: ARRAY[STRING]	-- The original arguments that were passed to the creation procedure.

end
