Indexing
	description: "Signature tagged table structure"
	keywords: "database,signature,item,tagged,manager"
	author: "Nelson Chui"
	revision: "$Revision: #13 $"
	path : "$File: //rose/source/kernel/persistence/databases/signature_tagged/clustered_tagged_virtual_record_predicate_cache.e $"
	
class
	CLUSTERED_TAGGED_VIRTUAL_RECORD_PREDICATE_CACHE [G -> COMPARABLE,
							 H -> COMPARABLE,
							 I -> CLUSTERED_SIGNATURE[G,H,ENTITY] create default_create end]
	
inherit
	SIGNATURE_TAGGED_VIRTUAL_RECORD_PREDICATE_CACHE [I]
		redefine
			base_cube_selection,
			dated_base_cube_selection,
			dated_base_cube_selection_statement,
			override_base_cube_selection,
			record_cube_selection,
			dated_record_cube_selection,
			signature_data_cube_selection,
			dated_signature_data_cube_selection,
			override_record_cube_selection,
			override_signature_data_cube_selection,
			signature_version_selection,
			override_signature_version_selection,
			multiple_field_cube_selection,
			multiple_field_dated_cube_selection,
			multiple_field_override_cube_selection
		end
	
create
	make
	
feature
	
	set_ad_hoc_query(a_value : BOOLEAN) is
		do
			ad_hoc_query := a_value
		end
	ad_hoc_query : BOOLEAN
	
	set_path_tree(a_path_tree : like path_tree) is
		do
			path_tree := a_path_tree
		end
	path_tree : PARTITION_TREE[G,H]
	
	set_signature_target(a_target : ANY) is
		do
			signature_target ?= a_target
		end
	signature_target : CLUSTERED_SIGNATURE[G,H,ENTITY]
	
feature {NONE} -- Data item selection predicates
	
	path(a_signature : CLUSTERED_SIGNATURE[G,H,ENTITY]) : INTEGER is
		local
			l_record : PARTITION_TREE_RECORD[G, H]
		do
			l_record := path_tree.leaf_record(a_signature.key_0, a_signature.key_1)
			if l_record /= Void then
				Result := l_record.path
			end
		end
	
feature -- Single field cube selection
	
	base_cube_selection (a_type: STRING; a_historical_mode, a_join_process_locally, a_is_partial_record : BOOLEAN): ANONYMOUS_PREDICATE is
		local
			--l_key: KEY_PAIR [STRING, INTEGER]
			l_fields, l_statement : STRING
			l_path : INTEGER
		do
			if ad_hoc_query then 
				Result := Precursor(a_type,a_historical_mode,a_join_process_locally,a_is_partial_record)
			else
				l_path := path(signature_target)
				if not a_join_process_locally then
					l_fields := "signature_id,item_id,file_date,value"
				else
					l_fields := "signature_id,item_id,process_id,value"
				end
				if a_historical_mode then
					l_statement := "select * from (select " + l_fields
				else
					l_statement := "select " + l_fields
				end
				l_statement := l_statement + " from " +
					table_structure.data_name_prefix + "data_" + a_type + "_value v "
				if not a_join_process_locally then
					l_statement := l_statement + " INNER JOIN process d " + " ON " + " v.process_id = d.process_id "
				end
				if a_is_partial_record then
					l_statement := l_statement + " INNER JOIN #variables c " + " ON " + " v.item_id = c.id "
				end
				l_statement := l_statement + " where path = " + l_path.out
				if a_historical_mode then
					l_statement := l_statement + 
						" union all select " + l_fields + " from " + 
						table_structure.data_name_prefix + "data_" + a_type + "_value_history v "
					if not a_join_process_locally then
						l_statement := l_statement + " INNER JOIN process d " + " ON " + " v.process_id = d.process_id "
					end
					if a_is_partial_record then
						l_statement := l_statement + " INNER JOIN #variables c " + " ON " + " v.item_id = c.id "
					end
					l_statement := l_statement + " where path = " + l_path.out + ") a order by signature_id, process_id"
				else
					l_statement := l_statement + " order by signature_id, process_id"
				end
				if not a_join_process_locally then
					l_statement := l_statement + ", file_date"
				end
				create Result.make_using_server(table_structure.signature_item_server,l_statement)
				if not Result.success then
					print(l_statement)
					(create {EXCEPTIONS}).raise("Failed to create base_cube_selection predicate")
				end
				--predicate_cache.force (Result, l_key)
				-- print(l_statement+"%N")
			end
		end
	
	override_base_cube_selection (a_type: STRING; a_historical_mode, a_is_partial_record : BOOLEAN): ANONYMOUS_PREDICATE is
		local
			l_statement : STRING
			l_path : INTEGER
		do
			if ad_hoc_query then 
				Result := Precursor(a_type,a_historical_mode,a_is_partial_record)
			else
				l_path := path(signature_target)
				l_statement := "SELECT v.signature_id,v.item_id,v.start_date,v.value,v.end_date,v.override_type_id,v.user_id,v.update_date,v.comment " +
					" FROM " + table_structure.override_family_name_prefix + "override_" + table_structure.data_name_prefix + "data_" + a_type + "_value v " +
					" JOIN data_signatures s ON s.signature_id = v.signature_id "
				if a_is_partial_record then
					l_statement := l_statement +
						" INNER JOIN #variables b " + " ON " + " v.item_id = b.id "
				end
				l_statement := l_statement +
					" where v.start_date <= ? "
				if not a_historical_mode then
					l_statement := l_statement +
						" and v.end_date > ? "
				end
				l_statement := l_statement + " AND s.path = " + l_path.out
				l_statement := l_statement + " ORDER BY v.signature_id, v.start_date"
				create Result.make_using_server(table_structure.signature_item_server,l_statement)
				if not Result.success then
					print(l_statement)
					(create {EXCEPTIONS}).raise("Failed to create override_base_cube_selection predicate")
				end
			end
		end
	
	dated_base_cube_selection (a_type: STRING): ANONYMOUS_PREDICATE is
		local
			l_statement : STRING
		do
			if ad_hoc_query then 
				Result := Precursor (a_type)
			else
				l_statement := dated_base_cube_selection_statement (a_type, false) + " union all " + 
					dated_base_cube_selection_statement (a_type, True) + " order by signature_id, start_date"
				create Result.make_using_server(table_structure.signature_item_server,l_statement)
				if not Result.success then
					print(l_statement)
					(create {EXCEPTIONS}).raise("Failed to create dated_base_cube_selection predicate")
				end
			end
		end
		
	dated_base_cube_selection_statement (a_type: STRING; a_is_history_table: BOOLEAN): STRING is
		local
			l_path: INTEGER
			l_history: STRING
		do
			-- path field in override table removal -- fa 200610
			-- removed path criteria on JOIN as redundant with signature_id criteria,
			-- but untested
			(create {EXCEPTIONS}).raise ("Untested")
			
			l_path := path (signature_target)
			if a_is_history_table then
				l_history := "_history"
			else
				l_history := ""
			end
			Result := "select v.signature_id as signature_id, v.item_id as item_id, max (p.file_date) as start_date, ( " +
				"select value from " + table_structure.data_name_prefix + "data_" + a_type + "_value" + l_history +
				" where signature_id = v.signature_id and item_id = v.item_id and process_id = max (v.process_id)) as value " +
				"from " + table_structure.data_name_prefix + "data_" + a_type + "_value" + l_history + " v " +
				"left outer join " + table_structure.override_family_name_prefix + "override_" + table_structure.data_name_prefix + "data_signature_versions ov on ov.signature_id = v.signature_id "
			--Result := Result + "left outer join #signatures t on t.path = v.path and t.id = v.signature_id and (t.signature_id is NULL or p.file_date <= t.version_date) "
			Result := Result + "join process p on v.process_id = p.process_id and (ov.signature_id is NULL or p.file_date <= ov.version_date) " +
				"where v.path = " + l_path.out + " group by v.signature_id, v.item_id"
		end

feature -- Multiple field cube selection

	record_cube_selection (a_template: ROSE_SIGNATURE_RECORD_ITEM; a_historical_mode, a_join_process_locally: BOOLEAN): ANONYMOUS_PREDICATE is
		local
			l_table_name: STRING
		do
			l_table_name := a_template.name + "_value"
			Result := multiple_field_cube_selection (l_table_name, a_template.record_fields, a_historical_mode, a_join_process_locally)
		end

	dated_record_cube_selection (a_template: ROSE_SIGNATURE_RECORD_ITEM): ANONYMOUS_PREDICATE is
		local
			l_table_name: STRING
		do
			l_table_name := a_template.name + "_value"
			Result := multiple_field_dated_cube_selection (l_table_name, a_template.record_fields)
		end

	signature_data_cube_selection (a_historical_mode, a_join_process_locally: BOOLEAN): ANONYMOUS_PREDICATE is
		local
			l_table_name: STRING
		do
			l_table_name := "signatures"
			Result := multiple_field_cube_selection (l_table_name, ur_signature.data_fields, a_historical_mode, a_join_process_locally)
		end

	dated_signature_data_cube_selection: ANONYMOUS_PREDICATE is
		local
			l_table_name: STRING
		do
			l_table_name := "signatures"
			Result := multiple_field_dated_cube_selection (l_table_name, ur_signature.data_fields)
		end
		
	override_record_cube_selection (a_template: ROSE_SIGNATURE_RECORD_ITEM; a_historical_mode: BOOLEAN): ANONYMOUS_PREDICATE is
		local
			l_table_name: STRING
		do
			l_table_name := a_template.name + "_value"
			Result := multiple_field_override_cube_selection (l_table_name, a_template.record_fields, a_historical_mode)
		end

	override_signature_data_cube_selection (a_historical_mode: BOOLEAN): ANONYMOUS_PREDICATE is
		local
			l_table_name: STRING
		do
			l_table_name := "signatures_data"
			Result := multiple_field_override_cube_selection (l_table_name, ur_signature.data_fields, a_historical_mode)
		end
	
feature -- Signature ID predicates
		
	signature_version_selection (a_join_process_locally: BOOLEAN): ANONYMOUS_PREDICATE is
		local
			l_statement: STRING
			l_path : INTEGER
		do
			if ad_hoc_query then 
				Result := Precursor(a_join_process_locally)
			else
				l_path := path(signature_target)
				l_statement := "select dsv.signature_id, dsv.process_id"
				if not a_join_process_locally then
					l_statement := l_statement + ", p.file_date"
				end
				l_statement := l_statement + " from " 
					+ table_structure.data_name_prefix + "data_signature_versions dsv "
				if not a_join_process_locally then
					l_statement := l_statement + " join process p on dsv.process_id = p.process_id "
				end
				l_statement := l_statement + " where path = " + l_path.out
				l_statement := l_statement + " order by signature_id"
				if not a_join_process_locally then
					l_statement := l_statement + ", file_date"
				end
				create Result.make_using_server (table_structure.signature_item_server, l_statement)
				if not Result.success then
					print (l_statement)
					(create {EXCEPTIONS}).raise ("Failed to create signature_version_selection predicate")
				end
				-- print(l_statement+"%N")
			end
		end

	override_signature_version_selection: ANONYMOUS_PREDICATE is
		local
			l_path: INTEGER
		do
			if ad_hoc_query then 
				Result := Precursor
			else
				-- fa 200610
				-- table not populated in current override implementation
				(create {EXCEPTIONS}).raise ("override data signature version not populated")
				
				l_path := path(signature_target)
				create Result.make_using_server (table_structure.signature_item_server,
					"select osv.signature_id, osv.version_date " +
					"from " + table_structure.override_family_name_prefix + "override_" + table_structure.data_name_prefix + "data_signature_versions osv " +
					"where osv.path = " + l_path.out + " order by osv.signature_id")
				if not Result.success then
					(create {EXCEPTIONS}).raise ("Could not create override signature version selection predicate.")
				end
			end
		end
		
feature
	
	multiple_field_cube_selection (a_table_name: STRING; a_fields: ROSE_LINEAR_ARRAY [STRING]; 
								   a_historical_mode, a_join_process_locally: BOOLEAN): ANONYMOUS_PREDICATE is
		local
			l_history_table_name, l_current_process_id_name, l_statement: STRING
			l_field_ix: INTEGER
			l_path : INTEGER
		do
			if ad_hoc_query then 
				Result := Precursor(a_table_name,a_fields,a_historical_mode,a_join_process_locally)
			else
				l_path := path(signature_target)
				if a_table_name.as_lower.is_equal ("signatures") then
					l_current_process_id_name := "signature_data_process_id"
					l_history_table_name := "signatures_data"
				else
					l_current_process_id_name := "process_id"
					l_history_table_name := a_table_name
				end
				if a_historical_mode then
					l_statement := "select * from (select a.signature_id"
				else
					l_statement := "select a.signature_id"
				end
				if not a_join_process_locally then
					l_statement := l_statement + ", c.file_date"
				else
					l_statement := l_statement + ", a." + l_current_process_id_name + " as process_id"
				end
				from l_field_ix := a_fields.lower until l_field_ix > a_fields.upper loop
					l_statement := l_statement + ", a.[" + a_fields.item (l_field_ix) + "]"
					l_field_ix := l_field_ix + 1
				end
				l_statement := l_statement + " from " + 
					table_structure.data_name_prefix + "data_" + a_table_name + " a "
				if not a_join_process_locally then
					l_statement := l_statement + " INNER JOIN process c" + " ON " + " a." + l_current_process_id_name + " = c.process_id "
				end
				l_statement := l_statement + " where path = " + l_path.out
				if a_historical_mode then
					l_statement := l_statement + " union all select d.signature_id"
					if not a_join_process_locally then
						l_statement := l_statement + ", f.file_date"
					else
						l_statement := l_statement + ", d.process_id"
					end
					from l_field_ix := a_fields.lower until l_field_ix > a_fields.upper loop
						l_statement := l_statement + ", d.[" + a_fields.item (l_field_ix) + "]"
						l_field_ix := l_field_ix + 1
					end
					l_statement := l_statement + " from " +
						table_structure.data_name_prefix + "data_" + l_history_table_name + "_history d "
					if not a_join_process_locally then
						l_statement := l_statement + " INNER JOIN process f " + " ON " + " d.process_id = f.process_id "
					end
					l_statement := l_statement + " where path = " + l_path.out
					l_statement := l_statement + ") g order by signature_id"
				else
					l_statement := l_statement + " order by signature_id"
				end
				if not a_join_process_locally then
					l_statement := l_statement + ", file_date"
				end
				create Result.make_using_server (table_structure.signature_item_server, l_statement)
				if not Result.success then
					print (l_statement)
					(create {EXCEPTIONS}).raise ("Failed to create record_cube_selection predicate")
				end
				-- print(l_statement+"%N")
			end
		end

	multiple_field_dated_cube_selection (a_table_name: STRING; a_fields: ROSE_LINEAR_ARRAY [STRING]): ANONYMOUS_PREDICATE is
			-- (export status {NONE})
		local
			l_history_table_name, l_current_process_id_name, l_statement: STRING
			l_field_ix: INTEGER
			l_path : INTEGER
		do
			if ad_hoc_query then 
				Result := Precursor(a_table_name,a_fields)
			else
				-- path field in override removal -- fa 200610
				-- remove in JOIN ON where path join (value/override) table is redundant with signature_id join
				-- but untested
				(create {EXCEPTIONS}).raise ("Override support untested.")
				
				l_path := path(signature_target)
				if a_table_name.as_lower.is_equal ("signatures") then
					l_current_process_id_name := "signature_data_process_id"
					l_history_table_name := "signatures_data"
				else
					l_current_process_id_name := "process_id"
					l_history_table_name := a_table_name
				end
				l_statement := "select v.signature_id as signature_id, p.file_date as start_date"
				from
					l_field_ix := a_fields.lower
				until
					l_field_ix > a_fields.upper
				loop
					l_statement := l_statement + ", v.[" + a_fields.item (l_field_ix) + "]"
					l_field_ix := l_field_ix + 1
				end
				l_statement := l_statement + " from " + table_structure.data_name_prefix + "data_" + a_table_name + " v "
				l_statement := l_statement + " left outer join " + table_structure.override_family_name_prefix + "override_" + table_structure.data_name_prefix + "data_signature_versions ov on v.signature_id = ov.signature_id "
				l_statement := l_statement + " join process p on v." + l_current_process_id_name + " = p.process_id and (ov.signature_id is NULL or p.file_date <= ov.version_date) "
				l_statement := l_statement + " where v.path = " + l_path.out
				l_statement := l_statement + " union all select v.signature_id as signature_id, p.file_date as start_date"
				from
					l_field_ix := a_fields.lower
				until
					l_field_ix > a_fields.upper
				loop
					l_statement := l_statement + ", v.[" + a_fields.item (l_field_ix) + "]"
					l_field_ix := l_field_ix + 1
				end
				l_statement := l_statement + " from " + table_structure.data_name_prefix + "data_" + l_history_table_name + "_history v "
				l_statement := l_statement + " left outer join " + table_structure.override_family_name_prefix + "override_" + table_structure.data_name_prefix + "data_signature_versions ov on v.signature_id = ov.signature_id "
				l_statement := l_statement + " join process p on v.process_id = p.process_id "
				l_statement := l_statement + " and v.process_id = (select max (a.process_id) from " + table_structure.data_name_prefix + "data_" + l_history_table_name + "_history a " + "join process b on a.process_id = b.process_id where a.signature_id = v.signature_id and (ov.signature_id is NULL or b.file_date <= ov.version_date group by signature_id) "
				l_statement := l_statement + " where v.path = " + l_path.out
				l_statement := l_statement + " order by signature_id, start_date"
				create Result.make_using_server (table_structure.signature_item_server, l_statement)
				if not Result.success then
					print (l_statement)
					(create {EXCEPTIONS}).raise ("Failed to create record_cube_selection predicate")
				end
			end
		end	
		
	multiple_field_override_cube_selection (a_table_name: STRING; a_fields: ROSE_LINEAR_ARRAY [STRING]; a_historical_mode: BOOLEAN): ANONYMOUS_PREDICATE is
		local
			l_statement: STRING
			l_field_ix: INTEGER
			l_path : INTEGER
		do
			if ad_hoc_query then 
				Result := Precursor(a_table_name,a_fields,a_historical_mode)
			else
				-- path field in override removal -- fa 200610
				-- following code contains SQL syntax error (see below)
				(create {EXCEPTIONS}).raise ("Unimplemented clustered multiple_field_override_cube_selection")
				l_path := path(signature_target)
				l_statement := "select signature_id,start_date,end_date,override_type_id,user_id,update_date,comment "
				from l_field_ix := a_fields.lower until l_field_ix > a_fields.upper loop
					l_statement := l_statement + ",[" + a_fields.item (l_field_ix) + "]"
					l_field_ix := l_field_ix + 1
				end
				l_statement := l_statement + 
					" from " + table_structure.override_family_name_prefix + "override_" + table_structure.data_name_prefix + "data_" + a_table_name + " v " +
					" where v.start_date <= ? "
				if not a_historical_mode then
					l_statement := l_statement +
						" and v.end_date > ? "
				end
				l_statement := l_statement + "  path = " + l_path.out -- SQL syntax error, missing AND!
				if a_table_name.as_lower.is_equal ("signatures") then
					l_statement := l_statement +
						" and override_type_id <> " + blocking_override_type_id.out
				end
				l_statement := l_statement + " order by signature_id, start_date"
				create Result.make_using_server (table_structure.signature_item_server, l_statement)
				if not Result.success then
					print (l_statement)
					(create {EXCEPTIONS}).raise ("Failed to create override_record_cube_selection predicate")
				end
			end
		end
	
end








