indexing
	description: "Cluster tagged virtual record set"
	author: "Mauricio Bustos"
	revision: "$Revision: #9 $"
	path : "$File: //rose/source/kernel/persistence/databases/signature_tagged/clustered_tagged_virtual_record_set.e $"

class
	CLUSTERED_TAGGED_VIRTUAL_RECORD_SET [G -> COMPARABLE,
										 H -> COMPARABLE,
										 I -> CLUSTERED_SIGNATURE [G, H, ENTITY] create default_create end]

inherit
	SIGNATURE_TAGGED_VIRTUAL_RECORD_SET [I]
		export
			{CLUSTERED_TAGGED_SERVER} populate_output_containers
		redefine
			make_new_similar_to,
			fill_signature_temp_table,
			drop_signatures_temp_table,
			predicate_cache
		end

create
	make, make_new_similar_to

feature {NONE} -- Initialization

	make_new_similar_to (a_other: like Current) is
			-- Make new similar to 'a_other'.
		do
			Precursor (a_other)
			set_path_tree (a_other.path_tree)
			set_ad_hoc_query (a_other.ad_hoc_query)
		end

feature -- Access

	path_tree: PARTITION_TREE [G, H]
	ad_hoc_query: BOOLEAN

feature -- Basic operations

	set_ad_hoc_query (a_value: BOOLEAN) is
		do
			ad_hoc_query := a_value
			predicate_cache.set_ad_hoc_query (a_value)
		end

	set_path_tree (a_path_tree: like path_tree) is
		do
			path_tree := a_path_tree
			predicate_cache.set_path_tree (a_path_tree)
		end

feature {SIGNATURE_TAGGED_VIRTUAL_RECORD_SET} -- Core data members

	predicate_cache: CLUSTERED_TAGGED_VIRTUAL_RECORD_PREDICATE_CACHE [G, H, I]

feature {NONE} -- Selection Space Configuration

	fill_signature_temp_table is
		local
			l_record : ROSE_VIRTUAL_RECORD [I]
		do
			if ad_hoc_query then
				Precursor
			else
				if not historical_mode then
					create cube_result.make (1, signatures.count)
				end
				from signatures.start until signatures.after loop
					l_record := cache_pool.next_signature (signatures.current_item.twin)
					--cube_result_table_by_id.extend(l_record,l_record.signature.signature_id)
					if not historical_mode then
						populate_output_containers (l_record, Void)
					end
					signatures.forth
				end
				predicate_cache.set_signature_target (signatures.first)
				if table_structure.version_pinning_enabled then
					pinned_version_retrieval := pinned_version_dates_filled and then not historical_mode
				end
			end
		end

	drop_signatures_temp_table is
		do
			if ad_hoc_query then
				Precursor
			else
				predicate_cache.set_signature_target (Void)
			end
		end

end
