indexing
	description: "Ancestor to dispatch based COM objects"
	author: ""
	date: "$Date: 2014/02/26 $"
	revision: "$Revision: #22 $"

class
	COM_DISPATCH

inherit

	COM_SMART_UNKNOWN

	COM_STRING_FUNCTIONS

	COM_DISPID

	COM_DISPATCH_CONSTANTS

	COM_ERRORS
create
	make_by_pointer

feature -- Initialization

	make_by_pointer (pp: POINTER) is
			-- Initialize with pp as the COM pointer
		do
			release
			make_by_interface_pointer (pp, iid_idispatch)
		end

feature -- Implementation	
	IDispatch_GetTypeInfoCount(this, pctinfo : POINTER): INTEGER is
			-- Encapsulation of GetTypeInfoCount
		external
			"C [macro <windows.h>](IDispatch*, unsigned int FAR* ): EIF_INTEGER"
		alias
			"IDispatch_GetTypeInfoCount"
		end
	IDispatch_GetIDsOfNames(this, riid, rgszNames: POINTER; cNames, a_lcid: INTEGER; rgDispId : POINTER): INTEGER is
			-- Encapsulation of GetIDsOfNames
		external
			"C [macro <windows.h>](IDispatch*, REFIID , OLECHAR FAR* FAR* , unsigned int , LCID , DISPID FAR* ): EIF_INTEGER"
		alias
			"IDispatch_GetIDsOfNames"
		end
	IDispatch_Invoke(this: POINTER; dispIdMember: INTEGER; riid: POINTER; a_lcid: INTEGER; wFlags: INTEGER;
			pDispParams: POINTER; pVarResult:POINTER; pExcepInfo: POINTER; puArgErr: POINTER): INTEGER is
			-- Encapsulation of Invoke
		external
			"C [macro <windows.h>](IDispatch*, DISPID , REFIID , LCID , WORD , DISPPARAMS FAR* , VARIANT FAR* , EXCEPINFO FAR* , unsigned int FAR* ): EIF_INTEGER"		alias
			"IDispatch_Invoke"
		end


	invoke (dispid, flags: INTEGER; params: COM_DISPPARAMS;
			res: COM_VARIANT; exception: COM_EXCEPINFO) is
			-- Invoke method or property with dispatch identifier `dispid'
			-- and arguments `params' according to `flags'. Result is
			-- stored in `res' and exception (if any) in `exception'.
			-- See class EOLE_METHODFLAGS for `flags' value.
			-- Not meant to be redefined; redefine `on_invoke' instead.
		require
			exists: exists
			valid_params: params /= Void and then params.exists
			valid_exception: exception /= Void and then exception.item /= default_pointer
		local
			arg_error: INTEGER
		do
			if res = Void then
				status := idispatch_invoke(p, dispid, IID_NULL, default_lcid, flags, params.item,
					default_pointer, exception.item, $arg_error)
			--	ole2_dispatch_invoke (ole_interface_ptr, dispid,
			--				flags, params.ole_ptr, default_pointer, exception.ole_ptr)
			else
				status := idispatch_invoke(p, dispid, IID_NULL, default_lcid, flags, params.item,
					res.item, exception.item, $arg_error)
			--	ole2_dispatch_invoke (ole_interface_ptr, dispid,
			--				flags, params.ole_ptr, res.item, exception.ole_ptr)
			end
		end

	IID_NULL: POINTER is
			-- Empty iid
		external
			"C [macro <windows.h>]"
		alias
			"&IID_NULL"
		end
	default_lcid: INTEGER is
			-- Default locale ID
		external
			"C [ macro <windows.h>]"
		alias
			"MAKELCID( MAKELANGID( LANG_NEUTRAL, SUBLANG_NEUTRAL ), SORT_DEFAULT )"
		end

	status: INTEGER
			-- Status of last call

	put_property (a_property_id: INTEGER; value: COM_VARIANT) is
			-- Set the COM property
		require
			positive_id: a_property_id >= 0
		do
			dispparams.init
			function_exception.init
			dispparams.add_named_arg_dispid (Dispid_propertyput)
			dispparams.add_argument (value)
			invoke (a_property_id, Dispatch_propertyput, dispparams, Void, function_exception)
			dispparams.reset
			check_result
		end

	set_string_property (a_property_id: INTEGER; a_value: STRING) is
			-- Sets a string property
		require

			positive_id: a_property_id >= 0
		local
			v: COM_VARIANT
		do
			create v
			v.init
			v.set_from_String (a_value)
			put_property (a_property_id, v)
			v.release
		end

	set_integer_property (a_property_id: INTEGER; a_value: INTEGER) is
			-- Set an integer property
		require
			positive_id: a_property_id >= 0
		local
			v: COM_VARIANT
		do
			create v
			v.init
			v.set_from_integer (a_value)
			put_property (a_property_id, v)
			v.release
		end
	set_boolean_property (a_property_id: INTEGER; a_value: BOOLEAN) is
			-- Set a boolean property
		require
			positive_id: a_property_id >= 0
		local
			v: COM_VARIANT
		do
			create v
			v.init
			v.set_boolean (a_value)
			put_property (a_property_id, v)
			v.release
		end

	set_double_property (a_property_id: INTEGER; a_value: DOUBLE) is
			-- Set an integer property
		require
			positive_id: a_property_id >= 0
		local
			v: COM_VARIANT
		do
			create v
			v.init
			v.set_real8 (a_value)
			put_property (a_property_id, v)
			v.release
		end



	get_integer_property (a_property_id: INTEGER): INTEGER is
			-- Get an integer property value
		do
			get_property (a_property_id)
			Result := g_variant.integer4
		end

	get_string_property (a_property_id: INTEGER): STRING is
			-- Get an integer property value
		do
			get_property (a_property_id)
			Result := g_variant.string
		end

	get_double_property (a_property_id: INTEGER): DOUBLE is
			-- Get an integer property value
		do
			get_property (a_property_id)
			Result := g_variant.real8
		end
	get_dispatch_property (property_id: INTEGER): POINTER is
			-- Get COM property
		do
			get_property (property_id)
			Result := g_variant.dispatch_pointer
		end

	get_boolean_property (property_id: INTEGER): BOOLEAN is
			-- Get a boolean property
		do
			get_property (property_id)
			Result := g_variant.boolean
		end

	get_property (a_property_id: INTEGER) is
			-- Implemenation for getting a property
		require
			positive_id: a_property_id >= 0
		do
			dispparams.init
			function_exception.init
			g_variant.init

			invoke (a_property_id, Dispatch_propertyget, dispparams, g_variant, function_exception)
			check_result
		end

	release_variant is
			-- Release the global variant
		do
			g_variant.clear
		end

	method (a_disp_id: INTEGER)is
			-- Invoke the method with a given id
		require
			positive_id: a_disp_id >= 0
		do
		--	dispparams.init
			function_exception.init
			g_variant.init
			g_variant.set_dispatch_pointer (default_pointer)
			invoke (a_disp_id, Dispatch_method, dispparams, g_variant, function_exception)
			check_result
		end


	com_init_error_process is
			-- Process COM intialization error
			-- By default: do nothing
		do
		end

	function_result: COM_VARIANT is
			-- Result of last `dispatch.invoke'
		once
			create Result
		end

	function_exception: COM_EXCEPINFO is
			-- Last exception that occured during `dispatch.invoke'
		once
			create Result
		end

	dispparams: COM_DISPPARAMS is
			-- `dispatch.invoke' arguments
		once
			create Result.make
		end

	g_variant: COM_VARIANT is
			-- Shared utility variant
		once
			create Result
		end

	check_result is
			-- Check result of last routine and display
			-- a message box if there was an error.
		local
			error_string: STRING
		do
			create error_string.make (20)
			if function_exception.error_code /= S_ok then
				error_string.append ("Error #")
				error_string.append_integer (function_exception.error_code)
				print(function_exception.error_description)
				print (error_string)
			end
		--	if status.last_hresult /= S_ok then
		--		error_string.append ("Dispatch Error #")
		--		error_string.append_integer (status.last_hresult)
		--		print( "Error during dispatcher call")
		--		print ( error_string)
		--	end
		end

	get_id (func: STRING): INTEGER is
			-- Get the dispatch identifier of `func'
		local
			a: ARRAY [STRING]
			k:STRING
		do
			k := generating_type + "." + func
			if (global_id_table.has (k)) then
				Result := global_id_table.item(k)
			else
			create a.make (1,1)
			a.put (func,1)
			Result := get_IDs_of_names (a).item (1)
				global_id_table.put (Result, k)
			end
		end

	global_id_table: HASH_TABLE [INTEGER, STRING] is
			-- Table of properties
		once
			create Result.make (0)
		end

	get_ids_of_names (names: ARRAY [STRING]): ARRAY [INTEGER] is
			-- Maps `names' to their 'dispids'. First element
			-- in arrays correspond to member name, others
			-- to parameter names.
			-- Not meant to be redefined; redefine `on_get_ids_of_names' instead.
		require
			valid_interface: exists
			valid_names: names /= Void
		local
			i, count, disp_id: INTEGER;
			str_p, str_p_i, str: POINTER
			disp_id_p, disp_id_p_i: POINTER
			hr: INTEGER
			pointer_size: INTEGER

		do
			pointer_size := (create {PLATFORM}).Pointer_bytes
			count := names.count
			str_p := calloc(count, pointer_size)
			disp_id_p := calloc(count, 4)
			from
				i := names.lower
			until
				i > names.upper
			loop
				str_p_i := str_p + pointer_size *(i - names.lower)
				str := string_to_widechar (names.item (i))
				str_p_i.memory_copy ($str, pointer_size)
				i := i + 1
			end
		hr := idispatch_GetIDsOfNames(p, IID_NULL, str_p, count, default_lcid, disp_id_p)
		if hr >= 0 then
			create Result.make (1, count)
			from i := 1 until i > count loop
				disp_id_p_i := disp_id_p + 4 * (i - 1)
				memcpy ($disp_id, disp_id_p_i, 4)
				Result.put (disp_id, i)
				i := i + 1
			end
		end
		free (disp_id_p)
			from
				i := names.lower
			until
				i > names.upper
			loop
				str_p_i := str_p + pointer_size *(i - names.lower)
				memcpy  ($str, str_p_i, pointer_size)
				free (str)
				i := i + 1
			end
		free (str_p)
		ensure
			good_mapping: Result /= Void implies names.count = Result.count
		end

end -- class COM_DISPATCH
