indexing
	description: "COM runtime functions"
	author: ""
	date: "$Date: 2008/12/01 $"
	revision: "$Revision: #17 $"

class
	COM_RUN_TIME

inherit
	COM_STRING_FUNCTIONS

feature -- Access

	CoCreateInstance (REFCLSID: POINTER; pUnkOuter: POINTER; dwClsContext: INTEGER; riid: POINTER; ppv: POINTER): INTEGER is
			-- Creates a single uninitialized object of the class associated with a specified CLSID.
			-- Call CoCreateInstance when you want to create only one object on the local system.
			-- To create a single object on a remote system, call CoCreateInstanceEx.
			-- To create multiple objects based on a single CLSID, refer to the CoGetClassObject function
		external
			"C blocking macro signature (REFCLSID, LPUNKNOWN, DWORD, REFIID, LPVOID *): EIF_INTEGER use <windows.h>"
		alias
			"CoCreateInstance"
		end

	CoCreateInstanceEx (rclsid: POINTER; punkOuter: POINTER; dwClsCtx: INTEGER; pServerInfo: POINTER; cmq: INTEGER; pResults: POINTER): INTEGER is
			-- Creates an instance of a specific class on a specific machine
		external
			"C blocking macro signature (REFCLSID, IUnknown *, DWORD, COSERVERINFO *, ULONG, MULTI_QI *): EIF_INTEGER use <objbase.h>"
		alias
			"CoCreateInstanceEx"
		end

	CoInitialize (pvReserved: POINTER): INTEGER is
			-- Initializes the COM library on the current thread and identifies the concurrency model as
			-- single-thread apartment (STA). Applications must initialize the COM library before they can
			-- call COM library functions other than CoGetMalloc and memory allocation functions.
			-- New applications should call CoInitializeEx instead of CoInitialize.
		external
			"C blocking macro signature (LPVOID): EIF_INTEGER use <windows.h>"
		alias
			"CoInitialize"
		end

	CoInitializeEx (pvReserved: POINTER; dwCoInit: INTEGER): INTEGER is
			-- Initializes the COM library for use by the calling thread,
			-- sets the thread's concurrency model, and creates a new apartment for the thread if one is required
		external
			"C blocking macro signature (void *, DWORD): EIF_INTEGER use <windows.h>"
		alias
			"CoInitializeEx"
		end

	CoUninitialize is
			-- Closes the COM library on the current thread, unloads all DLLs loaded by the thread,
			-- frees any other resources that the thread maintains, and forces all RPC connections on the thread to close.
		external
			"C blocking macro signature () use <windows.h>"
		alias
			"CoUninitialize()"
		end

	OleInitialize (pvReserved: POINTER): INTEGER is
			-- Initializes the COM library on the current apartment, identifies the concurrency model as
			-- single-thread apartment (STA), and enables additional functionality described in the
			-- Remarks section below. Applications must initialize the COM library before they can call
			-- COM library functions other than CoGetMalloc and memory allocation functions.
			-- Applications that do not require the additional functionality enabled by OleInitialize
			-- should call CoInitializeEx instead of OleInitialize.
		external
			"C blocking macro signature (LPVOID): EIF_INTEGER use <windows.h>"
		alias
			"OleInitialize"
		end

	OleRun (pUnknown: POINTER): INTEGER is
			-- Puts an OLE compound document object into the running state
		external
			"C blocking macro signature (LPUNKNOWN): EIF_INTEGER use <windows.h>"
		alias
			"OleRun"
		end

	CoRegisterClassObject (rclsid: POINTER; pUnk: POINTER; dwClsContext: INTEGER; flags: INTEGER; lpdwRegister: POINTER): INTEGER is
			-- Registers an EXE class object with OLE so other applications can connect to it.
			-- EXE object applications should call CoRegisterClassObject on startup.
			-- It can also be used to register internal objects for use by the same EXE or other code
			-- (such as DLLs) that the EXE uses
		external
			"C blocking macro signature (REFCLSID, IUnknown *, DWORD, DWORD, LPDWORD): EIF_INTEGER use <windows.h>"
		alias
			"CoRegisterClassObject"
		end

	guid_to_string (guid: POINTER): STRING is
		local
			p: POINTER
		do
			p := calloc (42, 2)
			StringFromGUID2 (guid, p, 41)
			Result := widechar_to_string (p)
			free (p)
		end

	StringFromGUID2 (rguid: POINTER; lpsz: POINTER; cbMax: INTEGER) is
			-- Converts a globally unique identifier (GUID) into a string of printable characters
		external
			"C blocking macro signature (REFGUID, LPOLESTR, int) use <windows.h>"
		alias
			"StringFromGUID2"
		end

	CLSIDFromString (lpsz, pclsid: POINTER): INTEGER is
			-- Converts a string generated by the StringFromCLSID function back into the original CLSID
		external
			"C blocking macro signature (LPOLESTR, LPCLSID): EIF_INTEGER use <windows.h>"
		alias
			"CLSIDFromString"
		end

	CLSIDFromProgID (lpsz, pclsid: POINTER): INTEGER is
			-- Looks up a CLSID in the registry, given a ProgID
		external
			"C blocking macro signature (LPOLESTR, LPCLSID): EIF_INTEGER use <windows.h>"
		alias
			"CLSIDFromProgID"
		end

	IsEqualGUID (rguid1, rguid2: POINTER): BOOLEAN is
			-- Determines whether two GUIDs are equal
		external
			"C blocking macro signature (REFGUID, REFGUID): EIF_BOOLEAN use <windows.h>"
		alias
			"IsEqualGUID"
		end

end
