indexing
	description: "Objects that ..."
	author: ""
	date: "$Date: 2008/12/01 $"
	revision: "$Revision: #13 $"

class
	COM_SAFEARRAY

inherit
	COM_STRUCTURE
		redefine
			out
		end
	COM_VARENUM
		undefine
			out,
			is_equal,
			copy
		end
	MEMORY
		rename
			free as eiffel_free
		undefine
			is_equal,
			copy
		redefine
			dispose,
			out
		end

create
	make,
	make_from_pointer

feature {NONE}

	make_from_pointer (a_pointer: POINTER) is
		do
			item := a_pointer
		end

	make (a_dim: INTEGER; a_type: INTEGER; bounds: COM_SAFEARRAY_BOUND) is
		require
			bounds_not_void: bounds /= Void
		do
			item := safearraycreate (a_type, a_dim, bounds.item)
		end

	dispose is
		do
			if item /= default_pointer then
				safearraydestroy (item)
				item := default_pointer
			end
		end

feature -- Access

	out: STRING is
		local
			i: INTEGER
		do
			create Result.make (0)
			Result.append ("%Ndimensions: ")
			Result.append (dimension.out)
			from
				i := 1
			until
				i > dimension
			loop
				Result.append ("%Nlower bound (")
				Result.append (i.out)
				Result.append ("): ")
				Result.append (lower_bound (i).out)
				Result.append ("%Nupper bound (")
				Result.append (i.out)
				Result.append ("): ")
				Result.append (upper_bound (i).out)
				i := i + 1
			end
			if dimension = 2 then
				Result.append (two_d_out)
			end
			Result.append ("%N")
		end

	two_d_out: STRING is
		local
			i, j: INTEGER
			var: COM_VARIANT
			l_index: ARRAY [INTEGER]
		do
			create Result.make (0)
			create var
			var.init
			from
				i := lower_bound (1)
			until
				i > upper_bound (1)
			loop
				from
					j := lower_bound (2)
				until
					j > upper_bound (2)
				loop
					l_index := <<i, j>>
					get_element (l_index, var)
					Result.append (var.variant_out)
					Result.append ("(" + var.var_type.out + ")")
					Result.append (" .. ")
					j := j + 1
				end
				Result.append ("%N")
				i := i + 1
			end
		end

	lower_bound (dim_number: INTEGER): INTEGER is
			-- Upper bound of dimension `dim_number'
		local
			i: INTEGER
		do
			i := safearraygetlbound (item, dim_number, $Result)
			if i /= 0 then
				print ("bad lower_bound index")
				print (dim_number)
			end
		end

	upper_bound (dim_number: INTEGER): INTEGER is
			-- Upper bound of dimension `dim_number'
		local
			i: INTEGER
		do
			i := safearraygetubound (item, dim_number, $Result)
			if i /= 0 then
				print ("bad upper_bound index")
				print (dim_number)
			end
		end

	dimension: INTEGER is
		require
			exists: exists
		do
			Result := safearraygetdim (item)
		end

	put_element (index: ARRAY [INTEGER]; var: COM_VARIANT) is
			-- Set element at `index' with `var'.
		require
			valid_c_structure: item /= default_pointer
			valid_variant: var /= Void and then var.item /= default_pointer and var.var_type /= vt_array and var.var_type /= vt_null
			valid_index: is_valid_index (index)
		local
			a: ANY
			i: INTEGER
		do
			a := index.to_c
			i := safearrayputelement (item, $a, var.item)
			if i /= 0 then
				print ("bad put_element")
			end
		end

	get_element (index: ARRAY [INTEGER]; var: COM_VARIANT) is
		require
			valid_c_structure: item /= default_pointer
			valid_index: is_valid_index (index)
		local
			a: ANY
			i: INTEGER
		do
			a := index.to_c
			i := safearraygetelement (item, $a, var.item)
			if i /= 0 then
				print ("bad get_element")
			end
		end

	is_valid_index (index: ARRAY [INTEGER]): BOOLEAN is
			-- Is `index' valid?
		local
			i: INTEGER
		do
			Result := index /= Void
			if not Result then
				print ("Void index")
			end
			if Result then
				from
					Result := index.count = dimension
					if not Result then
						print ("bad index dimension")
						print (index.count)
						print (" insead of ")
						print (dimension)
					end
					i := index.lower
				until
					not Result or i > index.upper
				loop
					Result := index.item (i) <= upper_bound (i) and index.item (i) >= lower_bound (i)
					if not Result then
						print ("bad dimension")
						print (i)
						print ("  : ")
						print (index.item (i))
						print ("is not in [")
						print (lower_bound (i))
						print (", ")
						print (upper_bound (i))
						print ("]")
					end
					i := i + 1
				end
			end
		end

feature {NONE}

	cwel_size_of_safearray: INTEGER is
		external
			"C [macro <windows.h>] "
		alias
			"sizeof(SAFEARRAY)"
		end

	structure_size: INTEGER is
		do
			Result := cwel_size_of_safearray
		end

	offsetofcdims: INTEGER is
		external
			"C [macro <windows.h>]"
		alias
			"offsetof(SAFEARRAY, cDims)"
		end

	set_cdims (a_value: INTEGER) is
		do
			set_integer_value (a_value, offsetofcdims)
		end

	get_cdims: INTEGER is
		do
			Result := get_integer_value (offsetofcdims)
		end

	offsetofffeatures: INTEGER is
		external
			"C [macro <windows.h>]"
		alias
			"offsetof(SAFEARRAY, fFeatures)"
		end

	set_ffeatures (a_value: INTEGER) is
		do
			set_integer_value (a_value, offsetofffeatures)
		end

	get_ffeatures: INTEGER is
		do
			Result := get_integer_value (offsetofffeatures)
		end

	offsetofcbelements: INTEGER is
		external
			"C [macro <windows.h>]"
		alias
			"offsetof(SAFEARRAY, cbElements)"
		end

	set_cbelements (a_value: INTEGER) is
		do
			set_integer_value (a_value, offsetofcbelements)
		end

	get_cbelements: INTEGER is
		do
			Result := get_integer_value (offsetofcbelements)
		end

	offsetofclocks: INTEGER is
		external
			"C [macro <windows.h> ]"
		alias
			"offsetof(SAFEARRAY, cLocks)"
		end

	set_clocks (a_value: INTEGER) is
		do
			set_integer_value (a_value, offsetofclocks)
		end

	get_clocks: INTEGER is
		do
			Result := get_integer_value (offsetofclocks)
		end

	offsetofpvdata: INTEGER is
		external
			"C [macro <windows.h>]"
		alias
			"offsetof(SAFEARRAY, pvData)"
		end

	set_pvdata (a_value: POINTER) is
		do
			set_pointer_value (a_value, offsetofpvdata)
		end

	get_pvdata: POINTER is
		do
			Result := get_pointer_value (offsetofpvdata)
		end

	offsetofrgsabound: INTEGER is
		external
			"C [macro <windows.h>]"
		alias
			"offsetof(SAFEARRAY, rgsabound)"
		end

	set_rgsabound (a_value: POINTER) is
		do
			set_pointer_value (a_value, offsetofrgsabound)
		end

	get_rgsabound: POINTER is
		do
			Result := get_pointer_value (offsetofrgsabound)
		end

	safearraygetlbound (psa: POINTER; ndim: INTEGER; plubound: POINTER): INTEGER is
		external
			"C [macro <oleauto.h>](SAFEARRAY FAR*, unsigned int, long FAR*): EIF_INTEGER"
		alias
			"SafeArrayGetLBound"
		end

	safearraygetubound (psa: POINTER; ndim: INTEGER; plubound: POINTER): INTEGER is
		external
			"C [macro <oleauto.h>](SAFEARRAY FAR*, unsigned int, long FAR*): EIF_INTEGER"
		alias
			"SafeArrayGetUBound"
		end

	safearrayputelement (psa, rgindices, pv: POINTER): INTEGER is
		external
			"C [macro <oleauto.h>](SAFEARRAY FAR*, long FAR*, void FAR*): EIF_INTEGER"
		alias
			"SafeArrayPutElement"
		end

	safearraygetelement (psa, rgindices, pv: POINTER): INTEGER is
		external
			"C [macro <oleauto.h>](SAFEARRAY FAR*, long FAR*, void FAR*): EIF_INTEGER"
		alias
			"SafeArrayGetElement"
		end

	safearraydestroy (psa: POINTER) is
		external
			"C [macro <oleauto.h>](SAFEARRAY FAR*)"
		alias
			"SafeArrayDestroy"
		end

	safearraygetdim (psa: POINTER): INTEGER is
		external
			"C [macro <oleauto.h>](SAFEARRAY FAR*): EIF_INTEGER"
		alias
			"SafeArrayGetDim"
		end

	safearraycreate (vt: INTEGER; cdims: INTEGER; b: POINTER): POINTER is
		external
			"C [macro <oleauto.h>](VARTYPE , UINT , SAFEARRAYBOUND * ): EIF_POINTER"
		alias
			"SafeArrayCreate"
		end

end -- class COM_SAFEARRAY
