indexing
	description: "Management of pointers to COM objects, accordingly to a memory model"
	author: "Pierre Nallet"
	date: "$Date: 2008/12/01 $"
	revision: "$Revision: #9 $"

class
	COM_SMART_POINTER [MM -> MEMORY_MODEL create default_create end]

inherit
	MEMORY
		rename
			free as eiffel_free
		redefine
			dispose
		end

create
	make

feature {NONE} -- Initialization

	make is
		local
			l_model: MM
		do
			create l_model.default_create
			memory_model := l_model.id
		end

feature -- Access

	memory_model: INTEGER
			-- Associated memory model

	item: POINTER
			-- Pointer to associated memory zone

	address: POINTER is
		do
			Result := id ($item)
		end

feature -- Basic operations

	dispose is
			-- Action to be executed just before garbage collection
			-- reclaims this object.
		do
			if item /= default_pointer then
--				memory_model.free (item)
				inspect memory_model
				when 1 then
					-- RUN_TIME_MEMORY_MODEL
					c_free (item)
				when 2 then
					-- COM_MEMORY_MODEL
					CoTaskMemFree (item)
				when 3 then
					-- STACK_MEMORY_MODEL
				else
					print ("Warning: uknown memory model " + memory_model.out + "%N")
				end
				item := default_pointer
			end
		end

feature {NONE} -- Implementation

	id (p: POINTER): POINTER is
		do
			Result := p
		end

	c_calloc (a_size, a_byte_size: INTEGER): POINTER is
		external
			"C [macro <string.h>] (size_t, size_t): EIF_POINTER"
		alias
			"calloc"
		end

	c_free (a_pointer: POINTER) is
		external
			"C [macro <string.h>] (void *) "
		alias
			"free"
		end

	CoTaskMemAlloc (cb: INTEGER) : POINTER is
		external
			"C [macro <windows.h>](ULONG): EIF_POINTER"
		alias
			"CoTaskMemAlloc"
		end

	CoTaskMemFree(pv: POINTER) is
		external
			"C [macro <windows.h>](LPVOID )"
		alias
			"CoTaskMemFree"
		end

end -- class COM_SMART_POINTER
