indexing
	description: "COM string manipulation functions"
	author: "Pierre Nallet"
	date: "$Date: 2008/12/01 $"
	revision: "$Revision: #19 $"

class
	COM_STRING_FUNCTIONS

inherit
	MEMORY_FUNCTIONS

feature -- Access

	widechar_to_string (p: POINTER): STRING is
		local
			str_length: INTEGER
			str_pointer: POINTER
		do
			str_length := WideCharToMultiByte (0, 0, p, -1, default_pointer, 0, default_pointer, default_pointer)
			str_pointer := calloc (1, str_length + 1)
			str_length := WideCharToMultiByte (0, 0, p, -1, str_pointer, str_length, default_pointer, default_pointer)
			create Result.make (0)
			Result.from_c (str_pointer)
			free (str_pointer)
		end

	widechar_length_to_string (p: POINTER; l: INTEGER): STRING is
		local
			str_pointer: POINTER
			i: INTEGER
			l_length_required: INTEGER
		do
			if l = 0 then
				create Result.make_empty
			elseif l > 0 then
				l_length_required := WideCharToMultiByte (0, 0, p, l, default_pointer, 0, default_pointer, default_pointer)
				str_pointer := calloc (1, l_length_required + 1)
				i := WideCharToMultiByte (0, 0, p, l, str_pointer, l_length_required + 1, default_pointer, default_pointer)
				if i /= l then
					--io.put_string ("Error in widechar_length_to_string%N")
				end
				create Result.make (0)
				Result.from_c (str_pointer)
				free (str_pointer)
			end
		end

	string_to_widechar (s: STRING): POINTER is
		require
			s_not_void: s /= Void
		local
			a: C_STRING
			c: INTEGER
		do
			create a.make (s)
			c := MultiByteToWideChar (0, 0, a.item, -1, default_pointer, 0)
			Result := calloc (2, c + 1)
			c := MultiByteToWideChar (0, 0, a.item, -1, Result, c)
		ensure
			valid_result: widechar_to_string (Result).is_equal (s)
		end

	string_to_bstr (s: STRING): POINTER is
		local
			p: POINTER
		do
			p := string_to_widechar (s)
			result := SysAllocString (p)
			free (p)
		end

	convert_widechar_to_integer (a: POINTER): INTEGER is
		local
			l_null: POINTER
		do
			Result := wcstol (a,$l_null,10)
		end

	compare_widechar_strings (a, b: POINTER; c: INTEGER): INTEGER is
		external
			"C [macro <string.h>] (const wchar_t *,const wchar_t *, size_t): EIF_INTEGER"
		alias
			"wcsncmp"
		end

feature {NONE} -- Implementation

	wcstol (a, b: POINTER; c: INTEGER): INTEGER is
		external
			"C [macro <stdlib.h>] (const wchar_t *,wchar_t **,int): EIF_INTEGER"
		alias
			"wcstol"
		end

	WideCharToMultiByte (CodePage, dwFlags: INTEGER; lpWideCharStr: POINTER; cchWideChar: INTEGER;
			lpMultiByteStr: POINTER; cchMultiByte: INTEGER; lpDefaultChar, lpUsedDefaultChar: POINTER): INTEGER is
		external
			"C [macro <Windows.h>] (UINT, DWORD , LPCWSTR , int , LPSTR , int, LPCSTR, LPBOOL): EIF_INTEGER"
		alias
			"WideCharToMultiByte"
		end

	MultiByteToWideChar (codepage, dwflags: INTEGER; lpMultiByteStr: POINTER; cbMultiByte: INTEGER;lpWideCharStr: POINTER; cchWideChar: INTEGER): INTEGER is
		external
			"C [macro <Windows.h>] (UINT, DWORD , LPCSTR , int , LPWSTR , int): EIF_INTEGER"
		alias
			"MultiByteToWideChar"
		end

	SysStringLen (bstr: POINTER): INTEGER is
		external
			"C [macro <allwindows.h>] (BSTR): EIF_INTEGER"
		alias
			"SysStringLen"
		end

	SysFreeString (bstr: POINTER) is
		external
			"C [macro <allwindows.h>] (BSTR)"
		alias
			"SysFreeString"
		end

	SysAllocString (bstr: POINTER): POINTER is
		external
			"C [macro <allwindows.h>] (OLECHAR * ): EIF_POINTER"
		alias
			"SysAllocString"
		end

end
