indexing
	description: "Functions for accessing external structures"
	author: "Pierre Nallet"
	date: "$Date: 2009/10/20 $"
	revision: "$Revision: #14 $"

deferred class
	COM_STRUCTURE_FUNCTIONS

inherit
	MEMORY_FUNCTIONS

feature -- Access

	item: POINTER is
			-- Pointer to the actual structure
		deferred
		end

	structure_size: INTEGER is
			-- Size of the structure
		deferred
		end

	exists: BOOLEAN is
			-- Is this object attached to an actual structure
		do
			Result := item /= default_pointer
		end

	get_pointer_value (a_offset: INTEGER): POINTER is
		require
			exists: exists
		local
			l_pointer: POINTER
		do
			l_pointer := item + a_offset
			memcpy ($Result, l_pointer, sizeof_pointer)
		end

	get_integer_value (a_offset: INTEGER): INTEGER is
		require
			exists: exists
		local
			l_pointer: POINTER
		do
			l_pointer := item + a_offset
			memcpy ($Result, l_pointer, 4)
		end

	get_integer_8_value (a_offset: INTEGER): INTEGER is
		require
			exists: exists
		local
			l_pointer: POINTER
		do
			l_pointer := item + a_offset
			memcpy ($Result, l_pointer, 1)
		end

	get_word_value (a_offset: INTEGER): INTEGER is
		require
			exists: exists
		local
			l_pointer: POINTER
		do
			l_pointer := item + a_offset
			memcpy ($Result, l_pointer, 2)
		end

	get_byte_value (a_offset: INTEGER): INTEGER is
		require
			exists: exists
		local
			l_pointer: POINTER
		do
			l_pointer := item + a_offset
			memcpy ($Result, l_pointer, 1)
		end

feature -- Basic operations

	set_integer_value (a_value: INTEGER; a_offset: INTEGER) is
		require
			exists: exists
		local
			l_pointer: POINTER
		do
			l_pointer := item + a_offset
			memcpy (l_pointer, $a_value, 4)
		ensure
			set: get_integer_value (a_offset) = a_value
		end

	set_integer_8_value (a_value: INTEGER_8; a_offset: INTEGER) is
		require
			exists: exists
		local
			l_pointer: POINTER
		do
			l_pointer := item + a_offset
			memcpy (l_pointer, $a_value, 1)
		ensure
			set: get_integer_8_value (a_offset) = a_value
		end

	set_word_value (a_value: INTEGER; a_offset: INTEGER) is
		require
			exists: exists
		local
			l_pointer: POINTER
		do
			l_pointer := item + a_offset
			memcpy (l_pointer, $a_value, 2)
		ensure
			set: get_word_value (a_offset) = a_value
		end

	set_byte_value (a_value: INTEGER; a_offset: INTEGER) is
		require
			exists: exists
		local
			l_pointer: POINTER
		do
			l_pointer := item + a_offset
			memcpy (l_pointer, $a_value, 1)
		ensure
			set: get_byte_value (a_offset) = a_value
		end

	set_pointer_value (a_value: POINTER; a_offset: INTEGER) is
			-- Set a pointer at a given offset.
		require
			exists: exists
		local
			l_pointer: POINTER
		do
			l_pointer := item + a_offset
			memcpy (l_pointer, $a_value, sizeof_pointer)
		ensure
			set: get_pointer_value (a_offset) = a_value
		end

feature {NONE} -- Implementation

	sizeof_pointer: INTEGER is
		-- Size of pointers
		external " C [macro <oledb.h>]" alias "sizeof (void*)" end


end
