note
	description:

		"Eiffel wrapper of the IUnknown interface."

	author: "Pierre Nallet"
	last_editor: "$Author: rj $"
	date: "$Date: 2012/07/19 $"
	revision: "$Revision: #24 $"
	path: "$File: //rose/source/kernel/infrastructure/windows/com/com_unknown.e $"

class
	COM_UNKNOWN

inherit
	COM_IUNKNOWN_INTERFACE

	COM_RUN_TIME

	COM_STRING_FUNCTIONS

	COM_IID

feature -- Features.
	p: POINTER

	exists: BOOLEAN is
		do
			Result := (p /= default_pointer)
		end

	set_pointer (a_p: POINTER) is
	do
		p := a_p
	end

	id_ (x: POINTER): POINTER is
		do
			Result := x
		end

	address: POINTER is
		do
			Result := id_ ($p)
		end

	create_instance (a_clsid: POINTER; dwClsContext: INTEGER; riid: POINTER) is
		local
			hr: INTEGER
			l_p : POINTER
		do
			l_p := p
			hr := CoCreateInstance(a_clsid, default_pointer, dwClsContext,  riid, $l_p)
			p := l_p

		end

	create_runnable_instance (a_clsid: POINTER; dwClsContext: INTEGER; riid: POINTER) is
		local
			hr: INTEGER
			pp: POINTER
		do
			hr := CoCreateInstance(a_clsid, default_pointer, dwClsContext,  IID_IUnknown, $pp)
			if hr >= 0 then
				hr := olerun (pp)
				hr := iunknown_queryinterface (pp, riid, $p)
				hr := IUnknown_release (pp)
			end
		end

	create_remote_instance (a_machine_name: STRING; a_clsid: POINTER; dwClsContext: INTEGER; riid: POINTER) is
		local
			coserverinfo: COM_COSERVERINFO [RUN_TIME_MEMORY_MODEL]
			multi_qi: COM_MULTI_QI [RUN_TIME_MEMORY_MODEL]
			wname: POINTER
			hr: INTEGER
		do
			create coserverinfo.make
			wname :=string_to_widechar (a_machine_name)
			coserverinfo.set_pwszname (wname)
			create multi_qi.make
			multi_qi.set_piid (riid)
			hr := CoCreateInstanceEx (a_clsid, default_pointer, dwClsContext, coserverinfo.item, 1, multi_qi.item)
			if hr < 0 then
				print ("CoCreateInstanceEx failed, hr = ")
				print (hr)
			end
			p := 	multi_qi.get_pitf
			free (wname)
		end

	make_by_interface_pointer (pp, iid: POINTER)is
		require
			pp_not_void: pp /= default_pointer
		local
			hr: INTEGER
			l_p: POINTER
		do
			l_p := p
			hr := IUnknown_QueryInterface (pp, iid, $l_p)
			p := l_p
		end

	query_interface (iid: POINTER; other: POINTER): INTEGER is
		require
			exists: exists
		do
			Result := IUnknown_QueryInterface (p, iid, other)
		end

	add_ref is
			-- Increment the reference count.
		require
			exists: exists
		local
			i: INTEGER
		do
			i := IUnknown_AddRef (p)
		end

	ref_count: INTEGER
			-- Increments the ref count and then decrements the ref count in order to determine the current true ref count.
		do
			if p/=default_pointer
			then
				add_ref
				Result := IUnknown_Release(p)
			else
				Result := -1
			end
		end

	release
			-- Tell OLEDB that we no longer need this object.  Wraps IUnknown::Release.
		local
			i: INTEGER
		do
			if p /= default_pointer then
				i := IUnknown_Release (p)
				p := default_pointer
			end
		end

end
