indexing
	description: "COM_VARIANT structure wrapper to set and retrieve data"
	author: "Pierre Nallet"
	last_editor: "$Author: er $"
	date: "$Date: 2014/02/26 $"
	revision: "$Revision: #36 $"

class
	COM_VARIANT

inherit
	MEMORY_FUNCTIONS
	COM_STRING_FUNCTIONS
	COM_VARENUM

feature -- Access

	item: POINTER
		-- Pointer

	get_date: DOUBLE is
		do
			Result := c_variant_get_date (item)
		end

	get_dblval, real8: DOUBLE is
		do
			Result := c_variant_get_dblval (item)
		end

	get_bstrval: POINTER is
		do
			Result := c_variant_get_bstrval (item)
		end

	get_ival: INTEGER is
		do
			Result := c_variant_get_ival (item)
		end

	integer4: INTEGER is
		-- Value as an integer
		do
			change_type (vt_i4)
			Result := get_lval
		end

	integer8: INTEGER_64 is
		do
			change_type (vt_i8)
			Result := c_variant_get_llval (item)
		end

	get_vt, var_type: INTEGER is
		do
			Result := c_variant_get_vt (item)
		end

	get_lval: INTEGER is
		do
			Result := c_variant_get_lval (item)
		end

	string: STRING is
		do
			Result := widechar_to_string (get_bstrval)
		end

	get_bval: POINTER is
		do
			Result := c_variant_get_bval (item)
		end

	get_fltval: REAL is
		do
			Result := c_variant_get_fltval (item)
		end

	real4: REAL is
		do
			Result := 	get_fltval
		end

	get_boolval: INTEGER is
		do
			Result := c_variant_get_boolval (item)
		end

	boolean: BOOLEAN is
			-- Value as a boolean
		do
			Result := get_boolval /= 0
		end

	get_scode: POINTER is
		do
			Result := c_variant_get_scode (item)
		end

	get_pdispval, dispatch_pointer: POINTER is
		do
			Result := c_variant_get_pdispval (item)
		end

	get_punkval: POINTER is
		do
			Result := c_variant_get_punkval (item)
		end

feature -- Basic operations

	init is
			-- Initialize
		do
			allocate
		end

	allocate is
			-- Allocate memory for the variant
		do
			set (calloc (cwel_size_of_variant, 1))
		end

	deallocate is
			-- Deallocate the memomery for the variant.
		do
			free (item)
			item := default_pointer
		end

	release is
			-- Clear and deallocate variant.
		do
			clear
			deallocate
		end

	clear is
			-- Clear this variant.
		do
			VariantClear (item)
		end

	change_type (new_type: INTEGER) is
		do
			VariantChangeType (item, item, 0, new_type)
		end

	set (p: POINTER) is
			-- Set the item to p
		do
			item := p
		end

	set_from_string (s: STRING) is
		do
			clear
			set_bstrval (string_to_bstr(s))
			set_vt (vt_bstr)
		end

	set_from_real, set_real4(r: REAL) is
		do
			clear
			set_fltval (r)
			set_vt (vt_r4)
		end

	set_real8 (d: DOUBLE) is
		do
			clear
			set_dblval (d)
			set_vt (vt_r8)
		end

	set_from_integer, set_integer4 (i: INTEGER) is
		do
			clear
			set_lval (i)
			set_vt (vt_i4)
		end

	set_from_integer_64 (i: INTEGER_64) is
			-- Set the value is integer 64
		do
			clear
			c_variant_set_llval (item, i)
			set_vt (vt_i8)
		end

	set_bval (a_value: POINTER) is
		do
			c_variant_set_bval (item, a_value)
		end

	set_fltval (a_value: REAL) is
		do
			c_variant_set_fltval (item, a_value)
		end

	set_boolean (b: BOOLEAN) is
		do
			if b then
				set_boolval (-1)
			else
				set_boolval (0)
			end
			set_vt (vt_bool)
		ensure
			set: boolean = b
		end

	set_boolval (a_value: INTEGER) is
		do
			c_variant_set_boolval (item, a_value)
		end

	set_scode (a_value: POINTER) is
		do
			c_variant_set_scode (item, a_value)
		end

	set_pdispval , set_dispatch_pointer(a_value: POINTER) is
		do
			c_variant_set_pdispval (item, a_value)
		end

	set_punkval (a_value: POINTER) is
		do
			c_variant_set_punkval (item, a_value)
		end

	set_date (a_value: ROSE_DATE) is
			-- Set the date value
		do
			set_double_date (a_value.to_double  - double_date_origin)
		end

	double_date_origin: DOUBLE is
			-- Origin date as a double
		local
			d: ROSE_DATE
		once
			create d.make (1900, 1, 1)
			Result := d.to_double - 2
		end

	set_double_date (a_value: DOUBLE) is
			-- Set the value as a date - double
		do
			clear
			c_variant_set_date (item, a_value)
			set_vt (vt_date)
		end

	set_dblval (a_value: DOUBLE) is
		do
			c_variant_set_dblval (item, a_value)
		end

	set_bstrval, set_bstr (a_value: POINTER) is
		do
			c_variant_set_bstrval (item, a_value)
			c_variant_set_vt (item, VT_BSTR)
		end

	set_ival (a_value: INTEGER) is
		do
			c_variant_set_ival (item, a_value)
		end

	set_vt, set_var_type(a_value: INTEGER) is
		do
			c_variant_set_vt (item, a_value)
		end

	set_lval (a_value: INTEGER) is
		do
			c_variant_set_lval (item, a_value)
		end

	set_dispatch (disp: POINTER) is
			-- Set value to IDispatch FAR * `disp'.
		require
			valid_c_structure: item /= default_pointer
		do
			clear
			set_dispatch_pointer (disp)
			set_vt(VT_DISPATCH)
		end

feature -- Conversion

	to_any: ANY is
			-- Convert to any object
		local
			l_safe_array: COM_SAFEARRAY
			l_var: COM_VARIANT
			l_array: ARRAY [ANY]
			i: INTEGER
			l_double: DOUBLE
			l_date: ROSE_DATE
			l_decimal: RP_DECIMAL
			l_ptr: MANAGED_POINTER
		do
			if (var_type = vt_empty) then
			elseif var_type = vt_i4 then Result := integer4
			elseif var_type = vt_r4 then Result := real4
			elseif var_type = vt_r8 then Result := real8
			elseif var_type = vt_bool then Result := boolean
			elseif var_type = vt_bstr then Result := string
			elseif var_type = vt_error then Result := integer4
			elseif var_type = vt_decimal then
				create l_ptr.share_from_pointer (Current.item, 16)
				create l_decimal.make_managed_pointer (l_ptr)
				Result := l_decimal
			elseif (var_type = 8204) then
				create l_safe_array.make_from_pointer (Current.v_array (item))
				create l_array.make (l_safe_array.lower_bound (1), l_safe_array.upper_bound (1))
				from i := l_safe_array.lower_bound (1) until i > l_safe_array.upper_bound (1) loop
					create l_var
					l_var.init
					l_safe_array.get_element (<<i>>, l_var)
					l_array.put (l_var.to_any, i)
					l_var.release
					i := i + 1
				end
				Result := l_array
				l_safe_array.set (default_pointer)
			elseif var_type = vt_array + vt_i4 then
				Result := to_int_array
			elseif var_type = vt_array + vt_bstr then
				--	print ("array of string")
				Result := to_string_array
			elseif var_type = vt_date then
				l_double := get_date
				create l_date.make_ccyymmdd (18991230)
				l_date.add_n_days (l_double.truncated_to_integer)
				l_date.second_add ((l_double - l_double.truncated_to_integer).truncated_to_integer * 24 * 3600)
				Result := l_date
			elseif var_type = vt_null then
				-- Result := Void
			else
				print("COM_VARIANT.to_any - no support for type: " + var_type.out + "%N")
			end
		end

	to_int_array: ARRAY [INTEGER] is
		local
			l_ptr: POINTER
			l_integer: INTEGER
			l_safe_array: COM_SAFEARRAY
			i: INTEGER
		do
			create l_safe_array.make_from_pointer (Current.v_array (item))

			SafeArrayAccessData(Current.v_array (item), $l_ptr)
			create Result.make (l_safe_array.lower_bound (1), l_safe_array.upper_bound (1))
			from i := Result.lower until i > Result.upper loop
				c_memcpy($l_integer, l_ptr, 4)
				Result.put (l_integer, i)
				l_ptr := l_ptr + sizeofbstrstar
				i := i + 1
			end
		end

	to_string_array: ARRAY [STRING] is
		local
			l_ptr: POINTER
			l_bstr: POINTER
			l_safe_array: COM_SAFEARRAY
			i: INTEGER
		do
			create l_safe_array.make_from_pointer (Current.v_array (item))

			SafeArrayAccessData(Current.v_array (item), $l_ptr)
			create Result.make (l_safe_array.lower_bound (1), l_safe_array.upper_bound (1))
			from i := Result.lower until i > Result.upper loop
				c_memcpy($l_bstr, l_ptr, sizeofbstrstar)
				Result.put (widechar_to_string (l_bstr), i)
				l_ptr := l_ptr + sizeofbstrstar
				i := i + 1
			end
		end

	variant_out: STRING is
		do
			--	if var_type = vt_variant + vt_array then
			--		 Result := safearray.out
			if var_type = vt_i4 then Result := integer4.out
			elseif var_type = vt_r4 then Result := real4.out
			elseif var_type = vt_r8 then Result := real8.out
			elseif var_type = vt_bool then Result := boolean.out
			elseif var_type = vt_bstr then Result := string.out
			elseif var_type = vt_error then Result := integer4.out
			else
				Result := "not implemented: "
				Result.append (var_type.out)
			end
		end

feature -- Exported externals

	v_array (p: POINTER): POINTER is
		external
			"C [macro <windows.h>] (VARIANT*): EIF_POINTER"
		alias
			"V_ARRAY"
		end

feature {NONE} -- Externals

	c_memcpy (destination, source: POINTER; count: INTEGER) is
			-- C memcpy
		external
			"C (void *, const void *, size_t) | <string.h>"
		alias
			"memcpy"
		end

	VariantClear (p: POINTER) is
		external
			"C [macro <OLEAUTO.H>] (VARIANTARG *)"
		alias
			"VariantClear"
		end

	c_variant_set_vt (struct: POINTER; a_value: INTEGER) is
		external
			"C [macro <ecom.h>]"
		alias
			"c_variant_set_vt"
		end

	c_variant_get_vt (struct: POINTER): INTEGER is
		external
			"C [macro <ecom.h>]"
		alias
			"c_variant_get_vt"
		end

	c_variant_set_lval (struct: POINTER; a_value: INTEGER) is
		external
			"C [macro <ecom.h>]"
		alias
			"c_variant_set_lval"
		end

	c_variant_set_llval (p: POINTER; v: INTEGER_64) is
			-- Set field llval of struct pointed by `p'.
		external
			"C inline use <windows.h>"
		alias
			"[
			#ifdef NONAMELESSUNION
				(((VARIANT *) $p)->__VARIANT_NAME_1.__VARIANT_NAME_2.__VARIANT_NAME_3.llVal) = (LONGLONG)($v);
			#else
				((VARIANT *) $p)->llVal = (LONGLONG)($v);
			#endif
			]"
		end

	c_variant_get_llval (p: POINTER): INTEGER_64 is
			-- Set field llval of struct pointed by `p'.
		external
			"C inline use <windows.h>"
		alias
			"[
			#ifdef NONAMELESSUNION
				return ((VARIANT *) $p)->__VARIANT_NAME_1.__VARIANT_NAME_2.__VARIANT_NAME_3.llVal;
			#else
				return ((VARIANT *) $p)->llVal;
			#endif
			]"
		end

	c_variant_get_lval (struct: POINTER): INTEGER is
			-- Encapsulation of lval
		external
			"C [macro <ecom.h>]"
		alias
			"c_variant_get_lval"
		end

	c_variant_set_bval (struct: POINTER; a_value: POINTER) is
		external
			"C [macro <ecom.h>]"
		alias
			"c_variant_set_bval"
		end

	c_variant_get_bval (struct: POINTER): POINTER is
		external
			"C [macro <ecom.h>]"
		alias
			"c_variant_get_bval"
		end

	c_variant_set_ival (struct: POINTER; a_value: INTEGER) is
		external
			"C [macro <ecom.h>]"
		alias
			"c_variant_set_ival"
		end

	c_variant_get_ival (struct: POINTER): INTEGER is
		external
			"C [macro <ecom.h>]"
		alias
			"c_variant_get_ival"
		end


	c_variant_set_fltval (struct: POINTER; a_value: REAL) is
		external
			"C [macro <ecom.h>]"
		alias
			"c_variant_set_fltval"
		end

	c_variant_get_fltval (struct: POINTER): REAL is
		external
			"C [macro <ecom.h>]"
		alias
			"c_variant_get_fltval"
		end

	c_variant_set_dblval (struct: POINTER; a_value: DOUBLE) is
		external
			"C [macro <ecom.h>]"
		alias
			"c_variant_set_dblval"
		end

	c_variant_get_dblval (struct: POINTER): DOUBLE is
		external
			"C [macro <ecom.h>]"
		alias
			"c_variant_get_dblval"
		end

	c_variant_set_boolval (struct: POINTER; a_value: INTEGER) is
		external
			"C [macro <ecom.h>]"
		alias
			"c_variant_set_boolval"
		end

	c_variant_get_boolval (struct: POINTER): INTEGER is
		external
			"C [macro <ecom.h>]"
		alias
			"c_variant_get_boolval"
		end

	c_variant_set_scode (struct: POINTER; a_value: POINTER) is
		external
			"C [macro <ecom.h>]"
		alias
			"c_variant_set_scode"
		end

	c_variant_get_scode (struct: POINTER): POINTER is
		external
			"C [macro <ecom.h>]"
		alias
			"c_variant_get_scode"
		end

	c_variant_set_date (struct: POINTER; a_value: DOUBLE) is
		external
			"C [macro <ecom.h>]"
		alias
			"c_variant_set_date"
		end

	c_variant_get_date (struct: POINTER): DOUBLE is
		external
			"C [macro <ecom.h>]"
		alias
			"c_variant_get_date"
		end

	c_variant_set_bstrval (struct: POINTER; a_value: POINTER) is
		external
			"C [macro <ecom.h>]"
		alias
			"c_variant_set_bstrval"
		end

	c_variant_get_bstrval (struct: POINTER): POINTER is
		external
			"C [macro <ecom.h>]"
		alias
			"c_variant_get_bstrval"
		end

	c_variant_set_punkval (struct: POINTER; a_value: POINTER) is
		external
			"C [macro <ecom.h>]"
		alias
			"c_variant_set_punkval"
		end

	c_variant_get_punkval (struct: POINTER): POINTER is
		external
			"C [macro <ecom.h>]"
		alias
			"c_variant_get_punkval"
		end

	c_variant_set_pdispval (struct: POINTER; a_value: POINTER) is
		external
			"C [macro <ecom.h>]"
		alias
			"c_variant_set_pdispval "
		end

	c_variant_get_pdispval (struct: POINTER): POINTER is
		external
			"C [macro <ecom.h>]"
		alias
			"c_variant_get_pdispval"
		end

	VariantChangeType ( pvargDest , pvarSrc: POINTER; wFlags , newtype: INTEGER) is
		external
			"C [macro <oleauto.h>] (VARIANTARG * , VARIANTARG * , unsigned short , VARTYPE )"
		alias
			"VariantChangeType"
		end

	SafeArrayAccessData (safeArray: POINTER; data: POINTER) is
		external
			"C [macro <oleauto.h>] (SAFEARRAY * , void HUGEP** )"
		alias
			"SafeArrayAccessData"
		end

feature {NONE} -- Size of externals

	cwel_size_of_variant: INTEGER is
		external
			"C [macro <windows.h>]"
		alias
			" sizeof(VARIANT)"
		end

	sizeofbstrstar: INTEGER is
		external
			"C [macro <windows.h>]"
		alias
			"sizeof(BSTR*)"
		end

end


