note

	description:

		"ROSE_CONTEXT version of PERSISTENT_COMPANY"

	author: "Mark Howard"
	last_editor: "$Author: ext-asurelia $"
	date: "$Date: 2015/02/22 $"
	revision: "$Revision: #118 $"
	path: "$File: //rose/source/kernel/investment_entities/company.e $"

class COMPANY

inherit

	INVESTMENT_ENTITY
		undefine

			peer_anchor,
			expression_tag,
			descriptive_name,
			name_identifier_keys,
			name_identifier_key
		end

	PERSISTENT_COMPANY
		undefine
			context,
			make,
			make_context,
			ur_targets,
			debug_output
		end

	ROSE_SHARED_GLOBAL
		undefine
			is_equal
		end

create
	make,
	make_anonymous,
	make_with_zone_xrf,
	make_surrogate

feature  --Initialization

	make_surrogate (a_company: like Current; a_zone: like zone) is
			-- Make surrogate
		do
			copy(a_company)
			zone_mnemonic := a_zone.mnemonic
			xrf_code := a_company.xrf_code_in (a_zone)
		end

feature --Access
	surrogate (a_zone : like zone) : like context.company_template is
			-- Surrogate
		do
			Result := a_zone.company(foreign_xrf_code(a_zone))
		end

	register is
			-- Register this entity.
		do
			if zone /= void then
				zone.add_company(Current)
			end
		end

	zonal_security (a_zone : like zone) : like context.security_template is
			-- first found security in a_zone
		local
			i : INTEGER
			l_found : BOOLEAN
		do
			from i := 1 until l_found or else i > security_count
			loop
				if security(i).zone /= Void and then security(i).zone.same_zone(a_zone) then
					l_found := true
					Result := security(i)
				end
				i := i+1
			end
		end

	zonal_listing (a_zone : like zone) : like context.listing_template
			--First found listing in `a_zone'
		local
			i : INTEGER
		do
			from i := 1 until Result /= Void or else i > security_count
			loop
				Result := security(i).zonal_listing(a_zone)
				i := i+1
			end
		end

	foreign_xrf_code(a_zone : like context.zone_template) : INTEGER is
		local
			l_records : ROSE_LINEAR_ARRAY[COMPANY_FCXRF_RECORD]
		do
			l_records := company_fcxrf_table.item(handle)
			if l_records /= Void then
				from l_records.start until l_records.off or Result /= 0 loop
					if l_records.current_item.zone.item(1) = a_zone.mnemonic then
						Result := l_records.current_item.fcxrf
					end
					l_records.forth
				end
			end
		end

	records : ROSE_LINEAR_ARRAY[ROSE_ENTITY_DATE[COMPANY]] is
		do
			create Result.make_empty
			Result.extend (create {ROSE_ENTITY_DATE[COMPANY]}.make (Current, today))
		end

	fund_name (a_date: ROSE_DATE): STRING
			-- Name of fund at `a_date', if `Current' is a composite asset
		local
			l_brimco: STRING
		do
			if is_composite_asset (a_date) then
				l_brimco := Current.brim_code (a_date)
				composite_assets_record_predicate.retrieve (Void, <<l_brimco, a_date.ccyymmdd, a_date.ccyymmdd>>)
				Result := composite_assets_record_predicate.array.item(1).fundname
			end
		end

	is_composite_asset(a_date:ROSE_DATE):BOOLEAN is
			-- Is this comapny is a composite asset?
		local
			l_comp_key:STRING
		do
			l_comp_key:=is_composite_asset_key (a_date)
			result:=l_comp_key/=void and then l_comp_key.as_lower.is_equal("true")
		end

	composite_assets_record_predicate: COLLECTING_PREDICATE [COMPOSITE_ASSETS_RECORD] is
			-- SQL predicate for checking if company is a composite asset
		once
			create Result.make (servers.composite_server,
			"select * from composite_assets where brimco= ? and from_date <= ? and to_date > ? order by brimco limit to 1 rows;" )
			if not Result.success then
				print ("Failed to create composite_assets_record_predicate.%N")
			end
		end

    local_xrf_code (a_zone: like context.zone_template): INTEGER is
    		-- local cxrf code of a_zone, if any
    	local
		 l_local_cxrf : STRING
		 i, j : INTEGER
		 l_found : BOOLEAN
      do
      	from i := 1 until l_found or else i > security_count
		loop
			if security (i) /= Void then
				 from j := 1
				 until j > security (i).listing_count or l_found
				 loop
				    if security (i).listing (j).is_dummy_listing_of (a_zone.mnemonic) then
				       l_local_cxrf := security (i).listing (j).local_fcxrf_key (void)
				       if l_local_cxrf /= Void then
						  Result := l_local_cxrf.to_integer
				       end
				       l_found := True
				    end
				    j := j + 1
				 end
			end
			i := i+1
		end
      end

	xrf_code_in (a_zone: like context.zone_template): INTEGER is
			-- Returns the foreign or local xrf_code of this company in 'a_zone'.
    	do
    		if zone.mnemonic = a_zone.mnemonic then
    			Result := xrf_code
    		else
    			Result := foreign_xrf_code (a_zone)
    			if Result = 0 then
    				Result := local_xrf_code (a_zone)
    			end
    		end
    	end

	primary_security (a_date : ROSE_DATE) : like context.security_template is
			-- Added for access to primary_security in descendants when accessed as COMPANY
		require
			a_date_not_void: zone /= Void implies a_date /= Void
		do
		end

	security_for_valuation (a_date : ROSE_DATE) : like context.security_template is
			-- Security used for valuation, especially for bringing dividends forward
			-- It avoids mondat call and uses only ref data.
		local
			l_index: INTEGER
			l_brim: STRING
		do
			if securities /= Void and then not securities.is_empty then
				-- Find the security that is traded in the headquarter_country
				Result := country_security_at (country(a_date), a_date)
				-- Or else find the security in the brim country
				if Result = Void then
					l_brim := brim_code_most_recent_key (a_date)
					if l_brim /= Void then
						Result := country_security_at (context.country_at (context.iso_code (l_brim.substring (1, 2)), Void), a_date)
					end
				end
				-- Or else find the security that trades in company zone
				if Result = Void then
					Result := zonal_security_at (a_date)
				end
				-- Or else fallback to the traditional implementation
				if Result = Void then
					from l_index := 1
					until (Result /= Void and then Result.is_valid_with_non_zero_xrf_code) or else l_index > security_count
					loop
						Result := security (l_index)
						l_index := l_index + 1
					end
				end
			end
		end

	country_security_at (a_country: like context.country_template; a_date: ROSE_DATE): like context.security_template
			-- Find the first valid security as of a_date that trades in a_country
			-- If a_date is void then considers all historical securities
		do
			if a_country /= Void then
				Result := security_at_with_condition (a_date, agent {like context.security_template}.has_valid_country_listing_for_valuation(a_country, a_date))
			end
		end

	zonal_security_at (a_date: ROSE_DATE): like context.security_template
			-- Find the first valid security as of a_date that trades in this company's zone
			-- If a_date is void then considers all historical securities
		do
			Result := security_at_with_condition (a_date, agent {like context.security_template}.has_valid_zonal_listing(zone))
		end

	security_at_with_condition (a_date: ROSE_DATE; a_condition: PREDICATE [ANY, TUPLE[like context.security_template]]): like context.security_template
			-- Find the first valid security as of a_date that meets a_condition
			-- If a_date is void then considers all historical securities.
			-- GIVES PREFERENCE TO COMMON SECURITY
		require
			valid_args: a_condition /= Void
		local
			l_securities, l_securities_1: ROSE_LINEAR_ARRAY[like context.security_template]
			l_security: like context.security_template
		do
			l_securities := securities.all_witnesses (a_date)
			l_securities := l_securities.filtered_by (agent {like context.security_template}.is_valid_with_non_zero_xrf_code)
			l_securities := l_securities.filtered_by (agent {like context.security_template}.is_live_at(a_date))
			l_securities_1 := l_securities.filtered_by (agent {like context.security_template}.is_common_security(a_date))
			if not l_securities_1.is_empty then
				l_securities := l_securities_1
			end

			from l_securities.start until Result /= Void or else l_securities.off loop
				l_security := l_securities.current_item
				if a_condition.item([l_security]) then
					Result := l_security
				end
				l_securities.forth
			end
		ensure
			result_meets_condition: Result /= Void implies a_condition.item ([Result])
			result_is_live: Result /= Void implies Result.is_live_at (a_date)
			result_is_valid_with_non_zero_xrf_code: Result /= Void implies Result.is_valid_with_non_zero_xrf_code
		end

	primary_listing (a_date: ROSE_DATE): like context.listing_template is
			-- Added for access to primary_listing in descendants when accessed as COMPANY
		require
			a_date_not_void: zone /= Void implies a_date /= Void
		do
		end

	primary_listing_with_agent(a_date: ROSE_DATE; a_agent: GENERIC_ACCESS_CONTROL_AGENT): like context.listing_template
	-- Primary listing at `a_date'.  Is redefined in RESEARCH_COMPANY.
	do
		Result := primary_listing(a_date)
	end

feature -- UR Targets

  	at (a_date : ROSE_DATE) : ROSE_ENTITY_DATE[COMPANY] is
  			--target for this company on a_date
  		do
  			create Result.make(Current,a_date)
  		end

feature -- MSCI Industry Classifications
	primary_msci_sub_industry(a_date : ROSE_DATE) : like context.sub_industry_template is
		local
			l_sub_history : ROSE_ENTITY_HISTORY[like context.sub_industry_template]
		do
			l_sub_history := company_to_sub_industry_history
			if l_sub_history /= Void then
				Result := l_sub_history.primary_entity(a_date)
			end
		end

	msci_industry_history : ROSE_ENTITY_HISTORY[PERSISTENT_INDUSTRY] is
		local
			l_sub_history : ROSE_ENTITY_HISTORY[like context.sub_industry_template]
		do
			l_sub_history := company_to_sub_industry_history
			if l_sub_history /= Void then
				Result := sub_industry_to_industry_mapping.map_and_aggregate(l_sub_history)
			end
		end


	primary_msci_industry(a_date : ROSE_DATE) : like context.industry_template is
		local
			l_history : ROSE_ENTITY_HISTORY[PERSISTENT_INDUSTRY]
		do
			l_history := msci_industry_history
			if l_history /= Void then
				Result ?= l_history.primary_entity (a_date)
			end
		end

	msci_industry_group_history : ROSE_ENTITY_HISTORY[like context.industry_group_template] is
		local
			l_history : ROSE_ENTITY_HISTORY[PERSISTENT_INDUSTRY]
		do
			l_history := msci_industry_history
			if l_history /= Void then
				Result := industry_to_industry_group_mapping.map_and_aggregate(l_history)
			end
		end

	primary_msci_industry_group(a_date : ROSE_DATE) : like context.industry_group_template is
		local
			l_industry_group_history : ROSE_ENTITY_HISTORY[PERSISTENT_INDUSTRY_GROUP]
		do
			l_industry_group_history := msci_industry_group_history
			if l_industry_group_history /= Void then
				Result := l_industry_group_history.primary_entity (a_date)
			end
		end

	msci_economic_sector_history : ROSE_ENTITY_HISTORY[PERSISTENT_ECONOMIC_SECTOR] is
		local
			l_industry_group_history : ROSE_ENTITY_HISTORY[PERSISTENT_INDUSTRY_GROUP]
		do
			l_industry_group_history := msci_industry_group_history
			if l_industry_group_history /= Void then
				Result := industry_group_to_economic_sector_mapping.map_and_aggregate(l_industry_group_history)
			end
		end

	primary_msci_economic_sector(a_date : ROSE_DATE) : ECONOMIC_SECTOR is
		local
			l_sector_history : ROSE_ENTITY_HISTORY[PERSISTENT_ECONOMIC_SECTOR]
		do
			l_sector_history := msci_economic_sector_history
			if l_sector_history /= Void then
				Result ?= l_sector_history.primary_entity (a_date)
			end
		end

feature -- Industry classification mappings

	sub_industry_to_industry_mapping : ROSE_ENTITY_HISTORY_MAPPING[PERSISTENT_SUB_INDUSTRY,PERSISTENT_INDUSTRY] is
		local
			l_table : HASH_TABLE[ROSE_ENTITY_HISTORY[PERSISTENT_INDUSTRY],PERSISTENT_SUB_INDUSTRY]
			l_from_entity: PERSISTENT_SUB_INDUSTRY
			l_relationships: ROSE_ENTITY_HISTORY[PERSISTENT_INDUSTRY]
			l_from_entity_handler : CATEGORY_HANDLER[SUPPLIER_CONTEXT,PERSISTENT_SUB_INDUSTRY]
		once
			create l_table.make(10)
			l_from_entity_handler := context.sub_industry_accessor
			from l_from_entity_handler.all_entities.start until l_from_entity_handler.all_entities.off
			loop
				l_from_entity:=l_from_entity_handler.all_entities.current_item
				if l_from_entity.gics_code_key (Today)/=Void then
					l_relationships:=l_from_entity.industry_to_sub_industry_co_history
					l_table.put(l_relationships,l_from_entity.entity)
				end
				l_from_entity_handler.all_entities.forth
			end
			create Result.make(l_table)
		end

	industry_to_industry_group_mapping : ROSE_ENTITY_HISTORY_MAPPING[PERSISTENT_INDUSTRY,PERSISTENT_INDUSTRY_GROUP] is
		local
			l_table : HASH_TABLE[ROSE_ENTITY_HISTORY[PERSISTENT_INDUSTRY_GROUP],PERSISTENT_INDUSTRY]
			l_from_entity: PERSISTENT_INDUSTRY
			l_relationships: ROSE_ENTITY_HISTORY[PERSISTENT_INDUSTRY_GROUP]
			l_from_entity_handler : CATEGORY_HANDLER[SUPPLIER_CONTEXT, PERSISTENT_INDUSTRY]
		once
			create l_table.make(10)
			l_from_entity_handler := context.industry_accessor
			from l_from_entity_handler.all_entities.start until l_from_entity_handler.all_entities.off
			loop
				l_from_entity:=l_from_entity_handler.all_entities.current_item
				if l_from_entity.gics_code_key (today)/=Void then
					l_relationships:=l_from_entity.industry_group_to_industry_co_history
					l_table.put(l_relationships,l_from_entity.entity)
				end
				l_from_entity_handler.all_entities.forth
			end
			create Result.make(l_table)
		end

	industry_group_to_economic_sector_mapping : ROSE_ENTITY_HISTORY_MAPPING[PERSISTENT_INDUSTRY_GROUP,PERSISTENT_ECONOMIC_SECTOR] is
		local
			l_table : HASH_TABLE[ROSE_ENTITY_HISTORY[PERSISTENT_ECONOMIC_SECTOR],PERSISTENT_INDUSTRY_GROUP]
			l_from_entity: PERSISTENT_INDUSTRY_GROUP
		 	l_relationships: ROSE_ENTITY_HISTORY[PERSISTENT_ECONOMIC_SECTOR]
			l_from_entity_handler : CATEGORY_HANDLER[SUPPLIER_CONTEXT,PERSISTENT_INDUSTRY_GROUP]
		once
			create l_table.make(10)
			l_from_entity_handler := context.industry_group_accessor
			from l_from_entity_handler.all_entities.start until l_from_entity_handler.all_entities.off
			loop
				l_from_entity:=l_from_entity_handler.all_entities.current_item
				if l_from_entity.gics_code_key (today)/=Void then
					l_relationships:=l_from_entity.economic_sector_to_industry_group_co_history
					l_table.put(l_relationships,l_from_entity.entity)
				end
				l_from_entity_handler.all_entities.forth
			end
			create Result.make(l_table)
		end

feature -- Capital Actions

	distributions_between (a_start_date, a_end_date: ROSE_DATE): ROSE_LINEAR_ARRAY[CAPACT_LEGACY_DISTRIBUTION] is
			-- Distributions for this company between target dates.
		require
			start_date_not_void: a_start_date /= Void
			end_date_not_void: a_end_date /= Void
			dates_order: a_start_date <= a_end_date
		local
			l_capact_signature: CAPACT_SIGNATURE_RECORD
			l_capacts_distributions: ROSE_LINEAR_ARRAY[like ur_capact_signature]
		do
			l_capacts_distributions := capact_distributions_between (a_start_date, a_end_date)
			l_capacts_distributions.sort_by (agent distributions_comparison)
			from l_capacts_distributions.start
			until l_capacts_distributions.after
			loop
				l_capact_signature := l_capacts_distributions.current_item
				if attached {CAPACT_LEGACY_DISTRIBUTION} l_capact_signature.typed_capital_action_from_signature as l_legacy_distribution then
					l_legacy_distribution.instantiate_all
					if Result = Void then
						create Result.make_empty
					end
					Result.extend (l_legacy_distribution)
				else
					warn ("distributions_between", "Capact signature $1 is not factorized into legacy distribution.", <<l_capact_signature.signature_id>>)
				end
				l_capacts_distributions.forth
			end
		end

	has_terminal_distribution (a_start_date, a_end_date: ROSE_DATE): BOOLEAN is
			-- Was there a terminal distribution between `a_start_date' and `a_end_date'?
		require
			a_start_date_not_void: a_start_date /= Void
			a_end_dte_not_void: a_end_date /= Void
			a_start_end_ordered: a_start_date <= a_end_date
		local
			l_listings: ROSE_LINEAR_ARRAY [LISTING]
		do
			l_listings := listings.all_witnesses (a_end_date)
			Result := l_listings.there_exists (agent {LISTING}.has_terminal_distribution (a_start_date, a_end_date))
		end

feature {NONE}  --Implementation

	company_fcxrf_table : HASH_TABLE[ROSE_LINEAR_ARRAY[COMPANY_FCXRF_RECORD],INTEGER] is
		do
			Result := company_fcxrf_tables.item(context.supplier_id)
			if Result = Void then
				Result := company_fcxrf_table_imp
				company_fcxrf_tables.put(Result, context.supplier_id)
			end
		end

	company_fcxrf_table_imp : HASH_TABLE[ROSE_LINEAR_ARRAY[COMPANY_FCXRF_RECORD],INTEGER] is
		local
			l_zone_code, l_fcxrf_code: INTEGER
		do
			l_zone_code := context.attribute_id_by_name ("ZONE CODE")
			l_fcxrf_code := context.attribute_id_by_name ("FCXRF CODE")
			create Result.make(1500)
			context.company_fcxrf_records(l_fcxrf_code,l_zone_code).do_all(agent add_fcxrf_record(?,Result))
		end

	add_fcxrf_record (a_record : COMPANY_FCXRF_RECORD; a_table : like company_fcxrf_table) is
			-- Add `a_record' to `a_table'.
		local
			l_records : ROSE_LINEAR_ARRAY[COMPANY_FCXRF_RECORD]
		do
			l_records := a_table.item(a_record.handle)
			if l_records = Void then
				create l_records.make_empty
				a_table.put(l_records,a_record.handle)
			end
			l_records.extend(clone(a_record))
		end

	company_fcxrf_tables : HASH_TABLE[HASH_TABLE[ROSE_LINEAR_ARRAY[COMPANY_FCXRF_RECORD],INTEGER], INTEGER] is
			-- Once access to company fcxrf tables.
		once
			create Result.make(10)
		ensure
			result_not_void: Result /= Void
		end


	ur_capact_signature: CAPACT_SIGNATURE_RECORD is
			-- empty capital action signature for access to features
		once
			create Result
		ensure
			not_void: Result /= void
		end

	distributions_comparison (a_first_distribution, a_second_distribution: CAPACT_SIGNATURE_RECORD): BOOLEAN is
			-- Comparison of distributions for ordering.
		require
			first_distribution_not_void: a_first_distribution /= Void
			second_distribution_not_void: a_second_distribution /= Void
		do
			if a_first_distribution.exdate < a_second_distribution.exdate then
				Result := True
			elseif a_first_distribution.exdate = a_second_distribution.exdate then
				if a_first_distribution.instrument_id < a_second_distribution.instrument_id then
					Result := True
				end
			end
		end

	capact_distributions_between (a_start_date, a_end_date: ROSE_DATE): ROSE_LINEAR_ARRAY[like ur_capact_signature] is
			-- Capact signatures for distributions for this company between target dates.
		require
			start_date_not_void: a_start_date /= Void
			end_date_not_void: a_end_date /= Void
			dates_order: a_start_date <= a_end_date
		local
			l_legacy_distribution_id: INTEGER
		do
			l_legacy_distribution_id := {CAPACT_LEGACY_DISTRIBUTION}.capital_action_type_id
			ur_capact_signature.set_zone (zone.mnemonic)
			Result := ur_capact_signature.signatures_by_company_id_type_interval (handle, l_legacy_distribution_id, a_start_date, a_end_date)
		end

end


