note

	description:

		"fundamental expression group for computed variables"

	keywords: "fundamental,expression,computed"
	author: "Mark Howard"
	last_editor: "$Author: it $"
	date: "$Date: 2014/03/14 $"
	revision: "$Revision: #38 $"
	path: "$File: //rose/source/kernel/math/computed_expression.e $"

class
	COMPUTED_EXPRESSION[reference G]

inherit
	ROSE_EXPRESSION[G]
		rename
			make as rose_expression_make
		undefine
			string_value
		redefine
			sub_expressions,
			units,
			partial_real_value,
			partial_double_value,
			partial_integer_value,
			date_value,
			is_defined_real,
			is_defined_double,
			computed_units,
			nominal_units
		end
	NEEDS_EXPRESSION_FROM_STRING
create
	make,make_from_expression
feature  --Properties
	computation : ROSE_EXPRESSION[like expression_target]
			-- Computation

	description : STRING is
			--description
		do
			Result := "computed variable"
		end
feature {NONE} --Creation
	make_from_expression_string (a_variables : ENTITY_VARIABLES[like expression_target];a_expression : STRING) is
		--make from string
	  local
		 l_unknown : UNKNOWN_EXPRESSION[like expression_target]
	  do
			if a_expression /= Void and then a_expression.count > 0 then
				a_expression.prune_all(' ')
				computation := a_variables.complex_expression(a_expression)
			end
			if computation = Void then
			   create  l_unknown.make("<Void>")
			   computation := l_unknown
			end
			computed_expressions_saved := computation.computed_expressions_saved
			type := computation.type
		end
	make_from_expression (a_expression : ROSE_EXPRESSION[like expression_target];a_variable_description : VARIABLE_DESCRIPTION_RECORD) is
			--just return what a_expression would return
		do
			rose_expression_make(a_variable_description)
			computation := a_expression
			computed_expressions_saved := computation.computed_expressions_saved
			type := computation.type
			if a_variable_description.variable_index = 0 then
				set_expression_index
			end
		end
	make (a_variables : ENTITY_VARIABLES[like expression_target];a_description : VARIABLE_DESCRIPTION_RECORD) is
		do
			rose_expression_make(a_description)
			make_from_expression_string(a_variables,variable_description.expression)
		end
feature --Access
	is_defined_real (a_real : REAL) : BOOLEAN is
		do
			Result := computation.is_defined_real(a_real)
		end
	is_defined_double (a_value : DOUBLE) : BOOLEAN is
		do
			Result := computation.is_defined_double(a_value)
		end
	is_defined_integer (a_value : INTEGER) : BOOLEAN is
		do
			Result := computation.is_defined_integer(a_value)
		end
	get_integer_value (a_entity : like expression_target) : INTEGER is
		do
			Result := computation.integer_value(a_entity)
		end
	get_double_value (a_entity : like expression_target) : DOUBLE is
		do
			Result := computation.generated_double (a_entity, Void)
		end
	get_real_value (a_entity : like expression_target) : REAL is
		do
			Result := computation.generated_real (a_entity, Void)
		end

	string_value (a_entity : like expression_target) : STRING is
			-- <Precursor>
		do
			Result := computation.generated_string (a_entity, Void)
		end
	date_value (a_entity : like expression_target) : ROSE_DATE is
			-- Date value
		do
			Result := computation.generated_date (a_entity, Void)
		end

	undefined_real : REAL is
		do
			Result := computation.undefined_real
		end
	undefined_double : DOUBLE is
		do
			Result := computation.undefined_double
		end
	undefined_integer : INTEGER is
		do
			Result := computation.undefined_integer
		end
	sub_expressions : ROSE_LINEAR_ARRAY[ROSE_EXPRESSION[like expression_target]] is
			--sub expressions
		local
			l_expressions : ROSE_LINEAR[ROSE_EXPRESSION[like expression_target]]
		do
			Result := Precursor
			l_expressions := computation.sub_expressions
			from l_expressions.start until l_expressions.off loop
				Result.extend(l_expressions.item)
				l_expressions.forth
			end
		end
	computed_units (a_target: like expression_target; a_agent: like access_agent_template): ROSE_UNITS
			-- <Precursor>
		do
			Result := computation.computed_units (a_target, a_agent)
			if Result = Void then
				Result := Precursor (a_target, a_agent)
			end
		end
	units (a_entity : like expression_target) : ROSE_UNITS is
			-- <Precursor>
		do
			Result := computation.units(a_entity)
		end
	currency (a_entity : like expression_target) : PERSISTENT_CURRENCY is
			-- <Precursor>
		do
			Result := computation.currency(a_entity)
		end
	partial_integer_value (a_entity : like expression_target) : INTEGER is
			-- <Precursor>
		do
			Result := computation.partial_integer_value(a_entity)
		end
	partial_double_value (a_entity : like expression_target) : DOUBLE is
			-- <Precursor>
		do
			Result := computation.partial_double_value(a_entity)
		end
	partial_real_value (a_entity : like expression_target) : REAL is
			-- <Precursor>
		do
			Result := computation.partial_real_value(a_entity)
		end

	nominal_units: ROSE_UNITS is
			-- <Precursor>
		do
			Result := computation.nominal_units
			if Result = Void then
				Result := pure_units
			end
		end

end -- class COMPUTED_EXPRESSION
