note

	description:

		"constant value expression"

	keywords: "fundamental,expression,constant"
	author: "Mark Howard"
	last_editor: "$Author: it $"
	date: "$Date: 2014/03/14 $"
	revision: "$Revision: #40 $"
	path: "$File: //rose/source/kernel/math/constant_expression.e $"
	storable_version: ""

class
	CONSTANT_EXPRESSION[reference G]

inherit
	ROSE_EXPRESSION[G]
		redefine
			units,
			value_in_market_currency,
			is_defined_real,
			is_defined_double,
			date_value
		end
	ROSE_IEEE_UNDEFINED
		undefine
			default_create
		redefine
			is_defined_real,
			is_defined_double,
			is_defined_integer
		end

create
	make_real,
	make_double,
	make_integer,
	make_character,
	make_undefined_double

feature {NONE} -- Initialization

	make_real (a_real : REAL) is
			--make REAL
		do
			set_expression_index
			type := {INTERNAL}.Real_type
			real_constant_value := a_real
			double_constant_value := a_real
			name := a_real.out
			create variable_description
			variable_description.set_name(name)
			variable_description.set_mnemonic(name)
			variable_description.set_type ("real")
		end

	make_double (a_double : DOUBLE) is
			-- Initialize as double constant with value `a_double'
		do
			set_expression_index
			type := {INTERNAL}.Double_type
			double_constant_value := a_double
			name := a_double.out
			create variable_description
			variable_description.set_name(name)
			variable_description.set_mnemonic(name)
			variable_description.set_type ("double")
		end

	make_integer (a_integer : INTEGER) is
			-- Initialize as integer constant with value `a_integer'
		do
			set_expression_index
			type := {INTERNAL}.Integer_type
			integer_constant_value := a_integer
			real_constant_value := a_integer
			double_constant_value := a_integer
			name := a_integer.out
			create variable_description
			variable_description.set_name(name)
			variable_description.set_mnemonic(name)
			variable_description.set_type ("integer")
		end

	make_character (a_character : CHARACTER) is
			-- Initialize as character constant with value `a_character'
		do
			set_expression_index
			type := {INTERNAL}.Character_type
			character_constant_value := a_character
			name := a_character.out
			create variable_description
			variable_description.set_name(name)
			variable_description.set_mnemonic(name)
			variable_description.set_type ("character")
		end

	make_undefined_double(a_double: DOUBLE) is
		do
			set_expression_index
			make_double(a_double)
			is_undefined := True
		end

feature -- Access

	get_real_value (a_entity : G) : REAL is
		do
			Result := real_constant_value
		end

	get_double_value (a_entity : G) : DOUBLE is
		do
			Result := double_constant_value
		end

	get_integer_value (a_entity : G) : INTEGER is
		do
			Result := integer_constant_value
		end


	character_value (a_entity : G) : CHARACTER is
		do
			Result := character_constant_value
		end


	date_value (a_entity : like expression_target) : ROSE_DATE is
			-- <Precursor>
		do
		end


	computation : like Current is
		do
			Result := Current
		end

	description : STRING is
		do
		end

	value_in_market_currency (a_market : MARKET;a_currency : PERSISTENT_CURRENCY;a_entity : like expression_target;a_date : ROSE_DATE) : DOUBLE is
			-- <Precursor>
		do
			Result := double_value(a_entity)
		end

	units (a_entity : like expression_target) : ROSE_UNITS is
		do
			Result := pure_units
		end

	currency (a_entity : G) : PERSISTENT_CURRENCY is
			-- Currency
		do
		end

	old_units_in_currency (a_currency : PERSISTENT_CURRENCY;a_entity : like expression_target;a_date : ROSE_DATE) : ROSE_UNITS is
		do
			Result := pure_units
		end

feature -- Set

	set_real (a_real : REAL) is
		do
			type := {INTERNAL}.Real_type
			real_constant_value := a_real
		end

	set_double (a_double : DOUBLE) is
		do
			type := {INTERNAL}.Double_type
			double_constant_value := a_double
		end

	set_integer (a_integer : INTEGER) is
		do
			type := {INTERNAL}.Integer_type
			integer_constant_value := a_integer
		end

	set_character (a_character : CHARACTER) is
		do
			type := {INTERNAL}.Character_type
			character_constant_value := a_character
		end





feature -- Values

	real_constant_value : REAL
	double_constant_value : DOUBLE
	integer_constant_value : INTEGER
	character_constant_value : CHARACTER


feature -- Status

	is_defined_real (a_real : REAL) : BOOLEAN is
			-- Is defined real?
		do
			Result := not is_undefined
		end

	is_defined_double (a_double : DOUBLE) : BOOLEAN is
		do
			Result := not is_undefined
		end

	is_defined_integer (a_integer : INTEGER) : BOOLEAN is
		do
			Result := not is_undefined
		end

feature {NONE} -- Implementation

	is_undefined: BOOLEAN

end
