indexing
	description: "Context Variables"
	author: "it"
	last_editor: "$Author: rj $"
	date: "$Date: 2013/11/25 $"
	revision: "$Revision: #9 $"
	path : "$File: //rose/source/kernel/entities/context_variables.e $"

class
	CONTEXT_VARIABLES

inherit
	ROSE_EXECUTION_ENVIRONMENT

	CONTEXT_CLIENT
		rename
			set_context as make
		end

create
	make

feature -- Access

	all_variable_descriptions : ROSE_LINEAR_ARRAY[VARIABLE_DESCRIPTION_RECORD] is
		do
			if all_variable_descriptions_saved = Void then
				all_variable_descriptions_saved := retrieved_variable_descriptions
			end
			Result := all_variable_descriptions_saved
		ensure
			Result /= void
		end

	variable_descriptions (a_target : STRING) : ROSE_LINEAR_ARRAY[VARIABLE_DESCRIPTION_RECORD] is
		local
			l_target : STRING
			l_more: ROSE_LINEAR_ARRAY[VARIABLE_DESCRIPTION_RECORD]
		do
			all_variable_descriptions.do_nothing
			create Result.make_empty
			l_target := a_target.twin
			fix_string (l_target)
			l_more := var_desc_by_target (l_target)
			Result.append_array (l_more)
			l_more := var_desc_by_target_name (l_target)
			Result.append_array (l_more)
		ensure
			Result /= void
		end

	variable_description_by_mnemonic (a_mnemonic : STRING) : VARIABLE_DESCRIPTION_RECORD is
		local
			l_mnemonic : STRING
			l_arr: ROSE_LINEAR_ARRAY[VARIABLE_DESCRIPTION_RECORD]
		do
			all_variable_descriptions.do_nothing
			l_mnemonic := a_mnemonic.twin
			fix_string (l_mnemonic)
			l_arr := var_desc_by_mnemonic (l_mnemonic)
			if l_arr /= void and then not l_arr.is_empty then
				Result := l_arr.first
			end
		end

	variable_description_by_name (a_name : STRING) : VARIABLE_DESCRIPTION_RECORD is
		local
			l_s : STRING
			l_arr: ROSE_LINEAR_ARRAY[VARIABLE_DESCRIPTION_RECORD]
		do
			all_variable_descriptions.do_nothing
			l_s := a_name.twin
			fix_string (l_s)
			l_arr := var_desc_by_name (l_s)
			if l_arr /= void and then not l_arr.is_empty then
				Result := l_arr.first
			end
		end

	variable_descriptions_by_source (a_source : STRING) : ROSE_LINEAR_ARRAY[VARIABLE_DESCRIPTION_RECORD] is
		local
			l_s : STRING
		do
			all_variable_descriptions.do_nothing
			l_s := a_source.twin
			fix_string (l_s)
			Result := var_desc_by_source (l_s)
		end

	fix_strings(a_var_desc: VARIABLE_DESCRIPTION_RECORD)
	-- Change certain fields to lower case and remove all trailing blanks.
	do
		fix_string(a_var_desc.target)
		fix_string(a_var_desc.target_name)
		fix_string(a_var_desc.name)
		fix_string(a_var_desc.mnemonic)
		fix_string(a_var_desc.source)
	end

feature {NONE} -- Implementation

	all_variable_descriptions_saved : ROSE_LINEAR_ARRAY[VARIABLE_DESCRIPTION_RECORD]

	var_desc_by_target_cache: HASH_TABLE[ROSE_LINEAR_ARRAY[VARIABLE_DESCRIPTION_RECORD], STRING]
	var_desc_by_target (a_target: STRING): ROSE_LINEAR_ARRAY[VARIABLE_DESCRIPTION_RECORD] is
			--
		do
			if var_desc_by_target_cache = void then
				create var_desc_by_target_cache.make (17)
			end
			var_desc_by_target_cache.search (a_target)
			if var_desc_by_target_cache.found then
				Result := var_desc_by_target_cache.found_item
			else
				create Result.make_empty
				var_desc_by_target_cache.force (Result, a_target)
			end
		end

	var_desc_by_target_name_cache: HASH_TABLE[ROSE_LINEAR_ARRAY[VARIABLE_DESCRIPTION_RECORD], STRING]
	var_desc_by_target_name (a_target: STRING): ROSE_LINEAR_ARRAY[VARIABLE_DESCRIPTION_RECORD] is
			--
		do
			if var_desc_by_target_name_cache = void then
				create var_desc_by_target_name_cache.make (17)
			end
			var_desc_by_target_name_cache.search (a_target)
			if var_desc_by_target_name_cache.found then
				Result := var_desc_by_target_name_cache.found_item
			else
				create Result.make_empty
				var_desc_by_target_name_cache.force (Result, a_target)
			end
		end

	var_desc_by_name_cache: HASH_TABLE[ROSE_LINEAR_ARRAY[VARIABLE_DESCRIPTION_RECORD], STRING]
	var_desc_by_name (a_target: STRING): ROSE_LINEAR_ARRAY[VARIABLE_DESCRIPTION_RECORD] is
			--
		do
			if var_desc_by_name_cache = void then
				create var_desc_by_name_cache.make (17)
			end
			var_desc_by_name_cache.search (a_target)
			if var_desc_by_name_cache.found then
				Result := var_desc_by_name_cache.found_item
			else
				create Result.make_empty
				var_desc_by_name_cache.force (Result, a_target)
			end
		end

	var_desc_by_mnemonic_cache: HASH_TABLE[ROSE_LINEAR_ARRAY[VARIABLE_DESCRIPTION_RECORD], STRING]
	var_desc_by_mnemonic (a_target: STRING): ROSE_LINEAR_ARRAY[VARIABLE_DESCRIPTION_RECORD] is
			--
		do
			if var_desc_by_mnemonic_cache = void then
				create var_desc_by_mnemonic_cache.make (17)
			end
			var_desc_by_mnemonic_cache.search (a_target)
			if var_desc_by_mnemonic_cache.found then
				Result := var_desc_by_mnemonic_cache.found_item
			else
				create Result.make_empty
				var_desc_by_mnemonic_cache.force (Result, a_target)
			end
		end

	var_desc_by_source_cache: HASH_TABLE[ROSE_LINEAR_ARRAY[VARIABLE_DESCRIPTION_RECORD], STRING]
	var_desc_by_source (a_target: STRING): ROSE_LINEAR_ARRAY[VARIABLE_DESCRIPTION_RECORD] is
			--
		do
			if var_desc_by_source_cache = void then
				create var_desc_by_source_cache.make (17)
			end
			var_desc_by_source_cache.search (a_target)
			if var_desc_by_source_cache.found then
				Result := var_desc_by_source_cache.found_item
			else
				create Result.make_empty
				var_desc_by_source_cache.force (Result, a_target)
			end
		end

	retrieved_variable_descriptions : like all_variable_descriptions is
		local
			l_var_desc: VARIABLE_DESCRIPTION_RECORD
			l_var_entities: ROSE_LINEAR_ARRAY[PERSISTENT_VARIABLE]
			l_flag: STRING
		do
			l_flag := get("USE_OLD_VARIABLE_DESCRIPTION_TABLE")
			if use_var_desc_table or (l_flag/=Void and then l_flag.count/=0) or True
			then
				Result := context.variable_description_records
				-- The old way reads from the variable_description table.
			else
				-- The new way reads entities.
				context.variable_accessor.cache_all_attributes
				l_var_entities ?= context.variable_accessor.all_entities

				from
					create Result.make_empty
					l_var_entities.start
				until
					l_var_entities.off
				loop
					l_var_desc := l_var_entities.current_item.variable_description_record
					if l_var_desc.security<=context.variable_security
					then
						Result.extend(l_var_desc)
					end

					l_var_entities.forth
				end
			end

			from
				Result.start
			until
				Result.off
			loop
				l_var_desc := Result.current_item
				if l_var_desc.target /= Void then
					fix_string (l_var_desc.target)
					var_desc_by_target (l_var_desc.target).extend (l_var_desc)
				end
				if l_var_desc.target_name /= Void then
					fix_string (l_var_desc.target_name)
					var_desc_by_target_name (l_var_desc.target_name).extend (l_var_desc)
				end
				if l_var_desc.name /= Void then
					fix_string (l_var_desc.name)
					var_desc_by_name (l_var_desc.name).extend (l_var_desc)
				end
				if l_var_desc.mnemonic /= Void then
					fix_string (l_var_desc.mnemonic)
					var_desc_by_mnemonic (l_var_desc.mnemonic).extend (l_var_desc)
				end
				if l_var_desc.source /= Void then
					fix_string (l_var_desc.source)
					var_desc_by_source (l_var_desc.source).extend (l_var_desc)
				end
				Result.forth
			end
		end

	fix_string (a_string: STRING) is
			-- Move to lower case and remove all trailing blanks
		do
			if a_string /= Void
			then
				a_string.to_lower
				a_string.prune_all_trailing (' ')
			end
		end

feature {NONE} -- Implementation

	use_var_desc_table: BOOLEAN

end
