note

	description:

		"entity for countries"

	keywords: "entities,country"
	author: "Mark Howard,Robert Bell"
	last_editor: "$Author: mb $"
	date: "$Date: 2012/09/26 $"
	revision: "$Revision: #66 $"
	path: "$File: //rose/source/kernel/investment_entities/country.e $"

class COUNTRY
inherit
	INVESTMENT_ENTITY
		undefine
			peer_anchor
		redefine
			make,
			debug_output
		end
	PERSISTENT_COUNTRY
		undefine
			context,
			make,
			make_context
		redefine
			debug_output
		end

	ROSE_BUSINESS_DATE
		undefine
			is_equal
		end
	ROSE_SHARED_GLOBAL
		undefine
			is_equal
		end
create
	make,make_with_zone_xrf,make_anonymous

feature {NONE} -- Access
	
	make (a_context : like context;a_category_id : INTEGER;a_handle : INTEGER;a_start_date,a_end_date : ROSE_DATE) is
			-- <Precursor>
		local
			l_id : STRING
			l_investable : STRING
			l_region : INVESTMENT_REGION
		do
			Precursor {INVESTMENT_ENTITY}(a_context,a_category_id,a_handle,a_start_date,a_end_date)
			l_id := old_positional_id_key (today)
			if l_id /= void and then l_id.is_integer then
				set_old_positional_id(l_id.to_integer)
			end
			name := name_key (today)
			if name /= void then
				name.prune_all_trailing(' ')
			end
			code:= clone(iso_code_key (today))
			if code /= void then
				code.prune_all_trailing(' ')
			end
			l_investable := investable_key (today)
			l_region := something_related_by_country_to_investment_region
			if name = Void or code = Void then
				make_anonymous(context,a_category_id)
			else
				if l_investable /= Void then
					l_investable.to_lower
					investable :=  l_investable.is_equal("true")
				end
				if l_region /= Void then
					zone_mnemonic := l_region.mnemonic
				end
				currencies := country_to_currency_history.all_witnesses (today)
				currency_history := country_to_currency_history
				set_is_member_of_european_union
			end
		end

feature -- Basic operations
	register
			-- Register this entity.
		do
		end

	code: STRING
			-- Code

	investable: BOOLEAN
			-- Is `Curent' classified as investable?

	set_investable (a_investable : BOOLEAN)
			-- Set investable status to `a_investable'.
		do
			investable := a_investable
		end

	currencies:  ROSE_LINEAR_ARRAY [like context.currency_template]
			-- Currencies associated with `Current'

	currency_history: like country_to_currency_history
			-- Currency history

	exchanges: EXTENDABLE_BOUNDED_ARRAY [like context.exchange_template]
			-- Exchanges

	companies: EXTENDABLE_BOUNDED_ARRAY [like context.company_template]
			-- Companies

	name: STRING
			-- Name

	set_name (a_value: like name)
			-- Set name to `a_value'.
		do
			name := a_value
		end

	index: INTEGER
			-- Old positional id
		do
			Result := old_positional_id
		end

	old_positional_id: INTEGER
			-- Old "positional_id"

	set_old_positional_id (a_value: like old_positional_id)
			-- Set old_positional_id to `a_value'.
		do
			old_positional_id := a_value
		end

	set_iso_code (a_value : STRING)
			-- Assign `a_value' to `code'.
		do
			code := a_value
		ensure
			iso_code_set: code = a_value
		end

	official_currency (a_date : ROSE_DATE): like context.currency_template is
			-- Offical currency on `a_date' (uses today when `a_date' is Void)
		local
			l_date: ROSE_DATE
			l_currencies: ROSE_LINEAR_ARRAY [like context.currency_template]
		do
			if currency_history /= Void then
				if a_date /= Void then
					l_date := a_date
				else
					l_date := today
				end
				l_currencies := currency_history.all_witnesses (l_date)
				if l_currencies.count = 1 then
					Result := l_currencies.first
				else
					from
						l_currencies.start
					until
						l_currencies.off or else Result /= Void
					loop
						if l_currencies.current_item.xrf_code /= 0 then
							Result := l_currencies.current_item
						end
						l_currencies.forth
					end
				end
			end
		end

	vms_currency_iso: STRING
			-- VMS currency ISO code
			-- To aid migration testing.
		local
			l_name: STRING
		do
			Result := country_currency_hash.item (name)
			if Result = Void then
				l_name := name.substring (1, 1) + name.as_lower.substring (2, name.count)
				Result := country_currency_hash.item (l_name)
			end
		end


	country_currency_hash: HASH_TABLE[STRING,STRING] is
			-- To aid migration testing.  Used with vms_currency
		local
			i, j, l_first_two_spaces, l_three_caps, l_name_end: INTEGER
			l_line, l_substring, l_iso, l_country_name: STRING
			l_found_iso: BOOLEAN
			l_file: ROSE_TEXT_FILE
		once
			create l_file.make_open_read (global.hardcoded_path ( "\\pinegv\PROD_B\REF_W\A\COUNTRY_CODES.PARM"))
			create Result.make(5)
			from
				i := 0
			until
				l_file.end_of_file
			loop
				i := i + 1
				l_found_iso := False
				l_file.read_line
				l_line := l_file.last_string
				if i >= 4 and not l_file.end_of_file and not l_line.substring(1,3).is_equal("--") then
					l_first_two_spaces := l_line.substring_index("  ", 3)
					from j := 3
					until j > 	l_line.count-2
					loop
						l_substring := l_line.substring (j,j+2)
						if l_substring.as_upper.is_equal(l_substring) and not l_substring.has(' ') then
							l_iso := l_substring
							l_three_caps := j
							j := l_line.count
							l_found_iso := True
						end
						j := j+1
					end
					if  l_found_iso then
						--l_name_end := min(l_first_two_spaces-1 ,l_three_caps-1)
						l_name_end := l_first_two_spaces.min(l_three_caps) -1
						l_country_name := l_line.substring (1,l_name_end)
						Result.put(l_iso,l_country_name)
					end
				end
			end
			l_file.close
		end


	is_member_of_european_union: BOOLEAN
			-- Is `Current' a member of the EU?

	set_is_member_of_european_union
			-- Set flag indicating membership in European Union
		local
			l_currency: like context.currency_template
		do
			if currencies /= Void then
				from currencies.start until currencies.off or is_member_of_european_union loop
					l_currency := currencies.current_item
					if l_currency /= Void and then l_currency.iso /= Void then
						is_member_of_european_union := l_currency.iso.is_equal(Euro_ISO)
					end
					currencies.forth
				end
			end
		end

feature -- Index access

	equity_return_market_index (a_date: ROSE_DATE): like context.market_index_template
			-- Equity return market index
		do
			Result := market_index_from_history(country_to_equity_return_market_index_history, a_date)
		end

	risk_free_index (a_date: ROSE_DATE): like context.market_index_template
			-- Risk-free (short-term bond) index
		do
			Result := market_index_from_history(country_to_risk_free_index_history, a_date)
		end

	long_bond_index (a_date: ROSE_DATE): like context.market_index_template
			-- Long-term bond index
		do
			Result := market_index_from_history(country_to_long_bond_index_history, a_date)
		end

feature -- Monthly information

	monthly_equity_return_history: DOUBLE_TIME_SERIES
			-- Monthly time-series of equity market returns
		local
			l_rets: like monthly_returns
		do
			l_rets := monthly_returns
			if l_rets /= Void then
				Result := l_rets.item (equity_market_ix)
			end
		end

	monthly_equity_index_history: DOUBLE_TIME_SERIES
			-- Monthly time-series of equity market index
		local
			l_ixs: like monthly_indices
		do
			l_ixs := monthly_indices
			if l_ixs /= Void then
				Result := l_ixs.item (equity_market_ix)
			end
		end

	monthly_risk_free_rate_history: DOUBLE_TIME_SERIES
			-- Monthly time-series of risk-free (i.e. short-term bond) rate (returns)
		local
			l_rets: like monthly_returns
		do
			l_rets := monthly_returns
			if l_rets /= Void then
				Result := l_rets.item (short_term_bond_ix)
			end
		end

	monthly_risk_free_index_history: DOUBLE_TIME_SERIES
			-- Monthly time-series of risk-free (i.e. short-term bond) index
		local
			l_ixs: like monthly_indices
		do
			l_ixs := monthly_indices
			if l_ixs /= Void then
				Result := l_ixs.item (short_term_bond_ix)
			end
		end

	monthly_long_bond_return_history: DOUBLE_TIME_SERIES
			-- Monthly time-series of long bond returns
		local
			l_rets: like monthly_returns
		do
			l_rets := monthly_returns
			if l_rets /= Void then
				Result := l_rets.item (long_term_bond_ix)
			end
		end

	monthly_long_bond_index_history: DOUBLE_TIME_SERIES
			-- Monthly time-series of long-term bond index
		local
			l_ixs: like monthly_indices
		do
			l_ixs := monthly_indices
			if l_ixs /= Void then
				Result := l_ixs.item (long_term_bond_ix)
			end
		end

	monthly_returns: ROSE_LINEAR_ARRAY [DOUBLE_TIME_SERIES]
			-- Time-series on index returns for equity market, short-term and long-term bond markets
		do
			monthly_returns_cache.search (Current)
			if monthly_returns_cache.found then
				Result := monthly_returns_cache.found_item
			else
				Result := ur_country_return_signature.monthly_returns (Current)
				monthly_returns_cache.force (Result, Current)
			end
		end

	monthly_indices: ROSE_LINEAR_ARRAY [DOUBLE_TIME_SERIES]
			-- Time-series on indices for equity market, short-term and long-term bond markets
		do
			monthly_indices_cache.search (Current)
			if monthly_indices_cache.found then
				Result := monthly_indices_cache.found_item
			else
				Result := ur_country_return_signature.monthly_indices (Current)
				monthly_indices_cache.force (Result, Current)
			end
		end

	equity_market_ix: INTEGER = 1
			-- Position of equity market index series in monthly time-series array

	short_term_bond_ix: INTEGER = 2
			-- Position of short-term bond index series in monthly time-series array

	long_term_bond_ix: INTEGER = 3
			-- Position of long-term bond index series in monthly time-series array

	monthly_returns_cache: HASH_TABLE [ROSE_LINEAR_ARRAY [DOUBLE_TIME_SERIES], COUNTRY]
			-- Cache for retrieved return series for equity and s/t & l/t bond returns
		once
			create Result.make (100)
		ensure
			return_cache_not_void: Result /= Void
		end

	monthly_indices_cache: HASH_TABLE [ROSE_LINEAR_ARRAY [DOUBLE_TIME_SERIES], COUNTRY]
			-- Cache for retrieved index series for equity and s/t & l/t bond returns
		once
			create Result.make (100)
		ensure
			indices_cache_not_void: Result /= Void
		end

	ur_country_return_signature: COUNTRY_MARKET_RETURN_RECORD
			-- Record for storage of market returns (bonds and equity market)
		once
			create Result
		ensure
			record_access_not_void: Result /= Void
		end

feature -- Exchange information

	exchange_count: INTEGER
			-- Number of exchanges in this country.
		local
			l_exchanges : like all_exchanges
		do
			l_exchanges := all_exchanges
			if l_exchanges /= Void then
				Result := l_exchanges.count
			end
		end


	exchange (i: INTEGER): like context.exchange_template
			-- Ith exchange from `all_exchanges' list
		local
			l_exchanges: like all_exchanges
		do
			l_exchanges := all_exchanges
			if l_exchanges /= Void and then l_exchanges.valid_index(i) then
				Result := l_exchanges.item(i)
			end
		end

	all_exchanges: like all_exchanges_saved
			-- All exchanges for `Current'
		do
			if all_exchanges_saved = Void then
				all_exchanges_saved ?= context.relationship(ids.country_exchange_relationship_type_id).related_entities(Current)
			end
			Result := all_exchanges_saved
		end

	all_exchanges_saved: ROSE_LINEAR_ARRAY [like context.exchange_template]
			-- Cache for `all_exchanges'

feature -- Basic operations

	market_calendar_mnemonic: STRING
			-- Market date mnemonic
		local
			l_country_iso: STRING
		do
			l_country_iso := iso.dated_key(Current, today)
			if l_country_iso/=Void
			then
				Result := l_country_iso.substring(1,2) + "M"
			else
				Result := "xxM"
			end
		end

	market_date_bitmap : BUSINESS_DATE_BITMAP
			-- Market date bitmap
		do
			create Result.retrieve(market_calendar_mnemonic)
		end

	bank_calendar_mnemonic: STRING
			-- Bank calendar mnemonic
		local
			l_country_iso: STRING
		do
			l_country_iso := iso.dated_key(Current, today)
			if l_country_iso/=Void
			then
				Result := l_country_iso.substring(1,2) + "B"
			else
				Result := "xxB"
			end
		end

	bank_date_bitmap: BUSINESS_DATE_BITMAP
			-- Bank date bitmap
		do
			create Result.retrieve (bank_calendar_mnemonic)
		end

	market_dates: ROSE_DATE_SET
			-- Market dates
		local
			l_key : STRING
		do
			l_key := table_key
			Result := market_dates_table.item(l_key)
			if Result = Void then
				create Result.make(market_date_bitmap,bitmap_start_date)
				market_dates_table.put(Result, l_key)
			end
		end

	bank_dates: ROSE_DATE_SET
			-- Bank dates
		local
			l_key : STRING
		do
			l_key := table_key
			Result := bank_dates_table.item(l_key)
			if  Result = Void then
				create Result.make(bank_date_bitmap,bitmap_start_date)
				bank_dates_table.put(Result, l_key)
			end
		end

	market_dates_table: HASH_TABLE [ROSE_DATE_SET, STRING]
			-- Market dates cache
		once
			create Result.make(10)
		end

	bank_dates_table: HASH_TABLE[ROSE_DATE_SET, STRING]
			-- Market dates cache
		once
			create Result.make(10)
		end

	table_key : STRING
			-- Key for table
		do
			Result := iso.dated_key (Current,today).substring (1, 2)
		end

	debug_output: STRING
			-- Debug output label
		do
			Result := Precursor {INVESTMENT_ENTITY}
			if Result = Void then
				Result := ""
			end
			if code /= Void then
				Result.append (":" + code)
			end
		end

feature {NONE} -- Implementation

	market_index_from_history (a_history: like country_to_market_index_history; a_date: ROSE_DATE): like context.market_index_template
			-- Market index from `a_history' on `a_date'
		do
			if a_history /= Void and then a_history.count > 0 then
				Result := a_history.witness (a_date)
			end
		end

	bitmap_start_date: ROSE_DATE
			-- Bitmap start date
		once
			create Result.make_ccyymmdd (19500101)
		end

	Euro_ISO: STRING = "EUR"
			-- ISO code for the Euro

end




