indexing

	description:

		"Signature tagged record for country market return"

	author: "Michael Zhang"
	last_editor: "$Author: mb $"
	date: "$Date: 2013/01/03 $"
	revision: "$Revision: #14 $"
	path: "$File: //rose/source/kernel/data/market/country_market_return_record.e $"

class COUNTRY_MARKET_RETURN_RECORD

inherit

	ROSE_SIGNATURE_RECORD [COUNTRY]
		rename
			date as record_date
		undefine
			context
		redefine
			set_field_with_string,
			output_string_by_name,
			data_fields_string_,
			data_manager,
			set_date,
			record_date
		end

	ROSE_LEGACY_UNDEFINED
		undefine
			is_equal
		end

	ATTRIBUTES
		undefine
			is_equal
		end

	ROSE_AWARE

create

	make,
	default_create

create {COUNTRY_MARKET_RETURN_RECORD}

	rose_entity_date_make

feature {NONE} -- Creation

	make (a_country_id: like country_id; a_date: like date; a_periodicity_id: like periodicity_id; a_market_date: like market_date; a_price_e, a_price_s, a_price_l, a_return_e, a_return_s, a_return_l: DOUBLE) is
			-- Create the record
		do
			country_id := a_country_id
			date := a_date
			periodicity_id := a_periodicity_id
			market_date := a_market_date
			price_e := a_price_e
			price_s := a_price_s
			price_l := a_price_l
			return_e := a_return_e
			return_s := a_return_s
			return_l := a_return_l
		end

feature -- Access

	country_id: INTEGER
			-- Country id

	date: ROSE_DATE
			-- (Record) date

	periodicity_id: INTEGER
			-- Periodicity code

	market_date: ROSE_DATE
			-- Market date

	price_e: DOUBLE
			-- Price level for the primary equity index

	price_s: DOUBLE
			-- Price level for the short term risk free benchmark

	price_l: DOUBLE
			-- Price level for the long term bond benchmark

	return_e: DOUBLE
			-- Return for the primary equity index

	return_s: DOUBLE
			-- Return for the short term risk free benchmark

	return_l: DOUBLE
			-- Return for the long term bond benchmark

	record_date: ROSE_DATE is
			-- Record date
		do
			Result := date
		end

	context_name: STRING is
			-- Name from context
		do
			Result := context.name
		end

	monthly_returns (a_country: COUNTRY): ROSE_LINEAR_ARRAY [DOUBLE_TIME_SERIES] is
			-- Equity, short term and long term bond returns for `a_country'
		require
			valid_country: a_country /= Void
		local
			l_record: like Current
			l_records: like targets
			l_ret_e, l_ret_s, l_ret_l: DOUBLE_TIME_SERIES
		do
			l_records := all_monthly_records (a_country)
			if l_records /= Void and then not l_records.is_empty then
				create l_ret_e.make_empty
				create l_ret_s.make_empty
				create l_ret_l.make_empty
				from
					l_records.start
				until
					l_records.off
				loop
					l_record := l_records.current_item
					l_ret_e.extend (safe_return (l_record.return_e), l_record.date)
					l_ret_s.extend (safe_return (l_record.return_s), l_record.date)
					l_ret_l.extend (safe_return (l_record.return_l), l_record.date)
					l_records.forth
				end
				create Result.make_from_array (<<l_ret_e, l_ret_s, l_ret_l>>)
			end
		end

	monthly_indices (a_country: COUNTRY): ROSE_LINEAR_ARRAY [DOUBLE_TIME_SERIES] is
			-- Equity, short term and long term bond indices for `a_country'
		require
			valid_country: a_country /= Void
		local
			l_record: like Current
			l_records: like targets
			l_index_e, l_index_s, l_index_l: DOUBLE_TIME_SERIES
		do
			l_records := all_monthly_records (a_country)
			if l_records /= Void and then not l_records.is_empty then
				create l_index_e.make_empty
				create l_index_s.make_empty
				create l_index_l.make_empty
				from
					l_records.start
				until
					l_records.off
				loop
					l_record := l_records.current_item
					l_index_e.extend (l_record.price_e, l_record.date)
					l_index_s.extend (l_record.price_s, l_record.date)
					l_index_l.extend (l_record.price_l, l_record.date)
					l_records.forth
				end
				create Result.make_from_array (<<l_index_e, l_index_s, l_index_l>>)
			end
		ensure
			three_indices_in_set: Result /= Void implies Result.count = 3
		end

	data_manager: SIGNATURE_TAGGED_ITEM_DATABASE [COUNTRY_MARKET_RETURN_RECORD] is
			-- Shared data manager
		once
			if server /= Void then
				create Result.make_named_with_process_manager ("", server, process_id_manager)
			end
		end

feature -- Element modification

	set_country_id (a_value: like country_id) is
			-- Set `country_id' to `a_value'.
		do
			country_id := a_value
		end

	set_date (a_value: like date) is
			-- Set `date' to `a_value'.
		do
			date := a_value
		end

	set_periodicity_id (a_value: like periodicity_id) is
			-- Set `periodocity_id' to `a_value'
		do
			periodicity_id := a_value
		end

	set_market_date (a_date: like market_date) is
			-- Set `market_date' to `a_date'.
		do
			market_date := a_date
		end

	set_price_e (a_value: like price_e) is
			-- Set `price_e' to `a_value'.
		do
			price_e := a_value
		end

	set_price_s (a_value: like price_s) is
			-- Set `price_s' to `a_value'.
		do
			price_s := a_value
		end

	set_price_l (a_value: like price_l) is
			-- Set `price_l' to `a_value'.
		do
			price_l := a_value
		end

	set_return_e (a_value: like return_e) is
			-- Set `return_e' to `a_value'.
		do
			return_e := a_value
		end

	set_return_s (a_value: like return_s) is
			-- Set `return_s' to `a_value'.
		do
			return_s := a_value
		end

	set_return_l (a_value: like return_l) is
			-- Set `return_l' to `a_value'
		do
			return_l := a_value
		end

	set_entity_id (a_id: INTEGER) is
			-- No implementation (d3 Mar 2009)
		do
		end

feature -- Basic operations

	commit is
			-- Commit any changes to the database and terminate the process id
		do
			data_manager.submit
			process_id_manager.end_process
			server.commit
		end
	
	set_field_with_string (a_value: STRING; a_field: STRING) is
			-- <Precursor>
		do
			if a_field ~ "country_id" then
				if attached {COUNTRY} context.country_at (iso_code (a_value), Void) as l_country and then l_country.is_valid then
					set_country_id (l_country.handle)
				end
			else
				Precursor (a_value, a_field)
			end
		end

	output_string_by_name (a_name: STRING): STRING is
			-- <Precursor>
		do
			if a_name ~ "country_id" then
				Result := value_by_name (a_name).out
				if Result.is_integer then
					if attached {COUNTRY} context.entity_at (context.ids.country_category_id, country_id) as l_country and then l_country.is_valid then
						Result := l_country.code
					end
				end
			else
				Result := Precursor (a_name)
			end
		end
	
feature -- Context

	expression_tag: STRING is "country_market_returns"

	fields_: STRING is "country_id,date,periodicity_id"

	data_fields_string_: STRING is "market_date,price_e,price_s,price_l,return_e,return_s,return_l"

	restriction_statement: STRING is "#country_id.id = country_id"

feature {NONE} -- Implementation

	safe_return (a_return: DOUBLE): DOUBLE is
			-- Return multiplier version of `a_return'
			-- Missing double set if not double
		do
			if is_defined_double (a_return) then
				Result := a_return + 1.
			else
				Result := constants.missing_double
			end
		end

	server: SQL_DATA_SERVER is
			-- The database where the returns are persisted
		do
			Result := servers.country_market_returns_server
		end

feature {RCMD_UPDATE_COUNTRY_MARKET_RETURNS} -- Implementation

	process_id_manager: SUPPLIER_PROCESS_ID_MANAGER is
			-- A default supplier process manager to handle updates
		once
			create Result.make (server)
			Result.set_name ("")
		end

	monthly_records_since (a_country: COUNTRY; a_date: ROSE_DATE): like targets is
			-- Available records for `a_country' since `a_date'
		require
			valid_country: a_country /= Void
			valid_date: a_date /= Void
		local
			l_target: like Current
			l_targets: like targets
		do
			create l_target
			l_target.set_country_id (a_country.handle)
			l_target.set_date (a_date)
			l_target.set_periodicity_id (context.ids.monthly_periodicity_id)
			create l_targets.make_from_array (<<l_target>>)
			Result := targets (l_targets, restriction_statement + " and #date.id <= date")
			Result.sort_by (agent order_by_date)
		end

	all_monthly_records (a_country: COUNTRY): like targets is
			-- Available records for a_country
		require
			valid_country: a_country /= Void
		local
			l_target: like Current
			l_targets: like targets
		do
			create l_target
			l_target.set_country_id (a_country.handle)
			l_target.set_periodicity_id (context.ids.monthly_periodicity_id)
			create l_targets.make_from_array (<<l_target>>)
			Result := targets (l_targets, restriction_statement)
			Result.sort_by (agent order_by_date)
		end

end
