indexing

	description:
	"[
		Credit Default Swap Payment.
		Extremely simplistic implementation for prototypical exploration.
	]"
	author: "Yves Chauvin"
	last_editor: "$Author: it $"
	date: "$Date: 2010/07/19 $"
	revision: "$Revision: #3 $"
	archive: "$Archive: $"

class CREDIT_DEFAULT_SWAP_PAYMENT

inherit

	RP_CONTRACT

	ROSE_MATH_FUNCTIONS
		export {NONE} all end

creation

	make

feature -- Access

	step: INTEGER
			-- should be date
			-- step in order

	probability: DOUBLE
			-- default probability

	present_value_factor: DOUBLE
			-- present value factor

	value: DOUBLE
			-- value of premium

	cds: CREDIT_DEFAULT_SWAP
			-- reference cds

feature -- Settings

	set_cds (a_cds: CREDIT_DEFAULT_SWAP) is
			-- setting reference CDS
		require
			cds_non_void: a_cds /= Void
		do
			cds := a_cds
		ensure
			cds_set: cds ~ a_cds
		end

	set_probability (a_value: DOUBLE) is
			-- setting probability
		require
			a_probability: 0 <= a_value and a_value <= 1
		do
			probability := a_value
		ensure
			probability_set: probability ~ a_value
		end

	set_step (a_step: INTEGER) is
			-- sets step (should be interval based)
		require
			positive_step: a_step >= 0
		do
			step := a_step
		ensure
			step_set: step ~ a_step
		end

	set_value (a_value: DOUBLE) is
			-- setting value
		do
			value := a_value
		ensure
			value_set: value ~ a_value
		end

feature -- Basic operations

	compute is
			-- computes parameters
		do
			compute_present_value_factor
		end

	expected_value: DOUBLE is
			-- expected value of premium
		do
			Result := probability * value
		end

	present_value: DOUBLE is
			-- present value of payment
		do
			Result := present_value_factor * value
		end

	present_expected_value: DOUBLE is
			-- present expected value of premium
		do
			Result := present_value_factor * expected_value
		end

	process (a_processor: RP_CONTRACT_PROCESSOR) is
			-- <Precursor>
		do
			a_processor.process_credit_default_swap_payment (Current)
		end

feature {NONE} -- Implementation

	compute_present_value_factor is
			-- computes present value factor
		require
			a_cds: cds /= Void
		do
			present_value_factor := 1 / (1 + cds.interest_rate) ^ (step)
		end

end
