indexing

	description: "Objects that parse an expression or a cross-supplier variable"

	keywords: "variable expression parser supplier cross domain"
	url: "http://prism/modules/wacko/wakka.php?wakka=ExpressionLanguage"
	author: "$Author: sm $ Colin Adams"
	group: "RC London (tools)"
	reviewed_by: "No-one"
	quality: "Unrated"
	revision: "$Revision: #8 $"
	date: "$Date: 2009/08/24 $"
	archive: "$File: //rose/source/kernel/math/cross_supplier_expression_parser.e $"

class CROSS_SUPPLIER_EXPRESSION_PARSER

inherit
	UT_SHARED_URL_ENCODING
		export {NONE} all
			{CROSS_SUPPLIER_EXPRESSION_PARSER} is_equal, standard_is_equal, deep_twin
		end

	ROSE_SHARED_GLOBAL
		export {NONE} all end

	CONTEXT_AWARE
		export {NONE} all end
	ANY

create
	make

feature {NONE} -- Initialization

	make (a_variables: ENTITY_VARIABLES [ROSE_TARGET [ENTITY]]; a_peering_entity: like peering_entity;
	      a_date: like date; a_selector: like target_selector) is
			-- Initialize `Current.
		require
			a_variables_not_void: a_variables /= Void
			a_peering_entity_not_void: a_peering_entity /= Void
			a_date_not_void: a_date /= Void
			a_selector_not_void: a_selector /= Void
		do
			create expression_parser.make (a_variables)
			expression_parser.set_cross_supplier_parser (Current)
			set_peering_entity (a_peering_entity)
			set_date (a_date)
			target_selector := a_selector
		ensure
			peering_entity_set: peering_entity = a_peering_entity
			date_set: date = a_date
			target_selector_set: target_selector = a_selector
		end

feature -- Access

	peering_entity: ENTITY
			-- Peering_Entitying entity for creation of cross-supplier targets

	date: ROSE_DATE
			-- Date for finding cross-supplier targets

	base_uri: UT_URI
			-- Optional base-URI for resolution of relative URIs

	expression_parser: ROSE_EXPRESSION_PARSER [ROSE_TARGET [ENTITY]]
			-- Parser for local expressions

	target_selector: ROSE_TARGET_SELECTOR
			-- Helper for selecting signatures

	last_expression: ROSE_EXPRESSION [ROSE_TARGET [ENTITY]]
			-- Last expression sucessfully parsed

	last_error: STRING
			-- Possible error from last call to parse routines

feature -- Status report

	is_error: BOOLEAN is
			-- Did last call to parse routines fail?
		do
			Result := last_error /= Void
		end

feature -- Element change

	set_peering_entity (a_peering_entity: like peering_entity) is
			-- Set `peering_entity' to `a_peering_entity'.
		do
			peering_entity := a_peering_entity
		ensure
			peering_entity_set: peering_entity = a_peering_entity
		end

	set_date (a_date: like date) is
			-- Set `date' to `a_date'.
		do
			date := a_date
		ensure
			date_set: date = a_date
		end

	set_base_uri (a_uri: like base_uri) is
			-- Set `base_uri' to `a_uri'.
		require
			a_uri_not_void: a_uri /= Void
			absolute_base_uri: a_uri.is_absolute
		do
			base_uri := a_uri
		ensure
			base_uri_set: base_uri = a_uri
		end

feature -- Basic operations

	parse (a_expression: STRING) is
			-- Parse `a_expression' as an expression or a cross-supplier variable.
		require
			a_expression_not_void: a_expression /= Void
		local
			l_uri: UT_URI
			l_splitter: ST_SPLITTER
			l_vars: DS_LIST [STRING]
			l_attribute: STRING
		do
			last_error := Void
			last_expression := Void
			if Url_encoding.has_excluded_characters (a_expression) then
				parse_local_expression (a_expression)
			elseif is_cross_supplier_uri (a_expression) then
				if base_uri = Void then
					create l_uri.make (a_expression)
				else
					create l_uri.make_resolve (base_uri, a_expression)
				end
				if not l_uri.is_absolute then
					last_error := "No scheme in URI: " + l_uri.full_reference + ". Expecting " + Variable_scheme
				elseif not l_uri.scheme.same_string (Variable_scheme) then
					last_error := "Invalid scheme: " + l_uri.scheme + ". Expecting " + Variable_scheme
				elseif l_uri.path_items.count /= 1 or not l_uri.has_authority or not l_uri.has_path_base or l_uri.has_query or l_uri.has_fragment then
					last_error := "Syntax: " + Variable_scheme + "://context-name/target-name/variable-mnemonic"
				else
					create l_splitter.make_with_separators (".")
					l_vars := l_splitter.split_greedy (l_uri.path_base)
					if l_vars.count > 2 then
						last_error := "Not more than one . is allowed in variable mnemonic. Found: " + l_uri.path_base
					else
						if l_vars.count = 2 then
							l_attribute := l_vars.item (2)
						end
						parse_cross_supplier_variable (l_uri.authority, l_uri.path_items.item (1).decoded, l_vars.item (1), l_attribute)
					end
				end
			else
				parse_local_expression (a_expression)
			end
		ensure
			error_or_local_expression_or_cross_supplier_variable: last_error /= Void xor last_expression /= Void
		end

	parse_local_expression (a_expression: STRING) is
			-- Parse `a_expression' as a local expression.
		require
			a_expression_not_void: a_expression /= Void
		do
			last_expression := expression_parser.variables.variable (a_expression)
			if last_expression = Void then
				last_expression := expression_parser.parsed_string (a_expression)
				if last_error /= Void then
					last_expression := Void
				end
				if error.signaled then
					error.flush_and_acknowledge
					last_expression := Void
					last_error := "Error signaled by ROSE_EXPRESSION_PARSER"
				else
					expression_parser.variables.variables.force (last_expression, a_expression)
				end
			end
		ensure
			error_or_local_expression_not_void: last_error /= Void xor last_expression /= Void
		end

	parse_cross_supplier_variable (a_supplier, a_target, a_mnemonic, a_attribute: STRING) is
			-- Find `a_mnemonic' for `a_target' in context of `a_supplier'.
		require
			a_supplier_not_void: a_supplier /= Void
			a_target_not_void: a_target /= Void
			a_mnemonic_not_void: a_mnemonic /= Void
		local
			l_supplier, l_id: STRING
			l_context: like existing_context_from_name
			l_peer: ENTITY
			l_target: ROSE_TARGET [ENTITY]
			l_targets: ROSE_LINEAR_ARRAY [ROSE_TARGET [ENTITY]]
			l_found: BOOLEAN
			l_expression: like last_expression
		do
			l_supplier := a_supplier.twin
			l_supplier.replace_substring_all ("_", " ")
			l_supplier.to_upper
			l_context := existing_context_from_name (l_supplier)
			if l_context = Void then
				last_error := "Unable to find context: " + l_supplier
			else
				l_peer := peering_entity.peer (l_context, date)
				l_targets := l_peer.ur_targets
				from l_targets.start until l_found or l_targets.after loop
					if a_target.is_case_insensitive_equal (l_targets.current_item.generating_type) then
						l_found := True
					else
						l_targets.forth
					end
				end
				if not l_found then
					last_error := a_target + " is not a ROSE_TARGET [ENTITY]"
				else
					l_target := l_targets.current_item
					if a_attribute = Void then
						l_id := a_mnemonic.twin
					else
						l_id := a_mnemonic + "." + a_attribute
					end
					l_id.to_lower
					l_expression := l_target.expressions.variable_from_mnemonic (l_id)
					if l_expression = Void then
						last_error := "Unable to find " + l_id + " for target " + a_target + " for supplier " + l_supplier
					else
						l_target := target_selector.target_from_ur_target (l_target, l_peer, date)
						if l_target = Void then
							last_error := "Unable to fetch target " + a_target + " from database for " + l_id + " from supplier " + l_supplier
						else
							create {ROSE_FOREIGN_EXPRESSION} last_expression.make (
								l_expression, l_context, a_target, target_selector)
						end
					end
				end
			end
		ensure
			error_or_cross_supplier_variable: last_error /= Void xor last_expression /= Void
		end

feature {NONE} -- Implementation

	Variable_scheme: STRING is "var"
			-- URI scheme for cross-supplier variables

	is_cross_supplier_uri (a_expression: STRING): BOOLEAN is
			-- Does `a_expression' look like a URI?
		require
			a_expression_not_void: a_expression /= Void
		do
			Result := a_expression.has (':') or
				a_expression.substring_index (".", 1) = 1 or
				a_expression.substring_index ("..", 1) = 1
		end

	fetched_target (a_ur_target: ROSE_TARGET [ENTITY]; a_entity: ENTITY): ROSE_TARGET [ENTITY] is
			-- Target fetched from database whenever possible
		require
			a_ur_target_not_void: a_ur_target /= Void
			a_entity_not_void: a_entity /= Void
		local
			l_record, l_record_2: ROSE_SIGNATURE_RECORD [ENTITY]
			l_statement: STRING
			l_targets: ROSE_LINEAR_ARRAY [ROSE_SIGNATURE_RECORD [ENTITY]]
			l_entity_date: ROSE_ENTITY_DATE [ENTITY]
		do
			l_record ?= a_ur_target
			if l_record /= Void then
				l_record_2 := l_record.twin
				l_record_2.set_entity (a_entity)
				l_statement := "#entity_id.id = entity_id and period_end_date = '" + date.ccyymmdd.out + "'"
				l_targets := l_record.targets (<<l_record_2>>, l_statement)
				if not l_targets.is_empty then
					Result := l_targets.first
					l_entity_date ?= Result
					if l_entity_date /= Void then
						l_entity_date.set_date (date)
					end
				end
			end
		end

invariant

	peering_entity_not_void: peering_entity /= Void
	date_not_void: date /= Void
	expression_parser_not_void: expression_parser /= Void
	absolute_base_uri: base_uri /= Void implies base_uri.is_absolute
	target_selector_not_void: target_selector /= Void

end

