indexing
	description: "Class that provides an interface for comparing 2 values from an expression"
	author: "Mauricio Bustos"
	last_updater: "$Author: it $"
	date: "$Date: 2008/12/01 $"
	revision: "$Revision: #11 $"
	path: "$File: //rose/source/kernel/persistence/databases/signature_tagged/data_comparison.e $"

class
	DATA_COMPARISON[reference G -> ROSE_SIGNATURE_RECORD[ENTITY]]

inherit
	ROSE_SHARED_GLOBAL
	STID_TESTENV_CONSTANTS
	ROSE_CONSTANTS
	
create
	make

feature {NONE} -- Creation
	
	make (a_target: G; a_x, a_y: DOUBLE; a_expression: ROSE_EXPRESSION [ANY]) is
			-- Create the comparator
		do
			target := a_target
			x := a_x
			y := a_y
			difference := ieee_math.qnan
			if a_expression = Void then
				normalized_x := ieee_math.qnan
				normalized_y := ieee_math.qnan
			else
				if a_expression.variable_description.source.is_equal (c_risk) then
					normalized_x := a_expression.normalized_value_from_raw (x,target)
					normalized_y := a_expression.normalized_value_from_raw (y,target)
				else
					normalized_x := x
					normalized_y := y
				end
			end
			if ieee_math.ieee_is_eq (normalized_x, normalized_y) then
				difference := 0.0
			elseif is_defined_double (normalized_x) and is_defined_double (normalized_y) then
				difference := (normalized_x - normalized_y) ^ 2
			elseif is_defined_double (x) and is_defined_double (y) then
				difference := (x - y) ^ 2
			end
		end

feature -- Access

	target: G
			-- Originating expression target
	x, normalized_x: DOUBLE
			-- Independent variable
	y, normalized_y: DOUBLE
			-- Depenent variable
	difference: DOUBLE
			-- Squared difference
	
	ratio: DOUBLE is
			-- The comparison ratio of y to x
		do
			Result := (normalized_x - normalized_y).abs / ((normalized_x.abs + normalized_y.abs) / 2)
		end

	display_string: STRING is
			-- Output string for the class
		do
			Result := target.signature_display_string + ": " +
			format_double.formatted (x) + ": " +
			format_double.formatted (y)
		end

	formatted_x: STRING is
			-- Formatted string for 'x'
		do
			if is_defined_double (x) then
				Result := format_double.formatted (x)
			else
				Result := "-"
			end
		end

	formatted_y: STRING is
			-- Formatted string for 'y'
		do
			if is_defined_double (y) then
				Result := format_double.formatted (y)
			else
				Result := "-"
			end
		end

	formatted_normalized_x: STRING is
			-- Formatted string for 'normalized_x'
		do
			if is_defined_double (normalized_x) then
				Result := format_double.formatted (normalized_x)
			else
				Result := "-"
			end
		end

	formatted_normalized_y: STRING is
			-- Formatted string for 'normalized_y'
		do
			if is_defined_double (normalized_y) then
				Result := format_double.formatted (normalized_y)
			else
				Result := "-"
			end
		end

	formatted_ratio: STRING is
			-- Formatted string for 'ratio'
		do
			Result := format_double.formatted (ratio)
		end

	formatted_difference: STRING is
			-- Formatted string for 'difference'
		do
			Result := format_double.formatted (difference)
		end

feature {NONE} -- Implementation

	format_double: FORMAT_DOUBLE is
			-- Double formatter, create each time incase precision changes
		do
			create Result.make (precision + 5,precision)
			Result.set_separator (',')
		end

	precision: INTEGER is
			-- Precision asked for by the user's preferences
		local
			l_value: STRING
		do
			l_value := Global.user.config.value_default (precision_key, precision_default)
			if l_value.is_integer then
				Result := l_value.to_integer
			end
		end

	ieee_math: IEEE_MATH is
			-- Ieee math for comparison of nan data
		once
			create Result
		end
	
	c_risk: STRING is "risk"
			-- String constant for determination of noralization
	
end

