indexing
	description : "[
		Representation of a record component which does not contain any subcomponents
		]"
	author : "Mark Howard"

deferred class
	DATA_FIELD

inherit
	DATA_COMPONENT

feature -- Initialization

	make (a_pointer: POINTER; a_length: INTEGER; a_name: STRING) is
		do
			pointer := a_pointer
			stored_length := a_length
			name := clone (a_name)
		end

feature -- Access

	type: INTEGER		-- Type

	class_text: STRING is
		do
		end

	external_record_text: STRING is
		do
		end

	component_closure: LINKED_LIST [DATA_COMPONENT] is
		do
			create Result.make
			Result.extend (Current)
		end

	length: INTEGER is
		do
			Result := stored_length
		end

	date_time_value: ROSE_DATE is
		deferred
		end

	boolean_value: BOOLEAN is
		deferred
		end

	integer_value: INTEGER is
		deferred
		end

	real_value: REAL is
		deferred
		end

	double_value: DOUBLE is
		deferred
		end

	string_value: STRING is
		deferred
		end

	character_value: CHARACTER is
		deferred
		end

	anonymous: ARRAY [ANY] is
		once
		create Result.make (1,0)
		end

	display_specific_text: STRING is
		local
			a_name: STRING
		do
			a_name := clone (name)
			a_name.prune_all (' ')
			if is_array then
				Result := (
					"%T%T%Tfrom i_ := " + min_index_out + " until i_ > " + max_index_out + "%N" +
					"%T%T%Tloop%N" +
					"%T%T%T%Tprint_all (<<%"" + a_name + "(%"" + ",i_," + "%"): %"," + " read_" + a_name + "(i_),%"%%N%">>)%N" +
					"%T%T%T%Ti_ := i_+1%N" +
					"%T%T%Tend%N"
				)
			else
				Result := "%T%T%Tprint_all (<<%"" + a_name + "%",%": %",read_" + a_name + ",%"%%N%">>)%N"
			end
		end

feature -- Status report

	needs_make: BOOLEAN is
		do
			Result := is_array
		end

feature -- Basic operations

	read_field (a_index: INTEGER; a_receptacle: ANY) is
		local
			a_array: ARRAY [ANY]
		do
			if a_receptacle.same_type (anonymous) then
				a_array ?= a_receptacle
				a_array.put (value,a_index)
			else
				get_field (a_index,a_receptacle)
			end
		end

	get_field (an_index: INTEGER; a_receptacle: ANY) is
		local
			l_string: STRING
		do
			inspect internal.field_type (an_index,a_receptacle)
			when {INTERNAL}.Boolean_type then
				internal.set_boolean_field (an_index,a_receptacle,boolean_value)
			when {INTERNAL}.Integer_type then
				internal.set_integer_field (an_index,a_receptacle,integer_value)
			when {INTERNAL}.Real_type then
				internal.set_real_field (an_index,a_receptacle,real_value)
			when {INTERNAL}.Double_type then
				internal.set_double_field (an_index,a_receptacle,double_value)
			when {INTERNAL}.Character_type then
				internal.set_character_field (an_index,a_receptacle,character_value)
			when {INTERNAL}.Reference_type then
				l_string ?= internal.field (an_index,a_receptacle)
				if l_string /= Void then
					l_string.from_c_substring (pointer,1,length)
				else
					internal.set_reference_field (an_index,a_receptacle,string_value)
				end
			end
		end

	read (a_receptacle: ANY) is
		do
		end

	attach_to_pointer (a_pointer: POINTER) is
		do
			pointer := a_pointer
		end

end
