indexing
	description: "Class that provides an interface for comparing 2 string values from an expression."
	author: "Mauricio Bustos"
	last_updater: "$Author: it $"
	date: "$Date: 2008/12/01 $"
	revision: "$Revision: #5 $"
	path: "$File: //rose/source/kernel/persistence/databases/signature_tagged/data_string_comparison.e $"

class 
	DATA_STRING_COMPARISON [reference G -> ROSE_SIGNATURE_RECORD [ENTITY]]

inherit
	DATA_COMPARISON [G]
		rename
			make as old_make
		redefine
			ratio,
			difference,
			display_string,
			formatted_x,
			formatted_y,
			formatted_normalized_x,
			formatted_normalized_y
		end
	
create 
	make

feature {NONE} -- Creation

	make (a_target: G; a_x, a_y: STRING; a_expression: ROSE_EXPRESSION [ANY]) is
			-- Create the comparator
		require
			valid_target: a_target /= Void
		do
			target := a_target
			x_string := a_x
			y_string := a_y
			if not equal (x_string, y_string) then
				difference := 1.0
			end
		end
	
feature -- Access

	x_string: STRING
			-- The first string in the comparison
	y_string: STRING
			-- The second string in the comparison

	ratio: DOUBLE is
			-- The comparison ratio of y to x, in the case of strings this is an equality test
		do
			if not equal (x_string, y_string) then
				Result := 1.0
			end
		end

	difference: DOUBLE
			-- In the case of strings this is used as an equality test

	display_string: STRING is
			-- Formatted display string for the target and the x and y values
		do
			Result := target.signature_display_string + ": " + formatted_x + ": " + formatted_y
		end

	formatted_x: STRING is
			-- Formatted version of the x_string
		do
			Result := x_string
		end

	formatted_y: STRING is
			-- Formatted version of the y_string
		do
			Result := y_string
		end

	formatted_normalized_x: STRING is
			-- Formatted version of the x_string
		do
			Result := x_string
		end

	formatted_normalized_y: STRING is
			-- Formatted version of the y_string
		do
			Result := y_string
		end

end


