indexing
	description: "Data Structure"
	author: "Mark Howard"
	keywords: "record,external,c access"
	date: "$Date: 2009/08/20 $"
	revision: "$Revision: #79 $"

class
	DATA_STRUCTURE [G->DATA_COMPONENT]

inherit
	ROSE_CONSTANTS
		undefine
			copy, is_equal
		end
	UTILITY
		undefine
			copy, is_equal
		end
	TRACEABLE
		undefine
			copy, is_equal
		redefine
			trace
		end
	DATA_COMPONENT
		rename
			display as dc_display
		undefine
			copy, is_equal, read_text, make_one_text, cpu_ticks
		redefine
			length_out, external_record_text_with_parents
		end
	DATA_COMPONENT
		undefine
			copy, is_equal, cpu_ticks
		redefine
			display, read_text, make_one_text, length_out, external_record_text_with_parents
		select
			display
		end
	DATA_DESCRIPTORS [G]
		redefine
			update_input_map, set_parameters, get_values, make
		end
	ROSE_EXECUTION_ENVIRONMENT
		rename
			put as ree_put,
			system as ree_system
		undefine
			copy,
			is_equal
		end
	MISMATCH_CORRECTOR
		undefine
			copy, is_equal
		redefine
			correct_mismatch
		end

create
	make
feature -- Initialization

	make (a_lower, a_upper: INTEGER) is
			-- Allocate array; set index interval to
			-- `min_index' .. `max_index'; set all values to default.
			-- (Make array empty if `min_index' = `max_index' + 1).
		do
			precursor (a_lower, a_upper)
			name := "NO NAME"
		end


feature -- Access

	map,input_object_map,input_array_map: ARRAY [INTEGER]		-- Maps
	input_name_map: ARRAY[STRING]
	offsets: ARRAY [INTEGER]
	target_type: INTEGER
	array_type: STRING is do end
	field_type_string: STRING is do end
	tuple: TUPLE

	receptacle_type : INTEGER
		-- Receptacle type
	Array_Receptacle_Type : INTEGER is 1
	Tuple_Receptacle_Type : INTEGER is 2
	Object_Receptacle_Type : INTEGER is 3

feature -- Basic operations

	display_values is
			-- Display values
		do
			linear_representation.do_all (agent display_value)
		end

	set_top (a_name: STRING) is
		do
			name := clone (a_name)
			name.to_lower
			type_string := clone (a_name)
		end

	trace is
		-- <Precursor>.
		do
			Precursor
			display_values
		end

	anonymous: ARRAY [ANY] is
		once
			create Result.make (1,0)
		end

	update_map (a_receptacle: ANY) is
			-- Update the internal map
		local
			a_field_name: STRING
			i,an_index: INTEGER
			l_array: ARRAY [ANY]
		do
			if a_receptacle /= Void and then internal.dynamic_type (a_receptacle) /= target_type then
				tuple ?= a_receptacle
				if tuple /= Void then
					receptacle_type := Tuple_Receptacle_Type
				else
					target_type := internal.dynamic_type (a_receptacle)
					create map.make (1,count)
					if a_receptacle.same_type (anonymous) then
						receptacle_type := Array_Receptacle_Type
						l_array ?= a_receptacle
						from
							i := 1
						until
							i > l_array.count
						loop
							map.put (i,i)
							i := i+ 1
						end
					else
						receptacle_type := Object_Receptacle_Type
						from
							i := 1
						until
							i > internal.field_count (a_receptacle)
						loop
							a_field_name := clone (internal.field_name (i,a_receptacle))
							a_field_name.to_upper
							if a_field_name.item (1) = '$' then
								a_field_name := a_field_name.substring (3,a_field_name.count)
							end
							a_field_name.prune_all (' ')
							an_index := column_index (a_field_name)
							if an_index > 0 then
								map.put (i,an_index)
							end
							i := i+1
						end
					end
				end
			end
		end

	update_input_map (l_input: ANY;l_binding: ARRAY [STRING]) is
		local
			a_field_name: STRING
			i,j: INTEGER
			l_name_table: HASH_TABLE [INTEGER,STRING]
			l_name: STRING
		do
			if l_input /= Void and then internal.dynamic_type (l_input) /= input_type then
				input_type := internal.dynamic_type (l_input)
				create input_object_map.make (1,count)
				create l_name_table.make (internal.field_count (l_input))
				from
					i := 1
				until
					i > internal.field_count (l_input)
				loop
					a_field_name := clone (internal.field_name (i,l_input))
					a_field_name.to_upper
					a_field_name.prune_all (' ')
					l_name_table.put (i,a_field_name)
					i := i+1
				end
				input_array_map := Void		--| Force the next section to execute.
			end
			if input_array_map = Void then
				create input_array_map.make (1,count)
				create input_name_map.make(1, count)
				from
					i := 1
					j := 1
				until
					i > count
				loop
					l_name := Void
					if l_binding.valid_index (i) then
						l_name := l_binding.item (i)
					end
					if l_name /= Void then
						if l_name.item (1)='?' then
								--| Handle parameters of the form ?999.
								--| Use the n'th ? parameter here.
							input_array_map.put (l_name.substring (2, l_name.count).to_integer, i)
						else
								--| Handle paramters of the form :xxx.
							if l_name_table/=Void then
								input_object_map.put (l_name_table.item (l_name),i)
								input_name_map.put(l_name, i)
							else
									--| If l_name_table is Void it means that either the input_type hasn't
									--| changed since the last call to this routine or somebody used named
									--| parameters but didn't supply a source object.
							end
						end
					else
						--| Handle ? parameters.
						input_array_map.put (j,i)
						j := j+1
					end
					i := i+1
				end
			end
		end

	input_type: INTEGER		-- Input type

	get_field (an_index: INTEGER;a_receptacle: ANY) is
		do
			read (internal.field (an_index,a_receptacle))
		end

	set_values (receptacle: ANY) is
		local
			i,an_index: INTEGER
		do
			update_map (receptacle)
			from
				i := 1
			until
				i > count
			loop
				an_index := map.item (i)
				if an_index > 0 then
					item (i).set_null_value (False)
					item (i).set_value (internal.field (an_index,receptacle))
				else
					item (i).set_null_value (True)
					item (i).set_value (Void)
				end
				i := i+1
			end
		end

	set_values_from_array (a_some_values: ARRAY [ANY]) is
		local
			i: INTEGER
			j: INTEGER
		do
			if a_some_values /= Void then
				if input_array_map /= Void then
					if (a_some_values.count > input_array_map.count) then
						raise ("Too many parameters")
						-- We can't check for too few since it is possible to
						-- specify that some input parameters are to be reused.
						-- "select * from table where start_date>=? and end_date<?1"
						-- input_array_map.count=2 but only one parameter will be supplied.
					end

					from
						i := 1
					until
						i > count
					loop
						j := input_array_map.item (i)
						if j > 0 then
							if a_some_values.valid_index(j) then
								item (i).set_value (a_some_values.item (j))
							else
								raise("A parameter was missing")
							end
						end
						i := i+1
					end
				else
						--| I think that this code is no longer used because input_array_map is now always built. RJ.
					from
						i := 1
					until
						i > count or i > a_some_values.count
					loop
						item (i).set_value (a_some_values.item (i))
						i := i+1
					end
				end
			end
		end

	set_parameters (l_object: ANY;some_values: ARRAY [ANY]) is
		local
			i,j: INTEGER
			l_message: STRING
		do
			if input_object_map = Void or else l_object = Void then
				set_values_from_array (some_values)
			else
				from
					i := 1
				until
					i > count
				loop
					j := input_object_map.item (i)
					if j /= 0 then
						item (i).set_value (internal.field (j,l_object))
					else
						j := input_array_map.item(i)
						if some_values/=Void and then
							some_values.valid_index(j) and then
							input_array_map.valid_index(i)
						then
							item(i).set_value(some_values.item(j))
						else
							l_message := "Parameter " + i.out
							if input_name_map=Void or else input_name_map.item(i)=Void
							then
								l_message.append(" (unnamed) is missing")
							else
								if input_name_map.valid_index(i)
								then
									l_message.append(" (" + input_name_map.item(i) + ") is missing")
								else
									l_message.append(" () is missing")
								end
							end

							raise (l_message)

							--| If we ever change this to check for the proper number
							--| of parameters we must also change INSERTION_PREDICATE:insert
							--| because it depends upon this behaviour.
						end
					end
					i := i+1
				end
			end
		end

	value: ANY is
		do
		end

	set_value (a_value: ANY) is
		do
		end

	set_null_value (a_value: BOOLEAN) is
		do
		end

	is_null: BOOLEAN is
		do
		end

	display is
		do
			dc_display
			display_values
		end

	length: INTEGER is
		local
			data_structure_length: DATA_STRUCTURE_LENGTH [G]
		do
			create data_structure_length.make (Current)
			Result := data_structure_length.length
		end

	length_out: STRING is
		local
			data_structure_length: DATA_STRUCTURE_LENGTH [G]
		do
			create data_structure_length.make (Current)
			if data_structure_length.uses_symbolic then
				Result := data_structure_length.symbolic_length
			else
				Result := data_structure_length.length.out
			end
		end

	attach_to_pointer (a_pointer: POINTER) is
		local
			i: INTEGER
			new_pointer: POINTER
		do
			pointer := a_pointer
			from
				i := 1
				new_pointer := a_pointer
			until
				i > count
			loop
				item (i).attach_to_pointer (new_pointer)
				new_pointer := pointer_plus (new_pointer,item (i).length * item (i).repeat_count)
				i := i+1
			end
		end

	component_closure: LINKED_LIST [DATA_COMPONENT] is
		local
			i: INTEGER
			next_list: LINKED_LIST [DATA_COMPONENT]
		do
			create Result.make
			Result.extend (Current)
			from
				i := 1
			until
				i > count
			loop
				next_list := item (i).component_closure
				if next_list /= Void then
					Result.append (next_list)
				end
				i := i+1
			end
		end

	external_record_text: STRING is
		do
			Result := "class " + declaration_string + "%N" +
				"%T-- Do NOT modify this file!%N" +
				"%T-- Created by the Classifier (DATA_STRUCTURE:external_record_text) on " + get_today + ".%N" +
				"%T-- User: " + current_user.full_login + "%N" +
				"inherit%N" +
				"%TEXTERNAL_RECORD%N" +
				"%T%Tredefine%N" +
				"%T%T%Tdisplay,%N" +
				"%T%T%Twrite,%N" +
				"%T%T%Tread,%N" +
				"%T%T%Trecord_length_,%N" +
				"%T%T%Tclassifier_major_version_,%N" +
				"%T%T%Tclassifier_minor_version_%N" +
				"%T%Tend%N" +
				"%TC_ACCESS_SUPPORT%N" +
				"feature%N" +
				"%Tclassifier_major_version_: INTEGER is 2%N" +
				"%Tclassifier_minor_version_: INTEGER is 0%N" +
				"%N"
			Result.append (external_record_body)
		end

	external_record_text_with_parents (a_parent_classes: ARRAY [STRING]; a_comment: STRING): STRING is
		local
			i : INTEGER
			l_parent_class_list, l_string: STRING
		do
			create l_parent_class_list.make (0)
			if a_parent_classes /= Void  then
				from i := a_parent_classes.lower
				until i > a_parent_classes.upper
				loop
					a_parent_classes.item (i).to_upper
					l_string := clone (a_parent_classes.item (i))
					l_string.prune_all (' ')
					if l_string.is_equal ("") then
						l_parent_class_list := ""
					else
						l_parent_class_list := "%T"+l_parent_class_list + a_parent_classes.item (i) + "%N"
					end
					i := i + 1
				end
			end
			Result := "class " + declaration_string + "%N" +
				"%T-- Do NOT modify this file!%N" +
				"%T-- Created by the Classifier (DATA_STRUCTURE:external_record_text) on " + get_today + ".%N" +
				"%T-- User: " + current_user.full_login + "%N" +
				a_comment +
				"inherit%N" +
				l_parent_class_list +
				"%TEXTERNAL_RECORD%N" +
				"%T%Tredefine%N" +
				"%T%T%Tdisplay,%N" +
				"%T%T%Twrite,%N" +
				"%T%T%Tread,%N" +
				"%T%T%Trecord_length_,%N" +
				"%T%T%Tclassifier_major_version_,%N" +
				"%T%T%Tclassifier_minor_version_%N" +
				"%T%Tend%N" +
				"%TC_ACCESS_SUPPORT%N" +
				"feature%N" +
				"%Tclassifier_major_version_: INTEGER is 2%N" +
				"%Tclassifier_minor_version_: INTEGER is 0%N" +
				"%N"
			Result.append (external_record_body)
		end

	external_record_body: STRING is
		local
			i: INTEGER
			a_feature_text: STRING
			a_read_text: STRING
			a_write_text: STRING
			a_field_name: STRING
			a_length_text: STRING
			l_interpreters: LINEAR [STRING]
			l_interpreter_string,l_interpreter_name: STRING
		do
			Result := ""
			a_feature_text := ""
			a_read_text := "%Tread is%N%T%Tlocal%N%T%T%Ti_: INTEGER%N%T%Tdo%N"
			a_write_text := "%Twrite is%N%T%Tlocal%N%T%T%Ti_: INTEGER%N%T%Tdo%N"
			l_interpreters := interpreters.linear_representation
			l_interpreter_string := ""
			from
				l_interpreters.start
			until
				l_interpreters.off
			loop
				if l_interpreters.item.count > 0 then
					l_interpreter_name := clone (l_interpreters.item)
					l_interpreter_name.to_lower
					l_interpreter_name.append ("_")
					l_interpreter_string.append ("%T" + l_interpreter_name + ": " + l_interpreters.item + " is%N" +
						"%T%Tonce%N" +
						"%T%T%Tcreate Result%N" +
						"%T%Tend%N")
				end
				l_interpreters.forth
			end
			Result.append (l_interpreter_string)
			from
				i := 1
			until
				i > count
			loop
				a_field_name := clone (item (i).name)
				a_field_name.prune_all (' ')
				a_feature_text.append (item (i).feature_body)
				a_feature_text.append ("%N")
				a_read_text.append (item (i).read_specific_text)
				a_write_text.append (item (i).write_specific_text)
				i := i+1
			end
			a_feature_text.append (display_text)
			a_feature_text.append ("end%N")
			a_feature_text.append ("%N")
			a_read_text.append ("%T%Tend%N%N")
			a_write_text.append ("%T%Tend%N%N")
			a_length_text := "%Trecord_length_: INTEGER is%N" +
				"%T%Tdo%N" +
				"%T%T%TResult := " + length_out + "%N" +
				"%T%Tend%N%N"
			Result.append (a_read_text + a_write_text + a_length_text + a_feature_text)
		end

	needs_make: BOOLEAN is
		do
			Result := True
		end

	array_receptacle_index : INTEGER is
		local
			i : INTEGER
		once
			i := internal.field_count(Current)
			from until i < 1 or Result /= 0 loop
				if internal.field_name(i,Current).is_equal("receptacle_array") then
					Result := i
				end
				i := i - 1
			end
		end
	receptacle_array : ARRAY[ANY]

	get_values (a_receptacle: ANY) is
		local
			i,an_index: INTEGER
		do
			update_map (a_receptacle)
			inspect receptacle_type
			when Tuple_Receptacle_Type then
				get_tuple (tuple)
			when Array_Receptacle_Type then
				internal.set_reference_field(array_receptacle_index,Current,a_receptacle)
				from
					i := 1
				until
					i > upper
				loop
					an_index := map.item (i)
					receptacle_array.put(item(i).value,an_index)
					i := i + 1
				end
			when Object_Receptacle_Type then
				from
					i := 1
				until
					i > upper
				loop
					--an_index := map.item (i)
					-- Calling area below to force inlining for performance
					an_index := map.area.item(i - 1)
					if an_index > 0 then
						--item (i).get_field (an_index,a_receptacle)
						-- Calling area below to force inlining for performance
						area.item (i - 1).get_field (an_index,a_receptacle)
					end
					i := i + 1
				end
			end
		end

	get_tuple (a_tuple: TUPLE) is
		require
			types_match: count = a_tuple.count
		local
			i: INTEGER
		do
			from
				i := 1
			until
				i > count
			loop
				if a_tuple.is_reference_item(i)
				then
					a_tuple.put(item(i).value, i)
				elseif a_tuple.is_double_item(i)
				then
					a_tuple.put(item(i).double_value, i)
				elseif a_tuple.is_real_item(i)
				then
					a_tuple.put(item(i).real_value, i)
				elseif a_tuple.is_integer_32_item(i)
				then
					a_tuple.put(item(i).integer_value, i)
				elseif a_tuple.is_boolean_item(i)
				then
					a_tuple.put(item(i).boolean_value, i)
				elseif a_tuple.is_character_item(i)
				then
					a_tuple.put(item(i).character_value, i)
				elseif a_tuple.is_integer_8_item(i)
				then
					a_tuple.put(item(i).integer_value.to_integer_8, i)
				elseif a_tuple.is_integer_16_item(i)
				then
					a_tuple.put(item(i).integer_value.to_integer_16, i)
				elseif a_tuple.is_integer_64_item(i)
				then
					a_tuple.put(item(i).double_value.truncated_to_integer_64, i)
--				elseif a_tuple.is_wide_character_item(i)
--				then
--					raise("Unsupported type (WIDE_CHARACTER).")
				elseif a_tuple.is_pointer_item(i)
				then
					raise("Unsupported type (POINTER).")
				else
					raise("Unhandled type.")
				end

				i := i + 1
			end
		end

	read_field (an_index: INTEGER;a_receptacle: ANY) is
		do
			read (internal.field (an_index,a_receptacle))
		end

	read (a_receptacle: ANY) is
		local
			record_array: RECORD_ARRAY [ANY]
		do
			record_array ?= a_receptacle
			if record_array /= Void then
				record_array.inflate (min_index,max_index)
				from
					repeat_index := 0
				until
					repeat_index > max_index - min_index
				loop
					attach_to_pointer (pointer_plus (pointer,repeat_index * length))
					get_values (record_array.item (min_index + repeat_index))
					repeat_index := repeat_index + 1
				end
			else
				get_values (a_receptacle)
			end
		end

	feature_body : STRING is
		local
			a_name: STRING
			l_return_declaration_string: STRING
		do
			a_name := clone (name)
			a_name.prune_all (' ')
				--| About 'l_return_declaration_string'.
				--| If we create a variable:
				--|		my_var: SOME_TYPE
				--| and the user wants some special processing for 'my_var'
				--| he can create a class called CHILD_SOME_TYPE.
				--| Note that 'l_return_declaration_string' would be 'return_my_var_type_'.
				--| The functions that we create do not use SOME_TYPE but rather
				--| 'like return_my_var_type_'. The user can redefine the function
				--| 'return_my_var_type_' to be of type CHILD_SOME_TYPE and the
				--| problem is solved.
			l_return_declaration_string := "return_" + a_name + "_type_"
			if is_array then
				Result := "%T" + l_return_declaration_string + ": " + declaration_string + " is %N" +
					"%T%Tdo%N" +
					"%T%T%T-- You don't call it, you say 'like return_XXX_type_'.%N" +
					"%T%Tend%N" +
					"%T" + a_name + ": ARRAY [like " + l_return_declaration_string + "]%N" +
					"%Tread_" + a_name + " (an_index: INTEGER): like " + l_return_declaration_string + " is %N" +
					"%T%Tdo%N" +
					"%T%T%Tcreate Result%N" +
					"%T%T%TResult.initialize_ (Current," +
					offset_out + "," + length_out + "," +
					min_index_out + "," + max_index_out + ")%N" +
					"%T%T%TResult.set_i1_ (an_index)%N" +
					"%T%Tend%N" +
					"%Tset_" + a_name + " (a_value: like " + a_name + ") is%N" +
					"%T%Tdo%N" +
					"%T%T%T" + a_name + " := a_value%N" +
					"%T%Tend%N"
			else
				Result := "%T" + l_return_declaration_string + ": " + declaration_string + " is %N" +
					"%T%Tdo%N" +
					"%T%T%T-- You don't call it, you say 'like return_XXX_type_'.%N" +
					"%T%Tend%N" +
					"%T" + a_name + ": like " + l_return_declaration_string + "%N" +
					"%Tread_" + a_name + ": " + "like " + a_name + " is %N" +
					"%T%Tdo%N" +
					"%T%T%Tcreate Result%N" +
					"%T%T%TResult.initialize_ (Current," +
					offset_out + "," + length_out + "," +
					min_index_out + "," + max_index_out + ")%N" +
					"%T%Tend%N" +
					"%Tset_" + a_name + " (a_value: like " + a_name + ") is%N" +
					"%T%Tdo%N" +
					"%T%T%T" + a_name + " := a_value%N" +
					"%T%T%T" + a_name + ".initialize_ (Current," +
					offset_out + "," + length_out + "," +
					min_index_out + "," + max_index_out + ")%N" +
					"%Tend%N"
			end
		end

	display_specific_text: STRING is
		local
			a_name: STRING
		do
			a_name := clone (name)
			a_name.prune_all (' ')
			if is_array then
				Result := ("%T%T%Tfrom i_ := " + min_index_out + " until i_ > " + max_index_out + "%N" +
					"%T%T%Tloop%N" +
					"%T%T%T%Tread_" + a_name + " (i_).display%N" +
					"%T%T%T%Ti_ := i_+1%N" +
					"%T%T%Tend%N")
			else
				Result := "%T%T%Tread_" + a_name + ".display%N"
			end
		end

	display_text: STRING is
		local
			i: INTEGER
		do
			Result := "%Tdisplay is%N%T%Tlocal%N%T%T%Ti_: INTEGER%N%T%Tdo%N"
			from
				i := 1
			until
				i > count
			loop
				Result.append (item (i).display_specific_text)
				i := i+1
			end
			Result.append ("%T%Tend%N")
		end

	write_specific_text: STRING is
		local
			a_name: STRING
		do
			a_name := clone (name)
			a_name.prune_all (' ')
			if is_array then
				Result := "%T%T%Tfrom i_ := " + min_index_out + " until i_ > " + max_index_out + "%N" +
					"%T%T%Tloop%N" +
					"%T%T%T%Tif " + a_name + ".item (i_).pointer_=default_pointer%N" +
					"%T%T%T%Tthen%N" +
					"%T%T%T%T%T" + a_name + ".item (i_).initialize_ (Current," +
					offset_out + "," + length_out + "," +
					min_index_out + "," + max_index_out + ")%N" +
					"%T%T%T%T%T" + a_name + ".item (i_).set_i1_ (i_)%N" +
					"%T%T%T%Tend%N" +
					"%T%T%T%T" + a_name + ".item (i_).write%N" +
					"%T%T%T%Ti_ := i_+1%N" +
					"%T%T%Tend%N"
			else
				Result := "%T%T%T" + a_name + ".write%N"
			end
		end

	read_specific_text: STRING is
		local
			a_name: STRING
		do
			a_name := clone (name)
			a_name.prune_all (' ')
			if is_array then
				Result :=
					"%T%T%Tif " + a_name + " = Void then create " + a_name + ".make (" + min_index_out + "," + max_index_out + ") end%N" +
					"%T%T%Tfrom i_ := " + min_index_out + " until i_ > " + max_index_out + "%N" +
					"%T%T%Tloop%N" +
					"%T%T%T%T" + a_name + ".put (read_" + a_name + " (i_),i_)%N" +
					"%T%T%T%T" + a_name + ".item (i_).read%N" +
					"%T%T%T%Ti_ := i_+1%N" +
					"%T%T%Tend%N"
			else
				Result := "%T%T%Tif " + a_name + " = Void then " + a_name + " := read_" + a_name + " end%N" +
					"%T%T%T" + a_name + ".read%N"
			end
		end

	set_from_text (l_other: STRING): STRING is
		local
			a_name: STRING
		do
			a_name := clone (name)
			a_name.prune_all (' ')
			Result := a_name + ".set_from (" + l_other + "." + a_name + ")%N"
		end

	read_text: STRING is
		local
			i: INTEGER
		do
			Result := "%Tread_ is%N%T%Tlocal%N%T%T%Ti_: INTEGER%N%T%Tdo%N"
			from
				i := 1
			until
				i > count
			loop
				Result.append (item (i).read_specific_text)
				i := i+1
			end
			Result.append ("%T%Tend%N")
		end

	interpreters: EXTENDABLE_BOUNDED_ARRAY [STRING]

	set_interpreters (a_interpreters: EXTENDABLE_BOUNDED_ARRAY [STRING]) is
		do
			interpreters := a_interpreters
		end

	external_class_text: STRING is
		local
			i: INTEGER
			a_feature_text: STRING
			l_transfer_text: STRING
			l_field_name: STRING
		do
			Result := "deferred class%N%T" + declaration_string + "%N%N" +
				"%T-- Do NOT modify this file!%N" +
				"%T-- Created by DATA_STRUCTURE.external_class_text on " + get_today + ".%N%N" +
				"%T-- User: " + current_user.full_login + "%N" +
				"inherit%N" +
				"%TEXTERNAL_RECORD%N" +
				"%T%Tredefine%N" +
				"%T%T%Tdisplay,write,read,record_length_,classifier_major_version_,classifier_minor_version_%N" +
				"%T%Tend%N" +
				"%TC_ACCESS_SUPPORT%N%N" +
				"feature -- Access%N%N" +
				"%Tclassifier_major_version_: INTEGER is 2%N" +
				"%Tclassifier_minor_version_: INTEGER is 0%N%N"
			a_feature_text := ""
			l_transfer_text := "%Tset_from (a_source: " + declaration_string + ") is %N%T%Tdo%N"
			from
				i := 1
			until
				i > count
			loop
				l_field_name := clone (item (i).name)
				l_field_name.prune_all (' ')
				if item (i).is_array then
					a_feature_text.append ("%T" + l_field_name + " (i: INTEGER): " + item (i).declaration_string + " is deferred end%N")
					a_feature_text.append ("%Tset_" + l_field_name + " (a_value: " + item (i).declaration_string + ", i: INTEGER) is deferred end%N%N")
				else
					a_feature_text.append ("%T" + l_field_name + ": " + item (i).declaration_string + " is deferred end%N")
					a_feature_text.append ("%Tset_" + l_field_name + " (a_value: " + item (i).declaration_string + ") is deferred end%N%N")
				end
				l_transfer_text.append (transfer_text (item (i)))
				i := i+1
			end
			a_feature_text.append (display_text)
			a_feature_text.append ("%Tend%N%N")
			l_transfer_text.append ("%Tend%N%N")
			Result.append (a_feature_text)
			Result.append (l_transfer_text)
			Result.append ("end%N")
		end

	external_receptacle_text: STRING is
		local
			i: INTEGER
			a_feature_text: STRING
			l_field_name: STRING
		do
			Result := "class%N%T" + declaration_string + "_RECORD%N%N" +
				"%T-- Do NOT modify this file!%N" +
				"%T-- Created by DATA_STRUCTURE.external_receptacle_text on " + get_today + ".%N%N" +
				"%T-- User: " + current_user.full_login + "%N" +
				"inherit%N" +
				"%T" + declaration_string + "%N" +
				"%T%Tredefine%N" +
				"%T%T%Tdisplay,write,read,record_length_,classifier_major_version_,classifier_minor_version_%N" +
				"%T%Tend%N" +
				"%TC_ACCESS_SUPPORT%N%N" +
				"feature -- Access%N%N" +
				"%Tclassifier_major_version_: INTEGER is 2%N" +
				"%Tclassifier_minor_version_: INTEGER is 0%N%N"
			a_feature_text := ""
			from
				i := 1
			until
				i > count
			loop
				l_field_name := clone (item (i).name)
				l_field_name.prune_all (' ')
				if item (i).is_array then
					a_feature_text.append ("%T" + l_field_name + " (i: INTEGER): " + item (i).declaration_string + "%N")
					a_feature_text.append ("%Tset_" + l_field_name + " (a_value: " + item (i).declaration_string + ", i: INTEGER) is deferred end%N%N")
				else
					a_feature_text.append ("%T" + l_field_name + ": " + item (i).declaration_string + "%N")
					a_feature_text.append ("%Tset_" + l_field_name + " (a_value: " + item (i).declaration_string + ")" + " is do " + l_field_name + " := a_value end%N%N")
				end
				i := i+1
			end
			a_feature_text.append ("end%N%N")
			Result.append (a_feature_text)
		end

	external_navigator_text: STRING is
		local
			i: INTEGER
			a_feature_text: STRING
			a_length_text: STRING
			l_interpreters: LINEAR [STRING]
			l_interpreter_string,l_interpreter_name: STRING
		do
			Result := "class%N%T" + declaration_string + "_NAVIGATOR%N%N" +
				"%T-- Do NOT modify this file!%N" +
				"%T-- Created by DATA_STRUCTURE.external_navigator_text on " + get_today + ".%N%N" +
				"%T-- User: " + current_user.full_login + "%N" +
				"inherit%N" +
				"%TEXTERNAL_RECORD%N" +
				"%T%Tredefine%N" +
				"%T%T%Tdisplay,write,read,record_length_,classifier_major_version_,classifier_minor_version_%N" +
				"%T%Tend%N" +
				"%TC_ACCESS_SUPPORT%N%N" +
				"feature -- Access%N%N" +
				"%Tclassifier_major_version_: INTEGER is 2%N" +
				"%Tclassifier_minor_version_: INTEGER is 0%N%N"
			a_feature_text := ""
			l_interpreters := interpreters.linear_representation
			l_interpreter_string := ""
			from
				l_interpreters.start
			until
				l_interpreters.off
			loop
				l_interpreter_name := clone (l_interpreters.item)
				l_interpreter_name.to_lower
				l_interpreter_name.append ("_")
				l_interpreter_string.append ("%T" + l_interpreter_name + ": " + l_interpreters.item + " is%N" +
				"%T%Tonce%N" +
				"%T%T%Tcreate Result%N" +
				"%T%Tend%N%N")
				l_interpreters.forth
			end
			Result.append (l_interpreter_string)
			from
				i := 1
			until
				i > count
			loop
				a_feature_text.append (navigator_field_texts (item (i)))
				i := i+1
			end
			a_feature_text.append (display_text)
			a_feature_text.append ("end%N%N")
			a_length_text := "%Trecord_length_: INTEGER is%N" +
				"%T%Tdo%N" +
				"%T%T%TResult := " + length_out + "%N" +
				"%T%Tend%N%N"
			Result.append (a_length_text + a_feature_text)
		end

	navigator_field_texts (a_field: G): STRING is
		local
			a_name: STRING
			l_interpreter: STRING
		do
			l_interpreter := a_field.interpreter_name
			if l_interpreter = Void then
				l_interpreter := a_field.default_interpreter
			end
			a_name := clone (a_field.name)
			if a_field.is_array then
				Result := "%T" + a_name + ": " + a_field.array_type + "%N" +
					"%T" + a_name + " (a_index: INTEGER): " + a_field.field_type_string + " is %N" +
					"%T%Tdo%N" +
					"%T%T%TResult := " + l_interpreter + ".decode (pointer_plus (pointer_," + a_field.offset_out + " + (a_index - " + min_index_out + ") * " + a_field.length_out + "), " + a_field.length_out + ")%N" +
					"%T%Tend%N%N" +
					"%Tget_" + a_name + "_pointer: POINTER is %N" +
					"%T%Tdo%N" +
					"%T%T%TResult := pointer_plus (pointer_, " + a_field.offset_out + ")%N" +
					"%T%Tend%N%N" +
					"%Tset_" + a_name + " (a_value: " + a_field.array_item_string + "; a_index: INTEGER) is %N" +
					"%T%Tdo%N" +
					"%T%T%T" + l_interpreter + ".encode (pointer_plus (pointer_, " + a_field.offset_out + " + (a_index - " + a_field.min_index_out + ") * " + a_field.length_out + "), a_value, " + a_field.length_out + ")%N" +
					"%T%Tend%N%N"
			else
				Result := "%T" + a_name + ": " + a_field.field_type_string + "%N" +
					"%T" + a_name + ": " + a_field.field_type_string + " is%N" +
					"%T%Tdo%N" +
					"%T%T%TResult := " + l_interpreter + ".decode (pointer_plus (pointer_, " + a_field.offset_out + "), " + a_field.length_out + ")%N" +
					"%T%Tend%N%N" +
					"%Tset_" + a_name + " (a_value: " + a_field.field_type_string + ") is %N" +
					"%T%Tdo%N" +
					"%T%T%T" + l_interpreter + ".encode (pointer_plus (pointer_, " + a_field.offset_out + "), a_value, " + a_field.length_out + ")%N" +
					"%T%Tend%N%N"
			end
		end

	transfer_text (a_field: G): STRING is
		local
			a_name: STRING
		do
			a_name := clone (a_field.name)
			if a_field.is_array then
				Result := "%T%T%Tfrom i_ := " + a_field.min_index_out + " until i_ > " + a_field.max_index_out + "%N" +
					"%T%T%Tloop%N" +
					"%T%T%T%Tset (" + a_name + " (a_source." + a_name + " (i_), i_))" +
					"%T%T%T%Ti_ := i_+1%N" +
					"%T%T%Tend%N"
			else
				Result := "%T%T%T" + a_field.text_of_set_from ("a_source") + "%N"
			end
		end

	text_of_set_from (a_other_source: STRING): STRING is
		local
			l_other_source: STRING
		do
			l_other_source := clone (a_other_source)
			l_other_source.prune_all (' ')
			Result := name + ".set_from (" + l_other_source + "." + name + ")"
		end

	descendant_header_template: STRING is
"[
class
	<type>

	-- Do NOT modify this file!
	-- Created by the Classifier (DATA_STRUCTURE:descendant_header_template) on <date>
	-- User: <user>

inherit
	<parent_type>
	C_ACCESS_SUPPORT

feature -- Access

]"

	class_header_text :	STRING 	is
		do
			Result := "deferred class%N%T" + declaration_string + "%N%N" +
"[
	-- Do NOT modify this file!
	-- Created by the Classifier (DATA_STRUCTURE:class_header_text) on <date>
	-- User: <user>

inherit
	EXTERNAL_RECORD
		redefine
			classifier_major_version_,
			classifier_minor_version_,
			display,
			record_length_
		end
	C_ACCESS_SUPPORT

feature -- Access

	classifier_major_version_: INTEGER is 2
	classifier_minor_version_: INTEGER is 0

]"
		end

	deferred_class_text: STRING is
		local
			i: INTEGER
		do
			Result := class_header_text
			Result.append (set_text)
			Result.append (display_text)
			Result.append (make_all_text)
			Result.append (
				"	made: BOOLEAN%N" +
				"	record_length_: INTEGER is %N" +
				"		do%N" +
				"			Result := " + length_out + "%N" +
				"		end%N%N")
			from
				i := 1
			until
				i > count
			loop
				if item (i).is_array then
					Result.append (item (i).deferred_set_ith_text)
					Result.append (item (i).deferred_ith_text)
				else
					Result.append (item (i).deferred_set_text)
					Result.append (item (i).deferred_field_text)
				end
				i := i+1
			end
			Result.append ("end%N")
		end

	record_class_text: STRING is
		local
			i: INTEGER
		do
			Result := clone (descendant_header_template)
			Result.replace_substring_all ("<type>",declaration_string + "_RECORD")
			Result.replace_substring_all ("<parent_type>",declaration_string)
			Result.replace_substring_all ("<date>", get_today)
			Result.replace_substring_all ("<user>", current_user.full_login)
			from
				i := 1
			until
				i > count
			loop
				if item (i).is_array then
					Result.append (item (i).record_set_ith_text)
					Result.append (item (i).record_ith_text)
				else
					Result.append (item (i).record_set_text)
					Result.append (item (i).record_field_text)
				end
				i := i+1
			end
			Result.append ("end%N")
		end

	navigator_class_text: STRING is
		local
			i: INTEGER
		do
			Result := clone (descendant_header_template)
			Result.replace_substring_all ("<type>",declaration_string + "_NAVIGATOR")
			Result.replace_substring_all ("<parent_type>",declaration_string)
			Result.replace_substring_all ("<date>", get_today)
			Result.replace_substring_all ("<user>", current_user.full_login)
			from
				i := 1
			until
				i > count
			loop
				if item (i).is_array then
					Result.append (item (i).navigator_set_ith_text)
					Result.append (item (i).navigator_ith_text)
				else
					Result.append (item (i).navigator_set_text)
					Result.append (item (i).navigator_field_text)
				end
				i := i+1
			end
			Result.append ("end%N")
		end

	set_text: STRING is
		local
			i: INTEGER
		do
			Result :=
				"	set (a_other: " + declaration_string + ") is%N" +
				"		local%N%
				%			i: INTEGER%N%
				%		do%N%
				%			make_all%N%
				%			a_other.make_all%N"
			from
				i := 1
			until
				i > count
			loop
				if item (i).is_array then
					Result.append (item (i).array_set_text)
				else
					Result.append (item (i).field_set_text)
				end
				i := i+1
			end
			Result.append (	"		end%N%N")
		end

	make_all_text: STRING is
		local
			i: INTEGER
		do
			Result :=
				"	make_all is%N" +
				"		do%N" +
				"			if not made then%N"
			from
				i := 1
			until
				i > count
			loop
				Result.append (item (i).make_me_text)
				i := i+1
			end
			Result.append (	"			made := True%N")
			Result.append (	"			end%N")
			Result.append (	"		end%N%N")
		end

	record_set_text: STRING is
		do
			Result :=
				"	set_" + name + " (a_other: " + declaration_string + ") is%N" +
				"		do%N" +
				"			" + name + ".set (a_other)%N" +
				"		end%N%N"
		end

	record_set_ith_text: STRING is
		do
			Result :=
				"	set_ith_" + name + " (a_value: " + declaration_string + ", i: INTEGER) is %N" +
				"		local%N" +
				"			l_record: " + declaration_string + "_RECORD%N" +
				"		do%N" +
				"			l_record := " + name + ".item (i)" +
				"			if l_record = Void then " +
				"				create l_record%N" +
				"				" + name + ".put (l_record,i)%N" +
				"			end%N" +
				"			l_record.set (a_value.item (i))" +
				"		end%N%N"
		end

	record_ith_text: STRING is
		do
			Result :=
				"	ith_" + name + " (i: INTEGER): " + declaration_string + "_RECORD is%N" +
				"		do%N" +
				"			Result := " + name + ".item (i)%N" +
				"		end%N%N"
		end

	record_field_text: STRING is
		-- code generation for attribute record declaration
		do
			Result := "	" + name + ": " + declaration_string + "_RECORD%N"
		end

	navigator_set_text: STRING is
		-- see `record_set_text'
		do
			Result := record_set_text
		end

	navigator_set_ith_text: STRING is
			-- code generation for navigator attribute item assignment
		do
			Result :=
				"	set_ith_" + name + " (a_value: " + declaration_string + ", i: INTEGER) is%N" +
				"		local%N" +
				"			l_navigator: " + declaration_string + "_NAVIGATOR%N" +
				"		do%N" +
				"			l_navigator := " + name + ".item (i)" +
				"			if l_navigator = Void then " +
				"				create l_navigator%N" +
				"				l_navigator.initialize_ (Current," + offset_out + "," + length_out + "," + min_index_out + "," + max_index_out + ")%N" +
				"				" + name + ".put (l_navigator,i)%N" +
				"			end%N" +
				"			l_navigator.set (a_value.item (i))" +
				"		end%N%N"
		end

	navigator_ith_text: STRING is
			-- code generation for navigator attribute item retrieval
		do
			Result :=
				"	ith_" + name + " (i: INTEGER): " + declaration_string + "_NAVIGATOR is%N" +
				"		do%N" +
				"			Result := " + name + ".item (i)%N" +
				"		end%N%N"
		end

	navigator_field_text: STRING is
		-- code generation for attribute declaration
		do
			Result := "	" + name + ": " + declaration_string + "%N"
		end

	make_one_text: STRING is
		-- code generation for attribute initialization
		do
			Result :=
				"				if " + name + " = Void then %N" +
				"					create " + name + "%N" +
				"					" + name + ".initialize_ (Current," + offset_out + "," + length_out + "," + min_index_out + "," + max_index_out + ")%N" +
				"				end%N"
		end

	integer_value: INTEGER is
			-- do nothing integer value
		do
		end

	boolean_value: BOOLEAN is
			-- do nothing boolean value
		do
		end

	real_value: REAL is
			-- do nothing real value
		do
		end

	double_value: DOUBLE is
			-- do nothing double value
		do
		end

	character_value: CHARACTER is
			-- do nothing character value
		do
		end

	date_time_value: ROSE_DATE is
			-- do nothing date_time value
		do
		end

	string_value: STRING is
			-- do nothing string value
		do
		end

	correct_mismatch is
			-- mismatch initialization???
		local
			l_mismatch: MISMATCH_INFORMATION
		do
			l_mismatch := mismatch_information
		end

feature {NONE} -- Implementation

	display_value (a_item: G) is
			-- Annoy people by writing on stdout.
		local
			l_displayable:DISPLAYABLE
		do
			if a_item /= Void then
				l_displayable ?= a_item
				if l_displayable /= Void then
					l_displayable.display
				else
					print(a_item)
				end
			else
				print("<Void>")
			end
		end

end
