note
	description: "Data server factory class"
	author: "Zoran Simic"
	last_editor: "$Author: zs $"
	date: "$Date: 2011/06/10 $"
	revision: "$Revision: #17 $"
	path: "$File: //rose/source/kernel/infrastructure/persistence/database_access.e $"

class
	DATABASE_ACCESS

inherit
	ROSE_SHARED_DATABASE_POOL

feature -- Access

	make_from_repository (a_logical: STRING): SQL_DATA_SERVER is
			-- New server pointed to by 'a_logical'
		require
			valid_logical: a_logical /= Void
		do
			Result := shared_server_from_repository (a_logical)
		end

	strict_shared_server_from_repository (a_logical: STRING): attached SQL_DATA_SERVER is
			-- server MUST be non-void
		do
			if attached {SQL_DATA_SERVER} shared_server_from_repository (a_logical ) as l_server then
				Result := l_server
			else
				(create {DATABASE_SERVER_NOT_AVAILABLE_EXCEPTION}.make_with_logical (a_logical)).raise
			end
		ensure
			result_attached: Result /= Void
		end

	shared_server_from_repository (a_logical: STRING): SQL_DATA_SERVER is
			-- Get data-server associated to 'a_logical' via the repository
			-- Returned object is shared amongst other callers of this feature with same logical
		require
			valid_logical: a_logical /= Void
		do
			Result := cached_connections.item (a_logical)
			if Result = Void then
				Result := new_server_from_repository (a_logical)
				cached_connections.put (Result, a_logical)
			elseif not Result.is_still_valid then
				output_timestamped ("Shared connection is no longer valid, reconnecting to $1%N", <<a_logical>>)
				warn ("shared_server_from_repository","Reconnecting to $1%N",<<a_logical>>)
				Result := new_server_from_repository (a_logical)
				cached_connections.force (Result, a_logical)
			end
		end

	new_server_from_repository (a_logical: STRING): SQL_DATA_SERVER is
			-- Get data-server associated to 'a_logical' via the repository
			-- Returned object is guaranteed to be a new connection object (not shared with anything)
		require
			valid_logical: a_logical /= Void
		do
			Result ?= pool.get_server_from_logical (a_logical)
		end

	make_from_repository_fast_load (a_logical: STRING): SQLOLEDB_DATA_SERVER is
			-- A fast load server created from 'a_logical'
		require
			valid_logical: a_logical /= Void
		local
			l_server: DATA_SERVER
		do
			l_server := pool.get_fast_load_server_from_logical (a_logical)
			if l_server /= Void then
				Result ?= l_server
				if Result = Void then
					error.abort (generator, "Data server of the wrong type, expected SQLOLEDB_DATA_SERVER but got $1", <<l_server.generating_type>>)
				end
			end
		end

	new_sqlsrv (a_logical: STRING): DATA_SERVER is
			-- New SQLSRV_DATA_SERVER
		require
			valid_logical: a_logical /= Void
		do
			Result := pool.get_server_from_logical (a_logical)
		end

	repository: SQL_DATA_SERVER is
			-- The repository server
		once
			Result ?= pool.get_repository_server
		end

feature -- Implementation

	cached_connections: HASH_TABLE [SQL_DATA_SERVER, STRING] is
			-- Cache for SQL server objects,
			-- so that client can share one connection.
		once
			create Result.make (0)
		end

end
