note

	description:

		"Database servers accessor"

	keywords: "database servers"
	author: "Mark Howard"
	last_editor: "$Author: an $"
	date: "$Date: 2015/03/16 $"
	revision: "$Revision: #310 $"
	path: "$File: //rose/source/kernel/infrastructure/persistence/database_servers.e $"

class DATABASE_SERVERS

inherit
	DATABASE_ACCESS
	GLOBAL_MODEL_AWARE

feature -- Access

	suppliers: ROSE_LINEAR_ARRAY [STRING]
			-- List of all supported suppliers, sorted alphabetically
		local
			l_suppliers: STRING_PREDICATE
		once
			create l_suppliers.make (rose_entity_foreground_server, "SELECT name FROM supplier_product")
			l_suppliers.retrieve (Void, Void)
			Result := l_suppliers.array
			Result.extend ("RESEARCH")
			Result.sort_by (agent compare_strings)
		ensure
			not_void: Result /= Void
		end

	is_valid_repository_logical (a_logical : STRING) : BOOLEAN
			-- a_logical defined in the pool?
		do
			Result := pool.definition (a_logical) /= Void
		end

	master_for_server (a_server_name: STRING): SQLOLEDB_DATA_SERVER
			-- A connection to the database 'master' on a particular SQL/Server.
		require
			a_server_name_not_void: a_server_name /= Void
		local
			l_server_name: STRING
			l_reconnect: BOOLEAN
		do
			l_server_name := a_server_name.as_upper
			master_cache.search (l_server_name)
			if master_cache.found and then master_cache.found_item /= Void then
				if master_cache.found_item.connected_by_testing then
					Result := master_cache.found_item
				else
					output (Global.interpreted ("($now) Connection to 'master' on $1 has been lost%N"), <<a_server_name>>)
					l_reconnect := True
				end
			end
			if Result = Void then
				create Result.make_integrated_security (l_server_name, "master")
				if Result.make_succeeded then
					Result.set_implicit_transaction_off
					master_cache.force (Result, l_server_name)
					if l_reconnect then
						output (Global.interpreted ("($now) Connection to 'master' on $1 has been restored%N"), <<a_server_name>>)
					end
				else
					Result := Void
					if l_reconnect then
						output (Global.interpreted ("($now) Connection to 'master' on $1 could not be restored%N"), <<a_server_name>>)
					end
				end
			end
		ensure
			result_can_be_void: True
		end

feature -- Servers

	common_server_fast_load: SQL_DATA_SERVER
			-- Server with logical name "common", in fast-load mode
		once
			Result := make_from_repository_fast_load ("common")
		end

	toyo_keizai_server: SQL_DATA_SERVER
			-- server connection to the toyo_keizai database
		once
			Result := shared_server_from_repository ("toyo_keizai")
		end

	data_workbench_server: SQL_DATA_SERVER
			-- server connection to the data_workbench database
		once
			Result := shared_server_from_repository ("data_workbench")
		end

	data_warehouse_server: SQL_DATA_SERVER
			-- server connection to the data_warehouse database
		once
			Result := shared_server_from_repository ("data_warehouse")
		end

	legacy_broker_server: SQL_DATA_SERVER
			-- Connection to legacy broker server.
		once
			Result := shared_server_from_repository ("legacy_broker_j")
		end

	country_market_returns_server: SQL_DATA_SERVER
			-- server connection to the country_market_returns database
		once
			Result := shared_server_from_repository ("country_market_returns")
		end

	user_parameters_server: SQL_DATA_SERVER
			-- server connection to the user_parameters database
		once
			Result := shared_server_from_repository ("user_parameters")
		end

	transitions_server: SQL_DATA_SERVER
			-- server connection to the transitions database
		once
			Result := shared_server_from_repository ("transitions")
		end

	reports_server: SQL_DATA_SERVER
			-- server connection to the reports database
		once
			Result := shared_server_from_repository ("reports")
		end

	rose_override_server: SQL_DATA_SERVER
		once
			Result := shared_server_from_repository ("rose_override")
		end

	rbc_server: SQL_DATA_SERVER
			-- standard rbc_seerver, points to the one used for the u-zone, has variable descriptions
		once
			Result := shared_server_from_repository ("rose_best_consensus")
		end

	rbc_j_server: SQL_DATA_SERVER
			-- j-zone daily/weekly rbc data output server
		once
			Result := shared_server_from_repository ("rose_best_consensus_j")
		end

	rbc_o_server: SQL_DATA_SERVER
			-- O-zone daily/weekly rbc data output server
		once
			Result := shared_server_from_repository ("rose_best_consensus_o")
		end

	rbc_u_server: SQL_DATA_SERVER
			-- u-zone daily/weekly rbc data output server
		once
			Result := shared_server_from_repository ("rose_best_consensus")
		end

	rbc_c_server: SQL_DATA_SERVER
			-- C-zone daily/weekly rbc data output server
		once
			Result := shared_server_from_repository ("rose_best_consensus_c")
		end

	rbc_e_server: SQL_DATA_SERVER
			-- E-zone daily/weekly rbc data output server
		once
			Result := shared_server_from_repository ("rose_best_consensus_e")
		end

	rbc_s_server: SQL_DATA_SERVER
			-- S-zone daily/weekly rbc data output server
		once
			Result := shared_server_from_repository ("rose_best_consensus_s")
		end


	rose_fundamental_server: SQL_DATA_SERVER
			-- rose fundamental server
		once
			Result := shared_server_from_repository ("rose_fundamental")
		end

	rose_capital_action_U_server: SQL_DATA_SERVER
			-- Capital action database server for U-zone
		once
			Result := shared_server_from_repository ("rose_capital_action_U")
		end

	rose_capital_action_C_server: SQL_DATA_SERVER
			-- Capital action database server for C-zone
		once
			Result := shared_server_from_repository ("rose_capital_action_C")
		end

	rose_capital_action_J_server: SQL_DATA_SERVER
			-- database server for the J-zone capital actions
		once
			Result := shared_server_from_repository ("rose_capital_action_J")
		end

	rose_capital_action_E_server: SQL_DATA_SERVER
			-- database server for the E-zone capital actions
		once
			Result := shared_server_from_repository ("rose_capital_action_E")
		end


	rose_capital_action_O_server: SQL_DATA_SERVER
			-- database server for the O-zone capital actions
		once
			Result := shared_server_from_repository ("rose_capital_action_O")
		end


	rose_capital_action_S_server: SQL_DATA_SERVER
			-- database server for the S-zone capital actions
		once
			Result := shared_server_from_repository ("rose_capital_action_S")
		end

	rose_zonal_capact_server (a_zone: CHARACTER): SQL_DATA_SERVER
			-- zonal capital action servers
		do
			Result := rose_zonal_capact_server_with_style(a_zone, 'N')
		end

	new_rose_zonal_capact_server (a_zone: CHARACTER): SQL_DATA_SERVER
			-- zonal capital action servers (returns the existing connection if there one)
		do
			Result := rose_zonal_capact_server_with_style(a_zone, 'O')
		end

	rose_zonal_capact_server_with_style(a_zone: CHARACTER; a_style: CHARACTER): SQL_DATA_SERVER
			-- database server for zonal capital actions, with style denoting whether we want an existing one, or a new connection
		local
			l_zone : CHARACTER
		do
			l_zone := a_zone.as_upper
			if a_style='N' then
				inspect l_zone
				when 'U' then Result := rose_capital_action_U_server
				when 'C' then Result := rose_capital_action_C_server
				when 'J' then Result := rose_capital_action_J_server
				when 'E' then Result := rose_capital_action_E_server
				when 'O' then Result := rose_capital_action_O_server
				when 'S' then Result := rose_capital_action_S_server
				end
			else
				inspect l_zone
				when 'U' then Result := new_server_from_repository ("rose_capital_action_U")
				when 'C' then Result := new_server_from_repository ("rose_capital_action_C")
				when 'J' then Result := new_server_from_repository ("rose_capital_action_J")
				when 'E' then Result := new_server_from_repository ("rose_capital_action_E")
				when 'O' then Result := new_server_from_repository ("rose_capital_action_O")
				when 'S' then Result := new_server_from_repository ("rose_capital_action_S")
				end
			end
		end

	rose_capital_action_common_server: SQL_DATA_SERVER
			-- database server capital action alerts and other common tables shared by all zones
		once
			Result := shared_server_from_repository ("rose_capital_action_common")
		end

	rose_issue_data_server: SQL_DATA_SERVER
			-- rose best output database forsue integration
		once
			Result := shared_server_from_repository ("rose_issue_data")
		end

	issue_data_server (a_zone:CHARACTER):SQL_DATA_SERVER
			-- retrieve zonal servers
		local
			l_zone : CHARACTER
		do
			l_zone := a_zone.as_upper
			inspect l_zone
			when 'U' then Result := issue_data_server_u
			when 'C' then Result := issue_data_server_c
			when 'J' then Result := issue_data_server_j
			when 'E' then Result := issue_data_server_e
			when 'O' then Result := issue_data_server_o
			when 'S' then Result := issue_data_server_s
			end
		end

	issue_data_server_u: SQL_DATA_SERVER
			-- u zone output database forsue integration
		once
			Result := shared_server_from_repository ("issue_data_u")
		end

	issue_data_server_c: SQL_DATA_SERVER
			-- c zone output database forsue integration
		once
			Result := shared_server_from_repository ("issue_data_c")
		end

	issue_data_server_j: SQL_DATA_SERVER
			-- j zone output database forsue integration
		once
			Result := shared_server_from_repository ("issue_data_j")
		end

	issue_data_server_e: SQL_DATA_SERVER
			-- e zone output database forsue integration
		once
			Result := shared_server_from_repository ("issue_data_e")
		end

	issue_data_server_o: SQL_DATA_SERVER
			-- o zone output database forsue integration
		once
			Result := shared_server_from_repository ("issue_data_o")
		end

	issue_data_server_s: SQL_DATA_SERVER
			-- s zone output database forsue integration
		once
			Result := shared_server_from_repository ("issue_data_s")
		end

	rose_universe_foreground_server: SQL_DATA_SERVER
			-- rose universe foreground server
		once
			Result := shared_server_from_repository ("rose_universe_foreground")
		end

	regression_server: SQL_DATA_SERVER
			-- regression server
		once
			Result := shared_server_from_repository ("regression")
		end

	rose_risk_model_server: SQL_DATA_SERVER
			-- rose risk model server
		once
			Result := shared_server_from_repository ("rose_risk_model")
		end

	rose_segment_server: SQL_DATA_SERVER
			-- rose segment server
		once
			Result := shared_server_from_repository ("rose_segment")
		end

	situations_server: SQL_DATA_SERVER
			-- should be obsolete, replaced by new data workbench
		once
			Result := shared_server_from_repository ("situations")
		end

	xpressfeed_server_daily: SQL_DATA_SERVER
			-- Xpressfeed server daily
		once
			Result := shared_server_from_repository ("xpressfeed_daily")
		ensure
			has_server: result /= Void
		end

	xpressfeed_server: SQL_DATA_SERVER
			-- Xpressfeed server
		once
			Result := shared_server_from_repository ("xpressfeed")
		end

	capitaliq_server: SQL_DATA_SERVER
			-- Capitaliq server
		once
			Result := shared_server_from_repository ("capitaliq")
		end

	northern_trust_server: SQL_DATA_SERVER
			-- Northern_trust server
		once
			Result := shared_server_from_repository ("northern_trust")
		end

	xpressfeed_native_server: SQL_DATA_SERVER
			-- Xpressfeed database maintained in Xpressfeed in-house formate
		once
			Result := shared_server_from_repository ("xpressfeed_native")
		end

	standard_and_poors_integration_server: SQL_DATA_SERVER
			-- S&P integration server
		once
			Result := shared_server_from_repository ("standard_and_poors_integration")
		end

	valueline_server: SQL_DATA_SERVER
			-- valueline server
		once
			Result := shared_server_from_repository ("valueline")
		end

	global_vantage_server: SQL_DATA_SERVER
			-- global vantage server
		once
			Result := shared_server_from_repository ("global_vantage")
		end

	compustat_canada_server: SQL_DATA_SERVER
			-- compustat candata server
		once
			Result := shared_server_from_repository ("compustat_canada")
		end

	compustat_segment_server: SQL_DATA_SERVER
			-- Compustat segment data prior to migration. This db NOT signature-tagged.
		once
			Result := shared_server_from_repository ("compustat_segment_production")
		end

	compustat_segment_server_tagged: SQL_DATA_SERVER
	 		-- new format Compustat segment data after migration. This db signature-tagged.
		once
			Result := shared_server_from_repository ("compustat_segment_tagged")
		end

	compustat_segment_legacy_server, compustat_segment_legacy_server_tagged: SQL_DATA_SERVER
	 		-- old format Compustat segment data after migration. This db signature-tagged.
		once
			Result := shared_server_from_repository ("compustat segment legacy tagged")
 		end

	compustat_segment_server_v2: SQL_DATA_SERVER
			-- old format compustat segment data server.
		once
			Result := shared_server_from_repository ("compustat_segment_v2")
		end

	compustat_server: SQL_DATA_SERVER
		once
			Result := shared_server_from_repository ("compustat")
		end

	compustat_fg_server: SQL_DATA_SERVER
		once
			Result := shared_server_from_repository ("compustat_fg")
		end

	compustat_fg_fast_load_server: SQLOLEDB_DATA_SERVER
		once
			Result := make_from_repository_fast_load ("compustat_fg")
		end

	sdc_server: SQL_DATA_SERVER
		once
			Result := shared_server_from_repository ("SDC")
		end

	process_server: SQL_DATA_SERVER
		once
			Result := shared_server_from_repository ("process")
		end

	process_metrics_server: SQL_DATA_SERVER
			-- Server connection to the DB where we memorize all our metrics on processes
		once
			Result := shared_server_from_repository ("process_metrics")
		end

	integration_w_server: SQL_DATA_SERVER
		once
			Result := shared_server_from_repository ("integration_w")
		end

	ibes_server: SQL_DATA_SERVER
		once
			Result := shared_server_from_repository ("ibes")
		end

	infoglobe_server: SQL_DATA_SERVER
		once
			Result := shared_server_from_repository ("infoglobe")
		end

	reuters_server: SQL_DATA_SERVER
		once
			if set_reuters_server(Void)=Void
			 then
				Result := shared_server_from_repository ("reuters")
			else
				Result := set_reuters_server(Void)
			end
		end

	set_reuters_server(a_server: SQL_DATA_SERVER): SQL_DATA_SERVER
		once
			Result := a_server
		end

	worldscope_raw_server: SQL_DATA_SERVER
			-- worldscope raw server
		once
			Result := shared_server_from_repository ("worldscope")
		end

	worldscope_server: SQL_DATA_SERVER
			-- worldscope server
		once
			if set_worldscope_server(Void)=Void
			 then
				Result := shared_server_from_repository ("worldscope_new")
			else
				Result := set_worldscope_server(Void)
			end
		end

	new_worldscope_server: SQL_DATA_SERVER
			-- worldscope new server
		once
			Result := new_server_from_repository ("worldscope_new")
		end

	worldscope_vms_fg_server: SQL_DATA_SERVER
		once
			Result := shared_server_from_repository ("worldscope_vms_fg")
		end

	set_worldscope_server(a_server: SQL_DATA_SERVER): SQL_DATA_SERVER
		once
			Result := a_server
		end

	thomson_deals_server: SQL_DATA_SERVER
			-- thomson deals server.
		once
			Result := shared_server_from_repository ("thomson_deals")
		end

	legacy_extel_business_segments: SQLOLEDB_DATA_SERVER
		once
			if set_legacy_extel_business_segments_server(Void)=Void then
				Result ?= shared_server_from_repository ("legacy_extel_business_segments")
			else
				Result ?= set_legacy_extel_business_segments_server(Void)
			end
		end

	set_legacy_extel_business_segments_server(a_server: SQL_DATA_SERVER): SQL_DATA_SERVER
		once
			Result := a_server
		end

	ibes_variables_server: SQL_DATA_SERVER
			-- IBES variables intermediary server
		once
			Result := shared_server_from_repository ("ibes_variables")
		end

	dataline_server: SQL_DATA_SERVER
			-- Dataline server.
		once
			Result := shared_server_from_repository ("dataline")
		end

	dataline_price_server: SQL_DATA_SERVER
			-- Dataline price server.
		once
			Result := shared_server_from_repository ("dataline_price")
		end

	dataline_capital_actions_server: SQL_DATA_SERVER
			-- Dataline capital actions server.
		once
			Result := shared_server_from_repository("dataline_capital_actions")
		end

	dataline_capital_actions_U_server: SQL_DATA_SERVER
		once
			Result := shared_server_from_repository("dataline_capital_actions_U")
		end
	dataline_capital_actions_C_server: SQL_DATA_SERVER
		once
			Result := shared_server_from_repository("dataline_capital_actions_C")
		end
	dataline_capital_actions_J_server: SQL_DATA_SERVER
		once
			Result := shared_server_from_repository("dataline_capital_actions_J")
		end
	dataline_capital_actions_E_server: SQL_DATA_SERVER
		once
			Result := shared_server_from_repository("dataline_capital_actions_E")
		end
	dataline_capital_actions_O_server: SQL_DATA_SERVER
		once
			Result := shared_server_from_repository("dataline_capital_actions_O")
		end
	dataline_capital_actions_S_server: SQL_DATA_SERVER
		once
			Result := shared_server_from_repository("dataline_capital_actions_S")
		end

	spectrum_server: SQL_DATA_SERVER
		once
			Result := shared_server_from_repository ("spectrum")
		end

	brstearns_short_server: SQL_DATA_SERVER
		once
			Result := shared_server_from_repository ("brstearns_short")
		end

	zacks_server: SQL_DATA_SERVER
		once
			Result := shared_server_from_repository ("zacks_new")
		end

	zacks_new_server: SQL_DATA_SERVER
		once
			Result := shared_server_from_repository ("zacks_new")
		end

	issue_override_server: SQL_DATA_SERVER
			--sue overrride server.
		once
			Result := shared_server_from_repository ("issue_overrides")
		end

	financial_calendar_server: SQL_DATA_SERVER
			-- server for financial_calendar supplier.
		once
			Result := shared_server_from_repository ("financial_calendar")
		end

feature -- Access

	return_server(a_zone : CHARACTER) : SQL_DATA_SERVER
			-- Zone specific return server
		local
			l_zone : CHARACTER
		do
			l_zone := a_zone.as_upper
			inspect l_zone
			when 'U' then Result := return_u_server
			when 'C' then Result := return_c_server
			when 'E' then Result := return_e_server
			when 'O' then Result := return_o_server
			when 'S' then Result := return_s_server
			when 'J' then Result := return_j_server
			else print("Invalid zone: " + a_zone.out + "%N")
			end
		end

	return_j_server: SQL_DATA_SERVER
			-- Returns server for J zone
		once
			Result := shared_server_from_repository( "return_j" )
		end

	return_u_server: SQL_DATA_SERVER
		once
			Result := shared_server_from_repository( "return_u" )
		end

	return_e_server: SQL_DATA_SERVER
		once
			Result := shared_server_from_repository( "return_e" )
		end

	return_o_server: SQL_DATA_SERVER
		once
			Result := shared_server_from_repository( "return_o" )
		end

	return_s_server: SQL_DATA_SERVER
		once
			Result := shared_server_from_repository( "return_s" )
		end

	return_c_server: SQL_DATA_SERVER
		once
			Result := shared_server_from_repository( "return_c" )
		end

feature -- Access

	nikkei_server: SQL_DATA_SERVER
		once
			Result := shared_server_from_repository ("nikkei")
		end

	nikkei_new_server: SQL_DATA_SERVER
		once
			Result := shared_server_from_repository ("nikkei_new")
		end

	rc_projects_server: SQL_DATA_SERVER
		once
			Result := shared_server_from_repository ("rc_projects")
		end

	nomura_server: SQL_DATA_SERVER
		once
			Result := shared_server_from_repository ("nomura")
		end

	crsp_server: SQL_DATA_SERVER
		once
			Result := shared_server_from_repository ("crsp")
		end

	idsi_daily_server:SQL_DATA_SERVER
		once
			Result := shared_server_from_repository ("idsi_daily")
		end

	idsi_weekly_server:SQL_DATA_SERVER
		once
			Result := shared_server_from_repository ("idsi_weekly")
		end

	idsi_weekly_canada_server:SQL_DATA_SERVER
		once
			Result := shared_server_from_repository ("idsi_weekly_canada")
		end

	idc_futures_server: SQL_DATA_SERVER
		once
			Result := shared_server_from_repository ("idc_futures")
		end

	idc_server: SQL_DATA_SERVER
		once
			Result := shared_server_from_repository ("idc")
		end

	brim_entities_server: SQL_DATA_SERVER
		once
			Result := shared_server_from_repository ("brim_entities")
		end

	rc_development_server: SQL_DATA_SERVER
		once
			Result := shared_server_from_repository ("rc_development")
		end

	brim_capital_actions_server: SQL_DATA_SERVER
			-- supplier database of capital actions generated from legacy vms backdiv and dist_w, source for capitla action integration
		once
			Result := shared_server_from_repository ("brim_capital_actions")
		end

	adr_server: SQL_DATA_SERVER
		once
			Result := shared_server_from_repository ("adr")
		end

	bloomberg_bond_server: SQL_DATA_SERVER
		once
			Result := shared_server_from_repository ("bloomberg_bond")
		end

	bloomberg_currency_server: SQL_DATA_SERVER
			-- server for access of bloombrg currency database
		once
			Result := shared_server_from_repository ("bloomberg_currency")
		end

	bloomberg_equity_server: SQL_DATA_SERVER
			-- server for accessing the bloomberg equity database
		once
			Result := shared_server_from_repository ("bloomberg_equity")
		end

	tej_server: SQL_DATA_SERVER
		once
			Result := shared_server_from_repository ("tej")
		end

	barra_server: SQL_DATA_SERVER
		once
			Result := shared_server_from_repository ("barra")
		end

	primark_server: SQL_DATA_SERVER
		once
			Result := shared_server_from_repository ("primark")
		end

	multex_server: SQL_DATA_SERVER
		once
			Result := shared_server_from_repository ("multex")
		end

	rose_entity_foreground_server: SQL_DATA_SERVER
		once
			Result := shared_server_from_repository ("rose_entity_foreground")
		end

	new_rose_entity_foreground_server: SQL_DATA_SERVER
		do
			Result := new_server_from_repository ("rose_entity_foreground")
		end

	rose_entity_background_server: SQL_DATA_SERVER
		once
			Result := shared_server_from_repository ("rose_entity_background")
		end

	rose_entity_archive_server: SQL_DATA_SERVER
		once
			Result := shared_server_from_repository ("rose_entity_archive")
		end

	supplier_product_server: SQL_DATA_SERVER
		once
			Result := shared_server_from_repository ("supplier_product")
		end

	tasks_server: SQL_DATA_SERVER
			-- Database access to TASK data
		once
			Result := shared_server_from_repository ("tasks")
		end

	process_flows_server: SQL_DATA_SERVER
			-- server connection to the database where process flows information stored
		once
			Result := shared_server_from_repository ("process_flows")
		end

	survey_server: SQL_DATA_SERVER
		once
			Result := shared_server_from_repository ("survey")
		end

	source_monitor_server: SQL_DATA_SERVER
			-- Database access to source monitor data
		once
			Result := shared_server_from_repository ("source_monitor")
		end

	docm_server: SQL_DATA_SERVER
			-- Database access to DevOps Configuration Management (DOCM) data
		once
			Result := shared_server_from_repository ("docm")
		end

	code_collaborator_server: SQL_DATA_SERVER
			-- Database access to Code Collaborator data
		once
			Result := shared_server_from_repository ("code_collaborator")
		end

	forex_foreground_server: SQL_DATA_SERVER
		once
			Result := shared_server_from_repository ("forex_foreground")
		end

	rose_change_cascade_server: SQL_DATA_SERVER
		once
			Result := shared_server_from_repository ("rose_change_cascade")
		end

	portfolio_characteristic_server: SQL_DATA_SERVER
			-- Server for portfolio_characteristics database.
		once
			Result := shared_server_from_repository ("portfolio_characteristics")
		end

	test_server: SQL_DATA_SERVER
			-- Server for Test database.
		once
			Result := shared_server_from_repository ("Test")
		end

	test_fast_load_server: SQLOLEDB_DATA_SERVER
			-- Fast load server for Test database.
		once
			Result := make_from_repository_fast_load ("Test")
		end

	entity_identifiers_server: SQL_DATA_SERVER
			-- Server for entity_identifiers database.
		once
			Result := shared_server_from_repository ("entity_identifiers")
		end

	telekurs_server: SQL_DATA_SERVER
		once
			Result := shared_server_from_repository ("telekurs")
		end

	msci_server: SQL_DATA_SERVER
			-- Server containing MSCI supplier data
		do
			Result := shared_server_from_repository ("msci")
		end

	msci_fast_load_server: SQLOLEDB_DATA_SERVER
			-- Fast load server containing MSCI supplier data
		do
			Result := make_from_repository_fast_load ("msci")
		end

	standard_and_poors_server: SQL_DATA_SERVER
		once
			Result := shared_server_from_repository ("standard_and_poors")
		end

	standard_and_poors_fast_load_server: SQLOLEDB_DATA_SERVER
		once
			Result := make_from_repository_fast_load ("standard_and_poors")
		end

	dow_jones_server: SQL_DATA_SERVER
		once
			Result := shared_server_from_repository ("dow_jones")
		end

	dow_jones_fast_load_server: SQLOLEDB_DATA_SERVER
		once
			Result := make_from_repository_fast_load ("dow_jones")
		end

	hsbc_server: SQL_DATA_SERVER
		once
			Result := shared_server_from_repository ("hsbc")
		end

	hsbc_fast_load_server: SQLOLEDB_DATA_SERVER
		once
			Result := make_from_repository_fast_load ("hsbc")
		end

	swx_server: SQL_DATA_SERVER
		once
			Result := shared_server_from_repository ("swx")
		end

	swx_fast_load_server: SQLOLEDB_DATA_SERVER
		once
			Result := make_from_repository_fast_load ("swx")
		end

	tsx_server: SQL_DATA_SERVER
		once
			Result := shared_server_from_repository ("tsx")
		end

	tsx_fast_load_server: SQLOLEDB_DATA_SERVER
		once
			Result := make_from_repository_fast_load ("tsx")
		end

	wilshire_server: SQL_DATA_SERVER
		once
			Result := shared_server_from_repository ("wilshire")
		end

	wilshire_fast_load_server: SQLOLEDB_DATA_SERVER
		once
			Result := make_from_repository_fast_load ("wilshire")
		end

	stoxx_server: SQL_DATA_SERVER
			-- STOXX equity index vendor
		once
			Result := shared_server_from_repository ("stoxx")
		end

	russell_server: SQL_DATA_SERVER
		once
			Result := shared_server_from_repository ("russell")
		end

	russell_fast_load_server: SQLOLEDB_DATA_SERVER
		once
			Result := make_from_repository_fast_load ("russell")
		end

	citibank_server: SQL_DATA_SERVER
		once
			Result := shared_server_from_repository ("citibank")
		end

	topix_server: SQL_DATA_SERVER
		once
			Result := shared_server_from_repository ("topix")
		end

	topix_fast_load_server: SQLOLEDB_DATA_SERVER
		once
			Result := make_from_repository_fast_load ("topix")
		end

	hfr_server: SQL_DATA_SERVER
			-- Data server for the HFR indexes context
		once
			Result := shared_server_from_repository ("hfr")
		end

	hfr_fast_load_server: SQLOLEDB_DATA_SERVER
			-- Fast load ata server for the HFR indexes context
		once
			Result := make_from_repository_fast_load ("hfr")
		end

	cri_server: SQL_DATA_SERVER
		once
			Result := shared_server_from_repository ("cri")
		end

	cri_fast_load_server: SQLOLEDB_DATA_SERVER
		once
			Result := make_from_repository_fast_load ("cri")
		end

	ftse_server: SQL_DATA_SERVER
		do
			Result := shared_server_from_repository ("ftse")
		end

	economatica_server: SQL_DATA_SERVER
		once
			Result := shared_server_from_repository ("economatica")
		end

	starmine_server: SQL_DATA_SERVER
		once
			Result := shared_server_from_repository ("starmine")
		end

	revere_server: SQL_DATA_SERVER
			-- server for Revere supplier
		once
			Result := shared_server_from_repository ("revere")
		end

	signature_tagged_statistics_server: SQL_DATA_SERVER
		once
			Result := shared_server_from_repository ("signature_tagged_statistics")
		end

	idsi_server: SQL_DATA_SERVER
		once
			Result := shared_server_from_repository ("idsi")
		end

	idsi_canada_server: SQL_DATA_SERVER
		once
			Result := shared_server_from_repository ("idsi_canada")
		end

	ftse_fast_load_server: SQLOLEDB_DATA_SERVER
			-- Fast load server for FTSE
		do
			Result := make_from_repository_fast_load ("ftse")
		end

	index_simulation_server: SQL_DATA_SERVER
		once
			Result := shared_server_from_repository ("index_simulation")
		end

	bogie_constituents_server: SQL_DATA_SERVER
		once
			Result := shared_server_from_repository ("bogie_constituents")
		end

	ntperf_server: SQL_DATA_SERVER
			-- NT Performance data
		once
			Result := shared_server_from_repository ("ntperf")
		end

	performance_server: SQL_DATA_SERVER
			-- "performance" server
		once
			Result := shared_server_from_repository ("performance")
		end

	performance_server_fast_load: SQL_DATA_SERVER
			-- "performance" server, in fast load mode
		once
			Result := make_from_repository_fast_load ("performance")
		end

	external_performance_server: SQL_DATA_SERVER
		once
			Result := shared_server_from_repository ("external_performance")
		end

	official_performance_server: SQL_DATA_SERVER
			-- Once access to Nperf data server.
		once
			Result := shared_server_from_repository ("Nperf")
		end

	global_client_fg_server: SQL_DATA_SERVER
			-- Once access to global_client_fg data database.
		once
			Result :=shared_server_from_repository ("global_client_fg")
		end

	market_data_server: SQL_DATA_SERVER
		once
			Result := shared_server_from_repository ("market_data")
			-- We will require writers to start their own transactions.
			Result.set_implicit_transaction_off
		end

	cusip_bureau_server: SQL_DATA_SERVER
		once
			Result := shared_server_from_repository ("cusip_bureau")
		end

	market_data_fast_load_server: SQLOLEDB_DATA_SERVER
		once
			Result := make_from_repository_fast_load ("market_data")
		end

	foreign_exchange_server: SQL_DATA_SERVER
		once
			Result := shared_server_from_repository ("foreign_exchange")
		end

	rose_foreign_exchange_server: SQL_DATA_SERVER
		once
			Result := shared_server_from_repository ("rose_foreign_exchange")
		end

	simulation_server : SQL_DATA_SERVER
		once
			Result := shared_server_from_repository ("simulation")
		end

	gii_server: SQL_DATA_SERVER
		once
			Result := shared_server_from_repository ("gii")
		end

	hdp_server: DATA_SERVER
		once
			Result := new_sqlsrv ("vms_hdp")
		end

	storable_path_server: DATA_SERVER
		once
			Result := new_sqlsrv ("vms_sorable_path")
		end

	backdiv_server: DATA_SERVER
			-- should not write to backdiv, only legacy update writes
		once
			Result  := new_sqlsrv ("vms_backdiv")
			Result.read_only
		end

	pacap_server: SQL_DATA_SERVER
		once
			Result := shared_server_from_repository ("pacap")
		end

	repository_server: SQL_DATA_SERVER
		once
			Result := shared_server_from_repository ("repository")
		end

	sdn_server: SQL_DATA_SERVER
			-- Sdn server
		once
			Result := shared_server_from_repository ("sdn")
		end

	supplier_data_server (a_supplier_product_id: INTEGER): SQL_DATA_SERVER
			-- Supplier server by supplier product ID
		require
			a_supplier_product_id_not_zero: a_supplier_product_id /= 0
		do
			if a_supplier_product_id = integration_w_supplier_product_id then
				Result := integration_w_server
			elseif a_supplier_product_id = reuters_supplier_product_id then
				Result := reuters_server
			elseif a_supplier_product_id = compustat_canada_supplier_product_id then
				Result := compustat_canada_server
			elseif a_supplier_product_id = compustat_segment_supplier_product_id then
				Result := compustat_segment_server	-- Compustat segment data prior to migration. This db NOT signature-tagged.
			elseif a_supplier_product_id = compustat_segment_tagged_supplier_product_id then
				Result := compustat_segment_server_tagged	-- new format Compustat segment data after migration
			elseif a_supplier_product_id = compustat_segment_legacy_tagged_supplier_product_id then
				Result := compustat_segment_legacy_server_tagged	-- old format Compustat segment data after migration
			elseif a_supplier_product_id = compustat_supplier_product_id then
				Result := compustat_server
			elseif a_supplier_product_id = rose_override_supplier_product_id then
				Result := rose_override_server
			elseif a_supplier_product_id = ibes_supplier_product_id then
				Result := ibes_server
			elseif a_supplier_product_id = primark_supplier_product_id then
				Result := primark_server
			elseif a_supplier_product_id = multex_supplier_product_id then
				Result := multex_server
			elseif a_supplier_product_id = telekurs_supplier_product_id then
				Result := telekurs_server
			elseif a_supplier_product_id = rose_entity_background_supplier_product_id then
				Result := rose_entity_background_server
			elseif a_supplier_product_id = rose_entity_archive_supplier_product_id then
				Result := rose_entity_archive_server
			elseif a_supplier_product_id = nikkei_supplier_product_id then
				Result := nikkei_new_server
			elseif a_supplier_product_id = global_vantage_supplier_product_id then
				Result := global_vantage_server
			elseif a_supplier_product_id = rc_projects_supplier_product_id then
				Result := rc_projects_server
			elseif a_supplier_product_id = nomura_supplier_product_id then
				Result := nomura_server
			elseif a_supplier_product_id = infoglobe_supplier_product_id then
				Result := infoglobe_server
			elseif a_supplier_product_id = worldscope_data_supplier_product_id then
				Result := worldscope_server
			elseif a_supplier_product_id = worldscope_supplier_product_id then
				Result := worldscope_server
			elseif a_supplier_product_id = toyo_keizai_supplier_product_id then
				Result := toyo_keizai_server
			elseif a_supplier_product_id = dataline_supplier_product_id then
				Result := dataline_server
			elseif a_supplier_product_id = adr_supplier_product_id then
				Result := adr_server
			elseif a_supplier_product_id = bloomberg_bond_supplier_product_id then
				Result := bloomberg_bond_server
			elseif a_supplier_product_id = bloomberg_equity_supplier_product_id then
				Result := bloomberg_equity_server
			elseif a_supplier_product_id = bloomberg_currency_supplier_product_id then
				Result := bloomberg_currency_server
			elseif a_supplier_product_id = dataline_price_supplier_product_id then
				Result := dataline_price_server
			elseif a_supplier_product_id = spectrum_supplier_product_id then
				Result := spectrum_server
			elseif a_supplier_product_id = zacks_supplier_product_id then
				Result := zacks_server
			elseif a_supplier_product_id = xpressfeed_supplier_product_id then
				Result := xpressfeed_server_daily
			elseif a_supplier_product_id = capitaliq_supplier_product_id then
				Result := capitaliq_server
			elseif a_supplier_product_id = northern_trust_supplier_product_id then
				Result := northern_trust_server
			elseif a_supplier_product_id = crsp_supplier_product_id then
				Result := crsp_server
			elseif a_supplier_product_id = idsi_supplier_product_id then
				Result := idsi_server
			elseif a_supplier_product_id = idsi_canada_supplier_product_id then
				Result := idsi_canada_server
			elseif a_supplier_product_id = idc_futures_supplier_product_id then
				Result := idc_futures_server
			elseif a_supplier_product_id = idc_supplier_product_id then
				Result := idc_server
			elseif a_supplier_product_id = brim_entities_supplier_product_id then
				Result := brim_entities_server
			elseif a_supplier_product_id = sdc_supplier_product_id then
				Result := sdc_server
			elseif a_supplier_product_id = ftse_supplier_product_id then
				Result := ftse_server
			elseif a_supplier_product_id = economatica_supplier_product_id then
				Result := economatica_server
			elseif a_supplier_product_id = starmine_supplier_product_id then
				Result := starmine_server
			elseif a_supplier_product_id = revere_supplier_product_id then
				Result := revere_server
			elseif a_supplier_product_id = thomson_deals_supplier_product_id then
				Result := thomson_deals_server
			elseif a_supplier_product_id = brstearns_short_supplier_product_id then
				Result := brstearns_short_server
			elseif a_supplier_product_id = msci_supplier_product_id then
				Result := msci_server
			elseif a_supplier_product_id = cri_supplier_product_id then
				Result := cri_server
			elseif a_supplier_product_id = barra_supplier_product_id then
				Result := barra_server
			elseif a_supplier_product_id = valueline_supplier_product_id then
				Result := valueline_server
			elseif a_supplier_product_id = tej_supplier_product_id then
				Result := tej_server
			elseif a_supplier_product_id = supplier_product_supplier_product_id then
				Result := supplier_product_server
			elseif a_supplier_product_id = tasks_supplier_product_id then
				Result := tasks_server
			elseif a_supplier_product_id = survey_supplier_product_id then
				Result := survey_server
			elseif a_supplier_product_id = rose_change_cascade_supplier_product_id then
				Result := rose_change_cascade_server
			elseif a_supplier_product_id = portfolio_characteristic_supplier_product_id then
				Result := portfolio_characteristic_server
			elseif a_supplier_product_id = dataline_capital_actions_supplier_product_id then
				Result := dataline_capital_actions_server
			elseif a_supplier_product_id = pacap_supplier_product_id then
				Result := pacap_server
			elseif a_supplier_product_id = legacy_extel_business_segments_supplier_product_id then
				Result := legacy_extel_business_segments
			elseif a_supplier_product_id = process_flows_supplier_product_id then
				Result := process_flows_server
			end
		end

	supplier_product_name (a_supplier_product_id: INTEGER): STRING
		require
			a_supplier_product_id_not_zero: a_supplier_product_id /= 0
			has_supplier_product_id: supplier_product_cache_by_id.has (a_supplier_product_id)
		do
			Result := supplier_product_cache_by_id.item (a_supplier_product_id).name
		end

	quop_instructions_supplier_product_id: INTEGER
		once
			Result := supplier_product_id_by_name ("QUOP_INSTRUCTIONS")
		end

	tasks_supplier_product_id: INTEGER
		once
			Result := supplier_product_id_by_name ("TASKS")
		end

	survey_supplier_product_id: INTEGER
		once
			Result := supplier_product_id_by_name ("SURVEY")
		end

	rose_change_cascade_supplier_product_id: INTEGER
		once
			Result := supplier_product_id_by_name ("ROSE_CHANGE_CASCADE")
		end

	toyo_keizai_supplier_product_id: INTEGER
		once
			Result := supplier_product_id_by_name ("TOYO_KEIZAI")
		end

	sdn_supplier_product_id: INTEGER
			-- Sdn supplier product id
		once
			Result := supplier_product_id_by_name ("SDN")
		end

	integration_w_supplier_product_id: INTEGER
		once
			Result := supplier_product_id_by_name ("INTEGRATION_W")
		end

	compustat_canada_supplier_product_id: INTEGER
		once
			Result := supplier_product_id_by_name ("COMPUSTAT CANADA")
		end

	compustat_segment_supplier_product_id: INTEGER
		once
			Result := supplier_product_id_by_name ("COMPUSTAT SEGMENT")
		end

	compustat_segment_tagged_supplier_product_id: INTEGER
		once
			Result := supplier_product_id_by_name ("COMPUSTAT SEGMENT TAGGED")
		end

	compustat_segment_legacy_tagged_supplier_product_id: INTEGER
		once
			Result := supplier_product_id_by_name ("COMPUSTAT SEGMENT LEGACY TAGGED")
		end

	compustat_supplier_product_id: INTEGER
		once
			Result := supplier_product_id_by_name ("COMPUSTAT")
		end

	legacy_extel_business_segments_supplier_product_id: INTEGER
		once
			Result := supplier_product_id_by_name ("LEGACY_EXTEL_BUSINESS_SEGMENTS")
		end

	rose_override_supplier_product_id: INTEGER
		once
			Result := supplier_product_id_by_name ("ROSE OVERRIDE")
		end

	nikkei_supplier_product_id: INTEGER
		once
			Result := supplier_product_id_by_name ("NIKKEI_NEW")
		end

	rc_projects_supplier_product_id: INTEGER
		once
			Result := supplier_product_id_by_name ("RC_PROJECTS")
		end

	nomura_supplier_product_id: INTEGER
		once
			Result := supplier_product_id_by_name ("NOMURA")
		end

	sdc_supplier_product_id: INTEGER
		once
			Result := supplier_product_id_by_name ("SDC")
		end

	ibes_supplier_product_id: INTEGER
		once
			Result := supplier_product_id_by_name ("IBES")
		end

	infoglobe_supplier_product_id: INTEGER
		once
			Result := supplier_product_id_by_name ("INFOGLOBE")
		end

	worldscope_data_supplier_product_id, worldscope_supplier_product_id: INTEGER
		once
			Result := worldscope_2_supplier_product_id
		end

	worldscope_2_supplier_product_id: INTEGER
		once
			Result := supplier_product_id_by_name ("WORLDSCOPE_2")
		end

	dataline_supplier_product_id: INTEGER
		once
			Result := supplier_product_id_by_name ("DATALINE")
		end

	dataline_price_supplier_product_id: INTEGER
		once
			Result := supplier_product_id_by_name ("DATALINE PRICE")
		end

	spectrum_supplier_product_id: INTEGER
		once
			Result := supplier_product_id_by_name ("SPECTRUM")
		end

	brstearns_short_supplier_product_id: INTEGER
		once
			Result := supplier_product_id_by_name ("BRSTEARNS_SHORT")
		end

	zacks_supplier_product_id: INTEGER
		once
			Result := supplier_product_id_by_name ("ZACKS")
		end

	crsp_supplier_product_id: INTEGER
		once
			Result := supplier_product_id_by_name ("CRSP")
		end

	idsi_daily_supplier_product_id: INTEGER
		once
			Result := supplier_product_id_by_name ("IDSI DAILY")
		end
	idsi_supplier_product_id: INTEGER
		once
			Result := supplier_product_id_by_name ("IDSI")
		end

	idsi_canada_supplier_product_id: INTEGER
		once
			Result := supplier_product_id_by_name ("IDSI CANADA")
		end

	idsi_weekly_supplier_product_id: INTEGER
		once
			Result := supplier_product_id_by_name ("IDSI WEEKLY")
		end

	idsi_weekly_canada_supplier_product_id: INTEGER
		once
			Result := supplier_product_id_by_name ("IDSI WEEKLY CANADA")
		end

	idc_futures_supplier_product_id: INTEGER
		once
			Result := supplier_product_id_by_name ("IDC FUTURES")
		end

	idc_supplier_product_id: INTEGER
		once
			Result := supplier_product_id_by_name ("IDC")
		end

	brim_entities_supplier_product_id: INTEGER
		once
			Result := supplier_product_id_by_name ("BRIM ENTITIES")
		end

	adr_supplier_product_id: INTEGER
		once
			Result := supplier_product_id_by_name ("ADR")
		end

	bloomberg_bond_supplier_product_id: INTEGER
		once
			Result := supplier_product_id_by_name ("BLOOMBERG BOND")
		end

	bloomberg_currency_supplier_product_id: INTEGER
		once
			Result := supplier_product_id_by_name ("BLOOMBERG CURRENCY")
		end

	bloomberg_equity_supplier_product_id: INTEGER
		once
			Result := supplier_product_id_by_name ("BLOOMBERG EQUITY")
		end

	msci_supplier_product_id: INTEGER
		once
			Result := supplier_product_id_by_name ("MSCI")
		end

	russell_supplier_product_id: INTEGER
		once
			Result := supplier_product_id_by_name ("RUSSELL")
		end

	citibank_supplier_product_id: INTEGER
		once
			Result := supplier_product_id_by_name ("CITIBANK")
		end

	hfr_supplier_product_id: INTEGER
		once
			Result := supplier_product_id_by_name ("HFR")
		end

	reuters_supplier_product_id: INTEGER
		once
			Result := supplier_product_id_by_name ("REUTERS")
		end

	standard_and_poors_supplier_product_id: INTEGER
		once
			Result := supplier_product_id_by_name ("STANDARD AND POORS")
		end

	dow_jones_supplier_product_id: INTEGER
		once
			Result := supplier_product_id_by_name ("DOW_JONES")
		end

	stoxx_supplier_product_id: INTEGER
		once
			Result := supplier_product_id_by_name ("STOXX")
		end

	hsbc_supplier_product_id: INTEGER
		once
			Result := supplier_product_id_by_name ("HSBC")
		end

	swx_supplier_product_id: INTEGER
		once
			Result := supplier_product_id_by_name ("SWX")
		end

	tsx_supplier_product_id: INTEGER
		once
			Result := supplier_product_id_by_name ("TSX")
		end

	wilshire_supplier_product_id: INTEGER
		once
			Result := supplier_product_id_by_name ("WILSHIRE")
		end

	valueline_supplier_product_id: INTEGER
			-- Valueline supplier product handle.
		once
			Result := supplier_product_id_by_name ("VALUELINE")
		end

	xpressfeed_supplier_product_id: INTEGER
			-- Xpressfeed supplier product handle.
		once
			Result := supplier_product_id_by_name ("XPRESSFEED")
		end

	capitaliq_supplier_product_id: INTEGER
			-- Capitaliq supplier product handle.
		once
			Result := supplier_product_id_by_name ("CAPITALIQ")
		end

	northern_trust_supplier_product_id: INTEGER
			-- Northern_trust supplier product handle.
		once
			Result := supplier_product_id_by_name ("NORTHERN TRUST")
		end

	global_vantage_supplier_product_id: INTEGER
			-- Global_vantage supplier product handle.
		once
			Result := supplier_product_id_by_name ("GLOBAL VANTAGE")
		end

	ftse_supplier_product_id: INTEGER
		once
			Result := supplier_product_id_by_name ("FTSE")
		end

	economatica_supplier_product_id: INTEGER
		once
			Result := supplier_product_id_by_name ("ECONOMATICA")
		end

	starmine_supplier_product_id: INTEGER
		once
			Result := supplier_product_id_by_name ("STARMINE")
		end

	revere_supplier_product_id: INTEGER
		once
			Result := supplier_product_id_by_name ("REVERE")
		end

	thomson_deals_supplier_product_id: INTEGER
		once
			Result := supplier_product_id_by_name ("THOMSON_DEALS")
		end

	cri_supplier_product_id: INTEGER
		once
			Result := supplier_product_id_by_name ("CRI")
		end

	test_supplier_product_id: INTEGER
		once
			Result := supplier_product_id_by_name ("TEST")
		end

	teJ_supplier_product_id: INTEGER
		once
			Result := supplier_product_id_by_name ("TEJ")
		end

	barra_supplier_product_id: INTEGER
		once
			Result := supplier_product_id_by_name ("BARRA")
		end

	primark_supplier_product_id: INTEGER
		once
			Result := supplier_product_id_by_name ("PRIMARK")
		end

	multex_supplier_product_id: INTEGER
		once
			Result := supplier_product_id_by_name ("MULTEX")
		end

	telekurs_supplier_product_id: INTEGER
		once
			Result := supplier_product_id_by_name ("TELEKURS")
		end

	rose_entity_background_supplier_product_id: INTEGER
		once
			Result := supplier_product_id_by_name ("ROSE ENTITY BACKGROUND")
		end

	rose_entity_foreground_supplier_product_id: INTEGER
		once
			Result := supplier_product_id_by_name ("ROSE ENTITY FOREGROUND")
		end

	rose_entity_archive_supplier_product_id: INTEGER
		once
			Result := supplier_product_id_by_name ("ROSE ENTITY ARCHIVE")
		end

	supplier_product_supplier_product_id: INTEGER
		once
			Result := supplier_product_id_by_name ("SUPPLIER_PRODUCT")
		end

	portfolio_characteristic_supplier_product_id: INTEGER
		once
			Result := supplier_product_id_by_name ("PORTFOLIO CHARACTERISTIC")
		end

	dataline_capital_actions_supplier_product_id: INTEGER
		once
			Result := supplier_product_id_by_name ("DATALINE CAPITAL ACTIONS")
		end

	pacap_supplier_product_id: INTEGER
		once
			Result := supplier_product_id_by_name ("PACAP")
		end

	process_flows_supplier_product_id: INTEGER
		once
			Result := supplier_product_id_by_name ("PROCESS_FLOWS")
		end

	supplier_product_id_by_name (a_name: STRING): INTEGER
        require
            a_name_not_void: a_name /= Void
        do
            supplier_product_cache_by_name.search (a_name)
            if supplier_product_cache_by_name.found then
                Result := supplier_product_cache_by_name.found_item.supplier_product_id
            else
                -- This normally shouldn't happen.  We need to have a better error messaging channel. (This can now happen since there are contexts without supplier_ids)
                --print ("Supplier product ID not found for " + a_name + "%N")
            end
        end

    supplier_product_cache_by_name: HASH_TABLE [SUPPLIER_PRODUCT_ROW, STRING]
		local
			l_array: EXTENDABLE_BOUNDED_ARRAY [SUPPLIER_PRODUCT_ROW]
			l_selection: COLLECTING_PREDICATE [SUPPLIER_PRODUCT_ROW]
			i: INTEGER
		once
			create l_array.make_empty
			create l_selection.make (rose_entity_foreground_server, "select * from supplier_product")
			l_selection.set_container (l_array)
			l_selection.retrieve (Void, Void)
			create Result.make (l_array.count + 2)
			from
				i := 1
			until
				i > l_array.count
			loop
				Result.put (l_array.item (i), l_array.item (i).name)
				i := i + 1
			end
		end

	supplier_product_cache_by_id: HASH_TABLE [SUPPLIER_PRODUCT_ROW, INTEGER]
		once
			create Result.make (supplier_product_cache_by_name.count)
			from
				supplier_product_cache_by_name.start
			until
				supplier_product_cache_by_name.off
			loop
				Result.put (supplier_product_cache_by_name.item_for_iteration,
							supplier_product_cache_by_name.item_for_iteration.supplier_product_id)
				supplier_product_cache_by_name.forth
			end
		end

	value_growth_server: SQL_DATA_SERVER
		once
			Result := shared_server_from_repository ("betas")
		end

	production_support_server: SQL_DATA_SERVER
		once
			Result := shared_server_from_repository ("production_support")
		end

	segments_server: SQL_DATA_SERVER
		once
			Result := shared_server_from_repository ("segments")
		end

	helpdesk_server : SQL_DATA_SERVER
		once
			Result := shared_server_from_repository ("helpdesk")
		end

	supplier_download_server: SQL_DATA_SERVER
			-- Supplier download server
		once
			Result := shared_server_from_repository("supplier_downloads")
		end

	investment_monitor_server: SQL_DATA_SERVER
			-- Server for investment monitor data.
		once
			Result := shared_server_from_repository ("investment_monitor")
		end

feature -- Access

	rbc_zonal_server(a_zone : STRING) : SQL_DATA_SERVER
		local
			l_zone : STRING
		do
			l_zone := a_zone.as_lower
			if l_zone.is_equal("j") then
				Result := rbc_j_server
			elseif l_zone.is_equal("u") then
				Result := rbc_u_server
			elseif l_zone.is_equal("o") then
				Result := rbc_o_server
			elseif l_zone.is_equal("c") then
				Result := rbc_c_server
			elseif l_zone.is_equal("e") then
				Result := rbc_e_server
			elseif l_zone.is_equal("s") then
				Result := rbc_s_server
			end
		end

feature -- Market Data

	rose_price_server: SQL_DATA_SERVER
		once
			Result := shared_server_from_repository ("rose_price")
		end

	rose_zonal_price_server(a_zone: STRING): SQL_DATA_SERVER
		do
			a_zone.to_lower
			if a_zone.is_equal("c") then
				Result := rose_price_c_server
			elseif a_zone.is_equal("u") then
				Result := rose_price_u_server
			elseif a_zone.is_equal("o") then
				Result := rose_price_o_server
			elseif a_zone.is_equal("e") then
				Result := rose_price_e_server
			elseif a_zone.is_equal("j") then
				Result := rose_price_j_server
			elseif a_zone.is_equal("s") then
				Result := rose_price_s_server
			elseif a_zone.is_equal("t") then
				Result := rose_price_t_server
			end
		end

	rose_price_c_server: SQL_DATA_SERVER
		once
			Result := shared_server_from_repository ("rose_price_c")
		end

	rose_price_o_server: SQL_DATA_SERVER
		once
			Result := shared_server_from_repository ("rose_price_o")
		end

	rose_price_u_server: SQL_DATA_SERVER
		once
			Result := shared_server_from_repository ("rose_price_u")
		end

	rose_price_j_server: SQL_DATA_SERVER
		once
			Result := shared_server_from_repository ("rose_price_j")
		end

	rose_price_e_server: SQL_DATA_SERVER
		once
			Result := shared_server_from_repository ("rose_price_e")
		end

	rose_price_s_server: SQL_DATA_SERVER
		once
			Result := shared_server_from_repository ("rose_price_s")
		end

	rose_price_t_server: SQL_DATA_SERVER
		once
			Result := shared_server_from_repository ("rose_price_t")
		end

	market_data_servers(a_zone: STRING): SQL_DATA_SERVER
		do
			a_zone.to_lower
			if a_zone.is_equal("c") then
				Result := market_data_c_server
			elseif a_zone.is_equal("u") then
				Result := market_data_u_server
			elseif a_zone.is_equal("o") then
				Result := market_data_o_server
			elseif a_zone.is_equal("e") then
				Result := market_data_e_server
			elseif a_zone.is_equal("j") then
				Result := market_data_j_server
			elseif a_zone.is_equal("s") then
				Result := market_data_s_server
			elseif a_zone.is_equal("i") then
				Result := market_data_i_server
			elseif a_zone.is_equal("f") then
				Result := market_data_f_server
			end
		end

	market_data_c_server: SQL_DATA_SERVER
		once
			Result := shared_server_from_repository ("market_data_c")
		end

	market_data_o_server: SQL_DATA_SERVER
		once
			Result := shared_server_from_repository ("market_data_o")
		end

	market_data_u_server: SQL_DATA_SERVER
		once
			Result := shared_server_from_repository ("market_data_u")
		end

	market_data_j_server: SQL_DATA_SERVER
		once
			Result := shared_server_from_repository ("market_data_j")
		end

	market_data_e_server: SQL_DATA_SERVER
		once
			Result := shared_server_from_repository ("market_data_e")
		end

	market_data_s_server: SQL_DATA_SERVER
		once
			Result := shared_server_from_repository ("market_data_s")
		end

	market_data_i_server: SQL_DATA_SERVER
		once
			Result := shared_server_from_repository ("market_data_i")
		end

	market_data_f_server: SQL_DATA_SERVER
		once
			Result := shared_server_from_repository ("market_data_f")
		end

feature -- Monthly Market Data

	zonal_monthly_rmdat_server (a_zone: STRING): SQL_DATA_SERVER
		do
			a_zone.to_lower
			if a_zone.is_equal("c") then
				Result := monthly_rmdat_c_server
			elseif a_zone.is_equal("u") then
				Result := monthly_rmdat_u_server
			elseif a_zone.is_equal("o") then
				Result := monthly_rmdat_o_server
			elseif a_zone.is_equal("e") then
				Result := monthly_rmdat_e_server
			elseif a_zone.is_equal("j") then
				Result := monthly_rmdat_j_server
			elseif a_zone.is_equal("s") then
				Result := monthly_rmdat_s_server
			elseif a_zone.is_equal("t") then
				Result := monthly_rmdat_t_server
			end
		end

	monthly_rmdat_c_server: SQL_DATA_SERVER
		once
			Result := shared_server_from_repository ("monthly_rmdata_c")
		end

	monthly_rmdat_o_server: SQL_DATA_SERVER
		once
			Result := shared_server_from_repository ("monthly_rmdata_o")
		end

	monthly_rmdat_u_server: SQL_DATA_SERVER
		once
			Result := shared_server_from_repository ("monthly_rmdata_u")
		end

	monthly_rmdat_j_server: SQL_DATA_SERVER
		once
			Result := shared_server_from_repository ("monthly_rmdata_j")
		end

	monthly_rmdat_e_server: SQL_DATA_SERVER
		once
			Result := shared_server_from_repository ("monthly_rmdata_e")
		end

	monthly_rmdat_s_server: SQL_DATA_SERVER
		once
			Result := shared_server_from_repository ("monthly_rmdata_s")
		end

	monthly_rmdat_t_server: SQL_DATA_SERVER
		once
			Result := shared_server_from_repository ("monthly_rmdata_t")
		end

feature -- Model Data

	model_data_server(a_zone_mnemonic: CHARACTER): SQL_DATA_SERVER
		do
			inspect a_zone_mnemonic
			when 'U' then
				Result := model_data_u_server
			when 'C' then
				Result := model_data_c_server
			when 'J' then
				Result := model_data_j_server
			when 'E' then
				Result := model_data_e_server
			when 'O' then
				Result := model_data_o_server
			when 'S' then
				Result := model_data_s_server
			else
			end
		ensure
			result_attached: Result /= Void
		end

	model_company_data_server(a_zone_mnemonic: CHARACTER): SQL_DATA_SERVER
			-- The company portion of the model.
		do
			if model_is_global
			then
				Result := model_data_w_company_server
			else
				Result := model_data_server(a_zone_mnemonic)
			end
		ensure
			result_attached: Result /= Void
		end

	model_security_data_server(a_zone_mnemonic: CHARACTER): SQL_DATA_SERVER
			-- The security portion of the model.
		do
			if model_is_global
			then
				Result := model_data_w_security_server
			else
				Result := model_data_server(a_zone_mnemonic)
			end
		ensure
			result_attached: Result /= Void
		end

	model_listing_data_server(a_zone_mnemonic: CHARACTER): SQL_DATA_SERVER
			-- The listing portion of the model.
		do
			if model_is_global
			then
				Result := model_data_w_listing_server
			else
				Result := model_data_server(a_zone_mnemonic)
			end
		ensure
			result_attached: Result /= Void
		end

	model_deployment_data_server: SQL_DATA_SERVER
			-- Database server to access the 'deployment' table
		once
			Result := shared_server_from_repository ("model_deployment")
		end

	model_data_u_server: SQL_DATA_SERVER
			-- Model database server for zone U
		once
			Result := shared_server_from_repository ("model_u")
		end

	model_data_c_server: SQL_DATA_SERVER
		once
			Result := shared_server_from_repository ("model_c")
		end

	model_data_e_server: SQL_DATA_SERVER
		once
			Result := shared_server_from_repository ("model_e")
		end

	model_data_j_server: SQL_DATA_SERVER
		once
			Result := shared_server_from_repository ("model_j")
		end

	model_data_o_server: SQL_DATA_SERVER
			-- O zone model data server
		once
			Result := shared_server_from_repository ("model_o")
		end

	model_data_s_server: SQL_DATA_SERVER
			-- S zone model data server
		once
			Result := shared_server_from_repository ("model_s")
		end

	model_data_global_server: SQL_DATA_SERVER
			-- Global model data server
		once
			Result := shared_server_from_repository ("model_global")
		end

	model_signoff_server: SQL_DATA_SERVER
			-- Signoff server for risk model data
		once
			Result := shared_server_from_repository ("model_signoff")
		end

	model_data_w_company_server: SQL_DATA_SERVER
			-- Global company model data.
		once
			Result := shared_server_from_repository ("model_w_c")
		end

	model_data_w_security_server: SQL_DATA_SERVER
			-- Global security model data.
		once
			Result := shared_server_from_repository ("model_w_s")
		end

	model_data_w_listing_server: SQL_DATA_SERVER
			-- Global listing model data.
		once
			Result := shared_server_from_repository ("model_w_l")
		end

	model_server: SQL_DATA_SERVER
		once
			Result := shared_server_from_repository ("model")
		end

	model_browser_server: SQL_DATA_SERVER
		once
			Result := shared_server_from_repository ("model_browser")
		end

	model_historical_data_quality: SQL_DATA_SERVER
		once
			Result := shared_server_from_repository ("historical_data_quality_project")
		end

feature -- Monthly Data

	monthly_data_server (a_zone_mnemonic: CHARACTER; a_server_type: CHARACTER; a_is_using_test_env: BOOLEAN): SQL_DATA_SERVER is
			-- 'a_server_type' monthly data server for 'a_zone_mnemonic'
			-- Use of 'a_is_using_test_env' is disabled
		do
			if a_is_using_test_env then
				(create {EXCEPTIONS}).raise ("Use of 'a_is_using_test_env' is disabled")
			else
				inspect a_server_type
				when 'C' then
					inspect a_zone_mnemonic
					when 'U' then
						Result := monthly_data_u_c_server
					when 'C' then
						Result := monthly_data_c_c_server
					when 'J' then
						Result := monthly_data_j_c_server
					when 'E' then
						Result := monthly_data_e_c_server
					when 'O' then
						Result := monthly_data_o_c_server
					when 'S' then
						Result := monthly_data_s_c_server
					else
						(create {EXCEPTIONS}).raise ("Invalid zone - " + a_zone_mnemonic.out)
					end
				when 'S' then
					inspect a_zone_mnemonic
					when 'U' then
						Result := monthly_data_u_s_server
					when 'C' then
						Result := monthly_data_c_s_server
					when 'J' then
						Result := monthly_data_j_s_server
					when 'E' then
						Result := monthly_data_e_s_server
					when 'O' then
						Result := monthly_data_o_s_server
					when 'S' then
						Result := monthly_data_s_s_server
					else
						(create {EXCEPTIONS}).raise ("Invalid zone - " + a_zone_mnemonic.out)
					end
				else
					(create {EXCEPTIONS}).raise ("Invalid type - " + a_server_type.out)
				end
			end
		ensure
			result_attached: Result /= Void
		end

	monthly_data_fast_load_server: SQLOLEDB_DATA_SERVER
		do
			Result := make_from_repository_fast_load ("monthly_data")
		end

	monthly_data_u_c_server: SQL_DATA_SERVER
			-- U zone company monthly data database
		do
			Result := shared_server_from_repository ("monthly_data_u_c")
		end

	monthly_data_u_s_server: SQL_DATA_SERVER
			-- U zone security monthly data database
		do
			Result := shared_server_from_repository ("monthly_data_u_s")
		end

	monthly_data_c_c_server: SQL_DATA_SERVER
			-- C zone company monthly data database
		do
			Result := shared_server_from_repository ("monthly_data_c_c")
		end

	monthly_data_c_s_server: SQL_DATA_SERVER
			-- C zone security monthly data database
		do
			Result := shared_server_from_repository ("monthly_data_c_s")
		end

	monthly_data_j_c_server: SQL_DATA_SERVER
			-- J zone company monthly data database
		do
			Result := shared_server_from_repository ("monthly_data_j_c")
		end

	monthly_data_j_s_server: SQL_DATA_SERVER
			-- J zone security monthly data database
		do
			Result := shared_server_from_repository ("monthly_data_j_s")
		end

	monthly_data_e_c_server: SQL_DATA_SERVER
			-- E zone company monthly data database
		do
			Result := shared_server_from_repository ("monthly_data_e_c")
		end

	monthly_data_e_s_server: SQL_DATA_SERVER
			-- E zone security monthly data database
		do
			Result := shared_server_from_repository ("monthly_data_e_s")
		end

	monthly_data_o_c_server: SQL_DATA_SERVER
			-- O zone company monthly data database
		do
			Result := shared_server_from_repository ("monthly_data_o_c")
		end

	monthly_data_o_s_server: SQL_DATA_SERVER
			-- O zone security monthly data database
		do
			Result := shared_server_from_repository ("monthly_data_o_s")
		end

	monthly_data_s_c_server: SQL_DATA_SERVER
			-- S zone company monthly data database
		do
			Result := shared_server_from_repository ("monthly_data_s_c")
		end

	monthly_data_s_s_server: SQL_DATA_SERVER
			-- S zone security monthly data database
		do
			Result := shared_server_from_repository ("monthly_data_s_s")
		end

feature -- Valuation data

	valuation_data_server(a_region: CHARACTER): SQL_DATA_SERVER
			-- valuation data server by region
		do
			inspect a_region.as_upper
			when 'J' then
				Result := valuation_data_j_server
			when 'R' then
				Result := valuation_data_row_server
			when 'U' then
				Result := valuation_data_u_server
			else
			end
		end

	valuation_data_j_server: SQL_DATA_SERVER
			-- valuation data j server
		once
			Result := shared_server_from_repository ("valuation_data_j")
		end

	valuation_data_row_server: SQL_DATA_SERVER
			-- valuation data row server
		once
			Result := shared_server_from_repository ("valuation_data_row")
		end

	valuation_data_u_server: SQL_DATA_SERVER
			-- valuation data u server
		once
			Result := shared_server_from_repository ("valuation_data_u")
		end

	valuation_data_override_server(a_region: CHARACTER): SQL_DATA_SERVER
			-- valuation data override server by region
		do
			inspect a_region.as_upper
			when 'J' then
				Result := valuation_data_override_j_server
			when 'R' then
				Result := valuation_data_override_row_server
			when 'U' then
				Result := valuation_data_override_u_server
			else
			end
		end

	valuation_data_override_j_server: SQL_DATA_SERVER
			-- valuation data override j server
		once
			Result := shared_server_from_repository ("valuation_data_override_j")
		end

	valuation_data_override_row_server: SQL_DATA_SERVER
			-- valuation data override row server
		once
			Result := shared_server_from_repository ("valuation_data_override_row")
		end

	valuation_data_override_u_server: SQL_DATA_SERVER
			-- valuation data override u server
		once
			Result := shared_server_from_repository ("valuation_data_override_u")
		end

	valuation_data_override_w_server: SQL_DATA_SERVER
			-- valuation data override w server
		once
			Result := shared_server_from_repository ("valuation_data_override")
		end

	valbase_server: SQL_DATA_SERVER
			-- valbase server
		once
			Result := shared_server_from_repository ("valbase")
		end

feature -- Mondex Data

	mondex_server (a_zone_mnemonic: CHARACTER): SQL_DATA_SERVER
			-- mondex server for 'a_zone_mnemonic'
		require
			valid_zone : a_zone_mnemonic /= '%U'
		do
			inspect a_zone_mnemonic
			when 'U' then
				Result := mondex_u_server
			when 'C' then
				Result := mondex_c_server
			when 'E' then
				Result := mondex_e_server
			when 'O' then
				Result := mondex_o_server
			when 'J' then
				Result := mondex_j_server
			when 'S' then
				Result := mondex_s_server
			else
				(create {EXCEPTIONS}).raise ("Invalid zone - " + a_zone_mnemonic.out)
			end
		end

	mondex_u_server: SQL_DATA_SERVER
			--mondex u server
		once
			Result := shared_server_from_repository ("mondex_u")
		end

	mondex_c_server: SQL_DATA_SERVER
			-- mondex c server
		once
			Result := shared_server_from_repository ("mondex_c")
		end

	mondex_e_server: SQL_DATA_SERVER
			-- mondex e server
		once
			Result := shared_server_from_repository ("mondex_e")
		end

	mondex_j_server: SQL_DATA_SERVER
			-- mondex j server
		once
			Result := shared_server_from_repository ("mondex_j")
		end

	mondex_o_server: SQL_DATA_SERVER
			-- mondex o server
		once
			Result := shared_server_from_repository ("mondex_o")
		end

	mondex_s_server: SQL_DATA_SERVER
			-- mondex s server
		once
			Result := shared_server_from_repository ("mondex_s")
		end

	mondex_variable_statistics_server: SQL_DATA_SERVER
		once
			Result := shared_server_from_repository ("mondex_variable_statistics")
		end

	variable_statistics_server: SQL_DATA_SERVER
		once
			Result := shared_server_from_repository ("variable_statistics")
		end

feature -- Earnings estimate foreground
	zonal_earnings_estiamtes_fg_server(a_zone_mnemonic : CHARACTER) : SQL_DATA_SERVER
			-- zonal earnings estimates fg server
		do
			if a_zone_mnemonic.out.as_upper.is_equal("U") then
				Result := earnings_estimate_u_fg_server
			elseif a_zone_mnemonic.out.as_upper.is_equal("C") then
				Result := earnings_estimate_c_fg_server
			elseif a_zone_mnemonic.out.as_upper.is_equal("E") then
				Result := earnings_estimate_e_fg_server
			elseif a_zone_mnemonic.out.as_upper.is_equal("O") then
				Result := earnings_estimate_o_fg_server
			elseif a_zone_mnemonic.out.as_upper.is_equal("J") then
				Result := earnings_estimate_j_fg_server
			elseif a_zone_mnemonic.out.as_upper.is_equal("S") then
				Result := earnings_estimate_s_fg_server
			end
		end

	earnings_estimate_u_fg_server: SQL_DATA_SERVER
			-- 	earnings_estimate_u_fg_server
		once
			Result := shared_server_from_repository ("earnings_estimate_u_fg")
		end

	earnings_estimate_s_fg_server: SQL_DATA_SERVER
			-- 	earnings_estimate_s_fg_server
		once
			Result := shared_server_from_repository ("earnings_estimate_s_fg")
		end

	earnings_estimate_o_fg_server: SQL_DATA_SERVER
			-- 	earnings_estimate_o_fg_server
		once
			Result := shared_server_from_repository ("earnings_estimate_o_fg")
		end

	earnings_estimate_e_fg_server: SQL_DATA_SERVER
			-- 	earnings_estimate_e_fg_server
		once
			Result := shared_server_from_repository ("earnings_estimate_e_fg")
		end

	earnings_estimate_j_fg_server: SQL_DATA_SERVER
			-- earnings_estimate_j_fg_server
		once
			Result := shared_server_from_repository ("earnings_estimate_j_fg")
		end

	earnings_estimate_c_fg_server: SQL_DATA_SERVER
			-- earnings_estimate_c_fg_server
		once
			Result := shared_server_from_repository ("earnings_estimate_c_fg")
		end

feature -- Events servers

	gedb_adjustments_fast_load: SQL_DATA_SERVER
			-- gedb_adjustments server, in fast load mode
		once
			Result := make_from_repository_fast_load ("gedb_adjustments")
		end

	gedb_fg: SQL_DATA_SERVER
			-- gedb_fg server (updated once a day from GEDB Gload)
		once
			Result := shared_server_from_repository ("gedb_fg")
		end

	gedb_fg_fast_load: SQL_DATA_SERVER
			-- gedb_fg in fast-load mode (updated once a day from GEDB Gload)
		once
			Result := make_from_repository_fast_load ("gedb_fg")
		end

	gedb_server: SQL_DATA_SERVER
			-- Global Events DB
			-- !!You should only use it if you want intra-day updates,
			-- otherwise use gedb_fg* version
		once
			Result := shared_server_from_repository ("gedb_gload")
		end

	gedb_server_fast_load: SQL_DATA_SERVER
			-- Global Events DB in fast-load mode
			-- !!You should only use it if you want intra-day updates,
			-- otherwise use gedb_fg* version
		once
			Result := make_from_repository_fast_load ("gedb_gload")
		end

	positions: SQL_DATA_SERVER
			-- Positions server (built from Northern Trust data)
		once
			Result := shared_server_from_repository ("positions")
		end

	composite_server: DATA_SERVER
		once
			Result := new_sqlsrv ("vms_composite")
		end

	composite_assets_server: SQL_DATA_SERVER
			-- Server for composite assets.
		once
			Result := shared_server_from_repository("composite_assets")
		end

	ucits_server: SQL_DATA_SERVER
		once
			Result := shared_server_from_repository("ucits_marketing")
		end

feature -- Trading servers

	global_trading_server:SQL_DATA_SERVER
			-- database server for global trading
		once
			Result := shared_server_from_repository ("globaltrading")
		end

	global_trading_dailysnap_server: SQL_DATA_SERVER
			-- globaltrading database server snashot daily from production
		once
			Result := shared_server_from_repository ("globaltrading_dailysnap")
		end

	global_trading_dailysnap_qa_server:SQL_DATA_SERVER
			-- QA globaltrading database server snashot daily from production
		once
			Result := shared_server_from_repository ("globaltrading_dailysnap_QA")
		end

	global_trading_test: SQL_DATA_SERVER
			-- Test globaltrading DB copied from time to time from production
		once
			Result := shared_server_from_repository ("globaltrading_devel")
		end

	longshortequity_devel: SQL_DATA_SERVER
			-- devel copy of long short equity database server
		once
			Result := shared_server_from_repository ("longshortequity_devel")
		end

	longshortequity: SQL_DATA_SERVER
			-- database server for longshort equity
		once
			Result := shared_server_from_repository ("longshortequity")
		end

	global_trading_longshort_server: SQL_DATA_SERVER
			-- database server for longshort dailysnap
		once
			Result := shared_server_from_repository ("globaltrading_longshort")
		end

	global_trading_longshort_static_server: SQL_DATA_SERVER
			-- database server for longshort dailysnap
		once
			Result := shared_server_from_repository ("globaltrading_longshort_static")
		end

	europe_trading_server: SQL_DATA_SERVER
			-- database server for europetrading (used by Europe engineering team)
		once
			Result := shared_server_from_repository ("europetrading")
		end

	global_engineering_server: SQL_DATA_SERVER
			-- database server for global engineering
		once
			Result := shared_server_from_repository ("globalengineering")
		end

	quop_staging_server: attached SQL_DATA_SERVER
			-- Database server for quop_loader to use as a staging database for fast inserts to GTA tables
		once
			Result := strict_shared_server_from_repository ("quop_staging")
		ensure
			result_attached: Result /= Void
		end

	realized_gain_loss_server: SQL_DATA_SERVER
			-- database server for Realized gain loss (populated by RGL toolbox app)
		once
			Result := shared_server_from_repository ("rgl")
		end

feature -- Quop servers

	quop_server:SQL_DATA_SERVER
			-- obsolete
		once
			Result := shared_server_from_repository("quop")
		end
	quop_u_server:SQL_DATA_SERVER
			-- Quop server for U zone.
		once
			Result := shared_server_from_repository("quop_u")
		end
	quop_j_server:SQL_DATA_SERVER
		once
			Result := shared_server_from_repository("quop_j")
		end
	quop_c_server:SQL_DATA_SERVER
		once
			Result := shared_server_from_repository("quop_c")
		end
	quop_e_server:SQL_DATA_SERVER
		once
			Result := shared_server_from_repository("quop_e")
		end
	quop_o_server:SQL_DATA_SERVER
		once
			Result := shared_server_from_repository("quop_o")
		end
	quop_s_server:SQL_DATA_SERVER
		once
			Result := shared_server_from_repository("quop_s")
		end

	zonal_quop_server (a_zone_mnemonic: CHARACTER): SQL_DATA_SERVER
		local
			l_zone: CHARACTER
		do
			l_zone := a_zone_mnemonic.as_upper
			inspect l_zone
			when 'U' then
				Result := quop_u_server
			when 'J' then
				Result := quop_j_server
			when 'C' then
				Result := quop_c_server
			when 'E' then
				Result := quop_e_server
			when 'O' then
				Result := quop_o_server
			when 'S' then
				Result := quop_s_server
			else
				print ("%N  Invalid zone of " + l_zone.out + "%N")
			end
		end

	quop_instruction_entities_server:SQL_DATA_SERVER
		once
			Result := shared_server_from_repository("quop_instruction_sets")
		end
	quop_instructions_u_server:SQL_DATA_SERVER
		once
			Result := shared_server_from_repository("quop_instructions_u")
		end
	quop_instructions_j_server:SQL_DATA_SERVER
		once
			Result := shared_server_from_repository("quop_instructions_j")
		end
	quop_instructions_c_server:SQL_DATA_SERVER
		once
			Result := shared_server_from_repository("quop_instructions_c")
		end
	quop_instructions_e_server:SQL_DATA_SERVER
		once
			Result := shared_server_from_repository("quop_instructions_e")
		end
	quop_instructions_o_server:SQL_DATA_SERVER
		once
			Result := shared_server_from_repository("quop_instructions_o")
		end
	quop_instructions_s_server:SQL_DATA_SERVER
		once
			Result := shared_server_from_repository("quop_instructions_s")
		end

	recommendation_server: attached SQL_DATA_SERVER
			-- Database server used by recommendation_server
		once
			Result := strict_shared_server_from_repository ("recommendations")
		ensure
			result_attached: Result /= Void
		end

	reuters_price_snapshot_server: attached SQL_DATA_SERVER
			-- database server for reuters realtime price snapshots
		once
			Result := strict_shared_server_from_repository ("reuters_price_snapshots")
		ensure
			result_attached: Result /= Void
		end

feature -- Access

	extranet_reporting_server:SQL_DATA_SERVER
		once
			Result := shared_server_from_repository ("web_extranet_reporting")
		end

	web_data_platform_server:SQL_DATA_SERVER
			-- Database server for the web data platform
		once
			Result := shared_server_from_repository ("web_data_platform")
		end

	axau_database_server: attached SQL_DATA_SERVER
			-- Database server for AXA Universe construction
		once
			Result := strict_shared_server_from_repository ("axa_universe")
		ensure
			result_attached: Result /= Void
		end

	axau_database_fast_load_server: attached SQL_DATA_SERVER
			-- Fast load database server for AXA Universe construction
		once
			Result := make_from_repository_fast_load ("axa_universe")
		ensure
			result_attached: Result /= Void
		end

	research_bogies_server: SQL_DATA_SERVER
			-- Supplier database for in-house bogies' constituents
		once
			Result := shared_server_from_repository ("research_bogies")
		end

feature -- Distributions from Rdb

	legacy_distributions_server: DATA_SERVER
			-- legacy global distribution server pre capact migration
		once
			Result := new_sqlsrv ("vms_legacy_distributions")
		end

	legacy_distribution_server_c: DATA_SERVER
			-- legacy distribution database C zone
		once
			Result := new_sqlsrv ("vms_legacy_distribution_c")
		end

	legacy_distribution_server_e: DATA_SERVER
			-- legacy distribution database E zone
		once
			Result := new_sqlsrv ("vms_legacy_distribution_e")
		end

	legacy_distribution_server_o: DATA_SERVER
			-- legacy distribution database O zone
		once
			Result := new_sqlsrv ("vms_legacy_distribution_o")
		end

	legacy_distribution_server_u: DATA_SERVER
			-- legacy distribution database U zone
		once
			Result := new_sqlsrv ("vms_legacy_distribution_u")
		end

	drlink_server: DATA_SERVER
		once
			Result := new_sqlsrv ("vms_drlink")
		end

feature -- CSI

	csi_server: SQL_DATA_SERVER
			-- Server for CSI database
		once
			Result := shared_server_from_repository("csi")
		end

feature -- Code quality

	eiffel_object_snapshot_server: SQL_DATA_SERVER
			-- server connection to the database where system object_count information stored
		once
			Result := shared_server_from_repository ("eiffel_object_snapshots")
		end

feature -- Data quality

	data_quality_server: SQL_DATA_SERVER
			-- Server connection to the databse where raw information on data quality of various DBs is stored
		once
			Result := shared_server_from_repository ("data_quality")
		end

feature -- IOO databases

	ossm_server: SQL_DATA_SERVER is
			-- Server for OSSM PostTradeProcessing
		once
			Result := strict_shared_server_from_repository ("post_trade_processing")
		ensure
			result_attached: Result /= Void
		end

feature -- Test management

	test_dependencies_server: SQL_DATA_SERVER
			-- Server for test_dependencies
		once
			Result:= shared_server_from_repository("test_dependencies")
		end

	parallel_test_environment_server: SQL_DATA_SERVER
			-- Server for the parallel test environment
		once
			Result := shared_server_from_repository ("parallel_test_environment")
		end

feature -- Contract helper

	is_valid_zone (a_zone : CHARACTER): BOOLEAN
			-- the character associated with a valid equity zone?
		do
			Result := ("UCEOJS").has (a_zone.as_upper)
		end

feature {NONE} -- Implementation

	compare_strings (s1, s2: STRING): BOOLEAN
			-- `s1' less than `s2'?
		require
			s1_not_void: s1 /= Void
			s2_not_void: s2 /= Void
		do
			Result := s1 < s2
		end

	master_cache: HASH_TABLE [SQLOLEDB_DATA_SERVER, STRING]
			-- Servers keyed by logical
		once
			create Result.make(0)
		end

end
