note

	description:

		"MATRIX in which the rows are associated with dates"

	author: "Feng Zhou"
	last_editor: "$Author: er $"
	date: "$Date: 2014/02/26 $"
	revision: "$Revision: #50 $"
	path: "$File: //rose/source/kernel/math/dated_matrix.e $"
	storable_version: ""

class
	DATED_MATRIX

inherit
	MATRIX
	redefine
		anchor,
		selected_columns,
		cloned,
		sub_matrix_c,
		sub_matrix_2,
		extend_matrix,
		row_weighted,
		column_weighted,
		selected_columns_by_names,
		itemwise_product,
		make_from_matrix
	end

create
	make, make_filled, make_identity, make_from_array2, retrieve_from_file, retrieve_from, make_from_matrix, make_all, make_from_vector_array, make_from_series_array, duplicate,make_top_rows,make_from_named_vectors

create {DATED_MATRIX}
	make_as_inverse

feature -- Creation

	make_from_matrix(a_matrix:MATRIX) is
		-- make only with a matrix, trying to infer the
		-- dates from row names if any.
		do
			Precursor(a_matrix)
			if row_names/=void then
				set_dates(dates_from_names(row_names))
			end
		end

	make_all(a_dates : ARRAY[ROSE_DATE]; a_matrix : ABSTRACT_READABLE_MATRIX) is
		-- make with matrix and dates
		require
			valid_size : (a_dates /= Void and a_matrix /= Void) and then a_dates.count = a_matrix.height
		local
			i, j : INTEGER
		do
			make(a_matrix.height,a_matrix.width)
			from i:=1 until i>height loop
				from j:=1 until j>width loop
					put(a_matrix.item(i,j),i,j)
					j:=j+1
				end
				i:=i+1
			end
			column_names:=a_matrix.column_names
			row_names:=a_matrix.row_names
			name:=a_matrix.name
			set_dates(a_dates)
		ensure
			height_set: height = a_matrix.height
			width_set: width = a_matrix.width
		end

	make_from_vector_array (a_vector_array: ARRAY [VECTOR]; a_dates: ARRAY [ROSE_DATE]) is
			-- Initialize from `a_vector_array' and `a_dates'.
		require
			a_vector_array_not_void: a_vector_array /= Void
			no_void_vector: not a_vector_array.has (Void)
			a_dates_not_void: a_dates /= Void
			same_size: a_vector_array.count = a_dates.count
		local
			i, l_lower, l_ix : INTEGER
			l_vector : VECTOR
			l_date : ROSE_DATE
			l_dates : ROSE_LINEAR_ARRAY[ROSE_DATE]
			l_table : ROSE_HASHED_LINEAR_ARRAY[ROSE_DATE]
		do
			l_lower := a_vector_array.lower
			make(a_vector_array.count, a_vector_array.item(l_lower).count)
			create l_dates.make_from_array(a_dates)
			l_dates.sort_by(Agent date_order)
			create l_table.make_from_array(a_dates)

			from i := 1 until i > a_vector_array.count loop
				l_date := l_dates.item(i)
				l_ix := l_table.new_index_of(l_date)
				l_vector := a_vector_array.item(l_lower + l_ix - 1)
				put_row_vector(l_vector, i)
				i := i + 1
			end
			set_dates(l_dates)
			set_column_names (a_vector_array.item (1).names)
		end

	make_from_series_array(a_array : ARRAY[DOUBLE_TIME_SERIES]) is
			-- make from array of time series
		local
			l_date, l_max_date : ROSE_DATE
			i, l_row, l_col : INTEGER
			l_series : DOUBLE_TIME_SERIES
			l_set : ROSE_DATE_SET
		do
			from i := a_array.lower until i > a_array.upper loop
				l_series := a_array.item(i)
				if l_series /= Void and then l_series.count > 0 then
					if l_max_date = Void then
						l_max_date := l_series.last_date
					else
						l_max_date := l_series.last_date.max(l_max_date)
					end
				end
				i := i + 1
			end

			from i := a_array.lower until i > a_array.upper loop
				if a_array.item(i) /= Void and then a_array.item(i).count > 0 then
					if l_set = Void then
						create l_set.make_from_array(a_array.item(i).date_set.dates(Void, Void), l_max_date)
					else
						l_set := l_set.or_with(a_array.item(i).date_set)
					end
				end
				i := i + 1
			end

			set_dates(l_set.dates(Void, Void))
			make(dates.count, a_array.count)
			from i := a_array.lower until i > a_array.upper loop
				l_series := a_array.item(i)
				l_col := i - a_array.lower + 1
				set_column_null(l_col)
				if l_series /= Void and then l_series.count > 0 then
					from l_series.start until l_series.off loop
						l_date := l_series.current_date
						l_row := dates.new_index_of(l_date)
						put(l_series.current_item, l_row, l_col)
						l_series.forth
					end
				end
				i := i + 1
			end
		end

	update_dates(a_dates:ARRAY[ROSE_DATE];a_vectors:ARRAY[VECTOR]) is
			-- Update existing rows for each date already in the matrix,
			-- or add new dates to the matrix.
		require
			a_dates_not_void: a_dates /= Void
			a_vectors_not_void: a_vectors /= Void
			size: a_dates.count = a_vectors.count
		local
			i,k:INTEGER
			l_dates:ROSE_LINEAR_ARRAY[ROSE_DATE]
			l_rows:ROSE_LINEAR_ARRAY[VECTOR]
		do
			create l_dates.make_from_array (dates)
			l_rows:=rows_array
			from i:=1 until i>a_dates.count loop
				k:=row_ix (a_dates.item(i))
				if k>0 then
					l_rows.put (a_vectors.item(i), k)
				else
					l_rows.extend (a_vectors.item(i))
					l_dates.extend(a_dates.item(i))
				end
				i:=i+1
			end
			make_from_vector_array (l_rows,l_dates)
		end

	update_dates_with_names (a_dates: ARRAY [ROSE_DATE]; a_vectors: ARRAY [VECTOR])
			-- Update this matrix with 'a_vectors' corresponding to 'a_dates'
			-- Replace existing date vectors by corresponding item in 'a_vectors'
			-- Insert new date vectors if none were already present
		local
			i, k: INTEGER
			l_dates: ROSE_LINEAR_ARRAY [ROSE_DATE]
			l_rows: ROSE_LINEAR_ARRAY [VECTOR]
			l_matrix: like current
		do
			create l_dates.make_from_array (dates)
			l_rows := rows_array
			from
				i := 1
			until
				i > a_dates.count
			loop
				k := row_ix (a_dates.item (i))
				if k > 0 then
					l_rows.put (a_vectors.item(i), k)
				else
					l_rows.extend (a_vectors.item (i))
					l_dates.extend (a_dates.item (i))
				end
				i := i + 1
			end
			create l_matrix.make_from_named_vectors (l_rows)
			make_from_vector_array (l_matrix.rows_array, l_dates)
			set_column_names (l_matrix.column_names)
		end

	cloned : like anchor is
			-- cloned
		do
			create Result.make_all(dates, Current)
			Result.set_currency_series (currency_series)
			Result.set_currency (currency)
		end

	sub_matrix_c (i0, i1: INTEGER; c: ARRAY[INTEGER]): like anchor is
			-- sub matrix C
		local
			l_matrix : MATRIX
		do
			l_matrix := Precursor(i0, i1, c)
			create Result.duplicate(l_matrix)
			if dates /= Void and i0 >= dates.lower and i1 <= dates.upper then
				Result.set_dates(dates.subarray (i0, i1))
			end
			if column_names /= Void then
				Result.set_column_names (column_names.subset(c))
			end
		end

	sub_matrix_2 (i0, i1, j0, j1: INTEGER): like anchor is
			-- sub matrix 2
		local
			l_matrix : MATRIX
		do
			l_matrix := Precursor(i0, i1, j0, j1)
			create Result.duplicate(l_matrix)
			if dates /= Void and i0 >= dates.lower and i1 <= dates.upper then
				Result.set_dates(dates.subarray (i0, i1))
			end
			if column_names /= Void and then (column_names.valid_index (j0) and column_names.valid_index (j1)) then
				Result.set_column_names (column_names.subarray (j0, j1))
			end
		end

	extend_matrix (a_matrix : ABSTRACT_READABLE_MATRIX): DATED_MATRIX is
			-- Extend the width of a matrix with new columns
		local
			i: INTEGER --,k:INTEGER
			l_v1,l_v2:VECTOR
			l_column_names : like column_names
		do
			create Result.make(height,width+a_matrix.width)
			from i:=1 until i>height loop
				l_v1:=row(i)
				l_v2:=a_matrix.row(i)
				l_v1.extend_vector(l_v2)
				Result.put_row_vector(l_v1,i)
				i:=i+1
			end
			Result.set_row_names(row_names)
			if  column_names /= Void and then a_matrix.column_names /= Void then
				l_column_names := clone(column_names)
				l_column_names.append_array(a_matrix.column_names)
				Result.set_column_names(l_column_names)
			end
			if dates /= Void then
				Result.set_dates (dates)
			end
		end

	row_weighted(a_vector:VECTOR): like Current is
			-- row weighted
		local
			l_matrix : MATRIX
		do
			l_matrix := Precursor(a_vector)
			create Result.make_all (dates, l_matrix)
		end

	column_weighted(a_vector:VECTOR): like Current is
			-- column weighted
		local
			l_matrix : MATRIX
		do
			l_matrix := Precursor(a_vector)
			create Result.make_all (dates, l_matrix)
		end

feature -- element change

	put_at (a_value: DOUBLE; a_date: ROSE_DATE; j: INTEGER)
			-- replace elements based on date and column index rather than explicit row index
		require
			date_is_valid: a_date /= Void
			dates_range_exists: dates /= Void
			date_in_range: first_date <= a_date and a_date <= last_date
			column_index_is_valid: valid_column_index (j)
		do
			put (a_value, row_ix (a_date), j)
		ensure
			value_set: item (row_ix (a_date), j) = a_value
		end

	put_at_name (a_value: DOUBLE; a_date: ROSE_DATE; a_name: STRING)
			-- replace elements based on date and column name rather than explicit row and column indices
		require
			date_is_valid: a_date /= Void
			dates_range_exists: dates /= Void
			date_in_range: first_date <= a_date and a_date <= last_date
			column_name_not_void: a_name /= Void
			column_table_exists: column_table /= Void
			column_name_in_table: column_table.item (a_name) > 0
		do
			put (a_value, row_ix (a_date), column_ix (a_name))
		ensure
			value_set: item (row_ix (a_date), column_ix (a_name)) = a_value
		end

feature -- implementation

	extend_cumulative
			-- Add `column_cumulative' row at end of matrix.
		do
			add_row (column_cumulative)
		end

	extend_average
			-- Add row with average of each columm at end of matrix.
		local
			l_cums: VECTOR
			i: INTEGER
		do
			create l_cums.make (width)
			from
				i := 1
			until
				i > width
			loop
				l_cums.put (column (i).average, i)
				i := i + 1
			end
			l_cums.set_name ("average")
			add_row (l_cums)
		end

	extend_delta
			-- Add a row called "delta" that contains the difference between the last row and the 1st row in this matrix
		local
			l_cums: VECTOR
			i, l_last: INTEGER
		do
			create l_cums.make (width)
			l_last := row_ix (dates.last)
			from
				i := 1
			until
				i > width
			loop
				l_cums.put (column (i).item (l_last) - column (i).item (1), i)
				i := i + 1
			end
			l_cums.set_name ("delta")
			add_row (l_cums)
		end

	column_cumulative:VECTOR is
		-- Vector containing cumulative return (in VECTOR.cum_return sense)
		-- of each column
	local
		i:INTEGER
	do
		create result.make(width)
		from i:=1 until i>width loop
			result.put(	column(i).sub_vector(1,dates.count).cum_return,i)
			i:=i+1
		end
		result.set_name("cumulative")
	ensure
		not_void: Result /= Void
		size: Result.count = width
	end

	extend_annualized is
		-- extend annualized
	local
		l_cums:VECTOR
		i:INTEGER
		l_vector:VECTOR
		l_return:DOUBLE
	do
		l_cums:=column_cumulative
		create l_vector.make(width)
		from i:=1 until i>width loop
			l_return:=(1+l_cums.item(i))^(12/(dates.count))-1
			l_vector.put(l_return,i)
			i:=i+1
		end
		l_vector.set_name("annualized")
		add_row(l_vector)
	end

feature -- variables

	first_date : ROSE_DATE is
			-- first date in dates from row labels range
		do
			Result := dates.first
		end

	last_date : ROSE_DATE is
			-- last date in dates from row labels range
		do
			Result := dates.last
		end

	column_ix(a_name : STRING) : INTEGER is
			-- Column index corresponding to column title
			-- Returns zero if column title is invalid (Void or missing from list)
		require
			column_table_not_void: column_table /= Void
		do
			if a_name /= Void then
				Result := column_table.item(a_name)
			end
		end

	row_ix(a_date : ROSE_DATE) : INTEGER is
			-- Row index corresponding to row date
			-- Returns zero if 'a_date' is Void or 'dates' range is void or missing from range
			-- Attempts conbversion into rose_date and rose_month to attempt match
		do
			if a_date /= Void and then dates /= Void then
				Result := dates.new_index_of (a_date)
				if not valid_row_index (Result) then
					Result := dates.new_index_of (a_date.rose_date)
					if not valid_row_index (Result) then
						Result := dates.new_index_of (a_date.rose_month.last)
					end
				end
			end
		end

	row_at(a_date:ROSE_DATE):VECTOR is
				-- Row corresponding to a date
			local
				i:INTEGER
			do
				i := row_ix(a_date)
				if i>0 then
					result := row(i)
					result.set_names(column_names)
					result.set_name(a_date.ccyymmdd.out)
				end
			end

	value_at(a_date:ROSE_DATE;a_variable:STRING): DOUBLE is
				-- Value for a date (row) and variable (column)
			local
				i, j : INTEGER
			do
				i := row_ix(a_date)
				j := column_table.item(a_variable)
				if i > 0 and j > 0 then
					Result := item(i, j)
				end
			end

	subset(a_dates:ARRAY[ROSE_DATE];a_variables:ARRAY[STRING]): like Current is
				-- Sub matrix for a set of dates and variables
				-- (all columns if `a_variables' is Void)
			local
				i,j:INTEGER
				l_vars:ARRAY[STRING]
			do
				if a_variables/=void then
					l_vars:=a_variables
				else
					l_vars:=column_names
				end
				create result.make(a_dates.count,l_vars.count)
				from i:=1	until i>a_dates.count	loop
					from j:=1  until j>l_vars.count		loop
						result.put(value_at(a_dates.item(i),l_vars.item(j)),i,j)
						j:=j+1
					end
					i:=i+1
				end
				result.set_dates(a_dates)
				result.set_column_names(l_vars)
			end

	column_series(a_col : INTEGER) : DOUBLE_TIME_SERIES is
			-- Time series for a column
		local
			i : INTEGER
		do
			if valid_column_index(a_col) then
				create Result.make_with_dates(dates)
				from i := 1 until i > height loop
					Result.put(item(i, a_col), i)
					i := i + 1
				end
			end
		end

	column_series_by_name(a_name : STRING) : DOUBLE_TIME_SERIES is
			-- Time series for a variable (column)
		local
			i, l_col : INTEGER
		do
			l_col := column_ix(a_name)
			if valid_column_index(l_col) then
				create Result.make_with_dates(dates)
				from i := 1 until i > height loop
					Result.put(item(i, l_col), i)
					i := i + 1
				end
			end
		end

	selected_columns(c: ARRAY[INTEGER]): like anchor is
			-- New matrix with subset of columns
		local
			i, l_ix : INTEGER
			l_matrix : MATRIX
			l_vars : ROSE_LINEAR_ARRAY[STRING]
		do
			if c /= Void then
				l_matrix := Precursor(c)
				create Result.make_from_matrix(l_matrix)

				Result.set_dates(dates)
				create l_vars.make_empty
				from i := c.lower until i > c.upper loop
					l_ix := c.item(i)
					l_vars.extend(column_names.item(l_ix))
					i := i + 1
				end
				Result.set_column_names(l_vars)
				Result.set_currency_series(currency_series)
			end
		end

	selected_columns_by_names(a_names : ARRAY[STRING]) : like anchor is
			-- selected columns by names
		local
			l_matrix : MATRIX
		do
			l_matrix := Precursor(a_names)
			if l_matrix /= Void then
				create Result.make_all (dates, l_matrix)
			end
		end

	columnwise_combine (a_matrix: like anchor): like anchor
			-- columnwise_combine
		local
			i: INTEGER
			l_date: ROSE_DATE
			l_height, l_width: INTEGER
			l_vector, l_subvector: VECTOR
			l_matrix: MATRIX
			l_dates: ROSE_HASHED_LINEAR_ARRAY [ROSE_DATE]
		do
			if dates /= Void and then a_matrix.dates /= Void then
				from
					create l_dates.make_from_array (a_matrix.dates)
					i := dates.lower
				until
					i > dates.upper
				loop
					l_date := dates.item (i)
					if not l_dates.has_value (l_date) then
						l_dates.extend (l_date)
					end
					i := i + 1
				end
				l_dates.sort_by (agent date_order)
				l_height := l_dates.count
				l_width := width + a_matrix.width
				from
					create l_matrix.make (l_height, l_width)
					i := 1
				until
					i > l_height
				loop
					create l_vector.make (l_width)
					l_vector.add_scalar (undefined_double)

					l_date := l_dates.item (i)
					if has_date (l_date) then
						l_subvector := row_at (l_date)
						l_vector.put_vector (l_subvector, 1)
					end
					if a_matrix.has_date (l_date) then
						l_subvector := a_matrix.row_at (l_date)
						l_vector.put_vector (l_subvector, width + 1)
					end
					l_matrix.put_row_vector (l_vector, i)
					i := i + 1
				end
				create Result.make_from_matrix (l_matrix)
				Result.set_column_names (column_names + a_matrix.column_names)
				Result.set_dates (l_dates)
			end
		end

	trailing_return_matrix(a_window : INTEGER) : like Current is
			-- trailing return matrix
		require
			valid_window : a_window > 0 and a_window <= height
		local
			i : INTEGER
			l_raw_vector, l_returns : VECTOR
			l_dates : ROSE_LINEAR_ARRAY[ROSE_DATE]
		do
			create Result.make(height, width)
			from i := 1 until i > width loop
				l_raw_vector := column(i)
				l_returns := l_raw_vector.trailing_returns_with_partial_values (a_window)
				Result.put_column_vector (l_returns, i)
				i := i + 1
			end
			Result.set_column_names (column_names)
			create l_dates.make_from_array (dates)
			Result.set_dates (l_dates)
		end

	month_cum_return (a_col: INTEGER): like Current is
			-- Cumulative return by month (in the sense of VECTOR.cum_return)
		local
			l_first_month: INTEGER
			i: INTEGER
			l_index: INTEGER
			l_value: DOUBLE
			l_month_seq: ROSE_MONTH_SEQUENCE
			l_result: VECTOR
		do
			if dates /= Void and then dates.count > 0 then
				l_first_month := dates.first.julian_month
				create l_month_seq.make_by_months (dates.first.ccyymm, dates.last.ccyymm)
				create l_result.make_identity (l_month_seq.count)
				from i := 1	until i > dates.count loop
					l_index := dates.item (i).julian_month - l_first_month + 1
					l_value := item (i, a_col)
					if is_defined_double (l_value) then
						l_result.put (l_result.item (l_index) * (1.0 + l_value), l_index)
					end
					i := i + 1
				end
				l_result.add_scalar (-1.0)

				create Result.make_from_matrix (l_result.to_column_matrix)
				Result.set_dates (l_month_seq.monthend_dates)
				if column_names /= Void then
					Result.set_column_names (<< column_names.item (a_col) >>)
				end
			end
		ensure
			result_not_void: Result /= Void
			result_size: Result.height = dates.last.julian_month - dates.first.julian_month + 1
		end


	month_sum(a_col : INTEGER) : like Current is
			-- month sum
		local
			i : INTEGER
			l_ix, l_beg_ix, l_end_ix : INTEGER
			l_sum : DOUBLE
			l_array : ROSE_LINEAR_ARRAY[DOUBLE]
			l_vector : VECTOR
			l_month_seq : ROSE_MONTH_SEQUENCE
		do
			if dates /= Void and then dates.count > 0 then
				l_beg_ix := dates.first.julian_month
				l_end_ix := dates.last.julian_month
				create l_array.make(l_beg_ix, l_end_ix)
				create l_month_seq.make_by_months(dates.first.ccyymm, dates.last.ccyymm)
				from i := 1 until i > height loop
					l_ix := dates.item(i).julian_month
					if is_defined_double(item(i, a_col)) then
						l_sum := l_array.item(l_ix)
						l_sum := l_sum + item(i, a_col)
						l_array.put(l_sum, l_ix)
					end
					i := i + 1
				end
				create l_vector.make_from_array(l_array)
				create Result.make_from_matrix(l_vector.to_column_matrix)
				Result.set_dates(l_month_seq.monthend_dates)
				if column_names /= Void then
					Result.set_column_names(<<column_names.item(a_col)>>)
				end
			end
		end

	month_sum_matrix : like Current is
			-- month sum matrix
		local
			i : INTEGER
			l_matrix : like Current
		do
			Result := month_sum(1)
			from i := 2 until i > width loop
				l_matrix := month_sum(i)
				Result := Result.columnwise_combine(l_matrix)
				i := i + 1
			end
		end

	monthend_matrix : like Current is
			-- monthend matrix
		local
			l_dates : ROSE_LINEAR_ARRAY[ROSE_DATE]
			l_date_set : ROSE_DATE_SET
			l_month : ROSE_MONTH
			l_currency_series : like currency_series
		do
			create l_date_set.make_from_array(dates, dates.last)
			l_dates := l_date_set.monthend_dates
			Result := subset(l_dates, Void)
			from Result.dates.start until Result.dates.off loop
				create l_month.make_ccyymm(Result.dates.current_item.ccyymm)
				Result.dates.put(l_month.last, Result.dates.index)
				Result.dates.forth
			end
			if currency_series /= Void then
				l_currency_series := currency_series.subdate_series(l_dates)
				if l_currency_series /= Void then
					create l_dates.make(1, l_currency_series.count)
					from l_currency_series.start until l_currency_series.off loop
						create l_month.make_ccyymm(l_currency_series.current_date.ccyymm)
						l_dates.put(l_month.last, l_currency_series.index)
						l_currency_series.forth
					end
					create l_currency_series.make(l_currency_series, l_dates)
					Result.set_currency_series(l_currency_series)
				end
			end
		end

	currency:PERSISTENT_CURRENCY
	set_currency(a_currency:PERSISTENT_CURRENCY)
			-- Set 'currency'
		do
			currency := a_currency
		end

	currency_series : ROSE_TIME_SERIES [PERSISTENT_CURRENCY]
	set_currency_series (a_series: like currency_series)
			-- Set 'currency_series'
		do
			currency_series := a_series
		end

	currency_at (a_date: ROSE_DATE): PERSISTENT_CURRENCY
			-- Currency at 'a_date'
		do
			if currency_series /= Void then
				Result := currency_series.item_at (a_date)
			end
		end

	matrix_in_currency(a_currency : PERSISTENT_CURRENCY; a_cols : ARRAY[INTEGER]) : like Current is
			-- matrix in currency
		local
			l_cols : ROSE_HASHED_LINEAR_ARRAY[INTEGER]
			i : INTEGER
			l_series : like column_series
			l_currency_series : like currency_series
		do
			create l_cols.make_from_array(a_cols)
			Result := cloned
			from i := 1 until i > width loop
				if l_cols.has_value(i) then
					l_series := column_series_in_currency(i, a_currency)
					Result.put_column_array(l_series, i)
				end
				i := i + 1
			end
			if currency_series /= Void then
				create l_currency_series.make(currency_series, currency_series.dates)
				from l_currency_series.start until l_currency_series.off loop
					l_currency_series.put(a_currency, l_currency_series.index)
					l_currency_series.forth
				end
				Result.set_currency_series(l_currency_series)
			end
		end

	column_series_in_currency(a_col : INTEGER; a_currency : PERSISTENT_CURRENCY) : DOUBLE_TIME_SERIES is
			-- column_series in currency
		local
			i : INTEGER
			l_value : DOUBLE
		do
			if valid_column_index(a_col) then
				create Result.make_with_dates(dates)
				from i := 1 until i > height loop
					l_value := item_in_currency(i, a_col, a_currency)
					Result.put(l_value, i)
					i := i + 1
				end
			end
		end

	item_in_currency(a_row, a_col : INTEGER; a_currency : PERSISTENT_CURRENCY) : DOUBLE is
			-- item in currency
		local
			l_currency : PERSISTENT_CURRENCY
			l_date : ROSE_DATE
			l_value, l_ex_rate : DOUBLE
		do
			if valid_row_index(a_row) and valid_column_index(a_col) then
				l_date := dates.item(a_row)
				l_currency := currency_at(l_date)
				l_value := item(a_row, a_col)
				if l_currency /= Void and then a_currency /= Void then
					if l_currency = a_currency then
						Result := l_value
					else
						l_ex_rate := a_currency.exchange_rate(l_currency, l_date)
						Result := l_value * l_ex_rate
					end
				end
			end
		end

	set_dates(a_dates:ARRAY[ROSE_DATE]) is
			-- set dates
		do
			create dates.make_from_array(a_dates)
			date_set_saved:=void
			set_row_names(date_strings(dates))
		end

	dates : ROSE_HASHED_LINEAR_ARRAY[ROSE_DATE]--dates

	has_date(a_date : ROSE_DATE) : BOOLEAN is
			-- has date
		do
			if a_date /= Void then
				Result := dates /= Void and then dates.has_value(a_date)
			end
		end

	date_set:ROSE_DATE_SET is
		--date_Set
		do
			result:=date_set_saved
			if Result=void then
				create date_set_saved.make_from_array (dates, void)
				result:=date_set_saved
			end
		end
	date_set_saved:like date_set --date_set_saved


	date(a_row_index : INTEGER) : ROSE_DATE is
			-- date
		do
			if valid_row_index(a_row_index) then
				Result := dates.item(a_row_index)
			end
		end

	has_column(a_column_name : STRING) : BOOLEAN is
			-- has column
		do
			if a_column_name /= Void then
				Result := column_table /= Void and then column_table.has(a_column_name)
			end
		end

	column_name(a_col : INTEGER) : STRING is
			-- column_name
		do
			if valid_column_index(a_col) and then column_names /= Void then
				Result := column_names.item(a_col)
			end
		end

	set_monthly_mode is
			-- set monthly mode
		local
			l_dates:ROSE_LINEAR_ARRAY[ROSE_DATE]
		do
			create l_dates.make_empty
			from dates.start until dates.off loop
				l_dates.extend(dates.current_item.rose_month.last)
				dates.forth
			end
			set_dates(l_dates)
		end
	previous_months:ROSE_MONTH_SEQUENCE is
			-- Previous months
		do
			create result.make(1,dates.count)
			from  dates.start until dates.off loop
				result.put(dates.current_item.rose_month.previous_month,dates.index)
				dates.forth
			end
		end

	date_strings(a_dates : ARRAY[ROSE_DATE]) : ROSE_LINEAR_ARRAY[STRING] is
			-- Display dates
		local
			i : INTEGER
			l_date : ROSE_DATE
			l_model_date : MODEL_DATE
		do
			if a_dates /= Void then
				create Result.make(1, a_dates.count)
				from i := a_dates.lower until i > a_dates.upper loop
					l_date := a_dates.item(i)
					l_model_date ?= l_date
					if l_date /= Void then
						if l_model_date /= Void then
							Result.put(l_model_date.out, i - a_dates.lower + 1)
						else
							if monthly_mode  then
								Result.put(l_date.ccyymm.out, i - a_dates.lower + 1)
							else
								Result.put(l_date.ccyymmdd.out, i - a_dates.lower + 1)
							end
						end
					end
					i := i + 1
				end
			end
		ensure
			result_if_input: (Result = Void) = (a_dates = Void)
			count: Result /= Void implies Result.count = a_dates.count
		end

	dates_from_names(a_names:ARRAY[STRING]):ROSE_LINEAR_ARRAY[ROSE_DATE] is
			-- Parsed dates as day (CCYYMMDD) or month (CCYYMM) strings
		local
			l_date:ROSE_DATE
			l_date_int:INTEGER
			l_name:STRING
			i:INTEGER
		do
			if a_names /= Void then
				create Result.make(1,a_names.count)
				from i:=1 until i>a_names.count loop
					l_name:=a_names.item(i)
					if l_name /= Void and then l_name.is_integer then
						l_date_int := l_name.to_integer
						if l_date_int < 1000000 then -- 6 digit = month, 8 digit = day
							l_date := (create {ROSE_MONTH}.make_ccyymm (l_date_int)).last
						else
							create l_date.make_ccyymmdd (l_date_int)
						end
						Result.put (l_date, i)
					end
					i:=i+1
				end
			end
		ensure
			result_if_input: (Result = Void) = (a_names = Void)
			count: Result /= Void implies Result.count = a_names.count
		end

	date_order(a_date1, a_date2 : ROSE_DATE) : BOOLEAN is
			--a_date1 < a_date2?
		do
			if a_date1 /= Void and a_date2 /= Void then
				Result := a_date1 < a_date2
			end
		end

	monthly_mode:BOOLEAN
			-- In monthly mode?

	itemwise_product(a_other: ABSTRACT_READABLE_MATRIX):DATED_MATRIX is
			--DATED_MATRIX with i,jth element the product of item(i,j) and a_other[i,j]
		local
			i,j : INTEGER
		do
			create Result.make(height,a_other.width)
			from i := 1 until i > height loop
				from j := 1 until j > a_other.width loop
					Result [i, j] := Current [i, j] * a_other [i, j]
					j := j + 1
				end
				i := i + 1
			end
		end
feature {NONE} -- Implementation

	anchor: like current
			-- anchor
		do
		end

end
