note

	description:

		"Date intervaled unitized value"

	author: "Mark Howard"
	last_editor: "$Author: it $"
	date: "$Date: 2014/10/29 $"
	revision: "$Revision: #8 $"
	path: "$File: //rose/source/kernel/dated_unitized_value.e $"

class
	DATED_UNITIZED_VALUE[G->NUMERIC]
inherit
	ROSE_UNITIZED_VALUE[G]
		redefine
			infix "+",
			infix "-",
			infix "*",
			infix "/"
		end
	ROSE_IEEE_UNDEFINED
create
	make,
	make_with_units,
	make_from_rose_unitized,
	default_create,
	make_pure

feature {NONE} --Initialization
	make (a_value : G; a_units : ROSE_UNITS; a_start, a_end: ROSE_DATE)
			--create from a_value, a_units and a date interval
		do
			make_with_units(a_value,a_units)
			start_date := a_start
			end_date := a_end
		end
	make_pure(a_value : G)
			--Make with pure units and no interval
		do
			make_with_units(a_value,Void)
		end
	make_from_rose_unitized(a_source : ROSE_UNITIZED_VALUE[G])
			--create from another ROSE_UNITIZED_VALUE with Void date
		require
			a_source_attached: a_source /= Void
		do
			make_with_units (a_source.value, a_source.units)
		end

feature -- Element modification

	set_start_and_end_date(a_start_date, a_end_date: ROSE_DATE)
			--set start and end date
		do
			start_date := a_start_date
			end_date := a_end_date
		end

	set_available_date(a_available_date: ROSE_DATE)
	-- Self explanatory.
	do
		available_date := a_available_date
	end

	set_dates(a_start_date, a_end_date, a_available_date: ROSE_DATE)
	-- Sets the corresponding attributes.
	do
		start_date := a_start_date
		end_date := a_end_date
		available_date := a_available_date
	end

feature --Access
	start_date : ROSE_DATE		-- Start date of value.
	end_date : ROSE_DATE  		-- End date of value.
	available_date: ROSE_DATE	-- Date at which we became aware of this data.

feature -- Query

	date_intersection(a_other : DATED_UNITIZED_VALUE[G]) : TUPLE[start_date : ROSE_DATE;end_date : ROSE_DATE] is
			--date interval representing the intersection of ours and that of a_other
		local
			l_start,l_end : ROSE_DATE
		do
			if start_date /= Void then
				if a_other.start_date /= Void then
					l_start := start_date.max(a_other.start_date)
				else
					l_start := start_date
				end
			else
				l_start := a_other.start_date
			end
			if end_date /= Void then
				if a_other.end_date /= Void then
					l_end := end_date.min(a_other.end_date)
				else
					l_end := end_date
				end
			else
				l_end := a_other.end_date
			end
			Result := [l_start,l_end]
		end

	is_defined : BOOLEAN
			--is value defined?
		do
			if attached {DOUBLE} value as l_double then
				Result := is_defined_double(l_double)
			end
		end

feature -- Basic operations

	infix "+" (a_other : DATED_UNITIZED_VALUE[G]) : DATED_UNITIZED_VALUE[G] is
		--Adding two unitized values will preserve the units if both sides have the same units
		--otherwise you end up with unknown units.  This case is not an error, it just causes us to lose the units, so
		--we do not handle it with a precondition.
		local
			l_mismatched_units: MISMATCHED_UNITS
			l_interval : like date_intersection
		do
			l_interval := date_intersection(a_other)
			if units = a_other.units then
				create Result.make(value + a_other.value,units,l_interval.start_date,l_interval.end_date)
			else
				create l_mismatched_units.make(units, a_other.units)
				create Result.make(value + a_other.value, l_mismatched_units,l_interval.start_date,l_interval.end_date)
			end
		end
	infix "-" (a_other : DATED_UNITIZED_VALUE[G]) : DATED_UNITIZED_VALUE[G] is
		--Subtracting two unitized values will preserve the units if both sides have the same units
		--otherwise you end up with unknown units.  This case is not an error, it just causes us to lose the units, so
		--we do not handle it with a precondition.
		local
			l_mismatched_units: MISMATCHED_UNITS
			l_interval : like date_intersection
		do
			l_interval := date_intersection(a_other)
			if units = a_other.units then
				create Result.make(value - a_other.value,units,l_interval.start_date,l_interval.end_date)
			else
				create l_mismatched_units.make(units, a_other.units)
				create Result.make(value - a_other.value, l_mismatched_units,l_interval.start_date,l_interval.end_date)
			end
		end
	infix "*" (a_other : DATED_UNITIZED_VALUE[G]) : DATED_UNITIZED_VALUE[G] is
		--multiply values and units
		local
			l_interval : like date_intersection
		do
			l_interval := date_intersection(a_other)
			create Result.make(value * a_other.value,units * a_other.units,l_interval.start_date,l_interval.end_date)
		end
	infix "/" (a_other : DATED_UNITIZED_VALUE[G]) : DATED_UNITIZED_VALUE[G] is
		--ratio of values and units
		local
			l_interval : like date_intersection
		do
			l_interval := date_intersection(a_other)
			create Result.make(value / a_other.value,units / a_other.units,l_interval.start_date,l_interval.end_date)
		end
end
