note

	description:

		"ROSE_CONTEXT version of ECONOMIC_SECTOR_GROUP"

	author: "Mark Howard"
	last_editor: "$Author: mk $"
	date: "$Date: 2012/06/22 $"
	revision: "$Revision: #24 $"
	path: "$File: //rose/source/kernel/investment_entities/economic_sector_group.e $"

class
	ECONOMIC_SECTOR_GROUP

inherit
	INVESTMENT_ENTITY
		undefine
			name_identifier_key
		end
	PERSISTENT_ECONOMIC_SECTOR_GROUP
		undefine
			context,
			make,
			make_context,
			name_identifier_key,
			debug_output
		end

create
	make, make_anonymous

feature -- Basic operation

	name_identifier_key : STRING is
		once
			Result := "name"
		end

	register is
			-- Register this entity.
		do
		end

	count : INTEGER is
			-- Number of economic sectors returned by `economic_sectors'.
		local
			l_econ_sectors: like economic_sectors
		do
			l_econ_sectors := economic_sectors
			if l_econ_sectors /= Void then
				Result := l_econ_sectors.count
			end
		end

	economic_sectors : ROSE_LINEAR_ARRAY[like context.economic_sector_template] is
			-- Economic sectors within this group
		local
			l_econ_sectors: like economic_sector_group_to_economic_sector_history
		do
			if economic_sectors_saved = Void then
				l_econ_sectors := economic_sector_group_to_economic_sector_history
				if l_econ_sectors /= Void then
					create economic_sectors_saved.make_empty
					from l_econ_sectors.start until l_econ_sectors.off loop
						economic_sectors_saved.extend(l_econ_sectors.current_item.entity)
						l_econ_sectors.forth
					end
				end
			end
			Result := economic_sectors_saved
		end

	has(a_economic_sector : like context.economic_sector_template) : BOOLEAN 
			--Does this group have `a_economic_sector'?
		local
			l_table : like economic_sector_table
		do
			l_table := economic_sector_table
			if l_table /= Void then
				l_table.search(a_economic_sector)
				Result := l_table.found
			end
		end

	economic_sector_table : HASH_TABLE[INTEGER, like context.economic_sector_template] is
		local
			l_econ_sectors: like economic_sectors
			l_econ_sector : like context.economic_sector_template
		do
			if economic_sector_table_saved = Void then
				l_econ_sectors := economic_sectors
				if l_econ_sectors /= Void then
					create economic_sector_table_saved.make(5)
					from l_econ_sectors.start until l_econ_sectors.off loop
						l_econ_sector := l_econ_sectors.current_item
						economic_sector_table_saved.put(l_econ_sectors.index, l_econ_sector)
						l_econ_sectors.forth
					end
				end
			end
			Result := economic_sector_table_saved
		end

	economic_sector_names : ROSE_LINEAR_ARRAY[STRING] is
		local
			l_econ_sectors: like economic_sectors
			l_econ_sector : like context.economic_sector_template
			l_date : ROSE_DATE
		do
			if economic_sector_names_saved = Void then
				l_econ_sectors := economic_sectors
				if l_econ_sectors /= Void then
					create l_date.make_today
					create economic_sector_names_saved.make_empty
					from l_econ_sectors.start until l_econ_sectors.off loop
						l_econ_sector := l_econ_sectors.current_item
						economic_sector_names_saved.extend(l_econ_sector.name_at(l_date))
						l_econ_sectors.forth
					end
				end
			end
			Result := economic_sector_names_saved
		end

feature {NONE} -- Implementation

	economic_sector_table_saved : like economic_sector_table
	economic_sector_names_saved : like economic_sector_names
	economic_sectors_saved : like economic_sectors
end
