indexing
	description: "Three State Variable"
	author: "it"
	date: "$Date: 2008/12/01 $"
	revision: "$Revision: #3 $"

class
	EGCDB_TRISTATE

create

	make_true, make_false, make_undefined, make_from_string

feature {NONE} -- Initialization

	make_from_string (a_string: STRING) is
			-- Create a new variable and set its valua appropriately
		do
			if a_string = void then
				make_undefined
			else
				if a_string.is_empty then
					make_undefined
				elseif a_string.is_equal ("Yes") then
					make_true
				else
					make_false
				end
			end
		end

	make_true is
			-- Create a new variable and set it to true.
		do
			set_true
		ensure
			is_true: is_true
		end

	make_false is
			-- Create a new variable and set it to false.
		do
			set_false
		ensure
			is_false: is_false
		end

	make_undefined is
			-- Create a new variable and set it to undefined.
		do
			set_undefined
		ensure
			is_undefined: is_undefined
		end

feature -- Status report

	is_true: BOOLEAN is
			-- Is variable true?
		do
			Result := (value = 'T')
		end

	is_false: BOOLEAN is
			-- Is variable false?
		do
			Result := (value = 'F')
		end

	is_undefined: BOOLEAN is
			-- Is variable undefined?
		do
			Result := not (is_true or is_false)
		end

feature -- Status setting

	set_true is
			-- Set variable to true.
		do
			value := 'T'
		ensure
			is_true: is_true
		end

	set_false is
			-- Set variable to false.
		do
			value := 'F'
		ensure
			is_false: is_false
		end

	set_undefined is
			-- Set variable to undefined.
		do
			value := 'U'
		ensure
			is_undefined: is_undefined
		end

feature {NONE} -- Implementation

	value: CHARACTER
			-- Internal state

invariant

	at_least_one: is_true or is_false or is_undefined
	is_true_definition: is_true = not (is_false or is_undefined)
	is_false_definition: is_false = not (is_true or is_undefined)
	is_undefined_definition: is_undefined = not (is_true or is_false)

end -- class EGCDB_TRISTATE
