indexing

	description: "Count the number of Eiffel objects"

	author: "Randy John"
	last_editor: "$Author: er $"
	date: "$Date: 2014/02/26 $"
	revision: "$Revision: #3 $"
	path: "$File: //rose/source/Kernel/infrastructure/containers/classic/eiffel_object_counter.e $"

class	EIFFEL_OBJECT_COUNTER

inherit

	INTERNAL
		export
			{NONE} all
		end

	MEMORY
		export
			{ANY} memory_count_map
		end

	ANY

feature -- Access

	counts_and_sizes: HASH_TABLE[PAIR[INTEGER_64, INTEGER_64], STRING] is
	-- Returns the counts and sizes of all objects in the entire system.
	-- The size includes the overhead (hard-coded for 8 bytes).
	-- WARNING - Some, but not all locals will be seen by this method.
	-- It include 'once' objects.

	-- Usage:
	--		inherit EIFFEL_OBJECT_COUNTER
	--		l_cas: HASH_TABLE[PAIR[INTEGER_64, INTEGER_64], STRING]
	--
	--		l_cas := counts_and_sizes
	--		dump_cas_by_size_desc(l_cas, 0)
	--		-- or
	--		dump_cas_by_size_asc(l_cas, 0)
	--		-- or
	--		dump_cas_by_count_desc(l_cas, 0)
	--		-- or
	--		dump_cas_by_count_asc(l_cas, 0)
	--		l_cas := Void	-- We don't want to count it next time and it's largish.
	local
		l_memory_map: HASH_TABLE[ARRAYED_LIST[ANY], INTEGER]
		l_list: ARRAYED_LIST[ANY]
		l_total_size: INTEGER_64
		l_any: ANY
		l_pair: PAIR[INTEGER_64, INTEGER_64]
	do
		l_memory_map := memory_map
		create Result.make(1000)

		from
			l_memory_map.start
		until
			l_memory_map.off
		loop
			l_list := l_memory_map.item_for_iteration
			l_any := l_list.first
			l_total_size := l_list.count * 8

			from
				l_list.start
			until
				l_list.off
			loop
				l_total_size := l_total_size + physical_size(l_list.item)
				l_list.forth
			end

			create l_pair.make(l_list.count, l_total_size)
			Result.put(l_pair, l_any.generating_type)
			l_memory_map.forth
		end

		l_list := Void
		l_memory_map := Void
	end

	count_objects (a_object: ANY): HASH_TABLE [INTEGER, STRING] is
			-- Count the number of different objects accessible from 'a_object'
			-- and return the counts by type in Result.
		local
			l_seen: HASH_TABLE [BOOLEAN, POINTER]
			l_collecting: BOOLEAN
		do
			create Result.make (1000)
			create l_seen.make (1000)
			l_collecting := collecting
			collection_off
			count_objects_internal(a_object, Result, l_seen, True)
			if l_collecting then
				collection_on
			end
		rescue
			if l_collecting then
				collection_on
			end
		end

	object_counts : HASH_TABLE [INTEGER, STRING] is
			-- Count the number of different objects in the entire system
			-- and return the counts by type in Result.
		local
			l_seen: HASH_TABLE [BOOLEAN, POINTER]
			l_collecting: BOOLEAN
			a: ARRAY [ANY]
			spec,l_instances : SPECIAL [ANY]
			i, j : INTEGER
			l_dyn_type: INTEGER
			l_tot: INTEGER
			l_special: SPECIAL[ANY]
		do
			create Result.make (1000)
			create l_seen.make (1000)
			l_collecting := collecting
			collection_off
			create a.make(0, 200)
			spec := a.area
			l_dyn_type := dynamic_type (spec)
			from i := 1 until i > 10000 loop
				l_instances := find_instance_of (i, l_dyn_type)
				if l_instances.count > 0 then
--					Result.put(l_instances.count,internal.type_name_of_type(i))
					Result.put(l_instances.count, l_instances.item(0).generating_type)
					if is_special_type(i)
					then
						from
							j := 0
							l_tot := 0
						until
							j>=l_instances.count
						loop
							l_special ?= l_instances.item(j)
							l_tot := l_tot + l_special.count
							j := j + 1
						end

						Result.put(l_tot, l_instances.item(0).generating_type + " count of elements")
					end
				end
				if i \\ 100 = 0 then
					print(i.out + "%N")
				end
				i := i + 1
			end
			if l_collecting then
				collection_on
			end
		rescue
			if l_collecting then
				collection_on
			end
		end

	dump_cas_by_size_asc(a_cas: HASH_TABLE[PAIR[INTEGER_64, INTEGER_64], STRING]; a_count_greater: INTEGER) is
	-- Prints the CAS by ascending count of objects.
	do
		dump_cas(a_cas, a_count_greater, -1, agent sort_cas_by_size_asc)
	end

	dump_cas_by_size_desc(a_cas: HASH_TABLE[PAIR[INTEGER_64, INTEGER_64], STRING]; a_count_greater: INTEGER) is
	-- Prints the CAS by descending count of objects.
	do
		dump_cas(a_cas, a_count_greater, -1, agent sort_cas_by_size_desc)
	end

	dump_cas_by_size_asc2(a_cas: HASH_TABLE[PAIR[INTEGER_64, INTEGER_64], STRING]; a_size_greater: INTEGER) is
	-- Prints the CAS by ascending total size of objects.
	do
		dump_cas(a_cas, -1, a_size_greater, agent sort_cas_by_size_asc)
	end

	dump_cas_by_size_desc2(a_cas: HASH_TABLE[PAIR[INTEGER_64, INTEGER_64], STRING]; a_size_greater: INTEGER) is
	-- Prints the CAS by descending total size of objects.
	do
		dump_cas(a_cas, -1, a_size_greater, agent sort_cas_by_size_desc)
	end

	dump_cas(
		a_cas: HASH_TABLE[PAIR[INTEGER_64, INTEGER_64], STRING];
		a_count_greater: INTEGER;
		a_size_greater: INTEGER;
		a_comparison: FUNCTION[ANY, TUPLE[PAIR[STRING, PAIR[INTEGER_64, INTEGER_64]], PAIR[STRING, PAIR[INTEGER_64, INTEGER_64]]], BOOLEAN]) is
	local
		l_table: ROSE_LINEAR_ARRAY[PAIR[STRING, PAIR[INTEGER_64, INTEGER_64]]]
		l_pair: PAIR[STRING, PAIR[INTEGER_64, INTEGER_64]]
	do
		create l_table.make_empty
		from
			a_cas.start
		until
			a_cas.off
		loop
			if (a_cas.item_for_iteration.first>a_count_greater and a_count_greater/=-1) or
			   (a_cas.item_for_iteration.second>a_size_greater and a_size_greater/=-1)
			then
				create l_pair.make(a_cas.key_for_iteration, a_cas.item_for_iteration)
				l_table.extend(l_pair)
			end

			a_cas.forth
		end

		l_table.sort_by(a_comparison)

		from
			l_table.start
		until
			l_table.off
		loop
			print(l_table.current_item.second.first.out + ", " + l_table.current_item.second.second.out + "%T" + l_table.current_item.first + "%N")
			l_table.forth
		end
	end

	dump_mcm_by_size_desc(a_mcm: HASH_TABLE[INTEGER, INTEGER]; a_count_greater: INTEGER) is
	do
		dump_mcm(a_mcm, a_count_greater, agent sort_by_size_desc)
	end

	dump_mcm(
		a_mcm: HASH_TABLE[INTEGER, INTEGER];
		a_count_greater: INTEGER;
		a_comparison: FUNCTION [ANY, TUPLE [PAIR[STRING, INTEGER], PAIR[STRING, INTEGER]], BOOLEAN]) is
	local
		l_table: ROSE_LINEAR_ARRAY[PAIR[STRING, INTEGER]]
		l_pair: PAIR[STRING, INTEGER]
	do
		create l_table.make_empty
		from
			a_mcm.start
		until
			a_mcm.off
		loop
			if a_mcm.item_for_iteration>a_count_greater
			then
				create l_pair.make(class_name_of_type(a_mcm.key_for_iteration), a_mcm.item_for_iteration)
				l_table.extend(l_pair)
			end

			a_mcm.forth
		end

		l_table.sort_by(a_comparison)

		from
			l_table.start
		until
			l_table.off
		loop
			print(l_table.current_item.second.out + "%T" + l_table.current_item.first + "%N")
			l_table.forth
		end
	end

	dump_eoc_hash_table_by_name(a_ht: HASH_TABLE [INTEGER, STRING]; a_count_greater: INTEGER) is
	do
		dump_eoc_hash_table(a_ht, a_count_greater, agent sort_by_name)
	end

	dump_eoc_hash_table_by_size_asc(a_ht: HASH_TABLE [INTEGER, STRING]; a_count_greater: INTEGER) is
	do
		dump_eoc_hash_table(a_ht, a_count_greater, agent sort_by_size_asc)
	end

	dump_eoc_hash_table_by_size_desc(a_ht: HASH_TABLE [INTEGER, STRING]; a_count_greater: INTEGER) is
	do
		dump_eoc_hash_table(a_ht, a_count_greater, agent sort_by_size_desc)
	end

	dump_eoc_hash_table(
		a_ht: HASH_TABLE [INTEGER, STRING];
		a_count_greater: INTEGER;
		a_comparison: FUNCTION [ANY, TUPLE [PAIR[STRING, INTEGER], PAIR[STRING, INTEGER]], BOOLEAN]) is
	-- A simple routine to dump the result of this class.
	local
		l_table: ROSE_LINEAR_ARRAY[PAIR[STRING, INTEGER]]
		l_pair: PAIR[STRING, INTEGER]
	do
		create l_table.make_empty
		from
			a_ht.start
		until
			a_ht.off
		loop
			if a_ht.item_for_iteration>a_count_greater
			then
				create l_pair.make(a_ht.key_for_iteration, a_ht.item_for_iteration)
				l_table.extend(l_pair)
			end

			a_ht.forth
		end

		l_table.sort_by(a_comparison)

		from
			l_table.start
		until
			l_table.off
		loop
			print(l_table.current_item.second.out + "%T" + l_table.current_item.first + "%N")
			l_table.forth
		end
	end


feature {NONE}	-- Implementation.

	sort_by_name(a_pair1, a_pair2: PAIR[STRING, INTEGER]): BOOLEAN is
	do
		Result := a_pair1.first<a_pair2.first
	end

	sort_by_size_asc(a_pair1, a_pair2: PAIR[STRING, INTEGER]): BOOLEAN is
	-- This is useful when you display more classes than your screen can handle.
	-- The big ones will be as the bottom.
	do
		Result := a_pair1.second<a_pair2.second
	end

	sort_by_size_desc(a_pair1, a_pair2: PAIR[STRING, INTEGER]): BOOLEAN is
	do
		Result := a_pair1.second>a_pair2.second
	end

	sort_cas_by_size_asc(a_pair1, a_pair2: PAIR[STRING, PAIR[INTEGER_64, INTEGER_64]]): BOOLEAN is
	do
		Result := a_pair1.second.second<a_pair2.second.second
	end

	sort_cas_by_size_desc(a_pair1, a_pair2: PAIR[STRING, PAIR[INTEGER_64, INTEGER_64]]): BOOLEAN is
	do
		Result := a_pair1.second.second>a_pair2.second.second
	end

	sort_cas_by_count_asc(a_pair1, a_pair2: PAIR[STRING, PAIR[INTEGER_64, INTEGER_64]]): BOOLEAN is
	do
		Result := a_pair1.second.first<a_pair2.second.first
	end

	sort_cas_by_count_desc(a_pair1, a_pair2: PAIR[STRING, PAIR[INTEGER_64, INTEGER_64]]): BOOLEAN is
	do
		Result := a_pair1.second.first>a_pair2.second.first
	end

	count_objects_internal(
		a_object: ANY;
		a_result: HASH_TABLE[INTEGER, STRING];
		a_seen: HASH_TABLE[BOOLEAN, POINTER];
		a_recursive: BOOLEAN) is
			-- Countabilize 'a_object' in 'a_result' if not already seen (remembered in 'a_seen').
		local
			i: INTEGER
			l_field_count: INTEGER
			l_special_boolean: SPECIAL[BOOLEAN]
			l_special_character: SPECIAL[CHARACTER]
			l_special_integer: SPECIAL[INTEGER]
			l_special_integer_8: SPECIAL[INTEGER_8]
			l_special_integer_16: SPECIAL[INTEGER_16]
			l_special_integer_64: SPECIAL[INTEGER_64]
			l_special_natural_8: SPECIAL[NATURAL_8]
			l_special_natural_16: SPECIAL[NATURAL_16]
			l_special_natural_32: SPECIAL[NATURAL_32]
			l_special_natural_64: SPECIAL[NATURAL_64]
			l_special_real: SPECIAL[REAL]
			l_special_double: SPECIAL[DOUBLE]
			l_special_pointer: SPECIAL[POINTER]
			l_special_any: SPECIAL[ANY]
		do
			if a_object = Void then
				increment (a_result, c_void, 1)
			elseif a_seen/=Void and then a_seen.item ($a_object) then
				--| Do nothing
			else
				if a_seen/=Void
				then
					a_seen.put (True,$a_object)
				end

				increment (a_result, a_object.generating_type, 1)
				if is_special (a_object) then
					l_special_boolean ?= a_object
					l_special_character ?= a_object
					l_special_integer ?= a_object
					l_special_integer_8 ?= a_object
					l_special_integer_16 ?= a_object
					l_special_integer_64 ?= a_object
					l_special_natural_8 ?= a_object
					l_special_natural_16 ?= a_object
					l_special_natural_32 ?= a_object
					l_special_natural_64 ?= a_object
					l_special_real ?= a_object
					l_special_double ?= a_object
					l_special_pointer ?= a_object
					l_special_any ?= a_object

					if l_special_boolean /= Void then
						increment (a_result, c_boolean_special, l_special_boolean.count)
					elseif l_special_character /= Void then
						increment (a_result, c_character_special, l_special_character.count)

					elseif l_special_integer /= Void then
						increment (a_result, c_integer_special, l_special_integer.count)
					elseif l_special_integer_8 /= Void then
						increment (a_result, c_integer_8_special, l_special_integer_8.count)
					elseif l_special_integer_16 /= Void then
						increment (a_result, c_integer_16_special, l_special_integer_16.count)
					elseif l_special_integer_64 /= Void then
						increment (a_result, c_integer_64_special, l_special_integer_64.count)

					elseif l_special_natural_8 /= Void then
						increment (a_result, c_natural_8_special, l_special_natural_8.count)
					elseif l_special_natural_16 /= Void then
						increment (a_result, c_natural_16_special, l_special_natural_16.count)
					elseif l_special_natural_32 /= Void then
						increment (a_result, c_natural_32_special, l_special_natural_32.count)
					elseif l_special_natural_64 /= Void then
						increment (a_result, c_natural_64_special, l_special_natural_64.count)

					elseif l_special_real /= Void then
						increment (a_result, c_real_special, l_special_real.count)
					elseif l_special_double /= Void then
						increment (a_result, c_double_special, l_special_double.count)
					elseif l_special_pointer /= Void then
						increment (a_result, c_pointer_special, l_special_pointer.count)
					elseif l_special_any /= Void then
						handle_special (l_special_any, a_result, a_seen, a_recursive)
					else
						increment (a_result, c_unknown_special, 1)
					end
				else
					l_field_count := field_count(a_object)

					from
						i := 1
					until
						i > l_field_count
					loop
						inspect field_type(i, a_object)
						when Character_type then
							increment(a_result, c_character, 1)
						when Boolean_type then
							increment(a_result, c_boolean, 1)
						when Integer_type then
							increment(a_result, c_integer, 1)
						when Real_type then
							increment(a_result, c_real, 1)
						when Double_type then
							increment(a_result, c_double, 1)
						when Pointer_type then
							increment(a_result, c_pointer, 1)
						when Reference_type then
							if a_recursive
							then
								count_objects_internal(field(i, a_object), a_result, a_seen, a_recursive)
							end
						else
							increment (a_result, c_unknown, 1)
						end

						i := i + 1
					end
				end
			end
		end

	list_dynamic_types_internal (a_object: ANY; a_result: HASH_TABLE [STRING,INTEGER]; a_seen: HASH_TABLE [BOOLEAN, POINTER]) is
			-- Countabilize 'a_object' in 'a_result' if not already seen (remembered in 'a_seen').
		local
			i: INTEGER
			l_field_count: INTEGER
			l_type : INTEGER
			l_special : SPECIAL[ANY]
			l_object : ANY
		do
			if a_object = Void then
				--do nothing
			elseif a_seen.item ($a_object) then
				--| Do nothing
			else
				a_seen.put (True,$a_object)
				l_type := dynamic_type(a_object)
				a_result.search(l_type)
				if not a_result.found then
					a_result.put(a_object.generator,l_type)
				end
				if is_special (a_object) then
					l_special ?= a_object
					from
						i := 0
					until
						i >= l_special.count
					loop
						l_object := l_special.item (i)
						if l_object /= Void then
							list_dynamic_types_internal (l_object, a_result, a_seen)
						end
						i := i + 1
					end
				else
					l_field_count := field_count (a_object)
					from
						i := 1
					until
						i > l_field_count
					loop
						inspect field_type (i, a_object)
						when Reference_type then
							list_dynamic_types_internal (field (i, a_object), a_result, a_seen)
						end
						i := i + 1
					end
				end
			end
		end

	handle_special(
		a_special: SPECIAL[ANY];
		a_result: HASH_TABLE[INTEGER,STRING];
		a_seen: HASH_TABLE[BOOLEAN, POINTER];
		a_recursive: BOOLEAN) is
			-- Countabilize 'a_special' in 'a_result'.
		local
			i, l_voids: INTEGER
			l_object: ANY
		do
			increment (a_result, c_reference_special, a_special.count)

			from
				i := 0
			until
				i >= a_special.count
			loop
				l_object := a_special.item (i)
				if l_object = Void then
						--| Optimize counting of Voids...
					l_voids := l_voids + 1
				else
					count_objects_internal (l_object, a_result, a_seen, a_recursive)
				end
				i := i + 1
			end

			increment (a_result, c_void_special, l_voids)
		end

	increment (a_result: HASH_TABLE [INTEGER,STRING]; a_key: STRING; a_count: INTEGER) is
			-- Increment count for 'a_key' in 'a_result' by 'a_count'.
		do
			a_result.force (a_result.item (a_key) + a_count, a_key)
		end

	new_object (i : INTEGER): ANY is
			-- New object of type `i'
		local
			l_failed : BOOLEAN
		do
			if not l_failed then
				Result := internal.new_instance_of (i)
			end
		rescue
			l_failed := true
			retry
		end

	internal : INTERNAL is
		once
			create Result
		end

feature {NONE} -- Constants

	c_character: STRING is "CHARACTER"
	c_boolean: STRING is "BOOLEAN"
	c_integer: STRING is "INTEGER"
	c_double: STRING is "DOUBLE"
	c_real: STRING is "REAL"
	c_pointer: STRING is "POINTER"
	c_reference: STRING is "REFERENCE"
	c_unknown: STRING is "Unknown???"
	c_void: STRING is "Void"
	c_character_special: STRING is "CHARACTER in SPECIAL"
	c_boolean_special: STRING is "BOOLEAN in SPECIAL"
	c_integer_special: STRING is "INTEGER in SPECIAL"
	c_integer_8_special: STRING is "INTEGER_8 in SPECIAL"
	c_integer_16_special: STRING is "INTEGER_16 in SPECIAL"
	c_integer_64_special: STRING is "INTEGER_64 in SPECIAL"
	c_natural_8_special: STRING is "NATURAL_8 in SPECIAL"
	c_natural_16_special: STRING is "NATURAL_16 in SPECIAL"
	c_natural_32_special: STRING is "NATURAL_32 in SPECIAL"
	c_natural_64_special: STRING is "NATURAL_64 in SPECIAL"
	c_double_special: STRING is "DOUBLE in SPECIAL"
	c_real_special: STRING is "REAL in SPECIAL"
	c_pointer_special: STRING is "POINTER in SPECIAL"
	c_reference_special: STRING is "REFERENCE in SPECIAL"
	c_unknown_special: STRING is "Unknown??? in SPECIAL"
	c_void_special: STRING is "Void in SPECIAL"
			-- These are defined as constants for optimization
			-- Eiffel won't create new strings for each count that way

end
