indexing
	description: "Contains all pieces of configuration items used to create an ELOG_LOGGER"

	author: "Emmanuel Bouyer"
	last_editor: "$Author: e2 $"
	date: "$Date: 2014/07/04 $"
	revision: "$Revision: #2 $"
	path: "$File: //rose/source/kernel/infrastructure/logging/elog_configuration.e $"

class ELOG_CONFIGURATION

create
	make

feature {NONE} -- Initialization

	make
			-- Configuration with good default values
		do
			filename := "logger.log"
			create exclude_filters.make (1, 0)
			create include_filters.make (1, 0)
			logger_name := "initial_logger"
			do_flush := True
			use_std_out := True
			use_utc := True
			should_append := True
			synchronous_write := True
		end

feature -- Setting

	set_logger_name (a_logger_name: STRING)
			-- Set logger_name to `a_logger_name'
		require
			a_logger_name_not_void: a_logger_name /= Void
			a_logger_name_not_empty: not a_logger_name.is_empty
		do
			logger_name := a_logger_name.twin
		ensure
			logger_name_set: logger_name ~ a_logger_name
			logger_name_copied: logger_name /= a_logger_name
		end

	set_filename (a_filename: STRING)
			-- Set filename to `a_filename'
		require
			a_filename_not_void: a_filename /= Void
			a_filename_not_empty: not a_filename.is_empty
		do
			filename := a_filename.twin
		ensure
			filename_set: filename ~ a_filename
			filename_copied: filename /= a_filename
		end

	set_exclude_filters (a_exclude_filters: ARRAY [STRING])
			-- Set exclude_filters to `a_exclude_filters'
		require
			a_exclude_filters_not_void: a_exclude_filters /= Void
			no_void_item: not a_exclude_filters.has (Void)
		do
			exclude_filters := a_exclude_filters.twin
		ensure
			exclude_filters_set: exclude_filters ~ a_exclude_filters
			exclude_filters_copied: exclude_filters /= a_exclude_filters
		end

	set_include_filters (a_include_filters: ARRAY [STRING])
			-- Set include_filters to `a_include_filters'
		require
			a_include_filters_not_void: a_include_filters/= Void
			no_void_item: not a_include_filters.has (Void)
		do
			include_filters := a_include_filters.twin
		ensure
			include_filters_set: include_filters ~ a_include_filters
			include_filters_copied: include_filters /= a_include_filters
		end

	set_use_utc (a_use_utc: BOOLEAN)
			-- Set use_utc to `a_use_utc'
		do
			use_utc := a_use_utc
		ensure
			use_utc_set: use_utc = a_use_utc
		end

	set_use_std_out (a_use_std_out: BOOLEAN)
			-- Set use_std_out to `a_use_std_out'
		do
			use_std_out := a_use_std_out
		ensure
			use_std_out_set: use_std_out = a_use_std_out
		end

	set_max_size (a_max_size: INTEGER)
			-- Set max_size to `a_max_size'
			-- Set max_defaults to -1 if max_size is -1
			-- otherwise set max_defaults to 0 if it was -1
		require
			a_max_size_greater_than_minus_one: a_max_size >= -1
		do
			max_size := a_max_size
			if max_size = -1 then
				max_backups := -1
			elseif max_backups = -1 then
				max_backups := 0
			end
		ensure
			max_size_set: max_size = a_max_size
		end

	set_max_backups (a_max_backups: INTEGER)
			-- Set max_backups to `a_max_backups'
		do
			max_backups := a_max_backups
		ensure
			max_backups_set: max_backups = a_max_backups
		end

	set_flush (a_flush: BOOLEAN)
			-- Set do_flush
		do
			do_flush := a_flush
		ensure
			do_flush_set: do_flush = a_flush
		end

feature {ELOG_LOGGER_FACTORY} -- Configuration

	filename: STRING
	do_flush: BOOLEAN
	exclude_filters: ARRAY [STRING]
	include_filters: ARRAY [STRING]
	logger_name: STRING
	use_std_out: BOOLEAN
	use_utc: BOOLEAN
	max_size: INTEGER
	max_backups: INTEGER
	should_append: BOOLEAN
	synchronous_write: BOOLEAN

invariant
	filename_not_void: filename /= Void
	filename_not_empty: not filename.is_empty
	exclude_filters_not_void: exclude_filters /= Void
	no_void_exclude_filter: not exclude_filters.has (Void)
	include_filters_not_void: include_filters /= Void
	no_void_include_filter: not include_filters.has (Void)
	logger_name_not_void: logger_name /= Void
	logger_name_not_empty: not logger_name.is_empty
	max_size_non_negative_or_minus_one: max_size >= -1
	max_backups_non_negative_or_minus_one: max_backups >= -1
	no_max_size_implies_one_logfile: (max_size = -1) implies (max_backups = -1)
end
