indexing
	description: "Summary of discarded events. "
	library: "elogger"
	copyright: "Copyright (c) 2006, AXA Rosenberg, and others (see copyright.txt)"
	author: "Ulrich Weiss, and others"
	license: "Eiffel Forum License v2 (see forum.txt)"
	date: "$Date: 2014/02/26 $"
	revision: "$Revision: #5 $"
	archive: "$File: //rose/source/kernel/infrastructure/logging/elog_discard_summary.e $"

class ELOG_DISCARD_SUMMARY

inherit
	ELOG_SEVERITY_CONSTANTS
		export
			{NONE} all
		end
	ANY
create
	make

feature {NONE} -- Initialization

	make (an_appender: ELOG_APPENDER) is
			-- creation procedure
		require
			an_appender_not_void: an_appender /= Void
		do
			appender := an_appender
			max_severity := Unknown_severity
		end

feature {ELOG_APPENDER} -- Access

	discarded: INTEGER
			-- nr of discarded events

	is_empty: BOOLEAN is
			-- is the summary empty?
		do
			Result := discarded = 0
		end

feature {ELOG_APPENDER} -- Basic operations

	discard_event (event: ELOG_EVENT) is
			-- discard a given event string
		require
			event_not_void: event /= Void
		do
			last_event := event
			discarded := discarded + 1
			if event.severity > max_severity then
				max_severity := event.severity
			end
		end

feature {ELOG_APPENDER} -- Status Report

	summary: ELOG_SIMPLE_EVENT is
			-- returns a summary of the discarded events and resets discard summary
		require
			not_empty: not is_empty
		local
			msg: STRING
		do
			if discarded = 1 then
				msg := "1 event"
			else
				msg := discarded.out+" events"
			end
			create Result.make (msg+" discarded due to full event buffer, including: "+last_event.message)
			Result.set_severity (max_severity)

			-- clean up
			last_event := Void
			discarded := 0
			max_severity := Unknown_severity
		end

feature {NONE} -- Implementation

	last_event: ELOG_EVENT
			-- the last discarded event

	appender: ELOG_APPENDER
			-- the appender, that discarded the events

	max_severity: ELOG_SEVERITY
			-- max severity of discarded events

end
