note

	description:

		"Consumer thread that dispatches the events of the event pool to the appropriate appenders."

	library: "elogger"
	copyright: "Copyright (c) 2006, AXA Rosenberg, and others (see copyright.txt)"
	license: "Eiffel Forum License v2 (see forum.txt)"
	author: "Ulrich Weiss, and others"
	last_editor: "$Author: er $"
	date: "$Date: 2014/02/12 $"
	revision: "$Revision: #6 $"
	path: "$File: //rose/source/kernel/infrastructure/logging/concurrency/elog_dispatcher.e $"

class ELOG_DISPATCHER

inherit
	THREAD
		rename
			make as make_thread
		end

	ELOG_SHARED_INTERNAL_LOGGER
		undefine
			default_create
		end

create
	make

feature {NONE} -- Initialization

	make (buf: ELOG_CONCURRENT_LINKED_QUEUE_CV [ELOG_EVENT]; an_appender: ELOG_APPENDER) is
			-- creation procedure.
		require
			buf_not_void: buf /= Void
			appender_not_void: an_appender /= Void
		do
			make_thread
			buffer := buf
			appender := an_appender
		end

feature -- Access

	buffer: ELOG_CONCURRENT_LINKED_QUEUE_CV [ELOG_EVENT]
			-- event queue

	is_appending: BOOLEAN
			-- is the dispatcher currently appending?

feature {NONE} -- Implementation

	appender: ELOG_APPENDER
			-- the appender to which this

	execute is
			-- waits for the queue to get elements until the
			-- buffer is closed.
		local
			e: ELOG_EVENT
			retried: BOOLEAN
		do
			if not retried then
				from
				until
					not appender.is_open or buffer.is_closed
				loop
					e := buffer.get

					if e /= Void then
						-- Void would indicate we were awakened by
						-- a close.
						--
						is_appending := True
						if not appender.use_backup_appender then
							appender.do_append (e)
						else
							appender.backup_appender.append (e)
						end
						is_appending := False
					end
				end
			end
		rescue
			-- an error occurred in do_append
			appender.set_use_backup_appender (True, e)
			retried := True
			retry
		end

end
