indexing
	description: "Deferred event class"
	library: "elogger"
	copyright: "Copyright (c) 2006, AXA Rosenberg, and others (see copyright.txt)"
	author: "Ulrich Weiss, and others"
	license: "Eiffel Forum License v2 (see forum.txt)"
	date: "$Date: 2008/12/01 $"
	revision: "$Revision: #3 $"
	archive: "$File: //rose/source/kernel/infrastructure/logging/events/elog_event.e $"

deferred class ELOG_EVENT

inherit

	ELOG_SHARED_SYSTEM_CLOCK
		undefine
			out
		end

	ANY
		redefine
			out
		end

feature {NONE} -- Initialization

	make (event_message: STRING) is
			-- Create a new logging event for the
			-- message object 'event_message'. The
			-- message will be converted to a string
			-- using '.out' or using layouts.
		require
			event_message_exists: event_message /= Void
		local
			rescued: BOOLEAN
		do
			make_without_time (event_message)
			if not rescued then
				-- system_clock.date_time_now might through exceptions
				creation_time := system_clock.date_time_now
			end
		rescue
			rescued := True
			retry
		end

	make_without_time (event_message: STRING) is
			-- Create a new logging event for the
			-- message object 'event_message'. The
			-- message will be converted to a string
			-- using '.out' or using layouts.
		require
			event_message_exists: event_message /= Void
		do
			message := event_message
		end

feature -- Status Report

	logger: ELOG_LOGGER
			-- The logger in which this logging event occurred.

	severity: ELOG_SEVERITY
			-- event severity. see ELOG_SEVERITY for more details
			-- on severity levels

	message: STRING
			-- event message. Will be converted with
			-- 'message.out'. 

	creation_time: DT_DATE_TIME
			-- Time stamp indicating when event was created.

	override_layout: ELOG_LAYOUT
			-- layout overriding appender.layout

feature -- Output

	out: STRING is
			-- simple event formatting
		do
			if message /= Void then
				Result := message
			else
				Result := "no message attached"
			end
		end

feature -- Element Change

	set_message (event_message: STRING) is
			-- set the event message
		require
			event_message_not_void: event_message /= Void
		do
			message := event_message
		end

	set_severity (a_severity: ELOG_SEVERITY) is
			-- Sets 'severity'.
		require
			a_severity_not_void: a_severity /= Void
		do
			severity := a_severity
		end

	set_creation_time_now is
			-- sets 'creation_time' to now.
		do
			creation_time := system_clock.date_time_now
		end

	set_creation_time (t: DT_DATE_TIME) is
			-- sets 'creation_time' to now.
		require
			t_not_void: t /= Void
		do
			creation_time := t
		end

	set_override_layout (a_override_layout: ELOG_LAYOUT) is
			-- sets `override_layout'
		do
			override_layout := a_override_layout
		end

feature -- Transformation

	as_storable: ELOG_STORABLE_EVENT is
			-- Convert to a storable memento object
		do
			create Result.make (Current)
		end

feature {ELOG_LOGGER, ELOG_SPREAD_APPENDER} -- Implementation

	set_logger (a_logger: ELOG_LOGGER) is
			-- Sets 'logger'.
		do
			logger := a_logger
		end

end
