note

	description:

		"Logging appender that writes to an already opened file"

	author: "Colin Adams"
	last_editor: "$Author: e2 $"
	date: "$Date: 2014/07/04 $"
	revision: "$Revision: #2 $"
	path: "$File: //rose/source/kernel/infrastructure/logging/appenders/elog_open_file_appender.e $"

class ELOG_OPEN_FILE_APPENDER

inherit

	ELOG_APPENDER
		rename
			make as appender_make
		export
			{NONE} appender_make
		end

	ELOG_SHARED_INTERNAL_LOGGER

create

	make

feature {NONE} -- Initialization

	make (a_file: FILE; a_synchronous: BOOLEAN) is
			-- Initialize to use `a_file' as message sink.
		require
			a_file_not_void: a_file /= Void
			a_file_open_write: a_file.is_open_write
		do
			appender_make (a_file.path.name.as_string_8, a_synchronous)
			file := a_file
			is_open := True
			create mutex.make
		ensure
			file_aliased: file = a_file
		end

feature -- Removal

	dispose is
			-- Close this appender when garbage collected. Perform
			-- minimal operations to release resources. Do not call
			-- other object as they may have been garbage collected.
		do
			-- nothing to do
		end

feature {NONE} -- Implementation

	file: FILE
			-- Message sink

	mutex: ELOG_MUTEX
			-- Ensures clean separation of elog messages from different threads;
			-- Does nothing about non-elog activity

	do_append_string (a_string: STRING)
			-- Append `a_string'.
		do
			mutex.lock

			file.put_string (a_string)
			if do_flush then
				file.flush
			end

			mutex.unlock
		rescue
			mutex.unlock
		end

	internal_close is
			-- Release any resources for this appender.
		do
			-- nothing to do
		end

invariant

	file_not_void: file /= Void
	file_open_write: file.is_open_write
	mutex_not_void: mutex /= Void

end
