indexing
	description: "Multiple filters and ored together to decide if an event is accepted or not. "
	library: "elogger"
	copyright: "Copyright (c) 2006, AXA Rosenberg, and others (see copyright.txt)"
	author: "Ulrich Weiss, and others"
	license: "Eiffel Forum License v2 (see forum.txt)"
	date: "$Date: 2008/12/01 $"
	revision: "$Revision: #3 $"
	archive: "$File: //rose/source/kernel/infrastructure/logging/filters/elog_or_filter.e $"

class ELOG_OR_FILTER

inherit

	ELOG_FILTER_CONTAINER

create

	make

feature {NONE} -- Initialization

	make (filter_a, filter_b: ELOG_FILTER; rejects_ored: BOOLEAN) is
			-- A minimum of 2 filters are added to this filter.
			--
			-- If rejects_ored is set to True, Filter_reject is
			-- returned when a single filter returns Filter_reject.
			-- If rejects_ored is set to False, Filter_accept is
			-- returned when a single filter returns Filter_accept.
			-- Filter_neutral are effectively ignored.
			-- If all contained filters return Filter_neutral, this
			-- filter also returns Filter_neutral.
		require
			filters_exist: filter_a /= Void and filter_b /= Void
		do
			create filters.make
			filters.force_last (filter_a)
			filters.force_last (filter_b)
			is_reject_ored := rejects_ored
		end

feature -- Element Change

	set_reject_ored (rejects_ored: BOOLEAN) is
			-- sets is_reject_ored to reject_ored
		do
			is_reject_ored := rejects_ored
		end

feature -- Status report

	decide (event: ELOG_EVENT): INTEGER is
			-- Should 'event' be logged. Return one of Filter_accept,
			-- Filter_reject, or Filter_neutral.
		local
			c: DS_LINKED_LIST_CURSOR [ELOG_FILTER]
			i: INTEGER
			has_accept, has_reject: BOOLEAN
		do
			Result := Filter_neutral
			from
				c := filters.new_cursor
				c.start
			until
				c.off or Result = Filter_accept or Result = Filter_reject
			loop
				i := c.item.decide (event)
				if i = Filter_reject then
					has_reject := True
				elseif i = Filter_accept then
					has_accept := True
				end
				if has_accept and not is_reject_ored then
					Result := Filter_accept
				elseif has_reject and is_reject_ored then
					Result := Filter_reject
				end
				c.forth
			end

			if Result /= Filter_accept and Result /= Filter_reject then
				if not has_accept and not has_reject then
					Result := Filter_neutral
				elseif has_accept and is_reject_ored then
					Result := Filter_accept
				elseif has_reject and not is_reject_ored then
					Result := Filter_reject
				end
			end
		ensure then
			result_valid: Result = Filter_reject or Result = Filter_accept or Result = Filter_neutral
		end

feature {NONE} -- Implementation

	is_reject_ored: BOOLEAN
			-- ores the rejects instead of the accepts.

invariant
	enough_filters: filters.count >= 2

end
