note

	description:

		"Eiffel Logger appenders that write to the log ROSE_OUTCHANNEL"

	author: "Colin Adams"
	last_editor: "$Author: c4 $"
	date: "$Date: 2014/07/16 $"
	revision: "$Revision: #1 $"
	path: "$File: //rose/source/kernel/infrastructure/logging/appenders/elog_outchannel_appender.e $"

class ELOG_OUTCHANNEL_APPENDER

inherit

	ELOG_APPENDER
		rename
			make as make_unopened,
			make_open as make
		export
			{NONE} make_unopened
		redefine
			do_append
		end
	
	ROSE_IO_VERBOSITY
	
	ROSE_SHARED_GLOBAL
		export {NONE} all end

	ROSE_IO_CONVENTION
		export {NONE} all end

create

	make

feature -- Removal

	dispose
			-- Close this appender when garbage collected. Perform
			-- minimal operations to release resources. Do not call
			-- other object as they may have been garbage collected.
		do
			is_open := False
		end

feature {ELOG_DISPATCHER} -- Implementation

	do_append (a_event: ELOG_EVENT)
			-- Append `a_event' to this appender.
		local
			l_msg: STRING
			l_channel: ROSE_OUTCHANNEL
		do
			if a_event.override_layout /= Void then
				l_msg := a_event.override_layout.format (a_event)
			else
				l_msg := layout.format (a_event)
			end
			l_channel := Global.io_handler.log
			if l_channel = Void then
				l_channel := Global.io_handler.output
			end
			l_channel.output_message (framework_level (a_event), l_msg)
		end
	
feature {NONE} -- Implementation

	do_append_string (a_string: STRING)
			-- Append `a_string'.
		do
			-- This is just implemented because it is deferred - we don't use it.
		end

	internal_close
			-- Release any resources for this appender.
		do
		end

	framework_level (a_event: ELOG_EVENT): INTEGER
			-- `a_event.severity.level' expressed as a framework message level
		require
			a_event_attached: a_event /= Void
		do
			Result := verbosity_from_severity (a_event.severity.level)
		ensure
			framework_level_in_range: Result >= minimum_verbosity and Result <= maximum_verbosity
		end

end
