indexing
	description: "Pattern converter"
	library: "elogger"
	copyright: "Copyright (c) 2006, AXA Rosenberg, and others (see copyright.txt)"
	author: "Ulrich Weiss, and others"
	license: "Eiffel Forum License v2 (see forum.txt)"
	date: "$Date: 2008/12/01 $"
	revision: "$Revision: #2 $"
	archive: "$File: //rose/source/kernel/infrastructure/logging/layouts/pattern/elog_pattern_converter.e $"

deferred class ELOG_PATTERN_CONVERTER

feature -- Initialisation

	make (formatting_info: ELOG_FORMATTING_INFO) is
			-- Initialise with 'formatting_info'.
		require
			formatting_info_exists: formatting_info /= Void
		do
			min := formatting_info.min
			max := formatting_info.max
			left_align := formatting_info.left_align
		end

feature -- Status report

	min: INTEGER
			-- Minimum length of converted string, -1 for
			-- no minimum.

	max: INTEGER
			-- Maximum length of converted string,
			-- Max_integer for no maximum.

	left_align: BOOLEAN
			-- Should the string be left aligned.

feature -- Element Change

	set_start_time (a_time: DT_DATE_TIME) is
			-- sets 'start_time'
		do
			start_time := a_time
		end

feature -- Basic operations

	format (sbuf: STRING; event: ELOG_EVENT) is
			-- Template routine for formatting in a converter
			-- specific way.
		require
			sbuf_exists: sbuf /= Void
			event_exists: event /= Void
		local
			s: STRING
			len: INTEGER
		do
			s := convert_event (event)
			if s = Void then
				if 0 < min then
					space_pad (sbuf, min)
				end
			else
				len := s.count
				if len > max then
					sbuf.append (s.substring (len - max + 1, len))
				elseif len < min then
					if left_align then
						sbuf.append (s)
						space_pad (sbuf, min - len)
					else
						space_pad (sbuf, min - len)
						sbuf.append (s)
					end
				else
					sbuf.append (s)
				end
			end
		end

feature {NONE} -- Implementation

	convert_event (event: ELOG_EVENT): STRING is
			-- Convert conversion specifiers appropriately.
		require
			event_exists: event /= Void
		deferred
		ensure
			converted_string_exists: Result /= Void
		end

	space_pad (str: STRING; length: INTEGER) is
			-- Pad 'str' with 'length' space characters.
		require
			str_exists: str /= Void
			positive_length: length >= 0
		local
			pad: STRING
		do
			create pad.make (length)
			pad.fill_blank
			str.append (pad)
		ensure
			correct_size: str.count >= old str.count + length
		end

	start_time: DT_DATE_TIME
			-- start time of the relative time

end -- class ELOG_PATTERN_CONVERTER


