indexing
	description: "A flexible layout using a pattern"
	usage: "[
		The conversion pattern is closely related to the printf function in C.
		is composed of literal text and format control expression called conversion
		specifiers.
		
		Each conversion specifier starts with an @ character and is followed by
		optional format modifiers and a conversion character. The conversion 
		character specifies the type of data, e.g., category, priority, date,
		message. The format modifiers control such things as field width, padding,
		left and right justification.
		]"
	library: "elogger"
	copyright: "Copyright (c) 2006, AXA Rosenberg, and others (see copyright.txt)"
	author: "Ulrich Weiss, and others"
	license: "Eiffel Forum License v2 (see forum.txt)"
	date: "$Date: 2008/12/01 $"
	revision: "$Revision: #3 $"
	archive: "$File: //rose/source/kernel/infrastructure/logging/layouts/elog_pattern_layout.e $"

class ELOG_PATTERN_LAYOUT

inherit

	ELOG_SIMPLE_LAYOUT
		redefine
			format
		end

create

	make, make_with_start_time

feature -- Initialisation

	make (pattern: STRING) is
			-- Create new pattern layout using 'new_pattern' to format
			-- the message
		require
			pattern_exists: pattern /= Void
		do
			start_time := system_clock.date_time_now
			create converters.make
			parse_pattern (pattern)
		end

	make_with_start_time (pattern: STRING; a_time: DT_DATE_TIME) is
			-- Create new pattern layout using 'new_pattern' to format
			-- the message
		require
			pattern_exists: pattern /= Void
		do
			start_time := a_time
			create converters.make
			parse_pattern (pattern)
		end

feature -- Rendering

	format (event: ELOG_EVENT): STRING is
			-- Format contents of 'event' according to this layout's
			-- formatting rules.
			-- Apply all converters to event in order.
		local
			cursor: DS_LINKED_LIST_CURSOR [ELOG_PATTERN_CONVERTER]
		do
			create Result.make (128)
			cursor := converters.new_cursor
			from
				cursor.start
			until
				cursor.off
			loop
				cursor.item.format (Result, event)
				cursor.forth
			end
		end

	set_start_time (a_time: DT_DATE_TIME) is
			-- sets 'start_time'
		require
			a_time_not_void: a_time /= Void
		local
			cursor: DS_LINKED_LIST_CURSOR [ELOG_PATTERN_CONVERTER]
		do
			start_time := a_time
			cursor := converters.new_cursor
			from
				cursor.start
			until
				cursor.off
			loop
				cursor.item.set_start_time (a_time)
				cursor.forth
			end
		end

feature -- Access

	Message_pattern: STRING is "@m@n"
			-- simple pattern

feature {NONE} -- Implementation

	parse_pattern (pattern: STRING) is
			-- Parse 'pattern' and create list of pattern elements for each
			-- distinct part.
		require
			pattern_exists: pattern /= Void
		local
			parser: ELOG_PATTERN_PARSER
		do
			create parser.make (pattern)
			parser.set_start_time (start_time)
			converters := parser.converters
		end

	converters: DS_LINKED_LIST [ELOG_PATTERN_CONVERTER]
			-- List of pattern converters to apply.

	start_time: DT_DATE_TIME
			-- start time needed for relative time pattern @r

invariant
	start_time_not_void: start_time /= Void

end -- class L4E_PATTERN_LAYOUT
