indexing

	description: "Event Severity constants of CBE. "
	instructions: "[ 
			The perceived severity of the status the event is 
			describing with respect to application that reports 
			the event. The associated values are 0 to 70. 
			Reserved values start at 0 increase by increments of 
			10 to 60 for Fatal. Other severities may be added 
			but must not exceed 70.
		]"
	library: "elogger"
	copyright: "Copyright (c) 2006, AXA Rosenberg, and others (see copyright.txt)"
	author: "Ulrich Weiss, and others"
	license: "Eiffel Forum License v2 (see forum.txt)"
	date: "$Date: 2009/12/11 $"
	revision: "$Revision: #5 $"
	archive: "$File: //rose/source/kernel/infrastructure/logging/events/elog_severity.e $"


class
	ELOG_SEVERITY

inherit
	DEBUG_OUTPUT
		undefine
			is_equal
		end
	COMPARABLE

create

	make

feature {NONE} -- Initialization

	make (a_level: INTEGER_8; a_abbreviation, a_description: STRING) is
			-- Create new severity
		require
			abbreviation_not_void: a_abbreviation /= Void
			description_not_void: a_description /= Void
		do
			level := a_level
			abbreviation := a_abbreviation
			description := a_description
		ensure
			level_set: level = a_level
			abbreviation_set: abbreviation = a_abbreviation
			descripotion_set: description = a_description
		end

feature -- Status Report

	level: INTEGER_8
			-- Level at which this severity is invoked.

	abbreviation: STRING
		-- Abbreviated version of the description
		-- ex: INFO instead of INFORMATION

	description: STRING
			-- String description of this severity

	debug_output: STRING is
		do
			Result := abbreviation + ":" + description
		end
	
feature -- Comparison

	infix "<" (other: like Current): BOOLEAN is
			-- Is 'other' less than Current?
		do
			Result := level < other.level
		end


invariant
	valid_level: level >= 0 and level <= 71
	abbreviation_not_void: abbreviation /= Void
	description_not_void: description /= Void

end
