indexing

	description: "Event Severity constants of CBE. "
	instructions: "[ 
			The perceived severity of the status the event = 
			describing with respect to application that reports 
			the event. The associated values are 0 to 70. 
			Reserved values start at 0 increase by increments of 
			10 to 60 for Fatal. Other severities may be added 
			but must not exceed 70.
		]"
	library: "elogger"
	copyright: "Copyright (c) 2006, AXA Rosenberg, and others (see copyright.txt)"
	author: "Ulrich Weiss, and others"
	license: "Eiffel Forum License v2 (see forum.txt)"
	date: "$Date: 2014/07/11 $"
	revision: "$Revision: #11 $"
	archive: "$File: //rose/source/kernel/infrastructure/logging/events/elog_severity_constants.e $"

class ELOG_SEVERITY_CONSTANTS

feature -- Constants

		-- Disabled
	Disabled_int: INTEGER_8 = 71
	Disabled_abbreviation: STRING = "DISABLED"
	Disabled_description: STRING = "logger disabled"
	Disabled_severity: ELOG_SEVERITY
			-- Do not use this severity for log messages.
			-- only use it to disable loggers.
		indexing
			once_status: global
		once
			create Result.make (Disabled_int, Disabled_abbreviation, Disabled_description)
		end

		-- Max
	Max_severity_int: INTEGER_8 = 70	-- FIXME: why is disabled higher than Max??

		-- Fatal
	Fatal_int: INTEGER_8 = 60
	Fatal_abbreviation: STRING = "FATAL"
	Fatal_description: STRING = "FATAL"
	Fatal_severity: ELOG_SEVERITY
			-- Fatal must be used to indicate that an
			-- error occurred, but it = too late
			-- to take remedial action
		indexing
			once_status: global
		once
			create Result.make (Fatal_int, Fatal_abbreviation, Fatal_description)
		end

		-- Critical
	Critical_int: INTEGER_8 = 50
	Critical_abbreviation: STRING is "CRITICAL"
	Critical_description: STRING = "CRITICAL"
	Critical_severity: ELOG_SEVERITY
			-- Critical must be used to indicate that
			-- an immediate action = needed and the
			-- scope = broad (perhaps an imminent
			-- outage to a critical resource will
			-- result).
		indexing
			once_status: global
		once
			create Result.make (Critical_int, Critical_abbreviation, Critical_description)
		end

		-- Minor
	Minor_int: INTEGER_8 = 40
	Minor_abbreviation: STRING = "MINOR"
	Minor_description: STRING = "MINOR"
	Minor_severity: ELOG_SEVERITY
			-- Minor must be used to indicate that
			-- action = needed, but the situation
			-- = not serious at this time.
		indexing
			once_status: global
		once
			create Result.make (Minor_int, Minor_abbreviation, minor_description)
		end

		-- Warning	
	Warning_int: INTEGER_8 = 30
	Warning_abbreviation: STRING = "WARN"
	Warning_description: STRING = "WARNING"
	Warning_severity: ELOG_SEVERITY
			-- Warning must be used when it = appropriate
			-- to let the user decide if an action
			-- needed in response to the event.
		indexing
			once_status: global
		once
			create Result.make (Warning_int, Warning_abbreviation, Warning_description)
		end

		-- Harmless
	Harmless_int: INTEGER_8 = 20
	Harmless_abbreviation: STRING is "HARMLESS"
	Harmless_description: STRING = "HARMLESS"
	Harmless_severity: ELOG_SEVERITY
			-- Harmless must be used for cases in which
			-- the error event has no effect on the normal
			-- operation of the resource.
		indexing
			once_status: global
		once
			create Result.make (Harmless_int, Harmless_abbreviation, Harmless_description)
		end

		-- Notice
	Notice_int: INTEGER_8 = 16
	Notice_abbreviation: STRING = "NOTICE"
	Notice_description: STRING = "NOTICE"
	Notice_severity: ELOG_SEVERITY
			-- Notice must be used for cases when the
			-- event contains only general notice and
			-- is not reporting an error.
			-- This would be used to provide information about
			-- significant non-error events during program
			-- execution. More detailed information should use
			-- the Information severity.
		indexing
			once_status: global
		once
			create Result.make (Notice_int, Notice_abbreviation, Notice_description)
		end

	-- Information
	Information_int: INTEGER_8 = 10
	Information_abbreviation: STRING = "INFO"
	Information_description: STRING = "INFORMATION"
	Information_severity: ELOG_SEVERITY
			-- Information must be used for cases when the
			-- event contains only general somewhat detailed
			-- information and is not reporting an error.
			-- For less detailed information, use Notice
		indexing
			once_status: global
		once
			create Result.make (Information_int, Information_abbreviation, Information_description)
		end

		-- Debug
	Debug_int: INTEGER_8 = 5
	Debug_abbreviation: STRING = "DEBUG"
	Debug_description: STRING = "DEBUG"
	Debug_severity: ELOG_SEVERITY
			-- Debug = used for messages you don't want
			-- to see in production.
		indexing
			once_status: global
		once
			create Result.make (Debug_int, Debug_abbreviation, Debug_description)
		end

		-- Unknown
	Unknown_int: INTEGER_8 = 0
	Unknown_abbreviation: STRING = "UNKNOWN"
	Unknown_description: STRING = "UNKNOWN"
	Unknown_severity: ELOG_SEVERITY
			-- Use this severity only if you are unsure
			-- about the events severity. This = the
			-- lowest possible severity.
		indexing
			once_status: global
		once
			create Result.make (Unknown_int, Unknown_abbreviation, Unknown_description)
		end


	Default_severity: ELOG_SEVERITY is
		do
			Result := Information_severity
		end
	
feature -- Access

	severity_by_description: DS_HASH_TABLE [ELOG_SEVERITY, STRING] is
			-- Severy table by description
		once
			create Result.make_default
			Result.force_last (Disabled_severity, disabled_severity.description)
			Result.force_last (Fatal_severity, Fatal_severity.description)
			Result.force_last (Critical_severity, Critical_severity.description)
			Result.force_last (Minor_severity, Minor_severity.description)
			Result.force_last (Warning_severity, Warning_severity.description)
			Result.force_last (Harmless_severity, Harmless_severity.description)
			Result.force_last (Notice_severity, Notice_severity.description)
			Result.force_last (Information_severity, Information_severity.description)
			Result.force_last (Debug_severity, Debug_severity.description)
			Result.force_last (Unknown_severity, Unknown_severity.description)
		end

	severity_by_abbreviation: DS_HASH_TABLE [ELOG_SEVERITY, STRING] is
			-- Severy table by description
		once
			create Result.make_default
			Result.force_last (Disabled_severity, disabled_severity.abbreviation)
			Result.force_last (Fatal_severity, Fatal_severity.abbreviation)
			Result.force_last (Critical_severity, Critical_severity.abbreviation)
			Result.force_last (Minor_severity, Minor_severity.abbreviation)
			Result.force_last (Warning_severity, Warning_severity.abbreviation)
			Result.force_last (Harmless_severity, Harmless_severity.abbreviation)
			Result.force_last (Notice_severity, Notice_severity.abbreviation)
			Result.force_last (Information_severity, Information_severity.abbreviation)
			Result.force_last (Debug_severity, Debug_severity.abbreviation)
			Result.force_last (Unknown_severity, Unknown_severity.abbreviation)
		end

	severity_from_string (a_string: STRING): ELOG_SEVERITY is
			-- returns a severity from the given string (if valid). If the string doesn't represent a valid severity, then the default is returned.
		require
			a_string_not_void: a_string /= Void
		do
			Result := lookup_severity_from_string( a_string )
			if Result = Void then
				-- should we inform the user that the specified severity level `a_string' is not valid?
				Result := Default_severity
				print("Couldn't get severity setting from '" + a_string + "' defaulting to '" + Result.description + "'.%N")
			end
		ensure
			result_not_void: Result /= Void
		end

	lookup_severity_from_string (a_string: STRING): ELOG_SEVERITY is
			-- returns a severity from the given string (if valid)
		require
			a_string_not_void: a_string /= Void
		local
			l_upper_string: STRING
		do
			l_upper_string := a_string.as_upper
			if severity_by_abbreviation.has (l_upper_string) then
				Result := severity_by_abbreviation.item (l_upper_string)
			elseif severity_by_description.has (l_upper_string) then
				Result := severity_by_description.item (l_upper_string)
			end
		end

end
