indexing
	description: "Filter matching on severity."
	library: "elogger"
	copyright: "Copyright (c) 2006, AXA Rosenberg, and others (see copyright.txt)"
	author: "Ulrich Weiss, and others"
	license: "Eiffel Forum License v2 (see forum.txt)"
	date: "$Date: 2008/12/01 $"
	revision: "$Revision: #3 $"
	archive: "$File: //rose/source/kernel/infrastructure/logging/filters/elog_severity_match_filter.e $"

class ELOG_SEVERITY_MATCH_FILTER

inherit

	ELOG_FILTER

create

	make

feature {NONE} -- Initialization

	make (severity: ELOG_SEVERITY; match_on_filter: BOOLEAN) is
			-- Create filter to match 'severity'. The decision is based on
			-- the following table:
			--
			-- Match Occurred           Match_on_filter
			--                       True             False
			--     True          Filter_accept    Filter_reject
			--     False         Filter_neutral   Filter_neutral
		require
			severity_exists: severity /= Void
		do
			matching_severity := severity
			match := match_on_filter
		end

feature -- Status report

	decide (event: ELOG_EVENT): INTEGER is
			-- Should 'event' be logged. Return one of Filter_accept,
			-- Filter_reject, or Filter_neutral.
		local
			match_occurred: BOOLEAN
		do
			if event.severity.is_equal (matching_severity) then
				match_occurred := True
			end
			if match_occurred then
				if match then
					Result := Filter_accept
				else
					Result := Filter_reject
				end
			else
				Result := Filter_neutral
			end
		end

feature {NONE} -- Implementation

	matching_severity: ELOG_SEVERITY
			-- Severity to match

	match: BOOLEAN

end -- class ELOG_SEVERITY_MATCH_FILTER
