indexing
	description: "Filter matching on a range of severities."
	library: "elogger"
	copyright: "Copyright (c) 2006, AXA Rosenberg, and others (see copyright.txt)"
	author: "Ulrich Weiss, and others"
	license: "Eiffel Forum License v2 (see forum.txt)"
	date: "$Date: 2008/12/01 $"
	revision: "$Revision: #3 $"
	archive: "$File: //rose/source/kernel/infrastructure/logging/filters/elog_severity_range_filter.e $"

class ELOG_SEVERITY_RANGE_FILTER

inherit

	ELOG_FILTER
	ELOG_SEVERITY_CONSTANTS

create

	make

feature {NONE} -- Initialization

	make (min_severity, max_severity: ELOG_SEVERITY; match_on_filter: BOOLEAN) is
			-- Create filter to match 'severity'. The decision is based on
			-- the following decisions:
			--
			-- Filter_accept := event.severtiy ElementOf [min_severity,max_severity] and match_on_filter
			-- Filter_reject := event.severity ElementOf [min_severity,max_severity] and not match_on_filter
			-- Filter_neutral := event.severity notElementOf [min_severity,max_severity]
			--
			-- Passing Void for 'min_severity' effectively sets no minimum range.
			-- Passing Void for 'max_severity' effectively sets no maximum range.
		require
			range_specified: min_severity /= Void or max_severity /= Void
			range_valid: min_severity /= Void and max_severity /= Void
				implies min_severity < max_severity
		do
			minimum_severity := min_severity
			maximum_severity := max_severity
			match := match_on_filter
		end

feature -- Status report

	decide (event: ELOG_EVENT): INTEGER is
			-- Should 'event' be logged. Return one of Filter_accept,
			-- Filter_reject, or Filter_neutral.
		local
			range_min_match, range_max_match: BOOLEAN
		do
			-- check minimum range
			if minimum_severity = Void then
				range_min_match := True
			elseif event.severity >= minimum_severity then
				range_min_match := True
			end

			-- check maximum range
			if maximum_severity = Void then
				range_max_match := True
			elseif event.severity <= maximum_severity then
				range_max_match := True
			end

			if range_min_match and range_max_match then
				if match then
					Result := Filter_accept
				else
					Result := Filter_reject
				end
			else
				Result := Filter_neutral
			end
		end

feature {NONE} -- Implementation

	minimum_severity, maximum_severity: ELOG_SEVERITY
			-- Severity to range

	match: BOOLEAN

end -- class ELOG_SEVERITY_MATCH_FILTER
