indexing
	description: "Logger shared by several libraries/components and set only once by the root class"

	author: "Emmanuel Bouyer"
	last_editor: "$Author: c4 $"
	date: "$Date: 2014/07/11 $"
	revision: "$Revision: #5 $"
	archive: "$File: //rose/source/kernel/infrastructure/logging/elog_shared_logger.e $"

class ELOG_SHARED_LOGGER

feature -- Access

	logger: ELOG_LOGGER is
			-- Shared file logger
		do
			Result := shared_logger.item
		ensure
			logger_not_void: Result /= Void
		end

feature -- Setting

	set_logger (a_logger: like logger) is
			-- Sets logger to `a_logger'
		require
			logger_not_void: a_logger /= Void
		local
			l_previous_logger: ELOG_LOGGER
		do
			l_previous_logger := logger
			l_previous_logger.info ("Replacing logger; new logger name is '" + a_logger.name + "'")
			shared_logger.replace (a_logger)
			logger.info ("Logger replaced from '" + l_previous_logger.name + "'")
		ensure
			logger_set: logger = a_logger
		end

feature {NONE} -- Implementation

	shared_logger: CELL [ELOG_LOGGER] is
			-- Cell to share logger between different components
		indexing
			once_status: global
		once
			create Result.put (default_logger)
		ensure
			shared_logger_not_void: Result /= Void
			logger_not_void: Result.item /= Void
		end

	default_logger: ELOG_LOGGER
			-- Logger to use before any initialization
		local
			l_hierarchy: ELOG_HIERARCHY
			l_constants: ELOG_SEVERITY_CONSTANTS
		do
			create l_constants
			create l_hierarchy.make (l_constants.unknown_severity)
			create Result.make (l_hierarchy, "default_logger")
		ensure
			default_logger_not_void: Result /= Void
		end

end
