indexing
	description: "shared system clock, able to switch from and to UTC and local clock, globally"
	library: "elogger"
	copyright: "Copyright (c) 2006, AXA Rosenberg, and others (see copyright.txt)"
	author: "Ulrich Weiss, and others"
	last_editor: "$Author: mk $"
	license: "Eiffel Forum License v2 (see forum.txt)"
	date: "$Date: 2009/06/23 $"
	revision: "$Revision: #4 $"
	archive: "$File: //rose/source/kernel/infrastructure/logging/elog_shared_system_clock.e $"

class ELOG_SHARED_SYSTEM_CLOCK

feature -- Setting

	set_use_utc (a_use: BOOLEAN) is
			-- Set whether to use utc or not
		do
			use_utc.put (a_use)
		ensure
			use_utc_set: use_utc.item = a_use
		end

feature -- Access

	system_clock: DT_CLOCK is
			-- shared system clock using the clock specified by 'use_utc'
		do
			if use_utc.item then
				Result := utc_system_clock
			else
				Result := local_system_clock
			end
		end

	local_system_clock: DT_SYSTEM_CLOCK is
			-- Local system clock
		indexing
			once_status: global
		local
			shared: DT_SHARED_SYSTEM_CLOCK
		once
			create shared
			Result := shared.system_clock
		ensure
			system_clock_not_void: Result /= Void
		end

	utc_system_clock: DT_UTC_SYSTEM_CLOCK is
			-- UTC system clock
		indexing
			once_status: global
		local
			shared: DT_SHARED_SYSTEM_CLOCK
		once
			create shared
			Result := shared.utc_system_clock
		ensure
			system_clock_not_void: Result /= Void
		end

feature {NONE} -- Implementation

	use_utc: CELL [BOOLEAN] is
			-- use local time by default.
		indexing
			once_status: global
		once
			create Result.put (False)
		end

end
