indexing
	description: "[
		Simple Logging Event
		
		Converters facilitate conversion to complex, formatted log messages.  If you find a type that is not supported, 
		a simple to_any(a_any: ANY) do Result := a_any end wrapping any types that you need to log will force an upcast to
		the converter that support make_with_any_values
		]"
	library: "elogger"
	copyright: "Copyright (c) 2006, AXA Rosenberg, and others (see copyright.txt)"
	author: "Ulrich Weiss, and others"
	license: "Eiffel Forum License v2 (see forum.txt)"
	date: "$Date: 2008/12/01 $"
	revision: "$Revision: #6 $"
	archive: "$File: //rose/source/kernel/infrastructure/logging/events/elog_simple_event.e $"

class ELOG_SIMPLE_EVENT

inherit

	ELOG_EVENT

create

	make,
	make_without_time,
	make_with_string,
	make_with_string_values,
	make_with_any_values,
	make_with_integer_values,
	make_with_double_values

convert

	make_with_string ({STRING}),
	make_with_string_values ({TUPLE [text: STRING; values: ARRAY [TUPLE[key: STRING; value: STRING]]]}),
	make_with_any_values({TUPLE [text: STRING; values: ARRAY [TUPLE[key: STRING; value: ANY]]]}),
	make_with_integer_values ({TUPLE [text: STRING; values: ARRAY [TUPLE[key: STRING; value: INTEGER]]]}),
	make_with_double_values ({TUPLE [text: STRING; values: ARRAY [TUPLE[key: STRING; value: DOUBLE]]]})

feature {NONE} -- Creation

	make_with_string (a_text: STRING) is
			-- convert from a simple string
		require
			not_void: a_text /= Void
			not_empty: not a_text.is_empty
		do
			make (a_text)
		end

	make_with_integer_values (a_values: TUPLE [text: STRING; values: ARRAY [TUPLE[key: STRING; value: INTEGER]]]) is
			-- tuple with integer, neede since convert requires exact match
		require
			not_void: a_values /= Void and then a_values.text /= Void
			not_empty: not a_values.text.is_empty
		local
			l_text: STRING
			l_index: INTEGER
		do
			-- should be
--			make_with_any_values(a_values)			

			-- FIXME must do in a loop due to 6.0.7 compiler bug with upcasting named TUPLE.  Delete all of this code
			-- once this is verified fixed
			l_text := a_values.text
			if a_values.values /= Void and then a_values.values.count > 0 then
				l_text.append (" [")
				from
					l_index := 1
				until
					l_index > a_values.values.count
				loop
					add_value (l_text, a_values.values[l_index].key, a_values.values[l_index].value)
					l_index := l_index + 1
				end
				l_text.append (" ]")
			end
			make (l_text)

		end

	make_with_double_values (a_values: TUPLE [text: STRING; values: ARRAY [TUPLE[key: STRING; value: DOUBLE]]]) is
			-- tuple with integer, neede since convert requires exact match
		require
			not_void: a_values /= Void and then a_values.text /= Void
			not_empty: not a_values.text.is_empty
		local
			l_text: STRING
			l_index: INTEGER
		do
			-- should be
--			make_with_any_values(a_values)			

			-- FIXME must do in a loop due to 6.0.7 compiler bug with upcasting named TUPLE.  Delete all of this code
			-- once this is verified fixed
			l_text := a_values.text
			if a_values.values /= Void and then a_values.values.count > 0 then
				l_text.append (" [")
				from
					l_index := 1
				until
					l_index > a_values.values.count
				loop
					add_value (l_text, a_values.values[l_index].key, a_values.values[l_index].value)
					l_index := l_index + 1
				end
				l_text.append (" ]")
			end
			make (l_text)

		end

	make_with_string_values (a_values: TUPLE [text: STRING; values: ARRAY [TUPLE[key: STRING; value: STRING]]]) is
			-- tuple with string, neede since convert requires exact match
		require
			not_void: a_values /= Void and then a_values.text /= Void
			not_empty: not a_values.text.is_empty
		do
			make_with_any_values(a_values)
		end

	make_with_any_values (a_values: TUPLE [text: STRING; values: ARRAY [TUPLE[key: STRING; value: ANY]]]) is
			-- convert from tuple of form ["message text', <<["key1", value1], ["key2", value2]>>]
		require
			not_void: a_values /= Void and then a_values.text /= Void
			not_empty: not a_values.text.is_empty
		local
			l_text: STRING
			l_index: INTEGER
		do
			l_text := a_values.text
			if a_values.values /= Void and then a_values.values.count > 0 then
				l_text.append (" [")
				from
					l_index := 1
				until
					l_index > a_values.values.count
				loop
					add_value (l_text, a_values.values[l_index].key, a_values.values[l_index].value)
					l_index := l_index + 1
				end
				l_text.append (" ]")
			end

			make (l_text)
		end

feature {NONE} -- Implementation

	add_value (a_result, a_key: STRING; a_value: ANY) is
			-- add a (key, value) pair to the message string being built
		require
			not_void: a_result /= Void
		do
			if not a_result.is_empty then
				a_result.append (" ")
			end

			if a_key /= Void and then not a_key.is_empty then
				a_result.append (a_key)
				a_result.append ("=")
			else
				a_result.append ("<<void-or-blank-key>>=")
			end

			if a_value = Void then
				a_result.append ("<<Void>>")
			elseif a_value.out.is_empty then
				a_result.append ("<<empty>>")
			else
				a_result.append (a_value.out)
			end
		end

end

