indexing
	description: "Filter event messages that do not match. "
	library: "elogger"
	copyright: "Copyright (c) 2006, AXA Rosenberg, and others (see copyright.txt)"
	author: "Ulrich Weiss, and others"
	license: "Eiffel Forum License v2 (see forum.txt)"
	date: "$Date: 2008/12/01 $"
	revision: "$Revision: #3 $"
	archive: "$File: //rose/source/kernel/infrastructure/logging/filters/elog_string_match_filter.e $"

class ELOG_STRING_MATCH_FILTER

inherit

	ELOG_FILTER

create

	make

feature {NONE} -- Initialization

	make (str_match: STRING; match_on_filter: BOOLEAN) is
			-- Create filter to match 'str_match' in
			-- message of event.
			-- The decision is Filter_accept if
			-- 'str_match' if found and 'match_on_filter' 
			-- is True. If a match is found and
			-- 'match_on_filter' is False then
			-- Filter_reject is returned.
			-- If no match is found then Filter_neutral
			-- is returned.
		require
			str_match_exists: str_match /= Void
		do
			string_match := str_match
			match := match_on_filter
		end

feature -- Status report

	decide (event: ELOG_EVENT): INTEGER is
			-- Should 'event' be logged. Return one of Filter_accept,
			-- Filter_reject, or Filter_neutral.
		do
			if event.message.substring_index (string_match, 1) = 0 then
				Result := Filter_neutral
			else
				if match then
					Result := Filter_accept
				else
					Result := Filter_reject
				end
			end
		end

feature {NONE} -- Implementation

	string_match: STRING
			-- Priority to match

	match: BOOLEAN

end -- class ELOG_STRING_MATCH_FILTER
