indexing
	description: "Object representing an entity attribute history"
	author: "Broos Nemanic"
	date: "$Date: 2010/06/15 $"
	revision: "$Revision: #7 $"
	archive: "$Archive: $"

class
	ENTITY_ATTRIBUTE_HISTORY

inherit
	ENTITY_ATTRIBUTE_OR_RELATIONSHIP_HISTORY [STRING]

create
	make,
	make_empty

feature -- Initialization
	make (a_entity : ENTITY; a_type : INTEGER) is
			-- Initialize
		require
			a_entity_not_void : a_entity /= Void
			a_type_valid : a_type > 0
		do
			make_empty
			entity := a_entity
			type_id := a_type
		end

feature -- Access
	type_id : INTEGER
			-- Attribute_type_id

	name : STRING is
			-- Attribute name
		do
			Result := context.attribute_id_types [type_id]
		end

	duplicate : like Current is
			-- Duplicate of `Current': same intervals but *not* the same constainers
		local
			i : INTEGER
		do
			create Result.make_empty
			Result.set_entity (entity)
			Result.set_type_id (type_id)
			from i := 1 until i > count loop
				Result.extend (item (i))
				i := i + 1
			end
		end

	is_valid : BOOLEAN
			-- Is `Current' valid?
		do
			Result := entity /= Void and then entity.context.attribute_name_by_id (type_id) /= Void
		end

feature -- Basic operations
	add_history_from_attribute_value_records (a_history : ROSE_LINEAR_ARRAY [ENTITY_ATTRIBUTE_RECORD]) is
			-- Convenience feature created for use in rose_platform INSTRUMENT_SELECTOR_LIST
		require
			a_history_not_void : a_history /= Void
		local
			l_record : ENTITY_ATTRIBUTE_RECORD
			l_interval : ROSE_CLOSED_OPEN_INTERVAL [STRING]
			i : INTEGER
		do
			from i := a_history.lower until i > a_history.count loop
				l_record := a_history [i]
				if l_record /= Void and then is_included (l_record) then
					create l_interval.make (l_record.attribute_value, l_record.start_date, l_record.end_date, 0)
					extend (l_interval)
				end
				i := i + 1
			end
		end

	set_history (a_history : ROSE_LINEAR_ARRAY [ROSE_CLOSED_OPEN_INTERVAL [STRING]]) is
			-- Set `Current' with `a_history'
		require
			a_history_not_void : a_history /= Void
		do
			wipe_out
			a_history.do_all (agent extend)
		end

	populate_history is
			-- Populate attribute history for `entity' at `type_id'
		require
			entity_not_void : entity /= Void
		local
			l_history : ROSE_LINEAR_ARRAY [ENTITY_ATTRIBUTE_RECORD]
			l_record : ENTITY_ATTRIBUTE_RECORD
			l_interval : ROSE_CLOSED_OPEN_INTERVAL [STRING]
			i : INTEGER
		do
			l_history := entity.attribute_history (entity.context.attribute_name_by_id (type_id))
			from i := 1 until i > l_history.count loop
				l_record := l_history [i]
				if l_record /= Void and then is_included (l_record) then
					create l_interval.make (l_record.attribute_value, l_record.start_date, l_record.end_date, 0)
					extend (l_interval)
				end
				i := i + 1
			end
		end

	set_type_id (a_type_id : INTEGER) is
			-- Set `type_id' to `a_type_id'
		do
			type_id := a_type_id
		end

	merge_record (a_record : ROSE_CLOSED_OPEN_INTERVAL [STRING])
			-- Merge `a_record' into current records replacing/truncating old when overlapping.
		require
			a_record_not_void : a_record /= Void
		local
			l_record : ROSE_CLOSED_OPEN_INTERVAL [STRING]
			i : INTEGER
			l_start, l_end : ROSE_DATE
		do
			from i := upper until i < lower loop
				l_record := item (i)
				l_start := l_record.start_date
				l_end := l_record.end_date
				if l_start >= a_record.start_date and l_start < a_record.end_date then
					l_record.set_start_date (a_record.end_date)
				end
				if l_end > a_record.start_date and l_end <= a_record.end_date then
					l_record.set_end_date (a_record.start_date)
				end
				if l_record.start_date >= l_record.end_date then
					remove_i_th (i)
				end
				i := i - 1
			end
			extend (a_record)
		end

feature {NONE} -- Implementation
--	is_included (a_entity_id, a_category_id, a_attribute_type_id : INTEGER) : BOOLEAN is
	is_included (a_record : ENTITY_ATTRIBUTE_RECORD) : BOOLEAN is
			-- Do arguements match `Current'?
		do
			Result := (a_record.entity_id = entity.handle) and (a_record.category_id = category) and (a_record.attribute_type_id = type_id)
		end


end
