indexing
	description: "Ancestor for classes which represent a specific attribute or relationship history for an entity"
	author: "Broos Nemanic"
	last_editor: "$Author: bn $"
	date: "$Date: 2010/06/15 $"
	revision: "$Revision: #7 $"
	archive: "$Archive: $"

deferred class
	ENTITY_ATTRIBUTE_OR_RELATIONSHIP_HISTORY [G]

inherit
	ROSE_INTERVAL_HISTORY [G, ROSE_CLOSED_OPEN_INTERVAL [G]]
		rename
			make as rose_interval_history_make
		end

	CONTEXT_ANCHOR

feature -- Access

	entity: ENTITY
			-- Entity `Current' is an attribute history of

	context: like context_anchor is
			-- Context of `entity'
		require
			entity_not_void : entity /= Void
		do
			Result := entity.context
		end

	category: INTEGER is
			-- Category_id of `entity'
		require
			entity_not_void : entity /= Void
		do
			Result := entity.category_id
		end

	category_name: STRING is
			-- Name of category
		do
			Result := context.category_name (category)
		end

	name: STRING is
			-- Name of attribute or relationship
		deferred
		end

	selected_interval: ROSE_CLOSED_OPEN_INTERVAL [G]
			-- Interval of interest

	duplicate : like Current is
			-- Dupllicate of `Current': same intervals but *not* the same constainers
		deferred
		end

feature -- Change

	set_entity (a_entity: ENTITY) is
			-- Set `entity' to `a_entity'
		do
			entity := a_entity
		ensure
			entity_set: entity = a_entity
		end

	set_selected_interval (a_interval: ROSE_CLOSED_OPEN_INTERVAL [G]) is
			-- Set `selected_interval' to `a_interval'
		require
			current_has_a_interval : has (a_interval)
		do
			selected_interval := a_interval
		ensure
			selected_interval_set: selected_interval = a_interval
		end

	truncate_all (a_date : ROSE_DATE)
			-- Truncate all records to `a_date' if applicable.
		local
			i : INTEGER
			l_record : ROSE_CLOSED_OPEN_INTERVAL [G]
		do
			from i := upper until i < lower loop
				l_record := item (i)
				if l_record.end_date.date > a_date.date then
					if l_record.start_date.date >= a_date.date then
						remove_i_th (i)
					else
						l_record.set_end_date (a_date.date)
					end
				end
				i := i - 1
			end
		end
end
