indexing
	description: "key for entity date pair, uses uid and date"
	author: "Mark Howard"
	last_editor: "$Author: rj $"
	date: "$Date: 2010/03/22 $"
	revision: "$Revision: #3 $"
	archive: "$File: //rose/source/Kernel/math/entity_date_key.e $"

class
	ENTITY_DATE_KEY
inherit
	DEBUG_OUTPUT
		undefine
			is_equal
		end
	HASHABLE
		undefine
			is_equal
		end
create
	make,
	make_from_uid_date,
	default_create

convert
	make_from_uid_date({TUPLE[uid: INTEGER_64; date: ROSE_DATE]})

feature --Initialization
	make(a_uid: INTEGER_64; a_date, a_available_date: ROSE_DATE) is
			--make from uid date pair
		do
			uid := a_uid
			date := a_date
			available_date := a_available_date
		end
	make_from_uid_date(a_key: TUPLE[uid: INTEGER_64; date: ROSE_DATE]) is
			--make from tuple
		do
			uid := a_key.uid
			date := a_key.date
			available_date := Void
		end
	debug_output : STRING is
			--display uid and date
		do
			if date/=Void
			then
				Result := uid.out + " : " + date.ccyymmdd.out
			else
				Result := uid.out + " : Void"
			end
		end

	set_available_date(a_date: like available_date)
	-- Although the key fields 'uid' and 'date' can't change, 'available_date' can.
	do
		available_date := a_date
	end

	set_valid
	-- In this class we are always valid so this call is legal.  Descendants may redefine this.
	do
	end

	set_invalid
	-- In this class we are always valid so this call is illegal.  Descendants may redefine this.
	do
		(create {EXCEPTIONS}).raise("Can't call 'set_invalid' in ENTITY_DATE_KEY")
	end

feature -- Serialization.
	custom_write_to_file(a_file: FILE; a_put_date_agent: PROCEDURE[ANY, TUPLE[FILE, ROSE_DATE]])
	-- Part of DATA_CUBE:custom_write_to_file.  Descendants will modify this code.
	do
		a_put_date_agent.call([a_file, available_date])
		a_put_date_agent.call([a_file, date])
		a_file.put_integer_64(uid)
	end

feature --Access
	uid : INTEGER_64
	date : ROSE_DATE
	available_date : ROSE_DATE
	hash_code : INTEGER is
			--hash code
		do
			Result := ([uid,date]).hash_code
		end

	is_equal(a_other: like Current): BOOLEAN is
			--compare uid and date
		do
			if a_other/= Void then
				Result:= uid = a_other.uid and then (date = a_other.date or else (date /= Void and then a_other.date /= Void and then date.same_date (a_other.date)))
			end
		end

	invalid: BOOLEAN
	-- In this class we are always valid.  Descendants may redefine this.
	do
	end

	valid: BOOLEAN
	-- See 'invalid'.
	do
		Result := not invalid
	end

end
