note

	description:

		"Possible matrix of data with each row corresponding to an entity and each column, a variable."

	author: "Feng Zhuo"
	last_editor: "$Author: er $"
	date: "$Date: 2013/06/18 $"
	revision: "$Revision: #77 $"
	path: "$File: //rose/source/kernel/entities/entity_matrix.e $"
	storable_version: ""

class ENTITY_MATRIX [reference G -> HAS_NAME]

inherit

	ENTITY_MATRIX_BASE[G]
		redefine
			matrix,
			extend_matrix,
			regenerate_variable_table
		end
create

	retrieve_from,
	make,
	make_all,
	retrieve_fixed,
	make_by_matrix,
	make_from_array,
	simple_make_from_array,
	make_from_entity_values

feature  --Access
	matrix : MATRIX
	add_new_entity_row (a_row: VECTOR; a_entity: G) is
			-- Add data row for `a_entity' if `a_entity' isn't present in `Currrent'.
		require
			entities_not_void: entities /= Void
			matrix_not_void: matrix /= Void
			a_row_not_void: a_row /= Void
			same_width: a_row.count = matrix.width
		do
			if not has_entity (a_entity) then
				matrix.add_row (a_row)
				entities.extend (a_entity)
				entity_table_saved := Void
			end
		end

	extend_matrix (a_matrix: MATRIX) is
			-- Extend `matrix' on the right with `a_matrix'.
		do
			Precursor(a_matrix)
			regenerate_variable_table
		end

	extend_normalized (a_name: STRING) is
			-- Extend `matrix' on the right with column named `a_name' normalized to sum to 1.0.
		require
			matrix_not_void: matrix /= Void
			valid_matrix_names: matrix.column_names /= Void
			variables_not_void: variables /= Void
			a_name_not_void: a_name /= Void
			variable_table_has_good_value_for_a_name: variable_index (a_name) >= 1 and variable_index (a_name) <= matrix.width
		local
			l_vector: VECTOR
		do
			l_vector := matrix.column (variable_index (a_name)).normalized
			l_vector.set_name (a_name + "_normalized")
			extend_matrix (l_vector.to_column_matrix)
		end

	regenerate_variable_table is
			-- Rebuild cache.
		do
			Precursor
			matrix.regenerate_column_table
		end

end
