note

	description:

		"Possible matrix of data with each row corresponding to an entity and each column, a variable."

	author: "Feng Zhuo"
	last_editor: "$Author: mb $"
	date: "$Date: 2014/03/03 $"
	revision: "$Revision: #5 $"
	path: "$File: //rose/source/kernel/entities/entity_matrix_base.e $"

class ENTITY_MATRIX_BASE [reference G -> HAS_NAME]

inherit

	ROSE_CONSTANTS
		export {NONE} all end

	ROSE_LEGACY_UNDEFINED
		export {NONE} all end

	ROSE_STORABLE_AS_FILE

	MATRIX_HELPER
		export {NONE} all end

	MISMATCH_CORRECTOR
		redefine
			correct_mismatch
		end

	ROSE_SHARED_GLOBAL
	
create
	make_all
	
feature {NONE} -- Initialization

	make is
			-- Creation procedure
		do
			do_nothing
		end

	make_all (a_entities: ROSE_LINEAR_ARRAY [G]; a_matrix: ABSTRACT_MATRIX; a_variables: ARRAY [STRING]) is
			-- Mutate `a_entities' then create from `a_entities', `a_matrix' and `a_variables'.
		require
			a_entities_not_void: a_entities /= Void
			a_entities_one_based: a_entities.lower = 1
			a_matrix_not_void: a_matrix /= Void
			a_variables_not_void: a_variables /= Void
		do
			set_entities (a_entities)
			set_matrix (a_matrix.as_matrix)
			create variables.make_from_array (a_variables)
			matrix.set_column_names (variables)
			set_dimension
		ensure
			a_entities_uses_equal: a_entities.object_comparison
			entities_aliased: entities = a_entities
			matrix_not_void: matrix /= Void
			variables_not_void: variables /= Void
			height_set: height = a_matrix.height
			width_set: width = a_matrix.width
		end

	make_by_matrix (a_entities: ROSE_LINEAR_ARRAY [G]; a_matrix: ABSTRACT_MATRIX) is
			-- Mutate `a_entities' then create from `a_entities' and `a_matrix'.
		require
			a_entities_not_void: a_entities /= Void
			a_entities_one_based: a_entities.lower = 1
			a_matrix_not_void: a_matrix /= Void
		do
			set_entities (a_entities)
			set_matrix (a_matrix.as_matrix)
			variables := a_matrix.column_names
			set_dimension
		ensure
			a_entities_uses_equal: a_entities.object_comparison
			entities_aliased: entities = a_entities
			matrix_not_void: matrix /= Void
			height_set: height = a_matrix.height
			width_set: width = a_matrix.width
			variables_set_from_matrix: variables ~ matrix.column_names
		end

	make_from_array (a_array: ARRAY [like Current]) is
			-- Create set of entities from entries in `a_array'.
		require
			a_array_not_void: a_array /= Void
			a_array_not_empty: not a_array.is_empty
			a_array_one_indexed: a_array.lower = 1
			no_void_entity_matrices: not a_array.has (Void)
			no_void_entities: True
				-- CATCALL: a_array.for_all (agent entities_not_void)
			no_void_matrix: True
				-- CATCALL: a_array.for_all (agent matrix_not_void)
			all_matrix_heights_match_entities_count: True
				-- CATCALL: a_array.for_all (item.matrix.height = item.entities.count)
			variables_not_void: a_array.item (1).variables /= Void
		local
			l_table: ROSE_HASH_TABLE [INTEGER, G]
			i, j, k, m, l_height: INTEGER
			l_entities, l_entities2: ROSE_LINEAR_ARRAY [G]
			l_entity: G
			l_matrix: MATRIX
		do
			width := a_array.item (1).matrix.width
			variables := a_array.item (1).variables
			k := a_array.count
			from
				i := 1
			until
				i > k
			loop
				l_height := l_height + a_array.item (i).height
				i := i + 1
			end
			create l_table.make (l_height)
			from
				i := 1
			until
				i > k
			loop
				l_entities := a_array.item (i).entities
				from
					l_entities.start
				until
					l_entities.after
				loop
					l_entity := l_entities.Current_item
					if not l_table.has (l_entity) then
						m := m + 1
						l_table.put (m, l_entity)
					end
					l_entities.forth
				end
				i := i + 1
			end
			create l_matrix.make (m, width)
			create l_entities2.make (1, m)
			from
				i := 1
			until
				i > k
			loop
				l_entities := a_array.item (i).entities
				from
					l_entities.start
				until
					l_entities.after
				loop
					l_entity := l_entities.current_item
					j := l_table.item (l_entity)
					l_entities2.put (l_entity, j)
					if {l_vector : VECTOR} (a_array.item (i).matrix.row (l_entities.index) + l_matrix.row (j)) then
						l_matrix.put_row_vector (l_vector, j)
					end
					l_entities.forth
				end
				i := i + 1
			end
			make_all (l_entities2, l_matrix, variables)
		ensure
			entities_not_void: entities /= Void
			matrix_not_void: matrix /= Void
			height_set: height = matrix.height
			width_set_from_first_entry: width = a_array.item (1).matrix.width
			variables_not_void: variables /= Void
		end

	simple_make_from_array (a_array: ARRAY [like Current]) is
			-- Create set of entities of entries in `a_array'.
			-- Takes into account difference in widths and variables of the matrices.
		require
			a_array_not_void: a_array /= Void
			a_array_not_empty: not a_array.is_empty
			a_array_one_indexed: a_array.lower = 1
			no_void_entries: not a_array.has (Void)
			a_array_last_entry_has_variables: a_array.item (a_array.count).variables /= Void
			no_void_entities: True
				-- CATCALL: a_array.for_all (agent entities_not_void)
		local
			l_entities: like entities
			l_matrice: ROSE_LINEAR_ARRAY [like matrix]
			i, k: INTEGER
			l_matrix: MATRIX
			l_variables: ROSE_LINEAR_ARRAY [STRING]
			l_new_array: like Current
		do
			k := a_array.count
			create l_matrice.make (1, k)
			from
				i := 1
			until
				i > k
			loop
				if i = 1 then
					l_entities := a_array.item (i).entities
					l_variables := a_array.item (i).variables
				else
					l_entities := l_entities + a_array.item (i).entities
					l_variables ?= union_string_array (l_variables, a_array.item (i).variables)
				end
				i := i + 1
			end
			from
				i := 1
			until
				i > k
			loop
				l_new_array := a_array.item (i).subset2 (l_variables)
				l_matrice.put (l_new_array.matrix, i)
				i := i + 1
			end
			create l_matrix.make_from_others (l_matrice)
			make_all (l_entities, l_matrix, l_variables)
		ensure
			entities_not_void: entities /= Void
			matrix_not_void: matrix /= Void
			variables_not_void: variables /= Void
		end

	make_from_entity_values (a_array: ARRAY [ENTITY_VALUES [G]]) is
			-- Create as horizontal concatenation of non-void entries from `a_array'.
			-- Result is a union of observations from `a_array'
		require
			a_array_not_void: a_array /= Void
			a_array_one_indexed: a_array.lower = 1
			non_empty: not a_array.is_empty
			first_entry_not_void: a_array [1] /= Void
		local
			l_table: ROSE_HASH_TABLE [INTEGER, G]
			l_next: ENTITY_VALUES [G]
			i, j, k: INTEGER
			l_max_height: INTEGER
			l_count: INTEGER
			l_matrix : MATRIX
		do
			l_count := a_array.count
			l_max_height := (((l_count / 6).max (3) * (a_array.item (1).count))).rounded
			create variables.make (1, l_count)
			create l_matrix.make (l_max_height, l_count)
			set_matrix(l_matrix)
			create entities.make (1, l_max_height)
			create l_table.make ((l_max_height / 2).rounded)
			from
				i := 1
			until
				i > a_array.count
			loop
				l_next := a_array.item (i)
				if l_next /= Void then
					variables.put (l_next.name, i)
					from
						l_next.start
					until
						l_next.after
					loop
						j := l_table.item (l_next.current_entity)
						if j > 0 then
							matrix.put (l_next.current_value, j, i)
						else
							-- Entity not yet seen, add to result
							k := k + 1
							if attached {MATRIX} matrix as al_non_sym_matrix then
								if k = matrix.height + 1 then
									set_matrix (al_non_sym_matrix.extend_row_vectors (<<create {VECTOR}.make (matrix.width)>>))
								end
							end
							matrix.put (l_next.current_value, k, i)
							entities.force (l_next.current_entity, k)
							l_table.put (k, l_next.current_entity)
						end
						l_next.forth
					end
				end
				i := i + 1
			end
			set_matrix (matrix.top_rows (k))
			matrix.set_column_names (variables)
			entities := entities.subarray (1, k)
		ensure
			variables_not_void: variables /= Void
			matrix_not_void: matrix /= Void
			entities_not_void: entities /= Void
		end
	
feature -- Basic operations

	to_excel (a_name: STRING): PAIR [ARRAY2 [ANY], STRING] is
			-- Nothing to do with excel
		require
			matrix_not_void: matrix /= Void
			entities_not_void: entities /= Void
			no_void_entity: not entities.has (Void)
		do
			Result := combined_to_excel (a_name, Void)
		ensure
			to_excel_not_void: Result /= Void
		end

	combined_to_excel (a_name: STRING; a_other: ARRAY2 [ANY]): PAIR [ARRAY2 [ANY], STRING] is
			-- Nothing to do with excel
		require
			matrix_not_void: matrix /= Void
			entities_not_void: entities /= Void
			no_void_entity: not entities.has (Void)
		do
			Result := matrix_combined_to_array2 (matrix, entity_names_with_brim, variables, a_name, a_other)
		ensure
			combined_to_excel_not_void: Result /= Void
		end

	combined_output (a_other: ARRAY2 [ANY]): ARRAY2 [ANY] is
			-- Untyped version of `matrix', suitable for writing to a CSV file
		require
			matrix_not_void: matrix /= Void
			entities_not_void: entities /= Void
			no_void_entity: not entities.has (Void)
		do
			Result := matrix.combined_output (entity_names, variables, a_other)
		end

	data_summary_statistics_to_excel (a_name: STRING): PAIR [ARRAY2 [ANY], STRING] is
			-- Data summary and covariance matrix paired with `a_name';
			-- This can abort due to a bug im MATRIX (ticket #82088).
		require
			matrix_not_void: matrix /= Void
		do
			Result := matrix_to_excel (data_summary, a_name)
		ensure
			data_summary_statistics_to_excel_not_void: Result /= Void
		end

	row_data_summary_statistics_to_excel (a_name: STRING): PAIR [ARRAY2 [ANY], STRING] is
			-- Data summary statistics of rows paired with `a_name';
			-- This can abort due to a bug im MATRIX (ticket #82088).
		require
			matrix_not_void: matrix /= Void
		do
			Result := matrix_to_excel (row_data_summary, a_name)
		ensure
			row_data_summary_statistics_to_excel_not_void: Result /= Void
		end

	row_data_summary: MATRIX is
			-- Data summary statistics of rows;
			-- Should not be commonly used.
			-- This can abort due to a bug im MATRIX (ticket #82088).
		require
			matrix_not_void: matrix /= Void
		do
			Result := matrix.statistics_of_the_rows (Void)
		end

	data_variance_covariance_to_excel (a_name: STRING): PAIR [ARRAY2 [ANY], STRING] is
			-- Data summary and covariance matrix paired with `a_name'
		require
			matrix_not_void: matrix /= Void
		do
			Result := matrix_to_excel (data_variance_covariance, a_name)
		ensure
			data_variance_covariance_to_excel_not_void: Result /= Void
		end

	normalize_column (a_name: STRING) is
			-- Normalize column named `a_name', if possible.
		require
			variables_not_void: variables /= Void
			matrix_not_void: matrix /= Void
		local
			i: INTEGER
		do
			i := variable_index (a_name)
			if i > 0 then
				if {l_column : VECTOR} matrix.column (i) then
					if l_column.sum > 0 then
						matrix.put_column_vector (l_column.normalized, i)
					end
				end
			end
		end

	store_fixed (a_name, a_ext: STRING) is
			-- Set `name' to `a_name' then store (in independent format) `Current' in file `fullname (a_name, a_ext)'.
		require
			a_name_not_void: a_name /= Void
		local
			l_path, l_dir: STRING
			l_file_system: KL_SHARED_FILE_SYSTEM
		do
			set_name (a_name)
			l_path := full_name (a_name, a_ext)
			create l_file_system
			l_dir := l_file_system.file_system.dirname (l_path)
			if not l_file_system.file_system.directory_exists (l_dir) then
				l_file_system.file_system.recursive_create_directory (l_dir)
			end
			store_as (l_path)
		ensure
			name_aliased: name = a_name
		end

	retrieve_fixed (a_name, a_ext: STRING) is
			-- Retrieve `Current' from `fullname (a_name, a_ext)' and set `name' to `a_name'
		require
			a_name_not_void: a_name /= Void
		do
			retrieve_from (full_name (a_name, a_ext))
			name := a_name
		ensure
			name_aliased: name = a_name
		end

	set_row_names is
			-- set row names
		require
			matrix_not_void: matrix /= Void
			entities_not_void: entities /= Void
			no_void_entity: not entities.has (Void)
		do
			matrix.set_row_names (entity_names)
		end

	set_dimension is
			-- Set `height' and `width' to match `matrix'.
		require
			matrix_not_void: matrix /= Void
		do
			height := matrix.height
			width := matrix.width
		ensure
			height_set: height = matrix.height
			width_set: width = matrix.width
		end

	regenerate_variable_table is
			-- Rebuild cache.
		require
			matrix_not_void: matrix /= Void
			variables_not_void: variables /= Void
		do
			variable_table_saved := Void
			variable_table.do_nothing
		end

	sort_descending (a_var: STRING) is
			-- Sort `current' so that the column named by `a_var' is in descending order.
		require
			matrix_not_void: matrix /= Void
			variables_not_void: variables /= Void
			a_var_good: variable_index (a_var) > 0 and variable_index (a_var) <= matrix.width
		local
			l_ents: ENTITY_VALUES [G]
			l_matrix: like matrix
			l_entities: ROSE_LINEAR_ARRAY [G]
		do
			l_ents := selected_values (a_var)
			l_ents.sort_descending
			l_entities := l_ents.entities
			l_matrix := subset (l_entities, Void).matrix
			reset (l_entities, l_matrix.as_matrix)
		end

	sort_ascending (a_var: STRING) is
			-- Sort `current' so that the column named by `a_var' is inaescending order.
		require
			matrix_not_void: matrix /= Void
			variables_not_void: variables /= Void
			a_var_good: variable_index (a_var) > 0 and variable_index (a_var) <= matrix.width
		local
			l_ents: ENTITY_VALUES [G]
			l_matrix: like matrix
			l_entities: ROSE_LINEAR_ARRAY [G]
		do
			l_ents := selected_values (a_var)
			l_ents.sort_ascending
			l_entities := l_ents.entities
			l_matrix := subset (l_entities, Void).matrix
			reset (l_entities, l_matrix.as_matrix)
		end

feature -- Setting

	set_source (a_source: STRING) is
			-- Set source to `a_source'
		do
			source := a_source
		ensure
			source_aliased: source = a_source
		end

	set_cash (a_cash: DOUBLE) is
			-- Set cash to `a_cash'.
		do
			cash := a_cash
		ensure
			cash_set: (a_cash = a_cash) implies cash = a_cash
		end

	set_aum (a_aum: DOUBLE) is
			-- Set `aum' to `a_aum'.
		do
			aum := a_aum
		ensure
			aum_set: (a_aum = a_aum) implies aum = a_aum
		end

	set_date (a_date: ROSE_DATE) is
			-- Set `date' to `a_date'.
		do
			date := a_date
		ensure
			date_set: date = a_date
		end

	set_entities (a_entities: ROSE_LINEAR_ARRAY [G]) is
			-- Mutate `a_entities' to use `equal' for comparison, then set `entities' to `a_entities'.
		require
			a_entities_not_void: a_entities /= Void
			a_entities_one_based: a_entities.lower = 1
		do
			entities := a_entities
			entities.compare_objects
		ensure
			a_entities_uses_equal: a_entities.object_comparison
			entities_aliased: entities = a_entities
		end

	set_matrix (a_matrix: MATRIX) is
			-- Set `matrix' to `a_matrix'.
		do
			matrix := a_matrix
		ensure
			matrix_not_void: matrix /= Void
			matrix_set: matrix.is_equivalent (a_matrix)
		end

	set_variables (a_variables: ROSE_LINEAR_ARRAY [STRING]) is
			-- Set `variables' to `a_variables'.
		require
			matrix_not_void: matrix /= Void
			a_variables_one_based: a_variables /= Void implies a_variables.lower = 1
		do
			variables := a_variables
			matrix.set_column_names (variables)
		ensure
			variables_aliased: variables = a_variables
		end

	set_name (a_name: STRING) is
			-- Set `name' to `a_name'.
		do
			name := a_name
		ensure
			name_aliased: name = a_name
		end

feature -- Transformations

	replace_matrix (a_matrix: like matrix): like Current is
			-- Version of `Current' with `matrix' replaced by `a_matrix'.
			-- All the application-specific data and additional indices associated with the class is dropped from `Result'.
		require
			entities_not_void: entities /= Void
			variables_not_void: variables /= Void
			a_matrix_not_void: a_matrix /= Void
		do
			create Result.make_all (entities, a_matrix, variables)
		ensure
			replace_matrix_not_void: Result /= Void
			entities_uses_equal: entities.object_comparison
			entities_aliased: Result.entities = entities
			matrix_not_void: Result.matrix /= Void
			variables_not_void: Result.variables /= Void
			height_set: Result.height = a_matrix.height
			width_set: Result.width = a_matrix.width
		end

feature -- Element change

--	extend_matrix (a_matrix: MATRIX) is
--			-- Extend `matrix' on the right with `a_matrix'.
--		require
--			matrix_not_void: matrix /= Void
--			a_matrix_not_void: a_matrix /= Void
--			same_height: matrix.height = a_matrix.height
--			valid_matrix_names: a_matrix.column_names /= Void
--			variables_not_void: variables /= Void
--		do
--			set_matrix (matrix.extend_matrix (a_matrix))
--			variables := variables + a_matrix.column_names
--			set_dimension
--			regenerate_variable_table
--		end

--	extend_normalized (a_name: STRING) is
--			-- Extend `matrix' on the right with column named `a_name' normalized to sum to 1.0.
--		require
--			matrix_not_void: matrix /= Void
--			valid_matrix_names: matrix.column_names /= Void
--			variables_not_void: variables /= Void
--			a_name_not_void: a_name /= Void
--			variable_table_has_good_value_for_a_name: variable_index (a_name) >= 1 and variable_index (a_name) <= matrix.width
--		local
--			l_vector: VECTOR
--		do
--			l_vector := matrix.column (variable_index (a_name)).normalized
--			l_vector.set_name (a_name + "_normalized")
--			extend_matrix (l_vector.to_column_matrix)
--		end

	replace_column_vector (a_variable: STRING; a_vector: attached VECTOR) is
			-- Replace that part of column named `a_variable' which is covered by `a_vector', with the contents of `a_vector'.
		require
			variables_not_void: variables /= Void
			matrix_not_void: matrix /= Void
		local
			l_index: INTEGER
		do
			l_index := variable_index (a_variable)
			if matrix.valid_column_index (l_index) then
				matrix.put_column_vector (a_vector, l_index)
			end
		end

	replace_column_null (a_vars: ROSE_LINEAR_ARRAY [STRING]) is
			-- Replaces -999s or undefineds with 0 in columns named in `a_vars'.
		require
			matrix_not_void: matrix /= Void
			a_vars_not_void: a_vars /= Void
		local
			l_vars: ROSE_LINEAR_ARRAY [INTEGER]
		do
			l_vars := variables_index (a_vars)
			from
				l_vars.start
			until
				l_vars.after
			loop
				matrix.replace_column_null (l_vars.current_item)
				l_vars.forth
			end
		end

	replace_matrix_element (a_entity: G; a_variable: STRING; a_value: DOUBLE) is
			-- Replace matrix element for `a_entity'-`a_variable' pair with `a_value'.
		require
			matrix_not_void: matrix /= Void
			entities_not_void: entities /= Void
			variables_not_void: variables /= Void
		do
			if has_entity (a_entity) and matrix.valid_column_index (variable_index (a_variable)) then
				matrix.put (a_value, entity_index (a_entity), variable_index (a_variable))
			end
		end

	add_values (a_row: VECTOR; a_entity: G) is
			-- Add `a_row' to row of `matrix' indexed by `a_entity'.
		require
			matrix_not_void: matrix /= Void
			a_row_not_void: a_row /= Void
			dimensions_match: matrix.width = a_row.count
			entities_not_void: entities /= Void
		local
			k: INTEGER
		do
			k := entity_index (a_entity)
			if k > 0 then
				matrix.row_add (a_row, k)
			end
		end

	do_all (a_agent: PROCEDURE [ANY, TUPLE [G, VECTOR]]) is
			-- Apply `a_agent' to all in `entities', paired with associated values.
			-- `a_agent' is assured that it's second argument is not `Void'.
		require
			a_agent_not_void: a_agent /= Void
			matrix_not_void: matrix /= Void
			entities_not_void: entities /= Void
			matrix_large_enough: matrix.height >= entities.count
		local
			i: INTEGER
			l_count: INTEGER
		do
			from
				i := 1
				l_count := entities.count
			until
				i > l_count
			loop
				a_agent.call ([entities.item (i), matrix.row (i)])
				i := i + 1
			end
		end

feature -- Access

	sub_values (a_entities: ROSE_LINEAR_ARRAY [G]; a_variables: ROSE_LINEAR_ARRAY [STRING]): MATRIX is
			-- Defined values indexed by both `a_entities' and `a_variables';
			-- Results appear in positions determined by order of `a_entities' and `a_variables',
			--  and NOT by their order in `matrix'.
		require
			entities_not_void: entities /= Void
			a_entities_not_void: a_entities /= Void
			a_variables_not_void: a_variables /= Void
			variables_not_void: variables /= Void
			matrix_not_void: matrix /= Void
		local
			i, j, l_eix, l_vix: INTEGER
			l_v: DOUBLE
		do
			create Result.make (a_entities.count, a_variables.count)
			from
				l_eix := a_entities.lower
			until
				l_eix > a_entities.upper
			loop
				i := entity_index (a_entities [l_eix])
				from
					l_vix := a_variables.lower
				until
					l_vix > a_variables.upper
				loop
					j := variable_index (a_variables [l_vix])
					if matrix.valid_column_index (j) and matrix.valid_row_index (i) then
						l_v := matrix.item (i, j)
						if not is_defined_double (l_v) then
							l_v := 0
						end
						Result.put (l_v, l_eix, l_vix)
					end
					l_vix := l_vix + 1
				end
				l_eix := l_eix + 1
			end
		ensure
			sub_values_not_void: Result /= Void
			all_values_defined: True
				-- MATRIX has no such routine!
			one_row_per_entity: Result.height = a_entities.count
			one_column_per_variable: Result.width = a_variables.count
			no_name: Result.name = Void
			no_column_names: Result.column_names = Void
			no_row_names: Result.row_names = Void
		end

feature -- Access

	name: STRING
			-- Optional name for `Current'

	source: STRING
			-- An application-specific (i.e. meaningless) attribute

	cash: DOUBLE
			-- Another application-specific attribute

	aum: DOUBLE
			-- Asset under management when used for portfolio holdings (i.e. another application-specific attribute)

	date: ROSE_DATE
			--	Optional date for `Current' - application specific

	entities: ROSE_LINEAR_ARRAY [G]
			-- Indices for some of the rows of `matrix'

	variables: ROSE_LINEAR_ARRAY [STRING]
			-- Column names for some of the rows of `matrix'

	matrix: ABSTRACT_MATRIX
			-- Optional (!?!) data

	height: INTEGER
			-- Ought to be `matrix.height' when `matrix' is non-Void. Why not use `matrix.height' to be sure?

	width: INTEGER
			-- Ought to be `matrix.width' when `matrix' is non-Void. Why not use `matrix.width' to be sure?

	ext: STRING is
			-- Extension for storable file names
		once
			Result := "emx"
		ensure
			ext_not_void: Result /= Void
		end

	full_name (a_name, a_ext: STRING): STRING is
			-- Varying naming convention for storable files
		require
			a_name_not_void: a_name /= Void
		do
			if a_ext = Void then
				Result := global.app.storable_path (ext + "\" + a_name + "." + ext)
			else
				Result := global.app.storable_path (a_ext + "\" + a_name + "." + a_ext)
			end
		ensure
			full_name_not_void: Result /= Void
			full_name_not_empty: not Result.is_empty
			full_name_is_absolute: global.app.formatter.is_absolute_path (Result)
		end

	stored_name: STRING is
			-- <Precursor>
		do
			do_nothing
		ensure then
			stored_name_void: Result = Void
		end

	selected_values (a_name: STRING): ENTITY_VALUES [G] is
			-- Values from column named by `a_name'.
		require
			matrix_not_void: matrix /= Void
			variables_not_void: variables /= Void
			entities_not_void: entities /= Void
			correct_entities_count: entities.count = matrix.height
			a_name_good: variable_index (a_name) > 0 and variable_index (a_name) <= matrix.width
		local
			i: INTEGER
		do
			i := variable_index (a_name)
			create Result.make_from_entities_vector (matrix.column (i), entities)
		ensure
			selected_values_not_void: Result /= Void
			correct_count: Result.count = entities.count
			name_void: Result.name = Void
			same_entities: Result.entities.same_items (entities)
			correct_values: Result.raw_vector.same_vector (matrix.column (variable_index (a_name)))
		end

	entity_table: ROSE_HASH_TABLE [INTEGER, G] is
			-- Map of `entities' to 1-based index of `entities';
			-- N.B. this code is slighly inefficeint in the light of the invariant that entities.lower = 1.
		require
			entities_not_void: entities /= Void
		local
			l_index: INTEGER
		do
			if entity_table_saved = Void then
				l_index := entities.index
				create Result.make (height)
				from
					entities.start
				until
					entities.after
				loop
					Result.put (entities.index - entities.lower + 1, entities.current_item)
					entities.forth
				end
				entities.go_i_th (l_index)
				entity_table_saved := Result
			else
				Result := entity_table_saved
			end
		ensure
			entity_table_not_void: Result /= Void
				-- and NOT the following :-( as `set_entities' does not void the cache:
				-- entities.for_all (agent entity_table.has)
				-- It would have been nice to add it as a conditional invariant
		end

	variable_table: ROSE_HASH_TABLE [INTEGER, STRING] is
			-- Index of 1-based index into `variables';
			-- N.B. this code is slighly inefficeint in the light of the invariant that variables.lower = 1.
		require
			variables_not_void: variables /= Void
		local
			l_index: INTEGER
		do
			if variable_table_saved = Void then
				create Result.make (variables.count)
				l_index := variables.index
				from
					variables.start
				until
					variables.after
				loop
					Result.put (variables.index - variables.lower + 1, variables.current_item)
					variables.forth
				end
				variables.go_i_th (l_index)
				variable_table_saved := Result
			else
				Result := variable_table_saved
			end
		ensure
			variable_table_not_void: Result /= Void
				-- and specifically, the following is NOT necessarily true, as `set_variables' does not void the cache :-) :
				--  variables.for_all (agent variable_table.has)
				-- It would have been nice to add it as a conditional invariant.
		end

	variables_index (a_variables: ARRAY [STRING]): ROSE_LINEAR_ARRAY [INTEGER] is
			-- Column index in `matrix' of each of `a_variables'
		require
			variables_not_void: variables /= Void
			a_variables_not_void: a_variables /= Void
		local
			i, k: INTEGER
		do
			k := a_variables.count
			create Result.make (1, k)
			from
				i := 1
			until
				i > k
			loop
				Result.put (variable_table.item (a_variables.item (i)), i)
				i := i + 1
			end
		ensure
			variables_index_not_void: Result /= Void
			one_based: Result.lower = 1
			correct_count: Result.count = a_variables.count
		end

	variable_index (a_variable: STRING): INTEGER is
			-- Column index in `matrix' for `a_variable'
		require
			variables_not_void: variables /= Void
		do
			Result := variable_table.item (a_variable)
		end

	entity_index (a_entity: G): INTEGER is
			-- Position of `a_entity' within `entities', with something possibly different for companies (???)
		require
			entities_not_void: entities /= Void
		local
			l_company, l_g: PERSISTENT_COMPANY
			l_persistent_company: PERSISTENT_COMPANY
			l_company1, l_company2: COMPANY
			l_idx: INTEGER
		do
			Result := entity_table.item (a_entity)
			l_persistent_company ?= a_entity
			l_company1 ?= a_entity
			if Result > 0 then
				if l_company1 /= Void then
					l_company2 ?= entities.item (Result)
					if l_company2 /= Void and then not (l_company1.xrf_code = l_company2.xrf_code) then
						l_idx := entities.index
						Result := entities.index_of (a_entity, 2)
						entities.go_i_th (l_idx)
					end
				elseif l_persistent_company /= Void then
					l_g ?= entities.item (Result)
					l_company ?= a_entity
					if (l_g /= Void and l_company /= Void) and then l_company.xrf_code_key (Void) /= Void and then not l_company.xrf_code_key (Void).is_equal (l_g.xrf_code_key (Void)) then
						l_idx := entities.index
						Result := entities.index_of (a_entity, 2)
						entities.go_i_th (l_idx)
					end
				end
			end
		end

	has_entity (a_entity: G): BOOLEAN is
			-- Does `entities' contain `a_entity'?
		require
			entities_not_void: entities /= Void
		do
			Result := a_entity /= Void and then entity_table.has (a_entity)
		ensure
			definition: Result implies entities.has (a_entity)
		end

	entities_index (a_entities: ARRAY [G]): ROSE_LINEAR_ARRAY [INTEGER] is
			-- `entity_index' for each of `a_entities' present in `Current';
			-- `Void' if all are zero (i.e. none found);
			-- If `a_entities' = `Void' then instead we have `1 |..| height' as an array.
		require
			entities_not_void: entities /= Void
		local
			i, j, k, m: INTEGER
		do
			if a_entities /= Void then
				k := a_entities.count
				create Result.make (1, k)
				from
					i := 1
				until
					i > k
				loop
					j := entity_index (a_entities.item (i))
					if j > 0 then
						m := m + 1
						Result.put (j, m)
					end
					i := i + 1
				end
				if m > 0 then
					Result := Result.subarray (1, m)
				else
					Result := Void
				end
			else
				create Result.make (1, height)
				from
					i := 1
				until
					i > height
				loop
					Result.put (i, i)
					i := i + 1
				end
			end
		ensure
			all_strictly_positive: Result /= Void implies Result.for_all (agent greater_than_zero)
		end

	item (a_entity: G; a_variable: STRING): DOUBLE is
			-- Value associated with `a_entity' at `a_variable';
			-- Same as `value_at' except `0.0' is returned for missing values.
		require
			entities_not_void: entities /= Void
			variables_not_void: variables /= Void
			matrix_not_void: matrix /= Void
		local
			i, j: INTEGER
		do
			i := entity_index (a_entity)
			j := variable_index (a_variable)
			if i > 0 and j > 0 then
				Result := matrix.item (i, j)
			else
				Result := 0.0
			end
		end

	scaled (a_ratio: DOUBLE): like Current is
			-- Multiplication of all values by `a_ratio'
		require
			matrix_not_void: matrix /= Void
			entities_not_void: entities /= Void
			variables_not_void: variables /= Void
		do
			Result := replace_matrix (matrix.scaled (a_ratio))
		ensure
			scaled_not_void: Result /= Void
			entities_uses_equal: Result.entities.object_comparison
			entities_aliased: Result.entities = entities
			matrix_not_void: Result.matrix /= Void
			variables_not_void: variables /= Void
			height_set: Result.height = matrix.height
			width_set: Result.width = matrix.width
		end

	value_at (a_entity: G; a_variable: STRING): DOUBLE is
			-- Value associated with `a_entity' at `a_variable';
			-- Same as `item' except `undefined_double' is returned for missing values.
		require
			entities_not_void: entities /= Void
			variables_not_void: variables /= Void
			matrix_not_void: matrix /= Void
		local
			i, j: INTEGER
		do
			i := entity_index (a_entity)
			j := variable_index (a_variable)
			if i > 0 and j > 0 then
				Result := matrix.item (i, j)
			else
				Result := undefined_double
			end
		end

	values_at (a_entity: G): VECTOR is
			-- All variable values of `a_entity'
		require
			entities_not_void: entities /= Void
			matrix_not_void: matrix /= Void
		local
			i: INTEGER
		do
			i := entity_index (a_entity)
			if i > 0 then
				Result := matrix.row (i)
				Result.set_names (variables)
			end
		ensure
			void_if_unknown: (Result = Void) = (entity_index (a_entity) = 0)
			correct_count: Result /= Void implies Result.count = matrix.width
			names_set: Result /= Void implies (Result.names ~ variables)
		end

	weighted_mean (a_etv: ENTITY_VALUES [G]; a_vars: ARRAY [STRING]; a_check_unavailables: BOOLEAN): VECTOR is
			-- Weighted mean of a subset of `variables' specified by `a_vars' (or all if `a_vars' is Void);
			-- Weights are supplied in `a_etv'.
			-- See also: `weighted_standard_deviation'
		require
			entities_not_void: entities /= Void
			a_etv_not_void: a_etv /= Void
			variables_not_void: variables /= Void
			a_vars_well_formed: a_vars /= Void implies a_vars.for_all (agent variables.has)
			matrix_not_void: matrix /= Void
		local
			l_entities: ROSE_LINEAR_ARRAY [G]
			l_ent_array: ROSE_LINEAR_ARRAY [INTEGER]
			l_weights: VECTOR
			l_sub_matrix: MATRIX
			l_vars: ROSE_LINEAR_ARRAY [STRING]
		do
			l_vars := defaulted_variables (a_vars)
			l_entities := a_etv.entities
			l_ent_array := entities_index (l_entities)
			if l_ent_array /= Void and then not l_ent_array.is_empty then
				l_entities := entities.subset (l_ent_array)
				l_weights := a_etv.vector_at (l_entities)
				create l_sub_matrix.make_sub_matrix_3 (matrix, l_ent_array, variables_index (l_vars))
				if a_check_unavailables then
					Result := l_sub_matrix.column_mean_with_unavailables (l_weights, agent is_defined)
				else
					l_sub_matrix.replace_null
					Result := l_sub_matrix.column_mean (l_weights)
				end
			else
				create Result.make (l_vars.count)
			end
			Result.set_names (l_vars)
		ensure
			weighted_mean_not_void: Result /= Void
		end

	valid_variable_flags (a_etv: ENTITY_VALUES [G]; a_vars: ARRAY [STRING]; a_min_wt_with_valid_values: DOUBLE): VECTOR is
			-- Validity flags for a subset of `variables' specified by `a_vars' (or all if `a_vars' is Void);
			-- Weights are supplied in `a_etv'.
			-- If the sum of weight with valid values for a variable is greater than a_min_wt_with_valid_values then 1, else 0.
		require
			entities_not_void: entities /= Void
			a_etv_not_void: a_etv /= Void
			variables_not_void: variables /= Void
			a_vars_well_formed: a_vars /= Void and then not a_vars.has(Void)
			matrix_not_void: matrix /= Void
		local
			l_entities: ROSE_LINEAR_ARRAY [G]
			l_ent_array: ROSE_LINEAR_ARRAY [INTEGER]
			l_weights: VECTOR
			l_sub_matrix: MATRIX
			l_vars: ROSE_LINEAR_ARRAY [STRING]
		do
			l_vars := defaulted_variables (a_vars)
			l_entities := a_etv.entities
			l_ent_array := entities_index (l_entities)
			if l_ent_array /= Void and then not l_ent_array.is_empty then
				l_entities := entities.subset (l_ent_array)
				l_weights := a_etv.vector_at (l_entities)
				create l_sub_matrix.make_sub_matrix_3 (matrix, l_ent_array, variables_index (l_vars))
				Result := l_sub_matrix.column_valid_variable_flags (l_weights, agent is_defined, a_min_wt_with_valid_values)
			else
				create Result.make (l_vars.count)
			end
			Result.set_names (l_vars)
		ensure
			result_not_void: Result /= Void
		end

	weighted_standard_deviation (a_etv: ENTITY_VALUES [G]; a_vars: ARRAY [STRING]; a_check_unavailables: BOOLEAN): VECTOR is
			-- Weighted standard deviation of a subset of `variables' specified by `a_vars' (or all if `a_vars' is Void);
			-- Weights are supplied in `a_etv'.
			-- See also: `weighted_mean'.
		require
			entities_not_void: entities /= Void
			a_etv_not_void: a_etv /= Void
			variables_not_void: variables /= Void
			a_vars_well_formed: a_vars /= Void implies a_vars.for_all (agent variables.has)
			matrix_not_void: matrix /= Void
		local
			l_entities: ROSE_LINEAR_ARRAY [G]
			l_ent_array: ROSE_LINEAR_ARRAY [INTEGER]
			l_weights: VECTOR
			l_sub_matrix: MATRIX
			l_vars: ROSE_LINEAR_ARRAY [STRING]
		do
			l_vars := defaulted_variables (a_vars)
			l_entities := a_etv.entities
			l_ent_array := entities_index (l_entities)
			if l_ent_array /= Void then
				l_entities := entities.subset (l_ent_array)
				l_weights := a_etv.vector_at (l_entities)
				create l_sub_matrix.make_sub_matrix_3 (matrix, l_ent_array, variables_index (l_vars))
				if a_check_unavailables then
					Result := l_sub_matrix.column_standard_deviation_with_unavailables (l_weights, agent is_defined)
				else
					l_sub_matrix.replace_null
					Result := l_sub_matrix.column_standard_deviation (l_weights)
				end
				Result.set_names (l_vars)
			end
		end

	valid_subset (a_vars: ARRAY [STRING]): like Current is
			-- If `a_vars' represents all columns, then row-subset of `Current' with all valid data
		require
			a_vars_not_void: a_vars /= Void
			variables_not_void: variables /= Void
			matrix_not_void: matrix /= Void
			entities_not_void: entities /= Void
		local
			l_entities: ROSE_LINEAR_ARRAY [G]
			l_var_array, l_rows: ROSE_LINEAR_ARRAY [INTEGER]
			l_matrix: MATRIX
			i, j, k: INTEGER
			l_vector: VECTOR
			l_valid: BOOLEAN
		do
			l_var_array := variables_index (a_vars)
			l_matrix := matrix.selected_columns (l_var_array)
			create l_rows.make (1, l_matrix.height)
			from
				i := 1
			until
				i > l_matrix.height
			loop
				l_vector := l_matrix.row (i)
				l_valid := True
				from
					j := 1
				until
					j > l_vector.count or not l_valid
				loop
					l_valid := l_valid and is_defined_double (l_vector.item (j))
					j := j + 1
				end
				if l_valid then
					k := k + 1
					l_rows.put (i, k)
				end
				i := i + 1
			end
			if k > 1 then
				l_rows := l_rows.subarray (1, k)
				l_entities := entities.subset (l_rows)
				l_matrix := matrix.selected_rows (l_rows)
				create Result.make_all (l_entities, l_matrix, variables)
			end
		ensure
			entities_uses_equal: Result /= Void implies (Result.entities /= Void and then Result.entities.object_comparison)
			matrix_not_void: Result /= Void implies Result.matrix /= Void
			variables_not_void: Result /= Void implies Result.variables /= Void
			all_defined: Result /= Void implies True
				-- MATRIX doesn't have such a feature
		end

	valid_subset_with_individual_checks (a_vars: ARRAY [STRING]; a_checks: ARRAY [FUNCTION [ANY, TUPLE [DOUBLE], BOOLEAN]]): like Current is
			-- If `a_vars' represents all columns, then row-subset of `Current' with all valid data
			--  and which pass `a_checks' [that_column]
		require
			a_vars_not_void: a_vars /= Void
			variables_not_void: variables /= Void
			matrix_not_void: matrix /= Void
			entities_not_void: entities /= Void
			a_checks_not_void: a_checks /= Void
			no_void_check: not a_checks.has (Void)
			same_count: a_checks.count >= a_vars.count
		local
			l_entities: ROSE_LINEAR_ARRAY [G]
			l_var_array, l_rows: ROSE_LINEAR_ARRAY [INTEGER]
			l_matrix: MATRIX
			i, j, k: INTEGER
			l_vector: VECTOR
			l_valid: BOOLEAN
		do
			l_var_array := variables_index (a_vars)
			l_matrix := matrix.selected_columns (l_var_array)
			create l_rows.make (1, l_matrix.height)
			from
				i := 1
			until
				i > l_matrix.height
			loop
				l_vector := l_matrix.row (i)
				l_valid := True
				from
					j := 1
				until
					j > l_vector.count or not l_valid
				loop
					l_valid := l_valid and is_defined_double (l_vector.item (j)) and then a_checks.item (j).item ([l_vector.item (j)])
					j := j + 1
				end
				if l_valid then
					k := k + 1
					l_rows.put (i, k)
				end
				i := i + 1
			end
			if k > 1 then
				l_rows := l_rows.subarray (1, k)
				l_entities := entities.subset (l_rows)
				l_matrix := matrix.selected_rows (l_rows)
				create Result.make_all (l_entities, l_matrix, variables)
			end
		ensure
			entities_uses_equal: Result /= Void implies (Result.entities /= Void and then Result.entities.object_comparison)
			matrix_not_void: Result /= Void implies Result.matrix /= Void
			variables_not_void: Result /= Void implies Result.variables /= Void
			all_defined: Result /= Void implies True
				-- MATRIX doesn't have such a feature
		end

	top_n_subset (a_n: INTEGER): like Current is
			-- Quite often the first `a_n' rows of `Current'
		require
			entities_not_void: entities /= Void
			matrix_not_void: matrix /= Void
			a_n_non_negative: a_n >= 0
		local
			l_entities: ROSE_LINEAR_ARRAY [G]
			l_matrix: MATRIX
		do
			if entities.count > a_n then
				l_entities := entities.subarray (entities.lower, a_n)
				l_matrix := matrix.top_rows (a_n)
				create Result.make_all (l_entities, l_matrix, variables)
				Result.set_date (date)
			elseif a_n > 0 then
				create l_matrix.duplicate (matrix)
				create Result.make_all (entities.twin, l_matrix, variables)
			end
		ensure
			top_n_subset_usually_not_void: (a_n > 0 or entities.count > a_n) implies Result /= Void
			entities_not_void: Result.entities /= Void
			matrix_not_void: Result.matrix /= Void
			correct_height: Result.height = a_n.min (entities.count)
			same_width: Result.width = matrix.width
			same_variables: Result.variables ~ variables
		end

	valid_subset_for (a_check_vars, a_selected_vars: ARRAY [STRING]): like Current is
			-- `valid_subset (a_check_vars)' updated by those colimns not named by `a_selected_vars'
			--  are set to legacy undefined doubles
		require
			a_check_vars_not_void: a_check_vars /= Void
			a_selected_vars_not_void: a_selected_vars /= Void
			variables_not_void: variables /= Void
			matrix_not_void: matrix /= Void
			entities_not_void: entities /= Void
		local
			l_var_array: ROSE_LINEAR_ARRAY [INTEGER]
			l_matrix: MATRIX
			l_vars: ROSE_LINEAR_ARRAY [STRING]
			l_emx: like Current
		do
			l_emx := valid_subset (a_check_vars)
			if l_emx /= Void then
				l_var_array := variables_index (a_selected_vars)
				l_var_array.prune_all (0)
				l_vars := variables.subset (l_var_array)
				l_matrix := l_emx.matrix.selected_columns (l_var_array)
				l_matrix.set_column_names (l_vars)
				create Result.make_all (l_emx.entities, l_matrix, l_vars)
			end
		ensure
			entities_uses_equal: Result /= Void implies (Result.entities /= Void and then Result.entities.object_comparison)
			matrix_not_void: Result /= Void implies Result.matrix /= Void
			variables_not_void: Result /= Void implies Result.variables /= Void
		end

	vector_for_entity (a_entity: G): VECTOR is
			-- Values in row identifed by `a_entity'
		require
			entities_not_void: entities /= Void
			a_entity_known: entity_index (a_entity) > 0
			matrix_not_void: matrix /= Void
		do
			Result := matrix.row (entity_index (a_entity))
			Result.set_names (matrix.column_names)
		ensure
			vector_for_entity_not_void: Result /= Void
			correct_count: Result.count = matrix.width
				-- not necessarily = `width'
		end

	vector_for_name (a_name: STRING): VECTOR is
			-- Column of `Current' identified by `a_name'
		require
			matrix_not_void: matrix /= Void
		do
			Result := matrix.column_by_name (a_name)
		ensure
			correct_count: Result /= Void implies Result.count = matrix.height
		end

	filtered_subset (a_filter: FUNCTION [ANY, TUPLE [G], BOOLEAN]): like Current is
			-- Subset consisting of rows whose entity satisfies `a_filter'
		require
			entities_not_void: entities /= Void
			entities_1_based: entities.lower = 1
		local
			l_entities: like entities
			i, k: INTEGER
		do
			create l_entities.make (1, entities.count)
			from
				i := 1
			until
				i > entities.count
			loop
				if a_filter.item ([entities.item (i)]) then
					k := k + 1
					l_entities.put (entities.item (i), k)
				end
				i := i + 1
			end
			l_entities := l_entities.subarray (1, k)
			Result := subset (l_entities, Void)
		ensure
			filtered_subset_not_void: Result /= Void
			entities_uses_equal: Result.entities /= Void and then Result.entities.object_comparison
			matrix_not_void: Result.matrix /= Void
			variables_not_void: variables /= Void
			height_set: Result.height = Result.matrix.height
			width_set: Result.width = matrix.width
			same_width: Result.width = width
		end

	subset (a_entities: ARRAY [G]; a_variables: ARRAY [STRING]): like Current is
			-- Possible subset of `Current' with `a_entities' selecting rows and `a_variables' selecting columns;
			-- Passing `Void' for `a_variables' selects all columns.
		require
			entities_not_void: entities /= Void
			variables_not_void: variables /= Void
			matrix_not_void: matrix /= Void
		local
			l_matrix: MATRIX
			l_ent_array, l_var_array: ROSE_LINEAR_ARRAY [INTEGER]
			l_entities: ROSE_LINEAR_ARRAY [G]
			l_vars: ARRAY [STRING]
			l_row_names: ROSE_LINEAR_ARRAY [STRING]
		do
			l_ent_array := entities_index (a_entities)
			if l_ent_array /= Void then
				if a_variables /= Void then
					l_vars := a_variables
				else
					l_vars := variables
				end
				l_var_array := variables_index (l_vars)
				l_entities := entities.subset (l_ent_array)
				create l_matrix.make_sub_matrix_3 (matrix, l_ent_array, l_var_array)
				if matrix.row_names /= Void then
					l_row_names := matrix.row_names.subset (l_ent_array)
					l_matrix.set_row_names (l_row_names)
				end
				create Result.make_all (l_entities, l_matrix, l_vars)
			end
		ensure
			entities_uses_equal: Result /= Void implies (Result.entities /= Void and then Result.entities.object_comparison)
			matrix_not_void: Result /= Void implies Result.matrix /= Void
			variables_not_void: variables /= Void
			height_set: Result /= Void implies (Result.height = Result.matrix.height)
			width_set: Result /= Void implies (Result.width = Result.matrix.width)
		end

	subset2 (a_variables: ARRAY [STRING]): like Current is
			-- `Current' with `a_variables' selecting columns
		require
			matrix_not_void: matrix /= Void
			entities_not_void: entities /= Void
			variables_not_void: variables /= Void
			a_variables_not_void: a_variables /= Void
		local
			l_matrix: MATRIX
			l_var_array: ROSE_LINEAR_ARRAY [INTEGER]
		do
			l_var_array := variables_index (a_variables)
			l_matrix := matrix.selected_columns (l_var_array)
			create Result.make_all (entities, l_matrix, a_variables)
		ensure
			subset2_not_void: Result /= Void
			entities_uses_equal: entities.object_comparison
			entities_aliased: Result.entities = entities
			matrix_not_void: Result.matrix /= Void
			variables_not_void: variables /= Void
			height_set: Result.height = matrix.height
			width_set: Result.width = Result.matrix.width
			same_height: Result.height = height
		end

	subset3 (a_entities: ROSE_LINEAR_ARRAY [G]; a_variables: ARRAY [STRING]): like Current is
			-- Subset of `Current' with `a_entities' selecting rows and `a_variables' selecting columns;
			-- Passing `Void' for `a_variables' selects all columns.
			-- Passing `Void' for `a_entities' selects all rows.
			-- Passing unknown entities seems to return a row of zeros.
		require
			variables_not_void: variables /= Void
			entities_not_void: entities /= Void or a_entities /= Void
			matrix_not_void: matrix /= Void
		local
			l_matrix: MATRIX
			l_var_array: ROSE_LINEAR_ARRAY [INTEGER]
			l_entity: G
			l_vars: ARRAY [STRING]
			i, k: INTEGER
			l_entities: ROSE_LINEAR_ARRAY [G]
		do
			if a_variables /= Void then
				l_vars := a_variables
			else
				l_vars := variables
			end
			if a_entities = Void then
				l_entities := entities
			else
				l_entities := a_entities
			end
			l_var_array := variables_index (l_vars)
			create l_matrix.make (l_entities.count, l_var_array.count)
			from
				i := 1
			until
				i > l_entities.count
			loop
				l_entity := l_entities.item (i)
				k := entity_index (l_entity)
				if k > 0 then
					if {l_vector : VECTOR} matrix.row (k).subset3 (l_var_array) then
						l_matrix.put_row_vector (l_vector, i)
					end
				end
				i := i + 1
			end
			create Result.make_all (l_entities, l_matrix, l_vars)
		ensure
			subset3_not_void: Result /= Void
			entities_uses_equal: Result.entities.object_comparison
			entities_aliased: (Result.entities = entities) or (Result.entities = a_entities)
			matrix_not_void: Result.matrix /= Void
			variables_not_void: variables /= Void
			height_set: Result.height = Result.matrix.height
			width_set: Result.width = Result.matrix.width
		end

	subset_by_value (a_name: STRING; a_value: DOUBLE): like Current is
			-- Possible subset of all columns of `Current' where
			--  the rows are selected as those containg values matching `a_value' in column selected by `a_name'
		require
			variables_not_void: variables /= Void
			entities_not_void: entities /= Void
			matrix_not_void: matrix /= Void
		local
			l_values: like entity_values_at
		do
			l_values := entity_values_at (a_name)
			l_values := l_values.subset_by_value (a_value)
			if l_values /= Void and l_values.count > 0 then
				Result := subset3 (l_values.entities, Void)
			end
		end

	entity_values_at (a_name: STRING): ENTITY_VALUES [G] is
			-- Values from column identified by `a_name'
		require
			variables_not_void: variables /= Void
			entities_not_void: entities /= Void
			matrix_not_void: matrix /= Void
			a_name_good: variable_index (a_name) > 0
		local
			l_vector: VECTOR
		do
			l_vector := matrix.column (variable_index (a_name))
			create Result.make_from_entities_vector (l_vector, entities)
		ensure
			entity_values_at_not_void: Result /= Void
			name_void: Result.name = Void
			same_entities: Result.entities.same_items (entities)
		end

	entity_names: ROSE_LINEAR_ARRAY [STRING] is
			-- Names of all entities
		require
			matrix_not_void: matrix /= Void
			entities_not_void: entities /= Void
			no_void_entity: not entities.has (Void)
			same_height: matrix.height = entities.count
		local
			l_last_underscore: INTEGER
			l_name: STRING
		do
			Result := entity_names_with_brim
			from
				Result.start
			until
				Result.after
			loop
				l_name := Result.current_item
				if l_name /= Void then
					l_last_underscore := l_name.last_index_of ('_', l_name.count)
					if l_last_underscore > 0 then
						l_name.remove_tail (l_name.count - l_last_underscore + 1)
					end
				end
				Result.forth
			end
		ensure
			entity_names_not_void: Result /= Void
			correct_count: Result.count = matrix.height
		end

	entity_names_with_brim: ROSE_LINEAR_ARRAY [STRING] is
			-- Entity names and brim codes
		require
			matrix_not_void: matrix /= Void
			entities_not_void: entities /= Void
			no_void_entity: not entities.has (Void)
			same_height: matrix.height = entities.count
		local
			l_strings: ROSE_LINEAR_ARRAY [STRING]
			l_string: STRING
			l_index: INTEGER
		do
			create Result.make (1, matrix.height)
			l_index := entities.index
			from
				entities.start
			until
				entities.after
			loop
				l_string := Void
				l_strings := entity_names_with_brim_for_entity (entities.current_item)
				if not l_strings.is_empty then
					l_string := ""
					from
						l_strings.start
					until
						l_strings.after
					loop
						l_string := l_string + l_strings.current_item + "_"
						l_strings.forth
					end
					l_string.prune_all_trailing ('_')
				end
				Result.put (l_string, entities.index)
				entities.forth
			end
			entities.go_i_th (l_index)
		ensure
			entity_names_with_brim_not_void: Result /= Void
			correct_count: Result.count = matrix.height
		end

	entity_names_with_xrfzone: ROSE_LINEAR_ARRAY [STRING] is
			-- Entity names appended with xrfzone
		require
			matrix_not_void: matrix /= Void
			entities_not_void: entities /= Void
			same_height: matrix.height = entities.count
		local
			l_strings: ROSE_LINEAR_ARRAY [STRING]
			l_string: STRING
			l_index: INTEGER
		do
			create Result.make (1, matrix.height)
			l_index := entities.index
			from
				entities.start
			until
				entities.after
			loop
				l_string := Void
				l_strings := entity_names_with_xrfzone_for_entity (entities.current_item)
				if not l_strings.is_empty then
					l_string := ""
					from
						l_strings.start
					until
						l_strings.after
					loop
						l_string := l_string + l_strings.current_item + "_"
						l_strings.forth
					end
					l_string.prune_all_trailing ('_')
				end
				Result.put (l_string, entities.index)
				entities.forth
			end
			entities.go_i_th (l_index)
		ensure
			entity_names_with_xrfzone_not_void: Result /= Void
			correct_count: Result.count = matrix.height
		end

	entity_name_arrays: ROSE_LINEAR_ARRAY [ROSE_LINEAR_ARRAY [STRING]] is
			-- Entity names and brim codes
		require
			matrix_not_void: matrix /= Void
			entities_not_void: entities /= Void
			no_void_entity: not entities.has (Void)
			same_height: matrix.height = entities.count
		local
			l_index: INTEGER
		do
			create Result.make (1, matrix.height)
			l_index := entities.index
			from
				entities.start
			until
				entities.after
			loop
				Result.put (entity_names_with_brim_for_entity (entities.current_item), entities.index)
				entities.forth
			end
			entities.go_i_th (l_index)
		ensure
			entity_name_arrays_not_void: Result /= Void
			one_array_per_row: Result.count = matrix.height
			no_void_array: (Result.count = entities.count) implies not Result.has (Void)
		end

	merged (a_other: ENTITY_MATRIX [G]; a_merger: FUNCTION [ANY, TUPLE [VECTOR, VECTOR], VECTOR]): like Current is
			-- Merger of `Current' with `a_other', using `a_merger' for values from entities assumed present in both matrices;
			-- Preconditions for `a_merger' must also be satisfied. It will be assured that both arguments are non-Void.
			-- Note that if `a_merger' returns `Void' then `matrix' will be unchanged at that row.
			-- Don't use this. There is no way to combine the correct rows, unles both matrices have the same entities present in
			-- the same order.
		require
			a_other_not_Void: a_other /= Void
			variables_not_void: variables /= Void
			a_other_variables_not_void: a_other.variables /= Void
			same_width: a_other.variables.count = variables.count
			a_merger_not_void: a_merger /= Void
			entities_not_void: entities /= Void
			a_other_entities_not_void: a_other.entities /= Void
			matrix_not_void: matrix /= Void
			a_other_matrix_not_void: a_other.matrix /= Void
		local
			l_matrix: MATRIX
			l_entities: like entities
		do
				-- Clone entities/matrix
			create l_entities.make_from_array (entities)
			create l_matrix.make (matrix.height + a_other.matrix.height, matrix.width)
			l_matrix.put_matrix (matrix, 1, 1)
			a_other.do_all (agent merge_one (a_merger, l_entities, l_matrix, ?, ?))
			l_matrix := l_matrix.top_rows (l_entities.count)
			create Result.make_all (l_entities, l_matrix, variables)
		ensure
			merged_not_void: Result /= Void
			entities_not_void: Result.entities /= Void
			variables_not_void: variables /= Void
			matrix_not_void: Result.matrix /= Void
			width_set: Result.width = matrix.width
		end

	common_entities_subset (a_entities: ARRAY [G]): like Current is
			-- Subset of `Current' with rows selected by `a_entities';
		require
			a_entities_not_void: a_entities /= Void
			entities_not_void: entities /= Void
			matrix_not_void: matrix /= Void
		local
			l_entities, l_entities_out: ROSE_LINEAR_ARRAY [G]
			l_matrix: MATRIX
		do
			create l_entities_out.make_empty
			create l_entities.make_from_array (entities)
			create l_matrix.make (0, matrix.width)
			from
				l_entities.start
			until
				l_entities.after
			loop
				if a_entities.has (l_entities.current_item) then
					l_entities_out.extend (l_entities.current_item)
					l_matrix.add_row (vector_for_entity (l_entities.current_item))
				end
				l_entities.forth
			end
			create Result.make_all (l_entities_out, l_matrix, matrix.column_names)
		ensure
			common_entities_subset_not_void: Result /= Void
			entities_not_void: Result.entities /= Void
			matrix_not_void: Result.matrix /= Void
			width_set: Result.width = matrix.width
			subset_definition: Result.entities.for_all (agent a_entities.has)
		end

	without_entities (a_entities: ARRAY [G]): like Current is
			-- Subset of `Current' such that rows are not in `a_entities';
		require
			a_entities_not_void: a_entities /= Void
			entities_not_void: entities /= Void
			matrix_not_void: matrix /= Void
		local
			l_entities, l_entities_out: ROSE_LINEAR_ARRAY [G]
			l_matrix: MATRIX
		do
			create l_entities_out.make_empty
			create l_entities.make_from_array (entities)
			create l_matrix.make (0, matrix.width)
			from
				l_entities.start
			until
				l_entities.after
			loop
				if not a_entities.has (l_entities.current_item) then
					l_entities_out.extend (l_entities.current_item)
					l_matrix.add_row (vector_for_entity (l_entities.current_item))
				end
				l_entities.forth
			end
			create Result.make_all (l_entities_out, l_matrix, matrix.column_names)
		ensure
			without_entities_not_void: Result /= Void
			entities_not_void: Result.entities /= Void
			matrix_not_void: Result.matrix /= Void
			width_set: Result.width = matrix.width
			subset_definition: not Result.entities.there_exists (agent a_entities.has)
		end

	diff_by (a_name: STRING; a_other: like Current): like Current is
			--differences of entity_values_at
		require
			a_other_not_void: a_other /= Void
			a_other_variables_not_void: a_other.variables /= Void
			a_other_entities_not_void: a_other.entities /= Void
			a_other_matrix_not_void: a_other.matrix /= Void
			variables_not_void: variables /= Void
			entities_not_void: entities /= Void
			matrix_not_void: matrix /= Void
			a_name_common: variable_index (a_name) > 0 and a_other.variable_index (a_name) > 0
		local
			l_ent_matrix: ENTITY_MATRIX [G]
			l_ent_1, l_ent_2, l_ent_diff: ENTITY_VALUES [G]
		do
			l_ent_1 := entity_values_at (a_name)
			l_ent_2 := a_other.entity_values_at (a_name)
			l_ent_diff := l_ent_1.diff (l_ent_2)
			l_ent_diff.set_name ("add_change_drop")
			if l_ent_diff.count > 0 then
				l_ent_matrix := l_ent_diff.to_entity_matrix
				Result := diff (a_other).subset (l_ent_matrix.entities, Void)
				Result.extend_matrix (l_ent_matrix.matrix)
				Result.set_date (date)
			end
		end

	diff, infix "-" (a_other: like Current): like Current is
			-- Subset where entities are in `Current' but not in `a_other'
		require
			a_other_not_void: a_other /= Void
			a_other_matrix_not_void: a_other.matrix /= Void
			a_other_entities_not_void: a_other.entities /= Void
			matrix_not_void: matrix /= Void
			variables_not_void: variables /= Void
			entities_not_void: entities /= Void
		local
			l_matrix: MATRIX
			l_entities: ROSE_LINEAR_ARRAY [G]
			l_table: ROSE_HASH_TABLE [INTEGER, G]
			i, j, k, l_index: INTEGER
			l_entity: G
		do
				-- TODO: this should be really implemented using 'merged', rather
				-- than this non-reusable version (which predates 'merged')
			k := matrix.height + a_other.matrix.height
			create l_matrix.make (k, variables.count)
			create l_entities.make (1, k)
			l_index := entities.index
			from
				entities.start
			until
				entities.after
			loop
				l_entities.put (entities.current_item, entities.index)
				if {l_row : VECTOR} matrix.row (entities.index) then
					l_matrix.put_row_vector (l_row, entities.index)
				end
				entities.forth
			end
			entities.go_i_th (l_index)
			l_table := entity_table
			l_index := a_other.entities.index
			from
				a_other.entities.start
			until
				a_other.entities.after
			loop
				l_entity := a_other.entities.current_item
				i := l_table.item (l_entity)
				if {l_vector : VECTOR} a_other.matrix.row (a_other.entities.index) then
					if i > 0 then
						if {l_row : VECTOR} (l_matrix.row (i) - l_vector) then
							l_matrix.put_row_vector (l_row, i)
						end
					else
						j := j + 1
						l_matrix.put_row_vector (l_vector.scaled (-1), j + matrix.height)
						l_entities.put (l_entity, j + matrix.height)
					end
				end
				a_other.entities.forth
			end
			a_other.entities.go_i_th (l_index)
			l_entities := l_entities.subarray (1, j + matrix.height)
			l_matrix := l_matrix.top_rows (matrix.height + j)
			create Result.make_all (l_entities, l_matrix, variables)
		ensure
			diff_not_void: Result /= Void
			entities_not_void: Result.entities /= Void
			matrix_not_void: Result.matrix /= Void
			variables_not_void: variables /= Void
			height_set: Result.height = Result.matrix.height
			width_set: Result.width = variables.count
		end

	extend_matrix (a_matrix: MATRIX) is
			-- Extend `matrix' on the right with `a_matrix'.
		require
			matrix_not_void: matrix /= Void
			a_matrix_not_void: a_matrix /= Void
			same_height: matrix.height = a_matrix.height
			valid_matrix_names: a_matrix.column_names /= Void
			variables_not_void: variables /= Void
		do
			set_matrix (matrix.extend_matrix (a_matrix))
			variables := variables + a_matrix.column_names
			set_dimension
		end

feature {NONE} -- Data summary

	data_summary: MATRIX is
			-- Data summary statistics;
			-- This can abort due to a bug im MATRIX (ticket #82088).
		require
			matrix_not_void: matrix /= Void
		do
			Result := matrix.statistics_of_the_columns (Void)
		end

	data_variance_covariance: SYMMETRIC_MATRIX is
			-- Covariance matrix of the xsection data
		require
			matrix_not_void: matrix /= Void
		do
			Result := matrix.covariance (Void, Void)
			Result.set_column_names (variables)
			Result.set_row_names (variables)
		ensure
			data_variance_covariance_not_void: Result /= Void
			height: Result.height = matrix.width
			width: Result.width = matrix.width
		end

feature -- Removal

	remove_entity (a_entity: G) is
			-- Removes first occurrence of `a_entity' (even if `Void') from `Current'.
		require
			entities_not_void: entities /= Void
			matrix_not_void: matrix /= Void
		local
			l_index: INTEGER
			l_indices: ROSE_LINEAR_ARRAY [INTEGER]
			l_submatrix: MATRIX
			l_column_names: ROSE_LINEAR_ARRAY [STRING]
		do
			if has_entity (a_entity) or entity_table.has (a_entity) then
				l_indices := entities_index (entities)
				l_index := entity_index (a_entity)
				l_indices.start
				l_indices.prune (l_index)
				entity_table_saved := Void
				entities.start
				entities.prune (a_entity)
				entity_table_saved := Void
				set_entities (entities)
				l_column_names := matrix.column_names
				l_submatrix := matrix.sub_matrix (l_indices, 1, matrix.width)
				l_submatrix.set_column_names (l_column_names)
				set_matrix (l_submatrix)
			end
		end

feature {NONE} -- Contract support

	greater_than_zero (a_value: INTEGER): BOOLEAN is
			-- Is `a_value' strictly greater than zero?
		do
			Result := a_value > 0
		ensure
			definition: Result = (a_value > 0)
		end

feature {NONE} -- Implementation

	entity_table_saved: like entity_table
			-- Cache for `entity_table'

	variable_table_saved: like variable_table
			-- Cache for `variable_table'

	reset (a_entities: ROSE_LINEAR_ARRAY [G]; a_matrix: MATRIX) is
			-- Resets entities and matrix, voids out entity table.
		do
			set_entities (a_entities)
			set_matrix (a_matrix)
			entity_table_saved := Void
		end

	defaulted_variables (a_vars: ARRAY [STRING]): like variables is
			-- `a_vars' iff not `Void', else `matrix.column_names'
		require
			a_vars_well_formed: a_vars /= Void implies a_vars.for_all (agent variables.has)
			matrix_not_void: a_vars = Void implies matrix /= Void
		do
			if a_vars /= Void then
				create Result.make_from_array (a_vars)
			else
				Result := matrix.safe_column_names
			end
		ensure
			defaulted_variables_not_void: (a_vars /= Void or matrix /= Void) implies Result /= Void
		end

	replace_row_null (a_ents: ROSE_LINEAR_ARRAY [G]) is
			-- Replaces -999s or undefineds with 0 in a row
		local
			l_ents: ROSE_LINEAR_ARRAY [INTEGER]
		do
			l_ents := entities_index (a_ents)
			from
				l_ents.start
			until
				l_ents.off
			loop
				matrix.replace_row_null (l_ents.current_item)
				l_ents.forth
			end
		end

	entity_names_with_brim_for_entity (a_entity: G): ROSE_LINEAR_ARRAY [STRING] is
			-- Entity name, brimcode
		require
			a_entity_not_void: a_entity /= Void
		local
			l_company: PERSISTENT_COMPANY
			l_security: PERSISTENT_SECURITY
			l_persistent_company: PERSISTENT_COMPANY
			l_persistent_listing: PERSISTENT_LISTING
			l_investment_entity: INVESTMENT_ENTITY
			l_company_name, l_xrf_code, l_brim_code, l_sedol_code, l_isin_code: STRING
		do
			create Result.make_empty
			l_persistent_listing ?= a_entity
			if l_persistent_listing /= Void then
				l_brim_code := safe_name (l_persistent_listing.brim_code_key (Void))
				l_sedol_code := safe_name (l_persistent_listing.sedol_most_recent_key (date))
				l_security := l_persistent_listing.security
				if l_security /= Void then
					l_isin_code := safe_name (l_security.isin_code_most_recent_key (date))
					l_company := l_security.company
				else
					l_isin_code := "Void"
				end
				if l_company /= Void then
					l_investment_entity ?= l_company
					if l_investment_entity /= Void then
							-- no DB lookup
						l_xrf_code := l_investment_entity.xrf_code.out
						l_xrf_code.append_character (l_investment_entity.zone_mnemonic)
					else
						l_xrf_code := safe_name (l_company.xrfzone_most_recent_key (date))
					end
					l_company_name := safe_name (l_company.display_name_most_recent_key (date))
				else
					l_xrf_code := "xxx"
					l_company_name := "???"
				end
				Result.extend (l_company_name)
				Result.extend (l_xrf_code)
				Result.extend (l_brim_code)
				Result.extend (l_sedol_code)
				Result.extend (l_isin_code)
			else
				l_persistent_company ?= a_entity
				if l_persistent_company /= Void then
					l_company ?= a_entity
					if l_company /= Void then
						Result.extend (safe_name (a_entity.name_at (Void)))
						Result.extend (safe_name (l_company.brim_code_key (Void)))
					end
				end
			end
			if Result.is_empty then
				Result.extend (safe_name (a_entity.name_at (Void)))
			end
		ensure
			entity_names_with_brim_for_entity_not_void: Result /= Void
		end

	entity_names_with_xrfzone_for_entity (a_entity: G): ROSE_LINEAR_ARRAY [STRING] is
			-- Entity name, xrfzone. Works only for investment entities
		local
			l_entity: INVESTMENT_ENTITY
			l_name, l_xrfzone: STRING
		do
			create Result.make_empty
			l_entity ?= a_entity
			if l_entity /= Void then
				l_xrfzone := safe_name (l_entity.xrf_zone_display)
				l_name := safe_name (l_entity.most_recent_attribute (l_entity.name_identifier_key, date))
				if l_name = Void then
					l_name := safe_name (l_entity.most_recent_attribute (l_entity.name_identifier_key, Void))
				end
			end
			Result.extend (l_name)
			Result.extend (l_xrfzone)
		ensure
			entity_names_with_xrfzone_for_entity_not_void: Result /= Void
		end

feature {NONE} -- More Implementation

	array2 (a_matrix: MATRIX): PAIR [ARRAY2 [ANY], STRING] is
			-- Data from `a_matrix' paired with it's name
		do
			Result := matrix_combined_to_array2 (a_matrix, a_matrix.row_names, a_matrix.column_names, a_matrix.name, Void)
		ensure
			array2_not_void: Result /= Void
		end

	matrix_to_excel (a_matrix: ABSTRACT_MATRIX; a_name: STRING): PAIR [ARRAY2 [ANY], STRING] is
			-- Data from `a_matrix' paired with `a_name';
			-- Nothing to do with excel.
		do
			Result := matrix_combined_to_array2 (a_matrix, a_matrix.row_names, a_matrix.column_names, a_name, Void)
		ensure
			matrix_to_excel_not_void: Result /= Void
		end

	correct_mismatch is
			-- Avoid triggering an exception when retrieving incomptable storables.
		do
			do_nothing
		end

	is_defined (a_matrix: MATRIX; i, j: INTEGER): BOOLEAN is
			-- Is `a_matrix [i, j]' a defined, finite value?
		do
			Result := is_defined_double (a_matrix.item (i, j))
		end

	safe_name (a_name: STRING): STRING is
			-- Non-void string
		do
			if a_name /= Void then
				Result := a_name
			else
				Result := "Void"
			end
		ensure
			safe_name_not_void: Result /= Void
		end

	union_string_array (a_a: ROSE_LINEAR_ARRAY [HASHABLE]; a_b: ROSE_LINEAR_ARRAY [HASHABLE]): ROSE_LINEAR_ARRAY [HASHABLE] is
			-- All elements of `a_a' plus those from `a_b' not `equal' to any in `a_a';
			-- Avoids using `deep_equal'.
		require
			a_a_not_void: a_a /= Void
			a_b_not_void: a_b /= Void
		local
			l_a_table: ROSE_HASH_TABLE [INTEGER, HASHABLE]
		do
			Result := a_a.twin
			create l_a_table.make (Result.count)
			from
				Result.start
			until
				Result.after
			loop
				l_a_table.put (1, Result.current_item)
				Result.forth
			end
			l_a_table.compare_objects
			Result.compare_objects
			from
				a_b.start
			until
				a_b.after
			loop
				if not l_a_table.has (a_b.current_item) then
					Result.extend (a_b.current_item)
					l_a_table.put (1, a_b.current_item)
				end
				a_b.forth
			end
		ensure
			union_string_array_not_void: Result /= Void
			minimum_count: Result.count >= a_a.count
			maximuim_count: Result.count <= (a_a.count + a_b.count)
		end

	merge_one (a_merger: FUNCTION [ANY, TUPLE [VECTOR, VECTOR], VECTOR]; a_entities: like entities; a_matrix: MATRIX; a_entity: G; a_row: attached VECTOR) is
			-- Use `a_merger' to combine values from `a_row' into `a_matrix' if `a_entity' is present in both sets.
			-- If it is not, then add `a_entity' into `a_entities' and put `a_row' unchanged onto end of `a_matrix'.
			-- Preconditions for `a_merger' must also be satisfied. It will be assured that both arguments are non-Void.
			-- N.B. There is no guarantee that the correct pairs of rows will be passed to `a_merger'.
			-- Nor that the vectors will be of the same length.
			-- Nor that the vector elements in corresponding positions are for the same variable.
			-- Note that if `a_merger' returns `Void' then `a_matrix' will be unchanged at that row.
		require
			a_merger_not_Void: a_merger /= Void
			a_entities_not_Void: a_entities /= Void
			a_matrix_not_Void: a_matrix /= Void
			a_entity_not_Void: a_entity /= Void
		local
			i: INTEGER
		do
			i := entity_table.item (a_entity)
			if i > 0 and i <= a_matrix.height then
					-- merge rows
				if {l_vector : VECTOR} a_merger.item ([a_matrix.row (i), a_row]) then
					a_matrix.put_row_vector (l_vector, i)
				end
			else
					-- entity not in Current
				a_entities.extend (a_entity)
				a_matrix.put_row_vector (a_row, a_entities.count)
			end
		end

invariant

	entities_one_based: entities /= Void implies entities.lower = 1
	variables_one_based: variables /= Void implies variables.lower = 1

end
