note

	description:

		"The list of variables available for a target.  The generic parameter is the target."

	last_updater: "$Author: it $"
	author: "Mark Howard"
	last_editor: "$Author: it $"
	date: "$Date: 2014/03/17 $"
	revision: "$Revision: #95 $"
	path: "$File: //rose/source/kernel/entities/entity_variables.e $"

class ENTITY_VARIABLES [reference G]

inherit

	ROSE_CONSTANTS

	ROSE_SHARED_LOGGER
	EXCEPTIONS
create

	make,
	make_anonymous,
	make_from_descriptions

feature {STID_TESTENV_DATABASE} -- Initialization

	make_anonymous is
			-- Instantiate but don't initialize
		do
			do_nothing
		end

	make_from_descriptions (a_variable_descriptions: ROSE_LINEAR_ARRAY [VARIABLE_DESCRIPTION_RECORD]; a_new_variable_agent: like new_variable_agent; a_interpretations: ROSE_LINEAR_ARRAY [MAPPINGS_RECORD]) is
			-- Creation procedure
		require
			valid_variables: a_variable_descriptions /= Void
		do
			new_variable_agent := a_new_variable_agent
			from
				a_variable_descriptions.start
			until
				a_variable_descriptions.off
			loop
				add_description (a_variable_descriptions.current_item)
				a_variable_descriptions.forth
			end
			if a_interpretations /= Void then
				mapping_records := a_interpretations
			end
		end

	make is
			--make
		do
			register_variables
		end

feature -- Access

	variables: HASH_TABLE [like expression_template, STRING] is
			-- Variables by mnemonic
		do
			Result := variables_saved
			if Result = Void then
				create Result.make (approximate_expected_number_of_variables)
				variables_saved := Result
			end
		ensure
			result_not_void: Result /= Void
		end

	variables_by_name: HASH_TABLE [like expression_template, STRING] is
			-- Variables by mnemonic
		do
			Result := variables_by_name_saved
			if Result = Void then
				create Result.make (approximate_expected_number_of_variables)
				variables_by_name_saved := Result
			end
		ensure
			result_not_void: Result /= Void
		end

	variables_by_variable_index: HASH_TABLE [like expression_template, INTEGER] is
			-- Variables by index
		do
			Result := variables_by_variable_index_saved
			if Result = Void then
				create Result.make (approximate_expected_number_of_variables)
				variables_by_variable_index_saved := Result
			end
		ensure
			result_not_void: Result /= Void
		end

	variables_caller_by_mnemonic: HASH_TABLE [ARRAY [STRING], STRING] is
			-- Variables by caller mnemonic
		do
			Result := variables_caller_by_mnemonic_saved
			if Result = Void then
				create Result.make (approximate_expected_number_of_variables)
				variables_caller_by_mnemonic_saved := Result
			end
		ensure
			result_not_void: Result /= Void
		end

	variables_caller_by_name: HASH_TABLE [ARRAY [STRING], STRING] is
			-- Variables by caller name
		do
			Result := variables_caller_by_name_saved
			if Result = Void then
				create Result.make (approximate_expected_number_of_variables)
				variables_caller_by_name_saved := Result
			end
		ensure
			result_not_void: Result /= Void
		end

	variables_caller_by_variable_index: HASH_TABLE [ARRAY [STRING], INTEGER] is
			-- Variables by caller id
		do
			Result := variables_caller_by_variable_index_saved
			if Result = Void then
				create Result.make (approximate_expected_number_of_variables)
				variables_caller_by_variable_index_saved := Result
			end
		ensure
			result_not_void: Result /= Void
		end

	variable_descriptions: HASH_TABLE [VARIABLE_DESCRIPTION, STRING] is
			--Variable descriptions by mnemonic or name
		do
			if variable_descriptions_saved = Void then
				create variable_descriptions_saved.make (1000)
			end
			Result := variable_descriptions_saved
		end

	variable_descriptions_by_name: HASH_TABLE [VARIABLE_DESCRIPTION, STRING] is
			-- Variable descriptions by name.
		do
			if variable_descriptions_by_name_saved = Void then
				create variable_descriptions_by_name_saved.make (1000)
			end
			Result := variable_descriptions_by_name_saved
		end

	variable_descriptions_by_id: HASH_TABLE [VARIABLE_DESCRIPTION, INTEGER] is
			-- Variable descriptions by id.
		do
			if variable_descriptions_by_id_saved = Void then
				create variable_descriptions_by_id_saved.make (1000)
			end
			Result := variable_descriptions_by_id_saved
		end

	interpretations: HASH_TABLE [HASH_TABLE [ROSE_EXPRESSION [like generic_template], STRING], STRING] is
			-- Mappings from current context expressions to target context expression
		local
			l_interpretation: MAPPINGS_RECORD
			l_interpretations: HASH_TABLE [ROSE_EXPRESSION [like generic_template], STRING]
			l_expression: like complex_expression
		do
			if interpretations_saved = Void then
				create interpretations_saved.make (100)
				if mapping_records /= Void then
					from
						mapping_records.start
					until
						mapping_records.off
					loop
							-- FIXME -- this looks loaded with side effects on the contents of mapping_records.
						l_interpretation := mapping_records.current_item
						l_interpretation.mnemonic.prune_all (' ')
						l_interpretation.mnemonic.to_lower
						l_interpretation.type.prune_all (' ')
						l_interpretation.type.to_lower
						l_interpretation.expression.prune_all_trailing (' ')
						l_interpretation.expression.to_lower
						l_expression := complex_expression (l_interpretation.expression)
						l_interpretations := interpretations_saved.item (l_interpretation.type)
						if l_interpretations = Void then
							create l_interpretations.make (mapping_records.count)
							interpretations_saved.put (l_interpretations, l_interpretation.type)
						end
						l_interpretations.put (l_expression, l_interpretation.mnemonic.as_lower)
						mapping_records.forth
					end
				end
			end
			Result := interpretations_saved
		ensure
			nonvoid: Result /= Void
		end

	interpretation (a_report_type: STRING; a_mnemonic: STRING): ROSE_EXPRESSION [like generic_template] is
			-- Interpretation of a_mnemonic
		require
			valid_mnemonic: a_mnemonic /= Void
		local
			l_report_type: STRING
		do
			if interpretations /= Void and then a_report_type /= Void then
				l_report_type := a_report_type.twin
				l_report_type.to_lower
				interpretations.search (l_report_type)
				if interpretations.found then
					Result := interpretations.found_item.item (a_mnemonic.as_lower)
				end
			else
				Result := complex_expression (a_mnemonic.as_lower)
			end
		end

	variables_all: ROSE_LINEAR_ARRAY [like new_variable] is
			-- All variables in array form
		do
			create Result.make_empty
			from
				variable_descriptions.start
			until
				variable_descriptions.off
			loop
				Result.extend (variable_from_mnemonic (variable_descriptions.key_for_iteration))
				variable_descriptions.forth
			end
		end

feature {NEEDS_EXPRESSION_FROM_STRING} --Expressions by name

	expression_from_name (a_string: STRING): like new_variable is
			--strongly typed expression ... will be void if it does not conform.
			--name or mnemonic
		require
			nonvoid: a_string /= Void
		do
			Result ?= complex_expression (a_string)
		end

	complex_expression (a_string: STRING): like expression_template is
			-- Get a ROSE_EXPRESSION tree from 'a_string'
		require
			nonvoid: a_string /= Void
		local
			l_string: STRING
			l_parser: ROSE_EXPRESSION_PARSER [like generic_template]
		do
			Result := variable (a_string)
			if Result = Void then
				l_string := a_string.as_lower
				lower (l_string)
				Result := variables.item (l_string)
				if Result = Void then
					create l_parser.make (Current)
					if attached {like variables.item}  l_parser.parsed_string (a_string) as l_typed then
						Result := l_typed
					end
					debug ("trace")
						if Result = Void or else attached {SIMPLE_EXPRESSION[ANY]} Result as l_simple then
							print("Could not find expression " + a_string + "%N")
						end
					end
					if l_parser.error.signaled then
						l_parser.error.signal (generator, "Can not parse '$1'", <<a_string>>)
					end
					l_parser.error.flush_and_acknowledge
					if Result /= Void then
						register_variable (Result)
					end
				end
				if Result = Void then
					do_nothing
				end
			end
		end

feature --Access

	is_valid_expression_string (a_string: STRING): BOOLEAN is
			-- Does a_string represent a valid expression
		local
			l_variable: ROSE_EXPRESSION [like generic_template]
			l_parser: ROSE_EXPRESSION_PARSER [like generic_template]
			l_simple_expression: SIMPLE_EXPRESSION [like generic_template]
		do
			create l_parser.make (Current)
			l_variable := l_parser.parsed_string (a_string)
			l_simple_expression ?= l_variable
			Result := (not l_parser.error.signaled) and (l_simple_expression = Void)
			l_parser.error.flush_and_acknowledge
		end

	variable (a_id: STRING): like new_variable is
			-- Variable designated by 'a_id'
		do
			Result := variable_from_name (a_id)
			if Result = Void then
				Result := variable_from_mnemonic (a_id)
			end
		end

	variable_from_name (a_name: STRING): detachable like new_variable is
			--ROSE_EXPRESSION [like generic_template] is
			-- Variable designated by 'a_name'
		require
			nonvoid: a_name /= Void
		local
			l_description: VARIABLE_DESCRIPTION
		do
			if {l_new_variable: like new_variable} variables_by_name [a_name] then
				Result := l_new_variable
			else
				l_description := variable_descriptions_by_name.item (a_name)
				if l_description = Void then
					l_description := variable_descriptions_by_name.item (a_name.as_lower)
				end
				if l_description = Void then
					l_description := variable_descriptions.item (a_name.as_lower)
				end
				if l_description /= Void then
					Result := variable_from_mnemonic (l_description.mnemonic.as_lower)
				end
			end
		end

	variable_from_mnemonic (a_mnemonic: STRING): like new_variable is
			-- Variable designated by its mnemonic
		require
			valid_mnemonic: a_mnemonic /= Void
		local
			l_variable_description: VARIABLE_DESCRIPTION_RECORD
				--			l_computed: like computed_expression_template
			l_mnemonic: STRING
		do
			l_mnemonic := a_mnemonic.as_lower
			Result ?= variables.item (l_mnemonic)
			if Result = Void then
				l_variable_description ?= variable_descriptions.item (l_mnemonic)
				if l_variable_description /= Void then
					Result := new_variable (l_variable_description)
				end
				if Result /= Void then
					register_variable (Result)
				end
			end
		end

	variable_from_index (a_index: INTEGER): ROSE_EXPRESSION [like generic_template] is
			-- Variable designated by its variable index
		do
			Result := variables_by_variable_index[a_index]
		end

	new_variable (a_description: VARIABLE_DESCRIPTION): like new_variable_template is
			-- New variable given in 'a_description'
		do
			if new_variable_agent /= Void then
				Result := new_variable_agent.item ([a_description])
			end
		end

	all_variables: ROSE_LINEAR_ARRAY [like new_variable] is
			-- Array of all double, integer and string variables
		do
			Result := double_variables + integer_variables + string_variables
		end

	double_variables: ROSE_LINEAR_ARRAY [like new_variable] is
			-- Array of all double type variables
		do
			Result := variables_by_type ("DOUBLE")
		end

	string_variables: ROSE_LINEAR_ARRAY [like new_variable] is
			-- Array of all string type variables
		do
			Result := variables_by_type ("STRING")
		end

	integer_variables: ROSE_LINEAR_ARRAY [like new_variable] is
			-- Array of all integer type variables
		do
			Result := variables_by_type ("INTEGER")
		end

	variables_by_type (a_type: STRING): ROSE_LINEAR_ARRAY [like new_variable] is
			-- Array of all a_type type variables
		local
			l_variable: like new_variable
			l_description: VARIABLE_DESCRIPTION
		do
			create Result.make_empty
			from
				variable_descriptions.start
			until
				variable_descriptions.off
			loop
				l_description := variable_descriptions.item_for_iteration
				if l_description.type /= Void and then l_description.type.as_upper.is_equal (a_type) then
					l_variable ?= variable_from_mnemonic (l_description.mnemonic)
					if l_variable /= Void then
						Result.extend (l_variable)
					end
				end
				variable_descriptions.forth
			end
		end

	variable_is_registered (a_variable: like expression_template): BOOLEAN is
			-- has this variable already been registered
		require
			a_variable_not_void: a_variable /= Void
				--			a_variable_has_valid_mnemonic: a_variable.mnemonic /= Void and then not a_variable.mnemonic.is_empty
				--			a_variable_has_valid_name: a_variable.name /= Void and then not a_variable.name.is_empty
		do
			Result := (a_variable.mnemonic /= Void and then variables.has (a_variable.mnemonic.as_lower)) or else (a_variable.name /= Void and then variables_by_name.has (a_variable.name.as_lower)) or else (a_variable.variable_index /= 0 and then variables_by_variable_index.has (a_variable.variable_index))
		end

feature -- Basic operations

	add_description (a_description: VARIABLE_DESCRIPTION_RECORD) is
			-- Process 'a_description' into the variable description caches
		require
			valid_description: a_description /= Void
		local
			l_display_name: STRING
			l_description: VARIABLE_DESCRIPTION_RECORD
		do
			l_description := a_description.twin
			lower (l_description.mnemonic)
			if l_description.name /= Void then
				l_display_name := l_description.name.twin
				l_display_name.replace_substring_all ("'", "")
				l_display_name.replace_substring_all (" - ", "_")
				l_display_name.replace_substring_all (" ", "_")
				l_display_name.replace_substring_all ("-", "_")
				l_display_name.replace_substring_all (" (", "")
				l_display_name.replace_substring_all (")", "")
				l_display_name.replace_substring_all ("\", "_")
				l_display_name.replace_substring_all ("/", "_")
				l_display_name.replace_substring_all ("&", "_")
				lower (l_display_name)
				l_description.set_display_name (l_display_name)
			end
			lower (l_description.name)
			lower (l_description.source)
			lower (l_description.group_name)
			if l_description.parent /= Void then
				l_description.parent.prune_all (' ')
				l_description.parent.to_lower
			end
			variable_descriptions.put (l_description, l_description.mnemonic)
			variable_descriptions_by_name.put (l_description, l_description.name)
			variable_descriptions_by_id.put (l_description, l_description.variable_index)
		end

	add_expression (a_expression: like new_variable; a_description: VARIABLE_DESCRIPTION_RECORD) is
			-- Add 'a_expression' with 'a_description'
		require
			a_description_not_void: a_description /= Void
		local
			l_expression: like new_variable
		do
			l_expression := variable_from_mnemonic (a_description.mnemonic)
			if l_expression = Void and then a_description.name /= Void then
				l_expression := variable_from_name (a_description.name)
				if l_expression /= Void then
					logger.notice ("ENTITY_VARIABLES.add_expression: Expression with name already exists " + a_description.name)
				end
			else
				logger.notice ("ENTITY_VARIABLES.add_expression: Expression with mnemonic already exists " + a_description.mnemonic)
			end
			if l_expression = Void then
				add_description (a_description)
				variables.put (a_expression, a_description.mnemonic)
			end
		end

	replace_expression (a_expression: like new_variable; a_description: VARIABLE_DESCRIPTION_RECORD) is
			--replace expression with this name
		local
			l_expression: like new_variable
		do
			l_expression := variable_from_mnemonic (a_description.mnemonic)
			if l_expression = Void and then a_description.name /= Void then
				l_expression := variable_from_name (a_description.name)
			end
			if l_expression /= Void then
				add_description (a_description)
				variables.force (a_expression, a_description.mnemonic)
			end
		end

feature -- Expression dependencies

	component_expressions (a_expression_mnemonic: STRING): ROSE_LINEAR_ARRAY [ROSE_EXPRESSION [like generic_template]] is
			-- Expressions which contribute to the value of `a_expression_mnemonic'
		require
			a_expression_mnemonic_not_void: a_expression_mnemonic /= Void
		local
			l_array: ROSE_LINEAR_ARRAY [ROSE_DEPENDENT_EXPRESSION_RECORD]
			l_component_expression,l_expression: like complex_expression
		do
			create Result.make_empty
			component_expressions_cache.search (a_expression_mnemonic)
			if component_expressions_cache.found then
				l_array := component_expressions_cache.found_item
				if l_array /= Void then
					from
						l_array.start
					until
						l_array.off
					loop
							-- Do this to handle then the generic_template is expanded
						l_expression := complex_expression (l_array.current_item.dependent)
						if l_expression = Void or else not l_expression.generator.as_lower.is_equal (l_array.current_item.dependent_target) then
							l_component_expression := complex_expression (l_array.current_item.dependent)
							if l_component_expression /= Void then
								Result.extend (l_component_expression)
							end
						end
						l_array.forth
					end
				end
			end
		ensure
			result_not_void: Result /= Void
		end

	dependent_expressions_records (a_expression_mnemonic: STRING): ROSE_LINEAR_ARRAY [ROSE_DEPENDENT_EXPRESSION_RECORD] is
			-- The raw records which contribute to the value of 'a_expression_mnemonic'
		require
			nonvoid: a_expression_mnemonic /= Void
		do
			create Result.make_empty
			component_expressions_cache.search (a_expression_mnemonic)
			if component_expressions_cache.found and component_expressions_cache.found_item /= Void then
				Result := component_expressions_cache.found_item
			else
				create Result.make_empty
			end
		ensure
			nonvoid: Result /= Void
		end


feature --Templates

	expression_template: ROSE_EXPRESSION [like generic_template] is
			--  Expression template
		do
		end

	expression_agent_template: FUNCTION [ANY, TUPLE, ROSE_EXPRESSION [like generic_template]] is
			--  Expression agent template
		do
		end

feature {NONE} -- Implementation

	register_variables is
			--Redefined in descendents
		do
		end

	register_variable_agent (a_mnemonic: STRING; a_agent: like expression_agent_template) is
			-- add `a_agent' keyed by `a_mnemonic' to the table of variable agents
		require
			a_mnemonic_not_void: a_mnemonic /= Void
			a_agent_not_void: a_agent /= Void
		do
--			debug ("register_variable")
--				if {l_variable: like expression_template} a_agent.item ([]) and then l_variable.mnemonic /= Void then
--					if not l_variable.mnemonic.as_lower.is_equal (a_mnemonic.as_lower) then
--						logger.warn ("Variable mnemonic does NOT match the mnemonic that was passed in: " + l_variable.generating_type + " variable.mnemonic=" + l_variable.mnemonic + " a_mnemonic=" + a_mnemonic)
--					end
--					register_variable (l_variable)
--				end
--			end
		end

	register_variable (a_variable: like variables.item) is
			-- add `a_variable' to `variables'
		require
			a_variable_not_void: a_variable /= Void
		local
			l_mnemonic : STRING
		do
			if not variable_is_registered (a_variable) then
				if a_variable.mnemonic /= Void then
					variables.force (a_variable, a_variable.mnemonic.as_lower)
				end
				if a_variable.name /= Void then
					variables_by_name.force (a_variable, a_variable.name.as_lower)
				end
				if a_variable.variable_index /= 0 then
					variables_by_variable_index.force (a_variable, a_variable.variable_index)
				end
				if a_variable.variable_description /= Void and then a_variable.variable_description.mnemonic /= Void then
					variable_descriptions [a_variable.variable_description.mnemonic] := a_variable.variable_description
				end
--				debug ("register_variable")
--					if {l_call_stack: ARRAY [STRING]} call_stack.call_stack then
--						if a_variable.mnemonic /= Void then
--							variables_caller_by_mnemonic.force (l_call_stack, a_variable.mnemonic.as_lower)
--						end
--						if a_variable.name /= Void then
--							variables_caller_by_name.force (l_call_stack, a_variable.name.as_lower)
--						end
--						if a_variable.variable_index /= 0 then
--							variables_caller_by_variable_index.force (l_call_stack, a_variable.variable_index)
--						end
--					end
--				end
			else
				if a_variable.mnemonic /= Void then
					l_mnemonic := a_variable.mnemonic
				else
					l_mnemonic := "<Unknown>"
				end
				logger.debugging("A variable with the same identifying characteristics has already been registered: mnemonic=" + l_mnemonic + " name=" + a_variable.name + " index=" + a_variable.variable_index.out )
			end
		ensure
			a_variable_registered: (a_variable.mnemonic /= Void or else a_variable.name /= Void or else a_variable.variable_index /= 0) implies variable_is_registered (a_variable)
		end

	variables_caller_by_mnemonic_saved: like variables_caller_by_mnemonic
			-- once per ... this should be once per???

	variables_caller_by_name_saved: like variables_caller_by_name
			-- once per ... this should be once per???

	variables_caller_by_variable_index_saved: like variables_caller_by_variable_index
			-- once per ... this should be once per???

	variables_by_name_saved: like variables_by_name
			-- once per ... this should be once per???

	variables_by_variable_index_saved: like variables_by_variable_index
			-- once per ... this should be once per???

	variables_saved: like variables
			-- once per ... this should be once per???

	variable_descriptions_saved: like variable_descriptions
			-- once per

	variable_descriptions_by_name_saved: like variable_descriptions_by_name
			-- once per

	variable_descriptions_by_id_saved: like variable_descriptions_by_id
			-- once per

	generic_template: detachable G is
			-- Template for generic parameter of expressions
		do
			do_nothing
		end

	new_variable_agent: FUNCTION [ANY, TUPLE [VARIABLE_DESCRIPTION], like new_variable]
			-- The agent used to create new variables.  This is usually defined in the target which is the generic parameter.

	component_expressions_cache: HASH_TABLE [ROSE_LINEAR_ARRAY [ROSE_DEPENDENT_EXPRESSION_RECORD], STRING] is
			-- Cached records of component expressions
		local
			l_server: SQL_DATA_SERVER
			l_servers: DATABASE_SERVERS
			l_predicate: COLLECTING_PREDICATE [ROSE_DEPENDENT_EXPRESSION_RECORD]
			l_current_mnemonic: STRING
			l_array: ROSE_LINEAR_ARRAY [ROSE_DEPENDENT_EXPRESSION_RECORD]
			l_expression: like new_variable
		once
			create Result.make (100)
			create l_servers
			l_server := l_servers.rose_entity_background_server
			if l_server /= Void then
				l_server := l_server.auxiliary_server
				l_server.begin_transaction
				if l_server.has_table ("variable_dependencies") then
					-- The auxiliary server requires the use of begin_trans/commit.
					create l_predicate.make_strict (l_server, "select * from variable_dependencies order by variable_target, variable, dependent")
					l_predicate.retrieve (Void, Void)
					from
						l_predicate.array.start
					until
						l_predicate.array.off
					loop
						if not equal (l_current_mnemonic, l_predicate.array.current_item.variable) then
							l_current_mnemonic := l_predicate.array.current_item.variable
							l_expression := expression_from_name (l_current_mnemonic)
							if (l_expression /= Void and then equal (l_expression.generator.as_lower, l_predicate.array.current_item.variable_target.as_lower)) or else l_expression = Void then
								create l_array.make_empty
								Result.force (l_array, l_predicate.array.current_item.variable)
							else
								l_array := Void
							end
						end
						if l_array /= Void and then (l_array.is_empty or else not equal (l_array.last.dependent, l_predicate.array.current_item.dependent)) then
							l_array.extend (l_predicate.array.current_item)
						end
						l_predicate.array.forth
					end
				end
				l_server.commit
			end
		ensure
			result_not_void: Result /= Void
		end

	interpretations_saved: like interpretations
			-- per-object cache

	mapping_records: ROSE_LINEAR_ARRAY [MAPPINGS_RECORD]
			-- Mapping records

feature {NONE} -- Templates

	new_variable_template: ROSE_EXPRESSION [like generic_template]
			--template
		do
		end

feature {NONE} -- Utilities

	lower (a_string: STRING) is
			-- Clean a_string to a standard of all lower characters with no leading spaces
		do
			if a_string /= Void then
				a_string.prune_all (' ')
				a_string.to_lower
			end
		end

	call_stack: EIFFEL_CALL_STACK is
			-- access to the eiffel call stack, useful for figuring out where register_variable is being called when there is a variable mnemonic/name/indec clash
		once
			create Result
		end

	approximate_expected_number_of_variables: INTEGER is
			-- the number of variables expected to be inserted into `variables', may be redefined in a descendant to more closely match the real expected number
		do
			Result := 10
		end

end
