indexing
	description: "Error logging facility."
	author: "David Stevens"
	date: "$Date: 2008/12/01 $"
	revision: "$Revision: #11 $"

class
	ERROR_LOG

create
	make

feature {NONE} -- Initialization

	make is
			-- Creation procedure.
		local
			l_temp: FILE_NAME
			retried: BOOLEAN
		do
			if not retried then
				create system_name.make (0)
				create class_name.make (0)
				create feature_name.make (0)
				create l_temp.make_temporary_name
				create log_file.make (l_temp)
				log_file.open_append
			end
		rescue
			if not retried then
				log_file := Void
				retried := True
				retry
			end
		end

feature -- Access

	system_name: STRING

	class_name: STRING

	feature_name: STRING

	log_file: ROSE_TEXT_FILE
			-- Actual log file

	message: STRING

	severity: INTEGER

	Fatal: INTEGER is unique

	Warning: INTEGER is unique

	Information: INTEGER is unique

feature -- Basic operations

	put is
		-- Write the error message and terminate if fatal.
		do
			if log_file /= Void then
				log_file.put_string (" SYSTEM: ")
				log_file.put_string (system_name)
				log_file.put_string (" CLASS: ")
				log_file.put_string (class_name)
				log_file.put_string (" FEATURE: ")
				log_file.put_string (feature_name)
				inspect severity
				when Information then
					log_file.put_string (" Information: ")
				when Warning then
					log_file.put_string (" Warning: ")
				when Fatal then
					log_file.put_string (" Fatal: ")
				else
					log_file.put_string (" : ")
				end
				log_file.put_string (message)
				log_file.new_line
				log_file.flush
				if severity = Fatal then
					(create {EXCEPTIONS}).raise (message)
				end
			end
		end

	put_message_with_full_information (a_system_name, a_class_name, a_feature_name: STRING; a_severity: INTEGER; a_message: STRING) is
			-- Put message with info about where it is coming from
		do
			set_system_name (a_system_name)
			set_class_name (a_class_name)
			set_feature_name (a_feature_name)
			set_severity (a_severity)
			set_message (a_message)
			put
		end


feature -- Element change

	append_message (in_message: STRING) is
			-- Set `message' to `in_message'.
		require
			in_message_not_void: in_message /= Void
		do
			message.append (in_message)
		end

	set_class_name (in_class_name: STRING) is
			-- Set `class_name' to `in_class_name'.
		require
			in_class_name_not_void: in_class_name /= Void
		do
			class_name := in_class_name
		ensure
			class_name_correct: class_name.is_equal (in_class_name)
		end

	set_feature_name (in_feature_name: STRING) is
			-- Set `feature_name' to `in_feature_name'.
		require
			in_feature_name_not_void: in_feature_name /= Void
		do
			feature_name := in_feature_name
		ensure
			feature_name_correct: feature_name.is_equal (in_feature_name)
		end

	set_log_file (in_file_name: STRING) is
			-- Create the log file.
		require
			in_file_name_not_void: in_file_name /= Void
		local
			retried: BOOLEAN
		do
			if not retried then
				create log_file.make (in_file_name)
				log_file.open_write
			end
		rescue
			if not retried then
				log_file := Void
				retried := True
				retry
			end
		end

	set_message (in_message: STRING) is
			-- Set `message' to `in_message'.
		require
			in_message_not_void: in_message /= Void
		do
			message := in_message
		ensure
			message_correct: message.is_equal (in_message)
		end

	set_severity (in_severity: INTEGER) is
			-- Set `severity' to `in_severity'.
		require
			valid_severity: (in_severity = Information) or (in_severity = Warning) or (in_severity = Fatal)
		do
			severity := in_severity
		ensure
			severity_correct: severity = in_severity
		end

	set_system_name (in_system_name: STRING) is
			-- Set `system_name' to `in_system_name'.
		require
			in_system_name_not_void: in_system_name /= Void
		do
			system_name := in_system_name
		ensure
			system_name_correct: system_name.is_equal (in_system_name)
		end

end
