note

	description:

		"EVENT_O_A-SWEPT_ACCOUNTS.PARM loader"

	author: "Ivan Tarnavsky"
	last_editor: "$Author: it $"
	date: "$Date: 2011/03/07 $"
	revision: "$Revision: #2 $"
	path: "$File: //rose/source/kernel/expert_system/aflows/event_o_a_corporate_actions_settlement_parm.e $"

class EVENT_O_A_CORPORATE_ACTIONS_SETTLEMENT_PARM

inherit

	RP_EVENT_TYPE_CONSTANTS
		export {NONE} all end

	ROSE_SHARED_GLOBAL
		export {NONE} all end

	ANY

create

	make

feature {NONE} -- Initialization

	make is
			-- Initialize `Current'
		do
			create entries.make_equal (0)
			parse_from_windows_share
		end

feature -- Access

	entries: DS_HASH_TABLE [ROSE_DATE, MULTI_KEY]
			-- Entries

feature {NONE} -- Implementation

	parm_file_name: FILE_NAME is
			-- Parm file name
		once
			create Result.make
			Result.set_directory (global.hardcoded_path ("\\axaros.com\shares\ProdData_NT\PortfolioAccounting\ParmFiles"))
			Result.set_file_name ("EVENT_O_A-CORPORATE_ACTIONS_SETTLEMENT.PARM")
		ensure
			parm_file_name_not_void: Result /= Void
		end

	parse_from_windows_share is
			-- Parse 'a_file_name' and populate internal table
		require
			entries_not_void: entries /= Void
		local
			l_zone_file: EVENT_Z_A_CORPORATE_ACTIONS_SETTLEMENT_PARM_FILE
		do
			create l_zone_file.make (parm_file_name)
			l_zone_file.enable_expand_tabs_to_spaces
			if l_zone_file.exists then
				l_zone_file.read_with_format
				l_zone_file.array.do_all (agent add_interval (?))
			else
				(create {EXCEPTIONS}).raise (parm_file_name.out + " does not exist")
			end
		end

	add_interval (a_record: EVENT_Z_A_CORPORATE_ACTIONS_SETTLEMENT_PARM_RECORD) is
			-- Add interval to `entries' with data from `a_record'
		require
			entries_not_void: entries /= Void
			a_record_not_void: a_record /= Void
		local
			l_record: EVENT_Z_A_CORPORATE_ACTIONS_SETTLEMENT_PARM
			l_key: MULTI_KEY
		do
			if not a_record.is_valid then
				(create {EXCEPTIONS}).raise ("Invalid record " + a_record.clicode + ":" + a_record.brimsym + ":" + a_record.event_type.out + ":" + a_record.exdate_ccyymmdd.out + ":" + a_record.paydate_ccyymmdd.out + " in " + parm_file_name)
			end
			create l_record.make (a_record)
			if l_record.is_stock_dividend then
					-- stock dividends only
					-- as POSITIONS on VMS adjusts
					-- settlement of event type 5 only
				create l_key.make (<<l_record.clicode, l_record.brimsym, l_record.exdate>>)
				entries.search (l_key)
				if entries.found then
					(create {EXCEPTIONS}).raise ("Duplicate entry " + l_record.debug_output + " in " + parm_file_name)
				end
				entries.force (l_record.paydate, l_key)
			end
		end

invariant

	entries_not_void: entries /= Void

end
