note

	description:

		"EVENT_O_A-SWEPT_ACCOUNTS.PARM loader"

	author: "Ivan Tarnavsky"
	last_editor: "$Author: it $"
	date: "$Date: 2011/03/07 $"
	revision: "$Revision: #2 $"
	path: "$File: //rose/source/kernel/expert_system/aflows/event_o_a_dividend_extension_parm.e $"

class EVENT_O_A_DIVIDEND_EXTENSION_PARM

inherit

	ROSE_SHARED_GLOBAL
		export {NONE} all end

	ANY

create

	make

feature {NONE} -- Initialization

	make is
			-- Initialize `Current'
		do
			create entries.make_equal (0)
			parse_from_windows_share
		end

feature -- Access

	entries: DS_HASH_TABLE [ROSE_INTERVAL_HISTORY [EVENT_Z_A_DIVIDEND_EXTENSION_PARM, ROSE_CLOSED_INTERVAL [EVENT_Z_A_DIVIDEND_EXTENSION_PARM]], MULTI_KEY]
			-- Entries

feature {NONE} -- Implementation

	parm_file_name: FILE_NAME is
			-- Parm file name
		once
			create Result.make
			Result.set_directory (global.hardcoded_path ("\\axaros.com\shares\ProdData_NT\PortfolioAccounting\ParmFiles"))
			Result.set_file_name ("EVENT_O_A-DIVIDEND_EXTENSION.PARM")
		ensure
			parm_file_name_not_void: Result /= Void
		end

	parse_from_windows_share is
			-- Parse 'a_file_name' and populate internal table
		require
			entries_not_void: entries /= Void
		local
			l_zone_file: EVENT_Z_A_DIVIDEND_EXTENSION_PARM_FILE
		do
			create l_zone_file.make (parm_file_name)
			if l_zone_file.exists then
				l_zone_file.read_with_format
				l_zone_file.array.do_all (agent add_interval (?))
			else
				(create {EXCEPTIONS}).raise (parm_file_name.out + " does not exist")
			end
		end

	add_interval (a_record: EVENT_Z_A_DIVIDEND_EXTENSION_PARM_RECORD) is
			-- Add interval to `entries' with data from `a_record'
		require
			entries_not_void: entries /= Void
			a_record_not_void: a_record /= Void
		local
			l_record: EVENT_Z_A_DIVIDEND_EXTENSION_PARM
			l_history: ROSE_INTERVAL_HISTORY [EVENT_Z_A_DIVIDEND_EXTENSION_PARM, ROSE_CLOSED_INTERVAL [EVENT_Z_A_DIVIDEND_EXTENSION_PARM]]
			l_interval: ROSE_CLOSED_INTERVAL [EVENT_Z_A_DIVIDEND_EXTENSION_PARM]
			l_key: MULTI_KEY
		do
			if not a_record.is_valid then
				(create {EXCEPTIONS}).raise ("Invalid record " + l_record.debug_output + " in " + parm_file_name)
			end
			create l_record.make (a_record)
			create l_key.make (<<l_record.country_iso_code, l_record.currency_iso_code>>)
			entries.search (l_key)
			if entries.found then
				l_history := entries.found_item
			else
				create l_history.make_empty
				entries.force (l_history, l_key)
			end
			from
				l_history.start
			until
				l_history.after
			loop
				l_interval := l_history.current_item
				if l_interval.brackets (l_record.start_date) or l_interval.brackets (l_record.end_date) then
					(create {EXCEPTIONS}).raise ("Overlapping dividend extension intervals " + l_record.debug_output + " and " + l_interval.item.debug_output + " in " + parm_file_name)
				end
				l_history.forth
			end
			create l_interval.make (l_record, l_record.start_date, l_record.end_date, l_record.number_of_days.to_double)
			l_history.extend (l_interval)
		end

invariant

	entries_not_void: entries /= Void

end
