indexing

	description:

		"Objects that facilitate reading and writing to Excel files"

	author: "Pierre Nallet"
	last_editor: "$Author: e2 $"
	date: "$Date: 2014/12/09 $"
	revision: "$Revision: #37 $"
	path: "$File: //rose/source/kernel/infrastructure/windows/classic/excel_application.e $"

class EXCEL_APPLICATION

inherit

	COM_DISPATCH

	MEMORY_FUNCTIONS

	COM_CLSCTX

create

	make,
	make_in_new_instance

feature {NONE} -- Initialization

	make
			-- Creation procedure.
		local
			hr: INTEGER
			l_pointer: POINTER
		do
			hr := oleinitialize (default_pointer)
			hr := GetActiveObject (clsid.item, default_pointer, $l_pointer)
			if hr >= 0 then
				hr := iunknown_queryinterface (l_pointer, iid_idispatch, $p)
				hr := iunknown_release (l_pointer)
			end
			if not exists then
				create_runnable_instance (clsid.item, clsctx_all, iid_idispatch)
			end
			is_shown := True
		ensure
			exists: exists
		end

	make_in_new_instance
			-- Creation procedure.
		local
			hr: INTEGER
		do
			hr := oleinitialize (default_pointer)
			create_runnable_instance (clsid.item, clsctx_all, iid_idispatch)
			is_shown := True
		ensure
			exists: exists
		end

feature -- Access

	get_active_object
			-- Get the active object for Excel
		local
			l_pointer: POINTER
			i: INTEGER
		do
			if CoInitialize (default_pointer) < s_ok then
				com_init_error_process
			end
			i := GetActiveObject (clsid.item, default_pointer, $l_pointer)
			if l_pointer /= default_pointer then
				make_by_interface_pointer (l_pointer, iid_IDispatch)
				i := IUnknown_Release (l_pointer)
			end
		end

	imported_worksheet_by_name (a_name: STRING; a_first_column, a_first_row, a_width, a_height_upper_bound: INTEGER): ARRAY2 [ANY]
			--	Imported version of `a_name' in memory;
			-- Require: `workbooks.open' has been called.
		require
			a_name_not_void: a_name /= Void
			positive_upper_bound: a_height_upper_bound > 0
			positive_width: a_width > 0
			positive_column: a_first_column > 0
			positive_row: a_first_row > 0
		local
			l_name: COM_VARIANT
			l_worksheet: EXCEL_WORKSHEET
		do
			create l_name
			l_name.init
			l_name.set_from_string (a_name)
			l_worksheet := sheets.com_item (l_name)
			if l_worksheet /= Void then
				Result := imported_worksheet (l_worksheet, a_first_column, a_first_row, a_width, a_height_upper_bound)
			end
		ensure
			correct_width: Result /= Void implies Result.width = a_width - a_first_column + 1
			correct_height: Result /= Void implies Result.height <= a_height_upper_bound - a_first_row + 1
		end

	imported_worksheet (a_worksheet: EXCEL_WORKSHEET; a_first_column, a_first_row, a_width, a_height_upper_bound: INTEGER): ARRAY2 [ANY]
			--	Imported version of `a_worksheet' in memory
		require
			a_worksheet_not_void: a_worksheet /= Void
			positive_upper_bound: a_height_upper_bound > 0
			positive_width: a_width > 0
			positive_column: a_first_column > 0
			positive_row: a_first_row > 0
		local
			l_range: EXCEL_RANGE
			l_array: ARRAY2 [ANY]
			i, j: INTEGER
			l_finished: BOOLEAN
		do
			l_range := a_worksheet.range_by_name (range_string (a_first_row, a_first_column, a_height_upper_bound - a_first_row + 1, a_width - a_first_column + 1))
			l_array := l_range.get_array2
			from
				i := 1
			until
				l_finished
			loop
				if l_array.item (i, 1) = Void then
					l_finished := True
					i := i - 1
				else
					i := i + 1
				end
			end
			create Result.make (i, a_width)
			from
				i := 1
			until
				i > Result.height
			loop
				from
					j := 1
				until
					j > a_width
				loop
					Result.put (l_array.item (i, j), i, j)
					j := j + 1
				end
				i := i + 1
			end
		ensure
			worksheet_not_void: Result /= Void
			correct_width: Result.width = a_width - a_first_column + 1
			correct_height: Result.height <= a_height_upper_bound - a_first_row + 1
		end

	get_data (a_name: STRING_8; a_column_index, a_row_index, a_column_span, a_row_span: INTEGER_32): ARRAY2 [ANY]
			-- Get a 2D array for a range
		require
			a_name_not_void: a_name /= Void
			a_column_index_positive: a_column_index > 0
			a_row_index_positive: a_row_index > 0
			a_column_span_positive: a_column_span > 0
			a_row_span_positive: a_row_span > 0
		local
			l_name: COM_VARIANT
			l_worksheet: EXCEL_WORKSHEET
			l_range: EXCEL_RANGE
		do
			create l_name
			l_name.init
			l_name.set_from_string (a_name)
			l_worksheet := sheets.com_item (l_name)
			if l_worksheet /= Void then
				l_range := l_worksheet.range_by_name (range_string (a_row_index, a_column_index, a_row_span + a_row_index - 1, a_column_span + a_column_index - 1))
				Result := l_range.get_array2
			end
		end

	visible: BOOLEAN
			-- Is Excel visible
		do
			Result := get_boolean_property (Visible_id)
		end

	workbooks: EXCEL_WORKBOOKS
			-- Collection of workbooks
		local
			l_pointer: POINTER
		do
			l_pointer := get_dispatch_property (get_id ("Workbooks"))
			if l_pointer /= default_pointer then
				create Result.make_by_pointer (l_pointer)
				release_variant
			end
		end

	charts: EXCEL_CHARTS
			-- Charts
		local
			l_pointer: POINTER
		do
			l_pointer := get_dispatch_property (get_id ("Charts"))
			if l_pointer /= default_pointer then
				create Result.make_by_pointer (l_pointer)
				release_variant
			end
		end

	active_sheet: EXCEL_WORKSHEET
			-- Active sheet
		local
			l_pointer: POINTER
		do
			l_pointer := get_dispatch_property (get_id ("ActiveSheet"))
			if l_pointer /= default_pointer then
				create Result.make_by_pointer (l_pointer)
				release_variant
			end
		end

	sheets: EXCEL_WORKSHEETS
			-- Worksheets
		local
			l_pointer: POINTER
		do
			l_pointer := get_dispatch_property (get_id ("Sheets"))
			if l_pointer /= default_pointer then
				create Result.make_by_pointer (l_pointer)
				release_variant
			end
		end

	active_chart: EXCEL_CHART
			-- Active chart
		local
			l_pointer: POINTER
		do
			l_pointer := get_dispatch_property (get_id ("ActiveChart"))
			if l_pointer /= default_pointer then
				create Result.make_by_pointer (l_pointer)
				release_variant
			end
		end

	range_string (a_upper_left_row, a_upper_left_column, a_lower_right_row, a_lower_right_column: INTEGER): STRING
			-- Label for a range
		do
			Result := column_name (a_upper_left_column) + a_upper_left_row.out + ":" + column_name (a_lower_right_column) + a_lower_right_row.out
		end

	automatic_formatting: BOOLEAN
			-- Is automatic formatting enabled?

	column_name (i: INTEGER): STRING
			-- 1 -> "A", 26 -> "Z", 27 -> "AA"
		require
			i_large_enough: i >= 1
			i_small_enough: i <= 256
		local
			d: INTEGER
		do
			if i <= 26 then
				Result := ('A' + (i - 1)).out
			else
				d := (i - 1) // 26
				Result := column_name (d)
				Result.append (column_name (i - 26 * d))
			end
		end

	format (a_value: ANY): STRING
			-- Strange function
		local
			l_string: STRING
		do
			l_string ?= a_value
			if l_string /= Void then
				Result := "@"
			else
				Result := "General"
			end
		end

feature -- Element modification

	set_visible (b: BOOLEAN)
			-- Set the visible property
		local
			var: COM_VARIANT
			l_retrying : BOOLEAN
		do
			if not l_retrying then
				function_exception.init
				create var
				var.init
				var.set_boolean (b)
				put_property (Visible_id, var)
			end
		rescue
			l_retrying := True
			retry
		end

	set_format (a_range, a_format: STRING)
			-- Set the format of a range
		local
			l_format: COM_VARIANT
			l_range: EXCEL_RANGE
		do
			l_range := active_sheet.range_by_name (a_range)
			create l_format
			l_format.init
			l_format.set_from_string (a_format)
			l_range.set_number_format (l_format)
		end

	set_automatic_formatting (a_value: BOOLEAN)
			-- Set automatic formatting
		do
			automatic_formatting := a_value
		end

feature -- Basic operations

	test
			-- Test
		local
			count: INTEGER
			hr: INTEGER
			i: INTEGER
			wbs: EXCEL_WORKBOOKS
			wb: EXCEL_WORKBOOK
		do
			if false then
				hr := idispatch_GetTypeInfoCount (p, $count)
				print (hr)
				print (count)
				from
					i := 1
				until
					i > 1
				loop
					print (get_ids_of_names (<<"visible">>))
					i := i + 1
				end
				set_visible (false)
				set_visible (false)
				print ("after show")
				set_visible (True)
				set_visible (False)
				wbs := workbooks
				print (wbs)
				set_visible (True)
				wb := wbs.add (Void)
			end
		end

	quit is
			-- Shut down excel
		do
			method (get_id ("Quit"))
		end

feature {ROSE_EXCEL_APPLICATION} -- Implementation

	add_workbook (a_array: ARRAY2 [ANY]; a_title: STRING)
			-- Add a workbook
		local
			l_workbooks: EXCEL_WORKBOOKS
			l_workbook: EXCEL_WORKBOOK
			l_worksheet: EXCEL_WORKSHEET
			l_range: EXCEL_RANGE
			s, l_column_name, l_title: STRING
			i, j: INTEGER
		do
			set_visible (is_shown)
			l_workbooks := workbooks
			l_workbook := l_workbooks.add (Void)
			l_worksheet := active_sheet
				-- Excel limit the length of the name string to 31
			if a_title /= Void and then a_title.count > 31 then
				l_title := a_title.substring (1, 31)
			else
				l_title := a_title
			end
			l_worksheet.set_name (l_title)
			if automatic_formatting then
				from
					i := 1
				until
					i > a_array.width
				loop
					l_column_name := column_name (i)
					from
						j := 1
					until
						j > a_array.height
					loop
						l_range := l_worksheet.range_by_name (l_column_name + j.out)
						l_range.set_format (format (a_array.item (j, i)))
						j := j + 1
					end
					i := i + 1
				end
			end
			s := "A1:"
			s.append (column_name (a_array.width))
			s.append (a_array.height.out)
			l_range := l_worksheet.range_by_name (s)
			l_range.set_value_array (a_array)
			l_worksheet.release
			l_range.release
			l_workbooks.release
			l_workbook.release
		end

	add_worksheet (a_array: ARRAY2 [ANY]; a_title: STRING)
			-- Add a worksheet
		local
			l_worksheets: EXCEL_WORKSHEETS
			l_worksheet: EXCEL_WORKSHEET
			l_range: EXCEL_RANGE
			s, l_column_name, l_title: STRING
			i, j: INTEGER
		do
			set_visible (True)
			l_worksheets := sheets
			l_worksheet := l_worksheets.add (Void, Void, Void, Void)
				-- Excel limit the length of the name string to 31
			if a_title /= Void and then a_title.count > 31 then
				l_title := a_title.substring (1, 31)
			else
				l_title := a_title
			end
			l_worksheet.set_name (l_title)
			if automatic_formatting then
				from
					i := 1
				until
					i > a_array.width
				loop
					l_column_name := column_name (i)
					from
						j := 1
					until
						j > a_array.height
					loop
						l_range := l_worksheet.range_by_name (l_column_name + j.out)
						l_range.set_format (format (a_array.item (j, i)))
						j := j + 1
					end
					i := i + 1
				end
			end
			if not a_array.is_empty then
				s := "A1:"
				s.append (column_name (a_array.width))
				s.append (a_array.height.out)
				l_range := l_worksheet.range_by_name (s)
				l_range.set_value_array (a_array)
				l_range.release
			end
			l_worksheet.release
		end

	is_shown: BOOLEAN
			-- Not sure what this is

	safe_add_workbook (a_array: ARRAY2 [ANY]; a_title: STRING)
			-- Add a workbook with data
		do
			if p = default_pointer then
				make
			end
			add_workbook (a_array, a_title)
			release
		end

feature {NONE} -- Implementation

	create_object
			-- Create a new excel application object
		local
			l_pointer: POINTER
			i: INTEGER
		do
			if CoInitialize (default_pointer) < s_ok then
				com_init_error_process
			end
			i := CoCreateInstance (clsid.item, default_pointer, Clsctx_all, IID_IUnknown, $l_pointer)
			if l_pointer /= default_pointer then
				make_by_interface_pointer (l_pointer, iid_IDispatch)
				i := IUnknown_Release (l_pointer)
			end
		end

	class_id_string: STRING
			-- Class identifier of Excel
		once
			Result := "{00020812-0000-0000-C000-000000000046}"
			Result := "{00024500-0000-0000-C000-000000000046}"
			Result := "{00024500-0000-0000-C000-000000000046}"
		end

	class_id: POINTER
			-- Class id
		once
			Result := clsid.item
		end

	clsid: COM_GUID [RUN_TIME_MEMORY_MODEL]
			-- Class id
		once
			create Result.make
			Result.set_from_string (class_id_string)
		end

	Visible_id: INTEGER
			-- Internal ID for visible property
		do
			Result := get_id ("visible")
		end

	GetActiveObject (rclsid: POINTER; pvReserved, ppunk: POINTER): INTEGER
			-- Implementation
		external
			"C [macro <oleauto.h>] (REFCLSID  , void FAR*, IUnknown FAR* FAR*): EIF_INTEGER"
		alias
			"GetActiveObject"
		end

end
