indexing

	description: "Frequently Used Excel Interface"

	author: "Michael Zhang"
	last_editor: "$Author: er $"
	date: "$Date: 2014/02/26 $"
	revision: "$Revision: #14 $"
	path: "$File: //rose/source/kernel/support/excel_interface.e $"

class EXCEL_INTERFACE

inherit

	MATRIX_HELPER

feature -- Vector exported to excel

	vector_to_excel_with_names (a_vector : ABSTRACT_READABLE_VECTOR; a_column: BOOLEAN; a_name: STRING)
			-- Export `a_vector' in row or column format to excel in tab with `a_name'.
		require
			names_not_void: a_vector.names /= Void
		local
			l_matrix: MATRIX
		do
			if a_column then
				l_matrix := a_vector.to_column_matrix
				if a_vector.names /= Void then
					l_matrix.set_row_names (a_vector.names)
				end
			else
				l_matrix := a_vector.to_row_matrix
				if a_vector.names /= Void then
					l_matrix.set_column_names (a_vector.names)
				end
			end
			l_matrix.set_name (a_name)
			matrix_to_excel_with_names (l_matrix)
		end

	vector_to_excel (a_vector : ABSTRACT_READABLE_VECTOR; a_is_column: BOOLEAN; a_name: STRING)
			-- Export `a_vector' in row or column format to excel in tab with `a_name'.
		local
			l_matrix: MATRIX
			l_column_names, l_row_names: ARRAY [ANY]
		do
			if a_is_column then
				l_matrix := a_vector.to_column_matrix
				if a_vector.names /= Void then
					l_row_names := a_vector.names
					l_column_names := <<a_vector.name>>
				end
			else
				l_matrix := a_vector.to_row_matrix
				if a_vector.names /= Void then
					l_column_names := a_vector.names
					l_row_names := <<a_vector.name>>
				end
			end
			if a_name /= Void and then not a_name.is_empty then
				l_matrix.set_name (a_name)
			else
				l_matrix.set_name ("data")
			end
			matrix_to_excel (l_matrix, l_row_names, l_column_names, l_matrix.name)
		end


feature -- Matrices exported to excel

	matrix_to_excel_with_names(a_matrix : ABSTRACT_READABLE_MATRIX) is
			-- matrix to excel
		local
			l_name : STRING
		do
			l_name := a_matrix.name
			if l_name = Void then
				l_name := "data"
			end
			matrix_to_excel (a_matrix, a_matrix.row_names, a_matrix.column_names, l_name)
		end

	matrix_to_excel(a_matrix : ABSTRACT_READABLE_MATRIX; a_row_label:ARRAY[ANY];a_column_label:ARRAY[ANY];a_name:STRING)
			-- matrix to excel
		do
			matrix_combined_to_excel (a_matrix, a_row_label, a_column_label, a_name, Void)
		end

	matrix_combined_to_excel(a_matrix : ABSTRACT_READABLE_MATRIX; a_row_label:ARRAY[ANY];a_column_label:ARRAY[ANY];a_name:STRING;a_other:ARRAY2[ANY])	is
			-- matrix combined to excel
		require
			valid_name : a_name/=void
		local
			l_pair : PAIR[ARRAY2[ANY], STRING]
		do
			l_pair := 	matrix_combined_to_array2 (a_matrix, a_row_label, a_column_label, a_name, a_other)
			if l_pair /= Void then
				array2_to_excel (l_pair.first, l_pair.second)
			end
		end

	matrix_array_to_excel(a_array : ARRAY[ABSTRACT_READABLE_MATRIX]) is
			-- array of matrix to excel
		local
--			i : INTEGER
--			l_matrix : ABSTRACT_READABLE_MATRIX
--			l_pair : PAIR[ARRAY2 [ANY],STRING]
--			l_array: ROSE_LINEAR_ARRAY [TUPLE [ARRAY2 [ANY],STRING]]
			l_excel : ROSE_EXCEL_APPLICATION
		do
--			create l_array.make_empty
--			from i := a_array.upper until i < a_array.lower loop
--				l_matrix := a_array[i]
--				l_pair := matrix_combined_to_array2(l_matrix, l_matrix.row_names, l_matrix.column_names, l_matrix.name, Void)
--				if l_pair /= Void then
--					l_array.extend([l_pair.first, l_pair.second])
--				end
--				i := i - 1
--			end
			create l_excel.make_single_workbook_general (matrix_array_to_array2_array (a_array), True)
		end



feature -- Array2 exported to excel

	array2_to_excel(a_array2 : ARRAY2[ANY]; a_name : STRING) is
			-- array2 to excel
		local
			l_excel : ROSE_EXCEL_APPLICATION
		do
			create l_excel.make_simple (a_array2, a_name)
		end

	array2_array_to_excel(a_array2_array : ARRAY[TUPLE[ARRAY2[ANY], STRING]]) is
			-- array2 to excel
		local
			l_excel : ROSE_EXCEL_APPLICATION
		do
			create l_excel.make_single_workbook_general (a_array2_array, True)
		end

	array2_pair_to_excel(a_array2 : PAIR[ARRAY2[ANY], STRING]) is
			-- array2 pair to excel
		do
			array2_pair_array_to_excel (<<a_array2>>)
		end

	array2_pair_array_to_excel (a_array2_array : ARRAY [PAIR [ARRAY2 [ANY], STRING]])
			-- array2 pair array to excel
		local
			l_array: ARRAY [TUPLE [ARRAY2[ANY], STRING]]
			i: INTEGER
		do
			if a_array2_array /= Void then
				create l_array.make (a_array2_array.lower, a_array2_array.upper)
				from
					i := 1
				until
					i > a_array2_array.count
				loop
					if a_array2_array.item (i) /= Void then
						l_array.put ([a_array2_array.item (i).first, a_array2_array.item (i).second], a_array2_array.count - i + 1)
					end
					i := i + 1
				end
				array2_array_to_excel (l_array)
			end
		end


feature -- Implementation
	matrix_to_array2(a_matrix : ABSTRACT_READABLE_MATRIX) : ARRAY2[ANY] is
			-- matrix to array2
		do
			Result := matrix_combined_to_array2(a_matrix, a_matrix.row_names, a_matrix.column_names, a_matrix.name, Void).first
		end


end
