indexing
	description: "Encapsulation of a COM Excel range"
	author: ""
	date: "$Date: 2014/04/16 $"
	revision: "$Revision: #30 $"

class
	EXCEL_RANGE

inherit
	COM_DISPATCH
	COM_VARENUM
	ROSE_LEGACY_UNDEFINED

create {EXCEL_WORKSHEET}
	make_by_pointer,
	make

feature {NONE}-- Initialization

	make (s: EXCEL_WORKSHEET; a_name: STRING) is
			-- Make the current range with in worksheet 's' at location a_name.
		require
			s_not_void: s /= Void
			a_name_not_void: a_name /= Void
		do
			dispparams.init
			function_exception.init
			g_variant.init
			g_variant.set_from_string (a_name)
			dispparams.add_argument (g_variant)
			g_variant.init
			s.invoke (s.get_id ("Range"), Dispatch_propertyget, dispparams, g_variant, function_exception)
			check_result
			if g_variant.dispatch_pointer /= default_pointer then
				make_by_pointer (g_variant.dispatch_pointer)
			end
			release_variant
		end

feature -- Access

	interior : EXCEL_INTERIOR is
			-- Background of the range
		do
			dispparams.init
			function_exception.init
			g_variant.init
			invoke (get_id ("Interior"), Dispatch_propertyget, dispparams, g_variant, function_exception)
			check_result
			if g_variant.dispatch_pointer /= default_pointer then
				create Result.make_by_pointer (g_variant.dispatch_pointer)
			end
			release_variant

		end

	set_value (a_value: COM_VARIANT) is
			-- Set the value of the current range
		require
			a_value_value_not_void: a_value /= Void
		do
			put_property (get_id ("Value"), a_value)
		end

	set_value_2 (a_value: ANY) is
			-- Set the value of the current range
		local
			v: COM_VARIANT2
		do
			create v
			v.init
			v.from_any (a_value)
			set_value (v)
		end


	get_value: COM_VARIANT2 is
			-- Get the value of the current range
		do
			dispparams.init
			function_exception.init
			create Result
			Result.init

			invoke (get_id ("Value"), Dispatch_propertyget, dispparams, Result, function_exception)
			check_result
		end

	get_array2 : ARRAY2[ANY] is
			-- Get the value as an array of objects
		local
			l_safe_array : COM_VARIANT2
		do
			l_safe_array ?= get_value
			if l_safe_array /= Void then
				Result := safe_array_to_array2(l_safe_array.get_safearray)
			end
		end

	set_value_array (array: ARRAY2 [ANY]) is
			-- Set the value as an array of objects
		require
			array_not_void: array /= Void
		local
			cs: COM_SAFEARRAY
			v: COM_VARIANT2
		do
			cs := array2_to_safe_array (array)
			create v
			v.init
			v.set_safearray (cs)
			set_value (v)
		end

	set_format (a_string : STRING) is
			-- Set the format
		do
			g_variant.init
			g_variant.set_from_string(a_string)
			set_number_format(g_variant)
			release_variant
		end

	auto_fit is
			-- Auto fit the range
		do
			dispparams.init
			function_exception.init
			g_variant.init
			invoke (get_id ("AutoFit"), Dispatch_method, dispparams, g_variant, function_exception)
			check_result
			if g_variant.dispatch_pointer /= default_pointer then
			end
			release_variant
		end

	set_column_width (i: DOUBLE) is
			-- Set the column width of ith column
		local
			v: COM_VARIANT
		do
			create v
			v.init
			v.set_real8 (i)
			put_property (get_id("ColumnWidth"), v)
		end

	set_Number_Format (a_format: COM_VARIANT) is
			-- Set  the number format
		do
			put_property (get_id ("NumberFormat"), a_format)
		end

	set_style (a_style: COM_VARIANT) is
			-- Set the style of the current range
		do
			put_property (get_id ("Style"), a_style)
		end

	font: EXCEL_FONT is
			-- Returns the font if uniform
		local
			pp: POINTER
		do
			pp := get_dispatch_property (get_id ("Font"))
			if pp /= default_pointer then
				create Result.make_by_pointer (pp)
				release_variant
			end
		end

feature {NONE} -- Implementation

	array2_to_safe_array (array: ARRAY2 [ANY]): COM_SAFEARRAY is
			-- Utility function to transform an array into a safearray
		local
			i, j: INTEGER
			l_bounds: COM_SAFEARRAY_BOUND
			a: ANY
			r: REAL_REF
			d: DOUBLE_REF
			s: STRING
			ii: INTEGER_REF
			l_date : ROSE_DATE
			var : COM_VARIANT
		do
			create l_bounds.make (2)
			create var
			var.init
			l_bounds.set_element_count (1, array.height)
			l_bounds.set_lower_bound (1, 1)
			l_bounds.set_element_count (2, array.width)
			l_bounds.set_lower_bound (2, 1)
			create Result.make (2, VT_VARIANT, l_bounds)
			from i := 1 until i > array.height loop
				from j := 1 until j > array.width loop
					a := array.item (i, j)
					r ?= a
					if r /= Void then
						if is_defined_real(r.item) then
							var.set_real4 (r.item)
						elseif not constants.ieee.isfinite(r.item) then
							var.set_from_string("#DIV/0!")
						else
							var.set_from_string("#N/A")
						end
					else
						d ?= a
						if d /= Void then
							if is_defined_double(d.item) then
								var.set_real8 (d.item)
							elseif not constants.ieee.isfinite(d.item) then
								var.set_from_string("#DIV/0!")
							else
								var.set_from_string("#N/A")
							end
						else
							ii ?= a
							if ii /= Void then
								var.set_integer4 (ii.item)
							else
								l_date ?= a
								if l_date /= Void then
									s := l_date.ccyymmdd.out
								else
									s ?= a
								end
								if s = Void then
									s := ""
								end
								var.set_from_string (s)
							end
						end
					end
					Result.put_element (<<i, j>>, var)
					j := j + 1
				end
				i := i + 1
			end
		end

	safe_array_to_array2 (a_array: COM_SAFEARRAY): ARRAY2 [ANY] is
			-- Utility function to transform a safe array into an array.
		local
			i, j: INTEGER
			l_lower_1_bound, l_upper_1_bound : INTEGER
			l_lower_2_bound, l_upper_2_bound : INTEGER
			d: DOUBLE
			s: STRING
			ii: INTEGER
			a : ANY
			l_var : COM_VARIANT
		do
			l_lower_1_bound := a_array.lower_bound(1)
			l_upper_1_bound := a_array.upper_bound(1)
			l_lower_2_bound := a_array.lower_bound(2)
			l_upper_2_bound := a_array.upper_bound(2)
			create Result.make (l_upper_1_bound,l_upper_2_bound)
			from i := 1 until i > l_upper_1_bound loop
				from j := 1 until j > l_upper_2_bound loop
					create l_var
					l_var.init
					a_array.get_element(<<i,j>>,l_var)
					if l_var.var_type /= 0 then
						a := l_var.to_any
						if a /= Void then
							--print(a.out + "%N")
							s := a.out
							if s /= Void then
								if s.is_double then
									d := s.to_double
									if is_defined_real(d.item.truncated_to_real) then
										Result.put(d,i,j)
									elseif not constants.ieee.isfinite(d) then
										Result.put(d,i,j)
									else
										Result.put(d,i,j)
									end
								elseif s.is_integer then
									ii := s.to_integer
									Result.put(ii,i,j)
								else
									Result.put(s,i,j)
								end
							end
						end
					end
					j := j + 1
				end
				i := i + 1
			end
		end

end
