indexing
	description : "[
		Objects which can modify a data source using a string description.  Database updates
		are a specific example.
		]"
	author : "Mark Howard"
	last_editor: "$Author: an $"
	date: "$Date: 2013/10/18 $"
	revision: "$Revision: #29 $"
	path: "$File: //rose/source/kernel/infrastructure/persistence/predicates/execution_predicate.e $"

class
	EXECUTION_PREDICATE

inherit
	ROSE_PREDICATE [ANY]
		redefine
			facilitator
		end

	ROSE_PROCESS_TIMER_COLLECTION_AWARE

create
	make, make_with_parameter_template

feature {NONE} -- Creation

	make (a_data_server: like data_server; a_selection: STRING) is
			-- Create the predicate
		require
			valid_server: a_data_server /= Void
			valid_selection: a_selection /= Void
		do
			make_with_parameter_template (a_data_server, a_selection, Void)
		end

	make_with_parameter_template (a_data_server: like data_server; a_selection: STRING; a_parameter_template: ARRAY[STRING]) is
			-- Create the predicate using a parameter template
			-- FIXME -- this should be merged with the version from general_selection_predicate, maybe to rose_predicate
		require
			valid_server: a_data_server /= Void
			valid_selection: a_selection /= Void
		local
			l_triple: TRIPLE [STRING, ARRAY [STRING], BOOLEAN]
		do
			debug("make_movies")
				create cam.make(generating_type + "/" + a_selection + "/" + a_data_server.node + "/" + a_data_server.database)
			end
			pool.record_sql_statement_preparation (a_data_server, a_selection)
			data_server := a_data_server
			selection := a_selection
			server_full_path := data_server.node + "." + data_server.database
			if playback_movie then
				l_triple ?= cam.next_frame("make_with_parameter_template").frame_data
				cam.compare_partial_movie_frame(l_triple.first, a_selection)
				cam.compare_partial_movie_frame(l_triple.second, a_parameter_template)
				success := l_triple.third
			else
				facilitator := data_server.new_executor (a_selection, a_parameter_template)
				success := facilitator.success
				debug("make_movies")
					if record_movie	then
						create l_triple.make(a_selection, a_parameter_template, success)
						cam.record_movie_frame("make_with_parameter_template", l_triple)
					end
				end
			end
		end

feature -- Access

	object_argument: ANY
			-- The current object argument
	array_argument: ARRAY [ANY]
			-- The current array argument
	facilitator: EXECUTOR
			-- The executor for the predicate

	server_full_path: STRING -- FIXME -- I think this can be removed
			-- complete spec for database connection
			-- FIXME -- this should be merged with the version from general_selection_predicate, maybe to rose_predicate

feature -- Basic operations

	execute_on_container (a_arguments: ARRAY [ANY]) is
			-- Execute the predicate using the current 'container' as the object input
		local
			l_linear: LINEAR [ANY]
		do
			debug("make_movies")
				if record_movie	then
					cam.start_superframe
				end
			end
			from
				l_linear := container.linear_representation
				clear_error
				l_linear.start
			until
				not success or else l_linear.off
			loop
				execute (l_linear.item, a_arguments)
				l_linear.forth
			end
			debug("make_movies")
				if record_movie	then
					cam.end_superframe
				end
			end
		end

	execute (a_argument: ANY; a_arguments: ARRAY [ANY]) is
			-- Execute the predicate
		local
			l_cloned_argument: ANY
			l_cloned_arguments: ARRAY[ANY]
			l_triple: TRIPLE[ANY, ARRAY[ANY], BOOLEAN]
			l_timer_minor: STRING
		do
			if data_server.definition /= Void and not facilitator.should_not_be_tracked then
				data_server.definition.increment_execution_count
				parse_sql_statement (selection, data_server.definition)
			end
			object_argument := a_argument
			array_argument := a_arguments
			if playback_movie then
				l_triple ?= cam.next_frame("execute").frame_data
				cam.compare_partial_movie_frame(a_argument, l_triple.first)
				cam.compare_partial_movie_frame(a_arguments, l_triple.second)
				success := l_triple.third
			else
				debug("make_movies")
					if record_movie	then
						l_cloned_argument ?= cam.safe_deep_clone(a_argument)
						l_cloned_arguments ?= cam.safe_deep_clone(a_arguments)
					end
				end
				debug("timers_predicates")
					l_timer_minor := "execute: " + generating_type + ": " + selection
					timers.resume(server_full_path, l_timer_minor)
				end

				facilitator.execute (a_argument, a_arguments)
				success := facilitator.success

				debug("timers_predicates")
					timers.interrupt(server_full_path, l_timer_minor)
				end
			end
			debug("make_movies")
				if record_movie	then
					create l_triple.make(l_cloned_argument, l_cloned_arguments, success)
					cam.record_movie_frame("execute", l_triple)
				end
			end
		end

end
