indexing
	description : "[
		Descendents of this class are generated by the classifier in order to facilitate
		communication between eiffel and external C structures
		]"
	author : "Mark Howard"
	last_editor: "$Author: rj $"
	date: "$Date: 2014/06/19 $"
	revision: "$Revision: #25 $"
	path: "$File: //rose/source/kernel/infrastructure/structures/external_record.e $"

class EXTERNAL_RECORD

inherit
	ACCESS_EXTERNALS;
	DISPLAYABLE

feature -- Operations

	classifier_major_version_ : INTEGER is
		do
			Result := 1
		end

	classifier_minor_version_ : INTEGER is
		do
			Result := 0
		end

	read is
		do
		end

	write is
		do
		end

	read_string (a_string : STRING) is
			-- read record from `a_string'
			-- (must be top level else source_pointer is ignored)
		require
			is_top_level_record: is_top_level
		local
			l_any : ANY
			l_collecting : BOOLEAN
		do
			l_collecting := memory_.collecting
			if l_collecting then memory_.collection_off end
			l_any := a_string.area
			set ($l_any)
			read
			--set (default_pointer)
			if l_collecting then memory_.collection_on end
		rescue
			--set (default_pointer)
			if l_collecting then memory_.collection_on end
		end

	write_string_ : STRING is
			-- string representation of Current
			-- (must be top level else source_pointer is ignored)
			-- cannot use C_STRING because record may conain embedded /0/ characters
		require
			is_top_level_record: is_top_level
		local
			l_any : ANY
			l_collecting : BOOLEAN
		do
			l_collecting := memory_.collecting
			if l_collecting then memory_.collection_off end
			create Result.make(record_length_)
			Result.fill_blank -- is this necessary at all? should it be fill_character('%U')?
			--? perhaps Result.make_filled ('%U', record_length_)? or Result.make_filled ('%/0xFF/', record_length_)?
			l_any := Result.area
			set ($l_any)
			write
			--set (default_pointer)
			if l_collecting then memory_.collection_on end
		rescue
			--set (default_pointer)
			if l_collecting then memory_.collection_on end
		end

	record_length_ : INTEGER is
		do
			Result := length_
		end

	set (a_source : POINTER) is
		--require
			--not memory.collecting
			--? is_top_level_record: a_source /= default_pointer implies is_top_level
		do
			source_pointer_ := a_source
		end

	allocate is
		require
			--not memory.collecting
			is_top_level_record: is_top_level
		do
			if (pointer_= default_pointer) then
				source_pointer_ := source_pointer_.memory_calloc (i_1_ - min_index_ + 1, record_length_)
			end
		end

	free is
		require
			is_top_level_record: is_top_level
		do
			source_pointer_.memory_free
			source_pointer_ := default_pointer
		end

	is_null : BOOLEAN is
		do
			Result := (pointer_ = default_pointer)
		end

	is_top_level: BOOLEAN is
			-- is Current a top level record (i.e. not contained within another record; has no parent)?
		do
			Result := parent_record_ = Void
		end

	pointer_ : POINTER is
				-- memory address of record
		--require
			--not memory.collecting
		do
			if parent_record_ /= Void then
				Result := pointer_plus(parent_record_.pointer_,offset_ )
			else
				Result := source_pointer_
			end
		end

	set_parent_ (a_parent : EXTERNAL_RECORD) is
		--?require
		--?	has_no_memory_address: source_pointer_ = default_pointer
		do
			parent_record_ := a_parent
			--?source_pointer := default_pointer
		end

	set_parent_from(a_peer: like Current) is
		--?require
		--?	source_pointer_null: source_pointer_ = default_pointer
		do
			set_parent_(a_peer.parent_record_)
		end


feature {EXTERNAL_RECORD} -- Setting

	set_offset_ (an_offset : INTEGER) is
			-- Set base offset.
		do
			base_offset_ := an_offset
		end
	set_length_ (a_length : INTEGER) is
		do
			length_ := a_length
		end

	initialize_array_ (a_rray: ARRAY[EXTERNAL_RECORD]; a_parent: EXTERNAL_RECORD; an_offset, a_length: INTEGER) is
			-- initialize array of records as a subcomponent of `a_parent'
		require
			compatible_classifier_versions :
					a_parent /= Void implies
						(a_parent.classifier_major_version_ = classifier_major_version_ and
						 a_parent.classifier_minor_version_ >= classifier_minor_version_)
			array_exists: a_rray /= Void
			offset_positive: an_offset >= 0		-- remove when arg changed to NATURAL?
			length_positive: a_length >= 0		-- (ditto)
		local
			ii: INTEGER
		do
			from ii := a_rray.lower
			until ii > a_rray.upper
			loop
				if a_rray.item(ii) /= Void then
					a_rray.item(ii).initialize_item_ (a_parent, an_offset, a_length, ii, a_rray.lower, a_rray.upper)
				end
				ii := ii + 1
			end
		end

	initialize_item_ (a_parent: EXTERNAL_RECORD; an_offset, a_length: INTEGER; an_index, a_lower, a_upper: INTEGER) is
		require
			compatible_classifier_versions :
					a_parent /= Void implies
						(a_parent.classifier_major_version_ = classifier_major_version_ and
						 a_parent.classifier_minor_version_ <= classifier_minor_version_)
			offset_positive: an_offset >= 0		-- remove when arg changed to NATURAL?
			length_positive: a_length >= 0		-- (ditto)
			index_in_bounds: an_index >= a_lower and then an_index <= a_upper
		do
			source_pointer_ := default_pointer
			parent_record_ := a_parent
			base_offset_ := an_offset
			length_ := a_length
			i_1_ := an_index
			min_index_ := a_lower
			max_index_ := a_upper
		end

	initialize_ (a_parent : EXTERNAL_RECORD; an_offset : INTEGER; a_length : INTEGER; a_min, a_max : INTEGER) is
		--obsolete "use initialize_item_ or initialize_array_"
		require
			matching_classifier_versions :
					a_parent /= Void implies
						(a_parent.classifier_major_version_ = classifier_major_version_ and
						 a_parent.classifier_minor_version_ = classifier_minor_version_)
		do
			parent_record_ := a_parent
			base_offset_ := an_offset
			length_ := a_length
			set_bounds_ (a_min,a_max)
		end

	parent_record_ : EXTERNAL_RECORD
	base_offset_ : INTEGER
	length_ : INTEGER
	source_pointer_ : POINTER
	offset_ : INTEGER is
	    do
		    Result := base_offset_ + length_ * (i_1_ - min_index_)
	    ensure
		    positive_offset: Result >= 0
	    end
	min_index_ : INTEGER
	max_index_ : INTEGER
	i_1_ : INTEGER
	set_i1_ (an_index : INTEGER) is
	    do
		    i_1_ := an_index
	    end
	set_bounds_ (a_min,a_max : INTEGER) is
	    do
		    min_index_ := a_min
		    max_index_ := a_max
	    end

	short_interpreter_ : ROSE_SHORT_INTERPRETER is
		once
			create Result
		end
	string_interpreter_ : ROSE_STRING_INTERPRETER is
		once
			create Result
		end
	real_interpreter_ : ROSE_REAL_INTERPRETER is
		once
			create Result
		end
	integer_interpreter_ : ROSE_INTEGER_INTERPRETER is
			-- Interpreter that can handle INTEGER.
		once
			create Result
		end

	natural_32_interpreter_ : ROSE_NATURAL_32_INTERPRETER
			-- Interpreter that can handle NATURAL_32.
		once
			create Result
		end

	integer_64_interpreter_ : ROSE_INTEGER_64_INTERPRETER is
		once
			create Result
		end
	double_interpreter_ : ROSE_DOUBLE_INTERPRETER is
		once
			create Result
		end
	character_interpreter_ : ROSE_CHARACTER_INTERPRETER is
		once
			create Result
		end
	boolean_interpreter_ : ROSE_BOOLEAN_INTERPRETER is
		once
			create Result
		end
	byte_interpreter_ : ROSE_BYTE_INTERPRETER is
		once
			create Result
		end
	rose_date_interpreter_ : ROSE_DATE_INTERPRETER is
		once
			create Result
		end

feature {NONE} -- Implementation

	memory_ : MEMORY is once create Result end

invariant
--?	parent_record_ /= Void implies source_pointer_ = default_pointer
--?	source_pointer_ /= default_pointer implies parent_record_ = Void
--?	allocated_flag implies source_pointer_ /= default_pointer

end  -- EXTERNAL_RECORD
