note

	description:

		"Object which represents a rational by the pair of sets of factors of the numerator and denominator"

	author: "Mark Howard"
	last_editor: "$Author: er $"
	date: "$Date: 2013/06/18 $"
	revision: "$Revision: #14 $"
	path: "$File: //rose/source/kernel/infrastructure/math/factored_rational.e $"
	storable_version: ""

class
	FACTORED_RATIONAL
inherit
	RP_SERIALIZABLE
	FACTORED_RATIONALS
		redefine
			is_equal
		end
	NUMBER_THEORY
		redefine
			is_equal
		end
create
	make_from_factors,default_create
feature --Access
	to_reference : CELL[FACTORED_RATIONAL] is
			--reference to current implemented as cell item
		do
			create Result.put (void)
			Result.put(Current)
		end
feature --Item change
	set (a_numerator : like numerator;a_denominator : like denominator) 
			--Set the numerator and denominator
		do
			numerator := a_numerator
			denominator := a_denominator
		end
feature {NONE} --Implementation
	make_from_factors (a_numerator,a_denominator : ARRAY[INTEGER])is
		--make from prime factorization of numerator and denominator
		local
			l_numerator : INTEGER
			l_denominator : INTEGER
			l_gcd,l_reduced_gcd : INTEGER
			l_factors : ROSE_LINEAR_ARRAY[INTEGER]
			l_numerator_rla,l_denominator_rla : ROSE_LINEAR_ARRAY[INTEGER]
		do
			create l_numerator_rla.make_from_array(a_numerator)
			create l_denominator_rla.make_from_array(a_denominator)
			l_numerator := 1
			l_denominator := 1
			from l_numerator_rla.start until l_numerator_rla.off loop
				l_numerator := l_numerator * l_numerator_rla.current_item
				l_numerator_rla.forth
			end
			from l_denominator_rla.start until l_denominator_rla.off loop
				l_denominator := l_denominator * l_denominator_rla.current_item
				l_denominator_rla.forth
			end
			l_gcd := gcd(l_numerator,l_denominator)
			numerator := l_numerator // l_gcd
			denominator := l_denominator // l_gcd
			if not factors.has(l_numerator) then
				factors.put(l_numerator_rla,l_numerator)
			end
			if not factors.has(l_denominator) then
				factors.put(l_denominator_rla,l_denominator)
			end
			if numerator /= 1 then
				if not factors.has(numerator) then
					create l_factors.make_empty
					l_reduced_gcd := l_gcd
					from l_numerator_rla.start until l_numerator_rla.off loop
						if l_reduced_gcd /= 1 and then l_reduced_gcd \\ l_numerator_rla.current_item = 0 then
							l_reduced_gcd := l_reduced_gcd // l_numerator_rla.current_item
						else
							l_factors.extend(l_numerator_rla.current_item)
						end
						l_numerator_rla.forth
					end
					factors.put(l_factors,numerator)
				end
				if denominator /= 1 then
					if not factors.has(denominator) then
						create l_factors.make_empty
						l_reduced_gcd := l_gcd
						from l_denominator_rla.start until l_denominator_rla.off loop
							if l_reduced_gcd /= 1 and then l_reduced_gcd \\ l_denominator_rla.current_item = 0 then
								l_reduced_gcd := l_reduced_gcd // l_denominator_rla.current_item
							else
								l_factors.extend(l_denominator_rla.current_item)
							end
							l_denominator_rla.forth
						end
						factors.put(l_factors,denominator)
					end
				end
			end
			if not rationals.has(pair(numerator,denominator)) then
				rationals.put(Current,new_pair(numerator,denominator))
			end
		end
feature --Access
	gcd (i, j : INTEGER) : INTEGER is
			--greatest common divisor of i and j
		do
			if i = j then
				Result := i
			elseif i = 1 or j = 1 then
				Result := 1
			elseif i = 0 then
				Result := j
			elseif j = 0 then
				Result := i
			elseif i < j then
				Result := gcd(i,j\\i)
			else
				Result := gcd(j,i\\j)
			end
		end
	numerator : INTEGER
	denominator : INTEGER
	infix "*" (a_other : like Current) : like Current is
		local
			l_numerator_factors,l_denominator_factors : ROSE_LINEAR_ARRAY[INTEGER]
			l_gcd,l_numerator, l_denominator : INTEGER
			l_unknown_rational : like Current
		do
			l_numerator := numerator * a_other.numerator
			l_denominator := denominator * a_other.denominator
			if l_denominator > 0 and then l_numerator > 0 then
				l_gcd := gcd(l_numerator,l_denominator)
				Result := rationals.item(pair(l_numerator // l_gcd,l_denominator // l_gcd))
				if Result = l_unknown_rational then
					if numerator = 1 then
						l_numerator_factors := factors.item(a_other.numerator)
					elseif a_other.numerator = 1 then
						l_numerator_factors := factors.item(numerator)
					else
						l_numerator_factors := factors.item(l_numerator)
						if l_numerator_factors = Void then
							l_numerator_factors := factors.item(numerator) + factors.item(a_other.numerator)
						end
						factors.put(l_numerator_factors,l_numerator)
					end
					if denominator = 1 then
						l_denominator_factors := factors.item(a_other.denominator)
					elseif a_other.denominator = 1 then
						l_denominator_factors := factors.item(denominator)
					else
						l_denominator_factors := factors.item(l_denominator)
						if l_denominator_factors = Void then
							l_denominator_factors := factors.item(denominator) + factors.item(a_other.denominator)
						end
						factors.put(l_denominator_factors,l_denominator)
					end
					create Result.make_from_factors(l_numerator_factors,l_denominator_factors)
				end
			else
				create Result
			end
		end
	infix "/" (a_other : like Current) : like Current is
		require
			a_other /= zero
		local
			l_numerator_factors,l_denominator_factors : ROSE_LINEAR_ARRAY[INTEGER]
			l_gcd,l_numerator, l_denominator : INTEGER
			l_unknown_rational : like Current
		do
			l_numerator := numerator * a_other.denominator
			l_denominator := denominator * a_other.numerator
			if l_denominator > 0 and then l_numerator > 0 then
				l_gcd := gcd(l_numerator,l_denominator)
				Result := rationals.item(pair(l_numerator // l_gcd,l_denominator // l_gcd))
				if Result = l_unknown_rational then
					if numerator = 1 then
						l_numerator_factors := factors.item(a_other.denominator)
					elseif a_other.denominator = 1 then
						l_numerator_factors := factors.item(numerator)
					else
						l_numerator_factors := factors.item(l_numerator)
						if l_numerator_factors = Void then
							l_numerator_factors := factors.item(numerator) + factors.item(a_other.denominator)
						end
						factors.put(l_numerator_factors,l_numerator)
					end
					if denominator = 1 then
						l_denominator_factors := factors.item(a_other.numerator)
					elseif a_other.numerator = 1 then
						l_denominator_factors := factors.item(denominator)
					else
						l_denominator_factors := factors.item(l_denominator)
						if l_denominator_factors = Void then
							l_denominator_factors := factors.item(denominator) + factors.item(a_other.numerator)
							factors.put(l_denominator_factors,l_denominator)
						end
					end
					create Result.make_from_factors(l_numerator_factors,l_denominator_factors)
				end
			else
				create Result
			end
		end
	numerator_factors : ROSE_LINEAR_ARRAY[INTEGER] is
		do
			Result := factors.item(numerator)
			if Result = void then
				Result := factor(numerator)
				factors.put(Result,numerator)
			end
		end
	denominator_factors : ROSE_LINEAR_ARRAY[INTEGER] is
		do
			Result := factors.item(denominator)
			if Result = void then
				Result := factor(denominator)
				factors.put(Result,denominator)
			end
		end
	is_equal(a_other : like Current) : BOOLEAN is
		do
			Result := denominator = a_other.denominator and numerator = a_other.numerator
		end
	zero : FACTORED_RATIONAL is
		once
			create Result
			Result.set(1,0)
		end
end


