note

	description:

		"Fast implementation of SYMMETRIC_MATRIX"

	author: "Mark Howard"
	last_editor: "$Author: er $"
	date: "$Date: 2014/02/26 $"
	revision: "$Revision: #3 $"
	path: "$File: //rose/source/kernel/infrastructure/math/fast_symmetric_matrix.e $"
	storable_version: ""

class
	FAST_SYMMETRIC_MATRIX
inherit
	SYMMETRIC_MATRIX
		undefine
			sub_symmetric_matrix,
			inverse,
			make_as_inverse,
			make_xtx
		end
create
	make_xtx_weighted,
	make,
	make_from_triangle,
	make_xtx_weighted_attached,
	make_as_inverse,
	matrix_make

feature {NONE} --Initialization

	make_xtx_weighted (a_m: MATRIX;a_w : VECTOR)
			-- Create weighted transposed product (x^t x) from matrix `a_m' and weights `a_w'.
		require
			not_void : a_m /= Void
			same_size : a_w /= Void implies a_w.count = a_m.height
		local
			i, j, k, l_height: INTEGER
			l_weight_squared : DOUBLE
			l_data : SPECIAL[SPECIAL[DOUBLE]]
			l_row,l_result : SPECIAL[DOUBLE]
			l_weights : SPECIAL[DOUBLE]
			l_vector : VECTOR
		do
			make (a_m.width)
			l_data := a_m.data
			l_height := a_m.height
			if a_w /= Void then
				l_weights := a_w.data
			else
				create l_vector.make_identity(l_height)
				l_weights := l_vector.data
			end
			from i := 1 until i > l_height loop
				l_row := l_data[i]
				l_weight_squared := l_weights[i] * l_weights[i]
				from j := 1 until j > height loop
					l_result := data[j]
					from k := 1 until k > j loop
						l_result[k] := l_result[k] + l_row[k] * l_row[j] * l_weight_squared
						k := k + 1
					end
					j := j + 1
				end
				i := i + 1
			end
		end

	make_xtx (a_m: MATRIX)
			-- Create transposed product (x^t x) from matrix `a_m'.
		local
			i, j, k: INTEGER
			l_dot_product: DOUBLE
			l_row,l_row_k : SPECIAL[DOUBLE]
		do
			make (a_m.width)
			from i := 1 until i > width loop
				l_row := data[i]
				from j := 1 until j > i loop
					l_dot_product := 0.0
					from k := 1 until k > a_m.height loop
						l_row_k := a_m.data[k]
						l_dot_product := l_dot_product + l_row_k[i] * l_row_k[j]
						k := k + 1
					end
					l_row[j] := l_dot_product
					j := j + 1
				end
				i := i + 1
			end
		end

	make_xtx_weighted_attached (a_m: MATRIX;a_w : VECTOR)
			-- Create transposed product (x^t x) from matrix `a_m'.
		do
			make (a_m.width)
			make_xtx_weighted(a_m,a_w)
		end

	make_as_inverse(a_other : FAST_SYMMETRIC_MATRIX) is
			--create as inverse of a_other
		local
			m: MATRIX
			i: INTEGER
			lu: LU_DECOMPOSITION
		do
			create m.make (a_other.width,a_other.width)
			from i := 1 until i > a_other.width loop
				m[i, i] := 1
				i := i + 1
			end
			create lu.make (a_other)
			duplicate(lu.solve (m))
		end

feature --Access

	inverse: like Current
			-- Inverse of matrix
		local
			l_matrix: FAST_SYMMETRIC_MATRIX
		do
			create l_matrix.make_as_inverse (Current)
			create Result.make_from_triangle (l_matrix, False)
		end

	sub_symmetric_matrix(a_ixs : ARRAY[INTEGER]) : FAST_SYMMETRIC_MATRIX is
			-- sub symmetric matrix
		local
			l_matrix : MATRIX
		do
			create l_matrix.make_sub_matrix_3(Current,a_ixs, a_ixs)
			create Result.make_from_triangle(l_matrix, False)
		end

end
