note

	description:

		"Signature tagged record for foreign exchange rates"

	author: "Mauricio Bustos"
	last_editor: "$Author: mb $"
	date: "$Date: 2013/06/06 $"
	revision: "$Revision: #15 $"
	path: "$File: //rose/source/kernel/data/market/forex_signature.e $"

class
	FOREX_SIGNATURE

inherit
	CLUSTERED_SIGNATURE[INTEGER,ROSE_DATE,CURRENCY]
		undefine
			context
		redefine
			data_manager,
			set_entity,
			set_field_with_string,
			output_string_by_name,
			date,
			set_date,
			data_fields_string_,
			entity_name,
			date_name
		end

	ROSE_AWARE

	ROSE_LEGACY_UNDEFINED
		undefine
			is_equal
		end

	IEEE_MATH
		undefine
			is_equal
		end

create
	default_create,
	make_ur

create {FOREX_SIGNATURE}
    rose_entity_date_make

feature -- Access

	key_0: INTEGER is
		do
			Result := currency_id
		end

	key_0_name: STRING is
		do
			Result := "currency_id"
		end

	key_1: ROSE_DATE is
		do
			Result := date
		end

	key_1_name: STRING is
		do
			Result := "date"
		end

	entity_name: STRING is
			-- <Precursor>
		do
			Result := "currency"
		end

	date_name: STRING is
			-- <Precursor>
		do
			Result := "date"
		end

	currency_id: INTEGER
	date: ROSE_DATE
	base_currency_id: INTEGER
	source: STRING
	bid: DOUBLE
	ask: DOUBLE
	spot: DOUBLE

	context_name: STRING is
		do
			Result := "RESEARCH"
		end

	expression_tag: STRING is
		do
			Result := "forex_rate"
		end

	forex_server: SQL_DATA_SERVER is
		once
			Result := servers.make_from_repository ("rose_foreign_exchange")
		end

	data_manager: CLUSTERED_TAGGED_ITEM_DATABASE[INTEGER,ROSE_DATE,FOREX_SIGNATURE] is
		once
			create Result.make ("update",forex_server)
		end

	signature_fields_: ROSE_LINEAR_ARRAY[STRING] is
		local
			l_fields: ROSE_DELIMITED_TEXT
		once
			create l_fields.make (fields_,',')
			Result ?= l_fields
			Result.compare_objects
		end

	data_fields_string_: STRING is "bid,ask,spot"

	fields_: STRING is "currency_id,date,base_currency_id,source"

	restriction_statement: STRING is
		do
		end

feature -- Output

	output_string_by_name (a_name: STRING): STRING is
			-- Currency iso code if 'a_name' is "currency_id" or "base_currency_id"
			-- <Precursor> representation otherwise
		local
			l_currency: CURRENCY
		do
			if a_name.is_equal ("currency_id") or a_name.is_equal ("base_currency_id") then
				Result ?= value_by_name (a_name).out
				if Result.is_integer then
					l_currency ?= context.entity_at (context.currency_category_id,Result.to_integer)
					if l_currency /= Void then
						Result := l_currency.iso
					end
				end
			else
				Result := Precursor (a_name)
			end
		end

feature -- Element modification

	set_currency_id (a_value: like currency_id) is
			-- Set 'currency_id' to 'a_value'
		do
			currency_id := a_value
			saved_multi_key := Void
		end

	set_entity_id (a_value: like currency_id) is
			-- Set 'currency_id' to 'a_value'
		do
			set_currency_id (a_value)
			saved_multi_key := Void
		end

	set_entity (a_entity: like entity) is
			-- Set 'entity' to 'a_entity'
		do
			entity := a_entity
			if entity /= Void then
				set_currency_id (entity.handle)
			end
		end

	set_date (a_date: like date) is
			-- Set 'date' to 'a_date'
		do
			date := a_date
			saved_multi_key := Void
		end

	set_base_currency_id (a_value: like currency_id) is
			-- Set 'base_currency_id' to 'a_value'
		do
			base_currency_id:= a_value
			saved_multi_key := Void
		end

	set_source (a_value: like source) is
			-- Set 'source' to 'a_value'
		do
			source := a_value
			saved_multi_key := Void
		end

	set_bid (a_value: like bid) is
			-- Set 'bid' to 'a_value'
		do
			bid := a_value
		end

	set_ask (a_value: like ask) is
			-- Set 'ask' to 'a_value'
		do
			ask := a_value
		end

	set_spot (a_value: like spot) is
			-- Set 'spot' to 'a_value'
		do
			spot := a_value
		end

	set_field_with_string (a_value: STRING; a_field: STRING) is
			-- Set 'a_field' to 'a_value'
		local
			l_currency: CURRENCY
		do
			if a_field.is_equal ("currency_id") then
				l_currency ?= context.currency_by_iso_code (a_value,Void)
				if l_currency /= Void and l_currency.is_valid then
					set_entity (l_currency)
				end
			elseif a_field.is_equal ("base_currency_id") then
				l_currency ?= context.currency_by_iso_code (a_value,Void)
				if l_currency /= Void and l_currency.is_valid then
					set_base_currency_id (l_currency.handle)
				end
			end
		end

feature -- Status

	any_data_at_all: BOOLEAN is
			-- Does this record have any data at all?
		do
			if (is_defined_double (bid) and bid > 0.0) or
				(is_defined_double (ask) and ask > 0.0) or
				(is_defined_double (spot) and spot > 0.0) then
				Result := True
			end
		end

	match_all_data (a_other_signature: FOREX_SIGNATURE): BOOLEAN is
			-- Does 'a_other_signature' match on data with 'Current'?
		local
			i: INTEGER
			l_double, l_other_double: DOUBLE_REF
		do
			Result:=True
			from i:=1 until i> data_fields.count
			loop
				l_double ?= value_by_name (data_fields.item (i))
				l_other_double ?= a_other_signature.value_by_name (data_fields.item (i))
				if l_double /= Void and l_other_double /= Void then
					if not ieee_is_eq (l_double.item,l_other_double.item) then
						Result := False
					end
				end
				i := i + 1
			end
		end

end













