note

	description:

		"Listing for funds other than our internal composite assets"

	author: "Janet Small"
	last_editor: "$Author: er $"
	date: "$Date: 2014/02/26 $"
	revision: "$Revision: #27 $"
	path: "$File: //rose/source/kernel/investment_entities/fund_listing.e $"

class FUND_LISTING
inherit
		INVESTMENT_ENTITY
		undefine
			peer_anchor,
			corelated_entities,
			corelationships,
			ur_targets
		redefine
			make,
			make_with_zone_xrf
		select
			make,
			make_with_zone_xrf
		end
	INVESTMENT_ENTITY
		rename
			make as ie_make,
			make_with_zone_xrf as ie_make_with_zone_xrf
		undefine
			peer_anchor,
			corelated_entities,
			corelationships
		end
	PERSISTENT_FUND_LISTING
		undefine
			context,
			make,
			make_context,
			ur_targets,
			corelated_entities,
			corelationships,
			debug_output
		end
	RMDAT_INSTRUMENT
		undefine
			is_equal
		redefine
			last_price_record,
			price_record
		end
	ATTRIBUTES
		rename
			msci as attributes_msci,
			brim_code as attributes_brim_code,
			cusip as attributes_cusip,
			sedol as attributes_sedol,
			company_name as attributes_company_name,
			xrf_code as attributes_xrf_code

		undefine is_equal end
create

	make,
	make_with_zone_xrf,
	make_anonymous

feature -- Creation

	make (a_context : like context;a_category_id : INTEGER;a_handle : INTEGER;a_start_date,a_end_date : ROSE_DATE) is
			--Initialization
		do
			ie_make(a_context,a_category_id,a_handle,a_start_date,a_end_date)
		end

	make_with_zone_xrf(a_context : like context;a_category_id : INTEGER;a_handle : INTEGER;a_start_date,a_end_date : ROSE_DATE;a_zone : ZONE[LOCATION];a_xrf : INTEGER) 
			--Make from handle setting zone and xrf
		do
			ie_make_with_zone_xrf(a_context,a_category_id,a_handle,a_start_date,a_end_date,a_zone,a_xrf)
		end


feature -- Basic operations

	register  is
			-- Register this entity.
		do
			if zone /= void then
				zone.add_fund_listing(Current)
			end
		end

	corelationships : ROSE_LINEAR_ARRAY[RELATIONSHIP_HANDLER[ENTITY,like Current]] is
			-- Co relationships
		do
			Result ?= Precursor
--  			if not Result.has (context.portfolio_to_listing) then
--  				Result.extend(context.portfolio_to_listing)
--  			end
		end

	corelated_entities (a_name: STRING) : ROSE_ENTITY_HISTORY[ENTITY] is
				-- Correlated entities
		do
--  			if a_name.is_equal(context.portfolio_to_listing.name) then
--  				Result := portfolio_to_listing_co_history
--  			else
				Result := Precursor(a_name)
--			end
		end

feature -- composites

	composites : HASH_TABLE[STRING,FUND_LISTING] is
		local
			l_reader : STRING_PREDICATE
			l_fund_issue : FUND_ISSUE
			l_fund_listing : FUND_LISTING
			i : INTEGER
		once
			create l_reader.make(servers.composite_server,"select distinct brimco || sec from composite_assets_nav")
			if not l_reader.success then
				print("fund_listing::composites - problem creating predicate")
				(create {EXCEPTIONS}).raise("")
			else
				l_reader.retrieve(void,void)
				if not l_reader.success then
					print("fund_listing::composites - problem executing predicate")
					(create {EXCEPTIONS}).raise("")
				else
					create Result.make(l_reader.array.count * 2)
					from l_reader.array.start until l_reader.array.off loop
						if l_reader.array.current_item /= Void then
							l_fund_issue := context.fund_issue_at(attributes_brim_code(l_reader.array.current_item),today)
							if l_fund_issue /= void then
								from i := 1 until i > l_fund_issue.fund_listing_count loop
									l_fund_listing := l_fund_issue.fund_listing(i)
									l_fund_listing.set_is_composite(True)
									Result.put(l_reader.array.current_item,l_fund_listing)
									i := i + 1
								end
							end
						end
						l_reader.array.forth
					end
				end
			end
		end
	composite_records : ROSE_CACHE[COMPOSITE_RECORD] is
		once
			create Result.make(100000,10000)
		end
	composite_record (a_date : ROSE_DATE) : COMPOSITE_RECORD is
		local
			l_interval : ROSE_CLOSED_OPEN_INTERVAL[COMPOSITE_RECORD]
			l_brim_code : STRING
		do
			l_brim_code := fund_issue.brim_code(a_date)
			composite_records.search([l_brim_code,a_date])
			if composite_records.found then
				Result := composite_records.found_item
			else
				if composite_record_intervals /= void then
					l_interval := composite_record_intervals.interval(a_date)
				end
				if l_interval /= Void  then
					Result := l_interval.item
				else
					create l_interval.make(Void, a_date, a_date, 0.0)
					composite_records_cross_sectional_reader.apply(agent add_composite_record(l_interval,?),void,<<a_date.ccyymmdd>>)
					Result := composite_records.item([l_brim_code,a_date])
				end
			end
		end
	composite_records_cross_sectional_reader : SIMPLE_PREDICATE[COMPOSITE_RECORD] is
		once
			create Result.make(servers.composite_server,
							"select brimco || sec as brimsec,brimco,trade_date,capital_balance,revenue_balance,expense_balance,outstanding_shares,currency,published_nav from composite_assets_nav natural inner join" +
							"(select brimco,sec,max(trade_date) as trade_date from composite_assets_nav where trade_date <= ? group by brimco,sec)" +
							"as foo")
		end
	composite_record_interval_table : HASH_TABLE[ROSE_INTERVAL_PARTITION[COMPOSITE_RECORD],STRING] is
		once
			create Result.make(5000)
		end
	composite_record_intervals : ROSE_INTERVAL_PARTITION[COMPOSITE_RECORD] is
		do
			Result := composite_record_interval_table.item(fund_issue.brim_code(void))
		end
	add_composite_record (a_interval : ROSE_CLOSED_OPEN_INTERVAL[COMPOSITE_RECORD];a_record : COMPOSITE_RECORD) is
			-- Add a record for 'a_interval'.
		local
			l_date,l_interim : ROSE_DATE
			l_record : COMPOSITE_RECORD
			l_brim_code : STRING
			l_composite_partition : ROSE_INTERVAL_PARTITION[COMPOSITE_RECORD]
		do
			l_record := a_record.twin
			create l_date.make_ccyymmdd(a_record.trade_date)
			l_brim_code := fund_issue.brim_code_key(l_date)
			if a_interval.start_date.ccyymmdd = a_record.trade_date then
				composite_records.put(l_record,[a_record.brimsec,a_interval.start_date])
			else
				if l_date.plus_n_date(5) < a_interval.start_date then -- more than 5 days old put in interval list
					l_composite_partition := composite_record_interval_table.item(a_record.brimsec)
					if l_composite_partition = Void then
						create l_composite_partition.make
						composite_record_interval_table.put(l_composite_partition,a_record.brimsec)
					end
					l_composite_partition.extend(l_record,l_date)
				else
					from l_interim := l_date until l_interim > a_interval.start_date loop
						composite_records.put(l_record,[a_record.brimsec,l_interim])
						l_interim := l_interim.plus_n_date (1)
					end
				end
			end
			if l_brim_code /= Void and then l_record.brimsec.is_equal(l_brim_code) then
				a_interval.make(l_record,l_date,a_interval.end_date, 0.0)
			end
		end

	is_composite : BOOLEAN is
		do
			composites.do_nothing --sets composite information for all listings as a side effect
			Result := is_composite_saved
		end
	is_composite_saved :  BOOLEAN
	set_is_composite (a_value : BOOLEAN) is
		do
			is_composite_saved := a_value
		end

	is_composite_listing : BOOLEAN is
		local
			l_key : STRING
		do
			l_key := attribute_at(attribute_ids.is_composite_type_id, end_date)
			if l_key /= Void then
				l_key.to_lower
				Result := l_key.is_equal("true")
			end
		end

	is_composite_fund_listing: BOOLEAN is
			--
		do
			Result := is_composite_listing
		end

	rmdat_listing_map : LISTING
	last_price_record(a_date : ROSE_DATE) : RMDAT_PRICE_RECORD is
		do
			if rmdat_listing_map = Void then
				Result := Precursor(a_date)
			else
				Result := rmdat_listing_map.last_price_record(a_date)
			end
		end

	price_record(a_date : ROSE_DATE) : RMDAT_PRICE_RECORD is
			-- Price record as of `a_date'.
		do
			if rmdat_listing_map = Void then
				Result := Precursor(a_date)
			else
				Result := rmdat_listing_map.price_record(a_date)
			end
		end

feature -- Rose best price
	ur_price_target : ROSE_PRICE_SIGNATURE is
		-- ur price target
		once create Result end
	price_report : ROSE_PRICE_REPORT is
			-- Price report
		once create Result.make end
	rmdat_report : RMDAT_PRICE_REPORT is
		-- rmdat report
		once create Result.make end

--	rmdat_price_set_series : DATED_MATRIX is do Result := rmdat_report.matrix(Current) end
end








