indexing

	description:
	"[
			Representation of a contract with a client. This class is not auto generated and
			can be modified to provide different behaviour than the base (auto-generated)
		]"
	author: "$Author: ext-asurelia $"
	last_editor: "$Author: ext-asurelia $"
	keywords: "entity, gcdb, account"
	date: "$Date: 2014/07/10 $"
	revision: "$Revision: #31 $"
	path: "$File: //rose/source/kernel/gcdb/generated/gcdb_account.e $"

class GCDB_ACCOUNT

inherit

	GCDB_ACCOUNT_BASE
		redefine
			code,
			benchmark_history,
			is_retail
		end

create

	make,
	make_anonymous

feature -- Basic operations

	code: STRING_8 is
			-- unique mnemonic identifier for the account
		do
			Result := attribute_value_history (attribute_account_code).first.attribute_value
		end

	is_global: BOOLEAN is
			-- Is this global account
		local
			l_ports: like portfolios_by_account
			l_p1: GCDB_PORTFOLIO
		do
			l_ports := portfolios_by_account
			from
				l_ports.start
			until
				l_ports.off or Result
			loop
				if l_p1 /= Void then
					Result := l_p1.investment_region /= l_ports.current_item.investment_region
				end
				l_p1 := l_ports.current_item
				l_ports.forth
			end
		end

	is_retail: BOOLEAN
			-- is account retail?
		do
			if account_type/=void then
				Result := account_type.is_retail
			end
		end

	is_account_non_equity: BOOLEAN is
			-- Account that has at least one portfolio with an instrument type that
			-- has this field checked will be considered non equity accounts.
			-- Requested by Lucia for use by the fee forecaster.
		local
			i: INTEGER
			p: like portfolios_by_account
		do
			from
				p := portfolios_by_account
				i := p.lower
			variant
				i_decreasing: p.upper - i + 1
			until
				i > p.upper or Result
			loop
				if p.item (i).is_account_non_equity then
					Result := True
				end
				i := i + 1
			end
		end

	is_in_composite_history: ROSE_LINEAR_ARRAY [ENTITY_ATTRIBUTE_RECORD] is
			-- history of the composite membership of the account
		do
			Result := attribute_value_history (attribute_is_in_composite)
		end

	is_gpc_account: BOOLEAN is
			-- latest value of the is_gpc_account attribute
		local
			l_hist: ROSE_LINEAR_ARRAY [ENTITY_ATTRIBUTE_RECORD]
		do
			l_hist := attribute_value_history (attribute_is_gpc)
			if l_hist /= Void and then l_hist.count > 0 then
				from l_hist.start until Result or else l_hist.off loop
					Result := l_hist.current_item.attribute_value.as_lower ~ "yes"
					l_hist.forth
				end
			end
		end

	is_hedged_as_of (a_date: ROSE_DATE): BOOLEAN is
			-- check if an account is hedged on a given date
		do
			is_hedged_pred.retrieve (Void, <<code, a_date, a_date>>)
			if is_hedged_pred.success then
				Result := (not is_hedged_pred.array.is_empty)
			else
				print ("is_hedged_as_of: ERROR retrrieving from DB")
			end
		end

	benchmark_history: ROSE_ENTITY_HISTORY [GCDB_BENCHMARK] is
			-- history of GCDB doesn't have description for this attribute/relationship
		do
			if portfolios_by_account_history.count = 1 and Precursor.is_empty then
				Result := portfolios_by_account_history.first.entity.benchmark_history
			else
				Result := Precursor
			end
		end

	last_known_name: STRING is
			-- Last known descriptive name
		do
			if attribute_value_history (attribute_name).is_empty then
				Result := ""
			else
				Result := attribute_value_history (attribute_name).last.attribute_value
			end
		end

	channel: STRING
			-- Finance/Cliserv classification on how the account came to be managed (Institutional Direct, Institutional via AXA-IM, AXA Funds, Retail as of 12/2008)
			-- Distributor table maintained in GCDB database
			-- This definition should match that of the view GCDB_account_channel in the Common database
		do
			if is_axa.is_true then
				Result := "I2"
			elseif is_retail then
				Result := "RE"
			elseif distributed_by/=void and then distributed_by.code.is_equal("AXA-IM") then
				Result := "II"
			else
				Result := "ID"
			end
		end

	benchmark_with_fallback: GCDB_BENCHMARK
			-- Return account's benchmark if exists
			-- Fallback to the first zonal portfolio with a benchmark if the account doesn't have one
		local
			l_p: ROSE_LINEAR_ARRAY [GCDB_PORTFOLIO]
		do
			if benchmark /= Void then
				Result := benchmark
			else
				l_p := portfolios_by_account
				from
					l_p.start
				until
					l_p.off or Result /= Void
				loop
					if l_p.item_for_iteration.benchmark /= Void then
						Result := l_p.item_for_iteration.benchmark
					end
					l_p.forth
				end
			end
		end


feature {NONE} --Implementation

	is_hedged_pred: STRING_PREDICATE is
			-- from view in the DB
		once
			create Result.make (hub_common_server, "select code from gcdb_account_hedged where code=? and start_date<=? and end_date>=?")
		end

	hub_common_server: SQL_DATA_SERVER is
			-- reference to the Common database
		local
			l_serv: DATABASE_SERVERS
		once
			create l_serv
			Result := l_serv.make_from_repository ("common")
		end

	attribute_account_code: INTEGER = 96
			-- Attribute type id for account code
	attribute_name: INTEGER = 46
			-- Attribute type id for account name
	attribute_is_in_composite: INTEGER = 10335
			-- Attribute type id for is_in_composite
	attribute_is_gpc: INTEGER = 770
			-- Attribute type id for is_gpc_account


end -- class GCDB_ACCOUNT
