note

	description:

		"Common Ancestor for all GCDB Entities"

	keywords: "entity, gcdb"
	author: "it"
	last_editor: "$Author: c4 $"
	date: "$Date: 2014/08/06 $"
	revision: "$Revision: #21 $"
	path: "$File: //rose/source/kernel/gcdb/gcdb_entity.e $"

class	GCDB_ENTITY

inherit

	ENTITY
		export
			{GCDB_ENTITY,GCDB_HOME} all
			{ANY} handle, is_equal, hash_code, category_id,
					start_date, set_start_date,
					end_date, set_end_date,
					as_of_date, set_as_of_date
		undefine
			context
		redefine
			make
		end

	GCDB_AWARE
		export
			{CATEGORY_HANDLER,RELATIONSHIP_HANDLER,GCDB_ENTITY,GCDB_HOME} all
		end

feature {CATEGORY_HANDLER,RELATIONSHIP_HANDLER} -- Initialization

	make (a_context : like context; a_category_id: INTEGER; a_handle: INTEGER; a_start_date, a_end_date: ROSE_DATE) is
			-- creation routine
		local
			l_end_date: like a_end_date
			l_today: like today
		do
			Precursor (a_context,a_category_id,a_handle,a_start_date,a_end_date)
			l_today := today
			if a_end_date <= l_today then
				l_end_date := a_end_date.twin
				l_end_date.add_n_days (-1)
				set_as_of_date (l_end_date)
			else
				set_as_of_date (l_today)
			end
		end

feature {GCDB_ENTITY,GCDB_HOME} -- Attribute Access

	attribute_from_supplier_attribute_id_at (a_supplier_attribute_id: INTEGER; a_date: ROSE_DATE): STRING is
			-- get attribute as a strin given an attribute id
		local
			l_handler : ENTITY_IDENTIFIER
		do
			l_handler := context.identifier (a_supplier_attribute_id)
			if l_handler /= Void then
				Result := l_handler.dated_key (Current, a_date)
			end
		end

	tristate_attribute_value (a_supplier_attribute_id: INTEGER; a_date: ROSE_DATE): EGCDB_TRISTATE is
			-- return tri-state attribute value
		local
			l_string: STRING
		do
			l_string := attribute_from_supplier_attribute_id_at (a_supplier_attribute_id, a_date)
			create Result.make_from_string (l_string)
		end

	date_attribute_value (a_supplier_attribute_id: INTEGER; a_date: ROSE_DATE): ROSE_DATE is
			-- return date attribute value
		local
			l_string: STRING
		do
			l_string := attribute_from_supplier_attribute_id_at (a_supplier_attribute_id, a_date)
			if l_string /= Void then
				create Result.make_w_string (l_string)
			end
		end

	boolean_attribute_value (a_supplier_attribute_id: INTEGER; a_date: ROSE_DATE): BOOLEAN is
			-- return boolean attribute value
		local
			l_string: STRING
		do
			l_string := attribute_from_supplier_attribute_id_at (a_supplier_attribute_id, a_date)
			Result := l_string /= void and then l_string.is_equal ("Yes")
		end

	boolean_attribute_value_history (a_supplier_attribute_id: INTEGER; a_val: BOOLEAN): ROSE_LINEAR_ARRAY [ENTITY_ATTRIBUTE_RECORD] is
			-- return a boolean attribute values with start end dates
		local
			l_hist: ROSE_LINEAR_ARRAY [ENTITY_ATTRIBUTE_RECORD]
		do
			l_hist := attribute_value_history(a_supplier_attribute_id)
			if l_hist /= void then
				create result.make_empty
				from l_hist.start until l_hist.off loop
					if l_hist.current_item.attribute_value /= void then
						if (a_val and l_hist.current_item.attribute_value.is_equal("Yes")) or (not a_val and l_hist.current_item.attribute_value.is_equal("No")) then
							result.extend(l_hist.current_item)
						end
					end
					l_hist.forth
				end
			end
		end

	attribute_value_history (a_supplier_attribute_id: INTEGER): ROSE_LINEAR_ARRAY [ENTITY_ATTRIBUTE_RECORD] is
			-- return attribute values with start end dates
		local
			l_handler: ENTITY_IDENTIFIER
		do
			l_handler := context.identifier(a_supplier_attribute_id)
			if l_handler /= void then
				Result := l_handler.key_history(Current)
			end
		end

feature {GCDB_ENTITY,GCDB_HOME} -- Status

	brackets (a_date: ROSE_DATE): BOOLEAN is
			-- True if `a_date' is inside the period for which current entity is valid
		do
			Result := start_date <= a_date and end_date_greater_than(a_date)
		end

feature {GCDB_ENTITY,GCDB_HOME} -- Implementation

	end_date_greater_than (a_date: ROSE_DATE): BOOLEAN is
			-- true if a_date is earlier than the end date of that entity
		do
			Result := (end_date > a_date or (a_date.is_undetermined and end_date.is_undetermined))
		end

	set_entities_as_of_date (a_entities: ROSE_LINEAR_ARRAY[GCDB_ENTITY]; a_as_of_date: ROSE_DATE) is
			-- change as of date of multiple entities
		local
			l_entity: GCDB_ENTITY
		do
			from
				a_entities.finish
			until
				a_entities.off and a_as_of_date /= void
			loop
				l_entity := a_entities.current_item
				if l_entity /= Void then
					if not l_entity.brackets (a_as_of_date) then
						a_entities.remove
					else
						set_entity_as_of_date (a_entities.current_item, a_as_of_date)
					end
				else
					a_entities.remove
				end
				a_entities.back
			end
		end

	set_entity_as_of_date (a_entity: GCDB_ENTITY; a_as_of_date: ROSE_DATE) is
			-- change as of date of a single entity
		do
			if a_entity /= void and a_as_of_date /= void then
				a_entity.set_as_of_date (a_as_of_date)
			end
		end

end -- class GCDB_ENTITY
