note
	description: "GCDB Portfolio"
	author: "Ivan Tarnavsky"
	keywords: "entity, gcdb, portfolio"
	last_editor: "$Author: sm $"
	date: "$Date: 2013/04/26 $"
	revision: "$Revision: #22 $"
	path: "$File: //rose/source/kernel/gcdb/generated/gcdb_portfolio.e $"

class GCDB_PORTFOLIO

inherit

	GCDB_PORTFOLIO_BASE
		redefine
			code
		end

create

	make,
	make_anonymous

feature -- Access

	code: STRING is
			-- Unique identifier for each portfolio
		do
			Result := attribute_value_history (96).first.attribute_value
		ensure then
			result_not_void: Result /= Void
		end

	first_event_date: ROSE_DATE is
			-- First event date
		local
			l_history: like attribute_value_history
			l_ccyymmdd: STRING
		do
			l_history := attribute_value_history (10763)
			if l_history /= Void and then not l_history.is_empty  then
				l_ccyymmdd := l_history.last.attribute_value
				if l_ccyymmdd/=Void and then l_ccyymmdd.is_integer then
					create Result.make_ccyymmdd (l_ccyymmdd.to_integer)
				end
			end
		end

	last_known_name: STRING is
			-- Last known descriptive name
		do
			if attribute_value_history (46).is_empty then
				Result := ""
			else
				Result := attribute_value_history (46).last.attribute_value
			end
		ensure
			result_not_void: Result /= Void
		end

	is_account_non_equity: BOOLEAN is
			-- Account that has at least one portfolio with an instrument type that
			-- has this field checked will be considered non equity accounts.
			-- Requested by Lucia for use by the fee forecaster.
		do
			if instrument_type /= Void then
				Result := instrument_type.is_account_non_equity
			end
		end

	is_in_composite: BOOLEAN is
			-- True if a portfolio ias set to be included in all composites as of today
		do
			Result := boolean_attribute_value (10335, as_of_date)
		end

	is_in_composite_history: ROSE_LINEAR_ARRAY [ENTITY_ATTRIBUTE_RECORD] is
			-- history of the is_in_composite flag
		do
			Result := attribute_value_history (10335)
		end

	performance_begin_date: ROSE_DATE is
			-- earliest date on which this portfolio got assigned a benchmark
		do
			if benchmark_history /= Void and then benchmark_history.first /= Void then
				Result := benchmark_history.first.start_date
			end
		end

	performance_end_date: ROSE_DATE is
			-- the day after the last day for which the portfolio had a benchmark assigned
		do
			if benchmark_history /= Void and then benchmark_history.last /= Void then
				Result := benchmark_history.last.end_date
			end
		end

end -- GCDB_PORTFOLIO
