 note
	description: "[
		Objects which can retrieve objects from a data server by a string description
		Database queries are a specific case
	]"
	author: "Mark Howard"
	last_editor: "$Author: an $"
	date: "$Date: 2013/10/18 $"
	revision: "$Revision: #57 $"
	path: "$File: //rose/source/kernel/infrastructure/persistence/predicates/general_selection_predicate.e $"

deferred class
	GENERAL_SELECTION_PREDICATE [G]

inherit
	ROSE_PREDICATE [G]
		redefine
			facilitator, container
		select
			global_trace
		end
	LINEAR [G]
	SELECTION_ERROR
	ROSE_SHARED_LOGGER
		rename
			trace as logger_trace
		end

	ROSE_PROCESS_TIMER_COLLECTION_AWARE

feature {NONE} -- Initialization

	make_verbose (a_server: DATA_SERVER; a_selection: STRING; a_context: STRING; a_name: STRING) is
		require
			server_not_void: a_server /= Void
			valid_selection: a_selection /= Void
		do
			make (a_server, a_selection)
			if not success then
				print (a_context + " - Failed to make " + a_name + " in SUPPLIER_CONTEXT.%N")
			end
		end

	make_strict (a_server: DATA_SERVER; a_selection: STRING) is
		require
			server_not_void: a_server /= Void
			valid_selection: a_selection /= Void
		do
			set_fail_on_error (True)
			make (a_server, a_selection)
		end

	make (a_server: DATA_SERVER; a_selection: STRING) is
			-- Create a new predicate.
		require
			server_not_void: a_server /= Void
			valid_selection: a_selection /= Void
		deferred
		ensure
			selection_set: selection = a_selection
		end

	make_extendable_iterator is
		deferred
		end

	make_indexable_iterator is
		deferred
		end

	make_explicit (a_server: DATA_SERVER; a_template: G; a_selection: STRING) is
		require
			server_not_void: a_server /= Void
			valid_selection: a_selection /= Void
		do
			make_with_parameter_template (a_server, a_template, a_selection, Void)
			if fail_on_error and then not success then
				exception_handler.raise ("Failed on make_explicit for " + a_selection)
			end
		end

	make_with_parameter_template (a_server: DATA_SERVER; a_template: G; a_selection: STRING; a_parameter_template: ARRAY [STRING]) is
			-- FIXME -- this should be merged with the version from execution_predicate, maybe to rose_predicate
		require
			a_server_not_void: a_server /= Void
			valid_selection: a_selection /= Void
		do
			debug("make_movies")
				create cam.make(generating_type + "/" + a_selection + "/" + a_server.node + "/" + a_server.database)
			end

			set_selection (a_server, a_selection, a_parameter_template)
			set_template (a_template)
			create iterator.make (Current)
		end

feature {NONE} -- Implementation

	open_close_count: INTEGER

feature -- Access

	is_empty: BOOLEAN is
		do
		end

	after: BOOLEAN is
			-- Has the cursor passed the last row?
		do
			if playback_movie
			then
				Result := cam.next_frame("after").frame_data_as_boolean
				cam.compare_not_needed
			else
				Result := not success or else facilitator.after
			end

			debug("make_movies")
				if record_movie
				then
					cam.record_boolean_movie_frame("after", Result)
				end
			end
		end

feature {NONE} -- Implementation

	container_extendable: BOOLEAN

feature -- Access

	indexable_container: INDEXABLE [G, INTEGER]

	iterator: PREDICATE_ITERATOR [G]

	container: ARRAY [G]

	facilitator: SELECTOR

feature -- Query

	success_one: BOOLEAN
			-- Has the last operation been succesful and does the container
			-- contain exactly one item?
		do
			Result := success and then container /= Void and then container.count = 1
		end

feature -- Basic operations

	set_prefetch (a_prefetch: INTEGER) is
			-- Set prefetch
		do
			if playback_movie then
				cam.compare_integer_movie_frame("set_prefetch", a_prefetch)
			end
			facilitator.set_prefetch (a_prefetch)
			debug("make_movies")
				if record_movie then
					cam.record_integer_movie_frame("set_prefetch", a_prefetch)
				end
			end
		end

	set_container (a_container: ARRAY [G]) is
			-- Set associated container used for query results
		do
			container_set := True
			set_container_internal (a_container)
		end

	set_selection (a_data_server: DATA_SERVER; a_selection: STRING; a_parameter_template: ARRAY [STRING]) is
			-- Set `facilitator' from `a_selection' and `a_parameter_template'.
			-- FIXME -- this should be merged with the version from execution_predicate, maybe to rose_predicate
		require
			valid_server: a_data_server /= Void
			valid_selection: a_selection /= Void
		local
			l_triple: TRIPLE [STRING, ARRAY [STRING], BOOLEAN]
		do
			pool.record_sql_statement_preparation (a_data_server, a_selection)
			data_server := a_data_server
			selection_already_parsed := False
			selection := a_selection
			server_full_path := data_server.node + "." + data_server.database
			if playback_movie then
				l_triple ?= cam.next_frame("set_selection").frame_data
				cam.compare_partial_movie_frame(l_triple.first, a_selection)
				cam.compare_partial_movie_frame(l_triple.second, a_parameter_template)
				-- 'facilitator' stays Void during a playback.
				success := l_triple.third
			else
				facilitator := data_server.new_selector (a_selection, a_parameter_template)
				success := facilitator.success
			end
			debug("make_movies")
				if record_movie then
					create l_triple.make(a_selection, a_parameter_template, success)
					cam.record_movie_frame("set_selection", l_triple)
				end
			end
			if fail_on_error and then not success then
				exception_handler.raise ("Failed to create selector for " + a_selection)
			end
		ensure
			meaningful_selection: fail_on_error implies success
		end

	set_template (a_template: G) is
		deferred
		end

	open (a_argument: ANY; a_arguments: ARRAY [ANY])
			-- set up cursor
		local
			i: INTEGER
			l_string: STRING
			l_triple: TRIPLE[ANY, ARRAY[ANY], BOOLEAN]
		do
			if data_server.definition /= Void then
				data_server.definition.increment_selection_count
				parse_sql_statement (selection, data_server.definition)
			end
			open_close_count := open_close_count + 1
			if playback_movie then
				l_triple ?= cam.next_frame("open").frame_data
				cam.compare_partial_movie_frame(l_triple.first, a_argument)
				cam.compare_partial_movie_frame(l_triple.second, a_arguments)
				success := l_triple.third
			else
				debug("make_movies")
					if record_movie	then
						-- We have to save the arg's BEFORE they are modified by 'facilitator.open'.
						create l_triple.make(a_argument.deep_twin, a_arguments.deep_twin, False)
					end
				end
				facilitator.open (a_argument, a_arguments)
				success := facilitator.success
			end

			debug("make_movies")
				if playback_movie then
					-- Let's modify the string arguments in the same way that 'facilitator.open' does.
					l_string ?= a_argument
					if l_string /= Void then
						l_string.to_c.do_nothing
					end
					if a_arguments /= Void then
						from
							i := a_arguments.lower
						until
							i > a_arguments.upper
						loop
							l_string ?= a_arguments.item(i)
							if l_string/=Void
							then
								l_string.to_c.do_nothing
							end
							i := i + 1
						end
					end
				end

				if record_movie
				then
					l_triple.put_third(success)
					cam.record_movie_frame("open", l_triple)
				end
			end

			if fail_on_error and then not success then
				exception_handler.raise ("open failed on " + selection)
			end
		rescue
			print ("general_selection_predicate::open%N")
		end

	finish is
			-- Iterate to end of linear representation,
			-- that is read all remaining rows.
		do
			from
			until
				after
			loop
				forth
			end
		end

	start
			-- Initialize reading
		do
			index := 0
			forth
		end

	forth is
			-- Read one row.
		deferred
		end

	new_vessel: like empty_vessel is
			-- New empty row object
		deferred
		end

	append (a_argument: ANY; a_arguments: ARRAY [ANY]) is
			-- Execute query and append result to container
		do
			debug("make_movies")
				if record_movie
				then
					cam.start_superframe
				end
			end

			open (a_argument, a_arguments)
			if success then
				from
					start
				until
					off
				loop
					container.force (item, index)
					forth
				end
			elseif fail_on_error then
				exception_handler.raise ("general_selection_predicate::append - Closed by exception%N")
			end

			close

			debug("make_movies")
				if record_movie
				then
					cam.end_superframe
				end
			end
		rescue
			--close
			print ("general_selection_predicate::append - Closed by exception%N")
		end

	retrieve (a_argument: ANY; a_arguments: ARRAY [ANY]) is
			-- Execute query and put result in container
		local
			l_timer_minor: STRING
		do
			-- FIXME -- these timers obviously don't include other database retrieve actions ... expand???
			debug("timers_predicates")
				l_timer_minor := "retrieve: " + generating_type + ": " + selection
				timers.resume(server_full_path, l_timer_minor)
			end

			reset_container
			append (a_argument, a_arguments)

			debug("timers_predicates")
				timers.interrupt(server_full_path, l_timer_minor)
			end
		end

	retrieve_atomic (a_argument: ANY; a_arguments: ARRAY [ANY]) is
			-- Retrieve in one possibly nested transaction
		do
			--data_server.begin_transaction
			retrieve (a_argument, a_arguments)
			--data_server.commit
		end

	close is
			-- Close this predicate
		do
			open_close_count := open_close_count - 1
			if playback_movie
			then
				success := cam.next_frame("close").frame_data_as_boolean
				cam.compare_not_needed
			else
				success := facilitator.success
			end

			debug("make_movies")
				if record_movie
				then
					cam.record_boolean_movie_frame("close", success)
				end
			end

			if fail_on_error and then not success then
				exception_handler.raise ("Close failed on " + selection)
			end

			if not playback_movie
			then
				facilitator.close
			end

		rescue
			print ("general_selection_predicate::close%N")
		end

	close_and_release_command is
		do
			if playback_movie
			then
				success := cam.next_frame("close_and_release_command").frame_data_as_boolean
				cam.compare_not_needed
			else
				facilitator.close_and_release_command
				success := facilitator.success
			end

			debug("make_movies")
				if record_movie
				then
					cam.record_boolean_movie_frame("close_and_release_command", success)
				end
			end

			if not success then
				exception_handler.raise ("Close_and_release failed on " + selection)
			end
		end

	retrieve_one_atomic (a_vessel: like vessel; a_argument: ANY; a_arguments: ARRAY [ANY]) is
			-- Retrieve one in a (possibly nested) transaction.
		require
			valid_vessel: a_vessel /= Void
		do
			data_server.begin_transaction
			retrieve_one (a_vessel, a_argument, a_arguments)
			data_server.commit
		end

	retrieve_one (a_vessel: like vessel; a_argument: ANY; a_arguments: ARRAY [ANY])
			-- Retrieve at most one item
		require
			vessel_not_void: a_vessel /= Void
		local
			l_sextuple: SIX_TUPLE[ANY, ARRAY[ANY], BOOLEAN, like vessel, BOOLEAN, BOOLEAN]
			l_success_1, l_success_2: BOOLEAN
			l_after: BOOLEAN
			l_new_vessel: like vessel
			l_argument_saved: ANY
			l_arguments_saved: ARRAY[ANY]
		do
			index := 0	-- Indicate that nothing was retrieved.
			-- Be careful to note that there are two possible reasons why
			-- nothing was retrieved.  The obvious is that there was nothing
			-- found but the other is that an error occured.  If 'nothing
			-- retrieved' is a valid condition then you must also test 'success'.

			if playback_movie then
				l_sextuple ?= cam.next_frame("retrieve_one").frame_data
				cam.compare_partial_movie_frame(a_argument, l_sextuple.first)
				cam.compare_partial_movie_frame(a_arguments, l_sextuple.second)
				l_success_1 := l_sextuple.third
				l_new_vessel := l_sextuple.fourth
				l_success_2 := l_sextuple.fifth
				l_after := l_sextuple.sixth
			else
				debug("make_movies")
					-- Save these before 'facilitator.open' modifies them.
					l_argument_saved := a_argument.deep_twin
					l_arguments_saved := a_arguments.deep_twin
				end
				facilitator.open (a_argument, a_arguments)
				l_success_1 := facilitator.success
			end
			if l_success_1 then
				if playback_movie then
					-- The next statement has a potential flaw.  If 'a_vessel' had some attributes
					-- previously filled in, the next statement will produce an 'a_vessel' that
					-- has the same values as those that were passed, but not the same objects.
					-- Is this a bug?  Time will tell.
					a_vessel.deep_copy(l_new_vessel)
					success := l_success_2
					if success and then not l_after	then
						index := 1			-- Something was retrieved.
					end
					--facilitator.close		-- No counterpart is needed.
				else
					-- Each line here has a counterpart just above (if playback).
					facilitator.fetch (a_vessel)
					success := facilitator.success
					l_success_2 := success
					if success and then not facilitator.after then
						index := 1			-- Something was retrieved.
					end
					facilitator.close
				end
			else
				success := False
			end
			debug("make_movies")
				if record_movie	then
					create l_sextuple.make(l_argument_saved, l_arguments_saved, l_success_1, a_vessel, l_success_2, l_after)
					cam.record_movie_frame("retrieve_one", l_sextuple)
				end
			end
			if not success and then fail_on_error then
				exception_handler.raise ("retrieve_one failed on " + selection)
			end
		end

	apply (a_action: PROCEDURE [ANY, TUPLE [G]]; a_object: ANY; a_arguments: ARRAY [ANY]) is
			-- Instead of retrieving rows into a container, this features applies 'a_agent' to each row retrieved.
			-- The last two arguments are the same as in 'retrieve'.
			-- WARNING!!!!!!  Please read "On the dangers of using 'apply' in PREDICATE's."
			-- http://prism/modules/wacko/wakka.php?wakka=applydangers
		do
			debug("make_movies")
				if record_movie
				then
					cam.start_superframe
				end
			end

			open (a_object, a_arguments)
			if success then
				iterator.do_all (a_action)
				if not playback_movie
				then
					facilitator.close
				end
			end

			debug("make_movies")
				if record_movie
				then
					cam.end_superframe
				end
			end
		end

	apply_atomic (a_action: PROCEDURE [ANY, TUPLE [G]]; a_object: ANY; a_arguments: ARRAY [ANY]) is
			-- Apply in a single possibly nested transaction.
			-- WARNING!!!!!!  Please read "On the dangers of using 'apply' in PREDICATE's."
			-- http://prism/modules/wacko/wakka.php?wakka=applydangers
		do
			data_server.begin_transaction
			apply (a_action, a_object, a_arguments)
			data_server.commit
		end

	apply_once (a_action: PROCEDURE [ANY, TUPLE [G]]; a_object: ANY; a_arguments: ARRAY [ANY]) is
			-- Same as 'apply' except that it uses 'close_and_release' instead of 'close'.
			-- WARNING!!!!!!  Please read "On the dangers of using 'apply' in PREDICATE's."
			-- http://prism/modules/wacko/wakka.php?wakka=applydangers
		do
			debug("make_movies")
				if record_movie
				then
					cam.start_superframe
				end
			end

			open (a_object, a_arguments)
			if success then
				iterator.do_all (a_action)
				if not playback_movie
				then
					close_and_release_command
				end
			end

			debug("make_movies")
				if record_movie
				then
					cam.end_superframe
				end
			end
		end

	apply_while (a_action: PROCEDURE [ANY, TUPLE [G]]; a_test: FUNCTION [ANY, TUPLE [G], BOOLEAN]; a_object: ANY; a_arguments: ARRAY [ANY]) is
			-- Like 'apply' except that it only retrieves rows while 'a_test' returns True.
			-- The row that fails 'a_test' is also processed (as opposed to 'while_apply').
			-- WARNING!!!!!!  Please read "On the dangers of using 'apply' in PREDICATE's."
			-- http://prism/modules/wacko/wakka.php?wakka=applydangers
		do
			debug("make_movies")
				if record_movie
				then
					cam.start_superframe
				end
			end

			open (a_object, a_arguments)
			if success then
				iterator.do_while (a_action, a_test)
				if not playback_movie
				then
					facilitator.close
				end

			end

			debug("make_movies")
				if record_movie
				then
					cam.end_superframe
				end
			end
		end

	apply_if (a_action: PROCEDURE [ANY, TUPLE [G]]; a_test: FUNCTION [ANY, TUPLE [G], BOOLEAN]; a_object: ANY; a_arguments: ARRAY [ANY]) is
			-- Like 'apply' except that only rows that pass 'a_test' as sent on to 'a_action'.
			-- WARNING!!!!!!  Please read "On the dangers of using 'apply' in PREDICATE's."
			-- http://prism/modules/wacko/wakka.php?wakka=applydangers
		do
			open (a_object, a_arguments)
			if success then
				iterator.do_if (a_action, a_test)
				if not playback_movie
				then
					facilitator.close
				end
			end
		end

	apply_until (a_action: PROCEDURE [ANY, TUPLE [G]]; a_test: FUNCTION [ANY, TUPLE [G], BOOLEAN]; a_object: ANY; a_arguments: ARRAY [ANY]) is
			-- Like 'apply' except that it stops retrieving rows when 'a_test' returns True.
			-- The row that satisfies 'a_test' is also processed (as opposed to 'until_apply').
			-- WARNING!!!!!!  Please read "On the dangers of using 'apply' in PREDICATE's."
			-- http://prism/modules/wacko/wakka.php?wakka=applydangers
		do
			debug("make_movies")
				if record_movie
				then
					cam.start_superframe
				end
			end

			open (a_object, a_arguments)
			if success then
				iterator.do_until (a_action, a_test)
				if not playback_movie
				then
					facilitator.close
				end
			end

			debug("make_movies")
				if record_movie
				then
					cam.end_superframe
				end
			end
		end

	until_apply (a_action: PROCEDURE [ANY, TUPLE [G]]; a_test: FUNCTION [ANY, TUPLE [G], BOOLEAN]; a_object: ANY; a_arguments: ARRAY [ANY]) is
			-- Like 'apply' except that it stops retrieving rows when 'a_test' returns True.
			-- The row that satisfies 'a_test' is NOT processed (as opposed to 'until_apply').
			-- WARNING!!!!!!  Please read "On the dangers of using 'apply' in PREDICATE's."
			-- http://prism/modules/wacko/wakka.php?wakka=applydangers
		do
			debug("make_movies")
				if record_movie
				then
					cam.start_superframe
				end
			end

			open (a_object, a_arguments)
			if success then
				iterator.until_do (a_action, a_test)
				if not playback_movie
				then
					facilitator.close
				end
			end

			debug("make_movies")
				if record_movie
				then
					cam.end_superframe
				end
			end
		end

	while_apply (a_action: PROCEDURE [ANY, TUPLE [G]]; a_test: FUNCTION [ANY, TUPLE [G], BOOLEAN]; a_object: ANY; a_arguments: ARRAY [ANY]) is
			-- Like 'apply' except that it only retrieves rows while 'a_test' returns True.
			-- The row that fails 'a_test' is NOT processed (as opposed to 'apply_while').
			-- WARNING!!!!!!  Please read "On the dangers of using 'apply' in PREDICATE's."
			-- http://prism/modules/wacko/wakka.php?wakka=applydangers
		do
			debug("make_movies")
				if record_movie
				then
					cam.start_superframe
				end
			end

			open (a_object, a_arguments)
			if success then
				iterator.while_do (a_action, a_test)
				if not playback_movie
				then
					facilitator.close
				end
			end

			debug("make_movies")
				if record_movie
				then
					cam.end_superframe
				end
			end
		end

feature {LINEAR_ITERATOR} -- Implementation

	item: G
			-- Current item (from LINEAR)

feature {SUPPLIER_CONTEXT} -- Implementation

	vessel: G

feature {NONE} -- Implementation

	empty_vessel: G
			-- Prototype row item

	reset_container is
			-- Reset container
		do
			if container /= Void then
				container.make_empty
			else
				create container.make_empty
			end
		end

	set_container_internal (a_container: ARRAY [G]) is
			-- Implementation of set_container
		require
			a_container_not_void: a_container /= Void
			a_container_has_valid_lower: a_container.lower = 1
		do
			container := a_container
			indexable_container ?= a_container
			if container.extendible then
				make_extendable_iterator
			else
				make_indexable_iterator
			end
			iterator.set (Current)
		end

	exception_handler: EXCEPTIONS is
			-- Shared exception routines
		once
			create Result
		end

	container_set: BOOLEAN
			-- Has container been created?

	server_full_path: STRING -- FIXME -- I think this can be removed
			-- complete spec for database connection
			-- FIXME -- this should be merged with the version from execution_predicate, maybe to rose_predicate

invariant
	selection_not_void: selection /= Void
	server_full_path_not_void: server_full_path /= Void
end
