note

	description:

		"Agent that holds computed value in various caches. Used in variable calculations"

	author: "Mauricio Bustos"
	last_editor: "$Author: mb $"
	date: "$Date: 2014/03/03 $"
	revision: "$Revision: #27 $"
	path: "$File: //rose/source/kernel/math/generic_access_control_agent.e $"

class
	GENERIC_ACCESS_CONTROL_AGENT

inherit
	FLUSHABLE_GENERIC_ACCESS_CONTROL_AGENT
	ROSE_SHARED_LOGGER

create
	make

feature {NONE} -- Creation

	make (a_compute_agent: like compute_agent) is
			-- Creation procedure
		require
			compute_agent_exists: a_compute_agent /= Void
		do
			compute_agent := a_compute_agent
			system_compute_agents.extend(Current)
		end

feature -- Element modification

	set_compute_agent (a_agent: like compute_agent) is
			-- Set 'compute_agent' to 'a_agent'
		do
			compute_agent := a_agent
		end

	set_cutoff_date(a_cutoff_date: ROSE_DATE) is
			-- See 'cutoff_date'.
		do
			cutoff_date := a_cutoff_date
		end

	set_cutoff_months(a_cutoff_months: INTEGER) is
			-- See 'cutoff_months'.
			-- Positive and negative values mean the same thing.
		do
			cutoff_months := a_cutoff_months.abs
		end

	set_start_date(a_start_date: ROSE_DATE) is
			-- See 'start_date'.
		do
			start_date := a_start_date
		end

	set_end_date(a_end_date: ROSE_DATE) is
			-- See 'end_date'.
		do
			end_date := a_end_date
		end
feature -- Access

	simple_access: ACCESS_CONTROL_AGENT

	start_date: ROSE_DATE --start target range

	end_date: ROSE_DATE --end target range

	cutoff_date: ROSE_DATE		-- Used by CUBE_ACCESS_CONTROL_AGENT:unitized_double (at least).
			-- It limits how far back in time to look for data.
			-- Not used if Void.

	cutoff_months: INTEGER		-- Used by CUBE_ACCESS_CONTROL_AGENT:unitized_double (at least).
			-- It limits how far back in time to look for data.
			-- This gives better control than 'cutoff_date' when multiple dates are going to be used.
			-- Not used if 0.

	unique_id_saved: INTEGER -- this is assumed to be unique throughout all agents
	unique_id: INTEGER is
			--unique id
		do
			if unique_id_saved = 0 then
				unique_id_cell.put(unique_id_cell.item + 1)
				unique_id_saved := unique_id_cell.item
			end
			Result := unique_id_saved
		end
	unique_id_cell: CELL[INTEGER] is
			-- 	once feature holding current number of access control agents
		once
			create Result.put (0)
		end
	compute_agent: FUNCTION[ANY,TUPLE[ANY,ROSE_EXPRESSION[ANY]],BOOLEAN]
	target_filter_agent: FUNCTION[ANY,TUPLE[ROSE_TARGET[ENTITY]],BOOLEAN]
	currency: CURRENCY
			--default currency for this agent
		do
		end

	currency_date: ROSE_DATE
			--date of currency
		do
		end
	as_of(a_date: ROSE_DATE): like Current
	--version of Current on a_date
		do
		end

	has_item(a_target : ANY;a_description : ROSE_EXPRESSION[ANY]) : BOOLEAN
			--is this expression handled by this agent? (Useful for STRING or expanded since signaling values don't help)
			--False by default
		do
		end

	double_value (a_target: ROSE_TARGET[ANY]; a_expression: ROSE_EXPRESSION[ANY]): DOUBLE
	--returned unitized double value
		do
		end

	integer_value (a_target: ROSE_TARGET[ANY]; a_description: ROSE_EXPRESSION[ANY]): INTEGER
			--returned integer value
		do
			Result := a_description.integer_value (a_target)
		end

	string_value (a_target: ROSE_TARGET[ANY]; a_description: ROSE_EXPRESSION[ANY]): STRING
	--returned string value
		do
		end

	unitized_double (a_target: ROSE_TARGET[ANY]; a_description: ROSE_EXPRESSION[ANY]): DATED_UNITIZED_VALUE[DOUBLE]
	--returned unitized double value
		do
		end

	unitized_integer (a_target: ROSE_TARGET[ANY]; a_description: ROSE_EXPRESSION[ANY]): DATED_UNITIZED_VALUE[INTEGER]
	--returned unitized integer value
		do
		end

	units (a_target: ROSE_TARGET[ANY]; a_description: ROSE_EXPRESSION[ANY]): ROSE_UNITS
	--returned units
		do
		end

feature -- Cache Search
	in_double_cache(a_target: ROSE_TARGET[ENTITY]; a_cutoff_date: ROSE_DATE; a_expression: ROSE_EXPRESSION[ANY]): BOOLEAN is
			-- does `a_target' exist in the double cache?
		require
			target_exists: a_target /= Void
			description_exists: a_expression /= Void and then a_expression.variable_description /= Void
		do
			generated_double_cache (a_expression).search(a_target)
			Result := double_cache.found_item.found
		end

	double_cache_found_item: DOUBLE is
			-- the found item in the double cache
		do
			Result := double_cache.found_item.found_item
		end

	in_real_cache(a_target: ROSE_TARGET[ENTITY]; a_expression: ROSE_EXPRESSION[ANY]): BOOLEAN is
			-- does `a_target' exist in the real cache?
		require
			target_exists: a_target /= Void
			description_exists: a_expression /= Void
		do
			generated_real_cache (a_expression).search (a_target)
			Result := real_cache.found_item.found
		end

	real_cache_found_item: REAL is
			-- the found item in the real cache
		do
			Result := real_cache.found_item.found_item
		end

	in_integer_cache(a_target: ROSE_TARGET[ENTITY]; a_expression: ROSE_EXPRESSION[ANY]): BOOLEAN is
			-- does `a_target' exist in the integer cache?
		require
			target_exists: a_target /= Void
			description_exists: a_expression /= Void
		do
			generated_integer_cache (a_expression).search (a_target)
			Result := integer_cache.found_item.found
		end

	integer_cache_found_item: INTEGER is
			-- the found item in the integer cache
		do
			Result := integer_cache.found_item.found_item
		end

	in_string_cache(a_target: ROSE_TARGET[ENTITY]; a_expression: ROSE_EXPRESSION[ANY]): BOOLEAN is
			-- does `a_target' exist in the string cache?
		require
			target_exists: a_target /= Void
			description_exists: a_expression /= Void
		do
			generated_string_cache (a_expression).search (a_target)
			Result := string_cache.found_item.found
		end

	string_cache_found_item: STRING is
			-- the found item in the string cache
		do
			Result := string_cache.found_item.found_item
		end

	in_boolean_cache(a_target: ROSE_TARGET[ENTITY]; a_expression: ROSE_EXPRESSION[ANY]): BOOLEAN is
			-- does `a_target' exist in the boolean cache?
		require
			target_exists: a_target /= Void
			description_exists: a_expression /= Void
		do
			generated_boolean_cache (a_expression).search (a_target)
			Result := boolean_cache.found_item.found
		end

	boolean_cache_found_item: BOOLEAN is
			-- the found item in the boolean cache
		do
			Result := boolean_cache.found_item.found_item
		end

	in_date_cache(a_target: ROSE_TARGET[ENTITY];  a_expression: ROSE_EXPRESSION[ANY]): BOOLEAN is
			-- does `a_target' exist in the date cache?
		require
			target_exists: a_target /= Void
			description_exists: a_expression /= Void
		do
			generated_date_cache (a_expression).search (a_target)
			Result := date_cache.found_item.found
		end

	date_cache_found_item: ROSE_DATE is
			-- the found item in the date cache
		do
			Result := date_cache.found_item.found_item
		end

feature -- Cache Modification

	force_double (a_value: DOUBLE; a_target: ROSE_TARGET[ENTITY]; a_expression: ROSE_EXPRESSION[ANY]) is
			-- force a double value into the double cache
		require
			target_exists: a_target /= Void
			description_exists: a_expression /= Void and then a_expression.variable_description /= Void
		do
			force_cache_any( "force_double", generated_double_cache (a_expression), a_value, a_target,  a_expression )
		end

	force_integer (a_value: INTEGER; a_target: ROSE_TARGET[ENTITY];  a_expression: ROSE_EXPRESSION[ANY]) is
			-- force an integer value into the integer cache
		do
			force_cache_any( "force_integer", generated_integer_cache (a_expression), a_value, a_target,  a_expression )
		end

	force_real (a_value: REAL; a_target: ROSE_TARGET[ENTITY];  a_expression: ROSE_EXPRESSION[ANY]) is
			-- force a real value into the real cache
		do
			force_cache_any( "force_real", generated_real_cache (a_expression), a_value, a_target,  a_expression )
		end

	force_string (a_value: STRING; a_target: ROSE_TARGET[ENTITY];  a_expression: ROSE_EXPRESSION[ANY]) is
			-- force a string value into the string cache
		do
			force_cache_any( "force_string", generated_string_cache (a_expression), a_value, a_target,  a_expression )
		end

	force_boolean (a_value: BOOLEAN; a_target: ROSE_TARGET[ENTITY];  a_expression: ROSE_EXPRESSION[ANY]) is
			-- force a boolean value into the boolean cache
		do
			force_cache_any( "force_boolean", generated_boolean_cache (a_expression), a_value, a_target,  a_expression )
		end

	force_date (a_value: ROSE_DATE; a_target: ROSE_TARGET[ENTITY];  a_expression: ROSE_EXPRESSION[ANY]) is
			-- force a date value into the date cache
		do
			force_cache_any( "force_date", generated_date_cache (a_expression), a_value, a_target,  a_expression )
		end

feature {NONE} -- Implementation
	force_cache_any( a_tag: STRING; a_cache: RP_HASH_TABLE [ANY,ROSE_TARGET[ENTITY]]; a_value: ANY; a_target: ROSE_TARGET[ENTITY];  a_expression: ROSE_EXPRESSION[ANY]) is
			-- common operations on force_XXX actions
		require
			a_tag_not_void: a_tag /= Void
			a_cache_not_void: a_cache /= Void
			a_target_not_void: a_target /= Void
			a_expression_not_void: a_expression /= Void
		do
			debug("trace_cache")
				if attached {COMPANY} a_target.item as l_company and then l_company.xrf_code = 2 then
					print(a_tag + " " + a_expression.name + ", target=" + a_target.target_name + "!" + a_target.date.ccyymmdd.out + "!" + a_target.item.debug_output + ", value=" + void_out( a_value ) + "%N")
				end
			end
			debug("cache_load")
				logger.debugging( a_tag + ": mnemonic=" + a_expression.mnemonic + ", target=" + a_target.target_name + "!" + a_target.date.ccyymmdd.out + "!" + a_target.item.debug_output + ", value=" + void_out( a_value ))
				if a_cache.has( a_target ) then
					logger.warn(a_tag + " cache already contains this target: " + a_target.item.debug_output + "|" + a_target.date.ccyymmdd.out + "; expression mnemonic=" + a_expression.mnemonic + "; new value=" + void_out( a_value.out ) + "; old value=" + void_out( generated_double_cache (a_expression).item( a_target ) ) )
				end
			end
			a_cache.force (a_value, a_target)
		end

	void_out( a_any: ANY ): STRING is
			-- output a safe string even if `a_any' is void
		do
			Result := "<Void>"
			if a_any /= Void then
				Result := a_any.out
			end
		ensure
			result_not_void: Result /= Void
		end

feature -- Cache Flush

	double_cache_flush is
		-- flush the double cache
		do
			debug("cache_load")
				logger.info( "double_cache_flush" )
			end
			create double_cache.make (8)
		end

	integer_cache_flush is
		-- flush the integer cache
		do
			debug("cache_load")
				logger.info( "integer_cache_flush" )
			end
			create integer_cache.make (8)
		end

	real_cache_flush is
		-- flush the real cache
		do
			debug("cache_load")
				logger.info( "real_cache_flush" )
			end
			create real_cache.make (8)
		end

	string_cache_flush is
		-- flush the string cache
		do
			debug("cache_load")
				logger.info( "string_cache_flush" )
			end
			create string_cache.make (8)
		end

	boolean_cache_flush is
		-- flush the boolean cache
		do
			debug("cache_load")
				logger.info( "boolean_cache_flush" )
			end
			create boolean_cache.make (8)
		end

	date_cache_flush is
		-- flush the date cache
		do
			debug("cache_load")
				logger.info( "date_cache_flush" )
			end
			create date_cache.make (8)
		end

	cache_flush is
		-- flush the caches
		do
			double_cache_flush
			integer_cache_flush
			real_cache_flush
			string_cache_flush
			boolean_cache_flush
			date_cache_flush
		end

	system_compute_agents_flush is
			-- Flushes all caches in all GENERIC_ACCESS_CONTROL_AGENT's.
		do
			from
				system_compute_agents.start
			until
				system_compute_agents.off
			loop
				if system_compute_agents.current_item /= Void
				then
					system_compute_agents.current_item.cache_flush
				end
				system_compute_agents.forth
			end
		end

	remove_references
			-- Call this when you want this agent destroyed.  It will remove all references to this object.
		do
			system_compute_agents.prune_all (Current) 
		end
	
feature -- Commit

	commit is
		-- persist the cache entries
		do
			commit_double_cache
			commit_integer_cache
		end

	commit_double_cache is
		-- persist the double cache entries
		local
			l_cache: RP_HASH_TABLE [DOUBLE,ROSE_TARGET[ENTITY]]
			l_expression: ROSE_EXPRESSION[ANY]
		do
			if double_cache /= Void then
				from double_cache.start until double_cache.off loop
					if double_cache.key_for_iteration /= Void then
						l_expression := double_cache.key_for_iteration
						if l_expression /= Void then
							l_cache := double_cache.item_for_iteration
							from l_cache.start until l_cache.off loop
								if target_filter_agent = Void or else target_filter_agent.item([l_cache.key_for_iteration]) then
									l_expression.write_double(l_cache.item_for_iteration,l_cache.key_for_iteration)
								end
								l_cache.forth
							end
							l_cache.clear_all
						end
					end
					double_cache.forth
				end
			end
		end

	commit_integer_cache is
		-- persist the integer cache entries
		local
			l_cache: RP_HASH_TABLE [INTEGER,ROSE_TARGET[ENTITY]]
			l_expression: ROSE_EXPRESSION[ANY]
		do
			if integer_cache /= Void then
				from integer_cache.start until integer_cache.off loop
					if integer_cache.key_for_iteration /= Void then
						l_expression := integer_cache.key_for_iteration
						if l_expression /= Void then
							l_cache := integer_cache.item_for_iteration
							from l_cache.start until l_cache.off loop
								if target_filter_agent = Void or else target_filter_agent.item([l_cache.key_for_iteration]) then
									l_expression.write_integer(l_cache.item_for_iteration,l_cache.key_for_iteration)
								end
								l_cache.forth
							end
							l_cache.clear_all
						end
					end
					integer_cache.forth
				end
			end
		end

feature {NONE} -- Caches

	generated_double_cache (a_expression: ROSE_EXPRESSION[ANY]): RP_HASH_TABLE [DOUBLE,ROSE_TARGET[ENTITY]] is
			-- cache for generated doubles
		require
			description_not_void: a_expression /= Void and then a_expression.variable_description /= Void
		do
			if double_cache = Void then
				create double_cache.make (8)
			end
			double_cache.search (a_expression)
			if double_cache.found then
				Result := double_cache.found_item
			else
				create Result.make (1_000)
				double_cache.extend (Result, a_expression)
				double_cache.search (a_expression)
			end
		ensure
			result_not_void: Result /= Void
		end

	generated_real_cache (a_expression: ROSE_EXPRESSION[ANY]): RP_HASH_TABLE [REAL,ROSE_TARGET[ENTITY]] is
			-- cache for generated reals
		require
			description_not_void: a_expression /= Void
		do
			if real_cache = Void then
				create real_cache.make (8)
			end
			real_cache.search (a_expression)
			if real_cache.found then
				Result := real_cache.found_item
			else
				create Result.make (1_000)
				real_cache.extend (Result, a_expression)
				real_cache.search (a_expression)
			end
		ensure
			result_not_void: Result /= Void
		end

	generated_integer_cache (a_expression: ROSE_EXPRESSION[ANY]): RP_HASH_TABLE[INTEGER,ROSE_TARGET[ENTITY]] is
			-- cache for generated integers
		require
			description_not_void: a_expression /= Void
		do
			if integer_cache = Void then
				create integer_cache.make (8)
			end
			integer_cache.search (a_expression)
			if integer_cache.found then
				Result := integer_cache.found_item
			else
				create Result.make (1_000)
				integer_cache.extend (Result, a_expression)
				integer_cache.search (a_expression)
			end
		ensure
			result_not_void: Result /= Void
		end

	generated_string_cache (a_expression: ROSE_EXPRESSION[ANY]): RP_HASH_TABLE [STRING,ROSE_TARGET[ENTITY]] is
			-- cache for generated strings
		require
			description_not_void: a_expression /= Void
		do
			if string_cache = Void then
				create string_cache.make (8)
			end
			string_cache.search (a_expression)
			if string_cache.found then
				Result := string_cache.found_item
			else
				create Result.make (1_000)
				string_cache.extend (Result, a_expression)
				string_cache.search (a_expression)
			end
		ensure
			result_not_void: Result /= Void
		end

	generated_boolean_cache (a_expression: ROSE_EXPRESSION[ANY]): RP_HASH_TABLE [BOOLEAN,ROSE_TARGET[ENTITY]] is
			-- cache for generated booleans
		require
			description_not_void: a_expression /= Void
		do
			if boolean_cache = Void then
				create boolean_cache.make (8)
			end
			boolean_cache.search (a_expression)
			if boolean_cache.found then
				Result := boolean_cache.found_item
			else
				create Result.make (1_000)
				boolean_cache.extend (Result, a_expression)
				boolean_cache.search (a_expression)
			end
		ensure
			result_not_void: Result /= Void
		end

	generated_date_cache (a_expression: ROSE_EXPRESSION[ANY]): RP_HASH_TABLE [ROSE_DATE,ROSE_TARGET[ENTITY]] is
			-- cache for generated dates
		require
			description_not_void: a_expression /= Void
		do
			if date_cache = Void then
				create date_cache.make (8)
			end
			date_cache.search (a_expression)
			if date_cache.found then
				Result := date_cache.found_item
			else
				create Result.make (1_000)
				date_cache.extend (Result, a_expression)
				date_cache.search (a_expression)
			end
		ensure
			result_not_void: Result /= Void
		end

feature {NONE} -- Implementation

	unitized_cache: HASH_TABLE [HASH_TABLE [ROSE_UNITIZED_VALUE[DOUBLE],ROSE_TARGET[ENTITY]], ROSE_EXPRESSION[ANY]]
	double_cache: RP_HASH_TABLE [RP_HASH_TABLE [DOUBLE,ROSE_TARGET[ENTITY]], ROSE_EXPRESSION[ANY]]
	integer_cache: RP_HASH_TABLE [RP_HASH_TABLE [INTEGER,ROSE_TARGET[ENTITY]], ROSE_EXPRESSION[ANY]]
	real_cache: RP_HASH_TABLE [RP_HASH_TABLE [REAL,ROSE_TARGET[ENTITY]], ROSE_EXPRESSION[ANY]]
	string_cache: RP_HASH_TABLE [RP_HASH_TABLE [STRING,ROSE_TARGET[ENTITY]], ROSE_EXPRESSION[ANY]]
	boolean_cache: RP_HASH_TABLE [RP_HASH_TABLE [BOOLEAN,ROSE_TARGET[ENTITY]], ROSE_EXPRESSION[ANY]]
	date_cache: RP_HASH_TABLE [RP_HASH_TABLE [ROSE_DATE,ROSE_TARGET[ENTITY]], ROSE_EXPRESSION[ANY]]

feature {MODEL_UNIVERSE_MAKER} -- Global agent cache

	system_compute_agents: ROSE_LINEAR_ARRAY[FLUSHABLE_GENERIC_ACCESS_CONTROL_AGENT] is
			-- cache for compute agents
		once
			create Result.make_empty
		ensure
			valid_result: Result /= Void
		end

end
