note

	description:
	"[
		A unary operator in principal is a functional which takes an expression as an argument and produces
		another expression.  This class represents the behavior of that resulting expression.  Other classes
		use this to produce the actual resulting expression. Note that the type of the resulting expression 
		is not the same as the argument ... this is because some operators
		change the type of the target, e.g. integration operators.
	]"
	author: "Mark Howard"
	last_editor: "$Author: mk $"
	date: "$Date: 2012/02/22 $"
	revision: "$Revision: #18 $"
	path: "$File: //rose/source/kernel/math/generic_unary_operator.e $"

deferred class
	GENERIC_UNARY_OPERATOR[reference G,reference H]

inherit
	ROSE_OPERATOR
feature -- Access
	real_value (a_left : ROSE_EXPRESSION[G];a_g : H) : REAL is
			-- Real value
		deferred
		end
	double_value (a_left : ROSE_EXPRESSION[G];a_g : H) : DOUBLE is
			--double value for target using underlying expression
		deferred
		end
	integer_value (a_left : ROSE_EXPRESSION[G];a_g : H) : INTEGER is
			-- Integer value
		deferred
		end
	boolean_value (a_left : ROSE_EXPRESSION[G];a_g : H) : BOOLEAN is
			-- Boolean value
		deferred
		end

	string_value (a_left : ROSE_EXPRESSION[G];a_g : H) : STRING is
			-- String value
		deferred
		end

	double_using_agent (a_left : ROSE_EXPRESSION[G];a_g : H;a_agent : like access_agent_template) : DOUBLE is
			--returned double value
		do
			Result := computed_double(a_left,a_g,a_agent)
		end
	value_in_market_currency(a_left : ROSE_EXPRESSION[G]; a_market: MARKET; a_curr: like currency_anchor; a_g : H; a_date: ROSE_DATE): DOUBLE is
			-- Value in market currency
		deferred
		end
	value_in_currency (a_left : ROSE_EXPRESSION[G]; a_currency: like currency_anchor; a_g : H; a_date : ROSE_DATE) : DOUBLE is
				-- The same implementation as in expressions
		do
			Result := value_in_market_currency(a_left, Void, a_currency, a_g, a_date)
		end
	units (a_left : ROSE_EXPRESSION[G];a_entity : H) : ROSE_UNITS is
			--units
		deferred
		end
	units_in_currency (a_left : ROSE_EXPRESSION[G];a_currency : PERSISTENT_CURRENCY;a_entity : H) : ROSE_UNITS is
		deferred
		end
	computed_double(a_left : ROSE_EXPRESSION[G]; a_g : H; a_agent : like access_agent_template) : DOUBLE is
			-- Computed double
		deferred
		end
	computed_real(a_left : ROSE_EXPRESSION[G]; a_g : H; a_agent : like access_agent_template) : REAL is
		deferred
		end
	computed_integer(a_left : ROSE_EXPRESSION[G]; a_g : H; a_agent : like access_agent_template) : INTEGER is
		deferred
		end
	computed_boolean(a_left : ROSE_EXPRESSION[G]; a_g : H; a_agent : like access_agent_template) : BOOLEAN is
			-- Computed boolean
		deferred
		end
	computed_string(a_left : ROSE_EXPRESSION[G]; a_g : H; a_agent : like access_agent_template) : STRING is
		deferred
		end
	computed_units(a_units : ROSE_UNITS) : ROSE_UNITS
		deferred
		end
--	computed_universe_double(a_left : ROSE_EXPRESSION[G]; a_g : UNIVERSE_TARGET[G]; a_agent : like access_agent_template) : DOUBLE is
--		deferred
--		end
--	computed_universe_real(a_left : ROSE_EXPRESSION[G]; a_g : UNIVERSE_TARGET[G]; a_agent : like access_agent_template) : REAL is
--		deferred
--		end
--	computed_universe_integer(a_left : ROSE_EXPRESSION[G]; a_g : UNIVERSE_TARGET[G]; a_agent : like access_agent_template) : INTEGER is
--		deferred
--		end
--	computed_universe_boolean(a_left : ROSE_EXPRESSION[G]; a_g : UNIVERSE_TARGET[G]; a_agent : like access_agent_template) : BOOLEAN is
--		deferred
--		end
--	computed_universe_STRING(a_left : ROSE_EXPRESSION[G]; a_g : UNIVERSE_TARGET[G]; a_agent : like access_agent_template) : STRING is
--		deferred
--		end
--	computed_universe_date(a_left : ROSE_EXPRESSION[G]; a_g : UNIVERSE_TARGET[G]; a_agent : like access_agent_template) : ROSE_DATE is
--		deferred
--		end
	generated_double(a_left : ROSE_EXPRESSION[G]; a_g : H; a_agent : like access_agent_template) : DOUBLE is
		deferred
		end
	generated_real(a_left : ROSE_EXPRESSION[G]; a_g : H; a_agent : like access_agent_template) : REAL is
		deferred
		end
	generated_integer(a_left : ROSE_EXPRESSION[G]; a_g : H; a_agent : like access_agent_template) : INTEGER is
		deferred
		end
	generated_string(a_left : ROSE_EXPRESSION[G]; a_g : H; a_agent : like access_agent_template) : STRING is
		deferred
		end
	generated_date(a_left : ROSE_EXPRESSION[G]; a_g : H; a_agent : like access_agent_template) : ROSE_DATE is
		deferred
		end

	generated_unitized_value(a_left : ROSE_EXPRESSION[G]; a_target : H; a_agent : like access_agent_template) : DATED_UNITIZED_VALUE[DOUBLE] is
		-- return unitized values
		do
			create Result.make (generated_double (a_left, a_target, a_agent), units (a_left, a_target), today_rdt, today_rdt)
		end

	currency_anchor: PERSISTENT_CURRENCY
			-- currency type, can be overridden in subclasses


feature  --Templates
	access_agent_template : GENERIC_ACCESS_CONTROL_AGENT is
			--template
		do  end
end
