note
	description: "Allows you to select whether to use the zonal or global model data."
	author: "Randy John"
	last_editor: "$Author: myeager $"
	date: "$Date: 2014/10/17 $"
	revision: "$Revision: #4 $"
	path: "$File: //rose/source/kernel/infrastructure/global_model_aware.e $"

class
	GLOBAL_MODEL_AWARE

feature -- Strategy names

	Global_mega: STRING = "GLOBAL_MEGA"
	Global_emerging: STRING = "GLOBAL_EMERGING"
	Global_emerging_small_cap: STRING = "GLOBAL_EMERGING_SMALLCAP"

feature {NONE} -- Strategies

	is_valid_type_for_strategy (a_type: STRING; a_strategy: detachable STRING): BOOLEAN
			-- Is `a_type' a valid type for `a_strategy'?
		require
			a_type_not_void: a_type /= Void
		do
			if a_strategy = Void then
				Result := True
			elseif not attached strategy_map.item (a_strategy) as l_type_regexps then
				Result := True
			else
				Result := l_type_regexps.there_exists (agent {RX_PCRE_REGULAR_EXPRESSION}.recognizes (a_type.as_upper))
			end
		end

	strategy_map: HASH_TABLE [ROSE_LINEAR_ARRAY [RX_PCRE_REGULAR_EXPRESSION], detachable STRING]
			-- Types per strategy
		local
			l_regexp: RX_PCRE_REGULAR_EXPRESSION
		once
			create Result.make (10)
			create l_regexp.make
			l_regexp.compile (".*GBL(FA)?")
			Result.extend (create {ROSE_LINEAR_ARRAY [RX_PCRE_REGULAR_EXPRESSION]}.make_from_array (<<l_regexp>>), Global_mega)
			create l_regexp.make
			l_regexp.compile (".*GBLEM(FA)?")
			Result.extend (create {ROSE_LINEAR_ARRAY [RX_PCRE_REGULAR_EXPRESSION]}.make_from_array (<<l_regexp>>), Global_emerging)
			create l_regexp.make
			l_regexp.compile (".*GBLEMSC(FA)?")
			Result.extend (create {ROSE_LINEAR_ARRAY [RX_PCRE_REGULAR_EXPRESSION]}.make_from_array (<<l_regexp>>), Global_emerging_small_cap)
		ensure
			strategy_map_not_void: Result /= Void
		end

feature -- Basic operations.

	use_global_model
	-- Call this if you want to use the global model data.
	do
		model_is_global_saved.set_item(True)
	end

	use_zonal_model
	-- Call this if you want to use the zonal model data.
	do
		model_is_global_saved.set_item(False)
	end

feature -- Status.

	model_is_global: BOOLEAN
	-- If True, the global model data is being used.
	-- If False, the zonal model data is being used.
	do
		Result := model_is_global_saved.item
	end

feature {NONE} -- Implementation.

	global_model_builder_type: STRING = "GBL"
			-- Sections using this type were generated by global model
			-- builder which keys data by xrf + zone

	model_is_global_saved: BOOLEAN_REF
	-- See 'model_is_global'.
	once
		create Result
		Result.set_item(False)
	end

end
