note

	description:

		"Investment entities issued by governments, such as TBills"

	author: "Janet Small"
	last_editor: "$Author: m5 $"
	date: "$Date: 2013/04/18 $"
	revision: "$Revision: #25 $"
	path: "$File: //rose/source/kernel/investment_entities/government_issue.e $"

class
	GOVERNMENT_ISSUE

inherit
	INSTRUMENT
		undefine
			is_equal
		end
	INVESTMENT_ENTITY
		undefine
			peer_anchor,
			ur_targets
		end
	PERSISTENT_GOVERNMENT_ISSUE
		undefine
			context,
			make,
			make_context,
			ur_targets,
			--ur_targets, --??
			--ur_targets, --??
			corelated_entities,
			corelationships,
			--portfolio_to_listing_co_history,
			debug_output
		end
	LEGACY_INSTRUMENT
	RMDAT_INSTRUMENT
		undefine
			is_equal
		redefine
			price_record,
			last_price_record,
			last_price_record_160
		end

create
	make,
	make_with_zone_xrf,
	make_anonymous

feature -- Basic operations

	name (a_date : ROSE_DATE) : STRING is
			-- Name as of `a_date'.
	  do
		 Result := name_at(a_date)
	  end

	register  is
			-- Register this entity.
		do
			if zone /= void then
				zone.add_government_issue(Current)
			end
		end

	ur_targets : ROSE_LINEAR_ARRAY[ROSE_ENTITY_DATE[GOVERNMENT_ISSUE]] is
		--list of target types for this entity
  		once
 			create Result.make_empty
		end

	asset_class_code (a_date: ROSE_DATE) : STRING is
			-- Asset class code
		do
			Result := "F" -- fixed income
		end

	risk_region_code (a_date: ROSE_DATE) : STRING is
			-- The risk region's ISO code
		local
			s: STRING
		do
			s := brim_code( a_date)
			if s /= void then
				Result := s.substring(1,2)
			end
		end

feature -- from RMDAT_INSTRUMENT

	is_composite: BOOLEAN is
			--<Precursor>
		do
		end

	composite_record (a_date : ROSE_DATE) : COMPOSITE_RECORD
			--<Precursor>
		do
		end

	adjusted_last_price (a_as_of_date,a_date : ROSE_DATE) : REAL is
			--<Precursor>
		do
		end
	return (a_start_date,a_end_date : ROSE_DATE) : REAL is
			--<Precursor>
		do
		end

	price_record (a_date : ROSE_DATE) : RMDAT_PRICE_RECORD is
			-- use rxrf instead of xrf_code, because xrf_code = sxrf
		do
			if price_record_saved = Void or else
				price_record_saved.date = Void or else
				not price_record_saved.date.same_date(a_date) then
				if price_record_saved = Void then
					create price_record_saved
				end
				price_record_saved := market_data.price_record(rxrf,a_date,a_date)
				price_record_saved.set_currency(market_data.currency(rxrf,a_date))
				last_price_record_saved := Void
			end
			Result := price_record_saved
		end

	last_price_record (a_date : ROSE_DATE) : RMDAT_PRICE_RECORD is
			-- Last price record recent to 'a_date'
		local
			l_last_price_record : like last_price_record_saved
			l_date : ROSE_DATE
		do
			l_date := last_market_day(a_date)
			if last_price_record_saved /= void and then last_price_record_saved.date/=Void then
				if a_date >= last_price_record_saved.date and then l_date <= last_price_record_saved.as_of_date then
					Result := last_price_record_saved
					if a_date >= last_price_record_saved.date and then l_date > last_price_record_saved.as_of_date then
						l_last_price_record := market_data.price_record(rxrf,l_date,last_price_record_saved.as_of_date)
						l_last_price_record.set_currency(market_data.currency(rxrf,a_date))
						if l_last_price_record = Void  or else not l_last_price_record.is_valid then
							last_price_record_saved.set_as_of_date(a_date)
						else
							last_price_record_saved := l_last_price_record
						end
					else
						last_price_record_saved := market_data.price_record(rxrf,l_date,Void)
						last_price_record_saved.set_currency(market_data.currency(rxrf,a_date))
					end
				end
			else
				last_price_record_saved := market_data.price_record(rxrf,a_date,Void)
				last_price_record_saved.set_currency(market_data.currency(rxrf,a_date))
			end
			Result := last_price_record_saved
		end

	last_price_record_160 (a_date : ROSE_DATE) : like last_price_record is
			-- last price record (160 mode not implemented for Govt Issue)
		do
			Result := last_price_record (a_date)
		end

	rxrf : INTEGER is
			-- RXRF
		local
			s: STRING
		do
			s := attribute_by_name ("RXRF", constants.now)
			if s /= Void then
				Result := s.to_integer
			end
		end

feature -- Additional attributes
	interest_rate : DOUBLE is
			-- INTEREST RATE
		local
			s: STRING
		do
			s := attribute_by_name ("INTEREST RATE", constants.now)
			if s /= Void then
				Result := s.to_double
			else
				Result := constants.missing_double
			end
		end

	maturity_date : ROSE_DATE is
			-- MATURITY DATE
		local
			s: STRING
		do
			s := attribute_by_name ("MATURITY DATE", constants.now)
			if s /= Void then
				Result := create {ROSE_DATE}.make_from_string_default (s)
			end
		end

	issue_date : ROSE_DATE is
			-- ISSUE DATE
		local
			s: STRING

		do
			s := attribute_by_name ("ISSUE DATE", constants.now)
			if s /= Void then
				Result := create {ROSE_DATE}.make_from_string_default (s)
			end
		end

	face_factor : DOUBLE is
			-- FACE FACTOR
		local
			s: STRING
		do
			s := attribute_by_name ("FACE FACTOR", constants.now)
			if s /= Void then
				Result := s.to_double
			else
				Result := constants.missing_double
			end
		end
end




