indexing

	description: "List of function or command arguments in GUI testing script language"

	author: "Colin Adams"
	last_editor: "$Author: c4 $"
	date: "$Date: 2009/05/20 $"
	revision: "$Revision: #1 $"
	path: "$File: //rose/source/kernel/infrastructure/framework/gui/script_interpreter/gui_script_argument_list.e $"

class GUI_SCRIPT_ARGUMENT_LIST


create

	make, make_empty

feature {NONE} -- Initialization

	make_empty is
			-- Initialize with zero arguments.
		do
			create arguments.make_default
		ensure
			zero_arguments: count = 0
		end

	make (a_arg: GUI_SCRIPT_VALUE) is
			-- Initialize with single argument.
		require
			a_arg_not_void: a_arg /= Void
		do
			make_empty
			arguments.put_last (a_arg)
		ensure
			one_argument: count = 1
		end

feature -- Access

	item (a_n: INTEGER): GUI_SCRIPT_VALUE is
			-- `a_n'-th argument
		require
			a_n_strictly_positive: a_n > 0
			a_n_small_enough: a_n <= count
		do
			Result := arguments.item (a_n)
		ensure
			item_not_void: Result /= Void
			definition: Result = arguments.item (a_n)
		end

	last: GUI_SCRIPT_VALUE is
			-- Last argument
		do
			Result := arguments.last
		ensure
			last_not_void: Result /= Void
			definition: Result = item (count)
		end

feature -- Measurement

	count: INTEGER is
			-- Number of arguments in list
		do
			Result := arguments.count
		ensure
			non_negative: Result >= 0
			definition: Result = arguments.count
		end

feature -- Status report

	is_empty: BOOLEAN is
			-- Are there no arguments?
		do
			Result := (count = 0)
		ensure
			definition: Result = (count = 0)
		end

feature -- Element change

	append (a_arg: GUI_SCRIPT_VALUE) is
			-- Add `a_arg' to end of argument list.
		require
			a_arg_not_void: a_arg /= Void
		do
			arguments.force_last (a_arg)
		ensure
			one_more_argument: arguments.count = old arguments.count + 1
			a_arg_added: arguments.last = a_arg
		end

feature {NONE} -- Implementation

	arguments: !DS_ARRAYED_LIST [GUI_SCRIPT_VALUE]
			-- Actual arguments for command or function call

invariant

	all_arguments_not_void: not arguments.has (Void)

end
