indexing

	description: "Functions that return the j-th item from the i-th row of an RP_DATA_GRID in GUI testing script language"

	author: "Colin Adams"
	last_editor: "$Author: c4 $"
	date: "$Date: 2009/05/20 $"
	revision: "$Revision: #1 $"
	path: "$File: //rose/source/kernel/infrastructure/framework/gui/script_interpreter/gui_script_cell.e $"

class GUI_SCRIPT_CELL

inherit

	GUI_SCRIPT_FUNCTION_CALL
		redefine
			copy_arguments
		end

create

	make

feature {NONE} -- Initialization

	make is
			-- Initialize `Current'.
		do
			name := "cell"
			function_agent := agent cell
		end

feature -- Setting

	copy_arguments is
			-- Set values from `arguments' into `function_agent'.
		local
			l_row, l_col: INTEGER
		do
			if arguments.count = 2 then
				if {l_row_value: GUI_SCRIPT_INTEGER} arguments.item (1) then
					l_row := l_row_value.value.item
					if l_row > 0 then
						if {l_col_value: GUI_SCRIPT_INTEGER} arguments.item (2) then
							l_col := l_col_value.value.item
							if l_col > 0 then
								function_agent.set_operands ([l_row, l_col])
							else
								set_error ("Second argument to cell must be greater than zero.")
							end
						else
							set_error ("Second argument to cell must be an integer.")
						end
					else
						set_error ("First argument to cell must be greater than zero.")
					end
				else
					set_error ("First argument to cell must be an integer.")
				end
			else
				set_error ("Cell requires precisely two arguments.")
			end
		end
	
feature {NONE} -- Implementation

	cell (a_row, a_col: INTEGER): GUI_SCRIPT_TARGET is
			-- `A_col'-th item in `a_row'-th row in `target.target' if it's an RP_DATA_GRID
		require
			target_not_void: target /= Void
		local
			l_row: RP_GRID_NODE [ANY]
		do
			if {l_grid: RP_DATA_GRID [ANY]} target.target then
				if l_grid.row_count >= a_row then
					check
						strictly_positive_row_index: a_row > 0
						-- because of `copy_arguments'
					end
					l_row := l_grid.row (a_row)
					if l_row.count >= a_col then
						check
							strictly_positive_column_index: a_col > 0
							-- because of `copy_arguments'
						end
						create Result.make (l_row.i_th (a_col))
					else
						set_error ("Column index to cell exceeds number of columns in target grid-row.")
					end
				else
					set_error ("Row index to cell exceeds number of rows in target grid.")
				end
			end
		end

end

	
