indexing

	description: "Global variable and function bindings in GUI testing script language"

	author: "Colin Adams"
	last_editor: "$Author: c4 $"
	date: "$Date: 2009/05/20 $"
	revision: "$Revision: #1 $"
	path: "$File: //rose/source/kernel/infrastructure/framework/gui/script_interpreter/gui_script_environment.e $"

class GUI_SCRIPT_ENVIRONMENT

inherit

	ANY

	KL_SHARED_STRING_EQUALITY_TESTER
		export {NONE} all end

create

	make

feature {NONE} -- Initialization

	make (a_application: like application; a_window: like main_window) is
			-- Initialize to minimal set of bindings.
		require
			a_application_not_void: a_application /= Void
			a_window_not_void: a_window /= Void
		do
			create variable_map.make_with_equality_testers (10, Void, string_equality_tester)
			create commands.make_with_equality_testers (10, Void, string_equality_tester)
			create functions.make_with_equality_testers (10, Void, string_equality_tester)
			application := a_application
			main_window := a_window
			set_variable (create {GUI_SCRIPT_TARGET}.make (main_window), "main_window")
			set_variable (create {GUI_SCRIPT_TARGET}.make (main_window.item), "main_widget")
			set_variable (create {GUI_SCRIPT_TARGET}.make (main_window.menu_bar), "main_menu_bar")
		ensure
			application_set: application = a_application
			main_window_set: main_window = a_window
			main_window_defined: has ("main_window")
			main_widget_defined: has ("main_widget")
			menu_bar_for_main_window_defined: has ("main_menu_bar")
		end

feature -- Access

	application: EV_APPLICATION
			-- Vision2 application object

	main_window: EV_TITLED_WINDOW
			-- Principal top-level window

	value (a_name: STRING): GUI_SCRIPT_TARGET is
			-- Value of variable named `a_name'.
		require
			a_name_not_void: a_name /= Void
			has_variable: has (a_name)
		do
			Result := variable_map.item (a_name)
		ensure
			value_not_void: Result /= Void
		end

	has (a_name: STRING): BOOLEAN is
			-- Is variable named `a_name' present?
		require
			a_name_not_void: a_name /= Void
		do
			Result := variable_map.has (a_name)
		end

	new_command (a_name: STRING): ?GUI_SCRIPT_COMMAND is
			-- Copy of command registered to `a_name'
		require
			a_name_not_void: a_name /= Void
		do
			if commands.has (a_name) then
				Result := commands.item (a_name).twin
			end
		end

	new_function (a_name: STRING): ?GUI_SCRIPT_FUNCTION_CALL is
			-- Copy of function registered to `a_name'
		require
			a_name_not_void: a_name /= Void
		do
			if functions.has (a_name) then
				Result := functions.item (a_name).twin
			end
		end

feature -- Element change

	set_variable (a_value: GUI_SCRIPT_TARGET; a_name: STRING) is
			-- Set value of `a_name' to `a_value'.
		require
			a_name_not_void: a_name /= Void
			a_value_not_void: a_value /= Void
			not_already_present: not has (a_name)
		do
			variable_map.force_new (a_value, a_name)
		ensure
			variable_set: has (a_name)
			correct_value: value (a_name) = a_value
		end

	register_command (a_command: GUI_SCRIPT_COMMAND) is
			-- Register `a_command.name' as a command and `a_command' as a clonable implementation for it.
			-- If `a_command.name' is already registered, then the old registration is discared.
		require
			a_command_not_void: a_command /= Void
		do
			commands.force (a_command, a_command.name)
		ensure
			command_registered: commands.has (a_command.name)
			correct_implementation_registered: commands.item (a_command.name) = a_command
		end

	register_function (a_function: GUI_SCRIPT_FUNCTION_CALL) is
			-- Register `a_function.name' as a function and `a_function' as a clonable implementation for it.
			-- If `a_function.name' is already registered, then the old registration is discared.
		require
			a_function_not_void: a_function /= Void
		do
			functions.force (a_function, a_function.name)
		ensure
			function_registered: functions.has (a_function.name)
			correct_implementation_registered: functions.item (a_function.name) = a_function
		end

feature {NONE} -- Implementation

	variable_map: DS_HASH_TABLE [GUI_SCRIPT_TARGET, STRING]
			-- Map of variable names to values

	commands: DS_HASH_TABLE [GUI_SCRIPT_COMMAND, STRING]
			-- Map of command names to commands

	functions: DS_HASH_TABLE [GUI_SCRIPT_FUNCTION_CALL, STRING]
			-- Map of function names to functions

invariant

	variable_map_not_void: variable_map /= Void
	commands_not_void: commands /= Void
	functions_not_void: functions /= Void
	application_not_void: application /= Void
	main_window_not_void: main_window /= Void

end
