indexing

	description: "Functions calls in GUI testing script language"

	author: "Colin Adams"
	last_editor: "$Author: c4 $"
	date: "$Date: 2009/05/20 $"
	revision: "$Revision: #1 $"
	path: "$File: //rose/source/kernel/infrastructure/framework/gui/script_interpreter/gui_script_function_call.e $"

class GUI_SCRIPT_FUNCTION_CALL

inherit

	GUI_SCRIPT_INSTRUCTION

create {NONE}

	default_create
	
feature -- Access

	name: STRING
			-- Name of function

	value: ?GUI_SCRIPT_TARGET
			-- Result of calling `execute' if no error.
	
	arguments: ?GUI_SCRIPT_ARGUMENT_LIST
			-- Actual arguments to command execution

	target: ?GUI_SCRIPT_TARGET
			-- Target for `function_agent'

	function_agent: FUNCTION [ANY, TUPLE, GUI_SCRIPT_TARGET]
			-- Agent implementing function

	environment: ?GUI_SCRIPT_ENVIRONMENT
			-- Execution environment

feature -- Setting

	set_environment (a_env: GUI_SCRIPT_ENVIRONMENT) is
			-- Set `environment' to `a_env'.
		require
			a_env_not_void: a_env /= Void
		do
			environment := a_env
		ensure
			environment_set: environment = a_env
		end
			
	set_target (a_target: like target) is
			-- Set `target' to `_target'.
		require
			a_target_not_void: a_target /= Void
		do
			target := a_target
		ensure
			target_set: target = a_target
		end

	set_arguments (a_args: like arguments) is
			-- Set `arguments' to `a_args'.
		require
			a_args_not_void: a_args /= Void
		do
			arguments := a_args
		ensure
			arguments_set: arguments = a_args
		end

	copy_arguments is
			-- Set values from `arguments' into `function_agent'.
		require
			function_agent_not_void: function_agent /= Void
			arguments_not_void: arguments /= Void
			correct_argument_count: function_agent.open_count = arguments.count
		do
			check
				nothing_to_do: arguments.count = 0
				-- Descendants with a non-zero argument requirement need to redefine
			end
		end

feature -- Basic operations
	
	execute (a_env: GUI_SCRIPT_ENVIRONMENT) is
			-- Execute `Current'.
		do
			is_error := False
			value := Void
			if target = Void then
				set_error ("Target has not been set for " + name)
			elseif arguments = Void then
				set_error ("Arguments have not been set for " + name)
			elseif function_agent.open_count /= arguments.count then
				set_error (name + " expects " + function_agent.open_count.out + " arguments, but was passed " + arguments.count.out)
			else
				copy_arguments
				if not is_error then
					set_environment (a_env)
					function_agent.set_target (Current)
					function_agent.apply
					if function_agent.last_result /= Void then
						value := function_agent.last_result
					end
				end
			end
			if not is_error and then value = Void then
				set_error (name + " returned Void reference")
			end
		ensure then
			value_not_void: not is_error implies value /= Void
		end

invariant

	name_not_void: name /= Void
	function_agent_not_void: function_agent /= Void

end
