indexing

	description: "Functions that return the i_th item from an EV_ITEM_LIST in GUI testing script language"

	author: "Colin Adams"
	last_editor: "$Author: c4 $"
	date: "$Date: 2009/05/20 $"
	revision: "$Revision: #1 $"
	path: "$File: //rose/source/kernel/infrastructure/framework/gui/script_interpreter/gui_script_ith.e $"

class GUI_SCRIPT_ITH

inherit

	GUI_SCRIPT_FUNCTION_CALL
		redefine
			copy_arguments
		end

create

	make

feature {NONE} -- Initialization

	make is
			-- Initialize `Current'.
		do
			name := "i_th"
			function_agent := agent i_th
		end

feature -- Setting

	copy_arguments is
			-- Set values from `arguments' into `function_agent'.
		local
			l_value: INTEGER
		do
			if arguments.count = 1 then
				if {l_integer: GUI_SCRIPT_INTEGER} arguments.item (1) then
					l_value := l_integer.value.item
					if l_value > 0 then
						function_agent.set_operands ([l_value])
					else
						set_error ("Argument to i_th must be greater than zero.")
					end
				else
					set_error ("Argument to i_th must be an integer.")
				end
			else
				set_error ("I_th requires precisely one argument.")
			end
		end
	
feature {NONE} -- Implementation

	i_th (a_index: INTEGER): GUI_SCRIPT_TARGET is
			-- I-th item in `target.target' if it's an EV_WIDGET_LIST
		require
			target_not_void: target /= Void
		local
			l_widget: EV_WIDGET
			l_item: EV_ITEM
		do
			if {l_list: EV_WIDGET_LIST} target.target then
				if l_list.count >= a_index then
					check
						strictly_positive_index: a_index > 0
						-- because of `copy_arguments'
					end
					l_widget := l_list.i_th (a_index)
					create Result.make (l_widget)
				else
					set_error ("Argument to i_th exceeds length of target.")
				end
			elseif {l_tool_list: EV_ITEM_LIST [EV_ITEM]} target.target then
				if l_tool_list.count >= a_index then
					check
						strictly_positive_index: a_index > 0
						-- because of `copy_arguments'
					end
					l_item := l_tool_list.i_th (a_index)
					create Result.make (l_item)
				else
					set_error ("Argument to i_th exceeds length of target.")
				end
			end
		end

end

	
