note

	description:

		"FX Market for GV Currency Exchange Rates"

	author: "Nelson Chui"
	last_editor: "$Author: c4 $"
	date: "$Date: 2014/10/29 $"
	revision: "$Revision: #14 $"
	path: "$File: //rose/source/kernel/market/gv_currency_exchange_market.e $"

class	GV_CURRENCY_EXCHANGE_MARKET

inherit

	MARKET

	DATABASE_SERVER
	ROSE_IEEE_UNDEFINED
	ROSE_AWARE

feature -- Basic operations

	exchange_rate_asof (a_currency_1, a_currency_2: CURRENCY; a_date: ROSE_DATE; a_asof: detachable ROSE_DATE): DOUBLE
			-- Replicate the behavior in Fortran module GET_GV_XRATE;
			-- `a_asof' is not supported by GV, so we return `undefined_double'
		local
			l_date: ROSE_DATE
			l_months_back : INTEGER
			l_xrates_1, l_xrates_2: ROSE_LINEAR_ARRAY [DOUBLE]
			l_xrate_1, l_xrate_2: DOUBLE
			l_exchange_rate_1, l_exchange_rate_2: EXCHANGE_RATE
			l_proxy_iso_1, l_proxy_iso_2: PAIR [STRING, DOUBLE]
		do
			if attached a_asof then
				Result := undefined_double
			elseif a_currency_1 = last_currency_1 and a_currency_2 = last_currency_2 and a_date.ccyymmdd = last_ccyymmdd then
				Result := last_exchange_rate
			elseif a_currency_1.iso ~ "TRY" and a_currency_2.iso ~ "TRL" then
				Result := 0.000001 -- to convert a TRL value into TRY, you multiply by Result.
			elseif a_currency_1.iso ~ "TRL" and a_currency_2.iso ~ "TRY" then
				Result := 1_000_000 -- to convert a TRY value into TRL, you multiply by Result.
			else
				if a_currency_1 = a_currency_2 or else a_currency_1.is_equal (a_currency_2) then
					Result := 1
				else
					Result := undefined_double
					if a_date.date.is_end_of_month then
						l_proxy_iso_1 := proxy_iso (a_currency_1, a_date)
						l_proxy_iso_2 := proxy_iso (a_currency_2, a_date)
						l_xrates_1 := usd_month_end_xrates.item (l_proxy_iso_1.first)
						l_xrates_2 := usd_month_end_xrates.item (l_proxy_iso_2.first)
						from
							l_date := a_date.twin
							l_months_back := 0
						until
							l_xrate_2 > 0 or else l_months_back > 6
						loop
							if l_xrates_1 /= Void then
								l_xrate_1 := l_xrates_1.safe_item (l_date.julian_month) * l_proxy_iso_1.second
							else
								l_exchange_rate_1 := fall_back_rate (a_currency_1,last_weekday (l_date))
								if l_exchange_rate_1 /= Void then
									l_xrate_1 := l_exchange_rate_1.rate
								end
							end
							if l_xrate_1 > 0 then
								if l_xrates_2 /= Void then
									l_xrate_2 := l_xrates_2.safe_item (l_date.julian_month) * l_proxy_iso_2.second
								else
									l_exchange_rate_2 := fall_back_rate (a_currency_2,last_weekday (l_date))
									if l_exchange_rate_2 /= Void then
										l_xrate_2 := l_exchange_rate_2.rate
									end
								end
							end
							if l_xrate_2 > 0 then
								Result := l_xrate_2 / l_xrate_1
							else
								l_date.month_back
								l_months_back := l_months_back + 1
							end
						end
					end
				end
				last_currency_1 := a_currency_1
				last_currency_2 := a_currency_2
				last_ccyymmdd := a_date.ccyymmdd
				last_exchange_rate := Result
			end
		end

feature {NONE} -- Implementation

	last_currency_1, last_currency_2: CURRENCY

	last_ccyymmdd: INTEGER

	last_exchange_rate: DOUBLE

	last_weekday (a_date: ROSE_DATE): ROSE_DATE is
			-- Previous weekday for `a_date',
			-- `a_date' if it's already a weekday
		local
			l_day: INTEGER
		do
			l_day := a_date.day_of_the_week
			if l_day = 1 then
				Result := a_date.plus_n_date(-2)
			elseif l_day = 7 then
				Result := a_date.plus_n_date(-1)
			else
				Result := a_date
			end
		end

	proxy_iso (a_currency: CURRENCY; a_date: ROSE_DATE): PAIR [detachable STRING, DOUBLE]
			-- Alternate currency for `a_currency' on `a_date'
			-- These are usually financial / market currency 
			-- relationships and specific to Global Vantage
		local
			l_iso: detachable STRING
			l_factor: DOUBLE
		do
			l_iso := a_currency.iso
			l_factor := 1.0
			if l_iso ~ "BEL" then
				l_iso := "BEF"
			elseif l_iso ~ "LUL" then
				l_iso := "LUF"
			elseif l_iso ~ "BEC" then
				l_iso := "BEF"
			elseif l_iso ~ "ZAL" then -- Fortran code forces all ZAR xrates into ZAL xrates; thus we can just look for ZAR xrates effectively
				l_iso := "ZAR"
			elseif l_iso ~ "ZAF" then
				l_iso := "ZAR"
			elseif l_iso ~ "VEF" and a_date.ccyymm <= 200712 then
				l_iso := "VEB"
				l_factor := 1000.0
			elseif l_iso ~ "MXN" and a_date.ccyymm <= 199212 then
				l_iso := "MXP"
				l_factor := 1000.0
			end
			create Result.make (l_iso, l_factor)
		ensure
			proxy_sio_attached: Result /= Void
		end

	usd_month_end_xrates: HASH_TABLE [ROSE_LINEAR_ARRAY [DOUBLE], STRING] is
			-- usd month end xrates
		local
			l_predicate: ANONYMOUS_PREDICATE
			l_iso, l_last_iso: detachable STRING
			l_julian_month: INTEGER_REF
			l_xrate: DOUBLE_REF
			l_xrates: ROSE_LINEAR_ARRAY [DOUBLE]
			l_current_julian_month: INTEGER
		once
			create Result.make (300)
			l_current_julian_month := (create {ROSE_DATE}.make_now).julian_month
			servers.global_vantage_server.begin_transaction
			create l_predicate.make_using_server (servers.global_vantage_server,
												  "select iso, julian_month, xrate from usd_xrate where xrate > 0.0 and xrate is not NULL order by iso, julian_month")
			if not l_predicate.success then
				(create {EXCEPTIONS}).raise ("Error creating anonymous predicate")
			end
			l_predicate.retrieve (Void, Void)
			if not l_predicate.success then
				(create {EXCEPTIONS}).raise ("Error retrieving anonymous predicate")
			end
			servers.global_vantage_server.commit
			from l_predicate.array.start
			until l_predicate.array.after
			loop
				l_iso ?= l_predicate.array.current_item.item (1)
				l_julian_month ?= l_predicate.array.current_item.item (2)
				l_xrate ?= l_predicate.array.current_item.item (3)
				if l_last_iso = Void or else not l_last_iso.is_equal (l_iso) then
					create l_xrates.make (1, l_current_julian_month)
					Result.extend (l_xrates, l_iso)
					l_last_iso := l_iso
				end
				l_xrates.put (l_xrate.item, l_julian_month.item)
				l_predicate.array.forth
			end
		end

feature {NONE} -- default rate when GV not available

	fall_back_rate(a_currency: CURRENCY; a_date: ROSE_DATE) : EXCHANGE_RATE is
			-- fall back rate
		local
			l_idc: IDC_CURRENCY_MARKET
			l_msci: MSCI_CURRENCY_MARKET
			l_extel: EXTEL_CURRENCY_MARKET
			l_date_iter, l_min_date: ROSE_DATE
			l_double : DOUBLE
		do
			l_min_date:=a_date.plus_n_date(-6)
			create l_idc
			from l_date_iter:=a_date until l_date_iter>l_min_date
			loop
				l_double:=l_idc.exchange_rate(a_currency,usd,l_date_iter)
			end
			if not is_defined_double(l_double) then
				create l_msci
				from l_date_iter:=a_date until l_date_iter>l_min_date
				loop
					l_double:=l_msci.exchange_rate(a_currency,usd,l_date_iter)
				end
			end
			if not is_defined_double(l_double) then
				create l_extel
				from l_date_iter:=a_date until l_date_iter>l_min_date
				loop
					l_double:=l_extel.exchange_rate(a_currency,usd,l_date_iter)
				end
			end
			if is_defined_double(l_double) then
				create Result.make(a_currency, usd, l_double, a_date)
			end
		end

	usd: CURRENCY
			-- U.S. Dollar
		once
			Result := context.usd
		end

end
