note

	description:
	"[
		A fast set inplemented with a HASH_TABLE.
		Only the HASH_TABLE key and value are always the same.  I tried to inherit from HASH_TABLE [G, INTEGER] but only got into trouble.
	]"
	author: "Randy John"
	last_editor: "$Author: er $"
	date: "$Date: 2013/06/18 $"
	revision: "$Revision: #24 $"
	path: "$File: //rose/source/kernel/infrastructure/containers/hashed_set.e $"
	storable_version: ""

class
	HASHED_SET [G -> HASHABLE]

inherit
	SUBSET [G]
		rename
			merge as merge_table,
			wipe_out as clear_all,
			disjoint as set_disjoint
		undefine
			changeable_comparison_criterion,
			is_equal,
			copy
		redefine
			extend,
			put,
			symdif,
			set_disjoint
		select
			put,
			extend
		end
-- We inherit twice from this class as a trick to make this
-- class compile both in compatible and experimental modes.
	SUBSET [G]
		rename
			merge as merge_table,
			disjoint as set_disjoint
		undefine
			changeable_comparison_criterion,
			is_equal,
			copy
		redefine
			extend,
			put,
			symdif,
			set_disjoint
		end
	HASH_TABLE [G, G]
		rename
			put as hash_table_put,
			force as hash_table_force,
			make as hash_table_make,
			extend as ht_extend
		export
			{ANY} start, forth, off, key_for_iteration, item_for_iteration, remove, current_keys, search, found, wipe_out
			{HASHED_SET} all
		redefine
			occurrences,
			prune
		select
			clear_all,
			has
		end

create
	make, make_with_size, make_by_array

create {HASHED_SET}
	hash_table_make

feature {NONE} -- Initialization

	make is
			-- Creation procedure
		do
			hash_table_make (initial_capacity)
		end


	make_with_size (n: INTEGER) is
			-- create with specified size
		do
			hash_table_make (n)
		end

	make_by_array (a_array: ARRAY [G]) is
			--- create by an array
		local
			i: INTEGER
		do
			if a_array = Void or else a_array.count = 0 then
				hash_table_make (1)
			else
				hash_table_make (a_array.count)
				from i := 1 until i > a_array.count loop
					put (a_array[i])
					i := i + 1
				end
			end
		end

feature -- Access

	occurrences (a_value: G): INTEGER is
			-- Occurences of 'a_value' in this set
		do
			if has (a_value) then
				Result := 1
			else
				Result := 0
			end
		end

	duplicate (n:INTEGER): like Current is
			-- New structure containing min (`n', `count')
			-- items from current structure
		local
			l_count: INTEGER
			old_iteration_position: INTEGER
		do
			old_iteration_position := iteration_position
			create Result.make
			from
				start
				l_count := 0
			until
				off or l_count>=n
			loop
				Result.extend(item_for_iteration)
				l_count := l_count + 1
				forth
			end
			iteration_position := old_iteration_position
		end

	is_subset (a_other: attached like Current): BOOLEAN is
			-- This was lifted from LINKED_SET.
			-- All 'item' replaced with 'item_for_iteration'.
			-- Is current set a subset of `a_other'?
		do
			if not a_other.is_empty and then count <= a_other.count then
				from
					start
				until
					off or else not a_other.has(item_for_iteration)
				loop
					forth
				end;
				if off then Result := true end
			elseif is_empty then
				Result := true
			end
		end

	set_disjoint (a_other: attached like Current): BOOLEAN is
			-- Do current set and `a_other' have no items in common?
			-- (This feature is redefined in all descendants. The default
			-- implementation given here is only to stay backward-compatible
			-- with older versions of EiffelBase.)
			--
			-- Note: temporarily renamed as `set_disjoint' to resolve
			-- name clash with HASH_TABLE.disjoint in 7.1-experimental.
		local
			temp: like Current
		do
			if not is_empty then
				temp := duplicate (count)
				temp.intersect (a_other)
				Result := temp.is_empty
			else
				Result := True
			end
		end

feature -- Basic operations

	prune (a_value: G) is
			-- Remove one occurrence of `a_value' if any.
		do
			remove (a_value)
		end

	extend, put (a_value: G) is
			-- Insert a new occurrence of `a_value'
		require else
				-- Needed in experimental mode, because `extendible' does
				-- not return true in this compilation mode.
			extendible: True
		do
			hash_table_put (a_value, a_value)
		end

	force (a_value: G) is
			-- Insert a new occurrence of `a_value'
		do
			hash_table_force (a_value, a_value)
		end

	subtract (a_other: attached like Current) is
			-- This was lifted from LINKED_SET.
			-- All 'item' replaced with 'item_for_iteration'.
			-- Remove all items also in `a_other'.
		do
			if not (a_other.is_empty or is_empty) then
				from
					start
					a_other.start
				until
					off
				loop
					if a_other.has(item_for_iteration) then
						remove (item_for_iteration)
					else
						forth
					end
				end
			end
		end

	merge_table (a_other: attached like Current) is
			-- This was lifted from LINKED_SET.
			-- All 'item' replaced with 'item_for_iteration'.
			-- Add all items of `a_other'.
		do
			from
				a_other.start
			until
				a_other.off
			loop
				extend (a_other.item_for_iteration)
				a_other.forth
			end
		end

	symdif (a_other: attached like Current) is
			-- Remove all items also in `a_other', and add all
			-- items of `a_other' not already present.
			-- (This feature is redefined in all descendants. The default
			-- implementation given here is only to stay backward-compatible
			-- with older versions of EiffelBase.)
		local
			temp: like Current
		do
			temp := duplicate (count)
			temp.intersect (a_other)
			merge (a_other)
			subtract (temp)
		end

	intersect (a_other: attached like Current) is
			-- This was lifted from LINKED_SET.
			-- All 'item' replaced with 'item_for_iteration'.
			-- Remove all items not in `a_other'.
			-- No effect if `a_other' is `empty'.
		do
			if not a_other.is_empty then
				from
					start
					a_other.start
				until
					off
				loop
					if a_other.has (item_for_iteration) then
						forth
					else
						remove (item_for_iteration)
					end
				end
			else
				wipe_out
			end
		end

feature {NONE} -- Implementation

	initial_capacity: INTEGER is 100
			-- Initial capacity

end
