indexing

	description: "[
				  This object provides a bridge to help migration to the 
				  windows capital action database.  Access to the legacy adjustments 
				  that were read from VMS can be switched on and off.
				  ]"
	author: "Mauricio Bustos"
	last_editor: "$Author: zs $"
	date: "$Date: 2010/04/08 $'"
	revision: "$Revision: #11 $"
	archive: "$File: //rose/source/kernel/data/market/hdp_adjustments_bridge.e $"

class
	HDP_ADJUSTMENTS_BRIDGE

inherit
	ARRAY [LINKED_LIST [RAW_ADJUSTMENT]]
		rename
			make as array_make
		redefine
			valid_index,
			item,
			infix "@"
		end

create

	make

feature -- Creation

	make (a_zone: ZONE[LOCATION]) is
			-- Create the bridge
		require
			valid_zone: a_zone /= Void
		do
			array_make (1, 1) -- Note array elements are currently not created.  But area has to be created to avoid ARRAY invariant exceptions.
			zone := a_zone
			market_data := zone.market_data
			hdp := market_data.hdp
		end

feature -- Access

	valid_index (a_index: INTEGER): BOOLEAN is
			--every index is valid
		do
			Result := True
		end

	item alias "[]", infix "@" (i: INTEGER): LINKED_LIST[RAW_ADJUSTMENT] is
			-- item
		local
			l_hdp_jul_index_low, l_hdp_jul_index_high, l_julian, l_minmax: INTEGER
			l_listing: LISTING
			l_adjustments: ROSE_ADJUSTMENT_HISTORY[LISTING]
			l_raw: RAW_ADJUSTMENT
			l_highdate_index, l_lowdate_index: INTEGER
			l_backadj: DOUBLE
			l_split_factor: DOUBLE
		do
			l_listing := zone.listing (i)
			if l_listing = last_listing then
				Result := last_raw_adjustments
			elseif l_listing /= Void and then l_listing.is_valid then
				-- l_adjustments := l_listing.new_adjustment_history_os -- use for CA migration S O zones
				l_adjustments := l_listing.new_adjustment_history -- production
				create Result.make
				from l_adjustments.finish until l_adjustments.off loop
					if hdp.jul_indexes.valid_index (l_adjustments.current_item.start_date.rose_julian_day) or
						l_adjustments.current_item.start_date.rose_julian_day < 0 then
						create l_raw
						Result.extend (l_raw)
						if l_backadj = 0.0 or l_adjustments.current_item.split_factor = 0 then
							l_backadj := 1.0
							l_split_factor := 0.0
						else
							l_backadj := l_backadj * l_adjustments.current_item.split_factor
							l_split_factor := l_adjustments.current_item.split_factor
						end
						l_raw.set_backadj (l_backadj.truncated_to_real)
						if l_adjustments.current_item.start_date.rose_julian_day > 0 and
							hdp.jul_indexes.valid_index (l_adjustments.current_item.start_date.rose_julian_day) and
							 hdp.jul_indexes.item (l_adjustments.current_item.start_date.rose_julian_day) = 0 then
							from l_lowdate_index := l_adjustments.current_item.start_date.rose_julian_day
							until l_raw.lowdate_index > 0 or l_lowdate_index < 0 loop
								l_raw.set_lowdate_index (hdp.jul_indexes.item (l_lowdate_index))
								l_lowdate_index := l_lowdate_index - 1
							end
						else
							l_raw.set_lowdate_index (hdp.jul_indexes.item (l_adjustments.current_item.start_date.rose_julian_day.max (hdp.jul_indexes.lower).min (hdp.jul_indexes.upper)))
						end
						if not hdp.jul_indexes.valid_index (l_adjustments.current_item.end_date.rose_julian_day) then
							-- Friday, set to Sunday for weekend processing
							from l_highdate_index := hdp.jul_indexes.upper
							until l_raw.highdate_index > 0 or l_highdate_index < 0 loop
								l_raw.set_highdate_index (hdp.jul_indexes.item (l_adjustments.current_item.end_date.rose_julian_day.min (l_highdate_index).max (hdp.jul_indexes.lower)))
								l_highdate_index := l_highdate_index - 1
							end
							--if l_raw.highdate_index <= 0 then
							--	l_raw.set_highdate_index (0)
							--else
							--	l_raw.set_highdate_index (l_raw.highdate_index + 3)
							--end
						else
							l_hdp_jul_index_low := hdp.jul_indexes.lower
							l_hdp_jul_index_high := hdp.jul_indexes.upper
							l_julian := l_adjustments.current_item.end_date.rose_julian_day
							l_minmax := l_julian.min (l_hdp_jul_index_high).max (l_hdp_jul_index_low)
							l_raw.set_highdate_index (hdp.jul_indexes.item (l_minmax))
						end
						l_raw.set_rxrf (i)
						l_raw.set_split_factor (l_split_factor.truncated_to_real)
						l_raw.set_zone (zone.mnemonic)
					end
					l_adjustments.back
				end
				last_listing := l_listing
				last_raw_adjustments := Result
			end
		end

feature {NONE} -- Implementation

	zone: ZONE[LOCATION]
			-- The zone that is being served
	market_data: MARKET_DATA
	hdp: HDP
	last_listing: LISTING
			-- The last listing that data was retrieved for (cacheing)
	last_raw_adjustments: LINKED_LIST[RAW_ADJUSTMENT]
			-- The last set of raw adjustments that were retrieved for last_listing (cacheing)

end
