indexing
	description: "Date intervaled representation of holdings for a single entity"
	author: "Michael Zhang"
	date: "$Date: 2010/04/28 $"
	revision: "$Revision: #6 $"

class
	HOLDING_HISTORY [G -> INVESTMENT_ENTITY, H -> HOLDING_INTERVAL [G]]

inherit
	ROSE_INTERVAL_HISTORY [G, H]
		rename
			make_empty as linear_make_empty
		end

create
	make_empty

create {HOLDING_HISTORY}
	linear_make_empty

feature -- Initialization
	make_empty (a_entity : G) is
			-- Initialize with no holdings.
		do
			linear_make_empty
			entity := a_entity
		end

feature -- Access
	units_at (a_date : ROSE_DATE) : DOUBLE is
		local
			l_holding : like holding_template
		do
			l_holding := witness_interval (a_date)
			if l_holding /= Void then
				Result := l_holding.units
			end
		end

	units_vector_at (a_dates : ARRAY [ROSE_DATE]) : VECTOR is
		require
			a_dates_valid : a_dates /= Void and then a_dates.count > 0
		local
			l_date : ROSE_DATE
			i, l_lower : INTEGER
			l_units : DOUBLE
		do
			create Result.make (a_dates.count)
			l_lower := a_dates.lower
			from i := a_dates.lower until i > a_dates.upper loop
				l_date := a_dates.item (i)
				l_units := units_at (l_date)
				Result.put (l_units, i - lower + 1)
				i := i + 1
			end
		end

	entity : G
			-- Entity holding `Current'.

feature -- Modify
	multiply (a_factor : DOUBLE) is
		do
			from start until off loop
				current_item.multiply (a_factor)
				forth
			end
		end

	cloned : like Current is
			-- New like `Current' with twinned items.
		do
			create Result.make_empty (entity)
			from start until off loop
				Result.extend (current_item.twin)
				forth
			end
		end

feature -- Templates
	holding_template : H is do  end

end
