indexing
	description: "A collection of interval histories of investment_entities."
	author: "Michael Zhang"
	date: "$Date: 2010/04/28 $"
	revision: "$Revision: #4 $"

class
	HOLDINGS [G -> INVESTMENT_ENTITY, F -> HOLDING_INTERVAL [G], H -> HOLDING_HISTORY [G, F] create make_empty end]

inherit
	ROSE_INDEXED_ARRAY [H, KEY_PAIR [INTEGER, INTEGER]]
		redefine
			infix "+"
		end

create
	make,
	make_empty,
	make_from_array,
	make_all

feature -- Modify
	multiply (a_factor: DOUBLE) is
			-- Multiply untis of all items in `Current' by `a_factor'.
		do
			from start until off loop
				if current_item /= Void then
					current_item.multiply (a_factor)
				end
				forth
			end
		end

	add_holding (a_holding : like holding_template) is
		local
			l_history: like holding_history_template
		do
			if a_holding /= Void then
				if has_key (key_from_holding (a_holding.entity)) then
					l_history := item_at (key_from_holding (a_holding.entity))
					if l_history /= void then
						l_history.add_interval (a_holding)
					end
				else
					create l_history.make_empty (a_holding.entity)
					l_history.extend (a_holding)
					extend (l_history, key_from_holding (a_holding.entity))
				end
			end
		end

	add_holding_history (a_holding_history : like holding_history_template) is
		local
			l_history : like holding_history_template
		do
			if a_holding_history /= void then
				if has_key (key_from_holding (a_holding_history.entity)) then
					l_history := item_at (key_from_holding (a_holding_history.entity))
					if l_history /= void then
						l_history.add_history (a_holding_history)
					end
				else
					extend (a_holding_history, key_from_holding (a_holding_history.entity))
				end
			end
		end

	add_holdings (a_holdings: like Current) is
		do
			if a_holdings /= void then
				from
					a_holdings.start
				until
					a_holdings.off
				loop
					add_holding_history (a_holdings.current_item)
					a_holdings.forth
				end
			end
		end

feature -- Infix
	infix "+" (a_other: like Current): like Current is
		do
			Result := cloned
			Result.add_holdings (a_other)
		end

	infix "*" (a_factor: DOUBLE): like Current is
		do
			Result := cloned
			Result.multiply (a_factor)
		end

	cloned: like Current is
		do
			create Result.make (lower, upper)
			from
				start
			until
				off
			loop
				if current_item /= void then
					Result.put (current_item.cloned, current_key, index)
				end
				forth
			end
		end
feature -- Access
	key_from_holding (a_holding : G) : KEY_PAIR [INTEGER, INTEGER]
			-- Create a key from `a_holding'.
		require
			a_holding_not_void : a_holding /= Void
		do
			create Result.make (a_holding.category_id, a_holding.handle)
		end

	has_holding_at(a_entity : G; a_date : ROSE_DATE) : BOOLEAN is
		local
			l_history: like holding_history_template
		do
			if a_entity /= Void then
				l_history := item_at (key_from_holding (a_entity))
				Result := l_history /= Void and then l_history.units_at(a_date) > 0
			end
		end

	holding_at, units_at(a_entity : G; a_date : ROSE_DATE) : DOUBLE is
		local
			l_history: like holding_history_template
		do
			if a_entity /= Void then
				l_history := item_at (key_from_holding (a_entity))
				if l_history /= Void then
					Result := l_history.units_at (a_date)
				end
			end
		end

	holding_history (a_entity : like holding_entity_template) : like holding_history_template is
		do
			if a_entity /= Void then
				Result := item_at (key_from_holding (a_entity))
			end
		end

	holding_matrix (a_entities: ARRAY [G]; a_dates: ARRAY [ROSE_DATE]): HASHED_MATRIX [ROSE_DATE, G] is
		require
			a_entities_not_void : a_entities /= Void
			a_dates_not_void : a_dates /= Void
		local
			i, j, l_entity_lower, l_date_lower: INTEGER
			l_date: ROSE_DATE
			l_entity: G
			l_history: like holding_history_template
			l_units: DOUBLE
		do
			create Result.make (a_dates.count, a_entities.count)
			l_entity_lower := a_entities.lower
			l_date_lower := a_dates.lower
			from
				i := 1
			until
				i > a_entities.count
			loop
				l_entity := a_entities.item (l_entity_lower + i - 1)
				l_history := item_at (key_from_holding (l_entity))
				if l_history /= void then
					from
						j := 1
					until
						j > a_dates.count
					loop
						l_date := a_dates.item (l_date_lower + j - 1)
						l_units := l_history.units_at (l_date)
						Result.put (l_units, j, i)
						j := j + 1
					end
				end
			end
			Result.set_row_keys (a_dates)
			Result.set_column_keys (a_entities)
		end

feature -- Templates
	holding_history_template: H
		do
		end

	holding_template : F
		do
		end

	holding_entity_template : G
		do
		end

end
