note
	description: "Class for keeping track of uid relationships"
	author: "Mark Howard"
	last_editor: "$Author: er $"
	date: "$Date: 2014/02/26 $"
	revision: "$Revision: #4 $"
	path: "$File: //rose/source/kernel/entities/reference/id_handler.e $"

class
	ID_HANDLER
inherit
	ROSE_ENTITY_FOREGROUND_AWARE
		rename
			context as foreground_context
		end
	ROSE_ENTITY_BACKGROUND_AWARE
		rename
			context as background_context
		end
	PES_AWARE
	UID_AWARE
	ROSE_CONSTANTS
create
	make
feature {NONE}--Creation
	make (a_supplier_id,a_category_id : INTEGER)
			--Creation
		do
			supplier_id := a_supplier_id
			category_id := a_category_id
			uses_rose_entity_background_ids := context_descriptors.context_uses_rose_background_ids (supplier_id)
			uses_rose_entity_foreground_ids := context_descriptors.context_uses_rose_foreground_ids (supplier_id)
		ensure
			supplier_id_set : supplier_id = a_supplier_id
		end
feature --Access
	supplier_id : INTEGER
	category_id : INTEGER
	uses_rose_entity_background_ids : BOOLEAN
	uses_rose_entity_foreground_ids : BOOLEAN
	uses_rose_entity_ids : BOOLEAN
			--Do we use rose ids?
		do
			Result := uses_rose_entity_background_ids or else uses_rose_entity_foreground_ids
		end
	mapping_records (a_use_rose_bg: BOOLEAN;a_min_entity_id,a_max_entity_id: INTEGER): ROSE_LINEAR_ARRAY [ENTITY_ID_MAP_ROW] is
			--Mapping using appropriate server
		local
			l_predicate: like mapping_predicate
		do
			create Result.make_empty
			l_predicate := mapping_predicate (a_use_rose_bg)
			l_predicate.set_container (Result)
			l_predicate.retrieve_atomic (Void,<<category_id, supplier_id, a_min_entity_id, a_max_entity_id>>)
		ensure
			exists : Result /= Void
		end

	mapping_to_supplier_records (a_use_rose_bg: BOOLEAN;a_category_id,a_other_supplier_id,a_entity_id: INTEGER): ROSE_LINEAR_ARRAY [ENTITY_ID_MAP_ROW] is
			--Mapping to supplier using appropriate server
		local
			l_predicate: like mapping_predicate_to_supplier
		do
			create Result.make_empty
			l_predicate := mapping_predicate_to_supplier (a_use_rose_bg)
			l_predicate.set_container (Result)
			l_predicate.retrieve_atomic (Void,<<a_category_id,a_other_supplier_id,a_entity_id>>)
		ensure
			exists : Result /= Void
		end

	mapping_to_rose_records (a_use_rose_bg: BOOLEAN;a_category_id,a_other_supplier_id,a_entity_id: INTEGER): ROSE_LINEAR_ARRAY [ENTITY_ID_MAP_ROW] is
			--Mapping to rose using appropriate server
		local
			l_predicate: like mapping_predicate_to_rose
		do
			create Result.make_empty
			l_predicate := mapping_predicate_to_rose (a_use_rose_bg)
			l_predicate.set_container (Result)
			l_predicate.retrieve_atomic (Void,<<a_category_id,a_other_supplier_id,a_entity_id>>)
		ensure
			exists : Result /= Void
		end

	mappings_history_from_id (a_entity_id,a_other_category_id,a_other_supplier_id : INTEGER): ROSE_INTERVAL_HISTORY [INTEGER, ROSE_CLOSED_OPEN_INTERVAL [INTEGER]] is
			--Mappings from entity_id and category_id
		local
			l_rose_peers: ROSE_INTERVAL_HISTORY [INTEGER, ROSE_CLOSED_OPEN_INTERVAL [INTEGER]]
			i: INTEGER
			l_other_handler : ID_HANDLER
		do
			l_other_handler := id_handler_from_id (a_other_supplier_id,a_other_category_id)
			if a_entity_id = 0  or else l_other_handler = Void then
				--| Do nothing and return Void
			elseif uses_rose_entity_ids and l_other_handler.uses_rose_entity_ids then
				create Result.make_empty
				Result.extend (create {ROSE_CLOSED_OPEN_INTERVAL [INTEGER]}.make (a_entity_id, constants.minimum_date_time, constants.maximum_date_time, 0.0))
			elseif uses_rose_entity_ids then
				Result := l_other_handler.rose_supplier_mapping (a_entity_id, a_other_category_id,uses_rose_entity_background_ids)
			elseif l_other_handler.uses_rose_entity_ids then
				Result := supplier_rose_mapping (a_entity_id,l_other_handler.uses_rose_entity_background_ids)
			else
				l_rose_peers := supplier_rose_mapping (a_entity_id, True)
				if l_rose_peers /= Void then
					from
						create Result.make_empty
						i := l_rose_peers.lower
					until
						i > l_rose_peers.upper
					loop
						if l_rose_peers.item (i) /= Void then
							Result.append (l_other_handler.rose_supplier_mapping (l_rose_peers.item (i).item, a_other_category_id,True))
						end
						i := i + 1
					end
				end
			end
		end
	peer_handle (a_uid : INTEGER_64; a_date: ROSE_DATE): INTEGER is
			-- First entity_id of a rose peer of entity with uid a_uid
		require
			a_date_not_void: a_date /= Void
		local
			l_mappings: ROSE_INTERVAL_HISTORY [INTEGER, ROSE_CLOSED_OPEN_INTERVAL [INTEGER]]
			l_interval: ROSE_CLOSED_OPEN_INTERVAL [INTEGER]
			l_supplier_id,l_category_id,l_handle: INTEGER
			l_other_handler : ID_HANDLER
		do
			l_supplier_id := uid_supplier_id (a_uid)
			l_category_id := uid_category_id (a_uid)
			l_other_handler := id_handler_from_id (l_supplier_id,l_category_id)
			l_handle := uid_handle (a_uid)
			if l_category_id <= 0 or else l_handle <= 0 then
				Result := 0
			else
				l_mappings := l_other_handler.mappings_history_from_id (l_handle, l_category_id,supplier_id)
				if l_mappings /= Void then
					l_interval := l_mappings.witness_interval (a_date)
					if l_interval /= Void then
						Result := l_interval.item
					end
				end
			end
		end
	peer_uid (a_uid : INTEGER_64; a_date: ROSE_DATE): INTEGER_64 is
			-- First uid of a rose peer of entity with uid a_uid
		local
			l_handle : INTEGER
		do
			l_handle := peer_handle(a_uid,a_date)
			Result := constructed_uid(supplier_id,uid_category_id(a_uid),l_handle)
		end
feature {CATEGORY_HANDLER,ID_HANDLER} -- Implementation

	mapping_predicate (a_use_rose_bg: BOOLEAN): SIMPLE_PREDICATE [ENTITY_ID_MAP_ROW] is
		-- Predicate used to fetch mappings ROSE to current supplier/category
		do
			if a_use_rose_bg then
				Result := mapping_predicate_bg
			else
				Result := mapping_predicate_fg
			end
		end

	mapping_predicate_statement: STRING is
			-- Statement used to fetch mappings ROSE to current supplier/category
		once
			Result :=
				"SELECT entity_id, supplier_entity_id, start_date, end_date " +
				"FROM entity_id_map with (readuncommitted) " +
				"WHERE " +
				"category_id = ? AND supplier_product_id = ? AND " +
				"((entity_id >= ? AND entity_id <= ?) OR " +
				"(supplier_entity_id >= ?3 AND supplier_entity_id <= ?4)) " +
				"order by start_date"
		ensure
			nonvoid : Result /= Void
		end

	mapping_predicate_bg: SIMPLE_PREDICATE [ENTITY_ID_MAP_ROW] is
			-- Mapping predicate using background
		once
			create Result.make_strict (background_context.server, mapping_predicate_statement)
		ensure
			nonvoid : Result /= Void
		end

	mapping_predicate_fg: SIMPLE_PREDICATE [ENTITY_ID_MAP_ROW] is
			--Mapping using foreground
		once
			create Result.make_strict (foreground_context.server, mapping_predicate_statement)
		ensure
			nonvoid : Result /= Void
		end

	mapping_predicate_to_rose (a_use_rose_bg: BOOLEAN): SIMPLE_PREDICATE [ENTITY_ID_MAP_ROW] is
			--Mapping using either rose foreground or rose background
		do
			if a_use_rose_bg then
				Result := mapping_predicate_to_rose_bg
			else
				Result := mapping_predicate_to_rose_fg
			end
		ensure
			nonvoid : Result /= Void
		end

	mapping_predicate_to_rose_statement: STRING
			-- Statement used to fetch mappings ROSE to current supplier/category
		once
			Result := "SELECT entity_id,start_date,end_date FROM entity_id_map with (readuncommitted) WHERE category_id=? AND supplier_product_id=? AND supplier_entity_id=?"
		ensure
			nonvoid : Result /= Void
		end

	mapping_predicate_to_rose_bg: SIMPLE_PREDICATE [ENTITY_ID_MAP_ROW] is
			--Mapping using rose background
		once
			create Result.make_strict (background_context.server, mapping_predicate_to_rose_statement)
		ensure
			nonvoid : Result /= Void
		end

	mapping_predicate_to_rose_fg: SIMPLE_PREDICATE [ENTITY_ID_MAP_ROW] is
			--Predicate using rose foreground
		once
			create Result.make_strict (foreground_context.server, mapping_predicate_to_rose_statement)
		ensure
			nonvoid : Result /= Void
		end

	mapping_predicate_to_supplier (a_use_rose_bg: BOOLEAN): SIMPLE_PREDICATE [ENTITY_ID_MAP_ROW] is
			-- Mapping predicate using supplier
		do
			if a_use_rose_bg then
				Result := mapping_predicate_to_supplier_bg
			else
				Result := mapping_predicate_to_supplier_fg
			end
		ensure
			nonvoid : Result /= Void
		end

	mapping_predicate_to_supplier_statement: STRING is
			-- Statement used to fetch mappings ROSE to current supplier/category
		once
			Result := "SELECT supplier_entity_id entity_id,start_date,end_date FROM entity_id_map with (readuncommitted) WHERE category_id=? AND supplier_product_id=? AND entity_id=?"
		ensure
			nonvoid : Result /= Void
		end

	mapping_predicate_to_supplier_bg: SIMPLE_PREDICATE [ENTITY_ID_MAP_ROW] is
			--Mapping to supplier background
		once
			create Result.make_strict (background_context.server, mapping_predicate_to_supplier_statement)
		ensure
			nonvoid : Result /= Void
		end

	mapping_predicate_to_supplier_fg: SIMPLE_PREDICATE [ENTITY_ID_MAP_ROW] is
			-- Mapping predicate to supplier foreground
		once
			create Result.make_strict (foreground_context.server, mapping_predicate_to_supplier_statement)
		ensure
			nonvoid : Result /= Void
		end

	rose_context_server (a_use_rose_bg: BOOLEAN): SQL_DATA_SERVER is
			-- Server for the rose context
		do
			if a_use_rose_bg then
				Result := background_context.server
			else
				Result := foreground_context.server
			end
		ensure
			nonvoid : Result /= Void
		end
	mappings_block_size: INTEGER is 1000

	mappings_block_cache: HASH_TABLE [INTEGER, INTEGER]
		-- Stores the beginning value of each block of 'rose_supplier_mappings' or
		-- 'supplier_rose_mappings' that have already been retrieved so that we don't
		-- try to get them twice.

	rose_supplier_mappings: HASH_TABLE [ROSE_INTERVAL_HISTORY [INTEGER, ROSE_CLOSED_OPEN_INTERVAL [INTEGER]], INTEGER]
			-- Table associating ROSE entity ids to current supplier/category entity ids
			-- Attention: Void if current category handler bound to a ROSE context

	supplier_rose_mappings: HASH_TABLE [ROSE_INTERVAL_HISTORY [INTEGER, ROSE_CLOSED_OPEN_INTERVAL [INTEGER]], INTEGER]
			-- Table associating current supplier/category entity ids to ROSE entity ids
			-- Attention: Void if current category handler bound to a ROSE context

	update_mappings_cache_loaded_data: BOOLEAN -- Do we need to look for new entity ids?

	rose_supplier_mapping (a_entity_id,a_category_id: INTEGER; a_use_rose_bg: BOOLEAN): ROSE_INTERVAL_HISTORY [INTEGER, ROSE_CLOSED_OPEN_INTERVAL [INTEGER]] is
			-- Supplier entity ids mapped from this rose entity id
		do
			if rose_supplier_mappings = Void then
				build_mapping_tables
			end
			rose_supplier_mappings.search (a_entity_id)
			if not rose_supplier_mappings.found	then
				update_mappings_cache (a_entity_id,a_use_rose_bg)
				if update_mappings_cache_loaded_data then
					rose_supplier_mappings.search (a_entity_id)
				end
			end
			if rose_supplier_mappings.found	then
				Result := rose_supplier_mappings.found_item
			end
		end

	supplier_rose_mapping (a_entity_id : INTEGER; a_use_rose_bg: BOOLEAN): ROSE_INTERVAL_HISTORY [INTEGER, ROSE_CLOSED_OPEN_INTERVAL [INTEGER]] is
			-- Rose entity ids mapped from this supplier entity id
		do
			if supplier_rose_mappings = Void	then
				build_mapping_tables
			end
			supplier_rose_mappings.search (a_entity_id)
			if not supplier_rose_mappings.found	then
				update_mappings_cache (a_entity_id,a_use_rose_bg)
				if update_mappings_cache_loaded_data then
					supplier_rose_mappings.search (a_entity_id)
				end
			end
			if supplier_rose_mappings.found then
				Result := supplier_rose_mappings.found_item
			end
		end

	build_mapping_tables is
			--Construct mapping tables
		do
			create rose_supplier_mappings.make (mappings_block_size * 10)
			create supplier_rose_mappings.make (mappings_block_size * 10)
			create mappings_block_cache.make (100)
		ensure
			rose_supplier_mappings_not_void : rose_supplier_mappings /= Void
			supplier_rose_mappings_not_void : supplier_rose_mappings /= Void
			mappings_cache_not_void : mappings_block_cache /= Void
		end

	update_mappings_cache (a_intended_entity_id : INTEGER; a_use_rose_bg: BOOLEAN) is
			-- Update mappings cache tables.
		require
			mappings_block_cache_not_void: mappings_block_cache /= Void
		local
			l_min_entity_id: INTEGER
			l_max_entity_id: INTEGER
		do
			l_min_entity_id := (a_intended_entity_id // mappings_block_size) * mappings_block_size
			mappings_block_cache.search (l_min_entity_id)
			if not mappings_block_cache.found then
				mappings_block_cache.extend (l_min_entity_id, l_min_entity_id)
				l_max_entity_id := l_min_entity_id + mappings_block_size - 1
				mapping_records (a_use_rose_bg, l_min_entity_id, l_max_entity_id).apply (agent add_mapping (?,  l_min_entity_id, l_max_entity_id))
				update_mappings_cache_loaded_data := True
			else
				update_mappings_cache_loaded_data := False
			end
		end

	add_mapping (a_record: ENTITY_ID_MAP_ROW; a_min_entity_id, a_max_entity_id: INTEGER) is
			-- Add 'a_record' to current mapping tables.
		require
			record_not_void : a_record /= Void
		local
			l_history: ROSE_INTERVAL_HISTORY [INTEGER, ROSE_CLOSED_OPEN_INTERVAL [INTEGER]]
			l_interval: ROSE_CLOSED_OPEN_INTERVAL [INTEGER]
		do
			-- NOTE - There was a bug in this code.  We used to cache ANY entity_id, even
			-- those outside of the range (min, max).  Since those entities were not in
			-- the 'mappings_block_cache' we would add them AGAIN if it was requested.
			-- This resulted in a duplicate of the interval and the intervals were out
			-- of order.  Note that by not caching the entities outside of the min/max
			-- range we don't really lose anything since they aren't in the
			-- 'mappings_block_cache'.  We will cache the WHOLE history of this entity
			-- when/if it is actually requested in the min/max range.
			-- NOTE - The rows must be in date order!
			--| rose -> supplier
			if a_record.entity_id >= a_min_entity_id and then a_record.entity_id <= a_max_entity_id then
				l_history := rose_supplier_mappings.item (a_record.entity_id)
				if l_history = Void then
					create l_history.make_empty
					rose_supplier_mappings.put (l_history, a_record.entity_id)
				end
				create l_interval.make (a_record.supplier_entity_id, a_record.start_date, a_record.end_date, 0.0)
				l_history.extend (l_interval)
			end
			--| supplier -> rose
			if a_record.supplier_entity_id >= a_min_entity_id and then a_record.supplier_entity_id <= a_max_entity_id then
				l_history := supplier_rose_mappings.item (a_record.supplier_entity_id)
				if l_history = Void then
					create l_history.make_empty
					supplier_rose_mappings.put (l_history, a_record.supplier_entity_id)
				end
				create l_interval.make (a_record.entity_id, a_record.start_date, a_record.end_date, 0.0)
				l_history.extend (l_interval)
			end
		end

	add_mapping_uncached (a_history: ROSE_INTERVAL_HISTORY [INTEGER, ROSE_CLOSED_OPEN_INTERVAL [INTEGER]]; a_record: ENTITY_ID_MAP_ROW) is
			-- Add 'a_record' to 'a_history', if not already present.
		require
			record_not_void : a_record /= Void
			history_not_void : a_history /= Void
		do
			from
				a_history.start
			until
				a_history.after or else
				(a_history.current_item /= Void and then a_history.current_item.item = a_record.entity_id)
			loop
				a_history.forth
			end
			if a_history.after then
				a_history.extend (create {ROSE_CLOSED_OPEN_INTERVAL [INTEGER]}.make (a_record.entity_id, a_record.start_date, a_record.end_date, 0.0))
			end
		end
feature --Update
	clear_cache
			--Start over
		do
			mappings_block_cache := Void
			rose_supplier_mappings := Void
			supplier_rose_mappings := Void
		ensure
			cleared : mappings_block_cache = Void and then
						rose_supplier_mappings = Void and then
						supplier_rose_mappings = Void
		end
feature --Implementaion
	context_descriptors : CONTEXT_DESCRIPTORS
			-- All registered contexts
		once
			create Result.make
		ensure
			exists : Result /= Void
		end

end
