indexing

	description: "[
		See IEEE_MATH_BASE for documentation.
		MSVC++/i386 implementation of IEEE_MATH.
		
		This code is optimized for speed so you will see some cut and paste.  For maximum
		performance, try to use these features directly, not through an intermediate 
		once function.
		
		Note that functions marked 'C signature' are simply inlined in the calling code.
		Functions marked 'C inline' are actual functions, however, every class that uses
		the function gets its own local copy in the generated C code.  Therefore, the C
		compiler may (at its discretion) inline the call.
	]"

	author: "Randy John"
	last_editor: "$Author: rj $"
	date: "$Date: 2012/10/01 $";
	revision: "$Revision: #8 $"
	path: "$File: //rose/source/kernel/infrastructure/math/windows/ieee_math.e $"

class IEEE_MATH

inherit

	IEEE_MATH_BASE

feature -- Basic operations
	isnan(a_x: DOUBLE): BOOLEAN
	-- Is 'a_x' an NaN (Not a Number)?
	external
		"C signature (double): int use <float.h>"
	alias
		"_isnan"
	end

	isfinite(a_x: DOUBLE): BOOLEAN
	-- Is 'a_x' finite?  (Note that NaN is not finite).
	external
		"C signature (double): int use <float.h>"
	alias
		"_finite"
	end

	is_missing_double(a_value: DOUBLE): BOOLEAN
	-- What is the difference between 'is_missing_double' and 'not defined_double'?
	-- At the moment - nothing.  However, in the future we may wish to exclude
	-- positive_infinity and negative_infinity from the definition of 'defined'.
	external
		"C signature (double): int use <float.h>"
	alias
		"_isnan"
--		Result := isnan (a_value)						-- From ROSE_CONSTANT_VALUES.
	end

	is_finite_double(a_value: DOUBLE): BOOLEAN
	-- Is 'a_value' finite?
	external
		"C signature (double): int use <float.h>"
	alias
		"_finite"
--		Result := isfinite (a_value)					-- From ROSE_CONSTANT_VALUES.
	end

	is_missing_real(a_value: REAL): BOOLEAN
	-- Is 'a_value' QNAN?
	external
		"C signature (double): int use <float.h>"
	alias
		"_isnan"
--		Result := isnan (a_value)						-- From ROSE_CONSTANT_VALUES.
	end

	defined_double(a_value: DOUBLE): BOOLEAN
	-- Is `a_value' a defined (not an unknown) value?
	external
		"C inline use <float.h>"
	alias
		"_isnan($a_value) == 0"
--		Result := not isnan (a_value)					-- From ROSE_CONSTANT_VALUES and MISSING_PARAMETERS.
	end

	is_defined_real(a_value: REAL): BOOLEAN
	-- Is `a_value' a defined REAL value (neither missing, NAN, nor infinite)?
	external
		"C inline use <float.h>"
	alias
		"$a_value != -999 && _finite($a_value) != 0"	-- Note that when '_finite' is True, 'a_value' is NOT NaN.
--		Result := a_value /= Legacy_missing_real and then not isnan (a_value.to_double) and then isfinite (a_value.to_double)		-- From ROSE_CONSTANTS.
	end

	is_defined_double(a_value: DOUBLE): BOOLEAN
	-- Is `a_value' a defined DOUBLE value (neither missing, NAN, nor infinite)?
	external
		"C inline use <float.h>"
	alias
		"$a_value != -999 && _finite($a_value) != 0"	-- Note that when '_finite' is True, 'a_value' is NOT NaN.
--		Result := a_value /= Legacy_missing_double and then not isnan (a_value) and then isfinite (a_value)							-- From ROSE_CONSTANTS.
	end

	is_missing(a_value: DOUBLE): BOOLEAN
	-- Is `a_value' a missing DOUBLE value (legacy_missing or NAN)?
	external
		"C inline use <float.h>"
	alias
		"$a_value == -999.0 || _isnan($a_value) != 0"
--		Result := a_value = Legacy_missing_double or else isnan (a_value)															-- From ROSE_CONSTANTS.
	end

	extended_less(a_left, a_right: DOUBLE): BOOLEAN
	-- This is a normal less_than if both arguments are defined.  The exact ordering is:
	--		NaN/-999 < -Infinity < normal number < +Infinity.
	-- Note that NaN and -999 have no defined ordering.
	external
		"C inline use <float.h>"
	alias
		"[
			if ($a_right != -999.0 && _isnan($a_right) == 0)
			{
				if ($a_left != -999.0 && _isnan($a_left) == 0)
					return $a_left < $a_right;
				else
					return EIF_TRUE;
			}
			else
				return EIF_FALSE;
		]"
--			if not is_missing (a_right) then																						-- From ROSE_CONSTANTS.
--				if not is_missing (a_left) then
--					Result := a_left < a_right
--				else
--					Result := True
--				end
--			end
	end

	trap_exceptions is
			-- Raise exception on invalid operations, arithmetic, overflow and divide-by-zero.
		do
			c_set_exceptions
		end

feature {NONE} -- Implementation

	c_set_exceptions is
			-- Enable floating point exceptions.
		external
			"C inline use <float.h>"
		alias
			"[
				_clearfp;
				_controlfp (_EM_INVALID | _EM_ZERODIVIDE | _EM_OVERFLOW, _MCW_EM);
			]"
		end

end
