note

	description:
	"[
		Represents the cell and generating path of a binary partition of a two dimensional array,
		where each split is either a horizontal or vertical partitioning.  Note that this is more
		flexible than a quad-tree representation
	]"
	author: "Mark Howard"
	last_editor: "$Author: mb $"
	date: "$Date: 2012/02/24 $"
	revision: "$Revision: #2 $"
	path: "$File: //rose/source/kernel/infrastructure/structures/axis/integer_date_partition_tree_record.e $"

class INTEGER_DATE_PARTITION_TREE_RECORD

inherit
	PARTITION_TREE_RECORD [INTEGER, ROSE_DATE]
		redefine
			set_min_1,
			set_max_1
		end
	
feature -- Element modification

	set_min_1 (a_value: like min_1) is
 			-- <Precursor>
		do
			if date_cache.found and then date_cache.found_item ~ a_value then
				min_1 := date_cache.found_item
			else
				date_cache.search (a_value)
				if date_cache.found then
					min_1 := date_cache.found_item
				else
					min_1 := a_value
					date_cache.force (min_1)
				end
			end
		end

	set_max_1 (a_value: like max_1) is
 			-- <Precursor>
		do
			if date_cache.found and then date_cache.found_item ~ a_value then
				max_1 := date_cache.found_item
			else
				date_cache.search (a_value)
				if date_cache.found then
					max_1 := date_cache.found_item
				else
					max_1 := a_value
					date_cache.force (max_1)
				end
			end
		end

feature {NONE} -- Implementation
	
	date_cache: DS_HASH_SET [ROSE_DATE] is
			-- Single source of ROSE_DATE objects for use by this type of 
			-- partition tree
		once
			create Result.make (1_000)
			Result.set_equality_tester (create {KL_EQUALITY_TESTER [ROSE_DATE]})
		ensure
			valid_result: Result /= Void
		end
	
end
