indexing
	description: "Array of Integer"
	author: "Michael Zhang"
	date: "$Date: 2014/02/26 $"
	revision: "$Revision: #16 $"

class
	INTEGER_VECTOR

inherit
	ROSE_LINEAR_ARRAY[INTEGER]
		undefine
			is_equal
		end

	MULTI_KEYED
		undefine
			copy
		end

create
	make, make_empty, make_from_array, make_from_linear, test, make_pulse, make_from_vector, make_sequence, make_from_vector_with_truncation, make_constant, make_from_delimited_string

feature -- creation
	make_constant(a_value, a_count : INTEGER) is
		-- constant array
		local
			i : INTEGER
		do
			make(1, a_count)
			from i := 1 until i > a_count loop
				put(a_value, i)
				i := i + 1
			end
		end

	make_sequence(a_beg, a_end, a_step : INTEGER) is
		-- sequence
		require
			non_zero_step: a_step /= 0
			same_sign_step: (a_end - a_beg) * a_step > 0 or a_beg = a_end
			a_step_small_enough : (a_end - a_beg).abs >= a_step.abs or a_beg = a_end
		local
			i, l_count : INTEGER
		do
			l_count := 1 + ((a_end - a_beg) / a_step).floor
			make(1, l_count)
			from i := 1 until i > l_count loop
				put(a_beg + (i-1) * a_step, i)
				i := i + 1
			end
		end

	make_pulse(a_vector : VECTOR; a_threshold : DOUBLE) is
			-- make pulse
		local
			i : INTEGER
		do
			is_pulse := True
			if a_vector /= Void then
				make(1, a_vector.count)
				from i := 1 until i > count loop
					if a_vector.item(i) > a_threshold then
						put(1, i)
					else
						put(0, i)
					end
					i := i + 1
				end
			else
				make_empty
			end
		end

	make_from_vector(a_vector : VECTOR; a_discretize_function : FUNCTION[ANY, TUPLE[DOUBLE], INTEGER]) is
			-- make from vector
		local
			i : INTEGER
			l_value : INTEGER
		do
			if a_vector /= Void and a_discretize_function /= Void then
				make(1, a_vector.count)
				from i := 1 until i > count loop
					l_value := a_discretize_function.item([a_vector.item(i)])
					put(l_value, i)
					i := i + 1
				end
			else
				make_empty
			end
		end

	make_from_vector_with_truncation(a_vector : VECTOR) is
			-- make from vector with truncation
		local
			i : INTEGER
			l_value : INTEGER
		do
			if a_vector /= Void then
				make(1, a_vector.count)
				from i := 1 until i > count loop
					l_value := a_vector.item(i).truncated_to_integer
					put(l_value, i)
					i := i + 1
				end
			else
				make_empty
			end
		end

	make_from_delimited_string(a_string : STRING; a_delimiter : CHARACTER) is
			-- make from delimited string
		local
			i : INTEGER
			l_string : STRING
			l_dtext : ROSE_DELIMITED_TEXT
		do
			create l_dtext.make (a_string, a_delimiter)
			make(1, l_dtext.count)
			from i := 1 until i > l_dtext.count loop
				l_string := l_dtext[i]
				if l_string.is_integer then
					Current[i] := l_string.to_integer
				end
				i := i + 1
			end
		end

feature -- conversion

	to_array : ARRAY[INTEGER] is
			-- to array
		local
			i : INTEGER
		do
			create Result.make(1, count)
			from i := 1 until i > count loop
				Result.put(item(i), i)
				i := i + 1
			end
		end

	to_vector : VECTOR is
			-- to vector
		local
			i : INTEGER
		do
			create Result.make(count)
			from i := 1 until i > count loop
				Result.put (item (i), i)
				i := i + 1
			end
		end

	to_strings : ROSE_LINEAR_ARRAY[STRING] is
			-- to strings
		do
			create Result.make(1, count)
			from start until off loop
				Result.put(current_item.out, index)
				forth
			end
		end
	to_integer_ref_array : ROSE_LINEAR_ARRAY[INTEGER_REF] is
		local
			i : INTEGER
		do
			create Result.make(1, count)
			from i := 1 until i > count loop
				Result.put(item(i), i)
				i := i + 1
			end
		end

feature -- pulse sequence
	is_pulse : BOOLEAN

	pulse_pattern : STRING is
			-- pulse pattern
		local
			l_beg_end_pos : INTEGER
		do
			if is_pulse then
				from l_beg_end_pos := 2 until l_beg_end_pos > count or item(l_beg_end_pos) /= first loop
					l_beg_end_pos := l_beg_end_pos + 1
				end
				if first = last then
					if l_beg_end_pos > count then
						Result := first.out
					else
						Result := first.out + (1 - first).out + first.out
					end
				else
					Result := first.out + last.out
				end
			end
		end
feature -- sub set
	indices_of(a_i : INTEGER) : like Current is
			-- indices of
		do
			create Result.make_empty
			from start until off loop
				if current_item = a_i then
					Result.extend(index)
				end
				forth
			end
		end
feature -- Access
	states : like Current is
			-- states
		local
			l_table : HASH_TABLE[INTEGER, INTEGER]
		do
			create l_table.make(10)
			from start until off loop
				if not l_table.has(current_item) then
					l_table.put(current_item, current_item)
				end
				forth
			end
			create Result.make_from_array(l_table.current_keys)
			Result.sort_by(Agent integer_order)
		end

	frequencies : PAIR[like Current, like Current] is
		local
			l_states : like states
			l_frequencies : like current
			l_table : HASH_TABLE[INTEGER, INTEGER]
			l_count : INTEGER
		do
			create l_table.make(10)
			from start until off loop
				if l_table.has(current_item) then
					l_count := l_table.item(current_item) + 1
					l_table.force(l_count, current_item)
				else
					l_table.put(1, current_item)
				end
				forth
			end

			l_states := states
			create l_frequencies.make(l_states.lower, l_states.upper)
			from l_states.start until l_states.off loop
				if l_table.has(l_states.current_item) then
					l_frequencies.put(l_table.item(l_states.current_item), l_states.index)
				end
				l_states.forth
			end
			create Result.make(l_states, l_frequencies)
		end

	runs : PAIR[like Current, like Current] is
		local
			l_quotient : like quotient
			l_states, l_counts : like Current
		do
			l_quotient := quotient(Agent compare_state)
			create l_states.make_empty
			create l_counts.make_empty
			from l_quotient.start until l_quotient.off loop
				l_states.extend(l_quotient.current_item.first)
				l_counts.extend(l_quotient.current_item.count)
				l_quotient.forth
			end
			create Result.make(l_states, l_counts)
		end

feature -- Access
	smoothed_vector(a_window_size : INTEGER) : like Current is
		-- only sustain run with size > window size will be considered as run (smoothing out noisy fluctuation)
		-- prevailing state won't change until a new run emerges
		local
			i : INTEGER
			l_prevailing_state, l_current_run_value : INTEGER
			l_run_count : INTEGER
			l_smoothed_vector : like Current
		do
			l_smoothed_vector := cloned
			if count > 0 then
				l_prevailing_state := item(lower)
				l_current_run_value := item(lower)
				l_run_count := 1
				from i := lower + 1 until i > upper loop
					if item(i) = l_current_run_value then
						l_run_count := l_run_count + 1
						if l_run_count < a_window_size then
							l_smoothed_vector.put(l_prevailing_state ,i)
						else
							l_prevailing_state := l_current_run_value
						end
					else
						l_current_run_value := item(i)
						l_run_count := 1
						if l_run_count < a_window_size then
							l_smoothed_vector.put(l_prevailing_state ,i)
						end
					end
					i := i + 1
				end
				Result := l_smoothed_vector
			end
		end

	smoothed_runs(a_window_size : INTEGER) : PAIR[like Current, like Current] is
		local
			l_smoothed_vector : like Current
		do
			l_smoothed_vector := smoothed_vector(a_window_size)
			Result := l_smoothed_vector.runs
		end

feature {NONE} -- Implementation
	compare_state(a_state_1, a_state_2 : INTEGER) : BOOLEAN is
		-- quotient can only be used when the array is ordered, or, OK when counting runs
		do
			Result := a_state_1 = a_state_2
		end

	integer_order(a, b : INTEGER) : BOOLEAN is
		do
			Result := a < b
		end

feature -- Operations
	name : STRING
	set_name(a_name : STRING) is
		do
			name := a_name
		end

	item_names : ROSE_LINEAR_ARRAY[STRING]
	set_item_names(a_names : ARRAY[STRING]) is
		require
			same_size : a_names /= Void and then a_names.count = count
		do
			if a_names /= Void then
				item_names ?= a_names
				if item_names = Void then
					create item_names.make_from_array(a_names)
				end
			end
		end

	current_name : STRING is
		do
			if item_names /= Void and then item_names.valid_index(index) then
				Result := item_names.item(index)
			end
		end

	cloned : like Current is
		local
			i : INTEGER
		do
			create Result.make(lower, upper)
			from i := lower until i > upper loop
				Result.put(item(i), i)
				i := i + 1
			end
			copy_names(Result)
		end

	copy_names(a_vector : like Current) is
		do
			if a_vector /= Void then
				if name /= Void then
					a_vector.set_name(name)
				end
				if item_names /= Void and then item_names.count = count then
					a_vector.set_item_names(item_names)
				end
			end
		end

feature -- Access
	hash_key(i : INTEGER) : HASHABLE is
		do
			Result := item(i)
		end

	key_count : INTEGER is
		do
			Result := count
		end

feature -- Testing
	test is
		local
			l_states : like states
			l_frequencies : like frequencies
			l_runs : like runs
			l_smoothed_runs : like smoothed_runs
		do
			make_from_array(<<3, 3, 1, 2, 2, 2, 3, 3, 3>>)
			l_smoothed_runs := smoothed_runs(2)
			make_from_array(<<3, 3, 1, 3, 3, 3, 3, 3, 3>>)
			l_smoothed_runs := smoothed_runs(2)
			l_states := states
			l_frequencies := frequencies
			l_runs := runs
		end

feature -- Access
	sum : INTEGER is
			-- Sum of all items
		local
			i : INTEGER
		do
			from i := lower until i > upper loop
				Result := Result + item(i)
				i := i + 1
			end
		end

	cum_vector : like Current is
		local
			i : INTEGER
			l_sum : INTEGER
		do
			create Result.make(1, count)
			from i := lower until i > upper loop
				l_sum := l_sum + item(i)
				Result.put(l_sum, i - lower + 1)
				i := i + 1
			end
		end

	cum_range_vector : like Current is
		-- first element set to be zero to make it easy for sub range
		local
			i : INTEGER
			l_sum : INTEGER
		do
			create Result.make(1, count + 1)
			from i := lower until i > upper loop
				l_sum := l_sum + item(i)
				Result.put(l_sum, i - lower + 2)
				i := i + 1
			end
		end
feature -- Access
	max : INTEGER is
			-- Maximum value
		local
			i : INTEGER
		do
			if count > 0 then
				Result := item(1)
			end
			from i := 2 until i > count loop
				if item(i) > Result then
					Result := item(i)
				end
				i := i + 1
			end
		end

	min : INTEGER is
		local
			i : INTEGER
		do
			if count > 0 then
				Result := item(1)
			end
			from i := 2 until i > count loop
				if item(i) < Result then
					Result := item(i)
				end
				i := i + 1
			end
		end

	index_of_max : INTEGER is
		local
			i : INTEGER
		do
			if count > 0 then
				Result := 1
			end
			from i := 1 until i > count loop
				if less_than(item(Result), item(i)) then
					Result := i
				end
				i := i + 1
			end
		end

	index_of_min : INTEGER is
		local
			i : INTEGER
		do
			if count > 0 then
				Result := i
			end
			from i := 1 until i > count loop
				if item(i) < item(Result) then
					Result := i
				end
				i := i + 1
			end
		end

end
