note

	description:

		"ancestor for entities which are of interest to the investment process"

	keywords: "investment,entity,persistent,persistence"
	author: "Mark Howard"
	last_editor: "$Author: ext-asurelia $"
	date: "$Date: 2015/02/19 $"
	revision: "$Revision: #68 $"
	path: "$File: //rose/source/kernel/investment_entities/investment_entity.e $"

deferred class
	INVESTMENT_ENTITY

inherit
	COMPARABLE
		undefine
			is_equal
		end
	ENTITY
		redefine
			context,
			make,
			make_context,
			debug_output
		end
	ROSE_LEGACY_UNDEFINED
		undefine
			is_equal
		end

	ROSE_ENTITY_FOREGROUND_AWARE -- for make_context, misguided?
		rename
			context as rose_foreground_context
		export
			{NONE} all
		end

	ROSE_INVESTMENT_ENTITIES
		undefine
			is_equal
		end

feature {NONE} -- Initialization

	make (a_context: like context; a_category_id: INTEGER; a_handle: INTEGER; a_start_date, a_end_date: ROSE_DATE)
			--Initialization
		local
			l_xrf: STRING
			l_zone: STRING
			l_zonal_xrf: ZONAL_XRF_RECORD
			l_xrf_code_identifier: like xrf
			l_xrf_code_is_old: BOOLEAN
			l_xrf_code_saved: like xrf_code
		do
			Precursor(a_context,a_category_id,a_handle,a_start_date,a_end_date)
			if a_handle /= 0 and then xrf_code = 0 then
				l_zonal_xrf := zonal_xrfs.item([a_category_id,a_handle])		-- The 'xrf_code's in 'zonal_xrfs' are always "as of" 9999/12/31.
				if l_zonal_xrf /= Void then
					set_xrf_code(l_zonal_xrf.xrf)
					l_zone := l_zonal_xrf.zone
					if l_zone /= Void and then l_zone.count > 0 then
						zone_mnemonic := l_zone.item(1)
					end
				else
					l_xrf_code_identifier := xrf

					-- Fetch the 'xrf' attribute (see also 'attributes_used_in_make').
					if a_end_date /= Void then
						l_xrf := entity_code(l_xrf_code_identifier, a_end_date.previous_date)
						l_xrf_code_is_old := not a_end_date.is_max_date
					else
						l_xrf := entity_code(l_xrf_code_identifier, Void)
						l_xrf_code_is_old := False
					end
					if l_xrf /= Void and then l_xrf.is_integer then
					else
						l_xrf := entity_code(l_xrf_code_identifier, Void)
						l_xrf_code_is_old := False
					end
					if l_xrf /= Void and then l_xrf.is_integer then
						set_xrf_code(l_xrf.to_integer)
						if xrf_code = 0 then
							l_xrf := l_xrf_code_identifier.most_recent_key (Current, a_end_date)
							if l_xrf /= Void and then l_xrf.is_integer then
								set_xrf_code(l_xrf.to_integer)
								l_xrf_code_is_old := not a_end_date.is_max_date
							else
								l_xrf_code_is_old := False
							end
						end
					else
						l_xrf_code_is_old := False
					end

					-- Fetch the 'zone_code' attribute (see also 'attributes_used_in_make').
					l_zone := zone_code.most_recent_key (Current,a_end_date)
					if l_zone = Void then
						l_zone := entity_code(zone_code, Void)
					end
					if l_zone /= Void and then l_zone.count > 0 then
						zone_mnemonic := l_zone.item(1)
					end
				end
			end

			if l_xrf_code_is_old
			then
				-- Don't allow old xrf_code's to be registered.  ZONE builds a HASH_TABLE keyed by them (for some categories).
				l_xrf_code_saved := xrf_code
				set_xrf_code(0)
			end

			register

			if l_xrf_code_is_old
			then
				xrf_code := l_xrf_code_saved
			end
		end

	make_context is
			-- Create `context'.
		do
			rose_foreground_context.do_nothing -- Initialize the once if not already done
			-- debatable whether initializing rose foreground is legitimate
			-- in general...
		end

feature {INVESTMENT_ENTITY} -- Initialization

	make_with_zone_xrf(a_context : like context;a_category_id : INTEGER;a_handle : INTEGER;a_start_date,a_end_date : ROSE_DATE;a_zone : ZONE[LOCATION];a_xrf : INTEGER) is
			-- creation with zone/xrf
		do
			set_xrf_code(a_xrf)
			zone_mnemonic := a_zone.mnemonic
			make(a_context,a_category_id,a_handle,a_start_date,a_end_date)
		end

feature -- Access

	context : ROSE_CONTEXT is
			-- Entity context
			-- (specialised to ROSE_CONTEXT)
		do
			Result ?= Precursor
		end


	zone : like context.zone_template is
			-- Zone object
		do
			Result := zone_by_mnemonic(zone_mnemonic)
		end

	same_zone (a_other: INVESTMENT_ENTITY): BOOLEAN
			-- Is this entity in same zone as `a_other'?
		do
			Result := zone_mnemonic.as_lower = a_other.zone_mnemonic.as_lower
		ensure
			definition: Result = zone.same_zone (a_other.zone)
		end

	zone_mnemonic : CHARACTER
			-- Zone code (from entity attribute)
	xrf_code : INTEGER
			-- XRF code

	xrf_zone_display: STRING is
			-- Xrf + zone
		do
			Result := xrf_code.out
			Result.append_character (zone_mnemonic)
		ensure
			not_void: Result /= Void
			not_empty: Result.count > 1
			xrf_at_start: Result.substring (1, Result.count - 1).to_integer = xrf_code
			zone_at_end: Result.item (Result.count) = zone_mnemonic
		end


	brim_code (a_date : ROSE_DATE) : STRING is
			-- BRIM_CODE entity attribute value
		do
			Result := entity_code(brim,a_date)
		end



	infix "<" (a_other:like Current):BOOLEAN is
			-- Order defined by zone and xrf_code
		do
			Result := zone_mnemonic < a_other.zone_mnemonic or
				(zone_mnemonic = a_other.zone_mnemonic and xrf_code<a_other.xrf_code)
		end

	set_zone (a_zone : like context.zone_template) is
			-- Assign `a_zone' to `zone_mnemonic'.
		do
			zone_mnemonic := a_zone.mnemonic
		ensure
			zone_mnemnonic_set: zone_mnemonic = a_zone.mnemonic
		end

	set_xrf_code (a_code : INTEGER) is
			-- Assign `a_code' to `xrf_code'.
		do
			xrf_code := a_code
		ensure
			xrf_code_set: xrf_code = a_code
		end

	register is
			-- The child can redefine this to do nothing but many do this:
			-- 	if zone/=Void
			--	then
			--		zone.add_XXXX(Current)	-- XXXX is the name of the category.
			--	end
		deferred
		end

	add_zonal_xrf (a_record : ZONAL_XRF_RECORD) is
			-- Add record to zonal cache.
		do
			zonal_xrfs.put(a_record,[a_record.category_id,a_record.entity_id])
			zonal_xrf_records.put(a_record,[a_record.category_id,a_record.zone.out,a_record.xrf])
		end

	attributes_used_in_make: ROSE_LINEAR_ARRAY[ENTITY_IDENTIFIER] is
			-- Used by CATEGORY_HANDLER:cache_common_attributes just before creating lots of entities.
			-- A null version of this feature could be placed in ENTITY but you will have to modify 35 other classes.
		do
			create Result.make_empty
			Result.extend(xrf)
			Result.extend(zone_code)
		ensure
			not_void: Result /= Void
		end

	debug_output: STRING is
			-- Debug display
		do
			Result := Precursor
			if Result=Void
			then
				Result := ""
			end

			if xrf_code/=0
			then
				Result.append(":" + xrf_code.out + zone_mnemonic.out)
			end
		end
	zone_handle : STRING
			--String comprised of zone + handle ... uniquely identifies zonal proxies with enough information to get the underlying entity directly from the handle
		do
			Result := zone_mnemonic.out + handle.out
		end

	is_valid_with_non_zero_xrf_code: BOOLEAN
			-- Is valid and has a non-zero xrf code
		do
			Result := is_valid and then xrf_code /= 0
		end
		
feature {ZONE} -- Implementation

	entity_from_zone_xrf (a_zone : like context.zone_template;a_xrf : INTEGER) : like Current is
			-- Entity of the same type as `Current' identified by `a_zone' and `a_xrf'
  		do
			if Result  = void then
				Result ?= zonal_xrf_entity(a_xrf,a_zone,category_id)
				if Result  = void then
					Result ?= context.handler(category_id).entity(0)
					if Result /= Void then
						-- create a clone of the descendant type via ur_entity and then set new parameters
						Result := Result.twin -- detach from ur_entity
						Result.make_with_zone_xrf(context,category_id,handle,start_date,end_date,a_zone,a_xrf)
					end
				end
			end
		end

feature {RESEARCH} -- Implementation

	entity_code (a_identifier : like context.entity_identifier_template;a_date : ROSE_DATE) : STRING is
			-- Value of an entity attribute described by `a_identifier'
		require
			non_void_identifier: a_identifier/=Void
		do
			if handle = 0 then
				Result := ""
			else
				Result := a_identifier.dated_key(Current,a_date)
			end
		end

feature {NONE} -- Implementation

	zonal_xrf_entity(i : INTEGER;a_zone : like context.zone_template;a_category_id : INTEGER) : ENTITY is
			-- Cache lookup of entity for `entity_from_zone_xrf' with fallback
			-- to context loookup
		local
			l_record : ZONAL_XRF_RECORD
		do
			l_record := zonal_xrf_records.item([a_category_id,a_zone.mnemonic.out,i])
			if l_record /= void then
				Result := context.handler(a_category_id).entity(l_record.entity_id)
			end
			if Result = void then
				Result := context.handler(a_category_id).entity_from_identifier (i.out + a_zone.mnemonic.out,today,xrf_zone)
			end
		end

	zonal_xrfs : ROSE_MULTI_KEYED_TABLE[ZONAL_XRF_RECORD] is
			-- FIXME shared cross-context!
		once
			create Result.make(10000)
		end

	zonal_xrf_records : ROSE_MULTI_KEYED_TABLE[ZONAL_XRF_RECORD] is
			-- FIXME shared cross-context!
		once
			create Result.make(10000)
		end

	--expression_anchor : ROSE_ENTITY_DATE[ENTITY] is deferred end

end
