note

	description:

		"Labeled two dimensional object"

	author: "Mark Howard"
	last_editor: "$Author: bschoeller $"
	date: "$Date: 2014/07/02 $"
	revision: "$Revision: #4 $"
	path: "$File: //rose/source/kernel/infrastructure/math/labeled_2d.e $"

deferred class
	LABELED_2D
inherit
	ROSE_CONSTANTS

feature --Access
	name : STRING
	set_name(a_name : STRING) is
			--Set the anme
		do
			name := a_name
		end


	height : INTEGER is
			-- Number of rows
		deferred
		ensure
			non_negative_height: Result >= 0
		end

	width : INTEGER is
			-- Number of columns
		deferred
		ensure
			non_negative_width: Result >= 0
		end
	row_names : ROSE_LINEAR_ARRAY[STRING]
	set_row_names(a_names : ARRAY[STRING])
			--Set the row names
		do
			if a_names /= Void then
				row_names ?= a_names
				if row_names = Void then
					create row_names.make_from_array(a_names)
				end
			end
		end

	column_names : ROSE_LINEAR_ARRAY[STRING]
	set_column_names(a_names : ARRAY[STRING])
			--Set the column names
		do
			if a_names /= Void then
				column_names ?= a_names
				if column_names = Void then
					create column_names.make_from_array(a_names)
				end
			end
		end
	safe_column_names:ROSE_LINEAR_ARRAY[STRING] is
		--column names if not provided
		do
			if column_names=void then
				result:=default_names ("column", width)
			else
				Result:=column_names
			end
		end
	safe_row_names:ROSE_LINEAR_ARRAY[STRING] is
		--column names if not provided
		do
			if row_names = Void then
				Result := default_names ("row", height)
			else
				Result:=row_names
			end
		end

	copy_names(a_matrix : ABSTRACT_READABLE_MATRIX) is
		--get names from another matrix
		require
			matrix_not_void : a_matrix /= void
		do
			if name /= Void then
				a_matrix.set_name(name)
			end
			if column_names /= Void then
				a_matrix.set_column_names(column_names)
			end
			if row_names /= Void then
				a_matrix.set_row_names(row_names)
			end
		end

	copy_names_clean (a_matrix: ABSTRACT_READABLE_MATRIX) is
			-- Copy names from another matrix without preserving reference links
		require
			matrix_not_void: a_matrix /= Void
		local
			l_name: STRING
			l_column_names, l_row_names: ROSE_LINEAR_ARRAY [STRING]
		do
			if name /= Void then
				create l_name.make_from_string (name)
				a_matrix.set_name (l_name)
			end
			if column_names /= Void then
				create l_column_names.make_from_array (column_names)
				a_matrix.set_column_names (l_column_names)
			end
			if row_names /= Void then
				create l_row_names.make_from_array (row_names)
				a_matrix.set_row_names (l_row_names)
			end
		end


	set_default_column_names is
	 	-- Default name of column is "column"
			do
				set_column_names(default_names("column",width))
			end

	set_default_row_names is
		-- Default name of row is "row"
			do
				set_row_names (default_names ("row", height))
			end

	default_names (a_header: STRING; a_count: INTEGER): ROSE_LINEAR_ARRAY[STRING] is
			-- Generated names consisting of `a_header' followed by a numeral
		require
			a_header_not_void: a_header /= void
		local
			i: INTEGER
		do
			create Result.make (1, a_count)
			from
				i := 1
			until
				i > a_count
			loop
				Result.put(a_header + "_" + i.out, i)
				i := i + 1
			end
		end

	column_indices(a_names:ARRAY[STRING]):ROSE_LINEAR_ARRAY[INTEGER] is
			-- Column indices of `Current' for those columns with names matching `a_names'
		local
			i,j,k:INTEGER
		do
			k:=a_names.count
			create result.make(1,k)
			from i:=1 until i>k loop
				j:=column_index(a_names.item(i))
				if j>0 then
					result.put(j,i)
				else
					print("can not get index for "+a_names.item(i)+"%N")
				end
				i:=i+1
			end
		end

	column_indices_with_start_check (a_names: ARRAY [STRING]): ROSE_LINEAR_ARRAY [INTEGER]
			-- Column indices of `Current' for those columns with names matching `a_names',
			-- with additional check for match on starting characters in `column_names'
		require
			a_names_not_void: a_names /= Void
			a_names_contains_no_void: not a_names.has (Void)
			column_names_exist: column_names /= Void
		local
			i, j, k: INTEGER
			l_substring: STRING
		do
			k := a_names.count
			create Result.make (1, k)
			from
				i := 1
			until
				i > k
			loop
				j := column_index (a_names [i])
				if j > 0 then
					Result [i] := j
				else
					from
						column_names.start
					until
						column_names.off or l_substring /= Void
					loop
						if column_names.current_item.starts_with (a_names [i]) then
							l_substring := column_names.current_item.string
							j := column_index (l_substring)
						end
						column_names.forth
					end
					if j > 0 then
						Result [i] := j
					else
						print ("Cannot retrieve index for " + a_names [i] + "%N")
					end
				end
				i := i + 1
			end
		end

	row_indices (a_names:ARRAY[STRING]):ROSE_LINEAR_ARRAY[INTEGER] is
			-- Row indices corresponding to `a_names'
		require
			a_names_not_void: a_names /= Void
		local
			i:INTEGER
		do
			create result.make(1,a_names.count)
			from i:=1 until i>a_names.count loop
				result.put(row_index(a_names.item (i)),i)
				i:=i+1
			end
		end

	row_by_name (a_name: STRING): VECTOR
			-- Row with 'a_name'
		local
			k: INTEGER
		do
			k := row_index (a_name)
			if k > 0 then
				Result := row (k)
				Result.set_names (column_names)
			else
				print ("can not get index for " + a_name + "%N")
			end
		end

	row_index (a_name: STRING): INTEGER is
			-- Column index corresponding to `a_name'
		local
			l_names: ROSE_LINEAR_ARRAY [STRING]
			i: INTEGER
		do
			l_names := row_names
			if l_names /= Void then
				i := l_names.index
				l_names.linear_search (a_name)
				Result := l_names.index
				l_names.go_i_th (i)
			end
		end

	item_by_name (a_row_name, a_column_name: STRING): like item
			-- Item at 'a_row_name' and 'a_column_name'
		local
			i, j: INTEGER
		do
			i := row_index (a_row_name)
			j := column_index (a_column_name)
			if i > 0 and j > 0 then
				Result := item (i, j)
			else
				Result := constants.missing_double
			end
		end

	item alias "[]" (i,j : INTEGER) : DOUBLE
			--i,jth item
		require
			valid_i: i >= 1 and i <= height
			valid_j: j >= 1 and j <= width
		deferred
		end
	row (i: INTEGER): VECTOR is
			-- I-th row as a vector
		require
			row_in_bounds: 1 <= i and i <= height
		deferred
		ensure
			result_not_void: Result /= Void
			correct_row: row_matches (Result, i)
		end
	column (j: INTEGER): VECTOR
			-- I-th column as a vector
		require
			column_in_bounds: 1 <= j and j <= width
		deferred
		ensure
			result_not_void: Result /= Void
		end
	column_index (a_name: STRING): INTEGER is
			-- Column index corresponding to `a_name'
		local
			l_names: ROSE_LINEAR_ARRAY [STRING]
			i: INTEGER
		do
			l_names := column_names
			if l_names /= Void then
				i := l_names.index
				l_names.linear_search (a_name)
				Result := l_names.index
				l_names.go_i_th (i)
			end
		end

	column_by_name (a_name: STRING): detachable VECTOR is
			-- Column of `Current' corresponding to that with name matching `a_name'
		local
			k: INTEGER
		do
			k := column_index (a_name)
			if k > 0 then
				Result:= column (k)
				Result.set_name (a_name)
			end
		end

	selected_columns_by_names(a_names : ARRAY[STRING]) : MATRIX is
			-- selected columns by names
		local
			i, l_col_ix : INTEGER
			l_name : STRING
			l_vector : VECTOR
		do
			if a_names /= Void and then a_names.count > 0 then
				create Result.make(height, a_names.count)
				from i := 1 until i > a_names.count loop
					l_name := a_names[a_names.lower + i - 1]
					l_col_ix := column_index (l_name)
					if l_col_ix > 0 then
						l_vector := column (l_col_ix)
						Result.put_column_vector (l_vector, i)
					end
					i := i + 1
				end
				Result.set_row_names(row_names)
				Result.set_column_names (a_names)
			end
		end

	column_matches(a_vector : ABSTRACT_READABLE_VECTOR; j: INTEGER): BOOLEAN is
			-- Is `a_vector' equal to column `j'?
		require
			a_vector_not_void: a_vector /= Void
		local
			i: INTEGER
		do
			Result := a_vector.count = height
			from i := 1 until i > height or not Result loop
				Result := ieee_is_eq(a_vector[i],Current [i, j])
				i := i + 1
			end
		ensure
			correct_height: Result implies a_vector.count = height
		end

	row_matches (a_vector: ABSTRACT_READABLE_VECTOR; i: INTEGER): BOOLEAN is
			-- Is `a_vector' equal to row `i', treating NaNs as equal to each other?
		require
			a_vector_not_void: a_vector /= Void
		local
			j: INTEGER
		do
			Result := a_vector.count = width
			from j := 1 until j > width or not Result loop
				Result := ieee_is_eq(a_vector[j],Current [i, j])
				j := j + 1
			end
		ensure
			correct_width: Result implies a_vector.count = width
		end
end
