indexing

	description: "Routines that assist writing assertions about GUI layout"

	author:  "Colin Adams"
	last_editor: "$Author: it $"
	date: "$Date: 2008/12/01 $"
	revision: "$Revision: #2 $"
	path: "$File: //rose/source/kernel/infrastructure/widgets/layout_assertion_routines.e $"

class LAYOUT_ASSERTION_ROUTINES

feature -- Access

	is_first (a_widget: EV_CONTAINABLE; a_container: EV_DYNAMIC_LIST [EV_CONTAINABLE]): BOOLEAN is
			-- Is `a_widget' first child of `a_container'?
		require
			a_widget_not_void: a_widget /= Void
			a_container_not_void: a_container /= Void
		do
			Result := a_container.index_of (a_widget, 1) = 1
		ensure
			definition: Result = (a_container.index_of (a_widget, 1) = 1)
		end

	is_last (a_widget: EV_CONTAINABLE; a_container: EV_DYNAMIC_LIST [EV_CONTAINABLE]): BOOLEAN is
			-- Is `a_widget' last child of `a_container'?
		require
			a_widget_not_void: a_widget /= Void
			a_container_not_void: a_container /= Void
		local
			l_count: INTEGER
		do
			l_count := a_container.count
			Result := a_container.index_of (a_widget, 1) = l_count and l_count > 0
		ensure
			definition: Result = (a_container.index_of (a_widget, 1) = a_container.count and not a_container.is_empty)
		end

	is_after (a_widget, a_anchor: EV_CONTAINABLE; a_container: EV_DYNAMIC_LIST [EV_CONTAINABLE]): BOOLEAN is
			-- Is `a_widget' after `a_anchor' in `a_container'?
		require
			a_widget_not_void: a_widget /= Void
			a_anchor_not_void: a_anchor /= Void
			a_container_not_void: a_container /= Void
		local
			l_position: INTEGER
		do
			l_position := a_container.index_of (a_anchor, 1)
			Result := a_container.index_of (a_widget, 1) > l_position and l_position > 0
		ensure
			definition: Result = (a_container.index_of (a_widget, 1) > a_container.index_of (a_anchor, 1) and a_container.index_of (a_anchor, 1) > 0)
		end

	is_immediately_after (a_widget, a_anchor: EV_CONTAINABLE; a_container: EV_DYNAMIC_LIST [EV_CONTAINABLE]): BOOLEAN is
			-- Is `a_widget' after `a_anchor' in `a_container', with no intervening item?
		require
			a_widget_not_void: a_widget /= Void
			a_anchor_not_void: a_anchor /= Void
			a_container_not_void: a_container /= Void
		local
			l_position: INTEGER
		do
			l_position := a_container.index_of (a_anchor, 1)
			Result := a_container.index_of (a_widget, 1) = l_position + 1 and l_position > 0
		ensure
			definition: Result = (a_container.index_of (a_widget, 1) = a_container.index_of (a_anchor, 1) + 1 and a_container.index_of (a_anchor, 1) > 0)
		end

	is_before (a_widget, a_anchor: EV_CONTAINABLE; a_container: EV_DYNAMIC_LIST [EV_CONTAINABLE]): BOOLEAN is
			-- Is `a_widget' before `a_anchor' in `a_container'?
		require
			a_widget_not_void: a_widget /= Void
			a_anchor_not_void: a_anchor /= Void
			a_container_not_void: a_container /= Void
		local
			l_position: INTEGER
		do
			l_position := a_container.index_of (a_widget, 1)
			Result :=  l_position > 0 and l_position < a_container.index_of (a_anchor, 1)
		ensure
			definition: Result = (a_container.index_of (a_widget, 1) > 0 and a_container.index_of (a_widget, 1) < a_container.index_of (a_anchor, 1))
		end

	is_immediately_before (a_widget, a_anchor: EV_CONTAINABLE; a_container: EV_DYNAMIC_LIST [EV_CONTAINABLE]): BOOLEAN is
			-- Is `a_widget' before `a_anchor' in `a_container', with no intervening item?
		require
			a_widget_not_void: a_widget /= Void
			a_anchor_not_void: a_anchor /= Void
			a_container_not_void: a_container /= Void
		local
			l_position: INTEGER
		do
			l_position := a_container.index_of (a_widget, 1)
			Result := l_position > 0 and l_position = a_container.index_of (a_anchor, 1) - 1
		ensure
			definition: Result = (a_container.index_of (a_widget, 1) > 0 and a_container.index_of (a_widget, 1) = a_container.index_of (a_anchor, 1) - 1)
		end

	is_top (a_widget: EV_CONTAINABLE; a_container: EV_VERTICAL_BOX): BOOLEAN is
			-- Is `a_widget' displayed topmost in `a_container'?
		require
			a_widget_not_void: a_widget /= Void
			a_container_not_void: a_container /= Void
		do
			Result := is_first (a_widget, a_container)
		ensure
			definition: Result = is_first (a_widget, a_container)
		end

	is_bottom (a_widget: EV_CONTAINABLE; a_container: EV_VERTICAL_BOX): BOOLEAN is
			-- Is `a_widget' displayed bottommost in `a_container'?
		require
			a_widget_not_void: a_widget /= Void
			a_container_not_void: a_container /= Void
		do
			Result := is_last (a_widget, a_container)
		ensure
			definition: Result = is_last (a_widget, a_container)
		end

	is_leftmost (a_widget: EV_CONTAINABLE; a_container: EV_HORIZONTAL_BOX): BOOLEAN is
			-- Is `a_widget' displayed leftmost in `a_container'?
		require
			a_widget_not_void: a_widget /= Void
			a_container_not_void: a_container /= Void
		do
			Result := is_first (a_widget, a_container)
		ensure
			definition: Result = is_first (a_widget, a_container)
		end

	is_rightmost (a_widget: EV_CONTAINABLE; a_container: EV_HORIZONTAL_BOX): BOOLEAN is
			-- Is `a_widget' displayed rightmost in `a_container'?
		require
			a_widget_not_void: a_widget /= Void
			a_container_not_void: a_container /= Void
		do
			Result := is_last (a_widget, a_container)
		ensure
			definition: Result = is_last (a_widget, a_container)
		end

	is_below (a_widget, a_anchor: EV_CONTAINABLE; a_container: EV_VERTICAL_BOX): BOOLEAN is
			-- Is `a_widget' displayed below `a_anchor' in `a_container'?
			require
			a_widget_not_void: a_widget /= Void
			a_anchor_not_void: a_anchor /= Void
			a_container_not_void: a_container /= Void
		do
			Result := is_after (a_widget, a_anchor, a_container)
		ensure
			definition: Result = is_after (a_widget, a_anchor, a_container)
		end

	is_immediately_below (a_widget, a_anchor: EV_CONTAINABLE; a_container: EV_VERTICAL_BOX): BOOLEAN is
			-- Is `a_widget' displayed immediately below `a_anchor' in `a_container'?
			require
			a_widget_not_void: a_widget /= Void
			a_anchor_not_void: a_anchor /= Void
			a_container_not_void: a_container /= Void
		do
			Result := is_immediately_after (a_widget, a_anchor, a_container)
		ensure
			definition: Result = is_immediately_after (a_widget, a_anchor, a_container)
		end

	is_right_of (a_widget, a_anchor: EV_CONTAINABLE; a_container: EV_HORIZONTAL_BOX): BOOLEAN is
			-- Is `a_widget' displayed further right than `a_anchor' in `a_container'?
			require
			a_widget_not_void: a_widget /= Void
			a_anchor_not_void: a_anchor /= Void
			a_container_not_void: a_container /= Void
		do
			Result := is_after (a_widget, a_anchor, a_container)
		ensure
			definition: Result = is_after (a_widget, a_anchor, a_container)
		end

	is_immediately_right_of (a_widget, a_anchor: EV_CONTAINABLE; a_container: EV_HORIZONTAL_BOX): BOOLEAN is
			-- Is `a_widget' displayed immediately right of `a_anchor' in `a_container'?
			require
			a_widget_not_void: a_widget /= Void
			a_anchor_not_void: a_anchor /= Void
			a_container_not_void: a_container /= Void
		do
			Result := is_immediately_after (a_widget, a_anchor, a_container)
		ensure
			definition: Result = is_immediately_after (a_widget, a_anchor, a_container)
		end

	is_left_of (a_widget, a_anchor: EV_CONTAINABLE; a_container: EV_HORIZONTAL_BOX): BOOLEAN is
			-- Is `a_widget' displayed further left than `a_anchor' in `a_container'?
			require
			a_widget_not_void: a_widget /= Void
			a_anchor_not_void: a_anchor /= Void
			a_container_not_void: a_container /= Void
		do
			Result := is_before (a_widget, a_anchor, a_container)
		ensure
			definition: Result = is_before (a_widget, a_anchor, a_container)
		end

	is_immediately_left_of (a_widget, a_anchor: EV_CONTAINABLE; a_container: EV_HORIZONTAL_BOX): BOOLEAN is
			-- Is `a_widget' displayed immediately left of `a_anchor' in `a_container'?
			require
			a_widget_not_void: a_widget /= Void
			a_anchor_not_void: a_anchor /= Void
			a_container_not_void: a_container /= Void
		do
			Result := is_immediately_before (a_widget, a_anchor, a_container)
		ensure
			definition: Result = is_immediately_before (a_widget, a_anchor, a_container)
		end

end
