note

	description:

		"ROSE_CONTEXT version of PERSISTENT_LISTING"

	author: "Mark Howard"
	last_editor: "$Author: it $"
	date: "$Date: 2015/02/23 $"
	revision: "$Revision: #248 $"
	path: "$File: //rose/source/kernel/investment_entities/listing.e $"

class
	LISTING

inherit

	INVESTMENT_ENTITY
		undefine
			peer_anchor,
			corelated_entities,
			corelationships,
			id_string,
			descriptive_name,
			name_identifier_keys,
			name_identifier_key
		redefine
			make,
			make_with_zone_xrf,
			ur_targets
		end

	SUPPLIER_LISTING
		undefine
			make,
			make_context,
			ur_targets,
			corelated_entities,
			corelationships,
			portfolio_to_listing_co_history,
			debug_output
		redefine
			ur_price_target
		end

	RMDAT_INSTRUMENT
		undefine
			is_equal
		redefine
			last_price_record,
			last_price_record_160,
			price_record
		end

	LEGACY_INSTRUMENT
		undefine
			descriptive_name,
			name_identifier_keys,
			name_identifier_key
		redefine
			cusip_code,
			isin_code,
			tse_code
		end

	ROSE_SHARED_GLOBAL
		undefine
			is_equal
		end

	ROSE_UNITS_ENVIRONMENT
		undefine
			is_equal
		end

create
	make,
	make_with_zone_xrf,
	make_anonymous

feature {LISTING} -- Inititialization

	make (a_context: like context;a_category_id: INTEGER;a_handle: INTEGER;a_start_date,a_end_date: ROSE_DATE) is
			-- make
		local
			l_rmdat_map: RMDAT_LISTING_MAP
		do
			Precursor{INVESTMENT_ENTITY} (a_context,a_category_id,a_handle,a_start_date,a_end_date)
			create l_rmdat_map
			rmdat_listing_map := l_rmdat_map.mapped_listing (Current)
		end

	make_with_zone_xrf (a_context: like context;a_category_id: INTEGER;a_handle: INTEGER;a_start_date,a_end_date: ROSE_DATE;a_zone: ZONE[LOCATION];a_xrf: INTEGER) is
			-- make with zone xrf
		local
			l_rmdat_map: RMDAT_LISTING_MAP
		do
			Precursor{INVESTMENT_ENTITY} (a_context,a_category_id,a_handle,a_start_date,a_end_date,a_zone,a_xrf)
			create l_rmdat_map
			rmdat_listing_map := l_rmdat_map.mapped_listing (Current)
		end

feature -- Dates	

	interval_market_dates (a_start, a_end: ROSE_DATE): ROSE_LINEAR_ARRAY[ROSE_DATE] is
			--market dates between a_start and a_end
		local
			l_first_and_last_dates: PAIR[ROSE_DATE, ROSE_DATE]
			l_first, l_last: ROSE_DATE
		do
			if a_start /= Void and a_end /= Void then
				l_first_and_last_dates := first_and_last_dates
				if l_first_and_last_dates /= Void and then (l_first_and_last_dates.first /= Void and l_first_and_last_dates.second /= Void) then
					l_first := l_first_and_last_dates.first
					l_last := l_first_and_last_dates.second
					if l_first < a_start then
						l_first := a_start
					end
					if l_last > a_end then
						l_last := a_end
					end
					if l_first <= l_last and then country /= Void then
						Result := country.market_day_interval_dates(l_first, l_last)
					end
				end
			end
		end

	available_market_dates: ROSE_LINEAR_ARRAY[ROSE_DATE] is
			--market dates of the country of the listing between first and last date for the listing
		local
			l_first_and_last_dates: PAIR[ROSE_DATE, ROSE_DATE]
		do
			l_first_and_last_dates := first_and_last_dates
			if l_first_and_last_dates /= Void and then (l_first_and_last_dates.first /= Void and l_first_and_last_dates.second /= Void) then
				if country /= Void then
					Result := country.market_day_interval_dates(l_first_and_last_dates.first, l_first_and_last_dates.second)
				end
			end
		end

	first_and_last_dates: PAIR [ROSE_DATE, ROSE_DATE] is
			-- first and last dates
		do
			if first_and_last_dates_saved=Void then
				if zone /= Void then
					first_and_last_dates_saved := zone.market_data.first_and_last_dates (xrf_code)
				end

				if first_and_last_dates_saved=Void then
					create first_and_last_dates_saved.make (Void, Void)
				end
			end

			Result := first_and_last_dates_saved
		ensure
			first_and_last_dates_not_void: Result /= Void
		end

	first_date: ROSE_DATE is
			-- Cached first available market date
		require
			first_and_last_dates_not_void: first_and_last_dates /= Void
		do
			Result := first_and_last_dates.first
		ensure
			definition: Result = first_and_last_dates_saved.first
		end

	last_date: ROSE_DATE is
			-- Cached last available market date
		require
			first_and_last_dates_not_void: first_and_last_dates /= Void
		do
			Result := first_and_last_dates.second
		ensure
			definition: Result = first_and_last_dates_saved.second
		end

	first_and_last_dates_saved: PAIR [ROSE_DATE, ROSE_DATE]
			-- Cached first and last available market dates

feature {NONE} -- Composite records

	composite_records: ROSE_CACHE[COMPOSITE_RECORD] is
			--composite record cache
		once
			create Result.make(100000,10000)
		end

	composite_records_cross_sectional_reader (a_date: ROSE_DATE): SIMPLE_PREDICATE[COMPOSITE_RECORD] is
			-- sql predicate for reading composite records
		require
			valid_date: a_date /= Void
		do
			create Result.make (servers.make_from_repository("composites_common"),
							   "select a.brimco + a.sec as brimsec,a.brimco,a.trade_date,capital_balance,revenue_balance, " +
							   "expense_balance,outstanding_shares,currency,published_nav from composite_assets_nav as a " +
							   "inner join " +
							   "(select brimco,sec,max (trade_date) as trade_date from composite_assets_nav where trade_date <= '" + a_date.ccyymmdd.out + "' group by brimco,sec) as b " +
							   "on " +
							   "a.brimco = b.brimco and a.sec = b.sec and a.trade_date = b.trade_date")
		end

	composite_record_interval_table: HASH_TABLE[ROSE_INTERVAL_PARTITION[COMPOSITE_RECORD],STRING] is
			-- Composite record interval cache.
		once
			create Result.make(5000)
		end

	composite_record_intervals: ROSE_INTERVAL_PARTITION[COMPOSITE_RECORD]
		do
			Result := composite_record_interval_table.item (security.brim_code (void))
		end

	add_composite_record (a_interval: ROSE_CLOSED_OPEN_INTERVAL[COMPOSITE_RECORD];a_record: COMPOSITE_RECORD) is
		local
			l_date,l_interim: ROSE_DATE
			l_record: COMPOSITE_RECORD
			l_brim_code: STRING
			l_composite_partition: ROSE_INTERVAL_PARTITION[COMPOSITE_RECORD]
		do
			l_record := a_record.twin
			create l_date.make_ccyymmdd (a_record.trade_date)
			l_brim_code := security.brim_code_key (l_date)
			if a_interval.start_date.ccyymmdd = a_record.trade_date then
				composite_records.put (l_record,[a_record.brimsec,a_interval.start_date])
			else
				if l_date.plus_n_date(10) < a_interval.start_date then -- more than 10 days old put in interval list
					l_composite_partition := composite_record_interval_table.item (a_record.brimsec)
					if l_composite_partition = Void then
						create l_composite_partition.make
						composite_record_interval_table.put (l_composite_partition,a_record.brimsec)
					end
					l_composite_partition.extend (l_record,l_date)
				else
					from l_interim := l_date until l_interim > a_interval.start_date loop
						composite_records.put (l_record,[a_record.brimsec,l_interim])
						l_interim := l_interim.plus_n_date (1)
					end
				end
			end
			if l_brim_code /= Void and then l_record.brimsec.is_equal (l_brim_code) then
				a_interval.make (l_record,l_date,a_interval.end_date,0.0)
			end
		end

feature -- Holdings

	is_held_ever: BOOLEAN is
			-- Has `Current' ever been held?
		do
			Result := is_held_during (create {ROSE_DATE}.make_min_date , create {ROSE_DATE}.make_max_date)
		end

	is_held_during (a_start_date, a_end_date : ROSE_DATE) : BOOLEAN is
			-- Weighted related portfolio history during [`a_start_date'. `a_end_date']
			-- Note that the data is stored as closed-open intervals but `a_start_date' and `a_end_date' are assumed to form a closed-closed interval
			-- Therefore we add a day to `a_end_date' and treat the interval as closed-open
		require
			valid_dates : a_start_date /= Void and then a_end_date /= Void and then a_start_date <= a_end_date
		local
			l_brimsec, l_brimsym, l_exchange: STRING_8
			i: INTEGER_32
			l_record: HOLDINGS_RECORD
			l_portfolio: like context.portfolio_template
			l_end_date : ROSE_DATE
		do
			l_brimsym := brim_code_most_recent_key (today)
			if l_brimsym /= Void then
				l_exchange := l_brimsym.substring (16, 18)
				l_brimsec := l_brimsym.substring (1, 16)
				l_end_date := a_end_date.twin
				l_end_date.day_add (1)
				l_end_date := l_end_date.min (l_end_date.max_date)
				holdings_predicate.retrieve (Void, <<l_brimsec, a_start_date, l_end_date>>)
				from i := 1 until i > holdings_predicate.array.count or Result loop
					l_record := holdings_predicate.array.item (i)
					if l_record.exchange.is_equal (l_exchange) then
						l_portfolio := portfolio_from_name (l_record.portfolio)
						if l_portfolio /= Void then
							Result := l_record.applicable_from < l_portfolio.end_date and l_record.applicable_to > l_portfolio.start_date
						end
					end
					i := i + 1
				end
			end
		end

	is_held_at (a_date: ROSE_DATE): BOOLEAN is
			-- Is current held at `a_date'
		local
			l_date: ROSE_DATE
		do
			l_date := a_date
			if l_date = Void then
				create l_date.make_today
			end
			Result := not holdings_at (a_date).is_empty
		end

	long_holdings_value_at (a_date: ROSE_DATE; a_currency: CURRENCY): DOUBLE is
			-- Value of long holdings at 'a_date' in 'a_currency', 'rmdat_currency' if 'a_currency' is void
		require
			valid_date: a_date /= Void
		local
			l_holdings: ROSE_ENTITY_HISTORY [like context.portfolio_template]
			l_price, l_rate: DOUBLE
		do
			Result := constants.missing_double
			l_holdings := holdings_at (a_date)
			if not l_holdings.is_empty then
				l_price := last_price (a_date)
				if is_defined_double (l_price) then
					Result := 0.0
					from l_holdings.start until l_holdings.off loop
						if is_defined_double (l_holdings.current_item.weight) and then l_holdings.current_item.weight > 0 then
							Result := Result + l_price * l_holdings.current_item.weight
						end
						l_holdings.forth
					end
					if a_currency /= Void then
						l_rate := a_currency.exchange_rate (rmdat_currency (a_date), a_date)
						if is_defined_double (l_rate) then
							Result := Result * l_rate
						else
							Result := constants.missing_double
						end
					end
				end
			end
		end

	short_holdings_value_at (a_date: ROSE_DATE; a_currency: CURRENCY): DOUBLE is
			-- Value of short holdings at 'a_date' in 'a_currency', 'rmdat_currency' if 'a_currency' is void
		require
			valid_date: a_date /= Void
		local
			l_holdings: ROSE_ENTITY_HISTORY [like context.portfolio_template]
			l_price, l_rate: DOUBLE
		do
			Result := constants.missing_double
			l_holdings := holdings_at (a_date)
			if not l_holdings.is_empty then
				l_price := last_price (a_date)
				if is_defined_double (l_price) then
					Result := 0.0
					from l_holdings.start until l_holdings.off loop
						if is_defined_double (l_holdings.current_item.weight) and then l_holdings.current_item.weight < 0 then
							Result := Result + l_price * l_holdings.current_item.weight
						end
						l_holdings.forth
					end
					if a_currency /= Void then
						l_rate := a_currency.exchange_rate (rmdat_currency (a_date), a_date)
						if is_defined_double (l_rate) then
							Result := Result * l_rate
						else
							Result := constants.missing_double
						end
					end
				end
			end
		end

	holdings_at (a_date: ROSE_DATE): ROSE_ENTITY_HISTORY [like context.portfolio_template] is
			-- Weighted portfolio history or `Current' at `a_date'
		do
			Result := holdings_during (a_date, a_date)
		ensure
			valid_result: Result /= Void
		end

	holdings_during (a_start_date, a_end_date : ROSE_DATE) : ROSE_ENTITY_HISTORY [like context.portfolio_template] is
			-- Weighted related portfolio history during [`a_start_date'. `a_end_date']
			-- Note that the data is stored as closed-open intervals but `a_start_date' and `a_end_date' are assumed to form a closed-closed interval
			-- Therefore we add a day to `a_end_date' and treat the interval as closed-open
		require
			valid_dates : a_start_date /= Void and then a_end_date /= Void and then a_start_date <= a_end_date
		local
			l_brimsec, l_brimsym, l_exchange: STRING_8
			i: INTEGER_32
			l_record: HOLDINGS_RECORD
			l_interval: ENTITY_INTERVAL [like context.portfolio_template]
			l_portfolio: like context.portfolio_template
			l_identifier: ENTITY_IDENTIFIER
			l_end_date : ROSE_DATE
		do
			create Result.make_empty
			l_brimsym := brim_code_most_recent_key (today)
			create l_identifier.make (context, 46)
			if l_brimsym /= Void then
				l_exchange := l_brimsym.substring (16, 18)
				l_brimsec := l_brimsym.substring (1, 16)
				l_end_date := a_end_date.twin
				l_end_date.day_add (1)
				l_end_date := l_end_date.min (l_end_date.max_date)
				holdings_predicate.retrieve (Void, <<l_brimsec, a_start_date, l_end_date>>)
				if not holdings_predicate.success then
					(create {EXCEPTIONS}).raise("LISTING.holdings_during - problem executing holdings_predicate.retrieve")
				else
					from i := holdings_predicate.array.lower until i > holdings_predicate.array.upper loop
						l_record := holdings_predicate.array.item (i)
						if l_record.exchange.is_equal (l_exchange) then
							l_record.portfolio.prune_all (' ')
							if gcdb_portfolio_names [l_record.portfolio] /= Void then
								l_portfolio := context.portfolio_at (canonical_name (l_record.portfolio), Void)
								if l_portfolio = Void then
									print ("Holdings info may be incorrect!%NProblem in LISTING.portfolio_to_listing_co_history: %NUnable to create portfolio entity from canonical name: " + l_record.portfolio + "%N")
								end
								if l_portfolio /= Void then
									create l_interval.make (l_portfolio, l_record.applicable_from.max (l_portfolio.start_date), l_record.applicable_to.min (l_portfolio.end_date), l_record.units)
									if (l_interval.start_date < l_end_date.date) and (l_interval.end_date.date > a_start_date.date) then
										Result.extend (l_interval)
									end
								end
							end
						end
						i := i + 1
					end
				end
			end
		ensure
			result_not_void : Result /= Void
		end

	portfolio_from_name (a_name: STRING): like context.portfolio_template is
			-- Portfolio from 'a_name'
		require
			valid_name: a_name /= Void
		do
			a_name.prune_all (' ')
			Result := context.portfolio_at (canonical_name (a_name), Void)
		end

feature {NONE} -- Holdings Implementation
	gcdb_portfolio_names : HASH_TABLE [STRING, STRING] is
			-- Current portfolio_names in gcdb
		local
			l_predicate : STRING_PREDICATE
			l_server : SQL_DATA_SERVER
			l_string : STRING
		once
			l_server := servers.make_from_repository (k_global_client_logical)
			create l_predicate.make (l_server, "SELECT DISTINCT code FROM portfolio WHERE code IS NOT NULL")
			if not l_predicate.success then
				(create {EXCEPTIONS}).raise("LISTING.gcdb_portfolio_names - problem creating predicate")
			else
				l_predicate.retrieve (Void, Void)
				if not l_predicate.success then
					(create {EXCEPTIONS}).raise("LISTING.gcdb_portfolio_names - problem with l_predicate.retrieve")
				else
					create Result.make (1)
					from l_predicate.array.start until l_predicate.array.off loop
						l_string := l_predicate.array.current_item
						l_string.prune_all (' ')
						Result [l_string] := l_string
						l_predicate.array.forth
					end
				end
			end
		end

	holdings_query_text: STRING is
			-- Holdings query text
		once
			Result := "SELECT * FROM dbo.TRADED_HOLDINGS WHERE (INSTRUMENT = ?) " +
						"AND (? < APPLICABLE_TO) AND (? > APPLICABLE_FROM) " +
						"AND UNITS <> 0 " +
						"ORDER BY PORTFOLIO, INSTRUMENT, APPLICABLE_FROM"
		end

	holdings_predicate: COLLECTING_PREDICATE[HOLDINGS_RECORD] is
			-- Retrieves portfolio_to_listing weights
		once
			create Result.make (servers.gedb_fg, holdings_query_text)
		end

	k_global_client_logical : STRING is "global_client_fg"

feature -- Access

	rmdat_listing_map: LISTING

	composites: HASH_TABLE [STRING,LISTING] is
			-- List of composite listings with the is_composite flag set
		local
			l_reader: STRING_PREDICATE
			l_security: SECURITY
			l_listing: LISTING
			i: INTEGER
		once
			create l_reader.make (servers.make_from_repository("composites_common"),"select distinct brimco + sec from composite_assets")
			if not l_reader.success then
				(create {EXCEPTIONS}).raise("listing.composites - problem creating predicate")
			else
				l_reader.retrieve (void,void)
				if not l_reader.success then
					(create {EXCEPTIONS}).raise("listing.composites - problem executing predicate")
				else
					create Result.make (l_reader.array.count * 2)
					from
						l_reader.array.start
					until
						l_reader.array.after
					loop
						if l_reader.array.current_item /= Void then
							l_security := context.security_at (attributes_brim_code (l_reader.array.current_item),today)
							if l_security /= void then
								from
									i := 1
								until
									i > l_security.listing_count
								loop
									l_listing := l_security.listing (i)
									l_listing.set_is_composite (True)
									Result.put (l_reader.array.current_item,l_listing)
									i := i + 1
								end
							end
						end
						l_reader.array.forth
					end
				end
			end
		end

	composite_record (a_date: ROSE_DATE): COMPOSITE_RECORD
			--Composite listing record for the security of this listing
		local
			l_interval: ROSE_CLOSED_OPEN_INTERVAL[COMPOSITE_RECORD]
			l_intervals: like composite_record_intervals
			l_brim_code: STRING
		do
			l_brim_code := security.brim_code (a_date)
			composite_records.search([l_brim_code,a_date])
			if composite_records.found then
				Result := composite_records.found_item
			else
				if composite_record_intervals /= void then
					l_interval := composite_record_intervals.interval (a_date)
				end
				if l_interval /= Void  then
					Result := l_interval.item
				else
					create l_interval.make (Void,a_date,a_date,0.0)
					--composite_records_cross_sectional_reader (a_date).apply(~add_composite_record (l_interval,?),void,<<a_date.ccyymmdd>>)
					composite_records_cross_sectional_reader (a_date).apply (agent add_composite_record (l_interval,?),Void,Void)
					Result := composite_records.item([l_brim_code,a_date])
 					if Result = void then
 						l_intervals := composite_record_intervals
 						if l_intervals /= Void then
 							l_interval := composite_record_intervals.interval (a_date)
 						end
						if l_interval /= Void  then
							Result := l_interval.item
						end
					end
				end
			end
		end

	is_composite: BOOLEAN is
			-- is Current object a composite?
		do
			composites.do_nothing --sets composite information for all listings as a side effect
			Result := is_composite_saved
		end

	is_composite_saved: BOOLEAN			-- See 'is_composite'.

	is_composite_listing: BOOLEAN is
			-- is Current object a composite?
		local
			l_key: STRING
		do
			l_key := attribute_at (attribute_ids.is_composite_type_id, end_date)
			if l_key /= Void then
				l_key.to_lower
				Result := l_key.is_equal("true")
			end
		end

	is_primary (a_date: ROSE_DATE): BOOLEAN is
			-- is primary
		local
			l_key: STRING
		do
			l_key := attribute_at (attribute_ids.primary_type_id, a_date)
			Result := l_key /= Void and then l_key.to_boolean
		end

	adjustment_history: ROSE_ADJUSTMENT_HISTORY[LISTING] is
			-- History of adjustments
		do
			if adjustment_history_saved = Void then
				--if zone_mnemonic.is_equal('J') or  zone_mnemonic.is_equal('S') then
				--	adjustment_history_saved := new_adjustment_history
				--else
				adjustment_history_saved := market_data.adjustment_history (xrf_code)
				--end
			end
			Result := adjustment_history_saved
		end

feature {NONE} -- Support for new_adjustment_history_os

	min_date: ROSE_DATE is
			-- minimum date
		once
			create Result.make_ccyymmdd(19000101)
		end

	max_date: ROSE_DATE is
			-- maximum date
		once
			create Result.make_ccyymmdd(99991231)
		end

	adjustment_from_mindate_to_marketstartdate (a_market_start_date: ROSE_DATE): ADJUSTMENT_INTERVAL[LISTING] is
			-- builds first adjustment from min date to market start date
		require
			a_start_date_non_void: a_market_start_date /= Void
		local
			l_end_date: ROSE_DATE
		do
			l_end_date := zone.ith_market_day_before (a_market_start_date, 1)
			create Result.make (Current, min_date, l_end_date, undefined_integer, undefined_integer)
		end

	adjustment_from_marketenddate_to_maxdate (a_market_end_date: ROSE_DATE): ADJUSTMENT_INTERVAL[LISTING] is
			-- build last adjument from market date end to max date
		require
			a_end_date_non_void: a_market_end_date /= Void
		local
			l_start_date: ROSE_DATE
		do
			l_start_date := zone.ith_market_day_after (a_market_end_date, 1)
			create Result.make (Current, l_start_date, max_date, undefined_integer, undefined_integer)
		end

feature -- Adjustments

	adjustment_anomalies: ROSE_LINEAR_ARRAY[STRING]
			-- collection of adjustment factor anomalies for migration of corporate actions	

	collect_anomaly (a_option: ROSE_COMPLEX_CAPITAL_ACTION) is
			-- collects anomaly from adjustment factor computations
		require
			option_not_void: a_option /= Void
		do
			if adjustment_anomalies = Void then
				create adjustment_anomalies.make_empty
			end
			adjustment_anomalies.extend (zone_mnemonic.out + "%T" + xrf_code.out + " "
				+ a_option.exdate.date.formatted_out ("yyyy/mm/dd") + " "
				+ a_option.exdate.date.formatted_out ("yyyy/mm/dd") + "%T"
				+ a_option.rmdat_adjustment_factor.out + "%TAnomaly")
			print ("Anomaly: undefined adjustment factor.%N")
		ensure
			adjustment_anomalies_not_void: adjustment_anomalies /= Void
		end

	new_adjustment_history_os:  ROSE_ADJUSTMENT_HISTORY[LISTING] is
			-- Returns the history of adjustments for this listing (contrast with 'adjustment_history').
			-- this is development code to test handle adjustments for S and O zone migration only
			-- do not call
		local
			l_before_price_start, l_rejected_price_start, l_rejected_price_end, l_rejected_factor_undefined: BOOLEAN
			l_option: ROSE_COMPLEX_CAPITAL_ACTION
			l_options: ROSE_LINEAR_ARRAY[ROSE_COMPLEX_CAPITAL_ACTION]
			l_adjustment: ADJUSTMENT_INTERVAL [LISTING]
			l_market_dates: PAIR[ROSE_DATE,ROSE_DATE]
			l_market_date_start, l_market_date_end: ROSE_DATE
			l_start_date: ROSE_DATE
			l_end_date: ROSE_DATE
		do
			create Result.make_empty
			if new_adjustment_history_saved = Void then
				l_before_price_start := False
				l_market_dates := zone.market_data.first_and_last_dates (xrf_code)
				if l_market_dates /= Void then
					l_market_date_start := l_market_dates.first
					l_market_date_end := l_market_dates.second
				end
				if l_market_date_end /= Void and l_market_date_end < today then
					l_adjustment := adjustment_from_marketenddate_to_maxdate (l_market_date_end)
					Result.add_interval (l_adjustment)
					create l_adjustment.make (Current, l_market_date_end, l_market_date_end, 1.0, 0)
				else
					create l_adjustment.make (Current, max_date, max_date, 1.0, 0.0)
				end
				l_options := capital_actions_between (min_date, Void)			-- Sorts ascending.  Be sure to call 'reversed' (below).
				if l_options = Void or else l_options.count = 0 or else l_market_dates = Void then
					if l_market_date_end = Void then
						l_end_date := max_date
					else
						l_end_date := l_market_date_end
					end
					if l_market_date_start = Void then
						print ("Anomaly: no market start date %N")
						create l_adjustment.make (Current, min_date, l_end_date, 1.0, 0.0)
						Result.add_interval (l_adjustment)
					else
						l_start_date := l_market_date_start
						create l_adjustment.make (Current, l_start_date, l_end_date, 1.0, 0.0)
						Result.add_interval (l_adjustment)
					end
				else
					l_options := l_options.reversed								-- Only call this when using 'capital_actions_between'.
					from l_options.start until l_options.after or l_before_price_start = True loop
						l_option := l_options.current_item
						l_rejected_price_start := False
						l_rejected_price_end := False
						l_rejected_factor_undefined := False
						if l_option.exdate < l_market_date_start then
							l_rejected_price_start := True
						end
						if l_option.exdate > l_market_date_end then
							l_rejected_price_end := True
						end
						if not is_defined_double (l_option.rmdat_adjustment_factor) then
							l_rejected_factor_undefined := True
							if l_rejected_price_start = False and l_rejected_price_end = False then
								collect_anomaly (l_option)
							end
						end
						if l_rejected_price_start = True then -- last add below.
							-- l_adjustment.set_start_date (l_market_date_start)
							-- Result.add_interval (l_adjustment)
							if l_market_date_start < l_adjustment.last_date then -- previous CA on pricing start date
								l_adjustment.set_start_date (l_market_date_start)
								Result.add_interval (l_adjustment)
							end
							l_before_price_start := True
						end
						if l_rejected_price_start = False and l_rejected_price_end = False then
							if l_options.index = l_options.upper then -- consolidate with below when reconciliation is done
								if l_option.rmdat_adjustment_factor = 1.0 then
									l_adjustment.set_start_date (l_market_date_start)
									Result.add_interval (l_adjustment)
								else
									l_adjustment.set_start_date (l_option.exdate)
									Result.add_interval (l_adjustment)
									if l_market_date_start < l_option.exdate then -- else, CA exdate is same as market start
										l_end_date := zone.ith_bank_day_before (l_option.exdate, 1)
										create l_adjustment.make (Current, l_market_date_start, l_end_date, l_adjustment.adjustment * l_option.rmdat_adjustment_factor , l_option.rmdat_adjustment_factor)
										Result.add_interval (l_adjustment)
									end
								end
							else
								l_start_date := l_option.exdate
								if l_option.rmdat_adjustment_factor = 1.0 then
									l_adjustment.set_start_date (l_start_date)
								else
									l_adjustment.set_start_date (l_option.exdate)
									Result.add_interval (l_adjustment)
									if l_market_date_start < l_option.exdate then -- else, CA exdate is same as market start
										l_end_date := zone.ith_bank_day_before (l_option.exdate, 1)
										create l_adjustment.make (Current, l_end_date, l_end_date, l_adjustment.adjustment * l_option.rmdat_adjustment_factor , l_option.rmdat_adjustment_factor)
									end
								end
							end
						end
						if l_options.index = l_options.upper then -- last CA is after end of pricing
							if l_rejected_price_end = True then
								l_adjustment.set_start_date (l_market_date_start)
								Result.add_interval (l_adjustment)
							end
						end
						l_options.forth
					end
				end
				if l_market_date_start = Void then
					print ("Anomaly: no market start date %N")
				else
					l_adjustment := adjustment_from_mindate_to_marketstartdate (l_market_date_start)
					Result.add_interval (l_adjustment)
				end
				Result.sort_by_start_date
				new_adjustment_history_saved := Result
			else
				Result := new_adjustment_history_saved
			end
		end

feature -- Adjustments

	new_adjustment_history: ROSE_ADJUSTMENT_HISTORY[LISTING] is
			-- Returns the history of adjustments for this listing (contrast with 'adjustment_history').
		local
			l_options: ROSE_LINEAR_ARRAY[ROSE_COMPLEX_CAPITAL_ACTION]
			l_adjustment: ADJUSTMENT_INTERVAL [LISTING]
			l_last_adjustment: ADJUSTMENT_INTERVAL [LISTING]
			l_start_date: ROSE_DATE
			l_min_date: ROSE_DATE
			l_end_date: ROSE_DATE
			l_rose_julian: INTEGER
			l_first_time: BOOLEAN
		do
			create Result.make_empty
			if new_adjustment_history_saved = Void then
				create l_min_date.make_ccyymmdd(19000101)
				--l_options := capital_action_options_after (l_min_date)		-- Sorts descending.  Do not call 'reversed' (below).
				l_options := capital_actions_between (l_min_date, Void)			-- Sorts ascending.  Be sure to call 'reversed' (below).
				if l_options = Void or else l_options.count = 0 then
					create l_adjustment.make (Current, l_min_date, create {ROSE_DATE}.make_ccyymmdd(99991231), 1.0 , 0.0)
					Result.add_interval (l_adjustment)
				else
					l_options := l_options.reversed								-- Only call this when using 'capital_actions_between'.
																				-- Never call this when using 'capital_action_options_after'.
					-- The following loop requires that 'l_options' be sorted by 'exdate' descending.
					from l_options.start
						if not is_defined_double (l_options.current_item.rmdat_adjustment_factor) then
							create l_adjustment.make (Current, l_options.current_item.exdate, create {ROSE_DATE}.make_ccyymmdd(99991231), undefined_integer, undefined_integer)
						else
							create l_adjustment.make (Current, l_options.current_item.exdate, create {ROSE_DATE}.make_ccyymmdd(99991231), 1.0 , 0.0)
						end
						l_last_adjustment := l_adjustment
						Result.add_interval (l_adjustment)
						l_first_time := True
					until l_options.after
					loop
						if l_options.index < l_options.upper then
							l_start_date := l_options.next_item.exdate	-- The start_date to use is the exdate of the next option.  Since
																		-- 'l_options' is descending by exdate, that's the previous option in time.
						else
							l_start_date := l_min_date					-- There is no previous (by time) option.
						end
						if (l_options.current_item.rmdat_adjustment_factor = 1.0 or
						    (not l_first_time and not is_defined_double(l_options.current_item.rmdat_adjustment_factor))) and
						   is_defined_double (l_last_adjustment.adjustment)
						then
							-- This option didn't affect adjustments so we can skip it but we must move the start date of the last
							-- added adjustment back to the exdate of the previous (by time) option.
							l_last_adjustment.set_start_date (l_start_date)	-- This changes the start_date of an adjustment already in Result.
						else
							l_rose_julian := l_options.current_item.exdate.rose_julian_day - 1
							l_end_date := zone.ith_market_day_before (l_options.current_item.exdate,1) -- perhaps do market_date back?
							if l_end_date.rose_julian_day = 1 and then
								-- This tries to handle issues with spotty calendars in S zone
								zone.market_data.hdp.jul_indexes.valid_index (l_rose_julian) then
								from until
									not zone.market_data.hdp.jul_indexes.valid_index (l_rose_julian) or
									zone.market_data.hdp.jul_indexes.item (l_rose_julian) > 0
								loop
									l_rose_julian := l_rose_julian - 1
								end
								if zone.market_data.hdp.jul_indexes.valid_index (l_rose_julian) then
									create l_end_date.make_from_rose_julian_day (l_rose_julian)
								end
							elseif l_end_date<l_start_date
							then
								l_end_date := l_start_date.twin
							end
							if is_defined_double (l_last_adjustment.adjustment) then
								create l_adjustment.make (Current, l_start_date, l_end_date,
									l_last_adjustment.adjustment * l_options.current_item.rmdat_adjustment_factor,
									l_options.current_item.rmdat_adjustment_factor)
							else
								create l_adjustment.make (Current, l_start_date, l_end_date, 1,0)
									--l_options.current_item.rmdat_adjustment_factor,
									--l_options.current_item.rmdat_adjustment_factor)
							end
							l_last_adjustment := l_adjustment
							Result.add_interval (l_adjustment)	-- Note that if 'add_interval' calls 'twin' on 'l_adjustment' we won't be able to
																-- change the start_date (via 'l_last_adjustment').  At this time 'add_interval'
																-- only 'twin's if the interval overlaps an existing interval - which shouldn't
																-- happen here.
						end

						l_first_time := False
						l_options.forth
					end
					Result.sort_by_start_date
				end
				new_adjustment_history_saved := Result
			else
				Result := new_adjustment_history_saved
			end
		end

feature {NONE} -- Implementation

	adjustment_history_saved: like adjustment_history
			-- Container

	new_adjustment_history_saved: like adjustment_history
			-- New adjustment_history

feature -- Basic Operations
	register  is
			-- Register this entity.
		do
			if zone /= void then
				zone.add_listing (Current)
			end
		end

	set_is_composite (a_value: BOOLEAN) is
			-- Assign `a_value' to `is_composite_saved'.
		do
			is_composite_saved := a_value
		ensure
			is_composite_saved_set: is_composite_saved = a_value
		end

feature -- Capital Actions
	capital_action_option (a_exdate: ROSE_DATE): ROSE_COMPLEX_CAPITAL_ACTION is
			-- all capital actions on the same exdate
			-- in future you can have mulitple options on the same exdate
		local
			l_targets: ROSE_LINEAR_ARRAY[like ur_capact_signature]
			l_option_target:like ur_capact_option_signature
			l_option_targets:ROSE_LINEAR_ARRAY[like ur_capact_option_signature]

		do
			ur_capact_signature.set_zone (zone_mnemonic)
			ur_capact_option_signature.set_zone(zone_mnemonic)
			if exchange /= void then
--				l_targets := ur_capact_signature.signatures_by_instrument_market_exdate (security.handle, exchange.handle, a_exdate.ccyymmdd)
--				l_option_targets := ur_capact_option_signature.signatures_by_instrument_market_exdate (security.handle, exchange.handle, a_exdate.ccyymmdd)
				l_targets := ur_capact_signature.signatures_by_listing_exdate (handle, a_exdate.ccyymmdd)
				l_option_targets := ur_capact_option_signature.signatures_by_listing_exdate (handle, a_exdate.ccyymmdd)
				if l_option_targets/= Void and then l_option_targets.count > 0 then
					l_option_target := l_option_targets.first
				end
				if l_targets /= Void and then l_targets.count > 0 and l_option_target/= Void then
					create Result.make (l_option_target,l_targets)
				end
			end
		end

	capital_action_options_after (a_exdate: ROSE_DATE; refresh_cache: BOOLEAN):  ROSE_LINEAR_ARRAY[ROSE_COMPLEX_CAPITAL_ACTION] is
			-- option history through today sorted by exdate desc, includes only valid options
	do
		Result := retrieved_capital_actions_after (True,a_exdate,refresh_cache)
	end

	all_capital_action_options_after (a_exdate: ROSE_DATE; refresh_cache: BOOLEAN):  ROSE_LINEAR_ARRAY[ROSE_COMPLEX_CAPITAL_ACTION] is
			-- option history through today sorted by exdate desc, including invalid options
		do
			Result := retrieved_capital_actions_after (False,a_exdate, refresh_cache)
		end

	capital_action_options_between (a_exdate, a_later: ROSE_DATE): ROSE_LINEAR_ARRAY [ROSE_COMPLEX_CAPITAL_ACTION] is
			-- Like capital_action_options_after, with exdate <= a_later
		require
			a_exdate_not_void: a_exdate /= Void
			a_later_not_void: a_later /= Void
		do
				-- could do a more selective query instead of filtering
				-- the 'after' result
			Result := capital_action_options_after (a_exdate, False)
			if Result /= Void then
				Result := Result.valid_subarray (agent capital_action_before (?, a_later))
			end
		ensure
			all_before: Result /= Void implies Result.for_all (agent capital_action_before (?, a_later))
		end

	capital_actions_between (a_start_date, a_end_date: ROSE_DATE): ROSE_LINEAR_ARRAY [ROSE_COMPLEX_CAPITAL_ACTION] is
			-- start_date < exdate <= end_date
		local
			l_actions: ROSE_COMPLEX_CAPITAL_ACTIONS
		do
			create l_actions.make (Current, a_start_date, a_end_date, True)
			Result := l_actions.actions
		end

	next_cash_dividend_option_after (a_date: ROSE_DATE): ROSE_COMPLEX_CAPITAL_ACTION is
			-- next complex action including a dividend after supplied date
		local
			l_option: ROSE_COMPLEX_CAPITAL_ACTION
			l_options: like all_capital_action_options_after
		do
			l_options := all_capital_action_options_after (a_date, False)
			if l_options /= Void then
				from l_options.finish until l_options.off or Result /= Void loop
					l_option := l_options.current_item
					if l_option /= Void and then l_option.is_valid_option and then l_option.has_cash_dividend then
						Result := l_option
					end
					l_options.back
				end
			end
		end

	ur_capact_signature: CAPACT_SIGNATURE_RECORD is
			-- empty capital action signature for access to features
		once
			create Result
		ensure
			not_void: Result /= void
		end

	ur_capact_option_signature: CAPACT_OPTION_SIGNATURE_RECORD is
			---- empty complex action signature for access to features
		once
			create Result
		end

	has_terminal_distribution (a_start, a_end: ROSE_DATE): BOOLEAN
			-- Did `Current' have a terminal distribution between `a_start' and `a_end'.
		require
			a_start_not_void: a_start /= Void
			a_end_not_void: a_end /= Void
			a_start_end_ordered: a_start <= a_end
		local
			l_actions: ROSE_LINEAR_ARRAY [ROSE_COMPLEX_CAPITAL_ACTION]
		do
			l_actions := capital_actions_between (a_start, a_end)
			Result := l_actions /= Void and then l_actions.there_exists (agent {ROSE_COMPLEX_CAPITAL_ACTION}.has_terminal_distribution)
		end

feature {NONE} -- Capital Actions Implementation

	capital_action_before (a_capact: ROSE_COMPLEX_CAPITAL_ACTION; a_date: ROSE_DATE): BOOLEAN is
			-- Is capital action exdate <= a_date
		require
			a_capact_not_void: a_capact /= Void
			a_date_not_void: a_date /= Void
		do
			Result := a_capact.option_signature.exdate <= a_date.ccyymmdd
		end

	retrieved_capital_actions_after (a_valid_only:BOOLEAN; a_exdate:ROSE_DATE; refresh_cache: BOOLEAN):ROSE_LINEAR_ARRAY[ROSE_COMPLEX_CAPITAL_ACTION] is
			-- rose capital actions from storage
			-- Void `a_exdate' means beginning of time
		local
			l_targets: ROSE_LINEAR_ARRAY[CAPACT_SIGNATURE_RECORD]
			l_quotient_array: ROSE_LINEAR_ARRAY[ROSE_LINEAR_ARRAY[like ur_capact_signature]]
			l_option: ROSE_COMPLEX_CAPITAL_ACTION
			l_date: INTEGER
			l_option_targets:ROSE_LINEAR_ARRAY[like ur_capact_option_signature]
			l_option_target: like ur_capact_option_signature
			l_exdate:INTEGER
		do
			if refresh_cache = True then
				retrieved_capital_actions_after_saved := Void
			end
			if a_exdate = Void then
				l_date := constants.minimum_date_time.ccyymmdd
			else
				l_date := a_exdate.ccyymmdd
			end
			if retrieved_capital_actions_after_saved /= Void and then l_date >=  retrieved_capital_actions_after_exdate_saved then
				Result := retrieved_capital_actions_after_saved
			else
				create Result.make_empty
				if exchange /= Void then
					ur_capact_signature.set_zone (zone_mnemonic)
					ur_capact_option_signature.set_zone (zone_mnemonic)
					l_targets := ur_capact_signature.signatures_by_listing_id_after (handle, l_date)
					if l_targets /= void then
						l_targets.sort_by (agent capact_option_comparison)
						l_quotient_array := l_targets.quotient (agent capact_quotient_comparison)
						from l_quotient_array.start
						until l_quotient_array.after
						loop
							l_exdate:=l_quotient_array.current_item.first.exdate
							l_option_targets := ur_capact_option_signature.signatures_by_listing_exdate (handle, l_exdate)
							if l_option_targets = Void then
								l_option_target := Void
							else
								l_option_target := l_option_targets.first
							end
							if l_option_target = Void and l_quotient_array.current_item /= Void then
								fail ("retrieved_capital_actions_after", "Capact has no matching complex option for listing handle $1 and exdate $2.", <<handle, l_exdate>>)
							end
							create l_option.make (l_option_target,l_quotient_array.current_item)
							if a_valid_only then
								if l_option.is_valid_option then
									Result.extend (l_option)
								end
							else
								Result.extend (l_option)
							end
							l_quotient_array.forth
						end
					end
				end
				retrieved_capital_actions_after_saved := Result
				retrieved_capital_actions_after_exdate_saved := l_date
			end
		ensure
			not_void: Result /= Void
		end

	retrieved_capital_actions_after_saved: ROSE_LINEAR_ARRAY[ROSE_COMPLEX_CAPITAL_ACTION]
		-- saved complex action history to avoid multiple retrieves when possible

	retrieved_capital_actions_after_exdate_saved: INTEGER
		-- first exdate date of retrieved capital actions saved	

	capact_option_comparison (a,b: like ur_capact_signature): BOOLEAN is
			-- comparison for ordering options
		require
			a_not_void: a /= Void
			b_not_void: b /= Void
			a_exdate_not_void: a.exdate /= Void
			b_exdate_not_void: b.exdate /= Void
		do
			Result := a.exdate > b.exdate
		ensure
			definition: Result = (a.exdate > b.exdate)
		end

	capact_quotient_comparison (a,b: like ur_capact_signature): BOOLEAN is
			-- comparison for gathering options by exdate
		require
			a_not_void: a /= Void
			b_not_void: b /= Void
			a_exdate_not_void: a.exdate /= Void
			b_exdate_not_void: b.exdate /= Void
		do
			Result := a.exdate = b.exdate
		ensure
			definition: Result = (a.exdate = b.exdate)
		end

feature -- Related Entities
	currency (a_date: ROSE_DATE): CURRENCY is
			-- rmdat currency for listing
		do
			Result := rmdat_available_currency (a_date)
		end

	company: like context.company_template is
		--  company being listed
		local
			l_security: like security
		do
			l_security := security
			if l_security /= Void then
				Result := l_security.company
			end
			if Result /= Void and then not Result.zone.is_equal (zone) then
				Result := Result.surrogate (zone)
			end
		end

	zone_from_exchange: like context.zone_template is
			-- zone of exchange
		local
			l_exchange: like context.exchange_template
		do
			l_exchange := exchange
			if l_exchange /= Void then
				Result := l_exchange.zone
			elseif zone_mnemonic.is_alpha then
				Result := zone
			end
		end

	corelationships: ROSE_LINEAR_ARRAY[RELATIONSHIP_HANDLER [ENTITY,like Current]] is
			-- relationships in which it is the second (right side) entity
		do
			Result ?= Precursor
			if not Result.has (context.portfolio_to_listing) then
				Result.extend (context.portfolio_to_listing)
			end
		end

	corelated_entities (a_name: STRING): ROSE_ENTITY_HISTORY[ENTITY] is
			-- entities to which this listing is related as the second (right side) entity
		do
			if a_name.is_equal (context.portfolio_to_listing.name) then
				Result := portfolio_to_listing_co_history
			else
				Result := Precursor (a_name)
			end
		end

	portfolio_to_listing_co_history: ROSE_ENTITY_HISTORY [like context.portfolio_template] is
			-- Weighter history of portfolios related to `Current'
		do
			Result := holdings_during (create {ROSE_DATE}.make_min_date, create {ROSE_DATE}.make_max_date)
		end

	sibling_listing_with_rmdat: like Current is
			--First other listing of security with zone_from_exchange equal to current zone_from_exchange
		local
			i: INTEGER
			l_listing: like Current
			l_security: like context.security_template
		do
			if	xrf_code /= 0 then
				Result := Current
			else
				l_security := security
				if l_security /= Void then
					from i := 1 until i > l_security.listing_count or Result /= Void loop
						l_listing := l_security.listing (i)
						if l_listing.xrf_code /= 0 and then zone_from_exchange = l_listing.zone_from_exchange then
							Result := l_listing
						end
						i := i + 1
					end
				end
			end
		end

feature -- Foreign Entities
	surrogated_company: like context.company_template is
			-- if this is a foreign listing of a company, transfer this company to a foreign company of the listing zone
		local
			l_company: like context.company_template
		do
			if security /= Void then
				l_company := security.company
				if l_company /= Void
				and then l_company.zone_mnemonic /= '%U'
				and then l_company.zone_mnemonic /= zone_mnemonic then
					create Result.make_surrogate (l_company, zone)
				end
			end
		end

	surrogated_security: like context.security_template is
			-- if this is a foreign listing of a security, transfer this security to a foreign security of the listing zone
		do
			if security /= Void
			and then security.zone_mnemonic /= '%U'
			and then security.zone_mnemonic /= zone_mnemonic then
				create Result.make_surrogate(security,zone)
			end
		end
	zonal_security: like context.security_template is
			-- if this is a foreign listing of a security, transfer this security to a foreign security of the listing zone
		do
			if security /= Void
			and then security.zone_mnemonic /= '%U'
			and then security.zone_mnemonic /= zone_mnemonic then
				 Result := security.surrogate (zone)
			else
				Result := security
			end
		end

	is_dummy_listing_of (a_zone_mnemonic: CHARACTER): BOOLEAN is
		-- check zone of local listing
    	local
			l_local_xrfzone: STRING
			l_dummy_zone: CHARACTER
		do
			inspect a_zone_mnemonic.as_upper
			when 'O' then
				-- o zone local xrf called "local_xrfzone"
			l_local_xrfzone := local_xrfzone_key (Void)
			when 'S' then
				l_local_xrfzone := s_xrfzone_key (Void)
			when 'E' then
				l_local_xrfzone := e_xrfzone_key (Void)
			else

			end
		 	if l_local_xrfzone /= Void then
		 		l_dummy_zone := l_local_xrfzone.item (l_local_xrfzone.count)
			    if l_dummy_zone.is_equal (a_zone_mnemonic) then
			    	Result := True
			    end
			end
		end

	local_xrf_code_in (a_zone_mnemonic: CHARACTER): INTEGER is
		-- xrf code for local representation in specified zone
		local
			l_local_code_key: STRING
		do
			inspect a_zone_mnemonic.as_upper
			when 'O' then
				-- o zone local xrf called "local_xrfzone"
				l_local_code_key := local_xrf_code_key (Void)
			when 'S' then
				l_local_code_key := s_xrf_code_key (Void)
			when 'E' then
				l_local_code_key := e_xrf_code_key (Void)
			else

			end
			if is_dummy_listing_of (a_zone_mnemonic) and then l_local_code_key /= Void then
				l_local_code_key := l_local_code_key.twin
				l_local_code_key.prune_all (a_zone_mnemonic)
				Result := l_local_code_key.to_integer
			end
		end

feature -- Return

	fast_return_new_ts (a_start_date: ROSE_DATE; a_end_date: ROSE_DATE): DOUBLE
			--Faster return calculation
		local
			l_lookup: ROSE_LOOKUP_TABLE[ DOUBLE, ROSE_DATE ]
			l_entity_id: INTEGER
			l_tot_new_one, l_tot_new_two: DOUBLE
			l_where: STRING
			l_start_date, l_end_date: ROSE_DATE
		do
			l_start_date := a_start_date.rose_date
			l_end_date := a_end_date.rose_date
			l_entity_id := handle
			if fast_return_new_ts_hash.has( l_entity_id ) then
				l_lookup := fast_return_new_ts_hash.item( l_entity_id )
			else
				l_where := "where entity_id = " + l_entity_id.out
				create l_lookup.make_table_with_where (return_server, "data_signatures",  "period_end_date", "total_return_new", l_where)
				fast_return_new_ts_hash.put( l_lookup, l_entity_id )
			end
			if l_lookup.has( l_start_date ) and l_lookup.has( l_end_date ) then
				l_tot_new_one := l_lookup.item( l_start_date )
				l_tot_new_two := l_lookup.item( l_end_date )
				if (not is_defined_double (l_tot_new_one)) or (not is_defined_double (l_tot_new_two)) then
					Result := return_new( a_start_date, a_end_date )
				else
					Result := l_tot_new_two / l_tot_new_one
				end
			else
			 	Result := return_new( a_start_date, a_end_date )
			end
		end

feature {NONE} -- Return implementation

	fast_return_new_ts_hash: ROSE_HASH_TABLE[ ROSE_LOOKUP_TABLE[ DOUBLE, ROSE_DATE ], INTEGER ] is
				-- hash table for fast return
		once
			create Result.make(300)
		end

feature {NONE} -- Return Calculations

	nlag_rrev: INTEGER is
	-- number of lags of return revesal counted
		once
			Result := 7
		end

	rrev_day_weights: ROSE_LINEAR_ARRAY[ DOUBLE ] is
	--- weights for the returns lag 1 day through lag 8 market days
		once
			Result := <<-0.10, -0.10, -0.06, -0.05, -0.04, -0.03, -0.02, -0.01>>
		end


	return_using_time_series (a_start_date: ROSE_DATE; a_end_date: ROSE_DATE): DOUBLE is
		local
			l_predicate: SELECTION_PREDICATE [PAIR [DOUBLE, ROSE_DATE]]
			l_start: DOUBLE
		do
			Result := constants.missing_double
			if time_series_for_return = Void then
				create time_series_for_return.make_empty
				l_predicate := zonal_return_predicates.item (zone_mnemonic)
				if l_predicate /= Void then
					l_predicate.set_container (time_series_for_return)
					l_predicate.retrieve (Void, <<handle>>)
					if not l_predicate.success then (create {EXCEPTIONS}).raise ("Failed select predicate in return_using_time_series for entity_id = " + handle.out) end
				end
			end
			if time_series_for_return.count >= 2 and then
				time_series_for_return.first.second <= a_start_date and then
				time_series_for_return.last.second >= a_end_date then
				time_series_for_return.dynamic_binary_search (agent time_series_for_return_comparison (?, a_start_date.date))
				if time_series_for_return.found and then is_defined_double (time_series_for_return.found_item.first) then
					l_start := time_series_for_return.found_item.first
					time_series_for_return.dynamic_binary_search (agent time_series_for_return_comparison (?, a_end_date.date))
					if time_series_for_return.found and then is_defined_double (time_series_for_return.found_item.first) then
						Result := time_series_for_return.found_item.first / l_start
					end
				end
			end
		end

	time_series_for_return: ROSE_LINEAR_ARRAY [PAIR [DOUBLE, ROSE_DATE]]

	time_series_for_return_comparison (a_pair: PAIR [DOUBLE, ROSE_DATE]; a_date: ROSE_DATE): INTEGER is
		do
			if a_pair.second < a_date then
				Result := 1
			elseif a_pair.second > a_date then
				Result := -1
			else
				Result := 0
			end
		end

	zonal_return_predicates: HASH_TABLE [SELECTION_PREDICATE [PAIR [DOUBLE, ROSE_DATE]], CHARACTER] is
		local
			l_predicate: SELECTION_PREDICATE [PAIR [DOUBLE, ROSE_DATE]]
			l_statement: STRING
			l_pair: PAIR [DOUBLE, ROSE_DATE]
		once
			create Result.make (0)
			l_statement := "select total_return_new as first, period_end_date as second from data_signatures where entity_id = ? order by period_end_date"
			create l_pair.make (0.0, create {ROSE_DATE}.make_min_date)
			create l_predicate.make (servers.return_u_server, l_pair, l_statement)
			if not l_predicate.success then (create {EXCEPTIONS}).raise ("Failed making U zone predicate statement in zonal_return_predicates") end
			Result.extend (l_predicate, 'U')
			create l_predicate.make (servers.return_c_server, l_pair, l_statement)
			if not l_predicate.success then (create {EXCEPTIONS}).raise ("Failed making C zone predicate statement in zonal_return_predicates") end
			Result.extend (l_predicate, 'C')
			create l_predicate.make (servers.return_j_server, l_pair, l_statement)
			if not l_predicate.success then (create {EXCEPTIONS}).raise ("Failed making J zone predicate statement in zonal_return_predicates") end
			Result.extend (l_predicate, 'J')
			create l_predicate.make (servers.return_e_server, l_pair, l_statement)
			if not l_predicate.success then (create {EXCEPTIONS}).raise ("Failed making E zone predicate statement in zonal_return_predicates") end
			Result.extend (l_predicate, 'E')
			create l_predicate.make (servers.return_o_server, l_pair, l_statement)
			if not l_predicate.success then (create {EXCEPTIONS}).raise ("Failed making O zone predicate statement in zonal_return_predicates") end
			Result.extend (l_predicate, 'O')
			create l_predicate.make (servers.return_s_server, l_pair, l_statement)
			if not l_predicate.success then (create {EXCEPTIONS}).raise ("Failed making S zone predicate statement in zonal_return_predicates") end
			Result.extend (l_predicate, 'S')
		end

feature -- Return

	return_reversal (a_as_of_market_date: ROSE_DATE): DOUBLE is
			-- 7 day return reversal
		local
			l_rrev: DOUBLE
			i,j: INTEGER
			l_nret: INTEGER -- number of returns
			l_market_date_set: ROSE_DATE_SET
			l_start, l_end: ROSE_DATE
			l_rm_return_new: DOUBLE
		do
			l_rrev := 0.0
			l_nret := 0
			j := 1
			l_market_date_set := country.market_dates
			from i := 1
			until i > nlag_rrev or j > 14
			loop
				l_start := l_market_date_set.ith_date_before (a_as_of_market_date,j)
				l_end := l_market_date_set.ith_date_before (a_as_of_market_date,j-1)
				l_rm_return_new := return (l_start, l_end)
				if l_rm_return_new < 0.0  or l_rm_return_new = 1 then
					-- -999, no daily reurn, or market not open
				else
					l_nret := l_nret + 1
					l_rrev := l_rrev + (l_rm_return_new - 1.0) * rrev_day_weights.item (rrev_day_weights.lower+i-1)
					i := i + 1
				end
--				i := i + 1
				j := j + 1
			end
			if l_nret = 0 then
				-- set to unavailable if no return available
				l_rrev := undefined_integer
			end
			Result := l_rrev
		end

feature {NONE} -- Return implementation

	return_server: SQL_DATA_SERVER is
			-- Server used to provide returns.
		do
			Result := servers.return_server (zone.mnemonic)
		end

	return_legacy (a_start_date,a_end_date: ROSE_DATE): DOUBLE
			-- return for a holding period between a_start_date and a_end_date
		do
			if rmdat_listing_map = Void then
				zone.returns.period_return (xrf_code,a_start_date,a_end_date)
				Result := zone.returns.return
			else
				Result := rmdat_listing_map.return (a_start_date,a_end_date)
			end
		end

	is_legacy_return_in_use: BOOLEAN
			-- Use new return calcuatlion always.
			-- FIXME: Routine kept for easy switchback in case of a problem.
			--        it should be removed after some suitable time of use in
			--        production. (8/18/13)
		do
			Result := False
		end

feature -- Returns

	return (a_start_date,a_end_date: ROSE_DATE): DOUBLE
			-- return for a holding period between a_start_date and a_end_date
		do
			if is_legacy_return_in_use then
				Result := return_legacy (a_start_date, a_end_date)
			else
				Result := return_new (a_start_date, a_end_date)
			end
		end

	return_in_currency (a_start_date,a_end_date: ROSE_DATE;a_currency:CURRENCY): DOUBLE is
			-- Total return (including capital action) for period
			-- including `a_currency' return relative to the listing's currency
		local
			l_forex_return,l_return:DOUBLE
			l_currency:CURRENCY
		do
			Result := undefined_double
			l_return := return (a_start_date,a_end_date)
			if is_defined_double (l_return) then
				l_currency := rmdat_available_currency (a_start_date)
				if l_currency /= Void then
					l_forex_return := l_currency.exchange_return (a_currency, a_start_date,a_end_date)
					if is_defined_double (l_forex_return) then
						result:=l_return * l_forex_return
					end
				end
			end
		end

	return_in_currency_new (a_start_date,a_end_date: ROSE_DATE;a_currency:CURRENCY): DOUBLE is
			-- return in specified currency
		local
			l_forex_return,l_return:DOUBLE
			l_currency:CURRENCY
		do
			--| to be merged with `return_in_currency' when capact migration complete
			--  as the only difference is calling return/return_new
			Result := undefined_double
			l_return := return_new (a_start_date,a_end_date)
			if is_defined_double (l_return) then
				l_currency := rmdat_available_currency (a_start_date)
				if l_currency /= Void then
					l_forex_return := l_currency.exchange_return (a_currency, a_start_date,a_end_date)
					if is_defined_double (l_forex_return) then
						result:=l_return * l_forex_return
					end
				end
			end
		end

	yield_new (a_start_date,a_end_date: ROSE_DATE): DOUBLE is
			-- Dividend yield
		do
			--| to be renamed `yield' after cap act migration complete
			rm_return_new.period_return (Current, a_start_date, a_end_date)
			Result := rm_return_new.yield
		end

	return_record: RETURN_RECORD is
		do
			if is_legacy_return_in_use then
				create Result.make (zone.returns)
			else
				Result := return_new_record
			end
		ensure
			not_void: Result /= Void
		end

feature {NONE} -- Return implementation

	return_new_record: RETURN_RECORD is
		do
			--| to be merged with `return_record' after cap act migration complete
			create Result.make_new (rm_return_new)
		end

feature -- Return

	return_new (a_start_date,a_end_date: ROSE_DATE): DOUBLE is
			-- Compute return using new capital action database
		do
			--| to be merged with `return' after cap act migration complete
			rm_return_new.period_return (Current, a_start_date, a_end_date)
			Result := rm_return_new.return
		end

	net_cash_flow_adjusted_to_start (a_start_date,a_end_date: ROSE_DATE;a_currency:CURRENCY): RETURN_RECORD is
		--returns the net cash inflow to the shareholder / outflow from the company
		-- adjusted in terms of the start_date shares, if no currency provided, Currency is that of the end-date for listing
		do
			rm_return_new.set_net_cash_flow_adjusted_to_start (Current, a_start_date, a_end_date, a_currency)
			Result := return_new_record
		end

	implied_return (a_start_date,a_end_date: ROSE_DATE): DOUBLE is
			-- Implied return
		do
			rm_return_new.implied_period_return (Current, a_start_date, a_end_date)
			Result := rm_return_new.return
		end

feature -- Last Price Record
	last_price_record (a_date: ROSE_DATE): like price_record is
			--|
		do
			if rmdat_listing_map = Void then
				Result := Precursor (a_date)
			else
				Result := rmdat_listing_map.last_price_record (a_date)
			end
		end

	last_price_record_160 (a_date: ROSE_DATE): like price_record
			-- Latest valid price using the rules found in B_RMDAT_SRC:RMDAT_INTERNAL.FOR at the 160 loop.
		do
			if rmdat_listing_map = Void then
				Result := Precursor (a_date)
			else
				Result := rmdat_listing_map.last_price_record_160 (a_date)
			end
		end

feature -- Price Record
	price_record (a_date: ROSE_DATE): RMDAT_PRICE_RECORD is
			-- Price record as of `a_date'.
		do
			if rmdat_listing_map = Void then
				Result := Precursor (a_date)
			else
				Result := rmdat_listing_map.price_record (a_date)
			end
		end

feature -- Market Data from REUTERS
	vwap(a_date: ROSE_DATE): DOUBLE is
			-- Returns the Volume Weighted Average Price for this listing on 'a_date'.  If need be, it will
			-- be converted into the currency in use by RMDAT on this date (see 'currency').
			-- Currently, this function looks at REUTERS but that must eventually change.  Also, note that
			-- the REUTERS VWAP is calculated differently for many exchanges and not every exchange is currently
			-- supported.
		local
			l_target: REUTERS2_PRICE_TARGET
			l_type_ids: REUTERS_ATTRIBUTE_TYPE_ID_USE
			l_attribute_value: STRING
			l_sig: REUTERS2_SIGNATURE_RECORD
			l_sigs: ROSE_LINEAR_ARRAY[REUTERS2_SIGNATURE_RECORD]
		do
			if reuters_entity=Void
			then
				create l_type_ids
				l_attribute_value := reuters_code_most_recent_key (a_date)
				if l_attribute_value/=Void
				then
					create reuters_entity.attempt_make_from_attribute (ids.Listing_category_id, l_type_ids.ric_type_id, l_attribute_value, Void)
				else
					create reuters_entity.default_create	-- This is invalid.
				end
			end

			if reuters_entity.valid
			then
				create l_sig
				l_sig.set_entity_id(reuters_entity.entity_id)
				l_sig.set_period_end_date(a_date)
				l_sig.set_periodicity_id(ids.daily_periodicity_id)
				l_sigs := l_sig.targets(create {ROSE_LINEAR_ARRAY[REUTERS2_SIGNATURE_RECORD]}.make_from_array(<<l_sig>>), Void)
				if l_sigs /= Void and then l_sigs.count > 0
				then
					create l_target
					l_target.set_direction_neighborhood
					l_target.set_entity_id (reuters_entity.entity_id)
					l_target.set_period_end_date(a_date)
					l_target.set_periodicity_id (Ids.daily_periodicity_id)
					-- This 49 is hard coded to keep non-kernel classes out of the kernel.  This stuff is all temporary anyway.
					l_target.set_value_description_id(49)		-- reuters_adjustment_id
					l_target.set_currency_id(l_sigs.item(1).currency_id)
--					Result := vwap_expr.double_value (l_target)
					Result := vwap_expr.value_in_currency(currency(a_date), l_target, a_date)
				else
					Result := constants.missing_double
				end
			else
				Result := constants.Missing_double
			end
		end

feature {NONE} -- Implementation

	reuters_entity: REUTERS_ENTITY			-- This is only for 'vwap' and I hope will go away someday.

	vwap_expr: REUTERS2_PRICE_EXPRESSION is
			-- vwap expression for `vwap' implementation
		local
			l_sig: REUTERS2_PRICE_TARGET
		once
			create l_sig
			Result := l_sig.expression_from_name("VWAP")
		end

feature -- Market Data

	last_price_date_160(a_date: ROSE_DATE): ROSE_DATE is
			-- Date of last price record.
		do
			Result := last_price_record_160(a_date).date
		end

	scaled_last_price (a_date: ROSE_DATE): DOUBLE
			-- Adjusted last price on or before `a_date'
		require
			a_date_attached: a_date /= Void
		do
			if rmdat_listing_map = Void then
				Result := zone.market_data.xdaylp (xrf_code,a_date)
			else
				Result := rmdat_listing_map.scaled_last_price (a_date)
			end
		end

	daylpis (a_date: ROSE_DATE): PAIR[DOUBLE, CURRENCY] is
		require
			a_date_attached: a_date /= Void
		local
			l_double: DOUBLE
			l_currency: CURRENCY
		do
			l_double := daylp (a_date)
			l_currency := rmdat_currency_160(a_date)
			create Result.make (l_double, l_currency)
		end

	daylp (a_date:ROSE_DATE):DOUBLE
			--Most recent daily price
		require
			a_date_attached: a_date /= Void
		do
			Result := zone.market_data.daylp (xrf_code,a_date)
		end

	unitized_daylp (a_date:ROSE_DATE): ROSE_UNITIZED_VALUE[DOUBLE]
			-- Unitized representation of last daily price
		require
			a_date_not_void: a_date /= Void
		local
			l_units : ROSE_UNITS
			l_currency_units : CURRENCY_UNITS
			l_price: DOUBLE
		do
			l_price := zone.market_data.daylp (xrf_code, a_date)
			if is_defined_double (l_price) and then l_price > 0.0 then
				if attached zone.market_data.price_record_saved as la_price_record and then la_price_record.is_valid and then attached la_price_record.currency as la_currency then
					l_currency_units := currency_units (la_currency)
				end
			else
				l_price := qnan
			end
			if l_currency_units = Void then
				l_currency_units := currency_units (zone.base_currency)
			end
			if l_currency_units.rational /= Void and then l_currency_units.currency /= Void and then l_currency_units.scale /= Void then
				l_units := l_currency_units / generic_share_units
			end
			create Result.make_with_units (l_price, l_units)
		end

	day_lp (a_date:ROSE_DATE;a_lookback:INTEGER):DOUBLE is
			-- Last price on or before `a_date'
		require
			a_date_attached: a_date /= Void
		do
			Result := zone.market_data.day_lp (xrf_code,a_date,a_lookback)
		end

	average_closing_price (a_date: ROSE_DATE; a_count: INTEGER): DOUBLE is
			-- Find the average over the NEXT 'a_count' market days.
			-- date computation details defined in time_series_something
		do
			Result := country.market_dates.time_series_of_agent_result(
				a_date, a_count, agent closing_price).average_or_undefined
		end

	average_volume (a_date: ROSE_DATE; a_count: INTEGER): DOUBLE is
			-- Find the average over the NEXT 'a_count' market days.
			-- date computation details defined in time_series_something
		do
			Result := country.market_dates.time_series_of_agent_result(
				a_date, a_count, agent volume).average_or_undefined
		end

	median_volume (a_date: ROSE_DATE; a_count: INTEGER): DOUBLE is
			-- Find the median over the NEXT 'a_count' market days.
			-- date computation details defined in time_series_something
		do
			Result := country.market_dates.time_series_of_agent_result(
				a_date, a_count, agent volume).median_or_undefined
		end

	average_vwap (a_date: ROSE_DATE; a_count: INTEGER): DOUBLE is
			-- Find the average over the NEXT 'a_count' market days.
			-- date computation details defined in time_series_something
		do
			Result := country.market_dates.time_series_of_agent_result(
				a_date, a_count, agent vwap).average_or_undefined
		End

	rmdat_currency (a_date: ROSE_DATE): CURRENCY is
		do
			Result := last_price_record (a_date).currency
		end

	rmdat_currency_160 (a_date: ROSE_DATE): CURRENCY is
		do
			Result := last_price_record_160(a_date).currency
		end

	rmdat_available_currency (a_date: ROSE_DATE): CURRENCY is
			-- Uses old VMS logic to fallback on exchange/country if currency is missing in record
		do
			Result:= rmdat_currency (a_date)
			if Result=void or else not Result.is_known_currency  then
				if (Result=void or else not Result.is_known_currency) and then exchange/=void  then
					Result:=exchange.country.official_currency (a_date)
				end
				if (Result = Void or else not Result.is_known_currency) and then country /= Void and then country.currency_history /= Void then
					Result := country.currency_history.last.entity
				end
			end
		end

	is_live: BOOLEAN is
			-- Checks whether listing has any valid prices in the recent month
		local
			l_today, l_date: ROSE_DATE
			l_check: ROSE_CONSTANTS
		do
			create l_today.make_today
			create l_check.default_create
			if last_date /= Void then
				if last_date >= l_today.plus_n_month (-1) then
					Result := True
				end
			else
				from
					l_date := l_today.plus_n_month (-1)
				until
					l_date > l_today or Result
				loop
					if l_date.is_weekday and then l_check.is_defined_double (closing_price (l_date)) then
						Result := True
						l_date := l_today
					end
					l_date.day_forth
				end
			end
		end

feature -- Legacy calls which emulate the 'item_codes' in RMDAT_W.
		-- These calls MUST replicate VMS exactly!  If you need different
		-- behaviour then make a new function.

	legacy_uacp (a_date: ROSE_DATE): DOUBLE is
			-- Legacy unadjusted close bid price
		obsolete "For testing only!"
		do
			Result := price_record (a_date).unadjusted_close_bid
			Result := value_at_default_currency (Result, a_date)
		end

feature {NONE} -- Implementation

	is_single_country_zone: BOOLEAN
			--Is the zone for this listing  a single country zone?
		do
			Result := zone = u_zone or zone = c_zone or zone = j_zone
		end

	value_at_default_currency (a_value: DOUBLE; a_date: ROSE_DATE): DOUBLE
			--value in default currency
		local
			l_ex_rate: DOUBLE
			l_default_currency, l_currency: CURRENCY
		do
			if is_defined_double (a_value) and then not is_single_country_zone and then a_date /= Void then
				l_currency := price_record (a_date).currency
				l_default_currency := default_rmdat_currency (a_date)
				if ( (l_currency /= Void and then l_currency.is_known_currency ) and l_default_currency /= Void) and then not equal (l_currency.iso, l_default_currency.iso)  then
					l_ex_rate :=  l_default_currency.exchange_rate (l_currency, a_date)
					if is_defined_double (l_ex_rate)
					then
						Result := a_value * l_ex_rate
					else
						Result := constants.missing_double
					end
				else
					Result := a_value
				end
			else
				Result := a_value
			end
		end


feature -- old rmdat currency handling

	default_rmdat_currency (a_date: ROSE_DATE): CURRENCY is
		-- check rmdat_internal.for for related logic
		do
			Result := zone.market_data.default_currency (xrf_code, a_date)
			if Result = Void  then
				Result := rmdat_currency (a_date)
			end
		end

	price_in_base_unit (a_date, a_base_date: ROSE_DATE): PAIR[DOUBLE, CURRENCY]
			--Price in base units
		require
			valid_dates: a_date /= Void and a_base_date /= Void
		local
			l_price: DOUBLE
			l_adj_history: like adjustment_history
			l_currency: CURRENCY
		do
			l_price := daylp (a_date)
			if is_defined_double (l_price) then
				l_currency := rmdat_available_currency (a_date)
				if not a_date.is_equal (a_base_date) then
					l_adj_history := adjustment_history
					if l_adj_history /= Void then
						l_price := l_adj_history.adjusted_value_as_of (l_price, a_date, a_base_date)
						create Result.make (l_price, l_currency)
					else
						create Result.make (constants.missing_double, Void)
					end
				else
					create Result.make (l_price, l_currency)
				end
			else
				create Result.make (constants.missing_double, Void)
			end
		end

feature -- Actions
	actions_loaded: BOOLEAN

	load_all_actions is
			-- Load all distribution actions
		local
			l_brim_code: STRING
			l_array: ROSE_LINEAR_ARRAY[DIST_W_RECORD]
			l_record: DIST_W_RECORD
			l_action: ROSE_DISTRIBUTION
		do
			if not actions_loaded then
				actions_loaded := true
				l_brim_code := brim_code_key (today)
				distribution_reader.retrieve (void,<<l_brim_code,l_brim_code>>)
				l_array := distribution_reader.array
				from l_array.start until l_array.off loop
					l_record := l_array.current_item
					create l_action.make_from_record (l_record)
					l_array.forth
				end
			end
		end

	to_actions: ROSE_LINEAR_ARRAY[ROSE_DISTRIBUTION]

	from_actions: ROSE_LINEAR_ARRAY[ROSE_DISTRIBUTION]

	add_to_action (a_action: ROSE_DISTRIBUTION) is
			-- Add `a_action' to `to_actions'.
		do
			load_all_actions
			if to_actions = Void then
				create to_actions.make_empty
			end
			to_actions.extend (a_action)
		end

	add_from_action (a_action: ROSE_DISTRIBUTION) is
			-- Add `a_action' to `from_actions'.
		do
			load_all_actions
			if from_actions = Void then
				create from_actions.make_empty
			end
			from_actions.extend (a_action)
		end


feature {NONE} -- Legacy Capital Actions
	distribution_reader: COLLECTING_PREDICATE [DIST_W_RECORD]
			-- Distribution reader
		once
			create Result.make (servers.legacy_distributions_server,"select * from dist_w where status_flag = ' ' and (brimsym = ? or ch_brim = ?) order by exdate desc")
		end

feature -- Targets
	ur_listing_date_target: ROSE_LISTING_DATE is
		once
			create Result
		end
	ur_rmdat_u_signature: RMDAT_SIGNATURE_RECORD is
			-- U zone RMDAT target
		once
			create Result.make_ur (Current)
			Result.set_zone (u_zone)
		end
	ur_rmdat_c_signature: RMDAT_SIGNATURE_RECORD is
			-- C zone RMDAT target
		once
			create Result.make_ur (Current)
			Result.set_zone (c_zone)
		end
	ur_rmdat_j_signature: RMDAT_SIGNATURE_RECORD is
			-- J zone RMDAT target
		once
			create Result.make_ur (Current)
			Result.set_zone (j_zone)
		end
	ur_rmdat_o_signature: RMDAT_SIGNATURE_RECORD is
			-- O zone RMDAT target
		once
			create Result.make_ur (Current)
			Result.set_zone (o_zone)
		end
	ur_rmdat_e_signature: RMDAT_SIGNATURE_RECORD is
			-- E zone RMDAT target
		once
			create Result.make_ur (Current)
			Result.set_zone (e_zone)
		end
	ur_rmdat_s_signature: RMDAT_SIGNATURE_RECORD is
			-- S zone RMDAT target
		once
			create Result.make_ur (Current)
			Result.set_zone (s_zone)
		end

	rmdat_target_at (a_date: ROSE_DATE): RMDAT_SIGNATURE_RECORD is
			-- RMDAT target focused on `a_date'
		do
			create Result
			Result.set_entity (Current)
			Result.set_date (a_date)
			Result.set_zone (zone_by_mnemonic (zone_mnemonic))
		ensure
			not_void: Result /= Void
		end

	ur_targets: ROSE_LINEAR_ARRAY [ROSE_ENTITY_DATE [ENTITY]] is
			-- Targets types for this entity
		do
			create Result.make_empty
			if zone = Void or zone.mnemonic.is_equal (u_zone.mnemonic) then
				Result.extend (ur_rmdat_u_signature)
			end
			if zone = Void or zone.mnemonic.is_equal (c_zone.mnemonic) then
				Result.extend (ur_rmdat_c_signature)
			end
			if zone = Void or zone.mnemonic.is_equal (o_zone.mnemonic) then
				Result.extend (ur_rmdat_o_signature)
			end
			if zone = Void or zone.mnemonic.is_equal (j_zone.mnemonic) then
				Result.extend (ur_rmdat_j_signature)
			end
			if zone = Void or zone.mnemonic.is_equal (e_zone.mnemonic) then
				Result.extend (ur_rmdat_e_signature)
			end
			if zone = Void or zone.mnemonic.is_equal (s_zone.mnemonic) then
				Result.extend (ur_rmdat_s_signature)
			end
			Result.append_array (ur_targets_saved)
		end
feature -- Unitized

	unitized_bid (a_date: ROSE_DATE): ROSE_UNITIZED_VALUE[DOUBLE]
			-- bid price
		do
			Result := price_record (a_date).unitized.bid
		end

	unitized_ask (a_date: ROSE_DATE): ROSE_UNITIZED_VALUE[DOUBLE] is
			-- ask price
		do
			Result := price_record (a_date).unitized.ask
		end

	unitized_low, unitized_loprc (a_date: ROSE_DATE): ROSE_UNITIZED_VALUE[DOUBLE]
			-- low price
		do
			Result := price_record (a_date).unitized.low
		end

	unitized_high,unitized_hiprc (a_date:ROSE_DATE): ROSE_UNITIZED_VALUE[DOUBLE]
			-- High with currency.
		do
			Result := price_record(a_date).unitized.high
		end


feature {NONE} -- Targets Implementation

	ur_targets_saved: ROSE_LINEAR_ARRAY [ROSE_ENTITY_DATE [ENTITY]] is
			-- Cached list of target types for the entity
		once
			create Result.make_empty
			Result.extend (ur_listing_date_target)
		ensure
			targets_not_void: Result /= Void
			no_void_target: not Result.has (Void)
		end

feature -- Rose best price
	ur_price_target: ROSE_PRICE_SIGNATURE is
		-- ur price target
		once create Result end

	price_report: ROSE_PRICE_REPORT is once create Result.make end

	rmdat_report: RMDAT_PRICE_REPORT is once create Result.make end

	rmdat_report_unadjusted: RMDAT_PRICE_REPORT is
			-- rmdat report unadjusted
		once
			create Result.make_unadjusted
		end

	rmdat_price_set_series: DATED_MATRIX is
			-- rmdat price set series
		do
			Result := rmdat_report.matrix (Current)
		end

	rmdat_price_set_series_partial (a_start_date, a_end_date: ROSE_DATE): DATED_MATRIX is
			-- price set series partial
		require
			valid_dates: a_start_date /= Void and a_end_date /= Void
		do
			Result := rmdat_report_unadjusted.matrix_interval (Current, a_start_date, a_end_date)
		end

	listing_by_ric (a_date: ROSE_DATE): like Current is
			--listing whose exchange matches the exchange of this listing's reuters code
		local
			i, l_count: INTEGER
			l_listing: like Current
			l_security: like context.security_template
			l_exchange: like context.exchange_template
		do
			l_exchange := ric_exchange (a_date)
			if l_exchange /= Void then
				if l_exchange /= exchange then
					l_security := security
					if l_security /= Void then
						l_count := l_security.listing_count
						from i := 1 until i > l_count or Result /= Void loop
							l_listing := l_security.listing (i)
							if l_listing /= Void and then l_listing.exchange = l_exchange then
								Result := l_listing
							end
							i := i + 1
						end
					end
				else
					Result := Current
				end
			end
		end

	ric_exchange (a_date: ROSE_DATE): like context.exchange_template is
			-- ric exchange
		local
			l_dlist: ROSE_DELIMITED_TEXT
			l_ric, l_ric_suffix: STRING
		do
			l_ric := reuters_code_key (a_date)
			if l_ric = Void and then a_date /= Void then
				l_ric := reuters_code_key (Void)
				if l_ric = Void then
					l_ric := reuters_code_key (start_date)
				end
			end
			if l_ric /= Void then
				create l_dlist.make (l_ric, '.')
				if l_dlist.count >= 2 then
					l_ric_suffix := l_dlist.item(2)
					l_ric_suffix.prune_all(' ')
					if l_ric_suffix /= Void and then l_ric_suffix.count > 0 then
						if l_ric_suffix.as_upper.is_equal ("OJ") then
							l_ric_suffix := "OS" -- special handling of "Hercules" (OTC) section of the Osaka exchange
						end
						Result := context.exchange_at (ric_suffix (l_ric_suffix), Void)
					end
				end
			end
		end

	rm_return_new: RM_RETURN_NEW is
		-- rm return
		once create Result end

feature -- Instrument overrides
	asset_class_code (a_date: ROSE_DATE): STRING is
			-- Asset class code
		do
			-- requires mapping from Norm Wolf
		end

	risk_region_code (a_date: ROSE_DATE): STRING is
			-- Risk region code
		local
			s: STRING
		do
			Result := risk_region.dated_key (security,a_date)
			if Result = void then
				s := brim_code( a_date)
				if s /= Void then
					Result := s.substring(1,2)
				end
			end
		end

	cusip_code (a_date: ROSE_DATE): STRING is
			-- Cusip associated with security of `Current' as of `a_date'
		local
			l_security: SECURITY
		do
			l_security := security
			if l_security /= Void then
				Result := l_security.cusip_most_recent_key (a_date)
			end
		end

	isin_code (a_date: ROSE_DATE): STRING is
			-- ISIN associated with security of `Current' as of `a_date'
		local
			l_security: SECURITY
		do
			l_security := security
			if l_security /= Void then
				Result := l_security.isin_code_most_recent_key (a_date)
			end
		end

	tse_code (a_date: ROSE_DATE): STRING is
			-- TSE associated with security of `Current' as of `a_date'
		local
			l_security: SECURITY
		do
			l_security := security
			if l_security /= Void then
				Result := l_security.tse_code_most_recent_key (a_date)
			end
		end

feature -- Supplier Attributes

	barra_ids (a_date: ROSE_DATE): ROSE_LINEAR_ARRAY [STRING] is
			-- All associated Barra Ids on `a_date'.
		require
			a_date_not_void: a_date /= Void
		do
			Result := mapped_supplier_attributes (barra_context, barra_id_identifier, a_date)
		ensure
			result_not_void: Result /= Void
		end

	barra_ids_non_synthetic (a_date: ROSE_DATE): ROSE_LINEAR_ARRAY [STRING] is
			-- All associated Barra Ids on `a_date' that have an instrument type and are not marked as "SYNTHETIC"
		require
			a_date_not_void: a_date /= Void
		local
			l_barra_ids: like mapped_supplier_attributes_with_handles
			i: INTEGER
			l_listing: PERSISTENT_LISTING
			l_security: PERSISTENT_SECURITY
			l_company: PERSISTENT_COMPANY
			l_instrument: STRING
		do
			create Result.make_empty
			l_barra_ids := mapped_supplier_attributes_with_handles (barra_context, barra_id_identifier, a_date)
			from
				i := l_barra_ids.lower
			until
				i > l_barra_ids.upper
			loop
				l_listing := barra_context.listing_accessor.entity (l_barra_ids [i].entity_handle)
				if l_listing /= Void then
					l_security := l_listing.security
					if l_security /= Void then
						l_company := l_security.company
						if l_company /= Void then
							l_instrument := l_company.attribute_by_name ("instrument", a_date)
							if l_instrument /= Void and then l_instrument /~ "SYNTHETIC" then
								Result.extend (l_barra_ids [i].attribute_value)
							end
						end
					end
				end
				i := i + 1
			end
		ensure
			result_not_void: Result /= Void
		end

feature {NONE} -- Supplier Attributes

	barra_id_identifier: STRING is "barrid"

	barra_context: ROSE_SUPPLIER_CONTEXT is
			-- Once access to a context representing Barra.
		once
			create Result.make (servers.barra_server, "Barra")
		ensure
			result_not_void: Result /= Void
		end

end

