note

	description:

		"A collection of interval histories of listings."

	author: "Michael Zhang"
	last_editor: "$Author: mk $"
	date: "$Date: 2012/06/22 $"
	revision: "$Revision: #10 $"
	path: "$File: //rose/source/kernel/investment_entities/listing_holdings.e $"

class
	LISTING_HOLDINGS

inherit
	HOLDINGS [LISTING, LISTING_HOLDING, HOLDING_HISTORY [LISTING, LISTING_HOLDING]]

	ROSE_ACCESS
		undefine
			is_equal, copy
		end
create
	make_empty,
	make,
	make_from_array
feature -- Access

	listings_at (a_date : ROSE_DATE) : XSECTION_MATRIX [LISTING] is
			-- Listings contained in `Current' with units and dollar value information.
		local
			l_matrix : MATRIX
			l_entities : ROSE_LINEAR_ARRAY[like holding_entity_template]
			l_entity : like holding_entity_template
			l_price, l_weight, l_share, l_rate : DOUBLE
			l_currency : CURRENCY
			i : INTEGER
		do
			create l_entities.make_empty
			create l_matrix.make(count, 2)
			from i := lower until i > upper loop
				l_entity := item (i).entity
				l_share := units_at (l_entity, a_date)
				if l_share.abs > 0 then
					l_entities.extend (l_entity)
					l_price := l_entity.daylp (a_date).max (0)
					l_currency := l_entity.currency (a_date)
					l_rate := usd.exchange_rate (l_currency, a_date)
					l_weight := l_share * l_price * l_rate
					l_matrix.put (l_weight, l_entities.count, 1)
					l_matrix.put (l_share, l_entities.count, 2)
				end
				i := i + 1
			end
			if l_entities.count > 0 then
				create Result.make_all (l_matrix.sub_matrix_2 (1, l_entities.count, 1, 2), l_entities, label)
			end
		end

	portfolio_name : STRING

	usd : CURRENCY is
		once
			Result := context.usd
		end

	label : ARRAY [STRING]
		once
			Result := (<<"weights", "shares">>)
		end
end
