indexing
	description: "Standard LU decomposition of a matrix"
	author: "Mark Howard"
class
	LU_DECOMPOSITION
create
	make

feature -- Initialization

	make (a_matrix: like matrix) is
		local
			l_rows, l_columns: INTEGER
			i, j, k: INTEGER
			l_lu_col_j: VECTOR
			l_lu_row_i: VECTOR
			l_kmax: INTEGER
			s: DOUBLE
			p: INTEGER
			t: DOUBLE
			v: INTEGER
		do
			matrix := a_matrix
			LU := matrix.to_matrix
			l_rows := matrix.height
			l_columns := matrix.width
			create pivot_vector.make (1, l_rows)
			from i := 1 until i > l_rows loop
				pivot_vector.put (i, i)
				i := i + 1
			end
			pivot_sign := 1
			create l_lu_col_j.make (l_rows)
			from j := 1 until j > l_columns loop
					-- Make a copy of the j-th column to localize references.
				from i := 1 until i > l_rows loop
					l_lu_col_j.put (lu [i, j], i)
					i := i + 1
				end
					-- Apply previous transformations.
				from i := 1 until i > l_rows loop
					l_lu_row_i := LU.i_th_row (i)
					l_kmax := min (i, j);
					s := 0.0
					from k := 1 until k >= l_kmax loop
						s := s + l_lu_row_i.item (k) * l_lu_col_j.item (k)
						k := k + 1
					end
					l_lu_col_j.put (l_lu_col_j.item (i) - s, i)
					l_lu_row_i.put (l_lu_col_j.item (i), j)

				i := i + 1
				end

					-- Find pivot and exchange if necessary.
				p := j;
				from i := j + 1 until i > l_rows loop
					if (abs (l_lu_col_j.item (i)) > abs (l_lu_col_j.item (p))) then
						p := i
					end
					i := i + 1
				end

				if (p /= j) then
					from k := 1 until k > l_columns loop
						t := LU [p, k]
					--	lu.put (lu.item (j, k), p, k)
						lu[p, k] := lu[j, k]
					--	lu.put (t, j, k)
						lu[j, k] := t
						k := k + 1
					end

					v := pivot_vector.item (p)
					pivot_vector.put (pivot_vector.item (j), p)
					pivot_vector.put (v, j)

					pivot_sign := - pivot_sign;
				end

					-- Compute multipliers.
				if (j <= l_rows and LU [j, j] /= 0.0) then
					from i := j + 1 until i > l_rows loop
				--		LU.put (LU [i, j]/ LU [j, j], i, j)
						LU[i, j] := LU [i, j]/ LU [j, j]
					i := i + 1
					end
				end

			j := j + 1
			end
		end
			pivot_sign: INTEGER
			pivot_vector: ARRAY[INTEGER]

	upper_triangular_factor: MATRIX is
		local
			l_rows, l_columns: INTEGER
			x: MATRIX
			i, j: INTEGER
		do
			l_rows := LU.height;
			l_columns := LU.width
			x := create {MATRIX}.make(l_rows, l_columns);
			from i := 1 until i > l_rows loop
				from j := 1 until j > l_columns loop
					if (i <= j) then
				--		x.put (LU.item (i, j), i, j)
						x[i, j] := LU[i, j]
					else
					--	x.put (0, i, j)
						x[i, j] := 0
					end
				j := j + 1
				end
			i := i + 1
			end
			Result := x
		end

		lower_triangular_factor: MATRIX is
			local
				l_rows, l_columns, i, j: INTEGER
				x: MATRIX
			do
				l_rows :=  LU.height
				l_columns := LU.width
				x := create {MATRIX}.make(l_rows, l_columns);
				from i := 1 until i > l_rows loop
					from j := 1 until j > l_columns loop
						if (i > j) then
						--	x.put (LU.item (i, j), i, j)
							x[i, j] := LU[i, j]
						elseif (i = j) then
--							x.put (1.0, i, j)
							x [i, j] := 1.0
						else
							--	X[i,j] = 0.0;
						end
					j := j + 1
					end
				i := i + 1
				end
				Result := x
			end

	determinant: DOUBLE is
			-- Determinant
		local
			j: INTEGER
		do
			Result := pivot_sign
			from j := 1 until j > LU.width loop
				Result := Result * LU [j, j]
			j := j + 1
			end
		end

	is_valid: BOOLEAN
			-- Was the last solve operation valid?


	-- <summary>Solves a set of equation systems of type <c>A * X = B</c>.</summary>
		-- <param name="rhs">Right hand side matrix with as many rows as <c>A</c> and any number of columns.</param>
		-- <returns>Matrix <c>X</c> so that <c>L * L' * X = B</c>.</returns>
		-- <exception cref="T:System.ArgumentException">Matrix dimensions do not match.</exception>
		-- <exception cref="T:System.InvalidOperationException">Matrix is not symmetrix and positive definite.</exception>

	Solve (B: MATRIX): MATRIX is
		local
			l_count: INTEGER
			X: MATRIX
			l_rows, l_columns: INTEGER
			i, j, k: INTEGER
		do
			is_valid := true
			l_count := B.width
			X := B.sub_matrix (pivot_vector, 1, l_count)
			l_rows := LU.height
			l_columns := LU.width

				-- Solve L*Y = B(piv,:)
			from k := 1 until k > l_columns loop
				from i := k + 1 until i > l_columns loop
					from j := 1 until j > l_count loop
					--	X.put (X.item (i, j) - (X.item (k, j) * lu.item (i, k)), i, j)
						X[i, j] := X[i, j] - X[k, j] * lu[i, k]
					j := j + 1
					end
				i := i + 1
				end
			k := k + 1
			end
				-- Solve U*X = Y;
			from k := l_columns until k < 1 loop
				from j := 1 until j > l_count loop
			--		X.put (X.item (k, j) / LU.item (k, k), k, j)
					if lu[k, k] = 0 then
						is_valid := false
					end
					X[k, j] := X[k, j] / LU[k, k]
				j := j + 1
				end
				from i := 1 until i >= k loop
					from j := 1 until j > l_count loop
				--		X.put (X.item (i, j) - (X.item (k, j) * LU.item (i, k)), i, j)
						X[i, j] := X[i, j] - X[k, j] * LU[i, k]
					j := j + 1
					end
				i := i + 1
				end
			k := k - 1
			end
			Result := X
		end

		preimage (v: VECTOR) : VECTOR is
			do
				create Result.make_from_column_matrix (solve (v.to_column_matrix))
			end


feature {NONE} -- Implementation		

	frozen min (d1, d2: INTEGER): INTEGER is
			-- Minimun of d1 and d2
		do
			if (d1 < d2) then
				Result := d1
			else
				Result := d2
			end
		end
	frozen abs (d: DOUBLE): DOUBLE is
		do
			if (d < 0) then
				Result := -d
			else
				Result := d
			end
		end


	LU : MATRIX
	matrix: ABSTRACT_READABLE_MATRIX
end
