note

	description:

		"Object representing the notion of a source for exchange rates"

	author: "Mark Howard"
	last_editor: "$Author: c4 $"
	date: "$Date: 2014/10/24 $"
	revision: "$Revision: #8 $"
	path: "$File: //rose/source/kernel/market/market.e $"

deferred class	MARKET

inherit
	
	ROSE_CONSTANTS

feature -- Access
	
	exchange_rate_asof (a_currency_1, a_currency_2: PERSISTENT_CURRENCY; a_date: ROSE_DATE; a_asof: detachable ROSE_DATE): DOUBLE
			-- Exchange rate from `a_currency_2' to `a_currency_1' at `a_date' as of `a_asof';
			-- If `a_asof' is `Void' then our latest data is used.
		require
			a_currency_1_attached: a_currency_1 /= Void
			a_currency_2_attached: a_currency_2 /= Void
			a_date_attached: a_date /= Void
		deferred
		end

	exchange_rate (a_currency_1, a_currency_2: PERSISTENT_CURRENCY; a_date: ROSE_DATE): DOUBLE
			-- Exchange rate from `a_currency_2' to `a_currency_1' at `a_date'
		require
			a_currency_1_attached: a_currency_1 /= Void
			a_currency_2_attached: a_currency_2 /= Void
			a_date_attached: a_date /= Void
		do
			if attached {CURRENCY} a_currency_1 as l_currency_1 then
				if  attached {CURRENCY} a_currency_2 as l_currency_2 then
					Result := exchange_rate_asof (l_currency_1, l_currency_2, a_date, Void)
				else
					(create {EXCEPTIONS}).raise ("Second PERSISTENT_CURRENCY is not a CURRENCY")				
				end
			else
				(create {EXCEPTIONS}).raise ("First PERSISTENT_CURRENCY is not a CURRENCY")				
			end
		end

	currency_return (a_currency, a_base_currency: CURRENCY; a_start_date, a_end_date: ROSE_DATE): DOUBLE
			-- currency return: to [a_base_currency /a_currency] rate
		local
			l_rate_start, l_rate_end : DOUBLE
		do
			if a_base_currency /= Void and a_currency /= Void and a_start_date /= Void then
				l_rate_start := exchange_rate (a_base_currency, a_currency, a_start_date)
			else
				l_rate_start := legacy_missing_double
			end
			if a_base_currency /= Void and a_currency /= Void and a_end_date /= Void then
				l_rate_end := exchange_rate (a_base_currency, a_currency, a_end_date)
			else
				l_rate_end := legacy_missing_double
			end
			if (is_defined_double (l_rate_start) and then l_rate_start > 0 ) and (is_defined_double (l_rate_end) and then l_rate_end > 0) then
				Result := l_rate_end / l_rate_start - 1.0
			end
		end

	monthly_country_currency_returns(a_country : COUNTRY; a_base_currency : CURRENCY; a_months : ARRAY[ROSE_MONTH]) : VECTOR is
			-- country currency returns
		local
			i : INTEGER
			l_return : DOUBLE
			l_month, l_previous_month : ROSE_MONTH
			l_currency : CURRENCY
		do
			create Result.make (a_months.count)
			from i := a_months.lower until i > a_months.upper loop
				l_month := a_months[i]
				l_previous_month := l_month.previous_month
				l_currency := a_country.currency_history.witness (l_month.last)
				if l_currency /= Void then
					l_return := currency_return(l_currency, a_base_currency, l_previous_month.last, l_month.last)
					Result[ i - a_months.lower + 1] := l_return
				end
				i := i + 1
			end
		end

	monthly_currency_return_matrix(a_countries : ARRAY[COUNTRY]; a_base_currency : CURRENCY; a_months : ARRAY[ROSE_MONTH]) : DATED_MATRIX is
			-- currency return matrix
		local
			i : INTEGER
			l_country : COUNTRY
			l_country_currency_returns : VECTOR
			l_dates: ROSE_LINEAR_ARRAY [ROSE_DATE]
		do
			create Result.make(a_months.count, a_countries.count)
			create l_dates.make (1, 0)
			from i := a_months.lower until i > a_months.upper loop
				l_dates.extend (a_months.item(i).last)
				i := i + 1
			end
			Result.set_dates(l_dates)
			from i := a_countries.lower until i > a_countries.upper loop
				l_country := a_countries[i]
				l_country_currency_returns := monthly_country_currency_returns(l_country, a_base_currency, a_months)
				Result.put_column_vector(l_country_currency_returns, i)
				i := i + 1
			end
		end

	name : STRING is
			-- name of market		
		do
			if name_saved = Void then
				name_saved := generating_type
			end
			Result := name_saved
		end

feature {NONE} -- implementation
	name_saved : STRING
		-- name
end
