note

	description:

		"Object contains all the methods to access zonal market data information"

	author: "Mark Howard"
	last_editor: "$Author: rj $"
	date: "$Date: 2015/04/24 $"
	revision: "$Revision: #142 $"
	path: "$File: //rose/source/kernel/data/market/market_data.e $"

class MARKET_DATA

inherit

	ZONAL_DATA

	ROSE_LEGACY_UNDEFINED

	ROSE_SHARED_GLOBAL
		rename
			error as global_error
		select
			global_error
		end

	EXCEPTIONS

	MEMORY

	ROSE_INVESTMENT_ENTITIES

	ROSE_SHARED_LOGGER
		rename
			error as logger_error
		end

create

	make

feature {NONE} -- Initialization

	make (a_zone: ZONE [LOCATION]) is
			-- make zonal object
		require else
			zone_not_void: a_zone /= Void
		do
			if not (a_zone.is_u_zone or else a_zone.is_e_zone or else a_zone.is_o_zone or else a_zone.is_c_zone or else a_zone.is_j_zone or else a_zone.is_s_zone) then
				do_nothing
			end
			set_cutoff_day_count_for_lookback(config.integer("cutoff_day_count_for_lookback", 200))
			zone := a_zone
			set_rmdat_first_ccyymmdd
			error := 0
			create date.make_undetermined
			if zone.use_persistent_server_as_market_data_source then
				enable_persistent_server
			else
				enable_background
			end
		ensure then
			zone_set: zone = a_zone
		end

feature -- Access

	jul_indexes: ARRAY [INTEGER] is
			--julian date indexes
		do
			Result := hdp.jul_indexes
		end

	adjustments: ARRAY [LINKED_LIST [RAW_ADJUSTMENT]] is
			--adjustment factors
		do
			if new_adjustments /= Void then
				Result := new_adjustments
			else
				Result := hdp.adjustments
			end
		end

	fast_adjustments: HASH_TABLE[ROSE_ADJUSTMENT_HISTORY[LISTING], INTEGER]
	-- All adjustments indexed by entity_id.
	local
		l_history: ROSE_ADJUSTMENT_HISTORY[LISTING]
		l_server: SQL_DATA_SERVER
		l_predicate: COLLECTING_PREDICATE[ADJUSTMENT_INTERVAL_WITH_RXRF[LISTING]]
		l_adjustment: ADJUSTMENT_INTERVAL_WITH_RXRF[LISTING]
		l_array: ROSE_LINEAR_ARRAY[ADJUSTMENT_INTERVAL_WITH_RXRF[LISTING]]
		l_old_rxrf: INTEGER
		l_interval: ADJUSTMENT_INTERVAL[LISTING]
		l_table_name: STRING
		l_timer: ROSE_PROCESS_TIMER
	do
		if fast_adjustments_saved=Void
		then
			debug("use_test_adj")
				create l_timer
			end

			--
			-- Read the entire 'adjustments' table.
			--
			l_server := servers.market_data_servers(zone.mnemonic.out).auxiliary_server
			debug("use_test_adj")
				l_server := create {SQLOLEDB_DATA_SERVER}.make_integrated_security("RCSQL5\x64Devel01", "adjustment_foreground")
			end

			-- The locking in the following transaction is designed to work in cooperation with the only
			-- other place in our code where this table is referenced.  The other is in
			-- FOREGROUND_ADJUSTMENTS_LOADER.  This transaction will block the other (and vice versa) thus
			-- preventing the reading of partial data.  The TABLOCK hint will only block until the statement,
			-- not the transaction, is finished.  Since TABLOCK is a shared lock, only writers are affected.
			l_server.begin_transaction
			l_table_name := "adjustments"
			debug("use_test_adj")
				l_table_name.append("_" + zone.mnemonic.as_lower.out)
			end

			create l_predicate.make(l_server, "select * from " + l_table_name + " with (TABLOCK) order by rxrf, start_date")
			if not l_predicate.success
			then
				raise("Failed to create the adjustments predicate.")
			end

			create l_array.make_empty
			l_predicate.set_container(l_array)
			l_predicate.retrieve(Void, Void)
			if not l_predicate.success
			then
				raise("The adjustments predicate failed.")
			end

			l_server.commit		-- I can't think of any reason to check for success.

			create fast_adjustments_saved.make(zone.max_listing_xrf)

			from
				l_array.start
			until
				l_array.off
			loop
				l_adjustment := l_array.current_item

				if l_old_rxrf/=l_adjustment.rxrf
				then
					if l_history/=Void
					then
						fast_adjustments_saved[l_old_rxrf] := l_history
					end

					l_old_rxrf := l_adjustment.rxrf
					create l_history.make_empty
				end

				create l_interval.make(zone.listing(l_adjustment.rxrf),
									   l_adjustment.start_date, l_adjustment.end_date, l_adjustment.adjustment, l_adjustment.split_factor)
				l_history.add_interval(l_interval)
				l_array.forth
			end

			fast_adjustments_saved[l_old_rxrf] := l_history
			debug("use_test_adj")
				l_timer.display("Loaded fast adjustments")
			end
		end

		Result := fast_adjustments_saved
	end

	fast_adjustments_saved: like fast_adjustments		-- See 'fast_adjustments'.

	use_fast_adjustments
	-- Call this to use the foreground adjustments table (loaded from the new capital actions).
	do
		uses_fast_adjustments := True
	end

	uses_fast_adjustments: BOOLEAN		-- True if we loaded 'fast_adjustments' from the foreground table 'adjustments' in 'market_data_Z'.

	use_new_adjustments is
			-- Enable use of new adjustment factors source
		do
			create new_adjustments.make (zone)
		end

	hdp: HDP is
			-- daily price header
		local
			l_object_predicate: DATA_SERVER_STREAM_COMMAND
--			l_storable_predicate: EXECUTION_PREDICATE
			l_stream: ROSE_STREAM
			l_integer_predicate: INTEGER_PREDICATE
			l_object_size: INTEGER
			l_data_server: SQL_DATA_SERVER
			l_env: EXECUTION_ENVIRONMENT
			l_retry: BOOLEAN
			l_collecting: BOOLEAN
			l_platform: PLATFORM
			l_timer: ROSE_PROCESS_TIMER
			l_eiffel_version: STRING
			l_fake_eiffel_version: BOOLEAN
		do
			if l_retry and l_data_server /= Void then
				l_data_server.rollback
					-- Rollback any partial transactions.
				l_data_server := Void
			end
			if hdp_saved = Void then
				create l_env
				create l_platform
				create l_timer
				if l_platform.is_dotnet then
					create hdp_saved.retrieve (zone)
				elseif not l_retry and (l_env.command_line.argument (0).has_substring ("data_platform") or l_env.command_line.argument (0).has_substring ("ptrade") or l_env.command_line.argument (0).has_substring ("fundamental_variable_server") or l_env.command_line.argument (0).has_substring ("vms_data_replication")) then
						-- FIX ME:
						-- This is ugly but until we can store objects knowing what version was used we are only going to
						-- allow these particular applications to get the header from the database.
					l_eiffel_version := Global.app.info.build_estudio
					if l_eiffel_version = Void then
						l_eiffel_version := "5.6.1218"
						l_fake_eiffel_version := True
					end
					create l_integer_predicate.make (servers.market_data_server, "select max (datalength (object)) as leng from hdp_object where zone = ? and compiler_version = ?")
					l_integer_predicate.retrieve (Void, <<zone.mnemonic.out, l_eiffel_version>>)
					if l_integer_predicate.success then
						l_object_size := l_integer_predicate.array.first.max (0)
					end
					create l_stream
					l_object_predicate := servers.market_data_server.stream_command ("set textsize " + l_object_size.out + "%Nselect object from hdp_object where zone = '" + zone.mnemonic.out + "' and compiler_version = '" + l_eiffel_version + "'")
					l_object_predicate.retrieve (Void)
					if l_object_predicate.query_result.count = 0 then
						create hdp_saved.retrieve (zone)
						if not l_fake_eiffel_version then
						-- per Mauricio 8/10/2010: "IMC_USER was removed as a writer from the db. This call is not really necessary and we can put it back in a smarter way later."
							--l_data_server := servers.new_server_from_repository (servers.market_data_server.logical)
							--create l_storable_predicate.make (l_data_server, "insert into hdp_object (zone,object,compiler_version) values (?,?,?)")
							--if not l_storable_predicate.success then
							--	global_error.abort ("market_data.make", "Failed to create hdp_object store", Void)
							--end
							--l_stream.independent_store (hdp_saved)
							--if l_platform.is_dotnet then
							--	l_eiffel_version := "dotnet"
							--end
							--l_storable_predicate.execute (Void, <<zone.mnemonic, l_stream, l_eiffel_version>>)
							--if not l_storable_predicate.success then
							--	global_error.abort ("market_data.make", "Failed to execute hdp_object store", Void)
							--end
							--l_data_server.commit
							l_data_server := Void
								-- So we don't rollback during a rescue.
						end
					else
						if collecting then
							l_collecting := True
							collection_off
						end
						hdp_saved ?= l_stream.retrieve_from_string (l_object_predicate.query_result)
						if l_collecting then
							collection_on
						end
					end
				else
					create hdp_saved.retrieve (zone)
				end
				if Global.app.is_running_in_estudio then
					logger.info (l_timer.display_string ("Retrieved hdp " + zone.mnemonic.out))
				end
			end
			Result := hdp_saved
		rescue
			if l_collecting then
				collection_on
			end
			if not l_retry then
				l_retry := True
				retry
			end
		end

	set_cutoff_day_count_for_lookback(a_cutoff_day_count_for_lookback: INTEGER)
	-- See 'cutoff_day_count_for_lookback'.
	do
		cutoff_day_count_for_lookback := a_cutoff_day_count_for_lookback
		rxrf := 0		-- To prevent caching that used the previous lookback.
	end

	cutoff_day_count_for_lookback: INTEGER
			-- Max days to lookback from requested market date during price fallback.

feature {NONE} -- Implementation

	new_adjustments: HDP_ADJUSTMENTS_BRIDGE

	first_and_last: ARRAY [FIRST_AND_LAST] is
			--first and last julian dates for all listings
		do
			Result := hdp.first_and_last
		end

	hdp_saved: HDP
			-- Saved version of the HDP created by 'hdp'.

	first_and_last_dates_saved: ROSE_LINEAR_ARRAY [PAIR [ROSE_DATE, ROSE_DATE]]

feature -- NT Background Market Data Access

	enable_background is
			-- Enable the use of the Windows based signature tagged database for retrieval
		local
			l_max_rxrf: INTEGER_PREDICATE
			l_server: SQL_DATA_SERVER
			l_mnemonic: STRING
		do
			use_background := True
			if background_data = Void then
				if attached {MARKET_DATA_I} Current then
					-- There are cases where zone is not set for Indexes
					l_mnemonic := "I"
				elseif attached {MARKET_DATA_F} Current then
					-- There are cases where zone is not set for Futures
					l_mnemonic := "F"
				else
					l_mnemonic := zone.mnemonic.out
				end
				l_server := servers.market_data_servers (l_mnemonic)
				logger.info ("Creating background market data access for zone: " + l_mnemonic)
				l_server.set_implicit_transaction_off
				create background_data.make ("market data access", l_server)
				background_data.set_server_for_writing (l_server)
				logger.info ("Retrieving background market data bounds")
				create l_max_rxrf.make_strict (l_server, "select max (rxrf) as rxrf from data_signatures")
				l_max_rxrf.retrieve (Void, Void)
				if l_max_rxrf.array.first <= 0 then
					raise ("empty market data database")
				end
				create background_bounds.make (1, l_max_rxrf.array.first)
				logger.info ("Background MARKET_DATA enabled")
			end
		end

	disable_persistent_server is
			-- Disable the persistent server
		do
			if uses_persistent then
				uses_persistent := False
				background_data := Void
			end
		end

	enable_persistent_server is
			-- Enable the use of a persistent server accessing the Windows based signature tagged database for retrieval
		local
			l_max_rxrf: INTEGER_PREDICATE
		do
			if not uses_persistent then
				logger.info ("Connecting to MARKET_DATA persistent server for zone: " + zone.mnemonic.out + "...")
				uses_persistent := True
				use_background := True
				enable_lookback_caching
				servers.market_data_servers (zone.mnemonic.out).set_implicit_transaction_off
				create {MARKET_DATA_CLUSTERED_DATABASE} background_data.make_persistent_remote (Void, "market data access", "PERSISTENT_MARKET_DATA_SERVER", <<"-zone=" + zone.mnemonic.out>>, "persistent_market_data_server.exe")
				background_data.set_server_for_writing (servers.market_data_servers (zone.mnemonic.out))
				create l_max_rxrf.make_strict (servers.market_data_servers (zone.mnemonic.out), "select max (rxrf) as rxrf from data_signatures")
				l_max_rxrf.retrieve (Void, Void)
				if l_max_rxrf.array.first <= 0 then
					raise ("empty market data database")
				end
				logger.info ("Creating background market data bounds")
				create background_bounds.make (1, l_max_rxrf.array.first)
				logger.info ("Connected to MARKET_DATA persistent server")
			end
		end

	uses_persistent: BOOLEAN
			-- True if we are using a persistent server.

	use_background: BOOLEAN
			-- Are we using the Windows based signature tagged database for retrieval?

	enable_lookback_caching is
		do
			cache_lookbacks_allowed := True
		end

	disable_lookback_caching is
		do
			cache_lookbacks_allowed := False
		end

	disable_background is
			-- Disable use of the background database.  In this mode you will never get any prices (it used
			-- to get prices from VMS).  It does have an effect on 'currency' and 'first_and_last_dates'.
		do
			use_background := False
		end

	background_access: like background_data is
			-- the background access database
		do
			Result := background_data
		end

	background_bounds_record (a_rxrf: INTEGER): PARTITION_TREE_RECORD [INTEGER, ROSE_DATE] is
			-- Date bounds for 'a_rxrf' based on the existing data in the background
		local
			l_partition: PARTITION_TREE_RECORD [INTEGER, ROSE_DATE]
		do
			if background_bounds.valid_index (a_rxrf) then
				if background_bounds.item (a_rxrf) = Void then
					background_bounds_predicate.retrieve (Void, <<a_rxrf>>)
					if not background_bounds_predicate.success then
						raise ("background_bounds_predicate failed.")
					end
					if background_bounds_predicate.array.is_empty then
						create l_partition
						background_bounds.put (l_partition, a_rxrf)
					else
						background_bounds.put (background_bounds_predicate.array.first, a_rxrf)
					end
				end
				Result := background_bounds.item (a_rxrf)
			end
		end

	last_process_id: INTEGER is
			-- Last available process id for this market data source
		local
			l_predicate: INTEGER_PREDICATE
		do
			create l_predicate.make (servers.market_data_servers (zone.mnemonic.out), "select max (process_id) as int_result from process")
			if not l_predicate.success then
				raise ("Failed to create last_process_id predicate")
			end
			l_predicate.retrieve (Void, Void)
			if not l_predicate.success then
				raise ("Failed to execute last_process_id predicate")
			end
			Result := l_predicate.array.item (1)
		end

	records_touched_since (a_process_id: INTEGER): ROSE_LINEAR_ARRAY [RMDAT_SIGNATURE_RECORD] is
			-- records touched since 'a_process_id'
		local
			l_predicate: COLLECTING_PREDICATE [RMDAT_SIGNATURE_RECORD]
		do
			create l_predicate.make (servers.market_data_servers (zone.mnemonic.out), "select rxrf,date from data_signatures where last_process_id > ?")
			if not l_predicate.success then
				raise ("Failed to create predicate")
			end
			l_predicate.retrieve (Void, <<a_process_id>>)
			if not l_predicate.success then
				raise ("Failed to execute predicate")
			end
			Result := l_predicate.array
		end

	records_touched_since_date_by_listing (a_listing: LISTING; a_date: ROSE_DATE): ROSE_LINEAR_ARRAY [RMDAT_SIGNATURE_RECORD] is
			-- records touched since 'a_date' for 'a_listing'
		require
			valid_args: a_listing /= Void and then a_date /= Void
		local
			l_predicate: COLLECTING_PREDICATE [RMDAT_SIGNATURE_RECORD]
			l_date_predicate: COLLECTING_PREDICATE [SUPPLIER_PROCESS]
		do
			create l_date_predicate.make_strict (servers.market_data_servers (zone.mnemonic.out), "select top 1 * from process where start_time > ? order by start_time")
			l_date_predicate.retrieve (Void, <<a_date>>)
			if not l_date_predicate.array.is_empty then
				create l_predicate.make_strict (servers.market_data_servers (zone.mnemonic.out), "select rxrf, date from data_signatures where last_process_id > ? and rxrf = ? order by date")
				l_predicate.retrieve (Void, <<l_date_predicate.array.first.process_id, a_listing.xrf_code>>)
				Result := l_predicate.array
			else
				create Result.make_empty
			end
		ensure
			valid_result: Result /= Void
		end

	listings_touched_since_date (a_date: ROSE_DATE): ROSE_LINEAR_ARRAY [LISTING] is
			-- array of listings touched since 'a_date'
		require
			valid_args: a_date /= Void
		local
			l_predicate: COLLECTING_PREDICATE [RMDAT_SIGNATURE_RECORD]
			l_date_predicate: COLLECTING_PREDICATE [SUPPLIER_PROCESS]
		do
			create l_date_predicate.make_strict (servers.market_data_servers (zone.mnemonic.out), "select top 1 * from process where start_time > ? order by start_time")
			l_date_predicate.retrieve (Void, <<a_date>>)
			if not l_date_predicate.array.is_empty then
				create l_predicate.make_strict (servers.market_data_servers (zone.mnemonic.out), "select distinct rxrf from data_signatures where last_process_id > ? order by rxrf")
				l_predicate.retrieve (Void, <<l_date_predicate.array.first.process_id>>)
				create Result.make_empty
				from
					l_predicate.array.start
				until
					l_predicate.array.off
				loop
					Result.extend (zone.listing (l_predicate.array.current_item.rxrf))
					l_predicate.array.forth
				end
			else
				create Result.make_empty
			end
		ensure
			valid_result: Result /= Void
		end

	flush_cache is
			-- Flush our caches.
		do
			if background_data /= Void then
				background_data.purge
			end
			price_record_saved := Void
			rxrf := 0
			signature_saved := Void
		end

feature {RMDAT_SIGNATURE_RECORD} -- Background Implementation

	background_data: CLUSTERED_TAGGED_ITEM_DATABASE [INTEGER, ROSE_DATE, RMDAT_SIGNATURE_RECORD]
			-- Source database for Windows "background" data.

feature {NONE} -- Implementation

	actual_day_lp (a_rxrf: INTEGER; a_start_date: ROSE_DATE; a_lookback: INTEGER; a_uses_lookback: BOOLEAN): DOUBLE is
			-- This is the code that does the actual work for 'daylp' and 'day_lp'.  It looks back in time for
			-- a date on which there is a valid price.
		require
			a_start_date_attached: a_start_date /= Void
		local
			l_date: ROSE_DATE
			l_earliest_date: ROSE_DATE
			l_done: BOOLEAN
		do
			if first_and_last.item (a_rxrf) /= Void then
				if a_uses_lookback then
					l_earliest_date := a_start_date.plus_n_date (-a_lookback)
				end
				from
					l_date := a_start_date.twin
				until
					l_done
				loop
					-- First and Last does not include dates prior to rmdat_first_ccyymmdd so do not lookback but
					-- make sure that you are looking at a valid market_date
					lookback_mode := 160
					if l_date.ccyymmdd >= rmdat_first_ccyymmdd then
						retrieve (a_rxrf, l_date, Void)
					else
						retrieve (a_rxrf, l_date, l_date)
					end
					lookback_mode := 0
					-- ALWAYS reset this back to zero.
					l_done := True
					-- 99.999% of the time we will NOT change this in the code below.
					if not a_uses_lookback then
						l_earliest_date := l_date
					end
					if error /= 0 then
						Result := Undefined_Real
					else
						if found_date /= Void and (found_date > l_earliest_date or not a_uses_lookback) then
							if a_start_date.ccyymmdd >= rmdat_first_ccyymmdd then
								adjustment := adjust_interval (a_rxrf, julian_index (found_date.rose_julian_day), juliana_index (a_start_date.rose_julian_day))
								if adjustment <= 0.0 then
									adjusted := False
								else
									adjusted := True
								end
							else
								-- There are no adjustments prior to rmdat_first_ccyymmdd for CRSP_U
								adjusted := True
								adjustment := 1.0
							end
							if error = 0 then
								if (has_close and adjusted) then
									Result := record_close * adjustment
								elseif (has_bid and adjusted) then
									Result := record_bid * adjustment
								elseif (has_ask and adjusted and zone.mnemonic /= 'J') then
									Result := record_ask * adjustment
								else
									Result := Undefined_Real
									l_date := found_date.plus_n_date (-1)
									-- Try again one day earlier than the date on the record that was found.
									if not a_uses_lookback or l_date > l_earliest_date then
										l_done := False
									else
										error := 1011
									end
								end
							else
								Result := Undefined_Real
							end
						else
							Result := Undefined_Real
						end
					end
				end
			else
				Result := Undefined_Real
				found_date := Void
			end
		end

	background_bounds_predicate: COLLECTING_PREDICATE [PARTITION_TREE_RECORD [INTEGER, ROSE_DATE]] is
			-- Predicate to get date bounds for a listing
		do
			if background_bounds_predicate_save = Void then
				create background_bounds_predicate_save.make (servers.market_data_servers (zone.mnemonic.out), "select min (date) as min_1, max (date) as max_1, rxrf as min_0 from data_signatures where rxrf = ? group by rxrf")
				if not background_bounds_predicate_save.success then
					raise ("Failed to create background_bounds_predicate")
				end
			end
			Result := background_bounds_predicate_save
		end

	background_bounds_predicate_save: COLLECTING_PREDICATE [PARTITION_TREE_RECORD [INTEGER, ROSE_DATE]]

	background_bounds: ROSE_LINEAR_ARRAY [PARTITION_TREE_RECORD [INTEGER, ROSE_DATE]]

	last_interval_date_predicate: DATE_PREDICATE
			-- Database predicate for use by `last_interval_date'

	last_interval_date (a_listing: LISTING; a_date: ROSE_DATE): ROSE_DATE is
		do
			if last_interval_date_predicate = Void then
				create last_interval_date_predicate.make (servers.market_data_servers (zone.mnemonic.out), "select max (date) as date from data_signatures where rxrf = ? and date < ?")
				if not last_interval_date_predicate.success then
					raise ("Could not create last_interval_date_predicate")
				end
			end
			last_interval_date_predicate.retrieve (Void, <<a_listing.xrf_code, a_date>>)
			if not last_interval_date_predicate.success then
				raise ("Could not retrieve with last_interval_date_predicate")
			end
			if last_interval_date_predicate.array.first /= Void then
				Result := last_interval_date_predicate.array.first
			else
				create Result.make_ccyymmdd (18000101)
			end
		end

feature {RMDAT_INSTRUMENT} -- Implementation

	lookback_mode: INTEGER

	set_lookback_mode (a_value: like lookback_mode) is
			-- See 'ok_for_lookback_imp'.
		do
			lookback_mode := a_value
		end

feature {NONE} -- Implementation

	entity_from_rxrf (a_rxrf: INTEGER): INVESTMENT_ENTITY is
			-- The listing entity for `a_rxrf'
		do
			Result := context.zone (zone.mnemonic).listing (a_rxrf)
			if Result = Void or not Result.is_valid then
				Result := context.zone (zone.mnemonic).fund_listing (a_rxrf)
				if Result = Void or not Result.is_valid then
					Result := context.zone (zone.mnemonic).bond_listing (a_rxrf)
				end
			end
		end

	background_record: ROSE_VIRTUAL_RECORD [RMDAT_SIGNATURE_RECORD]

	cache_lookbacks_allowed: BOOLEAN

	signature: RMDAT_SIGNATURE_RECORD is
		do
			if signature_saved = Void then
				create signature_saved
				signature_saved.set_zone (zone)
			end
			Result := signature_saved
		end

	signature_saved: RMDAT_SIGNATURE_RECORD

	last_lookback_mode: INTEGER

	ok_for_lookback (a_signature: RMDAT_SIGNATURE_RECORD): BOOLEAN is
			-- Is `a_signature' valid to stop looking back?
		require
			a_signature_not_void: a_signature /= Void
		do
			Result := ok_for_lookback_imp (a_signature.open, a_signature.close, a_signature.high, a_signature.low, a_signature.bid, a_signature.ask)
		end

	ok_for_lookback_imp (a_open, a_close, a_high, a_low, a_bid, a_ask: DOUBLE): BOOLEAN is
		local
			l_close_status: BOOLEAN
		do
			l_close_status := is_defined_double (a_close) and then a_close > 0.0
				-- Modes 160 and 170 come from DO loops in B_RMDAT_SRC:RMDAT_INTERNAL.FOR (see DO 160).
			inspect lookback_mode
			when 0 then
				inspect czone
				when 'S' then
					Result := l_close_status
				else
					Result := l_close_status or (is_defined_double (a_bid) and then a_bid > 0.0)
				end
			when 160 then
				inspect czone
				when 'U', 'C' then
					Result := l_close_status or (is_defined_double (a_bid) and then a_bid > 0.0)
				when 'J' then
					Result := l_close_status or (is_defined_double (a_bid) and then a_bid > 0.0)
				when 'E', 'O' then
					Result := l_close_status or (is_defined_double (a_high) and then a_high > 0.0) or (is_defined_double (a_low) and then a_low > 0.0) or (is_defined_double (a_bid) and then a_bid > 0.0) or (is_defined_double (a_ask) and then a_ask > 0.0)
				when 'S' then
					Result := l_close_status
				else
					Result := l_close_status
				end
			when 170 then
				inspect czone
				when 'U', 'C', 'J' then
					Result := l_close_status
				when 'E', 'O' then
					Result := l_close_status or (is_defined_double (a_high) and then a_high > 0.0) or (is_defined_double (a_low) and then a_low > 0.0) or (is_defined_double (a_bid) and then a_bid > 0.0) or (is_defined_double (a_ask) and then a_ask > 0.0)
				when 'S' then
					Result := l_close_status
				else
					Result := l_close_status
				end
			end
		end

	should_retrieve (a_rxrf: INTEGER; a_date: ROSE_DATE; a_lookback: BOOLEAN): BOOLEAN is
		require
			a_date_not_void: a_date /= Void
		do
				-- Did the target change?
			Result := rxrf /= a_rxrf or else not (signature.date /= Void and then a_date.same_date (signature.date))
				-- Is the cached record valid for this non-lookback call?
			if not Result then
				Result := not a_lookback and price_record_saved /= Void and then (price_record_saved.date /= Void and then not price_record_saved.date.same_date (a_date))
			end
				-- Did the lookback mode change?
			if not Result then
				Result := (a_lookback and last_lookback_mode /= lookback_mode)
			end
		end

	background_retrieve (a_rxrf: INTEGER; a_date: ROSE_DATE; a_start: ROSE_DATE) is
		require
			background_enabled: use_background
			valid_date: a_date /= Void
		local
			l_signature: RMDAT_SIGNATURE_RECORD
			l_lookback: BOOLEAN
			l_date: ROSE_DATE
			l_first, l_last: ROSE_DATE
			l_bounds: PARTITION_TREE_RECORD [INTEGER, ROSE_DATE]
			l_reuse: BOOLEAN
			l_entity: INVESTMENT_ENTITY
			l_virtual_recs: ROSE_LINEAR_ARRAY [ROSE_VIRTUAL_RECORD [RMDAT_SIGNATURE_RECORD]]
		do
			l_lookback := a_start = Void or else not a_date.is_equal (a_start)
			if should_retrieve (a_rxrf, a_date, l_lookback) then
				last_lookback_mode := lookback_mode
				if signature.rxrf /= a_rxrf or last_listing = Void then
					rxrf := a_rxrf
					last_listing := zone.listing (a_rxrf)
					signature.set_rxrf (a_rxrf)
					l_entity := entity_from_rxrf (a_rxrf)
					if l_entity /= Void and l_entity.is_valid then
						signature.set_entity_id (l_entity.handle)
					else
						signature.set_entity_id (-1)
					end
				end
				if last_listing.is_valid or True then
						-- handle may sometimes be 0.
					error := 0
						--signature.set_date (create {ROSE_DATE}.make_by_date (a_date))
					signature.set_date (a_date)
					background_record := background_data.record_by_signature (signature)
					if l_lookback and (background_record = Void or else not ok_for_lookback (background_record.signature)) then
						l_bounds := background_bounds_record (a_rxrf)
						if l_bounds /= Void and then l_bounds.min_0 /= 0 then
							l_first := l_bounds.min_1
							if a_start /= Void then
								l_first := l_first.max (create {ROSE_DATE}.make_by_date (a_start))
							end
							l_last := l_bounds.max_1
							if a_date >= l_first and a_date.rose_julian_day - l_last.rose_julian_day < cutoff_day_count_for_lookback then
								if a_date > l_last then
									l_date := l_last.twin
								else
									if a_start = Void or else not a_start.is_equal (a_date.previous_date) then
										l_date := last_interval_date (last_listing, a_date)
									else
										if a_start /= Void then
											l_date := a_start.twin
										end
									end
								end
								from
								until
									l_signature /= Void or (l_date < l_first) or equal (l_date, a_start)
								loop
									signature.set_date (l_date)
										--										background_record := background_data.ad_hoc_record_by_signature (signature)
									if cache_lookbacks_allowed then
										background_record := background_data.record_by_signature (signature)
									else
										if zone.mnemonic = 'S' then
											l_virtual_recs := background_data.retrieve_signature_with_limit (signature, "#date.id >= date and #rxrf.id = rxrf and [close] > 0.0", Void, 1)
										else
											l_virtual_recs := background_data.retrieve_signature_with_limit (signature, "#date.id >= date and #rxrf.id = rxrf", Void, 1)
												--I would really like to implement 'l_go_back_partial' (see MARKET_DATA_CLIENT).
												--"#date.id >= date and #rxrf.id = rxrf and (" + l_go_back_partial + ")",
										end
										if l_virtual_recs.count /= 0 then
											background_record := l_virtual_recs.first
										else
											background_record := Void
										end
									end
									if background_record /= Void and then background_record.signature /= Void then
										l_date := background_record.signature.date.previous_date
									else
										l_date := l_date.previous_date
									end
									l_date := zone.last_market_day (l_date)
									if l_date /= Void then
										l_date := l_date.twin
									end
									if background_record /= Void and ok_for_lookback (background_record.signature) then
										l_signature := background_record.signature
									else
										l_signature := Void
										background_record := Void
									end
								end
							end
						else
							background_record := Void
						end
					end
				else
					background_record := Void
				end
				record_index := 1
				if not l_reuse then
					if background_record /= Void then
						l_signature := background_record.signature
						found_date := l_signature.date
						record_index := 1
						adjust (a_rxrf, found_date)
						if rmdata_record = Void then
							create rmdata_record
						else
							rmdata_record.reset
						end
						if is_defined_double (l_signature.close) then
							rmdata_record.set_close (l_signature.close)
							rmdata_record.set_has_close (True)
						end
						if is_defined_double (l_signature.open) then
							rmdata_record.set_open (l_signature.open)
							rmdata_record.set_has_open (True)
						end
						if is_defined_double (l_signature.high) then
							rmdata_record.set_high (l_signature.high)
							rmdata_record.set_has_high (True)
						end
						if is_defined_double (l_signature.low) then
							rmdata_record.set_low (l_signature.low)
							rmdata_record.set_has_low (True)
						end
						if is_defined_double (l_signature.bid) then
							rmdata_record.set_bid (l_signature.bid)
							rmdata_record.set_has_bid (True)
						end
						if is_defined_double (l_signature.ask) then
							rmdata_record.set_ask (l_signature.ask)
							rmdata_record.set_has_ask (True)
						end
						if is_defined_double (l_signature.volume) then
							rmdata_record.set_vol (l_signature.volume)
							rmdata_record.set_has_vol (True)
						end
						create price_record_saved
						price_record_saved.set (last_listing, Current, rmdata_record, found_date, a_date.twin, Void, adjustment)
						if last_currency = Void or else l_signature.currency /= last_currency.handle then
							if is_defined_integer (l_signature.currency) then
								last_currency ?= context.entity_at (ids.currency_category_id, l_signature.currency)
							else
								last_currency := default_currency (l_signature.rxrf, l_signature.date)
							end
						end
						price_record_saved.set_currency (last_currency)
						if l_lookback then
							price_record_saved.set_validity (ok_for_lookback (l_signature))
						else
							price_record_saved.set_validity (True)
						end
					else
						date := a_date.twin
							-- Set 'date' BEFORE call to 'invalid_record'.
						price_record_saved := invalid_record (a_rxrf, a_date)
						rmdata_record := Void
						found_date := Void
					end
				else
					price_record_saved.set_as_of_date (a_date.twin)
				end
				date := a_date.twin
			end
		end

feature -- Records

	price_record_saved: like price_record
			-- Result of last call to `retrieve'

	price_record (a_rxrf: INTEGER; a_date: ROSE_DATE; a_start: ROSE_DATE): detachable RMDAT_PRICE_RECORD is
		require
			a_date_attached: a_date /= Void
		do
			retrieve (a_rxrf, a_date, a_start)
			Result := price_record_saved
		end

	target_is_valid (a_rxrf: INTEGER; a_date: ROSE_DATE; a_lookback: BOOLEAN): BOOLEAN is
		do
				-- Is date after first zonal price date?
			Result := a_date.ccyymmdd >= rmdat_first_ccyymmdd
				-- Is date before listing's last price date?
			if Result then
				Result := julian_index (a_date.rose_julian_day) <= first_and_last.item (a_rxrf).last
					-- If it is, are we looking back?
				if not Result then
					Result := a_lookback
				end
			end
		end

	retrieve (a_rxrf: INTEGER; a_date: ROSE_DATE; a_start: ROSE_DATE) is
			-- Retrieve price for `a_rxrf' in this zone for `a_date'
			-- looking back up to `a_start'
		require
			valid_date: a_date /= Void
		local
			l_first_and_last: FIRST_AND_LAST
		do
			l_first_and_last := first_and_last.item (a_rxrf)
			if use_background and l_first_and_last /= Void then
				background_retrieve (a_rxrf, a_date, a_start)
			else
					-- Make an invalid record.
				date := a_date.twin
					-- Set 'date' BEFORE call to 'invalid_record'.
				rmdata_record := Void
				price_record_saved := invalid_record (a_rxrf, date)
			end
		end

	valid_rmdata_record: BOOLEAN is
			-- True if the record has at least one value.
		do
			if rmdata_record /= Void then
				if (is_defined_double (rmdata_record.close) and rmdata_record.close > 0.0) or
				   (is_defined_double (rmdata_record.open) and rmdata_record.open > 0.0) or
				   (is_defined_double (rmdata_record.high) and rmdata_record.high > 0.0) or
				   (is_defined_double (rmdata_record.low) and rmdata_record.low > 0.0) or
				   (is_defined_double (rmdata_record.bid) and rmdata_record.bid > 0.0) or
				   (is_defined_double (rmdata_record.ask) and rmdata_record.ask > 0.0)
				then
					Result := True
				end
			end
		end

feature {NONE} -- Implementation

	invalid_record (a_rxrf: INTEGER; a_date: ROSE_DATE): RMDAT_PRICE_RECORD is
		do
			create Result
			adjust (a_rxrf, a_date)
			Result.set (zone.listing (a_rxrf), Current, Void, constants.minimum_date_time, date, Void, adjustment)
			Result.set_validity (False)
		end

feature -- Basic operations

	first_and_last_dates (a_rxrf: INTEGER): PAIR [ROSE_DATE, ROSE_DATE] is
		local
			l_first_and_last: FIRST_AND_LAST
			l_first_date, l_last_date: ROSE_DATE
			l_index: INTEGER
			l_julian_day: INTEGER
			l_predicate: SELECTION_PREDICATE [TRIPLE [ROSE_DATE, ROSE_DATE, INTEGER]]
			l_template: TRIPLE [ROSE_DATE, ROSE_DATE, INTEGER]
			l_rxrf_first_and_last: ROSE_LINEAR_ARRAY [TRIPLE [ROSE_DATE, ROSE_DATE, INTEGER]]
			l_rxrf: INTEGER
			l_date: ROSE_DATE
		do
			if use_background then
				if first_and_last_dates_saved = Void then
					create l_template.make (create {ROSE_DATE}.make_now, create {ROSE_DATE}.make_now, 0)
					create l_predicate.make (servers.market_data_servers (zone.mnemonic.out), l_template, "select rxrf as third, first, last as second from first_and_last")
					if not l_predicate.success then
						raise ("Failed to make the first_and_last predicate.")
					end
					create l_rxrf_first_and_last.make_empty
					l_predicate.set_container (l_rxrf_first_and_last)
					l_predicate.retrieve (Void, Void)
					if not l_predicate.success then
						raise ("The first_and_last predicate failed during execution.")
					end
					from
						create first_and_last_dates_saved.make_empty
						l_rxrf_first_and_last.start
					until
						l_rxrf_first_and_last.off
					loop
						l_rxrf := l_rxrf_first_and_last.current_item.third
						if hdp.adjustment_bounds.valid_index (l_rxrf) and then hdp.adjustment_bounds.item (l_rxrf) /= Void then
							l_date := hdp.adjustment_bounds.item (l_rxrf).start_date
							if l_date /= Void then
								l_first_date := l_rxrf_first_and_last.current_item.first.min (l_date)
							else
								l_first_date := l_rxrf_first_and_last.current_item.first
							end
							l_date := hdp.adjustment_bounds.item (l_rxrf).end_date
							if l_date /= Void then
								l_last_date := l_rxrf_first_and_last.current_item.second.max (l_date)
							else
								l_last_date := l_rxrf_first_and_last.current_item.second
							end
						else
							l_first_date := l_rxrf_first_and_last.current_item.first
							l_last_date := l_rxrf_first_and_last.current_item.second
						end
						first_and_last_dates_saved.force (create {PAIR [ROSE_DATE, ROSE_DATE]}.make (l_first_date, l_last_date), l_rxrf)
						l_rxrf_first_and_last.forth
					end
				end
				if first_and_last_dates_saved.valid_index (a_rxrf) then
					Result := first_and_last_dates_saved.item (a_rxrf)
				end
			else
				l_first_and_last := first_and_last.item (a_rxrf)
				if l_first_and_last /= Void then
					l_index := l_first_and_last.first
					if julian_days.valid_index (l_index) then
						l_julian_day := julian_days.item (l_index)
						if l_julian_day = 0 then
							from
							until
								l_julian_day > 0
							loop
								l_index := l_index + 1
								l_julian_day := julian_days.item (l_index)
							end
						end
						create l_first_date.make_from_rose_julian_day (l_julian_day)
					end
					l_index := l_first_and_last.last
					if julian_days.valid_index (l_index) then
						l_julian_day := julian_days.item (l_index)
						create l_last_date.make_from_rose_julian_day (l_julian_day)
					end
					create Result.make (l_first_date, l_last_date)
				else
					Result := Void
				end
			end
		end

	julian_days: ARRAY [INTEGER] is
		do
			Result := hdp.julian_days
		end

feature {NONE} -- Implementation

	adj (a_rxrf: INTEGER; a_date: ROSE_DATE): DOUBLE is
			-- adjustment for 'a_rxrf' on 'a_date'
		do
			retrieve (a_rxrf, a_date, a_date)
			Result := adjustment
				--	 if has_adj then
				--	    Result := record_adj
				--	 else
				--	    Result := undefined_real
				--	 end
		end

	open (a_rxrf: INTEGER; a_date: ROSE_DATE): DOUBLE is
			-- open for 'a_rxrf', 'a_date'
		do
			retrieve (a_rxrf, a_date, a_date)
			if has_open then
				Result := record_open
			else
				Result := undefined_real
			end
		end

	close (a_rxrf: INTEGER; a_date: ROSE_DATE): DOUBLE is
			--close for 'a_rxrf', 'a_date'
		do
			retrieve (a_rxrf, a_date, a_date)
			if has_close then
				Result := record_close
			else
				Result := undefined_real
			end
		end

	high (a_rxrf: INTEGER; a_date: ROSE_DATE): DOUBLE is
			--high for 'a_rxrf', 'a_date'
		do
			retrieve (a_rxrf, a_date, a_date)
			if has_high then
				Result := record_high
			else
				Result := undefined_real
			end
		end

	low (a_rxrf: INTEGER; a_date: ROSE_DATE): DOUBLE is
			--low for 'a_rxrf', 'a_date'
		do
			retrieve (a_rxrf, a_date, a_date)
			if has_low then
				Result := record_low
			else
				Result := undefined_real
			end
		end

	bid (a_rxrf: INTEGER; a_date: ROSE_DATE): DOUBLE is
			--bid for 'a_rxrf', 'a_date'
		do
			retrieve (a_rxrf, a_date, a_date)
			if has_bid then
				Result := record_bid
			else
				Result := undefined_real
			end
		end

	ask (a_rxrf: INTEGER; a_date: ROSE_DATE): DOUBLE is
			--ask for 'a_rxrf', 'a_date'
		do
			retrieve (a_rxrf, a_date, a_date)
			if has_ask then
				Result := record_ask
			else
				Result := undefined_real
			end
		end

	volume (a_rxrf: INTEGER; a_date: ROSE_DATE): DOUBLE is
			--volume for 'a_rxrf', 'a_date'
		do
			retrieve (a_rxrf, a_date, a_date)
			if has_volume then
				Result := record_volume
			else
				Result := undefined_real
			end
		end

feature {NONE} -- Implementation

	last_currency: CURRENCY

feature -- Currency

	currency (a_rxrf: INTEGER; a_date: ROSE_DATE): CURRENCY is
			--currency for 'a_rxrf', 'a_date'
		require
			a_date_not_void: a_date /= Void
		local
				--l_currencies: ROSE_LINEAR_ARRAY[CURRENCY]
			l_country: COUNTRY
			l_listing: LISTING
		do
			if rxrf /= a_rxrf or else not a_date.same_date (date) then
				retrieve (a_rxrf, a_date, Void)
			end
			if use_background then
				if price_record_saved /= Void then
					Result := price_record_saved.currency
				end
			elseif rmdata_record /= Void then
					--and then hdp.currencies.valid_index (a_rxrf)
				Result := hdp.currency (a_rxrf, rmdata_record.fx_quote + 1, a_date)
					--l_currencies := hdp.currencies.item (a_rxrf)
					--if l_currencies.valid_index (rmdata_record.fx_quote + 1) then
					--if l_currencies.item (rmdata_record.fx_quote + 1) /= Void then
					--Result := l_currencies.item (rmdata_record.fx_quote + 1)
					--end
					--end
					--if rmdata_record.fx_quote = 0 and l_currencies.has (Euro) then
				if rmdata_record.fx_quote = 0 and hdp.has_euro (a_rxrf) then
					l_listing := zone.listing (a_rxrf)
					if l_listing /= Void and then l_listing.is_valid then
						l_country := l_listing.country
						if l_country = Void then
							print ("RXRF " + a_rxrf.out + zone.mnemonic.out + " is not related to a country%N")
							if Result /= Void then
								l_country := Result.country_to_currency_co_history.first.entity
							else
								print ("RXRF " + a_rxrf.out + " has no currency from which to get a country%N")
							end
						end
						if l_country /= Void and l_country.is_member_of_european_union and l_country.official_currency (a_date) /= Void and l_country.official_currency (a_date).iso.is_equal (Euro.iso) then
							Result := l_country.official_currency (a_date)
						end
					else
						print ("RXRF " + a_rxrf.out + " is not a valid entity%N")
					end
				end
			end
			if Result = Void or else not Result.is_known_currency then
				if zone.mnemonic.is_equal ('U') then
					Result := usd
				elseif zone.mnemonic.is_equal ('C') then
					Result := cad
				elseif zone.mnemonic.is_equal ('J') then
					Result := yen
				end
			end
		end

	default_currency (a_rxrf: INTEGER; a_date: ROSE_DATE): CURRENCY is
			--default currency for 'a_rxrf', 'a_date'
		local
			l_official_currency: CURRENCY
			l_country: COUNTRY
			l_listing: LISTING
		do
			if zone.is_u_zone then
				Result := usd
			elseif zone.is_c_zone then
				Result := cad
			elseif zone.is_j_zone then
				Result := yen
			else
				l_listing := zone.listing (a_rxrf)
				if l_listing /= Void then
					if zone.is_e_zone then
						l_country := l_listing.country
						if l_country /= Void and then l_country.is_member_of_european_union then
							l_official_currency := l_country.official_currency (a_date)
							if equal (l_official_currency, euro) then
								if hdp.has_euro (a_rxrf) then
									Result := euro
								end
							end
						end
					end
					if Result = Void then
						Result := hdp.currency (a_rxrf, 1, a_date)
					end
					if Result = Void then
						Result := listing_country_currency (a_rxrf, a_date)
					end
				end
			end
		end

	listing_country_currency (a_rxrf: INTEGER; a_date: ROSE_DATE): CURRENCY is
			--listing country currency for 'a_rxrf', 'a_date'
		local
			l_country: COUNTRY
			l_listing: LISTING
		do
			l_listing := zone.listing (a_rxrf)
			if l_listing /= Void then
				l_country := l_listing.country
				if l_country /= Void then
					if l_country.is_member_of_european_union then
						Result := l_country.official_currency (a_date)
					else
						Result := l_country.country_to_currency_history.witness (now)
					end
				end
			end
		end

feature -- Adjustments

	juliana_index (a_julian: INTEGER): INTEGER is
			--juliana index for 'a_julian'
		local
			i: INTEGER
		do
			if jul_indexes.valid_index (a_julian) then
				from
					i := a_julian
				until
					i < 1 or else jul_indexes.item (i) > 0
				loop
					i := i - 1
				end
				if i > 0 then
					Result := jul_indexes.item (i)
				end
			end
		end

feature {NONE} -- Implementation

	adjust (a_rxrf: INTEGER; a_date: ROSE_DATE) is
			-- adjusts for 'a_rxrf', 'a_date'
		require
			date_not_void: a_date /= Void
		local
			l_valid_index: BOOLEAN
			l_adjustments: LINKED_LIST [RAW_ADJUSTMENT]
			l_index: INTEGER
		do
				-- temporary: the post condition on valid_index will fail otherwise as the collection of new_adjustments is currently not built
			if new_adjustments /= Void then
				l_valid_index := True
			else
				if adjustments /= Void then
					l_valid_index := adjustments.valid_index (a_rxrf)
				end
			end
			if a_date.ccyymmdd >= rmdat_first_ccyymmdd then
				if adjustments /= Void and then l_valid_index then
					l_adjustments := adjustments.item (a_rxrf)
					l_index := juliana_index (a_date.rose_julian_day)
					if l_adjustments = Void or else l_adjustments.is_empty then
						adjusted := False
						adjustment := undefined_double
						error := 1008
					elseif (new_adjustments /= Void and then l_index < l_adjustments.last.lowdate_index) or else (new_adjustments = Void and then l_index < l_adjustments.first.lowdate_index) then
							-- a kludge to deal with the in consistent order between new_adjustments and adjustments
						adjusted := False
						adjustment := undefined_double
						error := 1009
					else
						adjusted := False
						from
							l_adjustments.start
						until
							l_adjustments.after or else (adjusted and then l_index > l_adjustments.item.highdate_index)
						loop
							if (l_adjustments.item.lowdate_index <= l_index and then l_index <= l_adjustments.item.highdate_index) then
								adjustment := l_adjustments.item.backadj
								adjusted := True
							end
							l_adjustments.forth
						end
						if not adjusted then
							adjustment := undefined_double
						end
					end
				else
					adjusted := False
					adjustment := undefined_double
				end
			else
				adjusted := True
				adjustment := 1.0
			end
		end

feature -- adjustment interval history

	adjust_interval (a_rxrf: INTEGER; a_start_jul, a_end_jul: INTEGER): DOUBLE is
			--adjustment interval for 'a_rxrf', a_period ['a_start_jul','a_end_jul']
		local
			l_adjustments: LINKED_LIST [RAW_ADJUSTMENT]
			l_factor: DOUBLE
			l_start_jul: INTEGER
			l_end_jul: INTEGER
		do
			if (a_start_jul = a_end_jul) then
				Result := 1.0
			else
				if a_start_jul < a_end_jul then
					l_start_jul := a_start_jul
					l_end_jul := a_end_jul
				else
					l_start_jul := a_end_jul
					l_end_jul := a_start_jul
				end
				if adjustments.valid_index (a_rxrf) then
					l_adjustments := adjustments.item (a_rxrf)
				end
				if l_adjustments /= Void and then l_adjustments.count /= 0 and then l_start_jul >= l_adjustments.first.lowdate_index and record_index /= 0 then
					from
						l_adjustments.start
					until
						l_adjustments.off or l_adjustments.item.highdate_index >= l_start_jul
					loop
						l_adjustments.forth
					end
					if l_adjustments.item.highdate_index >= l_end_jul then
						Result := 1.0
							-- Start and end are both in the same interval.
					else
						Result := l_adjustments.item.split_factor
						if Result <= 0.0 then
							Result := ieee_undefined.qnan
						else
							Result := 1.0 / Result
						end
						from
							l_adjustments.forth
						until
							l_adjustments.off or (l_adjustments.item.lowdate_index <= l_end_jul and l_end_jul <= l_adjustments.item.highdate_index)
						loop
							l_factor := l_adjustments.item.split_factor
							if l_factor <= 0.0 then
								l_factor := ieee_undefined.qnan
							end
							Result := Result / l_factor
							l_adjustments.forth
						end
					end
				else
					Result := ieee_undefined.qnan
				end
				if not ieee_undefined.is_defined_double (Result) then
					Result := 1.0
						-- This hack should be removed someday so that people know
						-- that we could not determine the adjustment.
				end
			end
		end

feature -- adjustment interval history

	adjustment_history (a_rxrf: INTEGER): ROSE_ADJUSTMENT_HISTORY [LISTING] is
			--use new adjustment_history if avalable
			--adjustments is in different order under old and new
		local
			l_adjustments: LINKED_LIST [RAW_ADJUSTMENT]
			l_raw_adjustment: RAW_ADJUSTMENT
			l_interval, l_previous_interval: ADJUSTMENT_INTERVAL [LISTING]
			l_listing: LISTING
			l_start_date, l_end_date, l_previous_date: ROSE_DATE
			l_result: ROSE_ADJUSTMENT_HISTORY[LISTING]
		do
			l_listing := zone.listing (a_rxrf)
			create Result.make_empty
			if l_listing /= Void then
				if uses_fast_adjustments
				then
					l_result := fast_adjustments[a_rxrf]
					if l_result/=Void
					then
						Result := l_result
					end
				elseif new_adjustments = Void then
					l_adjustments := adjustments.item (a_rxrf)
					if l_adjustments /= Void then
						from
							l_adjustments.start
						until
							l_adjustments.off
						loop
							l_raw_adjustment := l_adjustments.item
							if l_raw_adjustment /= Void then
								l_start_date := jul_index_to_date (l_raw_adjustment.lowdate_index)
								if l_start_date = Void then
									l_start_date := constants.minimum_date_time
								else
									l_previous_date := l_start_date.plus_n_date (-1)
									if l_previous_interval /= Void then
										l_previous_interval.set_end_date (l_start_date.previous_date)
									end
								end
								l_end_date := jul_index_to_date (l_raw_adjustment.highdate_index)
								if l_end_date = Void then
									l_end_date := constants.maximum_date_time
								end
								create l_interval.make (l_listing, l_start_date, l_end_date, l_raw_adjustment.backadj, l_raw_adjustment.split_factor)
								l_previous_interval := l_interval
								Result.extend (l_interval)
							end
							l_adjustments.forth
						end
					end
				else
					Result := l_listing.new_adjustment_history
				end
			end
		end

	jul_index_to_date (a_ix: INTEGER): ROSE_DATE is
			-- date for jul_index 'a_ix'
		local
			l_rose_jul_day: INTEGER
		do
			l_rose_jul_day := reverse_julian_table.item (a_ix)
			if l_rose_jul_day > 0 then
				create Result.make_from_rose_julian_day (l_rose_jul_day)
			end
		end

	reverse_julian_table: HASH_TABLE [INTEGER, INTEGER] is
			-- Julian index to index
		local
			i: INTEGER
		do
			if reverse_julian_table_saved = Void then
				create reverse_julian_table_saved.make (jul_indexes.count * 2)
				from
					i := jul_indexes.lower
				until
					i > jul_indexes.upper
				loop
					if jul_indexes.item (i) > 0 then
						reverse_julian_table_saved.force (i, jul_indexes.item (i))
					end
					i := i + 1
				end
			end
			Result := reverse_julian_table_saved
		end

	reverse_julian_table_saved: HASH_TABLE [INTEGER, INTEGER]

feature -- Pricing

	daylp (a_rxrf: INTEGER; a_date: ROSE_DATE): DOUBLE is
			-- daylp for 'a_rxrf', 'a_date'
		require
			a_date_attached: a_date /= Void
		do
			Result := actual_day_lp (a_rxrf, a_date, 0, False)
		end

	day_lp (a_rxrf: INTEGER; a_date: ROSE_DATE; a_lookback: INTEGER): DOUBLE is
			-- dayip for 'a_rxrf', 'a_date'
		require
			a_date_attached: a_date /= Void
		do
			Result := actual_day_lp (a_rxrf, a_date, a_lookback, True)
		end

	xdaylp (a_rxrf: INTEGER; a_date: ROSE_DATE): DOUBLE is
			-- xdaylp for 'a_rxrf', 'a_date'
		require
			a_date_attached: a_date /= Void
		local
			l_saved_lookback_mode: INTEGER
		do
			l_saved_lookback_mode := lookback_mode
			set_lookback_mode (160)
			if attached price_record (a_rxrf, a_date, Void) as la_price_record then
				Result := la_price_record.unadjusted_close_bid_ask
			else
				Result := qnan
			end
			set_lookback_mode (l_saved_lookback_mode)
		end

feature -- Capital Actions

	capital_actions (a_rxrf: INTEGER): ROSE_LINEAR_ARRAY [FOREGROUND_DIVS__W] is
			-- capital actions for 'a_rxrf'
		do
			create Result.make_empty
		end

	get_latest_daily_price (a_rxrf: INTEGER; a_date: ROSE_DATE): RMDAT_PRICE_RECORD is
			-- retrieve the latest daily price record
		require
			valid_date: a_date /= Void
		local
			l_price_record: RMDAT_PRICE_RECORD
			l_lookback, l_valid_status: BOOLEAN
			l_back_one_day, l_date: ROSE_DATE
			j: INTEGER
		do
			l_lookback := False
			l_date := a_date.twin
			from
				j := 1
			until
				j > 31
			loop
				l_valid_status := target_is_valid (a_rxrf, l_date, l_lookback)
				if l_valid_status then
					l_price_record := price_record (a_rxrf, l_date, l_date)
				end
				if not target_is_valid (a_rxrf, l_date, l_lookback) or not l_price_record.has_close then
					l_back_one_day := l_date.previous_date
					if l_back_one_day.month /= a_date.month then
						l_date := Void
						j := 32
					else
						l_date := l_back_one_day
						j := j + 1
					end
				else
					j := 32
				end
			end
			if l_date /= Void then
				Result := l_price_record
			else
				l_date := a_date
				l_price_record := price_record (a_rxrf, l_date, l_date)
				Result := l_price_record
			end
		end

feature -- Access

	error: INTEGER

	czone: CHARACTER is
			-- zone mnemonic as a character
		do
			Result := zone.mnemonic
		end

	date: ROSE_DATE

	found_date: ROSE_DATE

	last_listing: LISTING

	rxrf: INTEGER

	record_index: INTEGER

	rmdata_record: RMDATA

	adjustment: DOUBLE

	adjusted: BOOLEAN

	zone: ZONE [LOCATION]					-- The zone for this object.  Set at creation and never changed.

	julian_index (a_julian_day: INTEGER): INTEGER is
			-- index of julian day
		do
			if jul_indexes.valid_index (a_julian_day) then
				Result := jul_indexes.item (a_julian_day)
			end
		end

feature -- Foreground Database Variables

	record_adj: DOUBLE is
			-- the adj from this record
		do
			Result := rmdata_record.adj
		end

	record_open: DOUBLE is
			-- the open from this record
		do
			Result := rmdata_record.open
		end

	record_close: DOUBLE is
			-- the close from this record
		do
			Result := rmdata_record.close
		end

	record_high: DOUBLE is
			-- the high from this record
		do
			Result := rmdata_record.high
		end

	record_low: DOUBLE is
			-- the low from this record
		do
			Result := rmdata_record.low
		end

	record_bid: DOUBLE is
			-- the bid from this record
		do
			Result := rmdata_record.bid
		end

	record_ask: DOUBLE is
			-- the ask from this record
		do
			Result := rmdata_record.ask
		end

	record_volume: DOUBLE is
			-- the volume from this record
		do
			Result := rmdata_record.vol
		end

	has_adj: BOOLEAN is
			-- does this record have adj?
		do
			if rmdata_record /= Void then
				Result := rmdata_record.has_adj
			end
		end

	has_open: BOOLEAN is
			-- does this record have open?
		do
			if rmdata_record /= Void then
				Result := rmdata_record.has_open
			end
		end

	has_close: BOOLEAN is
			-- does this record have close?
		do
			if rmdata_record /= Void then
				Result := rmdata_record.has_close
			end
		end

	has_high: BOOLEAN is
			-- does this record have high?
		do
			if rmdata_record /= Void then
				Result := rmdata_record.has_high
			end
		end

	has_low: BOOLEAN is
			-- does this record have low?
		do
			if rmdata_record /= Void then
				Result := rmdata_record.has_low
			end
		end

	has_bid: BOOLEAN is
			-- does this record have bid?
		do
			if rmdata_record /= Void then
				Result := rmdata_record.has_bid
			end
		end

	has_ask: BOOLEAN is
			-- does this record have ask?
		do
			if rmdata_record /= Void then
				Result := rmdata_record.has_ask
			end
		end

	has_volume: BOOLEAN is
			-- does this record have volume?
		do
			if rmdata_record /= Void then
				Result := rmdata_record.has_vol
			end
		end

feature {NONE} -- Variables

	ieee_undefined: ROSE_IEEE_UNDEFINED is
			-- access to IEEE undefined values
		once
			create Result
		end

	usd: CURRENCY is
			-- the UNITED STATED DOLLAR currency
		once
			Result := context.usd
		end

	cad: CURRENCY is
			-- the CANADIAN DOLLAR currency
		once
			Result := context.cad
		end

	yen: CURRENCY is
			-- the YEN currency
		once
			Result := context.jpy
		end

	euro: CURRENCY is
			-- the EURO currency
		once
			Result := context.eur
		end

	now: ROSE_DATE is
			-- now
		once
			create Result.make_now
		end

	rmdat_first_ccyymmdd: INTEGER

	set_rmdat_first_ccyymmdd is
			-- The values that are commented out come from b_mondat_wv_src:RMDAT_CROSS_ZONAL_PARAMETERS.FOR.
			-- The actual data goes back much farther and is what we really used.  Only J is OK.
		do
			inspect zone.mnemonic.as_upper
			when 'U' then
					--rmdat_first_ccyymmdd := 19850507
				rmdat_first_ccyymmdd := 19620702
					-- From select min (date) from data_signatures.
			when 'J' then
				rmdat_first_ccyymmdd := 19720101
					-- Should be 19720225; Adjustments jumped on 19720225 for some companies.
			when 'C' then
					--rmdat_first_ccyymmdd := 19811201
				rmdat_first_ccyymmdd := 19800102
			when 'E' then
					--rmdat_first_ccyymmdd := 19811201
				rmdat_first_ccyymmdd := 19750104
			when 'O' then
					--rmdat_first_ccyymmdd := 19811201
				rmdat_first_ccyymmdd := 19750104
			when 'S' then
					--rmdat_first_ccyymmdd := 19890101
				rmdat_first_ccyymmdd := 19700101
			else
				rmdat_first_ccyymmdd := 19000101
			end
		end

	context : ROSE_CONTEXT
			--relevant context
		do
			Result := zone.context
		end
end
