note

	description:

		"Market data for future or index"

	author: "Mauricio Bustos"
	last_editor: "$Author: rj $"
	date: "$Date: 2015/04/24 $"
	revision: "$Revision: #26 $"
	path: "$File: //rose/source/kernel/data/market/market_data_f_i.e $"

deferred class MARKET_DATA_F_I

inherit

	MISSING_PARAMETERS

	DATABASE_SERVER

	MARKET_DATA
		undefine
			first_and_last
		redefine
			has_low,
			has_high,
			has_close,
			close,
			high,
			low,
			rmdata_record,
			date,
			jul_indexes,
			julian_days,
			make,
			currency,
			invalid_record
		end

	ROSE_SHARED_GLOBAL
		rename
			error as global_error
		end

feature {NONE} -- Creation

	make (a_zone: like zone) is
			-- Initialize for `a_zone'
		do
			zone := a_zone
			create jul_indexes.make_empty
			create julian_days.make_empty
			sql_index_selector.apply (agent add_index, Void, Void)
			enable_background
		end

feature -- Access

	jul_indexes: ROSE_LINEAR_ARRAY [INTEGER]

	julian_days: ROSE_LINEAR_ARRAY [INTEGER]

	add_index (a_row: F_I_INDEX) is
		do
			jul_indexes.force (a_row.atlas_index, a_row.rose_julian)
			julian_days.force (a_row.rose_julian, a_row.atlas_index)
		end

	date: ROSE_DATE

	last_date_index: INTEGER

	rmdata_record: RMDATA

	currency (a_rxrf: INTEGER; a_date: ROSE_DATE): CURRENCY is
		do
		end

	first_and_last: ARRAY [FIRST_AND_LAST] is
		deferred
		end

feature -- Write

	write_close (a_rxrf: INTEGER; a_date: ROSE_DATE; a_close: DOUBLE) is
			-- Write `a_close' for `a_rxrf' on `a_date'
		local
			l_rmdata: RMDAT_SIGNATURE_RECORD
		do
			create l_rmdata
			l_rmdata.set_rxrf (a_rxrf)
			l_rmdata.set_date (a_date)
			l_rmdata.set_entity_id (-1)
			l_rmdata.set_data_null
			l_rmdata.set_close (a_close)
			background_data.replace_signature_data (l_rmdata)
			background_data.submit_and_commit
		end

	write_close_high_low (a_rxrf: INTEGER; a_date: ROSE_DATE; a_close: DOUBLE) is
			-- Write close, high and low for `a_rxrf' on `a_date'
		local
			l_rmdata: RMDAT_SIGNATURE_RECORD
		do
			create l_rmdata
			l_rmdata.set_rxrf (a_rxrf)
			l_rmdata.set_date (a_date)
			l_rmdata.set_entity_id (-1)
			l_rmdata.set_data_null
			l_rmdata.set_close (a_close)
			l_rmdata.set_high (a_close)
			l_rmdata.set_low (a_close)
			background_data.replace_signature_data (l_rmdata)
			background_data.submit_and_commit
		end

feature -- Access

	close (a_rxrf: INTEGER; a_date: ROSE_DATE): DOUBLE is
		do
			retrieve (a_rxrf, a_date, a_date)
			if has_close then
				Result := rmdata_record.close
			else
				Result := missing.real
			end
		end

	high (a_rxrf: INTEGER; a_date: ROSE_DATE): DOUBLE is
		do
			retrieve (a_rxrf, a_date, a_date)
			if has_high then
				Result := rmdata_record.high
			else
				Result := missing.real
			end
		end

	low (a_rxrf: INTEGER; a_date: ROSE_DATE): DOUBLE is
		do
			retrieve (a_rxrf, a_date, a_date)
			if has_low then
				Result := rmdata_record.low
			else
				Result := missing.real
			end
		end

feature -- Status

	has_close: BOOLEAN is
		do
			if rmdata_record /= Void then
				Result := rmdata_record.has_close
			end
		end

	has_high: BOOLEAN is
		do
			if rmdata_record /= Void then
				Result := rmdata_record.has_high
			end
		end

	has_low: BOOLEAN is
		do
			if rmdata_record /= Void then
				Result := rmdata_record.has_low
			end
		end

feature -- Query

	invalid_record (a_rxrf: INTEGER; a_date: ROSE_DATE): like price_record is
		do
			create Result
			Result.set (Void, Current, Void, constants.minimum_date_time, date, Void, adjustment)
			Result.set_validity (false)
		end
	
feature {NONE} -- Implementation
	
	sql_index_selector: SIMPLE_PREDICATE [F_I_INDEX] is
			-- Predicate to retrieve index mappings
		require
			server_not_void: servers.market_data_server /= Void
		once
			create Result.make (servers.market_data_server, "select * from f_i_index order by rose_julian desc")
			if not Result.success then
				global_error.raise ("Could not create predicate")
			end
		ensure
			result_not_void: Result /= Void
		end

end
