indexing
	description: "Entity class for a market index"
	author: "Janet Small"
	last_editor: "$Author: mk $"
	date: "$Date: 2012/06/22 $"
	revision: "$Revision: #24 $"
	path: "$File: //rose/source/kernel/investment_entities/market_index.e $"

class
	MARKET_INDEX

inherit
	INSTRUMENT
		undefine
			is_equal
		end
	INVESTMENT_ENTITY
		undefine
			peer_anchor,
			ur_targets,
			descriptive_name
		end
	PERSISTENT_MARKET_INDEX
		undefine
			context,
			make,
			make_context,
			ur_targets,
			corelated_entities,
			corelationships,
			debug_output,
			descriptive_name
		end
	RMDAT_INSTRUMENT
		undefine
			is_equal
		end
	LEGACY_INSTRUMENT
		undefine
			descriptive_name,
			name_identifier_keys
--		redefine
--			cusip_code,
--			isin_code,
--			tse_code
		end

create
	make,
	make_with_zone_xrf,
	make_anonymous

feature -- Access

	name (a_date: ROSE_DATE): STRING is
			-- Name as of `a_date'.
		do
			Result := name_at(a_date)
		end

	descriptive_name: STRING is
			-- A terse descriptive representation of `Current'
		do
			Result := company_name_key (today)
			if Result = Void then
				Result := Precursor
			end
		end

	adjusted_last_price (a_as_of_date,a_date: ROSE_DATE): REAL is
		-- It look like indexes don't have adjustments.  This should probably fail.
		do
			do_nothing
		end

	return (a_start_date,a_end_date: ROSE_DATE): REAL is
			-- Return from a_start_date to a_end_date
		do
			do_nothing
		end

	ur_targets: ROSE_LINEAR_ARRAY[ROSE_ENTITY_DATE[ENTITY]] is
			-- Collection of target available for this entity from which to query for data
		local
			l_signature: RMDAT_SIGNATURE_RECORD
		once
			create l_signature
			l_signature.set_zone(zone_by_mnemonic('I'))
			create Result.make_empty
			Result.extend(l_signature)
		end

	composite_record(a_date: ROSE_DATE): COMPOSITE_RECORD is
	-- I'm guessing that an index can't be a composite, right?  RJ.
	do
		(create {EXCEPTIONS}).raise("Not implementted.")
	end

	is_composite : BOOLEAN is
	-- I'm guessing that an index can't be a composite, right?  RJ.
	do
		Result := False
	end

	risk_region_code(a_date: ROSE_DATE): STRING is
	-- To be supplied.
	do
		(create {EXCEPTIONS}).raise("Not implementted.")
	end

	asset_class_code(a_date: ROSE_DATE): STRING is
	-- To be supplied.
	do
		(create {EXCEPTIONS}).raise("Not implementted.")
	end

feature -- Basic operations

	register  is
			-- Register this entity.
		do
			if zone /= void then
				zone.add_market_index(Current)
			end
		end

end




