note

	description:

		"Plain effective matrix"

	author: "Mark Howard"
	last_editor: "$Author: yc $"
	date: "$Date: 2014/10/06 $"
	revision: "$Revision: #156 $"
	path: "$File: //rose/source/kernel/infrastructure/math/matrix.e $"
	storable_version: ""

class
	MATRIX

inherit

	ABSTRACT_MATRIX
		undefine
			as_matrix,
			to_matrix,
			transpose,
			transposed,
			transpose_at,
			infix "*",
			add,
			subtract,
			inverse,
			anchor,
			replace_row_null,
			scale,
			scaled,
			column,
			scalar_multiply,
			vector_product,
			infix "@",
			to_correlation,
			row_sum,
			row_sums,
			xtx,
			column_sum,
			row,
			put_row_vector,
			duplicate,
			row_index,
			column_index
		end

	ROSE_STORABLE
		rename
			trace as global_trace,
			output as global_output
		end

	ROSE_FORMATTING

	MISMATCH_CORRECTOR
		redefine
			correct_mismatch
		end

	IEEE_MATH
		export {NONE}
			all
		end

create

	make,make_filled,make_from_array,make_from_others, make_from_array_of_vector, make_selected_rows,
	make_identity, make_from_array2,retrieve_from_file,retrieve_from,make_from_blocks,
	make_from_tab_delimited_text_file,make_unit,duplicate,make_from_columns, make_from_delimited_text_file,
	make_initialized,make_sub_matrix_2,make_sub_matrix_3,make_as_inverse,make_transpose,make_from_named_vectors,
	make_from_columns_by_name, make_from_diagonal_blocks, make_named_from_tab_delimited_text_file, make_named_from_delimited_text_file,
	make_weighted,duplicate_matrix,make_diagonal,make_xtx_weighted,make_top_rows

feature -- Initialization

	make_filled (a_height, a_width: INTEGER; a_value: DOUBLE)
			-- Make a matrix with size `a_height, `a_width' and filled with `a_value'.
		require
			a_height_non_negative: a_height >= 0
			a_width_non_negative: a_width >= 0
		local
			i: INTEGER
		do
			height := a_height
			width := a_width
			create data.make_filled (Void, a_height + 1)
			from
				i := 1
			until
				i > height
			loop
				data.put (create {SPECIAL [DOUBLE]}.make_filled (a_value, width + 1), i)
				i := i + 1
			end
		end

	make_identity (a_count: INTEGER) is
			-- Make identity matrix of height `a_count'.
		require
			a_count_non_negative: a_count >= 0
		local
			i: INTEGER
		do
			from
				make (a_count, a_count)
				i := 1
			until
				i > a_count
			loop
				put (1, i, i)
				i := i + 1
			end
		end

	make (a_height, a_width: INTEGER) is
			-- Make a matrix with size `a_height, `a_width' and filled with zeroes.
		do
			make_filled (a_height, a_width, 0.0)
		end
	make_diagonal (a_vector: ABSTRACT_READABLE_VECTOR)
			-- Make with `a_vector'.
		require
			valid_vector: a_vector /= Void
		local
			i: INTEGER
		do
			make (a_vector.count,a_vector.count)
			from
				i := 1
			until
				i > width
			loop
				put (a_vector.item (i), i, i)
				i := i + 1
			end
		end

	make_weighted(a_matrix : MATRIX;a_weights : VECTOR)
			--Make from a_matrix with the i_th row multiplied by a_weights[i]
		require
			counts_match : a_matrix.height = a_weights.count
		local
			i,j : INTEGER
			l_row_i,l_other_i : SPECIAL[DOUBLE]
			l_weight : DOUBLE
		do
			make(a_matrix.height,a_matrix.width)
			from i := 1 until i > height loop
				l_row_i := data[i]
				l_other_i := a_matrix.data[i]
				l_weight := a_weights[i]
				from j := 1 until j > width loop
					l_row_i[j] := l_other_i[j] * l_weight
					j := j + 1
				end
				i := i + 1
			end
		end

	make_xtx_weighted (a_m: MATRIX;a_w : VECTOR)
			-- Create weighted transposed product (x^t x) from matrix `a_m' and weights `a_w'.
		require
			valid_matrix: attached a_m
		local
			i, j, k, l_height: INTEGER
			l_weight_i_square : DOUBLE
			l_data : SPECIAL[SPECIAL[DOUBLE]]
			l_row_i,l_result,l_result_k : SPECIAL[DOUBLE]
			l_weight : SPECIAL[DOUBLE]
			l_vector : VECTOR
		do
			make (a_m.width,a_m.width)
			l_data := a_m.data
			l_height := a_m.height
			if a_w /= Void then
				l_weight := a_w.data
			else
				create l_vector.make_identity(l_height)
				l_weight := l_vector.data
			end
			from i := 1 until i > l_height loop
				l_row_i := l_data[i]
				l_weight_i_square := l_weight[i] * l_weight[i]
				from j := 1 until j > height loop
					l_result := data[j]
					from k := 1 until k > j loop
						l_result_k := data[k]
						l_result[k] := l_result[k] + l_row_i[k] * l_row_i[j] * l_weight_i_square
						l_result_k[j] := l_result[k]
						k := k + 1
					end
					j := j + 1
				end
				i := i + 1
			end
		ensure
			square_matrix: is_square
			width_ok: width = a_m.width
		end

	make_from_columns (a_columns: ARRAY [VECTOR]) is
			-- make from columns
		require
			a_columns_not_void: a_columns /= Void
			a_columns_not_empty: not a_columns.is_empty
			size: True -- TODO write a same_size agent a_columns.for_all (agent (a_columns.item (1)).same_size)
		local
			i,j : INTEGER
			l_row_i,l_column : SPECIAL[DOUBLE]
		do
			make(a_columns.item(1).count,a_columns.count)
			from i := 1 until i > height loop
				l_row_i := data[i]
				from j := 1 until j > width loop
					l_column := a_columns[j].data
					l_row_i[j] := l_column[i]
					j := j + 1
				end
				i := i + 1
			end
		end

	make_from_columns_by_name(a_matrix : MATRIX;a_names:ARRAY[STRING]) is
			-- make from columns by name
		do
			make_from_selected_columns (a_matrix,a_matrix.column_indices(a_names))
		end

	make_from_array (a_array: ARRAY [ARRAY [DOUBLE]]) is
			-- Create matrix from array of rows.
		require
			a_array_not_void: a_array /= Void
			a_array_matrix_shaped: is_array_matrix (a_array)
		local
			i, j: INTEGER
		do
			make (a_array.count, a_array.item (1).count)
			from
				i := 1
			until
				i > height
			loop
				from
					j := 1
				until
					j > width
				loop
					put (a_array.item (i).item (j), i, j)
					j := j + 1
				end
				i := i + 1
			end
		ensure
			row_count: height = a_array.count
			column_count: width = a_array.item (1).count
		end

	make_from_array2 (a_array: ARRAY2 [DOUBLE]) is
			-- make from array2
		local
			i, j: INTEGER
			l_array: ARRAY [ARRAY [DOUBLE]]
			l_row_i: ARRAY [DOUBLE]
		do
			create l_array.make (1, a_array.height)
			from
				i := 1
			until
				i > a_array.height
			loop
				create l_row_i.make (1, a_array.width)
				from
					j := 1
				until
					j > a_array.width
				loop
					l_row_i.put (a_array.item (i, j), j)
					j := j + 1
				end
				l_array.put (l_row_i, i)
				i := i + 1
			end
			make_from_array (l_array)
		end

	make_transpose (a_matrix: MATRIX) is
			-- Create as transpose of `a_matrix'.
		local
			i, j : INTEGER
			l_row_j,l_other_i : SPECIAL[DOUBLE]
		do
			make (a_matrix.width, a_matrix.height)
			from i := 1 until i > a_matrix.height loop
				l_other_i := a_matrix.data[i]
				from j := 1 until j > a_matrix.width loop
					l_row_j := data[j]
					l_row_j[i] := l_other_i[j]
					j := j + 1
				end
				i := i + 1
			end
			set_name(name)
			set_column_names(row_names)
			set_row_names(column_names)
		end

	make_sub_matrix_2 (a_matrix : ABSTRACT_READABLE_MATRIX;i0, i1, j0, j1: INTEGER) is
			-- make sub matrix 2
		local
			i, j: INTEGER
		do
			make (i1 - i0 + 1, j1 - j0 + 1)
			from i := i0 until i > i1 loop
				from j := j0 until j > j1 loop
				--	Result.put (item (i, j), i - i0 + 1, j - j0 + 1)
					Current[i - i0 + 1, j - j0 + 1] := a_matrix [i, j]
					j := j + 1
				end
				i := i + 1
			end
		end

	make_sub_matrix_3(a_matrix : ABSTRACT_READABLE_MATRIX;r,c : ARRAY[INTEGER]) is
			-- make sub matrix 3
		local
			i, j,k,m : INTEGER
			l_row_i,l_other_row : SPECIAL[DOUBLE]
			l_data : SPECIAL[SPECIAL[DOUBLE]]
		do
			make (r.count, c.count)
			l_data := a_matrix.as_matrix.data
			from i := 1 until i > r.count loop
				l_row_i := data[i]
				from j := 1 until j > c.count loop
					k:=r.item (i)
					m:=c.item(j)
					l_other_row := l_data[k]
					if a_matrix.valid_row_index(k) and a_matrix.valid_column_index(m) then
						l_row_i[j] := l_other_row[m]
					else
						l_row_i[j] := undefined_double
					end
				j := j + 1
				end
			i := i + 1
			end
		end

	make_selected_rows (a_matrix : ABSTRACT_READABLE_MATRIX; l_rows : ARRAY[INTEGER])is
	 	-- make selected rows
	  local
		 l_height,i: INTEGER --,j : INTEGER
	  do
		  	l_height:=l_rows.count
		  	make(l_height,a_matrix.width)
		  	from i:=1 until i> l_height loop
				put_row_vector(a_matrix.row(l_rows.item(i)),i)
		  		i:=i+1
		  	end
		  	set_column_names(a_matrix.column_names)
		  	if a_matrix.row_names /= Void then
		  		set_row_names(a_matrix.row_names.selected_items(l_rows))
		  	end
	  end

	 make_top_rows(a_matrix : ABSTRACT_READABLE_MATRIX;a_rows:INTEGER) is
			-- make top rows
		 local
		 	l_array:ARRAY[INTEGER]
		 	i:INTEGER
		 do
		 	create l_array.make(1,a_rows)
		 	from i:=1 until i>a_rows loop
		 		l_array.put(i,i)
		 		i:=i+1
		 	end
		 	make_selected_rows(a_matrix,l_array)
		 end

	make_from_named_vectors(a_vectors:ARRAY[VECTOR]) is
			--all vectors must be named,with name and names in vector
		local
			l_vector,l_added_vector:VECTOR
			i,j:INTEGER
		do
			create row_names.make(1,a_vectors.count)
			create l_vector.make(0)
			from i:=1 until i>a_vectors.count loop
				l_added_vector:=a_vectors.item(i)
				row_names.put(l_added_vector.name,i)
				from j:=1 until j>l_added_vector.count 	loop
					l_vector.extend_with_name (l_added_vector.item(j), l_added_vector.names.item(j))
					j:=j+1
				end
				i:=i+1
			end
			make(a_vectors.count,l_vector.count)
			create column_names.make_from_array(l_vector.names)
			column_names.sort_by( agent string_compare(?,?))
			from i:=1  until i>a_vectors.count loop
				l_vector:=a_vectors.item (i).values_at (column_names)
				put_row_vector(l_vector,i)
				i:=i+1
			end
		end

	make_from_others(a_others:ARRAY[ABSTRACT_READABLE_MATRIX]) is
			--require equal width
		local
			l_other:ABSTRACT_READABLE_MATRIX
			i,j,k,m:INTEGER
			l_rows_names_exists,l_ids_exists:BOOLEAN
			l_id_array:ROSE_LINEAR_ARRAY[Like identifiers]
			l_row_names:like row_names
		do
			k:=a_others.count
			from i:=1 until i>k  loop
				j:=j+a_others.item(i).height
				i:=i+1
			end
			make(j,a_others.item(1).width)
			from i:=1 until i>k  loop
				l_other:=a_others.item(i)
				from j:=1 until j>l_other.height loop
					m:=m+1
					put_row_vector(l_other.row(j),m)
					j:=j+1
				end
				i:=i+1
			end
			if attached {MATRIX} a_others[1] as l_other_1 and then l_other_1.identifiers /= Void then
				l_ids_exists:=true
				create l_id_array.make(1,a_others.count)
				from i:=1 until i>k  loop
					if attached {MATRIX} a_others[i] as l_other_i and then l_other_i.identifiers /= Void then
						l_ids_exists:=l_ids_exists and l_other_i.identifiers/=void
						l_id_array.put(l_other_i.identifiers,i )
					end
					i:=i+1
				end
				if l_ids_exists then
				  	create identifiers.make_from_others (l_id_array)
				end
			end
			if a_others.item (1).row_names/=void then
				l_rows_names_exists:=true
				create l_row_names.make_from_array(a_others.item(1).row_names)
				from i:=1 until i>k  loop
					l_rows_names_exists:=l_rows_names_exists and a_others.item(i).row_names/=void
					if i>1 then
						l_row_names:=l_row_names+a_others.item(i).row_names
					end
					i:=i+1
				end
				if l_rows_names_exists then
				  	set_row_names(l_row_names)
				end
			end
			if a_others.item (1).column_names/=void then
				create column_names.make_from_array(a_others.item (1).column_names)
			end
		end

	make_from_array_of_vector(a_array_of_vector : ARRAY[VECTOR]) is
			-- make from array of vector
		local
			i, l_lower : INTEGER
			l_vector : VECTOR
		do
			l_lower := a_array_of_vector.lower
			make(a_array_of_vector.count, a_array_of_vector.item(l_lower).count)
			from i := l_lower until i > a_array_of_vector.upper loop
				l_vector := a_array_of_vector.item(i)
				put_row_vector(l_vector, i - l_lower + 1)
				i := i + 1
			end
		end

	make_from_blocks (a_blocks : ARRAY[ARRAY[ABSTRACT_READABLE_MATRIX]]) is
			-- creates a matrix by sticking together the matrices in `a_blocks'
		require
			uniform : uniform_blocks(a_blocks)
		local
			i,j,l_height,l_width : INTEGER
			l_blocks : ARRAY[ABSTRACT_READABLE_MATRIX]
		do
			from i := 1 until i > a_blocks.count loop
				l_height := l_height + ((a_blocks[i])[1]).height
				i := i + 1
			end
			l_blocks := a_blocks[1]
			from i := 1 until i > l_blocks.count loop
				l_width := l_width + l_blocks.item(i).width
				i := i + 1
			end
			make(l_height,l_width)
			l_height := 0
			from i := 1 until i > a_blocks.count loop
				l_blocks := a_blocks[i]
				l_width := 0
				from j := 1 until j > l_blocks.count loop
					put_matrix (l_blocks[j], l_height+1, l_width+1)
					l_width := l_width + l_blocks.item(j).width
					j := j + 1
				end
				l_height := l_height + l_blocks.item(1).height
				i := i + 1
			end
		end

	make_from_diagonal_blocks (a_blocks: ARRAY [MATRIX])
			-- Make a matrix which has `a_blocks' down the "diagonal", and zero blocks everywhere else
		require
			block_array_not_void: a_blocks /= Void
			block_array_not_empty: not a_blocks.is_empty
			block_array_only_has_valid_matrices: not a_blocks.has (Void)
		local
			i, l_width, l_height, l_current_column, l_current_row: INTEGER
		do
				-- Calculate required width and height
			from
				i := 1
			until
				i > a_blocks.count
			loop
				l_width := l_width + a_blocks [i].width
				l_height := l_height + a_blocks [i].height
				i := i + 1
			end

				-- Create zero matrix for full matrix size
			make (l_height, l_width)

			 	-- Use `put_matrix' to insert diagonal blocks and overwrite zeroes
			l_current_row := 1
			l_current_column := 1
			from
				i := 1
			until
				i > a_blocks.count
			loop
				put_matrix (a_blocks [i], l_current_row, l_current_column)
				l_current_row := l_current_row + a_blocks [i].height
				l_current_column := l_current_column + a_blocks [i].width
				i := i + 1
			end
		end

	make_unit (a_height, a_width: INTEGER) is
			-- Make as `a_height' by `a_width' filed with ones.
		require
			a_height_non_negative: a_height >= 0
			a_width_non_negative: a_width >= 0
		local
			i,j: INTEGER
		do
			make (a_height, a_width)
			from i := 1 until i > a_height
			loop
				from j := 1 until j > a_width
				loop
					put (1, i, j)
					j := j + 1
				end
				i := i + 1
			end
		end

	make_named_from_tab_delimited_text_file(a_file: ROSE_TEXT_FILE) is
			-- this sets row and column names
			-- it expects the first line to be column names
			-- and the first item in each row to the row names
		do
			make_named_from_delimited_text_file(a_file, "%T ,", "\")
		end

	make_named_from_delimited_text_file(a_file: ROSE_TEXT_FILE; a_delimited_string, a_escape : STRING) is
			-- this sets row and column names
			-- it expects the first line to be column names
			-- and the first item in each row to the row names
		local
			l_width, l_height, l_line_num, l_item_num: INTEGER
			l_lines: ROSE_LINEAR_ARRAY [STRING]
			l_line: STRING
			l_items: ROSE_LINEAR_ARRAY [STRING]
		do
			if a_file.is_readable then
				create l_lines.make_from_linear(a_file.lines.linear_representation)
				-- Check if first line is a header
				l_line := l_lines[1]
				create l_items.make_from_linear(formatter.split(l_line, a_delimited_string, a_escape).linear_representation)
				l_width := l_items.count - 1
				l_height := l_lines.count - 1
				make(l_height, l_width)
				column_names := l_items.subarray (2, l_items.count)
				create row_names.make(1, height)
				from l_line_num := 2 until l_line_num > l_height + 1 loop
					l_line := l_lines[l_line_num]
					create l_items.make_from_linear(formatter.split(l_line, a_delimited_string, a_escape).linear_representation)
					row_names[l_line_num - 1] := l_items[1]
					from l_item_num := 2 until l_item_num > l_width + 1 loop
						put(l_items[l_item_num].to_double, l_line_num - 1, l_item_num - 1)
						l_item_num := l_item_num + 1
					end
					l_line_num := l_line_num + 1
				end
			end
		end


	make_initialized (a_value: DOUBLE; a_height, a_width: INTEGER) is
			-- Make with each cell filled with 'a_value'.
		do
			make (a_height, a_width)
			initialize (a_value)
		end

feature -- Access

	i_th_row (i: INTEGER): VECTOR is
			-- i th row
		do
			create Result.make_by_area (data.item (i))
		end

feature -- Display

	named_matrix_to_array2 : ARRAY2[ANY] is
			-- matrix combined to array2
		require
			has_row_names : row_names /= void
			has_column_names : column_names /= void
		local
			i, j:INTEGER
		do
			create result.make(height+1, width+1)
			result.put("", 1, 1)
			-- row names
			from i := 1 until i > height loop
				result.put(row_names[i], i+1, 1)
				-- contents
				from j := 1 until j > width loop
					result.put(item(i,j).out, i+1, j+1)
					j := j + 1
				end
				i := i + 1
			end
			-- column names
			from j := 1 until j > width loop
				result.put(column_names[j], 1, j+1)
				j := j + 1
			end
		end

	named_matrix_to_csv_file (a_file_name: STRING)
			-- Create csv file `a_file_name' with 'column_names', 'row_names', and matrix data
		local
			l_file : ROSE_TEXT_FILE
			i, j:INTEGER
		do
			create l_file.make_open_write (a_file_name)
			across column_names as c_column_names loop
				l_file.put_string (",")
				l_file.put_string (string_without_commas (c_column_names.item))
			end

			l_file.put_string ("%N")
			from i := 1 until i > height loop
				l_file.put_string (string_without_commas (row_names[i]))
				from j := 1 until j > width loop
					l_file.put_string (",")
					l_file.put_string (item (i, j).out)
					j := j + 1
				end

				l_file.put_string ("%N")
				i := i + 1
			end

			l_file.close
		end

feature -- Conversion


	resize(a_height, a_width : INTEGER) is
			-- increase `height' to `a_height' and `width' to `a_width'
			-- can only make the matrix bigger
		require
			valid_size : not (a_height <= height and a_width <= width)
		local
			l_new : like Current
		do
			create l_new.make(a_height, a_width)
			l_new.put_matrix(Current, 1, 1)
			data := l_new.data
			height := a_height
			width := a_width
		end

	set_height (a_height : INTEGER) is
			-- increase `height' to `a_height'
			-- cannot decrease height
		require
			restrict_only : a_height <= height
		do
			height := a_height
		end

	add_row (a_row: VECTOR) is
			-- Add a_row to the bottom of the matrix.
		require
			nonvoid : a_row /= Void
			same_width : a_row.count = width
		local
			l_data : SPECIAL[SPECIAL[DOUBLE]]
			i : INTEGER
		do
			if height + 1 < data.count then
				data.put((a_row.data).twin,height + 1)
			else
				l_data := data
				create data.make_filled (Void, height + 2)
				from i := 1 until i > height loop
					data.put(l_data.item(i),i)
					i := i + 1
				end
				data.put((a_row.data).twin,height + 1)
			end
			height := height + 1
			if row_names /= Void then
				if a_row.name /= Void then
					row_names.extend(a_row.name)
					row_table.put (height, a_row.name)
				else
					row_names.extend ("row" + width.out)
					row_table.put (height, "row" + width.out)
				end
			end
		ensure
			new_height : height = old height + 1
			added_to_bottom : row(height).is_equal(a_row)
		end

	add_column (a_column: ABSTRACT_READABLE_VECTOR) is
			-- Add `a_column' to right of `Current'.
		require
			nonvoid : a_column /= Void
			same_height : a_column.count = height
		local
			l_data: SPECIAL [DOUBLE]
			l_data_new: SPECIAL [DOUBLE]
			i, j: INTEGER
			l_data_count: INTEGER
		do
			from i := 1 until i > height loop
				l_data := data.item(i)
				l_data_count := l_data.count
				if l_data_count < (width + 2) then
					-- Allocate additional space based on Growth_percentage and Minimal_increase.
					data.put (create {SPECIAL [DOUBLE]}.make_filled (0.0, l_data_count + additional_space (l_data_count)), i)
					l_data_new := data.item(i)
					from j := 1 until j > width loop
						l_data_new.put(l_data.item(j), j)
						j := j + 1
					end
				end
				data.item(i).put(a_column.item(i), width + 1)
				i := i + 1
			end
			width := width + 1
			if column_names /= Void then
				if a_column.name /= Void then
					column_names.extend (a_column.name)
					column_table.put (width, a_column.name)
				else
					column_names.extend ("column" + width.out)
					column_table.put (width, "column" + width.out)
				end
			end
		ensure
			new_width: width = old width + 1
			added_to_right: a_column.is_equal (column (width))
		end

	extend (a_data: ABSTRACT_READABLE_MATRIX) is
			-- Add rows from `a_data' to bottom of `Current'.
		require
			a_data_not_void: a_data /= Void
			same_width: a_data.width = width
		local
			l_index, l_height: INTEGER
		do
			from
				l_index := 1
				l_height := a_data.height
			invariant
				valid_row_index: l_index > l_height xor a_data.valid_row_index (l_index)
			variant
				increasing_index: l_height + 1 - l_index
			until
				l_index > l_height
			loop
				add_row (a_data.row (l_index))
				l_index := l_index + 1
			end
		ensure
			all_rows_added: height = old height + a_data.height
		end

feature --Transformation

	scale (a_scale : DOUBLE)
			--Scale all items by `a_scale'
		local
			i,j : INTEGER
			l_row_i : SPECIAL[DOUBLE]
		do
			from i := 1 until i > height loop
				l_row_i := data[i]
				from j := 1 until j > width loop
					l_row_i[j] := l_row_i[j] * a_scale
					j := j + 1
				end
				i := i + 1
			end
		end

	scalar_multiply,scaled (a_value : DOUBLE) : like Current is
			-- returns a matrix like current, but with every element multiplied by `a_value'
		local
			i,j : INTEGER
			l_row_i,l_result_row : SPECIAL[DOUBLE]
		do
			create Result.make(height,width)
			from i := 1 until i > height loop
				l_row_i := data[i]
				l_result_row := Result.data[i]
				from j := 1 until j > width loop
					l_result_row[j] := l_row_i[j] * a_value
					j := j + 1
				end
				i := i + 1
			end
			if column_names /= Void then
				Result.set_column_names(column_names)
			end
			if row_names /= Void then
				Result.set_row_names(row_names)
			end
		end
	cloned : like anchor is
			-- cloned
		local
			i,j : INTEGER
		do
			create Result.make(height,width)
			from i := 1 until i > height loop
				from j := 1 until j > width loop
				--	Result.put(item(i,j),i,j)
					Result [i, j] := Current [i, j]
					j := j + 1
				end
				i := i + 1
			end
		end

	to_matrix : MATRIX
			-- Redefined for speed
		local
			i,j : INTEGER
			l_row_i,l_result : SPECIAL[DOUBLE]
		do
			create Result.make(height,width)
			from i := 1 until i > height loop
				l_row_i := data[i]
				l_result := Result.data[i]
				from j := 1 until j > width loop
					l_result[j] := l_row_i[j]
					j := j + 1
				end
				i := i + 1
			end
		end
	as_matrix : MATRIX is
			-- I guess this changes the type of current to MATRIX???
		do
			Result := Current
		end

	vector_product_ignoring_null (a_vector: VECTOR): VECTOR is
			-- vector production ignoring null
		local
			l_matrix: like current
			l_vector: VECTOR
		do
			l_matrix := deep_twin
			l_vector := a_vector.deep_twin
			l_matrix.replace_null
			l_vector.replace_null
			Result := l_matrix.vector_product (l_vector)
		end

	row_weighted(a_vector:VECTOR): MATRIX is
   	  	-- row weighted
   	  require
   	  	count_equals_width : a_vector.count=height
	  local
		 i,j : INTEGER
		 l_row_i,l_result,l_vector : SPECIAL[DOUBLE]
		 l_vector_i : DOUBLE
	  do
		 Create Result.make(height,width)
		 l_vector := a_vector.data
		 from i := 1 until i > height loop
		 	l_row_i := data[i]
		 	l_result := Result.data[i]
		 	l_vector_i := l_vector[i]
			from j := 1 until j > width loop
				if is_defined_double (l_row_i[j]) then
			   		l_result[j] :=l_row_i[j] * l_vector_i
			   	else
					l_result[j] := undefined_double
			   end
			   j := j+1
			end
			i := i+1
		 end
		 result.set_column_names(column_names)
		 result.set_row_names(row_names)
	  end

	row_index(a_name:STRING):INTEGER is
		--row_index by name, assuming unique
		do
			Result := row_table.item(a_name)
		end

	row_sum (i : INTEGER) : DOUBLE is
			-- Sum of row items
		local
			j : INTEGER
			l_row_i : SPECIAL[DOUBLE]
		do
			l_row_i := data[i]
			from j := 1 until j > width loop
				Result := Result + l_row_i[j]
				j := j + 1
			end
		end

	row_sums : VECTOR is
			-- vector of the sums of the rows
		local
			i : INTEGER
			l_result : SPECIAL[DOUBLE]
		do
			create Result.make(height)
			l_result := Result.data
			from i := 1 until i > height loop
				l_result[i] := row_sum(i)
				i := i + 1
			end
		end

	column_weighted(a_vector: VECTOR): MATRIX is
		--column weighted version
   	  require
   	  	count_equals_width : a_vector.count=width
	  local
		 l_matrix:MATRIX
	  do
	  	l_matrix:= transposed
	  	Result := l_matrix.row_weighted(a_vector).transposed
	  end

	lower_triangular : MATRIX is
			-- lower triangular part of matrix
		require
			is_sqaure: height = width
		local
			i,j: INTEGER
		do
			create Result.make (height, width.min(height))
			from i := 1 until i > height loop
				from j := 1 until j > i.min (width) loop
					Result.put (item(i,j), i,j)
					j := j + 1
				end
				i := i + 1
			end
		end

	upper_triangular : MATRIX is
			-- lower triangular part of matrix
		require
			is_sqaure: height = width
		local
			i,j: INTEGER
		do
			create Result.make (height.min(width), width)
			from i := 1 until i > height.min(width) loop
				from j := i until j > width loop
					Result.put (item(i,j), i,j)
					j := j + 1
				end
				i := i + 1
			end
		end

feature  --Modification
	set_row_null(a_row_ix : INTEGER)
			--Set the row with index `a_row_ix' to unavailable
		local
			i : INTEGER
		do
			if valid_row_index(a_row_ix) then
				from i := 1
				until i > width
				loop
				--	put(constants.missing_double, a_row_ix, i)
					Current[a_row_ix, i] := constants.missing_double
					i := i + 1
				end
			end
		end

	set_column_null (a_col_ix: INTEGER)
			-- Set column with 'a_col_ix' index with QNAN
		local
			i : INTEGER
		do
			if valid_column_index (a_col_ix) then
				from
					i := 1
				until
					i > height
				loop
					Current [i, a_col_ix] := constants.missing_double
					i := i + 1
				end
			end
		end



	replace_row_null(a_r: INTEGER) is
			-- Replace undefined doubles and null values with zero in row with index `a_r'.
		local
			j: INTEGER
			l_v: DOUBLE
		do
			if valid_row_index(a_r) then
				from j := 1 until j > width loop
					l_v := Current[a_r, j]
					if not is_defined_double(l_v) then
						Current[a_r, j] := 0.0
					end
					j := j + 1
				end
			end
		end

	bound (a_lower_bound, a_upper_bound: DOUBLE) is
			-- Bounds available matrix elements within given lower bound `a_lower_bound' and upper bound `a_upper_bound'.
		require
			lower_less_than_upper: a_lower_bound < a_upper_bound
		local
			i, j: INTEGER
		do
			from i := 1
			until i > height
			loop
				from j := 1
				until j > width
				loop
					if is_defined_double (Current [i, j]) then
						Current[i, j] := item (i, j).max(a_lower_bound).min(a_upper_bound)
					end
					j := j + 1
				end
				i := i + 1
			end
		end

	add_value(a_value:DOUBLE) is
			-- adds `a_value' to every defined element in the matrix
		local
			i,j:INTEGER
		do
			from i:=1 until i>height  loop
				from j:=1 until j>width loop
				if is_defined_double(Current[i,j]) then
					Current[i, j] := Current [i, j] + a_value
				end
				j:=j+1
				end
				i:=i+1
			end
		end

feature -- scalar function

	columnwise_cum_return_matrix : like Current is
			--Profoundly misguided place for this
		local
			i : INTEGER
			l_vec : VECTOR
		do
			create Result.make(height, width)
			from i := 1 until i > width loop
				l_vec := column(i).cum_returns
				Result.put_column_vector(l_vec, i)
				i := i + 1
			end
			Result.set_row_names(row_names)
			Result.set_column_names(column_names)
		end

	set_default_names
			-- Set default column/row names if needed
		do
			if column_names = Void then
				set_default_column_names
			end
			if row_names = Void then
				set_default_row_names
			end
		end


feature  --Initialization

	initialize (a_value: DOUBLE)
			--Set all values to `a_value'
		local
			i,j:INTEGER
		do
			from i:=1 until i> height loop
				from  j:=1 until j>width loop
					Current[i, j] := a_value
					j:=j+1
				end
				i:=i+1
			end
		end

	randomize (a_random: RANDOM)
			-- Sets matrix elements to instantiations of random variable.
			-- Do not remove (used for project).  
		require
			random_not_void: a_random /= Void
		local
			i, j: INTEGER
		do
			from i := 1
			until i > height
			loop
				from j := 1
				until j > width
				loop
					Current[i, j] := a_random.double_item
					a_random.forth
					j := j + 1
				end
				i := i + 1
			end
		end

feature --Access

	column_index (a_name: STRING): INTEGER is
			-- <Precursor>
		do
			Result := column_table.item (a_name)
		end




feature-- Additional features added by India Team (For use in CPLP and other applications)

	scale_row (a_row_ix: INTEGER; a_value : DOUBLE) is
			-- scale row
		local
			j : INTEGER
			l_row : SPECIAL[DOUBLE]
		do
			if valid_row_index (a_row_ix) then
				l_row := data[a_row_ix]
				from j := 1 until j > width loop
				--	put (item(a_row_ix, j) * a_value, a_row_ix, j)
					l_row[j] := l_row[j] * a_value
					j := j + 1
				end
			end
		end

	scale_rows (a_vector : ABSTRACT_READABLE_VECTOR) is
			-- multiplies the entries of each row by corresponding value in a vector
		require
			valid_vector : a_vector /= Void and then a_vector.count = height
		local
			i :  INTEGER
		do
			from i := 1 until i > height loop
				scale_row(i, a_vector[i])
				i := i + 1
			end
		end

	scale_column (a_column: INTEGER; a_value: DOUBLE) is
		-- multiplies the entries in 'a_column_ix' by 'a_value'
		-- added by b5 on 10/27/2006
		local
			i: INTEGER
			l_row_i : SPECIAL[DOUBLE]
		do
			if valid_column_index (a_column) then
				from
					i := 1
				until
					i > height
				loop
					l_row_i := data[i]
					l_row_i [a_column] := l_row_i [a_column] * a_value
					i := i + 1
				end
			end
		end

	scale_columns (a_vector : ABSTRACT_READABLE_VECTOR) is
			-- multiplies the entries of each column by corresponding value in a vector
		require
			valid_vector : a_vector /= Void and then a_vector.count = width
		local
			i :  INTEGER
		do
			from i := 1 until i > width loop
				scale_column(i, a_vector[i])
				i := i + 1
			end
		end


	subtract_scaled_row_from_row (a_source_row_ix: INTEGER; a_value: DOUBLE; a_target_row_ix: INTEGER) is
		-- perform the operation: target_row = target_row - multiplier * source_row
		local
			j: INTEGER
		do
			from j := 1
			until j > width
			loop
			--	put (item(a_target_row_ix, j) - a_value * item(a_source_row_ix, j), a_target_row_ix, j)
				Current[a_target_row_ix, j] := Current [a_target_row_ix, j] - a_value * Current[a_source_row_ix, j]
				j := j + 1
			end
		end

	row_moments (a_weights: ABSTRACT_READABLE_MATRIX): ROSE_LINEAR_ARRAY[ROSE_MOMENTS] is
			-- moments of data in rows
		require
			dimension_matched: a_weights /= Void implies (a_weights.height = height and then a_weights.width = width)
		local
			l_weights : MATRIX
			i: INTEGER
		do
			create Result.make (1, height)
			l_weights ?= a_weights
			if l_weights = Void then
				create l_weights.make_unit (height, width)
			end
			from i := 1 until i > height
			loop
				Result.put (Create {ROSE_MOMENTS}.make (row (i).to_array, l_weights.row(i).to_array), i)
				i := i + 1
			end
		ensure
			correct_count : Result.count = height
		end

	column_moments (a_weights: MATRIX): ROSE_LINEAR_ARRAY[ROSE_MOMENTS] is
			-- moments of data in columns
		require
			dimension_matched: a_weights /= Void implies (a_weights.height = height and then a_weights.width = width)
		local
			i: INTEGER
			l_weights : MATRIX
		do
			create Result.make (1, width)
			l_weights := a_weights
			if l_weights = Void then
				create l_weights.make_unit (height, width)
			end
			from i := 1 until i > width loop
				Result.put (Create {ROSE_MOMENTS}.make (column (i).to_array, l_weights.column(i).to_array), i)
				i := i + 1
			end
		ensure
			correct_count : Result.count = width
		end
	winsorized_row_moments (a_weights: ABSTRACT_READABLE_MATRIX;a_lower_z_value,a_upper_z_value : INTEGER): ROSE_LINEAR_ARRAY[ROSE_MOMENTS] is
			-- moments of data in rows
		require
			dimension_matched: a_weights /= Void implies (a_weights.height = height and then a_weights.width = width)
		local
			l_weights : MATRIX
			i: INTEGER
		do
			create Result.make (1, height)
			l_weights ?= a_weights
			if l_weights = Void then
				create l_weights.make_unit (height, width)
			end
			from i := 1 until i > height
			loop
				Result.put (Create {ROSE_MOMENTS}.make_winsorized (row (i).to_array, l_weights.row(i).to_array,a_lower_z_value,a_upper_z_value), i)
				i := i + 1
			end
		ensure
			dimensions_match: Result.count = height
		end

	winsorized_column_moments (a_weights: MATRIX;a_lower_z_value,a_upper_z_value : INTEGER): ROSE_LINEAR_ARRAY[ROSE_MOMENTS] is
			-- moments of data in columns
		require
			dimension_matched: a_weights /= Void implies (a_weights.height = height and then a_weights.width = width)
		local
			i: INTEGER
			l_weights : MATRIX
		do
			create Result.make (1, width)
			l_weights := a_weights
			if l_weights = Void then
				create l_weights.make_unit (height, width)
			end
			from i := 1 until i > width loop
				Result.put (Create {ROSE_MOMENTS}.make_winsorized (column (i).to_array, l_weights.column(i).to_array,a_lower_z_value,a_upper_z_value), i)
				i := i + 1
			end
		ensure
			dimensions_match: Result.count = width
		end

feature -- Transformation

	is_identity_with_epsilon (a_epsilon: DOUBLE): BOOLEAN is
			-- abs of item diffs is within epsilon
		local
			i, j : INTEGER
			l_data_item: DOUBLE
		do
			Result := height = width
			from i := 1 until i > height loop
				from j := 1 until j > width loop
					l_data_item := Current[i, j]
					Result := Result and (i = j implies (l_data_item-1.0).abs <= a_epsilon)
							 and (i /= j implies l_data_item.abs <= a_epsilon)
					j := j + 1
				end
				i := i + 1
			end
		end

	transformed (a_agent : FUNCTION[ANY,TUPLE[DOUBLE],DOUBLE]) : MATRIX is
			-- matrix transformed by `a_agent'
		local
			i,j : INTEGER
		do
			create Result.make (height, width)
			from i := 1 until i > height loop
				from j := 1 until j > width loop
					Result[i,j] := a_agent.item([item(i,j)])
					j := j + 1
				end
				i := i + 1
			end
		end

	is_lower_triangular : BOOLEAN is
			-- are all entries above the main diagonal 0?
		local
			i,j : INTEGER
		do
			Result := true
			from i := 1 until i > height or else not Result loop
				from j := i + 1 until j > width or else not Result loop
					Result := Current[i,j] = 0
					j := j + 1
				end
				i := i + 1
			end
		end

	is_upper_triangular : BOOLEAN is
			-- are all entries below the main diagonal 0?
		local
			i,j : INTEGER
		do
			Result := true
			from i := 1 until i > height or else not Result loop
				from j := 1 until j >= i or else not Result loop
					Result := Current[i,j] = 0
					j := j + 1
				end
				i := i + 1
			end
		end
feature -- Assertion support

	is_array_matrix (a_array: ARRAY [ARRAY [DOUBLE]]): BOOLEAN is
			-- Is array matrix shaped?
		require
			a_array_not_void: a_array /= Void
		do
			Result := a_array.lower = 1 and (a_array.count > 0
				and then a_array.for_all (agent same_array_count (a_array.item (1), ?)))
		ensure
			definition: Result = (a_array.lower = 1 and (a_array.count > 0
				and then a_array.for_all (agent same_array_count (a_array.item (1), ?))))
		end

	uniform_blocks (a_array : ARRAY[ARRAY[ABSTRACT_READABLE_MATRIX]]) : BOOLEAN is
			-- Are the provided blocks in `a_array' uniform, i.e. rows of same height and
			-- the total length of each row is the same?
		local
			i,j,l_old_width,l_width: INTEGER
			l_blocks : ARRAY[ABSTRACT_READABLE_MATRIX]
		do
			Result := True
			from i := 1 until i > a_array.count or else not Result loop
				l_blocks := a_array[i]
				l_old_width := l_width
				l_width := 0
				from j := 1 until j > l_blocks.count - 1 or else not Result loop
					Result := l_blocks.item(j).height = l_blocks.item(j+1).height
					l_width := l_width + l_blocks.item(j).width
					j := j + 1
				end
				l_width := l_width + l_blocks.item(j).width
				if i > 1 then
					Result := l_width = l_old_width
				end
				i := i + 1
			end
		end

feature {NONE} -- Assertion support implementation

	same_array_count (a_array, a_other: ARRAY [DOUBLE]): BOOLEAN is
			-- Have the two arrays got the same count?
		require
			a_array_not_void: a_array /= Void
			a_other_not_void: a_other /= Void
		do
			Result := a_array.count = a_other.count
		end
	string_compare(a,b:STRING):BOOLEAN is
			--compare strings
		do
			result:=a<b
		end

feature -- Measurement
	Growth_percentage: INTEGER is 20
			-- Percentage by which structure will grow automatically

	Minimal_increase: INTEGER is 5
			-- Minimal number of additional items

	duplicate (a_matrix : ABSTRACT_READABLE_MATRIX)
			-- A creation procedure that makes a duplicate of 'a_matrix'.
		local
			i,j : INTEGER
			l_row_i,l_other_i : SPECIAL[DOUBLE]
			l_other : SPECIAL[SPECIAL[DOUBLE]]
		do
			make(a_matrix.height,a_matrix.width)
			if attached {MATRIX} a_matrix as l_matrix then
				l_other := l_matrix.data
				from i := 1 until i > height loop
					l_row_i := data[i]
					l_other_i := l_other[i]
					from j := 1 until j > width loop
						l_row_i[j] := l_other_i[j]
						j := j + 1
					end
					i := i + 1
				end
			else
				from i := 1 until i > height loop
					l_row_i := data[i]
					from j := 1 until j > width loop
						l_row_i[j] := a_matrix[i,j]
						j := j + 1
					end
					i := i + 1
				end
			end
		end

	inverse : MATRIX is
		--return the inverse of Current
		do
			create Result.make_as_inverse(Current)
		end

feature --data related

	data : SPECIAL[SPECIAL[DOUBLE]]
			-- Elements of current matrix.
	height : INTEGER
			-- Height of current matrix.
	width : INTEGER
			-- Width of current matrix.

	frozen item alias "[]" (i,j : INTEGER) : DOUBLE assign put
			--Frozen so that it can be appropriately optimized
		local
			l_row_i : SPECIAL[DOUBLE]
		do
			if i >= 1 and then i <= height and then j >= 1 and then j <= width then
				l_row_i := data[i]
				Result := l_row_i[j]
			end
		end
	frozen put (a_value : DOUBLE;i,j : INTEGER)
			--Frozen so that it can be appropriately optimized
		local
			l_row_i : SPECIAL[DOUBLE]
		do
			if i >= 1 and then i <= height and then j >= 1 and then j <= width then
				l_row_i := data[i]
				l_row_i[j] := a_value
			end
		end
	transpose,transposed : MATRIX
			--Transpose
		local
			i,j : INTEGER
			l_row_i,l_column : SPECIAL[DOUBLE]
		do
			create Result.make(width,height)
			from i := 1 until i > height loop
				l_row_i := data[i]
				from j := 1 until j > width loop
					l_column := Result.data[j]
					l_column[i] := l_row_i[j]
					j := j + 1
				end
				i := i + 1
			end
			Result.set_name (name)
			Result.set_column_names (row_names)
			Result.set_row_names (column_names)
		end

	replace_diagonal_with_tolerance (a_tolerance: DOUBLE)
			-- Replace diagonal entries which are undefined doubles or lower than `a_tolerance' with 1.
		local
			i:INTEGER
			l_i_i : DOUBLE
			l_row_i : SPECIAL[DOUBLE]
		do
			from
				i:= 1
			until
				i > height.min (width)
			loop
				l_row_i := data[i]
				l_i_i := l_row_i[i]
				if l_i_i < a_tolerance or not is_defined_double(l_i_i) then
					l_row_i[i] := 1.0
				end
				i := i + 1
			end
		end
	replace_diagonal
			-- Replace diagonal entries which are undefined doubles or zero with 1.
		local
			i:INTEGER
			l_i_i : DOUBLE
			l_row_i : SPECIAL[DOUBLE]
		do
			from i:=1 until i>height loop
				l_row_i := data[i]
				l_i_i := l_row_i[i]
				if l_i_i=0 or not is_defined_double (l_i_i) then
					l_row_i[i] := 1
				end
				i:=i+1
			end
		end

	replace_null is
			-- Replace undefined doubles and null values with zero.
		local
			i,j:INTEGER
			l_v:DOUBLE
		do
			from i:=1 until i> height loop
				from  j:=1 until j>width loop
					l_v:=Current[i,j]
					if not is_defined_double (l_v) then
						Current[i, j] := 0.0
					end
					j:=j+1
				end
				i:=i+1
			end
		end

	frozen 	duplicate_matrix (a_matrix : MATRIX) is
			--Frozen so that it can be appropriately optimized
		local
			i,j : INTEGER
			l_row_i,l_other_i : SPECIAL[DOUBLE]
		do
			make(a_matrix.height,a_matrix.width)
			from i := 1 until i > height loop
				l_row_i := data[i]
				l_other_i := a_matrix.data[i]
				from j := 1 until j > width loop
					l_row_i[j] := l_other_i[j]
					j := j + 1
				end
				i := i + 1
			end
		end

	frozen infix "*" (a_other : MATRIX) : MATRIX is
			--Frozen so that it can be appropriately optimized
		local
			i,j,k : INTEGER
			l_dot_product : DOUBLE
			l_row_i : SPECIAL[DOUBLE]
			l_result_i : SPECIAL[DOUBLE]
			l_other: MATRIX
			l_other_data : SPECIAL[SPECIAL[DOUBLE]]
			l_other_j: SPECIAL[DOUBLE]
		do
			l_other := a_other.transpose
			l_other_data := l_other.data
			create Result.make(height,a_other.width)
			from i := 1 until i > height loop
				l_row_i := data.item(i)
				l_result_i := Result.data.item(i)
				from j := 1 until j > a_other.width loop
					l_dot_product := 0
					l_other_j := l_other_data[j]
					from k := 1 until k > a_other.height loop
						l_dot_product := l_dot_product + l_row_i[k] * l_other_j[k]
						k := k + 1
					end
					l_result_i[j] := l_dot_product
					j := j + 1
				end
				i := i + 1
			end
		end

	frozen transpose_at (a_vector : ABSTRACT_READABLE_VECTOR) : VECTOR is
		-- Using matrix.data and vector.data.
		local
			l_data : SPECIAL[DOUBLE]
			i,j : INTEGER
			l_vector_j: DOUBLE
			l_row_j : SPECIAL[DOUBLE]
		do
			create Result.make(width)
			l_data := Result.data

			from j := 1 until j > height loop
				l_row_j := data.item(j)

				l_vector_j := a_vector.item(j)
				from i := 1 until i > width loop
					l_data[i] := l_data[i] + l_row_j[i] * l_vector_j
					i := i + 1
				end
				j := j + 1
			end
		end

	frozen vector_product,infix "@" (a_vector : VECTOR) : VECTOR is
			-- `current' times `a_vector' where undefineds are treated as 0s
		local
			l_data,l_row_i,l_other : SPECIAL[DOUBLE]
			i,j : INTEGER
			l_sum : DOUBLE
		do
			create Result.make(height)
			l_data := Result.data
			l_other := a_vector.data
			from i := 1 until i > height loop
				l_sum := 0
				l_row_i := data[i]
				from j := 1 until j > width loop
					if is_defined_double(l_row_i[j]) and is_defined_double(l_other [j]) then
						l_sum := l_sum + l_row_i[j] * l_other [j]
					end
					j := j + 1
				end
				l_data[i] := l_sum
				i := i + 1
			end
		end

	frozen column (j: INTEGER): VECTOR is
			-- J-th column as a vector
		local
			i: INTEGER
			l_row_i : SPECIAL[DOUBLE]
		do
			create Result.make (height)
			from i := 1 until i > height loop
				l_row_i := data[i]
				Result[i] := l_row_i[j]
				i := i + 1
			end
			if column_names/=void then
				result.set_name (column_names.item (j))
			end
			Result.set_names (row_names)
		end

	to_correlation: FAST_SYMMETRIC_MATRIX is
		-- require semi positive definite matrix (like covariance matrix)
		local
			i, j: INTEGER
			l_ij, l_ii, l_jj: DOUBLE
			l_row_i,l_row_j,l_result : SPECIAL[DOUBLE]
		do
			create Result.make (width)
			from i := 1 until i > width loop
				l_row_i := data[i]
				l_result := Result.data[i]
				from j := 1 until j > i loop
					l_row_j := data[j]
					l_ii := l_row_i[i]
					l_jj := l_row_j[j]
					l_ij := l_row_i[j]
					if is_defined_double (l_ii) and then is_defined_double (l_jj) and then is_defined_double (l_ij) and then l_ii > 0. and then l_jj > 0. then
						l_result[j] := l_ij/sqrt (l_ii * l_jj)
					else
						l_result[j] := undefined_double
					end
					j := j + 1
				end
			i := i + 1
			end
			if column_names/=void then
				Result.set_column_names(column_names)
				Result.set_row_names(column_names)
			end
		end

	frozen xtx : FAST_SYMMETRIC_MATRIX is
			--Matrix times transpose
		do
			create Result.make_xtx_weighted(Current,Void)
			Result.set_column_names(column_names)
			Result.set_row_names(column_names)
		end

	frozen column_sum (j : INTEGER) : DOUBLE is
			-- Sum of column items
		local
			i : INTEGER
			l_row_i : SPECIAL[DOUBLE]
		do
			from i := 1 until i > height loop
				l_row_i := data[i]
				Result := Result + l_row_i[j]
				i := i + 1
			end
		end

	frozen row (i: INTEGER): VECTOR is
			-- I-th row as a vector
		local
			j: INTEGER
			l_row_i : SPECIAL[DOUBLE]
		do
			create Result.make (width)
			l_row_i := data[i]
			from j := 1 until j > width loop
				Result[j] := l_row_i[j]
				j := j + 1
			end
			Result.set_names (column_names)
			if row_names /= Void and then row_names.valid_index (i) then
				Result.set_name (row_names [i])
			end
		end

	put_row_vector(a_vector : attached VECTOR; a_row_ix : INTEGER) is
			--Replace row with index `a_row_ix' with `a_vector'
		local
			i : INTEGER
			l_row : SPECIAL[DOUBLE]
		do
			if  valid_row_index(a_row_ix) then
				l_row := data[a_row_ix]
				from i := 1 until i > a_vector.count or i > width loop
					l_row[i] := a_vector[i]
					i := i + 1
				end
			end
		end
	add (a_other: MATRIX) is
		-- Perfom addition in-place. (Compare to feature infix "+".)
		local
			l_row_i,l_other_i : SPECIAL[DOUBLE]
			i,j : INTEGER
		do
			from i := 1 until i > height loop
				l_row_i := data[i]
				l_other_i := a_other.data[i]
				from j := 1 until j > width loop
					l_row_i[j] := l_row_i[j] + l_other_i[j]
					j := j + 1
				end
				i := i + 1
			end
		end
	subtract (a_other: MATRIX) is
		-- Perfom subtraction in-place. (Compare to feature infix "-".)
		local
			l_row_i,l_other_i : SPECIAL[DOUBLE]
			i,j : INTEGER
		do
			from i := 1 until i > height loop
				l_row_i := data[i]
				l_other_i := a_other.data[i]
				from j := 1 until j > width loop
					l_row_i[j] := l_row_i[j] - l_other_i[j]
					j := j + 1
				end
				i := i + 1
			end
		end

feature -- Cache handling

	regenerate_column_table is
			-- Reset column table to Void
		obsolete
			"There should be no need to call this feature"
		do
			column_table_saved := void
			column_table.do_nothing
		end

feature{NONE} --Implementation

	correct_mismatch is
		--This should be removed! It says in essence to ignore any storable problems
		do  end

	anchor : MATRIX
			--Template
		do end

	additional_space (a_capacity: INTEGER): INTEGER is
			-- Proposed number of additional items
		do
			Result := (a_capacity * growth_percentage // 100).max (minimal_increase)
		ensure
			at_least_one: Result >= 1
		end

	make_from_selected_columns(a_matrix : ABSTRACT_READABLE_MATRIX;a_columns : ARRAY[INTEGER]) is
			-- make from selected columns
		require
			valid_matrix: attached a_matrix
			valid_columns: attached a_columns
		local
			l_row_index:ROSE_LINEAR_ARRAY[INTEGER]
			i: INTEGER
		do
			create l_row_index.make(1,a_matrix.height)
			from i:=1 until i> a_matrix.height loop
				l_row_index.put(i,i)
				i:=i+1
			end
			make_sub_matrix_3 (a_matrix, l_row_index, a_columns)
		  	set_row_names(a_matrix.row_names)
		  	if a_matrix.column_names /= Void then
		  		set_column_names(a_matrix.column_names.selected_items(a_columns))
		  	end
		end

	string_without_commas(a_string : STRING) : STRING is
			-- `a_string' without its commas, and Void replaced by empty string
		local
			i : INTEGER
		do
			if a_string = Void then
				Result := ""
			else
				create result.make(0)
				from i := 1 until i > a_string.count loop
					if a_string.item(i) /= ',' then
						result.extend(a_string.item(i))
					end
					i := i + 1
				end
			end
		end

end
