indexing
	description: "Utility functions for matrix/vector"
	author: "Pierre Nallet"
	last_editor: "$Author: ext-asurelia $"
	date: "$Date: 2014/07/11 $"
	revision: "$Revision: #5 $"
	path: "$File: //rose/source/kernel/Support/matrix_helper.e $"

class
	MATRIX_HELPER

feature -- Access

	max_column_count: INTEGER assign set_max_column_count
			-- The current max column count

	set_max_column_count (a_value: like max_column_count)
			-- Simple assignment
		do
			max_column_count := a_value
		end

feature -- Conversions and Combinations

	matrix_combined_to_array2 (a_matrix : ABSTRACT_READABLE_MATRIX; a_row_label:ARRAY[ANY]; a_column_label:ARRAY[ANY]; a_name:STRING; a_other:ARRAY2[ANY]): PAIR[ARRAY2[ANY], STRING]
			-- matrix combined to array2
		local
			l_name : STRING
			i, j:INTEGER
			l_output, l_output2 : ARRAY2[ANY]
			l_max_column_count: INTEGER
		do
			if max_column_count = 0 then
				l_max_column_count := k_max_column_count_default
			else
				l_max_column_count := max_column_count
			end
			l_output := a_matrix.combined_output (a_row_label, a_column_label, a_other)
			if l_output.width > l_max_column_count then
				create l_output2.make (l_output.height, l_max_column_count)
			    from i := 1 until i > l_output.height loop
			    	from j := 1 until j > l_max_column_count loop
							l_output2.put(l_output.item(i,j),i,j)
			    		j := j + 1
			    	end

			    	i := i + 1
			    end
			else
				l_output2 := l_output
			end

			if a_name /= Void then
				l_name := a_name
			else
				l_name := "data"
			end

			create Result.make (l_output2, l_name)
		end

	matrix_to_array_with_names (a_matrix : ABSTRACT_READABLE_MATRIX): PAIR[ARRAY2[ANY], STRING]
			-- matrix to excel
		local
			l_name : STRING
		do
			l_name := a_matrix.name
			if l_name = Void then
				l_name := "data"
			end

			Result := matrix_to_array (a_matrix, a_matrix.row_names, a_matrix.column_names, l_name)
		end

	vector_to_array_with_names (a_vector: ABSTRACT_READABLE_VECTOR; a_column: BOOLEAN; a_name: STRING): PAIR[ARRAY2[ANY], STRING]
			-- vector to excel
		require
			names: a_vector.names /= Void
		local
			l_matrix: MATRIX
		do
			if a_column then
				l_matrix:= a_vector.to_column_matrix
				if a_vector.names/= Void then
					l_matrix.set_row_names(a_vector.names)
				end
			else
				l_matrix:= a_vector.to_row_matrix
				if a_vector.names/= Void then
					l_matrix.set_column_names(a_vector.names)
				end
			end
			l_matrix.set_name(a_name)
			Result := matrix_to_array_with_names(l_matrix)
		end

	matrix_array_to_array2_array (a_array: ARRAY [ABSTRACT_READABLE_MATRIX]): ROSE_LINEAR_ARRAY[TUPLE[ARRAY2[ANY],STRING]]
			-- array of matrix to excel
		local
			i : INTEGER
			l_matrix : ABSTRACT_READABLE_MATRIX
			l_pair : PAIR[ARRAY2 [ANY],STRING]
		do
			create Result.make_empty
			from i := a_array.upper until i < a_array.lower loop
				l_matrix := a_array[i]
				l_pair := matrix_combined_to_array2(l_matrix, l_matrix.row_names, l_matrix.column_names, l_matrix.name, Void)
				if l_pair /= Void then
					Result.extend([l_pair.first, l_pair.second])
				end
				i := i - 1
			end
		end

	matrix_to_array (a_matrix: ABSTRACT_READABLE_MATRIX; a_row_label: ARRAY[ANY]; a_column_label: ARRAY[ANY]; a_name:STRING): PAIR[ARRAY2[ANY], STRING]
			-- matrix to excel
		do
			Result := 	matrix_combined_to_array(a_matrix, a_row_label,a_column_label,a_name,void)
		end

	matrix_combined_to_array (a_matrix: ABSTRACT_READABLE_MATRIX; a_row_label: ARRAY[ANY]; a_column_label: ARRAY[ANY]; a_name: STRING; a_other: ARRAY2[ANY]): PAIR[ARRAY2[ANY], STRING]
			-- matrix combined to excel
		require
			valid_name: attached a_name
		do
			if attached {PAIR[ARRAY2[ANY], STRING]} matrix_combined_to_array2 (a_matrix, a_row_label, a_column_label, a_name, a_other) as al_pair then
				Result := al_pair
			end
		end

feature {NONE} -- Constant

	k_max_column_count_default: INTEGER = 256
			-- Default for max column count allowed

end
