indexing
	description: "Extension of the WEL_SYSTEM_TIME class"
	author: "David Jenkins"
	date: "$Date: 2008/12/01 $"
	revision: "$Revision: #4 $"

class
	MC_SYSTEM_TIME

inherit
	WEL_SYSTEM_TIME
		redefine
			out
		end

create
	make,
	make_by_pointer,
	make_by_current_time,
	make_by_date,
	make_by_date_time

feature {NONE} -- Initialization

	make_by_date(d: DATE) is
		require
			date_exists: d /= Void
		do
			make
			set_date(d)
		end

	make_by_date_time(d_t:DATE_TIME) is
		require
			date_time_exists: d_t /= Void
		do
			make_by_date(d_t.date)
			set_hour(d_t.hour)
			set_minute(d_t.minute)
			set_second(d_t.second)
		end

feature -- Element change

	set_date (d: DATE) is
		require
			date_exists: d /= Void
		do
			set_month(d.month)
			set_year(d.year)
			set_day(d.day)
		ensure
			day_set: day = d.day
			month_set: month = d.month
			year_set: year = d.year
		end

	set_day(d: INTEGER) is
		require
			day_large_enough: d > 0
			day_small_enough: d <= days_in_month
		do
			c_set_syst_day(item,d)
		ensure
			day_set: day = d
		end

	set_month(m: INTEGER) is
		require
			month_large_enough: m > 0
			month_small_enough: m <= 12
		do
			c_set_syst_month(item,m)
		ensure
			month_set: month = m
		end

	set_year(y: INTEGER) is
		require
			year_positive: y >= 0
			year_small_enough: y<= 65535
		do
			c_set_syst_year(item,y)
		ensure
			year_set: year = y
		end

	set_hour(h:INTEGER) is
		require
			hour_positive: h >= 0
			hour_small_enough: h < 24
		do
			c_set_syst_hour(item,h)
		ensure
			hour_set: hour = h
		end

	set_minute(m:INTEGER) is
		require
			minute_positive: m >= 0
			minute_small_enough: m < 60
		do
			c_set_syst_minute(item,m)
		ensure
			minute_set: minute = m
		end

	set_second(s:INTEGER) is
		require
			second_positive: s >= 0
			second_small_enough: s < 60
		do
			c_set_syst_second(item,s)
		ensure
			second_set: second = s
		end

feature -- Conversion

	as_date: DATE is
		do
			create Result.make(year,month,day)
		end

	as_date_time: DATE_TIME is
		do
			create Result.make(year,month,day,hour,minute,second)
		end

feature -- Status report

	days_in_month: INTEGER is
		do
			Result := as_date.days_in_month
		end

feature -- Output

	out: STRING is
		do
			Result := as_date_time.out
		end

feature {NONE} -- Externals

	c_set_syst_year(handle: POINTER; yr: INTEGER) is
		external
			"C[struct %"wel.h%"](SYSTEMTIME,WORD)"
		alias
			"wYear"
		end

	c_set_syst_month(handle: POINTER; mn: INTEGER) is
		external
			"C[struct %"wel.h%"](SYSTEMTIME,WORD)"
		alias
			"wMonth"
		end

	c_set_syst_day(handle: POINTER; d: INTEGER) is
		external
			"C[struct %"wel.h%"](SYSTEMTIME,WORD)"
		alias
			"wDay"
		end

	c_set_syst_hour(handle: POINTER; d: INTEGER) is
		external
			"C[struct %"wel.h%"](SYSTEMTIME,WORD)"
		alias
			"wHour"
		end

	c_set_syst_minute(handle: POINTER; d: INTEGER) is
		external
			"C[struct %"wel.h%"](SYSTEMTIME,WORD)"
		alias
			"wMinute"
		end

	c_set_syst_second(handle: POINTER; d: INTEGER) is
		external
			"C[struct %"wel.h%"](SYSTEMTIME,WORD)"
		alias
			"wSecond"
		end

end -- class MC_SYSTEM_TIME
