note

	description:

		"Special date class for model"

	author: "Michael Zhang"
	last_editor: "$Author: c4 $"
	date: "$Date: 2014/01/08 $"
	revision: "$Revision: #15 $"
	path: "$File: //rose/source/kernel/infrastructure/datetime/model_date.e $"

class
	MODEL_DATE
	
inherit
	ROSE_DATE
		rename
			make as make_date
		undefine
			hash_code, is_equal
		redefine
			infix "<",
			out,
			debug_output,
			is_equal,
			display
		end
	MULTI_KEYED_TEMP
		undefine
			out,
			default_create,
			is_equal
		end
	
	create
    make
	
	create {MODEL_DATE}
    make_date
	
feature {NONE} -- Initialization

	make (a_ccyymmdd: INTEGER; a_type: STRING; a_as_of_date: like model_as_of_date) is
			-- Create the date object
		require
			valid_type: a_type /= Void
		do
			make_ccyymmdd (a_ccyymmdd)
			type := a_type
			model_as_of_date := a_as_of_date
			if model_as_of_date = Void then
				create model_as_of_date.make_ccyymmdd (a_ccyymmdd)
			end
		end
	
feature -- Access

	type: STRING
			-- Model type ("DP", "PD", etc.)

	set_type(a_type : like type) is
			-- set type
		do
			type:= a_type
		end

	model_as_of_date: ROSE_DATE
			-- Model base date if this is an FA type
	
	key_count: INTEGER is 2
			-- Number of keys used in the multi key
	
	out, debug_output, display: STRING is
			-- Display string for the object
		do
			if ccyymmdd = model_as_of_date.ccyymmdd and then model_as_of_date.persecs = 0 then
				Result := ccyymmdd.out + ": " + type
			else
				Result := ccyymmdd.out + ": " + type + ": "  + model_as_of_date.debug_output
			end
		end
	
feature -- Comparison

	hash_key (i: INTEGER): HASHABLE is
			-- The i'th hash code
		do
			inspect i
			when 1 then 
				Result := compact_date + persecs
			when 2 then 
				Result := model_as_of_date
			else
			end
		end

	infix "<" (a_other: like Current): BOOLEAN is
			-- Is Current less than 'a_other'?
		local
			l_model_date: MODEL_DATE
		do
			l_model_date ?= a_other
			if l_model_date /= Void then
				Result := Precursor (a_other) or else (ordered_compact_date = a_other.ordered_compact_date and persecs = a_other.persecs) and then model_as_of_date < a_other.model_as_of_date
			else
				Result := Precursor (a_other)
			end
		end
	
	is_equal (a_other: MODEL_DATE): BOOLEAN is
			-- need to relax this to allow for ROSE_DATE or ROSE_DATE_TIME
		do
			Result := Precursor{ROSE_DATE}( a_other )
		end

end












