indexing
	description: "Signature tagged vessel for model parameters"
	author: "Michael Zhang"
	date: "$Date: 2009/08/24 $"
	revision: "$Revision: #12 $"
	archive: "$File: //rose/source/kernel/model/parameter/model_parameter_record.e $"

deferred class
	MODEL_PARAMETER_RECORD

inherit
	ROSE_SIGNATURE_RECORD[ENTITY]
		undefine
			context,
			is_valid
		end

	ROSE_LEGACY_UNDEFINED
		undefine
			is_equal
		end

	ROSE_SHARED_GLOBAL
		undefine
			is_equal
		end

	ROSE_AWARE

feature -- Operations

	null_out is
		deferred
		end

	is_valid : BOOLEAN is
		deferred
		end

feature -- Access

	user_id : STRING


	time_stamp : ROSE_DATE

	context_name : STRING is
			-- Context name
		do
			Result := context.name
		end

feature -- Change

	set_user_id(a_id : like user_id) is
			-- Set 'user_id' to 'a_id'
		do
			user_id := a_id
		end

	set_time_stamp(a_stamp : like time_stamp) is
			-- Set 'time_stamp' to 'a_stamp'
		do
			time_stamp := a_stamp
		end

	set_entity_id(a_id : INTEGER) is
			-- Do nothing
		do
		end

feature -- Operations

	process_id_manager : SUPPLIER_PROCESS_ID_MANAGER is
		once
			create Result.make(server)
			result.set_name("model_w")
		end

	commit is
			-- Commit
		do
			data_manager.submit
			process_id_manager.end_process
			server.commit
		end

	server : SQL_DATA_SERVER is
			-- Sql server
		do
			Result := servers.model_server
		end

feature -- File access

	load_from_file(a_key : STRING; a_dir : STRING; a_date : ROSE_DATE; a_process_name : STRING) is
		local
			l_time : ROSE_DATE
			l_record : like Current
			l_existing_records : like key_records
			l_records : like records_from_file
		do
			l_existing_records := key_records(Current)
			create l_time.make_by_date (a_date)
			process_id_manager.start_process (a_process_name, l_time, l_time, l_time)

			l_records := records_from_file(a_key, a_dir)
			if l_records /= Void then
				from l_records.start until l_records.off loop
					data_manager.replace_signature_data (l_records.current_item)
					l_records.forth
				end
			end

			if l_records /= Void and l_existing_records /= Void then
				from l_existing_records.start until l_existing_records.off loop
					l_record := l_existing_records.current_item.deep_twin
					if not l_records.has_key(l_record.multi_key) then
						l_record.null_out
						data_manager.replace_signature_data (l_record)
					end
					l_existing_records.forth
				end
			end
			commit
		end

	records_from_file(a_key : STRING; a_dir : STRING) : ROSE_INDEXED_ARRAY[like Current, MULTI_KEY] is
		deferred
		end

	key_records(a_direct_record : like Current) : like targets is
		deferred
		end

end
