note

	description:

		"Model variable coefficient signature tagged record"

	author: "Michael Zhang"
	last_editor: "$Author: er $"
	date: "$Date: 2014/02/26 $"
	revision: "$Revision: #2 $"
	path: "$File: //rose/source/kernel/model/model_variable_coefficient_record.e $"

class MODEL_VARIABLE_COEFFICIENT_RECORD

inherit

	MODEL_VARIABLE_COEFFICIENT_RECORD_BASE
		redefine
			data_fields_string_,
			data_manager,
			read_only_data_manager,
			is_gui_editable,
			set_entity
		end
	EXCEPTIONS
		undefine
			is_equal
		end
create

	make,
	default_create

create {MODEL_VARIABLE_COEFFICIENT_RECORD}

	rose_entity_date_make

feature {NONE}-- create

	make (a_variable, a_component_variable, a_zone, a_country, a_size: STRING; a_coefficient: DOUBLE) is
			-- make
		do
			variable := a_variable
			component_variable := a_component_variable
			zone := a_zone
			country := a_country
			size := a_size
			normalization := ' '
			base_alternative := ' '
			winsor := undefined_double
			coefficient := a_coefficient
		end

feature -- attributes

	normalization: CHARACTER

	set_normalization (a_normalization: like normalization) is
			--Setter for normalization
		do
			normalization := a_normalization.as_lower
		end

	base_alternative: CHARACTER

	set_base_alternative (a_base_alternative: like base_alternative) is
			--Setter for base_alternative
		do
			base_alternative := a_base_alternative
		end

	winsor: DOUBLE

	set_winsor (a_winsor: like winsor) is
			-- Set 'winsor' to 'a_winsor'
		do
			winsor := a_winsor
		end

	set_entity (a_value: like entity) is
			-- <Precursor>
		do
			Precursor (a_value)
			if a_value /= Void then
				set_variable (a_value.descriptive_name)
			else
				set_variable (Void)
			end
		end

feature -- access

	is_gui_editable: BOOLEAN is
			-- May the contents of this record be edited manually from a GUI?
		do
			Result := True
		end

	null_out is
			-- Null all fields of `Current'.
		do
			coefficient := undefined_double
			winsor := undefined_double
			signature_data_process_id := 0
		end

	is_industry_normalized: BOOLEAN is
			--normalization is neither a space character nor a null character
		do
			Result := normalization /= ' ' and normalization /= '%U'
		end

	to_lower is
			-- Convert data to lowercase.
		do
			variable.to_lower
			component_variable.to_lower
			zone.to_lower
			country.to_lower
			size.to_lower
			normalization := normalization.as_lower
			base_alternative := base_alternative.as_lower
		end

	expression_tag: STRING is
		do
			Result := "Model Variable Coefficient"
		end

	fields_: STRING is "variable,component_variable,zone,country,size"

	data_fields_string_: STRING is
		once
			Result := "normalization,base_alternative,winsor,coefficient"
		end

	variable_key: MULTI_KEY is
		do
			create Result.make (<<component_variable.as_lower, zone.as_lower, country.as_lower, size.as_lower>>)
		end

	records_from_file (a_variable: STRING; a_dir: STRING): ROSE_INDEXED_ARRAY [like Current, MULTI_KEY] is
		local
			l_coefficient: DOUBLE
			l_variable, l_component_variable, l_zone, l_country, l_size: STRING
			l_string: STRING
			l_record: like Current
			l_file: ROSE_TEXT_FILE
			l_lines: ROSE_FILE_LINES
			l_fields: ROSE_DELIMITED_TEXT
		do
			if a_dir /= Void then
				create l_file.make (a_dir + a_variable + ".coef")
			end
			create Result.make_empty
			l_variable := a_variable.as_lower
			create l_lines.make (l_file)
			from
				l_lines.start
			until
				l_lines.off
			loop
				l_string := l_lines.item
				if l_string.count > 0 and then l_string.item (1) /= '!' then
					Create l_fields.make_removing_spaces (l_string, ';')
					if l_fields.count >= 5 then
						l_component_variable := l_fields.item (1).as_lower
						l_zone := l_fields.item (2).as_lower
						l_country := l_fields.item (3).as_lower
						l_size := l_fields.item (4).as_lower
						l_coefficient := l_fields.item (5).to_double
						create l_record.make (l_variable, l_component_variable, l_zone, l_country, l_size, l_coefficient)
						Result.extend (l_record, l_record.multi_key)
					end
				end
				l_lines.forth
			end
		end
	component_expression : ROSE_EXPRESSION[ROSE_ENTITY_DATE[COMPANY]]
	set_component(a_component : like component_expression)
			--Set the component expression
		do
			component_expression := a_component
		end
feature {NONE} --Implementation
	empty_record: like Current is
		do
			create Result
		end

	data_manager: SIGNATURE_TAGGED_ITEM_DATABASE [MODEL_VARIABLE_COEFFICIENT_RECORD] is
			--Signature tagged manager
		once
			create Result.make_named_with_process_manager ("variable_coefficient", server, process_id_manager)
		end

	read_only_data_manager (a_server: SQL_DATA_SERVER): like data_manager is
			-- Return a data manager used only for reading signatures.
		do
			create Result.make_named (data_manager.data_name, data_manager.process_name, a_server)
		end

end
