note

	description:

		"Base record for variable coefficient"

	author: "Michael Zhang"
	last_editor: "$Author: er $"
	date: "$Date: 2014/02/26 $"
	revision: "$Revision: #6 $"
	path: "$File: //rose/source/kernel/model/parameter/model_variable_coefficient_record_base.e $"

deferred class MODEL_VARIABLE_COEFFICIENT_RECORD_BASE

inherit

	MODEL_PARAMETER_RECORD
		redefine
			debug_output
		end

feature -- Access

	empty_record: like Current is
			--empty_record
		deferred
		end

	variable_key: MULTI_KEY is
			--variable_key
		deferred
		end

	variable: STRING
			--variable

	component_variable: STRING
			--component_variable

	zone: STRING
			--zone

	country: STRING
			--country

	size: STRING

	coefficient: DOUBLE

	lower_winsor: DOUBLE

	upper_winsor: DOUBLE

	winsorized (a_value: DOUBLE): DOUBLE is
		do
			if is_defined_double (a_value) then
				if is_defined_double (lower_winsor) and then (lower_winsor /= 0.0 and a_value < lower_winsor) then
					Result := lower_winsor
				elseif is_defined_double (upper_winsor) and then (upper_winsor /= 0.0 and a_value > upper_winsor) then
					Result := upper_winsor
				else
					Result := a_value
				end
			end
		end

	key_records (a_directed_record: like Current): like targets is
		local
			l_targets: like targets
		do
			process_id_manager.set_name ("read")
			create Result.make_empty
			l_targets := targets (<<a_directed_record>>, restriction_statement)
			if l_targets /= Void then
				from
					l_targets.start
				until
					l_targets.off
				loop
					if l_targets.current_item.is_valid then
						Result.extend (l_targets.current_item)
					end
					l_targets.forth
				end
			end
		end

	variable_coefficients_by_variable (a_variable: STRING): like variable_coefficients is
		local
			l_directed_record: like Current
		do
			l_directed_record := empty_record
			l_directed_record.set_variable (a_variable)
			Result := variable_coefficients (l_directed_record)
		end

	variable_coefficients (a_directed_record: like Current): HASH_TABLE [like Current, MULTI_KEY] is
		local
			l_record: like Current
			l_records: like key_records
		do
			create Result.make (10)
			l_records := key_records (a_directed_record)
			if l_records /= Void and then l_records.count > 0 then
				from
					l_records.start
				until
					l_records.off
				loop
					l_record := l_records.current_item
					l_record.to_lower
					Result.put (l_record, l_record.variable_key)
					l_records.forth
				end
			end
		end

feature -- Status

	is_valid: BOOLEAN is
		do
			Result := is_defined_double (coefficient)
		end

	is_valuation: BOOLEAN is
		do
			Result := component_variable /= Void and then component_variable.as_lower.is_equal ("calf")
		end

	is_dsa: BOOLEAN is
		do
			Result := component_variable /= Void and then not component_variable.as_lower.is_equal ("calf")
		end

feature -- Element modification

	set_variable (a_variable: like variable) is
			--set_variable
		do
			variable := a_variable
		end

	set_component_variable (a_component_variable: like component_variable) is
			--set_component_variable
		do
			component_variable := a_component_variable
		end

	set_zone (a_zone: like zone) is
			--set_zone
		do
			zone := a_zone
		end

	set_country (a_country: like country) is
		do
			country := a_country
		end
	set_size (a_size: like size) is
		do
			size := a_size
		end

	set_coefficient (a_coefficient: like coefficient) is
		do
			coefficient := a_coefficient
		end

	set_lower_winsor (a_winsor: like lower_winsor) is
		do
			lower_winsor := a_winsor
		end

	set_upper_winsor (a_winsor: like upper_winsor) is
		do
			upper_winsor := a_winsor
		end

feature -- Basic operations

	to_lower is
			-- to_lower
		deferred
		end

feature {NONE} -- Implementation

	restriction_statement: STRING is
		do
			Result := "#variable.id = variable"
		end

	debug_output: STRING is
			-- <Precursor>
		do
			Result := variable + ":" + component_variable + ":" + zone + ":" + country + ":" + size + ":" + format.formatted (coefficient)
		end

	format: FORMAT_DOUBLE is
		once
			create Result.make (10, 6)
		ensure
			valid_result: Result /= Void
		end

end











