note

	description:

		"[
			Currency exchange routines used by mondat variables
		]"

	author: "Ivan Tarnavsky"
	last_editor: "$Author: it $"
	date: "$Date: 2015/04/27 $"
	revision: "$Revision: #1 $"
	path: "$File: //rose/source/kernel/data/market/mondat_fx.e $"

class MONDAT_FX

inherit

	ANY

	IEEE_MATH
		export {NONE} all end

create {SHARED_MONDAT_FX}

	default_create

feature -- Access

	last_rate: DOUBLE
			-- Result of last call to `compute_rate'

	last_return: DOUBLE
			-- Result of last call to `compute_return'

feature -- Basic Operations

	compute_rate (a_base, a_foreign: PERSISTENT_CURRENCY; a_date: ROSE_DATE)
			-- Put the value of fx spot rate from `a_base' to `a_foreign' on `a_date' into `last_rate'
		require
			a_base_attached: a_base /= Void
			a_foreign_attached: a_foreign /= Void
			a_date_attached: a_date /= Void
		local
			l_rate: DS_CELL [DOUBLE]
		do
			create l_rate.make (qnan)
			fetch_spot_rate (a_base, a_foreign, a_date, l_rate)
			last_rate := l_rate.item
		ensure
			undefined_means_qnan: (not is_defined_double (last_rate)) implies isnan (last_rate)
			positive_when_defined: is_defined_double (last_rate) implies last_rate > 0.0
		end

	compute_return (a_base, a_foreign: PERSISTENT_CURRENCY; a_start, a_finish: ROSE_DATE)
			-- Compute `a_base' to `a_foreign' return from `a_start' to `a_finish' into `last_return'
		require
			a_base_attached: a_base /= Void
			a_foreign_attached: a_foreign /= Void
			a_start_attached: a_start /= Void
			a_finish_attached: a_finish /= Void
			a_finish_on_or_after_a_start: a_finish >= a_start
		local
			l_start, l_finish: DS_CELL [DOUBLE]
		do
			if a_base /~ a_foreign or a_start /~ a_finish then
				create l_start.make (qnan)
				fetch_spot_rate (a_base, a_foreign, a_start, l_start)
				create l_finish.make (qnan)
				fetch_spot_rate (a_base, a_foreign, a_finish, l_finish)
				if is_defined_double (l_start.item) and is_defined_double (l_finish.item) then
					last_return := l_finish.item / l_start.item
				else
					last_return := qnan
				end
			else
				last_return := 1.0
			end
		ensure
			undefined_means_qnan: (not is_defined_double (last_return)) implies isnan (last_return)
			positive_when_defined: is_defined_double (last_return) implies last_return > 0.0
		end

feature {NONE} -- Implementation

	fetch_spot_rate (a_base, a_foreign: PERSISTENT_CURRENCY; a_date: ROSE_DATE; a_rate: DS_CELL [DOUBLE])
			-- FX spot rate from `a_base' to `a_foreign' on `a_date'
		require
			a_base_attached: a_base /= Void
			a_foreign_attached: a_foreign /= Void
			a_date_attached: a_date /= Void
			a_rate_attached: a_rate /= Void
		local
			l_rate: DOUBLE
		do
			if a_base /~ a_foreign then
				l_rate := a_foreign.exchange_rate (a_base, a_date)
				if (not is_defined_double (l_rate)) or else l_rate <= 0.0 then
					l_rate := qnan
				end
			else
				l_rate := 1.0
			end
			a_rate.put (l_rate)
		ensure
			undefined_means_qnan: (not is_defined_double (a_rate.item)) implies isnan (a_rate.item)
			positive_when_defined: is_defined_double (a_rate.item) implies a_rate.item > 0.0
		end

end
