note

	description:

		"[
			Price lookups for mondat variable's use
		]"

	author: "Ivan Tarnavsky"
	last_editor: "$Author: it $"
	date: "$Date: 2015/04/27 $"
	revision: "$Revision: #1 $"
	path: "$File: //rose/source/kernel/data/market/mondat_listing_price_lookup.e $"

class MONDAT_LISTING_PRICE_LOOKUP

inherit

	ANY

	IEEE_MATH
		export {NONE} all end

create {SHARED_MONDAT_LISTING_PRICE_LOOKUP}

	default_create

feature -- Access

	last_price: DOUBLE
			-- Result of last call to `lookup_price'

	last_currency: detachable CURRENCY
			-- Result of last call to `lookup_price'

	last_found_date: detachable ROSE_DATE
			-- Result of last call to `lookup_price'

feature -- Basic Operations

	lookup_price (a_listing: LISTING; a_date: ROSE_DATE)
			-- Lookup price
		require
			listing_not_void: a_listing /= Void
			date_not_void: a_date /= Void
		local
			l_price: DOUBLE
		do
			l_price := a_listing.daylp (a_date)
			if is_defined_double (l_price) and then l_price > 0.0 then
				last_price := l_price
				last_found_date := a_listing.zone.market_data.found_date
				if 	attached a_listing.zone as la_zone and then
					attached la_zone.market_data as la_market_data and then
					attached la_market_data.price_record_saved as la_price_record
				then
					last_currency := la_price_record.currency
				else
					last_currency := Void
				end
				validate_currency (a_listing, a_date)
			else
				last_price := qnan
				last_currency := Void
				last_found_date := Void
			end
		end

feature {NONE} -- Implementation

	infer_currency (a_listing: LISTING; a_date: ROSE_DATE)
			-- Infer `last_currency' from parameters.
		require
			a_listing_not_void: a_listing /= Void
			a_date_not_void: a_date /= Void
		local
			a_zone: CHARACTER
		do
			a_zone := a_listing.zone_mnemonic.as_upper
			if a_zone = 'J' then
				last_currency := a_listing.context.jpy
			elseif a_zone = 'O' then
				last_currency := a_listing.currency (a_date)
			end
		end

	is_determinate_currency: BOOLEAN
			-- Is `last_currency' determinate?
		do
			if last_currency = Void then
				Result := False
			elseif last_currency.iso = Void or not last_currency.is_known_currency then
				Result := False
			else
				Result := True
			end
		end

	validate_currency (a_listing: LISTING; a_date: ROSE_DATE)
			-- Validate `last_currency'
		require
			a_listing_not_void: a_listing /= Void
			a_date_not_void: a_date /= Void
		do
			if not is_determinate_currency then
				infer_currency (a_listing, a_date)
			end
		end

end
