indexing
	description: "Movie - designed to allow for recording and playback of a system's IO"
	author: "$Author: fa $"
	date: "$Date: 2009/04/03 $"
	revision: "$Revision: #15 $"
	archive: "$File: //rose/source/kernel/infrastructure/movies/movie.e $"

class
	MOVIE

inherit
	ANY -- Export features from ANY.
	MOVIE_SPECIFIC
		export
			{MOVIE_CAMERA} all
		end
	EXCEPTIONS
		export
			{NONE} all
		end
	MEMORY
		export
			{NONE} all
		end
	SED_STORABLE_FACILITIES
		export
			{NONE} all
		end

create
	make_none,
	make_record,
	make_playback

feature {MOVIE_CAMERA, MOVIE_AWARE} -- Implementation

	make_none is
		do
			init
		end

	make_playback is
			-- Make in playback mode.
		require
			not record_movie
		do
			debug ("make_movies")
				init
				if use_sed then
					sed.set_for_reading
				end
				playback_movie_saved := True
				create cameras.make_empty
				create_server
				get_movie_id_predicate.retrieve (Void, <<exe, movie_variant>>)
				if not get_movie_id_predicate.success then
					raise ("The get_movie_id_predicate failed")
				end
				if get_movie_id_predicate.array.count /= 1 then
					raise ("Movie does not exist")
				end
				movie_id := get_movie_id_predicate.array.item (1)
				max_non_assert_sequence_predicate.retrieve (Void, <<movie_id>>)
				if not max_non_assert_sequence_predicate.success then
					raise ("The max_non_assert_sequence_predicate failed")
				end
				if max_non_assert_sequence_predicate.array.count /= 1 then
					raise ("max_non_assert_sequence_predicate indicates corruption")
				end
				max_non_assert_sequence := max_non_assert_sequence_predicate.array.item (1)
				server.sql_execute_immediate (
											 "update movie_frames set playback_frame_data = NULL %
											 %where movie_id = " + movie_id.out)
				if not server.success then
					raise ("Failed to delete the frames")
				end
				restore_state
				print ("playback_movie%N")
			end
		end

	set_raise_if_after_for_sockets is
		do
			raise_if_after_for_sockets := True
		end

feature {MOVIE_CAMERA} -- Implementation

	make_record is
		require
			playback_movie = False
		do
			debug ("make_movies")
				init
				if use_sed then
					sed.set_for_writing
				end

				record_movie_saved := True

				create cameras.make_empty
				create_server
				load_frame_types

				print ("Removing the old version of this movie.%N")
				server.sql_execute_immediate (
											 "delete from cameras where movie_id in %
											 % (select movie_id from movies where keep = 0 and executable_name = '" + exe + "' and variant = '" + movie_variant + "')")
				if not server.success then
					raise ("Failed to delete the cameras")
				end

				server.sql_execute_immediate (
											 "delete from movie_frames where movie_id in%
											 % (select movie_id from movies where keep = 0 and executable_name = '" + exe + "' and variant = '" + movie_variant + "')")
				if not server.success then
					raise ("Failed to delete the frames")
				end

				server.sql_execute_immediate (
											 "delete from movies where %
											 %keep = 0 and executable_name = '" + exe + "' and variant = '" + movie_variant + "'")
				if not server.success then
					raise ("Failed to delete the movies")
				end

				insert_movie_predicate.execute (Void, <<exe, movie_variant>>)
				if not insert_movie_predicate.success then
					raise ("The insert_movie_predicate failed")
				end

				get_movie_id_predicate.retrieve (Void, <<exe, movie_variant>>)
				if not get_movie_id_predicate.success then
					raise ("The get_movie_id_predicate failed")
				end

				if get_movie_id_predicate.array.count /= 1 then
					raise ("The insert_movie_predicate did not perform as expected")
				end

				movie_id := get_movie_id_predicate.array.item (1)
				restore_state
				print ("record_movie%N")
			end
		end

	new_camera_id (a_camera: MOVIE_CAMERA): INTEGER is
		require
			playback_movie = False
		do
			debug ("make_movies")
				clear_state
				next_camera_id := next_camera_id + 1
				register_camera_predicate.execute (Void, <<movie_id, next_camera_id, a_camera.camera_name.substring (1, a_camera.camera_name.count.min (max_camera_name_length))>>)
				if not register_camera_predicate.success then
					raise ("register_camera_predicate failed")
				end

				Result := next_camera_id
				cameras.force (a_camera, Result)
				restore_state
			end
		end

	camera_id (a_camera: MOVIE_CAMERA): INTEGER is
		require
			record_movie = False
		local
			i: INTEGER
			l_short_name: STRING
		do
			debug ("make_movies")
				clear_state
				next_camera_id := next_camera_id + 1
				camera_name_predicate.retrieve (Void, <<movie_id, next_camera_id>>)
				if not camera_name_predicate.success then
					raise ("camera_name_predicate failed")
				end

				if camera_name_predicate.array.count /= 1 then
					log_error (next_camera_id, 0, "Could not find camera " + next_camera_id.out)
					raise ("Could not find camera")
				end

				if a_camera.camera_name.count > 1000 then
					l_short_name := a_camera.camera_name.substring (1, 1000)
				else
					l_short_name := a_camera.camera_name
				end

				if not equal (camera_name_predicate.array.item (1), l_short_name) then
					log_error (next_camera_id, 0, "Camera name does not match, expecting " + l_short_name + ", found " + camera_name_predicate.array.item (1))
				end

				Result := next_camera_id
				cameras.force (a_camera, Result)

				-- This new camera needs to be up to date as far as it's checkpoints are concerned.
				-- Create enough checkpoints so that this camera has the same number of checkpoints
				-- as all of the other cameras.
				from
					i := 1
				until
					i>checkpoint_depth
				loop
					a_camera.checkpoint
					i := i + 1
				end

				restore_state
			end
		end

	--	record_movie_frames (a_camera_id, a_frame_number: INTEGER; a_frame_desc: STRING; a_frame_data: ARRAY[ANY]) is
	--	do
	--		fix_streams (a_frame_data)	-- Change STREAMS into MOVIE_STREAM's.
	--		record_movie_frame (a_camera_id, a_frame_number, a_frame_desc, a_frame_data)
	--	end

	rerecord_movie_frame (a_camera_id, a_frame_number: INTEGER; a_frame_desc: STRING; a_frame_data: ANY) is
		do
			rerecord := True
			record_movie_frame (a_camera_id, a_frame_number, a_frame_desc, a_frame_data)
			rerecord := False
		end

	record_movie_frame (a_camera_id, a_frame_number: INTEGER; a_frame_desc: STRING; a_frame_data: ANY) is
		require
			a_camera_id /= 0
		local
			l_time: ROSE_DATE
			l_frame_type_id: INTEGER
			l_superframe_stream: ROSE_STREAM
			l_superframe_size_stream: ROSE_STREAM
			l_superframe_type_stream: ROSE_STREAM
			l_size: INTEGER
		do
			debug ("make_movies")
				clear_state		-- This should be the FIRST statement!
				if exit_on_next_record then
					die (2)
				end
				if a_frame_number  = 1 or a_frame_number \\ 1000  = 0 and not rerecord then
					print ("rf " + a_camera_id.out + "/" + a_frame_number.out + "%N")
				end
				if a_frame_desc  /= Void then
					l_frame_type_id := frame_types.item (a_frame_desc)
					if l_frame_type_id  = 0 then
						make_new_frame_type_predicate.execute (Void, <<a_frame_desc>>)
						if not make_new_frame_type_predicate.success then
							raise ("make_new_frame_type_predicate failed")
						end
						load_frame_types
						l_frame_type_id := frame_types.item (a_frame_desc)
						if l_frame_type_id  = 0 then
							raise ("Could not find the frame type that I just made")
						end
					end
					create l_time.make_now
					if use_sed then
						sequence := sequence + 1
						sed.clear_buffer
						independent_store (a_frame_data, sed, True)
						record_frame_predicate.execute (Void, <<movie_id, a_camera_id, sequence, a_frame_number, l_frame_type_id, l_time, in_assertion, False, sed, Void>>)
					else
						if superframe then
							if superframe_streams  = Void then
								create superframe_streams.make_empty
								create superframe_item_1
								superframe_item_1.set_camera_id (a_camera_id)
								superframe_item_1.set_frame_number (a_frame_number)
								superframe_item_1.set_time (l_time)
								superframe_item_1.set_frame_data (l_frame_type_id)
							end
							superframe_count := superframe_count + 1
							-- Store the object.
							create l_superframe_stream
							--						l_super_stream.independent_store (a_frame_data)
							l_superframe_stream.general_store (a_frame_data)
							-- Store the size of the object.
							create l_superframe_size_stream.make_with_size (4)
							l_size := l_superframe_stream.object_stored_size
							l_superframe_size_stream.item.memory_copy ($l_size, 4)
							-- Store the type of the object.
							create l_superframe_type_stream.make_with_size (4)
							l_superframe_type_stream.item.memory_copy ($l_frame_type_id, 4)
							superframe_streams.extend (l_superframe_type_stream)
							superframe_streams.extend (l_superframe_size_stream)
							superframe_streams.extend (l_superframe_stream)
							if superframe_count >= 10000 then
								end_superframe
								start_superframe
								--create superframe_streams.make_empty
							end
						else
							--stream.independent_store (a_frame_data)
							stream.general_store (a_frame_data)
							if rerecord then
								rerecord_frame_predicate.execute (Void, <<stream, movie_id, a_camera_id, a_frame_number>>)
								if not rerecord_frame_predicate.success then
									raise ("rerecord_frame_predicate failed")
								end
							else
								sequence := sequence + 1
								record_frame_predicate.execute (Void, <<movie_id, a_camera_id, sequence, a_frame_number, l_frame_type_id, l_time, in_assertion, False, stream, Void>>)
								if not record_frame_predicate.success then
									raise ("record_frame_predicate failed")
								end
							end
						end
					end
				end
				restore_state		-- This should be the LAST statement!
			end
		end

	next_frame (a_camera_id, a_current_frame_number: INTEGER): MOVIE_FRAME is
		local
			l_frame_size: INTEGER
			l_storable_predicate: DATA_SERVER_STREAM_COMMAND
			l_size: INTEGER
			l_type: INTEGER
			l_pointer: POINTER
			l_is_superframe: BOOLEAN
		do
			clear_state		-- This should be the FIRST statement!
			needs_comparison := True
			if a_current_frame_number = 0 or (a_current_frame_number + 1) \\ 1000 = 0 then
				print ("nf " + a_camera_id.out + "/" + (a_current_frame_number + 1).out + "%N")
			end
			--		if a_camera_id = 179 and a_current_frame_number = 4523
			--		then
			--			do_nothing
			--		end
			if superstream = Void then
				-- Get the parts that aren't storables.
				frame_container.make_empty
				--			sequence := sequence + 1		-- This doesn't really make sense because we aren't forced to retrieve in sequential order
				playback_frame_size_invalid := False
				next_frame_predicate.set_container (frame_container)
				next_frame_predicate.retrieve (Void, <<movie_id, a_camera_id, a_current_frame_number + 1>>)
				if not next_frame_predicate.success then
					raise ("next_frame_predicate failed")
				end

				if frame_container.count /= 1 then
					if request_raise_if_after and frame_container.count = 0 then
						print ("MOVIE is raising an exception because we are 'after' on this camera.%N")
						print ("This is probably an attempt to simulate an exception that is supposed to happen now.%N")
						exception_was_raised := True
						request_raise_if_after := False
						compare_not_needed
						restore_state
						raise ("movie is after")
					end
					log_error (a_camera_id, a_current_frame_number + 1, "Could not find the frame")
					needs_comparison := False
					create Result
				else
					last_frame_read := frame_container.item (1)
					Result := last_frame_read
					l_is_superframe := Result.sf
					l_frame_size := Result.frame_size

					-- Get 'frame_data'.
					l_storable_predicate := next_frame_storable_predicate (a_camera_id, a_current_frame_number + 1, l_frame_size, "frame_data")
					l_storable_predicate.retrieve (Void)
					--				if not l_storable_predicate.success		-- No 'success' on a DATA_SERVER_STREAM_COMMAND.
					--				then
					--					raise ("l_storable_predicate failed")
					--				end

					if l_storable_predicate.query_result.count /= l_frame_size then
						log_error (a_camera_id, a_current_frame_number, "query_result has wrong size")
						needs_comparison := False
					end
				end
			end

			if use_sed then
				Result.set_frame_data (retrieved (sed, True))
			else
				if superstream = Void and l_is_superframe then
					create superstream.make_from_string (l_storable_predicate.query_result)
				end

				if superstream /= Void then
					create Result

					-- Get the type of the next item.
					l_pointer := $l_type
					l_pointer.memory_copy (superstream.item + superstream_pos, 4)
					Result.set_frame_type_id (l_type)
					superstream_pos := superstream_pos + 4

					-- Get the size of the next item.
					l_pointer := $l_size
					l_pointer.memory_copy (superstream.item + superstream_pos, 4)
					superstream_pos := superstream_pos + 4

					-- Get the item itself.
					Result.set_frame_data (superstream.retrieve_from (superstream.item + superstream_pos, l_size))
					superstream_pos := superstream_pos + l_size

					-- See if we are done with the superstream.
					if superstream_pos >= superstream.buffer_size then
						superstream := Void
						superstream_pos := 0
					end
				else
					if l_storable_predicate /= Void then
						Result.set_frame_data (stream.retrieve_from_string (l_storable_predicate.query_result))
					else
						-- Some error happened earlier and we never bothered to set 'l_storable_predicate'.
					end
				end
			end
			restore_state		-- This should be the LAST statement!
		end

	superframe: BOOLEAN is
		do
			Result := superframe_depth > 0
		end

	start_superframe is
		do
			if not superframe then
				superframe_count := 0
			end
			superframe_depth := superframe_depth + 1
		end

	end_superframe is
		require
			superframe
		local
			l_restore_state: BOOLEAN
			l_frame_type_id: INTEGER_REF
		do
			if record_movie or playback_movie then
				clear_state
				l_restore_state := True
			end
			create superstream.make_from_streams (superframe_streams)
			from
				superframe_streams.start
			until
				superframe_streams.off
			loop
				superframe_streams.current_item.close
				superframe_streams.forth
			end
			sequence := sequence + 1
			superframe_streams := Void
			l_frame_type_id ?= superframe_item_1.frame_data
			record_frame_predicate.execute (
										   Void,
										   <<movie_id, superframe_item_1.camera_id, sequence, superframe_item_1.frame_number, l_frame_type_id.item, superframe_item_1.time, False, True, superstream, Void>>)
			if not record_frame_predicate.success then
				raise ("record_frame_predicate failed")
			end
			superstream.close
			superstream := Void
			if l_restore_state then
				restore_state
			end
			superframe_depth := superframe_depth - 1
		end

	set_raise_if_after (a_value: BOOLEAN) is
		do
			request_raise_if_after := a_value
		end

	store_playback_data (a_camera: MOVIE_CAMERA; a_data: ANY) is
		local
			l_frame: MOVIE_FRAME
			l_is_superframe: BOOLEAN
			l_playback_frame_size: INTEGER
			l_storable_predicate: DATA_SERVER_STREAM_COMMAND
			l_data: ANY
			l_array: ROSE_LINEAR_ARRAY[ANY]
			l_superframes: HASH_TABLE[ANY, INTEGER]
		do
			clear_state
			if playback_frame_size_invalid then
				next_frame_predicate.set_container (frame_container)
				next_frame_predicate.retrieve (Void, <<movie_id, last_frame_read.camera_id, last_frame_read.frame_number>>)
				if not next_frame_predicate.success
				 then
					raise ("next_frame_predicate failed")
				end
			end

			if frame_container.count /= 1 then
				log_error (a_camera.camera_id, a_camera.current_frame_number, "Could not find the frame")
			else
				l_frame := frame_container.item (1)
				l_is_superframe := l_frame.sf
				l_playback_frame_size := l_frame.playback_frame_size

				if l_playback_frame_size > 0 then
					-- Get 'frame_data'.
					l_storable_predicate := next_frame_storable_predicate (last_frame_read.camera_id, last_frame_read.frame_number, l_playback_frame_size, "playback_frame_data")
					l_storable_predicate.retrieve (Void)
					--				if not l_storable_predicate.success		-- No 'success' on a DATA_SERVER_STREAM_COMMAND.
					--				then
					--					raise ("l_storable_predicate failed")
					--				end

					if l_storable_predicate.query_result.count /= l_playback_frame_size then
						log_error (a_camera.camera_id, a_camera.current_frame_number, "query_result has wrong size")
					end

					l_data ?= stream.retrieve_from_string (l_storable_predicate.query_result)
					if l_is_superframe then
						-- Superframe data is stored in a HASH_TABLE indexed by the frame number.
						l_superframes ?= l_data
						l_data := l_superframes.item (a_camera.current_frame_number)
					end
					l_array ?= l_data
				end

				if l_array /= Void then
					-- This frame already has at least two items.  Here, we add a third.
					l_array.extend (a_data)
				elseif l_data /= Void then
					-- We had previously found one item for this frame, now we have a second.
					-- We will have to put it into an array.
					create l_array.make_empty
					l_array.extend (l_data)
					l_array.extend (a_data)
					l_data := l_array
				else
					-- First time for this frame.  Just store the item 'as is'.  If another
					-- comes along later, build an array and stuff everything in there.
					l_data := a_data
				end

				if l_is_superframe then
					if l_superframes = Void then
						-- For superframes we use a HASH_TABLE indexed by the frame number.
						-- Each entry is either one single item or an array of items.
						create l_superframes.make (1)
					end

					l_superframes.force (l_data, a_camera.current_frame_number)
					l_data := l_superframes
				end

				-- At this point, 'l_data' is either an ANY, a ROSE_LINEAR_ARRAY[ANY] or a HASH_TABLE[ANY, INTEGER].
				-- Some of the ANY's in the hash table are actually ROSE_LINEAR_ARRAY[ANY].
				-- In all of these cases, the interesting data is the ANY.
				playback_frame_size_invalid := True
				if use_sed then
					raise ("sed is not supported")
					sed.clear_buffer
					independent_store (l_data, sed, True)
					update_playback_frame_predicate.execute (Void, <<sed, movie_id, a_camera.camera_id, a_camera.current_frame_number>>)
				else
					--				stream.independent_store (l_data)
					stream.general_store (l_data)
					update_playback_frame_predicate.execute (Void, <<stream, movie_id, a_camera.camera_id, a_camera.current_frame_number>>)

					if not update_playback_frame_predicate.success then
						raise ("update_playback_frame_predicate failed")
					end
				end
			end

			restore_state
		end

	log_bad_match_error_with_data (a_camera: MOVIE_CAMERA; a_message: STRING; a_data: ANY) is
		do
			log_error (a_camera.camera_id, a_camera.current_frame_number, a_message)
			store_playback_data (a_camera, a_data)
		end

	log_bad_match_error (a_camera: MOVIE_CAMERA; a_message: STRING) is
		do
			log_error (a_camera.camera_id, a_camera.current_frame_number, a_message)
			store_playback_data (a_camera, a_message)
		end

	log_error (a_camera_id, a_frame_number: INTEGER; a_message: STRING) is
		local
			l_camera: MOVIE_CAMERA
			l_camera_name: STRING
			l_has_state: BOOLEAN
		do
			l_has_state := has_state
			clear_state
			print (a_message)
			error_count := error_count + 1

			if a_camera_id /= 0 then
				l_camera := cameras.safe_item (a_camera_id)
				if l_camera /= Void then
					l_camera_name := l_camera.camera_name
				else
					l_camera_name := "?" + a_camera_id.out
				end
				print (", camera = " + l_camera_name)
			else
				print (", camera = 0")
			end
			if a_frame_number /= 0 then
				print (", frame = " + a_frame_number.out)
			end
			print ("%N")
			exit_on_next_record := True
			if l_has_state then
				restore_state
			end
		end

	has_state: BOOLEAN is
		do
			Result := playback_movie or record_movie
		end

	safe_deep_clone (a_frame_data: ANY): ANY is
			-- Calls 'deep_clone' except for items of type STREAM and MANAGED_POINTER.
			-- Those are replaced with MOVIE_STREAM or MOVIE_MANAGED_POINTER.
		local
			i: INTEGER
			l_stream: STREAM
			l_managed_pointer: MANAGED_POINTER
			l_array: ARRAY[ANY]
			l_result_array: ARRAY[ANY]
		do
			if a_frame_data /= Void then
				l_stream ?= a_frame_data
				if l_stream /= Void then
					create {MOVIE_STREAM}Result.make_from_stream (l_stream)
				else
					l_managed_pointer ?= a_frame_data
					if l_managed_pointer /= Void then
						do_nothing
					else
						l_array ?= a_frame_data
						if l_array = Void then
							Result := a_frame_data.deep_twin
						else
							l_result_array := l_array.twin -- Just a shallow clone here.
							Result := l_result_array
							from
								i := l_array.lower
							until
								i > l_array.upper
							loop
								l_result_array.put (safe_deep_clone (l_array.item (i)), i)	-- Recursion!
								i := i + 1
							end
						end
					end
				end
			end
		end

	set_exception_was_raised is
		do
			exception_was_raised := True
		end

	compare_not_needed is
		do
			needs_comparison := False
		end

	frame_description (a_frame_type_id: INTEGER): STRING is
		do
			Result := all_frame_descriptions.item (a_frame_type_id)
		end

	--	fix_streams (a_frame_data: ARRAY[ANY]) is
	--	-- Find all items of type STREAM and replace them with MOVIE_STREAM.
	--	local
	--		i: INTEGER
	--		l_stream: STREAM
	--		l_array: ARRAY[ANY]
	--		l_movie_stream: MOVIE_STREAM
	--	do
	--		if a_frame_data /= Void
	--		then
	--			from
	--				i := a_frame_data.lower
	--			until
	--				i>a_frame_data.upper
	--			loop
	--				l_stream ?= a_frame_data.item (i)
	--				if l_stream /= Void
	--				then
	--					create l_movie_stream.make (l_stream)
	--					a_frame_data.put (l_movie_stream, i)
	--				else
	--					l_array ?= a_frame_data.item (1)
	--					if l_array /= Void
	--					then
	--						fix_streams (l_array)		-- Ooooh, recursion!
	--					end
	--				end
	--	
	--				i := i + 1
	--			end
	--		end
	--	end

feature {NONE} -- Implementation

	next_rose_date_frame: ANY is
		local
			l_movie_frame: MOVIE_FRAME
		do
			rose_date_frame_number := rose_date_frame_number + 1
			l_movie_frame := next_frame (-1, rose_date_frame_number)
		end

	record_rose_date_movie_frame (a_frame_desc: STRING; a_rose_date: ROSE_DATE) is
		do
			rose_date_frame_number := rose_date_frame_number + 1
			record_movie_frame (-1, rose_date_frame_number, a_frame_desc, a_rose_date)
		end

feature {ENTITY_ACCESSORS} -- Implementation

	checkpoint is
			-- Checkpoint every camera in this movie.
		do
			checkpoint_depth := checkpoint_depth + 1
			from
				cameras.start
			until
				cameras.off
			loop
				cameras.current_item.checkpoint
				cameras.forth
			end
		end

	clear_checkpoint is
			-- We didn't need the checkpoints so clear them.
		do
			from
				cameras.start
			until
				cameras.off
			loop
				cameras.current_item.clear_checkpoint
				cameras.forth
			end

			checkpoint_depth := checkpoint_depth - 1
		end

	restore_from_checkpoint is
			-- Restore each camera to it's last checkpoint.
		do
			from
				cameras.start
			until
				cameras.off
			loop
				cameras.current_item.restore_from_checkpoint
				cameras.forth
			end
		end

feature {NONE} -- Implementation
	init is
		local
			l_pos: INTEGER
		do
			if objects_instance_of (Current).count /= 1 then
				raise ("Too many instances of MOVIE")
			end

			if not init_called then
				init_called := True

				movie_variant := ree.get ("movie_variant")
				if movie_variant = Void then
					movie_variant := ""
				end

				exe := ree.get ("movie_exe_name")
				if exe = Void or else exe.count = 0 then
					exe := ree.executable_path.twin
					l_pos := exe.last_index_of ('\', exe.count)
					if l_pos /= 0 then
						exe := exe.substring (l_pos + 1, exe.count)
					end
				end

				print ("Movie name  = " + exe + "%N")
				create frame_container.make_empty

				use_sed := False
				if use_sed then
					create sed.make
				else
					create stream
				end
			end
		end

	load_frame_types is
			-- Load frame types from database and
			-- populate `frame_types'
		local
			l_container: ROSE_LINEAR_ARRAY[TUPLE[INTEGER, STRING]]
			l_string: STRING
		do
			create l_container.make_empty
			frame_types_predicate.set_container (l_container)
			frame_types_predicate.retrieve (Void, Void)
			if not frame_types_predicate.success then
				raise ("frame_types_predicate failed")
			end

			create frame_types.make (l_container.count)

			from
				l_container.start
			until
				l_container.off
			loop
				l_string ?= l_container.current_item.reference_item (2)
				frame_types.put (l_container.current_item.integer_item (1), l_string)
				l_container.forth
			end
		end

	next_frame_storable_predicate (a_camera_id, a_frame_number, a_object_size: INTEGER; a_column_name: STRING): DATA_SERVER_STREAM_COMMAND is
		do
			Result := server.stream_command (
											"set textsize " + a_object_size.out + "%N%
																				  %select " + a_column_name + " " + next_frame_partial_statement (a_camera_id, a_frame_number))
																				  end

																				  next_frame_partial_statement (a_camera_id, a_frame_number: INTEGER): STRING is
		do
			Result :=
				"from movie_frames %
			%where movie_id = " + movie_id.out + " and camera_id = " + a_camera_id.out+ " and frame_number = " + a_frame_number.out
		end

	create_server is
			-- Connect to the movie server
		do
			if server = Void then
				create server.make_integrated_security ("RCSQL5\x64devel01", "movie")
				if not server.make_succeeded then
					raise ("Failed to make the movie server")
				end

				-- Since the program can exit at any time, we need to make sure that each
				-- write goes to the database when we tell it to.
				server.set_implicit_transaction_off
			end
		end

	clear_state is
		require
			record_movie_saved or playback_movie_saved
		do
			record_movie := False
			playback_movie := False
		end

	restore_state is
		require
			not record_movie and not playback_movie
		do
			record_movie := record_movie_saved
			playback_movie := playback_movie_saved
		end

feature {MOVIE_AWARE} -- Implementation
	stop is
		do
			clear_state
		end

feature {NONE}	-- Once routines.

	ree: ROSE_EXECUTION_ENVIRONMENT is
		once
			create Result
		end

	insert_movie_predicate: EXECUTION_PREDICATE is
		once
			create Result.make (server, "insert into movies (executable_name, variant) values (?, ?)")
			if not Result.success then
				raise ("Failed to make insert_movie_predicate")
			end
		end

	get_movie_id_predicate: INTEGER_PREDICATE is
		once
			create Result.make (server,
							   "select top 1 movie_id from movies where executable_name = ? and variant = ? order by time desc")
			if not Result.success then
				raise ("Failed to make get_movie_id_predicate")
			end
		end

	max_non_assert_sequence_predicate: INTEGER_PREDICATE is
		once
			create Result.make (server,
							   "select max (sequence) from movie_frames where movie_id = ? and in_assertion  = 0")
			if not Result.success then
				raise ("Failed to make max_non_assert_sequence_predicate")
			end
		end

	--	camera_id_predicate: INTEGER_PREDICATE is
	--	once
	--		create Result.make (server, "select camera_id from cameras where movie_id = ? and name = ?")
	--		if not Result.success
	--		then
	--			raise ("Failed to make camera_id_predicate")
	--		end
	--	end

	camera_name_predicate: STRING_PREDICATE is
		once
			create Result.make (server, "select name from cameras where movie_id = ? and camera_id = ?")
			if not Result.success then
				raise ("Failed to make camera_name_predicate")
			end
		end

	next_frame_predicate: COLLECTING_PREDICATE[MOVIE_FRAME] is
		once
			create Result.make (
							   server,
							   "select camera_id, frame_type_id, sequence, frame_number, time, in_assertion, is_sf as sf, datalength ([frame_data]) as frame_size, datalength ([playback_frame_data]) as playback_frame_size %
							   %from movie_frames %
							   %where movie_id = ? and camera_id = ? and frame_number = ?")
								   if not Result.success
									then
									   raise ("Failed to make next_frame_predicate")
								   end
							   end

							   frame_types_predicate: TUPLE_PREDICATE[TUPLE[INTEGER, STRING]] is
		once
			create Result.make (server, "select * from frame_types")
			if not Result.success then
				raise ("Failed to make frame_types_predicate")
			end
		end

	make_new_frame_type_predicate: EXECUTION_PREDICATE is
		once
			create Result.make (server, "insert into frame_types (description) values (?)")
			if not Result.success then
				raise ("Failed to make make_new_frame_type_predicate")
			end
		end

	record_frame_predicate: EXECUTION_PREDICATE is
		once
			create Result.make (server, "insert into movie_frames values (?, ?, ?, ?, ?, ?, ?, ?, ?, ?)")
			if not Result.success then
				raise ("Failed to make record_frame_predicate")
			end
		end

	rerecord_frame_predicate: EXECUTION_PREDICATE is
		once
			create Result.make (server, "update movie_frames set frame_data = ? where movie_id = ? and camera_id = ? and frame_number = ?")
			if not Result.success then
				raise ("Failed to make rerecord_frame_predicate")
			end
		end

	update_playback_frame_predicate: EXECUTION_PREDICATE is
		once
			create Result.make (server, "update movie_frames set playback_frame_data = ? where movie_id = ? and camera_id = ? and frame_number = ?")
			if not Result.success then
				raise ("Failed to make update_frame_predicate")
			end
		end

	register_camera_predicate: EXECUTION_PREDICATE is
		once
			create Result.make (server, "insert into cameras values (?, ?, ?)")
			if not Result.success then
				raise ("Failed to make register_camera_predicate")
			end
		end

	all_frame_descriptions_predicate: TUPLE_PREDICATE[TUPLE[INTEGER, STRING]] is
			-- SQL predicate to retrieve all frame type
		once
			create Result.make (server, "select * from frame_types")
			if not Result.success then
				raise ("Failed to make all_frame_descriptions_predicate")
			end
		end

	all_frame_descriptions: HASH_TABLE[STRING, INTEGER] is
		local
			l_container: ROSE_LINEAR_ARRAY[TUPLE[INTEGER, STRING]]
			l_string: STRING
		once
			clear_state
			create l_container.make_empty
			all_frame_descriptions_predicate.set_container (l_container)
			all_frame_descriptions_predicate.retrieve (Void, Void)
			if not all_frame_descriptions_predicate.success then
				raise ("The all_frame_descriptions_predicate failed")
			end

			create Result.make (l_container.count)
			from
				l_container.start
			until
				l_container.off
			loop
				l_string ?= l_container.current_item.reference_item (2)
				Result.put (l_string, l_container.current_item.integer_item (1))
				l_container.forth
			end

			restore_state
		end

feature {MOVIE_CAMERA, MOVIE_AWARE} -- Implementation

	playback_movie: BOOLEAN

	record_movie: BOOLEAN

feature {MOVIE_CAMERA, MOVIE_AWARE} -- Implementation

	error_count: INTEGER

	exception_was_raised: BOOLEAN

	max_non_assert_sequence: INTEGER

	needs_comparison: BOOLEAN

	superframe_depth: INTEGER

	raise_if_after_for_sockets: BOOLEAN

feature {NONE} -- Implementation

	cameras: ROSE_LINEAR_ARRAY[MOVIE_CAMERA]

	checkpoint_depth: INTEGER

	exe: STRING

	exit_on_next_record: BOOLEAN

	frame_container: ROSE_LINEAR_ARRAY[MOVIE_FRAME]

	frame_types: HASH_TABLE[INTEGER, STRING]

	init_called: BOOLEAN

	last_frame_read: MOVIE_FRAME

	max_camera_name_length: INTEGER is 1000		-- This is the max size of the varchar in the table 'cameras'.

	movie_id: INTEGER

	movie_variant: STRING

	next_camera_id: INTEGER

	playback_frame_size_invalid: BOOLEAN

	playback_movie_saved: BOOLEAN

	request_raise_if_after: BOOLEAN

	rerecord: BOOLEAN

	rose_date_frame_number: INTEGER

	sed: ROSE_SED_MEMORY_READER_WRITER

	sequence: INTEGER

	server: SQLOLEDB_DATA_SERVER

	stream: ROSE_STREAM

	superframe_count: INTEGER

	superframe_item_1: MOVIE_FRAME

	superframe_streams: ROSE_LINEAR_ARRAY[ROSE_STREAM]

	superstream: ROSE_STREAM

	superstream_pos: INTEGER

	record_movie_saved: BOOLEAN

	use_sed: BOOLEAN

invariant

	valid_superframe: superframe_depth >= 0

end
