indexing
	description: "Movie camera - designed to allow for recording and playback of a system's IO"
	author: "RJ"
	date: "$Date: 2009/04/03 $"
	revision: "$Revision: #6 $"
	archive: "$File: //rose/source/kernel/infrastructure/movies/movie_camera.e $"

class
	MOVIE_CAMERA

inherit
	MOVIE_AWARE
	INTERNAL
		export
			{NONE} all
		end

create {MOVIE_CAMERA_AWARE, MOVIE_CAMERA, TS_TEST_CASE}
	make,
	fake_camera

feature {MOVIE_CAMERA_AWARE, TS_TEST_CASE} -- Implementation

	fake_camera is
			-- This is for MOVIE_SYSTEM just so we can get 'movie'.
		do
		end

	make (a_name: STRING) is
		do
			create current_frame_number_stack.make (0)
			if playback_movie then
				find_camera(a_name)
			elseif record_movie then
				new_camera(a_name)
			else
				do_nothing
				-- I guess that he has 'make_movies' turned on but is not playing or recording.
				-- This happens when a database is created for MOVIE.
			end
		end

	new_camera (a_name: STRING) is
		require
			not playback_movie
			record_movie
		do
			debug("make_movies")
				camera_name := a_name
				camera_id := movie.new_camera_id (Current)
			end
		end

	find_camera (a_name: STRING) is
		require
			not record_movie
			playback_movie
		local
			l_class_name: STRING
			l_pos: INTEGER
		do
			debug("make_movies")
				camera_name := a_name
				camera_id := movie.camera_id(Current)
				l_pos := camera_name.index_of('/', 2)
				if l_pos /= 0 then
					l_class_name := camera_name.substring(1, l_pos - 1)
					if --movie.raise_if_after_for_sockets and
					   type_conforms_to(dynamic_type_from_string(l_class_name), socket_type)
					then
						raise_if_after := True
					end
				end
			end
		end

	next_frame(a_frame_desc: STRING): MOVIE_FRAME is
	-- Advances to the next frame.
	require
		playback_movie
	local
		l_skip: BOOLEAN
		l_frame_desc: STRING
	do
		debug("make_movies")
			from
				check_for_missing_comparison
				l_skip := True
			until
				not l_skip
			loop
				movie.set_raise_if_after(raise_if_after)
				after := True
				Result := movie.next_frame(camera_id, current_frame_number)
				after := False		-- If an exception is raised then we won't get here.
				movie.set_raise_if_after(False)
				current_frame_number := current_frame_number + 1
				if not Result.in_assertion
				then
					l_skip := False
				else
					if movie.in_assertion
					then
						-- We can skip frames while in an assertion is the description doesn't match.
						l_skip := not equal(a_frame_desc, movie.frame_description(Result.frame_type_id))
					end
				end

				if l_skip
				then
					compare_not_needed
				end
			end
		end

		l_frame_desc := movie.frame_description(Result.frame_type_id)
		if not equal(a_frame_desc, l_frame_desc)
		then
			if l_frame_desc=Void
			then
				l_frame_desc := "Unknown=" + Result.frame_type_id.out
			end

			movie.log_error(camera_id, current_frame_number, "Wrong frame type.  Expecting '" + a_frame_desc + "', found '" + l_frame_desc + "'")
		end
	end

	record_movie_frame(a_frame_desc: STRING; a_frame_data: ANY) is
	require
		record_movie
	do
		debug("make_movies")
			current_frame_number := current_frame_number + 1
			movie.record_movie_frame(camera_id, current_frame_number, a_frame_desc, a_frame_data)
		end
	end

	rerecord_movie_frame(a_frame_desc: STRING; a_frame_data: ANY; a_frame_number: INTEGER) is
	require
		record_movie
	do
		debug("make_movies")
			movie.rerecord_movie_frame(camera_id, a_frame_number, a_frame_desc, a_frame_data)
		end
	end

--	record_movie_frames(a_frame_desc: STRING; a_frame_data: ARRAY[ANY]) is
--	require
--		record_movie
--	do
--		debug("make_movies")
--			current_frame_number := current_frame_number + 1
--			movie.record_movie_frames(camera_id, current_frame_number, a_frame_desc, a_frame_data)
--		end
--	end

	record_boolean_movie_frame(a_frame_desc: STRING; a_frame_data: BOOLEAN) is
	require
		record_movie
	local
		l_cell: CELL[BOOLEAN]
	do
		debug("make_movies")
			current_frame_number := current_frame_number + 1
			create l_cell.put(a_frame_data)
			movie.record_movie_frame(camera_id, current_frame_number, a_frame_desc, l_cell)
		end
	end

	record_integer_movie_frame(a_frame_desc: STRING; a_frame_data: INTEGER) is
	require
		record_movie
	local
		l_cell: CELL[INTEGER]
	do
		debug("make_movies")
			current_frame_number := current_frame_number + 1
			create l_cell.put(a_frame_data)
			movie.record_movie_frame(camera_id, current_frame_number, a_frame_desc, l_cell)
		end
	end

	record_real_movie_frame(a_frame_desc: STRING; a_frame_data: REAL) is
	require
		record_movie
	local
		l_cell: CELL[REAL]
	do
		debug("make_movies")
			current_frame_number := current_frame_number + 1
			create l_cell.put(a_frame_data)
			movie.record_movie_frame(camera_id, current_frame_number, a_frame_desc, l_cell)
		end
	end

	record_double_movie_frame(a_frame_desc: STRING; a_frame_data: DOUBLE) is
	require
		record_movie
	local
		l_cell: CELL[DOUBLE]
	do
		debug("make_movies")
			current_frame_number := current_frame_number + 1
			create l_cell.put(a_frame_data)
			movie.record_movie_frame(camera_id, current_frame_number, a_frame_desc, l_cell)
		end
	end

	record_character_movie_frame(a_frame_desc: STRING; a_frame_data: CHARACTER) is
	require
		record_movie
	local
		l_cell: CELL[CHARACTER]
	do
		debug("make_movies")
			current_frame_number := current_frame_number + 1
			create l_cell.put(a_frame_data)
			movie.record_movie_frame(camera_id, current_frame_number, a_frame_desc, l_cell)
		end
	end

	compare_not_needed is
	require
		playback_movie
	do
		debug("make_movies")
			movie.compare_not_needed
		end
	end

	compare_movie_frame(a_frame_desc: STRING; a_frame_data: ANY) is
	-- Advances to the next frame and compares.
	require
		playback_movie
	local
		l_any: ANY
	do
		debug("make_movies")
			l_any := next_frame(a_frame_desc).frame_data
			compare_not_needed
			if not deep_equal(a_frame_data, l_any)
			then
				movie.log_bad_match_error_with_data(Current, "Bad match", a_frame_data)
			end
		end
	end

	compare_boolean_movie_frame(a_frame_desc: STRING; a_frame_data: BOOLEAN) is
	require
		playback_movie
	local
		l_data: BOOLEAN
	do
		debug("make_movies")
			l_data := next_frame(a_frame_desc).frame_data_as_boolean
			compare_not_needed
			if a_frame_data/=l_data
			then
				movie.log_bad_match_error(Current,
					"Bad match, expecting " + l_data.out + ", found " + a_frame_data.out)
			end
		end
	end

	compare_real_movie_frame(a_frame_desc: STRING; a_frame_data: REAL) is
	require
		playback_movie
	local
		l_data: REAL
	do
		debug("make_movies")
			l_data := next_frame(a_frame_desc).frame_data_as_real
			compare_not_needed
			if a_frame_data/=l_data
			then
				movie.log_bad_match_error(Current,
					"Bad match, movie says " + l_data.out + ", found " + a_frame_data.out)
			end
		end
	end

	compare_double_movie_frame(a_frame_desc: STRING; a_frame_data: DOUBLE) is
	require
		playback_movie
	local
		l_data: DOUBLE
	do
		debug("make_movies")
			l_data := next_frame(a_frame_desc).frame_data_as_double
			compare_not_needed
			if a_frame_data/=l_data
			then
				movie.log_bad_match_error(Current,
					"Bad match, movie says " + l_data.out + ", found " + a_frame_data.out)
			end
		end
	end

	compare_integer_movie_frame(a_frame_desc: STRING; a_frame_data: INTEGER) is
	require
		playback_movie
	local
		l_data: INTEGER
	do
		debug("make_movies")
			l_data := next_frame(a_frame_desc).frame_data_as_integer
			compare_not_needed
			if a_frame_data/=l_data
			then
				movie.log_bad_match_error(Current,
					"Bad match, movie says " + l_data.out + ", found " + a_frame_data.out)
			end
		end
	end

	compare_character_movie_frame(a_frame_desc: STRING; a_frame_data: CHARACTER) is
	require
		playback_movie
	local
		l_data: CHARACTER
	do
		debug("make_movies")
			l_data := next_frame(a_frame_desc).frame_data_as_character
			compare_not_needed
			if a_frame_data/=l_data
			then
				movie.log_bad_match_error(Current,
					"Bad match, movie says " + l_data.out + ", found " + a_frame_data.out)
			end
		end
	end

--	compare_partial_movie_frames(a_part1, a_unfixed_part2: ARRAY[ANY]) is
--	do
--		debug("make_movies")
--			movie.fix_streams(a_unfixed_part2)
--			compare_partial_movie_frame(a_part1, a_unfixed_part2)
--		end
--	end

	compare_partial_movie_frame(a_part1, a_part2: ANY) is
	require
		playback_movie
	do
		debug("make_movies")
			compare_not_needed
			if not deep_equal(a_part1, a_part2)
			then
				movie.log_bad_match_error_with_data(Current, "Bad match", [a_part1, a_part2])
			end
		end
	end

	start_superframe is
	do
		movie.start_superframe
	end

	end_superframe is
	do
		movie.end_superframe
	end

	exception_was_raised: BOOLEAN is
	do
		Result := movie.exception_was_raised
	end

	safe_deep_clone(a_frame_data: ANY): ANY is
	do
		Result := movie.safe_deep_clone(a_frame_data)
	end

	playback_movie: BOOLEAN is
	local
		l_movie: MOVIE
	do
		-- This is very fast when 'make_movies' is off and you use 'inlining('0')'.
		-- Between inlining and optimization, all of the code is removed.
		-- For instance this:
		-- 		if playback
		--		then
		--			statement_1
		--		else
		--			statement_2
		--		end
		-- is turned into this:
		--		statement_2
		-- The RTHOOK's are usually optimized away.
		debug("make_movies")
			l_movie := movie
			if l_movie/=Void
			then
				Result := l_movie.playback_movie
			else
				-- When 'movie' is still being created, 'l_movie' is Void.
			end
		end
	end

	record_movie: BOOLEAN is
	local
		l_movie: MOVIE
	do
		debug("make_movies")
			l_movie := movie
			if l_movie/=Void
			then
				Result := l_movie.record_movie
			else
				-- When 'movie' is still being created, 'l_movie' is Void.
			end
		end
	end

	check_for_missing_comparison is
	do
		if needs_comparison
		then
			mc_exceptions.raise("A frame was not compared")
		end		
	end

	raise_exception(a_tag: STRING) is
		-- Raise exception and record it
	do
		movie.set_exception_was_raised
		mc_exceptions.raise(a_tag)
	end

	needs_comparison: BOOLEAN is
	do
		Result := movie.needs_comparison
	end

	error_count: INTEGER is
	do
		Result := movie.error_count
	end

feature {MOVIE} -- Implementation

	checkpoint is
	-- Checkpoint this camera.
	do
		current_frame_number_stack.extend(current_frame_number)
	end

	clear_checkpoint is
	-- We didn't need the checkpoint so clear it.
	require
		stack_not_empty: not current_frame_number_stack.is_empty
	do
		current_frame_number_stack.remove
	end

	restore_from_checkpoint is
	-- Restore this camera to it's last checkpoint.
	require
		stack_not_empty: not current_frame_number_stack.is_empty
	do
		current_frame_number := current_frame_number_stack.item
	end

feature {NONE} -- Implementation

	mc_exceptions: EXCEPTIONS is
	-- We need our very own copy of EXCEPTIONS.  Some programs (like ROSE_DEDICATED_NETWORK_SERVER)
	-- get it from ROSE_ERROR_HANDLER and we don't want that to happen here.
	once
		create Result
	end

	socket_type: INTEGER is
	do
		Result := dynamic_type_from_string("SOCKET")
	end

feature -- Access

	camera_name: STRING

feature {MOVIE, MOVIE_CAMERA_AWARE} -- Implementation

	after: BOOLEAN

	camera_id: INTEGER

	current_frame_number: INTEGER

	current_frame_number_stack: ARRAYED_STACK[INTEGER]

	raise_if_after: BOOLEAN

end
