class
	NOTIFIER

feature -- Access

	mail: ROSE_MAIL is
		once
			create Result.make
		end

	notify_list: STRING

	log_file: ROSE_FILE

feature -- Basic operations

	set_log_file (a_value: like log_file) is
		do
			log_file := a_value
		end

	set_notify_list (a_list: STRING) is
		do
			if a_list /= Void then
				notify_list := a_list.twin
			else
				notify_list := Void
			end
		end

	add_to_notify_list (a_recipient: STRING) is
		require
			a_recipient /= Void
		do
			if notify_list = Void then
				create notify_list.make_empty
			end
			if not notify_list.is_empty then
				notify_list.append_character (',')
			end
			notify_list.append_string (a_recipient)
		end

	remove_from_notify_list (str: STRING) is
		do
			notify_list.replace_substring_all (str, "")
			notify_list.replace_substring_all (",,", ",")
		end

	notify (a_subject: STRING; a_message: STRING) is
		local
			l_content: ROSE_MAIL_CONTENT
		do
			if notify_list /= Void then
				create l_content.make
				l_content.to.set (notify_list)
				l_content.set_subject (a_subject)
				l_content.set_text (a_message)
				mail.send (l_content)
			end
			log (a_subject)	
			log (a_message)				
		end

	open_log (a_path: STRING) is
		local
			retried: BOOLEAN
		do
			if not retried and then a_path /= Void and then not a_path.is_empty then
				create log_file.make_open_write (a_path)
			else
				log_file := Void
			end
		rescue
			if not retried then
				retried := True
				retry
			end
		end

	log (str: STRING) is
		do
			if str /=void and log_file /= Void and log_file.file_writable then
				log_file.put_string (str)
				log_file.put_character ('%N')
				log_file.flush
			else
				print (str)
				print ("%N")
			end
		end

	close_log is
		do
			if log_file /= Void and then not log_file.is_closed then
				log_file.close
			end
		end

end
