note

	description: "NT_NON_CASH"

	generator: "[
		Classifier -class_name="NT_NON_CASH"
	]"

	keywords: "classifier,database,record,generated"
	author: "Sami Ben Grine"
	last_editor: "$Author: sm $"
	date: "$Date: 2013/06/26 $"
	revision: "$Revision: #1 $"
	path: "$File: //rose/source/kernel/expert_system/nt_non_cash.e $"

class NT_NON_CASH

inherit

	IEEE_MATH
		export {NONE} all end

	ANY

feature -- Access

	global_investment_date : ROSE_DATE
			-- Database column `global_investment_date'

	update_mode : STRING
			-- Database column `update_mode'

	account_code : STRING
			-- Database column `account_code'

	total_quantity : RP_DECIMAL
			-- Database column `total_quantity'

	errors : STRING
			-- Database column `errors'

	quantity_unavailable : RP_DECIMAL
			-- Database column `quantity_unavailable'

	account_currency : STRING
			-- Database column `account_currency'

	cost_account_base : RP_DECIMAL
			-- Database column `cost_account_base'

	cost_global_base : RP_DECIMAL
			-- Database column `cost_global_base'

	cost_local : RP_DECIMAL
			-- Database column `cost_local'

	brimsym : STRING
			-- Database column `brimsym'

	global_currency : STRING
			-- Database column `global_currency'

	local_currency : STRING
			-- Database column `local_currency'

	market_value_global_base : RP_DECIMAL
			-- Database column `market_value_global_base'

	market_value_local : RP_DECIMAL
			-- Database column `market_value_local'

	ref_uid : INTEGER_64
			-- Database column `ref_uid'

	accounting_date : ROSE_DATE
			-- Database column `accounting_date'

	contract_side : STRING
			-- Database column `contract_side'

	accounting_basis : STRING
			-- Database column `accounting_basis'

	fm_sec_id : STRING
			-- Database column `fm_sec_id'

feature -- Setting

	set_global_investment_date (a_value: like global_investment_date) is
			-- Set `global_investment_date' to `a_value'
		do
			global_investment_date := a_value
		ensure
			global_investment_date_assigned: global_investment_date = a_value
		end

	set_update_mode (a_value: like update_mode) is
			-- Set `update_mode' to `a_value'
		do
			update_mode := a_value
		ensure
			update_mode_assigned: update_mode = a_value
		end

	set_account_code (a_value: like account_code) is
			-- Set `account_code' to `a_value'
		do
			account_code := a_value
		ensure
			account_code_assigned: account_code = a_value
		end

	set_total_quantity (a_value: like total_quantity) is
			-- Set `total_quantity' to `a_value'
		do
			total_quantity := a_value
		ensure
			total_quantity_assigned_non_nan: not isnan (a_value) implies total_quantity = a_value
			total_quantity_assigned_nan: isnan (a_value) implies isnan (total_quantity)
		end

	set_errors (a_value: like errors) is
			-- Set `errors' to `a_value'
		do
			errors := a_value
		ensure
			errors_assigned: errors = a_value
		end

	set_quantity_unavailable (a_value: like quantity_unavailable) is
			-- Set `quantity_unavailable' to `a_value'
		do
			quantity_unavailable := a_value
		ensure
			quantity_unavailable_assigned_non_nan: not isnan (a_value) implies quantity_unavailable = a_value
			quantity_unavailable_assigned_nan: isnan (a_value) implies isnan (quantity_unavailable)
		end

	set_account_currency (a_value: like account_currency) is
			-- Set `account_currency' to `a_value'
		do
			account_currency := a_value
		ensure
			account_currency_assigned: account_currency = a_value
		end

	set_cost_account_base (a_value: like cost_account_base) is
			-- Set `cost_account_base' to `a_value'
		do
			cost_account_base := a_value
		ensure
			cost_account_base_assigned_non_nan: not isnan (a_value) implies cost_account_base = a_value
			cost_account_base_assigned_nan: isnan (a_value) implies isnan (cost_account_base)
		end

	set_cost_global_base (a_value: like cost_global_base) is
			-- Set `cost_global_base' to `a_value'
		do
			cost_global_base := a_value
		ensure
			cost_global_base_assigned_non_nan: not isnan (a_value) implies cost_global_base = a_value
			cost_global_base_assigned_nan: isnan (a_value) implies isnan (cost_global_base)
		end

	set_cost_local (a_value: like cost_local) is
			-- Set `cost_local' to `a_value'
		do
			cost_local := a_value
		ensure
			cost_local_assigned_non_nan: not isnan (a_value) implies cost_local = a_value
			cost_local_assigned_nan: isnan (a_value) implies isnan (cost_local)
		end

	set_brimsym (a_value: like brimsym) is
			-- Set `brimsym' to `a_value'
		do
			brimsym := a_value
		ensure
			brimsym_assigned: brimsym = a_value
		end

	set_global_currency (a_value: like global_currency) is
			-- Set `global_currency' to `a_value'
		do
			global_currency := a_value
		ensure
			global_currency_assigned: global_currency = a_value
		end

	set_local_currency (a_value: like local_currency) is
			-- Set `local_currency' to `a_value'
		do
			local_currency := a_value
		ensure
			local_currency_assigned: local_currency = a_value
		end

	set_market_value_global_base (a_value: like market_value_global_base) is
			-- Set `market_value_global_base' to `a_value'
		do
			market_value_global_base := a_value
		ensure
			market_value_global_base_assigned_non_nan: not isnan (a_value) implies market_value_global_base = a_value
			market_value_global_base_assigned_nan: isnan (a_value) implies isnan (market_value_global_base)
		end

	set_market_value_local (a_value: like market_value_local) is
			-- Set `market_value_local' to `a_value'
		do
			market_value_local := a_value
		ensure
			market_value_local_assigned_non_nan: not isnan (a_value) implies market_value_local = a_value
			market_value_local_assigned_nan: isnan (a_value) implies isnan (market_value_local)
		end

	set_ref_uid (a_value: like ref_uid) is
			-- Set `ref_uid' to `a_value'
		do
			ref_uid := a_value
		ensure
			ref_uid_assigned: ref_uid = a_value
		end

	set_accounting_date (a_value: like accounting_date) is
			-- Set `accounting_date' to `a_value'
		do
			accounting_date := a_value
		ensure
			accounting_date_assigned: accounting_date = a_value
		end

	set_contract_side (a_value: like contract_side) is
			-- Set `contract_side' to `a_value'
		do
			contract_side := a_value
		ensure
			contract_side_assigned: contract_side = a_value
		end

	set_accounting_basis (a_value: like accounting_basis) is
			-- Set `accounting_basis' to `a_value'
		do
			accounting_basis := a_value
		ensure
			accounting_basis_assigned: accounting_basis = a_value
		end

	set_fm_sec_id (a_value: like fm_sec_id) is
			-- Set `fm_sec_id' to `a_value'
		do
			fm_sec_id := a_value
		ensure
			fm_sec_id_assigned: fm_sec_id = a_value
		end

end
