note
	description: "OLEDB data server for SQL server"
	last_updater: "$Author: an $"
	author: "Pierre Nallet"
	last_editor: "$Author: an $"
	date: "$Date: 2013/10/18 $"
	revision: "$Revision: #74 $"
	path: "$File: //rose/source/kernel/infrastructure/persistence/windows/classic/oledb_any_data_server.e $"

deferred class
	OLEDB_ANY_DATA_SERVER

inherit
	SQL_DATA_SERVER
		rename
			make as old_make,
			set_status as old_set_status
		undefine
			begin_transaction,
			set_implicit_transaction_off,
			set_implicit_transaction_on,
			transaction_depth,
			implicit_transaction,
			explicit_transaction_depth,
			implicit_transactions_set,
			set_implicit_transactions_set
		redefine
			setup_sqlda, connected, new_executor,
			error_handler,
			new_error_handler_print,
			commit_if_necessary,
			is_error_negligible,
			allow_error,
			dont_allow_error,
			error_was_allowed,
			clear_last_allowed_error,
			statement_is_feature,
			tables_are_equal,
			has_table,
			drop_table,
			create_duplicate_table,
			column_names,
			has_column_name,
			has_trigger,
			trigger_creation_date,
			auxiliary_server
		end
	OLEDB_ANY_DATA_SERVER_COMMON
		rename
			transaction_depth_predicate as oadsc_transaction_depth_predicate,
			transaction_depth_predicate_saved as oadsc_transaction_depth_predicate_saved
		redefine
			transaction_depth
		end
	COM_RUN_TIME
		export
			{NONE} all
		end
	OLEDB_CONSTANTS
		export
			{NONE} all
		end
	OLEDB_ISOLATIONLEVEL_CONSTANTS
		export
			{NONE} all
		end
	SQL_SCRIPT_CAPTURE
	ACCESS_EXTERNALS
		export
			{NONE} all
		end

feature -- Access
	session: OLEDB_SESSION
			-- session
	error_handler: OLEDB_ERROR_HANDLER
			-- error handler
	default_command_propset: OLEDB_PROPERTY_SET
			-- default property set

	saved_auxiliary_server: SQLOLEDB_DATA_SERVER
			-- See 'auxiliary_server'.

feature -- Status
	eof: BOOLEAN is
			-- Are we at the end of the rowset?
		do
			Result := status = db_s_endofrowset
		end

	connected: BOOLEAN is
			-- Is this object connected to a valid session?
		do
			Result := (session /= Void and then session.exists) or playback_movie
		end

	statement_is_feature (a_statement: STRING): BOOLEAN is
			-- There are certain statments that we have built-in features for.  Don't call 'sql_execute_immediate'
			-- when there is a built-in feature.
		local
			l_statement: STRING
		do
			if not disable_statement_is_feature then
				l_statement := a_statement.twin
				l_statement.prune_all (' ')
				Result :=
					 (l_statement.count>=6 and then l_statement.substring (1, 6).is_case_insensitive_equal ("commit")) or
					 (l_statement.count>=8 and then l_statement.substring (1, 8).is_case_insensitive_equal ("rollback")) or
					 (l_statement.count>=10 and then l_statement.substring (1, 10).is_case_insensitive_equal ("begintrans")) or
					 (l_statement.count>=11 and then l_statement.substring (1, 11).is_case_insensitive_equal ("setimplicit"))
			end
		end

	is_error_negligible (a_rose_sql_error_code: INTEGER): BOOLEAN is
			-- Do we ignore the this error?
		do
			Result := error_handler /= Void and then
			error_handler.error_is_negligible (oledb_error_code_from_rose_sql_error_code (a_rose_sql_error_code))
		end

	transaction_depth: INTEGER
		-- Returns the true depth of the current set of transactions.  This version has a completely different implementation
		-- from the Precursor.
	local
		l_binary_output: STRING
		l_any: ANY
	do
		transaction_depth_predicate.retrieve(Void)
		-- This predicate is a DATA_SERVER_STREAM_COMMAND which doesn't has a 'success' feature.  See below.

		l_binary_output := transaction_depth_predicate.query_result
		if l_binary_output.count/=4
		then
			-- If an error occurs the output is the error message.
			print(l_binary_output + "%N")
			error.raise(node + "." + database + " - The transaction_depth_predicate failed.")
		end

		l_any := l_binary_output.area
		Result := dereference_integer($l_any)
	end

feature -- Child object creation

	new_selector (a_statement: STRING; a_parameter_template: ARRAY[STRING]): OLEDB_SELECTOR is
			-- The OLEDB_SELECTOR is the embodiment of a SELECT statement.  It performs the actual work.
		local
			l_retry: BOOLEAN
		do
			if not playback_movie then
				if not l_retry then
					create Result.make (Current, a_statement, a_parameter_template)
				else
					 error.raise (node + "." + database + " - Could not create selector, check database or repository entry")
				end
			else
				-- Selectors (facilitators) are always Void during playback.
			end
		rescue
			if not l_retry then
				l_retry := True
				retry
			end
		end

	new_executor (a_statement: STRING; a_parameter_template: ARRAY[STRING]): OLEDB_EXECUTOR
			-- For use by EXECUTION_PREDICATE.
		local
			l_retry: BOOLEAN
		do
			if not playback_movie then
				if not l_retry then
					create Result.make (Current, a_statement, a_parameter_template)
				else
					 error.raise (node + "." + database + " - Could not create executor, check database or repository entry")
				end
			else
				-- Executors (facilitators) are always Void during playback.
			end
		rescue
			if not l_retry then
				l_retry := True
				retry
			end
		end

	new_sqlvar (a_sqlda: OLEDB_SQLDA; a_index: INTEGER): OLEDB_SQLVAR is
			--create a new sqlvar
		require else
			a_sqlda_not_void: a_sqlda/=Void
		do
			if a_sqlda.is_parameter then
				create {OLEDB_PARAMETER_SQLVAR} Result.make (a_sqlda, a_index)
			else
				create {OLEDB_COLUMN_SQLVAR} Result.make (a_sqlda, a_index)
			end
		end

	new_error_handler_print: OLEDB_ERROR_HANDLER_PRINT is
			-- Returns an error handler suitable to pass to 'set_error_handler'.
		do
			create Result
		ensure then
			result_non_void: Result /= Void
		end


feature -- Connection handling

	release_connection is
			-- Release the OLEDB resources.
		do
			if session /= Void then
				session.release
			end

			if saved_auxiliary_server/=Void
			then
				saved_auxiliary_server.release_connection
			end
		end

	sql_disconnect
			-- Cleanup during a disconnect.
		do
			session := void
		end

	auxiliary_server: like saved_auxiliary_server is
			-- A server to use when you don't want to use the current connection.
			-- The use of this server is for little things.  Your transaction should be short and sweet.
			-- Clients of this feature are expected to leave the transaction depth at 0 by calling
			-- 'begin_transaction' and 'commit' within the client routine.  Be sure to call 'begin_transaction'
			-- BEFORE you create any predicates.
		do
			if saved_auxiliary_server = Void then
				create saved_auxiliary_server.make (node, database)
				saved_auxiliary_server.set_error_handler(saved_auxiliary_server.new_error_handler_print)
			end
			Result := saved_auxiliary_server
		end

feature -- Commit and rollback
	commit_if_necessary is
			-- Commit any changes to the data server and report is there was an error other than duplicate commit
		local
			old_handler: like error_handler
			h: OLEDB_ERROR_HANDLER_COMMIT
		do
			old_handler := error_handler
			create h
			error_handler := h
			commit
			if (not success) then
				success := h.is_error_negligible
			end
			error_handler := old_handler
		end

feature -- Open
	open (a_clsid: POINTER; a_property_set: ARRAY[OLEDB_PROPERTY_SET]) is
			-- create a new session
		local
			hr: INTEGER
			l_data_source: OLEDB_DATA_SOURCE
		do
			debug ("make_movies")
				create cam.make (generating_type + "/" + node + "/" + database)
			end
			if playback_movie then
				make_succeeded := True	-- If it didn't succeed then why bother with a movie?
			else
				hr := coinitialize (default_pointer)
				create l_data_source.open (a_clsid, a_property_set)
				create session.make
				hr := session.open (l_data_source)
				if session.p /= default_pointer then
					sql_error (Current)
					make_succeeded := (hr >= 0)
					if not make_succeeded then
						session.set_status (hr)
						set_status (session)
						session := void
					end
				end
				l_data_source.release
			end
		end

	open_with_service_components (a_clsid: POINTER; a_init_property_set: ARRAY[OLEDB_PROPERTY_SET]
								  a_ds_property_set: ARRAY [OLEDB_PROPERTY_SET]) is
			-- Like 'open' but with extra properties used during initialization of the connection.
		local
			hr: INTEGER
			l_data_source: OLEDB_DATA_SOURCE
		do
			success := true
			debug ("make_movies")
				create cam.make (generating_type + "/" + node + "/" + database)
			end
			if playback_movie then
				make_succeeded := True	-- If it didn't succeed then why bother with a movie?
			else
				hr := coinitialize (default_pointer)
				create l_data_source.open_with_service_components (a_clsid, a_init_property_set)
				if (a_ds_property_set /= Void) then
					hr := l_data_source.set_properties (a_ds_property_set)
				end
				if l_data_source.status >= 0 and hr >= 0 then
					-- Prevent implicit connections by the 'OLE DB Provider for SQL Server'.
					-- Implicit connections generally occurs when a routine is 'apply'ed to
					-- a select predicate and the 'apply'ed routine accesses the same database.
					-- Since the 'select' is still busy, a new connection is created.  This
					-- new connection is outside of the current transaction and locks!
					-- See Q271128 and C:\Program Files\Common Files\System\Ole DB\SQLOLEDB.TXT.
					--				create l_property_set.make (constants.DBPROPSET_DATASOURCE)
					--				l_property_set.add_boolean_property (constants.DBPROP_MULTIPLECONNECTIONS, False)
					--				hr := l_data_source.set_properties (l_property_set)

					create session.make
					hr := session.open (l_data_source)
					if session.p /= default_pointer then
						sql_error (Current)
						make_succeeded := (hr >= 0)
						if not make_succeeded then
							session.set_status (hr)
							set_status (session)
							session := void
						end
					end
				else
					status := l_data_source.status
				end
				l_data_source.release
				reset_windows_exception_filter
			end
		end

feature -- execution
	data_server: DATA_SERVER is
			-- Current? WTF?
		do
			Result := Current
		end

	set_status (a_traceable: TRACEABLE) is
		do
			if a_traceable /= Void then
				status := a_traceable.status
				if status < 0 then
					if error_handler /= void then
						error_handler.on_error (status, a_traceable, generating_type + ".set_status")
						status := a_traceable.status
					else
						a_traceable.trace
						new_error_handler_print.on_error (status, Void, generating_type + ".set_status")
					end
				end
			else
				status := 0
			end
			success := status >= 0
		end

	fetch (a_cursor: OLEDB_CURSOR)
			-- Fetch the next row from 'a_cursor'.
		do
			if a_cursor.has_data then
				a_cursor.statement.rowset.accessor.free_record_memory
			end
			a_cursor.statement.set_status (a_cursor.statement.rowset.move_next)
			set_status (a_cursor.statement)
			a_cursor.set_has_data
			a_cursor.set_success (success)
		end

	execute (a_statement: OLEDB_STATEMENT; a_object: ANY; a_args: ARRAY [ANY])
			-- Execute 'a_statement' with these arguments.
		do
			if a_object /= void or (a_args /= void and then a_args.count /= 0) then
				a_statement.in_sqlda.sqlvar.set_parameters (a_object, a_args)
			end
			a_statement.execute
			set_status (a_statement)
			rows_affected := a_statement.rows_affected
			a_statement.set_success (success)
		end

	sql_execute_immediate(a_statement: STRING) is
			-- Immediately executes an SQL statement.
		local
			l_statement: STRING

			l_c: OLEDB_SIMPLE_COMMAND[OLEDB_DYNAMIC_ACCESSOR, OLEDB_NO_ROWSET]
			l_retry: BOOLEAN
			l_pair: PAIR[STRING, OLEDB_SIMPLE_COMMAND[OLEDB_DYNAMIC_ACCESSOR, OLEDB_NO_ROWSET]]
		do
			if not implicit_transaction and then		-- It hasn't been set yet.
	   			explicit_transaction_depth=0			-- He hasn't started his own transaction.
			 then
				l_statement := a_statement.twin
				l_statement.prune_all(' ')
				l_statement.to_lower

				if implicit_transactions_set or
				   l_statement.starts_with("commit") or
				   l_statement.starts_with("rollback") or
			   	   l_statement.starts_with("begintran") or
			   	   l_statement.starts_with("setimplicit") or
				   l_statement.starts_with("setdeadlock_priority")
				then
					-- Do nothing.
				else
					-- If we get here then we assume (erring on the side of caution) that 'a_statement' writes
					-- to the database.
					debug("explain_implicit_transactions")
						print("implicit_transactions ON : " + node + "/" + database + "/" + a_statement + "%N")
					end

					set_implicit_transaction_on		-- This sets 'implicit_transactions_set'.
				end
			end
			if not l_retry then
				if sql_script_capture_required then
					sql_capture_immediate_execute_statements (a_statement)
				end
				if playback_movie then
					l_pair ?= cam.next_frame ("sql_execute_immediate").frame_data
					cam.compare_partial_movie_frame (l_pair.first, a_statement)
					l_c := l_pair.second
					rows_affected := l_c.rows_affected
					set_status (l_c)
				else
					create l_c
					l_c.set_status (l_c.open_execute (session, a_statement, void, False))
					rows_affected := l_c.rows_affected
					set_status (l_c)
					l_c.release
					l_statement := a_statement.twin
					l_statement.to_lower
					record_sql_statistics (l_statement)
					if definition /= Void then
						parse_sql_statement (a_statement, definition)
					end
				end

				debug ("make_movies")
					if record_movie then
						create l_pair.make (a_statement, l_c)
						cam.record_movie_frame ("sql_execute_immediate", l_pair)
					end
				end
			else
				 error.raise (node + "." + database + " - Could not execute, check database or repository entry")
			end

		rescue
			if not l_retry  then
				l_retry := True
				retry
			end
		end

	statement_modifies_tempdb(a_statement: STRING): BOOLEAN is
	-- Returns True if the statement is modifying the tempdb.
	local
		l_statement: STRING
	do
		l_statement := a_statement.twin
		l_statement.prune_all(' ')
		l_statement.to_lower
		if l_statement.has_substring("insert#") or
		   l_statement.has_substring("insertinto#")
		then
			Result := True
		end
	end

feature -- Error handling
	allow_error (a_rose_sql_error_code: INTEGER) is
			-- Make this error a negligible error.
		do
			error_handler.allow_error (oledb_error_code_from_rose_sql_error_code (a_rose_sql_error_code))
		end

	dont_allow_error (a_rose_sql_error_code: INTEGER) is
			-- Don't ignore this error.
		do
			error_handler.dont_allow_error (oledb_error_code_from_rose_sql_error_code (a_rose_sql_error_code))
		end

	error_was_allowed(a_rose_sql_error_code: INTEGER): BOOLEAN
	-- True if the error occurred but was 'allowed'.  Be sure to also call 'clear_last_allowed_error'.
	do
		Result := error_handler.error_was_allowed(oledb_error_code_from_rose_sql_error_code(a_rose_sql_error_code))
	end

	clear_last_allowed_error
	-- Called in conjunction with 'error_was_allowed'.
	do
		error_handler.clear_last_allowed_error
	end

	sql_error (a_traceable: TRACEABLE) is
		do
		end

	clear_error is
			-- Reset status
		do
			status := 0
			success := True
		end

feature	-- Query prepartion
	open_cursor (a_cursor: OLEDB_CURSOR) is
		do
			a_cursor.statement.execute
			set_status (a_cursor.statement)
			a_cursor.set_success (success)
		end

	prepare (a_statement: OLEDB_STATEMENT) is
			-- <Precursor>
		do
			a_statement.prepare (0)
			set_status (a_statement)
			a_statement.set_success (success)
		end

	setup_sqlda (a_sqlda: OLEDB_SQLDA) is
		local
			l_sqlvar_structure: SQL_DATA_STRUCTURE
			i, l_count: INTEGER
		do
			if a_sqlda.is_parameter then
				l_count := a_sqlda.accessor.parameter_count
			else
				l_count := a_sqlda.accessor.column_count
			end
			create l_sqlvar_structure.make (1, l_count)
			from
				i := 1
			until
				i > l_count
			loop
				l_sqlvar_structure.put (new_sqlvar (a_sqlda, i - 1), i)
				i := i + 1
			end
			a_sqlda.set_data_structure (l_sqlvar_structure)
		end

	set_default_command_propset (a_default_command_propset: OLEDB_PROPERTY_SET) is
			-- Use to set properties like timeouts on every predicate.
		do
			default_command_propset := a_default_command_propset
		end

feature -- Utility

	tables_are_equal(a_table_name1: STRING;
					 a_server: SQL_DATA_SERVER; a_table_name2: STRING;
					 a_column_names_to_ignore: ROSE_LINEAR_ARRAY[STRING]): BOOLEAN is
	-- Returns 'True' if the contents of the two tables are the same.  The method is up to the ancestor.
	-- Fill in 'a_column_names_to_ignore' with columns that you don't want to test (usually date/time columns).
	local
		l_server2: OLEDB_ANY_DATA_SERVER
	do
		l_server2 ?= a_server
		if l_server2=Void
		then
			-- Because 'table_checksum' is only defined in this class...
			error.raise("Other server must be an OLEDB_ANY_DATA_SERVER.")
		else
			Result := table_checksum(a_table_name1, a_column_names_to_ignore)=
					  l_server2.table_checksum(a_table_name2, a_column_names_to_ignore)
			if Result
			then
				-- It is possible for the first checksum to pass even when the data is different.
				-- This can happen when the individual fields are the same but just rearranged.
				-- So, that is why we now call this second form of checksum.
				Result := table_checksum_stdev(a_table_name1, a_column_names_to_ignore)=
					  	  l_server2.table_checksum_stdev(a_table_name2, a_column_names_to_ignore)
			end
		end
	end

	tables_are_equal_with_tolerance(a_table_name1: STRING;
					 a_server: SQL_DATA_SERVER; a_table_name2: STRING;
					 a_column_names_to_ignore: ROSE_LINEAR_ARRAY[STRING]; a_epsilon: DOUBLE): BOOLEAN is
	-- Returns 'True' if the contents of the two tables are the same.  Fill in 'a_column_names_to_ignore'
	-- with columns that you don't want to test (usually date/time columns). It may happen that stdev
	-- values for original table, copy table may differ slightly in precision(as they are double values),
	-- we check with provided epsilon value.
	local
		l_server2: OLEDB_ANY_DATA_SERVER
		l_original_table_stdev, l_copy_table_stdev: DOUBLE
	do
		l_server2 ?= a_server
		if l_server2=Void
		then
			-- Because 'table_checksum' is only defined in this class...
			error.raise("Other server must be an OLEDB_ANY_DATA_SERVER.")
		else
			Result := table_checksum(a_table_name1, a_column_names_to_ignore)=
					  l_server2.table_checksum(a_table_name2, a_column_names_to_ignore)
			if Result then
				-- It is possible for the first checksum to pass even when the data is different.
				-- This can happen when the individual fields are the same but just rearranged.
				-- So, that is why we now call this second form of checksum.
				l_original_table_stdev := table_stdev(a_table_name1, a_column_names_to_ignore)
				l_copy_table_stdev := l_server2.table_stdev(a_table_name2, a_column_names_to_ignore)
				-- If the difference of two tables is less than or equal to a_epsilon then
				-- result is set to true.
				if l_original_table_stdev - l_copy_table_stdev <= a_epsilon then
					Result  := True
				end
			end
		end
	end

	table_checksum(a_table_name: STRING; a_column_names_to_ignore: ROSE_LINEAR_ARRAY[STRING]): INTEGER is
	-- Returns a checksum based on ALL rows and ALL columns (some can be excluded) in the table.
	-- BINARY_CHECKSUM is case sensitive.
	do
		Result := actual_table_checksum(a_table_name, "CHECKSUM_AGG", a_column_names_to_ignore)
	end

	table_checksum_stdev(a_table_name: STRING; a_column_names_to_ignore: ROSE_LINEAR_ARRAY[STRING]): INTEGER is
	-- Returns a checksum based on ALL rows and ALL columns (some can be excluded) in the table.
	-- BINARY_CHECKSUM is case sensitive.
	do
		Result := actual_table_checksum(a_table_name, "STDEV", a_column_names_to_ignore)
	end

	table_stdev(a_table_name: STRING; a_column_names_to_ignore: ROSE_LINEAR_ARRAY[STRING]): DOUBLE is
	-- Returns checksum. This feature uses Double Predicate.
	do
		Result := stdev(a_table_name, "STDEV", a_column_names_to_ignore)
	end

	has_table(a_table_name: STRING): BOOLEAN is
	-- Does this server, connected to the current database, have 'a_table_name'?
	local
		l_has_table_predicate: ANONYMOUS_PREDICATE
	do
		auxiliary_server.begin_transaction
		create l_has_table_predicate.make_using_server(auxiliary_server, "sp_tables " + a_table_name)
		if not l_has_table_predicate.success
		then
			error.raise("Failed to make the 'has_table' predicate.")
		end

		l_has_table_predicate.retrieve(Void, Void)
		if not l_has_table_predicate.success
		then
			error.raise("The 'has_table' predicate failed.")
		end

		auxiliary_server.commit
		Result := l_has_table_predicate.array.count>0
	end

	drop_table(a_table_name: STRING) is
	-- Completely removes (kills) a table.  There is no error if the table does not exist.
	-- Contrast with 'delete_table'.
	local
		l_save_handler: like error_handler
	do
		auxiliary_server.begin_transaction
		l_save_handler := auxiliary_server.error_handler
		auxiliary_server.set_error_handler(auxiliary_server.new_error_handler_print)
		auxiliary_server.allow_error(rose_sql_error_no_table)		-- Allow the "No Table" error.
		auxiliary_server.sql_execute_immediate("drop table " + a_table_name)
		auxiliary_server.set_error_handler(l_save_handler)
		if not auxiliary_server.success
		then
			error.raise("The execution of 'drop table' failed.")
		end

		auxiliary_server.commit
	end

	create_duplicate_table(a_old_table_name, a_new_table_name: STRING) is
	-- Makes a new table whose columns are identical to an existing table.
	-- Indexes are not copied.
	-- Columns marked as identity will still be an identity.
	local
		l_pred: EXECUTION_PREDICATE
	do
		auxiliary_server.begin_transaction
		create l_pred.make(auxiliary_server, "select top 0 * into " + a_new_table_name + " from " + a_old_table_name)
		if not l_pred.success
		then
			error.raise("Failed to create the predicate.")
		end

		l_pred.execute(Void, Void)
		if not l_pred.success
		then
			error.raise("The predicate failed.")
		end

		auxiliary_server.commit
	end

	column_names(a_table_name: STRING): ROSE_LINEAR_ARRAY[STRING] is
	-- Returns the names of all of the columns in a table (in lowercase).
	local
		l_pred: STRING_PREDICATE
		l_prefix: STRING
		l_dotdot_ptr: INTEGER
		l_table_name: STRING
	do
		auxiliary_server.begin_transaction
		l_dotdot_ptr := a_table_name.substring_index("..", 1)
		if l_dotdot_ptr/=0
		then
			l_prefix := a_table_name.substring(1, l_dotdot_ptr + 1)
			l_table_name := a_table_name.substring(l_dotdot_ptr + 2, a_table_name.count)
		else
			l_prefix := ""
			l_table_name := a_table_name
		end

		create l_pred.make(auxiliary_server,
			"select lower(a.name) from " + l_prefix + "syscolumns a, (select OBJECT_ID('" + a_table_name + "') as id) b where a.id=b.id")
		if not l_pred.success
		then
			raise("Could not create the column_names predicate.")
		end

		l_pred.retrieve(Void, Void)
		if not l_pred.success
		then
			raise("The column_names predicate failed.")
		end

		Result := l_pred.array
		auxiliary_server.commit
	end

	has_column_name(a_table_name, a_column_name: STRING): BOOLEAN is
		-- Returns True if 'a_table_name' contains 'a_column_name'.
	local
		l_predicate: ANONYMOUS_PREDICATE
	do
		auxiliary_server.begin_transaction
		create l_predicate.make_using_server(auxiliary_server,
											"exec sp_columns @table_name = '" + a_table_name + "', @column_name = '" + a_column_name + "'")
		if not l_predicate.success
		then
			raise("Failed to create the sp_columns predicate.")
		end

		l_predicate.retrieve (Void,Void)
		if not l_predicate.success
		then
			raise("sp_columns failed")
		end

		Result := not l_predicate.array.is_empty
		auxiliary_server.commit
	end

	has_trigger(a_table_name, a_trigger_name: STRING): BOOLEAN
			-- Does a table have a trigger?
			-- If 'a_table_name' is Void then we will check to see if any table contains
			-- this trigger.  This makes sense in SQL/Server since they don't allow
			-- duplicate triggers on different tables and sometimes we have to hunt them
			-- down and kill them (the trigger that's in the wrong table).
		local
			l_predicate: INTEGER_PREDICATE
		do
			auxiliary_server.begin_transaction
			if a_table_name/=Void
			then
			create l_predicate.make(
				auxiliary_server,
					"select count(*) as cnt from dbo.sysobjects a, dbo.sysobjects b %N%
					%where %N%
					%	a.name='" + a_trigger_name + "' and %N%
					%	a.parent_obj=b.id and %N%
				%	b.name='" + a_table_name + "'")
			else
				create l_predicate.make(
					auxiliary_server,
					"select count(*) as cnt from dbo.sysobjects %N%
					%where id = object_id('" + a_trigger_name + "')")
			end

			if not l_predicate.success
			then
				raise("Failed to create the 'has_trigger' predicate")
			end

			l_predicate.retrieve(Void, Void)
			if not l_predicate.success or l_predicate.array.is_empty
			then
				raise("The 'has_trigger' predicate failed")
			end

			Result := l_predicate.array.first>0
			auxiliary_server.commit
		end

	trigger_creation_date(a_table_name, a_trigger_name: STRING): ROSE_DATE
			-- Returns the creation date of a trigger.
			-- Returns Void if not found.
		local
			l_predicate: DATE_PREDICATE
		do
			auxiliary_server.begin_transaction
			create l_predicate.make(
				auxiliary_server,
				"select a.crdate from dbo.sysobjects a, dbo.sysobjects b %N%
				%where %N%
				%	a.name='" + a_trigger_name + "' and %N%
				%	a.parent_obj=b.id and %N%
				%	b.name='" + a_table_name + "'")
			if not l_predicate.success
			then
				raise("Failed to create the 'trigger_creation_date' predicate")
			end

			l_predicate.retrieve(Void, Void)
			if not l_predicate.success
			then
				raise ("trigger_creation_date predicate failed.")
			end

			if not l_predicate.array.is_empty
			then
				Result := l_predicate.array.first
			end

			auxiliary_server.commit
		end

feature -- Not Callable
	close_cursor (a_cursor: OLEDB_CURSOR) is
			-- not implemented
		do
			print ("NOT CALLABLE")
		end

	declare_cursor (a_cursor: OLEDB_CURSOR) is
			-- not implemented
		do
			print ("NOT CALLABLE")
		end

	allocate_sqlda_data (a_sqlda: SQL_SQLDA) is
		do
			print ("NOT CALLABLE")
		end

	old_set_status (a_traceable: TRACEABLE) is
		do
			print ("NOT CALLABLE")
		end

	lock_detected: BOOLEAN is
		do
			print ("NOT CALLABLE")
		end

	prefetch (a_cursor: OLEDB_CURSOR;a_count: INTEGER) is
		do
			print ("NOT CALLABLE")
		end

	duplicate_detected: BOOLEAN is
		do
			print ("NOT CALLABLE")
		end

	last_error_text: STRING is
		do
			print ("NOT CALLABLE")
		end

	reserved (string_: STRING): BOOLEAN is
		do
			print ("NOT CALLABLE")
		end

	release_statement (a_statement: SQL_STATEMENT)
			-- <Precursor>
		do
			print ("NOT CALLABLE")
		end

	sqlda_sqld (a_sqlda: POINTER): INTEGER is
			-- <Precursor>
		do
			print ("NOT CALLABLE")
		end

feature {NONE} -- Implementation

	record_sql_statistics (a_statement: STRING) is
			-- Record statistics on selected SQL statements.
		require
			a_statement_not_void: a_statement /= Void
			a_statement_is_lower: True -- a_statement.is_lower - waiting for ISE to add this elementary query, or to add for_all
		local
			l_splitter: ST_SPLITTER
			l_words: DS_LIST [STRING]
			l_verb: STRING
			l_pos: INTEGER
			l_statement: STRING
		do
			-- The SQL statement 'create' will have a '('.  We only want what's to the left of that.
			l_pos := a_statement.index_of('(', 1)
			if l_pos/=0
			then
				l_statement := a_statement.substring(1, l_pos - 1)
			else
				l_statement := a_statement
			end

			create l_splitter.make
			l_words := l_splitter.split (l_statement)
			if l_words.count > 2 then
				if l_words.item (2) ~ "table" then
					l_verb := l_words.item (1)
					l_words.start
					l_words.remove_at		-- Remove the verb.
					l_words.remove_at		-- Remove 'table'.
					-- The remaining words are rejoined to form the table name; which may contain spaces (i.e. "[my table]").
					if definition /= Void then
						if l_verb ~ "truncate" then
							definition.increment_truncate_count (l_splitter.join_unescaped (l_words))
						elseif l_verb ~ "drop" then
							definition.increment_drop_count (l_splitter.join_unescaped (l_words))
						elseif l_verb ~ "create" then
							definition.increment_create_count (l_splitter.join_unescaped (l_words))
						end
					end
				end
			end
		end

	reset_windows_exception_filter is
			-- This call is here to reset the exception handler that SQSAPI32.DLL
			-- is invalidating at initialization.  We are calling this at a time
			-- when we are reasonably sure that the DLL has already been loaded and the
			-- filter has been lost.
		once
			set_windows_exception_filter
		end

  	constants: OLEDB_PROPERTY_CONSTANTS is
  			-- oledb-specific constants
  		once
  			create Result
  		end

	actual_table_checksum(a_table_name, a_aggregation_type: STRING; a_column_names_to_ignore: ROSE_LINEAR_ARRAY[STRING]): INTEGER is
	-- This does the real work of comparing two tables.  See table_checksum and table_checksum_stdev.
	require
		valid_table_name: a_table_name/=Void and then a_table_name.count>0
		valid_aggregation_type: a_aggregation_type/=Void and then a_aggregation_type.count>0
	local
		l_pred: INTEGER_PREDICATE
		l_column_list: STRING
		l_column_name: STRING
		l_column_names: ROSE_LINEAR_ARRAY[STRING]
		l_index: INTEGER
	do
		if a_column_names_to_ignore=Void or else a_column_names_to_ignore.count=0
		then
			l_column_list := "*"
		else
			from
				l_column_names := column_names(a_table_name)
				a_column_names_to_ignore.start
			until
				a_column_names_to_ignore.off
			loop
				l_column_name := a_column_names_to_ignore.current_item.as_lower
				l_index := l_column_names.index_of(l_column_name, 1)
				if l_index/=0
				then
					l_column_names.go_i_th(l_index)
					l_column_names.remove
				end

				a_column_names_to_ignore.forth
			end

			from
				l_column_list := ""
				l_column_names.start
			until
				l_column_names.off
			loop
				if l_column_names.index/=l_column_names.lower
				then
					l_column_list.append(",")
				end

				l_column_list.append(l_column_names.current_item)
				l_column_names.forth
			end
		end

		create l_pred.make(Current, "select " + a_aggregation_type + "(BINARY_CHECKSUM(" + l_column_list + ")) as value from " + a_table_name)
		if not l_pred.success
		then
			raise("Failed to create the table_validation predicate in actual_table_checksum.")
		end
		l_pred.retrieve(Void, Void)
		if not l_pred.success
		then
			raise("The table_validation predicate failed in actual_table_checksum.")
		end
		Result := l_pred.array.item(1)
	end

	stdev(a_table_name, a_aggregation_type: STRING; a_column_names_to_ignore: ROSE_LINEAR_ARRAY[STRING]): DOUBLE is
	-- This does the real work of comparing two tables.
	require
		valid_table_name: a_table_name/=Void and then a_table_name.count>0
		valid_aggregation_type: a_aggregation_type/=Void and then a_aggregation_type.count>0
	local
		l_column_list: STRING
		l_column_name: STRING
		l_column_names: ROSE_LINEAR_ARRAY[STRING]
		l_index: INTEGER
		l_pred: DOUBLE_PREDICATE
	do
		if a_column_names_to_ignore=Void or else a_column_names_to_ignore.count=0
		then
			l_column_list := "*"
		else
			from
				l_column_names := column_names(a_table_name)
				a_column_names_to_ignore.start
			until
				a_column_names_to_ignore.off
			loop
				l_column_name := a_column_names_to_ignore.current_item.as_lower
				l_index := l_column_names.index_of(l_column_name, 1)
				if l_index/=0
				then
					l_column_names.go_i_th(l_index)
					l_column_names.remove
				end

				a_column_names_to_ignore.forth
			end

			from
				l_column_list := ""
				l_column_names.start
			until
				l_column_names.off
			loop
				if l_column_names.index/=l_column_names.lower
				then
					l_column_list.append(",")
				end

				l_column_list.append(l_column_names.current_item)
				l_column_names.forth
			end
		end

		create l_pred.make(Current, "select " + a_aggregation_type + "(BINARY_CHECKSUM(" + l_column_list + ")) as value from " + a_table_name)
		if not l_pred.success
		then
			raise("Failed to create the table_validation predicate in actual_table_checksum.")
		end
		l_pred.retrieve(Void, Void)
		if not l_pred.success then
			raise("The table_validation predicate failed in actual_table_checksum.")
		end
		Result := l_pred.array.item(1)
	end

feature {NONE} -- Predicates.

	transaction_depth_predicate: like transaction_depth_predicate_saved
		-- Used by 'transaction_depth'.  This used to be a simple INTEGER_PREDICATE.  Unfortunately, when implicit_transactions are
		-- on and @@TRANCOUNT is 0, an INTEGER_PREDICATE calls sp_oledb_ro_usrname and starts a transaction.  You can see this
		-- in the profiler (Query Analyzer doesn't use OLEDB and doesn't exhibit this behavior).
	do
		if transaction_depth_predicate_saved=Void
		then
			transaction_depth_predicate_saved := stream_command("select @@trancount")
		end

		Result := transaction_depth_predicate_saved
	end

	transaction_depth_predicate_saved: DATA_SERVER_STREAM_COMMAND		-- See 'transaction_depth_predicate'.

feature {NONE} -- External
	set_windows_exception_filter is
			-- In the Eiffel run-time.  See 'reset_windows_exception_filter'.
		external
			"C"
		alias
			"set_windows_exception_filter"
		end

end
