note
	description: "Same as OLEDB_BULK_ROWSET but can be initialized with multiple results."
	author: "Pierre Nallet"
	last_editor: "$Author: rj $"
	date: "$Date: 2014/09/05 $"
	revision: "$Revision: #4 $"
	path: "$File: //rose/source/kernel/infrastructure/persistence/windows/classic2/oledb_bulk_rowset_with_multiple_results.e $"

class
	OLEDB_BULK_ROWSET_WITH_MULTIPLE_RESULTS

inherit
	OLEDB_BULK_ROWSET
		redefine
			set_rowset_pointer,
			get_iid,
			close
		end
	OLEDB_COMMAND_INTERFACES

creation
	make

feature -- Access

	get_iid: POINTER is
		-- COM iid for multiple results
	do
		Result := iid_imultipleresults
	end

feature -- Basic operations

	close is
		-- Close the rowset
	do
		precursor
		if (multiple_results/= default_pointer) then
			iunknown_release(multiple_results).do_nothing
			multiple_results := default_pointer
		end
	end

	set_rowset_pointer (a_pointer: POINTER; a_rows_affected: INTEGER): INTEGER
			-- Set the COM pointer and returns the rows affected
			local
				l_found: BOOLEAN
				hr : INTEGER
				l_p: POINTER
				l_i: INTEGER_64		-- Actually, we need 32 or 64 depending on the platform.
			do
				multiple_results := a_pointer
						if (multiple_results /= default_pointer) then
					from until l_found loop

						hr := imultipleresults_getresult (multiple_results, default_pointer, 0, iid_irowset, $l_i, $l_p)
						l_found := hr /= 0 or l_p /= default_pointer
						if (l_i >=0) then
							Result := l_i.to_integer
						end
						if (hr <= 0) then
							Result := hr
						end
					end
					set_pointer (l_p)
				end

			end

feature {NONE} -- Implementation

	multiple_results: POINTER
		-- Multiple results com pointer


feature {NONE} -- Implementation

	IMultipleResults_GetResult (This: POINTER; pUnkOuter: POINTER; lResultFlag: INTEGER; riid: POINTER; pcRowsAffected: POINTER; ppRowset: POINTER): INTEGER
		-- IMultipleResults_GetResult encapsulation
		-- Note that DBROWCOUNT is 32 or 64-bit depending on the platform.
		external
			"C [macro <oledb.h>] (IMultipleResults *, IUnknown *, DBRESULTFLAG, REFIID, DBROWCOUNT *, IUnknown **): EIF_INTEGER"
		alias
			"IMultipleResults_GetResult"
		end


end
