indexing

	description: "Object providing access to an OLEDB column's status and values"
	author: "Pierre Nallet"
	last_editor: "$Author: yg $"
	date: "$Date: 2014/11/12 $"
	revision: "$Revision: #74 $"
	path: "$File: //rose/source/kernel/infrastructure/persistence/windows/classic/oledb_column_sqlvar.e $"

class

	OLEDB_COLUMN_SQLVAR

inherit

	OLEDB_SQLVAR
		redefine
			get_field,
			is_valid
		end
	COM_STRING_FUNCTIONS
		export
			{NONE} all
		end
	ROSE_CONSTANTS
		export
			{NONE} all
		end
	OLEDB_DBTYPE_CONSTANTS
		export
			{NONE} all
		end
	ROSE_CONSTANT_VALUES
		export
			{NONE} all
		end

create

	make

feature -- Access

	is_valid: BOOLEAN is
			-- Is this column available for data queries?
		do
			Result := accessor.ready
		end

	status: INTEGER is
			-- Status of the column
		do
			Result := accessor.get_status (index)
		end

	status_is_ok: BOOLEAN is
			-- Is the status of this column ok after the last query?
		do
			Result := accessor.status_is_ok (index)
		end

feature -- Basic operations

	set_status (a_status: INTEGER) is
			-- Set the status of this column
		do
			accessor.set_status (index,a_status)
		end

	set_attributes is
			-- Set the attribute values of this column from the accessor
		do
			index := index + 1
			sqlname := accessor.get_column_name (index)
			sqltype := accessor.get_column_type (index)
			sqllen := accessor.get_length (index)
			sqldata := accessor.get_value (index)
			set_oledb_type_string
		end

	set_value (a_value: ANY) is
			-- Set the column value in the accessor to a_value
		local
			l_string: STRING
			l_any: ANY
			l_integer: INTEGER_REF
			l_integer_64: INTEGER_64_REF
			l_double: DOUBLE_REF
			l_date_time: ROSE_DATE
			l_date: ROSE_DATE
			l_bytes: ARRAY [CHARACTER]
			l_area: SPECIAL [CHARACTER]
			l_char: CHARACTER_REF
			l_timestamp: OLEDB_DBTIMESTAMP [STACK_MEMORY_MODEL]
			l_boolean: BOOLEAN_REF
			l_time: ROSE_TIME
		do
			if a_value = Void then
				set_null_value (True)
			else
				set_null_value (False)
				if sqltype = dbtype_bytes then
					l_bytes ?= a_value
					if l_bytes /= Void then
						l_area := l_bytes.area
					else
						l_string ?= a_value
						if l_string /= Void then
							l_any := l_string.to_c
						else
							raise ("wrong type: The Eiffel type and the database type do not match, ARRAY [CHARACTER} expected")
						end
					end
				elseif sqltype = dbtype_dbtimestamp or sqltype= dbtype_date then
					l_date_time ?= a_value
					l_date ?= a_value
					if l_date_time /= Void then
						l_time := l_date_time.time
							-- `l_timestamp' has a pointer to the SQLVAR data area and each set_
							-- modifies the data area directly
						create l_timestamp.make
						l_timestamp.set (accessor.get_value (index))
						l_timestamp.set_year (l_date_time.year)
						l_timestamp.set_month (l_date_time.month)
						l_timestamp.set_day (l_date_time.day)
						l_timestamp.set_hour (l_time.hour)
						l_timestamp.set_minute (l_time.minute)
						l_timestamp.set_second (l_time.second)
						l_timestamp.set_fraction(l_time.milli_second * 1_000_000)
					elseif l_date /= Void then
							-- `l_timestamp' has a pointer to the SQLVAR data area and each set_
							-- modifies the data area directly
						create l_timestamp.make
						l_timestamp.set (accessor.get_value (index))
						l_timestamp.set_year (l_date.year)
						l_timestamp.set_month (l_date.month)
						l_timestamp.set_day (l_date.day)
						l_timestamp.set_hour (0)
						l_timestamp.set_minute (0)
						l_timestamp.set_second (0)
						l_timestamp.set_fraction (0)
					else
						raise ("wrong type: The Eiffel type and the database type do not match, date expected")
					end
				elseif sqltype= dbtype_r8 then
					l_double ?= a_value
					if l_double /= Void then
						accessor.set_double_value (index, l_double.item)
					end
				elseif sqltype= dbtype_r4 then
					l_double ?= a_value
					if l_double /= Void then
					else
						raise ("wrong type: The Eiffel type and the database type do not match, double expected")
					end
				elseif sqltype = dbtype_i1 or else sqltype = dbtype_i2 or else sqltype = dbtype_i4
				or else sqltype = dbtype_ui1 or else sqltype = dbtype_ui2 or else sqltype = dbtype_ui4 then
					l_integer ?= a_value
					if l_integer /= Void then
						accessor.set_integer_value (index, l_integer.item)
					else
						print (a_value)
						print (a_value.generator)
						print ("%N wrong type: The Eiffel type and the database type do not match, integer expected, OLEDB type :")
						print (sqltype)
						raise ("wrong type: The Eiffel type and the database type do not match, integer expected")
					end
				elseif sqltype = dbtype_i8 or else sqltype = dbtype_ui8 then
					l_integer_64 ?= a_value
					if l_integer_64 /= Void then
						accessor.set_integer_64_value (index, l_integer_64.item)
					else
						raise ("wrong type: The Eiffel type and the database type do not match, INTEGER_64 expected")
					end
				elseif sqltype = dbtype_bool then
					l_boolean ?= a_value
					if l_boolean /= Void then
						if l_boolean.item then
							accessor.set_integer_value(index, -1)
						else
							accessor.set_integer_value(index, 0)
						end
					else
						print (a_value)
						print (a_value.generator)
						print ("%N wrong type: The Eiffel type and the database type do not match, boolean expected")
						raise ("wrong type: The Eiffel type and the database type do not match, boolean expected")
					end
				elseif sqltype = dbtype_str or sqltype = dbtype_wstr then
					l_string ?= a_value
					if l_string = Void then
						l_char ?= a_value
						if l_char /= Void then
							create l_string.make (0)
							l_string.append_character (l_char.item)
						end
					end
					if l_string /= Void then
						accessor.set_string_value(index, l_string)
					else
						raise ("wrong type: The Eiffel type and the database type do not match, string of character expected")
					end
				elseif sqltype = dbtype_iunknown then
					l_string ?= a_value
					if l_string = Void then
						l_char ?= a_value
						if l_char /= Void then
							create l_string.make (0)
							l_string.append_character (l_char.item)
						end
					end
					if l_string /= Void then
						l_any := l_string.to_c
					else
						raise ("wrong type: The Eiffel type and the database type do not match, string of character expected (dbtype_iunknown)")
					end
				end
			end
		end

feature -- Value access

	get_field (an_index: INTEGER; a_receptacle: ANY) is
			-- Get the field value for this column and put it into a_receptacle at field index a_index
		local
			l_integer_value: INTEGER
			l_dynamic_type: INTEGER
			l_double_value: DOUBLE
			l_double_ref: DOUBLE_REF;
			l_decimal: RP_DECIMAL
			l_date_time: ROSE_DATE
			l_date: ROSE_DATE
			l_type_id: INTEGER
		do
			-- Programs that read a lot of rows (most of our programs) spend the
			-- bulk of their time here. Anything that can speed up this routine
			-- is worth while. Note that the routine 'status' is very slow.
			inspect internal.field_type (an_index, a_receptacle)
			when {INTERNAL}.reference_type then
				inspect sqltype
				when dbtype_variant then
					internal.set_reference_field (an_index,a_receptacle,variant_value)
				when dbtype_bytes then
					if status_is_ok then
						internal.set_reference_field (an_index,a_receptacle,bytes_value)
					end
				when dbtype_dbtimestamp then
					if status_is_ok then
						timestamp_reader.set (accessor.get_value (index))
						l_type_id := internal.field_static_type_of_type (an_index, internal.dynamic_type (a_receptacle))
						if (l_type_id = rose_date_time_type_id) then
							create l_date_time.make_unchecked (timestamp_reader.get_year, timestamp_reader.get_month, timestamp_reader.get_day,
															   timestamp_reader.get_hour, timestamp_reader.get_minute, timestamp_reader.get_second + timestamp_reader.get_fraction / 1_000_000_000)
							internal.set_reference_field (an_index,a_receptacle,l_date_time)
						else
							create l_date.make_unchecked (timestamp_reader.get_year, timestamp_reader.get_month, timestamp_reader.get_day,
														  timestamp_reader.get_hour, timestamp_reader.get_minute, timestamp_reader.get_second + timestamp_reader.get_fraction / 1_000_000_000)
							internal.set_reference_field (an_index,a_receptacle,l_date)
						end
					else
						internal.set_reference_field (an_index,a_receptacle,Void)
					end
				when dbtype_date then
					internal.set_reference_field (an_index,a_receptacle,date_time_value)
				when dbtype_bstr then
					if status_is_ok then
						internal.set_reference_field (an_index,a_receptacle,string_value)
					end
				when dbtype_str then
					if status_is_ok then
						internal.set_reference_field (an_index,a_receptacle,string_value)
					end
				when dbtype_wstr then
					if status_is_ok then
						l_type_id := internal.field_static_type_of_type (an_index, internal.dynamic_type (a_receptacle))
						if (l_type_id = string_32_type_id) then
							internal.set_reference_field (an_index,a_receptacle,string_32_value)
						else
						internal.set_reference_field (an_index,a_receptacle,string_value)
						end
					end
				when DBTYPE_NUMERIC then
					if status_is_ok then
						internal.set_reference_field (an_index,a_receptacle,decimal_value)
					end

				when dbtype_iunknown then
					if status_is_ok then
						internal.set_reference_field (an_index,a_receptacle,string_value)
					end
				when dbtype_r8 then
					l_dynamic_type := internal.field_static_type_of_type (an_index, internal.dynamic_type (a_receptacle))
					if status_is_ok then
						l_double_ref ?= value;
						l_decimal ?= value;
						if (l_dynamic_type = decimal_dynamic_type) then
							if (l_decimal = void) then
								if (l_double_ref /= Void) then
									create l_decimal.make_from_double (l_double_ref.item)
								end
							end
							internal.set_reference_field (an_index, a_receptacle, l_decimal)
						else --	if (l_dynamic_type = double_ref_dynamic_type or l_dynamic_type = double_dynamic_type) then
							if (l_double_ref = void) then
								if (l_decimal /= Void) then
									l_double_ref := l_decimal.to_double
								end
							end
							internal.set_reference_field (an_index, a_receptacle, l_double_ref)
						end
--						internal.set_reference_field (an_index,a_receptacle,value)
					end
				else
					internal.set_reference_field (an_index, a_receptacle, value)
				end
			when {INTERNAL}.character_type then
				internal.set_character_field (an_index, a_receptacle, character_value)
			when {INTERNAL}.double_type then
				--if status = dbstatus_s_ok then
				if status_is_ok then
					inspect sqltype
					when DBTYPE_VARIANT then
						temp_variant.set (accessor.get_value (index))
						l_double_value := temp_variant.get_dblval
					when DBTYPE_numeric then
						l_double_value := double_value_from_numeric
					else
						l_double_value := accessor.get_double_value (index)
					end
				else
					l_double_value := missing_double
				end
				internal.set_double_field (an_index, a_receptacle, l_double_value)
			when {INTERNAL}.real_type then
				internal.set_real_field (an_index, a_receptacle, real_value)
			when {INTERNAL}.integer_type then
				if status_is_ok then
					if sqltype = DBTYPE_VARIANT then
						temp_variant.set (accessor.get_value (index))
						l_integer_value := temp_variant.integer4
					else
						l_integer_value := accessor.get_integer_value (index)
					end
				else
					l_integer_value := null_integer
				end
				inspect sqltype
				when Dbtype_i8 then
					l_integer_value := integer_64_value.to_integer_32
				when Dbtype_i4 then
				when Dbtype_i2 then
					-- Destroy the upper 3 bytes and extend the sign.
					-- This is slow. Need to put in faster solution.
					if l_integer_value /= null_integer then
						l_integer_value := (integer_value * 65536) // 65536
					end
				when Dbtype_i1 then
					-- Destroy the upper 3 bytes and extend the sign.
					-- This is slow. Need to put in faster solution.
					if l_integer_value /= null_integer then
						l_integer_value := (integer_value * 65536 * 256) // (65536 * 256)
					end
				when Dbtype_ui1 then
					-- Destroy the upper 3 bytes but DON'T extend the sign.
					if l_integer_value /= null_integer then
						l_integer_value := integer_value.bit_and (255)
					end
				when dbtype_numeric then
					l_integer_value := integer_value_from_numeric
				else
				end
				internal.set_integer_field (an_index, a_receptacle, l_integer_value)
			when {INTERNAL}.integer_16_type then
				internal.set_integer_16_field (an_index, a_receptacle, integer_value.to_integer_16)
			when {INTERNAL}.integer_64_type then
				internal.set_integer_64_field (an_index, a_receptacle, integer_64_value)
			when {INTERNAL}.boolean_type then
				internal.set_boolean_field (an_index, a_receptacle, boolean_value)
			when {INTERNAL}.natural_8_type then
				internal.set_natural_8_field (an_index, a_receptacle, natural_8_value)
			else
				print ("invalid type: ")
				print (internal.field_type (an_index, a_receptacle).out)
				print ("%N")
			end
		end

	variant_value: ANY is
			-- Column value as db variant type
		do
			temp_variant.set (accessor.get_value (index))
			Result := temp_variant.to_any
		end

	double_ref_dynamic_type: INTEGER is
			-- Dynamic type for double refs
		local
			l_double_ref: DOUBLE_REF
		once
			l_double_ref := 123.3;
			Result := internal.dynamic_type (l_double_ref)
		end

	double_dynamic_type: INTEGER is
			-- Dynamic type for double
		local
			l_double: DOUBLE
		once
			l_double := 123.3;
			Result := internal.dynamic_type (l_double)
		end

	decimal_dynamic_type: INTEGER is
			-- Dynamic for decimals
		local
			l_decimal: RP_DECIMAL
		once
			l_decimal := 123;
			Result := internal.dynamic_type (l_decimal)
		end

	value: ANY is
			-- Column value returned as an ANY
		do
			inspect sqltype
			when dbtype_bytes then
				if status_is_ok then
					Result := bytes_value
				end
			when dbtype_dbtimestamp then
				Result := date_time_value
			when dbtype_date then
				Result := date_time_value
			when dbtype_bool then
				Result := boolean_value
			when dbtype_r8 then
				Result := double_value
			when dbtype_numeric then
				Result := double_value
			when dbtype_r4 then
				Result := real_value
			when dbtype_i1 then
				Result := integer_value
			when dbtype_i2 then
				Result := integer_value
			when dbtype_i4 then
				Result := integer_value
			when dbtype_i8 then
				Result := integer_64_value
			when dbtype_ui1 then
				Result := natural_8_value
			when dbtype_ui2 then
				Result := integer_value
			when dbtype_ui4 then
				Result := integer_value
			when dbtype_ui8 then
				Result := integer_64_value
			when dbtype_bstr then
				if status_is_ok then
					Result := string_value
				end
			when dbtype_str then
				if status_is_ok then
					Result := string_value
				end
			when dbtype_wstr then
				if status_is_ok then
					Result := string_value
				end
			when dbtype_iunknown then
				if status_is_ok then
					Result := string_value
				end
			end
		end

	date_time_value: ROSE_DATE is
			-- Column value as rose_date
		do
			Result:= date_value
		end

	date_value: ROSE_DATE is
			-- Column value as rose_date
		do
			if status_is_ok then
				timestamp_reader.set (accessor.get_value (index))
				create Result.make_unchecked (timestamp_reader.get_year, timestamp_reader.get_month, timestamp_reader.get_day,
											  timestamp_reader.get_hour, timestamp_reader.get_minute, timestamp_reader.get_second + timestamp_reader.get_fraction / 1_000_000_000)
			elseif status /= DBSTATUS_S_ISNULL then
				print ("invalid status " + status.out)
			end
		end

	natural_8_value: NATURAL_8 is
			-- Column value as natural_8, for representing SQL type tinyint
		do
			if status_is_ok then
				inspect sqltype
				when DBTYPE_VARIANT then
					temp_variant.set (accessor.get_value (index))
					Result := temp_variant.integer4.as_natural_8
				when DBTYPE_NUMERIC then
					Result := integer_value_from_numeric.as_natural_8
				else
					Result := accessor.get_natural_8_value (index)
				end
			else
				Result := null_integer.as_natural_8
			end
		end

	integer_value: INTEGER is
			-- Column value as integer
		do
			if status_is_ok then
				inspect sqltype
				when DBTYPE_VARIANT then
					temp_variant.set (accessor.get_value (index))
					Result := temp_variant.integer4
				when DBTYPE_NUMERIC then
					Result := integer_value_from_numeric
				else
					Result := accessor.get_integer_value (index)
				end
			else
				Result := null_integer
			end
		end

	integer_64_value: INTEGER_64 is
			-- Column value as integer_64
		do
			if status_is_ok then
				if sqltype = DBTYPE_VARIANT then
					temp_variant.set (accessor.get_value (index))
					Result := temp_variant.integer8

				elseif sqltype = DBTYPE_NUMERIC then
					Result := integer_64_value_from_numeric
				else
					Result := accessor.get_integer_64_value (index)
				end
			else
				Result := null_integer
			end
		end

	real_value: REAL is
			-- Column value as real
		do
			if status_is_ok then
				if sqltype = DBTYPE_VARIANT then
					temp_variant.set (accessor.get_value (index))
					Result := temp_variant.real4
				else
					Result := accessor.get_real_value (index)
				end
			else
				Result := missing_real
			end
		end

	double_value: DOUBLE is
			-- Column value as double
		do
			if status_is_ok then
				inspect sqltype
				when DBTYPE_VARIANT then
					temp_variant.set (accessor.get_value (index))
					Result := temp_variant.get_dblval
				when DBTYPE_numeric then
					Result := double_value_from_numeric
				else
					Result := accessor.get_double_value (index)
				end
			else
				Result := missing_double
			end
		end

	boolean_value: BOOLEAN is
			-- Column value as boolean
		do
			if sqltype = DBTYPE_VARIANT then
				temp_variant.set (accessor.get_value (index))
				Result := temp_variant.boolean
			else
				Result := (integer_value \\ 65536) /= 0
			end
		end

	character_value: CHARACTER is
			-- Column value as character
		do
			memcpy ($Result, accessor.get_value (index), 1)
		end

	stream_value: ANY is
			-- Column value as stream
		local
			l_string: STRING
			l_rose_stream: ROSE_STREAM
		do
			l_string := string_value
			create l_rose_stream
			Result := l_rose_stream.retrieve_from_string (l_string)
		end

	decimal_value: RP_DECIMAL is
			-- Column value as RP_DECIMAL
		do
			if not is_null then
				if sqltype = DBTYPE_NUMERIC then
					Result := data_convert.numeric_to_decimal (accessor.get_value (index))
				end
			end
		end
	string_32_value: STRING_32 is
			-- Column value as a string 32
		local
			l_c_string: C_STRING_32
		do
			if not is_null then
				inspect sqltype
				when dbtype_bstr, dbtype_wstr then
					create l_c_string.make_by_pointer (accessor.get_value (index))
					Result := l_c_string.string
				else
					(create {EXCEPTIONS}).raise ("invalid type" +sqltype.out)
				end
			end
		end

	string_value: STRING is
			-- Column value as string
		local
			l_area_pointer: POINTER
			l_count: INTEGER
			l_total_count: INTEGER
			l_effective_count: INTEGER
			l_any: ANY
			l_stream: OLEDB_SEQUENTIAL_STREAM
			l_hr: INTEGER
			l_type: INTEGER
			l_p1, l_p2, l_pointer: POINTER
		do
			if not is_null then
				inspect sqltype
				when dbtype_iunknown then
					l_type := accessor.get_column_type (index)
					l_count := accessor.get_length (index)
					l_pointer := accessor.get_pointer_value (index)
					create l_stream.make (l_pointer)
					if l_count = 4 then
						l_count := 32768
						from
							l_p1 := calloc (l_count + 1, 1)
							l_hr := l_stream.isequentialstream_read (l_stream.p, l_p1, l_count, $l_effective_count)
							l_total_count := l_effective_count
						until
							l_effective_count < l_count
						loop
							l_count := l_total_count
							l_p2 := calloc (l_total_count + l_count+ 1, 1)
							memmove (l_p2, l_p1, l_count)
							free (l_p1)
							l_p1 := l_p2
							l_hr := l_stream.isequentialstream_read (l_stream.p, l_p1 + l_total_count, l_count, $l_effective_count)
							l_total_count := l_total_count + l_effective_count
						end
					else
						l_p1 := calloc (l_count + 1, 1)
						l_hr := l_stream.isequentialstream_read (l_stream.p, l_p1, l_count, $l_effective_count)
						l_total_count := l_effective_count
					end
					Result := makestr (l_p1, l_total_count)
					free (l_p1)
					l_stream.release
					l_hr := l_stream.iunknown_release(l_pointer)
				when dbtype_bytes then
					l_area_pointer := accessor.get_value (index)
					l_count := accessor.get_length (index)
					create Result.make (l_count)
					Result.fill_blank
					l_any := Result.to_c
					memcpy ($l_any, l_area_pointer, l_count)
				when dbtype_str then
					create Result.make (0)
					Result.from_c_substring (accessor.get_value (index),1,accessor.get_length (index))
				when dbtype_bstr, dbtype_wstr then
					Result := widechar_to_string (accessor.get_value (index))
				when dbtype_variant then
					temp_variant.set (accessor.get_value (index))
					Result := temp_variant.string
				else
				end
			end
		ensure then
			proper_string: (Result /= Void) implies (Result.count <= Result.capacity)
		end

feature {NONE} -- Data conversions

	integer_value_from_numeric: INTEGER is
			-- Convert the column value from a numeric type and return as an integer
		do
			Result := data_convert.numeric_to_integer (accessor.get_value (index))
		end

	integer_64_value_from_numeric: INTEGER_64 is
			-- Convert the column value from a numeric type and return as an integer_64
		do
			Result := data_convert.numeric_to_integer_64 (accessor.get_value (index))
		end

	double_value_from_numeric: DOUBLE is
			-- Convert the column value from a numeric type and return as a double
		do
			Result := data_convert.numeric_to_double (accessor.get_value (index))
		end

feature {NONE} -- Helpers

	data_convert: OLEDB_DATA_CONVERT is
			-- Utility to perform OLEDB convert operations
		once
			create Result.make
		end

	temp_variant: COM_VARIANT is
			-- Utility object to access com variant data
		once
			create Result
		end

	rose_date_time_type_id: INTEGER is
			-- The dynamic type id for ROSE_DATE in this system
		once
			Result := internal.dynamic_type_from_string ("ROSE_DATE")
		end

	string_32_type_id: INTEGER is
			-- The dynamic type id for STRING_32 in this system
		once
			Result := internal.dynamic_type_from_string ("STRING_32")
		end

	rose_date_type_id: INTEGER is
			-- The dynamic type id for ROSE_DATE in this system
		once
			Result := internal.dynamic_type_from_string  ("ROSE_DATE")
		end

	timestamp_reader: OLEDB_DBTIMESTAMP [STACK_MEMORY_MODEL] is
			-- An instance of the OLEDB timestamp reader object to facilite reading into a DATE object instance
		once
			create Result.make
		end

invariant

	valid_accessor: accessor /= Void

end







