note
	description: "Encapsulation of OLEDB commands"
	author: "Pierre Nallet"
	last_editor: "$Author: er $"
 	date: "$Date: 2014/02/26 $"
	revision: "$Revision: #32 $"
	path: "$File: //rose/source/kernel/infrastructure/persistence/windows/classic2/oledb_command.e $"
	url: "http://msdn.microsoft.com/isapi/msdnlib.idc?theURL=/library/psdk/dasdk/olpr8xyr.htm"

deferred class
	OLEDB_COMMAND [ACCESSOR-> OLEDB_ANY_ACCESSOR create make end ,
			ROWSET->OLEDB_ANY_ROWSET create make end]

inherit

	OLEDB_ACCESSOR_ROWSET [ACCESSOR, ROWSET]
		rename
			make as accessor_rowset_make
		end

	COM_SMART_UNKNOWN

	COM_STRING_FUNCTIONS

	OLEDB_COMMAND_INTERFACES

	OLEDB_GUID_CONSTANTS

	MOVIE_CAMERA_AWARE
		export
			{NONE} all
		end

feature -- Operations
	open (a_session: OLEDB_SESSION; a_text: STRING; a_property_set: OLEDB_PROPERTY_SET): INTEGER is
			-- Open the command for a given session and text.
		require
			a_session_not_void: a_session /= Void
			a_text_not_void: a_text /= Void
		do
			accessor_rowset_make
			Result := create_command(a_session, a_text)
		end


	open_execute (a_session: OLEDB_SESSION; a_text: STRING; a_property_set: OLEDB_PROPERTY_SET; bbind: BOOLEAN): INTEGER is
			-- Open and execute.
		require
			a_session_not_void: a_session /= Void
			a_text_not_void: a_text /= Void
		do
			accessor_rowset_make
			Result := create_command(a_session, a_text)
			if Result >= 0 then
				Result := execute_(rowset, Void, a_property_set)
				if Result >= 0 then
					if bbind then
						Result  := bind
					end
				end
			end
		end

	create_command (a_session: OLEDB_SESSION; a_text: STRING): INTEGER is
			-- Create the OLEDB command object.
		require
			a_session_not_void: a_session /= Void
			a_text_not_void: a_text /= Void
		local
			l_pDBCreateCommand: POINTER
			l_pCommandText: POINTER
			txt: POINTER
			hr: INTEGER
			l_p: POINTER
		do
			release
			Result := a_session.Query_Interface (IID_IDBCreateCommand, $l_pDBCreateCommand)
			if Result>= 0 then
				l_p := p
				Result := IDBCreateCommand_CreateCommand (l_pdbcreatecommand, default_pointer, IID_ICommand, $l_p)
				p := l_p
				if Result >= 0 then
					Result := query_Interface (IID_ICommandText, $l_pcommandtext)
					if Result >= 0 then
						txt := string_to_widechar (a_text)
						Result := ICommandText_SetCommandText(l_pcommandtext, DBGUID_DEFAULT, txt)
						free (txt)
						hr := IUnknown_Release(l_pcommandtext)
					end
				end
				hr := IUnknown_Release(l_pdbcreatecommand)
			end
		end

	 execute_(a_rowset: ROWSET; pParams: OLEDB_DBPARAMS[MEMORY_MODEL]; propset: OLEDB_PROPERTY_SET): INTEGER is
	 		-- Implementation of execute
		require
			exists: exists
		local
			l_pCommandProperties: POINTER
		--	l_multiple_results: POINTER
			prm: POINTER
			hr: INTEGER
			l_p: POINTER
			l_i: INTEGER
		do
			if propset /= Void then
				Result := query_interface(IID_ICommandProperties, $l_pcommandproperties)
				if Result >= 0 then
					Result := icommandproperties_SetProperties(l_pcommandproperties, 1, propset.item)
					hr := iunknown_release (l_pcommandproperties)
				end
			end
			if Result>= 0 then
				if pparams /= Void then
					prm := pparams.item
				end
				a_rowset.release	-- Since 'a_rowset' may get reused, we must release the previous (if any) rowset.
				l_p := a_rowset.p
				rows_affected := 0
				l_i := rows_affected
				iCommand_Execute2(p, default_pointer, a_rowset.get_iid, prm, $l_i,  $l_p, $Result)
				rows_affected := a_rowset.set_rowset_pointer (l_p, l_i)
			end
		end

	execute(bbind: BOOLEAN) : INTEGER is
			-- Execute the command
		local
			pparams: OLEDB_DBPARAMS[RUN_TIME_MEMORY_MODEL]
			l_propset: OLEDB_PROPERTY_SET
			l_constants: OLEDB_PROPERTY_CONSTANTS
		do
			if accessor.has_parameters and accessor.get_parameter_handle/=default_pointer then
				create pparams.make
				pparams.set_cparamsets (1)
				pparams.set_pdata(accessor.get_parameter_buffer)
				pparams.set_haccessor(accessor.get_parameter_handle)
			end

			-- The following code is based on the article "Rowsets and SQL Server Cursors".

			create l_constants
--			create l_propset.make(l_constants.DBPROPSET_ROWSET)

			-- Default (firehose) cursor.  This allows 'OLE DB Provider for SQL Server'
			-- to create implicit connections (that's bad).  The cursors below prevent most
			-- of these (preparation of a statememt may create one, that's OK as long
			-- as you don't do it too often).
			l_propset := default_command_propset

			-- Client cursor.  I've gotten this one to work.
--			l_propset.add_boolean_property(l_constants.DBPROP_CLIENTCURSOR, True)

			-- Static cursor.  I've gotten this one to fail.
--			l_propset.add_boolean_property(l_constants.DBPROP_SERVERCURSOR, True)

			-- Fast Forward-only cursor (include SERVERCURSOR).
--			l_propset.add_boolean_property(l_constants.DBPROP_OTHERINSERT, True)
--			l_propset.add_boolean_property(l_constants.DBPROP_OTHERUPDATEDELETE, True)
--			l_propset.add_boolean_property(l_constants.DBPROP_OWNINSERT, True)
--			l_propset.add_boolean_property(l_constants.DBPROP_OWNUPDATEDELETE, True)

			-- Keyset cursor (include SERVERCURSOR).
--			l_propset.add_boolean_property(l_constants.DBPROP_OTHERINSERT, True)
--			l_propset.add_boolean_property(l_constants.DBPROP_OTHERUPDATEDELETE, True)
--			l_propset.add_boolean_property(l_constants.DBPROP_OWNINSERT, True)
--			l_propset.add_boolean_property(l_constants.DBPROP_IMMOBILEROWS, True)

			-- Dynamic cursor (include SERVERCURSOR).  Fails immediately!  What's wrong?
--			l_propset.add_boolean_property(l_constants.DBPROP_OTHERINSERT, True)
--			l_propset.add_boolean_property(l_constants.DBPROP_OTHERUPDATEDELETE, True)
--			l_propset.add_boolean_property(l_constants.DBPROP_OWNINSERT, True)
--			l_propset.add_boolean_property(l_constants.DBPROP_OWNUPDATEDELETE, True)
--			l_propset.add_boolean_property(l_constants.DBPROP_REMOVEDELETED, True)

			Result := execute_(rowset, pparams, l_propset)
			if Result >= 0 and bbind then
				Result := bind
			end
		end

	prepare (a_expected_runs: INTEGER) is
		require
			exists: exists
		local
			l_pcommandprepare: POINTER
			hr: INTEGER
		do
			set_status (IUnknown_QueryInterface(p, IID_ICommandPrepare, $l_pcommandprepare))
			if status >= 0 then
				set_status(ICommandPrepare_Prepare(l_pcommandprepare, a_expected_runs))
				hr := IUnknown_release(l_pcommandprepare)
			end
		end

	rows_affected: INTEGER		-- Number of rows affected by a delete, insert or update.

	default_command_propset: OLEDB_PROPERTY_SET is
	do
		-- A descendant may implement this.
	end

feature -- Access

	set_status (a_integer : INTEGER) is
		deferred
		end
	status : INTEGER is
			-- Status of the command
		deferred
		end

end
