indexing
	description: "Access to constants used in OLEDB"
	author: "Pierre Nallet"
	last_editor: "$Author: rj $"
	date: "$Date: 2011/07/12 $"
	revision: "$Revision: #14 $"
	path: "$File: //rose/source/kernel/infrastructure/persistence/windows/classic/oledb_constants.e $"

class
	OLEDB_CONSTANTS

inherit
	SQL_CONSTANTS

feature {NONE} -- Access

	oledb_error_code_from_rose_sql_error_code (a_rose_sql_error_code: INTEGER): like error_template
			-- The oledb error code mapped to 'a_rose_sql_error_code'.  The first item is what you see
			-- as 'a_error_id' in OLEDB_ERROR_HANDLER_PRINT.on_error.  The second item is the native
			-- SQL error.  You will see this as 'l_sql_error_info.native_error'.  Zero is a wild
			-- card.  It's not really clear when to use the wild card.  When I was writing the deadlock
			-- stuff the first item was sometimes E_FAIL and sometimes -1.
		do
			inspect	a_rose_sql_error_code
			when Rose_sql_error_no_table, Rose_sql_error_no_trigger then
				Result := [Db_e_notable, 0]
			when Rose_sql_error_no_column then
				Result := [DB_E_ERRORSINCOMMAND, 0]
			when Rose_sql_error_dup_key then
				Result := [DB_E_INTEGRITYVIOLATION, 0]
			when Rose_deadlock_victim then
				Result := [0, deadlock_victim_native_error]
			else
				Result := Void
			end
		end

feature {NONE} -- Native SQL error constants.  For some reason, Microsoft doesn't have symbols for these.
	deadlock_victim_native_error: INTEGER = 1205	-- You have been chosen as the deadlock victim.

	dup_key_ignored_native_error: INTEGER = 3604	-- Duplicate key was ignored.

feature -- Once/constants.
	Db_countunavailable: INTEGER is -1

	Dbtype_null_type: INTEGER is 1

	Dbtype_i2_type: INTEGER is 2

	Dbtype_i4_type: INTEGER is 3

	Dbtype_r4_type: INTEGER is 4

	Dbtype_r8_type: INTEGER is 5

	Dbtype_cy_type: INTEGER is 6

	Dbtype_date_type: INTEGER is 7

	Dbtype_bstr_type: INTEGER is 8

	Dbtype_idispatch_type: INTEGER is 9

	Dbtype_error_type: INTEGER is 10

	Dbtype_bool_type: INTEGER is 11

	Dbtype_variant_type: INTEGER is 12

	Dbtype_iunknown_type: INTEGER is 13

	Dbtype_decimal_type: INTEGER is 14

	Dbtype_ui1_type: INTEGER is 17

	Dbtype_array_type: INTEGER is 8192

	Dbtype_byref_type: INTEGER is 16384

	Dbtype_i1_type: INTEGER is 16

	Dbtype_ui2_type: INTEGER is 18

	Dbtype_ui4_type: INTEGER is 19

	Dbtype_i8_type: INTEGER is 20

	Dbtype_ui8_type: INTEGER is 21

	Dbtype_guid_type: INTEGER is 72

	Dbtype_vector_type: INTEGER is 4096

	Dbtype_reserved_type: INTEGER is 32768

	Dbtype_bytes_type: INTEGER is 128

	Dbtype_str_type: INTEGER is 129

	Dbtype_wstr_type: INTEGER is 130

	Dbtype_numeric_type: INTEGER is 131

	Dbtype_udt_type: INTEGER is 132

	Dbtype_dbdate_type: INTEGER is 133

	Dbtype_dbtime_type: INTEGER is 134

	Dbtype_dbtimestamp_type: INTEGER is 135

	DBTYPE_IUNKNOWN: INTEGER is
			-- Oledb error value for unknown type
		external
			"C [macro <oledb.h>]"
		alias
			"DBTYPE_IUNKNOWN"
		end

	DBTYPE_STR: INTEGER is
		external
			"C [macro <oledb.h>]"
		alias
			"DBTYPE_STR"
		end

	DBTYPE_WSTR: INTEGER is
		external
			"C [macro <oledb.h>]"
		alias
			"DBTYPE_WSTR"
		end

	DB_E_ERRORSINCOMMAND: INTEGER is
		external
			"C [macro <oledberr.h>]"
		alias
			"DB_E_ERRORSINCOMMAND"
		end

	DB_E_INTEGRITYVIOLATION: INTEGER is
		external
			"C [macro <oledberr.h>]"
		alias
			"DB_E_INTEGRITYVIOLATION"
		end

	DB_SEC_E_AUTH_FAILED: INTEGER is
		external
			"C [macro <oledberr.h>]"
		alias
			"DB_SEC_E_AUTH_FAILED"
		end

	DB_S_ENDOFROWSET: INTEGER
	external
		"C [macro <oledberr.h>]"
	alias
		"DB_S_ENDOFROWSET"
	end

	DB_E_NOTABLE: INTEGER	-- DB_E_NOTABLE (means NO TABLE).
	external
		"C [macro <oledberr.h>]"
	alias
		"DB_E_NOTABLE"
	end

	E_FAIL: INTEGER
	external
		"C [macro <winerror.h>]"
	alias
		"E_FAIL"
	end

feature -- Anchors.
	error_template: TUPLE[oledb_error: INTEGER; native_error: INTEGER]
	-- Do not call.  Say "like error_template".
	do
	end

end


