indexing
	description: "Wrapper around IDataConvert::DataConvert."
	author: "Pierre Nallet"
	last_editor: "$Author: rj $"
	date: "$Date: 2014/09/05 $"
	revision: "$Revision: #15 $"
	path: "$File: //rose/source/kernel/infrastructure/persistence/windows/classic/oledb_data_convert.e $"

class
	OLEDB_DATA_CONVERT

inherit
	COM_UNKNOWN
	COM_CLSCTX
	OLEDB_DBTYPE_CONSTANTS
	OLEDB_SIZEOF

create
	make

feature {NONE} -- Initialization

	make is
		do
			create_instance (CLSID_OLEDB_CONVERSIONLIBRARY, CLSCTX_ALL, IID_IDataConvert)
		end

feature -- Access

	numeric_to_integer (ptr: POINTER): INTEGER is
		local
			hr: INTEGER
			len: INTEGER_64		-- Actually, we need 32 or 64 depending on the platform.
			status: INTEGER
			l_result: INTEGER
		do
			hr := idataconvert_dataconvert (p, DBTYPE_NUMERIC, DBTYPE_I4, sizeof_voidstar, $len, ptr, $l_result, sizeof_int, 0, $status, 0, 0, 0)
			Result := l_result
		end

	numeric_to_decimal (ptr: POINTER): RP_DECIMAL is
		local
			hr: INTEGER
			len: INTEGER_64		-- Actually, we need 32 or 64 depending on the platform.
			status: INTEGER
			l_data: MANAGED_POINTER
		do
			create l_data.make ({RP_DECIMAL}.byte_count)
			hr := idataconvert_dataconvert (p, DBTYPE_NUMERIC, DBTYPE_DECIMAL, sizeof_voidstar, $len, ptr, l_data.item, {RP_DECIMAL}.byte_count, 0, $status, 0, 0, 0)
			create Result.make_managed_pointer (l_data)
		end

	numeric_to_integer_64 (ptr: POINTER): INTEGER_64 is
		local
			hr: INTEGER
			len: INTEGER_64		-- Actually, we need 32 or 64 depending on the platform.
			status: INTEGER
			l_result: INTEGER_64
		do
			hr := idataconvert_dataconvert (p, DBTYPE_NUMERIC, DBTYPE_I8, sizeof_voidstar, $len, ptr, $l_result, sizeof_int_64, 0, $status, 0, 0, 0)
			Result := l_result
		end

	numeric_to_double (ptr: POINTER): DOUBLE is
		local
			hr: INTEGER
			len: INTEGER_64		-- Actually, we need 32 or 64 depending on the platform.
			status: INTEGER
			l_result: DOUBLE
		do
			hr := idataconvert_dataconvert (p, DBTYPE_NUMERIC, DBTYPE_R8, sizeof_voidstar, $len, ptr, $l_result, sizeof_double, 0, $status, 0, 0, 0)
			Result := l_result
		end

	double_to_numeric (d: DOUBLE; ptr: POINTER; precision, scale: INTEGER) is
		local
			hr: INTEGER
			len: INTEGER_64		-- Actually, we need 32 or 64 depending on the platform.
			status: INTEGER
			l_double: DOUBLE
		do
			l_double := d
			hr := idataconvert_dataconvert (p, DBTYPE_R8, DBTYPE_NUMERIC, sizeof_double, $len, $l_double, ptr, sizeof_voidstar, 0, $status, precision, scale, 0)
		end

	decimal_to_numeric (d: RP_DECIMAL; ptr: POINTER; precision, scale: INTEGER) is
		local
			hr: INTEGER
			len: INTEGER_64		-- Actually, we need 32 or 64 depending on the platform.
			status: INTEGER
		do
			hr := idataconvert_dataconvert (p, DBTYPE_DECIMAL, DBTYPE_NUMERIC, d.byte_count, $len, d.data.item, ptr, sizeof_voidstar, 0, $status, precision, scale, 0)
		end

	IDataConvert_DataConvert (This: POINTER;
		wSrcType: INTEGER;
		wDstType: INTEGER;
		cbSrcLength: INTEGER;
		pcbDstLength: POINTER;
		pSrc: POINTER;
		pDst: POINTER;
		cbDstMaxLength: INTEGER;
		dbsSrcStatus: INTEGER;
		pdbsStatus: POINTER;
		bPrecision: INTEGER;
		bScale: INTEGER
		dwFlags: INTEGER): INTEGER
			-- Calls IDataConvert::DataConvert.  Note that DBLENGTH is 32- or 64-bit depending on the platform.  The C code will
			-- do the right thing (this class only deals with very short items) except in the case of
			-- "DBLENGTH *".  IDataConvert::DataConvert will write to that address.  Always pass an INTEGER_64 even in 32-bit code.
			-- Also note that DBTYPE is 16-bits.  We use 32-bits.
		external
			"C [macro <oledb.h>] (IDataConvert*, DBTYPE, DBTYPE, DBLENGTH, DBLENGTH *, void *, void *, DBLENGTH, DBSTATUS, DBSTATUS *, BYTE, BYTE, DBDATACONVERT): EIF_INTEGER"
		alias
			"IDataConvert_DataConvert"
		end

	IID_IDATAConvert: POINTER is
			--
			external
				"C [macro <msdadc.h>]"
			alias
				"&IID_IDataConvert"
			end

	CLSID_OLEDB_CONVERSIONLIBRARY: POINTER is
			--
			external
				"C [macro <msdaguid.h>]"
			alias
				"&CLSID_OLEDB_CONVERSIONLIBRARY"
			end

end
