indexing
	description: "Object that has the COM connection to the OLEDB data source"
	author: "Pierre Nallet"
	last_editor: "$Author: er $"
	date: "$Date: 2014/04/28 $"
	revision: "$Revision: #1 $"
	path: "$File: //rose/source/kernel/infrastructure/persistence/windows/classic/oledb_data_source.e $"

class
	OLEDB_DATA_SOURCE

inherit
	COM_RUN_TIME
		export
			{NONE} all
		end
	COM_SMART_UNKNOWN
		export
			{NONE} all
			{ANY} exists, query_interface, release
		end
	SQLOLEDB_PROPERTY_CONSTANTS
		export
			{NONE} all
		end
	COM_CLSCTX
		export
			{NONE} all
		end
	COM_ERRORS
		export
			{NONE} all
		end
	TRACEABLE

create
	open,
	open_with_service_components,
	open_from_initialization_string

feature -- Creation
	open_from_initialization_string (a_string: STRING) is
		require
			valid_string: a_string /= Void
		local
			spDataInit: POINTER
			hr, hr2: INTEGER
			szInitializationString: POINTER
			l_p : POINTER
		do
			hr := coinitialize (default_pointer)
			spDataInit := p
			hr := CoCreateInstance(CLSID_MSDAINITIALIZE, default_pointer, CLSCTX_INPROC_SERVER,
								   IID_IDataInitialize, $spDataInit);

			if hr = REGDB_E_CLASSNOTREG
			then
				class_not_registered(CLSID_MSDAINITIALIZE)
			else
				check_hr(hr, "CoCreateInstance")
			end

			szInitializationString := string_to_widechar (a_string)
			l_p := p
			hr := IDataInitialize_GetDataSource(spDataInit, default_pointer, CLSCTX_INPROC_SERVER, szInitializationString,
											   IID_IDBInitialize, $l_p);
			p := l_p
			free (szInitializationString)
			hr2 := iunknown_release(spDataInit)		-- Store in 'hr2' to preserve 'hr'.
			check_hr(hr, "IDataInitialize_GetDataSource")

			hr := IDBInitialize_initialize (p)
			check_hr(hr, "IDBInitialize_initialize")
		end

	open (a_clsid: POINTER; a_property_set: ARRAY[OLEDB_PROPERTY_SET]) is
		local
			hr: INTEGER
			l_p : POINTER
		do
			l_p := p
			hr := CoCreateInstance (a_clsid, default_pointer, CLSCTX_ALL, IID_IDBInitialize, $l_p)
			p := l_p

			if hr = REGDB_E_CLASSNOTREG
			then
				class_not_registered(a_clsid)
			else
				check_hr(hr, "CoCreateInstance")
			end

			open_with_properties (a_property_set)
		end

	open_with_service_components (a_clsid: POINTER; a_property_set: ARRAY[OLEDB_PROPERTY_SET]) is
		local
			hr, hr2: INTEGER
			spDataInit: POINTER
			l_p : POINTER
		do
			-- Connect to OLEDB32.DLL (indicated by CLSID_MSDAINITIALIZE).
			hr := CoCreateInstance (CLSID_MSDAINITIALIZE, default_pointer, CLSCTX_ALL, IID_IDataInitialize, $spDataInit)

			if hr = REGDB_E_CLASSNOTREG
			then
				class_not_registered(CLSID_MSDAINITIALIZE)
			else
				check_hr(hr, "CoCreateInstance")
			end

			-- Release previous 'p' (if any).
			if (p /= default_pointer) then
				hr := iunknown_release (p)
				p := default_Pointer
			end

			-- Connect to SQLOLEDB.DLL or SQLNCLI11.DLL (according to 'a_clsid').  Save 'p'.
			l_p := p
			hr := IDatainitialize_createdbInstance ( spdatainit, a_clsid, default_pointer, CLSCTX_ALL, default_pointer,  IID_IDBInitialize, $l_p)
			p := l_p

			-- Free 'spdatainit'.
			hr2 := iunknown_release (spdatainit)		-- Store in 'hr2' to preserve 'hr'.

			-- Check the result of IDatainitialize_createdbInstance.
			if hr = REGDB_E_CLASSNOTREG
			then
				class_not_registered(a_clsid)
			else
				check_hr(hr, "IDatainitialize_createdbInstance")
			end

			-- Complete the connection (using 'p').
			open_with_properties (a_property_set)
		end

feature	 -- Properties
	set_properties (a_property_set: ARRAY[OLEDB_PROPERTY_SET]): INTEGER is
			-- Set the connection properties of this session to a_property_set
		require
			exists: exists
			a_property_set_not_void: a_property_set /= Void
		local
			dbproperties: POINTER
			hr: INTEGER
			i: INTEGER
		do
			from
				i := a_property_set.lower
			until
				i > a_property_set.upper
			loop
				hr := query_Interface (IID_IDBProperties, $dbproperties)
				if hr >= 0 then
					hr := IDBProperties_SetProperties(dbproperties, 1, a_property_set.item(i).item)
					if hr < 0 then
						Result := hr
					end
					hr := iunknown_release (dbproperties)
				else
					Result := hr
				end
				i := i + 1
			end
		end

feature	{NONE} -- Implementation
	open_with_properties (a_property_set: ARRAY[OLEDB_PROPERTY_SET])
			-- Finish making the connection to the database.
		require
			exists: exists
			a_property_set_not_void: a_property_set /= Void
		local
			l_hr: INTEGER
			l_error_handler: OLEDB_ERROR_HANDLER_PRINT
		do
			l_hr := set_properties (a_property_set)
			l_hr := IDBInitialize_initialize (p)
			if l_hr < 0 then
				create l_error_handler
				l_error_handler.on_error (l_hr, Void, generating_type + ".open_with_properties")
				set_status (l_hr)
			end
		end

	class_not_registered(a_clsid: POINTER)
	-- The class id was not found in the registry (HKEY_CLASSES_ROOT\CLSID\{...}).
	-- Class id's are usually defined in an .h file.  I believe this happens to us when the registry is unavailable.
	local
		l_ecom_guid: ECOM_GUID
	do
		create l_ecom_guid.make_from_pointer(a_clsid)

		if l_ecom_guid ~ E_SQLNCLI_CLSID
		then
			raise(E_SQLNCLI_CLSID_PROVIDER + " failed to load")
		elseif l_ecom_guid ~ E_CLSID_SQLOLEDB
		then
			raise("'Microsoft OLE DB Provider for SQL Server' (SQLOLEDB.DLL) failed to load")
		elseif l_ecom_guid ~ E_CLSID_MSDAINITIALIZE
		then
			raise("MSDAINITIALIZE (OLEDB32.DLL) failed to load")
		else
			raise("SQL provider failed to load.  CLSID = " + l_ecom_guid.out)
		end
	end

	check_hr(a_hr: INTEGER; a_text: STRING)
	-- Raises an exception if 'a_hr' is less than zero.
	require
		valid_text: a_text /= Void and then not a_text.is_empty
	do
		if a_hr < 0
		then
			raise(a_text + " failed with " + a_hr.out)
		end
	end

feature {NONE} -- Externals
	IID_IDBInitialize: POINTER is
		external
			"C [macro <oledb.h>]"
		alias
			"&IID_IDBInitialize"
		end

	IID_IDBPRoperties: POINTER is
		external
			"C [macro <oledb.h>]"
		alias
			"&IID_IDBProperties"
		end

	E_CLSID_MSDAINITIALIZE: ECOM_GUID
	-- Eiffel version of CLSID_MSDAINITIALIZE.
	once
		create Result.make_from_pointer(CLSID_MSDAINITIALIZE)
	end

	CLSID_MSDAINITIALIZE: POINTER is
		external
			"C [macro <MSDASC.H>]"
		alias
			"&CLSID_MSDAINITIALIZE"
		end

	IID_IDataInitialize: POINTER is
		external
			"C [macro <MSDASC.H>]"
		alias
			"&IID_IDataInitialize"
		end

	IDBProperties_GetProperties(This: POINTER; cPropertyIDSets: INTEGER; rgPropertyIDSets: POINTER; pcPropertySets: POINTER;prgPropertySets: POINTER): INTEGER is
		external
			"C [macro <oledb.h>] (IDBProperties*, ULONG, DBPROPIDSET*, ULONG*, DBPROPSET**): EIF_INTEGER"
		alias
			"IDBProperties_GetProperties"
		end

	IDBProperties_GetPropertyInfo(This: POINTER;  cPropertyIDSets: INTEGER;
								  rgPropertyIDSets: POINTER;
								  pcPropertyInfoSets: POINTER;
								  prgPropertyInfoSets: POINTER;
								  ppDescBuffer: POINTER): INTEGER is
		external
			"C [macro <oledb.h>] (IDBProperties*, ULONG , DBPROPIDSET *, ULONG *, DBPROPINFOSET **, OLECHAR **): EIF_INTEGER"
		alias
			"IDBProperties_GetPropertyInfo"
		end
	IDBProperties_SetProperties(This: POINTER; cPropertySets: INTEGER;
								rgPropertySets: POINTER): INTEGER is
		external
			"C [macro <oledb.h>] (IDBProperties*, ULONG , DBPROPSET* ): EIF_INTEGER"
		alias
			"IDBProperties_SetProperties"
		end

	IDBInitialize_Initialize(This: POINTER): INTEGER is
		external
			"C blocking signature (IDBInitialize*): EIF_INTEGER use <oledb.h>"
		alias
			"IDBInitialize_Initialize"
		end

	IDBInitialize_Uninitialize(This: POINTER): INTEGER is
		external
			"C [macro <oledb.h>] (IDBInitialize*): EIF_INTEGER"
		alias
			"IDBInitialize_Uninitialize"
		end

	IDataInitialize_GetDataSource(This: POINTER; pUnkOuter: POINTER; dwClsCtx: INTEGER;pwszInitializationString: POINTER;riid,ppDataSource: POINTER): INTEGER is
		external
			"C [macro <MSDASC.H>] (IDataInitialize*, IUnknown *,DWORD ,LPCOLESTR ,REFIID ,IUnknown **): EIF_INTEGER"
		alias
			"IDataInitialize_GetDataSource"
		end

	IDataInitialize_CreateDBInstance(This: POINTER;
									 clsidProvider: POINTER;
									 pUnkOuter: POINTER;
									 dwClsCtx: INTEGER;
									 pwszReserved: POINTER;
									 riid: POINTER;
									 ppDataSource: POINTER): INTEGER is
		external
			"C [macro <MSDASC.H>] (IDataInitialize*, REFCLSID ,   IUnknown * ,   DWORD,   LPOLESTR,   REFIID ,   IUnknown **): EIF_INTEGER"
		alias
			"IDataInitialize_CreateDBInstance"
		end

end

