note
	description: "Wrapper around DBPARAMBINDINFO."
	author: "Pierre Nallet"
	last_editor: "$Author: rj $"
	date: "$Date: 2014/09/05 $"
	revision: "$Revision: #23 $"
	path : "$File: //rose/source/kernel/infrastructure/persistence/windows/classic/oledb_dbparambindinfo.e $"

class
	OLEDB_DBPARAMBINDINFO [MM-> MEMORY_MODEL create default_create end]

inherit
	COM_SMART_STRUCTURE [MM]
		rename
			make as com_smart_make
		end
	COM_STRING_FUNCTIONS
		undefine
			is_equal,
			copy
		end
	OLEDB_DBPARAMFLAGS_CONSTANTS
		undefine
			is_equal,
			copy
		end
	OLEDB_COMMAND_INTERFACES
		undefine
			is_equal,
			copy
		end
	INTERNAL
		undefine
			is_equal,
			copy
		end
	OLEDB_SIZEOF
		undefine
			is_equal,
			copy
		end

create
	make

feature {NONE} -- Initialization
	make(a_command: OLEDB_COMMAND[OLEDB_DYNAMIC_PARAMETER_ACCESSOR, OLEDB_ROWSET]; a_parameter_template: ARRAY[STRING]) is
		local
			hr: INTEGER
			param_p: POINTER
			i, j: INTEGER
			ordinals: ARRAY [POINTER]	-- The size of POINTER matches the size of DB_UPARAMS as the platform change between 32- and 64-bit.  Yes, a hack.
			ordinals_: ANY
			cwp: POINTER
			l_parameter_template: STRING
			l_max_string_size: INTEGER
			l_parameter_template_count: INTEGER
		do
			set (default_pointer)
			if a_parameter_template /= Void then
				param_p := calloc (a_parameter_template.count, structure_size)
				create strs.make (0, a_parameter_template.count - 1)
				create ordinals.make (0, a_parameter_template.count - 1)
				from
					i := 0		-- For the parameter being built, always advances by 1.
					j := 1		-- For the parameter_template,  sometimes advances by 2 instead of by 1.
				until
					j > a_parameter_template.upper
				loop
					l_parameter_template_count := l_parameter_template_count + 1
					set (param_p + i * structure_size)
					set_pwszDataSourceType (default_pointer)
					ordinals.put (integer_to_pointer ( i + 1), i)
					l_parameter_template := clone(a_parameter_template.item (j))
					l_parameter_template.to_upper
					if l_parameter_template.is_equal("DBTYPE_I1") then
						set_param_bind_info(i, "DBTYPE_I1", 1, 3)
					elseif l_parameter_template.is_equal("DBTYPE_I2") then
						set_param_bind_info(i, "DBTYPE_I2", 2, 5)
					elseif l_parameter_template.is_equal("INTEGER") or l_parameter_template.is_equal("DBTYPE_I4")
						or l_parameter_template.is_equal("INTEGER_32") then
						set_param_bind_info(i, "DBTYPE_I4", sizeof_int, 10)
					elseif l_parameter_template.is_equal("INTEGER_64") or l_parameter_template.is_equal("DBTYPE_I8") then
						set_param_bind_info(i, "DBTYPE_I8", 8, 19)
					elseif l_parameter_template.is_equal("DBTYPE_UI1") then
						set_param_bind_info(i, "DBTYPE_UI1", 1, 3)
					elseif l_parameter_template.is_equal("DBTYPE_UI2") then
						set_param_bind_info(i, "DBTYPE_UI2", 2, 5)
					elseif l_parameter_template.is_equal("DBTYPE_UI4") then
						set_param_bind_info(i, "DBTYPE_UI4", sizeof_int, 5)
					elseif l_parameter_template.is_equal("DBTYPE_UI8") then
						set_param_bind_info(i, "DBTYPE_UI8", 8, 19)
					elseif type_string_conforms_to(l_parameter_template, "STRING") then
						if j + 1<=a_parameter_template.upper and then a_parameter_template.item(j + 1).is_integer then
							j := j + 1
							l_max_string_size := a_parameter_template.item(j).to_integer
						else
							l_max_string_size := 8000
						end
						set_param_bind_info(i, "DBTYPE_STR", l_max_string_size, (0).to_integer_8)
					elseif type_string_conforms_to(l_parameter_template, "STRING_8") then
						if j + 1<=a_parameter_template.upper and then a_parameter_template.item(j + 1).is_integer then
							j := j + 1
							l_max_string_size := a_parameter_template.item(j).to_integer
						else
							l_max_string_size := 8000
						end
						set_param_bind_info(i, "DBTYPE_STR", l_max_string_size, (0).to_integer_8)
					elseif type_string_conforms_to(l_parameter_template, "STRING_32") then
						if j + 1<=a_parameter_template.upper and then a_parameter_template.item(j + 1).is_integer then
							j := j + 1
							l_max_string_size := a_parameter_template.item(j).to_integer
						else
							l_max_string_size := 8000
						end
						set_param_bind_info(i, "DBTYPE_STR", l_max_string_size, (0).to_integer_8)
					elseif type_string_conforms_to(l_parameter_template, "ROSE_DATE") or
						   type_string_conforms_to(l_parameter_template, "ROSE_DATE") then
						set_param_bind_info(i, "DBTYPE_DATE", sizeof_double, 0)
					elseif l_parameter_template.is_equal("BOOLEAN") then
						set_param_bind_info(i, "DBTYPE_BOOL", 1, 0)
					elseif (l_parameter_template.is_equal("CHARACTER") or else l_parameter_template.is_equal("CHARACTER_8")) then
						set_param_bind_info(i, "DBTYPE_STR", 1, 0)
					elseif l_parameter_template.is_equal("REAL") then
						set_param_bind_info(i, "DBTYPE_R4", sizeof_float, 0)
					elseif l_parameter_template.is_equal("DOUBLE") then
						set_param_bind_info(i, "DBTYPE_R8", sizeof_double, 0)
					elseif l_parameter_template.is_equal ("DECIMAL") then
						set_param_bind_info (i, "DBTYPE_NUMERIC", 16, 0)
					else
						print("OLEDB_DBPARAMBINDINFO - Don't currently support " + l_parameter_template)
						(create {EXCEPTIONS}).raise("OLEDB_DBPARAMBINDINFO - Bad parameter template")
					end
					set_dwflags(DBPARAMFLAGS_ISINPUT)
					i := i + 1
					j := j + 1
				end
				hr  := a_command.Query_Interface (IID_ICommandWithParameters, $cwp)
				if hr >= 0 then
					ordinals_ := ordinals.to_c
					hr := a_command.ICommandWithParameters_SetParameterInfo(cwp, integer_to_pointer (l_parameter_template_count), $ordinals_, param_p)
					hr := a_command.iunknown_release(cwp)
				end
				from i := 0 until i >= a_parameter_template.count loop
					free (strs.item (i))
					i := i + 1
				end
				free (param_p)
			end
		end

feature -- Access

	structure_size: INTEGER is
		do
			Result := sizeofDBPARAMBINDINFO
		end

feature -- Basic Operations

	set_pwszDataSourceType (p: POINTER) is
		do
			set_pointer_value (p, offsetofpwszDataSourceType)
		end

	set_ulParamSize (p: INTEGER) is
		do
			set_Integer_value (p, offsetofulParamSize)
		end

	set_bPrecision (p: INTEGER_8) is
		do
			set_Integer_8_value (p, offsetofbPrecision)
		end

	set_dwFlags (p: INTEGER) is
		do
			set_Integer_value (p, offsetofdwFlags)
		end

feature {NONE} -- Implementation

	strs: ARRAY[POINTER]	-- The type names in Unicode.  The type names are string constants in the form DBTYPE_*.  See calls to 'set_param_bind_info'.

	set_param_bind_info(a_index: INTEGER; a_data_source_type: STRING; a_size : INTEGER; a_precision: INTEGER_8) is
		do
			strs.put (string_to_widechar (a_data_source_type), a_index)
			set_pwszDataSourceType(strs.item (a_index))
			set_ulParamSize(a_size)
			set_bPrecision(a_precision)
		end

	sizeofDBPARAMBINDINFO: INTEGER is
		external
			"C [macro <oledb.h>]"
		alias
			"sizeof(DBPARAMBINDINFO)"
		end

	offsetofpwszDataSourceType: INTEGER is
		external
			"C [macro <oledb.h>]"
		alias
			"offsetof (DBPARAMBINDINFO , pwszDataSourceType) "
		end

	offsetofulParamSize: INTEGER is
		external
			"C [macro <oledb.h>]"
		alias
			"offsetof (DBPARAMBINDINFO , ulParamSize) "
		end
	offsetofbPrecision: INTEGER is
		external
			"C [macro <oledb.h>]"
		alias
			"offsetof (DBPARAMBINDINFO , bPrecision) "
		end
	offsetofdwFlags: INTEGER is
		external
			"C [macro <oledb.h>]"
		alias
			"offsetof (DBPARAMBINDINFO , dwFlags) "
		end

	type_string_conforms_to(s1, s2: STRING): BOOLEAN is
		local
			l_type_1,l_type_2 : INTEGER
		do
			l_type_1 := dynamic_type_from_string (s1)
			l_type_2 := dynamic_type_from_string (s2)
			if l_type_1 >= 0 and then l_type_2 >= 0 then
				Result := type_conforms_to (l_type_1, l_type_2)
			end
		end

end
