indexing
	description: "[
                  Class that provides access to an oledb row and its column data.
				  This class gets used A LOT so be sure test the impact of any change on performance.
				  ]"
	author: "Mark Howard"
	last_editor: "$Author: rj $"
	date: "$Date: 2014/09/05 $"
	revision: "$Revision: #66 $"
	path: "$File: //rose/source/kernel/infrastructure/persistence/windows/classic2/oledb_dynamic_accessor.e $"

class
	OLEDB_DYNAMIC_ACCESSOR

inherit
	COM_MEMORY_FUNCTIONS
		undefine
			out
		end
	MEMORY_FUNCTIONS
		undefine
			out
		end
	OLEDB_ANY_ACCESSOR
		undefine
			out
		redefine
			Has_Output_Columns,
			free_record_memory
		end
	OLEDB_DBMEMOWNER_CONSTANTS
		undefine
			out
		end
	OLEDB_DBPART_CONSTANTS
		undefine
			out
		end
	OLEDB_BIT_OPERATIONS
		undefine
			out
		end
	OLEDB_DBACCESSOR_CONSTANTS
		undefine
			out
		end
	OLEDB_DBPARAMIO_CONSTANTS
		undefine
			out
		end
	COM_IUNKNOWN_INTERFACE
		undefine
			out
		end
	OLEDB_DBTYPE_CONSTANTS
		undefine
			out
		end
	OLEDB_DBSTATUS_CONSTANTS
		undefine
			out
		end
	OLEDB_IACCESSOR_INTERFACE
		undefine
			out
		end
	OLEDB_STGM_CONSTANTS
		undefine
			out
		end
	PLATFORM
		export
			{NONE} all
		undefine
			out
		end
	OLEDB_SIZEOF
		undefine
			out
		end

create
	default_create, make

feature -- Initialization
	make
			-- Initialize `Current'.
		do
		end

feature -- Operations
	close
			-- Close this accessor.
		do
			if buffer /= default_pointer then
				free (buffer)
				buffer := default_pointer
			end
			column_info := Void
		end

	old_free_record_memory
			-- Obsolete?
		local
			i: INTEGER
		do
			from i := 1 until i > column_count loop
				free_type (get_column_type (i), get_value (i))
				i := i + 1
			end
		end

	free_record_memory
			--|
		local
			i : INTEGER
		do
			if reference_fields /= void then
				from reference_fields.start until reference_fields.off loop
					i := reference_fields.item
					free_type (get_column_type (i), get_value (i))
					reference_fields.forth
				end
			end
		end

	free_type (type: INTEGER; ptr: POINTER)
			-- Free `ptr'.
		local
			hr: INTEGER
			pp: POINTER
		do
			if type = Dbtype_VARIANT then
				temp_variant.set (ptr)
				temp_variant.clear
			elseif type = dbtype_iunknown then
				memcpy ($pp, ptr, sizeof_iunknown)
				if pp /= default_pointer then
					hr := iunknown_release (pp)
				end
			end
		end

	add_reference_field (i : INTEGER)
			--|
		do
			if reference_fields = void then
				create reference_fields.make(0)
			end
			reference_fields.extend (i)
		end

	has_output_columns: BOOLEAN
			--|
		do
			Result := true
		end

	reference_fields : ARRAYED_LIST[INTEGER]
			--|

	temp_variant: COM_VARIANT
			--|
		once
			create Result
		end

	first_ordinal: INTEGER
		-- Ordinal of first column

	old_first_ordinal : INTEGER
			--|
		require
			ready: ready
		do
			Result :=  column_info.item (0).ordinal
		end

feature -- Mapping

	translatecolumnno (i: INTEGER): INTEGER
			--|
		require
			valid_index: valid_column_index (i)
		do
			Result := i - first_ordinal
		ensure
			large_enough:Result >= 0
			small_enough:Result < column_count
		end

feature -- Column Attributes

	column_count: INTEGER
			--|

	column_info: SPECIAL [OLEDB_COLUMN_INFO]
			--|

	get_column_type (i: INTEGER): INTEGER
			--|
		require
			valid_index: valid_column_index (i)
		do
			Result := column_info.item (TranslateColumnNo (i)).type
		end

	get_column_flags (i: INTEGER): INTEGER
			--|
		require
			valid_index: valid_column_index (i)
		do
			Result := column_info.item (TranslateColumnNo (i)).flags
		end

	get_column_size (i: INTEGER): INTEGER
			--|
		require
			valid_index: valid_column_index (i)
		do
			Result := column_info.item (TranslateColumnNo (i)).size
		end

	get_column_name (i: INTEGER): STRING
			--|
		require
			valid_index: valid_column_index (i)
		do
			Result := column_info.item (TranslateColumnNo (i)).name
		end

feature -- Get

	get_value (i: INTEGER): POINTER
			--|
		require
			valid_index: valid_column_index (i)
		local
			ii : INTEGER
			l_item: OLEDB_COLUMN_INFO
		do
			ii := i - first_ordinal
			l_item := column_info.item (ii)
			Result := buffer + l_item.offset
		end

	get_real_value (i: INTEGER): REAL
			--|
		require
			valid_index: valid_column_index (i)
		do
			memcpy ($Result, get_value (i), sizeofreal)
		end

	get_double_value (i: INTEGER): DOUBLE
			--|
		require
			valid_type: column_valid_for_type (i, Dbtype_R8)
		do
			memcpy ($Result, get_value (i), sizeofdouble)
		end

	get_integer_value (i: INTEGER): INTEGER
			--|
		require
			ready: ready
			valid_index: valid_column_index (i)
		do
			memcpy ($Result, get_value (i), sizeof_int)
		end

	get_natural_8_value (i: INTEGER): NATURAL_8
			-- Natural 8 value
		require
			ready: ready
			valid_index: valid_column_index (i)
		do
			memcpy ($Result, get_value (i), 1)
		end

	get_integer_64_value (i: INTEGER): INTEGER_64
			-- Integer 64 value
		require
			ready: ready
			valid_index: valid_column_index (i)
		do
			memcpy ($Result, get_value (i), sizeof_int_64)
		end

	get_pointer_value (i: INTEGER): POINTER
			--|
		do
			memcpy ($Result, get_value (i), Pointer_bytes)
		end

	get_string_value (i: INTEGER): STRING
			--|
		require
			valid_type: column_valid_for_type (i, Dbtype_STR)
		do
			create Result.make (0)
			Result.from_c_substring (get_value (i), 1, get_length (i))
		end

	get_uc_string_value (i: INTEGER): UC_STRING
			--|
		require
			valid_type: column_valid_for_type (i, Dbtype_WSTR)
		local
			s: STRING
		do
			create s.make_empty
			s.from_c_substring (get_value (i), 1, get_length (i))
			create Result.make_from_utf16 ("%/255/%/254/"+s)
		end

feature -- Set
	set_string_value (i: INTEGER; a_val: STRING)
			--|
		require
			valid_type: column_valid_for_type (i, Dbtype_STR)
		local
			a: ANY
			len: INTEGER
		do
			len := a_val.count.min (get_length (i))
			a := a_val.to_c
			memcpy (get_value (i), $a, len)
		end

	set_double_value (i: INTEGER; a_val: DOUBLE)
			--|
		require
			valid_type: column_valid_for_type (i, Dbtype_R8)
		do
			memcpy (get_value (i), $a_val, sizeofdouble)
		end

	set_real_value (i: INTEGER; a_val: REAL)
			--|
		require
			valid_type: column_valid_for_type (i, Dbtype_R4)
		do
			memcpy (get_value (i), $a_val, sizeof_float)
		end

	set_integer_value (i: INTEGER; a_val: INTEGER)
			--|
		require
			valid_type: column_valid_for_type (i, Dbtype_I4)
		do
			memcpy (get_value (i), $a_val, sizeof_int)
		end

	set_integer_64_value (i: INTEGER; a_val: INTEGER_64)
			--|
		require
			valid_type: column_valid_for_type (i, Dbtype_I8)
		do
			memcpy (get_value (i), $a_val, sizeof_int_64)
		end

	set_boolean_value (i: INTEGER; a_val: BOOLEAN)
			--|
		require
			valid_type: column_valid_for_type (i, Dbtype_BOOL)
		local
			ii: INTEGER
		do
			if (a_val) then
				ii := -1
			end
			memcpy (get_value (i), $ii, sizeof_db_bool)
		end

	set_timestamp_value (i: INTEGER; a_val: OLEDB_DBTIMESTAMP [MEMORY_MODEL])
			--|
		require
			valid_index: valid_column_index (i)
		do
			memcpy (get_value (i), a_val.item, sizeof_DBTIMESTAMP)
		end

	set_pointer_value (i: INTEGER; a_val: POINTER)
			--|
		require
			valid_index: valid_column_index (i)
		do
			memcpy (get_value (i), $a_val, sizeof_voidstar)
		end

	set_i2_value (i: INTEGER; a_val: INTEGER)
			--|
		require
			valid_type: column_valid_for_type (i, Dbtype_I2)
		do
			memcpy (get_value (i), $a_val, 2)
		end

	set_i1_value (i: INTEGER; a_val: INTEGER)
			--|
		require
			valid_type: column_valid_for_type (i, Dbtype_I1)
		do
			memcpy (get_value (i), $a_val, 1)
		end

	set_ui1_value (i: INTEGER; a_val: INTEGER)
			--|
		require
			valid_type: column_valid_for_type (i, Dbtype_UI1)
		do
			memcpy (get_value (i), $a_val, 1)
		end

	get_length (i: INTEGER): INTEGER
			--|
		require
			valid_index: valid_column_index (i)
		local
			ii: INTEGER
			l_ptr: POINTER
		do
			ii := translatecolumnno (i)
			l_ptr := add_offset (GetDataPtr (ii), column_info.item (ii).size)
			Result := int_content (l_ptr)
		end

	set_length (i: INTEGER; a_length: INTEGER)
			--|
		require
			valid_index: valid_column_index (i)
		local
			ii: INTEGER
			ptr: POINTER
		do
			ii := translatecolumnno (i)
			ptr:= add_offset (GetDataPtr (ii), column_info.item (ii).size)
			memcpy (ptr, $a_length, sizeof_int)
		end

feature -- Status
	column_valid_for_type (i, a_type: INTEGER): BOOLEAN
			--|
		do
			Result := valid_column_index (i) and get_column_type (i) = a_type
		end

	valid_column_index (i: INTEGER): BOOLEAN
			--|
		do
			Result := i >= first_ordinal and i < column_count + first_ordinal
		end

	get_status (i: INTEGER): INTEGER
			--|
		require
			ready: ready
			i_large_enough: i >= first_ordinal
			i_small_enough: i < column_count + first_ordinal
		local
			ii: INTEGER
			l_ptr: POINTER
		do
			ii := i - first_ordinal
			l_ptr := add_offset (getdataptr (ii), column_info.item (ii).size) + sizeof_dblength
			memcpy ($Result, l_ptr, sizeof_dbstatus)
		end

	status_is_ok (i: INTEGER): BOOLEAN
			--|
		do
			Result := get_status (i) = 0
		end

	set_status (i: INTEGER; status: INTEGER)
			--|
		require
			i_large_enough: i >= first_ordinal
			i_small_enough: i < column_count + first_ordinal
		local
			ii: INTEGER
			ptr: POINTER
		do
			ii := translatecolumnno (i)
		--	ptr:= add_offset (add_offset (GetDataPtr (ii), column_info.item (ii).size), sizeof_dblength)
			ptr := add_offset (getdataptr (ii), column_info.item (ii).size) + sizeof_dblength
			memcpy (ptr, $status, sizeof_dbstatus)
		end

	ready: BOOLEAN
			--|
		do
			Result := column_info /= Void
		end

	buffer_allocated: BOOLEAN
			--|
		do
			REsult := buffer /= default_pointer
		end

feature -- Implementation

	getdataptr (i: INTEGER): POINTER
			--|
		require
			buffer_allocated: buffer_allocated
			i_large_enough: i >= 0
			i_small_enough: i < column_count
		local
			l_item: OLEDB_COLUMN_INFO
		do
			l_item := column_info.item (i)
			Result := buffer + l_item.offset
		end

	get_column_info_ (irowset: POINTER): INTEGER
			--|
		local
			columnsinfo: POINTER
			hr: INTEGER
			cols: COM_SMART_POINTER [COM_MEMORY_MODEL]
			strs: POINTER
			db_col: OLEDB_DB_COLUMN_INFO[STACK_MEMORY_MODEL]
			col: OLEDB_COLUMN_INFO
			i: INTEGER
			l_column_info : ARRAY[OLEDB_COLUMN_INFO]
			l_column_count: INTEGER_64		-- Actually, we need 32 or 64 depending on the platform.
		do
			result := IUnknown_queryinterface (irowset, IID_IColumnsInfo, $columnsinfo)
			if result >= 0 then
				create cols.make
				result := IColumnsInfo_GetColumnInfo (columnsinfo, $l_column_count, cols.address, $strs)
				column_count := l_column_count.to_integer
				if result >= 0 then
		 			--create column_info.make (0, column_count - 1)
		 			create l_column_info.make (1, column_count)
					column_info := l_column_info.area
					from i :=0 until i >= column_count loop
						create db_col.set (cols.item + i * sizeofDBCOLUMNINFO)
						create col.make (db_col)
						column_info.put (col, i)
						i := i + 1
					end
					first_ordinal := column_info.item (0).ordinal
					--	create db_col.set (cols)
					--	CoTaskMemFree (cols)
					CoTaskMemFree (strs)
				end
				hr := iunknown_Release (columnsinfo)
			end
		end

	override: BOOLEAN
			--|

	haccessor: POINTER
			--|

	add_offset (a_current: POINTER; a_add: INTEGER): POINTER
			--|
		do
			Result := a_current + a_add + a_add \\ 4
		end

	add_offset_ (a_current, a_add: INTEGER): INTEGER
			--|
		do
			Result := a_current + a_add + a_add \\ 4
		end

	bind_columns (punk: POINTER): INTEGER
			--|
		local
			accessor: POINTER
			i: INTEGER
			offset: INTEGER
			length_offset: INTEGER
			status_offset: INTEGER
			columninfo: POINTER
			columninfo_array: POINTER
			string_buffer: POINTER
			pbinding: POINTER
			db_col: OLEDB_DB_COLUMN_INFO[STACK_MEMORY_MODEL ]
			col: OLEDB_COLUMN_INFO
			current_binding: OLEDB_DBBINDING[STACK_MEMORY_MODEL]
			hr: INTEGER
			dbobject: OLEDB_DBOBJECT[RUN_TIME_MEMORY_MODEL]
			dbobjects: LINKED_LIST [OLEDB_DBOBJECT[RUN_TIME_MEMORY_MODEL]]
			l_column_info : ARRAY[OLEDB_COLUMN_INFO]
			l_column_count: INTEGER_64		-- Actually, we need 32 or 64 depending on the platform.
		do
			create dbobjects.make
			Result := iunknown_queryinterface (punk, iid_iaccessor, $accessor)
			if Result >= 0 then
				if column_info = void then
					Result := iunknown_queryinterface (punk, iid_icolumnsinfo, $columninfo)
					if Result >= 0 then
						Result := icolumnsinfo_getcolumninfo (columninfo, $l_column_count, $columninfo_array, $string_buffer)
						column_count := l_column_count.to_integer
						if Result >= 0 then
							--create column_info.make (0, column_count - 1)
							create l_column_info.make (1,column_count)
							column_info := l_column_info.area
							from
								i := 0
							until
								i >= column_count
							loop
								create db_col.set (columninfo_array + i * sizeofdbcolumninfo)
								create col.make (db_col)
								column_info.put (col, i)
								i := i + 1
							end
							cotaskmemfree (columninfo_array)
							cotaskmemfree (string_buffer)
							first_ordinal := column_info.item (0).ordinal
						end
						hr := iunknown_release (columninfo)
					end
					override := False
				else
					override := True
				end
				if Result >= 0 then
					pbinding := calloc (column_count, sizeofdbbinding)
					create current_binding
					current_binding.set (pbinding)
					from
						i := 0
					until
						i >= column_count
					loop
						create dbobject.make
						dbobjects.extend (dbobject)
						if (column_info.item (i).type /= dbtype_variant and column_info.item (i).size > max_string_length) or column_info.item (i).type = dbtype_iunknown then
							dbobject.set_dwflags (STGM_READ)
							dbobject.set_iid (IID_ISequentialStream)
							column_info.item (i).set_type (Dbtype_IUNKNOWN)
							column_info.item (i).set_size (sizeof_iunknown)
						end
						if column_info.item (i).type = 0x4082 then
							column_info.item (i).set_type (dbtype_wstr)
						end
						if (column_info.item (i).type = dbtype_str) then
							column_info.item (i).set_size (column_info.item (i).size + 1)

						elseif (column_info.item (i).type = dbtype_wstr) then
							column_info.item (i).set_size (column_info.item (i).size * 2 + 2)
						elseif (column_info.item (i).type = Dbtype_NUMERIC) then
							--	column_info.item (i).set_type (dbtype_r8)
						end
						length_offset := add_offset_ (offset, column_info.item (i).size)
						status_offset := add_offset_ (length_offset, sizeof_dblength)
						bind (current_binding, column_info.item (i).ordinal, column_info.item (i).type, column_info.item (i).size,
							  column_info.item (i).precision, column_info.item (i).scale, dbparamio_notparam, offset, length_offset,
							  status_offset, dbobject.item)
						current_binding.set (current_binding.item + sizeofdbbinding)
						column_info.item (i).set_offset (offset)
						offset := add_offset_ (status_offset, sizeof_dbstatus)
						if column_info.item (i).type = Dbtype_VARIANT or column_info.item (i).type = dbtype_iunknown then
							--add_reference_field (i)
						end
						i := i + 1
					end
					if haccessor = default_pointer then
						allocateaccessormemory
					end
					buffer := calloc (offset, 1)
					buffer_length := offset
					if buffer = default_pointer then
					end
					current_binding.set (pbinding)
					Result := bind_entries (current_binding, column_count, $haccessor, offset, accessor)
					free (pbinding)
				end

				hr := iunknown_release (accessor)
			end
		ensure then
			valid_result: Result >= 0 implies haccessor /= db_invalid_haccessor
		end

	buffer_length: INTEGER
			--|

	max_string_length: INTEGER is 8000
			--|

	bind_entries (binding: OLEDB_DBBINDING[MEMORY_MODEL]; ncolumns: INTEGER; pHAccessor: POINTER; size: INTEGER; pAccessor: POINTER): INTEGER
			--|
		local
			AccessorFlags: INTEGER
			status_ptr, current_status: POINTER
			i: INTEGER
		do
			if binding.get_eParamIO  = DBPARAMIO_NOTPARAM then
				AccessorFlags := DBACCESSOR_ROWDATA
			else
				AccessorFlags := DBACCESSOR_PARAMETERDATA
			end
			status_ptr := calloc (ncolumns, sizeof_DBBINDSTATUS)
			Result := IAccessor_CreateAccessor (pAccessor, AccessorFlags, nColumns,
											   binding.item, size, pHAccessor, status_ptr)
			if Result < 0 then
				current_status := status_ptr
				from i := 1 until i > ncolumns loop
					if int_content (current_status) /= DBSTATUS_S_OK then
						print ("invalid binding, entry ")
						print (i)
						print (" status ")
						print (int_content (current_status))
						print ("%N")
					end
					current_status := current_status + sizeof_DBBINDSTATUS
					i := i + 1
				end
			end
			free (status_ptr)
			-- delete bindings...

		end

	AllocateAccessorMemory
			--|
		do
		end

	release_accessor (p: POINTER): INTEGER
			--|
		local
			pAccessor: POINTER
			hr: INTEGER
			refcount: INTEGER
		do
			Result := iunknown_queryinterface (p, IID_IAccessor, $pAccessor)
			if Result >= 0 then
				Result := iaccessor_ReleaseAccessor (pAccessor, haccessor, $refcount)
				hr := iunknown_release (pAccessor)
				haccessor := default_pointer
			end
		end

	bind (binding: OLEDB_DBBINDING[MEMORY_MODEL]; ordinal, type, length, precision, scale, paramIO, dataoffset, lengthoffset, statusoffset: INTEGER;
		  pobj: POINTER)
			--|
		require
			binding_not_void: binding /= VOid
		local
			dwPart: INTEGER
		do
			binding.set_dwMemOwner (DBMEMOWNER_CLIENTOWNED)	-- for now
			binding.set_pobject (pobj)
			binding.set_eparamio (paramIO)
			binding.set_iOrdinal (ordinal)
			binding.set_wType (type)
			binding.set_bPrecision (precision)
			binding.set_bScale (scale)
			binding.set_dwFlags (0)
			binding.set_obValue (dataoffset)
			binding.set_obLength (0)
			binding.set_obStatus (0)
			binding.set_pTypeInfo (default_pointer)
			binding.set_pBindExt (default_pointer)
			binding.set_cbMaxLen (length)
			dwPart := 	DBPART_VALUE
			if lengthoffset/= 0 then
				dwPart := bor (dwPart, DBPART_LENGTH)
				binding.set_obLength (LengthOffset )
			end
			if StatusOffset /= 0 then
				dwPart := bor (dwPart, DBPART_STATUS)
				binding.set_obStatus (StatusOffset )
			end
			binding.set_dwpart (dwpart)
		end

	buffer: POINTER
			--|

feature -- Externals

	sizeof_DBTIMESTAMP: INTEGER
			--|
		external
			"C [macro <oledb.h>]"
		alias
			"sizeof (DBTIMESTAMP)"
		end

	sizeofreal: INTEGER
			--|
		external
			"C [macro <oledb.h>]"
		alias
			"sizeof (float)"
		end

	sizeofdouble: INTEGER
			--|
		external
			"C [macro <oledb.h>]"
		alias
			"sizeof (double)"
		end

	IID_IAccessor: POINTER
			--|
		external
			"C [macro <oledb.h>]"
		alias
			"&IID_IAccessor"
		end

	sizeofDBCOLUMNINFO: INTEGER
			--|
		external
			"C [macro <oledb.h>]"
		alias
			"sizeof (DBCOLUMNINFO)"
		end

	sizeofDBBINDING: INTEGER
			--|
		external
			"C [macro <oledb.h>]"
		alias
			"sizeof (DBBINDING)"
		end

	IColumnsInfo_GetColumnInfo (This: POINTER; pcColumns: POINTER; prgInfo: POINTER; ppStringsBuffer: POINTER ): INTEGER
			-- Note that DBORDINAL is 32 or 64-bit depending on the platform.
		external
			"C [macro <oledb.h>] (IColumnsInfo*, DBORDINAL*, DBCOLUMNINFO**, OLECHAR**): EIF_INTEGER"
		alias
			"IColumnsInfo_GetColumnInfo"
		end

	IColumnsInfo_MapColumnIDs (This: POINTER; cColumnIDs: INTEGER; rgColumnIDs: POINTER; rgColumns: POINTER): INTEGER
			--|
		external
			"C [macro <oledb.h>] (IColumnsInfo*, ULONG, DBID*, ULONG*): EIF_INTEGER"
		alias
			"IColumnsInfo_MapColumnIDs"
		end

	IID_IColumnsInfo: POINTER
			--|
		external
			"C [macro <oledb.h>]"
		alias
			"&IID_IColumnsInfo"
		end

	int_content (pp: POINTER): INTEGER
			--|
		external
			"C [macro <oledb.h>] (EIF_POINTER): EIF_INTEGER"
		alias
			"* (EIF_INTEGER*)"
		end

	IID_ISequentialStream: POINTER
			--|
		external
			"C [macro <oledb.h>]"
		alias
			"&IID_ISequentialStream"
		end

feature -- Display
	out: STRING
			-- <Precursor>
		local
			i: INTEGER
		do
			Result := column_count.out
			Result.append ("%N")
			from i := 1 until i > column_count loop
				Result.append (column_info.item (i - 1).name)
				Result.append (" ")
				Result.append (column_info.item (i - 1).type.out)
				Result.append (" ")
				Result.append (get_length (i).out)
				Result.append (" ")
				Result.append (get_integer_value (i).out)
				Result.append (", %N")
				i := i + 1
			end
		end

invariant
	column_count_positive: column_count >= 0
end

