note

	description: "[
				  Class providing an interface to an OLEDB dynamic parameter accessor
				  ]"
	author: "Mark Howard"
	last_editor: "$Author: rj $"
	date: "$Date: 2014/12/12 $"
	revision: "$Revision: #55 $"
	path: "$File: //rose/source/kernel/infrastructure/persistence/windows/classic2/oledb_dynamic_parameter_accessor.e $"

class

	OLEDB_DYNAMIC_PARAMETER_ACCESSOR

inherit

	OLEDB_DYNAMIC_ACCESSOR
		redefine
			bind_parameters,
			has_parameters,
			get_parameter_buffer,
			get_parameter_handle,
			out,
			close_parameter,
			make
		end
	OLEDB_COMMAND_INTERFACES
		export
			{NONE} all
		undefine
			out
		end
	COM_STRING_FUNCTIONS
		export
			{NONE} all
		undefine
			out
		end
	OLEDB_DBPARAMFLAGS_CONSTANTS
		export
			{NONE} all
		undefine
			out
		end
	MEMORY
		rename
			free as eiffel_free
		export
			{NONE} all
		undefine
			out
		redefine
			dispose
		end

create

    make

feature {NONE} -- Creation

	make is
			-- Create the accessor
		do
			debug("accessor")
				print ("make%N")
			end
		end

feature -- Initialization

	dummy: POINTER
			--just to make sure this is not retrieved from storable.
			-- change the name periodically

	dispose, close_parameter is
			-- Cleanup
		do
			if 	m_pParameterBuffer /= default_pointer then
				free (m_pParameterBuffer)
				m_pParameterBuffer := default_pointer
				parameter_count := 0
				debug ("accessor")
					print ("dispose, close_parameter%N")
				end
			end
		end

feature -- Access

	get_parameter_buffer: POINTER is
			-- Access to the parameter buffer
		do
			Result := m_pParameterBuffer
		end

	get_parameter_handle: POINTER is
			-- Access to the parameter accessor
		do
			Result := HParameterAccessor
		end

feature -- Status

	has_parameter_buffer: BOOLEAN is
			-- Does this accessor have a valid buffer?
		do
			Result := m_pparameterBuffer /= default_pointer
		end

	has_parameters: BOOLEAN is
			-- Does this access have parameters?
		do
			Result := True
		end

feature -- Attribute access

	get_parameter_Type (i: INTEGER): INTEGER is
			-- Type of parameter at index i
		require
			i_large_enough: i >= 1
			i_small_enough: i <= parameter_count
		do
			Result := parameter_entries.item (i-1).Type;
		end

	get_parameter_length (i: INTEGER): INTEGER is
			-- Length of parameter at index i
		require
			has_parameter_buffer: has_parameter_buffer
			i_large_enough: i >= 1
			i_small_enough: i <= parameter_count
		local
			l_pointer: POINTER
		do
			l_pointer := m_pParameterBuffer + parameter_entries.item (i - 1).oblength
			Result := int_content(l_pointer)
		end

	get_parameter_status (i: INTEGER): INTEGER is
			-- Status of parameter at i
		require
			has_parameter_buffer: has_parameter_buffer
			i_large_enough: i >= 1
			i_small_enough: i <= parameter_count
		local
			l_pointer: POINTER
		do
			l_pointer:= m_pParameterBuffer + parameter_entries.item (i - 1).obstatus
			Result := int_content(l_pointer)
		end

	get_parameter_scale (i: INTEGER): INTEGER is
			-- Scale of parameter at index i
		require
			has_parameter_buffer: has_parameter_buffer
			i_large_enough: i >= 1
			i_small_enough: i <= parameter_count
		local
			l_pointer: POINTER
		do
			l_pointer:= m_pParameterBuffer + parameter_entries.item (i - 1).scale
			Result := int_content(l_pointer)
		end

	get_parameter_precision (i: INTEGER): INTEGER is
			-- Precision of parameter at index i
		require
			has_parameter_buffer: has_parameter_buffer
			i_large_enough: i >= 1
			i_small_enough: i <= parameter_count
		local
			l_pointer: POINTER
		do
			l_pointer:= m_pParameterBuffer + parameter_entries.item (i - 1).precision
			Result := int_content(l_pointer)
		end

	get_parameter_name (i: INTEGER): STRING is
			-- Name of parameter at index i
		require
			i_large_enough: i >= 1
			i_small_enough: i <= parameter_count
		do
			Result := m_ppParamName.item (i - 1)
		end

	get_parameter(i: INTEGER): POINTER is
			-- Access pointer to parameter at index i
		require
			has_parameter_buffer: has_parameter_buffer
			i_large_enough: i >= 1
			i_small_enough: i <= parameter_count
		do
			Result := m_pParameterBuffer + parameter_entries.item (i-1).obvalue
		end

feature -- Modify parameter attributes

	set_parameter_length (i: INTEGER; length: INTEGER) is
			-- Set the length of parameter at index i
		require
			has_parameter_buffer: has_parameter_buffer
			i_large_enough: i >= 1
			i_small_enough: i <= parameter_count
		local
			l_pointer: POINTER
			l_length_64: INTEGER_64
		do
			l_pointer:= m_pParameterBuffer + parameter_entries.item (i - 1).oblength
			if sizeof_dblength=4
			then
				memcpy (l_pointer, $length, sizeof_dblength)
			else
				l_length_64 := length
				memcpy (l_pointer, $l_length_64, sizeof_dblength)
			end
		ensure
			set: get_parameter_length (i) = length
		end

	set_parameter_status (i: INTEGER; s: INTEGER) is
			-- Set the status of parameter at index i
		require
			has_parameter_buffer: has_parameter_buffer
			i_large_enough: i >= 1
			i_small_enough: i <= parameter_count
		local
			l_pointer: POINTER
		do
			l_pointer:= m_pParameterBuffer + parameter_entries.item (i - 1).obstatus
			memcpy (l_pointer, $s, sizeof_DBSTATUS)
		ensure
			set: get_parameter_status (i) = s
		end

feature -- Parameter value access

	double_parameter (i: INTEGER): DOUBLE is
			-- Double value from parameter at index i
		require
			has_parameter_buffer: has_parameter_buffer
			i_large_enough: i >= 1
			i_small_enough: i <= parameter_count
		do
			memcpy ($Result, get_parameter (i), sizeof_double)
  		end

	real_parameter (i: INTEGER): REAL is
			-- Real value from parameter at index i
		require
			has_parameter_buffer: has_parameter_buffer
			i_large_enough: i >= 1
			i_small_enough: i <= parameter_count
		do
			memcpy ($Result, get_parameter (i), sizeof_float)
  		end

	integer_parameter (i: INTEGER): INTEGER is
			-- Integer value from parameter at index i
		require
			has_parameter_buffer: has_parameter_buffer
			i_large_enough: i >= 1
			i_small_enough: i <= parameter_count
		do
			memcpy ($Result, get_parameter (i), sizeof_int)
		end

	date_parameter (i: INTEGER): ROSE_DATE is
			-- Date value from parameter at index i
		require
			has_parameter_buffer: has_parameter_buffer
			i_large_enough: i >= 1
			i_small_enough: i <= parameter_count
		do
			timestamp_reader.set (get_parameter (i))
			create Result.make_unchecked (timestamp_reader.get_year, timestamp_reader.get_month, timestamp_reader.get_day,
										  timestamp_reader.get_hour, timestamp_reader.get_minute, timestamp_reader.get_second + timestamp_reader.get_fraction / 1_000_000_000)
		end

	string_parameter (i: INTEGER): STRING is
			-- String value from parameter at index i
		require
			has_parameter_buffer: has_parameter_buffer
			i_large_enough: i >= 1
			i_small_enough: i <= parameter_count
		local
			a: ANY
			len: INTEGER
			l_pointer:POINTER
		do
			len := get_parameter_length(i)
			create Result.make (len)
			Result.fill_blank
			l_pointer := get_parameter (i)
			a := Result.to_c
			memcpy ($a, l_pointer , len)
		end

feature -- Parameter set

	set_integer_parameter (i: INTEGER; val: INTEGER) is
			-- Set the integer parameter value at index i to val
		require
			has_parameter_buffer: has_parameter_buffer
			i_large_enough: i >= 1
			i_small_enough: i <= parameter_count
		do
			memcpy (get_parameter(i), $val, sizeof_int)
		end

	set_integer_64_parameter (i: INTEGER; val: INTEGER_64) is
			-- Set the integer_64 parameter value at index i to val
		require
			has_parameter_buffer: has_parameter_buffer
			i_large_enough: i >= 1
			i_small_enough: i <= parameter_count
		do
			memcpy (get_parameter(i), $val, 8)
		end

	set_pointer_parameter (i: INTEGER; val: POINTER) is
			-- Set the pointer parameter value at index i to val
		require
			has_parameter_buffer: has_parameter_buffer
			i_large_enough: i >= 1
			i_small_enough: i <= parameter_count
		do
			memcpy (get_parameter(i), $val, sizeof_voidstar)
		end

	set_double_parameter (i: INTEGER; val: DOUBLE) is
			-- Set the double parameter value at index i to val
		require
			has_parameter_buffer: has_parameter_buffer
			i_large_enough: i >= 1
			i_small_enough: i <= parameter_count
		do
			inspect get_parameter_Type(i)
			when dbtype_r8  then
				memcpy (get_parameter(i), $val, sizeof_double)
			when dbtype_numeric then
				data_convert.double_to_numeric (val, get_parameter(i), parameter_entries.item(i-1).precision, parameter_entries.item (i-1).scale)
			else
				print ("ni" + get_parameter_Type(i).out)
			end
		end

	set_decimal_parameter (i: INTEGER; val: RP_DECIMAL) is
			-- Set the decimal parameter value at index i to val
		require
			has_parameter_buffer: has_parameter_buffer
			i_large_enough: i >= 1
			i_small_enough: i <= parameter_count
		do
			inspect get_parameter_Type(i)
			when dbtype_r8  then
				set_double_parameter (i, val.to_double)
			when dbtype_numeric then
				data_convert.decimal_to_numeric (val, get_parameter(i), parameter_entries.item(i-1).precision, parameter_entries.item (i-1).scale)
			else
				print ("ni" + get_parameter_Type(i).out)
			end
		end

	set_real_parameter(i: INTEGER; val: REAL) is
			-- Set the real parameter value at index i to val
		require
			has_parameter_buffer: has_parameter_buffer
			i_large_enough: i >= 1
			i_small_enough: i <= parameter_count
		do
			memcpy (get_parameter(i), $val, sizeof_float)
		end

	set_string_32_parameter (i: INTEGER; a_value: STRING_32) is
			-- Set the i th parameter value to a_value
		require
			has_parameter_buffer: has_parameter_buffer
			i_large_enough: i >= 1
			i_small_enough: i <= parameter_count
		local
			type: INTEGER
			len: INTEGER
			l_pointer, l_pointer2: POINTER
			l_c_string_32: C_STRING_32
			l_exceptions: EXCEPTIONS
		do
			type := get_parameter_type (i)
			if (type = dbtype_wstr) then
				len := (a_value.count).min ((parameter_entries.item (i - 1).max_length - 1))
				l_pointer := get_parameter(i)
				create l_c_string_32.make (a_value)
				l_pointer2 := l_c_string_32.item
				memcpy (l_pointer, l_pointer2, len * 2 )
				set_parameter_length (i, len * 2 )
			--	l_pointer2.memory_free
			else
				create l_exceptions
				l_exceptions.raise("Unknown type (" + type.out + ")")
			end

		end

	set_string_parameter(i: INTEGER; val: STRING) is
			-- Set the string parameter value at index i to val
		require
			has_parameter_buffer: has_parameter_buffer
			i_large_enough: i >= 1
			i_small_enough: i <= parameter_count
		local
			a: ANY
			len: INTEGER
			l_pointer, l_pointer2: POINTER
			type: INTEGER
			l_exceptions: EXCEPTIONS
		do
			-- Although it may seem strange to equate Dbtype_str and DBTYPE_BYTES, this actually happens in
			-- REUTERS.  REUTERS uses case sensitive attributes for the RIC.  The select looks like this:
			--		select * from entity_attribute where ... CONVERT(BINARY, attribute_value)=CONVERT(BINARY, ?)
			-- The passed value (?) is a STRING but 'get_parameter_type' returns DBTYPE_BYTES.  The trace:
			--		... Convert([entity_attribute].[attribute_value])=[@P1]
			-- indicates to me that WE are responsible for the conversion since @P1 is expected to already
			-- be in BINARY.  So essentially, I am performing the CONVERT(BINARY, ?) right here.  This may
			-- not be the best solution but it is a solution.  RJ.
			type := get_parameter_type (i)
			if (type = Dbtype_str or type=DBTYPE_BYTES) then
				len := val.count.min(parameter_entries.item( i - 1).max_length - 1)
				l_pointer := get_parameter(i)
				a := val.to_c
				memcpy (l_pointer , $a, len + 1)
				set_parameter_length(i, len)
			elseif (type = dbtype_wstr) then
				len := (val.count).min ((parameter_entries.item (i - 1).max_length - 1))
				l_pointer := get_parameter(i)
				l_pointer2 := string_to_widechar (val)
				memcpy (l_pointer, l_pointer2, len * 2 + 2)
				set_parameter_length (i, len * 2 + 2)
				l_pointer2.memory_free
			else
				create l_exceptions
				l_exceptions.raise("Unknown type (" + type.out + ")")
			end
		ensure
			set: string_parameter (i) ~ val.substring(1, get_parameter_length(i))
		end

	set_timestamp_parameter(
		i: INTEGER; a_val: OLEDB_DBTIMESTAMP [MEMORY_MODEL])
			--|
		require
			has_parameter_buffer: has_parameter_buffer
			i_large_enough: i >= 1
			i_small_enough: i <= parameter_count
		local
			type: INTEGER
			l_exceptions: EXCEPTIONS
		do
			type := get_parameter_type (i)
			if type = DBTYPE_DBTIMESTAMP then
				memcpy (get_parameter (i), a_val.item, sizeof_DBTIMESTAMP)
				set_parameter_length (i, sizeof_DBTIMESTAMP)
			else
				create l_exceptions
				l_exceptions.raise("Unknown type (" + type.out + ")")
			end
		end

feature -- Binding

	bind_parameters (pcommand: POINTER): INTEGER is
			-- Bind this command's parameters
		local
			hr: INTEGER
			i: INTEGER
			l_paccessor: POINTER
			l_pcommandwithparameters: POINTER
			l_ulparams_64: INTEGER_64			-- Actually, we need 32 or 64 depending on the platform.
			l_ulparams: INTEGER
			l_pparaminfo: POINTER
			l_paraminfo: OLEDB_DBPARAMINFO
			l_pnamesbuffer: POINTER
			l_pcurrent: OLEDB_DBBINDING[STACK_MEMORY_MODEL]
			l_noffset: INTEGER
			l_binding: OLEDB_BINDING
			l_length_offset: INTEGER
			l_status_offset: INTEGER
			l_m_pparameterentry: POINTER
			l_dbobject: OLEDB_DBOBJECT[RUN_TIME_MEMORY_MODEL]
			l_dbobjects: LINKED_LIST [OLEDB_DBOBJECT[RUN_TIME_MEMORY_MODEL]]
		do
			hr := iunknown_queryinterface(pcommand, IID_IAccessor, $l_paccessor)
			create l_dbobjects.make
			if hr >= 0 then
				hr := iunknown_queryinterface(pcommand, IID_ICommandWithParameters, $l_pcommandwithparameters)
				if hr >= 0 then
					hr := icommandwithparameters_getparameterinfo(l_pcommandwithparameters, $l_ulparams_64, $l_pparaminfo, $l_pnamesbuffer)
					l_ulparams := l_ulparams_64.to_integer
					if hr >= 0 then
						create l_pcurrent
						l_m_pparameterentry := calloc (l_ulparams + 1, l_pcurrent.structure_size)
						create m_ppParamName.make (0, l_ulparams - 1)
						create parameter_entries.make (0, l_ulparams - 1)
						l_pcurrent.set (l_m_pparameterentry)
						create l_paraminfo
						from i := 0 until i >= l_ulparams loop
							l_pcurrent.set_eparamio(0)
							l_paraminfo.set (l_pparaminfo + (i * l_paraminfo.structure_size))
							if band(l_paraminfo.get_dwflags , DBPARAMFLAGS_ISINPUT) /= 0 then
								l_pcurrent.set_eparamio(bor (DBPARAMIO_INPUT, l_pcurrent.get_eparamio))
							end
							if band(l_paraminfo.get_dwflags , DBPARAMFLAGS_ISOUTPUT) /= 0 then
								l_pcurrent.set_eparamio(bor (DBPARAMIO_OUTPUT, l_pcurrent.get_eparamio))
							end
							create l_dbobject.make
							l_dbobjects.extend (l_dbobject)

							if (l_paraminfo.get_wtype = dbtype_bytes) then
								l_paraminfo.set_wtype (bor (dbtype_bytes, dbtype_byref))
								l_paraminfo.set_ulparamsize (sizeof_iunknown)
							elseif
								l_paraminfo.get_ulparamsize > 16384 or l_paraminfo.get_wtype = dbtype_iunknown then
								l_dbobject.set_dwflags (STGM_READ)
								l_dbobject.set_iid (IID_ISequentialStream)
								l_paraminfo.set_wtype (DBTYPE_IUNKNOWN)
								l_paraminfo.set_ulparamsize (sizeof_iunknown)
							end

							if l_paraminfo.get_ulparamsize > 16384 or l_paraminfo.get_wtype = dbtype_iunknown then
								l_dbobject.set_dwflags (STGM_READ)
								l_dbobject.set_iid (IID_ISequentialStream)
								l_paraminfo.set_wtype (DBTYPE_IUNKNOWN)
								l_paraminfo.set_ulparamsize (sizeof_iunknown)
							end
							if l_paraminfo.get_wtype = DBTYPE_STR or else l_paraminfo.get_wtype = DBTYPE_wSTR then
								l_paraminfo.set_ulparamsize (l_paraminfo.get_ulparamsize + 1)
							end
							l_length_offset := add_offset_(l_noffset, l_paraminfo.get_ulparamsize)
							if l_paraminfo.get_wtype = dbtype_wstr then
								l_length_offset := add_offset_(l_length_offset, l_paraminfo.get_ulparamsize)
							end
							l_status_offset := add_offset_(l_length_offset, sizeof_dblength)
							bind (l_pcurrent, l_paraminfo.get_iordinal, l_paraminfo.get_wtype, l_paraminfo.get_ulparamsize, l_paraminfo.get_bprecision,
								  l_paraminfo.get_bscale, l_pcurrent.get_eparamio, l_noffset, l_length_offset ,l_status_offset, l_dbobject.item)
							m_ppparamname.put (widechar_to_string (l_pnamesbuffer), i)
							create l_binding.make (l_pcurrent)
							parameter_entries.put(l_binding, i)
							l_pcurrent.set (l_pcurrent.item + l_pcurrent.structure_size)
							l_noffset := add_offset_(l_status_offset, sizeof_dbstatus)
							i := i + 1
						end
						m_pParameterBuffer := calloc (1, l_noffset + 1)
						m_nParameterBufferSize := l_noffset
						parameter_count := l_ulparams
						debug ("accessor")
							print ("set: m_pparameterbuffer = " + m_pparameterbuffer.out + ", paramameter_count = " +
								   parameter_count.out + " column_count " + column_count.out +  "%N")
						end
						l_pcurrent.set (l_m_pparameterentry)
						if l_ulparams > 0
						then
							Result := bind_entries (l_pcurrent, l_ulparams, $HParameterAccessor, l_noffset, l_paccessor)
						end
						cotaskmemfree (l_pparaminfo)
						cotaskmemfree (l_pnamesbuffer)
						free (l_m_pparameterentry)
					end
					hr := IUnknown_release (l_pcommandwithparameters)
				end
				hr := IUnknown_release (l_paccessor)
			end
		ensure then
			valid_buffer: parameter_count > 0 implies has_parameter_buffer
		end

feature -- Attributes

	parameter_entries: ARRAY [OLEDB_BINDING]
			-- Array of binding structures
	m_ppParamName: ARRAY [STRING]
			-- Array of parameter names in the parameter entries
	m_pParameterBuffer: POINTER
			-- Pointer to the parameter buffer
	m_nParameterBufferSize: INTEGER
			-- Size of the parameter buffer
	HParameterAccessor: POINTER
			-- Pointer to the parameter accessor
	parameter_count: INTEGER
			-- Count of parameters

feature -- Display

	out: STRING is
			-- Print out the parameter values
		local
			i: INTEGER
		do
			Result := Precursor {OLEDB_DYNAMIC_ACCESSOR}
			Result.append (parameter_count.out)
			Result.append ("%N")
			from i := 1 until i > parameter_count loop
				Result.append (i.out)
				Result.append (" ")
				Result.append (parameter_entries.item (i - 1).type.out)
				Result.append (" ")
				Result.append (get_parameter_length (i).out)
				Result.append (" ")
				Result.append (integer_parameter (i).out)
				Result.append (", %N")
				i := i + 1
			end
		end

feature {NONE} -- Helpers

	data_convert: OLEDB_DATA_CONVERT is
			-- Utility to do value conversion
		once
			create Result.make
		end

	timestamp_reader: OLEDB_DBTIMESTAMP [STACK_MEMORY_MODEL] is
			-- An instance of the OLEDB timestamp reader object to facilite reading into a DATE object instance
		once
			create Result.make
		end

invariant

	valid_buffer: parameter_count > 0 implies has_parameter_buffer

end
