indexing
	description: "Object that provides the OLEDB implementation for the SQL_EXECUTOR base class"
	author: "Pierre Nallet"
	last_editor: "$Author: rj $"
	date: "$Date: 2009/08/10 $"
	revision: "$Revision: #24 $"
	path: "$File: //rose/source/kernel/infrastructure/persistence/windows/classic2/oledb_executor.e $"

class
	OLEDB_EXECUTOR

inherit
	SQL_EXECUTOR
		rename
			execute as old_execute,
			release as sql_release,
			make as sql_make
		redefine
			in_sqlda,
			out_sqlda,
			data_server
		select
			sql_make
		end
	OLEDB_STATEMENT
		rename
			make as statement_make
		undefine
			old_execute,
			clear_error
		redefine
			statement_make,
			data_server,
			in_sqlda,
			out_sqlda
		end
	OLEDB_COMMAND [OLEDB_DYNAMIC_PARAMETER_ACCESSOR, OLEDB_ROWSET]
		rename
			execute as command_execute
		redefine
			default_command_propset
		end
	OLEDB_DBPARAMFLAGS_CONSTANTS
		export
			{NONE} all
		end

create
	make

feature {NONE} -- Initialization

	make (a_connection: OLEDB_ANY_DATA_SERVER; a_statement: STRING; a_parameter_template: ARRAY [STRING]) is
			-- Create the executor
		require
			valid_connection: a_connection /= Void
			valid_statement: a_statement /= Void
		local
			l_hr: INTEGER
			l_oledb_dbparambindinfo: OLEDB_DBPARAMBINDINFO [STACK_MEMORY_MODEL]
			l_error_handler: OLEDB_ERROR_HANDLER_PRINT
		do
			statement := a_statement
			if statement.as_lower.has_substring ("insert into #") then
				should_not_be_tracked := True
			end
			create predicate_statement.make (a_statement)
			predicate_statement.parse
			l_hr := open (a_connection.session, predicate_statement.parsed_string, Void)
			if a_parameter_template /= Void then
				create l_oledb_dbparambindinfo.make (Current, a_parameter_template)
			end
			prepare (0)
			l_hr := status
			if l_hr < 0 then
				create l_error_handler
				l_error_handler.on_error (l_hr, Void, generating_type + ".make")
				success := False
			else
				l_hr := accessor.bind_parameters (p)
				data_server := a_connection
				create in_sqlda.make (Current, True)
				create out_sqlda.make (Current, False)
				id := 1 -- dummy
				success := True
			end
		end

feature -- Access

	in_sqlda : OLEDB_SQLDA
			-- Input SQLDA for the statement used at creation
	out_sqlda : OLEDB_SQLDA
			-- Output SQLDA for the statement used at creation
	data_server: OLEDB_ANY_DATA_SERVER
			-- The data server on which this executor is built for

feature -- Basic operations

	statement_make (a_data_server: like data_server; a_statement: STRING) is
			-- Parse 'a_statement' for 'a_data_server'
		do
			make (a_data_server, a_statement, Void)
		end

	execute is
			-- Execute this command
		do
			if not data_server.implicit_transaction and then	-- It hasn't been set yet.
		   	   data_server.explicit_transaction_depth=0			-- He hasn't started his own transaction.
			then
				-- EXECUTION_STATEMENTs are assumed to modify something.  Here we test that that
				-- 'something' is not the tempdb.
				if not data_server.implicit_transactions_set and not data_server.statement_modifies_tempdb(statement)
				then
					debug("explain_implicit_transactions")
						print("implicit_transactions ON : " + data_server.node + "/" + data_server.database + "/" + statement)
					end

					data_server.set_implicit_transaction_on		-- This sets 'implicit_transactions_set'.
				end
			end

			set_status (command_execute (False))
		end

feature {NONE} -- Implementation

	default_command_propset: OLEDB_PROPERTY_SET is
			-- Access to the default command property set to be used for these implementations of executors
		do
			Result := data_server.default_command_propset
		end

end
