indexing
	description: "Object to use to fast load into OLEDB data servers"
	author: "Pierre Nallet"
	last_editor: "$Author: c4 $"
	date: "$Date: 2011/08/05 $"
	revision: "$Revision: #79 $"
	path: "$File: //rose/source/kernel/infrastructure/persistence/windows/classic/oledb_fast_load.e $"
	
class
	OLEDB_FAST_LOAD [G]
	
inherit
	OLEDB_DBSTATUS_CONSTANTS
	
	OLEDB_TABLE [OLEDB_DYNAMIC_ACCESSOR, OLEDB_ROWSET_FAST_LOAD]
		rename
			make as table_make
		end
	
	OLEDB_DBTYPE_CONSTANTS
	
	SQLOLEDB_PROPERTY_CONSTANTS
	TRACEABLE
		rename
			success as traceable_success
		end
	DATA_SERVER_FAST_LOAD 
		redefine 
			rowset_exists, 
			can_auto_commit 
		end
	ROSE_CONSTANT_VALUES
	OLEDB_SQL_SCRIPT_CAPTURE
	OLEDB_SIZEOF
	MOVIE_CAMERA_AWARE
		export
			{NONE} all
		end
	
	create
	make,
	make_anonymous,
	make_with_options,
	make_keep_identity
	
feature {NONE} -- Initialization
	
	make (a_server: SQLOLEDB_DATA_SERVER; a_table_name: STRING; a_template: ANY) is
			-- Make for 'a_server', 'a_table_name' and with 'a_template'.
		require
			a_server_not_void: a_server /= Void
			connected: a_server.connected
			a_table_name_not_void: a_table_name /= Void
			a_server_fast_load: a_server.is_fast_load
		local
			hr, hr2: INTEGER
			l_frame: FOUR_TUPLE[ANY, INTEGER, ARRAY [INTEGER], INTEGER]
		do
			server := a_server
			session := a_server.session
			debug("make_movies")
				create cam.make(generating_type + "/" + a_server.node + "/" + a_server.database + "/" + a_table_name)
			end

			if a_table_name.item(1).is_equal('#') then
				should_not_be_tracked := True
			end
			
			if playback_movie
			 then
				l_frame ?= cam.next_frame("make").frame_data
				cam.compare_partial_movie_frame(l_frame.first, a_template)
				hr := l_frame.second
				if hr>=0
				 then
					map := l_frame.third
				else
					print ("OLEDB_FAST_LOAD error ")
					print (hr)
					print ("%N")
				end
				
				hr := l_frame.fourth
			else
				table_make
				hr := open (session, a_table_name, Void)
				if hr >= 0 then
					hr := bind
					if hr >= 0 then
						calculate_indices (a_template)
					else
						print ("OLEDB_FAST_LOAD error ")
						print (hr)
						print ("%N")
					end
				else
					print ("OLEDB_FAST_LOAD error ")
					print (hr)
					print ("%N")
				end
			end
			
			debug("make_movies")
				if record_movie
				 then
					create l_frame.make(a_template, hr2, map, hr)
					cam.record_movie_frame("make", l_frame)
				end
			end
			
			last_result := hr
			set_status (last_result)
			server.set_status (Current)
			check_last_result
		end
	
	make_anonymous (a_server: SQLOLEDB_DATA_SERVER; a_table_name: STRING; a_column_names: ARRAY [STRING]) is
			-- Make with column names
		require
			a_server_not_void: a_server /= Void
			connected: a_server.connected
			a_table_name_not_void: a_table_name /= Void
			a_server_fast_load: a_server.is_fast_load
		local
			hr: INTEGER
			l_frame: TRIPLE[ARRAY[STRING], INTEGER, ARRAY[INTEGER]]
		do
			server := a_server
			session := a_server.session
			debug("make_movies")
				create cam.make(generating_type + "/" + a_server.node + "/" + a_server.database + "/" + a_table_name)
			end
			
			if a_table_name.item(1).is_equal('#') then
				should_not_be_tracked := True
			end

			if playback_movie
			 then
				l_frame ?= cam.next_frame("make_anonymous").frame_data
				cam.compare_partial_movie_frame(l_frame.first, a_column_names)
				hr := l_frame.second
				array_map := l_frame.third
			else
				table_make
				hr := open(session, a_table_name, Void)
				if hr >= 0 then
					hr := bind
					if hr >= 0 then
						set_column_names (a_column_names)
						-- calculate_indices (a_template)
					else
						print ("OLEDB_FAST_LOAD error ")
						print (hr)
						print("%N")
					end
				else
					print ("OLEDB_FAST_LOAD error ")
					print (hr)
					print("%N")
				end
				
				debug("make_movies")
					if record_movie
					 then
						create l_frame.make(a_column_names, hr, array_map)
						cam.record_movie_frame("make_anonymous", l_frame)
					end
				end
			end
			
			last_result:= hr
			set_status(last_result)
			server.set_status(Current)
			check_last_result
		end
	
	make_with_options (a_server: SQLOLEDB_DATA_SERVER; a_table_name: STRING; a_template: ANY; a_options: STRING) is
			-- Advanced make
		require
			a_server_not_void: a_server /= Void
			a_server_fast_load: a_server.is_fast_load
			connected: a_server.connected
			a_table_name_not_void: a_table_name /= Void
		local
			hr: INTEGER
			ps: OLEDB_PROPERTY_SET
			l_triple: TRIPLE[ANY, STRING, INTEGER]
		do
			server := a_server
			session := a_server.session
			debug("make_movies")
				create cam.make(generating_type + "/" + a_server.node + "/" + a_server.database + "/" + a_table_name)
			end
			
			if a_table_name.item(1).is_equal('#') then
				should_not_be_tracked := True
			end

			if playback_movie
			 then
				l_triple ?= cam.next_frame("make_with_options").frame_data
				cam.compare_partial_movie_frame(l_triple.first, a_table_name)
				cam.compare_partial_movie_frame(l_triple.second, a_options)
				hr := l_triple.third
			else
				table_make
				if a_options /= Void then
					create ps.make (DBPROPSET_SQLSERVERROWSET)
					ps.add_bstr_property (SSPROP_FASTLOADOPTIONS, a_options)
				end
				hr := open(session, a_table_name, ps)
				if hr >= 0 then
					hr := bind
					if hr >= 0 then
						calculate_indices (a_template)
					else
						print ("OLEDB_FAST_LOAD error ")
						print (hr)
						print("%N")
					end
				else
					print ("OLEDB_FAST_LOAD error ")
					print (hr)
					print("%N")
				end
				
				debug("make_movies")
					if record_movie
					 then
						create l_triple.make(a_table_name, a_options, hr)
						cam.record_movie_frame("make_with_options", l_triple)
					end
				end
			end
			
			last_result:= hr
			set_status(last_result)
			server.set_status(Current)
			check_last_result
		end
	
	make_keep_identity (a_server: SQLOLEDB_DATA_SERVER; a_table_name: STRING; a_template: ANY) is
			-- Make and keep identity
		require
			a_server_not_void: a_server /= Void
			a_server_fast_load: a_server.is_fast_load
			connected: a_server.connected
			a_table_name_not_void: a_table_name /= Void
		local
			hr: INTEGER
			ps: OLEDB_PROPERTY_SET
			l_pair: PAIR[ANY, INTEGER]
		do
			server := a_server
			session := a_server.session
			debug("make_movies")
				create cam.make(generating_type + "/" + a_server.node + "/" + a_server.database + "/" + a_table_name)
			end
			
			if a_table_name.item(1).is_equal('#') then
				should_not_be_tracked := True
			end

			if playback_movie
			 then
				l_pair ?= cam.next_frame("make_keep_identity").frame_data
				cam.compare_partial_movie_frame(l_pair.first, a_template)
				hr := l_pair.second
			else
				table_make
				create ps.make (DBPROPSET_SQLSERVERROWSET)
				ps.add_boolean_property (SSPROP_FASTLOADKEEPIDENTITY, true)
				hr := open(session, a_table_name, ps)
				if hr >= 0 then
					hr := bind
					if hr >= 0 then
						calculate_indices (a_template)
					else
						print ("OLEDB_FAST_LOAD error ")
						print (hr)
						print ("%N")
					end
				else
					print ("OLEDB_FAST_LOAD error ")
					print (hr)
					print ("%N")
				end
				
				debug("make_movies")
					if record_movie
					 then
						create l_pair.make(a_template, hr)
						cam.record_movie_frame("make_keep_identity", l_pair)
					end
				end
			end
			
			last_result:= hr
			set_status (last_result)
			server.set_status(Current)
			check_last_result
		end
	
feature -- Access
	
	row_count: INTEGER
			-- Inserted row count
	
	auto_commit_row_count: INTEGER
			-- Number of rows for auto commit
	
	set_auto_commit (a_row_count: INTEGER) is
			-- Set the number of rows for auto commit
		do
			auto_commit_row_count := a_row_count
		end
	can_auto_commit (a_row_count: INTEGER): BOOLEAN is
			-- Can it commit  a_row_count?
		do
			Result := a_row_count > 0 and a_row_count <= 2000
		end
	
	forget_auto_commit is
			-- Forget auto commit
		do
			auto_commit_row_count := 0
		end
	
	map: ARRAY [INTEGER]
			-- OLEDB -> EIFFEL
	array_map: ARRAY [INTEGER]
			-- OLEDB -> ARRAY
	
	session: OLEDB_SESSION
			-- Oledb session
	
	server: SQLOLEDB_DATA_SERVER
			-- Server
	
	column_name (i: INTEGER): STRING is
			-- Column name at index i
		local
			l_pair: PAIR[INTEGER, STRING]
		do
			if playback_movie
			 then
				l_pair ?=cam. next_frame("column_name").frame_data
				cam.compare_partial_movie_frame(l_pair.first, i)
				Result := l_pair.second
			else
				Result := accessor.get_column_name (i)
				debug("make_movies")
					if record_movie
					 then
						create l_pair.make(i, Result)
						cam.record_movie_frame("column_name", l_pair)
					end
				end
			end
		end
	
	column_names: ARRAY [STRING] is
			-- Column names
		local
			i: INTEGER
		do
			if playback_movie
			 then
				-- This is handled by the caller (calculate_indices).
			else
				create Result.make (1, accessor.column_info.count)
				from i := 1 until i > accessor.column_info.count loop
					Result.put (accessor.column_info.item (i-1).name, i)
					Result.item (i).to_lower
					i := i + 1
				end
				Result.compare_objects
				
				debug("make_movies")
					-- This is handled by the caller (calculate_indices).
				end
			end
		end
	
	column_type (i: INTEGER): INTEGER is
			-- Column type at index i
		local
			l_pair: PAIR[INTEGER, INTEGER]
		do
			if playback_movie
			 then
				l_pair ?= cam.next_frame("column_type").frame_data
				cam.compare_partial_movie_frame(l_pair.first, i)
				Result := l_pair.second
			else
				Result := accessor.get_column_type (i)
				debug("make_movies")
					if record_movie
					 then
						create l_pair.make(i, Result)
						cam.record_movie_frame("column_type", l_pair)
					end
				end
			end
		end
	
	calculate_indices (a_template: ANY) is
			-- Internal calulation for indices
		require
			accessor_has_columns: accessor.column_info /= Void
		local
			i, j: INTEGER
			l_column_names: like column_names
			l_field_count: INTEGER
			l_index: INTEGER
			l_field_name: STRING
			--l_field_mapper: FIELD_MAPPER
		do
			if playback_movie
			 then
				-- This is handled by the caller (make).
			else
				create map.make (1,accessor.column_info.count)
				from i := map.lower until i > map.upper loop
					map.put (-1,i)
					i := i + 1
				end
				l_field_count := internal.field_count (a_template)
				l_column_names := column_names.twin
				l_column_names.compare_objects
				-- l_field_mapper ?= a_template
				from i := 1 until i > internal.field_count (a_template) loop
					l_field_name := internal.field_name (i, a_template)
					from
						j := l_column_names.lower
					until
						j > l_column_names.upper
							or else accessor.column_info.item (j-1).name.is_equal (l_field_name)
						-- FIXME -- begin --
						-- the following is a horrible hack to deal with a situation where
						-- a field name in the database is a reserved word in eiffel (e.g. "is").
						-- So, the eiffel attribute name has an underscore appended to avoid the reserved word clash.
						-- Probably this should be handled by allowing the object or some object manager to produce the mapping.
						-- or else ( l_field_mapper /= Void and then accessor.column_info.item (j-1).name.is_equal ( l_field_mapper.map( accessor.column_info.item (j-1).name ) )
							or else l_field_name.is_equal( accessor.column_info.item (j-1).name + "_" )
                        -- FIXME -- end --
					loop
						j := j + 1
					end
					l_index := j
					if l_index <= accessor.column_info.count then
						map.force (i, j)
					end
					i := i + 1
				end
				
				debug("make_movies")
					-- This is handled by the caller (make).
				end
			end
		end
	
	set_column_names (a: ARRAY [STRING]) is
			-- Set column names
		require
			accessor_has_columns: accessor.column_info /= Void
		local
			i, j: INTEGER
			l_column_names: like column_names
			l_index: INTEGER
			l_field_name: STRING
		do
			debug("make_movies")
				-- Handled by the caller.
			end
			
			create array_map.make (1, accessor.column_info.count)
			from i := array_map.lower until i> array_map.upper loop
				array_map.put (-1, i )
				i := i + 1
			end
			l_column_names := column_names.twin
			l_column_names.compare_objects
			from i := a.lower until i > a.upper loop
				l_field_name := a.item(i).twin
				l_field_name.to_lower
				from j := l_column_names.lower until j > l_column_names.upper or accessor.column_info.item (j-1).name.is_equal (l_field_name) loop
					j := j + 1
				end
				l_index := j
				if l_index <= accessor.column_info.count then
					array_map.force (i, j)
				end
				i := i + 1
			end
		end
	rowset_exsists: BOOLEAN is
			--    Does the underlying rowset exist?
		
		do
			Result := rowset.exists
		end
	
	insert (a_item: ANY) is
			-- Insert an row
		local
			l_pair: PAIR[ANY, INTEGER]
		do
			if playback_movie
			 then
				l_pair ?= cam.next_frame("insert").frame_data
				cam.compare_partial_movie_frame(l_pair.first, a_item)
				last_result := l_pair.second
			else
				set_object (a_item)
				last_result:= rowset.insert_row
				debug("make_movies")
					if record_movie
					 then
						create l_pair.make(a_item, last_result)
						cam.record_movie_frame("insert", l_pair)
					end
				end
			end
			
			set_status(last_result)
			server.set_status(Current)
			check_last_result
			if auto_commit_row_count > 0 then
				row_count := row_count + 1
				if row_count >auto_commit_row_count then
					commit
				end
			end
			debug ("fastload")
				if row_count > 20000 then
					print ("too many rows" + row_count.out + "%N")
				end
			end
		end
	
	rowset_exists: BOOLEAN is
			-- Does the rowset exist?
		do
			Result := playback_movie or else rowset.exists
		end
	
	insert_array (a: ARRAY [ANY]) is
			-- Insert an array of rows
		require
			rowset_exists: rowset_exists
		local
			l_pair: PAIR[ARRAY[ANY], INTEGER]
		do
			if playback_movie
			 then
				l_pair ?= cam.next_frame("insert_array").frame_data
				cam.compare_partial_movie_frame(l_pair.first, a)
				last_result := l_pair.second
			else
				set_array (a)
				last_result:= rowset.insert_row
				debug("make_movies")
					if record_movie
					 then
						create l_pair.make(a, last_result)
						cam.record_movie_frame("insert_array", l_pair)
					end
				end
			end
			
			set_status(last_result)
			server.set_status(Current)
			check_last_result
			debug ("fastload")
				row_count := row_count + 1
				if row_count > 20000 then
					print ("too many rows" + row_count.out + "%N")
				end
			end
		end
	
	commit is
			-- Commit the rows to the db
		do
			if playback_movie
			 then
				 last_result := cam.next_frame("commit").frame_data_as_integer
				 cam.compare_not_needed
			else
				if server.definition /= Void
					and then not should_not_be_tracked
				 then
					server.definition.increment_execution_count
				end				
				last_result := rowset.commit(0)
				debug("make_movies")
					if record_movie
					 then
						cam.record_integer_movie_frame("commit", last_result)
					end
				end
			end
			
			set_status(last_result)
			server.set_status(Current)
			check_last_result
			row_count := 0
			if not playback_movie then
				if server.definition /= Void then
					server.definition.increment_bulk_insert_count (temp_table_name)
				end
			end
		end
	
	check_last_result is
			-- Not used
		do
			--if not success and server.error_handler /= Void then
			-- server.error_handler.on_error (last_result, Current, "fast load")
			--end
		end
	
	current_stream: OLEDB_IN_MEMORY_SEQUENTIAL_STREAM
			-- Stream for current row
	
	set_array (aa: ARRAY [ANY]) is
			-- Set array
		require
			has_array_map: array_map /= Void
		local
			l_pointer: POINTER
			i: INTEGER
			l_count: INTEGER
			l_type: INTEGER
			l_column_info: OLEDB_COLUMN_INFO
			s: STRING
			c: CHARACTER_REF
			a: ANY
			dt: ROSE_DATE
			d: ROSE_DATE
			ii: INTEGER_REF
			bb: BOOLEAN_REF
			dbl: DOUBLE_REF
			r: REAL_REF
			l_time: ROSE_TIME
			l_stream: STREAM
			p_stream: POINTER
			l_exceptions: EXCEPTIONS
			l_c_string_32: C_STRING_32
		do
			debug("make_movies")
				if playback_movie
				 then
					create l_exceptions
					l_exceptions.raise("playback should be implemented by caller")
				end
			end
			
			from i := 1 until i > accessor.column_info.count loop
				if array_map.item (i) >= 0 then
					l_column_info := accessor.column_info.item (i-1)
					l_type := l_column_info.type
					if l_type = Dbtype_bool then
						bb ?= aa.item (array_map.item (i))
						if bb /= Void then
							if bb.item then
								accessor.set_integer_value (accessor.column_info.item(i-1).ordinal, -1)
							else
								accessor.set_integer_value (accessor.column_info.item(i-1).ordinal, 0)
							end
							accessor.set_length (accessor.column_info.item(i-1).ordinal, sizeof_db_bool)
							accessor.set_status (accessor.column_info.item(i-1).ordinal, DBSTATUS_S_OK)
						else
							accessor.set_status (accessor.column_info.item(i-1).ordinal, DBSTATUS_S_ISNULL)
						end
					elseif l_type = Dbtype_i4 then
						debug ("FASTLOAD")
							print ("setting ")
							-- print (field_name (array_map.item (i), an_item))
						end
						ii ?= aa.item (array_map.item (i))
						if ii /= Void then
							accessor.set_integer_value (accessor.column_info.item(i-1).ordinal, ii.item)
							accessor.set_length (accessor.column_info.item(i-1).ordinal, sizeof_int)
							accessor.set_status (accessor.column_info.item(i-1).ordinal, DBSTATUS_S_OK)
						else
							accessor.set_status (accessor.column_info.item(i-1).ordinal, DBSTATUS_S_ISNULL)
						end
					elseif l_type = Dbtype_i2 then
						ii ?= aa.item(array_map.item (i))
						if ii /= Void then
							accessor.set_i2_value (accessor.column_info.item(i-1).ordinal, ii.item)
							accessor.set_length (accessor.column_info.item(i-1).ordinal, 2)
							accessor.set_status (accessor.column_info.item(i-1).ordinal, DBSTATUS_S_OK)
						else
							accessor.set_status (accessor.column_info.item(i-1).ordinal, DBSTATUS_S_ISNULL)
						end
					elseif l_type = Dbtype_i1 or l_type = Dbtype_ui1 then
						ii ?= aa.item(array_map.item (i))
						if ii /= Void then
							accessor.set_i1_value (accessor.column_info.item(i-1).ordinal, ii.item)
							accessor.set_length (accessor.column_info.item(i-1).ordinal, 1)
							accessor.set_status (accessor.column_info.item(i-1).ordinal, DBSTATUS_S_OK)
						else
							accessor.set_status (accessor.column_info.item(i-1).ordinal, DBSTATUS_S_ISNULL)
						end
					elseif l_type = Dbtype_str then
						s ?= aa.item (array_map.item (i))
						if s = Void then
							accessor.set_status (accessor.column_info.item(i-1).ordinal, DBSTATUS_S_ISNULL)
						else
							a := s.to_c
							accessor.set_status (accessor.column_info.item(i-1).ordinal, DBSTATUS_S_OK)
							l_count := s.count.min (accessor.column_info.item(i-1).size - 1)
							memcpy(accessor.get_value(accessor.column_info.item(i-1).ordinal), $a, l_count)
							accessor.set_length (accessor.column_info.item(i-1).ordinal, l_count)
						end
					elseif l_type = Dbtype_wstr then
						s ?= array_map.item (i)
						if s=Void
						 then
							c ?= array_map.item (i)
							if c/=Void
							 then
								s := c.out
							end
						end
						if s = Void then
							if { s32: STRING_32 } array_map.item (i) then
								l_count := (s32.count).min ((accessor.column_info.item (i - 1).size//2 - 1))
								create l_c_string_32.make (s32)
								l_pointer := l_c_string_32.item
								memcpy (accessor.get_value(accessor.column_info.item(i-1).ordinal), l_pointer, l_count * 2)
								accessor.set_length (accessor.column_info.item(i-1).ordinal, l_count * 2)
							else
								accessor.set_status (accessor.column_info.item(i-1).ordinal, DBSTATUS_S_ISNULL)
							end
						else
							l_count := (s.count).min ((accessor.column_info.item (i - 1).size - 1))
							l_pointer := string_to_widechar (s)
							memcpy (accessor.get_value(accessor.column_info.item(i-1).ordinal), l_pointer, l_count * 2)
							accessor.set_length (accessor.column_info.item(i-1).ordinal, l_count * 2)
							l_pointer.memory_free
						end
					elseif l_type = Dbtype_r8 then
						dbl ?= aa.item(array_map.item (i))
						if dbl /= Void then
							if is_missing_double(dbl.item) then
								accessor.set_status (accessor.column_info.item(i-1).ordinal, DBSTATUS_S_ISNULL)
							else
								accessor.set_double_value (accessor.column_info.item(i-1).ordinal, dbl.item)
								accessor.set_status (accessor.column_info.item(i-1).ordinal, DBSTATUS_S_OK)
								accessor.set_length (accessor.column_info.item(i-1).ordinal, sizeof_double)
							end
						else
							accessor.set_status (accessor.column_info.item(i-1).ordinal, DBSTATUS_S_ISNULL)
						end
					elseif l_type = Dbtype_r4 then
						r ?= aa.item (array_map.item (i))
						if r /= Void then
							if is_missing_real(r.item) then
								accessor.set_status (accessor.column_info.item(i-1).ordinal, DBSTATUS_S_ISNULL)
							else
								accessor.set_real_value (accessor.column_info.item(i-1).ordinal, r.item)
								accessor.set_status (accessor.column_info.item(i-1).ordinal, DBSTATUS_S_OK)
								accessor.set_length (accessor.column_info.item(i-1).ordinal, sizeof_float)
							end
						else
							dbl ?= aa.item(array_map.item (i))
							if dbl /= Void then
								if is_missing_double(dbl.item) then
									accessor.set_status (accessor.column_info.item(i-1).ordinal, DBSTATUS_S_ISNULL)
								else
									accessor.set_real_value (accessor.column_info.item(i-1).ordinal, dbl.item.truncated_to_real)
									accessor.set_status (accessor.column_info.item(i-1).ordinal, DBSTATUS_S_OK)
									accessor.set_length (accessor.column_info.item(i-1).ordinal, sizeof_double)
								end
							else
								accessor.set_status (accessor.column_info.item(i-1).ordinal, DBSTATUS_S_ISNULL)
							end
						end
					elseif l_type = Dbtype_dbtimestamp then
						dt ?= aa.item (array_map.item (i))
						if dt = Void then
							d ?= aa.item(array_map.item (i))
							if d = Void then
								accessor.set_status (accessor.column_info.item(i-1).ordinal, DBSTATUS_S_ISNULL)
							else
								accessor.set_status (accessor.column_info.item(i-1).ordinal, DBSTATUS_S_OK)
								if ts = Void then
									create ts.make
								end
								ts.set (accessor.get_value (accessor.column_info.item(i-1).ordinal))
								ts.set_year(d.year)
								ts.set_month(d.month)
								ts.set_day(d.day)
								ts.set_hour(d.hour)
								ts.set_minute(d.minute)
								ts.set_second(d.second)
								ts.set_second((d.fractional_second * 1000000000).rounded)
								-- ts.set_fraction(0)
								accessor.set_length (accessor.column_info.item(i-1).ordinal, ts.structure_size)
							end
						else
							accessor.set_status (accessor.column_info.item(i-1).ordinal, DBSTATUS_S_OK)
							if ts = Void then
								create ts.make
							end
							l_time := dt.time
							ts.set (accessor.get_value (accessor.column_info.item(i-1).ordinal))
							ts.set_year(dt.year)
							ts.set_month(dt.month)
							ts.set_day(dt.day)
							ts.set_hour(l_time.hour)
							ts.set_minute(l_time.minute)
							ts.set_second(l_time.second)
							ts.set_fraction((l_time.fractional_second * 1000000000).rounded)
							accessor.set_length (accessor.column_info.item(i-1).ordinal, ts.structure_size)
						end
					elseif l_type = dbtype_iunknown then
						s ?= aa.item (array_map.item (i))
						if (s /= void) then
							a := s.to_c;
							create current_stream.make_by_pointer ($a, s.count)
							accessor.set_pointer_value(accessor.column_info.item(i-1).ordinal, current_stream.p)
							current_stream.add_ref
							accessor.set_length(accessor.column_info.item(i-1).ordinal, s.count)
						else
							l_stream ?= aa.item (array_map.item (i))
							if (l_stream /= void) then
								memcpy ($p_stream, l_stream.item, sizeof_iunknown)
								create current_stream.make_by_pointer(p_stream,l_stream.object_stored_size)
								accessor.set_pointer_value(accessor.column_info.item(i-1).ordinal, current_stream.p)
								current_stream.add_ref
								accessor.set_length(accessor.column_info.item(i-1).ordinal, l_stream.object_stored_size)
							else
								print ("unable to get stream")
								print (l_type)
							end
						end
					else
						print ("type not implemented")
						print (l_type)
					end
				end
				i := i + 1
			end
		end
	
	temp_variant: COM_VARIANT is
			-- Variant helper
		once
			create Result
		end
	
	
	
	set_object (an_item: ANY) is
			-- Set the values from an_item
		local
			l_pointer: POINTER
			i: INTEGER
			l_count: INTEGER
			l_type: INTEGER
			l_column_info: OLEDB_COLUMN_INFO
			s: STRING
			c: CHARACTER_REF
			a: ANY
			dt: ROSE_DATE
			d: ROSE_DATE
			ii: INTEGER
			i64: INTEGER_64
			bb: BOOLEAN
			dbl: DOUBLE
			dr: DOUBLE_REF
			rr: REAL_REF
			l_time: ROSE_TIME
			l_stream: STREAM
			p_stream: POINTER
			l_exceptions: EXCEPTIONS
			l_decimal: RP_DECIMAL
			l_c_string_32: C_STRING_32
		do
			debug("make_movies")
				if playback_movie
				 then
					create l_exceptions
					l_exceptions.raise("playback should be implemented by caller")
				end
			end
			
			if sql_script_capture_required then
				sql_capture_temp_table_column_names_and_data(temp_table_name, an_item, accessor)
			end
			from i := 1 until i > accessor.column_info.count loop
				if map.item (i) >= 0 then
					l_column_info := accessor.column_info.item (i-1)
					l_type := l_column_info.type
					if l_type = Dbtype_bool then
						bb := internal.boolean_field (map.item (i), an_item)
						accessor.set_boolean_value (accessor.column_info.item(i-1).ordinal, bb)
						--                if bb then
						--                   accessor.set_integer_value (accessor.column_info.item(i-1).ordinal, -1)
						--                else
						--                   accessor.set_integer_value (accessor.column_info.item(i-1).ordinal, 0)
						--                end
						--                accessor.set_length (accessor.column_info.item(i-1).ordinal, sizeof_db_bool)
					elseif l_type = Dbtype_i8 then
						i64 := internal.integer_64_field (map.item (i), an_item)
						if (i64 = null_integer) then
							accessor.set_status (accessor.column_info.item(i-1).ordinal, DBSTATUS_S_ISNULL)
						else
							accessor.set_status (accessor.column_info.item(i-1).ordinal, DBSTATUS_S_OK)
							accessor.set_integer_64_value (accessor.column_info.item(i-1).ordinal, i64)
							accessor.set_length (accessor.column_info.item(i-1).ordinal, sizeof_int_64)
						end
					elseif l_type = Dbtype_i4 then
						ii := internal.integer_field (map.item (i), an_item)
						if (ii = null_integer) then
							accessor.set_status (accessor.column_info.item(i-1).ordinal, DBSTATUS_S_ISNULL)
						else
							accessor.set_status (accessor.column_info.item(i-1).ordinal, DBSTATUS_S_OK)
							accessor.set_integer_value (accessor.column_info.item(i-1).ordinal, ii)
							accessor.set_length (accessor.column_info.item(i-1).ordinal, sizeof_int)
						end
					elseif l_type = Dbtype_i2 then
						ii := internal.integer_field (map.item (i), an_item)
						accessor.set_i2_value (accessor.column_info.item(i-1).ordinal, ii)
						accessor.set_length (accessor.column_info.item(i-1).ordinal, 2)
					elseif l_type = Dbtype_i1 then
						ii := internal.integer_field (map.item (i), an_item)
						accessor.set_i1_value (accessor.column_info.item(i-1).ordinal, ii)
						accessor.set_length (accessor.column_info.item(i-1).ordinal, 1)
					elseif l_type = Dbtype_ui1 then
						ii := internal.integer_field (map.item (i), an_item)
						accessor.set_ui1_value (accessor.column_info.item(i-1).ordinal, ii)
						accessor.set_length (accessor.column_info.item(i-1).ordinal, 1)
					elseif l_type = Dbtype_str then
						s ?= internal.field (map.item (i), an_item)
						if s=Void
						 then
							c ?= internal.field (map.item (i), an_item)
							if c/=Void
							 then
								s := c.out
							end
						end
						if s = Void then
							accessor.set_status (accessor.column_info.item(i-1).ordinal, DBSTATUS_S_ISNULL)
						else
							a := s.to_c
							accessor.set_status (accessor.column_info.item(i-1).ordinal, DBSTATUS_S_OK)
							l_count := s.count.min (accessor.column_info.item(i-1).size - 1)
							memcpy(accessor.get_value(accessor.column_info.item(i-1).ordinal), $a, l_count)
							accessor.set_length (accessor.column_info.item(i-1).ordinal, l_count)
						end
					elseif l_type = Dbtype_wstr then
						s ?= internal.field (map.item (i), an_item)
						if s=Void
						 then
							c ?= internal.field (map.item (i), an_item)
							if c/=Void
							 then
								s := c.out
							end
						end
						if s = Void then
							if { s32: STRING_32 } internal.field (map.item (i), an_item) then
								l_count := (s32.count).min ((accessor.column_info.item (i - 1).size//2 - 1))
								create l_c_string_32.make (s32)
								l_pointer := l_c_string_32.item
								memcpy (accessor.get_value(accessor.column_info.item(i-1).ordinal), l_pointer, l_count * 2)
								accessor.set_length (accessor.column_info.item(i-1).ordinal, l_count * 2)
							else
								accessor.set_status (accessor.column_info.item(i-1).ordinal, DBSTATUS_S_ISNULL)
							end
						else
							l_count := (s.count).min ((accessor.column_info.item (i - 1).size - 1))
							l_pointer := string_to_widechar (s)
							memcpy (accessor.get_value(accessor.column_info.item(i-1).ordinal), l_pointer, l_count * 2)
							accessor.set_length (accessor.column_info.item(i-1).ordinal, l_count * 2)
							l_pointer.memory_free
						end
					elseif l_type = Dbtype_r8 then
						dr ?= internal.field (map.item (i), an_item)
						if (dr /= Void) then
							dbl := dr.item
						else
							rr ?= internal.field (map.item (i), an_item)
							if (rr /= Void) then
								dbl := rr.item
							else
								l_decimal ?= internal.field (map.item (i), an_item)
								dbl := l_decimal
							end
						end
						--dbl := internal.double_field (map.item (i), an_item)
						if is_missing_double(dbl) then
							accessor.set_status (accessor.column_info.item(i-1).ordinal, DBSTATUS_S_ISNULL)
						else
							accessor.set_double_value (accessor.column_info.item(i-1).ordinal, dbl)
							accessor.set_status (accessor.column_info.item(i-1).ordinal, DBSTATUS_S_OK)
							accessor.set_length (accessor.column_info.item(i-1).ordinal, sizeof_double)
						end
					elseif l_type = Dbtype_r4 then
						if is_missing_real(internal.real_field (map.item (i), an_item)) then
							accessor.set_status (accessor.column_info.item(i-1).ordinal, DBSTATUS_S_ISNULL)
						else
							accessor.set_real_value (accessor.column_info.item(i-1).ordinal, internal.real_field (map.item (i), an_item))
							accessor.set_status (accessor.column_info.item(i-1).ordinal, DBSTATUS_S_OK)
							accessor.set_length (accessor.column_info.item(i-1).ordinal, sizeof_float)
						end
					elseif l_type = Dbtype_dbtimestamp then
						dt ?= internal.field (map.item (i), an_item)
						if dt = Void then
							d ?= internal.field (map.item (i), an_item)
							if d = Void then
								accessor.set_status (accessor.column_info.item(i-1).ordinal, DBSTATUS_S_ISNULL)
							else
								accessor.set_status (accessor.column_info.item(i-1).ordinal, DBSTATUS_S_OK)
								if ts = Void then
									create ts.make
								end
								ts.set (accessor.get_value (accessor.column_info.item(i-1).ordinal))
								ts.set_year(d.year)
								ts.set_month(d.month)
								ts.set_day(d.day)
								ts.set_hour(d.hour)
								ts.set_minute(d.minute)
								ts.set_second(d.second)
								ts.set_fraction((d.fractional_second * 1000000000).rounded)
								accessor.set_length (accessor.column_info.item(i-1).ordinal, ts.structure_size)
							end
						else
							accessor.set_status (accessor.column_info.item(i-1).ordinal, DBSTATUS_S_OK)
							if ts = Void then
								create ts.make
							end
							l_time := dt.time
							ts.set (accessor.get_value (accessor.column_info.item(i-1).ordinal))
							ts.set_year(dt.year)
							ts.set_month(dt.month)
							ts.set_day(dt.day)
							ts.set_hour(l_time.hour)
							ts.set_minute(l_time.minute)
							ts.set_second(l_time.second)
							--ts.set_fraction(0)
							ts.set_fraction ((l_time.fractional_second * 1000000000).rounded)
							
							accessor.set_length (accessor.column_info.item(i-1).ordinal, ts.structure_size)
						end
					elseif l_type = Dbtype_bytes then
						l_stream ?= internal.field(map.item (i),an_item)
						s ?= internal.field(map.item (i),an_item)
						if l_stream = Void and s = void then
							accessor.set_status (accessor.column_info.item(i-1).ordinal, DBSTATUS_S_ISNULL)
						elseif l_stream /= void then
							l_count := l_stream.object_stored_size.min (accessor.column_info.item(i-1).size)
							memcpy(accessor.get_value (accessor.column_info.item(i-1).ordinal), l_stream.item, l_count)
							accessor.set_length(accessor.column_info.item(i-1).ordinal, l_count)
							accessor.set_status (accessor.column_info.item(i-1).ordinal, DBSTATUS_S_OK)
						else
							l_count := s.count.min (accessor.column_info.item(i-1).size)
							a := s.to_c
							memcpy(accessor.get_value(accessor.column_info.item(i-1).ordinal), $a, l_count)
							accessor.set_length (accessor.column_info.item(i-1).ordinal, l_count)
							accessor.set_status (accessor.column_info.item(i-1).ordinal, DBSTATUS_S_OK)
						end
					elseif l_type = dbtype_variant then
						set_variant (internal.field (map.item (i), an_item), accessor.column_info.item(i-1).ordinal)
					elseif l_type = Dbtype_Iunknown then
						s ?= internal.field (map.item (i), an_item)
						if (s /= void) then
							a := s.to_c;
							create current_stream.make_by_pointer ($a, s.count)
							accessor.set_pointer_value(accessor.column_info.item(i-1).ordinal, current_stream.p)
							current_stream.add_ref
							accessor.set_length(accessor.column_info.item(i-1).ordinal, s.count)
						else
							l_stream ?= internal.field(map.item (i),an_item)
							if l_stream /= Void then
								memcpy($p_stream, l_stream.item, sizeof_iunknown)
								create current_stream.make_by_pointer(p_stream,l_stream.object_stored_size)
								accessor.set_pointer_value(accessor.column_info.item(i-1).ordinal, current_stream.p)
								current_stream.add_ref
								accessor.set_length(accessor.column_info.item(i-1).ordinal, l_stream.object_stored_size)
							else
								print ("unable to get stream")
								print (l_type)
							end
						end
					elseif l_type = DBTYPE_NUMERIC then
						a := internal.field (map.item (i), an_item)
						if (a = Void) then
							accessor.set_status (accessor.column_info.item(i-1).ordinal, dbstatus_s_isnull)
						else
							l_decimal ?= a
							accessor.set_status (accessor.column_info.item(i-1).ordinal, DBSTATUS_S_OK)
							data_convert.decimal_to_numeric (l_decimal, accessor.get_value(accessor.column_info.item(i-1).ordinal), accessor.column_info.item(i-1).precision, accessor.column_info.item (i-1).scale)
						end
					else
						print ("type not implemented")
						print (l_type)
					end
					
				end
				i := i + 1
			end
		end
	
	data_convert: OLEDB_DATA_CONVERT is
			-- Data convert
		once
            create Result.make
		end
	
	set_variant (a_value: ANY; a_index: INTEGER_32)
	--    Set the value as a variant
		local
			ir: INTEGER_32_REF
			i8r: INTEGER_64_REF
			l_date: ROSE_DATE
			dr: REAL_64_REF
			rr: REAL_32_REF
			br: BOOLEAN_REF
			str: STRING_8
		do
			ir ?= a_value
			i8r ?= a_value
			l_date ?= a_value
            accessor.set_status (a_index, dbstatus_s_ok)
            accessor.set_length (a_index, 16)
			if (a_value = Void) then
				accessor.set_status (a_index, dbstatus_s_isnull)
			elseif ir /= Void then
				temp_variant.set (accessor.get_value (a_index))
				temp_variant.set_from_integer (ir.item)
			elseif (i8r /= Void) then
				temp_variant.set (accessor.get_value (a_index))
				temp_variant.set_from_integer_64 (i8r.item)
			elseif (l_date /= Void) then
				temp_variant.set (accessor.get_value (a_index))
				temp_variant.set_date (l_date)
			else
				dr ?= a_value
				if dr /= Void then
					if (ieee_math.isnan (dr.item)) then
						temp_variant.set (accessor.get_value (a_index))
						temp_variant.set_real8 (dr.item)
						accessor.set_status (a_index, dbstatus_s_isnull)
						
					else
						temp_variant.set (accessor.get_value (a_index))
						temp_variant.set_real8 (dr.item)
					end
				else
					rr ?= a_value
					if rr /= Void then
						temp_variant.set (accessor.get_value (a_index))
						temp_variant.set_real4 (rr.item)
					else
						br ?= a_value
						if br /= Void then
							temp_variant.set (accessor.get_value (a_index))
							temp_variant.set_boolean (br.item)
						else
							str ?= a_value
							if str /= Void then
								temp_variant.set (accessor.get_value (a_index))
								temp_variant.set_from_string (str)
							else
								print ("invalid type for variant" + a_value.out)
							end
						end
					end
				end
			end
		end
	
	
	ts: OLEDB_DBTIMESTAMP [STACK_MEMORY_MODEL]
			-- Implementation for time stamp
	
	last_result: INTEGER
			-- Last result
	
	success: BOOLEAN is
			-- Was the last operation successful?
		do
			--Result := last_result >= 0
			Result := server.success
		end
	
	should_not_be_tracked: BOOLEAN
			-- Should uses of this fast loader NOT be used in counting how a data server is used?
end
