indexing

	description: "Object providing access to an OLEDB parameter sqlvar's status and values"
	author: "Pierre Nallet"
	last_editor: "$Author: pn $"
	date: "$Date: 2010/03/29 $"
	revision: "$Revision: #56 $"
	path: "$File: //rose/source/kernel/infrastructure/persistence/windows/classic/oledb_parameter_sqlvar.e $"

class

	OLEDB_PARAMETER_SQLVAR

inherit

	OLEDB_SQLVAR
		redefine
			other_display
		end
	OLEDB_SIZEOF
		export
			{NONE} all
		end

create

    make

feature -- Access

	status: INTEGER is
			-- Status of the parameter
		do
			Result := accessor.get_parameter_status (index)
		end

	value: ANY is
			-- Value of this parameter as an ANY
		do
			if sqltype = dbtype_str  or sqltype = dbtype_wstr then
				Result := string_value
			elseif sqltype = dbtype_r4 then
				Result := real_value
			elseif sqltype = dbtype_r8 then
				Result := double_value
			elseif sqltype = dbtype_bytes then
				-- Not implemented
			elseif sqltype = dbtype_dbtimestamp  or sqltype = dbtype_date then
				Result := date_value
			elseif sqltype = dbtype_numeric then
				-- Not implemented
			elseif sqltype = dbtype_i1 or sqltype = dbtype_i2 or sqltype = dbtype_i4 or
				sqltype = dbtype_ui1 or sqltype = dbtype_ui2 or sqltype = dbtype_ui4 then
				Result := integer_value
			elseif sqltype = dbtype_i8 then
				-- Not implemented
			elseif sqltype = dbtype_bool then
				-- Not implemented
			elseif sqltype = dbtype_iunknown then
				-- Not implemented
			elseif sqltype = dbtype_variant then
				-- Not implemented
			else
				print ("unhandled type " + sqltype.out)
			end
		end

	date_time_value: ROSE_DATE is
			-- Date parameter value
		do
			Result:= date_value
		end

	date_value: ROSE_DATE is
			-- Date parameter value
		do
			Result := accessor.date_parameter (index)
		end

	integer_value: INTEGER is
			-- Parameter value as an integer
		do
			Result := accessor.integer_parameter (index)
		end

	real_value: REAL is
			-- Parameter value as a real
		do
			Result := accessor.real_parameter (index)
		end

	double_value: DOUBLE is
			-- Parameter value as a double
		do
			Result := accessor.double_parameter (index)
		end

	boolean_value: BOOLEAN is
			-- Parameter value as a boolean
		do
			-- Not implemented
		end

	character_value: CHARACTER is
			-- Parameter value as a character
		do
			-- Not implemented
		end

	string_value: STRING is
			-- Parameter value as a string
		do
			Result := accessor.string_parameter (index)
		end

	other_display: STRING
	-- This is called by 'display'.  It handles cases that 'display' can't.
	local
		l_string: STRING
		l_value: ANY
		l_status: like status
	do
		l_status := status
		if l_status/=dbstatus_s_ok and l_status/=dbstatus_s_isnull
		then
			Result := dbstatus_to_text(l_status) + " "
			l_value := value
			l_string ?= l_value
			if l_string/=Void and l_string.count>200
			then
				Result.append(l_string.substring(1, 200))
			elseif l_value/=Void
			then
				Result.append(l_value.out)
			end
		end
	end

feature -- Modification

	set_status (a_status: INTEGER) is
			-- Set the status of this parameter to a_status
		do
			accessor.set_parameter_status (index, a_status)
		end

	set_attributes is
			-- Set the attributes of this paramter from the accessor
		do
			index := index + 1
			sqlname := accessor.get_parameter_name (index)
			if sqlname = Void or else sqlname.is_empty then
				if { l_executor_statement: OLEDB_EXECUTOR } sqlda.statement then
					if { l_executor_predicate_statement: PREDICATE_STATEMENT } l_executor_statement.predicate_statement then
						if l_executor_predicate_statement.binding.valid_index(index) then
							sqlname := l_executor_predicate_statement.binding.item(index)
						end
					end
				elseif { l_selector_statement: OLEDB_SELECTOR } sqlda.statement then
					if { l_selector_predicate_statement: PREDICATE_STATEMENT } l_selector_statement.predicate_statement then
						if l_selector_predicate_statement.binding.valid_index(index) then
							sqlname := l_selector_predicate_statement.binding.item(index)
						end
					end
				end
			end
			if sqlname = Void or else sqlname.is_empty then
				sqlname := "?# " + index.out
			end
			sqltype := accessor.get_parameter_type (index)
			sqllen := accessor.get_parameter_length (index)
			sqldata := accessor.get_parameter(index)
			set_oledb_type_string
		ensure then
			sqlname_is_set: sqlname /= Void and then not sqlname.is_empty
		end

	current_pointer: MANAGED_POINTER

 	set_value (a_value: ANY) is
			-- Set the value of this parameter to a_value
		local
			strn: STRING
			l_string_32: STRING_32
			str: STRING
			a: ANY
			ir: INTEGER_REF
			i8r: INTEGER_64_REF
			i4r: INTEGER_REF
			dr: DOUBLE_REF
			fr: REAL_REF
			rr: REAL_REF
			l_date_time: ROSE_DATE
			l_date: ROSE_DATE
			l_time: ROSE_TIME
			l_bytes: ARRAY [CHARACTER]
			l_area: SPECIAL [CHARACTER]
			l_char: CHARACTER_REF
			l_stream: STREAM
			ts: OLEDB_DBTIMESTAMP[STACK_MEMORY_MODEL]
			br: BOOLEAN_REF
			l_decimal: RP_DECIMAL
			l_sed_writer: ROSE_SED_MEMORY_READER_WRITER
			p: POINTER
		do
			if a_value = void then
				set_null_value (True)
			else
				set_null_value (False)
				if sqltype = dbtype_bytes then
					l_bytes ?= a_value
					if l_bytes /= void then
						l_area := l_bytes.area
						memcpy(accessor.get_parameter (index), $l_area, l_bytes.count + sizeof_int)
					else
						strn ?= a_value
						if strn /= void then
							accessor.set_string_parameter(index, strn )
						else
							raise ("wrong type: The Eiffel type and the database type do not match, ARRAY [CHARACTER} expected")
						end
					end
				elseif sqltype = dbtype_bytes| dbtype_byref then
					l_stream ?= a_value
					l_bytes ?= a_value
					if (l_stream /= Void) then
						create current_pointer.make_from_pointer (l_stream.item, l_stream.object_stored_size)
						p := current_pointer.item
						memcpy (accessor.get_parameter (index), $p , sizeof_voidstar)
						accessor.set_parameter_length (index, l_stream.object_stored_size)

					elseif l_bytes /= void then
						l_area := l_bytes.area
						create current_pointer.make_from_pointer ($l_area, l_bytes.count)
						p := current_pointer.item
						memcpy (accessor.get_parameter (index), $p , sizeof_voidstar)
						accessor.set_parameter_length (index, l_bytes.count)
					else
						strn ?= a_value
						if strn /= void then
							a := strn.area
							create current_pointer.make_from_pointer ($a, strn.count)
							p := current_pointer.item
							memcpy (accessor.get_parameter (index), $p , sizeof_voidstar)
							accessor.set_parameter_length (index, strn.count)
						else
							raise ("wrong type: The Eiffel type and the database type do not match, ARRAY [CHARACTER} expected")
						end
					end
				elseif sqltype = dbtype_dbtimestamp  or sqltype = dbtype_date then
					l_date_time ?= a_value
					l_date ?= a_value
					if l_date_time /= void then
						l_time := l_date_time.time
						create ts.make
						ts.set (accessor.get_parameter (index))
						ts.set_year(l_date_time.year)
						ts.set_month(l_date_time.month)
						ts.set_day(l_date_time.day)
						ts.set_hour(l_time.hour)
						ts.set_minute(l_time.minute)
						ts.set_second(l_time.second)
						ts.set_fraction(l_time.milli_second * 1_000_000)
						accessor.set_parameter_length (index, ts.structure_size)
					elseif l_date /= void then
						create ts.make
						ts.set (accessor.get_parameter (index))
						ts.set_year(l_date.year)
						ts.set_month(l_date.month)
						ts.set_day(l_date.day)
						ts.set_hour(0)
						ts.set_minute(0)
						ts.set_second(0)
						ts.set_fraction(0)
						accessor.set_parameter_length (index, ts.structure_size)
					else
						raise ("wrong type: The Eiffel type and the database type do not match, date expected")
					end
				elseif sqltype = dbtype_r8 then
					dr ?= a_value
					fr ?= a_value
					l_decimal ?= a_value;
					if dr /= void then
						if constants.ieee.isnan(dr.item) then
							set_null_value (True)
							accessor.set_double_parameter (index, constants.ieee.qnan)
						else
							set_null_value (False)
							accessor.set_double_parameter (index, dr.item)
						end
					elseif fr /= Void then
						if constants.ieee.isnan(fr.item) then
							set_null_value (True)
							accessor.set_double_parameter (index, constants.ieee.qnan)
						else
							set_null_value (False)
							accessor.set_real_parameter (index, fr.item)
						end
					elseif l_decimal /= Void then
						if not l_decimal.is_defined then
							set_null_value (True)
						else
							set_null_value (False)
							accessor.set_decimal_parameter (index, l_decimal)
						end
					else
						raise ("wrong type: The Eiffel type and the database type do not match, double expected")
					end
				elseif sqltype = dbtype_numeric then
					l_decimal ?= a_value;
					if (l_decimal = Void) then
						dr ?= a_value
						ir ?= a_value
					end
					if l_decimal /= Void then
						accessor.set_decimal_parameter (index, l_decimal)
					else
						if dr /= void then
							if constants.ieee.isnan(dr.item) then
								set_null_value (True)
								accessor.set_double_parameter (index, constants.ieee.qnan)
							else
								set_null_value (False)
								accessor.set_double_parameter (index, dr.item)
							end
						elseif ir /= Void then
							accessor.set_integer_value (index, ir.item)
						else
							raise ("wrong type: The Eiffel type and the database type do not match, numeric expected")
						end
					end
				elseif sqltype = dbtype_r4 then
					fr ?= a_value
					if fr /= void then
						if constants.ieee.isnan(fr.item) then
							set_null_value (True)
							accessor.set_double_parameter (index, constants.ieee.qnan)
						else
							set_null_value (False)
							accessor.set_real_parameter (index, fr.item)
						end
					else
						raise ("wrong type: The Eiffel type and the database type do not match, real expected")
					end
				elseif sqltype = dbtype_i1 or sqltype = dbtype_i2 or sqltype = dbtype_i4 or
					sqltype = dbtype_ui1 or sqltype = dbtype_ui2 or sqltype = dbtype_ui4 then
					ir ?= a_value
					if ir /= void then
						-- Yes, you don't have to set the parameter if it is NULL but if you don't you get misleading diagnostics.
						accessor.set_integer_parameter(index, ir.item)
						accessor.set_parameter_length(index, sizeof_int)
						if ir.item = null_integer then
							accessor.set_parameter_status (index, DBSTATUS_S_ISNULL)
						else
							accessor.set_parameter_status(index, DBSTATUS_S_OK)
						end
					else
						print (a_value)
						print (a_value.generator)
						print ("%N wrong type: The Eiffel type and the database type do not match, integer expected")
						raise ("wrong type: The Eiffel type and the database type do not match, integer expected")
					end
				elseif sqltype = dbtype_i8 then
					i8r ?= a_value
					i4r ?= a_value
					if i8r /= void then
						-- Yes, you don't have to set the parameter if it is NULL but if you don't you get misleading diagnostics.
						accessor.set_parameter_length(index, 8)
						accessor.set_parameter_status(index, DBSTATUS_S_OK)
						if i8r.item = null_integer then
							accessor.set_parameter_status (index, DBSTATUS_S_ISNULL)
						else
							accessor.set_integer_64_parameter (index, i8r.item)
						end
					elseif (i4r /= Void) then
						-- Yes, you don't have to set the parameter if it is NULL but if you don't you get misleading diagnostics.
						accessor.set_parameter_length(index, 8)
						accessor.set_parameter_status(index, DBSTATUS_S_OK)
						if i4r.item = null_integer then
							accessor.set_parameter_status (index, DBSTATUS_S_ISNULL)
						else
							accessor.set_integer_64_parameter (index, i4r.item)
						end
					else
						print (a_value)
						print (a_value.generator)
						print ("%N wrong type: The Eiffel type and the database type do not match, integer expected")
						raise ("wrong type: The Eiffel type and the database type do not match, integer expected")
					end
				elseif sqltype = dbtype_bool then
					br ?= a_value
					if br /= void then
						if br.item then
							accessor.set_integer_parameter(index, -1)
						else
							accessor.set_integer_parameter(index, 0)
						end
						accessor.set_parameter_length(index, sizeof_db_bool)
						--++--oledb_set_integer_value (sqlda.id, index, ir.item)
					else
						print (a_value)
						print (a_value.generator)
						print ("%N wrong type: The Eiffel type and the database type do not match, boolean expected")
						raise ("wrong type: The Eiffel type and the database type do not match, boolean expected")
					end
				elseif sqltype = dbtype_str  or sqltype = dbtype_wstr then
					strn ?= a_value
					if strn = void then
						l_char ?= a_value
						if l_char /= void then
							create strn.make (0)
							strn.append_character (l_char.item)
						end
					end
					l_string_32 ?= a_value
					if strn /= void then
						accessor.set_string_parameter(index, strn)
					elseif (l_string_32 /= Void)then
						accessor.set_string_32_parameter(index, l_string_32)
					else
						raise ("wrong type: The Eiffel type and the database type do not match, string of character expected")
					end
				elseif sqltype = dbtype_iunknown then
					strn ?= a_value
					l_stream ?= a_value
					if strn = void then
						l_char ?= a_value
						if l_char /= void then
							create strn.make (0)
							strn.append_character (l_char.item)
						end
					end
					if strn /= void then
						a := strn.to_c
						create current_stream.make_by_pointer ($a, strn.count)
						accessor.set_pointer_parameter(index, current_stream.p)
						current_stream.add_ref
						accessor.set_parameter_length(index, strn.count)
					elseif l_stream /= void then
						create current_stream.make_by_pointer(dereference_pointer(l_stream.buffer),l_stream.object_stored_size)
						accessor.set_pointer_parameter(index, current_stream.p)
						current_stream.add_ref
						accessor.set_parameter_length(index, l_stream.object_stored_size)
						--accessor.set_parameter_length(index, sizeof_voidstar)
						--accessor.set_length (index, sizeof_voidstart)
				--		sqltype := dbtype_bytes
				--		memcpy(accessor.get_parameter (index), dereference_pointer(l_stream.buffer), l_stream.object_stored_size)
				--		accessor.set_parameter_length(index, l_stream.object_stored_size)


					else
						l_sed_writer ?= a_value
						if l_sed_writer/=Void
						 then
							create current_stream.make_by_pointer(dereference_pointer(l_sed_writer.buffer.item), l_sed_writer.buffer_position)
							accessor.set_pointer_parameter(index, current_stream.p)
							current_stream.add_ref
							accessor.set_parameter_length(index, l_sed_writer.buffer_position)
						else
							raise ("wrong type: The Eiffel type and the database type do not match, string of character expected (dbtype_iunknown)")
						end
					end

				elseif sqltype = dbtype_variant then
					ir ?= a_value
					i8r ?= a_value
					l_date ?= a_value
					if ir /= Void then
						temp_variant.set (accessor.get_parameter(index))
						temp_variant.set_from_integer(ir.item)
						accessor.set_parameter_status(index, DBSTATUS_S_OK)

						accessor.set_parameter_length(index, 16)
					elseif (i8r /= Void) then
						temp_variant.set (accessor.get_parameter(index))
						temp_variant.set_from_integer_64(i8r.item)
						accessor.set_parameter_status(index, DBSTATUS_S_OK)

						accessor.set_parameter_length(index, 16)

					elseif (l_date /= Void) then
						temp_variant.set (accessor.get_parameter(index))
						temp_variant.set_date(l_date)
						accessor.set_parameter_status(index, DBSTATUS_S_OK)
						accessor.set_parameter_length(index, 16)
					else
						dr ?= a_value
						if dr /= Void then
							temp_variant.set (accessor.get_parameter(index))
							temp_variant.set_real8(dr.item)
							accessor.set_parameter_status(index, DBSTATUS_S_OK)
							accessor.set_parameter_length(index, 16)
						else
							rr ?= a_value
							if rr /= Void then
								temp_variant.set (accessor.get_parameter(index))
								temp_variant.set_real4(rr.item)
								accessor.set_parameter_status(index, DBSTATUS_S_OK)
								accessor.set_parameter_length(index, 16)
							else
								br ?= a_value
								if br /= Void then
									temp_variant.set (accessor.get_parameter(index))
									temp_variant.set_boolean(br.item)
									accessor.set_parameter_status(index, DBSTATUS_S_OK)
									accessor.set_parameter_length(index, 16)
								else
									str ?= a_value
									if str /= Void then
										temp_variant.set (accessor.get_parameter(index))
										temp_variant.set_from_string(str )
										accessor.set_parameter_status(index, DBSTATUS_S_OK)
										accessor.set_parameter_length(index, 16)
									else
										print ("invalid type for variant" + a_value.out)
									end
								end
							end
						end
					end
				else
					print ("unhandled type " + sqltype.out)
				end
			end
		end

feature {NONE} -- Helpers

	temp_variant: COM_VARIANT is
			-- Utility to access variable type data
		once
			create Result
		end

	current_stream: OLEDB_IN_MEMORY_SEQUENTIAL_STREAM
			-- Temporary stream object to help with data setting

end
