indexing
	description: "OLEDB property set"
	author: ""
	date: "$Date: 2008/12/01 $"
	revision: "$Revision: #13 $"

class
	OLEDB_PROPERTY_SET

inherit
	MEMORY_FUNCTIONS
		redefine out end
	COM_MEMORY_FUNCTIONS
		undefine out end
	OLEDB_PROPERTY_CONSTANTS
		undefine out end
	COM_VARENUM
		undefine out end
	COM_STRING_FUNCTIONS
		undefine out end

create
	make

feature {NONE} -- Initialization

	make (a_property_guid: POINTER) is
			-- Make with 'a_property_guid'.
		do
			item := calloc (structure_size, 1)
			memcpy (get_guidpropertyset, a_property_guid, sizeofguid)
			create current_property
			create current_variant
		end

feature -- Access

	item: POINTER

	out: STRING is
		do
			Result := Precursor
			Result.append ("%N")
			Result.append (get_cproperties.out)
			Result.append (" properties%N")
		end

feature -- Basic operations

	add_integer_property (prop_id: INTEGER; val: INTEGER) is
		do
			add
			current_property.set_dwpropertyid (prop_id)
			current_variant.set_vt (VT_I4)
			current_variant.set_lval (val)
		end

	add_boolean_property (prop_id: INTEGER; val: BOOLEAN) is
		do
			add
			current_property.set_dwpropertyid (prop_id)
			current_variant.set_vt (VT_BOOL)
			if val then
				current_variant.set_lval (-1)
			else
				current_variant.set_lval (0)
			end
		end

	add_bstr_property (prop_id: INTEGER; val: STRING) is
		require
			val_not_void: val /= Void
		local
			p1: POINTER
			bstr: POINTER
			a: ANY
			i: INTEGER
		do
			add
			current_property.set_dwpropertyid (prop_id)
			current_variant.set_vt (VT_bstr)
			p1 := calloc (2, val.count + 1)
			a := val.to_c
			i := MultiByteToWideChar(0, 0, $a, val.count, p1, val.count)
			bstr:= sysallocstring (p1)
			current_variant.set_bstrval (bstr)
			free (p1)
		end

	dispose is
		local
			i: INTEGER
		do
			if item /= default_pointer then
				from i := 0 until i >= get_cproperties loop
					variantclear (get_rgproperties + i * sizeofdbprop)
					i := i + 1
				end
				free (item)
			end
		end

feature {NONE} -- Implementation

	structure_size: INTEGER is
		external
			"C [macro <oledb.h>]"
		alias
			"sizeof(DBPROPSET)"
		end

	sizeofDBPROP: INTEGER is
		external
			"C [macro <oledb.h>]"
		alias
			"sizeof(DBPROP)"
		end

	sizeofguid: INTEGER is
		external
			"C [macro <oledb.h>]"
		alias
			"sizeof(GUID)"
		end

	add is
		do
			set_rgproperties (cotaskmemrealloc (get_rgproperties, (get_cproperties + 1) * sizeofdbprop))
			current_property.set (get_rgproperties + (get_cproperties) * sizeofdbprop)
			current_property.set_dwOptions (0)
			current_property.set_colid (DB_NULLID)
			current_variant.set (current_property.get_vValue)
			set_cproperties (get_cproperties + 1)
		end

	cwel_size_of_dbpropset: INTEGER is
		external
			"C [macro <oledbext.h>]"
		alias
			" sizeof(DBPROPSET)"
		end

	c_dbpropset_set_rgproperties (struct: POINTER; a_value: POINTER) is
		external
			"C [macro <oledbext.h>]"
		alias
			"c_dbpropset_set_rgproperties"
		end

	c_dbpropset_get_rgproperties (struct: POINTER): POINTER is
		external
			"C [macro <oledbext.h>]"
		alias
			"c_dbpropset_get_rgproperties"
		end

	set_rgproperties (a_value: POINTER) is
		do
			c_dbpropset_set_rgproperties (item, a_value)
		end

	get_rgproperties: POINTER is
		do
			Result := c_dbpropset_get_rgproperties (item)
		end

	c_dbpropset_set_cproperties (struct: POINTER; a_value: INTEGER) is
		external
			"C [macro <oledbext.h>]"
		alias
			"c_dbpropset_set_cproperties"
		end

	c_dbpropset_get_cproperties (struct: POINTER): INTEGER is
		external
			"C [macro <oledbext.h>]"
		alias
			"c_dbpropset_get_cproperties"
		end

	set_cproperties (a_value: INTEGER) is
		do
			c_dbpropset_set_cproperties (item, a_value)
		end

	get_cproperties: INTEGER is
		do
			Result := c_dbpropset_get_cproperties (item)
		end

	c_dbpropset_set_guidpropertyset (struct: POINTER; a_value: POINTER) is
		external
			"C [macro <oledbext.h>]"
		alias
			"c_dbpropset_set_guidpropertyset"
		end

	c_dbpropset_get_guidpropertyset (struct: POINTER): POINTER is
		external
			"C [macro <oledbext.h>]"
		alias
			"c_dbpropset_get_guidpropertyset"
		end

	set_guidpropertyset (a_value: POINTER) is
		do
			c_dbpropset_set_guidpropertyset (item, a_value)
		end

	get_guidpropertyset: POINTER is
		do
			Result := c_dbpropset_get_guidpropertyset (item)
		end

	current_property: OLEDB_PROPERTY

	current_variant: COM_VARIANT

	VariantClear(p: POINTER) is
		external
			"C [macro < windows.h>](VARIANT *)"
		alias
			"VariantClear"
		end

end -- class OLEDB_PROPERTY_SET
