note
	description: "?"
	author: "Pierre Nallet"
	last_editor: "$Author: rj $"
	date: "$Date: 2014/09/05 $"
	revision: "$Revision: #21 $"
	path: "$File: //rose/source/kernel/infrastructure/persistence/windows/classic2/oledb_rowset.e $"

class
	OLEDB_ROWSET

inherit

	OLEDB_ANY_ROWSET
		redefine
			set_accessor
		end

	OLEDB_IROWSETUPDATE_INTERFACE

create

	make

feature -- Implementation.

	close is
		local
			hr: INTEGER
		do
			if (p /= default_pointer) then
				hr := Release_Rows
				release
			end
		end

	set_accessor (acc: OLEDB_ANY_ACCESSOR) is
		do
			accessor := acc
		end

	accessor: OLEDB_ANY_ACCESSOR

	release_rows: INTEGER is
		require
			exists: exists
		local
			l_pp: POINTER
		do
			l_pp := m_hRow
			if (l_pp /= default_pointer) then
				Result := IRowset_ReleaseRows(p, 1, $l_pp, default_pointer, default_pointer, default_pointer)
				m_hRow := default_pointer;
			end
		end

	move_first: INTEGER is
		require
			exists: exists
		do
			Result := release_rows
			REsult := irowset_restartposition(p, default_pointer)
			if Result >= 0 then
				Result :=move_next
			end
		end

	move_next : INTEGER is
		require
			exists: exists
		do
			Result := move_next_ (0, true)
		end

	move_next_ (skip: INTEGER; forward: BOOLEAN): INTEGER is
		require
			exists: exists
		local
			l_phRow: POINTER
			l_hrow: POINTER
			fwd: INTEGER
			ulRowsFetched: INTEGER_64		-- Actually, we need 32 or 64 depending on the platform.
			hr: INTEGER
		do
			Result := Release_Rows
			if forward then
				fwd := 1
			else
				fwd := -1
			end
			l_hrow := m_hrow
			l_phRow := id_($l_hrow)
			Result := IRowset_GetNextRows(p, default_pointer, skip, fwd, $ulRowsFetched, $l_phRow)
			m_hrow := l_hrow
			if Result >= 0 and then Result /= DB_S_ENDOFROWSET then
				Result := get_data
				if Result <0 then
					hr := release_rows
				end
			end
		end

	get_data: INTEGER is
		require
			exists: exists
			has_accessor: accessor.haccessor /= accessor.db_invalid_haccessor
		do
			Result := irowset_getdata (p, m_hRow, accessor.haccessor, accessor.buffer)
		end

	insert : INTEGER is
		require
			exists: exists
			has_accessor: accessor.haccessor /= accessor.db_invalid_haccessor
		local
			l_prowsetchange: POINTER
			hr:INTEGER
			l_hrow: POINTER
		do
			Result := query_interface (IID_IRowsetChange, $l_prowsetchange)
			if Result >= 0 then
				l_hrow := m_hrow
				Result := irowsetchange_InsertRow(l_prowsetchange, default_pointer, accessor.haccessor, accessor.buffer, $l_hrow)
				m_hrow := l_hrow
				hr := iunknown_release (l_prowsetchange)
			end
		end

	set_data : INTEGER is
		require
			exists: exists
			has_accessor: accessor.haccessor /= accessor.db_invalid_haccessor
		local
			l_prowsetchange: POINTER
			hr:INTEGER
		do
			Result := query_interface (IID_IRowsetChange, $l_prowsetchange)
			if Result >= 0 then
				Result := IRowsetChange_SetData(l_prowsetchange, m_hrow, accessor.haccessor, accessor.buffer)
				hr := iunknown_release (l_prowsetchange)
			end
		end

	delete: INTEGER is
		require
			exists: exists
			has_accessor: accessor.haccessor /= accessor.db_invalid_haccessor
		local
			hr:INTEGER
			l_prowsetchange: POINTER
			status: INTEGER
			l_hrow: POINTER
		do
			Result := query_interface (IID_IRowsetChange, $l_prowsetchange)
			if Result >= 0 then
				l_hrow := m_hrow
				Result := IRowsetChange_DeleteRows(l_prowsetchange, default_pointer, 1, $l_hrow, $status)
				m_hrow := l_hrow
				hr := iunknown_release (l_prowsetchange)
			end
		end

	row_status: INTEGER is
		require
			exists: exists
			has_accessor: accessor.haccessor /= accessor.db_invalid_haccessor
		local
			hr:INTEGER
			l_prowsetupdate: POINTER
			l_hrow: POINTER
		do
			Result := query_interface (IID_IRowsetUpdate, $l_prowsetupdate)
			if Result >= 0 then
				l_hrow := m_hrow
				hr := iRowsetUpdate_GetRowStatus (l_prowsetupdate, default_pointer, 1, $l_hrow, $Result)
				m_hrow := l_hrow
				hr := iunknown_release (l_prowsetupdate)
			end
		end

	undo : INTEGER is
		require
			exists: exists
			has_accessor: accessor.haccessor /= accessor.db_invalid_haccessor
		local
			hr:INTEGER
			l_prowsetupdate: POINTER
			status: INTEGER
			l_hrow: POINTER
		do
			Result := query_interface (IID_IRowsetUpdate, $l_prowsetupdate)
			if Result >= 0 then
				l_hrow := m_hrow
				Result := iRowsetUpdate_undo (l_prowsetupdate, default_pointer, 1, $l_hrow, default_pointer, default_pointer, $status)
				m_hrow := l_hrow
				hr := iunknown_release (l_prowsetupdate)
			end
		end


	update : INTEGER is
		require
			exists: exists
			has_accessor: accessor.haccessor /= accessor.db_invalid_haccessor
		local
			hr:INTEGER
			l_prowsetupdate: POINTER
			status: INTEGER
			l_hrow: POINTER
		do
			Result := query_interface (IID_IRowsetUpdate, $l_prowsetupdate)
			if Result >= 0 then
				l_hrow := m_hrow
				Result := iRowsetUpdate_update (l_prowsetupdate, default_pointer, 1, $l_hrow, default_pointer, default_pointer, $status)
				m_hrow := l_hrow
				hr := iunknown_release (l_prowsetupdate)
			end
		end

	m_hrow: POINTER

	get_iid: POINTER is
		do
			Result := IID_IRowset
		end

feature -- Externals.

	IID_IRowset: POINTER is
		external
			"C [macro <oledb.h>]"
		alias
			"&IID_IRowset"
		end

	IRowset_AddRefRows(This: POINTER; crows: INTEGER; dghrows: POINTER; rgercounts: POINTER; rgrowstatus: POINTER): INTEGER is
			-- Note that DBCOUNTITEM is 32- or 64-bit depending on the platform.
		external
			"C blocking signature (IRowset *, DBCOUNTITEM, HROW*, DBREFCOUNT*, DBROWSTATUS*): EIF_INTEGER use <oledb.h>"
		alias
			"IRowset_AddRefRows"
		end


	IRowset_GetData(This: POINTER; hRow: POINTER; hAccessor: POINTER; pData: POINTER): INTEGER is
		external
			"C blocking signature (IRowset*, HROW, HACCESSOR, void*): EIF_INTEGER use <oledb.h>"
		alias
			"IRowset_GetData"
		end

	IRowset_GetNextRows(This: POINTER; hReserved: POINTER; lRowsOffset: INTEGER; cRows: INTEGER; pcRowsObtained: POINTER; hrows: POINTER): INTEGER is
			-- Note that DBCOUNTITEM is 32- or 64-bit depending on the platform.
			-- Note that DBROWOFFSET is 32- or 64-bit depending on the platform.  However, no pointers are used here.
			-- Note that DBROWCOUNT is 32- or 64-bit depending on the platform.  However, no pointers are used here.
		external
			"C blocking signature (IRowset*, HCHAPTER, DBROWOFFSET, DBROWCOUNT, DBCOUNTITEM*, HROW**): EIF_INTEGER use <oledb.h>"
		alias
			"IRowset_GetNextRows"
		end

	IRowset_ReleaseRows(This: POINTER; cRows: INTEGER_64; rghRows: POINTER; rgRowOptions: POINTER; rgRefCounts: POINTER; rgRowStatus: POINTER): INTEGER is
			-- Note that DBCOUNTITEM is 32- or 64-bit depending on the platform.
		external
			"C blocking signature (IRowset*, DBCOUNTITEM, HROW*, DBROWOPTIONS*, DBREFCOUNT*, DBROWSTATUS*): EIF_INTEGER use <oledb.h>"
		alias
			"IRowset_ReleaseRows"
		end

	IRowset_RestartPosition(This: POINTER; hReserved: POINTER): INTEGER is
		external
			"c blocking signature (IRowset*, HCHAPTER): EIF_INTEGER use <oledb.h>"
		alias
			"IRowset_RestartPosition"
		end

	DB_S_ENDOFROWSET: INTEGER is
		external
			"C [macro <OLEDBERR.H>]"
		alias
			"DB_S_ENDOFROWSET"
		end

end
