indexing

	description:

		"Object that provides the OLEDB implementation for the SQL_SELECTOR base class"

	author: "Pierre Nallet"
	last_updater: "$Author: pn $"
	date: "$Date: 2009/07/29 $"
	revision: "$Revision: #40 $"
	path: "$File: //rose/source/kernel/infrastructure/persistence/windows/classic2/oledb_selector.e $"

class OLEDB_SELECTOR

inherit

	SQL_SELECTOR
		rename
			execute as old_execute,
			open as old_open,
			release as sql_release,
			make as sql_make
		undefine
			sql_release
		redefine
			close,
			in_sqlda,
			out_sqlda,
			close_and_release_command,
			data_server
		select
			sql_make
		end

	OLEDB_STATEMENT
		rename
			make as statement_make
		undefine
			old_execute,
			clear_error
		redefine
			statement_make,
			in_sqlda,
			out_sqlda,
			data_server
		end

	OLEDB_COMMAND [OLEDB_DYNAMIC_PARAMETER_ACCESSOR, OLEDB_BULK_ROWSET_WITH_MULTIPLE_RESULTS]
		rename
			close as command_close,
			execute as command_execute
		redefine
			default_command_propset
		end

create

	make

feature -- Initialization

	make (a_connection: OLEDB_ANY_DATA_SERVER; a_statement: STRING; a_parameter_template: ARRAY [STRING]) is
			-- Initialize `Current'
		require
			valid_connection: a_connection /= Void
			valid_statement: a_statement /= Void
		local
			hr: INTEGER
			l_db_param_bind_info: OLEDB_DBPARAMBINDINFO [STACK_MEMORY_MODEL]
			l_sqls: SQLOLEDB_DATA_SERVER
			l_error_handler: OLEDB_ERROR_HANDLER_PRINT
		do
			l_sqls ?= a_connection
			data_server := a_connection
			statement := a_statement
			create predicate_statement.make (a_statement)
				-- predicate_statement.parse
			hr := open (a_connection.session, predicate_statement.parsed_string, Void)
			if a_parameter_template /= Void then
				create l_db_param_bind_info.make (Current, a_parameter_template)
			end
			prepare (0)
			hr := status
			if hr < 0 then
				create l_error_handler
				l_error_handler.on_error (hr, Current, generating_type + ".make")
			else
				hr := accessor.bind_columns (p)
				set_status (hr)
				if hr < 0 then
					create l_error_handler
					l_error_handler.on_error (hr, Current, generating_type + ".make")
				end
				hr := accessor.bind_parameters (p)
				if hr < 0 then
					--create l_error_handler
					--l_error_handler.on_error (hr, Current, generating_type + ".make")
				end
				data_server := a_connection
				create in_sqlda.make (Current, True)
				create out_sqlda.make (Current, False)
				id := 1
					-- dummy
				create cursor.make (a_connection, Current)
				success := True
			end
		end

feature {NONE} -- Initialization

	statement_make (a_connection: like data_server; a_statement: STRING) is
		do
			make (a_connection, a_statement, Void)
		end

feature -- Access

	cursor: OLEDB_CURSOR

	in_sqlda: OLEDB_SQLDA

	out_sqlda: OLEDB_SQLDA

	data_server: OLEDB_ANY_DATA_SERVER

feature -- Basic operations

	close is
			-- Close the cursor and the command
		do
			cursor.close
			command_close
		end

	close_and_release_command is
			-- Close the command and release the com object
		do
			close
			release
		end

	sql_release is
			-- Release this command and its resources
		do
			command_close
			release
		end

	execute is
			-- Execute this selector
		do
			set_status (command_execute (True))
		end

feature {NONE} -- Implementation

	default_command_propset: OLEDB_PROPERTY_SET is
		do
			Result := data_server.default_command_propset
		end

end
