indexing
	description: ""
	date: "$Date: 2008/12/01 $"
	revision: "$Revision: #72 $"

deferred class
	OLEDB_SQLVAR

inherit
	OLEDB_DBTYPE_CONSTANTS
	SQL_SQLVAR
		rename
			make as old_make
		redefine
			get_field, is_null, set_null_value, sqlda
		end
	BASIC_ROUTINES
	MEMORY_FUNCTIONS
	ROSE_CONSTANTS
	OLEDB_DBSTATUS_CONSTANTS

feature -- Initialization

	make (a_sqlda: OLEDB_SQLDA; a_index: INTEGER) is
		do
			sqlda := a_sqlda
			accessor := sqlda.statement.accessor
			index := a_index
			set_attributes
		end

feature -- Access

	sqlda: OLEDB_SQLDA

	is_null: BOOLEAN is
		do
			Result := status = dbstatus_s_isnull
		end

	status: INTEGER is
		deferred
		end

	set_status (s: INTEGER) is
		deferred
		end

	accessor: OLEDB_DYNAMIC_PARAMETER_ACCESSOR

feature -- Basic operations

	set_null_value (a_value: BOOLEAN) is
		do
			if a_value then
				set_status (dbstatus_s_isnull)
			else
				set_status (dbstatus_s_OK)
			end
		end

	Field_type_string: STRING is ""

	Array_type: STRING is ""

	value: ANY is
		deferred
		end

	feature_body: STRING is
		do
		end

	get_field (an_index: INTEGER; receptacle: ANY) is
		local
			l_string: STRING
			l_date_time: ROSE_DATE
			l_date: ROSE_DATE
			l_integer_value: INTEGER
		do
			inspect internal.field_type (an_index, receptacle)
			when {INTERNAL}.reference_type then
				if status /= dbstatus_s_isnull and status /= dbstatus_s_OK then
					print ("invalid status" + status.out)
				end
				if is_null then
					internal.set_reference_field (an_index, receptacle, Void)
				else
					l_string ?= internal.field (an_index, receptacle)
					l_date_time ?= internal.field (an_index, receptacle)
					l_date ?= internal.field (an_index, receptacle)
					l_string ?= internal.field (an_index, receptacle)
					if l_string /= void then
						l_string.copy (string_value)
					elseif l_date_time /= void then
						internal.set_reference_field (an_index, receptacle, date_time_value)
					elseif l_date /= void then
						internal.set_reference_field (an_index, receptacle, date_value)
					else
						internal.set_reference_field (an_index, receptacle, value)
					end
				end
			when {INTERNAL}.character_type then
				internal.set_character_field (an_index, receptacle, character_value)
			when {INTERNAL}.double_type then
				internal.set_double_field (an_index, receptacle, double_value)
			when {INTERNAL}.real_type then
				internal.set_real_field (an_index, receptacle, real_value)
			when {INTERNAL}.integer_type then
				if sqltype=Dbtype_i4
				then
				   l_integer_value := integer_value
				elseif sqltype=Dbtype_i2
				then
				   -- This is slow.  Need to put in faster solution.
				   if integer_value /= null_integer then
				      l_integer_value := (integer_value * 65536) // 65536
				   end
				elseif sqltype=Dbtype_i1
				then
				   -- This is slow.  Need to put in faster solution.
				   if integer_value /= null_integer then
				      l_integer_value := (integer_value * 65536 * 256) // (65536 * 256)
				   end
				elseif sqltype=Dbtype_ui1
				then
				   if integer_value /= null_integer then
				      l_integer_value := integer_value.bit_and(255)
				   end
				end

			    internal.set_integer_field (an_index, receptacle, l_integer_value)
			when {INTERNAL}.boolean_type then
				internal.set_boolean_field (an_index, receptacle, boolean_value)
			else
				print ("invalid type:  %N")
				print (internal.field_type (an_index, receptacle).out)
			end
		end

	bytes_value: STRING is
		local
			l_count: INTEGER
			--l_any: ANY
		do
			l_count := accessor.get_length(index)
			Result := makestr(accessor.get_value(index),l_count)
			--create Result.make (1, l_count)
			--l_any := Result.to_c
			--memcpy ($l_any, accessor.get_value (index), l_count)
		end


	set_value (l_value: ANY) is
		deferred
		end

	set_type (type_: ANY) is
		do
		end

	set_null is
		do
			set_null_value (True)
		end

	set_non_null is
		do
			set_null_value (False)
		end

	set_oledb_type_string is
		do
			if sqltype= dbtype_bytes then
				type_string := "STRING"
			elseif sqltype=  dbtype_dbtimestamp or sqltype= dbtype_date then
				type_string := "ROSE_DATE"
			elseif sqltype= dbtype_bool then
				type_string := "BOOLEAN"
			elseif sqltype= dbtype_r8 or sqltype=DBTYPE_NUMERIC then
				type_string := "DOUBLE"
			elseif sqltype= dbtype_r4 then
				type_string := "REAL"
			elseif sqltype= dbtype_i1 or sqltype= dbtype_i2 or sqltype= dbtype_i4 or
				   sqltype= dbtype_ui1 or sqltype= dbtype_ui2 or sqltype= dbtype_ui4 then
				type_string := "INTEGER"
			elseif sqltype= dbtype_bstr or sqltype= dbtype_str or sqltype= dbtype_wstr or sqltype=  dbtype_iunknown  then
				type_string := "STRING"
			else
				type_string := clone ("Unknown type (")
				type_string.append (sqltype.out)
				type_string.append (")")
			end
		end

	date_time_value: ROSE_DATE is
		deferred
		end


	date_value: ROSE_DATE is
		deferred
		end

	read_specific_text: STRING is
		do
		end

	write_specific_text: STRING is
		do
		end

feature {NONE}

	set_attributes is
		deferred
		end

	string_out (s: STRING): STRING is
		local
			i: INTEGER
		do
			Result := "%N["
			Result.append (s.count.out)
			Result.append ("]")
			from i := 1 until i > s.count.min (20) loop
				Result.append (s.item(i).code.out)
				Result.append (".")
				i := i + 1
			end
			Result.append ("[")
		end

	raise (a_message: STRING) is
		do
			(create {EXCEPTIONS}).raise (a_message)
		end
		
end -- class OLEDB_SQLVAR
