note
	description: "Commands that return a stream instead of a rowset."
	author: "Pierre Nallet"
	last_editor: "$Author: rj $"
	date: "$Date: 2012/07/19 $"
	revision: "$Revision: #14 $"
	path: "$File: //rose/source/kernel/infrastructure/persistence/windows/classic/oledb_stream_command.e $"

class
	OLEDB_STREAM_COMMAND

inherit
	OLEDB_SIMPLE_COMMAND [OLEDB_DYNAMIC_PARAMETER_ACCESSOR, OLEDB_STREAM_ROWSET]
		rename
			execute as command_execute
		redefine
			bind,execute_
		end
	OLEDB_DBSTATUS_CONSTANTS
	OLEDB_DBTYPE_CONSTANTS
	ROSE_CONSTANTS
	DATA_SERVER_STREAM_COMMAND
	OLEDB_SIZEOF

create
	make

feature -- Initialization

	make (a_server: SQLOLEDB_DATA_SERVER; a_text: STRING) is
		do
			debug("make_movies")
				create cam.make(generating_type + "/" + a_server.node + "/" + a_server.database + "/" + a_text)
			end

			server := a_server
			open_stream (a_text)
		end

feature -- Access

	server: SQLOLEDB_DATA_SERVER	-- The server that this command was built from.

feature -- Basic operations

	open_stream (a_statement: STRING) is
			--
		local
			hr: INTEGER
		do
			if playback_movie
			then
				cam.compare_movie_frame("open_stream", a_statement)
			else
				hr := open (server.session, a_statement, Void)
				prepare (0)
				hr := status
				if hr < 0 then
					--| trace_error (hr)
				else
					--| hr := accessor.bind_columns (p)
					if hr < 0 then
						--| trace_error (hr)
					else
						hr := accessor.bind_parameters (p)
						if hr < 0 then
							--| trace_error (hr)
						else

						end
					end
				end

				debug("make_movies")
					if record_movie
					then
						cam.record_movie_frame("open_stream", a_statement)
					end
				end
			end
		end

	retrieve (a_parameters: ARRAY [ANY]) is
		local
			i: INTEGER
			l_pair: PAIR[ARRAY[ANY], STRING]
		do
			if playback_movie
			then
				l_pair ?= cam.next_frame("retrieve").frame_data
				cam.compare_partial_movie_frame(a_parameters, l_pair.first)
				query_result := l_pair.second
			else
				debug("make_movies")
					if record_movie
					then
						create l_pair.make(deep_clone(a_parameters), Void)
					end
				end
				set_status (command_execute (True))
				if a_parameters /= Void then
					from i := 0 until i >= a_parameters.Count loop
						set_parameter (a_parameters.item (i + a_parameters.lower), i + 1)
						i := i + 1
					end
				end

				query_result := rowset.read_string
				debug("make_movies")
					if record_movie
					then
						l_pair.set_second(query_result)
						cam.record_movie_frame("retrieve", l_pair)
					end
				end
			end
		end

	bind: INTEGER is
			-- (from OLEDB_ACCESSOR_ROWSET)
		do
		end

	query_result: STRING
	current_stream: OLEDB_IN_MEMORY_SEQUENTIAL_STREAM
			-- Current stream

 	set_parameter (l_value:ANY; index: INTEGER) is
			-----oledb_pointer_to_sqlvar($l_value,sqldata,sqltype,sqloctet_len,scale)
		local
			strn: STRING
			a: ANY
			ir: INTEGER_REF
			dr: DOUBLE_REF
			fr: REAL_REF
			l_date_time: ROSE_DATE
			l_date: ROSE_DATE
			l_time : ROSE_TIME
			l_bytes: ARRAY [CHARACTER]
			l_area: SPECIAL [CHARACTER]
			l_char: CHARACTER_REF
			l_stream : STREAM
			ts: OLEDB_DBTIMESTAMP [STACK_MEMORY_MODEL]
			br: BOOLEAN_REF
			sqltype: INTEGER
		do
			if l_value = Void then
				accessor.set_parameter_status (index, dbstatus_s_isnull)
			else
				accessor.set_parameter_status (index, dbstatus_s_ok)
				sqltype := accessor.get_parameter_type (index)
				--sqllen := accessor.get_parameter_length (index)
				if sqltype = dbtype_bytes then
					l_bytes ?= l_value
					if l_bytes /= Void then
						l_area := l_bytes.area
						memcpy (accessor.get_parameter (index), $l_area, l_bytes.count + sizeof_int)
					else
						strn ?= l_value
						if strn /= Void then
							accessor.set_string_parameter (index, strn )
						else
							raise ("wrong type: The Eiffel type and the database type do not match, ARRAY [CHARACTER} expected")
						end
					end
				elseif sqltype = dbtype_dbtimestamp or sqltype = dbtype_date then
					l_date_time ?= l_value
					l_date ?= l_value
					if l_date_time /= Void then
						l_time := l_date_time.time
						create ts.make
						ts.set (accessor.get_parameter (index))
						ts.set_year (l_date_time.year)
						ts.set_month (l_date_time.month)
						ts.set_day (l_date_time.day)
						ts.set_hour (l_time.hour)
						ts.set_minute (l_time.minute)
						ts.set_second (l_time.second)
						ts.set_fraction(l_time.milli_second * 1_000_000)
						accessor.set_parameter_length (index, ts.structure_size)
					elseif l_date /= Void then
						create ts.make
						ts.set (accessor.get_parameter (index))
						ts.set_year (l_date.year)
						ts.set_month (l_date.month)
						ts.set_day (l_date.day)
						ts.set_hour (0)
						ts.set_minute (0)
						ts.set_second (0)
						ts.set_fraction (0)
						accessor.set_parameter_length (index, ts.structure_size)
					else
						raise ("wrong type: The Eiffel type and the database type do not match, date expected")
					end
				elseif sqltype = dbtype_r8 then
					dr ?= l_value
					if dr /= Void then
						if constants.ieee.isnan (dr.item) then
							accessor.set_parameter_status (index, dbstatus_s_isnull)
						else
							accessor.set_parameter_status (index, dbstatus_s_OK)
							accessor.set_double_parameter (index, dr.item)
						end
					else
						raise ("wrong type: The Eiffel type and the database type do not match, double expected")
					end
				elseif sqltype = dbtype_r4 then
					fr ?= l_value
					if fr /= Void then
						if constants.ieee.isnan (fr.item) then
							accessor.set_parameter_status (index, dbstatus_s_isnull)
						else
							accessor.set_parameter_status (index, dbstatus_s_OK)
							accessor.set_real_parameter (index, fr.item)
						end
					else
						raise ("wrong type: The Eiffel type and the database type do not match, double expected")
					end
				elseif sqltype = dbtype_i1 or sqltype = dbtype_i2 or sqltype = dbtype_i4 or
					sqltype = dbtype_ui1 or sqltype = dbtype_ui2 or sqltype = dbtype_ui4 then
					ir ?= l_value
					if ir /= Void then
						if ir.item = constants.null_integer then
							accessor.set_parameter_status (index, DBSTATUS_S_ISNULL)
						else
							accessor.set_integer_parameter (index, ir.item)
							accessor.set_parameter_length (index, sizeof_int)
							accessor.set_parameter_status (index, DBSTATUS_S_OK)
						end
					else
						print (l_value)
						print (l_value.generator)
						print ("%N wrong type: The Eiffel type and the database type do not match, integer expected")
						raise ("wrong type: The Eiffel type and the database type do not match, integer expected")
					end
				elseif sqltype = dbtype_bool then
					br ?= l_value
					if br /= Void then
						if br.item then
							accessor.set_integer_parameter (index, -1)
						else
							accessor.set_integer_parameter (index, 0)
						end
						accessor.set_parameter_length (index, sizeof_db_bool)
						--++--oledb_set_integer_value (sqlda.id, index, ir.item)
					else
						print (l_value)
						print (l_value.generator)
						print ("%N wrong type: The Eiffel type and the database type do not match, boolean expected")
						raise ("wrong type: The Eiffel type and the database type do not match, boolean expected")
					end
				elseif sqltype = dbtype_str or sqltype = dbtype_wstr then
					strn ?= l_value
					if strn = Void then
						l_char ?= l_value
						if l_char /= Void then
							create strn.make (0)
							strn.append_character (l_char.item)
						end
					end
					if strn /= Void then
						accessor.set_string_parameter (index, strn)
					else
						raise ("wrong type: The Eiffel type and the database type do not match, string of character expected")
					end
				elseif sqltype = dbtype_iunknown then
					strn ?= l_value
					l_stream ?= l_value
					if strn = Void then
						l_char ?= l_value
						if l_char /= Void then
							create strn.make (0)
							strn.append_character (l_char.item)
						end
					end
					if strn /= Void then
						a := strn.to_c
						create current_stream.make_by_pointer ($a, strn.count)
						accessor.set_pointer_parameter (index, current_stream.p)
						current_stream.add_ref
						accessor.set_parameter_length (index, strn.count)
					elseif l_stream /= Void then
						create current_stream.make_by_pointer (dereference_pointer (l_stream.buffer),l_stream.object_stored_size)
						accessor.set_pointer_parameter (index, current_stream.p)
						current_stream.add_ref
						accessor.set_parameter_length (index, l_stream.object_stored_size)
					else
						raise ("wrong type: The Eiffel type and the database type do not match, string of character expected (dbtype_iunknown)")
					end
--				elseif sqltype = dbtype_variant then
--					ir ?= l_value
--					if ir /= Void then
--						temp_variant.set (accessor.get_parameter (index))
--						temp_variant.set_from_integer (ir.item)
--						accessor.set_parameter_status (index, DBSTATUS_S_OK)
--
--						accessor.set_parameter_length (index, 16)
--
--					else
--						dr ?= l_value
--						if dr /= Void then
--							temp_variant.set (accessor.get_parameter (index))
--							temp_variant.set_real8 (dr.item)
--							accessor.set_parameter_status (index, DBSTATUS_S_OK)
--							accessor.set_parameter_length (index, 16)
--						else
--							rr ?= l_value
--							if rr /= Void then
--								temp_variant.set (accessor.get_parameter (index))
--								temp_variant.set_real4 (rr.item)
--								accessor.set_parameter_status (index, DBSTATUS_S_OK)
--								accessor.set_parameter_length (index, 16)
--							else
--								br ?= l_value
--								if br /= Void then
--									temp_variant.set (accessor.get_parameter (index))
--									temp_variant.set_boolean (br.item)
--									accessor.set_parameter_status (index, DBSTATUS_S_OK)
--									accessor.set_parameter_length (index, 16)
--								else
--									str ?= l_value
--									if str /= Void then
--										temp_variant.set (accessor.get_parameter (index))
--										temp_variant.set_from_string (str )
--										accessor.set_parameter_status (index, DBSTATUS_S_OK)
--										accessor.set_parameter_length (index, 16)
--									else
--										print ("invalid type for variant" + l_value.out)
--									end
--								end
--							end
--						end
--					end
				end
			end
		end

feature {NONE} -- Implementation

	 execute_(a_rowset: OLEDB_STREAM_ROWSET; pParams: OLEDB_DBPARAMS[MEMORY_MODEL]; propset: OLEDB_PROPERTY_SET): INTEGER is
	 		-- Execute the command. no support for multiple commands
		local
			l_pCommandProperties: POINTER
			prm: POINTER
			hr: INTEGER
			l_p: POINTER
			l_i: INTEGER
		--	l_multiple_results: POINTER
			l_found: BOOLEAN
		do
			if propset /= Void then
				Result := query_interface(IID_ICommandProperties, $l_pcommandproperties)
				if Result >= 0 then
					Result := icommandproperties_SetProperties(l_pcommandproperties, 1, propset.item)
					hr := iunknown_release (l_pcommandproperties)
				end
			end
			if Result>= 0 then
				if pparams /= Void then
					prm := pparams.item
				end
				a_rowset.release	-- Since 'a_rowset' may get reused, we must release the previous (if any) rowset.
				l_p := a_rowset.p
				l_i := rows_affected
				iCommand_Execute2(p, default_pointer, a_rowset.get_iid, prm, $l_i,  $l_p, $Result)
--				iCommand_Execute2(p, default_pointer, IID_IMultipleResults, prm, $l_i,  $l_multiple_results, $Result)
				if (false ) then -- l_multiple_results /= default_pointer) then
				--	print ("Has multiple results")
					from until l_found loop

					--	hr := imultipleresults_getresult (l_multiple_results, default_pointer, 0, a_rowset.get_iid, $l_i, $l_p)
						l_found := hr /= 0 or l_p /= default_pointer
					end
--					if (l_p = default_pointer) then
--						hr := imultipleresults_getresult (l_multiple_results, default_pointer, 0, a_rowset.get_iid, $l_i, $l_p)
--					end
				--	hr := iunknown_release (l_multiple_results)
				end
				rows_affected := l_i
				a_rowset.set_pointer (l_p)
			end
		end

	dereference_pointer (source_: POINTER): POINTER is
		external
			"C"
		end

end
