note

	description:

		"Mechanism for simulating a once feature in zone which is specific to the actual zones"

	author: "Mark Howard"
	last_editor: "$Author: mb $"
	date: "$Date: 2011/03/09 $"
	revision: "$Revision: #9 $"
	path: "$File: //rose/source/kernel/investment_entities/once_per_zone.e $"

class ONCE_PER_ZONE [G -> ZONAL_DATA create make end]

inherit

	ROSE_LINEAR_ARRAY [G]
		rename
			make as array_make,
			item as array_item
		export
			{ONCE_PER_ZONE} all
			{ANY} clear_all
		end

create

	make

create {ONCE_PER_ZONE}

	make_empty,
	make_from_array,
	array_make

feature {NONE} -- Creation

	make is
			-- Initialization
		do
			array_make (0, 255)
		end
		
feature -- Access
	
	item (a_zone: ZONE [LOCATION]): G is
			-- Item specific to 'a_zone'
		require
			valid_zone: a_zone /= Void
		local
			l_code: INTEGER
		do
			l_code := a_zone.mnemonic.code
			if array_item (l_code) /= Void then
				Result := array_item (l_code)
			else
				create Result.make (a_zone)
				force (Result, l_code)
			end
		ensure
			valid_result: Result /= Void
		end

feature {ZONE} -- Implementation

	install (a_g: G; a_zone: ZONE [LOCATION]) is
			-- Force 'a_g' to be the item for 'a_zone'
		require
			valid_zone: a_zone /= Void
		local
			l_code: INTEGER
		do
			l_code := a_zone.mnemonic.code
			force (a_g, l_code)
		end

invariant

	valid_length: count = 256

end
