note

	description:

		"Override audit fields"

	author: "Srilatha Singh"
	last_editor: "$Author: er $"
	date: "$Date: 2013/01/30 $"
	revision: "$Revision: #6 $"
	path: "$File: //rose/source/kernel/data/override_audit_fields.e $"

class

	OVERRIDE_AUDIT_FIELDS

inherit

	ROSE_SHARED_GLOBAL

create

	make, make_null

feature -- Initialization

	make_null is do end

	make (a_start_valid_date, a_end_valid_date: ROSE_DATE; a_override_type_id: INTEGER; a_entry_date: ROSE_DATE; a_user_id, a_comment: STRING) is
		do
			start_valid_date := a_start_valid_date  -- DO WE NEED TO CLONE? I doubt it.
			end_valid_date := a_end_valid_date
			override_type_id := a_override_type_id
			entry_date := a_entry_date
			user_id := a_user_id
			comment := a_comment
		end

	copy_audit_fields (a_start_valid_date, a_end_valid_date: ROSE_DATE; a_override_type_id: INTEGER; a_entry_date: ROSE_DATE; a_user_id, a_comment: STRING) is
		do
			start_valid_date := a_start_valid_date
			end_valid_date := a_end_valid_date
			override_type_id := a_override_type_id
			entry_date := a_entry_date
			user_id := a_user_id
			comment := a_comment
		end

	set_audit_fields (a_object: OVERRIDE_AUDIT_FIELDS) is
		require
			a_object_not_void: a_object /= Void
		do
			start_valid_date := a_object.start_valid_date  -- DO WE NEED TO CLONE? I doubt it.
			end_valid_date := a_object.end_valid_date
			override_type_id := a_object.override_type_id
			entry_date := a_object.entry_date
			user_id := a_object.user_id
			comment := a_object.comment
		end

feature -- Signature fields

	start_valid_date:ROSE_DATE
	set_start_valid_date(a_value:ROSE_DATE) is
			-- Set `start_valid_date' to `a_value'.
		do
			start_valid_date:= a_value
		end

	end_valid_date:ROSE_DATE
	set_end_valid_date(a_value:ROSE_DATE) is
		do
			end_valid_date:= a_value
		end

	override_type_id: INTEGER
	set_override_type_id(a_value:INTEGER) is
		do
			override_type_id:= a_value
		end

	entry_date:ROSE_DATE
	set_entry_date(a_value:ROSE_DATE) is
		do
			entry_date:= a_value
		end

	user_id:STRING
	set_user_id(a_value:STRING) is
		do
			user_id:= a_value
		end

	abbreviated_user_id (a_user_id: STRING): STRING is
		-- convert "RIEM\HD" to "HD"
		local
			l_zero_index: INTEGER
		do
			if a_user_id /= Void then
				l_zero_index := a_user_id.index_of ('\', 1)
				if l_zero_index > 0 and l_zero_index + 1 <= a_user_id.count then
					Result := a_user_id.substring (l_zero_index + 1, a_user_id.count)
				else
					Result := a_user_id
				end
			end
		end

	username: STRING is
		once
			Result := global.app.user.full_login
			if Result = Void then
				Result := "?"
			end
		end

	comment:STRING
	set_comment(a_value:STRING) is
		do
			comment:= a_value
		end

feature -- comparison

	audit_fields_match (a_start_valid_date, a_end_valid_date: ROSE_DATE; a_override_type_id: INTEGER; a_entry_date: ROSE_DATE; a_user_id, a_comment: STRING): BOOLEAN is
		do
			Result := a_start_valid_date.is_equal (start_valid_date) and then
						a_end_valid_date.is_equal (end_valid_date) and then
						a_override_type_id = override_type_id and then
						a_entry_date.is_equal (entry_date) and then
						a_user_id.is_equal (user_id) and then
						string_algebra.strings_are_equal_or_both_void (a_comment, comment)
		end

	audit_fields_match_in_the_meta_sense (a_start_valid_date, a_end_valid_date: ROSE_DATE; a_override_type_id: INTEGER; a_entry_date: ROSE_DATE; a_user_id, a_comment: STRING): BOOLEAN is
		do
--			Result := a_start_valid_date.is_equal (start_valid_date) and then
--						a_end_valid_date.is_equal (end_valid_date) and then
			Result := a_override_type_id = override_type_id and then
						a_entry_date.is_equal (entry_date) and then
						a_user_id.is_equal (user_id) and then
						string_algebra.strings_are_equal_or_both_void (a_comment, comment)
		end

	string_algebra: STRING_ALGEBRA is
		once
			create Result
		end

feature -- A deleted override

	mark_as_deleted is
		local
			l_date: ROSE_DATE
		do
			create l_date.make_by_date (start_valid_date)
			l_date.add_n_days (-1)
			set_end_valid_date (l_date)
		ensure
			considered_deleted: is_considered_deleted
		end

	is_considered_deleted: BOOLEAN is
		do
			Result := end_valid_date < start_valid_date
		end

	is_alive: BOOLEAN is
		do
			Result := not is_considered_deleted and then
				start_valid_date <= current_date and then
				current_date <= end_valid_date 	-- added this last condition as an afterthought, on May 23, 05.
		end

	current_date: ROSE_DATE is
		once
			create Result.make_now
		end

	beginning_of_time:ROSE_DATE is
		once
			create Result.make_min_date -- 19000101
		end

	end_of_time: ROSE_DATE is
		once
			create Result.make_max_date -- 99991231
		end

feature -- A meta override (obsolete)

	mark_as_meta is
		local
			l_is_considered_deleted: BOOLEAN
		do
			l_is_considered_deleted := is_considered_deleted

			set_start_valid_date (first_current_date_ever)
			if not l_is_considered_deleted then
				set_end_valid_date (last_current_date_ever)
			else
				mark_as_deleted
			end
		end

	first_current_date_ever: ROSE_DATE is
		once
			create Result.make_min_date -- 19000101
		end

	last_current_date_ever: ROSE_DATE is
		once
			create Result.make_max_date -- 99991231	-- Once the date 9999/12/31 occurs, this will no longer be correct!
		end

feature {RCMD_SEGMENT_DATA_MAPPING_OVERRIDE_GUI} -- Dates

	min_date_or_date_itself (a_date: ROSE_DATE): ROSE_DATE is
		do
			Result := a_date
			if Result = Void then
				Result := min_date
			end
		end

	max_date_or_date_itself (a_date: ROSE_DATE): ROSE_DATE is
		do
			Result := a_date
			if Result = Void then
				Result := max_date
			end
		end

	min_date: ROSE_DATE is
		once
			create Result.make_min_date
		end

	max_date: ROSE_DATE is
		once
			create Result.make_max_date
		end

feature -- display

	display is
		do
			print ("Override_type_id        = " + override_type_id.out + "%N")
			print ("Start_valid_date        = " + display_date_even_if_void (start_valid_date) + "%N")
			print ("End_valid_date          = " + display_date_even_if_void (end_valid_date) + "%N")
			print ("Comment                 = " + string_algebra.displayed_string_even_if_void (comment) + "%N")
			print ("Entry date              = " + entry_date.out + "%N")
			print ("User name               = " + user_id.out + "%N")
		end

	display_date_even_if_void (a_date: ROSE_DATE): STRING is
		do
			if a_date /= Void then
				Result := a_date.ccyymmdd.out
			else
				Result := string_algebra.Void_string
			end
		end

end -- class OVERRIDE_AUDIT_FIELDS
