note

	description:

		"binary partition tree"

	keywords: "tree,partition,atlas"
	author: "Mark Howard"
	last_editor: "$Author: mb $"
	date: "$Date: 2011/06/21 $"
	revision: "$Revision: #14 $"
	path: "$File: //rose/source/kernel/infrastructure/structures/partition_tree.e $"

class PARTITION_TREE [G -> COMPARABLE, H -> COMPARABLE]

create

	make

feature {NONE} -- Initialization

	make (a_item: PARTITION_TREE_RECORD [G, H]) is
			-- Create the tree
		require
			valid_item: a_item /= Void
		do
			item := a_item
		end

feature -- Access

	item: PARTITION_TREE_RECORD [G, H]

	left_child: PARTITION_TREE [G, H]

	right_child: PARTITION_TREE [G, H]
	
feature -- Iteration

	do_all (a_action: PROCEDURE [ANY, TUPLE [PARTITION_TREE [G, H]]]) is
			-- Apply `a_action' to every tree item
		require
			valid_action: a_action /= Void
		do
			a_action.call ([Current])
			if left_child /= Void then
				left_child.do_all (a_action)
			end
			if right_child /= Void then
				right_child.do_all (a_action)
			end
		end
	
feature -- Query

	path (a_key_0: G; a_key_1: H): INTEGER is
			-- Path identifier for `a_key_0' and `a_key_1'
		do
			Result := leaf_record (a_key_0, a_key_1).path
		end

	leaf_record (a_key_0: G; a_key_1: H): PARTITION_TREE_RECORD [G, H] is
			-- Leaf for `a_key_0' and `a_key_1'
		do
			if last_leaf_record /= Void and then last_leaf_record.contains (a_key_0, a_key_1) then
				Result := last_leaf_record
			else
				Result := leaf_record_internal (a_key_0, a_key_1)
				last_leaf_record := Result
			end
		end

	sibling (a_leaf: PARTITION_TREE_RECORD [G, H]): PARTITION_TREE_RECORD [G, H] is
		do
			if left_child = Void then
			elseif left_child.item.min_0 <= a_leaf.min_0 and then
				left_child.item.max_0 >= a_leaf.max_0 and left_child.item.min_1 <= a_leaf.min_1 and then
				left_child.item.max_1 >= a_leaf.max_1 then
				Result := left_child.sibling (a_leaf)
				if Result = Void then
					Result := right_child.item
				end
			else
				Result := right_child.sibling (a_leaf)
				if Result = Void then
					Result := left_child.item
				end
			end
		end

	leaves (a_node: PARTITION_TREE [G, H]): ROSE_LINEAR_ARRAY [PARTITION_TREE_RECORD [G, H]] is
		do
			if a_node.left_child = Void then
				create Result.make_empty
				Result.extend (a_node.item)
			else
				Result := leaves (a_node.left_child)
				Result.append_array (leaves (a_node.right_child))
			end
		end

	parent (a_leaf: PARTITION_TREE_RECORD [G, H]): PARTITION_TREE [G, H] is
		do
			if item = a_leaf then
			elseif left_child.item.min_0 <= a_leaf.min_0 and then
				left_child.item.max_0 >= a_leaf.max_0 and left_child.item.min_1 <= a_leaf.min_1 and then
				left_child.item.max_1 >= a_leaf.max_1 then
				Result := left_child.parent (a_leaf)
				if Result = Void then
					Result := Current
				end
			else
				Result := right_child.parent (a_leaf)
				if Result = Void then
					Result := Current
				end
			end
		end

	key_0_paths (a_key_0: G): ROSE_LINEAR_ARRAY [INTEGER] is
		do
			create Result.make_empty
			if left_child = Void then
				Result.extend (item.path)
			else
				if left_child.item.min_0 <= a_key_0 and then left_child.item.max_0 >= a_key_0 then
					Result.append_array (left_child.key_0_paths (a_key_0))
				end
				if right_child.item.min_0 <= a_key_0 and then right_child.item.max_0 >= a_key_0 then
					Result.append_array (right_child.key_0_paths (a_key_0))
				end
			end
		end

	key_1_paths (a_key_1: H): ROSE_LINEAR_ARRAY [INTEGER] is
		do
			create Result.make_empty
			if left_child = Void then
				Result.extend (item.path)
			else
				if left_child.item.min_1 <= a_key_1 and then left_child.item.max_1 >= a_key_1 then
					Result.append_array (left_child.key_1_paths (a_key_1))
				end
				if right_child.item.min_1 <= a_key_1 and then right_child.item.max_1 >= a_key_1 then
					Result.append_array (right_child.key_1_paths (a_key_1))
				end
			end
		end

	key_0_leaves (a_key_0: G): ROSE_LINEAR_ARRAY [PARTITION_TREE_RECORD [G, H]] is
		do
			create Result.make_empty
			if left_child = Void then
				Result.extend (item)
			else
				if left_child.item.min_0 <= a_key_0 and then left_child.item.max_0 >= a_key_0 then
					Result.append_array (left_child.key_0_leaves (a_key_0))
				end
				if right_child.item.min_0 <= a_key_0 and then right_child.item.max_0 >= a_key_0 then
					Result.append_array (right_child.key_0_leaves (a_key_0))
				end
			end
		end

	key_1_leaves (a_key_1: H): ROSE_LINEAR_ARRAY [PARTITION_TREE_RECORD [G, H]] is
		do
			create Result.make_empty
			if left_child = Void then
				Result.extend (item)
			else
				if left_child.item.min_1 <= a_key_1 and then left_child.item.max_1 >= a_key_1 then
					Result.append_array (left_child.key_1_leaves (a_key_1))
				end
				if right_child.item.min_1 <= a_key_1 and then right_child.item.max_1 >= a_key_1 then
					Result.append_array (right_child.key_1_leaves (a_key_1))
				end
			end
		end

	key_0_leaves_count (a_key_0: G): INTEGER is
		do
			if left_child = Void then
				Result := Result + 1
			else
				if left_child.item.min_0 <= a_key_0 and then left_child.item.max_0 >= a_key_0 then
					Result := Result + left_child.key_0_leaves_count (a_key_0)
				end
				if right_child.item.min_0 <= a_key_0 and then right_child.item.max_0 >= a_key_0 then
					Result := Result + right_child.key_0_leaves_count (a_key_0)
				end
			end
		end

	key_1_leaves_count (a_key_1: H): INTEGER is
		do
			if left_child = Void then
				Result := Result + 1
			else
				if left_child.item.min_1 <= a_key_1 and then left_child.item.max_1 >= a_key_1 then
					Result := Result + left_child.key_1_leaves_count (a_key_1)
				end
				if right_child.item.min_1 <= a_key_1 and then right_child.item.max_1 >= a_key_1 then
					Result := Result + right_child.key_1_leaves_count (a_key_1)
				end
			end
		end

feature -- Element modification

	set_left_child (a_tree: PARTITION_TREE [G, H]) is
		do
			left_child := a_tree
		end

	set_right_child (a_tree: PARTITION_TREE [G, H]) is
		do
			right_child := a_tree
		end

	set_item (a_item: PARTITION_TREE_RECORD [G, H]) is
		require
			valid_item: a_item /= Void
		do
			item := a_item
		end

feature {PARTITION_TREE} -- Implementation

	leaf_record_internal (a_key_0: G; a_key_1: H): PARTITION_TREE_RECORD [G, H] is
		local
			l_current_tree: PARTITION_TREE [G, H]
			l_child: PARTITION_TREE [G, H]
			l_child_item: PARTITION_TREE_RECORD [G, H]
		do
			l_current_tree := Current
			from
			until
				Result /= Void
			loop
				l_child := l_current_tree.left_child
				if l_child = Void then
					Result := l_current_tree.item
				else
					l_child_item := l_child.item
					if l_child_item.min_0 <= a_key_0 and then
						l_child_item.max_0 >= a_key_0 and then
						l_child_item.min_1 <= a_key_1 and then
						l_child_item.max_1 >= a_key_1 then
						l_current_tree := l_child
					else
						l_current_tree := l_current_tree.right_child
					end
				end
			end
		end

	last_leaf_record: PARTITION_TREE_RECORD [G, H]
	
invariant

	available_item: item /= Void
	
end
