note

	description:
	"[
		Represents the cell and generating path of a binary partition of a two dimensional array,
		where each split is either a horizontal or vertical partitioning.  Note that this is more
		flexible than a quad-tree representation
	]"
	author: "Mark Howard"
	last_editor: "$Author: mb $"
	date: "$Date: 2011/06/20 $"
	revision: "$Revision: #7 $"
	path: "$File: //rose/source/kernel/infrastructure/structures/axis/partition_tree_record.e $"

class PARTITION_TREE_RECORD [G -> COMPARABLE, H -> COMPARABLE]

feature -- Access

	path: INTEGER

	min_0: G

	max_0: G

	min_1: H

	max_1: H
	
	record_count: INTEGER

feature -- Element modification
	
	set_from_other (a_value: PARTITION_TREE_RECORD [G, H]) is
			-- Set Current from `a_value'
		require
			other_valid: a_value /= Void
		do
			set_path (a_value.path)
			set_min_0 (a_value.min_0)
			set_max_0 (a_value.max_0)
			set_min_1 (a_value.min_1)
			set_max_1 (a_value.max_1)
			set_record_count (a_value.record_count)
		end
		
	set_path (a_value: like path) is
		do
			path := a_value
		end

	set_min_0 (a_value: like min_0) is
		do
			min_0 := a_value
		end

	set_max_0 (a_value: like max_0) is
		do
			max_0 := a_value
		end

	set_min_1 (a_value: like min_1) is
		do
			min_1 := a_value
		end

	set_max_1 (a_value: like max_1) is
		do
			max_1 := a_value
		end

	set_record_count (a_value: like record_count) is
		do
			record_count := a_value
		end
	
feature -- Query
	
	contains (a_key_0: G; a_key_1: H): BOOLEAN is
			-- Does this record contain `a_key_0' and `a_key_1'?
		do
			Result := a_key_0 >= min_0 and a_key_0 <= max_0 and a_key_1 >= min_1 and a_key_1 <= max_1
		end

end
