indexing
	description: "Chart showing performance monitor graph"
	author: "Mauricio Bustos"
	last_editor: "$Author: rj $"
	date: "$Date: 2014/05/16 $"
	revision: "$Revision: #21 $"
	path: "$File: //rose/source/portable/widgets/performance_monitor_graph_widget.e $"

class

	PERFORMANCE_MONITOR_GRAPH_WIDGET

inherit

	ROSE_CHART_WIDGET [ROSE_CHART_2D_POINT, ROSE_CHART_2D_POINT]
	redefine
		clear_all
	end
	DATABASE_SERVER
		undefine
			default_create,
			copy,
			is_equal
		end
	EXCEPTIONS
		export
			{NONE} all
		undefine
			default_create,
			copy,
			is_equal
		end

create

	make

feature -- Interface.

	graph_performance_objects_and_events(
			a_machines: ARRAY[STRING];
			a_events: ROSE_LINEAR_ARRAY[SERVER_REQUEST];
			a_length_code: INTEGER)
		local
			l_events: ROSE_LINEAR_ARRAY[SERVER_REQUEST]
			l_event: SERVER_REQUEST
			l_machines: ROSE_LINEAR_ARRAY[STRING]
			l_main_x_axis, l_secondary_x_axis: ROSE_CHART_AXIS_INDEXED
			l_start_time_delta: DOUBLE
			l_start_time: ROSE_DATE
			l_date: ROSE_DATE
		do
			inspect a_length_code
			when 1 then				-- 3 hours.
				l_start_time_delta := 3.0 / 24.0
			when 3 then				-- 7 days.
				l_start_time_delta := 7
			when 4 then				-- 30 days.
				l_start_time_delta := 30
			else					-- 24 hours.
				l_start_time_delta := 1
			end

			l_start_time := orinda_time_now
			l_start_time.day_add(-(l_start_time_delta.truncated_to_integer))
			l_start_time.hour_add(-((l_start_time_delta - l_start_time_delta.truncated_to_integer) * 24).truncated_to_integer)

			create l_machines.make_from_array(a_machines)
			l_machines.compare_objects
			create main_system_dates.make_empty

			graph_performance_objects(a_machines, l_start_time)

			if a_events /= Void and not a_events.is_empty then
				create l_events.make_empty
				from a_events.start until a_events.off loop
					l_event := clone(a_events.current_item)
					if l_event.machine.has('.') then
						l_event.set_machine(l_event.machine.substring(1,l_event.machine.index_of('.',1) - 1))
					end
					if l_machines.has(l_event.machine.as_lower) then
						l_events.extend(l_event)
					end
					a_events.forth
				end
				if not l_events.is_empty then
--					l_machines.compare_references
					graph_events(l_events, l_start_time)
				end
			end

			if l_machines.count/=0
			then
				main_system_dates.sort_by(agent date_less_than)

				from
					main_system_dates.start
				until
					main_system_dates.off
				loop
					if equal(main_system_dates.current_item, l_date)
					then
						main_system_dates.remove
					else
						l_date := main_system_dates.current_item
						main_system_dates.forth
					end
				end

				main_system.x_axis.set_dates(main_system_dates)
				secondary_system.x_axis.set_dates(main_system_dates)
				main_system.x_axis.sync_to_axis(secondary_system.x_axis)

--				main_system.x_axis.set_minimum_visible(main_system_dates.item(1).to_double)
--				main_system.x_axis.set_maximum_visible(main_system_dates.item(main_system_dates.count).to_double)
--				secondary_system.x_axis.set_minimum_visible(main_system_dates.item(1).to_double)
--				secondary_system.x_axis.set_maximum_visible(main_system_dates.item(main_system_dates.count).to_double)

				main_system.y_axis.set_minimum_visible(0)
				main_system.y_axis.set_maximum_visible(100)
				secondary_system.y_axis.set_minimum_visible(0)
			else
				clear_all
			end

			l_main_x_axis ?= main_system.x_axis
			l_secondary_x_axis ?= secondary_system.x_axis
			inspect a_length_code
			when 1 then				-- 3 hours.
				l_main_x_axis.set_hourly
				l_main_x_axis.set_tick_time_interval(1)
				l_secondary_x_axis.set_hourly
				l_secondary_x_axis.set_tick_time_interval(1)
			when 3 then				-- 7 days.
				l_main_x_axis.set_daily
				l_main_x_axis.set_tick_time_interval(1)
				l_secondary_x_axis.set_daily
				l_secondary_x_axis.set_tick_time_interval(1)
			when 4 then				-- 30 days.
				l_main_x_axis.set_daily
				l_main_x_axis.set_tick_time_interval(2)
				l_secondary_x_axis.set_daily
				l_secondary_x_axis.set_tick_time_interval(2)
			else					-- 24 hours.
				l_main_x_axis.set_hourly
				l_main_x_axis.set_tick_time_interval(2)
				l_secondary_x_axis.set_hourly
				l_secondary_x_axis.set_tick_time_interval(2)
			end

			refresh
		end

	set_memory_red_alert_level(a_level: INTEGER) is
	do
		memory_red_alert_level := a_level
	end

	set_memory_yellow_alert_level(a_level: INTEGER) is
	do
		memory_yellow_alert_level := a_level
	end

	set_processor_red_alert_level(a_level: INTEGER) is
	do
		processor_red_alert_level := a_level
	end


	set_processor_yellow_alert_level(a_level: INTEGER) is
	do
		processor_yellow_alert_level := a_level
	end

feature -- Events

	graph_events(a_events : ROSE_LINEAR_ARRAY[SERVER_REQUEST]; a_start_time: ROSE_DATE) is
		local
			l_current : SERVER_REQUEST
			i : INTEGER
		do
			a_events.sort_by(agent less_than_event)
			from
				event_count := 0
				a_events.start
				l_current := a_events.first
			until
				a_events.off
			loop
				if not a_events.current_item.machine.as_lower.is_equal(l_current.machine.as_lower) or
					not a_events.current_item.server_executable.as_lower.is_equal(l_current.server_executable.as_lower) or
					a_events.current_item = a_events.last
				then
					i := a_events.index
					if a_events.current_item = a_events.last then
						create_event_graph(a_events.subarray(a_events.index_of(l_current,1), i), a_start_time)
					else
						create_event_graph(a_events.subarray(a_events.index_of(l_current,1), i - 1), a_start_time)
					end

					a_events.go_i_th(i)
					l_current := a_events.current_item
				end

				if not a_events.off then
					a_events.forth
				end
			end
		end

	create_event_graph(a_events : ROSE_LINEAR_ARRAY[SERVER_REQUEST]; a_start_time: ROSE_DATE) is
		local
			l_plot: ROSE_CHART_2D_PLOT
			l_point: ROSE_CHART_2D_POINT
			l_first_time: BOOLEAN
		do
			create l_plot.make_empty (a_events.first.machine + " - " + a_events.first.server_executable)

			from
				l_first_time := True
				a_events.start
			until
				a_events.off
			loop
				if a_events.current_item.timestamp>a_start_time
				then
					if l_first_time
					then
						event_count := event_count + 1
						l_first_time := False
					end

					create l_point.make ([a_events.current_item.timestamp.to_double, event_count * 5.0], Void)
					l_point.set_x_label (formatted_date (a_events.current_item.timestamp))
					l_plot.extend_back (l_point)
					main_system_dates.extend(a_events.current_item.timestamp)
				end

				a_events.forth
			end

			if l_plot.count>0
			then
				main_system_plot_list.add_plot (l_plot)
			end
		end

	less_than_event(a, b : SERVER_REQUEST) : BOOLEAN is
		do
			if a.machine.as_lower < b.machine.as_lower or
				(a.machine.as_lower.is_equal(b.machine.as_lower) and
				 a.server_executable.as_lower < b.server_executable.as_lower) or
				(a.machine.as_lower.is_equal(b.machine.as_lower) and
				 a.server_executable.as_lower.is_equal(b.server_executable.as_lower) and
				 a.timestamp < b.timestamp) then
				Result := True
			end
		end

feature -- Performance Objects

	graph_performance_objects(a_machines : ARRAY[STRING]; a_start_time: ROSE_DATE) is
		local
			l_predicate : COLLECTING_PREDICATE[COUNTER_SUMMARY_VIEW]
			l_server : SQL_DATA_SERVER
			l_index : INTEGER
			l_object_tables : HASH_TABLE[HASH_TABLE[ROSE_LINEAR_ARRAY[DOUBLE],STRING],STRING]
			l_date : ROSE_DATE
			l_last_date_string : STRING
			l_servers : STRING
			l_point: ROSE_CHART_2D_POINT
			l_plot: ROSE_CHART_2D_PLOT
			l_plot_name: STRING
			l_new_plot_name: STRING
			l_old_recordindex: INTEGER
			l_counter_summary: COUNTER_SUMMARY_VIEW
			l_top: STRING
		do
			create l_object_tables.make(10)
			l_server := servers.make_from_repository("perf_counters")
			l_servers := "'test'"
			l_servers := ""
			from l_index := 1 until l_index > a_machines.count loop
				if l_servers.count/=0
				then
					l_servers.append(",")
				end

				l_servers.append("'\\" + a_machines.item(l_index) + "'")
				l_index := l_index + 1
			end
            l_index := 0

			if a_machines.count=0
			then
				l_top := "top 0"
				l_servers := "'abc'"
			else
				l_top := ""
			end

            -- This predicate was slow.
--			create l_predicate.make_strict(
--				l_server,
--				"select * from counter_summary2_view %
--				%where %
--				%machinename in (" + l_servers + ") and %
--				%counterdatetime>? %
--				%order by machinename, objectname, countername, counterdatetime")
--			l_predicate.retrieve(Void, <<a_start_time.formatted_out("YYYY-[0]MM-[0]DD [0]HH:[0]MI")>>)

			-- This predicate was faster.
			create l_predicate.make_strict(
				l_server,
				"SELECT " + l_top + "%N%
				% b.MachineName, b.ObjectName, b.CounterName, a.CounterDateTime,%N%
				% a.CounterValue, a.RecordIndex, a.FirstValueA, a.FirstValueB,%N%
				% a.SecondValueA, a.SecondValueB%N%
				%FROM%N%
				% CounterData a INNER JOIN CounterDetails b%N%
				%ON%N%
				% a.CounterID = b.CounterID and%N%
				% b.machinename in (" + l_servers + ") and%N%
				% a.counterdatetime>'" + a_start_time.formatted_out("YYYY-[0]MM-[0]DD [0]HH:[0]MI") + "'%N%
				%order by b.machinename, b.objectname desc, b.countername, a.counterdatetime")
			l_predicate.retrieve(Void, Void)
			clear_all
			main_system.set_renderer (line_renderer)
			secondary_system.set_renderer (line_renderer)
			create main_system_dates.make_empty

			from
				l_predicate.array.start
			until
				l_predicate.array.off
			loop
				l_new_plot_name :=
					l_predicate.array.current_item.machinename + "\" +
					l_predicate.array.current_item.objectname + "\" +
					l_predicate.array.current_item.countername
				if not equal(l_plot_name, l_new_plot_name)
				then
					l_old_recordindex := -1
					if l_plot/=Void
					then
						if l_plot_name.substring_index("%% Processor", 1)/=0
						then
							main_system_plot_list.add_plot (l_plot)
						else
							secondary_system_plot_list.add_plot (l_plot)
						end
					end

					l_plot_name := l_new_plot_name
					create l_plot.make_empty (l_plot_name)
				end

				l_predicate.array.current_item.counterdatetime.prune_all('%U')
				if not equal(l_last_date_string, l_predicate.array.current_item.counterdatetime)
				then
					l_last_date_string := l_predicate.array.current_item.counterdatetime
					create l_date.make_from_string(l_last_date_string,
												   "yyyy-[0]mm-[0]dd [0]hh:[0]mi:[0]ss.ff3")
					main_system_dates.extend(l_date)
				end

				l_counter_summary := l_predicate.array.current_item
				if l_counter_summary.recordindex<l_old_recordindex or
				   l_counter_summary.recordindex=1 or
				   (l_counter_summary.CounterValue=0 and l_counter_summary.FirstValueA=0 and
				    l_counter_summary.FirstValueB=0 and l_counter_summary.SecondValueA=0 and
				    l_counter_summary.SecondValueB=0)
				then
					-- This first point always has bogus cpu usage.  In any case, the qnan
					-- will show up nicely as a hole in the data.  The recordindex is 1 whever
					-- the record stream is started, thus a hole is appropriate.  If one of the servers
					-- starts late, it's first recordindex will be a low number (but not 1).
					-- Thus we now have l_old_recordindex.  In addition, the data sometime has
					-- all zeroes for data so we skip those too.
					create l_point.make([l_date.to_double, Qnan], Void)
					l_point.set_x_label (formatted_date (l_date))
				else
					create l_point.make([l_date.to_double, l_predicate.array.current_item.countervalue], Void)
					l_point.set_x_label (formatted_date (l_date))
					if l_plot_name.substring_index("%% Processor", 1)/=0
					then
						if l_predicate.array.current_item.countervalue>=processor_yellow_alert_level
						then
							if l_predicate.array.current_item.countervalue>=processor_red_alert_level
							then
								l_point.set_color(red_alert_color)
							else
								l_point.set_color(yellow_alert_color)
							end
						end
					else
						if l_predicate.array.current_item.countervalue<=memory_yellow_alert_level
						then
							if l_predicate.array.current_item.countervalue<=memory_red_alert_level
							then
								l_point.set_color(red_alert_color)
							else
								l_point.set_color(yellow_alert_color)
							end
						end
					end
				end

				l_old_recordindex := l_predicate.array.current_item.recordindex
				l_plot.extend_back (l_point)
				l_predicate.array.forth
			end

			if l_plot_name/=Void
			then
				if l_plot_name.substring_index("%% Processor", 1)/=0
				then
					main_system_plot_list.add_plot (l_plot)
				else
					secondary_system_plot_list.add_plot (l_plot)
				end
			end
		end

feature {NONE} -- Once/Constants.

	yellow_alert_color: EV_COLOR is
	once
		create Result.make_with_8_bit_rgb(255, 255, 0)
	end

	red_alert_color: EV_COLOR is
	once
		create Result.make_with_8_bit_rgb(255, 0, 0)
	end

	memory_red_alert_level: INTEGER
	memory_yellow_alert_level: INTEGER
	processor_red_alert_level: INTEGER
	processor_yellow_alert_level: INTEGER

	main_system_dates : ROSE_LINEAR_ARRAY[ROSE_DATE]
	main_system_names : ROSE_LINEAR_ARRAY[STRING]
	secondary_system_data : ROSE_LINEAR_ARRAY[ARRAY[DOUBLE]]
	secondary_system_names : ROSE_LINEAR_ARRAY[STRING]

	event_count: INTEGER

	date_less_than(a, b: ROSE_DATE): BOOLEAN is
	do
		Result := a<b
	end

	x_axis : ROSE_CHART_AXIS_INDEXED is
		once
			create Result.make
		end

	line_renderer : ROSE_CHART_RENDERER_LINE is
		once
			create Result.make(True,True)
		end

	ieee_math : IEEE_MATH is
		once
			create Result
		end

	time_zone_converter: TIME_ZONE_CONVERTER
	-- Used to convert the user's local time into Orinda time
	once
		create Result
	end

feature {NONE} -- Implementation

	main_system_plot_list: ROSE_CHART_2D_PLOT_LIST is
			-- plot list for the main system
		require
			renderer_set: main_system.renderer /= Void
		do
			if main_system_plot_list_impl = Void then
				create main_system_plot_list_impl.make_empty
				main_system.set_plot_list (main_system_plot_list_impl)
			end
			Result := main_system_plot_list_impl
		end

	secondary_system_plot_list: ROSE_CHART_2D_PLOT_LIST is
			-- plot list for the secondary system
		require
			renderer_set: secondary_system.renderer /= Void
		do
			if secondary_system_plot_list_impl = Void then
				create secondary_system_plot_list_impl.make_empty
				secondary_system.set_plot_list (secondary_system_plot_list_impl)
			end
			Result := secondary_system_plot_list_impl
		end

	clear_all is
			-- clear the widgets
		do
			Precursor {ROSE_CHART_WIDGET}
			if main_system.renderer /= Void then
				main_system_plot_list.wipe_out
			end
			if secondary_system.renderer /= Void then
				secondary_system_plot_list.wipe_out
			end
		end


	main_system_plot_list_impl: ROSE_CHART_2D_PLOT_LIST
	secondary_system_plot_list_impl: ROSE_CHART_2D_PLOT_LIST

	formatted_date (a_date: ROSE_DATE): STRING is
			-- String displaying formatted version of 'a_date'
		require
			a_date /= Void
		do
			Result := a_date.formatted_out ("yyyy/[0]mm/[0]dd [0]hh:[0]mi:[0]ss")
		end

	orinda_time_now: ROSE_DATE
	-- This is like ROSE_DATE.make_now but returns the time in the Orinda timezone even if you are not in Orinda.
	do
		-- Get the current time in UT.
		create Result.make_now_utc

		-- Convert to Pacific time.  In spite of the word "Standard", this also works during DST.
		Result.add(time_zone_converter.time_difference_with_utc("Pacific Standard Time"))
	end

end


