note

	description:

		"Objects that represent a COMPANY entity"

	author: "Mark Howard"
	last_editor: "$Author: mk $"
	date: "$Date: 2012/06/22 $"
	revision: "$Revision: #35 $"
	path: "$File: //rose/source/kernel/entities/persistence/persistent_company.e $"

class
	PERSISTENT_COMPANY

inherit
	ISSUER
		rename
			issuer_name_key as company_name_key,
			issuer_name_most_recent_key as company_name_most_recent_key
		end
	PERSISTENT_COMPANY_ACCESS
		rename
			something_related_by_company_to_investment_region as investment_region
		redefine
			name_identifier_key,
			peer_anchor,
			expression_tag,
			descriptive_name,
			name_identifier_keys
		end
	HAS_LISTINGS

create
	make,make_anonymous

feature -- Query

	security (i: INTEGER): like context.security_template is
			--i_th security
		do
			if securities /= void and then securities.valid_index (i) then
				Result := securities.item (i).entity
			end
		end

	security_count: INTEGER is
			-- Number of securities of `Current'
		do
			if securities /= void then
				Result := securities.count
			end
		end

	securities: ROSE_ENTITY_HISTORY[like context.security_template] is
			-- Get all securities for `Current'
		do
			Result := company_to_security_history
		end

	listings : ROSE_ENTITY_HISTORY[like context.listing_template] is
			-- Get all listings for `Current'		
		local
			l_securities : ROSE_ENTITY_HISTORY[like context.security_template]
		do
			l_securities := securities
			create Result.make_empty
			from l_securities.start until l_securities.off loop
				Result.append_array (l_securities.current_item.entity.listings)
				l_securities.forth
			end
		end

	country (a_date : ROSE_DATE) : like context.country_template is
			-- what is presumably being displayed as "headquarter_country" in rose_platform
		do
			Result := company_to_headquarter_country_history.witness(a_date)
			if Result = Void then
				Result := company_to_headquarter_country_history.witness(start_date)
			end
			if Result = Void then
				Result := company_to_headquarter_country_history.witness(today)
			end
		end

	risk_country (a_date: ROSE_DATE): like context.country_template
		-- feature is modeled after 'country' feature above. This is presumably display as "risk_country" in rose_platform
		do
			Result := company_to_risk_country_history.witness(a_date)
			if Result = Void then
				Result := company_to_risk_country_history.witness(start_date)
			end
			if Result = Void then
				Result := company_to_risk_country_history.witness(today)
			end
		end

--	valuation_report (a_report : VALUATION_REPORT[VALUATION_RECORD [PERSISTENT_COMPANY]];a_statements : ROSE_LINEAR_ARRAY[ROSE_FISCAL_REPORT_INSTANCE[VALUATION_RECORD[PERSISTENT_COMPANY]]];a_date : ROSE_DATE;a_company : COMPANY;a_periods_per_year : INTEGER) : like valuation_report_anchor is
--		do
--			create Result.make(a_report,a_statements,a_date,a_company,a_periods_per_year)
--		end

feature -- Descriptive_name features

	descriptive_name : STRING is
			-- String that describes this entity
		Local
			l_identifier : ENTITY_IDENTIFIER
		do
			if zone_code_key (today) /= Void then
				l_identifier := context.identifier (context.attribute_ids.display_name_type_id)
				if l_identifier /= Void then
					Result := l_identifier.dated_key (Current ,today)
				end
			end
			if Result = Void then
				Result := precursor
			end
		end

	name_identifier_key : STRING is
			-- Attribute that is used to describe this entity
		do
			Result := "company name"
		end

	name_identifier_keys: LIST[STRING] is
			-- Atrributes that are needed to compute the descriptive name
			-- Has to be redefined in descendants that use more than `name_identifier_key'
		local
			l_key: STRING
		once
			create {ARRAYED_LIST[STRING]} Result.make (3)
			l_key := name_identifier_key
			if l_key /= Void then
				Result.extend (l_key)
			end
			l_key := context.attribute_name_by_id (context.attribute_ids.zone_code_type_id)
			if l_key /= Void then
				Result.extend (l_key)
			end
			l_key := context.attribute_name_by_id (context.attribute_ids.display_name_type_id)
			if l_key /= Void then
				Result.extend (l_key)
			end
			Result.compare_objects
		end

	expression_tag: STRING is "company"
			-- Expression tag string


feature {NONE} -- Implementation

--	valuation_report_anchor : VALUATION_REPORT[VALUATION_RECORD [PERSISTENT_COMPANY]] is do end
--	valuation_record_anchor : VALUATION_RECORD [PERSISTENT_COMPANY] is do end

	peer_anchor: PERSISTENT_COMPANY
			-- Type returned by 'peer' function
		do
		end

end
