-- You MAY modify this class.
-- This class was automatically generated by ENTITY_WRAPPER on 03/15/2004 3:54 PM.

note

	description:

		"Persistent definition of a currency"

	author: "Mark Howard"
	last_editor: "$Author: rj $"
	date: "$Date: 2014/12/12 $"
	revision: "$Revision: #26 $"
	path: "$File: //rose/source/kernel/entities/persistence/persistent_currency.e $"

class
	PERSISTENT_CURRENCY
inherit
	PERSISTENT_CURRENCY_ACCESS
		redefine
			make,is_equal
		end
	ROSE_LEGACY_UNDEFINED
		undefine
			is_equal
		end
create
	make,make_anonymous
feature -- Initialization

	make (a_context : like context;a_category_id : INTEGER;a_handle : INTEGER;a_start_date,a_end_date : ROSE_DATE) is
			--creation
		do
			Precursor(a_context,a_category_id,a_handle,a_start_date,a_end_date)
			iso := iso_code_key(a_end_date)
		end

feature --Access
	is_equal(a_other:like Current): BOOLEAN is
			-- Is 'Current' equal to 'a_other'?
			-- This code is called a lot and has been designed for speed, not readability.
			-- Note that 'context' (called by 'supplier') is expensive.
		do
			if a_other = Current
			then
				Result := True
			else
				if handle = a_other.handle
				then
					if context.has_rose_handles and then a_other.context.has_rose_handles
					then
						Result := True		-- The 'handle' matches and they both use rose handles.
					elseif supplier_id = a_other.supplier_id
					then
						Result := True		-- Both 'handle' and 'supplier_id' match.
					end
				end
			end
		end

	same_currency(a_other:like Current): BOOLEAN is
			-- This is an exact copy of the text of 'is_equal'.  However, it does not inherit
			-- the post-condition 'symmetric' (from ANY.is_equal) which requires that 'a_other'
			-- be the same type as 'Current'.
		do
			if a_other = Current
			then
				Result := True
			else
				if handle = a_other.handle
				then
					if context.has_rose_handles and then a_other.context.has_rose_handles
					then
						Result := True		-- The 'handle' matches and they both use rose handles.
					elseif supplier_id = a_other.supplier_id
					then
						Result := True		-- Both 'handle' and 'supplier_id' match.
					end
				end
			end
		end

	iso : STRING
		-- ISO code for the current currency (eg USD)

	spot_rate_at(a_date:ROSE_DATE):DOUBLE is
			-- spot rate at
		local
			l_target:XRATE_DATA_SIGNATURE
			l_date:ROSE_DATE
			l_expression:INDEX_XRATE_EXPRESSION
		do
			Result := undefined_double
			create l_expression
			create l_target
			l_date:=a_date.current_or_previous_weekday
			l_target.rose_entity_date_make(Current,l_date)
			result:=l_expression.generated_unitized_value(l_target, void).value
		end

	exchange_rate_at (a_currency : like context.currency_template;a_date : ROSE_DATE) : DOUBLE is
			-- Exchange rate against `a_currency' at `a_date'
		local
			l_exchange_rate,l_other_rate : DOUBLE
		do
			if a_currency /= Void and a_currency.uid = uid then
				Result := 1
			else
				Result := undefined_double
				if a_currency /= Void then
					l_exchange_rate := spot_rate_at(a_date)
					if is_defined_double(l_exchange_rate) then
						l_other_rate := a_currency.spot_rate_at(a_date)
						if is_defined_double(l_other_rate) then
							Result := l_exchange_rate / l_other_rate
						end
					end
				end
			end
		end

	exchange_rate (a_currency: PERSISTENT_CURRENCY; a_date: ROSE_DATE): DOUBLE is
			-- Factor to convert from `a_currency' to `Current' at `a_date' in `default_market'
		do
			Result := market_exchange_rate (Void, a_currency, a_date)
		end

	exchange_rates_at(a_currency : like context.currency_template; a_dates : ARRAY[ROSE_DATE]) : VECTOR is
			----be carefule, this uses different denomination
		local
			i : INTEGER
			l_double : DOUBLE
			l_date : ROSE_DATE
		do
			if a_currency /= Void and then (a_dates /= Void and then a_dates.count > 0) then
				create Result.make(a_dates.count)
				from i := 1 until i > a_dates.count
				loop
					l_date := a_dates.item(i)
					if l_date /= Void then
						l_double := a_currency.exchange_rate(current, l_date)
						Result.put(l_double, i)
					end
					i := i + 1
				end
			end
		end

	exchange_return(a_currency:like context.currency_template; a_start,a_end:ROSE_DATE):DOUBLE is
			--- rate of return measured by other currency
   		do
			Result := market_exchange_return(Void, a_currency, a_start, a_end)
   		end

	market_exchange_return(a_market : MARKET; a_currency:like context.currency_template; a_start,a_end:ROSE_DATE):DOUBLE is
			--- rate of return measured by other currency
   		local
   			l_start_rate,l_end_rate:DOUBLE
   		do
			if iso.is_equal (a_currency.iso) then
				result:=1.0
			else
   				l_start_rate:=market_exchange_rate(a_market, a_currency,a_start)
   				l_end_rate:=market_exchange_rate(a_market, a_currency,a_end)
   				if is_defined_double(l_start_rate) and is_defined_double(l_end_rate) then
   					result:=l_start_rate/l_end_rate
   				else
					result:=undefined_double
   				end
			end
   		end

	exchange_returns_at(a_currency : like context.currency_template; a_dates : ARRAY[ROSE_DATE]) : VECTOR is
			--vector of returns at dates
		local
			i : INTEGER
			l_double : DOUBLE
			l_date,l_previous_date : ROSE_DATE
		do
			if a_currency /= Void and then (a_dates /= Void and then a_dates.count > 1) then
				create Result.make(a_dates.count-1)
				from i := 2 until i > a_dates.count
				loop
					l_date := a_dates.item(i)
					if l_date /= Void then
						l_previous_date:=a_dates.item(i-1)
						l_double := exchange_return(a_currency, l_previous_date,l_date)
						Result.put(l_double, i-1)
					end
					i := i + 1
				end
			end
		end

	market_exchange_rate (a_market: MARKET; a_currency: PERSISTENT_CURRENCY; a_date: ROSE_DATE): DOUBLE is
			-- Factor to convert from `a_currency' to `Current' at `a_date' in `a_market'
		local
			l_market: MARKET
		do
			l_market := a_market
			if l_market = Void then
				l_market := default_market
			end
			if a_currency /= Void and a_date /= Void then
				Result := l_market.exchange_rate (Current, a_currency, a_date)
			else
				Result := undefined_double
			end
		end

	minor_currency_unit (a_date: ROSE_DATE): INTEGER is
			-- the minor currency unit
		local
			s: STRING
		do
			s := minor_unit_key (a_date)
			if s = void or not s.is_integer then
				Result := 0
			elseif s /= void then
				Result := s.to_integer
			end
		end

	minor_currency_unit_precision (a_date: ROSE_DATE): DOUBLE is
			-- the precision of the minor currency unit
		local
			l_minor_unit: INTEGER
		do
			l_minor_unit := minor_currency_unit (a_date)
			Result := 10.0 ^ (- l_minor_unit)
		end

	minor_currency_unit_tolerance (a_date: ROSE_DATE): DOUBLE is
			-- the precision of the minor currency unit
		local
			l_minor_unit: INTEGER
		do
			l_minor_unit := minor_currency_unit (a_date)
			Result := minor_currency_unit_precision (a_date) - (10.0 ^ (- (l_minor_unit + 1)))
		end

feature {NONE} --Implementation

	default_market : MARKET
		-- Default market

end
