indexing
	description: ""
	author: "Mark Howard"
	date: "$Date: 2008/12/01 $"
	revision: "$Revision: #9 $"
	archive: "$File: //rose/source/kernel/entities/persistence/persistent_exchange.e $"

class
	PERSISTENT_EXCHANGE

inherit
	PERSISTENT_EXCHANGE_ACCESS
		rename
			something_corelated_by_country_to_exchange as country,
			something_related_by_exchange_to_composite_exchange as composite_exchange,
			something_related_by_exchange_to_time_zone as time_zone
		redefine
			name_identifier_key
		end

create
	make, make_anonymous

feature -- Access

	name_identifier_key : STRING is
		do
			Result := "exchange name"
		end

	utc_seconds_offset (a_date: ROSE_DATE): INTEGER is
			-- Time offset in seconds from UTC at 'a_date'
		local
			tz: PERSISTENT_TIME_ZONE
			l_country : PERSISTENT_COUNTRY
			l_region : PERSISTENT_INVESTMENT_REGION
			s: STRING
			l_zone: CHARACTER
		do
			tz := time_zone
			if tz /= Void then
				Result := tz.utc_seconds_offset (a_date)
			else
				-- If no TZ in ref, we try to guess...
				l_country := country
				if l_country /= Void then
					s := l_country.iso_code_key (a_date)
					if hardcoded_country_time_zones.has (s) then
						Result := hardcoded_country_time_zones.item (s)
					else
						l_region := l_country.something_related_by_country_to_investment_region
						if l_region /= Void then
							s := l_region.mnemonic_key (a_date)
							if s /= Void and then s.count > 0 then
								l_zone := s.item (1).as_upper
							end
							if l_zone = 'U' then
								Result := -4
							elseif l_zone = 'E' then
								Result := 0
							elseif l_zone = 'J' then
								Result := 9
							elseif l_zone = 'C' then
								Result := -4
							end
						end
					end
				end
				Result := Result * 60 * 60
			end
		end

	is_market_now_open: BOOLEAN is
			-- Is this exchange market open at 'a_date'?
		local
			l_now: ROSE_DATE
			t, t1, t2: ROSE_TIME
		do
			t1 := opening_time (Void)
			t2 := closing_time (Void)
			if t1 /= Void and then t2 /= Void then
				l_now := localized_date (Void)
				t := l_now.time
				Result := t >= t1 and then t <= t2
			end
		end

	opening_time (a_date: ROSE_DATE): ROSE_TIME is
			-- Opening time at 'a_date'
		do
			Result := time_from_string (market_open_time_key (a_date))
			if Result = Void then
				create Result.make (9, 0, 0)
			end
		end

	closing_time (a_date: ROSE_DATE): ROSE_TIME is
			-- Closing time at 'a_date'
		do
			Result := time_from_string (market_close_time_key (a_date))
			if Result = Void then
				create Result.make (17, 0, 0)
			end
		end

	localized_date (a_utc: ROSE_DATE): ROSE_DATE is
			-- Localized UTC date 'a_utc', according to 'exchange'
		do
			if a_utc = Void then
				create Result.make_now_utc
			else
				Result := a_utc.twin
			end
			Result.second_add (utc_seconds_offset (Result))
		ensure
			Result /= Void
		end

feature {NONE} -- Implementation

	time_from_string (a_string: STRING): ROSE_TIME is
		do
		end

	hardcoded_country_time_zones: HASH_TABLE [INTEGER, STRING] is
			-- Temporary, will be removed as soon as all data is in ref
			-- Time zone offsets to UTC for major countries main exchange...
		once
			create Result.make (0)
			Result.put (-4, "US")
			Result.put (1, "GB")
			Result.put (2, "FR")
			Result.put (8, "HK")
			Result.put (9, "JP")
			Result.put (10, "AU")
		end

end
