note

	description:

		"Persistent future entity."

	author: "Mark Howard"
	last_editor: "$Author: m5 $"
	date: "$Date: 2013/04/18 $"
	revision: "$Revision: #14 $"
	path: "$File: //rose/source/kernel/entities/persistence/persistent_future.e $"

class PERSISTENT_FUTURE

inherit

	PERSISTENT_FUTURE_ACCESS
		rename
			future_to_future_listing_history as listings
		redefine
			peer_anchor
		end

	MISSING_PARAMETERS
		undefine
			is_equal
		end

create

	make,
	make_anonymous

feature -- Attributes

	cftc_approved_key (a_date: ROSE_DATE): STRING is
		do
			if details /= Void then
				Result := details.cftc_approved_key (a_date)
			end
		end

	cftc_approved_most_recent_key (a_date: ROSE_DATE): STRING is
		do
			if details /= Void then
				Result := details.cftc_approved_most_recent_key (a_date)
			end
		end

	contract_end_date_code_key (a_date: ROSE_DATE): STRING is
		do
			if details /= Void then
				Result := details.contract_end_date_code_key (a_date)
			end
		end

	contract_end_date_code_most_recent_key (a_date: ROSE_DATE): STRING is
		do
			if details /= Void then
				Result := details.contract_end_date_code_most_recent_key (a_date)
			end
		end

	index_multiplier (a_date: ROSE_DATE): DOUBLE is
		local
			l_multiplier_string: STRING
		do
			if details /= Void then
				l_multiplier_string := details.index_multiplier_key (a_date)
				if l_multiplier_string /= Void and then l_multiplier_string.is_double then
					Result := l_multiplier_string.to_double
				else
					Result := missing.double
				end
			else
				Result := missing.double
			end
		end

	index_multiplier_most_recent (a_date: ROSE_DATE): DOUBLE is
		local
			l_multiplier_string: STRING
		do
			if details /= Void then
				l_multiplier_string := details.index_multiplier_most_recent_key (a_date)
				if l_multiplier_string /= Void and then l_multiplier_string.is_double then
					Result := l_multiplier_string.to_double
				else
					Result := missing.double
				end
			else
				Result := missing.double
			end
		end

	index_multiplier_key (a_date: ROSE_DATE): STRING is
		do
			if details /= Void then
				Result := details.index_multiplier_key (a_date)
			end
		end

	index_multiplier_most_recent_key (a_date: ROSE_DATE): STRING is
		do
			if details /= Void then
				Result := details.index_multiplier_most_recent_key (a_date)
			end
		end

	initial_margin_key (a_date: ROSE_DATE): STRING is
		do
			if details /= Void then
				Result := details.initial_margin_key (a_date)
			end
		end

	initial_margin_most_recent_key (a_date: ROSE_DATE): STRING is
		do
			if details /= Void then
				Result := details.initial_margin_most_recent_key (a_date)
			end
		end

	market_open_1_gmt_key (a_date: ROSE_DATE): STRING is
		do
			if details /= Void then
				Result := details.market_open_1_gmt_key (a_date)
			end
		end

	market_open_1_gmt_most_recent_key (a_date: ROSE_DATE): STRING is
		do
			if details /= Void then
				Result := details.market_open_1_gmt_most_recent_key (a_date)
			end
		end

	market_close_1_gmt_key (a_date: ROSE_DATE): STRING is
		do
			if details /= Void then
				Result := details.market_close_1_gmt_key (a_date)
			end
		end

	market_close_1_gmt_most_recent_key (a_date: ROSE_DATE): STRING is
		do
			if details /= Void then
				Result := details.market_close_1_gmt_key (a_date)
			end
		end

	market_open_2_gmt_key (a_date: ROSE_DATE): STRING is
		do
			if details /= Void then
				Result := details.market_open_2_gmt_key (a_date)
			end
		end

	market_open_2_gmt_most_recent_key (a_date: ROSE_DATE): STRING is
		do
			if details /= Void then
				Result := details.market_open_2_gmt_most_recent_key (a_date)
			end
		end

	market_close_2_gmt_key (a_date: ROSE_DATE): STRING is
		do
			if details /= Void then
				Result := details.market_close_2_gmt_key (a_date)
			end
		end

	market_close_2_gmt_most_recent_key (a_date: ROSE_DATE): STRING is
		do
			if details /= Void then
				Result := details.market_close_2_gmt_most_recent_key (a_date)
			end
		end

	mtms_code_key (a_date: ROSE_DATE): STRING is
		do
			if details /= Void then
				Result := details.mtms_code_key (a_date)
			end
		end

	mtms_code_most_recent_key (a_date: ROSE_DATE): STRING is
		do
			if details /= Void then
				Result := details.mtms_code_most_recent_key (a_date)
			end
		end

	type_key (a_date: ROSE_DATE): STRING is
		do
			if details /= Void then
				Result := details.type_key (a_date)
			end
		end

	type_most_recent_key (a_date: ROSE_DATE): STRING is
		do
			if details /= Void then
				Result := details.type_most_recent_key (a_date)
			end
		end

feature -- Relationships

	currency: like context.currency_template is
			--Arbitrary related currency
		do
			Result := something_related_by_future_to_currency
		end

	future_to_currency_history: ROSE_ENTITY_HISTORY [like context.currency_template] is
		do
			if details /= Void then
				Result := details.future_details_to_currency_history
			end
		end

	something_related_by_future_to_currency: like context.currency_template is
		do
			if details /= Void then
				Result := details.something_related_by_future_details_to_currency
			end
		end

feature {NONE} -- Constants

	peer_anchor: PERSISTENT_FUTURE is
		do
		end

	issuer: PERSISTENT_COMPANY is
			-- Type returned by 'peer' function
		do
		end

	issue_listing_template: PERSISTENT_FUTURE_LISTING is
		do
		end

	details: like context.future_details_template is
		require
			only_one_details: all_details /= Void implies all_details.count = 1
		do
			if all_details = Void then
				all_details := future_to_future_details_history
			end
			if not all_details.is_empty then
				Result := all_details.first.entity
			end
		end

	all_details: ROSE_ENTITY_HISTORY [like context.future_details_template]

end
