note

	description:

		"Persisted listing."

	keywords: "persistent, entity, listing"
	author: "Mark Howard"
	last_editor: "$Author: ext-asurelia $"
	date: "$Date: 2015/03/04 $"
	revision: "$Revision: #32 $"
	path: "$File: //rose/source/kernel/entities/persistence/persistent_listing.e $"

class
	PERSISTENT_LISTING

inherit
	PERSISTENT_LISTING_ACCESS
		rename
			something_corelated_by_security_to_listing as security,
			something_related_by_listing_to_investment_region as investment_region,
			something_related_by_listing_to_exchange as persistent_exchange
		redefine
			name_identifier_key,
			name_identifier_keys,
			peer_anchor,
			descriptive_name
		end
	HAS_LISTINGS
	ATTRIBUTES
		rename
			msci as attributes_msci,
			cusip as attributes_cusip,
			company_name as attributes_company_name,
			sedol as attributes_sedol,
			brim_code as attributes_brim_code,
			reuters_code as attributes_reuters_code,
			tse_code as attributes_tse_code,
			isin_code as attributes_isin_code,
			xrf_code as attributes_xrf_code
		undefine
			is_equal
		end

create
make,
make_anonymous

feature -- Access

	country: like context.country_template is
			-- Country, related through exchange, for this listing
		do
			if exchange /= Void then
				Result := exchange.country
			end
		end

	exchange: like context.exchange_template is
			-- Exchange related to this entity.
		local
			l_brimco,l_exchange_code:STRING
		do
			result:=persistent_exchange
			if result= Void or else not result.is_valid then
				l_brimco:=brim_code_key (void)
				if l_brimco/=void then
					l_exchange_code:=l_brimco.substring (l_brimco.count-2,l_brimco.count)
					result:=context.exchange_at (attributes_brim_code (l_exchange_code),void)
				end
			end
		end

	exchanges: ROSE_LINEAR_ARRAY [like context.exchange_template] is
			-- All exchanges related to this listing.
		local
			l_exchanges: ROSE_ENTITY_HISTORY [like context.exchange_template]
		do
			create Result.make_empty
			l_exchanges := context.listing_to_exchange.related_entity_intervals (entity)
			if l_exchanges /= Void then
				from l_exchanges.start until l_exchanges.off loop
					Result.extend (l_exchanges.current_item.entity)
					l_exchanges.forth
				end
			end
		end

	descriptive_name: STRING is
			-- Descriptive name for this listing
		local
			l_identifier: ENTITY_IDENTIFIER
			l_handle: INTEGER
			l_zone: STRING
		do
			l_zone := zone_code_key (today)
			if l_zone /= Void then
				if l_zone.as_upper.is_equal ("U") or l_zone.as_upper.is_equal ("C") then
					l_handle := context.rose_attribute_to_local.item (context.attribute_ids.ticker_type_id)
				elseif l_zone.as_upper.is_equal ("J") then
					l_handle := context.rose_attribute_to_local.item (context.attribute_ids.reuters_code_type_id)
				else
					l_handle := context.rose_attribute_to_local.item (context.attribute_ids.sedol_type_id)
				end
				l_identifier := context.identifier (l_handle)
			else
				l_handle := context.rose_attribute_to_local.item (context.attribute_ids.sedol_type_id)
				l_identifier := context.identifier (l_handle)
			end
			if l_identifier /= Void then
				Result := l_identifier.dated_key (Current,today)
			end
			if Result = Void then
				Result := Precursor
			end
		end

	name_identifier_key: STRING is
			-- Attribute that is used to describe this entity
		do
			Result := "listing description"
		end

	name_identifier_keys: LIST[STRING] is
			-- Atrributes that are needed to compute the descriptive name
			-- Has to be redefined in descendants that use more than `name_identifier_key'
		local
			l_key: STRING
		once
			create {ARRAYED_LIST[STRING]} Result.make (5)
			l_key := name_identifier_key
			if l_key /= Void then
				Result.extend (l_key)
			end
			l_key := context.attribute_name_by_id (context.attribute_ids.ticker_type_id)
			if l_key /= Void then
				Result.extend (l_key)
			end
			l_key := context.attribute_name_by_id (context.attribute_ids.reuters_code_type_id)
			if l_key /= Void then
				Result.extend (l_key)
			end
			l_key := context.attribute_name_by_id (context.attribute_ids.sedol_type_id)
			if l_key /= Void then
				Result.extend (l_key)
			end
			l_key := context.attribute_name_by_id (context.attribute_ids.zone_code_type_id)
			if l_key /= Void then
				Result.extend (l_key)
			end
			Result.compare_objects
		end

	reuters_code_after (a_date: ROSE_DATE): STRING is
			-- Reuters Code as of a_date or later (nearest if multiple)
		local
			l_identifier: ENTITY_IDENTIFIER
		do
			l_identifier := context.identifier_from_rose (attribute_ids.reuters_code_type_id)
			if l_identifier /= Void then
				Result := l_identifier.key_after (Current, a_date)
			end
		end

feature {NONE} -- Implementation.

	peer_anchor: PERSISTENT_LISTING is
			-- Type returned by 'peer' function
		do end

	listings: ROSE_ENTITY_HISTORY[like context.listing_template] is
			-- Listings for `Current'
		local
			l_interval: ENTITY_INTERVAL[like context.listing_template]
		do
			create Result.make_empty
			create l_interval.make (Current, start_date, end_date, 1)
			Result.extend (l_interval)
		end

end
