note

	description:

		"Persisted version of security entity."

	author: "Mark Howard"
	last_editor: "$Author: mk $"
	date: "$Date: 2012/06/22 $"
	revision: "$Revision: #26 $"
	path: "$File: //rose/source/kernel/entities/persistence/persistent_security.e $"

class
	PERSISTENT_SECURITY

inherit

	PERSISTENT_SECURITY_ACCESS
		rename
			something_corelated_by_company_to_security as company,
			something_related_by_security_to_investment_region as investment_region,
			security_to_listing_history as listings,
			security_to_market_listing_history as market_listings
		redefine
			name_identifier_key,
			name_identifier_keys,
			peer_anchor,
			descriptive_name
		end

	HAS_LISTINGS

create
	make,make_anonymous

feature -- Access

	listing_count: INTEGER is
			-- Number of listings
		do
			if listings /= void then
				Result := listings.count
			end
		ensure
			listings_count: (listings = Void and then Result = 0) or else (listings /= Void and then Result = listings.count)
		end

	listing (i: INTEGER): like context.listing_template is
			-- Listing related to this security
		local
			l_listings: like listings
		do
			l_listings := listings
			if l_listings /= void and then l_listings.valid_index (i) then
				Result ?= l_listings.item (i).entity
				check
					Result /= void
				end
			end
		end

	cusip_key_with_checksum (a_date : ROSE_DATE) : STRING is
			-- Cusip with computed checksum
		do
			Result := cusip_key_with_checksum_internal (cusip_key (a_date))
		end

	isin_calculated: STRING is
			-- ISIN Code for `Current'
		local
			l_country_code: STRING
			l_cusip: STRING
			l_maximum_date_time: ROSE_DATE
		do
			create l_maximum_date_time.make_max_date
			Result := isin_code_most_recent_key (l_maximum_date_time)
			Result := Result
			if Result /= Void and then Result.count > 2 then
				l_country_code := Result.substring (1, 2)
				if l_country_code.is_equal ("US") or l_country_code.is_equal ("CA") then
					l_cusip := cusip_key_with_checksum_internal (cusip_most_recent_key (l_maximum_date_time))
					if l_cusip /= Void and then l_cusip.count = 9 then
						if (l_cusip.item (1).is_digit or l_cusip.item (1) = 'U' or l_cusip.item (1) = 'C') and l_cusip.item (8) /= 'X' then
							Result := isin_key_with_checksum_internal (l_country_code + l_cusip)
						end
					end
				end
			end
		end

feature -- Descriptive name features

	descriptive_name: STRING is
			-- Descriptive name for this entity
		Local
			l_identifier: ENTITY_IDENTIFIER
			l_handle: INTEGER
			l_zone: STRING
			l_peer: PERSISTENT_SECURITY
			l_context : like context_anchor
		do
			l_zone := zone_code_most_recent_key (today)
			if l_zone = Void then
				l_context := contexts.descriptor_from_name ("RESEARCH").context
				if l_context /= Void then
					l_peer := peer (l_context, today)
				end
				if l_peer /= Void then
					l_zone := l_peer.zone_code_most_recent_key (today)
				end
			end
			if l_zone /= Void then
				if l_zone.as_upper.is_equal ("U") or l_zone.as_upper.is_equal ("C") then
					l_handle := context.rose_attribute_to_local.item (context.attribute_ids.cusip_type_id)
				elseif l_zone.as_upper.is_equal ("J") then
					l_handle := context.rose_attribute_to_local.item (context.attribute_ids.tse_code_type_id)
				else
					l_handle := context.rose_attribute_to_local.item (context.attribute_ids.isin_code_type_id)
				end
				l_identifier := context.identifier (l_handle)
				if l_identifier /= Void then
					Result := l_identifier.dated_key (Current,today)
				end
			end
			if Result = Void then
				Result := precursor
			end
		end

	name_identifier_keys: LIST[STRING] is
			-- Atrributes that are needed to compute the descriptive name
			-- Has to be redefined in descendants that use more than `name_identifier_key'
		local
			l_key: STRING
		once
			create {ARRAYED_LIST[STRING]} Result.make (5)
			l_key := name_identifier_key
			if l_key /= Void then
				Result.extend (l_key)
			end
			l_key := context.attribute_name_by_id (context.attribute_ids.cusip_type_id)
			if l_key /= Void then
				Result.extend (l_key)
			end
			l_key := context.attribute_name_by_id (context.attribute_ids.tse_code_type_id)
			if l_key /= Void then
				Result.extend (l_key)
			end
			l_key := context.attribute_name_by_id (context.attribute_ids.isin_code_type_id)
			if l_key /= Void then
				Result.extend (l_key)
			end
			l_key := context.attribute_name_by_id (context.attribute_ids.zone_code_type_id)
			if l_key /= Void then
				Result.extend (l_key)
			end
			Result.compare_objects
		end

	name_identifier_key: STRING is
			-- Attribute that is used to describe this entity
		do
			Result := "security name"
		end

feature {NONE} -- Constants

	isin_key_with_checksum_internal (a_isin: STRING): STRING is
		local
			i, j, l_in, l_check_digit : INTEGER
			l_character : CHARACTER
			l_sum, l_zero, l_a, l_multiplier : INTEGER
			l_string1, l_string2: STRING
		do
			if a_isin /= void and a_isin.count >= 11 then
					-- [sm] any reason why this is not once features?
				l_zero := ('0').code
				l_a := ('A').code
					--
				l_string1 := ""
				l_string2 := ""
				from i := 1 until i > 11 loop
					l_character := a_isin.item (i)
					if l_character.is_digit then
						l_in := l_character.code - l_zero
					else
						l_in := l_character.code - l_a + 10
					end
					l_string1.append (l_in.out)
					i := i + 1
				end
				from i := l_string1.count; j := 1  until i < 1 loop
						-- [sm] this can be replaced by: l_multiplier := 1 + (j \\ 2 /= 0).to_integer, and the l_multiplier local variable removed
					if j \\ 2 = 0 then
						l_multiplier := 1
					else
						l_multiplier := 2
					end
					l_string2.append ((l_multiplier * (l_string1.item (i).code - l_zero)).out)
					j := j + 1
					i := i - 1
				end
				from i := 1  until i > l_string2.count loop
					l_sum := l_sum + l_string2.item (i).code - l_zero
					i := i + 1
				end
				l_check_digit := (10 - (l_sum \\ 10)) \\ 10
				Result := a_isin.substring (1, 11) + l_check_digit.out
			end
		end

	cusip_key_with_checksum_internal (a_cusip: STRING): STRING 
			--Cusip with check digit
		local
			l_oddsum,l_evensum,i,l_in,l_check_digit : INTEGER
			l_character : CHARACTER
			l_zero,l_a : INTEGER
		do
			if a_cusip /= void then
				l_zero := ('0').code
				l_a := ('A').code
				l_oddsum := 0;
				from i :=1 until i > a_cusip.count loop
					l_character := a_cusip.item(i)
					if l_character.is_digit then
						l_in := l_character.code - l_zero
					else
						l_in := l_character.code - l_a + 10
					end
					l_in := (l_in \\ 10) + (l_in // 10)
					l_oddsum := l_in + l_oddsum
					i := i + 2
				end
				from i := 2 until i > a_cusip.count loop
					l_character := a_cusip.item(i)
					if l_character.is_digit then
						l_in := 2*(l_character.code - l_zero)
					else
						l_in := 2*(l_character.code - l_a + 10)
					end
					l_in := (l_in \\ 10) + (l_in // 10)
					l_evensum := l_in + l_evensum
					i := i + 2
				end
				l_check_digit := (10 - ((l_oddsum+l_evensum) \\ 10)) \\ 10
				Result := a_cusip + (('0') + l_check_digit).out
			end
		end

	peer_anchor: PERSISTENT_SECURITY is
			-- Type returned by 'peer' function
		 do
		 end

end
