note

	description:

		"[
        Class that allows you to use a persistent server as a client.
        Order of usage: make get_port_from_port_server connect_to_server.
        If you need to add arguments, use `make_with_arguments'.
        DISASTER!  Half of this code has been copied from ROSE_DEDICATED_NETWORK_CLIENT!
        ^^ This class was an attempt to consolidate the code needed to interact with a port server
        inheriting from ROSE_DEDICATED_NETWORK_CLIENT would have led to an even messier class.

       ]"

	author: "Jann Roeder"
	last_editor: "$Author: rj $"
	date: "$Date: 2015/03/25 $"
	revision: "$Revision: #35 $"
	path: "$File: //rose/source/kernel/persistence/databases/signature_tagged/persistent_server_client.e $"

class	PERSISTENT_SERVER_CLIENT

inherit

	PERSISTENT_SERVER_CLIENT_COMMON
		rename
			global_trace as trace
		end

	NETWORK_CLIENT
		rename
			make as client_make,
			error as client_error
		redefine
			send,
			receive,
			client_make,
			in_out,
			cleanup
		end

	DATABASE_SERVER
		rename
			servers as db_servers
		export
			{NONE} all
		end

	REFACTORING_HELPER
		export
			{NONE} all
		end

	PM_AWARE

create

	make, make_with_arguments, make_testing

feature {NONE} -- Initialization

	make (a_network_server_name: STRING) is
			-- Create client for server with name `a_network_server_name'
		require
			server_name_valid: a_network_server_name /= Void and then not a_network_server_name.is_empty
		local
			l_time: TIME
		do
			server_name := a_network_server_name

			create l_time.make_now
			create rand.set_seed (l_time.fine_seconds.truncated_to_integer)

			load_server_list
			timer_metric_name := server_name.twin
			if server_arguments /= Void then
				server_arguments.do_all (agent (a: STRING)
		                                     do
											    timer_metric_name.append (" ")
											    timer_metric_name.append (a)
											 end)
			end
		ensure
			server_name_set: server_name = a_network_server_name
		end

	make_with_arguments (a_network_server_name: STRING; a_server_arguments: like server_arguments) is
			-- Create client for a server with name `a_network_server_name' and `a_server-arguments'
		require
			server_name_valid: a_network_server_name /= Void and then not a_network_server_name.is_empty
			arguments_valid: a_server_arguments /= Void and then not a_server_arguments.is_empty and then not a_server_arguments.item (a_server_arguments.lower).is_empty
		do
			server_arguments := a_server_arguments
			server_arguments.compare_objects
			make (a_network_server_name)
		ensure
			server_arguments_set: server_arguments.is_equal (a_server_arguments)
			server_name_set: server_name = a_network_server_name
		end

	make_testing (a_network_server_name: STRING; a_server_arguments: like server_arguments;
		a_server_list, a_server_address: STRING; a_force_server, a_force_sed: BOOLEAN;
		a_server_port: INTEGER) is
			-- Create client for a server with name `a_network_server_name' and `a_server-arguments'.
			-- Enable by-passing port-server by supplying additional arguments.
		require
			server_name_valid: a_network_server_name /= Void and then not a_network_server_name.is_empty
			arguments_valid: a_server_arguments /= Void implies not a_server_arguments.is_empty and then not a_server_arguments.item (a_server_arguments.lower).is_empty
		do
			force_server := a_force_server
			force_is_sed_used := a_force_sed
			force_server_port := a_server_port
			force_server_address := a_server_address
			if a_server_list /= Void then
				Global.env.put (a_server_list, "PERSISTENT_SERVER_LIST")
			end
			if a_server_arguments = Void then
				make (a_network_server_name)
			else
				make_with_arguments (a_network_server_name, a_server_arguments)
			end
		ensure
			server_arguments_set: server_arguments ~ a_server_arguments
			server_name_set: server_name = a_network_server_name
		end

	client_make (a_peer_port: INTEGER; a_peer_name: STRING) is
			-- Connect to `a_peer_name' at `a_peer_port' and set storable mode on the socket
		do
			create in_out.make_client_by_port(a_peer_port, a_peer_name)		-- Copied from NETWORK_CLIENT.make.
			pre_connect
			in_out.connect													-- Copied from NETWORK_CLIENT.make.
			received_byte_count := 0
			reported_received_byte_count := 0
			sent_byte_count := 0
			reported_sent_byte_count := 0
			in_out.set_default_socket_options
			if is_sed_used then
				in_out.use_sed
			end
			if servers = Void then
					-- Servers should not be Void when the normal creation procedures
					-- for this class are used. However, for testing purposes, `client_make'
					-- is often called in descendents of this class, enabling us to bypass
					-- the port server.
				fixme ("Is there a cleaner way to perform this workaround?")
				create servers.make_empty
			end
		end

	pre_connect
	-- You may redefine this if you want to modify the socket 'in_out' before we call 'connect'.  The most common
	-- reason is to set the send or receive buffer size.  This must be done BEFORE the call to 'connect' (try finding
	-- that in the documentation).  Since this is a client, you probably receive large packets and send small ones.
	-- Therefore you probably want to create a large receive buffer and leave the send buffer alone.  But for this
	-- to do any good at all you MUST create a matching send buffer on the server.  Here is sample code:
	--
	--		in.set_receive_buf_size(512 * 1024)	-- Storables are sent in 256 KB chunks.  Make room for 2 of them.
	--
	do
	end

feature -- Access

	port_server_message: PORT_SERVER_MESSAGE
			-- Response of the port server

	server_failure_count: INTEGER

	is_sed_used: BOOLEAN
			-- Are we using the SED serialization framework

feature -- Operations

	timer_metric_name: STRING

	send (a_msg : ANY) is
			-- Send `a_msg' to the connected server.
		require else
			-- This should actually be in {CLIENT}.send
			message_not_void: a_msg /= Void
			connected: connected
		local
			l_timer: ROSE_PROCESS_TIMER
		do
			if is_verbose then
				trace ("$1: Sending to $2%N", <<generator,server_name>>)
			end
			if byte_count_enabled then
				sent_byte_count := sent_byte_count + Global.persist.size_of (a_msg)
				if sent_byte_count > reported_sent_byte_count and sent_byte_count > byte_count_threshold then
					reported_sent_byte_count := sent_byte_count + byte_count_threshold
					trace ("$1 ($2) has sent     $3 bytes%N",
								  <<server_name, argument_string (port_server_message), Global.formatter.formatted_integer (sent_byte_count, 20, ',')>>)
				end
			end
			create l_timer
			in_out.independent_store (a_msg)
			l_timer.interrupt
			add_metric (timer_metric_name + "/send", l_timer.elapsed)
		end

	send_packet (a_object: ANY) is
			-- Send `a_object' using the PACKET framework
		require
			object_not_void: a_object /= Void
			connected: connected
		local
			l_packet: PACKET
			l_stream: STREAM
			l_pointer: MANAGED_POINTER
			l_sed_writer: ROSE_SED_MEMORY_READER_WRITER
			l_timer: ROSE_PROCESS_TIMER
		do
			if is_sed_used then
				create l_sed_writer.make
				l_sed_writer.set_for_writing
				sed.independent_store (a_object, l_sed_writer, True)
				create l_packet.make_from_managed_pointer (l_sed_writer.data)
			else
				create l_stream.make
				l_stream.independent_store (a_object)
				create l_pointer.share_from_pointer (l_stream.item, l_stream.object_stored_size)
				create l_packet.make_from_managed_pointer (l_pointer)
			end
			create l_timer
			in_out.put_integer_32 (l_packet.count)
			in_out.send (l_packet, 0)
			l_timer.interrupt
			add_metric (timer_metric_name + "/send_packet", l_timer.elapsed)
		end

	receive is
			-- Receive message blocking
		local
			l_timer: ROSE_PROCESS_TIMER
		do
			if is_verbose then
				trace ("$1: Receiving from $2%N",<<generator,server_name>>)
			end
			create l_timer
			received := in_out.retrieved
			l_timer.interrupt
			add_metric (timer_metric_name + "/receive", l_timer.elapsed)
			if byte_count_enabled then
				received_byte_count := received_byte_count + Global.persist.size_of (received)
				if received_byte_count > reported_received_byte_count and received_byte_count > byte_count_threshold then
					reported_received_byte_count := received_byte_count + byte_count_threshold
					trace ("$1 ($2) has received $3 bytes%N",
								  <<server_name, argument_string (port_server_message), Global.formatter.formatted_integer (received_byte_count, 20, ',')>>)
				end
			end
		end

	receive_packet is
			-- Receive message blocking using the PACKET interface
		local
			l_packet: PACKET
			l_stream: STREAM
			l_received_data: MANAGED_POINTER
			l_sed_reader: ROSE_SED_MEMORY_READER_WRITER
			l_size: INTEGER
			l_timer: ROSE_PROCESS_TIMER
		do
			if is_verbose then
				trace ("$1: Receiving from $2%N",<<generator,server_name>>)
			end
			create l_timer
			in_out.read_integer_32
			l_size := in_out.last_integer_32
			l_packet := in_out.receive (l_size, 0)
			l_received_data := l_packet.data
			l_timer.interrupt
			add_metric (timer_metric_name + "/receive_packet", l_timer.elapsed)

			if is_sed_used then
				create l_sed_reader.make_with_buffer (l_received_data)
				l_sed_reader.set_for_reading
				received := sed.retrieved (l_sed_reader, False)
			else
				create l_stream.make_with_size (l_received_data.count)
				l_stream.item.item.memory_copy (l_received_data.item, l_received_data.count)
				received := l_stream.retrieved
			end

			if byte_count_enabled then
				received_byte_count := received_byte_count + Global.persist.size_of (received)
				if received_byte_count > reported_received_byte_count and received_byte_count > byte_count_threshold then
					reported_received_byte_count := received_byte_count + byte_count_threshold
					trace ("$1 ($2) has received $3 bytes%N",
								  <<server_name, argument_string (port_server_message), Global.formatter.formatted_integer (received_byte_count, 20, ',')>>)
				end
			end
		end

	cleanup is
			-- Shut down the client.
		do
			if in_out /= Void and then in_out.exists then
				Precursor
			end
			connected := False
		end

	connect_to_server is
			-- Connect to the server `current_server' at port `server_port'
			-- Raises an exception if connection fails
		require
			server_port_valid: server_port > 0
			not_connected: not connected
			port_server_message_not_void: port_server_message /= Void
		local
			l_collecting: BOOLEAN
			l_attempts: INTEGER
		do
			l_collecting := memory.collecting

			if force_server then
				-- This is for debugging only.
				-- Set correct storable type
				if force_is_sed_used then
					use_sed
				else
					use_storable
				end

				get_supported_security_systems_list(force_server_address, force_server_port, port_server_message)
				client_make (force_server_port, force_server_address)
			else
				-- Set correct storable type
				if current_server.is_sed_used then
					use_sed
				else
					use_storable
				end

				get_supported_security_systems_list(current_server.address, server_port, port_server_message)
				client_make (server_port, current_server.address)
			end

			-- After calling 'client_make' the server will only wait a few seconds for us to respond.  See PERSISTENT_SERVER.receive.

			if not in_out.socket_ok then
				if is_verbose then
					trace ("$1: $2 for attempt on $3%N",
								  <<generator, in_out.error, server_name>>)
				end
				raise ("Socket not OK")
			else
				connected := True
				if port_server_message.server_arguments /= Void and then not port_server_message.server_arguments.is_empty then
					trace ("$1: using $2 ($3) on $4:$5, client port $6%N",
						<<generator, server_name, argument_string (port_server_message), current_server.address, server_port, in_out.debug_output>>)
				else
					trace ("$1: using $2 on $3:$4, client port $5%N",
						<<generator, server_name, current_server.address, server_port, in_out.debug_output>>)
				end

				set_process_identifiers(port_server_message)
				port_server_message.set_user_id(global.env.current_user.login_id)
				port_server_message.set_machine(global.env.host_name)

				if allowed_security_system_names = Void
				then
					-- The server does not support security.
					-- Send magic cookie
					send (create {PAIR[INTEGER, ARRAY[ANY]]}.make (21, <<(True).to_reference, port_server_message>>))
				else
					secure_connect
				end
			end
		rescue
			if l_collecting	then
				memory.collection_on
			else
				memory.collection_off
			end
			if l_attempts < max_connection_attempts then
				l_attempts := l_attempts + 1
				retry
			else
				raise (connecting_failed_exception_string)
			end
		end

	use_storable is
			-- Call this to force storables on this socket to be the old storable type.
		do
			if in_out /= Void then
				in_out.use_storable
			end
			is_sed_used := False
		end

	use_sed is
			-- Call this to force storables on this socket to be the new SED type.
		do
			if in_out /= Void then
				in_out.use_sed
			end
			is_sed_used := True
		end

	get_port_from_port_server is
			-- connect to the port_server on the host identified as `current_server',
			-- send the port_server_message and receive the response, process the response
		require
			not_connected: not connected
		local
			l_collecting: BOOLEAN
			l_target_hostname: STRING
		do
			if not force_server then
				l_collecting := memory.collecting

				l_target_hostname := current_server.address

				if l_target_hostname = Void then
					l_target_hostname := "<Void>"
				end

				-- Connect to the port_server.
				-- The port server always uses the old storable type
				use_storable
				port_server_message := new_port_server_query_message
				client_make (port_server_port, current_server.address)
				if is_verbose then
					trace ("$1: Connecting to port_server $2:$3%N",
								  <<generator, l_target_hostname, port_server_port>>)
				end

				-- Send it our request.
				in_out.independent_store (port_server_message)
				if is_verbose then
					trace ("$1: Receiving from port_server%N",<<generator>>)
				end
				received := in_out.retrieved
				port_server_message ?= received
				if port_server_message /= Void then
					server_port := port_server_message.get_port
					if is_verbose then
						trace ("$1: Received port_server response, port: $2%N",
									  <<generator, server_port>>)
					end
				else
					server_port := -1
				end

				-- Now disconnect
				cleanup
			else
				server_port := force_server_port
				port_server_message := new_port_server_query_message
			end
		ensure
			--not_connected: in_out /= Void implies in_out.is_closed
		rescue
			if l_collecting	then
				memory.collection_on
			else
				memory.collection_off
			end
			if is_verbose then
				trace ("$1: port_server_connect: Error connecting to port_server on $2:$3%N$4%N",
							  <<generator, l_target_hostname, port_server_port, exception_trace>>)
				if in_out /= Void then
					trace ("Error: $1: $2%N",
								  <<in_out.error_number, in_out.error>>)
				end
			end
			trace ("$1: No port_server on $2 for $3%N",
						  <<generator, l_target_hostname,server_name>>)
			if not server_list_exhausted then
				next_server
				retry
			end
			server_port := -1
			cleanup
		end

	get_port_and_connect is
			-- Get the port from the port server and connect
		local
			l_server_name: STRING
			l_machine: STRING
		do
			get_port_from_port_server
			from
			until
				port_server_message.connect_to_server or server_list_exhausted
			loop
				trace ("$1: $2 not available on $3...failing over%N",
									  <<generator, server_name, current_server>>)
				next_server
			end
			if not server_list_exhausted then
				-- Sleep 1 second to give the server time to start up
				global.env.sleep (1000)

				if server_port > 0 then
					if server_name/=Void
					then
						l_server_name := server_name
					else
						l_server_name := "Void"
					end

					if port_server_message.machine/=Void
					then
						l_machine := port_server_message.machine		-- This will be modified by 'connect_to_server'.
					else
						l_machine := "Void"
					end

					connect_to_server

					global.file_access_log.record_persistent_server_read_access(
						l_server_name + args_string(server_arguments),
						l_machine + ":" + port_server_message.port.out)
				end
			else
				raise ("No server available")
			end
		rescue
			if tag_name.is_equal (connecting_failed_exception_string) then
				next_server
				retry
			else
				-- Let exception fall through
			end
		end

feature -- Constants

	max_connection_attempts: INTEGER is 2

	max_server_failures: INTEGER is 2

	default_port_server_port: INTEGER is 2000

	connecting_failed_exception_string: STRING is "Connection failed"

	byte_count_threshold: INTEGER is 1_000_000

	port_server_port_config_key: STRING is
			-- Config key name for port_server port
			-- Can be redefined to partition uses of port server
		do
			Result := "/port_server/port"
		ensure
			valid_result: Result /= Void
		end

feature -- Query

	connected: BOOLEAN
			-- Are we connected to the server?

	local_host_name: STRING is
			-- Hostname of this machine
		local
			l_host: HOST_ADDRESS
		once
			create l_host.make_local
			Result := l_host.local_host_name
		end

	server_name: STRING
			-- Name of the server we want to connect to, this is not a hostname but a service name like 'persistent_research_server'

	server_port: INTEGER
			-- Port of the server we want to connect to (this is received from the port server)

	server_arguments: ARRAY[STRING]
			-- Arguments for the server

	port_server_port: INTEGER is
			-- Return the port number of the port server
		local
			l_port : STRING
			l_config_port : STRING
		once
			l_config_port := Global.app.configured_value (port_server_port_config_key)
			l_port := global.env.get ("TEST_PORT_SERVER")
			if l_port /= Void and then l_port.is_integer then
				Result := l_port.to_integer
			elseif l_config_port /= Void and then l_config_port.is_integer then
				Result := l_config_port.to_integer
			else
				Result := default_port_server_port
			end
			if Result <= 0 then
				raise ("invalid port_number: " + Result.out)
			end
		ensure
			valid_result: Result > 0
		end

	process_id: INTEGER is
			-- Returns the process ID of the current application
		do
			Result := global.env.current_process_id
		end

	current_server: PERSISTENT_SERVER_LIST_ITEM is
			-- Address of the server we are talking to at the moment
		do
			if servers.valid_index (current_server_index) then
				Result := servers.item (current_server_index)
			end
		end

	debug_output: STRING
	-- <Precursor>
	do
		if server_name /= Void
		then
			Result := server_name.twin
		else
			Result := "Void"
		end

		Result.append("/")

		if args_string(server_arguments) /= Void
		then
			Result.append(args_string(server_arguments))
		else
			Result.append("Void")
		end
	end

feature -- Status reporting

	sent_byte_count, received_byte_count: INTEGER

	reported_sent_byte_count, reported_received_byte_count: INTEGER

	is_verbose : BOOLEAN is
			-- Is verbose mode enabeld?
		local
			l_verbose : STRING
		once
			l_verbose := global.env.get ("VERBOSE_EIFFEL")
			Result := l_verbose /= Void or config.boolean ("VERBOSE_EIFFEL",False)
		end

	byte_count_enabled : BOOLEAN is
			-- Is byte counting enabled?
		once
			Result := config.boolean ("rdnc_byte_count_enabled", False)
		end

feature {NONE} -- Utility functions

	argument_string (a_port_server_message: PORT_SERVER_MESSAGE): STRING is
			-- Get the server arguments from a PORT_SERVER_MESSAGE
		require
			message_not_void: a_port_server_message /= Void
		local
			i : INTEGER
		do
			from
				i:= a_port_server_message.server_arguments.lower
			until
				i > a_port_server_message.server_arguments.upper
			loop
				if Result = Void then
					Result := ""
				else
					Result.append (",")
				end
				if port_server_message.server_arguments.item (i) /= Void then
					Result.append (port_server_message.server_arguments.item (i))
				end
				i := i + 1
			end
		end

	create_argument_array (a_argument_string: STRING): ARRAY[STRING] is
			-- Create an array of strings with one argument per item from an argument string
			-- We use the equal sign in the argument string to determine when a new argument starts
			-- i.e. arguments must not use = other than to assign arguments
			-- The arguments have to be in the format used in the persistent_servers DB table:
			-- arg1=bla arg2=bla bla arg3=TRUE
			-- So no dashes or arguments without =
		require
			argument_string_not_void: a_argument_string /= Void
		local
			l_current_argument_start: INTEGER
			l_next_argument_start: INTEGER
			l_current_arg: STRING
			l_argument_count: INTEGER
			l_i: INTEGER
			l_last: BOOLEAN -- For the last arg, we don't have a next one, so we need to do some special tricks
		do
			l_argument_count := a_argument_string.occurrences ('=')
			create Result.make_filled (Void, 1, l_argument_count)
			from
				l_current_argument_start := 1
				l_next_argument_start := a_argument_string.index_of ('=', a_argument_string.index_of ('=', l_current_argument_start) + 1)
				if l_next_argument_start = 0 then
					l_next_argument_start := a_argument_string.count
					l_last := True
				else
					l_next_argument_start := a_argument_string.last_index_of (' ', l_next_argument_start) + 1
				end
				l_i := 0
			variant
				{INTEGER_32}.max_value - l_current_argument_start
			until
				l_next_argument_start = 0
			loop
				l_i := l_i + 1

				if not l_last then
					l_current_arg := a_argument_string.substring (l_current_argument_start, l_next_argument_start - 2)
				else
					l_current_arg := a_argument_string.substring (l_current_argument_start, l_next_argument_start)
				end
				Result.item (l_i) := l_current_arg
				l_current_argument_start := l_next_argument_start
				l_next_argument_start := a_argument_string.index_of ('=', a_argument_string.index_of ('=', l_current_argument_start) + 1)
				if l_next_argument_start = 0 then
					if not l_last then
						l_next_argument_start := a_argument_string.count
						l_last := True
					else
						l_next_argument_start := 0
					end
				else
					l_next_argument_start := a_argument_string.last_index_of (' ', l_next_argument_start)
					if l_next_argument_start /= 0 then
						l_next_argument_start := l_next_argument_start + 1
					end
				end
			end
			Result.compare_objects
		ensure
			result_not_void: Result /= Void
			compare_objects: Result.object_comparison
		end

	args_string (a_args: ARRAY [STRING]): STRING is
			-- A string representing 'a_args' used for resource access tracking.
		local
			i: INTEGER
		do
			if a_args = Void then
				Result := ""
			else
				from
					Result := ""
					i := a_args.lower
				until
					i > a_args.upper
				loop
					if a_args.item (i) /= Void and then not a_args.item (i).is_empty then
						Result.append (" " + a_args.item (i).as_upper)
					end
					i := i + 1
				end
			end
		ensure
			attached_result: Result /= Void
		end

feature {NONE} -- Implementation

	new_port_server_query_message: PORT_SERVER_MESSAGE is
			-- Get a port server message to send to the port server
		do
			create Result.make
			set_process_identifiers (Result)
			if server_arguments /= Void then
				Result.set_server_arguments (server_arguments)
			end
		ensure
			Result_not_void: Result /= Void
		end

	set_process_identifiers(a_port_server_message: PORT_SERVER_MESSAGE) is
			-- Set the attributes of `port_server_message'.
		local
			l_id, l_command: STRING
			l_executable_name, l_server_delivery: STRING
		do
			l_id := global.env.get ("USER")
			if l_id = Void then
				l_id := global.env.get ("USERNAME")
				if l_id = Void then
					l_id := global.env.get ("MACHINE")
					if l_id = Void then
						l_id := global.env.get ("COMPUTERNAME")
					end
				end
			end
			if l_id /= Void then
				a_port_server_message.set_user_id (l_id)
			else
				a_port_server_message.set_user_id ("???")
			end
			l_command := (create {ARGUMENTS}).argument (0)
			if l_command.count > 0 then
				a_port_server_message.set_command_line (l_command + " (" + process_id.out + ")")
			end
			a_port_server_message.set_machine (local_host_name)
			l_executable_name := "\" + server_name + ".exe"
			l_server_delivery := Config.value (server_name + "_delivery")
			if l_server_delivery = Void then
				l_server_delivery := Global.app.delivery.value (server_name + "_delivery")
			end
			if l_server_delivery /= Void and not l_server_delivery.is_empty then
				output (global.interpreted ("($now) Using delivery $1 for $2%N"), <<l_server_delivery.as_upper,server_name>>)
				if not l_server_delivery.item (l_server_delivery.count).is_equal ('\') THEN
					l_server_delivery := l_server_delivery.twin + "\"
				end
				l_executable_name := "\" + l_server_delivery + server_name + ".exe"
			end
			a_port_server_message.set_executable (server_directory_root + l_executable_name)
			a_port_server_message.set_rose_command (server_name.as_upper)
		end

	load_server_list is
			-- Get the server list for server name `server_name' from database and store in `servers'
			-- Note: If the environment variable PERSISTENT_SERVER_LIST is set, its content will be used instead of
			-- the values from the database
		require
			name_valid: server_name /= Void and then not server_name.is_empty
		local
			l_persistent_server_list: STRING
			l_server_string: STRING
			l_storable_type: STRING
			l_server_arguments: ARRAY[STRING]
			l_server_list_predicate: COLLECTING_PREDICATE[PERSISTENT_SERVERS_RECORD]
			l_list_from_db: ARRAY[PERSISTENT_SERVERS_RECORD]
			l_server_item: PERSISTENT_SERVER_LIST_ITEM
			i: INTEGER
			l_server_count: INTEGER
			l_server_args_without_dash: ARRAY[STRING]
			l_default_server: PERSISTENT_SERVERS_RECORD
			l_best_server: PERSISTENT_SERVERS_RECORD
			l_match_count: INTEGER
			l_old_match_count: INTEGER
		do
			l_persistent_server_list := global.env.get ("PERSISTENT_SERVER_LIST")
			if l_persistent_server_list = Void then
				create l_server_list_predicate.make (db_servers.repository_server, "select * from persistent_servers where name = ? and enabled = 1")
				if not l_server_list_predicate.success then
					raise ("Could not create l_server_list_predicate")
				end
				create l_list_from_db.make_from_array (<<>>)
				l_server_list_predicate.set_container (l_list_from_db)
				l_server_list_predicate.retrieve (Void, <<server_name>>)
				if not l_server_list_predicate.success then
					raise ("Could not retrieve with l_server_list_predicate")
				end
				create servers.make_empty

				if server_arguments /= Void then
					l_server_args_without_dash := server_arguments.deep_twin
					l_server_args_without_dash.do_all (agent {STRING}.remove_head (1))

					from
						i := l_list_from_db.lower
					until
						i > l_list_from_db.upper
					loop
						l_storable_type := l_list_from_db.item (i).storable_type
						l_server_string := l_list_from_db.item (i).machine
						if l_list_from_db.item (i).arguments /= Void then
							l_server_arguments := create_argument_array (l_list_from_db.item (i).arguments)
							if l_server_arguments.is_equal (l_server_args_without_dash) then
								create l_server_item.make (l_server_string, l_storable_type.is_equal ("SED"))
								servers.extend (l_server_item)
							end
						end

						i := i + 1
					end
					if servers.is_empty then
						-- If we did't find a server with matching arguments, look for one that doesn't restrict them
						from
							i := l_list_from_db.lower
						until
							i > l_list_from_db.upper
						loop
							l_storable_type := l_list_from_db.item (i).storable_type
							l_server_string := l_list_from_db.item (i).machine
							if l_list_from_db.item (i).arguments = Void or else l_list_from_db.item (i).arguments.is_empty then
								create l_server_item.make (l_server_string, l_storable_type.is_equal ("SED"))
								servers.extend (l_server_item)
							end

							i := i + 1
						end
					end
				else
					-- If `server_arguments' is Void ignore arguments
					from
						i := l_list_from_db.lower
					until
						i > l_list_from_db.upper
					loop
						l_storable_type := l_list_from_db.item (i).storable_type
						l_server_string := l_list_from_db.item (i).machine
						create l_server_item.make (l_server_string, l_storable_type.is_equal ("SED"))
						servers.extend (l_server_item)

						i := i + 1
					end
				end

				if servers.is_empty
				then
					-- See if this server runs on multiple machines.  The column 'required_arguments' determines
					-- which machine to actually run on.  Pick the one that has the most matches.
					from
						i := l_list_from_db.lower
					until
						i > l_list_from_db.upper
					loop
						if l_list_from_db[i].received_arguments=Void or else l_list_from_db[i].received_arguments.count=0
						then
							l_default_server := l_list_from_db[i]
						else
							l_match_count := matching_args_count(l_list_from_db[i].received_arguments, server_arguments)
							if l_match_count>l_old_match_count
							then
								l_best_server := l_list_from_db[i]
								l_old_match_count := l_match_count
							elseif l_match_count>0 and l_match_count=l_old_match_count
							then
								print("PERSISTENT_SERVER_CLIENT.load_server_list - More than one matching row.  The table 'persistent_servers' in 'repository' is corrupt.%N")
							end
						end

						i := i + 1
					end

					if l_best_server/=Void
					then
						create l_server_item.make(l_best_server.machine, l_best_server.storable_type.is_equal("SED"))
						servers.extend(l_server_item)
					elseif l_default_server/=Void
					then
						create l_server_item.make(l_default_server.machine, l_default_server.storable_type.is_equal("SED"))
						servers.extend(l_server_item)
					end
				end

				if servers.is_empty then
					raise ("No servers found, check repository")
				end
			else
				l_server_count := l_persistent_server_list.occurrences (';') + 1
				create servers.make_empty

				from
					i := 1
				until
					i > l_server_count
				loop
					if l_persistent_server_list.has (';') then
						l_server_string := l_persistent_server_list.substring (1, l_persistent_server_list.index_of (';', 1) - 1)
					else
						l_server_string := l_persistent_server_list
					end
					l_storable_type := l_server_string.substring (l_server_string.index_of (',', 1) + 1, l_server_string.count)
					l_server_string := l_server_string.substring (1, l_server_string.index_of (',', 1) - 1)

					if l_persistent_server_list.has (';') then
						l_persistent_server_list.remove_head (l_persistent_server_list.index_of (';', 1))
					end

					create l_server_item.make (l_server_string, l_storable_type.is_equal ("SED"))
					servers.extend (l_server_item)

					i := i + 1
				end
			end

			server_iteration_count := 0
			if not servers.is_empty then
				rand.forth
				current_server_index := (rand.item \\ servers.count) + servers.lower
			else
				current_server_index := -1
			end
		ensure
			servers_set: servers /= Void
		end

	matching_args_count(a_required_args: STRING; a_passed_args: ARRAY[STRING]): INTEGER
	-- Counts up how many args in 'a_passed_args' can be found in 'a_required_args'.
	-- ALL args in 'a_requiresd_args' must be matched or zero is returned.
	-- Note that 'a_required_args' comes from the column 'received_arguments' in the 'persistent_server' table.
	local
		i: INTEGER
		l_required_args: ROSE_DELIMITED_TEXT
		l_done: BOOLEAN
		l_passed_arg: STRING
	do
		create l_required_args.make(a_required_args, ' ')

		from
			l_required_args.start
		until
			l_required_args.off
		loop
			l_required_args.current_item.prune_all(' ')
			l_required_args.current_item.to_upper

			from
				l_done := False
				i := a_passed_args.lower
			until
				i>a_passed_args.upper or l_done
			loop
				l_passed_arg := a_passed_args[i].twin
				l_passed_arg.prune_all(' ')
				l_passed_arg.to_upper
				if equal(l_required_args.current_item, l_passed_arg)
				then
					Result := Result + 1
					l_done := True
				else
					i := i + 1
				end
			end

			l_required_args.forth
		end

		if Result/=l_required_args.count
		then
			Result := 0
		end
	end

	servers: ROSE_LINEAR_ARRAY[PERSISTENT_SERVER_LIST_ITEM]
			-- List of port servers for `server_name'

	current_server_index: INTEGER
			-- Index of the current server in `servers', -1 if none

	next_server is
			-- Set `current_server' to the next available server
		do
			current_server_index := ((current_server_index + 1) \\ servers.count) + servers.lower
			server_iteration_count := server_iteration_count + 1
		end

	server_iteration_count: INTEGER
			-- How many times has `next_server' been called

	server_list_exhausted: BOOLEAN is
			-- Have all servers been tried?
		do
			Result := server_iteration_count >= servers.count
		end

	server_directory_root: STRING is
			-- Root directory for the server executables on the port server
		once
			Result := global.env.get ("NT_SERVER_DIRECTORY")
			if Result = Void then
				Result := global.hardcoded_path ("c:\servers")
			end
		end

feature {NONE} -- Debugging

	force_server: BOOLEAN
			-- Do we always use the data entered below instead of the data from the DB.

	force_server_address: STRING
			-- Server to use when bypassing port server

	force_server_port: INTEGER
			-- Port to use when bypassing port server

	force_is_sed_used: BOOLEAN
			-- Should sed by used when bypassing port server?

feature {NONE} -- Implementation classes

	in_out: ROSE_NETWORK_STREAM_SOCKET

	sed: SED_STORABLE_FACILITIES is
			-- SED instance for serialization / deserialization
		once
			create Result
		end

	memory: MEMORY is
			-- Memory object
		once
			create Result
		end

	rand: RANDOM

invariant
	servers_exists: servers /= Void
--	no_void_servers: not servers.is_empty implies not servers.has (Void)		-- Not thread safe.  Caused mysterious assertion violations.
	server_arguments_valid: server_arguments /= Void implies (not server_arguments.is_empty and then
		not server_arguments.has (Void) and then not server_arguments.item (server_arguments.lower).is_empty)

end
