indexing
	description: "Objects representing persistent server commands."
	author: "MB"
	last_editor: "$Author: rj $"
	date: "$Date: 2015/03/25 $"
	revision: "$Revision: #13 $"
	path : "$File: //rose/source/kernel/persistence/databases/signature_tagged/persistent_server_command.e $"

class
	PERSISTENT_SERVER_COMMAND

inherit
	ROSE_DEDICATED_NETWORK_SERVER
		rename
			make as server_make,
			execute as server_execute
		redefine
			outflow
		end
	PERSISTENT_SERVER_COMMAND_TYPES
	POLL_COMMAND
		rename
			active_medium as outflow
		redefine
			outflow
		end
	MISMATCH_CORRECTOR
		export
			{NONE} all
		redefine
			correct_mismatch
		end

create
	make_command

feature -- Creation

	make_command(s: ROSE_NETWORK_STREAM_SOCKET)
			-- Create as data command.
		do
			actual_make(s)
			set_client_type(data_client_type_id)
		end

	actual_make(s: ROSE_NETWORK_STREAM_SOCKET)
	-- Guts are here so that PERSISTENT_SERVER_MONITOR_COMMAND.make_command can reuse them.
	do
		create start_time.make_now
		create last_message_time.make_now
		outflow := s
	end

feature -- Basic operations

	execute (a_any: like payload_anchor)
			-- Execute command, and expire on any exception.
		local
			l_retried, l_collecting: BOOLEAN
		do
			l_collecting := Global.memory.collecting
			if l_retried then
				recover
			else
				execute_body (a_any)
			end
		rescue
			if l_collecting then
				Global.memory.collection_on
			else
				Global.memory.collection_off
			end
			if not l_retried then
				l_retried := True
				retry
			end
		end

	execute_with_received(a_received: like received; a_any: like payload_anchor)
	-- Call 'execute' but first set 'received' and prevent 'execute' from setting 'received' again.
	do
		received := a_received
		execute_no_receive(a_any)
	end

	execute_no_receive(a_any: like payload_anchor)
	-- Calls 'execute' but instructs it that 'received' has already been set.
	do
		received_already_set := True
		execute(a_any)
		received_already_set := False
	end

	execute_body (a_any: like payload_anchor)
			-- Execute command, allowing exceptions to propogate.
			-- Redefine this routine to perform actual work.
		do
			-- to be redefined.
		end

	recover is
			-- Execute recovery procedure from exceptions raised by `execute_body'.
			-- Redefine for more elaborate procedures, such as conditional logging of information.
		do
			expired := True
		ensure
			connection_expired: expired
		end

	process_message is
		do
		end

	perform_maintenance is
		do
		end

feature -- Transformation

	summary_command : PERSISTENT_SERVER_COMMAND is
		do
			create Result.make_command(Void)
			Result.set_user_name(user_name)
			Result.set_machine_name(machine_name)
			Result.set_start_time(start_time)
			Result.set_last_message_time(last_message_time)
			Result.set_client_pid(client_pid)
			Result.set_number_of_messages(number_of_messages)
			Result.set_client_type(client_type)
			Result.set_command_line (command_line)
			Result.set_executable_name (executable_name)
			Result.set_port (port)
		end

feature -- Client

	user_name : STRING
	set_user_name(a_value : like user_name) is
			-- Assign `a_value' to `user_name'.
		do
			user_name := a_value
		end

	machine_name : STRING
	set_machine_name(a_value : like machine_name) is
			-- Assign `a_value' to `machine_name'.
		do
			machine_name := a_value
		end

	executable_name: STRING
	set_executable_name (a_executable_name: like executable_name) is
			-- Assign `a_executable_name' to `executable_name'.
		do
			executable_name := a_executable_name
		end

	command_line: STRING
	set_command_line (a_command_line: like command_line) is
			-- Assign `a_command_line' to `command_line'.
		do
			command_line := a_command_line
		end

	port: INTEGER
	set_port (a_port: like port) is
			-- Assign `a_port' to `port'.
		do
			port := a_port
		end
	start_time : ROSE_DATE
	set_start_time(a_value : like start_time) is
		do
			start_time := a_value
		end
	last_message_time : ROSE_DATE
	set_last_message_time(a_value : like last_message_time) is
		do
			last_message_time := a_value
		end

	client_pid : INTEGER	-- The process_id of the client.
	set_client_pid(a_value : like client_pid) is
		do
			client_pid := a_value
		end

	number_of_messages : INTEGER
	set_number_of_messages(a_value : like number_of_messages) is
		do
			number_of_messages := a_value
		end

	client_type : INTEGER					-- Either 'data_client_type_id' or 'monitor_client_type_id' (see PERSISTENT_SERVER_COMMAND_TYPES).
	set_client_type(a_value : like client_type) is
		do
			client_type := a_value
		end

	server: PERSISTENT_SERVER				-- If set, we can send commands to it.
	set_server(a_server: like server)
	-- Called by PERSISTENT_SERVER.register_monitor_command so we can call 'server.process_custom_monitor_message'.
	do
		server := a_server
	end

	is_data_client: BOOLEAN
	-- Is this client a 'data_client'?
	do
		Result := client_type=data_client_type_id
	end

	is_monitor_client: BOOLEAN
	-- Is this client a 'monitor_client'?
	do
		Result := client_type=monitor_client_type_id
	end

feature {NONE} -- Anchors.

	payload_anchor: ANY	-- The type of the argument to 'execute', etc.
	do end

feature {NONE} -- Mismatch.
	correct_mismatch
	-- Added to correct for the new attribute 'server'.
	do
		-- mismatch_information.do_nothing
	end

feature -- Status

	expired : BOOLEAN			-- Set to true if an error happens (like the client disconnects).
								-- All expired connections get destroyed in PERSISTENT_SERVER.

feature -- Attributes (alpha order).

	outflow: detachable like in				-- Copied from the definition in SERVER.

	received_already_set: BOOLEAN			-- If True then don't set 'received' (it has already been set).

end

