indexing
	description: "Used by PERSISTENT_SERVER's for connections that are for monitoring rather than the normal data client."
	author: "Mauricio Bustos"
	last_editor: "$Author: rj $"
	date: "$Date: 2012/01/10 $"
	revision: "$Revision: #10 $"
	path : "$File: //rose/source/kernel/persistence/databases/signature_tagged/persistent_server_monitor_command.e $"

class
	PERSISTENT_SERVER_MONITOR_COMMAND

inherit
	PERSISTENT_SERVER_COMMAND
		redefine
			execute,
			make_command
		end
	ROSE_SHARED_GLOBAL

create
	make_command

feature {NONE} -- Initialization.

	make_command(s : ROSE_NETWORK_STREAM_SOCKET)
			-- Just initialization declaring us to be a 'monitor_client'.
		do
			actual_make(s)
			set_client_type(monitor_client_type_id)
		end

feature -- Basic operations.

	execute(a_any : ANY) is
			-- Process the command that the remote monitor client wants us to execute.
		local
			l_retry: BOOLEAN
			l_message: PAIR[INTEGER, ARRAY[ANY]]
			l_connection_list_message: like connections
		do
			if not l_retry then
				create last_message_time.make_now
				number_of_messages := number_of_messages + 1
				l_message ?= outflow.retrieved
				if l_message /= Void then
					inspect l_message.first
					when connection_list_cmd_id then
						create l_connection_list_message.make
						from connections.start until connections.after loop
							l_connection_list_message.extend(connections.item.summary_command)
 							connections.forth
						end
						outflow.independent_store(l_connection_list_message)
					when custom_monitor_cmd_id then
						if server/=Void
						then
							server.process_custom_monitor_message(Current, l_message.second)
						end
					end
				end
			else
				if exceptions.exception/=(create {EXCEP_CONST}).runtime_io_exception and then
				   exceptions.exception/=(create {EXCEP_CONST}).io_exception and then
				   exceptions.exception/=(create {EXCEP_CONST}).retrieve_exception and
				   not (exceptions.is_developer_exception and then equal(exceptions.developer_exception_name, "Socket was closed"))
				   		-- "Socket was closed" is raised in ROSE_NETWORK_STREAM_SOCKET. Review and fix.
				then
					-- We will wind up here if the client closes the socket.  The port_server
					-- does that when testing to see if we are alive.  It never actually sends a command.
					output(Global.interpreted("($now) Failed while executing a monitor command (" +
											  "port: $1, " +
											  "user: $2, " +
											  "machine: $3, " +
											  "exeception: $4)%N"),
											  <<
											  	port_number_string,
											  	user_name,
											  	machine_name,
											  	exceptions.exception
											  >>)
					output (exceptions.exception_trace + "%N", Void)
				end

				expired := True
			end
		rescue
			if not l_retry
			then
				l_retry := True
				retry
			end
		end

	port_number_string: STRING
	-- Safely get the number of the port that we are talking on.
	local
		l_socket: ROSE_NETWORK_STREAM_SOCKET
	do
		if outflow/=Void
		then
			l_socket := outflow
			if l_socket/=Void
			then
				Result := l_socket.port_number_string
			else
				Result := "Void"
			end
		else
			Result := "Void"
		end
	end

feature -- Attributes.

	connections: LINKED_LIST[PERSISTENT_SERVER_COMMAND]		-- All of the current connections, client or data.
	set_connections(a_value : like connections) is
			-- Assign `a_value' to `connections'.
		do
			connections := a_value
		end

end













