note

	description:

		"Objects that are variable entities in a particular context"

	keywords: "variable, entity, persistent"
	author: "Bruno Pradal"
	last_editor: "$Author: mk $"
	date: "$Date: 2013/03/28 $"
	revision: "$Revision: #12 $"
	path: "$File: //rose/source/kernel/entities/persistence/persistent_variable.e $"

class
	PERSISTENT_VARIABLE

inherit
	PERSISTENT_VARIABLE_ACCESS
		redefine
			make,
			ur_targets
		end

create
    make,make_anonymous

feature {NONE} -- Creation

	make(a_context: SUPPLIER_CONTEXT; a_category_id, a_handle: INTEGER; a_start_date, a_end_date: ROSE_DATE) is
			-- Make the variable
		local
			l_today: ROSE_DATE
		do
			Precursor (a_context, a_category_id, a_handle,a_start_date, a_end_date)
			preload_attributes
			create l_today.make_today
			create variable_description_record
			variable_description_record.set_mnemonic (attribute_by_name ("MNEMONIC",l_today))
			variable_description_record.set_name (attribute_by_name ("NAME",l_today))
			variable_description_record.set_variable_index(to_integer (attribute_by_name ("VARIABLE_INDEX",l_today)))
			variable_description_record.set_expression (attribute_by_name ("EXPRESSION",l_today))
			variable_description_record.set_source (attribute_by_name ("SOURCE",l_today))
			variable_description_record.set_parent (attribute_by_name ("PARENT",l_today))
			variable_description_record.set_type (attribute_by_name ("TYPE",l_today))
			variable_description_record.set_scale (attribute_by_name ("SCALE",l_today))
			variable_description_record.set_security(to_integer (attribute_by_name ("SECURITY_LEVEL",l_today)))
			variable_description_record.set_units (attribute_by_name ("UNITS",l_today))
			variable_description_record.set_target (attribute_by_name ("TARGET",l_today))
			variable_description_record.set_stock_or_flow(first_character (attribute_by_name ("STOCK_OR_FLOW",l_today)))
			variable_description_record.set_source_table (attribute_by_name ("SOURCE_TABLE",l_today))
			variable_description_record.set_source_column (attribute_by_name ("SOURCE_COLUMN",l_today))
			variable_description_record.set_expression_type (attribute_by_name ("EXPRESSION_TYPE",l_today))
			variable_description_record.set_mnemonic8 (attribute_by_name ("MNEMONIC8",l_today))
			variable_description_record.set_weighting (attribute_by_name ("WEIGHTING",l_today))
			variable_description_record.set_period (attribute_by_name ("PERIOD",l_today))
			variable_description_record.set_result_string (attribute_by_name ("RESULT_STRING",l_today))
			variable_description_record.set_target_name (attribute_by_name ("TARGET_NAME",l_today))
			variable_description_record.set_j_index(to_integer (attribute_by_name ("J_INDEX",l_today)))
			variable_description_record.set_gv_index(to_integer (attribute_by_name ("GV_INDEX",l_today)))
			variable_description_record.set_spcs_index(to_integer (attribute_by_name ("SPCS_INDEX",l_today)))
			variable_description_record.set_risk_index(to_integer (attribute_by_name ("RISK_INDEX",l_today)))
			variable_description_record.set_group_name (attribute_by_name ("GROUP_NAME",l_today))
			variable_description_record.set_lower_winsor(to_double (attribute_by_name ("LOWER_WINSOR",l_today)))
			variable_description_record.set_upper_winsor(to_double (attribute_by_name ("UPPER_WINSOR",l_today)))
			variable_description_record.set_model_index  (attribute_by_name ("MODEL_INDEX",l_today))
			--variable_description_record.set_model_mnemonic  (attribute_by_name ("MODEL_MNEMONIC",l_today))
		end

feature -- Attributes

	variable_description_record: VARIABLE_DESCRIPTION_RECORD
	
	ur_targets: ROSE_LINEAR_ARRAY [ROSE_TARGET [ENTITY]] is
			-- <Precursor>
		do
			create Result.make_empty
			Result.extend  (create {MODEL_VARIABLE_COEFFICIENT_RECORD})
		end
	
feature {NONE} -- Operations

	preload_attributes is
			-- Preload attributes for PERSISTENT_VARIABLE entities
		require
			valid_category_id: category_id /= 0
		once
			context.identifier_from_name ("MNEMONIC").load_key_history_records_by_category (category_id)
			context.identifier_from_name ("NAME").load_key_history_records_by_category (category_id)
			context.identifier_from_name ("VARIABLE_INDEX").load_key_history_records_by_category (category_id)
			context.identifier_from_name ("EXPRESSION").load_key_history_records_by_category (category_id)
			context.identifier_from_name ("SOURCE").load_key_history_records_by_category (category_id)
			context.identifier_from_name ("PARENT").load_key_history_records_by_category (category_id)
			context.identifier_from_name ("TYPE").load_key_history_records_by_category (category_id)
			context.identifier_from_name ("SCALE").load_key_history_records_by_category (category_id)
			context.identifier_from_name ("SECURITY_LEVEL").load_key_history_records_by_category (category_id)
			context.identifier_from_name ("UNITS").load_key_history_records_by_category (category_id)
			context.identifier_from_name ("TARGET").load_key_history_records_by_category (category_id)
			context.identifier_from_name ("STOCK_OR_FLOW").load_key_history_records_by_category (category_id)
			context.identifier_from_name ("SOURCE_TABLE").load_key_history_records_by_category (category_id)
			context.identifier_from_name ("SOURCE_COLUMN").load_key_history_records_by_category (category_id)
			context.identifier_from_name ("EXPRESSION_TYPE").load_key_history_records_by_category (category_id)
			context.identifier_from_name ("MNEMONIC8").load_key_history_records_by_category (category_id)
			context.identifier_from_name ("VARIABLE_NAME").load_key_history_records_by_category (category_id)
			context.identifier_from_name ("WEIGHTING").load_key_history_records_by_category (category_id)
			context.identifier_from_name ("PERIOD").load_key_history_records_by_category (category_id)
			context.identifier_from_name ("RESULT_STRING").load_key_history_records_by_category (category_id)
			context.identifier_from_name ("TARGET_NAME").load_key_history_records_by_category (category_id)
			context.identifier_from_name ("J_INDEX").load_key_history_records_by_category (category_id)
			context.identifier_from_name ("GV_INDEX").load_key_history_records_by_category (category_id)
			context.identifier_from_name ("SPCS_INDEX").load_key_history_records_by_category (category_id)
			context.identifier_from_name ("RISK_INDEX").load_key_history_records_by_category (category_id)
			context.identifier_from_name ("GROUP_NAME").load_key_history_records_by_category (category_id)
			context.identifier_from_name ("LOWER_WINSOR").load_key_history_records_by_category (category_id)
			context.identifier_from_name ("UPPER_WINSOR").load_key_history_records_by_category (category_id)
			context.identifier_from_name ("MODEL_INDEX").load_key_history_records_by_category (category_id)
			--context.identifier_from_name ("MODEL_MNEMONIC").load_key_history_records_by_category (category_id)
		end

feature {NONE} -- Helpers

	to_integer(a_value: STRING): INTEGER is
			-- Convert to INTEGER
		do
			if a_value /= Void and then a_value.is_integer then
				Result := a_value.to_integer
			end
		end

	to_double(a_value: STRING): DOUBLE is
			-- Convert to double
		do
			if a_value /= Void and then a_value.is_double then
				Result := a_value.to_double
			end
		end

	first_character(a_value: STRING): CHARACTER is
			-- Get the first character of `a_value'
		do
			if a_value /= Void and then a_value.count > 0 then
				Result := a_value.item(1)
			end
		end

end
