note

	description:

		"Class that gives acces to PES features, classes that are bound to a specific context should inherit from this instead of PES_AWARE"

	author: "Jann Roeder"
	last_editor: "$Author: e2 $"
	date: "$Date: 2014/05/27 $"
	revision: "$Revision: #7 $"
	path: "$File: //rose/source/kernel/entities/pes/pes_aware_context.e $"

deferred class PES_AWARE_CONTEXT

inherit

	PES_AWARE

feature -- Access

	pes_api: PES_CLIENT_API is
			-- Return the correct PES_API object for `pes_context_id'
		require
			pes_available: use_pes
		do
			Result := pes_api_for_context_id (pes_context_id)
		ensure
			result_not_void: Result /= Void
		end

feature -- External Access

	use_pes: BOOLEAN is
			-- Query this feature to find out whether PES is enabled and a server is available
		do
			Result := use_pes_internal.item and then has_pes (pes_context_id)
		end

	pes_server: BOOLEAN is
			-- Is current program a PES for `pes_context_id' ?
		do
			Result := pes_server_context_internal.valid_index (pes_context_id) and then
				pes_server_context_internal.item (pes_context_id)
		end

feature -- Operations

	set_pes_server (a_value: BOOLEAN) is
			-- Set this program to be a PES for `pes_context_id'
		do
			pes_server_context_internal.force (a_value, pes_context_id)
		end

feature -- Implementation

	pes_context_id: INTEGER is
			-- Implement this feature to specify which context you are working with
		deferred
		end

end
