indexing
	description: "Query to get some attributes for an array of entities"
	documentation: "[
	]"
	author: "Ueli Peter"
	last_editor: "$Author: upeter $"
	date: "$Date: 2009/01/12 $"
	revision: "$Revision: #1 $"
	path: "$File: //rose/source/Kernel/Entities/pes/pes_query_attributes.e $"

class
	PES_QUERY_ATTRIBUTES

inherit
	ROSE_CS_QUERY

create
	make

feature -- Creation

	make (a_context_index: like context_index; a_entities: like entities; a_identifiers: like identifiers) is
			-- set values
		require
			valid_context: a_context_index > 0
			valid_entities: a_entities /= Void
			valid_identifiers: a_identifiers /= Void
		do
			context_index := a_context_index
			entities := a_entities
			identifiers := a_identifiers
		end

feature -- Access

	context_index: INTEGER
	entities: ARRAY[ENTITY]
	identifiers: ARRAY[STRING]

feature -- Basic operations

execute_query (a_pes_server: PES_WORKER[PES_CONTEXT_AWARE]): PES_RESULT_ATTRIBUTES is
			-- get identifiers
		local
			l_context: SUPPLIER_CONTEXT
			l_retried: BOOLEAN
			l_identifier: ENTITY_IDENTIFIER
			i,e: INTEGER
			l_identifier_records: ARRAY[ROSE_LINEAR_ARRAY[ENTITY_ATTRIBUTE_RECORD]]
			l_result: ARRAY[ARRAY[ROSE_LINEAR_ARRAY[ENTITY_ATTRIBUTE_RECORD]]]
		do
			if not l_retried then
				l_context ?= a_pes_server.context_from_id(context_index)
				if l_context /= Void then
					create l_result.make(1, identifiers.count)
					from
						i := identifiers.lower
					until
						i > identifiers.upper
					loop
						l_identifier := l_context.identifier_from_name (identifiers.item(i))
						if l_identifier /= Void then
							create l_identifier_records.make (1, entities.count)
							from
								e := entities.lower
							until
								e > entities.upper
							loop
								l_identifier_records.put(l_identifier.key_history (entities.item(e)), e)
								e := e + 1
							end
							l_result.put (l_identifier_records, i)
						end
						i := i + 1
					end
				end
				create Result.make (query_id, l_result)
			else
				create Result.make_exception (query_id, "Context not found on this server.")
			end
		rescue
			create Result.make_exception (query_id, tag_name)
			l_retried := True
			retry
		end

	query_information: STRING is
			-- Returns informations about this query
		do
			Result := "PES_QUERY_ATTRIBUTES for context " + context_index.out + " and Identifier " + identifiers.out
		end

feature {NONE} -- Implementation

end
