indexing
	description: "Object to query the server for a list of entities from feature {CATEGORY_HANDLER}.entity_list_from_identifier"
	author: "Jann Roeder"
	date: "$Date: 2009/08/24 $"
	revision: "$Revision: #7 $"
	path: "$File: //rose/source/kernel/entities/pes/pes_query_entity_list.e $"

class
	PES_QUERY_ENTITY_LIST

inherit
	ROSE_CS_QUERY

	CONTEXT_ANCHOR
		undefine
			out
		end

create
	make

feature {NONE} -- Creation

	make (a_category_id: like category_id; a_context_index: like context_index; a_key: like key; a_date: like date; a_identifier_id: like identifier_id) is
			-- Create with values in parameters
		do
			category_id := a_category_id
			context_index := a_context_index
			key := a_key
			date := a_date
			identifier_id := a_identifier_id
		ensure
			category_id_set: category_id = a_category_id
			context_index_set: context_index = a_context_index
			key_set: key = a_key
			date_set: date = a_date
			identifier_id_set: identifier_id = a_identifier_id
		end

feature -- Access

	date: ROSE_DATE
		-- Date to retrieve the entities

	key: STRING
		-- Search string

	identifier_id: INTEGER
		-- ID of the entity_identifier

	category_id: INTEGER
		-- identifier for category

	context_index: INTEGER
		-- index of context

feature -- Basic operations

	execute_query (a_pes_server: PES_WORKER[PES_CONTEXT_AWARE]): PES_RESULT_ENTITY_LIST is
			-- Execute the query
		local
			l_category_handler: CATEGORY_HANDLER[like context_anchor, ENTITY]
			l_result: ROSE_LINEAR_ARRAY[ENTITY]
			l_identifier: ENTITY_IDENTIFIER
			l_retried: BOOLEAN
		do
			if not l_retried then
				l_category_handler := a_pes_server.context_from_id(context_index).handler (category_id)
				check
					valid_category: l_category_handler /= Void
				end
				l_identifier := l_category_handler.identifiers.item (identifier_id)
				-- this call doesn't make sense because it goes to the DB
				-- Not always, there is a hash map that handles unique identifiers. and the entities are in the cache as well
				l_result := l_category_handler.entity_list_from_identifier (key, date, l_identifier)
				create Result.make (query_id, l_result)
			end
		rescue
			create Result.make (query_id, Void)
			Result.set_remote_exception (tag_name)
			l_retried := True
			retry
		end

	query_information: STRING is
			-- Informations about this query
		do
			Result := "PES_QUERY_ENTITY_LIST for context " + context_index.out + " and category " + category_id.out
		end

end
