indexing

	description: "replaces CATEGORY_HANDLER.load_alle_entities_into_cache"
	author: "Ueli Peter"
	last_editor: "$Author: c4 $"
	date: "$Date: 2011/11/10 $"
	revision: "$Revision: #7 $"
	path: "$File: //rose/source/kernel/entities/pes/pes_query_load_category.e $"


class	PES_QUERY_LOAD_CATEGORY

inherit

	ROSE_CS_QUERY

create

	make

feature {NONE} -- Initialization

	make (a_category_id: like category_id; a_context_index: like context_index) is
			-- Create with `a_category_id' and `a_context_index'
		do
			category_id := a_category_id
			context_index := a_context_index
		ensure
			category_id_set: category_id = a_category_id
			context_index_set: context_index = a_context_index
		end

feature -- Access

	category_id: INTEGER
		-- Identifier for category

	context_index: INTEGER
		-- Index of context

	only_table: BOOLEAN
		-- Is only table needed?

	only_id_table: BOOLEAN
		-- Is only id_table needed?

	only_identifiers: BOOLEAN
		-- Are only identifiers needed?

feature -- Status Setting

	set_only_identifiers is
			-- Request that only CATEGORY_HANDLER.identifiers is needed.
		do
			only_identifiers := True
			only_id_table := False
			only_table := False
		ensure
			only_identifiers_wanted: only_identifiers
			id_table_not_wanted: not only_id_table
			table_not_wanted: not only_table
		end

	set_only_id_table is
			-- Request that only CATEGORY_HANDLER.entity_id_table is needed.
		do
			only_id_table := True
			only_identifiers := False
			only_table := False	
		ensure
			only_id_table_wanted: only_id_table = True
			table_not_wanted: not only_table
			identifiers_not_wanted: not only_identifiers
		end

	set_only_table is
			-- Request that only CATEGORY_HANDLER.table is needed.
		do
			only_table := True
			only_identifiers := False
			only_id_table := False	
		ensure
			only_table_wanted: only_table = True
			id_table_not_wanted: not only_id_table
			identifiers_not_wanted: not only_identifiers
		end

feature -- Basic operations

	execute_query (a_pes_server: PES_WORKER [PES_CONTEXT_AWARE]): PES_RESULT_LOAD_CATEGORY is
			-- Get requested containers
		local
			l_category_handler: CATEGORY_HANDLER [SUPPLIER_CONTEXT, ENTITY]
			l_context: SUPPLIER_CONTEXT
			l_identifiers: HASH_TABLE [ENTITY_IDENTIFIER, INTEGER]
			l_table_saved: HASH_TABLE [ENTITY, INTEGER]
			l_entity_id_table_saved: HASH_TABLE [ENTITY_ID_BASE, INTEGER]
			l_retried: BOOLEAN
		do
			if not l_retried then
				l_context := a_pes_server.context_from_id (context_index)
				if l_context = Void then
					create Result.make_exception (query_id, "Invalid supplier context (configuration error)")
				else
					l_category_handler := l_context.handler (category_id)
					if l_category_handler = Void then
						create Result.make_exception (query_id, "Invalid category (configuration error)")
					else
						l_entity_id_table_saved := l_category_handler.get_entity_id_table_saved
						l_table_saved := l_category_handler.get_table_saved
						l_identifiers := l_category_handler.identifiers
						Result := exception_or_necessary_tables (l_entity_id_table_saved, l_table_saved, l_identifiers)
						if Result = Void then
							-- everything present as requested
							if only_id_table then
								create Result.make (query_id, Void, Void, l_entity_id_table_saved)
							elseif only_table then
								create Result.make (query_id, Void, l_table_saved, Void)
							elseif only_identifiers then
								create Result.make (query_id, l_identifiers, Void, Void)
							else
								create Result.make (query_id, l_identifiers, l_table_saved, l_entity_id_table_saved)
							end
						end
					end
				end
			end
		rescue
			create Result.make (query_id, Void, Void, Void)
			Result.set_remote_exception (tag_name)
			l_retried := True
			retry
		end

	query_information: STRING is
			-- Information about this query
		do
			Result := "PES_QUERY_CATEGORY for context " + context_index.out + " and category " + category_id.out
		end

feature {NONE} -- Implementation


	exception_or_necessary_tables (a_entity_id_table_saved: HASH_TABLE [ENTITY_ID_BASE, INTEGER];
		a_table_saved: HASH_TABLE [ENTITY, INTEGER];
		a_identifiers: HASH_TABLE [ENTITY_IDENTIFIER, INTEGER]): PES_RESULT_LOAD_CATEGORY is
			-- Exception result if all necessary tables are not present
		do
			if a_entity_id_table_saved = Void and then not (only_identifiers or only_table) then
				create Result.make_exception (query_id, "entity_id_table not available on server (needs cache refresh)")
			elseif a_identifiers = Void and then not (only_table or only_id_table) then
				create Result.make_exception (query_id, "identifiers not available on server (needs cache refresh)")
			elseif a_table_saved = Void and then not (only_id_table or only_identifiers) then
				create Result.make_exception (query_id, "table not available on server (needs cache refresh)")
			end
		ensure
			exception_or_void: Result /= Void implies Result.remote_exception
		end

end
