note
	description: "Query object to send recipe to PES"
	author: "Ueli Peter"
	last_editor: "$Author: zs $"
	date: "$Date: 2011/07/11 $"
	revision: "$Revision: #9 $"
	path: "$File: //rose/source/kernel/entities/pes/pes_query_recipe.e $"

class
	PES_QUERY_RECIPE

inherit
	ROSE_CS_QUERY

create
	make, make_with_identifiers, make_with_all

feature {NONE} -- Creation

	make_from_recipe (a_entities: like entities; a_recipe: ROSE_RECIPE)
			-- create from recipe
		require
			valid_entities: a_entities /= Void
			valid_recipe: a_recipe /= Void
		do
			make (a_entities, a_recipe.representation)
		end

	make (a_entities: like entities; a_recipe_string: like recipe_string)
			-- Creation
		require
			valid_entities: a_entities /= Void
			valid_recipe: a_recipe_string /= Void
		do
			entities := a_entities
			recipe_string := a_recipe_string
		ensure
			entities_set: a_entities.is_equal (entities)
			recipe_set: a_recipe_string.is_equal (recipe_string)
		end

	make_with_identifiers (a_identifiers: like identifiers; a_recipe_string: like recipe_string)
			-- Create with a ROSE_RECIPE_INPUT object containing identifier strings
		require
			valid_identifiers: a_identifiers /= Void
			valid_recipe: a_recipe_string /= Void
		do
			identifiers := a_identifiers
			recipe_string := a_recipe_string
		end

	make_with_all (a_source_category: like source_category; a_recipe_string: like recipe_string)
			-- Create with a ROSE_RECIPE_INPUT object containing identifier strings
		require
			valid_category: a_source_category /= Void
			valid_recipe: a_recipe_string /= Void
		do
			use_all_as_source := True
			source_category := a_source_category
			recipe_string := a_recipe_string
		end

feature -- Access

	entities: ARRAY[ENTITY]

	identifiers: ROSE_RECIPE_INPUT

	recipe_string: STRING

	include_descriptive_name: BOOLEAN
		-- If this is true the descriptive_name attribute will
		-- be incuded in the result regardless of the settings in the recipe

	use_all_as_source: BOOLEAN
		-- If this flag is set, the recipe will use all entities of a given category as source

	source_category: STRING
		-- Source category in "all" mode

feature -- Basic operations

	set_include_descriptive_name (a_value: BOOLEAN)
			-- Set `include_descriptive_name' according to `a_value'
		do
			include_descriptive_name := a_value
		end

	execute_query (a_pes_server: PES_WORKER[PES_CONTEXT_AWARE]): PES_RESULT_RECIPE
			-- execute recipe
		local
			l_recipe: ROSE_RECIPE
			l_result: ROSE_TRANSFORMATION_RESULT
			l_retry_count: INTEGER
		do
			if not (l_retry_count > 1) then
				l_recipe := recipe_manager.recipe (recipe_string)

				if entities /= Void then
					l_result := l_recipe.transformed (entities)
				elseif identifiers /= Void then
					l_result := l_recipe.transformed_from_identifiers (identifiers)
				elseif use_all_as_source then
					l_result := l_recipe.transformed_from_all (source_category)
				end
				create Result.make (query_id, l_result, include_descriptive_name, l_recipe.settings.display.identifier_names)
			else
				create Result.make_exception (query_id, tag_name)
			end
		rescue
			-- should it always be the research context??
			if l_recipe /= Void then
				l_recipe.do_all (agent (a_transformation: ROSE_TRANSFORMATION) do a_transformation.source_category.context.reconnect;  a_transformation.target_category.context.reconnect end)
				if l_recipe.transformation /= Void and then l_recipe.transformation.target_category /= Void and then l_recipe.transformation.target_category.context /= Void then
					l_recipe.transformation.target_category.context.reconnect
				end
			end
			l_retry_count := l_retry_count + 1
			retry
		end

	query_information: STRING
			-- Returns informations about this query
		do
			Result := "PES_QUERY_RECIPE with recipe: " + recipe_string
		end

feature {NONE} -- Implementation

	recipe_manager: ROSE_RECIPE_MANAGER
			-- Recipe manager, used to parse recipies
		once
			create Result.make
		end

invariant

	either_entities_or_identifiers: use_all_as_source or (entities /= Void xor identifiers /= Void)
	all_implies_category_not_void: use_all_as_source implies source_category /= Void

end
