note

	description:

		"Worker Thread for PES, Caches a context"

	documentation: ""
	author: "Ueli Peter"
	last_editor: "$Author: er $"
	date: "$Date: 2012/11/05 $"
	revision: "$Revision: #24 $"
	path: "$File: //rose/source/kernel/entities/pes/pes_worker.e $"

class
	PES_WORKER[G->PES_CONTEXT_AWARE create make end]

inherit
	ROSE_CS_WORKER
		redefine
			refresh_cache,
			initialize_cache,
			server
		end

	PES_AWARE
		undefine
			default_create
		end

create
	make_cs

feature -- Access

	context_helper: G

	set_contexts (a_contexts: LIST[STRING]) is
			-- set the context lists from a list of context id`s
		local
			l_tuple: TUPLE[context: SUPPLIER_CONTEXT; logical, path: STRING; last_update_date: ROSE_DATE]
			l_date: ROSE_DATE
		do
			create contexts.make
			create l_date.make_now
			from
				a_contexts.start
			until
				a_contexts.after
			loop
				create l_tuple.default_create
				l_tuple.context := context_helper.context_from_name (a_contexts.item)
				if l_tuple.context /= Void then
					l_tuple.logical := l_tuple.context.server.logical.twin
					l_tuple.path := l_tuple.context.server.definition.full_path.twin
					l_tuple.last_update_date := l_date
					contexts.extend (l_tuple)
					if l_tuple.context = context_helper.research_context then
						research_context_item := l_tuple
					end
				else
					output(Global.interpreted("($now) " + "Context: " + a_contexts.item + " not found!%N"),Void)
				end
				a_contexts.forth
			end
		end

	initialize_cache is
			-- initialize cache
		do
			create context_helper.make
			set_contexts (server.contexts)
			initialize_context

			if needs_refresh then
				contexts.do_all (agent refresh_context_cache)
			end
		end

	initialize_context is
			-- load all categories of this context
		do
			contexts.do_all (agent (item: TUPLE[context: SUPPLIER_CONTEXT; logical, path: STRING; last_update_date: ROSE_DATE]) do item.context.set_pes_server(True) end)
			contexts.do_all (agent (item: TUPLE[context: SUPPLIER_CONTEXT; logical, path: STRING; last_update_date: ROSE_DATE])do  output(Global.interpreted("($now) " + item.context.name + " loaded!%N"),Void) end)
		end

	load_context (a_item: TUPLE[context: SUPPLIER_CONTEXT; logical, path: STRING; last_update_date: ROSE_DATE]) is
			-- load context into cache
		local
			l_file: RAW_FILE
			l_context: TUPLE[context: SUPPLIER_CONTEXT; last_update_date: ROSE_DATE]
			l_date: ROSE_DATE
		do
			-- Turn off GC
			-- Unfortunately we can't turn it off here, because if we do, the program will lock afterwards for several hours
			-- I hope this will be better in the next eiffel version
			--	memory.collection_off

			create l_file.make (a_item.context.name + ".sto")
			if not is_research (a_item.context) then
				--if l_file.exists then
				if false then
					output(Global.interpreted("($now) " + "Retrive Context from file: " + a_item.context.name + "%N"),Void)
					l_context ?= Global.persist.retrieved (l_file.name)
					output(Global.interpreted("($now) " + "Retrived Context from file: " + a_item.context.name + "%N"),Void)
					a_item.context.load_from_context (l_context.context)
					create l_date.make_now
					a_item.context.update_cache (l_context.last_update_date, l_date)
					a_item.last_update_date := l_date
				else
					create l_date.make_now
					create l_context.default_create
					output(Global.interpreted("($now) " + "Load Context: " + a_item.context.name + "%N"),Void)
					a_item.context.refresh_pes_server_cache_fast
					a_item.context.load_all_entities
					output(Global.interpreted("($now) " + "Loaded Context: " + a_item.context.name + "%N"),Void)
					l_context.context := a_item.context
					l_context.last_update_date := l_date
			--		output(Global.interpreted("($now) " + "Store Contextto file: " + a_item.context.name + "%N"),Void)
				--	Global.persist.store (l_context, l_file.name)
				--	output(Global.interpreted("($now) " + "Stored Context to file: " + a_item.context.name + "%N"),Void)
				end
			else
				output(Global.interpreted("($now) " + "Load Context: " + a_item.context.name + "%N"),Void)
				a_item.context.refresh_pes_server_cache_fast
				a_item.context.load_all_entities
				output(Global.interpreted("($now) " + "Loaded Context: " + a_item.context.name + "%N"),Void)
			end

			--	print_memory_statistics
			--	memory.collection_on
			--	memory.full_collect
			--	memory.full_coalesce
			---	memory.full_collect
			--	print_memory_statistics
			--	if not l_gc then
			--		memory.collection_off
			--	end
		end

	print_memory_statistics is
			-- Print memory statistics on standard output.
		local
			l_mem: MEMORY
 			l_mem_info: MEM_INFO
			l_full_gc_info, l_part_gc_info: GC_INFO
		do
			create l_mem
			l_mem.enable_time_accounting
			l_mem_info := l_mem.memory_statistics ({MEM_CONST}.eiffel_memory)
			print ("Total memory is " + l_mem_info.total.out + "%N")
			print ("Used memory is " + (l_mem_info.used + l_mem_info.overhead).out + "%N")
			print ("Free memory is " + l_mem_info.free.out + "%N")

			l_full_gc_info := l_mem.gc_statistics ({MEM_CONST}.full_collector)
			print ("GC full cycle is " + l_full_gc_info.cycle_count.out + "%N")
 			print ("GC full cycle is " + l_full_gc_info.cpu_time_average.out + "%N")

  			l_part_gc_info := l_mem.gc_statistics ({MEM_CONST}.incremental_collector)
			print ("GC incremental cycle is " + l_part_gc_info.cycle_count.out + "%N")
			print ("GC incremental cycle is " + l_part_gc_info.cpu_time_average.out + "%N")
--			print ("CPU time " + l_part_gc_info.cpu_total_time.out + "%N")
--			print ("Kernel time " + l_part_gc_info.sys_total_time.out + "%N")
			print ("Full Collection period " + l_mem.collection_period.out + "%N")
--			print ("GC percentage time " +
--                                (100 * (((l_full_gc_info.cycle_count * l_full_gc_info.cpu_time_average) +
--                                 (l_part_gc_info.cycle_count * l_part_gc_info.cpu_time_average)) /
--                                 l_part_gc_info.cpu_total_time)).out + "%N%N")
		end

	clear_context (a_context: SUPPLIER_CONTEXT) is
			-- remove all entites from cache
		do
			a_context.clear_cache
		end

	context_from_id (a_id: INTEGER): SUPPLIER_CONTEXT is
			-- return the context to this id
		do
			from
				contexts.start
			until
				contexts.after
			loop
				if a_id = contexts.item.context.context_index then
					Result := contexts.item.context
				end
				contexts.forth
			end
		end

feature -- Operations

	db_versions: LIST[TUPLE[logical, path: STRING]] is
			-- returns informations to the loaded contexts
		do
			create {ARRAYED_LIST[TUPLE[logical, path: STRING]]} Result.make (contexts.count + 1)
			from
				contexts.start
			until
				contexts.after
			loop
				Result.extend ([contexts.item.logical, contexts.item.path])
				contexts.forth
			end
		ensure
			result_not_void: Result /= Void
			result_count_correct: Result.count = contexts.count
			no_void_results: not Result.has (Void)
		end

	check_for_updates is
				-- check the freezer
		local
			l_date: ROSE_DATE
		do
			create l_date.make_now
			from
				contexts.start
			until
				contexts.after
			loop
				contexts.item.context.update_cache (contexts.item.last_update_date, l_date)
				contexts.item.last_update_date := l_date
				contexts.forth
			end
		end

feature {NONE} -- Implementation

	is_research (a_context: SUPPLIER_CONTEXT): BOOLEAN is
			-- check whether a_context is the research context
		require
			not_void: a_context /= Void
		do
			if a_context.name.is_equal ("RESEARCH") then
				Result := True
			end
		end


	refresh_context_cache (a_item: TUPLE[context: SUPPLIER_CONTEXT; logical, path: STRING; last_update_date: ROSE_DATE]) is
			-- load context into cache
		local
			l_last_update_date: ROSE_DATE
		do
			create l_last_update_date.make_now
			clear_context (a_item.context)
			load_context (a_item)
			set_pes_server_loaded (True)
			a_item.last_update_date := l_last_update_date
			a_item.path := a_item.context.server.definition.full_path.twin
			a_item.logical := a_item.context.server.definition.logical.twin
		end

	refresh_cache is
			-- implements feature from ROSE_CS
		do
			contexts.do_all (agent refresh_context_cache)
		end


	contexts: LINKED_LIST[TUPLE[context: SUPPLIER_CONTEXT; logical, path: STRING; last_update_date: ROSE_DATE]]

	research_context_item: TUPLE[context: SUPPLIER_CONTEXT; logical, path: STRING; last_update_date: ROSE_DATE]

	server: PES_SERVER[G]

end
