note

	description:

		"Shared process metrics context."

	author: "Zoran Simic"
	last_editor: "$Author: da $"
	date: "$Date: 2012/11/29 $"
	revision: "$Revision: #8 $"
	path: "$File: //rose/source/kernel/pm/pm_aware.e $"

class
	PM_AWARE

inherit
	ROSE_SHARED_GLOBAL
		export
			{NONE} all
		end
	ANY
feature -- Access

	frozen process_metrics: PM_CONTEXT is
			-- Process metrics context, use this to start/terminate instance runs and other more advanced functionality
		once
			create Result.make
			Result.refresh
		end

feature -- Basic operations

	frozen add_metric (a_name: STRING; a_value: DOUBLE) is
			-- Add a metric of type 'a_name' and 'a_value' to current process instance.
			-- If no metric type with 'a_name' exists, it is automatically created on commit.
		do
			process_metrics.add_metric (a_name,a_value)
		end

	frozen set_metric_test (a_name: STRING; a_test: FUNCTION[ANY,TUPLE[PM_PROCESS_PARTITION,PM_METRIC],PM_METRIC_TEST_RESULT]) is
			-- Add a metric test for 'a_name'
		do
			process_metrics.set_metric_test (a_name,a_test)
		end

	frozen run_metric_test (a_name: STRING; a_test: FUNCTION [ANY, TUPLE[PM_PROCESS_PARTITION, PM_METRIC], PM_METRIC_TEST_RESULT]) is
			-- Run a metric test for 'a_name'
		do
			process_metrics.run_metric_test (a_name, a_test)
		end

	frozen current_metric_value (a_name: STRING): DOUBLE is
			-- Current value of produced metric of type 'a_name', if any
		do
			Result := process_metrics.current_metric_value (a_name)
		end

	frozen is_current_run_duration_too_short (a_standard_deviation: DOUBLE; a_lookback_count: INTEGER): BOOLEAN
			-- Check whether current run's duration is too short given `a_standard_deviation' (use non-positive number), looking back `a_lookback_count' observations.
		do
			Result := process_metrics.is_current_run_duration_too_short (a_standard_deviation, a_lookback_count)
		end

	frozen is_current_run_duration_too_long (a_standard_deviation: DOUBLE; a_lookback_count: INTEGER): BOOLEAN
			-- Check whether current run's duration is too long given `a_standard_deviation', looking back `a_lookback_count' observations.
		do
			Result := process_metrics.is_current_run_duration_too_long (a_standard_deviation, a_lookback_count)
		end

	frozen current_run_duration_message: STRING
			-- Status message for the duration of current run.
		do
			Result := process_metrics.current_run_duration_message
		end

end
