note

	description:

		"Object that provides access to the metric types that are available in the process metrics system"

	author: "Zoran Simic"
	last_editor: "$Author: e2 $"
	date: "$Date: 2015/01/08 $"
	revision: "$Revision: #33 $"
	path: "$File: //rose/source/kernel/windows/pm_context_metrics.e $"

class
	PM_CONTEXT_METRICS

inherit

	MOVIE_AWARE
	DATABASE_SERVER
	PM_METRIC_TYPE_ROUTINES
	PM_METRIC_TEST_TYPES

create {PM_CONTEXT}
	make

feature {NONE} -- Initialization

	make (a_context: PM_CONTEXT) is
			-- Create the metrics object
		require
			valid_context: a_context /= Void
		do
			process_metrics := a_context
			create metrics.make (0)
			create metrics_tests.make (0)
		end

feature -- Access

	process_metrics: PM_CONTEXT

	metric (a_type: PM_METRIC_TYPE; a_instance: PM_INSTANCE): PM_METRIC is
			-- Metric of 'a_type', for 'a_instance'
		do
			create Result
			sql_metric.retrieve_one (Result,Void,<<a_type.metric_type_id,a_instance.instance_id>>)
		end

	metrics_cached (a_instance: PM_INSTANCE): STRING is
			-- Textual representation of all metrics currently in the cache (waiting to be commited)
		require
			valid_instance: a_instance /= Void
		local
			l_type: PM_METRIC_TYPE
			l_metric: PM_METRIC
			l_metric_tests: ROSE_LINEAR_ARRAY[FUNCTION[ANY,TUPLE[PM_PROCESS_PARTITION,PM_METRIC],PM_METRIC_TEST_RESULT]]
			l_metric_test: FUNCTION[ANY,TUPLE[PM_PROCESS_PARTITION,PM_METRIC],PM_METRIC_TEST_RESULT]
			l_test_result: PM_METRIC_TEST_RESULT
			l_msg:STRING
			l_normal_msgs,l_warning_msgs,l_failing_msgs,l_untest_msgs:ROSE_LINEAR_ARRAY[STRING]
		do
			create Result.make (1000)
			Result.append ("Process '")
			if a_instance = Void then
				Result.append ("-no instance-")
			elseif a_instance.process = Void then
				Result.append ("-no process-")
			elseif a_instance.process.name = Void or else a_instance.process.name.is_empty then
				Result.append ("-no process name-")
			else
				Result.append (a_instance.process.name)
			end
			Result.append ("':%N")
			if metrics = Void or else metrics.is_empty then
				Result.append ("-No metrics generated-%N")
			else
				from
					metrics.start
				until
					metrics.off
				loop
					l_metric := metrics.item_for_iteration
					l_type := l_metric.type
					l_msg:=""
					if l_type = Void then
						l_msg.append ("-no type-")
					elseif l_type.name = Void or else l_type.name.is_empty then
						l_msg.append ("-no name-")
					else
						l_msg.append (l_type.name)
					end
					l_msg.append (": ")
					l_msg.append (formatted_metric_value (l_metric.value))
					metrics_tests.search (metrics.key_for_iteration)
					if metrics_tests.found then
						l_metric_tests := metrics_tests.found_item
						if l_metric_tests /= Void then
							from l_metric_tests.start until l_metric_tests.off loop
								l_metric_test := l_metric_tests.current_item
								if l_metric_test /= Void then
									l_test_result := l_metric_test.item ([a_instance.partition,l_metric])
									if l_test_result /= Void then
										l_msg.append (": ")
										l_msg.append (l_test_result.message_string)
										if l_test_result.enable_process_failure then
											if l_failing_msgs = Void then
												create l_failing_msgs.make_empty
											end
											l_failing_msgs.extend (l_msg)
										else
											if l_warning_msgs = Void then
												create l_warning_msgs.make_empty
											end
											l_warning_msgs.extend (l_msg)
										end
									else
										if l_normal_msgs = Void then
											create l_normal_msgs.make_empty
										end
										l_normal_msgs.extend (l_msg)
									end
								end
								l_msg := ""
								if l_type = Void then
									l_msg.append ("-no type-")
								elseif l_type.name = Void or else l_type.name.is_empty then
									l_msg.append ("-no name-")
								else
									l_msg.append (l_type.name)
								end
								l_msg.append (": ")
								l_msg.append (formatted_metric_value (l_metric.value))
								l_metric_tests.forth
							end
						end
					else
						if l_untest_msgs=void then
							create l_untest_msgs.make_empty
						end
						l_untest_msgs.extend (l_msg)
					end
					metrics.forth
				end
				if l_failing_msgs /= Void then
					Result.append("%N======== failed for the following reasons ========%N")
					Result.append (ordered_text (l_failing_msgs))
				end
				if l_warning_msgs/=void then
					Result.append("%N======== warnings for further check ========%N")
					Result.append (ordered_text (l_warning_msgs))
				end
				if  l_normal_msgs/=void then
					result.append("%N======== passed test ========%N")
					result.append (ordered_text (l_normal_msgs))
				end
				if  l_untest_msgs/=void then
					result.append("%N======== process info (untested) ========%N")
					result.append (ordered_text (l_untest_msgs))
				end
			end
		end

	register_dynamic_tests (a_instance: PM_INSTANCE) is
			-- Register dynamic tests for `a_instance'.
		require
			a_instance_not_void: a_instance /= Void
		do
			if process_metrics_server.has_table ("metric_test") then
				existing_tests_predicate.retrieve (Void, <<a_instance.process.name, a_instance.partition_key, create {ROSE_DATE}.make_now_utc, create {ROSE_DATE}.make_now_utc>>)
				if not existing_tests_predicate.success then
					error.abort ("register_dynamic_tests", "existing_tests_predicate `retrieve' failed.", Void)
				end
				existing_tests_predicate.array.do_all (agent register_dynamic_test)
			else
				warn ("register_dynamic_tests", "Process Metrics database " + process_metrics_server.definition.full_path + " does not contain table `metric_test'. Please ensure this is added to enable dynamic tests."  ,Void)
			end
		end

	watched_metrics_tests: PM_WATCHED_METRICS_TESTS is
			-- Once access to the watched metrics tests.
		once
			create Result
		end

	register_dynamic_test (a_record: METRIC_TEST_RECORD) is
			-- Register dynamic test represented by `a_record'.
		require
			a_record_not_void: a_record /= Void
		local
			l_test: FUNCTION [ANY, TUPLE [PM_PROCESS_PARTITION, PM_METRIC], PM_METRIC_TEST_RESULT]
			l_range_item: like test_record_range_items
			l_range_lookback_item: like test_record_range_lookback_items
			l_value_item: like test_record_value_items
			l_condition: METRIC_TEST_CONDITION
		do
			l_condition := condition (a_record)
			if a_record.test_type ~ metric_test_compare_with_yesterday or
				a_record.test_type ~ "Compare Sum With Yesterday" -- obsolete type
			then
				l_range_item := test_record_range_items (a_record, agent lower_and_upper_greater_than_zero)
				if l_range_item /= Void then
					l_test := watched_metrics_tests.compare_with_yesterday_test_and_notify_failure (l_condition, a_record.enable_failure = 1, l_range_item.lower_value, l_range_item.upper_value, create {ROSE_DATE}.make_today, a_record)
				end
			elseif a_record.test_type ~ metric_test_bounds then
				l_range_item := test_record_range_items (a_record, agent upper_greater_than_lower)
				if l_range_item /= Void then
					l_test := watched_metrics_tests.bounds_test_and_notify_failure (l_condition, a_record.enable_failure = 1, l_range_item.lower_value, l_range_item.upper_value, a_record)
				end
			elseif a_record.test_type ~ metric_test_mean_standard_deviation then
				l_range_lookback_item := test_record_range_lookback_items (a_record, agent upper_greater_than_lower)
				if l_range_lookback_item /= Void then
					l_test := watched_metrics_tests.mean_standard_deviation_test_and_notify_failure (l_condition, a_record.enable_failure = 1, l_range_lookback_item.lower_value, l_range_lookback_item.upper_value, l_range_lookback_item.lookback, a_record)
				end
			elseif a_record.test_type ~ metric_test_mean_bounds then
				l_range_lookback_item := test_record_range_lookback_items (a_record, agent lower_and_upper_greater_than_zero)
				if l_range_lookback_item /= Void then
					l_test := watched_metrics_tests.mean_bounds_test_and_notify_failure (l_condition, a_record.enable_failure = 1, l_range_lookback_item.lower_value, l_range_lookback_item.upper_value, l_range_lookback_item.lookback, a_record)
				end
			elseif a_record.test_type ~ metric_test_floor then
				l_value_item := test_record_value_items (a_record)
				if l_value_item /= Void then
					l_test := watched_metrics_tests.floor_test_and_notify_failure (l_condition, a_record.enable_failure = 1, l_value_item.value, a_record)
				end
			elseif a_record.test_type ~ metric_test_ceiling then
				l_value_item := test_record_value_items (a_record)
				if l_value_item /= Void then
					l_test := watched_metrics_tests.ceiling_test_and_notify_failure (l_condition, a_record.enable_failure = 1, l_value_item.value, a_record)
				end
			elseif a_record.test_type ~ metric_test_non_zero then
				l_test := watched_metrics_tests.non_zero_test_and_notify_failure (l_condition, a_record.enable_failure = 1, a_record)
			else
				warn_invalid_test (a_record)
			end
			if l_test /= Void then
				set_metric_test (type_by_id (a_record.metric_type_id).name, l_test)
			else
				warn_invalid_test (a_record)
			end
		end

	warn_invalid_test (a_record: METRIC_TEST_RECORD) is
			-- Warn if `a_record' is not a supported test type.
		require
			a_record_not_void: a_record /= Void
		do
			warn ("register_dynamic_test", "Invalid/unsupported test record found. Name : $1, Test Name : $2, Test Type : $3, Settings : $4", <<type_by_id (a_record.metric_type_id).name, a_record.name, a_record.test_type, a_record.settings>>)
		end

	test_record_value_items (a_record: METRIC_TEST_RECORD): TUPLE [value: DOUBLE] is
			-- Return value from `settings' of `a_record'.
		require
			a_record_not_void: a_record /= Void
			a_record_is_value_type: a_record.test_type ~ metric_test_ceiling or a_record.test_type ~ metric_test_floor
		local
			l_delimited_text: ROSE_DELIMITED_TEXT
		do
			create l_delimited_text.make (a_record.settings, ';')
			if l_delimited_text.count = 1 then
				if l_delimited_text.item (1).is_double then
					Result := [l_delimited_text.item (1).to_double]
				end
			end
		end

	test_record_range_items (a_record: METRIC_TEST_RECORD; a_lower_upper_valid: FUNCTION [ANY, TUPLE [DOUBLE, DOUBLE], BOOLEAN]): TUPLE [lower_value: DOUBLE; upper_value: DOUBLE] is
			-- Return lower and upper values from `settings' of `a_record' if they are valid, based on execution of `a_lower_upper_valid'.
		require
			a_record_not_void: a_record /= Void
			a_lower_upper_valid_not_void: a_lower_upper_valid /= Void
			a_record_is_value_type: a_record.test_type ~ metric_test_bounds or
				a_record.test_type ~ metric_test_compare_with_yesterday
		local
			l_delimited_text: ROSE_DELIMITED_TEXT
			l_lower, l_upper: DOUBLE
		do
			create l_delimited_text.make (a_record.settings, ';')
			if l_delimited_text.count = 2 then
				if l_delimited_text.item (1).is_double and then l_delimited_text.item (2).is_double then
					l_lower := l_delimited_text.item (1).to_double
					l_upper := l_delimited_text.item (2).to_double
					if a_lower_upper_valid /= Void then
						a_lower_upper_valid.call ([l_lower, l_upper])
					end
					if a_lower_upper_valid = Void or else a_lower_upper_valid.last_result then
						Result := [l_lower, l_upper]
					end
				end
			end
		end

	test_record_range_lookback_items (a_record: METRIC_TEST_RECORD; a_lower_upper_valid: FUNCTION [ANY, TUPLE [DOUBLE, DOUBLE], BOOLEAN]): TUPLE [lower_value: DOUBLE; upper_value: DOUBLE; lookback: INTEGER] is
			-- Return lower, upper and lookback values from `settings' of `a_record' if they are valid, based on execution of `a_lower_upper_valid'.
		require
			a_record_not_void: a_record /= Void
			a_lower_upper_valid_not_void: a_lower_upper_valid /= Void
			a_record_is_value_type: a_record.test_type ~ metric_test_mean_bounds or a_record.test_type ~ metric_test_mean_standard_deviation
		local
			l_delimited_text: ROSE_DELIMITED_TEXT
			l_lower, l_upper: DOUBLE
		do
			create l_delimited_text.make (a_record.settings, ';')
			if l_delimited_text.count = 3 then
				if l_delimited_text.item (1).is_double and then l_delimited_text.item (2).is_double and then l_delimited_text.item (3).is_integer then
					l_lower := l_delimited_text.item (1).to_double
					l_upper := l_delimited_text.item (2).to_double
					if a_lower_upper_valid /= Void then
						a_lower_upper_valid.call ([l_lower, l_upper])
					end
					if a_lower_upper_valid = Void or else a_lower_upper_valid.last_result then
						Result := [l_lower, l_upper, l_delimited_text.item (3).to_integer]
					end
				end
			end
		end

	lower_and_upper_greater_than_zero (a_lower, a_upper: DOUBLE): BOOLEAN is
			-- Are `a_lower' and `a_upper' both greater than 0?
		do
			Result := a_lower > 0 and a_upper > 0
		end

	upper_greater_than_lower (a_lower, a_upper: DOUBLE): BOOLEAN is
			-- Is `a_upper' greater than `a_lower'?
		do
			Result := a_upper > a_lower
		end

	existing_tests_predicate: COLLECTING_PREDICATE [METRIC_TEST_RECORD] is
			-- Once access to predicate to return existing test records for a process.
		once
			create Result.make (process_metrics_server, "select * from metric_test where process = ? and partition = ? and start_date < ? and end_date > ?")
			if not Result.success then
				error.abort ("existing_tests_predicate", "Failed on `make'", Void)
			end
		ensure
			result_not_void: Result /= Void
		end

	current_run_test (a_instance: PM_INSTANCE): PM_METRIC_TEST_RESULT is
			-- Metric test result for 'a_instance'
		require
			valid_instance: a_instance /= Void
		local
			l_type: PM_METRIC_TYPE
			l_metric: PM_METRIC
			l_metric_tests: ROSE_LINEAR_ARRAY[FUNCTION[ANY,TUPLE[PM_PROCESS_PARTITION,PM_METRIC],PM_METRIC_TEST_RESULT]]
			l_metric_test: FUNCTION[ANY,TUPLE[PM_PROCESS_PARTITION,PM_METRIC],PM_METRIC_TEST_RESULT]
			l_test_result: PM_METRIC_TEST_RESULT
		do
			if metrics /= Void and then not metrics.is_empty then
				from
					metrics.start
				until
					metrics.off
				loop
					l_metric := metrics.item_for_iteration
					l_type := l_metric.type
					metrics_tests.search (metrics.key_for_iteration)
					if metrics_tests.found then
						l_metric_tests := metrics_tests.found_item
						if l_metric_tests /= Void then
							from l_metric_tests.start until l_metric_tests.off loop
								l_metric_test := l_metric_tests.current_item
								if l_metric_test /= Void then
									l_test_result := l_metric_test.item([a_instance.partition,l_metric])
									if l_test_result /= Void then
										if Result = Void then
											create Result.make ("Metric Test Failure")
										end
										Result.set_enable_process_failure (l_test_result.enable_process_failure or Result.enable_process_failure)
										if l_test_result.enable_process_failure then
											sql_insert_metric_test.execute (Void,<<l_metric.type.metric_type_id, a_instance.current_run.run_id, l_test_result.message_string, 3>>)
										else
											sql_insert_metric_test.execute (Void,<<l_metric.type.metric_type_id, a_instance.current_run.run_id, l_test_result.message_string, 2>>)
										end
									end
								end
								l_metric_tests.forth
							end
						end
					end
					metrics.forth
				end
			end
		end

	current_metric_value (a_name: STRING): DOUBLE is
			-- Current value of produced metric of type 'a_name'
		local
			l_type: PM_METRIC_TYPE
			l_metric: PM_METRIC
		do
			l_type := type (a_name)
			if l_type /= Void then
				l_metric := metrics.item (l_type.metric_type_id)
				if l_metric /= Void then
					Result := l_metric.value
				end
			end
		end


	is_register_all_databases_enabled : BOOLEAN
			-- Tell put_registered_metrics to register all databases that were accessed	

feature -- Status report

	is_valid_type (a_name: STRING): BOOLEAN is
			-- Does 'a_name' correspond to a valid metric type?
		do
			Result := type (a_name) /= Void
		end

feature -- Basic operations

	reset is
			-- Clear all accumulated metrics so far
		do
			metrics.wipe_out
			metrics_tests.wipe_out
			if registered_sources /= Void then
				from
					registered_sources.start
				until
					registered_sources.off
				loop
					registered_sources.current_item.reset_metrics
					registered_sources.forth
				end
				registered_sources := Void
			end
		end

	refresh is
			-- Refresh this process_metrics.
		do
			type_names.wipe_out
			types.wipe_out
		end

	add_metric (a_type: STRING; a_value: DOUBLE) is
			-- Add 'a_value' to metric of 'a_type'.
		require
			valid_type: is_valid_type (a_type)
		local
			l_type: PM_METRIC_TYPE
			l_metric: PM_METRIC
		do
			l_type := type (a_type)
			l_metric := metrics.item (l_type.metric_type_id)
			if l_metric = Void then
				create l_metric.make (l_type)
				metrics.put (l_metric,l_type.metric_type_id)
			end
			l_metric.add_value (a_value)
		end

	set_metric_test (a_type: STRING; a_test: FUNCTION[ANY,TUPLE[PM_PROCESS_PARTITION,PM_METRIC],PM_METRIC_TEST_RESULT]) is
			-- Add 'a_test' to metric of 'a_type'.
		require
			valid_type: is_valid_type (a_type)
		local
			l_type: PM_METRIC_TYPE
			l_metric_tests: ROSE_LINEAR_ARRAY[FUNCTION[ANY,TUPLE[PM_PROCESS_PARTITION,PM_METRIC],PM_METRIC_TEST_RESULT]]
		do
			l_type := type (a_type)
			metrics_tests.search (l_type.metric_type_id)
			if not metrics_tests.found then
				create l_metric_tests.make_empty
				metrics_tests.force (l_metric_tests,l_type.metric_type_id)
			else
				l_metric_tests := metrics_tests.found_item
			end
			l_metric_tests.extend (a_test)
		end

	run_metric_test (a_type: STRING; a_test: FUNCTION [ANY, TUPLE [PM_PROCESS_PARTITION, PM_METRIC], PM_METRIC_TEST_RESULT]) is
			-- Run 'a_test' metric of 'a_type'.
		require
			valid_type: is_valid_type (a_type)
		local
			l_type: PM_METRIC_TYPE
			l_metric: PM_METRIC
			l_test_result: PM_METRIC_TEST_RESULT
			l_instance: PM_INSTANCE
			l_exceptions: EXCEPTIONS
		do
			l_instance := process_metrics.current_instance
			if l_instance /= Void then
				l_type := type (a_type)
				if l_type /= Void then
					l_metric := metrics [l_type.metric_type_id]
					if l_metric /= Void then
						l_test_result := a_test.item ([l_instance.partition, l_metric])
						if l_test_result /= Void then
							if l_test_result.enable_process_failure then
								sql_insert_metric_test.execute (Void, <<l_metric.type.metric_type_id, l_instance.current_run.run_id, l_test_result.message_string, 3>>)
								process_metrics.terminate
								create l_exceptions
								l_exceptions.die (-1)
							else
								sql_insert_metric_test.execute (Void, <<l_metric.type.metric_type_id, l_instance.current_run.run_id, l_test_result.message_string, 2>>)
							end
						end
					end
				end
			end
		end

	register_source (a_source: PM_METRIC_SOURCE) is
			-- Register all metric types from 'a_source' and it's co-sources
			-- If 'a_source' is already registered or is Void, does nothing
		do
			if a_source /= Void and then not a_source.registered then
				a_source.set_registered
				if registered_sources = Void then
					create registered_sources.make_empty
				end
				registered_sources.extend (a_source)
				if a_source.co_sources /= Void then
					from
						a_source.co_sources.start
					until
						a_source.co_sources.off
					loop
						register_source (a_source.co_sources.current_item)
						a_source.co_sources.forth
					end
				end
			end
		end

	add_registered_metrics is
			-- Add all metrics from registered sources to current metric cache.
		obsolete
			"Use put_registered_metrics instead."
		local
			l_metrics: LINEAR [PAIR [STRING,DOUBLE]]
		do
			from
				registered_sources.start
			until
				registered_sources.off
			loop
				l_metrics := registered_sources.current_item.resettable_metrics
				if l_metrics /= Void then
					from
						l_metrics.start
					until
						l_metrics.off
					loop
						add_metric (l_metrics.item.first,l_metrics.item.second)
						l_metrics.forth
					end
				end
				registered_sources.current_item.reset_metrics
				registered_sources.forth
			end
		end

	enable_register_all_databases is
			-- Tell put_registered_metrics to register all databases in ROSE_DATABASE_POOL.accessed
		do
			is_register_all_databases_enabled := True
		end

	disable_register_all_databases is
		-- Tell put_registered_metrics NOT to register all databases in ROSE_DATABASE_POOL.accessed
		do
			is_register_all_databases_enabled := False
		end

feature {PM_INSTANCE_RUN} -- Implementation

	write_to_database is
			-- Write gathered metrics to database.
		require
			process_metrics_is_running: process_metrics.is_running
		do
			put_system_metrics
			put_registered_metrics
			write_to_database_for_run (process_metrics.current_instance.current_run.run_id)
		end

feature {PM_CONTEXT} -- Implementation

	write_to_database_for_run (a_run_id: INTEGER) is
			-- Write gathered metrics to database for 'a_run_id'
		local
			l_metric: PM_METRIC
		do
			if not metrics.is_empty then
				from
					metrics.start
				until
					metrics.off
				loop
					l_metric := metrics.item_for_iteration
					sql_insert_metric.execute (Void,<<l_metric.type.metric_type_id, a_run_id, l_metric.value>>)
					metrics.forth
				end
				process_metrics.server.commit_if_necessary
			end
		end

feature {TS_TEST_CASE} -- Implementation

	metrics: HASH_TABLE [PM_METRIC,INTEGER]
			-- Current metrics counters, these are the metrics waiting to be inserted in database

	metrics_tests: HASH_TABLE [ROSE_LINEAR_ARRAY[FUNCTION[ANY,TUPLE[PM_PROCESS_PARTITION,PM_METRIC],PM_METRIC_TEST_RESULT]],INTEGER]
			-- Tests for metric value validity

	registered_sources: ROSE_LINEAR_ARRAY [PM_METRIC_SOURCE]

	put_system_metrics is
			-- A set of metrics that we always record.
		local
			l_handle: INTEGER
			l_result: BOOLEAN
			l_wbs: WINBASE_SERVICES
			l_memory_counter: PROCESS_MEMORY_COUNTERS
			l_memory_counter_32: PROCESS_MEMORY_COUNTERS_32
			l_creation_time, l_exit_time, l_kernel_time, l_user_time: WINDOWS_FILETIME
		do
			if is_recording_movie or is_playing_movie then
				do_nothing
			else
				create l_wbs
				l_handle := l_wbs.getcurrentprocess
				if global.env.is_running_64_bit then
					create l_memory_counter
					l_memory_counter.allocate
					l_result := l_wbs.getprocessmemoryinfo (l_handle, l_memory_counter.pointer_, l_memory_counter.record_length_)
					if l_result then
						l_memory_counter.read
					end
				else
					create l_memory_counter_32
					l_memory_counter_32.allocate
					l_result := l_wbs.getprocessmemoryinfo (l_handle, l_memory_counter_32.pointer_, l_memory_counter_32.record_length_)
					if l_result then
						l_memory_counter_32.read
						create l_memory_counter.make_from_counter_32(l_memory_counter_32)
						l_memory_counter_32 := Void
					else
						create l_memory_counter
					end
				end
				add_metric("System/PageFaultCount", l_memory_counter.pagefaultcount)
				add_metric("System/PeakPagefileUsage (MB)", l_memory_counter.peakpagefileusage / 1_000_000)
				add_metric("System/PeakWorkingSetSize (MB)", l_memory_counter.peakworkingsetsize / 1_000_000)
				create l_creation_time
				l_creation_time.allocate
				create l_exit_time
				l_exit_time.allocate
				create l_kernel_time
				l_kernel_time.allocate
				create l_user_time
				l_user_time.allocate
				l_result := l_wbs.getprocesstimes (l_handle, l_creation_time.pointer_, l_exit_time.pointer_, l_kernel_time.pointer_, l_user_time.pointer_)
				if l_result then
					l_creation_time.read
					l_exit_time.read
					l_kernel_time.read
					l_user_time.read
					add_metric("System/KernelTime", l_kernel_time.fine_seconds)
					add_metric("System/UserTime", l_user_time.fine_seconds)
				end
				l_result := l_wbs.closehandle (l_handle)
			end
		end

	put_registered_metrics is
			-- Two different approaches are supported :
			-- 1. SELECTED databases can be added to registered_sources using PM_CONTEXT.register_source
			-- 2. ALL databases can be registered using PM_CONTEXT.register_all_databases
			-- This feature will add the metrics from 1 and 2 to the `metrics' hash table.
		local
			l_accessed: ROSE_LINEAR_ARRAY [ROSE_DATABASE_DEFINITION]
		do
			if registered_sources /= Void then
				registered_sources.do_all (agent add_metric_from_metric_source)
			end

			if is_register_all_databases_enabled then
				l_accessed := servers.pool.accessed
				if l_accessed /= Void then
					l_accessed.do_all (agent add_metric_from_database_definition)
				end
			end
		end

	condition (a_record: METRIC_TEST_RECORD): METRIC_TEST_CONDITION
			-- Condition to test before we run any PM test
		require
			a_record_not_void: a_record /= Void
		do
			if a_record.condition.is_integer and then a_record.condition.to_integer > 0 then
				create {METRIC_TEST_CONDITION_AT_LEAST} Result.make (a_record.condition.to_integer)
			elseif a_record.condition.starts_with (condition_exactly_prefix) and then
				a_record.condition.substring (condition_exactly_prefix.count +1, a_record.condition.count).is_integer and then
				a_record.condition.substring (condition_exactly_prefix.count +1, a_record.condition.count).to_integer > 0
			then
				create {METRIC_TEST_CONDITION_EXACTLY} Result.make (a_record.condition.substring (condition_exactly_prefix.count +1, a_record.condition.count).to_integer)
			else
				create {METRIC_TEST_CONDITION_TRUE} Result.make
			end
		ensure
			condition_not_void: Result /= Void
		end

feature {NONE} -- Predicates

	sql_run_metric: COLLECTING_PREDICATE [PM_METRIC] is
			-- Retrieves all metrics associated to a run_id
		once
			create Result.make_strict (process_metrics.server,"SELECT * FROM metric WITH (NOLOCK) WHERE run_id = ?")
		end

	sql_metric: COLLECTING_PREDICATE [PM_METRIC] is
			-- Retrieves a particular metric, for a given metric_type_id and run_id
		once
			create Result.make_strict (process_metrics.server,
									   "SELECT * FROM metric WITH (NOLOCK) WHERE metric_type_id = ? AND run_id = ?")
		end

	sql_insert_metric_test: EXECUTION_PREDICATE is
			-- Inserts a metric test result.
		once
			create Result.make (process_metrics.server,"INSERT INTO metric_test_result (metric_type_id,run_id,message,severity) VALUES (?,?,?,?)")
		end

feature {NONE} -- Helpers

	formatted_metric_value (a_value: DOUBLE): STRING is
			-- A formatted version of `a_value' metric value.
		do
			if ieee_math.isnan (a_value) then
				Result := "%T    <null>"
			elseif a_value >= 1_000.0 or a_value <= -1_000.0 then
				Result := "%T" + double_formatter (30,3).formatted (a_value)
			else
				Result := "%T" + double_formatter (10,5).formatted (a_value)
			end
		end

	double_formatter (a_width, a_decimal: INTEGER): FORMAT_DOUBLE is
			-- Formatter for double values with comma as separator
		do
			create Result.make (a_width, a_decimal)
			Result.set_separator (',')
		ensure
			double_formatter_not_void: Result /= Void
		end

	ieee_math: IEEE_MATH is
			-- IEEE math routines
		once
			create Result
		end

feature {NONE} -- Sorting agents

	string_order (a,b: STRING): BOOLEAN is
			-- Is a less than b?
		do
			Result := a < b
		end

	ordered_text (a_texts: ROSE_LINEAR_ARRAY[STRING]): STRING is
			-- A string representing an ordered version of a_texts
		local
			l_texts: ROSE_LINEAR_ARRAY[STRING]
		do
			l_texts := a_texts.twin
			l_texts.sort_by (agent string_order(?,?))
			Result := ""
			from l_texts.start until l_texts.off or l_texts.index > 500 loop
 				Result := Result + l_texts.current_item + "%N"
				l_texts.forth
			end
		end

feature {NONE} -- Agents to add metrics from different types

	add_metric_from_pair ( a_pair : PAIR [STRING,DOUBLE] ) is
			-- Add `a_pair' to the `metrics' hash table
		require
			a_pair_not_vode : a_pair /= Void
		do
			add_metric (a_pair.first, a_pair.second)
		end

	add_metric_from_metric_source ( a_metric_source : PM_METRIC_SOURCE ) is
			-- Add all metrics in `a_metric_source' to the `metrics' hash table
		require
			a_metric_source_not_void : a_metric_source /= Void
		local
			l_metrics : LINEAR [PAIR [STRING, DOUBLE]]
		do
			l_metrics := a_metric_source.metrics
			if l_metrics /= Void then
				l_metrics.do_all ( agent add_metric_from_pair)
			end
		end

	add_metric_from_database_definition ( a_definition : ROSE_DATABASE_DEFINITION ) is
			-- Lookup the sql_data_server from `a_definition'
			-- Add all metrics in `l_server' to the `metrics' hash table
		require
			a_definition_not_void : a_definition /= Void
		local
			l_server: SQL_DATA_SERVER
		do
			l_server := servers.shared_server_from_repository ( a_definition.logical)
			if l_server /= Void then
				add_metric_from_metric_source( l_server )
			end
		end

end
