note
	description: "Provides read access to the instance_run table in process_metrics."
	author: "Mauricio Bustos"
	last_editor: "$Author: rj $"
	date: "$Date: 2012/02/14 $"
	revision: "$Revision: #3 $"
	path: "$File: //rose/source/kernel/pm/pm_context_runs.e $"

class
	PM_CONTEXT_RUNS

create {PM_CONTEXT}
	make

feature {NONE} -- Initialization

	make (a_context: PM_CONTEXT) is
		do
			process_metrics := a_context
			create runs.make (1000, 100)
		end

feature -- Access

	runs: ROSE_VOLATILE_CACHE [PM_INSTANCE_RUN, INTEGER]
			-- Instance cache, contains last fetched objects

	run_by_id (a_id: INTEGER): PM_INSTANCE_RUN is
			-- Instance with 'a_id' if any
		do
			Result := runs.item (a_id)
			if Result = Void and then process_metrics.is_available then
				sql_select_run_by_id.retrieve (Void, << a_id >>)
				if sql_select_run_by_id.array /= Void and then sql_select_run_by_id.array.count > 0 then
					Result := sql_select_run_by_id.array.item (1)
					cache_run (Result)
				end
			end
		end

feature {NONE} -- Implementation

	process_metrics: PM_CONTEXT			-- This is a reference back to our parent.

	cache_run (a_run: PM_INSTANCE_RUN) is
		do
			if a_run /= Void then
				runs.put (a_run, a_run.run_id)
			end
		end

	sql_select_run_by_id_saved: like sql_select_run_by_id
			-- See 'sql_select_run_by_id'.
		note
			option: transient
		attribute
		end

	sql_select_run_by_id: COLLECTING_PREDICATE [PM_INSTANCE_RUN]
		do
			if sql_select_run_by_id_saved=Void
			then
				create sql_select_run_by_id_saved.make_strict (process_metrics.server, "SELECT * FROM instance_run WITH(NOLOCK) WHERE run_id=?")
			end

			Result := sql_select_run_by_id_saved
		end

end
