note

	description:

		"Instance Collection"

	author: "Zoran Simic"
	last_editor: "$Author: mb $"
	date: "$Date: 2011/05/10 $"
	revision: "$Revision: #8 $"
	path: "$File: //rose/source/kernel/pm/pm_instance_collection.e $"

class PM_INSTANCE_COLLECTION

inherit

	PM_COLLECTION [PM_INSTANCE]

create

	make_for_partition,
	make_latest

feature {NONE} -- Initialization

	make_for_partition (a_partition: PM_PROCESS_PARTITION) is
			-- make from partition
		require
			partition_not_void: a_partition /= Void
		do
			get_items (sql_partition_instances, <<a_partition.process.process_id, a_partition.partition_key>>)
		end

	make_latest (a_partition: PM_PROCESS_PARTITION) is
			-- make from partition but only get latest instance
		require
			partition_not_void: a_partition /= Void
		do
			get_items (sql_latest_partition_instance, <<a_partition.process.process_id, a_partition.partition_key>>)
		end

feature -- Access

	instances_at_file_date (a_date: ROSE_DATE): ROSE_LINEAR_ARRAY [like item] is
			-- instances at file date
		local
			i: INTEGER
		do
			create Result.make_empty
			from
				i := lower
			until
				i > upper
			loop
				if item (i).file_date.is_equal (a_date) then
					Result.extend (item (i))
				end
				i := i + 1
			end
		end

	instances_within_file_date (a_date: ROSE_DATE): ROSE_LINEAR_ARRAY [like item] is
			-- instances at file date regardless of the time
		local
			i: INTEGER
		do
			create Result.make_empty
			from
				i := lower
			until
				i > upper
			loop
				if item (i).file_date.same_date (a_date) then
					Result.extend (item (i))
				end
				i := i + 1
			end
		end

	metrics_in_interval (a_metric_type_id: INTEGER; a_start_date, a_end_date: ROSE_DATE): ROSE_TIME_SERIES [PM_METRIC] is
			-- metrics in interval
		require
			valid_interval: a_start_date /= Void and a_end_date /= Void
		local
			i: INTEGER
			l_file_date: ROSE_DATE
			l_metric: PM_METRIC
		do
			create Result.make_empty
			from
				i := lower
			until
				i > upper
			loop
				l_file_date := item (i).file_date
				if l_file_date /= Void and then (l_file_date >= a_start_date and l_file_date <= a_end_date) then
					l_metric := item (i).metrics.metric (a_metric_type_id)
					if l_metric /= Void then
						Result.extend (l_metric, l_file_date)
					end
				end
				i := i + 1
			end
		end

feature {NONE} -- Implementation

	sql_partition_instances: COLLECTING_PREDICATE [PM_INSTANCE] is
			-- partition instances
		once
			create Result.make_strict (process_metrics.server, "SELECT * FROM instance WITH(NOLOCK) WHERE process_id=? AND partition_key=? order by file_date")
		ensure
			valid_result: Result /= Void
		end

	sql_latest_partition_instance: COLLECTING_PREDICATE [PM_INSTANCE] is
			-- partition instances
		once
			create Result.make_strict (process_metrics.server, "SELECT top 1 * FROM instance WITH(NOLOCK) WHERE process_id=? AND partition_key=? order by instance_id desc")
		ensure
			valid_result: Result /= Void
		end

end
