note

	description:

		"Definition of a metric"

	author: "Zoran Simic"
	last_editor: "$Author: c4 $"
	date: "$Date: 2012/06/11 $"
	revision: "$Revision: #3 $"
	path: "$File: //rose/source/kernel/infrastructure/pm/pm_metric.e $"

class PM_METRIC

inherit

	PM_METRIC_TYPE_ROUTINES
		rename
			type as type_for_name
		redefine
			out
		end

create

	default_create, make

feature {NONE} -- Initialization

	make (a_type: PM_METRIC_TYPE) is
			-- Create a metric.
		require
			a_type_not_void: a_type /= Void
		do
			metric_type_id := a_type.metric_type_id
			saved_type := a_type
		end

feature -- Access

	metric_type_id: INTEGER
			-- Unique identifier

	run_id: INTEGER
			-- Internal ids identifying this metric
			-- 'run_id' is 0 for aggregated metrics

	count: INTEGER
			-- Number of metrics aggregated (> 0 only for aggregated metrics)

	value: DOUBLE
			-- Metric value

	type: PM_METRIC_TYPE is
			-- Type of this metric
		do
			if saved_type = Void then
				saved_type := type_by_id (metric_type_id)
			end
			Result := saved_type
			if Result = Void then
				error.warn ("Unknown Process-metrics metric_type_id $1", <<metric_type_id>>)
				error.flush
			end
		end

	out: STRING is
			-- New string containing terse printable representation of current object
		do
			create Result.make (10)
			Result.append (value.out)
			Result.append (" ")
			if type /= Void then
				Result.append (type.out)
			else
				Result.append ("?")
			end
		end

feature -- Basic operations

	set_value (a_value: DOUBLE) is
			-- Set 'value'.
		do
			value := a_value
		end

	add_value (a_value: DOUBLE) is
			-- Add 'a_value' to this metric
		do
			value := value + a_value
		end

	aggregate_value (a_value: DOUBLE) is
			-- Add 'a_value' to this metric
		do
			value := value + a_value
			count := count + 1
		end

	aggregate (a_other: PM_METRIC) is
			-- Add `a_value' to this metric
		require
			same_typed_id: a_other /= Void and then metric_type_id = a_other.metric_type_id
		do
			aggregate_value (a_other.value)
		end

	write_to_database (a_run_id: INTEGER) is
			-- Record `Current' in DB for instance-run `a_run_id'.
			-- Used by kernel/infrastructure for recording SQL metrics.
		require
			a_run_id_non_zero: a_run_id /= 0
		do
			sql_insert_metric.execute (Void, <<type.metric_type_id, a_run_id, value>>)
		end

feature {NONE} -- Implementation

	saved_type: PM_METRIC_TYPE
			-- Cache for `type'

end
