indexing
	description: "Collection of metrics"
	author: "Zoran Simic"
	last_editor: "$Author: mb $"
	date: "$Date: 2009/08/20 $"
	revision: "$Revision: #6 $"
	path: "$File: //rose/source/kernel/pm/pm_metric_collection.e $"

class
	PM_METRIC_COLLECTION

inherit
	PM_COLLECTION [PM_METRIC]
		rename
			add_metric as generic_add_metric
		redefine
			get_items
		end

create
	make_for_process, make_for_partition, make_for_instance, make_for_instance_run

feature {NONE} -- Initialization

	make_for_process (a_process: PM_PROCESS; a_start_date: ROSE_DATE) is
		require
			a_process /= Void
		do
			if a_start_date /= Void then
				get_items (sql_process_metrics, <<a_start_date, a_process.process_id>>)
			else
				get_items (sql_process_metrics, <<min_date, a_process.process_id>>)
			end				
		end

	make_for_partition (a_partition: PM_PROCESS_PARTITION) is
		require
			a_partition /= Void
		do
			get_items (sql_partition_metrics,<<a_partition.process.process_id,a_partition.partition_key>>)
		end

	make_for_instance (a_instance: PM_INSTANCE) is
		require
			a_instance /= Void
		do
			if process_metrics.process_ids.item(a_instance.process.process_id).concurrency = 1 then
				get_items (sql_last_instance_metrics,<<a_instance.instance_id>>)
			else
				get_items (sql_instance_metrics,<<a_instance.instance_id>>)
			end
		end

	make_for_instance_run (a_run: PM_INSTANCE_RUN) is
		require
			a_run /= Void
		do
			get_items (sql_instance_run_metrics,<<a_run.run_id>>)
		end

feature -- Access

	hash: HASH_TABLE [PM_METRIC,INTEGER]
			-- Metrics in this collection, hashed by their metric_type_id

	types: ROSE_LINEAR_ARRAY [PM_METRIC_TYPE] is
			-- Types contained in this collecion
		local
			i: INTEGER
		do
			create Result.make (1,hash.count)
			from
				i := 1
				hash.start
			until
				hash.off
			loop
				Result.put (hash.item_for_iteration.type,i)
				i := i + 1
				hash.forth
			end
		end

	metric_by_name (a_name: STRING): PM_METRIC is
			-- Metric corresponding to metric type with 'a_name'
		local
			l_type: PM_METRIC_TYPE
		do
			l_type := process_metrics.metrics.type (a_name)
			if l_type /= Void then
				Result := hash.item (l_type.metric_type_id)
			end
		end

	metric (a_id: INTEGER):  PM_METRIC is
			-- Metric corresponding to metric type with 'a_id'
		do
			Result := hash.item (a_id)
		end

feature -- Basic operations

	add_metric_object (a_metric: PM_METRIC) is
		local
			l_metric: PM_METRIC
		do
			if a_metric /= Void then
				l_metric := hash.item (a_metric.metric_type_id)
				if l_metric /= Void then
					l_metric.aggregate (a_metric)
				elseif a_metric.run_id = 0 then
					hash.put (a_metric,a_metric.metric_type_id)
				else
					l_metric := a_metric.twin
					hash.put (l_metric,l_metric.metric_type_id)
				end
			end
		end

	add_collection (a_other: PM_METRIC_COLLECTION) is
		do
			if a_other /= Void then
				from
					a_other.hash.start
				until
					a_other.hash.off
				loop
					add_metric_object (a_other.hash.item_for_iteration)
					a_other.hash.forth
				end
			end
		end

	add_array (a_array: ARRAY [PM_METRIC]) is
		local
			i: INTEGER
		do
			if a_array /= Void then
				from
					i := a_array.lower
				until
					i > a_array.upper
				loop
					add_metric_object (a_array.item (i))
					i := i + 1
				end
			end
		end

feature {NONE} -- Implementation

	get_items (a_predicate: COLLECTING_PREDICATE [PM_METRIC]; a_args: ARRAY [ANY]) is
		local
			i: INTEGER
		do
			create hash.make (0)
			Precursor (a_predicate,a_args)
			from
				i := items.lower
			until
				i > items.upper
			loop
				hash.put (items.item (i),items.item (i).metric_type_id)
				i := i + 1
			end
		end

	sql_instance_run_metrics: COLLECTING_PREDICATE [PM_METRIC] is
			-- Retrieves a particular metric, for a given metric_type_id and run_id
		once
			create Result.make_strict (process_metrics.server,
				"SELECT * FROM metric WITH(NOLOCK) WHERE run_id=?")
		end

	sql_last_instance_metrics : COLLECTING_PREDICATE [PM_METRIC] is
			-- Retrieves a particular metric, for a given metric_type_id and run_id
		once
			create Result.make_strict (process_metrics.server,
									   "[
		SELECT     a.metric_type_id, a.[value], 1 as [count]
			FROM         metric a INNER JOIN
										instance_run b ON a.run_id = b.run_id
										WHERE     (b.instance_id = ?) AND (a.run_id IN
										(SELECT     MAX(run_id)
				FROM       instance_run
										WHERE      instance_id = b.instance_id))
										]"
									   )
				end
				
				
				sql_instance_metrics: COLLECTING_PREDICATE [PM_METRIC] is
			-- Retrieves a particular metric, for a given metric_type_id and run_id
		once
			create Result.make_strict (process_metrics.server,
									   "[
		SELECT metric.metric_type_id AS metric_type_id, SUM(metric.[value]) AS value, COUNT(*) AS [count]
			FROM instance WITH(NOLOCK) INNER JOIN
										instance_run ON instance.instance_id = instance_run.instance_id INNER JOIN
										metric ON instance_run.run_id = metric.run_id
										GROUP BY metric.metric_type_id, instance.instance_id
										HAVING (instance.instance_id=?)
										]"
									   )
			end
			
			sql_partition_metrics: COLLECTING_PREDICATE [PM_METRIC] is
			-- Retrieves a particular metric, for a given metric_type_id and run_id
		once
			create Result.make_strict (process_metrics.server,
									   "[
		SELECT metric.metric_type_id AS metric_type_id, SUM(metric.[value]) AS value, COUNT(*) AS [count]
			FROM instance WITH(NOLOCK) INNER JOIN
										instance_run ON instance.instance_id = instance_run.instance_id INNER JOIN
										metric ON instance_run.run_id = metric.run_id
										GROUP BY instance.process_id, instance.partition_key, metric.metric_type_id
										HAVING (instance.process_id=?) AND (instance.partition_key=?)
										]"
									   )
			end
			
			sql_process_metrics: COLLECTING_PREDICATE [PM_METRIC] is
			-- Retrieves a particular metric, for a given metric_type_id and run_id
		once
			create Result.make_strict (process_metrics.server,
									   "[
		SELECT metric.metric_type_id AS metric_type_id, SUM(metric.[value]) AS value, COUNT(*) AS [count]
			FROM instance WITH(NOLOCK) INNER JOIN
										instance_run ON instance.instance_id = instance_run.instance_id INNER JOIN
										metric ON instance_run.run_id = metric.run_id
										WHERE instance_run.start_time > ? and instance.process_id = ?
										GROUP BY instance.process_id, metric.metric_type_id
										]"
									   )
			end
			
			min_date: ROSE_DATE is 
			-- Default query start date
		once
			create Result.make_ccyymmdd (19000101)
		ensure
			valid_result: Result /= Void
		end
	
end














