indexing
	description: "Objects that can generate metrics."
	author: "RJ"
	date: "$Date: 2008/12/01 $"
	revision: "$Revision: #2 $"

class
	PM_METRIC_SOURCE

feature -- Access

	metrics: LINEAR [PAIR [STRING, DOUBLE]] is
			-- Metrics which grow and never reset to 0.
			-- The implementation should set 'metrics_called' to True.
		do
		end

	resettable_metrics: LINEAR[PAIR[STRING, DOUBLE]] is
			-- Metric which can be reset to 0.
			-- The implementation should set 'metrics_called' to True.
		do
		end

	full_name(a_name: STRING): STRING is
			-- A utility routine that can be used by 'metrics' and 'resettable_metrics' to add
			-- a prefix and/or suffix to the metric names.
		do
			create Result.make (0)
			if prefix_ /= Void then
				Result.append (prefix_)
			end
			Result.append (a_name)
			if suffix_ /= Void then
				Result.append (suffix_)
			end
		end

	metrics_called: BOOLEAN
			-- Both 'metrics' and 'resettable_metrics' should set this flag.

	registered: BOOLEAN

feature -- Basic operations

	reset_metrics is
			-- Resets all resettable_metrics to 0.
		do
		end

	add_metric_source (a_source: PM_METRIC_SOURCE) is
			-- Adds another source of metrics which contributes to the overall metrics of this object.
		require
			not_registered: not registered or a_source.registered
		do
			if co_sources = Void then
				create co_sources.make_empty
			end
			co_sources.extend (a_source)
		end

	set_prefix (a_prefix: STRING) is
		require
			metrics_not_called: not metrics_called
		do
			prefix_ := a_prefix
		end

	set_suffix (a_suffix: STRING) is
		require
			metrics_not_called: not metrics_called
		do
			suffix_ := a_suffix
		end

feature {PM_CONTEXT_METRICS} -- Implementation

	set_registered is
		do
			registered := True
		end

	co_sources: ROSE_LINEAR_ARRAY [PM_METRIC_SOURCE]

feature {NONE} -- Implementation

	prefix_: STRING

	suffix_: STRING

end
