indexing
	description: "Collection of metric tests"
	author: "Bradley Green"
	date: "$Date: 2009/07/30 $"
	revision: "$Revision: #1 $"

class
	PM_METRIC_TEST_COLLECTION

inherit
	PM_COLLECTION [PM_METRIC_TEST_RECORD]
		rename
			add_metric as generic_add_metric
		redefine
			get_items
		end

create
	make_for_instance_run

feature {NONE} -- Initialization

	make_for_instance_run (a_run: PM_INSTANCE_RUN) is
			-- create collection for a specific instance run
		require
			a_run_not_void: a_run /= Void
		do
			get_items (sql_instance_run_metrics,<<a_run.run_id>>)
		end

feature -- Access

	hash: HASH_TABLE [PM_METRIC_TEST_RECORD,INTEGER]
			-- Metrics in this collection, hashed by their metric_type_id

	types: ROSE_LINEAR_ARRAY [PM_METRIC_TYPE] is
			-- Types contained in this collecion
		local
			i: INTEGER
		do
			create Result.make (1,hash.count)
			from
				i := 1
				hash.start
			until
				hash.off
			loop
				Result.put (hash.item_for_iteration.type,i)
				i := i + 1
				hash.forth
			end
		ensure
			result_not_boid: Result /= Void
		end

	metric_test_by_name (a_name: STRING): PM_METRIC_TEST_RECORD is
			-- Metric test corresponding to metric type with 'a_name'
			-- if 'a_name' not found return Void
		require
			a_name_set: a_name /= Void
		local
			l_type: PM_METRIC_TYPE
		do
			l_type := process_metrics.metrics.type (a_name)
			if l_type /= Void then
				Result := metric_test (l_type.metric_type_id)
			end
		ensure
			result_may_be_void: True
		end

	metric_test (a_id: INTEGER):  PM_METRIC_TEST_RECORD is
			-- Metric test corresponding to metric type with 'a_id'
			-- if 'a_id' not found return Void
		do
			Result := hash.item (a_id)
		ensure
			result_may_be_void: True
		end

feature {NONE} -- Implementation

	get_items (a_predicate: COLLECTING_PREDICATE [PM_METRIC_TEST_RECORD]; a_args: ARRAY [ANY]) is
			-- query the database and store the items in the hash
		local
			i: INTEGER
		do
			create hash.make (0)
			Precursor (a_predicate,a_args)
			from
				i := items.lower
			until
				i > items.upper
			loop
				hash.put (items.item (i),items.item (i).metric_type_id)
				i := i + 1
			end
		ensure then
			results_hashed: hash.count = items.count
		end

	sql_instance_run_metrics: COLLECTING_PREDICATE [PM_METRIC_TEST_RECORD] is
			-- Retrieves a particular metric, for a given metric_type_id and run_id
		once
			create Result.make_strict (process_metrics.server,
				"SELECT * FROM metric_test_result WITH(NOLOCK) WHERE run_id=?")
		ensure
			result_not_void: Result /= Void
		end

invariant
	hash_not_void: hash /= Void

end

