note

	description:

		"Routines about process-metric types involving DB I/O"

	author: "Colin Adams"
	last_editor: "$Author: c4 $"
	date: "$Date: 2011/07/22 $"
	revision: "$Revision: #1 $"
	path: "$File: //rose/source/kernel/infrastructure/pm/pm_metric_type_routines.e $"

class PM_METRIC_TYPE_ROUTINES

inherit

	DATABASE_SERVERS

feature -- Access

	types: HASH_TABLE [PM_METRIC_TYPE, INTEGER] is
			-- All types indexed by unique identifier
		once
			create Result.make (0)
		ensure
			types_not_void: types /= Void
		end

	type (a_name: STRING): PM_METRIC_TYPE is
			-- Type corresponding to `a_name'
		local
			l_name: STRING
		do
			if a_name /= Void then
				l_name := a_name.as_lower
				l_name.left_adjust
				l_name.right_adjust
				if l_name.is_empty then
					-- Return Void
				elseif l_name.is_integer then
					Result := types.item (l_name.to_integer)
					if Result = Void then
						Result := retrieved_type_by_id (l_name.to_integer)
					end
				else
					Result := type_names.item (l_name)
					if Result = Void then
						Result := retrieved_type_by_name (l_name)
						if Result = Void then
							sql_insert_metric_type.execute (Void,<<a_name>>)
							Result := retrieved_type_by_name (l_name)
						end
					end
				end
			end
		end

	retrieved_type_by_id (a_id: INTEGER): PM_METRIC_TYPE is
			-- Metric type for `a_id'
		local
			l_type: PM_METRIC_TYPE
		do
			sql_metric_type_by_id.retrieve (Void, <<a_id>>)
			if sql_metric_type_by_id.array /= Void and then not sql_metric_type_by_id.array.is_empty then
				l_type := sql_metric_type_by_id.array.first
				types.put (l_type,l_type.metric_type_id)
				l_type.name.left_adjust
				l_type.name.right_adjust
				type_names.put (l_type,l_type.name.as_lower)
				Result := l_type
			end
		end

	retrieved_type_by_name (a_name: STRING): PM_METRIC_TYPE is
			-- Metric type for `a_name'
		local
			l_type: PM_METRIC_TYPE
		do
			sql_metric_type.retrieve (Void, <<a_name>>)
			if sql_metric_type.array /= Void and then not sql_metric_type.array.is_empty then
				l_type := sql_metric_type.array.first
				types.put (l_type,l_type.metric_type_id)
				l_type.name.left_adjust
				l_type.name.right_adjust
				type_names.put (l_type,l_type.name.as_lower)
				Result := l_type
			end
		end

	type_by_id (a_id: INTEGER): PM_METRIC_TYPE is
			-- Metric type with `a_id'
		do
			Result := type (a_id.out)
		end

feature {NONE} -- Predicates
	
	sql_metric_type_by_id: COLLECTING_PREDICATE [PM_METRIC_TYPE] is
			-- Retrieves a particular metric, for a given metric_type_id
		once
			create Result.make_strict (process_metrics_server,
				"SELECT * FROM metric_type WITH (NOLOCK) where metric_type_id = ?")
		ensure
			sql_metric_type_by_id_not_void: Result /= Void
		end

	sql_metric_type: COLLECTING_PREDICATE [PM_METRIC_TYPE] is
			-- Retrieves a particular metric, for a given name
		once
			create Result.make_strict (process_metrics_server,
				"SELECT * FROM metric_type WITH (NOLOCK) where name = ?")
		ensure
			sql_metric_type_not_void: Result /= Void
		end

	sql_insert_metric: EXECUTION_PREDICATE is
			-- Insert a particular metric into the metric table
		once
			create Result.make (process_metrics_server,
				"INSERT INTO metric (metric_type_id,run_id,[value]) VALUES (?,?,?)")
		ensure
			sql_insert_metric_not_void: Result /= Void
		end

	sql_insert_metric_type: EXECUTION_PREDICATE is
			-- Database predicate for inserting metric-types
		once
			create Result.make (process_metrics_server, "INSERT INTO metric_type (name) VALUES (?)")
		ensure
			sql_insert_metric_type_not_void: Result /= Void
		end

feature {NONE} -- Implementation

	type_names: HASH_TABLE [PM_METRIC_TYPE, STRING] is
			-- Metric type names associated to their ids
		once
				create Result.make (0)
		ensure
			type_names_not_void: Result /= Void
		end

end
