note

	description:

		"Definition of a process"

	author: "Zoran Simic"
	last_editor: "$Author: er $"
	date: "$Date: 2014/02/26 $"
	revision: "$Revision: #8 $"
	path: "$File: //rose/source/kernel/pm/pm_process_partition.e $"

class PM_PROCESS_PARTITION

inherit

	PM_COMMON
		redefine
			is_equal
		end

create

	make

feature {NONE} -- Creation

	make (a_process: PM_PROCESS; a_partition_key: STRING) is
			-- Create the partition
		require
			valid_process: a_process /= Void 
			valid_partition_key: a_partition_key /= Void
		do
			process := a_process
			partition_key := a_partition_key
		end

feature -- Basic Operations

	clear_instances is
			-- Clear data related to this partition
		local
			l_server: SQLOLEDB_DATA_SERVER
		do
			l_server ?= process_metrics.server
			l_server.sql_execute_immediate ("delete from message where run_id in " + " (select run_id from instance_run where instance_id in " + " (select instance_id from instance where process_id = " + process.process_id.out + " and partition_key = '" + partition_key + "'))")
			l_server.sql_execute_immediate ("delete from resource_access where run_id in " + " (select run_id from instance_run where instance_id in " + " (select instance_id from instance where process_id = " + process.process_id.out + " and partition_key = '" + partition_key + "'))")
			l_server.sql_execute_immediate ("delete from metric where run_id in " + " (select run_id from instance_run where instance_id in " + " (select instance_id from instance where process_id = " + process.process_id.out + " and partition_key = '" + partition_key + "'))")
			l_server.sql_execute_immediate ("delete from instance_run where instance_id in " + " (select instance_id from instance where process_id = " + process.process_id.out + " and partition_key = '" + partition_key + "')")
			l_server.sql_execute_immediate ("delete from instance where process_id = " + process.process_id.out + " and partition_key = '" + partition_key + "'")
			saved_instances := Void
			saved_statistics := Void
			saved_metrics := Void
			saved_resources := Void
		end

feature -- Access

	process: PM_PROCESS
			-- Associated process

	partition_key: STRING
			-- Associated partition key

	statistics: PM_METRIC_STATISTICS is
		do
			if saved_statistics = Void then
				create saved_statistics.make_for_partition (Current)
			end
			Result := saved_statistics
		end

	file_dates: ROSE_LINEAR_ARRAY [ROSE_DATE] is
			-- File dates corresponding to 'instances', ordered (ascending)
		local
			i: INTEGER
		do
			create Result.make (1, instances.count)
			from
				i := instances.lower
			until
				i > instances.upper
			loop
				Result.put (instances.item (i).file_date, i)
				i := i + 1
			end
		end

	instances: PM_INSTANCE_COLLECTION is
			-- All instances corresponding to this partition, ordered by file_date (ascending)
		do
			if saved_instances = Void then
				create saved_instances.make_for_partition (Current)
			end
			Result := saved_instances
		ensure
			valid_result: Result /= Void
		end

	metrics: PM_METRIC_COLLECTION is
			-- Metrics for this instance
		do
			if saved_metrics = Void then
				create saved_metrics.make_for_partition (Current)
			end
			Result := saved_metrics
		ensure
			valid_result: Result /= Void
		end

	resources: PM_RESOURCE_ACCESS_COLLECTION is
		do
			if saved_resources = Void then
				create saved_resources.make_for_partition (Current)
			end
			Result := saved_resources
		ensure
			valid_result: Result /= Void
		end

	most_recent_instance: PM_INSTANCE is
			-- Most recent instance for this partition
		local
			l_pm_instance: PM_INSTANCE_COLLECTION
		do
			if saved_instances /= Void then
				Result := saved_instances.last
			else
				create l_pm_instance.make_latest (Current)
				Result := l_pm_instance.last
			end
		end

	most_recent_succesful_instance: PM_INSTANCE is
			-- Most recent successful instance for this partition
		local
			i: INTEGER
		do
			if instances /= Void then
				from
					i := instances.upper
				until
					Result /= Void or else i < instances.lower
				loop
					if instances.item (i).runs.has_successful then
						Result := instances.item (i)
					end
					i := i - 1
				end
			end
		end

	most_recent_succesful_instance_before (a_date: ROSE_DATE): PM_INSTANCE is
			-- Most recent successful instance for this partition before a_date
		require
			a_date_not_void: a_date /= Void
		local
			i: INTEGER
		do
			if instances /= Void then
				from
					i := instances.upper
				until
					Result /= Void or else i < instances.lower
				loop
					if instances.item (i).runs.has_successful then
						if instances.item (i).file_date.date < a_date.date then
							Result := instances.item (i)
						end
					end
					i := i - 1
				end
			end
		end

	is_equal (a_other: like Current): BOOLEAN is
			-- Is `other' attached to an object considered
			-- equal to current object?
		do
			Result := a_other /= Void and then process = a_other.process and then equal (partition_key, a_other.partition_key)
		end

	out: STRING is
			-- New string containing terse printable representation of current object
		do
			Result := process.out + "/"
			Result.append_string (partition_key)
		end

feature {NONE} -- Implementation

	saved_instances: PM_INSTANCE_COLLECTION

	saved_statistics: PM_METRIC_STATISTICS

	saved_metrics: PM_METRIC_COLLECTION

	saved_resources: PM_RESOURCE_ACCESS_COLLECTION

end
