indexing
	description: "Attributes to describle a process metric resource like a database."
	author: "Zoran Simic"
	last_editor: "$Author: c4 $"
	date: "$Date: 2010/10/21 $"
	revision: "$Revision: #10 $"
	path: "$File: //rose/source/kernel/pm/pm_resource_access.e $"

class
	PM_RESOURCE_ACCESS

inherit
	PM_COMMON

create
	default_create

feature -- Access

	type_id: INTEGER
			-- Identifier for `type'

	run_id: INTEGER
			-- Internal ids identifying this metric
			-- 'run_id' is 0 for aggregated metrics

	path: STRING
			-- Path to used resource

	logical: STRING
			-- Logical of the resource

	count: INTEGER
			-- Number of times the resource was used

	type: PM_RESOURCE_TYPE is
			-- Type of this resource
		do
			if saved_type = Void then
				saved_type := process_metrics.resource_type_by_id (type_id)
			end
			Result := saved_type
			if Result = Void then
				create Result.make_unknown
			end
		ensure
			attached_result: Result /= Void
		end

	key: PM_RESOURCE_KEY is
			-- Key for resource type/path/logical;
			-- Suitable for aggregating resource usage in a hash table.
		do
			create Result.make (type_id, path, logical)
		ensure
			key_not_void: Result /= Void
		end

	is_file: BOOLEAN is
			-- Does `Current' represent a file?
		do
			Result := type_id = pm_constants.pm_file or is_input_file or is_output_file
		end

	is_input_file: BOOLEAN is
			-- Does `Current' represent a input file?
		do
			Result := type_id = pm_constants.pm_input_file
		end

	is_output_file: BOOLEAN is
			-- Does `Current' represent a output file?
		do
			Result := type_id = pm_constants.pm_output_file
		end

	is_database: BOOLEAN is
			-- Does `Current' represent a database?
		do
			Result := type_id = pm_constants.pm_database or
				is_input_database or is_output_database
		end

	is_input_database: BOOLEAN is
			-- Does `Current' represent a input database?
		do
			Result := type_id = pm_constants.pm_input_database
		end

	is_output_database: BOOLEAN is
			-- Does `Current' represent a output database?
		do
			Result := type_id = pm_constants.pm_output_database
		end

	is_storable: BOOLEAN is
			-- Does `Current' represent a database?
		do
			Result := is_input_storable or is_output_storable
		end

	is_input_storable: BOOLEAN is
			-- Does `Current' represent a input storable?
		do
			Result := type_id = pm_constants.pm_input_storable
		end

	is_output_storable: BOOLEAN is
			-- Does `Current' represent a output storable?
		do
			Result := type_id = pm_constants.pm_output_storable		
		end

	is_persistent_server: BOOLEAN is
			-- Does `Current' represent a database?
		do
			Result := is_input_persistent_server or is_output_persistent_server
		end

	is_input_persistent_server: BOOLEAN is
			-- Does `Current' represent a input persistent_server?
		do
			Result := type_id = pm_constants.pm_input_persistent_server
		end

	is_output_persistent_server: BOOLEAN is
			-- Does `Current' represent a output persistent_server?
		do
			Result := type_id = pm_constants.pm_output_persistent_server		
		end

	out: STRING is
			-- New string containing terse printable representation of current object
		do
			create Result.make (10)
			Result.append_string (path)
		end

feature {NONE} -- Implementation

	saved_type: PM_RESOURCE_TYPE
	saved_run: PM_INSTANCE_RUN

end





