indexing
	description: "Collection of resource accesses"
	author: "Zoran Simic"
	last_editor: "$Author: mb $"
	date: "$Date: 2009/08/20 $"
	revision: "$Revision: #5 $"
	path: "$File: //rose/source/kernel/pm/pm_resource_access_collection.e $"
	
class
	PM_RESOURCE_ACCESS_COLLECTION
	
inherit
	PM_COLLECTION [PM_RESOURCE_ACCESS]
	
	create
	make_for_process, make_for_partition, make_for_instance, make_for_run
	
feature {NONE} -- Initialization
	
	make_for_process (a_process: PM_PROCESS; a_start_date: ROSE_DATE) is
		require
			a_process /= Void
		do
			if a_start_date /= Void then
				get_items (sql_process_resources, <<a_start_date, a_process.process_id>>)
			else
				get_items (sql_process_resources, <<min_date, a_process.process_id>>)
			end				
		end
	
	make_for_partition (a_partition: PM_PROCESS_PARTITION) is
		require
			a_partition /= Void
		do
			get_items (sql_partition_resources,<<a_partition.process.process_id,a_partition.partition_key>>)
		end
	
	make_for_instance (a_instance: PM_INSTANCE) is
		require
			a_instance /= Void
		do
			get_items (sql_instance_resources,<<a_instance.instance_id>>)
		end
	
	make_for_run (a_run: PM_INSTANCE_RUN) is
		require
			a_run /= Void
		do
			get_items (sql_run_resources,<<a_run.run_id>>)
		end
	
feature {NONE} -- Implementation
	
	sql_process_resources: COLLECTING_PREDICATE [PM_RESOURCE_ACCESS] is
		once
			create Result.make_strict (process_metrics.server,
									   "[
		SELECT TOP 100 PERCENT resource_access.type_id, resource_access.path, resource_access.logical, COUNT(*) AS [count]
			FROM resource_access WITH(NOLOCK) INNER JOIN
										instance_run ON resource_access.run_id = instance_run.run_id INNER JOIN
										instance ON instance_run.instance_id = instance.instance_id
										where instance_run.start_time > ? and instance.process_id = ?
										GROUP BY resource_access.type_id, resource_access.path, resource_access.logical, instance.process_id
										ORDER BY resource_access.type_id, resource_access.path, resource_access.logical
										]")
			end
			
			sql_partition_resources: COLLECTING_PREDICATE [PM_RESOURCE_ACCESS] is
		once
			create Result.make_strict (process_metrics.server,
									   "[
		SELECT TOP 100 PERCENT resource_access.type_id, resource_access.path, resource_access.logical, COUNT(*) AS [count]
			FROM resource_access WITH(NOLOCK) INNER JOIN
										instance_run ON resource_access.run_id = instance_run.run_id INNER JOIN
										instance ON instance_run.instance_id = instance.instance_id
										GROUP BY resource_access.type_id, resource_access.path, resource_access.logical, instance.process_id, instance.partition_key
										HAVING (instance.process_id = ?) AND (instance.partition_key = ?)
										ORDER BY resource_access.type_id, resource_access.path, resource_access.logical
										]")
			end
			
			sql_instance_resources: COLLECTING_PREDICATE [PM_RESOURCE_ACCESS] is
		once
			create Result.make_strict (process_metrics.server,
									   "[
		SELECT resource_access.type_id, resource_access.path, resource_access.logical, COUNT(*) AS [count]
			FROM resource_access WITH(NOLOCK) INNER JOIN
										instance_run ON resource_access.run_id = instance_run.run_id
										GROUP BY resource_access.type_id, resource_access.path, resource_access.logical, instance_run.instance_id
										HAVING (instance_run.instance_id=?)
										ORDER BY resource_access.type_id, resource_access.path, resource_access.logical
										]")
			end
			
			sql_run_resources: COLLECTING_PREDICATE [PM_RESOURCE_ACCESS] is
		once
			create Result.make_strict (process_metrics.server,
									   "[
		SELECT resource_access.type_id, resource_access.path, resource_access.logical, 1 AS [count] FROM resource_access WITH(NOLOCK)
										WHERE resource_access.run_id=?
										ORDER BY resource_access.type_id, resource_access.path
										]")
		end
	
			min_date: ROSE_DATE is 
			-- Default query start date
		once
			create Result.make_ccyymmdd (19000101)
		ensure
			valid_result: Result /= Void
		end
	
end
