indexing

	description: "Key for resource. Suitable for aggregating resource usage in a hash table."

	author: "Colin Adams"
	last_editor: "$Author: c4 $"
	date: "$Date: 2010/11/17 $"
	revision: "$Revision: #2 $"
	path: "$File: //rose/source/kernel/pm/pm_resource_key.e $"

class PM_RESOURCE_KEY

inherit

	HASHABLE
		redefine
			is_equal
		end

	PM_CONSTANTS
		export
			{NONE} all
		redefine
			is_equal			
		end

create

	make

feature {NONE} -- Initialization

	make (a_type_id: INTEGER; a_path, a_logical: STRING) is
			-- Initialize attributes.
		do
			type_id := a_type_id
			path := a_path
			logical := a_logical
		ensure
			type_id_set: type_id = a_type_id
			path_set: equal (path, a_path)
			logical_set: equal (logical, a_logical)
		end

feature -- Access

	type_id: INTEGER
			-- Identifier for type of resource

	path: STRING
			-- Path to used resource

	logical: STRING
			-- Logical of the resource

	hash_code: INTEGER
			-- Hash code value
		do
			Result := type_id
			if path /= Void then
				Result := Result + path.hash_code
			end
			if logical /= Void then
				Result := Result + logical.hash_code
			end
			if Result < 0 then
				Result := -Result
			end
		end

	is_database: BOOLEAN is
			-- Does `Current' represent a database?
		do
			Result := (type_id = pm_database) or
				(type_id = pm_input_database) or
				(type_id = pm_output_database)
		end

	is_storable: BOOLEAN is
			-- Does `Current' represent an Eiffel Storable file?
		do
			Result := (type_id = pm_storable) or
				(type_id = pm_input_storable) or
				(type_id = pm_output_storable)
		end

	is_file: BOOLEAN is
			-- Does `Current' represent a file other than an Eiffel Storable?
		do
			Result := (type_id = pm_file) or
				(type_id = pm_input_file) or
				(type_id = pm_output_file)
		end

	is_persistent_server: BOOLEAN is
			-- Does `Current' represent a persistent_server?
		do
			Result := (type_id = pm_persistent_server) or
				(type_id = pm_input_persistent_server) or
				(type_id = pm_output_persistent_server)
		end

	is_output: BOOLEAN is
			-- Does `Current' represent a resource that might be written to?
		do
			Result := (type_id = pm_persistent_server) or
				(type_id = pm_output_persistent_server) or
				(type_id = pm_output_file) or
				(type_id = pm_file) or
				(type_id = pm_storable) or
				(type_id = pm_output_storable) or
				(type_id = pm_database) or
				(type_id = pm_output_database)
		end

	is_input: BOOLEAN is
			-- Does `Current' represent a resource that might be read from?
		do
			Result := (type_id = pm_persistent_server) or
				(type_id = pm_input_persistent_server) or
				(type_id = pm_input_file) or
				(type_id = pm_file) or
				(type_id = pm_storable) or
				(type_id = pm_input_storable) or
				(type_id = pm_database) or
				(type_id = pm_input_database)
		end
	
feature -- Comparison

	is_equal (a_other: like Current): BOOLEAN is
			-- Is `a_other' attached to an object considered
			-- equal to current object?
		do
			Result := type_id = a_other.type_id and
				equal (path, a_other.path) and
				equal (logical, a_other.logical)
		end

end
