note

	description:

		"[
        Objects that provide a run-identifier - from process-metrics when available,
         otherwise from ActiveBatch if applicable. Otherwise the user and program name.
       ]"

	author: "Colin Adams"
	last_editor: "$Author: mb $"
	date: "$Date: 2013/01/22 $"
	revision: "$Revision: #2 $"
	path: "$File: //rose/source/kernel/supplier_data/pm_run_id.e $"

class PM_RUN_ID

inherit

	PM_AWARE

	ROSE_SHARED_GLOBAL
		export {NONE} all	end

feature -- Access

	run_id: STRING
			-- ID identifying the currently executing process, using process_metrics or
			-- ActiveBatch ids when available
		local
			l_app: ROSE_APPLICATION
			l_activebatch_scheduler: STRING
			l_activebatch_job_id: INTEGER
		do
			if process_metrics.is_available and then process_metrics.is_running then
				Result := "PM:" + process_metrics.server.node + "." + process_metrics.server.database + ":" + process_metrics.current_instance.current_run.run_id.out
			else
				l_activebatch_scheduler := global.env.scheduler
				l_activebatch_job_id := global.env.job_id
				if not l_activebatch_scheduler.is_empty and then l_activebatch_job_id /= 0 then
					Result := "AB:" + l_activebatch_scheduler + ":" + l_activebatch_job_id.out
				else
					l_app := global.app
					Result :=  l_app.user.login_id + ":" + l_app.host_name + ":" + l_app.program_path
				end
			end
		ensure
			run_id_not_void: Result /= Void
		end

	run_id_from_process_message (a_message: STRING): INTEGER is
			-- The process metrics run id extracted from `a_message'
		local
			l_index: INTEGER
			l_run_id_string: STRING
		do
			if a_message /= Void and then a_message.has (':') and then a_message.starts_with ("PM") then
				l_index := a_message.last_index_of (':', a_message.count)
				l_run_id_string := a_message.substring (l_index + 1, a_message.count)
				if l_run_id_string.is_integer then
					Result := l_run_id_string.to_integer
				end
			end
		end
	
end
	
