indexing
	description: "Message sent to port_server from its client to request a server startup"
	author: "Mauricio Bustos"
	last_editor: "$Author: dz $"
	date: "$Date: 2009/07/20 $"
	revision: "$Revision: #16 $"
	file: "$Workfile: port_server_message.e $"
	path: "$File: //rose/source/kernel/infrastructure/network/port_server_message.e $"

class
	PORT_SERVER_MESSAGE

inherit
	MISMATCH_CORRECTOR
		redefine
			correct_mismatch
		end

create
	make, default_create

feature -- Initialization

	make is
		do
			executable := ""
			user_id := ""
			port := 0
			connect_to_server := True
		end

feature -- Access

	user_id : STRING
	executable : STRING
	port : INTEGER
	machine : STRING
	command_line : STRING
	rose_command : STRING
			-- If the executable to run is a framework application the execute this command
	server_arguments : ARRAY[STRING]
			-- If the executable to run is a framework application then these should be named arguments
			-- otherwise they can be freeform
	connect_to_server: BOOLEAN
			-- if set (normal), client should connect to nascent server after receiving acknowledgement
			-- from port_server, to ensure it is running.
	standard_output_filename: STRING
			-- name of standard output file (may be foreign platform file name, hence not FILE_NAME)
	working_directory: STRING
			-- default directory for server
	be_nice: BOOLEAN
			-- spawn process at lower priority (be nice to other users)?

feature -- Status setting

	set_command_line(a_value : like command_line) is
		do
			command_line := a_value
		end
	set_server_arguments(a_value : like server_arguments) is
		do
			server_arguments := a_value
		end
	set_user_id(a_value : like user_id) is
		do
			user_id := a_value
		end
	set_port(a_value : like port) is
		do
			port := a_value
		end
	set_machine(a_value : like machine) is
		do
			machine := a_value
		end
	set_executable(a_value : like executable) is
		do
			executable := a_value
		end
	set_connect_to_server (a_value: like connect_to_server) is
		do
			connect_to_server := a_value
		end
	set_rose_command (a_value: like rose_command) is
		do
			rose_command := a_value
		end
	set_standard_output_filename (a_file_name: like standard_output_filename) is
		require
			filename_exists: a_file_name /= Void and then not a_file_name.is_empty
		do
			standard_output_filename := a_file_name
		end
	set_working_directory (a_path: like working_directory) is
			-- set `working_directory' to `a_path'
		require
			path_not_void_nor_empty: a_path /= Void and then not a_path.is_empty
		do
			working_directory := a_path
		end
	set_be_nice (a_value: BOOLEAN) is
			-- set `be_nice' to `a_value'
		do
			be_nice := a_value
		ensure
			be_nice_set: be_nice = a_value
		end

feature	-- Get

	get_flat_arguments : STRING is
			-- rose command and server arguments concatenated together as lower case, with intervening spaces
		local
			i : INTEGER
		do
			if rose_command /= Void then
				Result := rose_command.as_lower
			else
				Result := ""
			end
			if server_arguments /= Void then
				from i := server_arguments.lower until i > server_arguments.upper loop
					if server_arguments.item(i) /= Void then
						Result := Result + " " + server_arguments.item(i).as_lower
					end
					i := i + 1
				end
			end
		end

	get_user_id : STRING is
		do
			if user_id /= Void then
				Result := user_id
			else
				Result := "invalid_user"
			end
		end

	get_port : INTEGER is
		do
			Result := port
		end

	get_machine : STRING is
		do
			if machine /= Void then
				Result := clone(machine)
			else
				Result := ""
			end
		end

	get_executable : STRING is
		do
			if executable /= Void then
				Result := clone(executable)
			else
				Result := ""
			end
		end

	get_command_line : STRING is
		do
			if command_line /= Void then
				Result := clone(command_line)
			else
				Result := "Unable to get command line"
			end
		end

	get_client_pid : INTEGER is
		local
			l_command, l_pid : STRING
		do
			l_command := get_command_line
			if l_command.has_substring("(") and l_command.has_substring(")") then
				l_pid := l_command.substring(l_command.index_of('(',1) + 1,l_command.index_of(')',1) - 1)
				if l_pid.is_integer then
					Result := l_pid.to_integer
				end
			end
		end

feature {NONE} -- Transformation

	correct_mismatch is
			-- Attempt to correct object mismatch using `mismatch_information'.
		local
			l_any: ANY
		do
			l_any := Mismatch_information
		end

end
