note

	description:

		"ROSE_CONTEXT version of PERSISTENT_PORTFOLIO"

	author: "Mark Howard"
	last_editor: "$Author: er $"
	date: "$Date: 2014/02/26 $"
	revision: "$Revision: #76 $"
	path: "$File: //rose/source/kernel/investment_entities/portfolio.e $"

class	PORTFOLIO

inherit

	INVESTMENT_ENTITY

	ABSTRACT_PORTFOLIO[LISTING]
		undefine
			is_equal
		end

	PERSISTENT_PORTFOLIO
		undefine
			context,
			make,
			make_context,
			ur_targets,
			expression,
			debug_output,
			listings
		end

	EVENT_ACCESS
		undefine
			is_equal
		end
	ATTRIBUTES
		rename
			msci as attributes_msci,
			name as attributes_name,
			brim_code as attributes_brim_code,
			company_name as attributes_company_name,
			cusip as attributes_cusip,
			sedol as attributes_sedol,
			xrf_code as attributes_xrf_code
		undefine is_equal
		end

create

	make, make_anonymous

feature -- Access

	name_array: ROSE_LINEAR_ARRAY [STRING]
			-- Long name of portfolio broken into descriptive units to aid parsing for characteristics
		do
			if name_array_saved /= Void then
				Result := name_array_saved
			else
				if name /= Void and then name.count > 2 then
					create Result.make_empty
					Result.extend(name.substring(1,1))
					if name.count >= 9 and then (name.substring(name.count - 5, name.count).as_upper.is_equal("_BOGIE") or else name.substring(name.count - 5, name.count).as_upper.is_equal("_SHORT") or else name.substring(name.count - 5, name.count).as_upper.is_equal("_COMBM")) then
						Result.extend(name.substring(3, name.count - 6))
						Result.extend(name.substring(name.count -4, name.count))
					elseif name.count >= 8 and then (name.substring(name.count - 4, name.count).as_upper.is_equal("_COMP") or else name.substring(name.count - 4, name.count).as_upper.is_equal("_LONG")) then
						Result.extend(name.substring(3, name.count - 5))
						Result.extend(name.substring(name.count - 3, name.count))
					elseif name.count >= 6 and then name.substring(name.count - 2, name.count).as_upper.is_equal("_MN") then
						Result.extend(name.substring(3, name.count - 3))
						Result.extend(name.substring(name.count - 1, name.count))
					else
						Result.extend(name.substring(3, name.count))
					end
					name_array_saved := Result
				end
			end
		end

	name_array_saved: ROSE_LINEAR_ARRAY [STRING]
		-- cached version of `name_array'
		-- Should this be moved to Implementation and exported to {NONE}?

	name: STRING is
			-- Name of `Current' portfolio
		do
			if name_saved = Void then
				Result := name_most_recent_key(Void)
				name_saved := Result
			else
				Result := name_saved
			end
		end

	name_saved: like name
			-- Cached result for `name'
			-- Should this be moved to Implementation and exported to {NONE}?

	old_name: STRING
			-- Name of portfolio retrieved by parsing long portfolio "name" (name_array) which includes type indicators
			-- Exception of handling TOPIX bogie text included
		do
			if name_array /= Void then
				Result := name_array.item(2)
				if is_bogie and then name.substring(3, 8).as_lower.is_equal("topix_") then
						Result := "TOPIX"
				end
			end
		end

	extension: STRING
			-- suffix of long name which should denote portfolio type (MN, bogie, composite, etc)
		local
			l_array: ROSE_LINEAR_ARRAY [STRING]
		do
			l_array := name_array
			if l_array /= Void then
				if l_array.count > 2 then
					Result := l_array.item (l_array.count)
				end
			end
		end

	extended_name: STRING is
			-- Short name with portfolio type extension appended
		do
			if extension /= Void then
				Result := old_name + "_" + extension
			else
				Result := old_name
			end
		end

	hedged_name: STRING is
			-- Suffixed version of name to allow access to VMS performance for hedged accounts
		do
			create result.make_from_string(old_name)
			Result.append("_H")
		end

	strategy_name (a_date: ROSE_DATE): STRING
			-- Name of associated strategy of portfolio for purposes of determing alpha signal mode and QUOP settings
		require
			a_date_not_void: a_date /= Void
		local
			l_strategy : like context.strategy_template
		do
			l_strategy := strategy (a_date)
			if l_strategy /= Void then
				Result := l_strategy.name_at (a_date)
			end
		end

	strategy (a_date: ROSE_DATE): like context.strategy_template
			-- Strategy associated with portfolio which indicates chosen alpha signaland other QUOP settings
		require
			a_date_not_void: a_date /= Void
		local
			l_history : like strategy_to_portfolio_co_history
		do
			l_history := strategy_to_portfolio_co_history
			if l_history /= Void then
				Result := l_history.witness (a_date)
			end
		end

	zone_c: CHARACTER
			-- Zone character directly associated with portfolio object
		do
			if name_array /= Void then
				Result := name_array.item(1).item(1)
			end
		end

	is_bogie: BOOLEAN is
			--is portfolio a bogie?
		do
			if extension /= Void then
				Result := extension.as_lower.is_equal("bogie")
			end
		end

	is_long_short: BOOLEAN is
			-- Is portfolio either a 'long' or a 'short' portfolio	
		do
			if extension /= Void then
				Result := extension.as_lower.is_equal ("long") or extension.as_lower.is_equal ("short")
			end
		end

	is_mn: BOOLEAN is
			-- Is portfolio a 'market-neutral' portfolio
			-- Note: Implementation as of 11 Feb 2009 returns true for LLS as well as MN portfolios. (d3)	
		do
			if extension /= Void then
				Result := extension.as_lower.is_equal ("mn")
			end
		end

	is_sim: BOOLEAN is
			-- Is it a sim portfolio?
		do
			if extension /= Void then
				Result := extension.as_lower.is_equal("sim")
			end
		end

	is_composite: BOOLEAN is
			-- Is portfolio a 'composite' portfolio?
		do
			if extension /= Void then
				Result := extension.as_lower.is_equal ("comp")
			end
		end

	is_composite_bogie: BOOLEAN is
			-- Is portfolio a 'composite bogie' portfolio?
		do
			if extension /= Void then
				Result := extension.as_lower.is_equal ("combm")
			end
		end

	is_short: BOOLEAN is
			-- Is portfolio a 'short' portfolio?
		do
			if extension /= Void then
				Result := extension.as_lower.is_equal ("short")
			end
		end

	is_long: BOOLEAN is
			-- Is portfolio a 'long' portfolio?
		do
			if extension /= Void then
				Result := extension.as_lower.is_equal ("long")
			end
		end

	is_global:BOOLEAN is
			-- Is `Current' global?
		do
			Result := zone_c.is_equal('W')
		end

	is_fake_global(a_name:STRING): BOOLEAN is
			-- Is 'a_name" fake global?
		do
			Result := zone_c.is_equal('W') and fake_global_accounts.has(a_name)
		end

	fake_global_accounts:ROSE_LINEAR_ARRAY[STRING] is
			-- Fake global accounts
		once
			create Result.make_empty
			Result.extend ("W_AMERICAW")
			Result.extend ("W_PARISW")
			Result.extend ("W_UAPW")
			Result.extend ("W_OPEN4W")
			Result.extend ("W_JAPONACW")
			Result.extend ("W_MATACTAW")
			Result.extend ("W_OEICW")
			Result.extend ("W_DENMARKW")
			Result.extend ("W_TOKYOW")
			Result.extend ("W_TMCW")
			Result.extend ("W_LAXUSW")
			Result.extend ("W_DAWNW")
			Result.extend ("W_PAOLOW")
			Result.extend ("W_EONW")
			Result.compare_objects
		end

	is_hedged (a_date:ROSE_DATE): BOOLEAN is
			-- Is the portfolio name in the hardcoded list of `hedged_accounts'
		do
			Result := hedged_accounts.has (name)
		end

	hedged_accounts: ROSE_LINEAR_ARRAY[STRING] is
			-- Hard-coded list of hedged accounts
		once
			create Result.make_empty
--			Result.extend("W_TOKYOW")
			Result.extend("W_TMCW")
--			Result.extend("W_LAXUSW")
--			Result.extend("W_KROGERW")
			Result.extend("E_DAWNH")
			Result.extend("W_MEDIC")
			Result.extend("W_ELBOW")
			Result.extend("W_CASSA")
			Result.extend("W_METRI")
			Result.extend("W_PAOLOW")
			Result.extend("W_ATOM")
			Result.extend("W_CUP")
			Result.extend("W_MUG")
			Result.extend("W_GOBLET")
			Result.compare_objects
		end



feature -- Basic operations

	listings: ROSE_ENTITY_HISTORY [like context.listing_template] is
			-- Associated listings of holdings
			-- Securities retrieved from holdings via BRIM code
			-- Entity interval for first listing associated to security retrieved
		local
			l_holdings_predicate : COLLECTING_PREDICATE[HOLDINGS_RECORD]
			i : INTEGER
			l_record : HOLDINGS_RECORD
			l_interval : ENTITY_INTERVAL[like context.listing_template]
			l_listing : like context.listing_template
			l_security : like context.security_template
			l_name : STRING
		do
			l_name := name_at (today)
			l_name := l_name.substring (3, l_name.count)
			create Result.make_empty
			l_holdings_predicate := holdings_predicates.item(zone_mnemonic)
			l_holdings_predicate.retrieve(void,<<l_name>>)
			from i := 1 until i > l_holdings_predicate.array.count loop
				l_record := l_holdings_predicate.array.item(i)
				l_record.instrument.prune_all(' ')
				l_security := context.security_at (attributes_brim_code(l_record.instrument), today)
				if l_security /= Void then
					l_listing := l_security.listing(1)
				end
				if l_listing /= Void then
					create l_interval.make (l_listing, l_record.applicable_from, l_record.applicable_to, l_record.units)
					Result.extend(l_interval)
				else
					print ("===> SKIPPING: " + l_record.instrument + " in " + l_record.portfolio + " from " + l_record.applicable_from.ccyymm.out + " to " + l_record.applicable_to.ccyymm.out + "%N")
				end
				i := i + 1
			end
		end

	register is
			-- Register this entity
			-- Implementation deferred to descendents
		do
		end

feature -- Access

	long_portfolio: like Current
			-- Associated portfolio of short positions if it exists (for market-neutral or short portfolios)	
		local
			l_name: STRING
			l_name_array: ROSE_LINEAR_ARRAY [STRING]
		do
			if is_mn or is_short then
				l_name_array := name_array
				if l_name_array /= Void then
					l_name := l_name_array.item(1) + "_" + l_name_array.item(2) + "_LONG"
					Result := context.portfolio_at (attributes_name (l_name), Void)
				end
			end
		end

	short_portfolio: like Current
			-- Associated portfolio of short positions if it exists (for is_mn and _long portfolios)
		local
			l_name: STRING
			l_name_array: ROSE_LINEAR_ARRAY [STRING]
		do
			if is_mn or is_long then
				l_name_array := name_array
				if l_name_array /= Void then
					l_name := l_name_array.item(1) + "_" + l_name_array.item(2) + "_SHORT"
					Result := context.portfolio_at (attributes_name (l_name), Void)
				end
			end
		end

feature {NONE} -- Implementation

	add_contribution_or_withdrawal(a_record : OPEN_ACCOUNT_EVENTS_RECORD) is
			-- Add contribution or witdrawal to `contributions_and_withdrawals_table'
			-- Create the table if does not yet exist
		local
			l_array : ROSE_LINEAR_ARRAY[OPEN_ACCOUNT_EVENTS_RECORD]
		do
			l_array := contributions_and_withdrawals_table.item(a_record.event_date)
			if l_array = Void then
				create l_array.make_empty
				contributions_and_withdrawals_table.put(l_array,a_record.event_date)
			end
			l_array.extend(a_record)
		end

	contributions_and_withdrawals_table : HASH_TABLE [ROSE_LINEAR_ARRAY [OPEN_ACCOUNT_EVENTS_RECORD], ROSE_DATE]
			-- Contributions and withdrawals table

feature -- Basic operations

	company_matrix_at (a_date: ROSE_DATE): ENTITY_MATRIX [like company_template] is
			-- Matrix of company-related information for companies held in the portfolio at `a_date'
			-- Implemented by descendents e.g. ROSE_PORTFOLIO
		do
		end

	holdings: PRODUCTION_HOLDINGS is
			-- Holdings of portfolio in production
		do
			if holdings_saved = Void then
				create holdings_saved.make_portfolio (zone_mnemonic, old_name)
			end
			Result := holdings_saved
		end

	holdings_saved : like holdings
			-- cached version of `holdings'
			-- Should this be moved to Implementation and exported to {NONE}?
end
