note
	description: "Parent for all account and portfolio positions (cash, non_cash, open_ffx), traded or settled"
	author: "Sami Ben Grine & Ivan Tarnavsky"
	last_editor: "$Author: sm $"
	date: "$Date: 2013/09/27 $"
	revision: "$Revision: #2 $"
	path: "$File: //rose/source/kernel/expert_system/position.e $"

deferred class POSITION [G->RP_ZONAL_INVESTMENT_GROUP]

inherit
	ANY

	ROSE_SHARED_GLOBAL
		export {NONE} all end

	AXAROSENBERG_AWARE
		rename axarosenberg as manager export {NONE} all undefine is_equal  end

feature -- Access

	account : RP_INVESTMENT_ACCOUNT
			-- Database column `account_code'

	global_investment_date: ROSE_DATE
		-- As-of-date

	update_mode: RP_POSITIONS_UPDATE_MODE
		-- Update mode

	accounting_basis: RP_ACCOUNTING_BASIS
		-- Accounting basis

	errors: ROSE_LINEAR_ARRAY [INTEGER]
		-- Errors if any

feature -- Queries

	quantity: RP_QUANTITY
		-- Quantity of 'instrument'
		deferred
		ensure
			quantity_not_void: Result /= Void
		end

	instrument: RP_INSTRUMENT
		-- Instrument
		deferred
		end

	valuation: RP_MONIES is
			-- Value in arbitrary currency
		deferred
		ensure
			result_not_void: Result /= Void
		end

	valuation_in_currency (a_currency: RP_CURRENCY): RP_MONIES
			-- Valuation in 'a_currency'
		require
			currency_not_void: a_currency /= Void
		local
			l_valuation: like valuation
			l_currency: RP_CURRENCY
			l_fx: RP_DECIMAL
			l_previous_end_of_day: ROSE_DATE
		do
				-- Get valuation
			l_valuation := valuation
			l_currency := l_valuation.currency

				-- Get fx rate
			create l_previous_end_of_day.make_by_date (global_investment_date)
			l_previous_end_of_day.add_n_days (-1)
			l_fx := manager.performance_system.fx.spot_rate (l_currency, a_currency, l_previous_end_of_day)

				-- Compute result
			if l_fx /= Void and then l_fx.is_defined then
				Result := a_currency.monies (l_valuation.amount * l_fx)
			end
		ensure
			result_not_void: Result /= Void
			result_in_currency: Result.currency ~ a_currency
		end

feature {NONE} -- Implementation

	set_errors (a_errors: STRING) is
			-- Set 'errors' codes from 'a_errors' (comma separated string with integer codes)
		require
			a_errors_not_Void: a_errors /= Void
		local
			l_strings:SEQUENCE [STRING]
			l_item: STRING
		do
			create errors.make_empty
			if not a_errors.is_empty then
				l_strings := a_errors.split (',')
				from
					l_strings.start
				until
					l_strings.after
				loop
					l_item := l_strings.item_for_iteration
					if l_item /= Void and then l_item.is_integer then
						errors.extend (l_item.to_integer)
					end
					l_strings.forth
				end
			end
		ensure
			errors_not_void: errors /= Void
			errors_set: global.formatter.join_array (errors, ",") ~ a_errors
		end

	zero_usd: RP_MONIES is
			-- Zero USD
		do
			Result := usd.monies (0)
		ensure
			result_not_void: Result /= Void
			result_is_usd: Result.currency ~ usd
			result_is_zero: Result.amount.is_zero
		end

	usd: RP_CURRENCY is
			-- USD currency
		once
			Result := manager.reference_system.currency_from_iso_code ("USD")
		ensure
			result_not_void: Result /= Void
			is_usd: Result.iso ~ "USD"
		end

invariant

	global_investment_date_exists: global_investment_date /= Void
	update_mode_exists: update_mode /= Void
	account_exists: account /= Void
	accounting_basis_exists: accounting_basis /= Void
 	errors_exists: errors /= Void

end
