indexing
	description : "[
		Handles the interactions between a PREDICATE, which represents a dynamic sql statement,
		and the actual database engine
		]"
	author : "Mark Howard"
	last_editor: "$Author: it $"
	date: "$Date: 2008/12/01 $"
	revision: "$Revision: #5 $"
	path: "$File: //rose/source/kernel/infrastructure/persistence/predicates/predicate_facilitator.e $"

deferred class
	PREDICATE_FACILITATOR

feature -- Access

	binding: ARRAY [STRING]
	input_type: INTEGER
	
	data_server: DATA_SERVER is
			-- The server on which this facilitator is to act on
		deferred
		end

	input: DATA_DESCRIPTORS [DATA_DESCRIPTOR] is
			-- The input data structure for the predicate
		deferred
		end

	output: DATA_DESCRIPTORS [DATA_DESCRIPTOR] is
			-- The output data structure for the predicate
		deferred
		end

	input_index (a_string: STRING): INTEGER is
			-- The index in the input data structure for the item named 'a_string'
		do
			Result := input.column_index (a_string)
		end

	output_index (a_string: STRING): INTEGER is
			-- The index in the output data structure for the item named 'a_string'
		do
			Result := output.column_index (a_string)
		end
	
feature -- Status
	
	should_not_be_tracked: BOOLEAN
			-- Should executions of this predicate not be used in counting how a data server is used?
	
	success: BOOLEAN is
			-- Was the last command successful?
		deferred
		end

feature -- Basic operations

	commit is
			-- Commit the last execution
		do
			data_server.commit
			set_success (data_server.success)
		end

	clear_error is
			-- Clear the last success status
		do
			set_success (True)
		end

	close_and_release_command is
			-- Clean up the facilitator
		do
			do_nothing
		end

feature -- Element modification	
	
	set_binding (a_binding: ARRAY [STRING]) is
			-- Set binding to 'a_binding'
		require
			valid_binding: a_binding /= Void		
		do
			binding := clone (a_binding)
		end

	set_success (a_boolean: BOOLEAN) is
			-- Set the success status to 'a_boolean'
		deferred
		end

end
