indexing
	description: "Representation of a listing price record"
	author: "Mark Howard"
	date: "$Date: 2010/10/21 $"
	last_editor: "$Author: tg $"
	revision: "$Revision: #17 $"
	change: "$Change: 151679 $"
	archive: "$File: //rose/source/kernel/data/market/price_record.e $"

class
	PRICE_RECORD
	
inherit
	ROSE_LEGACY_UNDEFINED
		undefine
			is_equal
		end
	ROSE_ACCESS
		redefine
			is_equal
		end
	
feature -- Basic Access
	
	open : DOUBLE is
			-- open price
		do
			if has_open and is_defined_double(adjustment) then
				Result := unadjusted_open / adjustment
			else
				Result := undefined_double
			end
		end
	close : DOUBLE is
			-- close price
		do
			if has_close and is_defined_double(adjustment) then
				Result := unadjusted_close / adjustment
			else
				Result := undefined_double
			end
		end
	high : DOUBLE is
			-- high price
		do
			if has_high and is_defined_double(adjustment) then
				Result := unadjusted_high / adjustment
			else
				Result := undefined_double
			end
		end

	high_ask : DOUBLE is
			-- high_ask price
		do
			if is_defined_double(adjustment)
			then
				if has_high
				then
					Result := unadjusted_high / adjustment
				elseif has_ask
				then
					Result := unadjusted_ask / adjustment
				else
					Result := Undefined_double
				end
			else
				Result := undefined_double
			end
		end

	low : DOUBLE is
			-- low price
		do
			if has_low and is_defined_double(adjustment) then
				Result := unadjusted_low / adjustment
			else
				Result := undefined_double
			end
		end
	bid : DOUBLE is
			-- bid price
		do
			if has_bid and is_defined_double(adjustment) then
				Result := unadjusted_bid / adjustment
			else
				Result := undefined_double
			end
		end
	ask : DOUBLE is
			-- ask price
		do
			if has_ask and is_defined_double(adjustment) then
				Result := unadjusted_ask / adjustment
			else
				Result := undefined_double
			end
		end
	volume : DOUBLE is
			-- volume price
		do
			if has_volume and is_defined_double(adjustment) then
				Result := unadjusted_volume * adjustment
			else
				Result := undefined_double
			end
		end
	settlement : DOUBLE is
			-- settlement price
		do
			if has_settlement then
				Result := unadjusted_settlement
			else
				Result := undefined_double
			end
		end

	unadjusted_close_bid : DOUBLE is
			-- unadjusted_close_bid price
		do
			if has_close then
				Result := unadjusted_close
			elseif has_bid then
				Result := unadjusted_bid
			else
				Result := undefined_double
			end
		end

	unadjusted_close_bid_ask : DOUBLE is
			-- unadjusted_close_bid_ask price
		do
			has_close_bid_ask := True
			if has_close then
				Result := unadjusted_close
			elseif has_bid then
				Result := unadjusted_bid
			elseif has_ask then
				Result := unadjusted_ask
			else
				Result := undefined_double
				has_close_bid_ask := False
			end
		end
	unadjusted_closing_price : DOUBLE is
			-- unadjusted_closing_price price
		do
			if has_close  then
				Result := unadjusted_close
			else
				Result := undefined_double
			end
		end		

	close_bid : DOUBLE is
			-- close_bid price
		do
			if is_defined_double(adjustment)
			then
				if has_close
				then
					Result := unadjusted_close / adjustment
				elseif has_bid
				then
					Result := unadjusted_bid / adjustment
				else
					Result := undefined_double
				end
			else
				Result := undefined_double
			end
		end

	low_bid : DOUBLE is
			-- low_bid price
		do
			if is_defined_double(adjustment)
			then
				if has_low
				then
					Result := unadjusted_low / adjustment
				elseif has_bid
				then
					Result := unadjusted_bid / adjustment
				else
					Result := undefined_double
				end
			else
				Result := undefined_double
			end
		end

	close_bid_ask : DOUBLE is
			-- close_bid_ask price
		do
			if has_close_bid_ask and is_defined_double(adjustment) then
				Result := unadjusted_close_bid_ask / adjustment
			else
				Result := undefined_double
			end
		end
feature -- Attribute Setters
	set_validity (a_boolean : BOOLEAN) is
		do
			is_valid := a_boolean
		end
	
	set_currency(a_currency : CURRENCY) is
		do
			currency := a_currency
		end
	
	set_as_of_date (a_value : ROSE_DATE) is
		do
			as_of_date := a_value
		end

feature -- Attributes
	
	date : ROSE_DATE
	is_valid : BOOLEAN
	currency : CURRENCY
	as_of_date : ROSE_DATE
	adjustment : DOUBLE
	unadjusted_open : DOUBLE
	unadjusted_close : DOUBLE
	unadjusted_high : DOUBLE
	unadjusted_low : DOUBLE
	unadjusted_bid : DOUBLE
	unadjusted_ask : DOUBLE
	unadjusted_volume : DOUBLE
	unadjusted_settlement : DOUBLE
	has_open : BOOLEAN
	has_close : BOOLEAN
	has_high : BOOLEAN
	has_low : BOOLEAN
	has_bid : BOOLEAN
	has_ask : BOOLEAN
	has_volume : BOOLEAN
	has_close_bid_ask : BOOLEAN
	has_settlement : BOOLEAN

feature -- Comparison
	
	is_equal(a_other : like Current) : BOOLEAN is
		do
			Result := 
				is_valid = a_other.is_valid and
				currency = a_other.currency and
				as_of_date = a_other.as_of_date and
				adjustment = a_other.adjustment and
				has_open = a_other.has_open and
				has_close = a_other.has_close and
				has_high = a_other.has_high and
				has_low = a_other.has_low and
				has_bid = a_other.has_bid and
				has_ask = a_other.has_ask and
				has_volume = a_other.has_volume and
				has_close_bid_ask = a_other.has_close_bid_ask and
				has_settlement = a_other.has_settlement and
				data_is_equal(a_other)
		end
	
	data_is_equal(a_other : like Current) : BOOLEAN is
		do
			Result := unadjusted_open = a_other.unadjusted_open and
				unadjusted_close = a_other.unadjusted_close and
				unadjusted_high = a_other.unadjusted_high and
				unadjusted_low = a_other.unadjusted_low and
				unadjusted_bid = a_other.unadjusted_bid and
				unadjusted_ask = a_other.unadjusted_ask and
				unadjusted_settlement = a_other.unadjusted_settlement and
				unadjusted_volume = a_other.unadjusted_volume
		end
	
	is_close_enough(a_other : like Current; a_tolerance : DOUBLE) : BOOLEAN is
		do
			Result := 
				is_valid = a_other.is_valid and
				currency = a_other.currency and
				as_of_date.is_equal(a_other.as_of_date) and
				adjustment = a_other.adjustment and
				unadjusted_volume = a_other.unadjusted_volume and
				has_open = a_other.has_open and
				has_close = a_other.has_close and
				has_high = a_other.has_high and
				has_low = a_other.has_low and
				has_bid = a_other.has_bid and
				has_ask = a_other.has_ask and
				has_volume = a_other.has_volume and
				has_settlement = a_other.has_settlement and
				has_close_bid_ask = a_other.has_close_bid_ask
			Result := Result and (unadjusted_open < a_other.unadjusted_open + a_tolerance)
			Result := Result and (unadjusted_close < a_other.unadjusted_close + a_tolerance)
			Result := Result and (unadjusted_high < a_other.unadjusted_high + a_tolerance)
			Result := Result and (unadjusted_low < a_other.unadjusted_low+ a_tolerance)
			Result := Result and (unadjusted_bid < a_other.unadjusted_bid + a_tolerance)
			Result := Result and (unadjusted_ask < a_other.unadjusted_ask + a_tolerance)
			Result := Result and (unadjusted_open > a_other.unadjusted_open - a_tolerance)
			Result := Result and (unadjusted_close > a_other.unadjusted_close - a_tolerance)
			Result := Result and (unadjusted_high > a_other.unadjusted_high - a_tolerance)
			Result := Result and (unadjusted_low > a_other.unadjusted_low - a_tolerance)
			Result := Result and (unadjusted_bid > a_other.unadjusted_bid - a_tolerance)
			Result := Result and (unadjusted_ask > a_other.unadjusted_ask - a_tolerance)
			Result := Result and (unadjusted_settlement > a_other.unadjusted_settlement - a_tolerance)
		end
	
invariant
	adjustment_is_non_zero: adjustment /= 0.0
end
