indexing
	description: "Common ancestor to instruments with a price"
	author: "Mark Howard"
	date: "$Date: 2010/02/05 $"
	revision: "$Revision: #12 $"
	archive: "$File: //rose/source/kernel/data/market/priceable.e $"

deferred class
	PRICEABLE

inherit
	ANY
		undefine
			is_equal
		end
	ROSE_LEGACY_UNDEFINED
		undefine
			is_equal
		end

feature -- Access

	last_price_record (a_date: ROSE_DATE): PRICE_RECORD is
			-- Last price record as of 'a_date' (most recent price record available on or before 'a_date')
		deferred
		end

	last_price_record_160 (a_date: ROSE_DATE): PRICE_RECORD is
			-- Last price record as of 'a_date' (most recent price record available on or before 'a_date')
			-- using mode 160 fallback (if available)
		deferred
		end

	price_record (a_date: ROSE_DATE): PRICE_RECORD is
			-- Price record at 'a_date'
		deferred
		end

	last_price (a_date: ROSE_DATE):  DOUBLE is
			-- Last price as of 'a_date' (most recent price available on or before 'a_date')
			-- Returns 0 if no price record available
		local
			l_record: PRICE_RECORD
		do
			l_record := last_price_record (a_date)
			if l_record /= Void then
				Result := l_record.close
			end
		end

	last_price_currency (a_date: ROSE_DATE): CURRENCY is
			-- currency of last_price
		local
			l_record: PRICE_RECORD
		do
			l_record := last_price_record (a_date)
			if l_record /= Void then
				Result := l_record.currency
			end
		end


	unadjusted_last_close_bid_price (a_date: ROSE_DATE):  DOUBLE is
			-- Last price as of 'a_date' (most recent price available on or before 'a_date')
			-- Returns 0 if no price record available
		local
			l_record: PRICE_RECORD
		do
			l_record := last_price_record (a_date)
			if l_record /= Void then
				Result := l_record.unadjusted_close_bid
			end
		end

	last_price_date (a_date: ROSE_DATE): ROSE_DATE is
			-- Actual date of most recent price available on or before 'a_date'
			-- Returns Void if no price record available
		local
			l_record: PRICE_RECORD
		do
			l_record := last_price_record (a_date)
			if l_record /= Void then
				Result := l_record.date
			end
		end

	closing_price (a_date: ROSE_DATE): DOUBLE is
			-- closing price
		do
			Result := price_record (a_date).close
		end

	opening_price (a_date: ROSE_DATE): DOUBLE is
			-- opening price
		do
			Result := price_record (a_date).open
		end

	bid (a_date: ROSE_DATE): DOUBLE is
			-- bid price
		do
			Result := price_record (a_date).bid
		end

	ask (a_date: ROSE_DATE): DOUBLE is
			-- ask price
		do
			Result := price_record (a_date).ask
		end

	high_ask (a_date: ROSE_DATE): DOUBLE is
			-- high ask price
		do
			Result := price_record (a_date).high_ask
		end

	high, hiprc (a_date: ROSE_DATE): DOUBLE is
			-- high price
		do
			Result := price_record (a_date).high
		end

	low, loprc (a_date: ROSE_DATE): DOUBLE is
			-- low price
		do
			Result := price_record (a_date).low
		end

	low_bid (a_date: ROSE_DATE): DOUBLE is
			-- low bid price
		do
			Result := price_record (a_date).low_bid
		end

	adjustment (a_date: ROSE_DATE): DOUBLE is
			-- adjustment
		do
			Result := price_record (a_date).adjustment
		end

	volume (a_date: ROSE_DATE): DOUBLE is
			-- volume
		do
			Result := price_record (a_date).volume
		end

	settlement (a_date: ROSE_DATE) : DOUBLE is
			-- settlement
		do
			Result := price_record (a_date).settlement
		end

	unadjusted_last_price (a_date: ROSE_DATE): DOUBLE is
			-- unadjusted last price
		do
			Result := last_price_record (a_date).unadjusted_closing_price 
		end

	unadjusted_closing_price (a_date: ROSE_DATE): DOUBLE is
			-- unadjusted closing price
		do
			Result := price_record (a_date).unadjusted_closing_price
		end

	unadjusted_close_bid (a_date: ROSE_DATE): DOUBLE is
			-- unadjusted closing bid
		do
			Result := price_record (a_date).unadjusted_close_bid
		end

	unadjusted_volume (a_date: ROSE_DATE): DOUBLE is
			-- unadjusted volume
		do
			Result := price_record (a_date).unadjusted_volume 
		end

	unadjusted_bid (a_date: ROSE_DATE): DOUBLE is
			-- unadjusted bid
		do
			Result := price_record (a_date).unadjusted_bid
		end

	unadjusted_ask (a_date: ROSE_DATE): DOUBLE is
			-- unadjusted ask
		do
			Result := price_record (a_date).unadjusted_ask
		end

	unadjusted_high (a_date: ROSE_DATE): DOUBLE is
			-- unadjusted high
		do
			Result := price_record (a_date).unadjusted_high
		end

	unadjusted_low (a_date: ROSE_DATE): DOUBLE is
			-- unadjusted low
		do
			Result := price_record (a_date).unadjusted_low
		end

	unadjusted_open (a_date: ROSE_DATE): DOUBLE is
			-- unadjusted open
		do
			Result := price_record (a_date).unadjusted_open
		end

feature -- series features

	price_series(a_dates : ARRAY[ROSE_DATE]; a_price_name : STRING) : DOUBLE_TIME_SERIES is
			-- return time series for 'a_price_name' for a_dates
		do
			if a_price_name.as_lower.is_equal("closing_price") then
				Result := price_series_with_agent(a_dates, Agent closing_price)
			elseif a_price_name.as_lower.is_equal("opening_price") then
				Result := price_series_with_agent(a_dates, Agent opening_price)
			elseif a_price_name.as_lower.is_equal("bid") then
				Result := price_series_with_agent(a_dates, Agent bid)
			elseif a_price_name.as_lower.is_equal("ask") then
				Result := price_series_with_agent(a_dates, Agent ask)
			elseif a_price_name.as_lower.is_equal("high_ask") then
				Result := price_series_with_agent(a_dates, Agent high_ask)
			elseif a_price_name.as_lower.is_equal("high") or a_price_name.as_lower.is_equal("hiprc") then
				Result := price_series_with_agent(a_dates, Agent high)
			elseif a_price_name.as_lower.is_equal("low") or a_price_name.as_lower.is_equal("loprc")  then
				Result := price_series_with_agent(a_dates, Agent low)
			elseif a_price_name.as_lower.is_equal("low_bid") then
				Result := price_series_with_agent(a_dates, Agent low_bid)
			elseif a_price_name.as_lower.is_equal("adjustment") then
				Result := price_series_with_agent(a_dates, Agent adjustment)
			elseif a_price_name.as_lower.is_equal("volume") then
				Result := price_series_with_agent(a_dates, Agent volume)
			elseif a_price_name.as_lower.is_equal("settlement") then
				Result := price_series_with_agent(a_dates, Agent settlement)
			elseif a_price_name.as_lower.is_equal("unadjusted_last_price") then
				Result := price_series_with_agent(a_dates, Agent unadjusted_last_price)
			elseif a_price_name.as_lower.is_equal("unadjusted_closing_price") then
				Result := price_series_with_agent(a_dates, Agent unadjusted_closing_price)
			elseif a_price_name.as_lower.is_equal("unadjusted_close_bid") then
				Result := price_series_with_agent(a_dates, Agent unadjusted_close_bid)
			elseif a_price_name.as_lower.is_equal("unadjusted_volume") then
				Result := price_series_with_agent(a_dates, Agent unadjusted_volume)
			end
		end

	price_series_with_agent(a_dates : ARRAY[ROSE_DATE]; a_agent : FUNCTION[ANY, TUPLE[ROSE_DATE], DOUBLE]) : DOUBLE_TIME_SERIES is
			-- price series for 'a_dates' using 'a_agent'
		local
			i : INTEGER
			l_value : DOUBLE
			l_date : ROSE_DATE
		do
			create Result.make_empty
			from i := a_dates.lower until i > a_dates.upper loop
				l_date := a_dates.item(i)
				l_value := a_agent.item ([l_date])
				if is_defined_double (l_value) and then l_value > 0 then
					Result.extend(l_value, l_date)
				end
				i := i + 1
			end
		end

end
