indexing
	description: "Object to access windows process memory structure data for 64-bit programs (see PROCESS_MEMORY_COUNTERS_32)."
	author: "Mauricio Bustos"
	last_editor: "$Author: rj $"
	date: "$Date: 2014/08/05 $"
	revision: "$Revision: #7 $"
	path: "$File: //rose/source/kernel/infrastructure/windows/process_memory_counters.e $"
	storable_version: "20140731"

class
	PROCESS_MEMORY_COUNTERS

inherit
	MISMATCH_CORRECTOR
		redefine
			correct_mismatch
		end
	EXTERNAL_RECORD
		redefine
			display,
			write,
			read,
			record_length_
		end
	C_ACCESS_SUPPORT

create
	make_from_counter_32,
	make_from_c_struct_process_memory_counter,
	default_create

feature {NONE} -- Creation

	make_from_c_struct_process_memory_counter (a_counter: C_STRUCT_PROCESS_MEMORY_COUNTERS) is
			-- Create and initialize using 'a_counter'
		require
			valid_counter: a_counter /= Void
		do
			cb := a_counter.cb
			pagefaultcount := a_counter.pagefaultcount.to_natural_32
			peakworkingsetsize := a_counter.peakworkingsetsize.to_integer_64
			workingsetsize := a_counter.workingsetsize.to_integer_64
			quotapeakpagedpoolusage := a_counter.quotapeakpagedpoolusage.to_integer_64
			quotapagedpoolusage := a_counter.quotapagedpoolusage.to_integer_64
			quotapeaknonpagedpoolusage := a_counter.quotapeaknonpagedpoolusage.to_integer_64
			quotanonpagedpoolusage := a_counter.quotanonpagedpoolusage.to_integer_64
			pagefileusage := a_counter.pagefileusage.to_integer_64
			peakpagefileusage := a_counter.peakpagefileusage.to_integer_64
		end

	make_from_counter_32 (a_counter: PROCESS_MEMORY_COUNTERS_32) is
			-- Create and initialize using 'a_counter'
		require
			valid_counter: a_counter /= Void
		do
			cb := a_counter.cb
			pagefaultcount := a_counter.pagefaultcount
			peakworkingsetsize := a_counter.peakworkingsetsize
			workingsetsize := a_counter.workingsetsize
			quotapeakpagedpoolusage := a_counter.quotapeakpagedpoolusage
			quotapagedpoolusage := a_counter.quotapagedpoolusage
			quotapeaknonpagedpoolusage := a_counter.quotapeaknonpagedpoolusage
			quotanonpagedpoolusage := a_counter.quotanonpagedpoolusage
			pagefileusage := a_counter.pagefileusage
			peakpagefileusage := a_counter.peakpagefileusage
		end

feature -- Basic operations

	read is
		do
			cb := read_cb
			pagefaultcount := read_pagefaultcount
			peakworkingsetsize := read_peakworkingsetsize
			workingsetsize := read_workingsetsize
			quotapeakpagedpoolusage := read_quotapeakpagedpoolusage
			quotapagedpoolusage := read_quotapagedpoolusage
			quotapeaknonpagedpoolusage := read_quotapeaknonpagedpoolusage
			quotanonpagedpoolusage := read_quotanonpagedpoolusage
			pagefileusage := read_pagefileusage
			peakpagefileusage := read_peakpagefileusage
		end

	write is
		do
			write_cb (cb)
			write_pagefaultcount (pagefaultcount)
			write_peakworkingsetsize (peakworkingsetsize)
			write_workingsetsize (workingsetsize)
			write_quotapeakpagedpoolusage (quotapeakpagedpoolusage)
			write_quotapagedpoolusage (quotapagedpoolusage)
			write_quotapeaknonpagedpoolusage (quotapeaknonpagedpoolusage)
			write_quotanonpagedpoolusage (quotanonpagedpoolusage)
			write_pagefileusage (pagefileusage)
			write_peakpagefileusage (peakpagefileusage)
		end

	display is
		do
			print_all (<<"cb",": ",read_cb,"%N">>)
			print_all (<<"pagefaultcount",": ",read_pagefaultcount,"%N">>)
			print_all (<<"peakworkingsetsize",": ",read_peakworkingsetsize,"%N">>)
			print_all (<<"workingsetsize",": ",read_workingsetsize,"%N">>)
			print_all (<<"quotapeakpagedpoolusage",": ",read_quotapeakpagedpoolusage,"%N">>)
			print_all (<<"quotapagedpoolusage",": ",read_quotapagedpoolusage,"%N">>)
			print_all (<<"quotapeaknonpagedpoolusage",": ",read_quotapeaknonpagedpoolusage,"%N">>)
			print_all (<<"quotanonpagedpoolusage",": ",read_quotanonpagedpoolusage,"%N">>)
			print_all (<<"pagefileusage",": ",read_pagefileusage,"%N">>)
			print_all (<<"peakpagefileusage",": ",read_peakpagefileusage,"%N">>)
		end

feature -- Access

	record_length_: INTEGER
			-- The size of this record in bytes.
		do
			Result := 72
		end

	cb: INTEGER
	pagefaultcount: NATURAL_32
	peakworkingsetsize: INTEGER_64
	workingsetsize: INTEGER_64
	quotapeakpagedpoolusage: INTEGER_64
	quotapagedpoolusage: INTEGER_64
	quotapeaknonpagedpoolusage: INTEGER_64
	quotanonpagedpoolusage: INTEGER_64
	pagefileusage: INTEGER_64
	peakpagefileusage: INTEGER_64

feature -- Element modification

	set_cb (a_value: INTEGER) is
		do
			cb := a_value
		end

	set_pagefaultcount (a_value: NATURAL_32) is
		do
			pagefaultcount := a_value
		end

	set_peakworkingsetsize (a_value: INTEGER_64) is
		do
			peakworkingsetsize := a_value
		end

	set_workingsetsize (a_value: INTEGER_64) is
		do
			workingsetsize := a_value
		end

	set_quotapeakpagedpoolusage (a_value: INTEGER_64) is
		do
			quotapeakpagedpoolusage := a_value
		end

	set_quotapagedpoolusage (a_value: INTEGER_64) is
		do
			quotapagedpoolusage := a_value
		end

	set_quotapeaknonpagedpoolusage (a_value: INTEGER_64) is
		do
			quotapeaknonpagedpoolusage := a_value
		end

	set_quotanonpagedpoolusage (a_value: INTEGER_64) is
		do
			quotanonpagedpoolusage := a_value
		end

	set_pagefileusage (a_value: INTEGER_64) is
		do
			pagefileusage := a_value
		end

	set_peakpagefileusage (a_value: INTEGER_64) is
		do
			peakpagefileusage := a_value
		end

feature {NONE} -- Implementation

	read_cb: INTEGER is
		do
			Result := integer_interpreter_.decode (pointer_ + (0),4)
		end

	write_cb (a_value: INTEGER) is
		do
			integer_interpreter_.encode (pointer_ + (0),a_value,4)
		end

	read_pagefaultcount: NATURAL_32 is
		do
			Result := natural_32_interpreter_.decode (pointer_ + (4),4)
		end

	write_pagefaultcount (a_value: NATURAL_32) is
		do
			natural_32_interpreter_.encode (pointer_ + (4),a_value,4)
		end

	read_peakworkingsetsize: INTEGER_64 is
		do
			Result := integer_64_interpreter_.decode (pointer_ + (8),8)
		end

	write_peakworkingsetsize (a_value: INTEGER_64) is
		do
			integer_64_interpreter_.encode (pointer_ + (8),a_value,8)
		end

	read_workingsetsize: INTEGER_64 is
		do
			Result := integer_64_interpreter_.decode (pointer_ + (16),8)
		end

	write_workingsetsize (a_value: INTEGER_64) is
		do
			integer_64_interpreter_.encode (pointer_ + (16),a_value,8)
		end

	read_quotapeakpagedpoolusage: INTEGER_64 is
		do
			Result := integer_64_interpreter_.decode (pointer_ + (24),8)
		end

	write_quotapeakpagedpoolusage (a_value: INTEGER_64) is
		do
			integer_64_interpreter_.encode (pointer_ + (24),a_value,8)
		end

	read_quotapagedpoolusage: INTEGER_64 is
		do
			Result := integer_64_interpreter_.decode (pointer_ + (32),8)
		end

	write_quotapagedpoolusage (a_value: INTEGER_64) is
		do
			integer_64_interpreter_.encode (pointer_ + (32),a_value,8)
		end

	read_quotapeaknonpagedpoolusage: INTEGER_64 is
		do
			Result := integer_64_interpreter_.decode (pointer_ + (40),8)
		end

	write_quotapeaknonpagedpoolusage (a_value: INTEGER_64) is
		do
			integer_64_interpreter_.encode (pointer_ + (40),a_value,8)
		end

	read_quotanonpagedpoolusage: INTEGER_64 is
		do
			Result := integer_64_interpreter_.decode (pointer_ + (48),8)
		end

	write_quotanonpagedpoolusage (a_value: INTEGER_64) is
		do
			integer_64_interpreter_.encode (pointer_ + (48),a_value,8)
		end

	read_pagefileusage: INTEGER_64 is
		do
			Result := integer_64_interpreter_.decode (pointer_ + (56),8)
		end

	write_pagefileusage (a_value: INTEGER_64) is
		do
			integer_64_interpreter_.encode (pointer_ + (56),a_value,8)
		end

	read_peakpagefileusage: INTEGER_64 is
		do
			Result := integer_64_interpreter_.decode (pointer_ + (64),8)
		end

	write_peakpagefileusage (a_value: INTEGER_64) is
		do
			integer_64_interpreter_.encode (pointer_ + (64),a_value,8)
		end

	correct_mismatch
			-- <Precursor>
		do
			if not attached mismatch_information.stored_version as l_stored_version or else l_stored_version.is_empty
			then
				if attached {INTEGER_32_REF} mismatch_information.item ("pagefaultcount") as al_int_32_ref
				then
					-- Older programs are still using INTEGER_32.
					if al_int_32_ref.item >= 0
					then
						pagefaultcount := al_int_32_ref.item.to_natural_32
					end
				else
					Precursor
				end
			else
				Precursor
			end
		end

end

