indexing
	description: "Object to access windows process memory structure data for 32-bit programs (see PROCESS_MEMORY_COUNTERS)."
	author: "Mauricio Bustos"
	last_editor: "$Author: rj $"
	date: "$Date: 2014/06/19 $"
	revision: "$Revision: #4 $"
	path: "$File: //rose/source/kernel/infrastructure/windows/process_memory_counters_32.e $"

class
	PROCESS_MEMORY_COUNTERS_32

inherit
	EXTERNAL_RECORD
		redefine
			display,
			write,
			read,
			record_length_
		end
	C_ACCESS_SUPPORT

feature -- Basic operations

	read is
		do
			cb := read_cb
			pagefaultcount := read_pagefaultcount
			peakworkingsetsize := read_peakworkingsetsize
			workingsetsize := read_workingsetsize
			quotapeakpagedpoolusage := read_quotapeakpagedpoolusage
			quotapagedpoolusage := read_quotapagedpoolusage
			quotapeaknonpagedpoolusage := read_quotapeaknonpagedpoolusage
			quotanonpagedpoolusage := read_quotanonpagedpoolusage
			pagefileusage := read_pagefileusage
			peakpagefileusage := read_peakpagefileusage
		end

	write is
		do
			write_cb(cb)
			write_pagefaultcount(pagefaultcount)
			write_peakworkingsetsize(peakworkingsetsize)
			write_workingsetsize(workingsetsize)
			write_quotapeakpagedpoolusage(quotapeakpagedpoolusage)
			write_quotapagedpoolusage(quotapagedpoolusage)
			write_quotapeaknonpagedpoolusage(quotapeaknonpagedpoolusage)
			write_quotanonpagedpoolusage(quotanonpagedpoolusage)
			write_pagefileusage(pagefileusage)
			write_peakpagefileusage(peakpagefileusage)
		end

	record_length_: INTEGER
			-- The size of this record in bytes.
		do
			Result := 40
		end

	cb : INTEGER
	read_cb : INTEGER is
		do
			Result := integer_interpreter_.decode(pointer_ + (0),4)
		end
	write_cb(a_value : INTEGER) is
		do
			integer_interpreter_.encode(pointer_ + (0),a_value,4)
		end
	set_cb(a_value : INTEGER) is
		do
			cb := a_value
		end

	pagefaultcount : NATURAL_32
	read_pagefaultcount : NATURAL_32 is
		do
			Result := natural_32_interpreter_.decode(pointer_ + (4),4)
		end
	write_pagefaultcount(a_value : NATURAL_32) is
		do
			natural_32_interpreter_.encode(pointer_ + (4),a_value,4)
		end
	set_pagefaultcount(a_value : NATURAL_32) is
		do
			pagefaultcount := a_value
		end

	peakworkingsetsize : NATURAL_32
	read_peakworkingsetsize : NATURAL_32 is
		do
			Result := natural_32_interpreter_.decode(pointer_ + (8),4)
		end
	write_peakworkingsetsize(a_value : NATURAL_32) is
		do
			natural_32_interpreter_.encode(pointer_ + (8),a_value,4)
		end
	set_peakworkingsetsize(a_value : NATURAL_32) is
		do
			peakworkingsetsize := a_value
		end

	workingsetsize : NATURAL_32
	read_workingsetsize : NATURAL_32 is
		do
			Result := natural_32_interpreter_.decode(pointer_ + (12),4)
		end
	write_workingsetsize(a_value : NATURAL_32) is
		do
			natural_32_interpreter_.encode(pointer_ + (12),a_value,4)
		end
	set_workingsetsize(a_value : NATURAL_32) is
		do
			workingsetsize := a_value
		end

	quotapeakpagedpoolusage : NATURAL_32
	read_quotapeakpagedpoolusage : NATURAL_32 is
		do
			Result := natural_32_interpreter_.decode(pointer_ + (16),4)
		end
	write_quotapeakpagedpoolusage(a_value : NATURAL_32) is
		do
			natural_32_interpreter_.encode(pointer_ + (16),a_value,4)
		end
	set_quotapeakpagedpoolusage(a_value : NATURAL_32) is
		do
			quotapeakpagedpoolusage := a_value
		end

	quotapagedpoolusage : NATURAL_32
	read_quotapagedpoolusage : NATURAL_32 is
		do
			Result := natural_32_interpreter_.decode(pointer_ + (20),4)
		end
	write_quotapagedpoolusage(a_value : NATURAL_32) is
		do
			natural_32_interpreter_.encode(pointer_ + (20),a_value,4)
		end
	set_quotapagedpoolusage(a_value : NATURAL_32) is
		do
			quotapagedpoolusage := a_value
		end

	quotapeaknonpagedpoolusage : NATURAL_32
	read_quotapeaknonpagedpoolusage : NATURAL_32 is
		do
			Result := natural_32_interpreter_.decode(pointer_ + (24),4)
		end
	write_quotapeaknonpagedpoolusage(a_value : NATURAL_32) is
		do
			natural_32_interpreter_.encode(pointer_ + (24),a_value,4)
		end
	set_quotapeaknonpagedpoolusage(a_value : NATURAL_32) is
		do
			quotapeaknonpagedpoolusage := a_value
		end

	quotanonpagedpoolusage : NATURAL_32
	read_quotanonpagedpoolusage : NATURAL_32 is
		do
			Result := natural_32_interpreter_.decode(pointer_ + (28),4)
		end
	write_quotanonpagedpoolusage(a_value : NATURAL_32) is
		do
			natural_32_interpreter_.encode(pointer_ + (28),a_value,4)
		end
	set_quotanonpagedpoolusage(a_value : NATURAL_32) is
		do
			quotanonpagedpoolusage := a_value
		end

	pagefileusage : NATURAL_32
	read_pagefileusage : NATURAL_32 is
		do
			Result := natural_32_interpreter_.decode(pointer_ + (32),4)
		end
	write_pagefileusage(a_value : NATURAL_32) is
		do
			natural_32_interpreter_.encode(pointer_ + (32),a_value,4)
		end
	set_pagefileusage(a_value : NATURAL_32) is
		do
			pagefileusage := a_value
		end

	peakpagefileusage : NATURAL_32
	read_peakpagefileusage : NATURAL_32 is
		do
			Result := natural_32_interpreter_.decode(pointer_ + (36),4)
		end
	write_peakpagefileusage(a_value : NATURAL_32) is
		do
			natural_32_interpreter_.encode(pointer_ + (36),a_value,4)
		end
	set_peakpagefileusage(a_value : NATURAL_32) is
		do
			peakpagefileusage := a_value
		end

	display is
		do
			print_all (<<"cb",": ",read_cb,"%N">>)
			print_all (<<"pagefaultcount",": ",read_pagefaultcount,"%N">>)
			print_all (<<"peakworkingsetsize",": ",read_peakworkingsetsize,"%N">>)
			print_all (<<"workingsetsize",": ",read_workingsetsize,"%N">>)
			print_all (<<"quotapeakpagedpoolusage",": ",read_quotapeakpagedpoolusage,"%N">>)
			print_all (<<"quotapagedpoolusage",": ",read_quotapagedpoolusage,"%N">>)
			print_all (<<"quotapeaknonpagedpoolusage",": ",read_quotapeaknonpagedpoolusage,"%N">>)
			print_all (<<"quotanonpagedpoolusage",": ",read_quotanonpagedpoolusage,"%N">>)
			print_all (<<"pagefileusage",": ",read_pagefileusage,"%N">>)
			print_all (<<"peakpagefileusage",": ",read_peakpagefileusage,"%N">>)
		end

end

