indexing
	
	description: "Definition of a rational number"

	author: "Mark Howard"
	date: "$Date: 2014/05/13 $"
	revision: "$Revision: #16 $"
	storable_version: ""

class	RATIONAL

inherit

	RP_SERIALIZABLE

	NUMERIC
		undefine
			is_equal,
			out, default_create
		select
			out
		end
	
	KEY_PAIR [INTEGER, INTEGER]
		rename
			first as numerator,
			second as denominator
		redefine
			make, out
		end

create
	make, make_from_string

feature -- Initialization

	make (a_numerator, a_denominator: INTEGER) is
		local
			l_gcd: INTEGER
		do
			l_gcd := gcd (a_numerator.abs,a_denominator.abs).max(1)
			numerator := a_numerator // l_gcd
			denominator := a_denominator // l_gcd
		end

	make_from_string (a_string: STRING) is
		require
			valid_string: a_string /= Void
		local
			l_numerator,l_denominator: STRING
			l_index: INTEGER
		do
			a_string.left_adjust
			a_string.right_adjust
			l_index := a_string.index_of ('/',1)
			if l_index = 0 then
				l_numerator := a_string
				l_denominator := "1"
			else
				l_numerator := a_string.substring (1,l_index-1)
				l_denominator := a_string.substring (l_index+1,a_string.count)
			end
			if l_numerator.is_integer and l_denominator.is_integer then
				make (l_numerator.to_integer,l_denominator.to_integer)
			end
		end

feature -- Access

	gcd (i, j: INTEGER): INTEGER is
			-- calculate greatest common divisor
		require
			non_negative_arguments: i >=0 and j >= 0
		do
			if i = j then
				Result := i
			elseif i = 1 or j = 1 then
				Result := 1
			elseif i = 0 then
				Result := j
			elseif j = 0 then
				Result := i
			elseif i < j then
				Result := gcd (i,j\\i)
			else
				Result := gcd (j,i\\j)
			end
		end

	zero: RATIONAL is
		once
			create Result.make (0,1)
		end

	 one: RATIONAL is
		once
			create Result.make (1,1)
		end

	to_double: DOUBLE is
		do
			if denominator /= 0 then
				Result := numerator / denominator
			end
		end

	infix "+" (a_other: like Current): like Current is
		do
			create Result.make (numerator * a_other.denominator + a_other.numerator * denominator,denominator * a_other.denominator)
		end

	infix "-" (a_other: like Current): like Current is
		do
			create Result.make (numerator * a_other.denominator - a_other.numerator * denominator,denominator * a_other.denominator)
		end

	infix "*" (a_other: like Current): like Current is
		do
			create Result.make (numerator * a_other.numerator,denominator * a_other.denominator)
		end

	infix "/" (a_other: like Current): like Current is
		do
			create Result.make (numerator * a_other.denominator, a_other.numerator * denominator)
		end

	prefix "-": like Current is
		do
			create Result.make (-numerator,denominator)
		end

	prefix "+": like Current is
		do
			Result := Current
		end

feature -- Status report

	divisible (a_other: like Current): BOOLEAN is
		do
			Result := a_other.numerator /= 0
		end

	exponentiable (a_other: like Current): BOOLEAN is
		do
			Result := False
		end

	out: STRING is do
		Result := numerator.out + "/" + denominator.out
	end

end -- class RATIONAL
