indexing
	description: "Display info on current application"
	author: "Zoran Simic"
	date: "$Date: 2009/08/07 $"
	revision: "$Revision: #9 $"

class
	RCMD_ABOUT

inherit
	ROSE_COMMAND_GENERAL

feature -- Access

	id: STRING is "about"
			-- Command id

	signature: STRING is "verbose:? comment:? config:? commands:? io:?"
			-- Argument signature of this command (defines valid parameters to be passed to the command)

feature -- Output

	help: STRING is
			-- Help on this command
"[
Display information about current application (name and version)
-verbose    Display all optional information below
-comment    Display pilot description
-config     Display config path and details
-commands   Display brief command information
-io         Display status of framework I/O channels
]"

feature {NONE} -- Implementation

	do_execute is
			-- Execute effectively the command.
		local
			l_verbose: BOOLEAN
		do
			output ("$1%N",<<Global.app.name_version>>)
			l_verbose := parameters.boolean ("verbose")
			if l_verbose or else parameters.boolean ("comment") then
				output ("$1%N",<<Global.app.description>>)
			end
			if l_verbose then
				output ("$1%N",<<Global.app.report>>)
			end
			if l_verbose or else parameters.boolean ("config") then
				output ("configuration : $1 [$2]%N",<<config.path, config_details (config)>>)
				output ("delivery: $1 [$2]%N",<<global.app.delivery.folder, config_details (global.app.delivery)>>)
				output ("site: $1 [$2]%N",<<global.app.site.folder, config_details (global.app.site)>>)
			end
			if l_verbose or else parameters.boolean ("commands") then
				output ("$1 commands in $2 categories (type help for info)%N",<<Global.commands.count,Global.commands.category_count>>)
			end
			if l_verbose or else parameters.boolean ("io") then
				output ("input on: $1%N",<<Global.io_handler.input>>)
				output ("output on: $1%N",<<Global.io_handler.output>>)
				output ("echo on: $1%N",<<Global.io_handler.echo>>)
				output ("logging on: $1%N",<<Global.io_handler.log>>)
				output ("monitoring on: $1%N",<<Global.io_handler.monitor>>)
			end
		end

	config_details (a_config: ROSE_CONFIGURATION): STRING is
			-- Details about 'a_config'
		do
			if a_config /= Void then
				create Result.make (20)
				if a_config.is_modified then
					Result.append ("*modified* ")
				end
				Result.append (a_config.count.out)
				Result.append (" leaves")
			end
		end

end
