indexing
	description: "Command alias display and/or set"
	author: "Zoran Simic"
	date: "$Date: 2008/12/01 $"
	revision: "$Revision: #5 $"

class
	RCMD_ALIAS

inherit
	ROSE_COMMAND_GENERAL

feature -- Access

	id: STRING is "alias"
			-- Command id

	signature: STRING is "name:s value:s"
			-- Argument signature of this command (defines valid parameters to be passed to the command)

feature -- Output

	help: STRING is
			-- Help on this command
"[
Set or display an aliased command
[no parameters]    Display all defined aliases
-name=<alias>      Display definition of <alias>
-value=<new value> Set the provided <alias> -name to <value>
]"

feature {NONE} -- Implementation

	do_execute is
			-- Execute effectively the command.
		local
			l_name, l_value: STRING
		do
			l_name := parameters.value ("name")
			l_value := parameters.value ("value")
			if l_value /= Void then
				if l_name = Void then
					error.signal (id,"Alias value '$1' provided without alias name",<<l_value>>)
				elseif Global.commands.is_command (l_name) then
					error.signal (id,"Can not alias existing built-in command '$1'",<<l_name>>)
				else
					Global.commands.set_alias (l_name,l_value)
				end
			elseif l_name /= Void then
				output_alias (l_name)
			else
				output_aliases (Global.commands.category ("alias"))
			end
		end

	output_aliases (a_cat: ROSE_SERVICE_MANAGER [ROSE_COMMAND]) is
			-- Output alias definitions for all items in 'a_cat'.
		local
			s: SEQUENCE [STRING]
		do
			if a_cat = Void then
				output ("No aliases defined%N",Void)
			else
				from
					s := a_cat.sorted_service_ids
					s.start
				until
					s.off
				loop
					output_alias (s.item)
					s.forth
				end
			end
		end

	output_alias (a_name: STRING) is
			-- Output alias definition for 'a_name'
		local
			s: STRING
		do
			s := Global.commands.alias_value (a_name)
			if s /= Void then
				output ("$1: $2%N",<<a_name,s>>)
			elseif Global.commands.has (a_name) then
				output ("$1: --built in command--%N",<<a_name>>)
			else
				output ("$1: --not defined--%N",<<a_name>>)
			end
		end

end -- class RCMD_ALIAS
