indexing
	description: "Configuration browser"
	author: "Zoran Simic"
	date: "$Date: 2008/12/01 $"
	revision: "$Revision: #4 $"

class
	RCMD_CONFIG_BROWSER

inherit
	RCMD_WINDOW

feature -- Access

	id: STRING is "config_browser"
			-- Command id

	signature: STRING is "config:? app:? env:? hide:? cmd:s"
			-- Argument signature of this command (defines valid parameters to be passed to the command)

feature -- Status report

	is_unique: BOOLEAN is True
			-- Is this window unique for the whole application?

feature -- Output

	help: STRING is
			-- Help on this command
"[
View application configuration and status
Opens a window showing various information on current application
-config  Open window on application's configuration view
-app     Open window on application status view
-env     Open window on environment view
-cmd     View commands (accepts command category as argument value)
-hide    Hide the window
]"

feature -- Basic operations

	check_prerequisites is
			-- Check that command's execution pre-requisites are met.
			-- Signal error if pre-requisistes are not met.
		do
		end

feature {NONE} -- Implementation

	do_execute is
			-- Execute effectively the command.
		do
			if parameters.provided ("env") then
				view_environment
			elseif parameters.boolean ("app") then
				view_application
			elseif parameters.provided ("cmd") then
				view_commands (parameters.value ("cmd"))
			else
				view_configuration (Void)
			end
			if parameters.boolean ("hide") then
				hide
			else
				show
				refresh
			end
		end

feature {NONE} -- Initialization

	notebook: EV_NOTEBOOK
			-- Notebook containing all offered views

	config_view: ROSE_CONFIGURATION_VIEW
	app_view: ROSE_APPLICATION_VIEW
	env_view: ROSE_ENVIRONMENT_VIEW
	command_view: ROSE_COMMAND_VIEW
			-- Various offered views

	initialize_window is
			-- Initialize 'window'.
		do
			window := Gui.factory.new_window ("Configuration browser","config_browser.ico",900,550)
			create notebook
			create config_view.make (notebook)
			create app_view.make (notebook)
			app_view.set_app (Global.app)
			create env_view.make (notebook)
			create command_view.make (notebook)
			notebook.set_item_text (config_view," Configuration ")
			notebook.set_item_text (app_view," Application ")
			notebook.set_item_text (env_view.box," Environment ")
			notebook.set_item_text (command_view.box," Commands ")
			notebook.selection_actions.extend (agent refresh)
			window.put (notebook)
			Gui.factory.add_menu (window,False,<<
				["View",
					["Configuration",
						["Application's",agent view_configuration (agent config_view.view_app_config)],
						["GUI",agent view_configuration (agent config_view.view_gui_config)],
						["Open...",agent view_configuration (agent config_view.load_config (Void,Void))]],
					["Application status",agent view_application],
					["Environment variables",agent view_environment],
					["Commands",agent view_commands (Void)],
					Void,
					["Refresh",agent refresh],
					Void,
					["Console","console"],
					[Global.app.name,Global.app.default_command_line]]
			>>)
		end

	view_configuration (a_agent: PROCEDURE [ANY,TUPLE []]) is
			-- View configuration tab.
		do
			notebook.select_item (config_view)
			if a_agent /= Void then
				a_agent.call ([])
			else
				refresh
			end
		end

	view_application is
			-- View configuration tab.
		do
			notebook.select_item (app_view)
			refresh
		end

	view_environment is
			-- View configuration tab.
		do
			notebook.select_item (env_view.box)
			refresh
		end

	view_commands (a_command: STRING) is
			-- View commands tab.
		do
			notebook.select_item (command_view.box)
			command_view.refresh (a_command)
		end

	refresh is
			-- Refresh the view.
		local
			l_widget: EV_WIDGET
		do
			l_widget := notebook.selected_item
			if config_view = l_widget then
				config_view.refresh
			elseif app_view = l_widget then
				app_view.refresh
			elseif env_view.box = l_widget then
				env_view.refresh
			elseif command_view.box = l_widget then
				command_view.refresh (Void)
			end
		end

end -- class RCMD_CONFIG_BROWSER
