indexing
	description: "Application about dialog"
	author: "Zoran Simic"
	date: "$Date: 2009/09/15 $"
	revision: "$Revision: #9 $"

class
	RCMD_EV_ABOUT

inherit
	RCMD_WINDOW

feature -- Access

	id: STRING is "ev_about"
			-- Command id

	signature: STRING is ""
			-- Argument signature of this command (defines valid parameters to be passed to the command)

feature -- Status report

	is_unique: BOOLEAN is True
			-- Is this window unique for the whole application?

feature -- Output

	help: STRING is
			-- Help on this command
"[
Show application's 'About' dialog
This dialog provides general information on current application
]"

feature -- Basic operations

	check_prerequisites is
			-- Check that command's execution pre-requisites are met.
			-- Signal error if pre-requisistes are not met.
		do
		end

feature {NONE} -- Initialization

	info: EV_TEXT
			-- Widget showing extensive application info

	do_execute is
			-- Execute effectively the command.
		do
			show
			info.remove_text
			output_in (info,Global.app.report,Void)
			output_in (info,"%NSite: '$1'%N",<<Global.app.site.path>>)
			output_in (info,"Delivery: '$1'%N",<<Global.app.delivery.path>>)
		end

	initialize_window is
			-- Initialize 'window'.
		local
			l_vbox, l_hbox, l_hbox2: EV_BOX
			l_label: EV_LABEL
			l_ok_button: EV_BUTTON
			l_widget: EV_WIDGET
			l_dialog: EV_DIALOG
		do
			is_auto_closed := True
			l_dialog := Gui.factory.new_dialog ("About " + Global.app.name,Gui.rep.pixmaps.Information_pixmap,700,0)
			window := l_dialog
			l_hbox := Gui.factory.new_hbox (window,4,4)
				-- First vertical box
			l_vbox := Gui.factory.new_vbox (l_hbox,0,4)
			l_widget := Gui.factory.new_boxed_pixmap_from_file (l_vbox,"axar-logov.png",50,200)
			l_ok_button := Gui.factory.new_button (l_vbox,"OK",agent hide,1)
			l_dialog.set_default_push_button (l_ok_button)
			l_dialog.set_default_cancel_button (l_ok_button)
				-- Second vertical box
			l_vbox := Gui.factory.new_vbox (l_hbox,0,-2)
				-- Name and version
			l_hbox2 := Gui.factory.new_hbox (l_vbox,0,3)
			highlight (Gui.factory.new_label (l_hbox2,Global.app.name,1),False)
			l_label := Gui.factory.new_label (l_hbox2,Void,0)
			highlight (Gui.factory.new_label (l_hbox2,"version",1),True)
			highlight (Gui.factory.new_label (l_hbox2,Global.app.version,1),True)
				-- Description
			l_label := Gui.factory.new_label (l_vbox,Global.app.description,1)
			l_label.set_foreground_color (Gui.rep.colors.Dark_blue)
			l_label := Gui.factory.new_label (l_vbox,"Copyright (c) AXA Rosenberg - Barr Rosenberg Research Center",1)
			l_label.set_foreground_color (Gui.rep.colors.Gray)
				-- Extensive info box
			info := Gui.factory.new_text (l_vbox,"",0,-200)
			info.disable_edit
			window.set_background_color (Gui.rep.colors.White)
			window.propagate_background_color
			l_ok_button.set_background_color (Gui.rep.colors.color_3d_highlight)
			info.set_background_color (Gui.rep.colors.color_3d_highlight)
		end

	highlight (a_label: EV_LABEL; a_italic: BOOLEAN) is
			-- Highlight 'a_label', and put it eventually in italic.
		local
			l_font: EV_FONT
		do
			create l_font
			l_font.set_weight (Gui.rep.fonts.weight_bold)
			l_font.set_family (Gui.rep.fonts.family_sans)
			if a_italic then
				l_font.set_shape (Gui.rep.fonts.shape_italic)
				a_label.set_foreground_color (Gui.rep.colors.Dark_gray)
			end
			a_label.set_font (l_font)
		end

	output_in (a_text: EV_TEXT; a_message: STRING; a_args: ARRAY [ANY]) is
			-- Output 'a_message' with 'a_args' in 'a_text'.
		local
			l_string: STRING
		do
			l_string := Global.formatter.argumented (a_message,a_args)
			l_string.prune_all ('%R')		--| FIXME: temporary fix for Vision2 problem
			a_text.append_text (l_string)
		end

end
