indexing
	description: "Eiffel Vision console window"
	author: "Zoran Simic"
	last_editor: "$Author: rj $"
	date: "$Date: 2013/09/17 $"
	revision: "$Revision: #11 $"
	path: "$File: //rose/source/kernel/infrastructure/framework/gui/rcmd_ev_console.e $"

class
	RCMD_EV_CONSOLE

inherit
	RCMD_WINDOW

feature -- Access

	id: STRING is "ev_console"
			-- Command id

	signature: STRING is "cmd:s"
			-- Argument signature of this command (defines valid parameters to be passed to the command)

	ask (a_question: STRING): STRING is
			-- Ask 'a_question' to user
			-- Void when input is exhausted
		do
			Result := Gui.dialogs.ask_for_string (window,"Question",a_question,Void)
		end

	output_box: EV_TEXT
			-- The widget that displays the messages.  Made public so that other classes may use it.

feature -- Status report

	is_unique: BOOLEAN is True
			-- Is this window unique for the whole application?

feature -- Output

	help: STRING is
			-- Help on this command
"[
Console window accepting user input
-cmd=<command>  Execute <command> before waiting for user input
]"

feature -- Basic operations

	check_prerequisites is
			-- Check that command's execution pre-requisites are met.
			-- Signal error if pre-requisistes are not met.
		do
			--| No prerequisites
		end

	output_message (a_level: INTEGER; a_message: STRING) is
			-- Output 'a_message' with 'a_level' on this channel.
		do
			output_colored (decorated_message (a_level,a_message),colors.item (a_level))
		end

	output_log (a_severity, a_id: INTEGER; a_class, a_feature, a_message: STRING) is
			-- Specific output for logs.
		do
			output_message (Global.convention.v_log,a_message)
		end

	output_time_measure (a_tm: ROSE_TIME_MEASURE) is
			-- Specific output for time measures...
		do
			if a_tm /= Void then
				output_message (Global.convention.v_trace,a_tm.status)
			end
		end

	decorated_message (a_level: INTEGER; a_message: STRING): STRING is
			-- FIXME: Temporary due to lack of colored widgets
		do
			Result := Global.prefs.decorated_verbose_message (a_level,a_message)
		end

	output_colored (a_message: STRING; a_color: EV_COLOR) is
			-- Output 'a_message' with 'a_color'.
		local
		do
			if a_message /= Void and then output_box /= Void then
				a_message.prune_all ('%R')
				output_box.append_text (a_message)
			end
		end

	flush is
			-- Flush the console.
		do
				--| Do nothing
		end

feature {NONE} -- Implementation

	do_execute is
			-- Execute effectively the command.
		do
			show
			if parameters.provided ("cmd") then
				execute_line (parameters.value ("cmd"))
			end
		end

	input_box: EV_TEXT_FIELD
			-- Widgets used outside of setup ('initialize_window')

	initialize_window is
			-- Initialize 'window'.
		local
			l_widget: EV_WIDGET
			l_vbox: EV_VERTICAL_BOX
			l_hbox: EV_HORIZONTAL_BOX
		do
				--| Window setup
			window := Gui.factory.new_window (Global.app.name + " console","console.ico",0,0)
			l_vbox := Gui.factory.new_vbox (window,2,2)
				--| Input box
			l_hbox := Gui.factory.new_hbox (l_vbox,0,2)
			input_box := Gui.remembered.new_text_field (l_hbox,"console","Command: ",0,-20)
			input_box.return_actions.extend (agent on_return)
			input_box.set_capacity (1024)
			l_widget := Gui.factory.new_button (l_hbox,"Execute",agent on_return,70)
				--| Output box
			output_box := Gui.factory.new_text (l_vbox,Void,-550,-150)
			l_widget := Gui.remembered.new_check_button (l_vbox,k_auto_pop_option,"Show this console when application starts",Void,1)
			window.show_actions.extend (agent input_box.set_focus)
			window.focus_in_actions.extend (agent input_box.set_focus)
				--| Menu setup
			Gui.factory.add_menu (window,False,<<
				["File",
					["Save output",agent file_save (Void)],
					["Save output as...",agent file_save (last_save)],
					Void,
					["Close this window",agent hide],
					["Exit application","ev_quit"]],
				["Edit",
					["Clear output",agent output_box.remove_text],
					Void,
					["Copy all",agent copy_all],
					["Copy selected",agent copy_selected],
					Void,
					["Find...",agent find (Void)],
					["Find next",agent find (last_find)]],
				["View",
					["Configuration","config_browser"],
					["Error log","ev_error"],
					["Monitor","ev_monitor"],
					Void,
					["Verbosity",
						[0, "error",agent set_verbosity (Global.convention.v_error)],
						[0, "warning",agent set_verbosity (Global.convention.v_warning)],
						[0, "log",agent set_verbosity (Global.convention.v_log)],
						[0, "output",agent set_verbosity (Global.convention.v_output)],
						[0, "trace",agent set_verbosity (Global.convention.v_trace)]
					],
					Void,
					[Global.app.name,Global.app.default_command_line]],
				["Help",[Gui.factory.default_help_menu]]
			>>)
				--| Auto pop-up if requested
			if Gui.remembered.boolean (k_auto_pop_option) then
				show
			end
		end

	on_return is
			-- Routine executed when 'return' key is pressed in the text box.
		do
			execute_line (input_box.text)
			input_box.remove_text
		end

	execute_line (a_line: STRING) is
			-- Execute 'a_line'.
		do
			Global.commands.execute (a_line)
			error.flush_and_acknowledge
		end

	colors: HASH_TABLE [EV_COLOR,INTEGER] is
			-- Colors associated to verbosity levels
		once
			create Result.make (0)
			Result.put (Gui.rep.colors.Red,Global.convention.v_error)
			Result.put (Gui.rep.colors.Dark_red,Global.convention.v_warning)
			Result.put (Gui.rep.colors.Dark_blue,Global.convention.v_log)
			Result.put (Gui.rep.colors.Dark_green,Global.convention.v_trace)
		end

	last_save: STRING

	file_save (a_path: STRING) is
		do
			output_message (Global.convention.v_error,"File/save not yet implemented%N")
		end

	set_verbosity (a_level: INTEGER) is
		do
		end

	copy_all is
		do
		end

	copy_selected is
		do
		end

	last_find: STRING

	find (a_string: STRING) is
		do
			output_message (Global.convention.v_error,"Find not yet implemented%N")
		end

	k_auto_pop_option: STRING is "autopop_console"
			-- Configuration key for auto-pop option

end
