indexing
	description: "Eiffel Vision monitor log window"
	author: "Zoran Simic"
	date: "$Date: 2008/12/01 $"
	revision: "$Revision: #8 $"

class
	RCMD_EV_MONITOR

inherit
	RCMD_WINDOW

feature -- Access

	id: STRING is "ev_monitor"
			-- Command id

	signature: STRING is ""
			-- Argument signature of this command (defines valid parameters to be passed to the command)

feature -- Status report

	is_unique: BOOLEAN is True
			-- Is this window unique for the whole application?

feature -- Output

	help: STRING is
			-- Help on this command
"[
View time measurements in a window
]"

feature -- Basic operations

	check_prerequisites is
			-- Check that command's execution pre-requisites are met.
			-- Signal error if pre-requisistes are not met.
		do
		end

	output_time_measure (a_message: STRING) is
			-- Output 'a_message' on the console.
		do
			if a_message /= Void then
				a_message.prune_all ('%R')		--| FIXME: temporary fix for Vision2
				output_box.append_text (a_message)
			end
		end

feature {NONE} -- Initialization

	do_execute is
			-- Execute effectively the command.
		do
			show
		end

	output_box: EV_TEXT
			-- Widgets used outside of setup ('initialize_window')

	initialize_window is
			-- Initialize 'window'.
		local
			l_vbox: EV_VERTICAL_BOX
			l_hbox: EV_HORIZONTAL_BOX
			l_widget: EV_WIDGET
			l_button: EV_BUTTON
			l_dialog: EV_DIALOG
		do
			l_dialog := Gui.factory.new_dialog (Global.app.name + " monitor log","monitor.ico",0,0)
			window := l_dialog
			l_vbox := Gui.factory.new_vbox (l_dialog,4,-4)
				--| Entity list drop box
			l_widget := Gui.factory.new_label (l_vbox,"This window catches all execution time measurements performed via the framework",1)
			output_box := Gui.factory.new_text (l_vbox,Void,-450,-100)
			l_hbox := Gui.factory.new_hbox (l_vbox,0,4)
			l_widget := Gui.remembered.new_check_button (l_hbox,k_auto_pop_option,"Monitor is active in this window",agent activate,-1)
			l_button := Gui.factory.new_button (l_hbox,"Close this window",agent hide,150)
			l_dialog.set_default_push_button (l_button)
			l_dialog.set_default_cancel_button (l_button)
			Gui.factory.add_menu (l_dialog,False,<<
				["File",
					["Save all"],
					["Save selected"],
					Void,
					["Close this window",agent hide],
					["Exit application","ev_quit"]],
				["Edit",
					["Clear output",agent output_box.remove_text],
					Void,
					["Copy all"],
					["Copy selected"],
					Void,
					["Find..."],
					["Find next"]],
				["Help",[Gui.factory.default_help_menu]]
			>>)
			is_auto_closed := True
			activate
		end

	activate is
			-- Activate monitor depending on 'a_checkbox' setting.
		do
			if Gui.remembered.boolean (k_auto_pop_option) then
				Global.commands.execute ("redirect -monitor=ev_monitor")
				show
			else
				Global.io_handler.set_monitor (Void)
			end
		end

	k_auto_pop_option: STRING is "autopop_monitor"
			-- Configuration key for auto-pop option

end
