indexing
	description: "Quit Eiffel Vision application"
	author: "Zoran Simic"
	date: "$Date: 2008/12/01 $"
	revision: "$Revision: #7 $"

class
	RCMD_EV_QUIT

inherit
	RCMD_WINDOW

feature -- Access

	id: STRING is "ev_quit"
			-- Command id

	signature: STRING is "ask:? silent:?"
			-- Argument signature of this command (defines valid parameters to be passed to the command)

feature -- Status report

	is_unique: BOOLEAN is True
			-- Is this window unique for the whole application?

	is_auto_pop: BOOLEAN is
			-- Pop up automaitcally this window when application is requested to be closed?
		do
			Result := Gui.remembered.boolean (k_auto_pop_option)
		end

feature -- Output

	help: STRING is
			-- Help on this command
"[
Quit application, optionally ask user's termination confirmation
-ask           Ask user to confirm application termination
-silent        Exit immediately, without asking user's confirmation
no arguments:  Ask for confirmation only if configured to do so
]"

feature -- Basic operations

	check_prerequisites is
			-- Check that command's execution pre-requisites are met.
			-- Signal error if pre-requisistes are not met.
		do
		end

feature {NONE} -- Initialization

	do_execute is
			-- Execute effectively the command.
		do
			if not parameters.boolean ("silent")
			and then (is_auto_pop or else parameters.boolean ("ask")) then
				show
			else
				Gui.destroy
			end
		end

	initialize_window is
			-- Initialize 'window'.
		local
			l_vbox, l_hbox: EV_BOX
			l_label: EV_LABEL
			l_widget: EV_WIDGET
			l_yes, l_no: EV_BUTTON
			l_dialog: EV_DIALOG
		do
			l_dialog := Gui.factory.new_dialog ("Quit confirm?",Gui.rep.pixmaps.Information_pixmap,0,0)
			window := l_dialog
				--| Dialog composition
			l_vbox := Gui.factory.new_vbox (l_dialog,6,6)
			l_label := Gui.factory.new_label (l_vbox,"Do you really want to exit%N" + Global.app.name + "?",0)
			l_label.align_text_center
			l_label.set_foreground_color (Gui.rep.colors.Dark_blue)
				--| Automatically ask?
			l_widget := Gui.remembered.new_check_button (l_vbox,k_auto_pop_option,"Always ask this question when closing?",Void,0)
				--| Buttons
			l_hbox := Gui.factory.new_hbox (l_vbox,0,15)
			l_widget := Gui.factory.new_label (l_hbox,Void,0)
			l_yes := Gui.factory.new_button (l_hbox,"Yes",agent Gui.destroy,80)
			l_no := Gui.factory.new_button (l_hbox,"No",agent hide,80)
			l_widget := Gui.factory.new_label (l_hbox,Void,0)
				--| Default buttons
			l_dialog.set_default_push_button (l_yes)
			l_dialog.set_default_cancel_button (l_no)
			l_dialog.disable_user_resize
			is_auto_closed := True
		end

	k_auto_pop_option: STRING is "autopop_quit"
			-- Configuration key for auto-pop option

end
