indexing
	description: "Quit Eiffel Vision application"
	author: "Zoran Simic"
	date: "$Date: 2009/07/22 $"
	revision: "$Revision: #7 $"

class
	RCMD_EV_SHOW

inherit
	RCMD_WINDOW
		redefine
			window_id
		end

feature -- Access

	id: STRING is "show"
			-- Command id

	signature: STRING is "*"
			-- Argument signature of this command (defines valid parameters to be passed to the command)

	window_id: STRING is
			-- Window id used to remember window position
			-- Return Void to disable position remembering
		do
		end

feature -- Status report

	is_unique: BOOLEAN is False
			-- Is this window unique for the whole application?

feature -- Output

	help: STRING is
			-- Help on this command
"[
Show a command
First argument should the command name
Any subsequent arguments are passed on and shown in the dialog
no arguments:  Show all commands
]"

feature -- Basic operations

	check_prerequisites is
			-- Check that command's execution pre-requisites are met.
			-- Signal error if pre-requisistes are not met.
		do
		end

feature {NONE} -- Initialization

	do_execute is
			-- Execute effectively the command.
		local
			l_id, l_params: STRING
			p: PAIR [STRING, STRING]
		do
			l_id := parameters.text
			if l_id /= Void and then l_id.count > 0 then
				l_id.left_adjust
				l_id.right_adjust
				if l_id.count > 5 and then l_id.starts_with ("-cmd=") then
					-- Remove any leading "-cmd=" to remain compatible
					-- with previous implementaiton of this command
					l_id.keep_tail (l_id.count - 5)
				end
				p := global.formatter.split_on_first (l_id, global.convention.argument_separators, Void)
				l_id := p.first
				l_params := p.second
			end
			if l_id = Void or else l_id.is_empty then
				Global.commands.execute ("config_browser -cmd=All")
			elseif Global.commands.has (l_id) then
				build_view (l_id, l_params)
				show
			elseif Global.commands.has_category (l_id) then
				Global.commands.execute ("config_browser -cmd=" + l_id)
			else
				error.signal (id,"Unknown command '$1'",<<l_id>>)
			end
		end

	initialize_window is
			-- Initialize 'window'.
		do
			window := Gui.factory.new_window ("Command view","bulb.ico",0,0)
		end

	parameters_view: ROSE_COMMAND_PARAMETERS_VIEW
	execute_button: EV_BUTTON

	build_view (a_id, a_params: STRING) is
			-- Build show view on 'a_command'
		local
			l_command: ROSE_COMMAND
			l_main_box, l_box: EV_BOX
			l_widget: EV_WIDGET
			l_frame: EV_FRAME
		do
			l_command := Global.commands.service (a_id)
			if l_command /= Void then
				window.set_title (l_command.id)
				l_main_box := Gui.factory.new_vbox (window,4,2)
				create parameters_view.make (l_main_box,l_command, agent set_is_valid)
				parameters_view.set_parameters (a_params)
				l_box := Gui.factory.new_hbox (l_main_box,0,10)
				execute_button := Gui.factory.new_button (l_box,"Execute",agent execute_command (l_command),80)
				l_widget := Gui.factory.new_button (l_box,"Show console",agent (Global.commands).execute ("console"),100)
				l_widget := Gui.factory.new_label (l_box,Void,20)
				l_widget := Gui.factory.new_button (l_box,"Close",agent Gui.close_window (window),80)
				l_box := Gui.factory.new_hbox (l_main_box,0,1)
				l_widget := Gui.remembered.new_check_button (l_box,auto_close_key (a_id),"Close when executed without errors",Void,1)
				l_frame := Gui.factory.new_frame (l_main_box,"Help",1,1)
				l_widget := Gui.factory.new_label (l_frame,l_command.help,1)
				l_widget.set_foreground_color (Gui.rep.colors.Dark_blue)
				parameters_view.validate
			end
		end

	auto_close_key (a_id: STRING): STRING is
			-- "autoclose_" + a_id
		do
			if a_id /= Void then
				Result := "autoclose_" + a_id
			end
		end

	set_is_valid (a_valid: BOOLEAN) is
			-- Called when parameters_view changed.
		do
			Gui.enable_sensitive (execute_button, a_valid)
		end

	execute_command (a_command: ROSE_COMMAND) is
			-- Execute 'a_command'.
		local
			retried: BOOLEAN
		do
			if not retried then
				Global.commands.execute (a_command.id + " " + parameters_view.value)
			end
			if not error.signaled and then Gui.remembered.boolean (auto_close_key (a_command.id)) then
				hide
			end
			error.flush_and_acknowledge
		rescue
			if not retried then
				retried := True
				retry
			end
		end

end
