indexing
	description: "Provide help on available commands"
	author: "Zoran Simic"
	date: "$Date: 2008/12/01 $"
	revision: "$Revision: #7 $"

class
	RCMD_HELP

inherit
	ROSE_COMMAND_GENERAL

feature -- Access

	id: STRING is "help"
			-- Command id

	signature: STRING is "*"
			-- Argument signature of this command (defines valid parameters to be passed to the command)

feature -- Output

	help: STRING is
			-- Help on this command
"[
Display help on application commands
Multiple commands or categories can be listed if ';'-separated
help            List all non-default and non-alias commands
help -all       List all commands, including default and aliases
help <command>  Display full help message on <command>
help <category> List commands from <category>
]"
--<command> ?     Display full help message on <command>

feature {NONE} -- Implementation

	do_execute is
			-- Execute effectively the command.
		local
			l_seq: SEQUENCE [STRING]
		do
			if parameters.text = Void or else parameters.text.is_empty then
				l_seq := sorted (Global.commands.categories.current_keys.linear_representation)
				if l_seq /= Void then
					from
						l_seq.start
					until
						l_seq.off
					loop
						if not Global.commands.is_default_category (l_seq.item) then
							category_help (l_seq.item)
						end
						l_seq.forth
					end
				end
			elseif equal (parameters.text,"-all") then
				l_seq := sorted (Global.commands.categories.current_keys.linear_representation)
				if l_seq /= Void then
					from
						l_seq.start
					until
						l_seq.off
					loop
						category_help (l_seq.item)
						l_seq.forth
					end
				end
			else
				l_seq := Global.formatter.split (parameters.text," %T,;+",Void)
				if l_seq /= Void then
					from
						l_seq.start
					until
						l_seq.off
					loop
						if Global.commands.has_category (l_seq.item) then
							category_help (l_seq.item)
						end
						l_seq.forth
					end
					from
						l_seq.start
					until
						l_seq.off
					loop
						if not Global.commands.has_category (l_seq.item) then
							command_help (l_seq.item,l_seq.count<3)
						end
						l_seq.forth
					end
				end
			end
		end

	category_help (a_category_name: STRING) is
			-- Display help on 'a_category_name'
		local
			l_category: ROSE_SERVICE_MANAGER [ROSE_COMMAND]
			l_seq: SEQUENCE [STRING]
		do
			l_category := Global.commands.category (a_category_name)
			if l_category = Void then
				output ("--  No command category '$1'",<<a_category_name>>)
			else
				output ("----------------%N--  '$1' commands%N----------------%N",<<l_category.id>>)
				l_seq := sorted (l_category.services.current_keys.linear_representation)
				if l_seq /= Void then
					from
						l_seq.start
					until
						l_seq.off
					loop
						command_help (l_seq.item,False)
						l_seq.forth
					end
				end
			end
		end

	command_help (a_command_name: STRING; a_complete_help: BOOLEAN) is
		local
			l_command: ROSE_COMMAND
			l_name, l_help: STRING
		do
			l_name := clone (a_command_name)
			l_name.to_lower
			l_command := Global.commands.service (l_name)
			if a_complete_help then
				if l_command = Void then
					output ("$1: -- Unknown command%N",<<l_name>>)
				else
					output ("$1: $2%N$3%N%N",<<l_name,l_command.signature,indented_help (l_command.help)>>)
				end
			else
				if l_command = Void then
					l_help := "-- Unknown command"
				else
					l_help := l_command.quick_help
				end
				l_name := Global.formatter.padded_string (l_name,' ',-14)
				l_help := Global.formatter.padded_string (l_help,' ',80)
				l_help.right_adjust
				output ("$1: $2%N",<<l_name,l_help>>)
			end
		end

	sorted (a_set: LINEAR [STRING]): SEQUENCE [STRING] is
			-- Sorted 'a_set
		do
			if a_set /= Void then
				create {SORTED_TWO_WAY_LIST [STRING]} Result.make
				from
					a_set.start
				until
					a_set.off
				loop
					Result.extend (a_set.item)
					a_set.forth
				end
			end
		end

	indented_help (a_string: STRING): STRING is
		do
			if a_string = Void then
				Result := "  -- No help available"
			else
				Result := "  " + a_string
				Result.replace_substring_all ("%N","%N  ")
			end
		end

end -- class RCMD_HELP
