indexing
	description: "Macro command"
	author: "Zoran Simic"
	date: "$Date: 2008/12/01 $"
	revision: "$Revision: #8 $"

class
	RCMD_MACRO

inherit
	ROSE_COMMAND_GENERAL
		redefine
			new_representative, close, pre_execute, post_execute
		end

create
	make_anonymous, make_with_id

feature {NONE} -- Initialization

	make_anonymous is
			-- Create an anonymous macros
		do
			create commands.make
			id := "_"
		end

	make_with_id (a_id: STRING) is
		do
			make_anonymous
			id := a_id
		end

feature -- Access

	id: STRING
			-- Command id

	signature: STRING is ""
			-- Argument signature of this command (defines valid parameters to be passed to the command)

	help: STRING is
			-- Help on this command
"[
Sequence of commands
This is an internal object, not visible to the user
]"

	commands: LINKED_LIST [ROSE_COMMAND]
			-- Commands constituting this macro

	count: INTEGER is
			-- Number of commands in this macro
		do
			Result := commands.count
		end

	new_representative: like Current is
			-- New independent version of Current service.
			-- If you want this service to be shared (unique instance used for the whole system),
			-- return Current (no clone)
		do
			create Result.make_with_id (id)
			from
				commands.start
			until
				commands.off
			loop
				Result.add_last (commands.item.new_representative)
				commands.forth
			end
		end

	has (a_id: STRING): BOOLEAN is
			-- Is command 'a_id' part of this macro ?
		local
			s: LINKED_LIST [ROSE_COMMAND]
		do
			from
				s := clone (commands)
				s.start
			until
				Result or else s.off
			loop
				Result := equal (a_id,s.item.id)
				s.forth
			end
		end

feature -- Basic operations

	close is
			-- Close this service.
		do
			from
				commands.start
			until
				commands.off
			loop
				commands.item.close
				commands.forth
			end
		end

	add_first (a_command: ROSE_COMMAND) is
			-- Add 'a_command' to this macro at first position.
		do
			if a_command /= Void then
				commands.put_front (a_command)
			end
		end

	add_after (a_id: STRING; a_command: ROSE_COMMAND) is
			-- Add 'a_command' right after first occurence of 'a_id' command.
			-- Add first if no command with 'a_id' in this macro.
		local
			l_found: BOOLEAN
		do
			if a_command /= Void and then commands /= Void then
				from
					commands.start
				until
					l_found or else commands.off
				loop
					l_found := equal (a_id,commands.item.id)
					if not l_found then
						commands.forth
					end
				end
			end
			if l_found then
				commands.put_right (a_command)
			else
				add_first (a_command)
			end
		end

	add_last (a_command: ROSE_COMMAND) is
			-- Add 'a_command' to this macro at last position.
		do
			if a_command /= Void then
				commands.extend (a_command)
			end
		end

	replace (a_command: ROSE_COMMAND) is
			-- Replace command with same id in this macro with 'a_command'
			-- or add 'a_command' to this macro if no such command in.
		local
			l_found: BOOLEAN
		do
			from
				commands.start
			until
				l_found or else commands.off
			loop
				l_found := equal (a_command.id,commands.item.id)
				if l_found then
					commands.replace (a_command)
				end
				commands.forth
			end
			if not l_found then
				add_first (a_command)
			end
		end

feature {NONE} -- Implementation

	pre_execute is
			-- Pre-execution actions.
		do
		end

	do_execute is
			-- Execute effectively the command.
		do
			from
				commands.start
			until
				error.signaled or else commands.off
			loop
				commands.item.execute_prepared
				commands.forth
			end
		end

	post_execute is
			-- Post-execution actions.
		do
		end

end
