indexing
	description: "Redirect one or more of the 5 communication channels (out,in,echo,log,monitor)"
	author: "Zoran Simic"
	date: "$Date: 2008/12/01 $"
	revision: "$Revision: #11 $"

class
	RCMD_REDIRECT

inherit
	ROSE_COMMAND_GENERAL

feature -- Access

	id: STRING is "redirect"
			-- Command id

	signature: STRING is "output:s input:s echo:s log:s monitor:s"
			-- Argument signature of this command (defines valid parameters to be passed to the command)

feature -- Output

	help: STRING is
			-- Help on this command
"[
Redirect conventional channels
-output=<channel>   Direct output to <channel>
-input=<channel>    Direct input to <channel>
-echo=<channel>     Direct echo to <channel>
-log=<channel>      Direct log to <channel>
-monitor=<channel>  Direct monitor to <channel>
]"

feature {NONE} -- Implementation

	do_execute is
			-- Execute effectively the command.
		local
			l_channel: ROSE_OUTCHANNEL
			l_in: ROSE_INCHANNEL
			s: STRING
		do
			s := parameters.value ("output")
			if not error.signaled and then s /= Void and then not s.is_empty then
				l_channel := Global.io_handler.manager.checked_output_channel (id,s)
				if not error.signaled then
					inform_redirected ("output",l_channel,False)
					Global.io_handler.set_output (l_channel)
				end
			end
			s := parameters.value ("input")
			if not error.signaled and then s /= Void and then not s.is_empty then
				l_in := Global.io_handler.manager.checked_input_channel (id,s)
				if not error.signaled then
					Global.io_handler.set_input (l_in)
					trace ("input redirected to '$1'%N",<<l_in>>)
				end
			end
			s := parameters.value ("log")
			if not error.signaled and then s /= Void and then not s.is_empty then
				l_channel := Global.io_handler.manager.checked_output_channel (id,s)
				if not error.signaled then
					Global.io_handler.set_log (l_channel)
					inform_redirected ("log",l_channel,True)
				end
			end
			s := parameters.value ("monitor")
			if not error.signaled and then s /= Void and then not s.is_empty then
				l_channel := Global.io_handler.manager.checked_output_channel (id,s)
				if not error.signaled then
					Global.io_handler.set_monitor (l_channel)
					inform_redirected ("monitor",l_channel,True)
				end
			end
			s := parameters.value ("echo")
			if not error.signaled and then s /= Void and then not s.is_empty then
				l_channel := Global.io_handler.manager.checked_output_channel (id,s)
				if not error.signaled then
					Global.io_handler.set_echo (l_channel)
					inform_redirected ("echo",l_channel,False)
				end
			end
		end

	inform_redirected (a_id: STRING; a_channel: ROSE_OUTCHANNEL; a_on_channel: BOOLEAN) is
			-- Inform of redirection to 'a_channel' for 'a_id'.
		local
			l_now: ROSE_DATE
		do
			trace ("$1 redirected to '$2'%N",<<a_id,a_channel>>)
			if a_on_channel then
				create l_now.make_now
				Global.io_handler.output_on (
					a_channel,
					Global.convention.v_trace,
					Global.formatted (
						"--  $1: $2 started on '$3'%N----------%N",
						<<l_now.formatted_out (Global.prefs.date_time_format),a_id,a_channel>>))
			end
		end

end
