indexing
	description: "Override or display database repository status"
	author: "Zoran Simic"
	last_editor: "$Author: mb $"
	date: "$Date: 2011/10/24 $"
	revision: "$Revision: #7 $"
    path: "$File: //rose/source/kernel/infrastructure/framework/command/rcmd_repository.e $"

class
	RCMD_REPOSITORY

inherit
	ROSE_COMMAND_GENERAL
	ROSE_SHARED_DATABASE_POOL

feature -- Access

	id: STRING is "repository"
			-- Command id

	signature: STRING is "logical:s path:s source:s override:? report:s check:?"
			-- Argument signature of this command (defines valid parameters to be passed to the command)

feature -- Output

	help: STRING is
			-- Help on this command
"[
Override or display database repository status
-logical    The logical you wish to show or override
-path       The path you wish to override the logical with
-source     The source name to use for overrides (default is 'repository command')
            'hardcoded' will be persistent for any downstream command execution
-override   Override given logical with given path
-report     Report all currently used definitions
-report=all Report all definitions
-check      Check connection to underlying server
]"

feature {NONE} -- Implementation

	do_execute is
			-- Execute effectively the command.
		local
			l_definition: ROSE_DATABASE_DEFINITION
		do
			if parameters.boolean ("override") then
				if not parameters.provided ("logical") or else not parameters.provided ("path") then
					error.signal (id, "You must provide a logical and a path for -override option", <<>>)
				else
					if source ~ "hardcoded" then
						pool.set_hardcoded_override (parameters.value ("logical"), parameters.value ("path"))
					else
						create l_definition.make_from_path (source, parameters.value ("logical"), parameters.value ("path"))
						if l_definition.overridden = Void then
							output ("Logical '$1' set to '$2'%N$3", <<l_definition.logical, l_definition.full_path, definition_report (l_definition.logical)>>)
						else
							output ("Logical '$1' overridden to '$2'%N$3", <<l_definition.logical, l_definition.full_path, definition_report (l_definition.logical)>>)
						end
						pool.extend (l_definition)
					end
				end
			elseif parameters.provided ("logical") then
				if parameters.provided ("path") then
					l_definition := pool.definition (parameters.value ("logical"))
					if l_definition = Void then
						create l_definition.make_from_path (source, parameters.value ("logical"), parameters.value ("path"))
						pool.extend (l_definition)
						output ("Logical '$1' set to '$2'%N$3", <<l_definition.logical, l_definition.full_path, definition_report (l_definition.logical)>>)
					elseif equal (l_definition.full_path, parameters.value ("path")) then
						output ("Logical '$1' is already defined to '$2'%N$3", <<l_definition.logical, l_definition.full_path, definition_report (l_definition.logical)>>)
					else
						error.signal (id, "Logical '$1' is already defined, please use option -override%N$2", <<l_definition.logical, definition_report (l_definition.logical)>>)
					end
				else
					output (definition_report (parameters.value ("logical")), Void)
				end
			elseif parameters.provided ("report") and then equal (parameters.value ("report").as_lower, "all") then
				output ("$1 databases defined (sorted by logical):%N$2%N", <<pool.count, full_report>>)
			else
				output ("$1%N", <<pool.report>>)
			end
		end

	source: STRING is
			-- Validated source string
		do
			Result := parameters.value ("source")
			if Result = Void or else Result.is_empty then
				Result := "repository command"
			end
		ensure
			valid_result: Result /= Void
		end

	definition_report (a_logical: STRING): STRING is
			-- Report on 'a_logical'
		local
			l_definition: ROSE_DATABASE_DEFINITION
			l_server: DATA_SERVER
		do
			l_definition := pool.definition (a_logical)
			create Result.make_empty
			Result.append_string (a_logical)
			Result.append_character (' ')
			Result.append_character ('=')
			Result.append_character (' ')
			if l_definition = Void then
				Result.append_string ("*not defined*%N")
			else
				Result.append_string (l_definition.full_path)
				Result.append_string (" [from ")
				Result.append_string (l_definition.source)
				Result.append_string ("]%N")
				from
					l_definition := l_definition.overridden
				until
					l_definition = Void
				loop
					Result.append_string ("  overrides -> ")
					Result.append_string (l_definition.full_path)
					Result.append_string (" [from '")
					Result.append_string (l_definition.source)
					Result.append_string ("']%N")
					l_definition := l_definition.overridden
				end
				if parameters.boolean ("check") then
					l_server := pool.get_server_from_logical (a_logical)
					if l_server = Void then
						Result.append_string ("Can't connect to underlying server%N")
					else
						Result.append_string ("Connection OK%N")
					end
				end
			end
		end

	full_report: STRING is
			-- Full report string for the repository entries
		local
			l_array: ROSE_LINEAR_ARRAY [ROSE_DATABASE_DEFINITION]
			i: INTEGER
		do
			from
				create Result.make (1024)
				l_array := pool.definition_array
				i := l_array.lower
			until
				i > l_array.upper
			loop
				Result.append (definition_report (l_array.item (i).logical))
				i := i + 1
			end
		ensure
			valid_result: Result /= Void
		end

end
