note

	description:

		"Command opening an Eiffel Vision window to execute"

	author: "Zoran Simic"
	last_editor: "$Author: c4 $"
	date: "$Date: 2014/07/16 $"
	revision: "$Revision: #6 $"
	path: "$File: //rose/source/kernel/infrastructure/framework/gui/rcmd_window.e $"

deferred class RCMD_WINDOW

inherit
	
	ROSE_LOGGER_FACILITY
		redefine
			new_representative, pre_execute, setup_logger
		end
	
	ELOG_SHARED_HIERARCHY
		export {NONE} all end
	
	ROSE_IO_VERBOSITY
		export {NONE} all end

	ROSE_SHARED_GLOBAL_GUI

feature -- Initialization

	auto_initialize_window is
			-- Create the command
		do
			if window = Void then
				initialize_window
			end
		end

feature -- Access

	window: EV_TITLED_WINDOW
			-- Window attached to this command

	new_representative: like Current is
			-- New independent version of Current service.
			-- If you want this service to be shared (unique instance used for the whole system),
			-- return Current (no clone)
		do
			if is_unique then
				Result := Current
			else
				Result := clone (Current)
			end
		end

	window_id: STRING is
			-- Window id used to remember window position
			-- Return Void to disable position remembering
		do
			if is_auto_closed then
				Result := "_" + id
			else
				Result := id
			end
		end

feature -- Status report

	is_unique: BOOLEAN is
			-- Is this window unique for the whole application?
		deferred
		end

	is_auto_closed: BOOLEAN
			-- Is the attached window to be automatically closed?
			-- Set this to true if you want to auto-close the attached window
			-- when all other application windows are closed

	show is
			-- Show 'window' associated to this command.
		do
			Gui.open_window (window_id,window)
		end

	hide is
			-- Hide 'window' associated to this command.
		do
			if window /= Void then
				Gui.close_window (window)
				if not is_unique then
					window := Void
				end
			end
		end

feature -- Scripting

	register_scripting_commands (a_env: GUI_SCRIPT_ENVIRONMENT) is
			-- Register commands for testing scripts.
			-- Descendants that redefine this routine can call
			--  Precursor at the start of the routine body, as later
			--  registrations override earlier ones.
		require
			a_env_not_void: a_env /= Void
		do
			a_env.register_command (create {GUI_SCRIPT_REFRESH_GUI}.make ("refresh_gui"))
			a_env.register_command (create {GUI_SCRIPT_EXIT}.make ("exit"))
			a_env.register_command (create {GUI_SCRIPT_WAIT}.make ("wait"))
			a_env.register_command (create {GUI_SCRIPT_FIRE_ACTIONS}.make ("fire_actions"))
			a_env.register_command (create {GUI_SCRIPT_SELECT_ROW}.make ("select_row"))
			a_env.register_command (create {GUI_SCRIPT_SELECT_ITEM}.make ("select_item"))
			a_env.register_command (create {GUI_SCRIPT_SELECT}.make ("select"))
			a_env.register_command (create {GUI_SCRIPT_DROP}.make ("drop"))
			a_env.register_command (create {GUI_SCRIPT_SET_PROPORTION}.make ("set_proportion"))
			a_env.register_command (create {GUI_SCRIPT_SORT_GRID_COLUMN}.make ("sort_grid_column"))
		end

	register_scripting_functions  (a_env: GUI_SCRIPT_ENVIRONMENT) is
			-- Register functions for testing scripts.
			-- Descendants that redefine this routine can call
			--  Precursor at the start of the routine body, as later
			--  registrations override earlier ones.
		require
			a_env_not_void: a_env /= Void
		do
			a_env.register_function (create {GUI_SCRIPT_ITEM}.make)
			a_env.register_function (create {GUI_SCRIPT_FIRST}.make)
			a_env.register_function (create {GUI_SCRIPT_SECOND}.make)
			a_env.register_function (create {GUI_SCRIPT_LAST}.make)
			a_env.register_function (create {GUI_SCRIPT_ITH}.make)
			a_env.register_function (create {GUI_SCRIPT_SELECTED_ITEM}.make)
			a_env.register_function (create {GUI_SCRIPT_CELL}.make)
			a_env.register_function (create {GUI_SCRIPT_PEBBLE}.make)
			a_env.register_function (create {GUI_SCRIPT_WINDOW_NAME_STARTING_WITH}.make)
		end

feature {NONE} -- Implementation
	
	setup_logger
			-- <Precursor>
		local
			l_logger: ELOG_LOGGER
			l_channel: ROSE_OUTCHANNEL
			l_appender: ELOG_OUTCHANNEL_APPENDER
		do
			if logfile_name /= Void or config.value (log_filename_key) /= Void then
				Precursor
			else
				-- Use the framework log channel
				l_logger := Log_hierarchy.logger ("GUI logger")
				create l_appender.make ("GUI error window", True)
				l_logger.add_appender (l_appender)
				l_logger.set_severity (logger_severity)
				l_channel := Global.io_handler.log
				if l_channel = Void then
					l_channel := Global.io_handler.output
				end
				l_channel.set_accepted_verbosity (verbosity_from_severity (logger_severity.level))
				l_logger.set_flush (Global.app.configured_boolean_value ("logging/flush", False))
				set_logger (l_logger)
			end
		end

	initialize_window is
			-- Initialize 'window'.
		deferred
		ensure
			window_or_error: window /= Void or else error.signaled
		end

	pre_execute is
			-- Pre-execution actions.
		do
			Precursor
			auto_initialize_window
		end

end

