indexing
	description : "ancestor of any class requiring a handle(rose_id)"
	author : "Mark Howard"
	keywords : "rose_id,handle,entity,persistent,persistence"
	last_editor: "$Author: c4 $"
	date: "$Date: 2014/08/06 $"
	revision: "$Revision: #16 $"
	path: "$File: //rose/source/kernel/entities/registered.e $"

deferred class REGISTERED

inherit
	
	MULTI_KEYED
		redefine
			is_equal
		end

feature -- Initialization

   make (a_handle: INTEGER) is
		require
			non_negative_handle: a_handle >= 0
		do
			handle := a_handle
		end

feature -- Access

	is_equal (a_other: like Current): BOOLEAN is
		do
			Result := supplier_id = a_other.supplier_id and then handle = a_other.handle
		end

	handle: INTEGER
			-- Non-negative identifier, unique within `supplier_id'

	key_count: INTEGER is 2

	hash_key (i: INTEGER): HASHABLE is
		do
			inspect i
			when 1 then
				Result := supplier_id
			else
				Result := handle
			end
		end

	supplier_id: INTEGER is
		deferred
		ensure
			supplier_id_is_unsigned_16_bit: Result >= 0 and Result <= {INTEGER_16}.max_value
		end

	is_valid: BOOLEAN is
		do
			Result := handle > 0
		end
		
	today: ROSE_DATE is
			-- Unshared version of today's date
		do
			create Result.make_today
		ensure
			today_attached: Result /= Void
		end

invariant

	non_negative_handle: handle >= 0

end
