note

	description:

		"Holds the solution of a REGRESSION"

	author: "Michael Zhang"
	last_editor: "$Author: it $"
	date: "$Date: 2013/02/13 $"
	revision: "$Revision: #5 $"
	path: "$File: //rose/source/kernel/infrastructure/math/regression_solution.e $"

class
	REGRESSION_SOLUTION

create
	make

feature {NONE} -- Initialization

	make(a_regression : REGRESSION; a_ix : INTEGER) is
			-- Initialize with a_regression, and index of dependent variable
		require
			valid_args: a_regression /= Void
		local
			l_yw2sum : DOUBLE
		do
			regression := a_regression
			dependent_variable_index := a_ix

			number_of_observations := regression.number_of_observations
			number_of_independent_variables := regression.number_of_independent_variables
			degree_of_freedom := regression.degree_of_freedom

			coefficients := regression.coefficients
			sigma_square := regression.sigma_square
			standard_errors := regression.standard_errors
			sum_of_squared_errors := regression.sum_of_squared_errors
			residual_variance := sum_of_squared_errors / regression.sum_w2

			l_yw2sum := regression.yw2sum.item(dependent_variable_index)
			y_average := l_yw2sum / regression.sum_w2
			y_variance := regression.y_total_variation / regression.sum_w2

			yhat_sum := coefficients * regression.xw2sum
			yhat_sumsq := coefficients * (regression.xtx @ coefficients)
			yhat_variance := (yhat_sumsq - yhat_sum * yhat_sum / regression.sum_w2) / regression.sum_w2

			r_square := regression.r_square
			adjusted_r_square := regression.adjusted_r_square

		 	independent_variable_names := regression.independent_variable_names
		end

	make_for_one_reg(a_regression : REGRESSION; a_ix : INTEGER) is
			-- Initialize for a_regression
		do
			regression := a_regression
			dependent_variable_index := a_ix

			number_of_observations := regression.number_of_observations
			number_of_independent_variables := regression.number_of_independent_variables
			degree_of_freedom := regression.degree_of_freedom

			coefficients := regression.solutions.item (a_ix).coefficients
			sigma_square := regression.solutions.item (a_ix).sigma_square
			sigma := regression.solutions.item (a_ix).sigma
			standard_errors := regression.solutions.item (a_ix).standard_errors
			sum_of_squared_errors := regression.solutions.item (a_ix).sum_of_squared_errors
			residual_variance := regression.solutions.item (a_ix).residual_variance

			y_average := regression.solutions.item (a_ix).y_average
			y_variance := regression.solutions.item (a_ix).y_variance

			yhat_sum := regression.solutions.item (a_ix).yhat_sum
			yhat_sumsq := regression.solutions.item (a_ix).yhat_sumsq
			yhat_variance := regression.solutions.item (a_ix).yhat_variance

			r_square := regression.solutions.item (a_ix).r_square
			adjusted_r_square := regression.solutions.item (a_ix).adjusted_r_square

		 	independent_variable_names := regression.independent_variable_names
		end

feature -- Solution
	number_of_observations : INTEGER
	number_of_independent_variables : INTEGER
	degree_of_freedom : INTEGER

	regression : REGRESSION
	coefficients : VECTOR
	standard_errors : VECTOR
	sum_of_squared_errors : DOUBLE
	sigma_square : DOUBLE
	sigma: DOUBLE

	yhat_sum, yhat_sumsq, yhat_variance : DOUBLE
	y_average, y_variance : DOUBLE
	residual_variance : DOUBLE
	r_square:DOUBLE
		-- R square

	adjusted_r_square: DOUBLE

	t_statistics : VECTOR is
			-- convert standard errors to t-statistics
		do
			Result := coefficients.divided_by(standard_errors)
		end

	display(a_output_file:PLAIN_TEXT_FILE; a_label:STRING) is
			-- display solution to text file
		local
			i: INTEGER
			l_tstat: VECTOR
			l_standard_errors : VECTOR
			l_fd85, l_fd74: FORMAT_DOUBLE
			l_output_file : PLAIN_TEXT_FILE
		do
			Create l_fd85.make(8,5); Create l_fd74.make(7,4)
			l_tstat := t_statistics
			l_standard_errors := standard_errors
			if a_output_file=Void then
				l_output_file := io.output
			else
				l_output_file := a_output_file
			end
			l_output_file.put_string(a_label+" Nobs=" + number_of_observations.out+"  Rsqr=" + l_fd74.formatted(r_square)+
					"  ARsqr="+l_fd74.formatted(adjusted_r_square)+"%N")
			from i:=1 until i> number_of_independent_variables loop
				l_output_file.put_string(variable_name(i) + "  " +
					l_fd85.formatted(coefficients.item(i))+
					" ("+l_fd85.formatted(l_tstat.item(i))+")   " +
					" ("+l_fd85.formatted(l_standard_errors.item(i))+") %N")
				i:=i+1
			end
			l_output_file.put_string("%N")
		end

	variable_name(i : INTEGER) : STRING is
			-- name of variable i
		do
			if independent_variable_names /= Void then
				Result := independent_variable_names.item(i)
			else
				Result := "Variable " + i.out
			end
		end

feature -- Description	
 	independent_variable_names : ROSE_LINEAR_ARRAY[STRING]	-- RHS variables
 	dependent_variable_name : STRING  -- LHS variable
 	dependent_variable_index : INTEGER

 	name : STRING
 	set_name(a_name : STRING) is
 			-- Set value of `name'
 		do
 			name := a_name
 		ensure
 			name_set: name = a_name
 		end
end
