note

	description:

		"Report expression"

	author: "Mark Howard"
	last_editor: "$Author: rj $"
	date: "$Date: 2012/12/13 $"
	revision: "$Revision: #38 $"
	path: "$File: //rose/source/kernel/data/reports/report_expression.e $"

class REPORT_EXPRESSION [reference G]

inherit

	ROSE_CONSTANTS

	ROSE_RECORD_ACCESS

	DEBUG_OUTPUT

	ROSE_IEEE_UNDEFINED

	CONTEXT_AWARE

create

	make

feature {NONE}--Creation

	make (a_parent: like expression_template; a_record: REPORT_RECORD) is
			--creation
		local
			l_spaces: STRING_8
		do
			record := a_record.twin
			parent := a_parent
			if parent /= Void then
				root := parent.root
			else
				root := Current
			end
			if record.component_adjustment /= Void then
				if record.component_adjustment.is_equal ("C") and parent /= Void then
					is_component := True
				elseif record.component_adjustment.is_equal ("R") then
					is_component := True
					is_residual := True
				end
			end
			description := existing_context_from_name ("RESEARCH").variable_description_by_mnemonic (a_record.mnemonic)
			if description /= Void then
				is_flow := description.stock_or_flow = 'F'
			end
			if a_parent /= Void then
				a_parent.add_child (Current)
				path := a_parent.path.twin
			else
				create path.make_empty
			end
			path.extend (Current)
			l_spaces := "  "
			l_spaces.multiply (depth)
			indented_name := l_spaces + name
		end

feature --Access

	tree_index: INTEGER_32

	set_tree_index (a_value: like tree_index) is
			--set tree index
		do
			tree_index := a_value
		end

	description: VARIABLE_DESCRIPTION

	is_component: BOOLEAN

	is_residual: BOOLEAN

	is_flow: BOOLEAN

	record: REPORT_RECORD

	indented_name: STRING_8

	children: ROSE_LINEAR_ARRAY [like expression_template]

	components: ROSE_LINEAR_ARRAY [like expression_template]

	parent: like expression_template

	add_child (a_child: like expression_template) is
		do
			if children = Void then
				create children.make_empty
				create components.make_empty
			end
			children.extend (a_child)
			a_child.set_order (children.count)
			if a_child.is_component then
				components.extend (a_child)
			end
		end

	data: COMPUTED_EXPRESSION [G]

	set_data (a_data: ROSE_EXPRESSION [G]; a_name: STRING_8; a_mnemonic: STRING_8) is
		local
			l_simple: UNKNOWN_EXPRESSION [G]
			l_variable_description: VARIABLE_DESCRIPTION_RECORD
			l_description: VARIABLE_DESCRIPTION
			l_character: CHARACTER_8
		do
			if a_data /= Void then
				l_simple ?= a_data
				if l_simple = Void then
					l_variable_description ?= a_data.variable_description
					if l_variable_description /= Void then
						l_variable_description := l_variable_description.twin
						l_variable_description.set_name (a_name)
						l_variable_description.set_mnemonic (a_mnemonic)
					else
						if a_data.variable_description /= Void then
							l_description := a_data.variable_description
							create l_variable_description.make (a_name, a_mnemonic, l_description.source, "unknown", "unknown", "unknown", l_description.parent, 0, 0, 0, 0, l_description.security, l_description.units, l_description.type, 0, 0, 0, 0, l_description.expression, "unknown", "unknown", l_description.scale, l_description.target, l_description.stock_or_flow, l_description.target_name, "unknown", l_description.variable_index)
						else
							create l_variable_description.make (a_name, a_mnemonic, "computed", "unknown", "unknown", "unknown", Void, 0, 0, 0, 0, 0, "", "", 0, 0, 0, 0, "", "unknown", "unknown", "", Void, l_character, "", "", 0)
						end
					end
					create data.make_from_expression (a_data, l_variable_description)
				end
			end
		end

	path: ROSE_LINEAR_ARRAY [like expression_template]	-- The list of expressions that leads to this expression (Current is last).

	reset_order (a_order: INTEGER_32) is
		local
			i: INTEGER_32
		do
			set_order (a_order)
			if children /= Void then
				from
					i := 1
				until
					i > children.count
				loop
					children.item (i).reset_order (i)
					i := i + 1
				end
			end
		end

	set_order (a_order: INTEGER_32) is
		do
			order := a_order
		end

	order: INTEGER_32

	display_string: STRING_8 is
		do
			if defined then
				if data /= Void then
					if data.type = Constants.String_type then
						Result := string_value
					else
						Result := data_value.out
					end
				else
					Result := ""
				end
				Result := Result + "[" + computed_value.out + "]"
			else
				if mapped then
					Result := "undefined" + "[" + computed_value.out + "]"
				else
					Result := "unmapped"
				end
			end
		end

	data_type: INTEGER_32

	value: REAL_64

	defined: BOOLEAN

	string_value: STRING_8

	date_value: ROSE_DATE

	computed_value: REAL_64

	partial_value: REAL_64

	target: like report_anchor

	market: MARKET

	currency: PERSISTENT_CURRENCY

	date: ROSE_DATE

	computed_mode: INTEGER_32


	mnemonic: STRING_8 is
		do
			Result := record.mnemonic
		end

	name: STRING_8 is
		do
			if description /= Void then
				Result := description.name
			elseif record /= Void and then record.name /= Void then
				Result := record.name
			else
				Result := "Unknown"
			end
		end

	debug_output: STRING_8 is
		do
			Result := Format.formatted (data_value) + " " + Format.formatted (computed_value) + name + " (" + mnemonic + ")"
		end

	format: FORMAT_DOUBLE is
		once
			create Result.make (12, 0)
		end

	component_sum_from_record (a_record: like component_record_template): REAL_64 is
		local
			i: INTEGER_32
			l_value: REAL_64
		do
			if components /= Void then
				from
					i := 1
				until
					i > components.count
				loop
					l_value := components.item (i).double_value_from_record (a_record)
					if is_defined_double (l_value) then
						Result := Result + l_value
					end
					i := i + 1
				end
			end
		end

	mapped: BOOLEAN

	depth: INTEGER_32 is
		do
			Result := path.count
		end

	report_anchor: G is
		do
		end

	key (a_any: ANY): BOOLEAN is
		do
			Result := True
		end

	root: like parent

feature  --Update

	add (a_expression: like Current) is
		do
			data_value := sum_imp (data_value, a_expression.data_value, a_expression.currency)
			computed_value := sum_imp (computed_value, a_expression.computed_value, a_expression.currency)
			partial_value := sum_imp (partial_value, a_expression.partial_value, a_expression.currency)
		end

	multiply (a_factor: REAL_64) is
		do
			if computed_mode /= 0 then
				if is_defined_double (data_value) then
					data_value := data_value * a_factor
				end
				if is_defined_double (computed_value) then
					computed_value := computed_value * a_factor
				end
				if is_defined_double (partial_value) then
					partial_value := partial_value * a_factor
				end
			end
		end

	init_computed_mode: INTEGER_32 is 1

	flat_computed_mode: INTEGER_32 is 2

	regular_computed_mode: INTEGER_32 is 3

	ai_computed_mode: INTEGER_32 is 4


	init (a_date: ROSE_DATE; a_market: MARKET; a_currency: PERSISTENT_CURRENCY) is
		local
			l_target: like report_anchor
		do
			save_states (l_target, a_date, a_market, a_currency)
			computed_mode := Init_computed_mode
			defined := True
			value := 0
			string_value := ""
			computed_value := 0
			data_value := 0
			partial_value := 0
			if children /= Void then
				from
					children.start
				until
					children.off
				loop
					children.current_item.init (a_date, a_market, a_currency)
					children.forth
				end
			end
		end

	compute (a_target: like report_anchor; a_date: ROSE_DATE; a_market: MARKET; a_currency: PERSISTENT_CURRENCY) is
		do
			compute_flat (a_target, a_date, a_market, a_currency)
			computed_mode := Regular_computed_mode
			if children /= Void then
				from
					children.start
				until
					children.off
				loop
					children.current_item.compute (a_target, a_date, a_market, a_currency)
					if children.current_item.is_component then
						if children.current_item.partial_value /= 0 then
							computed_value := computed_value + children.current_item.partial_value
						else
							computed_value := computed_value + children.current_item.computed_value
						end
					end
					children.forth
				end
			else
				computed_value := partial_value
			end
		end

	compute_ai (a_target: like report_anchor; a_date: ROSE_DATE; a_market: MARKET; a_currency: PERSISTENT_CURRENCY) is
		do
			compute_flat (a_target, a_date, a_market, a_currency)
			computed_mode := Ai_computed_mode
			if children /= Void then
				from
					children.start
				until
					children.off
				loop
					children.current_item.compute_ai (a_target, a_date, a_market, a_currency)
					if children.current_item.is_component then
						if children.current_item.computed_value = 0 then
							computed_value := computed_value + children.current_item.partial_value
						else
							computed_value := computed_value + children.current_item.computed_value
						end
					end
					children.forth
				end
			else
				computed_value := partial_value
			end
		end

	compute_flat (a_target: like report_anchor; a_date: ROSE_DATE; a_market: MARKET; a_currency: PERSISTENT_CURRENCY) is
		local
			l_simple: UNKNOWN_EXPRESSION [G]
			l_integer_data_value: INTEGER_32
		do
			save_states (a_target, a_date, a_market, a_currency)
			computed_mode := Flat_computed_mode
			defined := False
			value := 0
			string_value := ""
			computed_value := 0
			data_value := 0
			partial_value := 0
			mapped := False
			is_available := False
			if data /= Void then
				l_simple ?= data
				if l_simple = Void then
					data_value := data.undefined_double
					defined := data.defined (a_target)
					data_type := data.type
					if data.is_string then
						string_value := data.string_value (a_target)
						if string_value /= Void and then not string_value.is_equal (Constants.Missing_string) then
							is_available := True
						end
					elseif data.is_date then
						date_value := data.date_value (a_target)
						if date_value /= Void then
							is_available := True
						end
					elseif data.is_integer then
						l_integer_data_value := data.integer_value (a_target)
						if data.is_defined_integer (l_integer_data_value) then
							is_available := True
							data_value := l_integer_data_value
						end
					else
						data_value := data.value_in_market_currency (a_market, a_currency, a_target, a_date)
						if data.is_defined_double (data_value) then
							is_available := True
						else
							data_value := data.partial_value_in_market_currency (a_market, a_currency, a_target, a_date)
							is_available := data.is_defined_double (data_value)
						end
					end
					if data.is_defined_double (data_value) then
						partial_value := data_value
					else
						partial_value := 0
					end
					mapped := True
					if defined then
						value := data_value
					else
						value := Constants.Missing_double
					end
				end
			end
		end

	save_states (a_target: like report_anchor; a_date: ROSE_DATE; a_market: MARKET; a_currency: PERSISTENT_CURRENCY) is
		do
			target := a_target
			date := a_date
			market := a_market
			currency := a_currency
			if currency = Void and then data /= Void and then a_target /= Void then
				currency := data.currency (a_target)
			end
		end

feature {NONE} --Implementation

	sum_imp (a_local_value, a_input_value: REAL_64; a_currency: PERSISTENT_CURRENCY): REAL_64 is
		local
			l_value_1, l_value_2: REAL_64
		do
			l_value_1 := a_local_value
			l_value_2 := value_in_market_currency (a_input_value, a_currency)
			if not is_defined_double (l_value_1) then
				l_value_1 := 0
			end
			if not is_defined_double (l_value_2) then
				l_value_2 := 0
			end
			Result := l_value_1 + l_value_2
		end

	value_in_market_currency (a_value: REAL_64; a_currency: PERSISTENT_CURRENCY): REAL_64 is
		local
			l_factor: REAL_64
		do
			Result := a_value
			if data /= Void and then data.is_defined_double (Result) and then Result /= 0 then
				if (currency /= Void and a_currency /= Void) and then not currency.iso.is_equal (a_currency.iso) then
					l_factor := data.nominal_units.currency_conversion_factor (currency.market_exchange_rate (market, a_currency, date))
					if is_defined_double (l_factor) then
						Result := Result * l_factor
					else
						Result := Undefined_double
					end
				end
			end
		end



feature {NONE} --Templates

	expression_template: like Current is
		do
		end

	component_record_template: ANY is
		do
		end

end
