note

	description:

		"Report records"

	author: "Michael Zhang"
	last_editor: "$Author: er $"
	date: "$Date: 2013/02/28 $"
	revision: "$Revision: #10 $"
	path: "$File: //rose/source/kernel/data/reports/report_records.e $"

class
	REPORT_RECORDS

inherit

	ROSE_SHARED_GLOBAL

create
	make

feature {NONE} -- Initialization

	make (a_report_name: STRING)
			-- Create a new report records.
		do
			load_records(a_report_name)
		end

feature -- access

	variable_names: ROSE_LINEAR_ARRAY [STRING]
		local
			l_record : REPORT_RECORD
		do
			if records /= Void then
				create Result.make_empty
				from records.start until records.off loop
					l_record := records.item_for_iteration
					if l_record /= Void then
						Result.extend (l_record.name)
					end
					records.forth
				end
			end
		end

	all_mnemonics: ROSE_LINEAR_ARRAY [STRING]
		local
			l_record: REPORT_RECORD
		do
			if records /= Void then
				create Result.make_empty
				from records.start until records.off loop
					l_record := records.item_for_iteration
					if l_record /= Void then
						Result.extend (l_record.mnemonic)
					end
					records.forth
				end
			end
		end

	record_count: INTEGER
	records: HASH_TABLE[REPORT_RECORD, STRING]

feature {NONE} -- Implementation

	load_records (a_name: STRING)
		local
			l_hash_table_routines: HASH_TABLE_ROUTINES [REPORT_RECORD, STRING]
      	do
			 if not records_cache.has (a_name) then
			 	create records.make (10)
			    reader_predicate.apply (agent add_record, void, <<a_name>>)
			    if not reader_predicate.success then
			    	error.abort ("load_records", "Failed to retrieve report records for '$1'.", <<a_name>>)
			    elseif records /= Void then
			       records_cache.force(records, a_name)
			       record_count := records.count
			       create l_hash_table_routines
			       l_hash_table_routines.reorder_compatible (records)
			    end
			 else
			    records := records_cache.item (a_name)
			    record_count := records.count
			 end
		end

   	add_record (a_report_record: REPORT_RECORD)
   			-- Add a copy of `a_report_record' to `records'.
   		require
   			a_report_record_not_void: a_report_record /= Void
   			records_not_void: records /= Void
      	local
	 		l_record: REPORT_RECORD
     	do
			l_record := a_report_record.twin
			if l_record.mnemonic /= Void then
				l_record.mnemonic.to_lower
				l_record.mnemonic.prune_all (' ')
			end

			if l_record.parent /= void then
			    l_record.parent.to_lower
			    l_record.parent.prune_all (' ')
			end

			if l_record.name /= Void then
				l_record.name.to_lower
				l_record.name.prune_all (' ')
				records.force (l_record, l_record.mnemonic)
			end
      end

   server: DATA_SERVER
      local
			l_data_server : DATABASE_SERVER
      once
 			create l_data_server
	 		Result := l_data_server.servers.reports_server
      	end

   	reader_predicate: SIMPLE_PREDICATE [REPORT_RECORD]
      	once
	 		create Result.make (server,"select * from reports where report_name = ? order by mnemonic")
	 		if not Result.success then
	 			error.abort ("reader_predicate", "Failed to create predicate `reader_predicate'.", Void)
	 		end
	 	ensure
	 		reader_predicate_not_void: Result /= Void
      	end

   	records_cache: HASH_TABLE [HASH_TABLE [REPORT_RECORD, STRING], STRING]
      	once
	 		create Result.make (10)
	 	ensure
	 		records_cache_not_void: Result /= Void
      	end

end
