indexing
	description: "Record containning return information"
	author: "Michael Zhang"
	date: "$Date: 2009/03/03 $"
	revision: "$Revision: #8 $"
	archive: "$File: //rose/source/kernel/data/market/return_record.e $"

class
	RETURN_RECORD

inherit
	ROSE_CONSTANTS
	MISMATCH_CORRECTOR
		redefine
			correct_mismatch
		end

create
	make_undefined, make, make_new

feature -- Initialization

	make_undefined is
			-- make undefined
		do
			actions := Void
			terminal_distribution := False
			start_price := constants.missing_double
			end_price := constants.missing_double
			cum_factor := constants.missing_double
			apply_factor := False
			div := constants.missing_double
			distrib := constants.missing_double
			return := constants.missing_double
			yield := constants.missing_double
			currency := Void
			cum_cash_dividend := constants.missing_double
			cum_cash_inflow := constants.missing_double
			cum_cash_outflow := constants.missing_double
			net_cash_flow := constants.missing_double
		end

	make(rm_return : RM_RETURN) is
			-- make
		do
			if rm_return /= Void then
				actions := rm_return.actions_saved
				terminal_distribution := rm_return.terminal_distribution_saved
				start_price := rm_return.start_price_saved
				end_price := rm_return.end_price_saved
				cum_factor := rm_return.cum_factor_saved
				apply_factor := rm_return.apply_factor_saved
				div := rm_return.div_saved
				distrib := rm_return.distrib_saved
				return := rm_return.return - 1
				yield := rm_return.yield
				currency := rm_return.currency_saved
			end
		end

	make_new(rm_return : RM_RETURN_NEW) is
			-- make new
		do
			if rm_return /= Void then
				terminal_distribution := rm_return.has_terminal_distribution
				cum_cash_dividend := rm_return.cum_cash_dividend
				cum_cash_inflow := rm_return.cum_cash_inflow
				cum_cash_outflow := rm_return.cum_cash_outflow
				net_cash_flow := rm_return.net_cash_flow
				return := rm_return.return
				yield := rm_return.yield
				currency := rm_return.rmdat_end_currency
				cum_factor := rm_return.cum_factor
			end
		end

feature -- Access

	actions : ROSE_LINEAR_ARRAY[FOREGROUND_DIVS__W]
	terminal_distribution : BOOLEAN
	start_price : DOUBLE
	end_price : DOUBLE
	cum_factor : DOUBLE
	apply_factor : BOOLEAN
	div : DOUBLE
	distrib : DOUBLE
	return : DOUBLE
	yield : DOUBLE
	currency : CURRENCY

	cum_cash_dividend: DOUBLE
	cum_cash_inflow: DOUBLE
	cum_cash_outflow: DOUBLE
	net_cash_flow: DOUBLE

feature -- Basic operations

	set_cum_factor (a_value: like cum_factor) is
			-- set cum factor
		do
			cum_factor := a_value
		end

	set_currency (a_value: like currency) is
			-- set currency
		do
			currency := a_value
		end

	set_net_cash_flow (a_value: like net_cash_flow) is
			-- set net cash flow
		do
			net_cash_flow := a_value
		end

feature {NONE} -- Correction

	correct_mismatch is
			-- Attempt to correct object mismatch using `mismatch_information'.
		local
			l_mismatch: MISMATCH_INFORMATION
		do
			l_mismatch := mismatch_information
		end
end
