indexing
	description: "Signature tagged record for reuters prices"
	author: "Randy John"
	date: "$Date: 2009/08/24 $"
	revision: "$Revision: #12 $"
	path: "$File: //rose/source/kernel/supplier_data/reuters2_price_target.e $"

class
	REUTERS2_PRICE_TARGET

inherit
	SUPPLIER_DATA_SIGNATURE[REUTERS_PRICE_LISTING]
		undefine
			context
		redefine
			fields_,
			new_variable,
			new_variable_template,
			data_manager,
			restriction_statement,
			currency_imp,
			date,
			output_string_by_name,
			output_data_field_string_by_name
		end

	REUTERS_AWARE

	ROSE_AWARE
		rename
			context as rose_context
		end

	ROSE_PERIODICITY_ID_USE
		undefine
			is_equal
		end
create
	default_create

create {REUTERS2_PRICE_TARGET}
	rose_entity_date_make

feature -- Access

    new_variable (a_description : VARIABLE_DESCRIPTION_RECORD) : REUTERS2_PRICE_EXPRESSION is
    		-- New variable corresponding to 'a_description'
		do
			create Result.make(a_description)
			Result.set_restriction_statement(restriction_statement)
		end

	date: ROSE_DATE is
		do
			Result := period_end_date.date
		end

feature -- Output

	output_string_by_name(a_name: STRING): STRING is
	 		-- String representation
		local
			l_name: STRING
			l_currency: PERSISTENT_CURRENCY
		do
			l_name := a_name.as_lower
			if l_name.is_equal("currency_id")
			then
				l_currency ?= context.entity_at(rose_context.currency_category_id, currency_id)
				if l_currency/=Void
				then
					Result := l_currency.iso_code_key(Void)
				end
			elseif l_name.is_equal("periodicity_id")
			then
				Result := periodicity_ids.periodicity_name_from_id(periodicity_id)
				Result.prune_all_trailing(' ')
			elseif l_name.is_equal("entity_id")
			then
				if entity=Void
				then
					entity ?= context.entity_at(rose_context.listing_category_id, entity_id)
				end

				if entity/=Void
				then
					Result := entity.attribute_by_name ("REUTERS CODE", Void)
				end
			end

			if Result=Void
			then
				Result := Precursor(a_name)
			end
		end

	output_data_field_string_by_name(a_name: STRING): STRING is
		do
			Result := output_string_by_name(a_name)
		end

feature -- Access

	data_manager : SIGNATURE_TAGGED_ITEM_DATABASE[REUTERS2_PRICE_TARGET] is
		once
			create Result.make_named("REUTERS2", "read data", servers.reuters_server)
		end

feature --Descriptors

	fields_: STRING is "entity_id,period_end_date,currency_id,periodicity_id,value_description_id"	-- No spaces!
	expression_tag : STRING is "listing"
	context_name : STRING is "REUTERS"

	restriction_statement : STRING is
			-- restriction statement
		do
			if is_direction_time_series then
				Result := time_series_statement
			elseif is_direction_cross_sectional then
				Result := cross_sectional_statement
			elseif is_direction_neighborhood then
				Result := neighborhood_statement
			else
				Result := time_series_statement
			end
		end

	time_series_statement : STRING is
			-- time series statement
		"#entity_id.id = entity_id"

	cross_sectional_statement : STRING is
			-- cross section statement
		"#period_end_date.id = period_end_date"

	neighborhood_statement : STRING is
			-- neighborhood statement
		"#entity_id.id = entity_id and #period_end_date.id = period_end_date" -- and #currency_id.id = currency_id and #periodicity_id.id = periodicity_id"

feature --currency
	currency_imp : PERSISTENT_CURRENCY is
			-- currency imp
		local
			l_source_currency_id : INTEGER
			l_iso : STRING
		do
			if currency_id /= 0 then
				Result := context.currency_accessor.entity(currency_id)
				if Result /= Void and then Result.is_valid then
					l_iso := Result.attribute_by_name (iso_attribute_name, date)
					if l_iso /= Void and then currency_convert_mapping_lookup.has (l_iso) then
						l_source_currency_id := currency_convert_mapping_lookup.name_from_ukey (l_iso)
						if l_source_currency_id > 0 then
							Result := context.currency_accessor.entity(l_source_currency_id)
						end
					end
				end
			end
		end

	is_convert_currency : BOOLEAN	 is
		-- check if need convert
		local
			l_currency : like currency_imp
			l_iso_code : STRING
		do
			Result := False
			l_currency := context.currency_accessor.entity(currency_id)
			if l_currency /= Void then
				l_iso_code := l_currency.attribute_by_name (iso_attribute_name, date)
				if l_iso_code /=Void and then currency_convert_lookup.has (l_iso_code) then
					Result := True
					currency_convert_factor := currency_convert_lookup.name_from_ukey (l_iso_code)
				end
			end
		end

	iso_attribute_name : STRING is
			-- iso attribute name
		"ISO CODE"

	currency_convert_factor: INTEGER
				-- convert factor

	currency_convert_lookup: ROSE_LOOKUP_TABLE[INTEGER, STRING] is
				-- convert factor lookup table
		once
			create Result.make_table( servers.reuters_server, "currency_convert_lookup", "target_iso_code", "factor")
		end

	currency_convert_mapping_lookup: ROSE_LOOKUP_TABLE[INTEGER, STRING] is
				-- convert factor
		once
			create Result.make_table( servers.reuters_server, "currency_convert_lookup", "target_iso_code", "source_currency_id")
		end

feature --Templates

	new_variable_template : REUTERS2_PRICE_EXPRESSION is
			--template
		do end
end
