indexing
	description: "signature tagged record for reuters prices"
	author: "Randy John"
	date: "$Date: 2009/08/24 $"
	revision: "$Revision: #15 $"
	path: "$File: //rose/source/kernel/supplier_data/reuters_price_target.e $"

class
	REUTERS_PRICE_TARGET

inherit
	SUPPLIER_DATA_SIGNATURE [REUTERS_PRICE_LISTING]
		undefine
			context
		redefine
			new_variable,
			data_manager,
			restriction_statement,
			currency_imp,
			date,
			output_string_by_name,
			output_data_field_string_by_name
		end

	REUTERS_AWARE

	ROSE_AWARE
		rename
			context as rose_context
		end

	ROSE_PERIODICITY_ID_USE
		undefine
			is_equal
		end

create
	default_create

create {REUTERS_PRICE_TARGET}
	rose_entity_date_make

feature  --Access

    new_variable (a_description : VARIABLE_DESCRIPTION_RECORD) : REUTERS_PRICE_EXPRESSION is
    		-- New variable corresponding to 'a_description'
      do
	 	create Result.make(a_description)
	 	Result.set_restriction_statement(restriction_statement)
      end

	expression_tag : STRING is "listing"
	context_name : STRING is "REUTERS"

	date : ROSE_DATE is
			--period end date
		do
			Result := period_end_date.date
		end

	output_string_by_name(a_name: STRING): STRING is
		local
			l_name: STRING
			l_currency: PERSISTENT_CURRENCY
		do
			l_name := a_name.as_lower
			if l_name.is_equal("currency_id")
			then
				l_currency ?= context.entity_at(rose_context.currency_category_id, currency_id)
				if l_currency/=Void
				then
					Result := l_currency.iso_code_key(Void)
				end
			elseif l_name.is_equal("periodicity_id")
			then
				Result := periodicity_ids.periodicity_name_from_id(periodicity_id)
				Result.prune_all_trailing(' ')
			elseif l_name.is_equal("entity_id")
			then
				if entity=Void
				then
					entity ?= context.entity_at(rose_context.listing_category_id, entity_id)
				end

				if entity/=Void
				then
					Result := entity.attribute_by_name ("REUTERS CODE", Void)
				end
			end

			if Result=Void
			then
				Result := Precursor(a_name)
			end
		end

	output_data_field_string_by_name(a_name: STRING): STRING is
			-- String representation for 'a_name'
		do
			Result := output_string_by_name(a_name)
		end

feature -- Sql

	data_manager : SIGNATURE_TAGGED_ITEM_DATABASE[REUTERS_PRICE_TARGET] is
			-- Shared data manager (reuters server)
		once
			create Result.make("read data", servers.reuters_server)
		end

	restriction_statement : STRING is
		do
			if is_direction_time_series then
				Result := time_series_statement
			elseif is_direction_cross_sectional then
				Result := cross_sectional_statement
			elseif is_direction_neighborhood then
				Result := neighborhood_statement
			else
				Result := time_series_statement
			end
		end
	time_series_statement : STRING is "#entity_id.id = entity_id"
	cross_sectional_statement : STRING is "#period_end_date.id = period_end_date"
	neighborhood_statement : STRING is "#entity_id.id = entity_id and #period_end_date.id = period_end_date"

feature --currency

	currency_imp : PERSISTENT_CURRENCY is
		do
			if currency_id /= 0 then
				Result := context.currency_accessor.entity(currency_id)
			end
		end

	is_GBp : BOOLEAN	 is
		local
			l_currency : like currency_imp
			l_iso_code : STRING
		do
			Result := False
			l_currency := context.currency_accessor.entity(currency_id)
			if l_currency /= Void then
				l_iso_code := l_currency.attribute_by_name ("ISO CODE", date)
				if l_iso_code /=Void and then l_iso_code.is_equal("GBp") then
					Result := True
				end
			end
		end
		
end
