indexing
	description: "Computes the holding period return for a listing given a start and end date, using legacy capital action representation"
	author: "Original PF?,NC,SI"
	last_editor: "$Author: er $"
	date: "$Date: 2014/02/26 $"
	revision: "$Revision: #42 $"
	path: "$File: //rose/source/kernel/data/market/rm_return.e $"

			-- TO DO: fix l_end_price_date
			-- (come back from ermdat?)

class
	RM_RETURN

inherit
	DFO_CONSTANTS
	ROSE
	ROSE_CONSTANTS
	ROSE_LEGACY_UNDEFINED
	ROSE_ACCESS
	ATTRIBUTES
		rename
			sedol as attributes_sedol,
			cusip as attributes_cusip,
			company_name as attributes_company_name,
			msci as attributes_msci
		end


create
	-- obsolete do not create (legacy via ZONAL_RETURN)

feature	{NONE} -- Creation

	make(a_market_data: MARKET_DATA) is
			-- Create the return object
		require
			valid_market_data : a_market_data /= Void
		do
			market_data := a_market_data
			turn_printing_off_in_period_return
		end

feature -- Basic operations

	period_return(a_rxrf: INTEGER; a_start_date, a_end_date: ROSE_DATE) is
			-- Compute the period return for a_rxrf from a_start_date to a_end_date, store results in the object
		require
			valid_rxrf: a_rxrf > 0
			valid_dates: a_start_date /= Void and a_end_date /= Void
			meaningful_dates: a_start_date <= a_end_date
		local
			l_start_price,l_end_price: DOUBLE
			l_fx_conversion: DOUBLE
			-- l_fx_conversion_iso: DOUBLE
			l_factor: DOUBLE
			l_exchange_rate: DOUBLE
			l_div,l_cashdiv,l_distrib,l_dist,l_cum_factor,l_cum_factor_price: DOUBLE
			-- Do not delete variables and references below until migration to rm_return_new
			-- l_return_iso, l_div_iso, l_dist_iso, l_cashdiv_iso: DOUBLE -- l_cashdiv_rate, l_div_rate, l_return_rate, l_dist_w, l_distrib_w, l_return_distw: DOUBLE
			l_iso: STRING
			l_date: ROSE_DATE
			l_end_price_date: ROSE_DATE
			l_terminal_distribution: BOOLEAN
			l_apply_factor: BOOLEAN
			-- Do not delete variables and references below until migration to rm_return_new			
			-- l_terminal_distribution_distw, l_dist_in_period: BOOLEAN
			--/
			l_actions: ROSE_LINEAR[FOREGROUND_DIVS__W]
			l_distrib_markers:ROSE_LINEAR_ARRAY[FOREGROUND_DIVS__W]
			l_backdiv:BACKDIV
			l_action: FOREGROUND_DIVS__W
			--/-
			l_rmdat_start_currency, l_rmdat_end_currency, l_action_currency: CURRENCY
			-- l_action_currency_iso: CURRENCY
			l_action_country: COUNTRY
			-- l_action_country_iso: COUNTRY
			l_delta_shares_flag: BOOLEAN
			l_last_cap_date: INTEGER
			-- Do not delete below until migration to rm_return_new
			-- l_listing: LISTING l_holding: BOOLEAN
		do
			--/
			l_start_price := price(a_rxrf,a_start_date)
			if l_start_price > 0 then
				l_rmdat_start_currency := currency(a_rxrf,a_start_date)
				l_end_price := price(a_rxrf,a_end_date)
				l_rmdat_end_currency := currency(a_rxrf,a_end_date)
				if (l_rmdat_start_currency /= Void and l_rmdat_end_currency /= Void) and then l_rmdat_start_currency /= l_rmdat_end_currency then
					l_exchange_rate := l_rmdat_end_currency.exchange_rate(l_rmdat_start_currency,a_start_date)
					if is_defined_double (l_exchange_rate) then
						l_start_price := l_start_price * l_exchange_rate
					else
						l_start_price := undefined_real
					end
				end
				if l_start_price > 0 then
					l_actions := capital_actions(a_rxrf)
					-- loop over the dividends returned by latrdv (in market_data)
					create actions_saved.make_empty

					from
						l_actions.start
						l_fx_conversion := 1.0
						-- l_fx_conversion_iso := 1.0
						l_factor := 1.0
						l_apply_factor := True
						l_cum_factor := 1.0
						l_cum_factor_price := 1.0
						create l_distrib_markers.make_empty
					until
						l_actions.off
					loop
						l_action := l_actions.item
						if l_actions.index=41 then
							do_nothing
						end
						l_fx_conversion := 1.0
						-- l_fx_conversion_iso := 1.0
						l_cashdiv := 0.0
						-- l_cashdiv_rate := 0.0
						-- l_cashdiv_iso := 0.0
						l_dist := 0.0
						-- l_dist_w := 0.0
						if (not l_action.type.is_equal("84") and
							not l_action.type.is_equal("88") and
							not l_action.type.is_equal("89") and
							not l_action.type.is_equal("90")) then
							-- dfo exdate's are only yymmdd (not yyyy)
							-- nope now they're ccyymmdd
							create l_date.make_ccyymmdd(l_action.exdate)
							-- if l_date >= a_start_date and l_date <= a_end_date then
							-- <HD: Aug. 22,02> above line leads to double-counting; it is commented out and replaced by line below.
							if l_date > a_start_date and l_date <= a_end_date then
								actions_saved.extend(l_action)
								-- kluge introduced by <HD: Aug. 22,02>
								l_end_price_date := a_end_date
								l_action_country := context.country_at (iso_code(l_action.iso) ,Void)
								if zone_mnemonic ~ 'C' then
									if l_action.paymt ~ '1' then -- in DFO, iso is always CA; payment field reflects currency
										l_action_country := context.country_at (iso_code ("US"), Void)
									end
								end
								--	if l_action_country.is_member_of_european_union then
								if l_action.iso.substring(1,2).is_equal("EU")   then
									l_action_currency := context.eur
									-- l_action_currency_iso := euro
								else
									if l_action_country /= Void then
										from
											l_action_country.currency_history.start
										until
											l_action_country.currency_history.after or l_action_currency /= Void
										loop
											l_action_currency ?= l_action_country.currency_history.current_item.entity
											if l_action_country.currency_history.count > 1 then
												if not l_action_country.currency_history.current_item.brackets (l_date) then
													l_action_currency := Void
												end
											end
											if zone_mnemonic ~ 'C' then -- see comment above about incorrect ISO in DFO
												l_iso := "CA" -- this code reflects the Fortran code in event processing.
												if l_action.paymt ~ '1' then -- the paymt field 1 is in Fortran as provided by supplier
													l_iso := "US"
												end
											else
												l_iso := l_action.iso
											end
											if l_action_currency /= Void and then not l_action_currency.iso.substring (1, 2).is_equal (l_iso) then
												l_action_currency := Void
											end
											l_action_country.currency_history.forth
										end
									end

								end
								--if (l_action_currency_iso /= Void and l_rmdat_end_currency /= Void) and then l_action_currency_iso /= l_rmdat_end_currency then
								--	l_fx_conversion_iso := l_rmdat_end_currency.exchange_rate(l_action_currency_iso,create {ROSE_DATE}.make_ccyymmdd(l_action.exdate))
								--end
								if (l_action_currency /= Void and l_rmdat_end_currency /= Void) and then l_action_currency /= l_rmdat_end_currency then
									l_fx_conversion := l_rmdat_end_currency.exchange_rate(l_action_currency, create {ROSE_DATE}.make_ccyymmdd(l_action.exdate))
								end
								l_delta_shares_flag:= False
								--use this flag to avoid double counting of split
								if l_action.broad = STOCK_SPLIT or
									l_action.broad = STOCK_DIV or
									l_action.broad = DISTRIBUTION then
									if l_action.sh_factor /= 0.0 then
										l_factor := l_action.sh_factor
									end
									l_delta_shares_flag:= True
								end
								if l_action.broad = CASH then
									if zone_mnemonic ~ 'C' then
										l_cashdiv := l_action.cash_rate * l_fx_conversion
										-- l_cashdiv_iso := l_action.cash_rate * l_fx_conversion_iso
									else
										if l_action.cash_adj /= 0.0 then
									 		l_cashdiv := l_action.cash_adj*l_fx_conversion
										else
											l_cashdiv := l_action.cash_rate*l_fx_conversion
										end
									end
									-- l_cashdiv_rate := l_action.cash_rate * l_fx_conversion
									l_cashdiv := l_cashdiv*l_cum_factor
									-- l_cashdiv_rate := l_cashdiv_rate * l_cum_factor
									-- l_cashdiv_iso := l_cashdiv_iso * l_cum_factor
									if l_action.on_factor > 0.0 then
										l_cashdiv := l_cashdiv*l_action.on_factor
										-- l_cashdiv_rate := l_cashdiv_rate * l_action.on_factor
										-- l_cashdiv_iso := l_cashdiv_iso * l_action.on_factor
									end
								elseif l_action.broad = DISTRIBUTION and
									l_action.sh_factor > 0.0 and
									l_action.cash_rate < 0.0 then
									l_dist := l_action.cash_rate*l_fx_conversion
									-- l_dist_iso := l_action.cash_rate * l_fx_conversion_iso
									if l_action.on_factor = 0.0 and
										l_action.exdate = l_last_cap_date then
										l_apply_factor := False
									end
								elseif l_action.broad = DISTRIBUTION then
									l_distrib_markers.extend(l_action)
									l_dist := l_action.cash_rate*l_cum_factor+l_action.cash_adj*l_cum_factor
									-- l_dist_iso := l_action.cash_rate*l_cum_factor+l_action.cash_adj*l_cum_factor
									-- if zone_mnemonic /= 'U' and	zone_mnemonic /= 'C' then
									l_backdiv := get_distribution_value (a_rxrf, zone_mnemonic.out, l_action.exdate, l_rmdat_end_currency )
									if l_backdiv /= Void then
										l_dist := l_backdiv.cash_rate *l_cum_factor
										-- l_dist_iso := l_backdiv.cash_rate * l_cum_factor
										if l_backdiv.sh_factor > 0  then
											l_factor := l_backdiv.sh_factor
										end
									end
									-- end
									-- the "if" block below is to compare returns when using dist_w to production returns
									-- this is not intended for production but may be useful again.
									-- Do not delete until migration to rm_return_new
									--if zone_mnemonic = 'U' or zone_mnemonic = 'C' then
									--	l_dist_in_period := True
									--	l_backdiv := get_distribution_value (a_rxrf, zone_mnemonic.out, l_action.exdate, l_rmdat_end_currency )
									--	if l_backdiv /= Void then
									--		l_dist_w := l_backdiv.cash_rate *l_cum_factor
									--		if l_backdiv.sh_factor > 0  then
									--			l_factor := l_backdiv.sh_factor
									--		end
									--	end
									--end

									-- handle terminating dividend
									--if l_action.sh_factor = 0.0 then
									--	l_ex_price:= price(a_rxrf,l_date)
									--	if l_dist >= 0.8*l_start_price and
									--		l_end_price_date <= l_date then
									--		l_terminal_distribution := True
									--	elseif l_dist > 0.95* l_ex_price and l_actions.index = l_actions.count then
									--		--more stringent rule by tm
									--		l_terminal_distribution:= True
									--	end
									--end
									if l_backdiv /= Void then
										if l_backdiv.sh_factor = 0 then
											l_terminal_distribution := True
										end
									end
								end

								if l_delta_shares_flag then
									if l_apply_factor = True then
										l_cum_factor_price := l_cum_factor_price*l_factor
										l_cum_factor := l_cum_factor*l_factor
									else
										if l_cum_factor = 1.0 then
											l_cum_factor_price := l_cum_factor_price*l_factor
											l_cum_factor := l_cum_factor*l_factor
										else
											l_cum_factor_price := l_cum_factor_price+(l_factor-1.0)
											l_cum_factor := l_cum_factor+(l_factor-1.0)
										end
									end
								end
								l_div := l_div+l_cashdiv
								-- l_div_rate := l_div_rate + l_cashdiv_rate
								-- l_div_iso := l_div_iso + l_cashdiv_iso
								l_distrib := l_distrib+l_dist
								-- Do not delete below: used for test.  See comments above.
								-- l_distrib_w := l_distrib_w + l_dist_w
							end
							l_last_cap_date := l_action.exdate
						end
						l_actions.forth
					end

					-- compute return and yield
					yield := l_div / l_start_price
					if l_terminal_distribution then
						return := (l_div+l_distrib)/l_start_price
						-- l_return_rate := (l_div_rate + l_distrib) / l_start_price
						-- l_return_iso := (l_div_iso + l_distrib) / l_start_price
					else
						if l_end_price > 0 then
							-- next statement placed inside if-clause by <HD:Sep. 25,02>
							return := (l_end_price*l_cum_factor+l_div+l_distrib)/l_start_price
							-- l_return_rate := (l_end_price*l_cum_factor+ l_div_rate +l_distrib)/l_start_price
							-- l_return_iso := (l_end_price * l_cum_factor + l_div_iso + l_distrib) / l_start_price
						else
							return := undefined_real  -- this part of if-clause introduced by <HD:Sep. 25,02>
						end
					end
					-- the "if" block below is just to test terminal distributions based on parent units and compare to production
					-- this not intended for production
					-- Do not delete as it may become useful again for the migration to rm_return_new
					--if zone_mnemonic = 'E' or zone_mnemonic = 'O' then
					--	if l_backdiv /= Void then
					--		if l_terminal_distribution /= l_terminal_distribution_distw then
					--			l_listing := context.listing_from_xrfzone (a_rxrf.out + zone_mnemonic.out, a_start_date)
					--			if l_listing /= Void then
					--				l_holding := l_listing.is_held_during (a_start_date, a_end_date)
					--			end
					--			-- actually, if we replace "if l_backdiv.sh_factor > 0" by "if l_backdiv.sh_factor >= 0", then
					--			-- l_factor would be correct in all cases and equal to 0 for terminal distributions.  Then, returns would be correct in all cases
					--			-- without a need for terminal distribution checks, because terminal distributions will result in l_cum_factor = 0
					--			-- This code change is not done now because it would be require many other code changes to compute two sets of returns in parallel in code.
					--			-- that we would then compare and reconcile.
					--			-- For now, simpler code change is done to check on results.
					--			-- Better code change may not be needed anyway because of planned migration to rm_return_new
					--			if l_terminal_distribution then
					--				l_return_distw := (l_end_price*l_cum_factor+l_div+l_distrib)/l_start_price
					--			else
					--				l_return_distw := (l_div+l_distrib)/l_start_price
					--			end
					--			return_distribution_file.putstring (a_rxrf.out + "|" + a_start_date.ccyymmdd.out + "|" + l_terminal_distribution.out + "|" + l_backdiv.sh_factor.out + "|" + return.out + "|" + l_return_distw.out + "|" + l_end_price.out + "|" + l_cum_factor.out + "|" + l_div.out + "|" + l_distrib.out + "|" + l_start_price.out + "|" + l_holding.out + "%N")
					--			return_distribution_file.flush
					--			print (a_rxrf.out + "|" + a_start_date.ccyymmdd.out + "|" + l_terminal_distribution.out + "|" + l_backdiv.sh_factor.out + "|" + return.out + "|" + l_return_distw.out + "|" + l_end_price.out + "|" + l_cum_factor.out + "|" + l_div.out + "|" + l_distrib.out + "|" + l_start_price.out + "|" + l_holding.out + "%N")
					--		end
					--	end
					--end
					--if l_return_iso /= return then
					--	return_distribution_file.putstring (a_rxrf.out + "|" + a_start_date.ccyymmdd.out + "|" + l_start_price.out + "|" + l_end_price.out + "|" + l_div.out + "|" + l_div_iso.out + "|" + return.out + "|" + l_return_iso.out + "%N")
					--	return_distribution_file.flush
					--end

					-- the "if" block below is to test computations of C and U returns based on dist_w in parallel to production returns
					-- this is not intended for production.
					-- Do not delete as it may be useful again for the migration to rm_return_new
					--if zone_mnemonic = 'C' or zone_mnemonic = 'U' then
					--	if l_dist_in_period and l_backdiv /= Void then
					--		l_listing := context.listing_from_xrfzone (a_rxrf.out + zone_mnemonic.out, a_start_date)
					--		if l_listing /= Void then
					--			l_holding := l_listing.is_held_during (a_start_date, a_end_date)
					--		end
					--		if l_terminal_distribution_distw then
					--			l_return_distw := (l_div + l_distrib_w)/l_start_price
					--		else
					--			if l_end_price > 0 then
					--				l_return_distw := (l_end_price * l_cum_factor + l_div + l_distrib_w)/l_start_price
					--			else
					--				l_return_distw := undefined_real  -- this part of if-clause introduced by <HD:Sep. 25,02>
					--			end
					--		end
					--		return_distribution_file.put_string (a_rxrf.out + "|" + a_start_date.ccyymmdd.out + "|" + l_backdiv.sh_factor.out + "|" + l_end_price.out + "|" + l_cum_factor.out + "|" + l_div.out + "|" + l_distrib.out + "|" + l_distrib_w.out + "|" + l_start_price.out + "|" + return.out + "|" + l_return_distw.out + "|" + l_holding.out + "%N")
					--		return_distribution_file.flush
					--		print (a_rxrf.out + "|" + a_start_date.ccyymmdd.out + "|" + l_backdiv.sh_factor.out + "|" + l_end_price.out + "|" + l_cum_factor.out + "|" + l_div.out + "|" + l_distrib.out + "|" + l_distrib_w.out + "|" + l_start_price.out + "|" + return.out + "|" + l_return_distw.out + "|" + l_holding.out + "%N")
					--	end
					--end

				else
					return := undefined_real
					yield :=  undefined_real
				end
			else
				return := undefined_real
				yield :=  undefined_real
			end

			--save the status of this run
			terminal_distribution_saved := l_terminal_distribution
			start_price_saved := l_start_price
			end_price_saved := l_end_price
			cum_factor_saved := l_cum_factor
			apply_factor_saved := l_apply_factor
			div_saved := l_div
			distrib_saved := l_distrib
			currency_saved := l_rmdat_end_currency

			if execute_period_return_with_printing then
				if return > 2 or return < .5 then
					print_all(<<"!! return = ",return,"  rxrf = ",a_rxrf,"%N">>)
					print_all(<<"!! start_date = ",a_start_date,"  end_date = ",a_end_date,"%N">>)
					print_all(<<"!! start_price = ",l_start_price,"  end_price = ",l_end_price,"%N">>)

					--	    print("PRESS RETURN TO CONTINUE%N")
					--	    io.readline
				end
			end
			--/-			
		end

	-- test file: see usage in comments abobe
	-- Do not delete: may become useful again for migration to rm_return_new.
	--return_distribution_file: ROSE_TEXT_FILE is
	--		--
	--	once
	--		create Result.make_open_read_write ("\\axaros.com\shares\pub_nt\yc\projects\monthly_rmdata_update\returns_iso" + zone_mnemonic.out + ".txt")
	--	end


	turn_printing_on_in_period_return is
			-- turns printing on in period return
		do
			execute_period_return_with_printing := True
		ensure
			set: execute_period_return_with_printing
		end

	turn_printing_off_in_period_return is
			-- turns printing off in period return
		do
			execute_period_return_with_printing := False
		end

feature -- Access

	yield, return: DOUBLE
			-- return values
	error: INTEGER
			-- error reporting
	actions_saved: ROSE_LINEAR_ARRAY[FOREGROUND_DIVS__W]
	terminal_distribution_saved: BOOLEAN
	start_price_saved: DOUBLE
	end_price_saved: DOUBLE
	cum_factor_saved: DOUBLE
	apply_factor_saved: BOOLEAN
	div_saved: DOUBLE
	distrib_saved: DOUBLE
	currency_saved: CURRENCY

feature {NONE} -- Implementation

	execute_period_return_with_printing: BOOLEAN

	price(a_rxrf: INTEGER; a_date: ROSE_DATE): DOUBLE is
			-- last closing price
		do
			Result := market_data.daylp(a_rxrf, a_date)
		end

	currency(a_rxrf: INTEGER; a_date: ROSE_DATE): CURRENCY is
		do
			Result := market_data.currency(a_rxrf, a_date)
		end

	capital_actions(a_rxrf: INTEGER): ROSE_LINEAR[FOREGROUND_DIVS__W] is
		do
			Result := market_data.capital_actions(a_rxrf)
		end

	zone_mnemonic: CHARACTER is
			-- zone mnemonic
		do
			Result := market_data.czone
		end

feature {NONE} -- Implementation

	market_data: MARKET_DATA
		-- Market data
	get_distribution_value (a_rxrf: INTEGER; a_zone: STRING; a_exdate: INTEGER; a_currency: CURRENCY): BACKDIV is
			-- in returned BACKDIV sets (1) sh_factor from parent of distribution, (2) cash_rate with total of (a) cash
			-- from cash children and (b cash equivalent from other children, converted to input currency
			--(3) also sets rxrf,tzone and exdate for checking
			-- if Result=Void, there was no or invalid distribution to match the marker record; should probably warn of this
		local
			l_dist: DIST_W_ROW
			l_value, l_total_value: DOUBLE
			l_ex_rate: DOUBLE
			l_currency: CURRENCY
			--l_parent_currency: CURRENCY
			l_date: ROSE_DATE
			l_distribs: ROSE_LINEAR_ARRAY[DIST_W_ROW]
		do
			l_distribs := retrieve_distributions(a_zone, a_rxrf, a_exdate)
			if l_distribs /= Void then
				--for i=1, parent should=child; set sh_factor from units
				l_dist := l_distribs.first
				if l_dist.brimsym /= Void and then l_dist.brimsym.item(1) > ' ' and then l_dist.rxrf =  l_dist.ch_rxrf then
					create Result
					Result.set_sh_factor (l_dist.units)
					create l_date.make_ccyymmdd(a_exdate)
				else
					do_nothing ---- what action? this should be parent but isn't -- skip this distribution
				end
				-- accumulates values of children, both cash and equity
				if Result /= Void then
					Result.set_rxrf (l_dist.rxrf)
					Result.set_tzone (l_dist.user_zone)
					Result.set_exdate (l_dist.exdate)
					from l_distribs.start_forth
					until l_distribs.off
					loop
						l_dist := l_distribs.current_item
						l_value := l_dist.units * l_dist.basis_price
						if a_currency /= Void and then a_currency.iso_code_key (l_date) /= Void and then (not l_dist.isoq.substring(1,2).is_equal (a_currency.iso_code_key (l_date).substring(1,2))) then
							-- convert l_value to input currency
							l_currency := currency_from_country_iso (l_dist.isoq, l_date)
							if l_currency /= Void then
								l_ex_rate :=  a_currency.exchange_rate(l_currency, l_date)
								if is_defined_double (l_ex_rate) then
									l_value := l_value * l_ex_rate
								else
									print ("(RM_RETURN) Warning: missing currency data for listing " + a_rxrf.out + " " + a_zone + " on " + a_exdate.out + "%N")
								end
							else
								print ("(RM_RETURN) Warning: missing currency data for listing " + a_rxrf.out + " " + a_zone + " on " + a_exdate.out + "%N")
							end
						else
							print ("(RM_RETURN) Warning: missing currency data for listing " + a_rxrf.out + " " + a_zone + " on " + a_exdate.out + "%N")
						end
						l_total_value := l_total_value + l_value
						l_distribs.forth
					end
					Result.set_cash_rate (l_total_value)
				end
			end
		end

	retrieve_distributions(a_zone: STRING; a_rxrf: INTEGER;  a_exdate: INTEGER): ROSE_LINEAR_ARRAY[DIST_W_ROW] is
			-- retrieve distributions from zonal distribution tables
			-- note that U and C are for test purposes as U and C zones rely on backdiv arguments
		require
			valid_rxrf: a_rxrf > 0
			valid_distribution_zone: a_zone ~ "C" or a_zone ~ "E" or a_zone ~ "O" or a_zone ~ "U"
		local
			c: CHARACTER
			l_key: MULTI_KEY
		do
			create l_key.make(<<a_rxrf,a_exdate>>)
			c := a_zone.item(1)
			inspect c
			when 'E' then
				Result := distribution_e_table.item(l_key)
			when 'O' then
				Result := distribution_o_table.item(l_key)
			when 'U' then
				Result := distribution_u_table.item(l_key)
			when 'C' then
				Result := distribution_c_table.item (l_key)
			else

			end
		end

	currency_from_country_iso (a_iso: STRING; a_date: ROSE_DATE): CURRENCY is
			-- handles 2-char iso codes
		local
			l_country: COUNTRY
		do
			if a_iso.count=3 then
				Result := context.currency_by_iso_code (a_iso, a_date)
			elseif a_iso.is_equal ("EU") then
				Result := context.currency_by_iso_code ("EUR", a_date)
			elseif a_iso.count=2 then
				l_country := context.country_at (iso_code (a_iso), a_date)
				if l_country /= Void then
					Result := context.country_to_currency.related_entity_intervals (l_country).witness (a_date).entity
				else
					if a_iso.is_equal ("  ") then
						print ("(RM_RETURN) Can't find corresponding country to ISO code: '<blank>'%N")
					else
						print ("(RM_RETURN) Can't find corresponding country to ISO code: '" + a_iso.out + "'%N")
					end
				end
			end
		end

	initialize_distribution_table (a_zone: CHARACTER) is
			-- initiliazes distribution tables for each target zone
		require
			valid_distribution_zone: a_zone ~ 'C' or a_zone ~ 'E' or a_zone ~ 'O' or a_zone ~ 'U'
		local
			l_quotient: ROSE_LINEAR_ARRAY[ROSE_LINEAR_ARRAY[DIST_W_ROW]]
			l_retriever: COLLECTING_PREDICATE [DIST_W_ROW]
			l_table: HASH_TABLE [ROSE_LINEAR_ARRAY[DIST_W_ROW], MULTI_KEY]
			l_distrib: ROSE_LINEAR_ARRAY[DIST_W_ROW]
			l_key: MULTI_KEY
		do
			inspect a_zone
			when 'E' then
				l_retriever := distribution_e_retriever
				l_table := distribution_e_table
			when 'O' then
				l_retriever := distribution_o_retriever
				l_table := distribution_o_table
			when 'U' then
				l_retriever := distribution_u_retriever
				l_table := distribution_u_table
			when 'C' then
				l_retriever := distribution_c_retriever
				l_table := distribution_c_table
			else

			end
			l_retriever.retrieve(Void,<<>>)
			if not l_retriever.success then
				exceptions.raise ("initialize_distribution_table: "+a_zone.out+" "+"Retrieve failed")
			end
			l_retriever.data_server.commit

			if l_retriever.array /= void then
				l_quotient := l_retriever.array.quotient (agent distribution_comparison)
			end
			if l_quotient /= Void then
				from l_quotient.start
				until l_quotient.off
				loop
					l_distrib := l_quotient.current_item
					create l_key.make (<<l_distrib.first.rxrf, l_distrib.first.exdate>>)
					l_table.force (l_distrib, l_key)
					l_quotient.forth
				end
			end

		end

	distribution_comparison (a,b: DIST_W_ROW): BOOLEAN is
			-- are distributions equal?
		do
			Result := a.rxrf=b.rxrf and a.exdate=b.exdate
		end

	distribution_e_table: HASH_TABLE [ROSE_LINEAR_ARRAY[DIST_W_ROW], MULTI_KEY] is
			-- Distribution E table
		once
			create Result.make(8000)
			initialize_distribution_table('E')
		end

	distribution_e_retriever: COLLECTING_PREDICATE [DIST_W_ROW] is
			-- Distribution E retriever
		once
			create Result.make (distribution_e_server,
								"select * from dist_w where user_zone='E' and status_flag=' ' order by rxrf, exdate, I")
		end

	distribution_e_server: DATA_SERVER is
			-- distribution server E zone
		once
			Result := servers.legacy_distribution_server_e
		end

	distribution_o_table: HASH_TABLE [ROSE_LINEAR_ARRAY[DIST_W_ROW], MULTI_KEY] is
			-- Distribution O table
		once
			create Result.make(3000)
			initialize_distribution_table('O')
		end

	distribution_o_retriever: COLLECTING_PREDICATE [DIST_W_ROW] is
			-- Distribution O retriever
		once
			create Result.make (distribution_o_server,
								"select * from dist_w where user_zone='O' and status_flag=' ' order by rxrf, exdate, I")
		end

	distribution_c_table: HASH_TABLE [ROSE_LINEAR_ARRAY[DIST_W_ROW], MULTI_KEY] is
			-- Distribution C table
		once
			create Result.make(3000)
			initialize_distribution_table('C')
		end

	distribution_c_retriever: COLLECTING_PREDICATE [DIST_W_ROW] is
			-- Dsitribution C retriever
		once
			create Result.make (distribution_c_server,
								"select * from dist_w where user_zone='C' and status_flag=' ' order by rxrf, exdate, I")
		end

	distribution_c_server: DATA_SERVER is
			-- Distribution server C zone
			-- See comment on U zone below
		once
			Result := servers.legacy_distribution_server_c
		end

	distribution_u_table: HASH_TABLE [ROSE_LINEAR_ARRAY[DIST_W_ROW], MULTI_KEY] is
			-- Distribution U table
		once
			create Result.make(3000)
			initialize_distribution_table('U')
		end

	distribution_u_retriever: COLLECTING_PREDICATE [DIST_W_ROW] is
			-- Distribution U retriever
		once
			create Result.make (distribution_u_server,
								"select * from dist_w where user_zone='U' and status_flag=' ' order by rxrf, exdate, I")
		end

	distribution_u_server: DATA_SERVER is
			-- Distribution server U zone
			-- At this time, this server is only used to test C and U zones when making them consistent with other zones for return computations
			-- Production does not rely on distribution databases for C and U zones.
		once
			Result := servers.legacy_distribution_server_u
		end

	distribution_o_server: DATA_SERVER is
			-- Distribution server O zone
		once
			Result := servers.legacy_distribution_server_o
		end

	exceptions:EXCEPTIONS is
			-- Exceptions	
		once
			create Result
		end

end



