note

	description:

		"Compute returns using rose capital actions"

	author: "Srilatha Singh/JS"
	last_editor: "$Author: it $"
	date: "$Date: 2015/04/27 $"
	revision: "$Revision: #26 $"
	path: "$File: //rose/source/kernel/data/market/rm_return_new.e $"

class
	RM_RETURN_NEW

inherit
	ROSE_LEGACY_UNDEFINED
	CAPACT_OVERRIDE_AWARE

	SHARED_MONDAT_LISTING_PRICE_LOOKUP
		export {NONE} all end

	SHARED_MONDAT_FX
		export {NONE} all end

feature-- Operations

	period_return(a_listing:LISTING; a_start_date,a_end_date:ROSE_DATE) is
		-- compute return for listing during period defined by start and end date
	local
		l_end_date:ROSE_DATE
	do
		if a_end_date.ccyymmdd > constants.today_rdt.ccyymmdd then
			l_end_date := constants.today_rdt
		else
			l_end_date := a_end_date
		end
		set_return_from_capital_actions(a_listing,a_start_date,l_end_date,Void,Void)
	end

	set_return_from_capital_actions(a_listing:LISTING; a_start_date, a_end_date: ROSE_DATE; a_actions:ROSE_LINEAR_ARRAY[ROSE_COMPLEX_CAPITAL_ACTION]; a_currency: detachable CURRENCY) is
			-- compute return and yield for listing and period using rose capital actions
			-- if no actions, prices only are used
		require
			listing_not_void: a_listing /= Void
			start_date_not_void: a_start_date /= Void
			end_date_not_void: a_end_date /= Void
		local
			l_error: BOOLEAN
			l_start_price,l_end_price : DOUBLE
			l_cum_factor, l_exchange_rate: DOUBLE
			l_start_currency: CURRENCY
			l_actions : ROSE_LINEAR_ARRAY[ROSE_COMPLEX_CAPITAL_ACTION]
			l_option : ROSE_COMPLEX_CAPITAL_ACTION
			l_adj_factor,l_cash_outflow,l_cash_inflow, l_cash_dividend:DOUBLE
			l_vector:VECTOR
		do
			cum_cash_inflow:= 0.0
			cum_cash_outflow := 0.0
			rmdat_start_currency := Void
			rmdat_end_currency := Void
			cum_cash_dividend := 0.0
			has_terminal_distribution := False
			-- added for comparing new and old returns
			has_rights := False
			has_bonus := False
			has_multiple_capacts := False
			has_currency_conversion := False
			has_distribution := False
			-- end add			
			l_error := False
			start_date := a_start_date
			if a_end_date.ccyymmdd > constants.today_rdt.ccyymmdd then
				end_date := constants.today_rdt
			else
				end_date := a_end_date
			end

			create capact_data.make_empty

			-- Get start price/currency at the same time (loading the price also cached the currency).
			mondat_listing_price_lookup.lookup_price (a_listing, a_start_date)
			l_start_price := mondat_listing_price_lookup.last_price
			rmdat_start_currency := mondat_listing_price_lookup.last_currency
			l_start_currency := rmdat_start_currency

			-- Get end price/currency at the same time (loading the price also cached the currency).
			mondat_listing_price_lookup.lookup_price (a_listing, end_date)
			l_end_price := mondat_listing_price_lookup.last_price
			rmdat_end_currency := mondat_listing_price_lookup.last_currency
			currency := rmdat_end_currency

			if attached a_currency as la_requested_currency and attached currency as la_end_currency then
				mondat_fx.compute_rate (la_end_currency, la_requested_currency, end_date)
				l_end_price := l_end_price * mondat_fx.last_rate
				currency := la_requested_currency
			end

			if is_defined_double (l_start_price) and is_defined_double (l_end_price) then
				if l_start_currency /= Void and currency /= Void then
					if l_start_currency /~ currency then
						mondat_fx.compute_rate (l_start_currency, currency, a_start_date)
						l_exchange_rate := mondat_fx.last_rate
						has_currency_conversion := True
					else
						l_exchange_rate := 1.0
					end
				else
					l_exchange_rate := qnan
				end
				l_cum_factor := 1.0
				if a_actions /= Void  then
					l_actions := a_actions
				else
					l_actions := a_listing.capital_actions_between (start_date, end_date)
				end
				if is_defined_double(l_exchange_rate) then -- has prices and currencies
					l_start_price := l_start_price * l_exchange_rate
					if l_actions /= Void then
						from l_actions.start until l_actions.off or has_terminal_distribution loop
							-- march forward thru the capital actions from earliest to latest
							l_option := l_actions.current_item
							if l_option.is_valid_option and then l_option.option_signature.exdate <= end_date.ccyymmdd and then
							l_option.option_signature.exdate > a_start_date.ccyymmdd then
								l_cash_outflow := l_option.cash_outflow (currency)
								if is_defined_double(l_cash_outflow) then
									cum_cash_outflow := l_cash_outflow*l_cum_factor + cum_cash_outflow
								else
									l_error := True
								end
								l_cash_inflow := l_option.cash_inflow (currency)
								l_cash_dividend := l_option.cash_dividend (currency)
								if is_defined_double(l_cash_inflow) then
									cum_cash_inflow := l_cash_inflow*l_cum_factor + cum_cash_inflow
								else
									l_error:= True
								end
								if is_defined_double(l_cash_dividend) then
									cum_cash_dividend := l_cash_dividend*l_cum_factor + cum_cash_dividend
								else
									l_error:= True
								end
								l_adj_factor := l_option.rmdat_adjustment_factor
								create l_vector.make_from_array (<<l_adj_factor,l_cash_outflow,l_cash_inflow>>)
								capact_data.extend(l_vector,l_option.exdate)-- watch out exdate may be "18000101"
								l_cum_factor := l_cum_factor*l_adj_factor

								has_terminal_distribution := l_option.has_terminal_distribution
								has_split := l_option.has_split
								has_rights := l_option.has_rights
								has_bonus := l_option.has_bonus
								has_stock_dividend := l_option.has_stock_dividend
								has_multiple_capacts := l_option.has_multiple_capacts
								has_distribution := l_option.has_distribution
							end
							l_actions.forth
						end -- loop through actions
						if l_error then -- Has prices and currencies end and start, but error (no currencies) for capacts during period.
							yield := undefined_double
							return := undefined_double
						else
							yield := cum_cash_dividend/l_start_price
							if not has_terminal_distribution then
								return := (l_end_price*l_cum_factor + cum_cash_inflow)/(l_start_price + cum_cash_outflow)
							else
								return := (cum_cash_inflow)/(l_start_price + cum_cash_outflow)
							end
						end
					else -- has prices and currencies but no actions
						return := l_end_price/l_start_price
					end
				else -- Has prices but not currencies.
					if l_actions /= Void and l_actions.count > 0 then -- Has capacts during period.  l_actions may include splits.  Could be improved by assumptions on currency changes.
						return := undefined_double
					else -- Has no capacts during period.  Assumes same currency start and end.
						return := l_end_price/l_start_price
					end
				end
				cum_factor := l_cum_factor
			else -- start_price or end_price not available
				cum_factor := undefined_double
				return := undefined_double
				yield := undefined_double
			end
		end

feature--Computation of dividends/net cash_inflow in a holding period

	set_net_cash_flow_adjusted_to_start(a_listing:LISTING; a_start_date,a_end_date:ROSE_DATE; a_currency:CURRENCY) is
			-- set net cash flows adjusted to start for listing during period expressed in input currency
		local
			l_cum_factor: DOUBLE
			l_actions : ROSE_LINEAR_ARRAY[ROSE_COMPLEX_CAPITAL_ACTION]
			l_option : ROSE_COMPLEX_CAPITAL_ACTION
			l_adj_factor,l_cash_outflow,l_cash_inflow, l_cash_dividend:DOUBLE
			l_vector:VECTOR
			l_end_date:ROSE_DATE
		do
			if a_end_date.ccyymmdd > constants.today_rdt.ccyymmdd then
				l_end_date := constants.today_rdt
			else
				l_end_date := a_end_date
			end
			cum_cash_inflow:= 0
			cum_cash_outflow := 0
			cum_cash_dividend := 0
			rmdat_start_currency := Void
			rmdat_end_currency := Void
			has_terminal_distribution := False
			same_day_split_and_dividend := False
			create capact_data.make_empty
			if a_currency /= Void then
				rmdat_end_currency := a_currency
			else
				rmdat_end_currency := a_listing.rmdat_currency(l_end_date)
				if rmdat_end_currency = Void  then
					if a_listing.zone_mnemonic.as_upper = 'J' then
						rmdat_end_currency := a_listing.context.currency_by_iso_code ("JPY", constants.today_rdt)
					end
				end
			end
			if rmdat_end_currency /= Void then
				l_cum_factor := 1
				--l_actions := a_listing.capital_action_options_after (a_start_date)
				l_actions := a_listing.capital_actions_between (a_start_date, l_end_date)
				if l_actions /= Void then
					-- l_actions.sort_by(agent option_exdate_comparison)
					from l_actions.start until l_actions.off loop
						-- march forward thru the capital actions from earliest to latest
						l_option := l_actions.current_item
						if l_option.has_split_and_cash_dividend then
							same_day_split_and_dividend := True
						end
						if l_option.is_valid_option and then l_option.option_signature.exdate <= l_end_date.ccyymmdd then
							l_cash_outflow := l_option.cash_outflow(rmdat_end_currency)
							if is_defined_double(l_cash_outflow) then -- should we handle undefined value?
								cum_cash_outflow := l_cash_outflow*l_cum_factor + cum_cash_outflow
							end
							l_cash_inflow := l_option.cash_inflow(rmdat_end_currency)
							l_cash_dividend := l_option.cash_dividend(rmdat_end_currency)
							if is_defined_double(l_cash_inflow) then  -- should we handle undefined value?
								cum_cash_inflow := l_cash_inflow*l_cum_factor + cum_cash_inflow
							end
							if is_defined_double(l_cash_dividend) then  -- should we handle undefined value?
								cum_cash_dividend := l_cash_dividend*l_cum_factor + cum_cash_dividend
							end
							l_adj_factor := l_option.rmdat_adjustment_factor
							create l_vector.make_from_array (<<l_adj_factor,l_cash_outflow,l_cash_inflow>>)
							capact_data.extend(l_vector,l_option.exdate)-- watch out exdate may be "18000101"
							l_cum_factor := l_cum_factor*l_adj_factor
							if l_option.has_terminal_distribution then
								has_terminal_distribution := l_option.has_terminal_distribution
							end
						end
						l_actions.forth
					end-- from loop	
					net_cash_flow := cum_cash_inflow - cum_cash_outflow
				else
					net_cash_flow := 0.0
				end--if no capital_actions
				cum_factor := l_cum_factor
			else
				cum_factor := constants.missing_double
				net_cash_flow := constants.missing_double
			end-- no rmdat_end_currency
		end

	net_cash_flow: DOUBLE


feature--Using Overrides in cache

	implied_period_return (a_listing:LISTING; a_start_date,a_end_date:ROSE_DATE) is
	-- This returns the holding period return taking into account any overides made during the
	-- application, Currently only capact_overrides
	local
		l_capital_actions:ROSE_LINEAR_ARRAY[CAPACT_SIGNATURE_RECORD]
		l_options:ROSE_LINEAR_ARRAY[CAPACT_OPTION_SIGNATURE_RECORD]
		l_options_table:HASH_TABLE[CAPACT_OPTION_SIGNATURE_RECORD, INTEGER]
		l_quotient_array: ROSE_LINEAR_ARRAY[ROSE_LINEAR_ARRAY[CAPACT_SIGNATURE_RECORD]]
		l_virtual_record_history:ROSE_VIRTUAL_RECORD_HISTORY[CAPACT_SIGNATURE_RECORD]
		l_option_target:CAPACT_OPTION_SIGNATURE_RECORD
		l_found_capacts:ROSE_MULTI_KEYED_TABLE[BOOLEAN]
		l_exdate:INTEGER
		l_option:ROSE_COMPLEX_CAPITAL_ACTION
		l_actions : ROSE_LINEAR_ARRAY[ROSE_COMPLEX_CAPITAL_ACTION]
	do
		create l_found_capacts.make(5)
		create l_options_table.make(10)
		if has_override_for_listing(a_listing) then
--			create l_target
--			l_target.set_instrument_id (a_listing.security.handle)
--			l_target.set_market_id (a_listing.exchange.handle)
--			l_option_virtual_records := a_listing.ur_capact_option_signature.data_manager.retrieve_signature(l_target,l_target.instrument_market_restriction_statement, Void)

--			l_capital_actions := a_listing.ur_capact_signature.signatures_by_instrument_market_after (a_listing.security.handle, a_listing.exchange.handle,a_start_date.ccyymmdd)
			l_capital_actions := a_listing.ur_capact_signature.signatures_by_listing_id_after (a_listing.handle,a_start_date.ccyymmdd)
			--- modify the capital_action signatures of the ones that are present in the overide aware
			-- check to see if the exdate is still after the start_date
			if l_capital_actions /= Void then
				from l_capital_actions.start
				until l_capital_actions.off
				loop
					rose_best_capital_actions.search(l_capital_actions.current_item.key_tuple)
					if rose_best_capital_actions.found then
						l_found_capacts.put(True,l_capital_actions.current_item.key_tuple)
						create l_virtual_record_history.make_one(rose_best_capital_actions.found_item)
						a_listing.ur_capact_signature.ur_expression.expression_cache_force(l_virtual_record_history,rose_best_capital_actions.found_item.signature)
						if rose_best_capital_actions.found_item.signature.exdate > a_start_date.ccyymmdd then
							l_capital_actions.replace (rose_best_capital_actions.found_item.signature)
						else
							l_capital_actions.replace(Void)-- Careful when sorting
						end
					end
					l_capital_actions.forth
				end
			end
			--- now add to the signatures that were not found and are hence new, again check exdate
			if l_found_capacts.count < rose_best_capital_actions.count then
				from rose_best_capital_actions.start
				until rose_best_capital_actions.off
				loop
					l_found_capacts.search(rose_best_capital_actions.key_for_iteration)
					if not l_found_capacts.found then
						l_found_capacts.put(True,rose_best_capital_actions.key_for_iteration)
						create l_virtual_record_history.make_one(rose_best_capital_actions.item_for_iteration)
						a_listing.ur_capact_signature.ur_expression.expression_cache_force(l_virtual_record_history,rose_best_capital_actions.item_for_iteration.signature)
						if rose_best_capital_actions.item_for_iteration.signature.exdate > a_start_date.ccyymmdd then
							if l_capital_actions = Void then
								create l_capital_actions.make_empty
							end
							l_capital_actions.extend(rose_best_capital_actions.item_for_iteration.signature)
						end
					end
					rose_best_capital_actions.forth
				end
			end

--			l_options := a_listing.ur_capact_option_signature.signatures_by_instrument_market_after (a_listing.security.handle, a_listing.exchange.handle, a_start_date.ccyymmdd)
			l_options := a_listing.ur_capact_option_signature.signatures_by_listing_after (a_listing.handle, a_start_date.ccyymmdd)
			---option from database
			if l_options /= Void then
				from l_options.start
				until l_options.off
				loop
					rose_best_options.search(l_options.current_item.key_tuple)
					if rose_best_options.found then
						l_options.replace(rose_best_options.found_item)
						l_options_table.put(rose_best_options.found_item,rose_best_options.found_item.exdate)
					else
						l_options_table.put(l_options.current_item,l_options.current_item.exdate)
					end
					l_options.forth
				end
			end
			from rose_best_options.start
			until rose_best_options.off
			loop
				l_options_table.search(rose_best_options.item_for_iteration.exdate)
				if not l_options_table.found then
					l_options_table.force(rose_best_options.item_for_iteration,	rose_best_options.item_for_iteration.exdate)
				end
				rose_best_options.forth
			end
			create l_actions.make_empty
			if l_capital_actions /= Void then
				l_capital_actions.sort_by(agent exdate_comparison)
				l_quotient_array :=  l_capital_actions.quotient(agent quotient_comparison)

				from l_quotient_array.start
				until l_quotient_array.after
				loop
					if l_quotient_array.current_item.first /= Void then
						l_exdate:=l_quotient_array.current_item.first.exdate
						l_option_target := l_options_table.item(l_exdate)
						if l_option_target /= Void then
							create l_option.make (l_option_target,l_quotient_array.current_item)
								if l_option.is_valid_option then
									l_actions.extend(l_option)
								end
						else
							-- This should typically not happen
							--- look in the override rose_best to see if a new option has been added	--- All the overrides have been added above
						end
					end
					l_quotient_array.forth
				end
			end
			set_return_from_capital_actions(a_listing, a_start_date,a_end_date, l_actions, Void)
		else
			period_return (a_listing,a_start_date,a_end_date)
		end
	end

feature--access

	cum_cash_outflow,cum_cash_inflow,cum_cash_dividend:DOUBLE
	cum_factor:DOUBLE
	return:DOUBLE
	yield:DOUBLE
	capact_data:ROSE_TIME_SERIES[VECTOR] -- each vector is a triple of adj_factor,cash_outflow,cash_inflow
	rmdat_start_currency, rmdat_end_currency: detachable CURRENCY -- currencies of rmdat start and end prices

	currency: detachable CURRENCY
			-- Currency of `yield' and `return'

	start_date: detachable ROSE_DATE
			-- Start date from last call to `set_return_from_capital_actions'

	end_date: detachable ROSE_DATE
			-- End date from last call to `set_return_from_capital_actions'

feature -- Status

	has_terminal_distribution: BOOLEAN -- terminal distribution during return period
	same_day_split_and_dividend :BOOLEAN -- split and dividend on same day during return period

	has_split: BOOLEAN -- has split during return period
	has_rights: BOOLEAN -- has rights during return period
	has_bonus: BOOLEAN -- has bonus during return period
	has_stock_dividend: BOOLEAN
	has_multiple_capacts: BOOLEAN
	has_currency_conversion: BOOLEAN
	has_distribution: BOOLEAN


feature{NONE}--Implementation

	has_override_for_listing(a_listing:LISTING):BOOLEAN is
		-- capital action has override
	local
		l_listing_id: INTEGER
		l_lid_ref: INTEGER_REF
	do
		l_listing_id := a_listing.handle
		from override_capital_actions_table.start
		until Result or override_capital_actions_table.off
		loop
			l_lid_ref ?= override_capital_actions_table.key_for_iteration.item(1)
			if l_lid_ref /= Void  and then l_lid_ref.item = l_listing_id  then
				Result := True
			end
			override_capital_actions_table.forth
		end
	end

	exdate_comparison (a,b: CAPACT_SIGNATURE_RECORD): BOOLEAN is
			-- sort by exdate
		do
			if a/= Void and b/=Void then
				Result := a.exdate > b.exdate
			end
		end

	option_exdate_comparison (a,b: ROSE_COMPLEX_CAPITAL_ACTION): BOOLEAN is
			-- sort option by exdate
		do
			if a/= Void and b/=Void then
				Result := a.exdate < b.exdate
			end
		end
	quotient_comparison (a,b: CAPACT_SIGNATURE_RECORD): BOOLEAN is
			-- gather by exdate
		do
			if a /= Void and b/= Void then
				Result := a.exdate = b.exdate
			else
				Result := False
			end
		end

end
