note

	description:

		"Descendants of this class get their market data from RMDAT"

	author: "Randy John"
	last_editor: "$Author: it $"
	date: "$Date: 2015/02/23 $"
	revision: "$Revision: #21 $"
	path: "$File: //rose/source/kernel/investment_entities/rmdat_instrument.e $"

deferred class
	RMDAT_INSTRUMENT

feature -- Access

	zone: ZONE [LOCATION] is
			-- Zone for the instrument
		deferred
		ensure
			attached_result: Result /= Void
		end

	zone_mnemonic: CHARACTER is
			-- Zone mnemonic for the instrument
		deferred
		end

	xrf_code: INTEGER is
			-- Instrument xrf code
		deferred
		end

	price_record (a_date: ROSE_DATE): RMDAT_PRICE_RECORD is
			-- RMDAT price record for 'Current' on 'a_date'
		require
			valid_date: a_date /= Void
		local
			l_composite_record: COMPOSITE_RECORD
		do
			if price_record_saved = Void or else
				price_record_saved.date = Void or else
				not price_record_saved.date.same_date (a_date) then
				if price_record_saved = Void then
					create price_record_saved
				end
				if is_composite then
					create price_record_saved
					l_composite_record := composite_record (a_date)
					if l_composite_record /= Void then
						price_record_saved.set_composite (Current, l_composite_record, a_date)
					end
				else
					price_record_saved := market_data.price_record (xrf_code, a_date, a_date)
					price_record_saved.set_currency (market_data.currency (xrf_code, a_date))
					last_price_record_saved := Void
				end
			end
			Result := price_record_saved
		end

	composite_record (a_date: ROSE_DATE): COMPOSITE_RECORD is
		deferred
		end

	last_price_record (a_date: ROSE_DATE): RMDAT_PRICE_RECORD is
		do
			Result := last_price_record_with_lookback (a_date, 170)
		end

	last_price_record_160 (a_date: ROSE_DATE): RMDAT_PRICE_RECORD is
		do
			Result := last_price_record_with_lookback (a_date, 160)
		end

	last_price_record_saved: like last_price_record
	price_record_saved: like price_record

feature -- Element modification

	clear_price_records is
			-- Eliminate cached price records, usually used when
			-- planning on storing LISTINGs
		do
			price_record_saved := Void
			last_price_record_saved := Void
		end

feature -- Status

	is_composite: BOOLEAN is
			-- Is this instrument a composite?
		deferred
		end

feature {NONE}  -- Implementation

	last_lookback_mode: INTEGER

	last_price_record_with_lookback (a_date: ROSE_DATE; a_lookback_mode: INTEGER): RMDAT_PRICE_RECORD is
		require
			valid_date: a_date /= Void
			valid_lookback_mode: a_lookback_mode = 160 or a_lookback_mode = 170
		local
			l_last_price_record: like last_price_record_saved
			l_date: ROSE_DATE
			l_saved_lookback_mode: INTEGER
		do
			l_date := last_market_day (a_date)
			if last_price_record_saved /= Void and then last_price_record_saved.date /= Void then
				-- Attempt to reuse the previous result
				if a_date >= last_price_record_saved.date and then l_date <= last_price_record_saved.as_of_date and then last_lookback_mode = a_lookback_mode then
					-- Reuse last record
					Result := last_price_record_saved
				else
					-- 'a_date' is out of bounds or different lookback mode is requested, requery MARKET_DATA
					if is_composite then
						retrieve_composite_record (a_date, a_lookback_mode)
					else
						l_saved_lookback_mode := market_data.lookback_mode
						market_data.set_lookback_mode (a_lookback_mode)
						if a_date >= last_price_record_saved.date and then l_date > last_price_record_saved.as_of_date and then	last_lookback_mode = a_lookback_mode then
							-- Requery using 'last_price_record_saved.as_of_date' as the lookback boundary since we already know that to be
							-- at least the latest as of 'l_date'
							l_last_price_record := market_data.price_record (xrf_code, l_date, last_price_record_saved.as_of_date)
							l_last_price_record.set_currency (market_data.currency (xrf_code, a_date))

							if l_last_price_record.is_valid then
								last_price_record_saved := l_last_price_record
								last_lookback_mode := a_lookback_mode
							else
								last_price_record_saved.set_as_of_date (a_date)
							end
						else
							-- 'l_date' is either before previous result or lookback mode has changed
							last_price_record_saved := market_data.price_record (xrf_code, l_date, Void)
							last_price_record_saved.set_currency (market_data.currency (xrf_code, a_date))
							last_lookback_mode := a_lookback_mode
						end
						market_data.set_lookback_mode (l_saved_lookback_mode)
					end
				end
			else
				if is_composite then
					retrieve_composite_record (a_date, a_lookback_mode)
				else
					l_saved_lookback_mode := market_data.lookback_mode
					market_data.set_lookback_mode (a_lookback_mode)
					last_price_record_saved := market_data.price_record (xrf_code, a_date, Void)
					last_price_record_saved.set_currency (market_data.currency (xrf_code, a_date))
					last_lookback_mode := a_lookback_mode
					market_data.set_lookback_mode (l_saved_lookback_mode)
				end
			end
			Result := last_price_record_saved
		end

	retrieve_composite_record (a_date: ROSE_DATE; a_lookback_mode: INTEGER) is
		require
			valid_date: a_date /= Void
		local
			l_composite_record: COMPOSITE_RECORD
		do
			l_composite_record := composite_record (a_date)
			create last_price_record_saved
			last_lookback_mode := a_lookback_mode
			if l_composite_record /= Void then
				last_price_record_saved.set_composite (Current, l_composite_record, a_date)
			end
		end

	market_data: MARKET_DATA is
		do
			if not market_data_cell.item.zone.mnemonic.is_equal (zone_mnemonic) then
				market_data_cell.put (zone.market_data)
			end
			Result := market_data_cell.item
		ensure
			attached_result: Result /= Void
		end

	last_market_day (a_date: ROSE_DATE): ROSE_DATE is
			-- Most recent market day on or before `a_date'
		require
			valid_date: a_date /= Void
		do
			Result := market_data.zone.last_market_day (a_date)
		ensure
			attached_result: Result /= Void
			meaningful_result: Result <= a_date
		end

	market_data_cell: CELL[MARKET_DATA] is
			-- Cache for market data associated with zone
		once
			create Result.put (zone.market_data)
		ensure
			attached_result: Result /= Void
		end

end
