note

	description:

		"Rmdat price record"

	author: "Mark Howard"
	last_editor: "$Author: er $"
	date: "$Date: 2014/02/26 $"
	revision: "$Revision: #8 $"
	path: "$File: //rose/source/kernel/data/market/rmdat_price_record.e $"
	storable_version: ""

class
	RMDAT_PRICE_RECORD
inherit
	PRICE_RECORD
		undefine
			default_create
		end
	IEEE_MATH
		undefine
			is_equal,
			default_create
		end
feature --Default create
	default_create
			--Set adjustment to 1 by default
		do
			adjustment := 1
		end
feature -- Access

	listing : RMDAT_INSTRUMENT
		-- Listing

feature -- Basic operations


	set_composite(a_listing : RMDAT_INSTRUMENT;a_record : COMPOSITE_RECORD;a_date : ROSE_DATE) is
			-- Set composite.
		local
			l_shares : DOUBLE
			l_published_nav, l_capital_balance, l_revenue_balance, l_expense_balance: DOUBLE
		do
			listing := a_listing
			create date.make_ccyymmdd(a_record.trade_date)
			as_of_date := a_date
			if a_record.currency /= void and then a_record.currency.count > 0 then
				currency := context.currency_by_iso_code(a_record.currency,a_date)
			end
			adjustment := 1

			--  Check and correct for missing values

			if isnan(a_record.published_nav) then
				l_published_nav := 0
			else
				l_published_nav := a_record.published_nav
			end

			if isnan(a_record.capital_balance) then
				l_capital_balance := 0
			else
				l_capital_balance := a_record.capital_balance
			end

			if isnan(a_record.revenue_balance) then
				l_revenue_balance := 0
			else
				l_revenue_balance := a_record.revenue_balance
			end

			if isnan(a_record.expense_balance) then
				l_expense_balance := 0
			else
				l_expense_balance := a_record.expense_balance
			end

			if l_published_nav > 0 then
				has_close := True
				unadjusted_close := l_published_nav
			else
				l_shares := a_record.outstanding_shares
				if l_shares /= 0 then
					has_close := True
					unadjusted_close := (l_capital_balance +
										l_revenue_balance -
										l_expense_balance) / l_shares
				end
			end
		end
	set(a_listing : LISTING; a_market_data : MARKET_DATA;
		a_record : RMDATA; a_found_date, a_date : ROSE_DATE;
		a_currency : CURRENCY; a_adjustment : DOUBLE) is
			-- Set the record
		do
			listing := a_listing
			date := a_found_date
			as_of_date := a_date
			currency := a_currency
			adjustment := a_adjustment
			if a_record /= void then
				has_open := a_record.has_open
				if a_record.has_open then
					unadjusted_open := a_record.open
				end
				has_close := a_record.has_close
				if a_record.has_close then
					unadjusted_close := a_record.close
--				elseif a_record.has_bid then
--					has_close := True
--					unadjusted_close := a_record.bid
				end
				has_high := a_record.has_high
				if a_record.has_high then
					unadjusted_high := a_record.high
				end
				has_low := a_record.has_low
				if a_record.has_low then
					unadjusted_low := a_record.low
				end
				has_bid := a_record.has_bid
				if a_record.has_bid then
					unadjusted_bid := a_record.bid
				end
				has_ask := a_record.has_ask
				if a_record.has_ask then
					unadjusted_ask := a_record.ask
				end
				has_volume := a_record.has_vol
				if a_record.has_vol then
					unadjusted_volume := a_record.vol
				end
				has_settlement := a_record.has_settlement
				if a_record.has_settlement then
					unadjusted_settlement := a_record.settlement
				end
				if has_close or has_bid or has_ask then
					has_close_bid_ask := True
				end
			end
		end
feature --Unitized
	unitized : UNITIZED_PRICE_RECORD
			--Unitized record
		do
			if unitized_saved = Void then
				create unitized_saved.make(Current)
			end
			Result := unitized_saved
		end

	unitized_saved : like unitized
			--Cached unitized record
		note option: transient
		attribute
		end


end
