indexing
	description: "RMDAT_PRICE_REPORT"
	author: "Michael Zhang"
	date: "$Date: 2014/02/26 $"
	revision: "$Revision: #28 $"
	path: "$File: //rose/source/kernel/data/reports/rmdat_price_report.e $"

class
	RMDAT_PRICE_REPORT

inherit
	ROSE_SIMPLE_REPORT[ROSE_LISTING_DATE]
		rename
			make as simple_report_make
		redefine
			double_item,
			entity_anchor,
			set_entity_neighborhood,
			targets,
			matrix
		end

	ROSE_LEGACY_UNDEFINED

creation
	make, make_unadjusted

feature -- create

	make is
	-- make
		do
			create ur_target
			simple_report_make(ur_target.expressions,"price_data_set","price_data_set")

			adjustment_factor_expression ?= ur_target.expressions.variable_from_mnemonic("ADJF")
		end

	make_unadjusted is
	-- make
		do
			create ur_target
			simple_report_make(ur_target.expressions,"price_data_set","price_data_set")

			adjustment_factor_expression ?= ur_target.expressions.variable_from_mnemonic("ADJF")
			replace_adjusted_expressions_with_unadjusted
		end


	adjustment_factor_expression : ROSE_MARKET_VARIABLE

feature -- access
	replace_adjusted_expressions_with_unadjusted is
			-- replace
		local
			l_unadjust_closing_expression : ROSE_MARKET_VARIABLE
		do
			from expressions.keys.start
			until expressions.keys.off
			loop
				if expressions.keys.current_item.is_equal("close") then
					l_unadjust_closing_expression ?= ur_target.expressions.variable_from_name("unadjusted_closing_price")
					expressions.replace_at(l_unadjust_closing_expression,"close")
				elseif  expressions.keys.current_item.is_equal("volume") then
					replace_adjusted_expression("volume", "volume", "unadjusted_volume")
				elseif  expressions.keys.current_item.is_equal("high") then
					replace_adjusted_expression("high_price", "high", "unadjusted_high")
				elseif  expressions.keys.current_item.is_equal("low") then
					replace_adjusted_expression("low_price", "low", "unadjusted_low")
				elseif  expressions.keys.current_item.is_equal("bid") then
					replace_adjusted_expression("bid_price", "bid", "unadjusted_bid")
				elseif  expressions.keys.current_item.is_equal("ask") then
					replace_adjusted_expression("ask_price", "ask", "unadjusted_ask")
				elseif  expressions.keys.current_item.is_equal("open") then
					replace_adjusted_expression("opening_price", "open", "unadjusted_open")
				end
				expressions.keys.forth
			end
		end

	replace_adjusted_expression(a_name, a_mnemonic, a_unadjusted_name : STRING) is
			-- replace adjusted expression
		local
			l_unadjust_expression : ROSE_MARKET_VARIABLE
			l_var_desc_record: VARIABLE_DESCRIPTION_RECORD
		do
			l_var_desc_record ?= ur_target.expressions.variable_descriptions_by_name.item (a_name)
			l_var_desc_record.set_name (a_unadjusted_name)
			create l_unadjust_expression.make(l_var_desc_record)
			expressions.replace_at(l_unadjust_expression, a_mnemonic)
		end

	targets(a_entity : like entity_anchor) : ROSE_LINEAR_ARRAY[ROSE_LISTING_DATE] is
	-- get all targets for a given entity
		local
			l_entity : like entity_anchor
		do
			l_entity ?= a_entity
			if l_entity /= Void then
				Result := ur_target.entity_targets(l_entity)
			end
		end

	set_entity_neighborhood(a_entities : ARRAY[ENTITY]) is
	-- to set entity neighborhood
		local
			i : INTEGER
			l_entity : like entity_anchor
			l_entities : ROSE_LINEAR_ARRAY[like entity_anchor]
		do
			if a_entities /= Void and then a_entities.count > 0 then
				create l_entities.make_empty
				from i := 1 until i > a_entities.count loop
					l_entity ?= a_entities.item(i)
					if l_entity /= Void then
						l_entities.extend(l_entity)
					end
					i := i + 1
				end
				ur_target.load_target_cache(l_entities)
			end
		end

feature -- value
	double_item(a_expression : like expression_template; a_target : like ur_target) : DOUBLE is
		-- get value for a target and expression
		local
			l_listing : LISTING
			l_adj : DOUBLE
		do
			l_listing := a_target.entity
--			if l_listing /= Void then
--				--l_listing.zone.market_data.disable_background  --from vms
--				l_listing.zone.market_data.enable_background  --from windows
--			end

			if (a_expression /= Void and then a_expression.variable_description.mnemonic.is_equal("open")) and then (l_listing /= Void and then l_listing.zone_mnemonic.is_equal('J')) then
				Result := constants.missing_double
			else
				Result := Precursor(a_expression, a_target)
				if not valid_double(Result) then
					Result := constants.missing_double
				else
					l_adj := adjustment_factor_expression.generated_unitized_value(a_target, void).value
--					if  valid_double(Result) and then l_adj > 0 then
--						if a_expression.name.is_equal("volume") then
--							Result := Result / l_adj
--						else
--							Result := Result * l_adj
--						end
--					end
				end
			end
		end

	adjustment_series(a_targets : ROSE_LINEAR_ARRAY[ROSE_EXPRESSION_TARGET]) : ROSE_LINEAR_ARRAY[DOUBLE] is
		-- get adjustment series
		local
			l_double : DOUBLE
			l_target : like ur_target
			l_count : INTEGER
		do
			create Result.make_empty
			l_count := 1

			from a_targets.start
			until a_targets.off
			loop
				l_target ?= a_targets.current_item
				if l_target /= Void then
					l_double := adjustment_factor_expression.generated_unitized_value(l_target, void).value
					if  not constants.defined_double(l_double) then
						l_count := l_count + 1
					end
					Result.extend(l_double)
				else
					l_count := l_count + 1
					Result.extend(constants.missing_double)
				end
				a_targets.forth
			end
			if l_count = a_targets.count then
				Result := Void
			end
		end

	valid_double(a_double : DOUBLE) : BOOLEAN is
		-- check if it is double
		do
			if not is_defined_double(a_double)
				or (is_defined_double(a_double) and then a_double = 0) then
				Result := False
			else
				Result := True
			end
		end

	matrix(a_listing : like entity_anchor) : DATED_MATRIX is
		-- get matrix for given entity
		local
			i : INTEGER
			l_target : like ur_target
			l_array : like double_array
			l_dates : ROSE_LINEAR_ARRAY[ROSE_DATE]
			l_currency : PERSISTENT_CURRENCY
			l_currencies : like currencies
			l_targets : like targets
		do
			if last_listing = Void or else last_listing.handle /= a_listing.handle then
				l_targets := targets(a_listing)
				if l_targets /= Void and then l_targets.count > 0 then
					create Result.make(l_targets.count, count)
					create l_currencies.make_empty
					create l_dates.make_empty
					Result.set_column_names(expressions.keys)
					from i := 1 l_targets.start until l_targets.off loop
						l_target := l_targets.current_item
						l_array := double_array(l_target)
						if l_array /= Void then
							Result.put_row_vector(l_array, i)
						else
							Result.set_row_null(i)
						end
						l_dates.extend(l_target.date)
						--l_currency := a_listing.rmdat_currency (l_target.date)
						l_currency := a_listing.price_record(l_target.date).currency
						l_currencies.extend(l_currency, l_target.date)
						i := i + 1
						l_targets.forth
					end
					Result.set_dates (l_dates)
					Result.set_currency_series (l_currencies)
				end
				matrix_saved := Result
				last_listing := a_listing
			else
				Result := matrix_saved
			end
		end

	matrix_interval(a_listing : like entity_anchor; a_start_date, a_end_date: ROSE_DATE) : DATED_MATRIX is
		-- get matrix interval given  entity and start and end date
		local
			i, l_start_index, l_end_index: INTEGER
			l_target : like ur_target
			l_array : like double_array
			l_dates : ROSE_LINEAR_ARRAY[ROSE_DATE]
			l_currency : PERSISTENT_CURRENCY
			l_currencies : like currencies
			l_targets, l_targets_interval : like targets
			l_target_start, l_target_end: ROSE_LISTING_DATE
			l_first_date, l_last_date: ROSE_DATE
			l_no_targets: BOOLEAN
		do
			if last_listing = Void or else last_listing.handle /= a_listing.handle then
				l_targets := targets(a_listing)
				if l_targets /= Void and then l_targets.first /= Void then


					l_first_date := l_targets.first.date
					l_last_date := l_targets.last.date
					create l_target_start.make (a_listing, a_start_date)
					create l_target_end.make (a_listing, a_end_date)
					if l_targets /= Void and then l_targets.count > 0 then
						l_start_index := l_targets.new_index_of(l_target_start)
						l_end_index := l_targets.new_index_of(l_target_end)
						if l_start_index = 0 then
							if a_start_date.ccyymmdd > l_last_date.ccyymmdd or a_end_date.ccyymmdd < l_first_date.ccyymmdd then
								l_no_targets := True
							else
								l_start_index := l_targets.lower
							end
						end
						if l_end_index = 0 and not l_no_targets then
							l_end_index := l_targets.upper
						end
						if not l_no_targets then
							l_targets_interval := l_targets.subarray (l_start_index, l_end_index)
							create Result.make(l_targets_interval.count, count)
							create l_currencies.make_empty
							create l_dates.make_empty
							Result.set_column_names(expressions.keys)
							from i := 1 l_targets_interval.start until l_targets_interval.off loop
								l_target := l_targets_interval.current_item
								if l_target.date >= a_start_date and l_target.date <= a_end_date then
									l_array := double_array(l_target)
									if l_array /= Void then
										Result.put_row_vector(l_array, i)
									else
										Result.set_row_null(i)
									end
									l_dates.extend(l_target.date)
									--l_currency := a_listing.rmdat_currency (l_target.date)
									l_currency := a_listing.price_record(l_target.date).currency
									l_currencies.extend(l_currency, l_target.date)
								end
								i := i + 1
								l_targets_interval.forth
							end
							Result.set_dates (l_dates)
							Result.set_currency_series (l_currencies)
						end
						matrix_saved := Result
						last_listing := a_listing
					end
				end--if l_targets /= Void and then l_targets.first /= Void then	
			else
				Result := matrix_saved
			end
		end

	new_matrix(a_listing : like entity_anchor) : DATED_MATRIX is
		-- get new matrix for a listing
		local
			l_date : ROSE_DATE
			l_date_time : ROSE_DATE
			l_dates : ROSE_LINEAR_ARRAY[ROSE_DATE]
			l_vector_array : ROSE_LINEAR_ARRAY[VECTOR]
			l_sig : like ur_rmdat_target
			l_sigs : ROSE_LINEAR_ARRAY[like ur_rmdat_target]
			l_currency : PERSISTENT_CURRENCY
			l_currencies : ROSE_TIME_SERIES[PERSISTENT_CURRENCY]
		do
			l_sigs := ur_rmdat_target.listing_signatures(a_listing)
			if l_sigs /= Void and then l_sigs.count > 0 then
				create l_dates.make_empty
				create l_vector_array.make_empty
				create l_currencies.make_empty
				from l_sigs.start until l_sigs.off loop
					l_sig := l_sigs.current_item
					if l_sig.any_data_at_all then
						l_date_time ?= l_sig.date
						if l_date_time /= Void then
							create l_date.make_ccyymmdd(l_date_time.ccyymmdd)
							l_dates.extend(l_date)
						else
							l_dates.extend(l_sig.date)
						end
						l_vector_array.extend(l_sig.to_vector)
						if l_currency = Void or else l_currency.handle /= l_sig.currency then
							l_currency := a_listing.context.currency_accessor.entity (l_sig.currency)
						end
						l_currencies.extend (l_currency, l_date)
					end
					l_sigs.forth
				end
				if l_dates.count > 0 then
					create Result.make_from_vector_array (l_vector_array, l_dates)
					Result.set_column_names (names)
					Result.set_currency_series (l_currencies)
				end
			end
		end

	currencies(a_listing : LISTING; a_dates : ARRAY[ROSE_DATE]) : ROSE_TIME_SERIES[PERSISTENT_CURRENCY] is
		-- get currencies for a listing and a collection of dates
		local
			i : INTEGER
			l_date : ROSE_DATE
			l_currency : PERSISTENT_CURRENCY
		do
			create Result.make_empty
			from i := a_dates.lower until i > a_dates.upper loop
				l_date := a_dates.item(i)
				l_currency := a_listing.rmdat_currency (l_date)
				Result.extend(l_currency, l_date)
				i := i + 1
			end
		end

	names : ROSE_LINEAR_ARRAY[STRING] is
			-- Names
		once create Result.make_from_array(<<"ask", "bid", "close", "high", "low", "open", "volume">>) end

feature {NONE} -- non public attribute
	last_listing : like entity_anchor
	matrix_saved : like matrix

	entity_anchor : LISTING is
			-- entity anchor
		do end

	ur_rmdat_target : RMDAT_SIGNATURE_RECORD is
			-- ur rmdat target
		once create Result end

feature -- test
	test(a_id : INTEGER) is
			-- test
		local
			l_entity : like entity_anchor
			l_targets : like targets
			l_array : like double_series
			l_currencies : like currency_series
			l_matrix : like double_matrix
		do
			l_entity ?= ur_target.context.listing_accessor.entity(a_id)
			if l_entity /= Void then
				l_targets := ur_target.entity_targets(l_entity)
				l_matrix := double_matrix(l_targets)
				l_array := double_series(l_targets, "close")
				l_currencies := currency_series(l_targets, "close")
			end
		end
end
