note

	description:

		"Record for holding data from the rmdat signature tagged database"

	author: "Mauricio Bustos"
	last_editor: "$Author: mk $"
	date: "$Date: 2011/05/03 $"
	revision: "$Revision: #47 $"
	path: "$File: //rose/source/kernel/data/market/rmdat_signature_record.e $"

class
	RMDAT_SIGNATURE_RECORD

inherit
	CLUSTERED_SIGNATURE[INTEGER,ROSE_DATE,INVESTMENT_ENTITY]
		redefine
			date,
			data_manager,
			description,
			description_title,
			new_variable,
			set_date,
			data_fields_string_,
			target_name,
			multi_key,
			set_entity,
			expressions,
			reconcile_target,
			output_string_by_name,
			output_data_field_string_by_name,
			date_name,
			repaired_statement
		end

	ROSE_AWARE
		rename
			context as rose_context
		end

	ROSE_LEGACY_UNDEFINED
		undefine
			is_equal
		end

	ZONAL_SIGNATURE
		undefine
			is_equal
		end

	IEEE_MATH
		undefine
			is_equal
		end

create
	make_ur,
	default_create

create {RMDAT_SIGNATURE_RECORD}
    rose_entity_date_make

feature -- Access

	entity_id: INTEGER

	rxrf: INTEGER

	date: ROSE_DATE

	open: DOUBLE

	close: DOUBLE

	high: DOUBLE

	low: DOUBLE

	bid: DOUBLE

	ask: DOUBLE

	mid_close: DOUBLE

	volume: DOUBLE

	currency: INTEGER

	fields_: STRING is "entity_id,rxrf,date"

	data_fields_string_: STRING is "open,close,high,low,bid,ask,volume,currency"

	expression_tag: STRING is "listing"

	zone_mnemonic: CHARACTER

	expressions: like expressions_anchor is
			-- Expressions
		do
			Result ?= expressions_table.item (internal_type)
			if Result = Void then
				create Result.make_from_descriptions (context.variable_descriptions (expression_tag),agent new_variable,context.mappings_from_descriptions (context.variable_descriptions (expression_tag)))
				expressions_table.put (Result,internal_type)
			end
		end

	key_0: INTEGER is
			--key 0
		do
			Result := rxrf
		end

	key_0_name: STRING is
		do
			Result := "rxrf"
		end

	key_1: ROSE_DATE is
		do
			Result := date
		end

	key_1_name: STRING is
		do
			Result := "date"
		end

	multi_key: MULTI_KEY is
		local
			l_PAIR: KEY_PAIR[INTEGER, ROSE_DATE]
		do
			if saved_multi_key = Void then
				create l_pair.make (rxrf, date)
				create saved_multi_key.make(<<l_pair>>)
			end
			Result := saved_multi_key
		end

	description_title: ARRAY[STRING] is
	-- name of titles
		once
			Result := <<"Listing","Date","Signature">>
		end

	description : ARRAY[STRING] is
	-- column names
		do
			Result := <<entity_id.out,date.out,signature_id.out>>
		end

	date_name: STRING is
			-- Date field name
		do
			Result := "date"
		end

	listing_signatures (a_listing: LISTING): ROSE_LINEAR_ARRAY[like Current] is
		local
			l_predicate: COLLECTING_PREDICATE[like Current]
		do
			if a_listing /= Void and a_listing.xrf_code > 0 then
				zone_mnemonic := a_listing.zone.mnemonic
				if not zone_mnemonic.is_equal('%U') then
					create l_predicate.make (servers.market_data_servers (zone_mnemonic.out), "select * from data_signatures where rxrf = ? order by [date]")
					l_predicate.retrieve (Void, <<a_listing.xrf_code>>)
					Result := l_predicate.array
				end
			end
		end

	listing_signatures_in_interval(a_listing : LISTING; a_start_date, a_end_date : ROSE_DATE) : ROSE_LINEAR_ARRAY[like Current] is
			-- listing signatures in interval
		local
			l_predicate : COLLECTING_PREDICATE[like Current]
		do
			if a_listing /= Void and a_listing.xrf_code > 0 then
				zone_mnemonic := a_listing.zone.mnemonic
				if not zone_mnemonic.is_equal('%U') then
					create l_predicate.make(servers.market_data_servers (zone_mnemonic.out), "select * from data_signatures where rxrf = ? and [date] >= ? and [date] <= ? order by [date]")
					l_predicate.retrieve(Void, <<a_listing.xrf_code, a_start_date, a_end_date>>)
					Result := l_predicate.array
				end
			end
		end

	to_vector: VECTOR is
			-- to vector
		do
			create Result.make_from_array(<<ask, bid, close, high, low, open, volume>>)
		end

	new_variable (a_variable_description: VARIABLE_DESCRIPTION_RECORD): RMDAT_SIGNATURE_VARIABLE is
			-- new variable
		local
			l_source: STRING
		do
			l_source := a_variable_description.source
			if l_source /= void then
				if l_source.is_equal ("market") then
					create Result.make (a_variable_description)
				end
			end
		end

	target_name: STRING is
			-- target name
		do
			if not zone_mnemonic.is_equal ('%U') then
				Result := "MARKET_DATA_" + zone_mnemonic.out
			else
				Result := "market"
			end
		end

	context_name: STRING is
			-- Entity context name
		do
			if entity /= Void then
				Result := entity.context.name
			end
		end

	zone: ZONE[LOCATION] is
		do
			if not zone_mnemonic.is_equal ('%U') then
				Result := rose_context.zone (zone_mnemonic)
			end
		end

	data_manager: CLUSTERED_TAGGED_ITEM_DATABASE[INTEGER,ROSE_DATE,RMDAT_SIGNATURE_RECORD] is
			-- <Precursor>
		do
			if zone /= Void then
				Result := zone.market_data.background_data
			end
		end

	output_string_by_name (a_name: STRING): STRING is
			-- An interpreted version of the signature field attribute value for a_name for display
		local
			l_handle: INTEGER_REF
			l_market_index: MARKET_INDEX
		do
			if a_name /= Void then
				if a_name.as_lower.is_equal ("entity_id") and then zone /= Void and then zone.mnemonic = 'I' then
					l_handle ?= value_by_name (a_name)
					if l_handle /= Void then
						l_market_index ?= context.entity_at (context.ids.market_index_category_id, l_handle.item)
						if l_market_index /= Void and then l_market_index.is_valid then
							Result := l_market_index.descriptive_name
						end
						if Result = Void then
							Result := l_handle.item.out
						end
					else
						Result := "<Void>"
					end
				else
					Result := Precursor (a_name)
				end
			end
		end

	output_data_field_string_by_name (a_name: STRING): ANY is
		local
			l_currency: PERSISTENT_CURRENCY
		do
			a_name.to_lower
			if a_name.is_equal ("currency") then
				l_currency ?= rose_context.entity_at (rose_context.currency_category_id,currency)
				if l_currency /= Void then
					Result := l_currency.iso_code_key (Void)
				end
				if Result = Void then
					Result := "<null>"
				end
			else
				Result := Precursor (a_name)
			end
		end

	restriction_statement: STRING is
		do
		end

	repaired_statement (a_statement: STRING): STRING is
			-- A restriction statement based on 'a_statement' but adjusted to particulars of this signature
		do
			if zone /= Void and then zone.mnemonic = 'I' then
				if a_statement /= Void then
					Result := a_statement.twin
					Result.replace_substring_all ("entity_id", "rxrf")
				end
			else
				Result := Precursor (a_statement)
			end
		end

feature -- Status

	any_data_at_all: BOOLEAN is
			-- Does this record have any data?
		do
			if (is_defined_double (close) and close > 0.0) or
				(is_defined_double (open) and open > 0.0) or
				(is_defined_double (high) and high > 0.0) or
				(is_defined_double (low) and low > 0.0) or
				(is_defined_double (bid) and bid > 0.0) or
				(is_defined_double (ask) and ask > 0.0) or
				(is_defined_double (volume) and volume > 0.0) then
				Result := True
			end
		end

	match_all_data (a_other_signature: RMDAT_SIGNATURE_RECORD): BOOLEAN is
		local
			i: INTEGER
			l_double, l_other_double: DOUBLE_REF
		do
			Result := True
			from i := 1 until i > data_fields.count
			loop
				l_double ?= value_by_name (data_fields.item (i))
				l_other_double ?= a_other_signature.value_by_name (data_fields.item (i))
				if l_double /= Void and l_other_double /= Void then
					if not ieee_is_eq (l_double.item,l_other_double.item) then
						Result := False
					end
				end
				i := i + 1
			end
		end

feature -- Basic operations

	reconcile_target (a_target: like Current) is
		do
			a_target.set_zone (zone)
			if zone.mnemonic.is_equal('F') then
				a_target.set_entity (zone.future_listing (a_target.rxrf))
			elseif zone.mnemonic.is_equal('I') then
				a_target.set_entity (zone.market_index (a_target.rxrf))
			else
				a_target.set_entity (zone.listing (a_target.rxrf))
			end
		end

feature -- Element modification	

	set_zone (a_zone: ZONE[LOCATION]) is
			-- Set zone_mnemonic to 'a_zone's mnemonic
		do
			zone_mnemonic := a_zone.mnemonic
		end

	set_open (a_value: like open) is
			-- Set open to 'a_value'
		do
			open := a_value
		end

	set_close (a_value: like close) is
			-- Set close to 'a_value'
		do
			close := a_value
		end

	set_high (a_value: like high) is
			-- Set high to 'a_value'
		do
			high := a_value
		end

	set_low (a_value: like low) is
			-- Set low to 'a_value'
		do
			low := a_value
		end

	set_bid (a_value: like bid) is
			-- Set bid to 'a_value'
		do
			bid := a_value
		end

	set_ask (a_value: like ask) is
			-- Set ask to 'a_value'
		do
			ask := a_value
		end

	set_mid_close (a_value: like mid_close) is
			-- Set mid_close to 'a_value'
		do
			mid_close := a_value
		end

	set_volume (a_value: like volume) is
			-- Set volume to 'a_value'
		do
			volume := a_value
		end

	set_currency (a_value: like currency) is
			-- Set currency to 'a_value'
		do
			currency := a_value
		end

	set_entity_id (a_value: like entity_id) is
			-- Set entity_id to 'a_value'
		do
			entity_id := a_value
		end

	set_rxrf_date (a_value: like rxrf; a_date: ROSE_DATE) is
			-- Set rxrf and date at once
		do
			rxrf := a_value
			date := a_date
			saved_multi_key := Void
		end

	set_rxrf (a_value: like rxrf) is
			-- Set rxrf to 'a_value'
		do
			rxrf := a_value
			saved_multi_key := Void
		end

	set_date (a_date: ROSE_DATE) is
			-- Set date to 'a_date'
		do
			date := a_date
			saved_multi_key := Void
		end

	set_entity (a_entity: like entity) is
			-- Set entity to 'a_entity' and set dependent attributes
		do
			entity := a_entity
			set_rxrf (a_entity.xrf_code)
			if entity /= void then
				set_entity_id (entity.handle)
			end
		end

	set_data_null is
			-- set data null
		do
			open := constants.missing_double
			open := constants.missing_double
			close := constants.missing_double
			high := constants.missing_double
			low := constants.missing_double
			bid := constants.missing_double
			ask := constants.missing_double
			mid_close := constants.missing_double
			volume := constants.missing_double
			currency := unknown_currency.handle
		end

feature {NONE} -- Implementation

	key_0_map: HASH_TABLE[INTEGER,INTEGER] is
		once
			create Result.make(1000)
		end

	unknown_currency: PERSISTENT_CURRENCY is
			-- unknow currency
	once
		Result := rose_context.currency_from_name ("Unknown", void)
	end


end




