note

	description:

		"Expression which evaluates an rmdat signature"

	author: "Mauricio Bustos"
	last_editor: "$Author: er $"
	date: "$Date: 2014/01/03 $"
	revision: "$Revision: #8 $"
	path: "$File: //rose/source/kernel/data/market/rmdat_signature_variable.e $"

class
	RMDAT_SIGNATURE_VARIABLE

inherit

	DOUBLE_VALUED_EXPRESSION[RMDAT_SIGNATURE_RECORD]
		redefine
			make,
			string_value
		end

	ROSE_LEGACY_UNDEFINED
		undefine default_create end

create
	make

feature  --Initialization

	make (a_description : like variable_description) is
			--make
		do
			Precursor(a_description)
			if a_description.name.is_equal("unadjusted_closing_price") then
				action := agent unadjusted_closing_action
			elseif a_description.name.is_equal("closing_price") then
				action := agent close_action
			elseif a_description.name.is_equal("opening_price") then
				action := agent open_action
			elseif a_description.name.is_equal("high_price") then
				action := agent high_action
			elseif a_description.name.is_equal("low_price") then
				action := agent low_action
			elseif a_description.name.is_equal("ask_price") then
				action := agent ask_action
			elseif a_description.name.is_equal("bid_price") then
				action := agent bid_action
			elseif a_description.name.is_equal("volume") then
				action := agent volume_action
			elseif a_description.name.is_equal("last_price") then
				action := agent last_action
			elseif a_description.name.is_equal("adjustment_factor") then
				action := agent adjustment_action
			elseif a_description.name.is_equal("currency_iso") then
				type := string_type
			end
		end

feature {NONE}  --Implementation

	action : FUNCTION[ANY,TUPLE[like expression_target],DOUBLE]
	unadjusted_closing_action (a_target : like expression_target) : DOUBLE is
		do
			Result := listing(a_target).unadjusted_closing_price(a_target.date)
		end
	close_action (a_target : like expression_target) : DOUBLE is
		do
			Result := listing(a_target).closing_price(a_target.date)
		end
	open_action (a_target : like expression_target) : DOUBLE is
		do
			Result := listing(a_target).opening_price(a_target.date)
		end
	high_action (a_target : like expression_target) : DOUBLE is
		do
			Result := listing(a_target).high_ask(a_target.date)
		end
	low_action (a_target : like expression_target) : DOUBLE is
		do
			Result := listing(a_target).low_bid(a_target.date)
		end
	bid_action (a_target : like expression_target) : DOUBLE is
		do
			Result := listing(a_target).bid(a_target.date)
		end
	ask_action (a_target : like expression_target) : DOUBLE is
		do
			Result := listing(a_target).ask(a_target.date)
		end
	volume_action (a_target : like expression_target) : DOUBLE is
		do
			Result := listing(a_target).volume(a_target.date)
		end
	last_action (a_target : like expression_target) : DOUBLE is
		do
			Result := listing(a_target).last_price(a_target.date)
		end
	adjustment_action(a_target : like expression_target) : DOUBLE is
		do
			Result := listing(a_target).adjustment(a_target.date)
		end
	listing(a_target : like expression_target) : LISTING is
		do
			Result ?= a_target.entity
		end

feature  --Access

	currency (a_target : like expression_target) : CURRENCY is
		do
			Result := listing(a_target).rmdat_currency(a_target.date)
		end

	at (a_target : like expression_target) : DOUBLE is
			--value
		do
			if action /= Void then
				if a_target /= Void then
					Result := action.item([a_target])
				end
			end
		end

	string_value(a_target : like expression_target) : STRING is
			-- <Precursor>
		local
			l_currency : CURRENCY
		do
			if action = Void and variable_description.name.is_equal("currency_iso") then
				l_currency := listing(a_target).rmdat_currency(a_target.date)
				if l_currency /= Void then
					Result := l_currency.iso_code_key(a_target.date)
				end
			end
		end

end

