indexing
	description: "Objects that ..."
	author: "Michael Zhang"
	date: "$Date: 2008/12/01 $"
	revision: "$Revision: #11 $"

class
	ROSE_ADJUSTMENT_HISTORY[G->ENTITY]

inherit
	ROSE_INTERVAL_HISTORY[G, ADJUSTMENT_INTERVAL[G]]

	ROSE_LEGACY_UNDEFINED
	
create
	make_empty, make_from_adjustment_series

feature
	make_from_adjustment_series(a_entity : G; a_adjustment_series : DOUBLE_TIME_SERIES) is
		local
			l_interval : like interval
			l_cumadj : DOUBLE
			l_count, i : INTEGER
			l_backadj_array : ROSE_LINEAR_ARRAY[DOUBLE]
		do
			make_empty
			entity := a_entity
			
			l_count := a_adjustment_series.count
			create l_backadj_array.make(1, l_count + 1)
			l_backadj_array.put(1.0, l_count + 1)
			l_cumadj := 1.0
			from i := l_count until i < 1 loop
				l_cumadj := l_cumadj * a_adjustment_series.item(i)
				l_backadj_array.put(l_cumadj, i)
				i := i - 1
			end
			
			create l_interval.make(a_entity, constants.minimum_date_time, a_adjustment_series.date(1).previous_date, l_backadj_array.item(1), -999.0)		
			extend(l_interval)
			from i := 1 until i > (l_count - 1) loop
				create l_interval.make(a_entity, a_adjustment_series.date(i), a_adjustment_series.date(i + 1).previous_date, l_backadj_array.item(i + 1), a_adjustment_series.item(i))
				extend(l_interval)
				i := i + 1
			end
			create l_interval.make(a_entity, a_adjustment_series.date(l_count), constants.maximum_date_time, l_backadj_array.item(l_count + 1), a_adjustment_series.item(l_count))		
			extend(l_interval)
		end
		
feature
	return(a_start_price, a_end_price : DOUBLE; a_start_date, a_end_date : ROSE_DATE) : DOUBLE is
		local
			l_adj_start_price, l_adj_end_price : DOUBLE
			l_start_adj, l_end_adj : DOUBLE
		do
			l_start_adj := adjustment(a_start_date)
			l_end_adj := adjustment(a_end_date)
			
			if is_defined_double(l_start_adj) and then l_start_adj > 0 then
				l_adj_start_price := a_start_price / l_start_adj
			end
			
			if is_defined_double(l_end_adj) and then l_end_adj > 0 then
				l_adj_end_price := a_end_price / l_end_adj
			end
			
			if l_adj_start_price > 0 then
				Result := (l_adj_end_price - l_adj_start_price) / l_adj_start_price
			else
				Result := constants.missing_double
			end
		end

	return_at(a_price_series : DOUBLE_TIME_SERIES; a_date : ROSE_DATE) : DOUBLE is
		local
			l_beg_date : ROSE_DATE
			l_beg_price, l_end_price : DOUBLE
		do
			if a_price_series /= Void and then a_price_series.has_date(a_date) then
				l_beg_date := a_price_series.date_before(a_date)
				if l_beg_date /= Void then
					l_beg_price := a_price_series.item_at(l_beg_date)
					l_end_price := a_price_series.item_at(a_date)
					Result := return(l_beg_price, l_end_price, l_beg_date, a_date)
				end
			else
				Result := constants.missing_double
			end	
		end

	sub_return_series(a_price_series : DOUBLE_TIME_SERIES; a_dates : ARRAY[ROSE_DATE]) : DOUBLE_TIME_SERIES is
		do
			create Result.make_with_agent(Agent return_at(a_price_series, ?), a_dates)
		end
		
feature	
	adjustment(a_date : ROSE_DATE) : DOUBLE is
		local
			l_interval : like interval
		do
			l_interval := witness_interval(a_date)
			if l_interval /= Void then
				Result := l_interval.adjustment
			else
				Result := constants.missing_double
			end
		end
		
	adjustment_series(a_dates : ARRAY[ROSE_DATE]) : DOUBLE_TIME_SERIES is
		do
			create Result.make_with_agent(Agent adjustment, a_dates)
		end

feature
	has_valid_adjustment_at(a_date : ROSE_DATE) : BOOLEAN is
		local
			l_adjustment : DOUBLE
		do
			l_adjustment := adjustment(a_date)
			Result := is_defined_double(l_adjustment) and then l_adjustment > 0	
		end

	foradj(a_date, a_as_of_date : ROSE_DATE) : DOUBLE is
		require
			valid_date : (a_date /= Void and a_as_of_date /= Void) 
		local
			l_adjustment, l_as_of_adjustment : DOUBLE
		do
			l_adjustment := adjustment(a_date)
			l_as_of_adjustment := adjustment(a_as_of_date)
			if (is_defined_double(l_adjustment) and then l_adjustment > 0) and (is_defined_double(l_as_of_adjustment) and then l_as_of_adjustment > 0) then
				Result := l_as_of_adjustment / l_adjustment
			else
				Result := constants.missing_double
			end				
		end

	adjusted_value_as_of(a_original_value : DOUBLE; a_date, a_as_of_date : ROSE_DATE) : DOUBLE is
		require
			valid_date : (a_date /= Void and a_as_of_date /= Void) 
		local
			l_foradj : DOUBLE
		do
			l_foradj := foradj(a_date, a_as_of_date)
			if is_defined_double(l_foradj) then
				Result := a_original_value * l_foradj
			else
				Result := constants.missing_double
			end
		end

	adjusted_value(a_original_value : DOUBLE; a_date : ROSE_DATE) : DOUBLE is
		local
			l_adjustment : DOUBLE
		do
			l_adjustment := adjustment(a_date)
			if is_defined_double(l_adjustment) and then l_adjustment > 0 then
				Result := a_original_value / l_adjustment
			else
				Result := constants.missing_double
			end
		end

	adjusted_series(a_series : DOUBLE_TIME_SERIES) : DOUBLE_TIME_SERIES is
		do
			if a_series /= Void then
				Result := a_series.transformed_series(Agent adjusted_value)
			end
		end
		
feature
	entity : G
end 
