note
	description: "Application resource"
	author: "Zoran Simic"
	last_editor: "$Author: zs $"
	date: "$Date: 2011/04/19 $"
	revision: "$Revision: #8 $"
	path: "$File: //rose/source/kernel/infrastructure/framework/application/rose_application_resource.e $"

class
	ROSE_APPLICATION_RESOURCE

inherit
	ROSE_FRAMEWORK_CONVENTION

create {ROSE_APPLICATION}
	make

feature {NONE} -- Initialization

	make (a_type: STRING; a_app: ROSE_APPLICATION)
			-- Create for 'a_type' and 'a_app'
		require
			type_ok: a_type /= Void
			app_ok: a_app /= Void
		local
			l_path: STRING
		do
			type := a_type
			create search_path.make_empty
			search_path.compare_objects
			if a_app.eifgen_folder /= Void then
				l_path := a_app.eifgen_folder
			else
				l_path := a_app.program_folder
			end
			if a_app.platform.is_vms then
				-- Workaround for VMS, where program folder and eifgen folder are not correctly determined
				-- We use the current working directory....
				search_path.extend ("")
			end
			search_path.extend (l_path)
			add_path (formatter.composed_path (<<l_path, type>>))
		end

feature -- Access

	type: STRING
			-- Resource type

	search_path: ROSE_LINEAR_ARRAY [STRING]
			-- Search paths

	get_file (a_filename: STRING): STRING
			-- Return path to first found file in 'search_path'
		local
			i: INTEGER
		do
			if formatter.is_relative_path (a_filename) then
				from
					i := search_path.lower
				until
					Result /= Void or else i > search_path.upper
				loop
					Result := checked_path (search_path.item (i), a_filename)
					i := i + 1
				end
			else
				Result := a_filename
			end
		end

feature {ROSE_APPLICATION} -- Basic operations

	add_path (a_path: STRING)
			-- Add paths from 'a_path' (might be ';' separated paths) to 'search_path'.
		local
			l_paths: SEQUENCE [STRING]
		do
			l_paths := formatter.split (a_path, path_separators, Void)
			if l_paths /= Void then
				from
					l_paths.start
				until
					l_paths.off
				loop
					if is_directory_readable (l_paths.item) and then not search_path.has (l_paths.item) then
						search_path.extend (l_paths.item)
					end
					l_paths.forth
				end
			end
		end

	add_common_path (a_path: STRING)
			-- Add 'a_path' / common / 'type'
		do
			if a_path /= Void and then not a_path.is_empty then
				add_path (formatter.composed_path (<<a_path, "common", type>>))
			end
		end

end
