indexing
	description: "Application status view"
	author: "Zoran Simic"
	date: "$Date: 2008/12/01 $"
	revision: "$Revision: #4 $"

class
	ROSE_APPLICATION_VIEW

inherit
	ROSE_SHARED_GLOBAL_GUI
		undefine
			default_create, is_equal, copy
		end
	EV_VERTICAL_BOX

create
	make

feature {NONE} -- Initialization

	make (a_container: EV_CONTAINER) is
			-- Create an application status view, and put it in 'a_container' if provided.
		do
			default_create
			Gui.factory.set_box_defaults (a_container,Current,2,6)
			create info_list.make
				-- Application
			add_box ("Application")
			add_info ("application","Name","Application name")
			add_info ("version","Version","Application version")
			add_info ("project","Project","Application project")
			add_box (Void)
			add_info ("application_description",Void,"Application description")
				-- Config
			add_box ("Configuration")
			add_info ("user_full_name","User","User domain and name")
			add_button ("View config",agent on_config (False))
			add_info ("config","Config","Configuration path")
			add_button ("View app config",agent on_config (True))
				-- System
			add_box ("System")
			add_info ("host_name","Host name","Host name")
			add_info ("pid","Pid","Process id")
			add_info ("platform","Platform","Platform")
			add_box (Void)
			add_info ("program_path","Program","Program path")
			add_button ("Explore",agent on_explore ("program_path"))
			add_box (Void)
			add_info ("program_arguments","Arguments","Program arguments")
				-- Installation
			add_box ("Installation")
			add_info ("ROSE","Install dir","Installation directory")
			add_button ("Explore",agent on_explore ("ROSE"))
			add_box (Void)
			add_info ("ROSE_PATH","Install path","Installation search path")
			add_button ("Explore",agent on_explore ("ROSE_PATH"))
				-- Misc
			add_box ("Misc")
			add_info ("start_time","Started","Application's start time")
			add_info ("default_command","Default command","Default command")
			add_info ("default_command_line","Default arguments","Default command line")
		end

feature -- Access

	app: ROSE_APPLICATION
			-- Viewed application

feature -- Basic operations

	set_app (a_app: like app) is
		do
			app := a_app
			refresh
		end

	refresh is
			-- Refresh current view.
		local
			l_field: EV_TEXT_COMPONENT
			l_key: STRING
		do
			from
				info_list.start
			until
				info_list.off
			loop
				l_field := info_list.item
				l_key ?= l_field.data
				if l_key /= Void and then app /= Void then
					set_info (l_field,app.value (l_key))
				else
					set_info (l_field,Void)
				end
				info_list.forth
			end
		end

feature {NONE} -- Implementation

	info_list: LINKED_LIST [EV_TEXT_COMPONENT]
	user_config, app_config: PAIR [EV_WINDOW,ROSE_CONFIGURATION_VIEW]

	on_config (a_app: BOOLEAN) is
		local
			l_pair: like user_config
			l_title: STRING
		do
			if app_config = Void then
				create app_config
				create user_config
			end
			if a_app then
				l_pair := app_config
			else
				l_pair := user_config
			end
			if l_pair.first = Void then
				if a_app then
					l_title := "Application configuration"
				else
					l_title := "User configuration"
				end
				l_pair.set_first (Gui.factory.new_window (l_title, "config.ico", 500, 300))
				l_pair.set_second (create {ROSE_CONFIGURATION_VIEW}.make (l_pair.first))
			end
			if app /= Void then
				if a_app then
					l_pair.second.set_config (config)
				else
					l_pair.second.set_config (Global.user.config)
				end
				Gui.open_window (Void,l_pair.first)
			end
		end

	on_explore (a_key: STRING) is
			-- Explore path corresponding to variable with 'a_key'.
		local
			l_path: STRING
		do
			if app /= Void then
				l_path := app.value (a_key)
				if l_path /= Void then
					l_path := Global.formatter.split_on_first (l_path,";",Void).first
					Global.explore (l_path)
				end
			end
		end

	set_info (a_field: EV_TEXT_COMPONENT; a_object: ANY) is
			-- Set info 'a_filed' to 'a_object'.
		local
			l_string: STRING
		do
			if a_field = Void then
			elseif a_object = Void then
				a_field.remove_text
			else
				l_string := a_object.out
				a_field.set_text (l_string)
			end
		end

	current_vbox, current_hbox: EV_BOX
	add_box (a_title: STRING) is
			-- Add a new hbox, with frame with 'a_title' if provided.
		local
			l_frame: EV_FRAME
		do
			if a_title /= Void then
				l_frame := Gui.factory.new_frame (Current,a_title,1,1)
				l_frame.set_foreground_color (section_color)
				current_vbox := Gui.factory.new_vbox (l_frame,2,4)
			end
			current_hbox := Gui.factory.new_hbox (current_vbox,0,4)
		end

	add_button (a_label: STRING; a_agent: PROCEDURE [ANY,TUPLE []]) is
			-- Add a button with 'a_label' and 'a_agent'.
		local
			l_widget: EV_WIDGET
		do
			l_widget := Gui.factory.new_button (current_hbox,a_label,a_agent,1)
			l_widget.set_foreground_color (label_color)
			l_widget := Gui.factory.new_label (current_hbox,Void,5)
		end

	add_info (a_key, a_label, a_tooltip: STRING) is
			-- Add an info field for variable with 'a_key', with 'a_label' and 'a_tooltip'.
		require
			valid_key: a_key /= Void
		local
			l_field: EV_TEXT_FIELD
			l_label: EV_LABEL
		do
			l_label := Gui.factory.new_label (current_hbox,Void,3)
			l_label.set_background_color (Gui.rep.colors.Color_3d_shadow)
			if a_label /= Void then
				l_label := Gui.factory.new_label (current_hbox,a_label + ": ",1)
				l_label.set_foreground_color (label_color)
			end
			l_field := Gui.factory.new_text_field (current_hbox,Void,Void,Void,0)
			l_field.disable_edit
			l_field.set_background_color (Gui.rep.colors.Color_3d_highlight)
			l_field.set_data (a_key)
			if a_tooltip /= Void then
				l_field.set_tooltip (a_tooltip + " [ $" + a_key + " ]")
			end
			l_label := Gui.factory.new_label (current_hbox,Void,5)
			info_list.extend (l_field)
		end

	section_color: EV_COLOR is once Result := Gui.rep.colors.Blue end
	label_color: EV_COLOR is once Result := Gui.rep.colors.Dark_blue end

end
