note
	description: "Early class for transferring arrays between Eiffel and external routines"
	author: "Mark Howard"
	last_editor: "$Author: zs $"
	date: "$Date: 2011/05/02 $"
	revision: "$Revision: #15 $"
	path: "$File: //rose/source/kernel/infrastructure/translation/rose_array.e $"

deferred class
	ROSE_ARRAY [G -> COMPARABLE]

inherit
	ROSE_LINEAR_ARRAY [G]

feature -- Access

	byte_length: INTEGER
			-- Byte length for 'fill_from_bytes'
		deferred
		end

	item_from_bytes (a_bytes: ROSE_BYTES; i: INTEGER): G
			-- Convert 'a_bytes' into object of type G
		deferred
		end

feature -- Basic operations

   fill_from_string (a_string: STRING)
   		-- Fill with 'a_string', use 'item_from_bytes' to convert 'a_string' into object of type G
	  local
		 l_bytes: ROSE_BYTES
	  do
		 create l_bytes.make (a_string)
		 fill_from_bytes (l_bytes)
	  end

	fill_from_bytes (a_bytes: ROSE_BYTES)
			-- Fill with 'a_bytes', use 'item_from_bytes' to convert 'a_string' into object of type G
		local
			i: INTEGER
			l_index: INTEGER
		do
			make (1, ((a_bytes.count + byte_length - 1) // byte_length))
			from
				i := 1
				l_index := 1
			until
				i > a_bytes.count
			loop
				put (item_from_bytes (a_bytes, i), l_index)
				i := i + byte_length
				l_index := l_index + 1
			end
		end

	fill_from_pointer (a_pointer: POINTER; a_count: INTEGER)
			-- Fill with bytes pointed to by 'a_pointer' and with 'a_count' length, use 'item_from_bytes' to convert 'a_string' into object of type G
		local
			l_bytes: ROSE_BYTES
		do
			create l_bytes.make_from_pointer (a_pointer, a_count * byte_length)
			fill_from_bytes (l_bytes)
		end

end
