indexing

	description:

		"Two-dimensional arrays (Rose Extension)"

	author: "Michael Zhang"
	last_editor: "$Author: it $"
	date: "$Date: 2015/01/14 $"
	revision: "$Revision: #25 $"
	path: "$File: //rose/source/kernel/infrastructure/containers/rose_array2.e $"
	storable_version: ""

class	ROSE_ARRAY2[G]

inherit

	ARRAY2[G]

	MISMATCH_CORRECTOR
		undefine
			copy, is_equal
		redefine
			correct_mismatch
		end

	ROSE_IEEE_UNDEFINED
		export
			{NONE} all
		undefine
			copy, is_equal
		end


create

	make, make_filled, make_from_array_of_array, make_from_others, make_from_another

feature {NONE} -- Creation

	make_from_array_of_array(a_arrays: ARRAY [ARRAY[G]]) is
			-- Make from array of array
		require
			a_arrays_not_void: a_arrays /= Void
		local
			l_g: G
		do
			make_filled (l_g, a_arrays.count, a_arrays.item(a_arrays.upper).count)
			insert_array_of_array(1, a_arrays)
		end

	make_from_another(a_array2: ARRAY2 [G]) is
			-- Make from another array2
		local
			l_g: G
		do
			make_filled (l_g, a_array2.height, a_array2.width)
			set_area(a_array2.area.twin)
		end

	make_from_others (a_others: ARRAY [ROSE_ARRAY2[G]]) is
			-- Make from an array of array2s - this requires equal width objects
		require
			a_others_not_void: a_others /= Void
		local
			l_other: ROSE_ARRAY2[G]
			i,j,k,m: INTEGER
			l_rows_names_exists:BOOLEAN
			l_row_names: like row_names
			l_g: G
		do
			k := a_others.count
			from i := 1 until i > k  loop
				j := j + a_others.item(i).height
				i := i + 1
			end
			make_filled (l_g, j, a_others.item(1).width)
			from i := 1 until i > k  loop
				l_other:=a_others.item(i)
				from j := 1 until j>l_other.height loop
					m := m + 1
					insert_row(m, l_other.row(j))
					j := j + 1
				end
				i := i + 1
			end
			if a_others.item (1).column_names /= Void then
				set_column_names (a_others.item (1).column_names)
			end
			if a_others.item (1).row_names /= Void then
				l_rows_names_exists := True
				create l_row_names.make_from_array(a_others.item(1).row_names)
				from i := 1 until i > k loop
					l_rows_names_exists := l_rows_names_exists and a_others.item(i).row_names /= Void
					if i > 1 then
						l_row_names:=l_row_names+a_others.item(i).row_names
					end
					i := i + 1
				end
				if l_rows_names_exists then
				  	set_row_names(l_row_names)
				end
			end
		end


feature -- Access

	valid_indices (i, j: INTEGER): BOOLEAN is
			-- valid row and column indices
		do
			Result := (1 <= i and i <= height) and (1 <= j and j <= width)
		end


feature  --Access

	column_index(a_name:STRING):INTEGER is
			-- Index of column from name
		do
			Result := column_table.item(a_name)
		end

	row_index(a_name:STRING):INTEGER is
			-- Index of row from name
		do
			Result := row_table.item(a_name)
		end

	item_by_name(a_row_name,a_column_name:STRING):like item is
			-- Item by row and column names
		local
			i,j:INTEGER
		do
			i:=row_index (a_row_name)
			j:=column_index(a_column_name)
			if i>0 and j>0 then
				Result:=item (i,j)
			end
		end

feature -- Selections and subsets

	sub_array2 (a_row_1, a_row_2, a_col_1, a_col_2: INTEGER): like Current is
			-- Subset of `Current'
		require
			valid_row_index: a_row_1 >= 1 and a_row_1 <= a_row_2 and a_row_2 <= height
			valid_col_index: a_col_1 >= 1 and a_col_1 <= a_col_2 and a_col_2 <= width
		local
			l_width: INTEGER
			l_height: INTEGER
			i , j: INTEGER
			l_item: G
		do
			l_height := a_row_2 - a_row_1 + 1
			l_width := a_col_2 - a_col_1 + 1
			create Result.make_filled (l_item, l_height, l_width)

			from i := a_row_1 until i > a_row_2 loop
				from j := a_col_1 until j > a_col_2 loop
					l_item := item(i, j)
					Result.put(l_item, i - a_row_1 + 1, j - a_col_1 + 1)
					j := j + 1
				end
				i := i + 1
			end
		ensure
			not_void: Result /= Void
			width: Result.width = a_col_2 - a_col_1 + 1
			height: Result.height = a_row_2 - a_row_1 + 1
		end

	columns (a_col_1, a_col_2: INTEGER): like Current is
			-- Subset by a range of columns
		require
			valid_col_index : a_col_1 >= 1 and a_col_1 <= a_col_2 and a_col_2 <= width
		do
			Result := sub_array2(1, height, a_col_1, a_col_2)
		ensure
			not_void: Result /= Void
			height: Result.height = height
			width: Result.width = a_col_2 - a_col_1 + 1
		end

	rows (a_row_1, a_row_2: INTEGER): like Current is
			-- Subset by range of rows
		require
			valid_row_index: a_row_1 >= 1 and a_row_1 <= a_row_2 and a_row_2 <= height
		do
			Result := sub_array2 (a_row_1, a_row_2, 1, width)
		ensure
			not_void: Result /= Void
			width: Result.width = width
			height: Result.height = a_row_2 - a_row_1 + 1
		end

	column (a_col: INTEGER): ROSE_LINEAR_ARRAY[G] is
			-- Column of items by index
		require
			valid_index: a_col > 0 and a_col <= width
		local
			l_item: G
			i: INTEGER
		do
			create Result.make (1, height)
			from i := 1
			until i > height
			loop
				l_item := item (i, a_col)
				Result.put (l_item, i)
				i := i + 1
			end
		ensure
			not_void: Result /= Void
			size: Result.count = height
		end

	row (a_row: INTEGER): ROSE_LINEAR_ARRAY [G] is
			-- Row of items by index
		require
			valid_index : a_row > 0 and a_row <= height
		local
			l_item: G
			i: INTEGER
		do
			create Result.make (1, width)
			from i := 1
			until i > width
			loop
				l_item := item (a_row, i)
				Result.put (l_item, i)
				i := i + 1
			end
		ensure
			not_void: Result /= Void
			size: Result.count = width
		end

	row_by_name (a_name:STRING): like row is
			-- Row of items by row name
		do
			Result := row (row_index (a_name))
		end

	tabled_row (a_name: STRING): HASH_TABLE [G, STRING] is
			-- Row by name, with content indexed by column name
		local
			l_row: like row
			i: INTEGER
		do
			if column_names /= Void then
			l_row := row_by_name (a_name)
				create Result.make (l_row.count)
				from i:=1 until i > l_row.count loop
					Result.put (l_row.item(i), column_names.item (i))
					i := i + 1
				end
			end
		end

	as_matrix: MATRIX
			-- Convert current object to matrix, using `double_item'
		local
			i, j: INTEGER
		do
			create Result.make (height, width)
			from i := 1 until i > height loop
				from j := 1 until j > width loop
					Result.put (double_item (i, j), i, j)
					j := j + 1
				end
				i := i + 1
			end
			Result.set_column_names (column_names)
			Result.set_row_names (row_names)
		ensure
			not_void: Result /= Void
			size: Result.width = width and Result.height = height
		end

	as_array_of_array: ROSE_LINEAR_ARRAY [ARRAY [ANY]]
			-- Data as array of row arrays
		do
			create Result.make_empty;
			(1 |..| height).do_all (agent add_to_array_of_array (Result, ?))
		end

	double_item (i, j: INTEGER): DOUBLE
			-- `item (i, j)' converted to double if it's a double value or a double string
		require
			valid_indices: valid_indices (i, j)
		local
			l_item: like item
		do
			Result := undefined_double
			l_item := item (i, j)
			if attached {DOUBLE} l_item as l_double then
				Result := l_double
			elseif attached {REAL} l_item as l_real then
				Result := l_real
			elseif attached {RP_DECIMAL} l_item as l_decimal then
				Result := l_decimal.to_double
			elseif attached {STRING} l_item as l_string then
				if l_string.is_double then
					Result := l_string.to_double
				end
			end
		end

feature {NONE} -- Implementation

	add_to_array_of_array (a_result: like as_array_of_array; a_row_index: INTEGER)
			-- Implementation for `as_array_of_array'
		require
			a_result_not_void: a_result /= Void
			a_row_index: a_row_index > 0 and a_row_index <= height
		do
			a_result.extend (row (a_row_index))
		end

feature  -- Transformation

	transposed: like Current is
			-- Copy with rows and columns swapped
		local
			i, j: INTEGER
			l_g: G
		do
			create Result.make_filled (l_g, width, height)
			from i := 1 until i > height loop
				from j := 1 until j > width loop
					Result.put(item(i, j), j, i)
					j := j + 1
				end
				i := i + 1
			end
			Result.set_name(name)
			Result.set_row_names(column_names)
			Result.set_column_names(row_names)
		end

	sorted_by_rows (a_sort_function: FUNCTION [ANY, TUPLE [ROSE_LINEAR_ARRAY [G], ROSE_LINEAR_ARRAY [G]], BOOLEAN]): like Current is
			-- Return a copy of `Current', sorted by rows using `a_sort_function'.
		require
			a_sort_function_not_void: a_sort_function /= Void
		local
			l_rows: ROSE_LINEAR_ARRAY [ROSE_LINEAR_ARRAY [G]]
			i: INTEGER
			l_g: G
		do
			create l_rows.make_empty
			from
				i := 1
			until
				i > height
			loop
				l_rows.extend (row (i))
				i := i + 1
			end
			l_rows.sort_by (a_sort_function)
			create Result.make_filled (l_g, height, width)
			from
				i := 1
			until
				i > l_rows.count
			loop
				Result.insert_row (i, l_rows.item (i))
				i := i + 1
			end
		ensure
			result_not_void: Result /= Void
		end

	sorted_by_columns (a_sort_function: FUNCTION [ANY, TUPLE [ROSE_LINEAR_ARRAY [G], ROSE_LINEAR_ARRAY [G]], BOOLEAN]): like Current is
			-- Return a copy of `Current', sorted by columns using `a_sort_function'.
		require
			a_sort_function_not_void: a_sort_function /= Void
		local
			l_columns: ROSE_LINEAR_ARRAY [ROSE_LINEAR_ARRAY [G]]
			i: INTEGER
			l_g: G
		do
			create l_columns.make_empty
			from
				i := 1
			until
				i > height
			loop
				l_columns.extend (column (i))
				i := i + 1
			end
			l_columns.sort_by (a_sort_function)
			create Result.make_filled (l_g, height, width)
			from
				i := 1
			until
				i > l_columns.count
			loop
				Result.insert_column (i, l_columns.item (i))
				i := i + 1
			end
		ensure
			result_not_void: Result /= Void
		end

feature -- Insertion

	insert_array_of_array (a_start_row: INTEGER; a_arrays: ARRAY [ARRAY [G]]) is
			-- Replace rows from `a_start_row'.
		require
			a_arrays_not_void: a_arrays /= Void
			no_voids: not a_arrays.has (Void)
			good_start_row: a_start_row > 0
			conforming_arrays: a_start_row + a_arrays.count <= height
		local
			i: INTEGER
		do
			from i := a_arrays.lower until i > a_arrays.upper loop
				insert_row (a_start_row + i - a_arrays.lower, a_arrays.item(i))
				i := i + 1
			end
		end

	insert_column (a_col: INTEGER; a_array: ARRAY[G]) is
			-- Replace column.
		require
			valid_index : a_col > 0 and a_col <= width
			valid_array : a_array /= Void
		local
			i : INTEGER
			l_item : G
		do
			from i := 1
			until i > height.min (a_array.count)
			loop
				l_item := a_array.item(i)
				put (l_item, i, a_col)
				i := i + 1
			end
		end

	insert_row (a_row : INTEGER; a_array : ARRAY[G]) is
			-- Replace row.
		require
			valid_index : a_row > 0 and a_row <= height
			valid_array : a_array /= Void
		local
			i: INTEGER
			l_item: G
			k: INTEGER
		do
			k := width.min (a_array.count)
			from i := 1
			until i > k
			loop
				l_item := a_array.item(i)
				put (l_item, a_row, i)
				i := i + 1
			end
		end

	insert_array2 (a_row, a_col: INTEGER; a_array2: ARRAY2 [G]) is
			-- Replace substructure at position `a_row', `a_col'
		require
			valid_indices: valid_indices (a_row, a_col)
			valid_array2: a_array2 /= Void
		local
			i, j: INTEGER
		do
			from i := a_row until i >= a_row + a_array2.height or i > height loop
				from j := a_col until j >= a_col + a_array2.width or j > width loop
					put (a_array2.item(i - a_row + 1, j - a_col + 1),  i, j)
					j := j + 1
				end
				i := i + 1
			end
		end

feature  -- Names

	name: STRING
			-- Name

	set_name(a_name: like name) is
			-- Set name
		do
			name := a_name
		end

	row_names: ROSE_HASHED_LINEAR_ARRAY [STRING]
			-- Row names

	set_row_names(a_row_names: ARRAY [STRING]) is
			-- Set row names
		do
			if a_row_names /= Void then
				create row_names.make_from_array (a_row_names)
				row_table_saved := Void
			end
		end

	column_names: ROSE_HASHED_LINEAR_ARRAY[STRING]
			-- Column names

	set_column_names(a_column_names : ARRAY[STRING]) is
			-- Set column names
		do
			if a_column_names /= Void then
				create column_names.make_from_array (a_column_names)
				column_table_saved := Void
			end
		end

	column_indices (a_names: ARRAY [STRING]): ROSE_LINEAR_ARRAY [INTEGER] is
			-- Column indices for those columns with names in `a_names'
		require
			a_names_attached: a_names /= Void
			existing_names: a_names.for_all (agent column_table.has)
		do
			create Result.make(1, a_names.count)
			across
				a_names as i_names
			loop
				Result.put (column_table.item (i_names.item), i_names.cursor_index)
			end
		ensure
			not_void: Result /= Void
		end

	row_indices (a_names: ARRAY [STRING]): ROSE_LINEAR_ARRAY [INTEGER] is
			-- Row indices for those rows with names in `a_names'
		require
			a_names_attached: a_names /= Void
			existing_names: a_names.for_all (agent row_table.has)
		do
			create Result.make(1, a_names.count)
			across
				a_names as i_names
			loop
				Result.put (row_table.item (i_names.item), i_names.cursor_index)
			end
		ensure
			not_void: Result /= Void
		end

	column_table: HASH_TABLE [INTEGER,STRING] is
				-- Column indices indexed by names
			require
				column_names_not_void: column_names/=void
			local
				l_name: STRING
			do
				if column_table_saved = Void  then
					create Result.make (column_names.count)
					from column_names.start until column_names.off loop
						l_name := column_names.current_item.twin
						Result.put (column_names.index, l_name)
						column_names.forth
					end
					column_table_saved:=result
				else
					Result := column_table_saved
				end
			end

	column_table_saved: like column_table
			-- Cache for `column_table'

	row_table: HASH_TABLE [INTEGER,STRING] is
				-- Row indices indexed by names
			require
				row_names_not_void: row_names /= Void
			local
				l_name: STRING
			do
				if row_table_saved = Void  then
					create Result.make (row_names.count)
					from row_names.start until row_names.off loop
						l_name := row_names.current_item.twin
						Result.put (row_names.index, l_name)
						row_names.forth
					end
					row_table_saved := Result
				else
					Result := row_table_saved
				end
			end

	row_table_saved: like row_table
			-- Cache for `row_table'

feature {NONE} --Correct mismatch

	correct_mismatch is
			-- Attempt to correct object mismatch using `mismatch_information'.
		local
			l_mismatch: MISMATCH_INFORMATION
		do
			l_mismatch := mismatch_information
		end
end
