note

	description:

		"binary expression with aritmetic operations"

	keywords: "rose,expression,binary"
	author: "Mark Howard"
	last_editor: "$Author: it $"
	date: "$Date: 2014/03/14 $"
	revision: "$Revision: #43 $"
	path: "$File: //rose/source/kernel/math/rose_binary_expression.e $"
	storable_version: ""

class	ROSE_BINARY_EXPRESSION [reference G]

inherit
	RP_SERIALIZABLE
	ROSE_EXPRESSION [G]
		rename
			make as rose_expression_make
		export {ANY}
			computed_double,
			computed_real,
			computed_integer,
			computed_units
		undefine
			display_string,
			component_expressions,
			sub_expressions,
			units,
			partial_real_value,
			partial_double_value,
			partial_integer_value,
			partial_value_in_market_currency,
			value_in_market_currency,
			unitized_value_in_market_currency,
			boolean_value,
			string_value,
			computed_double,
			computed_real,
			computed_integer,
			computed_units,
			internal_generated_real,
			internal_generated_double,
			internal_generated_integer,
			internal_generated_string,
			internal_generated_date,
			is_defined_real,
			is_defined_double,
			decimal_value,
			generated_unitized_value,
			is_flow
		redefine
			mnemonic,
			computed_string,
			generated_value_in_market_currency,
			nominal_units
		end
create

	make

feature {NONE}--Initialization

	make (a_left : like left;a_right : like right;a_operator : like operator) is
			-- Create a binary expression.
		do
			operator_string := a_operator.name
			if a_left.mnemonic /= void and a_right.mnemonic /= void then
				mnemonic := a_left.mnemonic + operator_string + a_right.mnemonic
			end
			operator := a_operator
			left := a_left
			right := a_right
			name := "(" + left.name + " " + operator_string + " " + right.name + ")"
			rose_expression_make(void)
			type := operator.type
			create variable_description
			variable_description.set_name (name)
		end
feature --Access
	left : attached ROSE_EXPRESSION[G]
		-- Left expression
	right : attached ROSE_EXPRESSION[G]
		-- Right expression
	expression_string(a_target : like expression_target) : STRING is
			--readable text form
		do
			Result := "(" + left.display_string(a_target) + " " + operator_string + " " + right.display_string(a_target) + ")"
		end
	display_string(a_target : like expression_target) : STRING is
			--display string
		do
			Result := "(" + left.display_string(a_target) + " " + operator_string + " " + right.display_string(a_target) + ")"
		end
	operator_string : STRING
	operator : ROSE_BINARY_OPERATOR[like expression_target]
	description : STRING is
			--do nothing
		do
			--Result := "(" + left.description + ")" + operator_string + "(" + right.description + ")"
		end
	mnemonic : STRING
	get_real_value (a_target : like expression_target) : REAL is
			--reval value from target
		do
			if operator /= void then
				Result := operator.real_value(left,right,a_target)
			end
		end
	get_double_value (a_target : like expression_target) : DOUBLE is
			--double value -- bad name
		do
			if operator /= void then
				Result := operator.double_value(left,right,a_target)
			end
		end
	get_integer_value (a_target : like expression_target) : INTEGER is
			--integer value --bad name
		do
			if operator /= void then
				Result := operator.integer_value(left,right,a_target)
			end
		end
	boolean_value (a_target : like expression_target) : BOOLEAN is
			--Boolean value
		do
			if operator /= void then
				Result := operator.boolean_value(left,right,a_target)
			end
		end

	string_value (a_target : like expression_target) : STRING is
			-- String representation of expression.
		do
			if operator /= void and left /= void and right /= Void then
				Result := operator.string_value(left,right,a_target)
			end
		end

	decimal_value (a_target : like expression_target) : RP_DECIMAL is
			-- Decimal value
		do
			if operator /= void then
				Result := operator.decimal_value(left,right,a_target)
			end
		end

	generated_value_in_market_currency (a_market: MARKET; a_currency: PERSISTENT_CURRENCY; a_target: like expression_target; a_date: ROSE_DATE; a_agent: like access_agent_template): REAL_64
			-- <Precursor>
		local
			l_left, l_right, l_value : ROSE_UNITIZED_VALUE[DOUBLE]
			l_left_units, l_right_units : ROSE_UNITIZED_VALUE[DOUBLE]
		do
			create l_left_units.make_with_units (0.0, left.computed_units (a_target, a_agent))
			l_left_units := l_left_units.converted_unitized_value (a_market, a_currency, a_date)
			create l_left.make_with_units (left.generated_value_in_market_currency (a_market,a_currency,a_target,a_date,a_agent), l_left_units.units)
			create l_right_units.make_with_units (0.0, right.computed_units (a_target, a_agent))
			l_right_units := l_right_units.converted_unitized_value (a_market, a_currency, a_date)
			create l_right.make_with_units (right.generated_value_in_market_currency (a_market,a_currency,a_target,a_date,a_agent), l_right_units.units)
			l_value := operator.unitized_value (l_left, l_right)
			if l_value /= Void then
				Result := l_value.value
			else
				Result := qnan
			end
		end

	value_in_market_currency (a_market : MARKET;a_currency : PERSISTENT_CURRENCY;a_target : like expression_target;a_date : ROSE_DATE) : DOUBLE is
			-- <Precursor>
		do
			if operator /= void then
				Result := operator.value_in_market_currency(left,right,a_market,a_currency,a_target,a_date)
			end
		end

	unitized_value_in_market_currency (a_market: MARKET; a_currency: PERSISTENT_CURRENCY; a_target: like expression_target; a_date: ROSE_DATE; a_agent: like access_agent_template): ROSE_UNITIZED_VALUE[DOUBLE] is
			-- <Precursor>
		local
			l_left,l_right : ROSE_UNITIZED_VALUE[DOUBLE]
		do
			l_left := left.unitized_value_in_market_currency (a_market,a_currency,a_target,a_date,a_agent)
			l_right := right.unitized_value_in_market_currency (a_market,a_currency,a_target,a_date,a_agent)
			Result := operator.unitized_value (l_left, l_right)
		end
	units (a_target : like expression_target) : ROSE_UNITS is
			-- <Precursor>
		do
			if operator /= void then
				Result := operator.units(left,right,a_target)
			end
		end
	computed_units (a_target: like expression_target; a_agent :like access_agent_template): ROSE_UNITS is
			-- <Precursor>
		do
			if operator /= Void then
				Result := operator.computed_units(left.computed_units(a_target,a_agent),right.computed_units (a_target, a_agent))
			end
		end
	currency (a_target : like expression_target) : PERSISTENT_CURRENCY is
			-- Currency
		do
			Result := left.currency(a_target)
		end

	is_defined_real (a_value : REAL) : BOOLEAN is
		do
			Result := left.is_defined_real(a_value)
		end
	is_defined_double (a_value : DOUBLE) : BOOLEAN is
		do
			Result := left.is_defined_double(a_value)
		end
	is_defined_integer (a_value : INTEGER) : BOOLEAN is
		do
			Result := left.is_defined_integer(a_value)
		end
	is_flow : BOOLEAN
			--is this a transient value?
		do
			Result := left.is_flow or else right.is_flow
		end
	undefined_real : REAL is
		do
			Result := left.undefined_real
		end
	undefined_double : DOUBLE is
		do
			Result := left.undefined_double
		end
	undefined_integer : INTEGER is
		do
			Result := left.undefined_integer
		end
	sub_expressions : ROSE_LINEAR_ARRAY[ROSE_EXPRESSION[like expression_target]] is
		local
			l_expressions : ROSE_LINEAR[ROSE_EXPRESSION[like expression_target]]
		do
			Result := Precursor
			l_expressions := left.sub_expressions
			from l_expressions.start until l_expressions.off loop
				Result.extend(l_expressions.item)
				l_expressions.forth
			end
			l_expressions := right.sub_expressions
			from l_expressions.start until l_expressions.off loop
				Result.extend(l_expressions.item)
				l_expressions.forth
			end
		end
	partial_real_value (a_target : like expression_target) : REAL is
		do
			if operator /= void then
				Result := operator.partial_real_value(left,right,a_target)
			end
		end
	partial_double_value (a_target : like expression_target) : DOUBLE is
		do
			if operator /= void then
				Result := operator.partial_double_value(left,right,a_target)
			end
		end
	partial_integer_value (a_target : like expression_target) : INTEGER is
		do
			if operator /= void then
				Result := operator.partial_integer_value(left,right,a_target)
			end
		end
	partial_value_in_market_currency (a_market : MARKET;a_currency : PERSISTENT_CURRENCY;a_target : G;a_date : ROSE_DATE) : DOUBLE is
			-- Partial value in market currency
		do
			if operator /= void then
				Result := operator.partial_value_in_market_currency(left,right,a_market,a_currency,a_target,a_date)
			end
		end
feature --Computed values
	computed_double(a_target : like expression_target; a_agent : like access_agent_template) : DOUBLE is
			-- <Precursor>
		do
			if operator /= Void then
				Result := operator.computed_double(left, right, a_target, a_agent)
			end
		end

	computed_real(a_target : like expression_target; a_agent : like access_agent_template) : REAL is
			-- <Precursor>
		do
			if operator /= Void then
				Result := operator.computed_real(left, right, a_target, a_agent)
			end
		end

	computed_integer(a_target : like expression_target; a_agent : like access_agent_template) : INTEGER is
			-- <Precursor>
		do
			if operator /= Void then
				Result := operator.computed_integer(left, right, a_target, a_agent)
			end
		end

	computed_string(a_target : like expression_target; a_agent : like access_agent_template) : STRING is
			-- <Precursor>
		do
			if operator /= Void then
				Result := operator.computed_string(left, right, a_target, a_agent)
			end
		end

feature {NONE} -- Generate

	internal_generated_double (a_target : like expression_target; a_agent : like access_agent_template) : DOUBLE is
			-- <Precursor>
		do
			if operator /= Void then
				Result := operator.generated_double(left, right, a_target, a_agent)
			end
		end

	internal_generated_real (a_target : like expression_target; a_agent : like access_agent_template) : REAL is
			-- <Precursor>
		do
			if operator /= Void then
				Result := operator.generated_real(left, right, a_target, a_agent)
			end
		end

	internal_generated_integer (a_target : like expression_target; a_agent : like access_agent_template) : INTEGER is
			-- <Precursor>
		do
			if operator /= Void then
				Result := operator.generated_integer(left, right, a_target, a_agent)
			end
		end

	internal_generated_string (a_target : like expression_target; a_agent : like access_agent_template) : STRING is
			-- <Precursor>
		do
			if operator /= Void then
				Result := operator.generated_string(left, right, a_target, a_agent)
			end
		end

	internal_generated_date (a_target : like expression_target; a_agent : like access_agent_template) : ROSE_DATE is
			-- <Precursor>
		do
			if operator /= Void then
				Result := operator.generated_date(left, right, a_target, a_agent)
			end
		end

feature -- Generated values

	generated_unitized_value (a_target: like expression_target; a_agent: like access_agent_template): DATED_UNITIZED_VALUE[DOUBLE] is
		do
			if a_agent = void then
				if operator /= Void then
					Result := operator.generated_unitized_value(left, right, a_target, a_agent)
				end
			else
				Result := a_agent.unitized_double(entity_date_target(a_target),Current)
				if Result = Void then
					Result := operator.generated_unitized_value(left, right, a_target, a_agent)
				end
			end
		end

feature --Subexpressions
	component_expressions (a_target: ROSE_EXPRESSION_TARGET): ROSE_LINEAR_ARRAY [ROSE_EXPRESSION [ROSE_EXPRESSION_TARGET]] is
			-- Expressions which contribute to value of `Current'
		local
			l_count: INTEGER
			l_exp_1, l_exp_2: ROSE_EXPRESSION [ROSE_EXPRESSION_TARGET]
		do
			l_exp_1 ?= left
			if l_exp_1 /= Void then
				l_count := 1
			end
			l_exp_2 ?= right
			if l_exp_2 /= Void then
				l_count := l_count + 1
			end
			create Result.make (1, l_count)
			if l_exp_1 /= Void then
				Result.put (l_exp_1, 1)
			end
			if l_exp_2 /= Void then
				Result.put (l_exp_2, l_count)
			end
		end

	nominal_units: ROSE_UNITS is
			-- <Precursor>
		do
			if left /= Void and right /= Void and then
				left.nominal_units /= Void and then right.nominal_units /= Void then
				Result := operator.nominal_units (left.nominal_units, right.nominal_units)
			end
			if Result = Void then
				Result := pure_units
			end
		end

end
