note

	description:

		"Binary functional on ROSE_EXPRESSION"

	author: "Mark Howard"
	last_editor: "$Author: mk $"
	date: "$Date: 2011/02/15 $"
	revision: "$Revision: #32 $"
	path: "$File: //rose/source/kernel/math/rose_binary_operator.e $"

deferred class
	ROSE_BINARY_OPERATOR[reference G]
inherit
	ROSE_OPERATOR
feature -- Access

	new_binary_expression (a_left, a_right: ROSE_EXPRESSION [G]): ROSE_EXPRESSION [G] is
			-- Return a new binary expression
		do
			Result := create {ROSE_BINARY_EXPRESSION [G]}.make (a_left, a_right, Current)
		end

	real_value (a_left,a_right : ROSE_EXPRESSION[G];a_g : G) : REAL is
			-- Real value
		deferred
		end
	double_value (a_left,a_right : ROSE_EXPRESSION[G];a_g : G) : DOUBLE is
			--Double value result of applying this operator to a_left and a_right
		deferred
		end
	decimal_value (a_left,a_right : ROSE_EXPRESSION[G];a_g : G) : RP_DECIMAL is
			--returned decimal value
		deferred
		end
	integer_value (a_left,a_right : ROSE_EXPRESSION[G];a_g : G) : INTEGER is
			--returned integer value
		deferred
		end
	boolean_value (a_left,a_right : ROSE_EXPRESSION[G];a_g : G) : BOOLEAN is
			-- Boolean value
		deferred
		end

	string_value (a_left,a_right : ROSE_EXPRESSION[G];a_g : G) : STRING is
			-- String value
		deferred
		end

	value_in_market_currency (a_left,a_right : ROSE_EXPRESSION[G];a_market : MARKET;a_currency : PERSISTENT_CURRENCY;a_entity : G;a_date : ROSE_DATE) : DOUBLE is
		deferred
		end
	units (a_left,a_right : ROSE_EXPRESSION[G];a_entity : G) : ROSE_UNITS is
			--units
		do
			Result := computed_units(a_left.units(a_entity),a_right.units(a_entity))
		end
	nominal_units (a_left,a_right : ROSE_UNITS) : ROSE_UNITS is
		deferred
		end
	partial_real_value (a_left,a_right : ROSE_EXPRESSION[G];a_g : G) : REAL is
		do
			Result := real_value(a_left,a_right,a_g)
		end
	partial_double_value (a_left,a_right : ROSE_EXPRESSION[G];a_g : G) : DOUBLE is
		do
			Result := double_value(a_left,a_right,a_g)
		end
	partial_integer_value (a_left,a_right : ROSE_EXPRESSION[G];a_g : G) : INTEGER is
		do
			Result := integer_value(a_left,a_right,a_g)
		end

	partial_value_in_market_currency (a_left,a_right : ROSE_EXPRESSION[G];a_market : MARKET;a_currency : PERSISTENT_CURRENCY;a_g : G;a_date : ROSE_DATE) : DOUBLE is
			--return 0 if undefined
		do
			Result := value_in_market_currency(a_left,a_right,a_market,a_currency,a_g,a_date)
		end

	units_in_currency (a_left,a_right : ROSE_EXPRESSION[G];a_currency : PERSISTENT_CURRENCY;a_entity : G) : ROSE_UNITS is
		deferred
		end

	unitized_value(a_left,a_right : ROSE_UNITIZED_VALUE[DOUBLE]) : ROSE_UNITIZED_VALUE[DOUBLE]
		deferred
		end

feature  --Computed values
	computed_double(a_left,a_right : ROSE_EXPRESSION[G]; a_g : G; a_agent : like access_agent_template) : DOUBLE is
			-- Computed double
		deferred
		end
	computed_real(a_left,a_right : ROSE_EXPRESSION[G]; a_g : G; a_agent : like access_agent_template) : REAL is
		deferred
		end
	computed_integer(a_left,a_right : ROSE_EXPRESSION[G]; a_g : G; a_agent : like access_agent_template) : INTEGER is
		deferred
		end
	computed_string(a_left,a_right : ROSE_EXPRESSION[G]; a_g : G; a_agent : like access_agent_template) : STRING is
		deferred
		end
	computed_boolean(a_left,a_right : ROSE_EXPRESSION[G]; a_g : G; a_agent : like access_agent_template) : BOOLEAN is
			-- Computed boolean
		deferred
		end
	computed_units(a_left,a_right : ROSE_UNITS) : ROSE_UNITS is
		deferred
		end

feature  --Generated values
	generated_unitized_value(a_left,a_right : ROSE_EXPRESSION[G]; a_g : G; a_agent : like access_agent_template) : DATED_UNITIZED_VALUE[DOUBLE] is
		do
		end
	generated_double(a_left,a_right : ROSE_EXPRESSION[G]; a_g : G; a_agent : like access_agent_template) : DOUBLE is
		deferred
		end
	generated_real(a_left,a_right : ROSE_EXPRESSION[G]; a_g : G; a_agent : like access_agent_template) : REAL is
		deferred
		end
	generated_integer(a_left,a_right : ROSE_EXPRESSION[G]; a_g : G; a_agent : like access_agent_template) : INTEGER is
		deferred
		end
	generated_string(a_left,a_right : ROSE_EXPRESSION[G]; a_g : G; a_agent : like access_agent_template) : STRING is
		deferred
		end
	generated_date(a_left,a_right : ROSE_EXPRESSION[G]; a_g : G; a_agent : like access_agent_template) : ROSE_DATE is
		deferred
		end
feature  --Properties
	commutative : BOOLEAN is
			-- Is it Commutative?
		do
		end
	set_inverse (a_operator : ROSE_BINARY_OPERATOR[G]) is
		do
			inverse := a_operator
		end
	inverse : ROSE_BINARY_OPERATOR[G]
feature  --Templates
	access_agent_template : GENERIC_ACCESS_CONTROL_AGENT is
			--template
		do  end
end
