indexing
	description: "[
				  To cache objects of type G.
				  ]"

	author: "Mark Howard"
	last_editor: "$Author: e2 $"
	date: "$Date: 2010/11/23 $"
	revision: "$Revision: #9 $"
	path: "$File: //rose/source/kernel/entities/rose_cache.e $"

class ROSE_CACHE[G]

create
	make

feature {NONE} -- Initialization

	make (a_maximum_count, a_maximum_chunk_size : INTEGER) is
		do
			maximum_chunk_size := a_maximum_chunk_size
			chunk_count := 1
			maximum_count := a_maximum_count
			create expiration_table.make
			expiration_table.extend(1)
			create data.make(maximum_count)
		end

feature -- Access

	put (a_item: like item; a_key: TUPLE) is
			-- Put `a_item' at `a_key' in the cache
		local
			l_volatile : ROSE_VOLATILE[like item]
		do
			data.search(a_key)
			if data.found then
				l_volatile := data.found_item
			else
				create l_volatile
				data.put(l_volatile,a_key)
				if not locked then
					increment_chunk_size
				end
			end
			l_volatile.set(a_item,chunk_count)
		end
	force (a_item : like item;a_key : TUPLE) is
		local
			l_volatile : ROSE_VOLATILE[like item]
		do
			data.search(a_key)
			if data.found then
				l_volatile := data.found_item
			else
				create l_volatile
				data.force(l_volatile,a_key)
				increment_chunk_size
			end
			l_volatile.set(a_item,chunk_count)
		end
	extend(a_items : LINEAR[like item];a_keys : LINEAR[TUPLE]) is
		local
			a1, a2: ANY
			l_different: BOOLEAN
		do
			a1 := a_items
			a2 := a_keys
			l_different := a1 /= a2
			from
				a_items.start
				a_keys.start
			until
				a_items.off or a_keys.off
			loop
				put (a_items.item,a_keys.item)
				a_items.forth
				if l_different then
					a_keys.forth
				end
			end
		end
	expiration_table : ROSE_HASHED_QUEUE[INTEGER]
	data : ROSE_MULTI_KEYED_TABLE[ROSE_VOLATILE[like item]]
	increment_chunk_size is
		do
			chunk_size := chunk_size + 1
			if chunk_size >= maximum_chunk_size then
				chunk_size := 0
				chunk_count := chunk_count + 1
				expiration_table.extend(chunk_count)
			end
			from until data.count <= maximum_count loop
				sweep
			end
		end
	found : BOOLEAN
	found_item : G
	search (a_key : TUPLE) is
		local
			l_volatile : ROSE_VOLATILE[like item]
      do
	 	data.search(a_key)
	 	found := data.found
	 	if found then
			l_volatile := data.found_item
			if l_volatile /= Void then
				expiration_table.renew(l_volatile.expiration)
				found_item := l_volatile.item
			end
		     end
		end
	item (a_key : TUPLE) : G is
		local
			l_volatile : ROSE_VOLATILE[like item]
      do
	 data.search(a_key)
	 if data.found then
			l_volatile := data.found_item
			if l_volatile /= Void then
				expiration_table.renew(l_volatile.expiration)
				Result := l_volatile.item
			end
		     end
		end
	sweep is
		local
			l_expired : INTEGER
			l_keys : ROSE_LINEAR_ARRAY[TUPLE]
		do
			l_expired := expiration_table.oldest
			create l_keys.make_empty
			from data.start until data.off loop
				if data.item_for_iteration.expiration =  l_expired then
					l_keys.extend(data.key_for_iteration)
				end
				data.forth
			end
			from l_keys.start until l_keys.off loop
				data.remove(l_keys.current_item)
				l_keys.forth
			end
			expiration_table.pop
		end
	neighborhood (a_key : TUPLE) : ROSE_LINEAR_ARRAY[like item] is
		local
			l_expiration : INTEGER
			l_volatile : ROSE_VOLATILE[like item]
		do
			create Result.make_empty
			l_volatile := data.item(a_key)
			if l_volatile /= Void then
				l_expiration := l_volatile.expiration
				from data.start until data.off loop
					if data.item_for_iteration.expiration = l_expiration then
						Result.extend(data.item_for_iteration.item)
					end
					data.forth
				end
			end
		end
	maximum_chunk_size : INTEGER
	chunk_count : INTEGER
	chunk_size : INTEGER
	maximum_count : INTEGER
	locked : BOOLEAN --all items entered while locked get the same expiration
	lock_ is
		do
			locked := true
			increment_chunk_size
		end
	unlock is
		do
			locked := false
		end
end -- class ROSE_CACHE
