note

	description:

		"Signature for overrides of rose capital actions"

	author: "Srilatha Singh"
	last_editor: "$Author: mb $"
	date: "$Date: 2011/02/28 $"
	revision: "$Revision: #17 $"
	path: "$File: //rose/source/kernel/capital_actions/rose_capact_override_signature.e $"

class ROSE_CAPACT_OVERRIDE_SIGNATURE

inherit

	ROSE_SIGNATURE_RECORD [ENTITY]
		redefine
			data_manager,
			date_name,
			date,
--			reconcile_target,
			data_fields_string_,
			debug_output
		select
			is_equal
		end

	OVERRIDE_AUDIT_FIELDS
		rename
			user_id as user_name,
			set_user_id as set_user_name,
			is_equal as overrd_is_equal
		end

create

	default_create,
	make_from_signature

feature -- Creation

	make_from_signature (a_signature: CAPACT_SIGNATURE_RECORD) is
			-- make from overridden capital action signature
		do
			listing_id := a_signature.listing_id
			instrument_id := a_signature.instrument_id
			market_id := a_signature.market_id
			exdate := a_signature.exdate
			member_number := a_signature.member_number
				--data fields
			capact_type_id := a_signature.capact_type_id
			record_date := a_signature.record_date
			company_id := a_signature.company_id
			status_code := a_signature.status_code
			set_entry_date (create {ROSE_DATE}.make_now)
			set_end_valid_date (constants.maximum_date_time)
			set_user_name (global.user.full_login)
		end

feature -- Access
	-- signature key fields

	listing_id: INTEGER

	set_listing_id (a_value: like listing_id) is
			-- set the listing id
		do
			listing_id := a_value
		end

	exdate: INTEGER

	set_exdate (a_value: like exdate) is
			-- set the exdate
		do
			exdate := a_value
		end

	member_number: INTEGER

	set_member_number (a_value: like member_number) is
			-- Set 'member_number' to 'a_value'
		do
			member_number := a_value
		end

	capact_type_id: INTEGER
			-- signature data fields

	set_capact_type_id (a_value: like capact_type_id) is
		do
			capact_type_id := a_value
		end

	record_date: INTEGER

	set_record_date (a_value: like record_date) is
		do
			record_date := a_value
		end

	company_id: INTEGER

	set_company_id (a_value: like company_id) is
		do
			company_id := a_value
		end

	instrument_id: INTEGER

	set_instrument_id (a_value: like instrument_id) is
		do
			instrument_id := a_value
		end

	market_id: INTEGER

	set_market_id (a_value: like market_id) is
		do
			market_id := a_value
		end

	status_code: STRING

	set_status_code (a_value: like status_code) is
		do
			status_code := a_value
		end

	entity_id: INTEGER

	set_entity_id (a_value: like entity_id) is
		do
			entity_id := a_value
		end

	context_name: STRING is "ROSE"

	expression_tag: STRING is "capact_signature_record"

	date_name: STRING is
		once
			Result := "end_current_date"
		end

	date: ROSE_DATE is
		do
			Result := end_current_date
		end

feature -- Access

	signatures_by_company_id (a_company_id: INTEGER): like targets is
		local
			l_target: like Current
			l_targets: like targets
		do
			create l_target
			create l_targets.make_empty
			l_target.set_company_id (a_company_id)
			l_targets.extend (l_target)
			Result := targets (l_targets, restriction_statement)
		end

	signatures_by_instrument_market (a_instrument_id: INTEGER; a_market_id: INTEGER): like targets is
		local
			l_target: like Current
			l_targets: like targets
		do
			create l_target
			create l_targets.make_empty
			l_target.set_instrument_id (a_instrument_id)
			l_target.set_market_id (a_market_id)
			l_targets.extend (l_target)
			Result := targets (l_targets, instrument_market_restriction_statement)
		end

	signatures_by_instrument_market_exdate (a_instrument_id: INTEGER; a_market_id: INTEGER; a_exdate: INTEGER): like targets is
		local
			l_targets: like targets
		do
			l_targets := signatures_by_instrument_market (a_instrument_id, a_market_id)
			if l_targets /= Void then
				create Result.make_empty
				from
					l_targets.start
				until
					l_targets.after
				loop
					if l_targets.current_item.exdate = a_exdate then
						Result.extend (l_targets.current_item)
					end
					l_targets.forth
				end
				if Result.count < 1 then
					Result := Void
				end
			end
		end

	signatures_by_instrument_market_after (a_instrument_id: INTEGER; a_market_id: INTEGER; a_exdate: INTEGER): like targets is
		local
			l_targets: like targets
		do
			l_targets := signatures_by_instrument_market (a_instrument_id, a_market_id)
			if l_targets /= Void then
				create Result.make_empty
				from
					l_targets.start
				until
					l_targets.after
				loop
					if l_targets.current_item.exdate >= a_exdate then
						Result.extend (l_targets.current_item)
					end
					l_targets.forth
				end
				if Result.count < 1 then
					Result := Void
				end
			end
		end

feature -- Implementation

	data_manager: SIGNATURE_TAGGED_ITEM_DATABASE [ROSE_CAPACT_OVERRIDE_SIGNATURE] is
		once
			create Result.make_named ("capact_override", "browse", servers.rose_override_server)
		end

	signature_statement: STRING is "#listing_id.id = listing_id and #exdate.id = exdate and #member_number.id = member_number"

	instrument_market_restriction_statement: STRING is "#instrument_id.id = instrument_id and #market_id.id = market_id"

	listing_restriction_statement: STRING is "#listing_id.id = listing_id"

	option_restriction_statement: STRING is "#listing_id.id = listing_id and #exdate.id = exdate"

	restriction_statement: STRING is "#company_id.id = company_id"

	statement_: STRING is "select * from capact_override_data_signatures"

	fields_: STRING is "listing_id,exdate,member_number"

	data_fields_string_: STRING is "company_id,market_id,instrument_id,capact_type_id,record_date,status_code,user_name,comment,entry_date,end_valid_date"

	key_tuple: TUPLE is
		do
			Result := [listing_id, exdate, member_number]
		end

	debug_output: STRING is
			-- <Precursor>
		do
			Result := listing_id.out + " " + " :" + exdate.out + ":" + member_number.out + " :" + capact_type_id.out + " :" + status_code
		end

end
