indexing
	description: "Category access"
	path: "$File: //rose/source/kernel/investment_entities/rose_category_access.e $"
	date: "$Date: 2014/02/26 $"
	revision: "$Revision: #8 $"

class
	ROSE_CATEGORY_ACCESS

inherit
	ANY
	DATABASE_SERVER
		export
			{NONE} all
		end

feature -- Access

	category_id (a_name: STRING): INTEGER is
		require
			valid_name: a_name /= Void
		local
			l_name: STRING
		do
			l_name := a_name.twin
			l_name.to_upper
			Result := category_id_table.item (l_name)
		end

	category_name (a_category_id: INTEGER): STRING is
			-- Name of category with id 'a_category_id'
		do
			Result := category_names.item (a_category_id)
		end

feature {NONE} -- Implementation

	category_id_table: HASH_TABLE [INTEGER, STRING] is
		local
			l_reader: SIMPLE_PREDICATE [CATEGORY_ID_RECORD]
		once
			create Result.make (50)
			create l_reader.make (servers.rose_entity_foreground_server, "select category_id, name from category")
			if not l_reader.success then
				(create {EXCEPTIONS}).raise ("ROSE_CATEGORY_ID_USE:category_ids failed to make l_reader.")
			end
			l_reader.apply (agent add_category_id (Result, ?), Void, Void)
			if not l_reader.success then
				(create {EXCEPTIONS}).raise ("ROSE_CATEGORY_ID_USE:category_ids - l_reader predicate failed.")
			end
		end

	category_names: HASH_TABLE [STRING, INTEGER] is
		once
			create Result.make (50)
			category_id_table.do_nothing
		end

	add_category_id (a_hash_table: HASH_TABLE [INTEGER, STRING]; a_record: CATEGORY_ID_RECORD) is
		do
			a_record.name.to_upper
			a_record.name.prune_all_trailing (' ')
			a_hash_table.put (a_record.category_id, a_record.name)
			category_names.put (a_record.name, a_record.category_id)
		end

end
