indexing
	description: "Communication channel"
	author: "Zoran Simic"
	date: "$Date: 2008/12/01 $"
	revision: "$Revision: #7 $"

deferred class
	ROSE_CHANNEL

inherit
	ROSE_SERVICE
		redefine
			is_equal, out
		end
	ROSE_IO_CONVENTION
		undefine
			is_equal, out
		end

feature -- Access

	parameters: STRING
			-- Parameters of invocation

feature -- Status report

	is_interactive: BOOLEAN
			-- Is this channel considered to be interactive with user?

	is_equal (a_other: like Current): BOOLEAN is
			-- Is current output channel equivalent to 'a_other'
		do
			Result := equal (out,a_other.out)
		end

feature -- Setting

	set_interactive (a_value: BOOLEAN) is
			-- Set 'is_interactive'.
		do
			is_interactive := a_value
		end

	tie (a_object: ANY) is
			-- Tie this service to 'a_object' (this is service specific).
		do
			--| Not supported for channels
		ensure then
			dont_call: False
		end

feature -- Basic operations

	open (a_path: STRING) is
			-- Open this channel with 'a_path'.
		do
			parameters := a_path
			do_open
		end

	close is
			-- Close this channel.
		do
			do_close
			parameters := Void
		end

feature -- Output

	out, representation: STRING is
			-- String representation of this channel
		do
			create Result.make_empty
			Result.append_string (id)
			if parameters /= Void then
				Result.extend (':')
				Result.append (parameters)
			end
		end

feature {NONE} -- Implementation

	do_open is
			-- Open effectively this channel with 'parameters'.
		deferred
		end

	do_close is
			-- Close effectively this channel.
		deferred
		end

end
