indexing
	description: "ROSE_CHART_PLOT capable of handling ROSE_CHART_2D_POINTs, with auxiliary functions"
	author: "Urs Doenni"
	date: "$Date: 2008/12/01 $"
	revision: "$Revision: #6 $"
	archive: "$File: //rose/source/kernel/infrastructure/widgets/charts/rose_chart_2d_plot.e $"

class
	ROSE_CHART_2D_PLOT
inherit
	ROSE_CHART_PLOT[ROSE_CHART_2D_POINT]

create
	make_empty
create {ROSE_CHART_2D_PLOT}
	make_chart_list,
	make_sublist


feature {ROSE_CHART_COORDINATE_SYSTEM} -- get new plot list instance
	get_new_plot_list_instance: ROSE_CHART_2D_PLOT_LIST is
			-- new instance of plot list that belongs to this plot
		do
			create Result.make_empty
		ensure then
			result_not_void : Result /= void
		end


feature -- Basic operations
	add_points (a_x, a_y: ARRAY [DOUBLE]; a_x_labels, a_y_labels: ARRAY[ANY]; a_keys: ARRAY[HASHABLE]) is
			-- Add points from 'a_x' and 'a_y' coordinates to this plot
			-- TODO: handle the case that different arrays have different 'lower'
		local
			l_x: DOUBLE
			l_key: HASHABLE
			l_x_label, l_y_label: ANY
			i, n: INTEGER
			l_point : ROSE_CHART_2D_POINT
		do
			if a_y /= Void then
				reindex_double_array (a_x)
				reindex_double_array (a_y)
				reindex_array (a_x_labels)
				reindex_array (a_y_labels)
				reindex_array (a_keys)
				from
					i := a_y.lower
					n := a_y.upper
				until
					i > n
				loop
					if a_x /= Void and then a_x.valid_index (i) then
						l_x := a_x.item (i)
					else
						l_x := i
					end
					if a_x_labels /= Void and then a_x_labels.valid_index (i) then
						l_x_label := a_x_labels.item (i)
					end
					if a_y_labels /= Void and then a_y_labels.valid_index (i) then
						l_y_label := a_y_labels.item (i)
					end
					if a_keys /= Void and then a_keys.valid_index (i) then
						l_key := a_keys.item (i)
					else
						-- TODO: autogenerate keys? Or let that {ROSE_CHART_POINT}.make do?
						l_key := i
					end

					create l_point.make_with_labels ([l_x, a_y.item (i)], [l_x_label, l_y_label], l_key)
					extend_back (l_point)
					i := i + 1
				end
			end
		end

	add_dated_points (a_dates: ARRAY[ROSE_DATE]; a_y: ARRAY [DOUBLE]; a_y_labels: ARRAY[ANY]) is
			-- add points with x-values and x-labels generated from 'a_dates'
			-- TODO: this fails when there are fewer dates than y-values. Should this be caught by a precondition?
		local
			l_x_array: ARRAY[DOUBLE]
			i: INTEGER
		do
			reindex_array (a_dates)
			create l_x_array.make (a_dates.lower, a_dates.upper)
			from i := a_dates.lower
			until i > a_dates.upper
			loop
				l_x_array.put (a_dates.item(i).to_double, i)
--				l_x_array.put (i * 10, i)
				i := i + 1
			end
			add_points (l_x_array, a_y, a_dates, a_y_labels, a_dates)
		end

	set_data_to_points(a_data : ARRAY[ANY]) is
			-- set data to point
		local
			i : INTEGER
		do
			if a_data /= Void then
				from start i := a_data.lower until after or i > a_data.upper loop
					item.set_data (a_data[i])
					i := i + 1
					forth
				end
			end
		end

	set_tooltip_agent_to_points(a_tooltip_function : FUNCTION[ANY, TUPLE [ROSE_CHART_2D_POINT], STRING]) is
			-- set tooltip agent to ponints
		do
			from start until after loop
				item.set_tooltip_agent (a_tooltip_function)
				forth
			end
		end

	x_values: ROSE_LINEAR_ARRAY[DOUBLE] is
			-- Array with all x-values from the plot points
		do
			Result := values (1)
		end

	y_values: ROSE_LINEAR_ARRAY[DOUBLE] is
			-- Array with all y-values from the plot points
		do
			Result := values (2)
		end

	x_labels: ROSE_LINEAR_ARRAY[ANY] is
			-- Array with all x-labels from the plot points
		do
			Result := labels (1)
		end

	y_labels: ROSE_LINEAR_ARRAY[ANY] is
			-- Array with all y-labels from the plot points
		do
			Result := labels (2)
		end


feature {NONE} -- Implementation
	reindex_array (a_array: ARRAY [ANY]) is
			-- TODO: reindexing the array changes it. One should not change arguments that are passed from outside (make a copy)
		do
			if a_array /= Void then
				a_array.conservative_resize (1, a_array.count)
			end
		end

	reindex_double_array (a_array: ARRAY [DOUBLE]) is
			-- Use reindex_array instead
		obsolete
			"Use reindex_array instead"
		do
			if a_array /= Void then
				a_array.conservative_resize (1, a_array.count)
			end
		end
end
