indexing

	description: "ROSE_CHART_PLOT_LIST capable of handling ROSE_CHART_2D_POINTs with additional useful helper functions for ROSE_CHART_2D_POINTs"
	author: "Urs Doenni"
	last_editor: "$Author: mb $"
	date: "$Date: 2010/03/12 $"
	revision: "$Revision: #5 $"
	archive: "$File: //rose/source/kernel/infrastructure/widgets/charts/rose_chart_2d_plot_list.e $"

class ROSE_CHART_2D_PLOT_LIST

inherit

	ROSE_CHART_PLOT_LIST [ROSE_CHART_2D_PLOT]

create

	make_empty,
	make_with_plots

create {ROSE_CHART_2D_PLOT_LIST}

	make

feature -- Basic operations

	add_dated_plots_from_matrix (a_matrix: ABSTRACT_READABLE_MATRIX; a_dates: ARRAY [ROSE_DATE]; a_y_labels: ARRAY [ARRAY [ANY]]) is
			-- Add charts found in 'a_matrix', each column is taken as a series
		local
			i: INTEGER
			l_name: STRING
			l_plot: ROSE_CHART_2D_PLOT
			l_y_labels: ARRAY [ANY]
		do
			if a_matrix /= Void then
				from
					i := 1
				until
					i > a_matrix.width
				loop
					if a_matrix.column_names /= Void and then a_matrix.column_names.valid_index (i) then
						l_name := a_matrix.column_names.item (i)
					else
						l_name := Void
					end
					if l_name = Void or else l_name.is_empty then
						l_name := "Column " + i.out
					end
					create l_plot.make_empty (l_name)
					if a_y_labels /= Void then
						l_y_labels := a_y_labels.item (i)
					end
					l_plot.add_dated_points (a_dates, a_matrix.column (i).to_array, l_y_labels)
					add_plot (l_plot)
					i := i + 1
				end
			end
		end

	add_plots_from_matrix (a_matrix: ABSTRACT_READABLE_MATRIX) is
			-- Add charts found in 'a_matrix', each column is taken as a series
			-- 'column_names' provide each series name.
		local
			i, j: INTEGER
			l_name: STRING
			l_plot: ROSE_CHART_2D_PLOT
			l_point: ROSE_CHART_2D_POINT
		do
			if a_matrix /= Void then
				from
					i := 1
				until
					i > a_matrix.width
				loop
					if a_matrix.column_names /= Void and then a_matrix.column_names.valid_index (i) then
						l_name := a_matrix.column_names.item (i)
					else
						l_name := Void
					end
					if l_name = Void or else l_name.is_empty then
						l_name := "Column " + i.out
					end
					create l_plot.make_empty (l_name)
					from
						j := 1
					until
						j > a_matrix.height
					loop
						create l_point.make ([j.to_double, a_matrix.item (j, i)], j)
						l_plot.extend_back (l_point)
						j := j + 1
					end
					add_plot (l_plot)
					i := i + 1
				end
			end
		end

	add_plots_from_dates (a_names: ARRAY [STRING]; a_dates: ARRAY [ROSE_DATE]; a_y_values: ARRAY [ARRAY [DOUBLE]]; a_y_labels: ARRAY [ARRAY [ANY]]) is
			-- Add charts whose 'x' coordinates are taken from 'a_dates'.
		require
			a_names_not_void: a_names /= Void
		local
			i: INTEGER
			l_plot: ROSE_CHART_2D_PLOT
			l_y_labels: ARRAY [ANY]
		do
			if a_y_values /= Void then
				from
					i := a_names.lower
				until
					i > a_names.upper
				loop
					create l_plot.make_empty (a_names.item (i))
					if a_y_labels /= Void then
						l_y_labels := a_y_labels.item (i)
					end
					l_plot.add_dated_points (a_dates, a_y_values.item (i), l_y_labels)
					add_plot (l_plot)
					i := i + 1
				end
			end
		end

	add_plots_from_arrays (a_names: ARRAY [STRING]; a_x: ARRAY [DOUBLE]; a_y: ARRAY [ARRAY [DOUBLE]]; a_x_labels: ARRAY [ANY]; a_y_labels: ARRAY [ARRAY [ANY]]; a_keys: ARRAY [HASHABLE]) is
			-- Add charts with 'a_x' and 'a_y' coordinates, each array in 'a_y' gives one plot with same 'a_x'
			-- all plots are added to this system, using the current 'renderer'
			-- 'a_names' will be assigned to corresopnding charts if provided.
		local
			i: INTEGER
			l_name: STRING
			l_plot: ROSE_CHART_2D_PLOT
			l_y_labels: ARRAY [ANY]
		do
			if a_y /= Void then
				from
					i := a_y.lower
				until
					i > a_y.upper
				loop
					if a_names /= Void and then a_names.valid_index (i - a_names.lower + 1) then
						l_name := a_names.item (i - a_names.lower + 1)
					else
						l_name := "Series " + i.out
					end
					create l_plot.make_empty (l_name)
					if a_y_labels /= Void then
						l_y_labels := a_y_labels.item (i)
					end
					l_plot.add_points (a_x, a_y.item (i), a_x_labels, l_y_labels, a_keys)
					add_plot (l_plot)
					i := i + 1
				end
			end
		end

	add_plot_from_arrays (a_name: STRING; a_x, a_y: ARRAY [DOUBLE]; a_x_labels, a_y_labels: ARRAY [ANY]; a_keys: ARRAY [HASHABLE]) is
			-- Add one plot with given values to the plot list
		local
			l_plot: ROSE_CHART_2D_PLOT
		do
			if a_y /= Void and then a_y.count > 0 then
				create l_plot.make_empty (a_name)
				l_plot.add_points (a_x, a_y, a_x_labels, a_y_labels, a_keys)
				add_plot (l_plot)
			end
		end

end
