indexing
	
	description: "3D Point for ROSE_CHART_*"

	author: "Colin Adams"
	last_editor: "$Author: c4 $"
	date: "$Date: 2013/05/15 $"
	revision: "$Revision: #1 $"
	path: "$File: //rose/source/kernel/infrastructure/widgets/charts/rose_chart_3d_point.e $"
	
class	ROSE_CHART_3D_POINT
	
inherit

	ROSE_CHART_POINT
		redefine
			values,
			labels,
			get_data_array,
			get_data_description
		end

	IEEE_MATH
		export {NONE} all end

create
	
	make,	make_with_labels
	
feature -- Access
	
	values: TUPLE [x: like x; y: like y; z: like z]
			-- Position values
	
	labels: TUPLE [x: ANY; y: ANY; z: ANY]
			-- Labels for x, y and z components
	
	x: DOUBLE is
			-- X (usually horizontal) value
		do
			Result := values.x
		end
	
	y: DOUBLE is
			-- Y (usually vertical) value
		do
			Result := values.y
		end
		
	z: DOUBLE is
			-- Z (usually depth) value
		do
			Result := values.z
		end
	
feature -- Element change
	
	set_x_y_z (a_x: like x; a_y: like y; a_z: like z) is
			-- Set `x', `y', and `z' simultaneously.
		do
			values.x := a_x
			values.y := a_y
			values.z := a_z
			
			is_set := True
			
			change_actions.call ([Current])
		ensure
			x_set: ieee_is_eq (x, a_x)
			y_set: ieee_is_eq (y, a_y)
			z_set: ieee_is_eq (z, a_z)
		end
	
	set_x (a_x: like x) is
			-- Set `x' only. `y' and `z' remain unchanged.
		do
			values.x := a_x
			
			is_set := True

			change_actions.call ([Current])
		ensure
			x_set: ieee_is_eq (x, a_x)
			y_unchanged: ieee_is_eq (y, old y)
			z_unchanged: ieee_is_eq (z, old z)
		end
	
	set_y (a_y: like y) is
			-- Set `y' only. `x' and `z' remain unchanged.
		do
			values.y := a_y
			
			is_set := True

			change_actions.call ([Current])
		ensure
			x_unchanged: ieee_is_eq (x, old x)
			y_set: ieee_is_eq (y, a_y)
			z_unchanged: ieee_is_eq (z, old z)
		end
	
	set_z (a_z: like z) is
			-- Set `z' only. `x' and `y' remain unchanged.
		do
			values.z := a_z
			
			is_set := True

			change_actions.call ([Current])
		ensure
			x_unchanged: ieee_is_eq (x, old x)
			y_unchanged: ieee_is_eq (y, old y)
			z_set: ieee_is_eq (z, a_z)
		end

	set_x_label (a_x: ANY) is
			-- Set label for `x' to `a_x'.
		do
			labels.x := a_x
		ensure
			x_label_set: (a_x = a_x) implies (labels.x = a_x)
		end
	
	set_y_label (a_y: ANY) is
			-- Set label for `y' to `a_y'.
		do
			labels.y := a_y
		ensure
			y_label_set: (a_y = a_y) implies (labels.y = a_y)
		end
		
	set_z_label (a_z: ANY) is
			-- Set label for `z' to `a_z'.
		do
			labels.z := a_z
		ensure
			z_label_set: (a_z = a_z) implies (labels.z = a_z)
		end
	
	get_data_array: ARRAY [STRING] is
			-- <Yrecursor>
		local
			l_x, l_y, l_z, l_x_label, l_y_label, l_z_label: STRING
		do
			create Result.make (1, 6)
			if labels /= Void and then labels.x /= Void then
				l_x_label := labels.x.out
			else
				l_x_label := ""
			end
			if labels /= Void and then labels.y /= Void then
				l_y_label := labels.y.out
			else
				l_y_label := ""
			end
			if labels /= Void and then labels.z /= Void then
				l_z_label := labels.z.out
			else
				l_z_label := ""
			end
			if x /= Void then
				l_x := x.out
			else
				l_x := ""
			end
			if y /= Void then
				l_y := y.out
			else
				l_y := ""
			end
			if z /= Void then
				l_z := z.out
			else
				l_z := ""
			end
			Result := <<l_x_label, l_x, l_y_label, l_y, l_z_label, l_z>>
		end
	
	get_data_description: ARRAY [STRING]	is
			-- <Precursor>
		do
			create Result.make (1, 6)
			Result := <<"X-Label", "X-Value", "Y-Label", "Y-Value", "Z-label", "Z-value">>
		end
	
end
