indexing
	description: "Definition of a chart axis"
	author: "Zoran Simic"
	date: "$Date: 2014/12/02 $"
	path: "$File: //rose/source/kernel/infrastructure/widgets/charts/rose_chart_axis_ranged.e $"
	revision: "$Revision: #13 $"

class
	ROSE_CHART_AXIS_RANGED

inherit
	ROSE_CHART_AXIS

create
	make

feature -- Access

	label (a_index: INTEGER): STRING is
			-- Label associated to tick with 'a_index', if any
		do
		end

	index_coordinate (a_index: INTEGER): DOUBLE is
			-- Coordinate for 'a_index'
		do
			Result := scaled (a_index)
		end

	label_formatter: FUNCTION [ANY, TUPLE [value: DOUBLE; decimals: INTEGER], STRING]
			-- Formatter to display the label with value `value', considering
			-- that `decimals' digits after the dot need to be displayed;
			-- Use default formatting if Void

feature -- Basic operations

	ensure_count (a_count: INTEGER) is
			-- Ensure that 'a_count' indices are defined...
		do
		end

	reset is
			-- Reset this axis state
		do
		end

	set_indices (a_lower, a_upper: INTEGER) is
			-- Add 'a_lower'..'a_upper' range to this axis.
		do
		end

	set_dates (a_dates: ARRAY [ROSE_DATE]) is
			-- Add 'a_dates' as meaningful ticks
		do
		end

	set_labels (a_labels: ARRAY [ANY]) is
			-- Add 'a_labels' as indexed ticks.
		do
		end

	set_ticked_labels (a_ticks: ROSE_LINEAR_ARRAY [DOUBLE]; a_labels: ROSE_LINEAR_ARRAY [ANY]) is
			-- Add 'a_ticks' with 'a_labels' as miningful ticks.
		do
		end

	set_label_formatter (a_formatter: like label_formatter) is
			-- Set `label_formatter' to `a_formatter'.
		do
			label_formatter := a_formatter
		ensure
			label_formatter_set: label_formatter = a_formatter
		end

	set_date_range (a_start, a_end: ROSE_DATE) is
			-- Set date range
		do
			start_date := a_start
			end_date := a_end
			set_minimum_visible_date (a_start)
			set_maximum_visible_date (a_end)
			if coordinate_system /= Void then
				coordinate_system.state.set_needs_coordination
			end
		end

feature {NONE} -- Implementation

	start_date, end_date: ROSE_DATE		-- Start/end date for date ranges

	prepare_ticks (a_origin, a_scale: DOUBLE) is
			-- Populate 'projected_ticks' with relevant ticks for this projection
		do
			if start_date = Void then
				prepare_numeric_ticks (a_origin, a_scale)
			else
				prepare_date_range_ticks (a_origin, a_scale)
			end
		end

	prepare_date_range_ticks (a_origin, a_scale: DOUBLE) is
			-- Populate 'projected_ticks' with relevant ticks for this projection
		require
			is_date_range: start_date /= Void
		local
			l_min, l_max, l_days, l_months, l_years, l_y: DOUBLE
			l_tuple: TUPLE [date: ROSE_DATE; representation: STRING; level: INTEGER]
			l_ticks: ROSE_LINEAR_ARRAY [TUPLE [ROSE_DATE, STRING, INTEGER]]
			l_date, l_start, l_end: ROSE_DATE
			i, l_major_frequency: INTEGER
		do
			l_min := reverse_projected (l1.x, a_origin, a_scale)
			l_max := reverse_projected (l2.x, a_origin, a_scale)
			l_start := tick_to_date (l_min)
			l_end := tick_to_date (l_max)
			l_days := (l_max - l_min).abs
			l_months := l_days / 30
			l_years := l_days / 366
			create l_ticks.make_empty
			if l_days < 1/24 then					-- Less than an hour: show minutes
				from
					l_date := l_start.twin
				until
					l_date > l_end
				loop
					if l_date.minute = 0 then
						if l_date.hour = 0 then
							l_ticks.extend ([l_date, l_date.formatted_out ("yy/[0]mm/[0]dd"), 100])
						else
							l_ticks.extend ([l_date, l_date.formatted_out ("[0]mm/[0]dd [0]hh:[0]mi"), 100])
						end
					else
						l_ticks.extend ([l_date, l_date.formatted_out ("[0]mi:[0]ss"), 10])
					end
					l_date := l_date.twin
					l_date.second_add (60)
				end
			elseif l_days < 6/24 then				-- 6 hours: show 30 minute increments
				from
					l_date := l_start.twin
				until
					l_date > l_end
				loop
					if l_date.hour = 0 then
						l_ticks.extend ([l_date, l_date.formatted_out ("yy/[0]mm/[0]dd"), 100])
					else
						l_ticks.extend ([l_date, l_date.formatted_out ("[0]hh:[0]mi"), 10])
					end
					l_date := l_date.twin
					l_date.second_add (60*30)
				end
			elseif l_days < 3 then					-- 3 days: show hours
				from
					l_date := l_start.twin
				until
					l_date > l_end
				loop
					if l_date.hour = 0 then
						l_ticks.extend ([l_date, l_date.formatted_out ("yy/[0]mm/[0]dd"), 100])
					else
						l_ticks.extend ([l_date, l_date.formatted_out ("[0]hh:[0]mi"), 10])
					end
					l_date := l_date.twin
					l_date.second_add (3600)
				end
			elseif l_days < 90 then					-- 3 months: show days
				from
					l_date := l_start
				until
					l_date > l_end
				loop
					if l_date.day = 1 then
						l_ticks.extend ([l_date, l_date.formatted_out ("yy/[0]mm/[0]dd"), 100])
					elseif l_date.day_of_the_week = l_date.monday then
						l_ticks.extend ([l_date, l_date.formatted_out ("[0]dd"), 100])
					else
						l_ticks.extend ([l_date, l_date.formatted_out ("[0]dd"), 10])
					end
					l_date := l_date.plus_n_date (1)
				end
			elseif l_days < 366*2 then				-- 2 years: show months
				from
					l_date := l_start
				until
					l_date > l_end
				loop
					if l_date.month = 1 then
						l_ticks.extend ([l_date, l_date.formatted_out ("yy/[0]mm"), 100])
					else
						l_ticks.extend ([l_date, l_date.formatted_out ("[0]mm"), 10])
					end
					l_date := l_date.plus_n_month (1)
				end
			else									-- Show years + quarters
				if l_years < 10 then
					l_major_frequency := 1
				elseif l_years < 30 then
					l_major_frequency := 5
				else
					l_major_frequency := 10
				end
				from
					l_date := l_start
				until
					l_date > l_end
				loop
					if l_date.quarter = 1 then
						if l_date.year \\ l_major_frequency = 0 then
							l_ticks.extend ([l_date, l_date.year.out, 100])
						else
							l_ticks.extend ([l_date, l_date.year.out, 50])
						end
					elseif l_major_frequency = 1 then
						l_ticks.extend ([l_date, "Q" + l_date.quarter.out, 5])
					end
					l_date := l_date.plus_n_month (3)
				end
			end
			from
				i := l_ticks.lower
			until
				i > l_ticks.upper
			loop
				l_tuple := l_ticks.item (i)
				l_date := l_tuple.date
				l_y := date_to_tick (l_date)
				if is_logarithmic then
					add_tick (projected (l_y, a_origin, a_scale), l_tuple.representation, l_tuple.level)
				else
					add_tick (projected (l_y, a_origin, a_scale), l_tuple.representation, l_tuple.level)
				end
				i := i + 1
			end
		end

	prepare_numeric_ticks (a_origin, a_scale: DOUBLE) is
			-- Populate 'projected_ticks' with relevant ticks for this projection
		require
			is_numeric: start_date = Void
		local
			l_min, l_max, l_y, l_minor: DOUBLE
			l_power, l_power_fraction, l_level: INTEGER
			i, l_tick_count, l_minor_tick_frequency: INTEGER_64
			l_decimals: INTEGER
		do
			l_min := reverse_projected (l1.x, a_origin, a_scale)
			l_max := reverse_projected (l2.x, a_origin, a_scale)
			l_y := (l_max - l_min).abs
			if l_y > 0.0 then
				l_y := math.log10 (l_y)
				l_power := l_y.rounded
				l_power_fraction := ((l_y - l_power) * 10).rounded
				if l_power_fraction <= 1 then
					l_power := l_power - 1
				end
				l_minor := 10 ^ l_power
				if l_power_fraction.abs > 1 then
					l_minor := l_minor / 2
					l_decimals := (-l_power + 1).max (0)
					l_minor_tick_frequency := 2			-- Major ticks occur every second tick
				else
					l_decimals := (-l_power).max (0)
					l_minor_tick_frequency := 10		-- Major ticks occur every 10th tick
				end
				from
					i := (l_min.min (l_max) / l_minor).truncated_to_integer_64
					l_tick_count := (l_max.max (l_min) / l_minor).truncated_to_integer_64
				until
					i > l_tick_count
				loop
					l_y := l_minor * i
					if i \\ l_minor_tick_frequency = 0 then
						l_level := 100		-- Major tick (bold)
					else
						l_level := 10		-- Minor tick
					end
					if is_logarithmic then
						add_tick (projected (l_y, a_origin, a_scale), formatted_label (10 ^ l_y, l_decimals), l_level)
					else
						add_tick (projected (l_y, a_origin, a_scale), formatted_label (l_y, l_decimals), l_level)
					end
					i := i + 1
				end
			end
		end

	formatted_label (a_value: DOUBLE; a_decimals: INTEGER): STRING is
			-- Textual representation of the label with value `a_value', considering
			-- that `a_decimals' digits after the dot need to be displayed
		do
			if label_formatter /= Void then
				Result := label_formatter.item ([a_value, a_decimals])
			end
			if Result = Void then
				formatter.set_width (a_decimals + 1)
				formatter.set_decimals (a_decimals)
				Result := formatter.formatted (a_value)
				Result.prune_all_trailing ('.')
			end
		ensure
			formatted_label_not_void: Result /= Void
		end

	formatter: FORMAT_DOUBLE is
			-- Formatter for labels
		once
			create Result.make (1, 0)
			Result.comma_separate
		ensure
			formatter_not_void: Result /= Void
		end

end
