indexing
	description: "Definition of a chart axis tick"
	author: "Zoran Simic"
	date: "$Date: 2014/12/02 $"
	revision: "$Revision: #5 $"

class
	ROSE_CHART_AXIS_TICK

inherit
	ROSE_CHART_CONSTANTS

create
	make

feature {NONE} -- Initialization

	make (a_font: EV_FONT) is
		do
			create text
			text.set_font (a_font)
		end

feature -- Access

	projected: INTEGER
			-- Projected coordinate of this tick (x or y depending on axis)

	label: STRING

	level: INTEGER
			-- Tick level, 0..9=just a tick, 10..99=minor, 100=major
			-- minor and major ticks get a grid line

	color: EV_COLOR

	text_width: INTEGER is
		do
			Result := text.width
		end

	debug_output: STRING is
			-- String that should be displayed in debugger to represent `Current'.
		do
			Result := projected.out + " [" + level.out + "]"
		end

feature -- Basic operations

	set (a_projected: INTEGER; a_label: STRING; a_level: INTEGER) is
		do
			projected := a_projected
			label := a_label
			level := a_level
			if a_label /= Void then
				text.set_text (truncated_string (label))
			else
				text.set_text ("")
			end
			if a_level >= 50 then
				do_nothing
--				text.font.set_weight (Gui.rep.fonts.weight_bold)
			end
		end

	set_color (a_color: EV_COLOR) is
		do
			color := a_color
		end

	add_to_axis (a_axis: ROSE_CHART_AXIS; a_tx, a_ty: INTEGER) is
		do
			create tick
			a_axis.tick_holder.extend (tick)
			a_axis.set_position_a (tick, projected, a_axis.origin.y)
			a_axis.set_position_b (tick, projected, a_axis.tick_y)
			if color /= Void then
				tick.set_foreground_color (color)
			end
			if label /= Void then
				a_axis.set_tooltip (tick,label)
				a_axis.set_position (text, projected + a_tx, a_axis.tick_y + a_ty)
				a_axis.set_tooltip (text,label)
				a_axis.tick_holder.extend (text)
				if color /= Void then
					text.set_foreground_color (color)
				end
			end
			if level >= 10 then
				create grid_line
				a_axis.set_position_a (grid_line, projected, a_axis.origin.y)
				a_axis.set_position_b (grid_line, projected, a_axis.l2.y)
				a_axis.grid_holder.extend (grid_line)
				if level >= 100 then
					tick.set_line_width (2)
					grid_line.set_foreground_color (a_axis.display.settings.grid_major_color)
				else
					grid_line.enable_dashed_line_style
					grid_line.set_foreground_color (a_axis.display.settings.grid_minor_color)
				end
			end
		end

feature {NONE} -- Implementation

	tick: EV_MODEL_LINE
	text: EV_MODEL_TEXT
	grid_line: EV_MODEL_LINE

	truncated_string (a_string: STRING): STRING is
		do
			if a_string /= Void then
				if a_string.count > max_tick_characters then
					Result := a_string.twin
					Result.keep_head (max_tick_characters - 2)
					Result.append ("..")
				else
					Result := a_string
				end
			end
		end

	max_tick_characters: INTEGER is 20

end
