indexing
	description: "Background figure for the chart system"
	author: "Urs Doenni"
	date: "$Date: 2009/03/30 $"
	revision: "$Revision: #5 $"
	archive: "$File: //rose/source/kernel/infrastructure/widgets/charts/rose_chart_background_figure.e $"

class
	ROSE_CHART_BACKGROUND_FIGURE
inherit
	ROSE_CHART_PROJECTABLE

create
	make_from_coordinates, make_from_tuples

feature -- creation
	make_from_coordinates (a_coords: ARRAY[EV_COORDINATE]; a_foreground_color: EV_COLOR; a_background_color: EV_COLOR) is
			-- creates Background figure from a Array of Coordinates
		require
			coords_not_void: a_coords /= Void
		do
			coords := a_coords
			create figure.make_with_coordinates (a_coords)
			figure.set_foreground_color (a_foreground_color)
			figure.set_background_color (a_background_color)
		end

	make_from_tuples (a_coords: ARRAY [TUPLE[x: DOUBLE; y: DOUBLE]]; a_foreground_color: EV_COLOR; a_background_color: EV_COLOR) is
			-- creates a Backgroundfigure from a Array  of Double tuples
		require
			coords_not_void: a_coords /= Void
		local
			i: INTEGER
			l_coords: like coords
			l_coord: TUPLE[x: DOUBLE; y: DOUBLE]
		do
			create l_coords.make (a_coords.lower, a_coords.upper)
			from i := a_coords.lower
			until i > a_coords.upper
			loop
				l_coord := a_coords.item (i)
				if l_coord /= Void then
					l_coords.put (create {EV_COORDINATE}.make_precise (l_coord.x, l_coord.y), i)
				end
				i := i + 1
			end
			make_from_coordinates (l_coords, a_foreground_color, a_background_color)
		end

feature  -- access
		set_coordinate_system (a_system: like coordinate_system) is
			-- set the coordinate system to use
		do
			coordinate_system := a_system
			if coordinate_system /= Void then
				display := coordinate_system.widget.display
			else
				display := Void
			end
		end

		figure: EV_MODEL_POLYGON
			-- Figure used

feature {NONE} -- implementation
	project (a_x0, a_y0, a_x_scale, a_y_scale: DOUBLE) is
			-- project this backgroundfigure
		local
			l_x, l_y: INTEGER
			i: INTEGER
		do
			if figure /= Void and coordinate_system /= Void then
				-- TODO: Center the figure
--				figure.set_x_y (display.left + a_x, display.bottom + a_y)
				from i := 1
				until i > figure.point_count
				loop
					l_x := display.projected_x (coords.item(i).x_precise, a_x0, a_x_scale)
					l_y := display.projected_y (coords.item(i).y_precise, a_y0, a_y_scale)

					figure.set_i_th_point_position (i, display.left + l_x, display.bottom + l_y)
					i := i + 1
				end
			end
		end

feature {None} -- implementation


	display: ROSE_CHART_DISPLAY
			-- Reference to the display

	coordinate_system: ROSE_CHART_COORDINATE_SYSTEM[ROSE_CHART_POINT]
			-- Coordinate system this figure is in

	coords: ARRAY [EV_COORDINATE]
			-- List of coordinates this point consists of

end
