indexing

	description: "ROSE_CHART_PLOT capable of handling ROSE_CHART_BOXPLOT_POINTs, with auxiliary functions"

	author: "Urs Doenni"
	last_editor: "$Author: c4 $"
	date: "$Date: 2009/03/26 $"
	revision: "$Revision: #9 $"
	path: "$File: //rose/source/kernel/infrastructure/widgets/charts/rose_chart_boxplot_plot.e $"

class	ROSE_CHART_BOXPLOT_PLOT

inherit

	ROSE_CHART_PLOT [ROSE_CHART_BOXPLOT_POINT]

create

	make_empty

create {ROSE_CHART_BOXPLOT_PLOT}

	make_chart_list,
	make_sublist

feature {ROSE_CHART_COORDINATE_SYSTEM} -- Implementation

	get_new_plot_list_instance: ROSE_CHART_BOXPLOT_PLOT_LIST is
			-- new instance of plot list that belongs to this plot
		do
			create Result.make_empty
		ensure then
			not_void : Result /= void
		end

feature -- Basic operations

	add_boxplots_from_data (a_ordinates: ?ARRAY [DOUBLE]; a_datasets: ARRAY [ARRAY [DOUBLE]]; a_labels : ARRAY [STRING]; a_widget_data: ARRAY [ANY]) is
			-- Add boxplots of raw data given in each element of `a_datasets'.
			-- If `a_ordinates' is provided, then boxes will be drawn in specified positions on x-axis;
			--  otherwise they will be evenly spaced out to span the axis.
			-- If `a_widget_data' is present, the items will be attached to the corresponding box-and-whisker plots
			--  as `{EV_ANY}.data'.
		require
			a_datasets_not_void: a_datasets /= Void
			no_void_dataset: not a_datasets.has (Void)
			correct_ordinates_count: a_ordinates /= Void implies a_ordinates.count = a_datasets.count
			correct_labels_count: a_labels /= Void implies a_labels.count = a_datasets.count
			correct_widget_data_count: a_widget_data /= Void implies a_widget_data.count = a_datasets.count 
		local
			l_arrays: ROSE_LINEAR_ARRAY [SUMMARY_STATISTICS_RECORD]
		do
			create l_arrays.make_empty
			a_datasets.do_all (agent add_moments (l_arrays, ?))
			add_boxplot_data_by_summary (a_ordinates, l_arrays, a_labels, a_widget_data)
		ensure
			correct_number_of_boxplots_added: count = old count + a_datasets.count 
		end

	add_boxplot_data (a_x : ARRAY[DOUBLE]; a_y : ARRAY[ARRAY[DOUBLE]]) is
			-- Add boxplots by providing optional x-values and y-values (moments).
			-- You really don't want to use this (unless you have weighted data).
			--  You probably want to use `add_boxplots_from_data' instead, to calculate the moments for you.
		local
			l_x: DOUBLE
			l_y: ARRAY[DOUBLE]
			l_values: TUPLE[x: DOUBLE; min: DOUBLE; p5: DOUBLE; p25: DOUBLE; median: DOUBLE; p75:DOUBLE; p95: DOUBLE; max: DOUBLE]
			i, n: INTEGER
			l_point: ROSE_CHART_BOXPLOT_POINT
		do
			if a_y /= Void then
				reindex_array (a_x)
				from
					i := 1
					n := a_y.count
				until
					i > n
				loop
					if a_x /= Void and then a_x.valid_index (i) then
						l_x := a_x.item (i)
					else
						l_x := i
					end
					l_y := a_y.item (i)
					if l_y.count = 7 then
						create l_values

						l_values.x := l_x
						l_values.min := l_y.item (1)
						l_values.p5 := l_y.item (2)
						l_values.p25 := l_y.item (3)
						l_values.median := l_y.item (4)
						l_values.p75 := l_y.item (5)
						l_values.p95 := l_y.item (6)
						l_values.max := l_y.item (7)


						create l_point.make (l_values, l_x)
						extend_back (l_point)
					end
					i := i + 1
				end
			end
		end

	add_boxplot_data_by_summary (a_x : ARRAY[DOUBLE]; a_y : ARRAY[SUMMARY_STATISTICS_RECORD]; a_labels : ARRAY[STRING]; a_data_array : ARRAY[ANY]) is
			-- add boxplots built from several objects of type {SUMMARY_STATISTICS_RECORD}
		local
			l_x: DOUBLE
			i, n: INTEGER
			l_label : STRING
			l_point : ROSE_CHART_BOXPLOT_POINT
		do
			if a_y /= Void then
				reindex_array (a_x)
				from
					i := 1
					n := a_y.count
				until
					i > n
				loop
					if a_x /= Void and then a_x.valid_index (i) then
						l_x := a_x.item (i)
					else
						l_x := i
					end
					if a_labels /= Void and then a_labels.valid_index (i) then
						l_label := a_labels[i]
					end
					create l_point.make_from_summary(l_x, a_y[i], i)
					l_point.set_full_label (l_label)
					if a_data_array /= Void and then a_data_array.valid_index (i) then
						l_point.set_data (a_data_array[i])
					end

					extend_back (l_point)
					i := i + 1
				end
			end
		end

feature {NONE} -- Implementation

	reindex_array (a_array: ARRAY [DOUBLE]) is
			-- reindex 'a_array'
		do
			if a_array /= Void then
				a_array.conservative_resize (1,a_array.count)
			end
		end

	add_moments (a_arrays: ROSE_LINEAR_ARRAY [SUMMARY_STATISTICS_RECORD]; a_dataset: ARRAY [DOUBLE]) is
			-- Add seven moments of `a_dataset' for boxplot to `a_arrays'.
		require
			a_arrays_not_void: a_arrays /= Void
			a_dataset_not_void: a_dataset /= Void
		local
			l_moments: ROSE_MOMENTS
		do
			create l_moments.make (a_dataset, Void)
			a_arrays.extend (create {SUMMARY_STATISTICS_RECORD}.make_from_moments (l_moments))
		ensure
			one_more: a_arrays.count = old a_arrays.count + 1
		end
	
end
