indexing

	description: "[
                  ROSE_CHART_PLOT_LIST capable of handling ROSE_CHART_BOXPLOT_POINTs with additional useful
					   helper functions for ROSE_CHART_BOXPLOT_POINTs
                ]"

	author: "Urs Doenni"
	last_editor: "$Author: c4 $"
	date: "$Date: 2009/03/26 $"
	revision: "$Revision: #8 $"
	path: "$File: //rose/source/kernel/infrastructure/widgets/charts/rose_chart_boxplot_plot_list.e $"

class	ROSE_CHART_BOXPLOT_PLOT_LIST

inherit
	
	ROSE_CHART_PLOT_LIST[ROSE_CHART_BOXPLOT_PLOT]

create

	make_empty, make_with_plots

create {ROSE_CHART_BOXPLOT_PLOT_LIST}

	make

feature -- Basic operations

	add_boxplots_from_data (a_name: STRING; a_ordinates: ?ARRAY [DOUBLE]; a_datasets: ARRAY [ARRAY [DOUBLE]];
		a_labels : ARRAY [STRING]; a_widget_data: ARRAY [ANY]) is
			-- Add boxplots of raw data given in each element of `a_datasets'.
			-- If `a_ordinates' is provided, then boxes will be drawn in specified positions on x-axis;
			--  otherwise they will be evenly spaced out to span the axis.
			-- If `a_widget_data' is present, the items will be attached to the corresponding box-and-whisker plots
			--  as `{EV_ANY}.data'.			
		require
			items_table_exists: items_table_not_void
			name_not_void: a_name /= Void
			a_datasets_not_void: a_datasets /= Void
			no_void_dataset: not a_datasets.has (Void)
			correct_oordinates_count: a_ordinates /= Void implies a_ordinates.count = a_datasets.count
			correct_labels_count: a_labels /= Void implies a_labels.count = a_datasets.count
			correct_widget_data_count: a_widget_data /= Void implies a_widget_data.count = a_datasets.count 			
		local
			l_plot: ROSE_CHART_BOXPLOT_PLOT
		do
			create l_plot.make_empty (a_name)
			add_plot (l_plot)
			l_plot.add_boxplots_from_data (a_ordinates, a_datasets, a_labels, a_widget_data)
		ensure
			nothing: True -- `add_plot' has no postconditions (20090325)
		end

	add_boxplot_plot (a_name: STRING; a_x: ARRAY[DOUBLE]; a_y: ARRAY[ARRAY[DOUBLE]]) is
			-- Add plot from one series of optional x values matched to multiples series of y values (moments).
			-- You really don't want to use this (unless you have weighted data).
			--  You probably want to use `add_boxplots_from_data' instead, to calculate the moments for you.
		require
			items_table_exists: items_table_not_void
			name_not_void: a_name /= Void
		local
			l_plot: ROSE_CHART_BOXPLOT_PLOT
		do
			if a_y /= Void and then a_y.count > 0 then
				create l_plot.make_empty (a_name)
				add_plot (l_plot)
				l_plot.add_boxplot_data (a_x,a_y)
			end
		ensure
			plot_added : (a_y /= void and then a_y.count > 0) implies not items_table.is_empty
		end

	add_boxplots_from_moment_array2(a_names : ARRAY[STRING]; a_moment_array2 : ROSE_ARRAY2[ARRAY[DOUBLE]]) is
			-- add boxplots from moment array2
		require
			valid_args : a_names /= Void and a_moment_array2 /= Void
		local
			i : INTEGER
			l_name : STRING
		do
			if a_names /= Void and a_moment_array2 /= Void then
				from i := 1 until i > a_moment_array2.width loop
					if i <= a_names.count then
						l_name := a_names.item(i - a_names.lower + 1)
					end
					add_boxplot_plot (l_name, Void, a_moment_array2.column(i))
					i := i + 1
				end
			end
		end

	add_boxplot_from_summary_series(a_name : STRING; a_x : ARRAY[DOUBLE]; a_y : ARRAY[SUMMARY_STATISTICS_RECORD]) is
		require
			valid_args : a_name /= Void and a_x /= Void and a_y /= Void
			size: a_y.count = a_x.count
		local
			l_plot: ROSE_CHART_BOXPLOT_PLOT
		do
			if a_y /= Void and then a_y.count > 0 then
				create l_plot.make_empty (a_name)
				add_plot (l_plot)
				l_plot.add_boxplot_data_by_summary (a_x,  a_y,  Void, Void)
			end
		end


	add_boxplot_from_summary_time_series(a_name : STRING; a_series : ROSE_TIME_SERIES[SUMMARY_STATISTICS_RECORD]) is
			-- add boxplot of summary statistics indexed by date
		require
			valid_args : a_name /= Void and a_series /= Void
		local
			l_plot: ROSE_CHART_BOXPLOT_PLOT
		do
			if a_series /= Void then
				create l_plot.make_empty (a_name)
				add_plot (l_plot)
				l_plot.add_boxplot_data_by_summary (a_series.double_array,  a_series,  a_series.ccyymmdd_strings, Void)
			end
		end

	add_boxplots_from_summary_array2(a_names : ARRAY[STRING]; a_moment_array2 : ROSE_ARRAY2[SUMMARY_STATISTICS_RECORD]) is
			-- add boxplots from summary array2 of statistics record
		require
			valid_args : a_names /= Void and a_moment_array2 /= Void
		local
			i : INTEGER
			l_name : STRING
		do
			if a_names /= Void and a_moment_array2 /= Void then
				from i := 1 until i > a_moment_array2.width loop
					if i <= a_names.count then
						l_name := a_names.item(i - a_names.lower + 1)
					end
					add_boxplot_from_summary_series (l_name, Void, a_moment_array2.column(i))
					i := i + 1
				end
			end
		end
	items_table_not_void : BOOLEAN is
		--does items_table exist? Used for assertions
		do
			Result := items_table /= void
		ensure
			nonvoid : Result = (items_table /= void)
		end

end
