indexing
	description: "Boxplot point for ROSE_CHART_*"
	author: "Urs Doenni"
	date: "$Date: 2008/12/01 $"
	revision: "$Revision: #10 $"
	archive: "$$"

class
	ROSE_CHART_BOXPLOT_POINT

inherit
	ROSE_CHART_POINT
	redefine
		values,
		labels,
		get_data_array,
		get_data_description
	end

create
	make,
	make_with_labels,
	make_from_summary

feature -- Initialization

	make_from_summary(a_x : DOUBLE; a_summary : SUMMARY_STATISTICS_RECORD; a_key : like key) is
			-- Build object, using `a_summary' to fill `values'
		require
			summary_not_void: a_summary /= Void
		local
			l_values: like values
		do
			summary_record := a_summary

			create l_values
			l_values.x := a_x
			l_values.min := a_summary.min
			l_values.p5 := a_summary.min
			l_values.p25 := a_summary.first_quartile
			l_values.median := a_summary.median
			l_values.p75 := a_summary.third_quartile
			l_values.p95 := a_summary.max
			l_values.max := a_summary.max

			make_with_labels (l_values, Void, a_key)
		end

feature -- Access

	summary_record : SUMMARY_STATISTICS_RECORD
			-- Summary statistics record used to build this point

	values: TUPLE[x: like x; min: like min; p5: like p5; p25: like p25; median: like median; p75: like p75; p95: like p95; max: like max] -- values
	labels: TUPLE[x: ANY; min: ANY; p5: ANY; p25: ANY; median: ANY; p75: ANY; p95: ANY;  max: ANY] -- labels

	x: DOUBLE is
			-- x value
		do
			Result := values.x
		end

	min : DOUBLE is
			-- return minimum value
		do
			Result := values.min
		end

	p5 : DOUBLE is do Result := values.p5 end
	p25 : DOUBLE is do Result := values.p25 end
	median : DOUBLE is do Result := values.median end
	p75 : DOUBLE is do Result := values.p75 end
	p95 : DOUBLE is do Result := values.p95 end
	max : DOUBLE is do Result := values.max end

feature -- Basic operations

	set_all (a_x: like x; a_min: like min; a_p5: like p5; a_p25: like p25; a_p75: like p75; a_median: like median; a_p95: like p95; a_max: like max) is
			-- set x and y
		require
			valid_order : a_min < a_max
		do
			values.x := a_x
			values.min := a_min
			values.p5 := a_p5
			values.p25 := a_p25
			values.median := a_median
			values.p75 := a_p75
			values.p95 := a_p95
			values.max := a_max

			is_set := True

			change_actions.call ([Current])
		end

	set_x (a_x: like x) is do set_value (a_x, 1) end
	set_min (a_min: like min) is do set_value (a_min, 2) end
	set_p5 (a_p5: like p5) is do set_value (a_p5, 3) end
	set_p25 (a_p25: like p25) is do set_value (a_p25, 4) end
	set_median (a_median: like median) is do set_value (a_median, 5) end
	set_p75 (a_p75: like p75) is do set_value (a_p75, 6) end
	set_p95 (a_p95: like p95) is do set_value (a_p95, 7) end
	set_max (a_max: like max) is do set_value (a_max, 8) end

	get_data_array: ARRAY[STRING] is
			-- returns all data to this point as array of strings
		do
			Result := <<values.max.out, values.median.out, values.min.out, values.p25.out, values.p5.out, values.p75.out, values.p95.out, values.x.out>>
		end

	get_data_description: ARRAY[STRING]	is
			-- returns a description to the data in get_data_array
		do
			Result := <<"max", "median", "min", "p25", "p5", "p75", "p95", "x">>
		end

feature {NONE} -- access

	set_value (a_value: DOUBLE; a_tuple_index: INTEGER) is
			-- set value
		do
			values.put_double (a_value, a_tuple_index)
			is_set := True
			change_actions.call ([Current])
		end
end
