indexing
	description: "Class reprensents a function of type: Real -> EV_COLOR"
	author: "Ueli Peter"
	date: "$Date: 2008/12/01 $"
	revision: "$Revision: #2 $"

class
	ROSE_CHART_COLOR_FUNCTION
create
	make_default,
	make_with_color_array

feature -- Creation
	make_default (a_lower, an_upper: REAL_64) is
			-- default creation with highest and lowest values, this function is able to handle
		require
			lower_not_bigger: a_lower <= an_upper
		do
			upper_y_value := an_upper
			lower_y_value := a_lower
			color_array := standart_colors
		end

	make_with_color_array (a_color_array: ARRAY[EV_COLOR]; a_lower, an_upper: REAL_64) is
			-- create with highest, lowest value and an array with colors
		require
			lower_not_bigger: a_lower <= an_upper
		do
			upper_y_value := an_upper
			lower_y_value := a_lower
			color_array := a_color_array
		end

feature -- Interface
	get_color_from_value (a_value: REAL_64): EV_COLOR is
			-- Interpolates between `lower_y_value' and `upper_y_value' and returns color from Array
		require
			value_between_upper_and_lower: a_value <= upper_y_value and a_value >= lower_y_value
		do
			Result := color_array.item ( ((color_array.count * (a_value - lower_y_value)) / (upper_y_value - lower_y_value)).floor.max(1) )
		end

feature -- setters
	set_lower_y_value (a_lower: like lower_y_value) is
			-- Set `lower_y_value'.
		do
			lower_y_value := a_lower
		end

	set_upper_y_value (an_upper: like upper_y_value) is
			-- Set `lower_x_value'.
		do
			upper_y_value := an_upper
		end

feature -- Access
	color_array: ARRAY[EV_COLOR]
		-- an Array of colors

	upper_y_value: REAL_64
		-- the highest value this function is able to handle

	lower_y_value: REAL_64
		-- the lowest value this function is able to handle

feature -- implementation
	standart_colors: like color_array is
			-- returns a standart color array
			local
				i: INTEGER
				l_color: EV_COLOR
			do
				--Rainbow
				From
					i := 0
					create Result.make (1, 80)
				until
					(80 - i) < 1
				loop
					if (i <= 20) then
						-- red to yellow
						create l_color.make_with_8_bit_rgb (255, (i * 12.25).floor, 0)
					elseif (i <= 40) then
						-- yellow to green
						create l_color.make_with_8_bit_rgb ((255 - (i - 20) * 12.25).floor, 255, 0)
					elseif (i <= 60) then
						-- green to Blue
						create l_color.make_with_8_bit_rgb (0, (255 - (i - 40) * 12.25).floor, ((i - 40) * 12.25).floor)
					elseif (i <= 80) then
						-- blue to violette
						create l_color.make_with_8_bit_rgb (((i - 40) * 12.25).floor, 0, 255 )
					end
					Result.put (l_color, 80-i)
					i := i + 1
				end
			end
end
