note
	description: "Coordinate system for ROSE_CHART_WIDGET"
	author: "Zoran Simic"
	last_editor: "$Author: myeager $"
	date: "$Date: 2014/02/03 $"
	revision: "$Revision: #31 $"
	path: "$File: //rose/source/kernel/infrastructure/widgets/charts/rose_chart_coordinate_system.e $"

class
	ROSE_CHART_COORDINATE_SYSTEM[P -> ROSE_CHART_POINT]

inherit
	ROSE_CHART_CONSTANTS

feature -- Access

	widget: ROSE_CHART_WIDGET [ROSE_CHART_POINT, ROSE_CHART_POINT]
			-- Parent widget holding this system

	state: ROSE_CHART_STATE
			-- State of the current chart representation

	renderer: ROSE_CHART_RENDERER [P]
			-- Renderer to be used to represent the plots

	holder: EV_MODEL_GROUP
			-- Group holding figures for this coordinate system

	background_figures_holder: EV_MODEL_GROUP
			-- Container for background figures

	zone_figures_holder: EV_MODEL_GROUP
			-- Container for one figures

	display: ROSE_CHART_DISPLAY is
			-- Associated display
		do
			Result := widget.display
		end

	settings: ROSE_CHART_SETTINGS is
			-- Associated settings
		do
			Result := display.settings
		end

	x_axis: ROSE_CHART_AXIS
			-- X Axis, can be customized

	y_axis: ROSE_CHART_AXIS
			-- Y axis, determined automatically if not provided, can be customized

	plots: ROSE_CHART_LIST[ROSE_CHART_RENDER_PLOT[P]]
			-- Plots

	has_plots: BOOLEAN is
			-- Are there plots to display?
		do
			Result := plots.count > 0
		end

	axes_count (a_horizontal, a_high: BOOLEAN): INTEGER is
			-- Number of axes at position (a_horizontal,a_high)
		local
			l_axis: ROSE_CHART_AXIS
		do
			if has_plots then
				if a_horizontal then
					l_axis := x_axis
				else
					l_axis := y_axis
				end
				if l_axis /= Void and then l_axis.is_shown and then l_axis.is_high = a_high then
					Result := 1
				end
			end
		end

	point_count: INTEGER is
			-- Total number of points in this chart (counting all plots)
		local
			l_linkable: LINKABLE[ROSE_CHART_RENDER_PLOT[P]]
		do
			from
				l_linkable := plots.first_element
			until
				l_linkable = Void
			loop
				Result := Result + l_linkable.item.points.count
				l_linkable := l_linkable.right
			end
		end

	world: ROSE_RECTANGLE
		-- Smallest rectangle containing all points from all plots

	current_view: ROSE_RECTANGLE is
			-- Current view (either 'world' or 'view_impl' depending on which one is used)
		local
			l_relative_world: ROSE_RECTANGLE
		do
			if not world.is_set then
				world.set (0, 0, 1, 1)
			end
			l_relative_world := world.relative_rectangle (widget.view)
			if view_imp.is_set then
				create Result.set_from_rectangle (view_imp.relative_rectangle (widget.view))
				if not view_x_set then
					Result.set_x (l_relative_world.x1, l_relative_world.x2)
				elseif not view_y_set then
					Result.set_y (l_relative_world.y1, l_relative_world.y2)
				end
			else
				create Result.set_from_rectangle (l_relative_world)
			end
		end

	set_view_x (a_x1, a_x2: DOUBLE) is
			-- set left and right boundaries of the view
		require
			not_equal: a_x1 /= a_x2
		do
			set_view (a_x1, view_imp.y1, a_x2, view_imp.y2)
			view_x_set := True
		end

	set_view_y (a_y1, a_y2: DOUBLE) is
			-- set top and bottom boundaries of the view
		require
			not_equal: a_y1 /= a_y2
		do
			set_view (view_imp.x1, a_y1, view_imp.x2, a_y2)
			view_y_set := True
		end

	set_view (a_x1, a_y1, a_x2, a_y2: DOUBLE) is
			-- set left, right, top and bottom boundaries of the view
		require
			not_equal: a_x1 /= a_x2
			not_equal: a_y1 /= a_y2
		local
			l_normale: ROSE_RECTANGLE
		do
			create l_normale.make_normal
			view_imp.set (a_x1, a_y1, a_x2, a_y2)
			view_x_set := True
			view_y_set := True
			widget. state.set_needs_projection
			view_changed_actions.call ([])
		end

	view_changed_actions: ACTION_SEQUENCE [TUPLE]
			-- List of agents to be called when view has changed

	view_overlay: ROSE_CHART_VIEW_OVERLAY
			-- View overlay in this rectangle (used to show coordinates of another system)

	highlighted_plots (a_highlighted: BOOLEAN): ROSE_LINEAR_ARRAY [ROSE_CHART_RENDER_PLOT[P]] is
			-- Plots that are currently highlit
		local
			l_linkable: LINKABLE [ROSE_CHART_RENDER_PLOT [P]]
		do
			from
				create Result.make_empty
				l_linkable := plots.first_element
			until
				l_linkable = Void
			loop
				if l_linkable.item.is_highlighted = a_highlighted then
					Result.extend (l_linkable.item)
				end
				l_linkable := l_linkable.right
			end
		end

	shown_plots (a_shown: BOOLEAN): ROSE_LINEAR_ARRAY [ROSE_CHART_RENDER_PLOT [P]]
			-- Plots that are currently shown
		local
			l_linkable: LINKABLE [ROSE_CHART_RENDER_PLOT [P]]
		do
			from
				create Result.make_empty
				l_linkable := plots.first_element
			until
				l_linkable = Void
			loop
				if l_linkable.item.is_shown = a_shown then
					Result.extend (l_linkable.item)
				end
				l_linkable := l_linkable.right
			end
		end

	highlighted_plots_count (a_highlighted: BOOLEAN): INTEGER
			-- Number of plots that are currently (un)highlit, in same highlight state as 'a_highlighted'
		local
			l_linkable: LINKABLE [ROSE_CHART_RENDER_PLOT [P]]
		do
			from
				l_linkable := plots.first_element
			until
				l_linkable = Void
			loop
				if l_linkable.item.is_highlighted = a_highlighted then
					Result := Result + 1
				end
				l_linkable := l_linkable.right
			end
		end

	shown_plots_count (a_shown: BOOLEAN): INTEGER
			-- Number of plots that are currently shown/hidden, in same shown state as 'a_shown'
		local
			l_linkable: LINKABLE [ROSE_CHART_RENDER_PLOT [P]]
		do
			from
				l_linkable := plots.first_element
			until
				l_linkable = Void
			loop
				if l_linkable.item.is_shown = a_shown then
					Result := Result + 1
				end
				l_linkable := l_linkable.right
			end
		end

	plot_by_name (a_name: STRING): ROSE_CHART_RENDER_PLOT[P] is
			-- Plot with 'a_name', if any
		do
			Result := plots.item_from_name (a_name)
		end

	debug_output: STRING is
			-- String that should be displayed in debugger to represent `Current'.
		do
			Result := plots.count.out + " plots, " + point_count.out + " points"
		end

	set_renderer (a_renderer: like renderer) is
			-- Set 'renderer'.
		do
			renderer := a_renderer
			state.set_needs_rendering
		end

	refresh is
			-- Refresh this object depending on 'state'.
		do
			refresh_part1
			refresh_part2
		end

feature -- Commands

	show (a_on: BOOLEAN) is
			-- Show/hide the charts in this system.
		do
			if a_on then
				holder.show
			else
				holder.hide
			end
		end

	clear_all is
			-- Clear the coordinate system
		do
			x_axis.clear_all
			y_axis.clear_all
			plots.wipe_out
			holder.wipe_out
			world.unset
			view_imp.unset
			view_changed_actions.wipe_out
			show (True)
			state.set_needs_all
		end

	set_x_axis (a_axis: ROSE_CHART_AXIS) is
			-- set `x_axis' to `a_axis'
		require
			valid_axis: a_axis /= Void
		do
			if x_axis /= a_axis then
				if x_axis /= Void then
					x_axis.remove
				end
				x_axis := a_axis
				x_axis.set_system (Current)
			end
		end

	set_y_axis (a_axis: ROSE_CHART_AXIS) is
			-- Set `y_axis' to `a_axis'
		require
			valid_axis: a_axis /= Void and then (a_axis /= y_axis implies a_axis.coordinate_system = Void)
		do
			if y_axis /= a_axis then
				if y_axis /= Void then
					y_axis.remove
				end
				y_axis := a_axis
				y_axis.set_system (Current)
			end
		end

	set_remember_colors (a_value: BOOLEAN) is
			-- set whether colors should be remembered
		do
			if a_value then
				if remembered_colors = Void then
					create remembered_colors.make (0)
				end
			else
				remembered_colors := Void
			end
		end

	set_plot_list (a_list: like plot_list) is
			-- set the plot list to be used to be used
			-- This overrides any previous calls of 'set_plot'
		require
			renderer_set: renderer /= Void
		local
			l_linkable: LINKABLE [ROSE_CHART_PLOT [P]]
		do
			plot_list := a_list
			clear_all
			plots := renderer.new_render_plot_list
			if plot_list /= Void then
				from
					l_linkable := plot_list.first_element
				until
					l_linkable = Void
				loop
					add_plot (l_linkable.item, renderer)
					l_linkable := l_linkable.right
				end
				plot_list.wipe_out_actions.extend (agent clear_all)
				plot_list.plot_added_actions_extend (agent add_plot (?, renderer))
					-- FIXME why are we passing the renderer if it's an attribute?
				plot_list.plot_removed_actions.extend (agent remove_plot)
			else
				clear_all
			end
		end

	set_plot (a_plot: ROSE_CHART_PLOT[P]) is
			-- set plot to be used
			-- this creates 'plot_list' and overrides any previous calls of 'set_plot_list'
		require
			renderer_set: renderer /= Void
		local
			l_plot_list: like plot_list
		do
			if a_plot /= Void then
				l_plot_list := a_plot.get_new_plot_list_instance
				l_plot_list.add_plot (a_plot)
				set_plot_list (l_plot_list)
			else
				set_plot_list (Void)
			end
		end

	plot_list: ROSE_CHART_PLOT_LIST [ROSE_CHART_PLOT [P]]
			-- List of all plots in this coordinate system

	add_background_figure (a_figure: ROSE_CHART_BACKGROUND_FIGURE) is
			-- add a background figure. The figure uses the coordinates of this system
		require
			figure_not_void: a_figure /= Void
		do
			a_figure.set_coordinate_system (Current)
			background_figures_holder.extend (a_figure.figure)
			background_figures_list.extend (a_figure)
			background_figures_holder.send_to_back (a_figure.figure)
		end

	add_zone_figure (a_figure: ROSE_CHART_ZONE_FIGURE) is
			-- add a zone figure from `a_x1' to `a_x2' with color `a_color', label `a_label' and on
			-- relative y position `a_frac'
		local
			i: INTEGER
		do
			a_figure.set_coordinate_system (Current)
			zone_figures_list.extend (a_figure)
			zone_figures_holder.extend (a_figure.figure)
			-- extend label in background_figures_holder, such that it gets displayed before the zone
			-- and background elements
			from
				i := 1
			until
				i > a_figure.labels.count
			loop
				background_figures_holder.extend (a_figure.labels.item (i))
				i := i + 1
			end
		end

	remove_background_figure (a_figure: ROSE_CHART_BACKGROUND_FIGURE) is
			-- remove a_figure from the background
		do
			a_figure.set_coordinate_system (Void)
			background_figures_holder.prune_all (a_figure.figure)
			background_figures_list.prune_all (a_figure)
		end

	background_figures_list: LINKED_LIST [ROSE_CHART_BACKGROUND_FIGURE]
			-- List of background figures

	zone_figures_list: LINKED_LIST [ROSE_CHART_BACKGROUND_FIGURE]
			-- List of zone figures

feature {NONE, ROSE_CHART_PROJECTOR} -- Implementation

	view_imp: ROSE_RECTANGLE
			-- Initial view on the chart

	view_x_set, view_y_set: BOOLEAN
			-- Are x and y coordinates of 'view' defined?

	add_plot (a_plot: ROSE_CHART_PLOT [P]; a_renderer: ROSE_CHART_RENDERER [P]) is
			-- Add new render plot (from 'a_plot') to this coordinate system.
		local
			l_color: EV_COLOR
			l_index: INTEGER
			l_render_plot: ROSE_CHART_RENDER_PLOT [P]
		do
			l_render_plot := a_renderer.new_render_plot (a_plot)
			--create l_render_plot.make (a_plot, a_renderer)
			l_render_plot.set_coordinate_system (Current)

			plots.add_item (l_render_plot)
			holder.extend (l_render_plot.holder)
			holder.extend (view_overlay.figure)
			holder.extend (widget.selection.holder)

			l_index := widget.plot_count
			if l_render_plot.color /= Void then
				-- Do nothing
			elseif remembered_colors /= Void then
				l_color := remembered_colors.item (l_render_plot.name)
				if l_color = Void then
					l_color := settings.palette.color (remembered_colors.count + 1)
					remembered_colors.put (l_color, l_render_plot.name)
				end
				l_render_plot.set_color (l_color)
			else
				l_render_plot.set_color (settings.palette.color (l_index))
			end
			state.set_needs_rendering
			state.set_needs_coordination
			state.set_needs_refresh
		end

feature {ROSE_CHART_RENDER_PLOT} -- plot management

	remove_plot (a_name: STRING) is
			-- Remove 'a_plot' from this coordinate system.
		local
			l_plot: ROSE_CHART_RENDER_PLOT [P]
		do
			l_plot := plots.item_from_name (a_name)
			if l_plot /= Void then
				plots.remove_item (l_plot.name)
				holder.prune_all (l_plot.holder)
				l_plot.set_coordinate_system (Void)
				state.set_needs_coordination
				state.set_needs_refresh
			end
		end



feature {ROSE_CHART_WIDGET} -- Implementation

	set_widget (a_widget: like widget) is
		-- set the `a_widget' as currents widget
		require
			valid_widget: a_widget /= Void
		do
			widget := a_widget
			create state.make (widget.state)
			create plots.make_empty
			create world
			create view_imp
			create view_changed_actions
			create view_overlay.make (Current)
			create holder
			create background_figures_holder
			create zone_figures_holder
			create background_figures_list.make
			create zone_figures_list.make
			set_x_axis (create {ROSE_CHART_AXIS_INDEXED}.make)
			set_y_axis (create {ROSE_CHART_AXIS_RANGED}.make)
			state.set_needs_all

			display.chart_group.extend (background_figures_holder)
			display.chart_group.send_to_back (background_figures_holder)
			display.chart_group.extend (zone_figures_holder)
			display.chart_group.send_to_back (zone_figures_holder)
			display.chart_group.extend (holder)
		end

	add_plots_in_rectangle (a_result: ROSE_LINEAR_ARRAY [ROSE_CHART_RENDER_PLOT[P]]; a_rectangle: EV_RECTANGLE) is
			-- Put all plots that intersect with 'a_rectangle' in 'a_result'.
		require
			valid_result: a_result /= Void
		local
			l_plot: ROSE_CHART_RENDER_PLOT[P]
			l_linkable: LINKABLE[ROSE_CHART_RENDER_PLOT[P]]
		do
			from
				l_linkable := plots.first_element
			until
				l_linkable = Void
			loop
				l_plot := l_linkable.item
				if l_plot.intersects (a_rectangle) then
					a_result.extend (l_plot)
				end
				l_linkable := l_linkable.right
			end
		end

	-- 'refresh' was broken into two parts to allow synchronization between two axes that is shared
	-- between the main and secondary coodinate systems.  The call to coordinate (below) may very
	-- well call a feature in the 'other' coordinate system.
	-- 'coordinate' calls 'sync_x_axis' which may call 'sync_x_axis' on the other coordinate system.

	refresh_part1 is
			-- Refresh this object depending on 'state'.
		do
			if has_plots and then state.needs_redraw then
				if state.needs_coordination then
					start_time_measure ("coordinate",point_count.out)
					coordinate
					end_time_measure
				end
			end
		end

	refresh_part2 is
			-- Refresh this object depending on 'state'.
		do
			if has_plots and then state.needs_redraw then
				if state.needs_rendering then
					start_time_measure ("render",Void)
					renderer.render (plots)
					end_time_measure
				end
				if state.needs_projection then
					start_time_measure ("project",Void)
					project
					end_time_measure
				end
			end
			state.reset
		end

feature {ROSE_CHART_COORDINATE_SYSTEM} -- Implementation

	needs_sync_x_axis: ROSE_CHART_COORDINATE_SYSTEM[ROSE_CHART_POINT]
			-- System which needs sync_x_axis

	needs_sync_y_axis: ROSE_CHART_COORDINATE_SYSTEM[ROSE_CHART_POINT]
			-- System which needs sync_y_axis

	set_needs_sync_x_axis (a_coordinate_system: like needs_sync_x_axis) is
			-- set needs_sync_x_axis to `a_coordinate_system'
		require
			not_already_set: needs_sync_x_axis = Void
		do
			needs_sync_x_axis := a_coordinate_system
		end

	set_needs_sync_y_axis (a_coordinate_system: like needs_sync_y_axis) is
			-- set needs_sync_y_axis to `a_coordinate_system'
		require
			not_already_set: needs_sync_y_axis = Void
		do
			needs_sync_y_axis := a_coordinate_system
		end

	sync_x_axis is
			-- synchronzies x axis
		do
			if x_axis.sync_axis /= Void then
				if x_axis.sync_axis.coordinate_system.world.is_set then
					world.embed_point(x_axis.sync_axis.coordinate_system.world.x1, world.y1)
					world.embed_point(x_axis.sync_axis.coordinate_system.world.x2, world.y2)
				else
					x_axis.sync_axis.coordinate_system.set_needs_sync_x_axis(Current)
				end
			end
			if needs_sync_x_axis /= Void then
				needs_sync_x_axis.sync_x_axis
				needs_sync_x_axis := Void
			end
		end

	sync_y_axis is
			-- synchronzies y axis
		local
			l_needs_sync_y_axis: like needs_sync_y_axis
		do
			if y_axis.sync_axis /= Void then
				if y_axis.sync_axis.coordinate_system.world.is_set then
					world.embed_point(world.x1, y_axis.sync_axis.coordinate_system.world.y1)
					world.embed_point(world.x2, y_axis.sync_axis.coordinate_system.world.y2)
				else
					y_axis.sync_axis.coordinate_system.set_needs_sync_y_axis(Current)
				end
			end
			if needs_sync_y_axis /= Void then
				l_needs_sync_y_axis := needs_sync_y_axis
				needs_sync_y_axis := Void
				--needs_sync_y_axis.sync_y_axis
				l_needs_sync_y_axis.sync_y_axis
			end
		end

feature {NONE} -- Implementation

	remembered_colors: HASH_TABLE [EV_COLOR, STRING]
			-- Hash table of remembered colors
	coordinate is
			-- Refresh points' coordinates realtive to this coordinate system.
		local
			l_plot: ROSE_CHART_RENDER_PLOT[P]
			l_linkable: LINKABLE[ROSE_CHART_RENDER_PLOT[P]]
		do
			world.unset
			from
				l_linkable := plots.first_element
			until
				l_linkable = Void
			loop
				l_plot := l_linkable.item
				l_plot.coordinate
				if l_plot.holder.is_show_requested then
					world.embed (l_plot.world)
				end
				l_linkable := l_linkable.right
			end

			sync_x_axis
			sync_y_axis
		end

	project is
			-- Project all 'plot' from 'a_view' to 'a_target'.
			-- 'a_view' is a subrectangle of the normal rectangle, indicating which
			-- part relatively to each plot world to project on 'a_target'.
		do
			projector.project_list (plots, Current)

			if view_overlay.is_used then
				view_overlay.update
			end

			-- TODO: THIS IS CALLED TOO LATE!
			-- (if points get updated by listener, they will say "state.needs_all",
			-- but the state will be reset after refresh_part2 is finished
--			view_changed_actions.call ([])
			-- TODO: move to 'refresh_part2'

		end

feature {NONE} -- implementation
	projector: ROSE_CHART_PROJECTOR is
			-- class used to project all elements in the plot
		once
			create Result
		end


end -- class ROSE_CHART_COORDINATE_SYSTEM
