indexing
	description: "[
			Selection area which displays current cutout of main widget in a ROSE_CHART_MINI_WIDGET
			Current coordinates are looked up in {ROSE_GRAPH_COORDINATE_SYSTEM}
			]"
	author: "Ueli Peter"
	date: "$Date: 2014/02/26 $"
	revision: "$Revision: #5 $"

class
	ROSE_CHART_CUTOUT_AREA

inherit
	ROSE_CHART_SELECTION_AREA
	redefine
		initialize, drag_selected, finalize_selection, start_resizing, is_within_resize_icon
	end

create
	make

feature -- Access
		initialize is
			-- Initialize the object
		local
			i: INTEGER
			l_rect: EV_MODEL_RECTANGLE
		do
			set_ok_color (Gui.rep.colors.blue)
			set_not_ok_color (Gui.rep.colors.red)
			set_resize_icon_size (10)
			set_ok_width (resize_icon_size)
			set_ok_height (resize_icon_size)
			create holder
			create selected_model.make_rectangle(0, 0, 0, 0)
			selected_model.set_line_width (5)
			holder.extend (selected_model)
			create resize_icons_models.make (1, max_index)
			from i := resize_icons_models.lower until i > resize_icons_models.upper
			loop
				l_rect := create {EV_MODEL_RECTANGLE}.default_create
				resize_icons_models.put (l_rect, i)
				i := i + 1
			end
			holder.extend (resize_icons_models.item (8))
			holder.extend (resize_icons_models.item (6))
			create left_corner_text.default_create
			create right_corner_text.default_create
		end

	finalize_selection is
			-- Finalize selection at ('a_x','a_y').
		do
			if is_selecting then
				is_selecting := False
				if big_enough then
					create_selected
				else
					--do nothing
				end
			end
		end

	start_resizing (a_x, a_y: INTEGER) is
			-- Start resizing at ('a_x','a_y').
		do
			if resize_index = 6 or resize_index = 8 or (vertical and (resize_index = 5 or resize_index = 7))then
				precursor (a_x, a_y)
			end
		end

	is_within_resize_icon (a_x, a_y: INTEGER): BOOLEAN is
			-- Is point ('a_x','a_y') within the resize icon?
		local
			l_rect: ROSE_RECTANGLE
		do
			if not vertical then


				if resize_icons /= Void then

					l_rect := resize_icons.item (6)
					if l_rect /= Void then
						Result := l_rect.has_point (a_x, a_y)
						if Result = True then
							resize_index := 6
						else
							l_rect := resize_icons.item (8)
							if l_rect /= Void then
								Result := l_rect.has_point (a_x, a_y)
								if Result = True then
									resize_index := 8
								end
							end
						end
					end
				end
			else
				Result := precursor (a_x, a_y)
			end
		end

	drag_selected (a_x, a_y: INTEGER) is
			-- Drag selected area to ('a_x','a_y').
			-- needs to be redefined, because of the truncation
		local
			l_x, l_y, l_w, l_h, i: INTEGER
		do
			if is_dragging then
				l_x := (x1.min (x2) + a_x - old_x).max (selectable_area.x1.truncated_to_integer)
				l_y := (y1.min (y2) + a_y - old_y).max (selectable_area.y1.truncated_to_integer)
				l_w := selected.width.truncated_to_integer  -- proper truncated
				l_h := selected.height.truncated_to_integer
				i := selectable_area.x2.truncated_to_integer
				if l_x + l_w > i then
					l_x := i - l_w
				end
				i := selectable_area.y2.truncated_to_integer
				if l_y + l_h > i then
					l_y := i - l_h
				end
				draw_selected (False)
				x1 := l_x
				y1 := l_y
				x2 := l_x + l_w
				y2 := l_y + l_h
				old_x := a_x
				old_y := a_y

				setup_model

				draw_selected (True)
			end
		end
	set_vertical (a_bool: BOOLEAN) is
			-- set `vertical' to `a_bol'
			-- enable vertical resizing
		do
			if vertical and not a_bool then
				holder.prune (resize_icons_models.item (5))
				holder.prune (resize_icons_models.item (7))
			elseif not vertical and a_bool then
				holder.extend (resize_icons_models.item (5))
				holder.extend (resize_icons_models.item (7))
			end
			vertical := a_bool
		end



feature {None} -- implementation

	vertical: BOOLEAN

end
