indexing
	description: "Display definition for chart widget"
	author: "Zoran Simic"
	last_editor: "$Author: io $"
	date: "$Date: 2014/12/02 $"
	revision: "$Revision: #16 $"

class
	ROSE_CHART_DISPLAY

inherit
	ROSE_CHART_CONSTANTS

	ROSE_GRAPHICS_DISPLAY
		redefine
			initialize_figures, graphics, state, settings, update_figures,
			refresh, clear_all, check_legend_position, legend, new_legend,
			selection
		end

create
	make

feature {NONE} -- Initialization

	new_legend: like legend is
			-- new legend instance
		do
			create Result.make ("Legend")
			Result.set_y (Result.y + 25)
		end

	initialize_figures is
			-- Custom figure_world initialization, redefine if needed.
		do
			Precursor
				-- Groups in order of priority (least priority first)
			chart_background := new_rectangle (figure_world)
			grid_group := new_group (figure_world)
			chart_group := new_group (figure_world)
			clipping_group := new_group (figure_world)
			axis_group := new_group (figure_world)
				-- Clipping
			top_clip := new_rectangle (clipping_group)
			bottom_clip := new_rectangle (clipping_group)
			left_clip := new_rectangle (clipping_group)
			right_clip := new_rectangle (clipping_group)

			-- Defaults
			set_grid_shown (True)
			set_legend_position (position_left)
			set_legend_alignment (align_top)
		end

feature -- Access

	graphics: ROSE_CHART_WIDGET[ROSE_CHART_POINT, ROSE_CHART_POINT]
			-- Associated widget

	selection: ROSE_CHART_SELECTION_AREA
			-- Selection area

	state: ROSE_CHART_STATE
			-- Associated state

	settings: ROSE_CHART_SETTINGS
			-- Settings to use

	legend: ROSE_LEGEND_CHARTS
			-- Small rectangle displaying key to colours used

	left, right, top, bottom: INTEGER
			-- Coordinates of chart area

	chart_background: EV_MODEL_RECTANGLE
	top_clip, bottom_clip, left_clip, right_clip: EV_MODEL_RECTANGLE
	chart_width, chart_height: INTEGER
			-- Rectangles representing backgrounds for the various parts of the display

	grid_group, chart_group, clipping_group, axis_group: EV_MODEL_GROUP

	legend_docked: BOOLEAN
			-- If true, the legend is docked to one of the sides of the chart

	legend_position: INTEGER
			-- Position of the legend relative to the plot
			-- Can be one of the position_* constants

	legend_alignment: INTEGER
			-- Alignment of the legend
			-- Can be one of the align_* constants

	is_grid_shown: BOOLEAN is
			-- Show grid?
		do
			Result := grid_group.is_show_requested
		end

	debug_output: STRING is
			-- String that should be displayed in debugger to represent `Current'.
		do
			Result := "[" + left.out + "," + top.out + "] [" + right.out + "," + bottom.out + "]"
		end

feature -- Basic operations

	update_figures is
			-- Update figures from 'settings'.
		local
			b, f: EV_COLOR
		do
			Precursor
			b := settings.palette.background
			f := settings.palette.foreground
			set_color (chart_background, b, settings.chart_background)
			set_color (top_clip, b, b)
			set_color (bottom_clip, b, b)
			set_color (left_clip, b, b)
			set_color (right_clip, b, b)
		end

	enable_click_toggle_visibility (a_on, a_resize: BOOLEAN) is
			-- Set toggle visibility on legend to 'a_on'
		do
			legend.click_event.remove (agent toggle_plot_visibility (False, ?, ?))
			legend.click_event.remove (agent toggle_plot_visibility (True, ?, ?))
			legend.click_event.extend (agent toggle_plot_visibility (a_resize, ?, ?))
		end

	refresh is
			-- Refresh the display
		require else
			a_pos_valid: legend_position = position_top or legend_position = position_right or legend_position = position_bottom or legend_position = position_left or legend_position = position_manual
		local
			l_left, l_right, l_bottom, l_top: INTEGER -- Variables that hold pixel values that have to be subtracted from the corresponding positions
		do
			Precursor
			inspect legend_position
			when position_bottom then
				l_left := 0; l_right := 0; l_top := 0
				l_bottom := legend.height
				inspect legend_alignment
				when align_left then
					if legend_docked then
						legend.set_x (axes_size (False, False) + settings.margin_left)
					else
						legend.set_x (settings.margin_left)
					end
				when align_center then
					legend.set_x (width // 2 - legend.width // 2)
				when align_right then
					legend.set_x (width - axes_size (False, True) - settings.margin_right - legend.width)
				end
				legend.set_y (height - l_bottom - 10)
				legend.set_maximum_width (width - axes_size (False, False) + settings.margin_left - 10)
				legend.set_maximum_height ({INTEGER}.max_value)
			when position_right then
				l_bottom := 0; l_left := 0; l_top := 0
				l_right := legend.width + 10
				inspect legend_alignment
				when align_top then
					legend.set_y (settings.title_height + axes_size (True, True) + settings.margin_top)
				when align_center then
					legend.set_y (height // 2 - legend.height // 2)
				when align_bottom then
					legend.set_y (height - axes_size ( True, False) - settings.margin_bottom - legend.height)
				end
				legend.set_x (width - l_right)
				legend.set_maximum_width ({INTEGER}.max_value)
				legend.set_maximum_height ({INTEGER}.max_value)
			when position_left then
				l_bottom := 0; l_right := 0; l_top := 0
				l_left := legend.width
				inspect legend_alignment
				when align_top then
					legend.set_y (settings.title_height + axes_size (True, True) + settings.margin_top)
				when align_center then
					legend.set_y (height // 2 - legend.height // 2)
				when align_bottom then
					legend.set_y (height - axes_size ( True, False) - settings.margin_bottom - legend.height)
				end
				legend.set_x (settings.margin_left)
				legend.set_maximum_width ({INTEGER}.max_value)
				legend.set_maximum_height ({INTEGER}.max_value)
			when position_top then
				l_bottom := 0; l_right := 0; l_left := 0
				l_top := legend.height + 10
				inspect legend_alignment
				when align_left then
					legend.set_x (axes_size (False, False) + settings.margin_left)
				when align_center then
					legend.set_x (width // 2 - legend.width // 2)
				when align_right then
					legend.set_x (width - axes_size (False, True) - settings.margin_right - legend.width)
				end
				legend.set_y (settings.margin_top)
				legend.set_maximum_width (width - axes_size (False, False) + settings.margin_left - 10)
				legend.set_maximum_height ({INTEGER}.max_value)
			when position_manual then
				-- in this case, the client sets x and y directly, so we do not re-set them here
				do_nothing
			end
			if not legend_positioned_once then
				legend_positioned_once := True
				record_legend_relative_position
			end
			-- Calculate main chart area coordinates
			if not legend_docked then
				l_bottom := 0
				l_right := 0
				l_left := 0
				l_top := 0
			end
			left := axes_size (False, False) + settings.margin_left + l_left
			right := width - axes_size (False, True) - settings.margin_right - l_right
			top := settings.title_height + axes_size (True, True) + settings.margin_top + l_top
			bottom := height - axes_size (True, False) - settings.margin_bottom - l_bottom
			chart_width := right - left
			chart_height := bottom - top
			-- Position main figures accordingly
			drawn_area.make (left, bottom, right, top)
			if chart_background /= Void then
				chart_background.set_point_a_position (left, top)
				chart_background.set_point_b_position (right, bottom)
			end
			left_clip.set_point_b_position (left - settings.clip_offset, height)
			top_clip.set_point_b_position (width, top - settings.clip_offset)
			right_clip.set_point_a_position (right + settings.clip_offset, height)
			right_clip.set_point_b_position (width, top - settings.clip_offset)
			bottom_clip.set_point_a_position (0, bottom + settings.clip_offset)
			bottom_clip.set_point_b_position (right + settings.clip_offset, height)
			settings.refresh
		end

	axes_size (a_horizontal, a_high: BOOLEAN): INTEGER is
			-- Size of the axis
		do
			Result := graphics.axes_count (a_horizontal, a_high)
			if a_horizontal then
				Result := Result * settings.axis_height
			else
				Result := Result * settings.axis_width
			end
		end

	set_grid_shown (a_on: BOOLEAN) is
			-- Show/Hide `grid_group'
		do
			if a_on then
				grid_group.show
			else
				grid_group.hide
			end
		end

	clear_all is
			-- Clear out the charts.
		do
			Precursor
			legend.reset
		end

	set_center (a_figure: EV_MODEL; a_x, a_y: INTEGER) is
			-- Set 'a_figure' position to ('a_x','a_y') on the visible chart.
		do
			a_figure.set_x_y (left + a_x, bottom + a_y)
		end

	set_position (a_figure: EV_MODEL_SINGLE_POINTED; a_x, a_y: INTEGER) is
			-- Set 'a_figure' position to ('a_x','a_y') on the visible chart.
		do
			a_figure.set_point_position (left + a_x, bottom + a_y)
		end

	set_position_a (a_figure: EV_MODEL_DOUBLE_POINTED; a_x, a_y: INTEGER) is
			-- Set figure's 'a' point position.
		do
			a_figure.set_point_a_position (left + a_x, bottom + a_y)
		end

	set_position_b (a_figure: EV_MODEL_DOUBLE_POINTED; a_x, a_y: INTEGER) is
			-- Set figure's 'b' point position.
		do
			a_figure.set_point_b_position (left + a_x, bottom + a_y)
		end

	set_legend_docked (a_switch: BOOLEAN) is
			-- Set the legend to be in docked state
		do
			legend_docked := a_switch
			legend.set_docked_style (a_switch)
			refresh
		ensure
			legend_docked_correct: legend_docked = a_switch
			legend_style_set: legend.docked_style = a_switch
		end

	set_legend_position (a_pos: INTEGER) is
			-- Set `legend_position' according to `a_pos'
		require
			a_pos_valid: a_pos = position_top or a_pos = position_right or a_pos = position_bottom or a_pos = position_left or a_pos = position_manual
		do
			legend_position := a_pos
		ensure
			position_correct: legend_position = a_pos
		end

	set_legend_alignment (a_pos: INTEGER) is
			-- Set `legend_position' according to `a_pos'
		require
			a_pos_valid: a_pos = align_left or a_pos = align_right or a_pos = align_center or a_pos = align_bottom or a_pos = align_top
		do
			legend_alignment := a_pos
		ensure
			alignment_correct: legend_alignment = a_pos
		end

feature {NONE} -- Implementation

	toggle_plot_visibility (a_resize: BOOLEAN; a_button: INTEGER; a_explanation: STRING) is
			-- Toggle visibility of plot 'a_explanation'
		local
			l_plot: ROSE_CHART_RENDER_PLOT[ROSE_CHART_POINT]
		do
			if a_button = 1 and a_explanation /= Void then
				l_plot := graphics.main_system.plot_by_name (a_explanation)
				if l_plot /= Void then
					l_plot.show (not l_plot.is_shown)
				end
				if a_resize then
					graphics.main_system.state.set_needs_all
				end
			end
		end

	check_legend_position is
			-- Make sure legend is following
		do
			if legend.x > pixmap.width - 6 then
				legend.set_x (pixmap.width - 12)
			end
			if legend.y > pixmap.height - 6 then
				legend.set_y (pixmap.height - 12)
			end
		end

end
