indexing

	description: "Routines for exporting charts in various formats"

	author: "Colin Adams"
	last_editor: "$Author: it $"
	date: "$Date: 2008/12/01 $"
	revision: "$Revision: #3 $"
	path: "$File: //rose/source/kernel/infrastructure/widgets/rose_chart_export_routines.e $"

class ROSE_CHART_EXPORT_ROUTINES

inherit

	ROSE_SHARED_GLOBAL_GUI

feature -- Output

	print_to (a_widget: EV_WIDGET; a_world: EV_MODEL_WORLD) is
			-- Print `a_world' on printer selected from dialog.
			-- `a_widget' is used in selecting the print context.
		require
			a_widget_not_void: a_widget /= Void
			a_world_not_void: a_world /= Void
		local
			l_projector: EV_MODEL_PRINT_PROJECTOR
			l_context: EV_PRINT_CONTEXT
		do
			l_context := Gui.dialogs.select_print_context (a_widget, Void)
			if l_context /= Void then
				create l_projector.make_with_context (a_world, l_context)
				l_projector.project
			end
		end

	export_as_svg (a_widget: EV_WIDGET; a_world: EV_MODEL_WORLD) is
			-- Save `a_world' as a Scalable Vector Graphics file.
			-- A save-file dialog is displayed relative to `a_widget'.
		require
			a_widget_not_void: a_widget /= Void
			a_world_not_void: a_world /= Void			
		local
			l_file_name: STRING
		do
			l_file_name := Gui.dialogs.select_save_file (a_widget, "SVG", "*.svg")
			if l_file_name /= Void then
				save_as_svg (a_world, create {FILE_NAME}.make_from_string (l_file_name))
			end
		end

	send_to (a_widget: EV_WIDGET; a_world: EV_MODEL_WORLD) is
			-- Send `a_world' as an email attachment (image/svg+xml).
			-- Email addresses are obtained from a dialog relative to `a_widget'.
		require
			a_widget_not_void: a_widget /= Void
			a_world_not_void: a_world /= Void		
		local
			l_projector: SV_PROJECTOR
			l_dialog: ROSE_EMAIL_DIALOG
		do
			create l_projector.make (a_world, create {RP_XML_SERIALIZER}.make)
			l_projector.set_indenting (True)
			l_projector.set_wants_doctype (True)
			l_projector.set_omitting_xml_declaration (False)
			l_projector.project
			create l_dialog .make ("Attached SVG image", "Hello,%N%NI am sending you the attached SVG graphic.%N%N")
			l_dialog.set_icon ("question.ico")
			l_dialog.ok_button.select_actions.wipe_out
			l_dialog.ok_button.select_actions.extend (agent send_email (l_projector, l_dialog))
			l_dialog.show_relative (a_widget)
		end

	copy_to_clipboard (a_world: EV_MODEL_WORLD) is
			-- Process request to copy `a_world' to clipboard as SVG.
		require
			a_world_not_void: a_world /= Void
		local
			l_projector: SV_PROJECTOR
		do
			create l_projector.make (a_world, create {RP_XML_SERIALIZER}.make)
			l_projector.set_indenting (True)
			l_projector.set_wants_doctype (True)
			l_projector.set_omitting_xml_declaration (False)
			l_projector.project
			Gui.ev_application.clipboard.set_text (l_projector.string)
		end

	export_picture (a_widget: EV_WIDGET; a_pixmap: EV_PIXMAP) is
			-- Save `a_pixmap' as a Portable Network Graphics file.
			-- A save-file dialog is displayed relative to `a_widget'.
		do
			Gui.dialogs.save_as_png (a_widget, a_pixmap, Void, Void)
		end

	export_postscript (a_widget: EV_WIDGET; a_world: EV_MODEL_WORLD) is
			-- Save `a_world' as a Postscript file.
			-- A save-file dialog is displayed relative to `a_widget'.
		local
			l_projector: ROSE_MODEL_POSTSCRIPT_PROJECTOR
			l_file: FILE_NAME
			l_file_name: STRING
		do
			l_file_name := Gui.dialogs.select_save_file (a_widget, "postscript", "*.ps")
			if l_file_name /= Void then
				create l_file.make_from_string (l_file_name)
				create l_projector.make_ledger_with_filename (a_world, l_file, False)
				l_projector.project
			end
		end

feature {NONE} -- Implementation
		
	save_as_svg (a_world: EV_MODEL_WORLD; a_file_name: FILE_NAME) is
			-- Draw `model' as Scalable Vector Graphics into a text file.
		require
			a_world_not_void: a_world /= Void
			a_file_name_not_void: a_file_name /= Void
		local
			l_projector: SV_PROJECTOR
		do
			create l_projector.make_with_filename (a_world, a_file_name, create {RP_XML_SERIALIZER}.make)
			l_projector.set_indenting (True)
			l_projector.set_wants_doctype (True)
			l_projector.set_omitting_xml_declaration (False)
			l_projector.project
		end

	send_email (a_projector: SV_PROJECTOR; a_dialog: ROSE_EMAIL_DIALOG) is
			-- Send SVG from `a_projector' as an email.
		require
			a_projector_not_void: a_projector /= Void
			a_dialog_not_void: a_dialog /= Void
		local
			l_email: ROSE_MAIL
			l_content: ROSE_MAIL_CONTENT
			l_attachment: ROSE_MAIL_ATTACHMENT
			l_type: UT_MEDIA_TYPE
		do
			create l_content.make
			l_content.set_subject (a_dialog.subject_line)
			l_content.set_plain_text
			l_content.set_text (a_dialog.body_text)
			l_content.to.add_recipients (a_dialog.recipients)
			a_dialog.hide
			a_dialog.destroy
			create l_attachment.set_content ("graph.svg", a_projector.string)
			create l_type.make ("image", "svg+xml")
			l_type.add_parameter ("charset", "UTF-8")
			l_attachment.set_content_type (l_type)
			l_attachment.set_binary (True)
			l_content.add_attachment (l_attachment)
			create l_email.make_smtp
			l_email.send (l_content)
		end

end

