indexing
	description: "Figure representing a ROSE_CHART_POINT"
	author: "Zoran Simic"
	date: "$Date: 2009/11/13 $"
	revision: "$Revision: #12 $"

deferred class
	ROSE_CHART_FIGURE

inherit
	ROSE_CHART_CONSTANTS

feature {ROSE_CHART_RENDERER} -- Initialization

	set_chart_point (a_predecessor, a_chart_point: like chart_point) is
			-- Attach 'a_chart_point' to this figure.
		require
			valid_point: a_chart_point /= Void
		do
			predecessor := a_predecessor
			chart_point := a_chart_point
			build
			refresh
		end

feature -- Access

	holder: EV_MODEL_GROUP is
			-- Group that will hold the representation for this point
		do
			Result := plot.holder
		end

	chart_point: ROSE_CHART_RENDER_POINT[ROSE_CHART_POINT]
			-- Chart-point represented by this figure

	predecessor: like chart_point
			-- Chart-point preceeding the one represented by this figure if any
			-- (it's a bit of a leakage of the LINE type of descendant
			--  but harmles when it's not a line)

	figure: EV_MODEL_ATOMIC is
			-- EV_MODEL figure that represents this figure
		deferred
		end

	figure_text: EV_MODEL_TEXT
			-- Text that is displayed alongside the figure

	intersects (a_rectangle: EV_RECTANGLE): BOOLEAN is
			-- Does this figure intersect with 'a_rectangle'?
		deferred
		end

	debug_output: STRING is
			-- String that should be displayed in debugger to represent `Current'.
		do
			create Result.make (20)
			if figure /= Void then
				Result.append ("[" + figure.x.out + "," + figure.y.out + "]")
			end
		end

feature -- Basic operations

	refresh is
			-- Refresh the looks of this figure.
			-- This is typically called when display settings have changed (color,...)
		require
			valid_chart_point: chart_point /= Void
		deferred
		end

	highlight (a_on: BOOLEAN) is
			-- Highlight this point.
		require
			valid_chart_point: chart_point /= Void
		deferred
		end

	remove is
			-- Remove this point.
		do
			holder.prune_all (figure)
		end

	project (a_x, a_y, a_x0, a_y0, a_x_scale, a_y_scale: DOUBLE) is
			-- Project this figure for origin ('a_x0,'a_y0')
			-- and with given scales.
		do
			if figure /= Void then
				display.set_center (figure, projected_x (a_x, a_x0, a_x_scale), projected_y (a_y, a_y0, a_y_scale))
			end
		end

	build is
			-- Called when 'chart_point' has been provided by the renderer.
		require
			valid_chart_point: chart_point /= Void
		deferred
		end

feature {NONE} -- Implementation

	plot: ROSE_CHART_RENDER_PLOT[ROSE_CHART_POINT] is
			-- Corresponding plot
		require
			valid_chart_point: chart_point /= Void
		do
			Result := chart_point.render_plot
		end

	embed_point_in_world (a_x: DOUBLE; a_y: DOUBLE) is
			-- Embed a point into the world (this makes sure the point is visible)
			-- Call this for every point that should be visible in the graph
		do
			chart_point.render_plot.world.embed_point (a_x, a_y)
			chart_point.render_plot.coordinate_system.world.embed_point (a_x, a_y)
		end

	display: ROSE_CHART_DISPLAY is
			-- Associated display
		require
			valid_chart_point: chart_point /= Void
		do
			Result := plot.display
		end

	pebble_function: ANY is
		local
			l_widget: ROSE_CHART_WIDGET [ROSE_CHART_POINT, ROSE_CHART_POINT]
		do
			if chart_point /= Void then
				Result := chart_point.pebble
				if Result = Void then
					Result := chart_point.point.pebble
				end
				if Result = Void then
					Result := chart_point.render_plot.pebble
				end
				if Result = Void then
					l_widget := chart_point.render_plot.coordinate_system.widget
					if l_widget.pebble_agent /= Void then
						Result := l_widget.pebble_agent.item ([chart_point.point])
					end
				end
			end
			if Result /= Void then
				if l_widget = Void then
					l_widget := chart_point.render_plot.coordinate_system.widget
				end
				if l_widget /= Void then
					l_widget.set_is_picking_imp (True)
				end
			end
		end

	on_figure_hover (a_figure: EV_MODEL; a_on: BOOLEAN) is
			-- Called when mouse hovers in or out from 'a_figure'.
		do
			if chart_point /= Void and then plot.coordinate_system /= Void then
				-- Do nothing, we need this case because Vision2 still calls
				-- 'mouse_leave_actions' even when the figure is not in the model world anymore...
				plot.coordinate_system.widget.events.hover.call ([a_on, plot])
				if a_on then
					if chart_point.tooltip_agent /= Void then
						display.set_tooltip (chart_point.tooltip, Void)
					elseif chart_point.point.tooltip_agent /= Void then
						display.set_tooltip (chart_point.point.tooltip, Void)
					else
						display.set_tooltip ("$1 ($2)",<<chart_point.render_plot.name, chart_point.full_label>>)
					end
				else
					display.set_tooltip (Void, Void)
				end
			end
		end

	on_click (a_x, a_y, a_button: INTEGER; a_x_tilt, a_y_tilt, a_pressure: DOUBLE; a_screen_x, a_screen_y: INTEGER) is
			-- Called when mouse is clicked on this figure.
		do
			plot.coordinate_system.widget.events.click.call ([a_button, chart_point])
		end

	on_double_click (a_x, a_y, a_button: INTEGER; a_x_tilt, a_y_tilt, a_pressure: DOUBLE; a_screen_x, a_screen_y: INTEGER) is
			-- Called when mouse is double-clicked on this figure.
		do
			plot.coordinate_system.widget.events.double_click.call ([a_button, chart_point])
		end

end
