indexing
	description: "Figure representing a ROSE_CHART_BOXPLOT_POINT with a boxplot figure"
	author: "Michael Zhang"
	date: "$Date: 2008/12/01 $"
	revision: "$$"
	archive: "$$"

class
	ROSE_CHART_FIGURE_BOXPLOT

inherit
	ROSE_CHART_FIGURE
		redefine
			chart_point,
			on_figure_hover
		end
	ROSE_SHARED_GLOBAL

create
	make

feature {NONE} -- Initialization

	make is
			-- Figure representing 'a_point'.
		do
			create figure

			max_line := new_line
			min_line := new_line
			median_line := new_line
			p95line := new_line
			p5line := new_line

			lower_whisker := new_line
			upper_whisker := new_line

			create box

			median_line.set_line_width (2)
			figure.set_pebble_function (agent pebble_function)
			box.set_pebble_function (agent pebble_function)
		end

	new_line : EV_MODEL_LINE is
			-- Create a new line
		do
			create Result
			Result.pointer_enter_actions.extend (agent on_figure_hover (figure, True))
			Result.pointer_leave_actions.extend (agent on_figure_hover (figure, False))
			Result.pointer_button_release_actions.extend (agent on_click)
			Result.pointer_double_press_actions.extend (agent on_double_click)
			Result.set_pebble_function (agent pebble_function)
		end

	on_figure_hover (a_figure: EV_MODEL; a_on: BOOLEAN) is
			-- Executed when the user hovers over a_figure
		require else
			valid_figure : a_figure /= Void
		local
			l_tooltip : STRING
		do
			if chart_point /= Void and then chart_point.point.summary_record /= Void then
				if chart_point.label /= Void then
					l_tooltip := chart_point.label + "%N%N"
				else
					l_tooltip := ""
				end
				l_tooltip.append( item_string("Count", chart_point.point.summary_record.count))
				l_tooltip.append (item_string("Min", chart_point.point.summary_record.min))
				l_tooltip.append (item_string("1Qtile", chart_point.point.summary_record.first_quartile))
				l_tooltip.append (item_string("Median", chart_point.point.summary_record.median))
				l_tooltip.append (item_string("3Qtile", chart_point.point.summary_record.third_quartile))
				l_tooltip.append (item_string("Max", chart_point.point.summary_record.max))
				l_tooltip.append (item_string("Mean", chart_point.point.summary_record.mean))
				l_tooltip.append (item_string("WtMean", chart_point.point.summary_record.weighted_mean))
				l_tooltip.append (item_string("StDev", chart_point.point.summary_record.standard_deviation))
				l_tooltip.append (item_string("Skewness", chart_point.point.summary_record.skewness))
				l_tooltip.append (item_string("Kurtosis", chart_point.point.summary_record.kurtosis))
				display.set_tooltip (l_tooltip, Void)
			end
		end

	item_string(a_name : STRING; a_value : DOUBLE) : STRING is
			-- item string
		require
			valid_name : a_name /= Void
		do
			if is_defined_double(a_value) then
				if a_name.is_equal("Count") then
					Result := Global.formatter.padded_string (a_name, ' ', 10) + "= " + a_value.out + "%N"
				else
					Result := Global.formatter.padded_string (a_name, ' ', 10) + "= " + Global.formatter.formatted_double (a_value, 8, 4) + "%N"
				end
			else
				Result := Global.formatter.padded_string (a_name, ' ', 10) + "= n\a%N"
			end
		end

feature -- Access
	index : INTEGER
	count : INTEGER

	offset1, offset2: DOUBLE

	figure: EV_MODEL_RECTANGLE

	max_line : EV_MODEL_LINE
	min_line : EV_MODEL_LINE
	median_line : EV_MODEL_LINE
	p95line : EV_MODEL_LINE
	p5line : EV_MODEL_LINE

	lower_whisker : EV_MODEL_LINE
	upper_whisker : EV_MODEL_LINE

	box : EV_MODEL_RECTANGLE

	chart_point : ROSE_CHART_RENDER_BOXPLOT_POINT

	intersects (a_rectangle: EV_RECTANGLE): BOOLEAN is
			-- Does this figure intersect with 'a_rectangle'?
		do
			Result := figure /= Void and then figure.bounding_box.intersects (a_rectangle)
		end

feature -- Basic operations

	refresh is
			-- refresh
		do
		end

	highlight (a_on: BOOLEAN) is
			-- Highlight this point.
		do
			if a_on then
				max_line.set_line_width (2)
				min_line.set_line_width (2)
				median_line.set_line_width (4)
				p95line.set_line_width (2)
				p5line.set_line_width (2)
				box.set_line_width (2)
				lower_whisker.set_line_width (2)
				upper_whisker.set_line_width (2)
			else
				max_line.set_line_width (1)
				min_line.set_line_width (1)
				median_line.set_line_width (2)
				p95line.set_line_width (1)
				p5line.set_line_width (1)
				box.set_line_width (1)
				lower_whisker.set_line_width (1)
				upper_whisker.set_line_width (1)
			end
		end

	project_with_point (a_point : ROSE_CHART_RENDER_BOXPLOT_POINT; a_x0, a_y0, a_x_scale, a_y_scale: DOUBLE) is
		local
			l_x1, l_x2, l_mid_x : INTEGER
		do
			project (a_point.x, a_point.median, a_x0, a_y0, a_x_scale, a_y_scale)

			l_x1 := projected_x (a_point.x + offset1, a_x0, a_x_scale)
			l_x2 := projected_x (a_point.x + offset2, a_x0, a_x_scale)
			l_mid_x := (l_x1 + l_x2) // 2

			display.set_position_a (figure, l_x1, projected_y (a_point.min , a_y0, a_y_scale))
			display.set_position_b (figure, l_x2, projected_y (a_point.max , a_y0, a_y_scale))

			display.set_position_a (box, l_x1, projected_y (a_point.p25 ,a_y0, a_y_scale))
			display.set_position_b (box, l_x2, projected_y (a_point.p75 ,a_y0, a_y_scale))

			display.set_position_a (max_line, l_x1, projected_y (a_point.max ,a_y0, a_y_scale))
			display.set_position_b (max_line, l_x2, projected_y (a_point.max ,a_y0, a_y_scale))

			display.set_position_a (min_line, l_x1, projected_y (a_point.min ,a_y0, a_y_scale))
			display.set_position_b (min_line, l_x2, projected_y (a_point.min ,a_y0, a_y_scale))

			display.set_position_a (median_line, l_x1, projected_y (a_point.median ,a_y0, a_y_scale))
			display.set_position_b (median_line, l_x2, projected_y (a_point.median ,a_y0, a_y_scale))

			project_line (lower_whisker, l_mid_x, l_mid_x, a_point.min, a_point.p25, a_y0, a_y_scale)
			project_line (upper_whisker, l_mid_x, l_mid_x, a_point.p75, a_point.max, a_y0, a_y_scale)
		end

	project_line (a_line : EV_MODEL_LINE; a_x1, a_x2 : INTEGER; a_y1, a_y2, a_y0, a_y_scale: DOUBLE) is
		do
			display.set_position_a (a_line, a_x1, projected_y (a_y1 ,a_y0, a_y_scale))
			display.set_position_b (a_line, a_x2, projected_y (a_y2 ,a_y0, a_y_scale))
		end

feature {ROSE_CHART_RENDERER_BOXPLOT} -- Implementation

	build is
			-- Called when 'chart_point' has been provided by the renderer.
		do
			index := chart_point.render_plot.index
			count := chart_point.render_plot.coordinate_system.plots.count

			offset1 := -0.5 + (index - 0.45) / (count + 1)
			offset2 := -0.5 + (index  + 0.45) / (count + 1)

			embed_point_in_world (chart_point.given_x + offset1, chart_point.given_min)
			embed_point_in_world (chart_point.given_x + offset2, chart_point.given_max)

			if figure /= Void then
				figure.pointer_enter_actions.extend (agent on_figure_hover (figure, True))
				figure.pointer_leave_actions.extend (agent on_figure_hover (figure, False))

--				TODO (not exactly the behaviour initialy intended, but we don't have an index anymore
				if plot.settings.is_use_count_for_plot_color_index then
					box.set_background_color (plot.settings.palette.color (chart_point.key.hash_code))
				else
					box.set_background_color (plot.color)
				end

				holder.extend (box)
				holder.extend (max_line)
				holder.extend (min_line)
				holder.extend (median_line)
				holder.extend (p95line)
				holder.extend (p5line)
				holder.extend (lower_whisker)
				holder.extend (upper_whisker)
			end
		end

end
