indexing
	description: "Figure representing a ROSE_CHART_2D_POINT"
	author: "Zoran Simic"
	date: "$Date: 2009/11/13 $"
	revision: "$Revision: #30 $"

class
	ROSE_CHART_FIGURE_LINE

inherit
	ROSE_CHART_FIGURE
		redefine
			chart_point,
			project, on_figure_hover
		end

create
	make

feature {NONE} -- Initialization

	make (a_lined, a_marked: BOOLEAN; a_point_shape_index: INTEGER; a_normal_width, a_highlighted_width: INTEGER) is
			-- Figure representing `a_point'.
		require
			valid_line_widths: a_normal_width >= 0 and a_highlighted_width >= 0
		do
			set_width (a_normal_width, a_highlighted_width)

			figure := point_shape (a_point_shape_index)
			figure.set_pebble_function (agent pebble_function)
			if a_marked then
				figure.show
			else
				figure.hide
			end
			if a_lined then
				create line
				line.set_line_width (width_normal)
				line.set_pebble_function (agent pebble_function)
			end
		end

feature -- Access

	chart_point: ROSE_CHART_RENDER_2D_POINT
			-- Reference to the point this figure represents

	figure: EV_MODEL_ATOMIC
		-- Figure to represent this line

	line: EV_MODEL_LINE
			-- Line linking this point to the next (if lined)

	intersects (a_rectangle: EV_RECTANGLE): BOOLEAN is
			-- Does this figure intersect with 'a_rectangle'?
		do
			Result := line /= Void and then line.bounding_box.intersects (a_rectangle)
				or else (figure /= Void and then figure.bounding_box.intersects (a_rectangle))
		end

	width_normal: INTEGER
			-- Width of the line

	width_highlighted: INTEGER
			-- Width of the line when it's highlighted

feature -- Basic operations

	refresh is
			-- highlight if is marked as highlighted
		do
			highlight (chart_point.is_highlighted)
		end

	highlight (a_on: BOOLEAN) is
			-- Highlight this point.
		do
			if a_on then
				if figure /= Void then
					designe_figure_shape (display.settings.point_size + 5)
				end
				if line /= Void then
					line.set_line_width (width_highlighted)
				end
			else
				if figure /= Void then
					designe_figure_shape (display.settings.point_size + 1)
				end
				if line /= Void then
					line.set_line_width (width_normal)
				end
			end
		end

	project (a_x, a_y, a_x0, a_y0, a_x_scale, a_y_scale: DOUBLE) is
			-- Project this figure for origin ('a_x0,'a_y0')
			-- and with given scales.
		do
			Precursor (a_x, a_y, a_x0, a_y0, a_x_scale, a_y_scale)
			if figure /= Void and then predecessor /= Void then
				if line /= Void then
					line.set_point_a_position (figure.x, figure.y)
					if predecessor.figure /= Void then
						line.set_point_b_position (predecessor.figure.figure.x, predecessor.figure.figure.y)
					end
				end
			end
			if figure_text /= Void then
				figure_text.set_point_position(figure.x, figure.y)
			end
		end

	set_width (a_normal_width, a_highlighted_width: INTEGER) is
			-- set line widths
		do
			width_normal := a_normal_width
			width_highlighted := a_highlighted_width
		end

feature {NONE} -- Implementation

	build is
			-- Called when 'chart_point' has been provided by the renderer.
		do
			if line /= Void then
				if predecessor = Void or else predecessor.figure = Void then
					line.hide
				else
					line.show
					if not holder.has (line) then
						line.set_foreground_color (plot.color)
						line.pointer_enter_actions.wipe_out
						line.pointer_enter_actions.extend (agent on_figure_hover (line, True))
						line.pointer_leave_actions.wipe_out
						line.pointer_leave_actions.extend (agent on_figure_hover (line, False))
						holder.extend (line)
					end
				end
			end
			if chart_point.label /= Void then
				create figure_text.make_with_text (chart_point.label)
				figure_text.set_point_position(figure.x,figure.y)
				if chart_point.label_font /= Void then
					figure_text.set_font (chart_point.label_font)
				else
					figure_text.set_font (Gui.rep.fonts.new_roman_font (False,False,12))
				end
				figure_text.pointer_enter_actions.extend (agent on_figure_hover (figure_text, True))
				figure_text.pointer_leave_actions.extend (agent on_figure_hover (figure_text, False))
				figure_text.pointer_button_release_actions.extend (agent on_click)
				figure_text.pointer_double_press_actions.extend (agent on_double_click)
				-- TODO: this could be a problem? (memory leak)
				holder.extend (figure_text)
			end
			if figure /= Void then
				if not holder.has (figure) then
					figure.set_foreground_color (chart_point.color)
					figure.pointer_button_release_actions.wipe_out
					figure.pointer_button_release_actions.extend (agent on_click)
					figure.pointer_double_press_actions.wipe_out
					figure.pointer_double_press_actions.extend (agent on_double_click)
					figure.pointer_enter_actions.wipe_out
					figure.pointer_enter_actions.extend (agent on_figure_hover (figure, True))
					figure.pointer_leave_actions.wipe_out
					figure.pointer_leave_actions.extend (agent on_figure_hover (figure, False))
					holder.extend (figure)
				end
  				if figure.foreground_color /= chart_point.color then
  					figure.set_foreground_color (chart_point.color)
  				end
			end
		end

	on_figure_hover (a_figure: EV_MODEL; a_on: BOOLEAN) is
			-- event handler for pointer_enter events
		local
			l_x: STRING
			l_predecessor_x: STRING
			l_y: STRING
			l_predecessor_y: STRING
		do
			if chart_point /= Void then
				Precursor (a_figure, a_on)
				if a_on and then a_figure = line and then predecessor /= Void then
					if chart_point.point.labels.x /= Void then
						l_x := chart_point.point.labels.x.out
					else
						l_x := chart_point.x.out
					end
					if chart_point.point.labels.y /= Void then
						l_y := chart_point.point.labels.y.out
					else
						l_y := chart_point.y.out
					end
					if predecessor.point.labels.x /= Void then
						l_predecessor_x := predecessor.point.labels.x.out
					else
						l_predecessor_x := predecessor.x.out
					end
					if predecessor.point.labels.y /= Void then
						l_predecessor_y := predecessor.point.labels.y.out
					else
						l_predecessor_y := predecessor.y.out
					end
					if a_figure = line then
						--if it's a line, we set a new tooltop, otherwise the tooltip is set in precursor
						display.set_tooltip ("$1 ($2) -> ($3)",<<chart_point.render_plot.name, predecessor.full_label, chart_point.full_label>>)
					end
				end
			end
		end

feature {NONE} -- Point Shapes

	point_shape (a_index: INTEGER): EV_MODEL_ATOMIC is
		-- returns different shapes for different a_indexes
		local
			l_ellipse: EV_MODEL_ELLIPSE
			l_rectangle: EV_MODEL_RECTANGLE
		do
			inspect a_index \\ 3
			when 1 then
				create l_ellipse.make_with_positions (0, 0, 2, 2)
				--tracking doesnt work on closed figures without backgroundcolor
				l_ellipse.set_background_color (create {EV_COLOR}.default_create)
				Result := l_ellipse
			when 2 then
				create l_rectangle.make_with_positions (0, 0, 1, 1)
				--tracking doesnt work on closed figures without backgroundcolor
				l_rectangle.set_background_color (create {EV_COLOR}.default_create)
				Result := l_rectangle
			when 0 then
				create {EV_MODEL_STAR} Result.make_with_positions (0, 0, 2, 2)
			end

		end

	designe_figure_shape (a_size: INTEGER) is
			-- set `a_size' as figure size
			local
				l_rect: EV_MODEL_RECTANGLE
				l_star: EV_MODEL_STAR
				l_ellipse: EV_MODEL_ELLIPSE
				old_size: INTEGER
			do
				l_rect ?= figure
				l_star ?= figure
				l_ellipse ?= figure
				if l_rect /= void then
					l_rect.set_background_color (l_rect.foreground_color)
					l_rect.set_line_width (1)
					l_rect.set_height (a_size)
					l_rect.set_width (a_size)
				elseif l_star /= void then
					l_star.set_line_width ((a_size/5).ceiling)
					old_size := (l_star.point_b_x_relative - l_star.point_a_x_relative).abs
					if old_size /= 0 then
						l_star.scale_abs (a_size * 0.5 / old_size)
					end
				elseif  l_ellipse /= void then
					l_ellipse.set_background_color (l_ellipse.foreground_color)
					l_ellipse.set_line_width (1)
					l_ellipse.set_radius1 (((1+{MATH_CONST}.sqrt2)*a_size*0.25).ceiling)
					l_ellipse.set_radius2 (((1+{MATH_CONST}.sqrt2)*a_size*0.25).ceiling)
				end
			end

	icon_height: INTEGER is 10
			-- height of an icon
	icon_width: INTEGER is 10
			-- width of an icon

	new_coordinate (a_x, a_y: INTEGER): EV_COORDINATE is
			-- create a coordinate with `a_x' and `a_y'
		do
			create Result.set (a_x, a_y)
		end

invariant
	valid_line_widths: width_normal >= 0 and width_highlighted >= 0

end
