indexing
	description: "Widget that can display legend information"
	author: "Jann Roeder"
	date: "$Date: 2008/12/01 $"
	revision: "$Revision: #5 $"
	archive: "$File: //rose/source/kernel/infrastructure/widgets/charts/rose_chart_legend_widget.e $"
	last_review: ""

class
	ROSE_CHART_LEGEND_WIDGET

inherit

	EV_VERTICAL_BOX

create
	make_with_title

feature {NONE} -- Initialization

	make_with_title (a_title: STRING) is
			-- Creation procedure
		require
			title_not_void : a_title /= void
		do
			default_create
			create the_title
			create the_grid
			create plot_counts.make (1, 2)
			extend(the_title)
			extend(the_grid)
			disable_homogeneous
			enable_item_expand (the_grid)
			disable_item_expand (the_title)
			set_title_style (create {EV_FONT}.make_with_values ({EV_FONT_CONSTANTS}.family_sans, {EV_FONT_CONSTANTS}.weight_regular, {EV_FONT_CONSTANTS}.shape_regular,20))
			set_title( a_title )

			the_grid.set_minimum_height (50)
			setup_header
			the_grid.set_row_height (25)
			the_grid.enable_row_separators
			the_grid.enable_column_separators
			the_grid.set_separator_color (create {EV_COLOR}.make_with_8_bit_rgb (192, 192, 192))

			set_colors (Void)
		ensure
			grid_empty: the_grid.row_count = 0
			plot_count_zero: plot_counts.for_all (agent (x: INTEGER): BOOLEAN do Result := (x=0) end)
			grid_exists: the_grid /= Void
			title_exists: the_title /= Void
		end


feature -- Access

	palette: ROSE_PALETTE

feature -- Basic operations

	add_legend_item (a_index: TUPLE [INTEGER, INTEGER]; a_explanation: STRING; system_number: INTEGER) is
			-- Add an entry to the legend
		local
			l_grid_item: EV_GRID_LABEL_ITEM
		do
			plot_counts[system_number] := plot_counts[system_number] + 1
			create l_grid_item
			l_grid_item.set_pixmap (icon(a_index))
			l_grid_item.set_text (a_explanation)
			the_grid.set_item (system_number, plot_counts[system_number], l_grid_item)
		ensure
			plot_count_increased: plot_counts[system_number] = old plot_counts[system_number] + 1
		end

	clear_legend is
			-- Remove all entries
		local
			l_i : INTEGER
		do
			the_grid.clear
			the_grid.wipe_out
			from
				l_i := plot_counts.lower
			until
				l_i > plot_counts.upper
			loop
				plot_counts.item (l_i) := 0
				l_i := l_i + 1
			end
			setup_header
		ensure
			plot_count_zero: plot_counts.for_all (agent (x: INTEGER): BOOLEAN do Result := (x=0) end)
		end

	set_title (a_title: STRING) is
			-- Set the text for the label above the grid
		require
			title_not_void: a_title /= Void
		do
			the_title.set_text (a_title)
		ensure
			title_set: the_title.text.is_equal (a_title)
		end

	set_title_style (a_style: EV_FONT) is
			-- Set the style for the label above the grid
		require
			style_not_void: a_style /= Void
		do
			the_title.set_minimum_height( a_style.height + 8 )
			the_title.set_font (a_style)
		ensure
			style_set: the_title.font.is_equal (a_style)
		end


	set_colors (a_colors: ARRAY [EV_COLOR]) is
			-- Assign 'a_colors' to this legend's palette.
		do
			if palette = Void then
				create palette.make_default
			end
			if a_colors /= Void then
				palette.set_colors (a_colors)
			end
			set_palette (palette)
		end

	set_palette (a_palette: ROSE_PALETTE) is
			-- Define palette to be used
		require
			palette_not_void: a_palette /= Void
		do
			palette := a_palette
		ensure
			palette_set: palette = a_palette
		end

feature {NONE} -- Helper functions

	setup_header is
			-- Setup column titles in `the_grid'
		do
			the_grid.set_column_count_to (2)
			the_grid.header.i_th (1).set_text ("Main system")
			the_grid.header.i_th (2).set_text ("Secondary system")
			the_grid.hide_header
		end

	icon (a_id: TUPLE [INTEGER,INTEGER]): EV_PIXMAP is
			-- Icon corresponding to 'a_id', return a new object each time
			-- Do not return cloned objects, as this does not work with EV_MODELs
		local
			l_shape_id, l_color_id: INTEGER
			l_color: EV_COLOR
			l_pixmap_size: INTEGER
		do
			l_shape_id := a_id.integer_item (1)
			l_color_id := a_id.integer_item (2)
			l_color := palette.color (l_color_id)
			l_shape_id := l_shape_id \\ 7
			l_pixmap_size := 20
			create Result.make_with_size (l_pixmap_size, l_pixmap_size)
			Result.set_foreground_color (l_color)
			if l_shape_id = 1 then
				Result.fill_ellipse (0, 0, l_pixmap_size, l_pixmap_size)
			elseif l_shape_id = 2 then
				Result := draw_triangle (l_color, l_pixmap_size, False)
			elseif l_shape_id = 3 then
				Result := draw_triangle (l_color, l_pixmap_size, True)
			elseif l_shape_id = 4 then
				Result.fill_rectangle (0,0,l_pixmap_size,l_pixmap_size)
			elseif l_shape_id = 5 then
				Result := draw_diamond (l_color, l_pixmap_size)
			elseif l_shape_id = 6 then
				create Result
				Result.set_with_named_file ("excel_small.ico")
			else
				Result.fill_rectangle (0,0,l_pixmap_size,l_pixmap_size)
			end
		end

	draw_triangle (a_color: EV_COLOR; a_size: INTEGER; a_down: BOOLEAN): EV_PIXMAP is
			-- New triangle
			-- 'a_factor' of 1 returns a triangle heading down, -1 heading up
			-- if `a_down' is true, the triangle is reversed: \/ instead of /\
		require
			valid_size: a_size > 0
			a_color_valid: a_color /= Void
		do
			create Result
			Result.set_foreground_color (a_color)
			if a_down then
				Result.fill_polygon (<<
					new_coordinate (0, 0),
					new_coordinate ((a_size / 2).rounded, a_size),
					new_coordinate (0, a_size)
				>>)
			else
				Result.fill_polygon (<<
					new_coordinate (a_size, a_size),
					new_coordinate ((a_size / 2).rounded, 0),
					new_coordinate (0, a_size)
				>>)
			end
		end

	draw_diamond (a_color: EV_COLOR; a_size: INTEGER): EV_PIXMAP is
			-- New diamond
		require
			valid_size: a_size > 0
			a_color_valid: a_color /= Void
		local
			l_a_size_2: INTEGER
		do
			l_a_size_2 := (a_size/2).rounded
			create Result
			Result.set_foreground_color (a_color)
			Result.fill_polygon (<<
				new_coordinate (0, l_a_size_2),
				new_coordinate (l_a_size_2, 0),
				new_coordinate (a_size, l_a_size_2),
				new_coordinate (l_a_size_2, l_a_size_2)
			>>)
		end

	new_coordinate (a_x, a_y: INTEGER): EV_COORDINATE is
			-- New {EV_COORDINATE} with coordinates `a_x'/`a_y'
		do
			create Result.set (a_x,a_y)
		end

feature {NONE} -- Implementation

	plot_counts : ARRAY[INTEGER]
		-- Number of chart lines for each system

	the_title : EV_LABEL

	the_grid : EV_GRID

invariant
	--plot_count.max = the_grid.row_count
end
