note

	description:

		"List of ROSE_CHART_PLOTs"

	author: "Urs Doenni"
	last_editor: "$Author: c4 $"
	date: "$Date: 2013/05/07 $"
	revision: "$Revision: #13 $"
	path: "$File: //rose/source/kernel/infrastructure/widgets/charts/rose_chart_list.e $"

class
	ROSE_CHART_LIST[G -> ROSE_CHART_LIST_ITEM]

inherit
	
	LINKED_LIST[G]
		export
			{NONE} all
			{ANY} 	do_all, do_if, for_all,
					first, last, count, go_to, cursor, valid_cursor,
					extendible, prunable, readable, is_empty, new_cursor
			{ANY} start, off, item, forth, after, first_element
			{ROSE_CHART_LIST} index, compare_objects, object_comparison, extend, is_equal,
				standard_is_equal, deep_twin, changeable_comparison_criterion, --is_deep_equal,
				last_element
		redefine
			wipe_out
		end
	ANY
		undefine
			copy,
			out,
			is_equal
		end
create
	make_empty, make_with_items
create {ROSE_CHART_LIST}
	make

feature {NONE} -- Creation
	make_empty is
			-- create the group with no elements in it
		do
			create item_added_actions
			create item_removed_actions
			create wipe_out_actions
			create items_table.make (5)
			make
		end

	make_with_items (a_items: ARRAY[G]) is
			-- create the group filled with the elements in 'a_items'
		require
			items_not_void: a_items /= Void
		local
			i: INTEGER
		do
			make_empty
			from i := a_items.lower
			until i > a_items.upper
			loop
				add_item (a_items.item (i))
				i := i + 1
			end
		end

feature -- Access

	item_from_name (a_name: STRING): G is
			-- return the item with name 'a_name', Void if no item with this name
		do
			Result := items_table.item (a_name)
		ensure
			has_name_implies_found: has_name (a_name) implies Result /= Void
		end

	names: ROSE_LINEAR_ARRAY[STRING] is
			-- Names of the plots in this list
		local
			l_linkable: LINKABLE[G]
		do
			create Result.make_empty
			from
				l_linkable := first_element
			until
				l_linkable = Void
			loop
				Result.extend (l_linkable.item.name)
				l_linkable := l_linkable.right
			end
		end

	to_array: ROSE_LINEAR_ARRAY[G] is
			-- convert list to array (array only contains items, no actions or events like this list)
		local
			l_linkable: LINKABLE[G]
		do
			create Result.make_empty
			from
				l_linkable := first_element
			until
				l_linkable = Void
			loop
				Result.extend (l_linkable.item)
				l_linkable := l_linkable.right
			end
		end


feature -- Basic operations

	add_item (a_item: G) is
			-- add a item to the group
		require
			item_not_void: a_item /= Void
			item_unique: not has_name (a_item.name)
		do
			items_table.put (a_item, a_item.name)
			extend (a_item)
			a_item.set_index (count)
			item_added_actions.call ([a_item])
		end

	remove_item (a_name: STRING) is
			-- remove item with name 'a_name' from the group
		require
			name_exists: has_name (a_name)
		local
			l_item: G
		do
			l_item := items_table.item (a_name)
			items_table.remove (a_name)
			prune_all (l_item)
			item_removed_actions.call ([l_item.name])
		end

	wipe_out is
			-- wipe out the list
		do
			Precursor {LINKED_LIST}
			items_table.clear_all
			wipe_out_actions.call ([])
		end


	has_name (a_name: STRING): BOOLEAN is
			-- is there a item in the group with name 'a_name'?
		do
			Result := items_table.has (a_name)
		end

feature -- Actions

	item_added_actions: ACTION_SEQUENCE[TUPLE[G]]
			-- List of agents to be called when item is added to the list
	item_removed_actions: ACTION_SEQUENCE[TUPLE[STRING]]
			-- List of agents to be called when item is removed from the list
	wipe_out_actions: ACTION_SEQUENCE[TUPLE[]]
			-- List of agents to be called when list is wiped out

feature {NONE} -- Implementation

	items_table: HASH_TABLE[G, STRING]
			-- Hash table of all items in the list
end
