indexing
	description: 	"Widget which shows the cutout of `a_widget'"
					"Generic parameters are:"
					"P1: Main Coordinate System type of points"
					"P2: Secondary Coordinate System type of points"
	author: "Ueli Peter"
	date: "$Date: 2008/12/01 $"
	revision: "$Revision: #5 $"

class
	ROSE_CHART_MINI_WIDGET[P1 -> ROSE_CHART_POINT, P2 -> ROSE_CHART_POINT]

inherit
	ROSE_CHART_WIDGET[P1, P2]
	redefine
		on_mouse_click, on_mouse_release, on_mouse_move, on_timeout, selection, setup_widget
	end

create
	make, make_as_mini_widget, make_for_multi_chart

feature --creation

	make_as_mini_widget (a_widget: ROSE_CHART_WIDGET[P1, P2]) is
			-- creat this widget as a mini_widget with cutout of `a_widget'
		do

			default_create
			display.legend.figure.hide
			create main_widget_list.make
			if a_widget /= void then
				main_widget_list.extend (a_widget)
				a_widget.set_small_widget (current)
				adapt_cutout_rectangle (a_widget)
			end

			selection.set_selectable_area (display.drawn_area)
		end

	make_for_multi_chart (a_widget_list: LINKED_LIST[ROSE_CHART_WIDGET[P1, P2]]) is
			-- creat this widget as a mini_widget with cutout of all widgets in `a_widget_list'
		require
			widget_list_non_void: a_widget_list /= void
			list_not_empty: a_widget_list.count > 0
		do

			default_create
			display.legend.figure.hide
			main_widget_list := a_widget_list
			From
				main_widget_list.start
			until
				main_widget_list.after
			loop
				main_widget_list.item.set_small_widget (current)
				main_widget_list.forth
			end

			selection.set_selectable_area (display.drawn_area)
			adapt_cutout_rectangle (main_widget_list.first)
		end

feature -- Access
	add_main_widget (a_widget: ROSE_CHART_WIDGET[P1, P2]) is
			-- add `a_widget' to `main_widget_list'
		require
			non_void: a_widget /= void
		do
			a_widget.set_small_widget (current)
			main_widget_list.extend (a_widget)
			adapt_cutout_rectangle (a_widget)
		end

	set_vertical (a_bool: BOOLEAN) is
			-- set `vertical' to `a_bool'
		do
			vertical := a_bool
			selection.set_vertical (a_bool)
		end

	clear_main_widget_list is
			-- remove all Widgets from `main_widget_list'
		do
			create main_widget_list.make
		end


	selection: ROSE_CHART_CUTOUT_AREA

feature {ROSE_CHART_WIDGET} -- called from `main_widget'

	adapt_cutout_rectangle (a_widget: ROSE_CHART_WIDGET[P1, P2]) is
				-- set the cutout rectangle depending on view of `main_widget'
			require
				non_void_widget: a_widget /= void
			local
				l_current_main_view: ROSE_RECTANGLE
				l_x1, l_x2, l_y1, l_y2: INTEGER
			do
				if not selection.is_dragging then
					if a_widget.view.is_set then
						l_current_main_view := a_widget.view
						l_x1 := (display.left + ((display.right - display.left) * l_current_main_view.x1)).floor
						l_x2 := (display.left + ((display.right - display.left) * l_current_main_view.x2)).ceiling
						if (l_x2 - l_x1) < selection.ok_width then
							if (l_x2 + (selection.ok_width - (l_x2 - l_x1))) > display.left  then
								l_x2 := display.left
								l_x1 := display.left - selection.ok_width
							else
								l_x2 := l_x1 + selection.ok_width
							end
						end
						if vertical then
							l_y1 := display.bottom + ((display.top - display.bottom) * l_current_main_view.y2).floor
							l_y2 := display.bottom + ((display.top - display.bottom) * l_current_main_view.y1).ceiling
							if (l_y2 - l_y1) < selection.ok_height then
								if (l_y2 + (selection.ok_height - (l_y2 - l_y1))) > display.top then
									l_y2 := display.top
									l_y1 := display.top - selection.ok_height
								else
									l_y2 := l_y1 + selection.ok_width
								end
							end
						else
							l_y1 := display.bottom
							l_y2 := display.top
						end
						selection.start_selection (l_x1, l_y1)
						selection.extend_selection (l_x2, l_y2)
					else
						selection.start_selection (display.left, display.bottom)
						selection.extend_selection (display.right, display.top)
					end
					selection.finalize_selection
					state.set_needs_projection
				end
				adapt_cutout_view_from_selection
			end


feature {ROSE_GRAPH_TOOL_BAR}  -- mouse cursor agents

	on_mouse_move (a_x, a_y: INTEGER; a_x_tilt, a_y_tilt, a_pressure: DOUBLE; a_screen_x, a_screen_y: INTEGER) is
			-- Mouse moved.
		do
			mouse_x := a_x
			mouse_y := a_y

			if mouse_button_pressed = no_button_pressed then
				if selection.is_within_resize_icon (a_x, a_y) then
					pixmap.set_pointer_style (selection.resize_cursor)
				elseif selection.is_within_selected (a_x,a_y) then
					pixmap.set_pointer_style (display.hand_cursor)
				else
					pixmap.set_pointer_style (Gui.rep.pixmaps.Standard_cursor)
				end
			elseif mouse_button_pressed = 1 then
				if selection.is_selecting then
					selection.extend_selection (a_x,a_y)
				elseif selection.is_dragging then
					selection.drag_selected (a_x, a_y)
					selection.stop_dragging
					selection.finalize_selection
					adapt_cutout_view_from_selection
					selection.grab_selected (a_x, a_y)
				elseif selection.is_within_selected (a_x,a_y) then
					selection.grab_selected (a_x, a_y)
				end
			end
		end

	on_mouse_click (a_x, a_y, a_button: INTEGER; a_x_tilt, a_y_tilt, a_pressure: DOUBLE; a_screen_x, a_screen_y: INTEGER) is
			-- Mouse-key pressed.
		do
			mouse_button_pressed := a_button
			if not is_picking then
				if a_button = 1 then
					if display.legend.has_point (a_x,a_y) then
					elseif selection.is_within_resize_icon (a_x, a_y) then
						selection.start_resizing (a_x, a_y)
					elseif selection.is_within_selected (a_x,a_y) then
						adapt_cutout_view_from_selection
					elseif selection.is_within_selectable (a_x,a_y) then
					--	selection.start_selection (a_x,a_y)
					end
				end
			end
		end

	on_mouse_release (a_x, a_y, a_button: INTEGER; a_x_tilt, a_y_tilt, a_pressure: DOUBLE; a_screen_x, a_screen_y: INTEGER) is
			-- Mouse-key released.
		do
			if a_button = 1 then
				if selection.is_selecting then
					selection.finalize_selection
					adapt_cutout_view_from_selection
				end
				if selection.is_dragging then
					selection.stop_dragging
					selection.finalize_selection
					adapt_cutout_view_from_selection
				else
					is_dragging := False
				end
			end
			mouse_button_pressed := no_button_pressed
			set_is_picking_imp (gui.is_picking)
		end

feature {None} -- implementation

	setup_widget is
			-- setup widget
		do
			create selection.make

			create events.make
			create state
			create display.make (Current)
			state := display.state
			pixmap := display.pixmap
			create world
			create view.make_normal
				-- Pixmap (coming from the display)
			resize_actions.extend (agent on_resize)
			pixmap.pointer_motion_actions.extend (agent on_mouse_move)
			pixmap.pointer_button_press_actions.extend (agent on_mouse_click)
			pixmap.pointer_button_release_actions.extend (agent on_mouse_release)

			put (pixmap)
				-- Default settings
			create timeout.make_with_interval (100)
			timeout.actions.extend (agent on_timeout_event)
			selection.set_selectable_area (display.selection.selectable_area)
			initialize_widget
			clear_all
		end


	adapt_cutout_view_from_selection is
			-- set `main_widget' s view dependent of selection
		local
			l_rec: ROSE_RECTANGLE
		do
			if selection.selected /= void then
				l_rec := deep_clone (selection.normal_selected)
				l_rec.set_y (1 - l_rec.y1, 1 - l_rec.y2)
				from
					main_widget_list.start
				until
					main_widget_list.after
				loop
					main_widget_list.item.view.set_from_rectangle (l_rec)
					main_widget_list.item.state.set_needs_projection
					main_widget_list.forth
				end
			end
		end

	on_timeout is
			-- draw rectangle
		do
			precursor
			if selection.selected = void and main_widget_list.count > 0 then
				adapt_cutout_rectangle (main_widget_list.first)
			end
			adapt_cutout_view_from_selection
		end



	main_widget_list: LINKED_LIST[ROSE_CHART_WIDGET[P1, P2]]
			-- the widget on which this mini widget depends

	vertical: BOOLEAN
			-- is vertical sliding possible?
end
