indexing
	description: "List of ROSE_CHART_POINTs"
	author: "Urs Doenni"
	date: "$Date: 2009/03/24 $"
	revision: "$Revision: #19 $"
	archive: "$File: //rose/source/kernel/infrastructure/widgets/charts/rose_chart_plot.e $"

deferred class
	ROSE_CHART_PLOT [P -> ROSE_CHART_POINT]

inherit
	ROSE_CHART_TWO_WAY_LIST[P]
		rename
			make as make_chart_list
		redefine
			remove
		end

	ROSE_CHART_LIST_ITEM

feature {NONE} -- Creation

	make_empty (a_name: like name) is
			-- Create an empty plot
		require
			name_not_void: a_name /= Void
		do
			make_chart_list
			set_name (a_name)
			-- TODO: what to do with drop actions?
			create drop_actions
			extend_back_actions.extend (agent set_plot (?))
			extend_front_actions.extend (agent set_plot (?))
		end

feature -- Basic operations

	update_points (a_criterion: like update_criterion) is
			-- update all data in the sequence using 'a_criterion'
		do
			do_all (agent update_point (?, a_criterion))
		end

	update_points_if (a_criterion: like update_criterion; a_test: FUNCTION [ANY, TUPLE [P], BOOLEAN]) is
			-- update all data in the sequence using 'a_criterion' if the points fulfill 'a_test'
		do
			do_if (agent update_point (?, a_criterion),	a_test)
		end

	remove (a_key: HASHABLE) is
			-- remove point from the sequence
		do
			Precursor {ROSE_CHART_TWO_WAY_LIST}(a_key)
			do_all (agent wipe_out_change_actions (?))
		end


feature -- Access

	data: ANY
			-- Associated data

	set_data (a_data: like data) is
			-- set the data of this plot
		do
			data := a_data
		end

	labels (a_index: INTEGER): ROSE_LINEAR_ARRAY[ANY] is
			-- Array with all labels from all points.
			-- Label is taken from '{ROSE_CHART_POINT}.labels.item (a_index)'
		local
			l_linkable: LINKABLE[ROSE_CHART_POINT]
			l_labels: TUPLE[ANY]
			l_label: ANY
		do
			create Result.make_empty
			from
				l_linkable := first_element
			until
				l_linkable = Void
			loop
				l_labels := l_linkable.item.labels
				if l_labels /= Void and then l_labels.valid_index (a_index) then
					l_label := l_labels.item (a_index)
				end
				if l_label /= Void then
					Result.extend (l_label)
				else
					Result.extend ("")
				end
				l_linkable := l_linkable.right
			end
		end

	values (a_index: INTEGER): ROSE_LINEAR_ARRAY[DOUBLE] is
			-- Array with values from all points.
			-- Value is taken from '{ROSE_CHART_POINT}.values.item (a_index)'
		local
			l_linkable: LINKABLE[ROSE_CHART_POINT]
			l_values: TUPLE[DOUBLE]
			l_value: DOUBLE
		do
			create Result.make_empty
			from
				l_linkable := first_element
			until
				l_linkable = Void
			loop
				l_values := l_linkable.item.values
				l_value := 0
				if l_values /= Void and then l_values.valid_index (a_index) and then l_values.is_double_item (a_index) then
					l_value := l_values.double_item (a_index)
				end
				Result.extend (l_value)
				l_linkable := l_linkable.right
			end
		end


feature -- Basic operations

		get_data_array: ARRAY2[STRING] is
				-- return all points in this plot as string arrays
			local
				i, e: INTEGER
				l_row: ARRAY[STRING]
			do
				if count > 1 then
					create Result.make (1, count + 1)
					create Result.make (count + 1, first.get_data_array.count)
					from
						e := 1
						l_row := first.get_data_description
					until
						e > l_row.count
					loop
						Result.put (l_row.item (e), 1, e)
						e := e + 1
					end
					from
						i := 1
					until
						i > count
					loop
						from
							e := 1
							l_row := i_th (i).get_data_array
						until
							e > l_row.count
						loop
							Result.put (l_row.item (e), i + 1, e)
							e := e + 1
						end
						i := i + 1
					end
				elseif i > 0 then
					create Result.make (1, 1)
				end
			end


feature -- Actions

	drop_actions: EV_PND_ACTION_SEQUENCE
			-- List of agents to be called when something is dropped on the plot

feature {ROSE_CHART_COORDINATE_SYSTEM} -- Plot list

	get_new_plot_list_instance: ROSE_CHART_PLOT_LIST[ROSE_CHART_PLOT[P]] is
			-- Children have to implement this feature to make the feature
			-- {ROSE_CHART_COORINATE_SYSTEM}.set_plot_list
			-- work
		deferred
		ensure
			instance_created: Result /= Void
		end


feature {NONE} -- Agents

	update_point (a_point: P; a_crit: like update_criterion) is
			-- update the point if it has an update agent
		do
			a_point.update (a_crit)
		end

	wipe_out_change_actions (a_point: p) is
			-- wipe out change actions. Reason: when the plot gets removed, the corresponding
			-- render plot is removed too, but its points are still referenced through change_actions
		do
			a_point.change_actions.wipe_out
		end

	set_plot (a_point: P) is
			-- set the plot for 'a_point'
		do
			if a_point /= Void then
				a_point.set_plot (Current)
			end
		end



feature {NONE} -- Types

	update_criterion: TUPLE[ANY] is
			-- Type of update criterion
		once
		end

invariant
	name_set: name /= Void

end
