indexing
	description: "List of ROSE_CHART_PLOTs"
	author: "Urs Doenni"
	date: "$Date: 2008/12/01 $"
	revision: "$Revision: #5 $"
	archive: "$File: //rose/source/kernel/infrastructure/widgets/charts/rose_chart_plot_list.e $"

deferred class
	ROSE_CHART_PLOT_LIST[P -> ROSE_CHART_PLOT[ROSE_CHART_POINT]]


inherit
	ROSE_CHART_LIST[P]
	rename
		make_with_items as make_with_plots,
		item_from_name as plot_from_name,
		add_item as add_plot,
		remove_item as remove_plot,
		item_added_actions as plot_added_actions,
		item_removed_actions as plot_removed_actions
	end


feature -- Actions

	drop_actions: EV_PND_ACTION_SEQUENCE is
			-- actions to be taken when pebble is dropped on the list (via the legend)
		do
			if drop_actions_imp = Void then
				create drop_actions_imp
			end
			Result := drop_actions_imp
		end

feature -- Basic operations

	get_data_array: ARRAY[ARRAY2[STRING]] is
			-- return array with all plots of this plot-list
		local
			i: INTEGER
		do
			create Result.make (1, count)
			from
				i := 1
			until
				i > count
			loop
				Result.put (i_th (i).get_data_array, i)
				i := i + 1
			end
		end

	get_name_array: ARRAY[STRING] is
			-- return the names of the plots as array
		local
			i: INTEGER
		do
			create REsult.make (1, count)
			from
				i :=1
			until
				i > count
			loop
				Result.put (i_th (i).name, i)
				i := i + 1
			end
		end

feature -- Agents

	plot_added_actions_extend (a_agent: PROCEDURE [ANY, TUPLE [ROSE_CHART_PLOT [ROSE_CHART_POINT]]])
			-- Add action to `plot_added_action'.
			-- The difference between this and `plot_added_actions.extend' is that the type
			-- of the item is the base type here and not the type of P, which can be useful
			-- to pass a more general agent without introducing catcalls.
		require
			a_agent_not_void: a_agent /= Void
		do
			plot_added_actions.extend (agent base_typed_plot_added_agent (a_agent, ?))
		end


feature {NONE} -- Implementation

	drop_actions_imp: like drop_actions
			-- Stored version of `drop_actions'

	base_typed_plot_added_agent (a_agent: PROCEDURE [ANY, TUPLE [ROSE_CHART_PLOT [ROSE_CHART_POINT]]]; a_item: P)
			-- Implementation for `plot_added_actions_extend', agent for type
			-- conversion between P and the type of the constraint on P.
		require
			a_agent_not_void: a_agent /= Void
		do
			a_agent.call ([a_item])
		end

end
