indexing
	description: "Parent class for all points of ROSE_CHART_*"
	author: "Urs Doenni"
	date: "$Date: 2008/12/01 $"
	revision: "$Revision: #15 $"
	archive: "$File: //rose/source/kernel/infrastructure/widgets/charts/rose_chart_point.e $"

deferred class
	ROSE_CHART_POINT

inherit
	KEYED

feature {NONE} -- Creation

	make (a_values: like values; a_key: like key) is
			-- create the point
		do
			make_with_labels (a_values, Void, a_key)
		end

	make_with_labels (a_values: like values; a_labels: like labels; a_key: like key) is
			-- create point with values and labels
		do
			create change_actions

			values := a_values
			if values = Void then
				create values
			end
			labels := a_labels
			if labels = Void then
				create labels
			end
			key := a_key

			is_set := True
		end

feature -- Access

	values: TUPLE[DOUBLE]
		-- this points values

	labels: TUPLE[ANY]
		-- labels for this point

	update_agent: PROCEDURE[ANY, TUPLE[like Current, like update_criterion]]

	pebble_agent: FUNCTION [ANY, TUPLE, ANY]

	pebble: ANY is
			-- pebble for this point
		do
			if pebble_agent /= Void then
				Result := pebble_agent.item ([])
			end
		end

	tooltip_agent: FUNCTION[ANY, TUPLE[like Current], STRING]

	tooltip: STRING is
			-- tooltip for this point
		do
			if tooltip_agent /= Void then
				Result := tooltip_agent.item ([Current])
			end
		end

	color: EV_COLOR
			-- color of the point (render point uses this when when it's created if it's not void)

	label_font: EV_FONT
			-- Default font for label (render point uses this when when it's created if it's not void)

	full_label: STRING
			-- Full label of the point (render point uses this when when it's created if it's not void)

	plot: ROSE_CHART_PLOT[like Current]

	is_set: BOOLEAN

	unset is
			-- set is_set to false
		do
			is_set := False
		end

feature -- Basic operations

	set_pebble_agent (a_agent: like pebble_agent) is
			-- set the agent which generates a pebble
		do
			pebble_agent := a_agent
		end

	set_tooltip_agent (a_agent: like tooltip_agent) is
			-- set the agent which generates the tooltip
		do
			tooltip_agent := a_agent
		end

	set_color (a_color: like color) is
			-- set the color
		do
			color := a_color
		end

	set_full_label (a_label: like full_label) is
			-- set 'full_label'
		do
			full_label := a_label
		end

	set_label_font (a_font: like label_font) is
			-- set 'label_font'
		do
			label_font := a_font
		end

	set_update_agent (a_agent: like update_agent) is
			-- set the update agent
		do
			update_agent := a_agent
		end

	update (a_update_criterion: like update_criterion) is
			-- Update the point using 'a_update_criterion'
		do
			if update_agent /= Void then
				update_agent.call ([Current, a_update_criterion])
				change_actions.call ([Current])
			end
		end

	get_data_array: ARRAY[STRING] is
			-- returns all data to this point as array of strings
		deferred
		end

	get_data_description: ARRAY[STRING]	is
			-- returns a description to the data in get_data_array
		deferred
		end


feature -- Data

	data: ANY

	set_data (a_data: like data) is
			-- set the data of this plot
		do
			data := a_data
		end

feature -- Actions
	change_actions: ACTION_SEQUENCE[TUPLE[]]

feature {ROSE_CHART_PLOT}  -- basic operations for ROSE_CHART_PLOT

	set_plot (a_plot: like plot) is
			-- set the plot this point belongs to
		do
			plot := a_plot
		end


feature {NONE} -- Implementation

	update_criterion: TUPLE[ANY]

end
