indexing
	description: "Class that handles projection of a list of projectables into a view"
	author: "Urs Doenni"
	date: "$Date: 2014/02/26 $"
	revision: "$Revision: #4 $"
	archive: "$File: //rose/source/kernel/infrastructure/widgets/charts/rose_chart_projector.e $"

class
	ROSE_CHART_PROJECTOR


feature -- Implementation

	project_list (a_list: LINKED_LIST[ROSE_CHART_PROJECTABLE]; a_cs: ROSE_CHART_COORDINATE_SYSTEM[ROSE_CHART_POINT]) is
			-- project all elements in the list
		local
			l_viewed: ROSE_RECTANGLE
			l_x0, l_y0, l_x_scale, l_y_scale, w, h: DOUBLE
		do
			if a_cs.world.is_set and then a_list.count > 0 then
				l_viewed := calculate_viewed ( a_cs)
				w := l_viewed.width
				h := l_viewed.height
				if w > 0 and then h > 0 then
					l_x0 := l_viewed.x1
					l_y0 := l_viewed.y1
					l_x_scale := a_cs.display.drawn_area.width / w
					l_y_scale := a_cs.display.drawn_area.height / h
					a_cs.x_axis.set_projected_origin (a_cs.y_axis.projected (a_cs.y_axis.line_origin,l_y0,l_y_scale))
					a_cs.y_axis.set_projected_origin (a_cs.x_axis.projected (a_cs.x_axis.line_origin,l_x0,l_x_scale))
					a_cs.x_axis.project (l_x0,l_x_scale)
					a_cs.y_axis.project (l_y0,l_y_scale)

					from
						a_list.start
					until
						a_list.after
					loop
						a_list.item.project (l_x0,l_y0,l_x_scale,l_y_scale)
						a_list.forth
					end
				end
			end
		end

	project_zone_list (a_list: LINKED_LIST[ROSE_CHART_PROJECTABLE]; a_cs: ROSE_CHART_COORDINATE_SYSTEM[ROSE_CHART_POINT]) is
			-- project all elements in the list using the whole high of the Display
		local
			l_viewed: ROSE_RECTANGLE
			l_x0, l_y0, l_x_scale, l_y_scale, w, h: DOUBLE
		do
			if a_cs.world.is_set and then a_list.count > 0 then
				l_viewed := calculate_viewed ( a_cs)
				w := l_viewed.width
				h := l_viewed.height
				if w > 0 and then h > 0 then
					l_x0 := l_viewed.x1
					l_y0 := l_viewed.y1
					l_x_scale := a_cs.display.drawn_area.width / w
					l_y_scale := a_cs.display.drawn_area.height / h
					a_cs.x_axis.set_projected_origin (a_cs.y_axis.projected (a_cs.y_axis.line_origin,l_y0,l_y_scale))
					a_cs.y_axis.set_projected_origin (a_cs.x_axis.projected (a_cs.x_axis.line_origin,l_x0,l_x_scale))
					a_cs.x_axis.project (l_x0,l_x_scale)
					a_cs.y_axis.project (l_y0,l_y_scale)

					from
						a_list.start
					until
						a_list.after
					loop
						a_list.item.project (l_x0,0,l_x_scale, a_cs.display.drawn_area.height)
						a_list.forth
					end
				end
			end
		end


	calculate_viewed (a_cs: ROSE_CHART_COORDINATE_SYSTEM[ROSE_CHART_POINT]): ROSE_RECTANGLE is
			-- Calculates the viewed area
		local
			l_max_view: ROSE_RECTANGLE
			l_outside_right, l_outside_left, l_outside_bottom, l_outside_top: DOUBLE
		do
			-- If someone set the view of the coordinate system, we always
			-- use this instead of the world
			if a_cs.view_imp.is_set then
				create Result.set_from_rectangle(a_cs.view_imp.relative_rectangle (a_cs.widget.view))
			else
				create Result.set_from_rectangle(a_cs.world.relative_rectangle (a_cs.widget.view))
			end

			-- Limit the view to the world
			create l_max_view.set_from_rectangle (a_cs.world)

			l_outside_right := l_max_view.x2 - Result.x2
			l_outside_bottom := l_max_view.y2 - Result.y2

			if l_outside_right < 0 then
				Result.translate (l_outside_right, 0)
			end
			if l_outside_bottom < 0 then
				Result.translate (0, l_outside_bottom)
			end

			l_outside_left := l_max_view.x1 - Result.x1
			l_outside_top := l_max_view.y1 - Result.y1

			if l_outside_left > 0 then
				Result.translate (l_outside_left, 0)
			end
			if l_outside_top > 0 then
				Result.translate (0, l_outside_top)
			end

			if Result.width > l_max_view.width then
				Result.set_size (l_max_view.width, Result.height)
			end
			if Result.height > l_max_view.height then
				Result.set_size (Result.width, l_max_view.height)
			end

			if a_cs.view_imp.is_set then
				a_cs.widget.view.set_from_rectangle (a_cs.view_imp.normalized_rectangle (Result))
			else
				a_cs.widget.view.set_from_rectangle (a_cs.world.normalized_rectangle (Result))

			end
		end

end
