indexing
	description: "Render point capable of rendering points of type ROSE_CHART_2D_POINT"
	author: "Urs Doenni"
	last_editor: "$Author: it $"
	date: "$Date: 2008/12/01 $"
	revision: "$Revision: #9 $"
	path: "$File: //rose/source/kernel/infrastructure/widgets/charts/rose_chart_render_2d_point.e $"

class
	ROSE_CHART_RENDER_2D_POINT

inherit
	ROSE_CHART_RENDER_POINT[ROSE_CHART_2D_POINT]
		redefine
			full_label,
			figure
		end

	create
	make

feature {NONE} -- Initialization

	initialize (a_point: ROSE_CHART_2D_POINT) is
			-- Create a point at 'a_index' and given 'a_y' coordinate.
		do
			given_x := a_point.values.x
			given_y := a_point.values.y
			set_color (a_point.color)
		end

	update is
			-- Update the point
		do
			initialize (point)
		end

feature -- Access

	figure: ROSE_CHART_FIGURE
			-- Figure used to represent this point

	are_values_defined: BOOLEAN is
			-- says if the values that make up this points are defined
		do
			Result := is_defined_double (given_x) and is_defined_double (given_y)
		end

	given_x, given_y: DOUBLE
			-- Original given X and Y coordinates

	x, y: DOUBLE
			-- Actual coordinates of this point in corresponding coordinate system
			-- These can be logs if 'is_logarithmic' is set on the axis for example

	debug_output: STRING is
			-- String that should be displayed in debugger to represent `Current'.
		do
			create Result.make (20)
			Result.append ("[" + point.key.out + "] " + x.out + " , " + y.out)
		end

	full_label: STRING is
			-- A fully specified display string
		local
			l_x, l_y: STRING
			l_date: ROSE_DATE
		do
			create Result.make (20)
			if label /= Void then
				Result.append(label + ", ")
			end
			if point.labels.x /= Void then
				l_date ?= point.labels.x
				if l_date /= Void then
					if l_date.persecs = 0 then
						l_x := l_date.formatted_out ("yyyy/[0]mm/[0]dd")
					else
						l_x := l_date.formatted_out ("yyyy/[0]mm/[0]dd [0]hh:[0]mi:[0]ss")
					end
				else
					l_x := point.labels.x.out
				end
			else
				l_x := format_double.formatted (given_x)
			end
			if point.labels.y /= Void then
				l_y := point.labels.y.out
			else
				l_y := format_double.formatted (given_y)
			end
			Result.append (l_x + ", " + l_y)
		end

feature -- Basic operations

	coordinate (a_x_axis, a_y_axis: ROSE_CHART_AXIS; a_x_stats, a_y_stats: ROSE_MOMENTS) is
			-- Coordinate x and y values
		do
			x := coordinated_value (given_x, a_x_axis, a_x_stats)
			y := coordinated_value (given_y, a_y_axis, a_y_stats)
		end

	project (a_x0, a_y0, a_x_scale, a_y_scale: DOUBLE) is
			-- Project this point for origin ('a_x0,'a_y0')
			-- and with given scales.
		do
			if figure /= Void then
				figure.project (x,y,a_x0,a_y0,a_x_scale,a_y_scale)
			end
		end

feature {NONE} -- Implementation

	format_double: FORMAT_DOUBLE is
			-- Object for double data display
		once
			create Result.make (10,6)
			Result.comma_separate
			Result.no_separate_after_decimal
		end

end
