note 

	description: "[
                  Render point capable of rendering points of type ROSE_CHART_3D_POINT.

                  The Z-axis value is represented by a color from a gradient.
                 ]"

	author: "Colin Adams"
	last_editor: "$Author: c4 $"
	date: "$Date: 2013/05/15 $"
	revision: "$Revision: #1 $"
	path: "$File: //rose/source/kernel/infrastructure/widgets/charts/rose_chart_render_3d_point.e $"

class	ROSE_CHART_RENDER_3D_POINT

inherit

	ROSE_CHART_RENDER_POINT [ROSE_CHART_3D_POINT]
		rename
			make as make_point
		redefine
			full_label,
			figure
		end

create
	
	make

feature {NONE} -- Initialization
	
	make (a_point: ROSE_CHART_3D_POINT; a_gradient: like gradient) is
			-- Create `Current' using `a_point' for coordinates and `a_gradient' to color the point'
		require
			a_point_not_void: a_point /= Void
			a_gradient_not_void: a_gradient /= Void
		do

			gradient := a_gradient
			make_point (a_point)
		ensure
			point_aliased: point = a_point
			gradient_aliased: gradient = a_gradient
		end

	initialize (a_point: ROSE_CHART_3D_POINT) is
			-- Create using `a_point' 
		do
			given_x := a_point.x
			given_y := a_point.y
			if is_defined_double (a_point.z) then
				set_color (gradient.get_color_from_value (a_point.z))
			end
		end

	update is
			-- Update the point
		do
			initialize (point)
		end

feature -- Access
	
	gradient: ROSE_CHART_COLOR_FUNCTION
			-- Map of z-values to colors

	figure: ROSE_CHART_FIGURE
			-- Figure used to represent this point

	are_values_defined: BOOLEAN is
			-- Says if the values that make up this points are defined
		do
			Result := is_defined_double (given_x) and is_defined_double (given_y) and
				is_defined_double (point.z)
		end

	given_x, given_y: DOUBLE
			-- Original given X and Y coordinates

	x, y: DOUBLE
			-- Actual coordinates of this point in corresponding coordinate system
			-- These can be logs if 'is_logarithmic' is set on the axis for example

	debug_output: STRING is
			-- String that should be displayed in debugger to represent `Current'.
		do
			create Result.make (30)
			Result.append ("[" + point.key.out + "] " + x.out + " , " + y.out + " , " + point.z.out)
		end

	full_label: STRING is
			-- A fully specified display string
		local
			l_x, l_y, l_z: STRING
			l_date: ROSE_DATE
		do
			create Result.make (20)
			if label /= Void then
				Result.append(label + ", ")
			end
			if point.labels.x /= Void then
				l_date ?= point.labels.x
				if l_date /= Void then
					if l_date.persecs = 0 then
						l_x := l_date.formatted_out ("yyyy/[0]mm/[0]dd")
					else
						l_x := l_date.formatted_out ("yyyy/[0]mm/[0]dd [0]hh:[0]mi:[0]ss")
					end
				else
					l_x := point.labels.x.out
				end
			else
				l_x := format_double.formatted (given_x)
			end
			if point.labels.y /= Void then
				l_y := point.labels.y.out
			else
				l_y := format_double.formatted (given_y)
			end
			if point.labels.z /= Void then
				l_z := point.labels.z.out
			else
				l_z := format_double.formatted (point.z)
			end
			Result.append (l_x + ", " + l_y + ", " + l_z)
		end

feature -- Basic operations

	coordinate (a_x_axis, a_y_axis: ROSE_CHART_AXIS; a_x_stats, a_y_stats: ROSE_MOMENTS) is
			-- Coordinate x and y values.
		do
			x := coordinated_value (given_x, a_x_axis, a_x_stats)
			y := coordinated_value (given_y, a_y_axis, a_y_stats)
		end

	project (a_x0, a_y0, a_x_scale, a_y_scale: DOUBLE) is
			-- Project this point for origin ('a_x0,'a_y0')
			-- and with given scales.
		do
			if figure /= Void then
				figure.project (x,y,a_x0,a_y0,a_x_scale,a_y_scale)
			end
		end

feature {NONE} -- Implementation

	format_double: FORMAT_DOUBLE is
			-- Object for double data display
		once
			create Result.make (10,6)
			Result.comma_separate
			Result.no_separate_after_decimal
		end
	
invariant

	gradient_not_void: gradient /= Void

end
