indexing
	description: "Render point capable of rendering points of type ROSE_CHART_BOXPLOT_POINT"
	author: "Urs Doenni"
	date: "$Date: 2009/11/16 $"
	revision: "$Revision: #4 $"

class
	ROSE_CHART_RENDER_BOXPLOT_POINT

inherit
	ROSE_CHART_RENDER_POINT[ROSE_CHART_BOXPLOT_POINT]
	redefine
		full_label,
		figure
	end

create
	make

feature {NONE} -- Initialization

	initialize (a_point: ROSE_CHART_BOXPLOT_POINT) is
			-- Create a point at 'a_index' and given 'a_y' coordinate.
		do
			given_x := a_point.values.x

			given_min := a_point.values.min
			given_p5 := a_point.values.p5
			given_p25 := a_point.values.p25
			given_median := a_point.values.median
			given_p75 := a_point.values.p75
			given_p95 := a_point.values.p95
			given_max := a_point.values.max
		end

	update is
			-- Update the point
		do
			initialize (point)
		end


feature -- Access

	figure: ROSE_CHART_FIGURE_BOXPLOT
			-- Figure used to represent this point

	are_values_defined: BOOLEAN is
			-- says if the values that make up this points are defined
		do
			Result :=
				is_defined_double (given_x) and
				is_defined_double (given_min) and
				is_defined_double (given_p5) and
				is_defined_double (given_p25) and
				is_defined_double (given_median) and
				is_defined_double (given_p75) and
				is_defined_double (given_p95) and
				is_defined_double (given_max)
		end

	given_x, given_min, given_p5, given_p25, given_median, given_p75, given_p95, given_max: DOUBLE
		-- Original given coordinates

	x, min, p5, p25, median, p75, p95, max: DOUBLE
			-- Actual coordinates of this point in corresponding coordinate system
			-- These can be logs if 'is_logarithmic' is set on the axis for example

	debug_output: STRING is
			-- String that should be displayed in debugger to represent `Current'.
		do
			create Result.make (20)
			Result.append ("[" + point.key.out + "] " + point.values.out)
		end

	full_label : STRING is
			-- Full label
		do
			create Result.make (20)
			if label /= Void then
				Result.append(label + ", ")
			end
			Result.append (point.values.out)
		ensure then
			valid_result : Result /= Void
		end

feature -- Basic operations

	coordinate(a_x_axis, a_y_axis: ROSE_CHART_AXIS; a_x_stats, a_y_stats: ROSE_MOMENTS) is
			-- Coordinate all values
		do
			x := coordinated_value (given_x, a_x_axis, a_x_stats)

			min := coordinated_value (given_min, a_y_axis, a_y_stats)
			p5 := coordinated_value (given_p5, a_y_axis, a_y_stats)
			p25 := coordinated_value (given_p25, a_y_axis, a_y_stats)
			median := coordinated_value (given_median, a_y_axis, a_y_stats)
			p75 := coordinated_value (given_p75, a_y_axis, a_y_stats)
			p95 := coordinated_value (given_p95, a_y_axis, a_y_stats)
			max := coordinated_value (given_max, a_y_axis, a_y_stats)
		end

	project (a_x0, a_y0, a_x_scale, a_y_scale: DOUBLE) is
			-- Project this point for origin ('a_x0,'a_y0')
			-- and with given scales.
		do
			if figure /= Void then
				figure.project_with_point (Current, a_x0,a_y0,a_x_scale,a_y_scale)
			end
		end
end
