indexing
	description: "Chart renderer - represents charts with figures"
	author: "Zoran Simic"
	date: "$Date: 2009/03/25 $"
	revision: "$Revision: #10 $"

deferred class
	ROSE_CHART_RENDERER[P -> ROSE_CHART_POINT]


inherit
	ROSE_CHART_CONSTANTS

feature -- Access

	debug_output: STRING is
			-- String that should be displayed in debugger to represent `Current'.
		do
			Result := generator
		end

feature {ROSE_CHART_COORDINATE_SYSTEM} -- Typed factory for render points

	new_render_plot (a_plot: ROSE_CHART_PLOT [P]): ROSE_CHART_RENDER_PLOT [P] is
			-- create a render plot with the right types in synch with renderer
		require
			a_plot_not_void: a_plot /= Void
		do
			create Result.make (a_plot, Current)
		ensure
			not_void: Result /= Void
		end

	new_render_plot_list: ROSE_CHART_LIST [ROSE_CHART_RENDER_PLOT [P]] is
			-- create empty render plot list
		do
			create Result.make_empty
		ensure
			not_void: Result /= Void
			empty: Result.is_empty
		end

feature -- Basic operations

	new_render_point (a_point: P):  like render_point_type is
			-- create a new render_point
		deferred
		end

	render (a_plots: ROSE_CHART_LIST [ROSE_CHART_RENDER_PLOT [P]]) is
			-- Render all figures in 'a_system'.
		require
			valid_plots: a_plots /= Void
		local
			l_linkable: LINKABLE [ROSE_CHART_RENDER_PLOT[P]]
		do
			from
				l_linkable := a_plots.first_element
			until
				l_linkable = Void
			loop
				render_plot_if_requested (l_linkable.item)
				l_linkable := l_linkable.right
			end
		end

	render_plot_if_requested (a_plot: ROSE_CHART_RENDER_PLOT [P]) is
			-- Render representation of 'a_plot' if needed.
		require
			valid_plot: a_plot /= Void
		do
			if a_plot.holder.is_show_requested then
				render_plot (a_plot)
			end
		end

	render_plot (a_plot: ROSE_CHART_RENDER_PLOT[P]) is
			-- Render figures to represent 'a_plot'.
		require
			valid_plot: a_plot /= Void
		local
			l_points: ROSE_CHART_TWO_WAY_LIST [ROSE_CHART_RENDER_POINT[P]]
			l_point, l_previous_point:  ROSE_CHART_RENDER_POINT[P]
			l_figure: ROSE_CHART_FIGURE
		do
			from
				l_points := a_plot.points
				l_points.start
			until
				l_points.after
			loop
				l_point := l_points.item
				if l_point.figure /= Void then
					-- TODO: move build out of rendering (in rendering the figure should only be created)
					l_point.figure.build
				elseif l_point.are_values_defined then
					l_figure := new_figure (l_point)
					l_point.set_figure (l_figure)
					l_figure.set_chart_point (l_previous_point, l_point)
				end
				l_previous_point := l_point
				l_points.forth
			end
		end

	new_figure (a_point : ROSE_CHART_RENDER_POINT[ROSE_CHART_POINT]): ROSE_CHART_FIGURE is
			-- New figure.
		deferred
		ensure
			not_void : Result /= Void
		end

feature {NONE} -- Types

	render_point_type: ROSE_CHART_RENDER_POINT[P] is
			-- type of the render point
			-- (Redefining children should only redefine the returned type and implement the body void as it is not used)
		deferred
		end

end -- class ROSE_CHART_RENDERER
