note

	description:

		"Scatter plot renderer for 3D plots using a color gradient for the z-axis"

	author: "Colin Adams"
	last_editor: "$Author: c4 $"
	date: "$Date: 2013/06/13 $"
	revision: "$Revision: #2 $"
	path: "$File: //rose/source/kernel/infrastructure/widgets/charts/rose_chart_renderer_color_plot.e $"

class	ROSE_CHART_RENDERER_COLOR_PLOT

inherit
	
	ROSE_CHART_RENDERER [ROSE_CHART_3D_POINT]
		redefine
			render_point_type, render
		end
	
	ROSE_STOCK_COLORS
		export {NONE} all end

create
	
	make, make_rainbow
	
feature {NONE} -- Initalization

	make (a_gradient: like gradient) is
			-- Create `Current' using `a_gradient' to color points according to their z-value'
		require
			a_gradient_not_void: a_gradient /= Void
		do

			gradient := a_gradient
			pixmap_needs_redraw := True
		ensure
			gradient_aliased: gradient = a_gradient
		end
	
	make_rainbow (a_lower, a_upper: DOUBLE) is
			-- Create `Current' using a color gradient of the visible spectrum over the range `a_lower' |..| `a_upper'.
		require
			valid_range: a_lower <= a_upper
		do
			make (create {ROSE_CHART_COLOR_FUNCTION}.make_default (a_lower, a_upper))
		end

feature -- Access

	gradient: ROSE_CHART_COLOR_FUNCTION
		-- Map of z-values to colors
	
feature -- Setting

	set_gradient (a_gradient: like gradient) is
			-- Set `gradient' to `a_gradient'.
		require
			a_gradient_not_void: a_gradient /= Void
		do
			gradient := a_gradient
		ensure
			gradient_aliased: gradient = a_gradient
		end

feature -- Basic operations
	
	render (a_plots: ROSE_CHART_LIST [ROSE_CHART_RENDER_PLOT [ROSE_CHART_3D_POINT]]) is
			-- <Precursor>
			-- Redefined for custom legend.
		local
			l_system: ROSE_CHART_COORDINATE_SYSTEM [ROSE_CHART_POINT]
			l_model: EV_MODEL_PICTURE
		do
			if a_plots.count > 0 then
				l_system := a_plots.first.coordinate_system
				l_system.display.legend.wipe_out
				if pixmap_needs_redraw then
					build_legend
				end
				create l_model.make_with_pixmap (legend_pixmap)
				l_system.display.legend.add_figure_explanation (l_model, "")
				l_system.display.legend.set_icon_size (legend_pixmap.width, legend_pixmap.height)
				l_system.display.set_legend_position (position_right)
				l_system.display.set_legend_alignment (align_center)
			end
			Precursor (a_plots)
		end
	
	new_render_point (a_point: ROSE_CHART_3D_POINT):  like render_point_type is
			-- <Precursor>
		do
			create Result.make (a_point, gradient)
		end

	new_figure (a_point: ROSE_CHART_RENDER_POINT [ROSE_CHART_POINT]): ROSE_CHART_FIGURE_LINE is
			-- New figure.
		do
			create Result.make (False, True, a_point.render_plot.index, 0, 0)
		end

feature {NONE} --Templates
	
	render_point_type: ROSE_CHART_RENDER_3D_POINT is
			-- Template
		do
		end
	
feature {NONE} -- Legend
	
	legend_pixmap: EV_PIXMAP
			-- Pixmap drawn into legend

	pixmap_needs_redraw: BOOLEAN
			-- Have some values changed which affect the legend?
	
	build_legend is
			-- Create legend as pixmap and store in `legend_pixmap'.
		local
			i: INTEGER
		do
			--Font width of EV_PIXMAP is assumed to be less than 6
			create legend_pixmap.make_with_size (15 + 6 * 12, 140)
			legend_pixmap.draw_rectangle (0, 0, legend_pixmap.width, legend_pixmap.height)

			from
				i := 1
			until
				i > 100
			loop
				legend_pixmap.set_foreground_color (black)
				legend_pixmap.draw_text (5, 10, "Legend")
				legend_pixmap.set_foreground_color (gradient.get_color_from_value (
					gradient.lower_y_value + i * (
						gradient.upper_y_value - gradient.lower_y_value) / 100))
				legend_pixmap.draw_segment (1, 130 - i, 10, 130 - i)
				if (i \\ 10) = 0 then
					legend_pixmap.draw_text (15, 130 - i + 10  , convert_10_char_string (
						(gradient.lower_y_value + i *
							(gradient.upper_y_value - gradient.lower_y_value) / 100).floor))
				end
				i := i + 1
			end
			pixmap_needs_redraw := false
		end
	
	convert_10_char_string (a_i: INTEGER): STRING is
			-- Convert `a_i' to a ten character String in exponential representation.
		local
			l_minus: INTEGER
			l_sign: STRING
			l_point: STRING
			l_ten:STRING
		do
			l_point := "."
			l_ten := "*10^"
			l_minus := 0
			If a_i.sign = -1 then
				l_sign := "-"
				l_minus := 1
			else
				l_sign := "+"
			end
			Result := l_sign
			Result.append(a_i.out.item(1 + l_minus).out)
			Result.append (l_point)
			if a_i.abs > 9 then
				Result.append(a_i.out.item (2 + l_minus).out)
			else
				Result.append("0")
			end
			if a_i.abs > 99 then
				Result.append (a_i.out.item (3 + l_minus).out)
			else
				Result.append ("0")
			end
			Result.append (l_ten)
			Result.append ((a_i.out.count - l_minus - 1).out)
		end
	
invariant

	gradient_not_void: gradient /= Void

end


