indexing
	description: "Line-chart renderer"
	author: "Zoran Simic"
	date: "$Date: 2008/12/01 $"
	revision: "$Revision: #8 $"

class
	ROSE_CHART_RENDERER_LINE

inherit
	ROSE_CHART_RENDERER[ROSE_CHART_2D_POINT]
	redefine
		render_point_type
	end

create
	make,
	make_with_line_width

feature {NONE} -- Initialization

	make (a_lined, a_marked: BOOLEAN) is
			-- Create the line
			-- 'a_lined': are the lines between the points drawn?
			-- 'a_marked': are the points at the ends of the lines marked?
		do
			make_with_line_width (a_lined, a_marked, 1, 2)
		end

	make_with_line_width (a_lined, a_marked: BOOLEAN; a_line_width, a_highlighted_line_width: INTEGER) is
			-- Create the line, also specifying line widths
		require
			valid_line_widths: a_line_width >= 0 and a_highlighted_line_width >= 0
		do
			is_lined := a_lined
			is_marked := a_marked
			set_line_width (a_line_width, a_highlighted_line_width)
		end


feature -- Access

	is_lined: BOOLEAN
			-- Join the points with lines?

	is_marked: BOOLEAN
			-- Are points to be marked with a shape?

	is_stacked: BOOLEAN
			-- Are lines to be stacked?

	is_proportional: BOOLEAN
			-- Is stacking to be proportional to 100% (sum of all points being 100%)

	line_normal_width: INTEGER
			-- Normal width of the line

	line_highlighted_width: INTEGER
			-- Width of the line when it's highlighted

feature -- Basic operations

	new_render_point (a_point: ROSE_CHART_2D_POINT):  like render_point_type is
			-- create a new render_point
		do
			create Result.make (a_point)
		end

	new_figure (a_point: ROSE_CHART_RENDER_POINT[ROSE_CHART_POINT]): ROSE_CHART_FIGURE_LINE is
			-- New figure.
		do
			create Result.make (is_lined,is_marked,a_point.render_plot.index,line_normal_width,line_highlighted_width)
		end

	set_lined (a_value: BOOLEAN) is
		do
			is_lined := a_value
		end

	set_line_width (a_normal_width, a_highlighted_width: INTEGER) is
			-- set line widths
		do
			line_normal_width := a_normal_width
			line_highlighted_width := a_highlighted_width
		end


	set_marked (a_value: BOOLEAN) is
			-- Set 'is_marked'.
		do
			is_marked := a_value
		end

	set_stacked (a_value: BOOLEAN) is
			-- Set 'is_stacked'.
		do
			is_stacked := a_value
		end

	set_proportional (a_value: BOOLEAN) is
			-- Set 'is_proportional'.
		do
			is_proportional := a_value
		end

feature {NONE} --Templates
	render_point_type: ROSE_CHART_RENDER_2D_POINT is --template
		do end


invariant
	valid_line_widths: line_normal_width >= 0 and line_highlighted_width >= 0

end


