indexing
	description: "[
			Selection area which displays current coordinates on the edges.
			Current coordinates are looked up in {ROSE_GRAPH_COORDINATE_SYSTEM}
			]"
	author: "Urs Doenni"
	date: "$Date: 2008/12/01 $"
	revision: "$Revision: #2 $"
	archive: "$File: //rose/source/kernel/infrastructure/widgets/charts/rose_chart_selection_area.e $"

class
	ROSE_CHART_SELECTION_AREA

inherit
	ROSE_SELECTION_AREA
		redefine
			setup_model
		end

create
	make,
	make_proportional

feature -- Access

	main_system: ROSE_CHART_COORDINATE_SYSTEM[ROSE_CHART_POINT]
			-- Reference to main system (used to look up coordinates)

	secondary_system: ROSE_CHART_COORDINATE_SYSTEM[ROSE_CHART_POINT]
			-- Reference to secondary system (used to look up coordinates)

feature -- Basic operations

	set_main_coordinate_system (a_coordinate_system: like main_system) is
			-- Set `main_system' to `a_coordinate_system'
		do
			main_system := a_coordinate_system
		end

	set_secondary_coordinate_system (a_coordinate_system: like secondary_system) is
			-- Set `secondary_system' to `a_coordinate_system'
		do
			secondary_system := a_coordinate_system
		end

feature {NONE} -- Implementation

	setup_model is
			-- Position all the figures
		local
			l_offset: INTEGER
			l_main_coords, l_secondary_coords: ROSE_RECTANGLE
			l_main_left_text, l_main_right_text: STRING
			l_secondary_left_text, l_secondary_right_text: STRING
		do
				Precursor {ROSE_SELECTION_AREA}

				l_offset := 7

				l_main_coords := selection_in_coordinates (main_system)
				l_secondary_coords := selection_in_coordinates (secondary_system)

				if l_main_coords /= Void then
					l_main_left_text := double_out(l_main_coords.x1.min(l_main_coords.x2)) + "/" + double_out(l_main_coords.y1.max(l_main_coords.y2))
					l_main_right_text := double_out(l_main_coords.x2.max(l_main_coords.x1)) + "/" + double_out(l_main_coords.y2.min(l_main_coords.y1))
				end
				if l_secondary_coords /= Void then
					l_secondary_left_text := double_out(l_secondary_coords.x1.min(l_secondary_coords.x2)) + "/" + double_out(l_secondary_coords.y1.max(l_secondary_coords.y2))
					l_secondary_right_text := double_out(l_secondary_coords.x2.max(l_secondary_coords.x1)) + "/" + double_out(l_secondary_coords.y2.min(l_secondary_coords.y1))
				end

				if l_main_coords /= Void and l_secondary_coords /= Void then
					left_corner_text.set_text (l_main_left_text + "%N" + l_secondary_left_text)
					right_corner_text.set_text (l_main_right_text + "%N" + l_secondary_right_text)
				elseif l_main_coords /= Void and l_secondary_coords = Void then
					left_corner_text.set_text (l_main_left_text)
					right_corner_text.set_text (l_main_right_text)
				elseif l_main_coords = Void and then l_secondary_coords /= Void then
					left_corner_text.set_text (l_secondary_left_text)
					right_corner_text.set_text (l_secondary_right_text)
				end
				left_corner_text.set_point_position (
					(x1.min(x2) - left_corner_text.width - l_offset).max (selectable_area.x1.truncated_to_integer + l_offset),
					(y1.min(y2) - left_corner_text.height - l_offset).max (selectable_area.y1.truncated_to_integer + l_offset)
					)
				right_corner_text.set_point_position (
					(x2.max(x1) + l_offset).min (selectable_area.x2.truncated_to_integer - right_corner_text.width - l_offset),
					(y2.max(y1) + l_offset).min (selectable_area.y2.truncated_to_integer - right_corner_text.height - l_offset)
					)

		end

	selection_in_coordinates (a_coordinate_system: ROSE_CHART_COORDINATE_SYSTEM[ROSE_CHART_POINT]): ROSE_RECTANGLE is
			-- Get current coordinates which selection area covers in `a_coordinate_system'
		local
			l_selection_in_coordinates: ROSE_RECTANGLE
			l_coord_view: ROSE_RECTANGLE
			l_current: ROSE_RECTANGLE
			l_current_normal: ROSE_RECTANGLE
		do
			if a_coordinate_system /= Void and then a_coordinate_system.plot_list /= Void then
				create l_current.make (x1,y1,x2,y2)
				l_current_normal := selectable_area.normalized_rectangle (l_current)
				l_current_normal.set_y (1 - l_current_normal.y1, 1 - l_current_normal.y2)

				l_coord_view := a_coordinate_system.current_view

				if l_coord_view.is_set then
					create l_selection_in_coordinates.set_from_rectangle(l_coord_view.relative_rectangle (l_current_normal))
				else
					create l_selection_in_coordinates.set_from_rectangle(a_coordinate_system.world.relative_rectangle (l_current_normal))
				end
				l_selection_in_coordinates.set_y (l_selection_in_coordinates.y1, l_selection_in_coordinates.y2)

				Result := l_selection_in_coordinates
			end
		end

	double_out (a_double: DOUBLE): STRING is
			-- Convert `a_double' to a string, using `formatter'
		do
			Result := formatter.formatted (a_double)
		end


	formatter: FORMAT_DOUBLE is
			-- Double format, which formats doubles as xx.xx
		once
			create Result.make (2, 2)
			Result.comma_separate
		end

end
