indexing
	description: "Chart settings"
	author: "Zoran Simic"
	last_editor: "$Author: c4 $"
	date: "$Date: 2013/07/08 $"
	revision: "$Revision: #11 $"
	path: "$File: //rose/source/kernel/infrastructure/widgets/charts/rose_chart_settings.e $"

class
	ROSE_CHART_SETTINGS

inherit
	ROSE_GRAPHICS_SETTINGS
		redefine
			initialize, display
		end

create
	make

feature {NONE} -- Initialization

	initialize is
			-- Specific initialization, redefine if needed.
		do
			Precursor
			-- Default settings
			set_axis_font (Gui.rep.fonts.new_screen_font (False,False,11))
			set_axis_color (palette.foreground)
			set_chart_background (Gui.rep.colors.White)
			-- WAS: set_chart_background (Gui.rep.colors.rgb (255,252,242))
			set_axis_tick_size (4)
			set_margin (20)
			set_point_size (4)
		end

feature -- Access

	display: ROSE_CHART_DISPLAY
			-- Associated display

	axis_width: INTEGER
	axis_height: INTEGER
	axis_font_height: INTEGER
	axis_tick_size: INTEGER
	clip_offset: INTEGER is 4
			-- Various margins

	axis_font: EV_FONT
			-- Fonts to use

	axis_color: EV_COLOR
	axis_color_1: EV_COLOR
	axis_color_2: EV_COLOR
	chart_background: EV_COLOR
	grid_minor_color: EV_COLOR
	grid_major_color: EV_COLOR
			-- Colors to use (outside actual chart colors, which are taken from 'palette')

	is_use_count_for_plot_color_index: BOOLEAN

feature -- Element modification
	
	set_axis_height (a_height: INTEGER) is
			-- Set 'axis_height'
		do
			axis_height := a_height
		end
	
	set_axis_width (a_height: INTEGER) is
			-- Set 'axis_width'
		do
			axis_width := a_height
		end
	
	set_title_height (a_height: INTEGER) is
			-- Set 'title_height'
		do
			title_height := a_height
		end
	
	set_axis_font (a_font: EV_FONT) is
			-- Set 'axis_font'.
		do
			axis_font := a_font
			axis_width := a_font.string_width ("123456789")
			axis_font_height := a_font.ascent + a_font.descent + 1
			axis_height := axis_font_height * 3
		end

	set_axis_tick_size (a_value: INTEGER) is
			-- Set 'axis_tick_size'
		do
			axis_tick_size := a_value.max (2)
		end

	set_axis_color (a_color: EV_COLOR) is
			-- Set 'axis_color'.
		do
			axis_color := a_color
			axis_color_1 := Gui.rep.colors.emphasize (0,a_color)
			axis_color_2 := Gui.rep.colors.emphasize (100,axis_color_1)
		end

	set_chart_background (a_color: EV_COLOR) is
			-- Set background colors
		do
			if a_color /= Void then
				chart_background := a_color
			else
				chart_background := palette.background
			end
			grid_minor_color := Gui.rep.colors.emphasize (-44,chart_background)
			grid_major_color := Gui.rep.colors.emphasize (-24,grid_minor_color)
		end

	set_is_use_count_for_plot_color_index (a_flag: BOOLEAN) is
			-- Set 'is_use_count_for_plot_color_index'
		do
			is_use_count_for_plot_color_index := a_flag
		end

feature -- Basic operations
	
	show is
		local
			l_box, l_hbox: EV_BOX
			l_button: EV_BUTTON
			l_widget: EV_WIDGET
		do
			if dialog = Void then
				-- Settings dialog
				dialog := Gui.factory.new_dialog ("Chart settings","settings.ico",0,0)
				l_box := Gui.factory.new_vbox (dialog,4,4)

				l_hbox := Gui.factory.new_hbox (l_box,0,4)
				cb_use_count_as_color_index := Gui.factory.new_check_button (l_hbox, False, "Count as Color Index", agent set_use_count_color_index,1)
				-- Notebook
				create notebook
				l_box.extend (notebook)
				create main_cs.make (notebook,display.graphics.main_system)
				create secondary_cs.make (notebook,display.graphics.secondary_system)
				notebook.set_item_text (main_cs,"Main")
				notebook.set_item_text (secondary_cs,"Secondary")
				-- OK/Cancel
				l_hbox := Gui.factory.new_hbox (l_box,0,4)
				l_widget := Gui.factory.new_label (l_hbox,Void,0)
				l_button := Gui.factory.new_button (l_hbox,"Apply",agent apply,70)
				dialog.set_default_push_button (l_button)
				l_widget := Gui.factory.new_label (l_hbox,Void,4)
				l_button := Gui.factory.new_button (l_hbox,"Close",agent hide,70)
				dialog.set_default_cancel_button (l_button)
				l_widget := Gui.factory.new_label (l_hbox,Void,0)
			end
			Gui.open_window (Void,dialog)
			refresh
		end

	hide is
		do
			if dialog /= Void then
				Gui.close_window (dialog)
			end
		end

	refresh is
		do
			if dialog /= Void then
				main_cs.refresh
				secondary_cs.refresh
			end
		end

	apply is
			-- Activate changes.
		do
			if dialog /= Void then
				main_cs.apply
				secondary_cs.apply
			end
		end

	set_use_count_color_index is
		do
			is_use_count_for_plot_color_index := cb_use_count_as_color_index.is_selected
		end

feature {NONE} -- Implementation

	dialog: EV_DIALOG
	notebook: EV_NOTEBOOK
	cb_use_count_as_color_index: EV_CHECK_BUTTON
	main_cs, secondary_cs: ROSE_CHART_SETTINGS_CS

end
