indexing
	description: "Settings for chart axis"
	author: "Zoran Simic"
	date: "$Date: 2008/12/01 $"
	revision: "$Revision: #5 $"

class
	ROSE_CHART_SETTINGS_AXIS

inherit
	ROSE_CHART_CONSTANTS

create
	make

feature {NONE} -- Initialization

	make (a_container: EV_CONTAINER; a_name: STRING) is
			-- Creation procedure
		local
			l_label : EV_LABEL
			l_frame: EV_FRAME
			l_box, l_hbox: EV_BOX
		do
			l_frame := Gui.factory.new_frame (a_container,a_name,0,0)
			l_box := Gui.factory.new_vbox (l_frame,4,4)
			l_hbox := Gui.factory.new_hbox (l_box,0,4)
			t_title := Gui.factory.new_text_field (l_hbox,"Title: ",Void,Void,-50)
			l_hbox := Gui.factory.new_hbox (l_box,0,4)
			cb_origin := Gui.factory.new_check_button (l_hbox,False,"Origin",agent activate_widgets,1)
			t_origin := Gui.factory.new_text_field (l_hbox,Void,Void,Void,-40)
			cb_log := Gui.factory.new_check_button (l_box,False,"Logarithmic",agent do_nothing,1)
			cb_equidistant := Gui.factory.new_check_button (l_box,False,"Equidistant",agent do_nothing,1)
			l_hbox := Gui.factory.new_hbox (l_box,0,4)
			l_label := Gui.factory.new_label(l_hbox, " Min ", 40)
			t_min := Gui.factory.new_text_field (l_hbox,Void,Void,Void,-40)
			l_hbox := Gui.factory.new_hbox (l_box,0,4)
			l_label := Gui.factory.new_label(l_hbox, " Max ", 40)
			t_max := Gui.factory.new_text_field (l_hbox,Void,Void,Void,-40)
		end

feature -- Access

	axis: ROSE_CHART_AXIS
			-- Currently shown axis

	debug_output: STRING is
			-- String that should be displayed in debugger to represent `Current'.
		do
			Result := ""
		end

feature -- Basic operations

	refresh (a_axis: ROSE_CHART_AXIS) is
			-- Refresh these setting with 'a_axis'.
		require
			axis_ok: a_axis /= Void
		local
			l_indexed: ROSE_CHART_AXIS_INDEXED
		do
			axis := a_axis
			l_indexed ?= a_axis
			is_indexed := l_indexed /= Void
			Gui.output (t_title,a_axis.title,Void)
			Gui.output (t_origin,a_axis.line_origin.out,Void)
			Gui.enable_select (cb_origin,not a_axis.is_line_docked)
			Gui.enable_select (cb_log,a_axis.is_logarithmic)
			Gui.enable_select (cb_equidistant,a_axis.is_equidistant)
			Gui.enable_sensitive (cb_equidistant,t_title.is_sensitive and then is_indexed)
			activate_widgets
		end

	apply is
		local
			s: STRING
		do
			axis.set_title (t_title.text)
			s := t_origin.text
			if s /= Void and then s.is_double then
				axis.set_line_origin (s.to_double)
			end
			axis.set_line_docked (not cb_origin.is_selected)
			axis.set_logarithmic (cb_log.is_selected)
			if is_indexed then
				axis.set_equidistant (cb_equidistant.is_selected)
			end
			if t_min.text /= Void and then t_min.text.is_double then
				axis.set_minimum_visible (t_min.text.to_double)
			end
			if t_max.text /= Void and then t_max.text.is_double then
				axis.set_maximum_visible (t_max.text.to_double)
			end
		end

feature {NONE} -- Implementation

	t_title, t_origin: EV_TEXT_FIELD
	t_min, t_max : EV_TEXT_FIELD
	cb_log, cb_origin, cb_equidistant: EV_CHECK_BUTTON
	is_indexed: BOOLEAN

	activate_widgets is
		do
			if t_origin /= Void and then cb_origin /= Void then
				Gui.enable_sensitive (t_origin,cb_origin.is_selected)
			end
		end

end -- class ROSE_CHART_SETTINGS_AXIS
