note

	description:

		"Two way list used for ROSE_CHART_(RENDER_)PLOTs"

	author: "Urs Doenni"
	last_editor: "$Author: er $"
	date: "$Date: 2014/02/11 $"
	revision: "$Revision: #15 $"
	path: "$File: //rose/source/kernel/infrastructure/widgets/charts/rose_chart_two_way_list.e $"

class
	ROSE_CHART_TWO_WAY_LIST [P -> KEYED]

inherit
	TWO_WAY_LIST [P]
		rename
			remove as remove_two_way_list,
			extend as extend_back,
			index as index_two_way_list,
			put_front as extend_front
		export
			{NONE}	all
			{ANY}	extendible, do_all, do_if, for_all, start, finish, forth, back, go_to, cursor, new_cursor, valid_cursor,
					after, before, item, count, first, last, is_empty, readable, off, wipe_out, prunable
			{ROSE_CHART_TWO_WAY_LIST} last_element, first_element, index_two_way_list,
					object_comparison, compare_objects,
					changeable_comparison_criterion
		redefine
			make, extend_front, extend_back, wipe_out
		end

	ANY
		undefine
			is_equal,
			copy
		end

create
	make

create  {ROSE_CHART_TWO_WAY_LIST}
	make_sublist

feature {NONE} -- Initialization

	make is
			-- Creation procedure
		do
			Precursor {TWO_WAY_LIST}
			create extend_front_actions
			create extend_back_actions
			create removed_actions
			create hash_table.make (32)
		end

feature -- Basic operations

	extend_front (v: like item) is
			-- put the element first
		do
			Precursor {TWO_WAY_LIST} (v)
			hash_table.put (v, v.key)
			extend_front_actions.call([v])
		end

	extend_back (v: like item) is
			-- put the element last
		do
			Precursor {TWO_WAY_LIST} (v)
			hash_table.put (v, v.key)
			extend_back_actions.call([v])
		end

	remove (a_key: HASHABLE) is
			-- remove the point from the sequence
		local
			l_point: P
		do
			l_point := hash_table.item (a_key)
			hash_table.remove (a_key)
			prune (l_point)
			removed_actions.call([a_key])
		end

	wipe_out
			-- Remove all items.
		do
			Precursor
			hash_table.clear_all
		end

	item_from_key (a_key: HASHABLE): P is
			-- item at index 'a_index'
		require
			has_key: has_key (a_key)
		do
			Result := hash_table.item (a_key)
		end

	has_key (a_key: HASHABLE): BOOLEAN is
			-- does the sequence contain an item with a_key?
		do
			Result := hash_table.has (a_key)
		end

feature -- Actions

	extend_front_actions: ACTION_SEQUENCE[TUPLE[P]]
		-- Called at the end of feature 'put_front'

	extend_back_actions: ACTION_SEQUENCE[TUPLE[P]]
		-- Called at the end of feature 'put_back'

	removed_actions: ACTION_SEQUENCE[TUPLE[HASHABLE]]
		-- Called at the end of feature 'remove'

feature {NONE} -- Implementation

	hash_table: HASH_TABLE [P, HASHABLE]
			-- Hash table

end
