indexing
	description: "Rectangle showing the current view of another coordinate system"
	author: "Urs Doenni"
	date: "$Date: 2008/12/01 $"
	revision: "$Revision: #5 $"
	archive: "$File: //rose/source/kernel/infrastructure/widgets/charts/rose_chart_view_overlay.e $"

class
	ROSE_CHART_VIEW_OVERLAY

inherit
	ROSE_SHARED_GLOBAL_GUI
		export
			{NONE} all
		end

	DOUBLE_MATH
		export
			{NONE} all
		end
	ANY
create
	make

feature  --Creation

	make (a_coordinate_system: like coordinate_system) is
		require
			nonvoid : a_coordinate_system /= void
		do
			create figure.default_create
			set_color (Gui.rep.colors.Cyan)
			coordinate_system := a_coordinate_system

		end


feature -- Access

	figure: EV_MODEL_RECTANGLE
	pixmap: EV_PIXMAP is
			-- Associated pixmap
		do
			Result := coordinate_system.display.pixmap
		end

	color: EV_COLOR
			-- Colors to use to show

	coordinate_system: ROSE_CHART_COORDINATE_SYSTEM[ROSE_CHART_POINT]

	other_coordinate_system: like coordinate_system
		-- Coordinate system

feature -- Basic operations

	is_used: BOOLEAN is
		-- Says if the overlay is in use
		do
			Result := other_coordinate_system /= Void
		end

	update is
			-- Create the 'selected' and all resize-rectangles
		require
			is_in_use: is_used = True
		local
			l_view: ROSE_RECTANGLE
			l_other_view: ROSE_RECTANGLE
			l_normalized_view: ROSE_RECTANGLE
			l_pixmap_rect: ROSE_RECTANGLE
			l_pixmap_view: ROSE_RECTANGLE
		do
--			draw (False)
			arrange_coordinates
--			create selected.make (x1,y1,x2,y2)
--			normal_selected := selectable_area.normalized_rectangle (selected)

--			TODO: also call current when THIS coordinate_system is projected

			l_view := coordinate_system.current_view
			l_other_view := other_coordinate_system.current_view
			l_normalized_view := l_view.normalized_rectangle (l_other_view)
			l_normalized_view.set_y (1.0 - l_normalized_view.y1, 1.0 - l_normalized_view.y2)
			create l_pixmap_rect.make (coordinate_system.display.drawn_area.x1,coordinate_system.display.drawn_area.y1, coordinate_system.display.drawn_area.x2, coordinate_system.display.drawn_area.y2)
			l_pixmap_view := l_pixmap_rect.relative_rectangle (l_normalized_view)

			x1 := l_pixmap_view.x1.truncated_to_integer
			y1 := l_pixmap_view.y1.truncated_to_integer
			x2 := l_pixmap_view.x2.truncated_to_integer
			y2 := l_pixmap_view.y2.truncated_to_integer

			figure.set_point_a_position (x1, y1)
			figure.set_point_b_position (x2, y2)
			-- TODO: update

			coordinate_system.state.set_needs_redraw

--			draw (True)
		end

	reset is
			-- Reset current selection (forget about it).
		do
			is_drawn := False
		end

	remove is
			-- Remove current selection.
		do
			draw (False)
		end

	set_other_coordinate_system (a_coordinate_system: like other_coordinate_system) is
			-- set the other coordinate system (where the view is taken from)
		require
			not_void: a_coordinate_system /= Void
			not_yet_set: other_coordinate_system = Void -- System can only be set once
		do
			other_coordinate_system := a_coordinate_system
			other_coordinate_system.view_changed_actions.extend (agent update)
		end

	redraw is
		do
			if is_drawn then
				draw (True)
			end
		end

	set_color (a_color: EV_COLOR) is
			-- Set 'color'.
		require
			valid_color: a_color /= Void
		do
			color := a_color
			figure.set_foreground_color (color)
			figure.set_line_width (5)
		end

feature {NONE} -- Implementation

	is_drawn: BOOLEAN
	x1, y1, x2, y2: INTEGER
	old_x, old_y: INTEGER
	offset_x, offset_y: INTEGER
		-- Offset of mouse pointer when resizing

	width: INTEGER is
		do
			Result := (x2 - x1).abs
		end

	height: INTEGER is
		do
			Result := (y2 - y1).abs
		end




	draw (a_original: BOOLEAN) is
		do
			if a_original or else is_drawn then
				pixmap.set_xor_mode
				pixmap.enable_dashed_line_style
				pixmap.set_foreground_color (color)
				pixmap.draw_rectangle (x1.min (x2),y1.min (y2),width,height)
				pixmap.disable_dashed_line_style

				pixmap.set_copy_mode
				is_drawn := a_original
			end
		end

	arrange_coordinates is
			-- Make sure that x1 <= x2 and y1 <= y2.
		local
			i: INTEGER
		do
			if x2 < x1 then
				i := x1
				x1 := x2
				x2 := i
			end
			if y2 < y1 then
				i := y1
				y1 := y2
				y2 := i
			end
		end

end -- class ROSE_SELECTION_AREA
