indexing
	description: "Represents a Zone in the plots"
	author: "Ueli Peter"
	date: "$Date: 2008/12/01 $"
	revision: "$Revision: #4 $"

class
	ROSE_CHART_ZONE_FIGURE

inherit
	ROSE_CHART_BACKGROUND_FIGURE
		redefine
			project
		end

create
	make_zone_from_coordinates, make_zone_from_coordinates_and_array

feature -- creation
	make_zone_from_coordinates (a_x1, a_x2: DOUBLE; a_color: EV_COLOR; a_label: STRING; a_frac: DOUBLE) is
		-- create a zone figure between `a_x1' and `a_x2' with label on `a_frac' of the charts hight
		require
			fraction: a_frac >= 0 and a_frac <= 1
		local
			l_label: EV_MODEL_TEXT
		do
			create labels.make (1,1)
			create l_label.make_with_text (a_label)
			labels.put (l_label, 1)
			initialize (a_x1, a_x2, a_color, a_frac)
		end

	make_zone_from_coordinates_and_array (a_x1, a_x2: DOUBLE; a_color: EV_COLOR; a_labels: ARRAY[STRING]; a_frac: DOUBLE) is
		-- create a zone figure between `a_x1' and `a_x2' with label on `a_frac' of the charts hight
		require
			fraction: a_frac >= 0 and a_frac <= 1
		local
			l_label: EV_MODEL_TEXT
			i: INTEGER
		do
			create labels.make (1, a_labels.count)
			from
				i := 1
			until
				i > a_labels.count
			loop
				create l_label.make_with_text (a_labels.item (i))
				labels.put (l_label, i)
				i := i + 1
			end
			initialize (a_x1, a_x2, a_color, a_frac)
		end

feature -- initialization		

	initialize (a_x1, a_x2: DOUBLE; a_color: EV_COLOR; a_frac: DOUBLE)
			-- build figure
		local
			l_coords: ARRAY[EV_COORDINATE]
			l_coord: EV_COORDINATE
		do
			create l_coords.make (1, 4)
			create l_coord.make (a_x1.truncated_to_integer, 0)
			l_coords.put (l_coord, 1)
			create l_coord.make (a_x2.truncated_to_integer, 0)
			l_coords.put (l_coord, 2)
			create l_coord.make (a_x2.truncated_to_integer, 1)
			l_coords.put (l_coord, 3)
			create l_coord.make (a_x1.truncated_to_integer, 1)
			l_coords.put (l_coord, 4)
			if  a_color = void then
				make_from_coordinates (l_coords, create {EV_COLOR}.make_with_8_bit_rgb (255, 0, 0), create {EV_COLOR}.make_with_8_bit_rgb (255, 0, 0))
			else
				make_from_coordinates (l_coords, a_color, a_color)
			end
			label_y_position := a_frac
			if label_y_position < 0.2 then
				label_y_offset := -20
			elseif label_y_position > 0.8 then
				label_y_offset := 20
			else
				label_y_offset := 0
			end
			x1 := a_x1
			x2 := a_x2
		end
feature -- Status
	labels: ARRAY[EV_MODEL_TEXT]
			-- a label displayed in this zone

	label_y_position: DOUBLE
			-- labels y_position on the Display as fraction between (double between 1 and 0)

feature -- setting

	set_label_y_position (a_frac: DOUBLE) is
			-- set `label_y_position'
		require
			fraction: a_frac >= 0 and a_frac <= 1
		do
			label_y_position := a_frac
			if label_y_position < 0.2 then
				label_y_offset := -20
			elseif label_y_position > 0.8 then
				label_y_offset := 20
			else
				label_y_offset := 0
			end
		end

	pointer_enter_actions: EV_NOTIFY_ACTION_SEQUENCE is
				-- pointer_enter_actions
			do
				Result := figure.pointer_enter_actions
			end

	pointer_leave_actions: EV_NOTIFY_ACTION_SEQUENCE is
				-- pointer_leave_actions
			do
				Result := figure.pointer_leave_actions
			end

	pointer_button_press_actions: EV_POINTER_BUTTON_ACTION_SEQUENCE is
				-- pointer_button_press_actions
			do
				Result := figure.pointer_button_press_actions
			end

	pointer_button_release_actions: EV_POINTER_BUTTON_ACTION_SEQUENCE is
				-- pointer_button_release_actions
			do
				Result := figure.pointer_button_release_actions
			end

	pointer_double_press_actions: EV_POINTER_BUTTON_ACTION_SEQUENCE is
				-- pointer_double_press_actions
			do
				Result := figure.pointer_double_press_actions
			end

	pointer_motion_actions: EV_POINTER_MOTION_ACTION_SEQUENCE is
				-- pointer_motion_actions
			do
				Result := figure.pointer_motion_actions
			end



feature {NONE} -- Implementation

	x1: DOUBLE
			-- left border of zone

	x2: DOUBLE
			-- right border of zone

	label_y_offset: INTEGER
			-- the label needs a little offset it's negative for `label_y_position' > 0.5

	project (a_x0, a_y0, a_x_scale, a_y_scale: DOUBLE) is
			--Projects this figure
		local
			l_x0, l_x1, l_y0, l_y1: INTEGER
			i: INTEGER
		do
			--precursor {ROSE_CHART_BACKGROUND_FIGURE} (a_x0, a_y0, a_x_scale, a_y_scale)
			-- project figure
			if figure /= Void and coordinate_system /= Void then

					l_x0 := display.projected_x (x1, a_x0, a_x_scale)
					l_y0 := display.projected_y (0, a_y0, a_y_scale)
					l_x1 := display.projected_x (x2, a_x0, a_x_scale)
					l_y1 := display.projected_y (1, a_y0, a_y_scale)

					figure.set_i_th_point_position (1, display.left + l_x0, display.bottom + l_y0)
					figure.set_i_th_point_position (2, display.left + l_x1, display.bottom + l_y0)
					figure.set_i_th_point_position (3, display.left + l_x1, display.bottom + l_y1)
					figure.set_i_th_point_position (4, display.left + l_x0, display.bottom + l_y1)

			end

			-- project labels
			l_x0 := display.projected_x (coords.item(1).x, a_x0, a_x_scale)
			l_y0 := display.projected_y (coords.item(1).y, a_y0, a_y_scale)
			l_x1 := display.projected_x (coords.item(3).x, a_x0, a_x_scale)
			l_y1 := display.projected_y (coords.item(3).y, a_y0, a_y_scale)
			From
				i := 1
			until
				i > labels.count
			loop
				labels.item(i).set_point_position (display.left + ((l_x0 + l_x1) * 0.5).ceiling - (labels.item(i).width * 0.5).ceiling, display.bottom + ((l_y1 - l_y0) * label_y_position).ceiling + label_y_offset + (i - 1) * (labels.item(i).height + 2))
				i := i + 1
			end


		end
end
