indexing
	description : "[
		Represents a date interval where both the start and end dates are included
		]"
	author : "Mark Howard"
	date: "$Date: 2008/12/01 $"
	revision: "$Revision: #7 $"

class
	ROSE_CLOSED_INTERVAL[G]
inherit
	ROSE_INTERVAL[G]
create
	make
feature
	starts_by_date (a_date : ROSE_DATE) : BOOLEAN is
		do
			Result := start_is_unbounded or else start_date <= a_date
		end
	ends_after_date (a_date : ROSE_DATE) : BOOLEAN is
		do
			Result := end_is_unbounded or else a_date <= end_date
		end

	last_date : ROSE_DATE is do Result := end_date end
	first_date : ROSE_DATE is do Result := start_date end
	is_valid : BOOLEAN is
		do
			Result := item /= Void and then ((start_date /= Void and end_date /= Void) and then (start_date <=end_date))
		end

	interval_before(a_first_date : ROSE_DATE; a_weight : DOUBLE) : like Current is
		do
			create Result.make(item, a_first_date, start_date.previous_date, a_weight)
		end

	interval_after(a_last_date : ROSE_DATE; a_weight : DOUBLE) : like Current is
		-- interval after 
		do
			create Result.make(item, end_date.next_date, a_last_date, a_weight)
		end

	interval_between (a_start_date, a_end_date: ROSE_DATE; a_weight: DOUBLE) : like Current is
			-- joint inverval betwee current and [a_start, a_end]
		require
			a_start_date /= Void and then a_end_date /= Void
		local
			l_start, l_end: ROSE_DATE
		do
			l_start := start_date.max (a_start_date)
			l_end := end_date.min (a_end_date)
			if not (l_end < l_start) then
				Create Result.make (item, l_start, l_end, a_weight)
			end
		end

end -- class ROSE_CLOSED_INTERVAL
