indexing
	description : "Represents a date interval where the first date is included and the second is not"
	author : "Mark Howard"
	last_editor: "$Author: sm $"
	date: "$Date: 2009/08/25 $"
	revision: "$Revision: #7 $"
	path: "$File: //rose/source/kernel/infrastructure/structures/rose_closed_open_interval.e $"

class
	ROSE_CLOSED_OPEN_INTERVAL[G]

inherit
	ROSE_INTERVAL[G]

create
	make

feature -- Access

	first_date: ROSE_DATE is
			-- Start date
		do
			Result := start_date
		end

	interval_after(a_last_date: ROSE_DATE; a_weight: DOUBLE): like Current is
			-- Interval ['start_date' equal to current 'end_date', 'end_date' equal to the day after 'a_last_date']
			-- Item and weight identical to current
		do
			create Result.make(item, end_date, a_last_date.next_date, a_weight)
		end

	interval_before(a_first_date: ROSE_DATE; a_weight: DOUBLE): like Current is
			-- Interval ['start_date' equal to current 'a_first_date', 'end_date' equal to current 'start_date']
			-- Item and weight identical to current
		do
			create Result.make(item, a_first_date, start_date, a_weight)
		end

	last_date : ROSE_DATE is
			-- The day before 'end_date'
		do
			Result := end_date.previous_date
		end

feature -- Status Report

	ends_after_date (a_date : ROSE_DATE) : BOOLEAN is
			-- Is 'end_date' after 'a_date'?
		do
			Result := end_is_unbounded or else a_date < end_date
		end

	is_valid : BOOLEAN is
			-- Are 'item' referenced, dates available and ordered?
		do
			Result := item /= Void and then ((start_date /= Void and end_date /= Void) and then (start_date < end_date))
		end

	starts_by_date (a_date : ROSE_DATE) : BOOLEAN is
			-- Is 'start_date' before 'a_date'?
		do
			Result := start_is_unbounded or else start_date <= a_date
		end

end
