indexing
	description: "Color view"
	author: "Zoran Simic"
	date: "$Date: 2009/10/08 $"
	revision: "$Revision: #5 $"

class
	ROSE_COLOR_VIEW

inherit
	ROSE_WIDGET
		rename
			make as make_widget
		redefine
			set_pebble
		end

create
	make, make_with_text

feature {NONE} -- Initialization

	make (a_container: EV_CONTAINER; a_color: EV_COLOR) is
			-- Create a color grid.
		do
			make_with_text (a_container, Void, a_color)
		end

	make_with_text (a_container: EV_CONTAINER; a_text: STRING; a_color: EV_COLOR) is
			-- Create in 'a_container', with 'a_text and 'a_color'
		do
			text := a_text
			make_widget (a_container)
			set_color (a_color)
			set_text (a_text)
		end

	setup_widget is
			-- Setup this widget, do not populate it yet, just build out the structure.
			-- To populate the contents in your widget, use 'refresh'
		do
			default_width := 1
			default_height := 20
			box := gui.factory.new_hbox (Current, 2, 2)
			box.drop_actions.extend (agent on_drop)
			box.pointer_button_press_actions.extend (agent on_click)
			box.pointer_double_press_actions.extend (agent on_double_click)
			box.pointer_enter_actions.extend (agent on_enter (True))
			box.pointer_leave_actions.extend (agent on_enter (False))
			label := new_label (box, Void, 20)
			create color_changed_event
			create click_event
			create double_click_event
		end

feature -- Access

	name: STRING assign set_name
			-- Name assigned to this color (optional), this shows up in tooltip

	text: STRING
			-- Text to show next to this color (optional), this shows up next to the color
			-- This is settable only at creation time

	color: EV_COLOR assign set_color
			-- Currently selected color

	index: INTEGER assign set_index
			-- Index of this color (optional), this typically corresponds to the index of this color in a palette

	is_modifiable: BOOLEAN assign set_modifiable
			-- Is this color modifiable by the user?
			-- When True, clicking on this widget brings up a dialog
			-- allowing the user to select an arbitrary color

	color_changed_event: ROSE_EVENT_TYPE [TUPLE [ROSE_COLOR_VIEW]]
			-- Called when 'color' is changed

	click_event: ROSE_EVENT_TYPE [TUPLE [INTEGER,ROSE_COLOR_VIEW]]
			-- Called when this widget is clicked

	double_click_event: ROSE_EVENT_TYPE [TUPLE [INTEGER,ROSE_COLOR_VIEW]]
			-- Called when this widget is double-clicked

feature -- Basic operations

	refresh is
			-- Refresh the graphical representation of this widget.
		local
			l_tooltip: STRING
		do
			if color /= Void then
				label.set_background_color (color)
				create l_tooltip.make (10)
				if name /= Void then
					l_tooltip.append (name)
					l_tooltip.append (": ")
				end
				l_tooltip.append ("R " + color.red_8_bit.out + ", G " + color.green_8_bit.out + ", B " + color.blue_8_bit.out)
			else
				l_tooltip := "No color set"
			end
			set_tooltip (l_tooltip)
		end

	set_name (a_name: STRING) is
			-- Set 'name'.
		do
			name := a_name
			refresh
		end

	set_color (a_color: EV_COLOR) is
			-- Set 'color'.
		do
			if color /= a_color then
				color := a_color
				refresh
				color_changed_event.call ([Current])
			end
		end

	set_index (a_index: INTEGER) is
			-- Set 'index'
		do
			index := a_index
		end

	set_text (a_text: STRING) is
			-- Set 'text'.
		do
			text := a_text
			if text_label = Void and then a_text /= Void then
				text_label := new_label (box, a_text, 1)
			end
			if text_label = Void then
				-- Do nothing
			elseif a_text /= Void then
				text_label.set_text (a_text)
			else
				text_label.remove_text
			end
		end

	set_tooltip (a_tooltip: STRING) is
			-- Set 'tooltip'.
		do
			if a_tooltip = Void then
				label.remove_tooltip
				if text_label /= Void then
					text_label.remove_tooltip
				end
			else
				label.set_tooltip (a_tooltip)
				if text_label /= Void then
					text_label.set_tooltip (a_tooltip)
				end
			end
		end

	set_modifiable (a_value: BOOLEAN) is
			-- Set 'is_modifiable'.
		do
			is_modifiable := a_value
		end

	set_pebble (a_pebble: like pebble) is
			-- Assign `a_pebble' to `pebble'.
			-- Overrides `set_pebble_function'.
		do
			Precursor (a_pebble)
			label.set_pebble (a_pebble)
			if text_label /= Void then
				text_label.set_pebble (a_pebble)
			end
		end

	pick_color is
			-- Show a color dialog allowing user to choose a color
		local
			l_window: EV_WINDOW
		do
			if color /= Void then
				if color_dialog = Void then
					create color_dialog
					color_dialog.ok_actions.extend (agent on_color_dialog_ok)
				end
				color_dialog.set_color (color)
				l_window := gui.parent_window (Current)
				if l_window /= Void then
					color_dialog.show_modal_to_window (l_window)
				end
			end
		end

feature {NONE} -- Implementation

	box: EV_BOX							-- Box containing the labels composing this view
	color_dialog: EV_COLOR_DIALOG		-- Color selection dialog
	label, text_label: EV_LABEL			-- Labels used to show this color view widget

	on_color_dialog_ok is
			-- Called when 'OK' is pressed on 'color_dialog'
		do
			if color_dialog /= Void and then color /= Void and then color_dialog.color /= Void then
				color.copy (color_dialog.color)
				set_color (color)
			end
		end

	on_click (a_x, a_y, a_button: INTEGER; a_x_tilt, a_y_tilt, a_pressure: DOUBLE; a_screen_x, a_screen_y: INTEGER) is
			-- Mouse double-click handler.
		do
			if a_button = 1 then
				if is_modifiable then
					pick_color
				end
			end
			click_event.call ([a_button,Current])
		end

	on_double_click (a_x, a_y, a_button: INTEGER; a_x_tilt, a_y_tilt, a_pressure: DOUBLE; a_screen_x, a_screen_y: INTEGER) is
			-- Mouse double-click handler.
		do
			double_click_event.call ([a_button,Current])
		end

	new_label (a_container: EV_CONTAINER; a_text: STRING; a_width: INTEGER): EV_LABEL is
			-- New label in 'a_container', with 'a_text' and 'a_width'
		do
			Result := gui.factory.new_label (a_container, a_text, a_width)
			Result.drop_actions.extend (agent on_drop)
			Result.pointer_button_press_actions.extend (agent on_click)
			Result.pointer_double_press_actions.extend (agent on_double_click)
			Result.pointer_enter_actions.extend (agent on_enter (True))
			Result.pointer_leave_actions.extend (agent on_enter (False))
		end

	on_enter (a_in: BOOLEAN) is
			-- Called when mouse enters this widget.
		local
			l_color: EV_COLOR
		do
			if a_in then
				l_color := gui.rep.colors.Color_3d_highlight
			else
				l_color := gui.rep.colors.Color_3d_face
			end
			box.set_background_color (l_color)
			if text_label /= Void then
				text_label.set_background_color (l_color)
			end
		end

	on_drop (a_object: ANY) is
			-- Called when 'a_object' is dropped on this widget
		do
			drop_actions.call ([a_object])
		end

end
