indexing
	description: "Extended combo-box"
	author: "Mark Howard"
	date: "$Date: 2008/12/01 $"
	revision: "$Revision: #18 $"

class
	ROSE_COMBO_BOX

inherit
	EV_COMBO_BOX
	ROSE_FORMATTING
		undefine
			default_create, is_equal, copy
		end

create
	default_create,
	make_with_text,
	make_with_strings,
	make_with_autocomplete,
	make_with_autocomplete_restricted

feature {NONE} -- Initalization

	make_with_autocomplete is
			-- Create `Current' with automatic autocomplete for items.
		do
			default_create
			key_press_actions.extend (agent on_key)
			key_release_actions.extend (agent on_autocomplete_release)
			return_actions.extend (agent on_return)
		end

	make_with_autocomplete_restricted is
			-- Create `Current' with automatic autocomplete for items.
			-- Selection is restricted to what is in combo (no free text)
		do
			is_restricted := True
			make_with_autocomplete
		ensure
			restricted: is_restricted
		end

feature -- Access

	is_restricted: BOOLEAN
			-- Is text restricted to items within this combo?

	data_by_name (a_name: STRING): ANY is
			-- 'data' field of item with 'a_name'
		local
			l_item: like item
			i: INTEGER
		do
			if a_name /= Void then
				from
					i := 1
				until
					Result /= Void or else i > count
				loop
					l_item := i_th (i)
					if l_item.text.is_equal (a_name) then
						Result := l_item.data
					end
					i := i + 1
				end
			end
		end

	index_by_name (a_name: STRING): INTEGER is
			-- Index of item with 'a_name', if present
		local
			i: INTEGER
		do
			if a_name /= Void then
				from
					i := 1
				until
					Result > 0 or else i > count
				loop
					if i_th (i).text.is_equal (a_name) then
						Result := i
					end
					i := i + 1
				end
			end
		end

	current_index: INTEGER is
			-- Index of currently displayed 'text'
		do
			Result := index_by_name (text)
		end

feature -- Basic operations

	select_by_data (a_data: ANY) is
			-- Select item with 'a_data'.
		local
			l_item: like item
			l_found: BOOLEAN
			i: INTEGER
		do
			from
				i := 1
			until
				l_found or else i > count
			loop
				l_item := i_th (i)
				if equal (l_item.data, a_data) then
					l_item.enable_select
					l_found := True
				end
				i := i + 1
			end
			if not l_found and then is_restricted then
				remove_selection
				remove_text
			end
		end

	select_by_name (a_name: STRING) is
			-- Select item with 'a_name'.
		require
			name_not_void: a_name /= Void
		local
			l_item: like item
			l_found: BOOLEAN
			i: INTEGER
		do
			from
				i := 1
			until
				l_found or else i > count
			loop
				l_item := i_th (i)
				if l_item.text.is_equal (a_name) then
					l_item.enable_select
					l_found := True
				end
				i := i + 1
			end
			if not l_found and then is_restricted then
				remove_selection
				remove_text
			end
		end

	remove_by_name (a_name: STRING) is
			-- Remove item with 'a_name' from this combo box.
		require
			name_not_void: a_name /= Void
		local
			l_item: like item
		do
			from
				start
			until
				off
			loop
				l_item := item
				if l_item.text.is_equal (a_name) then
					remove
					finish
				end
				if not off then
					forth
				end
			end
		end

	select_first_item is
			-- Select first item of this combo-box if any.
		do
			if count > 0 then
				i_th (1).enable_select
			end
		end

	extend_data (a_data: ARRAY [ANY]) is
			-- Populate with 'a_data'.
		local
			l_item: EV_LIST_ITEM
			l_object: ANY
			i: INTEGER
		do
			if a_data /= Void then
				from
					i := a_data.lower
				until
					i > a_data.upper
				loop
					l_object := a_data.item (i)
					if l_object /= Void then
						l_item ?= l_object
						if l_item = Void then
							create l_item.make_with_text (formatter.object_representation (l_object, ""))
						end
						l_item.set_data (l_object)
						extend (l_item)
					end
					i := i + 1
				end
			end
		end

feature {NONE} -- Implementation

	previous_caret: INTEGER
			-- Position of cursor in `previous_text'

	previous_text: STRING
			-- Last memorized string for the combo box

	previous_selection_start: INTEGER
			-- Last memorized position of selected text

	on_key (a_key: EV_KEY) is
			-- What to do when we press a key?
		do
			if has_selection then
				previous_selection_start := selection_start
			else
				previous_selection_start := 0
			end
		end

	on_autocomplete_release (a_key: EV_KEY) is
			-- Auto complete action by BP
		require
			key_not_void: a_key /= Void
		local
			l_item: EV_LIST_ITEM
			l_text, l_input: like text
		do
			if is_editable and then a_key.code /= {EV_KEY_CONSTANTS}.key_enter then
				if previous_text = Void or else not previous_text.is_equal (text) then
					l_input := text.as_lower
					if a_key.code = {EV_KEY_CONSTANTS}.Key_back_space
					and then l_input.count > 0
					and then previous_selection_start > 1 then
						l_input.remove (l_input.count)
					end
					l_item := find_first_corresponding_to (l_input)
					if l_item /= Void then
						select_actions.block
						remove_selection
						select_actions.resume
						l_item.enable_select
						l_text := text
						if l_input.count < l_text.count then
							previous_caret := match_score (l_input, l_item.text) + 1
							select_region (previous_caret, l_text.count)
						elseif previous_text /= Void then
							deselect_all
							previous_caret := caret_position
						end
					elseif is_restricted then
						remove_selection
						remove_text
						previous_caret := caret_position
					else
						remove_selection
						previous_caret := caret_position
					end
				else
					previous_caret := caret_position
				end
				previous_text := text
			end
		end

	find_first_corresponding_to (a_input: STRING): EV_LIST_ITEM is
			-- Find first item corresponding to 'a_input'
		require
			input_not_void: a_input /= Void
		local
			l_item: EV_LIST_ITEM
			i, l_score, l_best_score: INTEGER
		do
			from
				i := 1
			until
				i > count
			loop
				l_item := i_th (i)
				l_score := match_score (a_input, l_item.text)
				if l_score > l_best_score then
					Result := l_item
					l_best_score := l_score
				end
				i := i + 1
			end
		end

	match_score (a_input, a_text: STRING): INTEGER is
			-- Matching score for `a_input' against `a_text'
		require
			input_not_void: a_input /= Void
			text_not_void: a_text /= Void
		local
			i: INTEGER
		do
			if is_restricted or else a_text.count >= a_input.count then
				from
					i := 1
				until
					i > a_input.count or else i > a_text.count or else a_input.item (i).as_lower /= a_text.item (i).as_lower
				loop
					Result := Result + 1
					i := i + 1
				end
				if not is_restricted and then Result < a_input.count then
					Result := 0
				end
			end
		end

	on_return is
			-- Make sure selected text is cleared on Return
		do
			deselect_all
		end

end
