indexing
	description: "Cell representing one command parameter"
	author: "Zoran Simic"
	last_editor: "$Author: c4 $"
	date: "$Date: 2011/11/29 $"
	revision: "$Revision: #11 $"
	path: "$File: //rose/source/kernel/infrastructure/framework/gui/rose_command_parameter_cell.e $"

class
	ROSE_COMMAND_PARAMETER_CELL

inherit
	ROSE_SHARED_GLOBAL_GUI
	DEBUG_OUTPUT

create
	make

feature {NONE} -- Initialization

	make (a_container: EV_CONTAINER; a_parameter: like parameter; a_validator: like validator) is
			-- Create a view on 'a_parameter' and put it in 'a_container' if provided.
			-- 'a_validator' is called to tell whether current parameter is valid.
		local
			l_widget: EV_WIDGET
			l_combo: RP_REMEMBERED_VALUE_COMBO_BOX
		do
			parameter := a_parameter
			validator := a_validator
			box := Gui.factory.new_hbox (a_container,0,4)
			if a_parameter = Void then
				l_widget := Gui.factory.new_label (box,"No parameter",1)
				l_widget.set_foreground_color (Gui.rep.colors.Dark_red)
			elseif a_parameter.type = Global.convention.command_parameter_type_string then
				l_widget := Gui.factory.new_label (box,a_parameter.name,100)
				parameter_widget := new_remembered_value_field (a_parameter.name)
				box.extend (parameter_widget)
			elseif a_parameter.type = Global.convention.command_parameter_type_character or
				a_parameter.type = Global.convention.command_parameter_type_zone or
				a_parameter.type = Global.convention.command_parameter_type_equity_zone
			 then
				l_widget := Gui.factory.new_label (box,a_parameter.name,100)
				create l_combo.make_named (a_parameter.name)
				l_combo.selection_text_field.focus_out_actions.extend (agent l_combo.remember_and_load_values)
				l_combo.load_remembered_values
				parameter_widget := l_combo
				box.extend (parameter_widget)
			elseif a_parameter.type = Global.convention.command_parameter_type_integer then
				l_widget := Gui.factory.new_label (box,a_parameter.name,100)
				create l_combo.make_named (a_parameter.name)
				l_combo.selection_text_field.focus_out_actions.extend (agent l_combo.remember_and_load_values)
				l_combo.load_remembered_values
				parameter_widget := l_combo
				box.extend (parameter_widget)
			elseif a_parameter.type = Global.convention.command_parameter_type_boolean then
				l_widget := Gui.factory.new_label (box, a_parameter.name, 100)
				parameter_widget := Gui.factory.new_check_button (box, False, Void, agent do_nothing, 30)
			elseif a_parameter.type = Global.convention.command_parameter_type_double then
				l_widget := Gui.factory.new_label (box,a_parameter.name,100)
				create l_combo.make_named (a_parameter.name)
				l_combo.selection_text_field.focus_out_actions.extend (agent l_combo.remember_and_load_values)
				l_combo.load_remembered_values
				parameter_widget := l_combo
				box.extend (parameter_widget)
			elseif a_parameter.type = Global.convention.command_parameter_type_date then
				l_widget := Gui.factory.new_label (box,a_parameter.name,100)
				parameter_widget := Gui.factory.new_date_field (box, Void, Void)
				parameter_widget.focus_out_actions.extend (agent validate (True))
			elseif a_parameter.type = Global.convention.command_parameter_type_date_time then
				l_widget := Gui.factory.new_label (box,a_parameter.name,100)
				parameter_widget := Gui.factory.new_text_field (box,Void,Void,Void,300)
			elseif a_parameter.type = Global.convention.command_parameter_type_month then
				l_widget := Gui.factory.new_label (box, a_parameter.name, 100)
				parameter_widget := Gui.factory.new_text_field (box, Void, Void, Void, 300)
			else
				l_widget := Gui.factory.new_label (box, "Unknown parameter type", 1)
				l_widget.set_foreground_color (Gui.rep.colors.Dark_red)
			end
			if parameter_widget /= Void then
				status := Gui.factory.new_label (box, Void, 1)
				parameter_widget.key_release_actions.extend (agent validate_from_key)
				validate (True)
			end
		end

	new_remembered_value_field (a_name: STRING): RP_REMEMBERED_VALUE_COMBO_BOX is
			-- New remembered value field from name
		require
			a_name_not_void: a_name /= Void
		do
			create Result.make_named (a_name)
			Result.set_minimum_width (300)
			Result.selection_text_field.focus_out_actions.extend (agent Result.remember_and_load_values)
			Result.load_remembered_values
		end

feature -- Access

	box: EV_BOX
			-- Box holding the whole view

	parameter: ROSE_COMMAND_PARAMETER_ITEM
			-- Command viewed

	validator: PROCEDURE [ANY, TUPLE []]
			-- Called to tell whether currently selected parameters are valid

	value: STRING is
			-- Value of this parameter
		do
			Result := text
			if Result = Void or else Result.is_empty then
				Result := Void
			else
				Result.left_adjust
				Result := "-" + parameter.name + "=%"" + Global.formatter.escape (Result,"%"",Global.convention.command_escapes.item (1)) + "%""
			end
		end

	text: STRING is
			-- Text displayed in text field of `parameter_widget'
		do
			if {l_text: EV_TEXT_COMPONENT} parameter_widget then
				Result := l_text.text
			elseif {l_date_widget: ROSE_EV_DATE_FIELD} parameter_widget then
				Result := l_date_widget.selected_date.ccyymmdd.out
			elseif {l_combo: RP_SELECTION_COMBO_BOX} parameter_widget then
				Result := l_combo.text
			elseif {l_bool: EV_CHECK_BUTTON} parameter_widget then
				if l_bool.is_selected then
					Result := "True"
				else
					Result := "False"
				end
			end
		end

	debug_output: STRING
			-- String that should be displayed in debugger to represent `Current'.
		do
			if parameter /= Void then
				Result := parameter.name
			end
		end

feature -- Basic Operations

	set_validator (a_validator: PROCEDURE [ANY, TUPLE]) is
			-- Set `validator' to `a_validator'
		do
			validator := a_validator
		end

	set_to_default_value is
			-- Set parameter to default_value
		do
			if {l_text: EV_TEXT_COMPONENT} parameter_widget then
				l_text.set_text ("")
			elseif {l_date_widget: ROSE_EV_DATE_FIELD} parameter_widget then
				l_date_widget.set_date (create {ROSE_DATE}.make_today)
				l_date_widget.enable_edit (True)
			elseif {l_combo: RP_SELECTION_COMBO_BOX} parameter_widget then
				l_combo.set_text ("")
			elseif {l_bool: EV_CHECK_BUTTON} parameter_widget then
				l_bool.disable_select
			end
			validate (True)
		end

	set_value_from_string (a_string: STRING) is
			-- Set value from `a_string'
		local
			l_date: ROSE_DATE
		do
			if a_string = Void then
				set_to_default_value
			elseif {l_text: EV_TEXT_COMPONENT} parameter_widget then
				l_text.set_text (a_string)
			elseif {l_date_widget: ROSE_EV_DATE_FIELD} parameter_widget then
				create l_date.make_today
				if l_date.is_ccyymmdd_valid (a_string) then
					create l_date.make_ccyymmdd (a_string.to_integer)
				end
				l_date_widget.set_date (l_date)
			elseif {l_combo: RP_SELECTION_COMBO_BOX} parameter_widget then
				l_combo.set_text (a_string)
			elseif {l_bool: EV_CHECK_BUTTON} parameter_widget then
				if a_string.as_lower.is_equal ("true") then
					l_bool.enable_select
				else
					l_bool.disable_select
				end
			end
		end

	validate (a_trigger_validation: BOOLEAN) is
			-- Set `is_valid' and attributes of `status'.
		local
			l_string: STRING
		do
			l_string := text
			if l_string.is_empty and then parameter.is_optional then
				is_valid := True
			else
				is_valid := parameter.valid_value (l_string)
			end
			if l_string = Void or else l_string.is_empty then
				if is_valid then
					status.set_text ("Optional [" + parameter.type_name + "]")
					status.set_foreground_color (Gui.rep.colors.Dark_green)
				else
					status.set_text ("Required [" + parameter.type_name + "]")
					status.set_foreground_color (Gui.rep.colors.Dark_red)
				end
			elseif is_valid then
				status.set_text ("OK [" + parameter.type_name + "]")
				status.set_foreground_color (Gui.rep.colors.Dark_green)
			else
				status.set_text ("Invalid [" + parameter.type_name + "]")
				status.set_foreground_color (Gui.rep.colors.Dark_red)
			end
			if a_trigger_validation and then validator /= Void then
				validator.call ([])
			end
		end

feature -- Status report

	is_valid: BOOLEAN
			-- Does this paramater have a valid value?

feature {NONE} -- Implementation

	status: EV_LABEL
			-- User-readable status

	parameter_widget: EV_WIDGET
			-- Widget holding parameter value

	validate_from_key (a_key: EV_KEY) is
			-- Key relaese actions wrapper for `validate'
		do
			if a_key /= Void then
				validate (True)
			end
		end

end
