indexing
	description: "Application commands view"
	author: "Zoran Simic"
	date: "$Date: 2008/12/01 $"
	revision: "$Revision: #8 $"

class
	ROSE_COMMAND_VIEW

inherit
	ROSE_SHARED_GLOBAL_GUI

create
	make

feature {NONE} -- Initialization

	make (a_container: EV_CONTAINER) is
			-- Create a view on application commands and put it in 'a_container' if provided.
		local
			l_seq: SEQUENCE [STRING]
			l_button: EV_TOOL_BAR_BUTTON
		do
			box := Gui.factory.new_vbox (a_container,1,4)
				--| Application commands
			create toolbar
			l_seq := sorted (Global.commands.categories.current_keys.linear_representation)
			if l_seq /= Void then
				from
					l_seq.start
				until
					l_seq.off
				loop
					toolbar.extend (toolbar_item (l_seq.item))
					l_seq.forth
				end
			end
			toolbar.extend (toolbar_item (s_all))
			create l_button.default_create
			Gui.factory.decorate_toolbar_item (l_button,[Gui.rep.pixmaps.Information_pixmap,agent show_legend,"Show legend",s_legend])
			toolbar.extend (l_button)
			box.extend (toolbar)
			box.disable_item_expand (toolbar)
			grid := Gui.factory.new_grid (box,<<["Id",150],["Signature",315],["Description",400]>>,agent on_grid_click,0,0)
			info_bar := Gui.factory.new_label (box,Void,1)
			refresh (Global.commands.application_category)
		end

feature -- Access

	box: EV_BOX
			-- Box holding the whole view

	toolbar: EV_TOOL_BAR

feature -- Basic operations

	refresh (a_category: STRING) is
			-- Refresh current view.
		local
			l_cat: ROSE_SERVICE_MANAGER [ROSE_COMMAND]
			l_category, l_command: STRING
		do
			if a_category /= Void and then not a_category.is_empty then
				if Global.commands.has_category (a_category) then
					l_category := a_category
				else
					l_cat := Global.commands.command_category (a_category)
					if l_cat /= Void then
						l_category := l_cat.id
						l_command := a_category
					end
				end
				if l_category = Void then
					l_category := s_all
				end
				Gui.factory.select_by_name (toolbar,l_category,True)
				show_commands (l_category)
				if l_command /= Void then
				end
			end
		end

feature {NONE} -- Implementation

	grid: RP_DATA_GRID[ANY]
	info_bar: EV_LABEL
			-- Widgets showing application commands

	show_commands (a_category: STRING) is
			-- Show commands in 'a_category' in right pane.
		local
			l_category: ROSE_SERVICE_MANAGER [ROSE_COMMAND]
		do
			grid.clear
			grid.set_titles (<<"Id","Signature","Description">>)
			grid.set_column_widths (<<150,315,400>>)
			if a_category = Void or else equal (a_category,s_all) then
				l_category := Global.commands
			else
				l_category := Global.commands.category (a_category)
			end
			if l_category /= Void then
				l_category.iterate (agent add_command,True)
				info_bar.set_text (Global.formatted ("$1 commands",<<l_category.count>>))
			else
				info_bar.set_text ("No commands")
			end
		end

	show_legend is
			-- Show icon legend.
		do
			grid.clear
			grid.set_titles (<<"Icon","Description">>)
			grid.set_column_widths (<<150,715>>)
			grid.extend (legend_row (Void,Global.commands.alias_category,"Command aliases","alias"))
			grid.extend (legend_row (Void,Global.commands.application_category,"Commands defined by application",Global.commands.application_category))
			grid.extend (legend_row (Void,Global.commands.default_category,"Default framework commands","default"))
			grid.extend (legend_row (Void,Global.commands.gui_category,"Default framework GUI commands","gui"))
			grid.extend (legend_row (Void,s_all,"All commands",s_all))
			grid.extend (legend_row (pixmap_gui,"GUI commands","Application GUI commands",Void))
			grid.extend (legend_row (pixmap_simple,"Simple commands","Commands implemented with an agent (no dedicated class)",Void))
			grid.extend (legend_row (pixmap_unsigned_command,"Unsigned commands","Commands with no require parameters",Void))
		end

	legend_row (a_pixmap: EV_PIXMAP; a_name, a_description: STRING; a_data: ANY): RP_GRID_NODE[ANY] is
		local
			l_node_array: ROSE_LINEAR_ARRAY[RP_DATA_GRID_ITEM]
			l_pixmap: EV_PIXMAP
		do
			create l_node_array.make_empty
			l_pixmap := a_pixmap
			if l_pixmap = Void then
				l_pixmap := get_pixmap (a_name)
			end
			l_node_array.extend (get_label_item (a_name, l_pixmap))
			l_node_array.extend (get_label_item (a_description, Void))

			create Result.make_from_array(l_node_array)
			Result.set_data (a_data)
		end

	add_command (a_command: ROSE_COMMAND) is
		local
			l_node_array: ROSE_LINEAR_ARRAY[RP_DATA_GRID_ITEM]
			l_node: RP_GRID_NODE[ANY]
		do
			if a_command /= Void then
				create l_node_array.make_empty

				l_node_array.extend (get_label_item (a_command.id, get_pixmap (a_command.id)))
				l_node_array.extend (get_label_item (a_command.signature, Void))
				l_node_array.extend (get_label_item (a_command.quick_help, Void))

				create l_node.make_from_array (l_node_array)
				l_node.set_data (a_command)
				grid.extend (l_node)
			end
		end

	on_grid_click (a_button: INTEGER; a_node: RP_GRID_NODE[ANY]) is
		local
			l_command: ROSE_COMMAND
			l_string: STRING
		do
			if a_button = 1 then
				if a_node /= Void then
					l_command ?= a_node.data
					if l_command /= Void then
						Global.commands.execute ("show " + l_command.id)
					else
						l_string ?= a_node.data
					end
					if l_string /= Void then
						Gui.factory.select_by_name (toolbar,l_string,True)
					end
				end
			end
		end

	is_significant (a_string: STRING): BOOLEAN is
			-- Has 'a_string' a significant value?
		do
			Result := a_string /= Void and then not a_string.is_empty
		end

	get_pixmap (a_name: STRING): EV_PIXMAP is
			-- Pismap associated to category or command with 'a_name'
		local
			l_cat: ROSE_SERVICE_MANAGER [ROSE_COMMAND]
			l_category: STRING
		do
			if Global.commands.has_category (a_name) then
				l_category := a_name
			else
				l_cat := Global.commands.command_category (a_name)
				if l_cat /= Void then
					l_category := l_cat.id
				end
				if equal (l_category,Global.commands.application_category) then
					Result := command_pixmap (Global.commands.service (a_name))
				end
			end
			if Result = Void then
				Result := pixmaps.item (l_category)
			end
			if Result = Void then
				Result := pixmap_default
			end
		end

	command_pixmap (a_command: ROSE_COMMAND): EV_PIXMAP is
			-- Pixmap associated to the non-framework 'a_command'
		require
			valid_command: a_command /= Void
		local
			l_window: RCMD_WINDOW
			l_simple: ROSE_SIMPLE_COMMAND
		do
			l_window ?= a_command
			if l_window /= Void then
				Result := pixmap_gui
			end
			if Result = Void then
				l_simple ?= a_command
				if l_simple /= Void then
					Result := pixmap_simple
				end
			end
			if Result = Void then
				if not is_significant (a_command.signature) then
					Result := pixmap_unsigned_command
				end
			end
		end

	pixmaps: HASH_TABLE [EV_PIXMAP,STRING] is
			-- Predefined pixmaps
		once
			create Result.make (0)
			Result.put (Gui.factory.new_pixmap ("cfg_directive.ico"),Global.commands.alias_category)
			Result.put (Gui.factory.new_pixmap ("cfg_app_vars.ico"),"application")
			Result.put (Gui.factory.new_pixmap ("cfg_default_command.ico"),Global.commands.default_category)
			Result.put (Gui.factory.new_pixmap ("cfg_gui_command.ico"),Global.commands.gui_category)
			Result.put (Gui.factory.new_pixmap ("cfg_node_open.ico"),s_all)
		end

	pixmap_default: EV_PIXMAP is once Result := Gui.factory.new_pixmap ("cfg_node.ico") end
	pixmap_gui: EV_PIXMAP is once Result := Gui.factory.new_pixmap ("cfg_app_config.ico") end
	pixmap_simple: EV_PIXMAP is once Result := Gui.factory.new_pixmap ("cfg_leaf.ico") end
	pixmap_unsigned_command: EV_PIXMAP is once Result := Gui.factory.new_pixmap ("cfg_env_vars.ico") end
	s_all: STRING is "All"
	s_legend: STRING is "Legend"
			-- Constants

	toolbar_item (a_name: STRING): EV_TOOL_BAR_BUTTON is
		do
			create Result.default_create
			Gui.factory.decorate_toolbar_item (Result,[get_pixmap (a_name),agent show_commands (a_name),"Show " + a_name + " commands",a_name])
		end

	sorted (a_set: LINEAR [STRING]): SORTED_TWO_WAY_LIST [STRING] is
			-- Sorted 'a_set'
		do
			if a_set /= Void then
				create Result.make
				from
					a_set.start
				until
					a_set.off
				loop
					Result.extend (a_set.item)
					a_set.forth
				end
			end
		end

feature {NONE} -- Implementation

	get_label_item (a_text: STRING; a_pixmap: EV_PIXMAP): RP_DATA_GRID_LABEL_ITEM is
			-- create a 'RP_RP_DATA_GRID_LABEL_ITEM' with scaled 'a_pixmap' and 'a_text'
		local
			l_pixmap: EV_PIXMAP
			l_factor: DOUBLE
		do
			create Result.make_with_text (a_text)
			if a_pixmap /= Void then
				create l_pixmap
				if grid.is_row_height_fixed and then a_pixmap.height > grid.row_height then
					l_pixmap.copy (a_pixmap)
					l_factor := grid.row_height / l_pixmap.height
					l_pixmap.stretch ((l_factor * l_pixmap.width).rounded, grid.row_height)
					Result.set_pixmap (l_pixmap)
				else
					Result.set_pixmap (a_pixmap)
				end
			end
		end

end -- class ROSE_COMMAND_VIEW
