indexing
	description: "Collection of rose capital actions by listing and exdate (aka option)"
	author: "Janet Small"
	last_editor: "$Author: it $"
	date: "$Date: 2014/03/14 $"
	revision: "$Revision: #24 $"
	archive: "$File: //rose/source/kernel/capital_actions/rose_complex_capital_action.e $"
	path : "$File: //rose/source/kernel/capital_actions/rose_complex_capital_action.e $"

class ROSE_COMPLEX_CAPITAL_ACTION

inherit

	IEEE_MATH
		redefine
			is_equal
		end

	SHARED_TODAY
		export
			{NONE} all
		redefine
			is_equal
		end

	ROSE_CAPACT_DATE_USE
		undefine
			is_equal
		end

	ELOG_SHARED_LOGGER
		undefine
			is_equal
		end

create

	make,
	make_with_signature

feature {NONE} -- Creation

	make_with_signature (a_option_signature: CAPACT_OPTION_SIGNATURE_RECORD; a_expressions: ROSE_LINEAR_ARRAY[CAPACT_EXPRESSION]) is
			-- make with signature and expressions of component capital actions
		do
			option_signature := a_option_signature
			create capact_signatures.make_empty
			create capact_expressions.make_empty
			exdate := safe_date_from_ccyymmdd (a_option_signature.exdate)
			if a_expressions /= Void then
				add_capact_expressions (a_expressions)
			else
				--try to get the capital_action_expressions from the database ??
			end
		end

	make (a_option_signature: CAPACT_OPTION_SIGNATURE_RECORD;a_signatures: ROSE_LINEAR_ARRAY[CAPACT_SIGNATURE_RECORD]) is
			-- make with signature and signatures of component capital actions
		require
			has_signatures: a_signatures /= Void
		local
			l_option_expression_string :STRING
			l_status_code: STRING
		do
			capact_signatures := a_signatures -- any reason to copy this?
			option_signature := a_option_signature
			if capact_signatures /= Void and then capact_signatures.count > 0 then
				if capact_signatures.count = 1 or
					(capact_signatures.count = 2 and then
					 capact_signatures.item (1) /= Void and then
					 capact_signatures.item (2) /= Void and then
					 capact_signatures.item (1).member_number < capact_signatures.item (2).member_number) then
					-- A little speed up
					do_nothing
				else
					capact_signatures.sort_by (agent member_comparison)
				end
				option_expression_string := a_option_signature.option_expression
				instantiate_capact_expressions
				exdate := safe_date_from_ccyymmdd (a_option_signature.exdate)
				if option_expression_string /= Void then
					l_option_expression_string := option_expression_string.twin
				else
					l_option_expression_string := ""
				end
				l_option_expression_string.prune_all (' ')
				if option_signature.status_code /= Void then
					l_status_code := option_signature.status_code.twin
				else
					l_status_code := ""
				end
				l_status_code.prune_all (' ')
				if (not l_option_expression_string.is_empty) and then exdate.is_available and l_status_code.is_empty then
					is_valid_option := True
					capact_option_expression := option_expression
					if capact_option_expression /= Void then
						rmdat_adjustment_factor := capact_option_expression.rmdat_adjustment_factor
					end
				end
			end
		end

feature -- Access

	signature: like ur_target -- signature of rose complex action (aka option) is
			-- use option_signature
		do
			Result := option_signature
		end

	option_number: INTEGER is
			-- distinguish among multiple alternative ways applying capital actions on this exdate
			-- eg. may have currency alternatives, or may be a stock vs cash alternative, or take up rights vs lapse, or whatever
			-- anticipated in the future for investment side, but not yet implemented
		do
			Result := option_signature.option_number
		end

	option_status: STRING is
			-- exception status of option
		require
			has_option_signature: option_signature /= Void
		do
			if option_signature.status_code /= Void then
				Result := option_signature.status_code
			else
				Result := " "
			end
		end

	capital_actions: ROSE_LINEAR_ARRAY [CAPACT_CAPITAL_ACTION]
			-- all capital actions for listing on exdate

	add_capital_action (a_capact: CAPACT_CAPITAL_ACTION) is
			-- add capital action to set
		require
			valid_capital_action: a_capact /= Void
		do
			if capital_actions = Void then
				create capital_actions.make_empty
			end
			capital_actions.extend (a_capact)
		end

	replace_capital_action (a_capact: CAPACT_CAPITAL_ACTION) is
		require
			valid_capital_action: a_capact /= Void
		local
			l_sig: CAPACT_SIGNATURE_RECORD
			l_found: BOOLEAN
		do
			if capital_actions = Void then
				create capital_actions.make_empty
				capital_actions.extend (a_capact)
			else
				from capital_actions.start
				until capital_actions.off or l_found
				loop
					l_sig := capital_actions.current_item.signature
					if a_capact.signature.multi_key.is_equal (l_sig.multi_key) then
						capital_actions.replace (a_capact)
						l_found := True
					end
					capital_actions.forth
				end
			end
		end

	add_capital_actions (a_capacts: ROSE_LINEAR_ARRAY [CAPACT_CAPITAL_ACTION]) is
			--
		require
			valid_capital_actions: a_capacts /= Void
		do
			if capital_actions = Void then
				create capital_actions.make_empty
			end
			capital_actions.append_array (a_capacts)
		end

	first_capital_action: CAPACT_CAPITAL_ACTION is
			-- First capital action if any
		do
			if capital_actions /= Void and then capital_actions.count >= 1 then
				Result := capital_actions.item (1)
			end
		end

feature -- Element modification

	set_listing (a_listing: LISTING) is
			-- Set 'listing_saved' to 'a_listing'
		do
			listing_saved := a_listing
		end

	set_option_number (a_value: like option_number) is
			-- convenience feature for access
		require
			option_number_valid: a_value > 0
		do
			option_signature.set_option_number (a_value)
		end

	set_rmdat_adjustment_factor (a_rmdat_adjustment_factor: like rmdat_adjustment_factor) is
			-- Set 'rmdat_adjustment_factor' to 'a_rmdat_adjustment_factor'
		do
			rmdat_adjustment_factor := a_rmdat_adjustment_factor
		end

feature -- used for return computation	

	cash_inflow (a_currency: CURRENCY):DOUBLE is
			-- cash inflow to a portfolio holding one share of parent as a result of capital actions
		do
			if capact_option_expression /= Void then
				Result := capact_option_expression.cash_inflow (a_currency)
			else
				Result := qnan
			end
		end

	cash_dividend (a_currency: CURRENCY):DOUBLE is
			-- cash dividend to a portfolio holding one share of parent as a result of capital actions
		do
			if capact_option_expression /= Void then
				Result := capact_option_expression.cash_dividend (a_currency)
			else
				Result := qnan
			end
		end

	cash_outflow (a_currency: CURRENCY):DOUBLE is
			-- cash spent by a portfolio holding one share of parent as a result of capital actions
		do
			if capact_option_expression /= Void then
				Result := capact_option_expression.cash_outflow (a_currency)
			else
				Result := qnan
			end
		end

feature--Access

	is_valid_option: BOOLEAN
			-- Is `Current' valid?

	is_equal (a_other: like Current): BOOLEAN is
			-- Is 'a_other' equal to 'Current'
		do
			Result := a_other.capact_signatures.is_equal (capact_signatures)
		end

	rmdat_adjustment_factor: DOUBLE

	exdate: ROSE_DATE
			-- Ex-date of capital action

	price_adjustment_factor: DOUBLE

	price_adjustment_factor_available :BOOLEAN is
		do
			Result :=  is_defined_double (price_adjustment_factor)
		end

	has_terminal_distribution: BOOLEAN

	has_split_and_cash_dividend: BOOLEAN is
		do
			Result := has_split and has_cash_dividend
		end

	has_split: BOOLEAN is
		do
			if capact_expressions /= Void then
				from capact_expressions.start	until capact_expressions.off  loop
					if capact_expressions.current_item.target.is_valid_status then
						Result := Result or (capact_expressions.current_item.target.capact_type_id=2) or
							(capact_expressions.current_item.target.capact_type_id=3)
					end
					capact_expressions.forth
				end
			end
		end

	has_cash_dividend: BOOLEAN is
		do
			if capact_expressions /= Void then
				from capact_expressions.start	until capact_expressions.off  loop
					if capact_expressions.current_item.target.is_valid_status then
						Result := Result or (capact_expressions.current_item.target.capact_type_id = 1)
					end
					capact_expressions.forth
				end
			end
		end

	has_rights: BOOLEAN is
			-- does complex action have rights expression
		do
			if capact_expressions /= Void then
				from capact_expressions.start	until capact_expressions.off  loop
					if capact_expressions.current_item.target.is_valid_status then
						Result := Result or (capact_expressions.current_item.target.capact_type_id = 13)
						Result := Result or (capact_expressions.current_item.target.capact_type_id = 14)
						Result := Result or (capact_expressions.current_item.target.capact_type_id = 15)
						Result := Result or (capact_expressions.current_item.target.capact_type_id = 16)
						Result := Result or (capact_expressions.current_item.target.capact_type_id = 6)

					end
					capact_expressions.forth
				end
			end
		end

	has_bonus: BOOLEAN is
		do
			if capact_expressions /= Void then
				from capact_expressions.start	until capact_expressions.off  loop
					if capact_expressions.current_item.target.is_valid_status then
						Result := Result or (capact_expressions.current_item.target.capact_type_id = 4)
						Result := Result or (capact_expressions.current_item.target.capact_type_id = 10)
						Result := Result or (capact_expressions.current_item.target.capact_type_id = 11)
						Result := Result or (capact_expressions.current_item.target.capact_type_id = 12)
					end
					capact_expressions.forth
				end
			end
		end

	has_stock_dividend: BOOLEAN is
			--
		do
			if capact_expressions /= Void then
				from capact_expressions.start	until capact_expressions.off  loop
					if capact_expressions.current_item.target.is_valid_status then
						Result := Result or (capact_expressions.current_item.target.capact_type_id = 5)
					end
					capact_expressions.forth
				end
			end
		end

	has_distribution: BOOLEAN is
		do
			if capact_expressions /= Void then
				from capact_expressions.start	until capact_expressions.off  loop
					if capact_expressions.current_item.target.is_valid_status then
						Result := Result or (capact_expressions.current_item.target.capact_type_id = 9)
					end
					capact_expressions.forth
				end
			end
		end

	has_multiple_capacts: BOOLEAN is
			--
		do
			if capact_expressions /= Void then
				Result := capact_expressions.count > 1
			end
		end

	option_signature: CAPACT_OPTION_SIGNATURE_RECORD

	capact_signatures: ROSE_LINEAR_ARRAY [CAPACT_SIGNATURE_RECORD]

	capact_expressions: ROSE_LINEAR_ARRAY [CAPACT_EXPRESSION]

	add_capact_expression (a_expression: CAPACT_EXPRESSION) is
		do
			if a_expression.target.member_number < capact_signatures.count then
				--Assuming that capact_signatures.count = total number of members and goes from member 1 to n.
				-- need to replace an existing expression, not add a new one
			elseif a_expression.target.member_number = capact_signatures.count + 1 then
				capact_signatures.extend (a_expression.target)
				capact_expressions.extend (a_expression)
			else
				-- cannot extend signatures and expressions out of order!
				exceptions.raise("out of order extension to capact_signatures members in class CAPACT_OPTION")
			end
		end

	add_capact_expressions (a_expressions: ROSE_LINEAR_ARRAY[CAPACT_EXPRESSION]) is
		do
			a_expressions.sort_by( agent member_expression_comparison)
			if capact_expressions = Void then
				create capact_expressions.make_empty
			end
			capact_expressions.append_array (a_expressions)
			from capact_expressions.start
			until capact_expressions.off
			loop
				capact_signatures.extend (capact_expressions.current_item.target)
				capact_expressions.forth
			end
		end

	max_member_number: INTEGER is
		do
			Result := capact_signatures.count
		end

	capact_option_expression: CAPACT_EXPRESSION

	listing: LISTING is
			--
		require
			valid_signature: option_signature /= Void and then
			option_signature.instrument_id > 0 and option_signature.market_id > 0
		do
			Result := option_signature.listing
		end

	option_expression_string: STRING
			-- an data item in the signature
			--F3 { F2{F1}}}
			-- F1 + F2 + F3

	instantiate_capact_expressions  is
			-- Instantiate company event options
		local
			l_capact: CAPACT_EXPRESSION
			l_potential_terminal_distribution_index: INTEGER
			l_child_is_parent_count: INTEGER
		do
			create capact_expressions.make_empty
			from
				capact_signatures.start
			until
				capact_signatures.after
			loop
				l_capact := instantiated_capact_expression (capact_signatures.current_item)
				if l_capact /= Void then
					capact_expressions.extend (l_capact)
					if capact_signatures.current_item.is_valid_status and then l_capact.is_terminal_distribution then
						l_potential_terminal_distribution_index := capact_expressions.count
					end
				end
				capact_signatures.forth
			end
			if l_potential_terminal_distribution_index > 0 then
				l_child_is_parent_count := child_is_parent_count (capact_expressions)
				has_terminal_distribution := l_child_is_parent_count < 2
				if not has_terminal_distribution then
					capact_expressions.item (l_potential_terminal_distribution_index).set_rmdat_adjustment_factor (0.0)
					capact_expressions.item (l_potential_terminal_distribution_index).set_is_terminal_distribution (False)
				end
			end
		end

	child_is_parent_count (a_capacts: like capact_expressions): INTEGER is
			-- Number of distributions in `a_capacts' where parent and child instruments are same
		do
			from
				a_capacts.start
			until
				a_capacts.after
			loop
				if child_is_parent (a_capacts.current_item) then
					Result := Result + 1
				end
				a_capacts.forth
			end
		end

	child_is_parent (a_capact: CAPACT_EXPRESSION): BOOLEAN is
			-- Is `a_capact' distribution where child instrument is parent instrument?
		do
			Result := attached {CAPACT_LEGACY_DISTRIBUTION_EXPRESSION} a_capact as l_distribution and then l_distribution.child_is_parent
		end

	instantiated_capact_expression (a_target: CAPACT_SIGNATURE_RECORD): CAPACT_EXPRESSION is
			-- expression instantiation
		do
			inspect a_target.capact_type_id
			when 1 then
				create {CAPACT_CASH_DIVIDEND_EXPRESSION} Result.instantiate (a_target)
			when 2 then
				create {CAPACT_SPLIT_EXPRESSION} Result.instantiate (a_target)
			when 3 then
				create {CAPACT_SPLIT_EXPRESSION} Result.instantiate (a_target)
			when 4 then
				--				create {CAPACT_STOCK_BONUS_EXPRESSION} Result.instantiate (a_target)
				create {CAPACT_BONUS_IN_SAME_EXPRESSION} Result.instantiate (a_target)
			when 5 then
				create {CAPACT_STOCK_DIVIDEND_EXPRESSION} Result.instantiate (a_target)
			when 6 then
				create {CAPACT_EXERCISE_RIGHTS_EXPRESSION} Result.instantiate (a_target)
			when 7 then
				create {CAPACT_CAPITAL_RETURN_OF_CASH_EXPRESSION} Result.instantiate (a_target)
			when 8 then
				create {CAPACT_RETURN_ON_SHARE_PREMIUM_EXPRESSION} Result.instantiate (a_target)
			when 9 then
				create {CAPACT_LEGACY_DISTRIBUTION_EXPRESSION} Result.instantiate (a_target)
			when 10 then
				create {CAPACT_BONUS_IN_OTHER_EXPRESSION} Result.instantiate (a_target)
			when 11 then
				create {CAPACT_RENOUNCEABLE_BONUS_IN_SAME_EXPRESSION} Result.instantiate (a_target)
			when 12 then
				create {CAPACT_RENOUNCEABLE_BONUS_IN_OTHER_EXPRESSION} Result.instantiate (a_target)
			when 13 then
				create {CAPACT_RIGHTS_IN_SAME_EXPRESSION} Result.instantiate (a_target)
			when 14 then
				create {CAPACT_RIGHTS_IN_OTHER_EXPRESSION} Result.instantiate (a_target)
			when 15 then
				create {CAPACT_RENOUNCEABLE_RIGHTS_IN_SAME_EXPRESSION} Result.instantiate (a_target)
			when 16 then
				create {CAPACT_RENOUNCEABLE_RIGHTS_IN_OTHER_EXPRESSION} Result.instantiate (a_target)
			when 17 then
				create {CAPACT_WRITEUP_OF_CAPITAL_EXPRESSION} Result.instantiate (a_target)
			when 18 then
				create {CAPACT_WRITEDOWN_OF_CAPITAL_EXPRESSION} Result.instantiate (a_target)
			when 19 then
				create {CAPACT_PUBLIC_OFFERING_EXPRESSION} Result.instantiate (a_target)
			when 20 then
				create {CAPACT_PRELIMINARY_SUBSCRIPTION_EXPRESSION} Result.instantiate (a_target)
			when 21 then
				create {CAPACT_REDENOMINATION_EXPRESSION} Result.instantiate (a_target)
			when 22 then
				create {CAPACT_RENOMINALIZATION_EXPRESSION} Result.instantiate (a_target)
			when 23 then
				create {CAPACT_CALL_PAYMENT_EXPRESSION} Result.instantiate (a_target)
			when 24 then
				create {CAPACT_STOCK_DIVIDEND_IN_OTHER_EXPRESSION} Result.instantiate (a_target)
			else
  				(create {EXCEPTIONS}).raise ("Unknown capital action type "+a_target.capact_type_id.out)
			end
			Result.set_target (a_target)
			Result.set_primary_capact_option (Current)
		end

	description: STRING is
		do
			if option_expression/=void then
				Result := option_expression.name
			else
				Result := "Invalid"
			end
		end

	capact_expression_factory: CAPACT_EXPRESSION_FACTORY is
			-- capact expression factory
		once
			create Result
		end

	option_expression: CAPACT_EXPRESSION is
		local
			l_retried: BOOLEAN
		do
			if not l_retried then
				capact_expression_factory.set_capact_expressions (capact_expressions)
				parser.error.flush_and_acknowledge
				Result := parser.parsed_string (option_expression_string)
				if parser.error.signaled then
					logger.error ("Invalid capact expression `" + option_expression_string + "' for #r" + listing.xrf_zone_display + " @ " + exdate.ccyymmdd.out)
				end
				parser.error.flush_and_acknowledge
			end
		rescue
			if not l_retried then
				l_retried := True
				retry
			end
		end

feature -- Adding options

	current_option_count: INTEGER is
		local
			l_result: CELL [INTEGER]
			l_options: ARRAY [CAPACT_OPTION_SIGNATURE_RECORD]
		do
			create l_result.put (0)
			if option_signature.listing_id /=0 and option_signature.exdate /= 0 and first_capital_action /= Void then
				l_options := option_signature.signatures_by_instrument_market_and_exdate (option_signature.instrument_id, option_signature.market_id, option_signature.exdate)
				l_options.do_all (agent max_option_number (l_result, ?))
			end
			Result := l_result.item
		end

feature {NONE} -- Implementation

	max_option_number (a_max_option_count: CELL [INTEGER]; a_option: CAPACT_OPTION_SIGNATURE_RECORD) is
			-- Keep maximum of `a_max_option_count.item' and `a_option.option_number' in `a_max_option_count'
		require
			a_max_option_count_not_void: a_max_option_count /= Void
			a_option_not_void: a_option /= Void
		do
			a_max_option_count.put (a_max_option_count.item.max (a_option.option_number))
		ensure
			definition: a_max_option_count.item = a_option.option_number.max (old a_max_option_count.item)
		end

feature -- Error checking

	option_expression_validity (a_string: STRING):PAIR[BOOLEAN,STRING] is
			-- checks for some validity conditions on the potential option expression string
			-- and returns a message with the warning
		local
			l_expression: CAPACT_EXPRESSION
			l_substring: STRING
		do
			capact_expression_factory.set_capact_expressions (capact_expressions)
			l_expression := parser.parsed_string (a_string)
			if a_string = Void or else a_string.count=0 then
				create Result.make (True,"Empty Option Expression String: No capital actions will be visible for return/dividend yield computation purposes")
			elseif l_expression  = Void then
				create Result.make (False,"Error: Cannot parse the string to an expression succesfully")
			elseif has_multiple_valid_splits then
				if not a_string.has_substring ("@") then
					create Result.make (False,"Error: Muliple valid splits participate in this option, there should be a sequential operator @")
				else
					create Result.make (True, "Option expression string validated")
				end
			else
				from capact_expressions.start until capact_expressions.off or else (Result /= Void and then Result.first = False) loop
					l_substring := "F"+capact_expressions.current_item.target.member_number.out
					if a_string.has_substring (l_substring) then
						if not capact_expressions.current_item.target.is_valid_status then
							create Result.make (False,"Error: Option expression string references an invalid capital action")
						else
							create Result.make (True, "Option expression string validated")
						end
					else
						if capact_expressions.current_item.target.is_valid_status then
							create Result.make (True, "Warning: Potentially ok, valid capital action " + l_substring + "not referenced in the expression string")
						else
							create Result.make (True, "Option expression string validated")
						end
					end
					capact_expressions.forth
				end
			end
		end

	has_multiple_valid_splits: BOOLEAN is
		local
			l_count :INTEGER
		do
			from capact_expressions.start
			until capact_expressions.off
			loop
				if capact_expressions.current_item.template_name.as_upper.has_substring ("SPLIT")
					and capact_expressions.current_item.target.is_valid_status then
					l_count := l_count + 1
				end
				capact_expressions.forth
			end
			if l_count >1 then
				Result := True
			end
		end

feature{NONE} -- Implementation

	ur_target, ur_complex_capital_action_target: CAPACT_OPTION_SIGNATURE_RECORD is
		once
			create Result
		end

	ur_capact_target, ur_component_capital_action_target: CAPACT_SIGNATURE_RECORD is
		once
			create Result
		end

	exceptions:	EXCEPTIONS is
		once
			create Result
		end

	listing_saved: LISTING

	parser: RX_EXPRESSION_PARSER [CAPACT_EXPRESSION] is
		once
			create Result.make (capact_expression_factory)
		end

	member_comparison (a,b: like ur_capact_target): BOOLEAN is
			--
		do
			Result := a.member_number < b.member_number
		end

	member_expression_comparison (a,b: CAPACT_EXPRESSION): BOOLEAN is
			--
		do
			Result := a.target.member_number < b.target.member_number
		end

end
