note

	description:

		"Collection of Complex Capital Actions"

	author: "Nelson Chui, Janet Small"
	last_editor: "$Author: e2 $"
	date: "$Date: 2014/11/07 $"
	revision: "$Revision: #9 $"
	path: "$File: //rose/source/kernel/capital_actions/rose_complex_capital_actions.e $"

class ROSE_COMPLEX_CAPITAL_ACTIONS

inherit

	CAPITAL_ACTIONS_CONTEXT_SHARED

	SHARED_CAPACT_RETRIEVAL
		export {NONE} all end

create

	make

feature -- Initialization

	make (a_listing: LISTING; a_start_date, a_end_date: ROSE_DATE; a_valid_only: BOOLEAN) is
			-- make
		require
			a_listing_not_void: a_listing /= Void
		local
			l_start_date, l_end_date, i, l_ix: INTEGER
			l_action_cache: like action_cache_by_database
			l_option_target, l_last_option_target: CAPACT_OPTION_SIGNATURE_RECORD
			l_option_targets: ROSE_LINEAR_ARRAY [CAPACT_OPTION_SIGNATURE_RECORD]
			l_capact_targets, l_capact_targets_by_exdate, l_capact_neighborhood: ROSE_LINEAR_ARRAY [CAPACT_SIGNATURE_RECORD]
			l_action: ROSE_COMPLEX_CAPITAL_ACTION
			l_opt_lower, l_opt_upper, l_cap_lower, l_cap_upper, l_upper, l_lower: INTEGER
			l_pair:PAIR [CAPACT_OPTION_SIGNATURE_RECORD, ROSE_COMPLEX_CAPITAL_ACTION]
			l_pairs: ROSE_LINEAR_ARRAY [PAIR [CAPACT_OPTION_SIGNATURE_RECORD, ROSE_COMPLEX_CAPITAL_ACTION]]
			l_missed_action: BOOLEAN
		do
			create actions.make_empty
			if a_listing /= Void and then a_listing.handle > 0 then
				if a_start_date = void then
					l_start_date := constants.minimum_date_time.ccyymmdd
				else
					l_start_date := a_start_date.ccyymmdd
				end
				if a_end_date = void then
					l_end_date := constants.maximum_date_time.ccyymmdd
				else
					l_end_date := a_end_date.ccyymmdd
				end

				l_cap_lower :=  constants.maximum_date_time.ccyymmdd
				l_cap_upper := constants.minimum_date_time.ccyymmdd
				create l_pairs.make_empty
				l_opt_lower := lower_range (l_start_date + 1) -- We don't care about the capital actions happened on start date, save some retrieval time
				l_opt_upper := upper_range (l_end_date)

				create l_option_target
				l_option_target.set_listing_id (a_listing.handle)
				l_option_targets := l_option_target.data_manager.retrieve_current_signatures (
						<<l_option_target>>,
						"#listing_id.id = listing_id and exdate between '" + l_opt_lower.out + "' and '" + l_opt_upper.out + "'"
					)
				l_option_targets.sort_by (
						-- The following code relies on ordered array to properly link pairs
						agent (a_1, a_2: CAPACT_OPTION_SIGNATURE_RECORD): BOOLEAN
							do
								Result := a_1.exdate < a_2.exdate
							end
					)
				l_action_cache := action_cache_by_database (l_option_target.data_manager.data_server_database)
				from i := l_option_targets.lower
				until i > l_option_targets.upper
				loop
					l_option_target := l_option_targets.item (i)
					if l_option_target.exdate > l_opt_lower and l_option_target.exdate <= l_opt_upper then
						if l_last_option_target = Void or else l_last_option_target.exdate /= l_option_target.exdate then -- Right now, we only choose the first option
							l_action := l_action_cache.item (l_option_target.signature_id)
							if l_action = Void then
								l_cap_upper := l_cap_upper.max (l_option_target.exdate)
								l_cap_lower := l_cap_lower.min (l_option_target.exdate)
								create l_pair.make (l_option_target, Void)
								l_pairs.extend (l_pair)
								if l_option_target.exdate > l_start_date or l_option_target.exdate <= l_end_date then
									l_missed_action := True
								end
							else
								if a_valid_only implies l_action.is_valid_option then
									create l_pair.make (l_option_target, l_action)
									l_pairs.extend (l_pair)
								end
							end
							l_last_option_target := l_option_target
						end
					end
					i := i + 1
				end
				-- Retrieve uncached capital actions
				if l_missed_action then
					l_upper := upper_range (l_cap_upper)
					l_lower := lower_range (l_cap_lower)
					l_capact_targets := capact_retrieval.capact_signatures_by_listing_id_and_exdate_between (a_listing.handle, l_lower, l_upper)

					create l_capact_neighborhood.make_empty
					from i := l_capact_targets.lower
					until i > l_capact_targets.upper
					loop
						l_capact_neighborhood.extend (l_capact_targets.item (i).twin)
						i := i + 1
					end

					l_capact_neighborhood.sort_by (agent order_by_signature_id)
					from i := l_capact_targets.lower
					until i > l_capact_targets.upper
					loop
						l_capact_targets.item (i).set_zone (a_listing.zone_mnemonic)
						l_capact_targets.item (i).set_target_neighborhood (l_capact_neighborhood)
						i := i + 1
					end
					from i := l_pairs.lower; l_ix := l_capact_targets.lower
					until i > l_pairs.upper or l_ix > l_capact_targets.upper
					loop
						l_pair := l_pairs.item (i)
						if l_capact_targets.item (l_ix).exdate < l_pair.first.exdate then
							l_ix := l_ix + 1
						elseif l_capact_targets.item (l_ix).exdate > l_pair.first.exdate then
							i := i + 1
						elseif l_pair.second = Void then
							from create l_capact_targets_by_exdate.make_empty
							until l_ix > l_capact_targets.upper or l_capact_targets.item (l_ix).exdate > l_pairs.item (i).first.exdate
							loop
								l_capact_targets_by_exdate.extend (l_capact_targets.item (l_ix))
								l_ix := l_ix + 1
							end
							create l_action.make (l_pair.first, l_capact_targets_by_exdate)
							l_pair.set_second (l_action)
							l_action_cache.put (l_action, l_pair.first.signature_id)
						else
							i := i + 1
						end
					end
				end
				from i := l_pairs.lower
				until i > l_pairs.upper
				loop
					l_pair := l_pairs.item (i)
					if l_pair.second /= Void and then l_pair.first.exdate > l_start_date and then l_pair.first.exdate <= l_end_date then
						if a_valid_only implies l_pair.second.is_valid_option then
							actions.extend (l_pair.second)
						end
					end
					i := i + 1
				end
			end
		end

feature -- Access

	actions: ROSE_LINEAR_ARRAY [ROSE_COMPLEX_CAPITAL_ACTION]
		-- actions

feature {NONE} -- Implementation

	action_cache_by_database (a_database_name: STRING): ROSE_VOLATILE_CACHE [ROSE_COMPLEX_CAPITAL_ACTION, INTEGER] is
			-- Capital actions cache for `a_database_name'
		do
			Result := action_cache.item (a_database_name)
			if Result = Void then
				create Result.make (5000, 500)
				action_cache.extend (Result, a_database_name)
			end
		ensure
			not_void: Result /= Void
		end

	action_cache: HASH_TABLE [ROSE_VOLATILE_CACHE [ROSE_COMPLEX_CAPITAL_ACTION, INTEGER], STRING] is
			-- Set of capital actions caches indexed by database name
		once
			create Result.make (8)
		ensure
			not_void: Result /= Void
		end

	upper_range (a_ccyymmdd: INTEGER): INTEGER is
			-- upper range
		local
			i: INTEGER
		do
			from i := exdate_clusters.lower + 1
			until i > exdate_clusters.upper or Result /= 0
			loop
				if a_ccyymmdd > exdate_clusters.item (i) then
					Result := exdate_clusters.item (i - 1)
				end
				i := i + 1
			end
		end

	lower_range (a_ccyymmdd: INTEGER): INTEGER is
			-- lower range
		local
			i: INTEGER
		do
			from i := exdate_clusters.upper - 1
			until i < exdate_clusters.lower or Result /= 0
			loop
				if a_ccyymmdd <= exdate_clusters.item (i) then
					Result := exdate_clusters.item (i + 1)
				end
				i := i - 1
			end
		end

	exdate_clusters: ROSE_LINEAR_ARRAY [INTEGER] is
			-- exdate clusters
		local
			l_date: ROSE_DATE
		once
			create Result.make_empty
			Result.extend (constants.maximum_date_time.ccyymmdd)
			l_date := constants.today_rdt.twin
			l_date.month_add (-4)
			l_date.set_to_end_of_month
			Result.extend (l_date.ccyymmdd)
			l_date := constants.today_rdt.twin
			l_date.month_add (-37)
			l_date.set_to_end_of_month
			Result.extend (l_date.ccyymmdd)
			from l_date.year_add (-10); l_date.set_to_end_of_month
			until l_date < constants.minimum_date_time
			loop
				Result.extend (l_date.ccyymmdd)
				l_date.year_add (-10)
				l_date.set_to_end_of_month
			end
			Result.extend (constants.minimum_date_time.ccyymmdd)
		end

	order_by_signature_id (a_1, a_2: CAPACT_SIGNATURE_RECORD): BOOLEAN is
			-- order by signature id
		do
			Result := a_1.signature_id < a_2.signature_id
		end

end
