indexing
	description: "Class representing a not-yet computed value"
	author: "Jann Roeder"
	date: "$Date: 2008/12/01 $"
	revision: "$Revision: #3 $"
	archive: "$File: //rose/source/kernel/infrastructure/containers/rose_concurrent_future.e $"

class
	ROSE_CONCURRENT_FUTURE[G]

create
	make

feature {NONE} -- Creation

	make is
			-- Creation
		do
			create wait_event.make (True, False)
			create is_done_atomic.make
		end

feature -- Basic Operations

	set (a_value: G) is
			-- Sets the result of this Future to the given value unless this future has already been set
		do
			if not is_done_atomic.get_and_set (True) then
				item := a_value
				wait_event.set
			end
		end

	reset is
			-- Reset this Future, so it can be re-used
		local
			l_default: G
		do
			wait_event.reset
			is_done_atomic.set_item (False)
			item := l_default
		end

feature -- Access

	get: G is
			-- Waits if necessary for the computation to complete, and then retrieves its result.
		do
			wait_event.wait
			Result := item
		end

	get_with_timeout (a_timeout: INTEGER): TUPLE[result_valid: BOOLEAN; result_value: G] is
			-- Waits if necessary for at most the given time (in ms, 1 ms = 1^(-3) seconds) for the computation to complete,
			-- and then retrieves its result, if available.
			-- The field `result_valid' in the returned tuple indicates whether the result `is_value' could be obtained in time
		local
			l_default: G
		do
			if wait_event.wait_with_timeout (a_timeout) then
				Result := [True, item]
			else
				Result := [False, l_default]
			end
		end

	is_done: BOOLEAN is
			-- True if this task completed.
		do
			Result := is_done_atomic.item
		end

feature {NONE} -- Implementation

	is_done_atomic: ATOMIC_BOOLEAN

	wait_event: ROSE_CONCURRENT_EVENT

	item: G

invariant
	is_done_exists: is_done_atomic /= Void
	event_exists: wait_event /=  Void

end
