note
	description: "Parser for CFG configuration file format"
	author: "Zoran Simic"
	last_editor: "$Author: er $"
	date: "$Date: 2012/09/04 $"
	revision: "$Revision: #10 $"
	path: "$File: //rose/source/kernel/infrastructure/framework/configuration/rose_configuration_cfg_parser.e $"

class
	ROSE_CONFIGURATION_CFG_PARSER

inherit
	YY_PARSER_SKELETON
		rename
			make as make_parser_skeleton
		redefine
			report_error
		end

	ROSE_CONFIGURATION_CFG_SCANNER

	ROSE_SHARED_GLOBAL

create
	make

feature {NONE} -- Initialization

	make
			-- Create a new Eiffel parser.
		do
			make_cfg_scanner
			make_parser_skeleton
		end

feature -- Access

	id: STRING = "CFG parser"
			-- Identifier

feature -- Status report

	is_inclusion: BOOLEAN
			-- Does this parsing come from an include directive?

feature -- Basic operations

	import (a_node: ROSE_CONFIGURATION_NODE; a_path: STRING; a_inclusion: BOOLEAN)
			-- Import (or include) configuration from 'a_path' to 'a_node'.
		local
			l_file: KL_TEXT_INPUT_FILE
			l_path, rescued: STRING
		do
			if rescued /= Void then
				error.signal (id,"Cannot read config from '$1': $2",<<a_path,rescued>>)
			end
			if not error.signaled then
					--| FIXME: temporary
				if a_path.substring_index ("file:",1) = 1 then
					l_path := a_path.substring (6,a_path.count)
				else
					l_path := a_path
				end
				is_inclusion := a_inclusion
				buffer_name := l_path
				create l_file.make (l_path)
				if not l_file.exists then
					error.signal (id,"Can not import config from '$1', file does not exist",<<l_path>>)
				elseif not l_file.is_readable then
					error.signal (id,"Can not import config from '$1', file is not readable",<<l_path>>)
				end
			end
			if not error.signaled then
				root_section := a_node
				current_section := a_node
				reset
				l_file.open_read
				set_input_buffer (new_file_buffer (l_file))
				parse
				l_file.close
			end
			root_section := Void
			current_section := Void
			buffer_name := Void
			is_inclusion := False
		rescue
			if rescued = Void then
				rescued := last_exception_explanation
				retry
			end
		end

	interpret (a_node: ROSE_CONFIGURATION_NODE; a_buffer: STRING)
			-- Interpret 'a_buffer' as an CFG-style configuration in 'a_node'
		do
			root_section := a_node
			current_section := a_node
			buffer_name := "*interpreted*"
			is_inclusion := False
			reset
			set_input_buffer (new_string_buffer (a_buffer))
			parse
			buffer_name := Void
		end


feature {NONE} -- Implementation

	yy_build_parser_tables
			-- Build parser tables.
		do
			yytranslate := yytranslate_template
			yyr1 := yyr1_template
			yytypes1 := yytypes1_template
			yytypes2 := yytypes2_template
			yydefact := yydefact_template
			yydefgoto := yydefgoto_template
			yypact := yypact_template
			yypgoto := yypgoto_template
			yytable := yytable_template
			yycheck := yycheck_template
		end

	yy_create_value_stacks
			-- Create value stacks.
		do
		end

	yy_init_value_stacks
			-- Initialize value stacks.
		do
			yyvsp1 := -1
			yyvsp2 := -1
			yyvsp3 := -1
			yyvsp4 := -1
		end

	yy_clear_value_stacks
			-- Clear objects in semantic value stacks so that
			-- they can be collected by the garbage collector.
		local
			l_yyvs1_default_item: ANY
			l_yyvs2_default_item: STRING
			l_yyvs3_default_item: ROSE_CONFIGURATION_NODE
			l_yyvs4_default_item: BOOLEAN
		do
			if yyvs1 /= Void then
				yyvs1.fill_with (l_yyvs1_default_item, 0, yyvs1.upper)
			end
			if yyvs2 /= Void then
				yyvs2.fill_with (l_yyvs2_default_item, 0, yyvs2.upper)
			end
			if yyvs3 /= Void then
				yyvs3.fill_with (l_yyvs3_default_item, 0, yyvs3.upper)
			end
			if yyvs4 /= Void then
				yyvs4.fill_with (l_yyvs4_default_item, 0, yyvs4.upper)
			end
		end

	yy_push_last_value (yychar1: INTEGER)
			-- Push semantic value associated with token `last_token'
			-- (with internal id `yychar1') on top of corresponding
			-- value stack.
		do
			inspect yytypes2.item (yychar1)
			when 1 then
				yyvsp1 := yyvsp1 + 1
				if yyvsp1 >= yyvsc1 then
					if yyvs1 = Void then
						debug ("GEYACC")
							std.error.put_line ("Create yyvs1")
						end
						create yyspecial_routines1
						yyvsc1 := yyInitial_yyvs_size
						yyvs1 := yyspecial_routines1.make (yyvsc1)
					else
						debug ("GEYACC")
							std.error.put_line ("Resize yyvs1")
						end
						yyvsc1 := yyvsc1 + yyInitial_yyvs_size
						yyvs1 := yyspecial_routines1.resize (yyvs1, yyvsc1)
					end
				end
				yyspecial_routines1.force (yyvs1, last_any_value, yyvsp1)
			when 2 then
				yyvsp2 := yyvsp2 + 1
				if yyvsp2 >= yyvsc2 then
					if yyvs2 = Void then
						debug ("GEYACC")
							std.error.put_line ("Create yyvs2")
						end
						create yyspecial_routines2
						yyvsc2 := yyInitial_yyvs_size
						yyvs2 := yyspecial_routines2.make (yyvsc2)
					else
						debug ("GEYACC")
							std.error.put_line ("Resize yyvs2")
						end
						yyvsc2 := yyvsc2 + yyInitial_yyvs_size
						yyvs2 := yyspecial_routines2.resize (yyvs2, yyvsc2)
					end
				end
				yyspecial_routines2.force (yyvs2, last_string_value, yyvsp2)
			else
				debug ("GEYACC")
					std.error.put_string ("Error in parser: not a token type: ")
					std.error.put_integer (yytypes2.item (yychar1))
					std.error.put_new_line
				end
				abort
			end
		end

	yy_push_error_value
			-- Push semantic value associated with token 'error'
			-- on top of corresponding value stack.
		local
			yyval1: ANY
		do
			yyvsp1 := yyvsp1 + 1
			if yyvsp1 >= yyvsc1 then
				if yyvs1 = Void then
					debug ("GEYACC")
						std.error.put_line ("Create yyvs1")
					end
					create yyspecial_routines1
					yyvsc1 := yyInitial_yyvs_size
					yyvs1 := yyspecial_routines1.make (yyvsc1)
				else
					debug ("GEYACC")
						std.error.put_line ("Resize yyvs1")
					end
					yyvsc1 := yyvsc1 + yyInitial_yyvs_size
					yyvs1 := yyspecial_routines1.resize (yyvs1, yyvsc1)
				end
			end
			yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
		end

	yy_pop_last_value (yystate: INTEGER)
			-- Pop semantic value from stack when in state `yystate'.
		local
			yy_type_id: INTEGER
		do
			yy_type_id := yytypes1.item (yystate)
			inspect yy_type_id
			when 1 then
				yyvsp1 := yyvsp1 - 1
			when 2 then
				yyvsp2 := yyvsp2 - 1
			when 3 then
				yyvsp3 := yyvsp3 - 1
			when 4 then
				yyvsp4 := yyvsp4 - 1
			else
				debug ("GEYACC")
					std.error.put_string ("Error in parser: unknown type id: ")
					std.error.put_integer (yy_type_id)
					std.error.put_new_line
				end
				abort
			end
		end

	yy_run_geyacc
			-- You must run geyacc to regenerate this class.
		do
		end

feature {NONE} -- Semantic actions

	yy_do_action (yy_act: INTEGER)
			-- Execute semantic action.
		local
			yyval1: ANY
			yyval3: ROSE_CONFIGURATION_NODE
			yyval2: STRING
			yyval4: BOOLEAN
		do
			inspect yy_act
when 1 then
--|#line 120 "rose_configuration_cfg_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'rose_configuration_cfg_parser.y' at line 120")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 0
	yyvsp1 := yyvsp1 + 1
	if yyvsp1 >= yyvsc1 then
		if yyvs1 = Void then
			debug ("GEYACC")
				std.error.put_line ("Create yyvs1")
			end
			create yyspecial_routines1
			yyvsc1 := yyInitial_yyvs_size
			yyvs1 := yyspecial_routines1.make (yyvsc1)
		else
			debug ("GEYACC")
				std.error.put_line ("Resize yyvs1")
			end
			yyvsc1 := yyvsc1 + yyInitial_yyvs_size
			yyvs1 := yyspecial_routines1.resize (yyvs1, yyvsc1)
		end
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 2 then
--|#line 121 "rose_configuration_cfg_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'rose_configuration_cfg_parser.y' at line 121")
end

			add_comment (yyvs3.item (yyvsp3),yyvs2.item (yyvsp2))
		
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 4
	yyvsp1 := yyvsp1 -1
	yyvsp3 := yyvsp3 -1
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 3 then
--|#line 126 "rose_configuration_cfg_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'rose_configuration_cfg_parser.y' at line 126")
end

yyval3 := Void 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 0
	yyvsp3 := yyvsp3 + 1
	if yyvsp3 >= yyvsc3 then
		if yyvs3 = Void then
			debug ("GEYACC")
				std.error.put_line ("Create yyvs3")
			end
			create yyspecial_routines3
			yyvsc3 := yyInitial_yyvs_size
			yyvs3 := yyspecial_routines3.make (yyvsc3)
		else
			debug ("GEYACC")
				std.error.put_line ("Resize yyvs3")
			end
			yyvsc3 := yyvsc3 + yyInitial_yyvs_size
			yyvs3 := yyspecial_routines3.resize (yyvs3, yyvsc3)
		end
	end
	yyspecial_routines3.force (yyvs3, yyval3, yyvsp3)
end
when 4 then
--|#line 127 "rose_configuration_cfg_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'rose_configuration_cfg_parser.y' at line 127")
end

yyval3 := yyvs3.item (yyvsp3) 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines3.force (yyvs3, yyval3, yyvsp3)
end
when 5 then
--|#line 128 "rose_configuration_cfg_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'rose_configuration_cfg_parser.y' at line 128")
end

yyval3 := yyvs3.item (yyvsp3) 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines3.force (yyvs3, yyval3, yyvsp3)
end
when 6 then
--|#line 129 "rose_configuration_cfg_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'rose_configuration_cfg_parser.y' at line 129")
end

yyval3 := yyvs3.item (yyvsp3) 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines3.force (yyvs3, yyval3, yyvsp3)
end
when 7 then
--|#line 130 "rose_configuration_cfg_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'rose_configuration_cfg_parser.y' at line 130")
end

yyval3 := yyvs3.item (yyvsp3) 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines3.force (yyvs3, yyval3, yyvsp3)
end
when 8 then
--|#line 133 "rose_configuration_cfg_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'rose_configuration_cfg_parser.y' at line 133")
end

yyval2 := Void 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 0
	yyvsp2 := yyvsp2 + 1
	if yyvsp2 >= yyvsc2 then
		if yyvs2 = Void then
			debug ("GEYACC")
				std.error.put_line ("Create yyvs2")
			end
			create yyspecial_routines2
			yyvsc2 := yyInitial_yyvs_size
			yyvs2 := yyspecial_routines2.make (yyvsc2)
		else
			debug ("GEYACC")
				std.error.put_line ("Resize yyvs2")
			end
			yyvsc2 := yyvsc2 + yyInitial_yyvs_size
			yyvs2 := yyspecial_routines2.resize (yyvs2, yyvsc2)
		end
	end
	yyspecial_routines2.force (yyvs2, yyval2, yyvsp2)
end
when 9 then
--|#line 134 "rose_configuration_cfg_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'rose_configuration_cfg_parser.y' at line 134")
end

yyval2 := yyvs2.item (yyvsp2) 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines2.force (yyvs2, yyval2, yyvsp2)
end
when 10 then
--|#line 137 "rose_configuration_cfg_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'rose_configuration_cfg_parser.y' at line 137")
end

			yyval3 := new_include (yyvs4.item (yyvsp4),yyvs2.item (yyvsp2 - 2),yyvs2.item (yyvsp2 - 1),yyvs2.item (yyvsp2))
			Global.config_manager.import (current_section,yyvs2.item (yyvsp2 - 1),yyvs2.item (yyvsp2),True,yyvs4.item (yyvsp4))
		
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 4
	yyvsp3 := yyvsp3 + 1
	yyvsp4 := yyvsp4 -1
	yyvsp2 := yyvsp2 -3
	if yyvsp3 >= yyvsc3 then
		if yyvs3 = Void then
			debug ("GEYACC")
				std.error.put_line ("Create yyvs3")
			end
			create yyspecial_routines3
			yyvsc3 := yyInitial_yyvs_size
			yyvs3 := yyspecial_routines3.make (yyvsc3)
		else
			debug ("GEYACC")
				std.error.put_line ("Resize yyvs3")
			end
			yyvsc3 := yyvsc3 + yyInitial_yyvs_size
			yyvs3 := yyspecial_routines3.resize (yyvs3, yyvsc3)
		end
	end
	yyspecial_routines3.force (yyvs3, yyval3, yyvsp3)
end
when 11 then
--|#line 143 "rose_configuration_cfg_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'rose_configuration_cfg_parser.y' at line 143")
end

yyval2 := Global.convention.default_config_type 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 0
	yyvsp2 := yyvsp2 + 1
	if yyvsp2 >= yyvsc2 then
		if yyvs2 = Void then
			debug ("GEYACC")
				std.error.put_line ("Create yyvs2")
			end
			create yyspecial_routines2
			yyvsc2 := yyInitial_yyvs_size
			yyvs2 := yyspecial_routines2.make (yyvsc2)
		else
			debug ("GEYACC")
				std.error.put_line ("Resize yyvs2")
			end
			yyvsc2 := yyvsc2 + yyInitial_yyvs_size
			yyvs2 := yyspecial_routines2.resize (yyvs2, yyvsc2)
		end
	end
	yyspecial_routines2.force (yyvs2, yyval2, yyvsp2)
end
when 12 then
--|#line 144 "rose_configuration_cfg_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'rose_configuration_cfg_parser.y' at line 144")
end

yyval2 := yyvs2.item (yyvsp2) 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines2.force (yyvs2, yyval2, yyvsp2)
end
when 13 then
--|#line 146 "rose_configuration_cfg_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'rose_configuration_cfg_parser.y' at line 146")
end

yyval4 := False 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 0
	yyvsp4 := yyvsp4 + 1
	if yyvsp4 >= yyvsc4 then
		if yyvs4 = Void then
			debug ("GEYACC")
				std.error.put_line ("Create yyvs4")
			end
			create yyspecial_routines4
			yyvsc4 := yyInitial_yyvs_size
			yyvs4 := yyspecial_routines4.make (yyvsc4)
		else
			debug ("GEYACC")
				std.error.put_line ("Resize yyvs4")
			end
			yyvsc4 := yyvsc4 + yyInitial_yyvs_size
			yyvs4 := yyspecial_routines4.resize (yyvs4, yyvsc4)
		end
	end
	yyspecial_routines4.force (yyvs4, yyval4, yyvsp4)
end
when 14 then
--|#line 147 "rose_configuration_cfg_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'rose_configuration_cfg_parser.y' at line 147")
end

yyval4 := True
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp4 := yyvsp4 + 1
	yyvsp2 := yyvsp2 -1
	if yyvsp4 >= yyvsc4 then
		if yyvs4 = Void then
			debug ("GEYACC")
				std.error.put_line ("Create yyvs4")
			end
			create yyspecial_routines4
			yyvsc4 := yyInitial_yyvs_size
			yyvs4 := yyspecial_routines4.make (yyvsc4)
		else
			debug ("GEYACC")
				std.error.put_line ("Resize yyvs4")
			end
			yyvsc4 := yyvsc4 + yyInitial_yyvs_size
			yyvs4 := yyspecial_routines4.resize (yyvs4, yyvsc4)
		end
	end
	yyspecial_routines4.force (yyvs4, yyval4, yyvsp4)
end
when 15 then
--|#line 150 "rose_configuration_cfg_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'rose_configuration_cfg_parser.y' at line 150")
end

			yyval3 := new_value (current_section,yyvs2.item (yyvsp2 - 1),yyvs2.item (yyvsp2))
			current_section := yyval3
		
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 4
	yyvsp3 := yyvsp3 + 1
	yyvsp1 := yyvsp1 -2
	yyvsp2 := yyvsp2 -2
	if yyvsp3 >= yyvsc3 then
		if yyvs3 = Void then
			debug ("GEYACC")
				std.error.put_line ("Create yyvs3")
			end
			create yyspecial_routines3
			yyvsc3 := yyInitial_yyvs_size
			yyvs3 := yyspecial_routines3.make (yyvsc3)
		else
			debug ("GEYACC")
				std.error.put_line ("Resize yyvs3")
			end
			yyvsc3 := yyvsc3 + yyInitial_yyvs_size
			yyvs3 := yyspecial_routines3.resize (yyvs3, yyvsc3)
		end
	end
	yyspecial_routines3.force (yyvs3, yyval3, yyvsp3)
end
when 16 then
--|#line 155 "rose_configuration_cfg_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'rose_configuration_cfg_parser.y' at line 155")
end

			yyval3 := new_value (current_section,yyvs2.item (yyvsp2 - 1),yyvs2.item (yyvsp2))
		
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp3 := yyvsp3 + 1
	yyvsp2 := yyvsp2 -2
	if yyvsp3 >= yyvsc3 then
		if yyvs3 = Void then
			debug ("GEYACC")
				std.error.put_line ("Create yyvs3")
			end
			create yyspecial_routines3
			yyvsc3 := yyInitial_yyvs_size
			yyvs3 := yyspecial_routines3.make (yyvsc3)
		else
			debug ("GEYACC")
				std.error.put_line ("Resize yyvs3")
			end
			yyvsc3 := yyvsc3 + yyInitial_yyvs_size
			yyvs3 := yyspecial_routines3.resize (yyvs3, yyvsc3)
		end
	end
	yyspecial_routines3.force (yyvs3, yyval3, yyvsp3)
end
when 17 then
--|#line 159 "rose_configuration_cfg_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'rose_configuration_cfg_parser.y' at line 159")
end

yyval3 := new_value (current_section,Void,yyvs2.item (yyvsp2)) 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp3 := yyvsp3 + 1
	yyvsp1 := yyvsp1 -1
	yyvsp2 := yyvsp2 -1
	if yyvsp3 >= yyvsc3 then
		if yyvs3 = Void then
			debug ("GEYACC")
				std.error.put_line ("Create yyvs3")
			end
			create yyspecial_routines3
			yyvsc3 := yyInitial_yyvs_size
			yyvs3 := yyspecial_routines3.make (yyvsc3)
		else
			debug ("GEYACC")
				std.error.put_line ("Resize yyvs3")
			end
			yyvsc3 := yyvsc3 + yyInitial_yyvs_size
			yyvs3 := yyspecial_routines3.resize (yyvs3, yyvsc3)
		end
	end
	yyspecial_routines3.force (yyvs3, yyval3, yyvsp3)
end
when 18 then
--|#line 161 "rose_configuration_cfg_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'rose_configuration_cfg_parser.y' at line 161")
end

yyval2 := Void 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 0
	yyvsp2 := yyvsp2 + 1
	if yyvsp2 >= yyvsc2 then
		if yyvs2 = Void then
			debug ("GEYACC")
				std.error.put_line ("Create yyvs2")
			end
			create yyspecial_routines2
			yyvsc2 := yyInitial_yyvs_size
			yyvs2 := yyspecial_routines2.make (yyvsc2)
		else
			debug ("GEYACC")
				std.error.put_line ("Resize yyvs2")
			end
			yyvsc2 := yyvsc2 + yyInitial_yyvs_size
			yyvs2 := yyspecial_routines2.resize (yyvs2, yyvsc2)
		end
	end
	yyspecial_routines2.force (yyvs2, yyval2, yyvsp2)
end
when 19 then
--|#line 162 "rose_configuration_cfg_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'rose_configuration_cfg_parser.y' at line 162")
end

yyval2 := yyvs2.item (yyvsp2) 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp1 := yyvsp1 -1
	yyspecial_routines2.force (yyvs2, yyval2, yyvsp2)
end
when 20 then
--|#line 165 "rose_configuration_cfg_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'rose_configuration_cfg_parser.y' at line 165")
end

yyval2 := yyvs2.item (yyvsp2) 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines2.force (yyvs2, yyval2, yyvsp2)
end
when 21 then
--|#line 166 "rose_configuration_cfg_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'rose_configuration_cfg_parser.y' at line 166")
end

yyval2 := yyvs2.item (yyvsp2) 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines2.force (yyvs2, yyval2, yyvsp2)
end
when 22 then
--|#line 169 "rose_configuration_cfg_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'rose_configuration_cfg_parser.y' at line 169")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -1
	if yyvsp1 >= yyvsc1 then
		if yyvs1 = Void then
			debug ("GEYACC")
				std.error.put_line ("Create yyvs1")
			end
			create yyspecial_routines1
			yyvsc1 := yyInitial_yyvs_size
			yyvs1 := yyspecial_routines1.make (yyvsc1)
		else
			debug ("GEYACC")
				std.error.put_line ("Resize yyvs1")
			end
			yyvsc1 := yyvsc1 + yyInitial_yyvs_size
			yyvs1 := yyspecial_routines1.resize (yyvs1, yyvsc1)
		end
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 23 then
--|#line 170 "rose_configuration_cfg_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'rose_configuration_cfg_parser.y' at line 170")
end

accept 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -1
	if yyvsp1 >= yyvsc1 then
		if yyvs1 = Void then
			debug ("GEYACC")
				std.error.put_line ("Create yyvs1")
			end
			create yyspecial_routines1
			yyvsc1 := yyInitial_yyvs_size
			yyvs1 := yyspecial_routines1.make (yyvsc1)
		else
			debug ("GEYACC")
				std.error.put_line ("Resize yyvs1")
			end
			yyvsc1 := yyvsc1 + yyInitial_yyvs_size
			yyvs1 := yyspecial_routines1.resize (yyvs1, yyvsc1)
		end
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
			else
				debug ("GEYACC")
					std.error.put_string ("Error in parser: unknown rule id: ")
					std.error.put_integer (yy_act)
					std.error.put_new_line
				end
				abort
			end
		end

	yy_do_error_action (yy_act: INTEGER)
			-- Execute error action.
		do
			inspect yy_act
			when 29 then
					-- End-of-file expected action.
				report_eof_expected_error
			else
					-- Default action.
				report_error ("parse error")
			end
		end

feature {NONE} -- Table templates

	yytranslate_template: SPECIAL [INTEGER]
			-- Template for `yytranslate'
		once
			Result := yyfixed_array (<<
			    0,    2,    2,    2,    2,    2,    2,    2,    2,    2,
			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
			    2,   14,   13,    2,    2,    2,    2,    2,    2,    2,
			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
			    2,   11,    2,   12,    2,    2,    2,    2,    2,    2,

			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,

			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
			    2,    2,    2,    2,    2,    2,    1,    2,    3,    4,
			    5,    6,    7,    8,    9,   10, yyDummy>>)
		end

	yyr1_template: SPECIAL [INTEGER]
			-- Template for `yyr1'
		once
			Result := yyfixed_array (<<
			    0,   25,   25,   15,   15,   15,   15,   15,   20,   20,
			   16,   23,   23,   24,   24,   17,   18,   19,   22,   22,
			   21,   21,   26,   26, yyDummy>>)
		end

	yytypes1_template: SPECIAL [INTEGER]
			-- Template for `yytypes1'
		once
			Result := yyfixed_array (<<
			    1,    1,    1,    1,    2,    2,    3,    3,    3,    3,
			    3,    4,    2,    2,    2,    2,    1,    2,    2,    2,
			    2,    1,    2,    2,    2,    1,    2,    2,    2,    2,
			    1,    1, yyDummy>>)
		end

	yytypes2_template: SPECIAL [INTEGER]
			-- Template for `yytypes2'
		once
			Result := yyfixed_array (<<
			    1,    1,    1,    2,    2,    2,    2,    2,    2,    2,
			    2,    1,    1,    1,    1, yyDummy>>)
		end

	yydefact_template: SPECIAL [INTEGER]
			-- Template for `yydefact'
		once
			Result := yyfixed_array (<<
			    1,   13,    0,    0,   18,   14,    8,    4,    5,    6,
			    7,    0,   20,   21,   17,    0,    0,   16,    9,    0,
			   11,   18,   19,   23,   22,    2,   12,    0,   15,   10,
			    0,    0, yyDummy>>)
		end

	yydefgoto_template: SPECIAL [INTEGER]
			-- Template for `yydefgoto'
		once
			Result := yyfixed_array (<<
			    6,    7,    8,    9,   10,   19,   14,   17,   27,   11,
			    1,   25, yyDummy>>)
		end

	yypact_template: SPECIAL [INTEGER]
			-- Template for `yypact'
		once
			Result := yyfixed_array (<<
			 -32768,    0,    8,   14,   -9, -32768,   16, -32768, -32768, -32768,
			 -32768,   11, -32768, -32768, -32768,   -2,    8, -32768, -32768,    9,
			    1,   -9, -32768, -32768, -32768, -32768, -32768,    8, -32768, -32768,
			    2, -32768, yyDummy>>)
		end

	yypgoto_template: SPECIAL [INTEGER]
			-- Template for `yypgoto'
		once
			Result := yyfixed_array (<<
			 -32768, -32768, -32768, -32768, -32768, -32768,  -15,    3, -32768, -32768,
			 -32768, -32768, yyDummy>>)
		end

	yytable_template: SPECIAL [INTEGER]
			-- Template for `yytable'
		once
			Result := yyfixed_array (<<
			   30,   22,   31,   -3,    5,   16,    4,   26,   -3,   -3,
			   21,    3,   29,    2,   13,   12,   20,   24,   23,   18,
			   15,    0,    0,    0,   28, yyDummy>>)
		end

	yycheck_template: SPECIAL [INTEGER]
			-- Template for `yycheck'
		once
			Result := yyfixed_array (<<
			    0,   16,    0,    3,    4,   14,    6,    6,    8,    9,
			   12,   11,   27,   13,    6,    7,    5,    8,    9,    3,
			    6,   -1,   -1,   -1,   21, yyDummy>>)
		end

feature {NONE} -- Semantic value stacks

	yyvs1: SPECIAL [ANY]
			-- Stack for semantic values of type ANY

	yyvsc1: INTEGER
			-- Capacity of semantic value stack `yyvs1'

	yyvsp1: INTEGER
			-- Top of semantic value stack `yyvs1'

	yyspecial_routines1: KL_SPECIAL_ROUTINES [ANY]
			-- Routines that ought to be in SPECIAL [ANY]

	yyvs2: SPECIAL [STRING]
			-- Stack for semantic values of type STRING

	yyvsc2: INTEGER
			-- Capacity of semantic value stack `yyvs2'

	yyvsp2: INTEGER
			-- Top of semantic value stack `yyvs2'

	yyspecial_routines2: KL_SPECIAL_ROUTINES [STRING]
			-- Routines that ought to be in SPECIAL [STRING]

	yyvs3: SPECIAL [ROSE_CONFIGURATION_NODE]
			-- Stack for semantic values of type ROSE_CONFIGURATION_NODE

	yyvsc3: INTEGER
			-- Capacity of semantic value stack `yyvs3'

	yyvsp3: INTEGER
			-- Top of semantic value stack `yyvs3'

	yyspecial_routines3: KL_SPECIAL_ROUTINES [ROSE_CONFIGURATION_NODE]
			-- Routines that ought to be in SPECIAL [ROSE_CONFIGURATION_NODE]

	yyvs4: SPECIAL [BOOLEAN]
			-- Stack for semantic values of type BOOLEAN

	yyvsc4: INTEGER
			-- Capacity of semantic value stack `yyvs4'

	yyvsp4: INTEGER
			-- Top of semantic value stack `yyvs4'

	yyspecial_routines4: KL_SPECIAL_ROUTINES [BOOLEAN]
			-- Routines that ought to be in SPECIAL [BOOLEAN]

feature {NONE} -- Constants

	yyFinal: INTEGER = 31
			-- Termination state id

	yyFlag: INTEGER = -32768
			-- Most negative INTEGER

	yyNtbase: INTEGER = 15
			-- Number of tokens

	yyLast: INTEGER = 24
			-- Upper bound of `yytable' and `yycheck'

	yyMax_token: INTEGER = 265
			-- Maximum token id
			-- (upper bound of `yytranslate'.)

	yyNsyms: INTEGER = 27
			-- Number of symbols
			-- (terminal and nonterminal)

feature -- User-defined features



feature -- Error handling

	report_error (a_message: STRING)
			-- Print error message.
		do
			error.signal (id,"$1 malformed line $2, next to $3 : $4",<<buffer_name,line_number,last_string_value,a_message>>)
		end

feature {NONE} -- Implementation

	buffer_name: STRING
			-- Name of analyzed buffer

	current_section: ROSE_CONFIGURATION_NODE
			-- Current section in parsing

	root_section: ROSE_CONFIGURATION_NODE
			-- Root section

	new_value (a_node: ROSE_CONFIGURATION_NODE; a_key, a_value: STRING): ROSE_CONFIGURATION_NODE
			-- Set 'a_key' to 'a_value' in 'a_node'
		require
			valid_node: a_node /= Void
		do
			if a_key = Void then
				create Result.make_anonymous (a_node, a_value)
				Result.set_included (is_inclusion)
			else
				Result := a_node.forced_node (a_key, is_inclusion)
				if a_value /= Void then
					Result.load_value (a_value)
				end
			end
				--| This allows overriden included nodes to be marked as not included
			Result.set_included (Result.is_included and then is_inclusion)
		end

	add_comment (a_node: ROSE_CONFIGURATION_NODE; a_comment: STRING)
			-- Add 'a_comment' to 'a_node', or to root section if 'a_node' is not provided
		local
			l_node: ROSE_CONFIGURATION_NODE
		do
			if a_comment /= Void then
				if a_node /= Void then
					a_node.set_comment (a_comment)
						--| This allows overriden included nodes to be marked as not included
					a_node.set_included (a_node.is_included and is_inclusion)
				else
					create l_node.make_comment (current_section,a_comment)
					l_node.set_included (is_inclusion)
				end
			end
		end

	new_include (a_try: BOOLEAN; a_keyword, a_type, a_value: STRING): ROSE_CONFIGURATION_NODE
			-- Add 'a_value' to root section
		require
			valid_keyword: a_keyword /= Void
			valid_value: a_value /= Void
		do
			create Result.make_directive (current_section,keyword_text (a_try,a_keyword,a_type),a_value)
			Result.set_included (is_inclusion)
		end

	keyword_text (a_try: BOOLEAN; a_keyword, a_type: STRING): STRING
			-- 'a_keyword' eventually preceded by "try "
		require
			valid_keyword: a_keyword /= Void
		do
			create Result.make (a_keyword.count + a_type.count + 5)
			if a_try then
				Result.append ("try ")
			end
			Result.append (a_keyword)
			Result.append (" ")
			Result.append (a_type)
		end

end -- class ROSE_CONFIGURATION_HANDLER_CFG
