indexing
	description: "EV view on configuration"
	author: "Zoran Simic"
	date: "$Date: 2008/12/01 $"
	revision: "$Revision: #17 $"

class
	ROSE_CONFIGURATION_VIEW

inherit
	ROSE_SHARED_GLOBAL_GUI
		undefine
			default_create, is_equal, copy
		end
	EV_VERTICAL_BOX

create
	make

feature {NONE} -- Initialization

	make (a_container: EV_CONTAINER) is
			-- Create a view on configuration, and put it in 'a_container' if provided.
		local
			l_box, l_change_box: EV_BOX
			l_widget: EV_WIDGET
			l_ok, l_cancel: EV_BUTTON
			l_split: EV_SPLIT_AREA
		do
			default_create
			Gui.factory.set_box_defaults (a_container,Current,2,6)
				--| Toolbar
			toolbar := Gui.factory.new_toolbar (Current,<<
				[	["open.png",agent load_config (Void,Void),"Open configuration file"],
					[True,"cfg_app_config.ico",agent view_app_config,"Application configuration"],
					["cfg_gui_config.ico",agent view_gui_config,"GUI configuration"]],
				["reload.ico",agent reload_app_config,"Reload application configuration"],
				["save.png",agent save_config (Void),"Save current configuration"],
				Void,
				[False,cfg_directive,agent toggle_directive,"Show directives"],
				[False,cfg_leaf,agent toggle_leaves,"Show leaves"],
				["refresh.ico",agent refresh,"Refresh"]
			>>)
				--| Config view, horizontal split containing tree and list views
			l_split := Gui.factory.new_hsplit2 (Current, 0)
			tree := Gui.factory.new_tree (l_split, Void, -300, 0)
			grid := Gui.factory.new_grid (l_split, <<["Key",200],["Value",430]>>, agent on_grid_select, 0, 0)
				--| Information status line
			info_line := Gui.factory.new_label (Current,Void,1)
				--| "Change value" window
			change_value := Gui.factory.new_dialog ("Set config value",Gui.rep.pixmaps.Question_pixmap,500,200)
			l_change_box := Gui.factory.new_vbox (change_value,4,4)
			l_box := Gui.factory.new_hbox (l_change_box,0,4)
			cv_key := Gui.factory.new_text_field (l_box,"Key: ",Void,Void,0)
			l_widget := Gui.factory.new_label (l_change_box,"Value:",1)
			cv_value := Gui.factory.new_text (l_change_box,Void,0,0)
			l_box := Gui.factory.new_hbox (l_change_box,0,4)
			l_ok := Gui.factory.new_button (l_box,"OK",agent change_value.hide,80)
			l_cancel := Gui.factory.new_button (l_box,"Cancel",agent cv_cancel_handler,80)
			l_widget := Gui.factory.new_label (l_box,Void,80)
			l_widget := Gui.factory.new_button (l_box,"Expand",agent cv_expand_handler,80)
			l_widget := Gui.factory.new_label (l_box,Void,20)
			cv_delete := Gui.factory.new_button (l_box,"Delete",agent cv_delete_handler,80)
			change_value.set_default_push_button (l_ok)
			change_value.set_default_cancel_button (l_cancel)
			view_app_config
		end

feature -- Access

	current_config: ROSE_CONFIGURATION
			-- Currently displayed configuration

feature -- Basic operations

	view_app_config is
			-- View application's configuration.
		do
			if current_config /= config then
				Gui.factory.select_by_name (toolbar,"Application configuration",True)
				current_node := Void
				current_config := config
				refresh
			end
		end

	view_gui_config is
			-- View user GUI configuration.
		do
			if current_config /= Global.user.config then
				Gui.factory.select_by_name (toolbar,"User configuration",True)
				current_node := Void
				current_config := Global.user.config
				refresh
			end
		end

	set_config (a_config: like current_config) is
		do
			if current_config /= a_config then
				Gui.factory.select_by_name (toolbar,"Open configuration file",True)
				current_node := Void
				current_config := a_config
				refresh
			end
		end

	load_config (a_type, a_path: STRING) is
			-- Load configuration of 'a_type' from 'a_path'
			-- Provide Void 'a_type' for default config type (cfg)
			-- If 'a_path' is Void, ask user to select a file to show.
		local
			l_type, l_path: STRING
			l_config: ROSE_CONFIGURATION
		do
			if a_path = Void then
				if window /= Void then
					l_path := Gui.dialogs.select_open_file (window,"config","*.cfg")
				end
			else
				l_path := a_path
				l_type := a_type
			end
			if l_path /= Void then
				create l_config.make_default
				Global.config_manager.load (l_config,l_type,l_path)
			end
			if not error.signaled and then l_config /= Void then
				set_config (l_config)
			end
			error.flush_and_acknowledge
			if current_config = config then
				Gui.factory.select_by_name (toolbar,"Application configuration",True)
			elseif current_config = Global.user.config then
				Gui.factory.select_by_name (toolbar,"GUI configuration",True)
			end
		end

	resource_view: ROSE_RESOURCE_VIEW

	reload_app_config is
			-- Reload application configuration.
		local
			l_path: STRING
		do
			if resource_view = Void then
				create resource_view.make
			end
			resource_view.select_type ("config")
			resource_view.set_filename (Global.app.program_name + Global.convention.default_config_extension)
			resource_view.show (Gui.parent_window (Current))
			l_path := resource_view.selected_file_name
			if l_path /= Void then
				load_config (Void,l_path)
			end
		end

	save_config (a_path: STRING) is
			-- Save current configuration to 'a_path'
			-- Ask interactively user to select path if 'a_path' is Void
			-- Use 'current_config' path if 'a_path' is empty.
		local
			l_path: STRING
		do
			if a_path = Void then
				if window /= Void then
					l_path := Gui.dialogs.select_save_file (window,"config","*.cfg")
				end
			elseif a_path.is_empty then
				if current_config /= Void then
					l_path := current_config.path
				end
			else
				l_path := a_path
			end
			if l_path /= Void and then not l_path.is_empty then
				Global.config_manager.save (current_config,Void,l_path)
			end
			error.flush_and_acknowledge
			refresh
		end

	export_config is
			-- Export configuration to a user-selected file path.
		local
			l_path: STRING
		do
			if window /= Void then
				l_path := Gui.dialogs.select_save_file (window,"config","*.cfg")
			end
			if l_path /= Void and then not l_path.is_empty then
				Global.config_manager.flat_export (current_config,Void,l_path)
			end
			error.flush_and_acknowledge
			refresh
		end

	refresh is
			-- Refresh the view
		local
			l_tree_node: EV_TREE_NODE
			l_config_node: ROSE_CONFIGURATION_NODE
		do
			if window = Void or else window.is_destroyed then
				window := Gui.parent_window (Current)
			end
				--| Repopulate the tree-view
			tree.wipe_out
			if current_config /= Void then
				l_config_node := current_config.root
			end
			l_tree_node := get_tree_node (l_config_node)
			if l_tree_node = Void then
				create {EV_TREE_ITEM} l_tree_node.default_create
				l_tree_node.set_pixmap (cfg_grey_folder)
				l_tree_node.set_text ("No configuration")
			end
			tree.extend (l_tree_node)
				--| Reselect last 'current_node', if any
			if current_node /= Void then
				reposition_tree (current_node)
			elseif l_tree_node.is_expandable then
				l_tree_node.expand
			end
			info_on_config
		end

	auto_refresh is
			-- Automatically refresh this view.
		do
		end

feature {NONE} -- Implementation

	window: EV_WINDOW
			-- Window showing this viewer
			-- Automatically determined

	toolbar: EV_TOOL_BAR
			-- Window's toolbar

	tree: EV_TREE
			-- Tree view displaying config structure

	grid: RP_DATA_GRID[ANY]
			-- Grid-view on selected config node

	info_line: EV_LABEL
			-- Info status line

	current_node: ROSE_CONFIGURATION_NODE
			-- Node currently selected
			-- Used to reposition tree-view on last selected node when refreshed

	reposition_tree (a_config_node: ROSE_CONFIGURATION_NODE) is
			-- Reposition tree-view on item corresponding to 'a_config_node'.
		do
			tree.do_all (agent do_reposition_tree (a_config_node,?))
			grid.do_all_rows (agent do_reposition_grid (a_config_node,?))
		end

	do_reposition_tree (a_config_node: ROSE_CONFIGURATION_NODE; a_node: EV_TREE_NODE) is
			-- Select item corresponding to 'a_node' in tree-view
			-- Called on 'refresh'
		local
			l_node: ROSE_CONFIGURATION_NODE
		do
			if a_node /= Void then
				l_node ?= a_node.data
			end
			if is_config_parent (l_node,a_config_node) then
				if a_node.is_expandable then
					a_node.expand
				end
				if a_node.is_selectable then
					a_node.enable_select
				end
				if l_node /= a_config_node then
					a_node.do_all (agent do_reposition_tree (a_config_node,?))
				end
			end
		end

	do_reposition_grid (a_config_node: ROSE_CONFIGURATION_NODE; a_node: RP_GRID_NODE[ANY]) is
			-- Select item corresponding to 'a_node' in tree-view
			-- Called on 'refresh'
		local
			l_node: ROSE_CONFIGURATION_NODE
		do
			if a_node /= Void then
				l_node ?= a_node.data
			end
			if l_node /= Void and then a_config_node = l_node then
				a_node.enable_select
			end
		end

	is_config_parent (a_parent, a_node: ROSE_CONFIGURATION_NODE): BOOLEAN is
			-- Is 'a_parent' a parent of 'a_node' in config?
		local
			l_node: ROSE_CONFIGURATION_NODE
		do
			if a_parent /= Void and then a_node /= Void then
				from
					l_node := a_node
					Result := l_node = a_parent
				until
					Result or else l_node = Void
				loop
					l_node := l_node.parent
					Result := l_node = a_parent
				end
			end
		end

	get_tree_node (a_node: ROSE_CONFIGURATION_NODE): EV_TREE_NODE is
			-- Tree-view item representing 'a_node'
		do
			if a_node = Void then
				--| Do nothing
			elseif a_node.is_directive_node then
				if is_show_directives then
					create {EV_TREE_ITEM} Result.default_create
					Result.select_actions.extend (agent clear_right_pane)
				end
			elseif a_node.is_value_node then
				if a_node.is_anonymous then
					if is_show_leaves then
						create {EV_TREE_ITEM} Result.default_create
						Result.select_actions.extend (agent show_all_values (a_node))
					end
				elseif a_node.children = Void then
					if is_show_leaves then
						create {EV_TREE_ITEM} Result.default_create
						Result.select_actions.extend (agent show_all_values (a_node))
						Result.pointer_double_press_actions.extend (agent on_tree_double_click)
					end
				else
					if (a_node.has_leaves_only implies is_show_leaves)
					or else (a_node.has_directives and then is_show_directives) then
						create {EV_DYNAMIC_TREE_ITEM} Result.make_with_function (agent expand_node (a_node))
						Result.expand_actions.extend (agent Result.set_pixmap (cfg_node_open))
						Result.collapse_actions.extend (agent Result.set_pixmap (cfg_node))
						Result.select_actions.extend (agent show_all_values (a_node))
					else
						create {EV_TREE_ITEM} Result.default_create
						Result.select_actions.extend (agent show_all_values (a_node))
					end
				end
			elseif a_node.is_commented then
				--| Do nothing
			elseif a_node.children = Void then
					--| Empty root
				create {EV_TREE_ITEM} Result.default_create
				Result.select_actions.extend (agent show_all_values (a_node))
			else
					--| Non-empty root
				create {EV_DYNAMIC_TREE_ITEM} Result.make_with_function (agent expand_node (a_node))
				Result.select_actions.extend (agent show_all_values (a_node))
			end
			if Result /= Void then
				Result.set_text (node_id (a_node))
				Result.set_pixmap (node_icon (a_node))
				Result.set_data (a_node)
			end
		end

	expand_node (a_node: ROSE_CONFIGURATION_NODE): LINKED_LIST [EV_TREE_NODE] is
			-- Called when a tree-view item is expanded
		local
			l_item: EV_TREE_NODE
			i: INTEGER
		do
			create Result.make
			if a_node.children /= Void then
				from
					i := a_node.children.lower
				until
					i > a_node.children.upper
				loop
					l_item := get_tree_node (a_node.children.item (i))
					if l_item /= Void then
						Result.extend (l_item)
					end
					i := i + 1
				end
			end
		end

	is_show_leaves, is_show_directives: BOOLEAN

	toggle_directive is
		do
			is_show_directives := not is_show_directives
			refresh
		end

	toggle_leaves is
		do
			is_show_leaves := not is_show_leaves
			refresh
		end

	change_value: EV_DIALOG
	cv_key: EV_TEXT_FIELD
	cv_value: EV_TEXT
	cv_delete: EV_BUTTON
			-- Change-value dialog and widgets

	modify (a_node: ROSE_CONFIGURATION_NODE; a_key_modifiable: BOOLEAN) is
			-- Modify config value associated to 'a_key'
			-- 'a_key' Void allows to insert new values
			-- 'a_value' is original value
		local
			l_key: STRING
		do
			if window /= Void and then a_node /= Void then
				cv_key.set_text (a_node.path)
				if a_key_modifiable then
					if cv_key.is_displayed then
						cv_key.set_focus
					end
					cv_key.set_caret_position (cv_key.text.count+1)
					cv_delete.disable_sensitive
				else
					cv_key.disable_sensitive
					if a_node.parent = Void then
						cv_delete.disable_sensitive
					else
						cv_delete.enable_sensitive
					end
				end
				if a_node.value /= Void then
					cv_value.set_text (a_node.value)
				else
					cv_value.remove_text
				end
				change_value.show_modal_to_window (window)
				cv_key.enable_sensitive
				l_key := cv_key.text
				if l_key /= Void and then not l_key.is_empty then
					if a_key_modifiable then
						config.set (l_key,cv_value.text)
						current_node := config.node (l_key)
					else
						a_node.set_value (cv_value.text)
						current_node := a_node
					end
					refresh
				end
			end
		end

	cv_cancel_handler is
			-- Cancel button pressed on change dialog
		do
			cv_key.remove_text
			change_value.hide
		end

	cv_expand_handler is
			-- Expand button pressed on change dialog
		do
			cv_value.set_text (Global.interpreted (cv_value.text))
		end

	cv_delete_handler is
			-- Delete button pressed on change dialog
		do
			delete_key (cv_key.text)
			cv_cancel_handler
		end

	delete_key (a_key: STRING) is
			-- Delete 'a_key' from configuration
			-- If SHIFT is hold down, no confirmation asked.
		do
			if current_config /= Void and then a_key /= Void and then not a_key.is_empty then
				if not Gui.ev_application.shift_pressed
				implies (window /= Void and then Gui.dialogs.answer (window,"config_confirm_delete","Really delete?","Delete key '$1' from this configuration",<<cv_key.text>>)) then
					current_config.delete (a_key)
					current_node := Void
					refresh
				end
			end
		end

feature {NONE} -- List-view building

	show_all_values (a_node: ROSE_CONFIGURATION_NODE) is
			-- Show value of 'a_node' in right pane.
			-- Called when config node selected in tree-view.
		do
			clear_right_pane
			if a_node = Void then
				--| Do nothing
			elseif a_node.is_directive_node then
				grid.extend (gui.factory.new_grid_node (cfg_directive,<<a_node.key,a_node.value>>,a_node))
			else
				if a_node.value /= Void and then not a_node.value.is_empty then
					if a_node.is_leave then
						grid.extend (gui.factory.new_grid_node (cfg_leaf,<<"[value]",a_node.value>>,a_node))
					else
						grid.extend (gui.factory.new_grid_node (cfg_node,<<"[value]",a_node.value>>,a_node))
					end
				end
				a_node.do_all (agent add_node_row)
			end
			current_node := a_node
		end

	add_node_row (a_node: ROSE_CONFIGURATION_NODE) is
		do
			grid.extend (gui.factory.new_grid_node (node_icon (a_node),<<node_id (a_node),a_node.value>>,a_node))
		end

	clear_right_pane is
			-- Clear right pane view.
		do
			grid.clear
		end

feature {NONE} -- Agents

	on_grid_select (a_button: INTEGER; a_node: RP_GRID_NODE[ANY]) is
			-- Called when item in grid-view double-clicked, selected or deselected.
		local
			l_node: ROSE_CONFIGURATION_NODE
			l_item: EV_TREE_NODE
		do
			if a_node /= Void then
				if a_button = -1 then
					--| Selected
					l_node ?= a_node.data
					current_node := l_node
				elseif a_button = 0 then
					--| Deselected
				elseif a_button = 1 then
					--| Double-clicked
					l_node ?= a_node.data
					if l_node = Void then
						--| Do nothing
					elseif l_node.is_leave or else is_value_representation (a_node) then
						modify (l_node,False)
					elseif l_node.children /= Void then
						current_node := l_node
						refresh
					end
				end
			elseif a_button = 1 then
				l_item := tree.selected_item
				if l_item /= Void then
					l_node ?= l_item.data
					if l_node /= Void then
						modify (l_node,True)
					end
				end
			end
		end

	is_value_representation (a_node: RP_GRID_NODE[ANY]): BOOLEAN is
		local
			l_string: STRING
			l_label: RP_DATA_GRID_LABEL_ITEM
		do
			if a_node /= Void and then a_node.column_count > 0 then
				l_label ?= a_node.i_th (1)
				l_string := l_label.text
			end
			Result := l_string /= Void
				and then l_string.count > 0
				and then l_string.item (1) = '['
		end

	on_tree_double_click (a_x, a_y, a_button: INTEGER; a_x_tilt, a_y_tilt, a_pressure: DOUBLE; a_screen_x, a_screen_y: INTEGER) is
		local
			l_tree_node: EV_TREE_NODE
			l_node: ROSE_CONFIGURATION_NODE
		do
			if a_button = 1 then
				l_tree_node := tree.selected_item
				if l_tree_node /= Void then
					l_node ?= l_tree_node.data
				end
				if l_node /= Void then
					modify (l_node,False)
				end
			end
		end

	node_icon (a_node: ROSE_CONFIGURATION_NODE): EV_PIXMAP is
			-- Icon to associate to 'a_node'
		do
			if a_node = Void then
				Result := cfg_grey_folder
			elseif a_node.is_directive_node then
				Result := cfg_directive
			elseif a_node.is_value_node then
				if a_node.is_anonymous then
					Result := cfg_anonymous
				elseif a_node.is_leave then
					Result := cfg_leaf
				else
					Result := cfg_node
				end
			elseif a_node.is_commented then
				Result := cfg_grey_folder
			else
				Result := cfg_root
			end
		end

	node_id (a_node: ROSE_CONFIGURATION_NODE): STRING is
			-- Id to show for a 'a_node'
		do
			if a_node = Void then
				Result := "-Void-"
			elseif a_node.is_directive_node then
				Result := a_node.key + " " + a_node.value
			elseif a_node.is_value_node then
				if a_node.is_anonymous then
					Result := ""
				elseif a_node.key /= Void then
					Result := a_node.key
				else
					Result := "?"
				end
			elseif a_node.is_commented then
				Result := a_node.comment
			elseif a_node.children = Void then
				Result := "Empty configuration"
			else
				Result := "Configuration"
			end
			if a_node = Void then
				--| Do nothing
			elseif a_node.is_modified then
				Result := Result + " *"
			elseif a_node.is_included then
				Result := "(" + Result + ")"
			end
		end

	cfg_root: EV_PIXMAP is once Result := Gui.factory.new_pixmap ("cfg_root.ico") end
	cfg_directive: EV_PIXMAP is once Result := Gui.factory.new_pixmap ("cfg_directive.ico") end
	cfg_node: EV_PIXMAP is once Result := Gui.factory.new_pixmap ("cfg_node.ico") end
	cfg_node_open: EV_PIXMAP is once Result := Gui.factory.new_pixmap ("cfg_node_open.ico") end
	cfg_leaf: EV_PIXMAP is once Result := Gui.factory.new_pixmap ("cfg_leaf.ico") end
	cfg_grey_folder: EV_PIXMAP is once Result := Gui.factory.new_pixmap ("cfg_grey_folder.ico") end
	cfg_anonymous: EV_PIXMAP is once Result := Gui.factory.new_pixmap ("cfg_anonymous.ico") end
	cfg_env_vars: EV_PIXMAP is once Result := Gui.factory.new_pixmap ("cfg_env_vars.ico") end
	cfg_app_vars: EV_PIXMAP is once Result := Gui.factory.new_pixmap ("cfg_app_vars.ico") end

	info_on_config is
			-- Display short info on config in status bar.
		local
			l_string, l_path, l_type: STRING
			l_count: INTEGER
		do
			if current_config = Void then
				l_string := "No configuration selected"
			elseif current_config = config then
				l_string := "App '$1' configuration, path '$2'"
			elseif current_config = Global.user.config then
				l_string := "User configuration, path '$2'"
			else
				l_string := "Config file '$2'"
			end
			if current_config /= Void then
				l_path := current_config.path
				l_type := current_config.type
				l_count := current_config.count
			end
			if l_path = Void or else l_path.is_empty then
				l_path := "-no path-"
			end
			l_string := Global.formatted (l_string,<<Global.app.name,l_path,l_type,l_count>>)
			info_line.set_text (Global.interpreted (l_string))
		end

end
