note

	description: "Definition of some useful constants"

	author: "Mark Howard"
	last_editor: "$Author: c4 $"
	date: "$Date: 2014/02/14 $"
	revision: "$Revision: #20 $"
	path: "$File: //rose/source/kernel/infrastructure/math/rose_constant_values.e $"

class	ROSE_CONSTANT_VALUES

inherit

	IEEE_MATH
	
	ROSE_LEGACY_CONSTANTS

feature -- Access

	missing_double: DOUBLE
			-- IEEE Not-a-number
		once
			Result := Qnan
		end
	
	missing_real: REAL
			-- IEEE Not-a-number
		once
			Result := Qnan_real_32
		end

	missing_integer: INTEGER is -999
	minus_infinity: INTEGER is -2147483647		-- Not the same as INT_MIN (-2147483648)!
	plus_infinity: INTEGER is 2147483647
	null_integer: INTEGER is -2147483646

	missing_string: STRING is once Result := "" end

	missing_entity_id: INTEGER is 0
	minimum_date_time: ROSE_DATE is once create Result.make_min_date end
	maximum_date_time: ROSE_DATE is once create Result.make_max_date end

	is_null_integer (a_value: INTEGER): BOOLEAN is
			-- Is 'a_value' equal to 'Null_integer'?
		do
			Result := a_value = Null_integer
		end

	ieee, ieee_math: IEEE_MATH
			-- Please try to avoid using this feature.  It will just slow you down.
			-- Instead, inherit from IEEE_MATH and use the feature directly.
		once
			create Result
		end

	now: ROSE_DATE is
		once
			create Result.make_now
		end

	today_rdt: ROSE_DATE is
		once
			create Result.make_today
		end

	maximum_integer: INTEGER is 2147483647

	maximum_process_id: INTEGER is once Result := Maximum_integer end

	undefined_integer_16: INTEGER_16 is
		local
			l_integer: INTEGER
		once
			l_integer := missing_integer  --was -999
			Result := l_integer.to_integer_16
		end

	integer_16_zero: INTEGER_16 is once end

	internal: INTERNAL is once create Result end

feature -- Type IDs

	Pointer_type: INTEGER is 0
	Reference_type: INTEGER is 1
	Character_type: INTEGER is 2
	Boolean_type: INTEGER is 3
	Integer_type: INTEGER is 4
	Integer_32_type: INTEGER is 4
	Real_type: INTEGER is 5
	Double_type: INTEGER is 6
	Expanded_type: INTEGER is 7
	Bit_type: INTEGER is 8
	Integer_8_type: INTEGER is 9
	Integer_16_type: INTEGER is 10
	Integer_64_type: INTEGER is 11
	Wide_character_type: INTEGER is 12
	Decimal_type: INTEGER is 13
			-- Constants defined in INTERNAL

	String_type: INTEGER is -1
	Date_type: INTEGER is -2
	Date_time_type: INTEGER is -3
			-- Additional types handled by us, but not defined in INTERNAL

	type_name (a_type: INTEGER): STRING is
			-- Type name of 'a_type' id
		do
			inspect a_type
			when Pointer_type then
				Result := "POINTER"
			when Reference_type then
				Result := "REFERENCE"
			when Character_type then
				Result := "CHARACTER"
			when Boolean_type then
				Result := "BOOLEAN"
			when Integer_type then
				Result := "INTEGER"
			when Real_type then
				Result := "REAL"
			when Double_type then
				Result := "DOUBLE"
			when Expanded_type then
				Result := "EXPANDED"
			when Bit_type then
				Result := "BIT"
			when Integer_8_type then
				Result := "INTEGER"
			when Integer_16_type then
				Result := "INTEGER"
			when Integer_64_type then
				Result := "INTEGER"
			when Wide_character_type then
				Result := "CHARACTER_32"
			when Decimal_type then
				Result := "DECIMAL"
			when String_type then
				Result := "STRING"
			when Date_type then
				Result := "ROSE_DATE"
			when Date_time_type then
				Result := "ROSE_DATE"
			else
				Result := internal.type_name_of_type (a_type)
			end
		end

feature -- Access

	min_type: INTEGER is
		-- Minimum type value
		do
			Result := Date_time_type
		ensure
			Date_time_type: Result = Date_time_type
		end

	max_type: INTEGER is
			-- Minimum type value
		do
			Result := Decimal_type
		ensure
			Decimal_type: Result = Decimal_type
		end

	is_valid_type (a_type: INTEGER): BOOLEAN is
			-- Is 'a_type' a valid type constants?
		do
			Result := (min_type <= a_type) and (a_type <= max_type)
		ensure
			from_continuous_enumeration: Result = ( (min_type <= a_type) and (a_type <= max_type) )
		end

end
