note

	description:

		"Official context for AxaRosenberg business objects"

	author: "Mark Howard"
	last_editor: "$Author: ext-asurelia $"
	date: "$Date: 2015/04/15 $"
	revision: "$Revision: #147 $"
	path: "$File: //rose/source/kernel/entities/rose/rose_context.e $"

deferred class ROSE_CONTEXT

inherit

	ROSE_CONSTANTS

	ROSE_SERVERS

	ROSE_SUPPLIER_CONTEXT
		rename
			make as supplier_context_make
		undefine
			company_template,
			security_template,
			market_listing_template,
			listing_template,
			currency_template,
			country_template,
			exchange_template,
			industry_template,
			similars_group_template,
			economic_sector_template,
			economic_sector_group_template,
			investment_region_template,
			entity_identifier_template,
			government_issuer_template,
			government_issue_template,
			government_issue_listing_template,
			bond_template,
			bond_listing_template,
			fund_template,
			fund_issue_template,
			fund_listing_template,
			future_details_template,
			future_template,
			future_listing_template,
			market_index_template,
			market_index_listing_template,
			sector_template,
			portfolio_template,
			entity_template,
			risk_model_template,
			risk_universe_template,
			common_attributes,
			load_all_entities
		redefine
			refresh,
			clear_cache,
			listing_accessor,
			load_special_caches
		end

	INVESTMENT_TEMPLATES

	ATTRIBUTES
		rename
			name as attributes_name,
			brim_code as attributes_brim_code,
			local_brim_code as attributes_local_brim_code,
			riemsym_code as attributes_riemsym_code,
			j_brim_code as attributes_j_brim_code,
			s_brim_code as attributes_s_brim_code,
			e_brim_code as attributes_e_brim_code
		end

feature {NONE} -- Creation

	make is
			-- Minimal creation for all rose contexts
		do
			set_parameters
			supplier_context_make (server, name)
			create zones.make_filled (Void, 256)
		ensure
			server_set: server /= Void
		end

feature -- Basic operations

	set_parameters is
			--set the scope
		do
			has_rose_fg_handles := True
			has_rose_bg_handles := False
			server := servers.rose_entity_foreground_server
			name := "ROSE"
				-- "ENTITY FOREGROUND"
		end

	refresh is
			-- Refresh completely: database, predicate and collections
		do
			set_new_server
			reset_predicates
			Precursor
		end

	set_new_server is
			-- Place holder for rose entity foreground
		do
		end

	enable_market_data_persistent_servers is
			-- Turn on all market data persistent servers
		do
			all_zones.do_all (agent {like zone_template}.set_use_persistent_server_as_market_data_source (True))
		end

	clear_cache is
			-- Wipe out all containers in the persistent entity server
		local
			i: INTEGER
		do
			precursor {ROSE_SUPPLIER_CONTEXT}
			from
				i := 0
			until
				i >= zones.count
			loop
					-- Exclude obsolete Zones
				if zones.item (i) /= Void and then not (zones.item (i).mnemonic.is_equal ('I') or zones.item (i).mnemonic.is_equal ('F')) then
					zones.item (i).clear_cache
				end
				i := i + 1
			end
		end

	load_special_caches is
			-- Load the composite listings hash map
		do
			composite_listings.do_nothing
		end

feature -- Access

	zone (a_mnemonic: CHARACTER): like zone_template is
			-- Investment zone.
		local
			l_region: like investment_region_template
		do
			if (a_mnemonic /= '%U') then
				Result := zones.item (a_mnemonic.as_upper.code)
				if Result = Void then
					if a_mnemonic = 'F' then
						Result := new_future_zone
					elseif a_mnemonic = 'I' then
						Result := new_index_zone
					elseif a_mnemonic = 'W' then
						Result := new_world_zone
					else
						l_region ?= investment_region_at (mnemonic (a_mnemonic.as_upper.out), Void)
						if l_region /= Void then
							create Result.make (Current, l_region)
						end
					end
					zones.put (Result, a_mnemonic.as_upper.code)
				end
			end
		end

	new_future_zone: like zone_template is
			--return a new f_zone
		do
			create Result.make_future (Current)
		end

	new_index_zone: like zone_template is
			--return a new i_zone
		do
			create Result.make_index (Current)
		end

	new_world_zone: like zone_template
	-- Returns a new W zone.
	do
		create Result.make_world(Current)
	end

	new_zone_from_mnemonic (a_mnemonic: CHARACTER): like zone_template is
			-- create a zone for `a_mnemonic'
		local
			l_region: like investment_region_template
		do
			l_region ?= investment_region_at (mnemonic (a_mnemonic.as_upper.out), Void)
			if l_region /= Void then
				create Result.make (Current, l_region)
			end
		end

	zone_by_index (a_ix: INTEGER): detachable like zone_template is
			-- zone of legacy index `a_ix' or Void
		do
			inspect a_ix
			when 0 then
				Result := t_zone
			when 1 then
				Result := u_zone
			when 2 then
				Result := j_zone
			when 3 then
				Result := c_zone
			when 4 then
				Result := e_zone
			when 5 then
				Result := o_zone
			when 6 then
				Result := s_zone
			else
			end
		end

	u_zone: like zone_template is
			-- return U_zone
		do
			Result := zone ('U')
		end

	e_zone: like zone_template is
			-- return E_zone
		do
			Result := zone ('E')
		end

	c_zone: like zone_template is
			-- return C_zone
		do
			Result := zone ('C')
		end

	j_zone: like zone_template is
			-- return J_zone
		do
			Result := zone ('J')
		end

	o_zone: like zone_template is
			-- return O_zone
		do
			Result := zone ('O')
		end

	s_zone: like zone_template is
			-- return S_zone
		do
			Result := zone ('S')
		end

	t_zone: like zone_template is
			-- return T_zone
		do
			Result := zone ('T')
		end

	w_zone: like zone_template is
			-- return W_zone
		do
			Result := zone ('W')
		end

	f_zone: like zone_template is
			-- return F_zone
		do
			Result := zone ('F')
		end

	i_zone: like zone_template is
			-- return I_zone
		do
			Result := zone ('I')
		end

	load_all_entities is
			-- return u_zone
			--
			-- Cache all entities in non-F or I zones
		local
			l_gc: BOOLEAN
			i: INTEGER
			l_zones: ROSE_LINEAR_ARRAY [ZONE [LOCATION]]
		do
			l_gc := memory.collecting
			output (Global.interpreted ("($now) start loading all zones %N"), Void)
			l_zones := all_zones
			from
				i := 1
			until
				i > l_zones.count
			loop
					-- Exclude obsolete Zones
				if not (l_zones.item (i).mnemonic.is_equal ('I') or l_zones.item (i).mnemonic.is_equal ('F')) then
					output (Global.interpreted ("($now) start loading zone: $1 %N"), <<l_zones.item (i).mnemonic>>)
					l_zones.item (i).load_all_entities
				end
				i := i + 1
			end
			output (Global.interpreted ("($now) end loading all zones %N"), Void)
		rescue
			if l_gc then
				memory.collection_on
			else
				memory.collection_off
			end
		end

	all_peers (a_entity: ENTITY; a_other_context: SUPPLIER_CONTEXT): ROSE_LINEAR_ARRAY [ENTITY] is
			-- All peers of a 'a_entity" in 'a_other_context'
		do
			if a_entity /= Void and then a_other_context /= Void then
				Result := a_other_context.handler (a_entity.category_id).all_peers (a_entity)
			end
		end

	currency_from_brim_code (a_key: STRING; a_date: ROSE_DATE): like currency_template is
			-- The CURRENCY whose brim matches a_key on a_date
			--   (the current date will be used if a_date is Void)
			-- or Void if no such CURRENCY is found
		require
			a_key_not_void: a_key /= Void
			a_date_may_be_void: True
		do
			Result := currency_accessor.entity_from_identifier (a_key, a_date, identifier_from_rose (attribute_ids.brim_code_type_id))
		ensure
			currency_from_brim_code_may_be_void: True
		end

	currency_by_iso_code (a_key: STRING; a_date: ROSE_DATE): like currency_template is
			--currency which has this iso code ... the date is not considered in caching so there
			--is an implicit assumption that iso codes are never retired
		require
			valid_key: a_key /= Void
		do
			if currencies_by_iso = Void then
				create currencies_by_iso.make (10)
			end
			currencies_by_iso.search (a_key.as_upper)
			if not currencies_by_iso.found then
				Result := currency_accessor.entity_from_identifier (a_key.as_upper, a_date, identifier_from_rose (attribute_ids.iso_code_type_id))
				if Result /= Void then
					currencies_by_iso.force (Result, a_key.as_upper)
				end
			else
				Result := currencies_by_iso.found_item
			end
		end

	currency_by_xrf_code (a_key: STRING; a_date: ROSE_DATE): like currency_template is
			-- Currency by xrf code
			-- Providing Void for `a_date' will return last known currency with `a_key' xrf code
		local
			l_currency_history: ROSE_ENTITY_HISTORY [like currency_template]
		do
			if currencies_by_xrf = Void then
				create currencies_by_xrf.make_equal (0)
				all_key_records (ids.currency_category_id, identifier (ids.xrf_code_type_id)).do_all (agent add_currency_by_xrf)
				currencies_by_xrf.do_all (agent {ROSE_ENTITY_HISTORY [like currency_template]}.sort_by (agent end_date_comparison))
			end
			currencies_by_xrf.search (a_key)
			if currencies_by_xrf.found then
				l_currency_history := currencies_by_xrf.found_item
				if a_date = Void then
					if l_currency_history.last /= Void then
						Result := l_currency_history.last.entity
					end
				else
					Result := l_currency_history.witness (a_date)
				end
			end
		ensure
			void_key_means_void_result: a_key = Void implies Result = Void
		end

	country_by_iso_code (a_key: STRING; a_date: ROSE_DATE): like country_template is
			-- The country identified by 'a_key' on 'a_date'
		do
			if countries_by_iso_code = Void then
				create countries_by_iso_code.make (10)
			end
			countries_by_iso_code.search (a_key)
			if not countries_by_iso_code.found then
				Result := country_accessor.entity_from_identifier (a_key, a_date, identifier_from_rose (attribute_ids.iso_code_type_id))
				countries_by_iso_code.force (Result, a_key)
			else
				Result := countries_by_iso_code.found_item
			end
		end

	country_by_old_positional_id (a_key: STRING; a_date: ROSE_DATE): like country_template is
			-- The country identified by 'a_key' on 'a_date'
		do
			if countries_by_old_positional_id = Void then
				create countries_by_old_positional_id.make (10)
			end
			countries_by_old_positional_id.search (a_key)
			if not countries_by_old_positional_id.found then
				Result := country_accessor.entity_from_identifier (a_key, a_date, identifier_from_rose (attribute_ids.old_positional_id_type_id))
				countries_by_old_positional_id.force (Result, a_key)
			else
				Result := countries_by_old_positional_id.found_item
			end
		end

	similars_group_by_display_order (a_key: STRING; a_date: ROSE_DATE): like similars_group_template is
			-- The similars group identified by 'a_key' on 'a_date'
		do
			if similars_groups_by_display_order = Void then
				create similars_groups_by_display_order.make (10)
			end
			similars_groups_by_display_order.search (a_key)
			if not similars_groups_by_display_order.found then
				Result := similars_group_accessor.entity_from_identifier (a_key, a_date, identifier_from_rose (attribute_ids.display_order_type_id))
				similars_groups_by_display_order.force (Result, a_key)
			else
				Result := similars_groups_by_display_order.found_item
			end
		end

	similars_group_by_old_positional_id (a_key: STRING; a_date: ROSE_DATE): like similars_group_template is
			-- The similars group identified by 'a_key' on 'a_date'
		do
			if similars_groups_by_old_positional_id = Void then
				create similars_groups_by_old_positional_id.make (10)
			end
			similars_groups_by_old_positional_id.search (a_key)
			if not similars_groups_by_old_positional_id.found then
				Result := similars_group_accessor.entity_from_identifier (a_key, a_date, identifier_from_rose (attribute_ids.old_positional_id_type_id))
				similars_groups_by_old_positional_id.force (Result, a_key)
			else
				Result := similars_groups_by_old_positional_id.found_item
			end
		end

	industry_by_old_positional_id (a_key: STRING; a_date: ROSE_DATE): like industry_template is
			-- The industry identified by 'a_key' on 'a_date'
		do
			if industries_by_old_positional_id = Void then
				create industries_by_old_positional_id.make (10)
			end
			industries_by_old_positional_id.search (a_key)
			if not industries_by_old_positional_id.found then
				Result := industry_accessor.entity_from_identifier (a_key, a_date, identifier_from_rose (attribute_ids.old_positional_id_type_id))
				industries_by_old_positional_id.force (Result, a_key)
			else
				Result := industries_by_old_positional_id.found_item
			end
		end

	economic_sector_group_by_name (a_key: STRING; a_date: ROSE_DATE): like economic_sector_group_template is
			-- The economic sector group identified by 'a_key' on 'a_date'
		require
			valid_key: a_key /= Void
		local
			l_key: KEY_PAIR [STRING, ROSE_DATE]
		do
			create l_key.make (a_key.as_upper, a_date)
			if economic_sector_groups_by_name = Void then
				create economic_sector_groups_by_name.make (10)
			end
			economic_sector_groups_by_name.search (l_key)
			if not economic_sector_groups_by_name.found then
				Result := economic_sector_group_accessor.entity_from_identifier (a_key.as_upper, a_date, identifier_from_rose (attribute_ids.name_type_id))
				economic_sector_groups_by_name.force (Result, l_key)
			else
				Result := economic_sector_groups_by_name.found_item
			end
		end

	currencies_with_data: ROSE_LINEAR_ARRAY [CURRENCY] is
			-- Currencies with available exchange rate data
		local
			l_predicate: like new_signature_predicate
			l_handler: CATEGORY_HANDLER [SUPPLIER_CONTEXT, CURRENCY]
			l_currencies: ROSE_LINEAR [CURRENCY]
		do
			if currencies_with_data_saved = Void then
				l_predicate := new_signature_predicate
				l_predicate.retrieve (Void, Void)
				if l_predicate.array.count > 0 then
					l_handler := currency_accessor
					if l_handler /= Void then
						create currencies_with_data_saved.make_empty
						l_currencies := l_handler.all_entities
						from
							l_currencies.start
						until
							l_currencies.off
						loop
							if l_predicate.array.has (l_currencies.item.handle) then
								currencies_with_data_saved.extend (l_currencies.item)
							end
							l_currencies.forth
						end
					end
				end
			end
			Result := currencies_with_data_saved
		end

	company_zonal_surrogate(a_tuple : TUPLE[zone : CHARACTER;handle : INTEGER]) : like company_template
			-- zonal surrogate for zone with mnenonic `a_tuple'.zone of company with handle `a_tuple'.handle
		local
			l_company : like company_template
			l_zone : like zone
		do
			l_company := company_accessor.entity (a_tuple.handle)
			if l_company /= Void then
				if l_company.zone_mnemonic = a_tuple.zone then
					Result := l_company
				else
					l_zone := zone(a_tuple.zone)
					if l_zone /= Void then
						Result := l_company.surrogate (l_zone)
					end
				end
			end
		end

feature -- Handler redefinition

	supplier_specific_mappings_history (a_entity_id: INTEGER; a_other_handler: CATEGORY_HANDLER [SUPPLIER_CONTEXT, ENTITY]): ROSE_INTERVAL_HISTORY [INTEGER, ROSE_CLOSED_OPEN_INTERVAL [INTEGER]] is
			-- If `mappings_history' = Void uses attribute value matches to find peers (i.e. kludge)
			-- Identifier_integeration should make this feature obsolete
		local
			l_entity: ENTITY
			l_entities: ROSE_LINEAR_ARRAY [ENTITY]
			l_attribute_records: ROSE_LINEAR_ARRAY [ENTITY_ATTRIBUTE_RECORD]
			l_history: ROSE_ENTITY_HISTORY [ENTITY]
			l_record: ENTITY_ATTRIBUTE_RECORD
			l_context: SUPPLIER_CONTEXT
			l_sedol: ENTITY_IDENTIFIER
			i, j: INTEGER
			l_interval: ENTITY_INTERVAL [ENTITY]
			l_int_interval: ROSE_CLOSED_OPEN_INTERVAL [INTEGER]
		do
			create Result.make_empty
			create l_entities.make_empty
				-- Bloomberg listing implementation
			l_context := a_other_handler.context
			if l_context.supplier_id = ids.bloomberg_equity_supplier_product_id then
				l_entity := entity_from_id (a_entity_id)
				if l_entity /= Void and then l_entity.category_id = ids.listing_category_id then
					l_attribute_records := l_entity.attribute_history (attribute_name_by_id (ids.sedol_type_id))
					from
						i := 1
					until
						i > l_attribute_records.count
					loop
						l_record := l_attribute_records [i]
						create l_sedol.make (l_context, l_context.rose_attribute_to_local [ids.sedol_type_id])
						l_history := a_other_handler.entity_history_from_identifier (l_record.attribute_value, l_sedol)
						from
							j := 1
						until
							j > l_history.count
						loop
							l_interval := l_history.item (j)
							if l_interval.start_date < l_record.end_date and then l_interval.end_date > l_record.start_date then
								if l_interval.entity.peers_history (Current).is_empty then
									l_entities.extend (l_interval.entity)
								end
							end
							j := j + 1
						end
						i := i + 1
					end
					from
						i := 1
					until
						i > l_entities.count
					loop
						l_entity := l_entities [i]
						create l_int_interval.make (l_entity.handle, l_entity.start_date, l_entity.end_date, 0)
						Result.extend (l_int_interval)
						i := i + 1
					end
				end
			end
		end

	listing_accessor: ROSE_CONTEXT_CATEGORY_HANDLER [like context, like listing_template] is
			-- Category handler for listing
		do
			debug ("make_movies")
				if movie.playback_movie then
					movie.checkpoint
				end
			end
			Result ?= category_handlers.safe_item (listing_category_id)
			if Result = Void then
				create Result.make (context, listing_category_id)
				if Result.identifiers /= Void then
					category_handlers.force (Result, listing_category_id)
				else
					debug ("make_movies")
						if movie.playback_movie then
							movie.restore_from_checkpoint
						end
					end
				end
			end
			debug ("make_movies")
				if movie.playback_movie then
					movie.clear_checkpoint
				end
			end
		end

feature -- Identifiers

	zonal_identifier (a_handle: INTEGER): like zonal_entity_identifier_template is
			-- Get a zonal identifier by handle
		require
			identifiers_not_void: identifiers_available
		do
			Result ?= identifiers.item (a_handle)
			if Result = Void then
				create Result.make (Current, a_handle)
				identifiers.force (Result, a_handle)
			end
		ensure
			valid_result: Result /= Void
		end

	zonal_identifier_from_name (a_name: STRING): like zonal_entity_identifier_template is
			-- Get a zonal identifier by name
		require
			identifiers_not_void: identifiers_available
		local
			l_handle, l_rose_handle: INTEGER
		do
			a_name.to_upper
			l_rose_handle := attribute_id_by_name (a_name)
			if l_rose_handle > 0 then
				if has_rose_handles then
					l_handle := l_rose_handle
				else
					l_handle := rose_attribute_to_local.item (l_rose_handle)
				end
				Result ?= identifiers.item (l_handle)
				if Result = Void then
					create Result.make (Current, l_handle)
					identifiers.force (Result, l_handle)
				end
			end
		end

feature -- Access

	all_zones: ROSE_LINEAR_ARRAY [like zone_template] is
			-- Array of zones
		local
			a: ARRAY [like zone_template]
		do
			if not all_zones_loaded then
				all_zones_loaded := True
				a := <<u_zone, e_zone, o_zone, j_zone, c_zone, s_zone, t_zone, f_zone, i_zone>>
				create all_zones_saved.make_from_array (a)
					--				trigger (u_zone)
					--				trigger (e_zone)
					--				trigger (o_zone)
					--				trigger (j_zone)
					--				trigger (c_zone)
					--				trigger (s_zone)
					--				trigger (t_zone)
					--				create all_zones_saved.make_from_linear (zones.linear_representation)
			end
			Result := all_zones_saved
		ensure
			all_zones_not_void: Result /= Void
			no_void_zones: not Result.has (Void)
		end

	all_zones_loaded: BOOLEAN
			-- Are zones loaded?

	all_zones_saved: ROSE_LINEAR_ARRAY [like zone_template]
			-- Zones cache

	t_zones: ROSE_LINEAR_ARRAY [like zone_template]
			-- Zones covered by the T zone
		local
			a: ARRAY [like zone_template]
		do
			a := <<u_zone, e_zone, o_zone, j_zone, c_zone, s_zone>>
			create Result.make_from_array (a)
		ensure
			t_zones_not_void: Result /= Void
			six_zones: Result.count = 6
			u_zone: Result.has (u_zone)
			e_zone: Result.has (e_zone)
			o_zone: Result.has (o_zone)
			j_zone: Result.has (j_zone)
			c_zone: Result.has (c_zone)
			s_zone: Result.has (s_zone)
		end

	t_zones_mnemonics: ROSE_LINEAR_ARRAY [CHARACTER]
			-- Zones covered by the T zone
		local
			l_t_zones: like t_zones
		once
			l_t_zones := t_zones
			create Result.make_empty
			across
				l_t_zones as ai_zone
			loop
				Result.extend (ai_zone.item.mnemonic.as_upper)
			end
		ensure
			not_void: Result /= Void
			six_zones: Result.count = 6
		end

	zones: SPECIAL [like zone_template]
			-- Cache for zones, indexed by mnemonics

	trigger (a_item: ANY) is
		do
		end

	listing_by_brim_code (a_key: STRING; a_date: ROSE_DATE): like listing_template is
			--listing which has this brimcode on this date

		do
			Result ?= listing_at (attributes_brim_code (a_key), a_date)
			if Result = Void then
				Result := listing_at (attributes_brim_code (a_key), a_date)
			end
		end

	common_attributes (a_entity: ENTITY): ROSE_LINEAR_ARRAY [ENTITY_IDENTIFIER] is
			-- Cache only those attributes that will be needed when making the new entities.
			-- Called by ROSE_ATTRIBUTE_TABLE:add_item when a lot of entities will be made.
			-- Note that this could be changed to handle any ENTITY but will require changes to 35 classes.
		local
			l_investment_entity: INVESTMENT_ENTITY
		do
			l_investment_entity ?= a_entity
			if l_investment_entity /= Void then
				Result := l_investment_entity.attributes_used_in_make
			end
		end

	composite_listings: HASH_TABLE [STRING, LISTING] is
			-- Composite listings hash map, used by LISTING
		local
			l_reader: STRING_PREDICATE
			l_security: SECURITY
			l_listing: LISTING
			i: INTEGER
			l_retried: BOOLEAN
			l_request: PES_QUERY_COMPOSITES
			l_result: PES_RESULT_COMPOSITES
		do
			if composite_listings_saved = Void then
				if use_pes and not l_retried then
					create l_request.make (context_index)
					l_result ?= pes_api.send_general_request (l_request).get
					check
						result_available: l_result /= Void
					end
					if l_result.remote_exception then
						error.raise (l_result.tag_name)
					end
					Result := l_result.composites_hash_map
				else
					create l_reader.make (servers.make_from_repository ("composites_common"), "select distinct brimco + sec from composite_assets_nav")
					if not l_reader.success then
						error.raise ("listing.composites - problem creating predicate")
					else
						l_reader.retrieve (Void, Void)
						if not l_reader.success then
							error.raise ("listing.composites - problem executing predicate")
						else
							create Result.make (l_reader.array.count * 2)
							from
								l_reader.array.start
							until
								l_reader.array.off
							loop
								if l_reader.array.current_item /= Void then
									l_security := security_at (attributes_brim_code (l_reader.array.current_item), create {ROSE_DATE}.make_today)
									if l_security /= Void then
										from
											i := 1
										until
											i > l_security.listing_count
										loop
											l_listing := l_security.listing (i)
											Result.put (l_reader.array.current_item, l_listing)
											i := i + 1
										end
									end
								end
								l_reader.array.forth
							end
							Result.compare_objects
						end
					end
				end
				composite_listings_saved := Result
			else
				Result := composite_listings_saved
			end
		ensure
			result_not_void: Result /= Void
			compare_object: Result.object_comparison
		rescue
			l_retried := True
			retry
		end

	composite_listings_saved: HASH_TABLE [STRING, LISTING]

feature -- Instrument

	instrument_from_brim_code (a_key: STRING; a_date: ROSE_DATE): LEGACY_INSTRUMENT is
			-- The LEGACY_INSTRUMENT whose brim matches a_key on a_date
		require
			a_key_not_void: a_key /= Void
		do
			Result := instrument_from_identifiers (a_key, a_date, <<brim_code, local_brim_code, j_brim_code, s_brim_code, e_brim_code>>)
		end

	instrument_from_xrfzone_code (a_key: STRING; a_date: ROSE_DATE): LEGACY_INSTRUMENT is
			-- The LEGACY_INSTRUMENT whose xrfzone code matches `a_key' on `a_date'
		do
			Result := instrument_from_identifiers (a_key, a_date, <<xrfzone_code, local_xrfzone_code, j_xrfzone_code, s_xrfzone_code, e_xrfzone_code>>)
		end

	instrument_from_riemsym_code (a_key: STRING; a_date: ROSE_DATE): LEGACY_INSTRUMENT is
		obsolete
			"Use ZONE[LOCATION].instrument_from_riemsym_code instead"
		do
		end

	instrument_from_identifiers (a_key: STRING; a_date: ROSE_DATE; a_identifiers: ARRAY [ENTITY_IDENTIFIER]): LEGACY_INSTRUMENT is
		local
			l_instruments: like instruments_from_identifiers
			l_identifier: ENTITY_IDENTIFIER
			i: INTEGER
		do
			from
				i := 1
			until
				i > a_identifiers.count or else Result /= Void
			loop
				l_identifier := a_identifiers.item (i)
				if l_identifier /= Void then
					l_instruments := instruments_from_identifiers (a_key, a_date, <<l_identifier>>)
					if not l_instruments.is_empty then
						Result := l_instruments.first
					end
				end
				i := i + 1
			end
		end

feature -- Instruments

	instruments_from_brim_code (a_key: STRING; a_date: ROSE_DATE): ROSE_LINEAR_ARRAY [LEGACY_INSTRUMENT] is
		do
			Result := instruments_from_identifiers (a_key, a_date, <<brim_code, local_brim_code, j_brim_code>>)
		end

	instruments_from_riemsym_code (a_key: STRING; a_date: ROSE_DATE): ROSE_LINEAR_ARRAY [LEGACY_INSTRUMENT]
			--Instruments from old riemsym key
		do
			Result := instruments_from_identifiers (a_key, a_date, <<riemsym_code>>)
		end

	instruments_from_identifiers (a_key: STRING; a_date: ROSE_DATE; a_identifiers: ARRAY [ENTITY_IDENTIFIER]): ROSE_LINEAR_ARRAY [LEGACY_INSTRUMENT] is
		local
			i: INTEGER
		do
			create Result.make_empty
			from
				i := 1
			until
				i > a_identifiers.count
			loop
				a_identifiers.item (i).all_entities (a_key, a_date).do_all (agent add_legacy_instrument (Result, ?))
				i := i + 1
			end
		ensure
			not_void: Result /= Void
		end

	add_legacy_instrument (a_array: ROSE_LINEAR_ARRAY [LEGACY_INSTRUMENT]; a_entity: ENTITY)
			-- If `a_entity' is an instrument then add it to `a_array'.
		local
			l_instrument: LEGACY_INSTRUMENT
		do
			l_instrument ?= a_entity
			if l_instrument /= Void then
				a_array.extend (l_instrument)
			end
		end

feature -- Instrument identifiers

	brim_code: ENTITY_IDENTIFIER is
			-- BRIM CODE attribute for this context
		do
			Result := identifier_from_rose (attribute_ids.brim_code_type_id)
		end

	local_brim_code: ENTITY_IDENTIFIER is
			-- LOCAL BRIM CODE attribute for this context
		do
			Result := identifier_from_rose (attribute_ids.local_brim_code_type_id)
		end

	j_brim_code: ENTITY_IDENTIFIER is
 			-- J BRIM CODE attribute for this context
		do
			Result := identifier_from_rose (attribute_ids.j_brim_code_type_id)
		end

	s_brim_code: ENTITY_IDENTIFIER is
 			-- S BRIM CODE attribute for this context
		do
			Result := identifier_from_rose (attribute_ids.s_brim_code_type_id)
		end

	e_brim_code: ENTITY_IDENTIFIER is
		do
			Result := identifier_from_rose (attribute_ids.e_brim_code_type_id)
		end

	riemsym_code: ENTITY_IDENTIFIER is
		do
			Result := identifier_from_rose (attribute_ids.riemsym_code_type_id)
		end

	xrfzone_code: ENTITY_IDENTIFIER is
			-- XRFZone code ID
		do
			Result := identifier_from_rose (attribute_ids.xrfzone_type_id)
		end

	local_xrfzone_code: ENTITY_IDENTIFIER is
		do
			Result := identifier_from_rose (attribute_ids.local_xrfzone_type_id)
		end

	j_xrfzone_code: ENTITY_IDENTIFIER is
		do
			Result := identifier_from_rose (attribute_ids.j_xrfzone_type_id)
		end

	s_xrfzone_code: ENTITY_IDENTIFIER is
		do
			Result := identifier_from_rose (attribute_ids.s_xrfzone_type_id)
		end

	e_xrfzone_code: ENTITY_IDENTIFIER is
		do
			Result := identifier_from_rose (attribute_ids.e_xrfzone_type_id)
		end

	fcxrf_code: like entity_identifier_template is
			-- FCXRF Code Type Id
		do
			Result := identifier_from_rose (attribute_ids.fcxrf_code_type_id)
		end

feature -- Zonal currencies and primary countries

	jp: like country_template is
			-- Japan
		do
			if jp_saved = Void then
				jp_saved := country_at (iso_code ("JP"), Void)
			end
			Result := jp_saved
		end

	gb: like country_template is
			-- United Kingdom
		do
			if gb_saved = Void then
				gb_saved := country_at (iso_code ("GB"), Void)
			end
			Result := gb_saved
		end

	us: like country_template is
			-- USA
		do
			if us_saved = Void then
				us_saved := country_at (iso_code ("US"), Void)
			end
			Result := us_saved
		end

	au: like country_template is
			-- Australia
		do
			if au_saved = Void then
				au_saved := country_at (iso_code ("AU"), Void)
			end
			Result := au_saved
		end

	mx: like country_template is
			-- Mexico
		do
			if mx_saved = Void then
				mx_saved := country_at (iso_code ("MX"), Void)
			end
			Result := mx_saved
		end

	ca: like country_template is
			-- Canada
		do
			if ca_saved = Void then
				ca_saved := country_at (iso_code ("CA"), Void)
			end
			Result := ca_saved
		end

	lu: like country_template is
			-- Luxembourg
		do
			if lu_saved = Void then
				lu_saved := country_at (iso_code ("LU"), Void)
			end
			Result := lu_saved
		end

	be: like country_template is
			-- Belgium
		do
			if be_saved = Void then
				be_saved := country_at (iso_code ("BE"), Void)
			end
			Result := be_saved
		end

	za: like country_template is
			-- South Africa
		do
			if za_saved = Void then
				za_saved := country_at (iso_code ("ZA"), Void)
			end
			Result := za_saved
		end

	il: like country_template is
			-- Israel
		do
			if il_saved = Void then
				il_saved := country_at (iso_code ("IL"), Void)
			end
			Result := il_saved
		end

	sg: like country_template is
			-- Singapore
		do
			if sg_saved = Void then
				sg_saved := country_at (iso_code ("SG"), Void)
			end
			Result := sg_saved
		end

	hk: like country_template is
			-- Hong Kong
		do
			if hk_saved = Void then
				hk_saved := country_at (iso_code ("HK"), Void)
			end
			Result := hk_saved
		end

	cn: like country_template is
			-- China
		do
			if cn_saved = Void then
				cn_saved := country_at (iso_code ("CN"), Void)
			end
			Result := cn_saved
		end

	jpy: like currency_template is
			-- Japanese Yen
		do
			if jpy_saved = Void then
				jpy_saved := currency_by_iso_code ("JPY", Void)
			end
			Result := jpy_saved
		end

	gbp: like currency_template is
			-- British Pound
		do
			if gbp_saved = Void then
				gbp_saved := currency_by_iso_code ("GBP", Void)
			end
			Result := gbp_saved
		end

	usd: like currency_template is
			-- US dollar
		do
			if usd_saved = Void then
				usd_saved := currency_by_iso_code ("USD", Void)
			end
			Result := usd_saved
		end

	eur: like currency_template is
			-- Euro
		do
			if eur_saved = Void then
				eur_saved := currency_by_iso_code ("EUR", Void)
			end
			Result := eur_saved
		end

	mxn: like currency_template is
			-- Mexican peso
		do
			if mxn_saved = Void then
				mxn_saved := currency_by_iso_code ("MXN", Void)
			end
			Result := mxn_saved
		end

	cad: like currency_template is
			-- Canadian dollar
		do
			if cad_saved = Void then
				cad_saved := currency_by_iso_code ("CAD", Void)
			end
			Result := cad_saved
		end

feature -- Industries

	industries: ROSE_LINEAR_ARRAY [like industry_template] is
			-- Collection of industries
		local
			l_industry: like industry_template
			l_industries: ROSE_LINEAR_ARRAY [like industry_template]
		do
			if industries_saved = Void then
				l_industries := industry_accessor.all_entities
				create industries_saved.make_empty
				from
					l_industries.start
				until
					l_industries.off
				loop
					l_industry := l_industries.current_item
					if l_industry /= Void and then l_industry.industry_id > 0 then
						industries_saved.force (l_industry, l_industry.old_positional_id)
					end
					l_industries.forth
				end
			end
			Result := industries_saved
		end

	industry_count(a_date : ROSE_DATE) : INTEGER
			--Number of industry entities on a_date ... for now it is the size of industries, which is not date-dependent (yet)
		do
			Result := industries.count
		end

	company_fcxrf_records (a_fcxrf_code, a_zone_code: INTEGER): ROSE_LINEAR_ARRAY [COMPANY_FCXRF_RECORD] is
		local
			l_reader: COLLECTING_PREDICATE [COMPANY_FCXRF_RECORD]
		do
			create l_reader.make (server, "select distinct a.entity_1_id as handle,cast (c.attribute_value as int) as fcxrf," + "d.attribute_value as zone from entity_relationship a,entity_relationship b," + "entity_attribute c,entity_attribute d where a.relationship_type_id = 1 " + "and b.relationship_type_id = 2 and a.entity_2_id = b.entity_1_id and " + "b.entity_2_id = c.entity_id and c.attribute_type_id = " + a_fcxrf_code.out + " and " + "d.entity_id = c.entity_id and d.attribute_type_id = " + a_zone_code.out + " order by a.entity_1_id")
			l_reader.retrieve (Void, Void)
			Result := l_reader.array
		end

	zonal_xrf_records (a_category_id: INTEGER; a_zone: CHARACTER): ROSE_LINEAR_ARRAY [ZONAL_XRF_RECORD] is
			-- Return a list of all xrf codes in zone `a_zone' for category `a_category'
		local
			l_predicate: COLLECTING_PREDICATE [ZONAL_XRF_RECORD]
		do
			l_predicate := zonal_xrfs_predicate_table.item (supplier_id)
			if l_predicate = Void then
				create l_predicate.make (server, "select * from xrf_zone where category_id = ? and zone = ?")
				zonal_xrfs_predicate_table.put (l_predicate, supplier_id)
			end
			l_predicate.retrieve (Void, <<a_category_id, a_zone>>)
			Result := l_predicate.array
		end

	zonal_fxrfs (a_attribute_id: INTEGER; a_zone: CHARACTER): ROSE_LINEAR_ARRAY [TUPLE [entity_id: INTEGER; category_id: INTEGER; fxrf_code: INTEGER]] is
			-- Return a list of all fxrf codes in zone `a_zone'
			-- `attribute_id' is either the ID for FCXRF or FSXRF
			-- e.g. `attribute_ids.fcxrf_code_type_id'
		do
			create Result.make_from_array (<<>>)
			fxrf_predicate.set_container (Result)
			fxrf_predicate.retrieve (Void, <<a_attribute_id, a_zone>>)
		ensure
			result_not_void: Result /= Void
		end

	min_company_xrf (a_zone: ZONE [LOCATION]): INTEGER is
			-- Minimum cxrf for 'a_zone'
		require
			zone_not_void: a_zone /= Void
		local
			l_predicate: like new_min_cxrf_reader
		do
			l_predicate := new_min_cxrf_reader
			l_predicate.retrieve (Void, <<a_zone.mnemonic.out>>)
			Result := l_predicate.array.first
		end

	max_category_xrf (a_zone_mnemonic: CHARACTER; a_category_id: INTEGER): INTEGER is
			-- Maximum xrf for 'a_category_id' for a zone
		local
			l_predicate: like new_category_max_reader
		do
			if category_maxes.is_empty then
				l_predicate := new_category_max_reader
				l_predicate.apply (agent handle_category_maxes (?, category_maxes), Void, <<context.attribute_id_by_name ("XRFZONE")>>)
			end
			Result := category_maxes.item (create {TUPLE_KEY}.make ([a_category_id, a_zone_mnemonic]))
		end

	min_category_xrf (a_zone_mnemonic: CHARACTER; a_category_id: INTEGER): INTEGER is
			-- Minimum xrf for 'a_category_id' for a zone
		local
			l_predicate: like new_category_min_reader
		do
			if category_mins.is_empty then
				l_predicate := new_category_min_reader
				l_predicate.apply (agent handle_category_maxes (?, category_mins), Void, <<context.attribute_id_by_name ("XRFZONE")>>)
			end
			Result := category_mins.item (create {TUPLE_KEY}.make ([a_category_id, a_zone_mnemonic]))
		end

	rmdat_is_active (a_zone: CHARACTER): BOOLEAN is
			-- Is the market data for 'a_zone' active on the migrated system?
		do
			active_windows_rmdat_predicate.retrieve (Void, <<a_zone.out>>)
			Result := active_windows_rmdat_predicate.array.count = 1
		end

	currency_exchange_rates (a_currency: CURRENCY): ROSE_LINEAR_ARRAY [EXCHANGE_RATE_RECORD] is
			-- Exchange rates for 'a_currency'
		require
			valid_currency: a_currency /= Void
		do
			create Result.make_empty
			exchange_rate_predicate.set_container (Result)
			exchange_rate_predicate.retrieve (Void, <<a_currency.handle>>)
		end

feature {NONE} -- predicate

	fx_server: SQL_DATA_SERVER is
		do
			if server = servers.rose_entity_background_server or server = servers.rose_entity_foreground_server then
				Result := servers.rose_foreign_exchange_server
			else
				Result := servers.foreign_exchange_server
			end
		end

	exchange_rate_predicate: COLLECTING_PREDICATE [EXCHANGE_RATE_RECORD] is
		do
			create Result.make_strict (fx_server, "select * from exchange_rates where currency_id = ?")
		ensure
			valid_result: Result /= Void
		end

	fxrf_predicate: TUPLE_PREDICATE [TUPLE [entity_id: INTEGER; category_id: INTEGER; fxrf_code: INTEGER]] is
			-- Predicate to retrieve fxrf codes
			-- Takes arguments attribute_id (INTEGER) and zone (STRING)
			-- attribute_id is either the ID for FCXRF or FSXRF
		require
			server_not_void: servers.rose_entity_foreground_server /= Void
		do
			if fxrf_predicate_saved = Void then
				create fxrf_predicate_saved.make (server, "SELECT dbo.entity_attribute.entity_id, dbo.entity_attribute.category_id, CAST (dbo.entity_attribute.attribute_value AS int) AS fxrf FROM dbo.entity_attribute INNER JOIN dbo.entity_attribute AS [entity_attribute_1 (with nolock)] ON dbo.entity_attribute.entity_id = [entity_attribute_1 (with nolock)].entity_id AND dbo.entity_attribute.category_id = [entity_attribute_1 (with nolock)].category_id WHERE (dbo.entity_attribute.attribute_type_id = ?) AND ([entity_attribute_1 (with nolock)].attribute_type_id = 83) AND ([entity_attribute_1 (with nolock)].attribute_value = ?)")
			end
			Result := fxrf_predicate_saved
		end

	fxrf_predicate_saved: like fxrf_predicate

	new_category_max_reader: SIMPLE_PREDICATE [ZONAL_XRF_RECORD] is
			-- Predicate to find max attribute value for a category
		do
			create Result.make (server, "select category_id, substring (attribute_value, datalength (attribute_value), datalength (attribute_value)) as zone, max (cast (substring (attribute_value, 1, datalength (attribute_value)-1) as int)) as xrf " + " from entity_attribute " + " where attribute_type_id = ? " + " group by category_id, substring (attribute_value, datalength (attribute_value), datalength (attribute_value)) " + " order by category_id, substring (attribute_value, datalength (attribute_value), datalength (attribute_value))")
			if not Result.success then
				error.raise ("could not create category_max_reader")
			end
		end

	new_category_min_reader: SIMPLE_PREDICATE [ZONAL_XRF_RECORD] is
			-- Predicate to find min attribute value for a category
		do
			create Result.make (server, "select category_id, substring (attribute_value, datalength (attribute_value), datalength (attribute_value)) as zone, min (cast (substring (attribute_value, 1, datalength (attribute_value)-1) as int)) as xrf " + " from entity_attribute " + " where attribute_type_id = ? " + " group by category_id, substring (attribute_value, datalength (attribute_value), datalength (attribute_value)) " + " order by category_id, substring (attribute_value, datalength (attribute_value), datalength (attribute_value))")
			if not Result.success then
				error.raise ("could not create category_min_reader")
			end
		end

	category_maxes: HASH_TABLE [INTEGER, TUPLE_KEY] is
			-- Cache table of maximum category xrfs keyed by zone and category_id
		once
			create Result.make (50)
		end

	category_mins: HASH_TABLE [INTEGER, TUPLE_KEY] is
			-- Cache table of minimum category xrfs keyed by zone and category_id
		once
			create Result.make (50)
		end

	handle_category_maxes (a_record: ZONAL_XRF_RECORD; a_table: HASH_TABLE [INTEGER, TUPLE_KEY]) is
		do
			a_table.force (a_record.xrf, create {TUPLE_KEY}.make ([a_record.category_id, a_record.zone.item (1)]))
		end

	active_windows_rmdat_predicate: STRING_PREDICATE is
		once
			create Result.make_strict (servers.repository_server, "select zone from active_rmdat where zone = ?")
		end

	zonal_xrfs_predicate_table: HASH_TABLE [COLLECTING_PREDICATE [ZONAL_XRF_RECORD], INTEGER] is
		once
			create Result.make (10)
		end

	new_min_cxrf_reader: INTEGER_PREDICATE is
			-- New minimum CXRF sql predicate
		do
			create Result.make (server, "select min (fxrf) from zonal_fxrf where zone = ?")
		end

	new_signature_predicate: INTEGER_PREDICATE is
			--signature predicate
		local
			l_server: SQL_DATA_SERVER
		do
			l_server := servers.rose_foreign_exchange_server
			create Result.make (l_server, "select distinct currency_id from data_signatures")
			if not Result.success then
				print ("Failed to create signature_predicate")
			end
		end

	jpy_saved: like jpy

	gbp_saved: like gbp

	usd_saved: like usd

	eur_saved: like eur

	mxn_saved: like mxn

	cad_saved: like cad

	jp_saved: like jp

	gb_saved: like gb

	us_saved: like us

	au_saved: like au

	mx_saved: like mx

	ca_saved: like ca

	lu_saved: like lu

	be_saved: like be
			-- Cache for `be'

	il_saved: like il
			-- Cache for `il'

	za_saved: like za
			-- Cache for `za'
	hk_saved: like hk
			-- Cache for `hk'
	sg_saved: like sg
			-- Cache for `sg'
	cn_saved: like cn
			-- Cache for `cn'

	industries_saved: ROSE_LINEAR_ARRAY [like industry_template]

feature {NONE} -- Local caches for simple entities

	currencies_by_iso: HASH_TABLE [like currency_template, STRING]
			-- Iso -> Currency

	economic_sector_groups_by_name: HASH_TABLE [like economic_sector_group_template, KEY_PAIR [STRING, ROSE_DATE]]
			-- Name -> Economic Sector Group

	currencies_by_xrf: DS_HASH_TABLE [ROSE_ENTITY_HISTORY [like currency_template], STRING]
			-- Currencies by xrf table

	add_currency_by_xrf (a_record: ENTITY_ATTRIBUTE_RECORD) is
			-- Add `a_record' corresponding interval to `currencies_by_xrf'
		require
			currencies_by_xrf_not_void: currencies_by_xrf /= Void
		local
			l_history: ROSE_ENTITY_HISTORY [like currency_template]
			l_interval: ENTITY_INTERVAL [like currency_template]
			l_currency: like currency_template
			l_start_date: ROSE_DATE
			l_end_date: ROSE_DATE
		do
			if a_record /= Void and then a_record.attribute_value /= Void then
				currencies_by_xrf.search (a_record.attribute_value)
				if currencies_by_xrf.found then
					l_history := currencies_by_xrf.found_item
				else
					create l_history.make_empty
					currencies_by_xrf.force_new (l_history, a_record.attribute_value)
				end
				l_currency := currency_accessor.entity (a_record.entity_id)
				if l_currency /= Void then
					l_start_date := a_record.start_date
					if l_start_date = Void then
						create l_start_date.make_min_date
					end
					l_end_date := a_record.end_date
					if l_end_date = Void then
						create l_end_date.make_max_date
					end
					create l_interval.make (l_currency, l_start_date, l_end_date, 0.0)
					l_history.extend (l_interval)
				end
			end
		end

	end_date_comparison (a, b: ENTITY_INTERVAL [like currency_template]): BOOLEAN is
			-- Compare end dates of `a' and 'b'
		do
			Result := (a /= Void and b /= Void) and then a.end_date < b.end_date
		end

	countries_by_iso_code: HASH_TABLE [like country_template, STRING]
			-- iso_code -> country table

	countries_by_old_positional_id: HASH_TABLE [like country_template, STRING]
			-- old_positional_id -> country table

	similars_groups_by_display_order: HASH_TABLE [like similars_group_template, STRING]
			-- display order -> similars group table

	similars_groups_by_old_positional_id: HASH_TABLE [like similars_group_template, STRING]
			-- old_positional_id -> similars group table

	industries_by_old_positional_id: HASH_TABLE [like industry_template, STRING]
			-- old_positional_id -> industry table

	currencies_with_data_saved: like currencies_with_data
			-- Cache for `currencies_with_data'

end
