note

	description:

		"Controller Class for Rose Cache Server"

	documentation: "http://prism/modules/wacko/wakka.php?wakka=PESServerdocumentation&v=1969"
	author: "Ueli Peter"
	last_editor: "$Author: c4 $"
	date: "$Date: 2012/02/29 $"
	revision: "$Revision: #19 $"
	path: "$File: //rose/source/kernel/entities/cache_server/rose_cs.e $"

deferred class	ROSE_CS

inherit

	DATABASE_SERVER
		rename
			servers as db_servers
		end

	PERSISTENT_SERVER_CLIENT
		export
			{NONE} all
			{ANY} server_name
		redefine
			load_server_list
		end

	ROSE_EXECUTION_ENVIRONMENT

feature {NONE} -- Initialization

	make_server (a_port: INTEGER; a_refresh: BOOLEAN;a_standalone: BOOLEAN)is
		-- initialization
		-- start worker thread and persistent server
		-- `a_standalone': server in standalone mode (without portserver) for debugging?
	do
		if represented_db = Void then
			represented_db := "No DB selected"
		end

		standalone := a_standalone
		port := a_port

		name := config.value ("/rose_cs/name")
		if name = Void then
			name := "persistent_entity_server"
		end
		machine := config.value ("/rose_cs/machine")
		server_directory := config.value ("/rose_cs/server_directory")

		-- make persistent server client from port server calls
		make (name)

		create input_queue.make
		create state_mutex.make
		create pers_server.make_cs (Current, a_port)
		create message_worker.make_cs (Current, a_refresh)
		message_worker.launch
		pers_server.execute
	end


feature -- Access

	arguments: STRING
		-- Arguments to start an new instance

	represented_db: STRING
		-- Versionof the db which is represented by this server

	input_queue: ROSE_CONCURRENT_LINKED_QUEUE[TUPLE[query: ROSE_CS_QUERY; future: ROSE_CONCURRENT_FUTURE[ROSE_CS_RESULT]]]
			-- input queue for requests

	state_transition is
			-- check current state and change to another state if needed
			-- this feature represents the logic of the server
			-- has to be implemented by a descendant
		deferred
		end

	set_state (a_state: INTEGER) is
			-- Set `state' to a `a_state', thread safe.
		require
			a_state_valid: is_valid_state (a_state)
		do
			state_mutex.lock
			state := a_state
			state_mutex.unlock
		ensure
			good_state: is_valid_state (state)
		end


	get_state: INTEGER is
			-- `state' (thread safe, for what it's worth)
		do
			state_mutex.lock
			Result := state
			state_mutex.unlock
		ensure
			good_state: is_valid_state (Result)
		end

feature -- States

	State_shuting_down: INTEGER is 2
			-- Server is ready to terminate.

	is_valid_state (a_state: INTEGER): BOOLEAN is
			-- Is `a_state' a valid state for `Current'?
		deferred
		end
	
feature {NONE} -- Port_Server_Commands	

	reconnect is
			-- reconnect and reset predicates
		do
			db_servers.pool.refresh
			if db_servers.repository_server /= Void then
				db_servers.repository_server.reconnect
			end
			if db_servers.repository /= Void then
				db_servers.repository.reconnect
			end
		end

	server_port_from_port_server: INTEGER
			-- check on which port this service is registred
		local
			l_message: PORT_SERVER_COMMAND
		do
			if not standalone then
				create l_message.make_from_message (new_port_server_query_message)
				l_message.set_server_arguments (<<arguments>>)
				l_message.set_do_not_check_alive (true)
				send_port_server_message (l_message)
				if port_server_message /= Void then
					Result := port_server_message.port
				end
			else
				Result := port
			end
		end

	persistent_server_instance_predicate: COLLECTING_PREDICATE[PORT_SERVER_MESSAGE] is
			-- get port on which the latest server is runnig
		local
			l_server: SQL_DATA_SERVER
		do
			l_server := db_servers.repository_server
			if l_server /= Void then
				create Result.make_strict(l_server,"select port from persistent_server_instances where machine = '" + machine +"' and name = '" + name + "'")
			end
		end

	change_port (a_arguments: STRING) is
			-- register this instance on the port server
			-- unregister the parent instance (the one which started this instance)
		local
			l_command: PORT_SERVER_COMMAND
			l_old_port: INTEGER
		do
			if not standalone then
				l_old_port := server_port_from_port_server
				if l_old_port /= 0 then
					--register service
					create l_command.make_from_message (new_port_server_query_message)
					l_command.set_process_id (global.env.current_process_id)
					l_command.set_port (port)
					l_command.set_register_instance (true)
					l_command.set_server_arguments (<<a_arguments>>)
					send_port_server_message (l_command)
					
					-- unregister parent
					l_command.set_port (l_old_port)
					l_command.set_process_id (0)
					l_command.set_register_instance (false)
					l_command.set_unregister_instance (true)
					send_port_server_message (l_command)
				else
					-- we couldn't contact the port server, so we won't try
				end
			end
		end

	start_new_instance is
			-- send a command to the portserver to spawn a new instance of this program
		local
			l_command: PORT_SERVER_COMMAND
		do
			if not standalone then
				create l_command.make_from_message (new_port_server_query_message)
				l_command.set_spawn_passive (true)
				l_command.set_server_arguments (<<"-refresh_first:true", arguments>>)
				send_port_server_message (l_command)
			end
		end

	send_port_server_message (a_message: PORT_SERVER_MESSAGE)
		-- send a_message to localhost
		local
			l_collecting: BOOLEAN
			l_target_hostname: STRING
		do
			l_collecting := memory.collecting
			l_target_hostname := current_server.address

			if l_target_hostname = Void then
				l_target_hostname := "<Void>"
			end

			-- Connect to the port_server.
			client_make (port_server_port, current_server.address)
			if is_verbose then
				trace ("$1: Connecting to port_server $2:$3%N",
							  <<generator, l_target_hostname, port_server_port>>)
			end

			-- Send it our request.
			in_out.independent_store (a_message)
			if is_verbose then
				trace ("$1: Receiving from port_server%N",<<generator>>)
			end
			sleep (1000)
			received := in_out.retrieved
			port_server_message ?= received
			--output(Global.interpreted("($now) " + "SENT PORT SERVER MESSAGE: %N"),Void)

			--output_message (a_message)
			--output(Global.interpreted("($now) " + "RECEIVED PORT SERVER MESSAGE %N"),Void)
			--output_message (port_server_message)

			-- Now disconnect
			cleanup
		rescue
			if l_collecting	then
				memory.collection_on
			else
				memory.collection_off
			end
			if is_verbose then
				output ("$1: port_server_connect: Error connecting to port_server on $2:$3%N$4%N",
							  <<generator, l_target_hostname, port_server_port, last_exception_explanation>>)
				if in_out /= Void then
					trace ("Error: $1: $2%N",
								  <<in_out.error_number, in_out.error>>)
				end
			end
			output ("$1: No port_server on $2 for $3%N",
						  <<generator, l_target_hostname,server_name>>)
			if not server_list_exhausted then
				next_server
				retry
			end
		end

		output_message (a_message: PORT_SERVER_MESSAGE) is
			-- for portserver debugging only. Outputs a port?server_message
		do
			io.put_string ("%N PORT SERVEr MESSAGE: %N Command line:" + a_message.command_line + "%N Arguments:" +a_message.get_flat_arguments + "%N" + a_message.out + "%N")

		end

feature {PES_SERVER} -- Implementation

	standalone: BOOLEAN
		-- true if server is in debug mode and not started by the portserver

	server_directory: STRING
		-- portservers directory

	machine: STRING
		--hostname of the local portserver

	name: STRING
		-- portserver nam of this executable

	message_worker_type: ROSE_CS_WORKER is
			-- must be redefined in descendants to decide which descendant of ROSE_CS_WORKER is used
		deferred
		end


	pers_server: ROSE_CS_PERSISTENT_SERVER
			-- the persistent server which handles client request

	message_worker: like message_worker_type
			-- this object represents the cache

	state: INTEGER
			-- Current server state

	state_mutex: CRITICAL_SECTION
			-- locking state

	port: INTEGER
			-- server port

	load_server_list  is
			-- localhost is the only server in serverlist
			-- we only want to communicate with the portserver on local host
		require else
			server_name_not_used: True
		local
			l_server_item: PERSISTENT_SERVER_LIST_ITEM
		do
			create l_server_item.make ("127.0.0.1", True)
			servers := <<l_server_item>>
			server_iteration_count := 0
			current_server_index := 1
		end

invariant

	good_state: is_valid_state (state)

end
