note

	description:

		"Class for accessing a Rose Cache Server as a client"

	author: "Jann Roeder"
	last_editor: "$Author: er $"
	date: "$Date: 2014/02/12 $"
	revision: "$Revision: #10 $"
	path: "$File: //rose/source/kernel/entities/cache_server/rose_cs_client_api.e $"

class	ROSE_CS_CLIENT_API

inherit

	ROSE_SHARED_GLOBAL

	THREAD_CONTROL
		export
			{NONE} all
		end

	EXCEPTIONS
		export
			{NONE} all
		end

create

	make_with_server_name

feature {NONE} -- Initialization

	make_with_server_name (a_server_name: STRING; a_arguments: ARRAY[STRING]) is
			-- Create client for server with name `a_server_name'
		require
			server_name_valid: a_server_name /= Void and then not a_server_name.is_empty
		do
			create result_available
			create requests.make (10)
			if a_arguments /= Void and then not a_arguments.is_empty then
				create client.make_with_arguments (Current, a_server_name, a_arguments)
			else
				create client.make (Current, a_server_name)
			end
		end

feature -- Events, these will only work if `gui' is set

	result_available: ACTION_SEQUENCE [TUPLE [ROSE_CS_FUTURE [ROSE_CS_RESULT]]]

	set_result_available (a_action_seq: like result_available) is
			-- Set the `result_available' action sequence to `a_action_seq'
		do
			result_available := a_action_seq
		end

feature -- Basic operations

	send_general_request (a_query: ROSE_CS_QUERY): ROSE_CS_FUTURE[ROSE_CS_RESULT] is
			-- Send a request to the server
		do
			create Result.make (Current)
			send_query (a_query, Void, Result)
		end

	set_always_check_for_latest_server (a_value: BOOLEAN) is
			-- Set `always_check_for_latest_server'
			-- See comment for {ROSE_CS_CLIENT}.always_check_for_latest_server
		do
			client.set_always_check_for_latest_server (a_value)
		end

feature {NONE} -- Operations

	send_query (a_object: ROSE_CS_QUERY; a_stored_request: ROSE_CS_STORED_REQUEST; a_result: ROSE_CS_FUTURE[ROSE_CS_RESULT]) is
			-- Send a query object to the server via a socket connection
			-- `a_stored_request' is used to restore the answer, can be void
		require
			object_not_void: a_object /= Void
			result_not_void: a_result /= Void
		local
			l_stored_request: like a_stored_request
			l_retry_count: INTEGER
		do
			a_object.set_query_id (new_query_id)
			a_result.set_query_id (a_object.query_id)
			-- Add request to Hash Table
			-- The entry is removed in the restore_result call of ROSE_CS_RESULT
			if a_stored_request = Void then
				create l_stored_request.make (a_result)
			else
				l_stored_request := a_stored_request
			end
			requests.put_thread_safe (l_stored_request, a_object.query_id)
			client.send_object (a_object)
		rescue
			client.disconnect
			l_retry_count := l_retry_count + 1
			if l_retry_count <= 10
			then
				global.env.sleep(1000)	-- In milliseconds.
				retry
			end
		end

feature -- Access

	client: ROSE_CS_CLIENT
		-- Object for sending queries over socket

feature {ROSE_CS_FUTURE, ROSE_CS_RESULT, ROSE_CS_CLIENT} -- Thread safe access

	requests: ROSE_CONCURRENT_HASH_MAP[ROSE_CS_STORED_REQUEST, INTEGER]
		-- Store requests in order to be able to restore the reply

feature {NONE} -- Implementation Access

	query_id_internal: INTEGER
			-- Previous result from `new_query_id'

feature -- Factory

	new_query_id: INTEGER is
			-- Get a fresh query ID;
			-- Not idempotent.
		do
			query_id_internal := query_id_internal + 1
			Result := query_id_internal
		ensure
			incremented_or_wrap_around: Result = old query_id_internal + 1
		end

end
