note

	description:

		"Objects that represent a not-yet computed ROSE_CS_RESULT"

	author: "Jann Roeder"
	last_editor: "$Author: c4 $"
	date: "$Date: 2011/11/22 $"
	revision: "$Revision: #8 $"
	path: "$File: //rose/source/kernel/entities/cache_server/rose_cs_future.e $"

class	ROSE_CS_FUTURE[G->ROSE_CS_RESULT create make_exception end]

inherit

	ROSE_CONCURRENT_FUTURE[G]
		rename
			make as make_future,
			get as old_get,
			get_with_timeout as old_get_with_timeout
		export
			{NONE} old_get, old_get_with_timeout
		end

	ROSE_SHARED_GLOBAL
		export {NONE} all end

create

	make

feature {NONE} -- Creation

	make_with_id (a_api: ROSE_CS_CLIENT_API; a_query_id: INTEGER) is
			-- Creation
		require
			api_not_void: a_api /= Void
		do
			api := a_api
			query_id := a_query_id
			restored := False
			make_future
		end

	make (a_api: ROSE_CS_CLIENT_API) is
			-- Creation
		require
			api_not_void: a_api /= Void
		do
			make_with_id (a_api, 0)
		end

feature -- Access

	get: G is
			-- Get result and restore
		require
			not_freed: not freed or restored
		do
			Result := old_get
			if not restored then
				if not Result.remote_exception then
					debug ("rose_cs_future")
						output(Global.interpreted("($now) start restore result!%N"),Void)
					end
					Result.restore_result (api)
					debug ("rose_cs_future")
						output(Global.interpreted("($now) end restore result!%N"),Void)
					end
					restored := True
				else
					output(Global.interpreted("($now) Remote exception occured, request removed!%N"),Void)
					api.requests.remove_thread_safe (query_id)
					restored := True
				end
			end
		end

	get_with_timeout (a_timeout: INTEGER): TUPLE[result_valid: BOOLEAN; result_value: G] is
			-- Get result and restore, timeout in ms
			-- Call `free' if you no longer plan on retrieving the result
		require
			not_freed: not freed or restored
		do
			Result := old_get_with_timeout (a_timeout)
			if Result.result_valid then
				if not restored then
					if not Result.result_value.remote_exception then
						debug ("rose_cs_future")
							output(Global.interpreted("($now) start restore result!%N"),Void)
						end
						Result.result_value.restore_result (api)
						debug ("rose_cs_future")
							output(Global.interpreted("($now) end restore result!%N"),Void)
						end
						restored := True
					else
						output(Global.interpreted("($now) Remote exception occured, request removed!%N"),Void)
						api.requests.remove_thread_safe (query_id)
						restored := True
					end
				end
			end
		end

	query_id: INTEGER

	freed: BOOLEAN
			-- Has the result been freed?

	restored: BOOLEAN
			-- Has restore_result already been called ?

feature -- Operations

	set_exception (a_tag: STRING) is
			-- Set the future variable to a Result that indicates that an exception during execution
		local
			l_result: G
		do
			create l_result.make_exception (query_id, a_tag)
			set (l_result)
		end

	set_query_id (a_id: like query_id) is
			-- Set `query_id'
		do
			query_id := a_id
		end

	free is
			-- Remove stored request, Result is no longer retrievable after `free' has been called
		require
			not_restored: not restored
		do
			api.requests.remove_thread_safe (query_id)
			freed := True
		end

feature {NONE} -- Implementation

	api: ROSE_CS_CLIENT_API

invariant
	api_not_void: api /= Void

end
