note

	description:

		"TCP Server for ROSE_CS"

	documentation: "http://prism/modules/wacko/wakka.php?wakka=PESServerdocumentation&v=1969"
	author: "Ueli Peter"
	last_editor: "$Author: rj $"
	date: "$Date: 2015/03/25 $"
	revision: "$Revision: #12 $"
	path: "$File: //rose/source/kernel/entities/cache_server/rose_cs_persistent_server.e $"

class	ROSE_CS_PERSISTENT_SERVER

inherit

	PERSISTENT_SERVER
		redefine
			server_command,
			create_new_client,
			connections,
			test_server,
			execute
		end

	THREAD_CONTROL

create

	make_cs

feature {NONE} -- Initialization

	make_cs (a_cs: ROSE_CS; a_port: INTEGER) is
			-- Listen on `a_port'.
		require
			a_cs_not_void: a_cs /= Void
		do
			(create {EXECUTION_ENVIRONMENT}).put("MAX","STID_CACHE_MULTIPLIER")
			make (a_port)
			max_to_poll := 1
			create poll.make_read_only
			create connections.make
			connections.compare_objects
			in.set_non_blocking
			is_sed_used := True
			in.use_sed
			cs := a_cs
		ensure
			cs_aliased: cs = a_cs
		rescue
			if connections /= Void then
				from
					connections.start
				until
					connections.after
				loop
					connections.item.outflow.close
					connections.forth
				end
			end
			cleanup
		end

feature -- Access

	test_server is
			-- do nothing
		do

		end

	server_command: ROSE_CS_PERSISTENT_SERVER_COMMAND is
			-- type of command
		do

		end

	server : SQL_DATA_SERVER is
			-- don't need a server
		do
		end

	create_new_client(a_client : PORT_SERVER_MESSAGE) is
			-- Handle a new client
		local
			l_new_connection: like server_command
			l_socket_name: STRING
		do
			test_server
			if max_to_poll <= outflow.descriptor then
				max_to_poll := outflow.descriptor + 1
			end
			create l_new_connection.make_command(outflow)
			l_new_connection.set_user_name (a_client.get_user_id)
			l_new_connection.set_port (a_client.get_port)
			l_new_connection.set_machine_name(a_client.get_machine)
			l_new_connection.set_client_pid(a_client.get_client_pid)
			l_new_connection.set_cs_and_launch_thread (cs)
			l_new_connection.execute (void)
			connections.extend (l_new_connection)
			poll.put_read_command (l_new_connection)
			l_socket_name := l_new_connection.outflow.debug_output

			output(Global.interpreted("($now) New Client (" +
									  "port: " + l_socket_name + ", " +
									  "user: " + a_client.get_user_id + ", " +
									  "machine: " + a_client.get_machine + ")%N"),Void)
		end

feature -- Basic operations

	execute is
			-- Execute transfer loop.
		do
			from
			until
				cs.get_state = {ROSE_CS}.state_shuting_down and then no_clients
			loop
				if cs.get_state /= {ROSE_CS}.state_shuting_down then
					receive
				end
				process_message
				close
				global.env.sleep (1)
			end
		end

	no_clients: BOOLEAN
			-- Are there no data clients connected?  (We ignore monitor clients).
		do
			from
				connections.start
			until
				connections.after or else connections.item.is_data_client
			loop
				connections.forth
			end

			Result := connections.after
		end

feature {NONE} -- Implementation

	cs: ROSE_CS
			-- Controller for cache server

	connections: LINKED_LIST [PERSISTENT_SERVER_COMMAND]
			-- Every active command

invariant

	cs_not_void: cs /= Void
	connections_not_void: connections /= Void
	no_void_connection: not connections.has (Void)

end

