note

	description:

		"Thread which has a cache and which handles the queries"

	documentation: "http://prism/modules/wacko/wakka.php?wakka=PESServerdocumentation&v=1969"
	author: "Ueli Peter"
	last_editor: "$Author: er $"
	date: "$Date: 2014/02/12 $"
	revision: "$Revision: #13 $"
	path: "$File: //rose/source/kernel/entities/cache_server/rose_cs_worker.e $"

class	ROSE_CS_WORKER

inherit

	THREAD
		rename
			make as make_thread
		end

	ROSE_SHARED_GLOBAL
		undefine
			default_create
		end

create

	make_cs

feature {NONE} -- Initialization

	make_cs (a_server: like server; a_refresh: BOOLEAN) is
			-- assigne `a_server' to `server'
		require
			valid_server: a_server /= void
		do
			needs_refresh := a_refresh
			server := a_server
			make_thread
		end


feature -- Access

	execute is
			-- executes this thread
		do
			initialize_cache
			needs_refresh := false
			run
		end

	initialize_cache is
			-- feature to initialize the cache before the server is up, should be redefined in descendants
		do
			if needs_refresh then
				refresh_cache
			end
		end

	run is
			-- start loop to get query from queue and execute it
		local
			l_query: TUPLE[query: ROSE_CS_QUERY; future: ROSE_CONCURRENT_FUTURE[ROSE_CS_RESULT]]
			l_tuple: TUPLE[bol: BOOLEAN; query:TUPLE[query: ROSE_CS_QUERY; future: ROSE_CONCURRENT_FUTURE[ROSE_CS_RESULT]]]
			l_result: ROSE_CS_RESULT
			i: INTEGER
		do
			server.state_transition
			from
				i := 1
			until
				server.get_state = {ROSE_CS}.state_shuting_down
			loop
				-- get query from queue non blocking
				l_tuple := server.input_queue.get_with_timeout (default_waiting_time)
				if l_tuple.bol then
					-- execute query and store result to future
					l_query := l_tuple.query
					output(Global.interpreted("($now) executing query: " + l_query.query.out + "!%N"), Void)
					l_result := l_query.query.execute_query (current)
					output(Global.interpreted("($now) query executed!%N"),Void)
					l_result.prepare_result
					l_query.future.set (l_result)
					output(Global.interpreted("($now) result prepared!%N"),Void)
					l_query := Void
					l_result := Void
					l_tuple := Void
				else
					memory.collect
				end
				if i >  default_cases_before_check then
					server.state_transition
					i := 1
				end
				i := i + 1
			end
			handle_remaining_queries
		rescue
			output(Global.interpreted("($now) Exception handled in ROSE_CS_WORKER.run!%N $1"), <<last_exception_explanation>>)
			retry
		end



	handle_remaining_queries is
			-- handle all message which are in queue
		local
			l_query: TUPLE[query: ROSE_CS_QUERY; future: ROSE_CONCURRENT_FUTURE[ROSE_CS_RESULT]]
			l_result: ROSE_CS_RESULT
		do
			from
			until
				server.input_queue.is_empty
			loop
				l_query := server.input_queue.get
				l_result := l_query.query.execute_query (current)
				l_result.prepare_result
				l_query.future.set (l_result)
			end
		rescue
			retry
		end

	shut_down is
			-- terminate server
		do
			server.set_state (server.state_shuting_down)
		end

feature {NONE} -- Implementation

	memory: MEMORY is
			-- MEMORY access
		once
			create Result
		end

	refresh_cache is
		-- refresh the cache
		do
		end

	needs_refresh: BOOLEAN

	server: ROSE_CS

	default_waiting_time: INTEGER is 100
			-- default waiting time in milliseconds
	default_cases_before_check: INTEGER is 400
			-- loop this times before checking the state

end
