indexing

	description:

		"a date set. `membership: ROSE_BITMAP' is the main container used. "

	author: "Mark Howard"
	last_editor: "$Author: c4 $"
	date: "$Date: 2013/06/11 $"
	revision: "$Revision: #38 $"
	path: "$File: //rose/source/kernel/infrastructure/datetime/rose_date_set.e $"

class ROSE_DATE_SET

inherit

	SET [ROSE_DATE]
		rename
			count as cardinality
		end

	LINEAR [ROSE_DATE]
		undefine
			changeable_comparison_criterion,
			has
		end

	ROSE_CONSTANTS
			-- To get `is_defined_double'
		export {NONE} all end

	MISMATCH_CORRECTOR
		redefine
			correct_mismatch
		end

create

	make,
	make_from_array,
	make_merged

feature {NONE} -- Initialization

	make (a_bitmap: ROSE_BITMAP_BASE; a_first_date: ROSE_DATE) is
			-- creates a rose date set from a given bitmap
		require
			a_first_date_not_void: a_first_date /= Void
			a_bitmap_not_void: a_bitmap /= Void
		do
			offset_date := a_first_date.twin
			membership := a_bitmap
			if membership.count > 0 then
				set_lower_bound (membership.ith_set_after (0, 1))
				set_upper_bound (membership.ith_set_before (count + 1, 1))
			else
				set_lower_bound (0)
				set_upper_bound (0)
			end
		ensure
			offset_date_set: offset_date ~ a_first_date
			offset_date_similar: offset_date /= a_first_date
			membership_set: membership = a_bitmap
		end

	make_from_array (a_array: ARRAY [ROSE_DATE]; a_max_date: ROSE_DATE) is
			-- creates a rose date set from a given date array
		require
			a_array_not_void: a_array /= Void
			no_void_date: not a_array.has (Void)
			some_data: a_max_date /= Void or not a_array.is_empty
			a_max_date_is_greater_than_all_dates: a_max_date /= Void implies a_array.for_all (agent a_max_date.is_greater_equal)
			a_max_date_big_enough: a_max_date /= Void implies a_max_date >= a_max_date.min_date
		local
			i: INTEGER
			l_max_date: ROSE_DATE
		do
			if a_max_date /= Void then
				l_max_date := a_max_date
			else
				from
					i := a_array.lower
					l_max_date := a_array.item (i)
					i := i+1
				until
					i > a_array.upper
				loop
					l_max_date := l_max_date.max (a_array.item (i))
					i := i + 1
				end
			end

			create offset_date.make_min_date
			create membership.make_empty (index_from_date (l_max_date))
			set_lower_bound (membership.count)
			set_upper_bound (0)
			a_array.do_all (agent put)
		end

	make_merged (a_arrays: ARRAY [ARRAY [ROSE_DATE]]) is
			-- each sub array are ordered
		require
			a_arrays_not_void: a_arrays /= Void
			a_arrays_not_empty: not a_arrays.is_empty
			no_void_array: not a_arrays.has (Void)
			no_void_item: not a_arrays.there_exists (agent {ARRAY [ROSE_DATE]}.has (Void))
			sub_arrays_sorted: a_arrays.for_all (agent is_in_ascending_order)
			all_dates_greater_than_min_date: a_arrays.for_all (agent {ARRAY [ROSE_DATE]}.for_all (agent (create {ROSE_DATE}.make_min_date).is_less_equal))
		local
			i: INTEGER
			l_max_date: ROSE_DATE
		do
			from
				i := a_arrays.lower
				create l_max_date.make_min_date
			until
				i > a_arrays.upper
			loop
				if not a_arrays.item (i).is_empty then
					l_max_date := l_max_date.max (a_arrays.item (i).item (a_arrays.item (i).upper))
				end
				i := i + 1
			end
			if l_max_date /= Void then
				make_from_arrays (a_arrays, l_max_date)
			end
		end

	make_from_arrays (a_arrays: ARRAY [ARRAY [ROSE_DATE]]; a_max_date: ROSE_DATE) is
			-- create date set from array of arrays. this simply unions all
			-- the given sets together.
		require
			a_arrays_not_void: a_arrays /= Void
			a_arrays_not_empty: not a_arrays.is_empty
			no_void_array: not a_arrays.has (Void)
			a_max_date_not_void: a_max_date /= Void
			no_void_item: not a_arrays.there_exists (agent {ARRAY [ROSE_DATE]}.has (Void))
			sub_arrays_sorted: a_arrays.for_all (agent is_in_ascending_order)
			all_dates_greater_than_min_date: a_arrays.for_all (agent {ARRAY [ROSE_DATE]}.for_all (agent (a_max_date.min_date).is_less_equal))
			all_dates_less_than_a_max_date: a_arrays.for_all (agent {ARRAY [ROSE_DATE]}.for_all (agent a_max_date.is_greater_equal))
		local
			l_set: like Current
			l_lower, i: INTEGER
		do
			from
				l_lower := a_arrays.lower
				make_from_array (a_arrays.item (l_lower), a_max_date)
				i := l_lower + 1
			until
				i > a_arrays.upper
			loop
				create l_set.make_from_array (a_arrays.item (i), a_max_date)
				membership.or_with (l_set.membership)
				make (membership, offset_date)
				i := i + 1
			end
		end

feature -- Set Operations

	and_with (a_other: like Current): like Current is
			-- Intersection of `current' with `other'
		local
			l_bitmap: ROSE_BITMAP_BASE
		do
			l_bitmap := membership.deep_twin
			if a_other /= Void then
				l_bitmap.and_with (a_other.membership)
			end
			create Result.make (l_bitmap, offset_date)
		ensure
			and_with_not_current: Result /= Current
			and_with_not_a_other: Result /= a_other
			and_with_not_void: Result /= Void
			same_offset: Result.offset_date ~ offset_date
			reduced_cardinality: Result.cardinality <= cardinality
		end

	or_with (a_other: like Current): like Current is
			-- Union of `current' with `other'.
		local
			l_bitmap: ROSE_BITMAP_BASE
		do
			l_bitmap := membership.deep_twin
			if a_other /= Void then
				l_bitmap.or_with (a_other.membership)
			end
			create Result.make (l_bitmap, offset_date)
		ensure
			or_with_not_current: Result /= Current
			or_with_not_a_other: Result /= a_other
			or_with_not_void: Result /= Void
			same_offset: Result.offset_date ~ offset_date
			increased_cardinality: Result.cardinality >= cardinality
			cardinality_small_enough: a_other /= Void implies (Result.cardinality <= (cardinality + a_other.cardinality))
		end

feature -- Access

	first: ROSE_DATE
			-- Date equivalent to `lower_bound'

	last: ROSE_DATE
			-- Date equivalent to `upper_bound'

	has (a_date: ROSE_DATE): BOOLEAN is
			-- Does the set contain `a_date'?
		local
			l_index: INTEGER
		do
			if a_date /= Void then
				l_index := index_from_date (a_date)
				if l_index >0 and l_index <= membership.count then
					Result := membership.get (l_index)
				end
			end
		end

	is_empty: BOOLEAN is
			-- Is the set empty?
		do
			Result := membership.cardinality = 0
		ensure then
			definition: Result = (membership.cardinality = 0)
		end

	extendible: BOOLEAN is True
			-- is the set extendible?

	prunable: BOOLEAN is True
			-- is the set prunable?

	count: INTEGER is
			-- Size of the bitmap containing the dates.
			-- This is NOT the number of dates in the set (see `cardinality')
		do
			Result := membership.count
		ensure
			definition: Result = membership.count
		end

	cardinality: INTEGER is
			-- Number of dates contained in the bitmap.
		do
			Result := membership.cardinality
		ensure then
			definition: Result = membership.cardinality
		end

	dates (a_start, a_end: ROSE_DATE): ROSE_LINEAR_ARRAY [ROSE_DATE] is
			-- List of dates equivalent to the set and between (inclusive of) 'a_start'/'a_end'.
			-- Either/both arguments may be Void.
			-- Note that in the past people would call "interval(a_start, a_end).dates".  Just use
			-- 'dates' now.
		require
			dates_in_order: (a_start /= Void and a_end /= Void) implies a_start <= a_end
		local
			i: INTEGER
			l_start: INTEGER
			l_end: INTEGER
			l_date: ROSE_DATE
		do
			if a_start=Void
			then
				l_start := lower_bound
			else
				l_start := index_from_date(a_start.max(first))
			end

			if a_end=Void
			then
				l_end := upper_bound
			else
				l_end := index_from_date(a_end.min(last))
			end

			create Result.make_empty
			from
				i := l_start
			until
				i > l_end
			loop
				if membership.get (i) then
					create l_date.make_by_days (days_from_index (i))
					Result.extend (l_date)
				end
				i := i + 1
			end
		ensure
			dates_not_void: Result /= Void
			no_void_date: not Result.has (Void)
			has_all_dates: Result.for_all (agent has)
			correct_number_of_dates: Result.count <= cardinality
			-- I changed this because the postcondition was failing when called
			--  on a monthend_dates.dates (or the new quarterend_dates.dates). But I expect
			-- this is the wrong thing to do, and that somwhere there is a bug in make_from_array
			-- so the cardinality is not set correctly. Mark, please comment in the review.
		end

feature -- Dates manipulation

	interval (a_start, a_end: ROSE_DATE): ROSE_DATE_SET
			-- Don't call this if you just want an array of dates.  Use 'dates' instead.
			-- Returns a date_set including days that are in the interval.
			-- This includes `a_start' and `a_end'.
		require
			a_start_not_void: a_start /= Void
			a_end_not_void: a_end /= Void
			dates_in_order: a_start <= a_end
		do
			create Result.make_from_array (dates(a_start, a_end), a_end.min (last))
		ensure
			interval_not_void: Result /= Void
			interval_first_ge_first: (Result.last < Result.last.min_date) or else Result.first >= first
			interval_last_le_last: Result.last <= last
		end

	ith_date_after (a_date: ROSE_DATE; a_days: INTEGER): ROSE_DATE is
			-- the date in the set that is a_days after a_date.
			-- returns Void if there is no valid date in the bitmap `membership'
		require
			a_date_not_void: a_date /= Void
			at_least_one_day: a_days /= 0
		local
			l_julian: INTEGER
			l_date_index: INTEGER
		do
			l_date_index := index_from_date (a_date)
			if l_date_index + a_days >= 0 and l_date_index + a_days <= membership.count then
					-- only return a value if the given parameters are valid
				l_julian := membership.ith_set_after (l_date_index, a_days)
				if l_julian > -1 then
					create Result.make_by_days (days_from_index (l_julian))
				end
			end
		ensure
			has_ith_date_after: Result /= Void implies has (Result)
			correct_date_interval: Result /= Void implies (Result.days - a_date.days).abs >= a_days.abs
			correct_side: Result /= Void implies ((a_days.sign = -1) = (Result < a_date))
		end

	ith_date_before (a_date: ROSE_DATE; a_days: INTEGER): ROSE_DATE is
			-- the date in the set that is a_days before a_date.
			-- returns the offset date of the set if `a_date' is smaller than lower_bound.
			-- FIXME: should it not rather return void in that case?
		require
			a_date_not_void: a_date /= Void
			at_least_one_day: a_days /= 0
		local
			l_julian: INTEGER
		do
			l_julian := index_from_date (a_date)
			if l_julian > upper_bound then
				l_julian := upper_bound + 1
			end
			if l_julian > a_days and l_julian <= membership.count+1 and l_julian >= lower_bound then
				l_julian := membership.ith_set_before (l_julian, a_days)
				if l_julian > -1 then
					create Result.make_by_days (days_from_index (l_julian))
				end
			end
			if Result = Void then
				Result := offset_date
			end
		ensure
			ith_date_before_not_void: Result /= Void
			has_ith_date_before: Result = offset_date or else has (Result)
			correct_date_interval: Result = offset_date or else (Result.days - a_date.days).abs >= a_days.abs
			correct_side: Result = offset_date or else ((a_days.sign = -1) = (Result > a_date))
		end

	at_or_after (a_date: ROSE_DATE): ROSE_DATE is
			-- returns the first day in the set that is at the given date or after.
			-- if no such date is in the set `at_or_after' will return void.
		require
			a_date_not_void: a_date /= Void
		do
			if has (a_date) then
				Result := a_date
			else
				Result := ith_date_after (a_date, 1)
			end
		ensure
			has_at_or_after: Result /= Void implies has (Result)
			at_or_after_is_at_or_after: Result /= Void implies (Result >= a_date)
		end

	at_or_before (a_date: ROSE_DATE): ROSE_DATE is
			-- returns the first day in the set that is at the given date or before.
			-- if no such date is in the set `at_or_before' will return `first'.
		require
			a_date_not_void: a_date /= Void
		do
			if has (a_date) then
				Result := a_date
			else
				Result := ith_date_before (a_date, 1)
			end
		ensure
			at_or_before_not_void: Result /= Void
			has_at_or_before: Result = offset_date or else has (Result)
			at_or_before_is_at_or_before: Result = offset_date or else (Result <= a_date)
		end

	time_series_of_agent_result (a_date: ROSE_DATE; a_count: INTEGER; a_function: FUNCTION [ANY, TUPLE [ROSE_DATE], DOUBLE]): VECTOR is
			-- return the time-series of data points returned by the agent, one per market day
			-- If 'a_count' is negative then use the PAST 'a_count' market days.
			-- If 'a_date' is a market day then that day is used,
			--	 otherwise the next or previous market day is used.
			-- If 'a_count' is 0 then the empty vector will be returned.
			-- If function returns invalid double, 0 will be assumed.
		require
			a_date_not_void: a_date /= Void
			a_function_not_void: a_function /= Void
		local
			i: INTEGER
			l_temp: DOUBLE
			l_date: ROSE_DATE
			l_dir: INTEGER
			l_args: TUPLE [ROSE_DATE]
			l_market_dates: ROSE_DATE_SET
		do
			create Result.make (a_count.abs)
			create l_args
			l_dir := a_count.sign
			l_market_dates := Current
			from
				l_date := a_date.twin
				i := a_count.abs
			until
				i = 0 or l_market_dates = Void
			loop
				if l_dir = 1 then
					l_date := l_market_dates.at_or_after (l_date)
				else
					l_date := l_market_dates.at_or_before (l_date)
				end
				l_args.put (l_date, 1)
				l_temp := a_function.item (l_args)
				if is_defined_double (l_temp) then
					Result.put (l_temp, i)
				end
				l_date.add_n_days (l_dir)
				i := i - 1
			end
		ensure
			time_series_of_agent_result_not_void: Result /= Void
			time_series_of_agent_result_same_count: Result.count = a_count.abs
			time_series_of_agent_result_all_defined: Result.for_all (agent is_defined_double)
		end

feature -- Iteration

	start is
			-- start iteration on `index'. sets index to the first contained date or to `after'.
		do
			from
				index := lower_bound
			until
				index > count or else membership.get (index)
			loop
				index := index + 1
			end
		end

	item: ROSE_DATE is
			-- returns the current item at `index'
		do
			if current_index /= index then
				create current_item.make_by_days (days_from_index (index))
				current_index := index
			end
			Result := current_item
		end

	forth is
			-- sets index to the next date in the set.
		do
			from
				index := index + 1
			until
				index > count or else membership.get (index)
			loop
				index := index + 1
			end
		end

	after: BOOLEAN is
			-- Is `index' out of bounds?
		do
			Result := index > count
		end

	finish is
			-- sets `index' to the end of the list.
		do
			index := membership.ith_set_before (count + 1, 1)
		end

	index: INTEGER
			-- cursor

feature -- Basic Operations

	wipe_out is
			-- clear the date set.
		do
			membership.clear_all
			set_lower_bound (membership.count)
			set_upper_bound (0)
		end

	extend (a_date: ROSE_DATE) is
			-- Equivalent to `put'
		do
			put (a_date)
		end

	prune (a_date: ROSE_DATE) is
			-- Removes `a_date' from the set.
		do
			if has (a_date) then
				membership.set (False, index_from_date (a_date))
				if lower_bound = index_from_date (a_date) then
					set_lower_bound (membership.ith_set_after (lower_bound, 1))
				end
				if upper_bound = index_from_date (a_date) then
					set_upper_bound (membership.ith_set_before (upper_bound, 1))
				end
			end
		end

	put (a_date: ROSE_DATE) is
			-- Inserts `a_date' into the set.
		local
			l_julian: INTEGER
		do
			l_julian := index_from_date (a_date)
			membership.set (True, l_julian)
			if l_julian < lower_bound then
				set_lower_bound (l_julian)
			end
			if l_julian > upper_bound then
				set_upper_bound (l_julian)
			end
		end

feature -- Agent iteration

	do_all_interval (a_action: PROCEDURE [ANY, TUPLE [ROSE_DATE, ROSE_DATE]]) is
			-- Apply 'a_action' to every successive pair of items.
		require
			a_action_not_void: a_action /= Void
		local
			t: TUPLE [ROSE_DATE, ROSE_DATE]
			l_with_cursor: CURSOR_STRUCTURE [ROSE_DATE]
			l_saved_cursor: CURSOR
		do
			l_with_cursor ?= Current
			if l_with_cursor /= Void then
				l_saved_cursor := l_with_cursor.cursor
			end
			create t
			from
				start
					-- Set start of first interval
				if not after then
					t.put (item, 1)
					forth
				end
			until
				after
			loop
				t.put (item, 2)
				a_action.call (t)
					-- End of previous interval becomes start of next one
				t.put (t.item (2), 1)
				forth
			end
			if l_with_cursor /= Void then
				l_with_cursor.go_to (l_saved_cursor)
			end
		end

feature -- Month-end and quarter-end dates in the context of the date set

	monthend_date_set: ROSE_DATE_SET is
			-- Date set consisting of only month ends
		local
			l_dates: ROSE_LINEAR_ARRAY [ROSE_DATE]
		do
			l_dates := monthend_dates
			if not l_dates.is_empty then
				create Result.make_from_array (l_dates, l_dates.last)
			else
				Result := Void
			end
		end

	monthend_dates: ROSE_LINEAR_ARRAY [ROSE_DATE] is
			-- Dates from all of `Current' restricted to month ends
		local
			l_dates: ROSE_LINEAR_ARRAY [ROSE_DATE]
		do
			create Result.make_empty
			l_dates := dates(Void, Void)
			l_dates.do_if (agent Result.extend, agent is_monthend)
			if not (Result.last.ccyymmdd = l_dates.last.ccyymmdd) then
				Result.extend (l_dates.last)
			end
		ensure
			monthend_dates_not_void: Result /= Void
		end

	quarterend_date_set: ROSE_DATE_SET is
			-- Date set consisting of only quarter ends
		local
			l_dates: ROSE_LINEAR_ARRAY [ROSE_DATE]
		do
			l_dates := quarterend_dates
			if not l_dates.is_empty then
				create Result.make_from_array (l_dates, l_dates.last)
			else
				Result := Void
			end
		end

	quarterend_dates: ROSE_LINEAR_ARRAY [ROSE_DATE] is
			-- Dates from all of `Current' restricted to quarter ends
		local
			l_dates: ROSE_LINEAR_ARRAY [ROSE_DATE]
		do
			create Result.make_empty
			l_dates := dates(Void, Void)
			l_dates.do_if (agent Result.extend, agent is_quarterend)
			if not (Result.last.ccyymmdd = l_dates.last.ccyymmdd) then
				Result.extend (l_dates.last)
			end
		ensure
			quarterend_dates_not_void: Result /= Void
		end

feature {ROSE_TWO_WAY_STRATEGY} -- Implementation

	position (a_date: ROSE_DATE): INTEGER is
			-- Index of a given date if the date
			-- is contained, otherwise 0.
		local
			l_end, i: INTEGER
		do
			if has (a_date) then
				from
					l_end := index_from_date (a_date)
					i := lower_bound
				until
					i > l_end
				loop
					if membership.get (i) then
						Result := Result + 1
					end
					i := i + 1
				end
			end
		ensure
			position_non_negative: Result >= 0
		end

feature {TS_TEST_CASE} -- Implementation

	is_monthend (a_date: ROSE_DATE): BOOLEAN is
			-- Is `a_date' the last day of the month in this set?
		require
			has_date: has (a_date)
		local
			l_next_date: ROSE_DATE
		do
			l_next_date := ith_date_after (a_date, 1)
			if l_next_date /= Void then
				Result := l_next_date.ccyymm > a_date.ccyymm
			else
				Result := True
			end
		end

	is_quarterend (a_date: ROSE_DATE): BOOLEAN is
			-- Is `a_date' the last day of the quarter in this set?
		require
			has_date: has (a_date)
		local
			l_next_date: ROSE_DATE
		do
			l_next_date := ith_date_after (a_date, 1)
			if l_next_date /= Void then
				Result := l_next_date.quarter > a_date.quarter
			else
				Result := True
			end
		end

	lower_bound: INTEGER
			-- lower bound of the set (as in indexes of dates)

	upper_bound: INTEGER
			-- upper bound of the set (as in indexes of dates)

	set_lower_bound (a_lower: INTEGER) is
			-- sets the lower bound
		do
			lower_bound := a_lower
			create first.make_by_days (days_from_index (lower_bound))
		ensure
			lower_bound_set: lower_bound = a_lower
			first_set: first /= Void
		end

	set_upper_bound (a_upper: INTEGER) is
			-- sets the upper bound
		do
			upper_bound := a_upper
			create last.make_by_days (days_from_index (upper_bound))
		ensure
			upper_bound_set: upper_bound = a_upper
			last_set: last /= Void
		end

	current_index: INTEGER
			-- cached index

	current_item: ROSE_DATE
			-- cached date

	days_from_index (a_index: INTEGER): INTEGER is
			-- convert `a_index' to number of days, using offset.
		do
			Result := a_index + offset - 1
		end

	index_from_date (a_date: ROSE_DATE): INTEGER is
			-- Number of days elapsed since `origin' minus the offset + 1.
		require
			a_date_not_void: a_date /= Void
		do
			Result := a_date.days - offset + 1
		end

	offset: INTEGER is
			-- the offset of the set (the index of the first day)
		do
			if offset_saved = 0 then
				offset_saved := offset_date.days
			end
			Result := offset_saved
		end

	offset_saved: INTEGER
			-- cached offset index

feature {TS_TEST_CASE, ROSE_DATE_SET} -- Implementation

	offset_date: ROSE_DATE
			-- cached offset date

feature {ROSE_DATE_SET, ZONE, RP_ZONE, REPORT_GRID} -- Implementation

	membership: ROSE_BITMAP_BASE
			-- bitmap containing market or bank dates
			-- Should be exported to ROSE_DATE_SET only, but has clients already

feature {NONE} --Mismatch handling
	correct_mismatch
			--  This handles the case where membership was previously of type ROSE_BITMAP and now is ROSE_BITMAP_BASE
		do
			membership ?= mismatch_information["membership"]
		end

feature -- Assertions helpers

	is_in_ascending_order (a_dates: ARRAY [ROSE_DATE]): BOOLEAN is
			-- Is `a_dates' sorted?
		require
			a_dates_not_void: a_dates /= Void
			no_void_date: not a_dates.has (Void)
		local
			i: INTEGER
		do
			from
				Result := True
				i := a_dates.lower+1
			until
				i > a_dates.upper or not Result
			loop
				Result := a_dates.item (i) >= a_dates.item (i-1)
				i := i+1
			end
		end

invariant
	membership_not_void: membership /= Void
end
