indexing
	description: "Dedicated network server"
	author: "$Author: rj $"
	last_editor: "$Author: rj $"
	date: "$Date: 2012/01/10 $"
	revision: "$Revision: #34 $"
 	path: "$File: //rose/source/kernel/infrastructure/network/rose_dedicated_network_server.e $"

deferred class
	ROSE_DEDICATED_NETWORK_SERVER

inherit
	NETWORK_SERVER
		rename
			error as socket_error
		redefine
			execute,
			receive,
			resend,
			cleanup,
			in,
			make
		end
	ROSE_SHARED_GLOBAL
	ROSE_SHARED_DATABASE_POOL
	MOVIE_CAMERA_AWARE
		export
			{NONE} all
		end

feature {NONE} -- Initialization

	make (a_port: INTEGER) is
		do
			global.ensure_storable_compatibility_with_56
			debug ("make_movies")
				create cam.make (generating_type + "/" + a_port.out)
			end
			Precursor (a_port)
		end

feature -- Basic operations

	execute is
		local
			l_poller: MEDIUM_POLLER
			l_retried: BOOLEAN
			l_ready: INTEGER
			l_collecting: BOOLEAN
		do
			l_collecting := Global.memory.collecting
			if not l_retried then
				Global.dependency_manager.register (create {ROSE_SYSTEM_DEPENDENCY_MINIMAL})
				if error.signaled then
					error.acknowledge
				end
				if not playback_movie then
					-- the command (LISTEN_POLL_COMMAND) is not actually used here, but is required by
					-- medium_poller in order to include the associated file handle in the select.
					create l_poller.make_read_only
					l_poller.put_read_command (create {LISTEN_POLL_COMMAND}.make (in))
				end
				from
					l_ready := initial_serve (l_poller)
				until
					done
				loop
					receive
					process_message
					respond
				end
			end
		rescue
			if exceptions.original_exception = exceptions.Runtime_io_exception or
			   exceptions.original_exception = exceptions.io_exception or
			   exceptions.original_exception = exceptions.Retrieve_exception or
			   (cam/=Void and then cam.exception_was_raised)
			then
				print ("Client disconnected (" + exceptions.original_exception.out + ").  Exiting.%N")
				l_retried := True
				on_error
				if l_collecting then
					Global.memory.collection_on
				else
					Global.memory.collection_off
				end
				retry
			end

			if l_collecting then
				Global.memory.collection_on
			else
				Global.memory.collection_off
			end
			print (exceptions.exception_trace)
			print ("Rescuing...%N")
		end

	initial_serve (a_poller: MEDIUM_POLLER): INTEGER is
			-- Wait for the first connection and then start serving or else timeout and shutdown
		local
			l_pair: PAIR[INTEGER, like outflow]
		do
			-- Time out if you don't get your first request within the limit
			if timeout = 0 then
				timeout := 120
			end
			if playback_movie then
				l_pair ?= cam.next_frame ("initial_serve").frame_data
				cam.compare_not_needed
				Result := l_pair.first
				--				outflow := l_pair.second
			else
				Result := a_poller.medium_select (in.descriptor + 1, timeout * 1000)
			end

			debug ("make_movies")
				if record_movie then
					create l_pair.make (Result, Void)
					cam.record_movie_frame ("initial_serve", l_pair)
				end
			end
			if Result > 0 then
				in.accept
				outflow ?= in.accepted
				in.close
				--set_delay (outflow)
				outflow.set_nodelay
			else
				exceptions.raise ("TIMEOUT")
			end
		end

	receive is
			-- Receive activity of server
		do
			received ?= outflow.retrieved
		end

	resend (a_msg: ANY) is
			-- Although not required, it is assumed that the client sent an object and the
			-- server (this class) fills in the missing portions and 'resend's it back to
			-- to the client.
		do
			outflow.independent_store (a_msg)
		rescue
			outflow_has_failed := True
			-- There is a bug in the Eiffel run-time (as of rev 83087 or 6.6.8.3087).
			-- If a socket is broken while 'a_msg' is being sent, some objects
			-- may remain marked with EO_STORE.  The next time this object is
			-- sent, the client will receive only a portion of it.  This 'rescue'
			-- should be removed when ISE fixes the bug.  RJ
			if outflow/=Void and then outflow.storable_type=outflow.storable_storable
			then
				-- The bug only happens when an exception happens during 'independent_store'.
				mark_unmark(a_msg)	-- Notice that we don't 'retry'.  This is just for clean up.
			end
		end

	mark_unmark(a_object: ANY)
		-- There is a bug in the Eiffel run-time (as of rev 83087 or 6.6.8.3087).
		-- If a socket is broken while 'a_msg' is being sent, some objects
		-- may remain marked with EO_STORE.  The next time this object is
		-- sent, the client will receive only a portion of it.  This
		-- should be removed when ISE fixes the bug.  RJ
	local
		l_stream: STREAM
	do
		create l_stream.make
		l_stream.independent_store(a_object)	-- This will remark the object and the unmark it.  The storable is unusable.
--		print("EO_STORE bug was found and corrected.%N")
	end

	use_storable is
			-- Use classic as the storable mechanism for communication with the client
		do
			outflow.use_storable
		end

	use_sed is
			-- Use sed as the storable mechanism for communication with the client
		do
			outflow.use_sed
		end

	resource_message: ARRAY[STRING]
	client_message: STRING
	user_id: STRING
	machine: STRING is
			-- What machine are we running on?
		once
			Result := Global.app.host_name
		end

	process_potential_messages_from_client is
		local
			l_logical, l_resource: STRING
			l_db_definition: ROSE_DATABASE_DEFINITION
		do
			resource_message ?= received
			if resource_message /= Void then
				if resource_message.count /= 2 then
					exceptions.raise ("Incorrect number of resource definition parameters")
				end
				l_logical ?= resource_message.item (1)
				if l_logical = Void then
					exceptions.raise ("Void Logical Name")
				end
				l_logical.to_lower
				l_resource ?= resource_message.item (2)
				if l_resource = Void then
					exceptions.raise ("Void Resource Name")
				end
				l_resource.to_lower
				create l_db_definition.make_from_path ("client_override",l_logical,"sql:" + l_resource)
				pool.extend (l_db_definition)
			else
				client_message ?= received
			end
		end

	exceptions: EXCEPTIONS is
		once
			Result := error.exceptions
		end

	on_error is
		do
			close
		end

	set_delay (a_socket: NETWORK_STREAM_SOCKET) is
		local
			l_retry: BOOLEAN
		do
			if not l_retry then
				a_socket.set_delay		-- A real speed up.
			end
		rescue
			if exceptions.exception = exceptions.Io_exception then
				-- There is a bug in the VMS EIFNET library that causes this to fail.
				-- It doesn't make things faster on VMS anyway.
				l_retry := True
				retry
			end
		end

	cleanup is
		do
			if in /= Void and then in.exists then
				Precursor
			end
		end

	timeout: INTEGER

	set_timeout (a_timeout: INTEGER) is
		do
			timeout := a_timeout
		end

feature {NONE} -- Implementation

	done: BOOLEAN
			-- Should we stop processing
	in: ROSE_NETWORK_STREAM_SOCKET

	handle_project_instance_request (a_args: STRING) is
		local
			l_command: RCMD_ABOUT_PROJECT_INSTANCE
			p: PAIR [STRING,STRING]
			l_path: STRING
		do
			if a_args /= Void then
				p := Global.formatter.split_on_first (a_args,"=","=")
				if p.second /= Void then
					p.second.prune_all ('"')
					l_path := p.second
				end
			end
			if l_path = Void then
				l_path := ""
			end
			create l_command
			l_command.execute ("")
			error.flush_and_acknowledge
		end

	outflow_has_failed: BOOLEAN		-- Servers like to notify their client when an error happens.  But if 'outflow' is the cause of the error...

end
