indexing
	description: "Dialog with OK/Cancel buttons at bottom left"
	notice: "[
		----------------------------------------------------------
		| |icon| title                                        _-X|
		----------------------------------------------------------
		| main_box                                               |
		|                                                        |
		|                                                        |
		----------------------------------------------------------
		| control_box             | ok_button | | cancel_button ||
		----------------------------------------------------------
	]"
	author: "Zoran Simic"
	date: "$Date: 2009/12/17 $"
	revision: "$Revision: #10 $"

class
	ROSE_DIALOG

inherit
	EV_DIALOG
		export
			{ANY} close_request_actions
		redefine
			initialize
		end
	ROSE_SHARED_GLOBAL_GUI
		export
			{NONE} all
			{ANY} gui
		undefine
			default_create, copy, is_equal
		end
	DEBUG_OUTPUT
		undefine
			default_create, copy, is_equal
		end

feature {NONE} -- Initialization

	initialize is
			-- Called by 'default_create'.
		local
			l_box, l_hbox: EV_BOX
		do
			Precursor
			l_box := gui.factory.new_vbox (Current, 2, 4)
			main_box := gui.factory.new_vbox (l_box, 0, -4)
			l_hbox := gui.factory.new_hbox (l_box, 0, 4)
			control_box := gui.factory.new_hbox (l_hbox, 0, -4)
			button_separator := gui.factory.new_label (l_hbox, Void, 0)
			-- Buttons
			button_box := gui.factory.new_hbox (l_hbox, 0, 4)
			ok_button := gui.factory.new_button (button_box, "OK", agent on_ok, 60)
			ok_button.font.set_weight (gui.rep.fonts.Weight_bold)
			ok_button.set_tooltip ("You may also hit ENTER to push this button")
			close_button := gui.factory.new_button (button_box, "Cancel", agent on_close, 60)
			close_button.set_tooltip ("You may also hit ESC to push this button")
			set_default_push_button (ok_button)
			set_default_cancel_button (close_button)
			show_actions.extend (agent on_show_dialog)
			-- Defaults
			setup_main_box
			if not control_box.is_empty then
				button_separator.set_minimum_width (1)
				l_hbox.disable_item_expand (button_separator)
			end
		ensure then
			main_box_created: main_box /= Void
			control_box_created: control_box /= Void
			button_box_created: button_box /= Void
		end

	setup_main_box is
			-- Set up the layout of this dialog
			-- Redefine this to compose your dialog instead of 'initialize'
			-- Called by 'initialize'
		do
		end

feature -- Access

	main_box, control_box, button_box: EV_BOX
			-- Boxes composing this dialog

	ok_button, close_button: EV_BUTTON
			-- OK/Close buttons

	is_ok_pressed: BOOLEAN
			-- Was OK button pushed during last 'show_modal'

	debug_output: STRING is
			-- String that should be displayed in debugger to represent `Current'.
		do
			Result := title
			if Result /= Void and then not Result.is_empty then
				Result := Result.twin
			else
				Result := generating_type
			end
		end

feature -- Basic operations

	show_modal (a_parent: EV_WIDGET) is
			-- Show modally to 'a_widget'.
		require
			parent_ok: gui.parent_window (a_parent) /= Void
		local
			l_parent: EV_WINDOW
		do
			is_ok_pressed := False
			l_parent := gui.parent_window (a_parent)
			show_modal_to_window (l_parent)
		end

	show_relative (a_parent: EV_WIDGET) is
			-- Show modally to 'a_widget'.
		local
			l_parent: EV_WINDOW
		do
			is_ok_pressed := False
			l_parent := gui.parent_window (a_parent)
			if l_parent /= Void then
				show_relative_to_window (l_parent)
			else
				show
			end
		end

	set_icon (a_icon: ANY) is
			-- Set dialog's icon (in the title bar).
		local
			l_pixmap: EV_PIXMAP
			l_name: STRING
			retried: BOOLEAN
		do
			if not retried then
				l_pixmap ?= a_icon
				if l_pixmap = Void then
					l_name ?= a_icon
					if l_name /= Void then
						l_pixmap := gui.factory.new_pixmap_from_file (l_name)
					end
				end
				if l_pixmap /= Void then
					set_icon_pixmap (l_pixmap)
				end
			end
		rescue
			if not retried then
				retried := True
				retry
			end
		end

	show_ok_button (a_on: BOOLEAN) is
			-- Show/hide OK button
		do
			if a_on then
				ok_button.show
				ok_button.enable_sensitive
				close_button.set_text ("Cancel")
			else
				ok_button.hide
				ok_button.disable_sensitive
				close_button.set_text ("Close")
			end
		end

feature {NONE} -- Implementation

	button_separator: EV_LABEL
			-- Separator between 'control_box' and 'button_box'

	output_in (a_field: EV_TEXTABLE; a_object: ANY) is
			-- Output 'a_string' to 'a_field' (allows Void for 'a_string', which Vision2 doesn't...)
		require
			field_ok: a_field /= Void
		local
			l_text: STRING
		do
			if a_object /= Void then
				l_text := a_object.out
			end
			if l_text = Void then
				a_field.remove_text
			else
				a_field.set_text (l_text)
			end
		end

	on_ok is
			-- Called when 'ok_button' is pressed.
		do
			if ok_button.is_sensitive then
				is_ok_pressed := True
				hide
			end
		ensure
			ok_pressed: ok_button.is_sensitive implies is_ok_pressed
		end

	on_close is
			-- Called when 'close_button' is pressed.
		do
			is_ok_pressed := False
			if close_button.is_sensitive then
				hide
			end
		ensure
			not_ok_pressed: not is_ok_pressed
		end

	on_show_dialog is
			-- Called when the dialog is actually shown.
		do
			if button_box.is_displayed then
				button_box.set_focus
			end
		end

end
