indexing
	description: "ROSE BACKGROUND context"
	author: "David Stevens"
	date: "$Date: 2009/05/06 $"
	revision: "$Revision: #12 $"
	
class
	
	ROSE_ENTITY_BACKGROUND_CONTEXT
	
inherit
	ROSE_CONTEXT
		redefine
			set_parameters
		end
	
create {ROSE_ENTITY_BACKGROUND_AWARE}
	make
	
feature-- creation
	
   set_parameters is
   		do
   			has_rose_fg_handles := False
   			has_rose_bg_handles := True
   			server := servers.rose_entity_background_server
   			name := "ROSE ENTITY BACKGROUND"
   		end
   		
feature -- Intra-Context Entity Mapping

	add_entity_mapping (rose_entity: like entity_template; supplier_entity: ENTITY) is
			-- Create mapping between ROSE and Supplier Entity
		require
			valid_args: rose_entity /= void and supplier_entity /= void
			valid_rose_entity: rose_entity.context = Current
			valid_category: rose_entity.category_id = supplier_entity.category_id
			valid_start_end_dates: rose_entity.start_date <= supplier_entity.end_date 
									and rose_entity.end_date >= supplier_entity.start_date
		local
			l_map_row: ENTITY_ID_MAP_ROW
		do
			create l_map_row
			l_map_row.set_category_id (rose_entity.category_id)
			l_map_row.set_end_date (create {ROSE_DATE}.make_by_date (
											rose_entity.end_date.max (supplier_entity.end_date)))
			l_map_row.set_entity_id (rose_entity.handle)
			l_map_row.set_start_date (create {ROSE_DATE}.make_by_date (
											rose_entity.start_date.min (supplier_entity.start_date)))
			l_map_row.set_supplier_entity_id (supplier_entity.handle)
			l_map_row.set_supplier_product_id (supplier_entity.supplier_id)
			l_map_row.set_update_date (create {ROSE_DATE}.make_now)

			entity_map_insert_predicate.execute (l_map_row, Void)
			if not entity_map_insert_predicate.success then 
				fail ("add_entity_mapping", "insert failed", Void) 
			end
		end
		
	remove_entity_mapping (rose_entity: like entity_template; supplier_entity: ENTITY) is
			-- Remove mapping between ROSE and Supplier Entity
		require
			valid_args: rose_entity /= void and supplier_entity /= void
			valid_rose_entity: rose_entity.context = Current
			valid_category: rose_entity.category_id = supplier_entity.category_id
		do
			entity_map_delete_predicate.execute (Void, <<
					rose_entity.category_id,
					rose_entity.handle,
					supplier_entity.handle,
					supplier_entity.supplier_id
				>>)
			if not entity_map_delete_predicate.success then 
				fail ("remove_entity_mapping", "delete failed", Void) 
			end
		end
		
	remove_all_entity_mappings_by_rose_entity (rose_entity: like entity_template) is
			-- Create all mappings for ROSE Entity
		require
			valid_args: rose_entity /= void
			valid_rose_entity: rose_entity.context = Current
		do
			entity_map_delete_by_rose_predicate.execute (Void, <<
					rose_entity.category_id,
					rose_entity.handle
				>>)
			if not entity_map_delete_by_rose_predicate.success then 
				fail ("remove_all_entity_mappings_by_rose_entity", "delete failed", Void) 
			end
		end
		
	remove_all_entity_mappings_by_supplier_entity (supplier_entity: ENTITY) is
			-- Remove all mappings for Supplier Entity
		require
			valid_args: supplier_entity /= void
		do
			entity_map_delete_by_supplier_predicate.execute (Void, <<
					supplier_entity.category_id, -- this is possibly a bug; table expects this column to be the rose entity's category (ND)
					supplier_entity.handle,
					supplier_entity.supplier_id
				>>)
			if not entity_map_delete_by_supplier_predicate.success then 
				fail ("remove_all_entity_mappings_by_supplier_entity", "delete failed", Void) 
			end
		end
	
	remove_all_entity_mappings_by_supplier_entity_and_rose_category (rose_entity: like entity_template; supplier_entity: ENTITY) is
			-- Remove all mappings for Supplier Entity that are mapped to a rose category
		require
			valid_args: rose_entity /= void and supplier_entity /= void
			valid_rose_entity: rose_entity.context = Current
		do
			entity_map_delete_by_supplier_and_rose_category_predicate.execute (Void, <<
					rose_entity.category_id,
					supplier_entity.supplier_id,
					supplier_entity.handle
				>>)
			if not entity_map_delete_by_supplier_and_rose_category_predicate.success then 
				fail ("remove_all_entity_mappings_by_supplier_entity_and_rose_category", "delete failed", Void) 
			end
		end
		
	update_foreground_entity (a_entity : ENTITY) is
			--attributes and relationships histories in foreground are REPLACED by background versions
		do
			if a_entity.context = Current and then a_entity.most_recent_attribute_at (brim_code_type_id, create {ROSE_DATE}.make_today) /= Void then
				update_entity_attribute_single_entity (a_entity)
				update_entity_relationship_single_entity (a_entity)
				update_entity_id_single_entity (a_entity)
				update_entity_id_map_single_entity (a_entity)
			else
			end
			servers.rose_entity_foreground_server.commit
		end
		
feature {NONE} -- Implementation

   	entity_map_insert_predicate: EXECUTION_PREDICATE is
			-- Insert into entity id map
      	once
			create Result.make (server, 
					"INSERT INTO entity_id_map "
   					+ "VALUES (:category_id, :entity_id, :supplier_product_id, :supplier_entity_id, "
   					+ 			":start_date, :end_date, :update_date)"
				)
			if not Result.success then 
				fail ("entity_map_insert_predicate", "make failed", Void) 
			end
      	end

   	entity_map_delete_predicate: EXECUTION_PREDICATE is
      	once
			create Result.make (server, 
					"DELETE FROM entity_id_map "
   					+ "WHERE category_id=? AND entity_id=? AND supplier_product_id=? AND supplier_entity_id=?"
				)
			if not Result.success then 
				fail ("entity_map_delete_predicate", "make failed", Void) 
			end
      	end

   	entity_map_delete_by_rose_predicate: EXECUTION_PREDICATE is
      	once
			create Result.make (server, 
					"DELETE FROM entity_id_map "
   					+ "WHERE category_id=? AND entity_id=?"
				)
			if not Result.success then 
				fail ("entity_map_delete_by_rose_predicate", "make failed", Void) 
			end
      	end

   	entity_map_delete_by_supplier_predicate: EXECUTION_PREDICATE is
      	once
			create Result.make (server, 
					"DELETE FROM entity_id_map "
   					+ "WHERE category_id=? AND supplier_product_id=? AND supplier_entity_id=?"
				)
			if not Result.success then 
				fail ("entity_map_delete_predicate", "make failed", Void) 
			end
      	end
    
    entity_map_delete_by_supplier_and_rose_category_predicate: EXECUTION_PREDICATE is
      	once
			create Result.make (server, 
					"DELETE FROM entity_id_map "
   					+ "WHERE category_id=? AND supplier_product_id=? AND supplier_entity_id=?"
				)
			if not Result.success then 
				fail ("entity_map_delete_predicate", "make failed", Void) 
			end
      	end
      	
	update_entity_attribute_single_entity (a_entity : ENTITY) is	
		local
			l_array : ROSE_LINEAR_ARRAY [ENTITY_ATTRIBUTE_RECORD]
			l_record : ENTITY_ATTRIBUTE_RECORD
			l_predicate : EXECUTION_PREDICATE
		do
			entity_attribute_single_entity_history_predicate.retrieve (Void, <<a_entity.category_id, a_entity.handle>>)
			if not entity_attribute_single_entity_history_predicate.success then
				fail ("update_entity_attribute_single_entity", "background entity_attribute selection failed for entity: $1", <<a_entity.handle>>)	
			end
			l_array := entity_attribute_single_entity_history_predicate.array
			if l_array /= Void and then not l_array.is_empty then
				create l_predicate.make (servers.rose_entity_foreground_server, "delete from entity_attribute where category_id=? and entity_id=?")
				l_predicate.execute (Void, <<a_entity.category_id, a_entity.handle>>)
				if not l_predicate.success then
					fail ("update_entity_attribute_single_entity", "deletion of foreground history for entity_id: $1 failed%N", <<a_entity.handle>>)	
				end
				create l_predicate.make (servers.rose_entity_foreground_server, "insert into entity_attribute (category_id,entity_id,attribute_type_id,start_date,end_date,attribute_value,update_date) values (?,?,?,?,?,?,?)")
				from l_array.start until l_array.off loop
					l_record := l_array.current_item
					l_predicate.execute (Void,<<l_record.category_id,l_record.entity_id,l_record.attribute_type_id,l_record.start_date,l_record.end_date,l_record.attribute_value,l_record.update_date>>)
					if not l_predicate.success then
						fail ("update_entity_attribute_single_entity", "insertion of foreground history for entity_id: $1 failed%N", <<a_entity.handle>>)	
					end									
					l_array.forth
				end
			end
		end
		
	update_entity_relationship_single_entity (a_entity : ENTITY) is	
		local
			l_array : ROSE_LINEAR_ARRAY [ENTITY_RELATIONSHIP_ROW]
			l_record : ENTITY_RELATIONSHIP_ROW
			l_predicate : EXECUTION_PREDICATE
		do
			entity_relationship_single_entity_history_predicate.retrieve (Void, <<a_entity.handle>>)
			if not entity_relationship_single_entity_history_predicate.success then
				fail ("update_entity_relationship_single_entity", "background entity_relationship selection failed for entity: $1", <<a_entity.handle>>)	
			end
			l_array := entity_relationship_single_entity_history_predicate.array
			if l_array /= Void and then not l_array.is_empty then
				create l_predicate.make (servers.rose_entity_foreground_server, "delete from entity_relationship where entity_1_id=?")
				l_predicate.execute (Void, <<a_entity.handle>>)
				if not l_predicate.success then
					fail ("update_entity_relationship_single_entity", "deletion of foreground history for entity_id: $1 failed%N", <<a_entity.handle>>)	
				end
				create l_predicate.make (servers.rose_entity_foreground_server, "insert into entity_relationship (relationship_id,relationship_type_id,entity_1_id,entity_2_id,start_date,end_date,update_date,weight) values (?,?,?,?,?,?,?,?)")
				from l_array.start until l_array.off loop
					l_record := l_array.current_item
					l_predicate.execute (Void,<<l_record.relationship_id,l_record.relationship_type_id,l_record.entity_1_id,l_record.entity_2_id,l_record.start_date,l_record.end_date,l_record.update_date,l_record.weight>>)
					if not l_predicate.success then
						fail ("update_entity_relationship_single_entity", "insertion of foreground history for entity_id: $1 failed%N", <<a_entity.handle>>)	
					end									
					l_array.forth
				end
			end
		end
		
	update_entity_id_single_entity (a_entity : ENTITY) is	
		local
			l_array : ROSE_LINEAR_ARRAY [SUPPLIER_ENTITY_ID_ROW]
			l_record : SUPPLIER_ENTITY_ID_ROW
			l_predicate : EXECUTION_PREDICATE
		do
			entity_id_single_entity_history_predicate.retrieve (Void, <<a_entity.category_id,a_entity.handle>>)
			if not entity_id_single_entity_history_predicate.success then
				fail ("update_entity_id_single_entity", "background entity_id selection failed for entity: $1", <<a_entity.handle>>)	
			end
			l_array := entity_id_single_entity_history_predicate.array
			if l_array /= Void and then not l_array.is_empty then
				create l_predicate.make (servers.rose_entity_foreground_server, "delete from entity_id where category_id=? and entity_id=?")
				l_predicate.execute (Void, <<a_entity.category_id, a_entity.handle>>)
				if not l_predicate.success then
					fail ("update_entity_id_single_entity", "deletion of foreground history for entity_id: $1 failed%N", <<a_entity.handle>>)	
				end
				create l_predicate.make (servers.rose_entity_foreground_server, "insert into entity_id (category_id,entity_id,start_date,end_date,update_date) values (?,?,?,?,?)")
				from l_array.start until l_array.off loop
					l_record := l_array.current_item
					l_predicate.execute (Void,<<l_record.category_id,l_record.entity_id,l_record.start_date,l_record.end_date,l_record.update_date>>)
					if not l_predicate.success then
						fail ("update_entity_id_single_entity", "insertion of foreground history for entity_id: $1 failed%N", <<a_entity.handle>>)	
					end									
					l_array.forth
				end
			end
		end
		
	update_entity_id_map_single_entity (a_entity : ENTITY) is	
		local
			l_array : ROSE_LINEAR_ARRAY [ENTITY_ID_MAP_ROW]
			l_record : ENTITY_ID_MAP_ROW
			l_predicate : EXECUTION_PREDICATE
		do
			entity_id_map_single_entity_history_predicate.retrieve (Void, <<a_entity.category_id,a_entity.handle>>)
			if not entity_id_map_single_entity_history_predicate.success then
				fail ("update_entity_id_map_single_entity", "background entity_id selection failed for entity: $1", <<a_entity.handle>>)	
			end
			l_array := entity_id_map_single_entity_history_predicate.array
			if l_array /= Void and then not l_array.is_empty then
				create l_predicate.make (servers.rose_entity_foreground_server, "delete from entity_id_map where category_id=? and entity_id=?")
				l_predicate.execute (Void, <<a_entity.category_id, a_entity.handle>>)
				if not l_predicate.success then
					fail ("update_entity_id_map_single_entity", "deletion of foreground history for entity_id: $1 failed%N", <<a_entity.handle>>)	
				end
				create l_predicate.make (servers.rose_entity_foreground_server, "insert into entity_id_map (category_id,entity_id,supplier_product_id,supplier_entity_id,start_date,end_date,update_date) values (?,?,?,?,?,?,?)")
				from l_array.start until l_array.off loop
					l_record := l_array.current_item
					l_predicate.execute (Void,<<l_record.category_id,l_record.entity_id,l_record.supplier_product_id,l_record.supplier_entity_id,l_record.start_date,l_record.end_date,l_record.update_date>>)
					if not l_predicate.success then
						fail ("update_entity_id_map_single_entity", "insertion of foreground history for entity_id: $1 failed%N", <<a_entity.handle>>)	
					end									
					l_array.forth
				end
			end
		end
end 
