note

	description:

		"entity date pair target for expressions"

	author: "Mark Howard"
	last_editor: "$Author: mk $"
	date: "$Date: 2012/02/22 $"
	revision: "$Revision: #51 $"
	path: "$File: //rose/source/kernel/targets/rose_entity_date.e $"

class
	ROSE_ENTITY_DATE [G -> ENTITY]

inherit
	ROSE_TARGET [G]
		rename
			item as entity
		redefine
			description_title,
			description,
			entity,
			new_variable,
			anchor,
			expression_target,
			expressions_anchor
		end
	ROSE_LAG_TARGET
		redefine
			lagged,
			conditional_lagged
		end
	DATABASE_SERVER
create
	make,
	make_ur

feature {NONE} -- Initialization

	make (a_entity: like entity;a_date: like date) is
			-- Initialize from `a_entity' and `a_date'.
		require
			a_entity_not_void: a_entity /= Void
			a_date_not_void: a_date /= void
		do
			date_saved := a_date
			entity := a_entity
		end

	make_ur (a_entity: like entity) is
			-- make with ur_entity
		require
			a_entity_non_void: a_entity /= void
			good_category_id: a_entity.context.handler (a_entity.category_id) /= Void
		do
			make(a_entity.handler.ur_entity,today)
		end

feature -- Access

	date: ROSE_DATE is
			-- Date to associate with `entity'
		do
			Result := date_saved
		end

	today: ROSE_DATE is
			-- Nearly arbitrary date (zone is unclear)
		once
			create Result.make_today
		ensure
			today_not_void: Result /= void
		end

	date_saved: ROSE_DATE
			-- Stored value for `date'

	entity: detachable G
			-- Comapny associated with `date'

	key : TUPLE[INTEGER_64,ROSE_DATE] is
			-- Entity uid, date pair
		do
			Result := [entity.uid,date]
		end

	entity_date_key: ENTITY_DATE_KEY
		-- enity_date_key
		do
			create Result.make( entity.uid, date, date)
		end

	context: like context_anchor is
			-- Context associated with `entity'
		do
			if entity /= Void then
				Result := entity.context
			end
		end

	context_name: STRING is
			-- Context name of `entity' (BUG)
		do
		end

	description_title: ARRAY[STRING] is
			-- Header for display
		once
			Result := <<"Name","Date">>
		ensure then
			description_title_not_void: Result /= Void
		end

	description: ARRAY[STRING] is
			-- Paired entity name and date
		do
			Result := <<entity.name_at(date),date.out>>
		ensure then
			description_not_void: Result /= Void
		end

	expression_tag: STRING is
			-- expression tag
		do
			if entity /= Void then
				Result := entity.expression_tag
			end
		end

	date_name: STRING is
			--"date"
		do
			Result := "date"
		end

	entity_name: STRING is
			--"entity"
		do
			Result := "entity"
		end

	new_variable (a_variable_description: VARIABLE_DESCRIPTION): like new_variable_template is
			--variable factory ... should be removed
		do
		end

	version(a_date : ROSE_DATE) : like Current
			--<Precursor>
		do
			Result := twin
			Result.set_date(a_date)
		end
feature -- Element modification

	set_date (a_date: ROSE_DATE) is
			-- Save `a_date'.
		do
			date_saved := a_date
		ensure
			date_set: date /= Void implies date.same_date (a_date)
		end

	set_entity(a_entity: G) is
			-- Set `entity' to `a_entity'.
		require
			a_entity_not_void: a_entity /= Void
		do
			entity := a_entity
		ensure
			entity_set: entity.uid = a_entity.uid
		end

feature -- Comparison

	compare_uid_date(a_1,a_2 : ROSE_ENTITY_DATE[G]) : BOOLEAN
			--compare by uid, then date
		do
			if a_1.entity.uid = a_2.entity.uid then
				Result := a_1.date < a_2.date
			else
				Result := a_1.entity.uid < a_2.entity.uid
			end
		end
	hash_code: INTEGER is
			--default hash code
		do
			Result := multi_key.hash_code
		end

	multi_key: MULTI_KEY is
			--default hashable version
		do
			if saved_multi_key = Void then
				create saved_multi_key.make(<<entity,date_saved>>)
			end
			Result := saved_multi_key
		ensure
			multi_key_not_void: Result /= Void
		end
	output_for_verification : STRING is
			--empty by default
		do
			Result := ""
		end
	saved_multi_key: like multi_key

	clear_saved_multi_key is
			-- clear saved multi key
		do
			saved_multi_key := Void
		ensure
			saved_multi_key_void: saved_multi_key = Void
		end

	order_by_date(a_sig1, a_sig2: like Current): BOOLEAN is
			-- Is a_sig1.date < a_sig2.date?
		require
			nonvoid: a_sig1 /= void and then a_sig2 /= void
		do
			Result := (a_sig1.date /= Void and a_sig2.date /= Void) and then a_sig1.date < a_sig2.date
		end

feature -- Lagged targets

	lagged (a_interval: ROSE_LAG_INTERVAL): like Current is
			-- lagged by an interval
		local
			l_date: ROSE_DATE
		do
			l_date := a_interval.lagged_date (Current, date)
			if l_date.julian_month > 0 then
				Result := duplicate
				Result.set_date (l_date)
				Result.clear_saved_multi_key
			end
		end

	conditional_lagged (a_interval: ROSE_LAG_INTERVAL; a_expression: ROSE_EXPRESSION [like Current]; a_agent: GENERIC_ACCESS_CONTROL_AGENT): like Current is
			--does not belong in this class...
		do
			Result := duplicate
			Result.set_date (a_interval.lagged_date (Current, date))
			Result.clear_saved_multi_key
		end

	duplicate: like Current is
			--this is used by lagged and conditional lagged
		do
			Result := twin
		end

feature -- Templates

	targets(a_targets: ARRAY[like Current];a_statement: STRING): ROSE_LINEAR_ARRAY[like Current] is
			-- templates
		do
		end

	anchor: ROSE_ENTITY_DATE[G] is
			-- template
		do
		end

	expression_target: ROSE_EXPRESSION[ROSE_ENTITY_DATE[G]] is
			-- template
		do
		end
	new_variable_template: ROSE_EXPRESSION[like Current] is
			-- template
		do
		end
	expressions_anchor: ENTITY_VARIABLES[ROSE_ENTITY_DATE[G]] is
			-- template
		do
		end

end











