note

	description:

		"Entity date expression"

	author: "Michael Zhang"
	last_editor: "$Author: mb $"
	date: "$Date: 2011/09/23 $"
	revision: "$Revision: #12 $"
	path: "$File: //rose/source/kernel/math/rose_entity_date_expression.e $"

deferred class ROSE_ENTITY_DATE_EXPRESSION [reference G -> ROSE_ENTITY_DATE [ENTITY]]

inherit

	ROSE_EXPRESSION [G]

	ROSE_CONSTANTS
		undefine
			default_create
		end

	ROSE_PERIODICITY_ID_USE
		undefine
			default_create
		end

feature -- Access

	double_series (a_targets: ARRAY [like expression_target]; a_agent: like access_agent_template): DOUBLE_TIME_SERIES is
			-- Series of observations for `a_targets'
		local
			i: INTEGER
			l_double: DOUBLE
			l_target: like expression_target
		do
			create Result.make_empty
			from
				i := a_targets.lower
			until
				i > a_targets.upper
			loop
				l_target := a_targets.item (i)
				l_double := generated_numeric_value (l_target, a_agent)
				Result.extend (l_double, l_target.date)
				i := i + 1
			end
		end

	lagged_series (a_target: like expression_target; a_lag_interval: ROSE_LAG_INTERVAL; a_length: INTEGER; a_agent: like access_agent_template): DOUBLE_TIME_SERIES is
			-- Series of lagged observations starting with `a_target'
		local
			l_targets: ROSE_LINEAR_ARRAY [like expression_target]
		do
			l_targets := a_target.trailing_targets (a_lag_interval, a_length)
			Result := double_series (l_targets, a_agent)
		end

	grow_rate (a_target: like expression_target; a_lag_interval: ROSE_LAG_INTERVAL; a_length, a_minimum_number_of_observation: INTEGER; a_is_percentage: BOOLEAN; a_agent: like access_agent_template): VECTOR is
			-- Growth rate vector
		require
			valid_lag_interval: a_lag_interval /= Void
			reasonable_observations: a_length >= a_minimum_number_of_observation and a_minimum_number_of_observation >= 0
		local
			l_values: VECTOR
			l_targets: ROSE_LINEAR_ARRAY [like expression_target]
		do
			l_targets := a_target.trailing_targets (a_lag_interval, a_length)
			l_values := generated_vector (l_targets, a_agent)
			Result := l_values.growth_rate (a_minimum_number_of_observation, a_is_percentage)
		end

	daily_interval (a_periods: INTEGER): ROSE_LAG_INTERVAL is
			-- Daily lag interval
		require
			valid_periods: a_periods >= 0
		do
			Result := lag_factory.daily_interval (a_periods)
		ensure
			valid_result: Result /= Void
		end

	monthly_interval (a_periods: INTEGER): ROSE_LAG_INTERVAL is
			-- Monthly lag interval
		require
			valid_periods: a_periods >= 0
		do
			Result := lag_factory.monthly_interval (a_periods)
		ensure
			valid_result: Result /= Void
		end

	annual_interval (a_periods: INTEGER): ROSE_LAG_INTERVAL is
			-- Annual lag interval
		require
			valid_periods: a_periods >= 0
		do
			Result := lag_factory.annual_interval (a_periods)
		ensure
			valid_result: Result /= Void
		end

feature {NONE} -- Implementation

	lag_factory: ROSE_LAG_INTERVAL_FACTORY is
			-- Factory for ROSE_LAG_INTERVAL objects
		once
			create Result
		ensure
			valid_result: Result /= Void
		end

end
