note

	description:

		"Provides facilities for displaying entity attributes for entities of a given category"

	author: "Mark Howard"
	last_editor: "$Author: bn $"
	date: "$Date: 2011/10/26 $"
	revision: "$Revision: #13 $"
	path: "$File: //rose/source/kernel/entities/rose_entity_display.e $"

deferred class
	ROSE_ENTITY_DISPLAY

inherit
	CONTEXT_ANCHOR

feature -- Access

	entity_strings (a_entity : ENTITY) : ROSE_LINEAR_ARRAY [STRING] is
			-- Identifier values of `a_entity' for identifiers marked as active
		require
			valid_entity : is_valid_entity (a_entity)
		local
			i : INTEGER
			l_identifier : ENTITY_IDENTIFIER
			l_key : STRING
		do
			if a_entity /= Void then
				create Result.make_empty
				if is_display_handle then
					Result.extend (a_entity.handle.out)
				end
				from i := 1 until i > identifier_count loop
					l_identifier := identifier_if_active (i)
					if l_identifier /= Void then
						l_key := l_identifier.dated_key (a_entity, restrict_date)
						if l_key = Void then
							l_key := k_void_string
						end
						Result.extend (l_key)
					end
					i := i + 1
				end
				if Result.is_empty then
					Result.extend (a_entity.descriptive_name)
				end
			end
		end

	entity_string_titles : ROSE_LINEAR_ARRAY [STRING] is
			-- Identifier names for identifiers marked as active
		local
			i : INTEGER
			l_identifier : ENTITY_IDENTIFIER
			l_display_handle : BOOLEAN
		do
			create Result.make_empty
			l_display_handle := is_display_handle
			if l_display_handle then
				Result.extend (k_handle_string)
			end
			from i := 1 until i > identifier_count loop
				l_identifier := identifier_if_active (i)
				if l_identifier /= Void then
					Result.extend (l_identifier.name)
				end
				i := i + 1
			end
			if Result.is_empty then
				Result.extend ("Descriptive Name")
			end
		ensure
			result_not_void : Result /= Void
		end

	category_handler : CATEGORY_HANDLER [like context_anchor, ENTITY] is
			-- Specifies category for all entity arguments used in `Current'
		deferred
		end

	is_display_handle : BOOLEAN is
			-- Should entity handle be displayed?
		deferred
		end

	identifier_if_active (i : INTEGER) : ENTITY_IDENTIFIER is
			-- Returns the i_th identifier if marked as active - else `Void'
		require
			index_in_range : i > 0 and then i <= identifier_count
		deferred
		end

	identifier (i : INTEGER) : ENTITY_IDENTIFIER is
			-- I_th identifier
		require
			index_in_range : i > 0 and then i <= identifier_count
		deferred
		end

	identifier_count : INTEGER is
			-- Count of identifiers
		deferred
		end

	active_identifier_count : INTEGER is
			-- Count of active (display requested) identifiers
		local
			i : INTEGER
		do
			if is_display_handle then
				Result := 1
			end
			from i := 1 until i > identifier_count loop
				if identifier_if_active (i) /= Void then
					Result := Result + 1
				end
				i := i + 1
			end
			Result := Result.max (1) -- If no identifiers are marked as active - descriptive name is active
		end

	i_th_active_identifier (a_i_th : INTEGER) : ENTITY_IDENTIFIER is
			-- I_th active identifier
		require
			a_i_th_in_range : a_i_th > 0 and then a_i_th <= active_identifier_count
		local
			i, l_n_th_active : INTEGER
			l_identifier : ENTITY_IDENTIFIER
		do
			from i := 1 until  i > identifier_count or Result /= Void loop
				l_identifier := identifier_if_active (i)
				if l_identifier /= Void then
					l_n_th_active := l_n_th_active + 1
					if l_n_th_active = a_i_th then
						Result := l_identifier
					end
				end
				i := i + 1
			end
		end

	restrict_date : ROSE_DATE is
			-- Default date computed from `start_date' and `end_date'
		do
			if end_date = Void then
				if start_date = Void then
					Result := today
				else
					Result := start_date
				end
			else
				Result := end_date
			end
		end

	today : ROSE_DATE is
			-- Today, duh
		once
			create Result.make_today
		end

	start_date : ROSE_DATE is
			-- Start date
		deferred
		end

	end_date : ROSE_DATE is
			-- End date
		deferred
		end

feature -- Basic Operations

	set_identifier_active (a_attribute_name : STRING; a_active : BOOLEAN) is
			-- Set active status for `a_attribute' to `a_active'
		require
			a_attribute_name_not_void : a_attribute_name /= Void
		deferred
		end

	inactivate_identifiers is
			-- Disable active for all identifiers
		deferred
		end

feature -- Assertion support

	is_valid_entity (a_entity : ENTITY) : BOOLEAN is
			-- Is `a_entity' valid for use with `Current'?
		do
			if category_handler /= Void then
				Result := (a_entity.category_id = category_handler.category_id) and then (a_entity.supplier_id = category_handler.context.supplier_id)
			end
		end

feature {NONE} -- Constants

	k_void_string : STRING is "Void"

	k_handle_string : STRING is "Handle"

	k_descriptive_name_string : STRING is "Descriptive Name"

end
