indexing
	
	description: "Entity History"

	author: "Mark Howard"
	last_editor: "$Author: er $"
	date: "$Date: 2013/06/18 $"
	revision: "$Revision: #19 $"
	path: "$File: //rose/source/kernel/entities/rose_entity_history.e $"
	storable_version: ""

class	ROSE_ENTITY_HISTORY [reference G->ENTITY]

inherit

	ROSE_INTERVAL_HISTORY [G, ENTITY_INTERVAL[G]]

create

	make_empty

feature -- Access
	primary_entity (a_date : ROSE_DATE) : G is
		-- Returns the entity with the max weight
		-- if G has an attribute "name" order them by "name"
		-- if not order by "handle"
		local
			l_max : ROSE_LINEAR_ARRAY[G]
		do
			l_max:=Current.primary_witnesses(a_date)
			if l_max.count>0 then
				if l_max.count>1 then
					from l_max.start until l_max.off
					loop
						if Result/=Void then
							if l_max.current_item.attribute_by_name ("name", a_date) /= Void then
								if l_max.current_item.attribute_by_name ("name", a_date) < Result.attribute_by_name ("name", a_date) then
									Result:=l_max.current_item
								end
							else
								if l_max.current_item.handle<Result.handle then
									Result:=l_max.current_item
								end
							end
						else
							Result:=l_max.current_item
						end
						l_max.forth
					end
				else
					Result:=l_max.item(1)
				end
			end
		end

	unique_witness (a_date: ROSE_DATE): G is
			-- Unique witness for a 1:1 relationship (as of a given date),
			-- or Void if no relationship.
			-- This is intended for cases where this is an invariant of the data, 
			-- so an exception is raised if it is broken (more than 1 witness).
		local
			l_witnesses: like witnesses_during
		do
			l_witnesses := witnesses_during (a_date, a_date)
			if l_witnesses.is_empty then
				check 
					return_void: Result = Void
					-- no item is OK
				end
			elseif l_witnesses.count = 1 then
				Result := l_witnesses.first
			else
				(create {EXCEPTIONS}).raise ("1:1 relationship has multiple witnesses")
			end
		ensure
			empty_case: witness (a_date) = Void implies Result = Void
		end

	distinct_witnesses_during(a_start,a_end : ROSE_DATE) : ROSE_HASHED_LINEAR_ARRAY[G] is
			-- remove duplicates
		local
			l_default : G
			i : INTEGER
			l_intervals : like intervals_during
		do
			create Result.make_empty
			l_intervals := intervals_during(a_start,a_end,l_default)
			from i := 1 until i > l_intervals.count loop
				if Result.new_index_of(l_intervals.item(i).entity) = 0 then
					Result.extend(l_intervals.item(i).entity)
				end
				i := i + 1
			end
		end

feature -- weights access
	weights(a_date : ROSE_DATE; a_to_decimal_flag : BOOLEAN) : ENTITY_VALUES[G] is
			-- weights
		require
			valid_date : a_date /= Void
		local
			l_interval : like witness_interval
			l_intervals : like witness_intervals
		do
			l_intervals := witness_intervals(a_date)
			if l_intervals /= Void then
				l_intervals.sort_by(Agent interval_order)
				create Result.make_empty
				from l_intervals.start until l_intervals.off loop
					l_interval := l_intervals.current_item
					if l_interval /= Void then
						if a_to_decimal_flag then
							Result.pair_extend(l_interval.entity, l_interval.weight / 100.)
						else
							Result.pair_extend(l_interval.entity, l_interval.weight)
						end
					end
					l_intervals.forth
				end
			end
		end

	weight_matrix(a_entities : ROSE_LINEAR_ARRAY[G]; a_dates : ARRAY[ROSE_DATE]) : DATED_MATRIX is
			-- weight matrix
		require
			valid_entities : a_entities /= Void
		local
			i : INTEGER
			l_date : ROSE_DATE
			l_vector : VECTOR
			l_weights : like weights
		do
			create Result.make(a_dates.count, a_entities.count)
			Result.set_dates (a_dates)
			from i := 1 until i > a_dates.count loop
				l_date := a_dates[i]
				l_weights := weights(l_date, True)
				if l_weights /= Void and then l_weights.count > 0 then
					l_vector := l_weights.vector_at (a_entities)
					l_vector.replace_null
					Result.put_row_vector (l_vector, i)
				end
				i := i + 1
			end
		end

	interval_order(a_interval_1, a_interval_2 : like witness_interval) : BOOLEAN is
			-- interval order
		do
			Result := (a_interval_1 /= Void and a_interval_2 /= Void) and then a_interval_1.weight > a_interval_2.weight
		end

end
