indexing

	description: "Map entities from an entity history"

	author: "Bruno Pradal"
	last_editor: "$Author: c4 $"
	date: "$Date: 2011/01/19 $"
	revision: "$Revision: #5 $"
	path: "$File: //rose/source/kernel/entities/rose_entity_history_mapping.e $"
	
class	ROSE_ENTITY_HISTORY_MAPPING [reference G->ENTITY, reference H->ENTITY]

create

	make

feature {NONE} -- Initialization

	make (a_mapping: HASH_TABLE [ROSE_ENTITY_HISTORY [H], G]) is
			-- Initialize.
		require
			a_mapping_not_void: a_mapping /= Void
		do
			mapping := a_mapping
		ensure
			mapping_aliased: mapping = a_mapping
		end

feature -- Basic operations	

	map_and_aggregate (a_history: ROSE_ENTITY_HISTORY [G]): ROSE_ENTITY_HISTORY[H] is
			-- Create the mapping and the aggregation of the data.
		require
			a_history_not_void: a_history /= Void
		local
			l_mapped_history: ROSE_ENTITY_HISTORY [H]
			l_entity: G
			l_current_interval: ENTITY_INTERVAL [G]
			l_mapped_interval: ENTITY_INTERVAL [H]
			l_mapped_entity: H
			l_mapping_history: ROSE_ENTITY_HISTORY [H]
			l_start, l_end: ROSE_DATE
			l_default: H
		do
			create l_mapped_history.make_empty
			from
				a_history.start
			until
				a_history.after
			loop
				l_current_interval := a_history.current_item
				l_entity := l_current_interval.entity
				mapping.search (l_entity)
				if mapping.found then
					l_mapping_history := mapping.found_item.intervals_during (l_current_interval.start_date, l_current_interval.end_date, l_default)
					from
						l_mapping_history.start
					until
						l_mapping_history.after
					loop
						l_mapped_entity := l_mapping_history.current_item.entity
						if l_mapping_history.current_item.brackets (l_current_interval.start_date) then
							l_start := l_current_interval.start_date
						else
							l_start := l_mapping_history.current_item.start_date
						end
						if l_mapping_history.current_item.brackets (l_current_interval.end_date) then
							l_end := l_current_interval.end_date
						else
							l_end := l_mapping_history.current_item.end_date
						end
						create l_mapped_interval.make (l_mapped_entity, l_start, l_end, l_current_interval.weight)
						l_mapped_history.add_interval (l_mapped_interval)
						l_mapping_history.forth
					end
				end
				a_history.forth
			end
			Result:=l_mapped_history.aggregate_contiguous
		end

	
feature {NONE} -- Implementation

	mapping: HASH_TABLE [ROSE_ENTITY_HISTORY [H], G]
			-- Index of histories by entity

invariant

	mapping_not_void: mapping /= Void

end
