note
	description: "Application environment view"
	author: "Zoran Simic"
	last_editor: "$Author: zs $"
	date: "$Date: 2011/04/19 $"
	revision: "$Revision: #12 $"
	path: "$File: //rose/source/kernel/infrastructure/framework/gui/rose_environment_view.e $"

class
	ROSE_ENVIRONMENT_VIEW

inherit
	ROSE_SHARED_GLOBAL_GUI

create
	make

feature {NONE} -- Initialization

	make (a_container: EV_CONTAINER)
			-- Create a view on app's environment, and put it in 'a_container'.
		local
			l_main_box, l_box: EV_BOX
			l_frame: EV_FRAME
			l_widget: EV_WIDGET
		do
			box := Gui.factory.new_vbox (a_container, 1, 4)
				--| Application variables
			l_frame := Gui.factory.new_frame (box, "Environment variables", 0, 0)
			l_main_box := Gui.factory.new_vbox (l_frame, 0, 2)
			var_grid := Gui.factory.new_grid (l_main_box, <<["Key", 200], ["Value", 460]>>, agent on_variable, 0, 0)
			l_box := Gui.factory.new_hbox (l_main_box, 0, 10)
			l_widget := Gui.factory.new_button (l_box, "Add", Void, 80)
			l_widget := Gui.factory.new_button (l_box, "Modify", Void, 80)
			l_widget := Gui.factory.new_label (l_box, Void, 15)
			browse_button := Gui.factory.new_button (l_box, "Browse", agent explore_selected, 80)
			browse_button.disable_sensitive
		end

feature -- Access

	box: EV_BOX
			-- Box holding the whole view

feature -- Basic operations

	refresh
			-- Refresh current view.
		do
			var_grid.clear
			env_vars.do_all (agent add_var_node (var_grid, ?))
		end

feature {NONE} -- Implementation

	browse_button: EV_BUTTON
	var_grid: RP_DATA_GRID[ANY]

	add_var_node (a_grid: like var_grid ; a_key: STRING)
			-- Add node to 'a_grid' representing environment variable 'a_key'
		local
			l_node: RP_GRID_NODE[ANY]
			l_pixmap: EV_PIXMAP
			l_value: STRING
		do
			l_value := Global.env.get (a_key)
			if l_value = Void then
				l_pixmap := pixmap_not_available
			elseif Global.is_directory_readable (l_value) then
				l_pixmap := pixmap_folder
			elseif Global.is_file_readable (l_value) then
				l_pixmap := pixmap_file
			else
				l_pixmap := pixmap_auto
			end
			l_node := Gui.factory.new_grid_node (l_pixmap, <<a_key, l_value>>, a_key)
			a_grid.extend (l_node)
		end

	on_variable (a_button: INTEGER; a_node: RP_GRID_NODE[ANY])
			-- Variable in 'a_node' has been double-clicked.
		do
			var_value_selected := Void
			if a_node /= Void then
				if a_button = -1 or else a_button = 1 then
					var_value_selected ?= a_node.data
				end
			end
			if var_value_selected /= Void then
				var_value_selected := Global.env.get (var_value_selected)
			end
			if browse_button.is_sensitive then
				if not Global.is_file_readable (var_value_selected) and then not Global.is_directory_readable (var_value_selected) then
					browse_button.disable_sensitive
				end
			else
				if Global.is_file_readable (var_value_selected) or else Global.is_directory_readable (var_value_selected) then
					browse_button.enable_sensitive
				end
			end
		end

	env_vars: SORTED_TWO_WAY_LIST [STRING]
			-- Selected environment variables to show
		once
			create Result.make
			Result.extend ("ROSE")
			Result.extend ("ROSE_PATH")
			Result.extend ("TESTENV")
			Result.extend ("PATH")
			Result.extend ("APPDATA")
		end

	pixmap_file: EV_PIXMAP is once Result := Gui.factory.new_pixmap ("cfg_leaf.ico") end
	pixmap_folder: EV_PIXMAP is once Result := Gui.factory.new_pixmap ("cfg_node.ico") end
	pixmap_auto: EV_PIXMAP is once Result := Gui.factory.new_pixmap ("cfg_app_vars.ico") end
	pixmap_not_available: EV_PIXMAP is once Result := Gui.factory.new_pixmap ("cfg_grey_folder.ico") end

	var_value_selected: STRING
	explore_selected
			-- Launch an explorer window on 'var_value_selected' if it points to a valid directory.
		do
			Global.explore (var_value_selected)
		end

	sorted (a_set: HASH_TABLE [ANY, STRING]): SORTED_TWO_WAY_LIST [STRING]
			-- Keys of 'a_set' sorted alphabetically, ascending order
		do
			if a_set /= Void then
				create Result.make
				from
					a_set.start
				until
					a_set.off
				loop
					Result.extend (a_set.key_for_iteration)
					a_set.forth
				end
			end
		end

end
