note

	description:

		"Equality functional for ROSE_EXPRESSIONs"

	author: "Mark Howard"
	last_editor: "$Author: mk $"
	date: "$Date: 2011/02/15 $"
	revision: "$Revision: #24 $"
	path: "$File: //rose/source/kernel/math/rose_equal_operator.e $"

class
	ROSE_EQUAL_OPERATOR[reference G]
inherit
	ROSE_RELATION_BINARY_OPERATOR[G]

create
	make

feature -- Access

	boolean_value (a_left,a_right : ROSE_EXPRESSION[G];a_g : G) : BOOLEAN is
			-- Boolean value
		do
			inspect a_left.type
			when Real_type then
				Result := a_left.real_value(a_g) = a_right.real_value(a_g)
			when Double_type then
				Result := a_left.double_value(a_g) = a_right.double_value(a_g)
			when Integer_type then
				Result := a_left.integer_value(a_g) = a_right.integer_value(a_g)
			when Boolean_type then
				Result := a_left.boolean_value(a_g) = a_right.boolean_value(a_g)
			when Character_type then
					 (create {EXCEPTIONS}).raise ("not implemented") --//\\--				Result := a_left.character_value(a_g) = a_right.character_value(a_g)
			when Date_type then
				Result := a_left.date_value(a_g).same_date(a_right.date_value(a_g))
			when String_type then
				Result := a_left.string_value(a_g) ~ a_right.string_value(a_g)
			else
			end
		end

feature -- Access
	computed_boolean(a_left,a_right : ROSE_EXPRESSION[G]; a_g : G; a_agent : like access_agent_template) : BOOLEAN is
			-- Computed boolean
		local
			l_left_date, l_right_date: ROSE_DATE
		do
			inspect a_left.type
			when Real_type then
				Result := a_left.computed_real(a_g, a_agent) = a_right.computed_real(a_g, a_agent)
			when Double_type then
				Result := a_left.computed_double(a_g, a_agent) = a_right.computed_double(a_g, a_agent)
			when Integer_type then
				Result := a_left.computed_integer(a_g, a_agent) = a_right.computed_integer(a_g, a_agent)
			when Boolean_type then
				Result := a_left.computed_boolean(a_g, a_agent) = a_right.computed_boolean(a_g, a_agent)
			when Date_type then
				l_left_date := a_left.computed_date (a_g, a_agent)
				l_right_date := a_right.computed_date (a_g, a_agent)
				Result := l_left_date.same_date(l_right_date)
			when String_type then
				Result := a_left.computed_string(a_g, a_agent) ~ a_right.computed_string(a_g, a_agent)
			else
			end
		end

	generated_boolean(a_left,a_right : ROSE_EXPRESSION[G]; a_g : G; a_agent : like access_agent_template) : BOOLEAN is
			-- Generated boolean
		do
			inspect a_left.type
			when Real_type then
				Result := a_left.generated_real(a_g, a_agent) = a_right.generated_real(a_g, a_agent)
			when Double_type then
				Result := a_left.generated_double(a_g, a_agent) = a_right.generated_double(a_g, a_agent)
			when Integer_type then
				Result := a_left.generated_integer(a_g, a_agent) = a_right.generated_integer(a_g, a_agent)
			when Date_type then
				Result := a_left.generated_date(a_g, a_agent).same_date(a_right.generated_date(a_g, a_agent))
			when String_type then
				Result := a_left.generated_string(a_g, a_agent) ~ a_right.generated_string(a_g, a_agent)
			else
			end
		end
end
