note
	description: "GUI common dialogs"
	notice: "Do not create instances (useless), inherit ROSE_SHARED_GLOBAL_GUI and use Gui.dialogs"
	author: "Zoran Simic"
	last_editor: "$Author: er $"
	date: "$Date: 2014/02/26 $"
	revision: "$Revision: #50 $"
	path: "$File: //rose/source/kernel/infrastructure/framework/gui/rose_ev_common_dialogs.e $"

class
	ROSE_EV_COMMON_DIALOGS

inherit
	ROSE_FORMATTING

feature -- Access

	select_open_file (a_widget: EV_WIDGET; a_id, a_filter: STRING): STRING
			-- Open a file dialog asking for a file name
			-- Dialog poped up is modal to 'a_widget'
			-- 'a_id' specifies what kind of file is wanted, 'a_filter' is of form "*.ext"
			-- Returns file name selected, or Void if user hits Cancel
			-- Void if selected file does not exist or is unreadable (+ a notification message is poped up)
			-- Last directory used for 'a_id' is remembered in config (if not Void)
		require
			valid_widget: a_widget /= Void
		local
			l_start_dir: STRING
			l_dialog: EV_FILE_OPEN_DIALOG
			l_title, l_config_key: STRING
			l_file: RAW_FILE
			rescued: BOOLEAN
		do
			if rescued then
				Result := Void
				inform (a_widget,
					"Something went wrong",
					"The file you requested presents a problem while examining it%NThis operation is cancelled", Void)
			else
				if a_id /= Void then
					l_title := "Open " + a_id
					l_config_key := k_filepath + a_id
					l_config_key.replace_substring_all (" ", "_")
				else
					l_title := "Open file"
				end
				create l_dialog
				l_dialog.set_title (l_title)
				l_start_dir := glob.Global.user.config.value_default (l_config_key, default_directory)
				if l_start_dir /= Void then
					l_dialog.set_start_directory (l_start_dir)
				end
				if a_filter /= Void then
					l_dialog.set_filter (a_filter)
				end
				l_dialog.show_modal_to_window (glob.Gui.parent_window (a_widget))
				Result := l_dialog.file_name
				if Result.is_empty then
					Result := Void
				else
					create l_file.make (Result)
					if not l_file.exists then
						inform (a_widget, l_title+" notification", "File '$1' does not exist", <<Result>>)
						Result := Void
					elseif not l_file.is_readable then
						inform (a_widget, l_title+" notification", "File '$1' is not readable", <<Result>>)
						Result := Void
					end
				end
				if Result /= Void and then l_config_key /= Void then
					glob.Global.user.config.set (l_config_key, formatter.directory (Result))
				end
			end
		rescue
			if not rescued then
				rescued := True
				retry
			end
		end

	select_open_files (a_widget: EV_WIDGET; a_id, a_filter: STRING): ARRAYED_LIST [STRING_32]
			-- Open a file dialog asking for one or several file names
			-- Dialog poped up is modal to 'a_widget'
			-- 'a_id' specifies what kind of file is wanted, 'a_filter' is of form "*.ext"
			-- Returns file name selected, or Void if user hits Cancel
			-- Void if selected file does not exist or is unreadable (+ a notification message is poped up)
			-- Last directory used for 'a_id' is remembered in config (if not Void)
		require
			widget_ok: a_widget /= Void
		local
			l_start_dir: STRING
			l_dialog: EV_FILE_OPEN_DIALOG
			l_title, l_config_key: STRING
			rescued: BOOLEAN
			l_filename: STRING
		do
			if rescued then
				Result := Void
				inform (a_widget,
					"Something went wrong",
					"The file you requested presents a problem while examining it%NThis operation is cancelled", Void)
			else
				if a_id /= Void then
					l_title := "Open " + a_id
					l_config_key := k_filepath + a_id
					l_config_key.replace_substring_all (" ", "_")
				else
					l_title := "Open file"
				end
				create l_dialog
				l_dialog.set_title (l_title)
				l_start_dir := glob.Global.user.config.value_default (l_config_key, default_directory)
				if l_start_dir /= Void then
					l_dialog.set_start_directory (l_start_dir)
				end
				if a_filter /= Void then
					l_dialog.set_filter (a_filter)
				end
				l_dialog.enable_multiple_selection
				l_dialog.show_modal_to_window (glob.Gui.parent_window (a_widget))
				Result := l_dialog.file_names
				if not Result.is_empty then
					l_filename := Result.first
					if l_filename /= Void and then l_config_key /= Void then
						glob.Global.user.config.set (l_config_key, formatter.directory (l_filename))
					end
				end
			end
		rescue
			if not rescued then
				rescued := True
				retry
			end
		end

	select_save_file (a_widget: EV_WIDGET; a_id, a_filter: STRING): STRING
			-- Open a save-file dialog asking for a file name
			-- Dialog poped up is modal to 'a_widget'
			-- 'a_id' specifies what kind of file is wanted, 'a_filter' is of form "*.ext"
			-- Returns file name selected, or Void if user hits Cancel
			-- Void if selected file is not writable (+ a notification message is poped up)
			-- Last directory used for 'a_id' is remembered in config (if not Void)
		require
			valid_widget: a_widget /= Void
		local
			l_start_dir: STRING
			l_dialog: EV_FILE_SAVE_DIALOG
			l_title, l_config_key: STRING
			l_file: RAW_FILE
			rescued: BOOLEAN
		do
			if rescued then
				Result := Void
				inform (a_widget,
					"Something went wrong",
					"The file you requested presents a problem while examining it%NThis operation is cancelled", Void)
			else
				if a_id /= Void then
					l_title := "Save " + a_id
					l_config_key := k_filepath + a_id
					l_config_key.replace_substring_all (" ", "_")
				else
					l_title := "Save file"
				end
				create l_dialog
				l_dialog.set_title (l_title)
				l_start_dir := glob.Global.user.config.value_default (l_config_key, default_directory)
				if l_start_dir /= Void then
					l_dialog.set_start_directory (l_start_dir)
				end
				if a_filter /= Void then
					l_dialog.set_filter (a_filter)
				end
				l_dialog.show_modal_to_window (glob.Gui.parent_window (a_widget))
				Result := l_dialog.file_name
				if Result.is_empty then
					Result := Void
				else
					create l_file.make (Result)
					if l_file.exists then
						if not answer (a_widget, Void, "Overwrite?", "Do you want to overwrite '$1'?", <<Result>>) then
							Result := Void
						elseif not l_file.is_writable then
							inform (a_widget, "I/O problem", "File '$1' is not writable", <<Result>>)
							Result := Void
						end
					end
				end
				if Result /= Void and then l_config_key /= Void then
					glob.Global.user.config.set (l_config_key, formatter.directory (Result))
				end
			end
		rescue
			if not rescued then
				rescued := True
				retry
			end
		end

	select_directory (a_widget: EV_WIDGET; a_id, a_starting_directory: STRING): STRING
			-- Open a directory selection dialog
			-- Dialog poped up is modal to 'a_widget'
			-- Void if user hits 'Cancel' or selected directory does not exist/not readable
			-- Last directory used for 'a_id' is remembered in config (if not Void)
		require
			valid_widget: a_widget /= Void
		local
			l_title, l_config_key, l_start_dir: STRING
			l_dialog: EV_DIRECTORY_DIALOG
			l_dir: DIRECTORY
			rescued: BOOLEAN
		do
			if rescued then
				Result := Void
				inform (a_widget,
					"Something went wrong",
					"The directory you selected presents a problem while examining it%NThis operation is cancelled", Void)
			else
				if a_id /= Void then
					l_title := "Select a " + a_id + " directory"
					l_config_key := k_filepath + a_id
					l_config_key.replace_substring_all (" ", "_")
				else
					l_title := "Select a directory"
				end
				create l_dialog
				l_dialog.set_title (l_title)
				if glob.Global.is_directory_readable (a_starting_directory) then
					l_start_dir := a_starting_directory
				else
					l_start_dir := glob.Global.user.config.value_default (l_config_key, default_directory)
				end
				if glob.Global.is_directory_readable (l_start_dir) then
					l_dialog.set_start_directory (l_start_dir)
				end
				l_dialog.show_modal_to_window (glob.Gui.parent_window (a_widget))
				Result := l_dialog.directory
				if Result.is_empty then
					Result := Void
				else
					Result := glob.Global.convention.clean_directory_path (Result)
					create l_dir.make (Result)
					if not l_dir.exists then
						inform (a_widget, l_title+" notification", "Directory '$1' does not exist", <<Result>>)
						Result := Void
					elseif not l_dir.is_readable then
						inform (a_widget, l_title+" notification", "Directory '$1' is not readable", <<Result>>)
						Result := Void
					end
				end
				if Result /= Void and then l_config_key /= Void then
					glob.Global.user.config.set (l_config_key, Result)
				end
			end
		rescue
			if not rescued then
				rescued := True
				retry
			end
		end

	select_print_context (a_widget: EV_WIDGET; a_title: STRING): EV_PRINT_CONTEXT
			-- Open a print dialog
			-- Dialog poped up is modal to 'a_widget'
			-- Void if user hits 'Cancel' or selected directory does not exist/not available
		require
			valid_widget: a_widget /= Void
		local
			l_title: STRING
			l_dialog: EV_PRINT_DIALOG
			rescued: BOOLEAN
		do
			if rescued then
				Result := Void
				inform (a_widget,
					"Something went wrong",
					"The printer you selected presents a problem while examining it%NThis operation is cancelled", Void)
			else
				if a_title /= Void then
					l_title := a_title
				else
					l_title := "Select a printer"
				end
				create l_dialog.make_with_title (l_title)
				l_dialog.set_maximum_to_page (100)
				l_dialog.select_all_pages
				l_dialog.show_modal_to_window (glob.Gui.parent_window (a_widget))
				if l_dialog.copies > 0 then
					Result := l_dialog.print_context
				end
			end
		rescue
			if not rescued then
				rescued := True
				retry
			end
		end

	answer (a_widget: EV_WIDGET; a_id, a_title, a_question: STRING; a_args: ARRAY [ANY]): BOOLEAN
			-- Returns user answer to a yes-no 'a_question' argumented with 'a_args'
			-- Dialog poped up is modal to 'a_widget'
			-- If 'a_id' is provided, an option allows to disable asking this question in the future
			-- 'a_title' is optional ("Question" is default)
		local
			l_dialog: EV_DIALOG
			l_pixmap: EV_PIXMAP
			l_main_box, l_box, l_box2: EV_BOX
			l_cell: EV_CELL
			l_dont_ask: EV_CHECK_BUTTON
			l_widget: EV_WIDGET
			l_title, l_id: STRING
		do
			if a_id /= Void and then not a_id.is_empty then
				l_id := k_auto_answer + a_id
			end
			if l_id /= Void
			and then glob.Global.user.config.has (l_id)
			and then not glob.Gui.ev_application.shift_pressed then
				Result := glob.Global.user.config.boolean (l_id, False)
			else
				if a_title /= Void and then not a_title.is_empty then
					l_title := a_title
				else
					l_title := k_question
				end
				l_pixmap := glob.Gui.rep.pixmaps.Question_pixmap
				l_dialog := glob.Gui.factory.new_dialog (l_title, l_pixmap, 0, 0)
				-- FIXME: The following line is a workaround to a Vision2 problem encountered
				-- when running a multithreaded app under .NET... the dialog would get garbage collected by mistake
				answer_dialog := l_dialog
				l_main_box := glob.Gui.factory.new_vbox (l_dialog, 6, 4)
				l_box := glob.Gui.factory.new_hbox (l_main_box, 0, -8)
				l_box2 := glob.Gui.factory.new_vbox (l_box, 0, 1)
				l_cell := glob.Gui.factory.new_cell (l_box2, 32, 32)
				protected_extend_pixmap (l_cell, l_pixmap)
				l_widget := glob.Gui.factory.new_label (l_box2, Void, 0)
				l_widget := glob.Gui.factory.new_label (l_box, glob.Global.formatted (a_question, a_args), 0)
				if l_id /= Void then
					l_dont_ask := glob.Gui.factory.new_check_button (l_main_box, glob.Global.user.config.has (l_id), "Do not ask this question in the future", Void, 0)
					l_dont_ask.enable_sensitive
				end
				add_ok_cancel (l_dialog, Void, k_yes)
				Result := ok_or_cancel (a_widget, l_dialog)
				if l_dont_ask /= Void and then l_dont_ask.is_selected then
					glob.Global.user.config.set (l_id, Result.out)
				else
					glob.Global.user.config.delete (l_id)
				end
			end
		end

	ask_for_string (a_widget: EV_WIDGET; a_title, a_question, a_initial: STRING): STRING
			-- Returns user entered string, in a pop up window
			-- Dialog poped up is modal to 'a_widget'
			-- 'a_title' is optional ("Question" is default)
		require
			valid_widget: a_widget /= Void
			valid_question: a_question /= Void
		local
			l_dialog: ROSE_DIALOG
			l_text: EV_TEXT_FIELD
			l_new_width: INTEGER
		do
			create l_dialog
			l_dialog.set_icon_pixmap (glob.Gui.rep.pixmaps.Question_pixmap)
			if a_title /= Void then
				l_dialog.set_title (a_title)
			else
				l_dialog.set_title (k_question)
			end
			l_text := glob.Gui.factory.new_text_field (l_dialog.main_box, a_question, a_initial, Void, 0)
			if l_text.text_length > 0 then
				if a_initial/=Void
				then
					l_new_width := l_text.font.string_width(a_initial) + 4
					if l_new_width>glob.gui.screen.width // 2
					then
						l_new_width := glob.gui.screen.width // 2		-- Please, no more than half of the screen!
					end
				end

				-- At this point we wish to call "l_text.select_all", however the behavior is odd.
				-- Half of the text is hidden.  By calling it during the 'show_actions' we fix this problem.
				l_dialog.show_actions.extend(agent l_text.select_all)
			end

			if l_new_width<100
			then
				l_new_width := 100		-- This forces the width of 'l_text' to a usable size.
			end

			l_text.set_minimum_width(l_new_width)	-- I would rather call 'set_width' but there isn't one.
			l_dialog.show_actions.extend (agent set_focus (l_text))
			l_dialog.show_modal (a_widget)
			if l_dialog.is_ok_pressed then
				Result := l_text.text
			end
		end

	select_from_a_list (a_widget: EV_WIDGET; a_title, a_question : STRING; a_list: ARRAY [STRING]): STRING is
			-- Returns user entered string, in a pop up window
			-- Dialog poped up is modal to 'a_widget'
			-- 'a_title' is optional ("Question" is default)
		require
			valid_widget: a_widget /= Void
			valid_question: a_question /= Void
		local
			l_dialog: ROSE_DIALOG
			l_hbox: EV_HORIZONTAL_BOX
			l_combo: EV_COMBO_BOX
			l_width: INTEGER
			l_dummy_combo: ROSE_COMBO_BOX
			l_dummy_scroll_bar: EV_VERTICAL_SCROLL_BAR
		do
			create l_dialog
			l_dialog.set_icon_pixmap (glob.Gui.rep.pixmaps.Question_pixmap)
			if a_title /= Void then
				l_dialog.set_title (a_title)
			else
				l_dialog.set_title (k_question)
			end
			l_hbox := glob.Gui.factory.new_hbox (l_dialog.main_box, 0, 0)
			create l_dummy_combo
			create l_dummy_scroll_bar
			l_width := longest_string (a_list, l_dummy_combo.font) + l_dummy_scroll_bar.width
			l_combo := glob.Gui.factory.new_combo (l_hbox, a_question, a_list, "default", Void, l_width)
			l_dialog.show_modal (a_widget)
			if l_dialog.is_ok_pressed then
				Result := l_combo.text
			end
		end

	is_automatically_answered (a_id: STRING): BOOLEAN
			-- Is question with 'a_id' automatically answered?
			-- Ie, did the user check the 'do not ask this again' check box for that question?
		do
			if a_id /= Void and then not a_id.is_empty then
				Result := glob.Global.user.config.has (k_auto_answer + a_id)
			end
		end

feature -- Basic operations

	show_modal (a_parent, a_widget: EV_WIDGET; a_title: STRING)
			-- Show 'a_widget' modally to 'a_parent' with 'a_title'
		require
			parent_ok: a_parent /= Void
			widget_ok: a_widget /= Void and then a_widget.parent = Void
		local
			l_dialog: ROSE_DIALOG
			l_box, l_hbox: EV_BOX
			l_button: EV_BUTTON
			l_widget: EV_WIDGET
		do
			create l_dialog
			if a_title /= Void then
				l_dialog.set_title (a_title)
			end
			l_dialog.set_icon ("info.ico")
			l_box := glob.Gui.factory.new_vbox (l_dialog, 0, 0)
			l_box.extend (a_widget)
			l_hbox := glob.Gui.factory.new_hbox (l_box, 0, 0)
			l_widget := glob.Gui.factory.new_label (l_hbox, Void, 0)
			l_button := glob.Gui.factory.new_button (l_hbox, "Close", agent l_dialog.hide, 1)
			l_widget := glob.Gui.factory.new_label (l_hbox, Void, 0)
			l_dialog.set_default_push_button (l_button)
			l_dialog.set_default_cancel_button (l_button)
			l_dialog.show_modal (a_parent)
		end

	popup_menu (a_activator: PROCEDURE [ANY, TUPLE [EV_MENU]]; a_description: TUPLE [STRING, TUPLE])
			-- Popup a menu from 'a_description', 'a_activation' indicates which
		local
			l_menu: EV_MENU
		do
			l_menu := glob.Gui.factory.new_menu (a_description)
			if l_menu.count > 0 then
				if a_activator /= Void then
					a_activator.call ([l_menu])
				end
				l_menu.show
			end
		end

	popup_info (a_message: STRING; a_args: ARRAY [ANY])
			-- Popup an untitled dialog with 'a_message' (formatted with 'a_args' in the usual way)
			-- The poped up dialog appears next to the mouse, has no title
			-- and disappears as soon as it looses focus, or ESC is hit.
		local
			l_dialog: EV_UNTITLED_DIALOG
			l_vbox: EV_BOX
			l_text: EV_TEXT
			l_message: STRING
			l_width, l_height: INTEGER
			l_size: TUPLE [INTEGER, INTEGER]
		do
			l_message := glob.Global.formatted (a_message, a_args)
			if l_message /= Void and then not l_message.is_empty then
				create l_dialog
				l_vbox := glob.Gui.factory.new_vbox (l_dialog, 0, 0)
				l_text := glob.Gui.factory.new_text (l_vbox, Void, 0, 0)
				l_text.set_background_color (popup_info_background_color)
				l_text.focus_out_actions.extend (agent l_dialog.destroy)
				l_text.key_press_actions.extend (agent on_close_key (l_text, ?))
				l_dialog.key_press_actions.extend (agent on_close_key (l_text, ?))
				l_text.disable_edit
				-- Sizing
				l_size := l_text.font.string_size (l_message)
				l_width := l_size.integer_item (1)
				l_height := l_size.integer_item (2)
				l_text.set_minimum_width (l_width)
				l_text.set_minimum_height (l_height)
				l_dialog.set_minimum_width (l_width+34)
				l_dialog.set_minimum_height (l_height+8)
				l_dialog.disable_user_resize
				glob.Gui.output (l_text, l_message, Void)
				-- Position and show
				position_around_pointer (l_dialog, l_width, l_height)
				l_dialog.focus_out_actions.extend (agent l_dialog.destroy)
				l_dialog.show
				l_dialog.set_focus
			end
		end

	show_tips (a_widget: EV_WIDGET; a_id, a_title: STRING; a_tips: ARRAY [STRING]; a_force : BOOLEAN)
			-- Pops up a tip from `a_tips' (either the first tip or the last remembered position)
			-- User can cycle through tips; last viewed will be remembered
			-- if `a_force' then tips will be shown even if cfg value set to suppress tips
			-- If 'a_widget' given, then pops up message modal to that widget's window
			-- All parameters except `a_tips' and `a_force' are optional (use void if not needed)
		require
			valid_tips: a_tips /= Void and then not a_tips.is_empty
		local
			l_title, l_id, l_page_id: STRING
			l_index : INTEGER
			l_window : EV_WINDOW
			l_tips : ROSE_LINEAR_ARRAY [STRING]
			l_dialog: EV_DIALOG
			l_pixmap: EV_PIXMAP
			l_main_box, l_box, l_box2: EV_BOX
			l_cell: EV_CELL
			l_dont_ask: EV_CHECK_BUTTON
			l_label: EV_LABEL
			l_button : EV_BUTTON
		do
			if a_id /= Void and then not a_id.is_empty then
				l_id := k_suppress_dialog + a_id
			end
			if l_id = Void or else a_force or else (l_id /= Void and then not glob.Global.user.config.boolean (l_id, False)) then
				if a_id /= Void and then not a_id.is_empty then
					l_page_id := k_page_number + a_id
				end
				if l_page_id /= Void then
					l_index := glob.global.user.config.integer (l_page_id, 1)
				end
				if not a_tips.valid_index (l_index) then
					l_index := 1
				end
				create l_tips.make_from_array (a_tips)
				l_tips.go_i_th (l_index)
				if a_title /= Void and then not a_title.is_empty then
					l_title := a_title
				else
					l_title := k_tips
				end
				l_pixmap := glob.gui.factory.new_pixmap ("ob_boolean0.ico")
				l_dialog := glob.Gui.factory.new_dialog (l_title, l_pixmap, 0, 0)
				-- FIXME: The following line is a workaround to a Vision2 problem encountered
				-- when running a multithreaded app under .NET... the dialog would get garbage collected by mistake
				tips_dialog := l_dialog
				l_main_box := glob.Gui.factory.new_vbox (l_dialog, 6, 4)
				l_box := glob.Gui.factory.new_hbox (l_main_box, 0, -8)
				l_box2 := glob.Gui.factory.new_vbox (l_box, 0, 1)
				l_cell := glob.Gui.factory.new_cell (l_box2, 32, 32)
				protected_extend_pixmap (l_cell, l_pixmap)
				l_label := glob.Gui.factory.new_label (l_box2, Void, 0)
				l_label := glob.Gui.factory.new_label (l_box, a_tips [l_index], 0)
				add_ok_cancel (l_dialog, Void, k_cancel)
				if a_tips.count > 1 then
					l_main_box ?= l_dialog.item
					l_box ?= l_main_box.last
					l_box.go_i_th (l_box.count)
					l_box.remove_left
					l_box.remove_left
					l_button := glob.gui.factory.new_button (Void, "< Previous tip", agent show_next_tip (l_label, a_id, l_tips, False), 75)
					l_box.put_left (l_button)
					l_button := glob.gui.factory.new_button (Void, "Next tip >", agent show_next_tip (l_label, a_id, l_tips, True), 75)
					l_box.put_left (l_button)
				end
				if l_id /= Void then
					l_dont_ask := glob.Gui.factory.new_check_button (l_main_box, False, "Do not show tips in the future", Void, 0)
					if glob.global.user.config.boolean (l_id, False) then
						l_dont_ask.enable_select
					end
					l_dont_ask.select_actions.extend (agent set_dialog_suppression (l_dont_ask, a_id))
					l_dont_ask.enable_sensitive
				end
				if a_widget /= Void then
					l_window := glob.Gui.parent_window (a_widget)
				end
				if l_window /= Void then
					l_dialog.show_modal_to_window (l_window)
				else
					l_dialog.show
					l_dialog.set_focus
				end
			end
		end

	position_around_pointer (a_window: EV_WINDOW; a_width, a_height: INTEGER)
			-- Position 'a_window' around mouse pointer on screen
			-- 'a_width' and 'a_height' indicate desire size of the window (0,0 to use window's own size)
		local
			l_x, l_y, l_width, l_height: INTEGER
		do
			if a_window /= Void then
				if a_width > 0 then
					l_width := a_width
				else
					l_width := a_window.width
				end
				if a_height > 0 then
					l_height := a_height
				else
					l_height := a_window.height
				end
				l_x := glob.Gui.screen.pointer_position.x + 5
				l_y := glob.Gui.screen.pointer_position.y + 5
				if l_x + l_width > glob.Gui.screen.width then
					l_x := glob.Gui.screen.width - l_width - 10
				end
				if l_y + l_height > glob.Gui.screen.height - 50 then
					l_y := glob.Gui.screen.height - l_height - 40
				end
				a_window.set_x_position (l_x.max (0))
				a_window.set_y_position (l_y.max (0))
			end
		end

	inform (a_widget: EV_WIDGET; a_title, a_message: STRING; a_args: ARRAY [ANY])
			-- Pops up an information message for 'a_widget' with 'a_title'
			-- and 'a_message' (argumented with 'a_args')
			-- If 'a_widget' given, then pops up message modal to that widget's window
			-- All parameters except 'a_message' are optional (use void if not needed)
		require
			valid_message: a_message /= Void
		local
			l_title: STRING
		do
			if a_title = Void then
				l_title := "For your information"
			else
				l_title := a_title
			end
			popup_info_message (a_widget, glob.Gui.rep.pixmaps.Information_pixmap, l_title, a_message, a_args)
		end

	warn (a_widget: EV_WIDGET; a_title, a_message: STRING; a_args: ARRAY [ANY])
			-- Pops up a warning message for 'a_widget' with 'a_title'
			-- and 'a_message' (argumented with 'a_args')
			-- If 'a_widget' given, then pops up message modal to that widget's window
			-- All parameters except 'a_message' are optional (use void if not needed)
		require
			valid_message: a_message /= Void
		local
			l_title: STRING
		do
			if a_title = Void then
				l_title := "Warning !"
			else
				l_title := a_title
			end
			popup_info_message (a_widget, glob.Gui.rep.pixmaps.Warning_pixmap, l_title, a_message, a_args)
		end

	error (a_widget: EV_WIDGET; a_title, a_message: STRING; a_args: ARRAY [ANY])
			-- Pops up an error message for 'a_widget' with 'a_title'
			-- and 'a_message' (argumented with 'a_args')
			-- If 'a_widget' given, then pops up message modal to that widget's window
			-- All parameters except 'a_message' are optional (use void if not needed)
		require
			valid_message: a_message /= Void
		local
			l_title: STRING
		do
			if a_title = Void then
				l_title := "Error !"
			else
				l_title := a_title
			end
			popup_info_message (a_widget, glob.Gui.rep.pixmaps.Error_pixmap, l_title, a_message, a_args)
		end

	interactive_report (a_widget: EV_WIDGET; a_id, a_recipients, a_subject, a_message: STRING; a_args: ARRAY [ANY])
			-- Report 'a_message' with 'a_subject' by e-mail
			-- Report dialog is shown modal to 'a_widget'
			-- 'a_id' Void defaults to application's executable name for 'a_id'
			-- Recipients are searched for in configuration under "/support/<a_id>", they default to 'a_recipients' if none configured
			-- Previous recipients are saved in user config under 'a_id'
			-- This pops up a dialog showing the message, recipients and subject.
			-- The user can send the mail, or cancel, if sending fails user is notified
			-- The user has the possibility to change recipients, subject or body of the mail
			-- Also, a report on the current application is appended to 'a_message'
		local
			l_dialog: ROSE_MAIL_DIALOG
			l_id, l_recipients: STRING
		do
			glob.error.acknowledge
				-- Recipients
			if a_id = Void or else a_id.is_empty then
				l_id := glob.Global.app.program_name
				if l_id = Void then
					l_id := ""
				end
			else
				l_id := a_id
			end
			if a_recipients = Void or else a_recipients.is_empty then
				l_recipients := glob.config.value_default ("support", l_recipients)
				l_recipients := glob.config.value_default ("support/" + l_id, l_recipients)
				l_recipients := glob.config.value_default ("$system/support", l_recipients)
				l_recipients := glob.config.value_default ("$system/support/" + l_id, l_recipients)
				if l_recipients = Void or else l_recipients.is_empty then
					l_recipients := "SoftwareTechandInfTools@axarosenberg.com"
				end
			else
				l_recipients := a_recipients
			end
				-- Dialog
			create l_dialog
			l_dialog.set_recipients (l_recipients)
			l_dialog.set_subject (glob.Global.interpreted (a_subject))
			l_dialog.set_text (formatter.argumented (a_message, a_args))
			l_dialog.show_modal (a_widget)
		end

	save_as_png (a_widget: EV_WIDGET; a_pixmap: EV_PIXMAP; a_id, a_filename: STRING)
			-- Save 'a_pixmap' as a PNG picture to 'a_filename'
			-- If 'a_filename' is not provided (Void) and 'a_widget' is provided,
			-- the user is prompted to select a destination file name, 'a_id' is then used
			-- to remember last directory used to save by the framework
		local
			l_filename, rescued: STRING
		do
			if rescued /= Void then
				inform (a_widget, "Could not save picture", "A problem arose when attempting to save picture:%N$1", <<rescued>>)
			else
				if a_filename /= Void then
					l_filename := a_filename
				elseif a_widget /= Void then
					l_filename := select_save_file (a_widget, a_id, "*.png")
				else
					inform (a_widget, "Could not save picture", "Can't determine destination file name", Void)
				end
				if l_filename /= Void then
					a_pixmap.save_to_named_file (png_format, create {FILE_NAME}.make_from_string (l_filename))
				end
			end
		rescue
			if not glob.error.signaled then
				rescued := glob.last_exception_explanation
				retry
			end
		end

feature {NONE} -- Implementation

	answer_dialog: EV_DIALOG
			-- Dialog shown on calling `answer'

	tips_dialog: EV_DIALOG
			-- Dialog displaying list of tips

	default_directory: STRING
			-- Default directory to start in for open/save file dialogs
		once
			Result := glob.global.hardcoded_path ("C:\")
		end

feature -- Measurement
	
	longest_string (a_list: ARRAY [STRING]; a_font: EV_FONT): INTEGER is
			-- Longest entry within `a_list';
			-- Zero, if a_list is Void, or has no contents
		require
			a_font_not_void: a_font /= Void
		local
			i: INTEGER
			l_string: STRING
		do
			if a_list /= Void then
				from i := a_list.lower
				until
					i > a_list.upper
				loop
					l_string := a_list [i]
					if l_string /= Void then
						Result := Result.max (a_font.string_width (l_string))
					end
					i := i + 1
				end
			end
		end

	show_next_tip (a_label : EV_LABEL; a_id: STRING; a_tips : ROSE_LINEAR_ARRAY [STRING]; a_forward : BOOLEAN)
			-- Display next tip/prev in `a_dialog'
			-- Set cfg to remember last page number
		require
			a_label_not_void : a_label /= Void
			valid_tips : a_tips /= Void and then not a_tips.is_empty
		local
			l_page_id : STRING
		do
			if a_forward then
				a_tips.forth
				if a_tips.after then
					a_tips.go_i_th (1)
				end
			else
				a_tips.back
				if a_tips.before then
					a_tips.go_i_th (a_tips.count)
				end
			end
			a_label.set_text (a_tips.current_item)
			l_page_id := k_page_number + a_id
			glob.Global.user.config.set (l_page_id, a_tips.index.out)
		end

feature {NONE} -- Implementation

	set_dialog_suppression (a_check : EV_CHECK_BUTTON; a_id : STRING)
			-- Set k_suppress_dialog cfg value to `a_check.is_selected'
		require
			a_check_not_void : a_check /= Void
			a_id_not_void : a_id /= Void
		local
			l_id : STRING
		do
			l_id := k_suppress_dialog + a_id
			glob.Global.user.config.set (l_id, a_check.is_selected.out)
		end

	popup_info_message (a_widget: EV_WIDGET; a_pixmap: EV_PIXMAP; a_title, a_message: STRING; a_args: ARRAY [ANY])
			-- Pops up am information message for 'a_widget' with 'a_title'
			-- and 'a_message' (argumented with 'a_args')
			-- If 'a_widget' given, then pops up message modal to that widget's window
			-- All parameters except 'a_message' are optional (use void if not needed)
		require
			valid_message: a_message /= Void
		local
			l_dialog: EV_INFORMATION_DIALOG
			l_window: EV_WINDOW
			l_title: STRING
		do
			create l_dialog
			l_dialog.set_icon_pixmap (a_pixmap)
			if a_title /= Void then
				l_dialog.set_title (a_title)
			else
				l_dialog.set_title (k_notification)
			end
			l_title := glob.Global.formatted (a_message, a_args)
			l_title.prune_all ('%R')
			l_dialog.set_text (l_title)
			if a_widget /= Void then
				l_window := glob.Gui.parent_window (a_widget)
			end
			if l_window /= Void then
				l_dialog.show_modal_to_window (l_window)
			else
				l_dialog.show
				l_dialog.set_focus
			end
		end

	add_ok_cancel (a_dialog: EV_DIALOG; a_enabler: ARRAY [EV_TEXT_COMPONENT]; a_okname: STRING)
			-- Add OK/Cancel button to 'a_dialog'
		local
			l_main_box, l_box: EV_BOX
			l_button: EV_BUTTON
			l_widget: EV_LABEL
			l_ok, l_cancel: STRING
		do
			l_main_box ?= a_dialog.item
			l_box := glob.Gui.factory.new_hbox (l_main_box, 0, 16)
			if a_okname = k_yes then
				l_ok := k_yes
				l_cancel := k_no
			else
				if a_okname /= Void then
					l_ok := a_okname
				else
					l_ok := k_ok
				end
				l_cancel := k_cancel
			end
			l_widget := glob.Gui.factory.new_label (l_box, Void, 0)
			l_button := glob.Gui.factory.new_button (l_box, l_ok, agent on_dialog_hide (a_dialog, True), 80)
			if a_okname = Void or else a_okname = k_ok or else a_okname = k_yes then
				a_dialog.set_default_push_button (l_button)
			end
			if a_enabler /= Void then
				a_dialog.key_release_actions.extend (agent on_key_validation (l_button, a_enabler, ?))
				on_key_validation (l_button, a_enabler, Void)
			end
			l_button := glob.Gui.factory.new_button (l_box, l_cancel, agent on_dialog_hide (a_dialog, False), 80)
			a_dialog.set_default_cancel_button (l_button)
			l_widget := glob.Gui.factory.new_label (l_box, Void, 0)
		end

	ok_or_cancel (a_widget: EV_WIDGET; a_dialog: EV_DIALOG): BOOLEAN
			-- Show 'a_dialog' modally to 'a_widget' and return whether 'OK' was pressed or not
		do
			a_dialog.show_modal_to_window (glob.Gui.parent_window (a_widget))
			Result := a_dialog.data /= Void
			a_dialog.set_data (Void)
		end

	on_dialog_hide (a_dialog: EV_DIALOG; a_ok: BOOLEAN)
			-- Hide 'a_dialog'
		do
			if a_ok then
				a_dialog.set_data (k_ok)
			end
			a_dialog.hide
		end

	popup_info_background_color: EV_COLOR
			-- Background color for 'popup_info' dialogs
		once
			Result := glob.Gui.rep.colors.rgb (255, 243, 198)
		end

	k_question: STRING = "Question"						-- Constant
	k_tips: STRING = "Tips"								-- Constant
	k_notification: STRING = "Notification"				-- Constant
	k_filepath: STRING = "filepath/"					-- Config key
	k_yes: STRING = "Yes"								-- Constant
	k_no: STRING = "No"									-- Constant
	k_ok: STRING = "OK"									-- Constant
	k_cancel: STRING = "Cancel"							-- Constant

	k_auto_answer: STRING = "auto_answer/"				-- Config key
	k_suppress_dialog: STRING = "suppress_dialog/"		-- Config key
	k_page_number: STRING = "page_number/"				-- Config key

	on_close_key (a_widget: EV_WIDGET; a_key: EV_KEY)
			-- Close 'a_window' if 'a_key' is ESC.
		local
			l_window: EV_WINDOW
		do
			if a_key /= Void and then a_key.code = glob.Gui.rep.keys.Key_escape then
				l_window := glob.Gui.parent_window (a_widget)
				if l_window /= Void then
					l_window.hide
				end
			end
		end

	on_key_validation (a_button: EV_BUTTON; a_enabler: ARRAY [EV_TEXT_COMPONENT]; a_key: EV_KEY)
			-- Called on key press to enable/disable 'a_button' relative to input in 'a_enabler'
		local
			l_string: STRING
			l_disable: BOOLEAN
			i: INTEGER
		do
			if a_enabler /= Void then
				from
					i := a_enabler.lower
				until
					l_disable or else i > a_enabler.upper
				loop
					l_string := a_enabler.item (i).text
					l_disable := l_string = Void or else l_string.is_empty
					i := i + 1
				end
				if l_disable then
					a_button.disable_sensitive
				else
					a_button.enable_sensitive
				end
			end
		end

	on_interactive_report_focus (a_recipients: EV_TEXT_FIELD; a_body: EV_TEXT)
			-- Called when interactive report is shown
		do
			a_recipients.set_caret_position (1)
			a_body.set_focus
		end

	protected_extend_pixmap (a_container: EV_CONTAINER; a_pixmap: EV_PIXMAP)
			-- Needed because of a bug in EV_PIXMAP_IMP in .NET only, v5.5.0308
		local
			retried: BOOLEAN
		do
			if not retried and then a_pixmap /= Void then
				a_container.extend (a_pixmap.twin)
			end
		rescue
			if not retried then
				retried := True
				retry
			end
		end

	set_focus (a_widget: EV_WIDGET)
			-- Set focus on 'a_widget' if it is valid
		do
			if a_widget /= Void and then not a_widget.is_destroyed
			and then a_widget.is_displayed and then a_widget.is_sensitive then
				a_widget.set_focus
			end
		end

	mail: attached ROSE_MAIL
			-- Used to send email
		once
			create Result.make
		end

	glob: attached ROSE_SHARED_GLOBAL_GUI
			-- Reference to global Gui object, to avoid inheriting (and inheritance clashes)
		once
			create Result
		end

	png_format: attached EV_PNG_FORMAT
			-- PNG format
		once
			create Result
		end

end
