indexing
	description: "EV date field widget - for entering dates via a dedicated EV widget"
	author: "Mark Howard"
	date: "$Date: 2013/04/24 $"
	revision: "$Revision: #11 $"

class
	ROSE_EV_DATE_FIELD

inherit
	EV_FONTABLE
		redefine
			initialize, is_in_default_state, implementation,
			replace_implementation
		end
	EV_PRIMITIVE
		redefine
			initialize, is_in_default_state, implementation,
			replace_implementation
		end

feature {NONE} -- Initialization

	initialize is
			-- Called by 'default_create'
		do
			Precursor
			create change_actions
			enable_edit (True)
		end

	is_in_default_state: BOOLEAN is
			-- Is `Current' in its default state?
			-- (export status {NONE})
		do
			Result := True
		end

feature -- Access

	is_editable: BOOLEAN is
			-- Is this date-field currently editable?
		do
			Result := implementation.is_editable
		end

	is_minimalistic: BOOLEAN
			-- Should this date be represented as 'min_date' when Void?
			-- (represented as 'max_date' otherwise)

	is_voidable: BOOLEAN
			-- Can this date hold Void? (if not, set_date (Void) yields current day)

	change_actions: ACTION_SEQUENCE [TUPLE []]
			-- Called whenever the date in this field was changed by user

	selected_date, current_date: ROSE_DATE is
			-- Currently selected date
		do
			if not is_void then
				Result := to_rose_date (implementation.system_time)
			end
		end

	selected_date_time: ROSE_DATE is
			-- Currently selected date-time
		do
			if not is_void then
				create Result.make_ccyymmdd (selected_date.ccyymmdd)
			end
		end

	minimum_date: ROSE_DATE is
			-- Minimum selectable date
		do
			Result := date_to_rose_date (implementation.min_date)
		end

	maximum_date: ROSE_DATE is
			-- Maximum selectable date
		do
			Result := date_to_rose_date (implementation.max_date)
		end

feature -- Basic operations

	enable_edit (a_on: BOOLEAN) is
		do
			implementation.enable_edit (a_on)
			arrange_sensitiveness
		end

	set_date (a_date: ROSE_DATE) is
		do
			if a_date = Void and then is_voidable then
				is_void := True
				effective_set_date (extreme_date)
			else
				is_void := False
				effective_set_date (a_date)
			end
			arrange_sensitiveness
			change_actions.call ([])
		end

	set_format (a_format: STRING) is
			-- Sets the display based on 'a_format'
			-- Calling with Void resets control to default display.
		do
			implementation.set_format (a_format)
		end

	set_range (a_min, a_max: ROSE_DATE) is
			-- Set minimum and maximum date range constraint.
		do
			implementation.set_range (a_min, a_max)
		end

	remove_range is
			-- Remove minimum and maximum date restrictions.
		do
			implementation.remove_range
		end

	set_minimalistic (a_minimalistic: BOOLEAN) is
		do
			is_minimalistic := a_minimalistic
		end

	set_voidable (a_voidable: BOOLEAN) is
		do
			is_voidable := a_voidable
			if is_void and then not a_voidable then
				set_date (Void)
			end
		end

feature {EV_ANY, EV_ANY_I} -- Implementation

	implementation : ROSE_EV_DATE_FIELD_IMP
			-- Responsible for interaction with native graphics toolkit.

	replace_implementation (new_implementation: like implementation) is
			-- Replace `implementation' with `new_implementation'.
			-- The caller has complete responsibility for releasing any
			-- resources held by the old `implementation'.
			--| See `clone' for useage of `c_check_assert'.
		local
			temp: BOOLEAN
		do
			temp := {ISE_RUNTIME}.check_assert (False)
			implementation.disable_initialized
			new_implementation.set_interface(Current)
			implementation := new_implementation
			implementation.enable_initialized
			implementation.set_state_flag ({EV_ANY_I}.interface_default_create_called_flag, True)
			implementation.set_state_flag ({EV_ANY_I}.interface_is_initialized_flag, True)
			temp := {ISE_RUNTIME}.check_assert (temp)
		end

feature {NONE} -- Implementation

	is_void: BOOLEAN

	create_implementation is
			-- See `{EV_ANY}.create_implementation'.
		do
			create implementation.make
		end

	arrange_sensitiveness is
			-- Make this field insensitive if it shouldn't be modified.
		do
			if is_void or else not is_editable then
				if is_sensitive then disable_sensitive end
			else
				if not is_sensitive then enable_sensitive end
			end
		end

	effective_set_date (a_date: ROSE_DATE) is
		do
			implementation.set_system_time (to_date_time (a_date), 0)
		end

	date_to_rose_date (a_date: DATE): ROSE_DATE is
		do
			if a_date /= Void then
				create Result.make (a_date.year, a_date.month, a_date.day)
			end
		end

	to_rose_date (a_date: DATE_TIME): ROSE_DATE is
		do
			if a_date /= Void then
				create Result.make (a_date.year, a_date.month, a_date.day)
			end
		end

	to_date_time (a_date: ROSE_DATE): DATE_TIME is
			-- Convert `a_date' to a DATE_TIME if not Void,
			-- Returns "now" date-time otherwise
		do
			if a_date /= Void then
				create Result.make (a_date.year, a_date.month, a_date.day, a_date.hour, a_date.minute, a_date.second)
			else
				create Result.make_now
			end
		end

	extreme_date: ROSE_DATE is
		do
			if is_minimalistic then
				Result := minimum_date
				if Result = Void then
					create Result.make_min_date
				end
			else
				Result := maximum_date
				if Result = Void then
					create Result.make_max_date
				end
			end
		end

end
