indexing
	description: "EiffelVision factory, producing objects with most commonly used settings"
	author: "Zoran Simic"
	last_editor: "$Author: dz $"
	date: "$Date: 2014/10/27 $"
	revision: "$Revision: #16 $"
	path: "$File: //rose/source/kernel/infrastructure/framework/gui/rose_ev_factory.e $"

class
	ROSE_EV_FACTORY

inherit
	ROSE_EV_FACTORY_WIDGET
	ROSE_EV_FACTORY_CONTROL

feature -- Access

	new_window (a_title: STRING; a_icon: ANY; a_width, a_height: INTEGER): EV_TITLED_WINDOW is
			-- Return a new titled window with 'a_title', 'a_icon'
			-- and 'a_width','a_height' (minimum size)
		do
			create Result
			decorate_window (Result,a_title,a_icon,a_width,a_height)
		end

	new_dialog (a_title: STRING; a_icon: ANY; a_width, a_height: INTEGER): EV_DIALOG is
			-- Return a new titled window with 'a_title', 'a_icon'
			-- and 'a_width','a_height' (minimum size)
		do
			create Result
			decorate_window (Result,a_title,a_icon,a_width,a_height)
		end

	decorate_window (a_window: EV_TITLED_WINDOW; a_title: STRING; a_icon: ANY; a_width, a_height: INTEGER) is
			-- Decorate 'a_window' to represent the delivery where it's running from
		require
			valid_window: a_window /= Void
		local
			l_title: STRING
			l_string: STRING
			l_pixmap: EV_PIXMAP
		do
			l_title := a_title
			if l_title = Void or else l_title.is_empty then
				l_title := "-No title-"
			end
			a_window.set_title (decorated_title (l_title))
			l_string ?= a_icon
			if l_string /= Void then
				l_pixmap := new_pixmap_from_file (l_string)
			else
				l_pixmap ?= a_icon
			end
			if l_pixmap /= Void then
				a_window.set_icon_pixmap (l_pixmap)
			end
			if a_width > 0 then
				a_window.set_minimum_width (a_width)
			end
			if a_height > 0 then
				a_window.set_minimum_height (a_height)
			end
		end

	decorated_title (a_title: STRING): STRING
			-- Decorated title for a window (shows which environment app is running)
		require
			title_ok: a_title /= Void
		local
			l_env: STRING
		do
			create Result.make_empty
			if global.app.is_running_in_estudio then
				Result.append ("-- ")
			end
			Result.append_string (a_title)
			Result.append_string (" v")
			Result.append_string (global.app.version)
			l_env := global.app.configured_value ("/path/name")
			if l_env = Void or else l_env.is_empty then
				l_env := global.formatter.directory (global.app.site.path)
				l_env := global.formatter.file_name (l_env)
			end
			if l_env = Void or else l_env.is_empty then
				l_env := "?"
			end
			Result.append (" (")
			if not global.app.is_running_within_site then
				Result.append ("-> ")
			end
			Result.append_string (l_env)
			Result.append_character (')')
		ensure
			result_ok: Result /= Void and then not Result.is_empty
		end

feature -- Basic operations

	set_contextual_menu (a_field: EV_TEXT_COMPONENT; a_agent: PROCEDURE [ANY, TUPLE [EV_MENU]]) is
			-- Wrapper to set a right click menu on a_field with a_agent
		require
			field_ok: a_field /= Void
			agent_ok: a_agent /= Void
		do
			a_field.pointer_button_release_actions.extend (agent on_contextual_menu (a_field, a_agent, ?, ?, ?, ?, ?, ?, ?, ?))
		end

	box_extend_and_disable_expand (a_box: EV_BOX; a_widget: EV_WIDGET) is
			-- Extend a_box with a_widget and disable item expand
		require
			valid_box: a_box /= Void
			valid_widget: a_widget /= Void
		do
			a_box.extend (a_widget)
			a_box.disable_item_expand (a_widget)
		end

	set_return_as_tab (a_container: EV_CONTAINER) is
			-- Find all EV_TEXT_FIELDs in 'a_container' and set them up so that when user
			-- presses RETURN on them, the next one active is focused
		local
			l_texts: ROSE_LINEAR_ARRAY [EV_TEXT_FIELD]
			i: INTEGER
		do
			create l_texts.make_empty
			find_text_fields (a_container, l_texts)
			from
				i := l_texts.lower
			until
				i > l_texts.upper
			loop
				l_texts.item (i).key_release_actions.extend (agent on_return_as_tab (i, l_texts, ?))
				i := i + 1
			end
		end

feature {NONE} -- Implementation

	on_contextual_menu (a_field: EV_TEXT_COMPONENT; a_agent: PROCEDURE [ANY, TUPLE [EV_MENU]]; a_x, a_y, a_button: INTEGER; a_x_tilt, a_y_tilt, a_pressure: DOUBLE; a_screen_x, a_screen_y: INTEGER) is
			-- Called back when user clicks on a widget passed to 'set_contextual_menu'
		require
			field_ok: a_field /= Void
			agent_ok: a_agent /= Void
		local
			l_menu: EV_MENU
		do
			if a_button = 3 and then not Gui.is_picking then
				create l_menu
				a_agent.call ([l_menu])
				if l_menu.count > 0 then
					Gui.set_default_processing (a_field, False)
					l_menu.show
				end
			end
		end

	on_return_as_tab (a_index: INTEGER; a_texts: ROSE_LINEAR_ARRAY [EV_TEXT_FIELD]; a_key: EV_KEY) is
			-- Called back when user presses RETURN on EV_TEXT_FIELD in 'a_texts' with 'a_index'
		local
			l_next: EV_TEXT_FIELD
			i: INTEGER
		do
			if a_key.code = {EV_KEY_CONSTANTS}.key_enter then
			from
				i := a_index + 1
			until
				l_next /= Void or else i > a_texts.upper
			loop
				l_next := suitable_as_next_tabbed (a_texts.item (i))
				i := i + 1
			end
			from
				i := a_texts.lower
			until
				l_next /= Void or else i >= a_index
			loop
				l_next := suitable_as_next_tabbed (a_texts.item (i))
				i := i + 1
			end
			if l_next /= Void then
				l_next.set_focus
			end
			end
		end

	suitable_as_next_tabbed (a_text: EV_TEXT_FIELD): EV_TEXT_FIELD is
			-- Returns next suitable EV_TEXT_FIELD to focus
		local
			l_combo: EV_COMBO_BOX
		do
			l_combo ?= a_text
			if l_combo /= Void then
				if l_combo.is_sensitive then
					Result := l_combo
				end
			elseif a_text /= Void and then a_text.is_sensitive and then a_text.is_editable then
				Result := a_text
			end
		end

	find_text_fields (a_widget: EV_WIDGET; a_texts: ROSE_LINEAR_ARRAY [EV_TEXT_FIELD]) is
			-- Populate 'a_texts' with all EV_TEXT_FIELDs found contained in 'a_widget'.
		require
			texts_ok: a_texts /= Void
		local
			l_container: EV_CONTAINER
			l_text: EV_TEXT_FIELD
			l_linear: LINEAR [EV_WIDGET]
		do
			l_container ?= a_widget
			l_text ?= a_widget
			if l_container /= Void then
				from
					l_linear := l_container.linear_representation
					l_linear.start
				until
					l_linear.off
				loop
					find_text_fields (l_linear.item, a_texts)
					l_linear.forth
				end
			elseif l_text /= Void then
				a_texts.extend (l_text)
			end
		end

end
