note

	description:

		"Custom version of EV_GRID_IMP providing a few additional facilities."

	author: "Broos Nemanic"
	last_editor: "$Author: er $"
	date: "$Date: 2013/04/24 $"
	revision: "$Revision: #20 $"
	path: "$File: //rose/source/kernel/infrastructure/widgets/grid/rose_ev_grid_imp.e $"

class
	ROSE_EV_GRID_IMP

inherit
	EV_GRID_IMP
		export {ANY}
			set_foreground_color,
			set_background_color,
			vertical_scroll_bar,
			tree_subrow_indent,
			item_cell_indent,
			drawable,
			node_pixmap_width,
			tree_node_spacing,
			pointer_button_press_received,
			uses_row_offsets
		{RP_DATA_GRID}
			drawer,
			fixed,
			viewport_x_offset,
			viewport_y_offset,
			internal_client_x,
			internal_client_y,
			row_indexes_to_visible_indexes
		redefine
			drawer,
			drawer_internal,
			enable_tree,
			disable_tree,
			item_accepts_pebble,
			recompute_row_offsets
		end

	REFACTORING_HELPER

create
	make

feature -- Basic operations

	enable_tree is
			-- Redefined to work around a slow down in EV_GRID_I `enable_tree'.
			-- There is no reason to recompute the offsets if the tree is already enabled
			-- and it is causing some of the systems to scroll very slowly.
			-- This fix has been suggested to ISE and as soon as a version is available in
			-- which it is contained, then this redefinition may be removed.
		do
			if not is_tree_enabled then
				Precursor {EV_GRID_IMP}
			end
		end

	disable_tree is
			-- See comment in `enable_tree'.
		do
			if is_tree_enabled then
				Precursor {EV_GRID_IMP}
			end
		end

	item_accepts_pebble (a_item: attached EV_GRID_ITEM; a_pebble: attached ANY): BOOLEAN is
			-- Do any actions accept `a_pebble' for `a_item'.
			-- Redefined to work around a bug in the EV_GRID_I version which
			-- was not firing the drop actions for the grid when you dropped on a position
			-- where there was no item. Eiffel Software reports that this has been fixed
			-- so remove this redefinition when we receive a fixed version.
		local
			cur: CURSOR
			a_tuple: TUPLE [EV_GRID_ITEM, ANY]
			a_action: like item_drop_actions
		do
			if item_drop_actions_internal /= Void and then a_item /= Void then
				from
					a_action := item_drop_actions_internal
					cur := a_action.cursor
					a_tuple := [a_item, a_pebble]
					a_action.start
				until
					a_action.after or Result
				loop
					Result := a_action.item.valid_operands (a_tuple)
					if
						Result and then
						item_veto_pebble_function /= Void and then
						item_veto_pebble_function.valid_operands (a_tuple)
					then
						Result := Result and then item_veto_pebble_function.item (a_tuple)
					end
					a_action.forth
				end
				a_action.go_to (cur)
			end
		end

	enable_capture_light is
			-- `enable_capture' but forced to use the real windows capture, not the
			-- WEL_HOOK heavy capture. This is necessary to work around a bug in the heavy capture where
			-- if you call `enable_capture' with the left mouse button down, move the mouse out of your window
			-- and then release the left mouse button, the capture ends.
		local
			wel_window: WEL_WINDOW
		do
			wel_window ?= drawable.implementation
			wel_window.set_capture
		end

	disable_capture_light is
			-- `disable_capture' but forced to use the real windows capture, not the
			-- WEL_HOOK heavy capture. This is necessary to work around a bug in the heavy capture where
			-- if you call `enable_capture' with the left mouse button down, move the mouse out of your window
			-- and then release the left mouse button, the capture ends.
		local
			wel_window: WEL_WINDOW
		do
			wel_window ?= drawable.implementation
			wel_window.release_capture
		end

	has_capture_light: BOOLEAN is
			-- Is a light capture executing on `Current'?
		local
			wel_window: WEL_WINDOW
		do
			wel_window ?= drawable.implementation
			Result := wel_window.has_capture
		end

	locked_column_i_for_column (a_column: EV_GRID_COLUMN): EV_GRID_LOCKED_I is
			-- `Result' is internal locked column for column `a_column'.
		require
			a_column_not_void: a_column /= Void
			a_column_locked: a_column.is_locked
		do
			Result := a_column.implementation.locked_column
		ensure
			result_not_void: Result /= Void
		end

	locked_row_i_for_row (a_row: EV_GRID_ROW): EV_GRID_LOCKED_I is
			-- `Result' is internal locked row for row `a_row'.
		require
			a_row_not_void: a_row /= Void
			a_row_locked: a_row.is_locked
		do
			Result := a_row.implementation.locked_row
		ensure
			result_not_void: Result /= Void
		end

	disable_tabable_to is
			-- Make `is_tabable_to' `False'.
		do
			drawable.disable_tabable_to
		end

	drawer: attached ROSE_EV_GRID_DRAWER_I
			-- <Precursor>
		do
			check drawer_internal /= Void end
			Result := drawer_internal
		end

	drawer_internal: ROSE_EV_GRID_DRAWER_I
			-- object used to draw `Current'.

	data_grid: RP_DATA_GRID [ANY]
		-- Associated data grid if set.

	set_data_grid (a_data_grid: like data_grid) is
			-- Assign `a_data_grid' to `data_grid'.
		do
			data_grid := a_data_grid
		ensure
			data_grid_set: data_grid = a_data_grid
		end

	move_rows_for_sort (a_start_index: INTEGER; a_rows: ROSE_LINEAR_ARRAY[ARRAY[RP_DATA_GRID_CELL[ANY]]]) is
			-- Move rows contained in `a_rows' to mimic the order in `a_rows', with the first being at position
			-- `a_start_index'. All subrows recursively are moved.
		require
			valid_start_index: a_start_index > 0 and a_start_index <= row_count
			a_rows_not_void: a_rows /= Void

		local
			l_node: RP_GRID_NODE [ANY]
			l_grid_row_i, l_sorted_grid_row_i: EV_GRID_ROW_I
			l_internal_row_data: like internal_row_data
			l_rows: like rows
			l_subrow_count_recursive, l_new_row_offset, l_child_counter, i, l_sorted_grid_row_original_index, l_grid_row_index: INTEGER
		do
				-- We clone these attributes as we will be moving the contents
				-- and need to know the original states.
			l_internal_row_data := internal_row_data.twin
			l_rows := rows.twin

			l_new_row_offset := a_start_index
			from
				i := a_rows.lower
			until
				i > a_rows.upper
			loop
				l_node := a_rows.item (i).item (1).node
					-- Retrieve the next row that has been sorted. It is this and all of it's subrows recursively that
					-- we will now move.
				l_sorted_grid_row_i := l_node.row.implementation

					-- Store the original index of the sorted grid row. This will not actually change
					-- until we call `update_grid_row_indices', but this is for clarity as we will be half way
					-- through moving it internally.
				l_sorted_grid_row_original_index := l_sorted_grid_row_i.index

					-- Calculate the number of recursive subrows that this row has.
				l_subrow_count_recursive := l_sorted_grid_row_i.subrow_count_recursive

					-- Move the row and data internally within the grid. Note that
					-- we move from the copy `l_internal_row_data' to `internal_row_data' so we don't step on our own toes.
				rows.put_i_th (l_sorted_grid_row_i, l_new_row_offset)
				internal_row_data.put_i_th (l_internal_row_data.i_th (l_sorted_grid_row_original_index), l_new_row_offset)

				if l_subrow_count_recursive > 0 then
						-- We have subrows, so we must recursvively move all of these. Their
						-- position relative to `l_sorted_grid_row_i' is unchanged, but we had just moved `l_sorted_grid_row_internally'
						-- and must move these in conjunction.
					from
						l_child_counter := 1
					until
						l_child_counter > l_subrow_count_recursive
					loop
						l_new_row_offset := l_new_row_offset + 1

							-- Move the row and data internally.
							-- Again, we move from copies of the internal data structures.
						l_grid_row_i := l_rows.i_th (l_sorted_grid_row_original_index + l_child_counter)
						rows.put_i_th (l_grid_row_i, l_new_row_offset)
						internal_row_data.put_i_th (l_internal_row_data.i_th (l_grid_row_i.index), l_new_row_offset)
						l_child_counter := l_child_counter + 1
					end
				end

				l_new_row_offset := l_new_row_offset + + 1
				i := i + 1
			end
			l_grid_row_i := l_sorted_grid_row_i.parent_row_i
				-- As we have just re-ordered all of the rows from `rows', we need to update `subrows' of the parent (if any)
				-- to reflect the new order.
				-- This code makes the assumption that all of these rows were moved by a sort and therefore are
				-- contained by the same parent. If this is not the case then this update of the single parent
				-- subrows below is flawed.
			if l_grid_row_i /= Void then
				l_grid_row_i.subrows.wipe_out
				l_grid_row_index := l_grid_row_i.index
				from
					i := 1
				until
					i > l_grid_row_i.subrow_count_recursive
				loop
						-- As we moved all subrows recursively, not all rows need updating.
						-- Only update those contained in `l_grid_row_i' which is the parent of the sorted rows.
					if rows.i_th (l_grid_row_index + i).parent_row_i = l_grid_row_i then
						l_grid_row_i.subrows.extend (rows.i_th (l_grid_row_index + i))
					end
					i := i + 1
				end
				l_grid_row_i.update_subrow_indices (1)
			end

				-- Now update the main indices of the grid.
			update_grid_row_indices (a_start_index)
			set_vertical_computation_required (a_start_index)
			if data_grid.are_sticky_parents_enabled then
				data_grid.set_are_sticky_parents_enabled (False)
				data_grid.set_are_sticky_parents_enabled (True)
			end
			redraw
		ensure
			row_count_unchanged: row_count = old row_count
		end

	recompute_row_offsets (an_index: INTEGER)
			-- Recompute contents of `row_offsets' from row index
			-- `an_index' to `row_count'.
		local
			current_row_offset, j, k: INTEGER
			current_item: EV_GRID_ROW_I
			index: INTEGER
			visible_count: INTEGER
			row_index: INTEGER
			l_row_count: INTEGER
			just_looped: BOOLEAN
			original_row_index: INTEGER
			l_row_indexes_to_visible_indexes: EV_GRID_ARRAYED_LIST [INTEGER]
			l_visible_indexes_to_row_indexes: EV_GRID_ARRAYED_LIST [INTEGER]
			l_row_offsets: EV_GRID_ARRAYED_LIST [INTEGER]
			l_rows: EV_GRID_ARRAYED_LIST [EV_GRID_ROW_I]
			l_is_row_height_fixed: BOOLEAN
			l_row_height: INTEGER
			l_original_computed_visible_row_count: INTEGER
			l_visible_row_count: INTEGER
		do
			l_original_computed_visible_row_count := computed_visible_row_count
			original_row_index := rows.index

			if row_count > 0 and is_tree_enabled then
					-- We only find the parent row when `Current' is not empty and we have the tree enabled.
				index := row_internal (an_index).parent_row_root.index
			else
					-- `Current' is empty, so simply keep the same index.
				index := an_index
			end
			if uses_row_offsets then
					-- Only perform recomputation if the rows do not all have the same height
					-- or there is tree functionality enabled. Otherwise, we do not need to
					-- use `row_offsets' and we can perform a shortcut.
				if row_offsets = Void then
					create row_offsets
					create row_indexes_to_visible_indexes
					create visible_indexes_to_row_indexes
				else
						-- Retrieve the count of visible rows calculated last time this feature was called.
					l_visible_row_count := visible_row_count
					if index > 1 then
							-- Check that `index' is within the previous range of computed visible row indexes. Note that
							-- this does not check it is was previously displayed, but just between the lowest and the highest that were
							-- previously displayed.
						if l_visible_row_count > 0 and then index <= visible_indexes_to_row_indexes.i_th (l_visible_row_count) then

							fixme ("This feature has been redefined and the following lines commented out to work around a bug when hiding/showing rows. The logic/optimization is not correct.")
							fixme ("The new code works, but is non-optimal. This should be re-implemented as a binary search on `visible_indexes_to_row_indexes' in order to calculate where")
							fixme ("the last row we can start recomputing from lies.")
--								-- Now we check that the row index we are trying to compute from was previously visible in `Current'.
--								-- If it was not visible, then we are unable to assertain the visible row count at this index, which we need
--								-- in order to continue computing from this index.
--								-- If the row at this index was not displayed, then we start a re-compute from the row at index 1. We could
--								-- find the next visible row before the one we wanted, but there is no way to determine this without iterating
--								-- so we simply start from the top.
--							if row_indexes_to_visible_indexes.i_th (index) > 0 and then visible_indexes_to_row_indexes.i_th (row_indexes_to_visible_indexes.i_th (index)) + 1 = index then
--								visible_count := row_indexes_to_visible_indexes.i_th (index)
--							else
							index := 1
							visible_count := 0
--							end
						else
								-- In this situation, we are adding a row that has not already been computed.
								-- Therefore, `visible_count' is set to the number of rows that was previously
								-- computed during the last call to this feature.
							visible_count := l_visible_row_count
						end
					else
							-- In this case, the feature has already been called when there are
							-- no rows in the grid. So, we reset these attributes to the start.
						visible_count := 0
					end
					current_row_offset := row_offsets @ (index)
				end

					-- Ensure we enlarge our data structures to accomodate the total number of rows.
					-- We do not reduce the size of these lists to avoid the performance overhead.
				if row_offsets.count < rows.count + 1 then
					row_offsets.resize (rows.count + 1)
					row_indexes_to_visible_indexes.resize (rows.count + 1)
					visible_indexes_to_row_indexes.resize (rows.count + 1)
				end

				from
					row_index := index

						-- We assign these attributes of the class to locals as this
						-- provides a speed improvement. It appears that this code is
						-- 20% faster when this is performed. It was tested with 1,000,000
						-- rows, expanding and collapsing the first. Julian
					l_row_count := rows.count
					l_row_indexes_to_visible_indexes := row_indexes_to_visible_indexes
					l_visible_indexes_to_row_indexes := visible_indexes_to_row_indexes
					l_rows := rows
					l_row_offsets := row_offsets
					l_is_row_height_fixed := is_row_height_fixed
					l_row_height := row_height
					just_looped := True

				until
					row_index > l_row_count
				loop
					current_item := l_rows.i_th (row_index)
					if current_item /= Void and then ((current_item.subrow_count > 0 and not current_item.is_expanded) or not current_item.is_show_requested) then
						from
							j := row_index
							k := j + current_item.subrow_count_recursive + 1
						until
							j = k
						loop
							l_row_offsets.put_i_th (current_row_offset, j)
							j := j + 1
						end
						if current_item.is_show_requested then
							l_row_indexes_to_visible_indexes.put_i_th (visible_count, row_index)
							l_visible_indexes_to_row_indexes.put_i_th (row_index, visible_count + 1)
						end
						row_index := k
					else
						l_row_offsets.put_i_th (current_row_offset, row_index)
						l_row_indexes_to_visible_indexes.put_i_th (visible_count, row_index)
						l_visible_indexes_to_row_indexes.put_i_th (row_index, visible_count + 1)
						row_index := row_index + 1
					end
					if current_item = Void or else current_item.is_show_requested then
						visible_count := visible_count + 1
					if current_item /= Void and not l_is_row_height_fixed then
						current_row_offset := current_row_offset + current_item.height
					else
							-- Use the default height here.
						current_row_offset := current_row_offset + l_row_height
					end
					end
				end
				if visible_count /= l_original_computed_visible_row_count then
						-- It is essential that we only modify `computed_visible_row_count'
						-- if it has actually changed. Otherwise, we may not have a correct value.
					computed_visible_row_count := visible_count
				end

					-- A final position is always stored in `row_offsets' which may be
					-- queried to determine the total height of all rows.
				l_row_offsets.put_i_th (current_row_offset, row_index)
			else
				row_offsets := Void
			end


				-- Now move the virtual position so that it is restricted to the maximum
				-- row position. This is used so that when removing rows,  `virtual_x_position' remains valid.
			restrict_virtual_y_position_to_maximum

			if virtual_size_changed_actions_internal /= Void then
				virtual_size_changed_actions_internal.call ([virtual_width, virtual_height])
			end
			rows.go_i_th (original_row_index)
		end


end
