indexing
	description: "Platform specific GUI code"
	author: "Zoran Simic"
	date: "$Date: 2013/04/24 $"
	revision: "$Revision: #11 $"

class
	ROSE_EV_PLATFORM_SPECIFICS

inherit
	EV_CELL

feature -- Access

	get_widget_handle (a_widget: EV_WIDGET): POINTER is
			-- Get the platform specific handle out of any EV_WIDGET
		local
			l_implementation: EV_WIDGET_IMP
		do
			if a_widget /= Void then
				l_implementation ?= a_widget.implementation
			end
			if l_implementation /= Void then
				Result := l_implementation.wel_item
			end
		end

	is_picking (a_app: EV_APPLICATION): BOOLEAN is
			-- Is a pick-and-drop operation currently in progress?
		do
			if a_app /= Void then
				Result := imp.is_picking (a_app.implementation)
			end
		end

feature -- Basic operations

	flash_window (a_window: EV_WINDOW) is
			-- Flash 'a_window' to draw user attention to that window.
		local
			s: WEL_FLASH_WINDOW_INFO
			p: POINTER
		do
			if a_window /= Void and then not a_window.has_focus then
				p := get_widget_handle (a_window)
				if p /= default_pointer then
					create s.make
					s.set_handle (p)
					s.set_flags (s.FLASHW_TIMERNOFG + s.FLASHW_TRAY)
					s.set_count (5)
					s.call_FlashWindowEx
				end
			end
		end

	set_default_processing (a_widget: EV_WIDGET; a_on: BOOLEAN) is
			-- Set default processing on or off for 'a_widget'.
		local
			l_implementation: WEL_RETURN_VALUE
		do
			if a_widget /= Void then
				l_implementation ?= a_widget.implementation
			end
			if l_implementation /= Void and then l_implementation.default_processing /= a_on then
				l_implementation.set_default_processing (a_on)
			end
		end

	show_border (a_text_field: EV_TEXT_FIELD; a_show: BOOLEAN) is
			-- Show/hide 3D border from 'a_text_field'
		do
			if a_text_field /= Void then
				if a_show then
				else
					a_text_field.implementation.hide_border
				end
			end
		end

feature {NONE} -- Implementation

	imp: ROSE_EV_PLATFORM_SPECIFICS_IMP is
			-- Needed in order to be able to use features exported only to EV_ANY_IMP
		once
			create Result.make
		end

end
