indexing
	description: "Interface to Excel via COM"
	author: "Pierre Nallet"
	last_editor: "$Author: e2 $"
	date: "$Date: 2014/12/09 $"
	revision: "$Revision: #21 $"
	path: "$File: //rose/source/kernel/infrastructure/windows/classic/rose_excel_application.e $"

class ROSE_EXCEL_APPLICATION

create

	make_simple,
	make_simple_no_release,
	make,
	make_simple_formatted,
	make_formatted,
	make_from_file,
	make_single_workbook,
	make_single_workbook_general

feature {NONE} -- Implementation

	make (a_arrays: ARRAY [TUPLE [ARRAY2 [ANY], STRING]]) is
			-- Make wih a array of data
		require
			valid_arrays: a_arrays /= Void
		local
			i: INTEGER
			l_array: ARRAY2 [ANY]
			l_name: STRING
		do
			create excel.make
			from
				i := a_arrays.lower
			until
				i > a_arrays.upper
			loop
				l_array ?= a_arrays.item (i).item (1)
				l_name ?= a_arrays.item (i).item (2)
				excel.add_workbook (l_array, l_name)
				i := i + 1
			end
			excel.release
		end

	make_single_workbook (a_arrays: ARRAY [TUPLE [ARRAY2 [ANY], STRING]]) is
			-- Make single workbook
		do
			make_single_workbook_general (a_arrays, True)
		end

	make_single_workbook_general (a_arrays: ARRAY [TUPLE [ARRAY2 [ANY], STRING]]; a_release_excel: BOOLEAN) is
			-- Make single workbook general
		local
			i: INTEGER
			l_array: ARRAY2 [ANY]
			l_name: STRING
		do
			create excel.make
			if not a_arrays.is_empty then
				l_array ?= a_arrays.item (a_arrays.lower).item (1)
				l_name ?= a_arrays.item (a_arrays.lower).item (2)
				excel.add_workbook (l_array, l_name)
				from
					i := a_arrays.lower + 1
				until
					i > a_arrays.upper
				loop
					l_array ?= a_arrays.item (i).item (1)
					l_name ?= a_arrays.item (i).item (2)
					excel.add_worksheet (l_array, l_name)
					i := i + 1
				end
			end
			if a_release_excel then
				excel.release
			end
		end

	make_from_file (a_filename: STRING) is
			-- Create from file
		do
			create excel.make
			excel.workbooks.open (a_filename)
		end

	make_simple (a_array: ARRAY2 [ANY]; a_name: STRING) is
			-- Make with one worksheet with name 'a_name' and cell values from 'a_array'. Then release the application.
		do
			create excel.make
			excel.add_workbook (a_array, a_name)
			excel.release
		end

	make_simple_no_release (a_array: ARRAY2 [ANY]; a_name: STRING) is
			-- Make with one worksheet with name 'a_name' and cell values from 'a_array'. Does not release the application.
		do
			create excel.make
			excel.add_workbook (a_array, a_name)
		end

	make_simple_formatted (a_array: ARRAY2 [ANY]; a_name: STRING; a_formats: ARRAY [TUPLE [STRING, STRING]]) is
			--make with specified formats
		local
			i: INTEGER
			l_range, l_format: STRING
		do
			create excel.make
			excel.set_automatic_formatting (True)
			excel.add_workbook (a_array, a_name)
			if a_formats /= Void then
				from
					i := a_formats.lower
				until
					i > a_formats.upper
				loop
					l_range ?= a_formats.item (i).item (1)
					l_format ?= a_formats.item (i).item (2)
					excel.set_format (l_range, l_format)
					i := i + 1
				end
			end
		end

	make_formatted (a_arrays: ARRAY [TUPLE [ARRAY2 [ANY], STRING, ARRAY [TUPLE [STRING, STRING]]]]) is
			-- make with supplied formatting applied for each array
		local
			l_index, i: INTEGER
			l_range, l_format: STRING
			l_formats: ARRAY [TUPLE [STRING, STRING]]
			l_array: ARRAY2 [ANY]
			l_name: STRING
		do
			create excel.make
			excel.set_automatic_formatting (True)
			from
				l_index := a_arrays.lower
			until
				l_index > a_arrays.upper
			loop
				l_array ?= a_arrays.item (l_index).item (1)
				l_name ?= a_arrays.item (l_index).item (2)
				l_formats ?= a_arrays.item (l_index).item (3)
				excel.add_workbook (l_array, l_name)
				if l_formats /= Void then
					from
						i := l_formats.lower
					until
						i > l_formats.upper
					loop
						l_range ?= l_formats.item (i).item (1)
						l_format ?= l_formats.item (i).item (2)
						excel.set_format (l_range, l_format)
						i := i + 1
					end
				end
				l_index := l_index + 1
			end
			excel.release
		end

feature -- Access

	active_sheet: EXCEL_WORKSHEET is
			-- Active sheet
		do
			Result := excel.active_sheet
		end

	range_string (a_upper_left_row, a_upper_left_column, a_lower_right_row, a_lower_right_column: INTEGER): STRING is
			-- String representation for range identified by bounds in parameters
		do
			Result := excel.range_string (a_upper_left_row, a_upper_left_column, a_lower_right_row, a_lower_right_column)
		end

	workbooks: EXCEL_WORKBOOKS is
			-- Pages in the Excel file
		do
			Result := excel.workbooks
		end

	imported_worksheet_by_name (a_name: STRING; a_first_column, a_first_row, a_width, a_height_upper_bound: INTEGER): ARRAY2 [ANY] is
			--	Imported version of `a_worksheet' in memory
		do
			Result := excel.imported_worksheet_by_name (a_name, a_first_column, a_first_row, a_width, a_height_upper_bound)
		end

	active_data (a_first_column, a_first_row, a_width, a_height_upper_bound: INTEGER): ARRAY2 [ANY] is
			--	Imported version of the active sheet in memory
		do
			Result := excel.imported_worksheet (excel.active_sheet, a_first_column, a_first_row, a_width, a_height_upper_bound)
		end

	charts: EXCEL_CHARTS is
			-- List of charts in the current workbook
		do
			Result := excel.charts
		end

	range_by_name (a_worksheet: EXCEL_WORKSHEET; s: STRING): EXCEL_RANGE is
			-- Get cells intervals from 'a_worksheet' and identified by 's'
		do
			Result := a_worksheet.range_by_name (s)
		end

	get_array2 (a_range: EXCEL_RANGE): ARRAY2 [ANY] is
			-- Get values in 'a-range'
		do
			Result := a_range.get_array2
		end

	close_workbook (a_number: INTEGER; a_save: BOOLEAN) is
			-- Close i-th workbook and save if requested
		local
			l_variant: COM_VARIANT
		do
			create l_variant
			l_variant.init
			l_variant.set_boolean (a_save)
			workbooks.item (a_number).close (l_variant, Void, Void)
			l_variant.release
			workbooks.item (a_number).release
		end

feature -- Basic operations

	add_worksheet_formatted (a_array: ARRAY2 [ANY]; a_title: STRING; a_formats: ARRAY [TUPLE [STRING, STRING]]) is
			-- Add a worksheet with data
		local
			i: INTEGER
			s1, s2: STRING
		do
			excel.add_worksheet (a_array, a_title)
			from
				i := a_formats.lower
			until
				i > a_formats.upper
			loop
				s1 ?= a_formats [i].item (1)
				s2 ?= a_formats [i].item (2)
				excel.set_format (s1, s2)
				i := i + 1
			end
		end

	save_as (a_pathname: STRING) is
			-- Save (first workbook?) under 'a_pathname' filename
		local
			l_variant: COM_VARIANT
		do
			if excel.p = default_pointer then
				create excel.make
			end
			excel.workbooks.item (1).save_as (a_pathname)
			create l_variant
			l_variant.init
			l_variant.set_boolean (False)
			excel.workbooks.item (1).close (l_variant, Void, Void)
			excel.quit
			excel.release
		end

	save_as_2003 (a_pathname: STRING) is
			-- Save to Excel 2003 XLS format (binary, not XML)
		local
			l_variant: COM_VARIANT
		do
			if excel.p = default_pointer then
				create excel.make
			end
			excel.workbooks.item (1).save_as_format (a_pathname, xlWorkbookNormal)
			create l_variant
			l_variant.init
			l_variant.set_boolean (False)
			excel.workbooks.item (1).close (l_variant, Void, Void)
			excel.quit
			excel.release
		end

	close_and_quit is
			-- Close current spreadsheet and quit Excel executable
		local
			l_variant: COM_VARIANT
		do
			if attached excel.workbooks as l_workbooks and then workbooks.count > 0 then
				create l_variant
				l_variant.init
				l_variant.set_boolean (False)
				l_workbooks.item (1).close (l_variant, Void, Void)
			end
			excel.quit
		end

	set_format (a_range, a_format: STRING) is
			-- Set format for a range
		do
			excel.set_format (a_range, a_format)
		end

	set_display_alerts (a_value: BOOLEAN) is
			-- Default:True. Microsoft Excel displays certain alerts and messages while a macro is running.
		do
			excel.set_boolean_property (excel.get_id ("DisplayAlerts"), a_value)
		end

	quit is
			-- Terminate Excel executable
		do
			excel.quit
		end

	release is
			-- Release the excel application object
		do
			excel.release
		end

feature {NONE} -- Implementation

	excel: EXCEL_APPLICATION
			-- Excel application

feature {NONE} -- Constants

	xlWorkbookNormal: INTEGER = -4143
		-- Identifier for BIFF8 Excel 2003 file format

end
