note

	description:

		"Exponential operator"

	author: "Shujing Li"
	last_editor: "$Author: it $"
	date: "$Date: 2013/04/12 $"
	revision: "$Revision: #16 $"
	path: "$File: //rose/source/kernel/math/rose_exponential_operator.e $"

class
	ROSE_EXPONENTIAL_OPERATOR [reference G]
inherit
	ROSE_UNARY_OPERATOR[G]
	DOUBLE_MATH

	ROSE_IEEE_UNDEFINED

create
	make
feature  -- Access

	real_value (a_left : ROSE_EXPRESSION[G];a_g : G) : REAL is
			-- <Precursor>
		do
			Result := numeric_value (a_left, a_g).truncated_to_real
		end
	double_value (a_left : ROSE_EXPRESSION[G];a_g : G) : DOUBLE is
			-- <Precursor>
		do
			Result := numeric_value(a_left, a_g)
		end
	integer_value (a_left : ROSE_EXPRESSION[G];a_g : G) : INTEGER is
			-- <Precursor>
		do
			Result := numeric_value(a_left, a_g).rounded
		end
	boolean_value (a_left : ROSE_EXPRESSION[G];a_g : G) : BOOLEAN is
			-- <Precursor>
		do
			Result := True
		end

	numeric_value(a_left : ROSE_EXPRESSION[G];a_g : G) : DOUBLE is
			-- <Precursor>
		local
			l_value : DOUBLE
		do
			l_value := a_left.numeric_value(a_g)
			if defined_double(l_value) then
				Result := exp (l_value)
			else
				Result := missing_double
			end
		end

	value_in_market_currency (a_left : ROSE_EXPRESSION[G];a_market : MARKET;a_currency : PERSISTENT_CURRENCY;a_g : G;a_date : ROSE_DATE) : DOUBLE is
			-- <Precursor>
		local
			l_value : DOUBLE
		do
			l_value := a_left.value_in_market_currency(a_market,a_currency,a_g,a_date)
			if defined_double(l_value) then
				Result := exp (l_value)
			else
				Result := missing_double
			end
		end

	computed_double(a_left : ROSE_EXPRESSION[G]; a_g : G; a_agent : like access_agent_template) : DOUBLE is
			-- <Precursor>
		do
			Result := computed_numeric_value(a_left,a_g,a_agent)
		end
	computed_real(a_left : ROSE_EXPRESSION[G]; a_g : G; a_agent : like access_agent_template) : REAL is
			-- <Precursor>
		do
			Result := computed_numeric_value (a_left, a_g, a_agent).truncated_to_real
		end
	computed_integer(a_left : ROSE_EXPRESSION[G]; a_g : G; a_agent : like access_agent_template) : INTEGER is
			-- <Precursor>
		do
			Result := computed_numeric_value(a_left,a_g,a_agent).rounded
		end
	computed_numeric_value(a_left : ROSE_EXPRESSION[G]; a_g : G; a_agent : like access_agent_template) : DOUBLE is
			-- <Precursor>
		local
			l_value: DOUBLE
		do
			l_value := a_left.computed_numeric_value(a_g,a_agent)
			if is_defined_double (l_value) then
				Result := exp (l_value)
			else
				Result := undefined_double
			end
		end
	computed_boolean(a_left : ROSE_EXPRESSION[G]; a_g : G; a_agent : like access_agent_template) : BOOLEAN is
			-- <Precursor>
		do
			Result := True
		end
	computed_string(a_left : ROSE_EXPRESSION[G]; a_g : G; a_agent : like access_agent_template) : STRING is
			-- <Precursor>
		do
		end
	computed_value(a_left : ROSE_EXPRESSION[G]; a_g : G; a_agent : like access_agent_template) : ANY  is
			-- Computed value
		do
		end
	generated_double(a_left : ROSE_EXPRESSION[G]; a_g : G; a_agent : like access_agent_template) : DOUBLE is
			-- <Precursor>
		do
			Result := generated_numeric_value(a_left,a_g,a_agent)
		end
	generated_real(a_left : ROSE_EXPRESSION[G]; a_g : G; a_agent : like access_agent_template) : REAL is
			-- <Precursor>
		do
			Result := generated_numeric_value (a_left, a_g, a_agent).truncated_to_real
		end
	generated_integer(a_left : ROSE_EXPRESSION[G]; a_g : G; a_agent : like access_agent_template) : INTEGER is
			-- <Precursor>
		do
			Result := generated_numeric_value(a_left,a_g,a_agent).rounded
		end
	generated_numeric_value(a_left : ROSE_EXPRESSION[G]; a_g : G; a_agent : like access_agent_template) : DOUBLE is
			-- <Precursor>
		local
			l_value: DOUBLE
		do
			l_value := a_left.generated_numeric_value(a_g,a_agent)
			if is_defined_double (l_value) then
				Result := exp (l_value)
			else
				Result := undefined_double
			end
		end
	generated_boolean(a_left : ROSE_EXPRESSION[G]; a_g : G; a_agent : like access_agent_template) : BOOLEAN is
			-- <Precursor>
		do
			Result := True
		end
	generated_string(a_left : ROSE_EXPRESSION[G]; a_g : G; a_agent : like access_agent_template) : STRING is
			-- <Precursor>
		do
		end
	generated_date(a_left : ROSE_EXPRESSION[G]; a_g : G; a_agent : like access_agent_template) : ROSE_DATE is
			-- <Precursor>
		do
		end

	character_value (a_left : ROSE_EXPRESSION[G];a_g : G) : CHARACTER is
			-- <Precursor>
		do
		end

	string_value (a_left: ROSE_EXPRESSION[G];a_g : G) : STRING is
			-- <Precursor>
		do
		end
	date_value (a_left : ROSE_EXPRESSION[G];a_g : G) : ROSE_DATE is
			-- <Precursor>
		do
		end
	date_time_value (a_left : ROSE_EXPRESSION[G];a_g : G) : ROSE_DATE is
			-- <Precursor>
		do
		end

	computed_units (a_left: ROSE_UNITS): ROSE_UNITS is
			-- <Precursor>
		do
			Result := a_left
		end
	units_in_currency (a_left : ROSE_EXPRESSION[G];a_currency : PERSISTENT_CURRENCY;a_entity : G) : ROSE_UNITS is
			-- <Precursor>
		do
			Result := a_left.units_in_currency(a_currency,a_entity)
		end

end
