note

	description:

		"Rose Expression interface"

	author: "Mark Howard"
	last_editor: "$Author: it $"
	date: "$Date: 2014/03/14 $"
	revision: "$Revision: #182 $"
	path: "$File: //rose/source/kernel/math/rose_expression.e $"

deferred class ROSE_EXPRESSION [reference G]

inherit

	ROSE_UNITS_ENVIRONMENT
		redefine
			default_create
		end

	RP_HASHABLE
		redefine
			default_create
		end

	ROSE_RECORD_ACCESS
		rename
			data_value as last_double
		redefine
			default_create
		end

	ROSE_CONSTANT_VALUES
		redefine
			default_create
		end

	DEBUG_OUTPUT
		redefine
			default_create
		end

	EXPRESSION_MATH [G]
		redefine
			default_create
		end

	ROSE_PROCESS_TIMER_COLLECTION_AWARE
		redefine
			default_create
		end

	VARIABLE_DESCRIPTIONS_AWARE
		redefine
			default_create
		end

	ROSE_EXPRESSION_SNAPSHOT_AWARE
		redefine
			default_create
		end

feature {NONE} -- Initialization

	default_create is
			-- Ensure invariant for descendants who specify `default_create' as a creation procedure..
		do
			set_expression_index
		end

feature -- Initialization

	make_risk (a_name: STRING; a_mnemonic: STRING; a_mnemonic8: STRING; a_parent: STRING; a_j_index: INTEGER; a_risk_index: INTEGER; a_security: INTEGER; a_units: STRING; a_lower_winsor: DOUBLE; a_upper_winsor: DOUBLE; a_default_mean: DOUBLE; a_default_standard_deviation: DOUBLE; a_expression: STRING; a_scale: STRING; a_stock_or_flow: CHARACTER; a_target_name: STRING; a_weighting: STRING; a_group_name: STRING; a_variable_index: INTEGER) is
			-- Creation procedure, don't use directly
		local
			l_record: VARIABLE_DESCRIPTION_RECORD
		do
			create l_record.make_risk (a_name, a_mnemonic, a_mnemonic8, a_parent, a_j_index, a_risk_index, a_security, a_units, a_lower_winsor, a_upper_winsor, a_default_mean, a_default_standard_deviation, a_expression, Void, a_scale, a_stock_or_flow, a_target_name, a_weighting, a_group_name, a_variable_index)
			make (l_record)
		end

	make_mondat (a_name: STRING; a_mnemonic: STRING; a_result_string: STRING; a_parent: STRING; a_j_index: INTEGER; a_gv_index: INTEGER; a_spcs_index: INTEGER; a_risk_index: INTEGER; a_security: INTEGER; a_units: STRING; a_type: STRING; a_scale: STRING; a_target: STRING; a_stock_or_flow: CHARACTER; a_target_name: STRING; a_weighting: STRING; a_group_name: STRING; a_variable_index: INTEGER) is
			-- Creation procedure, don't use directly
		local
			l_record: VARIABLE_DESCRIPTION_RECORD
		do
			create l_record.make_mondat (a_name, a_mnemonic, a_result_string, a_parent, a_j_index, a_gv_index, a_spcs_index, a_risk_index, a_security, a_units, a_type, Void, a_scale, a_target, a_stock_or_flow, a_target_name, a_weighting, a_group_name, a_variable_index)
			make (l_record)
		end

	make_mondex (a_name: STRING; a_mnemonic: STRING; a_result_string: STRING; a_parent: STRING; a_j_index: INTEGER; a_gv_index: INTEGER; a_spcs_index: INTEGER; a_risk_index: INTEGER; a_security: INTEGER; a_units: STRING; a_type: STRING; a_scale: STRING; a_target: STRING; a_stock_or_flow: CHARACTER; a_target_name: STRING; a_group_name: STRING; a_variable_index: INTEGER) is
			-- Creation procedure, don't use directly
		local
			l_record: VARIABLE_DESCRIPTION_RECORD
		do
			create l_record.make_mondex (a_name, a_mnemonic, "mondex", a_result_string, a_parent, a_j_index, a_gv_index, a_spcs_index, a_risk_index, a_security, a_units, a_type, Void, a_scale, a_target, a_stock_or_flow, a_target_name, a_group_name, a_variable_index)
			make (l_record)
		end

	make_valdata (a_name: STRING; a_mnemonic: STRING; a_parent: STRING; a_j_index: INTEGER; a_gv_index: INTEGER; a_spcs_index: INTEGER; a_security: INTEGER; a_units: STRING; a_type: STRING; a_target: STRING; a_stock_or_flow: CHARACTER; a_target_name: STRING; a_scale: STRING; a_group_name: STRING; a_variable_index: INTEGER) is
			-- Creation procedure, don't use directly
		local
			l_record: VARIABLE_DESCRIPTION_RECORD
		do
			create l_record.make_valdata (a_name, a_mnemonic, a_parent, a_j_index, a_gv_index, a_spcs_index, a_security, a_units, a_type, Void, a_target, a_stock_or_flow, a_target_name, a_scale, a_group_name, a_variable_index)
			make (l_record)
		end

	make (a_variable_description: like variable_description) is
			-- Creation procedure
		local
			l_type: STRING
		do
			set_expression_index
			if a_variable_description /= Void then
				variable_description := a_variable_description.twin
				mnemonic := variable_description.mnemonic.as_lower
				mnemonic.prune_all (' ')
				if variable_description.name = Void then
					name := ""
				else
					name := variable_description.name.twin
					name.prune_all_trailing (' ')
					name.to_lower
				end
				variable_index := variable_description.variable_index
				l_type := variable_description.type
			end
			if l_type = Void then
				type := Double_type
			else
				l_type.to_lower
				if l_type.is_equal ("real") then
					type := Real_type
				elseif l_type.is_equal ("double") then
					type := Double_type
				elseif l_type.is_equal ("integer") then
					type := Integer_type
				elseif l_type.is_equal ("character") then
					type := Character_type
				elseif l_type.is_equal ("boolean") then
					type := Boolean_type
				elseif l_type.is_equal ("string") then
					type := String_type
				elseif l_type.is_equal ("date") then
					type := Date_type
				elseif l_type.is_equal ("date_time") then
					type := Date_time_type
				else
					type := Double_type
				end
			end
		end

	make_tmp (a_name, a_type: STRING) is
			-- creation procedure
		local
			l_description: VARIABLE_DESCRIPTION_RECORD
		do
			create l_description
			l_description.set_name (a_name)
			l_description.set_mnemonic (a_name)
			l_description.set_type (a_type)
			make (l_description)
		end

feature -- Identification

	name: STRING
			-- Expression name

	hash_code: INTEGER is
			-- Hash code
		do
			Result := name.hash_code
		end

	keys_equal (a_other: RP_HASHABLE): BOOLEAN is
			-- <Precursor>
		do
			Result := attached {ROSE_EXPRESSION [G]} a_other as l_other and then (variable_description ~ l_other.variable_description)
		end

	type: INTEGER
			--preferred output type

	mnemonic: STRING
			-- From the variable_description.

	variable_description: VARIABLE_DESCRIPTION
			-- From the variable_description_table in rose_entity_xxxground.

	variable_index: INTEGER

	target_name: STRING is
			--what is the target name of the variable description
		do
			if variable_description /= Void then
				Result := variable_description.target_name
			end
		end

feature -- Basic operations

	set_name (a_name: STRING) is
			-- Set name to 'a_name'
		do
			name := a_name
		end

	set_mnemonic (a_mnemonic: STRING) is
			-- Set name to 'a_name'
		do
			mnemonic := a_mnemonic
		end

feature {NONE} -- Cacheing

	last_target: ?G

	last_real: REAL

	last_integer: INTEGER

	moments_cache: ROSE_MULTI_KEYED_TABLE [ROSE_MOMENTS] is
			--cache for moments, misguided
		once
			create Result.make (10)
		end

feature -- Basic Retrieval

	currency (a_target: like expression_target): PERSISTENT_CURRENCY is
			--currency for a_target
		deferred
		end

	generated_currency (a_target: like expression_target; a_agent: like access_agent_template): PERSISTENT_CURRENCY is
			--currency for a_target using an access_control_agent
		do
			Result := currency (a_target)
		end

	unitized_value (a_target: like expression_target; a_agent: like access_agent_template): ROSE_UNITIZED_VALUE [DOUBLE] is
			-- 'a_agent' is used in the descendant.
		do
			create Result.make_with_units (double_value (a_target), units (a_target))
		end

	generated_unitized_value (a_target: like expression_target; a_agent: like access_agent_template): DATED_UNITIZED_VALUE [DOUBLE] is
			-- generated unitized value
		do
			if a_agent = Void then
				create Result.make_with_units (double_value (a_target), units (a_target))
			else
				Result := a_agent.unitized_double (entity_date_target (a_target), Current)
				if Result = Void then
					create Result.make_with_units (generated_double (a_target, a_agent), computed_units (a_target, a_agent))
				end
			end
		end

	real_value (a_target: like expression_target): REAL is
			-- real value
		require
			target_not_void: a_target /= Void
		do
			if last_target = a_target then
				Result := last_real
			else
				last_target := a_target
				last_real := get_real_value (a_target)
				Result := last_real
			end
		end

	boolean_value (a_target: like expression_target): BOOLEAN is
			-- Boolean value for 'a_target'
		require
			target_not_void: a_target /= Void
		do
			Result := double_value (a_target) /= 0
		end

	string_value (a_target: like expression_target): STRING is
			-- String value for 'a_target'
		require
			target_not_void: a_target /= Void
		do
		end

	date_value (a_target: like expression_target): ROSE_DATE is
			-- Date value for 'a_target'
		require
			target_not_void: a_target /= Void
		do
		end

	decimal_value (a_target: like expression_target): RP_DECIMAL is
			-- Decimal value
		require
			target_not_void: a_target /= Void
		do
			Result := double_value (a_target)
		end

	double_value (a_target: like expression_target): DOUBLE is
			-- Value of `Current' as a double;
			-- Value will be retrieved (if available) rather than computed.
		require
			target_not_void: a_target /= Void
		do
			if last_target = a_target then
				Result := last_double
			else
				last_target := a_target
				last_double := get_double_value (a_target)
				Result := last_double
			end
		end

	integer_value (a_target: like expression_target): INTEGER is
			--  Integer value
		require
			target_not_void: a_target /= Void
		do
			if last_target = a_target then
				Result := last_integer
			else
				last_target := a_target
				last_integer := get_integer_value (a_target)
				Result := last_integer
			end
		end

	component_expressions (a_target: ROSE_EXPRESSION_TARGET): ROSE_LINEAR_ARRAY [ROSE_EXPRESSION [ROSE_EXPRESSION_TARGET]] is
			-- Expressions which contribute to value of `Current'
		require
			a_target_not_void: a_target /= Void
		do
				-- Default implementation is suitable for named variables.
				-- Descendants (such as ROSE_UNARY/BINARY_EXPRESSION) redefine this routine
			if mnemonic /= Void then
				Result := a_target.component_expressions (mnemonic)
			else
				create Result.make_empty
			end
		ensure
			result_not_void: Result /= Void
			all_items_non_void: not Result.has (Void)
		end

	numeric_value (a_target: like target_anchor): DOUBLE is
			--numeric value
		require
			target_not_void: a_target /= Void
		do
			Result := generated_numeric_value (a_target, Void)
		end

	partial_numeric_value (a_target: like target_anchor): DOUBLE
			-- Paritally evaluated numeric value
		require
			target_not_void: a_target /= Void
		local
			l_real: REAL
			l_integer: INTEGER
		do
			if is_real then
				l_real := partial_real_value (a_target)
				if is_defined_real (l_real) then
					Result := l_real
				else
					Result := undefined_double
				end
			elseif is_integer then
				l_integer := partial_integer_value (a_target)
				if is_defined_integer (l_integer) then
					Result := l_integer
				else
					Result := undefined_double
				end
			else
				Result := partial_double_value (a_target)
			end
		end

	vector (a_targets: ARRAY [G]): VECTOR is
		do
			Result := generated_vector (a_targets, Void)
		end

	matrix (a_target_matrix: ARRAY2 [G]): MATRIX is
		do
			Result := generated_matrix (a_target_matrix, Void)
		end

	generated_value_in_market_currency (a_market: MARKET; a_currency: PERSISTENT_CURRENCY; a_target: like expression_target; a_date: ROSE_DATE; a_agent: like access_agent_template): DOUBLE is
		local
			l_unitized : like unitized_value
		do
			l_unitized := generated_unitized_value (a_target, a_agent)
			if l_unitized.is_defined_double (l_unitized.value) and then a_currency /= Void then
				l_unitized := l_unitized.converted_unitized_value (a_market, a_currency, a_date)
			end
			Result := l_unitized.value
		end

	unitized_value_in_market_currency (a_market: MARKET; a_currency: PERSISTENT_CURRENCY; a_target: like expression_target; a_date: ROSE_DATE; a_agent: like access_agent_template): ROSE_UNITIZED_VALUE [DOUBLE] is
		do
			Result := unitized_value (a_target, a_agent)
			if Result.is_defined_double (Result.value) and then a_currency /= Void then
				Result := Result.converted_unitized_value (a_market, a_currency, a_date)
			end
		end

	generated_value_in_currency (a_currency: PERSISTENT_CURRENCY; a_target: like expression_target; a_date: ROSE_DATE; a_agent: like access_agent_template): DOUBLE is
		do
			Result := generated_value_in_market_currency (Void, a_currency, a_target, a_date, a_agent)
		end

	normalized_value (a_target: like expression_target): DOUBLE is
			-- Normalized value if available, otherwise raw value
		require
			target_not_void: a_target /= Void
		do
			Result := double_value (a_target)
		end

	normalized_value_from_raw (a_value: DOUBLE; a_target: like expression_target): DOUBLE is
			-- The normalized value for the raw value 'a_value'
		require
			target_not_void: a_target /= Void
		do
			Result := double_value (a_target)
		end

	historical_normalized_value_from_raw (a_value: DOUBLE; a_target: like expression_target; a_as_of_date: ROSE_DATE): DOUBLE is
		do
			Result := Undefined_Double
		end

	mean_value (a_target: like expression_target): DOUBLE is
			-- Mean used in calculation of `normalized_value'
		require
			a_target_not_void: a_target /= Void
		do
			Result := Undefined_Double
		end

	weighted_mean_value (a_target: like expression_target): DOUBLE is
			-- Weighted mean used in calculation of `normalized_value'
		require
			a_target_not_void: a_target /= Void
		do
			Result := Undefined_Double
		end

	stddev_value (a_target: like expression_target): DOUBLE is
			-- Standard deviation used in calculation of `normalized_value'
		require
			a_target_not_void: a_target /= Void
		do
			Result := Undefined_Double
		end

	value_in_market_currency (a_market: MARKET; a_currency: PERSISTENT_CURRENCY; a_target: like expression_target; a_date: ROSE_DATE): DOUBLE is
			-- Value converted from nominal currency to a_market currency
		require
			target_not_void: a_target /= Void
		do
			Result := generated_value_in_market_currency (a_market, a_currency, a_target, a_date, Void)
		end

	value_in_currency (a_currency: like currency_anchor; a_target: like expression_target; a_date: ROSE_DATE): DOUBLE is
			-- Value in currency
		require
			target_not_void: a_target /= Void
		do
			Result := value_in_market_currency (Void, a_currency, a_target, a_date)
		end

	currency_anchor: PERSISTENT_CURRENCY is
			-- Currency anchor
		do
		end

	scaled_value_in_market_currency (a_market: MARKET; a_currency: like currency_anchor; a_target: like expression_target; a_date: ROSE_DATE): DOUBLE is
			-- Scaled value in market currency
		require
			target_not_void: a_target /= Void
		do
			Result := value_in_market_currency (a_market, a_currency, a_target, a_date)
			if is_defined_double (Result) then
				Result := Result * nominal_units.scale.to_double
			end
		end

	units (a_target: like expression_target): ROSE_UNITS is
			--Units of this expression for a_target
		require
			target_not_void: a_target /= Void
		local
			l_currency: PERSISTENT_CURRENCY
		do
			Result := nominal_units
			if Result /= Void and then Result.uses_currency then
				l_currency := currency (a_target)
				if l_currency /= Void then
					Result := Result.in_currency (l_currency)
				else
					Result := pure_units
				end
			end
		end

	units_in_currency (a_currency: PERSISTENT_CURRENCY; a_target: like expression_target): ROSE_UNITS is
			--Nominal units converted to 'a_currency'
		require
			target_not_void: a_target /= Void
		do
			Result := nominal_units.in_currency (a_currency)
		end

	date (a_target: like expression_target): ROSE_DATE is
			-- Date
		require
			target_not_void: a_target /= Void
		do
		end

	display_string (a_target: like expression_target): STRING is
			-- Display string as a function of target type.
		require
			target_not_void: a_target /= Void
		do
			if defined (a_target) then
				inspect type
				when Real_type then
					Result := real_value (a_target).out
				when Double_type then
					Result := double_value (a_target).out
				when Integer_type then
					Result := integer_value (a_target).out
				when Character_type then
					(create {EXCEPTIONS}).raise ("not implemented")
						--//\\--					Result := character_value (a_target).out
				when String_type then
					Result := string_value (a_target)
				when Date_type then
					Result := date_value (a_target).out
				when Boolean_type then
					Result := boolean_value (a_target).out
				end
			else
				Result := "undefined"
			end
		end

	display_partial_string (a_target: like expression_target): STRING is
		require
			target_not_void: a_target /= Void
		do
			if partially_defined (a_target) then
				inspect type
				when Real_type then
					Result := real_value (a_target).out
				when Double_type then
					Result := double_value (a_target).out
				when Integer_type then
					Result := integer_value (a_target).out
				when Character_type then
					(create {EXCEPTIONS}).raise ("not implemented")
						--//\\--					Result := character_value (a_target).out
				when String_type then
					Result := string_value (a_target)
				when Date_type then
					Result := date_value (a_target).out
				when Boolean_type then
					Result := boolean_value (a_target).out
				end
			else
				Result := "undefined"
			end
		end

	undefined_real: REAL is
		deferred
		end

	undefined_double: DOUBLE is
			-- Which value does this expression consider to be the undefined value?
		deferred
		end

	undefined_decimal: RP_DECIMAL is
		once
			create Result.make_undefined
		end

	undefined_integer: INTEGER is
		deferred
		end

feature -- Status report

	is_unary_operator_available (a_name: STRING): BOOLEAN is
			-- Is `a_name' a unary operator currently available to `Current'?
		require
			a_name_not_void: a_name /= Void
		local
			l_operator: ROSE_OPERATOR
		do
			l_operator := operators.unary_operator (a_name)
			Result := (l_operator /= Void)
		end

	is_binary_operator_available (a_name: STRING): BOOLEAN is
			-- Is `a_name' a binary operator currently available to `Current'?
		require
			a_name_not_void: a_name /= Void
		local
			l_operator: ROSE_OPERATOR
		do
			l_operator := operators.binary_operator (a_name)
			Result := (l_operator /= Void)
		end

	unavailable_unitized_double: DATED_UNITIZED_VALUE[DOUBLE]
	-- Returns the proper 'unavailable' value based on the type of the expression.  Descendants
	-- (flags like FF1) redefine this.
	once
		create Result.make_with_units(missing_double, create {PURE_UNITS}.default_create)
	end

feature -- Infix

	infix "+" (a_other: like other_expression_template): like computed_expression_target is
			-- New expression that performs addition of `a_other' to `Current'
		require
			a_other_not_void: a_other /= Void
			plus_operator_available: is_binary_operator_available ("+")
		do
			Result := binary_expression (operators.plus_operator, a_other)
		end

	infix "-" (a_other: like other_expression_template): ROSE_BINARY_EXPRESSION [G] is
			-- New expression that performs subtraction of `a_other' from `Current'
		require
			a_other_not_void: a_other /= Void
			minus_operator_available: is_binary_operator_available ("-")
		do
			Result := binary_expression (operators.minus_operator, a_other)
		end

	infix "*" (a_other: like other_expression_template): like computed_expression_target is
			-- New expression that performs multiplication of `a_other' by `Current'
		require
			a_other_not_void: a_other /= Void
			multiply_operator_available: is_binary_operator_available ("*")
		do
			Result := binary_expression (operators.times_operator, a_other)
		end

	infix "/" (a_other: like other_expression_template): like computed_expression_target is
			-- New expression that performs division of `Current' by `a_other'
		require
			a_other_not_void: a_other /= Void
			division_operator_available: is_binary_operator_available ("/")
		do
			Result := binary_expression (operators.ratio_operator, a_other)
		end

	infix "#+" (a_value: DOUBLE): ROSE_BINARY_EXPRESSION [G] is
			-- New expression that performs addition of `a_other' to `Current'
		do
			Result := binary_expression (operators.plus_operator, constant (a_value))
		end

	infix "#-" (a_value: DOUBLE): ROSE_BINARY_EXPRESSION [G] is
			-- New expression that performs subtraction of `a_other' from `Current'
		do
			Result := binary_expression (operators.minus_operator, constant (a_value))
		end

	infix "#*" (a_value: DOUBLE): ROSE_BINARY_EXPRESSION [G] is
			-- New expression that performs multiplication of `a_other' by `Current'
		do
			Result := binary_expression (operators.times_operator, constant (a_value))
		end

	infix "#/" (a_value: DOUBLE): ROSE_BINARY_EXPRESSION [G] is
			-- New expression that performs division of `Current' by `a_other'
		do
			Result := binary_expression (operators.ratio_operator, constant (a_value))
		end

	infix "#and" (a_other: like other_expression_template): like computed_expression_target is
			-- New expression that performs `Current' and `a_other'
		require
			a_other_not_void: a_other /= Void
			and_operator_available: is_binary_operator_available ("and ")
		do
			Result := binary_expression (operators.and_operator, a_other)
		end

	infix "#or" (a_other: like other_expression_template): like computed_expression_target is
			-- New expression that performs `Current' or `a_other'
		require
			a_other_not_void: a_other /= Void
			mod_operator_available: is_binary_operator_available ("or ")
		do
			Result := binary_expression (operators.or_operator, a_other)
		end

	prefix "#abs": ROSE_UNARY_EXPRESSION [like expression_target] is
			-- New expression that returns positive value of `Current'
		require
			abs_operator_available: is_unary_operator_available ("#abs ")
		do
			create Result.make (Current, operators.absolute_operator)
		end

	prefix "#log": ROSE_UNARY_EXPRESSION [like expression_target] is
			-- New expression that returns logarithm of `Current'
		require
			log_operator_available: is_unary_operator_available ("log ")
		do
			create Result.make (Current, operators.log_operator)
		end

	prefix "#exp": ROSE_UNARY_EXPRESSION [like expression_target] is
			-- New expression that returns e to power of `Current'
		require
			exp_operator_available: is_unary_operator_available ("exp ")
		do
			create Result.make (Current, operators.exponential_operator)
		end

	prefix "-": ROSE_UNARY_EXPRESSION [like expression_target] is
			-- New expression that returns e to power of `Current'
		require
			exp_operator_available: is_unary_operator_available ("-")
		do
			create Result.make (Current, operators.negative_operator)
		end

feature -- Operators

	unary_operator (a_operator: STRING): ROSE_UNARY_OPERATOR [like expression_target] is
			-- New unary operator named `a_operator'
		require
			a_operator_not_void: a_operator /= Void
		do
			Result := operators.unary_operator (a_operator)
		ensure
			result_not_void_if_operator_available: is_unary_operator_available (a_operator) implies Result /= Void
		end

	binary_operator (a_operator: STRING): ROSE_BINARY_OPERATOR [like expression_target] is
			-- New binary operator named `a_operator'
		require
			a_operator_not_void: a_operator /= Void
		do
			Result := operators.binary_operator (a_operator)
		ensure
			result_not_void_if_operator_available: is_binary_operator_available (a_operator) implies Result /= Void
		end

	sub_expressions: ROSE_LINEAR_ARRAY [ROSE_EXPRESSION [like expression_target]] is
			-- expressions needed to compute this expression (should be freed from having to conform to expression_target)
		do
			create Result.make_empty
			Result.extend (Current)
		end

	do_all_leaves (a_action: PROCEDURE [ANY, TUPLE [ROSE_EXPRESSION [like expression_target]]]) is
			-- Apply `a_action' to every leaf item.
			-- Semantics not guaranteed if `action' changes the structure;
			-- in such a case, apply iterator to clone of structure instead.
		local
			l_sub_expressions: like sub_expressions
		do
			l_sub_expressions := sub_expressions
			from
				l_sub_expressions.start
			until
				l_sub_expressions.off
			loop
				if l_sub_expressions.current_item.is_leaf then
					a_action.call ([l_sub_expressions.current_item])
				end
				l_sub_expressions.forth
			end
		end

	is_leaf: BOOLEAN is
		do
			Result := sub_expressions.count = 1
		end

feature -- Units

	scale: RATIONAL
			-- Scale of `nominal_units'
		do
			if scale_saved = Void then
				if variable_description = Void or else variable_description.scale = Void then
					scale_saved := one
				else
					create scale_saved.make_from_string (variable_description.scale)
				end
			end
			Result := scale_saved
		ensure
			not_void: Result /= Void
		end

	nominal_units: ROSE_UNITS is
			-- Base units
		do
			if nominal_units_saved = Void then
				if variable_description /= Void and then variable_description.units /= Void then
					nominal_units_saved := units_from_expression (variable_description.units).scaled_units (scale)
				end
				if nominal_units_saved = Void then
					nominal_units_saved := pure_units
				end
			end
			Result := nominal_units_saved
		ensure
			not_void: Result /= Void
		end

	units_parser: ROSE_UNITS_PARSER is
		once
			create Result.make
		end

	units_from_expression (a_string: STRING): ROSE_UNITS is
			-- Units represented in 'a_string'
		do
			if a_string = Void or else a_string.is_empty then
				Result := pure_units
			else
				units_parser.acknowledge_error
				units_parser.parse_string (a_string)
				Result := units_parser.last_parsed
				if Result = Void then
					print (units_parser.last_error)
				end
			end
				--/-
		end

feature -- New expressions

	binary_expression (a_operator: ROSE_BINARY_OPERATOR [like expression_target]; a_other: like other_expression_template): like new_binary_expression is
			-- Binary expression from `Current' and `a_other' combined via `a_operator'
		require
			a_operator_not_void: a_operator /= Void
			a_other_not_void: a_other /= Void
		local
			l_inverse: ROSE_BINARY_OPERATOR [like expression_target]
		do
			Result ?= computed_expressions.item (operator_expressions.pair (a_operator, a_other.expression_index))
			if Result = Void then
				Result := new_binary_expression (a_other, a_operator)
				computed_expressions.put (Result, operator_expressions.new_pair (a_operator, a_other.expression_index))
				l_inverse ?= a_operator.inverse
				if l_inverse /= Void then
					Result.computed_expressions.put (Current, operator_expressions.new_pair (l_inverse, a_other.expression_index))
				end
				if a_operator.commutative then
					a_other.computed_expressions.put (Result, operator_expressions.new_pair (a_operator, expression_index))
					if l_inverse /= Void then
						Result.computed_expressions.put (a_other, operator_expressions.new_pair (l_inverse, expression_index))
					end
				end
			end
		end

	as_parent: ROSE_IDENTITY_EXPRESSION [G] is
			-- Returns an expression that will use 'parent' as opposed to 'consolidated' data.
			-- Currently this is only useful for J zone valuation_data and only in programs
			-- that will send this Result to a persistent_expression_server.
		do
			create Result.make (Current, create {PARENT_OPERATOR [G]})
		end

	as_consolidated: ROSE_IDENTITY_EXPRESSION [G] is
			-- Returns an expression that will use 'consolidated' as opposed to 'parent' data.
			-- Currently this is only useful for J zone valuation_data and only in programs
			-- that will send this Result to a persistent_expression_server.
		do
			create Result.make (Current, create {CONSOLIDATED_OPERATOR [G]})
		end

	as_quarterly: ROSE_EXPRESSION [G] is
			-- Returns an expression that will use 'quarterly' as opposed to 'annual' data.
			-- Currently this is only useful for valuation_fiscal_data and only in programs
			-- that will send this Result to a persistent_expression_server.
		local
			l_source: STRING
		do
			if variable_description /= Void and then variable_description.source /= Void then
				l_source := variable_description.source
			else
				l_source := "Void"
			end
			if equal (l_source, "valdata") then
				create {ROSE_IDENTITY_EXPRESSION [G]} Result.make (Current, create {VALUATION_DATA_INTERIM_OPERATOR [G]})
			else
				Result := Current
			end
		end

	as_trailing: ROSE_EXPRESSION [G] is
			-- Returns an expression that will use 'trailing' (normal valdata) rather than 'quarterly' or 'annual' data.
			-- Currently this is only useful for valuation_fiscal_data and only in programs
			-- that will send this Result to a persistent_expression_server.
		local
			l_source: STRING
		do
			if variable_description /= Void and then variable_description.source /= Void then
				l_source := variable_description.source
			else
				l_source := ""
			end
			if l_source ~ "valdata" then
				create {ROSE_IDENTITY_EXPRESSION [G]} Result.make (Current, create {VALUATION_DATA_TRAILING_OPERATOR [G]})
			else
				Result := Current
			end
		end

	as_annual: ROSE_EXPRESSION [G] is
			-- Returns an expression that will use 'annual' as opposed to 'quarterly' data.
			-- Currently this is only useful for valuation_fiscal_data and only in programs
			-- that will send this Result to a persistent_expression_server.
		local
			l_source: STRING
		do
			if variable_description /= Void and then variable_description.source /= Void then
				l_source := variable_description.source
			else
				l_source := "Void"
			end
			if equal (l_source, "valdata") then
				create {ROSE_IDENTITY_EXPRESSION [G]} Result.make (Current, create {VALUATION_DATA_ANNUAL_OPERATOR [G]})
			else
				Result := Current
			end
		end

	zero_if_undefined: ROSE_UNARY_EXPRESSION [G] is
			-- Version of this expression which evaluates to zero instead of undefined
		do
			create Result.make (Current, create {ROSE_ZERO_IF_UNDEFINED_OPERATOR [G]}.make_zero_if_undefined)
		ensure
			not_void: Result /= Void
		end

feature -- Write

	write_double (a_value: DOUBLE; a_target: like expression_target) is
			--write double
		do
		end

	write_integer (a_value: INTEGER; a_target: like expression_target) is
		do
		end

	write_real (a_value: REAL; a_target: like expression_target) is
		do
		end

	commit is
			-- Commit changes to database.
		do
		end

feature {NEEDS_EXPRESSION_COMPUTATION} -- Compute

	computed_numeric_value (a_target: like expression_target; a_agent: like access_agent_template): DOUBLE
			--computed value converted to double
		local
			l_real: REAL
			l_integer: INTEGER
		do
			if is_real then
				l_real := computed_real (a_target, a_agent)
				if is_defined_real (l_real) then
					Result := l_real
				else
					Result := undefined_double
				end
			elseif is_integer then
				l_integer := computed_integer (a_target, a_agent)
				if is_defined_integer (l_integer) then
					Result := l_integer
				else
					Result := undefined_double
				end
			else
				Result := computed_double (a_target, a_agent)
			end
		end

	computed_vector (a_targets: ARRAY [like expression_target]; a_agent: like access_agent_template): VECTOR is
			-- Computed vector
		require
			valid_targets: a_targets /= Void
		local
			i, l_offset: INTEGER
			l_target: like expression_target
			l_value: DOUBLE
		do
			create Result.make (a_targets.count)
			l_offset := a_targets.lower - 1
			from
				i := 1
			until
				i > a_targets.count
			loop
				l_target := a_targets.item (l_offset + i)
				if l_target /= Void then
					l_value := computed_numeric_value (l_target, a_agent)
					Result.put (l_value, i)
				else
					Result.put (missing_double, i)
				end
				i := i + 1
			end
		end

	computed_double (a_target: like expression_target; a_agent: like access_agent_template): DOUBLE is
			-- Computed value of `Current' as a double;
			-- Value will eventually be stored and can be retrieved by `double_value'.
		do
			Result := double_using_agent (a_target, a_agent)
		end

	computed_rt_double (a_target: like expression_target; a_agent: like access_agent_template): DOUBLE is
		do
			Result := computed_double (a_target, a_agent)
		end

	computed_real (a_target: like expression_target; a_agent: like access_agent_template): REAL is
			-- Computed real value
		do
			Result := real_value (a_target)
		end

	computed_integer (a_target: like expression_target; a_agent: like access_agent_template): INTEGER is
			-- Computed integer value
		do
			Result := integer_using_agent (a_target, a_agent)
		end

	computed_string (a_target: like expression_target; a_agent: like access_agent_template): STRING is
			-- Computed string
		do
			Result := string_value (a_target)
		end

	computed_boolean (a_target: like expression_target; a_agent: like access_agent_template): BOOLEAN is
			-- Computed boolean
		do
			Result := boolean_value (a_target)
		end

	computed_date (a_target: like expression_target; a_agent: like access_agent_template): ROSE_DATE is
			-- Computed string
		do
			Result := date_value (a_target)
		end

feature -- agent retrievals

	computed_currency (a_target: like expression_target; a_agent: like access_agent_template): PERSISTENT_CURRENCY is
			-- Computed currency
		do
			Result := currency (a_target)
		end

	computed_units (a_target: like expression_target; a_agent: like access_agent_template): ROSE_UNITS is
			-- Units for computed value
		do
			if a_agent /= Void then
				Result := a_agent.units (entity_date_target (a_target), Current)
			end
			if Result = Void then
				Result := units (a_target)
			end
		end

	unitized_using_agent (a_target: like expression_target; a_agent: like access_agent_template): ROSE_UNITIZED_VALUE[DOUBLE]
			-- Unitized value of `Current', first from a_agent
		do
			if a_agent /= Void then
				Result := a_agent.unitized_double (entity_date_target (a_target), Current)
			end
			if Result = Void then
				Result := unitized_value(a_target,a_agent)
			end
		end

	double_using_agent (a_target: like expression_target; a_agent: like access_agent_template): DOUBLE is
			-- Computed value of `Current' as a double;
			-- Value will eventually be stored and can be retrieved by `double_value'.
		local
			l_value: ROSE_UNITIZED_VALUE [DOUBLE]
		do
			l_value := unitized_using_agent (a_target,a_agent)
			if l_value /= Void then
				if a_agent /= Void and then a_agent.currency /= Void then
					Result := l_value.converted_unitized_value (Void, a_agent.currency, a_agent.currency_date).value
				else
					Result := l_value.value
				end
			end
		end

	found : BOOLEAN
	integer_using_agent (a_target: like expression_target; a_agent: like access_agent_template): INTEGER is
			-- Computed integer value
		local
			l_value: ROSE_UNITIZED_VALUE [INTEGER]
		do
			if a_agent /= Void and then a_agent.has_item (a_target, Current) then
				l_value := a_agent.unitized_integer (entity_date_target (a_target), Current)
				if l_value /= Void then
					Result := l_value.value
				end
				if l_value = Void then
					Result := a_agent.integer_value (entity_date_target (a_target),Current)
				end
			else
				Result := integer_value(a_target)
			end
		end

	string_using_agent (a_target: like expression_target; a_agent: like access_agent_template): STRING is
			-- String value checking agent first
		do
			if a_agent /= Void and then a_agent.has_item(a_target,Current) then
				Result := a_agent.string_value (entity_date_target (a_target), Current)
			else
				Result := string_value (a_target)
			end
		end
	computed_double_using_agent (a_target: like expression_target; a_agent: like access_agent_template): DOUBLE is
			-- Computed value of `Current' as a double;
			-- Checks agent first
		local
			l_value: ROSE_UNITIZED_VALUE [DOUBLE]
		do
			if a_agent /= Void then
				l_value := a_agent.unitized_double (entity_date_target (a_target), Current)
				if l_value /= Void then
					if a_agent.currency /= Void then
						l_value := l_value.converted_unitized_value (Void, a_agent.currency, a_agent.currency_date)
						Result := l_value.value.truncated_to_real
					else
						if {l_computed : COMPUTED_EXPRESSION[G]} Current then --or else {l_mondex : MONDEX_VARIABLE} Current then
							Result := l_value.value
						else
							Result := l_value.value.truncated_to_real
						end
					end
				end
			end
			if l_value = Void then
				Result := computed_double(a_target,a_agent)
			end
		end

	computed_integer_using_agent (a_target: like expression_target; a_agent: like access_agent_template): INTEGER is
			-- Computed integer value checking agent first
		local
			l_value: ROSE_UNITIZED_VALUE [INTEGER]
		do
			if a_agent /= Void and then a_agent.has_item (a_target, Current) then
				l_value := a_agent.unitized_integer (entity_date_target (a_target), Current)
				if l_value /= Void then
					Result := l_value.value
				end
				if l_value = Void then
					Result := a_agent.integer_value (entity_date_target (a_target),Current)
				end
			else
				Result := computed_integer(a_target,a_agent)
			end
		end

	computed_string_using_agent (a_target: like expression_target; a_agent: like access_agent_template): STRING is
			-- computed string value checking agent first
		do
			if a_agent /= Void and then a_agent.has_item(a_target,Current) then
				Result := a_agent.string_value (entity_date_target (a_target), Current)
			else
				Result := computed_string (a_target,a_agent)
			end
		end
feature -- Generate

	generate (a_target: like expression_target; a_agent: like access_agent_template) is
			-- Should be removed ... the only effect is whatever side effect the appropriate generated value has
		local
			l_real_value: REAL
			l_double_value: DOUBLE
			l_integer_value: INTEGER
			l_string_value: STRING
		do
			inspect type
			when Real_type then
				l_real_value := generated_real (a_target, a_agent)
			when Double_type then
				l_double_value := generated_double (a_target, a_agent)
			when Integer_type then
				l_integer_value := generated_integer (a_target, a_agent)
			when String_type then
				l_string_value := generated_string (a_target, a_agent)
			else
			end
		end

	frozen generated_double (a_target: like expression_target; a_agent: like access_agent_template): DOUBLE is
			-- ask the access control agent first how to get the value, and fall back on double_value
			-- NB: that this is modified to computed_double is some descendents, should be fixed
			--
			-- (This routine contains code instrumentation to record snapshots.
			-- Redefinitions should be done through `internal_generated_double.)
		local
			l_snapshot_recorder: ROSE_EXPRESSION_SNAPSHOT_RECORDER
			l_snapshot: ROSE_EXPRESSION_SNAPSHOT
		do
			l_snapshot_recorder := snapshot_recorder
			if l_snapshot_recorder /= Void then
				create l_snapshot.make (Current, a_target)
				l_snapshot_recorder.record_snapshot (l_snapshot)
			end
			Result := internal_generated_double (a_target, a_agent)
			if l_snapshot /= Void then
				l_snapshot.set_value (Result)
				l_snapshot.close
			end
		end

	frozen generated_decimal (a_target: like expression_target; a_agent: like access_agent_template): RP_DECIMAL is
			-- Generated decimal on target
			--
			-- (This routine contains code instrumentation to record snapshots.
			-- Redefinitions should be done through `internal_generated_decimal.)
		local
			l_snapshot_recorder: ROSE_EXPRESSION_SNAPSHOT_RECORDER
			l_snapshot: ROSE_EXPRESSION_SNAPSHOT
		do
			l_snapshot_recorder := snapshot_recorder
			if l_snapshot_recorder /= Void then
				create l_snapshot.make (Current, a_target)
				l_snapshot_recorder.record_snapshot (l_snapshot)
			end
			Result := internal_generated_decimal (a_target, a_agent)
			if l_snapshot /= Void then
				l_snapshot.set_value (Result)
				l_snapshot.close
			end
		end

	frozen generated_real (a_target: like expression_target; a_agent: like access_agent_template): REAL is
			-- ask the access control agent first how to get the value, and fall back on computed_real
			--
			-- (This routine contains code instrumentation to record snapshots.
			-- Redefinitions should be done through `internal_generated_real.)
		local
			l_snapshot_recorder: ROSE_EXPRESSION_SNAPSHOT_RECORDER
			l_snapshot: ROSE_EXPRESSION_SNAPSHOT
		do
			l_snapshot_recorder := snapshot_recorder
			if l_snapshot_recorder /= Void then
				create l_snapshot.make (Current, a_target)
				l_snapshot_recorder.record_snapshot (l_snapshot)
			end
			Result := internal_generated_real (a_target, a_agent)
			if l_snapshot /= Void then
				l_snapshot.set_value (Result)
				l_snapshot.close
			end
		end

	frozen generated_integer (a_target: like expression_target; a_agent: like access_agent_template): INTEGER is
			-- ask the access control agent first how to get the value, and fall back on integer_value
			-- NB: this is modified to computed_integer is some descendents, should be fixed
			--
			-- (This routine contains code instrumentation to record snapshots.
			-- Redefinitions should be done through `internal_generated_integer.)
		local
			l_snapshot_recorder: ROSE_EXPRESSION_SNAPSHOT_RECORDER
			l_snapshot: ROSE_EXPRESSION_SNAPSHOT
		do
			l_snapshot_recorder := snapshot_recorder
			if l_snapshot_recorder /= Void then
				create l_snapshot.make (Current, a_target)
				l_snapshot_recorder.record_snapshot (l_snapshot)
			end
			Result := internal_generated_integer (a_target, a_agent)
			if l_snapshot /= Void then
				l_snapshot.set_value (Result)
				l_snapshot.close
			end
		end

	frozen generated_string (a_target: like expression_target; a_agent: like access_agent_template): STRING is
			-- Generated string value
			--
			-- (This routine contains code instrumentation to record snapshots.
			-- Redefinitions should be done through `internal_generated_string.)
		local
			l_snapshot_recorder: ROSE_EXPRESSION_SNAPSHOT_RECORDER
			l_snapshot: ROSE_EXPRESSION_SNAPSHOT
		do
			l_snapshot_recorder := snapshot_recorder
			if l_snapshot_recorder /= Void then
				create l_snapshot.make (Current, a_target)
				l_snapshot_recorder.record_snapshot (l_snapshot)
			end
			Result := internal_generated_string (a_target, a_agent)
			if l_snapshot /= Void then
				l_snapshot.set_value (Result)
				l_snapshot.close
			end
		end

	frozen generated_date (a_target: like expression_target; a_agent: like access_agent_template): ROSE_DATE is
			-- Generated date value
			--
			-- (This routine contains code instrumentation to record snapshots.
			-- Redefinitions should be done through `internal_generated_date.)
		local
			l_snapshot_recorder: ROSE_EXPRESSION_SNAPSHOT_RECORDER
			l_snapshot: ROSE_EXPRESSION_SNAPSHOT
		do
			l_snapshot_recorder := snapshot_recorder
			if l_snapshot_recorder /= Void then
				create l_snapshot.make (Current, a_target)
				l_snapshot_recorder.record_snapshot (l_snapshot)
			end
			Result := internal_generated_date (a_target, a_agent)
			if l_snapshot /= Void then
				l_snapshot.set_value (Result)
				l_snapshot.close
			end
		end

	generated_numeric_value (a_target: like target_anchor; a_agent: like access_agent_template): DOUBLE is
		local
			l_real: REAL
			l_integer: INTEGER
		do
			if is_real then
				l_real := generated_real (a_target, a_agent)
				if is_defined_real (l_real) then
					Result := l_real
				else
					Result := undefined_double
				end
			elseif is_integer then
				l_integer := generated_integer (a_target, a_agent)
				if is_defined_integer (l_integer) then
					Result := l_integer
				else
					Result := undefined_double
				end
			else
				Result := generated_double (a_target, a_agent)
			end
		end

	generated_weighted_vector (a_targets: ARRAY [like expression_target]; a_weight: ROSE_EXPRESSION [G]; a_agent: like access_agent_template): VECTOR is
		do
			debug ("timers_computed_result")
				timers.resume (generating_type + "=" + name, "generated_weighted_vector -> generated_vector.cross")
			end
			Result := generated_vector (a_targets, a_agent).cross (a_weight.generated_vector (a_targets, a_agent))
			debug ("timers_computed_result")
				timers.interrupt (generating_type + "=" + name, "generated_weighted_vector -> generated_vector.cross")
			end
		end

	generated_vector (a_targets: ARRAY [like expression_target]; a_agent: like access_agent_template): VECTOR is
		require
			valid_targets: a_targets /= Void
		local
			i, l_offset: INTEGER
			l_target: like target_anchor
			l_value: DOUBLE
		do
			debug ("timers_computed_result")
				timers.resume (generating_type + "=" + name, "generated_vector")
			end
			create Result.make (a_targets.count)
			l_offset := a_targets.lower - 1
			from
				i := 1
			until
				i > a_targets.count
			loop
				l_target := a_targets.item (l_offset + i)
				if l_target /= Void then
					l_value := generated_numeric_value (l_target, a_agent)
					Result.put (l_value, i)
				else
					Result.put (missing_double, i)
				end
				i := i + 1
			end
			debug ("timers_computed_result")
				timers.interrupt (generating_type + "=" + name, "generated_vector")
			end
		end

	generated_matrix (a_target_matrix: ARRAY2 [like expression_target]; a_agent: like access_agent_template): MATRIX is
		require
			valid_target_matrix: a_target_matrix /= Void
		local
			l_target: like target_anchor
			l_value: DOUBLE
			l_height, l_width: INTEGER
			i, j: INTEGER
		do
			debug ("timers_computed_result")
				timers.resume (generating_type + "=" + name, "generated_matrix")
			end
			l_height := a_target_matrix.height
			l_width := a_target_matrix.width
			create Result.make (l_height, l_width)
			from
				i := 1
			until
				i > l_height
			loop
				from
					j := 1
				until
					j > l_width
				loop
					l_target := a_target_matrix.item (i, j)
					if l_target /= Void then
						l_value := generated_numeric_value (l_target, a_agent)
						Result.put (l_value, i, j)
					else
						Result.put (missing_double, i, j)
					end
					j := j + 1
				end
				i := i + 1
			end
			debug ("timers_computed_result")
				timers.interrupt (generating_type + "=" + name, "generated_matrix")
			end
		end

	generated_string_array (a_targets: ARRAY [like target_anchor]; a_agent: like access_agent_template): ROSE_LINEAR_ARRAY [STRING] is
		require
			valid_targets: a_targets /= Void and then a_targets.count > 0
			valid_type: is_string
		local
			i, l_offset: INTEGER
			l_target: like expression_target
			l_value: STRING
		do
			debug ("timers_computed_result")
				timers.resume (generating_type + "=" + name, "generated_string_array")
			end
			create Result.make (1, a_targets.count)
			l_offset := a_targets.lower - 1
			from
				i := 1
			until
				i > a_targets.count
			loop
				l_target := a_targets.item (l_offset + i)
				if l_target /= Void then
					l_value := generated_string (l_target, a_agent)
					Result.put (l_value, i)
				else
					Result.put ("", i)
				end
				i := i + 1
			end
			debug ("timers_computed_result")
				timers.interrupt (generating_type + "=" + name, "generated_string_array")
			end
		end

	entity_date_target (a_g: like expression_target): ROSE_TARGET [ENTITY] is
			--convert something of type G to a rose_entity_date if possible
			--this will be explicitly redefined in descendents to get rid of the
			--assignment attempt, but for backward compatibility we need to leave it
			--unresolved here
		do
			Result ?= a_g
		end

	generated_cache_flush is
			-- Does nothing
		do
		end

	frozen generated_double_using_agent (a_target: like expression_target; a_agent: like access_agent_template): DOUBLE is
			-- ask the access control agent first how to get the value, and fall back on double_using_agent
			--
			-- (This routine contains code instrumentation to record snapshots.
			-- Redefinitions should be done through `internal_generated_double_using_agent.)
		local
			l_snapshot_recorder: ROSE_EXPRESSION_SNAPSHOT_RECORDER
			l_snapshot: ROSE_EXPRESSION_SNAPSHOT
		do
			l_snapshot_recorder := snapshot_recorder
			if l_snapshot_recorder /= Void then
				create l_snapshot.make (Current, a_target)
				l_snapshot_recorder.record_snapshot (l_snapshot)
			end
			Result := internal_generated_double_using_agent (a_target, a_agent)
			if l_snapshot /= Void then
				l_snapshot.set_value (Result)
				l_snapshot.close
			end
		end

feature {NONE} -- Generate

	internal_generated_double (a_target: like expression_target; a_agent: like access_agent_template): DOUBLE is
			-- ask the access control agent first how to get the value, and fall back on double_value
			-- NB: that this is modified to computed_double is some descendents, should be fixed
		local
			l_value: ROSE_UNITIZED_VALUE [DOUBLE]
		do
			-- Code in condition below can be used/modified to inform on callers of expression as a function of current type
			-- if attached {RISK_INDEX} Current as l_rose_expression then
			-- 	print (variable_description.name + "|" + variable_description.mnemonic + "|" + variable_description.parent + "|" + variable_description.target + "|" + variable_description.target_name + "|" + variable_description.expression + "%N")
			-- end
			if a_agent /= Void and then a_agent.compute_agent.item ([entity_date_target (a_target), Current]) then
				if a_agent.in_double_cache (entity_date_target (a_target), Void, Current) then
					Result := a_agent.double_cache_found_item
				else
					debug ("timers_computed_result")
						timers.resume (generating_type + "=" + name, "generated_double -> computed_double")
					end
					Result := computed_double (a_target, a_agent)
					debug ("timers_computed_result")
						timers.interrupt (generating_type + "=" + name, "generated_double -> computed_double")
					end
					a_agent.force_double (Result, entity_date_target (a_target), Current)
				end
			elseif a_agent /= Void then
					--				Result := value_in_currency (a_agent.currency, a_target, a_agent.currency_date)
				debug ("timers_computed_result")
					timers.resume (generating_type + "=" + name, "generated_double -> unitized_double")
				end
				l_value := a_agent.unitized_double (entity_date_target (a_target), Current)
				debug ("timers_computed_result")
					timers.interrupt (generating_type + "=" + name, "generated_double -> unitized_double")
				end
				if l_value = Void then
					debug ("timers_computed_result")
						timers.resume (generating_type + "=" + name, "generated_double -> unitized_value")
					end
					l_value := unitized_value (a_target, a_agent)
					debug ("timers_computed_result")
						timers.interrupt (generating_type + "=" + name, "generated_double -> unitized_value")
					end
				end
				if l_value /= Void then
					if a_agent.currency /= Void then
						debug ("timers_computed_result")
							timers.resume (generating_type + "=" + name, "generated_double -> converted_unitized_value")
						end
						Result := l_value.converted_unitized_value (Void, a_agent.currency, a_agent.currency_date).value
						debug ("timers_computed_result")
							timers.interrupt (generating_type + "=" + name, "generated_double -> converted_unitized_value")
						end
					else
						Result := l_value.value
					end
				else
					debug ("timers_computed_result")
						timers.resume (generating_type + "=" + name, "generated_double -> double_value")
					end
					Result := double_value (a_target)
					debug ("timers_computed_result")
						timers.interrupt (generating_type + "=" + name, "generated_double -> double_value")
					end
				end
			else
				debug ("timers_computed_result")
					timers.resume (generating_type + "=" + name, "generated_double -> double_value")
				end
				Result := double_value (a_target)
				debug ("timers_computed_result")
					timers.interrupt (generating_type + "=" + name, "generated_double -> double_value")
				end
			end
		end

	internal_generated_decimal (a_target: like expression_target; a_agent: like access_agent_template): RP_DECIMAL is
			-- Generated decimal on target
		do
			debug ("timers_computed_result")
				timers.resume (generating_type + "=" + name, "generated_decimal")
			end
			Result := decimal_value (a_target)
			debug ("timers_computed_result")
				timers.interrupt (generating_type + "=" + name, "generated_decimal")
			end
		end

	internal_generated_real (a_target: like expression_target; a_agent: like access_agent_template): REAL is
			-- ask the access control agent first how to get the value, and fall back on computed_real
		do
			if a_agent /= Void and then a_agent.compute_agent.item ([entity_date_target (a_target), Current]) then
				if a_agent.in_real_cache (entity_date_target (a_target), Current) then
					Result := a_agent.real_cache_found_item
				else
					debug ("timers_computed_result")
						timers.resume (generating_type + "=" + name, "generated_real -> computed_real")
					end
					Result := computed_real (a_target, a_agent)
					debug ("timers_computed_result")
						timers.interrupt (generating_type + "=" + name, "generated_real -> computed_real")
					end
					a_agent.force_real (Result, entity_date_target (a_target), Current)
				end
			else
				debug ("timers_computed_result")
					timers.resume (generating_type + "=" + name, "generated_real -> real_value")
				end
				Result := real_value (a_target)
				debug ("timers_computed_result")
					timers.interrupt (generating_type + "=" + name, "generated_real -> real_value")
				end
			end
		end

	internal_generated_integer (a_target: like expression_target; a_agent: like access_agent_template): INTEGER is
			-- ask the access control agent first how to get the value, and fall back on integer_value
			-- NB: this is modified to computed_integer is some descendents, should be fixed
		local
			l_value: ROSE_UNITIZED_VALUE [INTEGER]
		do
			if a_agent /= Void then
				if a_agent.compute_agent.item ([entity_date_target (a_target), Current]) then
					if a_agent.in_integer_cache (entity_date_target (a_target), Current) then
						Result := a_agent.integer_cache_found_item
					else
						debug ("timers_computed_result")
							timers.resume (generating_type + "=" + name, "generated_integer -> computed_integer")
						end
						Result := computed_integer (a_target, a_agent)
						debug ("timers_computed_result")
							timers.interrupt (generating_type + "=" + name, "generated_integer -> computed_integer")
						end
						a_agent.force_integer (Result, entity_date_target (a_target), Current)
					end
				else
					l_value := a_agent.unitized_integer (entity_date_target (a_target), Current)
					if l_value /= Void then
						Result := l_value.value
					else
						Result := a_agent.integer_value (entity_date_target (a_target), Current)
					end
				end
			else
				debug ("timers_computed_result")
					timers.resume (generating_type + "=" + name, "generated_integer -> integer_value")
				end
				Result := integer_value (a_target)
				debug ("timers_computed_result")
					timers.interrupt (generating_type + "=" + name, "generated_integer -> integer_value")
				end
			end
		end

	internal_generated_string (a_target: like expression_target; a_agent: like access_agent_template): STRING is
		do
			if a_agent /= Void and then a_agent.compute_agent.item ([entity_date_target (a_target), Current]) then
				if a_agent.in_string_cache (entity_date_target (a_target), Current) then
					Result := a_agent.string_cache_found_item
				else
					debug ("timers_computed_result")
						timers.resume (generating_type + "=" + name, "generated_string -> computed_string")
					end
					Result := computed_string (a_target, a_agent)
					debug ("timers_computed_result")
						timers.interrupt (generating_type + "=" + name, "generated_string -> computed_string")
					end
					a_agent.force_string (Result, entity_date_target (a_target), Current)
				end
			else
				debug ("timers_computed_result")
					timers.resume (generating_type + "=" + name, "generated_string -> string_value")
				end
				Result := string_value (a_target)
				debug ("timers_computed_result")
					timers.interrupt (generating_type + "=" + name, "generated_string -> string_value")
				end
			end
		end

	internal_generated_date (a_target: like expression_target; a_agent: like access_agent_template): ROSE_DATE is
		do
			if a_agent /= Void and then a_agent.compute_agent.item ([entity_date_target (a_target), Current]) then
				if a_agent.in_date_cache (entity_date_target (a_target), Current) then
					Result := a_agent.date_cache_found_item
				else
					debug ("timers_computed_result")
						timers.resume (generating_type + "=" + name, "generated_date -> computed_value")
					end
					Result := computed_date (a_target, a_agent)
					debug ("timers_computed_result")
						timers.interrupt (generating_type + "=" + name, "generated_date -> computed_value")
					end
					a_agent.force_date (Result, entity_date_target (a_target), Current)
				end
			else
				Result := date_value (a_target)
			end
		end

	internal_generated_double_using_agent (a_target: like expression_target; a_agent: like access_agent_template): DOUBLE is
			-- ask the access control agent first how to get the value, and fall back on double_using_agent
		local
			l_value: ROSE_UNITIZED_VALUE [DOUBLE]
		do
			-- Code in condition below can be used/modified to inform on callers of expression as a function of current type
			-- if attached {RISK_INDEX} Current as l_rose_expression then
			-- 	print (variable_description.name + "|" + variable_description.mnemonic + "|" + variable_description.parent + "|" + variable_description.target + "|" + variable_description.target_name + "|" + variable_description.expression + "%N")
			-- end
			if a_agent /= Void and then a_agent.compute_agent.item ([entity_date_target (a_target), Current]) then
				if a_agent.in_double_cache (entity_date_target (a_target), Void, Current) then
					Result := a_agent.double_cache_found_item
				else
					Result := computed_double_using_agent (a_target, a_agent)
					a_agent.force_double (Result, entity_date_target (a_target), Current)
				end
			elseif a_agent /= Void then
				l_value := a_agent.unitized_double (entity_date_target (a_target), Current)
				if l_value = Void then
					l_value := unitized_value (a_target, a_agent)
				end
				if l_value /= Void then
					if a_agent.currency /= Void then
						Result := l_value.converted_unitized_value (Void, a_agent.currency, a_agent.currency_date).value
					else
						Result := l_value.value
					end
				else
					Result := double_using_agent (a_target, a_agent)
				end
			else
				Result := double_using_agent (a_target, a_agent)
			end
		end

feature -- partial values treat undefined as 0 in addition and subtraction

	new_binary_expression (a_expression: like other_expression_template; a_operator: ROSE_BINARY_OPERATOR [like expression_target]): ROSE_BINARY_EXPRESSION [like expression_target] is
		do
			if a_operator /= Void then
				create {ROSE_BINARY_EXPRESSION [like expression_target]} Result.make (Current, a_expression, a_operator)
			end
		end

	partial_real_value (a_target: like target_anchor): REAL is
		do
			Result := real_value (a_target)
		end

	partial_double_value (a_target: like target_anchor): DOUBLE is
		do
			Result := double_value (a_target)
		end

	partial_decimal_value (a_target: like target_anchor): RP_DECIMAL is
		do
			Result := decimal_value (a_target)
		end

	partial_integer_value (a_target: like target_anchor): INTEGER is
		do
			Result := integer_value (a_target)
		end

	target_anchor: like expression_target is
			-- Temporary until sure that target_anchor is the same as expresion_target
		do
		end

	partial_value_in_market_currency (a_market: MARKET; a_currency: PERSISTENT_CURRENCY; a_target: like expression_target; a_date: ROSE_DATE): DOUBLE is
		local
			l_factor: DOUBLE
		do
			Result := partial_double_value (a_target)
			if is_defined_double (Result) then
				if a_currency /= Void then
					l_factor := nominal_units.currency_conversion_factor (a_currency.market_exchange_rate (a_market, currency (a_target), a_date))
					if is_defined_double (l_factor) then
						Result := Result * l_factor
					else
						Result := undefined_double
					end
				end
			end
		end

	partial_scaled_value_in_market_currency (a_market: MARKET; a_currency: PERSISTENT_CURRENCY; a_target: like expression_target; a_date: ROSE_DATE): DOUBLE is
		do
			Result := partial_value_in_market_currency (a_market, a_currency, a_target, a_date)
			if is_defined_double (Result) then
				Result := Result * nominal_units.scale.to_double
			end
		end

	instantiated (a_data: ANY): like Current is
		do
		end

	get_real_value (a_target: like expression_target): REAL is
		deferred
		end

	get_double_value (a_target: like expression_target): DOUBLE is
		deferred
		end

	get_integer_value (a_target: like expression_target): INTEGER is
		deferred
		end

feature -- Override families

	bypassing_overrides: BOOLEAN is
			-- Does `Current' bypass any override database?
		do
		end

	set_use_overrides (a_value: BOOLEAN) is
		do
		ensure
			consistent_bypassing_overrides: bypassing_overrides implies not a_value
			consistent_not_using_overrides: a_value implies not bypassing_overrides
		end

feature -- Status report

	defined (a_target: like expression_target): BOOLEAN is
			-- Is this expression defined at 'a_target'?
		do
			inspect type
			when Real_type then
				Result := is_defined_real (real_value (a_target))
			when Double_type then
				Result := is_defined_double (double_value (a_target))
			when Integer_type then
				Result := is_defined_integer (integer_value (a_target))
			when Character_type then
				Result := True
			when String_type then
				Result := is_defined_string (string_value (a_target))
			when Date_type then
				Result := is_defined_date (date_value (a_target))
			when Boolean_type then
				Result := True
			else
				Result := is_defined_double (double_value (a_target))
			end
		end

	partially_defined (a_target: like expression_target): BOOLEAN is
			-- Is this expression partially defined at 'a_target'?
		do
			inspect type
			when Real_type then
				Result := is_defined_real (partial_real_value (a_target))
			when Double_type then
				Result := is_defined_double (partial_double_value (a_target))
			when Integer_type then
				Result := is_defined_integer (partial_integer_value (a_target))
			when Character_type then
				Result := True
			when String_type then
				Result := is_defined_string (string_value (a_target))
			when Date_type then
				Result := is_defined_date (date_value (a_target))
			when Boolean_type then
				Result := True
			else
				Result := is_defined_double (partial_double_value (a_target))
			end
		end

feature -- Undefined Values

	is_defined_integer (a_value: INTEGER): BOOLEAN is
		deferred
		end

	is_defined_string (a_value: STRING): BOOLEAN is
		do
			Result := a_value /= Void
		end

	is_defined_date (a_value: ROSE_DATE): BOOLEAN is
		do
			Result := a_value /= Void
		end

	is_defined_date_time (a_value: ROSE_DATE): BOOLEAN is
		do
			Result := a_value /= Void
		end

feature -- Type check

	is_real: BOOLEAN is
		do
			Result := type = Real_type
		end

	is_double: BOOLEAN is
		do
			Result := type = Double_type
		end

	is_integer: BOOLEAN is
		do
			Result := type = Integer_type
		end

	is_character: BOOLEAN is
		do
			Result := type = Character_type
		end

	is_boolean: BOOLEAN is
		do
			Result := type = Boolean_type
		end

	is_string: BOOLEAN is
		do
			Result := type = String_type
		end

	is_date: BOOLEAN is
			-- Is it of type date?
		do
			Result := type = Date_type
		end

	is_date_time: BOOLEAN is
		do
			Result := type = Date_time_type
		end

	is_flow: BOOLEAN is
		do
			Result := variable_description.stock_or_flow = 'F'
		end

feature -- Output

	debug_output: STRING is
			-- something nice to see in the debugger
		do
			Result := name
			if Result = Void then
				Result := "Unknown"
			end
		end

feature {ROSE_EXPRESSION} -- Implementation

	computed_expressions: HASH_TABLE [ROSE_EXPRESSION [like expression_target], KEY_PAIR [ROSE_OPERATOR, INTEGER]] is
			--should be modified to be specific to the arity of the expression, to get rid of the assignment attempt in computed_binary_expression
		do
			if computed_expressions_saved = Void then
				create computed_expressions_saved.make (5)
			end
			Result := computed_expressions_saved
		end

	computed_expressions_saved: HASH_TABLE [ROSE_EXPRESSION [like expression_target], KEY_PAIR [ROSE_OPERATOR, INTEGER]]

	operator_expressions: PAIRS [ROSE_OPERATOR, INTEGER] is
		once
			create Result.make
		end

	set_expression_index is
			-- Sets `expression_index' to maintain invariant.
		do
			expression_counter.set_item (expression_counter.item + 1)
			expression_index := expression_counter.item
		ensure
			strictly_positive_expression_index: expression_index > 0
		end

	expression_index: INTEGER

	expression_counter: INTEGER_REF is
			-- expression counter
		once
			create Result
		end

	operators: like expression_operators_template is
			-- operators
		do
			Result ?= operators_table.item (internal.dynamic_type (Current))
			if Result = Void then
				create Result.make
				operators_table.put (Result, internal.dynamic_type (Current))
			end
		end

	operators_table: HASH_TABLE [ANY, INTEGER] is
			--cache for operators
		once
			create Result.make (100)
		end

feature {NONE} -- Anchors

	expression_target: detachable G is
			-- Expression target
		do
		end

	expression_operators_template: EXPRESSION_OPERATORS [like expression_target] is
		do
		end

	access_agent_template: GENERIC_ACCESS_CONTROL_AGENT is
			-- Access agent template
		do
		end

	moments_anchor: ROSE_MOMENTS is
			-- Moments anchor
		do
		end

	computed_expression_target: ROSE_EXPRESSION [like expression_target] is
			-- Computed expression target
		do
		end

	other_expression_template: ROSE_EXPRESSION [like expression_target] is
			--template
		do
		end

feature {USES_ANONYMOUS_EXPRESSION_VALUE} --Anonymous access

	expression_value (a_target: like expression_target): ANY is
			-- Untyped value of this expression at 'a_target'
		do
			inspect type
			when Real_type then
				Result := real_value (a_target)
			when Double_type then
				Result := double_value (a_target)
			when Integer_type then
				Result := integer_value (a_target)
			when Character_type then
				(create {EXCEPTIONS}).raise ("not implemented")
					--//\\--				Result := character_value (a_target)
			when String_type then
				Result := string_value (a_target)
			when Date_type then
				Result := date_value (a_target)
			when Date_time_type then
				Result := date_value (a_target)
			when Boolean_type then
				Result := boolean_value (a_target)
			else
				Result := double_value (a_target)
			end
		end

	partial_expression_value (a_target: like expression_target): ANY is
			-- Untyped value of this expression at 'a_target'
		do
			inspect type
			when Real_type then
				Result := partial_real_value (a_target)
			when Double_type then
				Result := partial_double_value (a_target)
			when Integer_type then
				Result := partial_integer_value (a_target)
			when Character_type then
				(create {EXCEPTIONS}).raise ("not implemented")
					--//\\--				Result := character_value (a_target)
			when String_type then
				Result := string_value (a_target)
			when Date_type then
				Result := date_value (a_target)
			when Boolean_type then
				Result := boolean_value (a_target)
			else
				Result := partial_double_value (a_target)
			end
		end

feature {NONE} --Implementation

	constant (a_value: DOUBLE): like computed_expression_target is
		do
			create {CONSTANT_EXPRESSION [G]} Result.make_double (a_value)
		end

	scale_saved: like scale
			-- Scale of units

	nominal_units_saved: like nominal_units
			-- Base units

end
