note
	description: "Parser for ROSE_EXPRESSION-s"
	author: "Zoran Simic"
	last_editor: "$Author: er $"
	date: "$Date: 2012/09/04 $"
	revision: "$Revision: #31 $"
	path: "$File: //rose/source/kernel/math/rose_expression_parser.e $"

class
	ROSE_EXPRESSION_PARSER [reference G]

inherit
	YY_NEW_PARSER_SKELETON
		rename
			make as make_parser_skeleton
		redefine
			report_error
		end

	ROSE_EXPRESSION_SCANNER

	ROSE_SHARED_GLOBAL

create
	make

feature {NONE} -- Initialization

	make (a_variables: ENTITY_VARIABLES [G]) is
			-- Create a new Rose expression-node parser.
		do
			make_rose_expression_scanner
			make_parser_skeleton
			set_variables (a_variables)
		end

feature -- Access

	variables: ENTITY_VARIABLES [G]
			-- Variables accessor

	cross_supplier_parser: CROSS_SUPPLIER_EXPRESSION_PARSER
			-- Parser for cross-supplier variables

	last_list: ROSE_LINEAR_ARRAY [ROSE_EXPRESSION [G]]
			-- Last list parsed, Void if an error occured

	last_parsed: ROSE_EXPRESSION [G]
			-- Last node parsed, Void if an error occured

	id: STRING is "expression parser"
			-- id of this section

feature -- Status setting

	set_variables (a_variables: ENTITY_VARIABLES [G]) is
			-- Set 'variables' as accessor.
		do
			variables := a_variables
		end

	set_cross_supplier_parser (a_parser: like cross_supplier_parser) is
			-- Set `cross_supplier_parser' to `a_parser'.
		require
			a_parser_not_void: a_parser /= Void
		do
			cross_supplier_parser := a_parser
		ensure
			cross_supplier_parser_set: cross_supplier_parser = a_parser
		end

feature -- Basic operations

	parse_file (a_path: STRING) is
			-- Parse expressions from 'a_path'.
			-- 'error.signaled' is true if an error occurs during this operation
			-- Use 'error.flush_and_acknowledge' to get the error message output
		require
			valid_path: a_path /= Void
			errors_acknowledged: not error.signaled
		local
			l_file: KL_TEXT_INPUT_FILE
			rescued: STRING
		do
			if rescued /= Void then
				error.signal (id,"The following problem occurred while trying to parse expressions from '$1':%N$2",<<a_path,rescued>>)
			end
			last_list := Void
			last_parsed := Void
			if not error.signaled then
				buffer_name := a_path
				create l_file.make (buffer_name)
				if not l_file.exists then
					error.signal (id,"Can not read '$1', file does not exist",<<buffer_name>>)
				elseif not l_file.is_readable then
					error.signal (id,"Can not read '$1', file is not readable",<<buffer_name>>)
				end
			end
			if not error.signaled then
				reset
				l_file.open_read
				set_input_buffer (new_file_buffer (l_file))
				parse
				l_file.close
			end
			buffer_name := Void
		rescue
			if rescued = Void then
				rescued :=  last_exception_explanation
				retry
			end
		end

	parsed_string (a_string: STRING): ROSE_EXPRESSION [G] is
			-- Parse expressions from 'a_string'.
			-- 'error.signaled' is true if an error occur during this operation
			-- Use 'error.flush_and_acknowledge' to get the error message output
		require
			valid_string: a_string /= Void
			errors_acknowledged: not error.signaled
		local
			rescued: STRING
		do
			if rescued /= Void then
				error.signal (id,"The following problem occurred while trying to parse expression '$1':%N$2",<<a_string,rescued>>)
			end
			last_list := Void
			last_parsed := Void
			if not error.signaled then
				buffer_name := "*buffer*"
				reset
				set_input_buffer (new_string_buffer (a_string))
				parse
				buffer_name := Void
			end
			Result := last_parsed
		rescue
			if rescued = Void then
				rescued :=  last_exception_explanation
				retry
			end
		end


feature {NONE} -- Implementation

	yy_build_parser_tables
			-- Build parser tables.
		do
			yytranslate := yytranslate_template
			yyr1 := yyr1_template
			yytypes1 := yytypes1_template
			yytypes2 := yytypes2_template
			yydefact := yydefact_template
			yydefgoto := yydefgoto_template
			yypact := yypact_template
			yypgoto := yypgoto_template
			yytable := yytable_template
			yycheck := yycheck_template
		end

	yy_create_value_stacks
			-- Create value stacks.
		do
		end

	yy_init_value_stacks
			-- Initialize value stacks.
		do
			yyvsp1 := -1
			yyvsp2 := -1
			yyvsp3 := -1
			yyvsp4 := -1
		end

	yy_clear_value_stacks
			-- Clear objects in semantic value stacks so that
			-- they can be collected by the garbage collector.
		local
			l_yyvs1_default_item: ANY
			l_yyvs2_default_item: like last_list
			l_yyvs3_default_item: like last_parsed
			l_yyvs4_default_item: STRING
		do
			if yyvs1 /= Void then
				yyvs1.fill_with (l_yyvs1_default_item, 0, yyvs1.upper)
			end
			if yyvs2 /= Void then
				yyvs2.fill_with (l_yyvs2_default_item, 0, yyvs2.upper)
			end
			if yyvs3 /= Void then
				yyvs3.fill_with (l_yyvs3_default_item, 0, yyvs3.upper)
			end
			if yyvs4 /= Void then
				yyvs4.fill_with (l_yyvs4_default_item, 0, yyvs4.upper)
			end
		end

	yy_push_last_value (yychar1: INTEGER)
			-- Push semantic value associated with token `last_token'
			-- (with internal id `yychar1') on top of corresponding
			-- value stack.
		do
			inspect yytypes2.item (yychar1)
			when 1 then
				yyvsp1 := yyvsp1 + 1
				if yyvsp1 >= yyvsc1 then
					if yyvs1 = Void then
						debug ("GEYACC")
							std.error.put_line ("Create yyvs1")
						end
						create yyspecial_routines1
						yyvsc1 := yyInitial_yyvs_size
						yyvs1 := yyspecial_routines1.make (yyvsc1)
					else
						debug ("GEYACC")
							std.error.put_line ("Resize yyvs1")
						end
						yyvsc1 := yyvsc1 + yyInitial_yyvs_size
						yyvs1 := yyspecial_routines1.resize (yyvs1, yyvsc1)
					end
				end
				yyspecial_routines1.force (yyvs1, last_any_value, yyvsp1)
			when 4 then
				yyvsp4 := yyvsp4 + 1
				if yyvsp4 >= yyvsc4 then
					if yyvs4 = Void then
						debug ("GEYACC")
							std.error.put_line ("Create yyvs4")
						end
						create yyspecial_routines4
						yyvsc4 := yyInitial_yyvs_size
						yyvs4 := yyspecial_routines4.make (yyvsc4)
					else
						debug ("GEYACC")
							std.error.put_line ("Resize yyvs4")
						end
						yyvsc4 := yyvsc4 + yyInitial_yyvs_size
						yyvs4 := yyspecial_routines4.resize (yyvs4, yyvsc4)
					end
				end
				yyspecial_routines4.force (yyvs4, last_string_value, yyvsp4)
			else
				debug ("GEYACC")
					std.error.put_string ("Error in parser: not a token type: ")
					std.error.put_integer (yytypes2.item (yychar1))
					std.error.put_new_line
				end
				abort
			end
		end

	yy_push_error_value
			-- Push semantic value associated with token 'error'
			-- on top of corresponding value stack.
		local
			yyval1: ANY
		do
			yyvsp1 := yyvsp1 + 1
			if yyvsp1 >= yyvsc1 then
				if yyvs1 = Void then
					debug ("GEYACC")
						std.error.put_line ("Create yyvs1")
					end
					create yyspecial_routines1
					yyvsc1 := yyInitial_yyvs_size
					yyvs1 := yyspecial_routines1.make (yyvsc1)
				else
					debug ("GEYACC")
						std.error.put_line ("Resize yyvs1")
					end
					yyvsc1 := yyvsc1 + yyInitial_yyvs_size
					yyvs1 := yyspecial_routines1.resize (yyvs1, yyvsc1)
				end
			end
			yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
		end

	yy_pop_last_value (yystate: INTEGER)
			-- Pop semantic value from stack when in state `yystate'.
		local
			yy_type_id: INTEGER
		do
			yy_type_id := yytypes1.item (yystate)
			inspect yy_type_id
			when 1 then
				yyvsp1 := yyvsp1 - 1
			when 2 then
				yyvsp2 := yyvsp2 - 1
			when 3 then
				yyvsp3 := yyvsp3 - 1
			when 4 then
				yyvsp4 := yyvsp4 - 1
			else
				debug ("GEYACC")
					std.error.put_string ("Error in parser: unknown type id: ")
					std.error.put_integer (yy_type_id)
					std.error.put_new_line
				end
				abort
			end
		end

	yy_run_geyacc
			-- You must run geyacc to regenerate this class.
		do
		end

feature {NONE} -- Semantic actions

	yy_do_action (yy_act: INTEGER)
			-- Execute semantic action.
		local
			yyval1: ANY
			yyval2: like last_list
			yyval3: like last_parsed
		do
			inspect yy_act
when 1 then
--|#line 166 "..\..\External\parsing\rose_expression_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file '..\..\External\parsing\rose_expression_parser.y' at line 166")
end

accept
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 2 then
--|#line 168 "..\..\External\parsing\rose_expression_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file '..\..\External\parsing\rose_expression_parser.y' at line 168")
end

last_list := Void 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 0
	yyvsp2 := yyvsp2 + 1
	if yyvsp2 >= yyvsc2 then
		if yyvs2 = Void then
			debug ("GEYACC")
				std.error.put_line ("Create yyvs2")
			end
			create yyspecial_routines2
			yyvsc2 := yyInitial_yyvs_size
			yyvs2 := yyspecial_routines2.make (yyvsc2)
		else
			debug ("GEYACC")
				std.error.put_line ("Resize yyvs2")
			end
			yyvsc2 := yyvsc2 + yyInitial_yyvs_size
			yyvs2 := yyspecial_routines2.resize (yyvs2, yyvsc2)
		end
	end
	yyspecial_routines2.force (yyvs2, yyval2, yyvsp2)
end
when 3 then
--|#line 169 "..\..\External\parsing\rose_expression_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file '..\..\External\parsing\rose_expression_parser.y' at line 169")
end

add_expression (yyvs3.item (yyvsp3)) 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp2 := yyvsp2 + 1
	yyvsp3 := yyvsp3 -1
	if yyvsp2 >= yyvsc2 then
		if yyvs2 = Void then
			debug ("GEYACC")
				std.error.put_line ("Create yyvs2")
			end
			create yyspecial_routines2
			yyvsc2 := yyInitial_yyvs_size
			yyvs2 := yyspecial_routines2.make (yyvsc2)
		else
			debug ("GEYACC")
				std.error.put_line ("Resize yyvs2")
			end
			yyvsc2 := yyvsc2 + yyInitial_yyvs_size
			yyvs2 := yyspecial_routines2.resize (yyvs2, yyvsc2)
		end
	end
	yyspecial_routines2.force (yyvs2, yyval2, yyvsp2)
end
when 4 then
--|#line 170 "..\..\External\parsing\rose_expression_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file '..\..\External\parsing\rose_expression_parser.y' at line 170")
end

add_expression (yyvs3.item (yyvsp3)) 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp1 := yyvsp1 -1
	yyvsp3 := yyvsp3 -1
	yyspecial_routines2.force (yyvs2, yyval2, yyvsp2)
end
when 5 then
--|#line 173 "..\..\External\parsing\rose_expression_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file '..\..\External\parsing\rose_expression_parser.y' at line 173")
end

yyval2 := singleton_list (yyvs3.item (yyvsp3)) 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp2 := yyvsp2 + 1
	yyvsp3 := yyvsp3 -1
	if yyvsp2 >= yyvsc2 then
		if yyvs2 = Void then
			debug ("GEYACC")
				std.error.put_line ("Create yyvs2")
			end
			create yyspecial_routines2
			yyvsc2 := yyInitial_yyvs_size
			yyvs2 := yyspecial_routines2.make (yyvsc2)
		else
			debug ("GEYACC")
				std.error.put_line ("Resize yyvs2")
			end
			yyvsc2 := yyvsc2 + yyInitial_yyvs_size
			yyvs2 := yyspecial_routines2.resize (yyvs2, yyvsc2)
		end
	end
	yyspecial_routines2.force (yyvs2, yyval2, yyvsp2)
end
when 6 then
--|#line 174 "..\..\External\parsing\rose_expression_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file '..\..\External\parsing\rose_expression_parser.y' at line 174")
end

yyval2 := append_to_list (yyvs2.item (yyvsp2), yyvs3.item (yyvsp3)) 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp1 := yyvsp1 -1
	yyvsp3 := yyvsp3 -1
	yyspecial_routines2.force (yyvs2, yyval2, yyvsp2)
end
when 7 then
--|#line 177 "..\..\External\parsing\rose_expression_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file '..\..\External\parsing\rose_expression_parser.y' at line 177")
end

yyval3 := yyvs3.item (yyvsp3) 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines3.force (yyvs3, yyval3, yyvsp3)
end
when 8 then
--|#line 178 "..\..\External\parsing\rose_expression_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file '..\..\External\parsing\rose_expression_parser.y' at line 178")
end

yyval3 := yyvs3.item (yyvsp3) 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp1 := yyvsp1 -2
	yyspecial_routines3.force (yyvs3, yyval3, yyvsp3)
end
when 9 then
--|#line 179 "..\..\External\parsing\rose_expression_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file '..\..\External\parsing\rose_expression_parser.y' at line 179")
end

create {CONSTANT_BOOLEAN_EXPRESSION [G]} yyval3.make_boolean (True) 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp3 := yyvsp3 + 1
	yyvsp1 := yyvsp1 -1
	if yyvsp3 >= yyvsc3 then
		if yyvs3 = Void then
			debug ("GEYACC")
				std.error.put_line ("Create yyvs3")
			end
			create yyspecial_routines3
			yyvsc3 := yyInitial_yyvs_size
			yyvs3 := yyspecial_routines3.make (yyvsc3)
		else
			debug ("GEYACC")
				std.error.put_line ("Resize yyvs3")
			end
			yyvsc3 := yyvsc3 + yyInitial_yyvs_size
			yyvs3 := yyspecial_routines3.resize (yyvs3, yyvsc3)
		end
	end
	yyspecial_routines3.force (yyvs3, yyval3, yyvsp3)
end
when 10 then
--|#line 180 "..\..\External\parsing\rose_expression_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file '..\..\External\parsing\rose_expression_parser.y' at line 180")
end

create {CONSTANT_BOOLEAN_EXPRESSION [G]} yyval3.make_boolean (False) 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp3 := yyvsp3 + 1
	yyvsp1 := yyvsp1 -1
	if yyvsp3 >= yyvsc3 then
		if yyvs3 = Void then
			debug ("GEYACC")
				std.error.put_line ("Create yyvs3")
			end
			create yyspecial_routines3
			yyvsc3 := yyInitial_yyvs_size
			yyvs3 := yyspecial_routines3.make (yyvsc3)
		else
			debug ("GEYACC")
				std.error.put_line ("Resize yyvs3")
			end
			yyvsc3 := yyvsc3 + yyInitial_yyvs_size
			yyvs3 := yyspecial_routines3.resize (yyvs3, yyvsc3)
		end
	end
	yyspecial_routines3.force (yyvs3, yyval3, yyvsp3)
end
when 11 then
--|#line 181 "..\..\External\parsing\rose_expression_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file '..\..\External\parsing\rose_expression_parser.y' at line 181")
end

create {STRING_CONSTANT_EXPRESSION [G]} yyval3.make_string (yyvs4.item (yyvsp4)) 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp3 := yyvsp3 + 1
	yyvsp4 := yyvsp4 -1
	if yyvsp3 >= yyvsc3 then
		if yyvs3 = Void then
			debug ("GEYACC")
				std.error.put_line ("Create yyvs3")
			end
			create yyspecial_routines3
			yyvsc3 := yyInitial_yyvs_size
			yyvs3 := yyspecial_routines3.make (yyvsc3)
		else
			debug ("GEYACC")
				std.error.put_line ("Resize yyvs3")
			end
			yyvsc3 := yyvsc3 + yyInitial_yyvs_size
			yyvs3 := yyspecial_routines3.resize (yyvs3, yyvsc3)
		end
	end
	yyspecial_routines3.force (yyvs3, yyval3, yyvsp3)
end
when 12 then
--|#line 182 "..\..\External\parsing\rose_expression_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file '..\..\External\parsing\rose_expression_parser.y' at line 182")
end

create {CONSTANT_EXPRESSION [G]} yyval3.make_integer (yyvs4.item (yyvsp4).to_integer) 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp3 := yyvsp3 + 1
	yyvsp4 := yyvsp4 -1
	if yyvsp3 >= yyvsc3 then
		if yyvs3 = Void then
			debug ("GEYACC")
				std.error.put_line ("Create yyvs3")
			end
			create yyspecial_routines3
			yyvsc3 := yyInitial_yyvs_size
			yyvs3 := yyspecial_routines3.make (yyvsc3)
		else
			debug ("GEYACC")
				std.error.put_line ("Resize yyvs3")
			end
			yyvsc3 := yyvsc3 + yyInitial_yyvs_size
			yyvs3 := yyspecial_routines3.resize (yyvs3, yyvsc3)
		end
	end
	yyspecial_routines3.force (yyvs3, yyval3, yyvsp3)
end
when 13 then
--|#line 183 "..\..\External\parsing\rose_expression_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file '..\..\External\parsing\rose_expression_parser.y' at line 183")
end

create {CONSTANT_EXPRESSION [G]} yyval3.make_double (yyvs4.item (yyvsp4).to_double) 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp3 := yyvsp3 + 1
	yyvsp4 := yyvsp4 -1
	if yyvsp3 >= yyvsc3 then
		if yyvs3 = Void then
			debug ("GEYACC")
				std.error.put_line ("Create yyvs3")
			end
			create yyspecial_routines3
			yyvsc3 := yyInitial_yyvs_size
			yyvs3 := yyspecial_routines3.make (yyvsc3)
		else
			debug ("GEYACC")
				std.error.put_line ("Resize yyvs3")
			end
			yyvsc3 := yyvsc3 + yyInitial_yyvs_size
			yyvs3 := yyspecial_routines3.resize (yyvs3, yyvsc3)
		end
	end
	yyspecial_routines3.force (yyvs3, yyval3, yyvsp3)
end
when 14 then
--|#line 184 "..\..\External\parsing\rose_expression_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file '..\..\External\parsing\rose_expression_parser.y' at line 184")
end

create {STRING_CONSTANT_EXPRESSION [G]} yyval3.make_string (yyvs4.item (yyvsp4)) 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp3 := yyvsp3 + 1
	yyvsp4 := yyvsp4 -1
	if yyvsp3 >= yyvsc3 then
		if yyvs3 = Void then
			debug ("GEYACC")
				std.error.put_line ("Create yyvs3")
			end
			create yyspecial_routines3
			yyvsc3 := yyInitial_yyvs_size
			yyvs3 := yyspecial_routines3.make (yyvsc3)
		else
			debug ("GEYACC")
				std.error.put_line ("Resize yyvs3")
			end
			yyvsc3 := yyvsc3 + yyInitial_yyvs_size
			yyvs3 := yyspecial_routines3.resize (yyvs3, yyvsc3)
		end
	end
	yyspecial_routines3.force (yyvs3, yyval3, yyvsp3)
end
when 15 then
--|#line 185 "..\..\External\parsing\rose_expression_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file '..\..\External\parsing\rose_expression_parser.y' at line 185")
end

yyval3 := get_unary (yyvs3.item (yyvsp3),"+") 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp1 := yyvsp1 -1
	yyspecial_routines3.force (yyvs3, yyval3, yyvsp3)
end
when 16 then
--|#line 186 "..\..\External\parsing\rose_expression_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file '..\..\External\parsing\rose_expression_parser.y' at line 186")
end

yyval3 := get_unary (yyvs3.item (yyvsp3),"-") 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp1 := yyvsp1 -1
	yyspecial_routines3.force (yyvs3, yyval3, yyvsp3)
end
when 17 then
--|#line 187 "..\..\External\parsing\rose_expression_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file '..\..\External\parsing\rose_expression_parser.y' at line 187")
end

yyval3 := get_unary (yyvs3.item (yyvsp3),"not ") 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp1 := yyvsp1 -1
	yyspecial_routines3.force (yyvs3, yyval3, yyvsp3)
end
when 18 then
--|#line 188 "..\..\External\parsing\rose_expression_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file '..\..\External\parsing\rose_expression_parser.y' at line 188")
end

yyval3 := get_unary (yyvs3.item (yyvsp3),"abs ") 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp1 := yyvsp1 -1
	yyspecial_routines3.force (yyvs3, yyval3, yyvsp3)
end
when 19 then
--|#line 189 "..\..\External\parsing\rose_expression_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file '..\..\External\parsing\rose_expression_parser.y' at line 189")
end

yyval3 := get_unary (yyvs3.item (yyvsp3),"log ") 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp1 := yyvsp1 -1
	yyspecial_routines3.force (yyvs3, yyval3, yyvsp3)
end
when 20 then
--|#line 190 "..\..\External\parsing\rose_expression_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file '..\..\External\parsing\rose_expression_parser.y' at line 190")
end

yyval3 := get_binary (yyvs3.item (yyvsp3 - 1),yyvs3.item (yyvsp3),"+") 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp3 := yyvsp3 -1
	yyvsp1 := yyvsp1 -1
	yyspecial_routines3.force (yyvs3, yyval3, yyvsp3)
end
when 21 then
--|#line 191 "..\..\External\parsing\rose_expression_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file '..\..\External\parsing\rose_expression_parser.y' at line 191")
end

yyval3 := get_binary (yyvs3.item (yyvsp3 - 1),yyvs3.item (yyvsp3),"-") 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp3 := yyvsp3 -1
	yyvsp1 := yyvsp1 -1
	yyspecial_routines3.force (yyvs3, yyval3, yyvsp3)
end
when 22 then
--|#line 192 "..\..\External\parsing\rose_expression_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file '..\..\External\parsing\rose_expression_parser.y' at line 192")
end

yyval3 := get_binary (yyvs3.item (yyvsp3 - 1),yyvs3.item (yyvsp3),"#~") 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp3 := yyvsp3 -1
	yyvsp1 := yyvsp1 -1
	yyspecial_routines3.force (yyvs3, yyval3, yyvsp3)
end
when 23 then
--|#line 193 "..\..\External\parsing\rose_expression_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file '..\..\External\parsing\rose_expression_parser.y' at line 193")
end

yyval3 := get_binary (yyvs3.item (yyvsp3 - 1),yyvs3.item (yyvsp3),"*") 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp3 := yyvsp3 -1
	yyvsp1 := yyvsp1 -1
	yyspecial_routines3.force (yyvs3, yyval3, yyvsp3)
end
when 24 then
--|#line 194 "..\..\External\parsing\rose_expression_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file '..\..\External\parsing\rose_expression_parser.y' at line 194")
end

yyval3 := get_binary (yyvs3.item (yyvsp3 - 1),yyvs3.item (yyvsp3),"/") 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp3 := yyvsp3 -1
	yyvsp1 := yyvsp1 -1
	yyspecial_routines3.force (yyvs3, yyval3, yyvsp3)
end
when 25 then
--|#line 195 "..\..\External\parsing\rose_expression_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file '..\..\External\parsing\rose_expression_parser.y' at line 195")
end

yyval3 := get_binary (yyvs3.item (yyvsp3 - 1),yyvs3.item (yyvsp3),"^") 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp3 := yyvsp3 -1
	yyvsp1 := yyvsp1 -1
	yyspecial_routines3.force (yyvs3, yyval3, yyvsp3)
end
when 26 then
--|#line 196 "..\..\External\parsing\rose_expression_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file '..\..\External\parsing\rose_expression_parser.y' at line 196")
end

yyval3 := get_binary (yyvs3.item (yyvsp3 - 1),yyvs3.item (yyvsp3),"@") 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp3 := yyvsp3 -1
	yyvsp1 := yyvsp1 -1
	yyspecial_routines3.force (yyvs3, yyval3, yyvsp3)
end
when 27 then
--|#line 197 "..\..\External\parsing\rose_expression_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file '..\..\External\parsing\rose_expression_parser.y' at line 197")
end

yyval3 := get_binary (yyvs3.item (yyvsp3 - 1),yyvs3.item (yyvsp3),"div ") 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp3 := yyvsp3 -1
	yyvsp1 := yyvsp1 -1
	yyspecial_routines3.force (yyvs3, yyval3, yyvsp3)
end
when 28 then
--|#line 198 "..\..\External\parsing\rose_expression_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file '..\..\External\parsing\rose_expression_parser.y' at line 198")
end

yyval3 := get_binary (yyvs3.item (yyvsp3 - 1),yyvs3.item (yyvsp3),"mod ") 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp3 := yyvsp3 -1
	yyvsp1 := yyvsp1 -1
	yyspecial_routines3.force (yyvs3, yyval3, yyvsp3)
end
when 29 then
--|#line 199 "..\..\External\parsing\rose_expression_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file '..\..\External\parsing\rose_expression_parser.y' at line 199")
end

yyval3 := get_binary (yyvs3.item (yyvsp3 - 1),yyvs3.item (yyvsp3),"=") 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp3 := yyvsp3 -1
	yyvsp1 := yyvsp1 -1
	yyspecial_routines3.force (yyvs3, yyval3, yyvsp3)
end
when 30 then
--|#line 200 "..\..\External\parsing\rose_expression_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file '..\..\External\parsing\rose_expression_parser.y' at line 200")
end

yyval3 := get_binary (yyvs3.item (yyvsp3 - 1),yyvs3.item (yyvsp3),"<>") 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp3 := yyvsp3 -1
	yyvsp1 := yyvsp1 -1
	yyspecial_routines3.force (yyvs3, yyval3, yyvsp3)
end
when 31 then
--|#line 201 "..\..\External\parsing\rose_expression_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file '..\..\External\parsing\rose_expression_parser.y' at line 201")
end

yyval3 := get_binary (yyvs3.item (yyvsp3 - 1),yyvs3.item (yyvsp3),"<") 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp3 := yyvsp3 -1
	yyvsp1 := yyvsp1 -1
	yyspecial_routines3.force (yyvs3, yyval3, yyvsp3)
end
when 32 then
--|#line 202 "..\..\External\parsing\rose_expression_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file '..\..\External\parsing\rose_expression_parser.y' at line 202")
end

yyval3 := get_binary (yyvs3.item (yyvsp3 - 1),yyvs3.item (yyvsp3),">") 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp3 := yyvsp3 -1
	yyvsp1 := yyvsp1 -1
	yyspecial_routines3.force (yyvs3, yyval3, yyvsp3)
end
when 33 then
--|#line 203 "..\..\External\parsing\rose_expression_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file '..\..\External\parsing\rose_expression_parser.y' at line 203")
end

yyval3 := get_binary (yyvs3.item (yyvsp3 - 1),yyvs3.item (yyvsp3),"<=") 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp3 := yyvsp3 -1
	yyvsp1 := yyvsp1 -1
	yyspecial_routines3.force (yyvs3, yyval3, yyvsp3)
end
when 34 then
--|#line 204 "..\..\External\parsing\rose_expression_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file '..\..\External\parsing\rose_expression_parser.y' at line 204")
end

yyval3 := get_binary (yyvs3.item (yyvsp3 - 1),yyvs3.item (yyvsp3),">=") 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp3 := yyvsp3 -1
	yyvsp1 := yyvsp1 -1
	yyspecial_routines3.force (yyvs3, yyval3, yyvsp3)
end
when 35 then
--|#line 205 "..\..\External\parsing\rose_expression_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file '..\..\External\parsing\rose_expression_parser.y' at line 205")
end

yyval3 := get_binary (yyvs3.item (yyvsp3 - 1),yyvs3.item (yyvsp3),"and ") 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp3 := yyvsp3 -1
	yyvsp1 := yyvsp1 -1
	yyspecial_routines3.force (yyvs3, yyval3, yyvsp3)
end
when 36 then
--|#line 206 "..\..\External\parsing\rose_expression_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file '..\..\External\parsing\rose_expression_parser.y' at line 206")
end

yyval3 := get_binary (yyvs3.item (yyvsp3 - 1),yyvs3.item (yyvsp3),"or ") 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp3 := yyvsp3 -1
	yyvsp1 := yyvsp1 -1
	yyspecial_routines3.force (yyvs3, yyval3, yyvsp3)
end
when 37 then
--|#line 207 "..\..\External\parsing\rose_expression_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file '..\..\External\parsing\rose_expression_parser.y' at line 207")
end

yyval3 := get_binary (yyvs3.item (yyvsp3 - 1),yyvs3.item (yyvsp3),"xor ") 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp3 := yyvsp3 -1
	yyvsp1 := yyvsp1 -1
	yyspecial_routines3.force (yyvs3, yyval3, yyvsp3)
end
when 38 then
--|#line 208 "..\..\External\parsing\rose_expression_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file '..\..\External\parsing\rose_expression_parser.y' at line 208")
end

yyval3 := get_binary (yyvs3.item (yyvsp3 - 1),yyvs3.item (yyvsp3),"implies ") 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp3 := yyvsp3 -1
	yyvsp1 := yyvsp1 -1
	yyspecial_routines3.force (yyvs3, yyval3, yyvsp3)
end
when 39 then
--|#line 211 "..\..\External\parsing\rose_expression_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file '..\..\External\parsing\rose_expression_parser.y' at line 211")
end

yyval3 := get_variable (yyvs4.item (yyvsp4),Void) 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp3 := yyvsp3 + 1
	yyvsp4 := yyvsp4 -1
	if yyvsp3 >= yyvsc3 then
		if yyvs3 = Void then
			debug ("GEYACC")
				std.error.put_line ("Create yyvs3")
			end
			create yyspecial_routines3
			yyvsc3 := yyInitial_yyvs_size
			yyvs3 := yyspecial_routines3.make (yyvsc3)
		else
			debug ("GEYACC")
				std.error.put_line ("Resize yyvs3")
			end
			yyvsc3 := yyvsc3 + yyInitial_yyvs_size
			yyvs3 := yyspecial_routines3.resize (yyvs3, yyvsc3)
		end
	end
	yyspecial_routines3.force (yyvs3, yyval3, yyvsp3)
end
when 40 then
--|#line 213 "..\..\External\parsing\rose_expression_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file '..\..\External\parsing\rose_expression_parser.y' at line 213")
end

yyval3 := get_variable (yyvs4.item (yyvsp4 - 1),yyvs4.item (yyvsp4)) 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp3 := yyvsp3 + 1
	yyvsp4 := yyvsp4 -2
	yyvsp1 := yyvsp1 -1
	if yyvsp3 >= yyvsc3 then
		if yyvs3 = Void then
			debug ("GEYACC")
				std.error.put_line ("Create yyvs3")
			end
			create yyspecial_routines3
			yyvsc3 := yyInitial_yyvs_size
			yyvs3 := yyspecial_routines3.make (yyvsc3)
		else
			debug ("GEYACC")
				std.error.put_line ("Resize yyvs3")
			end
			yyvsc3 := yyvsc3 + yyInitial_yyvs_size
			yyvs3 := yyspecial_routines3.resize (yyvs3, yyvsc3)
		end
	end
	yyspecial_routines3.force (yyvs3, yyval3, yyvsp3)
end
when 41 then
--|#line 214 "..\..\External\parsing\rose_expression_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file '..\..\External\parsing\rose_expression_parser.y' at line 214")
end

yyval3 := get_function (yyvs4.item (yyvsp4), yyvs2.item (yyvsp2)) 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 4
	yyvsp3 := yyvsp3 + 1
	yyvsp4 := yyvsp4 -1
	yyvsp1 := yyvsp1 -2
	yyvsp2 := yyvsp2 -1
	if yyvsp3 >= yyvsc3 then
		if yyvs3 = Void then
			debug ("GEYACC")
				std.error.put_line ("Create yyvs3")
			end
			create yyspecial_routines3
			yyvsc3 := yyInitial_yyvs_size
			yyvs3 := yyspecial_routines3.make (yyvsc3)
		else
			debug ("GEYACC")
				std.error.put_line ("Resize yyvs3")
			end
			yyvsc3 := yyvsc3 + yyInitial_yyvs_size
			yyvs3 := yyspecial_routines3.resize (yyvs3, yyvsc3)
		end
	end
	yyspecial_routines3.force (yyvs3, yyval3, yyvsp3)
end
when 42 then
--|#line 215 "..\..\External\parsing\rose_expression_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file '..\..\External\parsing\rose_expression_parser.y' at line 215")
end

yyval3 := get_cross_supplier_variable (yyvs4.item (yyvsp4),Void) 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp3 := yyvsp3 + 1
	yyvsp1 := yyvsp1 -2
	yyvsp4 := yyvsp4 -1
	if yyvsp3 >= yyvsc3 then
		if yyvs3 = Void then
			debug ("GEYACC")
				std.error.put_line ("Create yyvs3")
			end
			create yyspecial_routines3
			yyvsc3 := yyInitial_yyvs_size
			yyvs3 := yyspecial_routines3.make (yyvsc3)
		else
			debug ("GEYACC")
				std.error.put_line ("Resize yyvs3")
			end
			yyvsc3 := yyvsc3 + yyInitial_yyvs_size
			yyvs3 := yyspecial_routines3.resize (yyvs3, yyvsc3)
		end
	end
	yyspecial_routines3.force (yyvs3, yyval3, yyvsp3)
end
when 43 then
--|#line 216 "..\..\External\parsing\rose_expression_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file '..\..\External\parsing\rose_expression_parser.y' at line 216")
end

yyval3 := get_cross_supplier_variable (yyvs4.item (yyvsp4 - 1),yyvs4.item (yyvsp4)) 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 5
	yyvsp3 := yyvsp3 + 1
	yyvsp1 := yyvsp1 -3
	yyvsp4 := yyvsp4 -2
	if yyvsp3 >= yyvsc3 then
		if yyvs3 = Void then
			debug ("GEYACC")
				std.error.put_line ("Create yyvs3")
			end
			create yyspecial_routines3
			yyvsc3 := yyInitial_yyvs_size
			yyvs3 := yyspecial_routines3.make (yyvsc3)
		else
			debug ("GEYACC")
				std.error.put_line ("Resize yyvs3")
			end
			yyvsc3 := yyvsc3 + yyInitial_yyvs_size
			yyvs3 := yyspecial_routines3.resize (yyvs3, yyvsc3)
		end
	end
	yyspecial_routines3.force (yyvs3, yyval3, yyvsp3)
end
			else
				debug ("GEYACC")
					std.error.put_string ("Error in parser: unknown rule id: ")
					std.error.put_integer (yy_act)
					std.error.put_new_line
				end
				abort
			end
		end

	yy_do_error_action (yy_act: INTEGER)
			-- Execute error action.
		do
			inspect yy_act
			when 78 then
					-- End-of-file expected action.
				report_eof_expected_error
			else
					-- Default action.
				report_error ("parse error")
			end
		end

feature {NONE} -- Table templates

	yytranslate_template: SPECIAL [INTEGER]
			-- Template for `yytranslate'
		once
			Result := yyfixed_array (<<
			    0,    2,    2,    2,    2,    2,    2,    2,    2,    2,
			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
			   35,   36,   25,   22,   34,   23,   37,   26,    2,    2,
			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
			   18,   16,   19,    2,   30,    2,    2,    2,    2,    2,
			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
			    2,    2,    2,    2,   29,    2,    2,    2,    2,    2,

			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,

			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
			    2,    2,    2,    2,    2,    2,    1,    2,    3,    4,
			    5,    6,    7,    8,    9,   10,   11,   12,   13,   14,
			   15,   17,   20,   21,   24,   27,   28,   31,   32,   33, yyDummy>>)
		end

	yyr1_template: SPECIAL [INTEGER]
			-- Template for `yyr1'
		once
			Result := yyfixed_array (<<
			    0,   42,   38,   38,   38,   39,   39,   41,   41,   41,
			   41,   41,   41,   41,   41,   41,   41,   41,   41,   41,
			   41,   41,   41,   41,   41,   41,   41,   41,   41,   41,
			   41,   41,   41,   41,   41,   41,   41,   41,   41,   40,
			   40,   40,   40,   40, yyDummy>>)
		end

	yytypes1_template: SPECIAL [INTEGER]
			-- Template for `yytypes1'
		once
			Result := yyfixed_array (<<
			    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
			    4,    4,    4,    4,    4,    2,    3,    3,    1,    3,
			    3,    3,    3,    3,    3,    1,    1,    1,    1,    1,
			    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
			    1,    1,    1,    1,    1,    1,    1,    1,    4,    1,
			    4,    2,    3,    3,    3,    3,    3,    3,    3,    3,
			    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,
			    3,    3,    3,    1,    1,    1,    4,    3,    1,    1,
			    1, yyDummy>>)
		end

	yytypes2_template: SPECIAL [INTEGER]
			-- Template for `yytypes2'
		once
			Result := yyfixed_array (<<
			    1,    1,    1,    4,    4,    4,    4,    4,    1,    1,
			    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
			    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
			    1,    1,    1,    1,    1,    1,    1,    1, yyDummy>>)
		end

	yydefact_template: SPECIAL [INTEGER]
			-- Template for `yydefact'
		once
			Result := yyfixed_array (<<
			    2,    0,    0,    0,    0,    0,    0,    0,   10,    9,
			   39,   14,   13,   12,   11,    0,    7,    3,    0,    0,
			   19,   18,   17,   16,   15,    0,    0,    0,    1,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,   42,    8,
			   40,    0,    5,    4,   26,   25,   28,   27,   24,   23,
			   22,   21,   20,   34,   33,   32,   31,   30,   29,   35,
			   37,   36,   38,    0,   41,    0,   43,    6,    0,    0,
			    0, yyDummy>>)
		end

	yydefgoto_template: SPECIAL [INTEGER]
			-- Template for `yydefgoto'
		once
			Result := yyfixed_array (<<
			   15,   51,   16,   17,   78, yyDummy>>)
		end

	yypact_template: SPECIAL [INTEGER]
			-- Template for `yypact'
		once
			Result := yyfixed_array (<<
			   43,   51,   43,   43,   43,   43,   43,   43, -32768, -32768,
			   33, -32768, -32768, -32768, -32768,   45, -32768,   94,   64,   69,
			 -32768, -32768, -32768, -32768, -32768,   62,   43,   43, -32768,   43,
			   43,   43,   43,   43,   43,   43,   43,   43,   43,   43,
			   43,   43,   43,   43,   43,   43,   43,   43,   35, -32768,
			 -32768,   19,   94,   94,   34,   34,   34,   34,   34,   34,
			   32,   32,   32,  134,  134,  134,  134,  134,  134,  125,
			  110,  110,   -7,   60, -32768,   43, -32768,   94,   54,   26,
			 -32768, yyDummy>>)
		end

	yypgoto_template: SPECIAL [INTEGER]
			-- Template for `yypgoto'
		once
			Result := yyfixed_array (<<
			 -32768, -32768, -32768,   -2, -32768, yyDummy>>)
		end

	yytable_template: SPECIAL [INTEGER]
			-- Template for `yytable'
		once
			Result := yyfixed_array (<<
			   19,   20,   21,   22,   23,   24,   46,   45,   44,   43,
			   42,   41,   40,   39,   38,   37,   36,   35,   34,   33,
			   32,   31,   30,   29,   52,   53,   80,   54,   55,   56,
			   57,   58,   59,   60,   61,   62,   63,   64,   65,   66,
			   67,   68,   69,   70,   71,   72,   14,   13,   12,   11,
			   10,    9,    8,   75,   79,   74,   28,   34,   33,   32,
			   31,   30,   29,   30,   29,    7,    6,   76,   26,   50,
			   25,   48,   73,   77,    5,    4,    3,   18,    2,   27,
			    1,   47,   46,   45,   44,   43,   42,   41,   40,   39,
			   38,   37,   36,   35,   34,   33,   32,   31,   30,   29,

			    0,    0,    0,    0,    0,   49,   47,   46,   45,   44,
			   43,   42,   41,   40,   39,   38,   37,   36,   35,   34,
			   33,   32,   31,   30,   29,   44,   43,   42,   41,   40,
			   39,   38,   37,   36,   35,   34,   33,   32,   31,   30,
			   29,   43,   42,   41,   40,   39,   38,   37,   36,   35,
			   34,   33,   32,   31,   30,   29,   37,   36,   35,   34,
			   33,   32,   31,   30,   29, yyDummy>>)
		end

	yycheck_template: SPECIAL [INTEGER]
			-- Template for `yycheck'
		once
			Result := yyfixed_array (<<
			    2,    3,    4,    5,    6,    7,   13,   14,   15,   16,
			   17,   18,   19,   20,   21,   22,   23,   24,   25,   26,
			   27,   28,   29,   30,   26,   27,    0,   29,   30,   31,
			   32,   33,   34,   35,   36,   37,   38,   39,   40,   41,
			   42,   43,   44,   45,   46,   47,    3,    4,    5,    6,
			    7,    8,    9,   34,    0,   36,   11,   25,   26,   27,
			   28,   29,   30,   29,   30,   22,   23,    7,   35,    7,
			   37,    7,   37,   75,   31,   32,   33,   26,   35,   34,
			   37,   12,   13,   14,   15,   16,   17,   18,   19,   20,
			   21,   22,   23,   24,   25,   26,   27,   28,   29,   30,

			   -1,   -1,   -1,   -1,   -1,   36,   12,   13,   14,   15,
			   16,   17,   18,   19,   20,   21,   22,   23,   24,   25,
			   26,   27,   28,   29,   30,   15,   16,   17,   18,   19,
			   20,   21,   22,   23,   24,   25,   26,   27,   28,   29,
			   30,   16,   17,   18,   19,   20,   21,   22,   23,   24,
			   25,   26,   27,   28,   29,   30,   22,   23,   24,   25,
			   26,   27,   28,   29,   30, yyDummy>>)
		end

feature {NONE} -- Semantic value stacks

	yyvs1: SPECIAL [ANY]
			-- Stack for semantic values of type ANY

	yyvsc1: INTEGER
			-- Capacity of semantic value stack `yyvs1'

	yyvsp1: INTEGER
			-- Top of semantic value stack `yyvs1'

	yyspecial_routines1: KL_SPECIAL_ROUTINES [ANY]
			-- Routines that ought to be in SPECIAL [ANY]

	yyvs2: SPECIAL [like last_list]
			-- Stack for semantic values of type like last_list

	yyvsc2: INTEGER
			-- Capacity of semantic value stack `yyvs2'

	yyvsp2: INTEGER
			-- Top of semantic value stack `yyvs2'

	yyspecial_routines2: KL_SPECIAL_ROUTINES [like last_list]
			-- Routines that ought to be in SPECIAL [like last_list]

	yyvs3: SPECIAL [like last_parsed]
			-- Stack for semantic values of type like last_parsed

	yyvsc3: INTEGER
			-- Capacity of semantic value stack `yyvs3'

	yyvsp3: INTEGER
			-- Top of semantic value stack `yyvs3'

	yyspecial_routines3: KL_SPECIAL_ROUTINES [like last_parsed]
			-- Routines that ought to be in SPECIAL [like last_parsed]

	yyvs4: SPECIAL [STRING]
			-- Stack for semantic values of type STRING

	yyvsc4: INTEGER
			-- Capacity of semantic value stack `yyvs4'

	yyvsp4: INTEGER
			-- Top of semantic value stack `yyvs4'

	yyspecial_routines4: KL_SPECIAL_ROUTINES [STRING]
			-- Routines that ought to be in SPECIAL [STRING]

feature {NONE} -- Constants

	yyFinal: INTEGER = 80
			-- Termination state id

	yyFlag: INTEGER = -32768
			-- Most negative INTEGER

	yyNtbase: INTEGER = 38
			-- Number of tokens

	yyLast: INTEGER = 164
			-- Upper bound of `yytable' and `yycheck'

	yyMax_token: INTEGER = 279
			-- Maximum token id
			-- (upper bound of `yytranslate'.)

	yyNsyms: INTEGER = 43
			-- Number of symbols
			-- (terminal and nonterminal)

feature -- User-defined features



feature -- Error handling

	report_error (a_message: STRING) is
			-- Report error 'a_message'.
		do
			error.signal (id, "$1 [near '$2', token '$3']", <<a_message, text, token_name (last_token)>>)
		end

feature {NONE} -- Implementation

	buffer_name: STRING
			-- Name of analyzed buffer

	singleton_list (a_expression: ROSE_EXPRESSION [G]): ROSE_LINEAR_ARRAY [ROSE_EXPRESSION [G]] is
			-- List of one expression
		require
			a_expression_not_void: a_expression /= Void
		do
			create Result.make_empty
			Result.extend (a_expression)
		ensure
			result_not_void: Result /= Void
			one_entry: Result.count = 1
			no_void_entry: not Result.has (Void)
		end

	append_to_list (a_list: ROSE_LINEAR_ARRAY [ROSE_EXPRESSION [G]];
						 a_expression: ROSE_EXPRESSION [G]): ROSE_LINEAR_ARRAY [ROSE_EXPRESSION [G]] is
			-- `A_list' extended by `a_expression'
		require
			a_list_not_void: a_list /= Void
			a_expression_not_void: a_expression /= Void
		do
			a_list.extend (a_expression)
			Result := a_list
		ensure
			result_is_a_list: Result = a_list
			one_more_in_list: a_list.count = old a_list.count + 1
		end

	add_expression (a_expression: ROSE_EXPRESSION [G]) is
			-- Add 'a_expressio' to 'last_list' of parsed expressions.
		do
			if last_list = Void then
				create last_list.make_empty
			end
			last_list.extend (a_expression)
			last_parsed := a_expression
		end

	get_unary (a_expression: ROSE_EXPRESSION [G]; a_operator: STRING): ROSE_UNARY_EXPRESSION [G] is
			-- Get unary expression corresponding to 'a_operator' and applied to 'a_expression'
		local
			l_operator: ROSE_UNARY_OPERATOR [G]
		do
			l_operator := a_expression.unary_operator (a_operator)
			if l_operator /= Void then
				create Result.make (a_expression,l_operator)
			else
				error.warn ("Could not find operator '$2' for expression '$1'",<<a_expression,a_operator>>)
			end
		rescue
			error.warn ("Could not find operator '$2' for expression '$1'",<<a_expression,a_operator>>)
		end

	get_binary (a_left, a_right: ROSE_EXPRESSION [G]; a_operator: STRING): ROSE_EXPRESSION [G] is
			-- Get binary expression corresponding to 'a_operator' and associating 'a_left' and 'a_right'
		local
			l_operator: ROSE_BINARY_OPERATOR [G]
		do
			l_operator := a_left.binary_operator (a_operator)
			if l_operator /= Void then
				Result := a_left.binary_expression (l_operator, a_right)
			end
			if Result = Void then
				error.warn ("Could not find operator '$3' for expressions '$1' and '$2'",<<a_left,a_right,a_operator>>)
			end
		rescue
			error.warn ("Could not find operator '$3' for expressions '$1' and '$2'",<<a_left,a_right,a_operator>>)
		end

	get_variable (a_id: STRING; a_attribute: STRING): ROSE_EXPRESSION [G] is
			-- Get variable identified by 'a_id', 'a_attribute' indicates which flavor or attribute of 'a_id' to consider...
		local
			l_id: STRING
		do
			if a_attribute = Void then
				l_id := a_id
			else
					--| FIXME: This should be taken care of properly,
					--| not by artificially qualifying variable names
				a_attribute.to_lower
				l_id := a_id + "." + a_attribute
			end
			if variables /= Void then
				Result := variables.variable (l_id)
			end
			if Result = Void then
				create {SIMPLE_EXPRESSION [G]} Result.make (l_id)
			end
		ensure
			result_not_void: Result /= Void
		rescue
			error.warn ("No variable '$1.$2'",<<a_id,a_attribute>>)
		end

	get_cross_supplier_variable (a_id: STRING; a_attribute: STRING): ROSE_EXPRESSION [G] is
			-- Get cross-supplier variable identified by 'a_id', 'a_attribute' indicates which flavor or attribute of 'a_id' to consider...
		local
			l_id: STRING
		do
			if a_attribute = Void then
				l_id := "./" + a_id
			else
				a_attribute.to_lower
				l_id := "./" + a_id + "." + a_attribute
			end
			if cross_supplier_parser = Void or else cross_supplier_parser.base_uri = Void then
				error.warn ("Cross-supplier variable '$1' cannot be resolved", <<l_id>>)
			else
				cross_supplier_parser.parse (l_id)
				if cross_supplier_parser.is_error then
					error.warn (cross_supplier_parser.last_error, Void)
				else
					Result ?= cross_supplier_parser.last_expression
				end
			end
			if Result = Void then
				create {SIMPLE_EXPRESSION [G]} Result.make (l_id)
			end
		ensure
			result_not_void: Result /= Void
		end

	get_function (a_name: STRING; a_args: ROSE_LINEAR_ARRAY [ROSE_EXPRESSION [G]]): ROSE_EXPRESSION [G] is
			-- Function `a_name'
		require
			a_name_not_void: a_name /= Void
			a_args_not_void: a_args /= Void
			at_least_one_argument: a_args.count > 0
		do
			if a_args.count = 1 then
				Result := get_unary (a_args.item (1), a_name)
			elseif a_args.count = 2 then
				Result := get_binary (a_args.item (1), a_args.item (2), a_name)
			else
				error.warn ("Function $1 has more than two arguments - not supported", <<a_name>>)
			end
			if Result = Void then
				create {SIMPLE_EXPRESSION [G]} Result.make (a_name)
			end
		ensure
			result_not_void: Result /= Void
		end

end -- class ROSE_EXPRESSION_PARSER
