indexing
	description: ""
	author: "Michael Zhang"
	date: "$Date: 2008/12/01 $"
	revision: "$Revision: #3 $"
	archive: "$File: //rose/source/kernel/math/rose_expression_snapshot.e $"

class
	ROSE_EXPRESSION_SNAPSHOT

inherit
	ROSE_LEGACY_UNDEFINED
	DEBUG_OUTPUT
		
create
	make
	
feature
	make(a_expression : ROSE_EXPRESSION[ANY]; a_target : like target) is
		do
			expression := a_expression
			target := a_target
			is_open := True
		end
	
	set_value(a_value : like value) is
		do
			value := a_value	
		end
		
	close is
		do
			is_open := False
		end
		
	debug_output : STRING is	
		do
			if expression /= Void and then expression.mnemonic /= Void then
				Result := expression.mnemonic.as_upper
			else
				Result := ""
			end
		end

feature 		
	expression : ROSE_EXPRESSION[ANY]
	target : ANY
	value : ANY
	is_open : BOOLEAN
	
feature	
	level_strings(a_level : INTEGER; a_target_function : FUNCTION[ANY, TUPLE[ANY], STRING]) : ROSE_LINEAR_ARRAY[STRING] is	
		local
			l_string : STRING
		do
			create Result.make_empty
			Result.extend(a_level.out)
			Result.extend(level_space(a_level) + expression_string)

			if a_target_function /= Void then
				l_string := a_target_function.item ([target])
				if l_string /= Void then
					Result.extend(level_space(a_level) + l_string)
				else
					Result.extend(level_space(a_level) + target_string)
				end
			else
				Result.extend(level_space(a_level) + target_string)	
			end

			Result.extend(value_string)
		end

feature
	expression_string : STRING is
		do
			if expression.name /= Void then
				Result := expression.generating_type + " [" + expression.name + "]"
			else
				Result := expression.generating_type
			end	
		end
	
	value_string : STRING is
		local
			l_integer_ref : INTEGER_REF
			l_double_ref : DOUBLE_REF
			l_boolean_ref : BOOLEAN_REF
			l_string : STRING		
		do
			if expression.is_double or expression.is_real then
				l_double_ref ?= value
				Result := formatted(l_double_ref)
			elseif expression.is_integer then
				l_integer_ref ?= value
				Result := l_integer_ref.out
			elseif expression.is_string then
				l_string ?= value
				if l_string /= Void then
					Result := l_string
				else
					Result := ""
				end
			elseif expression.is_boolean then
				l_boolean_ref ?= value
				Result := l_boolean_ref.out
			end			
		end
	
	target_string : STRING is	
		do
			if target /= Void then
				Result := target.generating_type
			else
				Result := ""
			end			
		end
		
feature		
	level_space(a_level : INTEGER) : STRING is
		local
			i : INTEGER
		do
			Result := ""
			from i := 1 until i > 5 * a_level loop
				Result := Result + " "
				i := i + 1
			end
		end

	formatted(a_value : DOUBLE) : STRING is
		do
			if is_defined_double(a_value) then
				Result := format.formatted(a_value)
			else
				Result := "             n/a"
			end
		end

	format : FORMAT_DOUBLE is
		once
			create Result.make(16,2)
			Result.comma_separate
		end			
			
end
