note

	description:

		"Default argument for ROSE_EXPRESSION value features"

	author: "Mark Howard"
	last_editor: "$Author: mk $"
	date: "$Date: 2011/05/03 $"
	revision: "$Revision: #37 $"
	path: "$File: //rose/source/kernel/targets/rose_expression_target.e $"

deferred class
	ROSE_EXPRESSION_TARGET

inherit
	FIELD_DESCRIPTORS

	CONTEXT_ANCHOR
	NEEDS_EXPRESSION_FROM_STRING
feature -- Access

	context: like context_anchor is
			-- Context
		deferred
		end

	expressions: like expressions_anchor is
			-- Expressions specific to this target type
		do
			if context.mappings_revision_number /= 0 and then -- Check if any mappings override has occurred
				mappings_revision_number_table.item (internal_type) /= context.mappings_revision_number then
				reinitialize_for_mappings_change
			end
			Result ?= expressions_table.item (internal_type)
			if Result = Void then
				create Result.make_from_descriptions (context.variable_descriptions (expression_tag),agent new_variable,context.mappings)
				expressions_table.force (Result,internal_type)
			end
		ensure
			result_not_void: Result /= Void
		end

	mappings_revision_number_table: HASH_TABLE[INTEGER,INTEGER] is
			--Table of mapping revision numbers
		once
			create Result.make (16)
		end

	reinitialize_for_mappings_change is
		do
			expressions_table.remove (internal_type)
			reports_by_name_table.remove (internal_type)
			parsers.remove (internal_type)
			mappings_revision_number_table.force (context.mappings_revision_number, internal_type)
		end

	expressions_table: HASH_TABLE[ENTITY_VARIABLES[ROSE_EXPRESSION_TARGET],INTEGER] is
			--hash table of expressions
		once
			create Result.make (100)
		end

	complex_expression (a_string: STRING): like expression_target is
			--computed or by name
		do
			Result := expressions.complex_expression (a_string)
		end

	expression_from_name(a_string : STRING) : like new_variable is
			--from the name or mnemonic ... guaranteed to typecheck
		do
			Result ?= expressions.expression_from_name (a_string)
		end

	double_value_from_expression_name (a_string: STRING): DOUBLE is
		do
			Result := expressions.complex_expression (a_string).generated_unitized_value (Current, void).value
		end

	expressions_from_names (a_variables: LINEAR[STRING]): ROSE_LINEAR_ARRAY[like expression_target] is
		do
			create Result.make_empty
			from a_variables.start until a_variables.off loop
				Result.extend (complex_expression (a_variables.item))
				a_variables.forth
			end
		end

	interpretation (a_report_type: STRING;a_string: STRING): like complex_expression is
		do
			Result := complex_expression (report_map (a_report_type,a_string))
		end

	interpreted_double_value (a_report_type: STRING;a_string: STRING): DOUBLE is
		do
			Result := complex_expression (report_map (a_report_type,a_string)).double_value (Current)
		end

	report_map (a_report_type: STRING;a_string: STRING): STRING is
		local
			l_report_type:STRING
			l_mnemonic:STRING
			l_key: TUPLE[STRING,STRING]
		do
			l_report_type := clone (a_report_type)
			l_report_type.to_lower
			l_report_type.right_adjust
			l_mnemonic := clone (a_string)
			l_mnemonic.to_lower
			l_mnemonic.right_adjust
			l_key := [l_report_type,l_mnemonic]
			mappings_table.search (l_key)
			if	mappings_table.found then
				Result := mappings_table.item (l_key).expression
			else
				Result := "notmapped"
			end
		end

	description: ARRAY[STRING] is
			-- Description (empty)
		do
			Result := <<>>
		end

	description_title: ARRAY[STRING] is
		do
			Result := <<>>
		end

	internal_type: INTEGER is
		do
			Result := internal.dynamic_type (Current)
		end

	date: ROSE_DATE is
		deferred
		end

	parser: like parser_template is
		do
			if context.mappings_revision_number /= 0 and then -- Check if any mappings override has occurred
				mappings_revision_number_table.item (internal_type) /= context.mappings_revision_number then
				reinitialize_for_mappings_change
			end
			Result ?= parsers.item (internal_type) --parsers is once so cannot have an anchored generic parameter, hence the assignment attempt
			if Result = Void then
				create Result.make (expressions)
				parsers.put (Result,internal_type)
			end
		end

	mappings_table:ROSE_MULTI_KEYED_TABLE[MAPPINGS_RECORD] is
		do
			Result := context.mappings_table
		end

	context_name: STRING is
		deferred
		end

feature {ROSE_EXPRESSION} -- Access

	component_expressions (a_expression_mnemonic: STRING): ROSE_LINEAR_ARRAY [ROSE_EXPRESSION [ROSE_EXPRESSION_TARGET]] is
			-- Named variables which contribute to value of `Current'
		require
			mnemonic_exists : a_expression_mnemonic /= Void
		do
			Result := expressions.component_expressions (a_expression_mnemonic)
		ensure
			result_not_void: Result /= Void
		end

feature --Reports

	report (a_name: STRING;a_type: STRING): like report_anchor is
		local
			l_name: STRING
			l_type: STRING
		do
			l_name := clone (a_name)
			l_name.to_lower
			l_type := clone (a_type)
			l_type.to_lower
			Result :=  reports_table.item ([l_name,l_type])
			if Result = void then
				Result := new_report (l_name,l_type)
				reports_table.put (Result,[l_name,l_type])
			end
		end

	new_report (a_name,a_type: STRING): like report_anchor is
		do
			create Result.make (expressions,a_name,a_type)
		end

	new_report_variable (a_variable_description: VARIABLE_DESCRIPTION_RECORD): ROSE_EXPRESSION[like anchor] is do end

	reports_table: ROSE_MULTI_KEYED_TABLE[like new_report] is
		do
			if context.mappings_revision_number /= 0 and then -- Check if any mappings override has occurred
				mappings_revision_number_table.item (internal_type) /= context.mappings_revision_number then
				reinitialize_for_mappings_change
			end
			Result ?= reports_by_name_table.item (internal_type)
			if Result = Void then
				create Result.make (10)
				reports_by_name_table.put (Result,internal_type)
			end
		end

	reports_by_name_table: HASH_TABLE [ROSE_MULTI_KEYED_TABLE [ROSE_REPORT [ROSE_EXPRESSION_TARGET]], INTEGER] is
		once
			create Result.make (20)
		end
feature -- place holders for real time

	iteration: INTEGER is do end

	is_realtime: BOOLEAN is do end

feature {ROSE_EXPRESSION} -- Implementation

	expression_tag: STRING is
		deferred
		end

feature {NONE} --Templates

	parser_template : ROSE_EXPRESSION_PARSER[like anchor] is
			--template
		do
		end

feature {NONE} -- Implementation

	parsers: HASH_TABLE[ROSE_EXPRESSION_PARSER[ROSE_EXPRESSION_TARGET],INTEGER] is
			-- Shared parsers
		once
			create Result.make (100)
		end

	new_variable (a_string: VARIABLE_DESCRIPTION): ROSE_EXPRESSION [like Current] is
		deferred
		end

	expressions_anchor: ENTITY_VARIABLES [like Current] is do end
	expression_target: ROSE_EXPRESSION [like Current] is do end
	complex_expression_target : ROSE_EXPRESSION[like Current] is
			--The sum of two expressions does not necessarily conform to the expressions
		do end
	anchor: like Current is
			-- Anchor
		do end
	report_anchor: ROSE_REPORT [like Current] is do end

end
