note

	description:
	"[
		Cross-supplier expression.
		Proxies real expression so as to ensureu
		correct target is used.
	]"
	author: "$Author: it $ Colin Adams"
	last_editor: "$Author: it $"
	date: "$Date: 2014/10/29 $"
	revision: "$Revision: #21 $"
	path: "$File: //rose/source/kernel/math/rose_foreign_expression.e $"

class	ROSE_FOREIGN_EXPRESSION
      -- There might be a potential problem here with the access control agents

inherit
	NEEDS_EXPRESSION_COMPUTATION
	USES_ANONYMOUS_EXPRESSION_VALUE
	ROSE_EXPRESSION [ROSE_TARGET [ENTITY]]
		rename
			make as make_expression,
			target_name as rose_expression_target_name
		redefine
			generated_currency, unitized_value, generated_unitized_value,
			expression_value, partial_expression_value, real_value,
			decimal_value, double_value, integer_value,
			component_expressions, numeric_value, partial_numeric_value,
			vector, generated_value_in_currency, unitized_value_in_market_currency,
			generated_value_in_market_currency, normalized_value_from_raw,
			normalized_value, value_in_market_currency, value_in_currency,
			scaled_value_in_market_currency, units, units_in_currency,
			date, display_partial_string, display_string,
			write_double,write_integer, write_real, commit,
			computed_double, computed_rt_double,
			computed_real, computed_integer, computed_string,
			computed_units, internal_generated_double, internal_generated_decimal,
			internal_generated_real, internal_generated_integer, internal_generated_date, internal_generated_string,
			generated_numeric_value, generated_vector,
			date_value,string_value,boolean_value,
			partial_real_value, partial_double_value, partial_integer_value,
			partial_decimal_value, partial_value_in_market_currency,
			instantiated, bypassing_overrides, set_use_overrides
		end

	EXCEPTIONS
		export
			{NONE} all
		undefine
			default_create
		end

create
	make

feature {NONE} -- Initialization

	make (a_expression: like foreign_expression; a_context: like context; a_name: like target_name;
	      a_selector: like target_selector) is
			-- Initialize `Current'.
		require
			a_expression_not_void: a_expression /= Void
			a_context_not_void: a_context /= Void
			a_name_not_void: a_name /= Void
			a_selector_not_void: a_selector /= Void
		do
			default_create
			foreign_expression := a_expression
			context := a_context
			target_name := a_name
			target_selector := a_selector
			name := foreign_expression.name
			type := foreign_expression.type
			mnemonic := foreign_expression.mnemonic
			variable_description := foreign_expression.variable_description
			variable_index := foreign_expression.variable_index
		ensure
			foreign_expression_set: foreign_expression = a_expression
			context_set: context = a_context
			target_name_set: target_name = a_name
			target_selector_set: target_selector = a_selector
		end

feature -- Access

	foreign_expression: ROSE_EXPRESSION [ROSE_TARGET [ENTITY]]
			-- Foreign expression being proxied

	context: SUPPLIER_CONTEXT
			-- Foreign context

	target_name: STRING
			-- Name of peer target

	target_selector: ROSE_TARGET_SELECTOR
			-- Helper for selecting signatures

	peer_target (a_target: like expression_target): ROSE_TARGET [ENTITY] is
			-- Peer for `a_target'
		require
			a_target_not_void: a_target /= Void
		local
			l_found: BOOLEAN
			l_entity, l_peer: ENTITY
			l_targets: ROSE_LINEAR_ARRAY [ROSE_TARGET [ENTITY]]
			l_date: ROSE_DATE
		do
			l_date := a_target.date
			l_entity := a_target.item
			check
				l_entity_not_void: l_entity /= Void
				-- Only an assumption
			end
			-- or should we first try `l_entity.as_of'?
			l_peer := l_entity.peer (context, l_date)
			l_targets := l_peer.ur_targets
			from l_targets.start until l_found or l_targets.after loop
				if l_targets.current_item.generating_type.is_case_insensitive_equal (target_name) then
					l_found := True
				else
					l_targets.forth
				end
			end
			if l_found then
				Result := target_selector.target_from_ur_target (l_targets.current_item, l_peer, l_date)
			end
		end

	mapped_targets (a_targets: ARRAY [ ROSE_TARGET [ENTITY]]): ARRAY [ROSE_TARGET [ENTITY]] is
			-- Array of peer targets
		require
			a_targets_not_void: a_targets /= Void
		local
			i: INTEGER
		do
			create Result.make (a_targets.lower, a_targets.upper)
			from
				i := 1
			variant
				a_targets.count + 1 - 1
			until
				i > a_targets.count
			loop
				if a_targets.item (i) /= Void then
					a_targets.put (peer_target (a_targets.item (i)), i)
				end
				i := i + 1
			end
		ensure
			mapped_targets_not_void: Result /= Void
			same_count: result.count = a_targets.count
		end

feature -- Basic Retrieval

	currency (a_target: like expression_target): PERSISTENT_CURRENCY is
			--currency of this expression for a_target
		local
			l_target: like peer_target
		do
			l_target := peer_target (a_target)
			if l_target /= Void then
				Result := foreign_expression.currency (l_target)
			end
		end

	generated_currency (a_target: like expression_target; a_agent: like access_agent_template): PERSISTENT_CURRENCY is
			-- Generated currency
		local
			l_target: like peer_target
		do
			l_target := peer_target (a_target)
			if l_target /= Void then
				Result := foreign_expression.generated_currency (l_target, a_agent)
			end
		end

	unitized_value (a_target: like expression_target; a_agent: like access_agent_template): detachable DATED_UNITIZED_VALUE [DOUBLE]
			--dated unitized value
		local
			l_target: like peer_target
		do
			l_target := peer_target (a_target)
			if l_target /= Void then
				if attached foreign_expression.unitized_value (l_target, Void) as l_value then
					create Result.make_from_rose_unitized (l_value)
				end
			end
		end

	generated_unitized_value (a_target: like expression_target; a_agent: like access_agent_template): detachable DATED_UNITIZED_VALUE [DOUBLE] is
		local
			l_target: like peer_target
		do
			l_target := peer_target (a_target)
			if l_target /= Void then
				Result := foreign_expression.generated_unitized_value (l_target, a_agent)
			end
		end

	expression_value (a_target: like expression_target): ANY is
			-- Untyped value of this expression at 'a_target'
		local
			l_target: like peer_target
		do
			l_target := peer_target (a_target)
			if l_target /= Void then
				Result := foreign_expression.expression_value (l_target)
			end
		end

	partial_expression_value (a_target: like expression_target): ANY is
			-- Untyped value of this expression at 'a_target'
		local
			l_target: like peer_target
		do
			l_target := peer_target (a_target)
			if l_target /= Void then
				Result := foreign_expression.partial_expression_value (l_target)
			end
		end

	real_value (a_target: like expression_target): REAL is
			-- <Precursor>
		local
			l_target: like peer_target
		do
			l_target := peer_target (a_target)
			if l_target /= Void then
				Result := foreign_expression.real_value (l_target)
			else
				Result := undefined_real
			end
		end

	decimal_value (a_target: like expression_target): RP_DECIMAL is
			-- Decimal value
		local
			l_target: like peer_target
		do
			l_target := peer_target (a_target)
			if l_target /= Void then
				Result := foreign_expression.decimal_value (l_target)
			end
		end

	double_value (a_target: like expression_target): DOUBLE is
			--Double value
		local
			l_target: like peer_target
		do
			l_target := peer_target (a_target)
			if l_target /= Void then
				Result := foreign_expression.double_value (l_target)
			else
				Result := undefined_double
			end
		end

	integer_value (a_target: attached like expression_target): INTEGER is
			--Integer value
		local
			l_target: like peer_target
		do
			l_target := peer_target (a_target)
			if l_target /= Void then
				Result := foreign_expression.integer_value (l_target)
			else
				Result := undefined_integer
			end
		end

	boolean_value (a_target: like expression_target): BOOLEAN is
			--Boolean value
		local
			l_target: like peer_target
		do
			l_target := peer_target (a_target)
			if l_target /= Void then
				Result := foreign_expression.boolean_value (l_target)
			end
		end


	string_value (a_target: like expression_target): STRING is
			-- String value
		local
			l_target: like peer_target
		do
			l_target := peer_target (a_target)
			if l_target /= Void and foreign_expression /= Void then
				Result := foreign_expression.string_value (l_target)
			end
		end

	date_value (a_target: like expression_target): ROSE_DATE is
			--Date value
		local
			l_target: like peer_target
		do
			l_target := peer_target (a_target)
			if l_target /= Void then
				Result := foreign_expression.date_value (l_target)
			end
		end




	component_expressions (a_target: attached ROSE_EXPRESSION_TARGET): ROSE_LINEAR_ARRAY [ROSE_EXPRESSION [ROSE_EXPRESSION_TARGET]] is
			-- Expressions which contribute to value of `Current'
		local
			l_target: ROSE_TARGET [ENTITY]
		do
			l_target ?= a_target
			if l_target = Void then
				create Result.make_empty
			else
				Result := foreign_expression.component_expressions (l_target)
			end
		end


	numeric_value (a_target: attached ROSE_TARGET [ENTITY]): DOUBLE is
		local
			l_target: like peer_target
		do
			l_target := peer_target (a_target)
			if l_target /= Void then
				Result := foreign_expression.numeric_value (l_target)
			else
				Result := undefined_double
			end
		end

	partial_numeric_value (a_target: attached ROSE_TARGET [ENTITY]): DOUBLE is
		local
			l_target: like peer_target
		do
			l_target := peer_target (a_target)
			if l_target /= Void then
				Result := foreign_expression.partial_numeric_value (l_target)
			else
				Result := undefined_double
			end
		end

	vector (a_targets: ARRAY [attached ROSE_TARGET [ENTITY]]): VECTOR is
		do
			if a_targets /= Void then
				Result := foreign_expression.vector (mapped_targets (a_targets))
			end
		end


	generated_value_in_market_currency (a_market: MARKET; a_currency: PERSISTENT_CURRENCY; a_target: like expression_target; a_date: ROSE_DATE; a_agent : like access_agent_template): DOUBLE is
			-- Generated value in specific currency
		local
			l_target: like peer_target
		do
			l_target := peer_target (a_target)
			if l_target /= Void then
				Result := foreign_expression.generated_value_in_market_currency (a_market, a_currency, l_target, a_date, a_agent)
			else
				Result := undefined_double
			end
		end

	unitized_value_in_market_currency (a_market: MARKET; a_currency: PERSISTENT_CURRENCY; a_target: like expression_target; a_date: ROSE_DATE; a_agent : like access_agent_template): ROSE_UNITIZED_VALUE [DOUBLE] is
		local
			l_target: like peer_target
		do
			l_target := peer_target (a_target)
			if l_target /= Void then
				Result := foreign_expression.unitized_value_in_market_currency (a_market, a_currency, l_target, a_date, a_agent)
			end
		end

	generated_value_in_currency (a_currency: PERSISTENT_CURRENCY; a_target: like expression_target; a_date: ROSE_DATE; a_agent: like access_agent_template): DOUBLE is
		local
			l_target: like peer_target
		do
			l_target := peer_target (a_target)
			if l_target /= Void then
				Result := foreign_expression.generated_value_in_currency (a_currency, l_target, a_date, a_agent)
			end
		end



	normalized_value (a_target: like expression_target): DOUBLE is
		local
			l_target: like peer_target
		do
			l_target := peer_target (a_target)
			if l_target /= Void then
				Result := foreign_expression.normalized_value (l_target)
			else
				Result := undefined_double
			end
		end

	normalized_value_from_raw (a_value : DOUBLE; a_target: like expression_target) : DOUBLE is
		local
			l_target: like peer_target
		do
			l_target := peer_target (a_target)
			if l_target /= Void then
				Result := foreign_expression.normalized_value_from_raw (a_value, l_target)
			else
				Result := undefined_double
			end
		end

	value_in_market_currency (a_market: MARKET; a_currency: PERSISTENT_CURRENCY; a_target: like expression_target; a_date: ROSE_DATE): DOUBLE is
		local
			l_target: like peer_target
		do
			l_target := peer_target (a_target)
			if l_target /= Void then
				Result := foreign_expression.value_in_market_currency (a_market, a_currency, l_target, a_date)
			else
				Result := undefined_double
			end
		end

	value_in_currency (a_currency: PERSISTENT_CURRENCY; a_target: like expression_target; a_date: ROSE_DATE): DOUBLE is
		local
			l_target: like peer_target
		do
			l_target := peer_target (a_target)
			if l_target /= Void then
				Result := foreign_expression.value_in_currency (a_currency, l_target, a_date)
			else
				Result := undefined_double
			end
		end

	scaled_value_in_market_currency (a_market: MARKET; a_currency: PERSISTENT_CURRENCY; a_target: like expression_target; a_date: ROSE_DATE): DOUBLE is
		local
			l_target: like peer_target
		do
			l_target := peer_target (a_target)
			if l_target /= Void then
				Result := foreign_expression.scaled_value_in_market_currency (a_market, a_currency, l_target, a_date)
			else
				Result := undefined_double
			end
		end

	units (a_target: like expression_target): ROSE_UNITS is
		local
			l_target: like peer_target
		do
			l_target := peer_target (a_target)
			if l_target /= Void then
				Result := foreign_expression.units (l_target)
			else
				Result := default_units
			end
		end

	units_in_currency (a_currency: PERSISTENT_CURRENCY; a_target: like expression_target): ROSE_UNITS is
		local
			l_target: like peer_target
		do
			l_target := peer_target (a_target)
			if l_target /= Void then
				Result := foreign_expression.units_in_currency (a_currency, l_target)
			else
				Result := default_units
			end
		end

	date (a_target: like expression_target): ROSE_DATE is
		local
			l_target: like peer_target
		do
			l_target := peer_target (a_target)
			if l_target /= Void then
				Result := foreign_expression.date (l_target)
			end
		end

	display_string (a_target: like expression_target): STRING is
		local
			l_target: like peer_target
		do
			l_target := peer_target (a_target)
			if l_target /= Void then
				Result := foreign_expression.display_string (l_target)
			end
		end

	display_partial_string (a_target: like expression_target): STRING is
		local
			l_target: like peer_target
		do
			l_target := peer_target (a_target)
			if l_target /= Void then
				Result := foreign_expression.display_partial_string (l_target)
			end
		end

	undefined_real: REAL is
		do
			Result := foreign_expression.undefined_real
		end

	undefined_double: DOUBLE is
		do
			Result := foreign_expression.undefined_double
		end

	undefined_integer: INTEGER is
		do
			Result := foreign_expression.undefined_integer
		end

feature -- Write

	write_double (a_value: DOUBLE; a_target : like expression_target) is
			-- Store `a_value' for `a_target'
		local
			l_target: like peer_target
		do
			l_target := peer_target (a_target)
			if l_target /= Void then
				foreign_expression.write_double (a_value, l_target)
			end
		end

	write_integer (a_value: INTEGER; a_target: like expression_target) is
			-- Store `a_value' for `a_target'
		local
			l_target: like peer_target
		do
			l_target := peer_target (a_target)
			if l_target /= Void then
				foreign_expression.write_integer (a_value, l_target)
			end
		end

	write_real (a_value: REAL; a_target: like expression_target) is
			-- Store `a_value' for `a_target'
		local
			l_target: like peer_target
		do
			l_target := peer_target (a_target)
			if l_target /= Void then
				foreign_expression.write_real (a_value, l_target)
			end
		end

	commit is
			-- Commit data to database
		do
			foreign_expression.commit
		end

feature -- Compute

	computed_double (a_target: like expression_target; a_agent: like access_agent_template): DOUBLE is
			-- Computed double
		local
			l_target: like peer_target
		do
			l_target := peer_target (a_target)
			if l_target /= Void then
				Result := foreign_expression.computed_double (l_target, a_agent)
			else
				Result := undefined_double
			end
		end

	computed_rt_double (a_target: like expression_target; a_agent: like access_agent_template): DOUBLE is
		local
			l_target: like peer_target
		do
			l_target := peer_target (a_target)
			if l_target /= Void then
				Result := foreign_expression.computed_rt_double (l_target, a_agent)
			else
				Result := undefined_double
			end
		end

	computed_real (a_target: like expression_target; a_agent: like access_agent_template): REAL is
			-- Computed real
		local
			l_target: like peer_target
		do
			l_target := peer_target (a_target)
			if l_target /= Void then
				Result := foreign_expression.computed_real (l_target, a_agent)
			else
				Result := undefined_real
			end
		end

	computed_integer (a_target: like expression_target; a_agent: like access_agent_template): INTEGER is
		local
			l_target: like peer_target
		do
			l_target := peer_target (a_target)
			if l_target /= Void then
				Result := foreign_expression.computed_integer (l_target, a_agent)
			else
				Result := undefined_integer
			end
		end

	computed_string (a_target: like expression_target; a_agent: like access_agent_template): STRING is
			--Get the foreign expression's computed string
		local
			l_target: like peer_target
		do
			l_target := peer_target (a_target)
			Result := foreign_expression.computed_string (l_target, a_agent)
		end



	computed_units (a_target: like expression_target; a_agent: like access_agent_template): ROSE_UNITS is
		local
			l_target: like peer_target
		do
			l_target := peer_target (a_target)
			if l_target /= Void then
				Result := foreign_expression.computed_units (l_target, a_agent)
			else
				Result := default_units
			end
		end

feature {NONE} -- Generate

	internal_generated_double (a_target: like expression_target; a_agent: like access_agent_template): DOUBLE is
			-- <Precursor>
		local
			l_target: like peer_target
		do
			l_target := peer_target (a_target)
			if l_target /= Void then
				Result := foreign_expression.generated_double (l_target, a_agent)
			else
				Result := undefined_double
			end
		end

	internal_generated_decimal (a_target: like expression_target; a_agent: like access_agent_template): RP_DECIMAL is
			-- <Precursor>
		local
			l_target: like peer_target
		do
			l_target := peer_target (a_target)
			if l_target /= Void then
				Result := foreign_expression.generated_decimal (l_target, a_agent)
			end
		end

	internal_generated_real (a_target: like expression_target; a_agent: like access_agent_template): REAL is
			-- <Precursor>
		local
			l_target: like peer_target
		do
			l_target := peer_target (a_target)
			if l_target /= Void then
				Result := foreign_expression.generated_real (l_target, a_agent)
			else
				Result := undefined_real
			end
		end

	internal_generated_integer (a_target: like expression_target; a_agent: like access_agent_template): INTEGER is
			-- <Precursor>
		local
			l_target: like peer_target
		do
			l_target := peer_target (a_target)
			if l_target /= Void then
				Result := foreign_expression.generated_integer (l_target, a_agent)
			else
				Result := undefined_integer
			end
		end

	internal_generated_string (a_target: like expression_target; a_agent: like access_agent_template): STRING is
			-- <Precursor>
		local
			l_target: like peer_target
		do
			l_target := peer_target (a_target)
			if l_target /= Void then
				Result := foreign_expression.generated_string (l_target, a_agent)
			end
		end

	internal_generated_date (a_target: like expression_target; a_agent: like access_agent_template): ROSE_DATE is
			-- <Precursor>
		local
			l_target: like peer_target
		do
			l_target := peer_target (a_target)
			if l_target /= Void then
				Result := foreign_expression.generated_date (l_target, a_agent)
			end
		end

feature -- Generate

	generated_numeric_value (a_target: attached ROSE_TARGET [ENTITY]; a_agent: like access_agent_template): DOUBLE is
		local
			l_target: like peer_target
		do
			l_target := peer_target (a_target)
			if l_target /= Void then
				Result := foreign_expression.generated_numeric_value (l_target, a_agent)
			else
				Result := undefined_double
			end
		end

	generated_vector (a_targets: ARRAY[attached ROSE_TARGET [ENTITY]]; a_agent: like access_agent_template): VECTOR is
		do
			Result := foreign_expression.generated_vector (mapped_targets (a_targets), a_agent)
		end

feature -- Partial values treat undefined as 0 in addition and subtraction

	partial_real_value (a_target: attached ROSE_TARGET [ENTITY]): REAL is
		local
			l_target: like peer_target
		do
			l_target := peer_target (a_target)
			if l_target /= Void then
				Result := foreign_expression.partial_real_value (l_target)
			else
				Result := undefined_real
			end
		end

	partial_double_value (a_target: attached ROSE_TARGET [ENTITY]): DOUBLE is
		local
			l_target: like peer_target
		do
			l_target := peer_target (a_target)
			if l_target /= Void then
				Result := foreign_expression.partial_double_value (l_target)
			else
				Result :=  undefined_double
			end
		end

	partial_decimal_value (a_target: attached ROSE_TARGET [ENTITY]): RP_DECIMAL is
		local
			l_target: like peer_target
		do
			l_target := peer_target (a_target)
			if l_target /= Void then
				Result := foreign_expression.partial_decimal_value (l_target)
			end
		end

	partial_integer_value (a_target: attached ROSE_TARGET [ENTITY]): INTEGER is
		local
			l_target: like peer_target
		do
			l_target := peer_target (a_target)
			if l_target /= Void then
				Result := foreign_expression.partial_integer_value (l_target)
			else
				Result := undefined_integer
			end
		end

	partial_value_in_market_currency (a_market: MARKET; a_currency: PERSISTENT_CURRENCY; a_target: like expression_target; a_date: ROSE_DATE): DOUBLE is
		local
			l_target: like peer_target
		do
			l_target := peer_target (a_target)
			if l_target /= Void then
				Result := foreign_expression.partial_value_in_market_currency (a_market, a_currency, l_target, a_date)
			else
				Result := undefined_double
			end
		end

	instantiated (a_data :ANY): like Current is
		do
			create Result.make (foreign_expression.instantiated (a_data), context, target_name, target_selector)
		end

	get_real_value (a_target: attached ROSE_TARGET [ENTITY]): REAL is
		local
			l_target: like peer_target
		do
			l_target := peer_target (a_target)
			if l_target /= Void then
				Result := foreign_expression.get_real_value (l_target)
			else
				Result := undefined_real
			end
		end

	get_double_value (a_target: attached ROSE_TARGET [ENTITY]): DOUBLE is
		local
			l_target: like peer_target
		do
			l_target := peer_target (a_target)
			if l_target /= Void then
				Result := foreign_expression.get_double_value (l_target)
			else
				Result := undefined_double
			end
		end

	get_integer_value (a_target: attached ROSE_TARGET [ENTITY]): INTEGER is
		local
			l_target: like peer_target
		do
			l_target := peer_target (a_target)
			if l_target /= Void then
				Result := foreign_expression.get_integer_value (l_target)
			else
				Result := undefined_integer
			end
		end

feature -- Override families

	bypassing_overrides: BOOLEAN is
			-- Does `Current' bypass any override database?
		do
			Result := foreign_expression.bypassing_overrides
		end

	set_use_overrides (a_value: BOOLEAN) is
		do
			foreign_expression.set_use_overrides (a_value)
		end

feature -- Undefined Values

	is_defined_integer (a_value: INTEGER): BOOLEAN is
		do
			Result := foreign_expression.is_defined_integer (a_value)
		end

	default_units: ROSE_UNITS is
			-- Void object pattern
		local
			l_rational: RATIONAL
			l_factored: FACTORED_RATIONAL
		do
			create l_factored
			create l_rational.make (0, 0)
			create Result.make_from_rational (l_factored, l_rational,False)
		ensure
			result_not_void: Result /= void
		end

invariant

	foreign_expression_not_void: foreign_expression /= Void
	context_not_void: context /= Void
	target_name_not_void: target_name /= Void
	target_selector_not_void: target_selector /= Void

end




