note

	description:
	"[
		Handles the mapping between a format and the attributes of an object.
		With named tuples we can simplify the interface significantly.
	]"
	author: "Mark Howard"
	last_editor: "$Author: er $"
	date: "$Date: 2014/02/26 $"
	revision: "$Revision: #12 $"
	path: "$File: //rose/source/kernel/infrastructure/text/rose_format.e $"

deferred class
	ROSE_FORMAT

inherit
	ABSTRACT_FORMAT
	IEEE_MATH_BASE

feature -- Access

	substring (a_string: STRING; i: INTEGER): STRING is
			-- String contents of 'i'-th column
		deferred
		end

feature -- Basic operations

	fill_vessel (a_vessel: ANY; a_string: STRING) is
			-- Fill 'a_vessel' from contents found in 'a_string'.
		local
			l_type, i, n: INTEGER
			l_real: REAL
			l_integer: INTEGER
			l_double: DOUBLE
			l_string: STRING
			l_date: ROSE_DATE
			l_info: TEXT_COLUMN_INFO
		do
			l_type := internal.dynamic_type (a_vessel)
			if l_type /= type then
				map_fields (a_vessel)
				type := l_type
			end
			from
				i := 1
				n := internal.field_count (a_vessel)
			until
				i > n
			loop
				if inverse_map.valid_index (i) and then inverse_map.item (i) > 0 then
					l_info := column_info.item (inverse_map.item (i))
					if l_info /= Void then
						l_string := substring (a_string, inverse_map.item (i))
						inspect l_info.type
						when {INTERNAL}.Real_type then
							if l_string.is_real then
								l_real := l_string.to_real
								internal.set_real_field (i, a_vessel, l_real)
							end
						when {INTERNAL}.Integer_type then
							if l_string.is_integer then
								l_integer := l_string.to_integer
								internal.set_integer_field (i, a_vessel, l_integer)
							end
						when {INTERNAL}.Double_type then
							if l_string /= Void then
								if l_string.count > 0 then
									l_double := global.formatter.parsed_double (l_string)
									internal.set_double_field (i, a_vessel, l_double)
								else
									internal.set_double_field (i, a_vessel, empty_string_interpretation_as_double)
								end
							end
						when String_type then
							internal.set_reference_field (i, a_vessel, l_string)
						when {INTERNAL}.Character_type then
							if l_string.count > 0 then
								internal.set_character_field (i, a_vessel, l_string.item (1))
							end
						when Date_type then
							if l_string.is_integer then
								l_integer := l_string.to_integer
								if l_integer > 999999 then
									create l_date.make_ccyymmdd (l_integer)
								else
									create l_date.make_yymmdd (l_integer)
								end
								internal.set_reference_field (i, a_vessel, l_date)
							else
								l_date := global.formatter.make_date_from_string (l_string)
								if l_date /= Void then
									internal.set_reference_field (i, a_vessel, l_date)
								end
							end
						else
							-- Do nothing
						end
					end
				end
				i := i + 1
			end
		end
	
feature -- Element modification
	
	set_empty_string_interpretation_as_double (a_value: DOUBLE) is
			-- Set the default interpretation of an empty string representing a double value
		do
			empty_string_interpretation_as_double := a_value
		ensure
			value_set: ieee_is_eq (empty_string_interpretation_as_double, a_value)
		end
	
feature {NONE} -- Implementation
	
	empty_string_interpretation_as_double: DOUBLE
	
end
