note

	description:

		"Operator to implement user defined functions"

	author: "Shujing Li"
	last_editor: "$Author: it $"
	date: "$Date: 2013/11/18 $"
	revision: "$Revision: #17 $"
	path: "$File: //rose/source/kernel/math/rose_function_operator.e $"

class
	ROSE_FUNCTION_OPERATOR [reference G]
inherit
	ROSE_UNARY_OPERATOR[G]
		redefine
			generated_unitized_value
		end
	ROSE_CONSTANTS
create
	make_with_function

feature	-- Initialization
	make_with_function (a_function: like function; a_name: STRING) is
			--!initialize given the function/agent and a name
		do
			make (constants.Double_type, 4, a_name)
			function := a_function
		end

feature	-- Access

	function: FUNCTION [ANY,TUPLE[DOUBLE,G,ROSE_EXPRESSION[G]], DOUBLE]
			-- Operator function

	real_value (a_left : ROSE_EXPRESSION[G];a_g : G) : REAL is
			-- REAL value
		do
			Result := double_value(a_left, a_g).truncated_to_real
		end
	double_value (a_left : ROSE_EXPRESSION[G];a_g : G) : DOUBLE is
			-- DOUBLE value
		local
			l_value : DOUBLE
		do
			l_value := a_left.double_value(a_g)
			if a_left.is_defined_double(l_value) then
				Result := function.item ([l_value,a_g,a_left])
			else
				Result := missing_double
			end
		end
	integer_value (a_left : ROSE_EXPRESSION[G];a_g : G) : INTEGER is
			-- INTEGER value
		do
			Result := double_value(a_left, a_g).rounded
		end
	boolean_value (a_left : ROSE_EXPRESSION[G];a_g : G) : BOOLEAN is --!
			-- Boolean value
		do
			Result := True
		end

	value_in_market_currency (a_left : ROSE_EXPRESSION[G];a_market : MARKET;a_currency : PERSISTENT_CURRENCY;a_g : G;a_date : ROSE_DATE) : DOUBLE is --!
			--double value expressed in a_currency
		local
			l_value : DOUBLE
		do
			l_value := a_left.value_in_market_currency(a_market,a_currency,a_g,a_date)
			if defined_double(l_value) and then l_value > 0 then
				Result := function.item ([l_value,a_g,a_left])
			else
				Result := missing_double
			end
		end

	computed_double (a_left: ROSE_EXPRESSION [G]; a_g: G; a_agent: like access_agent_template): DOUBLE is --!
			--computed double value
		local
			l_value : DOUBLE
		do
			l_value := a_left.computed_double(a_g,a_agent)
			if a_left.is_defined_double(l_value) then
				Result := function.item ([l_value,a_g,a_left])
			else
				Result := missing_double
			end
		end
	computed_real (a_left: ROSE_EXPRESSION [G]; a_g: G; a_agent: like access_agent_template): REAL is
			--computed real
		local
			l_value : DOUBLE
		do
			l_value := a_left.computed_real(a_g,a_agent)
			if a_left.is_defined_double(l_value) then
				Result := function.item ([l_value,a_g,a_left]).truncated_to_real
			else
				Result := missing_real
			end
		end
	computed_integer(a_left : ROSE_EXPRESSION[G]; a_g : G; a_agent : like access_agent_template) : INTEGER is --!
			--computed integer value
		local
			l_value : DOUBLE
		do
			l_value := a_left.computed_integer(a_g,a_agent)
			if a_left.is_defined_double(l_value) then
				Result := function.item ([l_value,a_g,a_left]).rounded
			else
				Result := missing_integer
			end
		end
	computed_boolean(a_left : ROSE_EXPRESSION[G]; a_g : G; a_agent : like access_agent_template) : BOOLEAN is --!
		do
			Result := True
		end
	computed_string(a_left : ROSE_EXPRESSION[G]; a_g : G; a_agent : like access_agent_template) : STRING is --!
		do
		end
	computed_value(a_left : ROSE_EXPRESSION[G]; a_g : G; a_agent : like access_agent_template) : ANY  is
			-- Computed value
		do
		end
	generated_double (a_left: ROSE_EXPRESSION [G]; a_g: G; a_agent: like access_agent_template): DOUBLE is  --!
		local
			l_value : DOUBLE
		do
			l_value := a_left.generated_double(a_g,a_agent)
			if a_left.is_defined_double(l_value) then
				Result := function.item ([l_value,a_g,a_left])
			else
				Result := missing_double
			end
		end
	generated_real (a_left: ROSE_EXPRESSION [G]; a_g: G; a_agent: like access_agent_template): REAL is --!
		local
			l_value : DOUBLE
		do
			l_value := a_left.generated_real(a_g,a_agent)
			if a_left.is_defined_double(l_value) then
				Result := function.item ([l_value,a_g,a_left]).truncated_to_real
			else
				Result := missing_real
			end
		end
	generated_integer(a_left : ROSE_EXPRESSION[G]; a_g : G; a_agent : like access_agent_template) : INTEGER is  --!
		local
			l_value : DOUBLE
		do
			l_value := a_left.generated_integer(a_g,a_agent)
			if a_left.is_defined_double(l_value) then
				Result := function.item ([l_value,a_g,a_left]).rounded
			else
				Result := missing_integer
			end
		end
	generated_boolean(a_left : ROSE_EXPRESSION[G]; a_g : G; a_agent : like access_agent_template) : BOOLEAN is
			--always true
		do
			Result := True
		end
	generated_string(a_left : ROSE_EXPRESSION[G]; a_g : G; a_agent : like access_agent_template) : STRING is
			--generated string
		do
		end
	generated_date(a_left : ROSE_EXPRESSION[G]; a_g : G; a_agent : like access_agent_template) : ROSE_DATE is
			-- Generated date
		do
		end

	generated_unitized_value (a_left: ROSE_EXPRESSION [G]; a_g: G; a_agent: like access_agent_template): DATED_UNITIZED_VALUE[DOUBLE] is  --!
		local
			l_double : DOUBLE
		do
			Result := a_left.generated_unitized_value(a_g,a_agent)
			if a_left.is_defined_double(Result.value) then
				l_double := function.item ([Result.value,a_g,a_left])
				Create Result.make(l_double,Result.units,Result.start_date,Result.end_date)
			end
		end

	computed_units (a_left: ROSE_UNITS): ROSE_UNITS is
			-- Computed units
		do
			Result := a_left
		end
	units_in_currency (a_left : ROSE_EXPRESSION[G];a_currency : PERSISTENT_CURRENCY;a_entity : G) : ROSE_UNITS is
		do
			Result := a_left.units_in_currency(a_currency,a_entity)
		end

feature	-- Not implemented
	character_value (a_left : ROSE_EXPRESSION[G];a_g : G) : CHARACTER is
		-- Character value
		do
		end

	string_value (a_left: ROSE_EXPRESSION[G];a_g : G) : STRING is do end
	date_value (a_left : ROSE_EXPRESSION[G];a_g : G) : ROSE_DATE is	do end
	date_time_value (a_left : ROSE_EXPRESSION[G];a_g : G) : ROSE_DATE is do end

end
