indexing
	description: "Global applicationwide data"
	warning: "Do not create instances of this type! Inherit from ROSE_SHARED_GLOBAL instead!"
	author: "Zoran Simic"
	last_editor: "$Author: c4 $"
	date: "$Date: 2014/05/13 $"
	revision: "$Revision: #87 $"
	path: "$File: //rose/source/kernel/infrastructure/framework/application/rose_global.e $"

class
	ROSE_GLOBAL

inherit
	
	ROSE_SINGLETON

	ROSE_PROGRESS
	
	ROSE_OBJECT_REPRESENTATION

	REFACTORING_HELPER

create
	make

feature {NONE} -- Initialization

	make
			-- Create a default global object containing general info on the current frameworked application.
		do
			check_instance_unicity
			date_format := "yyyy-[0]mm-[0]dd"
			date_time_format := "yyyy-[0]mm-[0]dd [0]hh:[0]mi"
				--| Error handling/recovery
			create last_resort_channels.make
			create termination_agents.make
			add_last_resort_channel ("file:>>error.log")
		end

feature -- Access

	termination_code: INTEGER
			-- Return code with which the application will be exiting
			-- Set this with care

	app: attached ROSE_APPLICATION
			-- Information on currently running program
		indexing
			once_status: global
		once
			create Result
			Result.make_current
			io_handler.error.flush_and_acknowledge
		ensure
			result_ok: Result /= Void
		end
	hardcoded_path (a_path: STRING): attached STRING
			-- This feature can be used to 'legitimate' hardcoded paths
			-- Hardcoded paths are highly discouraged, and will be red flagged by code analysis tools
			-- If you absolutely need to have a hardcoded path in your application,
			-- this feature gives you an opportunity to have a hardcoded flag that is not red flagged
			-- Simply pass your hardcoded path to this feature and use its return value
		require
			path_ok: a_path /= Void
		do
			if attached a_path as l_path then
				Result := l_path
			else
				create Result.make_empty
				report_unreachable_code_reached
			end
		end

	deliveries: attached ROSE_DELIVERIES_CONVENTION
			-- Deliveries-related conventions
			-- This class defines convention used for all supported deliveries, please always use this class when refering to files in a delivery
		once
			Result := convention.deliveries
		end

	dependency_manager: attached ROSE_SYSTEM_DEPENDENCY_MANAGER
			-- Dependency manager
		indexing
			once_status: global
		once
			create Result.make
		end

	convention: attached ROSE_FRAMEWORK_CONVENTION
			-- Framework conventions
		indexing
			once_status: global
		 once
		 	create Result
		 end

	config_manager: attached ROSE_CONFIGURATION_MANAGER
			-- Configuration manager (used to load/save configuration objects)
		indexing
			once_status: global
		once
			create Result.make
		end

	machine: attached ROSE_MACHINE_CONFIGURATION
			-- Machine configuration
		once
			create Result.make_current
		end

	prefs: attached ROSE_FRAMEWORK_PREFERENCE
			-- Application's framework related preferences, customizable via configuration
			-- Represented by an object for access optimization
		indexing
			once_status: global
		once
			create Result.make
		end

	user: attached ROSE_USER_PREFERENCES
			-- Holds accross-network preferences for this user
		indexing
			once_status: global
		local
			l_path: STRING
		once
			l_path := app.configured_value ("/path/user_preferences")
			if not is_directory_readable (l_path) then
				l_path := env.user_application_data_folder
			end
			l_path := formatter.composed_path (<<l_path, app.user.login_domain + "." + app.user.login_id + "." + app.program_name + ".cfg">>)
			create Result.make (app.user, l_path)
			add_cleanup_agent (agent Result.save_configuration)
		end

	login: attached ROSE_USER_PREFERENCES
			-- Holds machine-relative preferences for current user
		indexing
			once_status: global
		local
			l_path: STRING
		once
			l_path := env.user_application_data_folder
			if l_path /= Void and then not l_path.is_empty then
				l_path := formatter.composed_path (<<l_path, app.program_name + ".cfg">>)
			else
				l_path := Void
			end
			create Result.make (app.user, l_path)
			add_cleanup_agent (agent Result.save_configuration)
		end

	persist: attached ROSE_STORABLE_FACTORY
			-- Store/Retrieve Eiffel objects using STORABLE and/or XML serialization, and with error checking
		indexing
			once_status: global
		once
			create Result
		end

	commands: attached ROSE_COMMAND_MANAGER
			-- Set of commands available in this application
		indexing
			once_status: global
		once
			create Result.make ("commands")
		end

	io_handler: attached ROSE_IO_HANDLER
			-- Application IO handler (defines application conventional IO)
		indexing
			once_status: global
		once
			create Result.make
		end

	env: attached ROSE_EXECUTION_ENVIRONMENT
			-- Execution environment
		indexing
			once_status: global
		once
			Result := app.env
		end

	reg: attached ROSE_REGISTRY
			-- Access to the registry
		indexing
			once_status: global
		once
			create Result
		end

	internal: attached INTERNAL
			-- Internal
		indexing
			once_status: global
		once
			create Result
		end

	memory: attached MEMORY
			-- Memory
		indexing
			once_status: global
		once
			create Result
		end

	log_id: INTEGER
			-- Application id (identifying a run of current app), used for logging (optional)
			-- Use 'set_log_id' to specify it

	date_format: STRING
			-- Date format to use in 'represented_*_date'

	date_time_format: STRING
			-- Date-time format to use in 'represented_*_date'

	represented_date (a_date: ROSE_DATE): attached STRING
			-- Textual representation of 'a_date', which is expected to be a local date
			-- If time part is not present in 'a_date', then it is represented as-is
		do
			if a_date = Void or else a_date.year >= 9999 then
				Result := ""
			elseif a_date.persecs = 0 then
				Result := a_date.formatted_out (date_format)
			else
				Result := a_date.formatted_out (date_time_format)
			end
		end

	represented_utc_date (a_date: ROSE_DATE): attached STRING
			-- Textual representation of 'a_date', which is expected to be a UTC date
			-- If time part is not present in 'a_date', then it is represented as-is
		local
			l_date: ROSE_DATE
		do
			if a_date = Void or else a_date.year >= 9999 then
				Result := ""
			elseif a_date.persecs = 0 then
				Result := a_date.formatted_out (date_format)
			else
				l_date := a_date.twin
				l_date.second_add (env.utc_seconds_offset)
				Result := l_date.formatted_out (date_time_format)
			end
		end

	interpreted (a_string: STRING): STRING
			-- 'a_string' interpreted with application variables
		do
			Result := app.interpreted (a_string)
		end

	environment_interpreted (a_string: STRING): STRING
			-- Interpreted value of 'a_string' using environment variables only
		do
			Result := formatter.recursively_interpreted (a_string, 10, agent env.get)
		end

	formatted (a_message: STRING; a_args: ARRAY [ANY]): STRING
			-- Returns 'a_message' formatted with 'a_args'
		do
			Result := formatter.argumented (a_message, a_args)
		ensure
			result_ok: a_message /= Void implies Result /= Void
		end


	attached_string (a_string: STRING): attached STRING
			-- Return an attached version of `a_string'.
			-- If `a_string' is Void, an empy string is returned
			-- If `a_string' is not Void, `a_string' is returned.
		do
			if attached a_string as l_attached_string then
				Result := l_attached_string
			else
				create Result.make_empty
			end
		ensure
			result_is_a_string_when_not_void: a_string /= Void implies Result = a_string
			result_is_empty_string_when_void: a_string = Void implies Result.is_empty
		end

	attached_string_with_default (a_string: STRING; a_default: attached STRING): attached STRING
			-- Return an attached version of `a_string' with a default value for the Void case.
			-- If `a_string' is Void, Result is `a_default'
			-- If `a_string' is not Void, `a_string' is returned.
		do
			if attached a_string as l_attached_string then
				Result := l_attached_string
			else
				Result := a_default
			end
		ensure
			result_is_a_string_when_not_void: a_string /= Void implies Result = a_string
			result_is_a_default_when_void: a_string = Void implies Result = a_default
		end

	is_file_exists (a_path: STRING): BOOLEAN
			-- Does file with 'a_path' exist?
		local
			l_file: PLAIN_TEXT_FILE
			rescued: BOOLEAN
		do
			if not rescued and then a_path /= Void and then not a_path.is_empty then
				create l_file.make (a_path)
				Result := l_file.exists and then l_file.is_readable
			end
		rescue
			if not rescued then
				rescued := True
				retry
			end
		end

	is_file_readable (a_path: STRING): BOOLEAN
			-- Is file with 'a_path' readable?
		do
			Result := convention.is_file_readable (a_path)
		end

	is_file_writable (a_path: STRING): BOOLEAN
			-- Is file with 'a_path' writable?
		do
			Result := convention.is_file_writable (a_path)
		end

	is_file_writable_or_creatable (a_path: STRING): BOOLEAN
			-- Is file with 'a_path' writable (either exists and writable, or doesn't exist but creatable)?
		do
			Result := convention.is_file_writable_or_creatable (a_path)
		end

	is_directory_readable (a_path: STRING): BOOLEAN
			-- Is 'a_path' a valid directory (exists and is readable)?
		do
			Result := convention.is_directory_readable (a_path)
		end

	is_directory_writable (a_path: STRING): BOOLEAN
			-- Is 'a_path' pointing to a directory that exists and is writable?
		do
			Result := convention.is_directory_writable (a_path)
		end

	is_directory_writable_or_creatable (a_path: STRING): BOOLEAN
			-- Is 'a_path' pointing to a directory that is writable (either exists and writable, or doesn't exist but creatable)?
		do
			Result := convention.is_directory_writable_or_creatable (a_path)
		end

	is_valid_directory (a_path: STRING): BOOLEAN
			-- Is 'a_path' a valid directory (does it exist and is it readable)?
		obsolete
			"Use 'is_directory_readable' instead, which has a clearer name as to what this function does"
		do
			Result := is_directory_readable (a_path)
		end

	is_directory_exists (a_path: STRING): BOOLEAN
			-- Does directory `a_path' exist in the file system?
		do
			Result := convention.is_directory_exists (a_path)
		end

	seconds_elapsed (a_start, a_end: ROSE_DATE): INTEGER_32
			-- Number of seconds elapsed from 'a_start' to 'a_end' date-time
		require
			valid_dates: a_start /= Void and then a_end /= Void
		local
			l_duration: DATE_TIME_DURATION
		do
			l_duration := a_end.relative_duration (a_start)
			Result := l_duration.time.seconds_count
		end

	checked_path (a_dir, a_file: STRING): STRING
			-- Composed path from 'a_dir' and 'a_file', Void if such file does not exist
		do
			Result := convention.checked_path (a_dir, a_file)
		end

	timestamp: attached STRING
			-- Timestamp suitable for use in log outputs (of the form MM/DD hh:mi)
		do
			Result := (create {ROSE_DATE}.make_now).formatted_out ("[0]mm/[0]dd hh:[0]mi:[0]ss")
		end

	timestamp_utc: attached STRING
			-- UTC Timestamp suitable for use in log outputs (of the form MM/DD hh:mi)
		do
			Result := (create {ROSE_DATE}.make_now_utc).formatted_out ("[0]mm/[0]dd hh:[0]mi:[0]ss")
		end

	file_access_log: FILE_ACCESS_LOG
			-- Log for accessed files (input and output)
		once
			create Result
		end

feature -- Basic operations

	invoke (a_agent: PROCEDURE [ANY, TUPLE []])
			-- Invoke 'a_agent', execute it on main thread if possible
		require
			agent_ok: a_agent /= Void
		do
			if invoker_cell.item /= Void then
				invoker_cell.item.call ([a_agent])
			end
		end

	ensure_storable_compatibility_with_56
			-- Ensure that storable created are retrievable with estudio 5.6
			-- You may call this many times
			-- If called once, it sets the compatibility mode for the entire process
		indexing
			once_status: global
		once
			class_translation.do_nothing
		end

	set_termination_code (a_code: INTEGER)
			-- Set application's 'termination_code'.
			-- This code is returned to shell on exit, use with care.
		do
			termination_code := a_code
		end

	set_log_id (a_log_id: INTEGER)
			-- Set 'log_id', which is used to identify the run of this app, for logging.
		do
			log_id := a_log_id
		end

	execute (a_line: STRING)
			-- Execute 'a_line' as a framework command.
		do
			commands.execute (a_line)
		end

	execute_file (a_file: STRING)
			-- Execute 'a_file' with associated Windows handler.
			-- If 'a_file' is not readable, no operation is performed, just silently ignored.
		do
			if is_file_readable (a_file) then
				env.launch ("cmd /c start " + a_file)
			end
		end

	log_message (a_severity: INTEGER; a_class, a_feature, a_message: STRING; a_args: ARRAY [ANY])
			-- Log 'a_message' + 'a_args' with 'a_severity', marked coming from 'a_class'.'a_feature'.
			-- This call outputs to the current log channel only.
			-- 'a_severity' corresponds to one of the ROSE_FRAMEWORK_CONVENTION.log_* constants
			-- 'log_id' is used to differentiate logs between runs
		require
			valid_severity: convention.is_valid_severity (a_severity)
		do
			io_handler.output_log (a_severity, log_id, a_class, a_feature, a_message, a_args)
		end

	frozen explore (a_path: STRING)
			-- Launch an explorer window opened on 'a_path' if path is valid.
			-- Silently do nothing otherwise.
		do
			env.explore (a_path)
		end

	explore_url (a_url: STRING)
			-- Open 'a_url' in default web browser.
		do
			env.explore_url (a_url)
		end

	terminate (a_exit_code: INTEGER)
			-- Terminate application execution with 'a_exit_code'.
			-- This call does not return, once you call this the application exits
		local
			was_signaled: BOOLEAN
		do
			was_signaled := io_handler.error.signaled
			io_handler.error.acknowledge
			from
			until
				termination_agents.is_empty
			loop
				call_agent (termination_agents.item)
				was_signaled := was_signaled or else io_handler.error.signaled
				io_handler.error.acknowledge
				termination_agents.remove
			end
			io_handler.error.flush_and_acknowledge
			if not io_handler.error.is_flushed then
					--| Errors have not been flushed completely, try last-resort paths to flush them
				from
				until
					io_handler.has_some_output or else last_resort_channels.is_empty
				loop
					set_last_resort (last_resort_channels.item)
					last_resort_channels.remove
				end
				io_handler.error.flush
			end
			if a_exit_code /= 0 then
					--| Status code explicitly provided, use it
				io_handler.error.die (a_exit_code, Void, Void, Void)
			elseif io_handler.error.signaled then
					--| An error in last resort redirection happened, exit with status code -1
				io_handler.error.die (-1, Void, Void, Void)
			elseif was_signaled then
					--| An error occurred and was properly handled, exit with status code 1
				io_handler.error.die (1, Void, Void, Void)
			else
					--| No errors upon exit, quit normally
				io_handler.close_all
				io_handler.error.die (0, Void, Void, Void)
			end
		end

	add_last_resort_channel (a_path: STRING)
			-- Add 'a_path' as last-resort alternative to flush errors to.
			-- Last resort paths are stacked, tried in a FIFO fashion.
			--| ex: to flush to a file, use "file:path/to/log/file.log"
			--| ex: to send an email, use "mail:recipient1@domain, recipient2, ...%Nsubject" in this array
		do
			if a_path /= Void and then not a_path.is_empty then
				last_resort_channels.extend (a_path)
			end
		end

	add_cleanup_agent (a_agent: PROCEDURE [ANY, TUPLE []])
			-- Add 'a_agent' to be called before application termination.
			-- Clean-up agents are stacked, called in a FIFO fashion.
		do
			if a_agent /= Void then
				termination_agents.extend (a_agent)
			end
		end

	complete_command_with_implicit (a_command: RCMD_MACRO)
			-- Complete 'a_command' with mandatory commands in 'implicit_commands'.
			-- 'implicit_commands' that are not mentioned in 'a_command' are to it, in front
			--| Example: 'system -start' command is implicit
		local
			l_stack: LINKED_STACK [STRING]
			l_command: ROSE_COMMAND
		do
			from
				create l_stack.make
				app.implicit_commands.start
			until
				app.implicit_commands.off
			loop
				l_stack.extend (app.implicit_commands.item)
				app.implicit_commands.forth
			end
			from
			until
				l_stack.is_empty
			loop
				l_command := commands.parsed_command (l_stack.item)
				if l_command /= Void and then not a_command.has (l_command.id) then
					a_command.replace (l_command)
				end
				l_stack.remove
			end
		end

	ensure_directory_exists (a_path: STRING)
			-- ensure `a_path' exists (as a directory) in the filesystem
		require
			path_not_void: a_path /= Void
			path_is_valid_directory: (create {FILE_NAME}.make_from_string(a_path)).is_valid
		local
			l_path: STRING
			l_test: BOOLEAN
		do
			l_test := (create {FILE_NAME}.make_from_string(a_path + "\")).is_valid
			l_path := formatter.clean_directory_path (a_path)
			if not is_directory_exists (l_path) then
				recursive_create_directory (l_path)
			end
		ensure
			directory_exists: is_file_exists (formatter.clean_directory_path(a_path))
		rescue
			--output_timestamped ("$1.ensure_directory_exists: path: $2%N", << generator, a_path >>)
			--fail ("ensure_directory_exists", "$1", << a_path >>)
		end

	recursive_create_directory (a_path: STRING)
			-- Create directory with 'a_path'
		do
			convention.recursive_create_directory (a_path)
		end

	set_date_format (a_date_format: STRING)
			-- Set 'date_format'.
		require
			format_ok: a_date_format /= Void
		do
			date_format := a_date_format
		end

	set_date_time_format (a_date_time_format: STRING)
			-- Set 'date_time_format'.
		require
			format_ok: a_date_time_format /= Void
		do
			date_time_format := a_date_time_format
		end

	report_unreachable_code_reached
			-- Helper feature for finding bugs related to early handling of attached types
			-- Call this routine from parts of the code that should never be reached
			-- Once we provide full support for "on demand Void safety", we will be able to get rid of this routine
		do
			if app.is_running_in_estudio then
				-- FIXME: to be removed when the code integrates with "on demand Void safety"
				fixme ("This should never be reached as it means the precondition is violated. Review and fix this feature for Void Safety.")
				io_handler.error.raise ("Unexpected void string received in ROSE_GLOBAL.hardcoded_path")
			end
		end

feature {NONE} -- Implementation

	set_last_resort (a_path: STRING)
			-- Set last resort channel with channel from 'a_path'.
		local
			rescued, l_path: STRING
		do
			if rescued /= Void then
				io_handler.error.signal (app.program_name, "Last resort tentative to direct output to '$1' failed: $2", <<a_path, rescued>>)
			else
				l_path := interpreted (a_path)
				if l_path /= Void and then not l_path.is_empty then
					io_handler.set_log (io_handler.manager.checked_output_channel (app.program_name, l_path))
				end
			end
		rescue
			if rescued = Void then
				rescued := io_handler.error.last_exception_explanation
				retry
			end
		end

	call_agent (a_agent: PROCEDURE [ANY, TUPLE []])
			-- Call 'a_agent' safely (catch any exceptions and turn them into error message)
		local
			rescued: STRING
		do
			if rescued /= Void then
				io_handler.error.signal (app.program_name, "Calling termination agent crashed: $1", <<rescued>>)
			else
				a_agent.call (Void)
			end
		rescue
			if rescued = Void then
				rescued := io_handler.error.last_exception_explanation
				retry
			end
		end

	last_resort_channels: LINKED_STACK [STRING]
			-- Channels where to output any pending error when process exits

	termination_agents: LINKED_STACK [PROCEDURE [ANY, TUPLE []]]
			-- Agents to call when application exits

	class_translation: CLASS_NAME_TRANSLATIONS
			-- Class translation object for storables
		indexing
			once_status: global
		once
			create Result
			Result.put ("INTEGER_REF", "INTEGER_32_REF")
			Result.put ("INTEGER", "INTEGER_32")
			Result.put ("WIDE_CHARACTER", "CHARACTER_32")
			Result.put ("CHARACTER_REF", "CHARACTER_8_REF")
			Result.put ("REAL_REF", "REAL_32_REF")
			Result.put ("REAL", "REAL_32")
			Result.put ("STRING", "STRING_8")
			Result.put ("DOUBLE_REF", "REAL_64_REF")
			Result.put ("NATURAL", "NATURAL_32")
			Result.put ("DOUBLE", "REAL_64")
			Result.put ("WIDE_CHARACTER_REF", "CHARACTER_32_REF")
			Result.put ("CHARACTER", "CHARACTER_8")
		end

end
