indexing
	description: "Display features of a ROSE_GRAPHICS_WIDGET"
	author: "Zoran Simic"
	date: "$Date: 2009/11/17 $"
	revision: "$Revision: #8 $"

class
	ROSE_GRAPHICS_DISPLAY

inherit
	ROSE_SHARED_GLOBAL_GUI

create
	make

feature {NONE} -- Initialization

	make (a_graphics: like graphics) is
		require
			valid_graphics: a_graphics /= Void
		local
			l_pixmap: EV_PIXMAP
		do
			graphics := a_graphics
			create state
			create figure_world
			create pixmap
			pixmap.set_minimum_size (10,10)
			create projector.make (figure_world,pixmap)
			create drawn_area
			create selection.make
			selection.set_pixmap (pixmap)
			selection.set_selectable_area (drawn_area)
				-- Groups in order of priority (least priority first)
			initialize_figures
				-- Static figures
			create title_text
			title_text.set_point_position (0,-2)
			figure_world.extend (title_text)
			create message_text
			message_text.set_point_position (40,40)
			message_text.set_foreground_color (Gui.rep.colors.Red)
			message_text.set_font (Gui.rep.fonts.new_roman_font (True,False,16))
			figure_world.extend (message_text)
			set_legend (new_legend)
			legend.set_icon_size (4, 4)
			l_pixmap := Gui.factory.new_pixmap_from_file ("hand.ico")
			if l_pixmap = Void then
				hand_cursor := Gui.rep.pixmaps.Standard_cursor
			else
				create hand_cursor.make_with_pixmap (l_pixmap,11,4)
			end
			create settings.make (Current)
			update_figures
		end

	initialize_figures is
			-- Custom figure_world initialization, redefine if needed.
		do
		end

feature -- Access

	graphics: ROSE_GRAPHICS_WIDGET
			-- Associated widget

	state: ROSE_GRAPHICS_STATE
			-- Associated state

	settings: ROSE_GRAPHICS_SETTINGS
			-- Settings to use

	pixmap: EV_PIXMAP
			-- Pixmap where the graphics are drawn
			-- Use with caution, EV_PIXMAP_PROJECTOR changes the state of this pixmap heavily

	width, height: INTEGER
			-- Current width and height of the whole display

	selection: ROSE_SELECTION_AREA
			-- Selection area

	drawn_area: ROSE_RECTANGLE
			-- Part of the pixmap dedicated to showing the graphics

	hand_cursor: EV_CURSOR

	legend: ROSE_LEGEND [ANY]
			-- Legend associated to this graphics display

	palette: ROSE_PALETTE is
			-- Color palette used (comes from 'legend')
		require
			legend_not_void: legend /= Void
		do
			Result := legend.palette
		end

	title_text: EV_MODEL_TEXT
			-- Holds the title for this display

	message_text: EV_MODEL_TEXT
			-- Holds info messages set using 'widget.inform'

feature -- Basic operations

	redraw is
		local
			retried: BOOLEAN
		do
			if retried then
				graphics.clear_all
				set_title ("-  Figure projection crashed!  -")
			else
				start_time_measure ("redraw",generating_type)
				projector.full_project
			end
			selection.redraw
			end_time_measure
		rescue
			if not retried then
				retried := True
				retry
			end
		end

	set_title (a_title: STRING) is
		do
			if a_title = Void or else a_title.is_empty then
				title_text.set_text ("")
				title_text.hide
			else
				title_text.set_text (a_title)
				title_text.show
			end
		end

	set_legend (a_legend: like legend) is
		do
			if legend /= Void then
				figure_world.prune_all (legend.figure)
			end
			legend := a_legend
			if legend /= Void then
				legend.figure.end_actions.extend (agent on_legend_moved)
				figure_world.extend (legend.figure)
			end
		end

	update_figures is
			-- Update figures from 'settings'.
		do
			title_text.set_font (settings.title_font)
			title_text.set_foreground_color (settings.title_color)
		end

	refresh is
		do
			-- Calculate main drawn area coordinates
			width := pixmap.width
			height := pixmap.height
			-- Position main figures accordingly
			drawn_area.make (0,0,width,height)
			check_legend_position
		end

	set_tooltip (a_tooltip: STRING; a_args: ARRAY [ANY]) is
		do
			if a_tooltip = Void then
				pixmap.remove_tooltip
			else
				pixmap.set_tooltip (Global.formatted (a_tooltip,a_args))
			end
		end

	clear_all is
			-- Clear out the graphics.
		do
			selection.reset
			message_text.set_text ("")
		end

feature {ROSE_GRAPHICS_WIDGET} -- Implementation

	figure_world: EV_MODEL_WORLD

feature {NONE} -- Implementation

	new_legend: like legend is
			-- new legend instance
		do
			Result := create {ROSE_LEGEND_STANDARD}.make ("Legend")
		end

	legend_positioned_once: BOOLEAN
	legend_x_ratio: DOUBLE
	legend_y_ratio: DOUBLE

	record_legend_relative_position is
			-- Record legend's relative position to the view
		do
			legend_x_ratio := legend.x / width
			legend_y_ratio := legend.y / height
		end

	projector: EV_MODEL_PIXMAP_PROJECTOR

	on_legend_moved is
			-- Called when legend has been moved
		do
			record_legend_relative_position
		end

	check_legend_position is
			-- Make sure legend is following
		do
		end

	set_color (a_figure: EV_MODEL_CLOSED; a_foreground, a_background: EV_COLOR) is
		do
			if a_figure /= Void then
				if a_foreground = Void then
					a_figure.set_foreground_color (settings.palette.foreground)
				else
					a_figure.set_foreground_color (a_foreground)
				end
				if a_background = Void then
					a_figure.set_background_color (settings.palette.background)
				else
					a_figure.set_background_color (a_background)
				end
			end
		end

	new_group (a_parent: EV_MODEL_GROUP): EV_MODEL_GROUP is
		do
			create Result
			a_parent.extend (Result)
		end

	new_rectangle (a_parent: EV_MODEL_GROUP): EV_MODEL_RECTANGLE is
		do
			create Result
			a_parent.extend (Result)
		end

end
