indexing
	description: "Settings for a ROSE_GRAPHICS_WIDGET"
	author: "Zoran Simic"
	date: "$Date: 2008/12/01 $"
	revision: "$Revision: #4 $"

class
	ROSE_GRAPHICS_SETTINGS

inherit
	ROSE_SHARED_GLOBAL_GUI
		export
			{NONE} all
		end
	ANY

create
	make

feature {NONE} -- Initialization

	make (a_display: like display) is
			-- Create settings attached to 'a_display'.
		require
			valid_display: a_display /= Void
		do
			display := a_display
			set_title_font (Gui.rep.fonts.new_roman_font (True,False,24))
			set_title_color (Gui.rep.colors.Dark_blue)
				-- Default settings
			set_minimum_text_width (0)
			set_minimum_text_height (0)
			set_point_size (5)
			set_margin (2)
			initialize
		end

	initialize is
			-- Specific initialization, redefine if needed.
		do
		end

feature -- Access

	display: ROSE_GRAPHICS_DISPLAY
			-- Associated display

	palette: ROSE_PALETTE is
			-- Color palette (for indexed colors) + background and foreground colors
			-- Text is shown using palette.foreground_color except for title
		do
			Result := display.palette
		end

	minimum_text_width: INTEGER
	minimum_text_height: INTEGER
			-- Minimum text dimension (if text would be smaller than that, it is not shown)

	point_size: INTEGER
			-- Default dimensions of a point

	margin_right: INTEGER
			-- Margin on right

	margin_left: INTEGER
			-- Margin on right

	margin_bottom: INTEGER
			-- Margin on right

	margin_top: INTEGER
			-- Margin on right

	title_height: INTEGER
			-- Various margins

	title_font: EV_FONT
			-- Fonts to use

	title_color: EV_COLOR
			-- Color for title figure

feature -- Basic operations

	set_minimum_text_width (a_size: INTEGER) is
			-- Set 'minimum_text_width'.
		do
			minimum_text_width := a_size.max (8)
		end

	set_minimum_text_height (a_size: INTEGER) is
			-- Set 'minimum_text_width'.
		do
			minimum_text_height := a_size.max (8)
		end

	set_point_size (a_size: INTEGER) is
			-- Set 'point_size'.
		do
			point_size := a_size.max (1)
		end

	set_margin (a_margin: INTEGER) is
			-- Set all margins to the same value
		local
			l_value: INTEGER
		do
			l_value := a_margin.max (0)
			margin_right := l_value
			margin_left := l_value
			margin_bottom := l_value
			margin_top := l_value
		ensure
			margin_same: margin_right = margin_left and margin_left = margin_bottom and margin_bottom = margin_top
			margin_valid: margin_right >= 0 and then ( margin_right = a_margin or a_margin < 0 )
		end

	set_right_margin (a_margin: INTEGER) is
			-- Set `margin_right' to a_margin
		require
			margin_valid: a_margin >= 0
		do
			margin_right := a_margin.max (0)
		ensure
			margin_correct: margin_right = a_margin
		end

	set_left_margin (a_margin: INTEGER) is
			-- Set `margin_left' to a_margin
		require
			margin_valid: a_margin >= 0
		do
			margin_left := a_margin.max (0)
		ensure
			margin_correct: margin_left = a_margin
		end

	set_bottom_margin (a_margin: INTEGER) is
			-- Set `margin_bottom' to a_margin
		require
			margin_valid: a_margin >= 0
		do
			margin_bottom := a_margin.max (0)
		ensure
			margin_correct: margin_bottom = a_margin
		end

	set_top_margin (a_margin: INTEGER) is
			-- Set `margin_top' to a_margin
		require
			margin_valid: a_margin >= 0
		do
			margin_top := a_margin.max (0)
		ensure
			margin_correct: margin_top = a_margin
		end

	set_title_font (a_font: EV_FONT) is
			-- Set 'title_font'.
		require
			a_font_not_void: a_font /= Void
		do
			title_font := a_font
			title_height := a_font.height + a_font.descent
		end

	set_title_color (a_color: EV_COLOR) is
			-- Set 'title_color'.
		do
			title_color := a_color
		ensure
			color_set: title_color = a_color
		end

end -- class ROSE_GRAPHICS_SETTINGS
