indexing
	description: "File input channel"
	author: "Zoran Simic"
	date: "$Date: 2008/12/01 $"
	revision: "$Revision: #2 $"

class ROSE_INCHANNEL_FILE

inherit
	ROSE_INCHANNEL

feature -- Access

	id: STRING is "file"
			-- Channel type id

	medium: FILE
			-- Actual input file used for this channel

feature -- Status report

	is_open: BOOLEAN is
			-- Is this channel open ?
		do
			Result := medium /= Void and then medium.is_open_read
		end

	is_exhausted: BOOLEAN is
		do
			Result := medium = Void or else medium.after
		end

feature -- Basic operations

	ask (a_question: STRING): STRING is
			-- Ask 'a_question' on this channel
			-- Void when input is exhausted
		do
			if not is_exhausted then
				medium.read_line
				Result := clone (medium.last_string)
			end
		end

feature -- Output

	help: STRING is "Input contents of a text file (line by line)"
			-- Help on this command

feature {NONE} -- Implementation

	do_open is
			-- Open effectively this channel with 'parameters'.
		local
			rescued: BOOLEAN
		do
			if not rescued and then parameters /= Void and then not parameters.is_empty then
				create {PLAIN_TEXT_FILE} medium.make (parameters)
				if medium.exists and then medium.is_readable then
					medium.open_read
				end
			end
		rescue
			if not rescued then
				rescued := True
				retry
			end
		end

	do_close is
			-- Close effectively this channel.
		do
			if is_open then
				medium.close
			end
			medium := Void
		end

end -- class ROSE_INCHANNEL_FILE
