indexing
	description: "Standard files input channel"
	author: "Zoran Simic"
	date: "$Date: 2008/12/01 $"
	revision: "$Revision: #5 $"

class ROSE_INCHANNEL_STANDARD

inherit
	ROSE_INCHANNEL
		redefine
			new_representative
		end

create
	make

feature {NONE} -- Initialization

	make is
			-- Default input channel 'standard'.
		do
			is_interactive := True
		end

feature -- Access

	id: STRING is "standard"
			-- Channel type id

	medium: FILE is
			-- Actual medium used for input
		once
			Result := io.input
		end

	new_representative: like Current is
			-- New independent version of Current service.
			-- If you want this service to be shared (unique instance used for the whole system),
			-- return Current (no clone)
		do
			Result := Current
		end

feature -- Status report

	is_open: BOOLEAN is
			-- Is this channel open ?
		do
			Result := medium.is_open_read
		end

feature -- Basic operations

	ask (a_question: STRING): STRING is
			-- Ask 'a_question' on this channel
			-- Void when input is exhausted
		do
			if is_open and then not medium.after then
				if a_question /= Void then
					io.put_string (a_question)
				end
				medium.read_line
				Result := clone (medium.last_string)
			end
		end

feature -- Output

	help: STRING is "Input from standard file (console)"
			-- Help on this command

feature {NONE} -- Implementation

	do_open is
			-- Open effectively this channel with 'parameters'.
		do
				--| Relies on standard IO
		end

	do_close is
			-- Close effectively this channel.
		do
				--| Standard files are not closed by this channel
		end

end -- class ROSE_INCHANNEL_STANDARD
