indexing
	description: "Input history tracking"
	author: "Zoran Simic"
	date: "$Date: 2008/12/01 $"
	revision: "$Revision: #8 $"

class
	ROSE_INPUT_HISTORY

inherit
	ROSE_INPUT_HISTORY_GENERIC [STRING]
		rename
			make_shell as generic_make_shell,
			make_browser as generic_make_browser
		redefine
			add
		end
	DEBUG_OUTPUT

create
	make_shell, make_browser

feature {NONE} -- Implementation

	make_shell (a_capacity: INTEGER; a_config: ROSE_CONFIGURATION) is
			-- Create with a shell-like history behavior.
		do
			config := a_config
			generic_make_shell (a_capacity)
		end

	make_browser (a_capacity: INTEGER; a_config: ROSE_CONFIGURATION) is
			-- Create with a browser-like history behavior.
		do
			config := a_config
			generic_make_browser (a_capacity)
		end

feature -- Access

	config: ROSE_CONFIGURATION
			-- Configuration under which this history is remembered

	configuration_key: STRING
			-- Key under which this history is remembered

	debug_output: STRING is
			-- String that should be displayed in debugger to represent `Current'.
		do
			Result := count.out + " items"
		end

feature -- Basic operations

	add (a_item: STRING) is
			-- Add 'a_item' to this history.
			-- If 'is_shell_like' add 'a_item' in a stack like fashion (and only one item per history is present)
			-- otherwise, remove items following 'item' if any and add 'a_item' there (like a web browser)
		do
			if a_item /= Void and then not a_item.is_empty then
				Precursor (a_item)
			end
		end

	set_configuration_key (a_key: STRING) is
		require
			config /= Void and then a_key /= Void and then not a_key.is_empty
		local
			l_node: ROSE_CONFIGURATION_NODE
			i: INTEGER
		do
			configuration_key := k_history + a_key
			l_node := config.node (configuration_key)
			if l_node /= Void and then l_node.children /= Void then
				from
					i := l_node.children.lower
				until
					i > l_node.children.upper
				loop
					add (l_node.children.item (i).value)
					i := i + 1
				end
			end
		end

	save is
			-- Save this history to user config.
		require
			config /= Void and then configuration_key /= Void and then not configuration_key.is_empty
		local
			l_parent: ROSE_CONFIGURATION_NODE
			i: INTEGER
		do
			l_parent := config.root.forced_node (configuration_key,False)
			l_parent.delete_children
			from
				i := 1
			until
				i > count
			loop
				l_parent.add_anonymous (item (i))
				i := i + 1
			end
		end

	prune_all (a_text: STRING) is
		do
			remove_all (a_text)
		end

feature {NONE} -- Implementation

	k_history: STRING is "history/"

end
