note

	description:

		"integer division operator"

	author: "Xinhui Chen"
	last_editor: "$Author: it $"
	date: "$Date: 2013/04/12 $"
	revision: "$Revision: #13 $"
	path: "$File: //rose/source/kernel/math/rose_integer_division_operator.e $"

class
	ROSE_INTEGER_DIVISION_OPERATOR [reference G]

inherit
	ROSE_NUMERIC_BINARY_OPERATOR [G]
		redefine
			real_value,
			double_value,
			computed_real,
			computed_double,
			generated_real,
			generated_double,
			value_in_market_currency
		end

create
	make

feature -- Access

	nominal_units (a_left,a_right : ROSE_UNITS) : ROSE_UNITS is
			-- <Precursor>
		do
			Result := a_left / a_right
		end

	real_value (a_left, a_right: ROSE_EXPRESSION [G]; a_g: G): REAL is
			-- <Precursor>
		local
			l_integer : INTEGER
		do
			l_integer := integer_value(a_left, a_right, a_g)
			if is_defined_integer (l_integer) then
				Result := l_integer
			else
				Result := undefined_real
			end
		end

	double_value (a_left,a_right : ROSE_EXPRESSION[G];a_g : G) : DOUBLE is
			-- <Precursor>
		local
			l_integer : INTEGER
		do
			l_integer := integer_value(a_left, a_right, a_g)
			if is_defined_integer (l_integer) then
				Result := l_integer
			else
				Result := undefined_double
			end
		end

	value_in_market_currency (a_left,a_right : ROSE_EXPRESSION[G];a_market : MARKET;a_currency : PERSISTENT_CURRENCY;a_entity : G;a_date : ROSE_DATE) : DOUBLE is
			-- <Precursor>
		local
			l_left,l_right : INTEGER
		do
			Result := undefined_double
			l_left :=  a_left.value_in_market_currency(a_market,a_currency,a_entity,a_date).rounded
			if is_defined_integer(l_left) then
				l_right := a_right.value_in_market_currency(a_market,a_currency,a_entity,a_date).rounded
				if l_right /= 0 and then is_defined_integer(l_right) then
					Result := l_left // l_right
				end
			end
		end


	computed_units (a_left,a_right : ROSE_UNITS) : ROSE_UNITS is
			-- <Precursor>
		do
			Result := a_left / a_right
		end
	units_in_currency (a_left,a_right : ROSE_EXPRESSION[G];a_currency : PERSISTENT_CURRENCY;a_entity : G) : ROSE_UNITS is
			-- <Precursor>
		do
			Result := a_left.units_in_currency(a_currency,a_entity) / a_right.units_in_currency(a_currency,a_entity)
		end
	unitized_value(a_left,a_right : ROSE_UNITIZED_VALUE[DOUBLE]) : ROSE_UNITIZED_VALUE[DOUBLE]
			-- <Precursor>
		do
			if a_right.value.rounded /= 0 then
				create Result.make_with_units(a_left.value.rounded // a_right.value.rounded,a_left.units / a_right.units)
			else
				create Result.make_with_units(maximum_integer,a_left.units / a_right.units)
			end
		end

feature -- Access

	computed_real (a_left, a_right: ROSE_EXPRESSION [G]; a_g: G; a_agent: like access_agent_template): REAL is
			-- <Precursor>
		local
			l_integer : INTEGER
		do
			l_integer := computed_integer(a_left, a_right, a_g, a_agent)
			if is_defined_integer (l_integer) then
				Result := l_integer
			else
				Result := undefined_real
			end
		end

	computed_double (a_left, a_right: ROSE_EXPRESSION [G]; a_g: G; a_agent: like access_agent_template): DOUBLE is
			-- <Precursor>
		local
			l_integer : INTEGER
		do
			l_integer := computed_integer(a_left, a_right, a_g, a_agent)
			if is_defined_integer (l_integer) then
				Result := l_integer
			else
				Result := undefined_double
			end
		end
	generated_real (a_left, a_right: ROSE_EXPRESSION [G]; a_g: G; a_agent: like access_agent_template): REAL is
			-- <Precursor>
		local
			l_integer : INTEGER
		do
			l_integer := generated_integer(a_left, a_right, a_g, a_agent)
			if is_defined_integer (l_integer) then
				Result := l_integer
			else
				Result := undefined_real
			end
		end
	generated_double (a_left, a_right: ROSE_EXPRESSION [G]; a_g: G; a_agent: like access_agent_template): DOUBLE is
			-- <Precursor>
		local
			l_integer : INTEGER
		do
			l_integer := generated_integer(a_left, a_right, a_g, a_agent)
			if is_defined_integer (l_integer) then
				Result := l_integer
			else
				Result := undefined_double
			end
		end

feature {NONE} -- Implementation

	double_computation(a_left, a_right: ROSE_EXPRESSION [G]; a_left_value, a_right_value : DOUBLE) : DOUBLE is
			-- <Precursor>
		do
		end
	decimal_computation(a_left, a_right: ROSE_EXPRESSION [G]; a_left_value, a_right_value : RP_DECIMAL) : RP_DECIMAL is
			-- <Precursor>
		do
		end
	real_computation(a_left, a_right: ROSE_EXPRESSION [G]; a_left_value, a_right_value : REAL) : REAL is
			-- <Precursor>
		do
		end
	integer_computation(a_left, a_right: ROSE_EXPRESSION [G]; a_left_value, a_right_value : INTEGER) : INTEGER is
			-- <Precursor>
		do
			Result := undefined_integer
			if is_defined_integer (a_left_value) and (a_right_value /= 0 and then is_defined_integer (a_right_value)) then
				Result := a_left_value // a_right_value
			end
		end
	string_computation(a_left, a_right: ROSE_EXPRESSION [G]; a_left_value, a_right_value : STRING) : STRING is
			-- <Precursor>
		do
		end
	boolean_computation(a_left, a_right: ROSE_EXPRESSION [G]; a_left_value, a_right_value : BOOLEAN) : BOOLEAN is
			-- <Precursor>
		do
		end
	date_computation(a_left, a_right: ROSE_EXPRESSION [G]; a_left_value, a_right_value : ROSE_DATE) : ROSE_DATE is
			-- <Precursor>
		do
		end

end
