indexing

	description: "Collection of intervals"
	author: "Unknown"
	last_editor: "$Author: c4 $"
	date: "$Date: 2012/10/16 $"
	revision: "$Revision: #21 $"
	path: "$File: //rose/source/kernel/infrastructure/containers/rose_interval_history.e $"

class ROSE_INTERVAL_HISTORY [G, H -> ROSE_INTERVAL [G]]

inherit

	ROSE_LINEAR_ARRAY_OR_ONE [H]

	ROSE_CONSTANTS

create

	make_empty

feature -- Query

	min_start_date: ROSE_DATE is
			-- Minimum start date of all intervals
		local
			i: INTEGER
		do
			from
				i := 1
			until
				i > count
			loop
				if Result = Void or else Result > item (i).start_date then
					Result := item (i).start_date
				end
				i := i + 1
			end
		end

	max_end_date: ROSE_DATE is
			-- maximum end date of all intervals
		local
			i: INTEGER
		do
			from
				i := 1
			until
				i > count
			loop
				if Result = Void or else Result < item (i).end_date then
					Result := item (i).end_date
				end
				i := i + 1
			end
		end

	most_recent (a_date: ROSE_DATE): H is
			-- Most recent interval as of `a_date'
			-- Implementation does not assume that history is sorted
		require
			non_void_date: a_date /= Void
		local
			i: INTEGER
		do
			from
				i := 1
			until
				i > count
			loop
				if item (i).starts_by_date (a_date) then
					if Result = Void or else (item (i).ends_after (Result)) then
						Result := item (i)
					end
				end
				i := i + 1
			end
		end

	witness_interval (a_date: ROSE_DATE): H is
			-- Interval bracketing `a_date'
			-- NOTE: interval returned depends on order of intervals
			-- NOTE: uses internal iterator. Side effects.
		require
			a_date_not_void : a_date /= Void
		do
			from
				start
			until
				off or Result /= Void
			loop
				if current_item.brackets (a_date) then
					Result := current_item
				end
				forth
			end
		end

	witness_intervals (a_date: ROSE_DATE): ROSE_LINEAR_ARRAY [H] is
			-- All intervals bracketing `a_date'
			-- NOTE: uses internal iterator. Side effects.
		do
			create Result.make_empty
			from
				start
			until
				off
			loop
				if current_item.brackets (a_date) then
					Result.extend (current_item)
				end
				forth
			end
		end

	has_witness (a_item: G): BOOLEAN is
			-- Is there interval which has `a_item'?
			-- NOTE: uses internal iterator. Side effects.
		do
			from
				start
			until
				off or Result
			loop
				if current_item.item = a_item then
					Result := True
				end
				forth
			end
		end

	intervals_during (a_start, a_end: ROSE_DATE; a_item: G): like Current is
			--Void start is minus infinity, void end is infinity
			--Void a_entity means that you get all the intervals
		local
			l_default: G
			i: INTEGER
			l_start, l_end: ROSE_DATE
		do
			create Result.make_empty
			if a_start = Void and a_end = Void and a_item = l_default then
				Result := twin
			else
				if a_start = Void then
					l_start := constants.minimum_date_time
				else
					l_start := a_start
				end
				if a_end = Void then
					l_end := constants.maximum_date_time
				else
					l_end := a_end
				end
				from
					i := 1
				until
					i > count
				loop
					if a_item = l_default then
						if item (i).meets (l_start, l_end) then
							Result.extend (item (i))
						end
					else
						if item (i).meets (l_start, l_end) and item (i).item = a_item then
							Result.extend (item (i))
						end
					end
					i := i + 1
				end
			end
		end

	interval_before (a_date: ROSE_DATE): H is
		local
			i: INTEGER
			l_found: BOOLEAN
		do
			from
				i := 1
			until
				i > count or l_found
			loop
				if item (i).brackets (a_date) then
					l_found := True
					if i > 1 then
						Result := item (i - 1)
					end
				end
				i := i + 1
			end
		end

	interval_after (a_date: ROSE_DATE): H is
		local
			i: INTEGER
			l_found: BOOLEAN
		do
			from
				i := count
			until
				i < 1 or l_found
			loop
				if item (i).brackets (a_date) then
					l_found := True
					if i < count then
						Result := item (i + 1)
					end
				end
				i := i - 1
			end
		end

	witnesses_during (a_start, a_end: ROSE_DATE): ROSE_LINEAR_ARRAY [G] is
			-- Collection of items that meet time period defined by `a_start' and `a_date'
		local
			l_default: G
			i: INTEGER
			l_intervals: like intervals_during
		do
			create Result.make_empty
			l_intervals := intervals_during (a_start, a_end, l_default)
			from
				i := 1
			until
				i > l_intervals.count
			loop
				Result.extend (l_intervals.item (i).item)
				i := i + 1
			end
		end

	witnesses_covering (a_start, a_end: ROSE_DATE): ARRAY [G] is
			-- Collection of items whos interval cover time period defined by `a_start' and `a_end'
			-- i.e. interval start on or before `a_start' and ends on or after `a_end'
		require
			non_void_dates: a_start /= Void and a_end /= Void
			start_less_than_or_equal_end: a_start <= a_end
		local
			l_collection: DS_LINKED_LIST [G]
		do
			create l_collection.make_default
			do_if (agent add_item (l_collection, ?), agent covers (a_start, a_end, ?))
			Result := l_collection.to_array
		end

	witness (a_date: ROSE_DATE): G is
			-- Item from interval bracketing `a_date'
		require
			a_date_not_void : a_date /= Void
		local
			l_interval: H
		do
			l_interval := witness_interval (a_date)
			if l_interval /= Void then
				Result := l_interval.item
			end
		end

	all_witnesses (a_date: ROSE_DATE): ROSE_LINEAR_ARRAY [G] is
			-- All items from intervals bracketing `a_date'
		local
			l_array: ROSE_LINEAR_ARRAY [H]
		do
			create Result.make_empty
			if a_date /= Void then
				l_array := witness_intervals (a_date)
				if l_array /= Void then
					from
						l_array.start
					until
						l_array.off
					loop
						Result.extend (l_array.current_item.item)
						l_array.forth
					end
				end
			else
				from
					Current.start
				until
					Current.off
				loop
					if not Result.has_reference (Current.current_item.item) then
						Result.extend (Current.current_item.item)
					end
					Current.forth
				end
			end
		end

	primary_witnesses (a_date: ROSE_DATE): ROSE_LINEAR_ARRAY [G] is
			-- Returns item(s) with the max weight for intervals bracketing `a_date'
		local
			l_weight, l_max_weight: DOUBLE
		do
			create Result.make_empty
			if a_date /= Void then
				from
					Current.start
				until
					Current.off
				loop
					if Current.current_item.brackets (a_date) then
						l_weight := Current.current_item.weight
						if l_weight = l_max_weight then
							Result.extend (Current.current_item.item)
						else
							if l_weight > l_max_weight then
								l_max_weight := l_weight
								Result.wipe_out
								Result.extend (Current.current_item.item)
							end
						end
					end
					Current.forth
				end
			end
		end

	is_non_overlapping: BOOLEAN is
			-- Are all intervals disjoined?
		local
			i: INTEGER
		do
			if count = 1 or else count = 0 then
				Result := True
			else
				sort_by_start_date
				Result := True
				from
					i := lower
				until
					(i > upper - 1) or not Result
				loop
					Result := item (i).disjoint (item (i + 1))
					i := i + 1
				end
			end
		end

	event_dates: ROSE_LINEAR_ARRAY [ROSE_DATE] is
			-- Collection of all interval end dates
			-- NOTE: Uses built-in iterator. Side effects.
		do
			create Result.make (lower, upper)
			from
				start
			until
				off
			loop
				Result.put (current_item.start_date, index)
				forth
			end
		end

	is_continuous : BOOLEAN is
			-- Are there no gaps between intervals?
			-- Zero elements implies continuous
		local
			i : INTEGER
		do
			if count = 1 or else count = 0 then
				Result := True
			else
				sort_by_start_date
				Result := True
				from i := lower until (i > upper - 1) or not Result loop
					Result := item (i).end_date >= item (i + 1).start_date
					i := i + 1
				end
			end
		end

	is_contiguous : BOOLEAN is
			-- `is_non_overlapping' and `is_continuous'
		do
			Result := is_non_overlapping and is_continuous
		end

	sum_of_weights : DOUBLE
			-- Sum of weights of all intervals.
		local
			i : INTEGER
		do
			from i := lower until i > upper loop
				Result := Result + item (i).weight
				i := i + 1
			end
		end

	sum_of_signed_weights (a_is_positive : BOOLEAN) : DOUBLE
			-- Sum of positive/negative weights of all intervals.
		local
			i : INTEGER
		do
			from i := lower until i > upper loop
				Result := Result + item (i).weight * (a_is_positive = (item (i).weight > 0)).to_integer
				i := i + 1
			end
		end

feature -- Basic operations
	sort_by_start_date is
			-- Sort `Current' by interval start date
		do
			sort_by (agent order_by_start_date (?, ?))
		end

	append (a_other: like Current) is
			-- Append `a_other' to `Current'
		do
			if a_other /= Void then
				append_array (a_other)
			end
		end

	aggregate_contiguous: like Current is
			-- This interval history aggregate all the
			-- contiguous intervals that have the same weight
		local
			l_interval_history: like Current
			l_interval_array: ROSE_LINEAR_ARRAY [H]
			l_interval: H
			l_array: ROSE_LINEAR_ARRAY [G]
		do
			l_array := Current.all_witnesses (Void)
			if l_array.count > 0 then
				create Result.make_empty
				from
					l_array.start
				until
					l_array.off
				loop
					l_interval_history := intervals_during (Void, Void, l_array.current_item)
					l_interval_history.sort_by_start_date
					create l_interval_array.make_empty
					from
						l_interval_history.start
					until
						l_interval_history.off
					loop
						if l_interval_array.count > 0 then
							l_interval := l_interval_array.last
							if l_interval_history.current_item.start_date.is_equal (l_interval.end_date) and l_interval_history.current_item.weight.is_equal (l_interval.weight) then
								l_interval.make (l_interval.item, l_interval.start_date, l_interval_history.current_item.end_date, l_interval.weight)
							else
								l_interval_array.extend (l_interval_history.current_item)
							end
						else
							l_interval_array.extend (l_interval_history.current_item)
						end
						l_interval_history.forth
					end
					from
						l_interval_array.start
					until
						l_interval_array.off
					loop
						Result.add_interval (l_interval_array.current_item)
						l_interval_array.forth
					end
					l_array.forth
				end
			end
		end

	coalesce is
			-- Coalesce intervals with same weight and same item that are overlapping
		local
			i, j: INTEGER
			a, b: H
			l_coleasced, l_found: BOOLEAN
		do
			sort_by_start_date
			from
				i := 1
			until
				i > count
			loop
				a := item (i)
				from
					j := i + 1
					l_found := False
					l_coleasced := False
				until
					j > count or l_found
				loop
					b := item (j)
					if a.item = b.item then
						l_found := True
						if a.weight = b.weight and then a.ends_by_date (b.start_date) then
							a.set_end_date (b.end_date)
							remove_i_th (j)
							l_coleasced := True
						end
					end
					j := j + 1
				end
				if not l_coleasced then
					i := i + 1
				end
			end
		end

	add_interval (a_interval: H) is
		local
			l_interval: H
			l_interval_history: like Current
			l_interval_array: ROSE_LINEAR_ARRAY [H]
		do
			if a_interval /= Void then
				l_interval_history := intervals_during (a_interval.start_date, a_interval.end_date, a_interval.item)
				if l_interval_history /= Void and then l_interval_history.count > 0 then
					l_interval_history.sort_by_start_date
					create l_interval_array.make_empty
					l_interval := a_interval.twin
					from
						l_interval_history.start
					until
						l_interval_history.off
					loop
						l_interval_array := l_interval_array + (l_interval_history.current_item + l_interval)
						if l_interval_history.index /= l_interval_history.upper then
							l_interval := l_interval_array.last
							l_interval_array.go_i_th (l_interval_array.upper)
							l_interval_array.remove
						end
						l_interval_history.forth
					end
					remove_linear (l_interval_history, a_interval.item)
					from
						l_interval_array.start
					until
						l_interval_array.off
					loop
						extend (l_interval_array.current_item)
						l_interval_array.forth
					end
					sort_by_start_date
				else
					extend (a_interval)
				end
			end
		end

	add_history (a_history: like Current) is
			-- Add all of the intervals of `a_history' to `Current'
		do
			if a_history /= Void then
				from
					a_history.start
				until
					a_history.off
				loop
					add_interval (a_history.current_item)
					a_history.forth
				end
			end
		end

	infix "+" (a_other: like Current): like Current is
			-- Implementation of addition
		do
			Result := twin
			from
				a_other.start
			until
				a_other.off
			loop
				Result.add_interval (a_other.current_item)
				a_other.forth
			end
		end

	remove_linear (a_intervals: ROSE_LINEAR [H]; a_item: G) is
		local
			i: INTEGER
			l_interval: H
			l_table: ROSE_HASHED_LINEAR_ARRAY [ROSE_DATE]
		do
			create l_table.make (lower, upper)
			from
				start
			until
				off
			loop
				if current_item /= Void and current_item.item = a_item then
					l_table.put (current_item.start_date, index)
				end
				forth
			end
			from
				i := a_intervals.count
			until
				i < 1
			loop
				a_intervals.go_i_th (i)
				l_interval := a_intervals.item
				if l_interval /= Void and then l_table.has_value (l_interval.start_date) then
					remove_i_th (l_table.new_index_of (l_interval.start_date))
				end
				i := i - 1
			end
		end

	remove_i_th (i: INTEGER) is
			-- Remove `i'-th interval
		do
			if valid_index (i) then
				go_i_th (i)
				remove
			end
		end

feature {NONE} -- Implemenation

	interval: H is
			-- For anchoring
		do
		end

	order_by_start_date (a_interval_1, a_interval_2: H): BOOLEAN is
		do
			if a_interval_1 = Void then
				Result := a_interval_2 /= Void
			elseif a_interval_2 /= Void then
				Result := a_interval_1.before (a_interval_2)
			end
		end

	covers (a_start, a_end: ROSE_DATE; a_interval: H): BOOLEAN is
			-- Test wether `a_interval' covers time period defined by `a_start' and `a_end'
		require
			non_void_args: a_start /= Void and a_end /= Void and a_interval /= Void
		do
			Result := a_interval.covers (a_start, a_end)
		end

	add_item (a_collection: DS_LINKED_LIST [G]; a_interval: H) is
			-- Add item from `a_interval' to `a_collection'
		require
			non_void_args: a_collection /= Void and a_interval /= Void
		do
			a_collection.force_last (a_interval.item)
		ensure
			definition: a_collection.count > old a_collection.count
		end

end
