indexing

	description: "Objects that model a list of intervals in date order"
	author: "Mark Howard"
	last_editor: "$Author: rj $"
	date: "$Date: 2011/01/18 $"
	revision: "$Revision: #9 $"
	archive: "$File: //rose/source/kernel/infrastructure/structures/rose_interval_list.e $"

class ROSE_INTERVAL_LIST [G]

inherit

	LINEAR_PROXY [ROSE_INTERVAL [G]]
		rename
			linear as intervals,
			item as current_interval,
			search as lp_search
		redefine
			current_interval,
			occurrences
		end

	HASHABLE

create

	make,
	duplicate

feature {NONE} -- Creation

	make is
			-- Create the list
		do
			create intervals.make
		end

	duplicate (a_other: like Current) is
			-- Create the list and make it a duplicate of 'a_other'
		require
			valid_other: a_other /= Void
		do
			make
			from
				a_other.start
			until
				a_other.off
			loop
				insert (a_other.current_interval.twin)
				a_other.forth
			end
		end

feature -- Access	

	count: INTEGER
	-- Count of intervals.
	do
		Result := intervals.count
	end

	hash_code: INTEGER is
			-- <Precursor>
		do
			if intervals /= Void then
				Result := intervals.first.hash_code
			end
		end

	first_interval: like interval is
		do
			Result := intervals.first
		end

	first_item: G is
		do
			if intervals.first /= Void then
				Result := intervals.first.item
			end
		end

	last_interval: like interval is
		do
			Result := intervals.last
		end

	last_item: G is
		do
			if intervals.last /= Void then
				Result := intervals.last.item
			end
		end

	current_interval: like interval is
			-- <Precursor>
		do
			Result := intervals.item
		end

	current_item: G is
			-- Returns the G part of the current interval.
		do
			Result := intervals.item.item
		end

	occurrences (v: like current_interval): INTEGER is
			-- <Precursor>
		do
			from
				start
				search (v)
			until
				exhausted
			loop
				Result := Result + 1
				forth
				search (v)
			end
		end

	interval (a_date: ROSE_DATE): like interval_template is
			-- The first interval bracketing 'a_date'
			-- There may be multiple witneses but we are only concerned with the existence of any witness (and the first one at that)
		do
			from
				intervals.start
			until
				intervals.off or else Result /= Void
			loop
				if intervals.item.brackets (a_date) then
					Result := intervals.item
				end
				intervals.forth
			end
		end

	intersection (a_other: like Current): like Current is
			-- The overlapping intervals of 'Current' and 'a_other'
		require
			valid_other: a_other /= Void
		local
			l_other, l_new: like current_interval
			l_start, l_end: ROSE_DATE
		do
			create Result.make
			from
				start
				a_other.start
			until
				off
			loop
				from
				until
					a_other.off or else current_interval.end_date < a_other.current_interval.start_date
				loop
					l_other := a_other.current_interval
					l_start := l_other.start_date.max (current_interval.start_date)
					l_end := current_interval.end_date.min (l_other.end_date)
					l_new := l_other.twin
					l_new.set_start_date (l_start)
					l_new.set_end_date (l_end)
					if l_start <= l_end then
						Result.insert (l_new)
					end
					a_other.forth
				end
				forth
			end
		end

	cursor: CURSOR is
		do
			Result := intervals.cursor
		end

	search (v: like interval_template)
			-- The descendant will redefine this but will not execute the post-conditions in 'lp_search'.
		do
			lp_search(v)
		end

feature -- Basic operations

	insert (a_interval: like interval) is
			-- Insert 'a_interval' into 'Current' maintaining order by 'start_date'
		require
			valid_interval: a_interval /= Void
		do
			from
				intervals.start
			until
				intervals.off or else a_interval.start_date <= intervals.item.start_date
			loop
				intervals.forth
			end
			if intervals.off then
				intervals.extend (a_interval)
			else
				intervals.put_left (a_interval)
			end
		end

	append (a_other: like Current) is
		require
			valid_other: a_other /= Void
		do
			from
				a_other.start
			until
				a_other.off
			loop
				insert (a_other.current_interval.twin)
				a_other.forth
			end
		end

	go_to (a_cursor: like cursor) is
		do
			intervals.go_to (a_cursor)
		end

feature {NONE} -- Implementation

	interval_template: ROSE_INTERVAL [G] is
		do
		end

	intervals: TWO_WAY_LIST [like interval]			-- Storage for out actual data.

invariant

	valid_intervals: intervals /= Void

end
