indexing
	description: "Objects that model a disjoint sequence of intervals ordered by date"
	author: "Mark Howard"
	date: "$Date: 2011/01/18 $"
	last_editor: "$Author: rj $"
	revision: "$Revision: #15 $"
	archive: "$File: //rose/source/kernel/infrastructure/structures/rose_interval_partition.e $"

class
	ROSE_INTERVAL_PARTITION[G]

inherit
	ROSE_INTERVAL_LIST[G]
		redefine
			make,
			insert,
			interval_template,
			search,
			intersection
		end
	ROSE_CONSTANTS

create
	make

feature {NONE} -- Creation

	make is
			-- create, makes a single interval with datespan 1900/01/01 to 9999/12/31
		do
			initialize
		end

feature -- Element modification

	extend (a_item: G; a_date: ROSE_DATE) is
			-- Add an item/date by splitting an existing interval.  The new interval is ALWAYS placed at
			-- the end of the list.
		local
			l_new_interval,l_interval : like interval
		do
			l_interval := interval(a_date)
			if ((a_item = Void or l_interval.item = Void) and a_item /= l_interval.item) or else
			   (a_item /= Void and then l_interval.item /= Void and then not a_item.is_equal(l_interval.item))
			then
				l_new_interval := l_interval.twin
				l_new_interval.make(a_item,a_date,l_interval.end_date, 0.0)
				l_interval.make(l_interval.item,l_interval.start_date,a_date, 0.0)
				intervals.extend(l_new_interval)
			end
		end

	insert (a_interval: like interval) is
			-- You probably should be using 'extend'. This definition is as close as I can get to a proper implementation
			-- of 'insert'.
		do
			extend(a_interval.item, a_interval.first_date)
		end

feature -- Basic operations

	initialize is
			-- Clear out all data
		local
			l_void_g: G
			l_h : like interval
		do
			create intervals.make
			create l_h.make(l_void_g,constants.minimum_date_time.date,constants.maximum_date_time.date, 0.0)
			intervals.extend(l_h)
		ensure
			is_trivial: is_trivial
		end

feature -- access

	is_trivial: BOOLEAN is
			-- Returns true if current partition contains only the default interval spannning min date to max date
		do
			if intervals.count = 1 and then intervals.first.start_date.is_equal (constants.minimum_date_time.date)
			and then intervals.first.end_date.is_equal (constants.maximum_date_time.date)  then
				Result := True
			end
		end

	search(a_interval: like interval_template)
			-- This version only searches for the first interval with the same 'first_date' and same 'item'.
		do
			from
				-- We start from where ever the caller left us.
			until
				intervals.off or else
				(intervals.item.first_date ~ a_interval.first_date and intervals.item.item ~ a_interval.item)
			loop
				intervals.forth
			end
		end

	intersection (a_other: like Current): like Current is
			-- This feature requires that the list of intervals is sorted - which doesn't happen here.
		local
			l_exceptions: EXCEPTIONS
		do
			create l_exceptions
			l_exceptions.raise("Unsupported feature")
		end

feature {NONE} --- implementation

	interval_template : ROSE_CLOSED_OPEN_INTERVAL[G]

end
