indexing
	description: "Input/Output manager"
	author: "Zoran Simic"
	date: "$Date: 2008/12/01 $"
	revision: "$Revision: #11 $"

class
	ROSE_IO_MANAGER

inherit
	ROSE_SHARED_GLOBAL
	ANY

create
	make

feature {NONE} -- Initialization

	make is
			-- Create a default IO manager.
		do
			create input_channels.make ("input","file")
			create output_channels.make ("output","file")
				--| Default channels
			add_input (create {ROSE_INCHANNEL_FILE})
			add_input (create {ROSE_INCHANNEL_STANDARD}.make)
			add_output (create {ROSE_OUTCHANNEL_FILE}.make)
			add_output (create {ROSE_OUTCHANNEL_STANDARD}.make)
			add_output (create {ROSE_OUTCHANNEL_NULL})
			add_output (create {ROSE_OUTCHANNEL_BROADCAST}.make_empty)
		end

feature -- Access

	input_channels: ROSE_SERVICE_MANAGER [ROSE_INCHANNEL]
			-- Available input channel types

	output_channels: ROSE_SERVICE_MANAGER [ROSE_OUTCHANNEL]
			-- Available output channel types

	input_channel (a_path: STRING): ROSE_INCHANNEL is
			-- Input channel pointed by 'a_path', Void if no valid channel pointed by 'a_path'
		do
			if equal (a_path,Global.convention.current_IO_representation) then
				Result := Global.io_handler.input
			else
				Result := input_channels.open_service (Global.interpreted (a_path))
			end
			if Result /= Void and then not Result.is_open then
				Result := Void
			end
		ensure
			open_if_valid: Result /= Void implies Result.is_open
		end

	checked_input_channel (a_requirer, a_path: STRING): ROSE_INCHANNEL is
			-- Input channel pointed by 'a_path'
			-- 'a_requirer' indicates who asks for this channel, used for error signaling only
		do
			Result := input_channel (a_path)
			if Result = Void then
				error.signal (a_requirer,"No input channel '$1' available",<<a_path>>)
			end
		ensure
			open_if_valid: Result /= Void implies Result.is_open
			error_signaled: Result = Void implies error.signaled
		end

	output_channel (a_path: STRING): ROSE_OUTCHANNEL is
			-- Output channel pointed by 'a_path', Void if no valid channel pointed by 'a_path'
		do
			if equal (a_path,Global.convention.current_IO_representation) then
				Result := Global.io_handler.output
			else
				Result := output_channels.open_service (Global.interpreted (a_path))
			end
			if Result /= Void and then not Result.is_open then
				Result := Void
			end
		ensure
			open_if_valid: Result /= Void implies Result.is_open
		end

	checked_output_channel (a_requirer, a_path: STRING): ROSE_OUTCHANNEL is
			-- Output channel pointed by 'a_path'
			-- 'a_requirer' indicates who asks for this channel, used for error signaling only
		do
			Result := output_channel (a_path)
			if Result = Void then
				error.signal (a_requirer,"No output channel '$1' available",<<a_path>>)
			end
		ensure
			open_if_valid: Result /= Void implies Result.is_open
			error_signaled: Result = Void implies error.signaled
		end

feature -- Basic operations

	add_input (a_input: ROSE_INCHANNEL) is
			-- Add 'a_input' channel to this manager.
		do
			input_channels.add (a_input)
		end

	add_output (a_output: ROSE_OUTCHANNEL) is
			-- Add 'a_output' channel to this manager.
		do
			output_channels.add (a_output)
		end

end -- class ROSE_IO_MANAGER
