note

	description:

		"Factory to deliver ROSE_LAG_INTERVAL objects that are cached by type and lookback"

	author: "Nelson Chui"
	last_editor: "$Author: mb $"
	date: "$Date: 2011/09/23 $"
	revision: "$Revision: #2 $"
	path: "$File: //rose/source/kernel/targets/rose_lag_interval_factory.e $"

class ROSE_LAG_INTERVAL_FACTORY
	
inherit
	
	ROSE_PERIODICITY_ID_USE
	
feature -- Access
	
	daily_interval (a_lag: INTEGER): ROSE_LAG_INTERVAL is
			-- A daily periodicity lag interval with `a_lag'
		require
			valid_lag: a_lag >= 0
		do
			Result := daily_interval_cache.safe_item (a_lag)
			if Result = Void then
				create Result.make (a_lag, periodicity_ids.daily_periodicity_id)
				daily_interval_cache.force (Result, a_lag)
			end
		ensure
			valid_result: Result /= Void
		end

	monthly_interval (a_lag: INTEGER): ROSE_LAG_INTERVAL is
			-- A monthly periodicity lag interval with `a_lag'
		require
			valid_lag: a_lag >= 0
		do
			Result := monthly_interval_cache.safe_item (a_lag)
			if Result = Void then
				create Result.make (a_lag, periodicity_ids.monthly_periodicity_id)
				monthly_interval_cache.force (Result, a_lag)
			end
		ensure
			valid_result: Result /= Void
		end

	annual_interval (a_lag: INTEGER): ROSE_LAG_INTERVAL is
			-- A annual periodicity lag interval with `a_lag'
		require
			valid_lag: a_lag >= 0
		do
			Result := annual_interval_cache.safe_item (a_lag)
			if Result = Void then
				create Result.make (a_lag, periodicity_ids.annual_periodicity_id)
				annual_interval_cache.force (Result, a_lag)
			end
		ensure
			valid_result: Result /= Void
		end

feature {NONE} -- Implementation

	daily_interval_cache: ROSE_LINEAR_ARRAY [ROSE_LAG_INTERVAL] is
			-- Cache of daily periodicity lags
		once
			create Result.make_empty
		ensure
			valid_result: Result /= Void
		end
	
	monthly_interval_cache: ROSE_LINEAR_ARRAY [ROSE_LAG_INTERVAL] is
			-- Cache of monthly periodicity lags
		once
			create Result.make_empty
		ensure
			valid_result: Result /= Void
		end
	
	annual_interval_cache: ROSE_LINEAR_ARRAY [ROSE_LAG_INTERVAL] is
			-- Cache of annual periodicity lags
		once
			create Result.make_empty
		ensure
			valid_result: Result /= Void
		end
	
end
