note

	description:

		"Operators that yield the value of an expression lagged by one month"

	author: "$Author: mk $ Colin Adams"
	last_editor: "$Author: mk $"
	date: "$Date: 2011/02/15 $"
	revision: "$Revision: #16 $"
	path: "$File: //rose/source/kernel/math/rose_lag_month_operator.e $"

class ROSE_LAG_MONTH_OPERATOR [reference G]

inherit

	ROSE_UNARY_OPERATOR [G]
		redefine
			units
		end
	ROSE_PERIODICITY_ID_USE
		export {NONE} all end

	ROSE_SHARED_GLOBAL
		export {NONE} all end

create

	make

feature -- Access

	real_value (a_expression: ROSE_EXPRESSION [G]; a_g: G): REAL is
			-- Operation result as a REAL
		do
			fetch_operator (a_expression)
			if fetched_operator /= Void then
				Result := fetched_operator.real_value (a_expression, month_interval, a_g)
			end
		end

	double_value (a_expression: ROSE_EXPRESSION [G]; a_g: G): DOUBLE is
			-- Operation result as a DOUBLE
		do
			fetch_operator (a_expression)
			if fetched_operator /= Void then
				Result := fetched_operator.double_value (a_expression, month_interval, a_g)
			end
		end

	integer_value (a_expression: ROSE_EXPRESSION [G]; a_g: G): INTEGER is
			-- Operation result as an INTEGER
		do
			fetch_operator (a_expression)
			if fetched_operator /= Void then
				Result := fetched_operator.integer_value (a_expression, month_interval, a_g)
			end
		end

	boolean_value (a_expression: ROSE_EXPRESSION [G]; a_g: G): BOOLEAN is
			-- Operation result as a BOOLEAN
		do
			fetch_operator (a_expression)
			if fetched_operator /= Void then
				Result := fetched_operator.boolean_value (a_expression, month_interval, a_g)
			end
		end


	string_value (a_expression: ROSE_EXPRESSION [G]; a_g: G): STRING is
			-- Operation result as a STRING
		do
			fetch_operator (a_expression)
			if fetched_operator /= Void and month_interval /= Void then
				Result := fetched_operator.string_value (a_expression, month_interval, a_g)
			end
		end

	value_in_market_currency (a_expression: ROSE_EXPRESSION [G]; a_market: MARKET; a_currency: PERSISTENT_CURRENCY;
									  a_g: G; a_date: ROSE_DATE): DOUBLE is
			-- Value in currency of `a_market'
		do
			fetch_operator (a_expression)
			if fetched_operator /= Void then
				Result := fetched_operator.value_in_market_currency (a_expression, month_interval, a_market, a_currency, a_g, a_date)
			end
		end
	computed_units(a_units : ROSE_UNITS) : ROSE_UNITS
			--computed units
		do
			Result := a_units
		end
	units (a_expression: ROSE_EXPRESSION [G]; a_g: G): ROSE_UNITS is
			-- Units in which ??? (Mark?)
		do
			fetch_operator (a_expression)
			if fetched_operator /= Void then
				Result := fetched_operator.units (a_expression, month_interval, a_g)
			end
		end

	units_in_currency (a_expression: ROSE_EXPRESSION [G]; a_currency: PERSISTENT_CURRENCY; a_g: G): ROSE_UNITS is
			-- units expression in a_currency
		do
			fetch_operator (a_expression)
			if fetched_operator /= Void then
				Result := fetched_operator.units_in_currency (a_expression, month_interval, a_currency, a_g)
			end
		end

	computed_double (a_expression: ROSE_EXPRESSION [G]; a_g: G; a_agent: like access_agent_template): DOUBLE is
			-- Value computed via `a_agent' as a DOUBLE
		do
			fetch_operator (a_expression)
			if fetched_operator /= Void then
				Result := fetched_operator.computed_double (a_expression, month_interval, a_g, a_agent)
			end
		end

	computed_real (a_expression: ROSE_EXPRESSION [G]; a_g: G; a_agent: like access_agent_template): REAL is
			-- Value computed via `a_agent' as a REAL
		do
			fetch_operator (a_expression)
			if fetched_operator /= Void then
				Result := fetched_operator.computed_real (a_expression, month_interval, a_g, a_agent)
			end
		end

	computed_integer (a_expression: ROSE_EXPRESSION [G]; a_g: G; a_agent: like access_agent_template): INTEGER is
			-- Value computed via `a_agent' as an INTEGER
		do
			fetch_operator (a_expression)
			if fetched_operator /= Void then
				Result := fetched_operator.computed_integer (a_expression, month_interval, a_g, a_agent)
			end
		end

	computed_string (a_expression: ROSE_EXPRESSION [G]; a_g: G; a_agent: like access_agent_template): STRING is
			-- Value computed via `a_agent' as a STRING
		do
			fetch_operator (a_expression)
			if fetched_operator /= Void then
				Result := fetched_operator.computed_string (a_expression, month_interval, a_g, a_agent)
			end
		end

	computed_boolean (a_expression: ROSE_EXPRESSION [G]; a_g: G; a_agent: like access_agent_template): BOOLEAN is
			-- Value computed via `a_agent' as a BOOLEAN
		do
			fetch_operator (a_expression)
			if fetched_operator /= Void then
				Result := fetched_operator.computed_boolean (a_expression, month_interval, a_g, a_agent)
			end
		end


	generated_double (a_expression: ROSE_EXPRESSION [G]; a_g: G; a_agent: like access_agent_template): DOUBLE is
			-- Generated value as a DOUBLE
		do
			fetch_operator (a_expression)
			if fetched_operator /= Void then
				Result := fetched_operator.generated_double (a_expression, month_interval, a_g, a_agent)
			end
		end

	generated_real (a_expression: ROSE_EXPRESSION [G]; a_g: G; a_agent: like access_agent_template): REAL is
			-- Generated value as a REAL
		do
			fetch_operator (a_expression)
			if fetched_operator /= Void then
				Result := fetched_operator.generated_real (a_expression, month_interval, a_g, a_agent)
			end
		end

	generated_integer (a_expression: ROSE_EXPRESSION [G]; a_g: G; a_agent: like access_agent_template): INTEGER is
			-- Generated value as an INTEGER
		do
			fetch_operator (a_expression)
			if fetched_operator /= Void then
				Result := fetched_operator.generated_integer (a_expression, month_interval, a_g, a_agent)
			end
		end

	generated_string (a_expression: ROSE_EXPRESSION [G]; a_g: G; a_agent: like access_agent_template): STRING is
			-- Generated value as a STRING
		do
			fetch_operator (a_expression)
			if fetched_operator /= Void then
				Result := fetched_operator.generated_string (a_expression, month_interval, a_g, a_agent)
			end
		end

	generated_date (a_expression: ROSE_EXPRESSION [G]; a_g: G; a_agent: like access_agent_template): ROSE_DATE is
			-- Generated value as a ROSE_DATE
		do
			fetch_operator (a_expression)
			if fetched_operator /= Void then
				Result := fetched_operator.generated_date (a_expression, month_interval, a_g, a_agent)
			end
		end

feature {NONE} -- Implementation

	Lag_months: STRING is "lag_months"
			-- Name of proxied binary operator

	fetched_operator: ROSE_BINARY_OPERATOR [G]
			-- Operator fetched by `fetch_operator' on sucess

	month_interval: CONSTANT_EXPRESSION [G]
			-- One month interval;

	fetch_operator (a_expression: ROSE_EXPRESSION [G]) is
			-- Fetched value of `Lag_months' operator
		do
			fetched_operator := Void
			create month_interval.make_integer (1)
			if a_expression /= Void then
				fetched_operator ?=  a_expression.binary_operator (Lag_months)
			else
				error.warn ("Cannot find lag_months function for $1", <<a_expression>>)
			end
		ensure
			month_interval_not_void: fetched_operator /= Void implies month_interval /= Void
		end

end
