note

	description:

		"Operators that yield the value of an expression lagged by N months"

	author: "$Author: er $ Colin Adams"
	last_editor: "$Author: er $"
	date: "$Date: 2014/02/26 $"
	revision: "$Revision: #18 $"
	path: "$File: //rose/source/kernel/math/rose_lag_months_operator.e $"

class ROSE_LAG_MONTHS_OPERATOR [reference G]

inherit

	ROSE_BINARY_OPERATOR [G]
		redefine
			units
		end
	ROSE_PERIODICITY_ID_USE
		export {NONE} all end

create

	make

feature -- Access

	real_value (a_left, a_months: ROSE_EXPRESSION [G]; a_g: G): REAL is
			-- Operation result as a REAL
		local
			l_months: CONSTANT_EXPRESSION [G]
			l_count: INTEGER
			l_interval: ROSE_LAG_INTERVAL
			l_left: ROSE_EXPRESSION [like lag_target]
			l_g: like lag_target
		do
			l_months ?= a_months
			l_left ?= a_left
			l_g ?= a_g
			if l_months /= Void and l_g /= Void and l_left /= Void then
				l_count := l_months.integer_constant_value
				if l_count /= 0 then
					create l_interval.make (l_count, periodicity_ids.monthly_periodicity_id)
				end
				create lag_operator.make (l_interval)
				Result := lag_operator.real_value (l_left, l_g)
			end
		end

	double_value (a_left, a_months: ROSE_EXPRESSION [G]; a_g: G): DOUBLE is
			-- Operation result as a DOUBLE
		local
			l_months: CONSTANT_EXPRESSION [G]
			l_count: INTEGER
			l_interval: ROSE_LAG_INTERVAL
			l_left: ROSE_EXPRESSION [like lag_target]
			l_g: like lag_target
		do
			l_months ?= a_months
			l_left ?= a_left
			l_g ?= a_g
			if l_months /= Void and l_g /= Void and l_left /= Void then
				l_count := l_months.integer_constant_value
				if l_count /= 0 then
					create l_interval.make (l_count, periodicity_ids.monthly_periodicity_id)
				end
				create lag_operator.make (l_interval)
				Result := lag_operator.double_value (l_left, l_g)
			end
		end

	unitized_value(a_left,a_right : ROSE_UNITIZED_VALUE[DOUBLE]) : ROSE_UNITIZED_VALUE[DOUBLE]
			--unitized value
		do
--			l_months ?= a_months
--			l_left ?= a_left
--			l_g ?= a_g
--			if l_months /= Void and l_g /= Void and l_left /= Void then
--				l_count := l_months.integer_constant_value
--				if l_count /= 0 then
--					create l_interval.make (l_count, periodicity_ids.monthly_periodicity_id)
--				end
--				create lag_operator.make (l_interval)
--				create Result.make(lag_operator.double_value (l_left, l_g),pure_units)
--				--fixme
--			end
		end
	decimal_value (a_left, a_months: ROSE_EXPRESSION [G]; a_g: G): RP_DECIMAL is
			-- Operation result as a DOUBLE
		local
			l_months: CONSTANT_EXPRESSION [G]
			l_count: INTEGER
			l_interval: ROSE_LAG_INTERVAL
			l_left: ROSE_EXPRESSION [like lag_target]
			l_g: like lag_target
		do
			l_months ?= a_months
			l_left ?= a_left
			l_g ?= a_g
			if l_months /= Void and l_g /= Void and l_left /= Void then
				l_count := l_months.integer_constant_value
				if l_count /= 0 then
					create l_interval.make (l_count, periodicity_ids.monthly_periodicity_id)
				end
				create lag_operator.make (l_interval)
				Result := lag_operator.decimal_value (l_left, l_g)
			end
		end

	integer_value (a_left, a_months: ROSE_EXPRESSION [G]; a_g: G): INTEGER is
			-- Operation result as an INTEGER
		local
			l_months: CONSTANT_EXPRESSION [G]
			l_count: INTEGER
			l_interval: ROSE_LAG_INTERVAL
			l_left: ROSE_EXPRESSION [like lag_target]
			l_g: like lag_target
		do
			l_months ?= a_months
			l_left ?= a_left
			l_g ?= a_g
			if l_months /= Void and l_g /= Void and l_left /= Void then
				l_count := l_months.integer_constant_value
				if l_count /= 0 then
					create l_interval.make (l_count, periodicity_ids.monthly_periodicity_id)
					create lag_operator.make (l_interval)
					Result := lag_operator.integer_value (l_left, l_g)
				end
			end
		end

	boolean_value (a_left, a_months: ROSE_EXPRESSION [G]; a_g: G): BOOLEAN is
			-- Operation result as a BOOLEAN
		local
			l_months: CONSTANT_EXPRESSION [G]
			l_count: INTEGER
			l_interval: ROSE_LAG_INTERVAL
			l_left: ROSE_EXPRESSION [like lag_target]
			l_g: like lag_target
		do
			l_months ?= a_months
			l_left ?= a_left
			l_g ?= a_g
			if l_months /= Void and l_g /= Void and l_left /= Void then
				l_count := l_months.integer_constant_value
				if l_count /= 0 then
					create l_interval.make (l_count, periodicity_ids.monthly_periodicity_id)
					create lag_operator.make (l_interval)
					Result := lag_operator.boolean_value (l_left, l_g)
				end
			end
		end


	string_value (a_left, a_months: ROSE_EXPRESSION [G]; a_g: G): STRING is
			-- Operation result as a STRING
		local
			l_months: CONSTANT_EXPRESSION [G]
			l_count: INTEGER
			l_left: ROSE_EXPRESSION [like lag_target]
			l_interval: ROSE_LAG_INTERVAL
			l_g: like lag_target
		do
			l_months ?= a_months
			l_left ?= a_left
			l_g ?= a_g
			if l_months /= Void and l_g /= Void and l_left /= Void then
				l_count := l_months.integer_constant_value
				if l_count /= 0 then
					create l_interval.make (l_count, periodicity_ids.monthly_periodicity_id)
					create lag_operator.make (l_interval)
					Result := lag_operator.string_value (l_left, l_g)
				end
			end
		end

	date_value (a_left, a_months: ROSE_EXPRESSION [G]; a_g: G): ROSE_DATE is
			-- Operation result as a ROSE_DATE
		local
			l_months: CONSTANT_EXPRESSION [G]
			l_count: INTEGER
			l_interval: ROSE_LAG_INTERVAL
			l_left: ROSE_EXPRESSION [like lag_target]
			l_g: like lag_target
		do
			l_months ?= a_months
			l_left ?= a_left
			l_g ?= a_g
			if l_months /= Void and l_g /= Void and l_left /= Void then
				l_count := l_months.integer_constant_value
				if l_count /= 0 then
					create l_interval.make (l_count, periodicity_ids.monthly_periodicity_id)
					create lag_operator.make (l_interval)
					Result := lag_operator.date_value (l_left, l_g)
				end
			end
		end


	value_in_market_currency (a_left, a_months: ROSE_EXPRESSION [G]; a_market: MARKET; a_currency: PERSISTENT_CURRENCY;
									  a_g: G; a_date: ROSE_DATE): DOUBLE is
			-- Value in currency of `a_market'
		local
			l_months: CONSTANT_EXPRESSION [G]
			l_count: INTEGER
			l_interval: ROSE_LAG_INTERVAL
			l_left: ROSE_EXPRESSION [like lag_target]
			l_g: like lag_target
		do
			l_months ?= a_months
			l_left ?= a_left
			l_g ?= a_g
			if l_months /= Void and l_g /= Void and l_left /= Void then
				l_count := l_months.integer_constant_value
				if l_count /= 0 then
					create l_interval.make (l_count, periodicity_ids.monthly_periodicity_id)
					create lag_operator.make (l_interval)
					Result := lag_operator.value_in_market_currency (l_left, a_market, a_currency, l_g, a_date)
				end
			end
		end
	computed_units(a_left,a_right : ROSE_UNITS) : ROSE_UNITS
			--units as a function of the units of left and right
		do
			Result := lag_operator.computed_units(a_left)
		end
	units (a_left, a_months: ROSE_EXPRESSION [G]; a_g: G): ROSE_UNITS is
			-- Units in which ??? (Mark?)
		local
			l_months: CONSTANT_EXPRESSION [G]
			l_count: INTEGER
			l_interval: ROSE_LAG_INTERVAL
			l_left: ROSE_EXPRESSION [like lag_target]
			l_g: like lag_target
		do
			l_months ?= a_months
			l_left ?= a_left
			l_g ?= a_g
			if l_months /= Void and l_g /= Void and l_left /= Void then
				l_count := l_months.integer_constant_value
				if l_count /= 0 then
					create l_interval.make (l_count, periodicity_ids.monthly_periodicity_id)
					create lag_operator.make (l_interval)
					Result := lag_operator.units (l_left, l_g)
				end
			end
		end

	nominal_units (a_left, a_right: ROSE_UNITS): ROSE_UNITS is
			--default units for this operator
		do
		end

	units_in_currency (a_left, a_months: ROSE_EXPRESSION [G]; a_currency: PERSISTENT_CURRENCY; a_g: G): ROSE_UNITS is
			-- units represented in a_currency
		local
			l_months: CONSTANT_EXPRESSION [G]
			l_count: INTEGER
			l_interval: ROSE_LAG_INTERVAL
			l_left: ROSE_EXPRESSION [like lag_target]
			l_g: like lag_target
		do
			l_months ?= a_months
			l_left ?= a_left
			l_g ?= a_g
			if l_months /= Void and l_g /= Void and l_left /= Void then
				l_count := l_months.integer_constant_value
				if l_count /= 0 then
					create l_interval.make (l_count, periodicity_ids.monthly_periodicity_id)
					create lag_operator.make (l_interval)
					Result := lag_operator.units_in_currency (l_left, a_currency, l_g)
				end
			end
		end

	computed_double (a_left, a_months: ROSE_EXPRESSION [G]; a_g: G; a_agent: like access_agent_template): DOUBLE is
			-- Value computed via `a_agent' as a DOUBLE
		local
			l_months: CONSTANT_EXPRESSION [G]
			l_count: INTEGER
			l_interval: ROSE_LAG_INTERVAL
			l_left: ROSE_EXPRESSION [like lag_target]
			l_g: like lag_target
			l_agent: ACCESS_CONTROL_AGENT
		do
			l_months ?= a_months
			l_left ?= a_left
			l_agent ?= a_agent
			l_g ?= a_g
			if l_months /= Void and l_g /= Void and l_left /= Void and l_agent /= Void then
				l_count := l_months.integer_constant_value
				if l_count /= 0 then
					create l_interval.make (l_count, periodicity_ids.monthly_periodicity_id)
					create lag_operator.make (l_interval)
					Result := lag_operator.computed_double (l_left, l_g, l_agent)
				end
			end
		end

	computed_real (a_left, a_months: ROSE_EXPRESSION [G]; a_g: G; a_agent: like access_agent_template): REAL is
			-- Value computed via `a_agent' as a REAL
		local
			l_months: CONSTANT_EXPRESSION [G]
			l_count: INTEGER
			l_interval: ROSE_LAG_INTERVAL
			l_left: ROSE_EXPRESSION [like lag_target]
			l_g: like lag_target
			l_agent: ACCESS_CONTROL_AGENT
		do
			l_months ?= a_months
			l_left ?= a_left
			l_agent ?= a_agent
			l_g ?= a_g
			if l_months /= Void and l_g /= Void and l_left /= Void and l_agent /= Void then
				l_count := l_months.integer_constant_value
				if l_count /= 0 then
					create l_interval.make (l_count, periodicity_ids.monthly_periodicity_id)
					create lag_operator.make (l_interval)
					Result := lag_operator.computed_real (l_left, l_g, l_agent)
				end
			end
		end

	computed_integer (a_left, a_months: ROSE_EXPRESSION [G]; a_g: G; a_agent: like access_agent_template): INTEGER is
			-- Value computed via `a_agent' as an INTEGER
		local
			l_months: CONSTANT_EXPRESSION [G]
			l_count: INTEGER
			l_interval: ROSE_LAG_INTERVAL
			l_left: ROSE_EXPRESSION [like lag_target]
			l_g: like lag_target
			l_agent: ACCESS_CONTROL_AGENT
		do
			l_months ?= a_months
			l_left ?= a_left
			l_agent ?= a_agent
			l_g ?= a_g
			if l_months /= Void and l_g /= Void and l_left /= Void and l_agent /= Void then
				l_count := l_months.integer_constant_value
				if l_count /= 0 then
					create l_interval.make (l_count, periodicity_ids.monthly_periodicity_id)
					create lag_operator.make (l_interval)
					Result := lag_operator.computed_integer (l_left, l_g, l_agent)
				end
			end
		end

	computed_string (a_left, a_months: ROSE_EXPRESSION [G]; a_g: G; a_agent: like access_agent_template): STRING is
			-- Value computed via `a_agent' as a STRING
		local
			l_months: CONSTANT_EXPRESSION [G]
			l_count: INTEGER
			l_interval: ROSE_LAG_INTERVAL
			l_left: ROSE_EXPRESSION [like lag_target]
			l_g: like lag_target
			l_agent: ACCESS_CONTROL_AGENT
		do
			l_months ?= a_months
			l_left ?= a_left
			l_agent ?= a_agent
			l_g ?= a_g
			if l_months /= Void and l_g /= Void and l_left /= Void and l_agent /= Void then
				l_count := l_months.integer_constant_value
				if l_count /= 0 then
					create l_interval.make (l_count, periodicity_ids.monthly_periodicity_id)
					create lag_operator.make (l_interval)
					Result := lag_operator.computed_string (l_left, l_g, l_agent)
				end
			end
		end

	computed_boolean (a_left, a_months: ROSE_EXPRESSION [G]; a_g: G; a_agent: like access_agent_template): BOOLEAN is
			-- Value computed via `a_agent' as a BOOLEAN
		local
			l_months: CONSTANT_EXPRESSION [G]
			l_count: INTEGER
			l_left: ROSE_EXPRESSION [like lag_target]
			l_interval: ROSE_LAG_INTERVAL
			l_g: like lag_target
			l_agent: ACCESS_CONTROL_AGENT
		do
			l_months ?= a_months
			l_agent ?= a_agent
			l_left ?= a_left
			l_g ?= a_g
			if l_months /= Void and l_g /= Void and l_left /= Void and l_agent /= Void then
				l_count := l_months.integer_constant_value
				if l_count /= 0 then
					create l_interval.make (l_count, periodicity_ids.monthly_periodicity_id)
					create lag_operator.make (l_interval)
					Result := lag_operator.computed_boolean (l_left, l_g, l_agent)
				end
			end
		end

	computed_date (a_left, a_months: ROSE_EXPRESSION [G]; a_g: G; a_agent: like access_agent_template): ROSE_DATE is
			-- Value computed via `a_agent' as a ROSE_DATE
		local
			l_months: CONSTANT_EXPRESSION [G]
			l_count: INTEGER
			l_left: ROSE_EXPRESSION [like lag_target]
			l_interval: ROSE_LAG_INTERVAL
			l_g: like lag_target
			l_agent: ACCESS_CONTROL_AGENT
		do
			l_months ?= a_months
			l_agent ?= a_agent
			l_left ?= a_left
			l_g ?= a_g
			if l_months /= Void and l_g /= Void and l_left /= Void and l_agent /= Void then
				l_count := l_months.integer_constant_value
				if l_count /= 0 then
					create l_interval.make (l_count, periodicity_ids.monthly_periodicity_id)
					create lag_operator.make (l_interval)
					Result := lag_operator.computed_date (l_left, l_g, l_agent)
				end
			end
		end


	generated_double (a_left, a_months: ROSE_EXPRESSION [G]; a_g: G; a_agent: like access_agent_template): DOUBLE is
			-- Generated value as a DOUBLE
		local
			l_months: CONSTANT_EXPRESSION [G]
			l_count: INTEGER
			l_interval: ROSE_LAG_INTERVAL
			l_left: ROSE_EXPRESSION [like lag_target]
			l_g: like lag_target
			l_agent: ACCESS_CONTROL_AGENT
		do
			l_months ?= a_months
			l_left ?= a_left
			l_agent ?= a_agent
			l_g ?= a_g
			if l_months /= Void and l_g /= Void and l_left /= Void and l_agent /= Void then
				l_count := l_months.integer_constant_value
				if l_count /= 0 then
					create l_interval.make (l_count, periodicity_ids.monthly_periodicity_id)
					create lag_operator.make (l_interval)
					Result := lag_operator.generated_double (l_left, l_g, l_agent)
				end
			end
		end

	generated_real (a_left, a_months: ROSE_EXPRESSION [G]; a_g: G; a_agent: like access_agent_template): REAL is
			-- Generated value as a REAL
		local
			l_months: CONSTANT_EXPRESSION [G]
			l_count: INTEGER
			l_interval: ROSE_LAG_INTERVAL
			l_left: ROSE_EXPRESSION [like lag_target]
			l_g: like lag_target
			l_agent: ACCESS_CONTROL_AGENT
		do
			l_months ?= a_months
			l_left ?= a_left
			l_agent ?= a_agent
			l_g ?= a_g
			if l_months /= Void and l_g /= Void and l_left /= Void and l_agent /= Void then
				l_count := l_months.integer_constant_value
				if l_count /= 0 then
					create l_interval.make (l_count, periodicity_ids.monthly_periodicity_id)
					create lag_operator.make (l_interval)
					Result := lag_operator.generated_real (l_left, l_g, l_agent)
				end
			end
		end

	generated_integer (a_left, a_months: ROSE_EXPRESSION [G]; a_g: G; a_agent: like access_agent_template): INTEGER is
			-- Generated value as an INTEGER
		local
			l_months: CONSTANT_EXPRESSION [G]
			l_count: INTEGER
			l_interval: ROSE_LAG_INTERVAL
			l_left: ROSE_EXPRESSION [like lag_target]
			l_g: like lag_target
			l_agent: ACCESS_CONTROL_AGENT
		do
			l_months ?= a_months
			l_left ?= a_left
			l_agent ?= a_agent
			l_g ?= a_g
			if l_months /= Void and l_g /= Void and l_left /= Void and l_agent /= Void then
				l_count := l_months.integer_constant_value
				if l_count /= 0 then
					create l_interval.make (l_count, periodicity_ids.monthly_periodicity_id)
					create lag_operator.make (l_interval)
					Result := lag_operator.generated_integer (l_left, l_g, l_agent)
				end
			end
		end

	generated_string (a_left, a_months: ROSE_EXPRESSION [G]; a_g: G; a_agent: like access_agent_template): STRING is
			-- Generated value as a STRING
		local
			l_months: CONSTANT_EXPRESSION [G]
			l_count: INTEGER
			l_interval: ROSE_LAG_INTERVAL
			l_left: ROSE_EXPRESSION [like lag_target]
			l_g: like lag_target
			l_agent: ACCESS_CONTROL_AGENT
		do
			l_months ?= a_months
			l_left ?= a_left
			l_agent ?= a_agent
			l_g ?= a_g
			if l_months /= Void and l_g /= Void and l_left /= Void and l_agent /= Void then
				l_count := l_months.integer_constant_value
				if l_count /= 0 then
					create l_interval.make (l_count, periodicity_ids.monthly_periodicity_id)
					create lag_operator.make (l_interval)
					Result := lag_operator.generated_string (l_left, l_g, l_agent)
				end
			end
		end


	generated_date (a_left, a_months: ROSE_EXPRESSION [G]; a_g: G; a_agent: like access_agent_template): ROSE_DATE is
			-- Generated value as a ROSE_DATE
		local
			l_months: CONSTANT_EXPRESSION [G]
			l_count: INTEGER
			l_left: ROSE_EXPRESSION [like lag_target]
			l_interval: ROSE_LAG_INTERVAL
			l_g: like lag_target
			l_agent: ACCESS_CONTROL_AGENT
		do
			l_months ?= a_months
			l_left ?= a_left
			l_agent ?= a_agent
			l_g ?= a_g
			if l_months /= Void and l_g /= Void and l_left /= Void and l_agent /= Void then
				l_count := l_months.integer_constant_value
				if l_count /= 0 then
					create l_interval.make (l_count, periodicity_ids.monthly_periodicity_id)
					create lag_operator.make (l_interval)
					Result := lag_operator.generated_date (l_left, l_g, l_agent)
				end
			end
		end

feature {NONE} -- Implementation

	lag_target: ROSE_ENTITY_DATE[ENTITY] is
			-- Type of `lag_operator'
		do
		end

	lag_operator: ROSE_LAG_OPERATOR [like lag_target]
			-- Proxied operator

end

