note

	description:

		"An operator which return a lagged value or series"

	author: "Nelson Chui"
	last_editor: "$Author: mk $"
	date: "$Date: 2011/02/15 $"
	revision: "$Revision: #22 $"
	path: "$File: //rose/source/kernel/math/rose_lag_operator.e $"

class
	ROSE_LAG_OPERATOR [reference G -> ROSE_LAG_TARGET]

inherit
	ROSE_UNARY_OPERATOR[G]
		rename
			make as unary_make
		redefine
			units
		end

create
	make, make_series, make_conditional, make_conditional_series, make_with_lag_agent

feature -- Initialization

	make (a_interval: ROSE_LAG_INTERVAL) is
			-- Creation procedure
		do
			interval := a_interval
			unary_make (0, 0, "lag["+interval.name+"]")
			is_conditional := False
		end

	make_series (a_interval: ROSE_LAG_INTERVAL; a_length: INTEGER) is
			--make series
		do
			interval := a_interval
			length := a_length
			unary_make (0, 0, "lagx["+interval.name+"]")
			is_conditional := False
		end

	make_conditional (a_interval: ROSE_LAG_INTERVAL) is
			--make conditional
		do
			make (a_interval)
			is_conditional := True
		end

	make_conditional_series (a_interval: ROSE_LAG_INTERVAL; a_length: INTEGER) is
		do
			make_series (a_interval, a_length)
			is_conditional := True
		end

	make_with_lag_agent (a_lag_agent: FUNCTION [ANY, TUPLE [G, ROSE_EXPRESSION [G], ACCESS_CONTROL_AGENT], ROSE_LINEAR_ARRAY [G]]) is
		do
			lag_agent := a_lag_agent
			unary_make (0, 0, "lagx[agent]")
		end

feature -- Access

	interval: ROSE_LAG_INTERVAL

	length: INTEGER

	lag_agent: FUNCTION [ANY, TUPLE [G, ROSE_EXPRESSION [G], GENERIC_ACCESS_CONTROL_AGENT], ROSE_LINEAR_ARRAY [G]]

	is_conditional: BOOLEAN

feature -- Basic operations

	lagged_target (a_left : ROSE_EXPRESSION[G];a_g : G; a_agent : like access_agent_template) : G is
		do
			if lag_agent /= Void then
				Result := lag_agent.item ([a_g, a_left, a_agent]).first
			elseif is_conditional then
				Result := a_g.conditional_lagged (interval, a_left, a_agent)
			else
				Result := a_g.lagged (interval)
			end
		end

	lagged_universe_target (a_left : ROSE_EXPRESSION[G];a_g : UNIVERSE_TARGET[G]; a_agent : like access_agent_template) : UNIVERSE_TARGET[G] is
		do
		end

	real_value (a_left : ROSE_EXPRESSION[G];a_g : G) : REAL is
			--returned real value
		local
			l_lag_g: G
		do
			l_lag_g := lagged_target (a_left,a_g,Void)
			if l_lag_g /= Void then
				Result := a_left.real_value(l_lag_g)
			else
				Result := missing_real
			end
		end
	double_value (a_left : ROSE_EXPRESSION[G];a_g : G) : DOUBLE is
			--returned double value
		local
			l_lag_g: G
		do
			l_lag_g := lagged_target (a_left,a_g,Void)
			if l_lag_g /= Void then
				Result := a_left.double_value(l_lag_g)
			else
				Result := missing_double
			end
		end
	decimal_value (a_left : ROSE_EXPRESSION[G];a_g : G) : RP_DECIMAL is
		local
			l_lag_g: G
		do
			l_lag_g := lagged_target (a_left,a_g,Void)
			if l_lag_g /= Void then
				Result := a_left.decimal_value(l_lag_g)
			else
				create Result.make_undefined
			end
		end
	integer_value (a_left : ROSE_EXPRESSION[G];a_g : G) : INTEGER is
		local
			l_lag_g: G
		do
			l_lag_g := lagged_target (a_left,a_g,Void)
			if l_lag_g /= Void then
				Result := a_left.integer_value(l_lag_g)
			else
				Result := missing_integer
			end
		end
	boolean_value (a_left : ROSE_EXPRESSION[G];a_g : G) : BOOLEAN is
			-- Boolean value
		local
			l_lag_g: G
		do
			l_lag_g := lagged_target (a_left,a_g,Void)
			if l_lag_g /= Void then
				Result := a_left.boolean_value(l_lag_g)
			end
		end

	string_value (a_left: ROSE_EXPRESSION[G];a_g : G) : STRING is
		local
			l_lag_g: G
		do
			l_lag_g := lagged_target (a_left,a_g,Void)
			if l_lag_g /= Void then
				Result := a_left.string_value(l_lag_g)
			end
		end
	date_value (a_left : ROSE_EXPRESSION[G];a_g : G) : ROSE_DATE is
		local
			l_lag_g: G
		do
			l_lag_g := lagged_target (a_left,a_g,Void)
			if l_lag_g /= Void then
				Result := a_left.date_value(l_lag_g)
			end
		end

	value_in_market_currency (a_left : ROSE_EXPRESSION[G];a_market : MARKET;a_currency : PERSISTENT_CURRENCY;a_g : G;a_date : ROSE_DATE) : DOUBLE is
		local
			l_lag_g: G
		do
			l_lag_g := lagged_target (a_left,a_g,Void)
			if l_lag_g /= Void then
				Result := a_left.value_in_market_currency(a_market,a_currency,l_lag_g,a_date)
			else
				Result := missing_double
			end
		end
	units (a_left : ROSE_EXPRESSION[G];a_entity : G) : ROSE_UNITS is
		local
			l_lag_g: G
		do
			l_lag_g := lagged_target (a_left,a_entity,Void)
			if l_lag_g /= Void then
				Result := a_left.units(l_lag_g)
			end
		end
	units_in_currency (a_left : ROSE_EXPRESSION[G];a_currency : PERSISTENT_CURRENCY;a_entity : G) : ROSE_UNITS is
		local
			l_lag_g: G
		do
			l_lag_g := lagged_target (a_left,a_entity,Void)
			if l_lag_g /= Void then
				Result := a_left.units_in_currency(a_currency,l_lag_g)
			end
		end
	computed_double(a_left : ROSE_EXPRESSION[G]; a_g : G; a_agent : like access_agent_template) : DOUBLE is
			-- Computed double value
		local
			l_lag_g: G
		do
			l_lag_g := lagged_target (a_left,a_g,a_agent)
			if l_lag_g /= Void then
				Result := a_left.computed_double(l_lag_g,a_agent)
			else
				Result := missing_double
			end
		end
	computed_date(a_left : ROSE_EXPRESSION[G]; a_g : G; a_agent : like access_agent_template) : ROSE_DATE is
			-- Computed double value
		local
			l_lag_g: G
		do
			l_lag_g := lagged_target (a_left,a_g,a_agent)
			if l_lag_g /= Void then
				Result := a_left.computed_date(l_lag_g,a_agent)
			end
		end
	computed_real(a_left : ROSE_EXPRESSION[G]; a_g : G; a_agent : like access_agent_template) : REAL is
		local
			l_lag_g: G
		do
			l_lag_g := lagged_target (a_left,a_g,a_agent)
			if l_lag_g /= Void then
				Result := a_left.computed_real(l_lag_g,a_agent)
			else
				Result := missing_real
			end
		end
	computed_integer(a_left : ROSE_EXPRESSION[G]; a_g : G; a_agent : like access_agent_template) : INTEGER is
			-- Computed integer
		local
			l_lag_g: G
		do
			l_lag_g := lagged_target (a_left,a_g,a_agent)
			if l_lag_g /= Void then
				Result := a_left.computed_integer(l_lag_g,a_agent)
			else
				Result := missing_integer
			end
		end
	computed_boolean(a_left : ROSE_EXPRESSION[G]; a_g : G; a_agent : like access_agent_template) : BOOLEAN is
			-- Computed boolean
		local
			l_lag_g: G
		do
			l_lag_g := lagged_target (a_left,a_g,a_agent)
			if l_lag_g /= Void then
				Result := a_left.computed_boolean(l_lag_g,a_agent)
			end
		end
	computed_string(a_left : ROSE_EXPRESSION[G]; a_g : G; a_agent : like access_agent_template) : STRING is
		local
			l_lag_g: G
		do
			l_lag_g := lagged_target (a_left,a_g,a_agent)
			if l_lag_g /=  Void and a_left /= Void then
				Result := a_left.computed_string(l_lag_g,a_agent)
			end
		end
	computed_units (a_left: ROSE_UNITS): ROSE_UNITS is
		do
			Result := a_left
		end
	generated_double(a_left : ROSE_EXPRESSION[G]; a_g : G; a_agent : like access_agent_template) : DOUBLE is
		local
			l_lag_g: G
		do
			l_lag_g := lagged_target (a_left,a_g,a_agent)
			if l_lag_g /= Void then
				Result := a_left.generated_double(l_lag_g,a_agent)
			else
				Result := missing_double
			end
		end
	generated_real(a_left : ROSE_EXPRESSION[G]; a_g : G; a_agent : like access_agent_template) : REAL is
		local
			l_lag_g: G
		do
			l_lag_g := lagged_target (a_left,a_g,a_agent)
			if l_lag_g /= Void then
				Result := a_left.generated_real(l_lag_g,a_agent)
			else
				Result := missing_real
			end
		end
	generated_integer(a_left : ROSE_EXPRESSION[G]; a_g : G; a_agent : like access_agent_template) : INTEGER is
		local
			l_lag_g: G
		do
			l_lag_g := lagged_target (a_left,a_g,a_agent)
			if l_lag_g /= Void then
				Result := a_left.generated_integer(l_lag_g,a_agent)
			else
				Result := missing_integer
			end
		end
	generated_string(a_left : ROSE_EXPRESSION[G]; a_g : G; a_agent : like access_agent_template) : STRING is
		local
			l_lag_g: G
		do
			l_lag_g := lagged_target (a_left,a_g,a_agent)
			if l_lag_g /= Void then
				Result := a_left.generated_string(l_lag_g,a_agent)
			end
		end
	generated_date(a_left : ROSE_EXPRESSION[G]; a_g : G; a_agent : like access_agent_template) : ROSE_DATE is
			--generated date
		local
			l_lag_g: G
		do
			l_lag_g := lagged_target (a_left,a_g,a_agent)
			if l_lag_g /= Void then
				Result := a_left.generated_date(l_lag_g,a_agent)
			end
		end

feature -- Lagged Series

	trailing_targets (a_left : ROSE_EXPRESSION[G];a_g : G; a_agent : like access_agent_template) : ROSE_LINEAR_ARRAY [G] is
		do
			if lag_agent /= Void then
				Result := lag_agent.item ([a_g, a_left, a_agent])
			elseif is_conditional then
				Result := a_g.conditional_trailing_targets (interval, length, a_left, a_agent)
			else
				Result := a_g.trailing_targets (interval, length)
			end
		end

	computed_numeric_series (a_left : ROSE_EXPRESSION[G]; a_g : G; a_agent : like access_agent_template) : VECTOR is
		local
			l_lag_gs: ROSE_LINEAR_ARRAY [G]
		do
			l_lag_gs := trailing_targets (a_left,a_g,a_agent)
			if l_lag_gs /= Void then
				Result := a_left.computed_vector (l_lag_gs, a_agent)
			end
		end

	generated_numeric_series (a_left : ROSE_EXPRESSION[G]; a_g : G; a_agent : like access_agent_template) : VECTOR is
		local
			l_lag_gs: ROSE_LINEAR_ARRAY [G]
		do
			l_lag_gs := trailing_targets (a_left,a_g,a_agent)
			if l_lag_gs /= Void then
				Result := a_left.generated_vector (l_lag_gs, a_agent)
			end
		end

end
