note

	description:

		"A Rose target which can return a lagged target"

	author: "Nelson Chui"
	last_editor: "$Author: it $"
	date: "$Date: 2013/02/20 $"
	revision: "$Revision: #13 $"
	path: "$File: //rose/source/kernel/targets/rose_lag_target.e $"

deferred class
	ROSE_LAG_TARGET

inherit
	RP_HASHABLE
feature -- Access

	lagged (a_interval: ROSE_LAG_INTERVAL): like Current is
			-- Lagged target
		do
		end

	conditional_lagged (a_interval: ROSE_LAG_INTERVAL; a_expression: ROSE_EXPRESSION [like Current]; a_agent: GENERIC_ACCESS_CONTROL_AGENT): like Current is
			--conditional lagged target
		do
		end

	lagged_targets (a_interval: ROSE_LAG_INTERVAL; a_length: INTEGER): ROSE_LINEAR_ARRAY [like Current] is
			-- Array of lagged targets beginning from before Current
		require
			meaningful_length: a_length > 1
			valid_interval: a_interval /= Void
		local
			i: INTEGER
			l_target: like Current
		do
			create Result.make (1, a_length)
			l_target := Current
			from i := 1 until i > a_length or l_target = Void loop
				l_target := l_target.lagged (a_interval)
				Result.put (l_target, i)
				i := i + 1
			end
			if l_target = Void then
				create Result.make_from_array (Result.subarray (1, i - 2))
			end
		ensure
			valid_result: Result /= Void
		end

	conditional_lagged_targets (a_interval: ROSE_LAG_INTERVAL; a_length: INTEGER; a_expression: ROSE_EXPRESSION [like Current];
								a_agent: GENERIC_ACCESS_CONTROL_AGENT): ROSE_LINEAR_ARRAY [like Current] is
			-- lagged targets relative to a_agent
		local
			i: INTEGER
			l_target: like Current
		do
			create Result.make (1, a_length)
			l_target := Current
			from i := 1 until i > a_length or l_target = Void loop
				l_target := l_target.conditional_lagged (a_interval, a_expression, a_agent)
				Result.put (l_target, i)
				i := i + 1
			end
		end

	trailing_targets (a_interval: ROSE_LAG_INTERVAL; a_length: INTEGER): ROSE_LINEAR_ARRAY[like Current] is
			-- Array of lagged target beginning with Current
		require
			valid_length: a_length >= 0
		do
			if a_length > 1 then
				Result := lagged_targets (a_interval, a_length - 1)
				Result.put_front (Current)
			else
				create Result.make_empty
				Result.extend (Current)
			end
		end

	conditional_trailing_targets (a_interval: ROSE_LAG_INTERVAL; a_length: INTEGER; a_expression: ROSE_EXPRESSION [like Current];
								  a_agent: GENERIC_ACCESS_CONTROL_AGENT): ROSE_LINEAR_ARRAY[like Current] is
			-- Array of lagged target beginning with Current and subject to `conditional_lagged_targets'
		do
			create Result.make_empty
			Result.extend (Current)
			if a_length > 1 then
				Result.append_array (conditional_lagged_targets (a_interval, a_length - 1, a_expression, a_agent))
			end
		end

end












