indexing
	description: "Legend for drawings and graphs"
	author: "Zoran Simic"
	last_editor: "$Author: io $"
	date: "$Date: 2014/12/02 $"
	revision: "$Revision: #30 $"
	archive: "$File: //rose/source/kernel/infrastructure/graph/rose_legend.e $"

deferred class
	ROSE_LEGEND [G]

inherit
	ROSE_SHARED_GLOBAL_GUI
		redefine
			default_create
		end

feature {NONE} -- Initialization

	default_create is
			-- Create the object
		do
			create font
			create click_event
			create figure
			figure.disable_scaling
			create explanation_hash.make (10)
			figure.set_point_position (margin + 2,margin + 2)
			create background
			figure.extend (background)
			create title_figure
			figure.extend (title_figure)
			create holder
			figure.extend (holder)
			create seperator_lines.make
			-- Default settings
			create figure_explanation_fonts_hashed.make (4)
			set_font (Gui.rep.fonts.new_font (False,False,14))
			set_default_font_style (Gui.rep.fonts.shape_italic)
			set_colors (Void)
			set_icon_size (4,4)
			maximum_height := maximum_height.max_value
			maximum_width := maximum_width.max_value
		end

	update_layout is
			-- Update position of `background' and `title_figure'
		do
			background.set_point_a_position (x - margin, y - margin)
			background.set_point_b_position (x + width + margin, y + height + margin)
			title_figure.set_point_position (x + (width - title_figure.width) // 2, y)

			from
				seperator_lines.start
			until
				seperator_lines.after
			loop
				seperator_lines.item.set_point_a_position (x - margin + 1, seperator_lines.item.point_a_y )
				seperator_lines.item.set_point_b_position (x + width + margin - 1, seperator_lines.item.point_b_y)
				seperator_lines.forth
			end
		end

feature -- Access

	click_event: ROSE_EVENT_TYPE [TUPLE [INTEGER,STRING]]
			-- Event called when mouse clicked on legend item
			-- The INTEGER designates the mouse button, the STRING contains given 'explanation'

	title: STRING is
			-- Title of this legend
		do
			Result := title_figure.text
		end

	palette: ROSE_PALETTE
			-- Associated color palette

	figure: EV_MODEL_MOVE_HANDLE
			-- Figure representing this legend

	count: INTEGER
			-- Number of items in this legend

	x: INTEGER is
			-- Current 'x' position of legend
		do
			Result := figure.point_x
		end

	y: INTEGER is
			-- Current 'y' position of legend
		do
			Result := figure.point_y
		end

	icon (a_id: G): EV_MODEL_ATOMIC is
			-- Icon corresponding to 'a_id', return a new object each time
			-- Do not return cloned objects, as this does not work with EV_MODELs
		deferred
		ensure
			valid_result: Result /= Void
		end

	font: EV_FONT
			-- Font used to show legend items

	default_font_style: INTEGER
			-- Font style used to override when displaying

	width, height: INTEGER
			-- Width and height in pixels of current legend

	maximum_width, maximum_height: INTEGER
			-- Maximum width & height of current legend

	icon_width: INTEGER
			-- Maximum width of each icon

	icon_height: INTEGER
			-- Maximum height of each icon

	orientation_horizontal: BOOLEAN
			-- Is the orientation of the legend horizontal ?

	docked_style: BOOLEAN
			-- Is the legend in docked style (different rules for width/height)

	title_visible: BOOLEAN
			-- Is the title visible (and occupying space)

	pebble_function: FUNCTION [ANY, TUPLE [STRING], ANY]
			-- Optional pebble function to allow users to pick items from the legend

	last_text: ROSE_CHART_EV_MODEL_TEXT
			-- Last added overlay used for pick and drop

	has_point (a_x, a_y: INTEGER): BOOLEAN is
			-- Is point (`a_x'/`a_y') within background figure?
		do
			Result := background.position_on_figure (a_x,a_y)
		end

	new_line (a_color: EV_COLOR; a_x1, a_y1, a_x2, a_y2: INTEGER): EV_MODEL_LINE is
			-- New line with 'a_color' from (x1,y1) to (x2,y2)
		do
			create Result.make_with_positions (a_x1, a_y1, a_x2, a_y2)
			if a_color /= Void then
				Result.set_foreground_color (a_color)
			end
		end

	new_rectangle (a_color: EV_COLOR): EV_MODEL_RECTANGLE is
			-- New rectangle
		do
			create Result.make_with_positions (-icon_width, -icon_height, icon_width, icon_height)
			if a_color /= Void then
				Result.set_background_color (a_color)
			end
		end

	new_ellipse (a_color: EV_COLOR): EV_MODEL_ELLIPSE is
			-- New ellipse
		do
			create Result.make_with_positions (-icon_width, -icon_height, icon_width, icon_height)
			if a_color /= Void then
				Result.set_background_color (a_color)
			end
		end

	new_triangle (a_color: EV_COLOR; a_factor: INTEGER): EV_MODEL_POLYGON is
			-- New triangle
			-- 'a_factor' of 1 returns a triangle heading down, -1 heading up
		require
			valid_factor: a_factor = -1 or a_factor = 1
		do
			create Result.make_with_coordinates (<<
				new_coordinate (a_factor * icon_width, a_factor * icon_height),
				new_coordinate (0, -a_factor * icon_height),
				new_coordinate (-a_factor * icon_width, a_factor * icon_height)
			>>)
			if a_color /= Void then
				Result.set_background_color (a_color)
			end
		end

	new_diamond (a_color: EV_COLOR): EV_MODEL_POLYGON is
			-- New diamond
		do
			create Result.make_with_coordinates (<<
				new_coordinate (0,icon_height),
				new_coordinate (icon_width,0),
				new_coordinate (0,-icon_height),
				new_coordinate (-icon_width,0)
			>>)
			if a_color /= Void then
				Result.set_background_color (a_color)
			end
		end

	new_pixmap (a_icon_name: STRING): EV_MODEL_PICTURE is
			-- Load pixmap from `a_icon_name'
		do
			create Result.make_with_pixmap (Gui.factory.new_pixmap_from_file (a_icon_name))
		end

	new_point: EV_MODEL_DOT is
			-- Create new point
		do
			create Result.make_with_point (create {EV_COORDINATE})
		ensure
			Result_non_void: Result /= Void
		end

	new_coordinate (a_x, a_y: INTEGER): EV_COORDINATE is
			-- New {EV_COORDINATE} with coordinates `a_x'/`a_y'
		do
			create Result.set (a_x,a_y)
		end

	main_subtitle: STRING
			-- Title to display above list of main_system plot names.
		do
			Result := user_set_main_subtitle
			if Result = Void then
				Result := k_default_main_subtitle
			end
		ensure
			result_not_void: Result /= Void
		end

	secondary_subtitle: STRING
			-- Title to display above list of secondary_system plot names.
		do
			Result := user_set_secondary_subtitle
			if Result = Void then
				Result := k_default_secondary_subtitle
			end
		ensure
			result_not_void: Result /= Void
		end

feature -- Basic operations
	set_main_subtitle (a_title: STRING)
			-- Set `main_subtitle' with `a_title'.
			-- if void then generic subtitle is used; if empty string no subtitle included
		do
			user_set_main_subtitle := a_title
		end

	set_secondary_subtitle (a_title: STRING)
			-- Set `secondary_subtitle' with `a_title'.
			-- if void then generic subtitle is used; if empty string no subtitle included
		do
			user_set_secondary_subtitle := a_title
		end

	wipe_out, reset is
			-- Reset the legend, make it empty.
		do
			holder.wipe_out
			explanation_hash.clear_all
			seperator_lines.wipe_out
			count := 0
			width := 0
			height := 0

			x_position := x
			y_position := y + title_figure.height
			set_title (title)
		end

	add_explanation, add_icon (a_id: G; a_explanation: STRING) is
			-- Add 'a_explanation' for icon corresponding to 'a_id'.
			-- 'a_figure' is expected to be within rectangle of coordinates (-1,-1,1,1)
		require
			valid_explanation: a_explanation /= Void
		do
			add_figure_explanation (icon (a_id),a_explanation)
		end

	add_line_explanation, add_line (a_color: EV_COLOR; a_explanation: STRING) is
			-- Add 'a_explanation' legend entry for lines with 'a_color'.
		require
			valid_explanation: a_explanation /= Void
		do
			if a_color /= Void then
				add_figure_explanation (new_line (a_color,-icon_width,0,icon_width,0),a_explanation)
			else
				add_figure_explanation (Void,a_explanation)
			end
		end

	add_new_line is
			-- Begin a new line in the legend
		do
			y_position := y_position + 3*icon_height + 2*padding_y
			x_position := x
			height := height + 3*icon_height + 2*padding_y
		ensure
			new_line_added: orientation_horizontal implies x_position = x
		end

	add_separator is
			-- Add a separator to the legend
		local
			l_line: EV_MODEL_LINE
		do
			create l_line.make_with_positions (x - margin + 1, y_position + 3*icon_height, x + width + margin - 1, y_position + 3*icon_height)
			l_line.set_foreground_color (create {EV_COLOR}.make_with_8_bit_rgb (192, 192, 192))
			holder.extend (l_line)
			y_position := y_position + 2*icon_height
			x_position := x
			height := height + 2*icon_height

			seperator_lines.extend (l_line)
		end

	add_figure_explanation (a_figure: EV_MODEL; a_explanation: STRING) is
			-- Add 'a_figure' with 'a_explanation' to the legend figure.
		require
			valid_explanation: a_explanation /= Void
		local
			l_text: ROSE_CHART_EV_MODEL_TEXT
		do
			if a_figure /= Void then
				a_figure.pointer_button_press_actions.extend (agent on_item_clicked (?, ?, ?, ?, ?, ?, ?, ?, a_explanation))
				if pebble_function /= Void then
					a_figure.set_pebble_function (agent pebble_function.item ([a_explanation]))
				end
			end
			create l_text.make_with_text (a_explanation)
			l_text.pointer_button_press_actions.extend (agent on_item_clicked (?, ?, ?, ?, ?, ?, ?, ?, a_explanation))
			if pebble_function /= Void then
				l_text.set_pebble_function (agent pebble_function.item ([a_explanation]))
			end
			explanation_hash [a_explanation] := [a_figure, l_text]
			place_figure_explanation (a_figure, l_text, True, default_font_style)
			last_text := l_text
		end

	add_sub_title (a_title: STRING) is
			-- Add `a_title' as subtitle to the current legend
			-- Same as add explanation, just other font, and no icon
		local
			l_text: ROSE_CHART_EV_MODEL_TEXT
		do
			create l_text.make_with_text (a_title)
			place_figure_explanation (Void, l_text, False, Gui.rep.fonts.shape_regular)
			last_text := l_text
		end

	set_title (a_title: STRING) is
			-- Set 'a_title' for this legend
		local
			l_font: EV_FONT
			l_title: STRING
		do
			if a_title /= Void then
				l_title := a_title
			else
				l_title := "Legend"
			end
			l_font := font.twin
			l_font.set_weight (Gui.rep.fonts.weight_bold)
			title_figure.set_text (l_title)
			title_figure.set_font (l_font)

			if title_visible then
				y_position := y + (1.3*title_figure.height).rounded + padding_y
				x_position := x

				height := height.max ((1.3 * title_figure.height).rounded + padding_y + icon_height)
				width := width.max ((1.1 * title_figure.width).rounded)
				title_figure.show
			else
				y_position := y + 3*padding_y
				x_position := x

				height := height.max (3*padding_y)+ 2*icon_height
				width := width.max (20)
				title_figure.hide
			end

			update_layout
		end

	set_x (a_x: INTEGER) is
			-- Set x position of legend to `a_x'
		do
			-- Update x_position
			x_position := x_position - x + a_x
			figure.set_point_position (a_x, y)
		end

	set_y (a_y: INTEGER) is
			-- Set x position of legend to `a_x'
		do
			-- Update y_position
			y_position := y_position - y + a_y
			figure.set_point_position (x, a_y)
		end

	set_maximum_width (a_width: INTEGER) is
			-- set `maximum_width' to `a_width'
		require
			width_valid: a_width >= 0
		do
			maximum_width := a_width
		ensure
			width_correct: maximum_width = a_width
		end

	set_maximum_height (a_height: INTEGER) is
			-- set `maximum_height' to `a_height'
		require
			height_valid: a_height >= 0
		do
			maximum_height := a_height
		ensure
			height_correct: maximum_height = a_height
		end

	show (a_on: BOOLEAN) is
			-- Show/hide this entire legend.
		do
			if a_on then
				figure.show
			else
				figure.hide
			end
		end

	show_background (a_on: BOOLEAN) is
			-- Show/hide the background of this legend.
		do
			if a_on then
				background.show
			else
				background.hide
			end
		end

	set_colors (a_colors: ARRAY [EV_COLOR]) is
			-- Assign 'a_colors' to this legend's palette.
		do
			if palette = Void then
				create palette.make_default
			end
			if a_colors /= Void then
				palette.set_colors (a_colors)
			end
			set_palette (palette)
		end

	set_palette (a_palette: ROSE_PALETTE) is
			-- Define palette to be used
		require
			palette_not_void: a_palette /= Void
		do
			palette := a_palette
			background.set_background_color (palette.background)
			background.set_foreground_color (palette.foreground)
		end

	set_icon_size (a_width, a_height: INTEGER) is
			-- Set icon size to be used
		require
			width_height_valid: a_width > 0 and then a_height > 0
		do
			icon_width := a_width
			icon_height := a_height
			update_layout
		end

	set_font (a_font: like font) is
			-- Set font to be used
		require
			a_font_not_void: a_font /= Void
		do
			font := a_font
			figure_explanation_fonts_hashed.clear_all
		ensure
			font_set: font = a_font
		end

	set_default_font_style (a_style: like default_font_style) is
			-- Set font style to be used
		require
			valid_style: (create {EV_FONT_CONSTANTS}).valid_shape (a_style)
		do
			default_font_style := a_style
		end

	set_orientation_horizontal (a_switch: BOOLEAN) is
			-- Set `orientation_horizontal' according to `a_switch'
			-- You need to reset the legend before the new orientation is applied properly
		do
			orientation_horizontal := a_switch
		ensure
			orientation_set: orientation_horizontal = a_switch
		end

	set_docked_style (a_switch: BOOLEAN) is
			-- Set `docked_style' according to `a_switch'
		require
			figure_not_void: figure /= Void
		do
			docked_style := a_switch
			if a_switch then
				figure.disable_moving
			else
				figure.enable_moving
			end
		ensure
			style_correct: docked_style = a_switch
			moving_correct: a_switch /= figure.is_moving
		end

	set_title_visible (a_switch: BOOLEAN) is
			-- Set `title_visible' according to `a_switch'
		do
			title_visible := a_switch
		ensure
			title_visible_set: title_visible = a_switch
		end

	set_background_action (a_action: EV_PND_ACTION_SEQUENCE) is
			-- Add a drop action to the background
		do
			background.set_drop_actions (a_action)
		end

	set_pebble_function (a_pebble_function: like pebble_function) is
			-- Set 'pebble_function'.
		do
			pebble_function := a_pebble_function
		end

feature {NONE} -- Implementation
	user_set_main_subtitle, user_set_secondary_subtitle: STRING
			-- User-set subtitles.

	background: ROSE_CHART_EV_MODEL_RECTANGLE
			-- Background figure
	title_figure: EV_MODEL_TEXT
			-- Title figure
	holder: EV_MODEL_GROUP
			-- Model group holding all other figures

	margin: INTEGER is 4

	padding_x: INTEGER is 2

	padding_y: INTEGER is 2

	place_figure_explanation (a_figure: EV_MODEL; a_text: ROSE_CHART_EV_MODEL_TEXT; space_for_icon: BOOLEAN; font_shape: INTEGER) is
			-- Place a figure and explanation on the legend
		require
			text_valid: a_text /= Void
		local
			l_x, l_h: INTEGER
			l_font: EV_FONT
		do
			l_font := figure_explanation_fonts_hashed.item (font_shape)
			if l_font = Void then
				l_font := font.twin
				l_font.set_shape (font_shape)
				figure_explanation_fonts_hashed.put (l_font, font_shape)
			end
			a_text.set_font (l_font)
			l_h := a_text.height // 2
			holder.extend (a_text)
			if orientation_horizontal then
				-- Determine if we need to start a new line
				if x_position + a_text.width > maximum_width and x_position /= x then
					add_new_line
				end
				if a_figure /= Void then
					holder.extend (a_figure)
					a_figure.set_x_y (x_position + icon_width, y_position)
				end

				if not space_for_icon then
					x_position := x_position - 2*icon_width
				end

				a_text.set_point_position (x_position + 2*padding_x + 2*icon_width, y_position - l_h)
				x_position := x_position + padding_x + 4*icon_width + a_text.width

				width := width.max (x_position - x + margin)
			else
				if count >= 1 and not a_text.text.is_empty then
					add_new_line
				end
				if space_for_icon then
					l_x := icon_width + padding_x
				else
					l_x := padding_x - icon_width
				end
				if a_figure /= Void then
					holder.extend (a_figure)
					a_figure.set_x_y (x + icon_width, y_position)
				end
				a_text.set_point_position (x + l_x + icon_width, y_position - l_h)

				width := width.max (l_x + icon_width + a_text.width + margin)
			end
			if not a_text.text.is_empty then
				count := count + 1
			end
			update_layout
		end

	figure_explanation_fonts_hashed: HASH_TABLE [EV_FONT, INTEGER]

	on_hide_legend (a_x, a_y, a_button: INTEGER; a_x_tilt, a_y_tilt, a_pressure: DOUBLE; a_screen_x, a_screen_y: INTEGER) is
			-- Hide legend
		do
			if a_button = 1 then
				if holder.is_show_requested then
					holder.hide
					background.hide
				else
					holder.show
				end
			end
		end

	on_hide_background (a_x, a_y, a_button: INTEGER; a_x_tilt, a_y_tilt, a_pressure: DOUBLE; a_screen_x, a_screen_y: INTEGER) is
			-- Hide background
		do
			if a_button = 1 then
				if background.is_show_requested then
					background.hide
				else
					background.show
				end
			end
		end

feature {NONE} -- Actions

	on_item_clicked (a_x, a_y, a_button: INTEGER; a_x_tilt, a_y_tilt, a_pressure: DOUBLE; a_screen_x, a_screen_y: INTEGER; a_explanation: STRING) is
			-- When an item is clicked, fire the click_actions
		do
			click_event.call ([a_button, a_explanation])
		end

feature {NONE} -- Implementation

	explanation_hash: HASH_TABLE [TUPLE [figure: EV_MODEL; text: EV_MODEL_TEXT], STRING]
			-- Hash table containing all entries in the legend, hashed by their explanation

	items_in_line: INTEGER
			-- Number of items in current line

	x_position: INTEGER
			-- Position of the right edge of the last legend item

	y_position: INTEGER
			-- Position of the bottom edge of the last legend item

	seperator_lines: LINKED_LIST[EV_MODEL_LINE]
			-- Holds all seperator lines, so they can be resized when the box grows

feature {NONE} -- Constants
	k_default_main_subtitle : STRING is "---Main---"

	k_default_secondary_subtitle : STRING is "---Secondary---"

invariant
	dimensions_in_bounds: height <= maximum_height and width <= maximum_width
	figure_explanation_fonts_hashed_not_void: figure_explanation_fonts_hashed /= Void

end
