indexing

	description: "[

                   ROSE COMMAND with access to a logger: ELOG_LOGGER

                   For non UI commands, do not inherit directly. Inherit from RP_COMMAND.

                 ]"

	author: "Emmanuel Bouyer"
	last_editor: "$Author: e2 $"
	date: "$Date: 2014/07/04 $"
	revision: "$Revision: #16 $"
	path: "$File: //rose/source/kernel/infrastructure/logging/rose_dependant/rose_logger_facility.e $"

deferred class ROSE_LOGGER_FACILITY

inherit
	ROSE_COMMAND
		redefine
			pre_execute
		end
	ROSE_SHARED_LOGGER

feature {NONE} -- Implementation

	pre_execute
			-- <Precursor>
		local
			l_message: STRING
		do
			Precursor
			setup_logger
			l_message := id.twin
			l_message.append_character (' ')
			l_message.append (parameters.text)
			logger.notice (l_message)
			start_time_measure (id, parameters.text)
		ensure then
			logger_changed: not (logger ~ default_logger)
		end

	logfile_name: STRING
			-- redefine this to set a logfile name different from that in the config file
		do
		end

	setup_logger
			-- get the logger going
		local
			l_logfile_name, l_msg, l_logger_name: STRING
			l_config: ELOG_CONFIGURATION
		do
			l_logfile_name := logfile_name
			if l_logfile_name = Void then
				-- if the application redefined `logfile_name' then use that value
				-- otherwise use the config value
				-- or else the id of the command
				l_logfile_name := config.value_default (log_filename_key, id + ".log")
			end

			if l_logfile_name /= Void then
				-- Memorize the value back into the configuration
				if config.value (log_filename_key) = Void or else not l_logfile_name.is_equal (config.value (log_filename_key)) then
					config.set (log_filename_key, l_logfile_name)
				end
			end

			-- now, always use the value that is in the current config settings if it is defined
			l_logfile_name := config.value (log_filename_key)

			if config.value (log_filename_key) /= Void then
				l_logger_name := "app_specific_logger"
			else
				l_logger_name := "default_logger_" + id
			end

			create l_config.make
			l_config.set_logger_name (l_logger_name)
			l_config.set_use_std_out (use_std_out)
			set_logger (new_complex_logger (l_config, log_filename_key))

			if l_logfile_name /= Void then
				-- if the logfile name has changed, initialize with the new logfile_name
				-- and remove the previous appender
				create l_msg.make_empty
				if previous_logfile_name.item /= Void and then
					not previous_logfile_name.item.is_equal (l_logfile_name)
				then
					remove_appender (previous_logfile_name.item)
					l_msg := "re-"
				end
				logger.info ("Logger "+l_msg+"initialized: " + l_logfile_name)
				set_previous_logfile_name (logfile_name)
			end
		end

feature {NONE} -- Helpers

	use_std_out: BOOLEAN
			-- Should the logger prints log in standard output (on top of log file)?
		do
			if config.has (use_std_out_key) then
				Result := config.boolean (use_std_out_key, True)
			else
					-- If nothing specified in the configuration
					-- then log in std out if we're running in ES
					-- or if it is not a UI application
				Result := global.app.is_running_in_estudio or else
					not global.app.is_gui
			end
		end

	remove_appender (a_name: STRING)
			-- remove the old appender probably because the logfile name has changed.
		require
			a_name_not_void: a_name /= Void
		local
			l_filename: STRING
		do
			l_filename := global.app.log_path (a_name)
			if logger.has_appender_by_name (l_filename) then
				logger.remove_appender (l_filename)
			end
		end

	set_previous_logfile_name (a_name: STRING)
			-- keep track of the previous log file name so that we can remove the old appender if the log file name has changed.
		do
			previous_logfile_name.replace (a_name)
		ensure
			previous_logfile_name_replaced: previous_logfile_name.item ~ a_name
		end

	previous_logfile_name: CELL [STRING]
			-- when we change the name of the log file, we want to discontinue logging to the previous log file.
		once
			create Result.put (Void)
		ensure
			previous_logfile_name_not_void: Result /= Void
		end

feature {NONE} -- Config keys

	use_std_out_key: STRING is "logging/use_std_out"
end
