indexing
	description: "Lookup table"
	status: "Development"
	author: "Jun Zhu"
	date: "$Date: 2009/12/07 $"
	revision: "$Revision: #9 $"

class
	ROSE_LOOKUP_TABLE [ N -> HASHABLE, K -> HASHABLE ]

inherit
	HASH_TABLE [N, K]

create
	make_table,
	make_table_with_where

create {ROSE_LOOKUP_TABLE}
	make

feature -- Initialization

	make_table( 	a_data_server: 	DATA_SERVER;
					a_table_name: 	STRING;
					a_key_column: 	STRING;
					a_name_column: 	STRING ) is
	do
		create predicate.make( a_data_server, select_statement( a_table_name, a_key_column, a_name_column ))
		if not predicate.success then
			(create {EXCEPTIONS}).raise ("Could not make predicate for lookup table" )
		end

		load
	end

	make_table_with_where( 	a_data_server: 	DATA_SERVER;
							a_table_name: 	STRING;
							a_key_column: 	STRING;
							a_name_column: 	STRING;
							a_where:		STRING ) is
	do
		create predicate.make( a_data_server, select_statement( a_table_name, a_key_column, a_name_column ) + a_where )
		if not predicate.success then
			(create {EXCEPTIONS}).raise ("Could not make predicate for lookup table" )
		end

		load
	end

	reload is
	do
		clear_all
		name_cache.clear_all
		load
	end

	has_ukey (a_ukey: K): BOOLEAN is
			-- Does this lookup table have 'a_ukey'?
		do
			Result := has (a_ukey)
		end

	name_from_ukey( a_ukey: K ):N is
	do
		search( a_ukey )
		if found then
			Result := found_item
		else
			(create {EXCEPTIONS}).raise ("could not find name_for_ukey: " + a_ukey.out )
		end
	end

	has_name( a_name: N ): BOOLEAN is
	do
		Result := name_cache.has( a_name )
	end

	ukey_from_name( a_name: N ): K is
	do
		name_cache.search( a_name )
		if name_cache.found then
			Result := name_cache.found_item
		else
			(create {EXCEPTIONS}).raise ("could not find ukey_from_name: " + a_name.out )
		end
	end

feature {NONE} -- Implementation

	load is
	local
		l_array: EXTENDABLE_BOUNDED_ARRAY[ROSE_LOOKUP_RECORD[N,K]]
		i :	INTEGER
	do
		create l_array.make_empty
		predicate.set_container( l_array )
		predicate.retrieve(Void, Void)
	 	if not predicate.success then
	 		(create {EXCEPTIONS}).raise ("could not retrieve from lookup table" )
	 	end

		make( l_array.count )
		create name_cache.make( l_array.count )

		from i := 1 until i > l_array.count
		loop
			put( l_array.item(i).name, l_array.item(i).ukey )
			name_cache.put( l_array.item(i).ukey, l_array.item(i).name )
			i := i+1
		end
	end


	select_statement( a_table_name: STRING; a_ukey_column: STRING; a_name_column: STRING ): STRING is
	do
		Result := "select " + a_ukey_column + " as ukey, " + a_name_column + " as name from " + a_table_name + " "
	end

	name_cache:	HASH_TABLE[K, N]

   	template: ROSE_LOOKUP_RECORD[N,K]
	predicate: COLLECTING_PREDICATE [ROSE_LOOKUP_RECORD[N,K]]

end
