indexing
	description: "Dialog to send an email"
	author: "Zoran Simic"
	last_editor: "$Author: zs $"
	date: "$Date: 2011/01/26 $"
	revision: "$Revision: #1 $"
	archive: "$File: //rose/source/kernel/infrastructure/framework/gui/rose_mail_dialog.e $"

class
	ROSE_MAIL_DIALOG

inherit
	ROSE_RICH_TEXT_DIALOG
		redefine
			initialize, setup_main_box, on_ok
		end
	ROSE_SHARED_GLOBAL_GUI
		export
			{NONE} all
		undefine
			default_create, copy, is_equal
		end

feature {NONE} -- Initialization

	initialize
			-- Called by 'default_create'.
		do
			Precursor
			ok_button.set_text ("Send")
			set_minimum_size (600, 400)
			set_title ("Email report")
			set_icon ("sendmail.ico")
			status := Gui.factory.new_label (control_box, Void, 0)
			remove_default_push_button
		end

	setup_main_box
			-- Set up 'main_box'
		local
			l_hbox: EV_BOX
			l_recip_label, l_subject_label: EV_LABEL
			i: INTEGER
		do
			-- Recipients
			l_hbox := Gui.factory.new_hbox (main_box, 0, 0)
			l_recip_label := Gui.factory.new_label (l_hbox, "Recipients:  ", 1)
			l_recip_label.set_foreground_color (gui.rep.colors.dark_blue)
			recipients := Gui.factory.new_text_field (l_hbox, Void, Void, Void, 0)
			-- Subject
			l_hbox := Gui.factory.new_hbox (main_box, 0, 0)
			l_subject_label := Gui.factory.new_label (l_hbox, "Subject:  ", 1)
			l_subject_label.set_foreground_color (gui.rep.colors.dark_blue)
			subject := Gui.factory.new_text_field (l_hbox, Void, Void, Void, 0)
			i := l_recip_label.width.max (l_subject_label.width)
			l_recip_label.set_minimum_width (i)
			l_subject_label.set_minimum_width (i)
			Precursor
			text.enable_edit
			text.append ("Hi,%N%N", Void)
		end

feature -- Access

	recipients: EV_TEXT_FIELD		-- Who to send email to
	subject: EV_TEXT_FIELD			-- Subject

feature -- Basic operations

	set_recipients (a_recipients: STRING)
			-- Set 'recipients'
		do
			if a_recipients /= Void then
				recipients.set_text (" " + a_recipients)
			else
				recipients.remove_text
			end
		end

	set_subject (a_subject: STRING)
			-- Set 'subject'
		do
			if a_subject /= Void then
				subject.set_text (" " + a_subject)
			else
				subject.remove_text
			end
		end

	set_text (a_string: STRING)
			-- Set 'text' of the email
		do
			if a_string /= Void then
				text.set_text (a_string)
			else
				text.remove_text
			end
		end

	set_rtf_text (a_string: STRING)
			-- Set 'text' of the email to send, composed in RTF format
		do
			if a_string /= Void then
				text.set_rtf_text (a_string)
			else
				text.remove_text
			end
		end

feature {NONE} -- Implementation

	status: EV_LABEL
			-- Shows status of current selection

	mail: ROSE_MAIL
			-- Object used to send email
		once
			create Result.make
		end

	on_ok
			-- Called when 'OK' button is pressed
		local
			l_recipients, l_subject, l_body: STRING
			l_content: ROSE_MAIL_CONTENT
		do
			l_recipients := recipients.text
			l_subject := subject.text
			l_body := text.text
			l_recipients.left_adjust
			l_recipients.right_adjust
			l_subject.left_adjust
			l_subject.right_adjust
			l_body.left_adjust
			l_body.right_adjust
			if l_recipients.is_empty then
				gui.dialogs.inform (Current, "Problem", "Please fill in the 'recipients' box", Void)
			elseif l_subject.is_empty then
				gui.dialogs.inform (Current, "Problem", "Please fill in the 'subject' box", Void)
			elseif l_body.is_empty then
				gui.dialogs.inform (Current, "Problem", "Please provide a message (body) for the email", Void)
			else
				create l_content.make
				l_content.to.set (l_recipients)
				l_content.set_subject (l_subject)
				l_content.set_text (l_body)
				l_content.add_content_attachment ("application-log.txt", Global.app.report)
				mail.send (l_content)
				if mail.success then
					Gui.dialogs.inform (Current, "Email sent", "The email was sent successfully%NYou should have received a copy as well", Void)
					Precursor
				elseif Gui.dialogs.answer (Current, Void, "Can not send email", "Your mail could not be sent due to the following error:%N%N$1%N%NPlease send a screenshot of this to the Tools group%NTry again?%N", <<mail.last_error>>) then
					on_ok
				else
					hide
				end
			end
		end

end
