indexing
	description: "Mail recipients"
	author: "Zoran Simic"
	date: "$Date: 2008/12/01 $"
	revision: "Revision: $"

class
	ROSE_MAIL_RECIPIENTS

inherit
	DEBUG_OUTPUT

create
	default_create, make, set

convert
	set ({STRING})

feature {NONE} -- Initialization

	make (a_recipients: ARRAY [ROSE_MAIL_ADDRESS]) is
			-- Create with 'a_recipients'
		do
			recipients := a_recipients
		ensure
			recipients = a_recipients
		end

feature -- Access

	recipients: ARRAY [ROSE_MAIL_ADDRESS]
			-- List of addresses composing these recipients

	count: INTEGER is
			-- Number of addresses in these recipients
		do
			if recipients /= Void then
				Result := recipients.count
			end
		end

	is_empty: BOOLEAN is
			-- Are these recipients currently empty?
		do
			Result := recipients = Void or else recipients.is_empty
		end

	representation: STRING is
			-- String representation
		local
			l_address: ROSE_MAIL_ADDRESS
			i: INTEGER
		do
			create Result.make_empty
			if recipients /= Void then
				from
					i := recipients.lower
				until
					i > recipients.upper
				loop
					l_address := recipients.item (i)
					if l_address /= Void then
						Result.append_string (l_address.representation)
						if i < recipients.upper then
							Result.append_character (',')
							Result.append_character (' ')
						end
					end
					i := i + 1
				end
			end
		ensure
			Result /= Void
		end

	has (a_address: ROSE_MAIL_ADDRESS): BOOLEAN is
			-- Is 'a_address' part of these recipients?
		local
			i: INTEGER
		do
			if recipients /= Void then
				from
					i := 1
				until
					Result or else i > recipients.count
				loop
					Result := equal (a_address, recipients.item (i))
					i := i + 1
				end
			end
		end

	debug_output: STRING is
			-- String that should be displayed in debugger to represent `Current'.
		do
			Result := representation
		end

feature -- Basic operations

	set (a_string: STRING) is
			-- Set recipient list from the ones listed in 'a_string', comma or ';' separated
		do
			clear_all
			add (a_string)
		end

	add (a_string: STRING) is
			-- Add recipients listed in 'a_string', comma or ';' separated
		local
			l_string: STRING
			i, j: INTEGER
		do
			if a_string /= Void and then not a_string.is_empty then
				from
					j := 1
					i := 1
				until
					i = 0
				loop
					i := separator_index (a_string, j)
					l_string := Void
					if i > 0 then
						if i > j then
							l_string := a_string.substring (j, i - 1)
						end
					elseif j <= a_string.count then
						l_string := a_string.substring (j, a_string.count)
					end
					if l_string /= Void then
						l_string.left_adjust
						l_string.right_adjust
						if not l_string.is_empty then
							add_address (create {ROSE_MAIL_ADDRESS}.set (l_string))
						end
					end
					j := i + 1
				end
			end
		end

	add_address (a_address: ROSE_MAIL_ADDRESS) is
			-- Add 'a_address' to this list of recipients.
		do
			if a_address /= Void and then not has (a_address) then
				if recipients = Void then
					create recipients.make (1, 0)
				end
				recipients.force (a_address, recipients.upper + 1)
			end
		end

	add_recipients (a_other: ROSE_MAIL_RECIPIENTS) is
			-- Add all recipients from 'a_other'
		local
			i: INTEGER
		do
			if a_other /= Void and then not a_other.is_empty then
				from
					i := a_other.recipients.lower
				until
					i > a_other.recipients.upper
				loop
					add_address (a_other.recipients.item (i))
					i := i + 1
				end
			end
		end

	clear_all is
			-- Remove all recipients
		do
			recipients := Void
		ensure
			is_empty
		end

feature {NONE} -- Implementation

	separator_index (a_string: STRING; a_start: INTEGER): INTEGER is
			-- Return index of first separator in 'a_string' after 'a_start' index
		local
			i: INTEGER
		do
			from
				i := a_start
			until
				i > a_string.count or else Result > 0
			loop
				if a_string.item (i) = ',' or else a_string.item (i) = ';' then
					Result := i
				end
				i := i + 1
			end
		end

end
