indexing
	
	description: "[
		Windows version of objects that list users email adresses.
	]"

	author: "Colin Adams"
	last_editor: "$Author: it $"
	date: "$Date: 2008/12/01 $"
	revision: "$Revision: #2 $"
	path: "$File: //rose/source/kernel/infrastructure/network/windows/rose_mail_users.e $"

class	ROSE_MAIL_USERS

create

	make

feature {NONE} -- Initialization

	make is
			-- Initialize `Current'.
		do
			create domain.make
		end

feature -- Access

	Axa_rosenberg_group: INTEGER is 0
			-- Special group number indicating all AXA Rosenberg employees

	All_users_group: INTEGER is -1
			-- Special group number indicating all users

	all_groups_count: INTEGER is
			-- Groups
		do
			ensure_groups
			Result := groups.count
		ensure
			groups_not_void: groups /= Void
			definition: Result = groups.count
		end

	all_users_count: INTEGER is
			-- All users
		do
			ensure_users
			Result := all_users.count
		ensure
			all_users_not_void: all_users /= Void
			definition: Result = all_users.count
		end

	axa_rosenberg_count: INTEGER is
			-- Users in AXA Rosenberg
		do
			if axa_rosenberg_users = Void then
				axa_rosenberg_users := domain.group (All_axa_rosenberg).users
			end
			Result := axa_rosenberg_users.count
		ensure
			axa_rosenberg_users_not_void: axa_rosenberg_users /= Void
			definition: Result = axa_rosenberg_users.count
		end

	group_address (a_group_index: INTEGER): ROSE_MAIL_ADDRESS is
			-- Mail address for group number `a_group_index'
		require
			valid_group_index: is_valid_group_index (a_group_index)
		local
			l_group_address: STRING
		do
			ensure_groups
			l_group_address ?= groups.i_th (a_group_index).entry.property ("mail")
			if l_group_address /= Void and then not l_group_address.is_empty then
				create Result.make (groups.i_th (a_group_index).name, l_group_address)
			end
		end

	group_name (a_group_index: INTEGER): STRING is
			-- Name for group number `a_group_index'
		require
			valid_group_index: is_valid_group_index (a_group_index)
		do
			ensure_groups
			Result := groups.i_th (a_group_index).name
		ensure
			name_not_void: Result /= Void
		end

	user_address (a_group_index, a_user_index: INTEGER): ROSE_MAIL_ADDRESS is
			-- Mail address for user number `a_user_index' within group number `a_group_index'
		require
			a_group_index_valid: a_group_index = Axa_rosenberg_group or a_group_index = All_users_group or
				is_valid_group_index (a_group_index)
			a_user_index_valid: is_valid_user_index (a_group_index, a_user_index)
		local
			l_user_address, l_name: STRING
		do
			ensure_groups
			if a_group_index = Axa_rosenberg_group then
				l_user_address ?= axa_rosenberg_users.i_th (a_user_index).entry.property ("mail")
				l_name := employee_name (a_user_index)
			elseif a_group_index = All_users_group then
				l_user_address ?= all_users.item (a_user_index).entry.property ("mail")
				l_name := all_users.item (a_user_index).name
			else
				l_user_address ?= users.item (a_group_index).i_th (a_user_index).entry.property ("mail")
				l_name := users.item (a_group_index).i_th (a_user_index).name
			end
			if l_user_address /= Void and then not l_user_address.is_empty then
				create Result.make (l_name, l_user_address)
			end
		end

	user_name (a_group_index, a_user_index: INTEGER): STRING is
			-- Name for user number `a_user_index' in group `a_group_index'
		require
			a_group_index_valid: a_group_index = All_users_group or is_valid_group_index (a_group_index)
			a_user_index_valid: is_valid_user_index (a_group_index, a_user_index)
		do
			if a_group_index = All_users_group then
				Result := all_users.item (a_user_index).name
			else
				ensure_groups
				Result := users.item (a_group_index).i_th (a_user_index).name
			end
		ensure
			name_not_void: Result /= Void
		end

	employee_name (a_user_index: INTEGER): STRING is
			-- Name for group number `a_group_index'
		require
			a_user_index_valid: is_valid_user_index (Axa_rosenberg_group, a_user_index)
		do
			Result := axa_rosenberg_users.i_th (a_user_index).name
		ensure
			name_not_void: Result /= Void
		end

	is_valid_group_index (a_group_index: INTEGER): BOOLEAN is
			-- Does `a_group_index' represent a known group?
		do
			ensure_groups
			Result := a_group_index > 0 and a_group_index <= groups.count
		end

	is_valid_user_index (a_group_index, a_user_index: INTEGER): BOOLEAN is
			-- Does `a_user_index' represent a known user within `a_group_index'?
		require
			a_group_index_valid: a_group_index = Axa_rosenberg_group or a_group_index = All_users_group or
				is_valid_group_index (a_group_index)
		do
			ensure_groups
			if a_user_index < 1 then
				Result := False
			elseif a_group_index = Axa_rosenberg_group then
				Result := axa_rosenberg_users /= Void and then a_user_index <= axa_rosenberg_users.count
			elseif a_group_index = All_users_group then
				Result := all_users /= Void and then a_user_index <= all_users.count
			else
				Result := a_user_index > 0 and users.item (a_group_index).count >= a_user_index
			end
		ensure
			axa_rosenberg_users_not_void: (Result and a_group_index = Axa_rosenberg_group) implies axa_rosenberg_users /= Void
			all_users_not_void: (Result and a_group_index = All_users_group) implies all_users /= Void
		end

	user_count_for_group (a_group_index: INTEGER): INTEGER is
			-- Number of memebrs in `a_group_index'
		require
			valid_group_index: is_valid_group_index (a_group_index)
		do
			ensure_groups
			if not users.has (a_group_index) then
				users.put_new (groups.i_th (a_group_index).users, a_group_index)
			end
			Result := users.item (a_group_index).count
		ensure
			users_cached: users.has (a_group_index)
		end

feature {NONE} -- Implementation

	All_axa_rosenberg: STRING is "Employees_globally"
			-- Group name for all AXA Rosenberg employees

	domain: AD_DOMAIN
			-- Active directory domain

	groups: ARRAYED_LIST [AD_GROUP]
			-- Active Directory group objects

	users: DS_HASH_TABLE [ARRAYED_LIST [AD_USER], INTEGER]
			-- Group members indexed by index of group within `groups'

	axa_rosenberg_users: ARRAYED_LIST [AD_USER]
			-- All employees of AXA Rosenberg

	all_users: ROSE_LINEAR_ARRAY [AD_USER]
			-- All users known to Active Directory

	ensure_groups is
			-- Ensure `groups' and `users' are non-Void.
		do
			if groups = Void then
				groups := domain.groups
				create users.make_map (groups.count)
			end
		ensure
			groups_not_void: groups /= Void
			users_not_void: users /= Void
		end

	ensure_users is
			-- Ensure `all_users' is non-Void.
		do
			if all_users = Void then
				all_users := domain.users
			end
		ensure
			all_users_not_void: all_users /= Void
		end

invariant

	domain_not_void: domain /= Void

end
