indexing
	description: "Object representing an array of ROSE_MONTH where the day is the end of the month ... misguided"
	author : "Tom Mead"
	date: ""
	revision: ""
class
   ROSE_MONTH_SEQUENCE

inherit
   --ROSE_LINEAR_ARRAY[ROSE_MONTH]
   ROSE_DATE_ARRAY [ROSE_MONTH]
create
   make_by_frequency, make_by_months, make_by_quarters, make_by_years, make_from_array,make,make_empty,make_trailing, make_by_length_and_frequency, make_from_dates

feature -- creation
   make_by_months (a_begin_ccyymm:INTEGER; a_end_ccyymm:INTEGER) is
      -- 1 month periodicity
      do
		 make_empty
		 fill_months(a_begin_ccyymm,a_end_ccyymm, 1)
      end
   make_by_quarters (a_begin_ccyymm:INTEGER; a_end_ccyymm:INTEGER) is
      -- every 3 months from begin ccyymm
      do
		  make_empty
		  fill_months(a_begin_ccyymm,a_end_ccyymm, 3)
      end
   make_by_years (a_begin_ccyymm:INTEGER; a_end_ccyymm:INTEGER) is
      -- every twelve months from begin ccyymm
      do
	 	make_empty
	  	fill_months(a_begin_ccyymm,a_end_ccyymm, 12)
      end
   make_trailing (a_month  : ROSE_MONTH;a_trailing_months : INTEGER) is
	require
		a_trailing_months > 1
	do
		make_by_months(a_month.future_month(1-a_trailing_months).ccyymm,a_month.ccyymm)
	end
   make_by_frequency(a_begin_ccyymm:INTEGER; a_end_ccyymm:INTEGER; a_months_per_period:INTEGER) is
		-- make with frequency
      do
	 	make_empty
	 	fill_months(a_begin_ccyymm,a_end_ccyymm, a_months_per_period)
      end

	make_by_length_and_frequency(a_begin_ccyymm: INTEGER; a_length_in_periods: INTEGER; a_months_per_period: INTEGER) is
			--
		local
			l_end_ccyymm: INTEGER
			l_julian: INTEGER
			l_date: ROSE_MONTH
		do
			make_empty
			create l_date.make_ccyymm(a_begin_ccyymm)
			l_julian:= (l_date.julian_month + a_length_in_periods* a_months_per_period) - 1
			l_end_ccyymm:= (create {ROSE_MONTH}.make_from_julian_month(l_julian)).ccyymm
			fill_months(a_begin_ccyymm, l_end_ccyymm, a_months_per_period)
		end

	make_from_dates(a_dates : ARRAY[ROSE_DATE]) is
		local
			i : INTEGER
			l_date : ROSE_DATE
			l_month : ROSE_MONTH
		do
			if a_dates /= Void then
				make(1, a_dates.count)
				from i := 1 until i > a_dates.count loop
					l_date := a_dates.item(a_dates.lower + i - 1)
					create l_month.make_ccyymm(l_date.ccyymm)
					put(l_month, i)
					i := i + 1
				end
			end
		end

feature  -- internal
   fill_months(a_begin_ccyymm:INTEGER; a_end_ccyymm:INTEGER; a_months_per_period:INTEGER) is
      local
		 l_month: ROSE_MONTH
		 l_month_end: ROSE_MONTH
		 i: INTEGER
		 l_julian_month_now, l_julian_month_end,l_julian_month: INTEGER
	 	 l_number_of_dates:INTEGER
      do
			 l_julian_month_now:= (create{ROSE_DATE}.make_today).julian_month
			 create l_month.make_ccyymm(a_begin_ccyymm)
			 create l_month_end.make_ccyymm(a_end_ccyymm)
			 l_julian_month_end:= l_month_end.julian_month
			 l_julian_month:= l_month.julian_month
			 l_number_of_dates:= ((l_julian_month_end - l_julian_month)// a_months_per_period) + 1
		--	 if l_julian_month_end = l_julian_month_now then
		--	    l_number_of_dates:= l_number_of_dates -1
		--	 end
			 from i := 1 until i > l_number_of_dates
			 loop
			    create l_month.make_from_julian_month(l_julian_month)
			    extend(l_month)
			    l_julian_month := l_julian_month + a_months_per_period
			    i := i+ 1
			 end
      end

	set_backwards is
		do
			sort_by(agent compare_dates(?,?))
		end

	compare_dates(a,b:ROSE_MONTH) :BOOLEAN is
			do
				result:=a>b
			end

	months_label : ROSE_LINEAR_ARRAY[STRING] is
		do
			create result.make_empty
			from start until off
			loop
				Result.extend(current_item.ccyymmdd.out)
				forth
			end
		end

feature   --transformation
	monthend_dates: ROSE_LINEAR_ARRAY[ROSE_DATE] is
			-- provides array of monthend dates (rose_dates) rather than rose_months
			require
				not_empty: count > 0
			do
				create Result.make_empty
				from start until off loop
					Result.extend(current_item.date)
					forth
				end
			ensure
				same_size: Result.count = count
			end

end





