note

	description:

		"Hash-table with items identified by TUPLEs"

	author: "Mark Howard"
	last_editor: "$Author: er $"
	date: "$Date: 2013/06/18 $"
	revision: "$Revision: #13 $"
	path: "$File: //rose/source/kernel/infrastructure/containers/rose_multi_keyed_table.e $"
	storable_version: ""

class
	ROSE_MULTI_KEYED_TABLE [G]
create
	make

feature -- Initialization

	make (a_count: INTEGER)
			--Creation
		do
			create hash_table.make (a_count)
		end

feature -- Access

	do_all (a_agent: PROCEDURE [ANY, TUPLE [G]]) is
			-- Call 'a_agent' on all item in this hash table
		require
			agent_ok: a_agent /= Void
		do
			from
				start
			until
				off
			loop
				a_agent.call ([item_for_iteration])
				forth
			end
		end

	has (a_key: TUPLE): BOOLEAN is
			-- Is there an item in the table with key `a_key'?
		local
			l_item_tuple: TUPLE_KEY
		do
			create l_item_tuple.make (a_key)
			Result := hash_table.has (l_item_tuple)
		end

	item (a_key: TUPLE): G is
			-- Item associated with `a_key', if present
			-- otherwise default value of type `G'
		local
			l_item_tuple: TUPLE_KEY
		do
			create l_item_tuple.make (a_key)
			Result := hash_table.item (l_item_tuple)
		end

	found: BOOLEAN is
		do
			Result := hash_table.found
		end

	found_item: G is
		do
			Result := hash_table.found_item
		end

	count: INTEGER is
		do
			Result := hash_table.count
		end

	off: BOOLEAN is
		do
			Result := hash_table.off
		end

	item_for_iteration: G is
		do
			Result := hash_table.item_for_iteration
		end

	key_for_iteration: TUPLE is
		do
			Result := hash_table.key_for_iteration.tuple
		end

feature -- Basic operations

	put (a_g: G; a_key: TUPLE) is
		do
			hash_table.put (a_g, create {TUPLE_KEY}.make (a_key))
		end

	force (a_g: G; a_key: TUPLE) is
		do
			hash_table.force (a_g,create {TUPLE_KEY}.make (a_key))
		end

	search (a_key: TUPLE) is
		local
			l_item_tuple: TUPLE_KEY
		do
			create l_item_tuple.make (a_key)
			hash_table.search (l_item_tuple)
		end

	start is
		do
			hash_table.start
		end

	forth is
		do
			hash_table.forth
		end

	remove (a_key: TUPLE) is
		local
			l_item_tuple: TUPLE_KEY
		do
			create l_item_tuple.make (a_key)
			hash_table.remove (l_item_tuple)
		end

	remove_subset(a_subset_fun : FUNCTION[ANY, TUPLE[TUPLE_KEY], BOOLEAN]) is
		local
			l_keys : ROSE_LINEAR_ARRAY[TUPLE_KEY]
		do
			if a_subset_fun /= Void then
				create l_keys.make_empty
				from hash_table.start until hash_table.off loop
					if a_subset_fun.item([hash_table.key_for_iteration]) then
						l_keys.extend(hash_table.key_for_iteration)
					end
					hash_table.forth
				end
				from l_keys.start until l_keys.off loop
					hash_table.remove(l_keys.current_item)
					l_keys.forth
				end
			end
		end

	wipe_out, clear_all is
		do
			hash_table.clear_all
		end

	linear_representation: ARRAYED_LIST[G] is
			-- Linear representation of `hash_table'
		do
			Result := hash_table.linear_representation
		end

	merge (a_other : like Current) is
			-- Merge
		do
			hash_table.merge (a_other.hash_table)
		end


feature {ROSE_MULTI_KEYED_TABLE} -- Implementation

	hash_table: HASH_TABLE [G,TUPLE_KEY]

end
