indexing

	description: "Enhanced NETWORK_STREAM_SOCKET."

	author: "Randy John"
	last_editor: "$Author: er $"
	date: "$Date: 2013/09/19 $"
	revision: "$Revision: #26 $"
	path: "$File: //rose/source/kernel/infrastructure/network/rose_network_stream_socket.e $"

class	ROSE_NETWORK_STREAM_SOCKET

inherit

	NETWORK_STREAM_SOCKET
		redefine
			set_delay,
			set_nodelay,
			retrieved,
			independent_store,
			read_to_managed_pointer,
			make_server_by_port,
			create_from_descriptor,
			make_client_by_port,
			listen,
			accept,
			readable,
			maximum_seg_size,
			is_linger_on,
			is_out_of_band_inline,
			linger_time,
			set_linger_on,
			set_linger_off,
			set_out_of_band_inline,
			set_out_of_band_not_inline,
			basic_store,
			general_store,
			bind,
			connect,
			close,
			close_socket,
			putstring,
			put_string,
			put_managed_pointer,
			put_character,
			putchar,
			putreal,
			put_real,
			putdouble,
			put_double,
			put_integer,
			putint,
			put_boolean,
			putbool,
			write,
			send,
			read_real,
			readreal,
			read_double,
			readdouble,
			read_character,
			readchar,
			read_integer,
			readint,
			read_stream,
			readstream,
			read,
			receive,
			enable_debug,
			disable_debug,
			debug_enabled,
			do_not_route,
			route,
			route_enabled,
			set_receive_buf_size,
			receive_buf_size,
			set_send_buf_size,
			send_buf_size,
			is_socket_stream,
			set_non_blocking,
			set_blocking,
			set_owner,
			is_group_id,
			is_process_id,
			group_id,
			process_id,
			exists
		end

	MOVIE_CAMERA_AWARE
		export
			{NONE} all
		end

	EXECUTION_ENVIRONMENT
		export
			{NONE} all
		end

	DEBUG_OUTPUT

create

	make_server_by_port,
	make_client_by_port,
	create_from_descriptor

create {NETWORK_STREAM_SOCKET}
	make_from_descriptor_and_address

feature -- Initialization

	make_server_by_port (a_port: INTEGER)
			-- Create server socket on `a_port'.
		do
			debug ("make_movies")
				create cam.make (generating_type + "/" + a_port.out)
			end
			Precursor (a_port)
			set_default_socket_options
		end

	create_from_descriptor (a_fd: INTEGER)
			-- Create socket from descriptor `fd'.
		local
			l_six: SIX_TUPLE [INTEGER, INTEGER, INTEGER, BOOLEAN, BOOLEAN, BOOLEAN]
		do
			debug ("make_movies")
				create cam.make (generating_type + "/" + "fd=" + a_fd.out)
			end
			if playback_movie then
				l_six ?= cam.next_frame ("create_from_descriptor").frame_data
				timeout := l_six.first
--				descriptor := l_six.second
				cam.compare_partial_movie_frame (descriptor, a_fd)
				family := l_six.third
				descriptor_available := l_six.fourth
				is_open_read := l_six.fifth
				is_open_write := l_six.sixth
			else
				Precursor (a_fd)
				debug ("make_movies")
					if record_movie then
						create l_six.make (timeout, descriptor, family, descriptor_available, is_open_read, is_open_write)
						cam.record_movie_frame ("create_from_descriptor", l_six)
					end
				end
			end
		end

	make_client_by_port (a_peer_port: INTEGER; a_peer_host: STRING)
			-- Create a client connection to `a_peer_host' on
			-- `a_peer_port'.
		do
			debug ("make_movies")
				create cam.make (generating_type + "/" + "peer_port=" + a_peer_port.out + "/" + a_peer_host)
			end
			Precursor (a_peer_port, a_peer_host)
			set_default_socket_options
		end

	read_to_managed_pointer (p: MANAGED_POINTER; a_start_pos, a_nb_bytes: INTEGER)
			-- ISE's implementation loops forever if the socket is closed (as of 5.7.0414).
		local
			l_read: INTEGER
			l_bytes_read: INTEGER
			l_triple: TRIPLE [INTEGER, INTEGER, MOVIE_MANAGED_POINTER]
			l_movie_managed_pointer: MOVIE_MANAGED_POINTER
		do
			if playback_movie then
				l_triple ?= cam.next_frame ("read_to_managed_pointer").frame_data
				cam.compare_partial_movie_frame (l_triple.first, a_start_pos)
				cam.compare_partial_movie_frame (l_triple.second, a_nb_bytes)
				l_triple.third.copy_to_managed_pointer (p)
			else
				from
				until
					l_read = a_nb_bytes
				loop
					l_bytes_read := c_read_stream (descriptor, a_nb_bytes - l_read, p.item + a_start_pos + l_read)
					if l_bytes_read = 0 then
						raise ("Socket was closed")
					end
					l_read := l_read + l_bytes_read
				end
				bytes_read := l_read
				debug ("make_movies")
					if record_movie then
						l_movie_managed_pointer ?= cam.safe_deep_clone (p)
						create l_triple.make (a_start_pos, a_nb_bytes, l_movie_managed_pointer)
						cam.record_movie_frame ("read_to_managed_pointer", l_triple)
					end
				end
			end
		end

	independent_store (a_any: ANY)
			-- Produce an external representation of the
			-- entire object structure reachable from `a_any'.
			-- Retrievable from other systems for the same or other
			-- platform (machine architecture).
		local
			l_writer: SED_MEDIUM_READER_WRITER
		do
			if playback_movie then
				cam.compare_movie_frame ("independent_store", a_any)
			else
				if {PLATFORM}.is_dotnet and storable_type = storable_storable
				then
					storable_type := storable_sed
					send_auto_detect_header
				elseif add_auto_detect_header
				then
					send_auto_detect_header
				end

				inspect storable_type
				when storable_storable then
					Precursor (a_any)
				when storable_sed then
					create l_writer.make (Current)
					l_writer.set_for_writing
					sed.store (a_any, l_writer)
				end
			end
			debug ("make_movies")
				if cam.record_movie then
					cam.record_movie_frame ("independent_store", a_any)
				end
			end
		end

	send_auto_detect_header
	-- Sends a message (not in any Eiffel storable format) that indicates the type
	-- of storable to use (C or SED).
	do
		add_auto_detect_header := False			-- We only add this to the first message.
		set_storable_type_in_auto_detect_header
		send(auto_detect_header_packet, 0)
	end

	retrieved: ANY
			-- Retrieved object structure
			-- To access resulting object under correct type,
			-- use assignment attempt.
			-- Will raise an exception (code Retrieve_exception)
			-- if content is not a stored Eiffel structure.
		local
			l_reader: SED_MEDIUM_READER_WRITER
			l_was_blocking: BOOLEAN
			l_movie_data: ANY
			l_exception: ROSE_EXCEPTION_INSTANCE
			l_movie_frame_number: INTEGER
			l_error: SED_ERROR
			l_peek_data: MANAGED_POINTER
			l_auto_detected_type: NATURAL_8
		do
			if playback_movie then
				inspect storable_type
				when storable_storable then
					l_movie_data := cam.next_frame ("retrieved").frame_data
					cam.compare_not_needed
					l_exception ?= l_movie_data
					if l_exception /= Void then
						cam.raise_exception ("movie exception in ROSE_NETWORK_STREAM_SOCKET")
					end
					l_was_blocking := is_blocking
					set_blocking
					if not l_was_blocking then
						set_non_blocking
					end
				when storable_sed then
					l_movie_data := cam.next_frame ("retrieved").frame_data
					cam.compare_not_needed
					l_exception ?= l_movie_data
					if l_exception /= Void then
						raise ("movie exception in ROSE_NETWORK_STREAM_SOCKET")
					end
				end
				Result ?= l_movie_data
			else
				debug ("make_movies")
					if record_movie then
						cam.record_movie_frame ("retrieved", create {ROSE_EXCEPTION_INSTANCE})
						l_movie_frame_number := cam.current_frame_number
					end
				end

				if not skip_auto_detect and auto_detect_enabled
				then
					-- Here we check to see if the client has prefixed his first storable message with our 'auto_detect_header'.
					-- If he has then we can determine what storable type (C or SED) will follow.  If he hasn't then no
					-- harm is done as we only peek into the input buffer.
					skip_auto_detect := True							-- We only do the detection on the first message.
					l_peek_data := peek(auto_detect_header.count)		-- Peek at the socket's input buffer.

					if has_auto_detect_header_prefix(l_peek_data)		-- Check to see if 'l_peek_data' looks like the first part of 'auto_detect_header.'.
					then
						-- We are here because the client sent the auto detect header.  The last byte indicates the storable type.
						l_auto_detected_type := l_peek_data.read_natural_8(auto_detect_header.count - 1)
						if l_auto_detected_type = storable_storable or l_auto_detected_type = storable_sed
						then
							storable_type := l_auto_detected_type
							readstream(auto_detect_header.count)		-- Skip over our 'auto_detect_header'.
						end
					end
				end

				inspect storable_type
				when storable_storable then
					Result := Precursor
				when storable_sed then
					create l_reader.make (Current)
					l_reader.set_for_reading
					Result := sed.retrieved (l_reader, mem.collecting)
					if Result = Void then
						l_error := sed.retrieved_error
						if l_error /= Void then
							raise_retrieval_exception ("SED returned Void. Error was: " + l_error.error)
						else
							raise_retrieval_exception ("SED returned Void. No error information, so assume a bad storable type.")
						end
					end
				end
				debug ("make_movies")
					if record_movie then
						cam.rerecord_movie_frame ("retrieved", Result, l_movie_frame_number)
					end
				end
			end
		end

	has_sed_retrieved_error: BOOLEAN is
			-- Was a SED serialization error reported during the last call to `retrieved'?
		require
			uses_sed: is_sed_used
		do
			Result := sed.retrieved_error /= Void
		end

	sed_retrieved_error: STRING is
			-- Last serialization error reported by SED when executing `retrieved'
			-- If no errror, an empty STRING is returned.
		require
			uses_sed: is_sed_used
		do
			if sed.retrieved_error /= Void then
				Result := sed.retrieved_error.error
			else
				Result := ""
			end
		ensure
			result_not_void: Result /= Void
			result_empty_if_no_error: sed.retrieved_error = Void implies Result.is_equal (once "")
		end

	use_storable
			-- Use classic storables
		do
			storable_type := storable_storable
		end

	use_sed
			-- Use SED storables
		do
			storable_type := storable_sed
		end

	skip_auto_detection
	-- Call this if (for some crazy reason) you do not want to auto detect the received storable type.
	do
		skip_auto_detect := True
	end

	use_auto_detect_header
	-- Causes the auto detect header to be added to the first message sent so that the recipient can determine the storable type.
	do
		add_auto_detect_header := True
	end

	is_sed_used: BOOLEAN is
			-- Are SED storables used?
		do
			Result := storable_type = storable_sed
		end

	is_storable_used: BOOLEAN is
			-- Are classic storables used?
		do
			Result := storable_type = storable_storable
		end

	set_delay
			-- set TCP_NO_DELAY option off. ISE does this backwards before 5.4
		do
			if playback_movie then
				cam.compare_boolean_movie_frame ("set_delay", False)
			else
				c_set_sock_opt_int (descriptor, level_iproto_tcp, tcpno_delay, 0)
				debug ("make_movies")
					if record_movie then
						cam.record_boolean_movie_frame ("set_delay", False)
					end
				end
			end
		ensure then
			delay_is_set: is_delay_on
		end

	set_nodelay
			-- set TCP_NO_DELAY option on. ISE does this backwards before 5.4
		do
			if playback_movie then
				cam.compare_boolean_movie_frame ("set_nodelay", True)
			else
				c_set_sock_opt_int (descriptor, level_iproto_tcp, tcpno_delay, 1)
				debug ("make_movies")
					if record_movie then
						cam.record_boolean_movie_frame ("set_nodelay", True)
					end
				end
			end
		ensure then
			no_delay_is_set: not is_delay_on
		end

	is_delay_on: BOOLEAN
			-- is socket delay mode on (i.e. is TCP_NO_DELAY option off)?
		require
			socket_exists: exists
		local
			l_nodelay: INTEGER
		do
			if playback_movie then
				Result := cam.next_frame ("is_delay_on").frame_data_as_boolean
				cam.compare_not_needed
			else
				l_nodelay := c_get_sock_opt_int (descriptor, level_iproto_tcp, tcpno_delay)
				Result := not l_nodelay.to_boolean
				debug ("make_movies")
					if record_movie then
						cam.record_boolean_movie_frame ("is_delay_on", Result)
					end
				end
			end
		end

	set_receive_timeout (a_timeout_milliseconds: INTEGER)
			-- If a 'receive' does not complete in 'a_timeout_milliseconds', an exception will be raised.
			-- The tag is "Net error #10060".
		do
			c_set_sock_opt_int (descriptor, level_sol_socket, so_rcv_timeo, a_timeout_milliseconds)
		end

	set_send_timeout (a_timeout_milliseconds: INTEGER)
			-- If a 'send' does not complete in 'a_timeout_milliseconds', an exception will be raised.
			-- The tag is "Net error #10060".
		do
			c_set_sock_opt_int (descriptor, level_sol_socket, so_snd_timeo, a_timeout_milliseconds)
		end

	receive_timeout: INTEGER
			-- The socket timeout (in milliseconds) used during 'receive'.
		do
			Result := c_get_sock_opt_int (descriptor, level_sol_socket, so_rcv_timeo)
		end

	send_timeout: INTEGER
			-- The socket timeout (in milliseconds) used during 'send'.
		do
			Result := c_get_sock_opt_int (descriptor, level_sol_socket, so_snd_timeo)
		end

	set_default_socket_options
			-- Set socket options if the user has set certain logicals/environment_variables.
		local
			l_string: STRING
			l_old_value: INTEGER
			l_new_value: INTEGER
			l_desired_value: INTEGER
		do
			l_string := get ("TCP_NODELAY")
			if l_string /= Void and then l_string.is_integer then
				if l_string.to_integer /= 0 then
					print ("Setting port " + port_number_string + ", TCP_NODELAY to on.%N")
					set_nodelay
				else
					print ("Setting port " + port_number_string + ", TCP_NODELAY to off.%N")
					set_delay
				end
			end
			l_string := get ("SO_RCVBUF")
			if l_string /= Void then
				if l_string.is_integer then
					l_old_value := receive_buf_size
					l_desired_value := l_string.to_integer
					set_receive_buf_size (l_desired_value)
					l_new_value := receive_buf_size
					if l_new_value /= l_desired_value then
						print ("Failed to set SO_RCVBUF on port " + port_number_string + ", old/desired/actual " + l_old_value.out + "/" + l_desired_value.out + "/" + l_new_value.out + "%N")
					end
				else
					print ("Could not understand your SO_RCVBUF value (" + l_string + ") for port " + port_number_string + ".%N")
				end
			end
			l_string := get ("SO_SNDBUF")
			if l_string /= Void then
				if l_string.is_integer then
					print_socket_option ("SO_SNDBUF")
					print ("Setting port " + port_number_string + ", SO_SNDBUF to " + l_string + ".%N")
					set_send_buf_size (l_string.to_integer)
					print_socket_option ("SO_SNDBUF")
				else
					print ("Could not understand your SO_SNDBUF value for port " + port_number_string + ".%N")
				end
			end
		end

	print_socket_option (a_option: STRING)
			-- Print socket option
		local
			l_value: STRING
		do
			if a_option.is_equal ("SO_RCVBUF") then
				l_value := receive_buf_size.out
			elseif a_option.is_equal ("SO_SNDBUF") then
				l_value := send_buf_size.out
			else
				l_value := "?"
			end
			print ("Port " + port_number_string + ":" + a_option + " = " + l_value + ".%N")
		end

	debug_output: STRING
			-- String that should be displayed in debugger to represent `Current'.
		do
			if not playback_movie and not record_movie then
				Result := port_number_string
			end
		end

	port_number_string: STRING
			-- if connected then remote port number, else local (bind) port number followed by "?"
		local
			l_retry: BOOLEAN
		do
			if playback_movie then
				Result ?= cam.next_frame ("port_number_string").frame_data
			else
				if not l_retry then
					if peer_address /= Void then
						Result := peer_address.port.out
					else
						Result := port.out + "?"
					end
				else
					Result := "error"
				end
				debug ("make_movies")
					if record_movie then
						cam.record_movie_frame ("port_number_string", Result)
					end
				end
			end
		ensure
			exists: Result /= Void
		rescue
			l_retry := True
			retry
		end

	release_accepted
	-- Use this when you want a GC to collect 'accepted'.
	do
		accepted := Void
	end

feature -- Features that are overridden during 'make_movies'.

	listen (a_queue: INTEGER)
			-- Listen on socket for at most `queue' connections.
		do
			if playback_movie then
				cam.compare_integer_movie_frame ("listen", a_queue)
			else
				Precursor (a_queue)
				debug ("make_movies")
					if record_movie then
						cam.record_integer_movie_frame ("listen", a_queue)
					end
				end
			end
		end

	accept
			-- Accept a new connection on listen socket.
			-- Accepted service socket available in accepted.
		local
			l_descriptor: INTEGER
		do
			if playback_movie then
				l_descriptor := cam.next_frame ("accept").frame_data_as_integer
				cam.compare_not_needed
				create accepted.create_from_descriptor (l_descriptor)
			else
				Precursor
				debug ("make_movies")
					if record_movie then
						cam.record_integer_movie_frame ("accept", accepted.descriptor)
					end
				end
			end
		end

	readable: BOOLEAN
			-- Is there a current item that may be read?
		do
			if playback_movie then
				Result := cam.next_frame ("readable").frame_data_as_boolean
				cam.compare_not_needed
			else
				Result := Precursor
				debug ("make_movies")
					if record_movie then
						cam.record_boolean_movie_frame ("readable", Result)
					end
				end
			end
		end

	maximum_seg_size: INTEGER
			-- Maximum segment size
		do
			if playback_movie then
				Result := cam.next_frame ("maximum_seg_size").frame_data_as_integer
				cam.compare_not_needed
			else
				Result := Precursor
				debug ("make_movies")
					if record_movie then
						cam.record_integer_movie_frame ("maximum_seg_size", Result)
					end
				end
			end
		end

	is_linger_on: BOOLEAN
			-- Is lingering switched on?
		do
			if playback_movie then
				Result := cam.next_frame ("is_linger_on").frame_data_as_boolean
				cam.compare_not_needed
			else
				Result := Precursor
				debug ("make_movies")
					if record_movie then
						cam.record_boolean_movie_frame ("is_linger_on", Result)
					end
				end
			end
		end

	is_out_of_band_inline: BOOLEAN
			-- Are out-of-band packets sent inline?
		do
			if playback_movie then
				Result := cam.next_frame ("is_out_of_band_inline").frame_data_as_boolean
				cam.compare_not_needed
			else
				Result := Precursor
				debug ("make_movies")
					if record_movie then
						cam.record_boolean_movie_frame ("is_out_of_band_inline", Result)
					end
				end
			end
		end

	linger_time: INTEGER
			-- Linger time
		do
			if playback_movie then
				Result := cam.next_frame ("linger_time").frame_data_as_integer
				cam.compare_not_needed
			else
				Result := Precursor
				debug ("make_movies")
					if record_movie then
						cam.record_integer_movie_frame ("linger_time", Result)
					end
				end
			end
		end

	set_linger_on (a_time: INTEGER)
			-- Switch lingering on and set linger time to `time'.
		do
			if playback_movie then
				cam.compare_integer_movie_frame ("set_linger_on", a_time)
			else
				Precursor (a_time)
				debug ("make_movies")
					if record_movie then
						cam.record_integer_movie_frame ("set_linger_on", a_time)
					end
				end
			end
		end

	set_linger_off
			-- Switch lingering off.
		do
			if playback_movie then
				cam.compare_boolean_movie_frame ("set_linger_off", False)
			else
				Precursor
				debug ("make_movies")
					if record_movie then
						cam.record_boolean_movie_frame ("set_linger_off", False)
					end
				end
			end
		end

	set_out_of_band_inline
			-- Switch "out of band packets inline" on.
		do
			if playback_movie then
				cam.compare_boolean_movie_frame ("set_out_of_band_inline", True)
			else
				Precursor
				debug ("make_movies")
					if record_movie then
						cam.record_boolean_movie_frame ("set_out_of_band_inline", True)
					end
				end
			end
		end

	set_out_of_band_not_inline
			-- Switch "out of band packets inline" off.
		do
			if playback_movie then
				cam.compare_boolean_movie_frame ("set_out_of_band_not_inline", False)
			else
				Precursor
				debug ("make_movies")
					if record_movie then
						cam.record_boolean_movie_frame ("set_out_of_band_not_inline", False)
					end
				end
			end
		end

	basic_store (a_object: ANY)
			-- Produce an external representation of the
			-- entire object structure reachable from `object'.
			-- Retrievable within current system only.
		do
			if playback_movie then
				cam.compare_movie_frame ("basic_store", a_object)
			else
				Precursor (a_object)
				debug ("make_movies")
					if record_movie then
						cam.record_movie_frame ("basic_store", a_object)
					end
				end
			end
		end

	general_store (a_object: ANY)
			-- Produce an external representation of the
			-- entire object structure reachable from `object'.
			-- Retrievable from other systems for same platform
			-- (machine architecture).
		do
			if playback_movie then
				cam.compare_movie_frame ("general_store", a_object)
			else
				Precursor (a_object)
				debug ("make_movies")
					if record_movie then
						cam.record_movie_frame ("general_store", a_object)
					end
				end
			end
		end

	bind
			-- Bind socket to local address in address.
		do
			if playback_movie then
				is_open_read := cam.next_frame ("bind").frame_data_as_boolean
				cam.compare_not_needed
			else
				Precursor
				debug ("make_movies")
					if record_movie then
						cam.record_boolean_movie_frame ("bind", is_open_read)
					end
				end
			end
		end

	connect
			-- Connect socket to peer address.
		local
			l_pair: PAIR [BOOLEAN, BOOLEAN]
		do
			if playback_movie then
				l_pair ?= cam.next_frame ("connect").frame_data
				is_open_read := l_pair.first
				is_open_write := l_pair.second
				cam.compare_not_needed
			else
				Precursor
				debug ("make_movies")
					if record_movie then
						create l_pair.make (is_open_read, is_open_write)
						cam.record_movie_frame ("connect", l_pair)
					end
				end
			end
		end

	close
			-- Close medium.
		do
			if playback_movie then
				if is_open_read or is_open_write then
					close_socket
				end
			else
				Precursor
			end
		end

	close_socket
			-- Close socket for current context.
		do
			if playback_movie then
--				descriptor := -2
				descriptor_available := False
				is_open_read := False
				is_open_write := False
			else
				Precursor
			end
		end

	put_string (s: STRING)
			-- Write `s' to medium.
		do
			putstring (s)
		end

	putstring (s: STRING)
			-- Write `s' to medium.
		do
			if playback_movie then
				cam.compare_movie_frame ("putstring", s)
			else
				Precursor (s)
				debug ("make_movies")
					if record_movie then
						cam.record_movie_frame ("putstring", s)
					end
				end
			end
		end

	put_managed_pointer (a_p: MANAGED_POINTER; a_start_pos, a_nb_bytes: INTEGER)
			-- Put data of length `nb_bytes' pointed by `start_pos' index in `p' at
			-- current position.
		local
			l_triple: TRIPLE [MOVIE_MANAGED_POINTER, INTEGER, INTEGER]
			l_movie_mp: MOVIE_MANAGED_POINTER
		do
			if playback_movie then
				l_triple ?= cam.next_frame ("put_managed_pointer").frame_data
				l_movie_mp ?= cam.safe_deep_clone (a_p)
				cam.compare_partial_movie_frame (l_movie_mp, l_triple.first)
				cam.compare_partial_movie_frame (a_start_pos, l_triple.second)
				cam.compare_partial_movie_frame (a_nb_bytes, l_triple.third)
			else
				Precursor (a_p, a_start_pos, a_nb_bytes)
				debug ("make_movies")
					if record_movie then
						l_movie_mp ?= cam.safe_deep_clone (a_p)
						create l_triple.make (l_movie_mp, a_start_pos, a_nb_bytes)
						cam.record_movie_frame ("put_managed_pointer", l_triple)
					end
				end
			end
		end

	put_character (c: CHARACTER)
			-- Write `c' to medium.
		do
			putchar (c)
		end

	putchar (c: CHARACTER)
			-- Write `c' to medium.
		do
			if playback_movie then
				cam.compare_character_movie_frame ("putchar", c)
			else
				Precursor (c)
				debug ("make_movies")
					if record_movie then
						cam.record_character_movie_frame ("putchar", c)
					end
				end
			end
		end

	put_real (r: REAL)
			-- Write `r' to medium.
		do
			putreal (r)
		end

	putreal (r: REAL)
			-- Write `r' to medium.
		do
			if playback_movie then
				cam.compare_real_movie_frame ("putreal", r)
			else
				Precursor (r)
				debug ("make_movies")
					if record_movie then
						cam.record_real_movie_frame ("putreal", r)
					end
				end
			end
		end

	put_integer (i: INTEGER)
			-- Write `i' to medium.
		do
			putint (i)
		end

	putint (i: INTEGER)
			-- Write `i' to medium.
		do
			if playback_movie then
				cam.compare_integer_movie_frame ("putint", i)
			else
				Precursor (i)
				debug ("make_movies")
					if record_movie then
						cam.record_integer_movie_frame ("putint", i)
					end
				end
			end
		end

	put_double (d: DOUBLE)
			-- Write `d' to medium.
		do
			putdouble (d)
		end

	putdouble (d: DOUBLE)
			-- Write `d' to medium.
		do
			if playback_movie then
				cam.compare_double_movie_frame ("putdouble", d)
			else
				Precursor (d)
				debug ("make_movies")
					if record_movie then
						cam.record_double_movie_frame ("putdouble", d)
					end
				end
			end
		end

	put_boolean (b: BOOLEAN)
			-- Write `b' to medium.
		do
			putbool (b)
		end

	putbool (b: BOOLEAN)
			-- Write `b' to medium.
		do
			if playback_movie then
				cam.compare_boolean_movie_frame ("putbool", b)
			else
				Precursor (b)
				debug ("make_movies")
					if record_movie then
						cam.record_boolean_movie_frame ("putbool", b)
					end
				end
			end
		end

	write (a_packet: PACKET)
			-- Write packet `a_packet' to socket.
		local
			l_movie_mp: MOVIE_MANAGED_POINTER
		do
			if playback_movie then
				l_movie_mp ?= cam.safe_deep_clone (a_packet.data)
				cam.compare_movie_frame ("write", l_movie_mp)
			else
				Precursor (a_packet)
				debug ("make_movies")
					if record_movie then
						l_movie_mp ?= cam.safe_deep_clone (a_packet.data)
						cam.record_movie_frame ("write", l_movie_mp)
					end
				end
			end
		end

	send (a_packet: PACKET; flags: INTEGER)
			-- Send a packet `a_packet' of data to socket.
		local
			l_movie_mp: MOVIE_MANAGED_POINTER
			l_pair: PAIR [MOVIE_MANAGED_POINTER, INTEGER]
		do
			if playback_movie then
				l_pair ?= cam.next_frame ("send").frame_data
				l_movie_mp ?= cam.safe_deep_clone (a_packet.data)
				cam.compare_partial_movie_frame (l_movie_mp, l_pair.first)
				cam.compare_partial_movie_frame (flags, l_pair.second)
			else
				Precursor (a_packet, flags)
				debug ("make_movies")
					if record_movie then
						l_movie_mp ?= cam.safe_deep_clone (a_packet.data)
						create l_pair.make (l_movie_mp, flags)
						cam.record_movie_frame ("send", l_pair)
					end
				end
			end
		end

	read_real
			-- Read a new real.
			-- Make result available in last_real.
		do
			readreal
		end

	readreal
			-- Read a new real.
			-- Make result available in last_real.
		do
			if playback_movie then
				last_real := cam.next_frame ("readreal").frame_data_as_real
				cam.compare_not_needed
			else
				Precursor
				debug ("make_movies")
					if record_movie then
						cam.record_real_movie_frame ("readreal", last_real)
					end
				end
			end
		end

	read_double
			-- Read a new double.
			-- Make result available in last_double.
		do
			readdouble
		end

	readdouble
			-- Read a new double.
			-- Make result available in last_double.
		do
			if playback_movie then
				last_double := cam.next_frame ("readdouble").frame_data_as_double
				cam.compare_not_needed
			else
				Precursor
				debug ("make_movies")
					if record_movie then
						cam.record_double_movie_frame ("readdouble", last_double)
					end
				end
			end
		end

	read_character
			-- Read a new character.
			-- Make result available in last_character.
		do
			readchar
		end

	readchar
			-- Read a new character.
			-- Make result available in last_character.
		do
			if playback_movie then
				last_character := cam.next_frame ("readchar").frame_data_as_character
				cam.compare_not_needed
			else
				Precursor
				debug ("make_movies")
					if record_movie then
						cam.record_character_movie_frame ("readchar", last_character)
					end
				end
			end
		end

	read_integer
			-- Read a new 32-bit integer.
			-- Make result available in last_integer.
		do
			readint
		end

	readint
			-- Read a new 32-bit integer.
			-- Make result available in last_integer.
		do
			if playback_movie then
				last_integer := cam.next_frame ("readint").frame_data_as_integer
				cam.compare_not_needed
			else
				Precursor
				debug ("make_movies")
					if record_movie then
						cam.record_integer_movie_frame ("readint", last_integer)
					end
				end
			end
		end

	read_stream (nb_char: INTEGER)
			-- Read a string of at most `nb_char' bound characters
			-- or until end of medium is encountered.
			-- Make result available in last_string.
		do
			readstream (nb_char)
		end

	readstream (nb_char: INTEGER)
			-- Read a string of at most `nb_char' bound characters
			-- or until end of medium is encountered.
			-- Make result available in last_string.
		local
			l_pair: PAIR [INTEGER, STRING]
		do
			if playback_movie then
				l_pair ?= cam.next_frame ("readstream").frame_data
				cam.compare_partial_movie_frame (l_pair.first, nb_char)
				last_string := l_pair.second
			else
				Precursor (nb_char)
				debug ("make_movies")
					if record_movie then
						create l_pair.make (nb_char, last_string)
						cam.record_movie_frame ("readstream", l_pair)
					end
				end
			end
		end

	read (size: INTEGER): PACKET
			-- read from the network stream up to INTEGER bytes
		local
			l_pair: PAIR [INTEGER, MOVIE_MANAGED_POINTER]
			l_movie_mp: MOVIE_MANAGED_POINTER
		do
			if playback_movie then
				l_pair ?= cam.next_frame ("read").frame_data
				cam.compare_partial_movie_frame (l_pair.first, size)
				create Result.make_from_managed_pointer (l_pair.second)
			else
				Result := Precursor (size)
				debug ("make_movies")
					if record_movie then
						create l_movie_mp.make_from_managed_pointer (Result.data)
						create l_pair.make (size, l_movie_mp)
						cam.record_movie_frame ("read", l_pair)
					end
				end
			end
		end

	read_until_sequence (a_string: STRING)
			-- Read a stream of characters until `a_string' character sequence is received
		require else
			socket_exists: exists
			opened_for_read: is_open_read
			a_string_not_void: a_string /= Void
			a_string_not_empty: a_string.count > 0
		local
			l_potential_terminator: STRING
			l_terminator_index: INTEGER
			l_count: INTEGER
		do
			from
				create last_string.make (512)
				create l_potential_terminator.make (a_string.count)
				l_terminator_index := 0
				l_count := 0
			until
				l_potential_terminator.is_equal (a_string)
			loop
				read_character
				if last_character.is_equal (a_string.item (l_terminator_index + 1)) then
					l_potential_terminator.append_character (last_character)
					l_terminator_index := l_terminator_index + 1
				else
					if l_potential_terminator.count > 0 then
						last_string.append_string (l_potential_terminator)
						l_count := l_count + l_potential_terminator.count
						l_potential_terminator := ""
						l_terminator_index := 0
					end
					last_string.extend (last_character)
					l_count := l_count + 1
				end
			end
		end

	read_until_character (a_char: CHARACTER)
			-- Read a stream of characters until an `a_char' is received
		require else
			socket_exists: exists
			opened_for_read: is_open_read
		local
			l_count: INTEGER
		do
			create last_string.make (512)
			from
				read_character
				l_count := 0
			until
				last_character = a_char
			loop
				l_count := l_count + 1
				last_string.extend (last_character)
				read_character
			end
		end

	receive (size, flags: INTEGER): PACKET
			-- Receive a packet of maximum size `size'.
		local
			l_triple: TRIPLE [INTEGER, INTEGER, MOVIE_MANAGED_POINTER]
			l_movie_mp: MOVIE_MANAGED_POINTER
		do
			if playback_movie then
				l_triple ?= cam.next_frame ("receive").frame_data
				cam.compare_partial_movie_frame (l_triple.first, size)
				cam.compare_partial_movie_frame (l_triple.second, flags)
				create Result.make_from_managed_pointer (l_triple.third)
			else
				Result := Precursor (size, flags)
				debug ("make_movies")
					if record_movie then
						create l_movie_mp.make_from_managed_pointer (Result.data)
						create l_triple.make (size, flags, l_movie_mp)
						cam.record_movie_frame ("receive", l_triple)
					end
				end
			end
		end

	enable_debug
			-- Enable socket system debugging.
		do
			if playback_movie then
				cam.compare_boolean_movie_frame ("enable_debug", True)
			else
				Precursor
				debug ("make_movies")
					if record_movie then
						cam.record_boolean_movie_frame ("enable_debug", True)
					end
				end
			end
		end

	disable_debug
			-- Disable socket system debugging.
		do
			if playback_movie then
				cam.compare_boolean_movie_frame ("disable_debug", False)
			else
				Precursor
				debug ("make_movies")
					if record_movie then
						cam.record_boolean_movie_frame ("disable_debug", False)
					end
				end
			end
		end

	debug_enabled: BOOLEAN
			-- Is socket system debugging enabled?
		do
			if playback_movie then
				Result := cam.next_frame ("debug_enabled").frame_data_as_boolean
			else
				Result := Precursor
				debug ("make_movies")
					if record_movie then
						cam.record_boolean_movie_frame ("debug_enabled", Result)
					end
				end
			end
		end

	do_not_route
			-- Set socket to non-routing.
		do
			if playback_movie then
				cam.compare_boolean_movie_frame ("do_not_route", False)
			else
				Precursor
				debug ("make_movies")
					if record_movie then
						cam.record_boolean_movie_frame ("do_not_route", False)
					end
				end
			end
		end

	route
			-- Set socket to routing.
		do
			if playback_movie then
				cam.compare_boolean_movie_frame ("route", True)
			else
				Precursor
				debug ("make_movies")
					if record_movie then
						cam.record_boolean_movie_frame ("route", True)
					end
				end
			end
		end

	route_enabled: BOOLEAN
			-- Is routing enabled?
		do
			if playback_movie then
				Result := cam.next_frame ("route_enabled").frame_data_as_boolean
			else
				Result := Precursor
				debug ("make_movies")
					if record_movie then
						cam.record_boolean_movie_frame ("route_enabled", Result)
					end
				end
			end
		end

	set_receive_buf_size (s: INTEGER)
			-- Set receive buffer size.
		do
			if playback_movie then
				cam.compare_integer_movie_frame ("set_receive_buf_size", s)
			else
				Precursor (s)
				debug ("make_movies")
					if record_movie then
						cam.record_integer_movie_frame ("set_receive_buf_size", s)
					end
				end
			end
		end

	receive_buf_size: INTEGER
			-- Size of receive buffer.
		do
			if playback_movie then
				Result := cam.next_frame ("receive_buf_size").frame_data_as_integer
			else
				Result := Precursor
				debug ("make_movies")
					if record_movie then
						cam.record_integer_movie_frame ("receive_buf_size", Result)
					end
				end
			end
		end

	set_send_buf_size (s: INTEGER)
			-- Set the send buffer to size `s'.
		do
			if playback_movie then
				cam.compare_integer_movie_frame ("set_send_buf_size", s)
			else
				Precursor (s)
				debug ("make_movies")
					if record_movie then
						cam.record_integer_movie_frame ("set_send_buf_size", s)
					end
				end
			end
		end

	send_buf_size: INTEGER
			-- Size of send buffer.
		do
			if playback_movie then
				Result := cam.next_frame ("send_buf_size").frame_data_as_integer
			else
				Result := Precursor
				debug ("make_movies")
					if record_movie then
						cam.record_integer_movie_frame ("send_buf_size", Result)
					end
				end
			end
		end

	is_socket_stream: BOOLEAN
			-- Is the socket a stream?
		do
			if playback_movie then
				Result := cam.next_frame ("is_socket_stream").frame_data_as_boolean
			else
				Result := Precursor
				debug ("make_movies")
					if record_movie then
						cam.record_boolean_movie_frame ("is_socket_stream", Result)
					end
				end
			end
		end

	set_non_blocking
			-- Set socket to non-blocking mode.
		do
			if playback_movie then
				cam.compare_boolean_movie_frame ("set_non_blocking", True)
				is_blocking := False
			else
				Precursor
				debug ("make_movies")
					if record_movie then
						cam.record_boolean_movie_frame ("set_non_blocking", True)
					end
				end
			end
		end

	set_blocking
			-- Set socket to blocking mode.
		do
			if playback_movie then
				cam.compare_boolean_movie_frame ("set_blocking", False)
				is_blocking := True
			else
				Precursor
				debug ("make_movies")
					if record_movie then
						cam.record_boolean_movie_frame ("set_blocking", False)
					end
				end
			end
		end

	set_owner (own: INTEGER)
			-- Negative value sets group process id.
			-- positive value sets process id.
		do
			if playback_movie then
				cam.compare_integer_movie_frame ("set_owner", own)
			else
				Precursor (own)
				debug ("make_movies")
					if record_movie then
						cam.record_integer_movie_frame ("set_owner", own)
					end
				end
			end
		end

	is_group_id: BOOLEAN
			-- Is the owner id the socket group id?
		do
			if playback_movie then
				Result := cam.next_frame ("is_group_id").frame_data_as_boolean
			else
				Result := Precursor
				debug ("make_movies")
					if record_movie then
						cam.record_boolean_movie_frame ("is_group_id", Result)
					end
				end
			end
		end

	is_process_id: BOOLEAN
			-- Is the owner id the socket process id?
		do
			if playback_movie then
				Result := cam.next_frame ("is_process_id").frame_data_as_boolean
			else
				Result := Precursor
				debug ("make_movies")
					if record_movie then
						cam.record_boolean_movie_frame ("is_process_id", Result)
					end
				end
			end
		end

	group_id: INTEGER
			-- Group id of socket
		do
			if playback_movie then
				Result := cam.next_frame ("group_id").frame_data_as_integer
			else
				Result := Precursor
				debug ("make_movies")
					if record_movie then
						cam.record_integer_movie_frame ("group_id", Result)
					end
				end
			end
		end

	process_id: INTEGER
			-- Process id of socket
		do
			if playback_movie then
				Result := cam.next_frame ("process_id").frame_data_as_integer
			else
				Result := Precursor
				debug ("make_movies")
					if record_movie then
						cam.record_integer_movie_frame ("process_id", Result)
					end
				end
			end
		end

	exists: BOOLEAN
			-- Does medium exist?
		do
			if playback_movie then
				Result := cam.next_frame ("exists").frame_data_as_boolean
				cam.compare_not_needed
			else
				Result := Precursor
				debug ("make_movies")
					if record_movie then
						cam.record_boolean_movie_frame ("exists", Result)
					end
				end
			end
		end

feature {NONE} -- Implementation.

	set_storable_type_in_auto_detect_header
	-- Call this before sending 'auto_detect_header_packet'.
	do
		auto_detect_header.put_natural_8(storable_type.as_natural_8, auto_detect_header.count - 1)
	end

	has_auto_detect_header_prefix(a_peek_data: MANAGED_POINTER): BOOLEAN
	-- True if the first n - 1 bytes of the argument match 'auto_detect_header_prefix'.
	do
		Result := a_peek_data.item.memory_compare(auto_detect_header_prefix.item, auto_detect_header_prefix.count)
	end

feature -- Access

	peek(a_byte_count: INTEGER): MANAGED_POINTER
	-- Peek at the input buffer and return in Result the data found.  The actual peek
	-- into the buffer will continue until 'a_byte_count' bytes have been read or an error
	-- occurs.  The following table shows the detailed behavior.
	--
	--	Event				Outcome
	--	=====				========
	--	all data found		success
	--	some data found		sleep and try again
	--	no data				block		(A non blocking socket will fail after retrying for 1 second (l_count will be -1 (SOCKET_ERROR)).)
	--	socket failure		fail		(A non blocking socket will fail after retrying for 1 second (l_count will be -1 (SOCKET_ERROR)).)
	--	graceful close		fail		(l_count will be 0)
	local
		l_count: INTEGER
		l_attempts: INTEGER
		l_last_error: INTEGER
	do
		from
			create Result.make(a_byte_count)
		until
			l_count >= a_byte_count
		loop
			l_attempts := l_attempts + 1
			l_count := c_receive(descriptor, Result.item, a_byte_count, MSG_PEEK)

			if l_count <= 0
			then
				l_last_error := WSAGetLastError
				if is_blocking
				then
					raise("c_receive failed")
				end

				if l_attempts > 100 and then l_last_error = would_block
				then
					raise("would block")
				end
			end

			if l_count < a_byte_count
			then
				sleep(10_000_000)	-- Sleep for 0.01 seconds (about the minimum possible).
			end
		end
	ensure
		result_not_void: Result /= Void
	end

	storable_type: INTEGER
			-- Storable type to use (classic storable or SED)

	skip_auto_detect: BOOLEAN
			-- If False and a received storable is found to have our auto detect header we
			-- will set 'storable_type'.  The default is to NOT skip.

	add_auto_detect_header: BOOLEAN
			-- Add the auto detect header to the first message sent.

	auto_detect_enabled: BOOLEAN = False
			-- Auto-detect is not yet ready for prime time.

	mem: MEMORY
			-- Access to memory functions
		once
			create Result
		end

	sed: SED_STORABLE_FACILITIES
			-- SED facilities
		once
			create Result
		end

feature -- External.

	WSAGetLastError: INTEGER
	-- The last error that happened to a socket.
	external
		"C signature (): INTEGER use <winsock.h>"		-- Nuts!  When I use winsock2 I get 100's of linkage errors.
	alias
		"WSAGetLastError"
	end

	MSG_PEEK: INTEGER
	-- C constant which causes 'recv' to read the socket buffer without removing it.
	external
		"C inline"
	alias
		"MSG_PEEK"
	end

feature -- Once/constants.

	storable_storable: INTEGER = 0		-- Constant to indicate C storables.  Using 0 means that C storables are the default.
	storable_sed: INTEGER = 1			-- Constant to indicate SED storables.

	auto_detect_header_prefix: MANAGED_POINTER
	-- This object is the first n - 1 bytes of 'auto_detect_header'.  If does NOT contain the storable type.
	once
		create Result.make(auto_detect_header.count - 1)
		Result.item.memory_copy(auto_detect_header.item, Result.count)
	end

	auto_detect_header: MANAGED_POINTER
	-- The prefix to the header that is sent before the first storable to indicate to the receiver the storable type.
	-- The construction of the header is as follows:
	--		a. 8 bytes of 0
	--		b. 255
	--		c. storable_type	(this is not included in 'auto_detect_header_prefix')
	-- Because the first four bytes of a SED storable are the size of the packet - that can't be zero (in 7.1 the
	-- first four are 255).  In version 16 of the C storables, the 4th and 5th bytes are the number of classes -
	-- again, that can't be zero.  So in both types, the first 5 bytes can't be zero.  Older versions of C storables
	-- are different so I set part a to be 8 bytes long (instead of 5).  Part b is set to 255 to avoid detection
	-- simply because of a bunch of zeros.  Part c is the type of the storable.
	once
		create Result.make(10)
		Result.put_integer_32(0, 0)
		Result.put_integer_32(0, 4)
		Result.put_natural_8(255, 8)
		Result.put_natural_8(127, 9)	-- This byte is bogus and must be set to a meaningful value.
	end

	auto_detect_header_packet: PACKET
	-- This is just a wrapper around 'auto_detect_header' so we can pass it to 'send'.
	once
		create Result.make_from_managed_pointer(auto_detect_header)
	end

end
