indexing
	description: "NT specific system information"
	notice: "DO NOT USE THIS CLASS directly ! Use class ROSE_EXECUTION_ENVIRONMENT instead"
	author: "Zoran Simic"
	date: "$Date: 2009/10/09 $"
	revision: "$Revision: #15 $"

class
	ROSE_NT_SYSTEM_INFORMATION

create
	make

feature {NONE} -- Initialization

	make is
			-- Get info for current user
		local
			i: INTEGER
		do
			-- User information
			user_login := get_user_info (NameSamCompatible)
			if user_login /= Void then
				i := user_login.index_of ('\', 1)
				if i > 0 then
					user_domain := user_login.substring (1, i - 1)
					user_login := user_login.substring (i + 1, user_login.count)
				end
			else
				user_login := "unknown"
			end
			user_display_name := get_user_info (NameDisplay)
			if user_display_name = Void or else user_display_name.is_empty then
				user_display_name := user_login.twin
			end
			user_unique_id := get_user_info (NameUniqueId)
			if user_unique_id = Void or else user_unique_id.is_empty then
				user_unique_id := user_login.twin
			end
			user_active_directory_path := get_user_info (NameCanonicalEx)
			if user_active_directory_path /= Void then
				i := user_active_directory_path.index_of ('%R', 1)
				if i = 0 then
					i := user_active_directory_path.index_of ('%N', 1)
				end
				if i > 0 then
					user_active_directory_path := user_active_directory_path.substring (1, i - 1)
				end
			else
				user_active_directory_path := user_unique_id
			end
			-- Machine information
			host_name := get_computer_info (ComputerNameDnsHostname)
			if host_name = Void or else host_name.is_empty then
				host_name := "localhost"
			end
			if user_domain = Void or else user_domain.is_empty then
				user_domain := host_name
			end
			host_dns := get_computer_info (ComputerNameDnsDomain)
			if host_dns = Void or else host_dns.is_empty then
				host_dns := host_name
			end
			host_active_directory_path := get_computer_object (NameCanonicalEx)
			if host_active_directory_path /= Void then
				i := host_active_directory_path.index_of ('%R', 1)
				if i = 0 then
					i := host_active_directory_path.index_of ('%N', 1)
				end
				if i > 0 then
					host_active_directory_path := host_active_directory_path.substring (1, i - 1)
				end
			else
				host_active_directory_path := host_dns
			end
			user_spn := user_login + "@" + host_dns
		end

feature -- Access

	user_domain: STRING
			-- User domain, example: "RIEM"

	user_login: STRING
			-- User login, example: "zs"

	user_display_name: STRING
			-- Display name, example: "Zoran Simic"

	user_unique_id: STRING
			-- Unique id, example: "{1a225d1d-1ee0-487a-bbd0-95bec88ec4d5}"

	user_active_directory_path: STRING
			-- Active directory path, example: "axaros.com/Orinda/Users/RC"

	user_spn: STRING
			-- Service Principal Names, example: "zs@axaros.com"

	host_name: STRING
			-- Host name, example: "arpcj8"

	host_dns: STRING
			-- Host DNS, example: "axaros.com"

	host_active_directory_path: STRING
			-- Host's active directory path, example: "axaros.com/Orinda/Desktops"

feature {NONE} -- Implementation

	buffer_size: INTEGER is 1024
			-- Buffer size used to create buffers for external C calls

	get_user_info (a_name_format: INTEGER): STRING is
			-- User info corresponding to 'a_name_format'
		local
			cs: C_STRING
			i: MANAGED_INTEGER
			retried: BOOLEAN
		do
			if not retried then
				create i.set (buffer_size)
				create cs.make_empty (i.value)
				if c_get_user_name_ex (a_name_format, cs.item, i.item) then
					Result := cs.string
				end
			end
		rescue
			if not retried then
				retried := True
				retry
			end
		end

	get_computer_info (a_name_format: INTEGER): STRING is
			-- Computer info corresponding to 'a_name_format'
		local
			cs: C_STRING
			i: INTEGER
			retried: BOOLEAN
		do
			if not retried then
				i := buffer_size
				create cs.make_empty (i)
				if c_get_computer_name_ex (a_name_format, cs.item, $i) then
					Result := cs.string
				end
			end
		rescue
			if not retried then
				retried := True
				retry
			end
		end

	get_computer_object (a_name_format: INTEGER): STRING is
			-- Computer object corresponding to 'a_name_format'
		local
			cs: C_STRING
			i: INTEGER
			retried: BOOLEAN
		do
			if not retried then
				i := buffer_size
				create cs.make_empty (i)
				if c_get_computer_object_name (a_name_format, cs.item, $i) then
					Result := cs.string
				end
			end
		rescue
			if not retried then
				retried := True
				retry
			end
		end

	c_get_user_name_ex (a_name_format: INTEGER; a_name, a_size: POINTER): BOOLEAN is
			-- C external GetUserNameExA
		external
			"C blocking signature (EXTENDED_NAME_FORMAT, LPSTR, PULONG): BOOLEAN use <Security.h>"
		alias
			"GetUserNameExA"
		end

	c_get_computer_name_ex (a_name_format: INTEGER; a_name, a_size: POINTER): BOOLEAN is
			-- C external GetComputerNameExA
		external
			"C blocking signature (COMPUTER_NAME_FORMAT, LPSTR, LPDWORD): BOOL use <Winbase.h>"
		alias
			"GetComputerNameExA"
		end

	c_get_computer_object_name (a_name_format: INTEGER; a_name, a_size: POINTER): BOOLEAN is
			-- C external GetComputerObjectNameA
		external
			"C blocking signature (EXTENDED_NAME_FORMAT, LPSTR, PULONG): BOOLEAN use <Security.h>"
		alias
			"GetComputerObjectNameA"
		end

	NameUniqueId: INTEGER is
			-- C external NameUniqueId
		external
			"C blocking macro use <security.h>"
		alias
			"NameUniqueId"
		end

	NameSamCompatible: INTEGER is
			-- C external NameSamCompatible
		external
			"C blocking macro use <security.h>"
		alias
			"NameSamCompatible"
		end

	NameDisplay: INTEGER is
			-- C external NameDisplay
		external
			"C blocking macro use <security.h>"
		alias
			"NameDisplay"
		end

	NameCanonicalEx: INTEGER is
			-- C external NameCanonicalEx
		external
			"C blocking macro use <security.h>"
		alias
			"NameCanonicalEx"
		end

	ComputerNameDnsHostname: INTEGER is
			-- C external ComputerNameDnsHostname
		external
			"C blocking macro use <windows.h>"
		alias
			"ComputerNameDnsHostname"
		end

	ComputerNameDnsDomain : INTEGER is
			-- C external ComputerNameDnsDomain
		external
			"C blocking macro use <windows.h>"
		alias
			"ComputerNameDnsDomain "
		end

end
