note

	description:

		"Numeric binary operator"

	author: "Michael Zhang"
	last_editor: "$Author: it $"
	date: "$Date: 2013/04/12 $"
	revision: "$Revision: #17 $"
	path: "$File: //rose/source/kernel/math/rose_numeric_binary_operator.e $"

deferred class
	ROSE_NUMERIC_BINARY_OPERATOR[reference G]

inherit
	ROSE_BINARY_OPERATOR [G]
		redefine
			generated_unitized_value,
			value_in_market_currency,
			boolean_value,
			computed_string
		end

	ROSE_IEEE_UNDEFINED

feature -- Access
	value_in_market_currency (a_left,a_right : ROSE_EXPRESSION[G];a_market : MARKET;a_currency : PERSISTENT_CURRENCY;a_entity : G;a_date : ROSE_DATE) : DOUBLE is
			-- <Precursor>
		local
			l_left,l_right : DOUBLE
		do
			l_left :=  a_left.value_in_market_currency(a_market,a_currency,a_entity,a_date)
			l_right := a_right.value_in_market_currency(a_market,a_currency,a_entity,a_date)
			Result := double_computation(a_left, a_right, l_left, l_right)
		end

	real_value (a_left, a_right: ROSE_EXPRESSION [G]; a_g: G): REAL is
			-- <Precursor>
		local
			l_left, l_right: REAL
		do
			l_left :=  a_left.real_value (a_g)
			l_right := a_right.real_value (a_g)
			Result := real_computation(a_left, a_right, l_left, l_right)
		end

	double_value (a_left, a_right: ROSE_EXPRESSION [G]; a_g: G): DOUBLE is
			-- <Precursor>
		local
			l_left, l_right: DOUBLE
		do
			l_left :=  a_left.double_value (a_g)
			l_right := a_right.double_value (a_g)
			Result := double_computation(a_left, a_right, l_left, l_right)
		end

	decimal_value (a_left, a_right: ROSE_EXPRESSION [G]; a_g: G): RP_DECIMAL is
			-- <Precursor>
		local
			l_left, l_right: RP_DECIMAL
		do
			l_left :=  a_left.decimal_value (a_g)
			l_right := a_right.decimal_value (a_g)
			Result := decimal_computation(a_left, a_right, l_left, l_right)
		end

	integer_value (a_left, a_right: ROSE_EXPRESSION [G]; a_g: G): INTEGER is
			-- <Precursor>
		local
			l_left, l_right: INTEGER
		do
			l_left :=  a_left.integer_value (a_g)
			l_right := a_right.integer_value (a_g)
			Result := integer_computation(a_left, a_right, l_left, l_right)
		end

	boolean_value (a_left, a_right: ROSE_EXPRESSION [G]; a_g: G): BOOLEAN is
			-- <Precursor>
		do
			Result := boolean_computation(a_left, a_right, a_left.boolean_value (a_g), a_right.boolean_value (a_g))
		end

	string_value (a_left, a_right: ROSE_EXPRESSION [G]; a_g: G): STRING is
			-- <Precursor>
		do
			Result := string_computation(a_left, a_right, a_left.string_value (a_g), a_right.string_value (a_g))
		end
	character_value (a_left, a_right: ROSE_EXPRESSION [G]; a_g: G): CHARACTER is
			-- <Precursor>
		do
		end

	date_time_value (a_left, a_right: ROSE_EXPRESSION [G]; a_g: G): ROSE_DATE is
			-- <Precursor>
		do
		end

feature --computed
	computed_real (a_left, a_right: ROSE_EXPRESSION [G]; a_g: G; a_agent: like access_agent_template): REAL is
			-- <Precursor>
		local
			l_left, l_right: REAL
		do
			l_left :=  a_left.computed_real (a_g, a_agent)
			l_right := a_right.computed_real (a_g, a_agent)
			Result := real_computation(a_left, a_right, l_left, l_right)
		end

	computed_double (a_left, a_right: ROSE_EXPRESSION [G]; a_g: G; a_agent: like access_agent_template): DOUBLE is
			-- <Precursor>
		local
			l_left, l_right: DOUBLE
		do
			l_left :=  a_left.computed_double (a_g, a_agent)
			l_right := a_right.computed_double (a_g, a_agent)
			Result := double_computation(a_left, a_right, l_left, l_right)
		end


	computed_integer(a_left,a_right : ROSE_EXPRESSION[G]; a_g : G; a_agent : like access_agent_template) : INTEGER is
			-- <Precursor>
		local
			l_left, l_right: INTEGER
		do
			l_left :=  a_left.computed_integer (a_g, a_agent)
			l_right := a_right.computed_integer (a_g, a_agent)
			Result := integer_computation(a_left, a_right, l_left, l_right)
		end
	computed_string(a_left,a_right : ROSE_EXPRESSION[G]; a_g : G; a_agent : like access_agent_template) : STRING is
			-- <Precursor>
		do
			if (a_left /= Void and a_right /= Void) then
				Result := string_computation(a_left,a_right, a_left.computed_string (a_g, a_agent), a_right.computed_string (a_g, a_agent))
			end
		end
	computed_boolean(a_left,a_right : ROSE_EXPRESSION[G]; a_g : G; a_agent : like access_agent_template) : BOOLEAN is
			-- <Precursor>
		do
			Result := boolean_computation(a_left,a_right, a_left.computed_boolean (a_g, a_agent), a_right.computed_boolean (a_g, a_agent))
		end
	computed_date(a_left,a_right : ROSE_EXPRESSION[G]; a_g : G; a_agent : like access_agent_template) : ROSE_DATE is
			-- <Precursor>
		local
			l_left_date, l_right_date: ROSE_DATE
		do
			l_left_date := a_left.computed_date (a_g, a_agent)
			l_right_date := a_right.computed_date (a_g, a_agent)
			Result := date_computation(a_left,a_right, l_left_date, l_right_date)
		end


feature  --generated
	generated_real (a_left, a_right: ROSE_EXPRESSION [G]; a_g: G; a_agent: like access_agent_template): REAL is
			-- <Precursor>
		local
			l_left, l_right: REAL
		do
			l_left :=  a_left.generated_real (a_g, a_agent)
			l_right := a_right.generated_real (a_g, a_agent)
			Result := real_computation(a_left, a_right, l_left, l_right)
		end

	generated_double (a_left, a_right: ROSE_EXPRESSION [G]; a_g: G; a_agent: like access_agent_template): DOUBLE is
			-- <Precursor>
		local
			l_left, l_right: DOUBLE
		do
			l_left :=  a_left.generated_double (a_g, a_agent)
			l_right := a_right.generated_double (a_g, a_agent)
			Result := double_computation(a_left, a_right, l_left, l_right)
		end
	generated_unitized_value(a_left,a_right : ROSE_EXPRESSION[G]; a_g : G; a_agent : like access_agent_template) : DATED_UNITIZED_VALUE[DOUBLE] is
			-- <Precursor>
		do
			create Result.make_with_units(generated_double (a_left, a_right, a_g, a_agent),nominal_units(a_left.generated_unitized_value (a_g, a_agent).units,a_right.generated_unitized_value (a_g, a_agent).units))
		end
	generated_integer(a_left,a_right : ROSE_EXPRESSION[G]; a_g : G; a_agent : like access_agent_template) : INTEGER is
			-- <Precursor>
		local
			l_left, l_right: INTEGER
		do
			l_left :=  a_left.generated_integer (a_g, a_agent)
			l_right := a_right.generated_integer (a_g, a_agent)
			Result := integer_computation(a_left, a_right, l_left, l_right)
		end
	generated_string(a_left,a_right : ROSE_EXPRESSION[G]; a_g : G; a_agent : like access_agent_template) : STRING is
			-- <Precursor>
		do
			Result := string_computation(a_left,a_right, a_left.generated_string (a_g, a_agent), a_right.generated_string (a_g, a_agent))
		end
	generated_date(a_left,a_right : ROSE_EXPRESSION[G]; a_g : G; a_agent : like access_agent_template) : ROSE_DATE is
			-- <Precursor>
		do
			Result := date_computation(a_left,a_right, a_left.generated_date (a_g, a_agent), a_right.generated_date (a_g, a_agent))
		end

feature {NONE}  --implementation
	double_computation(a_left, a_right: ROSE_EXPRESSION [G]; a_left_value, a_right_value : DOUBLE) : DOUBLE is
		deferred
		end
	decimal_computation(a_left, a_right: ROSE_EXPRESSION [G]; a_left_value, a_right_value : RP_DECIMAL) : RP_DECIMAL is
		deferred
		end
	real_computation(a_left, a_right: ROSE_EXPRESSION [G]; a_left_value, a_right_value : REAL) : REAL is
		deferred
		end
	integer_computation(a_left, a_right: ROSE_EXPRESSION [G]; a_left_value, a_right_value : INTEGER) : INTEGER is
		deferred
		end
	string_computation(a_left, a_right: ROSE_EXPRESSION [G]; a_left_value, a_right_value : STRING) : STRING is
		deferred
		end
	boolean_computation(a_left, a_right: ROSE_EXPRESSION [G]; a_left_value, a_right_value : BOOLEAN) : BOOLEAN is
			-- Boolean computation
		deferred
		end
	date_computation(a_left, a_right: ROSE_EXPRESSION [G]; a_left_value, a_right_value : ROSE_DATE) : ROSE_DATE is
		deferred
		end
end
