indexing
	description: "Browse objects dynamically - tree-view and context view"
	author: "Zoran Simic"
	date: "$Date: 2008/12/01 $"
	revision: "$Revision: #7 $"

class
	ROSE_OBJECT_BROWSER

inherit
	EV_VERTICAL_BOX
	ROSE_OBJECT_BROWSER_MANAGER
		undefine
			default_create, is_equal, copy
		end

create
	make

feature -- Initialization

	make (a_container: EV_CONTAINER) is
		local
			l_contextual_box, l_hbox: EV_BOX
			l_split: EV_SPLIT_AREA
			l_notebook: EV_NOTEBOOK
			l_toolbar: EV_TOOL_BAR
			l_widget: EV_WIDGET
			l_counter: ROSE_OBJECT_BROWSER_COUNTER
			l_relations: ROSE_OBJECT_BROWSER_RELATION
		do
			default_create
				--| Toolbar and prefs
			l_hbox := Gui.factory.new_hbox (Current,0,4)
			l_toolbar := Gui.factory.new_toolbar (l_hbox,<<
				["new.png",agent clear,"Clear view"],
				["open.png",agent load_object (Void),"Retreive a stored/serialized object"],
				["save.png",agent save_object,"Store/serialize an object"],
				Void
			>>)
			listed_objects := Gui.factory.new_combo (l_hbox,"Object: ",Void,Void,Void,160)
			listed_objects.disable_edit
			l_widget := Gui.factory.new_button (l_hbox,"Show",agent show_listed_object,60)
				--| Tree-view and contextual view
			create tree.make (Void)
			tree.add_observer (agent on_tree_select)
			create l_notebook
			l_split := Gui.factory.new_hsplit (Current,<<[tree,350],l_notebook>>)
			l_contextual_box := Gui.factory.new_vbox (l_notebook,0,-4)
			l_widget := Gui.factory.new_label (l_contextual_box,"Drop an object here or select following of the tree-view selection",1)
			l_widget.set_foreground_color (Gui.rep.colors.Dark_green)
			header := Gui.factory.new_label (l_contextual_box,Void,1)
			create flat
			l_contextual_box.extend (flat)
			l_notebook.set_item_text (l_contextual_box," Detail ")
				--| Object counter
			create l_counter.make (l_notebook)
			l_notebook.set_item_text (l_counter," Counter ")
				--| Referers
			create l_relations.make (l_notebook,agent memory.referers)
			l_notebook.set_item_text (l_relations," Referers ")
				--| Siblings
			create l_relations.make (l_notebook,agent memory.objects_instance_of)
			l_notebook.set_item_text (l_relations," Siblings ")
			Gui.factory.set_box_defaults (a_container,Current,2,2)
		end

feature -- Basic operations

	clear is
			-- Clear objects from views.
		do
			tree.wipe_out
			on_tree_select
		end

	show_object (a_object: ANY) is
			-- Show 'a_object', remove previously shown objects.
		do
			tree.show_object (a_object)
		end

	add_object (a_object: ANY) is
			-- Show 'a_object', remove previously shown objects.
		do
			tree.add_object (a_object)
		end

	load_object (a_path: STRING) is
			-- Load object from 'a_path' and show it.
			-- If 'a_path' ends in .xml, deserialize the object, otherwise retrieve it.
		local
			l_path, l_extension: STRING
			l_object: ANY
		do
			if a_path /= Void then
				l_path := a_path
			else
				l_path := Gui.dialogs.select_open_file (Gui.parent_window (Current),"object_browser","*.*")
			end
			l_extension := Global.formatter.extension (l_path)
			if l_extension /= Void then
				l_extension.to_lower
			end
			if equal (l_extension,"xml") then
				l_object := Global.persist.deserialized (l_path)
			elseif l_path /= Void then
				l_object := Global.persist.retrieved (l_path)
			end
			if l_object /= Void then
				show_object (l_object)
			end
			error.flush_and_acknowledge
		end

	save_object is
			-- Save 'a_object' to user selected path.
			-- If provided path ends in .xml, serialize it, otherwise store it.
		local
			l_path, l_extension: STRING
			l_object: ANY
		do
			l_object := tree.selected_object
			if l_object /= Void then
				l_path := Gui.dialogs.select_save_file (Gui.parent_window (Current),"object_browser","*.*")
				l_extension := Global.formatter.extension (l_path)
				if l_extension /= Void then
					l_extension.to_lower
				end
				if equal (l_extension,"xml") then
					Global.persist.serialize (l_object,l_path)
				elseif l_path /= Void then
					Global.persist.store (l_object,l_path)
				end
			end
			error.flush_and_acknowledge
		end

	add_listed_object (a_name: STRING; a_object: ANY) is
			-- Add 'a_object' available via combo-box under 'a_name'.
		local
			l_item: EV_LIST_ITEM
		do
			create l_item
			if a_name /= Void then
				l_item.set_text (a_name)
			elseif a_object /= Void then
				l_item.set_text (a_object.generating_type)
			else
				l_item.set_text ("?")
			end
			l_item.set_data (a_object)
			listed_objects.extend (l_item)
		end

feature {NONE} -- Implementation

	listed_objects: ROSE_COMBO_BOX
	tree: ROSE_OBJECT_BROWSER_TREE
	header: EV_LABEL
	flat: EV_CELL

	on_tree_select is
		do
			describe (tree.selected_object)
		end

	describe (a_object: ANY) is
		local
			l_handler: ROSE_OBJECT_BROWSER_HANDLER
			l_widget: EV_WIDGET
		do
			if a_object = Void then
				header.set_text ("-")
				flat.wipe_out
			else
				header.set_text (object_description (a_object))
				l_handler := get_handler (a_object)
				l_widget := l_handler.widget (a_object)
				if l_widget /= Void then
					flat.put (l_widget)
				else
					flat.wipe_out
				end
			end
		end

	object_description (a_object: ANY): STRING is
		do
			if a_object = Void then
				Result := "Void"
			else
				Result := clone (a_object.generating_type)
				Result.append (" [")
				Result.append (($a_object).out)
				Result.append ("]")
			end
		end

	show_listed_object is
		local
			l_item: EV_LIST_ITEM
		do
			l_item := listed_objects.selected_item
			if l_item /= Void then
				show_object (l_item.data)
			end
		end

end -- class ROSE_OBJECT_BROWSER
