indexing
	description: "Counter widget for graphical object browser"
	author: "Zoran Simic"
	date: "$Date: 2008/12/01 $"
	revision: "$Revision: #7 $"

class
	ROSE_OBJECT_BROWSER_COUNTER

inherit
	EV_VERTICAL_BOX
	ROSE_OBJECT_PROCESSOR
		undefine
			default_create, is_equal, copy
		end
	ROSE_OBJECT_BROWSER_MANAGER
		undefine
			default_create, is_equal, copy
		end
	PLATFORM
		export
			{NONE} all
		undefine
			default_create, is_equal, copy
		end

create
	make

feature {NONE} -- Initialization

	make (a_container: EV_CONTAINER) is
		local
			l_widget: EV_WIDGET
		do
			default_create
			create counter.make (0)
			create refs.make (0)
			l_widget := Gui.factory.new_label (Current, "Drop an object here to see it's related objects", 1)
			l_widget.set_foreground_color (Gui.rep.colors.Dark_green)
			l_widget.drop_actions.extend (agent show_object)
			l_widget := Gui.factory.new_label (Current, "Warning: this may result in a lengthy operation...", 1)
			l_widget.set_foreground_color (Gui.rep.colors.Dark_red)
			l_widget.drop_actions.extend (agent show_object)
			info := Gui.factory.new_label (Current, Void, 1)
			mgrid := Gui.factory.new_grid (Current, <<["Type", 320], ["Occurences", 80]>>, Void, 0, 0)
			Gui.factory.enbox_and_size_simple (Current, a_container, 0)
			drop_actions.extend (agent show_object)
			info.drop_actions.extend (agent show_object)
			mgrid.drop_actions.extend (agent show_object)
		end

feature -- Access

	counter: HASH_TABLE [INTEGER, STRING]
			-- Number of objects found per type

	refs: HASH_TABLE [BOOLEAN, POINTER]
			-- Used to keep track of already counted objects, hashed by their address in memory

	total_size: INTEGER
			-- Total memory size occupied by shown object

feature -- Basic operations

	show_object (a_object: ANY) is
			-- Show counts for 'a_object'.
		do
			counter.clear_all
			total_size := 0
			memory.collection_off
			process (Void, a_object)
			memory.collection_on
			refs.clear_all
			mgrid.clear
			from
				counter.start
			until
				counter.off
			loop
				mgrid.extend(Gui.factory.new_grid_node (Void, <<counter.key_for_iteration, counter.item_for_iteration>>, Void))
				counter.forth
			end
			info.set_text (object_representation (a_object, Void, Void, Void) + "%N" + total_size.out + " bytes total")
		end

feature {NONE} -- Implementation

	info: EV_LABEL
	mgrid: RP_DATA_GRID[ANY]

	add (a_object: ANY; a_type: STRING; a_size, a_count: INTEGER) is
		local
			l_type: STRING
		do
			if a_type /= Void then
				l_type := a_type
			elseif a_object /= Void then
				l_type := a_object.generating_type
			else
				l_type := "Void"
			end
			counter.force (counter.item (l_type)+a_count, l_type)
			total_size := total_size + a_size
			if a_object /= Void and then not refs.item ($a_object) then
				refs.put (True, $a_object)
				process_fields (a_object)
			end
		end

	process_void (a_name, a_type: STRING) is
		do
			add (Void, a_type, Pointer_bytes, 1)
		end

	process_any (a_name: STRING; a_object: ANY) is
		do
			add (a_object, Void, Pointer_bytes, 1)
		end

	process_string (a_name: STRING; a_string: STRING) is
		do
			add (a_string, Void, Pointer_bytes, 1)
		end

	process_integer (a_name: STRING; a_integer: INTEGER) is
		do
			add (Void, "INTEGER", Integer_bytes, 1)
		end

	process_integer_8 (a_name: STRING; a_integer: INTEGER_8) is
		do
			add (Void, "INTEGER_8", Integer_8_bytes, 1)
		end

	process_integer_16 (a_name: STRING; a_integer: INTEGER_16) is
		do
			add (Void, "INTEGER_16", Integer_16_bytes, 1)
		end

	process_integer_64 (a_name: STRING; a_integer: INTEGER_64) is
		do
			add (Void, "INTEGER_64", Integer_64_bytes, 1)
		end

	process_double (a_name: STRING; a_double: DOUBLE) is
		do
			add (Void, "DOUBLE", Double_bytes, 1)
		end

	process_real (a_name: STRING; a_real: REAL) is
		do
			add (Void, "REAL", Real_bytes, 1)
		end

	process_character (a_name: STRING; a_character: CHARACTER) is
		do
			add (Void, "CHARACTER", Character_bytes, 1)
		end

	process_boolean (a_name: STRING; a_boolean: BOOLEAN) is
		do
			add (Void, "BOOLEAN", Boolean_bytes, 1)
		end

	process_pointer (a_name: STRING; a_pointer: POINTER) is
		do
			add (Void, "POINTER", Pointer_bytes, 1)
		end

	process_special_integer (a_name: STRING; a_special: SPECIAL [INTEGER]) is
			-- Process 'a_special' of type SPECIAL [INTEGER] with 'a_name'
			-- 'a_name' is Void if the 'process'-ed object is a special
		do
			add (Void, "SPECIAL [INTEGER]", Pointer_bytes, 1)
			if a_special /= Void then
				add (Void, "INTEGER", Integer_bytes*a_special.count, a_special.count)
			end
		end

	process_special_integer_8 (a_name: STRING; a_special: SPECIAL [INTEGER_8]) is
			-- Process 'a_special' of type SPECIAL [INTEGER_8] with 'a_name'
			-- 'a_name' is Void if the 'process'-ed object is a special
		do
			add (Void, "SPECIAL [INTEGER_8]", Pointer_bytes, 1)
			if a_special /= Void then
				add (Void, "INTEGER_8", Integer_8_bytes*a_special.count, a_special.count)
			end
		end

	process_special_integer_16 (a_name: STRING; a_special: SPECIAL [INTEGER_16]) is
			-- Process 'a_special' of type SPECIAL [INTEGER_16] with 'a_name'
			-- 'a_name' is Void if the 'process'-ed object is a special
		do
			add (Void, "SPECIAL [INTEGER_16]", Pointer_bytes, 1)
			if a_special /= Void then
				add (Void, "INTEGER_16", Integer_16_bytes*a_special.count, a_special.count)
			end
		end

	process_special_integer_64 (a_name: STRING; a_special: SPECIAL [INTEGER_64]) is
			-- Process 'a_special' of type SPECIAL [INTEGER_64] with 'a_name'
			-- 'a_name' is Void if the 'process'-ed object is a special
		do
			add (Void, "SPECIAL [INTEGER_64]", Pointer_bytes, 1)
			if a_special /= Void then
				add (Void, "INTEGER_64", Integer_64_bytes*a_special.count, a_special.count)
			end
		end

	process_special_character (a_name: STRING; a_special: SPECIAL [CHARACTER]) is
			-- Process 'a_special' of type SPECIAL [CHARACTER] with 'a_name'
			-- 'a_name' is Void if the 'process'-ed object is a special
		do
			add (Void, "SPECIAL [CHARACTER]", Pointer_bytes, 1)
			if a_special /= Void then
				add (Void, "CHARACTER", Character_bytes*a_special.count, a_special.count)
			end
		end

	process_special_boolean (a_name: STRING; a_special: SPECIAL [BOOLEAN]) is
			-- Process 'a_special' of type SPECIAL [BOOLEAN] with 'a_name'
			-- 'a_name' is Void if the 'process'-ed object is a special
		do
			add (Void, "SPECIAL [BOOLEAN]", Pointer_bytes, 1)
			if a_special /= Void then
				add (Void, "BOOLEAN", Boolean_bytes*a_special.count, a_special.count)
			end
		end

	process_special_double (a_name: STRING; a_special: SPECIAL [DOUBLE]) is
			-- Process 'a_special' of type SPECIAL [DOUBLE] with 'a_name'
			-- 'a_name' is Void if the 'process'-ed object is a special
		do
			add (Void, "SPECIAL [DOUBLE]", Pointer_bytes, 1)
			if a_special /= Void then
				add (Void, "DOUBLE", Double_bytes*a_special.count, a_special.count)
			end
		end

	process_special_real (a_name: STRING; a_special: SPECIAL [REAL]) is
			-- Process 'a_special' of type SPECIAL [REAL] with 'a_name'
			-- 'a_name' is Void if the 'process'-ed object is a special
		do
			add (Void, "SPECIAL [REAL]", Pointer_bytes, 1)
			if a_special /= Void then
				add (Void, "REAL", Real_bytes*a_special.count, a_special.count)
			end
		end

	process_special_pointer (a_name: STRING; a_special: SPECIAL [POINTER]) is
			-- Process 'a_special' of type SPECIAL [POINTER] with 'a_name'
			-- 'a_name' is Void if the 'process'-ed object is a special
		do
			add (Void, "SPECIAL [POINTER]", Pointer_bytes, 1)
			if a_special /= Void then
				add (Void, "POINTER", Pointer_bytes*a_special.count, a_special.count)
			end
		end

	process_special_any (a_name: STRING; a_special: SPECIAL [ANY]) is
			-- Process 'a_special' of type SPECIAL [ANY] with 'a_name'
			-- 'a_name' is Void if the 'process'-ed object is a special
		local
			i: INTEGER
		do
			add (Void, a_special.generating_type, Pointer_bytes, 1)
			from
				i := 0
			until
				i >= a_special.count
			loop
				add (a_special.item (i), Void, Pointer_bytes, 1)
				i := i + 1
			end
		end

end
