note

	description:

		"Object browser manager"

	author: "Zoran Simic"
	last_editor: "$Author: er $"
	date: "$Date: 2013/01/25 $"
	revision: "$Revision: #4 $"
	path: "$File: //rose/source/kernel/infrastructure/framework/gui/rose_object_browser_manager.e $"

class
	ROSE_OBJECT_BROWSER_MANAGER

inherit
	ROSE_SHARED_GLOBAL_GUI

feature -- Access

	get_handler (a_object: ANY): ROSE_OBJECT_BROWSER_HANDLER is
		do
			from
				handlers.start
			until
				Result /= Void or else handlers.off
			loop
				if handlers.item.is_handled (a_object) then
					Result := handlers.item
				end
				handlers.forth
			end
		end

	get_icon (a_object: ANY): EV_PIXMAP is
		do
		end

	maximum_items: CELL [INTEGER] is
		once
			create Result.put (50)
		end

	memory: MEMORY is
			-- Memory routines
		once
			create Result
		end

feature -- Status report

	show_addresses: CELL [BOOLEAN] is
		once
			create Result.put (False)
		end

feature -- Basic operations

	add_handler (a_handler: ROSE_OBJECT_BROWSER_HANDLER) is
		do
			handlers.extend (a_handler)
		end

	set_show_addresses (a_value: BOOLEAN) is
		do
			show_addresses.put (a_value)
		end

feature {NONE} -- Implementation

	handlers: LINKED_LIST [ROSE_OBJECT_BROWSER_HANDLER] is
		once
			create Result.make
			Result.extend (create {ROSE_OBJECT_BROWSER_HANDLER_STRING}.make)
			Result.extend (create {ROSE_OBJECT_BROWSER_HANDLER_SPECIAL}.make)
			Result.extend (create {ROSE_OBJECT_BROWSER_HANDLER_HASH}.make)
			Result.extend (create {ROSE_OBJECT_BROWSER_HANDLER_ARRAY}.make)
			Result.extend (create {ROSE_OBJECT_BROWSER_HANDLER_LINEAR}.make)
			Result.extend (create {ROSE_OBJECT_BROWSER_HANDLER_ANY}.make)
		end

	object_representation (a_object: ANY; a_name, a_type, a_value: STRING): STRING is
		local
			l_debug: DEBUG_OUTPUT
			l_string: STRING
		do
			create Result.make (40)
			if a_name /= Void then
				Result.append (a_name)
				Result.append (": ")
			end
			if a_type /= Void then
				Result.append (a_type)
			elseif a_object /= Void then
				Result.append (get_object_type (a_object))
			end
			if show_addresses.item and then a_object /= Void then
				Result.append (" [")
				Result.append (get_object_address (a_object))
				Result.append ("]")
			end
			if a_value /= Void then
				Result.append (" = ")
				Result.append (a_value)
			else
				l_debug ?= a_object
				l_string ?= a_object
				if l_debug /= Void then
					Result.append (" = ")
					Result.append (get_debug_output (l_debug))
				elseif l_string /= Void then
					Result.append (" = ")
					Result.append (l_string)
				elseif a_object = Void then
					Result.append (" = Void")
				end
			end
		end

	get_object_out (a_object: ANY): STRING is
		local
			rescued: BOOLEAN
		do
			if rescued then
				Result := "- Can't get object output -"
			elseif a_object = Void then
				Result := "Void"
			else
				Result := a_object.out
			end
		rescue
			rescued := True
			retry
		end

	get_object_type (a_object: ANY): STRING is
		local
			rescued: BOOLEAN
		do
			if rescued then
				Result := "- Can't get object type -"
			elseif a_object = Void then
				Result := "Void"
			else
				Result := a_object.generating_type
			end
		rescue
			rescued := True
			retry
		end

	get_object_address (a_object: ANY): STRING is
		local
			rescued: BOOLEAN
		do
			if rescued then
				Result := "- Can't get object address -"
			elseif a_object /= Void then
				Result := ($a_object).out
			end
		rescue
			rescued := True
			retry
		end

	get_debug_output (a_object: DEBUG_OUTPUT): STRING is
		local
			rescued: BOOLEAN
		do
			if rescued then
				Result := "- Can't get object output -"
			elseif a_object = Void then
				Result := "Void"
			else
				Result := a_object.debug_output.as_string_8
			end
		rescue
			rescued := True
			retry
		end

end -- class ROSE_OBJECT_BROWSER_MANAGER
