indexing
	description: "Browse objects dynamically"
	author: "Zoran Simic"
	date: "$Date: 2008/12/01 $"
	revision: "$Revision: #2 $"

class
	ROSE_OBJECT_BROWSER_TREE_PROCESSOR

inherit
	ROSE_OBJECT_PROCESSOR
	ROSE_OBJECT_BROWSER_MANAGER

feature -- Access

	subnodes (a_object: ANY): LINKED_LIST [EV_TREE_NODE] is
			-- Subnodes for 'a_object'
		do
			create subnode_list.make
			process_fields (a_object)
			Result := subnode_list
			subnode_list := Void
		end

	node (a_object: ANY): EV_TREE_NODE is
			-- Node representing 'a_object'
		do
			create subnode_list.make
			process (Void,a_object)
			if not subnode_list.is_empty then
				Result := subnode_list.first
			end
			subnode_list := Void
		end

feature {NONE} -- Implementation

	subnode_list: LINKED_LIST [EV_TREE_NODE]

	process_void (a_name, a_type: STRING) is
		do
			add_subnode (Void,a_name,a_type,Void,pix_void)
		end

	process_any (a_name: STRING; a_object: ANY) is
		do
			subnode_list.extend (create {ROSE_OBJECT_BROWSER_TREE_NODE}.make_named (a_name,a_object))
		end

	process_string (a_name: STRING; a_string: STRING) is
		do
			subnode_list.extend (create {ROSE_OBJECT_BROWSER_TREE_NODE}.make_named (a_name,a_string))
		end

	process_integer (a_name: STRING; a_integer: INTEGER) is
		do
			add_subnode (a_integer,a_name,a_integer.generating_type,a_integer.out,pix_integer)
		end

	process_integer_8 (a_name: STRING; a_integer: INTEGER_8) is
		do
			add_subnode (a_integer,a_name,a_integer.generating_type,a_integer.out,pix_integer)
		end

	process_integer_16 (a_name: STRING; a_integer: INTEGER_16) is
		do
			add_subnode (a_integer,a_name,a_integer.generating_type,a_integer.out,pix_integer)
		end

	process_integer_64 (a_name: STRING; a_integer: INTEGER_64) is
		do
			add_subnode (a_integer,a_name,a_integer.generating_type,a_integer.out,pix_integer)
		end

	process_double (a_name: STRING; a_double: DOUBLE) is
		do
			add_subnode (a_double,a_name,a_double.generating_type,a_double.out,pix_double)
		end

	process_real (a_name: STRING; a_real: REAL) is
		do
			add_subnode (a_real,a_name,a_real.generating_type,a_real.out,pix_double)
		end

	process_character (a_name: STRING; a_character: CHARACTER) is
		do
			add_subnode (a_character,a_name,a_character.generating_type,a_character.out,pix_character)
		end

	process_boolean (a_name: STRING; a_boolean: BOOLEAN) is
		do
			add_subnode (a_boolean,a_name,a_boolean.generating_type,a_boolean.out,pix_boolean)
		end

	process_pointer (a_name: STRING; a_pointer: POINTER) is
		do
			add_subnode (a_pointer,a_name,a_pointer.generating_type,a_pointer.out,pix_pointer)
		end

	process_special_integer (a_name: STRING; a_special: SPECIAL [INTEGER]) is
			-- Process 'a_object' of type SPECIAL [INTEGER] with 'a_name'
			-- 'a_name' is Void if the 'process'-ed object is a special
		do
			subnode_list.extend (create {ROSE_OBJECT_BROWSER_TREE_NODE_SPECIAL [INTEGER]}.make (a_special,a_name,"INTEGER",pix_integer))
		end

	process_special_integer_8 (a_name: STRING; a_special: SPECIAL [INTEGER_8]) is
			-- Process 'a_object' of type SPECIAL [INTEGER_8] with 'a_name'
			-- 'a_name' is Void if the 'process'-ed object is a special
		do
			subnode_list.extend (create {ROSE_OBJECT_BROWSER_TREE_NODE_SPECIAL [INTEGER_8]}.make (a_special,a_name,"INTEGER_8",pix_integer))
		end

	process_special_integer_16 (a_name: STRING; a_special: SPECIAL [INTEGER_16]) is
			-- Process 'a_object' of type SPECIAL [INTEGER_16] with 'a_name'
			-- 'a_name' is Void if the 'process'-ed object is a special
		do
			subnode_list.extend (create {ROSE_OBJECT_BROWSER_TREE_NODE_SPECIAL [INTEGER_16]}.make (a_special,a_name,"INTEGER_16",pix_integer))
		end

	process_special_integer_64 (a_name: STRING; a_special: SPECIAL [INTEGER_64]) is
			-- Process 'a_object' of type SPECIAL [INTEGER_64] with 'a_name'
			-- 'a_name' is Void if the 'process'-ed object is a special
		do
			subnode_list.extend (create {ROSE_OBJECT_BROWSER_TREE_NODE_SPECIAL [INTEGER_64]}.make (a_special,a_name,"INTEGER_64",pix_integer))
		end

	process_special_character (a_name: STRING; a_special: SPECIAL [CHARACTER]) is
			-- Process 'a_object' of type SPECIAL [CHARACTER] with 'a_name'
			-- 'a_name' is Void if the 'process'-ed object is a special
		do
			subnode_list.extend (create {ROSE_OBJECT_BROWSER_TREE_NODE_SPECIAL [CHARACTER]}.make (a_special,a_name,"CHARACTER",pix_character))
		end

	process_special_boolean (a_name: STRING; a_special: SPECIAL [BOOLEAN]) is
			-- Process 'a_object' of type SPECIAL [BOOLEAN] with 'a_name'
			-- 'a_name' is Void if the 'process'-ed object is a special
		do
			subnode_list.extend (create {ROSE_OBJECT_BROWSER_TREE_NODE_SPECIAL [BOOLEAN]}.make (a_special,a_name,"BOOLEAN",pix_boolean))
		end

	process_special_double (a_name: STRING; a_special: SPECIAL [DOUBLE]) is
			-- Process 'a_object' of type SPECIAL [DOUBLE] with 'a_name'
			-- 'a_name' is Void if the 'process'-ed object is a special
		do
			subnode_list.extend (create {ROSE_OBJECT_BROWSER_TREE_NODE_SPECIAL [DOUBLE]}.make (a_special,a_name,"DOUBLE",pix_double))
		end

	process_special_real (a_name: STRING; a_special: SPECIAL [REAL]) is
			-- Process 'a_object' of type SPECIAL [REAL] with 'a_name'
			-- 'a_name' is Void if the 'process'-ed object is a special
		do
			subnode_list.extend (create {ROSE_OBJECT_BROWSER_TREE_NODE_SPECIAL [REAL]}.make (a_special,a_name,"REAL",pix_double))
		end

	process_special_pointer (a_name: STRING; a_special: SPECIAL [POINTER]) is
			-- Process 'a_special' of type SPECIAL [POINTER] with 'a_name'
			-- 'a_name' is Void if the 'process'-ed object is a special
		do
			subnode_list.extend (create {ROSE_OBJECT_BROWSER_TREE_NODE_SPECIAL [POINTER]}.make (a_special,a_name,"POINTER",pix_pointer))
		end

	process_special_any (a_name: STRING; a_special: SPECIAL [ANY]) is
			-- Process 'a_object' of type SPECIAL [ANY] with 'a_name'
			-- 'a_name' is Void if the 'process'-ed object is a special
		do
			subnode_list.extend (create {ROSE_OBJECT_BROWSER_TREE_NODE_SPECIAL_ANY}.make (a_special,a_name))
		end

	add_subnode (a_object: ANY; a_name, a_type, a_value: STRING; a_pixmap: EV_PIXMAP) is
		do
			subnode_list.extend (get_subnode (a_object,a_name,a_type,a_value,a_pixmap))
		end

	get_subnode (a_object: ANY; a_name, a_type, a_value: STRING; a_pixmap: EV_PIXMAP): EV_TREE_ITEM is
		local
			l_handler: ROSE_OBJECT_BROWSER_HANDLER
		do
			l_handler := get_handler (a_object)
			create Result.make_with_text (object_representation (a_object,a_name,a_type,a_value))
			if a_pixmap /= Void then
				Result.set_pixmap (a_pixmap)
			else
				Result.set_pixmap (l_handler.icon)
			end
		end

	pix_void: EV_PIXMAP is once Result := Gui.factory.new_pixmap ("ob_void.ico") end
	pix_integer: EV_PIXMAP is once Result := Gui.factory.new_pixmap ("ob_integer.ico") end
	pix_string: EV_PIXMAP is once Result := Gui.factory.new_pixmap ("ob_string.ico") end
	pix_character: EV_PIXMAP is once Result := Gui.factory.new_pixmap ("ob_character.ico") end
	pix_boolean: EV_PIXMAP is once Result := Gui.factory.new_pixmap ("ob_boolean.ico") end
	pix_double: EV_PIXMAP is once Result := Gui.factory.new_pixmap ("ob_double.ico") end
	pix_pointer: EV_PIXMAP is once Result := Gui.factory.new_pixmap ("ob_pointer.ico") end
	pix_special: EV_PIXMAP is once Result := Gui.factory.new_pixmap ("ob_special.ico") end
	pix_any: EV_PIXMAP is once Result := Gui.factory.new_pixmap ("ob_any.ico") end

end -- class ROSE_OBJECT_BROWSER_TREE_PROCESSOR
