indexing
	description: "Object processor"
	author: "Zoran Simic"
	date: "$Date: 2008/12/01 $"
	revision: "$Revision: #4 $"

deferred class
	ROSE_OBJECT_PROCESSOR

inherit

	ANY

	INTERNAL
		export
			{NONE} all
		end

feature -- Basic operations

	process (a_name: STRING; a_object: ANY) is
			-- Process 'a_object' with 'a_name'.
		do
			process_reference (a_name,Void,a_object)
		end

	process_fields (a_object: ANY) is
			-- Process all fields of 'a_object'.
		local
			i, n, l_type: INTEGER
			l_name: STRING
		do
			if a_object /= Void then
				from
					n := field_count (a_object)
					i := 1
				until
					i > n
				loop
					l_name := field_name (i,a_object)
					l_type := field_type (i,a_object)
					inspect l_type
					when Reference_type then
						process_reference (l_name,Void,field (i,a_object)) --class_name_of_type (l_type))
					when Integer_type then
						process_integer (l_name,integer_field (i,a_object))
					when Integer_8_type then
						process_integer_8 (l_name,integer_8_field (i,a_object))
					when Integer_16_type then
						process_integer_16 (l_name,integer_16_field (i,a_object))
					when Integer_64_type then
						process_integer_64 (l_name,integer_64_field (i,a_object))
					when Double_type then
						process_double (l_name,double_field (i,a_object))
					when Real_type then
						process_real (l_name,real_field (i,a_object))
					when Character_type then
						process_character (l_name,character_field (i,a_object))
					when Boolean_type then
						process_boolean (l_name,boolean_field (i,a_object))
					when Pointer_type then
						process_pointer (l_name,pointer_field (i,a_object))
					when Bit_type, Wide_character_type then
						--| FIXME: missing interface in INTERNAL
					when Expanded_type then
						--| FIXME: missing interface in INTERNAL
					else
						--| FIXME: report an error?
					end
					i := i + 1
				end
			end
		end

feature {NONE} -- Implementation

	process_reference (a_name, a_type: STRING; a_object: ANY) is
		local
			l_string: STRING
		do
			if a_object = Void then
				process_void (a_name,a_type)
			else
				l_string ?= a_object
				if l_string /= Void then
					process_string (a_name,l_string)
				elseif is_special (a_object) then
					process_special_object (a_name,a_object)
				else
					process_any (a_name,a_object)
				end
			end
		end

	process_special_object (a_name: STRING; a_object: ANY) is
			-- Process 'a_object' of type SPECIAL [] with 'a_name'
			-- 'a_name' is Void if the 'process'-ed object is a special
		require
			special_object: a_object /= Void and then is_special (a_object)
		local
			sa: SPECIAL [ANY]
			si: SPECIAL [INTEGER]
			si8: SPECIAL [INTEGER_8]
			si16: SPECIAL [INTEGER_16]
			si64: SPECIAL [INTEGER_64]
			sc: SPECIAL [CHARACTER]
			sb: SPECIAL [BOOLEAN]
			sd: SPECIAL [DOUBLE]
			sr: SPECIAL [REAL]
			sp: SPECIAL [POINTER]
		do
			sa ?= a_object
			si ?= a_object
			si8 ?= a_object
			si16 ?= a_object
			si64 ?= a_object
			sc ?= a_object
			sb ?= a_object
			sd ?= a_object
			sr ?= a_object
			sp ?= a_object
			if si /= Void then
				process_special_integer (a_name,si)
			elseif si8 /= Void then
				process_special_integer_8 (a_name,si8)
			elseif si16 /= Void then
				process_special_integer_16 (a_name,si16)
			elseif si64 /= Void then
				process_special_integer_64 (a_name,si64)
			elseif sc /= Void then
				process_special_character (a_name,sc)
			elseif sb /= Void then
				process_special_boolean (a_name,sb)
			elseif sd /= Void then
				process_special_double (a_name,sd)
			elseif sr /= Void then
				process_special_real (a_name,sr)
			elseif sp /= Void then
				process_special_pointer (a_name,sp)
			elseif sa /= Void then
				process_special_any (a_name,sa)
			end
		end

	process_void (a_name, a_type: STRING) is
		deferred
		end

	process_any (a_name: STRING; a_object: ANY) is
		require
			valid_object: a_object /= Void
		deferred
		end

	process_string (a_name: STRING; a_string: STRING) is
		require
			valid_string: a_string /= Void
		deferred
		end

	process_integer (a_name: STRING; a_integer: INTEGER) is
		deferred
		end

	process_integer_8 (a_name: STRING; a_integer: INTEGER_8) is
		deferred
		end

	process_integer_16 (a_name: STRING; a_integer: INTEGER_16) is
		deferred
		end

	process_integer_64 (a_name: STRING; a_integer: INTEGER_64) is
		deferred
		end

	process_double (a_name: STRING; a_double: DOUBLE) is
		deferred
		end

	process_real (a_name: STRING; a_real: REAL) is
		deferred
		end

	process_character (a_name: STRING; a_character: CHARACTER) is
		deferred
		end

	process_boolean (a_name: STRING; a_boolean: BOOLEAN) is
		deferred
		end

	process_pointer (a_name: STRING; a_pointer: POINTER) is
		deferred
		end

	process_special_integer (a_name: STRING; a_special: SPECIAL [INTEGER]) is
			-- Process 'a_special' of type SPECIAL [INTEGER] with 'a_name'
			-- 'a_name' is Void if the 'process'-ed object is a special
		require
			valid_special: a_special /= Void and then is_special (a_special)
		deferred
		end

	process_special_integer_8 (a_name: STRING; a_special: SPECIAL [INTEGER_8]) is
			-- Process 'a_special' of type SPECIAL [INTEGER_8] with 'a_name'
			-- 'a_name' is Void if the 'process'-ed object is a special
		require
			valid_special: a_special /= Void and then is_special (a_special)
		deferred
		end

	process_special_integer_16 (a_name: STRING; a_special: SPECIAL [INTEGER_16]) is
			-- Process 'a_special' of type SPECIAL [INTEGER_16] with 'a_name'
			-- 'a_name' is Void if the 'process'-ed object is a special
		require
			valid_special: a_special /= Void and then is_special (a_special)
		deferred
		end

	process_special_integer_64 (a_name: STRING; a_special: SPECIAL [INTEGER_64]) is
			-- Process 'a_special' of type SPECIAL [INTEGER_64] with 'a_name'
			-- 'a_name' is Void if the 'process'-ed object is a special
		require
			valid_special: a_special /= Void and then is_special (a_special)
		deferred
		end

	process_special_character (a_name: STRING; a_special: SPECIAL [CHARACTER]) is
			-- Process 'a_special' of type SPECIAL [CHARACTER] with 'a_name'
			-- 'a_name' is Void if the 'process'-ed object is a special
		require
			valid_special: a_special /= Void and then is_special (a_special)
		deferred
		end

	process_special_boolean (a_name: STRING; a_special: SPECIAL [BOOLEAN]) is
			-- Process 'a_special' of type SPECIAL [BOOLEAN] with 'a_name'
			-- 'a_name' is Void if the 'process'-ed object is a special
		require
			valid_special: a_special /= Void and then is_special (a_special)
		deferred
		end

	process_special_double (a_name: STRING; a_special: SPECIAL [DOUBLE]) is
			-- Process 'a_special' of type SPECIAL [DOUBLE] with 'a_name'
			-- 'a_name' is Void if the 'process'-ed object is a special
		require
			valid_special: a_special /= Void and then is_special (a_special)
		deferred
		end

	process_special_real (a_name: STRING; a_special: SPECIAL [REAL]) is
			-- Process 'a_special' of type SPECIAL [REAL] with 'a_name'
			-- 'a_name' is Void if the 'process'-ed object is a special
		require
			valid_special: a_special /= Void and then is_special (a_special)
		deferred
		end

	process_special_pointer (a_name: STRING; a_special: SPECIAL [POINTER]) is
			-- Process 'a_special' of type SPECIAL [POINTER] with 'a_name'
			-- 'a_name' is Void if the 'process'-ed object is a special
		require
			valid_special: a_special /= Void and then is_special (a_special)
		deferred
		end

	process_special_any (a_name: STRING; a_special: SPECIAL [ANY]) is
			-- Process 'a_special' of type SPECIAL [ANY] with 'a_name'
			-- 'a_name' is Void if the 'process'-ed object is a special
		require
			valid_special: a_special /= Void and then is_special (a_special)
		deferred
		end

end -- class ROSE_OBJECT_PROCESSOR
