indexing
	description: "Definition of an observable - an object that notifies it's observers when it has changed"
	author: "Zoran Simic"
	date: "$Date: 2008/12/01 $"
	revision: "$Revision: #4 $"

class
	ROSE_OBSERVABLE

feature -- Basic operations

	notify_observers is
			-- Notify all registered observers of a change in this observable.
		do
			if registered_observers /= Void then
				from
					registered_observers.start
				until
					registered_observers.off
				loop
					registered_observers.item.call ([])
					registered_observers.forth
				end
			end
		end

	add_observer (a_observer: PROCEDURE [ANY,TUPLE []]) is
			-- Add 'a_observer' to this observable.
		do
			if a_observer /= Void then
				if registered_observers = Void then
					create registered_observers.make
				end
				registered_observers.extend (a_observer)
			end
		end

	remove_observer (a_observer: PROCEDURE [ANY,TUPLE []]) is
			-- Remove 'a_observer' from this observable.
		do
			if a_observer /= Void and then registered_observers /= Void then
				registered_observers.prune_all (a_observer)
				if registered_observers.is_empty then
					registered_observers := Void
				end
			end
		end

feature {NONE} -- Implementation

	registered_observers: LINKED_LIST [PROCEDURE [ANY,TUPLE []]]
			-- Observers on this pool, called whenever pool is modified

end -- class ROSE_OBSERVABLE
