indexing
	description: "Communication output channel"
	author: "Zoran Simic"
	date: "$Date: 2008/12/01 $"
	revision: "$Revision: #11 $"

deferred class ROSE_OUTCHANNEL

inherit
	ROSE_CHANNEL
		redefine
			open
		end

feature -- Access

	accepted_verbosity: INTEGER
			-- Messages with importance lower than 'accepted_verbosity' are filtered out

feature -- Setting

	set_accepted_verbosity (a_level: INTEGER) is
			-- Filter output on this channel to 'a_level'.
		do
			accepted_verbosity := a_level
		end

feature -- Basic operations

	open (a_path: STRING) is
			-- Open this channel with 'a_path'.
		local
			l_path: STRING
			l_pair: PAIR [STRING,STRING]
		do
			l_path := a_path
			if l_path /= Void and then not l_path.is_empty and then l_path.item (1) = '(' then
				l_pair := Global.formatter.split_delimited (l_path,Void)
				if l_pair /= Void and then Global.convention.is_valid_verbosity_name (l_pair.first) then
					set_accepted_verbosity (Global.convention.verbosity (l_pair.first))
					l_path := l_pair.second
				end
			end
			Precursor (l_path)
		end

	output_message (a_level: INTEGER; a_message: STRING) is
			-- Output 'a_message' with 'a_level' on this channel.
		deferred
		end

	output_log (a_severity, a_id: INTEGER; a_class, a_feature, a_message: STRING) is
			-- Specific output for logs (redefinable)
		do
			output_message (v_log,default_log_format (a_severity,a_id,a_class,a_feature,a_message))
		end

	output_time_measure (a_tm: ROSE_TIME_MEASURE) is
			-- Specific output for time measures...
		do
			if a_tm /= Void then
				output_message (v_trace,a_tm.status)
			end
		end

	flush is
			-- Flush this channel.
		deferred
		end

feature {NONE} -- Implementation

	default_log_format (a_severity, a_id: INTEGER; a_class, a_feature, a_message: STRING): STRING is
			-- Log output default formating
		local
			l_now: ROSE_DATE
		do
			create l_now.make_now
			Result := Global.formatter.argumented ("-- $1: $2 [$3] $4.$5: $6%N",<<severity_meaning (a_severity),l_now,a_id,a_class,a_feature,a_message>>)
		end

end
