indexing
	description: "Buffer channel: output is simply kept in a buffer"
	author: "Zoran Simic"
	date: "$Date: 2009/03/02 $"
	revision: "$Revision: #9 $"

class ROSE_OUTCHANNEL_BUFFER

inherit
	ROSE_OUTCHANNEL

create
	make

feature {NONE} -- Initialization

	make is
			-- Create a default buffer channel
		do
			accepted_verbosity := minimum_verbosity
		end

feature -- Access

	id: STRING is "buffer"
			-- Channel type id

	buffer: LINKED_QUEUE [PAIR [INTEGER,STRING]]
			-- Actual buffer

feature -- Status report

	is_open: BOOLEAN is
			-- Is this channel initialized ?
		do
			Result := buffer /= Void
		end

	is_empty: BOOLEAN is
			-- Is buffer empty ?
		do
			Result := buffer = Void or else buffer.is_empty
		end

feature -- Basic operations

	output_message (a_level: INTEGER; a_message: STRING) is
			-- Output 'a_message' with 'a_level' on this channel.
		do
			if a_message /= Void and then buffer /= Void and then a_level >= accepted_verbosity then
				buffer.put (new_pair (a_level,a_message))
			end
		end

	flush is
			-- Flush this channel.
		do
			--| Do nothing
		end

	flush_on (a_channel: ROSE_OUTCHANNEL) is
			-- Flush this buffer on 'a_channel'.
			-- Buffer is empty after this operation if 'a_channel' is open.
		local
			l_buffer: like buffer
		do
			if buffer /= Void
			and then a_channel /= Void
			and then a_channel.is_open then
				from
					l_buffer := buffer.twin
				until
					l_buffer.is_empty
				loop
					a_channel.output_message (l_buffer.item.first,l_buffer.item.second)
					l_buffer.remove
				end
				a_channel.flush
				buffer.wipe_out
			end
		end

	flush_to_string: STRING is
			-- All messages flushed to resulting STRING
			-- Buffer is empty after this operation
		do
			Result := flush_filtered_to_string (accepted_verbosity)
		ensure
			is_empty: is_empty
		end

	flush_filtered_to_string (a_level: INTEGER): STRING is
			-- All messages of greater importance than 'a_level' flushed to resulting STRING
			-- Buffer is empty after this operation
		do
			if buffer /= Void and then buffer.count > 0 then
				from
					create Result.make (buffer.count*80)
				until
					buffer.is_empty
				loop
					if buffer.item.first >= a_level then
						Result.append (Global.prefs.decorated_verbose_message (buffer.item.first,buffer.item.second))
					end
					buffer.remove
				end
			end
		ensure
			is_empty: is_empty
		end

feature -- Output

	help: STRING is "Output to a buffer in memory"
			-- Help on this command

feature {NONE} -- Implementation

	do_open is
			-- Open efectively this channel with 'parameters'.
		do
			create buffer.make
		end

	do_close is
			-- Close effectively this channel.
		do
			buffer := Void
		end

	new_pair (a_level: INTEGER; a_message: STRING): PAIR [INTEGER,STRING] is
			-- create a new pair
		do
			create Result.make (a_level,a_message)
		end

end -- class ROSE_OUTCHANNEL_BUFFER
