indexing
	description: "Database output channel"
	author: "Zoran Simic"
	date: "$Date: 2010/03/17 $"
	revision: "$Revision: #6 $"

class ROSE_OUTCHANNEL_DATABASE

inherit
	ROSE_OUTCHANNEL
		redefine
			output_log
		end

feature -- Access

	id: STRING is "db"
			-- Channel type id

	logical: STRING
	table: STRING
	id_column: STRING
			-- Database coordinates

	accessor: ROSE_DB_ACCESSOR
			-- Database accessor used for the connection to DB

feature -- Status report

	is_open: BOOLEAN is
			-- Is this channel open ?
		do
			Result := accessor /= Void and then accessor.is_connected
		end

feature -- Basic operations

	output_message (a_level: INTEGER; a_message: STRING) is
			-- Output 'a_message' with 'a_level' on this channel.
		do
			--| Do nothing
		end

	output_log (a_severity, a_id: INTEGER; a_class, a_feature, a_message: STRING) is
			-- Specific output for logs (redefinable)
		do
			if a_id > 0 and then a_message /= Void and then sql_insert_log /= Void and then not is_writing_log then
				is_writing_log := True
				sql_insert_log.execute (Void, <<a_id, create {ROSE_DATE}.make_now_utc, a_class, a_feature, a_severity, a_message>>)
				accessor.commit
				is_writing_log := False
			end
		end

	flush is
			-- Flush this channel.
		do
			--| Do nothing
		end

feature -- Output

	help: STRING is "Output to standard output file"
			-- Help on this command

feature -- Constants

	default_table: STRING is "message"
	default_id_column: STRING is "id"
			-- Default database coordinates


feature {NONE} -- Implementation

	is_writing_log: BOOLEAN
			-- Are we currently writing a log to the underlying DB?
			-- This is useful in the case where our own writing fails, and we end up writing the fact that our write failed... leading to an infinite loop

	do_open is
			-- Open effectively this channel with 'parameters'.
			-- Format: <logical DB name>[.<table>[.<id column>]]
			-- Default for <table> is 'default_table', default for <id column> is 'default_id_column'
		local
			l_pair: PAIR [STRING, STRING]
		do
			do_close
			l_pair := Global.formatter.split_on_first (parameters, ".", Void)
			if l_pair /= Void and then l_pair.first /= Void and then not l_pair.first.is_empty then
				logical := l_pair.first
				l_pair := Global.formatter.split_on_first (l_pair.second, ".", Void)
				if l_pair /= Void then
					table := l_pair.first
					id_column := l_pair.second
				end
				if table = Void or else table.is_empty then
					table := default_table
				end
				if id_column = Void or else id_column.is_empty then
					id_column := default_id_column
				end
				create accessor.make_accessor (logical, Void)
				accessor.connect
				if is_open then
					sql_insert_log := accessor.new_execution_predicate (
						"INSERT INTO " + table +
						" (" + id_column + ",time,class_name,feature_name,severity_id,message) VALUES (?,?,?,?,?,?)")
				end
			end
		end

	do_close is
			-- Close effectively this channel.
		do
			if accessor /= Void then
				accessor.disconnect
			end
			accessor := Void
			sql_insert_log := Void
			logical := Void
			table := Void
			id_column := Void
		end

	sql_insert_log: EXECUTION_PREDICATE
			-- Execution predicate used to insert log messages

end
