indexing
	description: "Output channel for Eiffel Vision error reporter"
	author: "Zoran Simic"
	date: "$Date: 2008/12/01 $"
	revision: "$Revision: #7 $"

class
	ROSE_OUTCHANNEL_EV_ERROR_REPORTER

inherit
	ROSE_OUTCHANNEL
		redefine
			new_representative, output_log
		end

create
	make

feature -- Initialization

	make is
			-- Create a default error-reporter channel.
		do
			accepted_verbosity := v_warning
		end

feature -- Access

	id: STRING is "ev_error"
			-- Channel type id

	new_representative: like Current is
			-- New independent version of Current service.
			-- If you want this service to be shared (unique instance used for the whole system),
			-- return Current (no clone)
		do
			Result := Current
		end

feature -- Status report

	is_open: BOOLEAN is True
			-- Is this channel open ?
			--| Always True when launched properly using ROSE_PILOT_GUI

feature -- Basic operations

	output_message (a_level: INTEGER; a_message: STRING) is
			-- Output 'a_message' with 'a_level' on this channel.
		do
			if a_message /= Void and then a_level >= accepted_verbosity then
				gui_access.gui.error_reporter.output_message (a_level,a_message)
			end
		end

	output_log (a_severity, a_id: INTEGER; a_class, a_feature, a_message: STRING) is
			-- Specific output for logs (redefinable)
		do
			--| Do nothing
		end

	flush is
			-- Flush this channel.
		do
			--| Do nothing
		end

feature -- Output

	help: STRING is "Output to Eiffel Vision error reporter"
			-- Help on this command

feature {NONE} -- Implementation

	do_open is
			-- Open effectively this channel with 'parameters'.
		do
			--| Do nothing
		end

	do_close is
			-- Close effectively this channel.
		do
			--| Do nothing
		end

	gui_access: ROSE_SHARED_GLOBAL_GUI is
			-- Access to 'gui', not inherited because of a redefinition problem with 'is_equal'
		once
			create Result
		end

end
