indexing
	description: "File output channel"
	author: "Zoran Simic"
	date: "$Date: 2008/12/01 $"
	revision: "$Revision: #7 $"

class ROSE_OUTCHANNEL_FILE

inherit
	ROSE_OUTCHANNEL

create
	make

feature {NONE} -- Initialization

	make is
			-- Create a default file channel.
		do
			accepted_verbosity := v_output
		end

feature -- Access

	id: STRING is "file"
			-- Channel type id

	file: FILE
		-- Actual output file used for this channel

feature -- Status report

	is_open: BOOLEAN is
			-- Is this channel open ?
		do
			Result := file /= Void and then not file.is_closed
		end

	append_mode: BOOLEAN
			-- Is file to be opened in append-mode ?

feature -- Basic operations

	output_message (a_level: INTEGER; a_message: STRING) is
			-- Output 'a_message' with 'a_level' on this channel.
		do
			if is_open and then a_message /= Void and then a_level >= accepted_verbosity then
				file.put_string (Global.prefs.decorated_verbose_message (a_level,a_message))
			end
		end

	flush is
			-- Flush this channel.
		do
			if is_open then
				file.flush
			end
		end

feature -- Output

	help: STRING is "Output to a file"
			-- Help on this command

feature {NONE} -- Implementation

	do_open is
			-- Open effectively this channel with 'parameters'.
		local
			rescued: STRING
			l_name: STRING
		do
			if rescued /= Void then
				error.signal (id,"Can not open file '$1':%N$2",<<parameters,rescued>>)
			else
				l_name := clone (parameters)
				if l_name /= Void and then not l_name.is_empty then
					if l_name.item (1) = '>' then
						l_name.remove (1)
						if l_name.item (1) = '>' then
							l_name.remove (1)
							append_mode := True
						end
					end
					create {PLAIN_TEXT_FILE} file.make (l_name)
					if not file.exists or else file.is_writable then
						if append_mode then
							file.open_append
						else
							file.open_write
						end
					else
						error.signal (id,"Can not write to file '$1': file is not writable!",<<l_name>>)
					end
				else
					error.signal (id,"No file path specified!",Void)
				end
			end
		rescue
			if rescued = Void then
				rescued := last_exception_explanation
				retry
			end
		end

	do_close is
			-- Close effectively this channel.
		do
			flush
			if is_open then
				file.close
			end
			file := Void
			append_mode := False
		end

end -- class ROSE_OUTCHANNEL_FILE
