indexing
	description: "Standard files output channel"
	author: "Zoran Simic"
	date: "$Date: 2008/12/01 $"
	revision: "$Revision: #13 $"

class ROSE_OUTCHANNEL_STANDARD

inherit
	ROSE_OUTCHANNEL
		redefine
			new_representative
		end

create
	make

feature {NONE} -- Initialization

	make is
			-- Create a default standard-output channel.
		do
			is_interactive := True
			accepted_verbosity := minimum_verbosity
		end

feature -- Access

	id: STRING is "standard"
			-- Channel type id

	new_representative: like Current is
			-- New independent version of Current service.
			-- If you want this service to be shared (unique instance used for the whole system),
			-- return Current (no clone)
		do
			Result := Current
		end

feature -- Status report

	is_open: BOOLEAN is
			-- Is this channel open ?
		do
			Result := io.output.is_open_write
		end

feature -- Basic operations

	output_message (a_level: INTEGER; a_message: STRING) is
			-- Output 'a_message' with 'a_level' on this channel.
		do
			if a_message /= Void and then is_open and then a_level >= accepted_verbosity then
				if a_level >= v_log and then io.error.is_open_write then
					io.error.put_string (Global.prefs.decorated_verbose_message (a_level,a_message))
				else
					io.put_string (Global.prefs.decorated_verbose_message (a_level,a_message))
				end
			end
		end

	flush is
			-- Flush this channel.
		do
				--| FIXME: flush deactivated because of a .NET eiffel runtime problem
--			if is_open then
--				io.output.flush
--			end
--			if io.error.is_open_write then
--				io.error.flush
--			end
		end

feature -- Output

	help: STRING is "Output to standard output file"
			-- Help on this command

feature {NONE} -- Implementation

	do_open is
			-- Open effectively this channel with 'parameters'.
		do
				--| Relies on standard IO
			parameters := Void
		end

	do_close is
			-- Close effectively this channel.
		do
				--| Standard files are not closed by this channel
		end

	prefixed_message (a_class, a_feature, a_message: STRING): STRING is
			-- 'a_message' eventually prefixed with 'a_class'.'a_feature'
		do
			if a_class = Void then
				Result := a_feature
			elseif a_feature = Void then
				Result := a_class
			else
				Result := a_class + "." + a_feature
			end
			if Result = Void then
				Result := a_message
			elseif a_message /= Void then
				Result := Result + ": " + a_message
			end
		end

end -- class ROSE_OUTCHANNEL_STANDARD
