indexing
	description: "Definition of a color palette"
	author: "Zoran Simic"
	date: "$Date: 2009/10/08 $"
	revision: "$Revision: #5 $"

class
	ROSE_PALETTE_CHOOSER

inherit
	ROSE_SHARED_GLOBAL_GUI

feature {NONE} -- Initialization

	initialize_dialog is
			-- Initialize 'dialog', if need be.
		local
			l_main_box, l_box: EV_BOX
			l_widget: EV_WIDGET
		do
			if dialog = Void then
				dialog := gui.factory.new_dialog ("Palette editor", "palette.ico", 0, 0)
				l_main_box := gui.factory.new_vbox (dialog, -4, 2)
				-- Size, attributes
				l_box := gui.factory.new_hbox (l_main_box, 0, 2)
				p_size := gui.factory.new_combo (l_box, "Size: ", <<2, 4, 8, 16, 32, 64, 128>>, 0, agent set_size, 50)
				p_size.return_actions.extend (agent set_size (p_size))
				set_dynamic (False)
				cb_cyclic := gui.factory.new_check_button (l_box, False, "Cyclic", Void, 1)
				cb_tolerant := gui.factory.new_check_button (l_box, False, "Tolerant", Void, 1)
				cb_cyclic.set_tooltip ("Indicates whether the indexes are cyclic")
				-- Background/foreground colors
				l_box := gui.factory.new_hbox (l_main_box, 0, 4)
				create bg_view.make_with_text (l_box, "Background", Void)
				create fg_view.make_with_text (l_box, "Foreground", Void)
				bg_view.set_modifiable (True)
				fg_view.set_modifiable (True)
				-- Palette colors
				colors_box := gui.factory.new_vbox (l_main_box,0,-2)
				-- Buttons
				l_box := gui.factory.new_hbox (l_main_box, 0, 2)
				l_widget := gui.factory.new_button (l_box, "Reset", agent on_reset, 60)
				l_widget := gui.factory.new_label (l_box, Void, 0)
				l_widget := gui.factory.new_button (l_box, "OK", agent on_ok, 60)
				l_widget := gui.factory.new_button (l_box, "Cancel", agent on_cancel, 60)
			end
		end

feature -- Access

	palette: ROSE_PALETTE
			-- Composed palette

feature -- Basic operations

	show (a_palette: like palette) is
			-- Show this palette chooser dialog for 'a_palette'.
		do
			initialize_dialog
			palette := a_palette
			create backup.make (8)
			if palette /= Void then
				backup.copy_from (palette)
			end
			refresh_palette_view
			dialog.show
		end

	show_modal (a_widget: EV_WIDGET; a_palette: like palette) is
			-- Show this palette chooser modally to 'a_widget', and for 'a_palette'
		require
			widget_ok: a_widget /= Void
		do
			initialize_dialog
			palette := a_palette
			create backup.make (8)
			if palette /= Void then
				backup.copy_from (palette)
			end
			refresh_palette_view
			dialog.show_modal_to_window (gui.parent_window (a_widget))
		end

	set_dynamic (a_on: BOOLEAN) is
			-- Allow or disallow dynamic editing
		do
			initialize_dialog
			gui.enable_sensitive (p_size, a_on)
			gui.enable_sensitive (cb_cyclic, a_on)
			gui.enable_sensitive (cb_tolerant, a_on)
		end

feature {NONE} -- Implementation

	dialog: EV_DIALOG							-- Dialog holding the palette chooser
	p_size: ROSE_COMBO_BOX						-- Size combo
	cb_cyclic, cb_tolerant: EV_CHECK_BUTTON		-- Checkboxes
	colors_box: EV_BOX							-- Box holding the colors of the palette
	bg_view, fg_view: ROSE_COLOR_VIEW			-- Background/foreground colors
	backup: like palette						-- Used for when 'Cancel' is pressed
	colors: ARRAY [ROSE_COLOR_VIEW]				-- Colors

	refresh_palette_view is
			-- Refresh palette view
		local
			l_color_box: EV_BOX
			l_view: ROSE_COLOR_VIEW
			i, j: INTEGER
		do
			gui.output (p_size, backup.count.out, Void)
			bg_view.set_color (backup.background)
			bg_view.set_index (-1)
			fg_view.set_color (backup.foreground)
			fg_view.set_index (0)
			gui.enable_select (cb_cyclic, backup.is_cyclic)
			gui.enable_select (cb_tolerant, backup.is_tolerant)
			colors_box.wipe_out
			from
				create colors.make (1, backup.count)
				i := 1
			until
				i > backup.count
			loop
				j := (i-1) \\ 16
				if j = 0 then
					l_color_box := gui.factory.new_hbox (colors_box, 0, 2)
				end
				create l_view.make (l_color_box, backup.color (i))
				l_view.set_index (i)
				l_view.set_modifiable (True)
				l_view.set_pebble (l_view)
				l_view.set_name ("Color " + i.out)
				l_view.drop_actions.extend (agent on_drop (i, ?))
				colors.put (l_view, i)
				i := i + 1
			end
			dialog.set_size (dialog.item.minimum_width, dialog.item.minimum_height)
		end

	on_drop (a_index: INTEGER; a_object: ANY) is
			-- Called when a color view is dropped on another
		local
			l_source: ROSE_COLOR_VIEW
		do
			l_source ?= a_object
			if l_source /= Void then
				backup.apply_gradient (l_source.index, a_index)
				refresh_palette_view
			end
		end

	on_reset is
			-- Called when 'Reset' button is pressed (we re-apply backup)
		do
			if palette /= Void then
				backup.copy_from (palette)
				refresh_palette_view
			end
		end

	on_ok is
			-- Called when 'OK' is pressed
		do
			on_apply
			dialog.hide
		end

	on_apply is
			-- Called when 'Apply' is pressed
		do
			if palette = Void then
				palette := backup
			else
				palette.copy_from (backup)
			end
		end

	on_cancel is
			-- Called when 'Cancel' is pressed
		do
			palette := Void
			dialog.hide
		end

	set_size (a_combo: ROSE_COMBO_BOX) is
			-- Called when a size is selected from 'p_size' combo.
		local
			l_string: STRING
		do
			l_string := a_combo.text
			if l_string /= Void and then l_string.is_integer then
				backup.resize (l_string.to_integer)
				refresh_palette_view
			end
		end

end
