indexing
	description: "Make rose periodicity_id's generally available."
	status: "Development"
	author: "David Stevens"
	date: "$Date: 2008/12/01 $"
	revision: "$Revision: #12 $"

class
	ROSE_PERIODICITY_IDS

inherit
	ROSE_ID_RETRIEVER

feature -- Access

	ad_hoc_periodicity_id: INTEGER is
	-- AD HOC
		once
			Result := rose_id.item (periodicity_statement ("AD HOC"))
		end
	daily_periodicity_id: INTEGER is
	-- Daily
		once
			Result := rose_id.item (periodicity_statement ("DAILY"))
		end
	weekly_periodicity_id: INTEGER is
	-- Weekly
		once
			Result := rose_id.item (periodicity_statement ("WEEKLY"))
		end
	monthly_periodicity_id: INTEGER is
	-- Monthly
		once
			Result := rose_id.item (periodicity_statement ("MONTHLY"))
		end
	quarterly_periodicity_id: INTEGER is
	-- Quarterly
		once
			Result := rose_id.item (periodicity_statement ("QUARTERLY"))
		end
	quarterly_cumulative_periodicity_id: INTEGER is
	-- Quarterly cumulative
		once
			Result := rose_id.item (periodicity_statement ("QUARTERLY CUMULATIVE"))
		end
	semi_annual_periodicity_id: INTEGER is
	-- Semi annual
		once
			Result := rose_id.item (periodicity_statement ("SEMI ANNUAL"))
		end
	annual_periodicity_id: INTEGER is
	-- Annual
		once
			Result := rose_id.item (periodicity_statement ("ANNUAL"))
		end
	long_term_periodicity_id: INTEGER is
	-- Long term
		once
			Result := rose_id.item (periodicity_statement ("LONG TERM"))
		end
	three_year_periodicity_id: INTEGER is
	-- Three year
		once
			Result := rose_id.item (periodicity_statement ("THREE YEAR"))
		end
	annual_trailing_periodicity_id: INTEGER is
	-- Annual trailing
		once
			Result := rose_id.item (periodicity_statement ("ANNUAL TRAILING"))
		end
	trimester_periodicity_id: INTEGER is
	-- Trimester
		once
			Result := rose_id.item (periodicity_statement ("TRIMESTER"))
		end
	trimester_cumulative_periodicity_id: INTEGER is
	-- Trimester cumulative
		once
			Result := rose_id.item (periodicity_statement ("TRIMESTER CUMULATIVE"))
		end
	nine_month_periodicity_id: INTEGER is
	-- Nine month
		once
			Result := rose_id.item (periodicity_statement ("NINE MONTH"))
		end
	real_time_periodicity_id: INTEGER is
	-- Real time
		once
			Result := rose_id.item (periodicity_statement ("REAL TIME"))
		end

	-- Feb 22, 2005 Added by India Team
	interim_periodicity_id: INTEGER is
	-- Interim
		once
			Result := rose_id.item (periodicity_statement ("INTERIM"))
		end

	-- Sep 06, 2007 Added by India Team
	annual_forward_periodicity_id: INTEGER is
	-- Annual forward
		once
			Result := rose_id.item (periodicity_statement ("ANNUAL FORWARD"))
		end


	periodicity_name_from_id(a_id : INTEGER) : STRING is
	-- Periodicity name corresponding to 'a_id'.
		do
			if name_from_id_map = Void then
				create_map_tables
			end
			Result := name_from_id_map.item(a_id)
		end

	id_from_periodicity_name(a_id : STRING) : INTEGER is
	-- Periodicity id corresponding to 'a_id'.
		do
			if id_from_name_map = Void then
				create_map_tables
			end
			Result := id_from_name_map.item(a_id)
		end

	periodicity_labels: ARRAY[STRING] is
	-- Periodicity labels
		once
			Result := <<"periodicity", "id">>
		end

	periodicity_content: ROSE_ARRAY2[ANY] is
	-- Values from periodicity table.
		local
			l_ids:ARRAY[INTEGER]
			i,k:INTEGER
			l_name:STRING
		once
			periodicity_name_from_id(monthly_periodicity_id).do_nothing
			l_ids:=name_from_id_map.current_keys
			create result.make(l_ids.count,2)
			from i:=1 until i>l_ids.count loop
				k:=l_ids.item(i)
				l_name:=periodicity_name_from_id(k)
				result.put(l_name,i,1)
				result.put(k.out,i,2)
				i:=i+1
			end
		end

feature {NONE} -- Implementation

	name_from_id_map: HASH_TABLE [STRING, INTEGER]
	-- Key: periodicity id; Value: periodicity name
	id_from_name_map: HASH_TABLE [INTEGER, STRING]
	-- Key: periodicity name; Value: periodicity id

	create_map_tables is
	-- Populate map tables after retrieving the records from periodicity table.
		local
			l_predicate : COLLECTING_PREDICATE[SUPPLIER_PROCESS]
		do
			create name_from_id_map.make(10)
			create id_from_name_map.make(10)
			create l_predicate.make(servers.rose_entity_foreground_server,"select periodicity_id as process_id,name from periodicity")
			l_predicate.retrieve(Void,Void)
			from l_predicate.array.start until l_predicate.array.off loop
				l_predicate.array.current_item.name.prune_all_trailing(' ')
				name_from_id_map.force(l_predicate.array.current_item.name,l_predicate.array.current_item.process_id)
				id_from_name_map.force(l_predicate.array.current_item.process_id,l_predicate.array.current_item.name)
				l_predicate.array.forth
			end
		end

	periodicity_statement (a_value: STRING): STRING is
	-- Select statement to retrieve periodicity id.
		require
			a_value_not_void: a_value /= Void
		do
			Result := "select periodicity_id from periodicity where name = '"
			Result.append (a_value)
			Result.append ("'")
		end

			end
