note

	description:

		"GUI pilot, using Eiffel Vision2"

	author: "Zoran Simic"
	last_editor: "$Author: zs $"
	date: "$Date: 2011/03/24 $"
	revision: "$Revision: #24 $"
	path: "$File: //rose/source/kernel/infrastructure/framework/gui/rose_pilot_gui_base.e $"

class
	ROSE_PILOT_GUI_BASE

inherit
	ROSE_PILOT_BASE
		redefine
			prepare, register, run, run_application
		end
	ROSE_SHARED_GLOBAL_GUI
		undefine
			default_create
		end

create
	default_create

feature -- Initialization

	prepare is
			-- Prepare this pilot.
		do
			global.io_handler.set_output (Void)
		end

	register is
			-- Register this pilot's specifics.
		do
			set_show_splash_screen (True)
			Precursor
			global.app.set_gui (True)
			global.dependency_manager.register (create {ROSE_SYSTEM_DEPENDENCY_GUI})
		end

feature -- Access

	is_splash_screen_shown: BOOLEAN
			-- Should splash screen be shown when application starts?

feature -- Basic operations

	run is
			-- Run the application and exit properly (with according status code).
		local
			rescued: STRING
		do
			if rescued /= Void then
				error.signal (global.app.program_name,"Unexpected exception caught!%NTrace:%N$1",<<rescued>>)
				launch (agent run_gui)
			elseif is_running_from_user_folder then
				launch (agent refuse_to_run)
			else
				global.add_cleanup_agent (agent gui.destroy)
				launch (agent run_application)
			end
			exit
		rescue
			if rescued = Void then
				rescued := last_exception_explanation
				retry
			end
		end

	run_application is
			-- Run the application, this defines every application's generic behaviour.
		do
			gui.show_splash_screen (is_splash_screen_shown)
				-- Refresh framework windows at startup
			revive (gui.console)
			revive (gui.error_reporter)
			revive (gui.quit)
			revive (gui.monitor)
			error.flush
			Precursor
			gui.show_splash_screen (False)
			run_gui
		end

	run_gui is
			-- This is executed by 'launch'.
		do
			if error.signaled then
				gui.error_reporter.show_fatal_error
			else
					--| The following instruction terminates the event loop
					--| if no window were opened via framework by initial command
				gui.close_window (Void)
			end
		end

	set_show_splash_screen (a_show: BOOLEAN) is
			-- Set 'is_splash_screen_shown'
		do
			is_splash_screen_shown := a_show
		ensure
			is_splash_screen_shown_set: is_splash_screen_shown = a_show
		end

feature {NONE} -- Implementation

	refuse_to_run is
			-- This is called if the user is running an exe copied to their Desktop
		local
			l_dialog: ROSE_DIALOG
			l_icon: STRING
			l_hbox, l_box: EV_BOX
			l_pixmap: EV_PIXMAP
			l_label: EV_LABEL
			l_button: EV_BUTTON
		do
			gui.show_splash_screen (True)
			create l_dialog
			l_icon := global.app.info.icon
			if l_icon /= Void and then not l_icon.is_empty then
				l_dialog.set_icon (l_icon)
			end
			l_dialog.set_title ("Executable should not be run from Desktop")
			l_dialog.show_ok_button (False)
			l_hbox := gui.factory.new_hbox (l_dialog.main_box, 6, 16)
			l_box := gui.factory.new_vbox (l_hbox, 6, 4)
			l_pixmap := gui.rep.pixmaps.error_pixmap.twin
			l_pixmap.set_minimum_size (l_pixmap.width, l_pixmap.height)
			l_box.extend (l_pixmap)
			l_box.disable_item_expand (l_pixmap)
			l_label := gui.factory.new_label (l_hbox, global.formatted (refuse_to_run_message, <<global.app.program_path, global.app.info.contacts>>), 0)
			l_label.set_foreground_color (gui.rep.colors.dark_red)
			l_hbox := gui.factory.new_hbox (l_dialog.main_box, 0, 12)
			l_label := gui.factory.new_label (l_hbox, Void, 0)
			l_button := gui.factory.new_button (l_hbox, " Show me production executable ", agent show_prod_exe (l_dialog), 1)
			l_button := gui.factory.new_button (l_hbox, " Show me this executable ", agent show_current_exe, 1)
			l_label := gui.factory.new_label (l_hbox, Void, 0)
			l_label := gui.factory.new_label (l_dialog.main_box, Void, -10)
			l_dialog.show_modal (gui.splash_screen.dialog)
			global.io_handler.error.die (0, Void, Void, Void)
		end

	show_prod_exe (a_dialog: ROSE_DIALOG) is
			-- Open explorer on the exe in production corresponding to current exe
		local
			l_center, l_exe, l_path: STRING
		do
			l_center := global.app.service_center
			if l_center = Void or else l_center.is_empty then
				gui.dialogs.error (a_dialog, "Executable does not conform to specification", "Unfortunately, this exe was not built using the standard AXA Rosenberg development tools...%NCan't determine where it should be coming from (no service center defined)", Void)
			else
				l_exe := global.app.info.internal_name
				l_path := global.formatter.composed_path (<<global.deliveries.production.folder, l_center, l_exe>>)
				if global.is_file_exists (l_path) then
					global.explore (l_path)
				else
					gui.dialogs.error (a_dialog, "Production exe not found", "The corresponding exe coulnt not be found, it is supposed to live in:%N$1", <<l_path>>)
				end
			end
		end

	show_current_exe is
			-- Open explorer on the current exe
		do
			global.explore (global.app.program_path)
		end

	refuse_to_run_message: STRING is
			-- Message shown in dialog for 'refuse_to_run'
"[
AXA Rosenberg application security and configuration requirements prevent
running them from C:\Documents and Settings\...
You probably took a copy of the executable instead of a shortcut to it.

Please proceed as follows:
- Delete $1
- Run this application as a Citrix published app
- Or run it from the production network share

The contacts for this application are:
$2
]"

	launch (a_agent: PROCEDURE [ANY, TUPLE]) is
			-- Launch Eiffel Vision event loop if not started.
		do
			if not gui.ev_application.is_destroyed and then not gui.ev_application.is_launched then
				gui.ev_application.uncaught_exception_actions.extend (agent exception_callback)
				gui.ev_application.post_launch_actions.extend (a_agent)
				gui.ev_application.launch
			end
		end

	exception_callback (a_exception: EXCEPTION) is
			-- Called whenever an unexpected exception is caught during the handling of an event.
		do
			error.signal (global.app.program_name, last_exception_explanation, Void)
			error.flush_and_acknowledge
		end

	revive (a_command: RCMD_WINDOW) is
			-- Revive 'a_command' if needed.
		do
			if a_command /= Void then
				a_command.auto_initialize_window
			end
		end

	is_running_from_user_folder: BOOLEAN is
			-- Is the current program running from the user's desktop or user's document folder?
		do
			Result := global.app.program_path.as_lower.substring_index ("documents and settings", 1) = 4
		end

end
