indexing
	description : "Platform dependent definitions"
	author : "Zoran Simic"
	date: "$Date: 2008/12/01 $"
	revision: "$Revision: #5 $"

class
	ROSE_PLATFORM

inherit
	DEBUG_OUTPUT
		redefine
			default_create, out
		end

create
	default_create

feature {NONE} -- Initialization

	default_create is
		local
			l_platform: PLATFORM
			l_env: ROSE_EXECUTION_ENVIRONMENT
		do
			create l_platform
			if l_platform.is_dotnet then
				name := dotnet
			elseif l_platform.is_windows then
				create l_env
				if l_env.is_running_64_bit then
					name := win64
				else
					name := win32
				end
			elseif l_platform.is_unix then
				name := unix
			elseif l_platform.is_vms then
				name := vms
			else
				name := unknown
			end
		ensure then
			valid_name: is_name_valid (name)
		end

feature -- Access

	name: STRING
			-- Name of this platform

	out, debug_output: STRING is
			-- String that should be displayed in debugger to represent `Current'.
		do
			Result := name
		end

	is_windows: BOOLEAN is do Result := is_win32 or else is_win64 end
	is_win32: BOOLEAN is do Result := name = win32 end
	is_win64: BOOLEAN is do Result := name = win64 end
	is_vms: BOOLEAN is do Result := name = vms end
	is_dotnet: BOOLEAN is do Result := name = dotnet end
	is_unix: BOOLEAN is do Result := name = unix end
	is_unknown: BOOLEAN is do Result := name = unknown end

	win32: STRING is "win32"
	win64: STRING is "win64"
	vms: STRING is "VMS"
	dotnet: STRING is ".NET"
	unix: STRING is "UNIX"
	unknown: STRING is "unknown"

	is_name_valid (a_name: STRING): BOOLEAN is
			-- Is 'a_name' a valid platform name?
		do
			Result := a_name = win32
				or else a_name = win64
				or else a_name = vms
				or else a_name = dotnet
				or else a_name = unix
		end

invariant
	valid_name: is_name_valid (name)

end
