indexing

	description: "[
        Widgets that display a regression line,
        and confidence intervals
        for the best fit between two selected datasets.
		]"

	author: "Colin Adams"
	last_editor: "$Author: c4 $"
	date: "$Date: 2009/03/30 $"
	revision: "$Revision: #2 $"
	path: "$File: //rose/source/kernel/infrastructure/widgets/charts/rose_plot_correlation.e $"

class	ROSE_PLOT_CORRELATION

inherit

	WITH_WIDGET
		redefine
			widget
		end

	ROSE_PLOT_REGRESSION_ROUTINES

	LAYOUT_ASSERTION_ROUTINES
		export {NONE} all end

	ROSE_SHARED_GLOBAL_GUI
		export {NONE} all end

create

	make

feature {NONE} -- Initialization

	make (a_plots: like plots) is
			-- Initialize `Current'.
		require
			a_plots_not_void: a_plots /= Void
			at_least_two_plots: a_plots.count >= 2
		do
			plots := a_plots
			create widget
			create selector_bar
			create independent_selector.make_with_autocomplete_restricted
			independent_selector.set_tooltip ("Independent variable")
			create dependent_selector.make_with_autocomplete_restricted
			Dependent_selector.set_tooltip ("Dependent variable")
			create go_button.make_with_text_and_action ("Go", agent show_graph)
			go_button.set_pixmap (Gui.factory.new_pixmap ("icon_green_arrow.ico").twin)
			go_button.disable_sensitive
			selector_bar.extend (dependent_selector)
			selector_bar.extend (independent_selector)
			selector_bar.extend (go_button)
			selector_bar.disable_item_expand (go_button)
			widget.extend (selector_bar)
			widget.disable_item_expand (selector_bar)
			fill_selectors
			independent_selector.select_first_item
			dependent_selector.select_first_item
			independent_selector.select_actions.extend (agent notify_selection_made)
			dependent_selector.select_actions.extend (agent notify_selection_made)
			initialized := True
		ensure
			plots_set: plots = a_plots
			initialized: initialized
			go_button_disabled: not go_button.is_sensitive
			first_independent_item_selected: independent_selector.current_index = 1
			first_dependent_item_selected: dependent_selector.current_index = 1
		end

feature -- Access

	window_title: STRING is "correlation"
			-- Title for window if opened separately using `open_in_window'
	
	window_icon_name: STRING is "graph3.ico"
			-- Name of icon used for window if opened separately `open_in_window'

	widget: EV_VERTICAL_BOX
			-- Top-level widget

feature -- Status report

	initialized: BOOLEAN
			-- Has creation procedure completed?

feature {INVARIANT_ASSUMING} -- Widgets

	selector_bar: EV_HORIZONTAL_BOX
			-- Container for selector widgets

	independent_selector: ROSE_COMBO_BOX
			-- Selector for independent variable

	dependent_selector: ROSE_COMBO_BOX
			-- Selector for dependent variable

	go_button: EV_BUTTON
			-- Button to initiate display of regression graph

	chart: ROSE_CHART_WIDGET [ROSE_CHART_2D_POINT, ROSE_CHART_2D_POINT]
			-- Widget displaying scatter plot and regression line

	fill_selectors is
			-- Populate `independent_selector' and `dependent_selector'.
		do
			plots.do_all (agent insert_plot)
		ensure
			filled: independent_selector.count = plots.count
			-- not quite true as plot.name might be void:
			same_order: True -- forall in plots, plots [1].name = (in)dependent_selector.item (i).text
		end

	insert_plot (a_plot: like plot_type) is
			--	Add name of `a_plot' to `independent_selector' and `dependent_selector'.
		require
			a_plot_not_void: a_plot /= Void
		local
			l_name: STRING
			l_label: EV_LIST_ITEM
		do
			l_name := a_plot.name
			if l_name = Void then
				l_name := Unknown_plot
			end
			create l_label.make_with_text (l_name)
			independent_selector.extend (l_label)
			create l_label.make_with_text (l_name)
			dependent_selector.extend (l_label)
		ensure
			one_more_independent_plot: independent_selector.count = old independent_selector.count + 1
			one_more_dependent_plot: dependent_selector.count = old dependent_selector.count + 1
		end

	notify_selection_made is
			-- Enable or disable `go_button' according to whether selections differ
		require
			initialized: initialized
		do
			if independent_selector.current_index = dependent_selector.current_index then
				go_button.disable_sensitive
			else
				go_button.enable_sensitive
			end
		ensure
			correct_state_for_go_button: go_button.is_sensitive =
				(independent_selector.current_index /= dependent_selector.current_index)
		end

	show_graph is
			-- Create and display regression line.
		require
			initialized: initialized
			differing_datasets: independent_selector.current_index /= dependent_selector.current_index
		do
			if widget.count = 2 then
				widget.go_i_th (2)
				widget.remove
			end
			create_chart
		ensure
			chart_not_void: chart /= Void
		end

	create_chart is
			-- Create and populate `chart'.
		require
			widget_has_one_child: widget.count = 1
			differing_datasets: independent_selector.current_index /= dependent_selector.current_index
		local
			l_x_dataset, l_y_dataset: ROSE_CHART_PLOT [ROSE_CHART_2D_POINT]
			l_x_labels, l_y_labels: ROSE_LINEAR_ARRAY [ANY]
			l_x_values, l_y_values: ROSE_LINEAR_ARRAY [DOUBLE]
			l_coords, l_coords2: ROSE_CHART_COORDINATE_SYSTEM [ROSE_CHART_2D_POINT]
			l_palette: ROSE_PALETTE
		do
			create chart.make (Void)
			widget.extend (chart)
			create l_palette.make_default
			l_palette.set_color (1, Dark_red)
			l_palette.set_color (2, Dark_gray)
			l_palette.set_color (3, Black)
			l_palette.set_color (4, Dark_gray)
			chart.display.legend.set_palette (l_palette)
			l_y_dataset := plots [dependent_selector.current_index].plot
			l_x_dataset := plots [independent_selector.current_index].plot
			l_x_values := l_x_dataset.values (2)
			l_x_labels := l_x_dataset.labels (2)
			l_y_values := l_y_dataset.values (2)
			l_y_labels := l_y_dataset.labels (2)
			l_coords := chart.main_system
			l_coords2 := chart.secondary_system
			run_regression (l_coords, l_coords2, l_x_values, l_y_values, l_x_labels, l_y_labels,
				l_y_dataset.name + " v. " + l_x_dataset.name)
			setup_regression_plot (
				l_coords, l_coords2, l_x_values)
			l_coords.x_axis.sync_to_axis (l_coords2.x_axis)
			l_coords.y_axis.sync_to_axis (l_coords2.y_axis)
		ensure
			chart_not_void: chart /= Void
		end

	setup_regression_plot (a_coords, a_coords2: ROSE_CHART_COORDINATE_SYSTEM [ROSE_CHART_2D_POINT]; a_x_values: ROSE_LINEAR_ARRAY [DOUBLE]) is
			-- Prepare plot for regression line on `a_coords2'.
		require
			a_coords_not_void: a_coords /= Void
			a_coords2_not_void: a_coords2 /= Void
			regression_equation_not_void: regression_equation /= Void
			estimated_dependent_values_not_void: estimated_dependent_values /= Void
			a_x_values_not_void: a_x_values /= Void
			same_x_values_count: a_x_values.count = estimated_dependent_values.count
		local
			l_plot_list2: ROSE_CHART_2D_PLOT_LIST
			l_lcl, l_ucl: ROSE_LINEAR_ARRAY [DOUBLE]
			l_estimates: VECTOR
			l_error: DOUBLE
		do
			l_error := regression.sigma_square * 2.0
			create l_estimates.make_from_array (estimated_dependent_values)
			l_lcl := (l_estimates.minus (l_error)).to_array
			l_ucl := (l_estimates.minus (-l_error)).to_array
			create l_plot_list2.make_empty
			l_plot_list2.add_plot_from_arrays (Lcl_name, a_x_values, l_lcl, Void, Void, Void)
			l_plot_list2.add_plot_from_arrays (regression_equation, a_x_values, estimated_dependent_values, Void, Void, Void)
			l_plot_list2.add_plot_from_arrays (Ucl_name, a_x_values, l_ucl, Void, Void, Void)
			a_coords2.set_plot_list (l_plot_list2)
			set_colors (a_coords, a_coords2)
			a_coords.x_axis.set_ticked_labels (a_x_values, formatted_labels (a_x_values))
		end
	
feature {NONE} -- Implementation

	Unknown_plot: STRING is "<unknown>"
			-- Name used when plot name is `Void'

	plot_type: ROSE_CHART_RENDER_PLOT [ROSE_CHART_2D_POINT] is
			-- Type of plots
		do  end

	plots: ROSE_LINEAR_ARRAY [like plot_type]
			-- All plots from which the user will choose 2

	set_colors (a_coords, a_coords2: ROSE_CHART_COORDINATE_SYSTEM [ROSE_CHART_2D_POINT]) is
			-- Set colors for each plot.
			-- Regression line will be black.
			-- Confidence lines will be light-grey.
			-- Scatter points will be `scatter_color'.
		require
			a_coords_not_void: a_coords /= Void
			a_coords2_not_void: a_coords2 /= Void
		do
			a_coords.plots.start
			a_coords.plots.item.set_color (Dark_red)
			a_coords2.plots.do_all (agent set_line_color)
		end

invariant

	at_most_two_widget_children: widget.count <= 2
	plots_not_void: plots /= Void
	at_least_two_plots: plots.count >= 2
	widget_has_chart_at_bottom: chart /= Void implies is_bottom (chart, widget)
	selector_bar_not_void: selector_bar /= Void
	selector_bar_at_top: is_first (selector_bar, widget)
	independent_selector_not_void: independent_selector /= Void
	dependent_selector_not_void: dependent_selector /= Void
	dependent_selector_on_left: is_leftmost (dependent_selector, selector_bar)
	independent_selector_on_right: is_right_of (independent_selector, dependent_selector, selector_bar)
	same_count: independent_selector.count = dependent_selector.count
	go_button_not_void: go_button /= Void
	go_button_far_right: is_rightmost (go_button, selector_bar)
	go_button_action: go_button.select_actions.first.is_equal (agent show_graph)
	filled: initialized implies independent_selector.count = plots.count
			-- not quite true as plot.name might be void (but the essence of the matter is True - i.e. we can rely on it):
	same_order: True -- initialized implies forall in plots, plots [1].name = (in)dependent_selector.item (i).text
	independent_selector_action: initialized implies independent_selector.select_actions.first.is_equal (agent notify_selection_made)
	dependent_selector_action: initialized implies dependent_selector.select_actions.first.is_equal (agent notify_selection_made)

end

			
