indexing

	description: "[
        Widgets that display a regression line,
        moments, and confidence intervals
        for a single dataset.
		]"

	author: "Colin Adams"
	last_editor: "$Author: c4 $"
	date: "$Date: 2009/03/30 $"
	revision: "$Revision: #2 $"
	path: "$File: //rose/source/kernel/infrastructure/widgets/charts/rose_plot_regression.e $"

class	ROSE_PLOT_REGRESSION

inherit

	WITH_WIDGET
		redefine
			widget
		end

	ROSE_PLOT_REGRESSION_ROUTINES

create

	make

feature {NONE} -- Initialization

	make (a_plot: ROSE_CHART_RENDER_PLOT [ROSE_CHART_2D_POINT]) is
			-- Initialize `Current'.
		require
			a_plot_not_void: a_plot /= Void
		do
			create widget
			scatter_color := a_plot.color
			populate_widgets (a_plot.plot)
		ensure
			
		end

feature -- Access

	window_title: STRING is "Regression line"
			-- Title for window if opened separately using `open_in_window'
	
	window_icon_name: STRING is "graph3.ico"
			-- Name of icon used for window if opened separately `open_in_window'

	widget: ROSE_PROPORTIONAL_HORIZONTAL_BOX
			-- Top-level widget

	scatter_color: EV_COLOR
			-- Color of original data series (from which we drilled down);
			-- Used for scatter plot and box-plot.

feature {INVARIANT_ASSUMING} -- Implementation

	line_chart: ROSE_CHART_WIDGET [ROSE_CHART_2D_POINT, ROSE_CHART_2D_POINT]
			-- Widget to display regression line

	box_chart: ROSE_CHART_WIDGET [ROSE_CHART_BOXPLOT_POINT, ROSE_CHART_BOXPLOT_POINT]
			-- Widget to display moments

	populate_widgets (a_plot: ROSE_CHART_PLOT [ROSE_CHART_2D_POINT]) is
			-- Add plots to `widget'.
		require
			a_plot_not_void: a_plot /= Void
		local
			l_palette: ROSE_PALETTE
		do
			create_box_chart (a_plot.name, a_plot.values (2))
			create_line_chart (a_plot)
			create l_palette.make_default
			l_palette.set_color (1, scatter_color)
			l_palette.set_color (2, Dark_gray)
			l_palette.set_color (3, Black)
			l_palette.set_color (4, Dark_gray)
			box_chart.display.legend.set_palette (l_palette)
			line_chart.display.legend.set_palette (l_palette)
			widget.extend_weighted (box_chart, 1)
			widget.extend_weighted (line_chart, 3)
		ensure
			line_chart_not_void: line_chart /= Void
			box_chart_not_void: box_chart /= Void
			widget_has_line_chart: widget.has (line_chart)
			widget_has_box_chart: widget.has (box_chart)
		end

	create_line_chart (a_plot: ROSE_CHART_PLOT [ROSE_CHART_2D_POINT]) is
			-- Create and populate `line_chart'.
		require
			a_plot_not_void: a_plot /= Void
		local
			l_coords, l_coords2: ROSE_CHART_COORDINATE_SYSTEM [ROSE_CHART_2D_POINT]
			l_x_labels, l_y_labels: ROSE_LINEAR_ARRAY [ANY]
			l_x_values, l_y_values: ROSE_LINEAR_ARRAY [DOUBLE]
			l_dated: BOOLEAN
		do
			create line_chart.make (Void)
			l_x_labels := a_plot.labels (1)
			l_y_labels := a_plot.labels (2)
			l_x_values := a_plot.values (1)
			l_y_values := a_plot.values (2)
			l_coords := line_chart.main_system
			l_coords2 := line_chart.secondary_system
			run_regression (l_coords, l_coords2, l_x_values, l_y_values, l_x_labels, l_y_labels, a_plot.name)
			l_dated := l_x_labels.for_all (agent is_date)
			setup_regression_plot (l_dated, l_x_labels, l_coords, l_coords2, l_x_values)
			l_coords.x_axis.sync_to_axis (l_coords2.x_axis)
			l_coords.y_axis.sync_to_axis (l_coords2.y_axis)
		ensure
			line_chart_not_void: line_chart /= Void
		end

	setup_regression_plot (a_dated: BOOLEAN; a_x_labels: ROSE_LINEAR_ARRAY [ANY];
		a_coords, a_coords2: ROSE_CHART_COORDINATE_SYSTEM [ROSE_CHART_2D_POINT]; a_x_values: ROSE_LINEAR_ARRAY [DOUBLE]) is
			-- Prepare plot for regression line on `a_coords2'.
		require
			a_coords_not_void: a_coords /= Void
			a_coords2_not_void: a_coords2 /= Void
			regression_equation_not_void: regression_equation /= Void
			estimated_dependent_values_not_void: estimated_dependent_values /= Void
			a_x_values_not_void: a_x_values /= Void
			same_x_values_count: a_x_values.count = estimated_dependent_values.count
		local
			l_labels: ROSE_LINEAR_ARRAY [ANY]
			l_dates: ROSE_LINEAR_ARRAY [ROSE_DATE]
			l_plot_list2: ROSE_CHART_2D_PLOT_LIST
			l_lcl, l_ucl: ROSE_LINEAR_ARRAY [DOUBLE]
			l_estimates: VECTOR
			l_error: DOUBLE
		do
			l_error := regression.sigma_square * 2.0
			create l_estimates.make_from_array (estimated_dependent_values)
			l_lcl := (l_estimates.minus (l_error)).to_array
			l_ucl := (l_estimates.minus (-l_error)).to_array
			if a_dated then
				create l_dates.make_empty
				a_x_labels.do_all (agent add_as_date (l_dates, ?))
				a_coords.set_x_axis (create {ROSE_CHART_AXIS_INDEXED}.make)
				a_coords.x_axis.set_date_format ("yyyy-[0]mm-[0]dd")
				a_coords2.set_x_axis (create {ROSE_CHART_AXIS_INDEXED}.make)
				a_coords2.x_axis.set_date_format ("yyyy-[0]mm-[0]dd")
				create l_plot_list2.make_empty
				l_plot_list2.add_plots_from_dates (<<Lcl_name, regression_equation, Ucl_name>>, l_dates, <<l_lcl, estimated_dependent_values, l_ucl>>, Void)
				l_labels := l_dates
			else
				create l_plot_list2.make_empty
				l_plot_list2.add_plot_from_arrays (Lcl_name, a_x_values, l_lcl, a_x_labels, Void, Void)
				l_plot_list2.add_plot_from_arrays (regression_equation, a_x_values, estimated_dependent_values, a_x_labels, Void, Void)
				l_plot_list2.add_plot_from_arrays (Ucl_name, a_x_values, l_ucl, a_x_labels, Void, Void)
				if are_good_labels (a_x_labels) then
					l_labels := a_x_labels
				end
			end
			a_coords2.set_plot_list (l_plot_list2)
			set_colors (a_coords, a_coords2)	
			a_coords.x_axis.set_ticked_labels (a_x_values, l_labels)
			a_coords2.x_axis.set_ticked_labels (a_x_values, l_labels)
		end

	set_colors (a_coords, a_coords2: ROSE_CHART_COORDINATE_SYSTEM [ROSE_CHART_2D_POINT]) is
			-- Set colors for each plot.
			-- Regression line will be black.
			-- Confidence lines will be light-grey.
			-- Scatter points will be `scatter_color'.
		require
			a_coords_not_void: a_coords /= Void
			a_coords2_not_void: a_coords2 /= Void
		do
			a_coords.plots.start
			a_coords.plots.item.set_color (scatter_color)
			a_coords2.plots.do_all (agent set_line_color)
		end
	
	create_box_chart (a_name: STRING; a_y_values: ROSE_LINEAR_ARRAY [DOUBLE]) is
			-- Create and populate `box_chart'.
		require
			a_name_not_void: a_name /= Void
			a_y_values_not_void: a_y_values /= Void
		local
			l_coords: ROSE_CHART_COORDINATE_SYSTEM [ROSE_CHART_BOXPLOT_POINT]
			l_plot_list: ROSE_CHART_BOXPLOT_PLOT_LIST
		do
			create box_chart.make (Void)
			l_coords := box_chart.main_system
			l_coords.set_renderer (create {ROSE_CHART_RENDERER_BOXPLOT})
			create l_plot_list.make_empty
			l_plot_list.add_boxplots_from_data (a_name, Void, <<a_y_values>>, Void, Void)
			l_coords.set_plot_list (l_plot_list)
			l_coords.x_axis.show (False)
			l_coords.plots.start
			l_coords.plots.item.set_color (scatter_color)
		ensure
			box_chart_not_void: box_chart /= Void
		end

feature {NONE} -- Agents

	is_date (a_any: ANY): BOOLEAN is
			-- Does `a_any' conform to ROSE_DATE?
		do
			if {l_date: ROSE_DATE} a_any then
				Result := True
			end
		ensure
			definition: Result = {l_d: ROSE_DATE} a_any
		end

	add_as_date (a_array: ROSE_LINEAR_ARRAY [ROSE_DATE]; a_any: ANY) is
			-- Add `a_any' to end of `a_array'.
		require
			a_array_not_void: a_array /= Void
			a_any_is_date: is_date (a_any)
		do
			if {l_date: ROSE_DATE} a_any then
				a_array.extend (l_date)
			else
				check
					impossible: False
					-- owing to precondition
				end
			end
		ensure
			one_more: a_array.count = old a_array.count + 1
			correct_date_added: {l_d: ROSE_DATE} a_any implies a_array.last.same_date (l_d)
		end

end
	
