indexing

	description: "Routines to help display a regression line"

	author: "Colin Adams"
	last_editor: "$Author: c4 $"
	date: "$Date: 2009/03/30 $"
	revision: "$Revision: #1 $"
	path: "$File: //rose/source/kernel/infrastructure/widgets/charts/rose_plot_regression_routines.e $"

class	ROSE_PLOT_REGRESSION_ROUTINES

inherit

	ST_FORMATTING_ROUTINES

	ROSE_STOCK_COLORS
		export {NONE} all end

feature -- Access

	Lcl_name: STRING is "Two standard errors lower"
			-- Name of lower confidence line

	Ucl_name: STRING is "Two standard errors higher"
			-- Name of upper confidence line
	
	regression: REGRESSION
			-- Object used to calculate `regression_equation'

	regression_equation: STRING
			-- Equation for fitted line

	estimated_dependent_values: ROSE_LINEAR_ARRAY [DOUBLE]
			-- Estimates for dependent values at indices to corresponding independt values

	are_good_labels (a_labels: ROSE_LINEAR_ARRAY [ANY]): BOOLEAN is
			-- Is at least one of `a_labels' non-empty?
		do
			if a_labels /= Void then
				Result := a_labels.there_exists (agent non_empty_label)
			end
		end

	formatted_labels (a_values: ROSE_LINEAR_ARRAY [DOUBLE]): ROSE_LINEAR_ARRAY [STRING] is
			-- Formatted versions of `a_values'
		require
			a_values_not_void: a_values /= Void
		do
			create Result.make_empty
			a_values.do_all (agent add_label (Result, ?))
		ensure
			formatted_labels_not_void: Result /= Void
			no_void_label: not Result.has (Void)
		end

feature -- Basic operations

	run_regression (a_coords, a_coords2: ROSE_CHART_COORDINATE_SYSTEM [ROSE_CHART_2D_POINT];
		a_x_values, a_y_values: ROSE_LINEAR_ARRAY [DOUBLE]; a_x_labels, a_y_labels: ROSE_LINEAR_ARRAY [ANY]
		a_name: STRING) is
			-- Run regression and prepare renderers.
		require
			a_coords_not_void: a_coords /= Void
			a_coords2_not_void: a_coords2 /= Void
			a_x_values_not_void: a_x_values /= Void
			a_y_values_not_void: a_y_values /= Void
			a_x_labels_not_void: a_x_labels /= Void
			a_y_labels_not_void: a_y_labels /= Void
			a_name_not_void: a_name /= Void
		local
			l_dependent, l_estimates: VECTOR
			l_matrix: INTERCEPT_MATRIX
			l_intercept, l_slope: DOUBLE
			l_plot_list: ROSE_CHART_2D_PLOT_LIST
		do
			a_coords.set_renderer (create {ROSE_CHART_RENDERER_LINE}.make_with_line_width (False, True, 2, 3))
			a_coords2.set_renderer (create {ROSE_CHART_RENDERER_LINE}.make_with_line_width (True, False, 2, 3))
			create l_plot_list.make_empty
			create l_dependent.make_from_array (a_y_values)
			create l_matrix.make (create {MATRIX}.make_from_columns (<<create {VECTOR}.make_from_array (a_x_values)>>))
			create regression.make (l_matrix, l_dependent)
			l_intercept := regression.coefficients [1]
			l_slope := regression.coefficients [2]
			regression_equation := "y = " + format ("$.2G", <<create {DS_CELL [DOUBLE]}.make (l_intercept)>>) +
				" " + format ("$+.2G", <<create {DS_CELL [DOUBLE]}.make (l_slope)>>) + "x, R-square = " +
				format ("$.2G", <<create {DS_CELL [DOUBLE]}.make (regression.r_square_with_intercept)>>)
			create l_estimates.make_from_array (a_x_values)
			l_estimates.scale (l_slope)
			l_estimates.add_scalar (l_intercept)
			estimated_dependent_values := l_estimates.to_array
			l_plot_list.add_plot_from_arrays (a_name, a_x_values, a_y_values, a_x_labels, a_y_labels, Void)
			a_coords.set_plot_list (l_plot_list)
		ensure
			regression_not_void: regression /= Void
			regression_equation_not_void: regression_equation /= Void
			estimated_dependent_values_not_void: estimated_dependent_values /= Void
		end

feature {NONE} -- Agents

	non_empty_label (a_label: ANY): BOOLEAN is
			-- Is `a_label' non-Void and non-empty-string?
		do
			if a_label /= Void then
				if {l_string: STRING_GENERAL} a_label then
					Result := not l_string.is_empty
				else
					Result := True
				end
			end
		end
	
	set_line_color (a_line: ROSE_CHART_RENDER_PLOT [ROSE_CHART_2D_POINT]) is
			-- Set color on `a_line'.
		require
			a_line_not_void: a_line /= Void
		do
			if a_line.name.same_string (Ucl_name) or  a_line.name.same_string (Lcl_name) then
				a_line.set_color (Dark_gray)
			else
				a_line.set_color (Black)
			end
		end

	add_label (a_array: ROSE_LINEAR_ARRAY [STRING]; a_value: DOUBLE) is
			-- Add label for `a_value' at end of `a_array'.
		require
			a_array_not_void: a_array /= Void
		do
			a_array.extend (format ("$.2G", <<create {DS_CELL [DOUBLE]}.make (a_value)>>))
		ensure
			one_more: a_array.count = old a_array.count + 1
			added_item_not_void: a_array.last /= Void
		end

end
