note

	description:

		"Addition functional on ROSE_EXPRESSION"

	author: "Mark Howard"
	last_editor: "$Author: er $"
	date: "$Date: 2013/06/18 $"
	revision: "$Revision: #25 $"
	path: "$File: //rose/source/kernel/math/rose_plus_operator.e $"
	storable_version: ""

class
	ROSE_PLUS_OPERATOR [reference G]

inherit
	ROSE_NUMERIC_BINARY_OPERATOR [G]
		redefine
			commutative,
			partial_real_value,
			partial_double_value,
			partial_integer_value,
			partial_value_in_market_currency
		end

create
	make

feature -- Access

	nominal_units (a_left, a_right: ROSE_UNITS): ROSE_UNITS is
			-- Nominal units
		do
			if a_left /= Void and then a_left.is_equal (a_right) then
				Result := a_left
			else
				create {MISMATCHED_UNITS} Result.make (a_left,a_right)
			end
		end

	computed_units (a_left, a_right: ROSE_UNITS): ROSE_UNITS is
			--units from left and right
		do
			Result := a_left + a_right
		end

	units_in_currency (a_left, a_right: ROSE_EXPRESSION [G]; a_currency: PERSISTENT_CURRENCY; a_entity: G): ROSE_UNITS is
		do
			Result := a_left.units_in_currency (a_currency,a_entity) + a_right.units_in_currency (a_currency,a_entity)
		end

	commutative: BOOLEAN is True

	partial_real_value (a_left, a_right: ROSE_EXPRESSION [G]; a_g: G): REAL is
		local
			l_left, l_right: REAL
		do
			l_left :=  a_left.partial_real_value (a_g)
			l_right := a_right.partial_real_value (a_g)
			if is_defined_real (l_right) then
				if is_defined_real (l_left) then
					Result := l_left + l_right
				else
					Result := l_right
				end
			else
				Result := l_left
			end
		end

	partial_double_value (a_left, a_right: ROSE_EXPRESSION [G]; a_g: G): DOUBLE is
		local
			l_left, l_right: DOUBLE
		do
			l_left :=  a_left.partial_double_value (a_g)
			l_right := a_right.partial_double_value (a_g)
			if is_defined_double (l_right) then
				if is_defined_double (l_left) then
					Result := l_left + l_right
				else
					Result := l_right
				end
			else
				Result := l_left
			end
		end

	partial_integer_value (a_left, a_right: ROSE_EXPRESSION [G]; a_g: G): INTEGER is
		local
			l_left, l_right: INTEGER
		do
			l_left :=  a_left.partial_integer_value (a_g)
			l_right := a_right.partial_integer_value (a_g)
			if is_defined_integer (l_right) then
				if is_defined_integer (l_left) then
					Result := l_left + l_right
				else
					Result := l_right
				end
			else
				Result := l_left
			end
		end

	partial_value_in_market_currency (a_left, a_right: ROSE_EXPRESSION [G]; a_market : MARKET;a_currency: PERSISTENT_CURRENCY; a_g: G; a_date: ROSE_DATE): DOUBLE is
		local
			l_left, l_right: DOUBLE
		do
			l_left :=  a_left.partial_value_in_market_currency (a_market,a_currency,a_g,a_date)
			l_right := a_right.partial_value_in_market_currency (a_market,a_currency,a_g,a_date)
			if is_defined_double (l_right) then
				if is_defined_double (l_left) then
					Result := l_left + l_right
				else
					Result := l_right
				end
			elseif is_defined_double (l_left) then
				Result := l_left
			end
		end

	unitized_value(a_left,a_right : ROSE_UNITIZED_VALUE[DOUBLE]) : ROSE_UNITIZED_VALUE[DOUBLE]
		do
			create Result.make_with_units(a_left.value + a_right.value,a_left.units + a_right.units)
		end

feature {NONE} --implementation
	double_computation(a_left, a_right: ROSE_EXPRESSION [G]; a_left_value, a_right_value : DOUBLE) : DOUBLE is
			-- <Precursor>
		do
			Result := undefined_double
			if is_defined_double (a_left_value) and is_defined_double (a_right_value) then
				Result := a_left_value + a_right_value
			end
		end
	decimal_computation(a_left, a_right: ROSE_EXPRESSION [G]; a_left_value, a_right_value : RP_DECIMAL) : RP_DECIMAL is
			-- Decimal computation
		do
			if a_left_value.is_defined and a_right_value.is_defined then
				Result := a_left_value + a_right_value
			else
				create Result.make_undefined
			end
		end
	real_computation(a_left, a_right: ROSE_EXPRESSION [G]; a_left_value, a_right_value : REAL) : REAL is
		do
			Result := undefined_real
			if is_defined_real (a_left_value) and is_defined_real (a_right_value) then
				Result := a_left_value + a_right_value
			end
		end
	integer_computation(a_left, a_right: ROSE_EXPRESSION [G]; a_left_value, a_right_value : INTEGER) : INTEGER is
		do
			Result := undefined_integer
			if is_defined_integer (a_left_value) and is_defined_integer (a_right_value) then
				Result := a_left_value + a_right_value
			end
		end
	string_computation(a_left, a_right: ROSE_EXPRESSION [G]; a_left_value, a_right_value : STRING) : STRING is
		do
			if a_left_value /= Void and a_right_value /= Void then
				Result := a_left_value + a_right_value
			end
		end
	boolean_computation(a_left, a_right: ROSE_EXPRESSION [G]; a_left_value, a_right_value : BOOLEAN) : BOOLEAN is
		do
			Result := a_left_value or a_right_value
		end
	date_computation(a_left, a_right: ROSE_EXPRESSION [G]; a_left_value, a_right_value : ROSE_DATE) : ROSE_DATE is
		do
		end
end
