indexing
	description: "Framework for probability distributions"
	keywords: "probability distribution"
	author: "YC"
	date: "$Date: 2008/12/01 $"
	revision: "$Revision: #9 $"
	archive: "$File: //rose/source/kernel/infrastructure/math/statistics/rose_pr_distribution.e $"

deferred class
	ROSE_PR_DISTRIBUTION

inherit
	ROSE_MATH_FUNCTIONS
	ROSE_SHARED_GLOBAL

feature -- Access

	name : STRING is
			-- Name of the distribution
		once
			Result := "Anonymous"
		end

feature -- Parameter Estimation

feature -- Basic operations

	entropy: DOUBLE is
			-- entropy
		deferred
		ensure
			positive_entropy: Result >= 0
		end

	estimated_tolerance: INTEGER is
			-- Estimate tolerance for comparing probabilistic measures
		do
			Result := tolerance_magnitude
		end

	is_distribution (a_x: ROSE_LINEAR_ARRAY[DOUBLE]): BOOLEAN is
			-- is it a distribution
		require
			a_x_exists: a_x /= Void
		local
			l_val, l_sum : DOUBLE
		do
			l_val := 0
			l_sum := 0
			Result := True
			from a_x.start until a_x.exhausted or Result = False loop
				l_val := a_x.current_item
				if is_probability(l_val) = False then
					Result := False
				end
				l_sum := l_sum + l_val
				a_x.forth
			end
			if Result = True then
				if l_sum /= 1 then
					Result := False
				end
			end
		end

	is_probability (a_x: DOUBLE): BOOLEAN is
			-- is a probability
			do
				Result := true
				if isnan(a_x) = True then
					Result := False
				else
					if a_x < 0 or a_x > 1 then
						Result := False
					end
				end
			end

	lorenz_curve (F: DOUBLE): DOUBLE is
			-- lorenz curve
		deferred
		end

	memoryless: BOOLEAN is
			-- memory less
		deferred
		end

feature {NONE} -- Implementation

	tolerance_magnitude: INTEGER is 7
		-- fixed tolerance for assertions

	not_yet_implemented: STRING is "Not yet implemented."
		-- string for failing when method is not implemented

end
