indexing
	description: "Bernoulli probability distribution"
	author: "Yves Chauvin"
	last_editor: "$Author: c4 $"
	date: "$Date: 2010/02/18 $"
	revision: "$Revision: #3 $"
	archive: "$Archive: $"

class
	ROSE_PR_DISTRIBUTION_BERNOULLI

inherit
	ROSE_PR_DISTRIBUTION_DISCRETE_UNIVARIATE
		redefine
			name
		end

feature -- Access

	name: STRING is
			-- Name of this distribution
		once
			Result := "Bernoulli"
		end

feature -- Distribution Parameters

	probability: DOUBLE
			-- probability of success

	set_probability (a_value: DOUBLE) is
			-- Set distribution parameter
		require
			is_probability: is_probability (a_value)
			success_probability: a_value > 0
		do
			probability := a_value
		end

feature -- Basic operations

	compute_probability_mass_function  is
			-- Computes pmf of this distribution
		do
			fail("pmf", not_yet_implemented, Void)
		end

	compute_support is
			-- Computes support of this distribution
		do
			fail("pmf", not_yet_implemented, Void)
		end

	cumulative_distribution_function (a_x: DOUBLE): DOUBLE is
			-- Probability that `random_variable' <= `a_x'
		do
			-- Note to reviewers:
			-- This is defaulting to 0.0 if 0.0 < `a_x' < 1.0 - is this correct?
			if a_x = 0.0 then
				Result := 1.0 - probability
			elseif a_x >= 1.0 then
				Result := 1.0
			end
		end

	entropy: DOUBLE is
			-- Entropy of this distribution
		do
			fail("pmf", not_yet_implemented, Void)
		end

	is_symmetric: BOOLEAN is
			-- Is this distribution symmetric?
		do
			Result := False
		end

	is_kurtosis_computable: BOOLEAN is
			-- is kurtosis computable for this distribution?
		do
			fail("pmf", not_yet_implemented, Void)
		end

	is_mode_computable: BOOLEAN is
			-- is mode computable for this distribution?
		do
			fail("pmf", not_yet_implemented, Void)
		end

	is_skewness_computable: BOOLEAN is
			-- is skewness computable for this distribution?
		do
			fail("pmf", not_yet_implemented, Void)
		end

	is_mean_computable: BOOLEAN is
			-- is mean computable for this distribution?
		do
			fail("pmf", not_yet_implemented, Void)
		end

	is_variance_computable: BOOLEAN is
			-- is variance computable for this distribution?
		do
			fail("pmf", not_yet_implemented, Void)
		end

	kurtosis: DOUBLE is
			-- Kurtosis of this distribution
		do
			fail("pmf", not_yet_implemented, Void)
		end

	lorenz_curve (F: DOUBLE): DOUBLE is
			-- Lorenz curve of this distribution
		do
			fail("pmf", not_yet_implemented, Void)
		end

	mean: DOUBLE is
			-- Mean of this distribution
		do
			fail("pmf", not_yet_implemented, Void)
		end

	median: DOUBLE is
			-- Median of this distribution
			-- Improve in case of possible multiple values
		do
			fail("pmf", not_yet_implemented, Void)
		end

	memoryless: BOOLEAN is
			-- Is this distribution memoryless?
		do
			Result := True
		end

	mode: DOUBLE is
			-- Mode of this distribution
		do
			fail("pmf", not_yet_implemented, Void)
		end

	probability_mass_function (x: like random_variable): DOUBLE is
			-- pmf of this distribution
		do
			if x = 0 then
				Result := 1 - probability
			elseif x = 1 then
				Result := probability
			end
		end

	skewness: DOUBLE is
			-- skewness of this distribution
		do
			fail("pmf", not_yet_implemented, Void)
		end

	variance: DOUBLE is
			-- variance of this distribution
		do
			fail("pmf", not_yet_implemented, Void)
		end

feature {NONE} -- Implementation

end
