indexing
	description: "Framework for probability distributions"
	keywords: "probability distribution continuous univariate"
	author: "Yves Chauvin"
	date: "$Date: 2009/01/09 $"
	revision: "$Revision: #8 $"
	archive: "$File: //rose/source/kernel/infrastructure/math/statistics/rose_pr_distribution_continuous_univariate.e $"

deferred class
	ROSE_PR_DISTRIBUTION_CONTINUOUS_UNIVARIATE

inherit
	ROSE_PR_DISTRIBUTION
	ROSE_PR_DISTRIBUTION_CONTINUOUS
	ROSE_PR_DISTRIBUTION_UNIVARIATE
		redefine
			random_variable
		end

feature -- Basic operations

	hazard_function (x: like random_variable): DOUBLE is
			-- definition of hazard function
			-- mathematically OK for all x but unclear semantics for x < 0
		require
			survival_non_zero: survival_function (x) /= 0
		do
			Result := probability_density_function (x) / survival_function (x)
		ensure
			positive_hazard: Result >= 0
		end

	probability_density_function (x: like random_variable): DOUBLE is
			-- probability density function
		deferred
		end

	less_than (a_low, a_high: like random_variable): BOOLEAN is
			-- proper interval order
		do
			Result := a_low < a_high
		end

feature {NONE} -- Implementation

	random_variable: DOUBLE_REF
		-- random variable underlying distribution

	is_negative (x: like random_variable): BOOLEAN is
			-- rv type is negative
		do
			Result := x < 0.
		end

	is_positive (x: like random_variable): BOOLEAN is
			-- rv type is positive
		do
			Result := x > 0.
		end



end -- class ROSE_PR_DISTRIBUTION_CONTINUOUS_UNIVARIATE

