indexing
	description: "Framework for probability distributions"
	keywords: "probability distribution discrete univariate"
	author: "Yves Chauvin"
	date: "$Date: 2009/01/09 $"
	revision: "$Revision: #7 $"
	archive: "$File: //rose/source/kernel/infrastructure/math/statistics/rose_pr_distribution_discrete_univariate.e $"

deferred class
	ROSE_PR_DISTRIBUTION_DISCRETE_UNIVARIATE

inherit
	ROSE_PR_DISTRIBUTION
	ROSE_PR_DISTRIBUTION_DISCRETE
	ROSE_PR_DISTRIBUTION_UNIVARIATE
		redefine
			random_variable,
			set_two_tail_probability
		end

feature -- Basic operations

	probability_mass_function (x: like random_variable): DOUBLE is
			--probability mass function
		deferred
		ensure
			is_probability: is_probability (Result)
		end

	less_than (a_low, a_high: like random_variable): BOOLEAN is
			-- proper interval order
		do
			Result := a_low < a_high
		end

feature {NONE} -- Vectors implementation

	set_two_tail_probability (a_target: VECTOR; a_value: DOUBLE; a_index: INTEGER) is
			-- Set `a_target [a_index]' to p-value corresponding to `a_target' from `a_student'.
		do
			a_target [a_index] := two_tail_probability (a_value.truncated_to_integer)
		end

feature {NONE} -- Implementation

	random_variable: INTEGER_REF
		-- random variable underlying distribution
		-- may be redefined as DOUBLE_REF in case discrete is on real support

	is_negative (x: like random_variable): BOOLEAN is
			-- rv type is negative
		do
			Result := x < 0
		end

	is_positive (x: like random_variable): BOOLEAN is
			-- rv type is positive
		do
			Result := x > 0
		end


end -- class ROSE_PR_DISTRIBUTION_DISCRETE_UNIVARIATE

