indexing
	description: "Geometric winsorized probability distribution"
	author: "Yves Chauvin"
	last_editor: "$Author: c4 $"
	date: "$Date: 2010/02/18 $"
	revision: "$Revision: #7 $"
	archive: "$Archive: $"

class
	ROSE_PR_DISTRIBUTION_GEOMETRIC_WINSORIZED

inherit
	ROSE_PR_DISTRIBUTION_DISCRETE_UNIVARIATE
		redefine
			name
		end
	ROSE_PR_GEOMETRIC_PARAMETERS

feature -- Access

	name: STRING is
			-- Name of the distribution
		once
			Result := "Geometric Winsorized"
		end

feature -- Distribution Parameters

	order: INTEGER
			-- order

	set_order (a_value: INTEGER) is
			-- Setting order as distribution parameter
		require
			positive_order: a_value > 0
		do
			order := a_value
		end

feature -- Basic operations

	probability_mass_function (x: like random_variable): DOUBLE is
			-- pmf of distribution
		do
			if x <= 0 then
				Result := 0
			elseif 0 < x and x < order then
				Result := (1 - probability)^(x.to_double - 1) * probability
			elseif x = order then
				Result := (1.0 - probability)^(x.to_double - 1)
			elseif order < x then
				Result := 0
			end
		end

	compute_probability_mass_function is
			-- computes cdf of finite geometric distribution
		do
			fail("pmf", not_yet_implemented, Void)
		end

	entropy: DOUBLE is
			-- Entropy of the distribution
		do
			fail("pmf", not_yet_implemented, Void)
		end

	is_symmetric: BOOLEAN is
			-- Geometric is not symmetric
		do
			Result := False
		end

	is_kurtosis_computable: BOOLEAN is
			-- is kurtosis computable for this distribution?
		do
			Result := True
		end

	is_mode_computable: BOOLEAN is
			-- is mode computable for this distribution?
		do
			Result := True
		end

	is_skewness_computable: BOOLEAN is
			-- is skewness computable for this distribution?
		do
			Result := True
		end

	is_mean_computable: BOOLEAN is
			-- is mean computable for this distribution?
		do
			Result := probability > 0
		end

	is_variance_computable: BOOLEAN is
			-- is variance computable for this distribution?
		do
			Result := True
		end

	kurtosis: DOUBLE is
			-- Kurtosis of distribution
		do
			fail("pmf", not_yet_implemented, Void)
		end

	lorenz_curve (F: DOUBLE): DOUBLE is
			-- Lorenz of distribution
		do
			fail("pmf", not_yet_implemented, Void)
		end

	compute_support is
			-- Computing support of distribution
		do
			fail("pmf", not_yet_implemented, Void)
		end

	mean: DOUBLE is
			-- Mean of distribution
		do
			Result := 1 / probability + ((1 - probability)^(order - 1)*(1 + ((1 - order)/probability)))
		end

	median: DOUBLE is
			-- Median of distribution
		do
			fail("pmf", not_yet_implemented, Void)
		end

	memoryless: BOOLEAN is
			-- Memory of distribution
		do
			Result := True
		end

	mode: DOUBLE is
			-- Mode of distribution
		do
			fail("pmf", not_yet_implemented, Void)
		end

	skewness: DOUBLE is
			-- Skewness of distribution
 		do
 			fail("pmf", not_yet_implemented, Void)
		end


	variance: DOUBLE is
			-- Variance of distribution
		do
			fail("pmf", not_yet_implemented, Void)
		end

	cumulative_distribution_function (a_x: DOUBLE): DOUBLE is
			-- Probability that `random_variable' <= `a_x'
		do
			if 0.0 < a_x and a_x < order then
				Result := 1.0 - (1.0 - probability) ^ a_x
			elseif a_x >= order then
				Result := 1.0
			end
		end

end
