note

	description:

		"Generic predicate"

	author: "$Author: an $Date: 06/11/09 4:18a $"
	last_editor: "$Author: an $"
	date: "$Date: 2013/10/18 $"
	revision: "$Revision: #29 $"
	path: "$File: //rose/source/kernel/infrastructure/persistence/predicates/rose_predicate.e $"

class
	ROSE_PREDICATE [G]

inherit
	TRACEABLE
		redefine
			trace
		end
	MOVIE_CAMERA_AWARE
		export
			{NONE} all
		end
	ROSE_SHARED_DATABASE_POOL
		rename
			trace as global_trace
		end

	SHARED_SQL_TABLE_NAME_PARSER
		export
			{NONE} all
		redefine
			parse_sql_statement
		end

feature -- Access

	data_server: DATA_SERVER
			-- Server used for execution

	predicate_statement: PREDICATE_STATEMENT
	container: COLLECTION [G]
	index: INTEGER
	records_fetched: INTEGER
	input_type: INTEGER
	facilitator: PREDICATE_FACILITATOR

	selection: STRING
			-- Text of SQL statement

	rows_affected: INTEGER is
			-- Number of rows affected by last execution predicate
		do
			Result := data_server.rows_affected
		end

	column_names: ROSE_LINEAR_ARRAY [STRING] is
			-- Column names
		do
			Result := facilitator.output.ordered_column_names
		end

	lower_input_index: INTEGER is
		do
			if playback_movie then
				Result := cam.next_frame ("lower_input_index").frame_data_as_integer
				cam.compare_not_needed
			else
				Result := facilitator.input.lower
				debug ("make_movies")
					if record_movie then
						cam.record_integer_movie_frame ("lower_input_index", Result)
					end
				end
			end
		end

	upper_input_index: INTEGER is
		do
			if playback_movie then
				Result := cam.next_frame ("upper_input_index").frame_data_as_integer
				cam.compare_not_needed
			else
				Result := facilitator.input.upper
				debug ("make_movies")
					if record_movie then
						cam.record_integer_movie_frame ("upper_input_index", Result)
					end
				end
			end
		end

	lower_output_index: INTEGER is
		do
			if playback_movie then
				Result := cam.next_frame ("lower_output_index").frame_data_as_integer
				cam.compare_not_needed
			else
				Result := facilitator.output.lower
				debug ("make_movies")
					if record_movie then
						cam.record_integer_movie_frame ("lower_output_index", Result)
					end
				end
			end
		end

	upper_output_index: INTEGER is
		do
			if playback_movie then
				Result := cam.next_frame ("upper_output_index").frame_data_as_integer
				cam.compare_not_needed
			else
				Result := facilitator.output.upper
				debug ("make_movies")
					if record_movie then
						cam.record_integer_movie_frame ("upper_output_index", Result)
					end
				end
			end
		end

	input_index (a_field_name: STRING): INTEGER is
		local
			l_pair: PAIR [STRING, INTEGER]
		do
			if playback_movie then
				l_pair ?= cam.next_frame ("input_index").frame_data
				cam.compare_partial_movie_frame (a_field_name, l_pair.first)
				Result := l_pair.second
			else
				Result := facilitator.input_index (a_field_name)
				debug ("make_movies")
					if record_movie then
						create l_pair.make (a_field_name, Result)
						cam.record_movie_frame ("input_index", l_pair)
					end
				end
			end
		end

	output_index (a_field_name: STRING): INTEGER is
		local
			l_pair: PAIR [STRING, INTEGER]
		do
			if playback_movie then
				l_pair ?= cam.next_frame ("output_index").frame_data
				cam.compare_partial_movie_frame (a_field_name, l_pair.first)
				Result := l_pair.second
			else
				Result := facilitator.output_index (a_field_name)
				debug ("make_movies")
					if record_movie then
						create l_pair.make (a_field_name, Result)
						cam.record_movie_frame ("output_index", l_pair)
					end
				end
			end
		end

	query_text : STRING is
   			-- Text of query
  		local
   			retried: BOOLEAN
   			l_facilitator : SQL_FACILITATOR
  		do
   			if not retried then
    			create Result.make_empty
     			l_facilitator ?= facilitator
     			if l_facilitator /= Void and then l_facilitator.predicate_statement /= Void then
      				Result.append_string (l_facilitator.predicate_statement.text)
     			end
	  		end
  		rescue
   			retried := True
  			retry
  		end

feature -- Basic operations

	parse_sql_statement (a_statement: STRING; a_db_definition: DATABASE_DEFINITION)
			-- <Precursor>
		do
			if not selection_already_parsed then
				selection_already_parsed := True
				Precursor (a_statement, a_db_definition)
			end
		end

	action (a_item: G) is
		do
		end

	trace is
			-- print trace message
		do
			Precursor
			print (predicate_statement)
			print ("%N")
		end

feature {NONE} -- Implementation

	selection_already_parsed: BOOLEAN
			-- Has `selection' already been parsed?
		note
			option: transient
		attribute
		end

end
