indexing
	description: "Price data access"
	author: "Michael Zhang"
	date: "$Date: 2008/12/01 $"
	revision: "$Revision: #6 $"

class
	ROSE_PRICE_DATABASE

inherit
	SIGNATURE_TAGGED_ITEM_DATABASE[ROSE_PRICE_SIGNATURE]
	redefine
		retrieve_current_signatures
	end
create
	make_with_override

feature  --Creation
	make_with_override(a_server : SQL_DATA_SERVER; a_process_manager : SUPPLIER_PROCESS_ID_MANAGER) is
			--make using override
		require
			server_not_void : a_server /= Void
			process_manager_not_void : a_process_manager /= Void
		do
			make_with_process_manager(a_server, a_process_manager)
			create override_database.make_named_with_process_manager("override", a_server, a_process_manager)
			server := a_server
		end

feature  --Access
	submit_override is
			-- submit override
		do
			override_database.submit
		end

	retrieve_current_signatures(a_signatures : ROSE_LINEAR_ARRAY[like ur_signature]; a_statement : STRING) : ROSE_LINEAR_ARRAY[ROSE_PRICE_SIGNATURE] is
			-- retrieve current signatures
		local
			l_override_signature, l_signature : like ur_signature
			l_override_table : HASH_TABLE[like ur_signature, MULTI_KEY]
			l_signature_table : HASH_TABLE[like ur_signature, MULTI_KEY]
			l_override_signatures : ROSE_LINEAR_ARRAY[like ur_override_signature]
			l_count : INTEGER
		do
			Result := Precursor(a_signatures, a_statement)
			if Result /= Void then
				l_override_signatures := retrieve_current_override_signatures(a_signatures, a_statement)
				if l_override_signatures /= Void and then l_override_signatures.count > 0 then
					create l_override_table.make(l_override_signatures.count * 2)
					from l_override_signatures.start until l_override_signatures.off loop
						l_override_signature := l_override_signatures.current_item
						l_override_table.put(l_override_signature, l_override_signature.multi_key)
						l_override_signatures.forth
					end
					from Result.start until Result.off loop
						l_signature := Result.current_item
						l_override_signature := l_override_table.item(l_signature.multi_key)
						if l_override_signature /= Void then
							Result.put(l_override_signature, Result.index)
							l_count := l_count + 1
						end
						Result.forth
					end
					if l_count < l_override_signatures.count then
						create l_signature_table.make(a_signatures.count * 2)
						from a_signatures.start until a_signatures.off loop
							l_signature := a_signatures.current_item
							l_signature_table.put(l_signature, l_signature.multi_key)
							a_signatures.forth
						end
						from l_override_signatures.start until l_override_signatures.off loop
							l_signature := l_override_signatures.current_item
							if not l_signature_table.has(l_signature.multi_key) then
								Result.extend(l_signature)
							end
							l_override_signatures.forth
						end
						Result.sort_by(Agent signature_order)
					end
				end
			end
		end

	replace_override_signature_data(a_signature : like ur_override_signature) is
			-- replace override signature data
		do
			override_database.replace_signature_data(a_signature)
		end

	retrieve_current_override_signatures(a_signatures : ROSE_LINEAR_ARRAY[like ur_signature]; a_statement : STRING) : ROSE_LINEAR_ARRAY[like ur_override_signature] is
			-- retrieve current override signatures
		do
			Result := override_database.retrieve_current_signatures(signatures_to_override_signatures(a_signatures), a_statement)
		end

	retrieve_historical_override_signatures(a_signatures : ROSE_LINEAR_ARRAY[like ur_signature]; a_statement : STRING) : ROSE_LINEAR_ARRAY[like ur_override_signature] is
			-- retrieve historical override signatures
		local
			l_signatures : like ur_override_signatures
		do
			l_signatures := retrieve_current_override_signatures(a_signatures, a_statement)
			Result := override_database.retrieve_signature_data_histories(l_signatures)
		end

	retrieve_override_signature_data_history(a_signature : like ur_override_signature) : like ur_override_signatures is
			-- retrieve override signature history
		do
			Result := override_database.retrieve_signature_data_history(a_signature)
		end

	entity_historical_override_signatures(a_entity_id : INTEGER) : like ur_override_signatures is
			-- entity historical override signatures
		local
			l_predicate : like historical_override_predicate
		do
			l_predicate := historical_override_predicate
			l_predicate.retrieve(Void, <<a_entity_id>>)
			Result := l_predicate.array
		end

	historical_override_predicate : COLLECTING_PREDICATE[like ur_override_signature] is
			-- historical override predicate
		do
			if historical_override_predicate_saved = Void then
				create historical_override_predicate_saved.make(server, "select a.currency_id as currency_id, a.[open] as [open], a.[close] as [close], a.[high] as [high], " +
					"a.[low] as [low], a.[bid] as [bid], a.[ask] as [ask], a.[volume] as [volume], a.[source] as [source], " +
					"a.[user_id] as [user_id], a.[update_time] as [update_time], a.[comment] as [comment], " +
					"entity_id, price_date, value_description_id, a.[process_id] as last_process_id " +
					"from override_data_signatures_data_history a " +
					"inner join override_data_signatures b on a.signature_id = b.signature_id where b.entity_id  = ?")

			end
			Result := historical_override_predicate_saved
		end

	historical_override_predicate_saved : like historical_override_predicate


	signatures_to_override_signatures(a_signatures : ROSE_LINEAR_ARRAY[like ur_signature]) : ROSE_LINEAR_ARRAY[like ur_override_signature] is
			-- signatures to override signatures
		local
			l_override_signature : like ur_override_signature
		do
			if a_signatures /= Void then
				create Result.make(1, a_signatures.count)
				from a_signatures.start until a_signatures.off loop
					create l_override_signature.make_from_signature(a_signatures.current_item)
					Result.put(l_override_signature, a_signatures.index)
					a_signatures.forth
				end
			end
		end

	signature_order(a_sig1, a_sig2 : like ur_signature) : BOOLEAN is
			-- signature order
		do
			Result := a_sig1.price_date < a_sig2.price_date
		end

feature {NONE}  --Implementation
	server : SQL_DATA_SERVER
	ur_signature : ROSE_PRICE_SIGNATURE is do  end
	ur_override_signature : ROSE_PRICE_OVERRIDE_SIGNATURE is do  end
	ur_override_signatures : ROSE_LINEAR_ARRAY[ROSE_PRICE_OVERRIDE_SIGNATURE] is do  end
	override_database : SIGNATURE_TAGGED_ITEM_DATABASE[ROSE_PRICE_OVERRIDE_SIGNATURE]
end
