indexing
	description: "Price record container"
	author: "Michael Zhang"
	date: "$Date: 2009/02/06 $"
	revision: "$Revision: #12 $"

class
	ROSE_PRICE_RECORD

inherit
	ROSE_LEGACY_UNDEFINED

create
	make_from_array, make_from_vector, default_create

feature -- creation
	make_from_array(a_array : ARRAY[DOUBLE]) is
			-- make from array
		require
			valid_array : a_array /= Void and then a_array.count = 7
		do
			set_data(a_array)
		end

	make_from_vector(a_vector : VECTOR) is
			-- make from vector
		require
			valid_vector : a_vector /= Void and then a_vector.count = 7
		do
			if a_vector /= Void and then a_vector.count = 7 then
				set_data(a_vector.to_array)
			end
		end

	to_vector : VECTOR is
			-- to vector
		do
			create Result.make_from_array(<<ask, bid, close, high, low, open, volume>>)
		end

feature -- access
	set_data(a_array : ARRAY[DOUBLE]) is
			-- set data
		do
			if a_array /= Void and then a_array.count >= 7 then
				ask := a_array.item(1)
				bid := a_array.item(2)
				close := a_array.item(3)
				high := a_array.item(4)
				low := a_array.item(5)
				open := a_array.item(6)
				volume := a_array.item(7)
			end
		end

	set_data_from_record(a_record : ROSE_PRICE_RECORD) is
			-- set data from record
		do
			if a_record /= Void then
				ask := a_record.ask
				bid := a_record.bid
				open := a_record.open
				close := a_record.close
				high := a_record.high
				low := a_record.low
				volume := a_record.volume
			end
		end

	copy_bid_ask_open(a_record : ROSE_PRICE_RECORD) is
			-- we only apply this to idsi
		do
			if not has_bid_ask and then a_record.has_bid_ask then
--				if disc(close, a_record.close) < 0.001 then
					set_bid(a_record.bid)
					set_ask(a_record.ask)
--				end
			end
			if valid_double(a_record.open) then
				-- and then ((not valid_double(high) or else (high + 0.00001) >= a_record.open) and (not valid_double(low) or else (low - 0.00001) <= a_record.open)) then
				set_open(a_record.open)
			end
		end

	disc(a, b : DOUBLE) : DOUBLE is
			-- discrepancy function
		do
			Result := (a - b).abs / ((a.abs + b.abs)/2.0).max(1.)
		end

feature -- attributes
	open : DOUBLE
	set_open(a_open : DOUBLE) is
			-- set open
		do
			open := a_open
		end

	close : DOUBLE
	set_close(a_close : DOUBLE) is
			-- set close
		do
			close := a_close
		end

	high : DOUBLE
	set_high(a_high : DOUBLE) is
			-- set high
		do
			high := a_high
		end

	low : DOUBLE
	set_low(a_low : DOUBLE) is
			-- set low
		do
			low := a_low
		end

	bid : DOUBLE
	set_bid(a_bid : DOUBLE) is
			-- set bid
		do
			bid := a_bid
		end

	ask : DOUBLE
	set_ask(a_ask : DOUBLE) is
			-- set ask
		do
			ask := a_ask
		end

	volume : DOUBLE
	set_volume(a_volume : DOUBLE) is
			-- set volume
		do
			volume := a_volume
		end

feature -- interface
	price_value_by_name(a_name : STRING) : DOUBLE is
			-- price value by name
		do
			if a_name /= Void then
				if a_name.is_equal("ask") then Result := ask
				elseif a_name.is_equal("bid") then Result := bid
				elseif a_name.is_equal("high") then Result := high
				elseif a_name.is_equal("low") then Result := low
				elseif a_name.is_equal("open") then Result := open
				elseif a_name.is_equal("close") then Result := close
				elseif a_name.is_equal("volume") then Result := volume
				end
			end
		end

	is_null : BOOLEAN is
			-- is null
		do
			Result := not (is_defined_double(open) or is_defined_double(close) or is_defined_double(high) or
					  is_defined_double(low) or is_defined_double(bid) or is_defined_double(ask) or
					  is_defined_double(volume))
		end

	is_valid_price_record : BOOLEAN is
			-- is valid price record
		local
			l_bid_ask_check : BOOLEAN
			l_high_low_check : BOOLEAN
			l_volume_check : BOOLEAN
			l_valid_bid_ask : BOOLEAN
			l_valid_high_low : BOOLEAN
		do
			l_valid_bid_ask := valid_double(bid) and valid_double(ask)
			l_bid_ask_check := (not l_valid_bid_ask) or (l_valid_bid_ask and then (bid <= ask))

			l_valid_high_low := valid_double(high) and valid_double(low)
			l_high_low_check := (not l_valid_high_low) or (l_valid_high_low and then (low <= high))

			l_volume_check := not valid_double(volume)
			if not l_bid_ask_check then
				price_check_message := "bid price has to be less than ask price"
			elseif not l_high_low_check then
				price_check_message := "low price has to be less than high price"
			end

			if  price_check_message = Void and then valid_double(close) then
				l_bid_ask_check := True
					-- (not l_valid_bid_ask) or (l_valid_bid_ask and then (bid <= close and close <= ask))
				l_high_low_check := (not l_valid_high_low) or (l_valid_high_low and then (low <= close and close <= high))
				l_volume_check := valid_double(volume) and then volume >= 0

				if not l_bid_ask_check then
					price_check_message := "close price is out of bid-ask range"
				elseif not l_high_low_check then
					price_check_message := "close price is out of high-low range"
				elseif not l_volume_check then
					price_check_message := "invalid volume number"
				end
			end

			if  price_check_message = Void and then not valid_double(close) then
				l_volume_check := not valid_double(volume)
				if not l_volume_check then
					price_check_message := "should not have volume without close price"
				end
			end

			Result := l_bid_ask_check and l_high_low_check and l_volume_check
		end

	price_check_message : STRING

	valid_double(a_double : DOUBLE) : BOOLEAN is
			-- valid double
		do
			Result := is_defined_double(a_double) and then a_double > 0
		end

	has_trade : BOOLEAN is
			-- has trade
		do
			Result := valid_double(volume)
		end

	has_close : BOOLEAN is
			-- has close
		do
			Result := valid_double(close)
		end

	has_bid_ask : BOOLEAN is
			-- has bid ask
		do
			Result := (valid_double(bid) and valid_double(ask))
		end -- and then (bid <= ask) end

	has_bid_or_ask : BOOLEAN is
			-- has bid ask
		do
			Result := (valid_double(bid) or valid_double(ask))
		end

	has_high_low : BOOLEAN is
			-- has high low
		do
			Result := (valid_double(high) and valid_double(low))
		end -- and then (low <= high) end
end
