indexing
	description: "Price report"
	author: "Michael Zhang"
	date: "$Date: 2014/02/26 $"
	revision: "$Revision: #15 $"

class
	ROSE_PRICE_REPORT

inherit
	ROSE_SIGNATURE_TAGGED_REPORT[LISTING, ROSE_PRICE_SIGNATURE]
		rename
			make as simple_signature_report_make
		redefine
			double_item,
			targets,
			targets_in_interval,
			expression_template,
			set_neighborhood
		end

	ROSE_LEGACY_UNDEFINED
	ROSE_SHARED_GLOBAL

create
	make

feature  --Creation
	make is
			-- make
		do
			simple_signature_report_make("price_data_set","price")
		end
feature --Entities
	entity_id(a_target : like ur_target) : INTEGER is
			-- entity id
		do
			if a_target /= Void then
				Result := a_target.entity_id
			end
		end

	entity_from_id(a_entity_id : INTEGER) : like entity_anchor is
			-- entity from id
		do
			Result ?= ur_target.context.handler(3).entity(a_entity_id)
		end

feature  --Access
	price_set_series(a_entity : like entity_anchor; a_dates : ARRAY[ROSE_DATE]) : like dated_matrix is
			-- price set series
		local
			l_targets : like targets
		do
			l_targets := ur_target.entity_target_series(a_entity, a_dates)
			if l_targets /= Void then
				Result := dated_matrix(l_targets)
			end
		end

	price_set_series_from_cache(a_entity : like entity_anchor; a_dates : ARRAY[ROSE_DATE]) : like dated_matrix is
			-- price set series from cache
		local
			l_targets : like targets
		do
			if is_cross_sectional_access then
				l_targets := records_at (a_entity.handle, a_dates)
			else
			l_targets := target_cache.item(a_entity.handle)
			end
			if l_targets /= Void and then l_targets.count > 0 then
				Result := dated_matrix(l_targets)
			end
		end

	targets(a_entity : like entity_anchor) : ROSE_LINEAR_ARRAY[like ur_target] is
			-- targets
		do
			if a_entity /= Void then
				Result := ur_target.entity_targets(a_entity)
			end
		end

	targets_in_interval(a_entity : like entity_anchor; a_start_date, a_end_date : ROSE_DATE) : ROSE_TIME_SERIES[like ur_target] is
			-- targets in interval
		local
			l_target : like target_at_date
			l_targets : like targets
			l_dates : ARRAY[ROSE_DATE]
		do
			if a_start_date /= Void and a_end_date /= Void then
				l_dates := a_end_date.dates_from(a_start_date)
				l_targets := ur_target.entity_target_series(a_entity, l_dates)
				create Result.make_empty
				from l_targets.start until l_targets.off loop
					l_target := l_targets.current_item
					Result.extend(l_target, l_target.date)
					l_targets.forth
				end
			else
				Result := Precursor(a_entity, a_start_date, a_end_Date)
			end
		end

	volume_ix : INTEGER is
			-- volume ix
		do
			Result := index("volume")
		end

	close_ix : INTEGER is
			-- close ix
		do
			Result := index("close")
		end

	set_neighborhood(a_entities : ARRAY[like entity_anchor]; a_dates : ARRAY[ROSE_DATE]) is
			-- set neighborhood
		local
			l_targets, l_entity_targets, l_date_targets : like targets
			l_database : ROSE_PRICE_DATABASE
		do
			l_entity_targets := entity_targets(a_entities)
			l_date_targets := date_targets(a_dates)
			if l_entity_targets /= Void then
				create l_targets.make_from_array(l_entity_targets)
			else
				create l_targets.make_empty
			end
			if  l_date_targets /= Void then
				l_targets.append_array(l_date_targets)
			end
			ur_target.set_direction_neighborhood
			l_database := ur_target.zonal_data_manager(a_entities.item(a_entities.lower).zone_from_exchange.mnemonic.out)
			l_targets := l_database.retrieve_current_signatures(l_targets, ur_target.restriction_statement)
			if l_targets /= Void and then l_targets.count > 0 then
				load_target_cache(a_entities, l_targets)
			end
		end

feature {NONE} --Templates
	expression_template : ROSE_PRICE_VARIABLE is
			--template
		do  end
	ur_override_signature : ROSE_PRICE_OVERRIDE_SIGNATURE is
			--template
		do  end

feature -- xsection access
	double_item(a_expression : like expression_template; a_target : like ur_target) : DOUBLE is
			-- double item
		local
			l_target : ROSE_PRICE_SIGNATURE
		do
			Result := constants.missing_double
			if is_cross_sectional_access then
				l_target := record_at(a_target.entity_id, a_target.date)
				if l_target /= Void then
					Result := a_expression.double_value (l_target)
				end
			else
				Result := Precursor(a_expression, a_target)
			end
		end

	records_at(a_entity_id : INTEGER; a_dates : ARRAY[ROSE_DATE]) : ROSE_LINEAR_ARRAY[like ur_target] is
			-- records at
		local
			i : INTEGER
			l_date : ROSE_DATE
			l_record : like ur_target
		do
			create Result.make_empty
			from i := a_dates.lower until i > a_dates.upper loop
				l_date := a_dates[i]
				l_record := record_at(a_entity_id, l_date)
				if l_record /= Void then
					Result.extend(l_record)
				end
				i := i + 1
			end
		end

	record_at(a_entity_id : INTEGER; a_date : ROSE_DATE) : like ur_target is
			-- record at
		local
			l_table : HASH_TABLE[ROSE_PRICE_SIGNATURE, INTEGER]
		do
			l_table := xsection_table_series.item(a_date.ccyymmdd)
			if l_table /= Void then
				Result := l_table.item(a_entity_id)
			end
		end

	load_zonal_cross_section_series(a_zone : STRING; a_dates : ARRAY[ROSE_DATE]) is
			-- load zonal cross section series
		local
			i : INTEGER
			l_date : ROSE_DATE
			l_table : HASH_TABLE[ROSE_PRICE_SIGNATURE, INTEGER]
			l_predicate : COLLECTING_PREDICATE[ROSE_PRICE_SIGNATURE]
		do
			is_cross_sectional_access := True
			create l_predicate.make (ur_target.servers.rose_zonal_price_server (a_zone), "select * from data_signatures where price_date = ?")
			if xsection_table_series = Void then
				create xsection_table_series.make(100)
			end
			from i := a_dates.lower until i > a_dates.upper loop
				l_date := a_dates[i]
				if not xsection_table_series.has_key(l_date.ccyymmdd) then
					create l_table.make (10000)
					l_predicate.retrieve (Void,  <<l_date>>)
					from l_predicate.array.start until l_predicate.array.off loop
						l_table.force(l_predicate.array.current_item, l_predicate.array.current_item.entity_id)
						l_predicate.array.forth
					end
					xsection_table_series.force(l_table, l_date.ccyymmdd)
				end
				i := i + 1
			end
		end

	xsection_table_series : HASH_TABLE[HASH_TABLE[ROSE_PRICE_SIGNATURE, INTEGER], INTEGER]

	is_cross_sectional_access : BOOLEAN

	test(a_id : INTEGER) is
			-- test
		local
			l_entity : like entity_anchor
			l_targets : like targets
			l_array : like double_series
			l_currencies : like currency_series
			l_matrix : like double_matrix
		do
			l_entity := entity_from_id(a_id)

			if l_entity /= Void then
				l_targets := targets(l_entity)
				l_matrix := double_matrix(l_targets)
				l_array := double_series(l_targets, "close")
				l_currencies := currency_series(l_targets, "close")
			end
		end

	test_override(a_id : INTEGER) is
			-- test override
		local
			i : INTEGER
			l_entity : like entity_anchor
			l_matrix : DATED_MATRIX
			l_start_date, l_end_date, l_today : ROSE_DATE
			l_signature : ROSE_PRICE_OVERRIDE_SIGNATURE
			l_db : ROSE_PRICE_DATABASE
			l_historical_signatures : ROSE_TIME_SERIES[ROSE_LINEAR_ARRAY[like ur_override_signature]]
			l_current_signatures : ROSE_LINEAR_ARRAY[like ur_override_signature]
		do
			l_entity := entity_from_id(a_id)
			l_current_signatures := ur_target.entity_override_targets(l_entity)
			l_historical_signatures := ur_target.entity_historical_override_targets(l_entity)

			create l_start_date.make_ccyymmdd(20030801)
			create l_end_date.make_ccyymmdd(20030930)
			create l_today.make_ccyymmdd(20031127)

			if l_entity /= Void then
				l_matrix := l_entity.price_set_interval_series(l_start_date, l_end_date)
				if l_matrix /= Void and then l_matrix.height > 0 then
					l_db := ur_target.zonal_data_manager(l_entity.zone_from_exchange.mnemonic.out)
					ur_target.start_process(l_entity.zone_from_exchange.mnemonic.out, "override", l_today)
					from i := 1 until i > l_matrix.height loop
						l_matrix.put(800, i, 1)
						create l_signature.make(l_entity.handle, l_matrix.dates.item(i), 1, 113930, l_matrix.row_array(i), 1)
						l_signature.set_comment("wrong data from Bloomberg")
						l_signature.set_user_and_update_time
						l_db.replace_override_signature_data(l_signature)
						i := i + 1
					end
					ur_target.commit(l_entity.zone_from_exchange.mnemonic.out)
				end
			end
		end
end
