note

	description:

		"Rose price signature"

	author: "Michael Zhang"
	last_editor: "$Author: mk $"
	date: "$Date: 2011/05/03 $"
	revision: "$Revision: #35 $"
	path: "$File: //rose/source/kernel/data/reports/rose_price_signature.e $"

class
	ROSE_PRICE_SIGNATURE

inherit
   	ROSE_SIGNATURE_RECORD[LISTING]
   		undefine
   			context
		redefine
			date,
			set_date,
			data_fields_string_,
			new_variable,
			expressions, currency_imp
		end

	ROSE_PRICE_RECORD
		undefine
			is_equal
		end

	ROSE_LEGACY_UNDEFINED
		undefine
			is_equal
		end

	ROSE_ENTITY_BACKGROUND_AWARE
		rename
			context as rose_context
		end

create
	make, default_create

feature {NONE} -- Initialization

	make(a_entity_id:INTEGER; a_date :ROSE_DATE; a_value_description_id, a_currency_id:INTEGER; a_array : ARRAY[DOUBLE]; a_source : INTEGER) is
			-- Create
		do
			entity_id:= a_entity_id
			price_date:= a_date
			value_description_id := a_value_description_id
			currency_id:= a_currency_id
			end_current_date := constants.maximum_date_time

			set_data(a_array)
			source := a_source
		end

feature -- fields

	entity_id:INTEGER
	set_entity_id(a_id:INTEGER) is
			-- set entity id
		do
			entity_id:= a_id
		end

	price_date : ROSE_DATE
	set_price_date(a_date : ROSE_DATE) is
			-- set price date
		do
			price_date := a_date
		end

	set_date(a_date : ROSE_DATE) is
			-- Set 'price_date' to 'a_date
		do
			price_date := a_date
		end

	date : ROSE_DATE is
			--price date
		do
			Result := price_date
		end

	value_description_id : INTEGER
	set_value_description_id(a_id : INTEGER) is
			--set value description id
		do
			value_description_id := a_id
		end

	currency_id:INTEGER
	set_currency_id(a_id:INTEGER) is
		do
			currency_id:= a_id
		end

	set_currency(a_currency : PERSISTENT_CURRENCY; a_date : ROSE_DATE) is
		local
			l_currency : PERSISTENT_CURRENCY
		do
			if is_rose(a_currency) then
				currency_id := a_currency.handle
			else
				l_currency := context.currency_accessor.peer(a_currency, a_date)
				if l_currency /= Void then
					currency_id := l_currency.handle
				end
			end
		end

feature -- Data
	source : INTEGER
	set_source(a_source : INTEGER) is
		do
			source := a_source
		end

feature -- signature related
    fields_:STRING is "entity_id,price_date,value_description_id"
	data_fields_string_ : STRING is
		once
			Result := "open,close,high,low,bid,ask,volume,source,currency_id"
		end

	currency_imp, currency : PERSISTENT_CURRENCY is
		do
			Result ?= context.currency_accessor.entity(currency_id)
		end

	context : ROSE_CONTEXT is do Result := rose_context  end
	context_name : STRING is "ROSE ENTITY BACKGROUND"
	expression_tag : STRING is "listing"

	is_rose(a_entity : ENTITY) : BOOLEAN is
		do
			Result := a_entity /= Void and then (a_entity.supplier_id = context.supplier_id)
		end

feature -- signature access
	entity_targets(a_entity : LISTING) : ROSE_LINEAR_ARRAY[ROSE_PRICE_SIGNATURE] is
		require
			valid_entity : a_entity /= Void
		local
			l_sig : like Current
			l_sigs : ROSE_LINEAR_ARRAY[like Current]
			l_manager : like zonal_data_manager
			l_zone : ZONE[LOCATION]
		do
			l_zone := a_entity.zone_from_exchange
			if l_zone /= Void then
				l_manager := zonal_data_manager(l_zone.mnemonic.out)
				if l_manager /= Void then
					create l_sig
					l_sig.set_entity_id(a_entity.handle)
					create l_sigs.make_empty
					l_sigs.extend(l_sig)
					Result := l_manager.retrieve_current_signatures(l_sigs, time_series_statement)
					if Result /= Void then
						Result.sort_by (Agent order_by_date)
					end
				end
			end
		end

	entity_target_series(a_entity : LISTING; a_dates : ARRAY[ROSE_DATE]) : ROSE_LINEAR_ARRAY[ROSE_PRICE_SIGNATURE] is
		require
			valid_entity : a_entity /= Void
		local
			i : INTEGER
			l_targets : like targets
			l_target : like Current
			l_manager : like zonal_data_manager
		do
			if a_entity /= Void then
				l_manager := zonal_data_manager(a_entity.zone_from_exchange.mnemonic.out)
				if l_manager /= Void then
					create l_targets.make_empty
					create l_target
					l_target.set_entity_id(a_entity.handle)
					l_targets.extend(l_target)
					from i := a_dates.lower until i > a_dates.upper loop
						create l_target
						l_target.set_price_date(a_dates.item(i))
						l_targets.extend(l_target)
						i := i + 1
					end
					Result := l_manager.retrieve_current_signatures(l_targets, neighborhood_statement)
					if Result /= Void then
						Result.sort_by (Agent order_by_date)
					end
				end
			end
		end
feature -- override_targets
	entity_override_targets(a_entity : LISTING) : ROSE_TIME_SERIES[like ur_override_target] is
		require
			valid_entity : a_entity /= Void
		local
			l_sig : like Current
			l_sigs : ROSE_LINEAR_ARRAY[like Current]
			l_manager : like zonal_data_manager
			l_override_targets : like ur_override_targets
		do
			l_manager := zonal_data_manager(a_entity.zone_from_exchange.mnemonic.out)
			if l_manager /= Void then
				create l_sig
				l_sig.set_entity_id(a_entity.handle)
				create l_sigs.make_empty
				l_sigs.extend(l_sig)
				l_override_targets := l_manager.retrieve_current_override_signatures(l_sigs, time_series_statement)
				Result := override_target_array_to_series(l_override_targets)
			end
		end

	entity_override_target_series(a_entity : LISTING; a_dates : ARRAY[ROSE_DATE]) : ROSE_TIME_SERIES[like ur_override_target] is
		require
			valid_entity : a_entity /= Void
		local
			i : INTEGER
			l_targets : like targets
			l_target : like Current
			l_manager : like zonal_data_manager
			l_override_targets : like ur_override_targets
		do
			if a_entity /= Void then
				l_manager := zonal_data_manager(a_entity.zone_from_exchange.mnemonic.out)
				if l_manager /= Void then
					create l_targets.make_empty
					create l_target
					l_target.set_entity_id(a_entity.handle)
					l_targets.extend(l_target)
					from i := a_dates.lower until i > a_dates.upper loop
						create l_target
						l_target.set_price_date(a_dates.item(i))
						l_targets.extend(l_target)
						i := i + 1
					end
					l_override_targets := l_manager.retrieve_current_override_signatures(l_targets, neighborhood_statement)
					Result := override_target_array_to_series(l_override_targets)
				end
			end
		end

	entity_historical_override_targets(a_entity : LISTING) : ROSE_TIME_SERIES[ROSE_LINEAR_ARRAY[like ur_override_target]] is
		require
			valid_entity : a_entity /= Void
		local
			l_sig : like Current
			l_sigs : ROSE_LINEAR_ARRAY[like Current]
			l_manager : like zonal_data_manager
			l_target : like ur_override_target
			l_override_targets, l_targets : like ur_override_targets
		do
			l_manager := zonal_data_manager(a_entity.zone_from_exchange.mnemonic.out)
			if l_manager /= Void then
				create l_sig
				l_sig.set_entity_id(a_entity.handle)
				create l_sigs.make_empty
				l_sigs.extend(l_sig)
				l_override_targets := l_manager.retrieve_current_override_signatures(l_sigs, time_series_statement)

				create Result.make_empty
--				Result := l_manager.entity_historical_override_signatures(a_entity.handle)					
				if l_override_targets /= Void then
					from l_override_targets.start until l_override_targets.off loop
						l_target := l_override_targets.current_item
						l_targets := l_manager.retrieve_override_signature_data_history(l_target)
						if l_targets /= Void and then l_targets.count > 0 then
							Result.extend(l_targets, l_target.price_date)
						end
						l_override_targets.forth
					end
				end
			end
		end

	entity_historical_override_target_series(a_entity : LISTING; a_dates : ARRAY[ROSE_DATE]) : like ur_override_targets is
		require
			valid_entity : a_entity /= Void
		local
			i : INTEGER
			l_targets : like targets
			l_target : like Current
			l_manager : like zonal_data_manager
		do
			if a_entity /= Void then
				l_manager := zonal_data_manager(a_entity.zone_from_exchange.mnemonic.out)
				if l_manager /= Void then
					create l_targets.make_empty
					create l_target
					l_target.set_entity_id(a_entity.handle)
					l_targets.extend(l_target)
					from i := a_dates.lower until i > a_dates.upper loop
						create l_target
						l_target.set_price_date(a_dates.item(i))
						l_targets.extend(l_target)
						i := i + 1
					end
					Result := l_manager.retrieve_historical_override_signatures(l_targets, neighborhood_statement)
				end
			end
		end

feature -- override access
	override_target_array_to_series(a_targets : like ur_override_targets) : ROSE_TIME_SERIES[like ur_override_target] is
		local
			l_override_target : like ur_override_target
		do
			create Result.make_empty
			if a_targets /= Void then
				from a_targets.start until a_targets.off loop
					l_override_target := a_targets.current_item
					Result.extend(l_override_target, l_override_target.price_date)
					a_targets.forth
				end
			end
		end

	ur_override_targets : ROSE_LINEAR_ARRAY[like ur_override_target] is do  end
	ur_override_target : ROSE_PRICE_OVERRIDE_SIGNATURE is do  end
feature -- data managers
	zonal_data_manager(a_zone : STRING) : ROSE_PRICE_DATABASE is
		local
			l_process_manager : like process_manager
		do
			if valid_zone_string(a_zone) then
				a_zone.to_lower
				data_manager_table.search(a_zone)
				if data_manager_table.found then
					Result := data_manager_table.found_item
				else
					l_process_manager := process_manager(a_zone)
					create Result.make_with_override(servers.rose_zonal_price_server(a_zone), l_process_manager)
					data_manager_table.force(Result, a_zone)
				end
			end
		end

	start_process(a_zone, a_process_name : STRING; a_date : ROSE_DATE) is
		local
			l_process_manager : like process_manager
		do
			l_process_manager := process_manager(a_zone)
			l_process_manager.start_process(a_process_name, a_date, a_date, a_date)
		end

	end_process(a_zone : STRING) is
		local
			l_process_manager : like process_manager
		do
			l_process_manager := process_manager(a_zone)
			l_process_manager.end_process
		end

	commit(a_zone : STRING) is
		do
			if valid_zone_string(a_zone) then
				zonal_data_manager(a_zone).submit
				end_process(a_zone)
				servers.rose_zonal_price_server(a_zone).commit
			end
		end

feature -- process access
	finished_like_specific_processes_since(a_zone : ZONE[LOCATION]; a_name : STRING; a_time : ROSE_DATE) : SUPPLIER_PROCESS_HISTORY is
		do
			Result := process_manager(a_zone.mnemonic.out).finished_like_specific_processes_since(a_name, a_time)
		end

	last_finished_like_specific_process(a_zone : ZONE[LOCATION]; a_name : STRING) : SUPPLIER_PROCESS is
		do
			Result := process_manager (a_zone.mnemonic.out).last_finished_like_specific_process (a_name)
		end

	last_file_date(a_zone : ZONE[LOCATION]) : ROSE_DATE is
		local
			l_process : SUPPLIER_PROCESS
			l_selection: COLLECTING_PREDICATE [SUPPLIER_PROCESS]
			l_server : SQL_DATA_SERVER
		do
			l_server := servers.rose_zonal_price_server(a_zone.mnemonic.out)
			create l_selection.make (l_server, "select top 1 * from process where name like 'Current Update%%' and end_time < ? order by file_date DESC")
			create l_process
			l_selection.retrieve_one(l_process, Void, <<constants.maximum_date_time>>)
			if l_process /= Void then
				Result := l_process.file_date
			end
		end

feature {NONE} -- implementation
	valid_zone_string(a_zone : STRING) : BOOLEAN is
			-- valid zone string
		do
			if a_zone /= Void then
				 a_zone.to_lower
				 Result := a_zone.is_equal("c") or a_zone.is_equal("u") or a_zone.is_equal("o") or a_zone.is_equal("e") or a_zone.is_equal("j") or a_zone.is_equal("s")
			end
		end

	data_manager_table : HASH_TABLE[ROSE_PRICE_DATABASE, STRING] is
			-- data manager table
		once
			create Result.make(5)
		end

	process_manager_table : HASH_TABLE[SUPPLIER_PROCESS_ID_MANAGER, STRING] is
			-- process manager table
		once
			create Result.make(5)
		end

	process_manager(a_zone : STRING) : SUPPLIER_PROCESS_ID_MANAGER is
			-- process manager
		do
			if valid_zone_string(a_zone) then
				a_zone.to_lower
				process_manager_table.search(a_zone)
				if process_manager_table.found then
					Result := process_manager_table.found_item
				else
					create Result.make(servers.rose_zonal_price_server(a_zone))
					Result.set_name("Price Update")
					process_manager_table.force(Result, a_zone)
				end
			end
		end
feature {ROSE_PRICE_REPORT, ROSE_PRICE_VARIABLE} -- access control
	restriction_statement : STRING is
			-- restriction statement
		do
			if is_direction_time_series then
				Result := time_series_statement
			elseif is_direction_cross_sectional then
				Result := cross_sectional_statement
			elseif is_direction_neighborhood then
				Result := neighborhood_statement
			else
				Result := time_series_statement
			end
		end

	time_series_statement : STRING is
			-- time series statement
		"#entity_id.id = entity_id"

	cross_sectional_statement : STRING is
			-- cross sectional statement
		"#price_date.id = price_date"

	neighborhood_statement : STRING is
			-- neighborhood statement
		"#entity_id.id = entity_id and #price_date.id = price_date"

feature -- variables
	new_variable(a_variable_description : VARIABLE_DESCRIPTION_RECORD) :  SIGNATURE_TAGGED_EXPRESSION[like Current] is
			-- new variable
		do
			create {ROSE_PRICE_VARIABLE} Result.make(a_variable_description)
		end

	expressions : like expressions_anchor is
			-- Expressions
		do
			Result ?= expressions_table.item(internal_type)
			if Result = Void then
				create Result.make_from_descriptions(context.variable_descriptions(expression_tag),agent new_variable, context.mappings)
				expressions_table.put(Result, internal_type)
			end
		end

feature -- RMDAT (fg database)
	rmdat_signature : RMDAT_SIGNATURE_RECORD is
			-- rmdat signature
		do
			create Result
			Result.set_date(price_date)
			Result.set_ask(ask)
			Result.set_bid(bid)
			Result.set_high(high)
			Result.set_low(low)
			Result.set_open(open)
			Result.set_close(close)
			Result.set_volume(volume)
			Result.set_currency (currency_id)
		end

feature -- formatted output

	formatted_output : STRING
			-- formatted output
		local
			l_string, l_currency_string : STRING
			l_currency : CURRENCY
		do
			l_currency_string := ""
			if currency_id > 0 then
				l_currency := context.currency_accessor.entity (currency_id)
				if l_currency /= Void then
					l_currency_string := l_currency.iso
				end
			end
			l_string := price_date.ccyymmdd.out
			l_string := l_string + "  " + formatted_double (close)
			l_string := l_string + "  " + formatted_double (bid)
			l_string := l_string + "  " + formatted_double (ask)
			l_string := l_string + "  " + formatted_double (high)
			l_string := l_string + "  " + formatted_double (low)
			l_string := l_string + "  " + formatted_integer_double (volume)
			l_string := l_string + "  " + l_currency_string

			Result := l_string
		end

	formatted_double (a_double: DOUBLE): STRING
			-- formatted double
		do
			if is_defined_double (a_double) then
				Result := format_double.formatted (a_double)
			else
				Result := "   n/a   "
			end
		end

	format_double: FORMAT_DOUBLE
			-- format double
		once
			create Result.make (8, 3)
		end

	formatted_integer_double (a_double: DOUBLE): STRING
			-- formatted integer
		do
			if is_defined_double (a_double) then
				Result := a_double.out
			else
				Result := "   n/a   "
			end
		end

	right_aligned_string (a_str: STRING; a_size: INTEGER): STRING
			-- right aligned string
		local
			l_spaces: STRING_8
		do
			Result := a_str
			if Result /= Void and then Result.count < a_size then
				create l_spaces.make_filled (' ', a_size - Result.count)
				Result := l_spaces + Result
			end
		end
end
