indexing
	description: "Progress status report via events"
	author: "Zoran Simic"
	date: "$Date: 2009/06/23 $"
	revision: "$Revision: #9 $"

class
	ROSE_PROGRESS

inherit
	ROSE_FORMATTING

feature -- Access

	frozen busy_event: ROSE_EVENT_TYPE [TUPLE [ROSE_STOPWATCH]] is
			-- Fired whenever 'start_busy' or 'end_busy' is called
		once
			create Result
		ensure
			result_ok: Result /= Void
		end

	frozen busy_stack: ROSE_STACK [ROSE_STOPWATCH] is
			-- Stack of current busy messages
		once
			create Result
		ensure
			result_ok: Result /= Void
		end

feature -- Basic operations

	start_busy (a_message: STRING) is
			-- Notify user that a busy operation described by 'a_message' is in progress.
		local
			l_stopwatch: ROSE_STOPWATCH
		do
			create l_stopwatch.make (a_message)
			l_stopwatch.start
			busy_stack.put (l_stopwatch)
			trigger_busy_event (l_stopwatch)
		end

	end_busy is
			-- Notify that current 'busy' operation is finished.
		local
			l_stopwatch: ROSE_STOPWATCH
		do
			if busy_stack.is_empty then
				-- This should be a 'require', but... 'busy_stack' is not exported,
				-- and we don't want to crash in finalized mode on this anyway
				check not busy_stack.is_empty end
				if glob.Global.app.is_running_in_estudio then
					-- If the above did not raise an exception, then we raise one...
					-- But only if running from estudio... (avoid crashing a finalized app)
					(create {EXCEPTIONS}).raise ("You called 'end_busy' too many times!")
				end
			else
				l_stopwatch := busy_stack.item
				check l_stopwatch /= Void end
				busy_stack.remove
				if l_stopwatch /= Void then
					l_stopwatch.stop
					trigger_busy_event (l_stopwatch)
					debug ("progress")
						print (l_stopwatch)
						print ("%N")
					end
				elseif glob.Global.app.is_running_in_estudio then
					(create {EXCEPTIONS}).raise ("A void 'busy_message' was stacked, please check!")
				end
			end
		end

feature {NONE} -- Implementation

	invoker_cell: CELL [PROCEDURE [ANY, TUPLE [PROCEDURE [ANY, TUPLE []]]]] is
			-- Invoke redirection, used to store gui.invoke without pulling GUI and Vision2 classes in the system
		indexing
			once_status : global
		once
			create Result.put (Void)
		end

	trigger_busy_event (a_stopwatch: ROSE_STOPWATCH) is
			-- Trigger 'busy_event' with 'a_stopwatch'
		require
			stopwatch_ok: a_stopwatch /= Void
		do
			busy_event.call ([a_stopwatch])
		end

	frozen glob: ROSE_SHARED_GLOBAL is
			-- Reference to global Global object, to avoid inheriting (and inheritance clashes)
		once
			create Result
		ensure
			result_ok: Result /= Void
		end

end
