indexing

	description: "Column of stacked widgets whose height is a fixed proportion of enclosing widget"

	author: "Franck Arnaud"
	last_editor: "$Author: mk $"
	team: "RC_London"
	revision: "$Revision: #3 $"
	date: "$Date: 2009/06/25 $"
	path: "$File: //rose/source/kernel/infrastructure/widgets/rose_proportional_vertical_box.e $"

class
	ROSE_PROPORTIONAL_VERTICAL_BOX

inherit
	EV_FIXED
		export
			{NONE} fill, put, prune_all -- prevent adding via unmanaged route (incomplete...)
		redefine
			initialize,
			extend,
			wipe_out
		end


create
	default_create

feature {NONE} -- Creation

	initialize is
			-- Creates empty colum,
			-- add widgets using extend_weighted.
		do
			set_minimum_size (0, 0)
			create weights.make_empty
			Precursor
			resize_actions.extend (agent on_resize)
		end

feature {NONE} -- Drawing

	on_resize (a_x, a_y, a_width, a_height: INTEGER_32)
			-- Respond to resize events and rescale the graph.
		require
			a_width_positive: a_width >= 0
			a_height_positive: a_height >= 0
		do
			if a_width > 0 and a_height > 0 and count > 0 then
				resize_widgets (a_width, a_height)
			end
		end

	resize_widgets (a_width, a_height: INTEGER_32)
			-- Resize all widgets proportionally to parent.
		require
			a_width_positive: a_width > 0
			a_height_positive: a_height > 0
		local
			l_y: CELL [INTEGER]
			l_widgets: ROSE_LINEAR_ARRAY [EV_WIDGET]
		do
			create l_y.put (0)
			create l_widgets.make_from_linear (Current) -- to get do_all_with_index (not in EV_LIST)
			l_widgets.do_all_with_index (agent resize_widget (a_width, a_height, l_y, ?, ?))
		end

	resize_widget (a_width, a_height: INTEGER; a_y: CELL [INTEGER]; a_widget: EV_WIDGET; a_index: INTEGER)
			-- Resize one widget within new frame of size (`a_width',`a_height')
			-- with `a_y' defining the position of the preceeding widget.
		require
			positive_size: a_width >= 0 and a_height >= 0
			a_y_not_void: a_y /= Void
			a_y_positive: a_y.item >= 0
			valid_weights: weights.valid_index (a_index)
			known_widget: has (a_widget)
		local
			l_height: INTEGER
		do
			l_height := a_height * weights.item (a_index) // total_weights
				-- Apply
			set_item_position_and_size (a_widget, 0, a_y.item, a_width, l_height)
				-- Set next y position
			a_y.put (a_y.item + l_height)
		end

feature -- Access

	weights: ROSE_LINEAR_ARRAY [INTEGER]
		-- Weights for widgets at same position

	total_weights: INTEGER
		-- Sum all weights.

feature -- Setting

	extend (a_widget: EV_WIDGET) is
			-- Add widget with average weight
		do
			if weights.count <= count then
				-- no weight pending, so this is a direct call (not via extend_weighted),
				-- so set default weight
				if count > 0 then
					-- average weight, if we have items to calibrate
					weights_extend ((total_weights // count).max (Min_extend_weight))
				else
					weights_extend (Min_extend_weight)
				end
			end
			Precursor (a_widget)
		end

	extend_weighted (a_widget: EV_WIDGET; a_weight: INTEGER)
			-- Add a widget in this vertical box, whose height will
			-- be kept proportional to weight relative to others.
		do
			-- extend will look at the weight at count + 1 position
			weights_extend (a_weight)
			extend (a_widget)
		end

	wipe_out is
			-- Remove all widgets and clear weights.
		do
			weights.wipe_out
			total_weights := 0
			Precursor
		end


feature {NONE} -- Implementation

	weights_extend (a_weight: INTEGER)
			-- Add weight to end of list and update total
		do
			weights.extend (a_weight)
			total_weights := total_weights + a_weight
		end

	Min_extend_weight: INTEGER is 1
			-- Default weight when first weight and non specified

invariant
	weights_not_void: weights /= Void
	weights_count: count = weights.count
	total_weights_positive: total_weights >= 0

end
