indexing
	description : "[
		Class for binary data ... only difference with ROSE_RAW_FILE is that it has
		a feature read_raw_data which is implemented using file_gss
		]"
	author : "Mark Howard"
	date: "$Date: 2010/01/24 $"
	revision: "$Revision: #6 $"

class
	ROSE_RAW_FILE

inherit
	RAW_FILE
		redefine
			open_read,
			open_write,
			open_append,
			open_read_write,
			create_read_write,
			open_read_append
		end
	FILE_ACCESS_LOG

create
	make_open_read, make_open_write,make

feature -- Basic operations

	read_raw_data (p: POINTER; nb_char: INTEGER) is
			-- Read a string of at most `nb_char' bound characters
			-- or until end of file.
			-- Make result available in `p'.
		local
			new_count: INTEGER
		do
			new_count := file_gss (file_pointer, p, nb_char)
		end

feature -- Initialization

	open_read is
			-- Open file in read-only mode.
		do
			Precursor
			record_file_read_access (name)
		rescue
			error.signal (generator, "Can't open_read file '$1'", <<name>>)
		end

	open_write is
			-- Open file in write-only mode;
			-- create it if it does not exist.
		do
			Precursor
			record_file_write_access (name)
		rescue
			error.signal (generator, "Can't open_write file '$1'", <<name>>)
		end


	open_append is
			-- Open file in append-only mode;
			-- create it if it does not exist.
		do
			Precursor
			record_file_write_access (name)
		rescue
			error.signal (generator, "Can't open_append file '$1'", <<name>>)
		end

	open_read_write is
			-- Open file in read and write mode.
		do
			Precursor
			record_file_write_access (name)
		rescue
			error.signal (generator, "Can't open_read_write file '$1'", <<name>>)
		end

	create_read_write is
			-- Open file in read and write mode;
			-- create it if it does not exist.
		do
			Precursor
			record_file_write_access (name)
		rescue
			error.signal (generator, "Can't create_read_write file '$1'", <<name>>)
		end

	open_read_append is
			-- Open file in read and write-at-end mode;
			-- create it if it does not exist.
		do
			Precursor
			record_file_write_access (name)
		rescue
			error.signal (generator, "Can't open_read_append file '$1'", <<name>>)
		end

end
