note
	description: "Entity transformation recipe manager"
	author: "Zoran Simic"
	last_editor: "$Author: zs $"
	date: "$Date: 2011/05/27 $"
	revision: "$Revision: #8 $"
	path: "$File: //rose/source/kernel/entities/transformation/rose_recipe_manager.e $"

class
	ROSE_RECIPE_MANAGER

inherit
	ROSE_RECIPE_UTILS

create
	make

feature {NONE} -- Initialization

	make
			-- Create a recipe manager
		do
			create types.make_empty
			create types_hash.make (0)
			create parser.make
			add_type (create {ROSE_TRANSFORMATION_RELATIONSHIP})
			add_type (create {ROSE_TRANSFORMATION_CORELATIONSHIP})
			add_type (create {ROSE_TRANSFORMATION_PEER})
			add_type (create {ROSE_TRANSFORMATION_ATTRIBUTE})
		end

feature -- Access

	types: ROSE_LINEAR_ARRAY [ROSE_TRANSFORMATION]				-- Available transformation types
	types_hash: HASH_TABLE [ROSE_TRANSFORMATION, STRING]		-- Available transformation types, hashed by name

	transformation (a_type: STRING): ROSE_TRANSFORMATION
			-- Transformation with 'a_type', uninitialized
		do
			Result := types_hash.item (a_type)
			if Result /= Void then
				Result := Result.twin
			end
		end

	recipe (a_description: STRING): ROSE_RECIPE
			-- Recipe parsed from 'a_description'
		do
			parser.parse_string (a_description)
			Result := parser.recipe
		end

	recipe_from_file (a_file_path: STRING): ROSE_RECIPE
			-- Recipe store in file with 'a_file_path'
		require
			valid_path: a_file_path /= Void and then not a_file_path.is_empty
		do
			parser.parse_file (a_file_path)
			Result := parser.recipe
		end

feature -- Basic operations

	add_type (a_type: ROSE_TRANSFORMATION)
			-- Add 'a_type' to available 'types'
		do
			types_hash.force (a_type,a_type.id)
			types.extend (a_type)
		end

	save_recipe_to_file (a_recipe: ROSE_RECIPE; a_filepath: STRING)
			-- Save 'a_recipe' to file with 'a_filepath'.
		local
			l_file: PLAIN_TEXT_FILE
			rescued: STRING
		do
			if rescued /= Void then
				error.signal (generator,"Can not save recipe to '$1': $2",<<a_filepath,rescued>>)
			end
			create l_file.make (a_filepath)
			if l_file.exists and then not l_file.is_writable then
				error.signal (generator,"Can not save recipe to '$1': file is not writable",<<a_filepath>>)
			else
				l_file.open_write
				l_file.put_string (a_recipe.representation)
				l_file.close
			end
		rescue
			if rescued = Void then
				rescued := last_exception_explanation
				retry
			end
		end

feature {NONE} -- Implementation

	parser: ROSE_RECIPE_PARSER
			-- Recipe parser

end
