note
	description: "Parser for entity recipe transformations"
	author: "Zoran Simic"
	last_editor: "$Author: er $"
	date: "$Date: 2012/09/04 $"
	revision: "$Revision: #12 $"
	path: "$File: //rose/source/kernel/entities/transformation/rose_recipe_parser.e $"

class
	ROSE_RECIPE_PARSER

inherit
	YY_PARSER_SKELETON
		rename
			make as make_parser_skeleton
		redefine
			report_error
		end
	ROSE_RECIPE_SCANNER
	ROSE_SHARED_GLOBAL

create
	make

feature {NONE} -- Initialization

	make is
			-- Create a new Rose recipe parser.
		do
			make_rose_recipe_scanner
			make_parser_skeleton
		end

feature -- Access

	id: STRING = "recipe parser"
			-- id of this section

	recipe: ROSE_RECIPE
			-- Last parsed recipe

	recipe_list: ROSE_LINEAR_ARRAY [ROSE_RECIPE]

feature -- Basic operations

	parse_file (a_path: STRING) is
			-- Parse a recipe from 'a_path'.
			-- 'error.signaled' is true if an error occurs during this operation
			-- Use 'error.flush_and_acknowledge' to get the error message output
		require
			valid_path: a_path /= Void
			errors_acknowledged: not error.signaled
		local
			l_file: KL_TEXT_INPUT_FILE
			rescued: STRING
		do
			if rescued /= Void then
				error.signal (id, "The following problem occurred while trying to parse recipe from '$1':%N$2", <<a_path, rescued>>)
			else
				buffer_name := a_path
				create l_file.make (buffer_name)
				if not l_file.exists then
					error.signal (id, "Can not read '$1', file does not exist", <<buffer_name>>)
				elseif not l_file.is_readable then
					error.signal (id, "Can not read '$1', file is not readable", <<buffer_name>>)
				else
					reset_recipe_parser
					l_file.open_read
					set_input_buffer (new_file_buffer (l_file))
					parse
					l_file.close
				end
			end
			buffer_name := Void
		rescue
			if rescued = Void then
				rescued :=  last_exception_explanation
				retry
			end
		end

	parse_string (a_string: STRING) is
			-- Parse recipe from 'a_string'.
			-- 'error.signaled' is true if an error occur during this operation
			-- Use 'error.flush_and_acknowledge' to get the error message output
		require
			valid_string: a_string /= Void
			errors_acknowledged: not error.signaled
		local
			rescued: STRING
		do
			if rescued /= Void then
				error.signal (id, "The following problem occurred while trying to parse recipe '$1':%N$2", <<a_string, rescued>>)
			else
				buffer_name := "*buffer*"
				reset_recipe_parser
				set_input_buffer (new_string_buffer (a_string))
				parse
				buffer_name := Void
			end
		rescue
			if rescued = Void then
				rescued :=  last_exception_explanation
				retry
			end
		end


feature {NONE} -- Implementation

	yy_build_parser_tables
			-- Build parser tables.
		do
			yytranslate := yytranslate_template
			yyr1 := yyr1_template
			yytypes1 := yytypes1_template
			yytypes2 := yytypes2_template
			yydefact := yydefact_template
			yydefgoto := yydefgoto_template
			yypact := yypact_template
			yypgoto := yypgoto_template
			yytable := yytable_template
			yycheck := yycheck_template
		end

	yy_create_value_stacks
			-- Create value stacks.
		do
		end

	yy_init_value_stacks
			-- Initialize value stacks.
		do
			yyvsp1 := -1
			yyvsp2 := -1
			yyvsp3 := -1
			yyvsp4 := -1
			yyvsp5 := -1
			yyvsp6 := -1
			yyvsp7 := -1
			yyvsp8 := -1
			yyvsp9 := -1
			yyvsp10 := -1
			yyvsp11 := -1
			yyvsp12 := -1
		end

	yy_clear_value_stacks
			-- Clear objects in semantic value stacks so that
			-- they can be collected by the garbage collector.
		local
			l_yyvs1_default_item: ANY
			l_yyvs2_default_item: ROSE_RECIPE
			l_yyvs3_default_item: STRING
			l_yyvs4_default_item: ROSE_TRANSFORMATION
			l_yyvs5_default_item: ROSE_TRANSFORMATION_SETTINGS
			l_yyvs6_default_item: ROSE_TRANSFORMATION_SETTING_DISPLAY
			l_yyvs7_default_item: ROSE_TRANSFORMATION_SETTING_FILTER
			l_yyvs8_default_item: ROSE_TRANSFORMATION_SETTING_CONSTRAINT
			l_yyvs9_default_item: ROSE_LINEAR_ARRAY [STRING]
			l_yyvs10_default_item: INTEGER
			l_yyvs11_default_item: ROSE_LINEAR_ARRAY [INTEGER]
			l_yyvs12_default_item: ROSE_DATE
		do
			if yyvs1 /= Void then
				yyvs1.fill_with (l_yyvs1_default_item, 0, yyvs1.upper)
			end
			if yyvs2 /= Void then
				yyvs2.fill_with (l_yyvs2_default_item, 0, yyvs2.upper)
			end
			if yyvs3 /= Void then
				yyvs3.fill_with (l_yyvs3_default_item, 0, yyvs3.upper)
			end
			if yyvs4 /= Void then
				yyvs4.fill_with (l_yyvs4_default_item, 0, yyvs4.upper)
			end
			if yyvs5 /= Void then
				yyvs5.fill_with (l_yyvs5_default_item, 0, yyvs5.upper)
			end
			if yyvs6 /= Void then
				yyvs6.fill_with (l_yyvs6_default_item, 0, yyvs6.upper)
			end
			if yyvs7 /= Void then
				yyvs7.fill_with (l_yyvs7_default_item, 0, yyvs7.upper)
			end
			if yyvs8 /= Void then
				yyvs8.fill_with (l_yyvs8_default_item, 0, yyvs8.upper)
			end
			if yyvs9 /= Void then
				yyvs9.fill_with (l_yyvs9_default_item, 0, yyvs9.upper)
			end
			if yyvs10 /= Void then
				yyvs10.fill_with (l_yyvs10_default_item, 0, yyvs10.upper)
			end
			if yyvs11 /= Void then
				yyvs11.fill_with (l_yyvs11_default_item, 0, yyvs11.upper)
			end
			if yyvs12 /= Void then
				yyvs12.fill_with (l_yyvs12_default_item, 0, yyvs12.upper)
			end
		end

	yy_push_last_value (yychar1: INTEGER)
			-- Push semantic value associated with token `last_token'
			-- (with internal id `yychar1') on top of corresponding
			-- value stack.
		do
			inspect yytypes2.item (yychar1)
			when 1 then
				yyvsp1 := yyvsp1 + 1
				if yyvsp1 >= yyvsc1 then
					if yyvs1 = Void then
						debug ("GEYACC")
							std.error.put_line ("Create yyvs1")
						end
						create yyspecial_routines1
						yyvsc1 := yyInitial_yyvs_size
						yyvs1 := yyspecial_routines1.make (yyvsc1)
					else
						debug ("GEYACC")
							std.error.put_line ("Resize yyvs1")
						end
						yyvsc1 := yyvsc1 + yyInitial_yyvs_size
						yyvs1 := yyspecial_routines1.resize (yyvs1, yyvsc1)
					end
				end
				yyspecial_routines1.force (yyvs1, last_any_value, yyvsp1)
			when 10 then
				yyvsp10 := yyvsp10 + 1
				if yyvsp10 >= yyvsc10 then
					if yyvs10 = Void then
						debug ("GEYACC")
							std.error.put_line ("Create yyvs10")
						end
						create yyspecial_routines10
						yyvsc10 := yyInitial_yyvs_size
						yyvs10 := yyspecial_routines10.make (yyvsc10)
					else
						debug ("GEYACC")
							std.error.put_line ("Resize yyvs10")
						end
						yyvsc10 := yyvsc10 + yyInitial_yyvs_size
						yyvs10 := yyspecial_routines10.resize (yyvs10, yyvsc10)
					end
				end
				yyspecial_routines10.force (yyvs10, last_integer_value, yyvsp10)
			when 12 then
				yyvsp12 := yyvsp12 + 1
				if yyvsp12 >= yyvsc12 then
					if yyvs12 = Void then
						debug ("GEYACC")
							std.error.put_line ("Create yyvs12")
						end
						create yyspecial_routines12
						yyvsc12 := yyInitial_yyvs_size
						yyvs12 := yyspecial_routines12.make (yyvsc12)
					else
						debug ("GEYACC")
							std.error.put_line ("Resize yyvs12")
						end
						yyvsc12 := yyvsc12 + yyInitial_yyvs_size
						yyvs12 := yyspecial_routines12.resize (yyvs12, yyvsc12)
					end
				end
				yyspecial_routines12.force (yyvs12, last_rose_date_value, yyvsp12)
			when 3 then
				yyvsp3 := yyvsp3 + 1
				if yyvsp3 >= yyvsc3 then
					if yyvs3 = Void then
						debug ("GEYACC")
							std.error.put_line ("Create yyvs3")
						end
						create yyspecial_routines3
						yyvsc3 := yyInitial_yyvs_size
						yyvs3 := yyspecial_routines3.make (yyvsc3)
					else
						debug ("GEYACC")
							std.error.put_line ("Resize yyvs3")
						end
						yyvsc3 := yyvsc3 + yyInitial_yyvs_size
						yyvs3 := yyspecial_routines3.resize (yyvs3, yyvsc3)
					end
				end
				yyspecial_routines3.force (yyvs3, last_string_value, yyvsp3)
			else
				debug ("GEYACC")
					std.error.put_string ("Error in parser: not a token type: ")
					std.error.put_integer (yytypes2.item (yychar1))
					std.error.put_new_line
				end
				abort
			end
		end

	yy_push_error_value
			-- Push semantic value associated with token 'error'
			-- on top of corresponding value stack.
		local
			yyval1: ANY
		do
			yyvsp1 := yyvsp1 + 1
			if yyvsp1 >= yyvsc1 then
				if yyvs1 = Void then
					debug ("GEYACC")
						std.error.put_line ("Create yyvs1")
					end
					create yyspecial_routines1
					yyvsc1 := yyInitial_yyvs_size
					yyvs1 := yyspecial_routines1.make (yyvsc1)
				else
					debug ("GEYACC")
						std.error.put_line ("Resize yyvs1")
					end
					yyvsc1 := yyvsc1 + yyInitial_yyvs_size
					yyvs1 := yyspecial_routines1.resize (yyvs1, yyvsc1)
				end
			end
			yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
		end

	yy_pop_last_value (yystate: INTEGER)
			-- Pop semantic value from stack when in state `yystate'.
		local
			yy_type_id: INTEGER
		do
			yy_type_id := yytypes1.item (yystate)
			inspect yy_type_id
			when 1 then
				yyvsp1 := yyvsp1 - 1
			when 2 then
				yyvsp2 := yyvsp2 - 1
			when 3 then
				yyvsp3 := yyvsp3 - 1
			when 4 then
				yyvsp4 := yyvsp4 - 1
			when 5 then
				yyvsp5 := yyvsp5 - 1
			when 6 then
				yyvsp6 := yyvsp6 - 1
			when 7 then
				yyvsp7 := yyvsp7 - 1
			when 8 then
				yyvsp8 := yyvsp8 - 1
			when 9 then
				yyvsp9 := yyvsp9 - 1
			when 10 then
				yyvsp10 := yyvsp10 - 1
			when 11 then
				yyvsp11 := yyvsp11 - 1
			when 12 then
				yyvsp12 := yyvsp12 - 1
			else
				debug ("GEYACC")
					std.error.put_string ("Error in parser: unknown type id: ")
					std.error.put_integer (yy_type_id)
					std.error.put_new_line
				end
				abort
			end
		end

	yy_run_geyacc
			-- You must run geyacc to regenerate this class.
		do
		end

feature {NONE} -- Semantic actions

	yy_do_action (yy_act: INTEGER)
			-- Execute semantic action.
		local
			yyval1: ANY
			yyval2: ROSE_RECIPE
			yyval3: STRING
			yyval4: ROSE_TRANSFORMATION
			yyval5: ROSE_TRANSFORMATION_SETTINGS
			yyval6: ROSE_TRANSFORMATION_SETTING_DISPLAY
			yyval7: ROSE_TRANSFORMATION_SETTING_FILTER
			yyval8: ROSE_TRANSFORMATION_SETTING_CONSTRAINT
			yyval9: ROSE_LINEAR_ARRAY [STRING]
			yyval11: ROSE_LINEAR_ARRAY [INTEGER]
		do
			inspect yy_act
when 1 then
--|#line 133 ".\rose_recipe_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file '.\rose_recipe_parser.y' at line 133")
end

accept 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 0
	yyvsp1 := yyvsp1 + 1
	if yyvsp1 >= yyvsc1 then
		if yyvs1 = Void then
			debug ("GEYACC")
				std.error.put_line ("Create yyvs1")
			end
			create yyspecial_routines1
			yyvsc1 := yyInitial_yyvs_size
			yyvs1 := yyspecial_routines1.make (yyvsc1)
		else
			debug ("GEYACC")
				std.error.put_line ("Resize yyvs1")
			end
			yyvsc1 := yyvsc1 + yyInitial_yyvs_size
			yyvs1 := yyspecial_routines1.resize (yyvs1, yyvsc1)
		end
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 2 then
--|#line 134 ".\rose_recipe_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file '.\rose_recipe_parser.y' at line 134")
end

accept 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -1
	if yyvsp1 >= yyvsc1 then
		if yyvs1 = Void then
			debug ("GEYACC")
				std.error.put_line ("Create yyvs1")
			end
			create yyspecial_routines1
			yyvsc1 := yyInitial_yyvs_size
			yyvs1 := yyspecial_routines1.make (yyvsc1)
		else
			debug ("GEYACC")
				std.error.put_line ("Resize yyvs1")
			end
			yyvsc1 := yyvsc1 + yyInitial_yyvs_size
			yyvs1 := yyspecial_routines1.resize (yyvs1, yyvsc1)
		end
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 3 then
--|#line 135 ".\rose_recipe_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file '.\rose_recipe_parser.y' at line 135")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 4 then
--|#line 138 ".\rose_recipe_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file '.\rose_recipe_parser.y' at line 138")
end

create yyval2
		yyval2.set_name (yyvs3.item (yyvsp3 - 1))
		yyval2.set_description (yyvs3.item (yyvsp3))
		yyval2.set_settings (yyvs5.item (yyvsp5))
		yyval2.add_transformation (yyvs4.item (yyvsp4))
		set_recipe (yyval2)
	
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 6
	yyvsp2 := yyvsp2 + 1
	yyvsp1 := yyvsp1 -2
	yyvsp3 := yyvsp3 -2
	yyvsp5 := yyvsp5 -1
	yyvsp4 := yyvsp4 -1
	if yyvsp2 >= yyvsc2 then
		if yyvs2 = Void then
			debug ("GEYACC")
				std.error.put_line ("Create yyvs2")
			end
			create yyspecial_routines2
			yyvsc2 := yyInitial_yyvs_size
			yyvs2 := yyspecial_routines2.make (yyvsc2)
		else
			debug ("GEYACC")
				std.error.put_line ("Resize yyvs2")
			end
			yyvsc2 := yyvsc2 + yyInitial_yyvs_size
			yyvs2 := yyspecial_routines2.resize (yyvs2, yyvsc2)
		end
	end
	yyspecial_routines2.force (yyvs2, yyval2, yyvsp2)
end
when 5 then
--|#line 152 ".\rose_recipe_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file '.\rose_recipe_parser.y' at line 152")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 0
	yyvsp3 := yyvsp3 + 1
	if yyvsp3 >= yyvsc3 then
		if yyvs3 = Void then
			debug ("GEYACC")
				std.error.put_line ("Create yyvs3")
			end
			create yyspecial_routines3
			yyvsc3 := yyInitial_yyvs_size
			yyvs3 := yyspecial_routines3.make (yyvsc3)
		else
			debug ("GEYACC")
				std.error.put_line ("Resize yyvs3")
			end
			yyvsc3 := yyvsc3 + yyInitial_yyvs_size
			yyvs3 := yyspecial_routines3.resize (yyvs3, yyvsc3)
		end
	end
	yyspecial_routines3.force (yyvs3, yyval3, yyvsp3)
end
when 6 then
--|#line 153 ".\rose_recipe_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file '.\rose_recipe_parser.y' at line 153")
end

yyval3 := yyvs3.item (yyvsp3) 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp1 := yyvsp1 -1
	yyspecial_routines3.force (yyvs3, yyval3, yyvsp3)
end
when 7 then
--|#line 156 ".\rose_recipe_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file '.\rose_recipe_parser.y' at line 156")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 0
	yyvsp4 := yyvsp4 + 1
	if yyvsp4 >= yyvsc4 then
		if yyvs4 = Void then
			debug ("GEYACC")
				std.error.put_line ("Create yyvs4")
			end
			create yyspecial_routines4
			yyvsc4 := yyInitial_yyvs_size
			yyvs4 := yyspecial_routines4.make (yyvsc4)
		else
			debug ("GEYACC")
				std.error.put_line ("Resize yyvs4")
			end
			yyvsc4 := yyvsc4 + yyInitial_yyvs_size
			yyvs4 := yyspecial_routines4.resize (yyvs4, yyvsc4)
		end
	end
	yyspecial_routines4.force (yyvs4, yyval4, yyvsp4)
end
when 8 then
--|#line 157 ".\rose_recipe_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file '.\rose_recipe_parser.y' at line 157")
end

yyval4 := yyvs4.item (yyvsp4) 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines4.force (yyvs4, yyval4, yyvsp4)
end
when 9 then
--|#line 158 ".\rose_recipe_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file '.\rose_recipe_parser.y' at line 158")
end

yyval4 := yyvs4.item (yyvsp4 - 1); yyval4.extend (yyvs4.item (yyvsp4)) 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp4 := yyvsp4 -1
	yyspecial_routines4.force (yyvs4, yyval4, yyvsp4)
end
when 10 then
--|#line 161 ".\rose_recipe_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file '.\rose_recipe_parser.y' at line 161")
end

yyval4 := yyvs4.item (yyvsp4); yyval4.set_settings (yyvs5.item (yyvsp5)) 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp5 := yyvsp5 -1
	yyspecial_routines4.force (yyvs4, yyval4, yyvsp4)
end
when 11 then
--|#line 166 ".\rose_recipe_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file '.\rose_recipe_parser.y' at line 166")
end

yyval4 := yyvs4.item (yyvsp4) 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines4.force (yyvs4, yyval4, yyvsp4)
end
when 12 then
--|#line 168 ".\rose_recipe_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file '.\rose_recipe_parser.y' at line 168")
end

yyval4 := yyvs4.item (yyvsp4) 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines4.force (yyvs4, yyval4, yyvsp4)
end
when 13 then
--|#line 169 ".\rose_recipe_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file '.\rose_recipe_parser.y' at line 169")
end

yyval4 := yyvs4.item (yyvsp4) 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines4.force (yyvs4, yyval4, yyvsp4)
end
when 14 then
--|#line 170 ".\rose_recipe_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file '.\rose_recipe_parser.y' at line 170")
end

yyval4 := yyvs4.item (yyvsp4) 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines4.force (yyvs4, yyval4, yyvsp4)
end
when 15 then
--|#line 173 ".\rose_recipe_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file '.\rose_recipe_parser.y' at line 173")
end

create {ROSE_TRANSFORMATION_RELATIONSHIP} yyval4.make (yyvs3.item (yyvsp3 - 2), yyvs3.item (yyvsp3 - 1), yyvs10.item (yyvsp10), yyvs3.item (yyvsp3)) 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 10
	yyvsp4 := yyvsp4 + 1
	yyvsp1 := yyvsp1 -6
	yyvsp3 := yyvsp3 -3
	yyvsp10 := yyvsp10 -1
	if yyvsp4 >= yyvsc4 then
		if yyvs4 = Void then
			debug ("GEYACC")
				std.error.put_line ("Create yyvs4")
			end
			create yyspecial_routines4
			yyvsc4 := yyInitial_yyvs_size
			yyvs4 := yyspecial_routines4.make (yyvsc4)
		else
			debug ("GEYACC")
				std.error.put_line ("Resize yyvs4")
			end
			yyvsc4 := yyvsc4 + yyInitial_yyvs_size
			yyvs4 := yyspecial_routines4.resize (yyvs4, yyvsc4)
		end
	end
	yyspecial_routines4.force (yyvs4, yyval4, yyvsp4)
end
when 16 then
--|#line 178 ".\rose_recipe_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file '.\rose_recipe_parser.y' at line 178")
end

create {ROSE_TRANSFORMATION_CORELATIONSHIP} yyval4.make (yyvs3.item (yyvsp3 - 2), yyvs3.item (yyvsp3 - 1), yyvs10.item (yyvsp10), yyvs3.item (yyvsp3)) 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 10
	yyvsp4 := yyvsp4 + 1
	yyvsp1 := yyvsp1 -6
	yyvsp3 := yyvsp3 -3
	yyvsp10 := yyvsp10 -1
	if yyvsp4 >= yyvsc4 then
		if yyvs4 = Void then
			debug ("GEYACC")
				std.error.put_line ("Create yyvs4")
			end
			create yyspecial_routines4
			yyvsc4 := yyInitial_yyvs_size
			yyvs4 := yyspecial_routines4.make (yyvsc4)
		else
			debug ("GEYACC")
				std.error.put_line ("Resize yyvs4")
			end
			yyvsc4 := yyvsc4 + yyInitial_yyvs_size
			yyvs4 := yyspecial_routines4.resize (yyvs4, yyvsc4)
		end
	end
	yyspecial_routines4.force (yyvs4, yyval4, yyvsp4)
end
when 17 then
--|#line 183 ".\rose_recipe_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file '.\rose_recipe_parser.y' at line 183")
end

create {ROSE_TRANSFORMATION_PEER} yyval4.make (yyvs3.item (yyvsp3 - 1), yyvs3.item (yyvsp3), yyvs10.item (yyvsp10)) 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 8
	yyvsp4 := yyvsp4 + 1
	yyvsp1 := yyvsp1 -5
	yyvsp3 := yyvsp3 -2
	yyvsp10 := yyvsp10 -1
	if yyvsp4 >= yyvsc4 then
		if yyvs4 = Void then
			debug ("GEYACC")
				std.error.put_line ("Create yyvs4")
			end
			create yyspecial_routines4
			yyvsc4 := yyInitial_yyvs_size
			yyvs4 := yyspecial_routines4.make (yyvsc4)
		else
			debug ("GEYACC")
				std.error.put_line ("Resize yyvs4")
			end
			yyvsc4 := yyvsc4 + yyInitial_yyvs_size
			yyvs4 := yyspecial_routines4.resize (yyvs4, yyvsc4)
		end
	end
	yyspecial_routines4.force (yyvs4, yyval4, yyvsp4)
end
when 18 then
--|#line 188 ".\rose_recipe_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file '.\rose_recipe_parser.y' at line 188")
end

create {ROSE_TRANSFORMATION_ATTRIBUTE} yyval4.make (yyvs3.item (yyvsp3 - 3), yyvs3.item (yyvsp3 - 2), yyvs10.item (yyvsp10), yyvs3.item (yyvsp3 - 1), yyvs3.item (yyvsp3)) 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 12
	yyvsp4 := yyvsp4 + 1
	yyvsp1 := yyvsp1 -7
	yyvsp3 := yyvsp3 -4
	yyvsp10 := yyvsp10 -1
	if yyvsp4 >= yyvsc4 then
		if yyvs4 = Void then
			debug ("GEYACC")
				std.error.put_line ("Create yyvs4")
			end
			create yyspecial_routines4
			yyvsc4 := yyInitial_yyvs_size
			yyvs4 := yyspecial_routines4.make (yyvsc4)
		else
			debug ("GEYACC")
				std.error.put_line ("Resize yyvs4")
			end
			yyvsc4 := yyvsc4 + yyInitial_yyvs_size
			yyvs4 := yyspecial_routines4.resize (yyvs4, yyvsc4)
		end
	end
	yyspecial_routines4.force (yyvs4, yyval4, yyvsp4)
end
when 19 then
--|#line 193 ".\rose_recipe_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file '.\rose_recipe_parser.y' at line 193")
end

create yyval5.make (yyvs6.item (yyvsp6), yyvs7.item (yyvsp7), yyvs8.item (yyvsp8)) 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp5 := yyvsp5 + 1
	yyvsp6 := yyvsp6 -1
	yyvsp7 := yyvsp7 -1
	yyvsp8 := yyvsp8 -1
	if yyvsp5 >= yyvsc5 then
		if yyvs5 = Void then
			debug ("GEYACC")
				std.error.put_line ("Create yyvs5")
			end
			create yyspecial_routines5
			yyvsc5 := yyInitial_yyvs_size
			yyvs5 := yyspecial_routines5.make (yyvsc5)
		else
			debug ("GEYACC")
				std.error.put_line ("Resize yyvs5")
			end
			yyvsc5 := yyvsc5 + yyInitial_yyvs_size
			yyvs5 := yyspecial_routines5.resize (yyvs5, yyvsc5)
		end
	end
	yyspecial_routines5.force (yyvs5, yyval5, yyvsp5)
end
when 20 then
--|#line 198 ".\rose_recipe_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file '.\rose_recipe_parser.y' at line 198")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 0
	yyvsp6 := yyvsp6 + 1
	if yyvsp6 >= yyvsc6 then
		if yyvs6 = Void then
			debug ("GEYACC")
				std.error.put_line ("Create yyvs6")
			end
			create yyspecial_routines6
			yyvsc6 := yyInitial_yyvs_size
			yyvs6 := yyspecial_routines6.make (yyvsc6)
		else
			debug ("GEYACC")
				std.error.put_line ("Resize yyvs6")
			end
			yyvsc6 := yyvsc6 + yyInitial_yyvs_size
			yyvs6 := yyspecial_routines6.resize (yyvs6, yyvsc6)
		end
	end
	yyspecial_routines6.force (yyvs6, yyval6, yyvsp6)
end
when 21 then
--|#line 199 ".\rose_recipe_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file '.\rose_recipe_parser.y' at line 199")
end

create yyval6.make_from_sequence (yyvs9.item (yyvsp9)) 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp6 := yyvsp6 + 1
	yyvsp1 := yyvsp1 -1
	yyvsp9 := yyvsp9 -1
	if yyvsp6 >= yyvsc6 then
		if yyvs6 = Void then
			debug ("GEYACC")
				std.error.put_line ("Create yyvs6")
			end
			create yyspecial_routines6
			yyvsc6 := yyInitial_yyvs_size
			yyvs6 := yyspecial_routines6.make (yyvsc6)
		else
			debug ("GEYACC")
				std.error.put_line ("Resize yyvs6")
			end
			yyvsc6 := yyvsc6 + yyInitial_yyvs_size
			yyvs6 := yyspecial_routines6.resize (yyvs6, yyvsc6)
		end
	end
	yyspecial_routines6.force (yyvs6, yyval6, yyvsp6)
end
when 22 then
--|#line 202 ".\rose_recipe_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file '.\rose_recipe_parser.y' at line 202")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 0
	yyvsp7 := yyvsp7 + 1
	if yyvsp7 >= yyvsc7 then
		if yyvs7 = Void then
			debug ("GEYACC")
				std.error.put_line ("Create yyvs7")
			end
			create yyspecial_routines7
			yyvsc7 := yyInitial_yyvs_size
			yyvs7 := yyspecial_routines7.make (yyvsc7)
		else
			debug ("GEYACC")
				std.error.put_line ("Resize yyvs7")
			end
			yyvsc7 := yyvsc7 + yyInitial_yyvs_size
			yyvs7 := yyspecial_routines7.resize (yyvs7, yyvsc7)
		end
	end
	yyspecial_routines7.force (yyvs7, yyval7, yyvsp7)
end
when 23 then
--|#line 203 ".\rose_recipe_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file '.\rose_recipe_parser.y' at line 203")
end

create yyval7.make_ask 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp7 := yyvsp7 + 1
	yyvsp1 := yyvsp1 -2
	if yyvsp7 >= yyvsc7 then
		if yyvs7 = Void then
			debug ("GEYACC")
				std.error.put_line ("Create yyvs7")
			end
			create yyspecial_routines7
			yyvsc7 := yyInitial_yyvs_size
			yyvs7 := yyspecial_routines7.make (yyvsc7)
		else
			debug ("GEYACC")
				std.error.put_line ("Resize yyvs7")
			end
			yyvsc7 := yyvsc7 + yyInitial_yyvs_size
			yyvs7 := yyspecial_routines7.resize (yyvs7, yyvsc7)
		end
	end
	yyspecial_routines7.force (yyvs7, yyval7, yyvsp7)
end
when 24 then
--|#line 204 ".\rose_recipe_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file '.\rose_recipe_parser.y' at line 204")
end

create yyval7.make_from_sequence (yyvs11.item (yyvsp11)) 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp7 := yyvsp7 + 1
	yyvsp1 := yyvsp1 -1
	yyvsp11 := yyvsp11 -1
	if yyvsp7 >= yyvsc7 then
		if yyvs7 = Void then
			debug ("GEYACC")
				std.error.put_line ("Create yyvs7")
			end
			create yyspecial_routines7
			yyvsc7 := yyInitial_yyvs_size
			yyvs7 := yyspecial_routines7.make (yyvsc7)
		else
			debug ("GEYACC")
				std.error.put_line ("Resize yyvs7")
			end
			yyvsc7 := yyvsc7 + yyInitial_yyvs_size
			yyvs7 := yyspecial_routines7.resize (yyvs7, yyvsc7)
		end
	end
	yyspecial_routines7.force (yyvs7, yyval7, yyvsp7)
end
when 25 then
--|#line 207 ".\rose_recipe_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file '.\rose_recipe_parser.y' at line 207")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 0
	yyvsp8 := yyvsp8 + 1
	if yyvsp8 >= yyvsc8 then
		if yyvs8 = Void then
			debug ("GEYACC")
				std.error.put_line ("Create yyvs8")
			end
			create yyspecial_routines8
			yyvsc8 := yyInitial_yyvs_size
			yyvs8 := yyspecial_routines8.make (yyvsc8)
		else
			debug ("GEYACC")
				std.error.put_line ("Resize yyvs8")
			end
			yyvsc8 := yyvsc8 + yyInitial_yyvs_size
			yyvs8 := yyspecial_routines8.resize (yyvs8, yyvsc8)
		end
	end
	yyspecial_routines8.force (yyvs8, yyval8, yyvsp8)
end
when 26 then
--|#line 208 ".\rose_recipe_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file '.\rose_recipe_parser.y' at line 208")
end

create yyval8.make (yyvs12.item (yyvsp12 - 1), yyvs12.item (yyvsp12)) 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 4
	yyvsp8 := yyvsp8 + 1
	yyvsp1 := yyvsp1 -2
	yyvsp12 := yyvsp12 -2
	if yyvsp8 >= yyvsc8 then
		if yyvs8 = Void then
			debug ("GEYACC")
				std.error.put_line ("Create yyvs8")
			end
			create yyspecial_routines8
			yyvsc8 := yyInitial_yyvs_size
			yyvs8 := yyspecial_routines8.make (yyvsc8)
		else
			debug ("GEYACC")
				std.error.put_line ("Resize yyvs8")
			end
			yyvsc8 := yyvsc8 + yyInitial_yyvs_size
			yyvs8 := yyspecial_routines8.resize (yyvs8, yyvsc8)
		end
	end
	yyspecial_routines8.force (yyvs8, yyval8, yyvsp8)
end
when 27 then
--|#line 211 ".\rose_recipe_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file '.\rose_recipe_parser.y' at line 211")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 0
	yyvsp9 := yyvsp9 + 1
	if yyvsp9 >= yyvsc9 then
		if yyvs9 = Void then
			debug ("GEYACC")
				std.error.put_line ("Create yyvs9")
			end
			create yyspecial_routines9
			yyvsc9 := yyInitial_yyvs_size
			yyvs9 := yyspecial_routines9.make (yyvsc9)
		else
			debug ("GEYACC")
				std.error.put_line ("Resize yyvs9")
			end
			yyvsc9 := yyvsc9 + yyInitial_yyvs_size
			yyvs9 := yyspecial_routines9.resize (yyvs9, yyvsc9)
		end
	end
	yyspecial_routines9.force (yyvs9, yyval9, yyvsp9)
end
when 28 then
--|#line 212 ".\rose_recipe_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file '.\rose_recipe_parser.y' at line 212")
end

create yyval9.make_empty; yyval9.extend (yyvs3.item (yyvsp3)) 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp9 := yyvsp9 + 1
	yyvsp3 := yyvsp3 -1
	if yyvsp9 >= yyvsc9 then
		if yyvs9 = Void then
			debug ("GEYACC")
				std.error.put_line ("Create yyvs9")
			end
			create yyspecial_routines9
			yyvsc9 := yyInitial_yyvs_size
			yyvs9 := yyspecial_routines9.make (yyvsc9)
		else
			debug ("GEYACC")
				std.error.put_line ("Resize yyvs9")
			end
			yyvsc9 := yyvsc9 + yyInitial_yyvs_size
			yyvs9 := yyspecial_routines9.resize (yyvs9, yyvsc9)
		end
	end
	yyspecial_routines9.force (yyvs9, yyval9, yyvsp9)
end
when 29 then
--|#line 213 ".\rose_recipe_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file '.\rose_recipe_parser.y' at line 213")
end

yyval9 := yyvs9.item (yyvsp9); yyval9.extend (yyvs3.item (yyvsp3)) 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp1 := yyvsp1 -1
	yyvsp3 := yyvsp3 -1
	yyspecial_routines9.force (yyvs9, yyval9, yyvsp9)
end
when 30 then
--|#line 216 ".\rose_recipe_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file '.\rose_recipe_parser.y' at line 216")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 0
	yyvsp11 := yyvsp11 + 1
	if yyvsp11 >= yyvsc11 then
		if yyvs11 = Void then
			debug ("GEYACC")
				std.error.put_line ("Create yyvs11")
			end
			create yyspecial_routines11
			yyvsc11 := yyInitial_yyvs_size
			yyvs11 := yyspecial_routines11.make (yyvsc11)
		else
			debug ("GEYACC")
				std.error.put_line ("Resize yyvs11")
			end
			yyvsc11 := yyvsc11 + yyInitial_yyvs_size
			yyvs11 := yyspecial_routines11.resize (yyvs11, yyvsc11)
		end
	end
	yyspecial_routines11.force (yyvs11, yyval11, yyvsp11)
end
when 31 then
--|#line 217 ".\rose_recipe_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file '.\rose_recipe_parser.y' at line 217")
end

create yyval11.make_empty; yyval11.extend (yyvs10.item (yyvsp10)) 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp11 := yyvsp11 + 1
	yyvsp10 := yyvsp10 -1
	if yyvsp11 >= yyvsc11 then
		if yyvs11 = Void then
			debug ("GEYACC")
				std.error.put_line ("Create yyvs11")
			end
			create yyspecial_routines11
			yyvsc11 := yyInitial_yyvs_size
			yyvs11 := yyspecial_routines11.make (yyvsc11)
		else
			debug ("GEYACC")
				std.error.put_line ("Resize yyvs11")
			end
			yyvsc11 := yyvsc11 + yyInitial_yyvs_size
			yyvs11 := yyspecial_routines11.resize (yyvs11, yyvsc11)
		end
	end
	yyspecial_routines11.force (yyvs11, yyval11, yyvsp11)
end
when 32 then
--|#line 218 ".\rose_recipe_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file '.\rose_recipe_parser.y' at line 218")
end

yyval11 := yyvs11.item (yyvsp11); yyval11.extend (yyvs10.item (yyvsp10)) 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp1 := yyvsp1 -1
	yyvsp10 := yyvsp10 -1
	yyspecial_routines11.force (yyvs11, yyval11, yyvsp11)
end
			else
				debug ("GEYACC")
					std.error.put_string ("Error in parser: unknown rule id: ")
					std.error.put_integer (yy_act)
					std.error.put_new_line
				end
				abort
			end
		end

	yy_do_error_action (yy_act: INTEGER)
			-- Execute error action.
		do
			inspect yy_act
			when 77 then
					-- End-of-file expected action.
				report_eof_expected_error
			else
					-- Default action.
				report_error ("parse error")
			end
		end

feature {NONE} -- Table templates

	yytranslate_template: SPECIAL [INTEGER]
			-- Template for `yytranslate'
		once
			Result := yyfixed_array (<<
			    0,    2,    2,    2,    2,    2,    2,    2,    2,    2,
			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
			   19,   21,    2,    2,   20,    2,    2,    2,    2,    2,
			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,

			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,

			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
			    2,    2,    2,    2,    2,    2,    1,    2,    3,    4,
			    5,    6,    7,    8,    9,   10,   11,   12,   13,   14,
			   15,   16,   17,   18, yyDummy>>)
		end

	yyr1_template: SPECIAL [INTEGER]
			-- Template for `yyr1'
		once
			Result := yyfixed_array (<<
			    0,   37,   37,   37,   22,   23,   23,   24,   24,   24,
			   25,   26,   26,   26,   26,   27,   28,   29,   30,   31,
			   32,   32,   33,   33,   33,   34,   34,   35,   35,   35,
			   36,   36,   36, yyDummy>>)
		end

	yytypes1_template: SPECIAL [INTEGER]
			-- Template for `yytypes1'
		once
			Result := yyfixed_array (<<
			    1,    1,    2,    1,    3,    2,    1,    3,    3,    1,
			    5,    6,    3,    9,    1,    1,    7,    1,    1,    1,
			    1,    1,    4,    4,    4,    4,    4,    4,    4,    1,
			   10,   11,    1,    8,    3,    1,    1,    1,    1,    4,
			    5,    1,   12,    3,    3,    3,    3,   10,    1,    1,
			    1,    1,    1,   12,    3,    3,    3,    3,    1,    1,
			    1,    1,   10,   10,   10,   10,    1,    1,    1,    1,
			    3,    3,    3,    1,    1,    1,    3,    1,    1,    1, yyDummy>>)
		end

	yytypes2_template: SPECIAL [INTEGER]
			-- Template for `yytypes2'
		once
			Result := yyfixed_array (<<
			    1,    1,    1,   10,   12,    3,    1,    1,    1,    1,
			    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
			    1,    1, yyDummy>>)
		end

	yydefact_template: SPECIAL [INTEGER]
			-- Template for `yydefact'
		once
			Result := yyfixed_array (<<
			    1,    0,    2,    0,    5,    3,    0,   20,    6,   27,
			    0,   22,   28,   21,    7,   30,   25,    0,    0,    0,
			    0,    0,    4,    8,   20,   11,   12,   13,   14,   23,
			   31,   24,    0,   19,   29,    0,    0,    0,    0,    9,
			   10,    0,    0,    0,    0,    0,    0,   32,    0,    0,
			    0,    0,    0,   26,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,   17,    0,    0,
			    0,    0,    0,    0,   16,   15,    0,   18,    0,    0, yyDummy>>)
		end

	yydefgoto_template: SPECIAL [INTEGER]
			-- Template for `yydefgoto'
		once
			Result := yyfixed_array (<<
			    2,    7,   22,   23,   24,   25,   26,   27,   28,   10,
			   11,   16,   33,   13,   31,    3, yyDummy>>)
		end

	yypact_template: SPECIAL [INTEGER]
			-- Template for `yypact'
		once
			Result := yyfixed_array (<<
			   55,   53, -32768,    1,   52, -32768,   51,   31, -32768,   50,
			   46,   43, -32768,   40,  -12,   -3,   14,   45,   38,   33,
			   32,   30,  -12, -32768,   31, -32768, -32768, -32768, -32768, -32768,
			 -32768,   28,   41, -32768, -32768,   39,   37,   36,   34, -32768,
			 -32768,   35,   27,   23,   17,   16,   15, -32768,   22,   29,
			   26,   25,   24, -32768,   13,   12,    8,    7,   21,   20,
			   19,   18,    0,   -4,   -1,   -2,   11, -32768,   10,    9,
			   -7,   -9,  -10,    5, -32768, -32768,  -13, -32768,    6, -32768, yyDummy>>)
		end

	yypgoto_template: SPECIAL [INTEGER]
			-- Template for `yypgoto'
		once
			Result := yyfixed_array (<<
			   59, -32768, -32768,   47, -32768, -32768, -32768, -32768, -32768,   42,
			 -32768, -32768, -32768, -32768, -32768, -32768, yyDummy>>)
		end

	yytable_template: SPECIAL [INTEGER]
			-- Template for `yytable'
		once
			Result := yyfixed_array (<<
			   30,   78,   21,   20,   19,   18,   79,    1,   77,   29,
			   76,   75,   74,   73,   72,   71,   70,   67,   69,   68,
			   66,   65,   64,   63,   62,   32,   53,   61,   60,   57,
			   56,   55,   59,   58,   54,   52,   51,   50,   47,   46,
			    9,   45,   44,   49,   43,   42,    0,   48,   41,   38,
			   34,   37,   36,   15,   14,   12,    8,   35,    4,    6,
			   17,    1,    5,    0,    0,    0,   40,    0,    0,   39, yyDummy>>)
		end

	yycheck_template: SPECIAL [INTEGER]
			-- Template for `yycheck'
		once
			Result := yyfixed_array (<<
			    3,    0,   14,   15,   16,   17,    0,    6,   21,   12,
			    5,   21,   21,   20,    5,    5,    5,   21,   20,   20,
			   20,    3,    3,    3,    3,   11,    4,   20,   20,    5,
			    5,    5,   20,   20,    5,   20,   20,   20,    3,    5,
			    9,    5,    5,   20,    5,    4,   -1,   20,   20,   19,
			    5,   19,   19,   10,    8,    5,    5,   19,    5,    7,
			   20,    6,    3,   -1,   -1,   -1,   24,   -1,   -1,   22, yyDummy>>)
		end

feature {NONE} -- Semantic value stacks

	yyvs1: SPECIAL [ANY]
			-- Stack for semantic values of type ANY

	yyvsc1: INTEGER
			-- Capacity of semantic value stack `yyvs1'

	yyvsp1: INTEGER
			-- Top of semantic value stack `yyvs1'

	yyspecial_routines1: KL_SPECIAL_ROUTINES [ANY]
			-- Routines that ought to be in SPECIAL [ANY]

	yyvs2: SPECIAL [ROSE_RECIPE]
			-- Stack for semantic values of type ROSE_RECIPE

	yyvsc2: INTEGER
			-- Capacity of semantic value stack `yyvs2'

	yyvsp2: INTEGER
			-- Top of semantic value stack `yyvs2'

	yyspecial_routines2: KL_SPECIAL_ROUTINES [ROSE_RECIPE]
			-- Routines that ought to be in SPECIAL [ROSE_RECIPE]

	yyvs3: SPECIAL [STRING]
			-- Stack for semantic values of type STRING

	yyvsc3: INTEGER
			-- Capacity of semantic value stack `yyvs3'

	yyvsp3: INTEGER
			-- Top of semantic value stack `yyvs3'

	yyspecial_routines3: KL_SPECIAL_ROUTINES [STRING]
			-- Routines that ought to be in SPECIAL [STRING]

	yyvs4: SPECIAL [ROSE_TRANSFORMATION]
			-- Stack for semantic values of type ROSE_TRANSFORMATION

	yyvsc4: INTEGER
			-- Capacity of semantic value stack `yyvs4'

	yyvsp4: INTEGER
			-- Top of semantic value stack `yyvs4'

	yyspecial_routines4: KL_SPECIAL_ROUTINES [ROSE_TRANSFORMATION]
			-- Routines that ought to be in SPECIAL [ROSE_TRANSFORMATION]

	yyvs5: SPECIAL [ROSE_TRANSFORMATION_SETTINGS]
			-- Stack for semantic values of type ROSE_TRANSFORMATION_SETTINGS

	yyvsc5: INTEGER
			-- Capacity of semantic value stack `yyvs5'

	yyvsp5: INTEGER
			-- Top of semantic value stack `yyvs5'

	yyspecial_routines5: KL_SPECIAL_ROUTINES [ROSE_TRANSFORMATION_SETTINGS]
			-- Routines that ought to be in SPECIAL [ROSE_TRANSFORMATION_SETTINGS]

	yyvs6: SPECIAL [ROSE_TRANSFORMATION_SETTING_DISPLAY]
			-- Stack for semantic values of type ROSE_TRANSFORMATION_SETTING_DISPLAY

	yyvsc6: INTEGER
			-- Capacity of semantic value stack `yyvs6'

	yyvsp6: INTEGER
			-- Top of semantic value stack `yyvs6'

	yyspecial_routines6: KL_SPECIAL_ROUTINES [ROSE_TRANSFORMATION_SETTING_DISPLAY]
			-- Routines that ought to be in SPECIAL [ROSE_TRANSFORMATION_SETTING_DISPLAY]

	yyvs7: SPECIAL [ROSE_TRANSFORMATION_SETTING_FILTER]
			-- Stack for semantic values of type ROSE_TRANSFORMATION_SETTING_FILTER

	yyvsc7: INTEGER
			-- Capacity of semantic value stack `yyvs7'

	yyvsp7: INTEGER
			-- Top of semantic value stack `yyvs7'

	yyspecial_routines7: KL_SPECIAL_ROUTINES [ROSE_TRANSFORMATION_SETTING_FILTER]
			-- Routines that ought to be in SPECIAL [ROSE_TRANSFORMATION_SETTING_FILTER]

	yyvs8: SPECIAL [ROSE_TRANSFORMATION_SETTING_CONSTRAINT]
			-- Stack for semantic values of type ROSE_TRANSFORMATION_SETTING_CONSTRAINT

	yyvsc8: INTEGER
			-- Capacity of semantic value stack `yyvs8'

	yyvsp8: INTEGER
			-- Top of semantic value stack `yyvs8'

	yyspecial_routines8: KL_SPECIAL_ROUTINES [ROSE_TRANSFORMATION_SETTING_CONSTRAINT]
			-- Routines that ought to be in SPECIAL [ROSE_TRANSFORMATION_SETTING_CONSTRAINT]

	yyvs9: SPECIAL [ROSE_LINEAR_ARRAY [STRING]]
			-- Stack for semantic values of type ROSE_LINEAR_ARRAY [STRING]

	yyvsc9: INTEGER
			-- Capacity of semantic value stack `yyvs9'

	yyvsp9: INTEGER
			-- Top of semantic value stack `yyvs9'

	yyspecial_routines9: KL_SPECIAL_ROUTINES [ROSE_LINEAR_ARRAY [STRING]]
			-- Routines that ought to be in SPECIAL [ROSE_LINEAR_ARRAY [STRING]]

	yyvs10: SPECIAL [INTEGER]
			-- Stack for semantic values of type INTEGER

	yyvsc10: INTEGER
			-- Capacity of semantic value stack `yyvs10'

	yyvsp10: INTEGER
			-- Top of semantic value stack `yyvs10'

	yyspecial_routines10: KL_SPECIAL_ROUTINES [INTEGER]
			-- Routines that ought to be in SPECIAL [INTEGER]

	yyvs11: SPECIAL [ROSE_LINEAR_ARRAY [INTEGER]]
			-- Stack for semantic values of type ROSE_LINEAR_ARRAY [INTEGER]

	yyvsc11: INTEGER
			-- Capacity of semantic value stack `yyvs11'

	yyvsp11: INTEGER
			-- Top of semantic value stack `yyvs11'

	yyspecial_routines11: KL_SPECIAL_ROUTINES [ROSE_LINEAR_ARRAY [INTEGER]]
			-- Routines that ought to be in SPECIAL [ROSE_LINEAR_ARRAY [INTEGER]]

	yyvs12: SPECIAL [ROSE_DATE]
			-- Stack for semantic values of type ROSE_DATE

	yyvsc12: INTEGER
			-- Capacity of semantic value stack `yyvs12'

	yyvsp12: INTEGER
			-- Top of semantic value stack `yyvs12'

	yyspecial_routines12: KL_SPECIAL_ROUTINES [ROSE_DATE]
			-- Routines that ought to be in SPECIAL [ROSE_DATE]

feature {NONE} -- Constants

	yyFinal: INTEGER = 79
			-- Termination state id

	yyFlag: INTEGER = -32768
			-- Most negative INTEGER

	yyNtbase: INTEGER = 22
			-- Number of tokens

	yyLast: INTEGER = 69
			-- Upper bound of `yytable' and `yycheck'

	yyMax_token: INTEGER = 273
			-- Maximum token id
			-- (upper bound of `yytranslate'.)

	yyNsyms: INTEGER = 38
			-- Number of symbols
			-- (terminal and nonterminal)

feature -- User-defined features



feature -- Error handling

	report_error (a_message: STRING) is
			-- Print error message.
		do
			recipe := Void
			error.signal (id, "$1 [near '$2', line $3 token '$4']", <<a_message, text, line, token_name (last_token)>>)
		end

feature {NONE} -- Implementation

	reset_recipe_parser is
			-- Reset the parser.
		do
			reset
			recipe := Void
			create recipe_list.make_empty
		end

	buffer_name: STRING
			-- Name of analyzed buffer

	set_recipe (a_recipe: ROSE_RECIPE) is
			-- Set `recipe' to `a_recipe' and add it to `recipe_list'.
		do
			recipe := a_recipe
			recipe_list.extend (a_recipe)
		end

end
