note

	description:

		"Objects which can set and get values from objects by attribute name"

	author: "Mark Howard"
	last_editor: "$Author: it $"
	date: "$Date: 2014/10/29 $"
	revision: "$Revision: #19 $"
	path: "$File: //rose/source/kernel/support/rose_record_access.e $"

deferred class
	ROSE_RECORD_ACCESS

inherit
	ROSE_CONSTANT_VALUES

feature -- Access

	name: STRING is
			-- Expression name
		deferred
		end

	is_available:BOOLEAN
			--- Is Current available?

	current_record_type: INTEGER
	record_index: INTEGER
	record_field_type: INTEGER
	data_value: DOUBLE
	last_string: STRING

	flat_name: STRING
			-- Flat name
		do
			Result := name
		end
	descriptor (a_record: ANY) : TUPLE[index,type : INTEGER] is
			-- Map the attributes of 'a_record'
		require
			valid_record: a_record /= Void
		local
			l_found: BOOLEAN
			l_field_name: STRING
			l_type,l_field_type: INTEGER
			l_name: STRING
			i,l_index: INTEGER
		do
			if maps = Void then
				create maps.make (5)
			end
			l_type := internal.dynamic_type (a_record)
			Result := maps.item (l_type)
			if Result = Void then
				l_index := -1
				l_name := flat_name.as_lower
				l_name.prune_all_trailing (' ')
				if l_name.has ('(') then --prune operators
					l_name.replace_substring_all ("(", "_")
					l_name.replace_substring_all (")", "_")
				end
				from i := 1 until i > internal.field_count (a_record) or else l_found loop
					l_field_name := internal.field_name (i, a_record).as_lower
					l_found := l_field_name.is_equal (l_name)
					if l_found then
						l_index := i
						l_field_type := internal.field_type (i, a_record)
					end
					i := i + 1
				end
				if l_index = -1 then
					l_name := flat_name.as_lower
					l_name.prune_all_trailing (' ')
					if l_name.has ('(') then --prune operators
						l_name := l_name.substring (l_name.index_of ('(', 1) + 1, l_name.index_of (')', 1)-1)
					end
					from i := 1 until i > internal.field_count (a_record) or else l_found loop
						l_field_name := internal.field_name (i, a_record).as_lower
						l_found := l_field_name.is_equal (l_name)
						if l_found then
							l_index := i
							l_field_type := internal.field_type (i, a_record)
						end
						i := i + 1
					end
				end
				Result := [l_index,l_field_type]
				maps.force (Result, l_type)
			end
		end
	map_record (a_record: ANY) is
			-- Map the attributes of 'a_record'
		require
			valid_record: a_record /= Void
		local
			l_descriptor : TUPLE[index,type : INTEGER]
			l_type : INTEGER
		do
			l_type := internal.dynamic_type (a_record)
			if l_type /= current_record_type then
				current_record_type := l_type
				l_descriptor := descriptor(a_record)
				record_index := l_descriptor.index
				record_field_type := l_descriptor.type
			end
		end

	maps: HASH_TABLE [TUPLE[index,type : INTEGER], INTEGER] note option : transient attribute end

	contained_in_record (a_record: ANY): BOOLEAN is
			-- Is this expression contained in 'a_record'?
		require
			valid_record: a_record /= Void
		do
			map_record (a_record)
			Result := record_index > 0
			if not Result and then attached {DYNAMIC_RECORD} a_record as l_dynamic then
				Result := l_dynamic.dynamic_values.has (name)
			end
		end

	keyed_value_from_record(a_record : ANY;a_key : STRING) : DATED_UNITIZED_VALUE[DOUBLE]
			-- Create dated_unitized value directly from item of associative array matching attribute in a_record
		do
			map_record (a_record)
			if record_index > 0 then
				if {l_keyed : TABLE[DATED_UNITIZED_VALUE[DOUBLE],STRING]} internal.field (record_index, a_record) then
					Result := l_keyed[a_key]
				end
			end
		end

	unitized_value_from_record(a_record : ANY) : ROSE_UNITIZED_VALUE[DOUBLE]
			-- Create unitized value directly from matching attribute in a_record
		do
			map_record (a_record)
			if record_index > 0 and then {l_unitized : ROSE_UNITIZED_VALUE[DOUBLE]} internal.field (record_index, a_record) then
				Result := l_unitized
			elseif attached {DYNAMIC_RECORD} a_record as l_dynamic then
				Result := l_dynamic.unitized_value(name)
			end
		end
	dated_unitized_value_from_record(a_record : ANY) : DATED_UNITIZED_VALUE[DOUBLE]
			-- Create unitized value directly from matching attribute in a_record
		do
			map_record (a_record)
			if record_index > 0 then
				if {l_unitized : DATED_UNITIZED_VALUE[DOUBLE]} internal.field (record_index, a_record) then
					Result := l_unitized
				elseif {l_double : DOUBLE} internal.field (record_index, a_record) then
					create Result.make_with_units (l_double, Void)
				end
				if Result = Void then
					create Result.make (missing_double, Void, Void, Void)
				end
			elseif attached {DYNAMIC_RECORD} a_record as l_dynamic then
				Result := l_dynamic.unitized_value(name)
			end
		end

	dated_unitized_integer_from_record(a_record : ANY) : DATED_UNITIZED_VALUE[INTEGER]
			-- Create unitized value directly from matching attribute in a_record
		do
			map_record (a_record)
			if record_index > 0 then
				if {l_unitized : DATED_UNITIZED_VALUE[INTEGER]} internal.field (record_index, a_record) then
					Result := l_unitized
				elseif {l_integer : INTEGER} internal.field (record_index, a_record) then
					create Result.make_with_units (l_integer, Void)
				end
			end
		end
	integer_value_from_record (a_record: ANY): INTEGER is
			--extract integer value for attribute from record with this name
		do
			map_record (a_record)
			if record_index > 0 then
				Result := internal.integer_field (record_index, a_record)
			else
				Result := null_integer
			end
		end

	string_value_from_record (a_record: ANY): STRING is
			--string value by name from record ... very dangerous
		do
			map_record (a_record)
			if record_index > 0 then
				Result ?= internal.field (record_index, a_record)
			end
		end

	double_value_from_record (a_record: ANY): DOUBLE is
			-- double attribute value with name like Current in  a_record
		require
			valid_record: a_record /= Void
		local
			l_type: INTEGER
			l_any : ANY
			l_value,l_integer : INTEGER
			l_double : DOUBLE
			l_real : REAL

		do
			map_record (a_record)
			if record_index > 0 then
				l_any := internal.field (record_index, a_record)
				if l_any = Void then
					Result:= missing_double
				else
					l_type:= internal.dynamic_type (l_any)
					Result := missing_double
					if l_any.same_type (l_double) or else l_any.same_type (double_ref) then
						Result := internal.double_field (record_index, a_record)
					elseif l_any.same_type (l_real) or else l_any.same_type (real_ref) then
						Result := internal.real_field (record_index, a_record)
					elseif l_any.same_type (l_integer) or else l_any.same_type (integer_ref) then
						l_value := internal.integer_field (record_index, a_record)
						if l_value /= missing_integer then
							Result := l_value
						end
					elseif l_type = rose_unitized_value_type then
						if attached {ROSE_UNITIZED_VALUE[DOUBLE]} internal.field (record_index, a_record) as l_ruv then
							Result := l_ruv.value
						end
					elseif l_type = dated_unitized_value_type or else l_type = dated_unitized_integer_type then
						if attached {DATED_UNITIZED_VALUE[DOUBLE]} internal.field (record_index, a_record) as l_duv then
							Result := l_duv.value
						elseif attached {DATED_UNITIZED_VALUE[INTEGER]} internal.field (record_index, a_record) as l_duv then
							Result := l_duv.value
						end
					else
						(create {EXCEPTIONS}).raise ("invalid type")
					end
				end
			else
				Result:= missing_double
			end
		end

	set_record_unitized_value (a_value: DATED_UNITIZED_VALUE [DOUBLE]; a_record: like record_template) is
			-- Set record unitized value
		do
			map_record (a_record)
			if record_index > 0 then
				internal.set_reference_field (record_index, a_record, a_value)
			elseif attached {DYNAMIC_RECORD} a_record as l_dynamic then
				l_dynamic.dynamic_values.force (a_value, name)
			end
		end

	set_record_double_value (a_value: DOUBLE; a_record: like record_template) is
			--Set the value of the attribute with name 'name' in 'a_record' to 'a_value'
		do
			map_record (a_record)
			if record_index > 0 then
				internal.set_double_field (record_index, a_record, a_value)
			end
		end

	set_record_date_value(a_value: ROSE_DATE; a_record: like record_template)
			-- Plunk 'a_value' into the field in 'a_record' whose name matches the name of this expression.
		do
			map_record(a_record)
			if record_index > 0 then
				internal.set_reference_field(record_index, a_record, a_value)
			end
		end

	record_template: ANY is
			-- Template for record types
		do
		end

	set_record_real_value (a_value: REAL;a_record: ANY) is
			--set the attribute with the same name as 'name' in 'a_record' to 'a_value'
		do
			map_record (a_record)
			if record_index > 0 then
				internal.set_real_field (record_index, a_record, a_value)
			end
		end

	set_record_integer_value (a_value: INTEGER;a_record: ANY) is
			--set the attribute with the same name as 'name' in 'a_record' to 'a_value'
		do
			map_record (a_record)
			if record_index > 0 then
				internal.set_integer_field (record_index, a_record, a_value)
			end
		end

	set_record_character_value (a_value: CHARACTER;a_record: ANY)
			--set the attribute with the same name as 'name' in 'a_record' to 'a_value'
		do
			map_record (a_record)
			if record_index > 0 then
				internal.set_character_field (record_index, a_record, a_value)
			end
		end

	set_record_boolean_value (a_value: BOOLEAN;a_record: ANY) is
			--set the attribute with the same name as 'name' in 'a_record' to 'a_value'
		do
			map_record (a_record)
			if record_index > 0 then
				internal.set_boolean_field (record_index, a_record, a_value)
			end
		end

	set_record_string_value (a_value: STRING;a_record: ANY) is
			--set the attribute with the same name as 'name' in 'a_record' to 'a_value'
		do
			map_record (a_record)
			if record_index > 0 then
				internal.set_reference_field (record_index, a_record, a_value)
			end
		end

	set_record_value (a_record: like record_template) is
			--set value of a_record, by name from the current data_value ... very dangerous
		do
			inspect record_field_type
			when string_type then
				if is_available then
					set_record_string_value (last_string, a_record)
				else
					set_record_string_value (missing_string, a_record)
				end
			else
				set_record_any_value (a_record, data_value, Void)
			end
		end

	set_record_any_value (a_record: like record_template; a_value: ANY;a_date: ROSE_DATE) is
			-- Set the attribute with the same name as 'name' in 'a_record' to 'a_value'
		do
			map_record (a_record)
			if record_index > 0 then
				if attached {DOUBLE} a_value as l_value then
					set_record_from_double_value (a_record, l_value)
				elseif attached {ROSE_UNITIZED_VALUE [DOUBLE]} a_value as l_unitized_value then
					if record_field_type = reference_type then
						if internal.field_static_type_of_type(record_index, internal.dynamic_type(a_record))=rose_date_type
						then
							if is_finite_double(l_unitized_value.value)
							then
								set_record_date_value(create {ROSE_DATE}.make_ccyymmdd_frac(l_unitized_value.value), a_record)
							else
								set_record_unitized_value(Void, a_record)
							end
						else
							if not {l_dated_unitized_value: DATED_UNITIZED_VALUE [DOUBLE]} a_value then
								set_record_unitized_value (create {DATED_UNITIZED_VALUE [DOUBLE]}.make (l_unitized_value.value, l_unitized_value.units, a_date, a_date), a_record)
							else
								set_record_unitized_value (l_dated_unitized_value, a_record)
							end
						end
					else
						set_record_from_double_value (a_record, l_unitized_value.value)
					end
				elseif a_value = Void and then record_field_type = double_type then
					set_record_from_double_value (a_record, missing_double)
				elseif attached {INTEGER} a_value as l_value then
					set_record_from_double_value (a_record, l_value)
				end
			elseif attached {ROSE_UNITIZED_VALUE [DOUBLE]} a_value as l_unitized then
				set_record_unitized_value (create {DATED_UNITIZED_VALUE [DOUBLE]}.make_from_rose_unitized (l_unitized), a_record)
			end
		end

	copy_data_from_other_record(a_record, a_from_record: like record_template)
			-- Copy the variable represented by this expression (Current) from 'a_from_record' into 'a_record'.
		require
			record_not_void: a_record/=Void
			from_record_not_void: a_from_record/=Void
			same_type: internal.dynamic_type(a_record)=internal.dynamic_type(a_from_record)
		do
			map_record(a_record)

			if record_index>0
			 then
				if record_field_type=reference_type
				 then
					internal.set_reference_field(record_index, a_record, internal.field(record_index, a_from_record))
				else
					set_record_from_double_value(a_record, double_value_from_record(a_from_record))
				end
			end
		end

	verify (a_record: like record_template) is
			-- Prints a message if this variable is not an attribute in 'a_record'.
		do
			map_record (a_record)
			if record_index<=0 then
				print ("Could not find the attribute called " + name + ".%N")
			end
		end

	set_record_from_double_value (a_record: like record_template; a_value: DOUBLE) is
			-- Set 'a_record' with 'a_value', depending on 'record_field_type'
		do
			inspect record_field_type
			when double_type then
				if defined_double (a_value) then
					set_record_double_value (a_value, a_record)
				else
					set_record_double_value (missing_double, a_record)
				end
			when integer_type then
				if defined_double (a_value) then
					set_record_integer_value (a_value.rounded, a_record)
				else
					set_record_integer_value (null_integer, a_record)
				end
			when  boolean_type then
				if defined_double (a_value) then
					set_record_boolean_value (a_value /= 0, a_record)
				else
					set_record_boolean_value (False, a_record)
				end
			when character_type then
				-- ?
			when real_type then
				if defined_double (a_value) then
					set_record_real_value (a_value.truncated_to_real, a_record)
				else
					set_record_real_value (missing_real, a_record)
				end
			when string_type then
				set_record_string_value (a_value.out, a_record)
			else
				-- Do nothing.
			end
		end

feature {NONE} -- once functions

	rose_unitized_value_type: INTEGER
			-- The dynamic type id of ROSE_UNITIZED_VALUE[DOUBLE].
		once
			Result := internal.dynamic_type_from_string ("ROSE_UNITIZED_VALUE[DOUBLE]")
		end

	dated_unitized_value_type: INTEGER
			-- The dynamic type id of DATED_UNITIZED_VALUE[DOUBLE].
		once
			Result := internal.dynamic_type_from_string ("DATED_UNITIZED_VALUE[DOUBLE]")
		end

	dated_unitized_integer_type: INTEGER
			-- The dynamic type id of DATED_UNITIZED_VALUE[DOUBLE].
		once
			Result := internal.dynamic_type_from_string ("DATED_UNITIZED_VALUE[INTEGER]")
		end

	double_ref_type: INTEGER
			-- The dynamic type id of DOUBLE_REF.
		once
			Result := internal.dynamic_type_from_string ("DOUBLE_REF")
		end

	double_ref2_type: INTEGER
			-- Estudio 6.5 and above morphs DOUBLEs into types that aren't really expanded, nor really reference...
			-- They do not have the internal dynamic type id 'double_type', nor 'double_ref_type',
			-- They do have the dynamic type of 'DOUBLE'
		once
			Result := internal.dynamic_type_from_string ("DOUBLE")
		end

	rose_date_type: INTEGER
			-- The dynamic type of ROSE_DATE.
		once
			Result := internal.dynamic_type_from_string("ROSE_DATE")
		end

	double_ref : DOUBLE_REF
			-- Type instance for DOUBLE_REF
		once
			create Result
		end
	real_ref : REAL_REF
			-- Type instance for REAL_REF
		once
			create Result
		end
	integer_ref : INTEGER_REF
			-- Type instance for INTEGER_REF
		once
			create Result
		end

end
