indexing
	description: "Structure describing a registry key value"
	notice: "This was taken from WEL_REGISTRY, because WEL should not be needed just to access the registry..."
	status: "See notice at end of class";
	date: "$Date: 2009/03/16 $";
	revision: "$Revision: #4 $"

class
	ROSE_REGISTRY_KEY_VALUE

inherit
	ROSE_REGISTRY_KEY_VALUE_TYPE

	PLATFORM
		export
			{NONE} all
		end

create
	make,
	make_with_data,
	make_with_dword_value

feature -- Initialization

	make (v: STRING) is
			-- Create a new string key value.
		require
			v_not_void: v /= Void
		do
			set_string_value (v)
		ensure
			string_value_set: string_value.is_equal (v)
		end

	make_with_data (t: like type; v: MANAGED_POINTER) is
			-- Create a new key value using data `v'.
		require
			v_not_void: v /= Void
		do
			set_type (t)
			data := v
		ensure
			type_set: type = t
			data_set: data = v
		end

	make_with_dword_value (v: like dword_value) is
			-- Set `dword_value' with `v'.
			-- Set `type' with `reg_dword'.
		do
			set_dword_value (v)
		ensure
			type_set: type = reg_dword
			dword_value_set: dword_value = v
		end

feature -- Access

	type: INTEGER
			-- Type of value

	data: MANAGED_POINTER
			-- Hold data for registry key value

	value: STRING is
			-- String data.
		obsolete
			"Use `string_value' instead."
		require
			valid_type: type = Reg_sz
		do
			Result := string_value
		ensure
			string_value_not_void: Result /= Void
		end

	string_value: STRING is
			-- String data.
		require
			valid_type: type = reg_sz
		local
			s: C_STRING
		do
			create s.make_by_pointer_and_count (data.item, data.count)
			Result := s.string
		ensure
			string_value_not_void: Result /= Void
		end

	dword_value: INTEGER is
			-- Data converted as integer.
		require
			valid_type: type = Reg_dword
		do
			($Result).memory_copy (data.item, Integer_32_bytes)
		end

feature -- Element Change

	set_type (t: INTEGER) is
			-- Set type of value with `t'.
		do
			type := t
		ensure
			type_set: type = t
		end

	set_dword_value (v: like dword_value) is
			-- Set `dword_value' with `v'.
			-- Set `type' with `reg_dword'.
		do
			type := reg_dword
			create data.make (integer_32_bytes)
			data.put_integer_32 (v, 0)
		ensure
			type_set: type = reg_dword
			dword_value_set: dword_value = v
		end

	set_string_value (v: like string_value) is
			-- Set `string_value' with `v'.
			-- Set `type' with `reg_sz'.
		require
			v_not_void: v /= Void
		local
			l_str: C_STRING
		do
			type := reg_sz
			create l_str.make (v)
			data := l_str.managed_data
		ensure
			type_set: type = reg_sz
			dword_value_set: string_value.is_equal (v)
		end

end
