indexing
	description: "Interface for access to a windows registry section"
	author: "Zoran Simic"
	date: "$Date: 2008/12/01 $"
	revision: "$Revision: #5 $"

class
	ROSE_REGISTRY_SECTION

create
	make

feature {NONE} -- Initialization

	make (a_root: STRING) is
		do
			create implementation.make (a_root)
		end

feature -- Access

	is_readable: BOOLEAN is
			-- Is this section readable by current user?
		once
			Result := implementation.is_readable
		end

	is_modifiable: BOOLEAN is
			-- Is this section modifiable by current user?
		once
			Result := implementation.is_modifiable
		end

	get (a_key: STRING): STRING is
			-- Value associated to 'a_key' in this section
			-- 'a_key' is expected of the form "key1\key2\...\subkey"
			-- <subkey> should be empty if you want the default value of preceding path
		require
			is_readable
		do
			Result := implementation.get (a_key)
		end

	get_dword (a_key: STRING): INTEGER is
			-- Dword value corresponding to 'a_key' under 'root'
		require
			is_readable
		do
			Result := implementation.get_dword (a_key)
		end

	has (a_key: STRING): BOOLEAN is
		require
			is_readable
		do
			Result := implementation.has (a_key)
		end

	sub_keys (a_key: STRING): ROSE_LINEAR_ARRAY [STRING] is
			-- Children keys of 'a_key' in this section
		require
			is_readable
		do
			Result := implementation.sub_keys (a_key)
		end

	sub_value_keys (a_key: STRING): ROSE_LINEAR_ARRAY [STRING] is
			-- Children sub-value keys of 'a_key' in this section
		require
			is_readable
		do
			Result := implementation.sub_value_keys (a_key)
		end

feature -- Basic operations

	set (a_key, a_value: STRING) is
		require
			is_modifiable
		do
			implementation.set (a_key,a_value)
		end

	set_dword (a_key: STRING; a_value: INTEGER) is
		require
			is_modifiable
		do
			implementation.set_dword (a_key,a_value)
		end

	delete (a_key: STRING) is
		require
			is_modifiable
		do
			implementation.delete (a_key)
		end

	delete_recursively (a_key: STRING) is
		require
			is_modifiable
		do
			implementation.delete_recursively (a_key)
		end

feature {NONE} -- Implementation

	implementation: ROSE_REGISTRY_IMPLEMENTATION

end
