indexing
	description: "Class to facilitate interaction with relationship model"
	author: "Mark Howard"
	last_editor: "$Author: it $"
	date: "$Date: 2008/12/01 $"
	revision: "$Revision: #9 $"
	path: "$File: //rose/source/kernel/entities/rose_relationship_access.e $"

class
	ROSE_RELATIONSHIP_ACCESS
   
inherit
	ANY
	DATABASE_SERVER
		export
			{NONE} all
		end

feature {SUPPLIER_CONTEXT} -- Access
	
	relationship_name (a_id: INTEGER): STRING is
			-- Relationship name for 'a_id'
		do
			if relationship_from_id.item (a_id) /= Void then
				Result := relationship_from_id.item (a_id).name
			end
			if Result = Void then
				Result := "<unknown>"
			end
		ensure
			valid_result: Result /= Void
		end
	
	relationship_by_id (a_id: INTEGER): RELATIONSHIP_TYPE_RECORD is
		do
			Result := relationship_from_id.item (a_id)
		end

	relationship_id (a_name: STRING): INTEGER is
			-- Relationship type id for 'a_name'
		require
			valid_name: a_name /= Void
		local
			l_name: STRING
		do
			l_name := a_name.twin
			l_name.prune_all_trailing(' ')
			l_name.to_upper
			relationship_ids.search (l_name)
			if relationship_ids.found then
				Result := relationship_ids.found_item.relationship_type_id
			end
		end
	
	entity_1_category (a_name: STRING): INTEGER is
			-- Category type id for the source entity type of relationship 'a_name'
		require
			valid_name: a_name /= Void
		local
			l_record: RELATIONSHIP_TYPE_RECORD
		do
			l_record := relationship_by_id (relationship_id (a_name))
			if l_record /= Void then
				Result := l_record.category_1_id
			end
		end
		
	entity_2_category (a_name: STRING): INTEGER is
			-- Category type id for the target entity type of relationship 'a_name'
		require
			valid_name: a_name /= Void
		local
			l_record: RELATIONSHIP_TYPE_RECORD
		do
			l_record := relationship_by_id (relationship_id (a_name))
			if l_record /= Void then
				Result := l_record.category_2_id
			end
		end
		
feature {NONE} -- Implementation

	relationship_from_id: HASH_TABLE[RELATIONSHIP_TYPE_RECORD,INTEGER] is
		once
			initialize_relationships
			Result := once_relationship_from_id
		end

	relationship_ids: HASH_TABLE[RELATIONSHIP_TYPE_RECORD, STRING] is
		once
			initialize_relationships
			Result := once_relationship_ids
		end
	
	once_relationship_ids: HASH_TABLE[RELATIONSHIP_TYPE_RECORD, STRING] is
		once
			create Result.make (17)
		end
		
	once_relationship_from_id: HASH_TABLE[RELATIONSHIP_TYPE_RECORD,INTEGER] is
		once
			create Result.make (17)
		end

	initialize_relationships is
		local
			l_reader: SIMPLE_PREDICATE[RELATIONSHIP_TYPE_RECORD]
		once
			create l_reader.make (servers.rose_entity_foreground_server, "select * from relationship_type")
			l_reader.apply (agent add_relationship_type_id, Void, Void)
		end

	add_relationship_type_id (a_record: RELATIONSHIP_TYPE_RECORD) is
		local
			l_record: RELATIONSHIP_TYPE_RECORD
		do
			l_record := a_record.twin
			l_record.name.prune_all_trailing (' ')
			l_record.name.to_upper
			once_relationship_ids.put (l_record,l_record.name)
			once_relationship_from_id.put (l_record, l_record.relationship_type_id)
		end

end

