indexing
	description: "A Report"
	author: "Randy John(Fiscal Report) modified by Mark Howard(Rose Report)"
	date: "$Date: 2014/02/26 $"

class
	ROSE_REPORT[reference G]

inherit
	ROSE_CONSTANTS
	EXCEPTIONS
		export
			{NONE} all
		end

create
	make--,make_simple

feature  --Create
	make (a_expressions : like expressions; a_report_name: STRING; a_report_type: STRING) is
			-- Creation procedure
		local
			l_report_expression: REPORT_EXPRESSION [like generic_template]
		do
			expressions := a_expressions
			report_name := a_report_name
			report_type := a_report_type
			create tree.make (report_name)
			from
				tree.start
			until
				tree.off
			loop
				l_report_expression := tree.current_item
				l_report_expression.set_data (a_expressions.interpretation(a_report_type, l_report_expression.mnemonic),l_report_expression.name,l_report_expression.mnemonic)
				tree.forth
			end
		end
feature --Access
	expression_name_order (a_expression_1,a_expression_2 : like report_expression_template) : BOOLEAN is
		require
			nonvoid : a_expression_1 /= void and then a_expression_2 /= void
		do
			Result := a_expression_1.name < a_expression_2.name
		end
	record_class_text  : STRING is
		local
			l_expressions : ROSE_LINEAR_ARRAY[like report_expression_template]
		do
			create l_expressions.make_from_linear(tree.linear_representation)
			l_expressions.sort_by(agent expression_name_order)
			create Result.make(1000)
			Result.append("class%N%T" + report_name.as_upper + "_RECORD%Nfeature%N")
			from l_expressions.start until l_expressions.off loop
				Result.append("%T" + l_expressions.current_item.name + " : DOUBLE --" + l_expressions.current_item.mnemonic + "%N")
				l_expressions.forth
			end
			from l_expressions.start until l_expressions.off loop
				Result.append("%Tset_" + l_expressions.current_item.name + "(a_value : DOUBLE) is%N" +
							  "%T%Tdo%N%T%T%T" + l_expressions.current_item.name + ":= a_value%N%T%Tend%N")
					l_expressions.forth
			end
			Result.append("end%N")
		end

feature --State change
	write_record (a_record : like record_template) is
		local
			l_expression : like report_expression_template
		do

			from tree.start until tree.off loop
				l_expression := tree.current_item
				if l_expression /= Void and then l_expression.data /=Void then
					l_expression.set_record_value(a_record)
					--					if  l_expression.data.type = constants.double_type  then
					--						if l_expression.defined then
					--							l_expression.set_record_double_value(l_expression.data_value,a_record)
					--						else
					--							l_expression.set_record_double_value(constants.missing_double,a_record)
					--						end
					--					elseif  l_expression.data.type = constants.integer_type then
					--						if l_expression.defined then
					--							l_double := l_expression.data_value.rounded
					--							l_expression.set_record_integer_value(l_double,a_record)
					--						else
					--							l_expression.set_record_integer_value(constants.missing_integer,a_record)
					--						end
					--					elseif  l_expression.data.type = constants.string_type then
					--						if l_expression.defined then
					--							l_expression.set_record_string_value(l_expression.string_value,a_record)
					--						else
					--							l_expression.set_record_string_value(constants.missing_string,a_record)
					--						end
					--					else
					--						if l_expression.defined then
					--							l_expression.set_record_double_value(l_expression.data_value,a_record)
					--						else
					--							l_expression.set_record_double_value(constants.missing_double,a_record)
					--						end
					--					end

				end
				tree.forth
			end
		end

	write_record_available_items (a_record : like record_template) is
		local
			l_expression : like report_expression_template
		do
			from tree.start until tree.off loop
				l_expression := tree.current_item
				if l_expression /= Void and then l_expression.data /=Void then
					if	l_expression.data.type = constants.double_type	then
						if l_expression.is_available then
							l_expression.set_record_double_value(l_expression.data_value,a_record)
						end
					elseif	l_expression.data.type = constants.integer_type then
						if l_expression.is_available then
							l_expression.set_record_integer_value(l_expression.data_value.rounded,a_record)
						end
					elseif	l_expression.data.type = constants.string_type then
						if l_expression.is_available then
							l_expression.set_record_string_value(l_expression.string_value,a_record)
						end
					else
						if l_expression.is_available then
							l_expression.set_record_double_value(l_expression.data_value,a_record)
						end
					end
				end
				tree.forth
			end
		end

	write_record_sum (a_record : like record_template) is
		local
			l_expression : like report_expression_template
		do
			from tree.start until tree.off loop
				l_expression := tree.current_item
				if l_expression /= Void then
					l_expression.set_record_double_value(l_expression.computed_value,a_record)
				end
				tree.forth
			end
		end
	set_record_to_constant (a_record : like record_template;a_value : DOUBLE) is
		local
			l_expression : ROSE_RECORD_ACCESS
		do
			from tree.start until tree.off loop
				l_expression := tree.current_item
				if l_expression /= Void then
					l_expression.set_record_double_value(a_value,a_record)
				end
				tree.forth
			end
		end
	transfer_record (a_target,a_source : like record_template) is
		local
			l_expression : ROSE_RECORD_ACCESS
		do
			from tree.start until tree.off loop
				l_expression := tree.current_item
				if l_expression /= Void then
					l_expression.set_record_double_value(l_expression.double_value_from_record(a_source),a_target)
				else
					l_expression.set_record_double_value(constants.missing_double,a_target)
				end
				tree.forth
			end
		end

feature  --Modify
	add(a_report : like Current) is
			-- Add a report
		do
			if report_name.is_equal(a_report.report_name) and then (tree.first.computed_mode /= 0 and a_report.tree.first.computed_mode /= 0) then
				from tree.start until tree.off loop
					tree.current_item.add(a_report.tree.item(tree.index))
					tree.forth
				end
			end
		end

	multiply(a_factor : DOUBLE) is
		do
			from tree.start until tree.off loop
				tree.current_item.multiply(a_factor)
				tree.forth
			end
		end

	infix "*" (a_factor : DOUBLE) : like Current is
		do
			Result := deep_twin
			Result.multiply(a_factor)
		end

	infix "+" (a_report : like Current) : like Current is
		do
			if a_report /= Void then
				Result := deep_twin
				Result.add(a_report)
			end
		end

	infix "-" (a_report : like Current) : like Current is
		do
			if a_report /= Void then
				Result := deep_twin
				Result.multiply (-1.0)
				Result.add(a_report)
				Result.multiply (-1.0)
			end
		end
feature  --Access
    report_name : STRING
    report_type : STRING

    report_date: ROSE_DATE
	set_report_date(a_date : like report_date) is
		do
			report_date := a_date
		end

	report_entity : ENTITY
	set_report_entity(a_entity : like report_entity) is
		do
			report_entity := a_entity
		end

	report_as_of_date : ROSE_DATE
	set_report_as_of_date(a_date : like report_as_of_date) is
		do
			report_as_of_date := a_date
		end

	currency : PERSISTENT_CURRENCY
	set_currency (a_currency : PERSISTENT_CURRENCY) is
		do
			currency := a_currency

		end
	market : MARKET
	set_market (a_market : MARKET) is
		do
			market := a_market
		end
	report_detail_text : STRING
	set_report_detail_text(a_text : like report_detail_text) is
		do
			report_detail_text := a_text
		end

	apply(a_agent: PROCEDURE[ANY, TUPLE[like report_expression_template]]) is
			-- Apply a agent to the tree
		do
			tree.do_all(a_agent)
		end


feature  --State change
	init(a_date : ROSE_DATE; a_market : like market; a_currency : like currency) is
		do
			report_as_of_date := a_date
			market := a_market
			currency := a_currency
			tree.init(report_as_of_date, market, currency)
		end

	compute(a_target : G;a_date : ROSE_DATE) is
			--use a_computation to get a computed value per node, and set the actual data_value for each node
		do
			tree.compute(a_target,a_date,market,currency)
			available := is_available
			entity := a_target
		end

	compute_ai(a_target : G;a_date : ROSE_DATE) is
			--using computed value if available
		do
			tree.compute_ai(a_target,a_date,market,currency)
			available := is_available
			entity := a_target
		end

	compute_flat(a_target : G;a_date : ROSE_DATE) is
		do
			tree.compute_flat(a_target,a_date,market,currency)
			available := is_available
			entity := a_target
		end

	--   items: ROSE_LINEAR_ARRAY[GROUP_EXPRESSION[ANY,like record_anchor]] is
	--      do
	--      end

	periodicity_of_report: INTEGER
	max_depth : INTEGER
	update_max_depth(a_expression : like report_expression_template) is
		do
			if max_depth < a_expression.depth then
				max_depth := a_expression.depth
			end
		end

	dump_it  is
		do
			tree.do_all(agent dump_it_agent)
		end

	dump_it_agent(a_expression: like report_expression_template) is
		local
			l_spaces: STRING
			l_depth: INTEGER
		do
			if a_expression.path /= void then
				l_depth := a_expression.path.count
			end

			if l_depth=0
			 then
				l_spaces := ""
			else
				l_spaces := "  "
				l_spaces.multiply(l_depth)
			end

			print(l_spaces + a_expression.name + "   ")
			if a_expression.defined
			 then
				if a_expression.data_type=constants.string_type
				 then
					print(a_expression.string_value.out + "%N")
				else
					print(a_expression.data_value.out + "%N")
				end
			else
				print("undefined%N")
			end
		end

	dump  is
		do
		end

	tree: ROSE_REPORT_TREE[G,like report_expression_template]

	expressions : ENTITY_VARIABLES[G]
	entity: G

	dump_agent(a_expression: like report_expression_template) is
		local
			l_spaces: STRING
			l_depth: INTEGER
			l_expression : like report_expression_template
		do
			if a_expression.name.is_equal("equity_reserves") then
				do_nothing
			end
			expression_count := expression_count + 1
			if a_expression.path /= void then
				l_depth := a_expression.path.count
			end
			if l_depth=0
			 then
				l_spaces := ""
			else
				l_spaces := "  "
				l_spaces.multiply(l_depth)
			end
			data_array.force(l_spaces + a_expression.name,expression_count, 1)
			if a_expression.mapped then
				if a_expression.defined then
					data_array.force(a_expression.data_value,expression_count,2)
				else
					data_array.force("#N/A",expression_count,2)
				end
			end
			data_array.force(a_expression.computed_value,expression_count, 3)
			data_array.force(a_expression.partial_value,expression_count,4)
			l_expression := a_expression
			--		print(l_depth.out + " - " + a_expression.dotted_mnemonic + "%N")
			--		print(l_spaces + a_expression.name + "%N")
		end

	is_available:BOOLEAN is
		do
			Result := False
			from
				tree.start
			until
				tree.off or Result
			loop
				if tree.current_item.is_available then
					Result := True
				end
				tree.forth
			end
		end

	data_array : ARRAY2[ANY]
	expression_count : INTEGER
	available:BOOLEAN
feature --Templates
	report_expression_template : REPORT_EXPRESSION[G] is do end
			--	identifier_header(a_target : like record_anchor) : ROSE_LINEAR_ARRAY[STRING] is
			--		once
			--			create Result.make_from_array(a_target.description_title)
			--			Result.append_array(<<"Report">>)
			--		end
			--	identifier(a_target : like record_anchor) : ROSE_LINEAR_ARRAY[STRING] is
			--		do
			--			create Result.make_from_array(a_target.description)
			--			Result.append_array(<<report_name>>)
			--		end
			--	identifier_string(a_target : like record_anchor) : STRING is
			--		local
			--			i : INTEGER
			--			l_identifier : ROSE_LINEAR_ARRAY[STRING]
			--		do
			--			l_identifier := identifier(a_target)
			--			Result := ""
			--			from
			--				i := 1
			--			until
			--				i > l_identifier.count
			--			loop
			--				if l_identifier.item(i) /= Void then
			--				   if i > 1 then
			--				      Result := Result + "|"
			--				   end
			--				   Result := Result + l_identifier.item(i)
			--				end
			--				i := i + 1
			--			end
			--		end

	generic_template : G is do end
	record_template : ANY is do end
end
