indexing
	description: "Objects that ..."
	author: ""
	date: "$Date: 2008/12/01 $"
	revision: "$Revision: #27 $"
	
class
	ROSE_REPORT_TREE[reference G, reference H->REPORT_EXPRESSION[G] create make end]
	
inherit
	ROSE_LINEAR_ARRAY[H]
		rename
			make as rose_linear_array_make
		end
	
create
	
	make
create {ROSE_REPORT_TREE}
	rose_linear_array_make,
	make_empty,
	make_from_array
feature
	
	make (a_name : STRING) is
		local
			l_count,i : INTEGER
			l_records : ROSE_LINEAR_ARRAY[REPORT_RECORD]
		do
			make_empty
			if not records_cache.has(a_name) then
				create records.make(10)
				reader_predicate.apply(agent add_record,void,<<a_name>>)
				if records /= Void then
					records_cache.force(records,a_name)
					l_count := records_cache.count
				end
			else
				records := records_cache.item(a_name)
				l_count := records_cache.count
			end
			create l_records.make_from_linear (records.linear_representation)
			l_records.sort_by(agent compare_suborder)
			if l_count > 0 then
				create expressions.make(l_count)
				create names.make(1,records.count)
				create mnemonics.make(1,records.count)
				i:=1
				from l_records.start until l_records.off loop
					expression(l_records.current_item.mnemonic).do_nothing -- side effect creates the expression
					mnemonics.put(l_records.current_item.mnemonic,i)
					names.put(l_records.current_item.name,i)
					i:=i+1
					l_records.forth
				end
			end
			sort_by(agent compare_path)
			create index_ordered_expressions.make_from_array (Current)
			index_ordered_expressions.sort_by (agent compare_index)
			from index_ordered_expressions.start
			until index_ordered_expressions.after
			loop
				index_ordered_expressions.current_item.set_tree_index (index_ordered_expressions.index)
				index_ordered_expressions.forth
			end
		end
	compare_suborder(a_1,a_2 : REPORT_RECORD) : BOOLEAN is
		do
			Result := a_1.suborder < a_2.suborder
		end
	reader_predicate : SIMPLE_PREDICATE[REPORT_RECORD] is
		once
			create Result.make(server,"select * from reports where report_name = ? order by mnemonic")
		end
	
	add_record (a_report_record : REPORT_RECORD) is
		local
			l_record : REPORT_RECORD
		do
			l_record := clone(a_report_record)
			l_record.mnemonic.to_lower
			l_record.mnemonic.prune_all(' ')
			if l_record.name /= Void then
				l_record.name.to_lower
				l_record.name.prune_all(' ')
			end
			if l_record.parent /= void then
				l_record.parent.to_lower
				l_record.parent.prune_all(' ')
			end
			records.force(l_record,l_record.mnemonic)
		end
	
	expression (a_mnemonic : STRING) : like expression_template is
		local
			l_parent : like expression_template
			l_record : REPORT_RECORD
		do
			if a_mnemonic /= void and then a_mnemonic.count > 0 then
				Result := expressions.item(a_mnemonic)
				if Result = void then
					l_record := records.item(a_mnemonic)
					if l_record /= void then
						l_parent := expression(l_record.parent)
						create Result.make(l_parent,l_record)
						expressions.put(Result,a_mnemonic)
						extend(Result)
					end
				end
			end
		end
	expression_template : H
	expressions : ROSE_HASH_TABLE[like expression_template,STRING]
	
	records : ROSE_HASH_TABLE[REPORT_RECORD,STRING]
	
	server : DATA_SERVER is
		local
			l_data_server : DATABASE_SERVER
		once
			create l_data_server
			Result := l_data_server.servers.reports_server
		end
	
	compare_path (a_expression_1,a_expression_2 : like expression_template) : BOOLEAN is
		local
			l_path_1,l_path_2 : ROSE_LINEAR_ARRAY[REPORT_EXPRESSION[G]]
			i : INTEGER
			l_count : INTEGER
			l_found : BOOLEAN
		do
			l_path_1 := a_expression_1.path
			l_path_2 := a_expression_2.path
			l_count := l_path_1.count
			if l_count > l_path_2.count then
				l_count	:= l_path_2.count
			end
			from i := 1 until i > l_count or l_found loop
				if l_path_1.item(i).name < l_path_2.item(i).name then
					Result := true
					l_found := true
				elseif l_path_1.item(i).name > l_path_2.item(i).name then
					Result := false
					l_found := true
				end
				i := i + 1
			end
			if not l_found then
				if l_path_1.count < l_path_2.count then
					Result := true
				else
					Result := false
				end
			end
		end
	
feature
	init(a_date : ROSE_DATE; a_market : MARKET; a_currency : PERSISTENT_CURRENCY) is
		do
			from expressions.start until expressions.off loop
				if expressions.item_for_iteration.parent = void then
					expressions.item_for_iteration.init(a_date, a_market, a_currency)
				end
				expressions.forth
			end
		end
	
	compute (a_target : like target;a_date : ROSE_DATE;a_market : MARKET;a_currency : PERSISTENT_CURRENCY) is
		do
			target := a_target
			from expressions.start until expressions.off loop
				if expressions.item_for_iteration.parent = void then
					expressions.item_for_iteration.compute(a_target,a_date,a_market,a_currency)
				end
				expressions.forth
			end
		end
	
	compute_flat (a_target : like target;a_date : ROSE_DATE;a_market : MARKET;a_currency : PERSISTENT_CURRENCY) is
		do
			target := a_target
			from expressions.start until expressions.off loop
				expressions.item_for_iteration.compute_flat(a_target,a_date,a_market,a_currency)
				expressions.forth
			end
		end
	
	compute_ai(a_target : like target;a_date : ROSE_DATE;a_market : MARKET;a_currency : PERSISTENT_CURRENCY) is
		do
			target := a_target
			from expressions.start until expressions.off loop
				if expressions.item_for_iteration.parent = void then
					expressions.item_for_iteration.compute_ai(a_target,a_date,a_market,a_currency)
				end
				expressions.forth
			end
		end
feature
	target : G
	names:ROSE_LINEAR_ARRAY[STRING]
	mnemonics:ROSE_LINEAR_ARRAY[STRING]
	index_ordered_expressions : ROSE_LINEAR_ARRAY[like expression_template]
	compare_index (a_expression_1,a_expression_2 : like expression_template) : BOOLEAN is
        do
			if a_expression_1.description /= Void and a_expression_2.description /= Void then
				Result := (a_expression_1.description.variable_index < a_expression_2.description.variable_index)
	        else
	        	Result := a_expression_2.description /= Void
			end
        end
	
feature -- Report Characteristics Cache
	
	records_cache : ROSE_HASH_TABLE[ROSE_HASH_TABLE[REPORT_RECORD,STRING],STRING] is
		once
			create Result.make(10)
		end
	
end
