note
	description: "EV view on resource availability"
	author: "Zoran Simic"
	last_editor: "$Author: zs $"
	date: "$Date: 2011/04/19 $"
	revision: "$Revision: #12 $"
	path: "$File: //rose/source/kernel/infrastructure/framework/gui/rose_resource_view.e $"

class
	ROSE_RESOURCE_VIEW

inherit
	ROSE_SHARED_GLOBAL_GUI

create
	make

feature {NONE} -- Initialization

	make
			-- Create a view on resources.
		local
			l_main_box, l_box: EV_BOX
			l_cancel: EV_BUTTON
		do
			dialog := Gui.factory.new_dialog ("Resource view", "resource.ico", 600, 400)
			l_main_box := Gui.factory.new_vbox (dialog, 2, 4)
			l_box := Gui.factory.new_hbox (l_main_box, 0, 4)
			types_combo := Gui.factory.new_combo (l_box, "Type: ", resource_types, Void, agent refresh, 120)
			file_name_field := Gui.factory.new_text_field (l_box, " File: ", Void, agent refresh, 120)
				--| Toolbar
			toolbar := Gui.factory.new_toolbar (l_box, <<
				["refresh.ico", agent refresh, "Refresh"]
			>>)
			grid := Gui.factory.new_grid (l_main_box, <<["Path", 478], ["Comment", 110]>>, agent on_grid, 0, 0)
			l_box := Gui.factory.new_hbox (l_main_box, 0, 8)
			select_button := Gui.factory.new_button (l_box, "OK", agent on_ok, 80)
			l_cancel := Gui.factory.new_button (l_box, "Cancel", agent on_cancel, 80)
			dialog.set_default_push_button (select_button)
			dialog.set_default_cancel_button (l_cancel)
		end

feature -- Access

	resource_type: STRING
			-- Selected resource type

	selected_resource: STRING
			-- Selected resource coordinate ($1 represents type, $2 file name)

	selected_file_name: STRING
			-- Selected full path to resource file if any
		do
			if selected_resource /= Void then
--				Result := Global.app.resource_expanded_path (selected_resource, resource_type, file_name)
				if not Global.is_file_readable (Result) then
					Result := Void
				end
			end
		end

feature -- Basic operations

	show (a_window: EV_WINDOW)
			-- Show modal to 'a_window' if given, just show otherwise
		do
			refresh
			if a_window /= Void then
				dialog.show_modal_to_window (a_window)
			else
				Gui.open_window (Void, dialog)
			end
		end

	refresh
			-- Refresh the view.
		local
			l_filename: STRING
		do
			grid.clear
			select_button.disable_sensitive
			resource_type := type
			l_filename := file_name
			add_resource_path (Global.app.program_folder, resource_type, l_filename)
			add_resource_path (Global.app.eifgen_folder, resource_type, l_filename)
		end

	select_type (a_type: STRING)
			-- Select resource with 'a_type'.
		do
			types_combo.enable_sensitive
			types_combo.select_by_name (a_type)
			types_combo.disable_sensitive
		end

	set_filename (a_filename: STRING)
			-- Set 'file_name'
		do
			if a_filename /= Void then
				file_name_field.set_text (a_filename)
			else
				file_name_field.remove_text
			end
		end

feature {NONE} -- Implementation

	dialog: EV_DIALOG
	toolbar: EV_TOOL_BAR
	types_combo: ROSE_COMBO_BOX
	file_name_field: EV_TEXT_FIELD
	grid: RP_DATA_GRID[ANY]
	select_button: EV_BUTTON

	type: STRING
		do
			Result := types_combo.text
		end

	file_name: STRING
		do
			Result := file_name_field.text
			if Result /= Void then
				Result.left_adjust
				Result.right_adjust
			end
			if Result = Void or else Result.is_empty then
				Result := Void
			end
		end

	resource_types: ARRAY [STRING]
		once
			Result := <<"config", "cache", "doc", "resource\icon", "resource\image">>
		end

	add_resource_path (a_path, a_type, a_filename: STRING)
		local
			l_path, l_fullpath, l_comment, l_pixmap: STRING
			l_node: RP_GRID_NODE[ANY]
		do
--			l_path := Global.app.resource_expanded_path (a_path, a_type, "")
			if a_filename /= Void then
--				l_fullpath := Global.app.resource_expanded_path (a_path, a_type, a_filename)
			end
			if Global.is_directory_readable (l_path) then
				if l_fullpath = Void then
					l_pixmap := "resource_nofile.ico"
					l_comment := "Unreadable file"
					l_fullpath := a_path
				elseif Global.is_file_readable (l_fullpath) then
					l_pixmap := "resource_available.ico"
					l_comment := "Available"
					l_fullpath := a_path
				else
					l_pixmap := "resource_nofile.ico"
					l_comment := "No file"
					l_fullpath := Void
				end
			else
				l_pixmap := "resource_nodir.ico"
				l_comment := "No directory"
				l_fullpath := Void
			end
			l_node := Gui.factory.new_grid_node (l_pixmap, <<l_path, l_comment>>, l_fullpath)
			grid.extend (l_node)
		end

	on_grid (a_button: INTEGER; a_node: RP_GRID_NODE[ANY])
			-- Called when item in list-view double-clicked, selected or deselected.
		local
			l_resource: STRING
		do
			if a_node /= Void then
				l_resource ?= a_node.data
			end
			if a_button = 0 then
				-- Deselected
				selected_resource := Void
			elseif a_button = -1 then
				-- Left click
				selected_resource := l_resource
			elseif a_button = 1 then
				-- Left double click
				on_ok
			end
			if selected_resource /= Void then
				select_button.enable_sensitive
			else
				select_button.disable_sensitive
			end
		end

	on_ok
		do
			if selected_resource /= Void then
				dialog.hide
			end
		end

	on_cancel
		do
			selected_resource := Void
			dialog.hide
		end

end
