indexing
	description: "Serialize a special eiffel object in XML"
	notice: "See rose_serializer.dtd"
	author: "Zoran Simic"
	date: "$Date: 2010/03/08 $"
	revision: "$Revision: #10 $"

class
	ROSE_SERIALIZER_SPECIFIC [G]

inherit
	STRING_HANDLER

create
	make

feature {NONE} -- Initialization

	make (a_byte_size: INTEGER; a_key: STRING) is
			-- Creation procedure
		require
			key_ok: a_key /= Void
		do
			byte_size := a_byte_size
			key := a_key
		end

feature -- Access

	byte_size: INTEGER
			-- Byte size of one 'G'

	key: STRING
			-- Associated key

feature -- Output

	output_special (a_serializer: ROSE_SERIALIZER; a_special: SPECIAL [G]; a_name: STRING; a_id: INTEGER; a_agent: PROCEDURE [ANY, TUPLE [STRING, G]]) is
			-- Serialize a special object.
		local
			i, n: INTEGER
			l_item: G
		do
			a_serializer.start_element (key, a_name, a_special.count.out, a_serializer.internal_type (a_special), a_id)
			from
				i := 0
				n := a_special.count
			until
				i >= n
			loop
				l_item := a_special.item (i)
				if l_item = Void then
					a_serializer.current_file.put_string (markup_void)
				else
					a_agent.call ([Void, l_item])
				end
				i := i + 1
			end
			a_serializer.terminate_element (key)
		end

	output_special_section (a_serializer: ROSE_SERIALIZER; a_special: SPECIAL [G]; a_name: STRING; a_id: INTEGER) is
			-- Serialize a special object.
		local
			t: INTEGER
			s: STRING
			p: MANAGED_POINTER
		do
			t := a_special.count
			create p.share_from_pointer (a_special.base_address, t * byte_size)
			s := base64.encoded (p)
			a_serializer.start_element (key, a_name, s.count.out, t, a_id)
			a_serializer.current_file.put_string (markup_cdata)
			a_serializer.current_file.put_string (s)
			a_serializer.current_file.put_string (markup_end_cdata)
			a_serializer.terminate_element (key)
		end

	get_special_encoded (a_deserializer: ROSE_DESERIALIZER; a_file: FILE; a_line: STRING): SPECIAL [G]
			-- SPECIAL [G] decoded from 'a_file' at current position.
		local
			l_buffer, l_id: STRING
			l_default: G
			p: MANAGED_POINTER
			i, j, t: INTEGER
		do
			if a_line.item (5) = 'n' then
				i := a_line.index_of ('"', 8) + 5
			else
				i := 8
			end
			j := a_line.index_of ('"', i)
			t := a_line.substring (i, j - 1).to_integer
			j := j + 5
			i := a_line.index_of ('"', j)
			l_id := a_line.substring (j, i - 1)
			i := i + 5
			j := a_line.index_of ('"', i)
			i := a_line.substring (i, j - 1).to_integer
			create Result.make_filled (l_default, t)
			a_deserializer.register_ref (Result, l_id)
			from
				a_file.readline
				a_file.readline
				create l_buffer.make (i + 10)
			until
				a_file.last_string.count > 0 and then a_file.last_string.item (1) = ']'
			loop
				l_buffer.append (a_file.last_string)
				a_file.readline
			end
			create p.share_from_pointer (Result.base_address, Result.count * byte_size)
			base64.decode (l_buffer, p)
			a_file.readline
		end

feature {NONE} -- Implementation

	markup_cdata: STRING is"<![CDATA[%N"		-- Start CDATA markup
	markup_end_cdata: STRING is "%N]]>%N"		-- End CDATA markup
	markup_void: STRING is "<v/>%N"				-- Void markup

	base64: ROSE_BASE64_ENCODING is
			-- Base64 encoder
		once
			create Result
		end

end
