note

	description:

		"shared access to ELOG_LOGGER with hooks into process metrics via ROSE_FILE"

	author: "Theo Lengyel"
	last_editor: "$Author: e2 $"
	date: "$Date: 2014/07/04 $"
	revision: "$Revision: #7 $"
	path: "$File: //rose/source/kernel/infrastructure/logging/rose_dependant/rose_shared_logger.e $"

class ROSE_SHARED_LOGGER

inherit
	ROSE_SHARED_GLOBAL
	ELOG_SHARED_LOGGER
	ROSE_SHARED_LOGGER_FACTORY

create
	set_logger_factory

feature {NONE} -- Factory

	initialize_logger_from_file (a_filename: STRING)
			-- Loads logger info from `a_filename'
		require
			filename_not_void: a_filename /= Void
			file_exists: (create {ROSE_FILE}.make (a_filename)).exists
			file_readable: (create {ROSE_FILE}.make (a_filename)).is_readable
		local
			l_parser: ELOG_XML_CONFIG_PARSER [ROSE_FILE]
		do
			create l_parser.make_with_hierarchy (a_filename, logger.hierarchy)
			if l_parser.parse_error then
				io.put_string ("Invalid configuration file ")
				io.put_string (a_filename)
				io.put_new_line
				io.put_string ("No logging will occur.")
				io.put_new_line
			end
		end

	initialize_logger (a_use_std_out: BOOLEAN)
			-- Set a logger with information from the configuration
			-- 1) filename at "logging/filename"
			-- 2) exclude/include at "logging/exclude_filter" and "logging/include_filter"
			-- 3) use UTC
		local
			l_config: ELOG_CONFIGURATION
		do
			create l_config.make
			l_config.set_use_std_out (a_use_std_out)
			set_logger (new_complex_logger (l_config, log_filename_key))
		end

	new_complex_logger (a_config: ELOG_CONFIGURATION;
		a_filename_key: STRING): ELOG_LOGGER
			-- Logger to use for this command
			-- with informations in `a_config'
			-- filename, lines to exclude/include are in the cfg (config) file
			-- at the keys provided here
		require
			a_config_not_void: a_config /= Void
		local
			l_filename: STRING
			l_config: ELOG_CONFIGURATION
		do
			l_config := a_config.twin
			l_filename := global.interpreted (config.value (a_filename_key))

			if l_filename = Void or else l_filename.is_empty then
				-- if no path is given in the cfg file, the following is used
				l_filename := "logger.log"
			end
			-- sets the file to the given path
			l_filename := global.app.log_path (l_filename)

			l_config.set_filename (l_filename)
			l_config.set_exclude_filters (children_values (exclude_filter_key))
			l_config.set_include_filters (children_values (include_filter_key))
			Result := logger_factory.new_complex_logger (l_config)
			Result.set_severity (logger_severity)
			Result.set_flush (global.app.configured_boolean_value ("logging/flush", False))
		end

feature {NONE} -- Helpers

	logger_severity: ELOG_SEVERITY
			-- Severity to use for logger
		local
			l_severity_name: STRING
			l_severity_constants: ELOG_SEVERITY_CONSTANTS
		do
			create l_severity_constants
			l_severity_name := global.app.env.get ("severity")
			if l_severity_name /= Void then
				Result := l_severity_constants.lookup_severity_from_string (l_severity_name)
				if Result = Void then
					l_severity_name := Void
				end
			end
			if l_severity_name = Void then
				l_severity_name := global.app.configured_value ("logging/severity")
			end
			if l_severity_name /= Void then
				Result := l_severity_constants.lookup_severity_from_string (l_severity_name)
			end
			if Result = Void then
				Result := l_severity_constants.Default_severity
			end
		ensure
			logger_severity_not_void: Result /= Void
		end

	children_values (a_key: STRING): ARRAY [STRING]
			-- Values of items under `a_key'
			-- either in the configuration file or else in the delivery.cfg file
		do
			Result := config.children_values (a_key)
			if Result = Void then
				Result := global.app.delivery.children_values (a_key)
			end
			if Result = Void then
				create Result.make_empty
			end
		ensure
			children_values_not_void: Result /= Void
			no_void_children_value: not Result.has (Void)
		end

feature {NONE} -- Keys in configuration file

	log_filename_key: STRING is "logging/filename"
	exclude_filter_key: STRING is "logging/exclude_filter"
	include_filter_key: STRING is "logging/include_filter"

end
