indexing
	description: "Signature tagged report"
	author: "Michael Zhang"
	date: "$Date: 2010/04/08 $"
	revision: "$Revision: #26 $"
	path: "$File: //rose/source/kernel/data/reports/reference/rose_signature_tagged_report.e $"

deferred class
	ROSE_SIGNATURE_TAGGED_REPORT[reference H -> ENTITY create make,make_anonymous end, reference G -> ROSE_SIGNATURE_RECORD[H] create default_create end]

inherit
	ROSE_SIMPLE_REPORT[G]
	rename
		make as simple_report_make
	redefine
		expression_template,
		targets,
		targets_in_interval,
		dated_matrix,
		entity_anchor
	end

feature -- Creation
	make(a_report_name: STRING; a_report_type: STRING) is
			-- make
		require
			valid_name : a_report_name /= Void
		local
			l_found : BOOLEAN
		do
			create ur_target
			simple_report_make(ur_target.expressions, a_report_name, a_report_type)
			if expressions /= Void then
				from expressions.start until expressions.off or l_found loop
					if expressions.current_item /= Void then
						ur_expression := expressions.current_item
						l_found := True
					end
					expressions.forth
				end
			end
		end

	make_from_target is
			-- make from target
		local
			l_expression : like expression_template
			l_descriptions : LINEAR[VARIABLE_DESCRIPTION]
			l_description : VARIABLE_DESCRIPTION
			l_variables : ENTITY_VARIABLES[like ur_target]
		do
			create ur_target
			l_variables := ur_target.expressions
			if l_variables /= Void then
				l_descriptions := l_variables.variable_descriptions.linear_representation

				if l_descriptions /= Void then
					create expressions.make_empty
					from l_descriptions.start until l_descriptions.off loop
						l_description := l_descriptions.item
						l_expression ?= l_variables.variable_from_mnemonic(l_description.mnemonic)
						if valid_expression(l_expression) then
							expressions.extend(l_expression, l_description.mnemonic)
						end
						l_descriptions.forth
					end
					count := expressions.count
				end
			end
		end
feature -- deferred features
	entity_id(a_target : G) : INTEGER is
	-- deferred
		deferred
		end

	entity_from_id(a_entity_id : INTEGER) : like entity_anchor is
	-- deferred
		deferred
		end

feature -- Access
	refresh_expression_cache is
	-- refresh cache
		do
			if ur_expression /= Void then
				ur_expression.refresh_cache
			end
		end

	targets(a_entity : like entity_anchor) : ROSE_LINEAR_ARRAY[G] is
		--time series target
		require else
			valid_etnity : a_entity /= Void
		local
			l_cache : like target_cache
			l_targets : ROSE_LINEAR_ARRAY[G]
		do
			if a_entity /= Void then
				l_cache := target_cache
				if l_cache /= Void then
					if l_cache.has(a_entity.handle) then
						Result := l_cache.item(a_entity.handle)
					else
						l_targets := entity_targets(<<a_entity>>)
						ur_target.set_direction_time_series
						Result := ur_target.targets(l_targets, ur_target.restriction_statement)
						if Result /= Void and then Result.count > 0 then
							Result.sort_by (Agent target_order_by_date)
						end
					end
				end
			end
		end

	targets_in_interval(a_entity : like entity_anchor; a_start_date, a_end_date : ROSE_DATE) : ROSE_TIME_SERIES[G] is
		-- get targets in interval
		require else
			valid_dates : a_start_date /= Void and a_end_date /= Void
		local
			l_start_date, l_end_date : ROSE_DATE
			l_target : like entity_target
			l_date_targets, l_targets : like targets
			l_dates : ARRAY[ROSE_DATE]
		do
			create Result.make_empty
			if a_start_date /= Void and a_end_date /= Void then
				l_target := entity_target(a_entity)
				l_dates := a_end_date.dates_from(a_start_date)
				l_date_targets := date_targets(l_dates)
				l_date_targets.extend(l_target)

				ur_target.set_direction_neighborhood
				l_targets := ur_target.targets(l_date_targets, ur_target.restriction_statement)
				if l_targets /= Void and then l_targets.count > 0 then
					l_targets.sort_by (Agent target_order_by_date)
					from l_targets.start until l_targets.off loop
						l_target := l_targets.current_item
						Result.extend (l_target, l_target.date)
						l_targets.forth
					end
				end
			else
--				l_targets := entity_targets(<<a_entity>>)
--				ur_target.set_direction_time_series
--				l_targets := ur_target.targets(l_targets, ur_target.restriction_statement)
				l_targets := targets (a_entity)

				if a_start_date = Void then
					l_start_date := constants.minimum_date_time
				else
					l_start_date := a_start_date
				end
				if a_end_date = Void then
					l_end_date := constants.maximum_date_time
				else
					l_end_date := a_end_date
				end
				if l_targets /= Void then
					from l_targets.start until l_targets.off loop
						l_target := l_targets.current_item
						if  l_target.date >= l_start_date and l_target.date <= l_end_date then
							Result.extend(l_target, l_target.date)
						end
						l_targets.forth
					end
				end
			end
		end

feature -- cross-sectional access, call set_neighborhood() first
	-- get double array at a date
	double_array_at_date(a_entity : like entity_anchor; a_date : ROSE_DATE) : like double_array is
			-- double_array_at_date
		require
			valid_entity : a_entity /= Void
			valid_date : a_date /= Void
		local
			l_target : like ur_target
		do
			if a_entity /= Void then
				l_target := target_at_date(a_entity, a_date)
				if l_target /= Void then
					Result := double_array(l_target)
				end
			end
		end

	target_at_date(a_entity : like entity_anchor; a_date : ROSE_DATE) : like ur_target is
		-- get target at a date
		require
			valid_date : a_date /= Void
		local
			l_targets : like targets_in_interval
		do
			if a_entity /= Void then
				l_targets := target_cache.item(a_entity.handle)
				if l_targets = Void then
					l_targets := targets_in_interval(a_entity, a_date, a_date)
				end
				if l_targets /= Void then
					Result := l_targets.item_at(a_date)
				end
			end
		end

	currency_at_date(a_entity : like entity_anchor; a_date : ROSE_DATE) : like currency_item is
		-- get currency at a date for a entity
		require
			valid_date : a_date /= Void
		local
			l_target : like ur_target
			l_targets : like targets
		do
			if a_entity /= Void then
				if cross_section_date /= Void and then a_date = cross_section_date then
					l_targets ?= target_cache.item(a_entity.handle)
				else
					l_targets := targets_in_interval(a_entity, a_date, a_date)
				end

				if l_targets /= Void and then l_targets.count = 1 then
					l_target := l_targets.first
				else
					l_target := target_at_date (a_entity, a_date)
				end
				if l_target /= Void then
					Result := l_target.rose_currency
				end
			end
		end

	cross_section_date : ROSE_DATE

feature	-- neighborhood

	set_cross_section_neighborhood(a_entities : ARRAY[like entity_anchor]; a_date : ROSE_DATE) is
		-- set cross section neighnorhood
		do
			refresh_target_cache
			cross_section_date := a_date
			set_neighborhood(a_entities, <<a_date>>)
		end

	set_neighborhood(a_entities : ARRAY[like entity_anchor]; a_dates : ARRAY[ROSE_DATE]) is
	-- set neightborhood
		local
			l_targets, l_entity_targets, l_date_targets : like targets
		do
			refresh_target_cache
			l_entity_targets := entity_targets(a_entities)
			l_date_targets := date_targets(a_dates)
			if l_entity_targets /= Void then
				create l_targets.make_from_array(l_entity_targets)
			else
				create l_targets.make_empty
			end
			if  l_date_targets /= Void then
				l_targets.append_array(l_date_targets)
			end
			ur_target.set_direction_neighborhood
			l_targets := neighborhood_targets(l_targets, ur_target.restriction_statement)
			if l_targets /= Void and then l_targets.count > 0 then
				load_target_cache(a_entities, l_targets)
				if ur_expression /= Void then
					ur_expression.set_neighborhood(l_targets)
				end
			end
		end

	neighborhood_targets(a_targets : like targets; a_statement : STRING) : like targets is
	-- targest for neighborhood
		do
			Result := ur_target.targets(a_targets, a_statement)
		end

feature -- target cache
	load_target_cache(a_entities : ARRAY[ENTITY]; a_targets : like targets) is
	-- load targetr cache
		local
			i, l_entity_id : INTEGER
			l_entity : ENTITY
			l_target : G
			l_cache : like target_cache
			l_targets : ROSE_TIME_SERIES[G]
		do
			l_cache := target_cache
			l_cache.clear_all

			if a_entities /= Void then
				from i := a_entities.lower until i > a_entities.upper loop
					l_entity := a_entities.item(i)
					if l_entity /= Void and then l_entity.is_valid then
						create l_targets.make_empty
						l_cache.force(l_targets, l_entity.handle)
					end
					i := i + 1
				end
			end

			if a_targets /= Void then
				from a_targets.start until a_targets.off loop
					l_target := a_targets.current_item
					l_entity_id := entity_id(l_target)
					l_cache.item(l_entity_id).extend(l_target, l_target.date)
					a_targets.forth
				end
			end
		end

	refresh_target_cache is
		-- refresh cache
		local
			l_cache : like target_cache
		do
			l_cache := target_cache
			l_cache.clear_all
		end

	target_cache : HASH_TABLE[ROSE_TIME_SERIES[G], INTEGER] is
		-- target cache
		do
			if target_cache_saved = Void then
				create target_cache_saved.make(2000)
			end
			Result := target_cache_saved
		end

	target_cache_saved : like target_cache

feature -- Implementation
	entity_target(a_entity : like entity_anchor) : like ur_target is
		-- get an entity target
		do
			if a_entity /= Void then
				create Result
				Result.set_entity_id(a_entity.handle)
			end
		end

	entity_targets(a_entities : ARRAY[like entity_anchor]) : like targets is
	-- get entity targest from a array of entities
		local
			i : INTEGER
			l_target : G
			l_entity : like entity_anchor
		do
			if a_entities /= Void and then a_entities.count > 0 then
				create Result.make_empty
				from i := a_entities.lower until i > a_entities.upper loop
					l_entity := a_entities.item(i)
					l_target := entity_target(l_entity)
					if l_target /= Void then
						Result.extend(l_target)
					end
					i := i + 1
				end
			end
		end

	date_targets(a_dates : ARRAY[ROSE_DATE]) : like targets is
	-- get date targesr from a arrany of dates
		local
			i : INTEGER
			l_target : G
			l_date : ROSE_DATE
		do
			if a_dates /= Void and then a_dates.count > 0 then
				create Result.make_empty
				from i := a_dates.lower until i > a_dates.upper loop
					l_date := a_dates.item(i)
					create l_target
					l_target.set_date(l_date)
					Result.extend(l_target)
					i := i + 1
				end
			end
		end

	monthly_targets(a_targets: like targets) : like targets is
	-- get monthly targets for a list of targets
		local
			l_target : G
			l_table : HASH_TABLE[ROSE_DATE, ROSE_DATE]
			l_dates : ROSE_LINEAR_ARRAY[ROSE_DATE]
			l_linear_rep : ARRAYED_LIST[ROSE_DATE]
			l_date_set : ROSE_DATE_SET
		do
			if a_targets /= Void then
				create l_table.make(10)
				from a_targets.start until a_targets.off loop
					l_target := a_targets.current_item
					if l_target /= Void and then not l_table.has(l_target.date) then
						l_table.put(l_target.date, l_target.date)
					end
					a_targets.forth
				end
				l_linear_rep := l_table.linear_representation
				if l_linear_rep /= Void then
					create l_dates.make_from_linear (l_linear_rep)
					l_dates.sort_by (agent compare_date)
					if l_dates.count > 0 then
						create l_date_set.make_from_array(l_dates, l_dates.last)
						l_date_set := l_date_set.monthend_date_set
						if l_date_set /= Void then
							create Result.make_empty
							from a_targets.start until a_targets.off loop
								l_target := a_targets.current_item
								if l_target /= Void and then l_date_set.has(l_target.date) then
									Result.extend(l_target)
								end
								a_targets.forth
							end
						end
					end
				end
			end
		end

feature -- matrix
	dated_matrix(a_targets : ROSE_LINEAR_ARRAY[G]) : DATED_MATRIX is
			-- get dated matric for a collection of targed
		local
			l_matrix : like double_matrix
			l_dates : like date_array_from_targets
			l_currencies : like currencies
		do
			if a_targets /= Void and then a_targets.count > 0 then
				l_dates := date_array_from_targets(a_targets)
				l_matrix := double_matrix(a_targets)

				create Result.make_from_matrix(l_matrix)
				Result.set_dates(l_dates)
				l_currencies := currencies(a_targets)
				Result.set_currency_series(l_currencies)
			end
		end

	currencies(a_targets : ROSE_LINEAR_ARRAY[G]) : ROSE_TIME_SERIES[PERSISTENT_CURRENCY] is
	-- get currencies for a target array
		local
			l_target : G
		do
			if a_targets /= Void and then a_targets.count > 0 then
				create Result.make_empty
				from a_targets.start until a_targets.off loop
					l_target := a_targets.current_item
					if l_target /= Void then
						Result.extend(l_target.rose_currency, l_target.date)
					else
						Result.extend(Void, Void)
					end
					a_targets.forth
				end
			end
		end

	target_order_by_date(a_t1, a_t2 : G) : BOOLEAN is
	-- target order by date
		do
			Result := (a_t1 /= Void and a_t2 /= Void) and then a_t1.date < a_t2.date
		end

	compare_date(a_date_1, a_date_2 : ROSE_DATE) : BOOLEAN is
	-- date comparison
		do
			Result := a_date_1.ccyymmdd < a_date_2.ccyymmdd
		end

	test_report(a_entity_id : INTEGER; a_mnemonic : STRING) is
	-- test report
		local
			l_entity : like entity_anchor
			l_sig : like ur_target
			l_sigs : ROSE_LINEAR_ARRAY[like ur_target]
			l_array : like double_array
			--l_currency_array : ROSE_LINEAR_ARRAY[CURRENCY]
		do
			l_entity := entity_from_id(a_entity_id)
			if l_entity /= Void then
				create l_sig
				l_sigs := targets(l_entity)
				l_array := double_array(l_sigs.item(1))
				l_array := double_series(l_sigs, a_mnemonic)
--				l_currency_array := currency_series(l_sigs)
			end
		end

feature {NONE} -- templates
	expression_template : SIGNATURE_TAGGED_EXPRESSION[G] is
	-- expression template
	do  end

	entity_anchor : H is
	-- entity anchor
	do  end
end
