note

	description:

		"Array of expressions with facilities for displaying values at targets"

	last_updater: "$Author: mb $"
	author: "Michael Zhang"
	last_editor: "$Author: mb $"
	date: "$Date: 2012/11/14 $"
	revision: "$Revision: #23 $"
	path: "$File: //rose/source/kernel/data/reports/reference/rose_simple_report.e $"

class ROSE_SIMPLE_REPORT [reference G -> ROSE_EXPRESSION_TARGET create default_create end]

inherit

	ROSE_CONSTANTS
		export {NONE} all end

	EXCEPTIONS
		export {NONE} all end

	REPORT_RECORDS
		rename
			make as report_records_make
		end

	NEEDS_EXPRESSION_FROM_STRING

create

	make

feature {NONE} -- Initialization

	make (a_expressions: ENTITY_VARIABLES [ROSE_EXPRESSION_TARGET]; a_report_name: STRING; a_report_type: STRING) is
			--create
		local
			l_mnemonic: STRING
			l_mnemonics: ROSE_LINEAR_ARRAY [STRING]
			l_expression: like expression_template
				-- ROSE_EXPRESSION[G]
		do
			report_name := a_report_name
			report_type := a_report_type
			load_records (a_report_name)
			if records /= Void and then records.count > 0 then
				create l_mnemonics.make_from_array (records.current_keys)
				l_mnemonics.sort_by (agent compare_string)
				count := records.count
				create expressions.make (1, count)
				from
					l_mnemonics.start
				until
					l_mnemonics.off
				loop
					l_mnemonic := l_mnemonics.current_item
					l_mnemonic.to_lower
					l_expression ?= a_expressions.complex_expression (l_mnemonic)
					if valid_expression (l_expression) then
						expressions.put (l_expression, l_mnemonic, l_mnemonics.index)
					else
						l_expression ?= a_expressions.interpretation (a_report_type, l_mnemonic)
						if valid_expression (l_expression) then
							expressions.put (l_expression, l_mnemonic, l_mnemonics.index)
						end
					end
					l_mnemonics.forth
				end
			end
		end

feature --Access

	count: INTEGER

	report_name: STRING

	report_type: STRING

	expressions: ROSE_INDEXED_ARRAY [like expression_template, STRING]

	mnemonics: ROSE_LINEAR_ARRAY [STRING] is
			--mnemonics
		do
			Result := expressions.keys
		end

	targets (a_entity: like entity_anchor): ROSE_LINEAR_ARRAY [G] is
			--do nothing
		do
		end

	targets_in_interval (a_entity: like entity_anchor; a_start_date, a_end_date: ROSE_DATE): ROSE_LINEAR_ARRAY [G] is
		do
		end

	set_entity_neighborhood (a_entities: ARRAY [ENTITY]) is
		do
		end

	is_monthly: BOOLEAN

	set_monthly is
		do
			is_monthly := True
		end

	set_daily is
		do
			is_monthly := False
		end

feature -- double

	double_item (a_expression: like expression_template; a_target: G): DOUBLE is
			-- The double value for `a_expression' evaluated at `a_target'
		local
			l_date: ROSE_DATE
		do
			if a_target /= Void then
				if a_expression /= Void then
					if a_expression.is_double then
						Result := a_expression.double_value (a_target)
					elseif a_expression.is_real then
						Result := a_expression.real_value (a_target)
					elseif a_expression.is_integer then
						Result := a_expression.integer_value (a_target)
					elseif a_expression.is_date then
						l_date := a_expression.date_value (a_target)
						if l_date /= Void then
							Result := l_date.ccyymmdd
						else
							Result := constants.missing_double
						end
					else
						Result := constants.missing_double
					end
				end
			else
				Result := constants.missing_double
			end
		end

	double_series (a_targets: ROSE_LINEAR_ARRAY [G]; a_mnemonic: STRING): VECTOR is
		local
			l_ix: INTEGER
			l_mnemonic: STRING
			l_expression: like expression_template
		do
			l_mnemonic := a_mnemonic
			l_ix := index (l_mnemonic)
			if l_ix >= 1 and l_ix <= count then
				l_expression := expressions.item (l_ix)
				if l_expression /= Void and then (l_expression.is_double or l_expression.is_real or l_expression.is_integer) then
					Result := l_expression.vector (a_targets)
					if Result /= Void and then Result.is_null then
						Result := Void
					end
				else
					Result := Void
				end
			else
				Result := Void
			end
		end

	double_array (a_target: G): VECTOR is
		local
			i: INTEGER
			l_count: INTEGER
			l_double: DOUBLE
			l_expression: like expression_template
		do
			if a_target /= Void then
				create Result.make (count)
				from
					i := 1
				until
					i > count
				loop
					l_expression := expressions.item (i)
					if l_expression /= Void then
						l_double := double_item (l_expression, a_target)
						if not constants.defined_double (l_double) then
							l_count := l_count + 1
						end
						Result.put (l_double, i)
					else
						l_count := l_count + 1
						Result.put (constants.missing_double, i)
					end
					i := i + 1
				end
				if l_count = count then
					Result := Void
				end
			else
				Result := Void
			end
		end

feature --Conversion

	double_matrix (a_targets: ROSE_LINEAR_ARRAY [G]): MATRIX is
		local
			i: INTEGER
			l_array: like double_array
			l_target: G
		do
			create Result.make (a_targets.count, count)
			Result.set_column_names (expressions.keys)
			from
				i := 1
				a_targets.start
			until
				a_targets.off
			loop
				l_target := a_targets.current_item
				l_array := double_array (l_target)
				if l_array /= Void then
					Result.put_row_vector (l_array, i)
				else
					Result.set_row_null (i)
				end
				i := i + 1
				a_targets.forth
			end
		end

	dated_matrix (a_targets: ROSE_LINEAR_ARRAY [G]): DATED_MATRIX is
		local
			l_matrix: like double_matrix
			l_dates: like date_array_from_targets
		do
			if a_targets /= Void and then a_targets.count > 0 then
				l_dates := date_array_from_targets (a_targets)
				l_matrix := double_matrix (a_targets)
				create Result.make_from_matrix (l_matrix)
				Result.set_dates (l_dates)
			end
		end

	matrix (a_entity: like entity_anchor): DATED_MATRIX is
		local
			l_targets: like targets
		do
			l_targets := targets (a_entity)
			Result := dated_matrix (l_targets)
		end

	matrix_in_interval (a_entity: like entity_anchor; a_start_date, a_end_date: ROSE_DATE): DATED_MATRIX is
		require
			valid_interval: (a_start_date /= Void and a_end_date /= Void) and then a_start_date <= a_end_date
		local
			l_targets: like targets
		do
			l_targets := targets_in_interval (a_entity, a_start_date, a_end_date)
			Result := dated_matrix (l_targets)
		end

	date_array_from_targets (a_targets: like targets): ROSE_LINEAR_ARRAY [ROSE_DATE] is
		local
			i, l_lower: INTEGER
			l_date_time: ROSE_DATE
			l_date: ROSE_DATE
			l_target: like ur_target
		do
			if a_targets /= Void and then a_targets.count > 0 then
				l_lower := a_targets.lower
				create Result.make (1, a_targets.count)
				from
					i := 1
				until
					i > a_targets.count
				loop
					l_target := a_targets.item (l_lower + i - 1)
					if l_target /= Void then
						l_date_time ?= l_target.date
						if l_date_time /= Void then
							create l_date.make_ccyymmdd (l_date_time.ccyymmdd)
							Result.put (l_date, i)
						else
							Result.put (l_target.date, i)
						end
					end
					i := i + 1
				end
			end
		end

feature -- date

	date_item (a_expression: like expression_template; a_target: G): ROSE_DATE is
		do
			if a_expression /= Void and then a_target /= Void then
				Result := a_expression.date_value (a_target)
			end
		end

	date_series (a_targets: ROSE_LINEAR_ARRAY [G]; a_mnemonic: STRING): ROSE_LINEAR_ARRAY [ROSE_DATE] is
		local
			l_ix: INTEGER
			l_target: G
			l_date: ROSE_DATE
			l_mnemonic: STRING
			l_expression: like expression_template
			l_count: INTEGER
		do
			l_mnemonic := a_mnemonic
			l_ix := index (l_mnemonic)
			if l_ix >= 1 and l_ix <= count then
				l_expression := expressions.item (l_ix)
				if l_expression /= Void and then l_expression.is_date then
					create Result.make_empty
					l_count := 1
					from
						a_targets.start
					until
						a_targets.off
					loop
						l_target := a_targets.current_item
						if l_target /= Void then
							l_date := date_item (l_expression, l_target)
							if l_date = Void or (l_date /= Void and then l_date.ccyymmdd = constants.minimum_date_time.ccyymmdd) then
								l_count := l_count + 1
							end
							Result.extend (l_date)
						else
							l_count := l_count + 1
							Result.extend (constants.minimum_date_time)
						end
						a_targets.forth
					end
					if l_count = a_targets.count then
						Result := Void
					end
				else
					Result := Void
				end
			else
				Result := Void
			end
		end

feature -- currency

	currency_item (a_expression: like expression_template; a_target: G): PERSISTENT_CURRENCY is
			-- Currency
		do
			if a_expression /= Void and then a_target /= Void then
				Result ?= a_expression.currency (a_target)
			end
		end

	currency_series (a_targets: ROSE_LINEAR_ARRAY [G]; a_mnemonic: STRING): ROSE_LINEAR_ARRAY [PERSISTENT_CURRENCY] is
		local
			l_ix: INTEGER
			l_target: G
			l_currency: PERSISTENT_CURRENCY
			l_mnemonic: STRING
			l_expression: like expression_template
		do
			l_mnemonic := a_mnemonic
			l_ix := index (l_mnemonic)
			if l_ix >= 1 and l_ix <= count then
				l_expression := expressions.item (l_ix)
				if l_expression /= Void then
					create Result.make_empty
					if a_targets /= Void and then a_targets.count > 0 then
						from
							a_targets.start
						until
							a_targets.off
						loop
							l_target := a_targets.current_item
							if l_target /= Void then
								l_currency := currency_item (l_expression, l_target)
								Result.extend (l_currency)
--								Result.extend(l_currency, l_target.date)
							else
								Result.extend (Void)
--								Result.extend(Void, l_target.date)
							end
							a_targets.forth
						end
					end
				else
					Result := Void
				end
			else
				Result := Void
			end
		end

	currency_matrix (a_targets: ROSE_LINEAR_ARRAY [G]): ROSE_ARRAY2 [PERSISTENT_CURRENCY] is
		local
			i: INTEGER
			l_array: ROSE_LINEAR_ARRAY [PERSISTENT_CURRENCY]
			l_mnemonic: STRING
		do
			create Result.make (a_targets.count, count)
			from
				i := 1
			until
				i > count
			loop
				l_mnemonic := expressions.key (i)
				l_array := currency_series (a_targets, l_mnemonic)
				if l_array /= Void then
					Result.insert_column (i, l_array)
				end
				i := i + 1
			end
		end

feature --Targets

	ur_target: detachable G

feature {NONE} --Implementation

	index (a_mnemonic: STRING): INTEGER is
		do
			if a_mnemonic /= Void then
				a_mnemonic.to_lower
				Result := expressions.index_at (a_mnemonic)
			end
		end

	compare_string (a_string_1, a_string_2: STRING): BOOLEAN is
		do
			Result := a_string_1 < a_string_2
		end

	valid_expression (a_expression: ROSE_EXPRESSION [ROSE_EXPRESSION_TARGET]): BOOLEAN is
			-- Is a_expression a valid expression?
		do
			Result := a_expression /= Void and then not a_expression.name.has_substring ("Unknown")
		end

	ur_expression: like expression_template

	expression_template: ROSE_EXPRESSION [G] is
		do
		end

	entity_anchor: ENTITY is
		do
		end

end
