indexing
	description: "Splash screen"
	author: "Zoran Simic"
	date: "$Date: 2009/09/15 $"
	revision: "$Revision: #13 $"

class
	ROSE_SPLASH_SCREEN

inherit
	ROSE_SHARED_GLOBAL_GUI
		redefine
			default_create
		end

feature -- Initialization

	default_create is
			-- Default creation procedure.
		do
			set_title (global.app.name)
			set_sub_title (global.formatted ("$1%NVersion $2",<<
				global.app.description,
				global.app.version
			>>))
			quick_report := global.formatted ("$1%NHost: $2, user: $3%NCompiled $4 GMT",<<
				global.app.program_path,
				global.app.host_name,
				global.app.user.full_login,
				global.app.program_date
			>>)
			bg := Gui.factory.new_pixmap_from_file ("splash_screen.png")
			width := 638
			height := 300
		end

feature -- Access

	dialog: EV_UNTITLED_DIALOG
			-- Dialog used to show the splash screen

	pixmap: EV_PIXMAP
			-- Pixmap shown as splash screen

	title: STRING
			-- Title to show

	sub_title: STRING
			-- Sub-title

	width, height: INTEGER
			-- Desired dimensions of the splash screen

	delay: INTEGER
			-- Time (in milliseconds) the splash screen will stay visible

	determined_title: STRING is
			-- Title for the splash screen
		do
			Result := title
			if Result = Void then Result := global.app.name end
			if Result = Void then Result := global.app.program_name end
			if Result = Void then Result := "" end
		end

feature -- Basic operations

	show is
			-- Show splash screen for 'a_delay' milliseconds and with 'a_title'.
			-- Splash screen stays forever if 'a_delay' <= 0.
		local
			l_title: STRING
			l_icon: EV_PIXMAP
		do
			hide
			global.busy_event.extend (agent adapt_busy_title)
			create dialog
			dialog.disable_user_resize
			dialog.set_size (width,height)
			dialog.set_position ((Gui.screen.width - dialog.width) // 2,(Gui.screen.height - dialog.height) // 2)
			create pixmap
			l_title := determined_title
			dialog.set_title (l_title)
			if global.app.info.icon /= Void and then not global.app.info.icon.is_empty then
				l_icon := gui.factory.new_pixmap_from_file (global.app.info.icon)
				if l_icon /= Void then
					dialog.set_icon_pixmap (l_icon)
				end
			end
			compose_pixmap
			dialog.show
			dialog.set_focus
			dialog.set_pointer_style (Gui.new_busy_cursor (Gui.rep.pixmaps.wait_cursor, "Starting " + l_title))
			Gui.ev_application.process_events
		end

	hide is
			-- Hide the splash screen.
		do
			if not Gui.ev_application.is_destroyed
			and then Gui.ev_application.shift_pressed
			and then Gui.ev_application.ctrl_pressed
			and then Gui.ev_application.alt_pressed then
				if dialog /= Void then
					dialog.key_release_actions.extend (agent on_key)
				end
			elseif dialog /= Void then
				close_dialog
			end
			global.busy_event.remove (agent adapt_busy_title)
		end

	set_size (a_width, a_height: INTEGER) is
			-- Set 'width' and 'height'.
		require
			width_ok: a_width > 0
			height_ok: a_height > 0
		do
			width := a_width
			height := a_height
		ensure
			width_set: width = a_width
			height_set: height = a_height
		end

	set_title (a_title: STRING) is
			-- Set 'title'.
		do
			if a_title /= Void then
				title := a_title
			end
		end

	set_sub_title (a_sub_title: STRING) is
			-- Set 'sub_title'.
		do
			if a_sub_title /= Void then
				sub_title := a_sub_title
			end
		end

feature {NONE} -- Implementation

	bg : EV_PIXMAP				-- Background pixmap
	quick_report: STRING		-- Quick report

	k_margin: INTEGER is 10				-- Margin
	k_min_text_height: INTEGER is 6		-- Minimum text height

	close_dialog is
			-- Close 'dialog'.
		do
			if dialog /= Void then
				dialog.set_pointer_style (Gui.rep.pixmaps.standard_cursor)
				dialog.destroy
				dialog := Void
			end
		end

	adapt_busy_title (a_stopwatch: ROSE_STOPWATCH) is
			-- Display busy reason
		local
			l_title, l_message: STRING
		do
			if dialog /= Void then
				create l_title.make_empty
				l_title.append_string (determined_title)
				if a_stopwatch = Void then
					-- Do nothing
				elseif a_stopwatch.is_running then
					l_message := a_stopwatch.message
				elseif not global.busy_stack.is_empty and then global.busy_stack.item /= Void then
					l_message := global.busy_stack.item.message
				end
				if l_message /= Void then
					l_title.append_character (' ')
					l_title.append_character ('-')
					l_title.append_character (' ')
					l_title.append_string (l_message)
				end
				dialog.set_title (l_title)
			end
		end

	compose_pixmap is
			-- Compose 'pixmap'.
		require
			pixmap_ok: pixmap /= Void
		local
			l_y1, l_y2, l_y3, l_indent : INTEGER
			retried: BOOLEAN
		do
			if not retried then
				pixmap.set_size (width,height)
				pixmap.set_background_color (Gui.rep.colors.White)
				pixmap.clear

				l_y1 := 14
				l_y2 := 88
				l_y3 := 238
				l_indent := 40

				if bg /= Void then
					pixmap.draw_pixmap (0, 0, bg)
				end

				show_text (
					title,
					l_indent,
					l_y1,
					Gui.rep.fonts.new_font (True,False,48),
					Gui.rep.colors.rgb (0, 51, 102)
				)
				show_text (
					sub_title,
					l_indent,
					l_y2,
					Gui.rep.fonts.new_screen_font (False,False,14),
					Gui.rep.colors.rgb (0, 51, 102)
				)
				show_text (
					quick_report,
					l_indent,
					l_y3,
					Gui.rep.fonts.new_screen_font (False,False,11),
					Gui.rep.colors.Dark_gray
				)
				-- Frame
				pixmap.set_foreground_color (Gui.rep.colors.rgb (0, 51, 102))
				pixmap.draw_rectangle (0,0,width,height)
				dialog.set_background_pixmap (pixmap)
			end
		rescue
			if not retried then
				retried := True
				retry
			end
		end

	show_text (a_text: STRING; a_x, a_y: INTEGER; a_font: EV_FONT; a_color: EV_COLOR) is
			-- Show 'a_text' at coordinates ('a_x', 'a_y') with 'a_font' and 'a_color'.
		local
			i, l_x, l_y, l_w, l_h, l_maxw, l_maxh: INTEGER
			l_size: TUPLE [INTEGER,INTEGER]
		do
			if a_text /= Void and then not a_text.is_empty then
				l_maxw := width - 2 * k_margin
				l_maxh := height - 2 * k_margin
				from
					i := a_font.height
					a_font.set_height (i)
					l_size := a_font.string_size (a_text)
					l_w := l_size.integer_item (1)
					l_h := l_size.integer_item (2)
				until
					i <= k_min_text_height
					or else (l_w < l_maxw and then l_h < l_maxh)
				loop
					a_font.set_height (i)
					l_size := a_font.string_size (a_text)
					l_w := l_size.integer_item (1)
					l_h := l_size.integer_item (2)
					i := i - 1
				end
				if i < k_min_text_height then
					i := k_min_text_height
					a_font.set_height (i)
					l_size := a_font.string_size (a_text)
					l_w := l_size.integer_item (1)
					l_h := l_size.integer_item (2)
				end
				pixmap.set_font (a_font)
				pixmap.set_foreground_color (a_color)
				l_x := a_x.abs
				l_y := a_y.abs
				if a_x < 0 then
					l_x := l_x - l_w
				else
					l_x := l_x.max (k_margin)
				end
				if a_y < 0 then
					l_y := l_y - l_h
				else
					l_y := l_y.max (k_margin)
				end
				pixmap.draw_text_top_left (l_x,l_y,a_text)
			end
		end

	on_key (a_key: EV_KEY) is
			-- Called when 'a_key' is pressed.
		do
			if a_key /= Void and then a_key.code > 0 then
				if a_key.code = Gui.rep.keys.key_escape then
					close_dialog
				end
			end
		end

end
