indexing
	description: "Last-in first-out stack cache"
	author: "Zoran Simic"
	date: "$Date: 2008/12/01 $"
	revision: "$Revision: #2 $"
	archive: "$File: //rose/source/kernel/infrastructure/framework/support/rose_stack.e $"

class
	ROSE_STACK [G]

inherit
	DEBUG_OUTPUT
		redefine
			default_create
		end

feature {NONE} -- Initialization

	default_create is
			-- Default creation procedure
		do
			create array.make (1, 0)
		end

feature -- Access

	count: INTEGER
			-- Number of items on stack

	item: G is
			-- Item on top of the stack
		do
			if count > 0 then
				Result := array.item (count)
			end
		end

	i_th (a_i_th: INTEGER): like item is
			-- 'a_i_th' item of the stack (last item 'count' is on top of stack)
		require
			i_ok: a_i_th > 0 and then a_i_th <= count
		do
			Result := array.item (a_i_th)
		end

	is_empty: BOOLEAN is
			-- Is this stack currently empty?
		do
			Result := count = 0
		end

	debug_output: STRING is
			-- String that should be displayed in debugger to represent `Current'.
		do
			Result := count.out + once " items"
		end

feature -- Basic operations

	push, put, extend (a_item: G) is
			-- Put 'a_item' on the stack.
		do
			array.force (a_item, count + 1)
			count := count + 1
		end

	pop, remove is
			-- Remove item associated with `key', if present.
		require
			not_empty: count > 0
		local
			l_default: like item
		do
			array.put (l_default, count)
			count := count - 1
		end

	do_all (a_agent: PROCEDURE [ANY, TUPLE [G]]) is
			-- Iterate 'a_agent' over all items
		require
			agent_ok: a_agent /= Void
		local
			i: INTEGER
		do
			from
				i := 1
			until
				i > count
			loop
				a_agent.call ([array.item (i)])
				i := i + 1
			end
		end

feature {NONE} -- Implementation

	array: ARRAY [like item]
			-- Array containing the items on this stack

invariant

	valid_count: count >= 0
	valid_array: array /= Void

end
