indexing
	description: "Part of ROSE_STATUS"
	author: "Zoran Simic"
	date: "$Date: 2008/12/01 $"
	revision: "$Revision: #2 $"

class
	ROSE_STATUS_MESSAGE

inherit
	ROSE_SHARED_GLOBAL
	DEBUG_OUTPUT

create
	make, make_output, make_warning, make_error

feature {NONE} -- Implementation

	make (a_level: INTEGER; a_text: STRING) is
			-- Make with 'a_level' and 'a_text'.
		require
			global.convention.is_valid_verbosity (a_level)
			a_text /= Void
		do
			level := a_level
			text := a_text
		end

	make_output (a_text: STRING) is
			-- Make output-level with 'a_text'.
		do
			make (global.convention.v_output, a_text)
		end

	make_warning (a_text: STRING) is
			-- Make warning-level with 'a_text'.
		do
			make (global.convention.v_warning, a_text)
		end

	make_error (a_text: STRING) is
			-- Make error-level with 'a_text'.
		do
			make (global.convention.v_error, a_text)
		end

feature -- Access

	level: INTEGER
			-- Verbosity level

	text: STRING
			-- Text

	is_error: BOOLEAN is
			-- Is this message an error-level message?
		do
			Result := level = global.convention.v_error
		end

	is_warning: BOOLEAN is
			-- Is this message a warning-level message?
		do
			Result := level = global.convention.v_warning
		end

	is_ok: BOOLEAN is
			-- Is this message OK (not an error and not a warning)?
		do
			Result := level = global.convention.v_output
		end

	representation: STRING is
			-- String representing `Current' with prefixes for errors and warnings
		do
			if text /= Void then
				if is_error then
					Result := "** " + text
				elseif is_warning then
					Result := "!! " + text
				else
					Result := text
				end
			end
		end

	debug_output: STRING is
			-- String that should be displayed in debugger to represent `Current'.
		do
			create Result.make_empty
			Result.append_character ('[')
			Result.append_string (global.convention.verbosity_meaning (level))
			Result.append_character (']')
			Result.append_character (' ')
			Result.append_string (text)
		end

end
