note
   description: "Rose extensions to Eiffel base class STORABLE"
   author: "Mark Howard"
   last_editor: "$Author: er $"
   date: "$Date: 2012/11/05 $";
   revision: "$Revision: #30 $"
   path: "$File: //rose/source/kernel/infrastructure/persistence/storable/rose_storable.e $"

class
	ROSE_STORABLE

inherit
	ANY
	STORABLE
		rename
			retrieved as do_not_use__retrieved,
			retrieve_by_name as do_not_use__retrieve_by_name,
			basic_store as do_not_use__basic_store,
			general_store as do_not_use__general_store,
			independent_store as do_not_use__independent_store,
			store_by_name as do_not_use__store_by_name,
			exception as inherited_exception_code
		export
			{NONE} all
		end

	ROSE_SHARED_GLOBAL
		export
			{NONE} all
		end

feature -- Access

	medium: IO_MEDIUM
			-- obsolete; will be removed

feature -- Status

	failed: BOOLEAN
			-- Did last retrieve fail?

	successfully_retrieved: BOOLEAN is
			-- Was last retrieve succesful?
		do
			Result := not failed
		end

feature -- Basic operations

	store_as (a_filename: STRING) is
			-- store (independent) Current in file `a_filename'
		require
			valid_name: a_filename /= Void
		do
			global.persist.store (Current, a_filename)
		end

	store_at (a_object: ANY; a_filename: STRING) is
			-- store (independent) `a_object' in file name `a_filename'
		require
			valid_name: a_filename /= Void
		do
			global.persist.store (a_object, a_filename)
		end

	retrieve_from_file, retrieve_from (a_filename: STRING) is
			-- Retrieve 'Current' from 'a_filename'
		require
			valid_path: a_filename /= Void
		local
			l_stored: like Current
			l_collecting, retried: BOOLEAN
			l_memory: MEMORY
		do
			if not retried then
				create l_memory
				l_collecting := l_memory.collecting
				l_memory.full_coalesce
				l_memory.collection_off
				l_stored ?= global.persist.retrieved (a_filename)
				if l_collecting then
					l_memory.collection_on
				end
				l_memory.full_coalesce
				if l_stored /= Void then
					copy (l_stored)
				else
					retried := True
				end
			end
			failed := retried
		rescue
			if l_collecting then
				l_memory.collection_on
			end
			retried := True
			retry
		end

end








