note

	description:

		"Storable as file"

	author: "Mark Howard"
	last_editor: "$Author: er $"
	date: "$Date: 2012/10/26 $"
	revision: "$Revision: #24 $"
	path: "$File: //rose/source/kernel/persistence/rose_storable_as_file.e $"

deferred class
	ROSE_STORABLE_AS_FILE

inherit
	ROSE_STORABLE

feature -- Access

	unix : INTEGER is unique		-- Unix
	vms : INTEGER is unique			-- VMS
	windows : INTEGER is unique		-- Windows

	localize (l_name : STRING) is
		local
			l_tuple : ARRAY[STRING]
		do
			l_tuple := localized (l_name)
			stored_directory := l_tuple.item (1)
			stored_file_name := l_tuple.item (2)
		end

	localized (l_name : STRING) : ARRAY [STRING] is
		local
			l_separator : CHARACTER
			l_index : INTEGER
			i,l_system : INTEGER
			l_stored_directory,l_stored_file_name : STRING
		do
			create Result.make(1,2)
			l_stored_directory := clone(l_name)
			l_separator := operating_environment.directory_separator
			inspect l_separator
			when '/','.' then
				if l_separator = '/' then
					l_system := unix
				else
					l_system := vms
				end
				l_separator := '/'
				l_stored_directory.replace_substring_all("\","/")
				--strip unc
				if l_stored_directory.substring_index("//",1) = 1 then
					l_stored_directory.keep_tail(l_stored_directory.count-2)
					l_index := l_stored_directory.index_of('/',1)
					if l_index > 0 then
						l_stored_directory.keep_tail(l_stored_directory.count-l_index+1)
					end
				end
			when '\' then
				l_system := windows
				if l_stored_directory.substring_index("\\",1) /= 1 and then
					l_stored_directory.substring_index("//",1) /= 1 then
					l_stored_directory.replace_substring_all("\","/")
					l_separator := '/'
					if l_stored_directory.item(1) = '/' then
						l_stored_directory.prepend("//pinegv")
					end
				end
			end
			l_stored_file_name := clone(l_stored_directory)
			from
				i := l_stored_directory.index_of(l_separator,1)
			until
				i >= l_stored_directory.count or l_stored_directory.index_of(l_separator,i+1) = 0
			loop
				i := l_stored_directory.index_of(l_separator,i+1)
			end
			if i > 0 then
				l_stored_directory.keep_head(i)
				l_stored_file_name.keep_tail(l_stored_file_name.count-i)
			end
			inspect l_system
			when unix then
				l_stored_directory.replace_substring_all("(dependent_store)","/eiffel/storables/unix")
			when vms then
				l_stored_directory.replace_substring_all("(dependent_store)","/eiffel/storables/vms")
			when windows then
				l_stored_directory.replace_substring_all("(dependent_store)","/eiffel/storables/nt")
			end
			if l_system = vms then
				l_stored_directory.put(' ',1)
				l_stored_directory.put(']',l_stored_directory.count)
				i := l_stored_directory.index_of('/',1)
				l_stored_directory.replace_substring(l_stored_directory.substring(2,i-1),1,i-2)
				l_stored_directory.put(':',i-1)
				l_stored_directory.put('[',i)
				l_stored_directory.replace_substring_all("/",".")
				l_stored_directory.to_upper
				l_stored_file_name.to_upper
				if not l_stored_file_name.has('.') then
					l_stored_file_name.append_character('.')
				end
			end
			Result.put(l_stored_directory,1)
			Result.put(l_stored_file_name,2)
		end

	stored_file_name : STRING
	stored_directory : STRING

	localized_name : STRING is
		local
			l_tuple : ARRAY[STRING]
		do
			l_tuple := localized(stored_name)
			Result := l_tuple.item(1)
			Result.append(l_tuple.item(2))
		end

	store is
			-- Store current object in file `localized_name'.
		do
			store_as (localized_name)
		end

	retrieve is
		do
			retrieve_from (localized_name)
			successful_retrieval := not failed
		end

	stored_name : STRING is
		deferred
		end

feature {NONE} -- Implementation

	path_names : HASH_TABLE [STRING,STRING] is
		local
			l_predicate : SIMPLE_PREDICATE [STORABLE_PATH]
			l_servers : DATABASE_SERVERS
		once
			create Result.make (200)
			create l_servers
			create l_predicate.make (l_servers.storable_path_server,"select * from paths")
			l_predicate.apply (agent add_path (?),Void,Void)
		end

	add_path (a_item : STORABLE_PATH) is
		do
			a_item.path.prune_all (' ')
			a_item.name.prune_all (' ')
			path_names.put (a_item.path,a_item.name)
		end

	path (a_name : STRING) : STRING is
		do
			Result := path_names.item (a_name)
		end

	successful_retrieval : BOOLEAN

end
