indexing

	description: "STREAM class with facilities for storing and retrieving from strings"

	author: "Randy John"
	last_editor: "$Author: it $"
	date: "$Date: 2008/12/01 $"
	revision: "$Revision: #5 $"
	path: "$File: //rose/source/kernel/infrastructure/system/rose_stream_base.e $"
	
deferred class	ROSE_STREAM_BASE
	
inherit

	STREAM
		redefine
			default_create
		end
	
feature -- Initialization
	
	default_create is
			-- Create with default size.
		do
			make
		end
	
	make_from_string (a_string: STRING) is
			-- Create from `a_string'.
		require
			string_not_void : a_string /= Void
		local
			l_any: ANY
		do
			make_with_size(a_string.count)
			l_any := a_string.area
			item.memory_copy($l_any, a_string.count)
		end
	
	make_from_streams(a_streams: LINEAR [STREAM]) is
		-- Create from `a_streams'.
		require
			streams_not_void : a_streams /= Void
		local
			l_current_pointer: POINTER
			l_object_stored_size: INTEGER
			l_buffer_size: INTEGER
			l_collecting: BOOLEAN
		do
			l_collecting := memory.collecting
			memory.collection_off
			
			from
				a_streams.start
			until
				a_streams.off
			loop
				l_object_stored_size := a_streams.item.object_stored_size
				l_buffer_size := a_streams.item.buffer_size
				if l_object_stored_size=0
				 then
					l_object_stored_size := l_buffer_size
				end
				
				object_stored_size := object_stored_size + l_object_stored_size
				a_streams.forth
			end
			
			make_with_size(object_stored_size)
			
			from
				a_streams.start
				l_current_pointer := item
			until
				a_streams.off
			loop
				l_object_stored_size := a_streams.item.object_stored_size
				l_buffer_size := a_streams.item.buffer_size
				if l_object_stored_size=0
				 then
					l_object_stored_size := l_buffer_size
				end
				
				l_current_pointer.memory_copy(a_streams.item.item, l_object_stored_size)
				l_current_pointer := l_current_pointer + l_object_stored_size
				a_streams.forth
			end
			
			if l_collecting then
				memory.collection_on
			end
		rescue
			if l_collecting then
				memory.collection_on
			end
		end
	
feature -- Access
	
	retrieve_from (a_pointer: POINTER; a_size: INTEGER): ANY is
		deferred
		end
	
	retrieved_size: INTEGER
	
	retrieve_from_string (a_string: STRING): ANY is
		local
			l_any: ANY
		do
			l_any := a_string.area
			Result := retrieve_from ($l_any,a_string.count)
		end
	
	string_representation: STRING is
			-- printable representation of `Current'
		local
			l_collecting: BOOLEAN
		do
			if item /= default_pointer and then object_stored_size > 0 then
				l_collecting:= memory.collecting
				memory.collection_off
				create Result.make (object_stored_size)
				Result.from_c_substring (item, 1, object_stored_size)
				if l_collecting then
					memory.collection_on
				end
			end
		rescue
			if l_collecting then
				memory.collection_on
			end
		end
	
	access_externals: ACCESS_EXTERNALS is once create Result end
	
feature {NONE}
	
	memory: MEMORY is
		once
			create Result
		end
	
end
