indexing
	description: "Management of available implementors in the system"
	notice: "[
		This class provide management for ROSE_SYSTEM_DEPENDECY implementers.
		A ROSE_SYSTEM_DEPENDECY lists the implementors made available statically
		to the system. The framework uses only interfaces to provide services such as :
			- commands
			- channels
			- configuration handlers
		Specific implementors are not supposed to be refered to directly by any code
		except the descendants of ROSE_SYSTEM_DEPENDECY. The different available pilots
		choose which implementors to include, the root class chooses the appropriate pilot.
		If some code needs a specific service, it should not refer to that service directly,
		but 'register' the appropriate dependency via this manager (which is a singleton
		available via 'Global.dependency_manager' (in class ROSE_SHARED_GLOBAL)
	]"
	author: "Zoran Simic"
	date: "$Date: 2009/03/16 $"
	revision: "$Revision: #3 $"

class
	ROSE_SYSTEM_DEPENDENCY_MANAGER

inherit
	ROSE_SINGLETON

create
	make

feature {NONE} -- Initialization

	make is
			-- Creation procedure
		do
			check_instance_unicity
		end

feature -- Access

	registered: HASH_TABLE [INTEGER,STRING] is
			-- Registered dependencies, and the number of times they were registered
		once
			create Result.make (0)
		end

feature -- Basic operations

	register (a_dependency: ROSE_SYSTEM_DEPENDENCY) is
			-- Register 'a_dependency'
		local
			l_generator: STRING
			l_count: INTEGER
		do
			l_generator := a_dependency.generator
			l_count := registered.item (l_generator)
			registered.put (l_count+1,l_generator)
			if l_count = 0 then
				a_dependency.register
			end
		end

end
