indexing

	description: "[
                  Helper objects for selecting an expression target.
                  This default implementation provides suitable
                  defaults for use with MONDAT and BLOOMBERG companies.
                 ]"

	keywords: "expression signature target mondat bloomberg"
	url: "http://prism/modules/wacko/wakka.php?wakka=ExpressionLanguage"
	author: "$Author: mk $ Colin Adams"
	group: "RC London (tools)"
	reviewed_by: "No-one"
	quality: "Unrated"
	revision: "$Revision: #7 $"
	date: "$Date: 2009/06/23 $"
	archive: "$File: //rose/source/kernel/math/rose_target_selector.e $"

class ROSE_TARGET_SELECTOR

create

	make,
	make_named

feature {NONE} -- Initialization

	make is
			-- Anonymous target selection:
			-- (`chosen_ur_target' returns single target, iff there is one only)
		do
		ensure
			target_selection_void: target_selection = Void
		end

	make_named (a_target: like target_selection) is
			-- Set target selection using signature class name.
		do
			target_selection := a_target
		ensure
			target_set: target_selection = a_target
		end

	-- TODO: - add periodicity selector? (talk to Mauricio)

feature -- Access

	chosen_ur_target (a_entity: ENTITY): ROSE_TARGET [ENTITY] is
			-- Ur target chosen from `a_entity'
		require
			a_entity_not_void: a_entity /= Void
		local
			l_targets: ROSE_LINEAR_ARRAY [like chosen_ur_target]
			l_result: CELL [like chosen_ur_target]
		do
			l_targets := a_entity.ur_targets
			if target_selection = Void then
					-- Default if no name
				if l_targets.count = 1 then
					Result := l_targets.first
				end
			else
					-- Is O(l_targets.count) performance acceptable?
				create l_result.put (Void)
				-- downcast dynamic type to ROSE_LINEAR_ARRAU [ROSE_TARGET [ENTITY]]
				-- to avoid catcall on do_all.
				-- copy the list, undesirable but still O(l_targets.count)
				create l_targets.make_from_array (l_targets)
				l_targets.do_all (agent chosen_ur_target_one (l_result, ?))
				Result := l_result.item
			end
			-- the behaviour of this routine is not specified
			-- as a postcondition to allow alternative strategies
			-- in descendants.
		end

	target_from_ur_target (a_ur_target: ROSE_TARGET [ENTITY]; a_entity: ENTITY; a_date: ROSE_DATE): ROSE_TARGET [ENTITY] is
			-- Target from `a_ur_target'
		require
			a_ur_target_not_void: a_ur_target /= Void
			a_entity_not_void: a_entity /= Void
			a_date_not_void: a_date /= Void
		local
			l_record, l_record_2: ROSE_SIGNATURE_RECORD [ENTITY]
			l_statement: STRING
			l_targets: ROSE_LINEAR_ARRAY [ROSE_SIGNATURE_RECORD [ENTITY]]
			l_entity_date: ROSE_ENTITY_DATE [ENTITY]
		do
			-- This default implementation searches the database for a signature
			--  and takes the first exact match on entity id and period-end date
			l_record ?= a_ur_target
			if l_record /= Void then
				l_record_2 := l_record.twin
				l_record_2.set_entity (a_entity)
				l_statement := "#entity_id.id = entity_id and period_end_date = '" + a_date.ccyymmdd.out + "'"
				l_targets := l_record.targets (l_record_2.as_targets, l_statement)
				if not l_targets.is_empty then
					Result := l_targets.first
					l_entity_date ?= Result
					if l_entity_date /= Void then
						l_entity_date.set_date (a_date)
					end
				end
			end
		end

feature {NONE} -- Implementation

	target_selection: STRING
			-- Target selection criteria (signature class name).

	chosen_ur_target_one (a_result: CELL [like chosen_ur_target]; a_target: like chosen_ur_target) is
			-- Set `a_target' in `a_result'.
		require
			a_result_not_void: a_result /= Void
			target_selection_not_void: target_selection /= Void
		do
			if a_target /= Void and then a_target.generating_type.same_string (target_selection) then
				a_result.put (a_target)
			end
		end

end

