note

	description:

		"PLAIN_TEXT_FILE which is much faster for large files on VMS"

	author: "Mark Howard"
	last_editor: "$Author: e2 $"
	date: "$Date: 2013/10/01 $"
	revision: "$Revision: #2 $"
	path: "$File: //rose/source/kernel/infrastructure/text/rose_text_file.e $"

class
	ROSE_TEXT_FILE

inherit
	ROSE_COMMON_FILE
	PLAIN_TEXT_FILE
		redefine
			exists,
			open_read,
			open_write,
			open_append,
			open_read_write,
			create_read_write,
			open_read_append
		end
	FILE_ACCESS_LOG

create
	make, make_open_read, make_open_write, make_open_append,
	make_open_read_write, make_create_read_write,
	make_open_read_append, make_with_name

feature	-- Logging

	open_read
			-- Open file in read-only mode.
		do
			Precursor
			record_file_read_access (name)
		rescue
			error.signal (generator, "Can not open_read file '$1'", <<path.name>>)
		end

	open_write
			-- Open file in write-only mode;
			-- create it if it does not exist.
		do
			Precursor
			record_file_write_access (name)
		rescue
			error.signal (generator, "Can not open_write file '$1'", <<path.name>>)
		end

	open_append
			-- Open file in append-only mode;
			-- create it if it does not exist.
		do
			Precursor
			record_file_write_access (name)
		rescue
			error.signal (generator, "Can not open_append file '$1'", <<path.name>>)
		end

	open_read_write
			-- Open file in read and write mode.
		do
			Precursor
			record_file_write_access (name)
		rescue
			error.signal (generator, "Can not open_read_write file '$1'", <<path.name>>)
		end

	create_read_write
			-- Open file in read and write mode;
			-- create it if it does not exist.
		do
			Precursor
			record_file_write_access (name)
		rescue
			error.signal (generator, "Can not create_read_write file '$1'", <<path.name>>)
		end

	open_read_append
			-- Open file in read and write-at-end mode;
			-- create it if it does not exist.
		do
			Precursor
			record_file_write_access (name)
		rescue
			error.signal (generator, "Can not open_read_append file '$1'", <<path.name>>)
		end

feature -- Access

	lines: ROSE_FILE_LINES
			-- The Result is treated like a container of lines of text but they are retrieved 10 MB at a time.
		do
			create Result.make (Current)
		end

	non_empty_lines_array: ROSE_LINEAR_ARRAY[STRING]
			-- Array of non-empty lines in the file. They are all retrieved in one go so it might be slower.
		local
			l_lines: ROSE_FILE_LINES
		do
			l_lines := lines
			create Result.make_empty
			l_lines.do_if (agent Result.extend, agent non_empty_regexp.matches)
		ensure
			non_void_result: Result /= Void
		end

	exists: BOOLEAN
			-- Does this file exist?
		do
			Result := not is_closed or else Precursor
		end

feature -- Basic operations

	reopen
			--closes and reopens the file at the current position, to deal with a "feature" of NT
		local
			l_position: INTEGER
			l_mode: INTEGER
		do
			l_position := position
			l_mode := mode
			close
			inspect l_mode
			when Read_write_file then
				make_open_read_write (path.name)
			when Read_file then
				make_open_read (path.name)
			end
			go (l_position)
		rescue
			error.signal (generator, "Can not reopen file '$1'", <<path.name>>)
		end

	read_all
			-- Read all
		do
			readstream (count)
		rescue
			error.signal (generator, "Can not read_all file '$1'", <<path.name>>)
		end

	write_items (a_array: ARRAY [ANY])
		local
			i: INTEGER
		do
			from
				i := a_array.lower
			until
				i > a_array.upper
			loop
				if a_array.item (i) /= Void then
					put_string (a_array.item (i).out)
				else
					put_string ("<Void>")
				end
				i := i + 1
			end
		rescue
			error.signal (generator, "Can not write_items on file '$1'", <<path.name>>)
		end

feature {NONE} -- Implementation

	rose_create_last_string (a_min_size: INTEGER)
			-- Create new instance of `last_string' with at least `a_min_size'
			-- as capacity.
		do
			create last_string.make (rose_default_last_string_size.max (a_min_size))
		end

	non_empty_regexp: RX_PCRE_REGULAR_EXPRESSION
			-- Regular expression for matching lines with at least one printable character
		once
			create Result.make
			Result.compile("\S") -- Non-whitespace characters
		end

end
