note
	description: "Attribute matching transformation"
	author: "Zoran Simic"
	last_editor: "$Author: zs $"
	date: "$Date: 2011/05/27 $"
	revision: "$Revision: #9 $"
	path: "$File: //rose/source/kernel/entities/transformation/rose_transformation_attribute.e $"

class
	ROSE_TRANSFORMATION_ATTRIBUTE

inherit
	ROSE_TRANSFORMATION

create
	default_create, make

feature {NONE} -- Initialization

	make (a_source, a_target: STRING; a_parent: INTEGER; a_source_attribute, a_target_attribute: STRING)
			-- Create an attribute transformation
		do
			make_transformation (a_source, a_target, a_parent)
			set_source_attribute (a_source_attribute)
			set_target_attribute (a_target_attribute)
		end

feature -- Access

	source_attribute: STRING
	target_attribute: STRING
			-- Mapped attributes

	source_identifier: ENTITY_IDENTIFIER
	target_identifier: ENTITY_IDENTIFIER

	help: STRING =
			-- Help on this transformation
"[
Match entities by their attribute values
Example: get the listings having their 'ticker' matching a company 'name'
ie: source=[RESEARCH]COMPANY:name, target=[RESEARCH]LISTING:ticker
or: [COMPUSTAT]LISTING:ticker to [RESEARCH]LISTING:ticker
]"

feature -- Basic operations

	set_source_attribute (a_source_attribute: STRING)
			-- Set `source_attribute'
		do
			source_attribute := a_source_attribute
			source_identifier := source_category.context.identifier_from_name (source_attribute)
		end

	set_target_attribute (a_target_attribute: STRING)
			-- Set `target_attribute'
		do
			target_attribute := a_target_attribute
			target_identifier := target_category.context.identifier_from_name (target_attribute)
		end

feature {NONE} -- Implementation

	implementor_problem: STRING
			-- Problem with current transformation (Void if valid)
		do
			if source_attribute = Void then
				Result := "No source attribute selected"
			elseif target_attribute = Void then
				Result := "No target attribute selected"
			elseif source_identifier = Void or else source_identifier.handle = 0 then
				Result := "Invalid identifier " + source_attribute + " in source " + source
			elseif not source_category.identifiers.has (source_identifier.handle) then
				Result := "Identifier " + source_attribute + " is not defined in source " + source
			elseif target_identifier = Void or else target_identifier.handle = 0 then
				Result := "Invalid identifier " + target_attribute + " in target " + target
			elseif not target_category.identifiers.has (target_identifier.handle) then
				Result := "Identifier " + target_attribute + " is not defined in target " + target
			end
		end

	specific_attributes: STRING
			-- Specific transformation attributes for 'representation'
		do
			create Result.make (20)
			Result.append_string (",%"")
			Result.append_string (source_attribute)
			Result.append_string ("%",%"")
			Result.append_string (target_attribute)
			Result.append_string ("%"")
		end

	actual_transformed (a_entity: ENTITY): ROSE_LINEAR_ARRAY [ENTITY]
			-- Actual transformation of 'a_entity' ('a_entity' has been checked to be valid)
		local
			l_key: STRING
			l_entity_history: ROSE_ENTITY_HISTORY [ENTITY]
			l_interval: ENTITY_INTERVAL [ENTITY]
		do
			l_key := source_identifier.dated_key (a_entity, Void)
			if l_key /= Void then
				l_entity_history := target_category.entity_history_from_identifier (l_key, target_identifier)
				if l_entity_history /= Void and then not l_entity_history.is_empty then
					from
						l_entity_history.start
						create Result.make_empty
					until
						l_entity_history.off
					loop
						l_interval := l_entity_history.current_item
--						if l_interval.meets (a_start, a_end) and then (a_range = Void or else a_range.has (l_interval.entity)) then
							Result.extend (l_interval.entity)
--						end
						l_entity_history.forth
					end
				end
			end
		end

feature {ROSE_TRANSFORMATION_LIST} -- Implementation

	specific_image (a_chains: ROSE_ENTITY_CHAINS; a_start, a_end: ROSE_DATE; a_range: HASH_TABLE [ENTITY, ENTITY]): ROSE_ENTITY_CHAINS
			-- This is only needed to bridge between old ROSE_ENTITY_CHAINS approach and ROSE_RECIPE
		do
			Result := a_chains.matching_attribute (source_category, source_identifier, target_category,  target_identifier, a_start, a_end, a_range)
		end

end
