note
	description: "Entity-corelationship transformation"
	author: "Zoran Simic"
	last_editor: "$Author: zs $"
	date: "$Date: 2011/05/27 $"
	revision: "$Revision: #8 $"
	path: "$File: //rose/source/kernel/entities/transformation/rose_transformation_corelationship.e $"

class
	ROSE_TRANSFORMATION_CORELATIONSHIP

inherit
	ROSE_TRANSFORMATION

create
	default_create, make

feature {NONE} -- Initialization

	make (a_source, a_target: STRING; a_parent: INTEGER; a_name: STRING)
		do
			make_transformation (a_source, a_target, a_parent)
			set_name (a_name)
		end

feature -- Access

	name: STRING
			-- Name of corelationship

	corelationship_handler: RELATIONSHIP_HANDLER [ENTITY,ENTITY]
			-- Corelationship handler corresponding to 'name'

	help: STRING =
			-- Help on this transformation
"[
Inverse relationship
Obtain related entities using a relationship in reverse order
For example, use 'Company / Security' corelationship to match the company of a given security
]"

feature -- Basic operations

	set_name (a_name: STRING)
			-- Set 'name'
		do
			name := a_name
			corelationship_handler := source_category.context.relationship_handler_from_name (name)
		end

feature {NONE} -- Implementation

	implementor_problem: STRING
			-- Problem with current transformation (Void if valid)
		do
			if name = Void then
				Result := "No corelationship selected"
			elseif corelationship_handler = Void then
				Result := "No corelationship " + name + " in category " + target
			elseif corelationship_handler.h_handler /= source_category then
				Result := "Corelationship " + name + " does not match source " + source
			elseif corelationship_handler.g_handler /= target_category then
				Result := "Corelationship " + name + " does not match target " + target
			end
		end

	specific_attributes: STRING
			-- Specific transformation attributes for 'representation'
		do
			create Result.make (20)
			Result.append_string (",%"")
			Result.append_string (name)
			Result.append_string ("%"")
		end

	actual_transformed (a_entity: ENTITY): ROSE_LINEAR_ARRAY [ENTITY]
			-- Actual transformation of 'a_entity' ('a_entity' has been checked to be valid)
		do
			Result := corelationship_handler.corelated_entities (a_entity)
		end

feature {ROSE_TRANSFORMATION_LIST} -- Implementation

	specific_image (a_chains: ROSE_ENTITY_CHAINS; a_start, a_end: ROSE_DATE; a_range: HASH_TABLE [ENTITY, ENTITY]): ROSE_ENTITY_CHAINS
			-- This is only needed to bridge between old ROSE_ENTITY_CHAINS approach and ROSE_RECIPE
		do
			Result := a_chains.related (corelationship_handler, False, parent + 1, a_start, a_end, a_range)
		end

end
