note
	description: "Entity-relationship transformation"
	author: "Zoran Simic"
	last_editor: "$Author: zs $"
	date: "$Date: 2011/05/27 $"
	revision: "$Revision: #9 $"
	path: "$File: //rose/source/kernel/entities/transformation/rose_transformation_relationship.e $"

class
	ROSE_TRANSFORMATION_RELATIONSHIP

inherit
	ROSE_TRANSFORMATION

create
	default_create, make

feature {NONE} -- Initialization

	make (a_source, a_target: STRING; a_parent: INTEGER; a_name: STRING)
		do
			make_transformation (a_source, a_target, a_parent)
			set_name (a_name)
		end

feature -- Access

	name: STRING
			-- Name of relationship

	relationship_handler: RELATIONSHIP_HANDLER [ENTITY,ENTITY]
			-- Relationship handler corresponding to 'name'

	help: STRING =
			-- Help on this transformation
"[
Obtain related entities
For example, Company / Security gives all securities of a company
]"

feature -- Basic operations

	set_name (a_name: STRING)
			-- Set 'name'
		do
			name := a_name
			relationship_handler := source_category.context.relationship_handler_from_name (name)
		end

feature {NONE} -- Implementation

	implementor_problem: STRING
			-- Problem with current transformation (Void if valid)
		do
			if name = Void then
				Result := "No relationship selected"
			elseif relationship_handler = Void then
				Result := "No relationship " + name + " in category " + source
			elseif relationship_handler.g_handler /= source_category then
				Result := "Relationship " + name + " does not match source " + source
			elseif relationship_handler.h_handler /= target_category then
				Result := "Relationship " + name + " does not match target " + target
			end
		end

	specific_attributes: STRING
			-- Specific transformation attributes for 'representation'
		do
			create Result.make (20)
			Result.append_string (",%"")
			Result.append_string (name)
			Result.append_string ("%"")
		end

	actual_transformed (a_entity: ENTITY): ROSE_LINEAR_ARRAY [ENTITY]
			-- Actual transformation of 'a_entity' ('a_entity' has been checked to be valid)
		do
			Result := relationship_handler.related_entities (a_entity)
		end

feature {ROSE_TRANSFORMATION_LIST} -- Implementation

	specific_image (a_chains: ROSE_ENTITY_CHAINS; a_start, a_end: ROSE_DATE; a_range: HASH_TABLE [ENTITY, ENTITY]): ROSE_ENTITY_CHAINS
			-- This is only needed to bridge between old ROSE_ENTITY_CHAINS approach and ROSE_RECIPE
		do
			Result := a_chains.related (relationship_handler, True, parent + 1, a_start, a_end, a_range)
		end

end
