indexing
	description: "Transformation result set"
	author: "Zoran Simic"
	date: "$Date: 2008/12/01 $"
	revision: "$Revision: #6 $"

class
	ROSE_TRANSFORMATION_RESULT

create
	make

feature {NONE} -- Initialization

	make (a_recipe: ROSE_RECIPE) is
			-- create with `recipe' set to `a_recipe'
		do
			recipe := a_recipe
		end

feature -- Access

	recipe: ROSE_RECIPE
			-- Attached recipe

	input: ARRAY [ENTITY]
			-- Input that led to this result set

	chains: ROSE_LINEAR_ARRAY [ROSE_TRANSFORMATION_CHAIN]
			-- Chains composing the result set

	is_empty: BOOLEAN is
			-- Is this result set empty?
		do
			Result := chains = Void or else chains.is_empty
		end

feature -- Basic operations

	set (a_input: ARRAY [ENTITY]) is
		local
			l_chain: ROSE_TRANSFORMATION_CHAIN
			i: INTEGER
		do
			input := a_input
			if input = Void then
				create chains.make (1, 0)
			else
				create chains.make (a_input.lower, a_input.upper)
				from
					i := a_input.lower
				until
					i > a_input.upper
				loop
					create l_chain.make_root (a_input.item (i), recipe.transformation)
					chains.put (l_chain, i)
					i := i + 1
				end
			end
		end

	set_recipe (a_recipe: ROSE_RECIPE) is
			-- set `recipe' to `a_recipe'
		do
			recipe := a_recipe
		end


	wipe_out is
		do
			input := Void
			chains := Void
		end

--	put (a_chain: ROSE_TRANSFORMATION_CHAIN; a_index: INTEGER) is
--			-- Put 'a_chain' at 'a_index' in the result set.
--		require
--			valid_chain: a_chain /= Void
--			valid_index: a_index >= 1 and then a_index <= chains.count
--		do
--			chains.put (a_chain, a_index)
--		end

	do_all (a_agent: PROCEDURE [ANY,TUPLE [ARRAY [ROSE_TRANSFORMATION_CHAIN]]]) is
			-- Call 'a_agent' on each combination of chains in this result set.
		require
			valid_agent: a_agent /= Void
		local
			i: INTEGER
			l_array: ARRAY [ROSE_TRANSFORMATION_CHAIN]
		do
			from
				i := chains.lower
				create l_array.make (0, recipe.depth)
			until
				i > chains.upper
			loop
--				l_array.put (chains.item (i), 0)
				chains.item (i).do_all (l_array, a_agent)
--				populate (l_array, chains.item (i), 0)
				i := i + 1
			end
		end

	to_csv: STRING is
		do
			create Result.make_empty
			do_all (agent append_csv (Result, ?))
		end

feature {NONE} -- Implementation

	append_csv (a_result: STRING; a_result_line: ARRAY [ROSE_TRANSFORMATION_CHAIN]) is
		require
			a_result /= Void
			a_result_line /= Void
		local
			i: INTEGER
		do
			from
				i := a_result_line.lower
			until
				i > a_result_line.upper
			loop
				a_result.append (a_result_line.item (i).entity.handle.out)
				a_result.append (",")
				i := i + 1
			end
			a_result.append ("%N")
		end

	populate (a_array: ARRAY [ROSE_TRANSFORMATION_CHAIN]; a_chain: ROSE_TRANSFORMATION_CHAIN; a_index: INTEGER) is
		local
			i: INTEGER
		do
			a_array.put (a_chain, a_index)
			if a_chain.children /= Void then
				from
					i := a_chain.children.lower
				until
					i > a_chain.children.upper
				loop
					populate (a_array, a_chain.children.item (i), a_index + 1)
					i := i + 1
				end
			end
		end

invariant
	no_void_chains: (chains /= Void) implies not chains.has (Void)

end
