note
	description: "Attributes to be displayed for a given source/target"
	author: "Zoran Simic"
	last_editor: "$Author: zs $"
	date: "$Date: 2011/05/27 $"
	revision: "$Revision: #5 $"
	path: "$File: //rose/source/kernel/entities/transformation/rose_transformation_setting_display.e $"

class
	ROSE_TRANSFORMATION_SETTING_DISPLAY

inherit
	ROSE_SHARED_GLOBAL
	DEBUG_OUTPUT

create
	make, make_from_sequence

feature {NONE} -- Initialization

	make
			-- Creation procedure.
		do
			create identifier_names.make_empty
			identifier_names.compare_objects
		end

	make_from_sequence (a_sequence: ROSE_LINEAR_ARRAY [STRING])
			-- Create from 'a_sequence'
		do
			make
			set_sequence (a_sequence, True)
		end

feature -- Access

	identifier_names: ROSE_LINEAR_ARRAY [STRING]
			-- Identifier names to display

	representation: STRING
		local
			i: INTEGER
		do
			create Result.make (20)
			if not identifier_names.is_empty then
				from
					Result.append ("%T%Tdisplay ")
					i := identifier_names.lower
				until
					i > identifier_names.upper
				loop
					Result.extend ('%"')
					Result.append_string (Global.formatter.eiffel_string (identifier_names.item (i)))
					Result.extend ('%"')
					if i < identifier_names.upper then
						Result.extend (',')
					end
					i := i + 1
				end
				Result.extend ('%N')
			end
		end

	debug_output: STRING
			-- String that should be displayed in debugger to represent `Current'.
		do
			Result := representation
		end

	has (a_identifier: STRING): BOOLEAN
		do
			if a_identifier /= Void then
				Result := identifier_names.has (a_identifier.as_upper)
			end
		end

feature -- Basic operations

	set_from_other (a_other: like Current)
			-- Set from 'a_other'
		do
			make
			if a_other /= Void then
				identifier_names.append_array (a_other.identifier_names)
			end
		end

	set (a_identifier: STRING; a_enabled: BOOLEAN)
		do
			if a_identifier /= Void then
				a_identifier.to_upper
				if a_enabled then
					if not identifier_names.has (a_identifier) then
						identifier_names.extend (a_identifier)
					end
				else
					if identifier_names.has (a_identifier) then
						identifier_names.prune_all (a_identifier)
					end
				end
			end
		end

	set_sequence (a_sequence: ARRAY [STRING]; a_enabled: BOOLEAN)
		local
			i: INTEGER
		do
			if a_sequence /= Void then
				from
					i := a_sequence.lower
				until
					i > a_sequence.upper
				loop
					set (a_sequence.item (i), a_enabled)
					i := i + 1
				end
			end
		end

	wipe_out
		do
			identifier_names.wipe_out
		end

end
