note
	description: "Filter to be apply for a given source/target"
	author: "Zoran Simic"
	last_editor: "$Author: zs $"
	date: "$Date: 2011/05/27 $"
	revision: "$Revision: #5 $"
	path: "$File: //rose/source/kernel/entities/transformation/rose_transformation_setting_filter.e $"

class
	ROSE_TRANSFORMATION_SETTING_FILTER

inherit
	ROSE_SHARED_GLOBAL
	DEBUG_OUTPUT

create
	make_from_sequence, make_ask

feature -- Initialization

	make_from_sequence (a_entity_ids: ROSE_LINEAR_ARRAY [INTEGER])
			-- Make with 'a_enitity_ids'
		do
			entity_ids := a_entity_ids
			is_ask_user := False
		ensure
			entity_ids_set: entity_ids = a_entity_ids
			dont_ask_user: not is_ask_user
		end

	make_ask
			-- Make this filter set to "ask the user" when evaluating a recipe
		do
			is_ask_user := True
		ensure
			ask_user: is_ask_user
		end

feature -- Access

	is_ask_user: BOOLEAN						-- Ask the user interactively to select items in the filter?
	entity_ids: ROSE_LINEAR_ARRAY [INTEGER]		-- Identifier names to display

	representation: STRING
			-- Textual representation of this filter, suitable for the recipe parser
		local
			i: INTEGER
		do
			create Result.make (20)
			if is_ask_user then
				Result.append ("%T%Tfilter ask%N")
			elseif entity_ids /= Void and then not entity_ids.is_empty then
				from
					Result.append ("%T%Tfilter ")
					i := entity_ids.lower
				until
					i > entity_ids.upper
				loop
					Result.append_integer (entity_ids.item (i))
					if i < entity_ids.upper then
						Result.extend (',')
					end
					i := i + 1
				end
				Result.extend ('%N')
			end
		end

	debug_output: STRING
			-- String that should be displayed in debugger to represent `Current'.
		do
			Result := representation
		end

feature -- Basic operations

	set_ask (a_ask_user: BOOLEAN)
			-- Set 'is_ask_user'
		do
			is_ask_user := a_ask_user
		end

	set_from_other (a_other: like Current)
			-- Set from 'a_other'
		do
			if a_other /= Void then
				is_ask_user := a_other.is_ask_user
				if a_other.entity_ids /= Void then
					entity_ids := a_other.entity_ids.twin
				else
					entity_ids := Void
				end
			else
				is_ask_user := False
				entity_ids := Void
			end
		end

end
