note
	description: "Transformation settings: display, filter and constraints"
	author: "Zoran Simic"
	last_editor: "$Author: zs $"
	date: "$Date: 2011/05/27 $"
	revision: "$Revision: #6 $"
	path: "$File: //rose/source/kernel/entities/transformation/rose_transformation_settings.e $"

class
	ROSE_TRANSFORMATION_SETTINGS

inherit
	DEBUG_OUTPUT

create
	make, make_copy

feature {NONE} -- Initialization

	make (a_display: ROSE_TRANSFORMATION_SETTING_DISPLAY; a_filter: ROSE_TRANSFORMATION_SETTING_FILTER; a_constraint: ROSE_TRANSFORMATION_SETTING_CONSTRAINT)
		do
			create display.make
			create filter.make_from_sequence (Void)
			create constraint
			set_display (a_display)
			set_filter (a_filter)
			set_constraint (a_constraint)
		end

	make_copy (a_other: ROSE_TRANSFORMATION_SETTINGS)
		do
			make (Void, Void, Void)
			set_from_other (a_other)
		end

feature -- Access

	display: attached ROSE_TRANSFORMATION_SETTING_DISPLAY			-- Holds columns to display for the attached category (recipe source, or transformation target)
	filter: attached ROSE_TRANSFORMATION_SETTING_FILTER				-- Filter to apply to a transformation target (list of entities to keep only)
	constraint: attached ROSE_TRANSFORMATION_SETTING_CONSTRAINT		-- Date constraints for transformations

	representation: STRING
			-- Representation of this transformation
		do
			create Result.make (100)
			Result.append_string (display.representation)
			Result.append_string (filter.representation)
			Result.append_string (constraint.representation)
		ensure
			valid: Result /= Void
		end

	debug_output: STRING
			-- String that should be displayed in debugger to represent `Current'.
		do
			Result := representation
		end

feature -- Basic operations

	set_display (a_display: ROSE_TRANSFORMATION_SETTING_DISPLAY)
			-- Set 'display'
		do
			display.set_from_other (a_display)
		end

	set_filter (a_filter: ROSE_TRANSFORMATION_SETTING_FILTER)
			-- Set 'filter'
		do
			filter.set_from_other (a_filter)
		end

	set_constraint (a_constraint: ROSE_TRANSFORMATION_SETTING_CONSTRAINT)
			-- Set 'constraint'
		do
			constraint.set_from_other (a_constraint)
		end

	set_identifiers_from_accessor(a_accessor : ROSE_ENTITY_DISPLAY)
		local
			i : INTEGER
			l_identifier : ENTITY_IDENTIFIER
		do
			from i := 1 until i > a_accessor.identifier_count loop
				l_identifier := a_accessor.identifier_if_active (i)
				if l_identifier /= void then
					display.set (l_identifier.name,true)
				else
					display.set (a_accessor.identifier(i).name,false)
				end
				i := i + 1
			end
		end

	set_from_other (a_other: like Current)
			-- Set from 'a_other'
		do
			if a_other /= Void then
				display.set_from_other (a_other.display)
				filter.set_from_other (a_other.filter)
				constraint.set_from_other (a_other.constraint)
			else
				display.set_from_other (Void)
				filter.set_from_other (Void)
				constraint.set_from_other (Void)
			end
		end

end
