note

	description:

		"binary expression with aritmetic operations"

	keywords: "rose,expression,binary"
	author: "Mark Howard"
	last_editor: "$Author: it $"
	date: "$Date: 2014/03/14 $"
	revision: "$Revision: #44 $"
	path: "$File: //rose/source/kernel/math/rose_unary_expression.e $"

class
	ROSE_UNARY_EXPRESSION [reference G]

inherit

	ROSE_EXPRESSION[G]
		rename
			make as rose_expression_make
		export {ANY}
			computed_double,
			computed_real,
			computed_integer,
			computed_units
		redefine
			generated_value_in_market_currency,
			sub_expressions,
			component_expressions,
			string_value,
			units,
			computed_double,
			computed_real,
			computed_integer,
			computed_string,
			internal_generated_double,
			internal_generated_real,
			computed_boolean,
			boolean_value,
			internal_generated_integer,
			internal_generated_string,
			internal_generated_date,
			is_defined_real,
			is_defined_double,
			value_in_market_currency,
			generated_unitized_value,
			is_flow,
			display_string,
			computed_units,
			double_using_agent,
			nominal_units
		end


create
	make
feature {NONE}-- Creation

	make (a_left : like left;a_operator : like operator) is
			--creation
		do
			operator_string := a_operator.name
			if a_left.mnemonic /= void  then
				mnemonic := a_operator.name + "_" + a_left.mnemonic
			end
			operator := a_operator
			left := a_left
			name := operator_string + "(" + left.name + ")"
			rose_expression_make(void)
			type := left.type
			create variable_description
			variable_description.set_name (name)
		end

feature -- Access

	left : ROSE_EXPRESSION[G]
			-- Operand

	currency (a_target : like expression_target) : PERSISTENT_CURRENCY is
			--currency of result
		do
			Result := left.currency(a_target)
		end
	expression_string(a_target : like expression_target) : STRING is
			--display string for expression
		do
			Result := "(" + operator_string + "(" + left.display_string(a_target) + "))"
		end

	operator_string : STRING
			-- String representation of `operator'

	operator : GENERIC_UNARY_OPERATOR[G,ANY]
			-- Operation

	description : STRING is
			-- do nothing for now
		do
			--Result := "(" + operator_string + "("+ left.description + "))"
		end

	nominal_units: ROSE_UNITS is
			-- <Precursor>
		do
			Result := left.nominal_units
			if Result = Void then
				Result := pure_units
			end
		end

feature -- Query
	is_defined_real (a_value : REAL) : BOOLEAN is
			-- Does the expression think that the value is defined?
		do
			Result := left.is_defined_real(a_value)
		end
	is_defined_double (a_value : DOUBLE) : BOOLEAN is
			-- Does the expression think that the value is defined?
		do
			Result := left.is_defined_double(a_value)
		end
	is_defined_integer (a_value : INTEGER) : BOOLEAN is
			--Is this a defined integer
		do
			Result := left.is_defined_integer(a_value)
		end
	is_flow : BOOLEAN
			--is this a transient value?
		do
			Result := left.is_flow
		end
	get_real_value (a_target : like expression_target) : REAL is
			--<Precursor>
		do
			Result := operator.real_value(left,a_target)
		end
	get_double_value (a_target : like expression_target) : DOUBLE is
			--<Precursor>
		do
			Result := operator.double_value(left,a_target)
		end
	get_integer_value (a_target : like expression_target) : INTEGER is
			--<Precursor>
		do
			Result := operator.integer_value(left,a_target)
		end
	boolean_value (a_target : like expression_target) : BOOLEAN is
			-- Boolean value
		do
			Result := operator.boolean_value(left,a_target)
		end

	string_value (a_target : like expression_target) : STRING is
			--	<Precursor>
		do
			Result := operator.string_value(left,a_target)
		end

	units (a_target : like expression_target) : ROSE_UNITS is
			--<Precursor>
		do
			Result := operator.units(left,a_target)
		end
	undefined_real : REAL is
			--<Precursor>
		do
			Result := left.undefined_real
		end
	undefined_double : DOUBLE is
			--<Precursor>
		do
			Result := left.undefined_double
		end
	undefined_integer : INTEGER is
			--<Precursor>
		do
			Result := left.undefined_integer
		end
	sub_expressions : ROSE_LINEAR_ARRAY[ROSE_EXPRESSION[like expression_target]] is
			--List of used expressions
		local
			l_expressions : ROSE_LINEAR[ROSE_EXPRESSION[like expression_target]]
		do
			Result := Precursor
			l_expressions := left.sub_expressions
			from l_expressions.start until l_expressions.off loop
				Result.extend(l_expressions.item)
				l_expressions.forth
			end
		end

	component_expressions (a_target: attached ROSE_EXPRESSION_TARGET): ROSE_LINEAR_ARRAY [ROSE_EXPRESSION [ROSE_EXPRESSION_TARGET]] is
			-- Expressions which contribute to value of `Current'
		local
			l_exp_1: ROSE_EXPRESSION [ROSE_EXPRESSION_TARGET]
		do
			l_exp_1 ?= left
			if l_exp_1 /= Void then
				create Result.make (1, 1)
				Result.put (l_exp_1, 1)
			else
				create Result.make_empty
			end
		end

feature -- Computation

	computed_double(a_target : like expression_target; a_agent : like access_agent_template) : DOUBLE is
				-- Compute double value
		do
			Result := operator.computed_double(left, a_target, a_agent)
		end

	computed_real(a_target : like expression_target; a_agent : like access_agent_template) : REAL is
			-- Computed real
		do
			Result := operator.computed_real(left, a_target, a_agent)
		end

	computed_integer(a_target : like expression_target; a_agent : like access_agent_template) : INTEGER is
			-- <Precursor>
		do
			Result := operator.computed_integer(left, a_target, a_agent)
		end

	computed_string(a_target : like expression_target; a_agent : like access_agent_template) : STRING is
			-- Computed string
		do
			Result := operator.computed_string(left, a_target, a_agent)
		end

	computed_boolean(a_target : like expression_target; a_agent : like access_agent_template) : BOOLEAN is
			-- Computed boolean
		do
			Result := operator.computed_boolean(left, a_target, a_agent)
		end

	generated_unitized_value(a_target : like expression_target; a_agent : like access_agent_template) : DATED_UNITIZED_VALUE[DOUBLE] is
			-- <Precursor>
		do
			Result := operator.generated_unitized_value(left, a_target, a_agent)
		end

	double_using_agent(a_target : like expression_target; a_agent : like access_agent_template) : DOUBLE is
			-- <Precursor>
		local
			l_value : ROSE_UNITIZED_VALUE[DOUBLE]
		do
			Result := missing_double
			if attached {ROSE_TARGET[ANY]} a_target as l_target then
				if a_agent /= Void then
					l_value := a_agent.unitized_double (l_target, Current)
				else
					Result := double_value (a_target)
				end
			end
			if not is_defined_double (Result) then
				if l_value /= Void then
					Result := l_value.value
				elseif a_agent /= Void then
					Result := operator.double_using_agent(left, a_target, a_agent)
				else
					Result := double_value (a_target)
				end
			end
		end

	value_in_market_currency(a_market: MARKET; a_currency: like currency_anchor; a_target: like expression_target; a_date: ROSE_DATE) : DOUBLE is
				-- Some unary expressions may be in currencies.
		do
			Result := operator.value_in_market_currency(left, a_market, a_currency, a_target, a_date)
		end
	computed_units (a_target: like expression_target; a_agent :like access_agent_template): ROSE_UNITS is
			-- <Precursor>
		do
			Result := operator.computed_units(left.computed_units(a_target,a_agent))
		end
	display_string(a_target : like expression_target) : STRING is
			--display string
		do
			Result := operator_string + "(" + left.display_string(a_target) + ")"
		end
	generated_value_in_market_currency (a_market: MARKET; a_currency: PERSISTENT_CURRENCY; a_target: like expression_target; a_date: ROSE_DATE; a_agent: like access_agent_template): REAL_64
			-- <Precursor>
		local
			l_value: DOUBLE
		do
			if attached {ROSE_FUNCTION_OPERATOR[G]} operator as l_operator then
				l_value := left.generated_value_in_market_currency (a_market,a_currency,a_target,a_date,a_agent)
				Result := l_operator.function.item ([l_value, a_target, Current])
			else
				Result := Precursor (a_market, a_currency, a_target, a_date, a_agent)
			end
		end

feature {NONE} -- Generate

	internal_generated_double (a_target : like expression_target; a_agent : like access_agent_template) : DOUBLE is
			-- <Precursor>
		do
			Result := operator.generated_double(left, a_target, a_agent)
		end

	internal_generated_real (a_target : like expression_target; a_agent : like access_agent_template) : REAL is
			-- <Precursor>
		do
			Result := operator.generated_real(left, a_target, a_agent)
		end

	internal_generated_integer(a_target : like expression_target; a_agent : like access_agent_template) : INTEGER is
			-- <Precursor>
		do
			Result := operator.generated_integer(left, a_target, a_agent)
		end

	internal_generated_string(a_target : like expression_target; a_agent : like access_agent_template) : STRING is
			-- <Precursor>
		do
			Result := operator.generated_string(left, a_target, a_agent)
		end

	internal_generated_date(a_target : like expression_target; a_agent : like access_agent_template) : ROSE_DATE is
			-- <Precursor>
		do
			Result := operator.generated_date(left, a_target, a_agent)
		end

invariant
	left_defined : left /= Void
	operator_defined : operator /= Void
end -- class ROSE_UNARY_EXPRESSION


