note

	description:

		"Operators that forward calls to their argument, with pre-and post-processing"

	keywords: "Rose expression language unary operator forward"
	url: ""
	group: "RC London (tools)"
	reviewed_by: "No-one"
	quality: "Unrated"
	author: "$Author: mk $ Colin Adams"
	last_editor: "$Author: mk $"
	date: "$Date: 2011/02/15 $"
	revision: "$Revision: #16 $"
	path: "$File: //rose/source/kernel/math/rose_unary_forwarding_operator.e $"

deferred class ROSE_UNARY_FORWARDING_OPERATOR [reference G]

inherit

	ROSE_UNARY_OPERATOR[G]

			-- Implementations should ensure that calling `preprocess' followed by `postprocess' on a given
			-- expression does not change the state of the expression
		redefine
			units
		end
feature -- Access

	real_value (a_left: ROSE_EXPRESSION[G]; a_g: G): REAL is
			--returned real value
		do
			preprocess (a_left)
			Result := a_left.real_value (a_g)
			postprocess (a_left)
		end

	double_value (a_left: ROSE_EXPRESSION[G]; a_g: G): DOUBLE is
			--returned double value
		do
			preprocess (a_left)
			Result := a_left.double_value (a_g)
			postprocess (a_left)
		end

	integer_value (a_left: ROSE_EXPRESSION[G]; a_g: G): INTEGER is
			--INTEGER value
		do
			preprocess (a_left)
			Result := a_left.integer_value (a_g)
			postprocess (a_left)
		end

	boolean_value (a_left: ROSE_EXPRESSION[G]; a_g: G): BOOLEAN is
			-- Boolean value
		do
			preprocess (a_left)
			Result := a_left.boolean_value (a_g)
			postprocess (a_left)
		end


	string_value (a_left: ROSE_EXPRESSION[G]; a_g: G): STRING is
		do
			preprocess (a_left)
			Result := a_left.string_value (a_g)
			postprocess (a_left)
		end

	date_value (a_left: ROSE_EXPRESSION[G]; a_g: G): ROSE_DATE is
			-- Date value
		do
			preprocess (a_left)
			Result := a_left.date_value (a_g)
			postprocess (a_left)
		end


	value_in_market_currency (a_left: ROSE_EXPRESSION[G]; a_market: MARKET; a_currency: PERSISTENT_CURRENCY; a_entity: G; a_date: ROSE_DATE): DOUBLE is
		do
			preprocess (a_left)
			Result := a_left.value_in_market_currency (a_market, a_currency, a_entity, a_date)
			postprocess (a_left)
		end

	computed_units (a_left: ROSE_UNITS): ROSE_UNITS is
		do
			Result := a_left
		end
	units (a_left: ROSE_EXPRESSION[G]; a_entity: G): ROSE_UNITS is
		do
			preprocess (a_left)
			Result := a_left.units (a_entity)
			postprocess (a_left)
		end

	units_in_currency (a_left: ROSE_EXPRESSION[G]; a_currency: PERSISTENT_CURRENCY; a_entity: G): ROSE_UNITS is
		do
			preprocess (a_left)
			Result := a_left.units_in_currency (a_currency, a_entity)
			postprocess (a_left)
		end

	computed_double (a_left: ROSE_EXPRESSION[G]; a_g: G; a_agent: like access_agent_template): DOUBLE is
			-- Computed double
		do
			preprocess (a_left)
			Result := a_left.computed_double (a_g, a_agent)
			postprocess (a_left)
		end

	computed_real (a_left: ROSE_EXPRESSION[G]; a_g: G; a_agent: like access_agent_template): REAL is
		do
			preprocess (a_left)
			Result := a_left.computed_real (a_g, a_agent)
			postprocess (a_left)
		end

	computed_integer (a_left: ROSE_EXPRESSION[G]; a_g: G; a_agent: like access_agent_template): INTEGER is
		do
			preprocess (a_left)
			Result := a_left.computed_integer (a_g, a_agent)
			postprocess (a_left)
		end

	computed_boolean (a_left: ROSE_EXPRESSION[G]; a_g: G; a_agent: like access_agent_template): BOOLEAN is
			-- Computed boolean
		do
			preprocess (a_left)
			Result := a_left.computed_boolean (a_g, a_agent)
			postprocess (a_left)
		end

	computed_string (a_left: ROSE_EXPRESSION[G]; a_g: G; a_agent: like access_agent_template): STRING is
		local
		do
			preprocess (a_left)
			Result := a_left.computed_string (a_g, a_agent)
			postprocess (a_left)
		end


	generated_double (a_left: ROSE_EXPRESSION[G]; a_g: G; a_agent: like access_agent_template): DOUBLE is
		do
			preprocess (a_left)
			Result := a_left.generated_double (a_g, a_agent)
			postprocess (a_left)
		end

	generated_real (a_left: ROSE_EXPRESSION[G]; a_g: G; a_agent: like access_agent_template): REAL is
		do
			preprocess (a_left)
			Result := a_left.generated_real (a_g, a_agent)
			postprocess (a_left)
		end

	generated_integer (a_left: ROSE_EXPRESSION[G]; a_g: G; a_agent: like access_agent_template): INTEGER is
		do
			preprocess (a_left)
			Result := a_left.generated_integer (a_g, a_agent)
			postprocess (a_left)
		end

	generated_string (a_left: ROSE_EXPRESSION[G]; a_g: G; a_agent: like access_agent_template): STRING is
		do
			preprocess (a_left)
			Result := a_left.generated_string (a_g, a_agent)
			postprocess (a_left)
		end

	generated_date (a_left: ROSE_EXPRESSION[G]; a_g: G; a_agent: like access_agent_template): ROSE_DATE is
			-- Generated date
		do
			preprocess (a_left)
			Result := a_left.generated_date (a_g, a_agent)
			postprocess (a_left)
		end

feature -- Additional processing

	preprocess (a_left: ROSE_EXPRESSION[G]) is
			-- Perform pre-processing on `a_left'.
		require
			a_left_not_void: a_left /= Void
		deferred
		end

	postprocess (a_left: ROSE_EXPRESSION[G]) is
			-- Perform post-processing on `a_left'.
		require
			a_left_not_void: a_left /= Void
		deferred
		end

end
