indexing
	description: "The class that represents a virtual record that is usually stored in a signature tagged database"
	author: "Mauricio Bustos"
	last_editor: "$Author: da $"
	date: "$Date: 2012/12/05 $";
	revision: "$Revision: #62 $"
	path: "$File: //rose/source/kernel/persistence/databases/signature_tagged/rose_virtual_record.e $"

class

	ROSE_VIRTUAL_RECORD[G->ROSE_ENTITY_DATE[ENTITY]]

inherit

	ACCESS_EXTERNALS
		export
			{NONE} all
		end
	DEBUG_OUTPUT
	INTERNAL
		export
			{NONE} all
		end
	MISMATCH_CORRECTOR
		export
			{NONE} all
		redefine
			correct_mismatch
		end

create

	make,
	default_create

feature -- Creation

	make (a_signature: G) is
			-- Create the record object
		do
			signature := a_signature
		end

feature -- Indexing

	multi_key: MULTI_KEY is
			-- Unique key for this record within the context of the signature's data manager
		do
			if signature /= Void then
				Result := signature.multi_key
			end
		end

	new_record(a_signature : G) : like Current
			--new record for a_signature
		do
			create Result.make(a_signature)
		end

feature {ROSE_SAX_CONTENT_HANDLER} -- XML Item Setting

	Real_Item: CHARACTER is 'r'
	Integer_Item: CHARACTER is 'i'
	Double_Item: CHARACTER is 'd'
	String_Item: CHARACTER is 's'
	Image_Item: CHARACTER is 'g'

	set_item_from_xml(a_type: STRING; a_value, a_item_id: INTEGER; a_style: INTEGER) is
			-- set item from xml
		local
			l_real: REAL
			l_double: DOUBLE
		do
			inspect a_type.item(1)
			when Real_Item then
				if a_value /= a_value.min_value then
					if a_style = xml_binary then
						copy_to_pointer($l_real,$a_value,4)
						set_real_item(l_real,a_item_id)
					else
						set_real_item(a_value,a_item_id)
					end
				else
					set_real_item(ieee_math.qnan_real_32,a_item_id)
				end
			when Integer_Item then
				if a_value /= a_value.min_value then
					if a_style = xml_binary then
						set_integer_item(a_value,a_item_id)
					else
						l_real := a_value
						set_integer_item(l_real.truncated_to_integer,a_item_id)
					end
				end
			when Double_Item then
				if a_value /= a_value.min_value then
					if a_style = xml_binary then
						copy_to_pointer($l_double,$a_value,8)
						set_double_item(l_double,a_item_id)
					else
						set_double_item(a_value,a_item_id)
					end
				else
					set_double_item(ieee_math.qnan,a_item_id)
				end
			when String_Item then
				if a_value /= a_value.min_value then
					set_string_item(a_value.out,a_item_id)
				else
					set_string_item(Void,a_item_id)
				end
			when Image_Item then
			end
		end

	set_record_item_from_xml(a_record: ROSE_SIGNATURE_RECORD_ITEM; a_record_values: ROSE_LINEAR_ARRAY[TUPLE[STRING,STRING]]; a_style: INTEGER) is
			-- Set 'a_record' from 'a_record_values'
		local
			i, j: INTEGER
			l_name, l_str_value: STRING
			l_value: INTEGER
			l_big_value: INTEGER_64
			l_real: REAL
			l_double: DOUBLE
		do
			from a_record_values.start until a_record_values.after loop
				i := field_count(a_record)
				l_name ?= a_record_values.current_item.item(1)
				from j := 1 until j > i loop
					if l_name.is_equal(field_name(j,a_record)) then
						inspect field_type(j,a_record)
						when Real_Type then
							l_value := a_record_values.current_item.integer_item(2).to_integer
							if l_value /= l_value.min_value then
								copy_to_pointer($l_real,$l_value,4)
								set_real_field(j,a_record,l_real)
							else
								set_real_field(j,a_record,ieee_math.qnan_real_32)
							end
						when Integer_32_Type then
							l_value := a_record_values.current_item.integer_item(2).to_integer
							if l_value /= l_value.min_value then
								set_integer_field(j,a_record,l_value)
							end
						when Double_Type then
							if l_value /= l_value.min_value then
								l_str_value ?= a_record_values.current_item.item(2)
								--l_big_value := integer_64_from_string(l_str_value)
								l_big_value := l_str_value.to_integer_64
								copy_to_pointer($l_double,$l_big_value,8)
								set_double_field(j,a_record,l_double)
							else
								set_double_field(j,a_record,ieee_math.qnan)
							end
							j := i + 1
						end
					end
					j := j + 1
				end
				a_record_values.forth
			end
			set_record_item(a_record)
		end

feature {NONE} -- XML parsing for integer_64 data

	integer_64_from_string(a_value: STRING): INTEGER_64 is
			-- Create an integer_64 from a_value
		local
			l_integer: INTEGER
			l_string: STRING
			l_value: INTEGER_64
		do
			l_integer := 1
			if a_value.count > 8 then
				l_string := a_value.substring(1,8)
				Result := l_string.to_integer
				from l_integer := 9 until l_integer > a_value.count loop
					Result := Result * 10
					l_integer := l_integer + 1
				end
				l_integer := 9
			end
			if a_value.count > 16 then
				l_string := a_value.substring(9,16)
				l_value := l_string.to_integer
				from l_integer := 17 until l_integer > a_value.count loop
					l_value := l_value * 10
					l_integer := l_integer + 1
				end
				Result := Result + l_value
				l_integer := 17
			end
			Result := Result + a_value.substring(l_integer,a_value.count).to_integer
		end

	xml_binary: INTEGER is 0
	xml_scientific: INTEGER is 1

feature  --update

	clear is
			-- Clear all data in the record by clearing the existing tables
		do
			if integer_items_saved /= Void then
				integer_items_saved.clear_all
			end
			if real_items_saved /= Void then
				real_items_saved.clear_all
			end
			if double_items_saved /= Void then
				double_items_saved.clear_all
			end
			if string_items_saved /= Void then
				string_items_saved.clear_all
			end
			if image_items_saved /= Void then
				image_items_saved.clear_all
			end
			if date_items_saved /= Void then
				date_items_saved.clear_all
			end
			if indexed_records /= Void then
				indexed_records.clear_all
			end
			if records /= Void then
				records.clear_all
			end
			if overridden_integer_items /= Void then
				overridden_integer_items.clear_all
			end
			if overridden_real_items /= Void then
				overridden_real_items.clear_all
			end
			if overridden_double_items /= Void then
				overridden_double_items.clear_all
			end
			if overridden_string_items /= Void then
				overridden_string_items.clear_all
			end
			if overridden_image_items /= Void then
				overridden_image_items.clear_all
			end
			if overridden_date_items /= Void then
				overridden_date_items.clear_all
			end
			if overridden_records /= Void then
				overridden_records.clear_all
			end
			if overridden_signature_data /= Void then
				overridden_signature_data := Void
			end
		end

feature -- Status

	output_for_verification: STRING is
			-- Used by STIM_VERIFICATION.  The returned STRING is just the concatenation of
			-- every bit of data in the record.
		local
			l_temp_array: ROSE_LINEAR_ARRAY[STRING]
		do
			Result := signature.output_for_verification
			if integer_items_saved /= Void then
				create l_temp_array.make_empty
				from
					integer_items_saved.start
				until
					integer_items_saved.off
				loop
					l_temp_array.extend("," +
										integer_items_saved.key_for_iteration.out + "/" +
										integer_items_saved.item_for_iteration.out)
						integer_items_saved.forth
				end
				-- This only works in 5.5!
				--			l_temp_array.sort_by(agent {STRING}.infix "<"(?))
				--			l_temp_array.do_all(agent Result.append(?))
			end
			if real_items_saved /= Void then
				create l_temp_array.make_empty
				from
					real_items_saved.start
				until
					real_items_saved.off
				loop
					l_temp_array.extend("," +
										real_items_saved.key_for_iteration.out + "/" +
										real_items_saved.item_for_iteration.out)
						real_items_saved.forth
				end
				--			l_temp_array.sort_by(agent {STRING}.infix "<"(?))
				--			l_temp_array.do_all(agent Result.append(?))
			end
			if double_items_saved /= Void then
				create l_temp_array.make_empty
				from
					double_items_saved.start
				until
					double_items_saved.off
				loop
					l_temp_array.extend("," +
										double_items_saved.key_for_iteration.out + "/" +
										double_items_saved.item_for_iteration.out)
						double_items_saved.forth
				end
				--			l_temp_array.sort_by(agent {STRING}.infix "<"(?))
				--			l_temp_array.do_all(agent Result.append(?))
			end
			if string_items_saved /= Void then
				create l_temp_array.make_empty
				from
					string_items_saved.start
				until
					string_items_saved.off
				loop
					l_temp_array.extend("," +
										string_items_saved.key_for_iteration.out + "/" +
										string_items_saved.item_for_iteration.out)
						string_items_saved.forth
				end
				--			l_temp_array.sort_by(agent {STRING}.infix "<"(?))
				--			l_temp_array.do_all(agent Result.append(?))
			end
			if image_items_saved /= Void then
				create l_temp_array.make_empty
				from
					image_items_saved.start
				until
					image_items_saved.off
				loop
					l_temp_array.extend("," +
										image_items_saved.key_for_iteration.out + "/" +
										image_items_saved.item_for_iteration.out)
						image_items_saved.forth
				end
				--			l_temp_array.sort_by(agent {STRING}.infix "<"(?))
				--			l_temp_array.do_all(agent Result.append(?))
			end
			if date_items_saved /= Void then
				create l_temp_array.make_empty
				from
					date_items_saved.start
				until
					date_items_saved.off
				loop
					l_temp_array.extend("," +
										date_items_saved.key_for_iteration.out + "/" +
										date_items_saved.item_for_iteration.out)
						date_items_saved.forth
				end
				--			l_temp_array.sort_by(agent {STRING}.infix "<"(?))
				--			l_temp_array.do_all(agent Result.append(?))
			end
			if records /= Void then
				create l_temp_array.make_empty
				from
					records.start
				until
					records.off
				loop
					l_temp_array.extend("," +
										records.key_for_iteration.out + "/" +
										records.item_for_iteration.output_for_verification)		-- Not 'out'.
						records.forth
				end
				--			l_temp_array.sort_by(agent {STRING}.infix "<"(?))
				--			l_temp_array.do_all(agent Result.append(?))
			end
			if overridden_integer_items /= Void then
				create l_temp_array.make_empty
				from
					overridden_integer_items.start
				until
					overridden_integer_items.off
				loop
					l_temp_array.extend("," +
										overridden_integer_items.key_for_iteration.out + "/" +
										overridden_integer_items.item_for_iteration.out)
						overridden_integer_items.forth
				end
				--			l_temp_array.sort_by(agent {STRING}.infix "<"(?))
				--			l_temp_array.do_all(agent Result.append(?))
			end
			if overridden_real_items /= Void then
				create l_temp_array.make_empty
				from
					overridden_real_items.start
				until
					overridden_real_items.off
				loop
					l_temp_array.extend("," +
										overridden_real_items.key_for_iteration.out + "/" +
										overridden_real_items.item_for_iteration.out)
						overridden_real_items.forth
				end
				--			l_temp_array.sort_by(agent {STRING}.infix "<"(?))
				--			l_temp_array.do_all(agent Result.append(?))
			end
			if overridden_double_items /= Void then
				create l_temp_array.make_empty
				from
					overridden_double_items.start
				until
					overridden_double_items.off
				loop
					l_temp_array.extend("," +
										overridden_double_items.key_for_iteration.out + "/" +
										overridden_double_items.item_for_iteration.out)
						overridden_double_items.forth
				end
				--			l_temp_array.sort_by(agent {STRING}.infix "<"(?))
				--			l_temp_array.do_all(agent Result.append(?))
			end
			if overridden_string_items /= Void then
				create l_temp_array.make_empty
				from
					overridden_string_items.start
				until
					overridden_string_items.off
				loop
					l_temp_array.extend("," +
										overridden_string_items.key_for_iteration.out + "/" +
										overridden_string_items.item_for_iteration.out)
						overridden_string_items.forth
				end
				--			l_temp_array.sort_by(agent {STRING}.infix "<"(?))
				--			l_temp_array.do_all(agent Result.append(?))
			end
			if overridden_image_items /= Void then
				create l_temp_array.make_empty
				from
					overridden_image_items.start
				until
					overridden_image_items.off
				loop
					l_temp_array.extend("," +
										overridden_image_items.key_for_iteration.out + "/" +
										overridden_image_items.item_for_iteration.out)
						overridden_image_items.forth
				end
				--			l_temp_array.sort_by(agent {STRING}.infix "<"(?))
				--			l_temp_array.do_all(agent Result.append(?))
			end
			if overridden_date_items /= Void then
				create l_temp_array.make_empty
				from
					overridden_date_items.start
				until
					overridden_date_items.off
				loop
					l_temp_array.extend("," +
										overridden_date_items.key_for_iteration.out + "/" +
										overridden_date_items.item_for_iteration.out)
						overridden_date_items.forth
				end
				--			l_temp_array.sort_by(agent {STRING}.infix "<"(?))
				--			l_temp_array.do_all(agent Result.append(?))
			end
			if overridden_records /= Void then
				create l_temp_array.make_empty
				from
					overridden_records.start
				until
					overridden_records.off
				loop
					l_temp_array.extend("," +
										overridden_records.key_for_iteration.out + "/" +
										overridden_records.item_for_iteration.output_for_verification)		-- Not 'out'.
						overridden_records.forth
				end
				--			l_temp_array.sort_by(agent {STRING}.infix "<"(?))
				--			l_temp_array.do_all(agent Result.append(?))
			end
			if overridden_signature_data /= Void then
				Result.append("," + overridden_signature_data.output_for_verification)		-- Not 'out'.
			end
		end


	has_any_data: BOOLEAN is
			-- Does this record have any data at all?
		do
			if integer_items_saved /= Void then
				Result := True
			elseif real_items_saved /= Void then
				Result := True
			elseif double_items_saved /= Void then
				Result := True
			elseif string_items_saved /= Void then
				Result := True
			elseif image_items_saved /= Void then
				Result := True
			elseif date_items_saved /= Void then
				Result := True
			elseif records /= Void then
				Result := True
			elseif overridden_integer_items /= Void then
				Result := True
			elseif overridden_real_items /= Void then
				Result := True
			elseif overridden_double_items /= Void then
				Result := True
			elseif overridden_string_items /= Void then
				Result := True
			elseif overridden_image_items /= Void then
				Result := True
			elseif overridden_date_items /= Void then
				Result := True
			elseif overridden_records /= Void then
				Result := True
			elseif overridden_signature_data /= Void then
				Result := True
			end
		end

	has_integer_items: BOOLEAN is
			-- Does this record have any integer data?
		do
			Result := integer_items_saved /= Void
		end

	has_real_items: BOOLEAN is
			-- Does this record have any real data?
		do
			Result := real_items_saved /= Void
		end

	has_double_items: BOOLEAN is
			-- Does this record have any double data?
		do
			Result := double_items_saved /= Void
		end

	has_string_items: BOOLEAN is
			-- Does this record have any string data?
		do
			Result := string_items_saved /= Void
		end

	has_image_items: BOOLEAN is
			-- Does this record have any image data?
		do
			Result := image_items_saved /= Void
		end

	has_date_items: BOOLEAN is
			-- Does this record have any date data?
		do
			Result := date_items_saved /= Void
		end

	has_record_items: BOOLEAN is
			-- Does this record have any record type data?
		do
			Result := records /= Void
		end

	is_signature_equal(a_signature: G): BOOLEAN is
			-- Compare the signature in this virtual record with passed signature.
			-- Normally, this is done by comparing the multi_key's but this function
			-- may (and has been) redefined.
		do
			Result := signature.is_equal(a_signature)
		end

feature -- Overrides

	has_any_overridden_data: BOOLEAN is
			-- Does this record have any overridden data?
		do
			if overridden_integer_items /= Void then
				Result := True
			elseif overridden_real_items /= Void then
				Result := True
			elseif overridden_double_items /= Void then
				Result := True
			elseif overridden_string_items /= Void then
				Result := True
			elseif overridden_image_items /= Void then
				Result := True
			elseif overridden_date_items /= Void then
				Result := True
			elseif overridden_records /= Void then
				Result := True
			elseif overridden_signature_data /= Void then
				Result := True
			end
		end

	has_overridden_integer_items: BOOLEAN is
			-- Does this record have overridden integer items?
		do
			Result := overridden_integer_items /= Void
		end

	has_overridden_real_items: BOOLEAN is
			-- Does this record have overridden real items?
		do
			Result := overridden_real_items /= Void
		end

	has_overridden_double_items: BOOLEAN is
			-- Does this record have overridden double items?
		do
			Result := overridden_double_items /= Void
		end

	has_overridden_string_items: BOOLEAN is
			-- Does this record have overridden string items?
		do
			Result := overridden_string_items /= Void
		end

	has_overridden_image_items: BOOLEAN is
			-- Does this record have overridden image items?
		do
			Result := overridden_image_items /= Void
		end

	has_overridden_date_items: BOOLEAN is
			-- Does this record have overridden date items?
		do
			Result := overridden_date_items /= Void
		end

	has_overridden_records: BOOLEAN is
			-- Does this record have overridden record items?
		do
			Result := overridden_records /= Void
		end

	has_overridden_signature_data: BOOLEAN is
			-- Does this record have overridden signature data?
		do
			Result := overridden_signature_data /= Void
		end

feature -- Containers

	signature: G
			-- The signature for this record

	integer_items: HASH_TABLE[INTEGER,INTEGER] is
		-- Integer items for current record
		do
			if integer_items_saved = Void then
				create integer_items_saved.make(initial_container_size)
			end
			Result := integer_items_saved
		end

	real_items: HASH_TABLE[REAL,INTEGER] is
		-- Real items for current record
		do
			if real_items_saved = Void then
				create real_items_saved.make(initial_container_size)
			end
			Result := real_items_saved
		end

	double_items: HASH_TABLE[DOUBLE,INTEGER] is
		-- Double items for current record
		do
			if double_items_saved = Void then
				create double_items_saved.make(initial_container_size)
			end
			Result := double_items_saved
		end

	string_items: HASH_TABLE[STRING,INTEGER] is
		-- String items for current record
		do
			if string_items_saved = Void then
				create string_items_saved.make(initial_container_size)
			end
			Result := string_items_saved
		end

	image_items: HASH_TABLE[STRING,INTEGER] is
		-- Image items for current record
		do
			if image_items_saved = Void then
				create image_items_saved.make(initial_container_size)
			end
			Result := image_items_saved
		end

	date_items: HASH_TABLE[ROSE_DATE,INTEGER] is
		-- Date items for current record
		do
			if date_items_saved = Void then
				create date_items_saved.make(initial_container_size)
			end
			Result := date_items_saved
		end

	records: HASH_TABLE[ROSE_SIGNATURE_RECORD_ITEM,STRING]

feature -- Data tables

	integer_items_saved: HASH_TABLE[INTEGER,INTEGER]
	real_items_saved: HASH_TABLE[REAL,INTEGER]
	double_items_saved: HASH_TABLE[DOUBLE,INTEGER]
	string_items_saved: HASH_TABLE[STRING,INTEGER]
	image_items_saved: HASH_TABLE[STRING,INTEGER]
	date_items_saved: HASH_TABLE[ROSE_DATE,INTEGER]
	indexed_records: HASH_TABLE[HASH_TABLE[ROSE_SIGNATURE_RECORD_ITEM, MULTI_KEY], STRING]

feature -- Overriden data tables

	overridden_integer_items: HASH_TABLE[OVERRIDE_DATA_RECORD[INTEGER],INTEGER]
	overridden_real_items: HASH_TABLE[OVERRIDE_DATA_RECORD[REAL],INTEGER]
	overridden_double_items: HASH_TABLE[OVERRIDE_DATA_RECORD[DOUBLE],INTEGER]
	overridden_string_items: HASH_TABLE[OVERRIDE_DATA_RECORD[STRING],INTEGER]
	overridden_image_items: HASH_TABLE[OVERRIDE_DATA_RECORD[STRING],INTEGER]
	overridden_date_items: HASH_TABLE[OVERRIDE_DATA_RECORD[ROSE_DATE],INTEGER]
	overridden_records: HASH_TABLE[OVERRIDE_DATA_RECORD[ROSE_SIGNATURE_RECORD_ITEM],STRING]
	overridden_signature_data: OVERRIDE_DATA_RECORD[G]

feature -- Set

	set_signature(a_signature: like signature) is
		-- Set signature
		do
			signature := a_signature
		end

	set_as_of_date(a_date: ROSE_DATE) is
			-- set as of date
		do
			-- This allows descendants to set as of date if record as of date is of interest
		end

	set_integer_item(a_value: INTEGER; a_item_id: INTEGER) is
		-- Set integer item
		do
			integer_items.force(a_value,a_item_id)
		end

	set_real_item(a_value: REAL; a_item_id: INTEGER) is
		-- Set Real item
		do
			real_items.force(a_value,a_item_id)
		end

	set_double_item(a_value: DOUBLE; a_item_id: INTEGER) is
		-- Set Double item
		do
			double_items.force(a_value,a_item_id)
		end

	set_string_item(a_value: STRING; a_item_id: INTEGER) is
		-- Set String item
		do
			string_items.force(a_value,a_item_id)
		end

	set_image_item(a_value: STRING; a_item_id: INTEGER) is
		-- Set Image item
		do
			image_items.force(a_value,a_item_id)
		end

	set_date_item(a_value: ROSE_DATE; a_item_id: INTEGER) is
		-- Set date item
		do
			date_items.force(a_value,a_item_id)
		end

	set_record_item (a_value: ROSE_SIGNATURE_RECORD_ITEM) is
		-- Set record item
		do
			if a_value.is_indexed then
				set_indexed_record_item(a_value)
			else
				if records = Void then
					create records.make(10)
				end
				records.force(a_value,a_value.name)
			end
		end

	set_indexed_record_item(a_value: ROSE_SIGNATURE_RECORD_ITEM) is
		-- Set indexed record item
		local
			l_records: HASH_TABLE[ROSE_SIGNATURE_RECORD_ITEM, MULTI_KEY]
		do
			if indexed_records = Void then
				create indexed_records.make(10)
			end
			indexed_records.search(a_value.name)
			if indexed_records.found then
				l_records := indexed_records.found_item
			else
				create l_records.make(1)
				indexed_records.force(l_records, a_value.name)
			end
			l_records.force(a_value, a_value.multi_key)
		end

	set_overridden_integer_item(a_value: OVERRIDE_DATA_RECORD[INTEGER]; a_item_id: INTEGER) is
		-- Set overridden integer item
		do
			if overridden_integer_items = Void then
				create overridden_integer_items.make(initial_container_size)
			end
			overridden_integer_items.force(a_value,a_item_id)
		end

	set_overridden_real_item(a_value: OVERRIDE_DATA_RECORD[REAL]; a_item_id: INTEGER) is
		-- Set overridden real item
		do
			if overridden_real_items = Void then
				create overridden_real_items.make(initial_container_size)
			end
			overridden_real_items.force(a_value,a_item_id)
		end

	set_overridden_double_item(a_value: OVERRIDE_DATA_RECORD[DOUBLE]; a_item_id: INTEGER) is
		-- Set overridden double item
		do
			if overridden_double_items = Void then
				create overridden_double_items.make(initial_container_size)
			end
			overridden_double_items.force(a_value,a_item_id)
		end

	set_overridden_string_item(a_value: OVERRIDE_DATA_RECORD[STRING]; a_item_id: INTEGER) is
		-- Set overridden string item
		do
			if overridden_string_items = Void then
				create overridden_string_items.make(initial_container_size)
			end
			overridden_string_items.force(a_value,a_item_id)
		end

	set_overridden_image_item(a_value: OVERRIDE_DATA_RECORD[STRING]; a_item_id: INTEGER) is
		-- Set overridden image item
		do
			if overridden_image_items = Void then
				create overridden_image_items.make(initial_container_size)
			end
			overridden_image_items.force(a_value,a_item_id)
		end

	set_overridden_date_item(a_value: OVERRIDE_DATA_RECORD[ROSE_DATE]; a_item_id: INTEGER) is
		-- Set overridden date item
		do
			if overridden_date_items = Void then
				create overridden_date_items.make(initial_container_size)
			end
			overridden_date_items.force(a_value,a_item_id)
		end

	set_overridden_record_item(a_value: OVERRIDE_DATA_RECORD[ROSE_SIGNATURE_RECORD_ITEM]; a_name: STRING) is
		-- Set overridden record item
		do
			if overridden_records = Void then
				create overridden_records.make(10)
			end
			overridden_records.force(a_value,a_name)
		end

	set_overridden_signature_data(a_value: OVERRIDE_DATA_RECORD[G]) is
		-- Set overridden signature data
		do
			overridden_signature_data := a_value
		end

feature -- Differences

	delta_record (a_other: like Current): like Current is
			-- Return a record that is this record minus a_other
		do
			Result := duplicate
			if has_integer_items then
				if a_other.has_integer_items then
					from a_other.integer_items.start until a_other.integer_items.off loop
						Result.integer_items.search (a_other.integer_items.key_for_iteration)
						if Result.integer_items.found and then Result.integer_items.found_item = a_other.integer_items.item_for_iteration then
							Result.integer_items.remove (a_other.integer_items.key_for_iteration)
						end
						a_other.integer_items.forth
					end
				end
			end
			if has_real_items then
				if a_other.has_real_items then
					from a_other.real_items.start until a_other.real_items.off loop
						Result.real_items.search (a_other.real_items.key_for_iteration)
						if Result.real_items.found and then Result.real_items.found_item = a_other.real_items.item_for_iteration then
							Result.real_items.remove (a_other.real_items.key_for_iteration)
						end
						a_other.real_items.forth
					end
				end
			end
			if has_double_items then
				if a_other.has_double_items then
					from a_other.double_items.start until a_other.double_items.off loop
						Result.double_items.search (a_other.double_items.key_for_iteration)
						if Result.double_items.found then
							if (not undefined_constants.is_defined_double (Result.double_items.found_item)) and (not undefined_constants.is_defined_double (a_other.double_items.item_for_iteration)) then
								Result.double_items.remove (a_other.double_items.key_for_iteration)
							elseif Result.double_items.found_item = a_other.double_items.item_for_iteration then
								Result.double_items.remove (a_other.double_items.key_for_iteration)
							end
						end
						a_other.double_items.forth
					end
				end
			end
			if has_string_items then
				if a_other.has_string_items then
					from a_other.string_items.start until a_other.string_items.off loop
						Result.string_items.search (a_other.string_items.key_for_iteration)
						if Result.string_items.found then
							if Result.string_items.found_item = void and a_other.string_items.item_for_iteration = void then
								Result.string_items.remove (a_other.string_items.key_for_iteration)
							elseif Result.string_items.found_item ~ a_other.string_items.item_for_iteration then
								Result.string_items.remove (a_other.string_items.key_for_iteration)
							end
						end
						a_other.string_items.forth
					end
				end
			end
		end

feature -- Clone

	duplicate: like Current is
			-- Return a full copy of this record
		do
			Result:= Current.twin
			Result.clone_virtual_record (Current)
		end

	clone_virtual_record (a_record: like Current) is
			-- Users who inherit this class need to redefine this feature if they are dealing with historical versions.
			-- If there is any feature needed to bring forward from one version to another, the user is responsible for copying the feature(s) over.
			-- Furthermore, the copy has to be done in a way that set_integer_item, set_real_item, and set_xxxxx_item would not
			-- change any way value of any of its clones.
			-- This way, the clones will remain independent from the original object when setting item values.
		require
			a_record_not_void: a_record /= Void
		do
			if a_record.signature /= Void then
				signature := a_record.signature.twin
			end
			if a_record.integer_items_saved /= Void then
				integer_items_saved := a_record.integer_items.twin
			end
			if a_record.double_items_saved /= Void then
				double_items_saved := a_record.double_items_saved.twin
			end
			if a_record.real_items_saved /= Void then
				real_items_saved := a_record.real_items_saved.twin
			end
			if a_record.string_items_saved /= Void then
				string_items_saved := a_record.string_items_saved.twin
			end
			if a_record.image_items_saved /= Void then
				image_items_saved := a_record.image_items_saved.twin
			end
			if a_record.date_items_saved /= Void then
				date_items_saved := a_record.date_items_saved.twin
			end
			if a_record.records /= Void then
				records := a_record.records.twin
			end
			if a_record.indexed_records /=Void then
				create indexed_records.make(a_record.indexed_records.count)
				from
					a_record.indexed_records.start
				until
					a_record.indexed_records.off
				loop
					indexed_records.force(a_record.indexed_records.item_for_iteration.twin,
										  a_record.indexed_records.key_for_iteration.twin)
					a_record.indexed_records.forth
				end
			end
			if a_record.overridden_integer_items /= Void then
				overridden_integer_items := a_record.overridden_integer_items.twin
			end
			if a_record.overridden_real_items /= Void then
				overridden_real_items := a_record.overridden_real_items.twin
			end
			if a_record.overridden_double_items /= Void then
				overridden_double_items := a_record.overridden_double_items.twin
			end
			if a_record.overridden_string_items /= Void then
				overridden_string_items := a_record.overridden_string_items.twin
			end
			if a_record.overridden_image_items /= Void then
				overridden_image_items := a_record.overridden_image_items.twin
			end
			if a_record.overridden_date_items /= Void then
				overridden_date_items := a_record.overridden_date_items.twin
			end
			if a_record.overridden_records /= Void then
				overridden_records := a_record.overridden_records.twin
			end
			if a_record.overridden_signature_data /= Void then
				overridden_signature_data := a_record.overridden_signature_data.twin
			end
		end

	copy_integer_items(a_values: HASH_TABLE[INTEGER,INTEGER]) is
		-- Replace existing integer items
		do
			integer_items.clear_all
			from a_values.start until a_values.off loop
				set_integer_item(a_values.item_for_iteration,a_values.key_for_iteration)
				a_values.forth
			end
		end

	copy_real_items(a_values: HASH_TABLE[REAL,INTEGER]) is
		-- Replace existing integer items
		do
			real_items.clear_all
			from a_values.start until a_values.off loop
				set_real_item(a_values.item_for_iteration,a_values.key_for_iteration)
				a_values.forth
			end
		end

	copy_double_items(a_values: HASH_TABLE[DOUBLE,INTEGER]) is
		-- Replace existing double items
		do
			double_items.clear_all
			from a_values.start until a_values.off loop
				set_double_item(a_values.item_for_iteration,a_values.key_for_iteration)
				a_values.forth
			end
		end

	copy_string_items(a_values: HASH_TABLE[STRING,INTEGER]) is
		-- Replace existing string items
		do
			string_items.clear_all
			from a_values.start until a_values.off loop
				set_string_item(a_values.item_for_iteration,a_values.key_for_iteration)
				a_values.forth
			end
		end

	copy_image_items(a_values: HASH_TABLE[STRING,INTEGER]) is
		-- Replace existing image items
		do
			image_items.clear_all
			from a_values.start until a_values.off loop
				set_image_item(a_values.item_for_iteration,a_values.key_for_iteration)
				a_values.forth
			end
		end

	copy_date_items(a_values: HASH_TABLE[ROSE_DATE,INTEGER]) is
		-- Replace existing date items
		do
			date_items.clear_all
			from a_values.start until a_values.off loop
				set_date_item(a_values.item_for_iteration,a_values.key_for_iteration)
				a_values.forth
			end
		end

	copy_records(a_values: HASH_TABLE[ROSE_SIGNATURE_RECORD_ITEM,STRING]) is
			--copy records from a_values
		do
			records.clear_all
			from a_values.start until a_values.off loop
				set_record_item(a_values.item_for_iteration)
				a_values.forth
			end
		end

feature -- Terminating

	terminating_record: like Current is
			-- use this to create a record that will terminate all the *item* values stored in the current virtual record
			-- needs to be enhanced to handle termination of record types and signature_data in the virtual_record
		do
			create Result
			Result := duplicate
			if Result.integer_items_saved /= Void then
				from Result.integer_items_saved.start
				until Result.integer_items.off
				loop
					Result.integer_items_saved.force(undefined_constants.undefined_integer,Result.integer_items_saved.key_for_iteration)
					Result.integer_items_saved.forth
				end
			end
			if Result.real_items_saved /= Void then
				from Result.real_items_saved.start
				until Result.real_items_saved.off
				loop
					Result.real_items_saved.force(undefined_constants.undefined_real,Result.real_items_saved.key_for_iteration)
					Result.real_items_saved.forth
				end
			end
			if Result.double_items_saved /= Void then
				from Result.double_items_saved.start
				until Result.double_items.off
				loop
					Result.double_items_saved.force(undefined_constants.undefined_double,Result.double_items_saved.key_for_iteration)
					Result.double_items_saved.forth
				end
			end
			if Result.string_items_saved /= Void then
				from Result.string_items_saved.start
				until Result.string_items.off
				loop
					Result.string_items_saved.force(Void,Result.string_items_saved.key_for_iteration)
					Result.string_items_saved.forth
				end
			end
			if Result.image_items_saved /= Void then
				from Result.image_items_saved.start
				until Result.image_items_saved.off
				loop
					Result.image_items_saved.force(Void,Result.image_items_saved.key_for_iteration)
					Result.image_items_saved.forth
				end
			end
			if Result.date_items_saved /= Void then
				from Result.date_items_saved.start
				until Result.date_items.off
				loop
					Result.date_items_saved.force(Void,Result.date_items_saved.key_for_iteration)
					Result.date_items_saved.forth
				end
			end
			if Result.records /= Void then
				Result.records.clear_all
			end
		end

feature -- Added by India Team
	-- When more than one source is using the same signature key fields,
	-- data items may belong to more than one source.
	-- When processing each source, terminate only those data items
	-- which belong to that particular source and
	-- DON'T terminate other data items which belongs to some other source.
	-- So, before terminating the **item** values, check whether
	-- termination is required or not using the agent function.

	-- E.g. Multex fiscal_data_signatures will use same signature data
	-- for the following set of sources:
	-- Annual Balance Sheet
	-- Annual Income Statement
	-- Annual Cash Flow

	terminating_record_with_agent(a_agent: FUNCTION [ANY, TUPLE[INTEGER], BOOLEAN]): like Current is
			-- use this to create a record that will terminate all the *item* values for which the agent function returns true
			-- needs to be enhanced to handle termination of record types and signature_data in the virtual_record
		do
			create Result
			Result := duplicate
			if Result.integer_items_saved /= Void then
				from Result.integer_items_saved.start
				until Result.integer_items.off
				loop
					if a_agent.item([Result.integer_items_saved.key_for_iteration]) then
						Result.integer_items_saved.force(undefined_constants.undefined_integer,Result.integer_items_saved.key_for_iteration)
					end
					Result.integer_items_saved.forth
				end
			end
			if Result.real_items_saved /= Void then
				from Result.real_items_saved.start
				until Result.real_items_saved.off
				loop
					if a_agent.item([Result.real_items_saved.key_for_iteration]) then
						Result.real_items_saved.force(undefined_constants.undefined_real,Result.real_items_saved.key_for_iteration)
					end
					Result.real_items_saved.forth
				end
			end
			if Result.double_items_saved /= Void then
				from Result.double_items_saved.start
				until Result.double_items.off
				loop
					if a_agent.item([Result.double_items_saved.key_for_iteration]) then
						Result.double_items_saved.force(undefined_constants.undefined_double,Result.double_items_saved.key_for_iteration)
					end
					Result.double_items_saved.forth
				end
			end
			if Result.string_items_saved /= Void then
				from Result.string_items_saved.start
				until Result.string_items.off
				loop
					if a_agent.item([Result.string_items_saved.key_for_iteration]) then
						Result.string_items_saved.force(Void,Result.string_items_saved.key_for_iteration)
					end
					Result.string_items_saved.forth
				end
			end
			if Result.image_items_saved /= Void then
				from Result.image_items_saved.start
				until Result.image_items_saved.off
				loop
					if a_agent.item([Result.image_items_saved.key_for_iteration]) then
						Result.image_items_saved.force(Void,Result.image_items_saved.key_for_iteration)
					end
					Result.image_items_saved.forth
				end
			end
			if Result.date_items_saved /= Void then
				from Result.date_items_saved.start
				until Result.date_items.off
				loop
					if a_agent.item([Result.date_items_saved.key_for_iteration]) then
						Result.date_items_saved.force(Void,Result.date_items_saved.key_for_iteration)
					end
					Result.date_items_saved.forth
				end
			end
			if Result.records /= Void then
				Result.records.clear_all
			end
		end

feature -- Display

	debug_output: STRING is
			-- Descriptive string for the debugger
		local
			l_date: ROSE_DATE
		do
			if {l_signature : ROSE_SIGNATURE_RECORD[ENTITY]} signature then
				Result := l_signature.signature_id.out
				l_date := l_signature.as_of_date
				if l_date /= Void then
					Result := Result + ":" + l_date.debug_output
				end
			else
				Result := ""
			end
		end

feature {NONE} -- Helpers

	ieee_math: IEEE_MATH is
			-- Access to ieee_math utilities
		once
			create Result
		end

	undefined_constants: ROSE_IEEE_UNDEFINED is
			--undefined values
		once
			create Result
		end

	initial_container_size: INTEGER is 50
			-- The initial size for item tables at creation

feature {NONE} -- Storables

	correct_mismatch is
			-- No mismatch corrector
		do
		end

end



