indexing
	description: "A version history of ROSE_VIRTUAL_RECORDs corresponding to a signature, with as of date in descending order (when sorted)."
	author: "Srilatha Singh, Nelson Chui"
	date: "$Date: 2008/12/01 $"
	revision: "$Revision: #9 $"
	path: "$Archive $"	 

class
	ROSE_VIRTUAL_RECORD_HISTORY [G -> ROSE_SIGNATURE_RECORD [ENTITY] create default_create end]

create
	make_empty, make_one, make_from_array

feature -- Initialization
	
	make_empty is
		do
			create data.make_empty
			is_sorted := True
		end
	
	make_one (a_record: like record_anchor) is
		require
			a_record_not_void: a_record /= Void
		do
			create data.make_empty
			data.extend (a_record)
			is_sorted := True
		end
	
	make_from_array (a_array : ARRAY [like record_anchor]) is
		require
			have_common_signature: have_common_signature (a_array)
		do
			create data.make_from_array (a_array)
			is_sorted := False
		ensure
			equal_size: data.count = a_array.count
		end

feature -- Access
	
	historical: BOOLEAN
	
	data: ROSE_LINEAR_ARRAY [like record_anchor]
	
feature -- Basic operations
	
	set_historical (a_value: BOOLEAN) is
		do
			historical := a_value
		end
	
	extend (a_record: like record_anchor) is
		require
			a_record_valid: a_record /= Void and (data.count > 0 implies a_record.signature.multi_key.is_equal (multi_key))
		do
			is_sorted := is_sorted and then (data.count > 0) implies (a_record.signature.as_of_date < data.last.signature.as_of_date)
			data.extend (a_record)
		end

	sort_data is
		do
			if not is_sorted then
				data.sort_by (agent sort_record)
			end
		end

	item (a_as_of_date: ROSE_DATE): like record_anchor is
			-- Assuming data is in a strictly chronological order
			-- Return the most recent record if a_as_of_date = Void
		do
			Result := Void
			if data.count > 0 then
				-- Direct acces if only one element in array
				if data.count = 1 then
					if a_as_of_date = Void or else data.first.signature.as_of_date <= a_as_of_date then
						Result := data.first
					end
					
				-- Search from array
				else
					-- Linear search
					if data.count <= 8 then
						Result := linear_data_search (a_as_of_date)
					-- Binary search
					else
						if not is_sorted then
							data.sort_by (agent sort_record)
							is_sorted := True
						end
						-- Most recent record if a_as_of_date = Void
						if a_as_of_date = Void then
							Result := data.first
						else
							data.dynamic_binary_search (agent compare_record (?, a_as_of_date))
							if data.found_index <= data.upper then
								Result := data.item (data.found_index)
							end
						end
					end
				end
			end
		end

feature -- Basic operations
	
	record_anchor: ROSE_VIRTUAL_RECORD [G] is do end
	
	multi_key: MULTI_KEY is
		do
			if data.count > 0 then
				Result := data.first.signature.multi_key
			end
		end

	have_common_signature (a_array: ARRAY [like record_anchor]): BOOLEAN is
		local
			i: INTEGER
		do
			Result := True
			if a_array /= Void then
				from 
					i := a_array.lower
				until 
					not Result or i > a_array.upper
				loop
					Result := (a_array.item (a_array.lower).signature.multi_key.is_equal (a_array.item (i).signature.multi_key))
					i := i + 1
				end
			end
		end
	
feature {NONE} -- Implementation
	
	is_sorted: BOOLEAN

	sort_record (a, b: like record_anchor): BOOLEAN is
		do
			Result := (a.signature.as_of_date > b.signature.as_of_date)
		end

	compare_record (a: like record_anchor; b: ROSE_DATE): INTEGER is
		do
			if (a.signature.as_of_date < b) then
				Result := -1
			elseif (a.signature.as_of_date > b) then
				Result := 1
			else
				Result := 0
			end
		end

	linear_data_search (a_as_of_date: ROSE_DATE): like record_anchor is
			-- Return the most recent record if a_as_of_date = Void
		local
			l_this_date, l_best_date: ROSE_DATE
		do
			-- default value -- Result := Void
			create l_best_date.make_min_date
			from
				data.start
			until
				data.off or (is_sorted and Result /= Void)
			loop
				l_this_date := data.current_item.signature.as_of_date
				if l_this_date > l_best_date then 
					if a_as_of_date = Void or else l_this_date <= a_as_of_date then 
						Result := data.current_item
						l_best_date := Result.signature.as_of_date
					end
				end
				data.forth
			end
		end
			
end -- class ROSE_VIRTUAL_RECORD_HISTORY
