indexing
	description: "Common ancestors to widgets developped by the RC"
	author: "Zoran Simic"
	date: "$Date: 2010/02/22 $"
	revision: "$Revision: #18 $"
	archive: "$File: //rose/source/kernel/infrastructure/widgets/rose_widget.e $"

deferred class
	ROSE_WIDGET

inherit
	EV_CELL
		rename
			show as ev_show,
			hide as ev_hide,
			item as ev_item,
			count as ev_count,
			-- Please do not use these inherited routines! use 'put' only, and use it once
			extend as ev_forbidden_extend,
			prune as ev_forbidden_prune,
			prune_all as ev_forbidden_prune_all,
			wipe_out as ev_forbidden_wipe_out
		export
			{ROSE_WIDGET} all
			{ANY} set_minimum_width, set_minimum_height, set_minimum_size, minimum_width, minimum_height, parent, is_show_requested
			{ANY} background_color, foreground_color, set_background_color, set_foreground_color
			{ANY} x_position, y_position, screen_x, screen_y, width, height
			{ANY} enable_sensitive, disable_sensitive
			{ANY} set_pointer_style, is_destroyed
			{ANY} drop_actions, pebble, set_pebble, remove_pebble
			{ANY} is_displayed, is_sensitive
		redefine
			initialize, set_background_color, set_foreground_color
		end
	ROSE_SHARED_GLOBAL_GUI
		export
			{ROSE_WIDGET} all
		undefine
			default_create, copy, is_equal
		end
	DEBUG_OUTPUT
		undefine
			default_create, copy, is_equal
		end

feature {NONE} -- Initialization

	make (a_container: like container_template) is
			-- Create widget and put it in 'a_container'.
		do
			default_create
			gui.factory.enbox_and_size (Current, a_container, default_width, default_height)
		end

	make_with_title (a_title: STRING; a_container: like container_template) is
			-- Make with 'a_title' and in 'a_container'
		do
			make (a_container)
			set_title (a_title)
		end

	initialize is
			-- Called by 'default_create'.
		do
			create change_event
			Precursor
			setup_widget
		end

	setup_widget is
			-- Setup this widget, do not populate it yet, just build out the structure.
			-- To populate the contents in your widget, use 'refresh'
		deferred
		end

	default_width: INTEGER		-- Default width (if positive, this widget will by default not resize horizontally)
	default_height: INTEGER		-- Default height (if positive, this widget will by default not resize vertically)

feature -- Access

	icon: STRING is
			-- Icon that represents this widget (optional)
		do
			Result := internal_icon
		end

	title: STRING is
			-- 'title' for this widget (optional)
		do
			Result := internal_title
		end

	parent_window: EV_WINDOW is
			-- Current parent window of this widget
		do
			Result := gui.parent_window (Current)
		end

	change_event: ROSE_SIMPLE_EVENT
			-- Fired whenever this widget is changed
			-- It is up to the descendant to choose when to fire this event
			-- By default, ROSE_WIDGET fires this event when 'title' is changed

	debug_output: STRING is
			-- String that should be displayed in debugger to represent `Current'.
		do
			if title /= Void and then not title.is_empty then
				Result := title
			else
				Result := generating_type
			end
		end

feature -- Basic operations

	show (a_show: BOOLEAN) is
			-- Show or hide this widget.
			-- This is a convenience feature
		do
			if a_show then
				ev_show
			else
				ev_hide
			end
		end

	set_title (a_title: STRING) is
			-- Set 'title'
			-- Note: we don't guarantee that 'title' is identical to 'a_title' given here.
			-- One example where this can be different, in ptrade when 'a_title' given is "transactions" for example,
			-- the actual 'title' is redefined to return "transactions [15]" to indicate how many transactions there are...
		do
			internal_title := a_title
			change_event.call
		end

	set_icon (a_icon: STRING) is
			-- Set 'icon'
			-- Note: we don't guarantee that 'icon' is identical to 'a_icon' given here.
			-- One can redefine 'icon' appropriately
		do
			internal_icon := a_icon
			change_event.call
		end

	refresh is
			-- Refresh the contents of this widget.
		deferred
		end

	set_foreground_color (a_color: like foreground_color)
			-- Assign `a_color' to `foreground_color'.
		do
			Precursor {EV_CELL} (a_color)
			propagate_foreground_color
		end

	set_background_color (a_color: like background_color)
			-- Assign `a_color' to `background_color'.
		do
			Precursor {EV_CELL} (a_color)
			propagate_background_color
		end

feature {NONE} -- Implementation

	internal_title: STRING			-- Title for this widget (can be used when widget is part of a notebook for example)
	internal_icon: STRING			-- Icon for this widget (can be used when widget is part of a notebook for example)

	output_in (a_field: EV_TEXTABLE; a_object: ANY) is
			-- Output 'a_string' to 'a_field' (allows Void for 'a_string', which Vision2 doesn't...)
			-- This is a convenience feature allowing to avoid always checking whether things are Void or not
		require
			field_ok: a_field /= Void
		local
			l_text: STRING
		do
			if a_object /= Void then
				l_text := a_object.out
			end
			if l_text = Void then
				a_field.remove_text
			else
				a_field.set_text (l_text)
			end
		end

feature {NONE} --Templates

	container_template: EV_CONTAINER is
			-- Container template used in make_* routines, useful only if you wish to constrain where this type of widget can be added to via make_*
			-- Note: it is always possible to extend this widget anywhere if one uses 'default_create' instead of the make_* routines
		do end

end
