indexing
	description: "Window repositionner"
	notice: "[
		Do not create instances (it's useless), use Gui.repositionner (inherit ROSE_SHARED_GLOBAL_GUI).
		Do not use this class directly, use Gui.open_window("my_window_name",my_window)
		]"
	author: "Zoran Simic"
	date: "$Date: 2009/03/31 $"
	revision: "$Revision: #12 $"

class
	ROSE_WINDOW_REPOSITIONNER

inherit
	ROSE_SHARED_GLOBAL_GUI

create
	make

feature {NONE} -- Initialization

	make is
			-- Creation procedure.
		do
			create names.make (0)
			create default_sizes.make_empty
		end

feature -- Access

	names: HASH_TABLE [EV_POSITIONABLE, STRING]
			-- Naming table

	default_sizes: ROSE_LINEAR_ARRAY [TUPLE [positionable: EV_POSITIONABLE; width: INTEGER; height: INTEGER]]
		-- All default sizes registered.

	name (a_positionable: EV_POSITIONABLE): STRING is
			-- Name 'a_positionable' was registered with
		do
			from
				names.start
			until
				Result /= Void or else names.off
			loop
				if a_positionable = names.item_for_iteration then
					Result := names.key_for_iteration
				end
				names.forth
			end
		end

	default_size (a_positionable: EV_POSITIONABLE): TUPLE [width: INTEGER; height: INTEGER] is
			-- Default size associated with `a_positionable'. Void if None.
		do
			from
				default_sizes.start
			until
				Result /= Void or default_sizes.off
			loop
				if default_sizes.current_item.positionable = a_positionable then
					Result := [default_sizes.current_item.width, default_sizes.current_item.height]
				end
				default_sizes.forth
			end
		end

feature -- Basic operations

	register (a_name: STRING; a_positionable: EV_POSITIONABLE) is
			-- Register 'a_positionable' under 'a_name' in GUI configuration.
		require
			valid_name: a_name /= Void and then not a_name.is_empty
			valid_positionable: a_positionable /= Void and then not a_positionable.is_destroyed
		do
			if names.item (a_name) /= a_positionable then
				names.force (a_positionable, a_name)
			end
		end

	set_default_size (a_positionable: EV_POSITIONABLE; a_width, a_height: INTEGER) is
			-- Assign `a_width' and `a_height' as the default size of `a_positionable'.
		require
			valid_positionable: a_positionable /= Void and then not a_positionable.is_destroyed
		local
			l_default_size: TUPLE [width: INTEGER; height: INTEGER]
		do
			l_default_size := default_size (a_positionable)
			if l_default_size = Void then
				default_sizes.extend ([a_positionable, a_width, a_height])
			end
		end

	remember (a_positionable: EV_POSITIONABLE): STRING is
			-- Remember position of 'a_positionable', and return it's name if any.
		require
			valid_positionable: a_positionable /= Void and then not a_positionable.is_destroyed
		local
			l_width, l_height: INTEGER
		do
			Result := name (a_positionable)
			if Result /= Void and then not Result.is_empty then
				l_width := capped (a_positionable.width, k_max_width)
				l_height := capped (a_positionable.height, k_max_height)
				Global.login.config.set (k_size_key + Result,l_width.out + k_size_separator + l_height.out)
			end
		end

	restore (a_positionable: EV_POSITIONABLE) is
			-- Restore position of 'a_positionable'.
		require
			valid_positionable: a_positionable /= Void and then not a_positionable.is_destroyed
		local
			l_name: STRING
			l_pair: PAIR [STRING,STRING]
			i: INTEGER
			l_default_size: TUPLE [width: INTEGER; height: INTEGER]
		do
			l_name := name (a_positionable)
			l_default_size := default_size (a_positionable)
			if l_name /= Void and then not l_name.is_empty then
				l_pair := Global.formatter.split_on_first (Global.login.config.value (k_size_key + l_name),k_size_separator,Void)
				if l_pair /= Void then
					i := to_integer (l_pair.first, k_max_width)
					if i > 0 then
						a_positionable.set_width (i)
					elseif l_default_size /= Void then
						a_positionable.set_width (l_default_size.width)
					end
					i := to_integer (l_pair.second, k_max_height)
					if i > 0 then
						a_positionable.set_height (i)
					elseif l_default_size /= Void then
						a_positionable.set_height (l_default_size.height)
					end
				end
			elseif l_default_size /= Void then
				a_positionable.set_size (l_default_size.width, l_default_size.height)
			end
		end

	position_near (a_widget: EV_WIDGET; a_window: EV_POSITIONABLE) is
			-- Position 'a_window' near 'a_widget' on the screen.
		local
			l_x, l_y, l_wx, l_wy: INTEGER
		do
			if a_widget /= Void and then a_window /= Void then
				l_x := a_widget.screen_x
				l_y := a_widget.screen_y + a_widget.height
				l_wx := a_window.width
				l_wy := a_window.height
				if l_x + l_wx + 2 < Gui.rep.screen.width then
					a_window.set_x_position (l_x)
				else
					a_window.set_x_position (Gui.rep.screen.width - l_wx)
				end
				if l_y + l_wy + 2 < Gui.rep.screen.height then
					a_window.set_y_position (l_y + 2)
				else
					a_window.set_y_position (Gui.rep.screen.height - l_wy - a_widget.height - 10)
				end
			end
		end

feature {NONE} -- Implementation

	to_integer (a_string: STRING; a_cap: INTEGER): INTEGER is
			-- Integer in 'a_string' if any
		do
			if a_string /= Void and then a_string.is_integer then
				Result := capped (a_string.to_integer, a_cap)
			end
		end

	capped (a_integer, a_cap: INTEGER): INTEGER is
			-- 'a_integer' capped to interval 0 .. 'a_cap'
		do
			Result := a_integer.max (0).min (a_cap)
		end

	k_size_key: STRING is "size/"			-- Configuration node to keep window sizes in
	k_size_separator: STRING is " "			-- Separator for sizes
	k_max_width: INTEGER is 1920			-- Max width
	k_max_height: INTEGER is 1600			-- Max height

end
