indexing
	description: "Objects that write XSLT error messages to a ROSE_ERROR_HANDLER"
	author: "$Author: it $"
	date: "$Date: 2008/12/01 $"
	revision: "$Revision: #3 $"
	archive: "$File: //rose/source/kernel/infrastructure/xslt/rose_xslt_error_listener.e $"

class ROSE_XSLT_ERROR_LISTENER

inherit

	XM_XSLT_ERROR_LISTENER

	XM_XPATH_ERROR_TYPES
		export {NONE} all end

	XM_XPATH_STANDARD_NAMESPACES
		export {NONE} all end

	KL_IMPORTED_STRING_ROUTINES
		export {NONE} all end

	ROSE_SHARED_GLOBAL
		rename
			error as error_handler
		export
			{NONE} all
		end

create

	make

feature {NONE} -- Initialization

	make (a_recovery_policy: like recovery_policy) is
			-- Initialize `Current'.
		require
			a_recovery_policy_in_range: a_recovery_policy >= Recover_silently and a_recovery_policy <= Do_not_recover	
		do
			recovery_policy := a_recovery_policy
			warning_threshold := 25
			recoverable_error_threshold := 25
			create error_change_stack.make_default
		ensure
			recovery_policy_set: recovery_policy = a_recovery_policy
		end

feature -- Access

	warnings, errors, fatal_errors: INTEGER
			-- Error counts

feature -- Events
	
	warning (a_message: STRING; a_locator: XM_XPATH_LOCATOR) is
			-- Receive notification of a warning.
		local
			l_error: XM_XPATH_ERROR_VALUE
		do
			if warnings_are_recoverable_errors then
				create l_error.make_from_string (a_message, Gexslt_eiffel_type_uri, "WARNING", Dynamic_error)
				error (l_error)
			elseif (warning_threshold > 0 and then warnings <= warning_threshold) or warning_threshold < 0 then
				if a_locator /= Void then display_location_information (a_locator, False) end
				error_handler.warn (a_message, Void)
				error_handler.flush
				warnings := warnings + 1
				if warnings > warning_threshold and warning_threshold > 0 then
					error_handler.warn ("No more warnings will be displayed", Void)
					error_handler.flush
				end
			end
		end

	error (a_error: XM_XPATH_ERROR_VALUE) is
			-- Receive notification of a recoverable error.
		local
			l_msg: STRING
		do
			if recovery_policy /= Recover_silently then
				display_location_information (a_error, True)
			end
			recovered := True
			if recovery_policy = Recover_with_warnings then
				l_msg := "Recoverable error: "
			elseif recovery_policy = Do_not_recover then
				recovered := False
				l_msg := "Error: "
			end
			if recovery_policy /= Recover_silently then
				error_handler.signal (generating_type, STRING_.concat (l_msg, a_error.error_message), Void)
				error_handler.flush_and_acknowledge
			end
			errors := errors + 1
			if recovery_policy = Recover_with_warnings and recoverable_error_threshold >=0 and errors >= recoverable_error_threshold then
				recovery_policy := Recover_silently
				error_handler.warn ("No more recoverable errors will be displayed", Void)
				error_handler.flush
			end
		end

	fatal_error (a_error: XM_XPATH_ERROR_VALUE) is
			-- Receive notification of a non-recoverable error.
		do
			display_location_information (a_error, True)
			error_handler.signal (generating_type, STRING_.concat ("Fatal error: ", a_error.error_message), Void)
			error_handler.flush_and_acknowledge
			fatal_errors := fatal_errors + 1
		end

feature {NONE} -- Implementation

	display_location_information (a_locator: XM_XPATH_LOCATOR; is_an_error: BOOLEAN) is
			-- Display location information.
		require
			locator_not_void: a_locator /= Void
		local
			l_message: STRING
		do
			if a_locator.line_number > 0 then
				l_message := "At line " + a_locator.line_number.out + " in " + a_locator.system_id + ":"
			else
				l_message := "In " + a_locator.system_id + ":"
			end
			if is_an_error then
				error_handler.signal (generating_type, l_message, Void)
				error_handler.flush_and_acknowledge
			else
				error_handler.warn (l_message, Void)
				error_handler.flush
			end
		end

end
