note
	description: "Traded and settled account cash position"
	author: "Sami Ben Grine & Ivan Tarnavsky"
	last_editor: "$Author: sm $"
	date: "$Date: 2013/06/26 $"
	revision: "$Revision: #1 $"
	path: "$File: //rose/source/kernel/expert_system/rp_account_cash_position.e $"

class
	RP_ACCOUNT_CASH_POSITION

inherit
	RP_CASH_POSITION [RP_INVESTMENT_ACCOUNT]

create
	make

feature {NONE} -- Initialization

	make (a_row: NT_CASH)
			-- Make with 'a_row'
		require
			row_exists: a_row /= Void
		do
			create global_investment_date.make_by_date (a_row.global_investment_date)
			create update_mode.make_from_string (a_row.update_mode)

			account := manager.reference_system.investment_account_by_code (a_row.account_code)

			create accounting_date.make_by_date (a_row.accounting_date)

			currency := manager.reference_system.currency_from_iso_code (a_row.iso_currency_code)

			total_quantity :=  currency.monies (a_row.total_quantity)
			create accounting_basis.make_from_code (a_row.accounting_basis.item (1))
			create cash_account_category.make_from_string (a_row.cash_account_category)

			unavailable := currency.monies (a_row.quantity_unavailable)

			if a_row.market_value_currency /= Void and then not a_row.market_value_currency.is_empty then
				market_value_currency := manager.reference_system.currency_from_iso_code (a_row.market_value_currency)
				if market_value_currency /= Void and then a_row.market_value_amount /= Void then
					market_value := market_value_currency.monies (a_row.market_value_amount)
				end
			end

			set_errors (a_row.errors)
		ensure
			valid_gid: (global_investment_date /= Void) and then (global_investment_date ~ a_row.global_investment_date)
			valid_update_mode: update_mode /= Void and then (update_mode.out ~ a_row.update_mode)
			valid_account_code: (account /= Void) and then (account.code ~ a_row.account_code)
			valid_accounting_date: (accounting_date /= Void) and then (accounting_date ~ a_row.accounting_date)
			valid_iso_currency_code: (currency /= Void) and then (currency.iso ~ a_row.iso_currency_code)
			valid_accounting_basis: (accounting_basis /= Void) and then (accounting_basis.code.out ~ a_row.accounting_basis)
			valid_cash_account_category: (cash_account_category /= Void) and then (cash_account_category ~ a_row.cash_account_category)
			valid_total_quantity: (total_quantity /= Void) and then (total_quantity.amount ~ a_row.total_quantity)
			valid_errors: (errors /= Void)
			valid_quantity_unavailable: (unavailable /= Void) and then (unavailable.amount ~ a_row.quantity_unavailable)
			valid_market_value_amount: (market_value /= Void) implies (market_value_currency /= Void and then market_value.currency ~ market_value_currency and then market_value.amount ~ a_row.market_value_amount)
			valid_market_value_currency: (market_value_currency /= Void) implies (market_value_currency.iso ~ a_row.market_value_currency)
		end

end
