note
	description: "Traded or settled accounting basis for positions"
	author: "Sami Ben Grine & Ivan Tarnavsky"
	last_editor: "$Author: sm $"
	date: "$Date: 2013/06/26 $"
	revision: "$Revision: #1 $"
	path: "$File: //rose/source/kernel/expert_system/rp_accounting_basis.e $"

class RP_ACCOUNTING_BASIS

create
	make_from_code,
	make_traded,
	make_settled

feature {NONE} -- Initialization

	make_from_code (a_code: CHARACTER)
			-- Initialize with 'a_code'
		require
			valid_code: a_code=traded_code or else a_code=settled_code
		do
			code := a_code
		ensure
			set_traded: code=traded_code implies is_traded
			set_settled: code=settled_code implies is_settled
		end

	make_traded
			-- Make 'T'
		do
			make_from_code (traded_code)
		ensure
			is_traded: is_traded
		end

	make_settled
			-- Make 'S'
		do
			make_from_code (settled_code)
		ensure
			is_settled: is_settled
		end

feature -- Access

	code: CHARACTER
		-- Code

feature -- Status Report

	is_traded: BOOLEAN
		-- Traded basis
		do
			Result := code = traded_code
		end

	is_settled: BOOLEAN
		-- Settled Basis
		do
			Result := code = settled_code
		end

feature -- Constants

	traded_code: CHARACTER = 'T'
		-- Traded

	settled_code: CHARACTER = 'S'
		-- Settled

invariant

	traded_or_settled: is_traded xor is_settled

end
