indexing

	description:

		"Default AXAR currency market data source"

	date: "$Date: 2010/03/01 $"
	revision: "$Revision: #13 $"

class RP_AXAROSENBERG_CURRENCY_MARKET_DATA_SOURCE

inherit

	RP_CURRENCY_MARKET_DATA_SOURCE

	DOUBLE_MATH
		export {NONE} all end

create

	make

feature {NONE} -- Initialization

	make (a_ref: like reference_system) is
			-- Initialize `Current'
		do
			reference_system := a_ref
			create fx_t_cache.make (100)
		end

feature -- Query

	spot_rate (a_bcu, a_fcu: RP_CURRENCY; a_date: ROSE_DATE): RP_DECIMAL is
			-- Spot fx rate from `a_bcu' to `a_fcu' on `a_date'
		do
			Result := market.exchange_rate (a_fcu.currency, a_bcu.currency, a_date)
		end


	short_term_rate (a_currency: RP_CURRENCY; a_near_date, a_far_date: ROSE_DATE): RP_DECIMAL is
			-- Short term interest rate for `a_currency' over given time period
		local
			l_dates: ARRAY[ROSE_DATE]
			l_rate_quotes: ARRAY[FX_DBA]
			l_fx_t: FX_T
			l_log_bid, l_log_ask, l_alpha: DOUBLE
			i,l_low,l_high: INTEGER
			l_days_in_year:INTEGER
		do
			create l_rate_quotes.make (1, 5)
			create l_dates.make (1, 5)
			from i := 1
			until i > 5
			loop
				l_dates[i] := create {ROSE_DATE}.make_by_date (a_near_date)
				if i <= 3 then
					l_dates[i].add_n_months(i)
				elseif i = 4 then
					l_dates[i].add_n_months(6)
				elseif i = 5 then
					l_dates[i].add_n_months (12)
				end
				i := i + 1
			end


			l_fx_t := fx_t_record (a_currency, a_near_date )


			if l_fx_t /= Void and then l_fx_t.euro_1m /= Void and then l_fx_t.euro_1m.bid > 0.0 and l_fx_t.euro_1m.ask > 0.0 then

				l_rate_quotes[1] := l_fx_t.euro_1m
				l_rate_quotes[2] := l_fx_t.euro_2m
				l_rate_quotes[3] := l_fx_t.euro_3m
				l_rate_quotes[4] := l_fx_t.euro_6m
				l_rate_quotes[5] := l_fx_t.euro_12m

				from i := 1
				until i > 5
				loop
					if l_rate_quotes[i].ask > 0 and l_rate_quotes[i].bid > 0 and l_dates[i] <= a_far_date then
						l_low := i
					end
					i := i + 1
				end

				from i := 5
				until i < 1
				loop
					if l_rate_quotes[i].ask > 0 and l_rate_quotes[i].bid > 0 and l_dates[i] > a_far_date then
						l_high := i
					end
					i := i - 1
				end

				l_days_in_year := days_in_year (a_currency)
				if l_low > 0 and l_high > 0 then
					l_alpha := a_far_date.diff_in_days (l_dates[l_low]) / l_dates[l_high].diff_in_days (l_dates[l_low])
					l_log_bid := log (1 + 0.01 * l_rate_quotes[l_high].bid * 365 / l_days_in_year) * l_alpha + (1 - l_alpha) * log (1 + 0.01 * l_rate_quotes[l_low].bid * 365 / l_days_in_year)
					l_log_ask := log (1 + 0.01 * l_rate_quotes[l_high].ask * 365 / l_days_in_year) * l_alpha + (1 - l_alpha) * log (1 + 0.01 * l_rate_quotes[l_low].ask * 365 / l_days_in_year)
					Result := sqrt (l_log_bid * l_log_ask)
				elseif l_low > 0 and l_high <= 0 then
					l_log_bid :=   log (1 + 0.01 * l_rate_quotes[l_low].bid * 365 / l_days_in_year)
					l_log_ask :=   log (1 + 0.01 * l_rate_quotes[l_low].ask * 365 / l_days_in_year)
					Result := sqrt (l_log_bid * l_log_ask)
				elseif l_low <= 0 and l_high > 0 then
					l_log_bid := log (1 + 0.01 * l_rate_quotes[l_high].bid * 365 / l_days_in_year)
					l_log_ask := log (1 + 0.01 * l_rate_quotes[l_high].ask * 365 / l_days_in_year)
					Result := sqrt (l_log_bid * l_log_ask)
				else
					Result := 1.0
				end
				Result := exp (Result * a_far_date.diff_in_days (a_near_date) / 365)
			else
				Result := 1.0
			end
		end

feature {NONE} -- Implementation

	reference_system: RP_REFERENCE_SYSTEM
			-- Reference system

	market: RP_LEGACY_PERF_FX is
			-- Currency market
		once
			create Result
		end

	fx_t_cache: HASH_TABLE [like fx_t_records_template, RP_CURRENCY]
			-- Cached FX_T records by currency

	fx_t_records_template: HASH_TABLE [like fx_template, ROSE_DATE] is
			-- Template for fx_t array
		do
		end

	fx_template : TUPLE[fx: FX_T; is_valid: BOOLEAN] is
			-- Template for cached fx record
		do
		end

	fx_t_record (a_currency: RP_CURRENCY; a_date: ROSE_DATE): FX_T is
			-- fx_t record for `a_currency' on `a_date'
		local
			l_date: ROSE_DATE
			l_date_key: ROSE_DATE
			l_fx_record: PAIR [FX_T, BOOLEAN]
			l_fx: like fx_template
			l_fx_t_records: like fx_t_records_template
		do
			l_date_key := a_date.date
			l_fx_t_records := fx_t_cache[a_currency]
			if l_fx_t_records = Void then
				create l_fx_t_records.make (1000)
				fx_t_cache[a_currency] := l_fx_t_records
			end
			l_fx := l_fx_t_records.item (l_date_key)
			if l_fx /= Void and then l_fx.is_valid then
				Result := l_fx.fx
			else
				from
					l_date := l_date_key
				until
					Result /= Void or l_date < cutoff_date
				loop
					if l_fx = Void then
						l_fx_record := a_currency.currency.fx_t_at (l_date)
						l_fx := [l_fx_record.first, l_fx_record.second]
						if l_fx.is_valid and not is_valid_fx(l_fx.fx) then
							-- a record was found, but it's not really valid
							l_fx.is_valid := False
						end
						l_fx_t_records.extend(l_fx, l_date)
					end
					if l_fx.is_valid then
						Result := l_fx.fx
					else
						l_date := l_date.previous_date
						l_fx := l_fx_t_records.item (l_date)
					end
				end
			end
		end

	is_valid_fx (a_fx: FX_T) : BOOLEAN is
			-- Is `a_fx' a valid fx_t record?
		do
			Result := a_fx /= Void and then a_fx.euro_1m /= Void and a_fx.euro_1m.bid > 0 and a_fx.euro_1m.ask > 0
		end

	cutoff_date: ROSE_DATE is
			-- Farthest lookback date
		once
			create Result.make_ccyymmdd(19850101)
		end

	days_in_year (a_currency:RP_CURRENCY): INTEGER is
			-- days in year
		do
			if a_currency.currency.iso.has_substring ("GB") then
				Result:=365
			else
				Result:=360
			end
		end

end
