indexing

	description:

		"Price Override Market Data Source"

	date: "$Date: 2010/04/30 $"
	fileid: "$Id: //rose/source/kernel/expert_system/mds/rp_axarosenberg_price_override_market_data_source.e#13 $"

class RP_AXAROSENBERG_PRICE_OVERRIDE_MARKET_DATA_SOURCE

inherit

	RP_MARKET_DATA_SOURCE_LAST_RECORD_SEARCH [RP_INSTRUMENT]

create

	make

feature {NONE} -- Initialization

	make (a_ref_system: RP_REFERENCE_SYSTEM) is
			-- Initialize `Current'
		require
			attached_ref_system: a_ref_system /= Void
		do
			reference_system := a_ref_system
		end

feature -- Query

	snapshot (a_instrument: RP_INSTRUMENT; a_date: ROSE_DATE): RP_MARKET_DATA_RECORD is
			-- Snapshot data for `a_instrument' on `a_date'
		local
			l_price: RP_PRICE
		do
			if price_override.has (a_instrument.brimsym) then
				l_price := price_override.item (a_instrument.brimsym).witness (a_date)
				if l_price /= Void then
					Result := create {RP_PRICE_OVERRIDE_MARKET_DATA_RECORD}.make (a_date, l_price)
				end
			end
		end

feature {NONE} -- Implementation

	reference_system: RP_REFERENCE_SYSTEM
			-- Ref system

	price_override: HASH_TABLE [ROSE_INTERVAL_HISTORY [RP_PRICE, ROSE_CLOSED_INTERVAL [RP_PRICE]], STRING] is
			-- Price override data
		do
			if cache = Void then
				create cache.make(0)
				retrieve (cache)
			end
			Result := cache
		end

	cache: like price_override
			-- Cached data.

	retrieve (a_hash: like price_override) is
			-- Load price overrides from db.
		local
			l_servers: DATABASE_ACCESS
			l_query: STRING_PREDICATE
		do
			create l_servers
			create l_query.make (l_servers.make_from_repository_fast_load ("performance"), "SELECT brimsym+'|'+cast(starting_date as varchar(8))+'|'+cast(ending_date as varchar(8))+'|'+cast(price as varchar(50))+'|'+currency FROM price_overrides")
			l_query.retrieve (Void, Void)
			l_query.array.do_all (agent add_override (a_hash, ?))
		end

	add_override (a_hash: like price_override; a_record: STRING) is
			-- Cache price override data.
		local
			l_per_instrument: ROSE_INTERVAL_HISTORY [RP_PRICE, ROSE_CLOSED_INTERVAL [RP_PRICE]]
			l_interval: ROSE_CLOSED_INTERVAL [RP_PRICE]
			l_instrument: RP_INSTRUMENT
			l_start, l_finish: ROSE_DATE
			l_currency: RP_CURRENCY
			l_tokenizer: ROSE_DELIMITED_TEXT
			l_price: RP_DECIMAL
		do
			create l_tokenizer.make (a_record, '|')
			create l_start.make_ccyymmdd (l_tokenizer.item (2).to_integer)
			l_instrument := reference_system.instrument_from_brimsym_at (l_tokenizer.item (1), l_start)
			if l_instrument /= Void then
				l_currency := reference_system.currency_from_iso_code (l_tokenizer.item (5))
				if l_currency /= Void then
					create l_finish.make_ccyymmdd (l_tokenizer.item (3).to_integer)
					create l_price.make_from_string (l_tokenizer.item (4))
					create l_interval.make (l_instrument.price_in_currency (l_price, l_currency, l_start), l_start, l_finish, 1.0)
					l_per_instrument := a_hash.item (l_instrument.brimsym)
					if l_per_instrument = Void then
						create l_per_instrument.make_empty
						a_hash.extend (l_per_instrument, l_instrument.brimsym)
					end
					l_per_instrument.extend (l_interval)
				end
			end
		end

end
