note

	description:
	"[
		Match GEDB event to BACKDIV records by
		minimizing the difference of BACKDIV cash rate
		and implied cash rate from the GEDN event
		in a +-5 day window
	]"
	author: "Ivan Tarnavsky"
	last_editor: "$Author: it $"
	date: "$Date: 2011/02/23 $"
	revision: "$Revision: #1 $"
	path: "$File: //rose/source/kernel/expert_system/legacy/rp_backdiv_matcher_with_local_minimum_search.e $"

class RP_BACKDIV_MATCHER_WITH_LOCAL_MINIMUM_SEARCH

inherit

	RP_GEDB_EVENT_TO_BACKDIV_MATCHER

	RP_AXAROSENBERG_AWARE
		export {NONE} all end

create {RP_LEGACY_SYSTEM}

	make

feature {NONE} -- Initialization

	make (a_events: DS_HASH_TABLE [RP_GEDB_RECORD, INTEGER]) is
			-- Initialize `Current'
		require
			a_events_not_void: a_events /= Void
			no_void_in_a_events: not a_events.has_item (Void)
		local
			l_dividends: DS_HASH_TABLE [RP_GEDB_RECORD, INTEGER]
		do
			create l_dividends.make_default
			a_events.do_if_with_key (agent l_dividends.force (?, ?), agent is_dividend_with_tax (?, ?))
			if l_dividends.is_empty then
				create cache.make (0)
			else
				create cache.make (l_dividends.count)
				assign_backdiv_records_to_events (l_dividends, cache)
			end
		end

feature -- Query

	match (a_event: RP_GEDB_EVENT): RP_BACKDIV_RECORD is
			-- <Precursor>
		do
			cache.search (a_event.icc)
			if cache.found then
				Result := cache.found_item
			end
		end

feature {NONE} -- Imeplementation

	date_range: INTEGER is 5
			-- Date range to look for backdiv records

	cache: DS_HASH_TABLE [RP_BACKDIV_RECORD, INTEGER]
			-- Event ICC to BACKDIV mappings

	assign_backdiv_records_to_events (a_events: DS_HASH_TABLE [RP_GEDB_RECORD, INTEGER]; a_mappings: like cache) is
			-- Map backdiv records to events
		require
			a_events_not_void: a_events /= Void
			a_events_not_empty: not a_events.is_empty
			no_void_in_a_events: not a_events.has_void_item
			a_mappings_not_void: a_mappings /= Void
			a_mappings_is_empty: a_mappings.is_empty
		local
			l_min_max_date_by_stock: UID_TABLE [PAIR [ROSE_DATE, ROSE_DATE], RP_STOCK]
			l_backdiv_records_by_stock: UID_TABLE [DS_HASH_TABLE [RP_BACKDIV_RECORD, INTEGER], RP_STOCK]
		do
			create l_min_max_date_by_stock.make_equal (a_events.count)
			find_min_max_dates_by_stock (a_events, l_min_max_date_by_stock)
			create l_backdiv_records_by_stock.make_equal (l_min_max_date_by_stock.count)
			l_min_max_date_by_stock.do_all (agent find_backdiv_records (l_backdiv_records_by_stock, ?))
			map_backdiv_records_to_events (a_events, a_mappings, l_backdiv_records_by_stock)
		end

	map_backdiv_records_to_events (a_events: DS_HASH_TABLE [RP_GEDB_RECORD, INTEGER]; a_mappings: like cache; a_backdiv_records: UID_TABLE [DS_HASH_TABLE [RP_BACKDIV_RECORD, INTEGER], RP_STOCK]) is
			-- Map `a_events' to `a_backdiv_records' and store mappings to `a_mappings'
		require
			a_events_not_void: a_events /= Void
			no_void_in_a_events: not a_events.has_void_item
			a_mappings_not_void: a_mappings /= Void
			a_mappings_is_empty: a_mappings.is_empty
			a_backdiv_records_not_void: a_backdiv_records /= Void
		do
			a_events.do_if (agent map_event (?, a_events, a_mappings, a_backdiv_records), agent not_mapped (a_mappings, ?))
		end

	map_event (a_event: RP_GEDB_RECORD; a_events: DS_HASH_TABLE [RP_GEDB_RECORD, INTEGER]; a_mappings: like cache; a_backdiv_records: UID_TABLE [DS_HASH_TABLE [RP_BACKDIV_RECORD, INTEGER], RP_STOCK]) is
			-- Map `a_event' to `a_backdiv_records' and store mappings to `a_mappings'
		require
			a_event_not_void: a_event /= Void
			a_events_not_void: a_events /= Void
			no_void_in_a_events: not a_events.has_void_item
			a_mappings_not_void: a_mappings /= Void
			a_mappings_is_empty: a_mappings.is_empty
			a_backdiv_records_not_void: a_backdiv_records /= Void
		local
			l_stock: RP_STOCK
			l_dividends: ROSE_LINEAR_ARRAY [RP_GEDB_RECORD]
		do
			l_stock ?= manager.reference_system.instrument_from_brimsym (a_event.instrument)
			if l_stock /= Void then
				a_backdiv_records.search (l_stock)
				if a_backdiv_records.found then
					create l_dividends.make_empty
					a_events.do_if (agent l_dividends.extend (?), agent is_same_exdate_and_stock (a_event.event_date, l_stock, ?))
					if not l_dividends.is_empty then
						match_dividends_to_backdiv_records (l_dividends, a_mappings, a_backdiv_records.found_item)
					end
				end
			end
		end

	not_mapped (a_mappings: like cache; a_event: RP_GEDB_RECORD): BOOLEAN is
			-- Does `a_mappings' not contain `a_event'?
		require
			a_mappings_not_void: a_mappings /= Void
			a_event_not_void: a_event /= Void
		do
			Result := not a_mappings.has (a_event.icc)
		end

	match_dividends_to_backdiv_records (a_dividends: ARRAY [RP_GEDB_RECORD]; a_mappings: like cache; a_backdiv_records: like cache) is
			-- Find best match of `a_backdiv_records' for `a_dividends' and store to `a_mappings'
		require
			a_dividends_not_void: a_dividends /= Void
			a_dividends_not_empty: not a_dividends.is_empty
			no_void_in_a_dividends: not a_dividends.has (Void)
			a_mappings_not_void: a_mappings /= Void
			a_backdiv_records_not_void: a_backdiv_records /= Void
		local
			l_backdiv_index: ROSE_LINEAR_ARRAY [INTEGER]
			l_best_variation: RP_VARIATIONS
			l_event: RP_GEDB_RECORD
			l_event_date: ROSE_DATE
		do
			l_event := a_dividends.item (1)
			l_event_date := l_event.event_date
			if l_event_date /= Void then
				create l_backdiv_index.make_empty
					-- pick backdiv records around ex-date
				a_backdiv_records.keys.do_if (agent l_backdiv_index.extend (?), agent is_within_date_range (a_backdiv_records, ?, l_event_date, date_range))
				if l_backdiv_index.count < a_dividends.count then
						-- append negative indexes for the number
						-- of elements in `a_dividends' so that we
						-- can test "empty" mappings for them and
						-- so that the set size is at least the number
						-- of items in `a_dividends'
					l_backdiv_index.append_array (((l_backdiv_index.count - a_dividends.count) |..| -1).as_array)
				end
					-- find best combination
				l_best_variation := best_variation (l_backdiv_index, a_dividends, a_backdiv_records)
					-- store mappings
				a_dividends.do_all_with_index (agent map (?, ?, l_best_variation, l_backdiv_index, a_mappings, a_backdiv_records))
			else
				(create {EXCEPTIONS}).raise ("NULL EVENT DATE for ICC " + l_event.icc.out)
			end
		end

	map (a_event: RP_GEDB_RECORD; a_index: INTEGER; a_best_variation: RP_VARIATIONS; a_backdiv_index: ARRAY [INTEGER]; a_mappings: like cache; a_backdiv_records: like cache) is
			-- Store mapping for `a_event'
		require
			a_event_not_void: a_event /= Void
			a_best_variation_not_void: a_best_variation /= Void
			valid_a_index: a_best_variation.valid_index (a_index)
			a_backdiv_index_not_void: a_backdiv_index /= Void
			a_mappings_not_void: a_mappings /= Void
			a_backdiv_records_not_void: a_backdiv_records /= Void
		local
			l_backdiv_index: INTEGER
			l_backdiv: RP_BACKDIV_RECORD
		do
			l_backdiv_index := a_backdiv_index.item (a_best_variation.item (a_index))
			a_backdiv_records.search (l_backdiv_index)
			if a_backdiv_records.found then
				l_backdiv := a_backdiv_records.found_item
				a_mappings.force (l_backdiv, a_event.icc)
					-- remove record from subsequent searches
				a_backdiv_records.remove (l_backdiv_index)
			end
			a_mappings.force (l_backdiv, a_event.icc)
		ensure
			mapping_stored: a_mappings.has (a_event.icc)
		end

	best_variation (a_backdiv_index: ARRAY [INTEGER]; a_dividends: ARRAY [RP_GEDB_RECORD]; a_backdiv_records: like cache): RP_VARIATIONS is
			-- Variation with mimimum rate difference
		require
			a_backdiv_index_not_void: a_backdiv_index /= Void
			a_dividends_not_void: a_dividends /= Void
			a_backdiv_records_not_void: a_backdiv_records /= Void
		local
			l_score: RP_DECIMAL
			l_best_score: RP_DECIMAL
			l_variations: RP_VARIATIONS
		do
			from
				create l_variations.make_variation (a_backdiv_index.count, a_dividends.count)
			until
				l_variations = Void
			loop
				l_score := compute_score (a_dividends, l_variations, a_backdiv_index, a_backdiv_records)
				if l_best_score = Void or else l_score < l_best_score then
					Result := l_variations
					l_best_score := l_score
				end
				l_variations := l_variations.next
			end
		ensure
			best_variation_not_void: Result /= Void
		end

	compute_score (a_dividends: ARRAY [RP_GEDB_RECORD]; a_variation: RP_VARIATIONS; a_backdiv_index: ARRAY [INTEGER]; a_backdiv_records: like cache): RP_DECIMAL is
			-- Compute score for current round
		require
			a_dividends_not_void: a_dividends /= Void
			a_variation_not_void: a_variation /= Void
			a_backdiv_index_not_void: a_backdiv_index /= Void
			a_backdiv_records_not_void: a_backdiv_records /= Void
			same_size: a_dividends.count = a_variation.k
		local
			i, j: INTEGER
			l_event: RP_GEDB_RECORD
			l_implied_rate: RP_DECIMAL
			l_cash_rate: RP_DECIMAL
			l_currency: RP_CURRENCY
			l_backdiv: RP_BACKDIV_RECORD
		do
			create Result.make_zero
			from
				i := 1
			until
				i > a_dividends.count
			loop
				j := a_backdiv_index.item (a_variation.item (i))
				a_backdiv_records.search (j)
				if not a_backdiv_records.found then
						-- penalty for unmatched event
					Result := Result + 1_000_000
				else
					l_backdiv := a_backdiv_records.found_item
					l_event := a_dividends.item (i)
					l_currency := manager.reference_system.currency_from_iso_code (l_event.currency)
					if l_currency = Void then
							-- penalty for invalid event currency
						Result := Result + 1_000_000
					else
						if l_event.performance_effect.is_zero then
							l_implied_rate := l_event.cash_effect / l_event.units_base
						else
							l_implied_rate := l_event.performance_effect / l_event.units_base
						end
						l_cash_rate := cashrate (l_currency, l_backdiv)
						Result := Result + (l_implied_rate - l_cash_rate) * (l_implied_rate - l_cash_rate)
					end
				end
				i := i + 1
			end
		end

	spot_rate: RP_LEGACY_PERF_FX is
			-- Legacy perf fx
		once
			create Result
		end

	cashrate (a_currency: RP_CURRENCY; a_backdiv: RP_BACKDIV_RECORD): RP_DECIMAL is
			-- Cashrate for given currency
		require
			a_currency_not_void: a_currency /= Void
			a_backdiv_not_void: a_backdiv /= Void
		local
			l_fx_rate: RP_DECIMAL
			l_ccy: RP_CURRENCY
		do
			Result := a_backdiv.cash_rate
			l_ccy := a_backdiv.currency
			if not l_ccy.is_same (a_currency) then
				l_fx_rate := spot_rate.exchange_rate (a_currency.currency, l_ccy.currency, a_backdiv.ex_date)
				Result := Result * l_fx_rate
			end
		end

	is_within_date_range (a_backdiv_records: like cache; a_index: INTEGER; a_exdate: ROSE_DATE; a_number_of_days: INTEGER): BOOLEAN is
			-- Is `a_backdiv_records.item (a_index).ex_date' within `a_number_of_days' of `a_exdate'?
		require
			a_backdiv_records_not_void: a_backdiv_records /= Void
			a_backdiv_records_has_a_index: a_backdiv_records.has (a_index)
			a_exdate_not_void: a_exdate /= Void
		do
			Result := a_backdiv_records.item (a_index).ex_date.diff_in_days (a_exdate).abs <= a_number_of_days
		end

	is_same_exdate_and_stock (a_exdate: ROSE_DATE; a_stock: RP_STOCK; a_event: RP_GEDB_RECORD): BOOLEAN is
			-- Does `a_event' has same `a_exdate' and `a_stock'?
		require
			a_exdate_not_void: a_exdate /= Void
			a_stock_not_void: a_stock /= Void
			a_event_not_void: a_event /= Void
		do
			if a_exdate ~ a_event.event_date then
				Result := a_stock ~ manager.reference_system.instrument_from_brimsym (a_event.instrument)
			end
		end

	find_backdiv_records (a_data: UID_TABLE [DS_HASH_TABLE [RP_BACKDIV_RECORD, INTEGER], RP_STOCK]; a_item: TUPLE [min_max: PAIR [ROSE_DATE, ROSE_DATE]; stock: RP_STOCK]) is
			-- Retrieve backdiv records
		require
			a_data_not_void: a_data /= Void
			a_item_not_void: a_item /= Void
			no_stock_data: not a_data.has (a_item.stock)
		local
			l_date: ROSE_DATE
			l_stock: RP_STOCK
			l_finish: ROSE_DATE
			l_corporate_actions: ARRAY [RP_CORPORATE_ACTION]
			l_source: RP_CAE_BACKDIV_DATA_SOURCE
			l_hash: like cache
		do
			from
				create l_hash.make_default
				l_stock := a_item.stock
				a_data.force (l_hash, l_stock)
				l_date := a_item.min_max.first.plus_n_date (-date_range)
				l_finish := a_item.min_max.second.plus_n_date (+date_range)
				l_source ?= manager.corporate_action_system.data_sources (l_stock, l_date).item (1)
			until
				l_date > l_finish or l_source = Void
			loop
				l_corporate_actions := l_source.instrument_action_and_events (l_stock, l_date)
				if l_corporate_actions /= Void then
					l_corporate_actions.do_all (agent index_corporate_action (a_data, l_stock, ?))
				end
				l_date := l_date.next_date
			end
		ensure
			data_has_stock: a_data.has (a_item.stock)
		end

	index_corporate_action (a_data: UID_TABLE [DS_HASH_TABLE [RP_BACKDIV_RECORD, INTEGER], RP_STOCK]; a_stock: RP_STOCK; a_ca: RP_CORPORATE_ACTION) is
			-- Index and store `a_ca' to `a_date'
		require
			a_data_not_void: a_data /= Void
			a_stock_not_void: a_stock /= Void
			data_has_stock: a_data.has (a_stock)
			a_ca_not_void: a_ca /= Void
		local
			l_hash: like cache
		do
			if attached {RP_BACKDIV_RECORD} a_ca.election as l_record then
				l_hash := a_data.item (a_stock)
				l_hash.force (l_record, l_hash.count + 1)
			end
		ensure
			ca_added: a_data.item (a_stock).count = 1 + old a_data.item (a_stock).count
			no_void_records: not a_data.item (a_stock).has_void_item
		end

	is_dividend_with_tax (a_event: RP_GEDB_RECORD; a_icc: INTEGER): BOOLEAN is
			-- Is `a_event' dividend with tax?
		require
			a_event_not_void: a_event /= Void
		do
			Result := a_event.is_dividend_taxed_event_type_33 or a_event.is_dividend_withdrawn_event_type_31
		end

	find_min_max_dates_by_stock (a_dividends: DS_HASH_TABLE [RP_GEDB_RECORD, INTEGER]; a_min_max_date_by_stock: UID_TABLE [PAIR [ROSE_DATE, ROSE_DATE], RP_STOCK]) is
			-- Find min and max dates per instrument
		require
			a_dividends_not_void: a_dividends /= Void
			a_min_max_date_by_stock_not_void: a_min_max_date_by_stock /= Void
			a_min_max_date_by_stock_is_empty: a_min_max_date_by_stock.is_empty
		do
			a_dividends.do_all (agent find_min_max_dates (?, a_min_max_date_by_stock))
		end

	find_min_max_dates (a_record: RP_GEDB_RECORD; a_min_max_date_by_stock: UID_TABLE [PAIR [ROSE_DATE, ROSE_DATE], RP_STOCK]) is
			-- Find min and max dates for `a_record.instrument'
		require
			a_record_not_void: a_record /= Void
			a_min_max_date_by_stock_not_void: a_min_max_date_by_stock /= Void
		local
			l_instrument: RP_INSTRUMENT
		do
			l_instrument := manager.reference_system.instrument_from_brimsym_at (a_record.instrument, a_record.event_date)
			if attached {RP_STOCK} l_instrument as l_stock then
				a_min_max_date_by_stock.search (l_stock)
				if a_min_max_date_by_stock.found then
					if a_record.event_date < a_min_max_date_by_stock.found_item.first then
						a_min_max_date_by_stock.found_item.set_first (a_record.event_date)
					end
					if a_record.event_date > a_min_max_date_by_stock.found_item.second then
						a_min_max_date_by_stock.found_item.set_second (a_record.event_date)
					end
				else
					a_min_max_date_by_stock.force (create {PAIR [ROSE_DATE, ROSE_DATE]}.make (a_record.event_date, a_record.event_date), l_stock)
				end
			end
		end

invariant

	cache_not_void: cache /= Void

end
