note

	description:

		"BACKDIV dividend information record"

	author: "Ivan Tarnavsky"
	last_editor: "$Author: it $"
	date: "$Date: 2011/02/23 $"
	revision: "$Revision: #25 $"
	path: "$File: //rose/source/kernel/expert_system/aflows/rp_backdiv_record.e $"

class RP_BACKDIV_RECORD

inherit

	RP_CORPORATE_ACTION_ELECTION [CAPACT_EXPRESSION, RP_DIVIDEND]
		rename
			make as make_election
		end

	IEEE_MATH
		export {NONE} all end

create {RP_CAE_BACKDIV_DATA_SOURCE}

	make

feature {NONE} -- Constructor

	make (a_backdiv: BACKDIV; a_reference_system: RP_REFERENCE_SYSTEM) is
			-- Initialize `Current'
		require
			a_backdiv_not_void: a_backdiv /= Void
			a_reference_system_not_void: a_reference_system /= Void
			ex_date_positive: a_backdiv.exdate > 0
			is_not_void: a_backdiv.iso /= Void
			txbas_not_void: a_backdiv.txbas /= Void
			cash_rate_is_defined: isfinite (a_backdiv.cash_rate) and not isnan (a_backdiv.cash_rate)
		do
			create ex_date.make_ccyymmdd (a_backdiv.exdate)
			currency := a_reference_system.currency_from_iso_code (a_backdiv.iso)
			create taxbas.make_from_string (a_backdiv.txbas)
			create cash_rate.make_from_double (a_backdiv.cash_rate)
		end

feature -- Access

	ex_date: ROSE_DATE
			-- Dividend exdate

	currency: RP_CURRENCY
			-- Dividend currency

	taxbas: STRING
			-- Dividend taxbase

	cash_rate: RP_DECIMAL
			-- Dividend cashrate

	contract (a_idate: ROSE_DATE; a_portfolio: RP_PORTFOLIO; a_holding: RP_QUANTITY): !RP_DIVIDEND is
			-- Create contract
		do
				-- Return Void on purpose. RP_BACKDIV_RECORD is used only in perf.
		end

invariant

	ex_date_not_void: ex_date /= Void
	currency_not_void: currency /= Void
	taxbas_not_void: taxbas /= Void
	cash_rate_not_void: cash_rate /= Void

end
