note

	description:

		"Cache to hold used backdiv records for given portfolio on given date"

	author: "gy"
	last_editor: "$Author: it $"
	date: "$Date: 2011/02/23 $"
	revision: "$Revision: #9 $"
	path: "$File: //rose/source/kernel/market_data/rp_backdiv_record_cache.e $"

class RP_BACKDIV_RECORD_CACHE

create {RP_BACKDIV_MATCHER_WITH_FIFO_ASSIGNMENT}

	make

feature {NONE} -- Initialization

	make is
			-- Reset table
		do
			create data.make_equal (0)
		ensure
			date_is_void: date = Void
		end

feature {RP_BACKDIV_MATCHER_WITH_FIFO_ASSIGNMENT} -- Basic operations

	has (a_stock: RP_STOCK): BOOLEAN is
			-- Do we have data for `a_stock'?
		require
			non_void_stock: a_stock /= Void
		do
			Result := data.has (a_stock)
		end

	item (a_stock: RP_STOCK): ARRAY [attached PAIR [attached RP_BACKDIV_RECORD, BOOLEAN]] is
			-- Backdiv records for `a_stock'
		require
			non_void_stock: a_stock /= Void
			has_stock: has (a_stock)
		do
			Result := data.item (a_stock)
		ensure
			item_not_void: Result /= Void
		end

feature {RP_BACKDIV_MATCHER_WITH_FIFO_ASSIGNMENT} -- Change

	initialize (a_date: ROSE_DATE) is
			-- Initialize cache for `a_date'
		require
			non_void_date: a_date /= Void
		do
			if date /~ a_date then
				create data.make_equal (0)
			end
			date := a_date
		ensure
			date_set: date = a_date
		end

	put (a_corpore_actions: ARRAY [RP_CORPORATE_ACTION]; a_stock: RP_STOCK) is
			-- Add dividends for `a_stock' from `a_corpore_actions'
		require
			non_void_array: a_corpore_actions /= Void
			non_void_stock: a_stock /= Void
		local
			l_backdiv: RP_BACKDIV_RECORD
			l_data: ROSE_LINEAR_ARRAY [attached PAIR [attached RP_BACKDIV_RECORD, BOOLEAN]]
			i, n: INTEGER
		do
			create l_data.make_empty
			from
				i := 1
				n := a_corpore_actions.count
			until
				i > n
			loop
				l_backdiv ?= a_corpore_actions.item (i).election
				if l_backdiv /= Void and then not l_backdiv.cash_rate.is_zero then
					l_data.extend (create {PAIR [attached RP_BACKDIV_RECORD, BOOLEAN]}.make (l_backdiv, False))
				end
				i := i + 1
			end
			data.force (l_data, a_stock)
		end

feature {NONE} -- Implementation

	date: ROSE_DATE
			-- Current processing date

	data: UID_TABLE [ROSE_LINEAR_ARRAY [attached PAIR [attached RP_BACKDIV_RECORD, BOOLEAN]], RP_STOCK]
			-- Dividends per stock

invariant

	data_not_void: data /= Void

end
