indexing
	description: "Benchmark"
	author: "Ivan Tarnavsky"
	last_editor: "$Author: sm $"
	date: "$Date: 2011/06/23 $"
	revision: "$Revision: #21 $"
	path: "$File: //rose/source/kernel/expert_system/rp_benchmark.e $"

class RP_BENCHMARK

inherit

	HASHABLE

create {RP_REFERENCE_SYSTEM}

	make

feature {NONE} -- Initialization

	make (a_ref: like reference_system; a_gcdb_benchmark: like gcdb_benchmark) is
			-- Initialize `Current'.
		require
			args_exists: a_ref /= Void
		do
			reference_system := a_ref
			gcdb_benchmark := a_gcdb_benchmark
			if gcdb_benchmark = Void then
				code := "Unknown"
				is_unknown := True
			else
				code := gcdb_benchmark.code
				is_hedged := gcdb_benchmark.is_hedged
			end
		end

feature -- Access

	code: STRING
			-- Unique benchmark identifier

	is_unknown: BOOLEAN
			-- Is `Current' instance of "unknown benchmark"?

	is_hedged: BOOLEAN
			-- Is `Current' hedged?

	portfolios: RP_PORTFOLIO_HISTORY is
			-- Interval history of portfolios related to `Current'.
		do
			Result := retrieve_portfolios_history
		ensure
			portfolios_not_void: Result /= Void
		end

	investment_accounts: RP_INVESTMENT_ACCOUNT_HISTORY is
			-- Interval history of investment accounts related to `Current'
		do
			Result := retrieve_investment_accounts_history
		ensure
			investment_accounts_not_void: Result /= Void
		end

	strategy: RP_STRATEGY is
			-- Investment strategy of `Current'
		do
			Result := retrieve_strategy
		ensure
			strategy_not_void: Result /= Void
		end

	name: STRING is
			-- Descriptive name
		do
			if gcdb_benchmark /= Void then
				Result := gcdb_benchmark.name
			end
		ensure
			name_may_be_void: True
		end

	currency: RP_CURRENCY is
			-- Descriptive name
		do
			if gcdb_benchmark /= Void and then gcdb_benchmark.currency /= Void then
				Result := reference_system.currency_from_iso_code (gcdb_benchmark.currency.code)

			end
		ensure
			name_may_be_void: True
		end

	month_to_date_return_source: STRING is
			-- Month-to-date return source
		do
			if gcdb_benchmark /= Void then
				Result := gcdb_benchmark.month_to_date_return_source
			end
		ensure
			month_to_date_return_source_may_be_void: True
		end

	monthly_return_source: STRING is
			-- Month-to-date return source
		do
			if gcdb_benchmark /= Void then
				Result := gcdb_benchmark.monthly_return_source
			end
		ensure
			monthly_return_source_may_be_void: True
		end

	hash_code: INTEGER is
			-- Hash code
		do
			Result := code.hash_code
		end

	global_products: ROSE_LINEAR_ARRAY [RP_GLOBAL_PRODUCT] is
			-- List of global products associated with `Current'
		local
			l: ROSE_LINEAR_ARRAY [GCDB_PRODUCT]
		do
			create Result.make_empty
			from
				l := gcdb_benchmark.products_by_benchmark
				l.start
			until
				l.off
			loop
				if l.current_item.is_global then
					Result.extend (reference_system.global_product_by_code (l.current_item.code))
				end
				l.forth
			end
		end

	zonal_products: ROSE_LINEAR_ARRAY [RP_ZONAL_PRODUCT] is
			-- List of zonal products associated with `Current'
		local
			l: ROSE_LINEAR_ARRAY [GCDB_PRODUCT]
		do
			create Result.make_empty
			from
				l := gcdb_benchmark.products_by_benchmark
				l.start
			until
				l.off
			loop
				if not l.current_item.is_global then
					Result.extend (reference_system.zonal_product_by_code (l.current_item.code))
				end
				l.forth
			end
		end

	products: ROSE_LINEAR_ARRAY [RP_PRODUCT] is
			-- List of all products associated with `Current'
		local
			l: ROSE_LINEAR_ARRAY [GCDB_PRODUCT]
		do
			create Result.make_empty
			from
				l := gcdb_benchmark.products_by_benchmark
				l.start
			until
				l.off
			loop
				if not l.current_item.is_global then
					Result.extend (reference_system.zonal_product_by_code (l.current_item.code))
				else
					Result.extend (reference_system.global_product_by_code (l.current_item.code))
				end
				l.forth
			end
		end

	last_known_name: STRING is
			-- Last known descriptive name
		do
			if gcdb_benchmark /= Void then
				Result := gcdb_benchmark.last_known_name
			else
				Result := ""
			end
		end

	master: RP_BENCHMARK
			-- master benchmark
		do
			if gcdb_benchmark /= Void and then gcdb_benchmark.master /= Void then
				Result := reference_system.benchmark_by_code (gcdb_benchmark.master.code)
			else
				Result := reference_system.unknown_benchmark
			end
		end

	gcdb_handle: INTEGER is
			-- GCDB entity handle
		do
			if gcdb_benchmark /= Void then
				Result := gcdb_benchmark.handle
			end
		end

feature {NONE} -- Implementation

	reference_system: RP_REFERENCE_SYSTEM
			-- Reference system which created `Current'

	gcdb_benchmark: GCDB_BENCHMARK
			-- GCDB Benchmark entity if known

	portfolios_history_cache: like portfolios
			-- Cached value of interval history of related portfolios

	retrieve_portfolios_history: like portfolios is
			-- Retrieve related portfolios
		do
			if portfolios_history_cache = Void then
				create portfolios_history_cache.make_empty
				if gcdb_benchmark /= Void then
					gcdb_benchmark.portfolios_by_benchmark_history.do_all (agent add_portfolio (portfolios_history_cache, ?))
				end
			end
			Result := portfolios_history_cache
		ensure
			non_void_result: Result /= Void
		end

	add_portfolio (a_cache: like portfolios; a_interval: ENTITY_INTERVAL [GCDB_PORTFOLIO]) is
			-- Add portfolio history interval to `portfolios_history_cache'
		local
			l_portfolio: RP_PORTFOLIO
			l_code: STRING
			l_interval: RP_PORTFOLIO_HISTORY_INTERVAL
		do
			l_code := a_interval.entity.code
			if l_code = Void then
				a_interval.entity.set_as_of_date (a_interval.start_date)
				l_code := a_interval.entity.code
			end
			l_portfolio := reference_system.portfolio_from_code (l_code)
			create l_interval.make (l_portfolio, a_interval.start_date, a_interval.end_date, a_interval.weight)
			a_cache.extend (l_interval)
		end

	investment_accounts_history_cache: like investment_accounts
			-- Cached valid of interval history of related investment accounts

	retrieve_investment_accounts_history: like investment_accounts is
			-- Retrieve related investment accounts
		local
			l_hash: HASHED_SET [STRING]
		do
			if investment_accounts_history_cache = Void then
				create investment_accounts_history_cache.make_empty
				if gcdb_benchmark /= Void then
					create l_hash.make
					gcdb_benchmark.portfolio_groups_by_benchmark_history.do_all (agent add_portfolio_group (investment_accounts_history_cache, l_hash, ?))
					gcdb_benchmark.accounts_by_benchmark_history.do_all (agent add_account (investment_accounts_history_cache, l_hash, ?))
				end
			end
			Result := investment_accounts_history_cache
		end

	add_portfolio_group (a_cache: like investment_accounts; a_hash: HASHED_SET [STRING]; a_interval: ENTITY_INTERVAL [GCDB_PORTFOLIO_GROUP]) is
			-- Add investment account history interval from portfolio group to `investment_accounts_history_cache'
		local
			l_acct: RP_INVESTMENT_ACCOUNT
			l_code: STRING
			l_interval: RP_INVESTMENT_ACCOUNT_HISTORY_INTERVAL
		do
			l_code := a_interval.entity.code
			if l_code = Void then
				a_interval.entity.set_as_of_date (a_interval.start_date)
				l_code := a_interval.entity.code
			end
			l_acct := reference_system.investment_account_by_code (l_code)
			create l_interval.make (l_acct, a_interval.start_date, a_interval.end_date, a_interval.weight)
			a_cache.extend (l_interval)
			a_hash.force (l_code)
		end

	add_account (a_cache: like investment_accounts; a_hash: HASHED_SET [STRING]; a_interval: ENTITY_INTERVAL [GCDB_ACCOUNT]) is
			-- Add investment account history interval from account to `investment_accounts_history_cache'
		local
			l_acct: RP_INVESTMENT_ACCOUNT
			l_code: STRING
			l_interval: RP_INVESTMENT_ACCOUNT_HISTORY_INTERVAL
		do
			l_code := a_interval.entity.code
			if l_code = Void then
				a_interval.entity.set_as_of_date (a_interval.start_date)
				l_code := a_interval.entity.code
			end
			if not a_hash.has (l_code) then
				l_acct := reference_system.investment_account_by_code (l_code)
				create l_interval.make (l_acct, a_interval.start_date, a_interval.end_date, a_interval.weight)
				a_cache.extend (l_interval)
			end
		end

	strategy_saved: like strategy
			-- Cached value for investment strategy

	retrieve_strategy: like strategy is
			-- Retrieve investment strategy
		do
			if strategy_saved = Void then
				if gcdb_benchmark /= Void and then gcdb_benchmark.type /= Void then
					strategy_saved := reference_system.strategy_by_code (gcdb_benchmark.type.code)
				end
			end
			if strategy_saved = Void then
				strategy_saved := reference_system.Unknown_strategy
			end
			Result := strategy_saved
		ensure
			non_void_result: Result /= Void
		end

invariant

	valid_code: code /= Void and then not code.is_empty

end -- class RP_BENCHMARK
