note

	description:

		"Market Data Source for Composite Assets Database"

	author: "nw"
	last_editor: "$Author: it $"
	date: "$Date: 2011/03/30 $"
	revision: "$Revision: #3 $"
	path: "$File: //rose/source/kernel/expert_system/mds/rp_cadb_pool_fund_market_data_source.e $"

class RP_CADB_POOL_FUND_MARKET_DATA_SOURCE

inherit

	RP_POOL_FUND_DATA_SOURCE

create

	make,
	make_with_lookback,
	make_with_restriction

feature {NONE} -- Implementation

	cache: DS_HASH_TABLE [DS_HASH_TABLE [RP_MARKET_DATA_RECORD, INTEGER], STRING]
			-- Cached market data by brimsec then by date

	pool_fund_data (a_fund: RP_POOL_FUND; a_date: ROSE_DATE): RP_MARKET_DATA_RECORD is
			-- <Precursor>
		local
			l_brimsec, l_brimco, l_sec: STRING
			l_data: DS_HASH_TABLE [RP_MARKET_DATA_RECORD, INTEGER]
		do
			l_brimsec := a_fund.brimsym.substring (1, 15)
			l_brimco := l_brimsec.substring (1, 9)
			l_sec := l_brimsec.substring (10, 15)
			if cache = Void then
				create cache.make_equal (0)
			end
			cache.search (l_brimsec)
			if cache.found then
				l_data := cache.found_item
			else
				create l_data.make_default
				cache.force (l_data, l_brimsec)
				prepared_query.retrieve (Void, <<l_brimco, l_sec, start_date.ccyymmdd, end_date.ccyymmdd>>)
				prepared_query.array.do_all (agent add_record (l_data, ?))
			end
			l_data.search (a_date.ccyymmdd)
			if l_data.found then
				Result := l_data.found_item
			end
		end

	add_record (a_data: DS_HASH_TABLE [RP_MARKET_DATA_RECORD, INTEGER]; a_record: RP_POOL_FUND_CLASS_DATA_CADB_RECORD) is
			-- Add data from `a_record' to `a_data'
		require
			a_data_not_void: a_data /= Void
			a_record_not_void: a_record /= Void
		local
			l_rec: RP_POOL_FUND_CLASS_DATA
		do
			create l_rec.make_from_cadb_record (reference_system, a_record)
			a_data.force (l_rec, l_rec.date.ccyymmdd)
		ensure
			record_added: a_data.has (a_record.trade_date)
		end

	common_server: DATA_SERVER is
			-- Common db
		once
			Result := (create {DATABASE_ACCESS}).make_from_repository_fast_load ("common")
		ensure
			result_not_void: Result /= Void
		end

	prepared_query: COLLECTING_PREDICATE [RP_POOL_FUND_CLASS_DATA_CADB_RECORD] is
			-- Query for composite asset records
		once
			create Result.make (common_server, query_template)
		ensure
			result_not_void: Result /= Void
		end

	query_template: STRING is
			-- Data query template
		once
			Result := "[
SELECT 
	[TRADE_DATE], 
	cast(isnull([CAPITAL_BALANCE],0) as decimal(30,5)) as CAPITAL_BALANCE,
	cast(isnull([REVENUE_BALANCE],0) as decimal(30,5)) as REVENUE_BALANCE,
	cast(isnull([EXPENSE_BALANCE],0) as decimal(30,5)) as EXPENSE_BALANCE,
	cast(isnull([OUTSTANDING_SHARES],0) as decimal(30,5)) as OUTSTANDING_SHARES,
	cast(isnull([SHARE_ACTIVITY],0) as decimal(30,5)) as SHARE_ACTIVITY,
	[CURRENCY], 
	cast(isnull([PUBLISHED_NAV],0) as decimal(30,5)) as PUBLISHED_NAV
FROM [COMPOSITE_ASSETS_NAV]
WHERE [BRIMCO] = ? AND [SEC] = ? AND [TRADE_DATE] BETWEEN ? AND ? 
 ORDER BY [TRADE_DATE]
]"
		ensure
			result_not_void: Result /= Void
		end

end
