indexing

	description:

		"BACKDIV data source"

	date: "$Date: 2010/01/20 $"
	revision: "$Revision: #24 $"
	archive: "$File: //rose/source/kernel/market_data/rp_cae_backdiv_data_source.e $"

class RP_CAE_BACKDIV_DATA_SOURCE

inherit
	RP_INVESTMENT_MANAGER_CLIENT
	RP_CAE_DATA_SOURCE
	DATABASE_SERVER

create {RP_CORPORATE_ACTION_SYSTEM}

	make

feature -- Query

	instrument_action_and_events (a_instrument: RP_INSTRUMENT; a_date: ROSE_DATE): ARRAY [RP_CORPORATE_ACTION] is
			-- Corporate actions and events for `a_instrument' on `a_date'
		local
			l_stock: RP_STOCK
		do
			l_stock ?= a_instrument
			if l_stock /= Void then
				Result := div_records (l_stock, a_date)
			end
			if Result = Void then
				Result := << >>
			end
		end

feature {NONE} -- Implementation

	backdiv_cache: UID_TABLE [DS_HASH_TABLE [ROSE_LINEAR_ARRAY [RP_CORPORATE_ACTION], ROSE_DATE], RP_STOCK]
			-- Cached backdiv data.

	connection: SQL_DATA_SERVER
			-- CAE_DIVIDENDS connection.

	connect is
			-- Connect to CAE_DIVIDENDS database.
		do
			connection := servers.performance_server_fast_load
		end

	div_records_predicate: COLLECTING_PREDICATE [BACKDIV] is
			-- Predicate to retrieve div_records
		once
			create Result.make (connection, "SELECT * FROM cae_dividends WHERE tzone = ? and rxrf = ?")
		end

	div_records (a_stock: RP_STOCK; a_date: ROSE_DATE): ARRAY [RP_CORPORATE_ACTION] is
			-- Retrieve CAE_DIVIDEND records
		local
			l_backdiv_history: DS_HASH_TABLE [ROSE_LINEAR_ARRAY [RP_CORPORATE_ACTION], ROSE_DATE]
		do
			if backdiv_cache = Void then
				create backdiv_cache.make_default
			end
			if backdiv_cache.has (a_stock) then
				l_backdiv_history := backdiv_cache.item (a_stock)
			else
					-- retrieve backdiv data for `a_stock'
				create l_backdiv_history.make_default
				backdiv_cache.force (l_backdiv_history, a_stock)
				if connection = Void then
					connect
				end
				div_records_predicate.retrieve (Void, <<a_stock.zone_mnemonic.out, a_stock.xrf_code>>)
				connection.commit_if_necessary
				div_records_predicate.array.do_all (agent add_backdiv_record (l_backdiv_history, ?))
			end
			if l_backdiv_history.has (a_date) then
				Result := l_backdiv_history.item (a_date)
			end
		end

	add_backdiv_record (a_backdiv_history: DS_HASH_TABLE [ROSE_LINEAR_ARRAY [RP_CORPORATE_ACTION], ROSE_DATE]; a_backdiv: BACKDIV) is
			-- Add backdiv record to history
		local
			l_backdiv_rec: !RP_BACKDIV_RECORD
			l_backdiv: !RP_SIMPLE_CORPORATE_ACTION
			l_array: ROSE_LINEAR_ARRAY [RP_CORPORATE_ACTION]
		do
			create l_backdiv_rec.make (a_backdiv, manager.reference_system)
			create l_backdiv.make (l_backdiv_rec.ex_date, l_backdiv_rec)
			if a_backdiv_history.has (l_backdiv_rec.ex_date) then
				l_array := a_backdiv_history.item (l_backdiv_rec.ex_date)
			else
				create l_array.make_empty
				a_backdiv_history.force (l_array, l_backdiv_rec.ex_date)
			end
			l_array.extend (l_backdiv)
		end

end
