note

	description:

		"Cash dividend contract"

	author: "Julian Rogers"
	last_editor: "$Author: e2 $"
	date: "$Date: 2011/11/29 $"
	revision: "$Revision: #4 $"
	path: "$File: //rose/source/kernel/expert_system/rp_cash_dividend.e $"

class RP_CASH_DIVIDEND

inherit

	RP_DIVIDEND

create

	make

feature {NONE} -- Initialization

	make (a_units_base: like units_base; a_gross, a_tax: RP_MONIES; a_event_date, a_settle_date: ROSE_DATE;
		a_portfolio: like portfolio; a_identifier: like identifier; a_system: like generating_system;
		a_paydate: like paydate; a_custody: RP_CUSTODY) is
			-- Create a dividend for a_portfolio on a_date
		require
			a_units_base_not_void: a_units_base /= Void
			a_gross_not_void: a_gross /= Void
			a_event_date_not_void: a_event_date /= Void
			a_settle_date_not_void: a_settle_date /= Void
			a_portfolio_not_void: a_portfolio /= Void
			a_identifier_not_void: a_identifier /= Void
			a_system_not_void: a_system /= Void
			same_currency_for_gross_and_tax: a_tax /= Void implies a_tax.currency ~ a_gross.currency
			a_paydate_not_void: a_paydate /= Void
			a_custody_not_void: a_custody /= Void
		local
			l_tax_man: RP_INSTRUMENT_CONTROL
		do
			paydate := a_paydate
				-- Contract
			make_capital_action (a_system, a_event_date, a_identifier, a_portfolio, a_units_base, a_settle_date)
				-- Flows
			add_flow (dividend_flow_type, unknown_instrument_control, portfolio.instrument_control (a_custody), a_event_date, a_settle_date, a_gross, a_event_date, 0, a_gross)
			if a_tax /= Void and then not a_tax.amount.is_zero then
				l_tax_man := units_base.instrument.country_of_issue (a_system.manager.reference_system, a_event_date).government_tax_authority
					--	create dividend_tax flow
				add_flow (dividend_withholding_country_tax_flow_type, portfolio.instrument_control (a_custody), l_tax_man, a_event_date, a_settle_date, a_tax, a_event_date, 0, a_tax)
			end
			custody := a_custody
		ensure
			paydate_set: paydate = a_paydate
			flows_without_tax_is_one: a_tax = Void or else a_tax.amount.is_zero implies flows.count = 1
			flows_with_tax_is_two: a_tax /= Void and then not a_tax.amount.is_zero implies flows.count = 2
			custody_set: custody = a_custody
		end

feature -- Access

	paydate: ROSE_DATE
			-- Pay date

feature {RP_CUSTODIAN_SYSTEM} -- Access

	custody: RP_CUSTODY
			-- Who has custody of the interest?

invariant
	custody_set: custody /= Void
	paydate_not_void: paydate /= Void

end
