note
	description: "Account or Portfolio cash position (traded or settled)"
	author: "Sami Ben Grine & Ivan Tarnavsky"
	last_editor: "$Author: sm $"
	date: "$Date: 2014/12/10 $"
	revision: "$Revision: #3 $"
	path: "$File: //rose/source/kernel/expert_system/rp_cash_position.e $"

deferred class RP_CASH_POSITION[G->RP_ZONAL_INVESTMENT_GROUP]

inherit
	POSITION [G]
		rename instrument as currency redefine currency end

feature -- Access

	accounting_date: ROSE_DATE
		-- Accounting date (beginning of day)


	currency: RP_CURRENCY
		-- Currency

	cash_account_category: STRING
		-- Cash Accounting category

	total_quantity: RP_MONIES
		-- Total

	unavailable: RP_MONIES
		-- Unavailable

	market_value: RP_MONIES
		-- Market valuation

	market_value_currency: RP_CURRENCY
		-- Market value currency

feature -- Query

	iso_currency_code: STRING
			-- Iso code of 'currency'
		do
			Result := currency.iso
		ensure
			currency_iso: Result ~ currency.iso
		end

	quantity:RP_QUANTITY
			-- <Precursor>
		do
			if total_quantity /= Void then
				Result := total_quantity
			else
				Result := zero_usd
			end
		end

	valuation: RP_MONIES
			-- <Precursor>
		do
			if market_value /= Void then
				Result := market_value
			else
				Result := zero_usd
			end
		end

invariant

	accounting_date_exists: accounting_date /= Void
	currency_exists: currency /= Void
	cash_account_category_exists: cash_account_category /= Void

	valid_total_quantity: (total_quantity /= Void) and then (total_quantity.currency ~ currency)
	valid_unavailable_quantity: (unavailable /= Void) and then (unavailable.currency ~ currency)
	valid_amounts: (unavailable.sign=0) or else (total_quantity.sign=unavailable.sign and unavailable.amount.abs <= total_quantity.amount.abs)

	valid_market_value: (market_value_currency = Void and market_value = Void) or else (market_value_currency /= Void and then market_value.currency ~ market_value_currency)
	valid_market_value_side: (market_value /= Void) implies (market_value.sign * total_quantity.sign >= 0)

end
