note

	description:

		"Cash zonal transfer(type 24)"

	author: "gy"
	last_editor: "$Author: e2 $"
	date: "$Date: 2011/11/29 $"
	revision: "$Revision: #8 $"
	path: "$File: //rose/source/kernel/expert_system/legacy/contracts/rp_cash_zonal_transfer.e $"

class RP_CASH_ZONAL_TRANSFER

inherit

	RP_PORTFOLIO_CONTRACT
		rename
			make as make_portfolio_contract
		end

create

	make_intrazonal_withdrawal,
	make_intrazonal_contribution,
	make_intrazonal_withdrawal_with_fx,
	make_intrazonal_contribution_with_fx

feature {NONE} -- Initialization

	make_intrazonal_withdrawal (a_system: like generating_system; a_agreement_time: like agreement_time;
		a_identifier: like identifier; a_portfolio: like portfolio; a_settle_date: like agreement_time;
		a_monies: RP_MONIES; a_custody: RP_CUSTODY) is
			-- Create intrazonal withdrawal
		require
			a_system_not_void: a_system /= Void
			a_agreement_time_not_void: a_agreement_time /= Void
			a_identifier_not_void: a_identifier /= Void
			a_portfolio_not_void: a_portfolio /= Void
			a_settle_date_not_void: a_settle_date /= Void
			a_monies_not_void: a_monies /= Void
			a_custody_not_void: a_custody /= Void
		do
			if a_monies.amount.sign < 0 then
				a_system.manager.log_minor ("Wrong sign in intrazonal withdrawal " + a_identifier)
			end
			make_portfolio_contract (a_system, a_agreement_time, a_identifier, a_portfolio)
			add_flow (intrazonal_withdrawal_flow_type, a_portfolio.instrument_control (a_custody), unknown_instrument_control, a_agreement_time, a_settle_date, a_monies, a_agreement_time, 0, a_monies)
		end

	make_intrazonal_contribution (a_system: like generating_system; a_agreement_time: like agreement_time;
		a_identifier: like identifier; a_portfolio: like portfolio; a_settle_date: like agreement_time;
		a_monies: RP_MONIES; a_custody: RP_CUSTODY) is
			-- Create intrazonal contribution
		require
			a_system_not_void: a_system /= Void
			a_agreement_time_not_void: a_agreement_time /= Void
			a_identifier_not_void: a_identifier /= Void
			a_portfolio_not_void: a_portfolio /= Void
			a_settle_date_not_void: a_settle_date /= Void
			a_monies_not_void: a_monies /= Void
			a_custody_not_void: a_custody /= Void
		do
			if a_monies.amount.sign < 0 then
				a_system.manager.log_minor ("Wrong sign in intrazonal contribution " + a_identifier)
			end
			make_portfolio_contract (a_system, a_agreement_time, a_identifier, a_portfolio)
			add_flow (intrazonal_contribution_flow_type, unknown_instrument_control, a_portfolio.instrument_control (a_custody), a_agreement_time, a_settle_date, a_monies, a_agreement_time, 0, a_monies)
		end

	make_intrazonal_withdrawal_with_fx (a_system: like generating_system; a_agreement_time: like agreement_time;
		a_identifier: like identifier; a_portfolio: like portfolio; a_settle_date: like agreement_time;
		a_converted_from_amount, a_transferred_to_amount: RP_MONIES; a_custody: RP_CUSTODY) is
			-- Create intrazonal withdrawal with fx
			-- XXX FIXME event type 24s should really generate two contracts FX + xfer
		require
			a_system_not_void: a_system /= Void
			a_agreement_time_not_void: a_agreement_time /= Void
			a_identifier_not_void: a_identifier /= Void
			a_portfolio_not_void: a_portfolio /= Void
			a_settle_date_not_void: a_settle_date /= Void
			a_converted_from_amount_not_void: a_converted_from_amount /= Void
			a_transferred_to_amount_not_void: a_transferred_to_amount /= Void
			a_custody_not_void: a_custody /= Void
		do
			if a_converted_from_amount.amount.sign >= 0 then
				a_system.manager.log_minor ("Wrong converted amount sign in intrazonal withdrawal with fx " + a_identifier)
			end
			if a_transferred_to_amount.amount.sign < 0 then
				a_system.manager.log_minor ("Wrong transferred amount sign in intrazonal withdrawal with fx " + a_identifier)
			end
			make_portfolio_contract (a_system, a_agreement_time, a_identifier, a_portfolio)
				-- FX give out `a_converted_from_amount' i.e. receive in portfolio with negative sign
			add_flow (quantity_flow_type, unknown_instrument_control, a_portfolio.instrument_control (a_custody), a_agreement_time, a_settle_date, a_converted_from_amount, a_agreement_time, 0, a_converted_from_amount)
				-- FX receive in `a_transferred_to_amount'
			add_flow (quantity_flow_type, unknown_instrument_control, a_portfolio.instrument_control (a_custody), a_agreement_time, a_settle_date, a_transferred_to_amount, a_agreement_time, 0, a_transferred_to_amount)
				-- Withdraw `a_transferred_to_amount' out of the portfolio
			add_flow (intrazonal_withdrawal_flow_type, a_portfolio.instrument_control (a_custody), unknown_instrument_control, a_agreement_time, a_settle_date, a_transferred_to_amount, a_agreement_time, 0, a_transferred_to_amount)
		end

	make_intrazonal_contribution_with_fx (a_system: like generating_system; a_agreement_time: like agreement_time;
		a_identifier: like identifier; a_portfolio: like portfolio; a_settle_date: like agreement_time;
		a_converted_to_amount, a_transferred_from_amount: RP_MONIES; a_custody: RP_CUSTODY) is
			-- Create intrazonal contribution with fx
			-- XXX FIXME event type 24s should really generate two contracts FX + xfer
		require
			a_system_not_void: a_system /= Void
			a_agreement_time_not_void: a_agreement_time /= Void
			a_identifier_not_void: a_identifier /= Void
			a_portfolio_not_void: a_portfolio /= Void
			a_settle_date_not_void: a_settle_date /= Void
			a_converted_to_amount_not_void: a_converted_to_amount /= Void
			a_transferred_from_amount_not_void: a_transferred_from_amount /= Void
			a_custody_not_void: a_custody /= Void
		do
			if a_converted_to_amount.amount.sign >= 0 then
				a_system.manager.log_minor ("Wrong converted amount sign in intrazonal contribution with fx " + a_identifier)
			end
			if a_transferred_from_amount.amount.sign < 0 then
				a_system.manager.log_minor ("Wrong transferred amount sign in intrazonal contribution with fx " + a_identifier)
			end
			make_portfolio_contract (a_system, a_agreement_time, a_identifier, a_portfolio)
				-- Contribute `a_transferred_from_amount' to the portfolio
			add_flow (intrazonal_contribution_flow_type, unknown_instrument_control, a_portfolio.instrument_control (a_custody), a_agreement_time, a_settle_date, a_transferred_from_amount, a_agreement_time, 0, a_transferred_from_amount)
				-- FX give out `a_transferred_from_amount'
			add_flow (quantity_flow_type, a_portfolio.instrument_control (a_custody), unknown_instrument_control, a_agreement_time, a_settle_date, a_transferred_from_amount, a_agreement_time, 0, a_transferred_from_amount)
				-- FX receive `a_converted_to_amount'
			add_flow (quantity_flow_type, a_portfolio.instrument_control (a_custody), unknown_instrument_control, a_agreement_time, a_settle_date, a_converted_to_amount, a_agreement_time, 0, a_converted_to_amount)
		end

feature -- Basic Operations

	process (a_processor: RP_CONTRACT_PROCESSOR) is
			-- <Precursor>
		do
			a_processor.process_cash_zonal_tranfer (Current)
		end

end
