indexing
	description: "Rose Production view of a client of AXA Rosenberg"
	author: "Ivan Tarnavsky"
	last_editor: "$Author: sm $"
	date: "$Date: 2011/06/23 $"
	revision: "$Revision: #16 $"
	path: "$File: //rose/source/kernel/expert_system/rp_client.e $"

class RP_CLIENT

inherit

	RP_ROLE
		rename
			make as rp_role_make
		undefine
			is_equal
		end

	RP_PARTY

create {RP_REFERENCE_SYSTEM}

	make

feature {NONE} -- Constructor

	make (a_ref: like reference_system; a_gcdb_client: GCDB_CLIENT) is
			-- Initialize `Current'.
		require
			a_ref_not_void: a_ref /= Void
		do
			reference_system := a_ref
			gcdb_client := a_gcdb_client
			if gcdb_client = Void then
				code := "Unknown"
				name := code
			else
				code := gcdb_client.code
				name := gcdb_client.name
			end
		ensure
			reference_system_set: reference_system = a_ref
			gcdb_client_set: gcdb_client = a_gcdb_client
		end

feature -- Access

	code: STRING
			-- Unique identifier

	name: STRING
			-- Long identifier

	is_ucits: BOOLEAN is
			-- Is this a UCITS Client?
		do
			Result := equal (code, "UCITS")
		end

	instrument_control: RP_INSTRUMENT_CONTROL is
			-- The cash account for this client
		do
			if saved_instrument_control = Void then
				create saved_instrument_control.make_with_owner (Current)
			end
			Result := saved_instrument_control
		ensure
			non_void_values: Result /= Void
		end

	investment_accounts: ROSE_LINEAR_ARRAY [RP_INVESTMENT_ACCOUNT] is
			-- All accounts for `Current'
		do
			if investment_accounts_saved = Void then
				create investment_accounts_saved.make_empty
				if gcdb_client /= Void then
					gcdb_client.accounts_by_client.do_all (agent expand_account (investment_accounts_saved, ?))
				end
			end
			Result := investment_accounts_saved
		ensure
			investment_accounts_not_void: Result /= Void
			result_cached: investment_accounts_saved /= Void
		end

	gcdb_handle: INTEGER
			-- Gcdb handle
			-- This will return 0 if the client is unknown
		do
			if gcdb_client /= Void then
				Result := gcdb_client.handle
			end
		end

feature {NONE} -- Implementation

	reference_system: RP_REFERENCE_SYSTEM
			-- Reference system

	gcdb_client: GCDB_CLIENT
			-- Access to the GCDB client

	saved_instrument_control: like instrument_control
			-- Cached investment control

	investment_accounts_saved: like investment_accounts
			-- Cached investment accounts

	expand_account (a_accounts: like investment_accounts; a_account: GCDB_ACCOUNT) is
			-- Lookup investment account from `a_account' and add to `a_accounts'
		local
			l_account: RP_INVESTMENT_ACCOUNT
		do
			l_account := reference_system.investment_account_by_code (a_account.code)
			if l_account /= Void then
				a_accounts.extend (l_account)
			end
		end

invariant

	reference_system_not_void: reference_system /= Void

end -- class RP_CLIENT
