indexing
	description: "Combines two calendars with a cutover date"
	date: "$Date: 2009/02/06 $"
	fileid: "$Id: //rose/source/Kernel/expert_system/ref/rp_combined_calendar_with_cutover.e#1 $"

class RP_COMBINED_CALENDAR_WITH_CUTOVER

inherit

	RP_CALENDAR

create

	make

feature {NONE} -- Initialization

	make (a_calendar_before: like calendar_before; a_calendar_on_and_after: like calendar_on_and_after; a_cutover_date: like cutover_date) is
			-- Initialize `Current'
		require
			non_void_args: a_calendar_before /= Void and a_calendar_on_and_after /= Void and a_cutover_date /= Void
		do
			calendar_before := a_calendar_before
			calendar_on_and_after := a_calendar_on_and_after
			cutover_date := a_cutover_date
		ensure
			calendar_before_set: calendar_before = a_calendar_before
			calendar_on_and_after_set: calendar_on_and_after = a_calendar_on_and_after
			cutover_date_set: cutover_date = a_cutover_date
		end

feature {NONE} -- Implementation

	calendar_before: RP_CALENDAR
			-- Calendar used before `cutover_date'

	calendar_on_and_after: RP_CALENDAR
			-- Calendar used on and after `cutover_date'

	cutover_date: ROSE_DATE
			-- Cutover date beween calendars

	is_work_day (a_date: ROSE_DATE): BOOLEAN is
			-- Is `a_date' work dat?
		do
			if a_date < cutover_date then
				Result := calendar_before.is_work_day (a_date)
			else
				Result := calendar_on_and_after.is_work_day (a_date)
			end
		end

	ith_work_day_after (a_date: ROSE_DATE; a_days: INTEGER): ROSE_DATE is
			-- Work date which is `a_days' after `a_date'
		do
			if a_date < cutover_date then
				Result := calendar_before.ith_work_day_after (a_date, a_days)
			else
				Result := calendar_on_and_after.ith_work_day_after (a_date, a_days)
			end
		end

	ith_work_day_before (a_date: ROSE_DATE; a_days: INTEGER): ROSE_DATE is
			-- Work date which is `a_days' before `a_date'
		do
			if a_date < cutover_date then
				Result := calendar_before.ith_work_day_before (a_date, a_days)
			else
				Result := calendar_on_and_after.ith_work_day_before (a_date, a_days)
			end
		end

	last_work_day (a_date: ROSE_DATE): ROSE_DATE is
			-- Work date on or before `a_date'
		do
			if a_date < cutover_date then
				Result := calendar_before.last_work_day (a_date)
			else
				Result := calendar_on_and_after.last_work_day (a_date)
			end
		end

	next_work_day (a_date: ROSE_DATE): ROSE_DATE is
			-- Work day on or after `a_date'
		do
			if a_date < cutover_date then
				Result := calendar_before.next_work_day (a_date)
			else
				Result := calendar_on_and_after.next_work_day (a_date)
			end
		end

end
