indexing

	description:

		"An agreement between two parties"

	date: "$Date: 2010/07/19 $"
	sccid: "$Id: //rose/source/kernel/contracts/rp_contract.e#17 $"

deferred class RP_CONTRACT

inherit

	ANY

	RP_ACCOUNTABLE

	RP_FLOW_TYPE_AWARE
		export {NONE} all end

	RP_UNKNOWN_INSTRUMENT_CONTROL_AWARE
		export {NONE} all end

feature {NONE} -- Initialization

	make (a_system: like generating_system; a_identifier: like identifier; a_agreement_time: like agreement_time) is
			-- creation routine	
		do
			generating_system := a_system
			identifier := a_identifier
			agreement_time := a_agreement_time
			create flows.make_empty
				-- Create signature
				-- create sha1.make
				-- sha1.process_block (debug_output)
		end

feature -- Access

	generating_system: RP_CONTRACT_SYSTEM
			-- Generating system

	identifier: STRING
			-- Contract identifier

	agreement_time: ROSE_DATE
			-- Contract agreement time

	flows: ROSE_LINEAR_ARRAY [RP_FLOW]
			-- Flows

	risk_carrying_counterparty: RP_PARTY
			-- Risk carying counterparty

	risk_carrying_imc: RP_PARTY
			-- Risk carying IMC

	executing_counterparty: RP_PARTY
			-- Executing party

	executing_imc: RP_PARTY
			-- Executing IMC

	initiation_time: ROSE_DATE
			-- Initiation date time

feature -- Status Report

	is_foreign_exchange_trade: BOOLEAN is
			-- Is this a foreign exchange trade contract?
		once
			Result := False
		end

	is_foreign_exchange_hedge: BOOLEAN is
			-- Is this a foreign exchange hedge contract?
		once
			Result := False
		end

feature -- Conversion

	as_foreign_exchange_contract: RP_FX is
			-- This object as a foreign exchange contract
		do
			Result := Void
		ensure
			is_fx_ff_implies_valid_result: (is_foreign_exchange_trade or is_foreign_exchange_hedge) implies Result /= Void
		end

feature -- Basic Operations

	process (a_processor: RP_CONTRACT_PROCESSOR) is
			-- Process 'Current' using 'a_processor'
		require
			processor_not_void: a_processor /= Void
		deferred
		end

invariant

	valid_generating_system: generating_system /= Void
	valid_identifier: identifier /= Void
	valid_agreement_time: agreement_time /= Void
	valid_flows: flows /= Void

end
