note

	description:

		"Corporate Action processor which create contracts and flows"

	author: "Ivan Tarnavsky"
	last_editor: "$Author: c4 $"
	date: "$Date: 2015/04/09 $"
	revision: "$Revision: #12 $"
	path: "$File: //rose/source/kernel/expert_system/rp_corporate_action_election.e $"

deferred class RP_CORPORATE_ACTION_ELECTION [G -> CAPACT_EXPRESSION, H -> RP_PORTFOLIO_CORPORATE_ACTION_CONTRACT]

inherit

	CAPACT_UTILS
		export {NONE} all end

	RP_UNKNOWN_CUSTODY_AWARE
		export {NONE} all end

	ANY

feature {NONE} -- Initialization

	make (a_processing_system: like processing_system; a_capact: G) is
			-- Create with 'a_capact'
		require
			a_processing_system_not_void: a_processing_system /= Void
			a_capact_not_void: a_capact /= Void
		do
			processing_system := a_processing_system
			capact := a_capact
		ensure
			processing_system_set: processing_system = a_processing_system
			capact_set: capact = a_capact
		end

feature -- Access

	processing_system: RP_CORPORATE_ACTION_PROCESSING_SYSTEM
			-- Processing will be done on behalf of this system

	capact: G
			-- Capital action

	contract (a_idate: ROSE_DATE; a_portfolio: RP_PORTFOLIO; a_holding: RP_QUANTITY): H is
			-- Generate contract
		require
			non_void_args: a_idate /= Void and a_portfolio /= Void and a_holding /= Void
		deferred
		end

	manager: RP_INVESTMENT_MANAGER is
			-- Manager
		do
			Result := processing_system.manager
		ensure
			result_not_void: Result /= Void
		end

	unique_identifier: STRING is
			-- Unique identifier for this transaction
		do
			create Result.make_empty
				-- 	FIXME: sha1.process_block (contract.debug_output)
		end

feature -- Query

	event_date (a_portfolio: RP_PORTFOLIO): ROSE_DATE is
			-- Event date (i.e. ex-date)
		require
			non_void_argument: a_portfolio /= Void
		do
			Result := a_portfolio.zone.legacy_perf_calendar.next_work_day (capact.exdate)
		end

	settle_date (a_portfolio: RP_PORTFOLIO): ROSE_DATE is
			-- Settle date (i.e. pay-date)
		require
			non_void_argument: a_portfolio /= Void
		do
			Result := settle_date_from_pay_date (a_portfolio, True)
		ensure
			result_not_void: Result /= Void
		end

	settle_date_without_work_day_enforcement (a_portfolio: RP_PORTFOLIO): ROSE_DATE is
			-- Settle date (i.e. pay-date) without enforcing the result to be work day
		require
			non_void_argument: a_portfolio /= Void
		do
			Result := settle_date_from_pay_date (a_portfolio, False)
		ensure
			result_not_void: Result /= Void
		end

feature {NONE} -- Implementation

	reference_system: RP_REFERENCE_SYSTEM is
			-- Reference System
		do
			Result := manager.reference_system
		end

	settle_date_from_pay_date (a_portfolio: RP_PORTFOLIO; a_enforce_work_day: BOOLEAN): ROSE_DATE is
			-- Settle date (i.e. pay-date)
		require
			a_portfolio_non_void: a_portfolio /= Void
		local
			l_pay_date: ROSE_DATE
		do
			if capact.payment_date \\ 100 = 0 or capact.payment_date \\ 100 = 99 then
				if a_portfolio.zone.is_orient or a_portfolio.zone.is_japan then
						-- 90 work days ahead of event date in O and J zones
					Result := a_portfolio.zone.legacy_perf_calendar.ith_work_day_after (event_date (a_portfolio), 90)
				elseif capact.payment_date \\ 100 = 0 then
						-- Month only available
					create l_pay_date.make_ccyymm_end (capact.payment_date // 100)
					Result := a_portfolio.zone.legacy_perf_calendar.last_work_day (l_pay_date)
				else
						-- Set pay date to 9999/12/31
					create Result.make_max_date
				end
			else
					-- Full date
				create l_pay_date.make_ccyymmdd (capact.payment_date)
				if a_enforce_work_day and then l_pay_date < l_pay_date.max_date then
					Result := a_portfolio.zone.legacy_perf_calendar.next_work_day (l_pay_date)
				else
					Result := l_pay_date
				end
			end
		ensure
			settle_date_from_pay_date_not_void: Result /= Void
		end

end
