note

	description:

		"Cash dividend capact to contract generator"

	author: "Xavier Ragot"
	last_editor: "$Author: e2 $"
	date: "$Date: 2011/11/29 $"
	revision: "$Revision: #29 $"
	path: "$File: //rose/source/Kernel/capital_actions/elections/rp_corporate_action_election_dividend.e $"

class RP_CORPORATE_ACTION_ELECTION_DIVIDEND

inherit

	RP_CORPORATE_ACTION_ELECTION [CAPACT_CASH_DIVIDEND_EXPRESSION, RP_CASH_DIVIDEND]

	RP_WITHHOLDING_TAX_GROSS_UP
		export {NONE} all end

create

	make

feature -- Access

	contract (a_idate: ROSE_DATE; a_portfolio: RP_PORTFOLIO; a_holding: RP_QUANTITY): RP_CASH_DIVIDEND is
			-- <Precursor>
		local
			l_value: RP_MONIES
			l_net, l_gross, l_tax_withholding: RP_MONIES
			l_shares_held: RP_DECIMAL
			l_cash_rate: RP_DECIMAL
			l_cash_rate_currency: RP_CURRENCY
			l_taxbas: STRING
			l_tax_date: ROSE_DATE
			l_units_base: like a_holding
		do
			if not a_holding.amount.is_zero then
				l_shares_held := a_holding.amount
				l_cash_rate := capact.cash_rate
				if capact.cash_currency = Void then
					manager.log_minor ("ERROR: NO DIVIDEND CURRENCY " + a_idate.ccyymmdd.out + " " + a_holding.instrument.brimsym)
					l_cash_rate_currency := a_portfolio.zone.legacy_base_currency
				else
					l_cash_rate_currency := manager.reference_system.currency_from_iso_code (capact.cash_currency.iso)
				end
				l_value := l_cash_rate_currency.monies (l_shares_held * l_cash_rate)
				l_taxbas := tax_basis
				if a_portfolio.zone.is_united_states then
					l_tax_date := event_date (a_portfolio)
				else
					l_tax_date := settle_date (a_portfolio)
				end
				l_net := net_amount (manager.corporate_action_processing_system.use_rate_overrides_for_cash_dividends, manager.reference_system, a_portfolio, a_holding.instrument, l_tax_date, a_holding, l_value, l_taxbas, l_value, l_cash_rate, l_cash_rate_currency, a_idate)
				l_gross := gross_amount (False, manager.reference_system, a_portfolio, a_holding.instrument, l_tax_date, a_holding, l_net, l_taxbas, l_value, l_cash_rate, l_cash_rate_currency, a_idate)
					-- This is both for long and short positions
				if (not a_holding.amount.abs.is_zero) and then (not (l_gross.amount - l_net.amount).abs.is_zero) then
						-- Generate type 9 (gross) after 20080101
					if (not a_portfolio.zone.is_japan or a_holding.amount >= a_holding.amount.zero) or else (a_holding.amount < a_holding.amount.zero and then event_date (a_portfolio).ccyymmdd < 20080101) then
						l_tax_withholding := l_gross - l_net
					end
				end
				if a_portfolio.zone.is_japan then
						-- XXX: Japan for now. Perhaps other zones too.
						-- round shares to whole shares
					l_units_base := a_holding.instrument.quantity (a_holding.amount.rounded (0))
				else
					l_units_base := a_holding
				end
				create {RP_CASH_DIVIDEND} Result.make (l_units_base, l_gross, l_tax_withholding, event_date (a_portfolio), settle_date (a_portfolio), a_portfolio, unique_identifier, processing_system, settle_date (a_portfolio), unknown_custody)
			end
		end

feature {NONE} -- Implementation

	tax_basis: attached STRING is
			-- Capact tax basis
		do
			if capact.legacy_vendor_key /= Void and then capact.legacy_vendor_key.count > 4 then
				create Result.make_from_string (capact.legacy_vendor_key.substring (5, capact.legacy_vendor_key.count))
				Result.prune_all (' ')
			elseif capact.dl_taxflag /= Void then
				create Result.make_from_string (capact.dl_taxflag)
				Result.prune_all (' ')
			end
			if Result = Void or else Result.is_empty then
				create Result.make_from_string ("##")
			end
			Result.to_upper
		ensure
			tax_basis_not_void: Result /= Void
			no_space_in_tax_basis: not Result.has (' ')
		end

	cash_currency (a_instrument: RP_INSTRUMENT): RP_CURRENCY is
			-- Cash currency
		do
			Result := rp_currency_inferred (reference_system, a_instrument, iso_from_currency (capact.cash_currency))
		end

	cash_rate (a_instrument: RP_INSTRUMENT): RP_MONIES is
			-- Cash rate
		do
			Result := cash_currency (a_instrument).monies (capact.cash_rate)
		end

end
