note

	description:
	"[
		Legacy exercise of rights capact to contract generator
		
				The exercise is controlled by two optianal files in \\axaros.com\shares\ProdData_NT\PortfolioAccounting\ParmFiles
					- including: the default is to consider we have an entry here
		
		
					- excluding_rights_<z>.parm: the default is to consider we have no entry
				Both have the same format: zone, portfolio, ex_date, description
				We have an entry for another portfolio in the file, then the default is set to false.
				There are two special portfolios named "ALL" and "ALLPORT" in the including / excluding that are wildcards for all portfolios to be included or excluded
		
				The Excluding_rights_e.parm file has two other columns to control the cash rate (not used?) and pay date
		
				The logic is as follows:
					if there is an inclusion, then we want to exercise the right and convert the rights into shares (creation of RP_RIGHTS)
					If there is an exclusion, then we don't want the rights and we give them away (creation of RP_DISTRIBUTION)
	]"
	author: "Xavier Ragot"
	last_editor: "$Author: e2 $"
	date: "$Date: 2011/11/29 $"
	revision: "$Revision: #14 $"
	path: "$File: //rose/source/Kernel/capital_actions/elections/rp_corporate_action_election_exercise_rights.e $"

class RP_CORPORATE_ACTION_ELECTION_EXERCISE_RIGHTS

inherit

	RP_CORPORATE_ACTION_ELECTION [CAPACT_EXERCISE_RIGHTS_EXPRESSION, RP_PORTFOLIO_CORPORATE_ACTION_CONTRACT]

	RP_INCLUDING_EXCLUDING_RIGHTS_AWARE
		export {NONE} all end

create

	make

feature -- Access

	contract (a_idate: ROSE_DATE; a_portfolio: RP_PORTFOLIO; a_holding: RP_QUANTITY): RP_PORTFOLIO_CORPORATE_ACTION_CONTRACT is
			-- <Precursor>
		local
			l_including, l_excluding: RP_INCLUDING_EXCLUDING_RIGHTS_PARM
			l_shares_held, l_shares_increment: RP_DECIMAL
			l_contract_units: RP_DECIMAL
			l_share_factor: RP_DECIMAL
			l_cash_effect: RP_MONIES
			l_contract_settle_date: ROSE_DATE
			l_instrument: RP_INSTRUMENT
			l_currency: RP_CURRENCY
			l_contract_units_based: RP_DECIMAL
		do
			if a_holding.amount.abs >= 1 then
					-- We decided to take the right
				l_instrument := a_holding.instrument
				l_currency := subscription_currency (l_instrument)
					-- FIXME: do we need to consider round lots
					-- Retrieve potential overrides
				l_including := including_rights (a_portfolio.zone, a_holding.instrument.brimsym, a_idate, a_portfolio.code)
				l_excluding := excluding_rights (a_portfolio.zone, a_holding.instrument.brimsym, a_idate, a_portfolio.code)
					-- The factor gives the number of shares we can acquire relative to the shares we hold
					-- The cash effect is that cash increment multiplied by the price per share to acquire the right
				l_shares_held := a_holding.amount
				if l_excluding = Void and l_including /= Void then
					l_share_factor := capact.share_factor - 1
					l_contract_settle_date := l_including.pay_date
					l_shares_increment := (l_share_factor * l_shares_held).truncated
						-- FIXME: probably not in U zone
					l_contract_units := l_shares_increment
					l_contract_units_based := l_shares_held
						-- we round to 2 decimals to be compatible with the event file
					l_cash_effect := l_currency.monies ((create {RP_DECIMAL}.make_from_double (capact.cash_rate * l_shares_held)).rounded (2))
					create {RP_RIGHTS} Result.make (a_holding, a_holding.instrument.quantity (l_contract_units), l_cash_effect, event_date (a_portfolio), l_contract_settle_date, a_portfolio, unique_identifier, processing_system, unknown_custody)
				elseif l_excluding = Void and l_including = Void then
						-- We decided something for some portfolios but it's not here for this one!
						-- FIXME: we used to write to EVENT_'//ZONE//'_L:NEW_RIGHTS.TMP
					(create {EXCEPTIONS}).raise ("Warning missing rights configuration for: " + a_holding.instrument.brimsym + " " + a_idate.ccyymmdd.out + "%N")
				else
						-- We don't take the right
					if l_excluding /= Void and l_including = Void then
						l_shares_increment := 0
						l_contract_units := l_shares_held
						l_contract_units_based := 0
						l_cash_effect := l_currency.monies (0)
						l_contract_settle_date := l_excluding.pay_date
							-- FIXME: we should record the cost basis (for O zone)
							-- We give away the rights
							-- Create a distribution, not an exercise of rights
						create {RP_DISTRIBUTION} Result.make_long_position (a_holding, a_holding.instrument.quantity (l_contract_units), l_cash_effect, a_idate, l_contract_settle_date, a_portfolio, unique_identifier, processing_system, unknown_custody)
					else
							-- FIXME: present in both file - abort??
						(create {EXCEPTIONS}).raise ("Inconsitent inclusion / exclusion for " + a_holding.instrument.brimsym + " " + capact.exdate.ccyymmdd.out)
					end
				end
			end
		end

feature {NONE} -- Implementation

	share_factor: RP_DECIMAL is
			-- Capact share factor
		do
			Result := capact.adjustment_factor
		end

	subscription_currency (a_instrument: RP_INSTRUMENT): RP_CURRENCY is
			-- Capact subscription
		do
			Result := axarosenberg.reference_system.currency_from_iso_code (subscription_price_per_share (a_instrument).currency.brimsym)
		end

	cash_currency_infered (a_instrument: RP_INSTRUMENT): RP_CURRENCY is
			-- Capact cash currency infered
		do
			Result := rp_currency_inferred (axarosenberg.reference_system, a_instrument, iso_from_currency (capact.cash_currency))
		end

	subscription_price_per_share (a_instrument: RP_INSTRUMENT): RP_MONIES is
			-- Capact subscription price per share
		do
			Result := cash_currency_infered (a_instrument).monies (capact.subscription_price)
		end

end
