note

	description:

		"Return of cash capact to contract generator"

	author: "Sami Ben-Grine"
	last_editor: "$Author: e2 $"
	date: "$Date: 2011/11/29 $"
	revision: "$Revision: #7 $"
	path: "$File: //rose/source/kernel/capital_actions/elections/rp_corporate_action_election_return.e $"

class RP_CORPORATE_ACTION_ELECTION_RETURN

inherit

	RP_CORPORATE_ACTION_ELECTION [CAPACT_RETURN_EXPRESSION, RP_RETURN_OF_CAPITAL]

	RP_WITHHOLDING_TAX_GROSS_UP
		export {NONE} all end

create

	make

feature -- Access

	contract (a_idate: ROSE_DATE; a_portfolio: RP_PORTFOLIO; a_holding: RP_QUANTITY): RP_RETURN_OF_CAPITAL is
			-- <Precursor>
		local
			l_value: RP_MONIES
			l_net, l_gross, l_tax_withholding: RP_MONIES
			l_shares_held: RP_DECIMAL
			l_cash_rate: RP_DECIMAL
			l_cash_rate_currency: RP_CURRENCY
			l_taxbas: STRING
			l_tax_date: ROSE_DATE
		do
			if a_holding.amount.abs >= 1 then
				l_shares_held := a_holding.amount
				l_cash_rate := capact.cash_rate
				if capact.cash_currency = Void then
					manager.log_minor ("ERROR: NO ROC CURRENCY " + a_idate.ccyymmdd.out + " " + a_holding.instrument.brimsym)
					l_cash_rate_currency := a_portfolio.zone.legacy_base_currency
				else
					l_cash_rate_currency := manager.reference_system.currency_from_iso_code (capact.cash_currency.iso)
				end
				l_value := l_cash_rate_currency.monies (l_shares_held * l_cash_rate)
				l_taxbas := tax_basis
				l_tax_date := settle_date (a_portfolio)
				l_net := net_amount (manager.corporate_action_processing_system.use_rate_overrides_for_cash_dividends, manager.reference_system, a_portfolio, a_holding.instrument, l_tax_date, a_holding, l_value, l_taxbas, l_value, l_cash_rate, l_cash_rate_currency, a_idate)
				l_gross := gross_amount (False, manager.reference_system, a_portfolio, a_holding.instrument, l_tax_date, a_holding, l_net, l_taxbas, l_value, l_cash_rate, l_cash_rate_currency, a_idate)
				if (a_holding.amount >= 0) and then not (l_gross.amount - l_net.amount).abs.is_zero then
					l_tax_withholding := l_gross - l_net
				end
				create Result.make (a_holding, l_gross, l_tax_withholding, event_date (a_portfolio), settle_date (a_portfolio), a_portfolio, unique_identifier, processing_system, unknown_custody)
			end
		end

feature {NONE} -- Implementation

	tax_basis: !STRING is
			-- Capact tax basis
		local
			s1, s2, s3: STRING
		do
			if capact.dl_taxflag /= Void then
				create Result.make_from_string (capact.dl_taxflag)
				Result.prune_all (' ')
			end
			if Result = Void and then capact.legacy_vendor_key /= Void and then capact.legacy_vendor_key.count = 6 then
				create Result.make_from_string (capact.legacy_vendor_key.substring (5, 6))
				Result.prune_all (' ')
			end
			if Result = Void or else Result.is_empty then
				create Result.make_from_string ("##")
			end
			debug ("rp_corporate_action_election_return")
				if capact.legacy_vendor_key /= Void then
					create s1.make_from_string (capact.legacy_vendor_key)
				else
					s1 := "n/a"
				end
				if capact.legacy_txbas /= Void then
					create s2.make_from_string (capact.legacy_txbas)
				else
					s2 := "n/a"
				end
				if capact.dl_taxflag /= Void then
					create s3.make_from_string (capact.dl_taxflag)
				else
					s3 := "n/a"
				end
				manager.log_debug ("TXBAS[" + Result + "] LVK[" + s1 + "] LTX[" + s2 + "] DL[" + s3 + "]")
			end
		ensure
			result_not_void: Result /= Void
		end

	cash_currency (a_instrument: RP_INSTRUMENT): RP_CURRENCY is
			-- Cash currency
		do
			Result := rp_currency_inferred (reference_system, a_instrument, iso_from_currency (capact.cash_currency))
		end

	cash_rate (a_instrument: RP_INSTRUMENT): RP_QUANTITY is
			-- Cash rate
		do
			Result := cash_currency (a_instrument).monies (capact.cash_rate)
		end

end
