note

	description:

		"Rights in same capact to contract generator"

	author: "Sami Ben-Grine"
	last_editor: "$Author: e2 $"
	date: "$Date: 2011/11/29 $"
	revision: "$Revision: #6 $"
	path: "$File: //rose/source/kernel/capital_actions/elections/rp_corporate_action_election_rights_in_same.e $"

class RP_CORPORATE_ACTION_ELECTION_RIGHTS_IN_SAME

inherit

	RP_CORPORATE_ACTION_ELECTION [CAPACT_RIGHTS_IN_SAME_EXPRESSION, RP_RIGHTS]

create

	make

feature -- Access

	contract (a_idate: ROSE_DATE; a_portfolio: RP_PORTFOLIO; a_holding: RP_QUANTITY): RP_RIGHTS is
			-- <Precursor>
		local
			l_cash_effect: RP_MONIES
			l_rights_units: RP_QUANTITY
			l_value: DOUBLE
			l_holding: RP_HOLDING
			l_rp_currency: RP_CURRENCY
			l_subscription_price_per_share: RP_MONIES
		do
			if not a_holding.amount.is_zero then
				l_rp_currency := cash_currency (a_holding.instrument)
				if l_rp_currency /= Void then
					if subscription_rate > 0 and subscription_price > 0 then
							-- determine price for parent instrument on ex date
						create l_holding.make (a_holding, a_portfolio.instrument_control (unknown_custody))
						l_value := processing_system.manager.performance_system.default_holding_valuator_factory.portfolio_holding_valuator (a_portfolio).value_in_currency (l_holding, l_rp_currency, event_date (a_portfolio), "").amount.to_double / a_holding.amount.to_double
						if l_value.abs < k_price_epsilon then
							manager.log_info ("Unable to get price for " + capact.parent_instrument.brim_code (event_date (a_portfolio)) + " on " + event_date (a_portfolio).ccyymmdd.out + " - forcing exercise")
						end
							-- simulation policy is to exercise if subscription price is less than market price
						l_subscription_price_per_share := subscription_price_per_share (a_holding.instrument)
						if l_subscription_price_per_share.amount < l_value or l_value.abs < k_price_epsilon then
								-- force exercise if price is 0
							l_rights_units := a_holding.instrument.quantity ((a_holding.amount * (share_factor - 1)).rounded (2))
							l_cash_effect := -l_subscription_price_per_share.currency.monies (l_rights_units.amount * l_subscription_price_per_share.amount)
						else
								-- create a 0 share impact contract
							l_rights_units := a_holding.instrument.quantity (0)
							l_cash_effect := l_subscription_price_per_share.currency.monies (0)
						end
						create Result.make (a_holding, l_rights_units, l_cash_effect, event_date (a_portfolio), settle_date (a_portfolio), a_portfolio, unique_identifier, processing_system, unknown_custody)
					end
				end
				if Result = Void then
					manager.log_minor ("*** ERROR processing " + generator + " " + a_holding.instrument.brimsym + " on " + capact.exdate.ccyymmdd.out)
					(create {EXCEPTIONS}).raise ("*** ERROR processing " + generator + " " + a_holding.instrument.brimsym + " on " + capact.exdate.ccyymmdd.out)
				end
			end
		end

feature -- Basic operations

	share_factor: RP_DECIMAL is
			-- Share factor
		do
			Result := capact.subscription_rate + 1
		end

	cash_currency (a_instrument: RP_INSTRUMENT): RP_CURRENCY is
			-- Cash currency
		do
			Result := rp_currency_inferred (reference_system, a_instrument, iso_from_currency (capact.cash_currency))
		end

	subscription_price_per_share (a_instrument: RP_INSTRUMENT): RP_MONIES is
			-- Subscription price per share
		do
			Result := cash_currency (a_instrument).monies (subscription_price)
		end

	subscription_price: RP_DECIMAL is
			-- Capact subscription price
		do
			Result := capact.subscription_price
		end

	subscription_rate: RP_DECIMAL is
			-- Subscription rate
		do
			Result := capact.subscription_rate
		end

feature -- Constants

	k_price_epsilon: RP_DECIMAL is
			-- Price epsilon
		once
			create Result.make_from_string ("0.0001")
		end

end
