indexing

	description: "Instructions on how to process corporate actions into contracts"

	date: "$Date: 2009/11/06 $"
	sccid: "$Id: //rose/source/kernel/capital_actions/rp_corporate_action_processing_instructions.e#9 $"

class RP_CORPORATE_ACTION_PROCESSING_INSTRUCTIONS

inherit

	RP_CORPORATE_ACTION_PROCESSING_INSTRUCTION_TYPES

	CAPACT_UTILS
		export {NONE} all end

	DEBUG_OUTPUT

create

	make

feature {NONE} -- Initialization

	make ( a_type: like type; a_ca: RP_CORPORATE_ACTION) is
			-- Initialize `Current'
		require
			valid_args: a_type /= Void and a_ca /= Void
		do
			type := a_type
			create corporate_actions.make_empty
			corporate_actions.extend (a_ca)
		ensure
			type_assigned: type = a_type
		end

feature -- Access

	type: RP_CORPORATE_ACTION_PROCESSING_SHARE_BASIS_TYPE
			-- Type of shares basis

	corporate_actions: ROSE_LINEAR_ARRAY [RP_CORPORATE_ACTION]
		-- Processors

feature -- Basic operations

	add_to_corporate_actions (a_corp_action: RP_CORPORATE_ACTION) is
			-- extend corporate action array
		require
			arg_not_void: a_corp_action /= Void
		do
			corporate_actions.extend (a_corp_action)
		end

feature -- Status Report

	is_use_original_shares: BOOLEAN is
			-- True if corporate actions are based on original shares
		do
			Result := type.type = original_shares
		end

	is_use_current_shares: BOOLEAN is
			-- True if corporate actions are based on current shares. current shares may be shares after some processing
		do
			Result := type.type = current_shares
		end

	is_use_delta_shares: BOOLEAN is
			-- True if corporate actions are based on change in shares.
		do
			Result := type.type = delta_shares
		end

	is_error: BOOLEAN
		-- Is in error?

feature -- Status Setting

	set_error (a_is_error: BOOLEAN) is
			--  set error flag
		do
			is_error := a_is_error
		ensure
			error_set: is_error = a_is_error
		end

feature -- Status report

	debug_output: STRING is
			-- <Precursor>
		do
			Result := type.debug_output
			Result.append (" ")
			from
				corporate_actions.start
			until
				corporate_actions.after
			loop
				Result.append (corporate_actions.item_for_iteration.debug_output)
				Result.append ("%T")
				corporate_actions.forth
			end
		end

invariant

	one_but_not_other: is_use_original_shares xor is_use_current_shares
	type_not_void: type /= Void
	at_least_one_corporate_action: (corporate_actions /= Void) and then (corporate_actions.count > 0)

end
