indexing

	description:

		"Corporate bond"

	date: "$Date: 2010/02/22 $"
	revision: "$Revision: #8 $"

class RP_CORPORATE_BOND

inherit

	RP_BOND
		redefine
			uid
		end

create {RP_REFERENCE_SYSTEM}

	make_with_bond

feature {NONE} -- Initialization

	make_with_bond (a_reference_system: RP_REFERENCE_SYSTEM; a_bond: BOND; a_brimsym: STRING) is
			-- Intialize `Current'.
		require
			non_void_args: a_bond /= Void and a_brimsym /= Void
		local
			s: STRING
		do
			uid := a_bond.uid
			name := "(" + a_brimsym + ")"
			rose_units := share_units (a_bond)
			brimsym := a_brimsym
			s := a_bond.currency_code_most_recent_key (Void)
			if s /= Void then
				currency := a_reference_system.currency_from_iso_code (s)
			else
				s := a_bond.country_code_most_recent_key (Void)
				if s /= Void then
					currency := a_reference_system.currency_from_iso_code (s)
				end
			end
			if a_bond.coupon_rate_key (Void) /= Void then
				s := a_bond.coupon_rate_most_recent_key (Void)
				create coupon_rate.make_from_string (s)
				coupon_method := a_bond.coupon_method_most_recent_key (Void).to_integer
				months := a_bond.coupon_frequency_in_months_most_recent_key (Void).to_integer
			else
				coupon_rate := 0
			end
			create maturity_date.make_ccyymmdd (a_bond.maturity_date_most_recent_key (Void).to_integer)
--			create inception_date.make_ccyymmdd (a_bond.first_data_date.ccyymmdd)
			create par_value.make_from_string (a_bond.par_value_most_recent_key (Void))
			accrual_method := a_bond.accrual_method_most_recent_key (Void).to_integer
			entity_id := a_bond.handle
			create frn_rates.make_empty
			create coupons.make_empty
		end

feature -- Access

	uid: INTEGER_64
			-- Unique identifier

	entity_id: INTEGER
			-- Underlying entity id

	currency: RP_CURRENCY
			-- Payment currency

	name: STRING
			-- Name of the instrument

	brimsym: STRING
			-- Brimsym.

	months: INTEGER
			--

	coupon_rate: RP_DECIMAL
			-- Coupon rate

	coupon_method: INTEGER
			-- Coupon method

	maturity_date: ROSE_DATE
			-- Maturity date

	par_value: RP_DECIMAL
			-- Par value paid

	accrual_method: INTEGER
			-- Accrual method

	inception_date: ROSE_DATE
			-- Inception date

	coupon_dates: ARRAY [ROSE_DATE] is
			-- Coupon dates
		do
			Result := coupons
		end

feature -- Query

	country_of_issue (a_ref_system: RP_REFERENCE_SYSTEM; a_date: ROSE_DATE): RP_COUNTRY is
			-- The country of issue for this instrument
		do
			Result := a_ref_system.country_from_iso_code (brimsym.substring (1, 2))
		end

	holding_value_in_currency (a_holding: RP_HOLDING; a_valuator: RP_HOLDING_VALUATOR; a_currency: RP_CURRENCY; a_date: ROSE_DATE; a_bucket:STRING): RP_MONIES is
			-- Value of `a_holding' on `a_date' in `a_currency' as computed by `a_valuator'
		do
			Result := a_valuator.corporate_bond_value_in_currency (a_holding, a_currency, a_date, a_bucket)
		end

	frn_rate (a_date: ROSE_DATE): RP_DECIMAL is
			-- Floating rate note (FRN) rate on `a_date'
			-- -1 means no FRN rate found
		require
			non_void_date: a_date /= Void
		do
			if frn_rates /= Void then
				Result := frn_rates.witness (a_date)
			end
			if Result = Void then
				Result := -1
			end
		end

feature {RP_REFERENCE_SYSTEM} -- Change

	add_frn_rate (a_start, a_finish: ROSE_DATE; a_rate: RP_DECIMAL) is
			-- Add FRN rate `a_rate' with validity range from `a_start' to and including `a_finish'
		require
			non_void_args: a_start /= Void and a_finish /= Void and a_rate /= Void
		local
			l_interval: ROSE_CLOSED_INTERVAL [RP_DECIMAL]
		do
			create l_interval.make (a_rate, a_start, a_finish, a_rate)
			frn_rates.extend (l_interval)
		end

	add_coupon_date (a_date: ROSE_DATE) is
			-- Add coupon date `a_date'
		require
			non_void_date: a_date /= Void
		do
			coupons.extend (a_date)
		end

feature {NONE} -- Implementation

	frn_rates: ROSE_INTERVAL_HISTORY [RP_DECIMAL, ROSE_CLOSED_INTERVAL [RP_DECIMAL]]

	coupons: ROSE_LINEAR_ARRAY [ROSE_DATE]

end
