indexing

	description: "Method to distribute cost using average between tax lots"

	date: "$Date: 2010/09/06 $"
	sccid: "$Id: //rose/source/kernel/expert_system/cost/rp_cost_accounting_method_average.e#5 $"

class RP_COST_ACCOUNTING_METHOD_AVERAGE

inherit

	RP_COST_ACCOUNTING_METHOD

feature -- Basis Operations

	disburse (a_flow: RP_FLOW; a_basis: RP_COST_BASIS) is
			-- <Precursor>
		local
			l_total_net: RP_DECIMAL
			l_pro_rata_ratio: RP_DECIMAL
			l_to_disburse, l_disbursed: RP_DECIMAL
			l_disbursed_units: DS_HASH_TABLE [RP_DECIMAL, INTEGER]
			l_disbursed_units_cursor: DS_HASH_TABLE_CURSOR [RP_DECIMAL, INTEGER]
			l_desired: RP_DECIMAL
			l_remainder: RP_COST
			l_to_collect: DS_LINKED_LIST [RP_COST]
			l_hash_cursor: DS_HASH_TABLE_CURSOR [RP_TAX_LOT, INTEGER]
		do
			l_to_disburse := a_flow.signed_quantity.amount.abs
			if a_basis.tax_lots.is_empty then
				debug ("rp_cost_accounting_method")
					print (a_flow.contract.identifier + " " + a_flow.quantity.instrument.brimsym + " -" + l_to_disburse.out + " NO LOTS%N")
				end
				l_remainder := a_flow.basis.proportion (a_flow.quantity.instrument.quantity (-l_to_disburse))
				a_basis.collect (l_remainder)
			else
				create l_to_collect.make_default
				if not a_basis.non_exhausted_tax_lots.is_empty then
					-- compute total net available units
					from
						l_total_net := zero
						l_hash_cursor := a_basis.non_exhausted_tax_lots.new_cursor
						l_hash_cursor.start
					until
						l_hash_cursor.after
					loop
						l_total_net := l_total_net + l_hash_cursor.item.net.abs
						l_hash_cursor.forth
					end
					-- limit amount to allocate to total available
					if l_to_disburse > l_total_net then
						debug ("rp_cost_accounting_method")
							print (a_flow.contract.identifier + " " + a_flow.quantity.instrument.brimsym + " " + l_to_disburse.out + " - " + l_total_net.out + " = " + (l_to_disburse - l_total_net).out + "%N")
						end
						l_to_disburse := l_to_disburse - l_total_net
						l_remainder := a_flow.basis.proportion (a_flow.quantity.instrument.quantity (-l_to_disburse))
						l_to_collect.force_last (l_remainder)
						l_to_disburse := l_total_net
					end
					-- compute pro-rata ratio
					l_pro_rata_ratio := l_to_disburse / l_total_net
					-- compute pro-rated available units
					create l_disbursed_units.make (a_basis.non_exhausted_tax_lots.count)
					from
						l_disbursed := zero
						l_hash_cursor := a_basis.non_exhausted_tax_lots.new_cursor
						l_hash_cursor.start
					until
						l_hash_cursor.after or l_disbursed >= l_to_disburse
					loop
						l_desired := l_hash_cursor.item.net.abs.min (special_rounding (l_pro_rata_ratio * l_hash_cursor.item.net.abs))
						if l_disbursed + l_desired > l_to_disburse then
							l_desired := l_to_disburse - l_disbursed
						end
						l_disbursed := l_disbursed + l_desired
						l_disbursed_units.force (l_desired, l_hash_cursor.key)
						l_hash_cursor.forth
					end
					-- is there a reminder?
					l_to_disburse := l_to_disburse - l_disbursed
					from
						l_disbursed := -1
					until
						l_disbursed.is_zero or l_to_disburse.is_zero
					loop
						l_disbursed := zero
						from
							l_hash_cursor := a_basis.non_exhausted_tax_lots.new_cursor
							l_hash_cursor.start
						until
							l_hash_cursor.after or l_to_disburse.is_zero
						loop
							l_desired := l_hash_cursor.item.net.abs
							l_disbursed_units.search (l_hash_cursor.key)
							if l_disbursed_units.found then
								l_desired := l_desired - l_disbursed_units.found_item
							end
							if l_desired > zero then
									-- disburse one unit
								l_desired := l_to_disburse.min (l_desired).min (l_desired.one)
								l_disbursed := l_disbursed + l_desired
								l_disbursed_units.force (l_disbursed_units.found_item + l_desired, l_hash_cursor.key)
								l_to_disburse := l_to_disburse - l_desired
							end
							l_hash_cursor.forth
						end
					end
					if not l_to_disburse.is_zero then
						debug ("rp_cost_accounting_method")
							print (a_flow.contract.identifier + " " + a_flow.quantity.instrument.brimsym + " -" + l_to_disburse.out + "%N")
						end
						l_remainder := a_flow.basis.proportion (a_flow.quantity.instrument.quantity (-l_to_disburse))
						l_to_collect.force_last (l_remainder)
					end
					-- disburse
					from
						l_disbursed_units_cursor := l_disbursed_units.new_cursor
						l_disbursed_units_cursor.start
					until
						l_disbursed_units_cursor.after
					loop
						a_basis.non_exhausted_tax_lots.search (l_disbursed_units_cursor.key)
						if a_basis.non_exhausted_tax_lots.found then
							disburse_units (l_disbursed_units_cursor.item, a_basis.non_exhausted_tax_lots.found_item, a_flow)
						else
							(create {EXCEPTIONS}).raise ("Disbursment for non-existent tax lot " + a_flow.contract.identifier)
						end
						l_disbursed_units_cursor.forth
					end
				else
					debug ("rp_cost_accounting_method")
						print (a_flow.contract.identifier + " " + a_flow.quantity.instrument.brimsym + " -" + l_to_disburse.out + " NET " + l_total_net.out + "%N")
					end
					l_remainder := a_flow.basis.proportion (a_flow.quantity.instrument.quantity (-l_to_disburse))
					l_to_collect.force_last (l_remainder)
				end
				l_to_collect.do_all (agent a_basis.collect (?))
			end
		end

feature {NONE} -- Implementation

	zero: RP_DECIMAL is
			-- Zero
		once
			create Result.make_zero
		end

	special_rounding (d: RP_DECIMAL): RP_DECIMAL is
			-- Round `d'
		require
			non_void_d: d /= Void
			positive: d >= d.zero
		do
			Result := d.rounded (8)
		end

	disburse_units (a_disbursed_units: RP_DECIMAL; a_tax_lot: RP_TAX_LOT; a_flow: RP_FLOW) is
			-- Disburse units
		do
			if not a_disbursed_units.is_zero then
				a_tax_lot.disburse (a_disbursed_units.abs, a_flow)
			end
		end

end
