indexing

	description: "FIFO cost accounting method"

	date: "$Date: 2010/08/16 $"
	sccid: "$Id: //rose/source/kernel/expert_system/cost/rp_cost_accounting_method_fifo.e#3 $"

class RP_COST_ACCOUNTING_METHOD_FIFO

inherit

	RP_COST_ACCOUNTING_METHOD

create

	make_fifo, make_lifo

feature {NONE} -- Initialization

	make_fifo is
			-- FIFO cost distribution method
		do
			is_lifo := False
			is_fifo := not is_lifo
		ensure
			fifo: is_fifo
			not_lifo: not is_lifo
		end

	make_lifo is
			-- LIFO cost distribution method
		do
			is_fifo := False
			is_lifo := not is_fifo
		ensure
			lifo: is_lifo
			not_fifo: not is_fifo
		end

feature -- Status

	is_fifo: BOOLEAN
			-- Is FIFO method?

	is_lifo: BOOLEAN
			-- Is LIFO method?

feature -- Basis Operations

	disburse (a_flow: RP_FLOW; a_basis: RP_COST_BASIS) is
			-- <Precursor>
		local
			l_to_disburse, l_disbursed: RP_DECIMAL
			l_remainder: RP_COST
			l_cursor: DS_LINKED_LIST_CURSOR [RP_TAX_LOT]
		do
			l_to_disburse := a_flow.signed_quantity.amount.abs
			if a_basis.tax_lots.is_empty then
				l_remainder := a_flow.basis.proportion (a_flow.quantity.instrument.quantity (-l_to_disburse))
				a_basis.collect (l_remainder)
			else
				if not a_basis.non_exhausted_tax_lots.is_empty then
					from
						l_cursor := a_basis.tax_lots.new_cursor
						begin (l_cursor)
					until
						traversed (l_cursor) or l_to_disburse.is_zero
					loop
						if not l_cursor.item.is_exhausted then
							l_disbursed := l_to_disburse.min (l_cursor.item.net)
							disburse_units (l_disbursed, l_cursor.item, a_flow)
							l_to_disburse := l_to_disburse - l_disbursed
						end
						advance (l_cursor)
					end
					if not l_to_disburse.is_zero then
						l_remainder := a_flow.basis.proportion (a_flow.quantity.instrument.quantity (-l_to_disburse))
						a_basis.collect (l_remainder)
					end
				else
					l_remainder := a_flow.basis.proportion (a_flow.quantity.instrument.quantity (-l_to_disburse))
					a_basis.collect (l_remainder)
				end
			end
		end

feature {NONE} -- Implementation

	begin (a_cursor: DS_LINKED_LIST_CURSOR [RP_TAX_LOT]) is
			-- Initiate cursor
		require
			non_void_cursor: a_cursor /= Void
		do
			if is_fifo then
				a_cursor.start
			else
				a_cursor.finish
			end
		end

	traversed (a_cursor: DS_LINKED_LIST_CURSOR [RP_TAX_LOT]): BOOLEAN is
			-- Check cursor position
		require
			non_void_cursor: a_cursor /= Void
		do
			if is_fifo then
				Result := a_cursor.after
			else
				Result := a_cursor.before
			end
		end

	advance (a_cursor: DS_LINKED_LIST_CURSOR [RP_TAX_LOT]) is
			-- Advance cursor
		require
			non_void_cursor: a_cursor /= Void
		do
			if is_fifo then
				a_cursor.forth
			else
				a_cursor.back
			end
		end

	disburse_units (a_disbursed_units: RP_DECIMAL; a_tax_lot: RP_TAX_LOT; a_flow: RP_FLOW) is
			-- Disburse units
		do
			if not a_disbursed_units.is_zero then
				a_tax_lot.disburse (a_disbursed_units.abs, a_flow)
			end
		end

invariant

	either_fifo_or_lifo: is_fifo xor is_lifo

end
