indexing

	description:

		"Cost computing system."

	date: "$Date: 2010/09/06 $"
	sccid: "$Id: //rose/source/kernel/expert_system/cost/rp_cost_accounting_system.e#5 $"

class RP_COST_ACCOUNTING_SYSTEM

inherit

	RP_INVESTMENT_MANAGER_CLIENT
		redefine
			make
		end

	RP_CONTRACT_PROCESSOR
		redefine
			process_tender,
			process_stock_bonus,
			process_split,
			process_rights,
			process_return_of_capital,
			process_redenomination,
			process_portfolio_corporate_action_compound_contract,
			process_dividend,
			process_delivery,
			process_distribution,
			process_delivery_distribution,
			process_income_unrelated,
			process_income,
			process_expense_unrelated,
			process_expense,
			process_fixed_deposit_transfer,
			process_mark_to_market,
			process_interest,
			process_inlieu,
			process_dividend_reinvestment,
			process_cash_swept_from_interest,
			process_cash_swept_from_income,
			process_cash_swept_from_dividend,
			process_collateral,
			process_client_withdrawal,
			process_client_contribution,
			process_cash_zonal_tranfer,
			process_short,
			process_sell,
			process_buy,
			process_cover,
			process_intraday_sell_to_adjust_gl,
			process_fx,
			process_ffx_open,
			process_ffx_close,
			process_futures_short,
			process_futures_sell,
			process_futures_cover,
			process_futures_buy,
			process_ffx_gain_loss,
			process_bank_transfer,
			process_income_unrelated_adjustment,
			process_income_adjustment,
			process_expense_unrelated_adjustment,
			process_expense_adjustment,
			process_dividend_tax_refund,
			process
		end

create

	make

feature {NONE} -- Initialization

	make (a_manager: like manager) is
			-- <Precursor>
		do
			Precursor (a_manager)
			clear_caches
		end

feature -- Initialization

	clear_caches is
			-- Clear caches
		do
			create costs.make (0)
		ensure
			costs_empty: (costs /= Void) and then (costs.is_empty)
		end

feature -- Status

	is_debug_output_enabled: BOOLEAN is
			-- Is debug output enabled?
		do
			Result := debug_output /= Void
		end

	send_debug_output_to (a_file_name: STRING) is
			-- Send debut output to `a_file_name'
		require
			non_void_args: a_file_name /= Void
			debug_disabled: not is_debug_output_enabled
		do
			create debug_output.make_open_write (a_file_name)
		end

	disable_debug_output is
			-- Disable debug output
		require
			debug_enabled: is_debug_output_enabled
		do
			debug_output.close
			debug_output := Void
		end

	debug_brimsym: STRING
			-- Brimsym of debugging interest

	set_debug_brimsym (v: like debug_brimsym) is
			-- Set value of `debug_brimsym' to `v'
		do
			debug_brimsym := v
		ensure
			debug_brimsym_set: debug_brimsym = v
		end

feature -- Access

	method (a_flow: RP_FLOW): !RP_COST_ACCOUNTING_METHOD is
			-- Cost accounting method for 'a_flow'
		require
			flow_not_void: a_flow /= Void
		local
			l_base_currency: RP_CURRENCY
		do
--			l_base_currency := a_flow.contract.account.base_currencies.witness (a_flow.contract.agreement_time)
			if l_base_currency = Void then
				l_base_currency := a_flow.contract.portfolio.zone.legacy_base_currency
			end
			if a_flow.instrument.is_equal (l_base_currency) then
					-- Use FIFO for account base currency
				Result := fifo
			else
					-- Very simplied for now
				if not methods.has (a_flow.contract.account) then
					create {RP_COST_ACCOUNTING_METHOD_AVERAGE} Result
					methods.force (Result, a_flow.contract.account)
				else
					Result := methods.item (a_flow.contract.account)
				end
			end
		ensure
			non_void_result: Result /= Void
		end

	long_side_basis (a_portfolio: RP_ZONAL_INVESTMENT_GROUP; a_instrument: RP_INSTRUMENT): RP_COST_BASIS is
			-- Tax lots in the order they have been created
		local
			l_key: TUPLE [RP_ZONAL_INVESTMENT_GROUP, RP_INSTRUMENT, BOOLEAN]
		do
			l_key := [a_portfolio, a_instrument, True]
			if not costs.has (l_key) then
				create Result.make (costs.count + 1, a_portfolio, a_instrument, True)
				costs.put (Result, l_key)
			end
			Result := costs.item (l_key)
		ensure
			result_not_void: Result /= Void
			basis_from_cost: Result = costs.item ([a_portfolio, a_instrument, True])
		end

	short_side_basis (a_portfolio: RP_ZONAL_INVESTMENT_GROUP; a_instrument: RP_INSTRUMENT): RP_COST_BASIS is
			-- Tax lots in the order they have been created
		local
			l_key: TUPLE [RP_ZONAL_INVESTMENT_GROUP, RP_INSTRUMENT, BOOLEAN]
		do
			l_key := [a_portfolio, a_instrument, False]
			if not costs.has (l_key) then
				create Result.make (costs.count + 1, a_portfolio, a_instrument, False)
				costs.put (Result, l_key)
			end
			Result := costs.item (l_key)
		ensure
			result_not_void: Result /= Void
			basis_from_cost: Result = costs.item ([a_portfolio, a_instrument, False])
		end

	basis_for_instruments (a_contract: RP_PORTFOLIO_CONTRACT): UID_TABLE [RP_COST_BASIS, RP_INSTRUMENT] is
			-- Basis for all instruments in the flows of 'a_contract'
		require
			contract_not_void: a_contract /= Void
		local
			l_portfolio: RP_PORTFOLIO
			l_instrument: RP_INSTRUMENT
		do
			create Result.make_default
			from
				a_contract.flows.start
			until
				a_contract.flows.after
			loop
				l_instrument := a_contract.flows.item_for_iteration.instrument
				l_portfolio := a_contract.flows.item_for_iteration.contract.portfolio
				if not Result.has (l_instrument) then
					Result.force (long_side_basis (l_portfolio, l_instrument), l_instrument)
				end
				a_contract.flows.forth
			end
		ensure
			result_not_void: Result /= Void
		end

feature -- Basic Operations

	process (a_contract: RP_CONTRACT) is
			-- Process 'a_contract'
		do
			debug ("rp_cost_accounting_system")
				if a_contract /= Void then
					manager.log_debug ("Processing cost for " + a_contract.identifier)
				end
			end
			Precursor (a_contract)
		end

feature {NONE} -- Implementation

	process_short (a_short: RP_SHORT) is
			-- Process 'a_short'
		do
			a_short.flows.do_all (agent process_flow (a_short, ?, True))
		end

	process_cover (a_cover: RP_COVER) is
			-- Process 'a_cover'
		do
			a_cover.flows.do_all (agent process_flow (a_cover, ?, True))
		end

	process_buy (a_buy: RP_BUY) is
			-- Process 'a_buy'
		do
			process_flows (a_buy)
		end

	process_sell (a_sell: RP_SELL) is
			-- Process 'a_sell'
		do
			process_flows (a_sell)
		end

	process_income (a_income: RP_INCOME) is
			-- Process 'a_income'
		do
			process_flows (a_income)
		end

	process_expense (a_expense: RP_EXPENSE) is
			-- Process 'a_expense'
		do
			process_flows (a_expense)
		end

	process_income_unrelated (a_unrelated_income: RP_INCOME_UNRELATED) is
			-- Process 'a_unrelated_income'
		do
			process_flows (a_unrelated_income)
		end

	process_expense_unrelated (a_unrelated_expense: RP_EXPENSE_UNRELATED) is
			-- Process 'a_unrelated_expense'
		do
			process_flows (a_unrelated_expense)
		end

	process_income_adjustment (a_income_adjustment: RP_INCOME_ADJUSTMENT) is
			-- Process 'a_income_adjustment'
		do
			process_flows (a_income_adjustment)
		end

	process_expense_adjustment (a_expense_adjustment: RP_EXPENSE_ADJUSTMENT) is
			-- Process 'a_expense_adjustment'
		do
			process_flows (a_expense_adjustment)
		end

	process_income_unrelated_adjustment (a_unrelated_adjustment_income: RP_INCOME_UNRELATED_ADJUSTMENT) is
			-- Process 'a_unrelated_adjustment_income'
		do
			process_flows (a_unrelated_adjustment_income)
		end

	process_expense_unrelated_adjustment (a_unrelated_adjustment_expense: RP_EXPENSE_UNRELATED_ADJUSTMENT) is
			-- Process 'a_unrelated_adjustment_expense'
		do
			process_flows (a_unrelated_adjustment_expense)
		end

	process_interest (a_interest: RP_INTEREST) is
			-- Process 'a_interest'
		do
			process_flows (a_interest)
		end

	process_client_contribution (a_client_contribution: RP_CLIENT_CONTRIBUTION) is
			-- Process 'a_client_contribution'
		do
			process_flows (a_client_contribution)
		end

	process_cash_zonal_tranfer (a_xfer: RP_CASH_ZONAL_TRANSFER) is
			-- <Precursor>
		do
			process_flows (a_xfer)
		end

	process_client_withdrawal (a_client_withdrawal: RP_CLIENT_WITHDRAWAL) is
			-- Process 'a_client_withdrawal'
		do
			process_flows (a_client_withdrawal)
		end

	process_delivery (a_delivery: RP_DELIVERY) is
			-- Process 'a_delivery'
		do
			process_flows (a_delivery)
		end

	process_distribution (a_distribution: RP_DISTRIBUTION) is
			-- Process 'a_distribution'
		do
			process_flows (a_distribution)
		end

	process_inlieu (a_inlieu: RP_INLIEU) is
			-- Process 'a_inlieu'
		do
			if a_inlieu.is_long_side then
				process_flows (a_inlieu)
			else
				a_inlieu.flows.do_all (agent process_flow (a_inlieu, ?, True))
			end
		end

	process_split (a_split: RP_SPLIT) is
			-- Process 'a_split'
		do
			long_side_basis (a_split.portfolio, a_split.instrument).scale (a_split.factor)
			short_side_basis (a_split.portfolio, a_split.instrument).scale (a_split.factor)
		end

	process_dividend (a_dividend: RP_DIVIDEND) is
			-- Process 'a_dividend'
		do
			process_flows (a_dividend)
		end

	process_dividend_tax_refund (a_dividend_tax_refund: RP_DIVIDEND_TAX_REFUND) is
			-- Process 'a_dividend_tax_refund'
		do
			process_flows (a_dividend_tax_refund)
		end

	process_cash_swept_from_interest (a_cash_swept_from_interest: RP_CASH_SWEPT_FROM_INTEREST) is
			-- Process 'a_cash_swept_from_interest'
		do
			process_flows (a_cash_swept_from_interest)
		end

	process_cash_swept_from_income (a_cash_swept_from_income: RP_CASH_SWEPT_FROM_INCOME) is
			-- Process 'a_cash_swept_from_income'
		do
			process_flows (a_cash_swept_from_income)
		end

	process_cash_swept_from_dividend (a_cash_swept_from_dividend: RP_CASH_SWEPT_FROM_DIVIDEND) is
			-- Process 'a_cash_swept_from_dividend'
		do
			process_flows (a_cash_swept_from_dividend)
		end

	process_rights (a_rights: RP_RIGHTS) is
			-- Process 'a_rights'
		do
			process_flows (a_rights)
		end

	process_return_of_capital (a_roc: RP_RETURN_OF_CAPITAL) is
			-- Process 'a_roc'
		do
			process_flows (a_roc)
		end

	process_mark_to_market (a_mark_to_market: RP_MARK_TO_MARKET) is
			-- Process 'a_mark_to_market'
		do
			process_flows (a_mark_to_market)
		end

	process_collateral (a_collateral: RP_COLLATERAL) is
			-- Process 'a_collateral'
		do
			process_flows (a_collateral)
		end

	process_tender (a_tender: RP_TENDER) is
			-- Process 'a_tender'
		do
			process_flows (a_tender)
		end

	process_stock_bonus (a_stock_bonus: RP_STOCK_BONUS) is
			-- Process 'a_stock_bonus'
		do
			process_flows (a_stock_bonus)
		end

	process_redenomination (a_redenomination: RP_REDENOMINATION) is
			-- Process 'a_redenomination'
		do
			process_flows (a_redenomination)
		end

	process_portfolio_corporate_action_compound_contract (a_portfolio_corporate_action_compound_contract: RP_PORTFOLIO_CORPORATE_ACTION_COMPOUND_CONTRACT) is
			-- Process 'a_portfolio_corporate_action_compound_contract'
		do
			process_flows (a_portfolio_corporate_action_compound_contract)
		end

	process_delivery_distribution (a_delivery_distribution: RP_DELIVERY_DISTRIBUTION) is
			-- Process 'a_delivery_distribution'
		do
			process_flows (a_delivery_distribution)
		end

	process_fixed_deposit_transfer (a_fixed_deposit_transfer: RP_FIXED_DEPOSIT_TRANSFER) is
			-- Process 'a_fixed_deposit_transfer'
		do
			process_flows (a_fixed_deposit_transfer)
		end

	process_dividend_reinvestment (a_dividend_reinvestment: RP_DIVIDEND_REINVESTMENT) is
			-- Process 'a_dividend_reinvestment'
		do
			process_flows (a_dividend_reinvestment)
		end

	process_intraday_sell_to_adjust_gl (a_intraday_sell_to_adjust_gl: RP_INTRA_DAY_SELL_TO_ADJUST_GL) is
			-- Process 'a_intraday_sell_to_adjust_gl'
		do
			process_flows (a_intraday_sell_to_adjust_gl)
		end

	process_fx (a_fx: RP_FX) is
			-- Process 'a_fx'
		do
			process_flows (a_fx)
		end

	process_ffx_open (a_ffx_open: RP_FFX_OPEN) is
			-- Process 'a_ffx_open'
		do
			process_flows (a_ffx_open)
		end

	process_ffx_close (a_ffx_close: RP_FFX_CLOSE) is
			-- Process 'a_ffx_close'
		do
			process_flows (a_ffx_close)
		end

	process_futures_short (a_futures_short: RP_FUTURES_SHORT) is
			-- Process 'a_futures_short'
		do
			process_flows (a_futures_short)
		end

	process_futures_sell (a_futures_sell: RP_FUTURES_SELL) is
			-- Process 'a_futures_sell'
		do
			process_flows (a_futures_sell)
		end

	process_futures_cover (a_futures_cover: RP_FUTURES_COVER) is
			-- Process 'a_futures_cover'
		do
			process_flows (a_futures_cover)
		end

	process_futures_buy (a_futures_buy: RP_FUTURES_BUY) is
			-- Process 'a_futures_buy'
		do
			process_flows (a_futures_buy)
		end

	process_ffx_gain_loss (a_ffx_gain_loss: RP_FFX_GAIN_LOSS) is
			-- Process 'a_ffx_gain_loss'
		do
			process_flows (a_ffx_gain_loss)
		end

	process_bank_transfer (a_bank_transfer: RP_BANK_TRANSFER) is
			-- Process 'a_bank_transfer'
		do
			process_flows (a_bank_transfer)
		end

feature {NONE} -- Implementation

	debug_output: ROSE_TEXT_FILE
			-- Debug output file

	costs: TUPLE_KEYED_HASH_TABLE [RP_COST_BASIS, TUPLE [RP_ZONAL_INVESTMENT_GROUP, RP_INSTRUMENT, BOOLEAN]]
			-- Tax lots

	methods: DS_HASH_TABLE [RP_COST_ACCOUNTING_METHOD, RP_INVESTMENT_ACCOUNT] is
			-- Methods per portfolio
			-- FIXME: this should be configurable
		once
			create Result.make_equal (17)
		end

	fifo: RP_COST_ACCOUNTING_METHOD_FIFO is
			-- FIFO method
		once
			create Result.make_fifo
		end

	process_flows (a_contract: RP_PORTFOLIO_CONTRACT) is
			-- Process flow by flow
		require
			contract_not_void: a_contract /= Void
		do
			a_contract.flows.do_all (agent process_flow (a_contract, ?, False))
		end

	process_flow (a_contract: RP_PORTFOLIO_CONTRACT; a_flow: RP_FLOW; a_short_side: BOOLEAN) is
			-- Process 'a_flow' from 'a_contract'
		require
			contract_has_flow: a_flow.contract = a_contract
		local
			l_cost: RP_COST
			l_basis: RP_COST_BASIS
			l_sign: INTEGER
		do
			l_cost := a_flow.basis
			if debug_output /= Void then
				if l_cost.flow.flow_value /= Void then
					debug_output.put_string (l_cost.flow.contract.identifier + "|" + l_cost.flow.type.as_string + "|" + l_cost.flow.flow_value.amount.rounded (2).out + "|" + l_cost.flow.flow_value.instrument.brimsym + "|" + l_cost.quantity.amount.rounded (2).out + "|" + l_cost.quantity.instrument.brimsym + "|" + l_cost.cost.amount.rounded (2).out + "|" + l_cost.cost.currency.name + "|xxx|" + (l_cost.cost.amount.sign >= 0).out + "|xxx%N")
				else
					debug_output.put_string (l_cost.flow.contract.identifier + "|" + l_cost.flow.type.as_string + "|" + "n/a" + "|" + "n/a" + "|" + l_cost.quantity.amount.rounded (2).out + "|" + l_cost.quantity.instrument.brimsym + "|" + l_cost.cost.amount.rounded (2).out + "|" + l_cost.cost.currency.name + "|xxx|" + (l_cost.cost.amount.sign >= 0).out + "|xxx%N")
				end
			end
			if a_flow.type.is_borrow_flow or a_flow.type.is_return_flow then
				l_basis := Void
			elseif a_flow.type.is_quantity_flow and a_short_side then
				l_basis := short_side_basis (a_contract.portfolio, a_flow.instrument)
				l_sign := -a_flow.signed_quantity.sign
			elseif a_flow.type.is_delivery_flow and a_flow.signed_quantity.sign > 0 then
				l_basis := short_side_basis (a_contract.portfolio, a_flow.instrument)
				l_sign := -a_flow.signed_quantity.sign
			elseif a_flow.type.is_distribution_flow and a_flow.signed_quantity.sign < 0 then
				l_basis := short_side_basis (a_contract.portfolio, a_flow.instrument)
				l_sign := -a_flow.signed_quantity.sign
			elseif a_flow.type.is_inkind_flow and a_flow.signed_quantity.sign < 0 then
				l_basis := short_side_basis (a_contract.portfolio, a_flow.instrument)
				l_sign := -a_flow.signed_quantity.sign
			elseif a_flow.type.is_outkind_flow and a_flow.signed_quantity.sign > 0 then
				l_basis := short_side_basis (a_contract.portfolio, a_flow.instrument)
				l_sign := -a_flow.signed_quantity.sign
			else
				l_basis := long_side_basis (a_contract.portfolio, a_flow.instrument)
				l_sign := a_flow.signed_quantity.sign
			end
			if l_basis /= Void then
				if l_sign > 0 then
					l_basis.collect (l_cost)
				elseif l_sign < 0 then
					l_basis.disburse (a_flow, method (a_flow))
				else
					debug ("rp_cost_accounting_system")
						manager.log_info ("Ignoring zero quantity flow " + a_flow.signed_quantity.sign.out + " " + a_flow.debug_output)
					end
				end
				if debug_output /= Void and then debug_brimsym /= Void and then a_flow.quantity.instrument.brimsym.is_equal (debug_brimsym) then
					debug_output.put_string (l_basis.to_string)
					debug ("rp_cost_accounting_system_brimsym")
						print (l_basis.to_string)
					end
				end
			end
		end

end
