note
	description: "Currency instrument"
	author: "Mark Howard"
	last_editor: "$Author: sm $"
	date: "$Date: 2012/11/08 $"
	revision: "$Revision: #39 $"
	path: "$File: //rose/source/kernel/expert_system/rp_currency.e $"

class RP_CURRENCY

inherit

	RP_CASH_EQUIVALENT
		rename
			quantity as monies
		redefine
			monies,
			name,
			is_currency,
			uid
		end

	RP_UNITS
		undefine
			name,
			debug_output,
			is_equal
		end

create {RP_CURRENCY_CACHE}

	make

feature {NONE} -- Initialization

	make (a_currency: CURRENCY) is
			-- Initialize `Current' with 'a_currency'
		require
			non_void_currency: a_currency /= Void
		do
			currency := a_currency
			units := units_environment.currency_units (currency)
			rose_units := units
			brimsym := name.substring (1, 2) + "M..CASH........."
		ensure
			currency_set: currency = a_currency
		end

feature -- Access

	brimsym: attached STRING
			-- Brimsym

	country_of_issue (a_ref_system: RP_REFERENCE_SYSTEM; a_date: ROSE_DATE): RP_COUNTRY is
			-- Code that does not work for EUR
		do
			Result := a_ref_system.country_from_iso_code (currency.iso_code_key (Void).substring (1, 2))
		end

	iso: STRING is
			-- Iso code
		do
			Result := currency.iso
		ensure
			three_chars: (Result /= Void) and then (Result.count = 3)
		end

	currency: attached CURRENCY
			-- Underlying entity

	description: attached STRING is
			-- Long name
		do
			Result := currency.name
			if Result = Void then
				Result := currency.iso
				if Result = Void then
					create Result.make_empty
				end
			end
		ensure
			result_not_void: Result /= Void
		end

	exchange_rate (a_other: RP_CURRENCY; a_date: ROSE_DATE): RP_DECIMAL is
			-- Exchange rate of 'Current' with 'a_currency)
		obsolete
			"Use RP_CURRENCY_MARKET_DATA_SOURCE for FX"
		require
			args_not_void: a_other /= Void and a_date /= Void
		do
			Result := currency.exchange_rate (a_other.currency, a_date)
		end

	exchange_rate_at (a_other: RP_CURRENCY; a_date: ROSE_DATE): RP_DECIMAL is
			-- Exchange rate of 'Current' with 'a_currency)
		obsolete
			"Use RP_CURRENCY_MARKET_DATA_SOURCE for FX"
		require
			args_not_void: a_other /= Void and a_date /= Void
		do
			Result := currency.exchange_rate_at (a_other.currency, a_date)
		end

	holding_value_in_currency (a_holding: RP_HOLDING; a_valuator: RP_HOLDING_VALUATOR; a_currency: RP_CURRENCY; a_date: ROSE_DATE; a_bucket: STRING): RP_MONIES is
			-- Value of `a_holding' on `a_date' in `a_currency' as computed by `a_valuator'
		do
			Result := a_valuator.currency_value_in_currency (a_holding, a_currency, a_date, a_bucket)
		end

	monies (a_amount: RP_DECIMAL): attached RP_MONIES is
			-- Amount of `Current'
		do
			create Result.make (Current, a_amount)
		end

	name: STRING is
			-- Currency symbol ('iso' code)
		do
			Result := currency.iso
		end

	uid: INTEGER_64 is
			-- Unique identifier
		do
			Result := currency.uid
		end

feature -- Status Report

	is_currency: BOOLEAN is
			-- Is the object a currency?
		do
			Result := True
		end

	is_usd: BOOLEAN is
			-- Is the currency code USD?
		do
			Result := currency ~ currency.usd
		end

	is_eur: BOOLEAN is
			-- Is the currency code EUR?
		do
			Result := currency ~ currency.euro
		end

	is_gbp: BOOLEAN is
			-- Is the currency code GBP?
		do
			Result := currency ~ currency.gbp
		end

	is_jpy: BOOLEAN is
			-- Is the currency code JPY?
		do
			Result := currency ~ currency.jpy
		end

invariant

	valid_iso: (currency.iso /= Void) and then (currency.iso.count = 3)
	brimsym_matches_iso: brimsym.substring (1, 2) ~ currency.iso.substring (1, 2)

end
