note
	description: "Cache of currencies - FIXME: not dated, this is not great. We should have a data parameter in currency_from_iso_code"
	author: "Sami Ben Grine"
	last_editor: "$Author: sm $"
	date: "$Date: 2012/11/08 $"
	revision: "$Revision: #1 $"
	path: "$File: //rose/source/kernel/expert_system/rp_currency_cache.e $"

class
	RP_CURRENCY_CACHE

inherit
	ANY
	ROSE_ENTITY_FOREGROUND_AWARE
		export {NONE} all end

	ATTRIBUTES
		export {NONE} all end

	DATABASE_SERVER
		export {NONE} all end

create
	make

feature {NONE} -- Initialization

	make is
			-- Create cache
		do
			create currency_cache.make (0)
		end

feature -- Access

	currency_from_brimsym (a_currency_brimsym: STRING): detachable RP_CURRENCY is
			-- Currency from currency brimsym
		require
			valid_brimcode: (a_currency_brimsym /= Void) and then (a_currency_brimsym.count = 18)
		do
			Result := currency_from_iso_code (a_currency_brimsym.substring (1, 2))
		end

	currency_from_iso_code (a_currency_iso_code: STRING): detachable RP_CURRENCY is
			-- Currency from 3-letter ISO code or 2-letter country code
			-- FIXME: this is confusing - code can be a currency iso or a country iso - we should split?
		require
			valid_code: (a_currency_iso_code /= Void)
		local
			l_currency: CURRENCY
			l_code: STRING
		do
			create l_code.make_from_string (a_currency_iso_code)
			l_code.prune_all (' ')
			if l_code.is_empty then
				Result := Void
			else
				check valid_code: l_code.count = 2 or l_code.count = 3 end
				if currency_cache.has (l_code) then
					Result := currency_cache.item (l_code)
				else
					l_currency := context.currency_by_iso_code (l_code, Void)
					if l_currency /= Void then
						create Result.make (l_currency)
					else
							-- Try cash brimsym using code
						Result ?= cash_from_country (l_code.substring (1, 2), today)
					end
					if Result /= Void then
						currency_cache.force (Result, l_code)
					end
				end
			end
		end

	cash_from_country (a_country_code: STRING; a_date: ROSE_DATE): detachable RP_CURRENCY is
			-- Retrieve currency from BRIMSYM country
		require
			non_void_args: a_country_code /= Void
			valid_length_of_country_code: a_country_code.count = 2
		local
			l_currency: CURRENCY
			l_country: COUNTRY
			l_date: ROSE_DATE
		do
			if a_country_code.is_equal ("EU") then
				Result :=  create {RP_CURRENCY}.make (context.currency_by_iso_code ("EUR", Void))
			else
				l_country := context.country_at (brim_code (a_country_code), Void)
				if l_country = Void then
					l_country := context.country_at (iso_code (a_country_code), Void)
				end
				if l_country /= Void and then l_country.currencies /= Void then
					if l_country.currencies.count > 0 then
						if a_date = Void then
							l_date := l_country.currency_history.max_end_date
						else
							l_date := a_date
						end
						l_currency := l_country.currency_history.witness (l_date)
					end
					if l_currency /= Void then
						Result := create {RP_CURRENCY}.make (l_currency)
					end
				end
			end
		ensure
			cash_from_country_can_be_void: True
		end

	currency_from_country_brim_code (a_brim_code: STRING): detachable RP_CURRENCY is
			-- Currency from `a_brim_code' as defined in ref_w_a:country_codes.parm
		require
			non_void_code: a_brim_code /= Void
			code_long_enough: a_brim_code.count = 2
		do
			retrieve_currency_from_brim_code_cache
			currency_from_brim_code_cache.search (a_brim_code.as_upper)
			if currency_from_brim_code_cache.found then
				Result := currency_from_brim_code_cache.found_item
			end
		end

feature {NONE} -- Implementation

	currency_cache: HASH_TABLE [RP_CURRENCY, STRING]
			-- Currency by iso code cache

	currency_from_brim_code_cache: DS_HASH_TABLE [RP_CURRENCY, STRING]
			-- Currency from brimcode cache

	cash_brimsym (a_country_or_iso_ccy_code: STRING): STRING is
			-- cash brimsym from country code (or ISO currency code)
		require
			attached_code: a_country_or_iso_ccy_code /= Void
		do
			if a_country_or_iso_ccy_code.count > 1 then
				Result := a_country_or_iso_ccy_code.substring (1, 2).as_upper + "M..CASH........."
			end
		end

	retrieve_currency_from_brim_code_cache_predicate: STRING_PREDICATE is
			-- Predicate to retrieve currency from brim code cache
		once
			create Result.make (servers.performance_server_fast_load, "SELECT currency_iso_code + '|' + brim_code FROM country_codes_parm")
		end

	retrieve_currency_from_brim_code_cache is
			-- Populate `currency_from_brim_code_cache'
		do
			if currency_from_brim_code_cache = Void then
				create currency_from_brim_code_cache.make_equal (17)
				retrieve_currency_from_brim_code_cache_predicate.retrieve (Void, Void)
				retrieve_currency_from_brim_code_cache_predicate.array.do_all (agent add_currency_from_brim_code (?))
			end
		end

	add_currency_from_brim_code (s: STRING) is
			-- Parse currency and brim code from `s' and add to `currency_from_brim_code_cache'
		require
			non_void_s: s /= Void
		local
			l_currency: RP_CURRENCY
		do
			if s.count = 6 and then s.item (4) = '|' then
				l_currency := currency_from_iso_code (s.substring (1, 3))
				if l_currency /= Void then
					currency_from_brim_code_cache.force (l_currency, s.substring (5, 6))
				end
			end
		end

feature -- Constants

	today: ROSE_DATE is
			-- Today's date
		do
			create Result.make_today
		end
end
