note

	description:

		"Daily Returns"

	author: "Ivan Tarnavsky"
	last_editor: "$Author: it $"
	date: "$Date: 2011/05/19 $"
	revision: "$Revision: #4 $"
	path: "$File: //rose/source/kernel/expert_system/perf/rp_daily_return.e $"

class RP_DAILY_RETURN

create

	make

feature {NONE} -- Initialization

	make (a_date: like date; a_currency: like currency; a_value: like value; a_return: like daily_return) is
			-- Initialize `Current'
		require
			a_date_not_void: a_date /= Void
			a_currency_not_void: a_currency /= Void
			a_value_not_void: a_value /= Void
			a_return_not_void: a_return /= Void
			a_return_is_pisitive: a_return > 0
		do
			date := a_date
			currency := a_currency
			value := a_value
			daily_return := a_return
			return := 1
			cumrel := 1
		ensure
			date_set: date = a_date
			currency_set: currency = a_currency
			value_set: value = a_value
			daily_return_set: daily_return = a_return
		end

feature -- Access

	date: ROSE_DATE
			-- Date

	currency: RP_CURRENCY
			-- Currency

	value: RP_DECIMAL assign set_value
			-- Portfolio value

	return: RP_DECIMAL
			-- Return

	cumrel: RP_DECIMAL
			-- Cumulative relative

	prev: like Current
			-- Previous node

	next: like Current
			-- Next node

feature -- Change

	set_next (a_next: like next) is
			-- Set `next'
		local
			l_item: like Current
		do
			next := a_next
			from
				l_item := next
			until
				l_item = Void
			loop
				l_item.update_cumrel
				l_item := l_item.next
			end
		ensure
			next_set: next = a_next
		end

	set_prev (a_prev: like prev) is
			-- Set `prev'
		do
			prev := a_prev
			if a_prev /= Void then
				update_return
				update_cumrel
			end
		ensure
			prev_set: prev = a_prev
		end

	set_value (a_value: like value) is
			-- Set `value'
		do
			value := a_value
		ensure
			value_set: value = a_value
		end

feature {RP_DAILY_RETURN} -- Implementation

	daily_return: RP_DECIMAL
			-- Daily return

	update_return is
			-- Update value of `return'
		do
			return := daily_return
		end

	update_cumrel is
			-- Update value of `cumrel'
		do
			if prev = Void then
				cumrel := 1.0
			else
				cumrel := (prev.cumrel * return)
			end
		end

invariant

	daily_return_not_void: daily_return /= Void
	cumrel_not_void: cumrel /= Void
	daily_return_is_positive: daily_return > 0
	cumrel_is_positive: cumrel > 0

end
