indexing

	description:

		"Doubly-linked list of daily returns"

	date: "$Date: 2008/12/01 $"
	revision: "$Revision: #2 $"

class RP_DAILY_RETURNS

create

	make

feature {NONE} -- Initialization

	make (a_code: STRING) is
			-- Initialize `Current'
		require
			non_void_args: a_code /= Void
		do
			create returns_cache.make_default
			code := a_code
		end

feature -- Access

	code: STRING
			-- Code of underlying entity

feature -- Query

	returns: ARRAY [RP_DAILY_RETURN] is
			-- Returns
		do
			Result := returns_cache.to_array
		ensure
			non_void_result: Result /= Void
		end

	return (a_date: ROSE_DATE): RP_DAILY_RETURN is
			-- Dailyreturn at `a_date'
		require
			valid_existing_date: valid_existing_date (a_date)
		do
			Result := returns_cache.item (a_date)
		ensure
			non_void_result: Result /= Void
		end

	dates: ARRAY [ROSE_DATE] is
			-- Dailyreturn dates
		do
			Result := returns_cache.keys.to_array
			sort.sort (Result)
		ensure
			non_void_result: Result /= Void
		end

feature -- Basic operations

	expand (a_dailyreturn: RP_DAILY_RETURN) is
			-- Add `a_dailyvalue' and compute dailyretun
		require
			valid_new_date: valid_new_date (a_dailyreturn.date)
		local
			l_item: like tail
		do
			if tail = Void or else tail.date < a_dailyreturn.date then
				a_dailyreturn.set_prev (tail)
				if tail /= Void then
					tail.set_next (a_dailyreturn)
				end
				tail := a_dailyreturn
			else
				from
					l_item := tail
				until
					l_item.prev = Void or else l_item.prev.date < a_dailyreturn.date and a_dailyreturn.date < l_item.date
				loop
					l_item := l_item.prev
				end
				a_dailyreturn.set_prev (l_item.prev)
				if l_item.prev /= Void then
					l_item.prev.set_next (a_dailyreturn)
				end
				l_item.set_prev (a_dailyreturn)
				a_dailyreturn.set_next (l_item)
			end
			returns_cache.force_new (a_dailyreturn, a_dailyreturn.date)
		end

feature -- Status

	valid_new_date (a_date: ROSE_DATE): BOOLEAN is
			-- Is `a_date' valid new date?
		do
			Result := not valid_existing_date (a_date)
		end

	valid_existing_date (a_date: ROSE_DATE): BOOLEAN is
			-- If `a_date' valid date?
		do
			Result := returns_cache.has (a_date)
		end

feature {RP_PERFORMANCE_DATE_INTERVAL_RETURNS} -- Access

	tail: RP_DAILY_RETURN
			-- Last dailyreturn

feature {NONE} -- Implementation

	returns_cache: DS_HASH_TABLE [RP_DAILY_RETURN, ROSE_DATE]
			-- Cache for dailyreturns by date

	sort: DS_ARRAY_SHELL_SORTER [ROSE_DATE] is
			-- Sorter
		once
			create Result.make (create {KL_COMPARABLE_COMPARATOR [ROSE_DATE]}.make)
		end

end
