indexing
	description: "Common ancestor to allow feature access to all RP_DATA_GRID-related CLASSES"
	author: "Julian Rogers"
	date: "$Date: 2014/02/26 $"
	revision: "$Revision: #8 $"
	archive: "$File: //rose/source/kernel/infrastructure/widgets/grid/rp_data_grid_access.e $"

class
	RP_DATA_GRID_ACCESS

inherit
	ROSE_SHARED_GLOBAL_GUI

	REFACTORING_HELPER
		export
			{NONE} all
		end

feature -- Conversion

	any_to_grid_item (a_any: ANY): RP_DATA_GRID_ITEM is
			-- Convert `a_any' into a data grid item.
			-- A number of object types have specialized conversion
			-- routines, all others use `out' on `a_any'.
		local
			l_string : STRING_GENERAL
			l_date : ROSE_DATE
			l_colored_string : TUPLE[STRING,EV_COLOR]
			l_full_colored_string : TUPLE[STRING,EV_COLOR,EV_COLOR]
			l_data_string: TUPLE [STRING, ANY]
			l_item : RP_DATA_GRID_LABEL_ITEM
			l_color : EV_COLOR
			l_background : EV_COLOR
			l_ev_item : EV_GRID_ITEM
			l_ev_label : EV_GRID_LABEL_ITEM
			l_ev_combo_item : EV_GRID_COMBO_ITEM
			l_ev_editable : EV_GRID_EDITABLE_ITEM
			l_pixmap : EV_PIXMAP
		do
			l_ev_item ?= a_any
			l_string ?= a_any
			l_colored_string ?= a_any
			l_full_colored_string ?= a_any
			l_pixmap ?= a_any
			l_date ?= a_any
			l_data_string ?= a_any
			Result ?= a_any
			if Result /= void then
			elseif l_ev_item /= Void then
				l_ev_label ?= l_ev_item
				l_ev_combo_item ?= l_ev_item
				l_ev_editable ?= l_ev_item
				if l_ev_label /= Void then
					Result := create {RP_DATA_GRID_LABEL_ITEM}.make_from_ev_grid_item (l_ev_label)
				elseif l_ev_combo_item /= Void then
					Result := create {RP_DATA_GRID_COMBO_ITEM}.make_from_ev_grid_item (l_ev_combo_item)
				elseif l_ev_editable /= Void then
					Result := create {RP_DATA_GRID_EDITABLE_ITEM}.make_from_ev_grid_item (l_ev_editable)
				end
			elseif l_string /= void then
				create l_item.make_with_text(l_string)
				Result := l_item
			elseif l_full_colored_string /= void then
				l_string ?= l_colored_string.item(1)
				l_color ?= l_colored_string.item(2)
				l_background ?= l_colored_string.item(3)
				create l_item.make_with_text(l_string)
				l_item.set_foreground_color(l_color)
				l_item.set_background_color(l_background)
				Result := l_item
			elseif l_data_string /= Void then
				l_string ?= l_data_string.item(1)
				create l_item.make_with_text(l_string)
				l_item.set_data (l_data_string.item (2))
				Result := l_item
			elseif l_colored_string /= void then
				l_string ?= l_colored_string.item(1)
				l_color ?= l_colored_string.item(2)
				create l_item.make_with_text(l_string)
				l_item.set_foreground_color(l_color)
				Result := l_item
			elseif l_pixmap /= void then
				create l_item
				l_item.set_pixmap(l_pixmap)
				Result := l_item
			elseif l_date /= void then
				create l_item.make_with_text(l_date.ccyymmdd.out)
				Result := l_item
			else
				if a_any = void then
					create l_item.make_with_text("")
				else
					create l_item.make_with_text(a_any.out)
				end
				Result := l_item
			end
			if l_data_string = Void then
					-- If we passed a data string, then we must not
					-- overwrite the data.
				Result.set_data (a_any)
			end
		ensure
			result_not_void: Result /= Void
		end

end
