note

	description:

		"RP_DATA_GRID_WRAPPING of EV_GRID_COLUMN"

	author: "Julian Rogers"
	last_editor: "$Author: jq $"
	date: "$Date: 2012/08/31 $"
	revision: "$Revision: #24 $"
	path: "$File: //rose/source/kernel/infrastructure/widgets/grid/rp_data_grid_column.e $"

class
	RP_DATA_GRID_COLUMN [G]

inherit
	ANY
		redefine
			default_create
		end

feature {NONE} -- Initialization

	default_create is
			-- Default creation procedure
		do
			is_user_resizeable := True
		end

feature -- Access

	data: ANY
		-- Arbitrary user data may be stored here.

	index: INTEGER is
			-- Position of Current in `parent'.
		require
			is_parented: parent /= Void
		do
			Result := column.index
		ensure
			index_positive: Result > 0
			index_less_than_column_count: Result <= parent.column_count
		end

	parent: RP_DATA_GRID [G]
			-- Grid to which current column belongs.

	width: INTEGER is
			-- `Result' is width of `Current'.
		require
			is_parented: parent /= Void
		do
			Result := column.width
		ensure
			Result_non_negative: Result >= 0
		end

	is_user_resizeable: BOOLEAN
			-- May `Current' be resized directly by a user via the mouse?

	is_locked: BOOLEAN is
			-- Is `Current' locked so that it no longer scrolls?
		require
			is_parented: parent /= Void
		do
			Result := column.is_locked
		end

	locked_position: INTEGER is
			-- Locked position of `Current' from left edge of viewable area of `parent'.
			-- `Result' is 0 if not `is_locked'.
		require
			is_parented: parent /= Void
		do
			Result := column.locked_position
		ensure
			not_locked_implies_result_zero: not is_locked implies result = 0
		end

	is_selected: BOOLEAN is
			-- Is selected?
		do
			if is_selectable then
				Result := column.is_selected
			end
		end

	is_selectable: BOOLEAN is
			-- May the object be selected?
		do
			Result := parent /= Void
		end

	i_th (i: INTEGER): RP_DATA_GRID_ITEM is
			-- Item at `i'-th row, Void if none.
		require
			i_positive: i > 0
			i_less_than_count: i <= count
			is_parented: parent /= Void
		do
			Result := parent.row (i).i_th (index)
		end

	count: INTEGER is
			-- Number of items in current.
		require
			is_parented: parent /= Void
		do
			Result := column.count
		ensure
			count_not_negative: count >= 0
		end

	virtual_x_position: INTEGER is
			-- Horizontal offset of `Current' in relation to the
			-- the virtual area of `parent' grid in pixels.
			-- `Result' is 0 if `parent' is `Void'.
		require
			is_parented: parent /= Void
		do
			Result := column.virtual_x_position
		ensure
			parent_void_implies_result_zero: parent = Void implies Result = 0
		end

	virtual_x_position_unlocked: INTEGER is
			-- Horizontal offset of unlocked position of `Current', in relation to the
			-- virtual area of `parent' grid in pixels.
			-- If not `is_locked', then `virtual_x_position' = `virtual_y_position_unlocked'.
			-- If `is_locked' then `Result' is the "shadow" position where the column would
			-- be if not locked.
		require
			is_parented: parent /= Void
		do
			Result := column.virtual_x_position_unlocked
		ensure
			parent_void_implies_result_zero: parent = Void implies Result = 0
		end

	is_show_requested: BOOLEAN is
			-- Will `Current' be displayed when its parent is?
		require
			is_parented: parent /= Void
		do
			Result := column.is_show_requested
		end

	is_displayed: BOOLEAN is
			-- May `Current' be displayed when its `parent' is?
			-- Will return False if `hide' has been called on `Current'.
			-- A column that `is_displayed' does not necessarily have to be visible on screen at that particular time.
			-- If `parent' is not displayed then this returns False.
		require
			is_parented: parent /= Void
		do
			Result := column.is_displayed
		end

	background_color: EV_COLOR is
			-- Color displayed as background of `Current' except where there are items contained that
			-- have a non-`Void' `background_color'. If `Void', `background_color' of `parent' is displayed.
			-- See header of `EV_GRID' for a description of this behavior.
		require
			is_parented: parent /= Void
		do
			Result := column.background_color
		end

	foreground_color: EV_COLOR is
			-- Color displayed for foreground features of `Current' except where there are items contained that
			-- have a non-`Void' `foreground_color'. If `Void', `foreground_color' of `parent' is displayed.
			-- See header of `EV_GRID' for a description of this behavior.
		require
			is_parented: parent /= Void
		do
			Result := column.foreground_color
		end

	required_width_of_item_span (start_row, end_row: INTEGER; include_header: BOOLEAN): INTEGER is
			-- Result is greatest `required_width' of all items from
			-- row index `start_row', `end_row'. If `include_header', then incldue
			-- the required width of the `grid.header_node' title item.
			-- Note that passing 0 and 0 as `start_row' and `endrow' respectively enables you
			-- to query the width of the header only.
		require
			parented: parent /= Void
			valid_rows: start_row >= 0 and end_row <= parent.row_count and start_row <= end_row
		local
			item_counter: INTEGER
			parent_row_count: INTEGER
			grid_item_i: RP_DATA_GRID_ITEM
			l_row: RP_GRID_NODE [G]
			l_title_row: RP_GRID_TITLE_NODE [G]
			l_width: INTEGER
			l_sort, l_lock, l_select: BOOLEAN
		do
				-- Note that we do not simply wrap this feature directly from `column'
				-- as we must force the items to be created if they have not already been
				-- shown in the grid.
			from
				item_counter := start_row.max (1)
				parent_row_count := parent.row_count
			until
				item_counter > end_row
			loop
				l_row := parent.row (item_counter)
				if l_row.is_show_requested then
					grid_item_i := l_row.i_th (index)
					l_width := grid_item_i.required_width + grid_item_i.horizontal_indent
					l_title_row ?= l_row
					if l_title_row /= Void then
						-- We now need to calculate the width of the displayed icons.
						if parent.is_title_grid then
							l_sort := parent.parent_grid.are_sort_icons_displayed_on_header_node
							l_lock := parent.parent_grid.are_lock_icons_displayed_on_header_node
							l_select := parent.parent_grid.are_selection_icons_displayed_on_header_node
						else
							l_sort := parent.are_sort_icons_displayed
							l_select := parent.are_selection_icons_displayed
						end
						if l_sort then
							l_width := l_width + l_title_row.unsorted_pixmap.width + parent.icon_spacing
						end
						if l_lock then
							l_width := l_width + parent.locked_pixmap.width + parent.icon_spacing
						end
						if l_select then
							l_width := l_width + parent.selection_pixmap.width + parent.icon_spacing
						end
					end
					if grid_item_i /= Void then
						Result := Result.max (l_width)
					end
					if l_row.subrow_count > 0 and not l_row.is_expanded then
						item_counter := item_counter + l_row.subrow_count_recursive
					end
				end
				item_counter := item_counter + 1
			end
			if include_header and not parent.is_title_grid then
				Result := Result.max (parent.title_grid.column (index).required_width_of_item_span (1, parent.title_grid.row_count, False))
			end
		ensure
			result_non_negative: Result >= 0
		end

	required_width_of_visible_items (include_header: BOOLEAN): INTEGER is
			-- Result is greatest `required_width' of all currently visible items in `parent'.
			-- If `include_header', then incldue the required width of the `grid.header_node' title item.
		require
			parented: parent /= Void
			parent_displayed: parent.is_displayed
		local
			l_first_visible_row, l_last_visible_row: RP_GRID_NODE [G]
		do
			l_first_visible_row := parent.first_visible_row
			l_last_visible_row := parent.last_visible_row
			if l_first_visible_row /= Void and then l_last_visible_row /= Void then
				Result := required_width_of_item_span (l_first_visible_row.index, l_last_visible_row.index, include_header)
			end
			if include_header and not parent.is_title_grid then
				Result := Result.max (parent.title_grid.column (index).required_width_of_item_span (1, parent.title_grid.row_count, False))
			end
		ensure
			result_non_negative: Result >= 0
		end

	text_alignment: INTEGER
			-- Current alignment.
			-- See class EV_TEXT_ALIGNABLE_CONSTANTS for possible values.
			-- By default, `text_alignment' is 0 to indicate that the alignment of any items contained in the
			-- column is respected and will not be changed.

			-- It is normally the responsibility of the grid item to handle it's own alignment,
			-- but the provision of these features for alignment ensure the alignment of the items is
			-- changed to match that of the column as they are inserted into the grid.
			-- If you call one of the three alignment features in
			-- this class then the alignment of all RP_DATA_GRID_LABEL items contained is set to match the new alignment
			-- and all subsequently inserted items have their alignment set to match upon insertion.
			-- This alignment has no effect on items that do not conform to RP_DATA_GRID_LABEL_ITEM.

	is_left_aligned: BOOLEAN is
			-- Is `Current' left aligned?
		do
			Result := text_alignment = {EV_TEXT_ALIGNMENT_CONSTANTS}.Ev_text_alignment_left
		end

	is_center_aligned: BOOLEAN is
			-- Is `Current' center aligned?
		do
			Result := text_alignment = {EV_TEXT_ALIGNMENT_CONSTANTS}.Ev_text_alignment_center
		end

	is_right_aligned: BOOLEAN is
			-- Is `Current' right aligned?
		do
			Result := text_alignment = {EV_TEXT_ALIGNMENT_CONSTANTS}.Ev_text_alignment_right
		end

feature -- Basic operations

	set_data (some_data: like data) is
			-- Assign `some_data' to `data'.
		do
			data := some_data
		ensure
			data_assigned: data = some_data
		end

	enable_select is
			-- Make `is_selected' True.
		require
			is_selectable: is_selectable
		do
			column.enable_select
		ensure
			is_selected: is_selected
		end

	disable_select is
			-- Deselect the object.
		do
			column.disable_select
		ensure
			unselected: not is_selected
		end

	lock is
			-- Ensure `is_locked' is `True'.
			-- `Current' is locked at it's current horizontal offset from
			-- the left edge of the viewable area of `parent'.
		require
			is_parented: parent /= Void
		do
			lock_at_position (virtual_x_position - parent.virtual_x_position)
			if not parent.is_title_grid then
				parent.title_grid.column (index).lock
			end
		ensure
			is_locked: is_locked
			locked_position_set: locked_position = virtual_x_position - parent.virtual_x_position
		end

	lock_at_position (a_position: INTEGER) is
			-- Ensure `is_locked' is `True' with the horizontal offset from
			-- the left edge of the viewable area of `parent' set to `a_position'.
		require
			is_parented: parent /= Void
		do
			column.lock_at_position (a_position)
			if not parent.is_title_grid then
				parent.title_grid.column (index).lock_at_position (a_position)
			end
			lock_actions.call (Void)
			parent.column_lock_actions.call ([index])
			parent.reset_widgets_contained
			parent.redraw
		ensure
			is_locked: is_locked
			locked_position_set: locked_position = a_position
		end

	unlock is
			-- Ensure `is_locked' is `False'.
		require
			is_parented: parent /= Void
		do
			if is_locked then
				column.unlock
				if not parent.is_title_grid then
					parent.title_grid.column (index).unlock
				end
				unlock_actions.call (Void)
				parent.column_unlock_actions.call ([index])
				parent.reset_widgets_contained
				parent.redraw
			end
		ensure
			not_is_locked: not is_locked
		end

	set_width (a_width: INTEGER) is
			-- Assign `a_width' to `width'.
		require
			width_non_negative: a_width >= 0
			is_parented: parent /= Void
		local
			l_width: INTEGER
		do
			l_width := column.width
			if a_width /= l_width then
				column.set_width (a_width)
				if resize_actions_internal /= Void then
					resize_actions.call ([a_width])
				end
				if parent.column_resize_actions_internal /= Void then
					parent.column_resize_actions.call ([Current, a_width])
				end
			end
			if not parent.is_title_grid and then not parent.is_synchronizing_columns.item then
				parent.synchronize_columns (Current)
			end
		ensure
			width_set: width = a_width
		end

	resize_to_required_width_of_item_span (start_row, end_row: INTEGER; include_header: BOOLEAN) is
			-- Set width to `required_width_of_item_span'.
			-- (see this routine for parameter definition)
		require
			parented: parent /= Void
			valid_rows: start_row >= 0 and end_row <= parent.row_count and start_row <= end_row
		do
			set_width (required_width_of_item_span (start_row, end_row, include_header))
		ensure
			definition: width >= required_width_of_item_span (start_row, end_row, include_header)
		end

	resize_to_content (include_header: BOOLEAN) is
			-- Resize `Current' to required width of content.
			-- If `include_header' then include required width of header.
		require
			is_displayed: is_displayed
			is_parented: parent /= Void
		do
			set_width (required_width_of_visible_items (include_header))
		end

	ensure_visible is
			-- Ensure `Current' is visible in viewable area of `parent'.
		require
			parented: parent /= Void
			shown: is_displayed
		do
			column.ensure_visible
		ensure
			parent_virtual_y_position_unchanged: old parent.virtual_y_position = parent.virtual_y_position
			--column_visible: virtual_x_position >= parent.virtual_x_position and virtual_x_position + width <= parent.virtual_x_position + (parent.viewable_width).max (width)
		end

	set_background_color (a_color: EV_COLOR) is
			-- Set `background_color' with `a_color'.
		require
			is_parented: parent /= Void
		do
			column.set_background_color (a_color)
		ensure
			background_color_set: background_color = a_color
		end

	set_foreground_color (a_color: EV_COLOR) is
			-- Set `foreground_color' with `a_color'.
		require
			is_parented: parent /= Void
		do
			column.set_foreground_color (a_color)
		ensure
			foreground_color_set: foreground_color = a_color
		end

	enable_user_resizing is
			-- Ensure `is_user_resizeable' is `True'.
		do
			is_user_resizeable := True
		ensure
			is_user_resizeable: is_user_resizeable
		end

	disable_user_resizing is
			-- Ensure `is_user_resizeable' is `False'.
		do
			is_user_resizeable := False
		ensure
			not_is_user_resizeable: not is_user_resizeable
		end

	align_text_center is
			-- Display `text' of all contained RP_DATA_GRID_LABEL items center aligned.
		local
			l_label_item: RP_DATA_GRID_LABEL_ITEM
			i: INTEGER
		do
			text_alignment := {EV_TEXT_ALIGNMENT_CONSTANTS}.Ev_text_alignment_center
			from
				i := 1
			until
				i > parent.row_count
			loop
				l_label_item ?= parent.row (i).i_th (index)
				if l_label_item /= Void then
					l_label_item.align_text_center
				end
				i := i + 1
			end
		ensure
			alignment_set: is_center_aligned
		end

	align_text_right is
			-- Display `text' of all contained RP_DATA_GRID_LABEL items right aligned.
		local
			l_label_item: RP_DATA_GRID_LABEL_ITEM
			i: INTEGER
		do
			text_alignment := {EV_TEXT_ALIGNMENT_CONSTANTS}.Ev_text_alignment_right
			from
				i := 1
			until
				i > parent.row_count
			loop
				l_label_item ?= parent.row (i).i_th (index)
				if l_label_item /= Void then
					l_label_item.align_text_right
				end
				i := i + 1
			end
		ensure
			alignment_set: is_right_aligned
		end

	align_text_left is
			-- Display `text' of all contained RP_DATA_GRID_LABEL items left aligned.
		local
			l_label_item: RP_DATA_GRID_LABEL_ITEM
			i: INTEGER
		do
			text_alignment := {EV_TEXT_ALIGNMENT_CONSTANTS}.Ev_text_alignment_left
			from
				i := 1
			until
				i > parent.row_count
			loop
				l_label_item ?= parent.row (i).i_th (index)
				if l_label_item /= Void then
					l_label_item.align_text_left
				end
				i := i + 1
			end
		ensure
			alignment_set: is_left_aligned
		end

feature -- Iteration

	do_all (action: PROCEDURE [ANY, TUPLE [RP_DATA_GRID_ITEM]]) is
			-- Apply `action' to item in `Current'.
		require
			action_not_void: action /= Void
		local
			t: TUPLE [RP_DATA_GRID_ITEM]
			i: INTEGER
		do
			from
				create t
				i := 1
			until
				i > count
			loop
				t.put (i_th (i), 1)
				action.call (t)
				i := i + 1
			end
		end

	do_if (action: PROCEDURE [ANY, TUPLE [RP_DATA_GRID_ITEM]]; test: FUNCTION [ANY, TUPLE [RP_DATA_GRID_ITEM], BOOLEAN]) is
			-- Apply `action' to every item that satisfies `test'.
		require
			action_not_void: action /= Void
			test_not_void: test /= Void
		local
			t: TUPLE [RP_DATA_GRID_ITEM]
			i: INTEGER
		do
			from
				create t
				i := 1
			until
				i > count
			loop
				t.put (i_th (i), 1)
				if test.item (t) then
					action.call (t)
				end
				i := i + 1
			end
		end

	there_exists (test: FUNCTION [ANY, TUPLE [RP_DATA_GRID_ITEM], BOOLEAN]): BOOLEAN is
			-- Is `test' true for at least one item?
		require
			test_not_void: test /= Void
		local
			t: TUPLE [RP_DATA_GRID_ITEM]
			i: INTEGER
		do
			from
				create t
				i := 1
			until
				i > count or Result
			loop
				t.put (i_th (i), 1)
				Result := test.item (t)
				i := i + 1
			end
		end

	for_all (test: FUNCTION [ANY, TUPLE [RP_DATA_GRID_ITEM], BOOLEAN]): BOOLEAN is
			-- Is `test' true for all items?
		require
			test_not_void: test /= Void
		local
			t: TUPLE [RP_DATA_GRID_ITEM]
			i: INTEGER
		do
			from
				create t
				i := 1
				Result := True
			until
				i > count or not Result
			loop
				t.put (i_th (i), 1)
				Result := test.item (t)
				i := i + 1
			end
		end

feature -- Event handling

	select_actions: EV_NOTIFY_ACTION_SEQUENCE is
			-- Actions to be performed when `Current' is selected.
		do
			Result := column.select_actions
		ensure
			result_not_void: Result /= Void
		end

	deselect_actions: EV_NOTIFY_ACTION_SEQUENCE is
			-- Actions to be performed when `Current' is deselected.
		do
			Result := column.deselect_actions
		ensure
			result_not_void: Result /= Void
		end

	resize_actions: EV_INTEGER_ACTION_SEQUENCE is
			-- Actions to be performed when `Current' is resized.
			-- Event data is new width of `Current' in pixels.
		do
			if resize_actions_internal = Void then
				create resize_actions_internal
			end
			Result := resize_actions_internal
		end

	lock_actions: EV_NOTIFY_ACTION_SEQUENCE is
			-- Actions to be performed when `Current' is locked.
		do
			if lock_actions_internal = Void then
				create lock_actions_internal
			end
			Result := lock_actions_internal
		end

	unlock_actions: EV_NOTIFY_ACTION_SEQUENCE is
			-- Actions to be performed when `Current' is locked.
		do
			if unlock_actions_internal = Void then
				create unlock_actions_internal
			end
			Result := unlock_actions_internal
		end

feature {RP_DATA_GRID} -- Implementation

	column: EV_GRID_COLUMN
		-- Internal column with which `Current' is implemented.

	set_column (a_column: EV_GRID_COLUMN) is
			-- Assign `a_column' to `column'.
		do
			column := a_column
		ensure
			column_set: column = a_column
		end

	set_parent (a_parent: RP_DATA_GRID [G]) is
			-- Assign `a_parent' to `parent'.
		do
			parent := a_parent
		ensure
			parent_set: parent = a_parent
		end

	resize_actions_internal: EV_INTEGER_ACTION_SEQUENCE
			-- Internal actions to be performed when `Current' is resized.

	lock_actions_internal: EV_NOTIFY_ACTION_SEQUENCE
			-- Internal actions to be performed when `Current' is locked.

	unlock_actions_internal: EV_NOTIFY_ACTION_SEQUENCE
			-- Internal actions to be performed when `Current' is locked.

end
