note

	description:

		"Objects that represent editable data grid items."

	author: "Julian Rogers"
	last_editor: "$Author: er $"
	date: "$Date: 2013/04/24 $"
	revision: "$Revision: #22 $"
	path: "$File: //rose/source/kernel/infrastructure/widgets/grid/rp_data_grid_editable_item.e $"

class
	RP_DATA_GRID_EDITABLE_ITEM
inherit
	RP_DATA_GRID_LABEL_ITEM
		undefine
			activate_action,
			deactivate,
			is_tab_navigatable
		redefine
			initialize,
			implementation,
			make_from_ev_grid_item,
			is_editable,
			make_with_text,
			activate
		end
	EV_GRID_EDITABLE_ITEM
		undefine
			initialize,
			is_in_default_state
		redefine
			initialize,
			implementation,
			make_with_text,
			activate,
			deactivate
		end
create
	make_from_ev_grid_item,
	make_with_text,
	default_create

feature {NONE} -- Initialization

	initialize is
			-- Initialize `Current'.
		do
			Precursor {EV_GRID_EDITABLE_ITEM}
			create return_actions
			set_right_border (2)
			set_editable_background_color (default_editable_background_color)
		end

feature --Access

	make_with_text (a_text: STRING) is
			-- Make with text `a_text'.
		do
			default_create
			set_text (a_text)
		end

	make_from_ev_grid_item (a_item : EV_GRID_ITEM) is
		local
			l_editable : EV_GRID_EDITABLE_ITEM
		do
			default_create
			l_editable ?= a_item
			if l_editable /= Void then
				if l_editable.text /= Void then
					set_text (l_editable.text)
				end
			end
		end

feature -- Access

	enable_combo (a_strings: like combo_strings) is
			-- Enable combo mode with entries `a_strings'.
			-- `Void' disables the combo mode.
		do
			combo_strings := a_strings
		end

	is_combo_enabled: BOOLEAN is
			-- Is `Current' behaving in combo mode?
		do
			Result := combo_strings /= Void
		end

	combo_strings: ROSE_LINEAR_ARRAY [STRING]

	is_editable: BOOLEAN is True
			-- Is this item editable?

	return_actions : EV_NOTIFY_ACTION_SEQUENCE

	editable_background_color : EV_COLOR
			-- Background color of editable popup

	activation_veto_action: FUNCTION [ANY, TUPLE [], BOOLEAN]
		-- Function to enable vetoing of activation.

feature -- Basic Operations

	activate is
			-- Activate `Current' if not vetoed.
		local
			l_vetoed: BOOLEAN
		do
			if activation_veto_action /= Void then
				l_vetoed := activation_veto_action.item (Void)
			end
			if not l_vetoed then
				Precursor {EV_GRID_EDITABLE_ITEM}
			end
		end

	set_editable_background_color (a_color : EV_COLOR) is
			-- Set `editable_background_color' to `a_color'
		do
			editable_background_color := a_color
		end

	set_activation_veto_action (a_activation_veto_action: like activation_veto_action) is
			-- Assign `a_activation_veto_action' to `activation_veto_action'.
		require
			a_activation_veto_action_not_void: a_activation_veto_action /= Void
		do
			activation_veto_action := a_activation_veto_action
		end

	deactivate
			-- Cleanup from previous call to activate.
			-- Redefined to work around bug where calling `destroy' on the text field was
			-- causing problems with the object being destroyed and then still receiving a windows notification.
		do
			if text_field /= Void then
				text_field.focus_out_actions.wipe_out
				if not user_cancelled_activation and then (validation_agent = Void or else validation_agent.item ([text_field.text])) then
					set_text (text_field.text)
				end
			end
			implementation.deactivate
			if text_field /= Void then
				text_field := Void
			end
		end

feature {RP_DATA_GRID} -- Implementation

	respond_to_activation (a_popup_window: EV_POPUP_WINDOW) is
			-- `Current' has been activated and `a_popup_window' displayed.
		require
			a_popup_window_not_void: a_popup_window /= Void
		local
			l_field : EV_TEXT_FIELD
			l_combo: EV_COMBO_BOX
		do
			if is_combo_enabled then
				create l_combo.make_with_strings (combo_strings)
				a_popup_window.wipe_out
				a_popup_window.extend (l_combo)
				l_field := l_combo
				text_field := l_combo
				l_combo.select_actions.block
				l_combo.set_text (text)
				l_combo.select_actions.resume
				from
					return_actions.start
				until
					return_actions.off
				loop
					l_combo.select_actions.extend (return_actions.item)
					return_actions.forth
				end
				l_combo.select_actions.extend (agent deactivate)
			else
				l_field ?= a_popup_window.item
				check
					popup_window_contained_field: l_field /= Void
				end
			end
			l_field.key_press_actions.extend (agent key_press_to_grid (?))
			from return_actions.start until return_actions.off loop
				l_field.return_actions.extend (return_actions.item)
				return_actions.forth
			end
			l_field.set_background_color (editable_background_color)
			if l_field.text.count > 0 then
				l_field.select_all
			end
		end

	key_press_to_grid (a_key : EV_KEY) is
			-- Propagate `a_key' to `grid' for handling.
		require
			a_key_not_void: a_key /= Void
		local
			l_row : RP_GRID_NODE [ANY]
		do
			if is_parented then
				l_row ?= row.data
				check
					l_row_not_void: l_row /= Void
				end
				l_row.grid.handle_key_press (a_key)
			end
		end

feature {EV_ANY, EV_ANY_I} -- Implementation

	default_editable_background_color : EV_COLOR is
			-- Default background color for editable popup
		once
			create Result.make_with_8_bit_rgb (222,255,222)
		end

	implementation: EV_GRID_LABEL_ITEM_I

end
