note

	description:

		"Items that may be contained in an RP_DATA_GRID."

	author: "Julian Rogers"
	last_editor: "$Author: c4 $"
	date: "$Date: 2013/04/04 $"
	revision: "$Revision: #18 $"
	path: "$File: //rose/source/kernel/infrastructure/widgets/grid/rp_data_grid_item.e $"

deferred class
	RP_DATA_GRID_ITEM

inherit
	EV_GRID_ITEM
		undefine
			is_in_default_state,
			initialize,
			activate_action,
			deactivate,
			create_implementation
		end
	RP_DATA_GRID_ACCESS
		undefine
			default_create, copy, is_equal
		end

feature {NONE} -- Initialization

	make_from_ev_grid_item (a_item: EV_GRID_ITEM) is
			-- Create `Current' from EV_GRID item `a_item', copying
			-- relevent attributes where possible.
		require
			a_item_not_void: a_item /= Void
		deferred
		end

feature -- Access

	node: RP_GRID_NODE [ANY] is
			-- Row to which `Current' belongs or Void if None.
		do
			if is_parented then
				Result ?= row.data
			end
		end

	grid: RP_DATA_GRID [ANY] is
			-- `Parent' grid of `Current' or Void if None.
		local
			l_node: RP_GRID_NODE [ANY]
		do
			l_node := node
			if l_node /= Void then
				Result := l_node.grid
			end
		end

	comparable: COMPARABLE is
			-- Comparable object used in sorting routines when non-Void.
			-- If `Void', the standard comparison is used for `Current'.
			-- Use in cases where the displayed data does not necessarily match the
			-- sort order.
			-- Returns `data' if set for backwards compatibility.
		do
			Result := internal_comparable
			if Result = Void then
				Result ?= data
			end
		end

	pebble_agent: FUNCTION[ANY,TUPLE[RP_DATA_GRID_ITEM],ANY]

	is_editable: BOOLEAN is
			-- Is this item editable?
		deferred
		end

	index_in_row: INTEGER is
			-- Index of current in `node'.
		local
			l_node: RP_GRID_NODE [ANY]
			i: INTEGER
			l_found : BOOLEAN
		do
			l_node ?= row.data
			from i := 0 until i > l_node.column_count or l_found loop
				i := i + 1
				l_found := l_node.i_th (i) = Current
				if i > l_node.column_count  then
					Result := 0
				else
					Result := i
				end
			end
		end

	displayed_background_color: EV_COLOR is
			-- `Result' is the background color in which `Current'
			-- should be displayed. Based on the `Void' status of
			-- `background_color', `row.background_color' and `parent.background_color'.
		do
			Result := implementation.displayed_background_color
		ensure
			result_not_void: Result /= Void
		end

	displayed_foreground_color: EV_COLOR is
			-- `Result' is the foreground color in which `Current'
			-- should be displayed. Based on the `Void'/ non `Void' status of
			-- `foreground_color', `row.foreground_color' and `parent.foreground_color'.
		do
			Result := implementation.displayed_foreground_color
		ensure
			result_not_void: Result /= Void
		end

feature -- Status setting

	set_comparable (a_comparable: COMPARABLE) is
			-- Asign `a_comparable' to `comparable'.
		do
			internal_comparable := a_comparable
		ensure
			comparable_set_if_not_void: a_comparable /= Void implies comparable = a_comparable
			comparable_set_to_data_if_void: a_comparable = Void implies comparable = data
		end

feature -- Basic operations

	set_pebble_agent (a_agent: like pebble_agent) is
			-- Assign `a_agent' to `pebble_agent'.
		do
			pebble_agent := a_agent
		ensure
			pebble_agent_set: pebble_agent = a_agent
		end

feature {NONE} -- Implementation

	internal_comparable: COMPARABLE

end
