indexing

	description: "Radio buttons in for an RP_DATA_GRID"

	keywords: "data grid", widget, "radio button"
	author: "Colin Adams"
	last_editor: "$Author: it $"
	team: "RC London (tools)"
	revision: "$Revision: #2 $"
	date: "$Date: 2008/12/01 $"
	path: "$File: //rose/source/kernel/infrastructure/widgets/grid/rp_data_grid_radio_item.e $"

class RP_DATA_GRID_RADIO_ITEM

inherit

	RP_DATA_GRID_LABEL_ITEM
		rename
			make_with_text as rp_data_grid_label_item_make_with_text
		redefine
			initialize, is_in_default_state
		end

	ROSE_SHARED_GLOBAL_GUI
		export
			{NONE} all
		undefine
			default_create, is_equal, copy
		end

create

	make_with_text, default_create

feature {NONE} -- Initialization

	make_with_text (a_text: STRING) is
			-- Create `Current' and assign `a_text' to `text'.
		require
			a_text_not_void: a_text /= Void
		do
			default_create
			set_text (a_text)
		ensure
			text_set: text.is_equal (a_text)
		end

	initialize is
			-- Initialize `Current'.
		do
			Precursor {RP_DATA_GRID_LABEL_ITEM}
			set_pixmap (not_selected_pix)
			pointer_button_press_actions.force_extend (agent select_on_click)
			is_sensitive := True
		end

	is_in_default_state: BOOLEAN is
			-- Is `Current' in it's default state?
		do
			Result := True			
		end

feature -- Access

	radio_select_actions: EV_NOTIFY_ACTION_SEQUENCE is
			-- Actions to perform when `is_on' is changed from `False' to `True'
		do
			if radio_select_actions_internal = Void then
				create radio_select_actions_internal
			end
			Result := radio_select_actions_internal
		ensure
			radio_select_actions_internal_not_void: radio_select_actions_internal /= Void
		end

feature -- Status report

	is_on : BOOLEAN
		-- Does `Current' display a filled circle?

	is_sensitive: BOOLEAN
			-- May `is_on' state of `Current' be changed directly by users?

feature -- Status setting

	set_is_on (a_selected : BOOLEAN) is
			-- Assign `a_selected' to `is_on'.
		do
			if a_selected /= is_on then
				is_on := a_selected
				refresh_pixmap
				if is_on then
					radio_select_actions.call ([])
				end
			end
		ensure
			state_set: is_on = a_selected
		end

	disable_sensitive is
			-- Prevent users from changing `is_on'.
		do
			is_sensitive := False
			refresh_pixmap
		ensure
			not_is_sensitive: not is_sensitive
		end

	enable_sensitive is
			-- Enable users to change `is_on'.
		do
			is_sensitive := True
			refresh_pixmap
		ensure
			is_sensitive: is_sensitive
		end

feature {NONE} -- Implementation

	radio_select_actions_internal: like radio_select_actions
			-- Actions to perform when `is_on' is changed from `False' to `True'

	select_on_click (a_x, a_y, a_button: INTEGER) is
			-- Ensure `Current' is selected if pointer is pressed on check icon.
		do
			if a_x > left_border and a_x < left_border + is_selected_pix.width and is_sensitive then
				if a_button = 1 then
					set_is_on (True)
				end
			end
		end

	not_selected_pix : EV_PIXMAP is
			-- Pixmap displayed when not selected, but sensitive.
		once
			Result := gui.factory.new_pixmap ("tr_checkbox_off.ico")
		end

	is_selected_pix : EV_PIXMAP is
			-- Pixmap displayed when selected and sensitive.
		once
			Result := gui.factory.new_pixmap ("tr_checkbox_3rd.ico")
		end

	not_selected_pix_disabled : EV_PIXMAP is
			-- Pixmap displayed when not selected and not sensitive.
		once
			Result := gui.factory.new_pixmap ("tr_checkbox_off_disabled.ico")
		end

	is_selected_pix_disabled : EV_PIXMAP is
			-- Pixamp displayed when selected and not sensitive.
		once
			Result := gui.factory.new_pixmap ("tr_checkbox_3rd_alternative.ico")
		end

	refresh_pixmap is
			-- Update pixmap displayed to match `is_on'.
		do
			if is_on then
				if is_sensitive then
					set_pixmap (is_selected_pix)
				else
					set_pixmap (is_selected_pix_disabled)
				end
			else
				if is_sensitive then
					set_pixmap (not_selected_pix)
				else
					set_pixmap (not_selected_pix_disabled)
				end
			end
		end

end
