indexing
	description: "Objects representing a sort state for use by RP_DATA_GRID"
	author: "Julian Rogers"
	date: "$Date: 2008/12/01 $"
	revision: "$Revision: #3 $"
	archive: "$File: //rose/source/kernel/infrastructure/widgets/grid/rp_data_grid_sort_state.e $"

class
	RP_DATA_GRID_SORT_STATE
	
create
	make_with_column_and_direction,
	make_with_column_ascending,
	make_with_column_descending
	
feature {NONE} -- Initialization

	make_with_column_and_direction (a_column_index: INTEGER; is_ascending: BOOLEAN) is
			-- Create `Current' representing a sort on column `a_column' with sort
			-- direction, `ascending' set to `is_ascending'.
		require
			a_column_index_positive: a_column_index > 0
		do
			column_index := a_column_index
			ascending := is_ascending
		ensure
			column_index: column_index = a_column_index
			ascending_set: ascending = is_ascending
		end

	make_with_column_ascending (a_column_index: INTEGER) is
			-- Create ascending sort order for `a_column'.
		require
			a_column_index_positive: a_column_index > 0
		do
			make_with_column_and_direction (a_column_index, True)
		ensure
			column_index: column_index = a_column_index
			ascending: ascending
		end

	make_with_column_descending (a_column_index: INTEGER) is
			-- Create descending sort order for `a_column'.
		require
			a_column_index_positive: a_column_index > 0
		do
			make_with_column_and_direction (a_column_index, False)
		ensure
			column_index: column_index = a_column_index
			descending: not ascending
		end

feature -- Access

	column_index: INTEGER
		-- Index of column for sort.
	
	ascending: BOOLEAN
		-- Is sort ascending

feature -- Status Setting

	set_ascending (a_ascending: BOOLEAN) is
			-- Assign `a_ascending' to `ascending'.
		do
			ascending := a_ascending
		ensure	
			ascending_set: ascending = a_ascending
		end

invariant
	column_index_positive: column_index > 0
	
end
