note

	description:

		"Toggle items displaying a checked state for use in RP_DATA_GRID."

	author: "Julian Rogers"
	last_editor: "$Author: jq $"
	date: "$Date: 2013/01/16 $"
	revision: "$Revision: #11 $"
	path: "$File: //rose/source/kernel/infrastructure/widgets/grid/rp_data_grid_toggle_item.e $"

class
	RP_DATA_GRID_TOGGLE_ITEM
inherit
	RP_DATA_GRID_LABEL_ITEM
		rename
			make_with_text as rp_data_grid_label_item_make_with_text
		redefine
			initialize, is_in_default_state
		end
	ROSE_SHARED_GLOBAL_GUI
		undefine
			default_create, is_equal, copy
		end
create
	make_with_text,
	default_create

feature {NONE} -- Initialization

	make_with_text (a_text: STRING) is
			-- Create `Current' and assign `a_text' to `text'.
		require
			a_text_not_void : a_text /= Void
		do
			default_create
			set_text (a_text)
		ensure
			text_set: text.is_equal (a_text)
		end

	initialize is
			-- Initialize `Current'.
		do
			Precursor {RP_DATA_GRID_LABEL_ITEM}
			refresh_pixmap
			pointer_button_press_actions.force_extend (agent toggle_on_click)
			is_sensitive := True
		end

	is_in_default_state: BOOLEAN is
			-- Is `Current' in it's default state?
		do
			Result := True
		end

feature -- Basic operations

	toggle_selected is
			-- See `toggle_state'.
		obsolete "Use toggle_state"
		do
			set_is_on (not is_on)
		end

	toggle_state is
			-- Toggle state between `is_on' and not `is_on'.
		do
			set_is_on (not is_on)
		ensure
			state_changed: is_on = not old is_on
		end

	set_is_on (a_selected : BOOLEAN) is
			-- Assign `a_selected' to `is_on'.
		do
			if a_selected /= is_on then
			is_on := a_selected
			refresh_pixmap
			toggle_actions.call ([])
			end
		ensure
			state_set: is_on = a_selected
		end

	is_on : BOOLEAN
		-- Does `Current' display a check mark?

	toggle_actions : EV_NOTIFY_ACTION_SEQUENCE is
			-- Actions to perform when changing toggle_state
		do
			if toggle_actions_internal = Void then
				create toggle_actions_internal
			end
			Result := toggle_actions_internal
		end

	disable_sensitive is
			-- Prevent users from changing `is_on'.
		do
			is_sensitive := False
			refresh_pixmap
		ensure
			not_is_sensitive: not is_sensitive
		end

	enable_sensitive is
			-- Enable users to change `is_on'.
		do
			is_sensitive := True
			refresh_pixmap
		ensure
			is_sensitive: is_sensitive
		end

	is_sensitive: BOOLEAN
			-- May `is_on' state of `Current' be changed directly by users?

feature {NONE} -- Implementation

	toggle_on_click (a_x, a_y, a_button: INTEGER) is
			-- Toggle state of `Current' if pointer is pressed on check icon.
		do
			if a_x > left_border and a_x < left_border + is_selected_pix.width and is_sensitive then
				if a_button = 1 then
					toggle_state
				else
					show_context_menu
				end
			end
		end

	not_selected_pix : EV_PIXMAP is
			-- Pixmap displayed when not selected, but sensitive.
		once
			Result := gui.factory.new_pixmap ("tr_checkbox_off.ico")
		end

	is_selected_pix : EV_PIXMAP is
			-- Pixmap displayed when selected and sensitive.
		once
			Result := gui.factory.new_pixmap ("tr_checkbox_on.ico")
		end

	not_selected_pix_disabled : EV_PIXMAP is
			-- Pixmap displayed when not selected and not sensitive.
		once
			Result := gui.factory.new_pixmap ("tr_checkbox_off_disabled.ico")
		end

	is_selected_pix_disabled : EV_PIXMAP is
			-- Pixamp displayed when selected and not sensitive.
		once
			Result := gui.factory.new_pixmap ("tr_checkbox_on_disabled.ico")
		end

	refresh_pixmap is
			-- Update pixmap displayed to match `is_on'.
		local
			l_pixmap: EV_PIXMAP
		do
			if is_on then
				if is_sensitive then
					l_pixmap := is_selected_pix
				else
					l_pixmap := is_selected_pix_disabled
				end
			else
				if is_sensitive then
					l_pixmap := not_selected_pix
				else
					l_pixmap := not_selected_pix_disabled
				end
			end
			if l_pixmap = Void then
				remove_pixmap
			else
				set_pixmap (l_pixmap)
			end
		end

	show_context_menu is
			-- Show context menu for selection handling in `column'.
		local
			l_menu: EV_MENU
			l_menu_item: EV_MENU_ITEM
		do
			create l_menu
			create l_menu_item.make_with_text ("Check All")
			l_menu_item.select_actions.extend (agent check_all)
			l_menu.extend (l_menu_item)
			create l_menu_item.make_with_text ("Un-Check All")
			l_menu_item.select_actions.extend (agent uncheck_all)
			l_menu.extend (l_menu_item)
			create l_menu_item.make_with_text ("Check Selected")
			l_menu_item.select_actions.extend (agent check_selected)
			l_menu.extend (l_menu_item)
			create l_menu_item.make_with_text ("Un-Check Selected")
			l_menu_item.select_actions.extend (agent uncheck_selected)
			l_menu.extend (l_menu_item)
			l_menu.show
		end

	check_all is
			-- Check all toggle items in `column'.
		do
			set_checked_state_for_all (True)
		end

	uncheck_all is
			-- Uncheck all toggle items in `column'.
		do
			set_checked_state_for_all (False)
		end

	check_selected is
			-- Check all selected toggle items in `column'.
		do
			set_checked_state_for_selected (True)
		end

	uncheck_selected is
			-- Uncheck all selected toggle items in `column'.
		do
			set_checked_state_for_selected (False)
		end

	set_checked_state_for_all (a_state: BOOLEAN) is
			-- Set checked state for all toggle items in `column' to `a_state'.
		local
			i: INTEGER
			l_grid: RP_DATA_GRID [ANY]
			l_node: RP_GRID_NODE [ANY]
			l_toggle_item: RP_DATA_GRID_TOGGLE_ITEM
		do
			l_grid := node.grid
			from
				i := 1
			until
				i > l_grid.row_count
			loop
				l_node := l_grid.row (i)
				if l_node.is_show_requested then
					l_toggle_item ?= l_node.i_th (column.index)
					if l_toggle_item /= Void then
						l_toggle_item.set_is_on (a_state)
					end
				end
				i := i + 1
			end
		end

	set_checked_state_for_selected (a_state: BOOLEAN) is
			-- Set checked state for all selected toggle items in `column' to `a_state'.
		local
			i: INTEGER
			l_grid: RP_DATA_GRID [ANY]
			l_node: RP_GRID_NODE [ANY]
			l_toggle_item: RP_DATA_GRID_TOGGLE_ITEM
		do
			l_grid := node.grid
			from
				i := 1
			until
				i > l_grid.row_count
			loop
				l_node := l_grid.row (i)
				if l_node.is_show_requested then
					l_toggle_item ?= l_node.i_th (column.index)
					if l_toggle_item /= Void then
						if l_toggle_item.is_selected then
							l_toggle_item.set_is_on (a_state)
						end
					end
				end
				i := i + 1
			end
		end

	toggle_actions_internal : EV_NOTIFY_ACTION_SEQUENCE
		-- Implementation of `toggle_actions'.

end
