indexing
	description: "Objects representing data grid items that display a widget."
	author: "Broos Nemanic"
	date: "$Date: 2008/12/01 $"
	revision: "$Revision: #11 $"
	archive: "$File: //rose/source/kernel/infrastructure/widgets/grid/rp_data_grid_widget_item.e $"

class
	RP_DATA_GRID_WIDGET_ITEM

inherit
	RP_DATA_GRID_LABEL_ITEM

	IDENTIFIED
		undefine
			copy, is_equal, default_create
		end

create
	make,
	make_with_agents

feature {NONE} -- initialization

	make_with_agents (a_instantiate_widget_agent : like instantiate_widget_agent; a_clear_widget_agent : like clear_widget_agent) is
			-- Creation procedure.
		do
			default_create
			instantiate_widget_agent := a_instantiate_widget_agent
			clear_widget_agent := a_clear_widget_agent
		end

	make (a_widget : EV_WIDGET) is
		do
			make_with_agents (agent instantiate_widget (?, a_widget), agent hide_widget (?, a_widget))
			set_widget (a_widget)
		end

feature -- Access

	clear_widget_agent : PROCEDURE [ANY, TUPLE [RP_DATA_GRID_WIDGET_ITEM]]

	instantiate_widget_agent : FUNCTION [ANY, TUPLE [RP_DATA_GRID_WIDGET_ITEM], EV_WIDGET]

	widget : EV_WIDGET
			-- Widget displayed in `Current'.

	is_widget_height_dominant : BOOLEAN
			--Will widget RP_GRID_NODE.set_height (widget.height.max (RP_GRID_NODE.height))?

	is_widget_width_dominant : BOOLEAN
			--Will widget column.set_width (widget.width.max (column.width))?

	widget_column_span : INTEGER
			--if n > 1 then widget will resize to span n columns

feature -- Status setting

	set_instantiate_widget_agent (a_agent : FUNCTION [ANY, TUPLE [RP_DATA_GRID_WIDGET_ITEM], EV_WIDGET]) is
			-- Assign `a_agent' to `instantiate_widget_agent'.
		do
			instantiate_widget_agent := a_agent
		ensure
			instantiate_widget_agent_set: instantiate_widget_agent = a_agent
		end

	set_clear_widget_agent (a_agent : PROCEDURE [ANY, TUPLE [RP_DATA_GRID_WIDGET_ITEM]]) is
			-- Assign `a_agent' to `clear_widget_agent'.
		do
			clear_widget_agent := a_agent
		ensure
			clear_widget_agent_set: clear_widget_agent = a_agent
		end

	set_widget_height_dominant (a_dominant : BOOLEAN) is
			-- Assign `a_dominant' to `is_widget_height_dominant'.
		do
			is_widget_height_dominant := a_dominant
		ensure
			is_widget_height_dominant_set: is_widget_height_dominant = a_dominant
		end

	set_widget_width_dominant (a_dominant : BOOLEAN) is
			-- Assign `a_dominant' to `is_widget_width_dominant'.
		do
			is_widget_width_dominant := a_dominant
		ensure
			is_widget_width_dominant_set: is_widget_width_dominant = a_dominant
		end

	set_widget_column_span (a_span : INTEGER) is
			-- Assign `a_span' to `widget_column_span'.
		do
			widget_column_span := a_span
		ensure
			widget_column_span_set: widget_column_span = a_span
		end

	clear_widget is
			-- Clear widget by calling `clear_widget_agent'.
		do
			clear_widget_agent.call ([Current])
		end

	get_widget is
			-- Get widget from `instantiate_widget_agent' if `widget' is Void.
		do
			if instantiate_widget_agent /= Void then
				if widget = Void then
					widget := instantiate_widget_agent.item ([Current])
				end
			end
		end

	get_new_widget is
			-- rebuild widget
		do
			if instantiate_widget_agent /= Void then
				widget := instantiate_widget_agent.item ([Current])
			end
		end

	set_widget (a_widget: EV_WIDGET) is
			-- Assign `a_widget' to `_widget'.
		do
			widget := a_widget
		ensure
			widget_set: widget = a_widget
		end

feature {NONE} -- Implementation

	instantiate_widget (a_item: like Current; a_widget : EV_WIDGET) : EV_WIDGET is
			-- Return `a_widget'.
		do
			Result := a_widget
		end

	hide_widget (a_item: like Current; a_widget : EV_WIDGET) is
			-- Hide `a_widget'.
		do
			a_widget.hide
		end

end
