indexing

	description:

		"Default Valuator Factory"

	last_editor: "$Author: bgum $"
	date: "$Date: 2010/08/27 $"
	revision: "$Revision: #20 $"
	archive: "$File: //rose/source/kernel/expert_system/aflows/rp_default_holding_valuator_factory.e $"

class RP_DEFAULT_HOLDING_VALUATOR_FACTORY

inherit

	RP_HOLDING_VALUATOR_FACTORY

	RP_INVESTMENT_MANAGER_CLIENT

create

	make

feature -- Factory methods

	portfolio_holding_valuator (a_portfolio: RP_PORTFOLIO): RP_HOLDING_VALUATOR is
			-- Create valuator for `a_portfolio.
		do
			Result := portfolio_holding_valuator_with_internal_prices (a_portfolio, Void)
		end

	portfolio_holding_valuator_simulation (a_portfolio: RP_PORTFOLIO;  a_prices: ROSE_TIME_SERIES [ROSE_HASH_TABLE [DOUBLE, STRING]]; a_system: RP_REFERENCE_SYSTEM): RP_HOLDING_VALUATOR_SIMULATION is
			-- Create valuator for `a_portfolio
		local
			l_trade_price: RP_TRADE_PRICE_MARKET_DATA_SOURCE
		do
			create l_trade_price.make (manager.accounting, a_portfolio)
			Result := create {RP_HOLDING_VALUATOR_SIMULATION}.make (equity_pricing_algorithm_internal (a_portfolio, l_trade_price), poolfund_pricing_algorithm (a_portfolio, l_trade_price, Void), currency_market_data_source, create {RP_PERFORMANCE_INDEX_FUTURE_PRICING_ALGORITHM}.make (index_futures_market_data_source, l_trade_price), corporate_bond_pricing_algorithm (a_portfolio, l_trade_price), corporate_bond_interest_algorithm (a_portfolio), create {RP_PERFORMANCE_TBILL_PRICING_ALGORITHM}.make (l_trade_price, tbills_market_data_source, price_override))
			Result.set_zone (a_portfolio.zone)
			Result.set_reference_system (a_system)
			Result.set_security_prices (a_prices)
		end

	portfolio_holding_valuator_with_internal_prices (a_portfolio: RP_PORTFOLIO; a_internal_prices: PAIR [STRING, DS_HASH_TABLE [RP_PERFORMANCE_DAILY_VALUES, ROSE_DATE]]): RP_HOLDING_VALUATOR is
			-- Create valuator for `a_portfolio.
		local
			l_trade_price: RP_TRADE_PRICE_MARKET_DATA_SOURCE
		do
			create l_trade_price.make (manager.accounting, a_portfolio)
			if a_portfolio.zone.is_japan then
				Result := create {RP_VALUATOR_JZONE_DEFAULT}.make (equity_pricing_algorithm_internal (a_portfolio, l_trade_price), poolfund_pricing_algorithm (a_portfolio, l_trade_price, a_internal_prices), currency_market_data_source, create {RP_PERFORMANCE_INDEX_FUTURE_PRICING_ALGORITHM}.make (index_futures_market_data_source, l_trade_price), corporate_bond_pricing_algorithm (a_portfolio, l_trade_price), corporate_bond_interest_algorithm (a_portfolio), create {RP_PERFORMANCE_TBILL_PRICING_ALGORITHM}.make (l_trade_price, tbills_market_data_source, price_override))
			else
				Result := create {RP_HOLDING_VALUATOR_AXAR_DEFAULT}.make (equity_pricing_algorithm_internal (a_portfolio, l_trade_price), poolfund_pricing_algorithm (a_portfolio, l_trade_price, a_internal_prices), currency_market_data_source, create {RP_PERFORMANCE_INDEX_FUTURE_PRICING_ALGORITHM}.make (index_futures_market_data_source, l_trade_price), corporate_bond_pricing_algorithm (a_portfolio, l_trade_price), corporate_bond_interest_algorithm (a_portfolio), create {RP_PERFORMANCE_TBILL_PRICING_ALGORITHM}.make (l_trade_price, tbills_market_data_source, price_override))
			end
		end

	equity_pricing_algorithm (a_portfolio: RP_PORTFOLIO): RP_PRICING_ALGORITHM [RP_STOCK] is
			-- `a_portfolio' specific stock pricing algorithm
		local
			l_trade_price: RP_TRADE_PRICE_MARKET_DATA_SOURCE
		do
			create l_trade_price.make (manager.accounting, a_portfolio)
			Result := equity_pricing_algorithm_internal (a_portfolio, l_trade_price)
		end

	default_poolfund_market_data_source: RP_CADB_POOL_FUND_MARKET_DATA_SOURCE is
			-- Default poolfund market data source
		once
			create Result.make (manager.reference_system)
		end

	enable_unlimited_u_zone_price_looback is
			-- Enable unlimited price lookback in U zone
		do
			unlimited_u_zone_price_looback := True
		end

	disable_unlimited_u_zone_price_looback is
			-- Disable unlimited price lookback in U zone
		do
			unlimited_u_zone_price_looback := False
		end

	is_unlimited_u_zone_price_looback_enabled: BOOLEAN is
			-- Is unlimited price lookback in U zone enabled?
		do
			Result := unlimited_u_zone_price_looback
		end

	enable_price_override_in_u_zone is
			-- Enable price override in U zone
		do
			dont_use_override_in_u_zone := False
		end

	disable_price_override_in_u_zone is
			-- Disable price override in U zone
		do
			dont_use_override_in_u_zone := True
		end

	is_price_override_in_u_zone_enabled: BOOLEAN is
			-- Is price override in U zone enabled?
		do
			Result := not dont_use_override_in_u_zone
		end

	enable_clio_pricing is
			-- Enable CLIO pricing
		do
			use_clio_pricing := True
		ensure
			clio_enabled: is_clio_pricing_enabled = True
		end

	disable_clio_pricing is
			-- Disable CLIO pricing
		do
			use_clio_pricing := False
		ensure
			clio_disabled: is_clio_pricing_enabled = False
		end

	is_clio_pricing_enabled: BOOLEAN is
			-- Is CLIO pricing enabled?
		do
			Result := use_clio_pricing
		end

feature {NONE} -- Implementation

	use_clio_pricing: BOOLEAN
			-- Use CLIO as first source for fund NAV's? (fallback to composite_asset db if False or not found there)

	unlimited_u_zone_price_looback: BOOLEAN
			-- Should U zone look back indefinitely into the past

	dont_use_override_in_u_zone: BOOLEAN
			-- Do not use price override in U zone

	corporate_bond_interest_algorithm (a_portfolio: RP_PORTFOLIO): !RP_PERFORMANCE_CORPORATE_BOND_INTEREST_ALGORITHM is
			-- Corporate bonds interest algorithm
		do
			create Result.make (corporate_bond_market_data_source, a_portfolio.zone.legacy_perf_calendar)
		end

	corporate_bond_market_data_source: RP_AXAROSENBERG_CORPORATE_BOND_MARKET_DATA_SOURCE is
			-- FIXME add to RP_MARKET_SYSTEM
		once
			create Result.make (manager.reference_system)
		end

	price_override: RP_AXAROSENBERG_PRICE_OVERRIDE_MARKET_DATA_SOURCE is
			-- Price override
		once
			create Result.make (manager.reference_system)
		end

	currency_market_data_source: RP_AXAROSENBERG_CURRENCY_MARKET_DATA_SOURCE is
			-- Currency market data source
		once
			create Result.make (manager.reference_system)
		end

	equities_market_data_source: RP_MARKET_DATA_SOURCE [RP_STOCK] is
			-- Eqities market data source
		once
			Result := manager.market_system.equities_market_data_sources.item (1)
		end

	tbills_market_data_source: RP_MARKET_DATA_SOURCE [RP_GOVERNMENT_BOND] is
			-- Eqities market data source
		once
			Result := manager.market_system.government_bonds_market_data_sources.item (1)
		end

	index_futures_market_data_source: RP_MARKET_DATA_SOURCE [RP_INDEX_FUTURE] is
			-- Index futures market data source
		once
			Result := manager.market_system.index_futures_market_data_sources.item (1)
		end

	poolfund_pricing_algorithm (a_portfolio: RP_PORTFOLIO; a_trade_price: RP_TRADE_PRICE_MARKET_DATA_SOURCE; a_internal_prices: PAIR [STRING, DS_HASH_TABLE [RP_PERFORMANCE_DAILY_VALUES, ROSE_DATE]]): RP_POOL_FUND_PRICING_ALGORITHM is
			-- poolfund price for `a_zone'
		require
			non_void_args: a_portfolio /= Void and a_trade_price /= Void
		local
			l_uzone: RP_PERFORMANCE_UZONE_POOL_FUND_PRICING_ALGORITHM
		do
			if a_portfolio.zone.is_united_states then
				create l_uzone.make (a_portfolio.zone, manager.reference_system, a_trade_price, a_portfolio.zone.legacy_perf_calendar, price_override)
				l_uzone.set_internal_prices (a_internal_prices)
				Result := l_uzone
			else
				create {RP_POOL_FUND_PRICING_ALGORITHM} Result.make (a_portfolio.zone, manager.reference_system, a_trade_price, a_portfolio.zone.legacy_perf_calendar, price_override)
			end
			if is_clio_pricing_enabled then
				Result.enable_clio_pricing
			end
		ensure
			result_not_void: Result /= Void
		end

	equity_pricing_algorithm_internal (a_portfolio: RP_PORTFOLIO; a_trade_price: RP_TRADE_PRICE_MARKET_DATA_SOURCE): RP_PRICING_ALGORITHM [RP_STOCK] is
			-- `a_portfolio' specific stock pricing algorithm
		local
			l_calendar: RP_CALENDAR
		do
			l_calendar := a_portfolio.zone.legacy_perf_calendar
			if a_portfolio.code.is_equal ("JOHNC") then
				Result := create {RP_PERFORMANCE_JOHNC_STOCK_PRICING_ALGORITHM}.make_with_trade_price (a_portfolio.zone, manager.reference_system, a_trade_price, l_calendar, equities_market_data_source, price_override)
			elseif a_portfolio.code.is_equal ("JOHNE") or a_portfolio.code.is_equal ("JOHNO") then
				Result := create {RP_PERFORMANCE_JOHNE_STOCK_PRICING_ALGORITHM}.make_with_trade_price (a_portfolio.zone, manager.reference_system, a_trade_price, l_calendar, equities_market_data_source, price_override)
			elseif a_portfolio.code.is_equal ("JOHNJ") then
				Result := create {RP_PERFORMANCE_JOHNJ_STOCK_PRICING_ALGORITHM}.make_with_trade_price (a_portfolio.zone, manager.reference_system, a_trade_price, l_calendar, equities_market_data_source, price_override)
			elseif a_portfolio.code.is_equal ("SMILEC") then
				Result := create {RP_PERFORMANCE_SMILEC_STOCK_PRICING_ALGORITHM}.make_with_trade_price (a_portfolio.zone, manager.reference_system, a_trade_price, l_calendar, equities_market_data_source, price_override)
			elseif a_portfolio.code.is_equal ("SMILEE") or a_portfolio.code.is_equal ("SMILEO") then
				Result := create {RP_PERFORMANCE_SMILEE_STOCK_PRICING_ALGORITHM}.make_with_trade_price (a_portfolio.zone, manager.reference_system, a_trade_price, l_calendar, equities_market_data_source, price_override)
			elseif a_portfolio.zone.code.is_equal ("S") then
				Result := create {RP_PERFORMANCE_SZONE_STOCK_PRICING_ALGORITHM}.make_with_trade_price (a_portfolio.zone, manager.reference_system, a_trade_price, l_calendar, equities_market_data_source, price_override)
			elseif a_portfolio.zone.code.is_equal ("J") then
				Result := create {RP_PERFORMANCE_JZONE_STOCK_PRICING_ALGORITHM}.make_with_trade_price (a_portfolio.zone, manager.reference_system, a_trade_price, l_calendar, equities_market_data_source, price_override)
			elseif a_portfolio.zone.code.is_equal ("U") then
				Result := create {RP_PERFORMANCE_UZONE_PRICING_ALGORITHM}.make (a_trade_price, equities_market_data_source, price_override, unlimited_u_zone_price_looback, dont_use_override_in_u_zone)
			elseif a_portfolio.zone.code.is_equal ("O") then
				Result := create {RP_PERFORMANCE_OZONE_STOCK_PRICING_ALGORITHM}.make_with_trade_price (a_portfolio.zone, manager.reference_system, a_trade_price, l_calendar, equities_market_data_source, price_override)
			elseif a_portfolio.zone.code.is_equal ("E") then
				Result := create {RP_PERFORMANCE_EZONE_STOCK_PRICING_ALGORITHM}.make_with_trade_price (a_portfolio.zone, manager.reference_system, a_trade_price, l_calendar, equities_market_data_source, price_override)
			else
				Result := create {RP_PERFORMANCE_STOCK_PRICING_ALGORITHM}.make_with_trade_price (a_portfolio.zone, manager.reference_system, a_trade_price, l_calendar, equities_market_data_source, price_override)
			end
		end

	corporate_bond_pricing_algorithm (a_portfolio: RP_PORTFOLIO; a_trade_price: RP_TRADE_PRICE_MARKET_DATA_SOURCE): RP_PERFORMANCE_CORPORATE_BOND_PRICING_ALGORITHM is
			-- `a_portfolio' specific corporate bonds pricing algorithm
		local
			l_calendar: RP_CALENDAR
		do
			l_calendar := a_portfolio.zone.legacy_perf_calendar
			create Result.make (a_trade_price, l_calendar, corporate_bond_market_data_source, price_override)
		end

end
