note

	description:

		"Distribution contract"

	author: "Ivan Tarnavsky"
	last_editor: "$Author: e2 $"
	date: "$Date: 2011/11/29 $"
	revision: "$Revision: #15 $"
	path: "$File: //rose/source/kernel/expert_system/rp_distribution.e $"

class RP_DISTRIBUTION

inherit

	RP_PORTFOLIO_CORPORATE_ACTION_CONTRACT

create

	make_long_position,
	make_short_position

feature {NONE} -- Constructor

	make_long_position (a_units_base, a_quantity: RP_QUANTITY; a_value: RP_MONIES;
		a_event_date, a_settle_date: ROSE_DATE; a_portfolio: like portfolio; a_identifier: like identifier;
		a_system: like generating_system; a_custody: RP_CUSTODY) is
			-- Create long position distribution
		require
			a_units_base_not_void: a_units_base /= Void
			a_quantity_not_void: a_quantity /= Void
			a_value_not_void: a_value /= Void
			a_event_date_not_void: a_event_date /= Void
			a_settle_date_not_void: a_settle_date /= Void
			a_portfolio_not_void: a_portfolio /= Void
			a_identifier_not_void: a_identifier /= Void
			a_system_not_void: a_system /= Void
			a_custody_not_void: a_custody /= Void
		local
			l_value: RP_MONIES
		do
				-- Contract
			make_capital_action (a_system, a_event_date, a_identifier, a_portfolio, a_units_base, a_settle_date)
			is_cash_distribution := a_quantity.instrument.is_currency
				-- Flow: units coming in
			l_value := a_value.abs
			if is_cash_distribution then
					-- XXX NB: cost of cash distribution in J zone = cash
				l_value ?= a_quantity.abs
			end
			add_flow (distribution_flow_type, unknown_instrument_control, a_portfolio.instrument_control (a_custody), a_event_date, a_settle_date, a_quantity.abs, a_event_date, 0, l_value)
		end

	make_short_position (a_units_base, a_quantity: RP_QUANTITY; a_value: RP_MONIES;
		a_event_date, a_settle_date: ROSE_DATE; a_portfolio: like portfolio; a_identifier: like identifier;
		a_system: like generating_system; a_custody: RP_CUSTODY) is
			-- Create short position distribution
		require
			a_units_base_not_void: a_units_base /= Void
			a_quantity_not_void: a_quantity /= Void
			a_value_not_void: a_value /= Void
			a_event_date_not_void: a_event_date /= Void
			a_settle_date_not_void: a_settle_date /= Void
			a_portfolio_not_void: a_portfolio /= Void
			a_identifier_not_void: a_identifier /= Void
			a_system_not_void: a_system /= Void
			a_custody_not_void: a_custody /= Void
		local
			l_value: RP_MONIES
		do
				-- Contract
			make_capital_action (a_system, a_event_date, a_identifier, a_portfolio, a_units_base, a_settle_date)
			is_cash_distribution := a_quantity.instrument.is_currency
				-- Flow: units going out
			l_value := a_value.abs
			if is_cash_distribution then
					-- XXX NB: cost of cash distribution in J zone = cash
				l_value ?= a_quantity.abs
			end
			add_flow (distribution_flow_type, a_portfolio.instrument_control (a_custody), unknown_instrument_control, a_event_date, a_settle_date, a_quantity.abs, a_event_date, 0, -l_value)
		end

feature -- Status Report

	is_cash_distribution: BOOLEAN
			-- Is this distributing cash?

feature -- Basic Operations

	process (a_processor: RP_CONTRACT_PROCESSOR) is
			-- <Precursor>
		do
			a_processor.process_distribution (Current)
		end

end
