indexing
	description: "Simple eLog client where the logger always has a default"
	author: "Portfolio Accountign Tech (sm)"
	last_editor: "$Author: sm $"
	date: "$Date: 2010/01/26 $"
	revision: "$Revision: #2 $"
	archive: "$File: //rose/source/kernel/expert_system/rp_elog_client.e $"

deferred class
	RP_ELOG_CLIENT

inherit
	ELOG_SHARED_HIERARCHY
		undefine
			default_create
		end

feature -- Access

	log: ELOG_LOGGER is
			-- Logger
		do
			if log_saved = Void then set_logger (default_logger) end
			Result := log_saved
		ensure
			log_non_void: Result /= Void
		end

feature -- Basic Operations

	log_debug (a_message: STRING) is
			-- Log debug event
		require
			message_not_void: a_message /= Void
		local
			l_event: ELOG_SIMPLE_EVENT
		do
			create l_event.make (a_message)
			log.log_debug (l_event)
		end

	log_info (a_message: STRING) is
			-- Log informational event
		require
			message_not_void: a_message /= Void
		local
			l_event: ELOG_SIMPLE_EVENT
		do
			create l_event.make (a_message)
			log.log_information (l_event)
		end

	log_minor (a_message: STRING) is
			-- Log minor event
		require
			message_not_void: a_message /= Void
		local
			l_event: ELOG_SIMPLE_EVENT
		do
			create l_event.make (a_message)
			log.log_minor (l_event)
		end

feature -- Change

	reset_logger is
			-- Reset logger to default
		do
			log_saved := void
		end

	set_logger (a_log: like log) is
			-- Set 'log' to 'a_log'
		require
			log_not_void: a_log /= Void
		do
			log_saved := a_log
		ensure
			log_set: log = a_log
		end

feature {NONE} -- Implementation

	log_saved: like log
			-- Logger

	default_logger: like log is
			-- Default logger
		local
			l_appender: ELOG_APPENDER
		do
			Result := log_hierarchy.logger ("default")
			l_appender := create {ELOG_STDOUT_APPENDER}.make ("RP_AXAROSENBERG stdout appender", True)
			l_appender.set_layout (create {ELOG_DATE_TIME_LAYOUT})
			Result.add_appender (l_appender)
		ensure
			result_not_void: Result /= Void
		end

end
