note
	description: "Portfolio end of day NAV - works before and after NT migration"
	author: "Unknown"
	last_editor: "$Author: it $"
	date: "$Date: 2014/03/11 $"
	revision: "$Revision: #5 $"
	path: "$File: //rose/source/kernel/expert_system/rp_end_of_day_nav.e $"

class RP_END_OF_DAY_NAV

inherit
	ANY

	RP_AXAROSENBERG_AWARE
		export {NONE} all end

feature -- Access (Portfolio)

	portfolio_value (a_accounting_date: ROSE_DATE; a_portfolio: RP_PORTFOLIO; a_currency: RP_CURRENCY): RP_MONIES
			-- <Precursor>
		local
			l_dailyvalues: RP_PERFORMANCE_DAILY_VALUES
		do
			if a_accounting_date < manager.end_of_day_positions.northerntrust_positions_migration_date then
				l_dailyvalues := manager.legacy.performance_system.portfolio_dailyvalues_at (a_portfolio, a_currency, a_accounting_date)
				if l_dailyvalues /= Void then
					Result := l_dailyvalues.portfolio_value
				else
					Result := a_currency.monies (create {RP_DECIMAL}.make_undefined)
				end
			else
				Result := manager.end_of_day_positions.portfolio_value (a_accounting_date, a_portfolio, a_currency)
			end
		end

	portfolio_cash_value (a_accounting_date: ROSE_DATE; a_portfolio: RP_PORTFOLIO; a_currency: RP_CURRENCY): RP_MONIES
			-- <Precursor>
		local
			l_dailyvalues: RP_PERFORMANCE_DAILY_VALUES
		do
			if a_accounting_date < manager.end_of_day_positions.northerntrust_positions_migration_date then
				l_dailyvalues := manager.legacy.performance_system.portfolio_dailyvalues_at (a_portfolio, a_currency, a_accounting_date)
				if l_dailyvalues /= Void then
					Result := l_dailyvalues.cash_value
				else
					Result := a_currency.monies (create {RP_DECIMAL}.make_undefined)
				end
			else
				Result := manager.end_of_day_positions.portfolio_cash_value (a_accounting_date, a_portfolio, a_currency)
			end
		end

	portfolio_equity_value (a_accounting_date: ROSE_DATE; a_portfolio: RP_PORTFOLIO; a_currency: RP_CURRENCY): RP_MONIES
			-- <Precursor>
		local
			l_dailyvalues: RP_PERFORMANCE_DAILY_VALUES
		do
			if a_accounting_date < manager.end_of_day_positions.northerntrust_positions_migration_date then
				l_dailyvalues := manager.legacy.performance_system.portfolio_dailyvalues_at (a_portfolio, a_currency, a_accounting_date)
				if l_dailyvalues /= Void then
					Result := l_dailyvalues.equity_value
				else
					Result := a_currency.monies (create {RP_DECIMAL}.make_undefined)
				end
			else
				Result := manager.end_of_day_positions.portfolio_equity_value (a_accounting_date, a_portfolio, a_currency)
			end
		end

	portfolio_dailyreturns (a_portfolio: RP_PORTFOLIO; a_currency: RP_CURRENCY): RP_DAILY_RETURNS is
			-- Daily returns for `a_portfolio' in `a_currency'
		require
			a_portfolio_not_void: a_portfolio /= Void
			a_currency_not_void: a_currency /= Void
		local
			l_returns: RP_DAILY_RETURNS
		do
			create Result.make (a_portfolio.code)
			if not a_portfolio.zone.is_transzonal then
				daily_returns_query.retrieve (Void, <<a_portfolio.zone.code, a_portfolio.code, a_portfolio.zone.legacy_base_currency.iso>>)
				create l_returns.make (a_portfolio.code)
				daily_returns_query.array.do_all (agent add_record_as_daily_return (l_returns, a_portfolio.zone.legacy_base_currency, ?))
				create Result.make (a_portfolio.code)
				l_returns.returns.do_all (agent translate_and_add_daily_return (Result, a_currency, a_portfolio.zone.legacy_base_currency, ?))
			end
		ensure
			non_void_result: Result /= Void
		end

	account_dailyreturns (a_account: RP_INVESTMENT_ACCOUNT; a_currency: RP_CURRENCY): RP_DAILY_RETURNS is
			-- Daily returns for `a_account' in `a_currency'
		require
			a_account_not_void: a_account /= Void
			a_currency_not_void: a_currency /= Void
		local
			l_usd_returns: RP_DAILY_RETURNS
		do
			create Result.make (a_account.code)
			if a_account.is_global then
				daily_returns_query.retrieve (Void, <<"T", a_account.code, usd.name>>)
				create l_usd_returns.make (a_account.code)
				daily_returns_query.array.do_all (agent add_record_as_daily_return (l_usd_returns, usd, ?))
			elseif not a_account.portfolios.is_empty then
				l_usd_returns := portfolio_dailyreturns (a_account.portfolios.first.portfolio, usd)
			end
			if l_usd_returns /= Void then
				l_usd_returns.returns.do_all (agent translate_and_add_daily_return (Result, a_currency, usd, ?))
			end
		ensure
			non_void_result: Result /= Void
		end

feature -- Account

	account_value (a_accounting_date: ROSE_DATE; a_account: RP_INVESTMENT_ACCOUNT; a_currency: RP_CURRENCY): RP_MONIES
			-- <Precursor>
		local
			l_dailyvalues: RP_PERFORMANCE_DAILY_VALUES
		do
			if a_accounting_date < manager.end_of_day_positions.northerntrust_positions_migration_date then
				l_dailyvalues := manager.legacy.performance_system.investment_account_dailyvalues_at (a_account, a_currency, a_accounting_date)
				if l_dailyvalues /= Void then
					Result := l_dailyvalues.portfolio_value
				else
					Result := a_currency.monies (create {RP_DECIMAL}.make_undefined)
				end
			else
				Result := manager.end_of_day_positions.account_value (a_accounting_date, a_account, a_currency)
			end
		end

	account_cash_value (a_accounting_date: ROSE_DATE; a_account: RP_INVESTMENT_ACCOUNT; a_currency: RP_CURRENCY): RP_MONIES
			-- <Precursor>
		local
			l_dailyvalues: RP_PERFORMANCE_DAILY_VALUES
		do
			if a_accounting_date < manager.end_of_day_positions.northerntrust_positions_migration_date then
				l_dailyvalues := manager.legacy.performance_system.investment_account_dailyvalues_at (a_account, a_currency, a_accounting_date)
				if l_dailyvalues /= Void then
					Result := l_dailyvalues.cash_value
				else
					Result := a_currency.monies (create {RP_DECIMAL}.make_undefined)
				end
			else
				Result := manager.end_of_day_positions.account_cash_value (a_accounting_date, a_account, a_currency)
			end
		end

	account_equity_value (a_accounting_date: ROSE_DATE; a_account: RP_INVESTMENT_ACCOUNT; a_currency: RP_CURRENCY): RP_MONIES
			-- <Precursor>
		local
			l_dailyvalues: RP_PERFORMANCE_DAILY_VALUES
		do
			if a_accounting_date < manager.end_of_day_positions.northerntrust_positions_migration_date then
				l_dailyvalues := manager.legacy.performance_system.investment_account_dailyvalues_at (a_account, a_currency, a_accounting_date)
				if l_dailyvalues /= Void then
					Result := l_dailyvalues.equity_value
				else
					Result := a_currency.monies (create {RP_DECIMAL}.make_undefined)
				end
			else
				Result := manager.end_of_day_positions.account_equity_value (a_accounting_date, a_account, a_currency)
			end
		end

feature {NONE} -- Implementation

	daily_returns_query: COLLECTING_PREDICATE [RP_DAILY_RETURN_RECORD] is
			-- Daily returns selection predicate
		once
			create Result.make ((create {DATABASE_ACCESS}).make_from_repository ("ntperf"), daily_returns_sql)
			if not Result.success then
				(create {EXCEPTIONS}).raise ("Can't create daily_returns_query")
			end
		ensure
			not_void: Result /= Void
		end

	daily_returns_sql: STRING = "SELECT * FROM vw_daily_returns WHERE zone=? AND portfolio=? AND iso=?"
			-- Daily returns selection sql

	fx: RP_CURRENCY_MARKET_DATA_SOURCE is
			-- FX spot rate provides
		once
			if manager.market_system.currency_market_data_sources.is_empty then
				(create {EXCEPTIONS}).raise ("Can't retrieve currency market data source")
			end
			Result := manager.market_system.currency_market_data_sources.item (1)
		ensure
			not_void: Result /= Void
		end

	usd: RP_CURRENCY is
			-- USD Currency instance
		once
			Result := manager.reference_system.currency_from_iso_code ("USD")
			if Result = Void then
				(create {EXCEPTIONS}).raise ("Can't retrieve USD entity")
			end
		ensure
			not_void: Result /= Void
		end

	add_record_as_daily_return (a_daily_returns: RP_DAILY_RETURNS; a_currency: RP_CURRENCY; a_record: RP_DAILY_RETURN_RECORD) is
			-- Convert `a_record' to RP_DAILY_RETURN and add to `a_daily_returns'
		require
			a_daily_returns_not_void: a_daily_returns /= Void
			a_currency_not_void: a_currency /= Void
			a_record_not_void: a_record /= Void
		local
			l_return: RP_DAILY_RETURN
		do
			create l_return.make (
					create {ROSE_DATE}.make_ccyymmdd (a_record.ccyymmdd),
					a_currency,
					a_record.value,
					a_record.daily_return
				)
			a_daily_returns.expand (l_return)
		end

	translate_and_add_daily_return (a_daily_returns: RP_DAILY_RETURNS; a_currency, a_base: RP_CURRENCY; a_daily_return: RP_DAILY_RETURN) is
			-- Translate `a_daily_return' into `a_currency' from `a_base' and then add to `a_daily_returns'
		require
			a_daily_returns_not_void: a_daily_returns /= Void
			a_currency_not_void: a_currency /= Void
			a_base_not_void: a_base /= Void
			a_daily_return_not_void: a_daily_return /= Void
		local
			l_return: RP_DAILY_RETURN
			l_fx, l_daily_return: RP_DECIMAL
		do
			l_fx := fx.spot_rate (a_base, a_currency, a_daily_return.date)
			if a_daily_return.prev = Void then
				l_daily_return := 1
			else
				l_daily_return := a_daily_return.return * l_fx / fx.spot_rate (a_base, a_currency, a_daily_return.prev.date)
			end
			create l_return.make (a_daily_return.date, a_currency, a_daily_return.value * l_fx, l_daily_return)
			a_daily_returns.expand (l_return)
		end

end
