indexing

	description:

		"Typed accounting system transaction"

	date: "$Date: 2010/05/14 $"
	sccid: "$Id: //rose/source/kernel/expert_system/rp_event_typed.e#3 $"

deferred class RP_EVENT_TYPED

inherit

	ANY

	RP_EVENT_TYPE_CONSTANTS
		export {NONE} all end

feature -- Access

	event_type: INTEGER is
			-- Type
		deferred
		end

feature -- Status Report

	is_buy_event_type_1: BOOLEAN is
			-- Purchase of security (purchase of long shares of an instrument)?
		do
			Result := (event_type = buy_event_type_1)
		ensure
			is_code_buy_event_type_1: Result = (event_type = buy_event_type_1)
		end

	is_sell_event_type_2: BOOLEAN is
			-- Sale of securities (sales of long shares of an instrument)?
		do
			Result := (event_type = sell_event_type_2)
		ensure
			is_code_sell_event_type_2: Result = (event_type = sell_event_type_2)
		end

	is_inkind_event_type_3: BOOLEAN is
			-- Inkind transfer (contribution from the from the client in the form of non-cash instrument)?
		do
			Result := (event_type = inkind_event_type_3)
		ensure
			is_code_inkind_event_type_3: Result = (event_type = inkind_event_type_3)
		end

	is_split_event_type_4: BOOLEAN is
			-- Stock split (adjustment of shares held due to a forward (more shares) or reverse split (less shares))?
		do
			Result := (event_type = split_event_type_4)
		ensure
			is_code_split_event_type_4: Result = (event_type = split_event_type_4)
		end

	is_stock_dividend_event_type_5: BOOLEAN is
			-- Stock dividend (dividend paid in stock)?
		do
			Result := (event_type = stock_dividend_event_type_5)
		ensure
			is_code_stock_dividend_event_type_5: Result = (event_type = stock_dividend_event_type_5)
		end

	is_stock_distribution_event_type_6: BOOLEAN is
			-- Stock distribution (shares of an instrument gained during a reorganization)?
		do
			Result := (event_type = stock_distribution_event_type_6)
		ensure
			is_code_stock_distribution_event_type_6: Result = (event_type = stock_distribution_event_type_6)
		end

	is_dividend_reinvestment_event_type_7: BOOLEAN is
			-- Dividend reinvestment (shares of an instrument gained through reinvestment of a cash dividend)?
			-- Aka DRIP
		do
			Result := (event_type = dividend_reinvestment_event_type_7)
		ensure
			is_code_dividend_reinvestment_event_type_7: Result = (event_type = dividend_reinvestment_event_type_7)
		end

	is_inlieu_event_type_8: BOOLEAN is
			-- Partial sale of securities (the sale or cover of shares to balance rounding of a capital action)?
		do
			Result := (event_type = inlieu_event_type_8)
		ensure
			is_code_inlieu_event_type_8: Result = (event_type = inlieu_event_type_8)
		end

	is_dividend_event_type_9: BOOLEAN is
			-- Dividend (dividend paid in cash)?
		do
			Result := (event_type = dividend_event_type_9)
		ensure
			is_code_dividend_event_type_9: Result = (event_type = dividend_event_type_9)
		end

	is_delivery_event_type_10: BOOLEAN is
			-- Delivery of a security (shares of an instrument given up during a reorganization)?
		do
			Result := (event_type = delivery_event_type_10)
		ensure
			is_code_delivery_event_type_10: Result = (event_type = delivery_event_type_10)
		end

	is_contribution_event_type_11: BOOLEAN is
			-- Contribution (contribution from the client in cash)?
		do
			Result := (event_type = contribution_event_type_11)
		ensure
			is_code_contribution_event_type_11: Result = (event_type = contribution_event_type_11)
		end

	is_withdrawal_event_type_12: BOOLEAN is
			-- Withdrawal (withdrawal by the client in cash)?
		do
			Result := (event_type = withdrawal_event_type_12)
		ensure
			is_code_withdrawal_event_type_12: Result = (event_type = withdrawal_event_type_12)
		end

	is_interest_event_type_13: BOOLEAN is
			-- Interest (interest received)?
		do
			Result := (event_type = interest_event_type_13)
		ensure
			is_code_interest_event_type_13: Result = (event_type = interest_event_type_13)
		end

	is_income_event_type_14: BOOLEAN is
			-- Miscellaneous income (other investment related income received)?
		do
			Result := (event_type = income_event_type_14)
		ensure
			is_code_income_event_type_14: Result = (event_type = income_event_type_14)
		end

	is_expense_event_type_15: BOOLEAN is
			-- Miscellaneous expense (investment related expenses paid)?
		do
			Result := (event_type = expense_event_type_15)
		ensure
			is_code_expense_event_type_15: Result = (event_type = expense_event_type_15)
		end

	is_unrelated_income_event_type_16: BOOLEAN is
			-- Unrelated income (other income unrelated to investment)?
		do
			Result := (event_type = unrelated_income_event_type_16)
		ensure
			is_code_unrelated_income_event_type_16: Result = (event_type = unrelated_income_event_type_16)
		end

	is_unrelated_expense_event_type_17: BOOLEAN is
			-- Unrelated expense (expenses unrelated to investment)?
		do
			Result := (event_type = unrelated_expense_event_type_17)
		ensure
			is_code_unrelated_expense_event_type_17: Result = (event_type = unrelated_expense_event_type_17)
		end

	is_cash_distribution_event_type_18: BOOLEAN is
			-- Cash distribution (cash distribution received as part of a reorganization)?
		do
			Result := (event_type = cash_distribution_event_type_18)
		ensure
			is_code_cash_distribution_event_type_18: Result = (event_type = cash_distribution_event_type_18)
		end

	is_package_trade_event_type_19: BOOLEAN is
			-- !Obsolete! Package trade (package trade)?
		do
			Result := (event_type = package_trade_event_type_19)
		ensure
			is_code_package_trade_event_type_19: Result = (event_type = package_trade_event_type_19)
		end

	is_dividend_adjustment_event_type_20: BOOLEAN is
			-- Dividend adjustment (dividend tax withholding refund)?
		do
			Result := (event_type = dividend_adjustment_event_type_20)
		ensure
			is_code_dividend_adjustment_event_type_20: Result = (event_type = dividend_adjustment_event_type_20)
		end

	is_short_event_type_21: BOOLEAN is
			-- Short sale of a security (sale of borrowed shares of an instrument)?
		do
			Result := (event_type = short_event_type_21)
		ensure
			is_code_short_event_type_21: Result = (event_type = short_event_type_21)
		end

	is_cover_event_type_22: BOOLEAN is
			-- Cover of a short sale (purchase of shares to cover borrowed shares)?
		do
			Result := (event_type = cover_event_type_22)
		ensure
			is_code_cover_event_type_22: Result = (event_type = cover_event_type_22)
		end

	is_outkind_event_type_23: BOOLEAN is
			-- Outkind transfer (withdrawal of shares of an instrument)?
		do
			Result := (event_type = outkind_event_type_23)
		ensure
			is_code_outkind_event_type_23: Result = (event_type = outkind_event_type_23)
		end

	is_cash_transfer_event_type_24: BOOLEAN is
			-- Cash transfer (transfer annd optional currency conversion of cash)?
		do
			Result := (event_type = cash_transfer_event_type_24)
		ensure
			is_code_cash_transfer_event_type_24: Result = (event_type = cash_transfer_event_type_24)
		end

	is_bhedge_event_type_25: BOOLEAN is
			-- Forex forward contract (open forward currency hedge for B class share-holders)?
		do
			Result := (event_type = bhedge_event_type_25)
		ensure
			is_code_bhedge_event_type_25: Result = (event_type = bhedge_event_type_25)
		end

	is_bclose_event_type_26: BOOLEAN is
			-- Close fx forward contract (close forward currency hedge for B class share-holders)?
		do
			Result := (event_type = bclose_event_type_26)
		ensure
			is_code_bclose_event_type_26: Result = (event_type = bclose_event_type_26)
		end

	is_forward_event_type_27: BOOLEAN is
			-- Forex forward contract (open forward currency hedge for A class share-holders)?
		do
			Result := (event_type = forward_event_type_27)
		ensure
			is_code_forward_event_type_27: Result = (event_type = forward_event_type_27)
		end

	is_fclose_event_type_28: BOOLEAN is
			-- Closed FX Forward contract (close forward currency heldge for A class share-holders)?
		do
			Result := (event_type = fclose_event_type_28)
		ensure
			is_code_fclose_event_type_28: Result = (event_type = fclose_event_type_28)
		end

	is_short_dividend_event_type_29: BOOLEAN is
			-- !Obsolete! short dividends ()?
		do
			Result := (event_type = short_dividend_event_type_29)
		ensure
			is_code_short_dividend_event_type_29: Result = (event_type = short_dividend_event_type_29)
		end

	is_tender_event_type_30: BOOLEAN is
			-- Cash payment from tender offer (sell shares of an instrument through a tender offer)?
		do
			Result := (event_type = tender_event_type_30)
		ensure
			is_code_tender_event_type_30: Result = (event_type = tender_event_type_30)
		end

	is_dividend_withdrawn_event_type_31: BOOLEAN is
			-- Dividends swept from account (dividend paid in cash and withdrawn)?
		do
			Result := (event_type = dividend_withdrawn_event_type_31)
		ensure
			is_code_dividend_withdrawn_event_type_31: Result = (event_type = dividend_withdrawn_event_type_31)
		end

	is_income_withdrawn_event_type_32: BOOLEAN is
			-- Income swept from account (other investment related income received with tax withheld)?
		do
			Result := (event_type = income_withdrawn_event_type_32)
		ensure
			is_code_income_withdrawn_event_type_32: Result = (event_type = income_withdrawn_event_type_32)
		end

	is_dividend_taxed_event_type_33: BOOLEAN is
			-- Dividend net of withholding tax (dividend paid in cash iwth tax withheld)?
		do
			Result := (event_type = dividend_taxed_event_type_33)
		ensure
			is_code_dividend_taxed_event_type_33: Result = (event_type = dividend_taxed_event_type_33)
		end

	is_mark_to_market_event_type_34: BOOLEAN is
			-- Mark to market on securities (cash transfer to blance Mark-To-Market for futures contracts)?
		do
			Result := (event_type = mark_to_market_event_type_34)
		ensure
			is_code_mark_to_market_event_type_34: Result = (event_type = mark_to_market_event_type_34)
		end

	is_right_event_type_35: BOOLEAN is
			-- Rights issues (shares received as a result of a rights offering)?
		do
			Result := (event_type = right_event_type_35)
		ensure
			is_code_right_event_type_35: Result = (event_type = right_event_type_35)
		end

	is_lending_event_type_36: BOOLEAN is
			-- !Obsolete! Stock lending (instrument lent or returned)?
		do
			Result := (event_type = lending_event_type_36)
		ensure
			is_code_lending_event_type_36: Result = (event_type = lending_event_type_36)
		end

	is_collateral_event_type_37: BOOLEAN is
			-- Declare holding is used a collateral for future contract?
		do
			Result := (event_type = collateral_event_type_37)
		ensure
			is_code_collateral_event_type_37: Result = (event_type = collateral_event_type_37)
		end

	is_breal_event_type_38: BOOLEAN is
			-- Hedge gain/loss (realized gain or loss on forward currency contract for B class share-holder)?
		do
			Result := (event_type = breal_event_type_38)
		ensure
			is_code_breal_event_type_38: Result = (event_type = breal_event_type_38)
		end

	is_freal_event_type_39: BOOLEAN is
			-- Hedge gain/loss (realized gain or loss on forward currency contract for A class share-holder)?
		do
			Result := (event_type = freal_event_type_39)
		ensure
			is_code_freal_event_type_39: Result = (event_type = freal_event_type_39)
		end

	is_cash_trn_event_type_40: BOOLEAN is
			-- Forex spot trade (currency conversion spot trade)?
		do
			Result := (event_type = cash_trn_event_type_40)
		ensure
			is_code_cash_trn_event_type_40: Result = (event_type = cash_trn_event_type_40)
		end

	is_internal_cash_contribution_event_type_41: BOOLEAN is
			-- Internal cash transfer contribution (transfer in of cash between of portfolios within an account)?
		do
			Result := (event_type = internal_cash_contribution_event_type_41)
		ensure
			is_code_internal_cash_contribution_event_type_41: Result = (event_type = internal_cash_contribution_event_type_41)
		end

	is_internal_cash_withdrawal_event_type_42: BOOLEAN is
			-- Internal cash transfer withdrawal (transfer out of cash between portfolios within an account)?
		do
			Result := (event_type = internal_cash_withdrawal_event_type_42)
		ensure
			is_code_internal_cash_withdrawal_event_type_42: Result = (event_type = internal_cash_withdrawal_event_type_42)
		end

	is_redenomination_event_type_43: BOOLEAN is
			-- Currency redenomination (change of denomination currency for a holding (adjusting cost))?
		do
			Result := (event_type = redenomination_event_type_43)
		ensure
			is_code_redenomination_event_type_43: Result = (event_type = redenomination_event_type_43)
		end

	is_fixed_deposit_event_type_44: BOOLEAN is
			-- Deposit/withdrawal from a fixed deposit (transfer of cash to/from a fixed deposit)?
		do
			Result := (event_type = fixed_deposit_event_type_44)
		ensure
			is_code_fixed_deposit_event_type_44: Result = (event_type = fixed_deposit_event_type_44)
		end

	is_fixed_deposit_interest_event_type_45: BOOLEAN is
			-- Accrual interest from fixed deposit (accrued interest received on a bond holding)?
		do
			Result := (event_type = fixed_deposit_interest_event_type_45)
		ensure
			is_code_fixed_deposit_interest_event_type_45: Result = (event_type = fixed_deposit_interest_event_type_45)
		end

	is_coupon_payment_event_type_46: BOOLEAN is
			-- Coupon payment (coupon payment received on a bond holding)?
		do
			Result := (event_type = coupon_payment_event_type_46)
		ensure
			is_code_coupon_payment_event_type_46: Result = (event_type = coupon_payment_event_type_46)
		end

	is_bank_transfer_event_type_47: BOOLEAN is
			-- Cash transfer between banks (transfer of cash between banks)?
		do
			Result := (event_type = bank_transfer_event_type_47)
		ensure
			is_code_bank_transfer_event_type_47: Result = (event_type = bank_transfer_event_type_47)
		end

	is_management_fee_event_type_48: BOOLEAN is
			-- Management fee (management fee paid)?
		do
			Result := (event_type = management_fee_event_type_48)
		ensure
			is_code_management_fee_event_type_48: Result = (event_type = management_fee_event_type_48)
		end

	is_received_event_type_49: BOOLEAN is
			-- Inkind of a receivable (contration of an instrument which is a receivable)?
		do
			Result := (event_type = received_event_type_49)
		ensure
			is_code_received_event_type_49: Result = (event_type = received_event_type_49)
		end

	is_swap_event_type_50: BOOLEAN is
			-- Swap	(interest rate swap)?
		do
			Result := (event_type = swap_event_type_50)
		ensure
			is_code_swap_event_type_50: Result = (event_type = swap_event_type_50)
		end

	is_tmploan_event_type_51: BOOLEAN is
			-- Internal loan transfer (transfer of cash related to a temporary loan)?
		do
			Result := (event_type = tmploan_event_type_51)
		ensure
			is_code_tmploan_event_type_51: Result = (event_type = tmploan_event_type_51)
		end

	is_realgl_event_type_52, is_max_event_type: BOOLEAN is
			-- Intraday sell to adjust gain/loss (realize gain loss through a virtual sell/buy)?
		do
			Result := (event_type = realgl_event_type_52)
		ensure
			is_code_realgl_event_type_52: Result = (event_type = realgl_event_type_52)
		end

feature -- Status Report (derived)

	is_cash_dividend: BOOLEAN is
			-- Can event type represent a transaction for a cash dividend
		do
			Result := is_dividend_adjustment_event_type_20 or is_dividend_withdrawn_event_type_31 or is_dividend_taxed_event_type_33
		end

end
