note

	description:

		"our accounting for e.g. management fees"

	author: "mk"
	last_editor: "$Author: c4 $"
	date: "$Date: 2015/04/09 $"
	revision: "$Revision: #15 $"
	path: "$File: //rose/source/kernel/expert_system/rp_financial_system.e $"

class RP_FINANCIAL_SYSTEM

inherit

	RP_INVESTMENT_MANAGER_CLIENT

	RP_MESSAGE_FACTORY[RP_FINANCIAL_MESSAGE]

	RP_CONTRACT_SYSTEM

create {RP_INVESTMENT_MANAGER}
	make

feature -- Query

	base_fees (a_account: RP_INVESTMENT_ACCOUNT; a_calendar: RP_CALENDAR): !DS_HASH_TABLE [RP_MONIES, ROSE_DATE] is
			-- Retrieve quartely base fees information for `a_account'
		do
			create Result.make_equal (17)
			retrieve_base_fees (False, Result, a_account, a_calendar)
		end

	performance_fees (a_account: RP_INVESTMENT_ACCOUNT; a_calendar: RP_CALENDAR): !DS_HASH_TABLE [RP_MONIES, ROSE_DATE] is
			-- Retrieve performance fees information for `a_account'
		do
			create Result.make_equal (17)
			retrieve_performance_fees (False, Result, a_account, a_calendar)
		end

	base_fees_estimates (a_account: RP_INVESTMENT_ACCOUNT; a_calendar: RP_CALENDAR): !DS_HASH_SET [ROSE_DATE] is
			-- Retrieve base fees information for `a_account'
		local
			l_fees: like base_fees
		do
			create l_fees.make_equal (17)
			create Result.make_equal (17)
			retrieve_base_fees (True, l_fees, a_account, a_calendar)
			l_fees.keys.do_all (agent Result.force (?))
		end

	performance_fees_estimates (a_account: RP_INVESTMENT_ACCOUNT; a_calendar: RP_CALENDAR): !DS_HASH_SET [ROSE_DATE] is
			-- Retrieve performance fees information for `a_account'
		local
			l_fees: like performance_fees
		do
			create l_fees.make_equal (17)
			create Result.make_equal (17)
			retrieve_performance_fees (True, l_fees, a_account, a_calendar)
			l_fees.keys.do_all (agent Result.force (?))
		end

	base_and_performance_fees (a_account: RP_INVESTMENT_ACCOUNT; a_calendar: RP_CALENDAR): !DS_HASH_TABLE [RP_MONIES, ROSE_DATE] is
			-- Retrieve quartely base and performance fees information for `a_account'
		do
			create Result.make_equal (17)
			retrieve_base_fees (False, Result, a_account, a_calendar)
			retrieve_performance_fees (False, Result, a_account, a_calendar)
		end

feature -- Basic Operations

	next_message : RP_FINANCIAL_MESSAGE is
			-- Next message
		do
			-- to_be_implemented("")
		end


feature {NONE} -- Implementation

	contract_template: RP_FINANCIAL_CONTRACT is
			-- Anchor template
		do
		end

	invoicer_connection: DATA_SERVER is
			-- Connection to invoicer database
		once
			Result := (create {DATABASE_ACCESS}).make_from_repository_fast_load ("Invoicing_PROD")
			Result.read_only
		end

	retrieve_base_fees (a_estimate: BOOLEAN; a_hash: like base_fees; a_account: RP_INVESTMENT_ACCOUNT; a_calendar: RP_CALENDAR) is
			-- Retrieve account base fees
		local
			l_query: STRING_PREDICATE
			l_usd: RP_CURRENCY
			l_start: ROSE_DATE
		do
			l_start := a_account.first_performance_date
				-- XXX USD hard coded atm
			l_usd := manager.reference_system.currency_from_iso_code ("USD")
			if a_estimate then
				create l_query.make (invoicer_connection, "select cast (datepart(year,applicable_from)*10000+datepart(month,applicable_from)*100+datepart(day,applicable_from) as varchar(50)) + '|' + cast (datepart(year,applicable_to)*10000+datepart(month,applicable_to)*100+datepart(day,applicable_to) as varchar(50)) + '|' + cast (base_fee as varchar(50)) from mgt_fees where base_fee <> 0 and estimate <> '' and account =?")
			else
				create l_query.make (invoicer_connection, "select cast (datepart(year,applicable_from)*10000+datepart(month,applicable_from)*100+datepart(day,applicable_from) as varchar(50)) + '|' + cast (datepart(year,applicable_to)*10000+datepart(month,applicable_to)*100+datepart(day,applicable_to) as varchar(50)) + '|' + cast (base_fee as varchar(50)) from mgt_fees where base_fee <> 0 and account =?")
			end
			l_query.retrieve (Void, <<a_account.name>>)
			l_query.data_server.commit
			l_query.array.do_all (agent parse_fee_record (a_hash, ?, l_usd, a_calendar, l_start))
		end

	retrieve_performance_fees (a_estimate: BOOLEAN; a_hash: like performance_fees; a_account: RP_INVESTMENT_ACCOUNT; a_calendar: RP_CALENDAR) is
			-- Retrieve portfolio fee records
		local
			l_query: STRING_PREDICATE
			l_usd: RP_CURRENCY
			l_start: ROSE_DATE
		do
			l_start := a_account.first_performance_date
				-- XXX USD hard coded atm
			l_usd := manager.reference_system.currency_from_iso_code ("USD")
			if a_estimate then
				create l_query.make (invoicer_connection, "select cast (datepart(year,applicable_from)*10000+datepart(month,applicable_from)*100+datepart(day,applicable_from) as varchar(50)) + '|' + cast (datepart(year,applicable_to)*10000+datepart(month,applicable_to)*100+datepart(day,applicable_to) as varchar(50)) + '|' + cast (perf_fee as varchar(50)) from mgt_fees where perf_fee <> 0 and estimate <> '' and account =?")
			else
				create l_query.make (invoicer_connection, "select cast (datepart(year,applicable_from)*10000+datepart(month,applicable_from)*100+datepart(day,applicable_from) as varchar(50)) + '|' + cast (datepart(year,applicable_to)*10000+datepart(month,applicable_to)*100+datepart(day,applicable_to) as varchar(50)) + '|' + cast (perf_fee as varchar(50)) from mgt_fees where perf_fee <> 0 and account =?")
			end
			l_query.retrieve (Void, <<a_account.name>>)
			l_query.data_server.commit
			l_query.array.do_all (agent parse_fee_record (a_hash, ?, l_usd, a_calendar, l_start))
		end

	parse_fee_record (a_hash: like performance_fees; s: STRING; a_currency: RP_CURRENCY; a_calendar: RP_CALENDAR; a_start: ROSE_DATE) is
			-- Parse fee record
		local
			l_tokenizer: ROSE_DELIMITED_TEXT
			l_start, l_finish: ROSE_DATE
			l_fee, l_monthly_fee: RP_MONIES
			l_no_of_months: INTEGER
		do
			create l_tokenizer.make (s, '|')
			create l_start.make_ccyymmdd (l_tokenizer.item (1).to_integer)
			create l_finish.make_ccyymmdd (l_tokenizer.item (2).to_integer)
			l_start := l_start.max (a_start).month_end.min (l_finish)
			l_fee := a_currency.monies (l_tokenizer.item (3))
			from
					-- minumum one month
				l_no_of_months := 1
			until
				l_start.ccyymmdd >= l_finish.ccyymmdd
			loop
				l_no_of_months := l_no_of_months + 1
				l_start := l_start.next_date.month_end.min (l_finish)
			end
			from
				l_monthly_fee := a_currency.monies ((l_fee.amount / l_no_of_months).rounded (2))
				create l_start.make_ccyymmdd (l_tokenizer.item (1).to_integer)
				l_start := l_start.max (a_start).month_end.min (l_finish)
			until
				l_fee < l_monthly_fee or l_start.ccyymmdd >= l_finish.ccyymmdd
			loop
				add_monthly_fee (a_hash,  a_calendar.last_work_day (l_start), l_monthly_fee)
				l_start := l_start.next_date.month_end.min (l_finish)
				l_fee := l_fee - l_monthly_fee
			end
			if l_fee.amount > 0 then
				add_monthly_fee (a_hash, a_calendar.last_work_day (l_start), l_fee)
			end
		end

	add_monthly_fee (a_hash: like base_fees; a_date: ROSE_DATE; a_fee: RP_MONIES) is
			-- Append quartely fee to `a_hash'
		do
			a_hash.search (a_date)
			if a_hash.found then
				a_hash.force (a_hash.found_item + a_fee, a_date)
			else
				a_hash.force (a_fee, a_date)
			end
		end

end
