note

	description:

		"Convert GEDB event to Contracts and Flows"

	author: "Sami Ben-Grine"
	last_editor: "$Author: e2 $"
	date: "$Date: 2012/07/25 $"
	revision: "$Revision: #8 $"
	path: "$File: //rose/source/Kernel/expert_system/legacy/contracts/rp_gedb_delivery_converter.e $"

class RP_GEDB_DELIVERY_CONVERTER

inherit

	RP_GEDB_EVENT_TO_CONTRACT_SIMPLE_CONVERTER [RP_GEDB_DELIVERY, RP_DELIVERY]
	RP_AXAROSENBERG_AWARE
		export
			{NONE} all
		end
		elog_shared_logger
create

	make

feature -- Access

	contract: !RP_DELIVERY is
			-- <Precursor>
		local
			l_cost: RP_MONIES
			l_instructions: ROSE_LINEAR_ARRAY [RP_CORPORATE_ACTION_PROCESSING_INSTRUCTIONS]
		do
			if units.amount > 0 then
				if manager.accounting.is_forwarding_contracts_to_cost_accounting_system then
					l_cost := manager.cost_accounting_system.short_side_basis (portfolio, instrument).
						total_cost (event.currency, Manager.market_system.currency_market_data_sources.item (1))
				else
					l_cost := performance_effect_with_sign
				end
				create Result.make_short_position (units_base, units, l_cost, event_date, settle_date, portfolio, identifier, system, custody)
			else
				if manager.accounting.is_forwarding_contracts_to_cost_accounting_system then
					l_cost := manager.cost_accounting_system.long_side_basis (portfolio, instrument).
						total_cost (event.currency, Manager.market_system.currency_market_data_sources.item (1))
				else
					l_cost := performance_effect_with_sign
				end
				create Result.make_long_position (units_base, units, l_cost, event_date, settle_date, portfolio, identifier, system, custody)
			end

			if manager.accounting.is_forwarding_contracts_to_cost_accounting_system then
				-- Get the type 6 event(s) corresponding to this one (could be 0, 1 or 2)
				-- then save them in manager.cost_accounting_system
				manager.corporate_action_system.clear_caches
				-- check that event date.previous weekday is fine (or use calendar): see with sql query
				manager.corporate_action_system.generate_corp_actions_for_listing (instrument, event_date.previous_weekday, event_date)
				l_instructions := manager.corporate_action_system.processing_instructions (instrument, event_date)
				if l_instructions /= Void then
					l_instructions.do_all (agent process_instructions (?, l_cost))
				end
			end
		end

feature {NONE} -- Implementation

	process_instructions (a_instructions: RP_CORPORATE_ACTION_PROCESSING_INSTRUCTIONS; a_cost: RP_MONIES) is
			-- Process `a_instructions' to link 10 and 6
		require
			a_instructions_not_void: a_instructions /= Void
			a_cost_not_void: a_cost /= Void
		do
			a_instructions.corporate_actions.do_all (agent process_corporate_action (?, a_cost))
		end

	process_corporate_action (a_corp_act: RP_CORPORATE_ACTION; a_cost: RP_MONIES) is
			-- Memorize `a_delivery' with a_corporate action
		require
			a_corp_act_not_void: a_corp_act /= Void
			a_cost_not_void: a_cost /= Void
		local
			l_subscription_currency: RP_CURRENCY
		do
			if attached {RP_CORPORATE_ACTION_ELECTION_UNITS_DISTRIBUTION} a_corp_act.election as l_distribution then
				manager.corporate_action_system.memorize_distribution_cost (l_distribution.child_instrument, portfolio, event_date, units.amount.abs, a_cost.currency.monies (a_cost.amount * l_distribution.capact.legacy_basis_prop))
			elseif attached {RP_CORPORATE_ACTION_ELECTION_DELIVERY_DISTRIBUTION} a_corp_act.election as l_delivery then
				manager.corporate_action_system.memorize_distribution_cost (l_delivery.child_instrument, portfolio, event_date, units.amount.abs, a_cost.currency.monies (a_cost.amount * l_delivery.capact.legacy_basis_prop))
			elseif attached {RP_CORPORATE_ACTION_ELECTION_RENOUNCEABLE_RIGHTS_IN_SAME} a_corp_act.election as l_same and then
				l_same.child_instrument /= Void
			then
				l_subscription_currency := manager.reference_system.currency_from_iso_code (l_same.capact.subscription_currency_iso)
				manager.corporate_action_system.memorize_distribution_cost (l_same.child_instrument, portfolio, event_date, units.amount.abs, l_subscription_currency.monies (l_same.capact.subscription_price * l_same.capact.new_shares_rate * units.amount))
				manager.corporate_action_system.memorize_distribution_cost (instrument, portfolio, event_date, units.amount.abs, a_cost.currency.monies (a_cost.amount * l_same.capact.parent_basis_prop))
			elseif attached {RP_CORPORATE_ACTION_ELECTION_RENOUNCEABLE_BONUS_IN_SAME} a_corp_act.election as l_same and then
				l_same.child_instrument /= Void
			then
				l_subscription_currency := manager.reference_system.currency_from_iso_code (l_same.capact.parent_price_iso)
				manager.corporate_action_system.memorize_distribution_cost (l_same.child_instrument, portfolio, event_date, units.amount.abs, l_subscription_currency.monies (l_same.capact.parent_price_saved * l_same.capact.new_shares_rate * units.amount))
				manager.corporate_action_system.memorize_distribution_cost (instrument, portfolio, event_date, units.amount.abs, a_cost.currency.monies (a_cost.amount * l_same.capact.parent_basis_prop))
			end
		end

end
