note

	description:

		"Compute Gross and Tax amounts for GEDB record"

	author: "Ivan Tarnavsky"
	last_editor: "$Author: it $"
	date: "$Date: 2011/03/31 $"
	revision: "$Revision: #14 $"
	path: "$File: //rose/source/kernel/expert_system/legacy/contracts/rp_gedb_dividend_withholding_tax_gross_up.e $"

deferred class RP_GEDB_DIVIDEND_WITHHOLDING_TAX_GROSS_UP

inherit

	ANY

	RP_EVENT_TYPE_CONSTANTS
		export {NONE} all end

	RP_WITHHOLDING_TAX_GROSS_UP
		export {NONE} all end

	RP_PERFORMANCE_CONSTANTS_SHARED
		export {NONE} all end

feature -- Settings

	enable_roundtrip_calculation is
			-- Enable net->gross->net calculation
		do
			is_roundtrip_calculation_enabled := True
		ensure
			definition: is_roundtrip_calculation_enabled
		end

	disable_roundtrip_calculation is
			-- Enable net->gross->net calculation
		do
			is_roundtrip_calculation_enabled := False
		ensure
			definition: not is_roundtrip_calculation_enabled
		end

feature -- Access

	is_roundtrip_calculation_enabled: BOOLEAN
			-- Is net->gross->net calculation enabled?

feature {NONE} -- Implementation

	system: RP_CONTRACT_SYSTEM is
			-- Generating system
		deferred
		end

	process_dividend_net_of_withholding_tax (a_event: RP_GEDB_EVENT): TUPLE [gross: RP_MONIES; tax: RP_MONIES] is
			-- Process `a_event' as dividend net of any withholding tax
		local
			l_backdiv_record: RP_BACKDIV_RECORD
			l_gross_amount: RP_MONIES
			l_stock: RP_STOCK
			l_net_amount: RP_MONIES
			l_force_rate: BOOLEAN
		do
			if a_event.portfolio.zone.is_united_states and a_event.is_dividend_taxed_event_type_33 then
				l_force_rate := True
				l_net_amount := a_event.cash_effect_with_sign + a_event.secondary_cash_effect
			else
				l_net_amount := a_event.cash_effect_with_sign
			end
			if a_event.portfolio.code.is_equal ("POST") then
				l_gross_amount := a_event.cash_effect_with_sign
			elseif a_event.portfolio.code.is_equal ("SHINE") and a_event.instrument.brimsym.substring (1, 2).is_equal ("KR") and a_event.catagory.has_substring ("ANNDX") then
				l_gross_amount := a_event.performance_effect_with_sign
			else
				if not a_event.units_base.amount.is_zero then
					l_stock ?= a_event.instrument
					if l_stock /= Void then
						l_backdiv_record := system.manager.legacy.match (a_event)
					end
				end
				l_gross_amount := calculate_gross (l_force_rate, a_event, l_net_amount, l_backdiv_record)
				if a_event.portfolio.zone.is_japan and is_roundtrip_calculation_enabled then
					l_net_amount := calculate_net (a_event, l_gross_amount, l_backdiv_record)
						-- check for rounding difference
					if (l_net_amount.amount - a_event.cash_effect_with_sign.amount).abs > 1 then
							-- Do not change; differences not due to rounding
						l_net_amount := a_event.cash_effect_with_sign
					end
				end
			end
			Result := legacy_tax_flows (a_event, l_gross_amount, l_net_amount)
		end

	legacy_tax_flows (a_event: RP_GEDB_EVENT; a_gross_dividend, a_net_amount: RP_MONIES): like process_dividend_net_of_withholding_tax is
			-- Tax part of dividend
		require
			a_event_not_void: a_event /= Void
			a_gross_dividend_not_void: a_gross_dividend /= Void
			a_net_amount_not_void: a_net_amount /= Void
		local
			l_gross: RP_MONIES
			l_net: RP_MONIES
		do
			if a_event.portfolio.zone.is_japan and is_roundtrip_calculation_enabled then
				l_gross := a_gross_dividend
				l_net := a_net_amount
				if (l_gross.amount < 0 or l_net.amount < 0) and a_event.event_date >= constants.jzone_first_short_dividend_date then
					l_gross := l_net
				elseif a_event.units_base.sign >= 0 and l_gross.amount < l_net.amount then
					l_gross := l_net
				elseif a_event.units_base.sign < 0 and l_gross.amount > l_net.amount then
					l_gross := l_net
				end
				Result := [l_gross, l_gross - l_net]
			else
				Result := [a_gross_dividend, a_gross_dividend - a_net_amount]
			end
		ensure
			tuple_not_void: Result /= Void
			two_items: Result.count = 2
			non_void_items: Result.at (1) /= Void and Result.at (2) /= Void
		end

	calculate_gross (a_force_client_rate: BOOLEAN; a_event: RP_GEDB_EVENT; a_net: RP_MONIES a_backdiv_record: RP_BACKDIV_RECORD): RP_MONIES is
			-- Gross amount
		require
			a_event_not_void: a_event /= Void
			a_net_not_void: a_net /= Void
			a_force_client_rate_allowed_only_for_u_zone_portfolios: a_force_client_rate implies a_event.portfolio.zone.is_united_states
			backdiv_record_may_be_void: True
		do
			if a_backdiv_record /= Void then
				Result := gross_amount (a_force_client_rate, system.manager.reference_system, a_event.portfolio, a_event.instrument, a_event.event_date, a_event.units_base, a_net, a_backdiv_record.taxbas, a_event.performance_effect_with_sign, a_backdiv_record.cash_rate, a_backdiv_record.currency, a_backdiv_record.ex_date)
			else
				Result := gross_amount (a_force_client_rate, system.manager.reference_system, a_event.portfolio, a_event.instrument, a_event.event_date, a_event.units_base, a_net, Void, a_event.performance_effect_with_sign, Void, Void, Void)
			end
		end

	calculate_net (a_event: RP_GEDB_EVENT; a_gross: RP_MONIES; a_backdiv_record: RP_BACKDIV_RECORD): RP_MONIES is
			-- Net amount
		require
			a_event_not_void: a_event /= Void
			a_gross_not_void: a_gross /= Void
			backdiv_record_may_be_void: True
		do
			if a_backdiv_record /= Void then
				Result := net_amount (system.manager.corporate_action_processing_system.use_rate_overrides_for_cash_dividends, system.manager.reference_system, a_event.portfolio, a_event.instrument, a_event.event_date, a_event.units_base, a_event.cash_effect_with_sign, a_backdiv_record.taxbas, a_gross, a_backdiv_record.cash_rate, a_backdiv_record.currency, a_backdiv_record.ex_date)
			else
				Result := net_amount (system.manager.corporate_action_processing_system.use_rate_overrides_for_cash_dividends, system.manager.reference_system, a_event.portfolio, a_event.instrument, a_event.event_date, a_event.units_base, a_event.cash_effect_with_sign, Void, a_gross, Void, Void, Void)
			end
		end

end
