indexing

	description:

		"Factory for gedb_events"

	date: "$Date: 2010/03/18 $"
	fileid: "$Id: //rose/source/kernel/expert_system/aflows/rp_gedb_event_factory.e#24 $"

class RP_GEDB_EVENT_FACTORY

inherit

	RP_INVESTMENT_MANAGER_CLIENT

	RP_EVENT_TYPE_CONSTANTS
		export {NONE} all end

create {RP_LEGACY_SYSTEM}

	make

feature {RP_LEGACY_SYSTEM} -- Factory

	gedb_event_from_record (a_gedb_record: RP_GEDB_RECORD): RP_GEDB_EVENT is
			-- create gedb_event fro a_gedb_record
		do
			inspect a_gedb_record.event_type
			when buy_event_type_1 then
					-- buy
				if manager.reference_system.valid_index_future_brimsym (a_gedb_record.instrument) then
					Result := create {RP_GEDB_FUTURES_BUY}.make (manager, a_gedb_record)
				else
					Result := create {RP_GEDB_BUY}.make (manager, a_gedb_record)
				end
			when sell_event_type_2 then
					-- sell
				if manager.reference_system.valid_index_future_brimsym (a_gedb_record.instrument) then
					Result := create {RP_GEDB_FUTURES_SELL}.make (manager, a_gedb_record)
				else
					Result := create {RP_GEDB_SELL}.make (manager, a_gedb_record)
				end
			when inkind_event_type_3 then
					-- inkind, inkinds used for conversion & inkinds used for adjustments
				Result := create {RP_GEDB_INKIND}.make (manager, a_gedb_record)
			when split_event_type_4 then
					-- split
				Result := create {RP_GEDB_SPLIT}.make (manager, a_gedb_record)
			when stock_dividend_event_type_5 then
					-- stock dividend
				Result := create {RP_GEDB_STOCK_DIVIDEND}.make (manager, a_gedb_record)
			when stock_distribution_event_type_6 then
					-- distribution
				Result := create {RP_GEDB_DISTRIBUTION}.make (manager, a_gedb_record)
			when dividend_reinvestment_event_type_7 then
					-- dividend reinvestment
				Result := create {RP_GEDB_DIVIDEND_REINVESTED}.make (manager, a_gedb_record)
			when inlieu_event_type_8 then
					-- inlieu
				Result := create {RP_GEDB_INLIEU}.make (manager, a_gedb_record)
			when dividend_event_type_9 then
				if a_gedb_record.event_details.as_upper.has_substring ("ROC") then
						-- return of capital
					Result := create {RP_GEDB_RETURN_OF_CAPITAL}.make (manager, a_gedb_record)
				else
						-- dividend
					Result := create {RP_GEDB_DIVIDEND}.make (manager, a_gedb_record)
				end
			when delivery_event_type_10 then
					-- delivery
				Result := create {RP_GEDB_DELIVERY}.make (manager, a_gedb_record)
			when contribution_event_type_11 then
					-- contribution
				Result := create {RP_GEDB_CONTRIBUTION}.make (manager, a_gedb_record)
			when withdrawal_event_type_12 then
					-- withdrawal
				Result := create {RP_GEDB_WITHDRAWAL}.make (manager, a_gedb_record)
			when interest_event_type_13 then
					-- interest
				Result := create {RP_GEDB_INTEREST}.make (manager, a_gedb_record)
			when income_event_type_14 then
					-- income
				if a_gedb_record.cash_effect >= 0 then
					Result := create {RP_GEDB_INCOME}.make (manager, a_gedb_record)
				else
					Result := create {RP_GEDB_INCOME_ADJUSTMENT}.make (manager, a_gedb_record)
				end
			when expense_event_type_15 then
					-- expense
				if a_gedb_record.cash_effect <= 0 then
					Result := create {RP_GEDB_EXPENSE}.make (manager, a_gedb_record)
				else
					Result := create {RP_GEDB_EXPENSE_ADJUSTMENT}.make (manager, a_gedb_record)
				end
			when unrelated_income_event_type_16 then
					-- income unrelated to investment
				if a_gedb_record.cash_effect >= 0 then
					Result := create {RP_GEDB_INCOME_UNRELATED}.make (manager, a_gedb_record)
				else
					Result := create {RP_GEDB_INCOME_UNRELATED_ADJUSTMENT}.make (manager, a_gedb_record)
				end
			when unrelated_expense_event_type_17 then
					-- expense unrelated to investment
				if a_gedb_record.cash_effect <= 0 then
					Result := create {RP_GEDB_EXPENSE_UNRELATED}.make (manager, a_gedb_record)
				else
					Result := create {RP_GEDB_EXPENSE_UNRELATED_ADJUSTMENT}.make (manager, a_gedb_record)
				end
			when cash_distribution_event_type_18 then
					-- cash distribution
				Result := create {RP_GEDB_CASH_DISTRIBUTION}.make (manager, a_gedb_record)
			when dividend_adjustment_event_type_20 then
					-- dividend adjustment...in O zone this can be a capital action contract
				if a_gedb_record.catagory.has_substring ("ANNDX") then
					if a_gedb_record.portfolio.has_substring("SHINE") and not a_gedb_record.instrument.substring (1, 2).is_equal ("KR") then
						Result := create {RP_GEDB_DIVIDEND_TAX_REFUND}.make (manager, a_gedb_record)
					else
						Result := create {RP_GEDB_DIVIDEND_NET_OF_WITHHOLDING_TAX}.make (manager, a_gedb_record)
					end
				else
					Result := create {RP_GEDB_DIVIDEND_TAX_REFUND}.make (manager, a_gedb_record)
				end
			when short_event_type_21 then
					-- short
				if manager.reference_system.valid_index_future_brimsym (a_gedb_record.instrument) then
					Result := create {RP_GEDB_FUTURES_SHORT}.make (manager, a_gedb_record)
				else
					Result := create {RP_GEDB_SHORT}.make (manager, a_gedb_record)
				end
			when cover_event_type_22 then
					-- cover and cover as short inlieu in U zone when units < 1
				if manager.reference_system.valid_index_future_brimsym (a_gedb_record.instrument) then
					Result := create {RP_GEDB_FUTURES_COVER}.make (manager, a_gedb_record)
				else
					Result := create {RP_GEDB_COVER}.make (manager, a_gedb_record)
				end
			when outkind_event_type_23 then
					-- outkind, outkind used for adjustment
				Result := create {RP_GEDB_OUTKIND}.make (manager, a_gedb_record)
			when cash_transfer_event_type_24 then
					-- xzone ... if it is actually an fx
				Result := create {RP_GEDB_CASH_ZONAL_TRANSFER}.make (manager, a_gedb_record)
			when bhedge_event_type_25 then
					-- bhedge fx forward
				Result := create {RP_GEDB_FFX_OPEN}.make (manager, a_gedb_record)
			when bclose_event_type_26 then
					-- bclose closed fx forward
				Result := create {RP_GEDB_FFX_CLOSE}.make (manager, a_gedb_record)
			when forward_event_type_27 then
					-- forward fx forward
				Result := create {RP_GEDB_FFX_OPEN}.make (manager, a_gedb_record)
			when fclose_event_type_28 then
					-- fclose closed fx forward
				Result := create {RP_GEDB_FFX_CLOSE}.make (manager, a_gedb_record)
			when tender_event_type_30 then
					-- tender
				Result := create {RP_GEDB_TENDER}.make (manager, a_gedb_record)
			when dividend_withdrawn_event_type_31 then
					-- dividend swept
				Result := create {RP_GEDB_DIVIDEND_SWEPT}.make (manager, a_gedb_record)
			when income_withdrawn_event_type_32 then
					-- income swept
				Result := create {RP_GEDB_INCOME_SWEPT}.make (manager, a_gedb_record)
			when dividend_taxed_event_type_33 then
				if a_gedb_record.catagory.as_upper.has_substring ("ROC") then
						-- return of capital
					Result := create {RP_GEDB_RETURN_OF_CAPITAL}.make (manager, a_gedb_record)
				else
						-- dividend net of withholding tax
					Result := create {RP_GEDB_DIVIDEND_NET_OF_WITHHOLDING_TAX}.make (manager, a_gedb_record)
				end
			when mark_to_market_event_type_34 then
					-- mark to market
				Result := create {RP_GEDB_MARK_TO_MARKET}.make (manager, a_gedb_record)
			when right_event_type_35 then
					-- rights issue
				Result := create {RP_GEDB_RIGHTS}.make (manager, a_gedb_record)
			when collateral_event_type_37 then
					-- collateral
				Result := create {RP_GEDB_COLLATERAL}.make (manager, a_gedb_record)
			when breal_event_type_38 then
					-- breal hedge gain/loss
				Result := create {RP_GEDB_FFX_GAIN_LOSS}.make (manager, a_gedb_record)
			when freal_event_type_39 then
					-- freal hedge gain/loss
				Result := create {RP_GEDB_FFX_GAIN_LOSS}.make (manager, a_gedb_record)
			when cash_trn_event_type_40 then
					-- fx spot trade
				Result := create {RP_GEDB_FX}.make (manager, a_gedb_record)
			when internal_cash_contribution_event_type_41 then
					-- intcshc internal cash transfer cont
				Result := create {RP_GEDB_INTERNAL_CASH_XFER_CONTRIBUTION}.make (manager, a_gedb_record)
			when internal_cash_withdrawal_event_type_42 then
					-- intcshw internal cash transfer with
				Result := create {RP_GEDB_INTERNAL_CASH_XFER_WITHDRAWAL}.make (manager, a_gedb_record)
			when redenomination_event_type_43 then
					-- redenomination
				Result := create {RP_GEDB_REDENOMINATION}.make (manager, a_gedb_record)
			when fixed_deposit_event_type_44 then
					-- deposit/withdrawal from a fixed deposit
				Result := create {RP_GEDB_FIXED_DEPOSIT_TRANSFER}.make (manager, a_gedb_record)
			when fixed_deposit_interest_event_type_45 then
					-- accrual from fixed deposit
				Result := create {RP_GEDB_FIXED_DEPOSIT_ACCRUAL}.make (manager, a_gedb_record)
			when coupon_payment_event_type_46 then
					-- bond coupon payment
				Result := create {RP_GEDB_BOND_COUPON_PAYMENT}.make (manager, a_gedb_record)
			when bank_transfer_event_type_47 then
					-- cash transfer between banks
				Result := create {RP_GEDB_BANK_TRANSFER}.make (manager, a_gedb_record)
			when management_fee_event_type_48 then
					-- management fee or rebate performanace related (14 or 15 in other zones)
				Result := create {RP_GEDB_MANAGEMENT_FEE}.make (manager, a_gedb_record)
			when received_event_type_49 then
					-- inkind of a receivable ... hopefully always dividends
				Result := create {RP_GEDB_RECEIVABLE_INKIND}.make (manager, a_gedb_record)
			when tmploan_event_type_51 then
					-- internal loan transfer
				Result := create {RP_GEDB_INTERNAL_LOAN_TRANSFER}.make (manager, a_gedb_record)
			when realgl_event_type_52 then
					-- intraday sell to adjust gain/loss
				Result := create {RP_GEDB_INTRADAY_SELL_TO_ADJUST_GL}.make (manager, a_gedb_record)
			end
		end

end
