note

	description:

		"Convert GEDB event to Contracts and Flows"

	author: "Portfolio Accounting Tech"
	last_editor: "$Author: e2 $"
	date: "$Date: 2011/11/29 $"
	revision: "$Revision: #7 $"
	path: "$File: //rose/source/Kernel/expert_system/legacy/contracts/rp_gedb_event_to_contract_converter.e $"

deferred class RP_GEDB_EVENT_TO_CONTRACT_CONVERTER [G -> RP_GEDB_EVENT]

inherit

	RP_FLOW_TYPE_AWARE
		export {NONE} all end

	RP_EVENT_TYPE_CONSTANTS
		export {NONE} all end

	RP_UNKNOWN_INSTRUMENT_CONTROL_AWARE
		export {NONE} all end

	REFACTORING_HELPER_AWARE
		export {NONE} all end

	ANY
			-- To export default features

feature {NONE} -- Initialization

	make (a_system: like system; a_event: like event) is
			-- Create from 'a_event' on behalf of 'a_system'
		require
			system_exists: a_system /= Void
			event_exists: a_event /= Void
		do
			system := a_system
			event := a_event
		ensure
			system_set: system = a_system
			event_set: event = a_event
		end

feature -- Access

	system: RP_GEDB_EVENT_TO_CONTRACT_FACTORY
			-- Contract system that will claim having created the contract based on the event

	event: G
			-- Legacy event used for the contract creation

	contract: !RP_PORTFOLIO_CONTRACT is
			-- Contracts view of the 'event
		deferred
		end

feature {NONE} -- Implementation

	identifier: STRING is
			-- Event identifier
		do
			Result := event.identifier
		end

	event_date: ROSE_DATE is
			-- Event date
		do
			Result := event.event_date
		end

	settle_date: ROSE_DATE is
			-- Settle date
		do
			Result := event.settle_date
		end

	portfolio: RP_PORTFOLIO is
			-- Portfolio.
		do
			Result := event.portfolio
		ensure
			non_void_result: Result /= Void
		end

	broker: RP_BROKER is
			-- Broker.
		do
			Result := event.broker
		ensure
			non_void_result: Result /= Void
		end

	instrument: RP_INSTRUMENT is
			-- Instrument.
		do
			Result := event.instrument
		ensure
			non_void_result: Result /= Void
		end

	units_base: RP_QUANTITY is
			-- Units base
		do
			Result := event.units_base
		ensure
			non_void_result: Result /= Void
		end

	units: RP_QUANTITY is
			-- Units
		do
			Result := event.units
		end

	secondary_cash_effect: RP_MONIES is
			-- Secondary cash effect
		do
			Result := event.secondary_cash_effect
		end

	cash_effect_with_sign: RP_MONIES is
			-- Cash effect
		do
			Result := event.cash_effect_with_sign
		ensure
			non_void_result: Result /= Void
		end

	performance_effect_with_sign: RP_MONIES is
			--|
		do
			Result := event.performance_effect_with_sign
		end

	commission: RP_MONIES is
			--|
		do
			Result := event.commission
		end

	fee: RP_MONIES is
			--|
		do
			Result := event.fee
		end

	custody: RP_CUSTODY is
			-- Custody of the contract
		do
			create Result.make (unknown_party, create {RP_CUSTODIAL_ACCOUNT}.make (unknown_party, event.subaccount))
		ensure
			custody_not_void: Result /= Void
		end


end
