indexing

	description:

		"Convert GEDB event to Contracts and Flows"

	date: "$Date: 2010/04/13 $"
	sccid: "$Id: //rose/source/Kernel/expert_system/legacy/contracts/rp_gedb_event_to_contract_factory.e#5 $"

class RP_GEDB_EVENT_TO_CONTRACT_FACTORY

inherit

	RP_CONTRACT_SYSTEM

feature -- Settings

	enable_dividend_rountrip_calculation is
			-- Enable net->gross->net calculatio
		do
			is_dividend_rountrip_calculation_enabled := True
		ensure
			definition: is_dividend_rountrip_calculation_enabled
		end

	disable_dividend_rountrip_calculation is
			-- Disable net->gross->net calculatio
		do
			is_dividend_rountrip_calculation_enabled := False
		ensure
			definition: not is_dividend_rountrip_calculation_enabled
		end

feature -- Access

	is_dividend_rountrip_calculation_enabled: BOOLEAN
			-- Is net->gross->net calculation enabled?

feature {NONE} -- Implementation (Corporate Action System)

	return_of_capital (a_gedb_event: RP_GEDB_RETURN_OF_CAPITAL): !RP_RETURN_OF_CAPITAL is
			-- Convert 'a_gedb_event' to contracts and flows
		do
			Result := (create {RP_GEDB_RETURN_OF_CAPITAL_CONVERTER}.make (Current, a_gedb_event)).contract
		end

	dividend_net_of_withholding_tax (a_gedb_event: RP_GEDB_DIVIDEND_NET_OF_WITHHOLDING_TAX): !RP_DIVIDEND is
			-- Convert 'a_gedb_event' to contracts and flows
		local
			l_converter: RP_GEDB_DIVIDEND_NET_OF_WITHHOLDING_TAX_CONVERTER
		do
			create l_converter.make (Current, a_gedb_event)
			if is_dividend_rountrip_calculation_enabled then
				l_converter.enable_roundtrip_calculation
			end
			Result := l_converter.contract
		end

	split (a_gedb_event: RP_GEDB_SPLIT): !RP_SPLIT is
			-- Convert 'a_gedb_event' to contracts and flows
		do
			Result := (create {RP_GEDB_SPLIT_CONVERTER}.make (Current, a_gedb_event)).contract
		end

	stock_dividend (a_gedb_event: RP_GEDB_STOCK_DIVIDEND): !RP_DIVIDEND is
			-- Convert 'a_gedb_event' to contracts and flows
		do
			Result := (create {RP_GEDB_STOCK_DIVIDEND_CONVERTER}.make (Current, a_gedb_event)).contract
		end

	distribution (a_gedb_event: RP_GEDB_DISTRIBUTION): !RP_DISTRIBUTION is
			-- Convert 'a_gedb_event' to contracts and flows
		do
			Result := (create {RP_GEDB_DISTRIBUTION_CONVERTER}.make (Current, a_gedb_event)).contract
		end

	dividend (a_gedb_event: RP_GEDB_DIVIDEND): !RP_PORTFOLIO_CONTRACT is
			-- Convert 'a_gedb_event' to contracts and flows
		do
			Result := (create {RP_GEDB_DIVIDEND_CONVERTER}.make (Current, a_gedb_event)).contract
		end

	delivery (a_gedb_event: RP_GEDB_DELIVERY): !RP_PORTFOLIO_CONTRACT is
			-- Convert 'a_gedb_event' to contracts and flows
		do
			Result := (create {RP_GEDB_DELIVERY_CONVERTER}.make (Current, a_gedb_event)).contract
		end

	cash_distribution (a_gedb_event: RP_GEDB_CASH_DISTRIBUTION): !RP_PORTFOLIO_CONTRACT is
			-- Convert 'a_gedb_event' to contracts and flows
		do
			Result := (create {RP_GEDB_CASH_DISTRIBUTION_CONVERTER}.make (Current, a_gedb_event)).contract
		end

	tender (a_gedb_event: RP_GEDB_TENDER): !RP_PORTFOLIO_CONTRACT is
			-- Convert 'a_gedb_event' to contracts and flows
		do
			Result := (create {RP_GEDB_TENDER_CONVERTER}.make (Current, a_gedb_event)).contract
		end

	rights_exercise (a_gedb_event: RP_GEDB_RIGHTS): !RP_PORTFOLIO_CONTRACT is
			-- Convert 'a_gedb_event' to contracts and flows
		do
			Result := (create {RP_GEDB_RIGHTS_CONVERTER}.make (Current, a_gedb_event)).contract
		end

	redenomination (a_gedb_event: RP_GEDB_REDENOMINATION): !RP_PORTFOLIO_CONTRACT is
			-- Convert 'a_gedb_event' to contracts and flows
		do
			Result := (create {RP_GEDB_REDENOMINATION_CONVERTER}.make (Current, a_gedb_event)).contract
		end

feature {NONE} -- Implementation (Client System)

	contribution (a_gedb_event: RP_GEDB_CONTRIBUTION): !RP_PORTFOLIO_CONTRACT is
			-- Convert 'a_gedb_event' to contracts and flows
		do
			Result := (create {RP_GEDB_CONTRIBUTION_CONVERTER}.make (Current, a_gedb_event)).contract
		end

	inkind (a_gedb_event: RP_GEDB_INKIND): !RP_PORTFOLIO_CONTRACT is
			-- Convert 'a_gedb_event' to contracts and flows
		do
			Result := (create {RP_GEDB_INKIND_CONVERTER}.make (Current, a_gedb_event)).contract
		end

	withdrawal (a_gedb_event: RP_GEDB_WITHDRAWAL): !RP_PORTFOLIO_CONTRACT is
			-- Convert 'a_gedb_event' to contracts and flows
		do
			Result := (create {RP_GEDB_WITHDRAWAL_CONVERTER}.make (Current, a_gedb_event)).contract
		end

	outkind (a_gedb_event: RP_GEDB_OUTKIND): !RP_PORTFOLIO_CONTRACT is
			-- Convert 'a_gedb_event' to contracts and flows
		do
			Result := (create {RP_GEDB_OUTKIND_CONVERTER}.make (Current, a_gedb_event)).contract
		end

feature {NONE} -- Implementaion (Custodian System)

	inlieu (a_gedb_event: RP_GEDB_INLIEU): !RP_PORTFOLIO_CONTRACT is
			-- Process inlue gedb event
		require
			non_void_gedb_event: a_gedb_event /= Void
		do
			Result := (create {RP_GEDB_INLIEU_CONVERTER}.make (Current, a_gedb_event)).contract
		ensure
			non_void_contract: Result /= Void
		end

	interest (a_gedb_event: RP_GEDB_INTEREST): !RP_PORTFOLIO_CONTRACT is
			-- Process interest gedb event
		require
			non_void_gedb_event: a_gedb_event /= Void
		do
			Result := (create {RP_GEDB_INTEREST_CONVERTER}.make (Current, a_gedb_event)).contract
		ensure
			non_void_contract: Result /= Void
		end

	income_swept (a_gedb_event: RP_GEDB_INCOME_SWEPT): !RP_INCOME is
			-- Process income swept gedb event
		require
			non_void_gedb_event: a_gedb_event /= Void
		do
			Result := (create {RP_GEDB_INCOME_SWEPT_CONVERTER}.make (Current, a_gedb_event)).contract
		ensure
			non_void_contract: Result /= Void
		end

	mark_to_market (a_gedb_event: RP_GEDB_MARK_TO_MARKET): !RP_PORTFOLIO_CONTRACT is
			-- Process mark-to-market gedb event
		require
			non_void_gedb_event: a_gedb_event /= Void
		do
			Result := (create {RP_GEDB_MARK_TO_MARKET_CONVERTER}.make (Current, a_gedb_event)).contract
		ensure
			non_void_contract: Result /= Void
		end

	dividend_reinvestment (a_gedb_event: RP_GEDB_DIVIDEND_REINVESTED): !RP_PORTFOLIO_CONTRACT is
			-- Process reinvested dividend gedb event
		require
			non_void_gedb_event: a_gedb_event /= Void
		do
			Result := (create {RP_GEDB_DIVIDEND_REINVESTED_CONVERTER}.make (Current, a_gedb_event)).contract
		ensure
			non_void_contract: Result /= Void
		end

	fixed_deposit_accrual (a_gedb_event: RP_GEDB_FIXED_DEPOSIT_ACCRUAL): !RP_PORTFOLIO_CONTRACT is
			-- Process fixed deposit accrual gedb event
		require
			non_void_gedb_event: a_gedb_event /= Void
		do
			Result := (create {RP_GEDB_FIXED_DEPOSIT_ACCRUAL_CONVERTER}.make (Current, a_gedb_event)).contract
		ensure
			non_void_contract: Result /= Void
		end

feature {NONE} -- Implementation (adjustment system)

	income_adjustment (a_gedb_event: RP_GEDB_INCOME_ADJUSTMENT): !RP_PORTFOLIO_CONTRACT is
			-- Process income adjustement gedb event
		require
			non_void_gedb_event: a_gedb_event /= Void
		do
			Result := (create {RP_GEDB_INCOME_ADJUSTMENT_CONVERTER}.make (Current, a_gedb_event)).contract
		ensure
			non_void_contract: Result /= Void
		end

	income_unrelated_adjustment (a_gedb_event: RP_GEDB_INCOME_UNRELATED_ADJUSTMENT): !RP_PORTFOLIO_CONTRACT is
			-- Process income unrelated adjustmnet gedb event
		do
			Result := (create {RP_GEDB_INCOME_UNRELATED_ADJUSTMENT_CONVERTER}.make (Current, a_gedb_event)).contract
		end

	expense_adjustment (a_gedb_event: RP_GEDB_EXPENSE_ADJUSTMENT): !RP_PORTFOLIO_CONTRACT is
			-- Process expense adjustment gedb event
		do
			Result := (create {RP_GEDB_EXPENSE_ADJUSTMENT_CONVERTER}.make (Current, a_gedb_event)).contract
		end

	dividend_tax_refund (a_gedb_event: RP_GEDB_DIVIDEND_TAX_REFUND): !RP_PORTFOLIO_CONTRACT is
			-- Process dividend tax refund gedb event
		do
			Result := (create {RP_GEDB_DIVIDEND_TAX_REFUND_CONVERTER}.make (Current, a_gedb_event)).contract
		end

	expense_unrelated_adjustment (a_gedb_event: RP_GEDB_EXPENSE_UNRELATED_ADJUSTMENT): !RP_PORTFOLIO_CONTRACT is
			-- Process expense unrelated adjustment
		do
			Result := (create {RP_GEDB_EXPENSE_UNRELATED_ADJUSTMENT_CONVERTER}.make (Current, a_gedb_event)).contract
		end

feature {NONE} -- Implementation (trading system)

	buy (a_gedb_event: RP_GEDB_BUY): !RP_PORTFOLIO_CONTRACT is
			-- Convert `a_gedb_event' to contract
		require
			non_void_gedb_event: a_gedb_event /= Void
		do
			Result := (create {RP_GEDB_BUY_CONVERTER}.make (Current, a_gedb_event)).contract
		ensure
			non_void_contact: Result /= Void
		end

	sell (a_gedb_event: RP_GEDB_SELL): !RP_PORTFOLIO_CONTRACT is
			-- Convert `a_gedb_event' to contract
		require
			non_void_gedb_event: a_gedb_event /= Void
		do
			Result := (create {RP_GEDB_SELL_CONVERTER}.make (Current, a_gedb_event)).contract
		ensure
			non_void_contact: Result /= Void
		end

	short (a_gedb_event: RP_GEDB_SHORT): !RP_PORTFOLIO_CONTRACT is
			-- Convert `a_gedb_event' to contract
		require
			non_void_gedb_event: a_gedb_event /= Void
		do
			Result := (create {RP_GEDB_SHORT_CONVERTER}.make (Current, a_gedb_event)).contract
		ensure
			non_void_contact: Result /= Void
		end

	cover (a_gedb_event: RP_GEDB_COVER): !RP_PORTFOLIO_CONTRACT is
			-- Convert `a_gedb_event' to contract
		require
			non_void_gedb_event: a_gedb_event /= Void
		do
			Result := (create {RP_GEDB_COVER_CONVERTER}.make (Current, a_gedb_event)).contract
		ensure
			non_void_contact: Result /= Void
		end

	fx (a_gedb_event: RP_GEDB_FX): !RP_PORTFOLIO_CONTRACT is
			-- Convert `a_gedb_event' to contract
		require
			non_void_gedb_event: a_gedb_event /= Void
		do
			Result := (create {RP_GEDB_FX_CONVERTER [RP_GEDB_FX, RP_FX]}.make (Current, a_gedb_event)).contract
		ensure
			non_void_contact: Result /= Void
		end

	forward_fx_open (a_gedb_event: RP_GEDB_FFX_OPEN): !RP_PORTFOLIO_CONTRACT is
			-- Convert `a_gedb_event' to contract
		require
			non_void_gedb_event: a_gedb_event /= Void
		do
			Result := (create {RP_GEDB_FFX_OPEN_CONVERTER}.make (Current, a_gedb_event)).contract
		ensure
			non_void_contact: Result /= Void
		end

	forward_fx_close (a_gedb_event: RP_GEDB_FFX_CLOSE): !RP_PORTFOLIO_CONTRACT is
			-- Convert `a_gedb_event' to contract
		require
			non_void_gedb_event: a_gedb_event /= Void
		do
			Result := (create {RP_GEDB_FFX_CLOSE_CONVERTER}.make (Current, a_gedb_event)).contract
		ensure
			non_void_contact: Result /= Void
		end

	forward_fx_gain_loss (a_gedb_event: RP_GEDB_FFX_GAIN_LOSS): !RP_PORTFOLIO_CONTRACT is
			-- Convert `a_gedb_event' to contract
		require
			non_void_gedb_event: a_gedb_event /= Void
		do
			Result := (create {RP_GEDB_FFX_GAIN_LOSS_CONVERTER}.make (Current, a_gedb_event)).contract
		ensure
			non_void_contact: Result /= Void
		end

	intraday_sell_to_adjust_gain_loss (a_gedb_event: RP_GEDB_INTRADAY_SELL_TO_ADJUST_GL): !RP_PORTFOLIO_CONTRACT is
			-- Convert `a_gedb_event' to contract
		require
			non_void_gedb_event: a_gedb_event /= Void
		do
			Result := (create {RP_GEDB_INTRADAY_SELL_TO_ADJUST_GL_CONVERTER}.make (Current, a_gedb_event)).contract
		ensure
			non_void_contact: Result /= Void
		end

	futures_buy (a_gedb_event: RP_GEDB_FUTURES_BUY): !RP_PORTFOLIO_CONTRACT is
			-- Convert `a_gedb_event' to contract
		require
			non_void_gedb_event: a_gedb_event /= Void
		do
			Result := (create {RP_GEDB_FUTURES_BUY_CONVERTER}.make (Current, a_gedb_event)).contract
		ensure
			non_void_contact: Result /= Void
		end

	futures_sell (a_gedb_event: RP_GEDB_FUTURES_SELL): !RP_PORTFOLIO_CONTRACT is
			-- Convert `a_gedb_event' to contract
		require
			non_void_gedb_event: a_gedb_event /= Void
		do
			Result := (create {RP_GEDB_FUTURES_SELL_CONVERTER}.make (Current, a_gedb_event)).contract
		ensure
			non_void_contact: Result /= Void
		end

	futures_short (a_gedb_event: RP_GEDB_FUTURES_SHORT): !RP_PORTFOLIO_CONTRACT is
			-- Convert `a_gedb_event' to contract
		require
			non_void_gedb_event: a_gedb_event /= Void
		do
			Result := (create {RP_GEDB_FUTURES_SHORT_CONVERTER}.make (Current, a_gedb_event)).contract
		ensure
			non_void_contact: Result /= Void
		end

	futures_cover (a_gedb_event: RP_GEDB_FUTURES_COVER): !RP_PORTFOLIO_CONTRACT is
			-- Convert `a_gedb_event' to contract
		require
			non_void_gedb_event: a_gedb_event /= Void
		do
			Result := (create {RP_GEDB_FUTURES_COVER_CONVERTER}.make (Current, a_gedb_event)).contract
		ensure
			non_void_contact: Result /= Void
		end

feature {NONE} -- Implementation (Financial system)

	income (a_gedb_event: RP_GEDB_INCOME): !RP_PORTFOLIO_CONTRACT is
			-- Process legacy income
		do
			Result := (create {RP_GEDB_INCOME_CONVERTER}.make (Current, a_gedb_event)).contract
		end

	income_unrelated (a_gedb_event: RP_GEDB_INCOME_UNRELATED): !RP_PORTFOLIO_CONTRACT is
			-- Process legacy income unrelated to performance
		do
			Result := (create {RP_GEDB_INCOME_UNRELATED_CONVERTER}.make (Current, a_gedb_event)).contract
		end

	expense (a_gedb_event: RP_GEDB_EXPENSE): !RP_PORTFOLIO_CONTRACT is
			-- Process legacy expense
		do
			Result := (create {RP_GEDB_EXPENSE_CONVERTER}.make (Current, a_gedb_event)).contract
		end

	expense_unrelated (a_gedb_event: RP_GEDB_EXPENSE_UNRELATED): !RP_PORTFOLIO_CONTRACT is
			-- Process legacy expense unrelated to performance
		do
			Result := (create {RP_GEDB_EXPENSE_UNRELATED_CONVERTER}.make (Current, a_gedb_event)).contract
		end

feature {NONE} -- Implementation (Portfolio Engineering system)

	bank_transfer (a_gedb_event: RP_GEDB_BANK_TRANSFER): !RP_PORTFOLIO_CONTRACT is
			-- Create RP_BANK_TRANSFER from GEDB event
		require
			non_void_event: a_gedb_event /= Void
		do
			Result := (create {RP_GEDB_BANK_TRANSFER_CONVERTER}.make (Current, a_gedb_event)).contract
		ensure
			contract_not_void: Result /= Void
		end

	fixed_deposit_transfer (a_gedb_event: RP_GEDB_FIXED_DEPOSIT_TRANSFER): !RP_PORTFOLIO_CONTRACT is
			-- Create RP_FIXED_DEPOSIT_TRANSFER from GEDB event
		require
			non_void_event: a_gedb_event /= Void
		do
			Result := (create {RP_GEDB_FIXED_DEPOSIT_TRANSFER_CONVERTER}.make (Current, a_gedb_event)).contract
		ensure
			contract_not_void: Result /= Void
		end

	collateral (a_gedb_event: RP_GEDB_COLLATERAL): !RP_PORTFOLIO_CONTRACT is
			-- Create RP_COLLATERAL from GEDB event
		require
			non_void_event: a_gedb_event /= Void
		do
			Result := (create {RP_GEDB_COLLATERAL_CONVERTER}.make (Current, a_gedb_event)).contract
		ensure
			contract_not_void: Result /= Void
		end

	cash_zonal_transfer (a_gedb_event: RP_GEDB_CASH_ZONAL_TRANSFER): !RP_PORTFOLIO_CONTRACT is
			-- RP_CASH_ZONAL_TRANSFER
		require
			non_void_event: a_gedb_event /= Void
		do
			Result := (create {RP_GEDB_CASH_ZONAL_TRANSFER_CONVERTER}.make (Current, a_gedb_event)).contract
		ensure
			contract_not_void: Result /= Void
		end

	intrazonal_withdrawal (a_gedb_event: RP_GEDB_INTERNAL_CASH_XFER_WITHDRAWAL): !RP_CASH_ZONAL_TRANSFER is
			-- Create intrazonal withdrawal contract from `a_gedb_event'
		require
			non_void_event: a_gedb_event /= Void
		do
			Result := (create {RP_GEDB_INTERNAL_CASH_XFER_WITHDRAWAL_CONVERTER}.make (Current, a_gedb_event)).contract
		ensure
			contract_not_void: Result /= Void
		end

	intrazonal_contribution (a_gedb_event: RP_GEDB_INTERNAL_CASH_XFER_CONTRIBUTION): !RP_CASH_ZONAL_TRANSFER is
			-- Create intrazonal contribution contract from `a_gedb_event'
		require
			non_void_event: a_gedb_event /= Void
		do
			Result := (create {RP_GEDB_INTERNAL_CASH_XFER_CONTRIBUTION_CONVERTER}.make (Current, a_gedb_event)).contract
		ensure
			contract_not_void: Result /= Void
		end

end
