note
	description: "GEDB accessor"
	author: "Sami Ben Grine"
	last_editor: "$Author: sm $"
	date: "$Date: 2013/04/26 $"
	revision: "$Revision: #1 $"
	path: "$File: //rose/source/kernel/expert_system/rp_gedb_predicates.e $"

class RP_GEDB_PREDICATES

inherit
	ANY
	DATABASE_SERVER
		export {NONE} all end

feature -- Access

	transaction_dates (a_code: STRING): ARRAY [ROSE_DATE] is
			-- All dates for which we had a transaction in 'a_portfolio'
		local
			l_predicate: DATE_PREDICATE
		do
			create l_predicate.make (gedb_predicates_server, "select distinct event_date from open_account_events where portfolio = ? order by event_date")
			l_predicate.retrieve (Void, <<a_code>>)
			create Result.make_from_array (l_predicate.array)
		end

	internal_gedb_record_processing_reader_exact: COLLECTING_PREDICATE [RP_GEDB_RECORD] is
				-- Query for 'internal_gedb_record_processing', match on portfolio and event_date
			do
				if p_internal_gedb_record_processing_reader_exact = Void then
					create p_internal_gedb_record_processing_reader_exact.make (gedb_predicates_server, "SELECT * FROM open_account_events WHERE portfolio = ? AND event_date = ? ORDER BY event_date, icc")
				end
				Result := p_internal_gedb_record_processing_reader_exact
			ensure
				exists: Result /= Void
			end

	internal_gedb_record_processing_reader_latest: COLLECTING_PREDICATE [RP_GEDB_RECORD] is
				-- Query for 'internal_gedb_record_processing', match on portfolio, latest event_date
			do
				if p_internal_gedb_record_processing_reader_latest = Void then
					create p_internal_gedb_record_processing_reader_latest.make (gedb_predicates_server, "SELECT * FROM open_account_events WHERE portfolio = ? AND event_date <= ? ORDER BY event_date, icc")
				end
				Result := p_internal_gedb_record_processing_reader_latest
			ensure
				exists: Result /= Void
			end

	internal_gedb_record_processing_reader_filter_exact: COLLECTING_PREDICATE [RP_GEDB_RECORD] is
			-- Query for 'internal_gedb_record_processing', match on portfolio, filter by instrument, exact event_date
			do
				if p_internal_gedb_record_processing_reader_filter_exact = Void then
					create p_internal_gedb_record_processing_reader_filter_exact.make (gedb_predicates_server, "SELECT * FROM open_account_events WHERE portfolio = ? AND instrument like ? AND event_date = ? ORDER BY event_date, icc")
				end
				Result := p_internal_gedb_record_processing_reader_filter_exact
			ensure
				exists: Result /= Void
			end

	internal_gedb_record_processing_reader_filter_latest: COLLECTING_PREDICATE [RP_GEDB_RECORD] is
			-- Query for 'internal_gedb_record_processing', match on portfolio, filter by instrument, latest event_date
			do
				if p_internal_gedb_record_processing_reader_filter_latest = Void then
					create p_internal_gedb_record_processing_reader_filter_latest.make (gedb_predicates_server, "SELECT * FROM open_account_events WHERE portfolio = ? AND instrument like ? AND event_date <= ? ORDER BY event_date, icc")
				end
				Result := p_internal_gedb_record_processing_reader_filter_latest
			ensure
				exists: Result /= Void
			end

feature -- Status Report

	is_using_intra_day: BOOLEAN
			-- Is using intraday changes?

feature -- Change

	set_use_intra_day (a_use: BOOLEAN) is
			-- Set `is_using_intra_day to `a_use' (and re-create predicates)
		do
			is_using_intra_day := a_use
		ensure
			is_using_intra_day_set: is_using_intra_day = a_use
		end

feature {NONE} -- Implementation

	gedb_predicates_server: SQL_DATA_SERVER is
		-- Server for GEDB Queries
		do
			if is_using_intra_day then
				Result := servers.gedb_server_fast_load
			else
				Result := servers.gedb_fg_fast_load
			end
		ensure
			exists: Result /= Void
		end

	p_internal_gedb_record_processing_reader_exact: like internal_gedb_record_processing_reader_exact
			-- Lazy initialized predicate

	p_internal_gedb_record_processing_reader_latest: like internal_gedb_record_processing_reader_latest
			-- Lazy initialized predicate

	p_internal_gedb_record_processing_reader_filter_exact: like internal_gedb_record_processing_reader_filter_exact
			-- Lazy initialized predicate

	p_internal_gedb_record_processing_reader_filter_latest: like internal_gedb_record_processing_reader_filter_latest
			-- Lazy initialized predicate

end
