indexing

	description:

		"GEDB Event Record Subset used in Accounting Flows"

	date: "$Date: 2010/06/14 $"
	sccid: "$Id: //rose/source/kernel/expert_system/rp_gedb_record.e#18 $"

class RP_GEDB_RECORD

inherit

	COMPARABLE

	RP_EVENT_TYPED
		undefine
			is_equal
		end

	RP_EVENT_SOURCE
		undefine
			is_equal
		end

	ROSE_FORMATTING
		export
			{NONE} all
		undefine
			is_equal
		end

	RP_EVENT_RECORD_CONSTANTS
		export
			{NONE} all
		undefine
			is_equal
		end

create

	default_create,
	make_default

feature {NONE} -- Initialization

	make_default is
			-- Initialize `Current' with default values
		do
			fill_void_decimals
			icc := undefined_icc
			soft_dollar := undefined_soft_dollar
			create broker.make_from_string (undefined_broker)
			create trader.make_from_string (undefined_trader)
			create catagory.make_from_string (undefined_catagory)
			create event_details.make_from_string (undefined_event_details)
			create subaccount.make_from_string (undefined_subaccount)
			create exchange.make_from_string (undefined_exchange)
			create from_instrument.make_from_string (undefined_from_instrument)
			prune_trailing_blanks
		end

feature -- Initialization

	prune_trailing_blanks is
			-- Remove trailing spaces
		do
			prune_all_trailing_from (portfolio, Whitespace_character)
			prune_all_trailing_from (event_details, Whitespace_character)
			prune_all_trailing_from (instrument, Whitespace_character)
			prune_all_trailing_from (currency, Whitespace_character)
			prune_all_trailing_from (secondary_currency, Whitespace_character)
			prune_all_trailing_from (broker, Whitespace_character)
			prune_all_trailing_from (catagory, Whitespace_character)
			prune_all_trailing_from (trader, Whitespace_character)
			prune_all_trailing_from (subaccount, Whitespace_character)
			prune_all_trailing_from (exchange, Whitespace_character)
			prune_all_trailing_from (from_instrument, Whitespace_character)
		end

	fill_void_decimals is
			-- Fill in void references so that everything shows up as zero
		do
			if units = Void then
				create units.make_zero
			end
			if price = Void then
				create price.make_zero
			end
			if fee = Void then
				create fee.make_zero
			end
			if commission = Void then
				create commission.make_zero
			end
			if cash_effect = Void then
				create cash_effect.make_zero
			end
			if secondary_cash_effect = Void then
				create secondary_cash_effect.make_zero
			end
			if performance_effect = Void then
				create performance_effect.make_zero
			end
			if units_base = Void then
				create units_base.make_zero
			end
			if rate = Void then
				create rate.make_zero
			end
		end

feature -- Compare

	is_different_than (a_other: like Current): BOOLEAN is
			-- Compare `Current' to `a_other'
		require
			a_other_not_void: a_other /= Void
		do
			Result := not is_same (a_other)
		end

	is_same (a_other: like Current): BOOLEAN is
			-- Is a_other ~ us?
		require
			a_other_not_void: a_other /= Void
		do
			Result := True
			Result := Result and then zone = a_other.zone
			Result := Result and then event_date ~ a_other.event_date
			Result := Result and then portfolio ~ a_other.portfolio
			Result := Result and then icc = a_other.icc
			Result := Result and then event_type = a_other.event_type
			Result := Result and then catagory ~ a_other.catagory
			Result := Result and then event_details ~ a_other.event_details
			Result := Result and then instrument ~ a_other.instrument
			Result := Result and then units ~ a_other.units
			Result := Result and then price ~ a_other.price
			Result := Result and then fee ~ a_other.fee
			Result := Result and then commission ~ a_other.commission
			Result := Result and then cash_effect ~ a_other.cash_effect
			Result := Result and then currency ~ a_other.currency
			Result := Result and then secondary_cash_effect ~ a_other.secondary_cash_effect
			Result := Result and then secondary_currency ~ a_other.secondary_currency
			Result := Result and then performance_effect ~ a_other.performance_effect
			Result := Result and then broker ~ a_other.broker
			Result := Result and then trader ~ a_other.trader
			Result := Result and then settle_date ~ a_other.settle_date
			Result := Result and then subaccount ~ a_other.subaccount
			Result := Result and then exchange ~ a_other.exchange
			Result := Result and then soft_dollar = a_other.soft_dollar
			Result := Result and then units_base ~ a_other.units_base
			Result := Result and then rate ~ a_other.rate
			Result := Result and then from_instrument ~ a_other.from_instrument
		end

	infix "<" (a_other: like Current): BOOLEAN is
			-- <Precursor>
		do
			if zone = a_other.zone then
				if portfolio ~ a_other.portfolio then
					Result := icc < a_other.icc
				else
					Result := portfolio < a_other.portfolio
				end
			else
				Result := zone < a_other.zone
			end
		end

feature -- Access

	event_date: ROSE_DATE
			-- Event date

	zone: CHARACTER
			-- Zone

	portfolio: STRING
			-- Portfolio

	icc: INTEGER
			-- ICC

	event_type: INTEGER
			-- Event type

	catagory: STRING
			-- catagory

	event_details: STRING
			-- Event details

	entered_at: ROSE_DATE
			-- Entered at

	entered_by: STRING
			-- Entered by

	modified_at: ROSE_DATE
			-- Modified at

	modified_by: STRING
			-- Modified by

	instrument: STRING
			-- Instrument

	units: RP_DECIMAL
			-- Units

	price: RP_DECIMAL
			-- Price

	fee: RP_DECIMAL
			-- Trade fees and taxes

	commission: RP_DECIMAL
			-- Trade commission

	cash_effect: RP_DECIMAL
			-- Cash effect

	currency: STRING
			-- Currency of `cash_effect' and `performance_effect'

	secondary_cash_effect: RP_DECIMAL
			-- Secondary cash effect

	secondary_currency: STRING
			-- Currency of `secondary_cash_effect'

	performance_effect: RP_DECIMAL
			-- Performance effect

	broker: STRING
			-- Broker

	trader: STRING
			-- Trader

	settle_date: ROSE_DATE
			-- Settle date

	subaccount: STRING
			-- Sub account (char (3))

	exchange: STRING
			-- Exchange

	transaction_time: ROSE_DATE
			-- XXX unknown purpose. CAPE sets this to the event_date

	soft_dollar: CHARACTER
			-- Soft dollar

	units_base: RP_DECIMAL
			-- Units base

	rate: RP_DECIMAL
			-- Rate

	from_instrument: STRING
			-- From instrumenbt ((char(15))

feature -- Queries

	is_fungible_conversion: BOOLEAN is
			-- Is `Current' fungible conversion?
		do
			Result := (event_type = 3 or event_type = 23) and catagory.substring_index ("CONVE", 1) > 0
		end

	is_fx_with_transfer: BOOLEAN is
			-- Is `Current' fx transfer?
		do
			Result := not instrument.substring (1, 2).is_equal (instrument.substring (4, 5))
		end

feature -- Element setting

	set_event_date (a_event_date: like event_date) is
			-- Set 'event_date' to 'a_event_date'
		do
			event_date := a_event_date
		ensure
			event_date_set: event_date = a_event_date
		end

	set_zone (a_zone: like zone) is
			-- Set 'zone' to 'a_zone'
		do
			zone := a_zone
		ensure
			zone_set: zone = a_zone
		end

	set_portfolio (a_portfolio: like portfolio) is
			-- Set 'portfolio' to 'a_portfolio'
		do
			portfolio := a_portfolio
		ensure
			portfolio_set: portfolio = a_portfolio
		end

	set_icc (a_icc: like icc) is
			-- Set 'icc' to 'a_icc'
		do
			icc := a_icc
		ensure
			icc_set: icc = a_icc
		end

	set_event_type (a_event_type: like event_type) is
			-- Set 'event_type' to 'a_event_type'
		do
			event_type := a_event_type
		ensure
			event_type_set: event_type = a_event_type
		end

	set_catagory (a_catagory: like catagory) is
			-- set catagory
		do
			catagory := a_catagory
		ensure
			catagory_set: catagory = a_catagory
		end

	set_event_details (a_event_details: like event_details) is
			-- Set 'event_details' to 'a_event_details'
		do
			event_details := a_event_details
		ensure
			event_details_set: event_details = a_event_details
		end

	set_entered_at (a_entered_at: like entered_at) is
			-- Set 'entered_at' to 'a_entered_at'
		do
			entered_at := a_entered_at
		ensure
			entered_at_set: entered_at = a_entered_at
		end

	set_entered_by (a_entered_by: like entered_by) is
			-- Set 'entered_by' to 'a_entered_by'
		do
			entered_by := a_entered_by
		ensure
			entered_by_set: entered_by = a_entered_by
		end

	set_modified_at (a_modified_at: like modified_at) is
			-- Set 'modified_at' like 'modified_at'
		do
			modified_at := a_modified_at
		ensure
			modified_at_set: modified_at = a_modified_at
		end

	set_modified_by (a_modified_by: like modified_by) is
			-- Set 'modified_by' like 'a_modified_by'
		do
			modified_by := a_modified_by
		ensure
			modify_by_set: modified_by = a_modified_by
		end

	set_instrument (a_instrument: like instrument) is
			-- Set 'instrument' to 'a_instrument'
		do
			instrument := a_instrument
		ensure
			instrument_set: instrument = a_instrument
		end

	set_units (a_units: like units) is
			-- Set 'units' to 'a_units'
		do
			units := a_units
		ensure
			units_set: units = a_units
		end

	set_price (a_price: like price) is
			-- Set 'price' to 'a_price'
		do
			price := a_price
		ensure
			price_set: price = a_price
		end

	set_fee (a_fee: like fee) is
			-- Set 'fee' to 'a_fee'
		do
			fee := a_fee
		ensure
			fee_set: fee = a_fee
		end

	set_commission (a_commission: like commission) is
			-- Set 'commission' to 'a_commission'
		do
			commission := a_commission
		ensure
			commission_set: commission = a_commission
		end

	set_cash_effect (a_cash_effect: like cash_effect) is
			-- Set 'cash_effect' to 'a_cash_effect'
		do
			cash_effect := a_cash_effect
		ensure
			cash_effect_set: cash_effect = a_cash_effect
		end

	set_currency (a_currency: like currency) is
			-- Set 'currency' to 'a_currency'
		do
			currency := a_currency
		ensure
			currency_set: currency = a_currency
		end

	set_secondary_cash_effect (a_secondary_cash_effect: like secondary_cash_effect) is
			-- Set 'secondary_cash_effect' to 'a_secondary_cash_effect'
		do
			secondary_cash_effect := a_secondary_cash_effect
		ensure
			secondary_cash_effect_set: secondary_cash_effect = a_secondary_cash_effect
		end

	set_secondary_currency (a_secondary_currency: like secondary_currency) is
			-- Set 'secondary_currency' to 'a_secondary_currency'
		do
			secondary_currency := a_secondary_currency
		ensure
			secondary_currency_set: secondary_currency = a_secondary_currency
		end

	set_performance_effect (a_performance_effect: like performance_effect) is
			-- Set 'performance_effect' to 'a_performance_effect'
		do
			performance_effect := a_performance_effect
		ensure
			performance_effect_set: performance_effect = a_performance_effect
		end

	set_broker (a_broker: like broker) is
			-- Set 'broker' to 'a_broker'
		do
			broker := a_broker
		ensure
			broker_set: broker = a_broker
		end

	set_trader (a_trader: like trader) is
			-- Set 'trader' to 'a_trader'
		do
			trader := a_trader
		ensure
			trader_set: trader = a_trader
		end

	set_settle_date (a_settle_date: like settle_date) is
			-- Set 'settle_date' to 'a_settle_date'
		do
			settle_date := a_settle_date
		ensure
			settle_date_set: settle_date = a_settle_date
		end

	set_subaccount (a_subaccount: like subaccount) is
			-- Set 'subaccount' to 'a_subaccount'
		do
			subaccount := a_subaccount
		ensure
			subaccount_set: subaccount = a_subaccount
		end

	set_exchange (a_exchange: like exchange) is
			-- Set 'exchange' to 'a_exchange'
		do
			exchange := a_exchange
		ensure
			exchange_set: exchange = a_exchange
		end

	set_transaction_time (a_transaction_time: like transaction_time) is
			-- Set 'transaction_time' to 'a_Transaction_time'
		do
			transaction_time := a_transaction_time
		ensure
			transaction_time_set: transaction_time = a_transaction_time
		end

	set_soft_dollar (a_soft_dollar: like soft_dollar) is
			-- Set 'soft_dollar' to 'a_soft_dollar'
		do
			soft_dollar := a_soft_dollar
		ensure
			soft_dollar_set: soft_dollar = a_soft_dollar
		end

	set_units_base (a_units_base: like units_base) is
			-- Set 'units_base' to 'a_units_base'
		do
			units_base := a_units_base
		ensure
			units_base_set: units_base = a_units_base
		end

	set_rate (a_rate: like rate) is
			-- Set 'rate' to 'a_rate'
		do
			rate := a_rate
		ensure
			rate_set: rate = a_rate
		end

	set_from_instrument (a_from_instrument: like from_instrument) is
			-- Set 'from_instrument' to 'a_from_instrument'
		do
			from_instrument := a_from_instrument
		ensure
			from_instrument_set: from_instrument = a_from_instrument
		end

feature -- Status Report

	is_pay_catagory: BOOLEAN is
			-- Is this event marked with PAY?
			--
			-- This is Etusuko's comment
			-- "It is our old process.
			-- We used to do the operation below for JTB(Local) account.
			-- 1. mark "PAY" on the div data when it is incase of the booking delay at bank side though our div data (payment date) is correct,
			-- 2. add an entry in the Reconciliation PARM file after this ex date to stop Event Processing from deleting the entry again,
			-- 3. when the div payment is confirmed with the bank, we re-change an entry to fix the payment date in the reconciliation parm
		do
			Result := catagory.substring (1, pay_catagory.count) ~ Pay_catagory
		ensure
			compare_catagory_with_pay: Result = (catagory.substring (1, pay_catagory.count) ~ Pay_catagory)
		end

feature -- Output

	character_delimited_out (a_character: CHARACTER): STRING is
			-- 'a_character' delimited representation of 'Current'
		do
			Result := formatter.join_array (<<event_date.ccyymmdd, portfolio, icc, event_type, event_details, instrument, units, fee, commission, cash_effect, currency, secondary_cash_effect, secondary_currency, performance_effect, broker, settle_date.ccyymmdd, units_base, catagory>>, a_character.out)
		ensure
			result_not_void_or_empty: (Result /= Void) and then (not Result.is_empty)
		end

feature -- Constant

	Pay_catagory: STRING is "PAY"
			-- Name of the PAY catagory

	Whitespace_character: CHARACTER is ' '
			-- Blank

feature {NONE} -- Implementation

	prune_all_trailing_from (a_string: STRING; a_character: CHARACTER) is
			-- Remove all trailing 'a_character' from 'a_string'
		do
			if (a_string /= Void) then
				a_string.prune_all_trailing (a_character)
			end
		end

	zero: RP_DECIMAL is
			-- Zero
		once
			create Result.make_zero
		end

end
