indexing

	description:

		"Composite made of investment accounts"

	date: "$Date: 2010/06/29 $"
	sccid: "$Id: //rose/source/kernel/expert_system/ref/rp_global_product.e#22 $"

class RP_GLOBAL_PRODUCT

inherit

	RP_PRODUCT

create {RP_REFERENCE_SYSTEM}

	make

feature -- Query

	is_global: BOOLEAN is True
			-- `Current' is global composite.

	investment_accounts: RP_INVESTMENT_ACCOUNT_HISTORY is
			-- Interval history of investment accounts included in `Current'.
		do
			Result := retrieve_investment_accounts
		end

	investment_accounts_weights (a_dv_query: FUNCTION [ANY, TUPLE [RP_INVESTMENT_ACCOUNT, RP_CURRENCY, ROSE_DATE], RP_PERFORMANCE_DAILY_VALUES]; a_currency: RP_CURRENCY; a_date_interval: ROSE_DATE_INTERVAL): DS_HASH_TABLE [TUPLE [value: RP_DECIMAL; date: ROSE_DATE], RP_INVESTMENT_ACCOUNT] is
			-- Investment accounts weights in date interval
		local
			l_investment_accounts: ROSE_LINEAR_ARRAY [RP_INVESTMENT_ACCOUNT]
		do
			l_investment_accounts := investment_accounts.all_witnesses (a_date_interval.last)
			create Result.make (l_investment_accounts.count)
			l_investment_accounts.do_all (agent query_investment_account_nav (a_dv_query, a_date_interval.first, a_currency, Result, ?))
			if Result.count /= l_investment_accounts.count then
					-- every account should have value
				Result.wipe_out
			end
		end

	benchmark_weights (a_dv_query: FUNCTION [ANY, TUPLE [RP_INVESTMENT_ACCOUNT, RP_CURRENCY, ROSE_DATE], RP_PERFORMANCE_DAILY_VALUES]; a_currency: RP_CURRENCY; a_date_interval: ROSE_DATE_INTERVAL): DS_HASH_TABLE [RP_DECIMAL, RP_BENCHMARK] is
			-- Benchmark weights in date interval
		local
			l_investment_accounts_weights: like investment_accounts_weights
		do
			l_investment_accounts_weights := investment_accounts_weights (a_dv_query, a_currency, a_date_interval)
			create Result.make (l_investment_accounts_weights.count)
			l_investment_accounts_weights.keys.do_all (agent accumulate_benchmark_weights (Result, l_investment_accounts_weights, a_date_interval.last, ?))
		end

	benchmark_weights_vms (a_dv_query: FUNCTION [ANY, TUPLE [RP_INVESTMENT_ACCOUNT, RP_CURRENCY, ROSE_DATE], RP_PERFORMANCE_DAILY_VALUES]; a_currency: RP_CURRENCY; a_date_interval: ROSE_DATE_INTERVAL): DS_HASH_TABLE [RP_DECIMAL, RP_BENCHMARK] is
			-- Benchmark weights in date interval using last known investment account benchmark
		local
			l_investment_accounts_weights: like investment_accounts_weights
		do
			l_investment_accounts_weights := investment_accounts_weights (a_dv_query, a_currency, a_date_interval)
			create Result.make (l_investment_accounts_weights.count)
			l_investment_accounts_weights.keys.do_all (agent accumulate_benchmark_weights_vms (Result, l_investment_accounts_weights, a_date_interval.last, ?))
		end

feature {NONE} -- Implementation

	saved_investment_accounts: like investment_accounts
			-- Placeholder for `investment_accounts'.

	retrieve_investment_accounts: like investment_accounts is
			-- Evaluate `investment_accounts'.
		do
			if saved_investment_accounts = Void then
				create saved_investment_accounts.make_empty
				load_global_composite_membership (saved_investment_accounts)
				apply_composite_composition (saved_investment_accounts)
				saved_investment_accounts.coalesce
			end
			Result := saved_investment_accounts
		end

	apply_composite_composition (a_composite_membership: like investment_accounts) is
			-- Add constituent composites to `Current'
		do
			if gcdb_product /= Void then
				gcdb_product.product_constituents.do_all (agent add_constituent (a_composite_membership, ?))
			end
		end

	add_constituent (a_composite_membership: like investment_accounts; a_product: GCDB_PRODUCT) is
			-- Add contituent `a_product'
		require
			non_void_product: a_product /= Void
		local
			l_product: like Current
		do
			l_product := reference_system.global_product_by_code (a_product.code)
			if not l_product.is_unknown then
				l_product.investment_accounts.do_all (agent add_constituent_account_from_itnerval (a_composite_membership, l_product, ?))
			end
		end

	add_constituent_account_from_itnerval (a_composite_membership: like investment_accounts; a_product: like Current; a_interval: RP_INVESTMENT_ACCOUNT_HISTORY_INTERVAL) is
			-- Add constituent account from `a_interval'
		require
			non_void_interval: a_interval /= Void
		do
			check_begin_close_dates (a_composite_membership, Void, a_interval.investment_account, a_interval.start_date, a_interval.end_date, "From" + a_product.code)
		end

	load_global_composite_membership (a_composite_membership: like investment_accounts) is
			-- Load composite membership.
		do
			a_composite_membership.append (directly_mapped_accounts)
			benchmarks.do_all (agent load_by_benchmark (a_composite_membership, ?))
			a_composite_membership.coalesce
		end

	load_by_benchmark (a_composite_membership: like investment_accounts; a_interval: RP_BENCHMARK_HISTORY_INTERVAL) is
			-- Load composite membership by benchmark.
		do
			if a_interval.weight > 0 then
				a_interval.benchmark.investment_accounts.do_all (agent by_investment_account (a_composite_membership, a_interval, ?))
			end
		end

	by_investment_account (a_composite_membership: like investment_accounts; a_benchmark_interval: RP_BENCHMARK_HISTORY_INTERVAL; a_acct_interval: RP_INVESTMENT_ACCOUNT_HISTORY_INTERVAL) is
			-- Check mapped account's composite membership.
		local
			l_account: RP_INVESTMENT_ACCOUNT
			l_start, l_end: ROSE_DATE
		do
			l_account := a_acct_interval.investment_account
				-- NOTE: Ignore all mappings to composites via benchmarks if there is any direct mapping
			if not l_account.has_direct_composite_mapping then
				l_start := a_benchmark_interval.start_date.max (a_acct_interval.start_date)
				l_end := a_benchmark_interval.end_date.min (a_acct_interval.end_date)
				if l_start < l_end then
					a_acct_interval.investment_account.composite_state.do_all (agent check_composite_state (a_composite_membership, l_start, l_end, l_account, ?))
				end
			end
		end

	check_composite_state (a_composite_membership: like investment_accounts; a_start, a_end: ROSE_DATE; a_investment_account: RP_INVESTMENT_ACCOUNT; a_value: RP_BOOLEAN_HISTORY_INTERVAL) is
			-- Check account's composite state.
		local
			l_start, l_end: ROSE_DATE
		do
			if a_value.item then
				l_start := a_start.max (a_value.start_date)
				l_end := a_end.min (a_value.end_date)
				if l_start < l_end then
					a_investment_account.no_direct_composite_mapping.do_all (agent check_direct_mapping (a_composite_membership, a_investment_account, l_start, l_end, ?))
				end
			end
		end

	check_direct_mapping (a_composite_membership: like investment_accounts; a_investment_account: RP_INVESTMENT_ACCOUNT; a_start: ROSE_DATE; a_end: ROSE_DATE; a_is_not_directly_mapped: RP_BOOLEAN_HISTORY_INTERVAL) is
			-- Check for direct mapping of `a_investment_account' to `Current'
		local
			l_start, l_end: ROSE_DATE
			s: STRING
		do
			l_start := a_start.max (a_is_not_directly_mapped.start_date)
			l_end := a_end.min (a_is_not_directly_mapped.end_date)
			if l_start < l_end then
				if a_is_not_directly_mapped.item then
					create s.make_from_string ("BenchmarkMapping")
					if not a_investment_account.is_global then
						s.append ("_NOT_GLOBAL")
					end
					if a_investment_account.end_date /= Void then
						l_end := l_end.min (a_investment_account.end_date)
					end
					check_begin_close_dates (a_composite_membership, Void, a_investment_account, l_start, l_end, s)
				end
			end
		end

	directly_mapped_accounts: RP_INVESTMENT_ACCOUNT_HISTORY is
			-- Directly mapped accounts.
		do
			Result := retrieve_directly_mapped_accounts
		ensure
			non_void_result: Result /= Void
		end

	saved_directly_mapped_accounts: like directly_mapped_accounts
			-- Cache for directly mapped accounts

	retrieve_directly_mapped_accounts: like directly_mapped_accounts is
			-- Retrieve directly mapped accounts
		do
			if saved_directly_mapped_accounts = Void then
				create saved_directly_mapped_accounts.make_empty
				load_directly_mapped_accounts (saved_directly_mapped_accounts)
			end
			Result := saved_directly_mapped_accounts
		end

	load_directly_mapped_accounts (a_history: like directly_mapped_accounts) is
			-- Load directly mapped accounts
		local
			l_pgs: DS_HASH_SET [RP_INVESTMENT_ACCOUNT]
		do
			if gcdb_product /= Void then
				create l_pgs.make_default
				gcdb_product.portfolio_groups_by_composite_history.do_all (agent add_directly_mapped_portfolio_group (a_history, l_pgs, ?))
				gcdb_product.accounts_by_composite_history.do_all (agent add_directly_mapped_account (a_history, l_pgs, ?))
			end
		end

	add_directly_mapped_portfolio_group (a_history: like directly_mapped_accounts; a_hash: DS_HASH_SET [RP_INVESTMENT_ACCOUNT]; a_interval: ENTITY_INTERVAL [GCDB_PORTFOLIO_GROUP]) is
			-- Add `a_interval' to `a_history'
		local
			l_account: RP_INVESTMENT_ACCOUNT
			s: STRING
			l_end: ROSE_DATE
		do
			l_account := reference_system.investment_account_by_code (a_interval.entity.code)
			if not l_account.is_unknown then
				create s.make_from_string ("DirectPortfolioGroupMapping")
				if not l_account.is_global then
					s.append ("_NOT_GLOBAL")
				end
				l_end := a_interval.end_date
				if l_account.end_date /= Void then
					l_end := l_end.min (l_account.end_date)
				end
				check_begin_close_dates (a_history, a_hash, l_account, a_interval.start_date, l_end, s)
			end
		end

	check_begin_close_dates (a_composite_membership: like investment_accounts; a_hash: DS_HASH_SET [RP_INVESTMENT_ACCOUNT]; a_account: RP_INVESTMENT_ACCOUNT; a_start: ROSE_DATE; a_end: ROSE_DATE; a_message: STRING) is
			-- Check composite begin/close dates
		local
			l_start, l_end: ROSE_DATE
			l_interval: RP_INVESTMENT_ACCOUNT_HISTORY_INTERVAL
		do
			l_start := a_start
			l_end := a_end
			if gcdb_product /= Void then
				if gcdb_product.begin_date /= Void then
					l_start := l_start.max (gcdb_product.begin_date)
				end
				if gcdb_product.close_date /= Void then
					l_end := l_end.min (gcdb_product.close_date)
				end
			end
			if l_start < l_end then
				create l_interval.make (a_account, l_start, l_end, 0)
				l_interval.set_message (a_message)
				a_composite_membership.extend (l_interval)
				if a_hash /= Void then
					a_hash.force (a_account)
				end
			end
		end

	add_directly_mapped_account (a_history: like directly_mapped_accounts; a_hash: DS_HASH_SET [RP_INVESTMENT_ACCOUNT]; a_interval: ENTITY_INTERVAL [GCDB_ACCOUNT]) is
			-- Add `a_interval' to `a_history'
		local
			l_account: RP_INVESTMENT_ACCOUNT
			l_interval: RP_INVESTMENT_ACCOUNT_HISTORY_INTERVAL
			s: STRING
		do
			l_account := reference_system.investment_account_by_code (a_interval.entity.code)
			if not l_account.is_unknown then
				if not a_hash.has (l_account) then
					create s.make_from_string ("DirectAccountMapping")
					if not l_account.is_global then
						s.append ("_NOT_GLOBAL")
					end
					if l_account.end_date /= Void then
						create l_interval.make (l_account, a_interval.start_date, a_interval.end_date.min (l_account.end_date), 0)
					else
						create l_interval.make (l_account, a_interval.start_date, a_interval.end_date, 0)
					end
					l_interval.set_message (s)
					a_history.extend (l_interval)
				end
			end
		end

	query_investment_account_nav (a_dv_query: FUNCTION [ANY, TUPLE [RP_INVESTMENT_ACCOUNT, RP_CURRENCY, ROSE_DATE], RP_PERFORMANCE_DAILY_VALUES]; a_date: ROSE_DATE; a_currency: RP_CURRENCY; a_table: like investment_accounts_weights; a_investment_account: RP_INVESTMENT_ACCOUNT) is
			-- Query investment accont value by using `a_dv_query' for `a_investment_account', `a_currency' on `a_date' and add to `a_table'
		local
			l_dailyvalues: RP_PERFORMANCE_DAILY_VALUES
			l_date: ROSE_DATE
		do
			l_date := a_investment_account.investment_region.legacy_perf_calendar.ith_work_day_before (a_date, 1)
			l_dailyvalues := a_dv_query.item ([a_investment_account, a_currency, l_date])
			if l_dailyvalues /= Void and then not l_dailyvalues.portfolio_value.amount.is_zero then
				a_table.force ([l_dailyvalues.portfolio_value.amount, l_date], a_investment_account)
			else
				l_date := a_investment_account.investment_region.legacy_perf_calendar.next_work_day (a_date)
				l_dailyvalues := a_dv_query.item ([a_investment_account, a_currency, l_date])
				if l_dailyvalues /= Void and then not l_dailyvalues.portfolio_value.amount.is_zero then
					a_table.force ([l_dailyvalues.portfolio_value.amount, l_date], a_investment_account)
				else
					(create {EXCEPTIONS}).raise ("No DailyValues for investment account " + a_investment_account.name + " @ " + a_date.ccyymmdd.out)
				end
			end
		end

	accumulate_benchmark_weights (a_table: like benchmark_weights; a_investment_accounts_weights: like investment_accounts_weights; a_date: ROSE_DATE; a_investment_account: RP_INVESTMENT_ACCOUNT) is
			-- Accumulate benchmark weights in `a_table'
		require
			non_void_args: a_table /= Void and a_investment_accounts_weights /= Void and a_date /= Void and a_investment_account /= Void
			portfolio_weights_has_portfolio: a_investment_accounts_weights.has (a_investment_account)
		local
			l_benchmark: RP_BENCHMARK
			l_weight: RP_DECIMAL
		do
			l_benchmark := a_investment_account.non_hedged_benchmarks.witness (a_date)
			if l_benchmark /= Void then
				a_table.search (l_benchmark)
				if a_table.found then
					l_weight := a_table.found_item
				else
					create l_weight.make_from_integer (0)
				end
				l_weight := l_weight + a_investment_accounts_weights.item (a_investment_account).value
				a_table.force (l_weight, l_benchmark)
			end
		end

	accumulate_benchmark_weights_vms (a_table: like benchmark_weights; a_investment_accounts_weights: like investment_accounts_weights; a_date: ROSE_DATE; a_investment_account: RP_INVESTMENT_ACCOUNT) is
			-- Accumulate benchmark weights in `a_table' using last known investment account benchmark
		require
			non_void_args: a_table /= Void and a_investment_accounts_weights /= Void and a_date /= Void and a_investment_account /= Void
			portfolio_weights_has_portfolio: a_investment_accounts_weights.has (a_investment_account)
		local
			l_benchmark: RP_BENCHMARK
			l_weight: RP_DECIMAL
		do
			if not a_investment_account.non_hedged_benchmarks.is_empty then
				l_benchmark := a_investment_account.non_hedged_benchmarks.last.benchmark
			end
			if l_benchmark /= Void then
				a_table.search (l_benchmark)
				if a_table.found then
					l_weight := a_table.found_item
				else
					create l_weight.make_from_integer (0)
				end
				l_weight := l_weight + a_investment_accounts_weights.item (a_investment_account).value
				a_table.force (l_weight, l_benchmark)
			end
		end

	membership_begin_date: like begin_date is
			-- Begin of memberhip.
		do
			Result := investment_accounts.min_start_date
		end

	membership_close_date: like close_date is
			-- Last membership date.
		do
			Result := investment_accounts.max_end_date
		end

end
