note

	description:

		"Objects that represent rows for RP_DATA_GRID"

	author: "Mark Howard"
	last_editor: "$Author: jq $"
	date: "$Date: 2015/03/11 $"
	revision: "$Revision: #112 $"
	path: "$File: //rose/source/kernel/infrastructure/widgets/grid/rp_grid_node.e $"

class
	RP_GRID_NODE [G]

inherit
	RP_DATA_GRID_ACCESS

create
	make, make_from_agents, make_expandable_from_array, make_from_array, make_expandable_static

feature {NONE} -- Initialization

	make_from_agents (a_display: like display; a_expand: like expand_agent; a_column_count: INTEGER) is
			-- Create `Current' with `display' set to `a_display', an expand agent `a_expand' and
			-- a column count of `a_column_count'.
		require
			column_count_non_negative: a_column_count >= 0
		do
			column_count := a_column_count
			count := a_column_count
			display := a_display
			expand_agent := a_expand
			id := global_row_index.item
			global_row_index.put (global_row_index.item + 1)
		end

	make_from_array(a_array: ARRAY [ANY]) is
			-- Create `Current' with items `a_array'.
		require
			array_not_void: a_array /= Void
		do
			make_expandable_from_array (a_array, Void)
		end

	make_expandable_from_array(a_array: ARRAY [ANY];a_expand: like expand_agent) is
			-- create `Current' with items `a_array' and `expand_agent' set to `a_expand'.
		require
			array_not_void: a_array /= Void
		local
			l_default: like data
		do
			make (a_array, a_expand, l_default)
		end

	make_expandable_static (a_array: ARRAY [ANY]) is
			-- preset expand agent which references static_children array
		require
			array_not_void: a_array /= Void
		do
			make_expandable_from_array (a_array, agent expand_with_static_rows)
			create static_children.make_empty
		end

	make (a_array: ARRAY [ANY]; a_expand: like expand_agent; a_data: like data) is
			-- Make with 'a_array' of cells, 'a_expand' agent and 'a_data'.
		require
			array_not_void: a_array /= Void
		do
			make_from_agents (agent array_item(a_array, ?), a_expand, a_array.count)
			set_data (a_data)
		end

feature -- Access

	count: INTEGER
		-- Number of items within `Current', each corresponds to a column within `grid'.

	data: G
		-- Custom data associated with `Current'.

	index: INTEGER is
			-- Position of Current in parent.
		require
			parented: grid /= Void
		do
			if row.parent /= Void then
				Result := row.index
			end
		ensure
			index_positive: Result > 0
			index_less_than_or_equal_to_grid_row_count: Result <= grid.row_count
		end

	grid: RP_DATA_GRID [G]
		-- Parent grid of `Current' or `Void' if `Current'
		-- is not yet parented.

	height: INTEGER is
			-- Height of `Current' in pixels.
		do
			if row /= Void then
				Result := row.height
			else
				Result := height_saved
			end
		end

	is_visible: BOOLEAN is
			-- Is `Current' visible in `grid'?
		require
			parented: grid /= Void
		do
			Result := ((index >= grid.first_visible_row.index) and (index <= grid.last_visible_row.index) and grid.all_parents_are_expanded (Current))
		end

	is_title_row: BOOLEAN is
			-- Is `Current' a title node?
		do
			Result := False
		end

	primary_title_node: RP_GRID_TITLE_NODE [G]
			-- Title_node Current is a secondary_title_node of if any

	is_expandable: BOOLEAN is
			-- May `Current' be expanded?
		require
			parented: grid /= Void
		do
			Result := row /= Void and then row.is_expandable
		end

	is_expanded: BOOLEAN is
			-- Is `Current' expanded?
		require
			parented: grid /= Void
		do
			Result := row /= Void and then row.is_expanded
		end

	is_selected: BOOLEAN is
			-- Is `Current' selected?
		do
			if row /= Void then
				Result := row.is_selected
			end
		end

	expand_agent: FUNCTION [ANY, TUPLE, RP_TITLED_NODES [G]]
		-- Agent called when `Current' is expanded which fills all children.

	column_count: INTEGER
		-- Number of items within `Current', each corresponds to a column within `grid'.

	row: EV_GRID_ROW
		-- EV_GRID_ROW with which `Current' is implemented.
		-- May be `Void' if `grid = Void'.

	depth: INTEGER is
			-- Nested depth of `Current' within parent `grid'.
		require
			parented: grid /= Void
		local
			l_parent: RP_GRID_NODE [G]
		do
			l_parent := parent_row
			Result := 1
			from until l_parent = Void loop
				Result := Result + 1
				l_parent := l_parent.parent_row
			end
		ensure
			result_non_negative: Result >= 0
		end

	i_th (i: INTEGER): RP_DATA_GRID_ITEM is
			-- Item `i' within `Current'.
		local
			l_any: ANY
			l_item: RP_DATA_GRID_LABEL_ITEM
			l_item_created: BOOLEAN
		do
			Result := items.safe_item(i)
			if Result = Void then
				if display /= Void then
					l_any := display.item([i])
				end
				Result ?= l_any
				if Result /= Void then
					l_item ?= Result
				else
					Result := any_to_grid_item (l_any)
					l_item_created := True
				end
				items.force (Result, i)
			end
			if grid /= Void and then Result.parent = Void then
				grid.grid.set_item (i, index, Result)
				update_item_for_insertion (i)
			end
		ensure
			result_not_void: Result /= Void
		end

	parent_row: RP_GRID_NODE [G] is
			-- Parent row of `Current'.
		require
			parented: grid /= Void
		do
			if row.parent_row /= Void then
				Result ?= row.parent_row.data
			end
		end

	subrow_count: INTEGER is
			-- Number of children.
		do
			if row /= Void then
				Result := row.subrow_count
			end
		ensure
			subrow_count_non_negative: Result >= 0
			subrow_count_in_range: grid /= Void implies (Result <= (grid.row_count - index))
		end

	subrow_count_recursive: INTEGER is
			-- Number of child rows and their child rows recursively.
		do
			if row /= Void then
				Result := row.subrow_count_recursive
			end
		ensure
			subrow_count_recursive_greater_or_equal_to_subrow_count: Result >= subrow_count
			subrow_count_recursive_in_range: grid /= Void implies (Result <= (grid.row_count - index))
		end

	subrow (subrow_index: INTEGER): RP_GRID_NODE [G] is
			-- `Result' is subrow with index `subrow_index' within `Current'.
		require
			parented: grid /= Void
			valid_subrow_index: subrow_index >= 1 and subrow_index <= subrow_count
		do
			Result ?= row.subrow (subrow_index).data
		ensure
			subrow_not_void: Result /= Void
		end

	background_color: EV_COLOR is
			-- Color displayed as background of `Current' except where there are items contained that
			-- have a non-`Void' `background_color'. If `Void', `background_color' of `parent' is displayed..
		do
			if row /= Void then
				Result := row.background_color
			else
				Result := background_color_saved
			end
		end

	foreground_color: EV_COLOR is
			-- Color displayed for foreground features of `Current' except where there are items contained that
			-- have a non-`Void' `foreground_color'. If `Void', `foreground_color' of `parent' is displayed.
		do
			if row /= Void then
				Result := row.foreground_color
			else
				Result := foreground_color_saved
			end
		end

	child_title: RP_GRID_TITLE_NODE [G] is
			-- Titile_node of subrows if any
		do
			if subrow_count > 0 then
				Result ?= subrow (1)
			end
		end

	virtual_y_position: INTEGER is
			-- Vertical offset of `Current' in relation to the
			-- the virtual area of `parent' grid in pixels.
			-- `Result' is 0 if `parent' is `Void'.
		require
			parented: grid /= Void
		do
			Result := row.virtual_y_position
		ensure
			parent_void_implies_result_zero: grid = Void implies Result = 0
		end

	virtual_y_position_unlocked: INTEGER is
			-- Vertical offset of unlocked position of `Current', in relation to the
			-- virtual area of `parent' grid in pixels.
			-- If not `is_locked', then `virtual_y_position' = `unlocked_virtual_y_position'.
			-- If `is_locked' then `Result' is the "shadow" position where the row would
			-- be if not locked.
		require
			parented: grid /= Void
		do
			Result := row.virtual_y_position_unlocked
		ensure
			parent_void_implies_result_zero: grid = Void implies Result = 0
		end

	pebble: ANY
		-- Data to be transported by pick and drop mechanism.

	pebble_agent: FUNCTION [ANY, TUPLE [RP_GRID_NODE [G]], ANY]
		-- Agent used to compute data to be transported via pick and drop.
		-- Only executed at start of pick and drop if `pebble' is Void.

	is_show_requested: BOOLEAN is
			-- May `Current' be displayed?
			-- Will return `False' if `hide' has been called on `Current'.
		require
			parented: grid /= Void
		do
			Result := row.is_show_requested
		end

	is_locked: BOOLEAN is
			-- Is `Current' locked so that it no longer scrolls?
		require
			parented: grid /= Void
		do
			Result :=row.is_locked
		end

	locked_position: INTEGER is
			-- Locked position of `Current' from top edge of viewable area of `parent'.
			-- `Result' is 0 if not `is_locked'.
		require
			parented: grid /= Void
		do
			Result := row.locked_position
		ensure
			not_locked_implies_result_zero: not is_locked implies Result = 0
		end

feature -- Status setting


	set_height (a_height: INTEGER) is
			-- Assign `a_height' to `height'.
		require
			a_height_non_negative: a_height >= 0
		do
			if row /= Void and grid /= Void then
				if row.height /= a_height then
					row.set_height (a_height)
					if grid.header_node = Current then
						grid.update_title_height
					end
				end
			end
			if grid /= Void and then not grid.is_synchronizing_rows.item then
				grid.synchronize_rows (Current)
			end
			height_saved := a_height
		ensure
			height_set: height = a_height
		end

	set_primary_title_node (a_title_node: RP_GRID_TITLE_NODE [G]) is
			-- Set `primary_title_node' := `a_title_node'
		do
			primary_title_node := a_title_node
		end

	enable_select is
			-- Ensure `Current' is selected.
		require
			parented: grid /= Void
		do
			row.enable_select
		end

	disable_select is
			-- Ensure `Current' is deselected.
		require
			parented: grid /= Void
		do
			row.disable_select
		end

	ensure_visible is
			-- Ensure `Current' is visible in `grid'.
		require
			parented: grid /= Void
		do
			row.ensure_visible
		end

	expand is
			-- Display all subrows of `Current'.
		require
			parented: grid /= Void
		local
			l_nodes: RP_TITLED_NODES [G]
			i, j: INTEGER
			l_count: INTEGER
			l_titles: ACTIVE_LIST [RP_GRID_NODE [G]]
		do
			Global.start_busy (Void)
			if row.subrow_count = 0 and expand_agent /= Void then
				l_nodes := expand_agent.item ([])
				if l_nodes /= Void then
					if l_nodes.titles /= Void then
						create l_titles
						l_titles.append (l_nodes.titles.secondary_title_nodes)
						l_titles.put_front (l_nodes.titles)
						from l_titles.start until l_titles.off loop
							grid.insert_subrow_internal (row, l_titles.item, l_titles.index)
							l_count := l_count.max (l_titles.item.count)
							l_titles.forth
						end
						j := l_titles.count
					end
					from
						i := 1
					until
						i > l_nodes.count
					loop
						insert_subrow (l_nodes.i_th (i), j + 1)
						l_count := l_count.max (l_nodes.i_th (i).column_count)
						j := j + 1
						i := i + 1
					end
					grid.ensure_sufficient_columns (l_count)
				end
			end
			if row.is_expandable then
					-- Block to prevent `expand' being called again.
				row.expand_actions.block
				row.expand
				row.expand_actions.resume
			end
			if row.subrow_count = 0 then
				is_ensured_non_expandable := True
			end
			expand_actions.call ([])
			Global.end_busy
		end

	collapse is
			-- Hide all subrows of `Current'.
		require
			parented: grid /= Void
		do
			row.collapse
			is_ensured_non_expandable := False
		ensure
			not_is_expanded: not is_expanded
		end

	set_data (a_data: like data) is
			-- Assign `a_data' to `data'.
		do
			data := a_data
		ensure
			data_set: data = a_data
		end

	insert_sorted (a_node: RP_GRID_NODE [G]) is
			-- Insert 'a_node' in appropriate place accordingly to current sort of `child_title'.
		require
			parented: grid /= Void
			node_not_void: a_node /= Void
			grid_enabled_as_tree: grid.is_tree_enabled
			has_title_node: child_title /= Void
		do
			insert_subrow (a_node, subrow_count + 1)
			child_title.sort
		ensure
			subrow_count_increased: subrow_count = old subrow_count + 1
			grid_row_count_increased: grid.row_count = old grid.row_count + 1
		end

	insert_subrow (a_node: RP_GRID_NODE [G]; subrow_index: INTEGER) is
			-- Add a new row to `grid' as a subrow of `Current'
			-- with index in subrows of `Current' given by `subrow_index'.
		require
			parented: grid /= Void
			node_not_void: a_node /= Void
			node_not_parented: a_node.grid = Void
			grid_enabled_as_tree: grid.is_tree_enabled
			not_inserting_before_child_title: subrow_count > 0 and then subrow (1).is_title_row implies subrow_index > 1
			valid_subrow_index: subrow_index >= 1 and subrow_index <= subrow_count + 1
		local
			l_index: INTEGER
			l_subrow: EV_GRID_ROW
		do
			is_ensured_non_expandable := False
			row.insert_subrow (subrow_index)
			if subrow_index = 1 then
				l_index := row.index + 1
			else
					-- There is a subrow before the current insert index so
					-- add at a position in `parent_i' based on the item before the insert
					-- As this item may have rows of it's own the subrow count recursive must also be added.
				l_subrow := row.subrow (subrow_index - 1)
				l_index := l_subrow.index + l_subrow.subrow_count_recursive + 1
			end
			a_node.associate_with_row (row.subrow (subrow_index), grid)
			if child_title /= Void and not a_node.is_title_row then
				child_title.ensure_sufficient_filter_columns (a_node.column_count)
			end
		ensure
			subrow_count_increased: subrow_count = old subrow_count + 1
			grid_row_count_increased: grid.row_count = old grid.row_count + 1
		end

	set_background_color (a_color: EV_COLOR) is
			-- Assign `a_color' to `background_color'
		do
			if row /= Void then
				row.set_background_color (a_color)
			end
			background_color_saved := a_color
		ensure
			background_color_set: background_color = a_color
		end

	set_foreground_color (a_color: EV_COLOR) is
			-- Assign `a_color' to `foreground_color'
		do
			if row /= Void then
				row.set_foreground_color (a_color)
			end
			foreground_color_saved := a_color
		ensure
			foreground_color_set: foreground_color = a_color
		end

	ensure_expandable is
			-- Ensure `Current' displays an expand pixmap to simulate a `row_count' greater than 0.
			-- May be used for dynamic behavior by filling subrows upon firing of `grid.row_expand_actions'.
			-- If no items are added to `Current' during the firing of `grid.row_expand_actions' then
			-- `Current' is no longer expandable. This may be re-instated by calling `ensure_expandable' again
			-- from `grid.row_expand_actions'. Note that once a subrow is added to `Current' after a call to
			-- `ensure_expandable', it is no longer be displayed as expandable if all subrows are then removed.
			-- In this case, you must explicitly call `ensure_expandable' again after removing all subrows.
		require
			parented: grid /= Void
			parent_tree_enabled: grid.is_tree_enabled
		do
			is_ensured_non_expandable := False
			row.ensure_expandable
		ensure
			is_expandable: is_expandable
		end

	ensure_non_expandable is
			-- Restore expanded state of `Current' after a call to `ensure_expandable'. Note that if a row
			-- has one or more subrows, it is always drawn as expanded, hence the "no_subrows_contained" precondition.
		require
			no_subrows_contained: subrow_count = 0
			parented: grid /= Void
		do
			is_ensured_non_expandable := True
			row.ensure_non_expandable
		ensure
			not_is_expandable: not is_expandable
		end

	set_item (a_item: RP_DATA_GRID_ITEM; a_column: INTEGER) is
			-- Replace item at position `a_column' with `a_item' extending column count if necessary
			-- This overrides the display function
		require
			valid_a_column: a_column > 0
			item_not_void_implies_item_not_parented: a_item /= void implies a_item.grid = Void
		do
			items.force (a_item, a_column)
			if grid /= Void then
				grid.grid.set_item (a_column, index, a_item)
			end
			if a_column > count then
				count := a_column
				column_count := a_column
			end
		ensure
			item_set: i_th (a_column) = a_item
		end

	set_pebble (a_pebble: like pebble) is
			-- Assign `a_pebble' to `pebble'.
			-- Sets `pebble_agent' to Void.
		do
			pebble := a_pebble
			pebble_agent := Void
		ensure
			pebble_set: pebble = a_pebble
			pebble_agent_void: pebble_agent = Void
		end

	set_pebble_agent (a_agent: like pebble_agent) is
			-- Assign `a_agent' to `pebble_agent'.
			-- Sets `pebble' to `Void'.
		do
			pebble_agent := a_agent
			pebble := Void
		ensure
			pebble_agent_set: pebble_agent = a_agent
			pebble_void: pebble = Void
		end

	hide is
			-- Prevent `Current' from being shown in `parent'.
		require
			parented: grid /= Void
		do
			row.hide
		ensure
			not_is_show_requested: not is_show_requested
		end

	show is
			-- Ensure `Current' is shown in `parent'.
		require
			parented: grid /= Void
		do
			row.show
		ensure
			is_show_requested: is_show_requested
		end

	set_is_show_requested (a_is_show_requested: BOOLEAN) is
			-- Assign `a_is_show_requested' to `is_show_requested'.
		require
			parented: grid /= Void
		do
			if a_is_show_requested then
				show
			else
				hide
			end
		ensure
			is_show_requested_set: is_show_requested = a_is_show_requested
		end

	lock is
			-- Ensure `is_locked' is `True'.
			-- `Current' is locked at it's current vertical offset from
			-- the top edge of the viewable area of `parent'.
		require
			parented: grid /= Void
		do
			row.lock_at_position (virtual_y_position - grid.virtual_y_position)
		ensure
			is_locked: is_locked
			locked_position_set: locked_position = virtual_y_position - grid.virtual_y_position
		end

	lock_at_position (a_position: INTEGER) is
			-- Ensure `is_locked' is `True' with the vertical offset from
			-- the top edge of the viewable area of `parent' set to `a_position'.
		require
			parented: grid /= Void
		do
			row.lock_at_position (a_position)
			grid.reset_widgets_contained
			grid.redraw
		ensure
			is_locked: is_locked
			locked_position_set: locked_position = a_position
		end

	unlock is
			-- Ensure `is_locked' is `False'.
		require
			parented: grid /= Void
		do
			row.unlock
			grid.reset_widgets_contained
			grid.redraw
		ensure
			not_is_locked: not is_locked
		end

	extend (a_item: RP_DATA_GRID_ITEM) is
			-- Add a new column with `a_item'
		require
			item_not_void: a_item /= Void
			not_parented: a_item.grid = Void
		do
			set_item (a_item, column_count + 1)
		ensure
			column_count_increased: column_count = old column_count + 1
			item_added: i_th (column_count) = a_item
		end

	remove_subrow (a_subrow_index: INTEGER) is
			-- Remove subrow `a_subrow_index'.
		require
			valid_subrow_index: a_subrow_index >= 1 and a_subrow_index <= subrow_count
		local
			l_subrow: RP_GRID_NODE [ANY]
		do
			l_subrow := subrow (a_subrow_index)
			grid.remove_rows (l_subrow.index, l_subrow.index + l_subrow.subrow_count_recursive)
		ensure
			subrow_count_decreased: subrow_count = old subrow_count - 1
		end

	remove_all_subrows is
			-- Remove all subrows of `Current'.
		do
			if subrow_count > 0 then
				grid.remove_rows (index + 1, index + subrow_count_recursive)
			end
		ensure
			subrow_count_is_zero: subrow_count = 0
		end

feature -- Iteration

	do_all (action: PROCEDURE [ANY, TUPLE [RP_DATA_GRID_ITEM]]) is
			-- Apply `action' to item in `Current'.
		require
			action_not_void: action /= Void
		local
			t: TUPLE [RP_DATA_GRID_ITEM]
			i: INTEGER
		do
			from
				create t
				i := 1
			until
				i > count
			loop
				t.put (i_th (i), 1)
				action.call (t)
				i := i + 1
			end
		end

	do_if (action: PROCEDURE [ANY, TUPLE [RP_DATA_GRID_ITEM]]; test: FUNCTION [ANY, TUPLE [RP_DATA_GRID_ITEM], BOOLEAN]) is
			-- Apply `action' to every item that satisfies `test'.
		require
			action_not_void: action /= Void
			test_not_void: test /= Void
		local
			t: TUPLE [RP_DATA_GRID_ITEM]
			i: INTEGER
		do
			from
				create t
				i := 1
			until
				i > count
			loop
				t.put (i_th (i), 1)
				if test.item (t) then
					action.call (t)
				end
				i := i + 1
			end
		end

	there_exists (test: FUNCTION [ANY, TUPLE [RP_DATA_GRID_ITEM], BOOLEAN]): BOOLEAN is
			-- Is `test' true for at least one item?
		require
			test_not_void: test /= Void
		local
			t: TUPLE [RP_DATA_GRID_ITEM]
			i: INTEGER
		do
			from
				create t
				i := 1
			until
				i > count or Result
			loop
				t.put (i_th (i), 1)
				Result := test.item (t)
				i := i + 1
			end
		end

	for_all (test: FUNCTION [ANY, TUPLE [RP_DATA_GRID_ITEM], BOOLEAN]): BOOLEAN is
			-- Is `test' true for all items?
		require
			test_not_void: test /= Void
		local
			t: TUPLE [RP_DATA_GRID_ITEM]
			i: INTEGER
		do
			from
				create t
				i := 1
				Result := True
			until
				i > count or not Result
			loop
				t.put (i_th (i), 1)
				Result := test.item (t)
				i := i + 1
			end
		end

feature -- Event handling

	select_actions: EV_NOTIFY_ACTION_SEQUENCE is
			-- Actions to be performed when `Current' is selected.
		do
			if select_actions_saved = Void then
				create select_actions_saved
			end
			Result := select_actions_saved
		ensure
			result_not_void: Result /= Void
		end

	deselect_actions: EV_NOTIFY_ACTION_SEQUENCE is
			-- Actions to be performed when `Current' is deselected.
		do
			if deselect_actions_saved = Void then
				create deselect_actions_saved
			end
			Result := deselect_actions_saved
		ensure
			result_not_void: Result /= Void
		end

	collapse_actions: EV_NOTIFY_ACTION_SEQUENCE is
			-- Actions to be performed when `Current' is collapsed.
		do
			if collapse_actions_saved = Void then
				create collapse_actions_saved
			end
			Result := collapse_actions_saved
		ensure
			result_not_void: Result /= Void
		end

	expand_actions: EV_NOTIFY_ACTION_SEQUENCE is
			-- Actions to be performed when `Current' is expanded.
		do
			if expand_actions_saved = Void then
				create expand_actions_saved
			end
			Result := expand_actions_saved
		ensure
			result_not_void: Result /= Void
		end

	pointer_button_press_actions: ACTION_SEQUENCE [TUPLE [INTEGER, INTEGER, INTEGER, RP_DATA_GRID_ITEM]] is
			-- Actions to be performed when `Current' is pressed.
			-- Event data is X coordinate, Y coordinate and button index and item.
		do
			if pointer_button_press_actions_saved = Void then
				create pointer_button_press_actions_saved
			end
			Result := pointer_button_press_actions_saved
		end

	pointer_double_press_actions: ACTION_SEQUENCE [TUPLE [INTEGER, INTEGER, INTEGER, RP_DATA_GRID_ITEM]] is
			-- Actions to be performed when `Current' is double clicked.
			-- Event data is X coordinate, Y coordinate, button index and item.
		do
			if pointer_double_press_actions_saved = Void then
				create pointer_double_press_actions_saved
			end
			Result := pointer_double_press_actions_saved
		end

	pointer_motion_actions: ACTION_SEQUENCE [TUPLE [INTEGER, INTEGER, RP_DATA_GRID_ITEM]] is
			-- Actions to be performed when mouse pointer moves above `Current'.
			-- Event data is X coordinate, Y coordinate and item.
		do
			if pointer_motion_actions_saved = Void then
				create pointer_motion_actions_saved
			end
			Result := pointer_motion_actions_saved
		end

feature -- To be refactored/hidden/removed/fixed

	children: ROSE_LINEAR_ARRAY [RP_GRID_NODE [G]] is
			-- Children nodes
		require
			parented: grid /= Void
		local
			l_grid_node: RP_GRID_NODE [G]
			l_title_node: RP_GRID_TITLE_NODE [G]
			i: INTEGER
		do
			create Result.make_empty
			from i := 1 until i > row.subrow_count loop
				l_grid_node ?= row.subrow(i).data
				if l_grid_node /= Void then
					l_title_node ?= l_grid_node
					if l_title_node = Void and l_grid_node.primary_title_node = Void then -- Is l_grid_node a title_node or a secondary_title_node?
						Result.extend (l_grid_node)
					end
				end
				i := i + 1
			end
		ensure
			result_ok: Result /= Void
		end

	children_count: INTEGER
			-- Number of children in this node
			-- Introduced because many callers use simply children.count, which is obviously ineffective (creating an array just to get its count while a more effective solution exists)
		do
			Result := children.count		-- This is unoptimal
--			Result := subrow_count			-- Replace by call to 'subrow_count', after verifying that subrow_count = children.count in all cases
		end

	set_static_children (a_titles: ARRAY [ANY]; a_rows: ARRAY [RP_GRID_NODE [G]]) is
			-- Set 'static_children'.
		do
			set_static_children_title (a_titles)
			create static_children.make_from_array (a_rows)
		end

	set_static_children_from_node (a_node: RP_GRID_NODE [G]) is
			-- Set 'static_children' from 'a_node'.
		local
			l_titled_nodes: RP_TITLED_NODES [G]
			i: INTEGER
			l_array: ROSE_LINEAR_ARRAY [STRING]
			l_children: ROSE_LINEAR_ARRAY [RP_GRID_NODE [G]]
			l_row, l_child_row: RP_GRID_NODE [G]
			l_label: RP_DATA_GRID_LABEL_ITEM
		do
			create l_children.make_empty
			if a_node.is_expanded then
				l_titled_nodes := a_node.expand_agent.item ([])
				create l_children.make_empty
				if l_titled_nodes.titles /= Void then
					create l_array.make_empty
					from i := 1 until i > l_titled_nodes.titles.column_count loop
						l_label ?= l_titled_nodes.titles.i_th (i)
						l_array.extend (l_label.text)
						i := i + 1
					end
				end
				from l_titled_nodes.start until l_titled_nodes.off loop
					l_row := l_titled_nodes.item
					create l_child_row.make_from_agents (l_row.display, agent l_child_row.expand_with_static_rows, l_row.column_count)
					l_children.extend (l_child_row)
					l_titled_nodes.forth
				end
				set_static_children (l_array, l_children)
			end
		end

	static_children_title: RP_GRID_TITLE_NODE [G]
			-- Static children title

	set_static_children_title (a_titles: ARRAY [ANY]) is
			-- Set static children title
		do
			if a_titles = Void then
				static_children_title := Void
			else
				create static_children_title.make (a_titles)
			end
		end

	static_children: ROSE_LINEAR_ARRAY [RP_GRID_NODE [G]]
			-- Static children

	expand_with_static_rows: RP_TITLED_NODES [G] is
			-- Expand this row with subrows in 'static_children'
		do
			if static_children = Void then
				create static_children.make_empty
			end
			create Result.make_from_array (static_children_title, static_children)
		end

	clear_widgets is
			-- Clear widgets from all columns in this row.
		local
			i: INTEGER
		do
			from
				i := 1
			until
				i > column_count
			loop
				if {l_widget_item: RP_DATA_GRID_WIDGET_ITEM} i_th (i) then
					l_widget_item.clear_widget
				end
				i := i + 1
			end
		end

	refresh is
			-- Reset `items_saved' and remove items, so they are computed again when displayed.
		do
			items_saved := Void
			if row /= Void then
				row.clear
				row.redraw
			end
		ensure
			items_saved_is_void: items_saved = Void
		end

	saved_items_wipe_out is
			-- forces items to be recreated from embeded array; useful for making unparented copies
		do
			if items_saved /= Void then
				items_saved.wipe_out
				if row /= Void and row.parent /= Void then
					row.redraw
				end
				if grid /= Void then
					grid.set_refresh_needed (0, 0)
				end
			end
		end

	saved_item_clear (i: INTEGER) is
			-- Void out entry i in items_saved
		do
			if items_saved /= Void then
				items_saved.go_i_th (i)
				items_saved.replace (Void)
				if row /= Void then
					row.redraw
				end
				if grid /= Void then
					grid.set_refresh_needed (0, 0)
				end
			end
		end

	set_expand_agent (a_expand: like expand_agent) is
			-- Assign `a_expand' to `expand_agent'.
		do
			if row /= Void then
					-- Remove expand_agent
				row.expand_actions.prune_all (once_expand_agent)
			end
			expand_agent := a_expand
			if row /= Void then
				if expand_agent /= Void then
					grid.enable_tree (True)
					row.ensure_expandable
					row.expand_actions.extend (once_expand_agent)
				else
					row.ensure_non_expandable
				end
			end
		ensure
			expand_agent_set: expand_agent = a_expand
		end

	set_display_agent (a_display: like display) is
			-- Assign `a_display' to `display'.
		do
			display := a_display
		ensure
			display_set: display = a_display
		end

	is_ensured_non_expandable: BOOLEAN
		-- Has current been flagged to be non-expandable?

feature	{RP_DATA_GRID_ACCESS} -- Implementation

	fixed: EV_FIXED is
			-- Access to internal EV_FIXED available from EV_GRID
		do
			Result := grid.fixed
		ensure
			result_not_void: Result /= Void
		end

	internal_data: ANY
			-- Internal data

	set_internal_data (a_data: ANY) is
			-- Assign `a_data' to `internal_data'.
		do
			internal_data := a_data
		end

	global_row_index: CELL [INTEGER] is
			--holds latest unique row index
		once
			create Result.put (0)
		end

	id: INTEGER
		-- Unique id representing Current'.

feature {RP_DATA_GRID, RP_GRID_NODE} -- Implementation

	items: ROSE_LINEAR_ARRAY [RP_DATA_GRID_ITEM] is
			-- Items which have been created and stored
		do
			if items_saved = Void then
				create items_saved.make_empty
			end
			Result := items_saved
		end

	items_saved: ROSE_LINEAR_ARRAY [RP_DATA_GRID_ITEM]
		-- All items current contained in `Current'.

	display: FUNCTION [ANY, TUPLE [INTEGER], ANY]
			-- Display agent

feature {RP_DATA_GRID, RP_GRID_NODE} -- Implementation

	item_indent: INTEGER is
			-- Indent of first item from edge, non-zero when tree expand node displayed.
		do
			Result := grid.grid.implementation.item_cell_indent (1, index)
		ensure
			result_non_negative: Result >= 0
		end

	integer_sort (a_int_1, a_int_2: INTEGER): BOOLEAN is
			-- sorts integers ascending
		do
			Result := a_int_1 < a_int_2
		end

	height_saved: INTEGER
		-- Internally stored height when no `row' is associated with `Current'.

	background_color_saved: EV_COLOR
		-- Internally stored background color when no `row' is associated with `Current'.

	foreground_color_saved: EV_COLOR
		-- Internally stored foreground color when no `row' is associated with `Current'.

	collapse_actions_saved: like collapse_actions
		-- Internally stored collapse actions.

	expand_actions_saved: like expand_actions
		-- Internally stored expand actions.

	select_actions_saved: EV_NOTIFY_ACTION_SEQUENCE
		-- Internally stored select_actions.

	deselect_actions_saved: EV_NOTIFY_ACTION_SEQUENCE
		-- Internally stored deselect_actions.

	is_show_requested_saved: BOOLEAN
		-- Internally stored `is_show_requested'.

	set_is_show_requested_saved (a_is_show_requested_saved: BOOLEAN) is
			-- Assign `a_is_show_requested_saved' to `is_show_requested_saved'.
		do
			is_show_requested_saved := a_is_show_requested_saved
		end

	pointer_double_press_actions_saved: ACTION_SEQUENCE [TUPLE [INTEGER, INTEGER, INTEGER, RP_DATA_GRID_ITEM]]
			-- Internally stored double press actions.

	pointer_button_press_actions_saved: ACTION_SEQUENCE [TUPLE [INTEGER, INTEGER, INTEGER, RP_DATA_GRID_ITEM]]
			-- Internally stored button press actions.

	pointer_motion_actions_saved: ACTION_SEQUENCE [TUPLE [INTEGER, INTEGER, RP_DATA_GRID_ITEM]]
			-- Internally stored motion actions.

	set_grid (a_grid: like grid) is
			-- Assign `a_grid' to `grid'.
		do
			grid := a_grid
		end

	set_row (a_row: like row) is
			-- Assign `a_row' to `row'.
		do
			row := a_row
		ensure
			row_set: row = a_row
		end

	execute_select_actions is
			-- Call all agents within `select_actions'.
		do
			from select_actions.start until select_actions.off loop
				select_actions.item.call ([])
				select_actions.forth
			end
		end

	execute_collapse_actions is
			-- Call all agents within `collapse_actions'.
		do
			if collapse_actions_saved /= Void then
				from collapse_actions.start until collapse_actions.off loop
					collapse_actions.item.call ([])
					collapse_actions.forth
				end
			end
		end

	display_widgets (a_first_visible, a_last_visible: INTEGER) is
			-- Note having widgets enabled when the grid contains a widget item with widget_height_dominant enabled can result in
			-- grid.enable_vertical_overscroll and grid.disable_row_height_fixed being set silently.
			-- FIXME: find a way to allow expected functionality of widgets without silently changing grid settings.
		local
			i, l_x, l_y, l_length, l_width: INTEGER
			l_cseparator, l_rseparator: INTEGER
			l_widget: EV_WIDGET
			l_widgets_just_refreshed: HASH_TABLE [RP_DATA_GRID_WIDGET_ITEM, INTEGER]
			l_widgets_contained_in_fixed: HASH_TABLE [RP_DATA_GRID_WIDGET_ITEM, INTEGER]
			l_height: INTEGER
			l_fixed: EV_FIXED
			l_vertical_buffer_offset, l_horizontal_buffer_offset: INTEGER
			l_is_column_locked, l_is_row_locked: BOOLEAN
			l_widget_just_inserted: BOOLEAN
			l_parents_expanded: BOOLEAN
			l_widget_span: INTEGER
			l_other_spans: INTEGER
			l_this_span: INTEGER
		do
			l_parents_expanded := grid.all_parents_are_expanded (Current)
			l_fixed := grid.fixed
			l_vertical_buffer_offset := grid.virtual_vertical_offset_of_fixed
			l_horizontal_buffer_offset := grid.virtual_horizontal_offset_of_fixed
			l_widgets_just_refreshed := grid.widgets_just_refreshed
			l_widgets_contained_in_fixed := grid.widgets_contained_in_fixed
			if grid.grid.are_column_separators_enabled then
				l_cseparator := 1
			end
			if grid.grid.are_row_separators_enabled then
				l_rseparator := 1
			end
			from i := (a_first_visible - grid.largest_widget_span).max (1) until i > a_last_visible loop
				if grid.column_displayed (i) then
					if {l_widget_item: RP_DATA_GRID_WIDGET_ITEM} i_th (i) then
						if l_parents_expanded then
							if grid.column (i).is_locked then
								l_is_column_locked := True
								l_fixed := grid.locked_column_i_for_column (grid.grid.column (i)).fixed
							elseif row.is_locked then
								l_is_row_locked := True
								l_fixed := grid.locked_row_i_for_row (row).fixed
							else
								l_fixed := grid.fixed
								l_is_column_locked := False
								l_is_row_locked := False
							end
							l_widgets_just_refreshed.put (l_widget_item, l_widget_item.object_id)
							if l_widget_item.parent /= Void then
								l_widget_item.get_widget
								l_widget := l_widget_item.widget
								if l_widget /= Void then
									l_length := l_widget.height.max (l_widget.width)
									if l_widget.parent = Void then
										l_widgets_contained_in_fixed.put (l_widget_item, l_widget_item.object_id)
										l_fixed.extend (l_widget)
										l_widget_just_inserted := True
										l_widget.resize_actions.force_extend (agent widget_minimum_size_increased (?, ?, ?, ?, l_widget))
									else
										l_widget_just_inserted := False
									end
									l_widget.resize_actions.block
									l_x := l_widget_item.virtual_x_position + l_cseparator - l_horizontal_buffer_offset
									l_y := l_widget_item.virtual_y_position - l_vertical_buffer_offset - l_rseparator
									if (parent_row = Void or else grid.all_parents_are_expanded (Current)) then
										if l_widget.x_position /= l_x or l_widget.y_position /= l_y or l_widget_just_inserted then
											if l_is_column_locked then
												l_fixed.set_item_position (l_widget, l_cseparator, l_y)
											elseif l_is_row_locked then
												l_fixed.set_item_position (l_widget, l_x, l_rseparator)
											else
												l_fixed.set_item_position (l_widget, l_x, l_y)
											end
										end
										if l_widget_item.widget_column_span > grid.largest_widget_span then
											grid.set_largest_widget_span (l_widget_item.widget_column_span)
										end
										l_widget_span := l_widget_item.widget_column_span.max (1)

										if l_widget_item.is_widget_width_dominant then
												-- If the width of the widget is dominant then we set the required
												-- width to the widgets minimum width.
											l_width := l_widget.minimum_width
										else
											l_width := 0
										end
										l_this_span := grid.column_span (i, i + l_widget_span - 1)
											-- Determine the total width of the columns associated with the
											-- widget span.
										if l_widget_item.index_in_row = 1 then
												-- If the span crosses the first column then we need to
												-- subtract the number of pixels used for the tree icons
												-- as this space is not available for the widget.
											l_this_span := l_this_span - item_indent
										end
										l_width := l_width.max (l_this_span - l_cseparator)
											-- Set the required width to the column span if larger than
											-- the required minimum width (if any).

										if l_widget_item.is_widget_width_dominant then
											if l_width > l_this_span then
													-- If the widget width is dominant and the required width is greater
													-- than the current span, then increase the width of the first spanned
													-- column, so that the total span now matches the required width.
												l_other_spans := l_this_span - grid.column (i).width
												grid.set_column_width (l_width - l_other_spans + l_cseparator, i)
											end
										else
												-- The widget's width is not dominant so set the minimum width to
												-- the current width, ensuring the widget can be positioned to the
												-- current size.
											l_widget.set_minimum_width (l_width)
										end
										if l_widget.width /= l_width then
												-- Position the widget in the the parent container.
											l_fixed.set_item_width (l_widget, l_width)
										end

										if l_widget_item.is_widget_height_dominant then
											if l_widget.minimum_height + l_rseparator > height then
												if not grid.is_vertical_overscroll_enabled then
													grid.enable_vertical_overscroll
												end
												if grid.is_row_height_fixed then
													grid.disable_row_height_fixed
												end
												l_height := l_widget.minimum_height + l_rseparator
												if height /= l_height then
													set_height (l_height)
												end
											end
											l_height := height - l_rseparator
											if l_widget.height /= l_height then
												l_fixed.set_item_height (l_widget, l_height)
											end
										else
											l_height := height - l_rseparator
											l_widget.set_minimum_height (l_height)
											if l_widget.height /= l_height then
												l_fixed.set_item_height (l_widget, l_height)
											end
										end
									end
									l_widget.resize_actions.resume
								end
							end
						end
					end
				end
				i := i + 1
			end
		end

	widget_minimum_size_increased (a_x, a_y, a_width, a_height: INTEGER; a_widget: EV_WIDGET) is
			-- Respond to widget `a_widget' resizing. This is
			-- used by `display_widgets' to expand the size of a grid item
			-- as the minimum size of the widget expands
		require
			widget_not_void: a_widget /= Void
		do
			if a_widget.parent = fixed and (a_width /= a_widget.width or a_height /= a_widget.height) then
				grid.set_refresh_needed (0, 0)
				grid.refresh_widgets
			end
		end

	associate_with_row (a_row: EV_GRID_ROW; a_data_grid: like grid) is
			-- Perform association of `Current' to real grid row `a_row'.
		require
			row_not_void: a_row /= Void
			data_grid_not_void: a_data_grid /= Void
		do
			a_row.set_data (Current)
			set_row (a_row)
			set_grid (a_data_grid)
			if expand_agent /= Void and not is_ensured_non_expandable then
				grid.enable_tree (True)
				row.ensure_expandable
			end
			if height_saved > 0 then
				a_row.set_height (height_saved)
			end
			a_data_grid.ensure_sufficient_columns (column_count)
			if background_color_saved /= Void then
				a_row.set_background_color (background_color_saved)
			end
			if foreground_color_saved /= Void then
				a_row.set_foreground_color (foreground_color_saved)
			end
		end

	dissociate is
			-- Perform dissociation of `Current' to `grid'.
		do
			if grid.are_sticky_parents_enabled then
				grid.locked_rows.prune_all (Current)
			end
			set_row (Void)
			grid.remove_event.call ([Current])
			grid.set_refresh_needed (0, 0)
			set_grid (Void)
		end

feature {RP_DATA_GRID} -- Implementation

	update_item_for_insertion (a_column: INTEGER) is
			-- Update `a_column' item of `Current' for insertion to `grid'.
		require
			grid_not_void: grid /= Void
			valid_column: a_column >= 1 and a_column <= grid.column_count
		local
			l_height: INTEGER
			l_label_item: RP_DATA_GRID_LABEL_ITEM
			l_column: RP_DATA_GRID_COLUMN [G]
		do
			if a_column = 1 and then expand_agent /= Void then
				grid.enable_tree (True)
				if not is_ensured_non_expandable then
					row.ensure_expandable
				end
				items.item (a_column).row.expand_actions.extend (once_expand_agent)
			end
			l_label_item ?= items.item (a_column)
			if l_label_item /= Void then
				l_column := grid.column (a_column)
				if l_column.is_left_aligned then
					l_label_item.align_text_left
				elseif l_column.is_center_aligned then
					l_label_item.align_text_center
				elseif l_column.is_right_aligned then
					l_label_item.align_text_right
				end
			end
			l_height := row.height.max (height)
			if height_saved > 0 and height_saved /= l_height then
				set_height (l_height)
			end
		end

feature {NONE} -- Implementation

	array_item(a_array: ARRAY [ANY]; i: INTEGER): ANY is
			-- `Result' is item `i' in `a_array' or an empty
			-- string if `i' is not a valid index in `a_array'.
		do
			if a_array.valid_index (i) then
				Result := a_array.item(i)
			else
				Result := ""
			end
		end

	once_expand_agent: PROCEDURE [ANY, TUPLE []] is
			-- Agent representing call to `expand', once per node.
			-- Necessary to add and prune from `row.expand_actions'.
		do
			if expand_agent_internal = Void then
				expand_agent_internal := agent expand
			end
			Result := expand_agent_internal
		end

	expand_agent_internal: PROCEDURE [ANY, TUPLE []]
		-- Implementation of `once_expand_object'.

end
