note

	description:

		"Objects that represent title rows within an RP_DATA_GRID."

	author: "Julian Rogers"
	last_editor: "$Author: er $"
	date: "$Date: 2014/02/11 $"
	revision: "$Revision: #68 $"
	path: "$File: //rose/source/kernel/infrastructure/widgets/grid/rp_grid_title_node.e $"

class
	RP_GRID_TITLE_NODE[G]
inherit
	RP_GRID_NODE[G]
		rename
			make as rp_grid_node_make,
			make_from_array as make
		redefine
			make,
			is_title_row,
			i_th
		end

	EV_FONT_CONSTANTS

	RP_DATA_GRID_ACCESS
		undefine
			default_create, copy, is_equal
		end

create
	make

feature {NONE} -- Initialization

	make(a_array : ARRAY[ANY]) is
			-- Create with titles `a_array'
		do
			count := a_array.count
			set_titles (a_array)
			create sort_states
			sort_states.add_actions.extend (agent item_added_to_sort_states)
			sort_states.remove_actions.extend (agent item_removed_from_sort_states)
				-- Now connect sort agents.
			sort_state_agent := agent update_sort_states
			sort_agent := agent perform_sort
			create secondary_title_nodes
			secondary_title_nodes.compare_references
			secondary_title_nodes.add_actions.extend (agent item_added_to_secondary_title_nodes)
			secondary_title_nodes.remove_actions.extend (agent item_removed_from_secondary_title_nodes)
			create non_user_sortable_items.make_empty
		end

feature -- Setter

	set_titles (a_array: ARRAY [ANY]) is
			-- Set the titles
		require
			non_void_array: a_array /= Void
		do
			saved_items_wipe_out
			display := agent title_item(a_array,?)
			column_count := a_array.count
		end


feature -- Access

	sort_states: ACTIVE_LIST [RP_DATA_GRID_SORT_STATE]
		-- Information specifying sort behavior at next call to `sort'.
		-- Each item represents a sort within the previous item (if any).
		-- For example, to sort ascending on column 3 and then descending on
		-- column 4 within the sort of column 3, perform the following (assuming `sort_states' is already empty):
		-- sort_states.extend (create {RP_DATA_GRID_SORT_STATE}.make_with_column_and_direction (3, True))
		-- sort_states.extend (create {RP_DATA_GRID_SORT_STATE}.make_with_column_and_direction (4, False))
		-- Modifying the contents of `Current' causes the associated sort icons for a column (if displayed) to
		-- be updated to reflect the new state.
		-- Note that the contents of `sort_states' may be modified in repsonse to a user selecting the
		-- sort icons if displayed.

	regular_expression : LX_DFA_REGULAR_EXPRESSION
		-- Regular expression applied when sorting.

	sort_state_of_column (a_column_index: INTEGER): RP_DATA_GRID_SORT_STATE is
			-- `Result' is sort state within `sort_states' with `column_index'
			-- matching `a_column_index', or Void if None.
		local
			cursor: CURSOR
		do
			cursor := sort_states.cursor
			from
				sort_states.start
			until
				sort_states.off or Result /= Void
			loop
				if sort_states.item.column_index = a_column_index then
					Result := sort_states.item
				end
				sort_states.forth
			end
			sort_states.go_to (cursor)
		ensure
			result_not_void_implies_column_index_matches: Result /= Void implies Result.column_index = a_column_index
			sort_states_index_not_changed: sort_states.index = old sort_states.index
		end

	secondary_title_nodes : ACTIVE_LIST [RP_GRID_NODE[G]]
		-- Additional nodes to be displayed under Current and not moved when sort is called
		-- Useful for multi-line titles or displaying header widgets

	is_filter_shown : BOOLEAN is
			-- Is filter_node active?
		do
			Result := secondary_title_nodes.has (filter_node)
		end

	filter_node : RP_GRID_NODE [G] is
			-- Specialized node which displays fields for reg_ex filtering of sibling rows
		local
			i : INTEGER
			l_text_field : EV_TEXT_FIELD
			l_items : ROSE_LINEAR_ARRAY [RP_DATA_GRID_WIDGET_ITEM]
		do
			if filter_node_saved = Void then
				create l_items.make_empty
				from i := 1 until i > column_count loop
					create l_text_field
					l_text_field.return_actions.extend (agent filter_from_filter_node)
					l_text_field.change_actions.extend (agent update_regular_expression_field (l_text_field))
					l_text_field.key_press_actions.extend (agent handle_key_press_in_filter_text)
					l_items.extend (create {RP_DATA_GRID_WIDGET_ITEM}.make (l_text_field))
					l_text_field.enable_sensitive
					l_text_field.enable_edit
					l_items.last.set_widget_height_dominant (True)
					i := i + 1
				end
				create filter_node_saved.make_from_array (l_items)
			end
			Result := filter_node_saved
		end

	sort_state_agent: PROCEDURE [ANY, TUPLE [INTEGER, INTEGER]]
		-- Agent to update `sort_states' in response to a user clicking
		-- a sort icon. Replace this agent via `set_sort_state_agent' to
		-- customize the default behavior.
		-- Event data is as follows:
		-- 1. The index of the column
		-- 2. The pointer button pressed

	sort_agent: PROCEDURE [ANY, TUPLE [INTEGER, INTEGER]]
		-- Agent to perform sorting in response to a user clicking
		-- a sort icon. Replace this agent via `set_sort_agent' to
		-- customize the default behavior.
		-- Event data is as follows:
		-- 1. The index of the column
		-- 2. The pointer button pressed

	i_th (i : INTEGER) : RP_DATA_GRID_ITEM is
			-- Item `i' within `Current'.
		local
			l_item : RP_DATA_GRID_LABEL_ITEM
		do
			Result := Precursor {RP_GRID_NODE} (i)
			l_item ?= Result
			if l_item /= Void and then l_item.sort_pixmap = Void then
				decorate_title_item (l_item, i)
			end
		end

feature -- Basic Operations

	filter (a_expressions : HASH_TABLE [LX_DFA_REGULAR_EXPRESSION, INTEGER]) is
			-- Remove (/restore) sibling nodes which do not (/do) pass filter
			-- i.e. text representation of i_th item does not match regex at a_expressions.item (i)
		require
			a_expressions_not_void: a_expressions /= Void
		local
			l_filtered_nodes : ROSE_LINEAR_ARRAY [RP_GRID_NODE [G]]
			l_is_grid_filter : BOOLEAN
			l_row : RP_GRID_NODE [G]
			l_child_rows : ROSE_LINEAR_ARRAY [RP_GRID_NODE [G]]
			l_parent_grid : RP_DATA_GRID [G]
			i : INTEGER
		do
			global.start_busy (Void)
			l_is_grid_filter := grid.is_title_grid
			if filtered_nodes = Void then
				create filtered_nodes.make_empty
				filtered_nodes.compare_objects
			end
			create l_filtered_nodes.make_from_array (filtered_nodes)
			filtered_nodes.wipe_out
			from l_filtered_nodes.start until l_filtered_nodes.off loop
				l_row := l_filtered_nodes.current_item
				if l_row.grid /= Void then
					if passes_filter (a_expressions, l_row) then
						l_row.show
					end
				end
				l_filtered_nodes.forth
			end
			if l_is_grid_filter then
				l_parent_grid := grid.parent_grid
				from i := l_parent_grid.row_count until i < 1 loop
					l_row := l_parent_grid.row (i)
					if l_row.parent_row = Void then -- Is this a top level row?
						if not passes_filter (a_expressions, l_row) then
							l_row.hide
							filtered_nodes.extend (l_row)
						end
					end
					i := i - 1
				end
			else
				l_child_rows := parent_row.children
				from i := l_child_rows.lower until i > l_child_rows.upper loop
					l_row := l_child_rows.item (i)
					if not passes_filter (a_expressions, l_row) then
						l_row.hide
						filtered_nodes.extend (l_row)
					end
					i := i + 1
				end
			end
				-- FIXME scroll to first visible child row if any which may be hidden above header
				-- Needs row.hide to be implemented
			if filter_actions_internal /= Void then
				filter_actions_internal.call ([])
			end
			Global.end_busy
		end

	filter_node_texts: ROSE_LINEAR_ARRAY [STRING] is
			-- All texts currently entered in `filter_node'.
		local
			i: INTEGER
			l_widget_item : RP_DATA_GRID_WIDGET_ITEM
			l_text_field : EV_TEXT_FIELD
			l_text: STRING
		do
			create Result.make_empty
			from
				i := 1
			until
				i > count
			loop
				l_widget_item ?= filter_node.i_th (i)
				if l_widget_item /= Void then
					l_text_field ?= l_widget_item.widget
					if l_text_field /= Void then -- l_widget_item.widget_access.widget will be Void if the item has never been displayed
						l_text := l_text_field.text
					end
				end
				Result.extend (l_text)
				i := i + 1
			end
		ensure
			result_not_void: Result /= Void
			result_count_equals_count: Result.count = count
			result_does_not_contain_void: Result.for_all (agent string_not_void)
		end

	filter_from_filter_node is
			-- Create regular expressions from text in fields of `filter_node' and fire `filter'
		local
			l_regex_hash : HASH_TABLE [LX_DFA_REGULAR_EXPRESSION, INTEGER]
			i : INTEGER
			l_string : STRING
			l_reg_exp: LX_DFA_REGULAR_EXPRESSION
			l_filter_texts: like filter_node_texts
		do
			create l_regex_hash.make (1)
			l_filter_texts := filter_node_texts
			from
				i := 1
			until
				i > l_filter_texts.count
			loop
				l_string := l_filter_texts.item (i)
				if not l_string.is_empty then
					create l_reg_exp.make
					l_reg_exp.compile (l_string, True)
					if l_reg_exp.is_compiled then
						l_regex_hash.force (l_reg_exp, i)
					end
				end
				i := i + 1
			end
			filter (l_regex_hash)
		end

	passes_filter (a_expressions : HASH_TABLE [LX_DFA_REGULAR_EXPRESSION, INTEGER]; a_row : RP_GRID_NODE [G]) : BOOLEAN is
			-- Does text representation of items of `a_row' match corresponding regexs in `a_expressions'?
		require
			a_expressions_not_void : a_expressions /= Void
			a_row_not_vid : a_row /= Void
		local
			 l_failed : BOOLEAN
			 l_item : RP_DATA_GRID_ITEM
			 l_string : STRING
			 l_regex : LX_DFA_REGULAR_EXPRESSION
		do
			from a_expressions.start until a_expressions.off or l_failed loop
				l_regex := a_expressions.item_for_iteration
				l_item := a_row.i_th (a_expressions.key_for_iteration)
				if {l_label_item : RP_DATA_GRID_LABEL_ITEM} l_item then
					if {l_widget_item : RP_DATA_GRID_WIDGET_ITEM} l_label_item then
						if {l_textable : EV_TEXTABLE} l_widget_item.widget then
							l_string := l_textable.text
						end
					else
						l_string := l_label_item.text
					end
				else
					l_string := l_item.out
				end
				if l_string = Void or else l_string.count = 0 or else not l_regex.matches (l_string) then
					l_failed := True
				end
				a_expressions.forth
			end
			Result := not l_failed
		end

	show_filter (a_show : BOOLEAN) is
			-- Show or hide gui for filtering subrows
		require
			parented: grid /= Void
		do
			if a_show then
				if not secondary_title_nodes.has (filter_node) then
					secondary_title_nodes.extend (filter_node) -- Callback from ACTIVE_LIST will insert row into grid
					if row.is_locked and grid.are_sticky_parents_enabled then
						filter_node.row.lock_at_position (grid.height_of_locked_rows)
						grid.locked_rows.extend (filter_node)
					end
					if filter_show_actions_internal /= Void then
						filter_show_actions_internal.call (Void)
					end
				end
			else
				if filter_node.row.is_locked then
					filter_node.row.unlock
					grid.locked_rows.prune_all (filter_node)
				end
				secondary_title_nodes.start
				secondary_title_nodes.prune (filter_node)
				if filter_hide_actions_internal /= Void then
					filter_hide_actions_internal.call (Void)
				end
			end
		end

	clear_filter_node is
			-- Ensure all fields in `filter_node' are empty.
			-- Note that this does not explicitly re-filter, so you may wish to
			-- call `filter_from_filter_node' if you need that to occur.
		local
			i: INTEGER
		do
			from
				i := 1
			until
				i > count
			loop
				if {l_widget_item: RP_DATA_GRID_WIDGET_ITEM} filter_node.i_th (i) then
					if {l_text_field: EV_TEXT_FIELD} l_widget_item.widget then
						l_text_field.remove_text
					end
				end
				i := i + 1
			end
		end

	set_column_user_sortable (a_sortable: BOOLEAN; a_column_index: INTEGER) is
			-- Turn on or off the ability for users to sort the column at index `a_column_index'
			-- based on `a_sortable'.
			-- Note that the grid level sortable settings must be enabled on `Current' for the
			-- sort icon to be displayed.
		do
			non_user_sortable_items.force (not a_sortable, a_column_index)
		ensure
			column_user_sortable_set: is_column_user_sortable (a_column_index) = a_sortable
		end

	is_column_user_sortable (a_column_index: INTEGER): BOOLEAN is
			-- Is column `a_column_index' sortable by users?
		do
			Result := True
			if non_user_sortable_items.valid_index (a_column_index) then
				Result := not non_user_sortable_items.item (a_column_index)
			end
		end

	is_filtered (a_item : RP_GRID_NODE [G]) : BOOLEAN is
			-- Hos the item been hidden due to a filter
		do
			Result := filtered_nodes /= Void and then filtered_nodes.has (a_item)
		end

feature -- Status setting

	set_sort_state_agent (a_sort_state_agent: like sort_state_agent) is
			-- Assign `a_sort_state_agent' to `sort_state_agent', enabling customization of
			-- sorting states in response a user clicking on a sort icon.
		require
			a_sort_state_agent_not_void: a_sort_state_agent /= Void
		do
			sort_state_agent := a_sort_state_agent
		ensure
			sort_state_agent_set: sort_state_agent = a_sort_state_agent
		end

	set_sort_agent (a_sort_agent: like sort_agent) is
			-- Assign `a_sort_agent' to `sort_agent', enabling customization of
			-- sorting in response a user clicking on a sort icon.
		require
			a_sort_agent_not_void: a_sort_agent /= Void
		do
			sort_agent := a_sort_agent
		ensure
			sort_agent_set: sort_agent = a_sort_agent
		end

	set_regular_expression (a_string : STRING) is
			-- Assign `a_string' to `regular_expression'.
		do
			create regular_expression.compile_case_insensitive (a_string)
		end

	sort is
			-- Sort all subrows associated with `Current' based on information in `sort_states'.
		local
			l_is_child_row: BOOLEAN
			l_final_row_index: INTEGER
			l_next_title_row: like Current
			l_current_row: RP_GRID_NODE [G]
			i: INTEGER
		do
			sort_started_actions.call ([Current])
			if grid.is_title_grid then
				if grid.parent_grid.row_count > 0 then
					grid.parent_grid.internal_sort (1, grid.parent_grid.row_count, True, Void, grid.parent_grid.sort_states, Void)
				end
			else
				l_is_child_row := parent_row /= Void
				if l_is_child_row then
					l_final_row_index := parent_row.index + parent_row.subrow_count_recursive
				else
					l_final_row_index := grid.row_count
				end

					-- We now determine if there is another title row before the already computed final row.
					-- If so, we will only sort up until the next title row.
				from
					i := row.index + 1 + secondary_title_nodes.count
				until
					i > l_final_row_index or l_next_title_row /= Void
				loop
					l_current_row := grid.row (i)
					l_next_title_row ?= l_current_row
					i := i + l_current_row.subrow_count_recursive + 1
				end
				if l_next_title_row /= Void then
						-- We found another title row, so adjust the final index accordingly.
					l_final_row_index := l_next_title_row.index - 1
				end
				if row.index + 1 + secondary_title_nodes.count < l_final_row_index then
					sort_rows (row.index + 1 + secondary_title_nodes.count, l_final_row_index)
				end
			end
			sort_ended_actions.call ([Current])
		end

	sort_rows (a_start_index, a_end_index: INTEGER) is
			-- Sort all rows associated with `Current' from `a_start_index' `to a_end_index' inclusive,
			-- based on information in `sort_states'. The row indexes are the actual indexes of the
			-- rows in the grid. For example, you could sort all rows associated with a title node by performing the
			-- following: title_node.sort_rows (title_node.index + 1, title_node.parent.index + title_node.parent.subrow_count_recursive)
		require
			sort_states_exist: sort_states /= Void
			parented: grid /= Void
			valid_indexes_when_subrow: parent_row /= Void implies a_start_index > index and a_end_index <= parent_row.index + parent_row.subrow_count_recursive and a_start_index <= a_end_index
			valid_indexes_when_not_subrow: parent_row = Void implies a_start_index > index and a_end_index <= grid.row_count
		do
			if not sort_states.is_empty then
					-- Nothing to do if there are no sort states defined.
				grid.internal_sort (a_start_index, a_end_index, True, row.parent_row, sort_states, Void)
			end
		end

	set_secondary_nodes_above_regular (a_value: like secondary_nodes_above_regular) is
			-- Call this with `True' if you want the secondary nodes above the regular header
		do
			secondary_nodes_above_regular := a_value
		end

feature -- Event handling

	filter_actions: EV_NOTIFY_ACTION_SEQUENCE is
			-- Actions called after filtering has been performed.
		do
			if filter_actions_internal = Void then
				create filter_actions_internal
			end
			Result := filter_actions_internal
		end

	filter_show_actions: EV_NOTIFY_ACTION_SEQUENCE is
			-- Actions called after the filter has been shown.
		do
			if filter_show_actions_internal = Void then
				create filter_show_actions_internal
			end
			Result := filter_show_actions_internal
		end

	filter_hide_actions: EV_NOTIFY_ACTION_SEQUENCE is
			-- Actions called after the filter has been hidden.
		do
			if filter_hide_actions_internal = Void then
				create filter_hide_actions_internal
			end
			Result := filter_hide_actions_internal
		end

	sort_started_actions: EV_NOTIFY_ACTION_SEQUENCE is
			-- Actions called at the start of feature `sort'
		do
			if sort_started_actions_internal = Void then
				create sort_started_actions_internal
			end
			Result := sort_started_actions_internal
		end

	sort_ended_actions: EV_NOTIFY_ACTION_SEQUENCE is
			-- Actions called after `sort' has been called.
		do
			if sort_ended_actions_internal = Void then
				create sort_ended_actions_internal
			end
			Result := sort_ended_actions_internal
		end

feature {RP_DATA_GRID, RP_DATA_GRID_COLUMN} -- Implementation

	up_pixmap : EV_PIXMAP is
			-- Pixmap used for the up sort icon.
		once
			Result := pixmaps.gui.factory.new_pixmap_from_file ("up_triangle.ico")
			if Result = Void then
				Result := pixmaps.gui.factory.new_arrow_pixmap ('^', 15, 15, colors.light_gray,colors.blue)
			end
		ensure
			result_not_void: Result /= Void
		end
	down_pixmap : EV_PIXMAP is
			-- Pixmap used for the down sort icon.
		once
			Result := pixmaps.gui.factory.new_pixmap_from_file ("down_triangle.ico")
			if Result = Void then
				Result := pixmaps.gui.factory.new_arrow_pixmap ('v', 15, 15, colors.light_gray,colors.blue)
			end
		ensure
			result_not_void: Result /= Void
		end

	unsorted_pixmap: EV_PIXMAP is
			-- Pixmap for the Inactive Sort header state
		once
			Result := pixmaps.gui.factory.new_pixmap_from_file ("inactive_sort.ico")
			if Result = Void then
				Result := pixmaps.gui.factory.new_arrow_pixmap ('v', 15, 15, colors.light_gray, colors.gray)
			end
		end

	title_item(a_array : ARRAY[ANY];i : INTEGER) : RP_DATA_GRID_ITEM is
			-- The data grid item representing the title
		require
			non_void_array: a_array /= Void
		local
			l_label_item: RP_DATA_GRID_LABEL_ITEM
		do
			if a_array.valid_index (i) then
				Result := any_to_grid_item (a_array.item (i))
				Result.set_data(a_array.item(i))
			else
				Result := create {RP_DATA_GRID_LABEL_ITEM}.make_with_text("")
			end
			l_label_item ?= Result
			if l_label_item /= Void and then l_label_item.sort_pixmap = Void then
				decorate_title_item (l_label_item, i)
			end
			count := count.max (i)
		end

	decorate_title_item (l_label_item: RP_DATA_GRID_LABEL_ITEM; i: INTEGER) is
			-- Apply title decoration to `l_label_item' at column `i'.
		require
			l_label_item_not_void: l_label_item /= Void
			i_non_negative: i >= 0
		local
			l_pixmap : EV_PIXMAP
			sort_state: RP_DATA_GRID_SORT_STATE
		do
			l_label_item.set_font(bold_font)
			if i <= count then
					-- As `title_item' is called to silently insert a new column,
					-- it is possible that this index is not valid.
				sort_state := sort_state_of_column (i)
				if sort_state = Void then
					l_label_item.set_sort_pixmap (unsorted_pixmap)
				else
					if sort_state.ascending then
						l_pixmap := up_pixmap
					else
						l_pixmap := down_pixmap
					end
					l_label_item.set_sort_pixmap (l_pixmap)
				end
			else
				l_label_item.set_sort_pixmap (unsorted_pixmap)
			end
		end

feature {RP_DATA_GRID_FILTER_GUI} -- Implementation

	update_sort_states_on_click (a_column : INTEGER) is
			-- Update sort states in response to a click on column `a_column'.
			-- If colunm `a_column' is not sorted, then sort ascending, otherwise
			-- toggle ascending state.
			-- If Ctrl pressed then add to current sort.
			-- If Alt pressed, always sort in reverse.
		require
			valid_column: a_column >= 1 and a_column <= count
		local
			sort_state: RP_DATA_GRID_SORT_STATE
		do
			sort_state := sort_state_of_column (a_column)
			if not gui.ev_application.ctrl_pressed then
				sort_states.wipe_out
			end
			if sort_state /= Void then
				if not gui.ev_application.alt_pressed then
					sort_state.set_ascending (not sort_state.ascending)
				else
					sort_state.set_ascending (False)
				end
				if not sort_states.has (sort_state) then
					sort_states.extend (sort_state)
				end
			else
				sort_states.extend (create {RP_DATA_GRID_SORT_STATE}.make_with_column_and_direction (a_column, not gui.ev_application.alt_pressed))
			end
		end

feature {RP_DATA_GRID, RP_GRID_NODE} -- Implementation

	ensure_sufficient_filter_columns (a_column : INTEGER) is
			-- Ensure filter_node has at least as many columns as longest sibling row
		local
			i : INTEGER
			l_field : EV_TEXT_FIELD
			l_item :RP_DATA_GRID_WIDGET_ITEM
		do
			if filter_node_saved /= Void and then filter_node_saved.column_count < a_column.max (max_sibling_columns) then
				from i := filter_node_saved.column_count + 1 until i > a_column.max (max_sibling_columns) loop
					create l_field
					l_field.return_actions.extend (agent filter_from_filter_node)
					l_field.change_actions.extend (agent update_regular_expression_field (l_field))
					create l_item.make (l_field)
					filter_node_saved.set_item (l_item, i)
					i := i + 1
				end
			end
			max_sibling_columns := max_sibling_columns.max (a_column)
		end

	max_sibling_columns : INTEGER
			-- Maximim columns in any siblinb_row Current is a title for
			-- Used to inititalize filter_node

	respond_to_button_press (i : INTEGER; a_x, a_y, a_button: INTEGER) is
			-- Perform processing in response to a button `a_button' being pressed on item `i' in `Current'.
		require
			valid_item_index: i >= 1 and i <= count
		local
			l_pixmap_offset: INTEGER
			label_item: RP_DATA_GRID_LABEL_ITEM
--			menu: EV_MENU
--			unlock_menu_item, lock_menu_item: EV_MENU_ITEM
			l_total_icon_width: INTEGER
		do
			if not grid.is_showing_as_resizeable then
				label_item ?= i_th (i)
				if label_item /= Void then
						-- This calculation is taken directly from `draw_title_icons' of RP_DATA_GRID.
						-- This shared code should be refactored at some point.
					if (not grid.is_title_grid and grid.are_sort_icons_displayed) or (grid.is_title_grid and grid.parent_grid.are_sort_icons_displayed_on_header_node) then
						if is_column_user_sortable (i) then
							l_total_icon_width := l_total_icon_width + label_item.sort_pixmap.width + grid.icon_spacing
						end
					end
					if (not grid.is_title_grid and grid.are_selection_icons_displayed) or (grid.is_title_grid and grid.parent_grid.are_selection_icons_displayed_on_header_node) then
						l_total_icon_width := l_total_icon_width + grid.selection_pixmap.width + grid.icon_spacing
					end
					if grid.is_title_grid and grid.parent_grid.are_lock_icons_displayed_on_header_node then
						l_total_icon_width := l_total_icon_width + grid.locked_pixmap.width
					end
					if label_item.is_left_aligned then
						l_pixmap_offset := label_item.text_width + label_item.left_border + grid.icon_spacing
					elseif label_item.is_center_aligned then
						l_pixmap_offset := (label_item.width - label_item.left_border - label_item.right_border - label_item.text_width) // 2 + label_item.text_width + grid.icon_spacing
					elseif label_item.is_right_aligned then
						l_pixmap_offset := label_item.width - label_item.right_border - label_item.text_width - grid.icon_spacing - l_total_icon_width
					end
--					l_pixmap_offset := label_item.text_width + label_item.left_border + grid.icon_spacing
--					if grid.is_title_grid and then a_x > label_item.width - 8 and a_x < label_item.width - 2 then
--						create menu
--						create lock_menu_item.make_with_text_and_action ("Lock Column", agent lock_column (i))
--						menu.extend (lock_menu_item)
--						create unlock_menu_item.make_with_text_and_action ("Unlock Column", agent unlock_column (i))
--						if grid.column (i).is_locked then
--							lock_menu_item.disable_sensitive
--						else
--							unlock_menu_item.disable_sensitive
--						end
--						menu.extend (unlock_menu_item)
--						menu.show
--					end
					if ((not grid.is_title_grid and grid.are_sort_icons_displayed) or (grid.is_title_grid and grid.parent_grid.are_sort_icons_displayed_on_header_node)) then
						if is_column_user_sortable (i) then
							if a_x >= l_pixmap_offset and then a_x <= l_pixmap_offset + label_item.sort_pixmap.width then
								sort_state_agent.call ([i, a_button])
								sort_agent.call ([i, a_button])
							end
							l_pixmap_offset := l_pixmap_offset + label_item.sort_pixmap.width + grid.icon_spacing
						end
					end
					if (not grid.is_title_grid and grid.are_selection_icons_displayed) or (grid.is_title_grid and grid.parent_grid.are_selection_icons_displayed_on_header_node) then
						if a_x >= l_pixmap_offset and then a_x <= l_pixmap_offset + grid.selection_pixmap.width then
							if grid.is_title_grid then
								grid.parent_grid.select_items_in_column (i, 1, grid.parent_grid.row_count)
							else
								if parent_row.subrow_count > 1 then
										-- There is nothing to do if there are no other children in the parent row.
									grid.select_items_in_column (i, parent_row.index + 2, parent_row.index + parent_row.subrow_count)
								end
							end
						end
						l_pixmap_offset := l_pixmap_offset + grid.selection_pixmap.width + grid.icon_spacing
					end
					if grid.is_title_grid and grid.parent_grid.are_lock_icons_displayed_on_header_node then
						if a_x >= l_pixmap_offset and then a_x <= l_pixmap_offset + grid.unlocked_pixmap.width then
							if grid.column (i).is_locked then
								grid.parent_grid.column (i).unlock
							else
								grid.parent_grid.column (i).lock
							end
						end
						l_pixmap_offset := l_pixmap_offset + grid.unlocked_pixmap.width + grid.icon_spacing
					end
				end
			end
		end

feature {NONE} -- Implementation

	non_user_sortable_items: ROSE_LINEAR_ARRAY [BOOLEAN]
		-- `item' `i' contains the status of whether that column should display a sort icon.
		-- If `i' is outside the bounds, then column `i' is in the default state which is sortable.

	filter_node_saved : RP_GRID_NODE [G]
			-- Node that contains the boxes used for allowing filtering

	filtered_nodes : ROSE_LINEAR_ARRAY [RP_GRID_NODE [G]]
			-- Nodes which have been filtered and removed from grid
			-- Used to restore filtered nodes.

	filter_actions_internal: EV_NOTIFY_ACTION_SEQUENCE
			-- Action sequence called when filtering

	filter_show_actions_internal: EV_NOTIFY_ACTION_SEQUENCE
			-- Action sequence called when the filter is shown

	filter_hide_actions_internal: EV_NOTIFY_ACTION_SEQUENCE
			-- Action sequence called when the filter is hidden

	sort_ended_actions_internal: EV_NOTIFY_ACTION_SEQUENCE
			-- Action sequence called when sorting has completed

	sort_started_actions_internal: EV_NOTIFY_ACTION_SEQUENCE
			-- Action sequence called when sorting begins

	is_title_row: BOOLEAN is
			-- Is `Current' a title node?
		do
			Result := True
		end

	colors : ROSE_STOCK_COLORS is
			-- Once access to stock colors.
		once
			create Result
		end

	pixmaps : ROSE_STOCK_PIXMAPS is
			-- Once access to stock pixmaps.
		once
			create Result
		end

	compare_rows(a_1,a_2 : ARRAY[RP_DATA_GRID_CELL[G]]) : BOOLEAN is
			-- Compare the data in two rows
		local
			l_1,l_2 : COMPARABLE
			l_item_1,l_item_2 : EV_GRID_ITEM
			l_rp_data_grid_item: RP_DATA_GRID_ITEM
			l_string_1,l_string_2 : STRING
			i : INTEGER
			l_differ : INTEGER
		do
			from i := 1 until i > a_1.count or else l_differ /= 0 loop
				l_differ := 0
				l_item_1 := a_1.item(i).item
				l_item_2 := a_2.item(i).item
				if l_item_1 /= void then
					l_1 ?= l_item_1.data
					l_rp_data_grid_item ?= l_item_1
					if l_rp_data_grid_item /= Void then
						l_1 ?= l_rp_data_grid_item.comparable
					end
				else
					l_1 ?= a_1.item(i).data
				end
				if l_item_2 /= void then
					l_2 ?= l_item_2.data
					l_rp_data_grid_item ?= l_item_2
					if l_rp_data_grid_item /= Void then
						l_2 ?= l_rp_data_grid_item.comparable
					end
				else
					l_2 ?= a_2.item(i).data
				end
				if a_1.item(i).matches_filter /= a_2.item(i).matches_filter then
					if a_1.item(i).matches_filter then
						l_differ := -1
					else
						l_differ := 1
					end
				else
					if l_1 /= void and then l_2 /= void and then l_1.same_type(l_2) then
						if l_1 < l_2 then
							l_differ := -1
						elseif l_2 < l_1 then
							l_differ := 1
						end
					else
						l_string_1 := a_1.item(i).string
						l_string_2 := a_2.item(i).string
						if l_string_1 < l_string_2 then
							l_differ := -1
						elseif l_string_1 > l_string_2 then
							l_differ := 1
						end
					end
				end
				if reverse_sorted_columns.item (i) then
					l_differ := - l_differ
				end
				i := i + 1
			end
			Result := l_differ = -1
		end

	reverse_sorted_columns: ARRAY [BOOLEAN]
		-- This is temporarily used during a sort to permit a quick lookup of
		-- the sort state from a paticular column index. It is only valid during
		-- a sort and is built from `sort_states'.

	item_added_to_sort_states (an_item: RP_DATA_GRID_SORT_STATE) is
			-- `an_item' has been added to `sort_states' so update icon
			-- of column `an_item.column_index' to reflect state.
		require
			an_item_not_void: an_item /= Void
		do
			update_sort_icon (an_item.column_index)
		end

	item_removed_from_sort_states (an_item: RP_DATA_GRID_SORT_STATE) is
			-- `an_item' has been removed from `sort_states' so update icon
			-- of column `an_item.column_index' to reflect state.
		do
			update_sort_icon (an_item.column_index)
		end

	update_sort_icon (a_column_index: INTEGER) is
			-- Update displayed sort icon for item `a_column_index'
		require
			a_column_index_valid: a_column_index >= 1 and a_column_index <= grid.column_count
		local
			label_item: RP_DATA_GRID_LABEL_ITEM
			l_pixmap: EV_PIXMAP
			sort_state: RP_DATA_GRID_SORT_STATE
		do
			label_item ?= i_th (a_column_index)
			sort_state := sort_state_of_column (a_column_index)
			if sort_state = Void then
				label_item.set_sort_pixmap (unsorted_pixmap)
			else
				if sort_state.ascending then
					l_pixmap := up_pixmap
				else
					l_pixmap := down_pixmap
				end
				label_item.set_sort_pixmap (l_pixmap)
			end
				-- As the icons are drawn as a post-draw, we need to ensure the
				-- complete row gets re-drawn.
			row.redraw
		end

	update_sort_states (a_column: INTEGER; a_button: INTEGER) is
			-- Update `sort_states' in response to a user clicking a sort icon for
			-- column `a_column' with button `a_button'.
		local
			sort_state: RP_DATA_GRID_SORT_STATE
		do
			if grid.is_title_grid then
				grid.parent_grid.update_sort_states_on_click (a_column, a_button)
			else
				if a_button = 1 then
					update_sort_states_on_click (a_column)
				elseif a_button = 3 then
					sort_state := sort_state_of_column (a_column)
					if sort_state /= Void then
						sort_state.set_ascending (True)
					end
				end
			end
				-- As the icons are drawn as a post-draw, we need to ensure the
				-- complete row gets re-drawn.
			row.redraw
		end

	perform_sort (a_column: INTEGER; a_button: INTEGER) is
			-- Perform sorting in response to a user clicking a sort icon for
			-- column `a_column' with button `a_button'.
		local
			l_dialog: EV_POPUP_WINDOW
			l_filter: RP_DATA_GRID_FILTER_GUI [G]
			l_horizontal_box: EV_HORIZONTAL_BOX
			l_grid: RP_DATA_GRID [G]
		do
			if grid.is_title_grid then
				l_grid := grid.parent_grid
			else
				l_grid := grid
			end
			if a_button = 1 then
				sort
			elseif a_button = 3 then
				create l_dialog
				create l_filter.make (a_column, Current, l_dialog)
				if l_grid.filter_gui /= Void then
					l_grid.clear_filter_gui
				end
				l_grid.set_filter_gui (l_filter)
				create l_horizontal_box
				l_horizontal_box.set_background_color (colors.black)
				l_horizontal_box.set_border_width (2)
				l_dialog.extend (l_horizontal_box)
				l_horizontal_box.extend (l_filter)
				l_dialog.set_x_position (l_grid.grid.screen_x - l_grid.virtual_x_position + i_th (a_column).virtual_x_position)
				l_dialog.set_y_position (l_grid.grid.screen_y - l_grid.virtual_y_position + l_grid.grid.viewable_y_offset + i_th (a_column).virtual_y_position + i_th (a_column).height - 1)
				l_dialog.show_actions.extend (agent l_filter.set_focus)
				l_dialog.show
				l_filter.show_items
				l_filter.set_heights
				l_dialog.set_minimum_size (l_grid.column (a_column).width - i_th (a_column).horizontal_indent + 1, l_filter.height)
				l_filter.items_grid.grid.hide_horizontal_scroll_bar
				l_filter.items_grid.set_column_width (l_dialog.width, 1)
			end
		end

	item_added_to_secondary_title_nodes (a_node : RP_GRID_NODE [G]) is
			-- Fired when an item is added to `secondary_title_nodes'
		require
			a_node_not_void : a_node /= Void
--		local
--			error : STRING
		do
--			if a_node = Current then
--				error.append (" ")
--			end
			a_node.set_primary_title_node (Current)
			update_secondary_nodes (a_node, True)
		end

	item_removed_from_secondary_title_nodes (a_node : RP_GRID_NODE [G]) is
			-- Fired when an item is removed from `secondary_title_nodes'
		require
			a_node_not_void : a_node /= Void
		do
			a_node.set_primary_title_node (Void)
			update_secondary_nodes (a_node, False)
		end

	update_secondary_nodes (a_node : RP_GRID_NODE [G]; a_addition : BOOLEAN) is
			-- Synchronize secondary nodes in `grid' with `secondary_nodes'
		require
			a_node_not_void : a_node /= Void
		local
			l_parent_row : RP_GRID_NODE [G]
			l_index : INTEGER
		do
			if grid /= Void then
				if a_addition and a_node.grid = Void then
					l_index := secondary_title_nodes.index_of (a_node, 1)
					l_parent_row := parent_row
					if l_parent_row /= void then
						l_parent_row.insert_subrow (a_node, l_index + 1)
					else
						if secondary_nodes_above_regular then
							grid.insert_row (a_node, index)
							grid.move_row (index, grid.row_count + 1)
						else
							grid.insert_row (a_node, index + l_index)
						end
					end
				elseif a_node.grid /= Void then
					grid.remove_row (a_node)
				end
				grid.update_title_height
			end
		end

	secondary_nodes_above_regular: BOOLEAN
			-- If this is true, the secondary nodes will be displayed above the regular header, otherwise below

	update_regular_expression_field (a_text_field: EV_TEXT_FIELD) is
			-- Update `a_text_field' based on whether its contents are a valid regular expression.
		require
			a_text_field_not_void: a_text_field /= Void
		local
			l_re: LX_DFA_REGULAR_EXPRESSION
		do
			create l_re.make
			l_re.compile (a_text_field.text, True)
			if l_re.is_compiled then
				a_text_field.set_foreground_color (colors.black)
				a_text_field.remove_tooltip
			else
				a_text_field.set_foreground_color (colors.red)
				a_text_field.set_tooltip ("'" + a_text_field.text + "' is not a valid regular expression, and will be ignored")
			end
		end

	handle_key_press_in_filter_text (a_key: EV_KEY) is
			-- Respond to a key press in one of the filter text fields.
		require
			a_key_not_void: a_key /= Void
		do
			if a_key.code = {EV_KEY_CONSTANTS}.key_f then
				if gui.ev_application.ctrl_pressed then
					show_filter (False)
				end
			end
		end

	string_not_void (a_string: STRING): BOOLEAN is
			-- Is `a_string' not Void?
		do
			Result := a_string /= Void
		end

	bold_font: EV_FONT is
			--  Once acccess to a bold font.
		once
			create Result
			Result.set_weight (weight_bold)
		ensure
			result_not_void: Result /= Void
		end

invariant
	sort_state_agent_not_void: sort_state_agent /= Void
	sort_agent_not_void: sort_agent /= Void

end
