indexing
	description: "Objects representing a collection of people."
	author: "Julian Rogers"
	date: "$Date: 2009/05/11 $"
	revision: "$Revision: #6 $"
	archive: "$File: //rose/source/kernel/expert_system/rp_group.e $"

class
	RP_GROUP

inherit
	DEBUG_OUTPUT

create
	make_with_name

feature {NONE} -- Creation

	make_with_name (a_name: STRING) is
			-- Create `Current' and assign `a_name' to `name'.
		require
			a_name_not_void: a_name /= Void
			--a_name_not_empty: not a_name.is_empty
			-- Does not hold due to curent workflow usage to represent "Undefined" role.
		do
			name := a_name
			create cached_members.make_empty
		ensure
			name_set: name = a_name
		end

feature -- Access

	name: STRING
		-- Name of `Current'.

	members: ROSE_LINEAR_ARRAY [RP_DYNAMIC_PERSON] is
			-- All members of `Current'.
		do
			Result := cached_members
		end

	group_memberships: HASH_TABLE [RP_GROUP, STRING] is
			-- All groups of which `Current' is a member directly.
			-- Non-recursive. See `member_of_recursive' to find all
			-- groups.
		do

		ensure
			result_not_void: Result /= Void
		end

	group_memberships_recursive: HASH_TABLE [RP_GROUP, STRING] is
			-- All groups of which `Current' is a member.
			-- Recursive call to find all groups.
		do
			create Result.make (10)
		ensure
			result_not_void: Result /= Void
		end

	email_addresses: ROSE_LINEAR_ARRAY [STRING] is
			-- Email addresses of all members of `Current'.
		local
			i: INTEGER
			l_members: ROSE_LINEAR_ARRAY [RP_DYNAMIC_PERSON]
		do
			create Result.make_empty
			if members = Void or (members /= Void and then members.is_empty) then
				Result.extend (name)
			else
				l_members := members
				from
					i := l_members.lower
				until
					i > l_members.upper
				loop
					Result.extend (l_members.item (i).name)
					i := i + 1
				end
			end
		ensure
			result_not_void: Result /= Void
		end

feature {NONE} -- Implementation

	cached_members: ROSE_LINEAR_ARRAY [RP_DYNAMIC_PERSON]
			-- All members cached so they only need to be retrieved once.

	debug_output: STRING is
			-- String that should be displayed in debugger to represent `Current'.
		do
			Result := name
		end

invariant
	members_not_void: members /= Void
		-- fixme It seems that we have a way to store an empty name. Ensure this
		-- is no longer the case and then restore this invariant.
	--name_not_void_or_empty: name /= Void and then not name.is_empty
	name_not_void: name /= Void

end
