indexing

	description:

		"Interval history of honding intervals"

	author: "it"
	date: "$Date: 2009/11/06 $"
	revision: "$Revision: #12 $"
	archive: "$File: //rose/source/kernel/expert_system/aflows/rp_holding_history.e $"

class RP_HOLDING_HISTORY

create

	make_empty

feature {NONE} -- Initialization

	make_empty (a_date_agent: like date_agent) is
			-- Initialize `Current'
		do
			date_agent := a_date_agent
		end

feature -- Query

	witness_interval (a_date: ROSE_DATE): RP_HOLDING_INTERVAL is
			-- Stab query
		require
			non_void_date: a_date /= Void
		local
			l_cursor: like witness_interval
		do
			if item /= Void then
				from
					l_cursor := item
				until
					l_cursor = Void or else l_cursor.brackets (a_date)
				loop
					if a_date < l_cursor.start_date then
						l_cursor := l_cursor.previous
					else
						l_cursor := l_cursor.next
					end
				end
			end
			Result := l_cursor
		end

	all_intervals: ARRAY [like witness_interval] is
			-- All intervals
		local
			l_result: ROSE_LINEAR_ARRAY [like witness_interval]
			l_cursor: like witness_interval
		do
			create l_result.make_empty
			if head /= Void then
				from
					l_cursor := head
				until
					l_cursor = Void
				loop
					l_result.extend (l_cursor)
					l_cursor := l_cursor.next
				end
			end
			Result := l_result
		ensure
			non_void_interval_array: Result /= Void
		end

feature {RP_HOLDINGS} -- Element Change

	adjust_with (a_flow: RP_FLOW; a_is_debit: BOOLEAN) is
			-- Adjust holdings with `a_flow'.
		require
			not_void_args: a_flow /= Void
		do
			if head = Void then
					-- First node ever
				create item.make_new_double_linked_node (a_flow, date_agent.item ([a_flow]), a_is_debit, Void, Void)
				head := item
			else
				if item.start_date.same_date_time (date_agent.item ([a_flow])) then
						-- Same date as last flow, update only
					item.add_adjustment_flow (a_flow, a_is_debit)
				elseif date_agent.item ([a_flow]) < head.start_date then
						-- Insert at the beginning
					create item.make_new_double_linked_node (a_flow, date_agent.item ([a_flow]), a_is_debit, Void, head)
					head := item
				else
						-- Search for interval to insert after
						-- We are guaranteed a match as there is
						-- an interval with end_date plus infinity
					item := witness_interval (date_agent.item ([a_flow]))
					if item.start_date.same_date_time (date_agent.item ([a_flow])) then
							-- Same date as last flow, update only
						item.add_adjustment_flow (a_flow, a_is_debit)
					else
							-- Insert to the right of item
						create item.make_new_double_linked_node (a_flow, date_agent.item ([a_flow]), a_is_debit, item, item.next)
					end
				end
			end
		ensure
			item_not_void: item /= Void
		end

feature {RP_HOLDINGS} -- Implementation

	date_agent: FUNCTION [ANY, TUPLE [RP_FLOW], ROSE_DATE]

	head: like witness_interval
			-- Head of linked list.

	item: like witness_interval
			-- Most recent item in linked list.

end
