indexing

	description:

		"Double-linked Instrument Holding Interval"

	date: "$Date: 2009/04/23 $"
	revision: "$Revision: #12 $"

class RP_HOLDING_INTERVAL

inherit

	RP_HOLDING
		rename
			make as holding_make
		end

	COMPARABLE_INTERVAL [ROSE_DATE]
		rename
			lower as start_date,
			upper as end_date
		undefine
			is_equal
		end

create {RP_HOLDING_HISTORY}

	make_new_double_linked_node

feature {NONE} -- Initialization

	make_new_double_linked_node (a_flow: RP_FLOW; a_date: ROSE_DATE; a_is_debit: BOOLEAN; a_prev, a_next: like Current) is
			-- Initialize `Current' and adjust left and right neighbours appropriately.
		local
			l_quantity: RP_QUANTITY
			l_end_date: ROSE_DATE
			l_instrument_control: RP_INSTRUMENT_CONTROL
		do
			create internal_adjustment_flows.make_empty
			previous := a_prev
			next := a_next
			if previous /= Void then
				l_quantity := previous.quantity
				previous.set_end_date (a_date)
				previous.set_next (Current)
			else
				l_quantity := a_flow.quantity.instrument.quantity (0)
			end
			if next /= Void then
				l_end_date := next.start_date
				next.set_previous (Current)
			else
				l_end_date := end_of_the_universe
			end
			if a_is_debit then
				l_instrument_control := a_flow.source_control
			else
				l_instrument_control := a_flow.destination_control
			end
			holding_make (l_quantity, l_instrument_control)
			lower_bound := a_date
			set_end_date (l_end_date)
			add_adjustment_flow (a_flow, a_is_debit)
		ensure
			start_date_is_flow_date: start_date.same_date (a_date)
		end

feature -- Access

	previous: like Current
			-- Left neighbour.

	next: like Current
			-- Right neighbour.

	adjustments: ARRAY [RP_FLOW] is
			-- Adjustments to `Current'.
		do
			Result := internal_adjustment_flows
		end

	start_date: ROSE_DATE is
			-- Interval start date.
		do
			Result := lower_bound
		end

	end_date: ROSE_DATE is
			-- Interval end date.
		do
			Result := upper_bound
		end

feature {RP_HOLDING_HISTORY} -- Basic Operations

	add_adjustment_flow (a_flow: RP_FLOW; a_is_debit: BOOLEAN) is
			-- Add adjustment flow
		require
			flow_not_void: a_flow /= Void
		do
			internal_adjustment_flows.extend (a_flow)
			adjust (Current, a_flow.quantity, a_is_debit)
		end

feature {RP_HOLDING_INTERVAL} -- Implementation

	internal_adjustment_flows: ROSE_LINEAR_ARRAY [RP_FLOW]
			-- Adjustments

	set_previous (v: like previous) is
			-- Set left neighbour.
		do
			previous := v
		end

	set_next (v: like next) is
			-- Set right neighbour.
		do
			next := v
		end

	set_end_date (v: like upper_bound) is
			-- Set end date.
		do
			upper_bound := v
		end

	add_quantity (a_quantity: RP_QUANTITY; a_is_debit: BOOLEAN) is
			-- Adjust interval quantity.
		require
			non_void_qty: a_quantity /= Void
		do
			if a_is_debit then
				quantity := quantity - a_quantity
			else
				quantity := a_quantity + quantity
			end
		end

	adjust (a_interval: like Current; a_quantity: RP_QUANTITY; a_is_debit: BOOLEAN) is
			-- Adjust quantity of `Current' all right neighbours
		require
			non_void_args: a_interval /= Void and a_quantity /= Void
		local
			l_interval: like Current
		do
			from
				l_interval := a_interval
			until
				l_interval = Void
			loop
				l_interval.add_quantity (a_quantity, a_is_debit)
				l_interval := l_interval.next
			end
		end

	end_of_the_universe: ROSE_DATE is
			-- End of time.
		once
			create Result.make_from_ccyymmddhhmmss_frac_string ("999912312359599999")
		end

invariant

	quantity_not_void: quantity /= Void
	non_void_dates: start_date /= Void and end_date /= Void
	adjusments_not_void: adjustments /= Void
	adjustments_not_empty: not adjustments.is_empty

end
