indexing

	description:

		"Valuator for holdings"

	date: "$Date: 2010/04/30 $"
	revision: "$Revision: #2 $"

deferred class RP_HOLDING_VALUATOR

feature -- Output

	set_output (a_output: like output) is
			-- Enable debug output
		do
			output := a_output
		end

	disable_output is
			-- Disable debug output
		do
			output := Void
		end

	output: PROCEDURE [ANY, TUPLE [STRING, RP_PRICE, RP_QUANTITY, ROSE_DATE, STRING]]
			-- Debug agent

	output_no_price: PROCEDURE [ANY, TUPLE [STRING, RP_QUANTITY, ROSE_DATE, STRING]]
			-- Debug agent

	set_output_no_price (a_output_no_price: like output_no_price) is
			-- Enable debug output
		do
			output_no_price := a_output_no_price
		end

	disable_output_no_price is
			-- Disable debug output
		do
			output_no_price := Void
		end

feature {NONE} -- Output Implementation

	log_price (a_portfolio: STRING; a_price: RP_PRICE; a_position: RP_QUANTITY; a_date: ROSE_DATE; a_bucket: STRING) is
			-- Log computed `a_price' for instrument in `a_position' on `a_date'
		do
			if output /= Void then
				output.call ([a_portfolio, a_price, a_position, a_date, a_bucket])
			end
		end

	log_no_price (a_portfolio: STRING; a_position: RP_QUANTITY; a_date: ROSE_DATE; a_bucket: STRING) is
			-- Log missing price for instrument in `a_position' on `a_date'
		do
			if output_no_price /= Void then
				output_no_price.call ([a_portfolio, a_position, a_date, a_bucket])
			end
		end

feature -- Basic Operations

	value_in_currency (a_holding: RP_HOLDING; a_currency: RP_CURRENCY; a_date: ROSE_DATE; a_bucket: STRING): RP_MONIES is
			-- Value of `a_holding' in `a_current' on `a_date'.
		require
			non_void_args: a_holding /= Void and a_currency /= Void and a_date /= Void
		do
			Result := a_holding.quantity.instrument.holding_value_in_currency (a_holding, Current, a_currency, a_date, a_bucket)
		end

feature {RP_INSTRUMENT} -- Implementation

	government_bond_value_in_currency (a_holding: RP_HOLDING; a_currency: RP_CURRENCY; a_date: ROSE_DATE; a_bucket: STRING): RP_MONIES is
			-- Value of `a_holding' in `a_currency' on `a_date'.
		require
			non_void_args: a_holding /= Void and a_currency /= Void and a_date /= Void
		deferred
		ensure
			non_void_result: Result /= Void
			same_currency: Result.currency.is_equal (a_currency)
		end

	corporate_bond_value_in_currency (a_holding: RP_HOLDING; a_currency: RP_CURRENCY; a_date: ROSE_DATE; a_bucket: STRING): RP_MONIES is
			-- Value of `a_holding' in `a_currency' on `a_date'.
		require
			non_void_args: a_holding /= Void and a_currency /= Void and a_date /= Void
		deferred
		ensure
			non_void_result: Result /= Void
			same_currency: Result.currency.is_equal (a_currency)
		end

	index_future_value_in_currency (a_holding: RP_HOLDING; a_currency: RP_CURRENCY; a_date: ROSE_DATE; a_bucket: STRING): RP_MONIES is
			-- Value of `a_holding' in `a_currency' on `a_date'.
		require
			non_void_args: a_holding /= Void and a_currency /= Void and a_date /= Void
		deferred
		ensure
			non_void_result: Result /= Void
			same_currency: Result.currency.is_equal (a_currency)
		end

	currency_future_value_in_currency (a_holding: RP_HOLDING; a_currency: RP_CURRENCY; a_date: ROSE_DATE; a_bucket: STRING): RP_MONIES is
			-- Value of `a_holding' in `a_currency' on `a_date'.
		require
			non_void_args: a_holding /= Void and a_currency /= Void and a_date /= Void
		deferred
		ensure
			non_void_result: Result /= Void
			same_currency: Result.currency.is_equal (a_currency)
		end

	forward_value_in_currency (a_holding: RP_HOLDING; a_currency: RP_CURRENCY; a_date: ROSE_DATE; a_bucket: STRING): RP_MONIES is
			-- Value of `a_holding' in `a_currency' on `a_date'.
		require
			non_void_args: a_holding /= Void and a_currency /= Void and a_date /= Void
		deferred
		ensure
			non_void_result: Result /= Void
			same_currency: Result.currency.is_equal (a_currency)
		end

	stock_value_in_currency (a_holding: RP_HOLDING; a_currency: RP_CURRENCY; a_date: ROSE_DATE; a_bucket: STRING): RP_MONIES is
			-- Value of `a_holding' in `a_currency' on `a_date'.
		require
			non_void_args: a_holding /= Void and a_currency /= Void and a_date /= Void
		deferred
		ensure
			non_void_result: Result /= Void
			same_currency: Result.currency.is_equal (a_currency)
		end

	currency_value_in_currency (a_holding: RP_HOLDING; a_currency: RP_CURRENCY; a_date: ROSE_DATE; a_bucket: STRING): RP_MONIES is
			-- Value of `a_holding' in `a_currency' on `a_date'.
		require
			non_void_args: a_holding /= Void and a_currency /= Void and a_date /= Void
		deferred
		ensure
			non_void_result: Result /= Void
			same_currency: Result.currency.is_equal (a_currency)
		end

	pool_fund_value_in_currency (a_holding: RP_HOLDING; a_currency: RP_CURRENCY; a_date: ROSE_DATE; a_bucket: STRING): RP_MONIES is
			-- Value of `a_holding' in `a_currency' on `a_date'.
		require
			non_void_args: a_holding /= Void and a_currency /= Void and a_date /= Void
		deferred
		ensure
			non_void_result: Result /= Void
			same_currency: Result.currency.is_equal (a_currency)
		end

	fixed_deposit_value_in_currency (a_holding: RP_HOLDING; a_currency: RP_CURRENCY; a_date: ROSE_DATE; a_bucket: STRING): RP_MONIES is
			-- Value of `a_holding' in `a_currency' on `a_date'.
		require
			non_void_args: a_holding /= Void and a_currency /= Void and a_date /= Void
		deferred
		ensure
			non_void_result: Result /= Void
			same_currency: Result.currency.is_equal (a_currency)
		end

end
