indexing

	description:

		"Default holdings valuator"

	date: "$Date: 2010/04/30 $"
	sccid: "$Id: //rose/source/kernel/expert_system/valuators/rp_holding_valuator_axar_default.e#6 $"

class RP_HOLDING_VALUATOR_AXAR_DEFAULT

inherit

	RP_HOLDING_VALUATOR

create {RP_HOLDING_VALUATOR_FACTORY}

	make

feature {NONE} -- Initialization

	make (a_equity_pricing_algorithm: like equity_pricing_algorithm; a_poolfund_pricing_algorithm: like poolfund_pricing_algorithm; a_currency_marked_data_source: like currency_marked_data_source; a_index_future_pricing_algorithm: like index_future_pricing_algorithm; a_corporate_bond_pricing_algorithm: like corporate_bond_pricing_algorithm; a_corporate_bond_interest_algorithm: like corporate_bond_interest_algorithm; a_government_bond_pricing_algorithm: like government_bond_pricing_algorithm) is
			-- Initialize `Current'.
		require
			non_void_arguments: a_equity_pricing_algorithm /= Void and a_poolfund_pricing_algorithm /= Void and a_currency_marked_data_source /= Void and a_index_future_pricing_algorithm /= Void
			corporate_bond_pricing_algorithm_not_void: a_corporate_bond_pricing_algorithm /= Void
			corporate_bond_interest_algorithm_not_void: a_corporate_bond_interest_algorithm /= Void
			government_bond_pricing_algorithm_not_void: a_government_bond_pricing_algorithm /= Void
		do
			equity_pricing_algorithm := a_equity_pricing_algorithm
			poolfund_pricing_algorithm := a_poolfund_pricing_algorithm
			currency_marked_data_source := a_currency_marked_data_source
			index_future_pricing_algorithm := a_index_future_pricing_algorithm
			corporate_bond_pricing_algorithm := a_corporate_bond_pricing_algorithm
			corporate_bond_interest_algorithm := a_corporate_bond_interest_algorithm
			government_bond_pricing_algorithm := a_government_bond_pricing_algorithm
		ensure
			equity_pricing_algorithm_set: equity_pricing_algorithm = a_equity_pricing_algorithm
			poolfund_pricing_algorithm_set: poolfund_pricing_algorithm = a_poolfund_pricing_algorithm
			currency_marked_data_source_set: currency_marked_data_source = a_currency_marked_data_source
			index_future_pricing_algorithm_set: index_future_pricing_algorithm = a_index_future_pricing_algorithm
			corporate_bond_pricing_algorithm_set: corporate_bond_pricing_algorithm = a_corporate_bond_pricing_algorithm
			corporate_bond_interest_algorithm_set: corporate_bond_interest_algorithm = a_corporate_bond_interest_algorithm
			government_bond_pricing_algorithm_set: government_bond_pricing_algorithm = a_government_bond_pricing_algorithm
		end

feature {NONE} -- Implementation

	equity_pricing_algorithm: RP_PRICING_ALGORITHM [RP_STOCK]
			-- Stock pricing algorithm

	government_bond_pricing_algorithm: RP_PRICING_ALGORITHM [RP_GOVERNMENT_BOND]
			-- Government bond pricing algorithm

	poolfund_pricing_algorithm: RP_PRICING_ALGORITHM [RP_POOL_FUND]
			-- Poolfund pricing algorithm

	currency_marked_data_source: RP_CURRENCY_MARKET_DATA_SOURCE
			-- FX source

	index_future_pricing_algorithm: RP_PRICING_ALGORITHM [RP_INDEX_FUTURE]
			-- Index future pricing algorithm

	corporate_bond_pricing_algorithm: RP_PRICING_ALGORITHM [RP_CORPORATE_BOND]
			-- Corporate bond pricing algorithm

	corporate_bond_interest_algorithm: RP_PRICING_ALGORITHM [RP_CORPORATE_BOND]
			-- Corporate bond interest algorithm

	portfolio (a_holding: RP_HOLDING): STRING is
			-- Portfolio
		require
			not_void: a_holding /= Void
		do
			Result := a_holding.control.management.account.code
		ensure
			account_code: Result = a_holding.control.management.account.code
		end

	calendar: RP_CALENDAR
			-- Market day calendar

	lag_price_cache: DS_HASH_TABLE [DS_HASH_TABLE [RP_PRICE, INTEGER], INTEGER]
			-- Lag price cache

	stock_value_in_currency (a_holding: RP_HOLDING; a_currency: RP_CURRENCY; a_date: ROSE_DATE; a_bucket: STRING): RP_MONIES is
			-- Value of `a_holding' in `a_currency' on `a_date'.
		local
			l_price: RP_PRICE
			l_value, l_fx_rate: RP_DECIMAL
			l_stock: RP_STOCK
		do
			Result := a_currency.monies (0)
			l_stock ?= a_holding.quantity.instrument
			if l_stock /= Void then
				l_value := a_holding.quantity.amount
				if not l_value.is_zero then
					l_price := equity_pricing_algorithm.price (l_stock, a_date)
					if l_price /= Void then
						l_fx_rate := fx_rate (l_price.currency, a_currency, a_date)
						l_value := l_value * l_price.price * l_fx_rate
						Result := a_currency.monies (l_value)
						log_price (portfolio (a_holding), l_price, a_holding.quantity, a_date, a_bucket)
					else
						log_no_price (portfolio (a_holding), a_holding.quantity, a_date, a_bucket)
					end
				end
			end
		end

	pool_fund_value_in_currency (a_holding: RP_HOLDING; a_currency: RP_CURRENCY; a_date: ROSE_DATE; a_bucket: STRING): RP_MONIES is
			-- Value of `a_holding' in `a_currency' on `a_date'.
		local
			l_price: RP_PRICE
			l_value, l_fx_rate: RP_DECIMAL
			l_pool_fund: RP_POOL_FUND
		do
			Result := a_currency.monies (0)
			l_pool_fund ?= a_holding.quantity.instrument
			if l_pool_fund /= Void then
				l_value := a_holding.quantity.amount
				if not l_value.is_zero then
					l_price := poolfund_pricing_algorithm.price (l_pool_fund, a_date)
					if l_price /= Void then
						l_fx_rate := fx_rate (l_price.currency, a_currency, a_date)
						l_value := l_value * l_price.price * l_fx_rate
						Result := a_currency.monies (l_value)
						log_price (portfolio (a_holding), l_price, a_holding.quantity, a_date, a_bucket)
					else
						log_no_price (portfolio (a_holding), a_holding.quantity, a_date, a_bucket)
					end
				end
			end
		end

	currency_value_in_currency (a_holding: RP_HOLDING; a_currency: RP_CURRENCY; a_date: ROSE_DATE; a_bucket: STRING): RP_MONIES is
			-- Value of `a_holding' in `a_currency' on `a_date'.
		local
			l_price: RP_PRICE
			l_fx_rate, l_value: RP_DECIMAL
			l_currency: RP_CURRENCY
		do
			Result := a_currency.monies (0)
			l_currency ?= a_holding.quantity.instrument
			if l_currency /= Void then
				l_value := a_holding.quantity.amount
				if not l_value.is_zero then
					l_fx_rate := fx_rate (l_currency, a_currency, a_date)
					l_price := l_currency.price_in_currency (l_fx_rate, a_currency, a_date)
					l_value := l_value * l_price.price
					Result := a_currency.monies (l_value)
					log_price (portfolio (a_holding), l_price, a_holding.quantity, a_date, a_bucket)
				end
			end
		end

	forward_value_in_currency (a_holding: RP_HOLDING; a_currency: RP_CURRENCY; a_date: ROSE_DATE; a_bucket: STRING): RP_MONIES is
			-- value fx forward
		local
			l_price: RP_PRICE
			l_value, l_fx_rate: RP_DECIMAL
			l_short_term_rate: RP_DECIMAL
			l_currency_forward: RP_CURRENCY_FORWARD
		do
			Result := a_currency.monies (0)
			l_currency_forward ?= a_holding.quantity.instrument
			if l_currency_forward /= Void then
				l_value := a_holding.quantity.amount
				if not l_value.is_zero and l_currency_forward.settlement_date.ccyymmdd >= a_date.ccyymmdd then
					l_short_term_rate := currency_marked_data_source.short_term_rate (l_currency_forward.currency, a_date, l_currency_forward.settlement_date)
					l_price := l_currency_forward.price_in_currency (create {RP_DECIMAL}.make_from_integer (1) / l_short_term_rate, l_currency_forward.currency, a_date)
					l_fx_rate := fx_rate (l_price.currency, a_currency, a_date)
					l_value := l_value * l_price.price * l_fx_rate
					Result := a_currency.monies (l_value)
					log_price (portfolio (a_holding), l_price, a_holding.quantity, a_date, a_bucket)
				end
			end
		end

	fixed_deposit_value_in_currency (a_holding: RP_HOLDING; a_currency: RP_CURRENCY; a_date: ROSE_DATE; a_bucket: STRING): RP_MONIES is
			-- Value of `a_holding' in `a_currency' on `a_date'.
		local
			l_fixed_deposit: RP_FIXED_DEPOSIT
			l_price: RP_PRICE
			l_value: RP_DECIMAL
			l_fx_rate: RP_DECIMAL
		do
			Result := a_currency.monies (0)
			l_fixed_deposit ?= a_holding.quantity.instrument
			if l_fixed_deposit /= Void then
				l_value := a_holding.quantity.amount
				if not l_value.is_zero then
					l_price := l_fixed_deposit.price (a_date)
					l_fx_rate := fx_rate (l_price.currency, a_currency, a_date)
					l_value := l_value * l_price.price * l_fx_rate
					Result := a_currency.monies (l_value)
					log_price (portfolio (a_holding), l_price, a_holding.quantity, a_date, a_bucket)
				end
			end
		end

	government_bond_value_in_currency (a_holding: RP_HOLDING; a_currency: RP_CURRENCY; a_date: ROSE_DATE; a_bucket: STRING): RP_MONIES is
			-- Value of `a_holding' in `a_currency' on `a_date'.
		local
			l_tbill: RP_GOVERNMENT_BOND
			l_price: RP_PRICE
			l_value: RP_DECIMAL
			l_fx_rate: RP_DECIMAL
		do
			Result := a_currency.monies (0)
			l_tbill ?= a_holding.quantity.instrument
			if l_tbill /= Void then
				l_value := a_holding.quantity.amount
				if not l_value.is_zero then
					l_price := government_bond_pricing_algorithm.price (l_tbill, a_date)
					if l_price /= Void then
						l_fx_rate := fx_rate (l_price.currency, a_currency, a_date)
						l_value := l_value * l_price.price * l_fx_rate
						Result := a_currency.monies (l_value)
						log_price (portfolio (a_holding), l_price, a_holding.quantity, a_date, a_bucket)
					else
						log_no_price (portfolio (a_holding), a_holding.quantity, a_date, a_bucket)
					end
				end
			end
		end

	corporate_bond_value_in_currency (a_holding: RP_HOLDING; a_currency: RP_CURRENCY; a_date: ROSE_DATE; a_bucket: STRING): RP_MONIES is
			-- Value of `a_holding' in `a_currency' on `a_date'.
		local
			l_bond: RP_CORPORATE_BOND
			l_fx_rate, l_value, l_accrued_interest: RP_DECIMAL
			l_price, l_interest: RP_PRICE
		do
			Result := a_currency.monies (0)
			l_bond ?= a_holding.quantity.instrument
			if l_bond /= Void then
				l_value := a_holding.quantity.amount
				if not l_value.is_zero then
					l_price := corporate_bond_pricing_algorithm.price (l_bond, a_date)
					if l_price /= Void then
						log_price (portfolio (a_holding), l_price, a_holding.quantity, a_date, a_bucket)
						l_fx_rate := fx_rate (l_price.currency, a_currency, a_date)
						l_value := l_value * l_price.price * l_fx_rate
					else
						l_value := 0
						log_no_price (portfolio (a_holding), a_holding.quantity, a_date, a_bucket)
					end
					l_interest := corporate_bond_interest_algorithm.price (l_bond, a_date)
					if l_interest /= Void then
						log_price (portfolio (a_holding), l_interest, a_holding.quantity, a_date, a_bucket)
						l_fx_rate := fx_rate (l_interest.currency, a_currency, a_date)
						l_accrued_interest := a_holding.quantity.amount * l_interest.price * l_fx_rate
					else
						l_accrued_interest := 0
					end
					Result := a_currency.monies (l_value + l_accrued_interest)
				end
			end
		end

	index_future_value_in_currency (a_holding: RP_HOLDING; a_currency: RP_CURRENCY; a_date: ROSE_DATE; a_bucket: STRING): RP_MONIES is
			-- Value of `a_holding' in `a_currency' on `a_date'.
		do
				-- Index futures are not valued in the default valuator
			Result := a_currency.monies (0)
			log_no_price (portfolio (a_holding), a_holding.quantity, a_date, a_bucket)
		end

	currency_future_value_in_currency (a_holding: RP_HOLDING; a_currency: RP_CURRENCY; a_date: ROSE_DATE; a_bucket: STRING): RP_MONIES is
			-- Value of `a_holding' in `a_currency' on `a_date'.
		do
				-- Currency futures are not valued in the default valuator
			Result := a_currency.monies (0)
			log_no_price (portfolio (a_holding), a_holding.quantity, a_date, a_bucket)
		end

	fx_rate (a_source_currency: RP_CURRENCY; a_target_currency: RP_CURRENCY; a_date: ROSE_DATE): RP_DECIMAL is
			-- FX rate
		require
			non_void_args: a_source_currency /= Void and a_target_currency /= Void and a_date /= Void
		do
			if a_target_currency.is_equal (a_source_currency) then
				Result := 1
			else
				Result := currency_marked_data_source.spot_rate (a_source_currency, a_target_currency, a_date)
			end
		end

end
