indexing
	description: "Holding valuator for simulation"
	author: "XR"
	last_editor: "$Author: er $"
	date: "$Date: 2014/02/26 $"
	revision: "$Revision: #2 $"
	archive: "$File: //rose/source/kernel/expert_system/valuators/rp_holding_valuator_simulation.e $"

class
	RP_HOLDING_VALUATOR_SIMULATION

inherit
	RP_HOLDING_VALUATOR_AXAR_DEFAULT
		redefine
			stock_value_in_currency
		end

	RP_REFERENCE_SYSTEM
		rename
			make as make_ref
		end

	create
		make

feature -- Implementation

	stock_value_in_currency (a_holding: RP_HOLDING; a_currency: RP_CURRENCY; a_date: ROSE_DATE; a_bucket: STRING): RP_MONIES is
			-- Value of `a_holding' in `a_currency' on `a_date'.
		local
			l_price: RP_PRICE
			l_value, l_fx_rate: RP_DECIMAL
			l_stock: RP_STOCK
		do
			Result := a_currency.monies (0)
			l_stock ?= a_holding.quantity.instrument
			if l_stock /= Void then
				l_value := a_holding.quantity.amount
				if not l_value.is_zero then
					l_price := stock_price (l_stock, a_date)
					if l_price /= Void then
						l_fx_rate := fx_rate (l_price.currency, a_currency, a_date)
						l_value := l_value * l_price.price * l_fx_rate
						Result := a_currency.monies (l_value)
						log_price (portfolio (a_holding), l_price, a_holding.quantity, a_date, a_bucket)
					else
						log_no_price (portfolio (a_holding), a_holding.quantity, a_date, a_bucket)
					end
				end
			end
		end

	reference_system: RP_REFERENCE_SYSTEM
	zone: RP_ZONE

	set_zone (a_zone: RP_ZONE) is
			-- set zone
		do
			zone := a_zone
		end

	set_reference_system (a_system: RP_REFERENCE_SYSTEM) is
			-- set zone
		do
			reference_system := a_system
		end

	set_security_prices (a_prices:  ROSE_TIME_SERIES [ROSE_HASH_TABLE [DOUBLE, STRING]]) is
			-- set portfolio system
		do
			security_prices := a_prices
		end

	stock_price (a_stock: RP_STOCK; a_date: ROSE_DATE): RP_PRICE is
			-- Price of `a_stock' on `a_date'
		local
			l_brimsym, l_brimsec: STRING
			l_price: RP_DECIMAL
			l_inst: RP_INSTRUMENT
			l_month_prices: ROSE_HASH_TABLE[DOUBLE,STRING]
		do
			l_brimsym := a_stock.brimsym
			l_brimsec := l_brimsym.substring (1, 15)
			l_inst := reference_system.instrument_from_brimsym (l_brimsym)
			if l_inst /= Void then
				l_month_prices := end_of_month_prices (a_date)
				if l_month_prices.has (l_brimsec) then
					l_price := l_month_prices.item (l_brimsec)
					Result := l_inst.price_in_currency (l_price, base_currency, a_date)
				end
			end
		end

	base_currency: RP_CURRENCY is
			-- base currency
		local
			l_curr: CURRENCY
		once
			l_curr := context.zone (zone.code.item (1)).base_currency
			Result := reference_system.currency_from_iso_code (l_curr.iso)
		end

	security_prices: ROSE_TIME_SERIES [ROSE_HASH_TABLE[DOUBLE,STRING]]

	end_of_month_prices (a_date: ROSE_DATE): ROSE_HASH_TABLE[DOUBLE,STRING]
			-- retrieves the closest end of month prices to a_date
		local
			l_date: ROSE_DATE
		do
			if a_date.day > 15 then
				l_date := a_date.month_end
			else
				l_date := a_date.previous_month_end
			end
			Result := security_prices.item_at (l_date)
		end
end
