indexing

	description:

		"Aggregated positions per Instrument"

	date: "$Date: 2009/04/23 $"
	revision: "$Revision: #17 $"

class RP_HOLDINGS

create {RP_ACCOUNTING_SYSTEM}

	make

feature {NONE} -- Initialization

	make (a_date_agent: like date_agent) is
			-- Initialize `Current'.
		do
			date_agent := a_date_agent
			create hash.make_equal (17)
		end

feature -- Basic Operations

	add_account_flow (a_flow: RP_FLOW; a_is_debit: BOOLEAN) is
			-- aggregate flows
		require
			non_void_args: a_flow /= Void
		local
			l_instrument: RP_INSTRUMENT
			l_holding_history: RP_HOLDING_HISTORY
		do
			l_instrument := a_flow.quantity.instrument
			if hash.has (l_instrument) then
				l_holding_history := hash.item (l_instrument)
			else
				create l_holding_history.make_empty (date_agent)
				hash.force (l_holding_history, l_instrument)
			end
			l_holding_history.adjust_with (a_flow, a_is_debit)
		end

feature -- Query

	holdings_at (a_date: ROSE_DATE): ROSE_LINEAR_ARRAY [RP_HOLDING_INTERVAL] is
			-- Holdings on `a_date'.
		require
			non_void_date: a_date /= Void
		do
			create Result.make_empty
			hash.do_all (agent add_holding (Result, ?, a_date))
		ensure
			non_void_result: Result /= Void
		end

	add_holding (a_array: ROSE_LINEAR_ARRAY [RP_HOLDING_INTERVAL]; a_holding: TUPLE [history: RP_HOLDING_HISTORY; instrument: RP_INSTRUMENT]; a_date: ROSE_DATE) is
			-- add `a_holding' to `a_array'
		local
			l_holding: RP_HOLDING_INTERVAL
		do
			l_holding := a_holding.history.witness_interval (a_date)
			if l_holding /= Void then
				a_array.extend (l_holding)
			end
		end

	instruments: ROSE_LINEAR_ARRAY [RP_INSTRUMENT] is
			-- All holdings instruments.
		do
			Result := hash.keys
		ensure
			non_void_result: Result /= Void
		end

	holdings_for (a_instrument: RP_INSTRUMENT): RP_HOLDING_HISTORY is
			-- Holding intervals for `a_instrument'.
		require
			non_voidinstrument: a_instrument /= Void
		do
			if hash.has (a_instrument) then
				Result := hash.item (a_instrument)
			end
		ensure
			may_return_void: True
		end

feature {NONE} -- Implementation

	date_agent: FUNCTION [ANY, TUPLE [RP_FLOW], ROSE_DATE]
			-- Date agent

	hash: UID_TABLE [RP_HOLDING_HISTORY, RP_INSTRUMENT]
			-- Holding intervals storage.

end -- class RP_HOLDINGS
