indexing
	description: "Right overrides (including or excluding) coming from EVENT_<Z>_A:EXCLUDING_RIGHTS_<Z>.PARM or EVENT_<Z>_A:INCLUDING_RIGHTS_<Z>.PARM"
	author: ""
	last_editor: "$Author: sm $"
	date: "$Date: 2009/10/06 $"
	revision: "$Revision: #1 $"
	archive: "$File: //rose/source/kernel/capital_actions/elections/rp_including_excluding_rights.e $"

class
	RP_INCLUDING_EXCLUDING_RIGHTS

inherit
	EXCEPTIONS
		export {NONE} all end

	ROSE_SHARED_GLOBAL
		export {NONE} all end

	RP_EVENT_TYPE_CONSTANTS
		export {NONE} all end

	ANY

create
	make_excluding_from_windows_share, 	make_including_from_windows_share


feature {NONE} -- Initialization

	make_excluding_from_windows_share (a_zone: like zone) is
			-- Create for 'a_zone'
		do
			zone := a_zone
			is_excluding := true
			parse_from_windows_share (windows_excluding_filename, zone)
		ensure
			zone_set: zone.is_equal (a_zone)
			is_excluding: is_excluding
		end

	make_including_from_windows_share (a_zone: like zone) is
			-- Create for 'a_zone'
		do
			zone := a_zone
			is_including := true
			parse_from_windows_share (windows_including_filename, zone)
		ensure
			zone_set: zone.is_equal (a_zone)
			is_including: is_including
		end

feature -- Access

	zone: RP_ZONE
		-- Zone

	windows_including_filename: STRING is
		-- Filename for this zone
		do
			Result := windows_path+"\including_rights_"+zone.code.as_lower+".parm"
		end

	windows_excluding_filename: STRING is
		-- Filename for this zone
		do
			Result := windows_path+"\excluding_rights_"+zone.code.as_lower+".parm"
		end

	windows_path: !STRING is
			-- Windows file path
		do
			Result := global.hardcoded_path ("\\axaros.com\shares\ProdData_NT\PortfolioAccounting\ParmFiles")
		end

	item (a_brimsym: STRING; a_ex_date: ROSE_DATE; a_clicode: STRING): RP_INCLUDING_EXCLUDING_RIGHTS_PARM is
			-- Override for instrument, ex-date and portfolio
			-- We expect the client code to be in upper case
		require
			brimsym_not_void: a_brimsym /= Void
			valid_ex_date: a_ex_date /= Void
			valid_clicode: a_clicode /= Void and then a_clicode.is_equal (a_clicode.as_upper)
		local
			l_key: STRING
		do
			l_key := key (a_brimsym, a_ex_date, a_clicode)
			entries.search (l_key)
			if entries.found then
				Result := entries.found_item

			else -- Search on the wildcard 'ALL' account which represents all possible clicodes (portfolio names)
				l_key := key (a_brimsym, a_ex_date, a_clicode)
				entries.search (l_key)
				if entries.found then
					Result := entries.found_item
				end
			end
		ensure
			valid_result: (Result /= Void) implies (Result.brimsym.is_equal (a_brimsym) and Result.ex_date.is_equal (a_ex_date) and  ( Result.clicode.is_equal (a_clicode) or Result.clicode.is_equal ("ALL")))
		end

feature -- Status Report

	exists: BOOLEAN
		 -- Do we have settings for this zone?

	is_including: BOOLEAN
		-- Is it an inclusion file?

	is_excluding: BOOLEAN
		-- Is it an exclusion file?

feature {NONE} -- Implementation

	parse_from_windows_share (a_filename: STRING; a_zone: RP_ZONE) is
			-- Parse 'a_file_name' and populate internal table
		local
			l_file: INCLUDING_EXCLUDING_RIGHTS_PARM_FILE
			l_array: ROSE_LINEAR_ARRAY [INCLUDING_EXCLUDING_RIGHTS_PARM_RECORD]
			l_parm: RP_INCLUDING_EXCLUDING_RIGHTS_PARM
			i: INTEGER
			l_key: STRING
		do
			create entries.make_equal (17)
			output ("Reading "+ a_filename+"...", <<>>) -- Does substitute with elogger
			create l_file.make (a_filename)
			exists := l_file.exists
			if exists then
				l_file.parse
				l_array := l_file.array
				if l_array /= Void then
					from
						i := l_array.lower
					until
						i > l_array.upper
					loop
						create l_parm.make (l_array [i])
						l_key := key (l_parm.brimsym, l_parm.ex_date, l_parm.clicode)
						if not entries.has (l_key) then
							entries.force (l_parm, l_key)
							check available: item (l_parm.brimsym, l_parm.ex_date, l_parm.clicode) = l_parm end
						else
							output ("Warning! Duplicate entry is ignored: "+l_parm.debug_output+"%N", <<>>)
						end
						i := i + 1
					end
				end
			end
			output ("done ["+entries.count.out+" entrie(s) in cache for "+zone.code+"]%N", <<>>)
		end

	key (a_brimsym: STRING; a_ex_date: ROSE_DATE; a_clicode: STRING): STRING is
			-- Key for 'entries'
		require
			brimsym_not_void: a_brimsym /= Void
			valid_ex_date: a_ex_date /= Void
			valid_clicode: a_clicode /= Void
		do
			Result := a_brimsym + "|" + "|" + a_ex_date.ccyymmdd.out+ "|" + a_clicode
		ensure
			not_void: Result /= Void
		end

	entries: DS_HASH_TABLE [RP_INCLUDING_EXCLUDING_RIGHTS_PARM, STRING]
		-- All entries

invariant

	including_or_excluding: is_including or else is_excluding

end
