indexing
	description: "Shared accessors for parm files specifing inclusion or exclusions of rights"
	author: "Portfolio Accounting Tech (sm)"
	last_editor: "$Author: sm $"
	date: "$Date: 2009/10/06 $"
	revision: "$Revision: #1 $"
	archive: "$File: //rose/source/kernel/capital_actions/elections/rp_including_excluding_rights_aware.e $"

class
	RP_INCLUDING_EXCLUDING_RIGHTS_AWARE


feature -- Access

	including_rights (a_zone: RP_ZONE; a_brimsym: STRING; a_ex_date: ROSE_DATE; a_clicode: STRING): RP_INCLUDING_EXCLUDING_RIGHTS_PARM is
			-- Override to include rights for instrument, ex-date and portfolio
			-- We expect the client code to be in upper case
		require
			valid_zone: a_zone /= Void
			brimsym_not_void: a_brimsym /= Void
			valid_ex_date: a_ex_date /= Void
			valid_clicode: a_clicode /= Void and a_clicode.as_upper.is_equal (a_clicode)
		local
			l_including: RP_INCLUDING_EXCLUDING_RIGHTS
		do
				-- Try to read the file
			l_including := including_rights_for_zone (a_zone)
			if l_including.exists then
					-- If there is a file, then we must have an entry in the file...
				Result := l_including.item (a_brimsym, a_ex_date, a_clicode)
					-- ... or the file must have an entry for ALL portfolios
				if Result = Void then
					Result := l_including.item (a_brimsym, a_ex_date, Including_all_portfolios_wildcard)
				end
			else
				-- If there is no file, it means we exercise all the rights
				create Result.make (create {INCLUDING_EXCLUDING_RIGHTS_PARM_RECORD}.make(a_brimsym, a_ex_date.ccyymmdd, a_clicode, create {STRING}.make_empty, 0, a_ex_date.ccyymmdd))
			end

		ensure
			correct_result: (Result /= Void) implies (Result.brimsym.is_equal (a_brimsym) and Result.ex_date.is_equal (a_ex_date) and Result.clicode.is_equal (a_clicode))
		end

	excluding_rights (a_zone: RP_ZONE; a_brimsym: STRING; a_ex_date: ROSE_DATE; a_clicode: STRING): RP_INCLUDING_EXCLUDING_RIGHTS_PARM is
			-- Override to exclude rights for instrument, ex-date and portfolio
			-- We expect the client code to be in upper case
		require
			valid_zone: a_zone /= Void
			brimsym_not_void: a_brimsym /= Void
			valid_ex_date: a_ex_date /= Void
			valid_clicode: a_clicode /= Void and a_clicode.as_upper.is_equal (a_clicode)
		local
			l_excluding: RP_INCLUDING_EXCLUDING_RIGHTS
		do
				-- Try to read the file
			l_excluding := excluding_rights_for_zone (a_zone)
			if l_excluding.exists then
					-- If there is a file, then we must have an entry in the file...
				Result := l_excluding.item (a_brimsym, a_ex_date, a_clicode)
					-- ... or the file must have an entry for ALL portfolios
				if Result = Void then
					Result := l_excluding.item (a_brimsym, a_ex_date, Excluding_all_portfolios_wildcard)
				end
				-- Note that as opposed to the including logic, it does not exclude by default
			end
		ensure
			correct_result: (Result /= Void) implies (Result.brimsym.is_equal (a_brimsym) and Result.ex_date.is_equal (a_ex_date) and Result.clicode.is_equal (a_clicode))
		end

feature -- Constants

	Including_all_portfolios_wildcard: STRING is "ALL"
		-- Pattern found in the inclusion file to mark an inclusion for all portfolios in the exercise of a right

	Excluding_all_portfolios_wildcard: STRING is "ALLPORT"
		-- Pattern found in the exclusion file to mark an exclusion for all portfolios in the exercise of a right

feature {NONE} -- Implementation

	including_rights_for_zone (a_zone: RP_ZONE): RP_INCLUDING_EXCLUDING_RIGHTS is
			-- Corporate actions overrides for 'a_zone'
		do
			if including_rights_cache.has (a_zone) then
				Result := including_rights_cache.item (a_zone)
			else
				create Result. make_including_from_windows_share (a_zone)
				including_rights_cache.put (Result, a_zone)
			end
		ensure
			result_not_void: Result /= Void
			correct_zone: Result.zone.is_equal (a_zone)
		end

	excluding_rights_for_zone (a_zone: RP_ZONE): RP_INCLUDING_EXCLUDING_RIGHTS is
			-- Corporate actions overrides for 'a_zone'
		do
			if excluding_rights_cache.has (a_zone) then
				Result := excluding_rights_cache.item (a_zone)
			else
				create Result. make_excluding_from_windows_share (a_zone)
				excluding_rights_cache.put (Result, a_zone)
			end
		ensure
			result_not_void: Result /= Void
			correct_zone: Result.zone.is_equal (a_zone)
		end

	including_rights_cache: HASH_TABLE[RP_INCLUDING_EXCLUDING_RIGHTS, RP_ZONE]
			-- Shared zonal CA overrides
		once
			create Result.make (0)
		ensure
			result_not_void: Result /= Void
		end

	excluding_rights_cache: HASH_TABLE[RP_INCLUDING_EXCLUDING_RIGHTS, RP_ZONE]
			-- Shared zonal CA overrides
		once
			create Result.make (0)
		ensure
			result_not_void: Result /= Void
		end

end
