note

	description:

		"typically a sale of partial shares as a result of CA"

	author: "Mark Howard"
	last_editor: "$Author: e2 $"
	date: "$Date: 2011/11/29 $"
	revision: "$Revision: #11 $"
	path: "$File: //rose/source/kernel/expert_system/rp_inlieu.e $"

deferred class RP_INLIEU

inherit

	RP_CUSTODIAN_CONTRACT

feature {NONE} -- Implementation

	make (a_generating_system: like generating_system; a_event_date, a_settle_date: ROSE_DATE;
		a_identifier: like identifier; a_portfolio: like portfolio; a_broker: RP_BROKER;
		a_quantity: RP_QUANTITY; a_gross_monies, a_net_monies, a_commission, a_fees: RP_MONIES;
		a_custody: RP_CUSTODY) is
			-- Initialize `Current'
		require
			a_generating_system_not_void: a_generating_system /= Void
			a_event_date_not_void: a_event_date /= Void
			a_settle_date_not_void: a_settle_date /= Void
			a_identifier_not_void: a_identifier /= Void
			a_portfolio_not_void: a_portfolio /= Void
			a_broker_not_void: a_broker /= Void
			a_quantity_not_void: a_quantity /= Void
			a_gross_monies_not_void: a_gross_monies /= Void
			a_net_monies_not_void: a_net_monies /= Void
			a_commission_not_void: a_commission /= Void
			a_fees_not_void: a_fees /= Void
			a_custody_not_void: a_custody /= Void
		do
			gross_amount := a_gross_monies
			make_portfolio_contract (a_generating_system, a_event_date, a_identifier, a_portfolio)
			add_inlieu_flows (a_portfolio, a_broker, a_quantity, a_gross_monies, a_net_monies, a_event_date, a_settle_date, a_custody)
			if not a_commission.amount.is_zero then
				add_flow (commission_flow_type, a_portfolio.instrument_control (a_custody), a_broker.instrument_control, a_event_date, a_settle_date, a_commission, a_event_date, 0, -a_commission)
			end
			if not a_fees.amount.is_zero then
				add_flow (exchange_fee_flow_type, a_portfolio.instrument_control (a_custody), a_broker.instrument_control, a_event_date, a_settle_date, a_fees, a_event_date, 0, -a_fees)
			end
		ensure
			generating_system_set: generating_system = a_generating_system
			agreement_time_set: agreement_time = a_event_date
			identifier_set: identifier = a_identifier
			portfolio_set: portfolio = a_portfolio
			gross_amount_set: gross_amount = a_gross_monies
		end

feature -- Access

	gross_amount: RP_MONIES
			-- Gross amount

	is_long_side: BOOLEAN is
			-- Long side inlieu?
		deferred
		end

feature -- Basic Operations

	process (a_processor: RP_CONTRACT_PROCESSOR) is
			-- <Precursor>
		do
			a_processor.process_inlieu (Current)
		end

feature {NONE} -- Implementation

	add_inlieu_flows (a_portfolio: RP_PORTFOLIO; a_broker: RP_BROKER; a_quantity: RP_QUANTITY; a_gross_monies, a_net_monies: RP_MONIES; a_event_date, a_settle_date: ROSE_DATE; a_custody: RP_CUSTODY) is
			-- Add inliue flows
		require
			a_portfolio_not_void: a_portfolio /= Void
			a_broker_not_void: a_broker /= Void
			a_quantity_not_void: a_quantity /= Void
			a_gross_monies_not_void: a_gross_monies /= Void
			a_net_monies_not_void: a_net_monies /= Void
			a_event_date_not_void: a_event_date /= Void
			a_settle_date_not_void: a_settle_date /= Void
			a_custody_not_void: a_custody /= Void
		deferred
		ensure
			flows_not_void: flows /= Void
			flows_added: flows.count > old flows.count
		end

end
