note

	description:

		"An Instrument"

	author: "mk"
	last_editor: "$Author: e2 $"
	date: "$Date: 2012/02/03 $"
	revision: "$Revision: #62 $"
	path: "$File: //rose/source/kernel/expert_system/rp_instrument.e $"

deferred class RP_INSTRUMENT

inherit

	HASH_KEYED [INTEGER_64]
		rename
			key as uid
		redefine
			is_equal
		end

	DEBUG_OUTPUT
		undefine
			is_equal
		end

feature -- Access

	rose_units: ROSE_UNITS
			-- Underlying rose_units

	name: STRING is
			-- Name of the instrument
		deferred
		end

	brimsym: STRING is
			-- Brimsym.
		deferred
		end

	country_of_issue (a_ref_system: RP_REFERENCE_SYSTEM; a_date: ROSE_DATE): RP_COUNTRY is
			-- The country of issue for this instrument
		deferred
		end

feature -- Status Report

	is_equal (a_other: like Current): BOOLEAN is
			-- <Precursor>
		do
			Result := uid = a_other.uid
		end

	is_currency: BOOLEAN is
			-- Is this instrument a currency?
		do
			Result := False
		end

	is_cash_equivalent: BOOLEAN is
			-- Is this instrument equivalent to cash?
		do
			Result := False
		end

	is_future: BOOLEAN is
			-- Is `Current' a future?
		do
			Result := False
		end

	is_pool_fund: BOOLEAN is
			-- Is `Current' a pool fund?
		do
			Result := False
		end

	is_equity: BOOLEAN is
			-- is equity?
		do
			Result := False
		end

	is_forward: BOOLEAN is
			-- is forward?
		do
			Result := False
		end

	is_adr: BOOLEAN is
			-- is ADR (American Depositary Receipt)?
		do
			Result := False
		end

feature -- Query

	new_quantity (a_value: DOUBLE): RP_INSTRUMENT_QUANTITY [like Current] is
			-- `a_value' amount of `Current'.
		obsolete
			"Use quantity() and RP_QUANTITY instead of RP_INSTRUMENT_QUANTITY"
		do
			create Result.make (a_value, create {RP_INSTRUMENT_UNITS [like Current]}.make (Current))
		end

	quantity (a_amount: RP_DECIMAL): attached RP_QUANTITY is
			-- Quantity of `Current'
		require
			non_void_amount: a_amount /= Void
		do
			create Result.make (Current, a_amount)
		ensure
			same_amount: Result.amount ~ a_amount
		end

	price_in_currency (a_price: RP_DECIMAL; a_currency: RP_CURRENCY; a_date: ROSE_DATE): attached RP_PRICE is
			-- Price of this instrument in `a_currency' on `a_date' constructed from given `a_price'
		require
			non_void_price: a_price /= Void
			non_void_currency: a_currency /= Void
			non_void_date: a_date /= Void
			non_negative_price: a_price >= a_price.zero
		do
			create Result.make (a_price, Current, a_currency, a_date)
		ensure
			same_price: Result.price ~ a_price
			same_currency: Result.currency ~ a_currency
			same_date: Result.date ~ a_date
		end

	uid: INTEGER_64 is
			-- Unique id
		do
			Result := rose_units.numerator.bit_shift_left (32).bit_or (rose_units.denominator.as_integer_32)
		end

	local_brimsym (a_zone: RP_ZONE; a_date: ROSE_DATE): STRING is
			-- `a_zone' specific brimsym on `a_date'
		require
			non_void_zone: a_zone /= Void
		do
				-- By default return the main brimsym
			Result := brimsym
		end

feature -- Output

	debug_output: STRING is
			-- Debug information
		do
			create Result.make_empty
			if name /= Void then
				Result.append (name)
			end
			Result.append_character (' ')
			Result.append (generator)
		end

feature {RP_HOLDING_VALUATOR} -- Valuation

	holding_value_in_currency (a_holding: RP_HOLDING; a_valuator: RP_HOLDING_VALUATOR; a_currency: RP_CURRENCY; a_date: ROSE_DATE; a_bucket: STRING): attached RP_MONIES is
			-- Value of `a_holding' on `a_date' in `a_currency' as computed by `a_valuator'
		require
			non_void_holding: a_holding /= Void
			non_void_valuator: a_valuator /= Void
			non_void_currency: a_currency /= Void
			non_void_date: a_date /= Void
			same_instrument: Current.rose_units ~ a_holding.quantity.instrument.rose_units
		deferred
		ensure
			same_currency: Result.currency ~ a_currency
		end

feature -- Constants

	Valid_brimsym_length: INTEGER is 18
			-- Length of valid BRIMSYM

invariant

	non_void_name: name /= Void
	non_void_units: rose_units /= Void
	non_void_brimsym: brimsym /= Void
	brimsym_correct_length: brimsym.count = Valid_brimsym_length

end
