indexing

	description:

		"Control of holding of instrument"

	date: "$Date: 2009/04/22 $"
	fileid: "$Id: //rose/source/kernel/expert_system/rp_instrument_control.e#11 $"

class RP_INSTRUMENT_CONTROL

inherit

	ANY
		undefine
			is_equal
		end

	RP_UNKNOWN_CUSTODY_AWARE
		export
			{NONE} all
		redefine
			is_equal
		end

	RP_UNKNOWN_MANAGEMENT_AWARE
		export
			{NONE} all
		redefine
			is_equal
		end

	RP_UNSPECIFIED_POSSESSION_AWARE
		export
			{NONE} all
		undefine
			is_equal
		end

	ROSE_FORMATTING
		export
			{NONE} all
		undefine
			is_equal
		end

	DEBUG_OUTPUT
		undefine
			is_equal
		end

create

	make_with_all_details,
	make_with_owner

feature {NONE} -- Initialization

	make_with_all_details (a_owner: like owner; a_custody: like custody;
	a_management: like management;a_possession: like possession) is
		require
			non_void_args: a_owner /= Void and a_custody /= Void and a_management /= Void
				and a_possession /= Void
		do
			owner := a_owner
			custody := a_custody
			management := a_management
			possession := a_possession
		ensure
			owner_set: owner = a_owner
			custody_set: custody = a_custody
			management_set: management = a_management
			possession_set: possession = a_possession
		end

	make_with_owner (a_owner: like owner) is
		require
			non_void_ower: a_owner /= Void
		do
			make_with_all_details (a_owner, unknown_custody, unknown_management, unspecified_possession)
		ensure
			owner_set: owner = a_owner
			custody_set: custody = unknown_custody
			management_set: management = unknown_management
		end

feature -- Access

	owner: RP_PARTY
			-- Owner

	custody: RP_CUSTODY
			-- Relationship to custodian

	management: RP_MANAGEMENT
			-- Relationship to manager

	possession: RP_POSSESSION
		-- Who has the instrument right now


feature -- Output

	debug_output: STRING is
			-- <Precursor>
		do
			Result := display
		end

	display: STRING is
			 -- String representation
		do
			Result := formatter.argumented ("$1,$2,$3,$4,$5",
				<<owner.code, custody.custodian.code,custody.account.code,
				management.manager.code, management.account.code>>)
		end

feature -- Comparison

	is_equal (other: like Current): BOOLEAN is
			-- <Precursor>	
		do
			Result := owner.is_equal (other.owner)
				and custody.is_equal (other.custody)
				and management.is_equal (other.management)
				and possession.is_equal (other.possession)
		end

invariant

	owner_not_void: owner /= Void
	custody_not_void: custody /= Void
	management_not_void: management /= Void
	possession_not_void: possession /= Void

end
