indexing
	description: "Instrument created with a LISTING, that can have different local brimsyms"

	author: "Emmanuel Bouyer"
	last_editor: "$Author: e2 $"
	date: "$Date: 2009/11/26 $"
	revision: "$Revision: #1 $"
	path: "$File: //rose/source/kernel/expert_system/rp_instrument_with_listing.e $"

deferred class RP_INSTRUMENT_WITH_LISTING

inherit
	RP_INSTRUMENT
		redefine
			local_brimsym
		end

feature {NONE} -- Initialization

	initialize_local_brimsyms (a_listing: LISTING)
			-- Initializes local brimsyms from `a_listing'
		require
			a_listing_not_void: a_listing /= Void
		do
			create o_zone_brimsym.make_empty
			create e_zone_brimsym.make_empty
			create j_zone_brimsym.make_empty
			create s_zone_brimsym.make_empty
			a_listing.context.local_brim_code.key_history (a_listing).do_all (agent add_history_interval (o_zone_brimsym, ?))
			a_listing.context.e_brim_code.key_history (a_listing).do_all (agent add_history_interval (e_zone_brimsym, ?))
			a_listing.context.j_brim_code.key_history (a_listing).do_all (agent add_history_interval (j_zone_brimsym, ?))
			a_listing.context.s_brim_code.key_history (a_listing).do_all (agent add_history_interval (s_zone_brimsym, ?))
		end

feature -- Query

	local_brimsym (a_zone: RP_ZONE; a_date: ROSE_DATE): STRING is
			-- `a_zone' specific brimsym on `a_date'
		do
			if a_zone.code.is_equal ("O") then
				Result := o_zone_brimsym.witness (a_date)
			elseif a_zone.code.is_equal ("E") then
				Result := e_zone_brimsym.witness (a_date)
			elseif a_zone.code.is_equal ("J") then
				Result := j_zone_brimsym.witness (a_date)
			elseif a_zone.code.is_equal ("S") then
				Result := s_zone_brimsym.witness (a_date)
			end
			if Result = Void then
					-- By default return the main brimsym
				Result := brimsym
			end
		end

feature {NONE} -- Implementation

	o_zone_brimsym: RP_STRING_HISTORY
			-- O zone specific brimsym

	e_zone_brimsym: RP_STRING_HISTORY
			-- E zone specific brimsym

	j_zone_brimsym: RP_STRING_HISTORY
			-- J zone specific brimsym

	s_zone_brimsym: RP_STRING_HISTORY
			-- S zone specific brimsym

	add_history_interval (a_history: RP_STRING_HISTORY; a_attribute: ENTITY_ATTRIBUTE_RECORD) is
			-- Add string history interval to `a_history'
		require
			a_history_not_void: a_history /= Void
			a_attribute_not_void: a_attribute /= Void
		local
			l_interval: RP_STRING_HISTORY_INTERVAL
		do
			create l_interval.make (a_attribute.attribute_value, a_attribute.start_date, a_attribute.end_date, 1.0)
			a_history.extend (l_interval)
		ensure
			one_more_interval: a_history.count = old a_history.count + 1
		end

invariant
	o_zone_brimsym_not_void: o_zone_brimsym /= Void
	e_zone_brimsym_not_void: e_zone_brimsym /= Void
	j_zone_brimsym_not_void: j_zone_brimsym /= Void
	s_zone_brimsym_not_void: s_zone_brimsym /= Void
end
