note

	description:

		"Entry for a Corporate Action account sweeping info"

	author: "Portfolio Accounting Tech (sm)"
	last_editor: "$Author: bn $"
	date: "$Date: 2014/02/19 $"
	revision: "$Revision: #4 $"
	path: "$File: //rose/source/kernel/capital_actions/elections/rp_invest_w_a_swept_accounts_parm.e $"

class RP_INVEST_W_A_SWEPT_ACCOUNTS_PARM

inherit

	RP_PORTFOLIO_PARM [INVEST_W_A_SWEPT_ACCOUNTS_PARM_RECORD]
		redefine
			make
		end

	DEBUG_OUTPUT

	ROSE_CAPACT_DATE_USE

create

	make

feature {NONE} -- Initialization

	make (a_record: like record) is
			-- <Precursor>
		do
			Precursor (a_record)
			non_special_starting := safe_date_from_ccyymmdd (record.non_special_starting)
			non_special_ending := safe_date_from_ccyymmdd (record.non_special_ending)
			roc_sweep_starting := safe_date_from_ccyymmdd (record.roc_sweep_starting)
			roc_sweep_ending := safe_date_from_ccyymmdd (record.roc_sweep_ending)
		end

feature -- Access

	non_special_starting: ROSE_DATE
			-- Start date for non-special dividends

	non_special_ending: ROSE_DATE
			-- End date for non-special dividend

	roc_sweep_starting: ROSE_DATE
			-- Start date for swept ROC

	roc_sweep_ending: ROSE_DATE
			-- End date for swept ROC

feature -- Status Report

	is_sweeping_roc: BOOLEAN is
			-- Is sweeping of ROC required?
		do
			Result := roc_sweep_starting < roc_sweep_ending
		end

	non_special_brackets (a_date: ROSE_DATE): BOOLEAN is
			-- Is `a_date' in ordinary dividend sweeping interval?
		require
			date_not_void: a_date /= Void
		do
			Result := (non_special_starting <= a_date) and (a_date <= non_special_ending)
		ensure
			definition: Result implies non_special_starting <= a_date and a_date <= non_special_ending
		end

	roc_sweep_brackets (a_date: ROSE_DATE): BOOLEAN is
			-- Is `a_date' in ROC sweeping interval?
		require
			date_not_void: a_date /= Void
		do
			Result := is_sweeping_roc and roc_sweep_starting <= a_date and a_date <= roc_sweep_ending
		ensure
			definition: Result implies is_sweeping_roc and roc_sweep_starting <= a_date and a_date <= roc_sweep_ending
		end

feature -- Output

	debug_output: STRING is
			-- <Precursor>
		do
			create Result.make_empty
			Result.append (clicode)
			Result.append_character (':')
			Result.append_integer (non_special_starting.ccyymmdd)
			Result.append_character ('-')
			Result.append_integer (non_special_ending.ccyymmdd)
			Result.append_character (':')
			Result.append_integer (roc_sweep_starting.ccyymmdd)
			Result.append_character ('-')
			Result.append_integer (roc_sweep_ending.ccyymmdd)
		end

invariant

	non_special_starting_not_void: non_special_starting /= Void
	non_special_starting_set: non_special_starting.ccyymmdd = record.non_special_starting
	non_special_ending_not_void: non_special_ending /= Void
	non_special_ending_set: non_special_ending.ccyymmdd = record.non_special_ending
	valid_non_special_interval: non_special_starting <= non_special_ending
	roc_sweep_starting_not_void: roc_sweep_starting /= Void
	roc_sweep_starting_set: roc_sweep_starting.ccyymmdd = record.roc_sweep_starting
	roc_sweep_ending_not_void: roc_sweep_ending /= Void
	roc_sweep_ending_set: roc_sweep_ending.ccyymmdd = record.roc_sweep_ending

end
