indexing

	description:

		"Aggregate of investment account constituent portfolio daily values"

	date: "$Date: 2010/04/13 $"
	sccid: "$Id: //rose/source/kernel/expert_system/legacy/rp_investment_account_daily_values.e#19 $"

class RP_INVESTMENT_ACCOUNT_DAILY_VALUES

create

	make

feature {NONE} -- Initialization

	make (a_cutoffdates: DS_HASH_TABLE [ROSE_DATE, RP_ZONE]; a_account: RP_INVESTMENT_ACCOUNT; a_fx: RP_CURRENCY_MARKET_DATA_SOURCE; a_portfolio_dv: FUNCTION [ANY, TUPLE [RP_PORTFOLIO], DS_AVL_TREE [RP_PERFORMANCE_DAILY_VALUES, ROSE_DATE]]; a_calendar: RP_CALENDAR; a_ignore_check_portfolios_dv: BOOLEAN) is
			-- Initialize `Current'
		local
			l_dv: DS_HASH_TABLE [DS_AVL_TREE [RP_PERFORMANCE_DAILY_VALUES, ROSE_DATE], RP_PORTFOLIO]
			l_all_dates: DS_HASH_SET [ROSE_DATE]
			l_should_stop: BOOLEAN
			l_date, l_finish, l_cutoffdate: ROSE_DATE
			l_sorted_dates: DS_ARRAYED_LIST [ROSE_DATE]
			i: INTEGER
			l_zone: RP_ZONE
			l_to_be_adjusted, l_dv_on_cutover_date: RP_PERFORMANCE_DAILY_VALUES
			l_portfolios: ROSE_LINEAR_ARRAY [RP_PORTFOLIO_HISTORY_INTERVAL]
			l_portfolio: RP_PORTFOLIO
			l_dailyvalues: RP_PERFORMANCE_DAILY_VALUES
			l_vms_value, l_vms_fxed_value: RP_MONIES
			l_nperf_value, l_nperf_fxed_value: RP_MONIES
			l_rate: RP_DECIMAL
		do
			ignore_check := a_ignore_check_portfolios_dv
			create dailyvalues.make_default
			create l_dv.make_default
			a_account.portfolios.do_all (agent fetch_dv (a_account, l_dv, a_portfolio_dv, ?))
			create l_all_dates.make_equal (17)
			l_dv.do_all (agent add_dates (l_all_dates, ?))
			if not l_all_dates.is_empty then
				create l_sorted_dates.make_from_array (l_all_dates.to_array)
				l_sorted_dates.sort (create {DS_QUICK_SORTER [ROSE_DATE]}.make (create {KL_COMPARABLE_COMPARATOR [ROSE_DATE]}.make))
				from
					l_date := l_sorted_dates.first
					l_finish := l_sorted_dates.last
				until
					l_date > l_finish or l_should_stop
				loop
					l_should_stop := aggregate_dv (a_account, a_fx, a_account.investment_region.legacy_base_currency, l_date, l_dv, a_calendar)
					l_date := a_calendar.ith_work_day_after (l_date, 1)
				end
				if a_cutoffdates /= Void then
					create l_all_dates.make_equal (a_cutoffdates.count)
					a_cutoffdates.do_all_with_key (agent add_cutover_date (l_all_dates, ?, ?))
					from
						i := 2
					until
						i > dailyvalues.count
					loop
						l_dv_on_cutover_date := dailyvalues.item (i)
						l_date := l_dv_on_cutover_date.perf_date
						if l_all_dates.has (l_date) then
							from
								a_cutoffdates.start
							until
								a_cutoffdates.after
							loop
								l_cutoffdate := a_cutoffdates.item_for_iteration
								l_zone := a_cutoffdates.key_for_iteration
								if l_date.ccyymmdd = l_zone.legacy_perf_calendar.ith_work_day_after (l_cutoffdate, 1).ccyymmdd then
									l_to_be_adjusted := dailyvalues.item (i - 1)
									l_portfolios := a_account.portfolios.witness_intervals (l_cutoffdate)
									from
										l_portfolios.start
									until
										l_portfolios.after
									loop
										l_portfolio := l_portfolios.item_for_iteration.portfolio
										if l_portfolio.zone.code.is_equal (l_zone.code) then
											l_dailyvalues := l_dv.item (l_portfolio).item (l_cutoffdate)
											debug ("rp_investment_account_daily_values")
												print (l_date.ccyymmdd.out + "|" + a_account.code + "|" + l_dv_on_cutover_date.portfolio_value.amount.out + "|" + l_to_be_adjusted.portfolio_value.currency.name + "|" + l_to_be_adjusted.previous_portfolio_value.amount.rounded (2).out + "|" + l_to_be_adjusted.previous_portfolio_value.currency.name + "%N")
											end
												-- substract portfolio value
											l_vms_value := -l_dailyvalues.portfolio_value
											l_rate := a_fx.spot_rate (l_vms_value.currency, a_account.investment_region.legacy_base_currency, l_date)
											l_vms_fxed_value := a_account.investment_region.legacy_base_currency.monies (l_vms_value.amount * l_rate)
											debug ("rp_investment_account_daily_values")
												print (l_cutoffdate.ccyymmdd.out + "|" + l_portfolio.code + "|" + l_vms_value.amount.out + "|" + l_vms_value.currency.name + "|" + l_rate.rounded (4).out + "|" + l_vms_fxed_value.amount.rounded (2).out + "|" + l_vms_fxed_value.currency.name + "%N")
											end
											l_to_be_adjusted.set_previous_portfolio_value (l_to_be_adjusted.previous_portfolio_value + l_vms_fxed_value)
												-- add previous portfolio value
											l_nperf_value := l_dailyvalues.previous_portfolio_value
											l_rate := a_fx.spot_rate (l_nperf_value.currency, a_account.investment_region.legacy_base_currency, l_date)
											l_nperf_fxed_value := a_account.investment_region.legacy_base_currency.monies (l_nperf_value.amount * l_rate)
											debug ("rp_investment_account_daily_values")
												print (l_cutoffdate.ccyymmdd.out + "|" + l_portfolio.code + "|" + l_nperf_value.amount.out + "|" + l_nperf_value.currency.name + "|" + l_rate.rounded (4).out + "|" + l_nperf_fxed_value.amount.rounded (2).out + "|" + l_nperf_fxed_value.currency.name + "%N")
											end
											l_to_be_adjusted.set_previous_portfolio_value (l_to_be_adjusted.previous_portfolio_value + l_nperf_fxed_value)
											debug ("rp_investment_account_daily_values")
												print (l_date.ccyymmdd.out + "|" + a_account.code + "|" + (((l_nperf_fxed_value.amount + l_vms_fxed_value.amount) / l_dv_on_cutover_date.portfolio_value.amount) * 100).rounded (2).out + "%N")
											end
										end
										l_portfolios.forth
									end
								end
								a_cutoffdates.forth
							end
						end
						i := i + 1
					end
				end
			end
		end

feature -- Query

	daily_values: ARRAY [RP_PERFORMANCE_DAILY_VALUES] is
			-- Daily values
		do
			Result := dailyvalues.to_array
		end

feature {NONE} -- Implementation

	add_cutover_date (a_dates: DS_HASH_SET [ROSE_DATE]; a_date: ROSE_DATE; a_zone: RP_ZONE) is
			-- Add cutover date for `a_zone' to `a_dates'
			-- This is the next work day after `a_date'
		do
			a_dates.force (a_zone.legacy_perf_calendar.ith_work_day_after (a_date, 1))
		end

	dailyvalues: DS_ARRAYED_LIST [RP_PERFORMANCE_DAILY_VALUES]
			-- Underlying dailyvalues

	fetch_dv (a_account: RP_INVESTMENT_ACCOUNT; a_dv: DS_HASH_TABLE [DS_AVL_TREE [RP_PERFORMANCE_DAILY_VALUES, ROSE_DATE], RP_PORTFOLIO]; a_portfolio_dv: FUNCTION [ANY, TUPLE [RP_PORTFOLIO], DS_AVL_TREE [RP_PERFORMANCE_DAILY_VALUES, ROSE_DATE]]; a_interval: RP_PORTFOLIO_HISTORY_INTERVAL) is
			-- Retrieve dailyvaues for portfolio in `a_interval'
		local
			l_data: DS_AVL_TREE [RP_PERFORMANCE_DAILY_VALUES, ROSE_DATE]
		do
			if not a_dv.has (a_interval.portfolio) then
				l_data := a_portfolio_dv.item ([a_interval.portfolio])
				if l_data /= Void then
					a_dv.force (l_data, a_interval.portfolio)
				end
			end
		end

	add_dates (a_all_dates: DS_HASH_SET [ROSE_DATE]; a_data: DS_AVL_TREE [RP_PERFORMANCE_DAILY_VALUES, ROSE_DATE]) is
			-- Add dates from `a_date' to `a_all_dates'
		do
			if a_data /= Void then
				a_data.do_all_with_key (agent add_date (a_all_dates, ?, ?))
			end
		end

	add_date (a_all_dates: DS_HASH_SET [ROSE_DATE]; a_data: RP_PERFORMANCE_DAILY_VALUES; a_date: ROSE_DATE) is
			-- Add date from `a_date' to `a_all_dates'
		do
			a_all_dates.force (a_date)
		end

	aggregate_dv (a_account: RP_INVESTMENT_ACCOUNT; a_fx: RP_CURRENCY_MARKET_DATA_SOURCE; a_currency: RP_CURRENCY; a_date: ROSE_DATE; a_dv: DS_HASH_TABLE [DS_AVL_TREE [RP_PERFORMANCE_DAILY_VALUES, ROSE_DATE], RP_PORTFOLIO]; a_calendar: RP_CALENDAR): BOOLEAN is
			-- Aggregate dailyvalues
		local
			l_portfolios: ROSE_LINEAR_ARRAY [RP_PORTFOLIO_HISTORY_INTERVAL]
			l_dv: RP_PERFORMANCE_DAILY_VALUES
			l_cell: CELL [BOOLEAN]
		do
			if a_calendar.is_work_day (a_date) then
				l_portfolios := a_account.portfolios.witness_intervals (a_date)
				if l_portfolios.count > 0 then
					create l_cell.put (True)
					l_portfolios.do_all (agent check_dv_available (a_account, l_cell, a_date, a_dv, ?))
					Result := not l_cell.item
					if l_cell.item then
						create l_dv.make (a_date, a_currency)
						l_dv.set_portfolio_value (aggregate_with_lookback (l_portfolios, a_fx, a_currency, a_date, a_dv, retrieve_portfolio_value))
						l_dv.set_equity_value (aggregate_with_lookback (l_portfolios, a_fx, a_currency, a_date, a_dv, retrieve_equity_value))
						l_dv.set_cash_value (aggregate_with_lookback (l_portfolios, a_fx, a_currency, a_date, a_dv, retrieve_cash_value))
						l_dv.set_income (aggregate (l_portfolios, a_fx, a_currency, a_date, a_dv, retrieve_income))
						l_dv.set_expense (aggregate (l_portfolios, a_fx, a_currency, a_date, a_dv, retrieve_expense))
						l_dv.set_perf_unrelated_expense (aggregate (l_portfolios, a_fx, a_currency, a_date, a_dv, retrieve_perf_unrelated_expense))
						l_dv.set_perf_unrelated_income (aggregate (l_portfolios, a_fx, a_currency, a_date, a_dv, retrieve_perf_unrelated_income))
						l_dv.set_dividend (aggregate (l_portfolios, a_fx, a_currency, a_date, a_dv, retrieve_dividend))
						l_dv.set_dividend_tax_withholding (aggregate (l_portfolios, a_fx, a_currency, a_date, a_dv, retrieve_dividend_tax_withholding))
						l_dv.set_withdrawal (aggregate (l_portfolios, a_fx, a_currency, a_date, a_dv, retrieve_withdrawal))
						l_dv.set_contribution (aggregate (l_portfolios, a_fx, a_currency, a_date, a_dv, retrieve_contribution))
						l_dv.set_intrazonal_withdrawal (aggregate (l_portfolios, a_fx, a_currency, a_date, a_dv, retrieve_intrazonal_withdrawal))
						l_dv.set_intrazonal_contribution (aggregate (l_portfolios, a_fx, a_currency, a_date, a_dv, retrieve_intrazonal_contribution))
						l_dv.set_transfer (aggregate (l_portfolios, a_fx, a_currency, a_date, a_dv, retrieve_transfer))
						l_dv.set_withindaybuy (aggregate (l_portfolios, a_fx, a_currency, a_date, a_dv, retrieve_withindaybuy))
						l_dv.set_withindaybuy2 (aggregate (l_portfolios, a_fx, a_currency, a_date, a_dv, retrieve_withindaybuy2))
						l_dv.set_commtaxbuy (aggregate (l_portfolios, a_fx, a_currency, a_date, a_dv, retrieve_commtaxbuy))
						l_dv.set_buycost (aggregate (l_portfolios, a_fx, a_currency, a_date, a_dv, retrieve_buycost))
						l_dv.set_sellnet (aggregate (l_portfolios, a_fx, a_currency, a_date, a_dv, retrieve_sellnet))
						l_dv.set_sellnet2 (aggregate (l_portfolios, a_fx, a_currency, a_date, a_dv, retrieve_sellnet2))
						l_dv.set_realized_hedge_gain_loss (aggregate (l_portfolios, a_fx, a_currency, a_date, a_dv, retrieve_realized_hedge_gain_loss))
						l_dv.set_share_poolfund_expense (aggregate (l_portfolios, a_fx, a_currency, a_date, a_dv, retrieve_share_poolfund_expense))
						l_dv.set_share_poolfund_income (aggregate (l_portfolios, a_fx, a_currency, a_date, a_dv, retrieve_share_poolfund_income))
						dailyvalues.force_last (l_dv)
					end
				end
			end
		end

	ignore_check: BOOLEAN
			-- Ignore portfolio dv check

	check_dv_available (a_account: RP_INVESTMENT_ACCOUNT; a_cell: CELL [BOOLEAN]; a_date: ROSE_DATE; a_dv: DS_HASH_TABLE [DS_AVL_TREE [RP_PERFORMANCE_DAILY_VALUES, ROSE_DATE], RP_PORTFOLIO]; a_interval: RP_PORTFOLIO_HISTORY_INTERVAL) is
			-- Check whether all DailyValues for zonal portfolios are available
		local
			l_date: ROSE_DATE
			l_dv: DS_AVL_TREE [RP_PERFORMANCE_DAILY_VALUES, ROSE_DATE]
		do
			a_dv.search (a_interval.portfolio)
			if a_dv.found then
				if not ignore_check then
					l_dv := a_dv.found_item
					l_date := a_interval.portfolio.zone.legacy_perf_calendar.last_work_day (a_date)
					if not l_dv.has (l_date) then
						if l_date.ccyymmdd = 20080320 and then a_interval.portfolio.zone.code.is_equal ("S") then
								-- XXX ignore 2008/3/20 in S zone
								-- XXX due to buggy calendar
						else
							a_cell.put (False)
						end
					end
				end
			else
				a_cell.put (False)
			end
		end

	aggregate_with_lookback (a_portfolios: ROSE_LINEAR_ARRAY [RP_PORTFOLIO_HISTORY_INTERVAL]; a_fx: RP_CURRENCY_MARKET_DATA_SOURCE; a_currency: RP_CURRENCY; a_date: ROSE_DATE; a_dv: DS_HASH_TABLE [DS_AVL_TREE [RP_PERFORMANCE_DAILY_VALUES, ROSE_DATE], RP_PORTFOLIO]; a_value: FUNCTION [ANY, TUPLE [RP_PERFORMANCE_DAILY_VALUES], RP_MONIES]): RP_MONIES is
			-- Aggregate dailyvalue with lookback
		local
			l_portfolio: RP_PORTFOLIO
			l_dv: DS_AVL_TREE [RP_PERFORMANCE_DAILY_VALUES, ROSE_DATE]
			l_date: ROSE_DATE
			l_found: BOOLEAN
			l_value: RP_MONIES
			l_rate: RP_DECIMAL
		do
			Result := a_currency.monies (0)
			from
				a_portfolios.start
			until
				a_portfolios.off
			loop
				l_portfolio := a_portfolios.current_item.portfolio
				l_dv := a_dv.item (l_portfolio)
				l_date := l_portfolio.zone.legacy_perf_calendar.last_work_day (a_date)
				if l_dv.has (l_date) then
					l_found := True
					l_value := a_value.item ([l_dv.item (l_date)])
					l_rate := a_fx.spot_rate (l_value.currency, a_currency, a_date)
					l_value := a_currency.monies (l_value.amount * l_rate)
					Result := Result + l_value
				end
				a_portfolios.forth
			end
		end

	aggregate (a_portfolios: ROSE_LINEAR_ARRAY [RP_PORTFOLIO_HISTORY_INTERVAL]; a_fx: RP_CURRENCY_MARKET_DATA_SOURCE; a_currency: RP_CURRENCY; a_date: ROSE_DATE; a_dv: DS_HASH_TABLE [DS_AVL_TREE [RP_PERFORMANCE_DAILY_VALUES, ROSE_DATE], RP_PORTFOLIO]; a_value: FUNCTION [ANY, TUPLE [RP_PERFORMANCE_DAILY_VALUES], RP_MONIES]): RP_MONIES is
			-- Aggregate dailyvalue
		local
			l_portfolio: RP_PORTFOLIO
			l_dv: DS_AVL_TREE [RP_PERFORMANCE_DAILY_VALUES, ROSE_DATE]
			l_value: RP_MONIES
			l_rate: RP_DECIMAL
		do
			Result := a_currency.monies (0)
			from
				a_portfolios.start
			until
				a_portfolios.off
			loop
				l_portfolio := a_portfolios.current_item.portfolio
				l_dv := a_dv.item (l_portfolio)
				if l_dv.has (a_date) then
					l_value := a_value.item ([l_dv.item (a_date)])
					l_rate := a_fx.spot_rate (l_value.currency, a_currency, a_date)
					l_value := a_currency.monies (l_value.amount * l_rate)
					Result := Result + l_value
				end
				a_portfolios.forth
			end
		end

	retrieve_portfolio_value: FUNCTION [ANY, TUPLE [RP_PERFORMANCE_DAILY_VALUES], RP_MONIES] is
			-- Retrieve `' from DailyValues
		once
			Result := agent {RP_PERFORMANCE_DAILY_VALUES}.portfolio_value
		end

	retrieve_equity_value: FUNCTION [ANY, TUPLE [RP_PERFORMANCE_DAILY_VALUES], RP_MONIES] is
			-- Retrieve `' from DailyValues
		once
			Result := agent {RP_PERFORMANCE_DAILY_VALUES}.equity_value
		end

	retrieve_cash_value: FUNCTION [ANY, TUPLE [RP_PERFORMANCE_DAILY_VALUES], RP_MONIES] is
			-- Retrieve `' from DailyValues
		once
			Result := agent {RP_PERFORMANCE_DAILY_VALUES}.cash_value
		end

	retrieve_income: FUNCTION [ANY, TUPLE [RP_PERFORMANCE_DAILY_VALUES], RP_MONIES] is
			-- Retrieve `' from DailyValues
		once
			Result := agent {RP_PERFORMANCE_DAILY_VALUES}.income
		end

	retrieve_expense: FUNCTION [ANY, TUPLE [RP_PERFORMANCE_DAILY_VALUES], RP_MONIES] is
			-- Retrieve `' from DailyValues
		once
			Result := agent {RP_PERFORMANCE_DAILY_VALUES}.expense
		end

	retrieve_perf_unrelated_expense: FUNCTION [ANY, TUPLE [RP_PERFORMANCE_DAILY_VALUES], RP_MONIES] is
			-- Retrieve `' from DailyValues
		once
			Result := agent {RP_PERFORMANCE_DAILY_VALUES}.perf_unrelated_expense
		end

	retrieve_perf_unrelated_income: FUNCTION [ANY, TUPLE [RP_PERFORMANCE_DAILY_VALUES], RP_MONIES] is
			-- Retrieve `' from DailyValues
		once
			Result := agent {RP_PERFORMANCE_DAILY_VALUES}.perf_unrelated_income
		end

	retrieve_dividend: FUNCTION [ANY, TUPLE [RP_PERFORMANCE_DAILY_VALUES], RP_MONIES] is
			-- Retrieve `' from DailyValues
		once
			Result := agent {RP_PERFORMANCE_DAILY_VALUES}.dividend
		end

	retrieve_dividend_tax_withholding: FUNCTION [ANY, TUPLE [RP_PERFORMANCE_DAILY_VALUES], RP_MONIES] is
			-- Retrieve `' from DailyValues
		once
			Result := agent {RP_PERFORMANCE_DAILY_VALUES}.dividend_tax_withholding
		end

	retrieve_withdrawal: FUNCTION [ANY, TUPLE [RP_PERFORMANCE_DAILY_VALUES], RP_MONIES] is
			-- Retrieve `' from DailyValues
		once
			Result := agent {RP_PERFORMANCE_DAILY_VALUES}.withdrawal
		end

	retrieve_contribution: FUNCTION [ANY, TUPLE [RP_PERFORMANCE_DAILY_VALUES], RP_MONIES] is
			-- Retrieve `' from DailyValues
		once
			Result := agent {RP_PERFORMANCE_DAILY_VALUES}.contribution
		end

	retrieve_intrazonal_withdrawal: FUNCTION [ANY, TUPLE [RP_PERFORMANCE_DAILY_VALUES], RP_MONIES] is
			-- Retrieve `' from DailyValues
		once
			Result := agent {RP_PERFORMANCE_DAILY_VALUES}.intrazonal_withdrawal
		end

	retrieve_intrazonal_contribution: FUNCTION [ANY, TUPLE [RP_PERFORMANCE_DAILY_VALUES], RP_MONIES] is
			-- Retrieve `' from DailyValues
		once
			Result := agent {RP_PERFORMANCE_DAILY_VALUES}.intrazonal_contribution
		end

	retrieve_transfer: FUNCTION [ANY, TUPLE [RP_PERFORMANCE_DAILY_VALUES], RP_MONIES] is
			-- Retrieve `' from DailyValues
		once
			Result := agent {RP_PERFORMANCE_DAILY_VALUES}.transfer
		end

	retrieve_withindaybuy: FUNCTION [ANY, TUPLE [RP_PERFORMANCE_DAILY_VALUES], RP_MONIES] is
			-- Retrieve `' from DailyValues
		once
			Result := agent {RP_PERFORMANCE_DAILY_VALUES}.withindaybuy
		end

	retrieve_withindaybuy2: FUNCTION [ANY, TUPLE [RP_PERFORMANCE_DAILY_VALUES], RP_MONIES] is
			-- Retrieve `withindaybuy2' from DailyValues
		once
			Result := agent {RP_PERFORMANCE_DAILY_VALUES}.withindaybuy2
		end

	retrieve_commtaxbuy: FUNCTION [ANY, TUPLE [RP_PERFORMANCE_DAILY_VALUES], RP_MONIES] is
			-- Retrieve `' from DailyValues
		once
			Result := agent {RP_PERFORMANCE_DAILY_VALUES}.commtaxbuy
		end

	retrieve_buycost: FUNCTION [ANY, TUPLE [RP_PERFORMANCE_DAILY_VALUES], RP_MONIES] is
			-- Retrieve `' from DailyValues
		once
			Result := agent {RP_PERFORMANCE_DAILY_VALUES}.buycost
		end

	retrieve_sellnet: FUNCTION [ANY, TUPLE [RP_PERFORMANCE_DAILY_VALUES], RP_MONIES] is
			-- Retrieve `' from DailyValues
		once
			Result := agent {RP_PERFORMANCE_DAILY_VALUES}.sellnet
		end

	retrieve_sellnet2: FUNCTION [ANY, TUPLE [RP_PERFORMANCE_DAILY_VALUES], RP_MONIES] is
			-- Retrieve `' from DailyValues
		once
			Result := agent {RP_PERFORMANCE_DAILY_VALUES}.sellnet2
		end

	retrieve_realized_hedge_gain_loss: FUNCTION [ANY, TUPLE [RP_PERFORMANCE_DAILY_VALUES], RP_MONIES] is
			-- Retrieve `' from DailyValues
		once
			Result := agent {RP_PERFORMANCE_DAILY_VALUES}.realized_hedge_gain_loss
		end

	retrieve_share_poolfund_expense: FUNCTION [ANY, TUPLE [RP_PERFORMANCE_DAILY_VALUES], RP_MONIES] is
			-- Retrieve `' from DailyValues
		once
			Result := agent {RP_PERFORMANCE_DAILY_VALUES}.share_poolfund_expense
		end

	retrieve_share_poolfund_income: FUNCTION [ANY, TUPLE [RP_PERFORMANCE_DAILY_VALUES], RP_MONIES] is
			-- Retrieve `' from DailyValues
		once
			Result := agent {RP_PERFORMANCE_DAILY_VALUES}.share_poolfund_income
		end

end
