note

	description:

		"A client of RP_INVESTMENT_MANAGER"

	author: "Mark Howard"
	last_editor: "$Author: it $"
	date: "$Date: 2013/06/28 $"
	revision: "$Revision: #11 $"
	path: "$File: //rose/source/kernel/expert_system/rp_investment_manager_client.e $"

deferred class RP_INVESTMENT_MANAGER_CLIENT

feature {NONE} -- Initialization

	make (a_manager: like manager) is
			-- Initialize a new manager client given a manager
		require
			non_void_manager: a_manager /= Void
		do
			manager_id := a_manager.generating_type.hash_code
			if not known_managers.has (manager_id) then
				known_managers.put (a_manager, manager_id)
			end
		ensure
			cell_has_my_manager: known_managers.has (manager_id)
		end

feature -- Access

	manager: like manager_template is
			-- The investment manager for 'manager_id'
		do
			Result := known_managers.item (manager_id)
		end

feature -- NAV queries

	portfolio_nav (a_portfolio: RP_PORTFOLIO; a_date: ROSE_DATE): RP_INSTRUMENT_QUANTITY [RP_CURRENCY] is
			-- `a_portfolio' value on `a_date' in USD
		local
			l_value: RP_MONIES
		do
			l_value := manager.end_of_day_nav.portfolio_value (a_date, a_portfolio, usd)
			if l_value /= Void then
				Result := usd.new_quantity (l_value.amount)
			end
		end

	global_account_nav (a_account: RP_INVESTMENT_ACCOUNT; a_date: ROSE_DATE): RP_INSTRUMENT_QUANTITY [RP_CURRENCY] is
			-- `a_account' value on `a_date' in USD
		local
			l_value: RP_MONIES
		do
			l_value := manager.end_of_day_nav.account_value (a_date, a_account, usd)
			if l_value /= Void then
				Result := usd.new_quantity (l_value.amount)
			end
		end

feature {NONE} -- Implementations

	manager_id: INTEGER
			-- Id for this type of manager (not unique)

	known_managers: DS_HASH_TABLE [like manager_template, INTEGER] is
			-- Known managers by type_id
		do
			Result ?= known_managers_table [generating_type]
			if Result = Void then
				create Result.make_default
				known_managers_table.put (Result, generating_type)
			end
		end

	known_managers_table: HASH_TABLE [DS_HASH_TABLE [RP_INVESTMENT_MANAGER, INTEGER], STRING] is
			-- Table for once per type
		once
			create Result.make (10)
		end

	manager_template: RP_INVESTMENT_MANAGER is
			-- Template for the manager
		do
			do_nothing
		end

	usd: RP_CURRENCY is
			-- USD.
		once
			Result := manager.reference_system.currency_from_iso_code ("USD")
		end

end
