note

	description:

		"The Legacy Performance System"

	author: "Ivan Tarnavsky"
	last_editor: "$Author: it $"
	date: "$Date: 2014/04/14 $"
	revision: "$Revision: #52 $"
	path: "$File: //rose/source/kernel/expert_system/legacy/rp_legacy_perf.e $"

class RP_LEGACY_PERF

inherit

	RP_INVESTMENT_MANAGER_CLIENT
		redefine
			make
		end

	RP_LEGACY_PERF_DAILYVALUE_AWARE
		export {NONE} all end

create {RP_LEGACY_SYSTEM}

	make

feature {NONE} -- Initialization

	make (a_manager: like manager) is
			-- Initialize `Current'.
		do
			Precursor (a_manager)
			clear_caches
		end

feature -- Query

	calendar (a_investment_region: RP_ZONE): RP_CALENDAR is
			-- Legacy perf calendar for `a_investment_region'.
		require
			non_void_zone: a_investment_region /= Void
		do
			Result := a_investment_region.legacy_perf_calendar
		ensure
			non_void_result: Result /= Void
		end

	clear_caches is
			-- Wipeout caches.
		do
			create portfolio_cached.make_default
			create portfolio_dv_cache.make_default
			create account_dv_cache.make_equal (17)
		ensure
			portfolio_cached_not_void: portfolio_cached /= Void
			portfolio_dv_cache_not_void: portfolio_dv_cache /= Void
			account_dv_cache_not_void: account_dv_cache /= Void
			portfolio_cached_is_empty: portfolio_cached.is_empty
			portfolio_dv_cache_is_empty: portfolio_dv_cache.is_empty
			account_dv_cache_is_empty: account_dv_cache.is_empty
		end

	portfolio_dailyvalues (a_portfolio: RP_PORTFOLIO; a_currency: RP_CURRENCY): ARRAY [RP_PERFORMANCE_DAILY_VALUES] is
			-- Dailyvalues for `a_portfolio' in `a_currency'
		require
			non_void_args: a_portfolio /= Void and a_currency /= Void
		local
			l_result: DS_LINKED_LIST [RP_PERFORMANCE_DAILY_VALUES]
			l_dv: like portfolio_dv
		do
			l_dv := portfolio_dv (a_portfolio, False)
			if l_dv /= Void then
				create l_result.make_equal
				l_dv.do_all (agent fx_to (l_result, a_currency, ?))
				Result := l_result.to_array
			else
				Result := <<>>
			end
		ensure
			non_void_result: Result /= Void
		end

	portfolio_dailyvalues_at (a_portfolio: RP_PORTFOLIO; a_currency: RP_CURRENCY; a_date: ROSE_DATE): RP_PERFORMANCE_DAILY_VALUES is
			-- Dailyvalues for `a_portfolio' in `a_currency' at `a_date'
		require
			non_void_args: a_portfolio /= Void and a_currency /= Void and a_date /= Void
		local
			l_dv: like portfolio_dv
		do
			l_dv := portfolio_dv (a_portfolio, False)
			if l_dv /= Void then
				if l_dv.has (a_date) then
					Result := l_dv.item (a_date).in_currency (a_currency, fx)
				end
			end
		ensure
			result_may_be_void: True
		end

	portfolio_dailyreturns (a_portfolio: RP_PORTFOLIO; a_currency: RP_CURRENCY): RP_DAILY_RETURNS is
			-- Dailyreturns for `a_portfolio' in `a_currency'
		require
			non_void_args: a_portfolio /= Void and a_currency /= Void
		do
			Result := internal_portfolio_dailyreturns (a_portfolio, a_currency)
		ensure
			non_void_result: Result /= Void
		end

	portfolio_equity_only_dailyreturns (a_portfolio: RP_PORTFOLIO; a_currency: RP_CURRENCY): RP_DAILY_RETURNS is
			-- Dailyreturns for `a_portfolio' in `a_currency'
		require
			non_void_args: a_portfolio /= Void and a_currency /= Void
		do
			Result := internal_equityonly_dailyreturns (a_portfolio, a_currency)
		ensure
			non_void_result: Result /= Void
		end

	portfolio_net_of_divwithtax_dailyreturns (a_portfolio: RP_PORTFOLIO; a_currency: RP_CURRENCY): RP_DAILY_RETURNS is
			-- Dailyreturns for `a_portfolio' in `a_currency'
		require
			non_void_args: a_portfolio /= Void and a_currency /= Void
		local
			l_dv: like portfolio_dailyvalues
			l_adj: RP_PERFORMANCE_NET_DIVWITHTAX_AND_SHAREPOOLFUND_INCEXP
			l_result: !RP_PERFORMANCE_DAILY_RETURNS
		do
			create l_adj
			l_dv := portfolio_dailyvalues (a_portfolio, a_currency)
			create l_result.make (a_portfolio.code, regular_performance_numerator, regular_performance_denominator, l_adj)
			l_dv.do_all (agent add_daily_return (l_result, ?))
			Result := l_result
		ensure
			non_void_result: Result /= Void
		end

	investment_account_dailyvalues (a_account: RP_INVESTMENT_ACCOUNT; a_currency: RP_CURRENCY): ARRAY [RP_PERFORMANCE_DAILY_VALUES] is
			-- Dailyvalues for `a_account' in `a_currency'
		require
			non_void_args: a_account /= Void and a_currency /= Void
		do
			Result := investment_account_dailyvalues_with_calendar (a_account, a_currency, a_account.investment_region.legacy_perf_calendar)
		ensure
			non_void_result: Result /= Void
		end

	investment_account_dailyvalues_with_calendar (a_account: RP_INVESTMENT_ACCOUNT; a_currency: RP_CURRENCY; a_calendar: RP_CALENDAR): ARRAY [RP_PERFORMANCE_DAILY_VALUES] is
			-- Dailyvalues for `a_account' in `a_currency'
		require
			non_void_args: a_account /= Void and a_currency /= Void and a_calendar /= Void
		local
			l_result: DS_LINKED_LIST [RP_PERFORMANCE_DAILY_VALUES]
			l_dv: like retrieve_account_dailyvalues
		do
			l_dv := retrieve_account_dailyvalues (a_account, a_calendar)
			if l_dv /= Void then
				create l_result.make_equal
				l_dv.do_all (agent fx_to (l_result, a_currency, ?))
				Result := l_result.to_array
			else
				Result := <<>>
			end
		ensure
			non_void_result: Result /= Void
		end

	investment_account_dailyvalues_at (a_account: RP_INVESTMENT_ACCOUNT; a_currency: RP_CURRENCY; a_date: ROSE_DATE): RP_PERFORMANCE_DAILY_VALUES is
			-- Dailyvalues for `a_account' in `a_currency' at `a_date'
		require
			non_void_args: a_account /= Void and a_currency /= Void and a_date /= Void
		do
			Result := investment_account_dailyvalues_at_with_calendar (a_account, a_currency, a_date, a_account.investment_region.legacy_perf_calendar)
		ensure
			result_may_be_void: True
		end

	investment_account_dailyvalues_at_with_calendar (a_account: RP_INVESTMENT_ACCOUNT; a_currency: RP_CURRENCY; a_date: ROSE_DATE; a_calendar: RP_CALENDAR): RP_PERFORMANCE_DAILY_VALUES is
			-- Dailyvalues for `a_account' in `a_currency' at `a_date'
		require
			non_void_args: a_account /= Void and a_currency /= Void and a_date /= Void and a_calendar /= Void
		local
			l_dv: like retrieve_account_dailyvalues
		do
			l_dv := retrieve_account_dailyvalues (a_account, a_calendar)
			if l_dv /= Void then
				if l_dv.has (a_date) then
					Result := l_dv.item (a_date).in_currency (a_currency, fx)
				end
			end
		ensure
			result_may_be_void: True
		end

	investment_account_dailyreturns (a_account: RP_INVESTMENT_ACCOUNT; a_currency: RP_CURRENCY): RP_DAILY_RETURNS is
			-- Dailyreturns for `a_account' in `a_currency'
		require
			non_void_args: a_account /= Void and a_currency /= Void
		do
			Result := investment_account_dailyreturns_with_calendar (a_account, a_currency, a_account.investment_region.legacy_perf_calendar)
		ensure
			non_void_result: Result /= Void
		end

	investment_account_dailyreturns_with_calendar (a_account: RP_INVESTMENT_ACCOUNT; a_currency: RP_CURRENCY; a_calendar: RP_CALENDAR): RP_DAILY_RETURNS is
			-- Dailyreturns for `a_account' in `a_currency' with calendar from `a_zone'
		require
			non_void_args: a_account /= Void and a_currency /= Void and a_calendar /= Void
		local
			l_dv: like investment_account_dailyvalues
			l_adj: RP_PERFORMANCE_NET_OF_SHAREPOOLFUND_INCEXP
			l_result: RP_PERFORMANCE_DAILY_RETURNS
		do
			create l_adj
			l_dv := investment_account_dailyvalues_with_calendar (a_account, a_currency, a_calendar)
			create l_result.make (a_account.name, regular_performance_numerator, regular_performance_denominator, l_adj)
			l_dv.do_all (agent add_daily_return (l_result, ?))
			Result := l_result
		ensure
			non_void_result: Result /= Void
		end

	investment_account_net_of_fees_dailyreturns (a_account: RP_INVESTMENT_ACCOUNT; a_currency: RP_CURRENCY): RP_DAILY_RETURNS is
			-- Net of fees dailyreturns for `a_account' in `a_currency'
		require
			non_void_args: a_account /= Void and a_currency /= Void
		do
			Result := investment_account_net_of_fees_dailyreturns_with_calendar (a_account, a_currency, a_account.investment_region.legacy_perf_calendar)
		ensure
			non_void_result: Result /= Void
		end

	investment_account_net_of_fees_dailyreturns_with_calendar (a_account: RP_INVESTMENT_ACCOUNT; a_currency: RP_CURRENCY; a_calendar: RP_CALENDAR): RP_DAILY_RETURNS is
			-- Net of fees dailyreturns for `a_account' in `a_currency' with `a_calendar'
		require
			non_void_args: a_account /= Void and a_currency /= Void and a_calendar /= Void
		local
			l_dv: like investment_account_dailyvalues
			l_adj: RP_PERFORMANCE_NET_OF_FEES_AND_SHAREPOOLFUND_INCEXP
			l_result: RP_PERFORMANCE_DAILY_RETURNS
			l_fees: !DS_HASH_TABLE [RP_MONIES, ROSE_DATE]
		do
			l_fees := manager.financial_system.base_and_performance_fees (a_account, a_calendar)
			create l_adj
			l_dv := investment_account_dailyvalues_with_calendar (a_account, a_currency, a_calendar)
			l_dv.do_all (agent add_fees (l_fees, ?))
			create l_result.make (a_account.name, regular_performance_numerator, regular_performance_denominator, l_adj)
			l_dv.do_all (agent add_daily_return (l_result, ?))
			Result := l_result
		ensure
			non_void_result: Result /= Void
		end

	investment_account_net_of_divwithtax_dailyreturns (a_account: RP_INVESTMENT_ACCOUNT; a_currency: RP_CURRENCY): RP_DAILY_RETURNS is
			-- Net of fees dailyreturns for `a_account' in `a_currency'
		require
			non_void_args: a_account /= Void and a_currency /= Void
		do
			Result := investment_account_net_of_divwithtax_dailyreturns_with_calendar (a_account, a_currency, a_account.investment_region.legacy_perf_calendar)
		ensure
			non_void_result: Result /= Void
		end

	investment_account_net_of_divwithtax_dailyreturns_with_calendar (a_account: RP_INVESTMENT_ACCOUNT; a_currency: RP_CURRENCY; a_calendar: RP_CALENDAR): RP_DAILY_RETURNS is
			-- Net of fees dailyreturns for `a_account' in `a_currency' with `a_calendar'
		require
			non_void_args: a_account /= Void and a_currency /= Void and a_calendar /= Void
		local
			l_dv: like investment_account_dailyvalues
			l_adj: RP_PERFORMANCE_NET_DIVWITHTAX_AND_SHAREPOOLFUND_INCEXP
			l_result: RP_PERFORMANCE_DAILY_RETURNS
		do
			create l_adj
			l_dv := investment_account_dailyvalues_with_calendar (a_account, a_currency, a_calendar)
			create l_result.make (a_account.name, regular_performance_numerator, regular_performance_denominator, l_adj)
			l_dv.do_all (agent add_daily_return (l_result, ?))
			Result := l_result
		ensure
			non_void_result: Result /= Void
		end

	investment_account_hedged_dailyreturns (a_account: RP_INVESTMENT_ACCOUNT; a_currency: RP_CURRENCY): RP_DAILY_RETURNS is
			-- Hedged dailyreturns for `a_account' in `a_currency'
		require
			non_void_args: a_account /= Void and a_currency /= Void
		do
			Result := investment_account_hedged_dailyreturns_with_calendar (a_account, a_currency, a_account.investment_region.legacy_perf_calendar)
		ensure
			non_void_result: Result /= Void
		end

	investment_account_hedged_dailyreturns_with_calendar (a_account: RP_INVESTMENT_ACCOUNT; a_currency: RP_CURRENCY; a_calendar: RP_CALENDAR): RP_DAILY_RETURNS is
			-- Hedged dailyreturns for `a_account' in `a_currency' with calendar from `a_zone'
		require
			non_void_args: a_account /= Void and a_currency /= Void and a_calendar /= Void
		local
			l_dv: like investment_account_dailyvalues
			l_adj: RP_PERFORMANCE_NET_HEDGEGL_SHAREPOOLFUND_INCEXP
			l_result: RP_PERFORMANCE_DAILY_RETURNS
		do
			create l_adj
			l_dv := investment_account_dailyvalues_with_calendar (a_account, a_currency, a_calendar)
			create l_result.make (a_account.name, regular_performance_numerator, regular_performance_denominator, l_adj)
			l_dv.do_all (agent add_daily_return (l_result, ?))
			Result := l_result
		ensure
			non_void_result: Result /= Void
		end

	zonal_product_dailyreturns (a_zonal_product: RP_ZONAL_PRODUCT; a_currency: RP_CURRENCY): RP_DAILY_RETURNS is
			-- DailyReturns in `a_currency' for `a_zonal_product'
		require
			non_void_args: a_zonal_product /= Void and a_currency /= Void
		do
			Result := zonal_product_dailyreturns_with_feedback (a_zonal_product, a_currency, Void, Void)
		ensure
			result_not_void: Result /= Void
		end

	zonal_product_dailyreturns_with_feedback (a_zonal_product: RP_ZONAL_PRODUCT; a_currency: RP_CURRENCY; a_bmv_feedback, a_monthlybmv_feedback: PROCEDURE [ANY, TUPLE [STRING]]): RP_DAILY_RETURNS is
			-- DailyReturns in `a_currency' for `a_zonal_product'
		require
			non_void_args: a_zonal_product /= Void and a_currency /= Void
		local
			l_result: RP_ZONAL_PRODUCT_DAILY_RETURNS
		do
			create l_result.make (a_zonal_product, a_currency, agent internal_portfolio_dailyreturns (?, a_currency), agent zonal_product_portfolio_weights (a_zonal_product, a_currency, ?), a_bmv_feedback, a_monthlybmv_feedback)
			Result := l_result.daily_returns
		ensure
			result_not_void: Result /= Void
		end

	global_product_dailyreturns (a_global_product: RP_GLOBAL_PRODUCT; a_currency: RP_CURRENCY): RP_DAILY_RETURNS is
			-- DailyReturns in `a_currency' for `a_global_product'
		require
			non_void_args: a_global_product /= Void and a_currency /= Void
		do
			Result := global_product_dailyreturns_with_feedback (a_global_product, a_currency, Void, Void)
		ensure
			result_not_void: Result /= Void
		end

	global_product_dailyreturns_with_feedback (a_global_product: RP_GLOBAL_PRODUCT; a_currency: RP_CURRENCY; a_bmv_feedback, a_monthlybmv_feedback: PROCEDURE [ANY, TUPLE [STRING]]): RP_DAILY_RETURNS is
			-- DailyReturns in `a_currency' for `a_global_product'
		require
			non_void_args: a_global_product /= Void and a_currency /= Void
		local
			l_result: RP_GLOBAL_PRODUCT_DAILY_RETURNS
		do
			create l_result.make (a_global_product, a_currency, agent investment_account_dailyreturns_with_calendar (?, a_currency, a_global_product.investment_region.legacy_perf_calendar), agent global_product_account_weights (a_global_product, a_currency, ?, a_global_product.investment_region), a_bmv_feedback, a_monthlybmv_feedback)
			Result := l_result.daily_returns
		ensure
			result_not_void: Result /= Void
		end

	set_dailyvalues_server_and_database (a_server: STRING; a_database: STRING) is
			-- Set DailyValues database server and database names
		require
			a_server_not_void: a_server /= Void
			a_database_not_void: a_database /= Void
		do
			clear_caches
			dailyvalues_sql_server := a_server
			dailyvalues_sql_database := a_database
			internal_daily_values_query := Void
		ensure
			dailyvalues_sql_server_set: dailyvalues_sql_server = a_server
			dailyvalues_sql_database_set: dailyvalues_sql_database = a_database
			internal_daily_values_query_is_void: internal_daily_values_query = Void
			portfolio_cached_is_empty: portfolio_cached.is_empty
			portfolio_dv_cache_is_empty: portfolio_dv_cache.is_empty
			account_dv_cache_is_empty: account_dv_cache.is_empty
		end

feature {NONE} -- Implementation

	portfolio_cached: DS_HASH_SET [RP_PORTFOLIO]
			-- Indicates that DailyValues for given portfolio are cached

	portfolio_dv_cache: DS_HASH_TABLE [like portfolio_dv, RP_PORTFOLIO]
			-- Cached portfolio DailyValues

	account_dv_cache: DS_HASH_TABLE [like retrieve_account_dailyvalues, STRING]
			-- Cached account DailyValues

	cutoff_dates: DS_HASH_TABLE [ROSE_DATE, RP_ZONE] is
			-- Cutoff dates between VMS and Eiffel DailyPerf by zone
		local
			l_query: STRING_PREDICATE
		once
			create Result.make_with_equality_testers (7, Void, create {DYNAMIC_COMPARATOR [RP_ZONE]}.make (agent zone_less_than))
			create l_query.make (dailyvalues_connection, "SELECT zone + '|' + cast(cutoff as varchar(8)) FROM dbo.zone_cutoff")
			l_query.retrieve (Void, Void)
			l_query.data_server.commit
			l_query.array.do_all (agent add_cutoff_date (Result, ?))
		end

	zone_less_than (a_zone1, a_zone2: RP_ZONE): BOOLEAN is
			-- Is `a_zone1' considered less than `a_zone2'?
		require
			a_zone1_not_void: a_zone1 /= Void
			a_zone2_not_void: a_zone2 /= Void
		do
			Result := a_zone1.code < a_zone2.code
		end

	fx: RP_AXAROSENBERG_CURRENCY_MARKET_DATA_SOURCE is
			-- Currency market data source
		once
			create Result.make (manager.reference_system)
		end

	fx_to (a_result: DS_LINKED_LIST [RP_PERFORMANCE_DAILY_VALUES]; a_currency: RP_CURRENCY; a_dv: RP_PERFORMANCE_DAILY_VALUES) is
			-- Translate `a_dv' to currency `a_currency' and add to `a_result'
		do
			if a_currency.is_same (a_dv.currency) then
				a_result.force_last (a_dv)
			else
				a_result.force_last (a_dv.in_currency (a_currency, fx))
			end
		end

	portfolio_dv (a_portfolio: RP_PORTFOLIO; a_for_account: BOOLEAN): DS_AVL_TREE [RP_PERFORMANCE_DAILY_VALUES, ROSE_DATE] is
			-- Dailyvalues for `a_portfolio' hashed by ccyymmdd
		do
			retrieve_daily_values (a_portfolio, a_for_account)
			portfolio_dv_cache.search (a_portfolio)
			if portfolio_dv_cache.found then
				Result := portfolio_dv_cache.found_item
			end
		end

	retrieve_account_dailyvalues (a_account: RP_INVESTMENT_ACCOUNT; a_calendar: RP_CALENDAR): DS_AVL_TREE [RP_PERFORMANCE_DAILY_VALUES, ROSE_DATE] is
			-- Retrieve dailyvalues for `a_account'
		require
			non_void_args: a_account /= Void and a_calendar /= Void
		local
			l_dv: RP_INVESTMENT_ACCOUNT_DAILY_VALUES
		do
			account_dv_cache.search (a_account.name)
			if account_dv_cache.found then
				Result := account_dv_cache.found_item
			else
				create l_dv.make (cutoff_dates, a_account, fx, agent portfolio_dv (?, True), a_calendar, False)
				create Result.make (create {KL_COMPARABLE_COMPARATOR [ROSE_DATE]}.make)
				l_dv.daily_values.do_all (agent add_account_dailyvalues (Result, ?))
				account_dv_cache.force (Result, a_account.name)
			end
		end

	add_account_dailyvalues (a_dvs: like retrieve_account_dailyvalues; a_dv: RP_PERFORMANCE_DAILY_VALUES) is
			-- Add `a_dv' to `a_dvs'
		require
			non_void_args: a_dvs /= Void and a_dv /= Void
		do
			a_dvs.force (a_dv, a_dv.perf_date)
		end

	retrieve_daily_values (a_portfolio: RP_PORTFOLIO; a_for_account: BOOLEAN) is
			-- Retrieve daily values for `a_portfolio'
		local
			l_date: ROSE_DATE
			l_hash: like portfolio_dv
			l_dailyvalues: ARRAY [RP_PERFORMANCE_DAILY_VALUES]
		do
			if not portfolio_cached.has (a_portfolio) then
				portfolio_dv_cache.search (a_portfolio)
				if portfolio_dv_cache.found then
					l_hash := portfolio_dv_cache.found_item
				else
					create l_hash.make (create {KL_COMPARABLE_COMPARATOR [ROSE_DATE]}.make)
					portfolio_dv_cache.force (l_hash, a_portfolio)
				end
				l_date := cutoff_date (a_portfolio.zone)
				if a_for_account then
					l_dailyvalues := daily_values_query.nperf_dailyvalues_for_account_on_and_after (a_portfolio, l_date)
				else
					l_dailyvalues := daily_values_query.nperf_dailyvalues_on_and_after (a_portfolio, l_date)
				end
				l_dailyvalues.do_all (agent cache_row (l_hash, a_portfolio, ?, False, l_date))
				if a_for_account then
					l_dailyvalues := daily_values_query.vms_dailyvalues_for_account_on_and_before (a_portfolio, l_date)
				else
					l_dailyvalues := daily_values_query.vms_dailyvalues_on_and_before (a_portfolio, l_date)
				end
				l_dailyvalues.do_all (agent cache_row (l_hash, a_portfolio, ?, True, l_date))
				portfolio_cached.force (a_portfolio)
			end
		end

	cutoff_date (a_zone: RP_ZONE): ROSE_DATE is
			-- Cutoff date between vms and new perf for `a_zone'
		do
			if cutoff_dates.has (a_zone) then
				Result := cutoff_dates.item (a_zone)
			else
				create Result.make_ccyymmdd (99991231)
			end
		end

	add_cutoff_date (a_cutoff_dates: like cutoff_dates; s: STRING) is
			-- Add cutoff date definition from ppipe delimited zone|ccyymmd from `s'
		local
			l_zone: RP_ZONE
			l_date: ROSE_DATE
		do
			l_zone := manager.reference_system.investment_region_by_code (s.substring (1, 1))
			if not l_zone.is_unknown then
				create l_date.make_ccyymmdd (s.substring (3, s.count).to_integer)
				a_cutoff_dates.force (l_date, l_zone)
			end
		end

	cache_row (a_cache: like portfolio_dv; a_portfolio: RP_PORTFOLIO; a_row: RP_PERFORMANCE_DAILY_VALUES; a_is_legacy: BOOLEAN; a_cutoff_date: ROSE_DATE) is
			-- Cache `a_row'
		require
			a_cache_not_void: a_cache /= Void
			a_portfolio_not_void: a_portfolio /= Void
			a_row_not_void: a_portfolio /= Void
			a_row_perf_date_not_void: a_row.perf_date /= Void
			a_cutoff_date_not_void: a_cutoff_date /= Void
		local
			l_cutoff: RP_PERFORMANCE_DAILY_VALUES
		do
			if a_is_legacy and a_row.perf_date.is_equal (a_cutoff_date) then
					-- Lookup the new perf dailyvalues on the cut-off date
					-- so that the portfolio value from nperf will be used
					-- when computing daily returns
				if a_cache.has (a_row.perf_date) then
					l_cutoff := a_cache.item (a_row.perf_date)
				else
					l_cutoff := a_row
				end
			else
				l_cutoff := a_row
			end
			a_row.set_previous_portfolio_value (l_cutoff.portfolio_value)
			if a_is_legacy then
				if a_row.perf_date <= a_cutoff_date then
					a_cache.force (a_row, a_row.perf_date)
				end
			elseif a_row.perf_date >= a_cutoff_date then
				a_cache.force (a_row, a_row.perf_date)
			end
		end


	add_daily_return (a_daily_returns: RP_PERFORMANCE_DAILY_RETURNS; a_daily_values: RP_PERFORMANCE_DAILY_VALUES) is
			-- Add `a_daily_values' to `a_daily_returns'
		require
			non_void_args: a_daily_returns /= Void and a_daily_values /= Void
		do
			a_daily_returns.add_dailyvalues (a_daily_values)
		end

	zonal_product_portfolio_weights (a_zonal_product: RP_ZONAL_PRODUCT; a_currency: RP_CURRENCY; a_date_interval: ROSE_DATE_INTERVAL): DS_HASH_TABLE [TUPLE [value: RP_DECIMAL; date: ROSE_DATE], RP_PORTFOLIO] is
			-- Query portfolio weights for `a_zonal_product'
		do
			Result := a_zonal_product.portfolio_weights (agent portfolio_dailyvalues_at (?, ?, ?), a_currency, a_date_interval)
		end

	global_product_account_weights (a_global_product: RP_GLOBAL_PRODUCT; a_currency: RP_CURRENCY; a_date_interval: ROSE_DATE_INTERVAL; a_zone: RP_ZONE): DS_HASH_TABLE [TUPLE [value: RP_DECIMAL; date: ROSE_DATE], RP_INVESTMENT_ACCOUNT] is
			-- Query account weights for `a_global_product'
		do
			Result := a_global_product.investment_accounts_weights (agent investment_account_dailyvalues_at_with_calendar (?, ?, ?, a_zone.legacy_perf_calendar), a_currency, a_date_interval)
		end

	dailyvalues_sql_server: STRING
			-- SQL server name of the DailyValues database

	dailyvalues_sql_database: STRING
			-- SQL database name of the DailyValues database

	internal_daily_values_query: like daily_values_query
			-- Cached DailyValues query

	initialize_daiyvalues_sql_server_and_database is
			-- Initialize `dailyvalues_sql_database' and `dailyvalues_sql_server'
		local
			l_connection: SQL_DATA_SERVER
		do
			if dailyvalues_sql_server = Void or dailyvalues_sql_database = Void then
				l_connection := (create {DATABASE_ACCESS}).make_from_repository_fast_load ("nperf")
				dailyvalues_sql_server := l_connection.node
				dailyvalues_sql_database := l_connection.database
				if dailyvalues_sql_server = Void or dailyvalues_sql_database = Void then
					(create {EXCEPTIONS}).raise ("Invalid repository entry for `nperf'. NODE or DATABASE is NULL")
				end
			end
		ensure
			dailyvalues_sql_server_not_void: dailyvalues_sql_server /= Void
			dailyvalues_sql_database_not_void: dailyvalues_sql_database /= Void
		end

	dailyvalues_connection: SQL_DATA_SERVER is
			-- Connection to the DailyValues database
		do
			initialize_daiyvalues_sql_server_and_database
			create {SQLOLEDB_DATA_SERVER} Result.make_integrated_security_fast_load (dailyvalues_sql_server, dailyvalues_sql_database)
		ensure
			dailyvalues_connection_not_void: Result /= Void
		end

	daily_values_query: RP_PERFORMANCE_DAILY_VALUES_QUERY is
			-- Query interface for dailyvalues
		do
			if internal_daily_values_query = Void then
				create internal_daily_values_query.make (dailyvalues_connection)
			end
			Result := internal_daily_values_query
		ensure
			daily_values_query_not_void: Result /= Void
		end

	internal_portfolio_dailyreturns (a_portfolio: RP_PORTFOLIO; a_currency: RP_CURRENCY): !RP_DAILY_RETURNS is
			-- Dailyreturns for `a_portfolio' in `a_currency'
		require
			non_void_args: a_portfolio /= Void and a_currency /= Void
		local
			l_dv: like portfolio_dailyvalues
			l_adj: RP_PERFORMANCE_GROSS
			l_result: !RP_PERFORMANCE_DAILY_RETURNS
		do
			create l_adj
			l_dv := portfolio_dailyvalues (a_portfolio, a_currency)
			create l_result.make (a_portfolio.code, regular_performance_numerator, regular_performance_denominator, l_adj)
			l_dv.do_all (agent add_daily_return (l_result, ?))
			Result := l_result
		end

	internal_equityonly_dailyreturns (a_portfolio: RP_PORTFOLIO; a_currency: RP_CURRENCY): !RP_DAILY_RETURNS is
			-- EquityOnly Dailyreturns for `a_portfolio' in `a_currency'
		require
			non_void_args: a_portfolio /= Void and a_currency /= Void
		local
			l_dv: like portfolio_dailyvalues
			l_result: !RP_PERFORMANCE_DAILY_RETURNS
		do
			l_dv := portfolio_dailyvalues (a_portfolio, a_currency)
			create l_result.make (a_portfolio.code, equityonly_performance_numerator, equityonly_performance_denominator, equityonly_performance_numerator_adjustments)
			l_dv.do_all (agent add_daily_return (l_result, ?))
			Result := l_result
		end

	equityonly_performance_numerator: RP_PERFORMANCE_EQUITY_ONLY_DAILY_RETURN_NUMERATOR is
			-- Regular performance numerator
		once
			create Result
		end

	equityonly_performance_numerator_adjustments: RP_PERFORMANCE_EQUITY_ONLY_DAILY_RETURN_NUMERATOR_ADJUSTMENTS is
			-- Regular performance numerator adjustments
		once
			create Result
		end

	equityonly_performance_denominator: RP_PERFORMANCE_EQUITY_ONLY_DAILY_RETURN_DENOMINATOR is
			-- Regular performance denominator
		once
			create Result
		end

	regular_performance_numerator: RP_PERFORMANCE_DAILY_RETURN_NUMERATOR is
			-- Regular performance numerator
		once
			create Result
		end

	regular_performance_denominator: RP_PERFORMANCE_DAILY_RETURN_DENOMINATOR is
			-- Regular performance denominator
		once
			create Result
		end

	add_fees (a_fees: !DS_HASH_TABLE [RP_MONIES, ROSE_DATE]; a_dailyvalues: RP_PERFORMANCE_DAILY_VALUES) is
			-- Add fees to `a_dailyvalues'
		require
			non_void_arg: a_dailyvalues /= Void
		local
			l_fees: RP_MONIES
			l_fx_rate: RP_DECIMAL
		do
			a_fees.search (a_dailyvalues.perf_date)
			if a_fees.found then
				l_fees := a_fees.found_item
				l_fx_rate := fx.spot_rate (l_fees.currency, a_dailyvalues.currency, a_dailyvalues.perf_date)
				a_dailyvalues.set_fees (a_dailyvalues.currency.monies (l_fx_rate * l_fees.amount))
			end
		end

invariant

	portfolio_cached_not_void: portfolio_cached /= Void
	portfolio_dv_cache_not_void: portfolio_dv_cache /= Void
	account_dv_cache_not_void: account_dv_cache /= Void

end
