note

	description:

		"Representation of the legacy system"

	author: "Ivan Tarnavsky"
	last_editor: "$Author: sm $"
	date: "$Date: 2013/04/26 $"
	revision: "$Revision: #49 $"
	path: "$File: //rose/source/kernel/expert_system/rp_legacy_system.e $"

class RP_LEGACY_SYSTEM

inherit

	ANY

	DT_SHARED_SYSTEM_CLOCK
		export {NONE} all end

	RP_GEDB_EVENT_TO_CONTRACT_FACTORY
		export
			{ANY}
				enable_dividend_rountrip_calculation,
				disable_dividend_rountrip_calculation
			{RP_GEDB_EVENT} all
		redefine
			make
		end

	RP_GEDB_PREDICATES

	RP_PERFORMANCE_CONSTANTS_SHARED
		export {NONE} all end

	RP_GEDB_EVENT_TO_BACKDIV_MATCHER

create {RP_INVESTMENT_MANAGER}

	make

feature {NONE} -- Initialization

	make (a_manager: like manager) is
			-- Initialize `Current'
		do
			Precursor (a_manager)
			create factory.make (manager)
			create performance_system.make (manager)
		end

feature -- Access

	performance_system: RP_LEGACY_PERF
			-- Legacy performance system.

feature -- Status

	is_statistics_collection_enabled: BOOLEAN
			-- Are statistics enabled?

	is_ordering_by_settle_date: BOOLEAN
			-- Does settle_date plays role in event processing order?

	is_using_local_minima_backdiv_record_matcher: BOOLEAN
			-- Should local minima backdiv record matcher be used?

feature -- Setting

	enable_local_minima_backdiv_record_matcher is
			-- Enable local_minima_backdiv_record_matcher
		do
			is_using_local_minima_backdiv_record_matcher := True
		ensure
			definition: is_using_local_minima_backdiv_record_matcher
		end

	disable_local_minima_backdiv_record_matcher is
			-- Disable local_minima_backdiv_record_matcher
		do
			is_using_local_minima_backdiv_record_matcher := False
		ensure
			definition: not is_using_local_minima_backdiv_record_matcher
		end

	enable_ordering_by_settle_date is
			-- Enable ordering by settle_date
		do
			is_ordering_by_settle_date := True
		ensure
			definition: is_ordering_by_settle_date
		end

	disable_ordering_by_settle_date is
			-- Disable ordering by settle_date
		do
			is_ordering_by_settle_date := False
		ensure
			definition: not is_ordering_by_settle_date
		end

	enable_statistics_collection is
			-- Enable stats.
		do
			is_statistics_collection_enabled := True
			create event_type_stats_count.make (0)
			create event_type_stats_time.make (0)
		ensure
			definition: is_statistics_collection_enabled
			event_type_stats_count_not_void: event_type_stats_count /= Void
			event_type_stats_count_empty: event_type_stats_count.is_empty
			event_type_stats_time_not_void: event_type_stats_time /= Void
			event_type_stats_time_empty: event_type_stats_time.is_empty
		end

	disable_statistics_collection is
			-- Disable stats.
		do
			is_statistics_collection_enabled := False
		ensure
			definition: not is_statistics_collection_enabled
		end

feature -- Basic Operations

	update_for_investment_account (a_account: RP_INVESTMENT_ACCOUNT) is
			-- Generate contract from GEDB events for all portfolios in `a_account'.
		require
			non_void_account: a_account /= Void
		do
			a_account.portfolios.do_all (agent update_for_portfolio_interval (?, Void))
		end

	update_for_investment_account_with_filter (a_account: RP_INVESTMENT_ACCOUNT; a_filter: PREDICATE [ANY, TUPLE [RP_GEDB_RECORD]]) is
			-- Generate contract from GEDB events for all portfolios in `a_account'.
		require
			non_void_account: a_account /= Void
		do
			a_account.portfolios.do_all (agent update_for_portfolio_interval (?, a_filter))
		end

	update_for_portfolio (a_portfolio: RP_PORTFOLIO) is
			-- Fill `a_portfolio' with transactions
			-- This gives the latest view of the portfolio
		require
			portfolio_not_void: a_portfolio /= Void
		do
			update_for_portfolio_and_instrument_at_with_filter (a_portfolio, "%%", create {ROSE_DATE}.make_max_date, Void)
		end

	update_for_portfolio_at (a_portfolio: RP_PORTFOLIO; a_valid_time: ROSE_DATE) is
			-- Fill 'a_portfolio' with transactions
			-- This will be the view of transactions for 'a_valid_time' trade_date
			-- The transaction_time will be the most recent date, so 'a_portfolioevent)date
		require
			portfolio_not_void: a_portfolio /= Void
			date_not_void: a_valid_time /= Void
		do
			update_for_portfolio_and_instrument_at_with_filter (a_portfolio, "%%", a_valid_time, Void)
		end

	update_for_portfolio_at_with_filter (a_portfolio: RP_PORTFOLIO; a_valid_time: ROSE_DATE; a_filter: PREDICATE [ANY, TUPLE [RP_GEDB_RECORD]]) is
			-- Fill 'a_portfolio' with transactions
			-- This will be the view of transactions for 'a_valid_time' trade_date
			-- The transaction_time will be the most recent date, so 'a_portfolioevent)date
		require
			portfolio_not_void: a_portfolio /= Void
			date_not_void: a_valid_time /= Void
		do
			update_for_portfolio_and_instrument_at_with_filter (a_portfolio, "%%", a_valid_time, a_filter)
		end

	update_for_portfolio_on_day (a_portfolio: RP_PORTFOLIO; a_valid_time: ROSE_DATE) is
			-- Fill 'a_portfolio' with transactions
			-- This will be the view of transactions for 'a_valid_time' trade_date
			-- The transaction_time will be the most recent date, so 'a_portfolioevent)date
		require
			portfolio_not_void: a_portfolio /= Void
			date_not_void: a_valid_time /= Void
		do
			internal_gedb_record_processing (False, False, a_portfolio, "%%", a_valid_time, Void)
		end

	update_for_portfolio_on_day_with_filter (a_portfolio: RP_PORTFOLIO; a_valid_time: ROSE_DATE; a_filter: PREDICATE [ANY, TUPLE [RP_GEDB_RECORD]]) is
			-- Fill 'a_portfolio' with transactions
			-- This will be the view of transactions for 'a_valid_time' trade_date
			-- The transaction_time will be the most recent date, so 'a_portfolioevent)date
		require
			portfolio_not_void: a_portfolio /= Void
			date_not_void: a_valid_time /= Void
		do
			update_for_portfolio_and_instrument_on_day_with_filter (a_portfolio, "%%", a_valid_time, a_filter)
		end

	update_for_portfolio_and_instrument (a_portfolio: RP_PORTFOLIO; a_brimsym: STRING) is
			-- Update for `a_portfolio' and instrument identified by `a_brimsym'.
		require
			portfolio_not_void: a_portfolio /= Void
			brimsym_not_void: a_brimsym /= Void
		do
			update_for_portfolio_and_instrument_at_with_filter (a_portfolio, a_brimsym, create {ROSE_DATE}.make_max_date, Void)
		end

	update_for_portfolio_and_instrument_at_with_filter (a_portfolio: RP_PORTFOLIO; a_brimsym: STRING; a_valid_time: ROSE_DATE; a_filter: PREDICATE [ANY, TUPLE [RP_GEDB_RECORD]]) is
			-- Fill 'a_portfolio' with transactions
			-- This will be the view of transactions for 'a_valid_time' trade_date
			-- The transaction_time will be the most recent date, so 'a_portfolioevent)date
		require
			portfolio_not_void: a_portfolio /= Void
			brimsym_not_void: a_brimsym /= Void
			date_not_void: a_valid_time /= Void
		do
			internal_gedb_record_processing (False, False, a_portfolio, a_brimsym, a_valid_time, a_filter)
		end

	update_for_portfolio_and_instrument_on_day_with_filter (a_portfolio: RP_PORTFOLIO; a_brimsym: STRING; a_valid_time: ROSE_DATE; a_filter: PREDICATE [ANY, TUPLE [RP_GEDB_RECORD]]) is
			-- Fill 'a_portfolio' with transactions
			-- This will be the view of transactions for 'a_valid_time' trade_date
			-- The transaction_time will be the most recent date, so 'a_portfolioevent)date
		require
			portfolio_not_void: a_portfolio /= Void
			brimsym_not_void: a_brimsym /= Void
			date_not_void: a_valid_time /= Void
		do
			internal_gedb_record_processing (True, False, a_portfolio, a_brimsym, a_valid_time, a_filter)
		end

	update_for_portfolio_and_instrument_at_with_filter_adjusted (a_portfolio: RP_PORTFOLIO; a_brimsym: STRING; a_valid_time: ROSE_DATE; a_filter: PREDICATE [ANY, TUPLE [RP_GEDB_RECORD]]) is
			-- Fill 'a_portfolio' with transactions
			-- This will be the view of transactions for 'a_valid_time' trade_date
			-- The transaction_time will be the most recent date, so 'a_portfolioevent)date
		require
			portfolio_not_void: a_portfolio /= Void
			brimsym_not_void: a_brimsym /= Void
			date_not_void: a_valid_time /= Void
		do
			internal_gedb_record_processing (False, True, a_portfolio, a_brimsym, a_valid_time, a_filter)
		end

	update_for_portfolio_and_instrument_on_day_with_filter_adjusted (a_portfolio: RP_PORTFOLIO; a_brimsym: STRING; a_valid_time: ROSE_DATE; a_filter: PREDICATE [ANY, TUPLE [RP_GEDB_RECORD]]) is
			-- Fill 'a_portfolio' with transactions
			-- This will be the view of transactions for 'a_valid_time' trade_date
			-- The transaction_time will be the most recent date, so 'a_portfolioevent)date
		require
			portfolio_not_void: a_portfolio /= Void
			brimsym_not_void: a_brimsym /= Void
			date_not_void: a_valid_time /= Void
		do
			internal_gedb_record_processing (True, True, a_portfolio, a_brimsym, a_valid_time, a_filter)
		end

	clean_up_gedb_record (a_gedb_record: RP_GEDB_RECORD) is
			-- Clean up data in the input record (like trailing blank characters
		require
			gedb_record_not_void: a_gedb_record /= Void
		do
			a_gedb_record.prune_trailing_blanks
			a_gedb_record.fill_void_decimals
		end

	process_new_gedb_record (a_filter: PREDICATE [ANY, TUPLE [RP_GEDB_RECORD]]; a_gedb_record: RP_GEDB_RECORD) is
			-- Generate portfolio contract from GEDB record
		local
			l_gedb_event: RP_GEDB_EVENT
			l_contracts: ARRAY [RP_PORTFOLIO_CONTRACT]
			l_start: DT_TIME
			l_millis: INTEGER
		do
			if a_filter = Void or else a_filter.item ([a_gedb_record]) then
				l_gedb_event := factory.gedb_event_from_record (a_gedb_record)
				if is_statistics_collection_enabled then
					if event_type_stats_count.has (a_gedb_record.event_type) then
						event_type_stats_count.force (event_type_stats_count.item (a_gedb_record.event_type) + 1, a_gedb_record.event_type)
					else
						event_type_stats_count.force (1, a_gedb_record.event_type)
						event_type_stats_time.force (0, a_gedb_record.event_type)
					end
				end
				l_start := system_clock.time_now
				l_contracts := l_gedb_event.portfolio_contracts
				l_millis := system_clock.time_now.duration (l_start).millisecond_count
				debug ("rp_legacy_system")
					print (l_gedb_event.identifier + ": " + l_millis.out + " ms%N")
				end
				if is_statistics_collection_enabled then
					event_type_stats_time.force (event_type_stats_time.item (a_gedb_record.event_type) + l_millis, a_gedb_record.event_type)
				end
				if l_contracts = Void or else l_contracts.is_empty then
					manager.log_minor ("NO contracts generated for " + l_gedb_event.identifier)
				else
					l_contracts.do_all (agent manager.post (?))
				end
			end
		end

feature -- Query

	match (a_event: RP_GEDB_EVENT): RP_BACKDIV_RECORD is
			-- <Precursor>
		do
			if backdiv_matchers /= Void then
				backdiv_matchers.search (a_event.portfolio)
				if backdiv_matchers.found then
					Result := backdiv_matchers.found_item.match (a_event)
				end
			end
		end

	adjusted_cash_dividend_settle_date (a_settle_date: ROSE_DATE; a_portfolio: RP_PORTFOLIO;
		a_country: RP_COUNTRY; a_currency: RP_CURRENCY; a_is_long: BOOLEAN): ROSE_DATE is
			-- Settlement date from `a_settle_date' for cash dividend posted in `a_portfolio' for `a_country' and `a_currency'
		require
			a_settle_date_not_void: a_settle_date /= Void
			a_portfolio_not_void: a_portfolio /= Void
			a_country_not_void: a_country /= Void
			a_currency_not_void: a_currency /= Void
		local
			l_history: ROSE_INTERVAL_HISTORY [EVENT_Z_A_DIVIDEND_EXTENSION_PARM, ROSE_CLOSED_INTERVAL [EVENT_Z_A_DIVIDEND_EXTENSION_PARM]]
			l_interval: ROSE_CLOSED_INTERVAL [EVENT_Z_A_DIVIDEND_EXTENSION_PARM]
			l_key: MULTI_KEY
		do
			if a_portfolio.zone.is_orient and then
				a_is_long and then
				not (a_portfolio.code ~ "ASIEACTO" and a_currency.iso ~ "THB")
					-- E2: code review 3000; 2012/04/16
					-- Logic from VMS o_event_src:POSITIONS.FOR
					-- For ASIEACTO, THB is not a restricted currency
					-- as we get the money as planned (ie on settlement date)
			then
				create l_key.make (<<a_country.iso_code, a_currency.iso>>)
				o_zone_settlement_date_extension.search (l_key)
				if o_zone_settlement_date_extension.found then
					l_history := o_zone_settlement_date_extension.found_item
					l_interval := l_history.witness_interval (a_settle_date)
					if l_interval /= Void then
						Result := a_portfolio.zone.legacy_perf_calendar.ith_work_day_after (a_settle_date, l_interval.item.number_of_days)
					end
				end
			end
			if Result = Void then
				Result := a_settle_date
			end
		ensure
			cash_dividend_settle_date_not_void: Result /= Void
		end

	stock_dividend_settle_date (a_event_date, a_settle_date: ROSE_DATE; a_portfolio: RP_PORTFOLIO; a_stock: RP_STOCK): ROSE_DATE is
			-- Settlement date for `a_stock' dividend posted to `a_portfolio' on `a_event_date' with paydate of `a_settle_date'
		require
			a_event_date_not_void: a_event_date /= Void
			a_settle_date_not_void: a_settle_date /= Void
			a_portfolio_not_void: a_portfolio /= Void
			a_stock_not_void: a_stock /= Void
		local
			l_key: MULTI_KEY
		do
			if a_portfolio.zone.is_orient then
				--            IF(ZONE .EQ. 'O' .AND. LT .EQ. JSTKDIV)THEN
				--              HASHKEY(1:8)=CLICODE(IP)
				--              HASHKEY(9:26)=EVENT(IEV).ROWSYM
				--              WRITE(UNIT=HASHKEY(27:28),FMT='(I2)') JSTKDIV
				--              WRITE(UNIT=HASHKEY(29:36),FMT='(I8)') EVENT(IEV).PERF_DATE
				--              ALLHASHKEY = 'ALL     '//HASHKEY(9:36)
				--              IF(HASHLOOKUP(HASHID,HASHKEY,BANKDAY) .NE. 0 )THEN
				--                EVENT(IEV).SETTLE_DATE=BANKDAY
				--              ELSE IF(HASHLOOKUP(HASHID,ALLHASHKEY,BANKDAY) .NE. 0 )THEN
				--                EVENT(IEV).SETTLE_DATE=BANKDAY
				--              ELSE IF (EVENT(IEV).SETTLE_DATE.GT.20040824 .AND. EVENT(IEV).SETTLE_DATE.LT.99999999) THEN
				--                ! only affect records after paydate of 20040824, i.e. don't change past records
				--                ! delay settlement 30 + payday
				--                CALL NEXTMARKETDAY_ZONE(EVENT(IEV).SETTLE_DATE,30,ZONE,EVENT(IEV).SETTLE_DATE)
				--              ELSE
				--                ! delay settlement beyond 90 perday
				--                CALL NEXTMARKETDAY_ZONE(EVENT(IEV).PERF_DATE,90,ZONE,EVENT(IEV).SETTLE_DATE)
				--              END IF
				--            END IF
				create l_key.make (<<a_portfolio.code, a_stock.brimsym, a_event_date>>)
				o_zone_stock_dividend_settlement_date.search (l_key)
				if o_zone_stock_dividend_settlement_date.found then
					Result := o_zone_stock_dividend_settlement_date.found_item
				else
					create l_key.make (<<constants.ozone_stock_dividend_portfolio_wildcard_name, a_stock.brimsym, a_event_date>>)
					o_zone_stock_dividend_settlement_date.search (l_key)
					if o_zone_stock_dividend_settlement_date.found then
						Result := o_zone_stock_dividend_settlement_date.found_item
					elseif a_settle_date > constants.ozone_stock_dividend_first_date_for_30_days_settlement and a_settle_date < a_settle_date.max_date then
						Result := a_portfolio.zone.legacy_perf_calendar.ith_work_day_after (a_settle_date, 30)
					else
						Result := a_portfolio.zone.legacy_perf_calendar.ith_work_day_after (a_event_date, 90)
					end
				end
			end
			if Result = Void then
				Result := a_settle_date
			end
		ensure
			stock_dividend_settle_date_not_void: Result /= Void
		end

feature {NONE} -- Implementation

	factory: RP_GEDB_EVENT_FACTORY
			-- GEDB event factory

	process_gedb_records (a_gedb_records: ROSE_LINEAR_ARRAY [RP_GEDB_RECORD]; a_filter: PREDICATE [ANY, TUPLE [RP_GEDB_RECORD]]) is
			-- Process gedb records
		require
			a_gedb_records_not_void: a_gedb_records /= Void
		do
			a_gedb_records.sort_by (agent is_event_order (?, ?))
			a_gedb_records.do_all (agent process_new_gedb_record (a_filter, ?))
		end

	is_event_order (a, b: RP_GEDB_RECORD): BOOLEAN is
			-- is a < b?
		require
			a_not_void: a /= Void
			b_not_void: b /= Void
		do
			if a.event_date < b.event_date then
				Result := True
			elseif a.event_date ~ b.event_date then
				if event_type_order.item (a.event_type) < event_type_order.item (b.event_type) then
					Result := True
				elseif event_type_order.item (a.event_type) = event_type_order.item (b.event_type) then
					if is_ordering_by_settle_date then
						if a.settle_date < b.settle_date then
							Result := True
						elseif a.settle_date ~ b.settle_date then
							Result := a.icc < b.icc
						end
					else
						Result := a.icc < b.icc
					end
				end
			end
		end

	event_type_order: ARRAY [INTEGER] is
			-- Daily event type order
		once
				-- event type number --------------------------------------------------------------------------------------------------------------------------------------------------------------
				--      1  2  3  4  5  6  7  8  9 10 11 12  13  14  15  16  17 18  19  20 21 22 23 24 25 26 27 28 29 30 31  32 33  34 35 36 37  38  39  40 41 42 43  44 45 46 47  48  49  50  51 52
			Result := <<6, 7, 6, 0, 4, 2, 6, 7, 3, 1, 5, 5, 10, 10, 10, 10, 10, 2, 10, 10, 8, 9, 7, 5, 7, 7, 7, 7, 3, 1, 3, 10, 3, 10, 4, 7, 7, 10, 10, 10, 5, 5, 9, 10, 5, 5, 5, 10, 10, 10, 10, 5>>
		ensure
			result_not_void: Result /= Void
			fifty_two_events: Result.count = 52
		end

	reset_stats is
			-- Print and reset statistics
		do
			if is_statistics_collection_enabled then
				if event_type_stats_count /= Void then
					from
						event_type_stats_count.start
					until
						event_type_stats_count.off
					loop
						manager.log_debug (event_type_stats_count.key_for_iteration.out + " " + event_type_stats_count.item_for_iteration.out + " " + event_type_stats_time.item (event_type_stats_count.key_for_iteration).out)
						event_type_stats_count.forth
					end
				end
				create event_type_stats_count.make (0)
				create event_type_stats_time.make (0)
			end
		ensure
			event_type_stats_count_not_void: is_statistics_collection_enabled implies event_type_stats_count /= Void
			event_type_stats_count_empty: is_statistics_collection_enabled implies event_type_stats_count.is_empty
			event_type_stats_time_not_void: is_statistics_collection_enabled implies event_type_stats_time /= Void
			event_type_stats_time_empty: is_statistics_collection_enabled implies event_type_stats_time.is_empty
		end

	event_type_stats_count: HASH_TABLE [INTEGER, INTEGER]
			-- Count by event type

	event_type_stats_time: HASH_TABLE [INTEGER, INTEGER]
			-- Elapsed time by event type

	update_for_portfolio_interval (a_portfolio_interval: RP_PORTFOLIO_HISTORY_INTERVAL; a_filter: PREDICATE [ANY, TUPLE [RP_GEDB_RECORD]]) is
			-- Generate contracts and flows for portfolio in `a_portfolio_interval'
		require
			non_void_interval: a_portfolio_interval /= Void
			filter_may_be_void: True
		do
			update_for_portfolio_at_with_filter (a_portfolio_interval.portfolio, create {ROSE_DATE}.make_max_date, a_filter)
		end

	backdiv_matchers: DS_HASH_TABLE [RP_GEDB_EVENT_TO_BACKDIV_MATCHER, RP_PORTFOLIO]
			-- BACKDIV record matcher per portfolio

	internal_gedb_record_processing (a_on_event_date_only, a_apply_adjusments: BOOLEAN; a_portfolio: RP_PORTFOLIO; a_brimsym: STRING; a_valid_time: ROSE_DATE; a_filter: PREDICATE [ANY, TUPLE [RP_GEDB_RECORD]]) is
			-- Fill 'a_portfolio' with transactions
			-- This will be the view of transactions for 'a_valid_time' trade_date
			-- The transaction_time will be the most recent date, so 'a_portfolioevent)date
		require
			portfolio_not_void: a_portfolio /= Void
			brimsym_not_void: a_brimsym /= Void
			date_not_void: a_valid_time /= Void
		local
			l_reader: COLLECTING_PREDICATE [RP_GEDB_RECORD]
			l_hash: DS_HASH_TABLE [RP_GEDB_RECORD, INTEGER]
			l_sw: ROSE_STOPWATCH
			l_msg: STRING
			l_backdiv: RP_GEDB_EVENT_TO_BACKDIV_MATCHER
		do
			debug ("legacy_system")
				l_msg := "legacy_system.internal_gedb_record_processing"
				create l_sw.make (l_msg)
				manager.log_debug ("starting " + l_msg + " " + a_portfolio.code + " " + a_brimsym + " " + a_valid_time.ccyymmdd.out)
				l_sw.start
			end
				-- Read gedb records from database
			if a_brimsym.is_equal ("%%") then
				if a_on_event_date_only then
					l_reader := internal_gedb_record_processing_reader_exact
				else
					l_reader := internal_gedb_record_processing_reader_latest
				end
				l_reader.retrieve (Void, <<a_portfolio.code, a_valid_time>>)
			else
				if a_on_event_date_only then
					l_reader := internal_gedb_record_processing_reader_filter_exact
				else
					l_reader := internal_gedb_record_processing_reader_filter_latest
				end
				l_reader.retrieve (Void, <<a_portfolio.code, a_brimsym, a_valid_time>>)
			end
				-- Clean up records
			l_reader.array.do_all (agent clean_up_gedb_record)
				-- Hash by ICC
			create l_hash.make (l_reader.array.count)
			l_reader.array.do_all (agent hash_record_by_icc (l_hash, ?))
				-- Force adjustment if necessary
			if a_apply_adjusments then
				adjust_records (l_hash, a_portfolio)
			end

				-- BACKDIV record matching
			if backdiv_matchers = Void then
				create backdiv_matchers.make_equal (0)
			end
			l_backdiv := new_backdiv_record_matcher (l_hash)
			backdiv_matchers.force (l_backdiv, a_portfolio)
				-- Process records
			reset_stats
			process_gedb_records (create {ROSE_LINEAR_ARRAY [RP_GEDB_RECORD]}.make_from_array (l_hash.to_array), a_filter)
			reset_stats
			debug ("legacy_system")
				l_sw.stop
				manager.log_debug ("Completed " + l_msg + " in " + l_sw.total_wallclock_milliseconds.out + " ms")
			end
		end

	hash_record_by_icc (a_hash: DS_HASH_TABLE [RP_GEDB_RECORD, INTEGER]; a_record: RP_GEDB_RECORD) is
			-- Add `a_record' to `a_hash' using `a_record.icc' as key
		require
			non_void_args: a_hash /= Void and a_record /= Void
		do
			a_hash.force (a_record, a_record.icc)
		ensure
			record_hashed: a_hash.has (a_record.icc)
		end

	adjust_records_reader: COLLECTING_PREDICATE [RP_GEDB_RECORD] is
			-- Reader for 'adjust_records'
		once
			create Result.make (servers.gedb_adjustments_fast_load, "select portfolio, icc, cash_effect_adj as cash_effect, performance_effect_adj as performance_effect, units_adj as units, units_base_adj as units_base from gedb_adjustments where portfolio=?")
		end

	adjust_records (a_hash: DS_HASH_TABLE [RP_GEDB_RECORD, INTEGER]; a_portfolio: RP_PORTFOLIO) is
			-- Adjust records for `a_portfolio' in `a_hash'
		require
			non_void_args: a_hash /= Void and a_portfolio /= Void
		do
			adjust_records_reader.retrieve (Void, <<a_portfolio.code>>)
			adjust_records_reader.array.do_all (agent apply_adjustment (a_hash, ?))
		end

	apply_adjustment (a_hash: DS_HASH_TABLE [RP_GEDB_RECORD, INTEGER]; a_record: RP_GEDB_RECORD) is
			-- Apply adjustments from `a_record' to data in `a_hash'
		require
			non_void_args: a_hash /= Void and a_record /= Void
		do
			a_hash.search (a_record.icc)
			if a_hash.found then
				a_hash.found_item.set_cash_effect (a_hash.found_item.cash_effect + a_record.cash_effect)
				a_hash.found_item.set_performance_effect (a_hash.found_item.performance_effect + a_record.performance_effect)
				a_hash.found_item.set_units (a_hash.found_item.units + a_record.units)
				a_hash.found_item.set_units_base (a_hash.found_item.units_base + a_record.units_base)
			end
		end

	new_backdiv_record_matcher (a_hash: DS_HASH_TABLE [RP_GEDB_RECORD, INTEGER]): RP_GEDB_EVENT_TO_BACKDIV_MATCHER is
			-- Create instance of backdiv record matcher
		require
			a_hash_not_void: a_hash /= Void
			a_hash_not_empty: not a_hash.is_empty
			no_void_item_in_a_hash: not a_hash.has_void_item
		do
			if is_using_local_minima_backdiv_record_matcher then
				Result := create {RP_BACKDIV_MATCHER_WITH_LOCAL_MINIMUM_SEARCH}.make (a_hash)
			else
				Result := create {RP_BACKDIV_MATCHER_WITH_FIFO_ASSIGNMENT}.make
			end
		ensure
			matcher_not_void: Result /= Void
		end

	o_zone_settlement_date_extension: DS_HASH_TABLE [ROSE_INTERVAL_HISTORY [EVENT_Z_A_DIVIDEND_EXTENSION_PARM, ROSE_CLOSED_INTERVAL [EVENT_Z_A_DIVIDEND_EXTENSION_PARM]], MULTI_KEY] is
			-- O zone cash dividend settlement extensions
		local
			l_parm: EVENT_O_A_DIVIDEND_EXTENSION_PARM
		once
			create l_parm.make
			Result := l_parm.entries
		ensure
			o_zone_settlement_date_extension_not_void: Result /= Void
		end

	o_zone_stock_dividend_settlement_date: DS_HASH_TABLE [ROSE_DATE, MULTI_KEY] is
			-- O zone stock dividend settlement dates
		local
			l_parm: EVENT_O_A_CORPORATE_ACTIONS_SETTLEMENT_PARM
		once
			create l_parm.make
			Result := l_parm.entries
		ensure
			o_zone_stock_dividend_settlement_date_not_void: Result /= Void
		end

invariant

	internal_gedb_record_processing_reader_exact_not_void: internal_gedb_record_processing_reader_exact /= Void
	internal_gedb_record_processing_reader_latest_not_void: internal_gedb_record_processing_reader_latest /= Void
	internal_gedb_record_processing_reader_filter_exact_not_void: internal_gedb_record_processing_reader_filter_exact /= Void
	internal_gedb_record_processing_reader_filter_latest_not_void: internal_gedb_record_processing_reader_filter_latest /= Void
	event_type_stats_count_not_void: is_statistics_collection_enabled implies event_type_stats_count /= Void
	event_type_stats_time_not_void: is_statistics_collection_enabled implies event_type_stats_time /= Void

end
