indexing

	description:

		"Market data record interface to RMDAT price record"

	date: "$Date: 2010/04/30 $"
	revision: "$Revision: #17 $"

class RP_LISTING_MARKET_DATA

inherit

	RP_MARKET_DATA_RECORD

	ROSE_CONSTANTS
		export {NONE} all end

create

	make_with_legacy_price_record

feature {NONE} -- Constructor

	make_with_legacy_price_record (a_ref_system: RP_REFERENCE_SYSTEM; a_instrument: RP_INSTRUMENT; a_legacy_price_record: PRICE_RECORD) is
			-- Initialize `Current'.
		require
			non_void_arguments: a_ref_system /= Void and a_legacy_price_record /= Void and a_instrument /= Void
		local
			l_currency: RP_CURRENCY
			l_precision: INTEGER
			l_value: RP_DECIMAL
		do
			if a_instrument.is_future then
				l_precision := 2
			else
				l_precision := 4
			end
			if a_legacy_price_record.currency /= Void then
				l_currency := a_ref_system.currency_from_iso_code (a_legacy_price_record.currency.iso)
			else
				l_currency := a_ref_system.currency_from_iso_code (a_instrument.brimsym.substring (1, 2))
			end
			date := a_legacy_price_record.date
			adjustment_value := a_legacy_price_record.adjustment
			volume_value := a_legacy_price_record.volume
			if l_currency /= Void then
				if a_legacy_price_record.has_ask and then is_defined_double (a_legacy_price_record.unadjusted_ask) then
					create l_value.make_from_double (a_legacy_price_record.unadjusted_ask)
					l_value := l_value.rounded (l_precision)
					ask_price := a_instrument.price_in_currency (l_value, l_currency, date)
				end
				if a_legacy_price_record.has_bid and then is_defined_double (a_legacy_price_record.unadjusted_bid) then
					create l_value.make_from_double (a_legacy_price_record.unadjusted_bid)
					l_value := l_value.rounded (l_precision)
					bid_price := a_instrument.price_in_currency (l_value, l_currency, date)
				end
				if a_legacy_price_record.unadjusted_close <= 0 then
					if a_legacy_price_record.has_bid and then is_defined_double (a_legacy_price_record.unadjusted_bid) then
						create l_value.make_from_double (a_legacy_price_record.unadjusted_bid)
						l_value := l_value.rounded (l_precision)
						close_price := a_instrument.price_in_currency (l_value, l_currency, date)
					end
				elseif a_legacy_price_record.has_close and then is_defined_double (a_legacy_price_record.unadjusted_close) then
					create l_value.make_from_double (a_legacy_price_record.unadjusted_close)
					l_value := l_value.rounded (l_precision)
					close_price := a_instrument.price_in_currency (l_value, l_currency, date)
				end
				if a_legacy_price_record.has_low and then is_defined_double (a_legacy_price_record.unadjusted_low) then
					create l_value.make_from_double (a_legacy_price_record.unadjusted_low)
					l_value := l_value.rounded (l_precision)
					low_price := a_instrument.price_in_currency (l_value, l_currency, date)
				end
			end
		end

feature -- Access

	date: ROSE_DATE
			-- Record date.

	has (a_field: RP_MARKET_DATA_FIELD): BOOLEAN is
			-- Does record have `a_field'?
		do
			if a_field.is_adjustment then
				Result := adjustment_value /= Void and then adjustment_value.is_defined and then adjustment_value > 0
			elseif a_field.is_unadjusted_ask then
				Result := ask_price /= Void
			elseif a_field.is_unadjusted_bid then
				Result := bid_price /= Void
			elseif a_field.is_unadjusted_close then
				Result := close_price /= Void
			elseif a_field.is_volume then
				Result := volume_value /= Void and then volume.is_defined
			elseif a_field.is_unadjusted_low then
				Result := low_price /= Void
			end
		end

	decimal_field (a_field: RP_MARKET_DATA_FIELD): RP_DECIMAL is
			-- Decimal value of `a_field'.
		do
			if a_field.is_adjustment then
				Result := adjustment_value
			elseif a_field.is_volume then
				Result := volume_value
			end
		end

	monies_field (a_field: RP_MARKET_DATA_FIELD): RP_MONIES is
			-- Value of `a_field' as monies.
		do
			Result := Void
				-- No money fields
		end

	price_field (a_field: RP_MARKET_DATA_FIELD): RP_PRICE is
			-- Price value of `a_field'.
		do
			if a_field.is_unadjusted_ask then
				Result := ask_price
			elseif a_field.is_unadjusted_bid then
				Result := bid_price
			elseif a_field.is_unadjusted_close then
				Result := close_price
			elseif a_field.is_unadjusted_low then
				Result := low_price
			end
		end

feature {NONE} -- Implementation

	adjustment_value: RP_DECIMAL
			-- Adjustment factor

	volume_value: RP_DECIMAL
			-- Volume

	close_price: like price_template
			-- Close

	bid_price: like price_template
			-- Bid

	ask_price: like price_template
			-- Ask

	low_price: like price_template
			-- Low

feature {NONE} -- Anchor

	price_template: RP_PRICE
		-- A template for prices
end
