indexing

	description:

		"Market Data Record interface"

	date: "$Date: 2010/04/30 $"
	revision: "$Revision: #12 $"
	archive: "$File: //rose/source/kernel/expert_system/aflows/rp_market_data_record.e $"

deferred class RP_MARKET_DATA_RECORD

inherit

	RP_MARKET_DATA_FIELD_AWARE

feature -- Query

	date: ROSE_DATE is
			-- Record date.
		deferred
		end

	has (a_field: RP_MARKET_DATA_FIELD): BOOLEAN is
			-- Is field identified by `a_field' present in `Current'?
		require
			non_void_field: a_field /= Void
		deferred
		end

	decimal_field (a_field: RP_MARKET_DATA_FIELD): RP_DECIMAL is
			-- Decimal value of `a_field'.
		require
			non_void_field: a_field /= Void
		deferred
		end

	monies_field (a_field: RP_MARKET_DATA_FIELD): RP_MONIES is
			-- Value of `a_field' as monies.
		require
			non_void_field: a_field /= Void
		deferred
		end

	price_field (a_field: RP_MARKET_DATA_FIELD): RP_PRICE is
			-- Price value of `a_field'.
		require
			non_void_field: a_field /= Void
		deferred
		end

feature -- Convenience

	adjustment_factor: RP_DECIMAL is
			-- adjustment factor
		do
			Result := decimal_field (adjustment_type)
		end

	unadjusted_close: RP_PRICE is
			-- unadjusted close price
		do
			Result := price_field (unadjusted_close_type)
		end

	unadjusted_bid: RP_PRICE is
			-- unadjusted bid price
		do
			Result := price_field (unadjusted_bid_type)
		end

	unadjusted_ask: RP_PRICE is
			-- unadjusted ask price
		do
			Result := price_field (unadjusted_ask_type)
		end

	volume: RP_DECIMAL is
			-- Volume.
		do
			Result := decimal_field (volume_type)
		end

	has_close: BOOLEAN is
			-- has unadjusted close price
		do
			Result := has (unadjusted_close_type)
		end

	has_bid: BOOLEAN is
			-- has bid price
		do
			Result := has (unadjusted_bid_type)
		end

	has_ask: BOOLEAN is
			-- has ask price
		do
			Result := has (unadjusted_ask_type)
		end

	has_adjustment_factor: BOOLEAN is
			-- adjustment factor exists
		do
			Result := has (adjustment_type)
		end

	has_volume: BOOLEAN is
			-- volume is available
		do
			Result := has (volume_type)
		end

end
