note

	description:

		"RP_MARKET_DATA_SOURCE with default implementation of last_record_with_valid_field"

	author: "Ivan Tarnavsky"
	last_editor: "$Author: it $"
	date: "$Date: 2011/03/30 $"
	revision: "$Revision: #5 $"
	path: "$File: //rose/source/kernel/market_data/rp_market_data_source_last_record_search.e $"

deferred class RP_MARKET_DATA_SOURCE_LAST_RECORD_SEARCH [G -> RP_INSTRUMENT]

inherit

	RP_MARKET_DATA_SOURCE [G]

feature -- Access

	default_start_date: ROSE_DATE is
			-- Default start date
		once
			create Result.make_ccyymmdd (19850101)
		ensure
			result_not_void: Result /= Void
			is_dertermined: not Result.is_undetermined
		end

	start_date: ROSE_DATE
			-- Start date to use if date range in effect

	end_date: ROSE_DATE
			-- End date to use if date range in effect

	max_lookback_days: INTEGER
			-- Maximum days to lookback for a price from given date

feature -- Query

	last_record_with_valid_field (a_instrument: G; a_date: ROSE_DATE; a_field: RP_MARKET_DATA_FIELD): RP_MARKET_DATA_RECORD is
			-- Last record as of `a_date' for `a_instrument' with valid `a_field'
		local
			l_search_for: like search_for
		do
			l_search_for := search_for (a_field)
			if l_search_for /= Void then
				Result := lookback (a_instrument, a_date, l_search_for)
			end
		end

feature {NONE} -- Implementation

	search_for (a_field: RP_MARKET_DATA_FIELD): PREDICATE [ANY, TUPLE [RP_MARKET_DATA_RECORD]] is
			-- Record search agent based on `a_field'
		require
			non_void_field: a_field /= Void
		do
			if a_field.is_adjustment then
				Result := search_for_adjustment_factor
			elseif a_field.is_civ_fund_class_capital_balance then
				Result := search_for_civ_fund_class_capital_balance
			elseif a_field.is_civ_fund_class_expense_balance then
				Result := search_for_civ_fund_class_expense_balance
			elseif a_field.is_civ_fund_class_nav then
				Result := search_for_civ_fund_class_nav
			elseif a_field.is_civ_fund_class_revenue_balance then
				Result := search_for_civ_fund_class_revenue_balance
			elseif a_field.is_civ_fund_class_shares then
				Result := search_for_civ_fund_class_shares
			elseif a_field.is_unadjusted_ask then
				Result := search_for_ask
			elseif a_field.is_unadjusted_bid then
				Result := search_for_bid
			elseif a_field.is_unadjusted_close then
				Result := search_for_close
			elseif a_field.is_unadjusted_high then
				Result := search_for_unadjusted_high
			elseif a_field.is_unadjusted_low then
				Result := search_for_unadjusted_low
			elseif a_field.is_volume then
				Result := search_for_volume
			elseif a_field.is_civ_fund_shares_activity then
				Result := search_for_civ_fund_shares_activity
			end
		end

	search_for_civ_fund_shares_activity: PREDICATE [ANY, TUPLE [RP_MARKET_DATA_RECORD]] is
			-- Search for civ_fund_shares_activity  predicate
		once
			Result := agent has_civ_fund_shares_activity (?)
		end

	has_civ_fund_shares_activity (a_record: RP_MARKET_DATA_RECORD): BOOLEAN is
			-- Does `a_record' have civ_fund_shares_activity?
		require
			non_void_arecord: a_record /= Void
		do
			Result := a_record.has (a_record.civ_fund_shares_activity)
		end

	search_for_civ_fund_class_capital_balance: PREDICATE [ANY, TUPLE [RP_MARKET_DATA_RECORD]] is
			-- Search for civ_fund_class_capital_balance  predicate
		once
			Result := agent has_civ_fund_class_capital_balance (?)
		end

	has_civ_fund_class_capital_balance (a_record: RP_MARKET_DATA_RECORD): BOOLEAN is
			-- Does `a_record' have civ_fund_class_capital_balance?
		require
			non_void_arecord: a_record /= Void
		do
			Result := a_record.has (a_record.civ_fund_class_capital_balance_type)
		end

	search_for_civ_fund_class_expense_balance: PREDICATE [ANY, TUPLE [RP_MARKET_DATA_RECORD]] is
			-- Search for civ_fund_class_expense_balance  predicate
		once
			Result := agent has_civ_fund_class_expense_balance (?)
		end

	has_civ_fund_class_expense_balance (a_record: RP_MARKET_DATA_RECORD): BOOLEAN is
			-- Does `a_record' have civ_fund_class_expense_balance?
		require
			non_void_arecord: a_record /= Void
		do
			Result := a_record.has (a_record.civ_fund_class_expense_balance_type)
		end

	search_for_civ_fund_class_nav: PREDICATE [ANY, TUPLE [RP_MARKET_DATA_RECORD]] is
			-- Search for civ_fund_class_nav predicate
		once
			Result := agent has_civ_fund_class_nav (?)
		end

	has_civ_fund_class_nav (a_record: RP_MARKET_DATA_RECORD): BOOLEAN is
			-- Does `a_record' have civ_fund_class_nav?
		require
			non_void_arecord: a_record /= Void
		do
			Result := a_record.has (a_record.civ_fund_class_nav_type)
		end

	search_for_civ_fund_class_revenue_balance: PREDICATE [ANY, TUPLE [RP_MARKET_DATA_RECORD]] is
			-- Search for civ_fund_class_revenue_balance predicate
		once
			Result := agent has_civ_fund_class_revenue_balance (?)
		end

	has_civ_fund_class_revenue_balance (a_record: RP_MARKET_DATA_RECORD): BOOLEAN is
			-- Does `a_record' have civ_fund_class_revenue_balance?
		require
			non_void_arecord: a_record /= Void
		do
			Result := a_record.has (a_record.civ_fund_class_revenue_balance_type)
		end

	search_for_civ_fund_class_shares: PREDICATE [ANY, TUPLE [RP_MARKET_DATA_RECORD]] is
			-- Search for civ_fund_class_shares predicate
		once
			Result := agent has_civ_fund_class_shares (?)
		end

	has_civ_fund_class_shares (a_record: RP_MARKET_DATA_RECORD): BOOLEAN is
			-- Does `a_record' have civ_fund_class_shares?
		require
			non_void_arecord: a_record /= Void
		do
			Result := a_record.has (a_record.civ_fund_class_shares_type)
		end

	search_for_unadjusted_high: PREDICATE [ANY, TUPLE [RP_MARKET_DATA_RECORD]] is
			-- Search for unadjusted_high predicate
		once
			Result := agent has_unadjusted_high (?)
		end

	has_unadjusted_high (a_record: RP_MARKET_DATA_RECORD): BOOLEAN is
			-- Does `a_record' have unadjusted_high?
		require
			non_void_arecord: a_record /= Void
		do
			Result := a_record.has (a_record.unadjusted_high_type)
		end

	search_for_unadjusted_low: PREDICATE [ANY, TUPLE [RP_MARKET_DATA_RECORD]] is
			-- Search for unadjusted_low predicate
		once
			Result := agent has_unadjusted_low (?)
		end

	has_unadjusted_low (a_record: RP_MARKET_DATA_RECORD): BOOLEAN is
			-- Does `a_record' have unadjusted_low?
		require
			non_void_arecord: a_record /= Void
		do
			Result := a_record.has (a_record.unadjusted_low_type)
		end

	search_for_adjustment_factor: PREDICATE [ANY, TUPLE [RP_MARKET_DATA_RECORD]] is
			-- Search for adjustment_factor predicate
		once
			Result := agent has_adjustment_factor (?)
		end

	has_adjustment_factor (a_record: RP_MARKET_DATA_RECORD): BOOLEAN is
			-- Does `a_record' have adjustment_factor?
		do
			Result := a_record.has_adjustment_factor
		end

	search_for_bid: PREDICATE [ANY, TUPLE [RP_MARKET_DATA_RECORD]] is
			-- Search for bid predicate
		once
			Result := agent has_bid (?)
		end

	has_bid (a_record: RP_MARKET_DATA_RECORD): BOOLEAN is
			-- Does `a_record' have bid?
		do
			Result := a_record.has_bid
		end

	search_for_ask: PREDICATE [ANY, TUPLE [RP_MARKET_DATA_RECORD]] is
			-- Search for ask predicate
		once
			Result := agent has_ask (?)
		end

	has_ask (a_record: RP_MARKET_DATA_RECORD): BOOLEAN is
			-- Does `a_record' have ask?
		do
			Result := a_record.has_ask
		end

	search_for_close: PREDICATE [ANY, TUPLE [RP_MARKET_DATA_RECORD]] is
			-- Search for close predicate
		once
			Result := agent has_close (?)
		end

	has_close (a_record: RP_MARKET_DATA_RECORD): BOOLEAN is
			-- Does `a_record' have close?
		do
			Result := a_record.has_close
		end

	search_for_volume: PREDICATE [ANY, TUPLE [RP_MARKET_DATA_RECORD]] is
			-- Search for volume predicate
		once
			Result := agent has_volume (?)
		end

	has_volume (a_record: RP_MARKET_DATA_RECORD): BOOLEAN is
			-- Does `a_record' have volume?
		do
			Result := a_record.has_volume
		end

	lookback (a_instrument: G; a_date: ROSE_DATE; a_predicate: PREDICATE [ANY, TUPLE [RP_MARKET_DATA_RECORD]]): RP_MARKET_DATA_RECORD is
			-- Lookback for record as of `a_date' for `a_instrument' with valid `a_field'
		require
			non_void_args: a_instrument /= Void and a_date /= Void and a_predicate /= Void
		local
			l_date: ROSE_DATE
			l_record: like snapshot
			l_start_date: ROSE_DATE
		do
			from
				l_date := a_date
				if start_date /= Void then
					l_start_date := start_date
				else
					l_start_date := default_start_date
				end
			until
				l_date < l_start_date or Result /= Void or (max_lookback_days > 0 and then a_date.diff_in_days (l_date) > max_lookback_days)
			loop
				l_record := snapshot (a_instrument, l_date)
				if l_record /= Void then
					if a_predicate.item ([l_record]) then
						Result := l_record
					end
				end
				l_date := l_date.previous_date
			end
		end

invariant

	max_lookback_days_ge_zero: max_lookback_days >= 0
	start_date_lt_end_date: (start_date /= Void and end_date /= Void) implies start_date < end_date

end
