note
	description: "Account or Portfolio non_cash position (traded)"
	author: "Sami Ben Grine & Ivan Tarnavsky"
	last_editor: "$Author: sm $"
	date: "$Date: 2013/09/27 $"
	revision: "$Revision: #2 $"
	path: "$File: //rose/source/kernel/expert_system/rp_non_cash_position.e $"

class
	RP_NON_CASH_POSITION[G->RP_ZONAL_INVESTMENT_GROUP]

inherit
	POSITION[G]


feature -- Access

	instrument: RP_INSTRUMENT
		-- Instrument

	total_quantity: RP_QUANTITY
		-- Total

	unavailable: RP_QUANTITY
		-- Unavailable


	local_currency: RP_CURRENCY
		-- Local currency

	cost_local: RP_MONIES
		-- Cost in local currency

	market_value_local: RP_MONIES
		-- Market valuation in local currency


	account_currency: RP_CURRENCY
		-- Base currency of the account

	cost_account_base: RP_MONIES
		-- Cost in account base currency


	global_currency: RP_CURRENCY
		-- Global currency

	cost_global_base: RP_MONIES
		-- Global currency for cost

	market_value_global_base: RP_MONIES
		-- Market valuation in global currency


	accounting_date: ROSE_DATE
		-- Accounting date (beginning of day)

	contract_side: CHARACTER
		-- (L)ong/(S)hort

	fm_sec_id: STRING
		-- FmSecID i.e. NT unique Front office security identifier

feature -- Query

	brimsym: STRING
		-- Brimsym
		do
			Result := instrument.brimsym
		end

	ref_uid: INTEGER_64
		-- UID of corresponding entity
		do
			Result := instrument.uid
		end

	quantity:RP_QUANTITY
			-- <Precursor>
		do
			if total_quantity /= Void then
				Result := total_quantity
			else
				Result := instrument.quantity (0)
			end
		end

	valuation: RP_MONIES
			-- <Precursor>
		do
			if market_value_local /= Void then
				Result := market_value_local
			elseif market_value_global_base /= Void then
				Result := market_value_global_base
			else
				Result := zero_usd
			end
		end

feature -- Status Report

	is_long: BOOLEAN
			-- Long?
		do
			Result := contract_side = 'L'
		end

	is_short: BOOLEAN
			-- Short?
		do
			Result := contract_side = 'S'
		end

invariant

	instrument_exists: instrument /= Void
	valid_uid: ref_uid ~ instrument.uid and then ref_uid /= 0
	valid_brimsym: brimsym ~ instrument.brimsym and then brimsym /= Void and then brimsym.count = 18

	valid_total_quantity: (total_quantity /= Void) and then (total_quantity.instrument ~ instrument)
	valid_quantity_unavailable: (unavailable /= Void) and then (unavailable.instrument ~ instrument)
	valid_amounts: (unavailable.sign=0) or else (total_quantity.sign=unavailable.sign and unavailable.amount.abs <= total_quantity.amount.abs)

	valid_local_currency: (local_currency = Void) implies (cost_local = Void and market_value_local = Void)
	valid_cost_local: (cost_local /= Void) implies (local_currency /= Void and then cost_local.currency ~ local_currency)
	valid_market_value_local: (market_value_local /= Void) implies (market_value_local.currency ~ local_currency)

	valid_account_base: (account_currency = Void implies cost_account_base = Void)
	valid_cost_account_base: (cost_account_base /= Void implies account_currency /= Void and then cost_account_base.currency ~ account_currency)

	valid_global_base: (global_currency = Void implies cost_global_base = Void and market_value_global_base = Void)
	valid_cost_global_base: (cost_global_base /= Void) implies (global_currency/=Void and then cost_global_base.currency ~ global_currency)
	valid_market_value_global_base: (market_value_global_base /= Void) implies (global_currency /= Void and then market_value_global_base.currency ~ global_currency)


	accounting_date_exists: accounting_date /= Void

	fm_sec_id_exists: fm_sec_id /= Void

	long_or_else_short: is_long xor is_short

end
