note
	description: "Account or Portfolio open currency forward (always traded)"
	author: "Sami Ben Grine & Ivan Tarnavsky"
	last_editor: "$Author: sm $"
	date: "$Date: 2013/09/27 $"
	revision: "$Revision: #2 $"
	path: "$File: //rose/source/kernel/expert_system/rp_open_ffx_position.e $"

deferred class
	RP_OPEN_FFX_POSITION[G->RP_ZONAL_INVESTMENT_GROUP]

inherit
	POSITION [G]
		rename instrument as forward redefine forward end

feature -- Access

	buy: RP_MONIES
		-- Buy

	sell: RP_MONIES
		-- Sell

	contract_date: ROSE_DATE
		-- Contract date

	settlement_date: ROSE_DATE
		-- Settlement date

	broker: STRING
		-- Broker

	settlement_currency: RP_CURRENCY
		-- Settlement currency

	foid: STRING
		-- Front Office ID sent to NT when the trade was created

	net_market_value: RP_MONIES
		-- Net market_value

	forward: RP_CURRENCY_FORWARD
			-- Instrument
		do
			Result := manager.reference_system.currency_forward (sell.currency, settlement_date)
		ensure then
			forward_attached: attached Result
		end

feature -- Query

	quantity: RP_QUANTITY
			-- <Precursor>
		do
			Result := forward.quantity (1)
		end

	buy_currency: RP_CURRENCY
		-- Buy currency
		do
			Result := buy.currency
		ensure
			buy_currency: Result ~ buy.currency
		end

	sell_currency: RP_CURRENCY
		-- Sell currency
		do
			Result := sell.currency
		ensure
			sell_currency: Result ~ sell.currency
		end

	valuation: RP_MONIES
		-- <Precursor>
		do
			Result := net_market_value
		end

invariant

	buy_exists: buy /= Void
	sell_exists: sell /= Void
	contract_date_exists: contract_date /= Void
	settlement_date_exists: settlement_date /= Void
	contract_date_before_settlement_date: contract_date <= settlement_date
	broker_exists: broker /= Void
	settlement_currency_exists: settlement_currency /= Void
	foid_exists: (foid /= Void) and (23 <= foid.count and foid.count <= 27)
	net_market_value_exists: net_market_value /= Void
	is_traded: accounting_basis /= Void and then accounting_basis.is_traded

end
