indexing
	description: "override_distributions_<Z>.PARM file"

	author: "Portfolio Accounting Tech"
	last_editor: "$Author: sm $"
	date: "$Date: 2009/09/28 $"
	revision: "$Revision: #5 $"
	archive: "$File: //rose/source/kernel/capital_actions/elections/rp_override_distributions.e $"

class
	RP_OVERRIDE_DISTRIBUTIONS

inherit
	EXCEPTIONS
		export {NONE} all end

	ROSE_SHARED_GLOBAL
		export {NONE} all end

	RP_EVENT_TYPE_CONSTANTS
		export {NONE} all end

	ANY

create
	make_from_windows_share

feature {NONE} -- Initialization

	make_from_windows_share (a_zone: like zone) is
			-- Create for 'a_zone'
		do
			zone := a_zone
			parse_from_windows_share (zone)
		ensure
			zone_set: zone.is_equal (a_zone)
		end

feature -- Access

	zone: RP_ZONE
		-- Zone

	windows_filename: STRING is
		-- Filename for this zone
		do
			Result := windows_path+"\override_distributions_"+zone.code.as_lower+".parm"
		end

	windows_path: !STRING is
			-- Windows file path
		do
			Result := global.hardcoded_path ("\\axaros.com\shares\ProdData_NT\PortfolioAccounting\ParmFiles")
		end

	item (a_brimsym: STRING; a_type: INTEGER; a_ex_date: ROSE_DATE; a_clicode: STRING): RP_OVERRIDE_DISTRIBUTIONS_PARM is
			-- Override for instrument, corporate action type, ex-date and portfolio
			-- We expect the client code to be in upper case
		require
			brimsym_not_void: a_brimsym /= Void
			valid_type: overridable_corporate_action_types.has (a_type)
			valid_ex_date: a_ex_date /= Void
			valid_clicode: a_clicode /= Void
			upper_case: a_clicode.is_equal (upper_case(a_clicode))
		local
			l_key: STRING
		do
			l_key := key (a_brimsym, a_type, a_ex_date, a_clicode)
			entries.search (l_key)
			if entries.found then
				Result := entries.found_item
			end
		ensure
			valid_result: (Result /= Void) implies (Result.brimsym.is_equal (a_brimsym) and Result.type.is_equal (a_type) and Result.ex_date.is_equal (a_ex_date) and Result.clicode.is_equal (a_clicode))
		end

	upper_case (a_clicode: STRING): STRING is
			-- Same string in upper case
		require
			clicode_not_void: a_clicode /= Void
		do
			create Result.make_from_string (a_clicode)
			Result.to_upper
		ensure
			same_length: Result.count = a_clicode.count
		end

feature -- Constants

	overridable_corporate_action_types: ARRAY [INTEGER] is
			-- Types of corporate actions that can be overriden
		once
			Result := <<
				split_event_type_4,
				stock_dividend_event_type_5,
				stock_distribution_event_type_6,
				delivery_event_type_10,
				tender_event_type_30,
				dividend_taxed_event_type_33
			>>
		end

feature {NONE} -- Implementation

	parse_from_windows_share (a_zone: RP_ZONE) is
			-- Parse 'a_file_name' and populate internal table
		local
			l_file: OVERRIDE_DISTRIBUTIONS_PARM_FILE
			l_array: ROSE_LINEAR_ARRAY [OVERRIDE_DISTRIBUTIONS_PARM_RECORD]
			l_parm: RP_OVERRIDE_DISTRIBUTIONS_PARM
			i: INTEGER
			l_key: STRING
		do
			create entries.make_equal (17)
			print ("Reading "+ windows_filename+"...") -- FIXME: replace wiht logging
			create l_file.make (windows_filename)
			if l_file.exists then
				l_file.parse
				l_array := l_file.array
				from
					i := l_array.lower
				until
					i > l_array.upper
				loop
					create l_parm.make (l_array [i])
					l_key := key (l_parm.brimsym, l_parm.type, l_parm.ex_date, l_parm.clicode)
					if not entries.has (l_key) then
						entries.force (l_parm, l_key)
						check available: item (l_parm.brimsym, l_parm.type, l_parm.ex_date, l_parm.clicode) = l_parm end
					else
						print ("Warning! Duplicate entry is ignored: "+l_parm.debug_output+"%N")
					end
					i := i + 1
				end
			end
			print ("done ["+entries.count.out+" entrie(s) in cache for "+zone.code+"]%N")
		end

	key (a_brimsym: STRING; a_type: INTEGER; a_ex_date: ROSE_DATE; a_clicode: STRING): STRING is
			-- Key for 'entries'
		require
			brimsym_not_void: a_brimsym /= Void
			valid_type: overridable_corporate_action_types.has (a_type)
			valid_ex_date: a_ex_date /= Void
			valid_clicode: a_clicode /= Void
		do
			Result := a_brimsym + "|" + a_type.out + "|" + a_ex_date.ccyymmdd.out+ "|" + a_clicode
		ensure
			not_void: Result /= Void
		end

	entries: DS_HASH_TABLE [RP_OVERRIDE_DISTRIBUTIONS_PARM, STRING]
		-- All entries

end
