indexing
	description: "Access to corporate actions overrides"
	author: "Portfolio Accounting Tech"
	last_editor: "$Author: sm $"
	date: "$Date: 2009/09/25 $"
	revision: "$Revision: #2 $"
	archive: "$File: //rose/source/kernel/capital_actions/elections/rp_override_distributions_aware.e $"

class
	RP_OVERRIDE_DISTRIBUTIONS_AWARE

inherit
	RP_EVENT_TYPE_CONSTANTS

feature -- Access

	override_distribution (a_zone: RP_ZONE; a_brimsym: STRING; a_type: INTEGER; a_ex_date: ROSE_DATE; a_clicode: STRING): RP_OVERRIDE_DISTRIBUTIONS_PARM is
			-- Override for instrument, corporate action type, ex-date and portfolio
			-- We expect the client code to be in upper case
		require
			valid_zone: a_zone /= Void
			brimsym_not_void: a_brimsym /= Void
			valid_type: (<<split_event_type_4, stock_dividend_event_type_5, delivery_event_type_10, tender_event_type_30 >>).has (a_type) -- FIXME
			valid_ex_date: a_ex_date /= Void
			valid_clicode: a_clicode /= Void and a_clicode.as_upper.is_equal (a_clicode)
		local
			l_overrides: RP_OVERRIDE_DISTRIBUTIONS
		do
			l_overrides := override_distributions_for_zone (a_zone)
			Result := l_overrides.item (a_brimsym, a_type, a_ex_date, a_clicode)

		ensure
			correct_result: (Result /= Void) implies (Result.brimsym.is_equal (a_brimsym) and Result.type.is_equal (a_type) and Result.ex_date.is_equal (a_ex_date) and Result.clicode.is_equal (a_clicode))
		end

feature {NONE} -- Implementation

	override_distributions_for_zone (a_zone: RP_ZONE): RP_OVERRIDE_DISTRIBUTIONS is
			-- Corporate actions overrides for 'a_zone'
		do
			if override_distributions_cache.has (a_zone) then
				Result := override_distributions_cache.item (a_zone)
			else
				create Result.make_from_windows_share (a_zone)
				override_distributions_cache.put (Result, a_zone)
			end
		ensure
			result_not_void: Result /= Void
			correct_zone: Result.zone.is_equal (a_zone)
		end

	override_distributions_cache: HASH_TABLE[RP_OVERRIDE_DISTRIBUTIONS, RP_ZONE]
			-- Shared zonal CA overrides
		once
			create Result.make (0)
		ensure
			result_not_void: Result /= Void
		end

end
