note

	description:

		"Constants and magic numbers used in performance"

	author: "Ivan Tarnavsky"
	last_editor: "$Author: it $"
	date: "$Date: 2011/03/04 $"
	revision: "$Revision: #4 $"
	path: "$File: //rose/source/kernel/expert_system/perf/rp_performance_constants.e $"

frozen class RP_PERFORMANCE_CONSTANTS

feature -- Access

	smallcap_share_activity_first_date: ROSE_DATE is
			-- First date of SMALLCAP share activity
		once
			create Result.make_ccyymmdd (20020315)
		ensure
			result_not_void: Result /= Void
		end

	smallcap_share_activity_end_date_of_special_lookup: ROSE_DATE is
			-- Last date of special share activity lookup for SMALLCAP
		once
			create Result.make_ccyymmdd (20050101)
		ensure
			result_not_void: Result /= Void
		end

	jzone_first_short_dividend_date: ROSE_DATE is
			-- Minimum date for J zone agreement time to consider short dividend
		once
			create Result.make (2008, 1, 1)
		ensure
			jzone_first_short_dividend_date_not_void: Result /= Void
		end

	uzone_last_non_resident_alien_rate_date: ROSE_DATE is
			-- Date from which we withhold 30% tax on dividends
		once
			create Result.make (2001, 1, 1)
		ensure
			uzone_last_non_resident_alien_rate_date_not_void: Result /= Void
		end

	thirty_percent: RP_DECIMAL is
			-- 30% in decimal
		once
			create Result.make_from_string ("0.30")
		ensure
			thirty_percent_not_void: Result /= Void
		end

	ozone_stock_dividend_first_date_for_30_days_settlement: ROSE_DATE
			-- After this date stock dividends in O zone have 30 day settlement
		once
			create Result.make_ccyymmdd (20040824)
		ensure
			result_not_void: Result /= Void
		end

	ozone_stock_dividend_portfolio_wildcard_name: STRING is "ALL"
			-- Wildcard portfolio name for O zone stock dividend matching

feature -- Query

	is_smallcap_special_share_activity_lookup (a_pool_fund: RP_POOL_FUND; a_date: ROSE_DATE): BOOLEAN is
			-- Check whether `a_pool_fund' is SMALLCAL
			-- and whether special share activity lookup is needed for `a_date'
		require
			a_pool_fund_not_void: a_pool_fund /= Void
			pool_fund_portfolio_not_void: a_pool_fund.portfolio /= Void
			a_date_not_void: a_date /= Void
		do
			Result := a_pool_fund.portfolio.code ~ "SMALLCAP" and then a_date >= smallcap_share_activity_first_date and a_date < smallcap_share_activity_end_date_of_special_lookup
		end

	is_sharepool_income_and_expense_distributed (a_portfolio: RP_PORTFOLIO): BOOLEAN is
			-- Is share pool income and expense distributed for `a_portfolio'
			-- NB: U zone fund participants with few exceptions
			-- NB: TOSHIBU JASONU PEERU
			-- XXX: At present there is no GCDB attribute to specify the excluded portfolios
		require
			a_portfolio_not_void: a_portfolio /= Void
		do
			Result := a_portfolio.zone.is_united_states and a_portfolio.code /~ "TOSHIBU" and a_portfolio.code /~ "JASONU" and a_portfolio.code /~ "PEERU"
		end

feature -- Dividend announcement categories

	dividend_announcement_category_taxfree: STRING is "TAXFREE"
			-- Dividend is announced TAXFREE

	dividend_announcement_category_gross: STRING is "GROSS"
			-- Dividend is announced GROSS of withholding tax

	dividend_announcement_category_net: STRING is "NET"
			-- Dividend is announced NET of withholding tax

	dividend_announcement_category_force_client_rate: STRING is "UNUSED"
			-- This category forces taxable dividends with client withholding rate

end
