indexing

	description:

		"Corporate Bond Interest calculation algorithm"

	date: "$Date: 2010/04/30 $"
	revision: "$Revision: #6 $"

class RP_PERFORMANCE_CORPORATE_BOND_INTEREST_ALGORITHM

inherit

	RP_PRICING_ALGORITHM [RP_CORPORATE_BOND]

create

	make

feature {NONE} -- Initializations

	make (a_market_data_source: RP_MARKET_DATA_SOURCE [RP_CORPORATE_BOND]; a_calendar: RP_CALENDAR) is
			-- Initialize `Current'
		require
			market_data_source_not_void: a_market_data_source /= Void
			calendar_not_void: a_calendar /= Void
		do
			market_data_source := a_market_data_source
			calendar := a_calendar
			create weekdays_calendar
			latest_update_date:=calendar.ith_work_day_before (create {ROSE_DATE}.make_ccyymmdd (20200101), 1)
		ensure
			market_data_source_set: market_data_source = a_market_data_source
			calendar_set: calendar = a_calendar
		end

feature -- Query

	price (a_bond: RP_CORPORATE_BOND; a_date: ROSE_DATE): RP_PRICE is
			-- Price of `a_bond' on `a_date' in units of RP_CURRENCY
		local
			l_payday_prior, l_payday_next: ROSE_DATE
			l_payment, l_accrued_interest: RP_DECIMAL
			l_numdays, l_basedays: RP_DECIMAL
			l_julmon, l_days: INTEGER
			l_currency: RP_CURRENCY
			l_record: RP_MARKET_DATA_RECORD
			l_int_acc: RP_PRICE
		do
			l_accrued_interest := 0
			l_currency := a_bond.currency
				-- Check for INT_ACC from the LOW price field
			l_record := market_data_source.snapshot (a_bond, a_date)

			if a_bond.months.is_equal (0) and a_bond.coupon_rate.is_equal (0.0) then
				-- zero coupon bond
				Result := a_bond.price_in_currency (0.0 , l_currency, a_date)
			else
				if a_bond.maturity_date /= Void and then a_date <= a_bond.maturity_date and
					( a_bond.inception_date = Void or else a_bond.inception_date < a_date)
					then
					if l_record /= Void and then l_record.has (l_record.unadjusted_low_type) and then l_record.price_field (l_record.unadjusted_low_type).price > 0 then
						l_int_acc := l_record.price_field (l_record.unadjusted_low_type)
						l_accrued_interest := l_int_acc.price / 100
						l_currency := l_int_acc.currency
					else

						l_payment := 0
						l_numdays := 0
						l_basedays := 1
						l_payday_prior := prior_coupon_date (a_bond, a_date, True)
						l_payday_next := next_coupon_date (a_bond, l_payday_prior)

						if not (l_payday_prior.is_equal (a_date) or l_payday_next.is_equal (a_date)) then
							l_payment := compute_coupon_payment (a_bond, a_date)
							inspect a_bond.accrual_method
							when 1 then
									-- Actual / Actual
								l_payday_prior := prior_coupon_schedule (a_bond, a_date)
								if a_bond.months = 12 then
									l_numdays := a_date.diff_in_days (l_payday_prior).abs
									l_basedays := 365

								else
									if a_bond.brimsym.substring (10, 12).is_equal ("UST") then
										l_payday_prior := prior_coupon_schedule (a_bond, a_date)
										l_payday_next := next_coupon_schedule (a_bond, l_payday_prior)
									end
									l_numdays := a_date.diff_in_days (l_payday_prior).abs
									l_basedays := l_payday_next.diff_in_days (l_payday_prior).abs

								end
							when 2 then
									-- Actual / 365
								if a_bond.coupon_method = 1 then
									l_numdays := a_date.diff_in_days (l_payday_prior).abs
									l_basedays := 365
									l_payment :=  a_bond.par_value * (a_bond.coupon_rate / 100)
								else
									l_numdays := a_date.diff_in_days (l_payday_prior).abs
									l_basedays := l_payday_next.diff_in_days (l_payday_prior).abs
								end
							else
									-- 30E / 360
								l_payment := a_bond.par_value * (a_bond.coupon_rate / 100)
								l_payday_prior := prior_coupon_schedule (a_bond, a_date)
								l_numdays := 0
								l_basedays := 360
								if a_date.julian_month = l_payday_prior.julian_month then
									l_numdays := a_date.diff_in_days (l_payday_prior)
									if not a_bond.brimsym.substring (16, 18).is_equal ("USD") then
										l_numdays := l_numdays + 1
									end
									if a_date.day > 30 then
										l_numdays := l_numdays - 1
									end
								else
									from
										l_julmon := l_payday_prior.julian_month
									until
										l_julmon > a_date.julian_month
									loop
										if l_julmon = l_payday_prior.julian_month then
											l_days := l_payday_prior.day
											l_numdays := (0).max (30 - l_days)
										elseif l_julmon = l_payday_next.julian_month then
											l_days := l_payday_next.day
											l_numdays := l_numdays + (30).min (l_days)
										elseif l_julmon = a_date.julian_month then
											l_days := a_date.day
											l_numdays := l_numdays + (30).min (l_days)
										else
											l_numdays := l_numdays + 30
										end
										l_julmon := l_julmon + 1
									end
								end
							end
						end
						l_accrued_interest := l_payment * l_numdays.abs / l_basedays.abs
					end
					Result := a_bond.price_in_currency (l_accrued_interest , l_currency, a_date)
				else
					Result := a_bond.price_in_currency (0.0 , l_currency, a_date)
				end
			end
		end

feature {NONE} -- Implementation

	market_data_source: RP_MARKET_DATA_SOURCE [RP_CORPORATE_BOND]
			-- Market data source

	calendar: RP_CALENDAR
			-- Market calendar

	weekdays_calendar: RP_WEEKDAYS_CALENDAR
			-- weekday calendar

	latest_update_date: ROSE_DATE
			-- the date up to which the calendar is updated

	prior_coupon_schedule (a_bond: RP_CORPORATE_BOND; a_date: ROSE_DATE): ROSE_DATE is
			-- Prior to `a_date' coupon schedule date for `a_bond'
		require
			attached_bond: a_bond /= Void
			attached_date: a_date /= Void
		local
			l_found, l_done: BOOLEAN
			l_dates: ROSE_LINEAR_ARRAY [ROSE_DATE]
			i: INTEGER
			l_exdate: ROSE_DATE
			l_date, l_start: ROSE_DATE
		do
			if a_bond.months > 0 and a_bond.coupon_rate > 0 then
				l_dates := coupon_schedule_override (a_bond)
				if l_dates = Void then
					create l_dates.make_empty
					if a_bond.inception_date /= Void then
						create l_start.make_ccyymmdd (a_bond.inception_date.ccyymmdd)
						from
							l_date := l_start
							l_dates.extend (l_start)
						until
							l_date > a_date
						loop
							l_date.set_day (l_date.days_in_i_th_month (l_date.month, l_date.year).min (l_start.day))
							l_date := l_date.plus_n_month (a_bond.months)
							l_dates.extend (l_date)
						end
					else
						create l_start.make_ccyymmdd (a_bond.maturity_date.ccyymmdd)
						from
							l_date := l_start
							l_dates.extend (l_start)
						until
							l_date < a_date
						loop
							l_date.set_day (l_date.days_in_i_th_month (l_date.month, l_date.year).min (l_start.day))
							l_date := l_date.plus_n_month (-a_bond.months)
							l_dates.extend (l_date)
						end
					end
				end

				l_dates.sort_by (agent comparison(?,?))
				from
					i := 1
				until
					i > l_dates.count or l_done
				loop
					if a_date  <= l_dates.item (i)  then
						l_done := True
					else
						l_exdate := l_dates.item(i)
						l_found := True
						i := i + 1
					end
				end
			end
			if l_found then
				Result := coupon_payment_schedule (a_bond, l_exdate)
				if Result = Void then
					Result := l_exdate
				end
			else
				create Result.make_max_date
			end
		end

	coupon_schedule_override (a_bond: RP_CORPORATE_BOND): ROSE_LINEAR_ARRAY [ROSE_DATE] is
			-- Hardcoded schedules from DUP_W_A:OVERRIDE_COUPON_SCHEDULE.PARM
			-- FIXME import the data to a database.
			-- Returns Void when there is no override
		require
			attached_bond: a_bond /= Void
		do
			if a_bond.brimsym.is_equal ("HKB000235WING..USD") then
				create Result.make_from_array (<<create {ROSE_DATE}.make_ccyymmdd (20030310), create {ROSE_DATE}.make_ccyymmdd (20030910), create {ROSE_DATE}.make_ccyymmdd (20040410), create {ROSE_DATE}.make_ccyymmdd (20041010)>>)
			end
			if a_bond.brimsym.is_equal ("SGB000052SGS0..SGD") then
				create Result.make_from_array (<<create {ROSE_DATE}.make_ccyymmdd (20030910), create {ROSE_DATE}.make_ccyymmdd (20040329), create {ROSE_DATE}.make_ccyymmdd (20040929)>>)
			end
		ensure
			may_return_void_if_no_override_found: True
		end

	coupon_payment_schedule (a_bond: RP_CORPORATE_BOND; a_date: ROSE_DATE): ROSE_DATE is
			-- Coupon payment schedule
		require
			attached_bond: a_bond /= Void
			attached_date: a_date /= Void
		local
			l_coupons: ARRAY [ROSE_DATE]
			i: INTEGER
		do
			l_coupons := a_bond.coupon_dates
			if l_coupons /= Void and then not l_coupons.is_empty then
				from
					i := 1
				until
					i > l_coupons.count or Result /= Void
				loop
					if a_date.ccyymm = l_coupons.item (i).ccyymm then
						Result := l_coupons.item (i)
					end
					i := i + 1
				end
			end
		end

	prior_coupon_date (a_bond: RP_CORPORATE_BOND; a_date: ROSE_DATE; a_accrual_mode: BOOLEAN): ROSE_DATE is
			-- Prior to `a_date' coupon date for `a_bond'
		require
			attached_bond: a_bond /= Void
			attached_date: a_date /= Void
		local
			l_found, l_done: BOOLEAN
			l_dates: ROSE_LINEAR_ARRAY [ROSE_DATE]
			i: INTEGER
			l_exdate: ROSE_DATE
			l_date, l_start,l_temp_date: ROSE_DATE
		do

			if a_bond.months > 0 and a_bond.coupon_rate > 0 then
				l_dates := coupon_schedule_override (a_bond)
				if l_dates = Void then
					create l_dates.make_empty
					if a_bond.inception_date /= Void then
						create l_start.make_ccyymmdd (a_bond.inception_date.ccyymmdd)

						from
							l_date := l_start
							if l_date < a_date then
								l_dates.extend (l_start)
							end
						until
							l_date > a_date
						loop
							l_date.set_day (l_date.days_in_i_th_month (l_date.month, l_date.year).min (l_start.day))
							if not a_accrual_mode then
								if l_date > latest_update_date  then
									l_temp_date:=weekdays_calendar.ith_work_day_before (l_date, 1)
								else
									l_temp_date :=calendar.ith_work_day_before (l_date, 1)
								end

								if l_temp_date /=void then
									l_date := l_temp_date
								end
								if l_date > latest_update_date then
									l_temp_date:=weekdays_calendar.ith_work_day_after (l_date, 1)
								else
									l_temp_date :=calendar.ith_work_day_after (l_date, 1)
								end

								if  l_temp_date/=void then
									l_date := l_temp_date
								end
							end

							l_date := l_date.plus_n_month (a_bond.months)
							l_dates.extend (l_date)
						end
					else
						create l_start.make_ccyymmdd (a_bond.maturity_date.ccyymmdd)
--						l_start := l_start.plus_n_month (-12)
						from
							l_date := l_start
							if l_date > a_date then
								l_dates.extend (l_start)
							end
						until
							l_date < a_date
						loop
							l_date.set_day (l_date.days_in_i_th_month (l_date.month, l_date.year).min (l_start.day))
							if not a_accrual_mode then
								if l_date > latest_update_date then
									l_temp_date:=weekdays_calendar.ith_work_day_before (l_date, 1)
								else
									l_temp_date :=calendar.ith_work_day_before (l_date, 1)
								end
								if l_temp_date /=void then
									l_date := l_temp_date
								end
								if l_date > latest_update_date then
									l_temp_date:=weekdays_calendar.ith_work_day_after (l_date, 1)
								else
									l_temp_date :=calendar.ith_work_day_after (l_date, 1)
								end
								if  l_temp_date/=void then
									l_date := l_temp_date
								end
							end

							l_date := l_date.plus_n_month (-a_bond.months)
							l_dates.extend (l_date)
						end
					end
				end

				l_dates.sort_by (agent comparison(?,?))
				from
					i := 1
				until
					i > l_dates.count or l_done
				loop
					if a_date <= l_dates.item (i) then
						l_done := True
					else
						l_exdate := l_dates.item (i)
						l_found := True
						i := i + 1
					end
				end
			end
			if l_found then
				Result := coupon_payment_schedule (a_bond, l_exdate)
				if Result = Void then
					Result := l_exdate
				end
			else
				create Result.make_max_date
			end
		end

	next_coupon_schedule (a_bond: RP_CORPORATE_BOND; a_prior_coupon_date: ROSE_DATE): ROSE_DATE is
			-- Next coupon schedule
		require
			non_void_args: a_bond /= Void and a_prior_coupon_date /= Void
		do
			create Result.make_by_date( a_prior_coupon_date)
			Result.add_n_months (a_bond.months)
		end

	next_coupon_date (a_bond: RP_CORPORATE_BOND; a_prior_coupon_date: ROSE_DATE): ROSE_DATE is
			-- Next coupon date
		require
			non_void_args: a_bond /= Void and a_prior_coupon_date /= Void
		do
			create Result.make_by_date( a_prior_coupon_date)
			Result.add_n_months (a_bond.months)
		end

	compute_coupon_payment (a_bond: RP_CORPORATE_BOND; a_date: ROSE_DATE): RP_DECIMAL is
			-- Coupon payment
		require
			non_void_args: a_bond /= Void and a_date /= Void
		local
			l_prior_payday, l_next_payday: ROSE_DATE
			l_frn_rate: RP_DECIMAL
		do
			if a_bond.coupon_method = 1 then
				if a_bond.accrual_method = 6 then
					l_prior_payday := prior_coupon_schedule (a_bond, a_date)
				else
					l_prior_payday := prior_coupon_date (a_bond, a_date, False)
				end
				l_next_payday := next_coupon_date (a_bond, l_prior_payday)
				l_frn_rate := a_bond.frn_rate (a_date)
				if l_frn_rate >= 0 then
					Result := a_bond.par_value * a_bond.months / 12 * l_frn_rate / 100
				else
					Result := a_bond.coupon_rate * a_bond.months / 12 * a_bond.par_value / 100
				end
			else
				l_frn_rate := a_bond.frn_rate (a_date)
				if l_frn_rate >= 0 then
					Result := a_bond.par_value * a_bond.months / 12 * l_frn_rate / 100
				else
					Result := a_bond.par_value * a_bond.months / 12 * a_bond.coupon_rate / 100
				end
			end
		end

	comparison(a_date_1,a_date_2:ROSE_DATE): BOOLEAN
		-- comparison
	require
		non_void_args: a_date_1 /= Void and a_date_2 /= Void
	do
		Result := a_date_1.ccyymmdd < a_date_2.ccyymmdd
	end

end
