indexing

	description:

		"Corporate Bond pricing algorithm (capital price)"

	date: "$Date: 2010/04/30 $"
	revision: "$Revision: #5 $"

class RP_PERFORMANCE_CORPORATE_BOND_PRICING_ALGORITHM

inherit

	RP_PRICING_ALGORITHM [RP_CORPORATE_BOND]

create

	make

feature {NONE} -- Initializations

	make (a_trade_price: like trade_price; a_calendar: like calendar; a_corporate_bond_market_data_source: like corporate_bond_market_data_source; a_price_override: like price_override) is
			-- Initialize `Current'
		require
			trade_price_vot_void: a_trade_price /= Void
			calendar_not_void: a_calendar /= Void
			corporate_bond_market_data_source_not_void: a_corporate_bond_market_data_source /= Void
			price_override_not_void: a_price_override /= Void
		do
			trade_price := a_trade_price
			calendar := a_calendar
			corporate_bond_market_data_source := a_corporate_bond_market_data_source
			price_override := a_price_override
			create last_price_cache.make_equal (17)
		ensure
			trade_price_set: trade_price = a_trade_price
			calendar_set: calendar = a_calendar
			corporate_bond_market_data_source_set: corporate_bond_market_data_source = a_corporate_bond_market_data_source
			price_override_set: price_override = a_price_override
		end

feature -- Query

	price (a_bond: RP_CORPORATE_BOND; a_date: ROSE_DATE): RP_PRICE is
			-- Price of `a_bond' on `a_date' in units of RP_CURRENCY
		local
			l_record: RP_MARKET_DATA_RECORD
		do
				-- Check for price override first
			if price_override /= Void then
				l_record := price_override.snapshot (a_bond, a_date)
				if l_record /= Void then
					Result := l_record.unadjusted_close
				end
			end
				-- Retrieve market price if no price override
			if Result = Void then
				Result := bond_price (a_bond, a_date, calendar)
					-- Update lag price
				if Result /= Void then
					last_price_cache.force (Result, a_bond)
				end
			end
				-- Trade price
			if Result = Void then
				l_record := trade_price.snapshot (a_bond, a_date)
				if l_record /= Void then
					Result := l_record.unadjusted_close
					last_price_cache.force (Result, a_bond)
				end
			end
				-- Use lag price if everything else fails
			if Result = Void then
				last_price_cache.search (a_bond)
				if last_price_cache.found then
					Result := last_price_cache.found_item
				end
			end
		ensure then
			cache_set_if_price_found: Result /= Void implies last_price_cache.has (a_bond)
		end

feature {NONE} -- Implementation

	last_price_cache: UID_TABLE [RP_PRICE, RP_INSTRUMENT]
			-- Last computed price cache

	price_override: RP_MARKET_DATA_SOURCE [RP_INSTRUMENT]
			-- Price override market data source

	lookback_range: INTEGER is 60
			-- How long to look back for price

	corporate_bond_market_data_source: RP_MARKET_DATA_SOURCE [RP_CORPORATE_BOND]
			-- Corporate bond market data source

	trade_price: RP_TRADE_PRICE_MARKET_DATA_SOURCE
			--	trade price

	calendar: RP_CALENDAR
			-- Market calendar

	bond_price (a_bond: RP_CORPORATE_BOND; a_date: ROSE_DATE; a_calendar: RP_CALENDAR): RP_PRICE is
			-- Price of `a_bond' on `a_date' using `a_calendar'
		require
			attached_bond: a_bond /= Void
			attached_date: a_date /= Void
			attached_calendar: a_calendar /= Void
		local
			l_record: RP_MARKET_DATA_RECORD
			l_count: INTEGER
			l_date: ROSE_DATE
		do

			from
				create l_date.make_by_date (a_date)
				l_count := 1
				l_record := corporate_bond_market_data_source.snapshot (a_bond, a_date)
			until l_record /= Void and then l_record.unadjusted_close.price > 0.0 or l_count > max_bond_price_lookback_days
			loop
				l_date := a_calendar.ith_work_day_before (l_date, 1)
				l_record := corporate_bond_market_data_source.snapshot (a_bond, l_date)
				l_count := l_count + 1
			end

			if l_record /= Void then
				if l_record.has_close then
					Result := l_record.unadjusted_close.scale_by (bond_price_scaling_factor)
				end
			end
		end

	max_bond_price_lookback_days: INTEGER is 30
		-- Max bus. days to lookback for bond price

	bond_price_scaling_factor: DOUBLE is 0.01
		-- Scaling factor for bond price

end
