note

	description:

		"Account Flow to Daily Performance Affecting Quantities Aggregator"

	author: "Ivan Tarnavsky"
	last_editor: "$Author: er $"
	date: "$Date: 2013/01/30 $"
	revision: "$Revision: #19 $"
	path: "$File: //rose/source/kernel/expert_system/aflows/rp_performance_daily_bucket_flow_aggregator.e $"

class RP_PERFORMANCE_DAILY_BUCKET_FLOW_AGGREGATOR

inherit

	RP_UNKNOWN_CUSTODY_AWARE
		export {NONE} all end

	RP_FLOW_VISITOR

create

	make

feature {NONE} -- Ininitialization

	make (a_bucket: like bucket; a_portfolio: RP_PORTFOLIO; a_valuator: like valuator) is
			-- Initialize `Current'.
		require
			a_bucket_not_void: a_bucket /= Void
			a_portfolio_not_void: a_portfolio /= Void
		do
			bucket := a_bucket
			if a_portfolio.zone.is_united_states then
					-- 0.05 minimum holding for U zone
					-- FIXME this can be changes after discussion
					-- FIXME with CIOs
				min_holding := 0.05
				has_legacy_u_zone_behaviour := True
			else
				create min_holding.make_zero
			end
			valuator := a_valuator
			portfolio := a_portfolio
			holding_control := portfolio.instrument_control (unknown_custody)
		ensure
			bucket_set: bucket = a_bucket
			valuator_set: valuator = a_valuator
			portfolio_set: portfolio = a_portfolio
		end

feature -- Update

	adjust_with (a_holding_interval: RP_HOLDING_INTERVAL) is
			-- aggregates adjustment flows associated with a holding
		do
				-- adjust portfolio value
			add_to_portfolio_value (a_holding_interval)
			if a_holding_interval.start_date.ccyymmdd = bucket.perf_date.ccyymmdd then
					-- and the other buckets if there were any adjustments
				a_holding_interval.adjustments.do_all (agent process (?))
			end
		end

feature {RP_PERFORMANCE_DAILY_BUCKET_FLOW_AGGREGATOR} -- Update

	add_to_perf_unrelated_expense (a_flow: RP_FLOW) is
			-- add to unrelated expense bucket
		require
			a_flow_not_void: a_flow /= Void
		do
			if is_not_zero (a_flow.quantity.amount) then
				bucket.set_perf_unrelated_expense (bucket.perf_unrelated_expense + value_of ("EUNR", as_holding (a_flow)))
			end
		end

	add_to_perf_unrelated_income (a_flow: RP_FLOW) is
			-- add to perf_unrelated_income bucket
		require
			a_flow_not_void: a_flow /= Void
		do
			if is_not_zero (a_flow.quantity.amount) then
				bucket.set_perf_unrelated_income (bucket.perf_unrelated_income + value_of ("IUNR", as_holding (a_flow)))
			end
		end

	add_to_dividend_tax_withholding (a_flow: RP_FLOW) is
			-- add to dividend tax withholding bucket
		require
			a_flow_not_void: a_flow /= Void
		do
			if is_not_zero (a_flow.quantity.amount) then
				if a_flow.type.is_dividend_tax_refund_flow and then has_legacy_u_zone_behaviour and then a_flow.contracted_ownership_time.ccyymmdd < 20060101 then
					bucket.set_income (bucket.income + value_of ("T20R", as_holding (a_flow)))
				else
					bucket.set_dividend_tax_withholding (bucket.dividend_tax_withholding + value_of ("DTWH", as_holding (a_flow)))
				end
			end
		end

	add_to_withdrawal (a_flow: RP_FLOW) is
			-- add to withdrawal bucket
		require
			a_flow_not_void: a_flow /= Void
		do
			if is_not_zero (a_flow.quantity.amount) then
				bucket.set_withdrawal (bucket.withdrawal + value_of ("WITH", as_holding (a_flow)))
			end
		end

	add_to_contribution (a_flow: RP_FLOW) is
			-- add to contribution bucket
		require
			a_flow_not_void: a_flow /= Void
		do
			if is_not_zero (a_flow.quantity.amount) then
				bucket.set_contribution (bucket.contribution + value_of ("CONT", as_holding (a_flow)))
			end
		end

	add_to_intrazonal_withdrawal (a_flow: RP_FLOW) is
			-- add to intrazonal_withdrawal bucket
		require
			a_flow_not_void: a_flow /= Void
		do
			if is_not_zero (a_flow.quantity.amount) then
				bucket.set_intrazonal_withdrawal (bucket.intrazonal_withdrawal + value_of ("INTW", as_holding (a_flow)))
			end
		end

	add_to_intrazonal_contribution (a_flow: RP_FLOW) is
			-- add to contribution bucket
		require
			a_flow_not_void: a_flow /= Void
		do
			if is_not_zero (a_flow.quantity.amount) then
				bucket.set_intrazonal_contribution (bucket.intrazonal_contribution + value_of ("INTC", as_holding (a_flow)))
			end
		end

	add_to_transfer (a_flow: RP_FLOW) is
			-- add to transfer bucket
		require
			a_flow_not_void: a_flow /= Void
		do
			if is_not_zero (a_flow.quantity.amount) then
				if a_flow.type.is_inkind_flow and then has_legacy_u_zone_behaviour then
					bucket.set_transfer (bucket.transfer + value_of ("INKI", convert_flow_value_to_holding (a_flow)))
				elseif a_flow.type.is_outkind_flow and then has_legacy_u_zone_behaviour then
					bucket.set_transfer (bucket.transfer + value_of ("OUTK", convert_flow_value_to_holding (a_flow)))
				else
					bucket.set_transfer (bucket.transfer + value_of ("XFER", as_holding (a_flow)))
				end
			end
		end

	add_to_realized_hedge_gain_loss (a_flow: RP_FLOW) is
			-- realized g/l
		require
			a_flow_not_void: a_flow /= Void
		do
			if is_not_zero (a_flow.quantity.amount) then
				bucket.set_realized_hedge_gain_loss (bucket.realized_hedge_gain_loss + value_of ("RHGL", as_holding (a_flow)))
			end
		end

	add_to_income (a_flow: RP_FLOW) is
			-- income bucket
		require
			a_flow_not_void: a_flow /= Void
		do
			if is_not_zero (a_flow.quantity.amount) then
				bucket.set_income (bucket.income + value_of ("INCOME", as_holding (a_flow)))
			end
		end

	add_to_expense (a_flow: RP_FLOW) is
			-- expense bucket
		require
			a_flow_not_void: a_flow /= Void
		do
			if is_not_zero (a_flow.quantity.amount) then
				bucket.set_expense (bucket.expense + value_of ("EXPENSE", as_holding (a_flow)))
			end
		end

	add_to_dividend (a_flow: RP_FLOW) is
			-- dividend bucket
		require
			a_flow_not_void: a_flow /= Void
		do
			if is_not_zero (a_flow.quantity.amount) then
				bucket.set_dividend (bucket.dividend + value_of ("DIVIDEND", convert_flow_value_to_holding (a_flow)))
			end
		end

	add_to_buycost (a_flow: RP_FLOW) is
			-- buycost bucket
		require
			a_flow_not_void: a_flow /= Void
		do
			if is_not_zero (a_flow.quantity.amount) then
					-- use minus (-) because the flow value is negative
				bucket.set_buycost (bucket.buycost - value_of ("BUYCOST", convert_flow_value_to_holding (a_flow)))
			end
		end

	substract_from_buycost (a_flow: RP_FLOW) is
			-- adjust buycost bucket
		require
			a_flow_not_void: a_flow /= Void
		do
			if is_not_zero (a_flow.quantity.amount) then
					-- use minus (+) because the flow value is negative
				bucket.set_buycost (bucket.buycost + value_of ("BUYCOST", convert_flow_value_to_holding (a_flow)))
			end
		end

	add_to_sellnet (a_flow: RP_FLOW) is
			-- sellnet bucket
		require
			a_flow_not_void: a_flow /= Void
		do
			if is_not_zero (a_flow.quantity.amount) then
				bucket.set_sellnet (bucket.sellnet + value_of ("SELLNET", convert_flow_value_to_holding (a_flow)))
			end
		end

	add_to_sellnet2 (a_flow: RP_FLOW) is
			-- sellnet bucket
		require
			a_flow_not_void: a_flow /= Void
		do
			if is_not_zero (a_flow.quantity.amount) then
				bucket.set_sellnet2 (bucket.sellnet2 + value_of ("SELLNET2", as_holding (a_flow)))
			end
		end

	add_to_withindaybuy (a_bucket: STRING; a_holding: RP_HOLDING) is
			-- withindaybuy bucket
		require
			a_holding_not_void: a_holding /= Void
			a_bucket_not_void: a_bucket /= Void
		do
			if is_not_zero (a_holding.quantity.amount) then
				bucket.set_withindaybuy (bucket.withindaybuy + value_of (a_bucket, a_holding))
			end
		end

	add_to_withindaybuy2 (a_flow: RP_FLOW) is
			-- withindaybuy2 bucket
		require
			a_flow_not_void: a_flow /= Void
		do
			if is_not_zero (a_flow.quantity.amount) then
				bucket.set_withindaybuy2 (bucket.withindaybuy2 + value_of ("WDBUY2_PR", as_holding (a_flow)) - value_of ("WDBUY2_VA", convert_flow_value_to_holding (a_flow)))
			end
		end

	add_to_commtaxbuy (a_flow: RP_FLOW) is
			-- commission and tax
		require
			a_flow_not_void: a_flow /= Void
		do
			if is_not_zero (a_flow.quantity.amount) then
				bucket.set_commtaxbuy (bucket.commtaxbuy + value_of ("COMMTAXBUY", as_holding (a_flow)))
			end
		end

feature {RP_PERFORMANCE_SYSTEM} -- Setters

	add_to_shared_poolfund_expense (a_amount: RP_MONIES) is
			-- Add `a_amount' to `share_poolfund_expense'
		require
			non_void_amount: a_amount /= Void
		do
			bucket.set_share_poolfund_expense (bucket.share_poolfund_expense + a_amount)
		end

	add_to_shared_poolfund_income (a_amount: RP_MONIES) is
			-- Add `a_amount' to `share_poolfund_income'
		require
			non_void_amount: a_amount /= Void
		do
			bucket.set_share_poolfund_income (bucket.share_poolfund_income + a_amount)
		end

feature {NONE} -- Implemenation

	has_legacy_u_zone_behaviour: BOOLEAN
			-- Treat certain flows as legacy U zone does

	bucket: RP_PERFORMANCE_DAILY_VALUES
			-- DailyValues to update

	valuator: RP_HOLDING_VALUATOR
			-- Valuator

	portfolio: RP_PORTFOLIO
			-- Control for debit/credit

	holding_control: RP_INSTRUMENT_CONTROL
			-- Control to build holding

	min_holding: RP_DECIMAL
			-- Minimum number of chares for holding to be included in portfolio value

	is_holding_big_enough (a_value: RP_DECIMAL): BOOLEAN is
			-- Check if `a_value' is >= `min_holding'
		do
			Result := a_value.abs >= min_holding
		end

	add_to_portfolio_value (a_holding: RP_HOLDING) is
			-- aggregate holdings to portfolio value
		require
			non_void_holding: a_holding /= Void
		local
			l_value: like value_of
		do
			if is_not_zero (a_holding.quantity.amount) then
				if a_holding.quantity.instrument.is_currency then
					l_value := value_of ("PVAL", a_holding)
					bucket.set_portfolio_value (bucket.portfolio_value + l_value)
					bucket.set_cash_value (bucket.cash_value + l_value)
				elseif a_holding.quantity.instrument.is_forward then
					l_value := value_of ("PVAL", a_holding)
					bucket.set_unrealized_hedge_contract_value (bucket.unrealized_hedge_contract_value + l_value)
					bucket.set_portfolio_value (bucket.portfolio_value + l_value)
				else
					if is_holding_big_enough (a_holding.quantity.amount) then
						l_value := value_of ("PVAL", a_holding)
						bucket.set_portfolio_value (bucket.portfolio_value + l_value)
					end
				end
				bucket.set_equity_value (bucket.portfolio_value - bucket.cash_value - bucket.unrealized_hedge_contract_value)
				bucket.set_previous_portfolio_value (bucket.portfolio_value)
			end
		end

	convert_flow_value_to_holding (a_flow: RP_FLOW): RP_HOLDING is
			-- convert flow value to holding
		require
			a_flow_not_void: a_flow /= Void
		do
			if a_flow.is_portfolio_debit_flow (portfolio) then
				create Result.make (-a_flow.flow_value, holding_control)
			else
				create Result.make (a_flow.flow_value, holding_control)
			end
		ensure
			non_void_result: Result /= Void
		end

	as_holding (a_flow: RP_FLOW): RP_HOLDING is
			-- convert a flow to holding for valuation
		require
			a_flow_not_void: a_flow /= Void
		do
			if a_flow.is_portfolio_debit_flow (portfolio) then
				create Result.make (-a_flow.quantity, holding_control)
			else
				create Result.make (a_flow.quantity, holding_control)
			end
		ensure
			non_void_result: Result /= Void
		end

	value_of (a_bucket: STRING; a_holding: RP_HOLDING): RP_MONIES is
			-- value of `a_holding'
		require
			non_void_args: a_bucket /= Void and a_holding /= Void
		do
			Result := valuator.value_in_currency (a_holding, bucket.currency, bucket.perf_date, a_bucket)
		ensure
			non_void_result: Result /= Void
		end

	is_not_zero (d: RP_DECIMAL): BOOLEAN is
			-- Is `d' not zero?
		do
			Result := not d.is_zero
		end

feature {NONE} -- Basic Operations

	process (v: RP_FLOW) is
			-- Process `v'
		do
			v.process (Current)
		end

feature {RP_FLOW_TYPE} -- Processing

	process_commission (v: RP_FLOW) is
			-- <Precursor>
		do
			if attached {RP_TRADE} v.contract as l_trade then
				if not l_trade.traded_quantity.instrument.is_future then
					if l_trade.is_buy or l_trade.is_cover then
						add_to_commtaxbuy (v)
						add_to_withindaybuy ("WDBUY_CO", convert_flow_value_to_holding (v))
					end
				elseif l_trade.is_buy or l_trade.is_cover then
					add_to_commtaxbuy (v)
					substract_from_buycost (v)
				elseif l_trade.is_sell or l_trade.is_short then
					add_to_sellnet (v)
				end
			end
			if attached {RP_INLIEU} v.contract as l_inlieu then
				add_to_sellnet2 (v)
			end
		end

	process_tax (v: RP_FLOW) is
			-- <Precursor>
		do
		end

	process_exchange_fee (v: RP_FLOW) is
			-- <Precursor>
		do
			if not v.quantity.instrument.is_future then
				if attached {RP_TRADE} v.contract as l_trade then
					if l_trade.is_buy then
						add_to_commtaxbuy (v)
						add_to_withindaybuy ("WDBUY_FE", convert_flow_value_to_holding (v))
					end
					if l_trade.is_cover then
						add_to_commtaxbuy (v)
						add_to_withindaybuy ("WDBUY_FE", convert_flow_value_to_holding (v))
					end
				end
			end
			if attached {RP_INLIEU} v.contract as l_inlieu then
				add_to_sellnet2 (v)
			end
		end

	process_stamp_duty (v: RP_FLOW) is
			-- <Precursor>
		do
		end

	process_borrow (v: RP_FLOW) is
			-- <Precursor>
		do
		end

	process_return (v: RP_FLOW) is
			-- <Precursor>
		do
		end

	process_transfer (v: RP_FLOW) is
			-- <Precursor>
		do
			add_to_transfer (v)
		end

	process_interest (v: RP_FLOW) is
			-- <Precursor>
		do
		end

	process_income (v: RP_FLOW) is
			-- <Precursor>
		do
			add_to_income (v)
		end

	process_income_adjustment (v: RP_FLOW) is
			-- <Precursor>
		do
			add_to_income (v)
		end

	process_income_unrelated (v: RP_FLOW) is
			-- <Precursor>
		do
			add_to_perf_unrelated_income (v)
		end

	process_income_unrelated_adjustment (v: RP_FLOW) is
			-- <Precursor>
		do
			add_to_perf_unrelated_income (v)
		end

	process_expense (v: RP_FLOW) is
			-- <Precursor>
		do
			add_to_expense (v)
		end

	process_expense_adjustment (v: RP_FLOW) is
			-- <Precursor>
		do
			add_to_expense (v)
		end

	process_expense_unrelated (v: RP_FLOW) is
			-- <Precursor>
		do
			add_to_perf_unrelated_expense (v)
		end

	process_expense_unrelated_adjustment (v: RP_FLOW) is
			-- <Precursor>
		do
			add_to_perf_unrelated_expense (v)
		end

	process_ffx_gain_loss (v: RP_FLOW) is
			-- <Precursor>
		do
			add_to_realized_hedge_gain_loss (v)
		end

	process_dividend (v: RP_FLOW) is
			-- <Precursor>
		do
			add_to_dividend (v)
		end

	process_dividend_withholding_country_tax (v: RP_FLOW) is
			-- <Precursor>
		do
			add_to_dividend_tax_withholding (v)
		end

	process_dividend_withholding_nrs_tax (v: RP_FLOW) is
			-- <Precursor>
		do
		end

	process_dividend_tax_refund (v: RP_FLOW) is
			-- <Precursor>
		do
			add_to_dividend_tax_withholding (v)
		end

	process_tender (v: RP_FLOW) is
			-- <Precursor>
		do
			if v.quantity.instrument.is_currency then
				add_to_sellnet2 (v)
			end
		end

	process_distribution (v: RP_FLOW) is
			-- <Precursor>
		do
			if v.quantity.instrument.is_currency then
				add_to_sellnet2 (v)
			end
		end

	process_split (v: RP_FLOW) is
			-- <Precursor>
		do
		end

	process_delivery (v: RP_FLOW) is
			-- <Precursor>
		do
		end

	process_stock_dividend (v: RP_FLOW) is
			-- <Precursor>
		do
		end

	process_inkind (v: RP_FLOW) is
			-- <Precursor>
		do
			add_to_transfer (v)
			if not v.quantity.instrument.is_future then
				add_to_withindaybuy2 (v)
			end
		end

	process_outkind (v: RP_FLOW) is
			-- <Precursor>
		do
			add_to_transfer (v)
		end

	process_mark_to_market (v: RP_FLOW) is
			-- <Precursor>
		do
		end

	process_rights_exercise_money (v: RP_FLOW) is
			-- <Precursor>
		do
		end

	process_rights_exercise_quantity (v: RP_FLOW) is
			-- <Precursor>
		do
			if not v.quantity.instrument.is_future then
				add_to_withindaybuy ("WDBUY_RTPR", as_holding (v))
				add_to_withindaybuy ("WDBUY_RTVA", create {RP_HOLDING}.make (-v.flow_value.abs, holding_control))
				substract_from_buycost (v)
			end
		end

	process_dividend_reinvestment_money (v: RP_FLOW) is
			-- <Precursor>
		do
		end

	process_dividend_reinvestment_quantity (v: RP_FLOW) is
			-- <Precursor>
		do
			if not v.quantity.instrument.is_future then
				add_to_withindaybuy ("WDBUY_DRPR", as_holding (v))
				add_to_withindaybuy ("WDBUY_DRVA", convert_flow_value_to_holding (v))
				add_to_buycost (v)
			end
		end

	process_ffx_open (v: RP_FLOW) is
			-- <Precursor>
		do
		end

	process_ffx_close (v: RP_FLOW) is
			-- <Precursor>
		do
		end

	process_withdrawal (v: RP_FLOW) is
			-- <Precursor>
		do
			add_to_withdrawal (v)
		end

	process_contribution (v: RP_FLOW) is
			-- <Precursor>
		do
			add_to_contribution (v)
		end

	process_fungible_conversion (v: RP_FLOW) is
			-- <Precursor>
		do
		end

	process_quantity_intraday_sell (v: RP_FLOW) is
			-- <Precursor>
		do
		end

	process_gross_monies_intraday_sell (v: RP_FLOW) is
			-- <Precursor>
		do
		end

	process_fixed_deposit_transfer (v: RP_FLOW) is
			-- <Precursor>
		do
		end

	process_redenomination (v: RP_FLOW) is
			-- <Precursor>
		do
		end

	process_quantity (v: RP_FLOW) is
			-- <Precursor>
		do
			if not v.quantity.instrument.is_future then
				if attached {RP_TRADE} v.contract as l_trade then
					if l_trade.is_buy or l_trade.is_cover then
						add_to_withindaybuy ("WDBUY_PR", as_holding (v))
						add_to_withindaybuy ("WDBUY_VA", convert_flow_value_to_holding (v))
						add_to_buycost (v)
					end
					if l_trade.is_sell or l_trade.is_short then
						add_to_sellnet (v)
					end
				end
			end
		end

	process_gross_monies (v: RP_FLOW) is
			-- <Precursor>
		do
			if attached {RP_INLIEU} v.contract as l_inlieu then
				add_to_sellnet2 (v)
			end
		end

	process_debt_reduction (v: RP_FLOW) is
			-- <Precursor>
		do
		end

	process_debt_payment (v: RP_FLOW) is
			-- <Precursor>
		do
		end

	process_stock_bonus (v: RP_FLOW) is
			-- <Precursor>
		do
		end

	process_intrazonal_withdrawal (v: RP_FLOW) is
			-- <Precursor>
		do
			add_to_intrazonal_withdrawal (v)
		end

	process_intrazonal_contribution (v: RP_FLOW) is
			-- <Precursor>
		do
			add_to_intrazonal_contribution (v)
		end

end
