indexing

	description:

		"Daily Values used to compute performance"

	date: "$Date: 2010/04/13 $"
	sccid: "$Id: //rose/source/kernel/expert_system/perf/rp_performance_daily_values.e#17 $"
	keywords: "perf,daily values"

class RP_PERFORMANCE_DAILY_VALUES

create

	make

feature {RP_PERFORMANCE_DAILY_VALUES} -- Initialization

	make (a_perf_date: like perf_date; a_currency: like currency) is
			-- Initialize `Current'.
		require
			non_void_args: a_perf_date /= Void and a_currency /= Void
		do
			perf_date := a_perf_date
			currency := a_currency
			portfolio_value := a_currency.monies (0)
			previous_portfolio_value := a_currency.monies (0)
			equity_value := a_currency.monies (0)
			cash_value := a_currency.monies (0)
			income := a_currency.monies (0)
			expense := a_currency.monies (0)
			perf_unrelated_expense := a_currency.monies (0)
			perf_unrelated_income := a_currency.monies (0)
			dividend := a_currency.monies (0)
			dividend_tax_withholding := a_currency.monies (0)
			withdrawal := a_currency.monies (0)
			contribution := a_currency.monies (0)
			intrazonal_withdrawal := a_currency.monies (0)
			intrazonal_contribution := a_currency.monies (0)
			transfer := a_currency.monies (0)
			withindaybuy := a_currency.monies (0)
			withindaybuy2 := a_currency.monies (0)
			sellnet := a_currency.monies (0)
			sellnet2 := a_currency.monies (0)
			buycost := a_currency.monies (0)
			commtaxbuy := a_currency.monies (0)
			realized_hedge_gain_loss := a_currency.monies (0)
			unrealized_hedge_contract_value := a_currency.monies (0)
			daily_change_of_hedge_gain_loss := a_currency.monies (0)
			share_poolfund_expense := a_currency.monies (0)
			share_poolfund_income := a_currency.monies (0)
			fees := a_currency.monies (0)
		end

feature -- Access

	perf_date: ROSE_DATE
			-- Date

	currency: RP_CURRENCY
			-- Currency of all values

	portfolio_value: RP_MONIES
			-- Portfolio value

	cash_value: RP_MONIES
			-- Cash

	perf_unrelated_expense: RP_MONIES
			-- Unrelated expenses

	perf_unrelated_income: RP_MONIES
			-- Unrelated income

	dividend_tax_withholding: RP_MONIES
			-- Dividend withholding tax

	withdrawal: RP_MONIES
			-- Client withdrawals

	contribution: RP_MONIES
			-- Client contributions

	intrazonal_withdrawal: RP_MONIES
			-- Intrazonal withdrawals

	intrazonal_contribution: RP_MONIES
			-- Intrazonal contributions

	transfer: RP_MONIES
			-- In/out kinds

	realized_hedge_gain_loss: RP_MONIES
			-- Realized hedging g&l

	unrealized_hedge_contract_value: RP_MONIES
			-- Unrealized hedge contract value

	daily_change_of_hedge_gain_loss: RP_MONIES
			-- daily fluctuation due to hedge

	share_poolfund_expense: RP_MONIES
			-- Unrelated expenses

	share_poolfund_income: RP_MONIES
			-- Unrelated income

	previous_portfolio_value: RP_MONIES
			-- Previous portfolio value used in daily return calculation

	equity_value: RP_MONIES
			-- equity only value

	withindaybuy: RP_MONIES
			-- withindaybuy

	withindaybuy2: RP_MONIES
			-- withindaybuy2

	commtaxbuy: RP_MONIES
			-- commission, tax on buy

	buycost: RP_MONIES
			-- buy cost

	sellnet: RP_MONIES
			-- net sale

	sellnet2: RP_MONIES
			-- sell net2

	dividend: RP_MONIES
			-- dividend

	expense: RP_MONIES
			-- expense

	income: RP_MONIES
			-- income

	fees: RP_MONIES
			-- fees

feature -- Change

	set_previous_portfolio_value (v: like previous_portfolio_value) is
			-- Set value of `previous_portfolio_value' to `v'
		require
			non_void_v: v /= Void
			same_currency: v.currency.is_equal (currency)
		do
			previous_portfolio_value := v
		ensure
			previous_portfolio_value_set: previous_portfolio_value = v
		end

	set_portfolio_value (v: like portfolio_value) is
			-- Set value of `portfolio_value' to `v'
		require
			non_void_v: v /= Void
			same_currency: v.currency.is_equal (currency)
		do
			portfolio_value := v
			set_previous_portfolio_value (v)
		ensure
			portfolio_value_set: portfolio_value = v
		end

	set_equity_value (v: like equity_value) is
			-- Set value of `equity_value' to `v'
		require
			non_void_v: v /= Void
			same_currency: v.currency.is_equal (currency)
		do
			equity_value := v
		ensure
			equity_value_set: equity_value = v
		end

	set_cash_value (v: like cash_value) is
			-- Set value of `cash_value' to `v'
		require
			non_void_v: v /= Void
			same_currency: v.currency.is_equal (currency)
		do
			cash_value := v
		ensure
			cash_value_set: cash_value = v
		end

	set_expense (v: like expense) is
			-- Set value of `expense' to `v'
		require
			non_void_v: v /= Void
			same_currency: v.currency.is_equal (currency)
		do
			expense := v
		ensure
			expense_set: expense = v
		end

	set_income (v: like income) is
			-- Set value of `income' to `v'
		require
			non_void_v: v /= Void
			same_currency: v.currency.is_equal (currency)
		do
			income := v
		ensure
			income_set: income = v
		end

	set_perf_unrelated_expense (v: like perf_unrelated_expense) is
			-- Set value of `perf_unrelated_expense' to `v'
		require
			non_void_v: v /= Void
			same_currency: v.currency.is_equal (currency)
		do
			perf_unrelated_expense := v
		ensure
			perf_unrelated_expense_set: perf_unrelated_expense = v
		end

	set_perf_unrelated_income (v: like perf_unrelated_income) is
			-- Set value of `perf_unrelated_income' to `v'
		require
			non_void_v: v /= Void
			same_currency: v.currency.is_equal (currency)
		do
			perf_unrelated_income := v
		ensure
			perf_unrelated_income_set: perf_unrelated_income = v
		end

	set_dividend (v: like dividend) is
			-- Set value of `dividend' to `v'
		require
			non_void_v: v /= Void
			same_currency: v.currency.is_equal (currency)
		do
			dividend := v
		ensure
			dividend_set: dividend = v
		end

	set_dividend_tax_withholding (v: like dividend_tax_withholding) is
			-- Set value of `dividend_tax_withholding' to `v'
		require
			non_void_v: v /= Void
			same_currency: v.currency.is_equal (currency)
		do
			dividend_tax_withholding := v
		ensure
			dividend_tax_withholding_set: dividend_tax_withholding = v
		end

	set_withdrawal (v: like withdrawal) is
			-- Set value of `withdrawal' to `v'
		require
			non_void_v: v /= Void
			same_currency: v.currency.is_equal (currency)
		do
			withdrawal := v
		ensure
			withdrawal_set: withdrawal = v
		end

	set_contribution (v: like contribution) is
			-- Set value of `contribution' to `v'
		require
			non_void_v: v /= Void
			same_currency: v.currency.is_equal (currency)
		do
			contribution := v
		ensure
			contribution_set: contribution = v
		end

	set_intrazonal_withdrawal (v: like intrazonal_withdrawal) is
			-- Set value of `intrazonal_withdrawal' to `v'
		require
			non_void_v: v /= Void
			same_currency: v.currency.is_equal (currency)
		do
			intrazonal_withdrawal := v
		ensure
			intrazonal_withdrawal_set: intrazonal_withdrawal = v
		end

	set_intrazonal_contribution (v: like intrazonal_contribution) is
			-- Set value of `contribution' to `v'
		require
			non_void_v: v /= Void
			same_currency: v.currency.is_equal (currency)
		do
			intrazonal_contribution := v
		ensure
			intrazonal_contribution_set: intrazonal_contribution = v
		end

	set_transfer (v: like transfer) is
			-- Set value of `transfer' to `v'
		require
			non_void_v: v /= Void
			same_currency: v.currency.is_equal (currency)
		do
			transfer := v
		ensure
			transfer_set: transfer = v
		end

	set_withindaybuy (v: like withindaybuy) is
			-- Set value of `withindaybuy' to `v'
		require
			non_void_v: v /= Void
			same_currency: v.currency.is_equal (currency)
		do
			withindaybuy := v
		ensure
			withindaybuy_set: withindaybuy = v
		end

	set_withindaybuy2 (v: like withindaybuy2) is
			-- Set value of `withindaybuy2' to `v'
		require
			non_void_v: v /= Void
			same_currency: v.currency.is_equal (currency)
		do
			withindaybuy2 := v
		ensure
			withindaybuy2_set: withindaybuy2 = v
		end

	set_commtaxbuy (v: like commtaxbuy) is
			-- Set value of `commtaxbuy' to `v'
		require
			non_void_v: v /= Void
		do
			commtaxbuy := v
		ensure
			commtaxbuy_set: commtaxbuy = v
		end

	set_buycost (v: like buycost) is
			-- Set value of `buycost' to `v'
		require
			non_void_v: v /= Void
			same_currency: v.currency.is_equal (currency)
		do
			buycost := v
		ensure
			buycost_set: buycost = v
		end

	set_sellnet (v: like sellnet) is
			-- Set value of `sellnet' to `v'
		require
			non_void_v: v /= Void
			same_currency: v.currency.is_equal (currency)
		do
			sellnet := v
		ensure
			sellnet_set: sellnet = v
		end

	set_sellnet2 (v: like sellnet2) is
			-- Set value of `sellnet2' to `v'
		require
			non_void_v: v /= Void
			same_currency: v.currency.is_equal (currency)
		do
			sellnet2 := v
		ensure
			sellnet2_set: sellnet2 = v
		end

	set_realized_hedge_gain_loss (v: like realized_hedge_gain_loss) is
			-- Set value of `realized_hedge_gain_loss' to `v'
		require
			non_void_v: v /= Void
			same_currency: v.currency.is_equal (currency)
		do
			realized_hedge_gain_loss := v
		ensure
			realized_hedge_gain_loss_set: realized_hedge_gain_loss = v
		end

	set_unrealized_hedge_contract_value (v: like unrealized_hedge_contract_value) is
			-- Set value of `unrealized_hedge_contract_value' to `v'
		require
			non_void_v: v /= Void
			same_currency: v.currency.is_equal (currency)
		do
			unrealized_hedge_contract_value := v
		ensure
			unrealized_hedge_contract_value_set: unrealized_hedge_contract_value = v
		end

	set_daily_change_of_hedge_gain_loss (v: like daily_change_of_hedge_gain_loss) is
			-- Set value of `daily_change_of_hedge_gain_loss' to `v'
		require
			non_void_v: v /= Void
			same_currency: v.currency.is_equal (currency)
		do
			daily_change_of_hedge_gain_loss := v
		ensure
			daily_change_of_hedge_gain_loss_set: daily_change_of_hedge_gain_loss = v
		end

	set_share_poolfund_expense (v: like share_poolfund_expense) is
			-- Set value of `share_poolfund_expense' to `v'
		require
			non_void_v: v /= Void
			same_currency: v.currency.is_equal (currency)
		do
			share_poolfund_expense := v
		ensure
			share_poolfund_expense_set: share_poolfund_expense = v
		end

	set_share_poolfund_income (v: like share_poolfund_income) is
			-- Set value of `share_poolfund_income' to `v'
		require
			non_void_v: v /= Void
			same_currency: v.currency.is_equal (currency)
		do
			share_poolfund_income := v
		ensure
			share_poolfund_income_set: share_poolfund_income = v
		end

	set_fees (v: like fees) is
			-- Set value of `fees' to `v'
		require
			non_void_v: v /= Void
			same_currency: v.currency.is_equal (currency)
		do
			fees := v
		ensure
			fees_set: fees = v
		end

feature -- Basic Operations

	in_currency (a_currency: RP_CURRENCY; a_fx: RP_CURRENCY_MARKET_DATA_SOURCE): RP_PERFORMANCE_DAILY_VALUES is
			-- Translate to `a_currency' using `a_fx'
		local
			l_rate: RP_DECIMAL
		do
			l_rate := a_fx.spot_rate (currency, a_currency, perf_date)
			create Result.make (perf_date, a_currency)
			Result.set_portfolio_value (a_currency.monies (portfolio_value.amount * l_rate))
			Result.set_equity_value (a_currency.monies (equity_value.amount * l_rate))
			Result.set_cash_value (a_currency.monies (cash_value.amount * l_rate))
			Result.set_expense (a_currency.monies (expense.amount * l_rate))
			Result.set_income (a_currency.monies (income.amount * l_rate))
			Result.set_perf_unrelated_expense (a_currency.monies (perf_unrelated_expense.amount * l_rate))
			Result.set_perf_unrelated_income (a_currency.monies (perf_unrelated_income.amount * l_rate))
			Result.set_dividend (a_currency.monies (dividend.amount * l_rate))
			Result.set_dividend_tax_withholding (a_currency.monies (dividend_tax_withholding.amount * l_rate))
			Result.set_withdrawal (a_currency.monies (withdrawal.amount * l_rate))
			Result.set_contribution (a_currency.monies (contribution.amount * l_rate))
			Result.set_intrazonal_withdrawal (a_currency.monies (intrazonal_withdrawal.amount * l_rate))
			Result.set_intrazonal_contribution (a_currency.monies (intrazonal_contribution.amount * l_rate))
			Result.set_transfer (a_currency.monies (transfer.amount * l_rate))
			Result.set_withindaybuy (a_currency.monies (withindaybuy.amount * l_rate))
			Result.set_withindaybuy2 (a_currency.monies (withindaybuy2.amount * l_rate))
			Result.set_commtaxbuy (a_currency.monies (commtaxbuy.amount * l_rate))
			Result.set_buycost (a_currency.monies (buycost.amount * l_rate))
			Result.set_sellnet (a_currency.monies (sellnet.amount * l_rate))
			Result.set_sellnet2 (a_currency.monies (sellnet2.amount * l_rate))
			Result.set_realized_hedge_gain_loss (a_currency.monies (realized_hedge_gain_loss.amount * l_rate))
			Result.set_share_poolfund_expense (a_currency.monies (share_poolfund_expense.amount * l_rate))
			Result.set_share_poolfund_income (a_currency.monies (share_poolfund_income.amount * l_rate))
			Result.set_fees (a_currency.monies (fees.amount * l_rate))
		end

invariant

	non_void_attributes: perf_date /= Void and currency /= Void
	portfolio_value: portfolio_value /= Void
	cash_value: cash_value /= Void
	perf_unrelated_expense: perf_unrelated_expense /= Void
	perf_unrelated_income: perf_unrelated_income /= Void
	dividend_tax_withholding: dividend_tax_withholding /= Void
	withdrawal: withdrawal /= Void
	contribution: contribution /= Void
	intrazonal_withdrawal: intrazonal_withdrawal /= Void
	intrazonal_contribution: intrazonal_contribution /= Void
	transfer: transfer /= Void
	realized_hedge_gain_loss: realized_hedge_gain_loss /= Void
	share_poolfund_expense: share_poolfund_expense /= Void
	share_poolfund_income: share_poolfund_income /= Void
	previous_portfolio_value: previous_portfolio_value /= Void
	equity_value: equity_value /= Void
	withindaybuy: withindaybuy /= Void
	withindaybuy2: withindaybuy2 /= Void
	commtaxbuy: commtaxbuy /= Void
	buycost: buycost /= Void
	sellnet: sellnet /= Void
	sellnet2: sellnet2 /= Void
	dividend: dividend /= Void
	expense: expense /= Void
	income: income /= Void
	fees: fees /= Void

end
