note

	description:

		"Database queries for DailyValues"

	author: "Ivan Tarnavsky"
	last_editor: "$Author: it $"
	date: "$Date: 2011/02/23 $"
	revision: "$Revision: #1 $"
	path: "$File: //rose/source/kernel/expert_system/perf/rp_performance_daily_values_query.e $"

class RP_PERFORMANCE_DAILY_VALUES_QUERY

create

	make

feature {NONE} -- Initialization

	make (a_dailyvalues_connection: like dailyvalues_connection) is
			-- Initialize `Current'
		require
			a_dailyvalues_connection_not_void: a_dailyvalues_connection /= Void
		do
			dailyvalues_connection := a_dailyvalues_connection
		ensure
			dailyvalues_connection_set: dailyvalues_connection = a_dailyvalues_connection
		end

feature -- Query

	nperf_dailyvalues (a_portfolio: RP_PORTFOLIO): ARRAY [RP_PERFORMANCE_DAILY_VALUES] is
			-- nperf dailyvalues for `a_portfolio'
		require
			a_portfolio_not_void: a_portfolio /= Void
		do
			Result := nperf_dailyvalues_on_and_after (a_portfolio, min_date)
		ensure
			nperf_dailyvalues_not_void: Result /= Void
			no_void_item_in_nperf_dailyvalues: not Result.has (Void)
		end

	nperf_dailyvalues_for_account (a_portfolio: RP_PORTFOLIO): ARRAY [RP_PERFORMANCE_DAILY_VALUES] is
			-- Query nperf dailyvalues
		require
			a_portfolio_not_void: a_portfolio /= Void
		do
			Result := nperf_dailyvalues_for_account_on_and_after (a_portfolio, min_date)
		ensure
			nperf_dailyvalues_for_account_not_void: Result /= Void
			no_void_item_in_nperf_dailyvalues_for_account: not Result.has (Void)
		end

	vms_dailyvalues (a_portfolio: RP_PORTFOLIO): ARRAY [RP_PERFORMANCE_DAILY_VALUES] is
			-- Query vms dailyvalues
		require
			a_portfolio_not_void: a_portfolio /= Void
		do
			Result := vms_dailyvalues_on_and_before (a_portfolio, max_date)
		ensure
			vms_dailyvalues_not_void: Result /= Void
			no_void_item_in_vms_dailyvalues_for_account: not Result.has (Void)
		end

	vms_dailyvalues_for_account (a_portfolio: RP_PORTFOLIO): ARRAY [RP_PERFORMANCE_DAILY_VALUES] is
			-- Query vms dailyvalues
		require
			a_portfolio_not_void: a_portfolio /= Void
		do
			Result := vms_dailyvalues_for_account_on_and_before (a_portfolio, max_date)
		ensure
			vms_dailyvalues_for_account_not_void: Result /= Void
			no_void_item_in_vms_dailyvalues_for_account: not Result.has (Void)
		end

	nperf_dailyvalues_on_and_after (a_portfolio: RP_PORTFOLIO; a_date: ROSE_DATE): ARRAY [RP_PERFORMANCE_DAILY_VALUES] is
			-- Query nperf dailyvalues
		require
			a_portfolio_not_void: a_portfolio /= Void
			a_date_not_void: a_date /= Void
		do
			Result := retrieve (a_portfolio, query_nperf_dailyvalues (a_portfolio, a_date))
		ensure
			nperf_dailyvalues_on_and_after_not_void: Result /= Void
			no_void_item_in_nperf_dailyvalues_on_and_after: not Result.has (Void)
		end

	nperf_dailyvalues_for_account_on_and_after (a_portfolio: RP_PORTFOLIO; a_date: ROSE_DATE): ARRAY [RP_PERFORMANCE_DAILY_VALUES] is
			-- Query nperf dailyvalues
		require
			a_portfolio_not_void: a_portfolio /= Void
			a_date_not_void: a_date /= Void
		do
			Result := retrieve (a_portfolio, query_nperf_dailyvalues_for_account (a_portfolio, a_date))
		ensure
			nperf_dailyvalues_for_account_on_and_after_not_void: Result /= Void
			no_void_item_in_nperf_dailyvalues_for_account_on_and_after: not Result.has (Void)
		end

	vms_dailyvalues_on_and_before (a_portfolio: RP_PORTFOLIO; a_date: ROSE_DATE): ARRAY [RP_PERFORMANCE_DAILY_VALUES] is
			-- Query vms dailyvalues
		require
			a_portfolio_not_void: a_portfolio /= Void
			a_date_not_void: a_date /= Void
		do
			Result := retrieve (a_portfolio, query_legacy_dailyvalues (a_portfolio, a_date))
		ensure
			vms_dailyvalues_on_and_before_not_void: Result /= Void
			no_void_item_in_vms_dailyvalues_on_and_before: not Result.has (Void)
		end

	vms_dailyvalues_for_account_on_and_before (a_portfolio: RP_PORTFOLIO; a_date: ROSE_DATE): ARRAY [RP_PERFORMANCE_DAILY_VALUES] is
			-- Query vms dailyvalues
		require
			a_portfolio_not_void: a_portfolio /= Void
			a_date_not_void: a_date /= Void
		do
			Result := retrieve (a_portfolio, query_legacy_dailyvalues_for_account (a_portfolio, a_date))
		ensure
			vms_dailyvalues_for_account_on_and_before_not_void: Result /= Void
			no_void_item_in_vms_dailyvalues_for_account_on_and_before: not Result.has (Void)
		end

feature {NONE} -- Implementation

	dailyvalues_connection: SQL_DATA_SERVER
			-- Connection to the DailyValues database

	query_nperf_dailyvalues (a_portfolio: RP_PORTFOLIO; a_date: ROSE_DATE): STRING_PREDICATE is
			-- Query nperf dailyvalues
		do
			create Result.make (dailyvalues_connection, "SELECT * FROM dbo.eiffel_dailyperf_dailyvalues ('" + a_portfolio.zone.code + "','" + a_portfolio.code + "','" + a_portfolio.zone.legacy_base_currency.name + "'," + a_date.ccyymmdd.out + ")")
		end

	query_nperf_dailyvalues_for_account (a_portfolio: RP_PORTFOLIO; a_date: ROSE_DATE): STRING_PREDICATE is
			-- Query nperf dailyvalues
		do
			create Result.make (dailyvalues_connection, "SELECT * FROM dbo.eiffel_dailyperf_dailyvalues_for_account ('" + a_portfolio.zone.code + "','" + a_portfolio.code + "','" + a_portfolio.zone.legacy_base_currency.name + "'," + a_date.ccyymmdd.out + ")")
		end

	query_legacy_dailyvalues (a_portfolio: RP_PORTFOLIO; a_date: ROSE_DATE): STRING_PREDICATE is
			-- Query vms dailyvalues
		do
			create Result.make (dailyvalues_connection, "SELECT * FROM dbo.vms_dailyperf_dailyvalues ('" + a_portfolio.zone.code + "','" + a_portfolio.code + "','" + a_portfolio.zone.legacy_base_currency.name + "'," + a_date.ccyymmdd.out + ")")
		end

	query_legacy_dailyvalues_for_account (a_portfolio: RP_PORTFOLIO; a_date: ROSE_DATE): STRING_PREDICATE is
			-- Query vms dailyvalues
		do
			create Result.make (dailyvalues_connection, "SELECT * FROM dbo.vms_dailyperf_dailyvalues_for_account ('" + a_portfolio.zone.code + "','" + a_portfolio.code + "','" + a_portfolio.zone.legacy_base_currency.name + "'," + a_date.ccyymmdd.out + ")")
		end

	daily_values_from_pipe_delim_string (a_portfolio: RP_PORTFOLIO; a_row: STRING): RP_PERFORMANCE_DAILY_VALUES is
			-- DailyValues from pipe delimited string
		require
			a_portfolio_not_void: a_portfolio /= Void
			a_row_not_void: a_row /= Void
		local
			l_tokenizer: ROSE_DELIMITED_TEXT
			l_currency: RP_CURRENCY
		do
			l_currency := a_portfolio.zone.legacy_base_currency
			create l_tokenizer.make (a_row, '|')
			create Result.make (create {ROSE_DATE}.make_ccyymmdd (l_tokenizer.item (1).to_integer), l_currency)
			Result.set_portfolio_value (l_currency.monies (l_tokenizer.item (2)))
			Result.set_cash_value (l_currency.monies (l_tokenizer.item (3)))
			Result.set_perf_unrelated_expense (l_currency.monies (l_tokenizer.item (4)))
			Result.set_perf_unrelated_income (l_currency.monies (l_tokenizer.item (5)))
			Result.set_dividend_tax_withholding (l_currency.monies (l_tokenizer.item (6)))
			Result.set_withdrawal (l_currency.monies (l_tokenizer.item (7)))
			Result.set_contribution (l_currency.monies (l_tokenizer.item (8)))
			Result.set_transfer (l_currency.monies (l_tokenizer.item (9)))
			Result.set_realized_hedge_gain_loss (l_currency.monies (l_tokenizer.item (10)))
			Result.set_daily_change_of_hedge_gain_loss (l_currency.monies (l_tokenizer.item (10)))
			Result.set_share_poolfund_expense (l_currency.monies (l_tokenizer.item (11)))
			Result.set_share_poolfund_income (l_currency.monies (l_tokenizer.item (12)))
			if l_tokenizer.count > 12 then
					-- equity only buckets
				Result.set_equity_value (l_currency.monies (l_tokenizer.item (13)))
				Result.set_withindaybuy (l_currency.monies (l_tokenizer.item (14)))
				Result.set_withindaybuy2 (l_currency.monies (l_tokenizer.item (15)))
				Result.set_commtaxbuy (-l_currency.monies (l_tokenizer.item (16)))
				Result.set_buycost (-l_currency.monies (l_tokenizer.item (17)))
				Result.set_sellnet (-l_currency.monies (l_tokenizer.item (18)))
				Result.set_sellnet2 (l_currency.monies (l_tokenizer.item (19)))
				Result.set_income (l_currency.monies (l_tokenizer.item (20)))
				Result.set_dividend (l_currency.monies (l_tokenizer.item (21)))
				Result.set_expense (l_currency.monies (l_tokenizer.item (22)))
			end
			if l_tokenizer.count > 22 then
					-- intrazonal cont/with
				Result.set_intrazonal_withdrawal (l_currency.monies (l_tokenizer.item (23)))
				Result.set_intrazonal_contribution (l_currency.monies (l_tokenizer.item (24)))
			end
		ensure
			dailyvalues_not_void: Result /= Void
		end

	retrieve (a_portfolio: RP_PORTFOLIO; a_query: STRING_PREDICATE): ARRAY [RP_PERFORMANCE_DAILY_VALUES] is
			-- Retrieve dDailyValues from `a_query'
		require
			a_portfolio_not_void: a_portfolio /= Void
			a_query_not_void: a_query /= Void
		local
			l_result: ROSE_LINEAR_ARRAY [RP_PERFORMANCE_DAILY_VALUES]
		do
			create l_result.make_empty
			Result := l_result
			a_query.data_server.begin_transaction
			a_query.retrieve (Void, Void)
			a_query.array.do_all (agent add_dailyvalues_from_string (a_portfolio, l_result, ?))
			a_query.data_server.commit
		ensure
			array_not_void: Result /= Void
			no_void_in_array: not Result.has (Void)
		end

	add_dailyvalues_from_string (a_portfolio: RP_PORTFOLIO; a_array: ROSE_LINEAR_ARRAY [RP_PERFORMANCE_DAILY_VALUES]; a_row: STRING) is
			-- Add pipedelmited dailyvalues from `a_row' to `a_array'
		require
			a_portfolio_not_void: a_portfolio /= Void
			a_array_not_void: a_array /= Void
			a_row_not_void: a_row /= Void
		do
			a_array.extend (daily_values_from_pipe_delim_string (a_portfolio, a_row))
		ensure
			new_dailyvalues_added: a_array.count = 1 + old a_array.count
			no_void_added: not a_array.has (Void)
		end

feature {NONE} -- Constants

	min_date: ROSE_DATE is
			-- Minimum possible date
		once
			create Result.make_min_date
		ensure
			min_date_not_void: min_date /= Void
		end

	max_date: ROSE_DATE is
			-- Maximum possible date
		once
			create Result.make_max_date
		ensure
			max_date_not_void: max_date /= Void
		end

invariant

	dailyvalues_connection_not_void: dailyvalues_connection /= Void

end
