indexing

	description:

		"E specific zone pricing algorithm"

	date: "$Date: 2010/04/30 $"
	fileid: "$Id: //rose/source/Kernel/expert_system/valuators/rp_performance_ezone_stock_pricing_algorithm.e#5 $"

class RP_PERFORMANCE_EZONE_STOCK_PRICING_ALGORITHM

inherit

	RP_PERFORMANCE_STOCK_PRICING_ALGORITHM
		redefine
			stock_market_price,
			retrieve_last_stock_price,
			price_lookback
		end

create

	make_with_trade_price

feature {NONE} -- Implementation

	stock_market_price (a_stock: RP_STOCK; a_date: ROSE_DATE; a_calendar: RP_CALENDAR): RP_PRICE is
			-- Retrieve market price for `a_listing' on `a_date'
		local
			l_listing_market_data: RP_MARKET_DATA_RECORD
			l_price: RP_PRICE
			l_stock: RP_STOCK
		do
			l_stock := a_stock
			l_price := search_price (l_stock, a_date)
			if l_price = Void then
					-- retrive price record
				l_listing_market_data := market_data_source.snapshot (l_stock, a_date)
				if l_listing_market_data /= Void and then is_valid (l_listing_market_data) then
						-- found and valid
					if l_listing_market_data.has_close or l_listing_market_data.has_bid or l_listing_market_data.has_ask then
						if l_listing_market_data.has_close then
							Result := l_listing_market_data.unadjusted_close
						elseif l_listing_market_data.has_bid then
							Result := l_listing_market_data.unadjusted_bid
						elseif l_listing_market_data.has_ask then
							Result := l_listing_market_data.unadjusted_ask
						end
					else
						Result := retrieve_last_stock_price (l_listing_market_data.adjustment_factor, l_stock, a_date, a_calendar)
					end
				elseif l_listing_market_data /= Void and then l_listing_market_data.has_adjustment_factor then
						-- found but has only adj factor
					Result := retrieve_last_stock_price (l_listing_market_data.adjustment_factor, l_stock, a_date, a_calendar)
				else
						-- lookback
					l_listing_market_data := price_lookback (market_data_source, l_stock, a_date, adjustment_factor_look_back_range, a_calendar, agent has_adjustment_factor (?))
					if l_listing_market_data /= Void then
						Result := retrieve_last_stock_price (l_listing_market_data.adjustment_factor, l_stock, a_date, a_calendar)
					end
				end
				save_price (l_stock, Result, a_date)
			else
				Result := l_price
			end
		end

	retrieve_last_stock_price (a_adjustment_factor: RP_DECIMAL; a_stock: RP_STOCK; a_date: ROSE_DATE; a_calendar: RP_CALENDAR): RP_PRICE is
			-- Retrieve last market price
		local
			l_listing_market_data: RP_MARKET_DATA_RECORD
		do
			l_listing_market_data := price_lookback (market_data_source, a_stock, a_date, lookback_range, a_calendar, agent has_any_price (?))
			if l_listing_market_data /= Void then
				if l_listing_market_data.has_close then
					Result := l_listing_market_data.unadjusted_close
				elseif l_listing_market_data.has_bid then
					Result := l_listing_market_data.unadjusted_bid
				elseif l_listing_market_data.has_ask then
					Result := l_listing_market_data.unadjusted_ask
				end
				if Result /= Void then
					if a_adjustment_factor /= l_listing_market_data.adjustment_factor then
						if l_listing_market_data.date < a_date and then l_listing_market_data.date.diff_in_days (a_date).abs <= lookback_range then
							if l_listing_market_data.adjustment_factor > 0.0 then
								Result := Result.instrument.price_in_currency (Result.price * a_adjustment_factor / l_listing_market_data.adjustment_factor, Result.currency, Result.date)
							end
						end
					end
				end
			end
		end

	price_lookback (a_mkt: RP_MARKET_DATA_SOURCE [RP_STOCK]; a_instrument: RP_STOCK; a_date: ROSE_DATE; a_max_days_to_go_back: INTEGER; a_calendar: RP_CALENDAR; a_query: FUNCTION [ANY, TUPLE, BOOLEAN]): RP_MARKET_DATA_RECORD is
			-- Price lookback
		local
			l_date: ROSE_DATE
			l_counter: INTEGER
			l_found: BOOLEAN
			l_result: RP_MARKET_DATA_RECORD
		do
			from
				create l_date.make_by_date (a_date)
				l_counter := 0
			until
				a_date.diff_in_days (l_date).abs > a_max_days_to_go_back or l_found
			loop
				l_result := a_mkt.snapshot (a_instrument, l_date)
				if l_result /= Void then
					if a_query = Void then
						l_found := True
					else
						l_found := a_query.item ([l_result])
					end
				end
				l_counter := l_counter + 1
					-- go back one market day
				l_date := a_calendar.ith_work_day_before (l_date, 1)
			end
			if l_found then
				Result := l_result
			end
		end

	adjustment_factor_look_back_range: INTEGER is
			-- look back for adjustement factor > 0
		do
			Result := 15
		end

end
