indexing

	description:

		"Index Futures pricing algorithm"

	date: "$Date: 2010/04/30 $"
	revision: "$Revision: #4 $"

class RP_PERFORMANCE_INDEX_FUTURE_PRICING_ALGORITHM

inherit

	RP_PRICING_ALGORITHM [RP_INDEX_FUTURE]

create

	make

feature {NONE} -- Initialization

	make (a_market_data: like market_data; a_trade_price: like trade_price) is
			-- Initialize `Current'
		require
			non_void_args: a_market_data /= Void and a_trade_price /= Void
		do
			market_data := a_market_data
			trade_price := a_trade_price
			create last_price_cache.make_equal (17)
		ensure
			market_data_set: market_data = a_market_data
			trade_price_set: trade_price = a_trade_price
		end

feature -- Query

	price (a_future: RP_INDEX_FUTURE; a_date: ROSE_DATE): RP_PRICE is
			-- Price of `a_future' on `a_date'
		local
			l_data: RP_MARKET_DATA_RECORD
		do
				-- Market price
			l_data := market_data.snapshot (a_future, a_date)
			if l_data /= Void and then l_data.has_close then
				Result := l_data.unadjusted_close
				if Result /= Void then
					last_price_cache.force (Result, a_future)
				end
			end
			if Result = Void then
					-- Lag price
				last_price_cache.search (a_future)
				if last_price_cache.found then
					Result := last_price_cache.found_item
				end
			end
			if Result = Void then
					-- Futures trade price is cost based
					-- Use as last resort
				l_data := trade_price.snapshot (a_future, a_date)
				if l_data /= Void then
					Result := l_data.unadjusted_close
					if Result /= Void then
						last_price_cache.force (Result, a_future)
					end
				end
			end
		end

feature {NONE} -- Implementation

	last_price_cache: UID_TABLE [RP_PRICE, RP_INSTRUMENT]
			-- Last computed price cache

	trade_price: RP_TRADE_PRICE_MARKET_DATA_SOURCE
			-- Trade price market data source

	market_data: RP_MARKET_DATA_SOURCE [RP_INDEX_FUTURE]
			-- Index future market data source

end
