indexing

	description:

		"JOHNC specific pricing algorithm"

	date: "$Date: 2010/04/30 $"
	fileid: "$Id: //rose/source/kernel/expert_system/valuators/rp_performance_johnc_stock_pricing_algorithm.e#6 $"

class RP_PERFORMANCE_JOHNC_STOCK_PRICING_ALGORITHM

inherit

	RP_PERFORMANCE_STOCK_PRICING_ALGORITHM
		redefine
			stock_market_price,
			lookback_range,
			retrieve_last_stock_price
		end

create

	make_with_trade_price

feature {NONE} -- Impementation

	lookback_range: INTEGER is 30
			-- Max days to look back.

	stock_market_price (a_instrument: RP_STOCK; a_date: ROSE_DATE; a_calendar: RP_CALENDAR): RP_PRICE is
			-- Retrieve market price for `a_listing' on `a_date'
		local
			l_listing_market_data: RP_MARKET_DATA_RECORD
		do
			l_listing_market_data := market_data_source.snapshot (a_instrument, a_date)
			if l_listing_market_data /= Void and then (l_listing_market_data.has_adjustment_factor and then l_listing_market_data.adjustment_factor > 0) then
				if ((l_listing_market_data.has_close and then l_listing_market_data.unadjusted_close.price > 0) or (l_listing_market_data.has_bid and then l_listing_market_data.unadjusted_bid.price > 0)) and (l_listing_market_data.has_volume and then l_listing_market_data.volume > 0) then
					if l_listing_market_data.has_close then
						Result := l_listing_market_data.unadjusted_close
					else
						Result := l_listing_market_data.unadjusted_bid
					end
				else
					if (l_listing_market_data.has_ask and then l_listing_market_data.unadjusted_ask.price > 0) and (l_listing_market_data.has_bid and then l_listing_market_data.unadjusted_bid.price > 0) then
						Result := a_instrument.price_in_currency ((l_listing_market_data.unadjusted_ask.price + l_listing_market_data.unadjusted_bid.price) / 2, l_listing_market_data.unadjusted_ask.currency, l_listing_market_data.unadjusted_ask.date)
					else
						Result := retrieve_last_stock_price (l_listing_market_data.adjustment_factor, a_instrument, a_date, a_calendar)
					end
				end
			else
				if l_listing_market_data /= Void and then (l_listing_market_data.has_adjustment_factor and then l_listing_market_data.adjustment_factor > 0) then
					l_listing_market_data := price_lookback (market_data_source, a_instrument, a_date, lookback_range, a_calendar, agent has_adjustment_factor (?))
					Result := retrieve_last_stock_price (l_listing_market_data.adjustment_factor, a_instrument, a_date, a_calendar)
				else
					l_listing_market_data := price_lookback (market_data_source, a_instrument, a_date, lookback_range, a_calendar, Void)
					if l_listing_market_data /= Void then
						Result := retrieve_last_stock_price (l_listing_market_data.adjustment_factor, a_instrument, a_date, a_calendar)
					end
				end
			end
		end

	retrieve_last_stock_price (a_adjustment_factor: RP_DECIMAL; a_instrument: RP_STOCK; a_date: ROSE_DATE; a_calendar: RP_CALENDAR): RP_PRICE is
			-- Retrieve last market price
		local
			l_listing_market_data: RP_MARKET_DATA_RECORD
		do
			l_listing_market_data := price_lookback (market_data_source, a_instrument, a_date, lookback_range, a_calendar, agent has_any_price (?))
			if l_listing_market_data /= Void then
				if l_listing_market_data.has_close then
					Result := l_listing_market_data.unadjusted_close
				elseif l_listing_market_data.has_bid then
					Result := l_listing_market_data.unadjusted_bid
				elseif l_listing_market_data.has_ask then
					Result := l_listing_market_data.unadjusted_ask
				else
					l_listing_market_data := price_lookback (market_data_source, a_instrument, a_date, lookback_range, a_calendar, agent has_close_or_bid (?))
					if l_listing_market_data /= Void then
						if l_listing_market_data.has_close then
							Result := l_listing_market_data.unadjusted_close
						else
							Result := l_listing_market_data.unadjusted_bid
						end
					end
				end
				if Result /= Void and a_adjustment_factor /= Void then
					if not a_adjustment_factor.is_equal (l_listing_market_data.adjustment_factor) then
						if l_listing_market_data.date < a_date and then l_listing_market_data.date.diff_in_days (a_date).abs <= lookback_range then
							if l_listing_market_data.adjustment_factor > 0 then
								if a_instrument.zone_mnemonic = 'U' or else a_date.ccyymmdd >= 20020101 then
									Result := Result.instrument.price_in_currency (Result.price * a_adjustment_factor / l_listing_market_data.adjustment_factor, Result.currency, Result.date)
								end
							end
						end
					end
				end
			end
		end

end
