indexing

	description:

		"S Zone specific Stock Pricing Algorithm"

	date: "$Date: 2010/04/30 $"
	fileid: "$Id: //rose/source/kernel/expert_system/valuators/rp_performance_szone_stock_pricing_algorithm.e#11 $"

class RP_PERFORMANCE_SZONE_STOCK_PRICING_ALGORITHM

inherit

	RP_PERFORMANCE_STOCK_PRICING_ALGORITHM
		redefine
			stock_market_price
		end

create

	make_with_trade_price

feature {NONE} -- Implementation

	stock_market_price (a_instrument: RP_STOCK; a_date: ROSE_DATE; a_calendar: RP_CALENDAR): RP_PRICE is
			-- Retrieve market price for `a_listing' on `a_date'
		local
			l_listing_market_data: RP_MARKET_DATA_RECORD
		do
			if a_instrument.is_adr then
				Result := Precursor (a_instrument, a_date, a_calendar)
			else
				l_listing_market_data := market_data_source.snapshot (a_instrument, a_date)
				if l_listing_market_data /= Void then
					if l_listing_market_data /= Void and then is_valid (l_listing_market_data) then
						if l_listing_market_data.has_close then
							Result := l_listing_market_data.unadjusted_close
						else
							Result := retrieve_last_price_for_szone (l_listing_market_data.adjustment_factor, a_instrument, a_date, a_calendar)
						end
					elseif l_listing_market_data /= Void and then l_listing_market_data.has_adjustment_factor then
						Result := retrieve_last_price_for_szone (l_listing_market_data.adjustment_factor, a_instrument, a_date, a_calendar)
					else
						l_listing_market_data := price_lookback (market_data_source, a_instrument, a_date, lookback_range, a_calendar, agent has_adjustment_factor (?))
						if l_listing_market_data /= Void then
							Result := retrieve_last_price_for_szone (l_listing_market_data.adjustment_factor, a_instrument, a_date, a_calendar)
						end
					end
				end
			end
		end

	retrieve_last_price_for_szone (a_adjustment_factor: RP_DECIMAL; a_instrument: RP_STOCK; a_date: ROSE_DATE; a_calendar: RP_CALENDAR): RP_PRICE is
			-- retrieve last Rose best prices.
		local
			l_listing_market_data: RP_MARKET_DATA_RECORD
		do
			l_listing_market_data := price_lookback (market_data_source, a_instrument, a_date, lookback_range, a_calendar, agent has_any_price (?))
			if l_listing_market_data /= Void then
				if l_listing_market_data.has_close then
					Result := l_listing_market_data.unadjusted_close
				end
				if Result = Void then
					l_listing_market_data := price_lookback (market_data_source, a_instrument, a_date, lookback_range, a_calendar, agent has_close (?))
					if l_listing_market_data /= Void then
						Result := l_listing_market_data.unadjusted_close
					end
				end
				if Result /= Void then
					if l_listing_market_data.date.diff_in_days (a_date).abs <= lookback_range then
						if l_listing_market_data.date < a_date and a_date.ccyymmdd >= 20020101 then
							if not l_listing_market_data.adjustment_factor.is_zero then
								Result := Result.instrument.price_in_currency (Result.price * l_listing_market_data.adjustment_factor / a_adjustment_factor, Result.currency, Result.date)
							end
						end
					end
				end
			end
		end

end
