indexing
	description:
		"Pooled Fund Instrument"

	author: "Mark Howard"
	last_editor: "$Author: e2 $"
	date: "$Date: 2010/02/22 $"
	revision: "$Revision: #19 $"
	path: "$File: //rose/source/kernel/expert_system/rp_pool_fund.e $"

class RP_POOL_FUND

inherit
	RP_INSTRUMENT_WITH_LISTING
		redefine
			uid,
			is_pool_fund
		end

	ROSE_UNITS_ENVIRONMENT
		export
			{NONE} all
		undefine
			is_equal
		end

create {RP_REFERENCE_SYSTEM}
	make

feature {NONE} -- Initialization

	make (a_listing: LISTING; a_brimsym: like brimsym; a_portfolio: like portfolio) is
			-- Initialize `Current'.
		require
			non_void_listing: a_listing /= Void
			non_void_brimsym: a_brimsym /= Void
			portfolio_may_be_void: True
		do
			uid := a_listing.uid
			rose_units := share_units (a_listing)
			name := "(" + a_brimsym + ")"
			brimsym := a_brimsym
			portfolio := a_portfolio
			initialize_local_brimsyms (a_listing)
		end

feature -- Access

	uid: INTEGER_64
			-- Unique identifier

	name: STRING
			-- Name.

	brimsym: STRING
			-- Brimsym.

	portfolio: RP_PORTFOLIO
			-- Underlying portfolio if any, Void if none

	holding_value_in_currency (a_holding: RP_HOLDING; a_valuator: RP_HOLDING_VALUATOR; a_currency: RP_CURRENCY; a_date: ROSE_DATE;a_bucket:STRING): RP_MONIES is
			-- Value of `a_holding' on `a_date' in `a_currency' as computed by `a_valuator'
		do
			Result := a_valuator.pool_fund_value_in_currency (a_holding, a_currency, a_date, a_bucket)
		end

	country_of_issue (a_ref: RP_REFERENCE_SYSTEM; a_date: ROSE_DATE): RP_COUNTRY
			-- Country of issue
		do
			Result := a_ref.country_from_iso_code (brimsym.substring (1, 2))
		end

	is_pool_fund: BOOLEAN is
			-- Is `Current' pool fund?
		do
			Result := True
		end

end
