indexing

	description:

		"Pool fund class information"

	last_editor: "$Author: nw $"
	date: "$Date: 2010/04/30 $"
	revision: "$Revision: #13 $"
	archive: "$File: //rose/source/kernel/expert_system/aflows/rp_pool_fund_class_data.e $"

class RP_POOL_FUND_CLASS_DATA

inherit

	RP_MARKET_DATA_RECORD

create

	make_from_cadb_record,
	make_from_cliodb_record

feature {NONE} -- Initialization

	make_from_cadb_record (a_ref: RP_REFERENCE_SYSTEM; a_record: RP_POOL_FUND_CLASS_DATA_CADB_RECORD) is
			-- Initialize `Current'.
		require
			non_void_args: a_record /= Void and a_ref /= Void
		local
			l_currency: RP_CURRENCY
		do
			create date.make_ccyymmdd (a_record.trade_date)
			if a_record.currency = Void then
				l_currency := a_ref.currency_from_iso_code ("USD")
			else
				l_currency := a_ref.currency_from_iso_code (a_record.currency)
			end
			if a_record.published_nav /= Void and then a_record.published_nav.is_defined and then a_record.published_nav > 0 then
				class_nav := l_currency.monies (a_record.published_nav)
			end
			if a_record.capital_balance /= Void and then a_record.capital_balance.is_defined then
				class_capital_balance := l_currency.monies (a_record.capital_balance)
			end
			if a_record.revenue_balance /= Void and then a_record.revenue_balance.is_defined then
				class_revenue_balance := l_currency.monies (a_record.revenue_balance)
			end
			if a_record.expense_balance /= Void and then a_record.expense_balance.is_defined then
				class_expense_balance := l_currency.monies (a_record.expense_balance)
			end
			if a_record.outstanding_shares /= Void and then a_record.outstanding_shares.is_defined and then a_record.outstanding_shares > 0 then
				class_shares := a_record.outstanding_shares
			end
			if a_record.share_activity /= Void and then a_record.share_activity.is_defined and then not a_record.share_activity.is_zero then
				share_activity := a_record.share_activity
			end
		end

	make_from_cliodb_record (a_ref: RP_REFERENCE_SYSTEM; a_record: RP_POOL_FUND_CLASS_DATA_CLIODB_RECORD) is
			-- Initialize `Current'.
		require
			non_void_args: a_record /= Void and a_ref /= Void
		local
			l_currency: RP_CURRENCY
		do
			create date.make_ccyymmdd (a_record.valuationdate.ccyymmdd)
			if a_record.quotecurrency = Void then
				l_currency := a_ref.currency_from_iso_code ("USD")
			else
				l_currency := a_ref.currency_from_iso_code (a_record.quotecurrency)
			end
			if a_record.navperunit /= Void and then a_record.navperunit.is_defined and then a_record.navperunit > 0 then
				class_nav := l_currency.monies (a_record.navperunit)
			end
			if a_record.unitsinissue /= Void and then a_record.unitsinissue.is_defined and then not a_record.unitsinissue.is_zero then
				class_shares := a_record.unitsinissue
			end
		end

feature -- Access

	date: ROSE_DATE
			-- Record date.

feature -- Query

	has (a_field: RP_MARKET_DATA_FIELD): BOOLEAN is
			-- Is `a_field' present?
		do
			Result := (a_field.is_civ_fund_class_nav and class_nav /= Void and then class_nav.amount.is_defined)
			Result := Result or else (a_field.is_civ_fund_class_capital_balance and class_capital_balance /= Void and then class_capital_balance.amount.is_defined)
			Result := Result or else (a_field.is_civ_fund_class_expense_balance and class_expense_balance /= Void and then class_expense_balance.amount.is_defined)
			Result := Result or else (a_field.is_civ_fund_class_revenue_balance and class_revenue_balance /= Void and then class_revenue_balance.amount.is_defined)
			Result := Result or else (a_field.is_civ_fund_class_shares and class_shares /= Void and then class_shares.is_defined)
			Result := Result or else (a_field.is_civ_fund_shares_activity and share_activity /= Void)
		end

	decimal_field (a_field: RP_MARKET_DATA_FIELD): RP_DECIMAL is
			-- Decimal value of `a_field'.
		do
			if a_field.is_civ_fund_class_shares then
				Result := class_shares
			elseif a_field.is_civ_fund_shares_activity then
				Result := share_activity
			end
		end

	monies_field (a_field: RP_MARKET_DATA_FIELD): RP_MONIES is
			-- Money value of `a_field'.
		do
			if a_field.is_civ_fund_class_nav then
				Result := class_nav
			elseif a_field.is_civ_fund_class_capital_balance then
				Result := class_capital_balance
			elseif a_field.is_civ_fund_class_expense_balance then
				Result := class_expense_balance
			elseif a_field.is_civ_fund_class_revenue_balance then
				Result := class_revenue_balance
			end
		end

	price_field (a_field: RP_MARKET_DATA_FIELD): RP_PRICE is
			-- Price of `a_field'.
		do
			Result := Void
				-- No price fields
		end

feature {NONE} -- Implementation

	class_nav: RP_MONIES
			-- Published NAV.

	class_capital_balance: RP_MONIES
			-- Capital balance.

	class_revenue_balance: RP_MONIES
			-- Revenue balance.

	class_expense_balance: RP_MONIES
			-- Expense balance.

	class_shares: RP_DECIMAL
			-- Outstanding shares.

	share_activity: RP_DECIMAL
			-- Share activity

end
