note

	description:

		"Market Data Source for Composite Assets Database"

	author: "nw"
	last_editor: "$Author: it $"
	date: "$Date: 2011/03/30 $"
	revision: "$Revision: #2 $"
	path: "$File: //rose/source/kernel/expert_system/mds/rp_pool_fund_data_source.e $"

deferred class RP_POOL_FUND_DATA_SOURCE

inherit

	RP_MARKET_DATA_SOURCE_LAST_RECORD_SEARCH [RP_POOL_FUND]

feature {NONE} -- Initialization

	make (a_ref: like reference_system) is
			-- Initialize `Current'
		require
			reference_system_attached: a_ref /= Void
		do
			reference_system := a_ref
			start_date := default_start_date
			create end_date.make_max_date
		ensure
			reference_system_set: reference_system = a_ref
			default_start_date: start_date ~ default_start_date
		end

	make_with_lookback (a_ref: like reference_system; a_max_lookback: INTEGER) is
			-- Initialize with maximum `a_max_lookback' lookback days
		require
			reference_system_attached: a_ref /= Void
			non_neg_lookback: a_max_lookback >= 0
		do
			make (a_ref)
			max_lookback_days := a_max_lookback
		ensure
			reference_system_set: reference_system = a_ref
			default_start_date: start_date ~ default_start_date
			max_lookback_days_set: max_lookback_days = a_max_lookback
		end

	make_with_restriction (a_ref: like reference_system; a_start, a_end: ROSE_DATE; a_max_lookback: INTEGER) is
			-- Initialize `Current' for a date range with maximum `a_max_lookback' lookback days
		require
			reference_system_attached: a_ref /= Void
			dates_attached: a_start /= Void and a_end /= Void
			start_date_lt_end_date: a_start < a_end
			non_neg_lookback: a_max_lookback >= 0
		do
			make_with_lookback (a_ref, a_max_lookback)
			create start_date.make_by_date (a_start)
			start_date.add_n_days (-a_max_lookback)
			create end_date.make_by_date (a_end)
		ensure
			reference_system_set: reference_system = a_ref
			max_lookback_days_set: max_lookback_days = a_max_lookback
			start_date_set: a_start.diff_in_days (start_date) = max_lookback_days
			end_date_set: end_date ~ a_end
		end

feature -- Basic operations

	snapshot (a_fund: RP_POOL_FUND; a_date: ROSE_DATE): RP_MARKET_DATA_RECORD is
			-- Market data for `a_fund' on `a_date'.
		do
			Result := pool_fund_data (a_fund, a_date)
		end

feature {NONE} -- Implementation

	reference_system: RP_REFERENCE_SYSTEM
			-- Reference system

	pool_fund_data (a_fund: RP_POOL_FUND; a_date: ROSE_DATE): RP_MARKET_DATA_RECORD is
			-- Market data for `a_fund' on `a_date'
		require
			a_fund_not_void: a_fund /= Void
			a_date_not_void: a_date /= Void
		deferred
		end

invariant

	reference_system_not_void: reference_system /= Void
	start_date_not_void: start_date /= Void
	end_date_not_void: end_date /= Void

end
