note

	description:

		"[
		Pool Fund Outstanding Shares lookup and lookup of change to outstanding shares
		]"

	author: "Ivan Tarnavsky"
	last_editor: "$Author: it $"
	date: "$Date: 2011/03/30 $"
	revision: "$Revision: #1 $"
	path: "$File: //rose/source/kernel/expert_system/rp_pool_fund_outstanding_shares_lookup.e $"

class RP_POOL_FUND_OUTSTANDING_SHARES_LOOKUP

create

	make

feature {NONE} -- Initialization

	make (a_reference_system: RP_REFERENCE_SYSTEM) is
			-- Initialize `Current'
		require
			a_reference_system_not_void: a_reference_system /= Void
		do
			create clio_poolfund_market_data_source.make_with_lookback (a_reference_system, 60)
			create cadb_poolfund_market_data_source.make_with_lookback (a_reference_system, 60)
		end

feature -- Query

	outstanding_shares (a_fund: RP_POOL_FUND; a_date: ROSE_DATE): TUPLE [date: ROSE_DATE; shares: RP_DECIMAL] is
			-- Outstanding shares for `a_fund' on `a_date'
		require
			a_fund_not_void: a_fund /= Void
			a_date_not_void: a_date /= Void
		local
			l_record: RP_MARKET_DATA_RECORD
		do
			l_record := clio_poolfund_market_data_source.last_record_with_valid_field (a_fund, a_date, clio_poolfund_market_data_source.civ_fund_class_shares_type)
			if l_record = Void then
				l_record := cadb_poolfund_market_data_source.last_record_with_valid_field (a_fund, a_date, cadb_poolfund_market_data_source.civ_fund_class_shares_type)
			end
			if l_record /= Void then
				Result := [l_record.date, l_record.decimal_field (l_record.civ_fund_class_shares_type)]
			end
		end

	previous_change_to_outstanding_shares (a_fund: RP_POOL_FUND; a_date: ROSE_DATE; a_shares: RP_DECIMAL): TUPLE [date: ROSE_DATE; shares: RP_DECIMAL] is
			-- Shares and date before `a_date' on which the number of `a_fund's outstanding shares is different than `a_shares'
		require
			a_fund_not_void: a_fund /= Void
			a_date_not_void: a_date /= Void
			a_shares_not_void: a_date /= Void
		local
			l_record: RP_MARKET_DATA_RECORD
			l_date: ROSE_DATE
		do
			from
				l_date := a_date.previous_date
				l_record := clio_poolfund_market_data_source.last_record_with_valid_field (a_fund, l_date, clio_poolfund_market_data_source.civ_fund_class_shares_type)
				if l_record = Void then
					l_record := cadb_poolfund_market_data_source.last_record_with_valid_field (a_fund, l_date, cadb_poolfund_market_data_source.civ_fund_class_shares_type)
				end
			until
				l_record = Void or else l_record.decimal_field (l_record.civ_fund_class_shares_type) /~ a_shares
			loop
				l_date := l_record.date.previous_date
				l_record := clio_poolfund_market_data_source.last_record_with_valid_field (a_fund, l_date, clio_poolfund_market_data_source.civ_fund_class_shares_type)
				if l_record = Void then
					l_record := cadb_poolfund_market_data_source.last_record_with_valid_field (a_fund, l_date, cadb_poolfund_market_data_source.civ_fund_class_shares_type)
				end
			end
			if l_record /= Void then
				Result := [l_record.date, l_record.decimal_field (l_record.civ_fund_class_shares_type)]
			end
		ensure
			previous_date: Result /= Void implies a_date > Result.date
			different_shares: Result /= Void implies a_shares /~ Result.shares
		end

feature {NONE} -- Implementation

	clio_poolfund_market_data_source: RP_CLIO_POOL_FUND_MARKET_DATA_SOURCE
			-- CLIO Poolfund market data source

	cadb_poolfund_market_data_source: RP_CADB_POOL_FUND_MARKET_DATA_SOURCE
			-- Poolfund market data source

invariant

	clio_poolfund_market_data_source_not_void: clio_poolfund_market_data_source /= Void
	cadb_poolfund_market_data_source_not_void: cadb_poolfund_market_data_source /= Void

end
