indexing

	description:

		"Poolfund pricing algorithm"

	last_editor: "$Author: nw $"
	date: "$Date: 2010/06/01 $"
	revision: "$Revision: #3 $"
	archive: "$File: //rose/source/kernel/expert_system/valuators/rp_pool_fund_pricing_algorithm.e $"

class RP_POOL_FUND_PRICING_ALGORITHM

inherit

	RP_PRICING_ALGORITHM [RP_POOL_FUND]

create

	make,
	make_axys

feature {NONE} -- Initializations

	make (a_zone: like zone; a_reference_system: like reference_system; a_trade_price: like trade_price; a_calendar: like calendar; a_price_override: like price_override) is
			-- Initialize `Current'
		require
			reference_system_not_void: a_reference_system /= Void
			non_void_args: a_trade_price /= Void and a_calendar /= Void and a_price_override /= Void and a_zone /= Void
		do
			zone := a_zone
			reference_system := a_reference_system
			trade_price := a_trade_price
			calendar := a_calendar
			price_override := a_price_override
			create last_price_cache.make_equal (17)
		ensure
			reference_system_set: reference_system = a_reference_system
			trade_price_set: trade_price = a_trade_price
			calendar_set: calendar = a_calendar
			price_override_set: price_override = a_price_override
		end

	make_axys (a_reference_system: like reference_system; a_calendar: like calendar; a_price_override: like price_override) is
			-- Initialize `Current'
		require
			reference_system_attached: a_reference_system /= Void
			calendar_attached: a_calendar /= Void
		do
			reference_system := a_reference_system
			calendar := a_calendar
			price_override := a_price_override
			create last_price_cache.make_equal (17)
		ensure
			reference_system_set: reference_system = a_reference_system
			calendar_set: calendar = a_calendar
			price_override_set: price_override = a_price_override
		end

feature -- Query

	price (a_fund: RP_POOL_FUND; a_date: ROSE_DATE): RP_PRICE is
			-- Price of `a_fund' on `a_date' in units of RP_CURRENCY
		local
			l_record: RP_MARKET_DATA_RECORD
		do
				-- Check for price override first
			if price_override /= Void then
				l_record := price_override.snapshot (a_fund, a_date)
				if l_record /= Void then
					Result := l_record.unadjusted_close
					Result.set_source (const_price_source_price_override)
				end
			end
				-- Retrieve market price if no price override
			if Result = Void then
				Result := poolfund_price (a_fund, a_date, calendar)
					-- Update lag price
				if Result /= Void then
					last_price_cache.force (Result, a_fund)
				end
			end
				-- Trade price
			if Result = Void and trade_price /= Void then
				l_record := trade_price.snapshot (a_fund, a_date)
				if l_record /= Void then
					Result := l_record.unadjusted_close
					Result.set_source (const_price_source_trade_price)
					last_price_cache.force (Result, a_fund)
				end
			end
				-- Use lag price if everything else fails
			if Result = Void then
				last_price_cache.search (a_fund)
				if last_price_cache.found then
					Result := last_price_cache.found_item
					Result.set_source (const_price_source_lag_price)
				end
			end
		end

	use_clio : BOOLEAN
			-- Use clio as first source for fund NAV's? (fallback to composite_asset db if False or not found there)

	const_price_source_price_override: STRING is "PO"
			-- Price Override used

	const_price_source_trade_price: STRING is "TP"
			-- Trade price used

	const_price_source_lag_price: STRING is "LP"
			-- Lag price used

feature	-- CLIO pricing

	enable_clio_pricing is
			-- Enable CLIO pricing
		do
			use_clio := True
		ensure
			clio_enabled: use_clio = True
		end

	disable_clio_pricing is
			-- Disable CLIO pricing
		do
			use_clio := False
		ensure
			clio_disabled: use_clio = False
		end

feature {NONE} -- Implementation

	zone: RP_ZONE
			-- Zone

	reference_system: RP_REFERENCE_SYSTEM
			-- Reference system

	last_price_cache: UID_TABLE [RP_PRICE, RP_INSTRUMENT]
			-- Last computed price cache

	price_override: RP_MARKET_DATA_SOURCE [RP_INSTRUMENT]
			-- Price override market data source

	lookback_range: INTEGER is 60
			-- How long to look back for price

	clio_poolfund_market_data_source: RP_CLIO_POOL_FUND_MARKET_DATA_SOURCE is
			-- CLIO Poolfund market data source
		once
			create Result.make (reference_system)
		end

	cadb_poolfund_market_data_source: RP_CADB_POOL_FUND_MARKET_DATA_SOURCE is
			-- Poolfund market data source
		once
			create Result.make (reference_system)
		end

	trade_price: RP_TRADE_PRICE_MARKET_DATA_SOURCE
			--	trade price

	calendar: RP_CALENDAR
			-- Market calendar

	use_nav_first: BOOLEAN is
			-- Use given nav first if true, else use computed nav first (switches order of checking)
		do
			Result := True
		end

	poolfund_price (a_fund: RP_POOL_FUND; a_date: ROSE_DATE; a_calendar: RP_CALENDAR): RP_PRICE is
			-- Price of `a_fund' on `a_date' using `a_calendar'
		local
			l_nav: RP_MONIES
			l_data: RP_MARKET_DATA_RECORD
			l_instrument: RP_POOL_FUND
			s: STRING
			l_clio_price_found: BOOLEAN
		do
			if zone = Void then
				l_instrument := a_fund
			else
				l_instrument ?= reference_system.cross_zonal_translation (a_fund, zone, a_date)
			end
			if use_clio then
				l_data := price_lookback (clio_poolfund_market_data_source, a_fund, a_date, lookback_range, a_calendar, agent is_valid_fund_record (?))
			end
			if l_data = Void then
				l_data := price_lookback (cadb_poolfund_market_data_source, a_fund, a_date, lookback_range, a_calendar, agent is_valid_fund_record (?))
			else
				l_clio_price_found := True
			end
			if l_data /= Void and is_valid_fund_record (l_data) then
				if use_nav_first then
					if l_data.has (l_data.civ_fund_class_nav_type) then
						l_nav := l_data.monies_field (l_data.civ_fund_class_nav_type)
						s := const_used_published_nav + source_days_display(l_data.date, a_date)
					elseif has_required_fields_to_compute_nav (l_data) then
						l_nav := nav_from_balances_and_outstanding_shares (l_data)
						s := const_used_computed_nav + source_days_display(l_data.date, a_date)
					end
				else
					if has_required_fields_to_compute_nav (l_data) then
						l_nav := nav_from_balances_and_outstanding_shares (l_data)
						s := const_used_computed_nav + source_days_display(l_data.date, a_date)
					elseif l_data.has (l_data.civ_fund_class_nav_type) then
						l_nav := l_data.monies_field (l_data.civ_fund_class_nav_type)
						s := const_used_published_nav + source_days_display(l_data.date, a_date)
					end
				end
				if l_nav /= Void then
					if a_fund.uid /= l_instrument.uid then
						s := const_used_cross_zonal_instrument_price + s
					end
					if l_clio_price_found then
						s := const_used_clio_price + s
					end
					Result := a_fund.price_in_currency (l_nav.amount, l_nav.currency, l_data.date)
					Result.set_source (s)
				end
			end
		end

	const_used_computed_nav: STRING is "CMP"
		-- Indicator that computed NAV was used as price source

	const_used_published_nav: STRING is "NAV"
		-- Indicator that published NAV was used as price source

	const_used_cross_zonal_instrument_price: STRING is "x"
		-- Indicator that a cross zonal instrument was used to price the fund

	const_used_clio_price: STRING is "c"
		-- Indicator that a cross zonal instrument was used to price the fund

	source_days_display (a_target_date, a_found_date: ROSE_DATE) : STRING is
			-- Days difference between 'a_target_date' and 'a_found_date' as a string (blank string if equal)
		require
			target_date_attached: a_target_date /= Void
			found_date_attached: a_found_date /= Void
		do
			if a_target_date.is_equal (a_found_date) then
				Result := ""
			else
				Result := a_target_date.diff_in_days (a_found_date).out
			end
		end

	is_valid_fund_record (a_market_data: RP_MARKET_DATA_RECORD): BOOLEAN is
			-- Is `a_market_data` valid?
		do
			Result := a_market_data /= Void and then (a_market_data.has (a_market_data.civ_fund_class_nav_type) or else has_required_fields_to_compute_nav (a_market_data))
		end

	price_lookback (a_mkt: RP_MARKET_DATA_SOURCE [RP_INSTRUMENT]; a_instrument: RP_INSTRUMENT; a_date: ROSE_DATE; a_max_days_to_go_back: INTEGER; a_calendar: RP_CALENDAR; a_query: FUNCTION [ANY, TUPLE, BOOLEAN]): RP_MARKET_DATA_RECORD is
			-- Price lookback
		require
			source_not_void: a_mkt /= Void
			instrument_not_void: a_instrument /= Void
			date_not_void: a_date /= Void
			calendar_not_void: a_calendar /= Void
		local
			l_date: ROSE_DATE
			l_counter: INTEGER
			l_found: BOOLEAN
			l_result: RP_MARKET_DATA_RECORD
		do
			from
				create l_date.make_by_date (a_date)
				l_counter := 0
			until
				l_counter > a_max_days_to_go_back or l_found
			loop
				l_result := a_mkt.snapshot (a_instrument, l_date)
				if l_result /= Void then
					if a_query = Void then
						l_found := True
					else
						l_found := a_query.item ([l_result])
					end
				end
				l_counter := l_counter + 1
					-- go back one market day
				l_date := decrement_date (a_calendar, l_date)
			end
			if l_found then
				Result := l_result
			end
		end

	decrement_date (a_calendar: RP_CALENDAR; a_date: ROSE_DATE): ROSE_DATE is
			-- Previous date to `a_date'
		do
				-- Default is market days
			Result := a_calendar.ith_work_day_before (a_date, 1)
		end

	has_required_fields_to_compute_nav (a_record: RP_MARKET_DATA_RECORD): BOOLEAN is
			-- Does `a_record' have all fields required to compute NAV from balances and outstanding shares?
		do
			Result := a_record.has (a_record.civ_fund_class_capital_balance_type) and a_record.has (a_record.civ_fund_class_revenue_balance_type) and a_record.has (a_record.civ_fund_class_expense_balance_type) and a_record.has (a_record.civ_fund_class_shares_type)
		end

	nav_from_balances_and_outstanding_shares (a_record: RP_MARKET_DATA_RECORD): RP_MONIES is
			-- Fund NAV from balances and outstanding shares
		require
			non_void_record: a_record /= Void
			has_required_fields: has_required_fields_to_compute_nav (a_record)
			same_currency: a_record.monies_field (a_record.civ_fund_class_capital_balance_type).currency.is_equal (a_record.monies_field (a_record.civ_fund_class_revenue_balance_type).currency) and a_record.monies_field (a_record.civ_fund_class_capital_balance_type).currency.is_equal (a_record.monies_field (a_record.civ_fund_class_expense_balance_type).currency)
		local
			l_amount: RP_DECIMAL
			l_currency: RP_CURRENCY
		do
			l_currency := a_record.monies_field (a_record.civ_fund_class_capital_balance_type).currency
			l_amount := (a_record.monies_field (a_record.civ_fund_class_capital_balance_type) + a_record.monies_field (a_record.civ_fund_class_revenue_balance_type) - a_record.monies_field (a_record.civ_fund_class_expense_balance_type)).amount / a_record.decimal_field (a_record.civ_fund_class_shares_type)
			Result := l_currency.monies (l_amount.rounded (4))
		ensure
			non_void_result: Result /= Void
		end

end
