indexing

	description:

		"Contract in which one of the parties is a portfolio"

	date: "$Date: 2010/07/19 $"
	sccid: "$Id: //rose/source/kernel/contracts/rp_portfolio_contract.e#15 $"

deferred class RP_PORTFOLIO_CONTRACT

inherit

	RP_ACCOUNT_CONTRACT
		rename
			make as make_account_contract
		end

feature {NONE} -- Initialization

	make (a_generating_system: like generating_system; a_agreement_time: like agreement_time; a_identifier: like identifier; a_portfolio: like portfolio) is
			-- Initialize `Current'
		require
			generating_system_not_void: a_generating_system /= Void
			agreement_time_not_void: a_agreement_time /= Void
			identifier_not_void: a_identifier /= Void
			portfolio_not_void: a_portfolio /= Void
		do
			portfolio := a_portfolio
			make_account_contract (a_generating_system, a_identifier, a_agreement_time)
		ensure
			generating_system_set: generating_system = a_generating_system
			agreement_time_set: agreement_time = a_agreement_time
			identifier_set: identifier = a_identifier
			portfolio_set: portfolio = a_portfolio
		end

feature -- Access

	portfolio: RP_PORTFOLIO
			-- Portfolio

	account: RP_INVESTMENT_ACCOUNT is
			-- <Precursor>
		do
			Result := portfolio.account
		ensure then
			portfolio_account: Result = portfolio.account
		end

feature -- Change

	add_flow (a_type: RP_FLOW_TYPE; a_source_control: RP_INSTRUMENT_CONTROL; a_destination_control: RP_INSTRUMENT_CONTROL; a_contracted_ownership_time: ROSE_DATE; a_contracted_settlement_time: ROSE_DATE; a_quantity: RP_QUANTITY; a_value_date: ROSE_DATE; a_exchanged_subset: INTEGER; a_flow_value: RP_QUANTITY) is
			-- Add contract flow to `Current'.
		require
			non_void_type: a_type /= Void
			non_void_source_control: a_source_control /= Void
			non_void_destination_control: a_destination_control /= Void
			non_void_quantity: a_quantity /= Void
			non_void_value_date: a_value_date /= Void
			non_void_contracted_ownership_time: a_contracted_ownership_time /= Void
			non_void_contracted_settlement_time: a_contracted_settlement_time /= Void
		local
			l_flow: RP_FLOW
		do
			create l_flow.make (Current, a_type, a_source_control, a_destination_control, a_contracted_ownership_time, a_contracted_settlement_time, a_quantity, a_value_date, a_exchanged_subset, a_flow_value)
			flows.extend (l_flow)
			debug ("void_flow_value")
				if a_flow_value = Void then
					print ("VoidValue===>" + l_flow.contract.identifier + ":" + l_flow.debug_output + "%N")
				end
			end
		end

	add_external_holding_flow (a_type: RP_FLOW_TYPE; a_contracted_ownership_time: ROSE_DATE; a_contracted_settlement_time: ROSE_DATE; a_holding: RP_EXTERNAL_HOLDING; a_source_control: RP_INSTRUMENT_CONTROL; a_destination_control: RP_INSTRUMENT_CONTROL) is
			-- Add external holding flow to `Current'.
		local
			l_flow: RP_EXTERNAL_HOLDING_FLOW
		do
			create l_flow.make (Current, a_type, a_contracted_ownership_time, a_contracted_settlement_time, a_holding, a_source_control, a_destination_control)
			flows.extend (l_flow)
		end

feature -- Conversion

	as_dividend: RP_DIVIDEND is
			-- `Current' as dividend
		do
			Result := Void
		end

	as_interest: RP_INTEREST is
			-- `Current' as interest
		do
			Result := Void
		end

	as_income: RP_INCOME is
			-- `Current' as income
		do
			Result := Void
		end

feature -- Status Report

	is_dividend: BOOLEAN is
			-- Is `Current' a dividend?
		do
			Result := False
		end

	is_interest: BOOLEAN is
			-- Is `Current' a interest?
		do
			Result := False
		end

	is_income: BOOLEAN is
			-- Is `Current' a income?
		do
			Result := False
		end

invariant

	portfolio_not_void: portfolio /= Void

end
