note

	description:

		"Portfolio Engineering System"

	author: "Mark Howard"
	last_editor: "$Author: e2 $"
	date: "$Date: 2011/11/29 $"
	revision: "$Revision: #10 $"
	path: "$File: //rose/source/kernel/expert_system/rp_portfolio_engineering_system.e $"

class RP_PORTFOLIO_ENGINEERING_SYSTEM

inherit

	RP_CONTRACT_SYSTEM

	RP_UNKNOWN_CUSTODY_AWARE
		export {NONE} all end

create {RP_INVESTMENT_MANAGER}

	make

feature -- Basic Operations

	generate_fund_participant_contracts (a_portfolio: RP_PORTFOLIO) is
			-- Generate contract for fund participants
		require
			non_void_portfolio: a_portfolio /= Void
			portfolio_is_fund_participant: a_portfolio.is_fund_participant
			account_is_not_segregate: not a_portfolio.account.is_segregate
		local
			l_shares: CELL [RP_DECIMAL]
			l_query: STRING_PREDICATE
		do
			create l_shares.put (0)
			create l_query.make ((create {DATABASE_ACCESS}).make_from_repository_fast_load ("common"), "[
				select 	cast(trade_date as varchar(50))
						+'|'+
						cast(cast(outstanding_shares as decimal(35,6)) as varchar(50))
						+'|'+
						cast(cast(capital_balance as decimal(35,6)) as varchar(50))
				from composite_assets_nav
				where brimco+sec = ?
				order by 1
			]")
			l_query.retrieve (Void, <<a_portfolio.composite_asset.brimsym>>)
			l_query.array.do_all (agent post_new_fund_participant_contract (a_portfolio, l_shares, ?))
		end

feature {NONE} -- Implementation

	post_new_fund_participant_contract (a_portfolio: RP_PORTFOLIO; a_shares: CELL [RP_DECIMAL]; s: STRING) is
			-- Post fund participant contract to adjust shares if needed
		require
			non_void_argss: a_portfolio /= Void and a_shares /= Void and s /= Void
		local
			l_contract: RP_CLIENT_CONTRACT
			l_cost: RP_LEGACY_COST
			l_usd: RP_CURRENCY
			l_qty: RP_QUANTITY
			l_shares: RP_DECIMAL
			l_capital: RP_DECIMAL
			l_date: ROSE_DATE
			l_tokenizer: ROSE_DELIMITED_TEXT
		do
			create l_tokenizer.make (s, '|')
			create l_shares.make_from_string (l_tokenizer.item (2))
			if not l_shares.is_equal (a_shares.item) then
				create l_date.make_ccyymmdd (l_tokenizer.item (1).to_integer)
				create l_capital.make_from_string (l_tokenizer.item (3))
				l_usd := manager.reference_system.currency_from_iso_code ("USD")
				if l_shares > a_shares.item then
					l_qty := a_portfolio.composite_asset.quantity (l_shares - a_shares.item)
					create l_cost.make (l_usd.monies ((l_shares - a_shares.item) * l_capital / l_shares), l_date, l_usd)
					l_contract := manager.client_system.new_inkind (a_portfolio, l_date, l_date, l_qty, l_cost, unknown_custody)
				else
					l_qty := a_portfolio.composite_asset.quantity (a_shares.item - l_shares)
					create l_cost.make (l_usd.monies ((a_shares.item - l_shares) * l_capital / l_shares), l_date, l_usd)
					l_contract := manager.client_system.new_outkind (a_portfolio, l_date, l_date, l_qty, l_cost, unknown_custody)
				end
				manager.post (l_contract)
				a_shares.put (l_shares)
			end
		end

end
