note
	description: "Trade Portfolio non_cash position"
	author: "Sami Ben Grine & Ivan Tarnavsky"
	last_editor: "$Author: er $"
	date: "$Date: 2014/02/26 $"
	revision: "$Revision: #2 $"
	path: "$File: //rose/source/kernel/expert_system/rp_portfolio_non_cash_position.e $"

class
	RP_PORTFOLIO_NON_CASH_POSITION

inherit
	RP_NON_CASH_POSITION[RP_PORTFOLIO]

create
	make

feature {NONE} -- Initialization

	make (a_row: NT_PORTFOLIO_NON_CASH)
			-- Make with 'a_row'
		require
			valid_row: a_row /= Void
			valid_instrument: (a_row.ref_uid/=0) and then (a_row.brimsym/=Void and then a_row.brimsym.count=18) and then (manager.reference_system.instrument_from_brimsym (a_row.brimsym) /= Void)
		do
			create global_investment_date.make_by_date (a_row.global_investment_date)
			create update_mode.make_from_string (a_row.update_mode)

			portfolio := manager.reference_system.portfolio_from_code (a_row.portfolio_code)
			account := manager.reference_system.investment_account_by_code (a_row.account_code)
			check valid_account: portfolio.account.code ~ account.code end

			instrument := manager.reference_system.instrument_from_brimsym (a_row.brimsym)

				-- Quantities
			total_quantity := instrument.quantity (a_row.total_quantity)
			unavailable := instrument.quantity (a_row.quantity_unavailable)

				-- Local currency
			if (a_row.local_currency/=Void and then not a_row.local_currency.is_empty) then
				local_currency := manager.reference_system.currency_from_iso_code (a_row.local_currency)
				if local_currency /= Void then
					if a_row.cost_local /= Void and then a_row.cost_local.is_defined then
						cost_local := local_currency.monies (a_row.cost_local)
					end
					if a_row.market_value_local /= Void and then a_row.market_value_local.is_defined then
						market_value_local := local_currency.monies (a_row.market_value_local)
					end
				else
					warn (generator, "Could not load cost_local, market_value_local", <<>>)
				end
			end
				-- Account currency
			if (a_row.account_currency/=Void and then not a_row.account_currency.is_empty) then
				account_currency := manager.reference_system.currency_from_iso_code (a_row.account_currency)
				if account_currency /= Void and then a_row.cost_account_base/=Void and then a_row.cost_account_base.is_defined then
					cost_account_base := account_currency.monies (a_row.cost_account_base)
				else
					warn (generator, "Could not load cost_account_base", <<>>)
				end
			end
				-- Global currency
			if (a_row.global_currency/=Void and then not a_row.global_currency.is_empty) then
				global_currency := manager.reference_system.currency_from_iso_code (a_row.global_currency)
				if global_currency /= Void then
					if a_row.cost_global_base /= Void and then a_row.cost_global_base.is_defined then
						cost_global_base := global_currency.monies (a_row.cost_global_base)
					end
					if a_row.market_value_global_base /= Void and then a_row.market_value_global_base.is_defined then
						market_value_global_base := global_currency.monies (a_row.market_value_global_base)
					end
				else
					warn (generator, "Could not load cost_global_base, market_value_global_base", <<>>)
				end
			end

			create accounting_date.make_by_date (a_row.accounting_date)

			contract_side := a_row.contract_side.item (1)

			create accounting_basis.make_from_code (a_row.accounting_basis.item (1))

			create fm_sec_id.make_from_string (a_row.fm_sec_id)

			set_errors (a_row.errors)
		ensure
			valid_gid: (global_investment_date /= Void) and then (global_investment_date ~ a_row.global_investment_date)
			valid_update_mode: update_mode /= Void and then (update_mode.out ~ a_row.update_mode)
			valid_portfolio: (portfolio /= Void) and then (portfolio.code ~ a_row.portfolio_code)
			valid_investment_region: (portfolio.zone.code ~ a_row.investment_region)
			valid_account_code: (account /= Void) and then (account.code ~ a_row.account_code) and then (portfolio.account ~ account)
			valid_instrument: (instrument /= Void)
--			valid_uid: (instrument.uid /= 0) and then (instrument.uid ~ a_row.ref_uid)
			valid_brimsym: (brimsym /= Void) and then (instrument.brimsym ~ a_row.brimsym)
			valid_errors: errors /= Void
			valid_cost_local: (cost_local /= Void) implies (local_currency /= Void and then cost_local.currency ~ local_currency) and then (cost_local.amount ~ a_row.cost_local)
			valid_market_value_local: (market_value_local /= Void) implies (market_value_local.currency ~ local_currency) and then (market_value_local.amount ~ a_row.market_value_local)
			valid_account_currency: (account_currency /= Void) implies (account_currency.iso.as_upper ~ a_row.account_currency.as_upper)
			valid_global_currency: (global_currency /= Void) implies (global_currency.iso.as_upper ~ a_row.global_currency.as_upper)
			valid_local_currency: (local_currency /= Void) implies (local_currency.iso.as_upper ~ a_row.local_currency.as_upper)
			valid_cost_account_base: (cost_account_base /= Void) implies (account_currency /= Void and then cost_account_base.currency ~ account_currency) and then (cost_account_base.amount ~ a_row.cost_account_base)
			valid_cost_global_base: (cost_global_base /= Void) implies (global_currency/=Void and then cost_global_base.currency ~ global_currency) and then (cost_global_base.amount ~ a_row.cost_global_base)
			valid_market_value_global_base: (market_value_global_base /= Void) implies (market_value_global_base.currency ~ global_currency) and then (market_value_global_base.amount ~ a_row.market_value_global_base)
			valid_accounting_date: (accounting_date /= Void) and then (accounting_date ~ a_row.accounting_date)
			valid_contract_side: contract_side.out ~ a_row.contract_side
			valid_accounting_basis: (accounting_basis /= Void) and then (accounting_basis.code.out ~ a_row.accounting_basis)
			valid_fm_sec_id: (fm_sec_id /= Void) and then (fm_sec_id ~ a_row.fm_sec_id)
		end

feature -- Access

	portfolio: RP_PORTFOLIO
		-- Portfolio

	investment_region: RP_ZONE
			-- Investment Region
		do
			Result := portfolio.zone
		end

invariant

	portfolio_exists: portfolio /= Void

end
