note
	description: "Portfolio open currency forward position (always traded)"
	author: "Sami Ben Grine & Ivan Tarnavsky"
	last_editor: "$Author: sm $"
	date: "$Date: 2013/06/26 $"
	revision: "$Revision: #1 $"
	path: "$File: //rose/source/kernel/expert_system/rp_portfolio_open_ffx_position.e $"

class
	RP_PORTFOLIO_OPEN_FFX_POSITION

inherit
	RP_OPEN_FFX_POSITION[RP_PORTFOLIO]

create
	make

feature {NONE} -- Initialization

	make (a_row: NT_PORTFOLIO_OPEN_FFX)
			-- Make with 'a_row'
		require
			row_exists: a_row /= Void
		do
			create global_investment_date.make_by_date (a_row.global_investment_date)
			create update_mode.make_from_string (a_row.update_mode)

			portfolio := manager.reference_system.portfolio_from_code (a_row.portfolio_code)
			account := manager.reference_system.investment_account_by_code (a_row.account_code)
			check valid_account: portfolio.account.code ~ account.code end

			buy := manager.reference_system.currency_from_iso_code (a_row.buy_currency).monies (a_row.buy_amount)
			sell := manager.reference_system.currency_from_iso_code (a_row.sell_currency).monies (a_row.sell_amount)
			create contract_date.make_by_date (a_row.contract_date)
			create settlement_date.make_by_date (a_row.settlement_date)
			create broker.make_from_string (a_row.broker)
			settlement_currency := manager.reference_system.currency_from_iso_code (a_row.settlement_currency)
			if  a_row.net_market_value /= Void and then a_row.net_market_value.is_defined then
				net_market_value := manager.reference_system.currency_from_iso_code (a_row.net_currency).monies (a_row.net_market_value)
			else
				net_market_value := manager.reference_system.currency_from_iso_code (a_row.net_currency).monies (0)
			end

			set_errors (a_row.errors)

		ensure
			valid_gid: (global_investment_date /= Void) and then (global_investment_date ~ a_row.global_investment_date)
			valid_update_mode: update_mode /= Void and then (update_mode.out ~ a_row.update_mode)
			valid_portfolio: (portfolio /= Void) and then (portfolio.code ~ a_row.portfolio_code)
			valid_investment_region: (portfolio.zone.code ~ a_row.investment_region)
			valid_account_code: (account /= Void) and then (account.code ~ a_row.account_code) and then (portfolio.account ~ account)

			valid_buy: (buy /= Void) and then (buy.currency ~ buy.currency)
			valid_buy_currency:(buy_currency /= Void) and then (buy.currency.iso ~ a_row.buy_currency)
			valid_buy_amount: (buy /= Void) and then (buy.amount ~ a_row.buy_amount)

			valid_sell: (sell /= Void) and then (sell_currency ~ sell.currency)
			valid_sell_currency: (sell_currency /= Void) and then (sell.currency.iso ~ a_row.sell_currency)
			valid_sell_amount: (sell /= Void) and then (sell.amount ~ a_row.sell_amount)
			valid_contract_date: (contract_date /= Void) and then (contract_date ~ a_row.contract_date)
			valid_settlement_date: (settlement_date /= Void) and then (settlement_date ~ a_row.settlement_date)
			valid_broker: (broker /= Void) and then (broker ~ a_row.broker)
			valid_settlement_currency: (settlement_currency /= Void) and then (settlement_currency.iso ~ a_row.settlement_currency)
			valid_errors: (errors /= Void)
			valid_net_market_value: net_market_value /= Void implies (net_market_value.amount ~ a_row.net_market_value)
			valid_net_currency: net_market_value /= Void implies net_market_value.currency.iso ~ a_row.net_currency
		end

feature -- Access

	portfolio: RP_PORTFOLIO
		-- Portfolio

	investment_region: RP_ZONE is
			-- Investment Region
		do
			Result := portfolio.zone
		end

invariant

	portfolio_exists: portfolio /= Void

end
