indexing

	description: "Maintains a list of positions"

	author: "it"
	last_editor: "$Author: it $"
	date: "$Date: 2010/09/06 $"
	sccid: "$Id: //rose/source/kernel/expert_system/aflows/position/rp_positions.e#1 $"

class RP_POSITIONS

inherit

	ANY

	STORABLE
		export
			{NONE} all
			{ANY}
				store_by_name,
				retrieve_by_name
		end

create

	make

feature {NONE} -- Initialization

	make (a_instrument: RP_INSTRUMENT) is
			-- Initialize `Current'.
		require
			instrument_not_void: a_instrument /= Void
		local
			l_default: RP_POSITION
		do
			create positions.make (create {KL_COMPARABLE_COMPARATOR [ROSE_DATE]}.make)
			create l_default.make_default (a_instrument)
			positions.force (l_default, l_default.start_date)
		ensure
			positions_not_void: positions /= Void
			has_default: positions.has (create {ROSE_DATE}.make_min_date)
		end

feature -- Query

	position (a_date: ROSE_DATE): RP_POSITION is
			-- Position on `a_date'
		local
			l_cursor: DS_AVL_TREE_CURSOR [RP_POSITION, ROSE_DATE]
		do
			l_cursor := positions.new_cursor
			l_cursor.go_at_or_before_key (a_date)
			if not l_cursor.off then
				if l_cursor.item.brackets (a_date) then
					Result := l_cursor.item
				end
			end
		ensure
			void_result_means_not_found: True
		end

feature {RP_ACCOUNTING_SYSTEM} -- Implementation

	positions: RP_EQUITY_MARKET_DATA_TREE [RP_POSITION, ROSE_DATE]

	adjust_with (a_date: ROSE_DATE; a_flow: RP_FLOW) is
			-- Adjust positions from `a_date' with `a_flow'
		require
			date_not_void: a_date /= Void
			flow_not_void: a_flow /= Void
			same_instrument: positions.first.instrument.is_equal (a_flow.instrument)
		local
			l_cursor: DS_AVL_TREE_CURSOR [RP_POSITION, ROSE_DATE]
			l_existing_position: RP_POSITION
			l_new_position: RP_POSITION
		do
			l_cursor := positions.new_cursor
			l_cursor.go_at_or_before_key (a_date)
			if not l_cursor.before then
				l_existing_position := l_cursor.item
				if l_existing_position.start_date.ccyymmdd = a_date.ccyymmdd then
					from
						l_existing_position.adjust_with (a_flow)
						l_cursor.forth
					until
						l_cursor.after
					loop
						l_cursor.item.plus (a_flow)
						l_cursor.forth
					end
				elseif l_existing_position.brackets (a_date) then
					from
						create l_new_position.make (a_flow, a_date, l_existing_position)
						l_existing_position.close_on (a_date)
						l_cursor.forth
					until
						l_cursor.after
					loop
						l_cursor.item.plus (a_flow)
						l_cursor.forth
					end
					positions.force (l_new_position, a_date)
				else
					(create {EXCEPTIONS}).raise ("Not possible")
				end
			else
				(create {EXCEPTIONS}).raise ("Not possible")
			end
		end

invariant

	positions_not_void: positions /= Void
	positions_not_empty: not positions.is_empty

end
