note
	description: "Access to computed positions from Northern Trust data feed"
	author: "Sami Ben Grine"
	last_editor: "$Author: sm $"
	date: "$Date: 2014/12/10 $"
	revision: "$Revision: #6 $"
	path: "$File: //rose/source/kernel/expert_system/rp_positions_system.e $"

deferred class RP_POSITIONS_SYSTEM

inherit

	ANY

	ROSE_SHARED_GLOBAL
		export {NONE} all end

	RP_AXAROSENBERG_AWARE
		export {NONE} all end

feature -- Queries

	NorthernTrust_positions_migration_date: attached ROSE_DATE
			-- Date when we switched to NT for positions
		deferred
		end

feature -- Access (Portfolio beginning of day positions)

	portfolio_positions  (a_accounting_date: ROSE_DATE; a_portfolio: RP_PORTFOLIO): ROSE_LINEAR_ARRAY[POSITION[RP_PORTFOLIO]]
			-- Cash positions at the beginning of 'a_accounting_date' for 'a_portfolio'
		require
			valid_accounting_date: a_accounting_date /= Void and then a_accounting_date >= Northerntrust_positions_migration_date
			valid_portfolio: a_portfolio /= Void
		deferred
		ensure
			result_not_void: Result /= Void
		end

	portfolio_cash_positions (a_accounting_date: ROSE_DATE; a_portfolio: RP_PORTFOLIO): ROSE_LINEAR_ARRAY[RP_PORTFOLIO_CASH_POSITION]
			-- Cash positions at the beginning of 'a_accounting_date;' for 'a_portfolio'
		require
			valid_accounting_date: a_accounting_date /= Void and then a_accounting_date >= Northerntrust_positions_migration_date
			valid_portfolio: a_portfolio /= Void
		deferred
		ensure
			result_not_void: Result /= Void
		end

	portfolio_settled_cash_positions (a_global_investment_date: ROSE_DATE; a_portfolio: RP_PORTFOLIO): ROSE_LINEAR_ARRAY[RP_PORTFOLIO_CASH_POSITION]
			-- <Precursor>
		require
			valid_global_investment_date: a_global_investment_date /= Void and then a_global_investment_date >= Northerntrust_positions_migration_date
			valid_portfolio: a_portfolio /= Void
		deferred
		ensure
			result_not_void: Result /= Void
		end

	portfolio_non_cash_positions (a_accounting_date: ROSE_DATE; a_portfolio: RP_PORTFOLIO): ROSE_LINEAR_ARRAY[RP_PORTFOLIO_NON_CASH_POSITION]
			-- Equity positions at the beginning of 'a_accounting_date;' for 'a_portfolio'
		require
			valid_accounting_date: a_accounting_date /= Void and then a_accounting_date >= Northerntrust_positions_migration_date
			valid_portfolio: a_portfolio /= Void
		deferred
		ensure
			result_not_void: Result /= Void
			empty_for_closed_portfolios: True
		end

	portfolio_open_ffx_positions (a_accounting_date: ROSE_DATE; a_portfolio: RP_PORTFOLIO): ROSE_LINEAR_ARRAY[RP_PORTFOLIO_OPEN_FFX_POSITION]
			-- Currency Forward positions at the beginning of 'a_accounting_date' for 'a_portfolio'
		require
			valid_accounting_date: a_accounting_date /= Void and then a_accounting_date >= Northerntrust_positions_migration_date
			valid_portfolio: a_portfolio /= Void
		deferred
		ensure
			result_not_void: Result /= Void
			empty_for_closed_portfolios: True
			only_in_T_zone: not Result.is_empty implies a_portfolio.zone.is_transzonal
		end

	portfolio_settled_cash_position_for_currency (a_accounting_date: ROSE_DATE; a_portfolio: RP_PORTFOLIO; a_currency: RP_CURRENCY): RP_PORTFOLIO_CASH_POSITION
			-- Cash positions at the beginning of 'a_accounting_date' for 'a_portfolio'
		require
			valid_accounting_date: a_accounting_date /= Void and then a_accounting_date >= Northerntrust_positions_migration_date
			valid_portfolio: a_portfolio /= Void
			valid_currency: a_currency /= Void
		deferred
		end

feature -- Access (Portfolio beginning of day valuation)

	portfolio_value (a_accounting_date: ROSE_DATE; a_portfolio: RP_PORTFOLIO; a_currency: RP_CURRENCY): RP_MONIES
			-- Value of `a_portfolio' at the beginning of `a_accounting_date' in `a_currency'
		require
			a_accounting_date_not_void: a_accounting_date /= Void
			a_portfolio_not_void: a_portfolio /= Void
			a_currency_not_void: a_currency /= Void
		deferred
		ensure
			not_void: Result /= Void
			same_currency: Result.currency.is_same (a_currency)
		end

	portfolio_cash_value (a_accounting_date: ROSE_DATE; a_portfolio: RP_PORTFOLIO; a_currency: RP_CURRENCY): RP_MONIES
			-- Traded cash value of `a_portfolio' at the beginning of `a_accounting_date' in `a_currency'
		require
			a_accounting_date_not_void: a_accounting_date /= Void
			a_portfolio_not_void: a_portfolio /= Void
			a_currency_not_void: a_currency /= Void
		deferred
		ensure
			not_void: Result /= Void
			same_currency: Result.currency.is_same (a_currency)
		end

	portfolio_settled_cash_value (a_accounting_date: ROSE_DATE; a_portfolio: RP_PORTFOLIO; a_currency: RP_CURRENCY): RP_MONIES
			-- Settled cash value of `a_portfolio' at the beginning of `a_accounting_date' in `a_currency'
		require
			a_accounting_date_not_void: a_accounting_date /= Void
			a_portfolio_not_void: a_portfolio /= Void
			a_currency_not_void: a_currency /= Void
		deferred
		ensure
			not_void: Result /= Void
			same_currency: Result.currency.is_same (a_currency)
		end

	portfolio_equity_value (a_accounting_date: ROSE_DATE; a_portfolio: RP_PORTFOLIO; a_currency: RP_CURRENCY): RP_MONIES
			-- Cash value of `a_portfolio' at the beginning of `a_accounting_date' in `a_currency'
		require
			a_accounting_date_not_void: a_accounting_date /= Void
			a_portfolio_not_void: a_portfolio /= Void
			a_currency_not_void: a_currency /= Void
		deferred
		ensure
			not_void: Result /= Void
			same_currency: Result.currency.is_same (a_currency)
		end

	portfolio_ffx_value (a_accounting_date: ROSE_DATE; a_portfolio: RP_PORTFOLIO; a_currency: RP_CURRENCY): RP_MONIES
			-- FFX net value of `a_portfolio' for `a_accounting_date' in `a_currency'
		require
			a_accounting_date_not_void: a_accounting_date /= Void
			a_portfolio_not_void: a_portfolio /= Void
			a_currency_not_void: a_currency /= Void
		deferred
		ensure
			not_void: Result /= Void
			same_currency: Result.currency.is_same (a_currency)
		end

    latest_portfolio_data_date( a_accounting_date: ROSE_DATE; a_portfolio: RP_PORTFOLIO ): ROSE_DATE
			-- latest date for which there is data for a portfolio - only to prevent re-looking this up on calls made many times
			-- May be Void!
		require
			accounting_date_not_void: a_accounting_date /= Void
			a_portfolio_non_void: a_portfolio /= Void
		deferred
		end

	price_of (a_latest_portfolio_data_date: ROSE_DATE; a_portfolio: RP_PORTFOLIO; a_position: RP_QUANTITY; a_currency: RP_CURRENCY; a_date: ROSE_DATE): RP_DECIMAL
			-- Price value.
		require
			non_void_portfolio: a_portfolio /= Void
			non_void_position: a_position /= Void
			non_void_currency: a_currency /= Void
			non_void_date: a_date /= Void
		deferred
		end

	portfolio_position_for_instrument (a_latest_portfolio_data_date: ROSE_DATE; a_accounting_date: ROSE_DATE; a_portfolio: RP_PORTFOLIO; a_instrument: RP_INSTRUMENT): RP_PORTFOLIO_NON_CASH_POSITION
			-- Long position of `a_instrument' in portfolio
		require
			accounting_date_not_void: a_accounting_date /= Void
			a_portfolio_non_void: a_portfolio /= Void
			a_instrument_not_void: a_instrument /= Void
		deferred
		ensure
			valid_position: (Result /= Void) implies (Result.total_quantity.instrument.uid = a_instrument.uid and Result.accounting_date ~ a_accounting_date and Result.portfolio ~ a_portfolio)
		end

feature -- Access (Portfolio beginning of day positions)

	account_positions  (a_accounting_date: ROSE_DATE; a_account: RP_INVESTMENT_ACCOUNT): ROSE_LINEAR_ARRAY[POSITION[RP_INVESTMENT_ACCOUNT]]
			-- Cash positions at the beginning of 'a_accounting_date;' for 'a_account'
		require
			valid_accounting_date: a_accounting_date /= Void and then a_accounting_date >= Northerntrust_positions_migration_date
			valid_account: a_account /= Void
		deferred
		ensure
			result_not_void: Result /= Void
		end

	account_cash_positions (a_accounting_date: ROSE_DATE; a_account: RP_INVESTMENT_ACCOUNT): ROSE_LINEAR_ARRAY[RP_ACCOUNT_CASH_POSITION]
			-- Cash positions at the beginning of 'a_accounting_date;' for 'a_account'
		require
			valid_accounting_date: a_accounting_date /= Void and then a_accounting_date >= Northerntrust_positions_migration_date
			valid_account: a_account /= Void
		deferred
		ensure
			result_not_void: Result /= Void
		end

	account_non_cash_positions (a_accounting_date: ROSE_DATE; a_account:RP_INVESTMENT_ACCOUNT): ROSE_LINEAR_ARRAY[RP_ACCOUNT_NON_CASH_POSITION]
			-- Equity positions at the beginning of 'a_accounting_date;' for 'aaccount'
		require
			valid_accounting_date: a_accounting_date /= Void and then a_accounting_date >= Northerntrust_positions_migration_date
			valid_account: a_account /= Void
		deferred
		ensure
			result_not_void: Result /= Void
			empty_for_closed_portfolios: True
		end

	account_open_ffx_positions (a_accounting_date: ROSE_DATE; a_account: RP_INVESTMENT_ACCOUNT): ROSE_LINEAR_ARRAY[RP_ACCOUNT_OPEN_FFX_POSITION]
			-- Currency Forward positions at the beginning of 'a_accounting_date' for 'a_account'
		require
			valid_accounting_date: a_accounting_date /= Void and then a_accounting_date >= Northerntrust_positions_migration_date
			valid_account: a_account /= Void
		deferred
		ensure
			result_not_void: Result /= Void
		end

feature -- Access (Account beginning of day positions)

	account_value (a_accounting_date: ROSE_DATE; a_account: RP_INVESTMENT_ACCOUNT; a_currency: RP_CURRENCY): RP_MONIES
			-- Value of `a_portfolio' on `a_accounting_date' in `a_currency'
		require
			a_accounting_date_not_void: a_accounting_date /= Void
			a_account_not_void: a_account /= Void
			a_currency_not_void: a_currency /= Void
		deferred
		ensure
			not_void: Result /= Void
			same_currency: Result.currency.is_same (a_currency)
		end

	account_cash_value (a_accounting_date: ROSE_DATE; a_account: RP_INVESTMENT_ACCOUNT; a_currency: RP_CURRENCY): RP_MONIES
			-- Cash value of `a_portfolio' on `a_accounting_date' in `a_currency'
		require
			a_accounting_date_not_void: a_accounting_date /= Void
			a_account_not_void: a_account /= Void
			a_currency_not_void: a_currency /= Void
		deferred
		ensure
			not_void: Result /= Void
			same_currency: Result.currency.is_same (a_currency)
		end

	account_equity_value (a_accounting_date: ROSE_DATE; a_account: RP_INVESTMENT_ACCOUNT; a_currency: RP_CURRENCY): RP_MONIES
			-- Equity value of `a_portfolio' on `a_as_of_date' in `a_currency'
		deferred
		ensure
			not_void: Result /= Void
			same_currency: Result.currency.is_same (a_currency)
		end

	account_ffx_value (a_accounting_date: ROSE_DATE; a_account: RP_INVESTMENT_ACCOUNT; a_currency: RP_CURRENCY): RP_MONIES
			-- FFX net value of `a_account' for `a_accounting_date' in `a_currency'
		require
			a_accounting_date_not_void: a_accounting_date /= Void
			a_account_not_void: a_account /= Void
			a_currency_not_void: a_currency /= Void
		deferred
		ensure
			not_void: Result /= Void
			same_currency: Result.currency.is_same (a_currency)
		end

feature -- Access (Portfolio with specific as_of_date)

	portfolio_value_as_of (a_as_of_date: ROSE_DATE; a_accounting_date: ROSE_DATE; a_portfolio: RP_PORTFOLIO; a_currency: RP_CURRENCY): RP_MONIES
			-- Value of `a_portfolio' on `a_as_of_date' in `a_currency'
		require
			a_as_of_date_not_void: a_as_of_date /= Void
			a_accounting_date_not_void: a_accounting_date /= Void
			a_portfolio_not_void: a_portfolio /= Void
			a_currency_not_void: a_currency /= Void
		deferred
		ensure
			not_void: Result /= Void
			same_currency: Result.currency.is_same (a_currency)
		end

	portfolio_cash_value_as_of (a_as_of_date: ROSE_DATE; a_accounting_date: ROSE_DATE; a_portfolio: RP_PORTFOLIO; a_currency: RP_CURRENCY): RP_MONIES
			-- Traded cash value of `a_portfolio' on `a_as_of_date' in `a_currency'
		require
			a_as_of_date_not_void: a_as_of_date /= Void
			a_accounting_date_not_void: a_accounting_date /= Void
			a_portfolio_not_void: a_portfolio /= Void
			a_currency_not_void: a_currency /= Void
		deferred
		ensure
			not_void: Result /= Void
			same_currency: Result.currency.is_same (a_currency)
		end

	portfolio_settled_cash_value_as_of (a_as_of_date: ROSE_DATE; a_accounting_date: ROSE_DATE; a_portfolio: RP_PORTFOLIO; a_currency: RP_CURRENCY): RP_MONIES
			-- Settled cash value of `a_portfolio' on `a_as_of_date' in `a_currency'
		require
			a_as_of_date_not_void: a_as_of_date /= Void
			a_accounting_date_not_void: a_accounting_date /= Void
			a_portfolio_not_void: a_portfolio /= Void
			a_currency_not_void: a_currency /= Void
		deferred
		ensure
			not_void: Result /= Void
			same_currency: Result.currency.is_same (a_currency)
		end

	portfolio_equity_value_as_of (a_as_of_date: ROSE_DATE; a_accounting_date: ROSE_DATE; a_portfolio: RP_PORTFOLIO; a_currency: RP_CURRENCY): RP_MONIES
			-- Equity value of `a_portfolio' on `a_as_of_date' in `a_currency'
		require
			a_as_of_date_not_void: a_as_of_date /= Void
			a_accounting_date_not_void: a_accounting_date /= Void
			a_portfolio_not_void: a_portfolio /= Void
			a_currency_not_void: a_currency /= Void
		deferred
		ensure
			not_void: Result /= Void
			same_currency: Result.currency.is_same (a_currency)
		end

	portfolio_ffx_value_as_of (a_as_of_date: ROSE_DATE; a_accounting_date: ROSE_DATE; a_portfolio: RP_PORTFOLIO; a_currency: RP_CURRENCY): RP_MONIES
			-- FFX Net value of `a_portfolio' on `a_as_of_date' in `a_currency'
		deferred
	end

feature -- Access (Account with specific as of date)

	account_value_as_of (a_as_of_date: ROSE_DATE; a_accounting_date: ROSE_DATE; a_account: RP_INVESTMENT_ACCOUNT; a_currency: RP_CURRENCY): RP_MONIES
			-- Value of `a_account' on `a_as_of_date' in `a_currency'
		require
			a_as_of_date_not_void: a_as_of_date /= Void
			a_account_not_void: a_account /= Void
			a_currency_not_void: a_currency /= Void
		deferred
		ensure
			not_void: Result /= Void
			same_currency: Result.currency.is_same (a_currency)
		end

	account_cash_value_as_of (a_as_of_date: ROSE_DATE; a_accounting_date: ROSE_DATE; a_account: RP_INVESTMENT_ACCOUNT; a_currency: RP_CURRENCY): RP_MONIES
			-- Cash value of `a_account' on `a_as_of_date' in `a_currency'
		require
			a_as_of_date_not_void: a_as_of_date /= Void
			a_account_not_void: a_account /= Void
			a_currency_not_void: a_currency /= Void
		deferred
		ensure
			not_void: Result /= Void
			same_currency: Result.currency.is_same (a_currency)
		end

	account_equity_value_as_of (a_as_of_date: ROSE_DATE; a_accounting_date: ROSE_DATE; a_account: RP_INVESTMENT_ACCOUNT; a_currency: RP_CURRENCY): RP_MONIES
			-- Equity value of `a_account' on `a_as_of_date' in `a_currency'
		require
			a_as_of_date_not_void: a_as_of_date /= Void
			a_account_not_void: a_account /= Void
			a_currency_not_void: a_currency /= Void
		deferred
		ensure
			not_void: Result /= Void
			same_currency: Result.currency.is_same (a_currency)
		end

	account_ffx_value_as_of (a_as_of_date: ROSE_DATE; a_accounting_date: ROSE_DATE; a_account: RP_INVESTMENT_ACCOUNT; a_currency: RP_CURRENCY): RP_MONIES
			-- FFX Net value of `a_account' on `a_as_of_date' in `a_currency'
		deferred
		end

end
