note
	description: "Access to computed positions from Northern Trust data feed"
	author: "Sami Ben Grine"
	last_editor: "$Author: sm $"
	date: "$Date: 2015/01/15 $"
	revision: "$Revision: #10 $"
	path: "$File: //rose/source/kernel/expert_system/rp_positions_system_implementation.e $"

class RP_POSITIONS_SYSTEM_IMPLEMENTATION

inherit

	RP_POSITIONS_SYSTEM

create
	{RP_INVESTMENT_MANAGER} default_create

feature -- Access (Portfolio beginning of day positions)

	NorthernTrust_positions_migration_date: attached ROSE_DATE
			-- Date when we switched to NT for positions
		once
			Result := Source.Northerntrust_positions_migration_date
		end

	portfolio_positions  (a_accounting_date: ROSE_DATE; a_portfolio: RP_PORTFOLIO): ROSE_LINEAR_ARRAY[POSITION[RP_PORTFOLIO]]
			-- <Precursor>
		do
			create Result.make_empty
			Result.append_linear (portfolio_cash_positions (a_accounting_date, a_portfolio))
			Result.append_linear (portfolio_non_cash_positions (a_accounting_date, a_portfolio))
			Result.append_linear (portfolio_open_ffx_positions (a_accounting_date, a_portfolio))
		end

	portfolio_cash_positions (a_accounting_date: ROSE_DATE; a_portfolio: RP_PORTFOLIO): ROSE_LINEAR_ARRAY[RP_PORTFOLIO_CASH_POSITION]
			-- <Precursor>
		local
			l_gid: ROSE_DATE
			l_rows: ROSE_LINEAR_ARRAY [NT_PORTFOLIO_CASH]
			l_position: RP_PORTFOLIO_CASH_POSITION
			l_row: NT_PORTFOLIO_CASH
		do
			create Result.make_empty
			l_gid := source.portfolio_cash_most_recent_global_investment_date (a_accounting_date, a_portfolio.code, source.accounting)
			if (l_gid /= Void and then not l_gid.is_min_date) then
				l_rows := source.portfolio_cash_as_of (l_gid, a_accounting_date, a_portfolio.code)
				from
					l_rows.start
				until
					l_rows.after
				loop
					l_row := l_rows.item_for_iteration
					create l_position.make (l_row)
					Result.extend (l_position)
					l_rows.forth
				end
			end
		end

	portfolio_settled_cash_positions (a_global_investment_date: ROSE_DATE; a_portfolio: RP_PORTFOLIO): ROSE_LINEAR_ARRAY[RP_PORTFOLIO_CASH_POSITION]
			-- <Precursor>
		local
			l_rows: ROSE_LINEAR_ARRAY [NT_PORTFOLIO_CASH]
			l_position: RP_PORTFOLIO_CASH_POSITION
			l_row: NT_PORTFOLIO_CASH
			l_hash: DS_HASH_SET [RP_PORTFOLIO_CASH_POSITION]
			l_tester: KL_AGENT_EQUALITY_TESTER [RP_PORTFOLIO_CASH_POSITION]
		do
			create Result.make_empty
			if (a_global_investment_date /= Void and then not a_global_investment_date.is_min_date) then
				l_rows := source.portfolio_settled_cash_entries_as_of (a_global_investment_date, a_portfolio.code)

				create l_hash.make_default
				create l_tester.make(agent (a_pc1, a_pc2: RP_PORTFOLIO_CASH_POSITION): BOOLEAN
				require a_pc1_not_void: a_pc1 /= Void a_pc2_not_void: a_pc2 /= Void do Result := a_pc1.iso_currency_code ~ a_pc2.iso_currency_code end)
				l_hash.set_equality_tester (l_tester)

				from
					l_rows.start
				until
					l_rows.after
				loop
					l_row := l_rows.item_for_iteration
					create l_position.make (l_row)

					if (not l_hash.has(l_position)) then
						l_hash.force (l_position)
					else
						if l_hash.item (l_position).accounting_date < l_position.accounting_date then
							l_hash.force (l_position)
						end
					end
					l_rows.forth
				end
				create Result.make_from_array (l_hash.to_array)
			end
		end

	portfolio_non_cash_positions (a_accounting_date: ROSE_DATE; a_portfolio: RP_PORTFOLIO): ROSE_LINEAR_ARRAY[RP_PORTFOLIO_NON_CASH_POSITION]
			-- <Precursor>
		local
			l_gid: ROSE_DATE
			l_rows: ROSE_LINEAR_ARRAY [NT_PORTFOLIO_NON_CASH]
			l_position: RP_PORTFOLIO_NON_CASH_POSITION
		do
			create Result.make_empty
			l_gid := source.portfolio_non_cash_most_recent_global_investment_date (a_accounting_date, a_portfolio.code, source.accounting)
			if (l_gid /= Void and then not l_gid.is_min_date) then
				l_rows := source.portfolio_non_cash_as_of (l_gid, a_accounting_date, a_portfolio.code)
				from
					l_rows.start
				until
					l_rows.after
				loop
					if attached {NT_PORTFOLIO_NON_CASH} l_rows.item_for_iteration as l_row
					and then attached {STRING} l_row.brimsym as l_brimsym
					and then attached {RP_INSTRUMENT} manager.reference_system.instrument_from_brimsym (l_row.brimsym) then
						if l_row.ref_uid/=0 then
							create l_position.make (l_row)
							Result.extend (l_position)
						else
							warn (generator,"Could not load portfolio non cash position $1 $2 $3 $4 $5 $6 $7 $8",
							<<l_row.global_investment_date, l_row.update_mode, l_row.portfolio_code, l_row.ref_uid, l_row.accounting_date, l_row.contract_side, l_row.accounting_basis, l_row.fm_sec_id>>)
						end
					end
					l_rows.forth
				end
			end
		end

	portfolio_open_ffx_positions (a_accounting_date: ROSE_DATE; a_portfolio: RP_PORTFOLIO): ROSE_LINEAR_ARRAY[RP_PORTFOLIO_OPEN_FFX_POSITION]
			-- <Precursor>
		local
			l_gid: ROSE_DATE
			l_rows: ROSE_LINEAR_ARRAY [NT_PORTFOLIO_OPEN_FFX]
			l_position: RP_PORTFOLIO_OPEN_FFX_POSITION
			l_row: NT_PORTFOLIO_OPEN_FFX
		do
			create Result.make_empty
			l_gid := source.portfolio_open_ffx_most_recent_global_investment_date (a_accounting_date, a_portfolio.code, source.accounting)
			if (l_gid /= Void and then not l_gid.is_min_date) then
				l_rows := source.portfolio_open_ffx_as_of (l_gid, a_accounting_date, a_portfolio.code)
				from
					l_rows.start
				until
					l_rows.after
				loop
					l_row := l_rows.item_for_iteration
					create l_position.make (l_row)
					Result.extend (l_position)
					l_rows.forth
				end
			end
		end

	portfolio_settled_cash_position_for_currency (a_accounting_date: ROSE_DATE; a_portfolio: RP_PORTFOLIO; a_currency: RP_CURRENCY): RP_PORTFOLIO_CASH_POSITION
			-- <Precursor>
		local
			l_gid: ROSE_DATE
		do
			l_gid := source.portfolio_cash_most_recent_global_investment_date (a_accounting_date, a_portfolio.code, source.accounting)
			if (l_gid /= Void and then not l_gid.is_min_date)
			and then attached {NT_PORTFOLIO_CASH} source.portfolio_settled_cash_for_currency_as_of (l_gid, a_accounting_date, a_portfolio.code, a_currency.iso) as l_row then
				create Result.make (l_row)
			end
		end

feature -- Access (Portfolio beginning of day valuation)

	portfolio_value (a_accounting_date: ROSE_DATE; a_portfolio: RP_PORTFOLIO; a_currency: RP_CURRENCY): RP_MONIES
			-- <Precursor>
		local
			l_gid: ROSE_DATE
		do
			l_gid := source.portfolio_most_recent_global_investment_date (a_accounting_date, a_portfolio.code, source.accounting)
			if (l_gid /= Void and then not l_gid.is_min_date) then
				Result := portfolio_value_as_of (l_gid, a_accounting_date, a_portfolio, a_currency)
			else
				Result := a_currency.monies (create {RP_DECIMAL}.make_undefined)
			end
		end

	portfolio_cash_value (a_accounting_date: ROSE_DATE; a_portfolio: RP_PORTFOLIO; a_currency: RP_CURRENCY): RP_MONIES
			-- <Precursor>
		local
			l_gid: ROSE_DATE
		do
			l_gid := source.portfolio_cash_most_recent_global_investment_date (a_accounting_date, a_portfolio.code, source.accounting)
			if (l_gid /= Void and then not l_gid.is_min_date) then
				Result := portfolio_cash_value_as_of (l_gid, a_accounting_date, a_portfolio, a_currency)
			else
				Result := a_currency.monies (create {RP_DECIMAL}.make_undefined)
			end
		end

	portfolio_settled_cash_value (a_accounting_date: ROSE_DATE; a_portfolio: RP_PORTFOLIO; a_currency: RP_CURRENCY): RP_MONIES
			-- <Precursor>
		local
			l_gid: ROSE_DATE
		do
			l_gid := source.portfolio_cash_most_recent_global_investment_date (a_accounting_date, a_portfolio.code, source.accounting)
			if (l_gid /= Void and then not l_gid.is_min_date) then
				Result := portfolio_settled_cash_value_as_of (l_gid, a_accounting_date, a_portfolio, a_currency)
			else
				Result := a_currency.monies (create {RP_DECIMAL}.make_undefined)
			end
		end

	portfolio_equity_value (a_accounting_date: ROSE_DATE; a_portfolio: RP_PORTFOLIO; a_currency: RP_CURRENCY): RP_MONIES
			-- <Precursor>
		local
			l_gid: ROSE_DATE
		do
			l_gid := source.portfolio_non_cash_most_recent_global_investment_date (a_accounting_date, a_portfolio.code, source.accounting)
			if (l_gid /= Void and then not l_gid.is_min_date) then
				Result := portfolio_equity_value_as_of (l_gid, a_accounting_date, a_portfolio, a_currency)
			else
				Result := a_currency.monies (create {RP_DECIMAL}.make_undefined)
			end
		end

	portfolio_ffx_value (a_accounting_date: ROSE_DATE; a_portfolio: RP_PORTFOLIO; a_currency: RP_CURRENCY): RP_MONIES
			-- <Precursor>
		local
			l_gid: ROSE_DATE
		do
			l_gid := source.portfolio_open_ffx_most_recent_global_investment_date (a_accounting_date, a_portfolio.code, source.accounting)
			if (l_gid /= Void and then not l_gid.is_min_date) then
				Result := portfolio_ffx_value_as_of (l_gid, a_accounting_date, a_portfolio, a_currency)
			else
				Result := a_currency.monies (create {RP_DECIMAL}.make_undefined)
			end
		end

    latest_portfolio_data_date( a_accounting_date: ROSE_DATE; a_portfolio: RP_PORTFOLIO ): ROSE_DATE
			-- <Precursor>
		do
			Result := source.portfolio_most_recent_global_investment_date (a_accounting_date, a_portfolio.code, source.accounting)
		end

	price_of (a_latest_portfolio_data_date: ROSE_DATE; a_portfolio: RP_PORTFOLIO; a_position: RP_QUANTITY; a_currency: RP_CURRENCY; a_date: ROSE_DATE): RP_DECIMAL
			-- <Precursor>
		local
			l_position : like portfolio_position_for_instrument
			l_fx: RP_DECIMAL
		do
			l_position := portfolio_position_for_instrument (a_latest_portfolio_data_date, a_date, a_portfolio, a_position.instrument)
			if l_position /= Void and then l_position.accounting_date /= Void and l_position.global_currency /= Void and not l_position.total_quantity.is_zero then
				l_fx := manager.performance_system.fx.spot_rate (l_position.global_currency, a_currency, l_position.accounting_date)
				if l_fx /= Void and then l_fx.is_defined then
					Result := l_position.market_value_global_base.amount * l_fx / l_position.total_quantity.amount
				end
			end
		end

	portfolio_position_for_instrument (a_latest_portfolio_data_date: ROSE_DATE; a_accounting_date: ROSE_DATE; a_portfolio: RP_PORTFOLIO; a_instrument: RP_INSTRUMENT): RP_PORTFOLIO_NON_CASH_POSITION
			-- <Precursor>
		local
			l_position: NT_PORTFOLIO_NON_CASH
		do
			if (a_latest_portfolio_data_date /= Void) then
				if not a_instrument.is_currency
				and then not a_instrument.is_forward then
					l_position := source.portfolio_non_cash_for_brimsym_as_of (a_latest_portfolio_data_date, a_accounting_date, a_portfolio.code, a_instrument.brimsym)
					if l_position /= Void then
						create Result.make (l_position)
					end
				end
			end
		end

feature -- Access (Portfolio beginning of day positions)

	account_positions  (a_accounting_date: ROSE_DATE; a_account: RP_INVESTMENT_ACCOUNT): ROSE_LINEAR_ARRAY[POSITION[RP_INVESTMENT_ACCOUNT]]
			-- <Precursor>
		do
			create Result.make_empty
			Result.append_linear (account_cash_positions (a_accounting_date, a_account))
			Result.append_linear (account_non_cash_positions (a_accounting_date, a_account))
			Result.append_linear (account_open_ffx_positions (a_accounting_date, a_account))
		end

	account_cash_positions (a_accounting_date: ROSE_DATE; a_account: RP_INVESTMENT_ACCOUNT): ROSE_LINEAR_ARRAY[RP_ACCOUNT_CASH_POSITION]
			-- <Precursor>
		local
			l_gid: ROSE_DATE
			l_rows: ROSE_LINEAR_ARRAY [NT_CASH]
			l_position: RP_ACCOUNT_CASH_POSITION
			l_row: NT_CASH
		do
			create Result.make_empty
			l_gid := source.account_cash_most_recent_global_investment_date (a_accounting_date, a_account.code)
			if (l_gid /= Void and then not l_gid.is_min_date) then
				l_rows := source.account_cash_as_of (l_gid, a_accounting_date, a_account.code)
				from
					l_rows.start
				until
					l_rows.after
				loop
					l_row := l_rows.item_for_iteration
					create l_position.make (l_row)
					Result.extend (l_position)
					l_rows.forth
				end
			end
		end

	account_non_cash_positions (a_accounting_date: ROSE_DATE; a_account:RP_INVESTMENT_ACCOUNT): ROSE_LINEAR_ARRAY[RP_ACCOUNT_NON_CASH_POSITION]
			-- <Precursor>
		local
			l_gid: ROSE_DATE
			l_rows: ROSE_LINEAR_ARRAY [NT_NON_CASH]
			l_position: RP_ACCOUNT_NON_CASH_POSITION
			l_instrument: RP_INSTRUMENT
			l_row: NT_NON_CASH
		do
			create Result.make_empty
			l_gid := source.account_cash_most_recent_global_investment_date (a_accounting_date, a_account.code)
			if (l_gid /= Void and then not l_gid.is_min_date) then
				l_rows := source.account_non_cash_as_of (l_gid, a_accounting_date, a_account.code)
				from
					l_rows.start
				until
					l_rows.after
				loop
					l_row := l_rows.item_for_iteration
					l_instrument := manager.reference_system.instrument_from_brimsym (l_row.brimsym)
					if l_instrument /= Void then
						if l_row.ref_uid /= 0 then
							create l_position.make (l_row)
							Result.extend (l_position)
						else
							warn (generator,"Could not load account non cash position $1 $2 $3 $4 $5 $6 $7 $8",
							<<l_row.global_investment_date, l_row.update_mode, l_row.account_code, l_row.ref_uid, l_row.accounting_date, l_row.contract_side, l_row.accounting_basis, l_row.fm_sec_id>>)
						end
					end
					l_rows.forth
				end
			end
		end

	account_open_ffx_positions (a_accounting_date: ROSE_DATE; a_account: RP_INVESTMENT_ACCOUNT): ROSE_LINEAR_ARRAY[RP_ACCOUNT_OPEN_FFX_POSITION]
			-- <Precursor>
		local
			l_gid: ROSE_DATE
			l_rows: ROSE_LINEAR_ARRAY [NT_OPEN_FFX]
			l_position: RP_ACCOUNT_OPEN_FFX_POSITION
			l_row: NT_OPEN_FFX
		do
			create Result.make_empty
			l_gid := source.account_cash_most_recent_global_investment_date (a_accounting_date, a_account.code)
			if (l_gid /= Void and then not l_gid.is_min_date) then
				l_rows := source.account_open_ffx_as_of (l_gid, a_accounting_date, a_account.code)
				from
					l_rows.start
				until
					l_rows.after
				loop
					l_row := l_rows.item_for_iteration
					create l_position.make(l_row)
					Result.extend (l_position)
					l_rows.forth
				end
			end
		end

feature -- Access (Account beginning of day positions)

	account_value (a_accounting_date: ROSE_DATE; a_account: RP_INVESTMENT_ACCOUNT; a_currency: RP_CURRENCY): RP_MONIES
			-- <Precursor>
		local
			l_gid: ROSE_DATE
		do
			l_gid := source.account_cash_most_recent_global_investment_date (a_accounting_date, a_account.code)
			if (l_gid /= Void and then not l_gid.is_min_date) then
				Result := account_value_as_of (l_gid, a_accounting_date, a_account, a_currency)
			else
				Result := a_currency.monies (create {RP_DECIMAL}.make_undefined)
			end
		end

	account_cash_value (a_accounting_date: ROSE_DATE; a_account: RP_INVESTMENT_ACCOUNT; a_currency: RP_CURRENCY): RP_MONIES
			-- <Precursor>
		local
			l_gid: ROSE_DATE
		do
			l_gid := source.account_cash_most_recent_global_investment_date (a_accounting_date, a_account.code)
			if (l_gid /= Void and then not l_gid.is_min_date) then
				Result := account_cash_value_as_of (l_gid, a_accounting_date, a_account, a_currency)
			else
				Result := a_currency.monies (create {RP_DECIMAL}.make_undefined)
			end
		end

	account_equity_value (a_accounting_date: ROSE_DATE; a_account: RP_INVESTMENT_ACCOUNT; a_currency: RP_CURRENCY): RP_MONIES
			-- <Precursor>
		local
			l_gid: ROSE_DATE
		do
			l_gid := source.account_cash_most_recent_global_investment_date (a_accounting_date, a_account.code)
			if (l_gid /= Void and then not l_gid.is_min_date) then
				Result := account_equity_value_as_of (l_gid, a_accounting_date, a_account, a_currency)
			else
				Result := a_currency.monies (create {RP_DECIMAL}.make_undefined)
			end
		end

	account_ffx_value (a_accounting_date: ROSE_DATE; a_account: RP_INVESTMENT_ACCOUNT; a_currency: RP_CURRENCY): RP_MONIES
			-- <Precursor>
		local
			l_gid: ROSE_DATE
		do
			l_gid := source.account_cash_most_recent_global_investment_date (a_accounting_date, a_account.code)
			if (l_gid /= Void and then not l_gid.is_min_date) then
				Result := account_ffx_value_as_of (l_gid, a_accounting_date, a_account, a_currency)
			else
				Result := a_currency.monies (create {RP_DECIMAL}.make_undefined)
			end
		end

feature -- Access (Portfolio with specific as_of_date)

	portfolio_value_as_of (a_as_of_date: ROSE_DATE; a_accounting_date: ROSE_DATE; a_portfolio: RP_PORTFOLIO; a_currency: RP_CURRENCY): RP_MONIES
			-- <Precursor>
		local
			l_cash_value: like portfolio_cash_value
			l_portfolio_equity_value: like portfolio_equity_value
			l_open_ffx_value: like portfolio_ffx_value
		do
			l_cash_value := portfolio_cash_value_as_of (a_as_of_date, a_accounting_date, a_portfolio, a_currency)
			l_portfolio_equity_value := portfolio_equity_value_as_of (a_as_of_date, a_accounting_date, a_portfolio, a_currency)
			l_open_ffx_value := portfolio_ffx_value_as_of (a_as_of_date, a_accounting_date, a_portfolio, a_currency)
			Result := a_currency.monies (l_cash_value.amount + l_portfolio_equity_value.amount + l_open_ffx_value.amount)
		end

	portfolio_cash_value_as_of (a_as_of_date: ROSE_DATE; a_accounting_date: ROSE_DATE; a_portfolio: RP_PORTFOLIO; a_currency: RP_CURRENCY): RP_MONIES
			-- <Precursor>
		local
			l_cash_values: ROSE_LINEAR_ARRAY[NT_PORTFOLIO_CASH]
			l_total: CELL [RP_DECIMAL]
		do
			l_cash_values := source.portfolio_cash_as_of (a_as_of_date, a_accounting_date, a_portfolio.code)
			create l_total.put (create {RP_DECIMAL}.make_zero)
			l_cash_values.do_all (agent add_portfolio_cash (l_total, ?, a_currency))
			Result := a_currency.monies (l_total.item)
		end

	portfolio_settled_cash_value_as_of (a_as_of_date: ROSE_DATE; a_accounting_date: ROSE_DATE; a_portfolio: RP_PORTFOLIO; a_currency: RP_CURRENCY): RP_MONIES
			-- <Precursor>
		local
			l_cash_values: ROSE_LINEAR_ARRAY[NT_PORTFOLIO_CASH]
			l_total: CELL [RP_DECIMAL]
		do
			if a_as_of_date < source.northerntrust_positions_migration_date then
				error.raise ("Not supported")
			else
				l_cash_values := source.portfolio_settled_cash_as_of (a_as_of_date, a_accounting_date, a_portfolio.code)
				create l_total.put (create {RP_DECIMAL}.make_zero)
				l_cash_values.do_all (agent add_portfolio_cash (l_total, ?, a_currency))
				Result := a_currency.monies (l_total.item)
			end
		end

	portfolio_equity_value_as_of (a_as_of_date: ROSE_DATE; a_accounting_date: ROSE_DATE; a_portfolio: RP_PORTFOLIO; a_currency: RP_CURRENCY): RP_MONIES
			-- <Precursor>
		local
			l_non_cash_values: ROSE_LINEAR_ARRAY[NT_PORTFOLIO_NON_CASH]
			l_total: CELL [RP_DECIMAL]
			l_dailyvalues: RP_PERFORMANCE_DAILY_VALUES
		do
			if a_as_of_date < source.northerntrust_positions_migration_date then
				l_dailyvalues := manager.legacy.performance_system.portfolio_dailyvalues_at (a_portfolio, a_currency, a_accounting_date)
				if l_dailyvalues /= Void then
					Result := l_dailyvalues.equity_value
				else
					Result := a_currency.monies (create {RP_DECIMAL}.make_undefined)
				end
			else
				l_non_cash_values := source.portfolio_non_cash_as_of (a_as_of_date, a_accounting_date, a_portfolio.code)
				create l_total.put (create {RP_DECIMAL}.make_zero)
				l_non_cash_values.do_all (agent add_portfolio_non_cash (l_total, ?, a_currency))
				Result := a_currency.monies (l_total.item)
			end
		end

	portfolio_ffx_value_as_of (a_as_of_date: ROSE_DATE; a_accounting_date: ROSE_DATE; a_portfolio: RP_PORTFOLIO; a_currency: RP_CURRENCY): RP_MONIES
			-- <Precursor>
		local
			l_open_ffx_values: ROSE_LINEAR_ARRAY[NT_PORTFOLIO_OPEN_FFX]
			l_total: CELL [RP_DECIMAL]
		do
			l_open_ffx_values := source.portfolio_open_ffx_as_of (a_as_of_date, a_accounting_date, a_portfolio.code)
			create l_total.put (create {RP_DECIMAL}.make_zero)
			l_open_ffx_values.do_all (agent add_portfolio_open_ffx (l_total, ?, a_currency, a_accounting_date))
			Result := a_currency.monies (l_total.item)
		end

feature -- Access (Account with specific as of date)

	account_value_as_of (a_as_of_date: ROSE_DATE; a_accounting_date: ROSE_DATE; a_account: RP_INVESTMENT_ACCOUNT; a_currency: RP_CURRENCY): RP_MONIES
			-- <Precursor>
		local
			l_cash_value: like account_cash_value
			l_portfolio_equity_value: like account_equity_value
			l_open_ffx_value: like account_ffx_value
		do
			l_cash_value := account_cash_value_as_of (a_as_of_date, a_accounting_date, a_account, a_currency)
			l_portfolio_equity_value := account_equity_value_as_of (a_as_of_date, a_accounting_date, a_account, a_currency)
			l_open_ffx_value := account_ffx_value_as_of (a_as_of_date, a_accounting_date, a_account, a_currency)
			Result := a_currency.monies (l_cash_value.amount + l_portfolio_equity_value.amount + l_open_ffx_value.amount)
		end

	account_cash_value_as_of (a_as_of_date: ROSE_DATE; a_accounting_date: ROSE_DATE; a_account: RP_INVESTMENT_ACCOUNT; a_currency: RP_CURRENCY): RP_MONIES
			-- <Precursor>
		local
			l_cash_values: ROSE_LINEAR_ARRAY[NT_CASH]
			l_total: CELL [RP_DECIMAL]
		do
			l_cash_values := source.account_cash_as_of (a_as_of_date, a_accounting_date, a_account.code)
			create l_total.put (create {RP_DECIMAL}.make_zero)
			l_cash_values.do_all (agent add_cash (l_total, ?, a_currency))
			Result := a_currency.monies (l_total.item)
		end

	account_equity_value_as_of (a_as_of_date: ROSE_DATE; a_accounting_date: ROSE_DATE; a_account: RP_INVESTMENT_ACCOUNT; a_currency: RP_CURRENCY): RP_MONIES
			-- <Precursor>
		local
			l_non_cash_values: ROSE_LINEAR_ARRAY[NT_NON_CASH]
			l_total: CELL [RP_DECIMAL]
		do
			l_non_cash_values := source.account_non_cash_as_of (a_as_of_date, a_accounting_date, a_account.code)
			create l_total.put (create {RP_DECIMAL}.make_zero)
			l_non_cash_values.do_all (agent add_non_cash (l_total, ?, a_currency))
			Result := a_currency.monies (l_total.item)
		end

	account_ffx_value_as_of (a_as_of_date: ROSE_DATE; a_accounting_date: ROSE_DATE; a_account: RP_INVESTMENT_ACCOUNT; a_currency: RP_CURRENCY): RP_MONIES
			-- <Precursor>
		local
			l_open_ffx_values: ROSE_LINEAR_ARRAY[NT_OPEN_FFX]
			l_total: CELL [RP_DECIMAL]
		do
			l_open_ffx_values := source.account_open_ffx_as_of (a_as_of_date, a_accounting_date, a_account.code)
			create l_total.put (create {RP_DECIMAL}.make_zero)
			l_open_ffx_values.do_all (agent add_open_ffx (l_total, ?, a_currency, a_accounting_date))
			Result := a_currency.monies (l_total.item)
		end

feature {NONE} -- Implementation

	source: attached RP_POSITIONS_DATA_SOURCE
			-- Positions data provider
		once
			create Result
		ensure
			not_void: Result /= Void
		end

	add_portfolio_cash (a_total: CELL [RP_DECIMAL]; a_position: NT_PORTFOLIO_CASH; a_currency: RP_CURRENCY) is
			-- Add value of `a_position' in `a_currency' to `a_total'
		require
			a_total_not_void: a_total /= Void
			a_total_item_not_void: a_total.item /= Void
			a_position_not_void: a_position /= Void
			a_currency_not_void: a_currency /= Void
		local
			l_currency: RP_CURRENCY
			l_value: RP_DECIMAL
			l_fx: RP_DECIMAL
			l_previous_end_of_day: ROSE_DATE;
		do
			l_value := a_total.item.undefined_value
			if a_total.item.is_defined and a_position.accounting_date /= Void and a_position.iso_currency_code /= Void and a_position.total_quantity /= Void and then
				 a_position.total_quantity /= Void and then a_position.total_quantity.is_defined then
				l_currency := manager.reference_system.currency_from_iso_code (a_position.iso_currency_code)
				if l_currency /= Void then
					create l_previous_end_of_day.make_by_date (a_position.accounting_date)
					l_previous_end_of_day.add_n_days (-1)
					l_fx := manager.performance_system.fx.spot_rate (l_currency, a_currency, l_previous_end_of_day)
					if l_fx /= Void and then l_fx.is_defined then
						l_value := a_total.item + a_position.total_quantity * l_fx
					end
				end
			end
			a_total.put (l_value)
		ensure
			a_total_item_not_void: a_total.item /= Void
		end

	add_cash (a_total: CELL [RP_DECIMAL]; a_position: NT_CASH; a_currency: RP_CURRENCY) is
			-- Add value of `a_position' in `a_currency' to `a_total'
		require
			a_total_not_void: a_total /= Void
			a_total_item_not_void: a_total.item /= Void
			a_position_not_void: a_position /= Void
			a_currency_not_void: a_currency /= Void
		local
			l_currency: RP_CURRENCY
			l_value: RP_DECIMAL
			l_fx: RP_DECIMAL
			l_previous_end_of_day: ROSE_DATE;
		do
			l_value := a_total.item.undefined_value
			if a_total.item.is_defined and a_position.accounting_date /= Void and a_position.iso_currency_code /= Void and a_position.total_quantity /= Void and then a_position.total_quantity.is_defined then
				l_currency := manager.reference_system.currency_from_iso_code (a_position.iso_currency_code)
				if l_currency /= Void then
					create l_previous_end_of_day.make_by_date (a_position.accounting_date)
					l_previous_end_of_day.add_n_days (-1)
					l_fx := manager.performance_system.fx.spot_rate (l_currency, a_currency, l_previous_end_of_day)
					if l_fx /= Void and then l_fx.is_defined then
						l_value := a_total.item + a_position.total_quantity * l_fx
					end
				end
			end
			a_total.put (l_value)
		ensure
			a_total_item_not_void: a_total.item /= Void
		end

	add_portfolio_non_cash (a_total: CELL [RP_DECIMAL]; a_position: NT_PORTFOLIO_NON_CASH; a_currency: RP_CURRENCY) is
			-- Add value of `a_position' in `a_currency' to `a_total'
		require
			a_total_not_void: a_total /= Void
			a_total_item_not_void: a_total.item /= Void
			a_position_not_void: a_position /= Void
			a_currency_not_void: a_currency /= Void
		local
			l_currency: RP_CURRENCY
			l_value: RP_DECIMAL
			l_fx: RP_DECIMAL
			l_previous_end_of_day: ROSE_DATE
		do
			l_value := a_total.item.undefined_value
			if a_total.item.is_defined
			and then a_position.accounting_date /= Void
			and then a_position.market_value_global_base /= Void
			and then a_position.market_value_global_base.is_defined then
				l_currency := manager.reference_system.currency_from_iso_code ("USD")
				if l_currency /= Void then
					create l_previous_end_of_day.make_by_date (a_position.accounting_date)
					l_previous_end_of_day.add_n_days (-1)
					l_fx := manager.performance_system.fx.spot_rate (l_currency, a_currency, l_previous_end_of_day)
					if l_fx /= Void and then l_fx.is_defined then
						l_value := a_total.item + a_position.market_value_global_base * l_fx
					end
				end
			end
			a_total.put (l_value)
		ensure
			a_total_item_not_void: a_total.item /= Void
		end

	add_non_cash (a_total: CELL [RP_DECIMAL]; a_position: NT_NON_CASH; a_currency: RP_CURRENCY) is
			-- Add value of `a_position' in `a_currency' to `a_total'
		require
			a_total_not_void: a_total /= Void
			a_total_item_not_void: a_total.item /= Void
			a_position_not_void: a_position /= Void
			a_currency_not_void: a_currency /= Void
		local
			l_currency: RP_CURRENCY
			l_value: RP_DECIMAL
			l_fx: RP_DECIMAL
			l_previous_end_of_day: ROSE_DATE
		do
			l_value := a_total.item.undefined_value
			if a_total.item.is_defined
			and then a_position.accounting_date /= Void
			and then a_position.market_value_global_base /= Void
			and then a_position.market_value_global_base.is_defined then
				l_currency := manager.reference_system.currency_from_iso_code ("USD")
				if l_currency /= Void then
					create l_previous_end_of_day.make_by_date (a_position.accounting_date)
					l_previous_end_of_day.add_n_days (-1)
					l_fx := manager.performance_system.fx.spot_rate (l_currency, a_currency, l_previous_end_of_day)
					if l_fx /= Void and then l_fx.is_defined then
						l_value := a_total.item + a_position.market_value_global_base * l_fx
					end
				end
			end
			a_total.put (l_value)
		ensure
			a_total_item_not_void: a_total.item /= Void
		end

	add_portfolio_open_ffx (a_total: CELL [RP_DECIMAL]; a_position: NT_PORTFOLIO_OPEN_FFX; a_currency: RP_CURRENCY; a_accounting_date: ROSE_DATE) is
			-- Add value of `a_position' in `a_currency' to `a_total'
		require
			a_total_not_void: a_total /= Void
			a_total_item_not_void: a_total.item /= Void
			a_position_not_void: a_position /= Void
			a_currency_not_void: a_currency /= Void
		local
			l_currency: RP_CURRENCY
			l_value: RP_DECIMAL
			l_fx: RP_DECIMAL
			l_previous_end_of_day: ROSE_DATE
		do
			l_value := a_total.item.undefined_value
			if a_total.item.is_defined and a_position.global_investment_date /= Void and a_position.net_currency /= Void and then a_position.net_market_value /= Void and then a_position.net_market_value.is_defined then
				l_currency := manager.reference_system.currency_from_iso_code (a_position.net_currency)
				if l_currency /= Void then
					create l_previous_end_of_day.make_by_date (a_accounting_date)
					l_previous_end_of_day.add_n_days (-1)
					l_fx := manager.performance_system.fx.spot_rate (l_currency, a_currency, l_previous_end_of_day)
					if l_fx /= Void and then l_fx.is_defined then
						l_value := a_total.item + a_position.net_market_value * l_fx
					end
				end
			end
			a_total.put (l_value)
		ensure
			a_total_item_not_void: a_total.item /= Void
		end

	add_open_ffx (a_total: CELL [RP_DECIMAL]; a_position: NT_OPEN_FFX; a_currency: RP_CURRENCY; a_accounting_date: ROSE_DATE) is
			-- Add value of `a_position' in `a_currency' to `a_total'
		require
			a_total_not_void: a_total /= Void
			a_total_item_not_void: a_total.item /= Void
			a_position_not_void: a_position /= Void
			a_currency_not_void: a_currency /= Void
		local
			l_currency: RP_CURRENCY
			l_value: RP_DECIMAL
			l_fx: RP_DECIMAL
			l_previous_end_of_day: ROSE_DATE
		do
			l_value := a_total.item.undefined_value
			if a_total.item.is_defined and a_position.global_investment_date /= Void and a_position.net_currency /= Void
				and then a_position.net_market_value /= Void and then  a_position.net_market_value.is_defined then
				l_currency := manager.reference_system.currency_from_iso_code (a_position.net_currency)
				if l_currency /= Void then
					create l_previous_end_of_day.make_by_date (a_accounting_date)
					l_previous_end_of_day.add_n_days (-1)
					l_fx := manager.performance_system.fx.spot_rate (l_currency, a_currency, l_previous_end_of_day)
					if l_fx /= Void and then l_fx.is_defined then
						l_value := a_total.item + a_position.net_market_value * l_fx
					end
				end
			end
			a_total.put (l_value)
		ensure
			a_total_item_not_void: a_total.item /= Void
		end

end
