note
	description: "Abstraction for Investment Manager"
	author: "Unknown"
	last_editor: "$Author: sm $"
	date: "$Date: 2014/12/10 $"
	revision: "$Revision: #5 $"
	path: "$File: //rose/source/kernel/expert_system/rp_positions_system_proxy.e $"

deferred class RP_POSITIONS_SYSTEM_PROXY

inherit
	ANY

	ROSE_SHARED_GLOBAL
		export {NONE} all end

	RP_AXAROSENBERG_AWARE
		export {NONE} all end

	RP_POSITIONS_SYSTEM

feature {RP_INVESTMENT_MANAGER} -- Initialization

	make (a_implementation: like implementation)
		-- Create with 'a_implementation'
		require
			implementation_exists: a_implementation /= Void
		do
			implementation := a_implementation
		ensure
			initialized: implementation = a_implementation
		end

feature -- Access (Portfolio beginning of day positions)

	NorthernTrust_positions_migration_date: attached ROSE_DATE is
			-- <Precursor>
		do
			Result := implementation.northerntrust_positions_migration_date
		end

	portfolio_positions  (a_accounting_date: ROSE_DATE; a_portfolio: RP_PORTFOLIO): ROSE_LINEAR_ARRAY[POSITION[RP_PORTFOLIO]]
			-- <Precursor>
		do
			Result := implementation.portfolio_positions (adjusted (a_accounting_date), a_portfolio)
		end

	portfolio_cash_positions (a_accounting_date: ROSE_DATE; a_portfolio: RP_PORTFOLIO): ROSE_LINEAR_ARRAY[RP_PORTFOLIO_CASH_POSITION]
			-- <Precursor>
		do
			Result := implementation.portfolio_cash_positions (adjusted (a_accounting_date), a_portfolio)
		end

	portfolio_settled_cash_positions (a_global_investment_date: ROSE_DATE; a_portfolio: RP_PORTFOLIO): ROSE_LINEAR_ARRAY[RP_PORTFOLIO_CASH_POSITION]
			-- <Precursor>
		do
				Result := implementation.portfolio_settled_cash_positions (a_global_investment_date, a_portfolio)
		end

	portfolio_non_cash_positions (a_accounting_date: ROSE_DATE; a_portfolio: RP_PORTFOLIO): ROSE_LINEAR_ARRAY[RP_PORTFOLIO_NON_CASH_POSITION]
			-- <Precursor>
		do
			Result := implementation.portfolio_non_cash_positions (adjusted (a_accounting_date), a_portfolio)
		end

	portfolio_open_ffx_positions (a_accounting_date: ROSE_DATE; a_portfolio: RP_PORTFOLIO): ROSE_LINEAR_ARRAY[RP_PORTFOLIO_OPEN_FFX_POSITION]
			-- <Precursor>
		do
			Result := implementation.portfolio_open_ffx_positions (adjusted (a_accounting_date), a_portfolio)
		end

	portfolio_settled_cash_position_for_currency (a_accounting_date: ROSE_DATE; a_portfolio: RP_PORTFOLIO; a_currency: RP_CURRENCY): RP_PORTFOLIO_CASH_POSITION
			-- <Precursor>
		do
			Result := implementation.portfolio_settled_cash_position_for_currency (adjusted (a_accounting_date), a_portfolio, a_currency)
		end

feature -- Access (Portfolio beginning of day valuation)

	portfolio_value (a_accounting_date: ROSE_DATE; a_portfolio: RP_PORTFOLIO; a_currency: RP_CURRENCY): RP_MONIES
			-- <Precursor>
		do
			Result := implementation.portfolio_value (adjusted (a_accounting_date), a_portfolio, a_currency)
		end

	portfolio_cash_value (a_accounting_date: ROSE_DATE; a_portfolio: RP_PORTFOLIO; a_currency: RP_CURRENCY): RP_MONIES
			-- <Precursor>
		do
			Result := implementation.portfolio_cash_value (adjusted (a_accounting_date), a_portfolio, a_currency)
		end

	portfolio_settled_cash_value (a_accounting_date: ROSE_DATE; a_portfolio: RP_PORTFOLIO; a_currency: RP_CURRENCY): RP_MONIES
			-- <Precursor>
		do
			Result := implementation.portfolio_settled_cash_value (adjusted (a_accounting_date), a_portfolio, a_currency)
		end

	portfolio_equity_value (a_accounting_date: ROSE_DATE; a_portfolio: RP_PORTFOLIO; a_currency: RP_CURRENCY): RP_MONIES
			-- <Precursor>
		do
			Result := implementation.portfolio_equity_value (adjusted (a_accounting_date), a_portfolio, a_currency)
		end

	portfolio_ffx_value (a_accounting_date: ROSE_DATE; a_portfolio: RP_PORTFOLIO; a_currency: RP_CURRENCY): RP_MONIES
			-- <Precursor>
		do
			Result := implementation.portfolio_ffx_value (adjusted (a_accounting_date), a_portfolio, a_currency)
		end

    latest_portfolio_data_date( a_accounting_date: ROSE_DATE; a_portfolio: RP_PORTFOLIO ): ROSE_DATE
			-- <Precursor>
		do
			Result := implementation.latest_portfolio_data_date (adjusted(a_accounting_date), a_portfolio )
		end

	price_of (a_latest_portfolio_data_date: ROSE_DATE; a_portfolio: RP_PORTFOLIO; a_position: RP_QUANTITY; a_currency: RP_CURRENCY; a_accounting_date: ROSE_DATE): RP_DECIMAL
			-- <Precursor>
		do
			Result := implementation.price_of (a_latest_portfolio_data_date, a_portfolio, a_position, a_currency, adjusted (a_accounting_date))
		end

	portfolio_position_for_instrument (a_latest_portfolio_data_date: ROSE_DATE; a_accounting_date: ROSE_DATE; a_portfolio: RP_PORTFOLIO; a_instrument: RP_INSTRUMENT): RP_PORTFOLIO_NON_CASH_POSITION
			-- <Precursor>
		do
			Result := implementation.portfolio_position_for_instrument (a_latest_portfolio_data_date, adjusted (a_accounting_date), a_portfolio, a_instrument)
		end

feature -- Access (Portfolio beginning of day positions)

	account_positions  (a_accounting_date: ROSE_DATE; a_account: RP_INVESTMENT_ACCOUNT): ROSE_LINEAR_ARRAY[POSITION[RP_INVESTMENT_ACCOUNT]]
			-- <Precursor>
		do
			Result := implementation.account_positions (adjusted (a_accounting_date), a_account)
		end

	account_cash_positions (a_accounting_date: ROSE_DATE; a_account: RP_INVESTMENT_ACCOUNT): ROSE_LINEAR_ARRAY[RP_ACCOUNT_CASH_POSITION]
			-- <Precursor>
		do
			Result := implementation.account_cash_positions (adjusted (a_accounting_date), a_account)
		end

	account_non_cash_positions (a_accounting_date: ROSE_DATE; a_account:RP_INVESTMENT_ACCOUNT): ROSE_LINEAR_ARRAY[RP_ACCOUNT_NON_CASH_POSITION]
			-- <Precursor>
		do
			Result := implementation.account_non_cash_positions (adjusted (a_accounting_date), a_account)
		end

	account_open_ffx_positions (a_accounting_date: ROSE_DATE; a_account: RP_INVESTMENT_ACCOUNT): ROSE_LINEAR_ARRAY[RP_ACCOUNT_OPEN_FFX_POSITION]
			-- <Precursor>
		do
			Result := implementation.account_open_ffx_positions (adjusted (a_accounting_date), a_account)
		end

feature -- Access (Account beginning of day positions)

	account_value (a_accounting_date: ROSE_DATE; a_account: RP_INVESTMENT_ACCOUNT; a_currency: RP_CURRENCY): RP_MONIES
			-- <Precursor>
		do
			Result := implementation.account_value (adjusted (a_accounting_date), a_account, a_currency)
		end

	account_cash_value (a_accounting_date: ROSE_DATE; a_account: RP_INVESTMENT_ACCOUNT; a_currency: RP_CURRENCY): RP_MONIES
			-- <Precursor>
		do
			Result := implementation.account_cash_value (adjusted (a_accounting_date), a_account, a_currency)
		end

	account_equity_value (a_accounting_date: ROSE_DATE; a_account: RP_INVESTMENT_ACCOUNT; a_currency: RP_CURRENCY): RP_MONIES
			-- <Precursor>
		do
			Result := implementation.account_equity_value (adjusted (a_accounting_date), a_account, a_currency)
		end

	account_ffx_value (a_accounting_date: ROSE_DATE; a_account: RP_INVESTMENT_ACCOUNT; a_currency: RP_CURRENCY): RP_MONIES
			-- <Precursor>
		do
			Result := implementation.account_ffx_value (adjusted (a_accounting_date), a_account, a_currency)
		end

feature -- Access (Portfolio with specific as_of_date)

	portfolio_value_as_of (a_as_of_date: ROSE_DATE; a_accounting_date: ROSE_DATE; a_portfolio: RP_PORTFOLIO; a_currency: RP_CURRENCY): RP_MONIES
			-- <Precursor>
		do
			Result := implementation.portfolio_value_as_of (a_as_of_date, adjusted (a_accounting_date), a_portfolio, a_currency)
		end

	portfolio_cash_value_as_of (a_as_of_date: ROSE_DATE; a_accounting_date: ROSE_DATE; a_portfolio: RP_PORTFOLIO; a_currency: RP_CURRENCY): RP_MONIES
			-- <Precursor>
		do
			Result := implementation.portfolio_cash_value_as_of (a_as_of_date, adjusted (a_accounting_date), a_portfolio, a_currency)
		end

	portfolio_settled_cash_value_as_of (a_as_of_date: ROSE_DATE; a_accounting_date: ROSE_DATE; a_portfolio: RP_PORTFOLIO; a_currency: RP_CURRENCY): RP_MONIES
			-- <Precursor>
		do
			Result := implementation.portfolio_settled_cash_value_as_of (a_as_of_date, adjusted (a_accounting_date), a_portfolio, a_currency)
		end

	portfolio_equity_value_as_of (a_as_of_date: ROSE_DATE; a_accounting_date: ROSE_DATE; a_portfolio: RP_PORTFOLIO; a_currency: RP_CURRENCY): RP_MONIES
			-- <Precursor>
		do
			Result := implementation.portfolio_equity_value_as_of (a_as_of_date, adjusted (a_accounting_date), a_portfolio, a_currency)
		end

	portfolio_ffx_value_as_of (a_as_of_date: ROSE_DATE; a_accounting_date: ROSE_DATE; a_portfolio: RP_PORTFOLIO; a_currency: RP_CURRENCY): RP_MONIES
			-- <Precursor>
		do
			Result := implementation.portfolio_ffx_value_as_of (a_as_of_date, adjusted (a_accounting_date), a_portfolio, a_currency)
	end

feature -- Access (Account with specific as of date)

	account_value_as_of (a_as_of_date: ROSE_DATE; a_accounting_date: ROSE_DATE; a_account: RP_INVESTMENT_ACCOUNT; a_currency: RP_CURRENCY): RP_MONIES
			-- <Precursor>
		do
			Result := implementation.account_value_as_of (a_as_of_date, adjusted (a_accounting_date), a_account, a_currency)
		end

	account_cash_value_as_of (a_as_of_date: ROSE_DATE; a_accounting_date: ROSE_DATE; a_account: RP_INVESTMENT_ACCOUNT; a_currency: RP_CURRENCY): RP_MONIES
			-- <Precursor>
		do
			Result := implementation.account_cash_value_as_of (a_as_of_date, adjusted (a_accounting_date), a_account, a_currency)
		end

	account_equity_value_as_of (a_as_of_date: ROSE_DATE; a_accounting_date: ROSE_DATE; a_account: RP_INVESTMENT_ACCOUNT; a_currency: RP_CURRENCY): RP_MONIES
			-- <Precursor>
		do
			Result := implementation.account_equity_value_as_of (a_as_of_date, adjusted (a_accounting_date), a_account, a_currency)
		end

	account_ffx_value_as_of (a_as_of_date: ROSE_DATE; a_accounting_date: ROSE_DATE; a_account: RP_INVESTMENT_ACCOUNT; a_currency: RP_CURRENCY): RP_MONIES
			-- FFX Net value of `a_account' on `a_as_of_date' in `a_currency'
			-- <Precursor>
		do
			Result := implementation.account_ffx_value_as_of (a_as_of_date, adjusted (a_accounting_date), a_account, a_currency)
		end

feature {NONE} -- Implementation

	implementation: RP_POSITIONS_SYSTEM_IMPLEMENTATION
		-- Provider

	adjusted (a_accounting_date: ROSE_DATE): ROSE_DATE is
			-- Adjust 'a_date' if necessary
		do
			create Result.make_by_date (a_accounting_date)
			if Result.day_of_the_week = 7 then Result.add_n_days (1) end
			if Result.day_of_the_week = 1 then Result.add_n_days (1) end
		end

end
