note
	description: "Positions processing flags"
	author: "Sami Ben Grine"
	last_editor: "$Author: sm $"
	date: "$Date: 2014/12/10 $"
	revision: "$Revision: #3 $"
	path: "$File: //rose/source/kernel/expert_system/rp_positions_update_mode.e $"

class
	RP_POSITIONS_UPDATE_MODE

inherit
	ANY
		redefine out end

create
	make_2PM,
	make_5PM,
	make_intraday,
	make_accounting,
	make_approve,
	make_from_string

feature {NONE} -- Initialization

	make_from_string (a_value: STRING) is
			-- Make from 'a_value'
		require
			value_not_void_or_empty: a_value /= Void and then not a_value.is_empty
		local
		do
			if 		a_value.as_lower.has_substring (internal_2PM_string.as_lower) then make_2pm
			elseif	a_value.as_lower.has_substring (internal_5PM_string.as_lower) then make_5pm
			elseif	a_value.as_lower.has_substring (internal_8PM_string.as_lower) then make_8pm
			elseif	a_value.as_lower.has_substring (internal_intraday_string.as_lower) then make_intraday
			elseif	a_value.as_lower.has_substring (internal_accounting_string.as_lower) then make_accounting
			elseif	a_value.as_lower.has_substring (internal_approve_string.as_lower) then make_approve
			else
				exceptions.raise ("Invalid value")
			end
		end

	make_2PM
			-- Create as '2PM'
		do
			code := internal_2PM_code
		ensure
			is_2PM: is_2PM
		end

	make_5PM
			-- Create as '5PM'
		do
			code := internal_5PM_code
		ensure
			is_5PM: is_5PM
		end

	make_8PM
			-- Create as '8PM'
		do
			code := internal_8PM_code
		ensure
			is_8PM: is_8PM
		end

	make_intraday
			-- Create as 'intraday'
		do
			code := internal_intraday_code
		ensure
			is_intraday: is_intraday
		end

	make_accounting
			-- Create as 'Accounting'
		do
			code := internal_accounting_code
		ensure
			is_accounting: is_accounting
		end

	make_approve
			-- Create as 'Approve'
		do
			code := internal_approve_code
		ensure
			is_accounting: is_approve
		end

feature -- Status Report

	is_2PM: BOOLEAN
		-- First update of positions in the day
		-- This an early estimate
		do
			Result := code.is_equal (internal_2pm_code)
		end

	is_5PM: BOOLEAN
		-- Second update of positions in the day
		do
			Result := code.is_equal (internal_5pm_code)
		end

	is_8PM: BOOLEAN
		-- Third update of positions in the day (Futures for PART only)
		do
			Result := code.is_equal (internal_8pm_code)
		end

	is_intraday: BOOLEAN
		-- Cumulative update of positions (every 15 minutes and includes OSSM non-intraday trades)
		do
			Result := code.is_equal (internal_intraday_code)
		end

	is_accounting: BOOLEAN
		-- Accounting mode positions
		do
			Result := code.is_equal (internal_accounting_code)
		end

	is_approve: BOOLEAN
		-- Approve mode positions
		do
			Result := code.is_equal (internal_approve_code)
		end

feature -- Output

	out: STRING
			-- Output
		do
			inspect code
			when internal_2PM_code then
				Result := internal_2PM_string
			when internal_5PM_code then
				Result := internal_5PM_string
			when internal_8PM_code then
				Result := internal_8PM_string
			when internal_intraday_code then
				Result := internal_intraday_string
			when internal_accounting_code then
				Result := internal_accounting_string
			when internal_approve_code then
				Result := internal_approve_string
			else
				exceptions.raise ("inconsistent code")
			end
		end

feature {NONE} -- Implementation

	code: CHARACTER
		-- Code

	exceptions: EXCEPTIONS
			-- Exceptions
		once
			create Result
		end

feature {NONE} -- Constant

	internal_2PM_code: CHARACTER = '2'
		-- Internal code for 2PM update

	internal_2PM_string: STRING = "Forecast"
		-- Internal string representation for 2PM update

	internal_5PM_code: CHARACTER = '5'
		-- Internal code for 5PM update

	internal_5PM_string: STRING = "Final"
		-- Internal string representation for 5PM update

	internal_8PM_code: CHARACTER = '8'
		-- Internal code for 8PM update (PARTS ONLY)

	internal_8PM_string: STRING = "Derivatives"
		-- Internal string representation for 8PM update (PARTS ONLY)

	internal_intraday_code: CHARACTER = 'I'
		-- Internal code for intraday update

	internal_intraday_string: STRING = "Intraday"
		-- Internal string representation for intraday update

	internal_accounting_code: CHARACTER = 'A'
		-- Internal code for accounting update

	internal_accounting_string: STRING = "Accounting"
		-- Internal string representation for accounting update

	internal_approve_code: CHARACTER = 'D'
		-- Internal code for approve mode

	internal_approve_string: STRING = "Approve"
		-- Internal string representation for approve update

invariant

	one_of_valid: is_2PM or else is_5PM or else is_8PM or else is_intraday or else is_accounting or else is_approve

end
