indexing
	description: "Price"
	author: "it"
	date: "$Date: 2010/05/04 $"
	revision: "$Revision: #3 $"
	archive: "$File: //rose/source/kernel/expert_system/rp_price.e $"

class
	RP_PRICE

inherit

	DEBUG_OUTPUT
		export
			{NONE} all
		undefine
			out
		end
	ANY
		redefine
			out
		end

create {RP_INSTRUMENT, RP_PRICE}

	make

feature {NONE}-- Initialization

	make (a_price: like price; a_instrument: like instrument; a_currency: like currency; a_date: like date) is
			-- Initialize `Current'
		require
			non_void_price: a_price /= Void
			non_negative_price: a_price >= a_price.zero
			non_void_instrument: a_instrument /= Void
			non_void_currency: a_currency /= Void
			non_void_date: a_date /= Void
		do
			price := a_price
			instrument := a_instrument
			currency := a_currency
			date := a_date
		ensure
			price_set: price = a_price
			instrument_set: instrument = a_instrument
			currency_set: currency = a_currency
			date_set: date = a_date
		end

feature -- Access

	price: attached RP_DECIMAL
			-- Price

	instrument: attached RP_INSTRUMENT
			-- Instrument

	currency: attached RP_CURRENCY
			-- Currency

	date: attached ROSE_DATE
			-- Date of price

	units: attached ROSE_UNITS is
			-- Underlying units
		do
			Result := currency.rose_units / instrument.rose_units
		end

	source: detachable STRING
			-- Source of price

feature	-- Output

	out: attached STRING is
			-- Output representation
		do
			Result := debug_output
		end

feature -- Change

	set_source (a_source: like source) is
			-- Set the value of `source' to `a_source'
		do
			source := a_source
		ensure
			source_set: source = a_source
		end

	scale_by (a_scalar: RP_DECIMAL): attached like Current is
			-- Scale price by `a_scalar'
		require
			non_void_scalar: a_scalar /= Void
			non_negative_scalar: a_scalar >= a_scalar.zero
		do
			create Result.make (price * a_scalar, instrument, currency, date)
		ensure
			scaled: Result.price ~ (price * a_scalar)
		end

feature {NONE}	-- Debugging

	debug_output: attached STRING is
			-- Debug information
		do
			create Result.make_empty
			if price /= Void then
				Result.append (price.out)
			end
			Result.append_character ('|')
			if units /= Void then
				Result.append (currency.name)
			end
			Result.append_character ('|')
			if date /= Void then
				Result.append (date.ccyymmdd.out)
			end
			Result.append_character ('|')
			if source /= Void then
				Result.append (source)
			end
		end

invariant

	non_negative_price: price >= price.zero

end
