indexing

	description:

		"Quantity of units of instrument"

	date: "$Date: 2010/06/04 $"
	revision: "$Revision: #17 $"

class RP_QUANTITY

inherit

	COMPARABLE

	DEBUG_OUTPUT
		undefine
			is_equal
		end

create {RP_QUANTITY, RP_INSTRUMENT}

	make

feature {NONE} -- Initialization

	make (a_instrument: like instrument; a_amount: like amount) is
			-- Initialize `Current'.
		require
			non_void_instrument: a_instrument /= Void
			non_void_amount: a_amount /= Void
		do
			instrument := a_instrument
			amount := a_amount
		ensure
			instrument_set: instrument = a_instrument
			amount_set: amount = a_amount
		end

feature -- Copy

	duplicate: like current is
			-- Duplicate copy of current (because RP_DECIMAL is not expanded)
		do
			Result := instrument.quantity (amount+0)
		end

feature -- Access

	instrument: attached RP_INSTRUMENT
			-- Instrument

	amount: attached RP_DECIMAL
			-- Amount

	units: attached ROSE_UNITS is
			-- Units
		do
			Result := instrument.rose_units
		end

	sign: INTEGER is
			-- Sign of 'amount'
		do
			Result := amount.sign
		ensure
			amount_sign: Result = amount.sign
		end

feature -- Basic Operations

	add (a_other: like Current) is
			-- Add 'a_other' to Current
		require
			same_instrument: instrument ~ a_other.instrument
		do
			amount := amount + a_other.amount
		end

	mul (a_other: like Current) is
			-- Multiply 'a_other' to Current
		require
			same_instrument: instrument ~ a_other.instrument
		do
			amount := amount * a_other.amount
		end

	round (a_positive_decimals: INTEGER) is
			-- Round 'amount' to 'a_positive_decimals'
		require
			positive_decimals: a_positive_decimals >= 0
		do
			amount.round (a_positive_decimals)
		end

	truncate is
			-- Truncate 'amount'
		do
			amount.truncate
		end

feature -- Status Report

	is_zero: BOOLEAN is
			-- Is 'amount' zero?
		do
			Result := amount.is_zero
		end

feature -- Comparison

	infix "<" (a_other: like Current): BOOLEAN is
			-- Is current object less than `a_other'?
		do
			Result := instrument.brimsym < a_other.instrument.brimsym
			if not Result and then instrument.brimsym ~ a_other.instrument.brimsym then
				Result := amount < a_other.amount
			end
		end

feature -- Basic operations

	prefix "-": attached like Current is
			-- Negative of `Current'
		do
			create Result.make (instrument, -amount)
		end

	infix "+" (a_other: like Current): attached like Current is
			-- Addition of `a_other' to `Current'
		require
			other_not_void: a_other /= Void
			same_units: units.is_equal (a_other.units)
			same_instrument: instrument.is_equal (a_other.instrument)
		do
			create Result.make (instrument, amount + a_other.amount)
		end

	infix "-" (a_other: like Current): attached like Current is
			-- Substraction of `a_other' from `Current'
		require
			other_not_void: a_other /= Void
			same_units: units ~ a_other.units
			same_instrument: instrument ~ a_other.instrument
		do
			create Result.make (instrument, amount - a_other.amount)
		end

	infix "*" (a_other: like Current): attached like Current is
			-- Multiplication of `a_other' from `Current'
		require
			other_not_void: a_other /= Void
			same_units: units ~ a_other.units
			same_instrument: instrument ~ a_other.instrument
		do
			create Result.make (instrument, amount * a_other.amount)
		end

	infix "/" (a_other: like Current): like Current is
			-- Multiplication of `a_other' from `Current'
		require
			other_not_void: a_other /= Void
			same_units: units.is_equal (a_other.units)
			same_instrument: instrument.is_equal (a_other.instrument)
			not_zero: not a_other.amount.is_zero
		do
			create Result.make (instrument, amount / a_other.amount)
		ensure
			non_void_result: Result /= Void
		end

	abs: attached like Current is
			-- Absolute value of `Current'
		do
			create Result.make (instrument, amount.abs)
		end

feature -- Output

	debug_output: attached STRING is
			-- Debug information
		do
			create Result.make_empty
			if amount /= Void then
				Result.append (amount.out)
			end
			Result.append_character (' ')
			if instrument /= Void and then instrument.brimsym /= Void then
				Result.append (instrument.brimsym)
			end
		end

end
