note

	description:

		"The Reference Environment"

	author: "Ivan Tarnavsky"
	last_editor: "$Author: c4 $"
	date: "$Date: 2015/04/09 $"
	revision: "$Revision: #93 $"
	path: "$File: //rose/source/kernel/expert_system/rp_reference_system.e $"

class RP_REFERENCE_SYSTEM

inherit

	ANY

	GCDB_SHARED_HOME
		rename
			singleton as gcdb
		export
			{NONE} all
		end

	ROSE_ENTITY_FOREGROUND_AWARE
		export {NONE} all end

	ATTRIBUTES
		export {NONE} all end

	DATABASE_SERVER
		export {NONE} all end

create {RP_INVESTMENT_MANAGER}

	make

feature {NONE} -- Initialization

	make is
			-- Initialize `Current'.
		do
			create instrument_cache.make_default
			create brokers_cache.make_default
			create currency_cache.make
			create portfolios_cache.make_equal (17)
			create participants.make_equal (17)
			create investment_accounts_cache.make_equal (17)
			create clients_cache.make_equal (17)
			create benchmarks_cache.make_equal (0)
			create zonal_products_cache_by_code.make_equal (17)
			create global_products_cache_by_code.make_equal (17)
			create countries_cache.make (0)
			create strategies_cache.make_equal (17)
			create selection_universe_cache.make_equal (17)
		end

feature -- Queries

	c_zone: !RP_ZONE is
			-- Instance of C zone
		once
			create Result.make (Current, gcdb.investment_region_by_code ("C", Void), context.c_zone, create {ROSE_DATE}.make_ccyymmdd (19900607))
		end

	e_zone: !RP_ZONE is
			-- Instance of E zone
		once
			create Result.make (Current, gcdb.investment_region_by_code ("E", Void), context.e_zone, create {ROSE_DATE}.make_ccyymmdd (19900425))
		end

	j_zone: !RP_ZONE is
			-- Instance of J zone
		once
			create Result.make (Current, gcdb.investment_region_by_code ("J", Void), context.j_zone, create {ROSE_DATE}.make_ccyymmdd (19881228))
		end

	o_zone: !RP_ZONE is
			-- Instance of O zone
		once
			create Result.make (Current, gcdb.investment_region_by_code ("O", Void), context.o_zone, create {ROSE_DATE}.make_ccyymmdd (19900425))
		end

	s_zone: !RP_ZONE is
			-- Instance of S zone
		once
			create Result.make (Current, gcdb.investment_region_by_code ("S", Void), context.s_zone, create {ROSE_DATE}.make_ccyymmdd (20051019))
		end

	u_zone: !RP_ZONE is
			-- Instance of U zone
		once
			create Result.make (Current, gcdb.investment_region_by_code ("U", Void), context.u_zone, create {ROSE_DATE}.make_ccyymmdd (19850508))
		end

	t_zone: !RP_ZONE is
			-- Instance of T zone
		once
			create Result.make (Current, gcdb.investment_region_by_code ("T", Void), context.t_zone, create {ROSE_DATE}.make_ccyymmdd (19900424))
		end

	w_zone: RP_ZONE is
			-- Instance of W zone
		once
			create Result.make (Current, gcdb.investment_region_by_code ("W", Void), context.w_zone, create {ROSE_DATE}.make_ccyymmdd (20140101))
		ensure
			w_zone_not_void: Result /= Void
		end

	cross_zonal_translation (a_from_instrument: RP_INSTRUMENT; a_zone: RP_ZONE; a_date: ROSE_DATE): attached RP_INSTRUMENT is
			-- Translate `a_from_instrument' on `a_date' to main instrument a la RMDAT_SPECIFIC
		require
			non_void_arguments: a_from_instrument /= Void and a_zone /= Void and a_date /= Void
		local
			l_brimsym: STRING
			l_mapping: ROSE_INTERVAL_HISTORY [STRING, ROSE_CLOSED_INTERVAL [STRING]]
		do
			if cached_cross_zonal_linkages = Void then
				retrieve_cross_zonal_linkages
			end
			create l_brimsym.make_from_string (a_from_instrument.local_brimsym (a_zone, a_date))
			cached_cross_zonal_linkages.search (l_brimsym)
			if cached_cross_zonal_linkages.found then
				l_mapping := cached_cross_zonal_linkages.found_item
				if (l_mapping.witness (a_date) /= Void) then
					create l_brimsym.make_from_string (l_mapping.witness (a_date))
				end
			end
			if l_brimsym.count = 18 and then l_brimsym.substring (13, 15).is_equal ("NVD") then
					-- non-voting from Thailand, get data from local share
				l_brimsym.replace_substring ("091...", 10, 15)
			end
			Result := instrument_from_brimsym_at (l_brimsym, a_date)
				-- Return the input instrument
				-- when the mapped to instrument
				-- does not exist
			if Result = Void then
				Result := a_from_instrument
			end
		ensure
			attached_result: Result /= Void
		end

	cross_zonal_translation_for_mds (a_interval: ROSE_CLOSED_OPEN_INTERVAL [STRING]; a_zone: RP_ZONE): ROSE_LINEAR_ARRAY [PAIR [STRING, ROSE_DATE]] is
			-- get the intervals for MDS
		local
			l_mapping: ROSE_INTERVAL_HISTORY [STRING, ROSE_CLOSED_INTERVAL [STRING]]
			l_brimsym: STRING
			l_converted_brimsym: STRING
			l_instrument: RP_INSTRUMENT
		do
			create Result.make_empty
			create l_brimsym.make_from_string (a_interval.item)
			if cached_cross_zonal_linkages = Void then
				retrieve_cross_zonal_linkages
			end
			l_instrument := instrument_from_brimsym (l_brimsym)
			if l_instrument.local_brimsym (a_zone, a_interval.first_date) /= Void then
				l_brimsym := l_instrument.local_brimsym (a_zone, a_interval.first_date)
			end
			cached_cross_zonal_linkages.search (l_brimsym)
			if cached_cross_zonal_linkages.found then
				l_mapping := cached_cross_zonal_linkages.found_item
				from
					l_mapping.start
				until
					l_mapping.off
				loop
					create l_converted_brimsym.make_from_string (l_mapping.item_for_iteration.item)
					if l_converted_brimsym.count = 18 and then l_converted_brimsym.substring (13, 15).is_equal ("NVD") then
							-- non-voting from Thiland, get data from local share
						l_converted_brimsym.replace_substring ("091...", 10, 15)
					end
					Result.extend (create {PAIR [STRING, ROSE_DATE]}.make (l_converted_brimsym, l_mapping.item_for_iteration.start_date))
					l_mapping.forth
				end
			else
				create l_converted_brimsym.make_from_string (l_brimsym)
				if l_converted_brimsym.count = 18 and then l_converted_brimsym.substring (13, 15).is_equal ("NVD") then
						-- non-voting from Thiland, get data from local share
					l_converted_brimsym.replace_substring ("091...", 10, 15)
					Result.extend (create {PAIR [STRING, ROSE_DATE]}.make (l_converted_brimsym, a_interval.start_date))
				end
			end
		end

	currencies: ARRAY [RP_CURRENCY] is
			-- Valid currencies
		local
			c: ARRAY [CURRENCY]
			i, n: INTEGER
			l_currency: RP_CURRENCY
		do
			from
				c := context.currencies_with_data
				n := c.count
				create Result.make_filled (Void, 1, n)
				i := 1
			until
				i > n
			loop
				l_currency := currency_from_iso_code (c.item (i).iso)
				if l_currency /= Void then
					Result.put (l_currency, i)
				end
				i := i + 1
			end
		ensure
			currencies_not_void: Result /= Void
			no_void_currencies: not Result.has (Void)
		end

	valid_zones: STRING is "UCEOJSW"
			-- Valid investment zones

	investment_regions: ARRAY [RP_ZONE] is
			-- All investment regions
		once
			Result := <<c_zone, e_zone, j_zone, o_zone, s_zone, u_zone, t_zone, w_zone>>
		end

	is_valid_investment_region (a_region: CHARACTER): BOOLEAN is
			-- Is `a_region' a valid investment region?
		do
			Result := investment_regions.there_exists (agent zone_code_matches (?, a_region))
		end

	zone_code_matches (a_zone: RP_ZONE; a_zone_code: CHARACTER): BOOLEAN is
			-- Does `a_zone' `code' match `a_zone_code'? Case Insensitive.
		require
			a_zone_not_void: a_zone /= Void
		do
			Result := a_zone.code.as_upper.is_equal (a_zone_code.as_upper.out)
		end

	is_valid_zone (a_zone: CHARACTER): BOOLEAN is
			-- Is `a_zone' a valid investment zone?
		do
			Result := valid_zones.has (a_zone.as_upper)
		end

	imc_by_code (a_code: STRING): attached RP_INVESTMENT_MANAGEMENT_COMPANY is
			-- IMC by code
		require
			code_not_void: a_code /= Void
		do
			if attached {like imc_by_code} imcs_by_code.item (a_code) as l_result then
				Result := l_result
			else
				Result := unknown_imc
			end
		ensure
			attached_result: Result /= Void
		end

	security_from_brim_code (a_brim_code: STRING; a_date: ROSE_DATE): detachable RP_SECURITY is
			-- Security from `a_brim_code' on `a_date'
		do
			Result ?= instrument_from_brimsym_at (a_brim_code, a_date)
		end

	instrument_from_brimsym (a_brimsym: STRING): detachable RP_INSTRUMENT is
			-- Instrument from BRIMSYM
		require
			brimsym_not_void: a_brimsym /= Void
		do
			Result := instrument_from_brimsym_at (a_brimsym, create {ROSE_DATE}.make_today)
		end

	instrument_from_brimsym_at (a_brimsym: STRING; a_date: ROSE_DATE): detachable RP_INSTRUMENT is
			-- Instrument from `a_brimsym' on `a_date'
		require
			brimsym_not_void: a_brimsym /= Void
			date_not_void: a_date /= Void
		local
			l_instrument: RP_INSTRUMENT
			l_brimsym, l_original_brimsym: STRING
			l_legacy_instrument: LEGACY_INSTRUMENT
			l_cache_brimsym: STRING
		do
			create l_original_brimsym.make_from_string (a_brimsym)
			l_original_brimsym.left_adjust
			l_original_brimsym.right_adjust
			if not l_original_brimsym.is_empty then
				l_original_brimsym.to_upper
				l_brimsym := fix_bad_gedb_brimsym (l_original_brimsym)
				l_cache_brimsym := l_brimsym
				if valid_index_future_brimsym (l_cache_brimsym) then
					l_cache_brimsym := index_future_brimsym (l_cache_brimsym, a_date)
				end
				instrument_cache.search (l_cache_brimsym)
				if instrument_cache.found then
					l_instrument := instrument_cache.found_item
				else
					l_legacy_instrument := legacy_instrument_from_brim_code (l_brimsym)
					l_instrument := make_instrument_from_brimsym (l_brimsym, l_legacy_instrument, a_date)
					if l_instrument /= Void then
						instrument_cache.force (l_instrument, l_cache_brimsym)
					end
				end
			end
			Result := l_instrument
		end

	instrument_from_legacy_instrument_at (a_legacy_instrument: LEGACY_INSTRUMENT; a_date: ROSE_DATE): detachable RP_INSTRUMENT is
			-- Instrument from a Legacy Instrument on `a_date'
		do
			if {l_brimsym: STRING} a_legacy_instrument.brim_code (a_date) then
				Result := instrument_from_brimsym_at (l_brimsym, a_date)
			end
		end

	broker_from_code (a_broker_code: STRING): attached RP_BROKER is
			-- Broker from broker unique code
		require
			args_present: a_broker_code /= Void
		do
			if brokers_cache.has (a_broker_code) then
				Result := brokers_cache.item (a_broker_code)
			else
				if all_brokers = Void then
					create Result.make (Current, a_broker_code)
					brokers_cache.force (Result, Result.code)
				else
					Result := unknown_broker
				end
			end
		ensure
			attached_result: Result /= Void
		end

	brokers: attached ARRAY [RP_BROKER] is
			-- List of all brokers.
		do
			Result := retrieve_all_brokers
		ensure
			attached_result: Result /= Void
		end

	currency_from_iso_code (a_currency_iso_code: STRING): detachable RP_CURRENCY is
			-- Currency from 3-letter ISO code or 2-letter country code
		require
			args_present: a_currency_iso_code /= Void
		do
			Result := currency_cache.currency_from_iso_code (a_currency_iso_code)
		end

	currency_from_country_code (a_country_code: STRING; a_date: ROSE_DATE): detachable RP_CURRENCY is
			-- Currency of country defined by `a_country_code' on `a_date'
		require
			non_void_args: a_country_code /= Void and a_date /= Void
--			valid_country_code: not country_from_iso_code (a_country_code).is_unknown
				-- e2[2010/05/13]: `a_country_code' can be "EU" or a brim code, ... so it can be "unknown"
		do
			Result := cash_from_country (a_country_code, a_date)
		ensure
			currency_can_be_void: True
		end

	currency_from_brim_code (a_brim_code: STRING): detachable RP_CURRENCY is
			-- Currency from country brim code `a_brim_code' as defined in ref_w_a:country_codes.parm
		require
			non_void_code: a_brim_code /= Void
			code_long_enough: a_brim_code.count = 2
		do
			Result := currency_cache.currency_from_country_brim_code (a_brim_code)
		end

	portfolios: attached ARRAY [RP_PORTFOLIO] is
			-- All known portfolios
		do
			retrieve_all_portfolios
			Result := all_portfolios
		ensure
			attached_result: Result /= Void
		end

	fund_participant_portfolios: attached ARRAY [RP_PORTFOLIO] is
			-- All fund participant portfolios
		do
			retrieve_all_portfolios
			Result := all_fund_participant_portfolios
		ensure
			attached_result: Result /= Void
		end

	portfolio_from_code (a_portfolio_code: STRING): attached RP_PORTFOLIO is
			-- Portfolio from portfolio unique code
		require
			args_present: a_portfolio_code /= Void
		local
			l_gcdb_portfolio: GCDB_PORTFOLIO
			l_gcdb_fund_participant: GCDB_FUND_PARTICIPANT
			l_portfolio_code: STRING
		do
			create l_portfolio_code.make_from_string (a_portfolio_code)
			l_portfolio_code.prune_all (' ')
			l_portfolio_code.to_upper
			if portfolios_cache.has (l_portfolio_code) then
				Result := portfolios_cache.item (l_portfolio_code)
			elseif all_portfolios = Void then
				l_gcdb_portfolio := gcdb.portfolio_by_code (l_portfolio_code, Void)
				if l_gcdb_portfolio /= Void then
					add_portfolio (l_gcdb_portfolio)
					if portfolios_cache.has (l_portfolio_code) then
						Result := portfolios_cache.item (l_portfolio_code)
					end
				else
					l_gcdb_fund_participant := gcdb.fund_participant_by_code (l_portfolio_code, Void)
					if l_gcdb_fund_participant /= Void then
						add_gcdb_fund_participant (l_gcdb_fund_participant)
						if portfolios_cache.has (l_portfolio_code) then
							Result := portfolios_cache.item (l_portfolio_code)
						end
					end
				end
			end
			if Result = Void then
				Result := unknown_portfolio
			end
		ensure
			attached_result: Result /= Void
			result_as_expected: portfolio_matches_name (a_portfolio_code, Result)
			portfolio_cached: portfolio_cached (a_portfolio_code, Result)
		end

	bank_from_bic (a_bic: STRING): attached RP_BANK is
			-- Bank fom bank code
		do
			create Result.make_with_bic (a_bic)
				-- should be cached so we use flyweight pattern for these
		ensure
			attached_result: Result /= Void
		end

	investment_region_by_code (a_zone: STRING): attached RP_ZONE is
			-- Zone from code
		do
			Result := zones_by_code.item (a_zone)
			if Result = Void then
				Result := unknown_investment_region
			end
		ensure
			attached_result: Result /= Void
		end

	investment_accounts: attached ARRAY [RP_INVESTMENT_ACCOUNT] is
			-- All known investment accounts
		do
			Result := retrieve_all_investment_accounts
		ensure
			attached_result: Result /= Void
			no_void_items: not Result.has (Void)
		end

	nof_investment_accounts: attached ARRAY [RP_INVESTMENT_ACCOUNT] is
			-- All investment accounts with fee information
		do
			Result := retrieve_nof_investment_accounts
		ensure
			attached_result: Result /= Void
			no_void_items: not Result.has (Void)
		end

	investment_account_by_code (a_account_code: STRING): attached RP_INVESTMENT_ACCOUNT is
			-- Investment account identified by `a_account_code'
		local
			l_gcdb_account: GCDB_ACCOUNT
			l_gcdb_portfolio_group: GCDB_PORTFOLIO_GROUP
			l_gcdb_fund_participant: GCDB_FUND_PARTICIPANT
		do
			if investment_accounts_cache.has (a_account_code) then
				Result := investment_accounts_cache.item (a_account_code)
			else
				if all_investment_accounts = Void then
					l_gcdb_portfolio_group := gcdb.portfolio_group_by_code (a_account_code, Void)
					l_gcdb_account := gcdb.account_by_code (a_account_code, Void)
					l_gcdb_fund_participant := gcdb.fund_participant_by_code (a_account_code, Void)
					if l_gcdb_portfolio_group /= Void or l_gcdb_account /= Void or l_gcdb_fund_participant /= Void then
						if l_gcdb_fund_participant = Void or else l_gcdb_fund_participant.brimsym /= Void then
							create Result.make (Current, l_gcdb_account, l_gcdb_portfolio_group, l_gcdb_fund_participant)
							investment_accounts_cache.force (Result, a_account_code)
						end
					end
				end
			end
			if Result = Void then
				Result := unknown_investment_account
			end
		ensure
			attached_result: Result /= Void
			result_as_expected: Result.is_unknown or Result.name.is_equal (a_account_code)
			account_cached: not Result.is_unknown implies investment_accounts_cache.has (a_account_code)
		end

	clients: attached ARRAY [RP_CLIENT] is
			-- List of all clients.
		do
			Result := retrieve_all_clients
		ensure
			attached_result: Result /= Void
		end

	client_by_code (a_client_code: STRING): attached RP_CLIENT is
			-- Client identified by `a_client_code'.
		require
			args_exist: a_client_code /= Void
		local
			l_client: GCDB_CLIENT
		do
			if clients_cache.has (a_client_code) then
				Result := clients_cache.item (a_client_code)
			elseif all_clients = Void then
				l_client := gcdb.client_by_code (a_client_code, Void)
				if l_client /= Void then
					add_client (l_client)
					Result := clients_cache.item (a_client_code)
				end
			end
			if Result = Void then
				Result := unknown_client
			end
		ensure
			attached_result: Result /= Void
		end

	benchmarks: attached ARRAY [RP_BENCHMARK] is
			-- List of all benchmarks.
		do
			Result := retrieve_benchmarks
		ensure
			attached_result: Result /= Void
		end

	benchmark_by_code (a_code: STRING): attached RP_BENCHMARK is
			-- Benchmark identified by `a_code'.
		require
			args_exist: a_code /= Void
		local
			l_bogie: GCDB_BENCHMARK
		do
			benchmarks_cache.search (a_code)
			if benchmarks_cache.found then
				Result := benchmarks_cache.found_item
			elseif all_benchmarks = Void then
				l_bogie := gcdb.benchmark_by_code (a_code, Void)
				if l_bogie /= Void then
					create Result.make (Current, l_bogie)
					benchmarks_cache.force (Result, Result.code)
				end
			end
			if Result = Void then
				Result := unknown_benchmark
			end
		ensure
			attached_result: Result /= Void
		end

	products: attached ARRAY [RP_PRODUCT] is
			-- List of all products (composites).
		do
			Result := retrieve_products
		ensure
			attached_result: Result /= Void
		end

	examined_products: attached ARRAY [RP_PRODUCT] is
			-- List of all GIPS examined products (composites).
		do
			if saved_examined_products = Void then
				create saved_examined_products.make_empty
				products.do_if (agent saved_examined_products.extend (?), agent {RP_PRODUCT}.is_examined)
			end
			Result := saved_examined_products
		ensure
			attached_result: Result /= Void
		end

	product_by_code (a_code: STRING): attached RP_PRODUCT is
			-- Retrieve product (composite) by code
		require
			code_exist: a_code /= Void
		do
			Result := global_product_by_code (a_code)
			if Result = unknown_global_product then
				Result := zonal_product_by_code (a_code)
			end
		ensure
			attached_result: Result /= Void
		end

	products_by_portfolio (a_portfolio: RP_PORTFOLIO): attached RP_PRODUCT_HISTORY is
			-- Products assigned to `a_portfolio'
		do
			create Result.make_empty
			zonal_products_cache_by_code.do_all (agent retrieve_portfolio_products (Result, a_portfolio, ?))
		ensure
			attached_result: Result /= Void
		end

	zonal_product_by_code (a_code: STRING): attached RP_ZONAL_PRODUCT is
			-- zonal product for a_code
		require
			code_exist: a_code /= Void
		local
			l_gcdb_product: GCDB_PRODUCT
		do
			if zonal_products_cache_by_code.has (a_code) then
				Result := zonal_products_cache_by_code.item (a_code)
			elseif all_products = Void then
				l_gcdb_product := gcdb.product_by_code (a_code, Void)
				if l_gcdb_product /= Void then
					if not l_gcdb_product.is_global then
						create Result.make (Current, l_gcdb_product)
						zonal_products_cache_by_code.force (Result, a_code)
					end
				end
			end
			if Result = Void then
				Result := unknown_zonal_product
			end
		ensure
			attached_result: Result /= Void
		end

	global_product_by_code (a_code: STRING): attached RP_GLOBAL_PRODUCT is
			-- global product for a_code
		require
			code_exist: a_code /= Void
		local
			l_gcdb_product: GCDB_PRODUCT
		do
			if global_products_cache_by_code.has (a_code) then
				Result := global_products_cache_by_code.item (a_code)
			elseif all_products = Void then
				l_gcdb_product := gcdb.product_by_code (a_code, Void)
				if l_gcdb_product /= Void then
					if l_gcdb_product.is_global then
						create Result.make (Current, l_gcdb_product)
						global_products_cache_by_code.force (Result, a_code)
					end
				end
			end
			if Result = Void then
				Result := unknown_global_product
			end
		ensure
			attached_result: Result /= Void
		end

	country_from_iso_code (a_iso_3166_alpha2_code: STRING): attached RP_COUNTRY is
			-- Country from its ISO 3166 alpha-2 code
		do
			Result := country_by_iso_3166_alpha2_code (a_iso_3166_alpha2_code)
		ensure
			attached_result: Result /= Void
		end

	strategy_by_code (a_code: STRING): attached RP_STRATEGY is
			-- Strategy from its code
		do
			Result := retrieve_strategy (a_code)
		ensure
			attached_result: Result /= Void
		end

	selection_universes: attached ARRAY [RP_SELECTION_UNIVERSE] is
			-- All known selection universes
		do
			if all_selection_universes = Void then
				selection_universe_cache.wipe_out
				gcdb.context.universe_accessor.all_entities.do_all (agent add_selection_universe)
				all_selection_universes := selection_universe_cache.to_array
			end
			Result := all_selection_universes
		ensure
			attached_result: Result /= Void
		end

	selection_universe_from_code (a_code: STRING): attached RP_SELECTION_UNIVERSE is
			-- Selection universe (aka strategy) identified by `a_code'
		require
			a_code_not_void: a_code /= Void
		local
			l_universe: GCDB_SELECTION_UNIVERSE
		do
			if selection_universe_cache.has (a_code) then
				Result := selection_universe_cache.item (a_code)
			else
				l_universe := gcdb.selection_universe_by_code (a_code, Void)
				if l_universe = Void then
					Result := unknown_selection_universe
				else
					create Result.make (Current, l_universe)
				end
				selection_universe_cache.force_new (Result, a_code)
			end
		ensure
			attached_result: Result /= Void
			result_as_expected: Result.is_unknown or Result.code.is_equal (a_code)
			result_cached: not Result.is_unknown implies selection_universe_cache.has (a_code)
		end

	currency_future (a_currency: RP_CURRENCY; a_maturity_date: ROSE_DATE; a_future_symbol: STRING): RP_CURRENCY_FUTURE is
			-- Currency future for `a_currency'
		require
			non_void_args: a_currency /= Void and a_maturity_date /= Void and a_future_symbol /= Void
		local
			l_cache_brimsym: STRING
		do
			l_cache_brimsym := currency_future_brimsym (a_currency, a_maturity_date, a_future_symbol)
			instrument_cache.search (l_cache_brimsym)
			if instrument_cache.found then
				Result ?= instrument_cache.found_item
			else
				create Result.make (l_cache_brimsym, a_currency, 1, a_maturity_date, a_future_symbol)
				instrument_cache.force (Result, l_cache_brimsym)
			end
		end

	currency_forward (a_currency: RP_CURRENCY; a_settlement_date: ROSE_DATE): RP_CURRENCY_FORWARD is
			-- Currency forward for `a_currency' and `a_settlement_date'
		require
			non_void_args: a_currency /= Void and a_settlement_date /= Void
		local
			l_cache_brimsym: STRING
		do
			l_cache_brimsym := currency_forward_brimsym (a_currency, a_settlement_date)
			instrument_cache.search (l_cache_brimsym)
			if instrument_cache.found then
				Result ?= instrument_cache.found_item
			else
				create Result.make (l_cache_brimsym, a_currency, a_settlement_date)
				instrument_cache.force (Result, l_cache_brimsym)
			end
		end

	taxation_basis (a_taxbas: STRING; a_date: ROSE_DATE): detachable RP_TAX_BASE_PARM is
			-- Taxation basis information for `a_taxbas' on `a_date'
		require
			non_void_args: a_taxbas /= Void and a_date /= Void
		do
			tax_base_perf_cache.search (a_taxbas)
			if tax_base_perf_cache.found then
				Result := tax_base_perf_cache.found_item.witness (a_date)
			end
		end

feature -- Unknown entities

	unknown_portfolio: RP_PORTFOLIO is
			-- Unknown portfolio instance
		once
			create Result.make (Current, Void, Void, Void)
		end

	unknown_client: RP_CLIENT is
			-- Unknown client instance
		once
			create Result.make (Current, Void)
		end

	unknown_investment_account: RP_INVESTMENT_ACCOUNT is
			-- Unknown investment account instance
		once
			create Result.make (Current, Void, Void, Void)
		end

	unknown_broker: RP_BROKER is
			-- Unknown broker instance
		once
			create Result.make (Current, Void)
		end

	unknown_benchmark: RP_BENCHMARK is
			-- Unknown benchmark instance
		once
			create Result.make (Current, Void)
		end

	unknown_imc: attached RP_INVESTMENT_MANAGEMENT_COMPANY is
			-- Unknown IMC instance
		once
			create {RP_UNKNOWN_IMC} Result.make (Current)
		ensure
			attached_result: Result /= Void
		end

	unknown_zonal_product: RP_ZONAL_PRODUCT is
			-- Unknown zonal product instance
		once
			create Result.make (Current, Void)
		end

	unknown_global_product: RP_GLOBAL_PRODUCT is
			-- Unknown global product instance
		once
			create Result.make (Current, Void)
		end

	unknown_investment_region: RP_ZONE is
			-- Unknown investment region instance
		once
			create Result.make (Current, Void, Void, create {ROSE_DATE}.make_ccyymmdd (99991231))
		end

	unknown_country: RP_COUNTRY is
			-- Unknown country instance
		once
			create Result.make_with_country (Current, Void)
		end

	unknown_strategy: RP_STRATEGY is
			-- Unknown strategy instance
		once
			create Result.make (Void)
		end

	unknown_selection_universe: RP_SELECTION_UNIVERSE is
			-- Unknown selection universe instance
		once
			create Result.make (Current, Void)
		end

feature {NONE} -- Implementation

	saved_examined_products: ROSE_LINEAR_ARRAY [RP_PRODUCT]
			-- Cached list of GIPS eamined composites

	portfolio_cached (a_portfolio_code: STRING; a_portfolio: RP_PORTFOLIO): BOOLEAN is
			-- Does 'a_porfolio' match 'a_portfolio_code'?
		require
			valid_code: a_portfolio_code /= Void
			valid_portfolio: a_portfolio /= Void
		local
			l_portfolio_code: STRING
		do
			create l_portfolio_code.make_from_string (a_portfolio_code)
			l_portfolio_code.prune_all (' ')
			l_portfolio_code.to_upper
			Result := a_portfolio.is_unknown or else portfolios_cache.has (l_portfolio_code)
		end

	portfolio_matches_name (a_portfolio_code: STRING; a_portfolio: RP_PORTFOLIO): BOOLEAN is
			-- Does 'a_porfolio' match 'a_portfolio_code'?
		require
			valid_code: a_portfolio_code /= Void
			valid_portfolio: a_portfolio /= Void
		local
			l_portfolio_code: STRING
		do
			create l_portfolio_code.make_from_string (a_portfolio_code)
			l_portfolio_code.prune_all (' ')
			l_portfolio_code.to_upper
			Result := a_portfolio.is_unknown or else a_portfolio.code.is_equal (l_portfolio_code)
		end

	fix_bad_gedb_brimsym (a_brimsym: STRING): STRING is
			-- Translate futures and riemsyms to brimsym
		require
			brimsym_not_void: a_brimsym /= Void
		local
			l_future: FUTURE
			l_listing: LISTING
		do
			if a_brimsym.count = 5 and then a_brimsym.item (1) = 'X' then
					-- GEDB Loader does not convert Xxxxx to futures brimsym
					-- we will do it here
				l_future := context.future_at (brim_code ("JPF.." + a_brimsym.substring (2, 5) + "...%%"), Void)
				if l_future /= Void then
					Result := l_future.brim_code (Void)
				end
			elseif a_brimsym.count = 10 and then a_brimsym.substring (1, 2).is_equal ("G ") and a_brimsym.substring (3, 10).is_integer then
					-- "G ccyymmdd" RIEMSYM
				Result := "USG" + a_brimsym.substring (3, 10) + "....US."
			elseif a_brimsym.is_integer then
					-- RIEMSYM
				l_listing := context.listing_at (riemsym_code (a_brimsym), Void)
				if l_listing /= Void then
					Result := l_listing.brim_code (Void)
				end
			end
			if Result = Void then
				Result := a_brimsym
			end
		ensure
			non_void_result: Result /= Void
		end

	legacy_instrument_from_brim_code (a_brimsym: STRING): LEGACY_INSTRUMENT is
			-- Legacy instrument for a_brimsym
		require
			brimsym_not_void: a_brimsym /= Void
		local
			l_recs: ROSE_LINEAR_ARRAY [ENTITY_ATTRIBUTE_RECORD]
		do
			l_recs := context.entity_records (a_brimsym, context.listing_accessor, context.brim_code)
			if l_recs.count > 0 then
				Result := context.listing_accessor.entity (l_recs.first.entity_id)
			else
				l_recs := context.entity_records (a_brimsym, context.listing_accessor, context.brim_code)
				if l_recs.count > 0 then
					Result := context.listing_accessor.entity (l_recs.first.entity_id)
				else
					l_recs := context.entity_records (a_brimsym, context.listing_accessor, context.local_brim_code)
					if l_recs.count > 0 then
						Result := context.listing_accessor.entity (l_recs.first.entity_id)
					else
						l_recs := context.entity_records (a_brimsym, context.listing_accessor, context.e_brim_code)
						if l_recs.count > 0 then
							Result := context.listing_accessor.entity (l_recs.first.entity_id)
						else
							l_recs := context.entity_records (a_brimsym, context.listing_accessor, context.j_brim_code)
							if l_recs.count > 0 then
								Result := context.listing_accessor.entity (l_recs.first.entity_id)
							else
								l_recs := context.entity_records (a_brimsym, context.listing_accessor, context.s_brim_code)
								if l_recs.count > 0 then
									Result := context.listing_accessor.entity (l_recs.first.entity_id)
								else
									l_recs := context.entity_records (a_brimsym, context.bond_accessor, context.brim_code)
									if l_recs.count > 0 then
										Result := context.bond_accessor.entity (l_recs.first.entity_id)
									else
										l_recs := context.entity_records (a_brimsym, context.government_issue_accessor, context.brim_code)
										if l_recs.count > 0 then
											Result := context.government_issue_accessor.entity (l_recs.first.entity_id)
										else
											l_recs := context.entity_records (a_brimsym, context.future_accessor, context.brim_code)
											if l_recs.count > 0 then
												Result := context.future_accessor.entity (l_recs.first.entity_id)
											end
										end
									end
								end
							end
						end
					end
				end
			end
		end

	make_instrument_from_brimsym (a_brimsym: STRING; a_legacy_instrument: LEGACY_INSTRUMENT; a_date: ROSE_DATE): RP_INSTRUMENT is
			-- create rp_instrument from a_brimsym and a_legacy_instrument on a_date
		local
			l_brimsym_type: CHARACTER
			l_listing: LISTING
			l_bond: BOND
			l_government_issue: GOVERNMENT_ISSUE
			l_corporate_bond: RP_CORPORATE_BOND
		do
				-- parse the brimsym and
				-- handle each brimsym type and
				-- instantiate the appropriate RP_INSTRUMENT type
				-- get an entity from the brimsym (listing, bond, future, etc.)
			l_brimsym_type := a_brimsym.item (3)
			inspect l_brimsym_type
			when 'B' then
					-- Bond
				if a_legacy_instrument /= Void then
					l_bond ?= a_legacy_instrument
				end
				if l_bond = Void then
					l_bond := context.bond_at (brim_code (a_brimsym), a_date)
				end
				if l_bond /= Void then
					create l_corporate_bond.make_with_bond (Current, l_bond, a_brimsym)
					add_frn_rates (l_corporate_bond)
					Result := l_corporate_bond
				end
			when 'C' then
					-- Composite
				if a_legacy_instrument /= Void then
					l_listing ?= a_legacy_instrument
				end
				if l_listing = Void then
					l_listing := context.listing_by_brim_code (a_brimsym, a_date)
				end
				if l_listing = Void then
					l_listing := context.listing_by_brim_code (a_brimsym + "%%", a_date)
				end
				if l_listing /= Void then
					if a_brimsym.item (4) = 'F' and a_brimsym.item (5) = '0' then
							-- treat O zone ??CF0* listings as stocks
						Result := create {RP_STOCK}.make (Current, a_brimsym, l_listing)
					else
						Result := create {RP_POOL_FUND}.make (l_listing, l_listing.brim_code_key (a_date), pool_fund_portfolio (a_brimsym))
					end
				end
			when 'D' then
					-- FixedDeposit
				Result := fixed_deposit_from_brimsym (a_brimsym)
			when 'O', 'E' then
					-- Equity, Cover the only odd USOOPTION100...US.
				if a_legacy_instrument /= Void then
					l_listing ?= a_legacy_instrument
				end
				if l_listing = Void then
					l_listing := context.listing_by_brim_code (a_brimsym, a_date)
				end
				if l_listing = Void then
					l_listing := context.listing_at (local_brim_code (a_brimsym), a_date)
				end
				if l_listing = Void then
					l_listing := context.listing_at (e_brim_code (a_brimsym), a_date)
				end
				if l_listing = Void then
					l_listing := context.listing_at (j_brim_code (a_brimsym), a_date)
				end
				if l_listing = Void then
					l_listing := context.listing_at (s_brim_code (a_brimsym), a_date)
				end
				if l_listing /= Void then
					Result := create {RP_STOCK}.make (Current, a_brimsym, l_listing)
				end
			when 'F' then
					-- Index Future
				if valid_index_future_brimsym (a_brimsym) then
					Result := index_future_from_brimsym (a_brimsym, a_date)
				end
			when 'G' then
					-- Government Bond
				if a_legacy_instrument /= Void then
					l_government_issue ?= a_legacy_instrument
				end
				if l_government_issue = Void then
					l_government_issue := context.government_issue_accessor.any_entity_at (brim_code (a_brimsym), a_date)
				end
				if l_government_issue /= Void then
					Result := create {RP_GOVERNMENT_BOND}.make_with_government_issue (l_government_issue, a_brimsym)
				end
			when 'I' then
					-- Index (really ETF)
				if a_legacy_instrument /= Void then
					l_listing ?= a_legacy_instrument
				end
				if l_listing = Void then
					l_listing := context.listing_by_brim_code (a_brimsym, a_date)
				end
				if l_listing = Void then
					l_listing := context.listing_at (local_brim_code (a_brimsym), a_date)
				end
				if l_listing = Void then
					l_listing := context.listing_at (e_brim_code (a_brimsym), a_date)
				end
				if l_listing = Void then
					l_listing := context.listing_at (j_brim_code (a_brimsym), a_date)
				end
				if l_listing = Void then
					l_listing := context.listing_at (s_brim_code (a_brimsym), a_date)
				end
				if l_listing /= Void then
					Result := create {RP_EXCHANGE_TRADED_FUND}.make (Current, a_brimsym, l_listing)
				end
			when 'M' then
					-- Cash
				Result := cash_from_country (a_brimsym.substring (1, 2), a_date)
			when 'T' then
					-- Temp or TBILL
				if a_legacy_instrument /= Void then
					l_listing ?= a_legacy_instrument
				end
				if l_listing = Void then
					l_listing := context.listing_by_brim_code (a_brimsym, a_date)
				end
				if l_listing /= Void then
					if a_brimsym.is_equal ("USTTBILL0100...US.") then
						Result := create {RP_GOVERNMENT_BOND}.make_from_listing (a_brimsym, l_listing)
					else
						Result := create {RP_STOCK}.make (Current, a_brimsym, l_listing)
					end
				end
			when '/' then
					-- FX and Bank Transfers ( for type 24,40,47)
					-- For all practical purposes this is cash
				Result := cash_from_country (a_brimsym.substring (1, 2), a_date)
			else
					-- Do nothing (return Void)
			end
		end

	instrument_cache: DS_HASH_TABLE [RP_INSTRUMENT, STRING]
			-- Instrument by brimsym cache

	currency_cache: RP_CURRENCY_CACHE
			-- Currency cache

	brokers_cache: DS_HASH_TABLE [RP_BROKER, STRING]
			-- Brokers cache

	all_brokers: like brokers
			-- List of all brokers

	retrieve_all_brokers: like brokers is
			-- Retrieve all brokers
		do
			if all_brokers = Void then
				brokers_cache.wipe_out
				gcdb.context.broker_accessor.all_entities.do_all (agent add_broker)
				all_brokers := brokers_cache.to_array
			end
			Result := all_brokers
		ensure
			non_void_result: Result /= Void
		end

	add_broker (a_broker: GCDB_BROKER) is
			-- Add broker from `a_broker'
		local
			l_br: RP_BROKER
		do
			create l_br.make (Current, a_broker.code)
			brokers_cache.force (l_br, l_br.code)
		end

	portfolios_cache: DS_HASH_TABLE [RP_PORTFOLIO, STRING]
			-- Portfolios by code cache

	all_portfolios: like portfolios
			-- List of all portfolios

	all_fund_participant_portfolios: like fund_participant_portfolios
			-- List of all fund partitipant portfolios

	participants: DS_HASH_SET [RP_PORTFOLIO]
			-- Fund participant portfolios by code cache

	retrieve_all_portfolios is
			-- Retrieve all portfolios
		do
			if all_portfolios = Void then
				create portfolios_cache.make_equal (17)
				create participants.make_equal (17)
				gcdb.context.all_relationship_records (gcdb.context.portfolio_to_investment_region).do_nothing
				gcdb.context.portfolio_accessor.all_entities.do_all (agent add_portfolio (?))
				gcdb.context.fund_participant_accessor.all_entities.do_all (agent add_gcdb_fund_participant (?))
				all_portfolios := portfolios_cache.to_array
				all_fund_participant_portfolios := participants.to_array
			end
		ensure
			non_void_result: all_portfolios /= Void and all_fund_participant_portfolios /= Void
		end

	add_portfolio (a_portfolio: GCDB_PORTFOLIO) is
			-- Add portfolio from `a_portfolio'
		require
			non_void_portfolio: a_portfolio /= Void
		local
			l_port: RP_PORTFOLIO
		do
			create l_port.make (Current, a_portfolio, Void, Void)
			portfolios_cache.force (l_port, l_port.code)
		ensure
			cache_updated: portfolios_cache.has (a_portfolio.code)
		end

	add_gcdb_fund_participant (a_gcdb_fund_participant: GCDB_FUND_PARTICIPANT) is
			-- Add portfolio from `a_gcdb_fund_participant'
		require
			non_void_entity: a_gcdb_fund_participant /= Void
		local
			l_port: RP_PORTFOLIO
			l_instrument: RP_INSTRUMENT
		do
			if a_gcdb_fund_participant.brimsym /= Void then
				l_instrument := instrument_from_brimsym (a_gcdb_fund_participant.brimsym)
				if {l_pool_fund: RP_POOL_FUND} l_instrument then
					create l_port.make (Current, Void, Void, a_gcdb_fund_participant)
					portfolios_cache.force (l_port, l_port.code)
					participants.force (l_port)
				end
			end
		end

	investment_accounts_cache: DS_HASH_TABLE [RP_INVESTMENT_ACCOUNT, STRING]
			-- Investment accounts by code cache

	all_investment_accounts: like investment_accounts
			-- List of all investment accounts

	retrieve_all_investment_accounts: like investment_accounts is
			-- Retrieve all investment accounts
		local
			l_acct_by_code: DS_HASH_TABLE [GCDB_ACCOUNT, STRING]
		do
			if all_investment_accounts = Void then
				investment_accounts_cache.wipe_out
				create l_acct_by_code.make_equal (17)
				gcdb.context.all_relationship_records (gcdb.context.account_to_porfolio).do_nothing
				gcdb.context.all_relationship_records (gcdb.context.portfolio_group_member_to_portfolio).do_nothing
				gcdb.context.account_accessor.all_entities.do_all (agent add_gcdb_account (l_acct_by_code, ?))
				gcdb.context.portfolio_group_accessor.all_entities.do_all (agent add_portfolio_group (l_acct_by_code, ?))
				l_acct_by_code.to_array.do_all (agent add_account)
				gcdb.context.fund_participant_accessor.all_entities.do_all (agent add_fund_participant_account (?))
				all_investment_accounts := investment_accounts_cache.to_array
			end
			Result := all_investment_accounts
		ensure
			non_void_result: Result /= Void
			no_void_items: not Result.has (Void)
			result_cached: all_investment_accounts /= Void
		end

	add_gcdb_account (a_acct_by_code: DS_HASH_TABLE [GCDB_ACCOUNT, STRING]; a_acct: GCDB_ACCOUNT) is
			-- Add account from `a_acct'
		do
			a_acct_by_code.force (a_acct, a_acct.code)
		end

	add_portfolio_group (a_acct_by_code: DS_HASH_TABLE [GCDB_ACCOUNT, STRING]; a_portfolio_group: GCDB_PORTFOLIO_GROUP) is
			-- Add account from `a_portfolio_group'
		local
			l_acct: RP_INVESTMENT_ACCOUNT
			l_gcdb_acct: GCDB_ACCOUNT
		do
			if a_acct_by_code.has (a_portfolio_group.code) then
				l_gcdb_acct := a_acct_by_code.item (a_portfolio_group.code)
				a_acct_by_code.remove (a_portfolio_group.code)
			end
			create l_acct.make (Current, l_gcdb_acct, a_portfolio_group, Void)
			investment_accounts_cache.force (l_acct, l_acct.name)
		end

	add_fund_participant_account (a_gcdb_fund_participant: GCDB_FUND_PARTICIPANT) is
			-- Add investment account from fund participant
		local
			l_acct: RP_INVESTMENT_ACCOUNT
			l_instrument: RP_INSTRUMENT
		do
			if a_gcdb_fund_participant.brimsym /= Void then
				l_instrument := instrument_from_brimsym (a_gcdb_fund_participant.brimsym)
				if {l_pool_fund: RP_POOL_FUND} l_instrument then
					create l_acct.make (Current, Void, Void, a_gcdb_fund_participant)
					investment_accounts_cache.force (l_acct, l_acct.name)
				end
			end
		end

	add_account (a_account: GCDB_ACCOUNT) is
			-- Add account from `a_account'
		local
			l_acct: RP_INVESTMENT_ACCOUNT
		do
			create l_acct.make (Current, a_account, Void, Void)
			investment_accounts_cache.force (l_acct, l_acct.name)
		end

	clients_cache: DS_HASH_TABLE [RP_CLIENT, STRING]
			-- Clients by code cache

	all_clients: like clients
			-- List of all clients

	retrieve_all_clients: like clients is
			-- Retrieve all clients
		do
			if all_clients = Void then
				clients_cache.wipe_out
				gcdb.context.client_accessor.all_entities.do_all (agent add_client)
				all_clients := clients_cache.to_array
			end
			Result := all_clients
		ensure
			non_void_result: Result /= Void
		end

	add_client (a_client: GCDB_CLIENT) is
			-- Add client from `a_client'
		local
			l_client: RP_CLIENT
		do
			create l_client.make (Current, a_client)
			clients_cache.force (l_client, l_client.code)
		end

	tokyo: RP_TOKYO is
			-- Instance of J zone IMC
		once
			create Result.make (Current)
		end

	orinda: RP_ORINDA is
			-- Instance of U zone IMC
		once
			create Result.make (Current)
		end

	singapore: RP_SINGAPORE is
			-- Instance of O Zone IMC
		once
			create Result.make (Current)
		end

	london: RP_LONDON is
			-- Instance of E zone IMC
		once
			create Result.make (Current)
		end

	imcs_by_code: HASH_TABLE [RP_INVESTMENT_MANAGEMENT_COMPANY, STRING] is
			-- IMC by code cache
		once
			create Result.make (4)
			Result.extend (tokyo, tokyo.code)
			Result.extend (orinda, orinda.code)
			Result.extend (singapore, singapore.code)
			Result.extend (london, london.code)
		end

	zones_by_code: HASH_TABLE [!RP_ZONE, STRING] is
			-- Zone by code cache
		once
			create Result.make (7)
			Result.extend (c_zone, c_zone.code)
			Result.extend (e_zone, e_zone.code)
			Result.extend (j_zone, j_zone.code)
			Result.extend (o_zone, o_zone.code)
			Result.extend (s_zone, s_zone.code)
			Result.extend (u_zone, u_zone.code)
			Result.extend (t_zone, t_zone.code)
			Result.extend (w_zone, w_zone.code)
		ensure
			zones_by_code_not_void: Result /= Void
			no_void_zone_code: not Result.has (Void)
			no_void_zone: not Result.has_item (Void)
		end

	benchmarks_cache: DS_HASH_TABLE [RP_BENCHMARK, STRING]
			-- Benchmarks by code cache

	all_benchmarks: like benchmarks
			-- List of all benchmarks

	retrieve_benchmarks: like benchmarks is
			-- Retrieve all benchmarks.
		do
			if all_clients = Void then
				benchmarks_cache.wipe_out
				gcdb.context.benchmark_accessor.all_entities.do_all (agent add_benchmark (?))
				all_benchmarks := benchmarks_cache.to_array
			end
			Result := all_benchmarks
		ensure
			non_void_list: Result /= Void
		end

	add_benchmark (a_gcdb_benchmark: GCDB_BENCHMARK) is
			-- Convert `a_gcdb_benchmark' to RP_BENCHMARK and add to `benchmarks_cache'.
		local
			l_benchmark: RP_BENCHMARK
		do
			create l_benchmark.make (Current, a_gcdb_benchmark)
			benchmarks_cache.force (l_benchmark, l_benchmark.code)
		end

	countries_cache: DS_HASH_TABLE [like country_from_iso_code, STRING]
			-- Countries by code cache

	country_by_iso_3166_alpha2_code (a_code: STRING): like country_from_iso_code is
			-- Country by `a_code'
		require
			non_void_code: a_code /= Void
		local
			l_ctry: COUNTRY
		do
			if a_code.count = 2 then
				if countries_cache.has (a_code.as_upper) then
					Result := countries_cache.item (a_code.as_upper)
				else
					l_ctry := context.country_at (iso_code (a_code), Void)
					if l_ctry /= Void then
						create Result.make_with_country (Current, l_ctry)
						countries_cache.force (Result, a_code.as_upper)
					end
				end
			end
			if Result = Void then
				Result := unknown_country
			end
		ensure
			country_not_void: Result /= Void
		end

	strategies_cache: DS_HASH_TABLE [RP_STRATEGY, STRING]
			-- cache for strategies

	retrieve_strategy (a_code: STRING): like strategy_by_code is
			-- retrieve strategy a_code
		local
			l_strat: GCDB_STRATEGY_TYPE
		do
			if a_code /= Void then
				if strategies_cache.has (a_code) then
					Result := strategies_cache.item (a_code)
				else
					l_strat := gcdb.strategy_type_by_code (a_code, Void)
					if l_strat /= Void then
						create Result.make (l_strat)
						strategies_cache.force (Result, a_code)
					end
				end
			end
			if Result = Void then
				Result := unknown_strategy
			end
		ensure
			non_void_result: Result /= Void
		end

	selection_universe_cache: DS_HASH_TABLE [RP_SELECTION_UNIVERSE, STRING]
			-- Map of codes to cached selection universes

	all_selection_universes: like selection_universes
			-- Cached result for `selection_universes'

	add_selection_universe (a_universe: GCDB_SELECTION_UNIVERSE) is
			-- Add wrapper `a_universe' to `selection_universe_cache'.
		require
			a_universe_not_void: a_universe /= Void
			no_duplicates: not selection_universe_cache.has (a_universe.code)
		local
			l_universe: RP_SELECTION_UNIVERSE
		do
			create l_universe.make (Current, a_universe)
			selection_universe_cache.force_new (l_universe, l_universe.code)
		ensure
			one_more: selection_universe_cache.count = old selection_universe_cache.count + 1
			correct_universe_added: selection_universe_cache.has (a_universe.code)
		end

	zonal_products_cache_by_code: DS_HASH_TABLE [RP_ZONAL_PRODUCT, STRING]
			-- Zonal products cache

	global_products_cache_by_code: DS_HASH_TABLE [RP_GLOBAL_PRODUCT, STRING]
			-- Global products cache

	all_products: like products
			-- all products

	retrieve_products: like products is
			-- retrieve products
		local
			l_result: ROSE_LINEAR_ARRAY [RP_PRODUCT]
		do
			if all_products = Void then
				zonal_products_cache_by_code.wipe_out
				global_products_cache_by_code.wipe_out
				gcdb.context.composite_accessor.all_entities.do_all (agent cache_product (?))
				create l_result.make_from_array (zonal_products_cache_by_code.to_array)
				l_result.append_array (global_products_cache_by_code.to_array)
				l_result.sort_by (agent compare_product_codes (?, ?))
				all_products := l_result
			end
			Result := all_products
		end

	cache_product (a_product: GCDB_PRODUCT) is
			-- Convert GCDB_PRODUCT to RP_PRODUCT by retrieving it by code
		require
			non_void_arg: a_product /= Void
		do
			product_by_code (a_product.code).do_nothing
		end

	compare_product_codes (a, b: RP_PRODUCT): BOOLEAN is
			-- Compare product codes
		require
			non_void_args: a /= Void and b /= Void
		do
			Result := a.code < b.code
		end

	cash_from_country (a_country_code: STRING; a_date: ROSE_DATE): detachable RP_CURRENCY is
			-- Retrieve currency from BRIMSYM country
		require
			non_void_args: a_country_code /= Void
			valid_length_of_country_code: a_country_code.count = 2
		do
			Result := currency_cache.cash_from_country (a_country_code, a_date)
		ensure
			cash_from_country_can_be_void: True
		end

	retrieve_portfolio_products (a_result: like products_by_portfolio; a_portfolio: RP_PORTFOLIO; a_product: RP_ZONAL_PRODUCT) is
			-- Match intervals in `a_product.portfolios' to `a_portfolio' and then add to `a_result'
		local
			l_interval: RP_PRODUCT_HISTORY_INTERVAL
			l_history: RP_PORTFOLIO_HISTORY
			l_portfolio_interval: RP_PORTFOLIO_HISTORY_INTERVAL
			i: INTEGER
		do
			from
				l_history := a_product.portfolios
				i := 1
			until
				i > l_history.count
			loop
				l_portfolio_interval := l_history.item (i)
				if l_portfolio_interval.portfolio.is_equal (a_portfolio) then
					create l_interval.make (a_product, l_portfolio_interval.start_date, l_portfolio_interval.end_date, l_portfolio_interval.weight)
					a_result.extend (l_interval)
				end
				i := i + 1
			end
		end

feature -- Brimsym Checks

	valid_brimsym (a_brimsym: STRING): BOOLEAN is
			-- Is `a_brimsym' valid brimsym code?
		do
			if a_brimsym /= Void then
				Result := a_brimsym.count = 18
			end
		end

	valid_index_future_brimsym (a_brimsym: STRING): BOOLEAN is
			-- Is `a_brimsym' valid index future brimsym code?
		do
			if valid_brimsym (a_brimsym) then
				Result := a_brimsym.item (3) = 'F'
			end
		end

feature {NONE} -- Fixed deposits

	valid_fixed_deposit_brimsym (a_brimsym: STRING): BOOLEAN is
			-- Is `a_brimsym' valid fixed deposit brimsym?
		do
			if valid_brimsym (a_brimsym) then
				Result := a_brimsym.item (3) = 'D'
			end
		end

	fixed_deposit_from_brimsym (a_brimsym: STRING): RP_FIXED_DEPOSIT is
			-- Lookup fixed deposit instrument identified by `a_brimsym'
		require
			valid_fixed_deposit_brimsym: valid_fixed_deposit_brimsym (a_brimsym)
		do
			create Result.make (Current, a_brimsym, fixed_deposit_rate_override (a_brimsym))
		ensure
			non_void_result: Result /= Void
		end

	fixed_deposit_rate_override (a_brimsym: STRING): RP_DECIMAL is
			-- Fixed deposit rate override for `a_brimsym'
		require
			valid_brimsym: valid_fixed_deposit_brimsym (a_brimsym)
		do
			retrieve_fixed_deposit_rate_overrides
			cached_fixed_deposit_rate_overrides.search (a_brimsym)
			if cached_fixed_deposit_rate_overrides.found then
				Result := cached_fixed_deposit_rate_overrides.found_item
			else
				create Result.make_from_integer (0)
			end
		ensure
			non_void_result: Result /= Void
		end

	cached_fixed_deposit_rate_overrides: DS_HASH_TABLE [RP_DECIMAL, STRING]
			-- Cached fixed deposit rate overrides

	retrieve_fixed_deposit_rate_overrides_predicate: STRING_PREDICATE is
			-- Predicate to retrieve fixed deposit rate overrides
		once
			create Result.make (servers.performance_server, "SELECT brimsym + '|' + CAST(interest_rate AS varchar(50)) FROM fixed_deposit_rate_override")
		end

	retrieve_fixed_deposit_rate_overrides is
			-- Retrieve fixed deposite rate overrides
		do
			if cached_fixed_deposit_rate_overrides = Void then
				create cached_fixed_deposit_rate_overrides.make_equal (17)
				retrieve_fixed_deposit_rate_overrides_predicate.retrieve (Void, Void)
				retrieve_fixed_deposit_rate_overrides_predicate.array.do_all (agent add_fixed_deposite_rate_override (cached_fixed_deposit_rate_overrides, ?))
			end
		ensure
			cache_create: cached_fixed_deposit_rate_overrides /= Void
		end

	add_fixed_deposite_rate_override (a_cache: like cached_fixed_deposit_rate_overrides; s: STRING) is
			-- Add rate for brimsym encoded in `s' to `a_cache'
		require
			non_void_args: a_cache /= Void and s /= Void
		local
			i: INTEGER
			l_brimsym: STRING
			l_rate: RP_DECIMAL
		do
			i := s.index_of ('|', 1)
			if i > 0 then
				create l_rate.make_from_string (s.substring (i + 1, s.count))
				l_brimsym := s.substring (1, i - 1)
				a_cache.force (l_rate, l_brimsym)
			end
		end

feature {NONE} -- Cross-zonal linkage ala vms RMDAT_W call

	cached_cross_zonal_linkages: DS_HASH_TABLE [ROSE_INTERVAL_HISTORY [STRING, ROSE_CLOSED_INTERVAL [STRING]], STRING]
			-- Cached cross-zonal linkages

	retrieve_cross_zonal_linkages_predicate: STRING_PREDICATE is
			-- Predicate to retrieve cross zonal linkages
		once
			create Result.make (servers.performance_server, "select from_brimsym + '|' + to_brimsym + '|' + cast(start_ccyymmdd as varchar(8))+ '|' + cast(finish_ccyymmdd as varchar(8)) from main_listings_parm")
		end

	retrieve_cross_zonal_linkages is
			-- Retrieve cross zonal linkages
		do
			if cached_cross_zonal_linkages = Void then
				create cached_cross_zonal_linkages.make_equal (17)
				retrieve_cross_zonal_linkages_predicate.retrieve (Void, Void)
				retrieve_cross_zonal_linkages_predicate.array.do_all (agent add_cross_zonal_linkage (cached_cross_zonal_linkages, ?))
			end
		ensure
			cache_create: cached_cross_zonal_linkages /= Void
		end

	add_cross_zonal_linkage (a_cache: like cached_cross_zonal_linkages; s: STRING) is
			-- Add cross-zonal linkage from `s' to `a_cache'
		require
			non_void_args: a_cache /= Void and s /= Void
		local
			l_tokenizer: ROSE_DELIMITED_TEXT
			l_from, l_to: STRING
			l_start, l_finish: ROSE_DATE
			l_interval: ROSE_CLOSED_INTERVAL [STRING]
			l_mapping: ROSE_INTERVAL_HISTORY [STRING, ROSE_CLOSED_INTERVAL [STRING]]
		do
			create l_tokenizer.make (s, '|')
			l_from := l_tokenizer.item (1)
			if l_from /= Void then
				l_to := l_tokenizer.item (2)
				if l_to /= Void then
					create l_start.make_ccyymmdd (l_tokenizer.item (3).to_integer)
					create l_finish.make_ccyymmdd (l_tokenizer.item (4).to_integer)
					create l_interval.make (l_to, l_start, l_finish, 1)
					a_cache.search (l_from)
					if a_cache.found then
						l_mapping := a_cache.found_item
					else
						create l_mapping.make_empty
						a_cache.force (l_mapping, l_from)
					end
					l_mapping.extend (l_interval)
				end
			end
		end

feature {NONE} -- Bonds

	add_frn_rates_predicate: STRING_PREDICATE is
			-- Predicate to add FRN rates
		once
			create Result.make (servers.performance_server_fast_load, "select cast(start_ccyymmdd as varchar(8)) + '|' + cast(finish_ccyymmdd as varchar(8)) + '|' + cast (rate as varchar(50)) from bond_frn_rate_parm WHERE brimsym = ?")
		end

	add_frn_rates (a_corporate_bond: RP_CORPORATE_BOND) is
			-- Add FRN rates to `a_corporate_bond'
		require
			non_void_args: a_corporate_bond /= Void
		do
			add_frn_rates_predicate.retrieve (Void, <<a_corporate_bond.brimsym>>)
			add_frn_rates_predicate.array.do_all (agent add_frn_rate (a_corporate_bond, ?))
		end

	add_frn_rate (a_corporate_bond: RP_CORPORATE_BOND; s: STRING) is
			-- Add FRN rate encoded in `s' to `a_corporate_bond'
		require
			non_void_args: a_corporate_bond /= Void and s /= Void
		local
			l_tokenizer: ROSE_DELIMITED_TEXT
			l_start, l_finish: ROSE_DATE
			l_rate: RP_DECIMAL
		do
			create l_tokenizer.make (s, '|')
			create l_start.make_ccyymmdd (l_tokenizer.item (1).to_integer)
			create l_finish.make_ccyymmdd (l_tokenizer.item (2).to_integer)
			create l_rate.make_from_string (l_tokenizer.item (3))
			a_corporate_bond.add_frn_rate (l_start, l_finish, l_rate)
		end

	add_coupon_dates_predicate: INTEGER_PREDICATE is
			-- Predicate to add coupon dates
		once
			create Result.make (servers.performance_server_fast_load, "select coupon_ccyymmdd from bond_coupon_date_parm where brimsym = ? order by coupon_ccyymmdd")
		end

	add_coupon_dates (a_corporate_bond: RP_CORPORATE_BOND) is
			-- Add coupon dates to `a_corporate_bond'
		require
			non_void_args: a_corporate_bond /= Void
		do
			add_coupon_dates_predicate.retrieve (Void, <<a_corporate_bond.brimsym>>)
			add_coupon_dates_predicate.array.do_all (agent add_coupon_date (a_corporate_bond, ?))
		end

	add_coupon_date (a_corporate_bond: RP_CORPORATE_BOND; a_ccyymmdd: INTEGER) is
			-- Add Coupon date in `a_ccyymmdd' to `a_corporate_bond'
		require
			non_void_args: a_corporate_bond /= Void
		do
			a_corporate_bond.add_coupon_date (create {ROSE_DATE}.make_ccyymmdd (a_ccyymmdd))
		end

feature {NONE} -- Currency from brimcode



	pool_fund_portfolio (a_brimsym: STRING): RP_PORTFOLIO is
			-- Map pool fund `a_brimsym' to portfolio
		require
			non_void_brimsym: a_brimsym /= Void
			valid_brimsym: a_brimsym.count > 14
		do
			if brimcosec_to_code.has (a_brimsym.substring (1, 15)) then
				Result := portfolio_from_code (brimcosec_to_code.item (a_brimsym.substring (1, 15)))
			else
				Result := unknown_portfolio
			end
		end

	brimcosec_to_code: DS_HASH_TABLE [STRING, STRING] is
			-- BRIMCO+SEC to FUNDNAME mappings from COMPOSITE_ASSETS
		local
			l_query: STRING_PREDICATE
		once
			create l_query.make (servers.common_server_fast_load, "SELECT DISTINCT brimco + sec + '|' + fundname FROM dbo.composite_assets")
			l_query.retrieve (Void, Void)
			create Result.make_equal (l_query.array.count)
			l_query.array.do_all (agent add_brimcosec_to_code_mapping (Result, ?))
		ensure
			non_void_mappings: Result /= Void
		end

	add_brimcosec_to_code_mapping (a_mappings: like brimcosec_to_code; s: STRING) is
			-- Add pipe delmited brimcosec|code in `s' to `a_mappings'
		require
			non_void_args: a_mappings /= Void and s /= Void
		local
			i: INTEGER
		do
			i := s.index_of ('|', 1)
			if i > 1 then
				a_mappings.force (s.substring (i + 1, s.count), s.substring (1, i - 1))
			end
		end

	brimcosec_to_start_date: DS_HASH_TABLE [ROSE_DATE, STRING] is
			-- BRIMCO+SEC to start date mappings from COMPOSITE_ASSETS
		local
			l_query: STRING_PREDICATE
		once
			create l_query.make (servers.common_server_fast_load, "SELECT DISTINCT brimco + sec + '|' + cast(from_date as varchar(50)) FROM dbo.composite_assets")
			l_query.retrieve (Void, Void)
			create Result.make_equal (l_query.array.count)
			l_query.array.do_all (agent add_brimcosec_to_date_mapping (Result, ?))
		ensure
			non_void_mappings: Result /= Void
		end

	add_brimcosec_to_date_mapping (a_mappings: DS_HASH_TABLE [ROSE_DATE, STRING]; s: STRING) is
			-- Add pipe delmited brimcosec|ccyymmdd in `s' to `a_mappings'
		require
			non_void_args: a_mappings /= Void and s /= Void
		local
			i: INTEGER
		do
			i := s.index_of ('|', 1)
			if i > 1 then
				a_mappings.force (create {ROSE_DATE}.make_ccyymmdd (s.substring (i + 1, s.count).to_integer), s.substring (1, i - 1))
			end
		end

	brimcosec_to_end_date: DS_HASH_TABLE [ROSE_DATE, STRING] is
			-- BRIMCO+SEC to end date mappings from COMPOSITE_ASSETS
		local
			l_query: STRING_PREDICATE
		once
			create l_query.make (servers.common_server_fast_load, "SELECT DISTINCT brimco + sec + '|' + cast(to_date as varchar(50)) FROM dbo.composite_assets")
			l_query.retrieve (Void, Void)
			create Result.make_equal (l_query.array.count)
			l_query.array.do_all (agent add_brimcosec_to_date_mapping (Result, ?))
		ensure
			non_void_mappings: Result /= Void
		end

feature {RP_PORTFOLIO, RP_INVESTMENT_ACCOUNT} -- Fund participation

	is_fund_participant (a_portfolio: RP_PORTFOLIO): BOOLEAN is
			-- Is `a_portfolio' fund participant?
		require
			non_void_portfolio: a_portfolio /= Void
		do
			retrieve_fund_participants
			fund_participants.search (a_portfolio.code)
			Result := fund_participants.found
		end

	fund_participant_start_date (a_brimcosec: STRING): ROSE_DATE is
			-- Fund participant start date
		require
			non_void_arg: a_brimcosec /= Void
		do
			brimcosec_to_start_date.search (a_brimcosec)
			if brimcosec_to_start_date.found then
				Result := brimcosec_to_start_date.found_item
			end
		end

	fund_participant_end_date (a_brimcosec: STRING): ROSE_DATE is
			-- Fund participant end date
		require
			non_void_arg: a_brimcosec /= Void
		do
			brimcosec_to_end_date.search (a_brimcosec)
			if brimcosec_to_end_date.found then
				Result := brimcosec_to_end_date.found_item
			end
		end

feature {NONE} -- Fund participation implementation

	fund_participants: DS_HASH_SET [STRING]
			-- Fund participant portfolio codes

	retrieve_fund_participants_predicate: STRING_PREDICATE is
			-- Predicate to retrieve fund participants
		once
			create Result.make (servers.gedb_server, "select h.portfolio from gedb_gload..traded_holdings h left join common..composite_assets c on c.brimco+c.sec=h.instrument where h.applicable_from<=getdate() and h.applicable_to>getdate() and h.units<>0 group by h.portfolio having sum(case when c.brimco is null then 1 else 0 end)=0 and sum(case when c.brimco is null then 0 else 1 end)>0 order by h.portfolio")
		end

	retrieve_fund_participants is
			-- Retrieve fund participants and populate `fund_participants'
		do
			if fund_participants = Void then
				create fund_participants.make_equal (17)
				retrieve_fund_participants_predicate.retrieve (Void, Void)
				retrieve_fund_participants_predicate.array.do_all (agent add_fund_participant)
			end
		ensure
			non_void_fund_participants: fund_participants /= Void
		end

	add_fund_participant (a_code: STRING) is
			-- Add portfolio code to list of fund particpants,
			-- having stripped any trailing spaces off the retrieved code
		local
			l_code: STRING
		do
			l_code := a_code.twin
			l_code.prune_all_trailing (' ')
			fund_participants.force (l_code)
		end

feature {NONE} -- Portfolio First Event Date Implementation

	saved_nof_investment_accounts: like nof_investment_accounts
			-- Cached nof_investment_accounts

	retrieve_nof_investment_accounts_predicate: STRING_PREDICATE is
			-- Predicate to retrieve NOF investment accounts
		once
			create Result.make ((create {DATABASE_ACCESS}).make_from_repository_fast_load ("Invoicing_PROD"), "select distinct rtrim(ltrim(account)) from mgt_fees")
		end

	retrieve_nof_investment_accounts: like nof_investment_accounts is
			-- Retrieve nof_investment_accounts
		local
			l_list: DS_LINKED_LIST [RP_INVESTMENT_ACCOUNT]
		do
			if saved_nof_investment_accounts = Void then
				create l_list.make_default
				retrieve_nof_investment_accounts_predicate.retrieve (Void, Void)
				retrieve_nof_investment_accounts_predicate.array.do_all (agent add_nof_account (l_list, ?))
				saved_nof_investment_accounts := l_list.to_array
			end
			Result := saved_nof_investment_accounts
		end

	add_nof_account (a_list: DS_LINKED_LIST [RP_INVESTMENT_ACCOUNT]; s: STRING) is
			-- Add account identified by `s' to `a_list'
		local
			l_account: RP_INVESTMENT_ACCOUNT
		do
			l_account := investment_account_by_code (s)
			if not l_account.is_unknown then
				a_list.force_last (l_account)
			end
		end

feature {NONE} -- Tax rates

	tax_base_perf_cache: HASH_TABLE [ROSE_INTERVAL_HISTORY [RP_TAX_BASE_PARM, ROSE_CLOSED_INTERVAL [RP_TAX_BASE_PARM]], STRING] is
			-- cash for perf tax base
		local
			l_query: STRING_PREDICATE
			l_tokenizer: ROSE_DELIMITED_TEXT
			l_tax_base_interval: ROSE_CLOSED_INTERVAL [RP_TAX_BASE_PARM]
			l_tax_base: RP_TAX_BASE_PARM
			l_key: STRING
			l_tax_base_history: ROSE_INTERVAL_HISTORY [RP_TAX_BASE_PARM, ROSE_CLOSED_INTERVAL [RP_TAX_BASE_PARM]]
			i, l_starting_date: INTEGER
			l_ending_date: INTEGER
		once
			create Result.make (0)
			create l_query.make (servers.performance_server_fast_load, "SELECT [start_date]+'|'+[end_date]+'|'+[taxbas]+'|'+[description]+'|'+[Gross_Net]+'|'+[Recoverable]+'|'+[Use_Special_Rate]+'|'+[Variable_rate]+'|'+[Special_rate]+'|' FROM [tax_base_perf_parm]")
			l_query.retrieve (Void, Void)
			from
				i := l_query.array.lower
			until
				i > l_query.array.upper
			loop
				create l_tax_base
				create l_tokenizer.make_removing_spaces (l_query.array.item (i), '|')
				l_tax_base.set_start_date (l_tokenizer.item (1))
				l_tax_base.set_end_date (l_tokenizer.item (2))
				l_tokenizer.item (3).prune_all_trailing ('%U')
				l_tokenizer.item (3).prune_all_trailing (' ')
				l_tax_base.set_taxbas (l_tokenizer.item (3))
				l_tax_base.set_description (l_tokenizer.item (4))
				l_tax_base.set_gross_net (l_tokenizer.item (5))
				l_tax_base.set_recoverable (l_tokenizer.item (6))
				l_tax_base.set_use_special_rate (l_tokenizer.item (7))
				l_tax_base.set_variable_rate (l_tokenizer.item (8))
				l_tokenizer.item (9).prune_all ('%%')
				if l_tokenizer.item (9).is_empty then
					l_tokenizer.item (9).copy ("0.00")
				end
				l_tax_base.set_special_rate (l_tokenizer.item (9))
				l_key := l_tax_base.taxbas
				l_tax_base_history := Result.item (l_key)
				if l_tax_base_history = Void then
					create l_tax_base_history.make_empty
					Result.extend (l_tax_base_history, l_key)
				end
				l_starting_date := l_tax_base.start_date.to_integer
				l_ending_date := l_tax_base.end_date.to_integer
				if l_starting_date < 9999999 then
					l_starting_date := 19000101
				end
				if l_ending_date > 100000000 then
					l_ending_date := 99999999
				end
				create l_tax_base_interval.make (l_tax_base, create {ROSE_DATE}.make_ccyymmdd (l_starting_date), create {ROSE_DATE}.make_ccyymmdd (l_ending_date), l_tax_base.special_rate.to_double)
				l_tax_base_history.extend (l_tax_base_interval)
				i := i + 1
			end
		end

feature {NONE} -- Index futures

	index_future_from_brimsym (a_brimsym: STRING; a_date: ROSE_DATE): RP_INDEX_FUTURE is
			-- Index future from `a_brimsym' on `a_date'
		require
			vaid_brimsym: valid_brimsym (a_brimsym)
			valid_index_future_brimsym: valid_index_future_brimsym (a_brimsym)
			non_void_date: a_date /= Void
		local
			l_future: FUTURE
			l_future_listing: FUTURE_LISTING
			l_index: RP_STOCK_INDEX
			l_future_brimsym: STRING
		do
			l_future := context.future_at (brim_code (a_brimsym), a_date)
			if l_future = Void then
				l_future_listing := context.future_listing_at (brim_code (a_brimsym), a_date)
				if l_future_listing /= Void then
					l_future := l_future_listing.something_corelated_by_future_to_future_listing
				end
			end
			if l_future /= Void then
					-- XXX above lookup may fail
				create l_index.make (a_brimsym)
				l_future_brimsym := index_future_brimsym (a_brimsym, a_date)
				create Result.make (l_future, l_future_brimsym, a_date, l_index)
			end
		end

	index_future_brimsym (a_brimsym: STRING; a_date: ROSE_DATE): STRING is
			-- Index future brimsym from `a_brimsym' and `a_date'
		local
			l_maturity_date: ROSE_DATE
			i: INTEGER
		do
			l_maturity_date := maturity_date_from_date_and_brimsym_month (a_date, a_brimsym.item (9))
			create Result.make_from_string (a_brimsym)
			i := Result.substring_index ("....", 1)
			if i > 0 then
				Result.replace_substring (l_maturity_date.ccyymmdd.out.substring (3, 8), i, i + 5)
			end
		end

	maturity_date_from_date_and_brimsym_month (a_date: ROSE_DATE; a_month: CHARACTER): ROSE_DATE is
			-- Decode the maturity date from brimsym encoded month
		local
			l_month: INTEGER
		do
			inspect a_month
			when '1', '2', '3', '4', '5', '6', '7', '8', '9' then
				l_month := a_month.code - ('0').code
			when 'O' then
				l_month := 10
			when 'N' then
				l_month := 11
			when 'D' then
				l_month := 12
			end
			from
				Result := a_date
			until
				Result.month = l_month
			loop
				Result := Result.month_end
				Result := Result.next_date
			end
			Result := Result.month_end
		end

	currency_future_brimsym (a_currency: RP_CURRENCY; a_maturity_date: ROSE_DATE; a_future_symbol: STRING): STRING is
			-- Currency future brimsym from `a_currency', `a_maturity_date' and `a_future_symbol'
		require
			non_void_args: a_currency /= Void and a_maturity_date /= Void and a_future_symbol /= Void
		do
			Result := a_currency.name + "FUTR" + a_maturity_date.ccyymmdd.out + a_future_symbol
		end

	currency_forward_brimsym (a_currency: RP_CURRENCY; a_settlement_date: ROSE_DATE): STRING is
			-- Create currency forward brimsym from `a_currency' and `a_settlement_date'
		require
			non_void_args: a_currency /= Void and a_settlement_date /= Void
		do
			Result := a_currency.name + "FWD_" + a_settlement_date.ccyymmdd.out + "..."
		end

end
