indexing
	description: "Auto-completing combo-box with facility to remember past values"
	author: "Broos Nemanic"
	last_editor: "$Author: it $"
	date: "$Date: 2008/12/01 $"
	revision: "$Revision: #2 $"
	archive: "$File: //rose/source/kernel/infrastructure/framework/gui/rp_remembered_value_combo_box.e $"

class
	RP_REMEMBERED_VALUE_COMBO_BOX

inherit
	RP_SELECTION_COMBO_BOX

create
	make,
	make_dynamic,
	make_named


feature -- Initialization
	make_named (a_name : STRING) is
			-- Create with a name used to store values
		do
			make
			name := a_name
		end

feature -- Access
	name : STRING
			-- Name `values' is stored against
			-- Note `name' needs to be unique within an application

feature -- Basic operations

	remember_and_load_values is
			-- Remember current value and load all remembered values
		do
			remember_value
			load_remembered_values
		end


	load_remembered_values is
			-- Retrieve remembered values
		local
			l_values : ROSE_LINEAR_ARRAY [STRING]
		do
			l_values := remembered_values
			if l_values /= Void then
				set_strings (l_values)
			end
		end

	remember_value is
			-- Add current text value if any to remembered values
		local
			l_values : ROSE_LINEAR_ARRAY [STRING]
			i : INTEGER
			l_count : INTEGER
			l_is_void : BOOLEAN
			l_values_string : STRING
			l_text, l_value : STRING
		do
			l_text := text
			if l_text /= Void and then not l_text.is_empty then
				l_values := remembered_values
				l_values_string := ""
				if l_values /= Void then
					from i := 1 until i > l_values.count or else l_count = (k_max_remembered_values - 1) or else l_is_void loop
						l_value := l_values [i]
						l_is_void := (l_value = Void)
						if not l_is_void and then not l_value.is_equal (l_text) then
							l_values_string.append (l_values [i] + ",")
							l_count := l_count + 1
						end
						i := i + 1
					end
				end
				l_values_string.append (l_text)
				global.user.set_remembered_value (remembered_values_key, l_values_string)
			end
		end

feature {NONE} -- Implementation
	remembered_values_key : STRING is
			-- Key against which remembered values_string is stored
		local
			l_name : STRING
		do
			l_name := name.twin
			l_name.replace_substring_all (" ", "_")
			Result := k_remembered_values_string + "/" + l_name
		end

	remembered_values : ROSE_LINEAR_ARRAY [STRING] is
			-- Retrieve remembered values
		local
			l_values_string : STRING
		do
			l_values_string := global.user.remembered_value (remembered_values_key)
			if l_values_string /= Void then
				Result := gui.formatter.split_array (l_values_string, ",", Void, False)
			end
		end

feature {NONE} -- Constants
	k_remembered_values_string : STRING is "rembered_values"

	k_max_remembered_values : INTEGER is 5
end
