indexing
	description: "A role a person can play"
	author: "mk"
	date: "$Date: 2009/09/24 $"
	revision: "$Revision: #9 $"

class
	RP_ROLE

inherit
	DEBUG_OUTPUT

create
	make,
	make_with_groups

feature -- Initialization

	make (a_group: RP_GROUP) is
			-- Create `Current' with group `a_group'.
		do
			create groups.make_empty
			groups.extend (a_group)
		end

	make_with_groups (a_groups: ROSE_LINEAR_ARRAY [RP_GROUP]) is
			-- Create `Current' and assign `a_groups' to groups.
		require
			a_groups_not_void: a_groups /= Void
		do
			create groups.make_empty
			groups := a_groups
		end

feature -- Access

	people: ROSE_LINEAR_ARRAY [RP_PERSON] is
			-- All people contained in groups comprising `Current'.
		do
			create Result.make_empty
			from
				groups.start
			until
				groups.off
			loop
				Result.append_linear (groups.current_item.members)
				groups.forth
			end
		ensure
			result_not_void: Result /= Void
		end

	groups: ROSE_LINEAR_ARRAY [RP_GROUP]
			-- All groups comprising `Current'.

	named_members: ROSE_LINEAR_ARRAY [STRING] is
			-- All named people or groups that are members of `Current' recursively.
		local
			l_people: like people
			l_groups: like groups
		do
			create Result.make_empty
			l_people := people
			from
				l_people.start
			until
				l_people.off
			loop
				Result.extend (l_people.current_item.name)
				l_people.forth
			end
			l_groups := groups
			from
				l_groups.start
			until
				l_groups.off
			loop
				Result.extend (l_groups.current_item.name)
				l_groups.forth
			end
		end

	email_addresses: ROSE_LINEAR_ARRAY [STRING] is
			-- Email addresses of all groups that comprise `Current'.
		local
			i: INTEGER
		do
			create Result.make_empty
			from
				i := groups.lower
			until
				i > groups.upper
			loop
				Result.append_linear (groups.item (i).email_addresses)
				i := i + 1
			end
		ensure
			result_not_void: Result /= Void
		end

feature {RP_TASK_GROUP_COMPARISON} -- Implementation

	debug_output: STRING is
			-- String that should be displayed in debugger to represent `Current'.
		local
			i: INTEGER
		do
			Result := "["
			from
				i := groups.lower
			until
				i > groups.upper
			loop
				Result.append (groups.item (i).name)
				if i < groups.upper then
					Result.append (once ", ")
				end
				i := i + 1
			end
			Result.append (once "]")
		end

end -- class RP_ROLE
