indexing

	description:

		"Calendar based on rose_business_date"

	date: "$Date: 2008/12/01 $"
	revision: "$Revision: #5 $"
	pattern: "Facade"

class RP_ROSE_BUSINESS_DATE_CALENDAR

inherit

	RP_CALENDAR

create

	make

feature {NONE} -- Initialization

	make (a_business_date_bitmap: like business_date_bitmap) is
			-- Initialize `Current' from `a_business_date_bitmap'
		require
			business_date_bitmap_exists: a_business_date_bitmap /= Void
		do
			business_date_bitmap := a_business_date_bitmap
		end

feature -- Query

	is_work_day (a_date: ROSE_DATE): BOOLEAN is
			-- Is `a_date' a work day?
		do
			Result := business_date_bitmap.has (a_date)
		end

	ith_work_day_after (a_date: ROSE_DATE; a_days: INTEGER): ROSE_DATE is
			-- i-th work day after `a_date'
			-- this feature will return Void if no such date
			-- is entered into the data (rose_date_bitmap).
		local
			l_date: ROSE_DATE
		do
			l_date := business_date_bitmap.ith_date_after (a_date, a_days)
			if l_date /= Void then
				Result := next_work_day (l_date)
			end
		end

	ith_work_day_before (a_date: ROSE_DATE; a_number_of_days: INTEGER): ROSE_DATE is
			-- Work day which is `a_number_of_days' before `a_date'
		do
			Result := last_work_day (business_date_bitmap.ith_date_before (a_date, a_number_of_days))
		end

	last_work_day (a_date: ROSE_DATE): ROSE_DATE is
		do
			if business_date_bitmap.has (a_date) then
				create Result.make_ccyymmdd (a_date.ccyymmdd)
			else
				Result := business_date_bitmap.ith_date_before (a_date, 1)
			end
		end

	next_work_day (a_date: ROSE_DATE): ROSE_DATE is
		do
			if business_date_bitmap.has (a_date) then
				create Result.make_ccyymmdd (a_date.ccyymmdd)
			else
				Result := business_date_bitmap.ith_date_after (a_date, 1)
			end
		end

feature {NONE} -- Implementation

	business_date_bitmap: ROSE_DATE_SET
			-- Underlying work days bitmap

end
