indexing
	description: "[
Rm412a> type rounding_distributions.parm
EXDATE____Z__BRIMSYM__________1/0_____________________________________
19990903  C  CAE000232102...    0    ! 2 for round down 1 for round up
20021011  O  MYE000085101...    2    ! Round down
20021203  O  SGE000042101...    2    ! Round down
20030108  O  HKE000734101...    2    ! Round down
20030106  O  HKE000978101...    2    ! Round down
20030325  O  SGE000524101...    2    ! Round down
20030702  O  AUE000895101...    2    ! Round down
20031201  O  NZE000076101...    2    ! Round down
20040719  E  GBE000021101...    2    ! Round down
20050606  E  GBE000198101...    2    ! Round down
20050701  O  NZE000107102...    2    ! Round down
20050711  E  GBE001821101...    1    ! Round up
20051117  O  HKE000349101...    2    ! Round down
20070313  O  AUE000132101...    2    ! Round down
20080801  O  TWE000268091...    2    ! Round down
20090217  O  KRE001459091NILSEO 2    ! Round down
20090724  O  KRE000737101NILSEO 2    ! Round down

	]"
	author: "Sami Ben Grine"
	last_editor: "$Author: sm $"
	date: "$Date: 2009/09/23 $"
	revision: "$Revision: #1 $"
	archive: "$File: //rose/source/kernel/capital_actions/elections/rp_rounding_distributions_parm.e $"

class
	RP_ROUNDING_DISTRIBUTIONS_PARM

inherit

	DOUBLE_MATH
		export
			{NONE} all
		end

	EXCEPTIONS
		export
			{NONE} all
		end

	ANY

create
	make_up, make_down, make_from_code

feature {NONE} -- Initialization

	make_from_code (a_ex_date: like ex_date; a_zone_code: like zone_code; a_brimsym: like brimsym; a_code: INTEGER) is
			-- Make from 'a_code'
		require
			valid_date: a_ex_date /= Void
			valid_zone_code: (<<'C','E','O','J','U','S'>>).has (a_zone_code) -- FIXME use shared object from
			valid_brimsym: (a_brimsym /= Void) and then (a_brimsym.count = 18)
			valid_code: (a_code = up_code) or (a_code = down_code)
		do
			if a_code = up_code then
				make_up (a_ex_date, a_zone_code, a_brimsym)
			elseif a_code = down_code then
				make_down (a_ex_date, a_zone_code, a_brimsym)
			else
				raise ("invalid rounding code")
			end
		ensure
			ex_date_set: ex_date = a_ex_date
			zone_code_set: zone_code = a_zone_code
			brimsym_set: brimsym = a_brimsym
			up_set: (a_code = up_code) implies is_up
			down_set: (a_code = down_code) implies is_down
		end

	make_up (a_ex_date: like ex_date; a_zone_code: like zone_code; a_brimsym: like brimsym) is
			-- Make as UP rounding
		require
			valid_date: a_ex_date /= Void
			valid_zone_code: (<<'C','E','O','J','U','S'>>).has (a_zone_code) -- FIXME use shared object from
			valid_brimsym: (a_brimsym /= Void) and then (a_brimsym.count = 18)
		do
			ex_date := a_ex_date
			zone_code := a_zone_code
			brimsym := a_brimsym
			is_up := True
		ensure
			ex_date_set: ex_date = a_ex_date
			zone_code_set: zone_code = a_zone_code
			brimsym_set: brimsym = a_brimsym
			is_up: is_up
		end

	make_down (a_ex_date: like ex_date; a_zone_code: like zone_code; a_brimsym: like brimsym) is
			-- Make as DOWN rounding
		require
			valid_date: a_ex_date /= Void
			valid_zone_code: (<<'C','E','O','J','U','S'>>).has (a_zone_code) -- FIXME use shared object from
			valid_brimsym: (a_brimsym /= Void) and then (a_brimsym.count = 18)
		do
			ex_date := a_ex_date
			zone_code := a_zone_code
			brimsym := a_brimsym
			is_down := True
		ensure
			ex_date_set: ex_date = a_ex_date
			zone_code_set: zone_code = a_zone_code
			brimsym_set: brimsym = a_brimsym
			is_down: is_down
		end

feature -- Access

	ex_date: ROSE_DATE
		-- Ex date

	zone_code: CHARACTER
		-- Zone id

	brimsym: STRING
		-- Brimsym

feature -- Status Report

	is_up: BOOLEAN
			-- Is an up rounding?

	is_down: BOOLEAN
		-- Is a down rounding?

feature -- Basic Operations

	round (a_decimal: RP_DECIMAL): RP_DECIMAL is
			-- Rounded value for 'a_decimal'
		require
			decimal_not_void: a_decimal /= Void
		do
				-- FIXME: implement this in RP_DECIMAL...
			if is_up then
				Result := a_decimal.ceiling
			elseif is_down then
				Result := a_decimal.floor
			end
		ensure
			not_void: (Result /= Void)
			up: is_up implies (Result >= a_decimal)
			down: is_down implies (Result <= a_decimal)
		end

feature -- Constants

	up_code: INTEGER is 1
		-- Round up

	down_code: INTEGER is 2
		-- Round down

invariant

	exdate_not_Void: ex_date /= Void
	instrument_not_void: brimsym /= Void
	up_or_down: is_up or else is_down

end
