indexing
	description:
		"Equity class instrument"

	author: "Mark Howard"
	last_editor: "$Author: sm $"
	date: "$Date: 2011/09/13 $"
	revision: "$Revision: #28 $"
	path: "$File: //rose/source/kernel/expert_system/rp_security.e $"

deferred class RP_SECURITY

inherit

	ROSE_UNITS_ENVIRONMENT
		undefine
			is_equal
		end

	RP_INSTRUMENT_WITH_LISTING
		redefine
			uid
		end

feature {NONE} -- Initialization

	make (a_reference_system: like reference_system; a_brimsym: like brimsym; a_listing: LISTING) is
			-- Initialize `Current'
		require
			reference_system_not_void: a_reference_system /= Void
			brimsym_not_void: a_brimsym /= Void
			listing_not_void: a_listing /= Void
		do
			reference_system := a_reference_system
			brimsym := a_brimsym
			listing := a_listing
			security := a_listing.security
			rose_units := share_units (a_listing)
			xrf_code := a_listing.xrf_code_most_recent_key (Void).to_integer
			zone_mnemonic := a_listing.zone.mnemonic
			xrf_plus_zone := xrf_code*100 + a_listing.zone.mnemonic.code
			name := "[" + xrf_code.out + zone_mnemonic.out + "](" + brimsym + ")"
			uid := a_listing.uid
			entity_id := a_listing.handle
			initialize_local_brimsyms (a_listing)
		ensure
			reference_system_set: reference_system = a_reference_system
			brimsym_set: brimsym.is_equal (a_brimsym)
		end

	make_from_security (a_reference_system: like reference_system; a_security: SECURITY) is
			-- make from security
		require
			security_not_void: a_security /= Void
		local
			l_listing: LISTING
		do
			l_listing := a_security.listing (1)
			make (a_reference_system, l_listing.brim_code (Void), l_listing)
		end

feature -- Query

	name: STRING
			-- Name

	brimsym: STRING
			-- Brimsym identifier

	exchange: EXCHANGE is
			-- (Primary) security exchange
		do
			Result := listing.exchange
		end

	currency (a_date: ROSE_DATE): RP_CURRENCY is
			-- Currency
		require
			date_not_void: a_date /= Void
		do
			Result := reference_system.currency_from_iso_code (listing.currency (a_date).iso)
		end

	xrf_code: INTEGER
			-- XRF identifier

	xrf_plus_zone: INTEGER
		-- xrf plus zone

	zone_mnemonic: CHARACTER
			-- Zone

	uid: INTEGER_64
			-- Unique identifier

	entity_id: INTEGER
			-- Entity Id

	company: RP_COMPANY is
			-- Company
		do
			create Result.make (reference_system, security.company)
		ensure
			result_not_void: Result /= Void
		end

feature {NONE} -- Implementation

	reference_system: RP_REFERENCE_SYSTEM
		-- Reference system

	security: SECURITY
		-- Underlying security

feature -- Temp

	listing: LISTING
		-- Underlying listing

invariant
	security_not_void: security /= Void

end
