indexing
	description: "Combo box type field which shows the choices extending what you have typed as you type"
	documentation: ""
	keywords: ""
	author: "Julian Rogers"
	last_editor: "$Author: er $"
	date: "$Date: 2013/04/23 $"
	revision: "$Revision: #6 $"
	path: "$File: //rose/source/kernel/infrastructure/framework/gui/rp_selection_combo_box.e $"

class
	RP_SELECTION_COMBO_BOX

inherit
	EV_HORIZONTAL_BOX
		redefine
			initialize,
			key_release_actions,
			key_press_actions,
			is_in_default_state
		end

	ROSE_SHARED_GLOBAL_GUI
		undefine
			default_create, copy, is_equal
		end

create
	make,
	make_dynamic

feature {NONE} -- Initialization

	make is
			-- vanilla make provides an empty list.
		do
			create selection_text_field.make
			default_create
		end

	make_dynamic(a_string_agent : like string_agent) is
			--allows the user to specify the list dynamically and what to do on removal.
		do
			create selection_text_field.make_dynamic (a_string_agent)
			default_create
		end

	initialize is
			-- Initialize `Current'.
		do
			Precursor {EV_HORIZONTAL_BOX}
			extend (selection_text_field)
			create drop_down_button
			extend (drop_down_button)
			disable_item_expand (drop_down_button)
			drop_down_button.set_minimum_size (16, 16)
			drop_down_button.disable_tabable_to
			drop_down_button.select_actions.extend (agent toggle_selection_list_visibility)
			drop_down_button.set_pixmap (drop_down_pixmap)
			drop_down_button.focus_out_actions.extend (agent selection_text_field.hide_popup_if_necessary)
			selection_text_field.set_extended_list_width (16)
		end

feature -- Access

	text: STRING is
			-- String displayed in `Current'
		do
			Result := selection_text_field.text
		end

	selection_text_field: RP_SELECTION_TEXT_FIELD
		-- Selection text field widget with which `Current' is implemented.

	string_agent :  FUNCTION [ANY, TUPLE, ARRAY [STRING]] is
			--user supplied agent to dynamically provide choices
		do
			Result := selection_text_field.string_agent
		end

	is_case_sensitive: BOOLEAN is
			-- Is search case sensitive?
		do
			Result := selection_text_field.is_case_sensitive
		end

	is_string_agent_called_once_only: BOOLEAN is
			-- Is `string_agent' only called once when first required?
			-- If `False', then it is called each time a key is pressed.
		do
			Result := selection_text_field.is_string_agent_called_once_only
		end

	is_list_shown: BOOLEAN is
			-- Is the selection list shown?
		do
			Result := selection_text_field.is_list_shown
		end

	key_release_actions : EV_KEY_ACTION_SEQUENCE is
			-- Redirect to key_release_actions of `selection_text_field'
		do
			Result := selection_text_field.key_release_actions
		end

	key_press_actions : EV_KEY_ACTION_SEQUENCE is
			-- Redirect to `key_press_actions' of `selection_text_field'
		do
			Result := selection_text_field.key_press_actions
		end

feature -- Basic operations

	set_text (a_text: STRING) is
			-- Assign `a_text' to `text'.
		do
			if a_text /= Void then
				selection_text_field.set_text (a_text)
			else
				selection_text_field.remove_text
			end
		end

	set_strings (a_strings : ARRAY[STRING]) is
			--create the strings attribute from the input ... the strings attribute is then sorted
		require
			not_void : a_strings /= void
		do
			selection_text_field.set_strings (a_strings)
		ensure
			strings_sorted : is_sorted (selection_text_field.strings)
		end

	set_string_agent (a_action : like string_agent)
			--set the agent which provides the contents of the list
		do
			selection_text_field.set_string_agent (a_action)
		ensure
			value_set : string_agent = a_action
		end

	set_case_sensitivity (a_sensitivity: BOOLEAN) is
			-- Assign `a_sensitivity' to `is_case_sensitive'.
		do
			selection_text_field.set_case_sensitivity (a_sensitivity)
		ensure
			is_case_sensitive_set: is_case_sensitive = a_sensitivity
		end

	set_is_string_agent_called_once_only (a_once_only: BOOLEAN) is
			-- Assign `a_once_only' to `is_string_agent_called_once_only'.
		do
			selection_text_field.set_is_string_agent_called_once_only (a_once_only)
		ensure
			is_string_agent_called_once_only_set: is_string_agent_called_once_only = a_once_only
		end

	is_sorted (a_strings : ROSE_LINEAR_ARRAY[STRING]) : BOOLEAN is
			--is a_strings sorted?
		do
			Result := selection_text_field.is_sorted (a_strings)
		end

	text_is_contained_list_item: BOOLEAN is
			-- Does `text' of `Current' match one of the list items contained?
		do
			Result := selection_text_field.text_is_contained_list_item
		end

feature -- Event handling

	select_actions: EV_NOTIFY_ACTION_SEQUENCE is
			-- Actions fired when an item is selected (`text' of `Current' matches an item in the drop down list)
		do
			Result := selection_text_field.select_actions
		ensure
			result_not_void: Result /= Void
		end

	deselect_actions: EV_NOTIFY_ACTION_SEQUENCE is
			-- Actions fired when an item is deselected (`text' of `Current' matches an item in the drop down list)
		do
			Result := selection_text_field.deselect_actions
		ensure
			result_not_void: Result /= Void
		end

feature {NONE} -- Implementation

	toggle_selection_list_visibility is
			-- Toggle visibility of the selection list.
		do
			if selection_text_field.is_list_shown then
				selection_text_field.hide_list
			else
				selection_text_field.show_list
				selection_text_field.set_focus_to_list_and_select_first_item
			end
		end

	drop_down_button: EV_BUTTON
		-- Button used to drop down the list.

	drop_down_pixmap: EV_PIXMAP is
			-- Pixmap for the Inactive Sort header state
		once
			Result := gui.factory.new_arrow_pixmap ('v', 10, 10, gui.rep.colors.default_background_color,  gui.rep.colors.black)
		ensure
			result_not_void: Result /= Void
		end

	is_in_default_state: BOOLEAN is
			-- Is `Current' in its default state?
		do
			Result := True
		end

invariant
	selection_text_field_not_void: selection_text_field /= Void

end
