indexing
	description: "EV_TEXT_FIELD which shows the choices extending what you have typed as you type"
	author: "Mark Howard"
	date: "$Date: 2009/04/15 $"
	revision: "$Revision: #14 $"
	archive: "$File: //rose/source/kernel/infrastructure/framework/gui/rp_selection_text_field.e $"

class
	RP_SELECTION_TEXT_FIELD
inherit
	EV_TEXT_FIELD
create
	make,
	make_dynamic
feature {NONE} --Initialization
	make  is
		--vanilla make provides an empty list
		do
			default_create
			create list
			initialize_list
			create strings.make_empty
			focus_out_actions.extend(agent hide_popup_if_necessary)
			key_release_actions.extend(agent handle_key_release)
			key_press_actions.extend(agent handle_key_press)
			key_press_actions.force_extend (agent key_release_actions.resume)
			is_string_agent_called_once_only := True
		ensure
			is_string_agent_called_once_only: is_string_agent_called_once_only
		end

	make_dynamic(a_string_agent : like string_agent) is
		--allows the user to specify the list dynamically and what to do on removal
		do
			set_string_agent(a_string_agent)
			make
		ensure
			string_agent_set : string_agent = a_string_agent
		end

	initialize_list is
			-- Initialize `list' for use in `Current'.
		do
			list.enable_partial_dynamic_content
			list.set_dynamic_content_function (agent get_list_item)
			list.set_column_count_to (1)
			list.hide_header

			list.key_press_actions.wipe_out
			list.key_press_actions.extend(agent handle_key_release_on_list)
			list.pointer_double_press_actions.wipe_out
			list.pointer_double_press_actions.extend (agent double_click_action)
			list.enable_single_row_selection
		end

feature --Setters
	set_popup_entered(a_value : BOOLEAN) is
		--set whether or not in popup
		do
			in_popup := a_value
		ensure
			value_set : in_popup = a_value
		end
	set_strings (a_strings : ARRAY[STRING]) is
			--create the strings attribute from the input.
			-- the strings attribute is then sorted if not already.
		require
			not_void : a_strings /= void
		do
			create strings.make_from_array(a_strings)
			if not is_sorted (strings) then
				strings.sort_by (agent strings_less_than)
			end
			list.wipe_out
			list.set_column_count_to (1)
			refresh_list
		ensure
			strings_sorted : is_sorted(strings)
		end
	set_string_agent (a_action : like string_agent)
		--set the agent which provides the contents of the list
		do
			string_agent := a_action
		ensure
			value_set : string_agent = a_action
		end

	set_case_sensitivity (a_sensitivity: BOOLEAN) is
			-- Assign `a_sensitivity' to `is_case_sensitive'.
		do
			is_case_sensitive := a_sensitivity
		ensure
			is_case_sensitive_set: is_case_sensitive = a_sensitivity
		end

	set_is_string_agent_called_once_only (a_once_only: BOOLEAN) is
			-- Assign `a_once_only' to `is_string_agent_called_once_only'.
		do
			is_string_agent_called_once_only := a_once_only
		ensure
			is_string_agent_called_once_only_set: is_string_agent_called_once_only = a_once_only
		end

feature --Actions

	show_list is
			-- Ensure drop down list is displayed.
		do
			show_selection
		ensure
			is_list_shown: is_list_shown
		end

	hide_list is
			-- Ensure drop down list is hidden.
		do
			if popup_window /= Void then
				popup_window.hide
			end
		ensure
			list_not_shown: not is_list_shown
		end

	move_action(a_x,a_y,a_width,a_height : INTEGER) is
			--this action allows the popup window to track the text field
		do
			if popup_window /= void then
				popup_window.set_x_position(popup_window.x_position + a_x - parent_x)
				popup_window.set_y_position(popup_window.y_position + a_y - parent_y)
				parent_x := a_x
				parent_y := a_y
			end
		end

	handle_key_release_on_list(a_key : EV_KEY) is
			--handle selects on the list
		require
			not_void : a_key /= void
		local
			l_selected_items: ARRAYED_LIST [EV_GRID_ITEM]
			l_viewable_indexes: ARRAYED_LIST [INTEGER]
		do
			inspect a_key.code
			when {EV_KEY_CONSTANTS}.key_enter then
				if {l_label_item: EV_GRID_LABEL_ITEM} list.selected_items.first then
					set_text(l_label_item.text)
					set_focus
					hide_popup
				else
					check
						item_was_label: False
					end
				end
			when {EV_KEY_CONSTANTS}.key_escape then
				set_focus
				hide_popup
			when {EV_KEY_CONSTANTS}.key_up then
				l_selected_items := list.selected_items
				if not l_selected_items.is_empty and then row_index_is_first_displayed (l_selected_items.first.row.index) then
					key_release_actions.block
					set_focus
				end
			when {EV_KEY_CONSTANTS}.key_down then
				l_selected_items := list.selected_items
				l_viewable_indexes := list.viewable_row_indexes
				if not l_selected_items.is_empty and then row_index_is_last_displayed (l_selected_items.first.row.index) then
					key_release_actions.block
					set_focus
				end
			else
			end
		end

	select_actions: EV_NOTIFY_ACTION_SEQUENCE is
			-- Actions fired when an item is selected (`text' of `Current' matches an item in the drop down list)
		do
			if select_actions_internal = Void then
				create select_actions_internal
			end
			Result := select_actions_internal
		ensure
			result_not_void: Result /= Void
		end

	deselect_actions: EV_NOTIFY_ACTION_SEQUENCE is
			-- Actions fired when an item is deselected (`text' of `Current' matches an item in the drop down list)
		do
			if deselect_actions_internal = Void then
				create deselect_actions_internal
			end
			Result := deselect_actions_internal
		ensure
			result_not_void: Result /= Void
		end

feature --Status

	is_case_sensitive: BOOLEAN
		-- Is search case sensitive?

	is_string_agent_called_once_only: BOOLEAN
		-- Is `string_agent' only called once when first required?
		-- If `False', then it is called each time a key is pressed.

	is_list_shown: BOOLEAN is
			-- Is drop down list currently displayed?
		do
			Result := popup_window /= Void and then popup_window.is_show_requested
		end

	text_is_contained_list_item: BOOLEAN is
			-- Does `text' of current match one of the list items contained?
		do
			strings.set_comparison (agent strings_less_than)
			strings.binary_search (text)
			Result := strings.found
		end

feature --Agents
	string_agent :  FUNCTION [ANY, TUPLE, ARRAY [STRING]]
		--user supplied agent to dynamically provide choices
feature {NONE} -- Actions
	refresh_list is
			-- Rebuild list
		do
			if strings /= Void then
				list.set_row_count_to (strings.count)
			end
		end

	show_selection is
		--display the list of choices extending the current text
		require
			text_not_void : text /= void
		local
			i : INTEGER
			l_text: STRING
			l_strings_count: INTEGER
		do
			if list.row_count = 0 or else not is_string_agent_called_once_only then
				list.wipe_out
				list.set_column_count_to (1)
				refresh_list
				if string_agent /= void then
					set_strings (string_agent.item([]))
					l_strings_count := strings.count
					list.set_row_count_to (l_strings_count)
				end
			end
			if strings /= void then
				refresh_list
				l_strings_count := strings.count
				strings.set_comparison (agent strings_less_than)
				strings.binary_search (text)
				l_text := text
				if not is_case_sensitive then
					l_text := l_text.as_lower
				end
					-- Firstly, hide all rows before those that are to be displayedst
				from
					i := 1
				until
					i = strings.found_index
				loop
					list.row (i).hide
					i := i + 1
				end
					-- Now show all rows to be displayed
				from
				until
					i > l_strings_count or else (l_text.count > 0 and then not string_starts_with (strings [i], l_text))
				loop
					list.row (i).show
					i := i + 1
				end
					-- Now hide all rows after those to be displayed.
				from
				until
					i > l_strings_count
				loop
					list.row (i).hide
					i := i + 1
				end
				if list.column_count > 0 then
					popup
				end
			end
		end


	popup is
		--manage the popup_window
		local
			l_border_box: EV_VERTICAL_BOX
		do
			if parent_window /= void then
				if parent_window_move_actions_agent = Void then
					parent_window_move_actions_agent := agent move_action
					parent_window.move_actions.extend(parent_window_move_actions_agent)
				end
				parent_x := parent_window.x_position
				parent_y := parent_window.y_position
			end
			if popup_window = Void then
				create popup_window
				popup_window.disconnect_from_window_manager
				create l_border_box
				popup_window.extend(l_border_box)
				l_border_box.extend (list)
				l_border_box.set_background_color ((create {EV_STOCK_COLORS}).black)
				l_border_box.set_border_width (1)
				popup_window.pointer_enter_actions.extend(agent set_popup_entered(true))
				popup_window.pointer_leave_actions.extend(agent set_popup_entered(false))
				popup_window.focus_in_actions.extend(agent set_popup_entered(true))
				popup_window.focus_out_actions.extend(agent set_popup_entered(false))
			end
			popup_window.set_width(width + extended_list_width)
			popup_window.set_height ((list.row_count * 15).min (200))
			popup_window.set_position (screen_x, screen_y + height)
			popup_window.show_relative_to_window (parent_window)
		end

	double_click_action(a_x: INTEGER; a_y: INTEGER; a_button: INTEGER; a_x_tilt: DOUBLE; a_y_tilt: DOUBLE; a_pressure: DOUBLE; a_screen_x: INTEGER; a_screen_y: INTEGER) is
		--agent for double clicking in the list ... sets the text to the selected value
		require
			list_not_void : list /= void
		do
			if {l_label_item: EV_GRID_LABEL_ITEM} list.selected_items.first then
				set_text(l_label_item.text)
				set_focus
				hide_popup
			else
				check
					item_was_label: False
				end
			end
		end
	handle_key_release(a_key : EV_KEY) is
		--key press handler
		require
			not_void : a_key /= void
			list_not_void : list /= void
		do
			inspect a_key.code
			when {EV_KEY_CONSTANTS}.key_escape then
				hide_popup
			when {EV_KEY_CONSTANTS}.key_enter then
				hide_popup
			when {EV_KEY_CONSTANTS}.key_tab then
			else
				show_selection
				set_focus
			end
			fire_select_actions_if_necessary
		end

	handle_key_press (a_key : EV_KEY) is
		--key press handler
		require
			not_void : a_key /= void
			list_not_void : list /= void
		local
			l_row: EV_GRID_ROW
			l_viewable_indexes: ARRAYED_LIST [INTEGER_32]
		do
			inspect a_key.code
			when {EV_KEY_CONSTANTS}.key_down then
				show_selection
				set_focus_to_list_and_select_first_item
			when {EV_KEY_CONSTANTS}.key_up then
				show_selection
				list.set_focus
				l_viewable_indexes := list.viewable_row_indexes
				if not l_viewable_indexes.is_empty then
					l_row := list.row (list.viewable_row_indexes.last)
					list.remove_selection
					l_row.enable_select
					l_row.ensure_visible
				end
			else
			end
		end

	hide_popup is
		--hide the popup
		do
			if popup_window /= void then
				popup_window.hide
				if parent_window /= Void and then parent_window_move_actions_agent /= Void then
					parent_window.move_actions.prune_all (parent_window_move_actions_agent)
					parent_window_move_actions_agent := Void
				end
				fire_select_actions_if_necessary
			end
		end

	row_index_is_first_displayed (a_index: INTEGER): BOOLEAN is
			-- Is `a_index' the index of the first row displayed in `Current'?
		require
			valid_a_index: a_index >= 1 and a_index <= list.row_count
		local
			l_viewable_indexes: ARRAYED_LIST [INTEGER]
		do
			if list.visible_row_count = list.row_count then
					-- We cannot always rely on `l_viewable_indexes' as it returns an empty list when no rows are hidden.
					-- The above check is to see that no rows are hidden.
				Result := a_index = 1
			else
				l_viewable_indexes := list.viewable_row_indexes
				if not l_viewable_indexes.is_empty then
					Result := a_index = list.viewable_row_indexes.first
				end
			end
		end

	row_index_is_last_displayed (a_index: INTEGER): BOOLEAN is
			-- Is `a_index' the index of the last row displayed in `Current'?
		require
			valid_a_index: a_index >= 1 and a_index <= list.row_count
		local
			l_viewable_indexes: ARRAYED_LIST [INTEGER]
		do
			if list.visible_row_count = list.row_count then
					-- We cannot always rely on `l_viewable_indexes' as it returns an empty list when no rows are hidden.
					-- The above check is to see that no rows are hidden.
				Result := a_index = list.row_count
			else
				l_viewable_indexes := list.viewable_row_indexes
				if not l_viewable_indexes.is_empty then
					Result := a_index = list.viewable_row_indexes.last
				end
			end
		end

feature -- Assertion checking

	is_sorted (a_strings : ROSE_LINEAR_ARRAY[STRING]) : BOOLEAN is
			--is a_strings sorted?
		require
			a_strings_not_void: a_strings /= Void
		local
			i : INTEGER
		do
			Result := true
			if a_strings /= void and then a_strings.count > 1 then
				from i := 1 until i >= a_strings.count or else not Result loop
					Result := (strings_less_than (a_strings[i], a_strings[i+1])) or (strings_equal (a_strings[i], a_strings [i + 1]))
					i := i + 1
				end
			end
		end

feature {NONE} --Implementation
	popup_window : EV_POPUP_WINDOW
		--popup window for choices
	parent_x : INTEGER
		--keep track of where the window was when popped up
	parent_y : INTEGER
		--keep track of where the window was when popped up
	parent_window : EV_WINDOW is
		--first ancestor of type EV_WINDOW
		local
			l_widget : EV_WIDGET
		do
			from l_widget := Current until l_widget = void or else Result /= void loop
				Result ?= l_widget
				if Result = void then
					l_widget := l_widget.parent
				end
			end
		end

	strings_less_than (string1, string2: STRING): BOOLEAN is
			-- Is `string1' considered less than `string2'?
		require
			strings_not_void: string1 /= Void and string2 /= Void
		do
			if is_case_sensitive then
				Result := string1 < string2
			else
				Result := string1.as_lower < string2.as_lower
			end
		end

	strings_equal (string1, string2: STRING): BOOLEAN is
			-- Is `string`' considered equal to `string2'?
		require
			strings_not_void: string1 /= Void and string2 /= Void
		do
			if is_case_sensitive then
				Result := string1.is_equal (string2)
			else
				Result := string1.as_lower.is_equal (string2.as_lower)
			end
		end

	string_starts_with (a_string, a_start_string: STRING): BOOLEAN is
			-- Does `a_string' start with `a_start_string'? Case insensitive if
			-- not `is_case_sensitive'.
		require
			strings_not_void: a_string /= Void and a_start_string /= Void
		do
			if is_case_sensitive then
				Result := a_string.substring (1, a_start_string.count).is_equal (a_start_string)
			else
				Result := a_string.as_lower.substring (1, a_start_string.count).is_equal (a_start_string)
			end
		end

	parent_window_move_actions_agent: PROCEDURE [ANY, TUPLE [INTEGER, INTEGER, INTEGER, INTEGER]]
		-- Agent currently contained in `parent_window' `move_actions' or Void if None.
		-- Used to enable us to only add an agent once.

	in_popup : BOOLEAN
		--are we in the popup_window?

	fire_select_actions_if_necessary is
			-- Fire `select_actions' and `deselect_actions' if required.
		do
			if not (last_selected_text ~ text) then
				if text_is_contained_list_item then
					select_actions.call (Void)
					last_selected_text := text
				elseif last_selected_text /= Void then
					deselect_actions.call (Void)
					last_selected_text := Void
				end
			end
		end

	last_selected_text: STRING
		-- Text of last selected item in `Current'.

	deselect_actions_internal: EV_NOTIFY_ACTION_SEQUENCE

	select_actions_internal: EV_NOTIFY_ACTION_SEQUENCE

feature {RP_SELECTION_COMBO_BOX} -- Implementation

	set_focus_to_list_and_select_first_item is
			-- Assign focus to `list' and select the first visible item.
		local
			l_viewable_indexes: ARRAYED_LIST [INTEGER_32]
			l_row: EV_GRID_ROW
		do
			if list.is_displayed then
				list.set_focus
				l_viewable_indexes := list.viewable_row_indexes
				if not l_viewable_indexes.is_empty then
					l_row := list.row (list.viewable_row_indexes.first)
					list.remove_selection
					l_row.enable_select
					l_row.ensure_visible
				end
			end
		end

	list : EV_GRID
		--list holding choices

	extended_list_width: INTEGER
			-- Extra width added to drop down list when displayed.

	set_extended_list_width (a_width: INTEGER) is
			-- Assign `a_width' to `extended_list_width'.
		do
			extended_list_width := a_width
		end

	hide_popup_if_necessary is
		--hide the popup
		do
			if popup_window /= void and then not in_popup and then not has_focus then
				hide_popup
			end
		end

	get_list_item (a_column_index, a_row_index: INTEGER): EV_GRID_LABEL_ITEM is
			-- Return the grid label item to be displayed at position `a_column_index', `a_row_index'.
		require
			valid_row_index: strings.valid_index (a_row_index)
		local
			l_column: EV_GRID_COLUMN
		do
			create Result.make_with_text (strings.item (a_row_index))
			l_column := list.column (1)
			l_column.set_width (l_column.width.max (Result.required_width))
		ensure
			result_not_void: Result /= Void
		end

	strings : ROSE_LINEAR_ARRAY[STRING]
		--container for strings appearing in the list of choices

end
