indexing
 
   description: "[
                  Rose Production view of a selection universe
                 ]"

	keywords: "Selection universe strategy"
	author: "$Author: it $ Colin Adams" 
	team: "RC_tools"
	reviewed_by: "N/A"
	reviewed_at: "N/A"
	quality: "not yet reviewed"
	url: ""
	path: "$File: //rose/source/kernel/expert_system/rp_selection_universe.e $" 
	date: "$Date: 2008/12/01 $"
	revision: "$Revision: #2 $"
	
class RP_SELECTION_UNIVERSE

create {RP_REFERENCE_SYSTEM}

	make


feature {NONE} -- Initialization

	make (a_ref: like reference_system; a_gcdb_universe: like gcdb_universe) is
			-- Initialize `Current'.
		require
			a_ref_not_void: a_ref /= Void
		do
			reference_system := a_ref
			gcdb_universe := a_gcdb_universe
			if gcdb_universe = Void then
				initialize_unknown_universe
			else
				initialize_from_gcdb_universe
			end
		end

	initialize_unknown_universe is
			-- Complete initialization for Void Object.
		require
			void_universe: gcdb_universe = Void
		do
			code := Code_unknown_constant
			is_unknown := True
		ensure
			void_object: is_unknown
			correct_code: code.same_string (Code_unknown_constant)
		end

	initialize_from_gcdb_universe is
			-- Complete initialization for known universe.
		require
			known_universe: gcdb_universe /= Void
		do
			code := gcdb_universe.code
		ensure
			not_void_object: not is_unknown
			correct_code: code.same_string (gcdb_universe.code)
		end

feature -- Access

	code: STRING
			-- Short identifier

	portfolios: ROSE_LINEAR_ARRAY [RP_PORTFOLIO] is
			-- All portfolios using `Current'
		local
			l_portfolios: ROSE_LINEAR_ARRAY[GCDB_PORTFOLIO]
		do
			if portfolios_cache = Void then
				create portfolios_cache.make_empty
				l_portfolios := gcdb_universe.portfolios_by_selection_universe
				l_portfolios.do_if (agent add_one_portfolio, agent is_gcdb_portfolio_not_void)
			end
			Result := portfolios_cache
		ensure
			result_not_void: Result /= Void
			no_void_entries: not Result.has (Void)
			-- Is the next line true? Consider, do we want to be able to list all portfolios without an assigned strategy?
			-- Are there any, and if so, can we implement it?
			-- If not, the RP_PORTFOLIO's call to selection_universe will be returning a void-object, which
			--  in turn will not list it as one of it's portfolios. This can be asserted, so is not a problem.
			void_object_has_empty_list: is_unknown implies Result.is_empty
			result_cached: portfolios_cache /= Void
		end

feature -- Status report

	is_unknown: BOOLEAN
			-- Does `Current' represent the "Unknown" selection universe? (Void Object pattern)

feature -- Status setting

	refresh is
			-- Ensure information is not stale.
		do
			portfolios_cache := Void
			-- The following should be done, if it were ever to exist:
			-- gcdb_universe.refresh
		ensure
			portfolios_not_cached: portfolios_cache = Void
			same_unknown_status: is_unknown = old is_unknown
		end

feature {NONE} -- Implementation

	Code_unknown_constant: STRING is "Unknown"
			-- Code for Void Object (should be common to all RP_* entities - talk to Ivan about it)

	reference_system: RP_REFERENCE_SYSTEM
			-- Reference system used to create `Current'

	gcdb_universe: GCDB_SELECTION_UNIVERSE
			-- Optional wrapped GCDB object

	portfolios_cache: like portfolios
			-- Cache for portfolios

	is_gcdb_portfolio_not_void (a_portfolio: GCDB_PORTFOLIO): BOOLEAN is
			-- Is `a_portfolio' non-Void?
		do
			Result := a_portfolio /= Void
		ensure
			definition: Result implies a_portfolio /= Void
			corollary: not Result implies a_portfolio = Void
		end

	add_one_portfolio (a_portfolio: GCDB_PORTFOLIO) is
			-- Add wrapper for `a_portfolio' to `portfolios_cache'.
		require
			a_portfolio_not_void: a_portfolio /= Void
			portfolios_cache_not_void: portfolios_cache /= Void
		local
			l_portfolio: RP_PORTFOLIO
		do
			-- Is this best, or should we call `{RP_PORTFOLIO}.make' (with consequent export change needed)?
			-- Caching considerations - consult with Ivan on this.
			l_portfolio := reference_system.portfolio_from_code (a_portfolio.code)
			portfolios_cache.extend (l_portfolio)
		ensure
			one_more_portfolio: portfolios_cache.count = old portfolios_cache.count + 1
			correct_portfolio_added: portfolios_cache.last.code.same_string (a_portfolio.code)
		end

invariant

	reference_system_not_void: reference_system /= Void
	no_gcdb_wrapper_for_unknown_universe: is_unknown implies gcdb_universe = Void
	gcdb_wrapper_if_known: not is_unknown implies gcdb_universe /= Void
	code_not_void: code /= Void
	code_not_empty: not code.is_empty

end
