note

	description:

		"Split contract"

	author: "Mark Howard"
	last_editor: "$Author: e2 $"
	date: "$Date: 2011/11/29 $"
	revision: "$Revision: #19 $"
	path: "$File: //rose/source/kernel/expert_system/rp_split.e $"

class RP_SPLIT

inherit

	RP_PORTFOLIO_CORPORATE_ACTION_CONTRACT

create

	make

feature {NONE} -- Initialization

	make (a_old_shares, a_new_shares: RP_QUANTITY; a_portfolio: like portfolio; a_identifier: like identifier;
		a_date: like agreement_time; a_settle_date: like settle_date;
		a_system: like generating_system; a_custody: RP_CUSTODY) is
			-- Create with old shares and new shares
		require
			a_old_shares_not_void: a_old_shares /= Void
			a_new_shares_not_void: a_new_shares /= Void
			a_portfolio_not_void: a_portfolio /= Void
			a_identifier_not_void: a_identifier /= Void
			a_agreement_time_not_void: a_date /= Void
			a_settle_date_not_void: a_settle_date /= Void
			a_system_not_void: a_system /= Void
			same_instrument: a_old_shares.instrument ~ a_new_shares.instrument
			same_sign: a_old_shares.sign = a_new_shares.sign
			had_shares: not a_old_shares.is_zero
			a_custody_not_void: a_custody /= Void
		do
				-- Store split factor which is used when we don't have a price for processing date
				-- We fall back to the old price and adjust it with this split factor
			factor := a_new_shares.amount / a_old_shares.amount
				-- Contract
			make_capital_action (a_system, a_date, a_identifier, a_portfolio, a_old_shares, a_settle_date)
				-- Flows
			add_flow (split_flow_type, portfolio.instrument_control (a_custody), unknown_instrument_control, a_date, a_settle_date, a_old_shares, a_date, 0, Void)
			add_flow (split_flow_type, unknown_instrument_control, portfolio.instrument_control (a_custody), a_date, a_settle_date, a_new_shares, a_date, 0, Void)
		ensure
			factor_set: factor /= Void
			correct_factor: factor ~ (a_new_shares.amount / a_old_shares.amount)
			generating_system_set: generating_system = a_system
			agreement_time_set: agreement_time = a_date
			identifier_set: identifier = a_identifier
			portfolio_set: portfolio = a_portfolio
			settle_date_set: settle_date ~ a_settle_date
			units_base_set: units_base = a_old_shares
		end

feature -- Access

	factor: RP_DECIMAL
			-- Number of shares after the split divided by number of shares before the split

	instrument: RP_INSTRUMENT is
			-- Instrument
		do
			Result := flows.first.instrument
		ensure
			instrument_not_void: Result /= Void
		end

feature -- Basic Operations

	process (a_processor: RP_CONTRACT_PROCESSOR) is
			-- <Precursor>
		do
			a_processor.process_split (Current)
		end

invariant

	factor_not_void: factor /= Void
	factor_positive: factor >= factor.zero
	at_least_one_flow: not flows.is_empty
end
