indexing

	description:

		"Portfolio dividend swept policy by date. Here (W) means any zone other than (U) zone"

	author: "Portfolio Accounting Tech (sm, it)"
	date: "$Date: 2010/01/25 $"
	sccid: "$Id: //rose/source/kernel/capital_actions/elections/rp_swept_accounts_aware.e#2 $"

class RP_SWEPT_ACCOUNTS_AWARE

inherit

	AXAROSENBERG_AWARE

feature -- Status Report

	w_is_non_special_dividend_swept_at (a_portfolio: RP_PORTFOLIO; a_trade_date, a_settle_date: ROSE_DATE): BOOLEAN is
			-- Are non-special dividends swept on 'a_date' for 'a_portfolio'?
		require
			portfolio_not_void: a_portfolio /= Void
			dates_not_void: a_trade_date /= Void and a_settle_date /= Void
		local
			l_swept_accounts: RP_SWEPT_ACCOUNTS_W
			l_setting: RP_INVEST_W_A_SWEPT_ACCOUNTS_PARM
		do
			l_swept_accounts := swept_accounts_for_w_zone (a_portfolio.zone)
			l_setting := l_swept_accounts.item (a_portfolio.code)
			if l_setting /= Void then
				if a_settle_date.ccyymmdd = 99991231 then
						-- for unknown pay-dates use ex-date to check for sweep
					Result := l_setting.non_special_brackets (a_trade_date)
				else
					Result := l_setting.non_special_brackets (a_settle_date)
				end
			end
		end

	w_is_roc_swept_at (a_portfolio: RP_PORTFOLIO; a_date: ROSE_DATE): BOOLEAN is
			-- Is Return Of Capital swept on 'a_date' for 'a_portfolio'?
		require
			portfolio_not_void: a_portfolio /= Void
			date_not_void: a_date /= Void
		local
			l_swept_accounts: RP_SWEPT_ACCOUNTS_W
			l_setting: RP_INVEST_W_A_SWEPT_ACCOUNTS_PARM
		do
			l_swept_accounts := swept_accounts_for_w_zone (a_portfolio.zone)
			l_setting := l_swept_accounts.item (a_portfolio.code)
			if l_setting /= Void then
				Result := l_setting.roc_sweep_brackets (a_date)
			end
		end

	u_is_swept_at (a_portfolio: RP_PORTFOLIO; a_trade_date, a_settle_date: ROSE_DATE): BOOLEAN is
			-- Are dividends swept for 'a_portfolio'?
		require
			portfolio_not_void: a_portfolio /= Void
			u_zone_portfolio: a_portfolio.zone.is_united_states
			dates_not_void: a_trade_date /= Void and a_settle_date /= Void
		local
			l_list: DS_ARRAYED_LIST [EDB_CLIENT_INTERVAL]
			l_interval, l_found, l_secondary, l_last_interval: EDB_CLIENT_INTERVAL
		do
			edb_client_intervals.search (a_portfolio)
			if edb_client_intervals.found then
					-- EVENT MANAGER CODE:
					--  while (link)
					--    {
					--      interval = link->data;
					--      if (wrap_yymmdd_to_ccyymmdd(interval->start_date,50) <= wrap_yymmdd_to_ccyymmdd(settle_date,50)
					--          && wrap_yymmdd_to_ccyymmdd(interval->end_date,50) >= wrap_yymmdd_to_ccyymmdd(settle_date,50))
					--        if (interval->date_type == 0)
					--         found = interval;
					--        else
					--         secondary = interval;
					--      if (wrap_yymmdd_to_ccyymmdd(interval->start_date,50) <= wrap_yymmdd_to_ccyymmdd(trade_date,50)
					--          && wrap_yymmdd_to_ccyymmdd(interval->end_date,50) >= wrap_yymmdd_to_ccyymmdd(trade_date,50))
					--        if (interval->date_type == 1)
					--         found = interval;
					--        else
					--         secondary = interval;
					--      if (wrap_yymmdd_to_ccyymmdd(last_interval->start_date,50) < wrap_yymmdd_to_ccyymmdd(interval->start_date,50)) last_interval = interval;
					--      link = link->next;
					--    }
					--  if (found) return found;
					--  if (secondary) return secondary;
					--  if (wrap_yymmdd_to_ccyymmdd(settle_date,50) > wrap_yymmdd_to_ccyymmdd(last_interval->end_date,50)
					--      && wrap_yymmdd_to_ccyymmdd(settle_date,50) < wrap_yymmdd_to_ccyymmdd(last_interval->end_date + 7,50)) return (last_interval);
				l_list := edb_client_intervals.found_item
				from
					if not l_list.is_empty then
						l_last_interval := l_list.first
					end
					l_list.start
				until
					l_list.after
				loop
					l_interval := l_list.item_for_iteration
					if l_interval.start_date <= a_settle_date.ccyymmdd and l_interval.end_date >= a_settle_date.ccyymmdd then
						if l_interval.date_type = 0 then
							l_found := l_interval
						else
							l_secondary := l_interval
						end
					end
					if l_interval.start_date <= a_trade_date.ccyymmdd and l_interval.end_date >= a_trade_date.ccyymmdd then
						if l_interval.date_type = 1 then
							l_found := l_interval
						else
							l_secondary := l_interval
						end
					end
					if l_last_interval.start_date < l_interval.start_date then
						l_last_interval := l_interval
					end
					l_list.forth
				end
				if l_found /= Void then
					Result := l_found.swept /= 0
				elseif l_secondary /= Void then
					Result := l_secondary.swept /= 0
				elseif a_settle_date.ccyymmdd > l_last_interval.end_date and a_settle_date.ccyymmdd < (l_last_interval.end_date + 7) then
					Result := l_last_interval.swept /= 0
				end
			end
		end

feature {NONE} -- Implementation

	swept_accounts_for_w_zone (a_zone: RP_ZONE): RP_SWEPT_ACCOUNTS_W is
			-- Table of swept accounts for 'a_zone'
		require
			zone_not_void: a_zone /= Void
		do
			if not a_zone.is_united_states then
				if not w_swep_accounts_shared.has (a_zone) then
					w_swep_accounts_shared.force (create {RP_SWEPT_ACCOUNTS_W}.make_from_windows_share (a_zone), a_zone)
				end
				Result := w_swep_accounts_shared.item (a_zone)
			end
		ensure
			result_not_void: Result /= Void
			result_for_zone: Result.zone.is_equal (a_zone)
		end

	w_swep_accounts_shared: DS_HASH_TABLE [RP_SWEPT_ACCOUNTS_W, RP_ZONE] is
			-- Share swept accounts info
		once
			create Result.make (17)
		end

	edb_client_intervals: DS_HASH_TABLE [DS_ARRAYED_LIST [EDB_CLIENT_INTERVAL], RP_PORTFOLIO] is
			-- U zone EDB client intervals
		local
			l_file: ROSE_FORMATTED_TEXT_FILE [EDB_CLIENT_INTERVAL]
			l_format: ROSE_DELIMITED_FORMAT
		once
			create Result.make_equal (17)
			create l_format.make (<<[constants.string_type, "client"], [constants.integer_type, "start_date"], [constants.integer_type, "end_date"], [constants.string_type, "tax_factor"], [constants.integer_type, "swept"], [constants.integer_type, "date_type"]>>, '|')
			create l_file.read ("\\axaros.com\shares\ProdData_NT\PortfolioAccounting\ParmFiles\edb_client_intervals.txt", l_format)
			l_file.array.do_all (agent add_edb_client_interval (Result, ?))
		end

	add_edb_client_interval (a_intervals: like edb_client_intervals; a_interval: EDB_CLIENT_INTERVAL) is
			-- Add `a_interval' to `a_intervals'
		require
			non_void_args: a_interval /= Void and a_intervals /= Void
		local
			l_portfolio: RP_PORTFOLIO
			l_list: DS_ARRAYED_LIST [EDB_CLIENT_INTERVAL]
		do
			a_interval.client.to_upper
			a_interval.client.prune_all (' ')
			a_interval.client.prune_all ('%T')
			l_portfolio := axarosenberg.reference_system.portfolio_from_code (a_interval.client)
			if not l_portfolio.is_unknown then
				a_intervals.search (l_portfolio)
				if a_intervals.found then
					l_list := a_intervals.found_item
				else
					create l_list.make_default
					a_intervals.force (l_list, l_portfolio)
				end
				l_list.force_last (a_interval)
			end
		end

	constants: ROSE_CONSTANT_VALUES is
			-- Constants
		once
			create Result
		end

end
