note

	description:

		"Trade contract"

	author: "Mark Howard"
	last_editor: "$Author: e2 $"
	date: "$Date: 2011/11/29 $"
	revision: "$Revision: #26 $"
	path: "$File: //rose/source/kernel/expert_system/rp_trade.e $"

deferred class RP_TRADE

inherit

	RP_BROKERAGE_CONTRACT
		rename
			make as make_brokerage_contract
		end

feature {NONE} -- Initialization

	make (a_system: like generating_system; a_broker: like broker; a_event_date: like agreement_time; a_settle_date: like agreement_time;
		a_portfolio: like portfolio; a_identifier: like identifier;
		a_commission: like commission; a_fee: like fee; a_traded_quantity: like traded_quantity;
		a_gross_quantity: like gross_quantity; a_net_quantity: like net_quantity; a_custody: RP_CUSTODY) is
			-- initialize trade items
		require
			a_system_non_void: a_system /= Void
			a_broker_non_void: a_broker /= Void
			a_event_date_non_void: a_event_date /= Void
			a_settle_date_non_void: a_settle_date /= Void
			a_portfolio_non_void: a_portfolio /= Void
			a_identifier_non_void: a_identifier /= Void
			a_commission_not_void: a_commission /= Void
			a_fee_not_void: a_fee /= Void
			a_traded_quantity_not_void: a_traded_quantity /= Void
			a_gross_quantity_not_void: a_gross_quantity /= Void
			a_net_quantity_not_void: a_net_quantity /= Void
			a_custody_not_void: a_custody /= Void
		do
			make_brokerage_contract (a_system, a_event_date, a_identifier, a_portfolio, a_broker)
			settle_date := a_settle_date
			commission := a_commission
			fee := a_fee
			gross_quantity := a_gross_quantity
			traded_quantity := a_traded_quantity
			net_quantity := a_net_quantity
			add_exchange_flows (traded_quantity, gross_quantity, net_quantity, agreement_time, settle_date, a_custody)
			add_commission_and_fees (a_custody)
		ensure
			settle_date_set: settle_date = a_settle_date
			commission_set: commission = a_commission
			fee_set: fee = a_fee
			gross_quantity_set: gross_quantity = a_gross_quantity
			traded_quantity_set: traded_quantity = a_traded_quantity
			net_quantity_set: net_quantity = a_net_quantity
		end

feature {RP_GEDB_EVENT} -- Initialization

	add_commission_and_fees (a_custody: RP_CUSTODY) is
			-- Add commission and fees
		require
			a_custody_not_void: a_custody /= Void
		do
			if not commission.amount.is_zero then
				add_flow (commission_flow_type, portfolio.instrument_control (a_custody), broker.instrument_control, agreement_time, settle_date, commission, agreement_time, 0, -commission)
			end
			if not fee.amount.is_zero then
				add_flow (exchange_fee_flow_type, portfolio.instrument_control (a_custody), broker.instrument_control, agreement_time, settle_date, fee, agreement_time, 0, -fee)
			end
		end

	add_exchange_flows (a_quantity: like traded_quantity; a_gross_monies: like gross_quantity; a_net_monies: like net_quantity; a_contracted_ownership_time, a_contracted_settlement_time: ROSE_DATE; a_custody: RP_CUSTODY) is
			-- Add exchange flows
		require
			a_quantity_non_void: a_quantity /= Void
			a_gross_monies_non_void: a_gross_monies /= Void
			a_net_monies_non_void: a_net_monies /= Void
			a_contracted_ownership_time_non_void: a_contracted_ownership_time /= Void
			a_contracted_settlement_time_non_void: a_contracted_settlement_time /= Void
			a_custody_not_void: a_custody /= Void
		deferred
		end

feature -- Access

	commission: RP_MONIES
			-- Broker commission

	fee: RP_MONIES
			-- Various fees (exchange, other)

	settle_date: ROSE_DATE
			-- Settle date

	traded_quantity: RP_QUANTITY
			-- First instrument quantity

	gross_quantity: RP_QUANTITY
			-- Second instrument quantity

	net_quantity: RP_QUANTITY
			-- Net amount of `gross_quantity'

feature -- Change

	set_commission (a_commission: like commission) is
			-- Set `commission' to `a_commission'
		do
			commission := a_commission
		ensure
			commission_set: commission = a_commission
		end

	set_fee (a_fee: like fee) is
			-- Set `fee' to `a_fee'
		do
			fee := a_fee
		ensure
			fee_set: fee = a_fee
		end

	set_settle_date (a_settle_date: like settle_date) is
			-- Set `settle_date' to `a_settle_date'
		do
			settle_date := a_settle_date
		ensure
			settle_date_set: settle_date = a_settle_date
		end

	set_traded_quantity (a_traded_quantity: like traded_quantity) is
			-- Set `traded_quantity' to `a_traded_quantity'
		do
			traded_quantity := a_traded_quantity
		end

	set_gross_quantity (a_gross_quantity: like gross_quantity) is
			-- Set `gross_quantity' to `a_gross_quantity'
		do
			gross_quantity := a_gross_quantity
		end

feature -- Status Report

	is_buy: BOOLEAN
			-- Is it a long buy?

	is_sell: BOOLEAN
			-- Is it a sell of a long position?

	is_short: BOOLEAN
			-- Is it a buy of a short position?

	is_cover: BOOLEAN
			-- Is it a sell of a short position?

invariant
	commission_set: commission /= Void
	fee_set: fee /= Void
	settle_date_set: settle_date /= Void
	traded_quantity_set: traded_quantity /= Void
	gross_quantity_set: gross_quantity /= Void
	net_quantity_set: net_quantity /= Void
--	one_state_buy: is_buy = not (is_sell or is_short or is_cover)
--	one_state_sell: is_sell = not (is_buy or is_short or is_cover)
--	one_state_short: is_short = not (is_buy or is_sell or is_cover)
--	one_state_cover: is_cover = not (is_buy or is_sell or is_short)
	same_currency_for_commision_and_fee: commission.currency ~ fee.currency
	same_instrument_for_gross_and_net_quantity: gross_quantity.instrument ~ net_quantity.instrument
end
