note

	description:

		"Trade prices"

	author: "Ivan Tarnavsky"
	last_editor: "$Author: e2 $"
	date: "$Date: 2011/11/29 $"
	revision: "$Revision: #24 $"
	path: "$File: //rose/source/kernel/expert_system/valuators/rp_trade_price_holding_valuator_flows.e $"

class RP_TRADE_PRICE_HOLDING_VALUATOR_FLOWS

inherit

	RP_FLOW_VISITOR

create {RP_TRADE_PRICE_MARKET_DATA_SOURCE}

	make

feature {NONE} -- Initialization

	make (a_portfolio: like portfolio; a_instrument: like instrument) is
			-- initiate current
		do
			create flows.make_empty
			portfolio := a_portfolio
			instrument := a_instrument
		end

feature {RP_TRADE_PRICE_MARKET_DATA_SOURCE, RP_HOLDING_VALUATOR} -- dealing with pricing

	process (a_flow: RP_FLOW) is
			-- general process call
		do
			if a_flow.quantity.instrument.is_equal (instrument) then
				price_lookup := False
				a_flow.process (Current)
			end
		end

	price (a_date: ROSE_DATE): RP_PRICE is
			-- get a trade price
		require
			non_void_date: a_date /= Void
		local
			l_found: BOOLEAN
			l_flow: RP_FLOW
		do
			if not flows.is_empty then
				if a_date >= flows.item (1).contracted_ownership_time then
					from
						flows.start
					until
						flows.off or l_found
					loop
						l_flow := flows.current_item
						if l_flow.contracted_ownership_time.is_equal (a_date) then
							l_found := True
						else
							flows.forth
						end
					end
					if l_found then
						from
							price_currency := Void
							split_factor := 1
						until
							flows.off or not l_flow.contracted_ownership_time.is_equal (a_date)
						loop
							l_flow := flows.current_item
							if l_flow.contracted_ownership_time.is_equal (a_date) then
								pricing (l_flow)
							end
							flows.forth
						end
						if price_currency /= Void then
							Result := instrument.price_in_currency (trade_price / split_factor, price_currency, a_date)
						end
					end
				end
			end
		end

feature {RP_FLOW_TYPE} -- Processing

	exchange (v: RP_FLOW) is
			-- exchange type
		do
			if price_lookup then
				price_flow (v)
			else
				flows.extend (v)
			end
		end

	process_commission (v: RP_FLOW) is
			-- commission
		do
		end

	process_tax (v: RP_FLOW) is
			-- tax
		do
		end

	process_exchange_fee (v: RP_FLOW) is
			-- exchange fee
		do
		end

	process_stamp_duty (v: RP_FLOW) is
			--stamp duty
		do
		end

	process_borrow (v: RP_FLOW) is
			-- borrow
		do
		end

	process_return (v: RP_FLOW) is
			-- return
		do
		end

	process_transfer (v: RP_FLOW) is
			-- transfer
		do
		end

	process_interest (v: RP_FLOW) is
			-- interest
		do
		end

	process_income (v: RP_FLOW) is
			-- income
		do
		end

	process_income_adjustment (v: RP_FLOW) is
			-- income adjustment
		do
		end

	process_income_unrelated (v: RP_FLOW) is
			-- income unr
		do
		end

	process_income_unrelated_adjustment (v: RP_FLOW) is
			-- income unr adj
		do
		end

	process_expense (v: RP_FLOW) is
			-- expense
		do
		end

	process_expense_adjustment (v: RP_FLOW) is
			-- exp adj
		do
		end

	process_expense_unrelated (v: RP_FLOW) is
			-- exp unre
		do
		end

	process_expense_unrelated_adjustment (v: RP_FLOW) is
			-- exp unr adj
		do
		end

	process_ffx_gain_loss (v: RP_FLOW) is
			-- ffx g/l
		do
		end

	process_dividend (v: RP_FLOW) is
			-- div
		do
		end

	process_dividend_withholding_country_tax (v: RP_FLOW) is
			-- div whtax
		do
		end

	process_dividend_withholding_nrs_tax (v: RP_FLOW) is
			-- div nrs tax
		do
		end

	process_dividend_tax_refund (v: RP_FLOW) is
			-- div tax ref
		do
		end

	process_tender (v: RP_FLOW) is
			-- tender
		do
		end

	process_distribution (v: RP_FLOW) is
			--dist
		do
			if price_lookup then
				price_flow (v)
			else
				flows.extend (v)
			end
		end

	process_split (v: RP_FLOW) is
			-- split
		do
			if price_lookup then
				if v.is_portfolio_credit_flow (portfolio) then
					if {l_split: RP_SPLIT} v.contract then
						split_factor := l_split.factor
					end
				end
			else
				flows.extend (v)
			end
		end

	process_delivery (v: RP_FLOW) is
			-- delivery
		do
		end

	process_stock_dividend (v: RP_FLOW) is
			-- stock div
		do
			if price_lookup then
				if ({l_dividend: RP_STOCK_DIVIDEND} v.contract) then
					split_factor := l_dividend.payment_factor
				end
			else
				flows.extend (v)
			end
		end

	process_inkind (v: RP_FLOW) is
			-- inkind
		do
			if price_lookup then
				price_flow (v)
			else
				flows.extend (v)
			end
		end

	process_outkind (v: RP_FLOW) is
			-- outkind
		do
			if price_lookup then
				price_flow (v)
			else
				flows.extend (v)
			end
		end

	process_mark_to_market (v: RP_FLOW) is
			-- MTM
		do
		end

	process_rights_exercise_money (v: RP_FLOW) is
			-- rights
		do
		end

	process_rights_exercise_quantity (v: RP_FLOW) is
			-- rights
		do
		end

	process_dividend_reinvestment_money (v: RP_FLOW) is
			-- div reinvest
		do
		end

	process_dividend_reinvestment_quantity (v: RP_FLOW) is
			-- div reinvest
		do
			if price_lookup then
				price_flow (v)
			else
				flows.extend (v)
			end
		end

	process_ffx_open (v: RP_FLOW) is
			-- ffx open
		do
		end

	process_ffx_close (v: RP_FLOW) is
			-- ffx close
		do
		end

	process_withdrawal (v: RP_FLOW) is
			-- withdrawal
		do
		end

	process_contribution (v: RP_FLOW) is
			-- contribution
		do
		end

	process_intrazonal_withdrawal (v: RP_FLOW) is
			-- withdrawal
		do
		end

	process_intrazonal_contribution (v: RP_FLOW) is
			-- contribution
		do
		end

	process_fungible_conversion (v: RP_FLOW) is
			-- process fungible conversion
		do
		end

	process_quantity_intraday_sell (v: RP_FLOW) is
			-- process intraday sell
		do
			exchange (v)
		end

	process_gross_monies_intraday_sell (v: RP_FLOW) is
			-- process intraday sell monies
		do
			exchange (v)
		end

	process_fixed_deposit_transfer (v: RP_FLOW) is
			-- process fixed deposit transfer
		do
		end

	process_redenomination (v: RP_FLOW) is
			-- process redenomination
		do
			exchange (v)
		end

	process_quantity (v: RP_FLOW) is
			-- process instrument quantity
		do
			exchange (v)
		end

	process_gross_monies (v: RP_FLOW) is
			-- process monies
		do
			exchange (v)
		end

	process_debt_reduction (v: RP_FLOW) is
			-- process debt reduction
		do
		end

	process_debt_payment (v: RP_FLOW) is
			-- process debt payment
		do
		end

	process_stock_bonus (v: RP_FLOW) is
			-- process stock bonus
		do
		end

feature {NONE} -- Implementation

	flows: ROSE_LINEAR_ARRAY [RP_FLOW]
			-- Cached flows

	price_lookup: BOOLEAN
			-- Indicates current visitor is pricing flows

	price_currency: RP_CURRENCY
			-- Currency of price

	split_factor: RP_DECIMAL
			-- Split factor

	trade_price: RP_DECIMAL
			-- trade price

	portfolio: RP_PORTFOLIO
			-- Portfolio

	instrument: RP_INSTRUMENT
			-- Instrument

	pricing (a_flow: RP_FLOW) is
			-- Apply visitor pattern for price lookup
		do
			price_lookup := True
			a_flow.process (Current)
		end

	price_flow (a_flow: RP_FLOW) is
			-- get price from a flow
		local
			l_currency_future: RP_CURRENCY_FUTURE
			l_future: RP_FUTURE [RP_INSTRUMENT]
			l_value: RP_QUANTITY
		do
				-- Make sure we price only the first transaction for the day
			if price_currency = Void then
				if {l_trade: RP_TRADE} a_flow.contract then
					l_value := l_trade.gross_quantity
				elseif {l_inlieu: RP_INLIEU} a_flow.contract then
					l_value := l_inlieu.gross_amount
				elseif a_flow.flow_value /= Void then
					l_value := a_flow.flow_value
				end
				if l_value /= Void then
					if not l_value.amount.is_zero then
						trade_price := (l_value.amount / a_flow.quantity.amount).abs
						price_currency ?= l_value.instrument
						if price_currency = Void then
							l_currency_future ?= l_value.instrument
							l_future ?= a_flow.quantity.instrument
							if l_currency_future /= Void and l_future /= Void then
								price_currency := l_currency_future.currency
								trade_price := trade_price / l_future.contract_size
							end
						end
					end
				end
			end
		end

end
