indexing

	description:

		"Trade/Cost price source"

	date: "$Date: 2010/04/30 $"
	revision: "$Revision: #12 $"

class RP_TRADE_PRICE_MARKET_DATA_SOURCE

inherit

	RP_MARKET_DATA_SOURCE_LAST_RECORD_SEARCH [RP_INSTRUMENT]

create

	make

feature {NONE} -- Initialization

	make (a_accounting_system: RP_ACCOUNTING_SYSTEM; a_portfolio: RP_PORTFOLIO) is
			-- Initialize `Current'
		do
			accounting_system := a_accounting_system
			portfolio := a_portfolio
		end

feature -- Query

	snapshot (a_instrument: RP_INSTRUMENT; a_date: ROSE_DATE): RP_MARKET_DATA_RECORD is
			-- Market data for `a_instrument' on `a_date'.
		local
			l_price: like trade_price
		do
			l_price := trade_price (a_instrument, a_date)
			if l_price /= Void then
				Result := create {RP_PRICE_OVERRIDE_MARKET_DATA_RECORD}.make (a_date, l_price)
			end
		end

feature {NONE} -- Implementation

	accounting_system: RP_ACCOUNTING_SYSTEM
			-- Accounting system

	portfolio: RP_PORTFOLIO
			-- Portfolio

	cache: DS_HASH_TABLE [RP_TRADE_PRICE_HOLDING_VALUATOR_FLOWS, INTEGER_64]
			-- Cached flows by instrument.uid

	trade_price (a_instrument: RP_INSTRUMENT; a_date: ROSE_DATE): RP_PRICE is
			-- Price of `a_stock' on `a_date' derived from flows
		require
			non_void_args: a_instrument /= Void and a_date /= Void
		local
			l_flows: RP_TRADE_PRICE_HOLDING_VALUATOR_FLOWS
		do
			l_flows := flows (a_instrument)
			Result := l_flows.price (a_date)
		ensure
			consistent_price: Result /= Void implies Result.instrument.is_equal (a_instrument)
		end

	flows (a_instrument: RP_INSTRUMENT): RP_TRADE_PRICE_HOLDING_VALUATOR_FLOWS is
			-- Flows for `a_instrument'
		require
			non_void_args: a_instrument /= Void
		local
			l_flows: ROSE_LINEAR_ARRAY [RP_FLOW]
		do
			if cache = Void then
				create cache.make_default
			end
			cache.search (a_instrument.uid)
			if cache.found then
				Result := cache.found_item
			else
				create Result.make (portfolio, a_instrument)
				l_flows := accounting_system.flows_by_portfolio (portfolio)
				l_flows.do_all (agent Result.process (?))
				cache.force (Result, a_instrument.uid)
			end
		end

end
