indexing
	description: "Value with units of measure"
	date: "$Date: 2008/12/01 $"
	revision: "$Revision: #18 $"

class RP_UNITIZED_VALUE[G->RP_UNITS]

inherit

	DEBUG_OUTPUT

	ROSE_CONSTANTS

	RP_BAG

create

	make, set_bag

feature {NONE} -- Initialization

	make (a_value: DOUBLE; a_units: like units) is
			-- Initialize `Current'
		require
			non_void_units: a_units /= Void
		do
			value := a_value
			units := a_units
		end

	set_bag (a_values : ARRAY [RP_UNITIZED_VALUE [RP_UNITS]]) is
			-- Initialize `Current'
		do
			create bag.make_from_unitized_values (a_values)
		end

feature -- Access

	value: DOUBLE
			-- Value.

	units: G
			-- Units.

	bag: RP_UNITIZED_BAG[RP_UNITS]
			-- Bag of unitized values.

feature -- Debug output

	debug_output : STRING is
		do

			Result := format.formatted(value)
			if units /= void then
				Result := Result + " " + units.debug_output
			end
		end

	format : FORMAT_DOUBLE is
		once
			create Result.make(20,6)
		end

feature -- Status reporting

	is_defined : BOOLEAN is
		do
			Result := is_defined_double (value)
		end

	is_close (a_other: RP_UNITIZED_VALUE[RP_UNITS]; a_tolerance: DOUBLE): BOOLEAN is
		require
			known_units: units /= Void
			other_not_void: a_other /= Void
			sensible_tolerance : a_tolerance >= 0
		do
			Result := units.is_same (a_other.units) and then
				(value - a_other.value).abs <= a_tolerance
		end

feature -- Basic operations

	infix "+" (a_other : like Current) : like Current is
		do
			if units.is_same(a_other.units) then
				create Result.make(value + a_other.value,units)
			else
				create Result.set_bag(<<Current,a_other>>)
			end
		end

	infix "-" (a_other : like Current) : like Current is
		do
			if units.is_same(a_other.units) then
				create Result.make(value - a_other.value,units)
			else
				create Result.set_bag(<<Current,a_other>>)
			end
		end

	infix "*" (a_other : RP_UNITIZED_VALUE[RP_UNITS]) : RP_UNITIZED_VALUE[RP_UNITS] is
		do
			create Result.make(value * a_other.value,units * a_other.units)
		end

	infix "/" (a_other : RP_UNITIZED_VALUE[RP_UNITS]) : RP_UNITIZED_VALUE[RP_UNITS] is
		do
			create Result.make(value / a_other.value,units / a_other.units)
		end

	infix ">" (a_other: RP_UNITIZED_VALUE[RP_UNITS]):BOOLEAN is
		do
			REsult:= value > a_other.value
		end

	prefix "-" : like Current is
		do
			create Result.make(-value,units)
		end

	scale_by (a_factor: DOUBLE): like Current is
			-- Scale `value' by `a_factor'.
		do
			create Result.make (value * a_factor, units)
		end

	abs : like Current is
		do
			create Result.make(value.abs,units)
		end

end -- class RP_UNITIZED_VALUE
