note

	description:

		"Investment Region"

	author: "$Author: rj $"
	last_editor: "$Author: rj $"
	date: "$Date: 2014/08/08 $"
	revision: "$Revision: #31 $"
	path: "$File: //rose/source/kernel/expert_system/aflows/rp_zone.e $"

class RP_ZONE

inherit

	HASHABLE

create {RP_REFERENCE_SYSTEM}

	make

feature {NONE} -- Constructor

	make (a_ref: like reference_system; a_gcdb_zone: like gcdb_zone; a_zone: like zone; a_inception_date: like inception_date) is
			-- Initialize `Current'
		require
			non_void_ref: a_ref /= Void
		do
			reference_system := a_ref
			gcdb_zone := a_gcdb_zone
			zone := a_zone
			inception_date := a_inception_date
			if gcdb_zone = Void then
				code := "?"
				is_unknown := True
			else
				code := gcdb_zone.code
			end
		ensure
			reference_system_set_and_not_void: (reference_system /= Void) and then (reference_system = a_ref)
			gcdb_zone_set: gcdb_zone = a_gcdb_zone
			zone_set: zone = a_zone
			inception_date_set: inception_date = a_inception_date
			based_on_gcdb: (gcdb_zone = Void) implies (code.is_equal ("?") and is_unknown)
			not_unknow_if_gcdb_zone_available: (gcdb_zone /= Void) implies (not is_unknown)
		end

feature -- Setting

	refresh
			-- Refresh all cached data
		do
			saved_portfolios := Void
			retrieve_portfolios.do_nothing
			saved_investment_accounts := Void
			retrieve_investment_accounts.do_nothing
			cached_bank_calendar := Void
			retrieve_bank_calendar.do_nothing
			cached_legacy_perf_calendar := Void
			retrieve_legacy_perf_calendar.do_nothing
			cached_market_calendar := Void
			retrieve_market_calendar.do_nothing
			cached_products := Void
			retrieve_products.do_nothing
		ensure
			saved_portfolios_not_void: saved_portfolios /= Void
			new_saved_portfolios: saved_portfolios /= old saved_portfolios
			saved_investment_accounts_not_void: saved_investment_accounts /= Void
			new_saved_investment_accounts: saved_investment_accounts /= old saved_investment_accounts
			cached_bank_calendar_not_void: cached_bank_calendar /= Void
			new_cached_bank_calendar: cached_bank_calendar /= old cached_bank_calendar
			cached_legacy_perf_calendar_not_void: cached_legacy_perf_calendar /= Void
			new_cached_legacy_perf_calendar: cached_legacy_perf_calendar /= old cached_legacy_perf_calendar
			cached_market_calendar_not_void: cached_market_calendar /= Void
			new_cached_market_calendar: cached_market_calendar /= old cached_market_calendar
			cached_products_not_void: cached_products /= Void
			new_cached_products: cached_products /= old cached_products
		end

feature -- Access

	code: STRING
			-- Unique code

	hash_code: INTEGER is
			-- Hash code value
		do
			if code /= Void then
				Result := code.hash_code
			end
		end

	inception_date: ROSE_DATE
			-- Inception date

	legacy_index: INTEGER is
			-- Index on VMS
		do
			if is_unknown or zone = Void then
				Result := -1
			else
				Result := zone.index
			end
		end

feature {RP_ZONE} -- Access

	zone: ZONE [LOCATION]
			-- Underlying entity.
			-- Will be Void if `is_unknown' = `True' (at least).

feature -- Query

	portfolios: RP_PORTFOLIO_HISTORY is
			-- list of portfolios
		do
			Result := retrieve_portfolios
		end

	investment_accounts: RP_INVESTMENT_ACCOUNT_HISTORY is
			-- list of accounts
		do
			Result := retrieve_investment_accounts
		end

	products: ARRAY [RP_PRODUCT] is
			-- Products
		do
			Result := retrieve_products
		end

	bank_calendar: RP_CALENDAR is
			-- Bank calendar
		do
			Result := retrieve_bank_calendar
		end

	market_calendar: RP_CALENDAR is
			-- Market calendar
		do
			Result := retrieve_market_calendar
		end

	legacy_perf_calendar: RP_CALENDAR is
			-- Legacy perf calendar
		do
			Result := retrieve_legacy_perf_calendar
		end

	legacy_base_currency: RP_CURRENCY is
			-- Legacy base currency
		do
			inspect code.item (1)
			when 'C' then
				Result := reference_system.currency_from_iso_code ("CAD")
			when 'E' then
				Result := reference_system.currency_from_iso_code ("GBP")
			when 'J' then
				Result := reference_system.currency_from_iso_code ("JPY")
			else
				Result := reference_system.currency_from_iso_code ("USD")
			end
		ensure
			non_void_currency: Result /= Void
		end

feature -- Status Report

	is_canada: BOOLEAN is
			-- Is (C)anada?
		do
			Result := (zone /= Void) and then (zone.is_c_zone)
		end

	is_europe: BOOLEAN is
			-- Is (E)urope?
		do
			Result := (zone /= Void) and then (zone.is_e_zone)
		end

	is_japan: BOOLEAN is
			-- Is (J)apan?
		do
			Result := (zone /= Void) and then (zone.is_j_zone)
		end

	is_orient: BOOLEAN is
			-- Is (O)rient?
		do
			Result := (zone /= Void) and then (zone.is_o_zone)
		end

	is_south_america: BOOLEAN is
			-- Is (S)outh America?
		do
			Result := (zone /= Void) and then (zone.is_s_zone)
		end

	is_transzonal: BOOLEAN is
			-- Is (T)ranzonal?
		do
			Result := (zone /= Void) and then (zone.is_t_zone)
		end

	is_united_states: BOOLEAN is
			-- Is (U)nited (S)tates?
		do
			Result := (zone /= Void) and then (zone.is_u_zone)
		end

	is_unknown: BOOLEAN
			-- Is `Current' instance of Unknown investment region

feature {NONE} -- Implementations

	gcdb_zone: GCDB_INVESTMENT_REGION
		-- Underlying zone provider

	reference_system: RP_REFERENCE_SYSTEM
		-- Reference system that created current

	retrieve_portfolios: like portfolios is
			-- Load all GCDB portfolios for this zone
		do
			if saved_portfolios = Void then
				create saved_portfolios.make_empty
				if gcdb_zone /= Void then
					gcdb_zone.portfolios_by_investment_region_history.do_all (agent add_portfolio (saved_portfolios, ?))
				end
			end
			Result := saved_portfolios
		ensure
			retrieve_portfolios_not_void: Result /= Void
		end

	saved_portfolios: like portfolios
			-- Portfolios cache

	add_portfolio (a_portfolios: like portfolios; a_interval: ENTITY_INTERVAL [GCDB_PORTFOLIO]) is
			-- add portfolio
		require
			a_portfolios_not_void: a_portfolios /= Void
			no_void_portfolio: not a_portfolios.has (Void)
			a_interval_not_void: a_interval /= Void
		local
			l_code: STRING
		do
			l_code := a_interval.entity.code
			if l_code = Void then
				a_interval.entity.set_as_of_date (a_interval.entity.start_date)
				l_code := a_interval.entity.code
				if l_code = Void then
					a_interval.entity.set_as_of_date (a_interval.entity.end_date)
				end
			end
			if l_code /= Void then
				a_portfolios.extend (create {RP_PORTFOLIO_HISTORY_INTERVAL}.make (reference_system.portfolio_from_code (l_code), a_interval.start_date, a_interval.end_date, 1))
			end
		end

	retrieve_investment_accounts: like investment_accounts is
			-- Get accounts from GCDB
		do
			if saved_investment_accounts = Void then
				create saved_investment_accounts.make_empty
				if gcdb_zone /= Void then
					gcdb_zone.accounts_by_domicile_region_history.do_all (agent add_investment_account (saved_investment_accounts, ?))
				end
			end
			Result := saved_investment_accounts
		ensure
			retrieve_investment_accounts_not_void: Result /= Void
		end

	saved_investment_accounts: like investment_accounts
			-- Saved investment accounts

	add_investment_account (a_investment_accounts: like investment_accounts; a_interval: ENTITY_INTERVAL [GCDB_ACCOUNT]) is
			-- add investment account
		local
			l_code: STRING
			l_acct: RP_INVESTMENT_ACCOUNT
		do
			l_code := a_interval.entity.code
			if l_code = Void then
				a_interval.entity.set_as_of_date (a_interval.entity.start_date)
				l_code := a_interval.entity.code
				if l_code = Void then
					a_interval.entity.set_as_of_date (a_interval.entity.end_date)
				end
			end
			if l_code /= Void then
				l_acct := reference_system.investment_account_by_code (l_code)
				if not l_acct.is_unknown then
					a_investment_accounts.extend (create {RP_INVESTMENT_ACCOUNT_HISTORY_INTERVAL}.make (l_acct, a_interval.start_date, a_interval.end_date, 1))
				end
			end
		end

	cached_bank_calendar: RP_CALENDAR
			-- Cached bank calendar

	retrieve_bank_calendar: RP_CALENDAR is
			-- retrieve bank calendar
		do
			if cached_bank_calendar = Void then
				if zone /= Void then
					cached_bank_calendar := create {RP_ROSE_BUSINESS_DATE_CALENDAR}.make (zone.bank_dates)
				else
					cached_bank_calendar := create {RP_WEEKDAYS_CALENDAR}
				end
			end
			Result := cached_bank_calendar
		ensure
			retrieve_bank_calendar_not_void: Result /= Void
		end

	cached_market_calendar: RP_CALENDAR
			-- Cached market calendar

	retrieve_market_calendar: RP_CALENDAR is
			-- Retrieve market calendar
		do
			if cached_market_calendar = Void then
				if zone /= Void then
					cached_market_calendar := create {RP_ROSE_BUSINESS_DATE_CALENDAR}.make (zone.market_dates)
				else
					cached_market_calendar := create {RP_WEEKDAYS_CALENDAR}
				end
			end
			Result := cached_market_calendar

		ensure
			retrieve_market_calendar_not_void: Result /= Void
		end

	compute_legacy_perf_bitmap: ROSE_BITMAP is
			-- Compute perf calendar bitmap as defined in VMS
		local
			l_countries: ARRAY [STRING]
			l_country: RP_COUNTRY
			l_bank_calendar, l_market_calendat: ROSE_BITMAP
			i, n, k: INTEGER
			l_date: ROSE_DATE
		do
			legacy_country_by_zone.search (code)
			if legacy_country_by_zone.found then
				l_countries := legacy_country_by_zone.found_item
				from
					i := 1
					create l_bank_calendar.make_empty (36500)
					create l_market_calendat.make_empty (36500)
				until
					i > l_countries.count
				loop
					l_country := reference_system.country_from_iso_code (l_countries.item (i))
					if not l_country.is_unknown then
						l_bank_calendar.or_with (l_country.country.bank_date_bitmap)
						l_market_calendat.or_with (l_country.country.market_date_bitmap)
					end
					i := i + 1
				end
					-- legacy perf uses the NEXTMARKETDAY_ZONE[LOCATION] Fortran function
					-- which searches the CALENDAR.PARM file for 3 or 1
					-- hence => (bank and market) or market
				l_bank_calendar.and_with (l_market_calendat)
				l_bank_calendar.or_with (l_market_calendat)
					-- account for discrepancies between VMS and Eiffel calendar for BH
				if l_countries.has ("BH") then
					from
						create l_date.make_ccyymmdd (bh_start.ccyymmdd)
						n := bitmap_start_date.days
					until
						l_date > calendar_cutover_date
					loop
						i := l_date.days
						k := (i - 1) \\ 7 + 1
						if k > 1 and k < 7 then
							l_bank_calendar.set (True, i - n + 1)
						end
						l_date := l_date.next_date
					end
				end
				Result := l_bank_calendar
			else
				create Result.make_empty (0)
			end
		end

	compute_perf_bitmap: ROSE_BITMAP is
			-- Compute perf calendar bitmap.
		require
			zone_not_void: zone /= Void
		do
			create Result.make_empty (36500)
				-- legacy perf uses the NEXTMARKETDAY_ZONE[LOCATION] Fortran function
				-- which searches the CALENDAR.PARM file for 3 or 1
				-- hence => (bank and market) or market
			Result.or_with (zone.bank_dates.membership)
			Result.and_with (zone.market_dates.membership)
			Result.or_with (zone.market_dates.membership)
		end

	cached_legacy_perf_calendar: like legacy_perf_calendar
			-- Cached value for `legacy_perf_calendar'

	retrieve_legacy_perf_calendar: like legacy_perf_calendar is
			-- Retrieve value for `legacy_perf_calendar'.
		local
			l_date_set: ROSE_DATE_SET
			l_calendar_before: RP_CALENDAR
			l_calendar_after: RP_CALENDAR
		do
			if cached_legacy_perf_calendar = Void then
				if zone = Void then
					create {RP_WEEKDAYS_CALENDAR} cached_legacy_perf_calendar
				else
					create l_date_set.make (compute_legacy_perf_bitmap, bitmap_start_date)
					l_calendar_before := create {RP_ROSE_BUSINESS_DATE_CALENDAR}.make (l_date_set)
					create l_date_set.make (compute_perf_bitmap, bitmap_start_date)
					l_calendar_after := create {RP_ROSE_BUSINESS_DATE_CALENDAR}.make (l_date_set)
					cached_legacy_perf_calendar := create {RP_COMBINED_CALENDAR_WITH_CUTOVER}.make (l_calendar_before, l_calendar_after, calendar_cutover_date)
				end
			end
			Result := cached_legacy_perf_calendar
		ensure
			retrieve_legacy_perf_calendar_not_void: Result /= Void
		end

	calendar_cutover_date: ROSE_DATE is
			-- Calendar cutover date as defined in #44132
		once
			create Result.make_ccyymmdd (20080317)
		end

	bitmap_start_date: ROSE_DATE is
			-- Start date for bitmap
		once
			create Result.make_ccyymmdd (19500101)
		end

	bh_start: ROSE_DATE is
			-- Bahrein discrepancy start date
		once
			create Result.make_ccyymmdd (20040102)
		end

	legacy_country_by_zone: DS_HASH_TABLE [ARRAY [STRING], STRING]
			-- Legacy countries by zone code
		local
			l_all_country_names: ARRAY[STRING]
		once
			create Result.make_equal (8)
			Result.force (<<"CA">>, "C")
			Result.force (<<"AT", "BE", "BH", "CH", "DE", "DK", "ES", "FI", "FR", "GB", "GR", "IE", "IT", "LU", "NL", "NO", "PT", "SE", "ZA">>, "E")
			Result.force (<<"JP">>, "J")
			Result.force (<<"AU", "HK", "KR", "MY", "NZ", "PH", "SG", "TH", "TW">>, "O")
			Result.force (<<"AR", "MX">>, "S")
			Result.force (<<"US">>, "U")

			l_all_country_names := <<"CA", "AT", "BE", "BH", "CH", "DE", "DK", "ES", "FI", "FR", "GB", "GR", "IE", "IT", "LU", "NL", "NO", "PT", "SE", "ZA", "JP", "AU", "HK", "KR", "MY", "NZ", "PH", "SG", "TH", "TW", "AR", "MX", "US">>
			Result.force (l_all_country_names, "T")
			Result.force (l_all_country_names, "W")

			Result.do_all (agent {ARRAY[STRING]}.compare_objects)
		ensure
			all_zones_set: across reference_system.investment_regions as c_investment_regions all Result.has(c_investment_regions.item.zone.mnemonic.out) end
		end

	cached_products: like products
			-- Cached products

	retrieve_products: like products is
			-- Retrieve products
		local
			l_result: ROSE_LINEAR_ARRAY [RP_PRODUCT]
		do
			if cached_products = Void then
				create l_result.make_empty
				if gcdb_zone /= Void then
					gcdb_zone.products_by_investment_region_history.do_all (agent add_product (l_result, ?))
				end
				cached_products := l_result
			end
			Result := cached_products
		ensure
			retrieve_products_not_void: Result /= Void
		end

	add_product (a_array: ROSE_LINEAR_ARRAY [RP_PRODUCT]; a_interval: ENTITY_INTERVAL [GCDB_PRODUCT]) is
			-- Add product
		local
			l_product: RP_PRODUCT
		do
			l_product := reference_system.product_by_code (a_interval.entity.code)
			if not l_product.is_unknown then
				a_array.extend (l_product)
			end
		end

end
