note
	description: "Parser for RX_EXPRESSION-s"
	author: "Zoran Simic"
	last_editor: "$Author: er $"
	generator: "Gobo - geyacc"
	date: "$Date: 2012/09/04 $"
	revision: "$Revision: #8 $"
	path: "$File: //rose/source/kernel/expressions/rx_expression_parser.e $"

class
	RX_EXPRESSION_PARSER [reference G -> RX_EXPRESSION]

inherit
	YY_NEW_PARSER_SKELETON
		rename
			make as make_parser_skeleton
		redefine
			report_error
		end

	RX_EXPRESSION_SCANNER

	ROSE_SHARED_GLOBAL

create
	make

feature {NONE} -- Initialization

	make (a_factory: RX_EXPRESSION_FACTORY [G]) is
			-- Create a new Rose expression-node parser.
		require
			factory_ok: a_factory /= Void
		do
			make_scanner
			make_parser_skeleton
			set_factory (a_factory)
		end

feature -- Access

	factory: RX_EXPRESSION_FACTORY [G]
			-- Factory returning expressions associated to expression-ids

	last_list: ROSE_LINEAR_ARRAY [G]
			-- Last list parsed, Void if an error occured

	last_parsed: G
			-- Last node parsed, Void if an error occured

feature -- Status setting

	set_factory (a_factory: like factory) is
		require
			factory_ok: a_factory /= Void
		do
			factory := a_factory
		end

feature -- Basic operations

	parse_file (a_path: STRING) is
			-- Parse expressions from 'a_path'.
			-- 'error.signaled' is true if an error occurs during this operation
			-- Use 'error.flush_and_acknowledge' to get the error message output
		require
			valid_path: a_path /= Void
			errors_acknowledged: not error.signaled
		local
			l_file: KL_TEXT_INPUT_FILE
			l_default: G
			rescued: STRING
		do
			if rescued /= Void then
				error.signal (generator,"The following problem occurred while trying to parse expressions from '$1':%N$2",<<a_path,rescued>>)
			end
			last_list := Void
			last_parsed := l_default
			if not error.signaled then
				buffer_name := a_path
				create l_file.make (buffer_name)
				if not l_file.exists then
					error.signal (generator,"Can not read '$1', file does not exist",<<buffer_name>>)
				elseif not l_file.is_readable then
					error.signal (generator,"Can not read '$1', file is not readable",<<buffer_name>>)
				end
			end
			if not error.signaled then
				reset
				l_file.open_read
				set_input_buffer (new_file_buffer (l_file))
				parse
				l_file.close
			end
			buffer_name := Void
		rescue
			if rescued = Void then
				rescued :=  last_exception_explanation
				retry
			end
		end

	parsed_string (a_string: STRING): G is
			-- Parse expressions from 'a_string'.
			-- 'error.signaled' is true if an error occur during this operation
			-- Use 'error.flush_and_acknowledge' to get the error message output
		require
			valid_string: a_string /= Void
			errors_acknowledged: not error.signaled
		local
			l_default: G
			rescued: STRING
		do
			if rescued /= Void then
				error.signal (generator,"The following problem occurred while trying to parse expression '$1':%N$2",<<a_string,rescued>>)
			end
			last_list := Void
			last_parsed := l_default
			if not error.signaled then
				buffer_name := "*buffer*"
				reset
				set_input_buffer (new_string_buffer (a_string))
				parse
				buffer_name := Void
			end
			Result := last_parsed
		rescue
			if rescued = Void then
				rescued :=  last_exception_explanation
				retry
			end
		end


feature {NONE} -- Implementation

	yy_build_parser_tables
			-- Build parser tables.
		do
			yytranslate := yytranslate_template
			yyr1 := yyr1_template
			yytypes1 := yytypes1_template
			yytypes2 := yytypes2_template
			yydefact := yydefact_template
			yydefgoto := yydefgoto_template
			yypact := yypact_template
			yypgoto := yypgoto_template
			yytable := yytable_template
			yycheck := yycheck_template
		end

	yy_create_value_stacks
			-- Create value stacks.
		do
		end

	yy_init_value_stacks
			-- Initialize value stacks.
		do
			yyvsp1 := -1
			yyvsp2 := -1
			yyvsp3 := -1
			yyvsp4 := -1
		end

	yy_clear_value_stacks
			-- Clear objects in semantic value stacks so that
			-- they can be collected by the garbage collector.
		local
			l_yyvs1_default_item: ANY
			l_yyvs2_default_item: like last_list
			l_yyvs3_default_item: G
			l_yyvs4_default_item: STRING
		do
			if yyvs1 /= Void then
				yyvs1.fill_with (l_yyvs1_default_item, 0, yyvs1.upper)
			end
			if yyvs2 /= Void then
				yyvs2.fill_with (l_yyvs2_default_item, 0, yyvs2.upper)
			end
			if yyvs3 /= Void then
				yyvs3.fill_with (l_yyvs3_default_item, 0, yyvs3.upper)
			end
			if yyvs4 /= Void then
				yyvs4.fill_with (l_yyvs4_default_item, 0, yyvs4.upper)
			end
		end

	yy_push_last_value (yychar1: INTEGER)
			-- Push semantic value associated with token `last_token'
			-- (with internal id `yychar1') on top of corresponding
			-- value stack.
		do
			inspect yytypes2.item (yychar1)
			when 1 then
				yyvsp1 := yyvsp1 + 1
				if yyvsp1 >= yyvsc1 then
					if yyvs1 = Void then
						debug ("GEYACC")
							std.error.put_line ("Create yyvs1")
						end
						create yyspecial_routines1
						yyvsc1 := yyInitial_yyvs_size
						yyvs1 := yyspecial_routines1.make (yyvsc1)
					else
						debug ("GEYACC")
							std.error.put_line ("Resize yyvs1")
						end
						yyvsc1 := yyvsc1 + yyInitial_yyvs_size
						yyvs1 := yyspecial_routines1.resize (yyvs1, yyvsc1)
					end
				end
				yyspecial_routines1.force (yyvs1, last_any_value, yyvsp1)
			when 4 then
				yyvsp4 := yyvsp4 + 1
				if yyvsp4 >= yyvsc4 then
					if yyvs4 = Void then
						debug ("GEYACC")
							std.error.put_line ("Create yyvs4")
						end
						create yyspecial_routines4
						yyvsc4 := yyInitial_yyvs_size
						yyvs4 := yyspecial_routines4.make (yyvsc4)
					else
						debug ("GEYACC")
							std.error.put_line ("Resize yyvs4")
						end
						yyvsc4 := yyvsc4 + yyInitial_yyvs_size
						yyvs4 := yyspecial_routines4.resize (yyvs4, yyvsc4)
					end
				end
				yyspecial_routines4.force (yyvs4, last_string_value, yyvsp4)
			else
				debug ("GEYACC")
					std.error.put_string ("Error in parser: not a token type: ")
					std.error.put_integer (yytypes2.item (yychar1))
					std.error.put_new_line
				end
				abort
			end
		end

	yy_push_error_value
			-- Push semantic value associated with token 'error'
			-- on top of corresponding value stack.
		local
			yyval1: ANY
		do
			yyvsp1 := yyvsp1 + 1
			if yyvsp1 >= yyvsc1 then
				if yyvs1 = Void then
					debug ("GEYACC")
						std.error.put_line ("Create yyvs1")
					end
					create yyspecial_routines1
					yyvsc1 := yyInitial_yyvs_size
					yyvs1 := yyspecial_routines1.make (yyvsc1)
				else
					debug ("GEYACC")
						std.error.put_line ("Resize yyvs1")
					end
					yyvsc1 := yyvsc1 + yyInitial_yyvs_size
					yyvs1 := yyspecial_routines1.resize (yyvs1, yyvsc1)
				end
			end
			yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
		end

	yy_pop_last_value (yystate: INTEGER)
			-- Pop semantic value from stack when in state `yystate'.
		local
			yy_type_id: INTEGER
		do
			yy_type_id := yytypes1.item (yystate)
			inspect yy_type_id
			when 1 then
				yyvsp1 := yyvsp1 - 1
			when 2 then
				yyvsp2 := yyvsp2 - 1
			when 3 then
				yyvsp3 := yyvsp3 - 1
			when 4 then
				yyvsp4 := yyvsp4 - 1
			else
				debug ("GEYACC")
					std.error.put_string ("Error in parser: unknown type id: ")
					std.error.put_integer (yy_type_id)
					std.error.put_new_line
				end
				abort
			end
		end

	yy_run_geyacc
			-- You must run geyacc to regenerate this class.
		do
		end

feature {NONE} -- Semantic actions

	yy_do_action (yy_act: INTEGER)
			-- Execute semantic action.
		local
			yyval1: ANY
			yyval2: like last_list
			yyval3: G
		do
			inspect yy_act
when 1 then
--|#line 156 "..\..\External\parsing\rx_expression_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file '..\..\External\parsing\rx_expression_parser.y' at line 156")
end

accept
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 2 then
--|#line 158 "..\..\External\parsing\rx_expression_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file '..\..\External\parsing\rx_expression_parser.y' at line 158")
end

last_list := Void 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 0
	yyvsp2 := yyvsp2 + 1
	if yyvsp2 >= yyvsc2 then
		if yyvs2 = Void then
			debug ("GEYACC")
				std.error.put_line ("Create yyvs2")
			end
			create yyspecial_routines2
			yyvsc2 := yyInitial_yyvs_size
			yyvs2 := yyspecial_routines2.make (yyvsc2)
		else
			debug ("GEYACC")
				std.error.put_line ("Resize yyvs2")
			end
			yyvsc2 := yyvsc2 + yyInitial_yyvs_size
			yyvs2 := yyspecial_routines2.resize (yyvs2, yyvsc2)
		end
	end
	yyspecial_routines2.force (yyvs2, yyval2, yyvsp2)
end
when 3 then
--|#line 159 "..\..\External\parsing\rx_expression_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file '..\..\External\parsing\rx_expression_parser.y' at line 159")
end

add_expression (yyvs3.item (yyvsp3)) 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp2 := yyvsp2 + 1
	yyvsp3 := yyvsp3 -1
	if yyvsp2 >= yyvsc2 then
		if yyvs2 = Void then
			debug ("GEYACC")
				std.error.put_line ("Create yyvs2")
			end
			create yyspecial_routines2
			yyvsc2 := yyInitial_yyvs_size
			yyvs2 := yyspecial_routines2.make (yyvsc2)
		else
			debug ("GEYACC")
				std.error.put_line ("Resize yyvs2")
			end
			yyvsc2 := yyvsc2 + yyInitial_yyvs_size
			yyvs2 := yyspecial_routines2.resize (yyvs2, yyvsc2)
		end
	end
	yyspecial_routines2.force (yyvs2, yyval2, yyvsp2)
end
when 4 then
--|#line 160 "..\..\External\parsing\rx_expression_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file '..\..\External\parsing\rx_expression_parser.y' at line 160")
end

add_expression (yyvs3.item (yyvsp3)) 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp1 := yyvsp1 -1
	yyvsp3 := yyvsp3 -1
	yyspecial_routines2.force (yyvs2, yyval2, yyvsp2)
end
when 5 then
--|#line 163 "..\..\External\parsing\rx_expression_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file '..\..\External\parsing\rx_expression_parser.y' at line 163")
end

yyval3 := yyvs3.item (yyvsp3) 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines3.force (yyvs3, yyval3, yyvsp3)
end
when 6 then
--|#line 164 "..\..\External\parsing\rx_expression_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file '..\..\External\parsing\rx_expression_parser.y' at line 164")
end

yyval3 := yyvs3.item (yyvsp3) 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp1 := yyvsp1 -2
	yyspecial_routines3.force (yyvs3, yyval3, yyvsp3)
end
when 7 then
--|#line 175 "..\..\External\parsing\rx_expression_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file '..\..\External\parsing\rx_expression_parser.y' at line 175")
end

yyval3 := factory.binary_operator_plus (yyvs3.item (yyvsp3 - 1), yyvs3.item (yyvsp3)) 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp3 := yyvsp3 -1
	yyvsp1 := yyvsp1 -1
	yyspecial_routines3.force (yyvs3, yyval3, yyvsp3)
end
when 8 then
--|#line 180 "..\..\External\parsing\rx_expression_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file '..\..\External\parsing\rx_expression_parser.y' at line 180")
end

yyval3 := factory.binary_operator_at (yyvs3.item (yyvsp3 - 1), yyvs3.item (yyvsp3)) 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp3 := yyvsp3 -1
	yyvsp1 := yyvsp1 -1
	yyspecial_routines3.force (yyvs3, yyval3, yyvsp3)
end
when 9 then
--|#line 181 "..\..\External\parsing\rx_expression_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file '..\..\External\parsing\rx_expression_parser.y' at line 181")
end

yyval3 := factory.binary_operator_delta (yyvs3.item (yyvsp3 - 1), yyvs3.item (yyvsp3)) 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp3 := yyvsp3 -1
	yyvsp1 := yyvsp1 -1
	yyspecial_routines3.force (yyvs3, yyval3, yyvsp3)
end
when 10 then
--|#line 182 "..\..\External\parsing\rx_expression_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file '..\..\External\parsing\rx_expression_parser.y' at line 182")
end

yyval3 := factory.binary_operator_unknown (yyvs3.item (yyvsp3 - 1), yyvs3.item (yyvsp3)) 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp3 := yyvsp3 -1
	yyvsp1 := yyvsp1 -1
	yyspecial_routines3.force (yyvs3, yyval3, yyvsp3)
end
when 11 then
--|#line 197 "..\..\External\parsing\rx_expression_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file '..\..\External\parsing\rx_expression_parser.y' at line 197")
end

yyval3 := factory.expression_from_id (yyvs4.item (yyvsp4)) 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp3 := yyvsp3 + 1
	yyvsp4 := yyvsp4 -1
	if yyvsp3 >= yyvsc3 then
		if yyvs3 = Void then
			debug ("GEYACC")
				std.error.put_line ("Create yyvs3")
			end
			create yyspecial_routines3
			yyvsc3 := yyInitial_yyvs_size
			yyvs3 := yyspecial_routines3.make (yyvsc3)
		else
			debug ("GEYACC")
				std.error.put_line ("Resize yyvs3")
			end
			yyvsc3 := yyvsc3 + yyInitial_yyvs_size
			yyvs3 := yyspecial_routines3.resize (yyvs3, yyvsc3)
		end
	end
	yyspecial_routines3.force (yyvs3, yyval3, yyvsp3)
end
			else
				debug ("GEYACC")
					std.error.put_string ("Error in parser: unknown rule id: ")
					std.error.put_integer (yy_act)
					std.error.put_new_line
				end
				abort
			end
		end

	yy_do_error_action (yy_act: INTEGER)
			-- Execute error action.
		do
			inspect yy_act
			when 19 then
					-- End-of-file expected action.
				report_eof_expected_error
			else
					-- Default action.
				report_error ("parse error")
			end
		end

feature {NONE} -- Table templates

	yytranslate_template: SPECIAL [INTEGER]
			-- Template for `yytranslate'
		once
			Result := yyfixed_array (<<
			    0,    2,    2,    2,    2,    2,    2,    2,    2,    2,
			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
			    2,    2,    2,    2,    2,    2,    2,    2,   24,    2,
			   35,   36,   26,   22,   34,   23,    2,   27,    2,    2,
			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
			   18,   16,   19,   25,   31,    2,    2,    2,    2,    2,
			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
			    2,    2,    2,    2,   30,    2,    2,    2,    2,    2,

			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,

			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
			    2,    2,    2,    2,    2,    2,    1,    2,    3,    4,
			    5,    6,    7,    8,    9,   10,   11,   12,   13,   14,
			   15,   17,   20,   21,   28,   29,   32,   33, yyDummy>>)
		end

	yyr1_template: SPECIAL [INTEGER]
			-- Template for `yyr1'
		once
			Result := yyfixed_array (<<
			    0,   40,   37,   37,   37,   39,   39,   39,   39,   39,
			   39,   38, yyDummy>>)
		end

	yytypes1_template: SPECIAL [INTEGER]
			-- Template for `yytypes1'
		once
			Result := yyfixed_array (<<
			    1,    1,    4,    2,    3,    3,    3,    1,    1,    1,
			    1,    1,    1,    1,    3,    3,    3,    3,    3,    1,
			    1,    1, yyDummy>>)
		end

	yytypes2_template: SPECIAL [INTEGER]
			-- Template for `yytypes2'
		once
			Result := yyfixed_array (<<
			    1,    1,    1,    4,    4,    4,    4,    4,    1,    1,
			    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
			    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
			    1,    1,    1,    1,    1,    1,    1, yyDummy>>)
		end

	yydefact_template: SPECIAL [INTEGER]
			-- Template for `yydefact'
		once
			Result := yyfixed_array (<<
			    2,    0,   11,    0,    5,    3,    0,    0,    1,    0,
			    0,    0,    0,    6,    4,    8,   10,    9,    7,    0,
			    0,    0, yyDummy>>)
		end

	yydefgoto_template: SPECIAL [INTEGER]
			-- Template for `yydefgoto'
		once
			Result := yyfixed_array (<<
			    3,    4,    5,   19, yyDummy>>)
		end

	yypact_template: SPECIAL [INTEGER]
			-- Template for `yypact'
		once
			Result := yyfixed_array (<<
			   -7,   -7, -32768,  -10, -32768,    1,  -20,   -7, -32768,   -7,
			   -7,   -7,   -7, -32768,    1,  -24,  -24,  -24,  -24,    6,
			    3, -32768, yyDummy>>)
		end

	yypgoto_template: SPECIAL [INTEGER]
			-- Template for `yypgoto'
		once
			Result := yyfixed_array (<<
			 -32768, -32768,    8, -32768, yyDummy>>)
		end

	yytable_template: SPECIAL [INTEGER]
			-- Template for `yytable'
		once
			Result := yyfixed_array (<<
			    2,    8,   12,   21,   11,   10,   20,    9,    0,    6,
			    0,    9,    0,    0,    0,   14,   13,   15,   16,   17,
			   18,    0,    0,   12,    7,   11,   10,    0,    1,    0,
			    0,    0,    9, yyDummy>>)
		end

	yycheck_template: SPECIAL [INTEGER]
			-- Template for `yycheck'
		once
			Result := yyfixed_array (<<
			    7,   11,   22,    0,   24,   25,    0,   31,   -1,    1,
			   -1,   31,   -1,   -1,   -1,    7,   36,    9,   10,   11,
			   12,   -1,   -1,   22,   34,   24,   25,   -1,   35,   -1,
			   -1,   -1,   31, yyDummy>>)
		end

feature {NONE} -- Semantic value stacks

	yyvs1: SPECIAL [ANY]
			-- Stack for semantic values of type ANY

	yyvsc1: INTEGER
			-- Capacity of semantic value stack `yyvs1'

	yyvsp1: INTEGER
			-- Top of semantic value stack `yyvs1'

	yyspecial_routines1: KL_SPECIAL_ROUTINES [ANY]
			-- Routines that ought to be in SPECIAL [ANY]

	yyvs2: SPECIAL [like last_list]
			-- Stack for semantic values of type like last_list

	yyvsc2: INTEGER
			-- Capacity of semantic value stack `yyvs2'

	yyvsp2: INTEGER
			-- Top of semantic value stack `yyvs2'

	yyspecial_routines2: KL_SPECIAL_ROUTINES [like last_list]
			-- Routines that ought to be in SPECIAL [like last_list]

	yyvs3: SPECIAL [G]
			-- Stack for semantic values of type G

	yyvsc3: INTEGER
			-- Capacity of semantic value stack `yyvs3'

	yyvsp3: INTEGER
			-- Top of semantic value stack `yyvs3'

	yyspecial_routines3: KL_SPECIAL_ROUTINES [G]
			-- Routines that ought to be in SPECIAL [G]

	yyvs4: SPECIAL [STRING]
			-- Stack for semantic values of type STRING

	yyvsc4: INTEGER
			-- Capacity of semantic value stack `yyvs4'

	yyvsp4: INTEGER
			-- Top of semantic value stack `yyvs4'

	yyspecial_routines4: KL_SPECIAL_ROUTINES [STRING]
			-- Routines that ought to be in SPECIAL [STRING]

feature {NONE} -- Constants

	yyFinal: INTEGER = 21
			-- Termination state id

	yyFlag: INTEGER = -32768
			-- Most negative INTEGER

	yyNtbase: INTEGER = 37
			-- Number of tokens

	yyLast: INTEGER = 32
			-- Upper bound of `yytable' and `yycheck'

	yyMax_token: INTEGER = 277
			-- Maximum token id
			-- (upper bound of `yytranslate'.)

	yyNsyms: INTEGER = 41
			-- Number of symbols
			-- (terminal and nonterminal)

feature -- User-defined features



feature -- Error handling

	report_error (a_message: STRING) is
			-- Report error 'a_message'.
		do
			error.signal (generator,"$1 [near '$2', token '$3']",<<a_message,text,token_name (last_token)>>)
		end

feature {NONE} -- Implementation

	buffer_name: STRING
			-- Name of analyzed buffer

	add_expression (a_expression: G) is
			-- Add 'a_expressio' to 'last_list' of parsed expressions.
		do
			if last_list = Void then
				create last_list.make_empty
			end
			last_list.extend (a_expression)
			last_parsed := a_expression
		end

end
