indexing
	description: "Used by the Watcher.  This represents a node (with children) in the grid of port served applications."
	author: "RJ"
	last_editor: "$Author: rj $"
	date: "$Date: 2014/05/16 $"
	revision: "$Revision: #14 $"
	path: "$File: //rose/source/applications/system/data_connection_viewer/server_request_tree_node.e $"

class
	SERVER_REQUEST_TREE_NODE

inherit
	RP_GRID_NODE[SQL_SERVER_SYSPROCESS]
		rename
			make as rpgn_make
		end
	SQL_SERVER_SYSPROCESS_GUI
		rename
			title_node as child_title_node,
			grid_titles as child_grid_titles,
			string_titles as child_string_titles,
			should_right_justify as should_right_justify_child
		export
			{NONE} all
		end
	ROSE_STOCK_COLORS
		export
			{NONE} all
		end
	EXCEPTIONS
		export
			{NONE} all
		end

create
	make

feature -- Interface.
	make(
		a_server_request: SERVER_REQUEST;
		a_sysprocesses: ROSE_LINEAR_ARRAY[SQL_SERVER_SYSPROCESS];
		a_status_message: SERVER_STATUS_MESSAGE;
		a_window: EV_WINDOW;
--		a_show_blocker_detail_action: PROCEDURE [ANY, TUPLE];
--		a_show_blocker_detail_by_spid: PROCEDURE [ANY, TUPLE];
		a_sub_row_menu_agent: PROCEDURE [ANY, TUPLE[SQL_SERVER_SYSPROCESS, INTEGER, INTEGER, INTEGER, RP_DATA_GRID_ITEM]];
		a_refresh_selected_agent: PROCEDURE[ANY, TUPLE])
	-- The arguments:
	--	a_server_request				The data to be displayed at the top level.
	--	a_sysprocesses					The sysprocesses connected to the job specified in 'a_server_request'.
	--	a_status_message				A message from the port_server with run-time info (cpu, etc.).
	--	a_window						The window in which our dialogs will be 'modal' to.
	--	a_sub_row_menu_agent			Called when the user press MB3 on this row.
	--	a_refresh_selected_agent		Called when the user want to refresh this row (as opposed to all rows).
	local
		l_strings: ROSE_LINEAR_ARRAY[STRING]
		l_string: STRING
		l_label_item: RP_DATA_GRID_LABEL_ITEM
		l_label_items: ARRAY[RP_DATA_GRID_LABEL_ITEM]
		i: INTEGER
		l_color: EV_COLOR
		l_process: SQL_SERVER_SYSPROCESS
	do
		server_request := a_server_request
--		show_blocker_detail_action := a_show_blocker_detail_action
--		show_blocker_detail_by_spid_action := a_show_blocker_detail_by_spid
		sysprocesses := a_sysprocesses
		window := a_window

		from
			sysprocesses.start
		until
			sysprocesses.off
		loop
			l_process := sysprocesses.current_item
			if l_process.is_blocker
			then
				set_blocker
			elseif l_process.is_blockee then
				set_blockee
			end

			sysprocesses.forth
		end

		l_strings := server_request.string_array
		if a_status_message/=Void
		then
			l_strings.append_array(a_status_message.string_array)
		end

		if is_blocker and is_blockee
		then
			l_color := blockee_blocker_color
		elseif is_blocker
		then
			l_color := Blocker_color
		elseif is_blockee
		then
			l_color := Blockee_color
		else
			l_color := black
		end

		create l_label_items.make(1, l_strings.count)

		from
			i := 1
			l_strings.start
		until
			l_strings.off
		loop
			l_string := l_strings.current_item
			if l_string/=Void
			then
				create l_label_item.make_with_text(l_string)
				if should_right_justify.item(i)
				then
					l_label_item.align_text_right
				end

				l_label_item.set_foreground_color(l_color)
				l_string := l_string.twin
				l_string.prune(',')
				if l_string.is_double
				then
					l_label_item.set_comparable(l_string.to_double)
				else
					l_string.to_upper
					l_label_item.set_comparable(l_string)
				end

				l_label_items.put(l_label_item, i)
			end

			i := i + 1
			l_strings.forth
		end

		make_expandable_from_array(l_label_items, agent child_nodes(a_sub_row_menu_agent, a_refresh_selected_agent))
	ensure
		window_set: window /= Void
	end

	set_blocker
	-- See 'is_blocker'.
	do
		is_blocker := True
	end

	set_blockee
	-- See 'is_blockee'.
	do
		is_blockee := True
	end

	set_color_to(a_color: EV_COLOR) is
	-- Sets all columns in one row to 'a_color'.
	local
		i: INTEGER
		l_label_item: RP_DATA_GRID_LABEL_ITEM
	do
		from
			i := 1
		until
			i > column_count
		loop
			l_label_item ?= i_th(i)
			l_label_item.set_foreground_color(a_color)
			i := i + 1
		end
	end

	child_nodes(
		a_sub_row_menu_agent: PROCEDURE[ANY, TUPLE[SQL_SERVER_SYSPROCESS, INTEGER, INTEGER, INTEGER, RP_DATA_GRID_ITEM]];
		a_refresh_selected_agent: PROCEDURE[ANY, TUPLE]): RP_TITLED_NODES[SQL_SERVER_SYSPROCESS]
	-- Returns the child nodes of this node.  Used as an expand agent (other users, too).
	local
		l_node: RP_GRID_NODE[SQL_SERVER_SYSPROCESS]
		l_nodes: ROSE_LINEAR_ARRAY[RP_GRID_NODE[SQL_SERVER_SYSPROCESS]]
		l_process: SQL_SERVER_SYSPROCESS
		l_color: EV_COLOR
	do
		create l_nodes.make_empty

		from
			sysprocesses.start
		until
			sysprocesses.off
		loop
			l_process := sysprocesses.current_item
			if l_process.is_blocker
			then
				l_color := Blocker_color
			elseif l_process.is_blockee then
				l_color := Blockee_color
			else
				l_color := black
			end

			create l_node.make_from_array(child_grid_items(l_process, l_color))
			l_node.set_data(sysprocesses.current_item)
			if a_sub_row_menu_agent/=Void
			then
				l_node.pointer_button_press_actions.extend(agent call_a_sub_row_menu_agent(a_sub_row_menu_agent, sysprocesses.current_item, ?, ?, ?, ?))
			end

			if a_refresh_selected_agent/=Void
			then
				l_node.pointer_double_press_actions.extend(agent call_refresh_selected(a_refresh_selected_agent, ?, ?, ?, ?))
			end

			l_nodes.extend(l_node)

			sysprocesses.forth
		end

		if is_persistent
		then
			create l_node.make_from_array(<<"Clients">>)
			l_node.set_expand_agent(agent client_child_nodes)
			l_node.collapse_actions.extend(agent remove_subnodes(l_node))
			l_nodes.extend(l_node)
		end

		if l_nodes.count/=0
		then
			create Result.make_from_array(child_title_node, l_nodes)
		end

		from
			sysprocesses.start
			l_nodes.start
		until
			sysprocesses.off
		loop
			l_process := sysprocesses.current_item
			l_node := l_nodes.current_item
--			l_node.set_background_color(Result.titles.background_color)
			if l_process.is_blocker
			then
--				l_node.select_actions.extend(agent show_blocker_detail(l_process))
----				l_node.i_th(1).select_actions.extend(agent show_blocker_detail(l_process))
			elseif l_process.is_blockee then
--				l_node.select_actions.extend(
--					agent show_blocker_detail_by_id(l_process.parent.name, l_process.blocked))
----				l_node.i_th(1).select_actions.extend(
----					agent show_blocker_detail_by_id(l_process.parent.name, l_process.blocked))
			end

			sysprocesses.forth
			l_nodes.forth
		end
	end

	remove_subnodes(a_node: RP_GRID_NODE[ANY]) is
	-- Removes all of the subnodes from this node.
	do
		if a_node.row.subrow_count_recursive>0 and not gui.ev_application.ctrl_pressed
		then
			grid.remove_rows(a_node.row.index + 1, a_node.row.index + a_node.row.subrow_count_recursive)
			a_node.row.ensure_expandable
		end
	end

	highlight_if(a_sysprocess: SQL_SERVER_SYSPROCESS) is
	-- Changes the background color of this row if this node contains a blocker of 'a_sysprocess'.
	require
		a_sysprocess_not_void: a_sysprocess/=Void
	local
		l_done: BOOLEAN
		i: INTEGER
		l_label_item: RP_DATA_GRID_LABEL_ITEM
	do
		from
			sysprocesses.start
		until
			sysprocesses.off or l_done
		loop

			if equal(a_sysprocess.hostname, sysprocesses.current_item.hostname) and
			   equal(a_sysprocess.hostprocess, sysprocesses.current_item.hostprocess)
			then
				l_done := True
				from
					i := 1
				until
					i > column_count
				loop
					l_label_item ?= i_th(i)
					l_label_item.set_background_color(blocking_process_background_color)
					i := i + 1
				end
			end

			sysprocesses.forth
		end
	end

	custom_monitor_response(a_message: ARRAY[ANY]): ARRAY[ANY]
	-- The first element of the array should be a string.  We will send it to the server and return the response.
	local
		i: INTEGER
		l_response: ARRAY[ANY]
		l_text_response: STRING
		l_text_item: STRING
		l_dialog: ROSE_DIALOG
		l_text: EV_TEXT
		l_wait_started: BOOLEAN
	do
		if monitor=Void
		then
			create_the_monitor
		end

		l_wait_started := True
		gui.start_wait_msg("Timeout = " + monitor.timeout.out + " seconds.")
		l_response := monitor.custom_monitor_response(a_message)
		gui.end_wait_msg(Void)
		l_wait_started := False

		if l_response/=Void
		then
			from
				l_text_response := ""
				i := l_response.lower
			until
				i>l_response.upper
			loop
				l_text_item ?= l_response[i]
				if l_text_item/=Void
				then
					if l_text_response.count>0
					then
						l_text_response.append("%N")
					end

					l_text_response.append(l_text_item)
				end

				i := i + 1
			end
		else
			-- Something bad probably happened.
			l_text_response := "No response was received!"
			monitor.disconnect
			monitor := Void
		end

		create l_text
		l_text.set_text(l_text_response)

		create l_dialog
		l_dialog.set_title("Response")
		l_dialog.set_size(400, 200)
		l_dialog.main_box.extend(l_text)
		l_dialog.close_request_actions.extend(agent l_dialog.hide)

		l_dialog.show_modal_to_window(window)
	rescue
		if l_wait_started
		then
			gui.end_wait
		end
	end

	is_persistent: BOOLEAN
	-- Is this server a persistent server?
	local
		l_command: STRING
	do
		if server_request/=Void
		then
			l_command := server_request.command
			if l_command/=Void and then
			   (l_command.as_upper.has_substring("PERSISTENT") or l_command.as_upper.has_substring("RTP_SNAPSHOT_SERVER"))
			then
				Result := True
			end
		end
	end

feature {NONE} -- Implementation.
	create_the_monitor
	-- The 'monitor' allows us to send 'monitor' commands to the server.
	local
		l_env: EXECUTION_ENVIRONMENT
		l_saved_persistent_server_list: STRING
	do
		create l_env
		l_saved_persistent_server_list := l_env.get("PERSISTENT_SERVER_LIST")
		l_env.put(server_request.machine, "PERSISTENT_SERVER_LIST")
		l_env.put(server_request.port.out, server_request.command)
		create monitor.make(server_request.command, Void, "")
		monitor.set_timeout(30)
		if l_saved_persistent_server_list=Void
		then
			l_env.put("", "PERSISTENT_SERVER_LIST")		-- We really need to delete the variable - but how?
		else
			l_env.put(l_saved_persistent_server_list, "PERSISTENT_SERVER_LIST")
		end
	end

	client_child_nodes: RP_TITLED_NODES[SQL_SERVER_SYSPROCESS]
	-- This 'expand agent' is called when the user clicks the plus on the 'Clients' rows.
	-- This returns all of the clients of a single persistent server.
	local
		l_client: PERSISTENT_SERVER_COMMAND
		l_client_list: LINKED_LIST[PERSISTENT_SERVER_COMMAND]
		l_types: PERSISTENT_SERVER_COMMAND_TYPES
		l_node: RP_GRID_NODE[SQL_SERVER_SYSPROCESS]
		l_nodes: ROSE_LINEAR_ARRAY[RP_GRID_NODE[SQL_SERVER_SYSPROCESS]]
		l_item: RP_DATA_GRID_LABEL_ITEM
		l_items: ROSE_LINEAR_ARRAY[RP_DATA_GRID_LABEL_ITEM]
		l_monitor_count: INTEGER
		l_retry_allowed: BOOLEAN
		l_collecting: BOOLEAN
		l_retry: BOOLEAN
		l_wait_started: BOOLEAN
	do
		if l_retry
		then
			create l_nodes.make_empty
			create l_node.make_from_array(<<"Failed!">>)
			l_nodes.extend(l_node)
			create Result.make_from_array(client_titles, l_nodes)
			monitor := Void
		else
			l_collecting := (create {MEMORY}).collecting
			create l_types
			create l_nodes.make_empty
			if monitor=Void
			then
				l_retry_allowed := True
				create_the_monitor
				l_retry_allowed := False
			end

			l_wait_started := True
			gui.start_wait_msg("Timeout = " + monitor.timeout.out + " seconds.")
			l_client_list := monitor.client_list
			gui.end_wait_msg(Void)
			l_wait_started := False

			if l_client_list=Void
			then
				l_retry_allowed := True
				raise("Something bad happened")
			else
				from
					l_client_list.start
				until
					l_client_list.off
				loop
					l_client := l_client_list.item
					if l_client.client_type=l_types.monitor_client_type_id
					then
						l_monitor_count := l_monitor_count + 1
					else
						create l_items.make(1, 6)

						-- User name.
						if l_client.user_name/=Void
						then
							create l_item.make_with_text(l_client.user_name)
							l_item.set_comparable(l_client.user_name.as_upper)
						else
							create l_item.make_with_text("")
						end

						l_items.put(l_item, 1)

						-- Manchine name.
						if l_client.machine_name/=Void
						then
							create l_item.make_with_text(l_client.machine_name)
							l_item.set_comparable(l_client.machine_name.as_upper)
						else
							create l_item.make_with_text("")
						end

						l_items.put(l_item, 2)

						-- Client PID.
						create l_item.make_with_text(l_client.client_pid.out)
						l_item.set_comparable(l_client.client_pid)
						l_item.align_text_right
						l_items.put(l_item, 3)

						-- Start time.
						create l_item.make_with_text(l_client.start_time.formatted_out(time_code))
						l_items.put(l_item, 4)

						-- Last time.
						create l_item.make_with_text(l_client.last_message_time.formatted_out(time_code))
						l_items.put(l_item, 5)

						-- Number of messages.
						create l_item.make_with_text(l_client.number_of_messages.out)
						l_item.set_comparable(l_client.number_of_messages)
						l_item.align_text_right
						l_items.put(l_item, 6)

						-- Build the node.
						create l_node.make_from_array(l_items)
						l_node.pointer_double_press_actions.extend(agent refresh_clients(?, ?, ?, ?))
						l_nodes.extend(l_node)
					end

					l_client_list.forth
				end

				if l_nodes.count>0 or l_monitor_count>1		-- Only if there are Data clients or a Monitor client besides us.
				then
					if l_monitor_count>1
					then
						-- Create the final row which contains the number of Monitor clients.
						create l_items.make(1, 6)

						create l_item.make_with_text("Monitors")
						l_items.put(l_item, 1)

						create l_item.make_with_text(l_monitor_count.out)
						l_item.align_text_right
						l_items.put(l_item, 6)

						create l_node.make_from_array(l_items)
						l_nodes.extend(l_node)
					end

					create Result.make_from_array(client_titles, l_nodes)
				end
			end
		end
	rescue
		if l_collecting
		then
			(create {MEMORY}).collection_on
		end

		if l_wait_started
		then
			gui.end_wait
		end

		if l_retry_allowed
		then
			l_retry_allowed := False
			l_retry := True
			retry
		end
	end

	call_a_sub_row_menu_agent(
		a_sub_row_menu_agent: PROCEDURE[ANY, TUPLE[]];
		a_sysprocess: SQL_SERVER_SYSPROCESS; a_x, a_y, a_button: INTEGER; a_item: RP_DATA_GRID_ITEM) is
	-- Calls the agent to handle a pointer button press action on a child of this node.
	do
		a_sub_row_menu_agent.call([a_sysprocess, a_x, a_y, a_button, a_item])
	end

	call_refresh_selected(
		a_refresh_selected_agent: PROCEDURE[ANY, TUPLE[]];
		a_x, a_y, a_button: INTEGER; a_item: RP_DATA_GRID_ITEM) is
	-- Calls the agent to handle a pointer double press action on a child of this node.
	do
		a_refresh_selected_agent.call([])
	end

	refresh_clients(a_x, a_y, a_button: INTEGER; a_item: RP_DATA_GRID_ITEM) is
	-- Called when the client double-clicks a row.
	local
		l_item: RP_DATA_GRID_LABEL_ITEM
		l_parent_row: RP_GRID_NODE[ANY]
	do
		l_item ?= a_item
		l_parent_row := l_item.node.parent_row
		l_parent_row.collapse
		l_parent_row.expand
	end

	should_right_justify: ARRAY[BOOLEAN] is
	-- True if a PARENT column should be right justified.
	once
		Result :=
			<<
				False,			-- "machine"
				False,			-- "server_executable"
				False,			-- "user_id"
				False,			-- "client"
				False,			-- "arguments"
				False,			-- "command"
				True,			-- "server_pid"
				True,			-- "client_pid"
				True,			-- "port"
				False,			-- "log_file"
				False,			-- "timestamp"
				False,			-- "terminated_by"
				False,			-- "terminated_at"
				True,			-- "workingset"
				True,			-- "pagefile"
				True,			-- "user time"
				True			-- "kernel time"
			>>
	end

	child_grid_items(a_process: SQL_SERVER_SYSPROCESS; a_color: EV_COLOR): ARRAY[RP_DATA_GRID_LABEL_ITEM] is
	-- Returns all of the items needed to make one child row.
	local
		i: INTEGER
		l_item: RP_DATA_GRID_LABEL_ITEM
		l_string_array: ROSE_LINEAR_ARRAY[TUPLE[text: STRING; comparable: COMPARABLE]]
	do
		l_string_array := a_process.string_array
		create Result.make(1, l_string_array.count)
		from
			i := 1
		until
			i>Result.count
		loop
			create l_item.make_with_text(l_string_array[i].text)
			l_item.set_foreground_color(a_color)
			l_item.set_background_color(create {EV_COLOR}.make_with_8_bit_rgb(255, 255, 192))

			if l_string_array[i].comparable = Void
			then
				l_item.set_comparable(l_string_array[i].text.as_upper)
			else
				l_item.set_comparable(l_string_array[i].comparable)
			end

			if should_right_justify_child.item(i)
			then
				l_item.align_text_right
			end

			Result.put(l_item, i)
			i := i + 1
		end
	end

	client_titles: RP_GRID_TITLE_NODE[SQL_SERVER_SYSPROCESS]
	-- Header titles for the part of the grid that displays clients of persistent servers.
	local
		l_item: RP_DATA_GRID_LABEL_ITEM
		l_grid_titles: ARRAY[RP_DATA_GRID_LABEL_ITEM]
	do
		-- You can't make this a 'once' because each set of titles must be different objects.
		create l_grid_titles.make(1, 6)
		l_grid_titles.put(create {RP_DATA_GRID_LABEL_ITEM}.make_with_text("user_id"), 1)

		l_grid_titles.put(create {RP_DATA_GRID_LABEL_ITEM}.make_with_text("machine"), 2)

		create l_item.make_with_text("pid")
		l_item.align_text_right
		l_grid_titles.put(l_item, 3)

		create l_item.make_with_text("start_time")
		l_grid_titles.put(l_item, 4)

		create l_item.make_with_text("last_time")
		l_grid_titles.put(l_item, 5)

		create l_item.make_with_text("messages")
		l_item.align_text_right
		l_grid_titles.put(l_item, 6)

		create Result.make(l_grid_titles)
	end

--	show_blocker_detail(a_process: SQL_SERVER_SYSPROCESS) is
--	do
--		show_blocker_detail_action.call([a_process])
--	end

--	show_blocker_detail_by_id(a_name: STRING; a_blocker_spid: INTEGER) is
--	do
--		show_blocker_detail_by_spid_action.call([a_name, a_blocker_spid])
--	end


feature {NONE} -- Once functions.
	blocker_color: EV_COLOR is
	-- Used by the node or child to indicate a blocker.
	once
		create Result.make_with_8_bit_rgb(255, 0, 0)
	end

	blockee_color: EV_COLOR is
	-- Used by the node or child to indicate a blockee.
	once
		create Result.make_with_8_bit_rgb(0, 0, 255)
	end

	blockee_blocker_color: EV_COLOR is
	-- Color used if a port served app contains both a blocker and a blockee.
	once
		create Result.make_with_8_bit_rgb(255, 0, 255)
	end

	blocking_process_background_color: EV_COLOR is
	-- Background color used if a port served app contains the selected blocker.
	once
		create Result.make_with_8_bit_rgb(255, 255, 177)
	end

	time_code: STRING is "yyyy/[0]mm/[0]dd [0]hh:[0]mi:[0]ss"	-- Used for 'start_time' and 'last_time'.

feature {RCMD_DATA_CONNECTION_VIEWER} -- Attributes.
	server_request: SERVER_REQUEST								-- Record recorded by the port server when this server was started.

feature {NONE} -- Attributes
	is_blocker: BOOLEAN											-- This node is a blocker.

	is_blockee: BOOLEAN											-- This node is a blockee.

	monitor: PERSISTENT_CLUSTERED_TAGGED_SERVER_MONITOR			-- Connection used to monitor this server if it is persistent.

--	show_blocker_detail_action: PROCEDURE [ANY, TUPLE]

--	show_blocker_detail_by_spid_action: PROCEDURE [ANY, TUPLE]

	sysprocesses: ROSE_LINEAR_ARRAY[SQL_SERVER_SYSPROCESS]		-- Data from a 'select * from master..sysprocesses'.

	window: EV_WINDOW											-- The parent window (so we can show modal dialogs).

end
