note

	description:

		"Access Currency Signature"

	author: "Bruno Pradal"
	last_editor: "$Author: rj $"
	date: "$Date: 2014/12/12 $"
	revision: "$Revision: #20 $"
	path: "$File: //rose/source/kernel/market/signature_tagged_currency_market.e $"

deferred class SIGNATURE_TAGGED_CURRENCY_MARKET

inherit
	MARKET
	ROSE_AWARE
	ROSE_IEEE_UNDEFINED

feature -- Access

	exchange_rate_asof (a_currency_1, a_currency_2: CURRENCY; a_date: ROSE_DATE; a_asof: detachable ROSE_DATE): DOUBLE
			-- <Precursor>
		local
			l_rate_1, l_rate_2, l_rate: EXCHANGE_RATE
			l_factor_1, l_factor_2: DOUBLE
		do
			if a_currency_1.same_currency (a_currency_2) then
				Result := 1
			elseif a_currency_1.iso /= Void and then a_currency_2.iso /= Void and then
				   a_currency_1.iso.substring(1, 2) ~ a_currency_2.iso.substring(1, 2) and then
				   has_potential_factor_change (a_currency_1)									-- Call this last, it's pretty slow.
			then
				Result := 1
			else
				l_rate_1 := rate (a_currency_1, a_date, a_asof)
				if l_rate_1 /= Void
				then
					l_rate_2 := rate (a_currency_2, a_date, a_asof)
					if l_rate_2 /= Void
					then
						l_rate := (l_rate_1 / l_rate_2)
						Result := l_rate.rate
					else
						Result := undefined_double
					end
				else
					Result := undefined_double
				end
			end
			if is_defined_double (Result) then
				l_factor_1 := get_factor (a_currency_1, a_date, False) / get_factor (a_currency_1, a_date, True)
				l_factor_2 := get_factor (a_currency_2, a_date, False) / get_factor (a_currency_2, a_date, True)
				Result := Result * l_factor_1 / l_factor_2
			end
		end

feature -- Deferred

	base_currency (a_date: ROSE_DATE): CURRENCY
			--base currency on a_date
		deferred
		end

	is_valid_record (a_record: ROSE_VIRTUAL_RECORD [FOREX_SIGNATURE]): BOOLEAN
			-- Does `a_record' have sufficient details for `actual_record'?
		deferred
		end

	actual_rate (a_record: ROSE_VIRTUAL_RECORD [FOREX_SIGNATURE]): DOUBLE
			-- Rate from `a_record'
		require
			a_record_attached: a_record /= Void
			valid_record: is_valid_record (a_record)
		deferred
		end

	source: STRING
			-- Supplier name
		deferred
		ensure
			attached_result: Result /= Void
		end
feature -- Basic operations

	reset_xrate_cache is
			-- Clear-out `xrate_cache'.
		do
			xrate_cache.make (1_000)
		end

feature {NONE} -- Implementation

	rate (a_currency: CURRENCY; a_date: ROSE_DATE; a_asof: detachable ROSE_DATE): EXCHANGE_RATE
			-- Rate for `a_currency' on `a_date' as of `a_asof' ;
			-- Base currency depends on date and is determined here.
		require
			valid_currency: a_currency /= Void
			a_date_attached: a_date /= Void
		local
			l_double: DOUBLE
			l_currency_signature: FOREX_SIGNATURE
			l_currency_record: ROSE_VIRTUAL_RECORD [FOREX_SIGNATURE]
			l_currency_signatures: ROSE_LINEAR_ARRAY [FOREX_SIGNATURE]
			l_base_currency: CURRENCY
			l_asof_2: detachable ROSE_DATE
		do
			l_double := undefined_double
			l_base_currency := base_currency (a_date) -- should this be sensitive to `a_asof'?
			if a_currency.same_currency (l_base_currency) then
				l_double := 1
			else
				xrate_cache.search ([a_currency.handle, source, l_base_currency.handle, a_date, a_asof])
				if xrate_cache.found then
					l_currency_record := xrate_cache.found_item
				else
					create l_currency_signature
					l_currency_signature.set_currency_id (a_currency.handle)
					l_currency_signature.set_source (source)
					l_currency_signature.set_base_currency_id (l_base_currency.handle)
					l_currency_signature.set_date (a_date)
					if attached a_asof as l_asof then
						l_currency_signature.set_as_of_date (l_asof)
						l_currency_signatures := l_currency_signature.data_manager.retrieve_historical_signatures (
							create {ROSE_LINEAR_ARRAY [FOREX_SIGNATURE]}.make_from_array (<<l_currency_signature>>), Void)
						l_currency_signature := signature_as_of (l_currency_signatures, l_asof)
					end
					if l_currency_signature /= Void then
						l_currency_record := l_currency_signature.data_manager.record_by_signature (l_currency_signature)
					end
					l_asof_2 := cloned_asof (a_asof)
					if l_currency_record /= Void then
						xrate_cache.force (l_currency_record.twin, [a_currency.handle, source, l_base_currency.handle, a_date.twin, l_asof_2])
					else
						xrate_cache.force (Void, [a_currency.handle, source, l_base_currency.handle, a_date.twin, l_asof_2])
					end
				end
				if is_valid_record (l_currency_record) then
					l_double := actual_rate (l_currency_record)
				end
			end
			if is_defined_double (l_double) then
				create Result.make (a_currency, l_base_currency, l_double, a_date)
			end
		end

	cloned_asof (a_asof: detachable ROSE_DATE): detachable ROSE_DATE
			-- Copy of `a_asof' (if not Void)
		do
			if attached a_asof as l_asof then
				Result := l_asof.twin
			end
		end

	signature_as_of (a_currency_signatures: ROSE_LINEAR_ARRAY [FOREX_SIGNATURE]; a_asof: ROSE_DATE): detachable FOREX_SIGNATURE
			-- Latest signature from `a_currency_signatures' whose timestamp <= `a_asof'
		require
			a_currency_signatures_attached: a_currency_signatures /= Void
			no_void_signature: not a_currency_signatures.has (({FOREX_SIGNATURE}).default)
			no_void_asof: across a_currency_signatures as i_sig all attached i_sig.item.as_of_date end
			a_asof_attached: a_asof /= Void
		local
			l_signature: FOREX_SIGNATURE
		do
			from
				a_currency_signatures.start
			invariant
				signature_has_as_of_date: attached Result as l_result implies attached l_result.as_of_date
			until
				a_currency_signatures.after
			loop
				l_signature := a_currency_signatures.current_item
				if l_signature.as_of_date <= a_asof then
					if Result = Void then
						Result := l_signature
					elseif l_signature.as_of_date > Result.as_of_date then
						Result := l_signature
					end
				end
				a_currency_signatures.forth
			end
		ensure
			signature_has_as_of_date: attached Result as l_result implies attached l_result.as_of_date
		end

	xrate_cache: TUPLE_KEYED_HASH_TABLE [ROSE_VIRTUAL_RECORD [FOREX_SIGNATURE], TUPLE [currency_handle: INTEGER; source_string: STRING; base_currency_handle: INTEGER; date: ROSE_DATE; asof: detachable ROSE_DATE]]
			-- Cache for xrate data to avoid running through the data manager
		once
			create Result.make (1_000)
		ensure
			attached_result: Result /= Void
		end

	exchange_rate_server: SQL_DATA_SERVER is
			-- Data source for xrate data
		once
			Result := context.servers.rose_foreign_exchange_server
		end

	get_factor (a_currency: CURRENCY; a_date: ROSE_DATE; a_exact: BOOLEAN): DOUBLE is
			-- Currency quoting factor, this changes periodically for emerging markets (ie. Turkey, Poland, Russia)
		local
			l_record: FX_CHANGE_RECORD
			l_iso, l_country_iso: STRING
			l_today: ROSE_DATE
			i: INTEGER
			l_records: ROSE_LINEAR_ARRAY [FX_CHANGE_RECORD]
		do
			Result := 1.0
			if has_potential_factor_change (a_currency) then
				if factor_history_hash = Void then
					create factor_history_hash.make (15)
				end
				create l_today.make_today
				l_country_iso := a_currency.iso.substring (1, 2)
				if a_exact then
					l_iso := a_currency.iso
				else
					l_iso := l_country_iso
				end
				if not factor_history_hash.has (l_country_iso) then
					fx_change_predicate.retrieve (Void, <<a_currency.iso.substring (1, 2) + "%%">>)
					if not fx_change_predicate.success then
						(create {EXCEPTIONS}).raise ("Error retrieving data from FX_Change")
					elseif not fx_change_predicate.array.is_empty then
						factor_history_hash.force (fx_change_predicate.array, l_country_iso)
						from
							i := 1
						until
							i > fx_change_predicate.array.count
						loop
							l_record := fx_change_predicate.array.item (i)
							create l_records.make_empty
							l_records.extend (l_record)
							factor_history_hash.force (l_records, l_record.iso_old)
							i := i + 1
						end
					end
				end
				factor_history_hash.search (l_iso)
				if factor_history_hash.found then
					l_records := factor_history_hash.found_item
					from
						i := 1
					until
						i > l_records.count
					loop
						l_record := l_records.item (i)
						if l_record.end_date.date >= a_date and l_record.end_date.date < l_today then
							Result := Result * l_record.value_new / l_record.value_old
						end
						i := i + 1
					end
				end
				if Result <= 0.0 then
					(create {EXCEPTIONS}).raise ("Invalid factor change")
				end
			end
		ensure
			valid_result: Result > 0.0
		end

	fx_change_predicate: COLLECTING_PREDICATE [FX_CHANGE_RECORD] is
			-- Predicate to retrieve factor change date information
		once
			create Result.make (exchange_rate_server, "select * from fx_change where ISO_old like ?")
			if not Result.success then
				print ("Error creating fx_change predicate%N")
			end
		end

	factor_history_hash: HASH_TABLE [ROSE_LINEAR_ARRAY [FX_CHANGE_RECORD], STRING]
			-- Cache the result for both the country(ISO first two character) and the currency 	

	usd: CURRENCY is
			-- US Dollar
		once
			Result := context.usd
		end

	gbp: CURRENCY is
			-- GB Pound
		once
			Result := context.gbp
		end

	jpy: CURRENCY is
			-- Japanese Yen
		once
			Result := context.jpy
		end

	eur: CURRENCY is
			-- Euro
		once
			Result := context.eur
		end

	cad: CURRENCY is
			-- Canadian dollar
		once
			Result := context.cad
		end

	has_potential_factor_change (a_currency: CURRENCY): BOOLEAN is
			-- needed in order to spare the querying of the database in all the cases
			-- and get reasonable performance
		local
			l_country: COUNTRY
		do
			if a_currency /= Void and then a_currency /~ usd and then a_currency /~ gbp and then a_currency /~ jpy and then a_currency /~ eur and then a_currency /~ cad then
				l_country := a_currency.something_corelated_by_country_to_currency
				if l_country /= Void and then l_country.zone_mnemonic.is_equal ('S') then
					Result := True
				elseif a_currency.iso /= Void and then a_currency.iso.count >= 2 then
					inspect a_currency.iso.item (1)
					when 'T' then
						Result := a_currency.iso.substring (1, 2).is_equal ("TR")
					when 'R' then
						Result := a_currency.iso.substring (1, 2).is_equal ("RU")
					when 'P' then
						Result := a_currency.iso.substring (1, 2).is_equal ("PL")
					else
						do_nothing
					end
				end
			end
		end

end










