note

	description:

		"An expression to retrieve data from signature tagged databases"

	author: "Mauricio Bustos"
	last_editor: "$Author: er $"
	date: "$Date: 2014/01/03 $"
	revision: "$Revision: #83 $"
	path: "$File: //rose/source/kernel/data/reports/reference/signature_tagged_expression.e $"

class
	SIGNATURE_TAGGED_EXPRESSION [reference G -> ROSE_SIGNATURE_RECORD [ENTITY] create default_create end]

inherit
	ROSE_ENTITY_DATE_EXPRESSION [G]
		redefine
			make, integer_value, real_value, double_value, string_value,date_value
		select
			make
		end

	ROSE_EXPRESSION [G]
		rename
			make as old_make
		undefine
			string_value,
			date_value,
			integer_value,
			real_value, double_value
		end

	ROSE_IEEE_UNDEFINED
		undefine default_create end

create
	make, default_create

feature {NONE}-- Initialization

	make (a_variable_description: like variable_description) is
			-- Create the expression
		local
			l_ur_clustered_signature: CLUSTERED_SIGNATURE [COMPARABLE, COMPARABLE, ENTITY]
		do
			Precursor (a_variable_description)
			create ur_signature
			set_signature_field
			l_ur_clustered_signature ?= ur_signature
			is_clustered_signature := (l_ur_clustered_signature /= Void)
		end

feature -- Basic operations

	signature_anchor:  like expression_target is do end

    ur_signature: like signature_anchor

	currency (a_target: like signature_anchor): PERSISTENT_CURRENCY is
			-- currency
		do
			if a_target /= Void then
				Result := a_target.rose_currency
			end
		end

	ad_hoc: BOOLEAN
	set_ad_hoc (a_value: like ad_hoc) is
			-- Set 'ad_hoc' to 'a_value'
		do
			ad_hoc := a_value
		end

	periodicity_id: INTEGER
	set_periodicity_id(a_id: like periodicity_id) is
			-- set periodicity id
		do
			periodicity_id := a_id
		end

feature -- Expression caching management

	is_clustered_signature: BOOLEAN

	set_expression_cache_size (a_count, a_chunk_size: INTEGER) is
			-- set expression cache size
		local
			l_parameter: PAIR [INTEGER, INTEGER]
		do
			create l_parameter.make (a_count, a_chunk_size)
			expression_cache_size_table.force (l_parameter, expression_cache_key)
		end

	refresh_global_caches is
			--clear caches
		do
			clear_local_caches
			global_cache_version.put (global_cache_version.item + 1)
		end

feature {NONE} -- Expression cache management

	expression_cache_key: STRING is
			--expression cache key
		do
			if ur_signature = Void then
				create ur_signature
			end
			Result := ur_signature.generating_type
		end

	expression_cache_saved: SIGNATURE_TAGGED_EXPRESSION_CACHE [like signature_anchor]

	expression_cache: SIGNATURE_TAGGED_EXPRESSION_CACHE [like signature_anchor] is
		local
			l_cache: SIGNATURE_TAGGED_EXPRESSION_CACHE [ROSE_SIGNATURE_RECORD [ENTITY]]
		do
			if  local_cache_version < global_cache_version.item then
				clear_last_references
				local_cache_version := global_cache_version.item
			end
			if expression_cache_saved = Void then
				expression_cache_table.search (expression_cache_key)
				if expression_cache_table.found then
					expression_cache_saved ?= expression_cache_table.found_item
				else
					-- Check if there is a customized cache size set for this expression cache key
					expression_cache_size_table.search (expression_cache_key)
					if expression_cache_size_table.found then
						create expression_cache_saved.make (expression_cache_key, expression_cache_size_table.found_item.first,
															expression_cache_size_table.found_item.second)
					elseif ur_signature.expression_cache_size /= Void then
						create expression_cache_saved.make (expression_cache_key, ur_signature.expression_cache_size.first,
															ur_signature.expression_cache_size.second)
					else
						create expression_cache_saved.make (expression_cache_key, 0, 0)
					end
					l_cache ?= expression_cache_saved
					expression_cache_table.extend (l_cache, expression_cache_key)
				end
				set_record_name
			end
			Result := expression_cache_saved
		end

	expression_cache_table: HASH_TABLE [SIGNATURE_TAGGED_EXPRESSION_CACHE [ROSE_SIGNATURE_RECORD [ENTITY]], STRING] is
		once
			create Result.make (8)
		end

	expression_cache_size_table: HASH_TABLE [PAIR [INTEGER, INTEGER], STRING] is
			-- PAIR [COUNT, CHUNK_SIZE]
		once
			create Result.make (8)
		end

	expression_cache_item (a_target: like signature_anchor): ROSE_VIRTUAL_RECORD_HISTORY [like signature_anchor] is
		do
			Result := expression_cache.virtual_record_cache_by_key.item (a_target.multi_key)
		end

	expression_cache_neighborhood (a_target: like signature_anchor): ROSE_LINEAR_ARRAY [like signature_anchor] is
		do
			create Result.make (1, 1)
			Result.put (a_target, 1)
		end

	global_cache_version: CELL [INTEGER] is
		once
			create Result.put (0)
		end

	local_cache_version: INTEGER

	clear_last_references is
			-- clear last references
		local
			l_void_target: detachable G
		do
			last_target := l_void_target
			last_date := Void
			last_double := 0
			last_image := Void
			last_integer := 0
			last_real := 0
			last_string := Void
		end

feature -- need this to be exported in order to view and compute expressions based on uncomitted data

	clear_local_caches is
			-- clear local caches
		do
			clear_last_references
			from expression_cache_table.start
			until expression_cache_table.after
			loop
				expression_cache_table.item_for_iteration.wipe_out
				expression_cache_table.forth
			end
		end

	expression_cache_force (a_record_history: ROSE_VIRTUAL_RECORD_HISTORY [like signature_anchor]; a_target: like signature_anchor) is
		do
			expression_cache.virtual_record_cache_by_key.force (a_record_history, a_target.multi_key)
		end

feature {NONE} -- Record retrieval

	retrieve_virtual_record (a_target: like signature_anchor): ROSE_VIRTUAL_RECORD [like signature_anchor] is
			-- Record for 'a_target'
		local
			l_as_of_date: ROSE_DATE
			l_historical: BOOLEAN
			l_last_signature: like signature_anchor
			l_records: ROSE_LINEAR_ARRAY [ROSE_VIRTUAL_RECORD [like signature_anchor]]
			l_record_history: ROSE_VIRTUAL_RECORD_HISTORY [like signature_anchor]
			l_clustered_db: CLUSTERED_TAGGED_ITEM_DATABASE[COMPARABLE, COMPARABLE, CLUSTERED_SIGNATURE [COMPARABLE, COMPARABLE, ENTITY]]
		do
			if not a_target.latest_version_retrieval_on then
				l_as_of_date := a_target.as_of_date
			end
			l_historical := (l_as_of_date /= Void)
			l_clustered_db ?= a_target.data_manager
			if l_clustered_db /= Void then
				if l_historical then
					l_last_signature := expression_cache.last_virtual_record.first
					if l_last_signature /= Void and then a_target = l_last_signature then
						Result := expression_cache.last_virtual_record.second
					else
						l_record_history := expression_cache_item (a_target)
						if l_record_history = Void then
							l_records := a_target.data_manager.historical_records_by_signature (a_target, restriction_statement, Void)
							if l_records /= Void then
								load_virtual_records_to_cache (l_records, l_historical, True)
								l_record_history := expression_cache_item (a_target)
							end
						end
						if l_record_history /= Void and then (l_historical implies l_record_history.historical) then
							Result := l_record_history.item (l_as_of_date)
						end
						expression_cache.set_last_virtual_record (a_target, Result)
					end
				else
					if ad_hoc and then {l_clustered_target: CLUSTERED_SIGNATURE [COMPARABLE, COMPARABLE, ENTITY]} a_target then
						Result ?= l_clustered_db.ad_hoc_record_by_signature (l_clustered_target)
					else
						Result := a_target.data_manager.record_by_signature (a_target)
					end
				end
			else
				l_last_signature := expression_cache.last_virtual_record.first
				if l_last_signature /= Void and then a_target = l_last_signature then
					Result := expression_cache.last_virtual_record.second
				else
					l_record_history := expression_cache_item (a_target)
					if l_record_history /= Void and then (l_historical implies l_record_history.historical) then
						Result := l_record_history.item (l_as_of_date)
					else
						Result := retrieve_virtual_record_from_external (a_target)
					end
					expression_cache.set_last_virtual_record (a_target, Result)
				end
			end
		end

	retrieve_virtual_record_from_external (a_target: like signature_anchor): ROSE_VIRTUAL_RECORD [like signature_anchor] is
		local
			l_as_of_date: ROSE_DATE
			l_historical: BOOLEAN
			l_record_history: ROSE_VIRTUAL_RECORD_HISTORY [like signature_anchor]
		do
			if not a_target.latest_version_retrieval_on then
				l_as_of_date := a_target.as_of_date
			end
			l_historical := (l_as_of_date /= Void)
			cache_virtual_records (expression_cache_neighborhood (a_target), l_historical).do_nothing
			l_record_history := expression_cache_item (a_target)
			if l_record_history /= Void then
				Result := l_record_history.item (l_as_of_date)
			end
		end

	cache_virtual_records (a_targets: ROSE_LINEAR_ARRAY [like signature_anchor]; a_historical: BOOLEAN): ROSE_LINEAR_ARRAY [ROSE_VIRTUAL_RECORD [like signature_anchor]] is
			-- Cache virtual records and return cached targets
		do
			if a_historical then
				Result := a_targets.first.historical_records (a_targets, restriction_statement)
--				Result := ur_signature.historical_records (a_targets, restriction_statement)
			else
				Result := a_targets.first.records (a_targets, restriction_statement)
--				Result := ur_signature.records (a_targets, restriction_statement)
			end
			if Result.count = 0 then
				a_targets.do_all (agent expression_cache_force (Void, ?))
			else
				load_virtual_records_to_cache (Result, a_historical, True)
			end
		end

	load_virtual_records_to_cache (a_records: ROSE_LINEAR_ARRAY [ROSE_VIRTUAL_RECORD [like signature_anchor]]; a_historical: BOOLEAN; a_duplicate: BOOLEAN) is
		require
			a_records_not_void: a_records /= Void
		local
			l_last_signature_id: INTEGER
			l_current_record: ROSE_VIRTUAL_RECORD [like signature_anchor]
			l_record_history: ROSE_VIRTUAL_RECORD_HISTORY [like signature_anchor]
		do
			-- Since a_records are sorted by signature_id, as_of_date, we shall take advantage of the ordering in loading cache
			from a_records.finish
			until a_records.off
			loop
				if a_duplicate then
					l_current_record := a_records.current_item.duplicate
				else
					l_current_record := a_records.current_item
				end
				if l_record_history = Void or else l_last_signature_id /= l_current_record.signature.signature_id then
					l_last_signature_id := l_current_record.signature.signature_id
					create l_record_history.make_one (l_current_record)
					l_record_history.set_historical (a_historical)
					expression_cache_force (l_record_history, l_current_record.signature)
				else
					l_record_history.extend (l_current_record)
				end
				a_records.back
			end
		end

feature -- Evaluate data value

	integer_value (a_target:  like signature_anchor): INTEGER is
			-- <Precursor>
		do
			if last_target = a_target and then local_cache_version >= global_cache_version.item then
				Result := last_integer
			else
				last_integer := get_integer_value (a_target)
				last_target := a_target
				Result := last_integer
			end
		end

	real_value (a_target: like signature_anchor): REAL is
			-- <Precursor>
		do
			if last_target = a_target and then local_cache_version >= global_cache_version.item then
				Result := last_real
			else
				last_real := get_real_value (a_target)
				last_target := a_target
				Result := last_real
			end
		end

	double_value (a_target: like signature_anchor): DOUBLE is
			-- Double value
		do
			if last_target = a_target and then local_cache_version >= global_cache_version.item then
				Result := last_double
			else
				last_double := get_double_value (a_target)
				last_target := a_target
				Result := last_double
			end
		end

	string_value (a_target: like signature_anchor): STRING is
			-- <Precursor>
		do
			if last_target = a_target and then local_cache_version >= global_cache_version.item then
				Result := last_string
			else
				last_string := get_string_value (a_target)
				last_target := a_target
				Result := last_string
			end
		end

	image_value (a_target: like signature_anchor): STRING is
		do
			if last_target = a_target and then local_cache_version >= global_cache_version.item then
				Result := last_image
			else
				last_image := get_image_value (a_target)
				last_target := a_target
				Result := last_image
			end
		end

	date_value (a_target: like signature_anchor): ROSE_DATE is
			-- Date value
		do
			if last_target = a_target and then local_cache_version >= global_cache_version.item then
				Result := last_date
			else
				last_date := get_date_value (a_target)
				last_target := a_target
				Result := last_date
			end
		end


feature -- Get data value

	get_integer_value (a_target: like signature_anchor): INTEGER is
		local
			l_virtual_record:ROSE_VIRTUAL_RECORD [like signature_anchor]
			l_value_ref: INTEGER_REF
		do
			Result := undefined_integer
			if is_signature_field then
				l_value_ref ?= signature_to_item (a_target)
				if l_value_ref /= Void then
					Result := l_value_ref.item
				end
			else
				l_virtual_record := retrieve_virtual_record (a_target)
				if l_virtual_record /= Void then
					if not is_record_field then
						if l_virtual_record.integer_items_saved /= Void then
							l_virtual_record.integer_items_saved.search (variable_index)
							if	l_virtual_record.integer_items_saved.found then
								Result := l_virtual_record.integer_items_saved.found_item
							end
						end
					else
						l_value_ref ?= record_to_item (l_virtual_record)
						if l_value_ref /= Void then
							Result := l_value_ref.item
						end
					end
				end
			end
		end

	get_real_value (a_target: like signature_anchor): REAL is
		local
			l_virtual_record: ROSE_VIRTUAL_RECORD [like signature_anchor]
			l_value_ref: REAL_REF
		do
			Result := undefined_real
			if is_signature_field then
				l_value_ref ?= signature_to_item (a_target)
				if l_value_ref /= Void then
					Result := l_value_ref.item
				end
			else
				l_virtual_record := retrieve_virtual_record (a_target)
				if l_virtual_record /= Void then
					if not is_record_field then
						if l_virtual_record.real_items_saved /= Void then
							l_virtual_record.real_items_saved.search (variable_index)
							if	l_virtual_record.real_items_saved.found then
								Result := l_virtual_record.real_items_saved.found_item
							end
						end
					else
						l_value_ref ?= record_to_item (l_virtual_record)
						if l_value_ref /= Void then
							Result := l_value_ref.item
						end
					end
				end
			end
		end

	get_double_value (a_target: like signature_anchor): DOUBLE is
		local
			l_virtual_record: ROSE_VIRTUAL_RECORD [like signature_anchor]
			l_value_ref: DOUBLE_REF
		do
			Result := undefined_double
			if is_signature_field then
				l_value_ref ?= signature_to_item (a_target)
				if l_value_ref /= Void then
					Result := l_value_ref.item
				end
			else
				l_virtual_record := retrieve_virtual_record (a_target)
				if l_virtual_record /= Void then
					if not is_record_field then
						if l_virtual_record.double_items_saved /= Void	then
							l_virtual_record.double_items_saved.search (variable_index)
							if	l_virtual_record.double_items_saved.found then
								Result := l_virtual_record.double_items_saved.found_item
							end
						end
					else
						l_value_ref ?= record_to_item (l_virtual_record)
						if l_value_ref /= Void then
							Result := l_value_ref.item
						end
					end
				end
			end
		end

	get_string_value (a_target: like signature_anchor): STRING is
		local
			l_virtual_record: ROSE_VIRTUAL_RECORD [like signature_anchor]
			l_value_ref: STRING
		do
			Result := Void
			if is_signature_field then
				l_value_ref ?= signature_to_item (a_target)
				if l_value_ref /= Void then
					Result := l_value_ref
				end
			else
				l_virtual_record := retrieve_virtual_record (a_target)
				if l_virtual_record /= Void then
					if not is_record_field then
						if l_virtual_record.string_items_saved /= Void then
							l_virtual_record.string_items_saved.search (variable_index)
							if	l_virtual_record.string_items_saved.found then
								Result := l_virtual_record.string_items_saved.found_item
							end
						end
					elseif not is_attribute_field then
						l_value_ref ?= record_to_item (l_virtual_record)
						if l_value_ref /= Void then
							Result := l_value_ref
						end
					else
						Result := attribute_value (a_target)
					end
				end
			end
		end

	get_image_value (a_target: like signature_anchor): STRING is
		local
			l_virtual_record: ROSE_VIRTUAL_RECORD [like signature_anchor]
			l_value_ref: STRING
		do
			Result := Void
			if is_signature_field then
				l_value_ref ?= signature_to_item (a_target)
				if l_value_ref /= Void then
					Result := l_value_ref
				end
			else
				l_virtual_record := retrieve_virtual_record (a_target)
				if l_virtual_record /= Void then
					if not is_record_field then
						if l_virtual_record.image_items_saved /= Void then
							l_virtual_record.image_items_saved.search (variable_index)
							if	l_virtual_record.image_items_saved.found then
								Result := l_virtual_record.image_items_saved.found_item
							end
						end
					else
						l_value_ref ?= record_to_item (l_virtual_record)
						if l_value_ref /= Void then
							Result := l_value_ref
						end
					end
				end
			end
		end

	get_date_value (a_target: like signature_anchor): ROSE_DATE is
		local
			l_virtual_record: ROSE_VIRTUAL_RECORD [like signature_anchor]
			l_value_ref: ROSE_DATE
		do
			Result := Void
			if is_signature_field then
				l_value_ref ?= signature_to_item (a_target)
				if l_value_ref /= Void then
					Result := l_value_ref
				end
			else
				l_virtual_record := retrieve_virtual_record (a_target)
				if l_virtual_record /= Void then
					if not is_record_field then
						if l_virtual_record.date_items_saved /= Void then
							l_virtual_record.date_items_saved.search (variable_index)
							if	l_virtual_record.date_items_saved.found then
								Result := l_virtual_record.date_items_saved.found_item
							end
						end
					else
						l_value_ref ?= record_to_item (l_virtual_record)
						if l_value_ref /= Void then
							Result := l_value_ref
						end
					end
				end
			end
		end

	last_image: STRING

	last_date: ROSE_DATE

feature -- record / storable value

	record_value (a_target: like signature_anchor): ROSE_SIGNATURE_RECORD_ITEM is
		local
			l_virtual_record: ROSE_VIRTUAL_RECORD[like signature_anchor]
		do
			if variable_description.source_table /= Void then
				l_virtual_record := retrieve_virtual_record (a_target)
				if l_virtual_record /= Void and then l_virtual_record.records /= Void then
					Result := l_virtual_record.records.item(variable_description.source_table)
				end
			end
		end

	storable_value(a_target: like signature_anchor): ANY is
		local
			l_string: STRING
		do
			l_string := image_value(a_target)
			if l_string /= Void then
				Result := ur_stream.retrieve_from_string(l_string)
			end
		end

feature {NONE} -- Attribute type data management

	ur_stream: ROSE_STREAM is once create Result end

	is_attribute_field: BOOLEAN

	attribute_value (a_target: like signature_anchor): STRING is
			-- Attribute value
		do end

feature {NONE} -- Record type data management

	is_record_field: BOOLEAN

	record_name: STRING

	record_field_index: INTEGER

	set_record_name is
		local
			l_source_table: STRING
		do
			is_record_field := False
			is_attribute_field := False
			if variable_description /= Void then
				l_source_table := clone (variable_description.source_table)
				if l_source_table /= Void then
					l_source_table.right_adjust
					l_source_table.to_lower
					if l_source_table.count > 0 and then not l_source_table.is_equal ("item") then
						if l_source_table.is_equal ("attribute") then
							is_attribute_field := True
						else
							is_record_field := True
							record_name := l_source_table
							set_record_field_index (l_source_table)
						end
					end
				end
			end
		end

	set_record_field_index (a_record_name: STRING) is
		local
			i: INTEGER
			l_source_column, l_field_name: STRING
			l_key_pair: KEY_PAIR [STRING, STRING]
			l_index_cache: HASH_TABLE [INTEGER, KEY_PAIR [STRING, STRING]]
			l_manager: SIGNATURE_TAGGED_ITEM_MANAGER [ROSE_SIGNATURE_RECORD [ENTITY]]
			l_record_item: ROSE_SIGNATURE_RECORD_ITEM
		do
			record_field_index := 0
			l_source_column := clone (variable_description.source_column)
			if l_source_column /= Void then
				l_source_column.right_adjust
				l_source_column.to_lower
				create l_key_pair.make (a_record_name, l_source_column)
				l_index_cache := expression_cache.record_field_index_cache
				l_index_cache.search (l_key_pair)
				if l_index_cache.found then
					record_field_index := l_index_cache.found_item
				else
					l_manager ?= ur_signature.data_manager
					if l_manager /= Void and then l_manager.record_templates /= Void then
						l_record_item := l_manager.record_templates.item (a_record_name)
						if l_record_item /= Void then
							from i := 1
							until i > internal.field_count (l_record_item)
							loop
								l_field_name := internal.field_name (i, l_record_item)
								create l_key_pair.make (a_record_name, l_field_name)
								l_index_cache.force (i, l_key_pair)
								if l_field_name.is_equal (l_source_column) then
									record_field_index := i
								end
								i := i + 1
							end
						end
					end
				end
			end
			-- Check if record field index is mapped correctly
			if record_field_index = 0 then
				if l_source_column = Void then l_source_column := "" end
				print ("EXCEPTIONS: Record field not found:" + a_record_name + ", " + l_source_column + "%N")
				(create {EXCEPTIONS}).raise ("Record field not found:" + a_record_name + ", " + l_source_column)
			end
		end

	record_to_item (a_record: ROSE_VIRTUAL_RECORD [like signature_anchor]): ANY is
			-- return item value from a record table
		local
			l_record_item: ROSE_SIGNATURE_RECORD_ITEM
		do
			Result := Void
			if a_record.records /= Void then
				l_record_item := a_record.records.item (record_name)
				if l_record_item /= Void then
					Result := internal.field (record_field_index, l_record_item)
				end
			end
		end

feature {NONE} -- Signature data value implementation

	is_signature_field: BOOLEAN

	signature_field_index: INTEGER

	set_signature_field is
		local
			l_source_table: STRING
		do
			is_signature_field := False
			if variable_description /= Void then
				l_source_table := clone (variable_description.source_table)
				if l_source_table /= Void then
					l_source_table.right_adjust
					l_source_table.to_lower
					if l_source_table.count > 0 and then l_source_table.is_equal ("signature_data") then
						is_signature_field := True
						set_signature_field_index (ur_signature, variable_description.source_column)
					end
				end
			end
		end

	set_signature_field_index (a_target: like signature_anchor; a_field_name: STRING) is
		local
			j, l_field_count: INTEGER
			l_field_name: STRING
		do
			signature_field_index := 0
			l_field_count := internal.field_count(a_target)
			from j := 1 until (j > l_field_count) or (signature_field_index /= 0) loop
				l_field_name := internal.field_name (j , a_target)
				if (l_field_name.is_equal(a_field_name)) then
					signature_field_index := j
				end
				j := j + 1
			end
		end

	signature_to_item (a_target: like signature_anchor): ANY is
			-- return field value from a signature
		do
			Result := Void
			if signature_field_index > 0 then
				Result := internal.field (signature_field_index, a_target)
			end
		end

feature -- Restriction Statement

	restriction_statement: STRING is
		do
			if not ur_signature.is_direction_default then
				Result := ur_signature.restriction_statement
			end

			if Result = Void then
				if restriction_statement_saved /= Void then
					Result := restriction_statement_saved
				else
					Result := default_restriction_statement
					if Result = Void then
						Result := ur_signature.restriction_statement
					end
				end
			end
		end

	restriction_statement_saved: STRING
	default_restriction_statement: STRING is
		do
		end

	set_restriction_statement(a_value: STRING) is
		do
			restriction_statement_saved := a_value
		end

feature -- Neighbourhoods

--	neighborhood:ROSE_LINEAR_ARRAY[like signature_anchor]
	set_neighborhood(a_targets: ROSE_LINEAR_ARRAY [like signature_anchor]) is
		do
			if ur_signature = Void then create ur_signature end
			ur_signature.set_direction_neighborhood
--			neighborhood := a_targets
--			load_neighborhood
			cache_virtual_records(a_targets, True).do_nothing
		end

--	load_neighborhood is
--		do
--			cache_virtual_records(neighborhood, True).do_nothing
--		end

	cache_neighborhood (a_targets: ROSE_LINEAR_ARRAY [like signature_anchor]): ROSE_LINEAR_ARRAY [like signature_anchor] is
			-- cache the neighborhood items and return the cached targets
		local
			l_records: ROSE_LINEAR_ARRAY [ROSE_VIRTUAL_RECORD [like signature_anchor]]
		do
			if ur_signature = Void then create ur_signature end
			ur_signature.set_direction_neighborhood
			l_records := cache_virtual_records (a_targets, True)
			create Result.make_empty
			from l_records.start until l_records.after loop
				Result.extend (l_records.current_item.signature)
				l_records.forth
			end
		end

	refresh_cache is
		local
			l_cache: ROSE_VOLATILE_CACHE [ROSE_VIRTUAL_RECORD_HISTORY [like signature_anchor], MULTI_KEY]
		do
			l_cache := expression_cache.virtual_record_cache_by_key
			if l_cache /= Void then
				l_cache.refresh
			end
		end

feature -- Replace items

	replace_record_item(a_value: ROSE_SIGNATURE_RECORD_ITEM; a_signature: like signature_anchor) is
		local
			l_manager: SIGNATURE_TAGGED_RECORD_MANAGER[like signature_anchor, ROSE_RECORD_ITEM_TABLE]
		do
			if is_record_field then
				l_manager ?= a_signature.data_manager
				if l_manager /= Void then
					l_manager.replace_record_item(a_value, a_signature, record_name)
				end
			end
		end
	replace_date_item(a_value: ROSE_DATE; a_signature: like signature_anchor) is
		do
			if is_date then
				a_signature.data_manager.replace_date_item(a_value, variable_index, a_signature)
			end
		end
	replace_double_item(a_value: DOUBLE; a_signature: like signature_anchor) is
		do
			if is_double then
				a_signature.data_manager.replace_double_item(a_value, variable_index, a_signature)
			end
		end
	replace_image_item(a_value: STRING; a_signature: like signature_anchor) is
		do
--			if is_image then
				a_signature.data_manager.replace_image_item(a_value, variable_index, a_signature)
--			end
		end
	replace_integer_item(a_value: INTEGER; a_signature: like signature_anchor) is
		do
			if is_integer then
				a_signature.data_manager.replace_integer_item(a_value, variable_index, a_signature)
			end
		end
	replace_real_item(a_value: REAL; a_signature: like signature_anchor) is
		do
			if is_real then
				a_signature.data_manager.replace_real_item(a_value, variable_index, a_signature)
			end
		end
	replace_string_item(a_value: STRING; a_signature: like signature_anchor) is
		do
			if is_string then
				a_signature.data_manager.replace_string_item(a_value, variable_index, a_signature)
			end
		end

	replace_signature_data(a_signature: like signature_anchor) is
		do
			a_signature.data_manager.replace_signature_data(a_signature)
		end

	replace_storable_item(a_data: ANY; a_signature: like signature_anchor) is
		do
			a_signature.data_manager.replace_storable_item(a_data, variable_index, a_signature)
		end

end
