indexing
	description: "Signature Tagged Expression Cache"
	author: "Nelson Chui"
	date: "$Date: 2008/12/01 $"
	revision: "$Revision: #11 $"
	path: "$File: //rose/source/kernel/data/reports/signature_tagged_expression_cache.e $"

class
	SIGNATURE_TAGGED_EXPRESSION_CACHE [G -> ROSE_SIGNATURE_RECORD [ENTITY] create default_create end]

inherit
	ROSE_SHARED_GLOBAL

create
	make

feature -- Initialization

	make (a_expression_cache_key: STRING; a_count, a_chunk_size: INTEGER) is
		local
			l_count, l_chunk_size: INTEGER
			l_void_g: G
		do
			expression_cache_key := a_expression_cache_key
			create last_virtual_record.make (l_void_g, Void)
			create record_field_index_cache.make (8)
			if a_chunk_size > 0 and a_count > a_chunk_size then
				create virtual_record_cache_by_id.make (a_count, a_chunk_size)
				create virtual_record_cache_by_key.make (a_count, a_chunk_size)
			else
				l_count := config.integer ("dataaccess\cachesize", 100_000)
				l_chunk_size := (.1 * l_count).rounded
				create virtual_record_cache_by_id.make (l_count, l_chunk_size)
				create virtual_record_cache_by_key.make (l_count, l_chunk_size)
			end
		end

feature -- Access

	expression_cache_key: STRING

	last_virtual_record: PAIR [G, ROSE_VIRTUAL_RECORD [G]]

	virtual_record_cache_by_id: ROSE_VOLATILE_CACHE [ROSE_VIRTUAL_RECORD_HISTORY [G], INTEGER]

	virtual_record_cache_by_key: ROSE_VOLATILE_CACHE [ROSE_VIRTUAL_RECORD_HISTORY [G], MULTI_KEY]

	record_field_index_cache: HASH_TABLE [INTEGER, KEY_PAIR [STRING, STRING]]

feature -- Basic operations

	set_last_virtual_record (a_target: G; a_virtual_record: ROSE_VIRTUAL_RECORD [G]) is
			-- set last virtual record
		do
			last_virtual_record.make (a_target, a_virtual_record)
		end

	wipe_out is
			-- wipe out content
		local
			l_count, l_chunk_size: INTEGER
			l_void_g: G
		do
			create last_virtual_record.make (l_void_g, Void)
			l_count := virtual_record_cache_by_id.maximum_count
			l_chunk_size := virtual_record_cache_by_id.maximum_chunk_size
			create virtual_record_cache_by_id.make (l_count, l_chunk_size)
			l_count := virtual_record_cache_by_key.maximum_count
			l_chunk_size := virtual_record_cache_by_key.maximum_chunk_size
			create virtual_record_cache_by_key.make (l_count, l_chunk_size)
		end

end -- END OF CLASS
