indexing
	description: "Object used to identify the list of commands available for the persistent servers"
	author: "Mauricio Bustos"
	last_editor: "$Author: rj $"
	date: "$Date: 2014/10/23 $"
	revision: "$Revision: #16 $"
    path: "$File: //rose/source/kernel/persistence/databases/signature_tagged/signature_tagged_names.e $"

class
	SIGNATURE_TAGGED_NAMES

inherit
	MISMATCH_CORRECTOR
		redefine
			correct_mismatch
		end

feature -- Access

	data_name: STRING
			-- The data name partition prefix for this instance of a signature tagged database within a physical database

	data_name_prefix: STRING is
			-- A formatted prefix to be used for predicate construction
		do
			if data_name = Void or else data_name.count = 0 then
				Result := ""
			else
				Result := data_name + "_"
			end
		end

	table_name: STRING is
			-- The data signatures table name for this signature tagged database
		do
			if data_name = Void then
				Result := "data_signatures"
			else
				Result := data_name_prefix + "data_signatures"
			end
		end

	versions_table_name: STRING is
			-- The data signature versions table name for this signature tagged database
		do
			Result := table_name.substring(1, table_name.count - 1) + "_versions"
		end

	all_table_names: ROSE_LINEAR_ARRAY[STRING] is
			-- Collection of table names used by this signature tagged database
			-- This list is appended to by descendents depending on available data types
		do
			create Result.make_from_array(<<table_name, versions_table_name>>)
		end

feature {NONE} 	-- Command id's.  Used by remote clients to request a server to execute a function or subroutine.

	path_tree_id: INTEGER is 3
	remake_virtual_record_set_id: INTEGER is 4
	set_current_cache_pool_id: INTEGER is 5
	ordered_cube_of_items_id: INTEGER is 6
	set_query_signatures_id: INTEGER is 7
	set_new_query_id: INTEGER is 8
	set_process_name_id: INTEGER is 9
	create_data_items_id: INTEGER is 10
	create_selected_column_ids_id: INTEGER is 11
	set_record_limit_id: INTEGER is 12
	set_restriction_clause_id: INTEGER is 13
	columns_from_restriction_statement_id: INTEGER is 14
	clear_column_comparison_id: INTEGER is 15
	last_rose_signature_id: INTEGER is 16
	virtual_record_set_id: INTEGER is 17
	unique_signature_server_name_id: INTEGER is 18
	compare_version_id: INTEGER is 19
	stats_string_id: INTEGER is 20
	set_data_client_id: INTEGER is 21
	set_historical_mode_id: INTEGER is 22
	clear_historical_mode_id: INTEGER is 23
	prepare_to_retrieve_signatures_id: INTEGER is 24
	select_signature_ids_id: INTEGER is 25
	set_ad_hoc_query_id: INTEGER is 26
	last_cube_signature_result_id: INTEGER is 27
	create_trigger_id: INTEGER is 28
	retrieve_signature_data_history_id: INTEGER is 29
	process_id_map_id: INTEGER is 30
	refresh_process_id_map_id: INTEGER is 31
	signature_id_id: INTEGER is 32
	process_id_id: INTEGER is 33
	update_end_time_id: INTEGER is 34
	records_count_id: INTEGER is 35
	start_process_id: INTEGER is 36
	terminate_process_id_id: INTEGER = 37
			-- Add new commands above this line, change the value below to match.
	last_valid_command_id: INTEGER = 37

feature {NONE} -- Implementation

	correct_mismatch is
		local
			l_mismatch: MISMATCH_INFORMATION
		do
			l_mismatch := mismatch_information
		end

end





