indexing
	description: "Manager for a signature tagged item database with records"
	keywords: "database,signature,item,tagged,manager,record"
	author: "Mauricio Bustos"
	last_editor: "$Author: rj $"
	date: "$Date: 2014/09/11 $"
	revision: "$Revision: #12 $"
	path: "$File: //rose/source/kernel/persistence/databases/signature_tagged/reference/signature_tagged_record_database.e $"

class

	SIGNATURE_TAGGED_RECORD_DATABASE[reference G -> ROSE_SIGNATURE_RECORD[ENTITY] create default_create end,
					 reference H -> ROSE_RECORD_ITEM_TABLE create make end]

inherit

	SIGNATURE_TAGGED_ITEM_MANAGER[G]
		redefine
            record_templates,
			--/
--			insert_signature,
			--/-
			replace_virtual_record,
			signature_tagged_server
		end

create

	make,
	make_named,
	make_named_with_process_manager,
	make_with_process_manager

feature -- Replace

	replace_virtual_record(a_record : ROSE_VIRTUAL_RECORD[G]) is
		do
			Precursor(a_record)
			if a_record.records /= Void then
				from a_record.records.start until a_record.records.off loop
					a_record.records.item_for_iteration.set_signature_id(a_record.signature.signature_id)
					if a_record.records.item_for_iteration.process_id = 0 then
						a_record.records.item_for_iteration.set_process_id(process_id)
					end
					replace_record_item(a_record.records.item_for_iteration,
										a_record.signature,
										a_record.records.key_for_iteration)
					a_record.records.forth
				end
			end
		end

	replace_record_item(a_value : ROSE_SIGNATURE_RECORD_ITEM; a_signature : G; a_name : STRING) is
		local
			l_match : BOOLEAN
			l_virtual_record : ROSE_VIRTUAL_RECORD[G]
			l_record : ROSE_SIGNATURE_RECORD_ITEM
			l_any_from, l_any_to : ANY
		do
			set_results_from_cache(a_signature)
			l_virtual_record := virtual_record(a_signature, True, True)
			a_value.set_signature_id(l_virtual_record.signature.signature_id)
			if a_value.process_id = 0 then
				a_value.set_process_id(process_id)
			end

			if l_virtual_record.records /= Void then
				l_record := l_virtual_record.records.item(a_name)
			end
			if l_record /= Void then
				l_match := True
				from l_record.record_fields.start until l_record.record_fields.off loop
					l_any_from ?= l_record.value_by_name(l_record.record_fields.current_item)
					l_any_to ?= a_value.value_by_name(l_record.record_fields.current_item)
					if l_any_from /= Void and l_any_to /= Void then
						if not l_any_from.is_equal(l_any_to) then
							l_match := False
						end
					elseif l_any_from = Void and l_any_to /= Void or
						l_any_from /= Void and l_any_to = Void then
						l_match := False
					end
					l_record.record_fields.forth
				end
				if not l_match then
					signature_tagged_server.record_data_replace(a_name, a_value)
					l_virtual_record.set_record_item(a_value)
				end
			elseif a_value /= Void then
				signature_tagged_server.record_data_replace(a_name, a_value)
				l_virtual_record.set_record_item(a_value)
			end
		end

	replace_indexed_record_item(a_value : ROSE_SIGNATURE_RECORD_ITEM; a_signature : G) is
		local
			l_match : BOOLEAN
			l_virtual_record : ROSE_VIRTUAL_RECORD[G]
			l_records : HASH_TABLE[ROSE_SIGNATURE_RECORD_ITEM, MULTI_KEY]
			l_record : ROSE_SIGNATURE_RECORD_ITEM
			l_any_from, l_any_to : ANY
		do
			set_results_from_cache(a_signature)
			l_virtual_record := virtual_record(a_signature, True, True)
			a_value.set_signature_id(l_virtual_record.signature.signature_id)
			if a_value.process_id = 0 then
				a_value.set_process_id(process_id)
			end

			if l_virtual_record.indexed_records /= Void then
				l_records := l_virtual_record.indexed_records.item(a_value.name)
				if l_records/=Void
				then
					from
						l_records.start
					until
						l_records.off or l_match
					loop
						if equal(l_records.item_for_iteration.multi_key, a_value.multi_key)
						then
							l_record := l_records.item_for_iteration
						end

						l_records.forth
					end
				end
			end

			if l_record /= Void then
				l_match := True
				from l_record.record_fields.start until l_record.record_fields.off loop
					l_any_from ?= l_record.value_by_name(l_record.record_fields.current_item)
					l_any_to ?= a_value.value_by_name(l_record.record_fields.current_item)
					if l_any_from /= Void and l_any_to /= Void then
						if not l_any_from.is_equal(l_any_to) then
							l_match := False
						end
					elseif l_any_from = Void and l_any_to /= Void or
						l_any_from /= Void and l_any_to = Void then
						l_match := False
					end
					l_record.record_fields.forth
				end
				if not l_match then
					signature_tagged_server.indexed_record_data_replace(a_value)
					l_virtual_record.set_indexed_record_item(a_value)
				end
			elseif a_value /= Void then
				signature_tagged_server.indexed_record_data_replace(a_value)
				l_virtual_record.set_indexed_record_item(a_value)
			end
		end

feature {NONE} -- Item Creation

	new_record(a_value : ROSE_SIGNATURE_RECORD_ITEM; a_signature : ROSE_VIRTUAL_RECORD[G]) : DATA_RECORD[ROSE_SIGNATURE_RECORD_ITEM] is
			-- Created record for value and signature,
			-- and attach signature to `a_value'
		do
			create Result
			Result.set_value(a_value)
			Result.set_path(a_signature.signature.path)
			Result.set_signature_id(a_signature.signature.signature_id)
			a_value.set_signature_id(a_signature.signature.signature_id)
		end

feature {SIGNATURE_TAGGED_EXPRESSION} -- Access

	record_templates : H is
		do
			if record_templates_saved = Void then
				create record_templates_saved.make
			end
			Result := record_templates_saved
		end

	record_templates_saved : H
			-- Cache for `record_templates'

feature {NONE} -- Implementation

	signature_tagged_server: SIGNATURE_TAGGED_RECORD_SERVER[G, H]
		-- <Precursor>

end
