indexing
	description: "Server for signature tagged records"
	author: "Randy John"
	date: "$Date: 2008/12/01 $"
	revision: "$Revision: #8 $"

class
	SIGNATURE_TAGGED_RECORD_SERVER[reference G -> ROSE_SIGNATURE_RECORD[ENTITY] create default_create end,
								   H -> ROSE_RECORD_ITEM_TABLE create make end]

inherit
	SIGNATURE_TAGGED_SERVER[G]
		redefine
			insert_signature
		end
create {SIGNATURE_TAGGED_RECORD_DATABASE}
	make
feature {SIGNATURE_TAGGED_RECORD_DATABASE}

	record_data_replace(a_name : STRING; a_value : ROSE_SIGNATURE_RECORD_ITEM) is
	do
		record_data(a_name).replace(a_value)
	end

	indexed_record_data_replace(a_value: ROSE_SIGNATURE_RECORD_ITEM) is
	do
		record_data(a_value.name).replace(a_value)
	end

feature {NONE} -- Insert

	insert_signature(a_signature : ROSE_VIRTUAL_RECORD[G]) is
		local
			l_records: HASH_TABLE[ROSE_SIGNATURE_RECORD_ITEM, MULTI_KEY]
		do
			Precursor(a_signature)

			if a_signature.records /= Void then
				from a_signature.records.start until a_signature.records.off loop
					a_signature.records.item_for_iteration.set_signature_id(a_signature.signature.signature_id)
					if a_signature.records.item_for_iteration.process_id = 0 then
						a_signature.records.item_for_iteration.set_process_id(process_id)
					end
					record_data(a_signature.records.key_for_iteration).insert(a_signature.records.item_for_iteration)
					a_signature.records.forth
				end
			end

			if a_signature.indexed_records /= Void then
				from a_signature.indexed_records.start until a_signature.indexed_records.off loop
					l_records := a_signature.indexed_records.item_for_iteration
					from
						l_records.start
					until
						l_records.off
					loop
						l_records.item_for_iteration.set_signature_id(a_signature.signature.signature_id)
						if l_records.item_for_iteration.process_id = 0 then
							l_records.item_for_iteration.set_process_id(process_id)
						end

						(create {EXCEPTIONS}).raise("not implemented")
--						record_data(a_signature.indexed_records.key_for_iteration).insert(a_signature.indexed_records.item_for_iteration)

						l_records.forth
					end
				end
			end
		end

feature {NONE} -- Data Tables

	record_data(a_name : STRING) : SIGNATURE_TAGGED_RECORD_ITEM is
		local
			l_record_item : SIGNATURE_TAGGED_RECORD_ITEM
			l_record_name : STRING
		do
			if records = Void then
				create records.make(10)
				from record_templates.start until record_templates.off loop
					create l_record_item.make(signature_item_server, process_id, record_templates.key_for_iteration, data_name,
											  record_templates.item_for_iteration)
					l_record_name := clone(record_templates.key_for_iteration)
					l_record_name.to_lower
					records.force(l_record_item,l_record_name)
					data_items.extend(l_record_item)
					last_record ?= l_record_item
					record_templates.forth
				end
			end
			if records.count = 1 then
				Result := last_record
			else
				a_name.to_lower
				records.search(a_name)
				if records.found then
					Result := records.found_item
				else
					raise("Invalid Record Name")
				end
			end
		end

	record_templates : H is
		do
			if record_templates_saved = Void then
				create record_templates_saved.make
			end
			Result := record_templates_saved
		end
	record_templates_saved : H

	last_record : SIGNATURE_TAGGED_RECORD_ITEM

	-- 'records' is not a table of records but a table of objects that can access the
	-- particular table that holds the records.
	records : HASH_TABLE[SIGNATURE_TAGGED_RECORD_ITEM,STRING]

end
