note

	description:
	"[
		Class that acts as the server portion of the signature tagged item database.
						  There are no clients to this class except for STID related systems.
						  No user should use this class directly.
	]"
	last_updater: "$Author: er $"
	author: "Mauricio Bustos"
	last_editor: "$Author: er $"
	date: "$Date: 2014/10/28 $"
	revision: "$Revision: #151 $"
	path: "$File: //rose/source/kernel/persistence/databases/signature_tagged/signature_tagged_server.e $"

class
	SIGNATURE_TAGGED_SERVER[reference G -> ROSE_SIGNATURE_RECORD[ENTITY] create default_create end]

inherit
	ANY

	SIGNATURE_TAGGED_NAMES
		export
			{NONE} all
		end
	EXCEPTIONS
		rename
			class_name as exceptions_class_name
		export
			{NONE} all
		end
	DATABASE_SERVER
		export
			{NONE} all
		end
	PM_METRIC_SOURCE
		export
			{NONE} all
		redefine
			metrics,
			reset_metrics
		end
	IEEE_MATH
		export
			{NONE} all
		end
	ROSE_SHARED_GLOBAL
		rename
			trace as global_trace
		export
			{NONE} all
		end
	MOVIE_CAMERA_AWARE
		export
			{NONE} all
		end
	SQL_CONSTANTS
		export
			{NONE} all
		end
	SIGNATURE_TAGGED_OUTPUT_LEVEL
		export
			{NONE} all
		end

create
	make

feature {SIGNATURE_TAGGED_ITEM_DATABASE} -- Initialization

	make (a_table_structure: SIGNATURE_TAGGED_TABLE_STRUCTURE) is
			-- Create the server
		require
			valid_table_structure: a_table_structure /= Void
		local
			l_camera_name: STRING
		do
			table_structure := a_table_structure
			data_name := table_structure.data_name
			debug ("make_movies")
				l_camera_name :=
					generating_type + "/" +
					table_structure.signature_item_server.node + "/" +
					table_structure.signature_item_server.database
				if data_name /= Void then
					l_camera_name.append ("/" + data_name)
				end
				create cam.make (l_camera_name)
			end
			--		create signature_touches.make_empty
			create signature_data_updates.make_empty
			create data_items.make_empty
			create selected_columns.make_empty
			create selected_column_ids.make_empty
			create cube_signature_result.make_empty
			create cube_result_table.make (0)
			create cube_position_result.make (0)
			--		create manager_cache.make (100,1)
			--		create cache_history.make_empty
			create cache_pool.make (100,1)
			--/
			create virtual_record_set.make (table_structure, Void)
			--/-
		ensure
			table_structure_set: table_structure = a_table_structure
		end

feature {SIGNATURE_TAGGED_ITEM_DATABASE} -- Debugging.

	post_rollback_cleanup is
	-- Call this ONLY when debugging and you ALWAYS call 'rollback' instead of 'commit'.
	-- Call this AFTER the call to 'rollback'.  This call is only for debugging and can not
	-- make any guarantees if a 'commit' has ever been called.
	-- Purpose:
	-- To reset anything that needs to be reset when you ALWAYS call 'rollback'.
	do
		virtual_record_set.post_rollback_cleanup
	end

feature {SIGNATURE_TAGGED_ITEM_DATABASE} -- Database server naming

	database_name: STRING is
			-- Name of the database that this server is attached to.  No machine name.
		do
			Result := signature_item_server.database
		end

	data_server_full_name: STRING is
			-- Name of the node and database that this server is attached to in the form 'node.database_name'.
		do
			Result := signature_item_server.node + "." + signature_item_server.database
		ensure
			valid_result: Result /= Void
		end

	data_server_node: STRING is
			-- The node (commonly a machine) of the server being used in this database object
		do
			Result := signature_item_server.node
		end

	data_server_logical_name: STRING is
			-- The logical name (from the repository) of the server that this server is attached to.
		do
			Result := signature_item_server.logical
		end

feature {SIGNATURE_TAGGED_ITEM_DATABASE} -- Table structure

	table_structure: SIGNATURE_TAGGED_TABLE_STRUCTURE

feature {SIGNATURE_TAGGED_ITEM_DATABASE} -- Data server access

	signature_item_server_not_void: BOOLEAN is
			-- This is used as a 'require' so that we don't have to export 'signature_item_server'.
		do
			Result := signature_item_server /= Void
		end

	cleanup_and_commit is
			-- cleanup and commit
		obsolete "Use cleanup and then commit the database yourself."
		do
			cleanup_items
			terminate_process_id
			signature_item_server.commit
		end

	set_submit_frequency (a_freq: INTEGER) is
			--set the submit frequency to a_freq
		require
			frequency_non_negative: a_freq>=0
		do
			data_items.do_all (agent set_item_submit_frequency (?,a_freq))
		end

	submit (a_signature_touches: ROSE_LINEAR_ARRAY[G]) is
		-- submit data & signature changes to the server
		do
			data_items.do_all (agent submit_item)
			submit_signature_touches (a_signature_touches)
			submit_signature_data_updates
		end

	discard_all_updates is
			-- Discard the staged update data
		do
			data_items.do_all (agent discard_updates)
		end

	commit is
		-- Commits the underlying database.
		obsolete "Commit the database yourself."
		do
			-- You might think that we don't need 'uncommitted' now that we test the transaction depth.
			-- However, 'transaction_depth' is only supported on SQL Server so I'm being extra safe.
			-- In addition, there is a slight possibility that some users are depending on calling
			-- this routine with 'uncommitted' set to True but no actual transaction is active.
			if uncommitted or signature_item_server.transaction_depth>0 then
				data_items.do_all (agent truncate_item_temp_tables)
				signature_item_server.commit
				uncommitted := False
			end
		end

	begin_transaction
			-- Begins an transaction on the underlying server.
		do
			signature_item_server.begin_transaction
		end

	data_server_commit
			-- Commits the transaction on the underlying server.
		do
			signature_item_server.commit
		end

	data_server_transaction_depth: INTEGER is
			-- Current transaction depth for `signature_item_server'
		do
			Result := signature_item_server.transaction_depth
		end

	uncommitted: BOOLEAN		-- I don't think that this is trustworthy. RJ.

	set_committed is
			-- Assign `False' to `uncommitted'.
		do
			uncommitted := False
		ensure
			committed: uncommitted = False
		end

	set_uncommitted is
			-- Assign `True' to `uncommitted'.
		do
			uncommitted := True
		ensure
			uncommitted: uncommitted = True
		end

	set_signature_item_server (a_server: SQL_DATA_SERVER) is
			-- Set `signature_item_server' to `a_server'
		require
			valid_server: a_server /= Void
		do
			signature_item_server := a_server
		end

	cleanup_items is
			-- Allow each of the 'data_items' to perform its clean up routine.
		do
			data_items.do_all (agent cleanup_items_all)
		end

	create_data_items is
			-- Create `data_items' as empty structure.
		do
			create data_items.make_empty
		ensure
			data_items_not_void: data_items /= Void
		end

	has_write_permission: BOOLEAN is
			-- Does current user have write permission on `Current'?
		do
			Result := signature_item_server.has_write_permission (table_name)
		end

	has_read_permission: BOOLEAN is
			-- Does current user have read permission on `Current'?
		do
			Result := signature_item_server.has_read_permission (table_name)
		end

feature {NONE} -- Database server

	signature_item_server: SQL_DATA_SERVER

	signature_item_fast_load_server: SQL_DATA_SERVER is
		do
			if signature_item_fast_load_server_saved = Void then
				signature_item_fast_load_server_saved := servers.make_from_repository_fast_load (signature_item_server.logical)
				signature_item_fast_load_server_saved.set_prefix ("FAST_")
			end
			Result := signature_item_fast_load_server_saved
		ensure
			valid_result: Result /= Void
		end
	signature_item_fast_load_server_saved: SQL_DATA_SERVER

feature {SIGNATURE_TAGGED_ITEM_DATABASE} -- Data Type Access Control

	testenv_overlay_enabled: BOOLEAN

	testenv_database: STID_TESTENV_DATABASE[G]

	enable_testenv_overlay is
		do
			testenv_overlay_enabled := True
			if testenv_database = Void then
				create testenv_database.make_for_overlay_retrieval (table_structure)
			end
		end

	disable_testenv_overlay is
		do
			testenv_overlay_enabled := False
		end

	check_full_update_mode is
		do
			if signature_item_server /= Void then
				full_update_mode := config.boolean (signature_item_server.logical + "_stid_full_update",False)
			end
		end

	client_can_write: BOOLEAN is
		local
			l_predicate: ANONYMOUS_PREDICATE
			l_string_predicate: STRING_PREDICATE
		do
			if not client_can_write_checked then
				create l_predicate.make_using_server (signature_item_server, "exec sp_tables 'client_write_access'")
				l_predicate.retrieve (Void, Void)
				if not l_predicate.success then
					 (create {EXCEPTIONS}).raise ("sp_tables failed.")
				elseif l_predicate.array.is_empty then
					client_can_write_saved := True
				else
					create l_string_predicate.make_strict (signature_item_server,"select client from client_write_access")
					l_string_predicate.retrieve (Void,Void)
					from l_string_predicate.array.start until l_string_predicate.array.off loop
						if Global.app.program_folder.as_lower.has_substring (l_string_predicate.array.current_item.as_lower) or
							l_string_predicate.array.current_item.as_lower.has_substring (Global.app.program_name.as_lower) then
							client_can_write_saved := True
						end
						l_string_predicate.array.forth
					end
				end
				debug ("make_movies")
					client_can_write_saved := True
				end
				client_can_write_checked := True
			end
			Result := client_can_write_saved
		end
	client_can_write_checked: BOOLEAN
	client_can_write_saved: BOOLEAN
	full_update_mode: BOOLEAN

feature {SIGNATURE_TAGGED_ITEM_DATABASE} -- Data record suppression

	showing_suppressed_historical_data: BOOLEAN is
		do
			Result := table_structure.showing_suppressed_historical_data
		end

	show_suppressed_historical_data is
		do
			table_structure.show_suppressed_historical_data
		end

	hide_suppressed_historical_data is
		do
			table_structure.hide_suppressed_historical_data
		end

feature {SIGNATURE_TAGGED_ITEM_DATABASE} -- Data record pinning

	enable_version_pinning is
		do
			table_structure.enable_version_pinning
		end

	disable_version_pinning is
		do
			table_structure.disable_version_pinning
		end

feature {SIGNATURE_TAGGED_ITEM_DATABASE} -- Data type enabling

	disable_data_type (a_type: STRING; a_is_override: BOOLEAN) is
		do
			table_structure.disable_data_type (a_type, a_is_override)
		end

	disable_regular_data_type (a_type: STRING) is
		do
			table_structure.disable_data_type (a_type, False)
		end

	disable_override_data_type (a_type: STRING) is
		do
			table_structure.disable_data_type (a_type, True)
		end

	disable_all_regular_data_types is
		do
			table_structure.disable_data_types (False)
		end

	disable_all_override_data_types is
		do
			table_structure.disable_data_types (True)
		end

	disable_all_data_types is
		do
			table_structure.disable_all_data_types
		end

	enable_data_type (a_type: STRING; a_is_override: BOOLEAN) is
		do
			table_structure.enable_data_type (a_type, a_is_override)
		end

	enable_regular_data_type (a_type: STRING) is
		do
			table_structure.enable_data_type (a_type, False)
		end

	enable_override_data_type (a_type: STRING) is
		do
			table_structure.enable_data_type (a_type, True)
		end

	enable_all_regular_data_types is
		do
			table_structure.enable_data_types (False)
		end

	enable_all_override_data_types is
		do
			table_structure.enable_data_types (True)
		end

	enable_all_data_types is
		do
			table_structure.enable_all_data_types
		end

feature {SIGNATURE_TAGGED_ITEM_DATABASE} -- Data type availability

	data_type_available, regular_data_type_available (a_type: STRING): BOOLEAN is
		do
			Result := table_structure.data_type_available (a_type, False)
		end

	override_data_type_available (a_type: STRING): BOOLEAN is
		do
			Result := table_structure.data_type_available (a_type, True)
		end

	set_override_family_name (a_name: STRING) is
		do
			table_structure.set_override_family_name (a_name)
		end

	override_family_name: STRING is
		do
			Result := table_structure.override_family_name
		end

	override_family_names: ROSE_LINEAR_ARRAY [STRING] is
		do
			Result := table_structure.override_family_names
		end

feature {SIGNATURE_TAGGED_ITEM_DATABASE} -- Query mode

	set_ad_hoc_query (a_value: BOOLEAN) is
		do
			ad_hoc_query := a_value
		end

	set_historical_mode is
		do
			historical_mode := True
		end

	clear_historical_mode is
		do
			historical_mode := False
		end

feature {NONE}-- Attributes

	cache_pool: ROSE_VOLATILE_CACHE[SIGNATURE_TAGGED_CACHE[G], INTEGER]
	current_cache_pool: SIGNATURE_TAGGED_CACHE[G]
	cache_pool_id: INTEGER

	column_comparison: ROSE_LINEAR_ARRAY[STRING]
	selected_columns: ROSE_LINEAR_ARRAY[STRING]
	selected_column_ids: ROSE_LINEAR_ARRAY[INTEGER]
	data_items: ROSE_LINEAR_ARRAY[like submittable_anchor]
	set_process_id_value: INTEGER
			-- The value of an externally set process id
	record_limit: INTEGER
	restriction_clause: STRING

	virtual_record_template: ROSE_VIRTUAL_RECORD[G]
			-- Type of virtual records that client would like to request, see descendents of ROSE_VIRTUAL_RECORD
	new_query: BOOLEAN

	cube_result_table: HASH_TABLE[ROSE_VIRTUAL_RECORD[G],MULTI_KEY]
	signature_data_updates: ROSE_LINEAR_ARRAY[PAIR[G,G]]
	actual_last_rose_signature: ROSE_VIRTUAL_RECORD[G]
	retry_limit: INTEGER is 5

	trigger_saved: SIGNATURE_TAGGED_TRIGGER
	view_saved: SIGNATURE_TAGGED_VIEW[G]
	is_path_clustered: BOOLEAN

	signatures_created: INTEGER
			-- metric: number of signatures created

	signatures_replaced: INTEGER
			-- metric: number of signatures replaced

	signature_loader_table_name: STRING

	temp_signature_loader_table_name: STRING

feature {SIGNATURE_TAGGED_ITEM_DATABASE} -- Attributes

	ad_hoc_query: BOOLEAN
	historical_mode: BOOLEAN

	cube_signature_result: ROSE_LINEAR_ARRAY [G]
	cube_position_result: HASH_TABLE[G, INTEGER]
	virtual_record_set: SIGNATURE_TAGGED_VIRTUAL_RECORD_SET [G]

	last_signature_created: BOOLEAN

feature {SIGNATURE_TAGGED_ITEM_DATABASE} -- Cacheing

	set_new_query is
		do
			new_query := True
		end

	last_rose_signature: ROSE_VIRTUAL_RECORD[G] is
		do
			Result := actual_last_rose_signature
		end

	last_cube_signature_result: ROSE_LINEAR_ARRAY[G] is
		do
			Result := cube_signature_result
		end

	set_cache_from_results (a_virtual_record: ROSE_VIRTUAL_RECORD[G]; a_client: SIGNATURE_TAGGED_ITEM_DATABASE[G]) is
			-- This rather ugly method of passing an object so that we can set stuff into it
			-- allows us to redefine this feature when this server is actually a client.  The
			-- redefined version saves a lot of network traffic and memory.
		local
			l_path: INTEGER
			l_virtual_record_set: TRIPLE[ROSE_LINEAR_ARRAY[ROSE_VIRTUAL_RECORD[G]],
									     HASH_TABLE[ROSE_VIRTUAL_RECORD[G], MULTI_KEY],
									     INTEGER]
		do
			if (a_virtual_record /= Void or cache_pool_id /= 0) then
				if ad_hoc_query then
					l_path := 0
				elseif a_virtual_record /= Void and then a_virtual_record.signature /= Void then
					l_path := path (a_virtual_record.signature)
				else
					l_path := cache_pool_id
				end
				create l_virtual_record_set.make (virtual_record_set.cube_result, virtual_record_set.cube_result_table, l_path)
				a_client.force_manager_cache (l_virtual_record_set, l_path)
				--			cache_pool.force (current_cache_pool,l_path)	-- Now done in set_current_cache_pool.
				--			cache_id := l_path
				a_client.set_cube_result_table (cube_result_table)
				a_client.set_cube_signature_result (cube_signature_result)
				a_client.set_cube_result (virtual_record_set.cube_result)
				a_client.set_cube_position_result (cube_position_result)
			end
		end

	set_virtual_record_set_cube_result_temp (a_cube_result: ROSE_LINEAR_ARRAY[ROSE_VIRTUAL_RECORD[G]]) is
			-- This is redefined in CLUSTERED_TAGGED_SERVER_CLIENT.
		do
		end

	stats_string: STRING is
			-- This is redefined in CLUSTERED_TAGGED_SERVER_CLIENT.
		do
			Result := ""
		end

	build_cube_position_result is
		do
			should_build_cube_position_result := True
		end

	do_not_build_cube_position_result is
		do
			should_build_cube_position_result := False
		end

	should_build_cube_position_result: BOOLEAN

feature {SIGNATURE_TAGGED_ITEM_DATABASE} -- Trigger management

	create_trigger (a_data_name: STRING) is
			-- Creates the standard trigger.
		require
			server_not_void: signature_item_server_not_void
		do
			if trigger_saved = Void then
				create trigger_saved.make (signature_item_server, a_data_name)
			end

			trigger_saved.create_trigger
		end
	drop_trigger (a_data_name: STRING) is
		do
			create_trigger (a_data_name)
			trigger_saved.drop_trigger
		end

feature {SIGNATURE_TAGGED_ITEM_DATABASE} -- Data bounds

	data_bounds (a_signature: G; a_data_name: STRING): SIGNATURE_TAGGED_DATA_BOUNDS is
			-- What are the value bounds of the a_data_name column for a_signature.
			-- See 'preload_data_bounds'.
		do
			if view_saved = Void then
				create view_saved.make (signature_item_server, a_data_name)
			end
			Result := view_saved.data_bounds (a_signature)
		end

	preload_data_bounds(a_signatures: ROSE_LINEAR_ARRAY[G]; a_data_name: STRING)
	-- Quickly preload the data_bounds for fast access.
	do
		if view_saved = Void
		then
			create view_saved.make(signature_item_server, a_data_name)
		end

		view_saved.preload_data_bounds(a_signatures)
	end

	column_bounds (a_column_name: STRING): COLUMN_BOUNDS is
			-- What is the value range for a_column_name
		require
			valid_name: a_column_name /= Void
		local
			l_predicate: COLLECTING_PREDICATE[COLUMN_BOUNDS]
		do
			create l_predicate.make_strict (signature_item_server,
										   "select count (distinct (" + a_column_name + ")) as cnt, " +
										   "cast (min (" + a_column_name + ") as varchar (100)) as minimum, " +
										   "cast (max (" + a_column_name + ") as varchar (100)) as maximum " +
										   "from " + data_name_prefix + "data_signatures")
			l_predicate.retrieve (Void,Void)
			if l_predicate.array.count = 1 then
				Result := l_predicate.array.first
			end
		end

feature {SIGNATURE_TAGGED_ITEM_DATABASE} -- Process update characteristics

	records_touched_since (a_process_id: INTEGER; a_items: ROSE_LINEAR_ARRAY[INTEGER]): ROSE_LINEAR_ARRAY[G] is
			-- a Void 'a_items' retrieve all the records
		local
			l_statement: STRING
			l_predicate: COLLECTING_PREDICATE[G]
			l_signature: G
			l_select, l_from_clause, l_where_clause: STRING
			l_types: ROSE_LINEAR_ARRAY[STRING]
		do
			create l_signature.default_create
			create Result.make_empty
			l_select := "select distinct " + data_name_prefix + "data_signatures.signature_id," + l_signature.signature_and_data_fields_clause
			if a_items/=Void then
				create l_types.make_from_array (<<"REAL","INTEGER","DOUBLE","STRING","IMAGE","DATETIME">>)
				from l_types.start until l_types.off
				loop
					if data_type_available (l_types.current_item) then
						l_from_clause:=" from " + data_name_prefix + "data_signatures INNER JOIN " + data_name_prefix +
									   "data_" + l_types.current_item + "_value ON " + data_name_prefix + "data_signatures.signature_id = " +
									   data_name_prefix + "data_" + l_types.current_item + "_value.signature_id"
						l_where_clause := " where " + data_name_prefix + "data_" + l_types.current_item + "_value.process_id > ? and " +
										  data_name_prefix + "data_" + l_types.current_item + "_value.item_id IN ("
						from a_items.start until a_items.off
						loop
							l_where_clause:=l_where_clause + a_items.current_item.out + ","
							a_items.forth
						end
						l_where_clause.prune_all_trailing (',')
						l_where_clause:=l_where_clause + ")"
						l_statement:=l_select + l_from_clause + l_where_clause
						create l_predicate.make (signature_item_server, l_statement)
						if not l_predicate.success then
							raise ("Failed to create predicate")
						end
						l_predicate.retrieve (Void,<<a_process_id>>)
						if not l_predicate.success then
							raise ("Failed to execute predicate")
						end
						Result.append_array (l_predicate.array)
					end
					l_types.forth
				end
			else
				l_from_clause := " from " + data_name_prefix + "data_signatures"
				l_where_clause:=" where " + data_name_prefix + "data_signatures.last_process_id > ?"
				l_statement:=l_select + l_from_clause + l_where_clause
				create l_predicate.make (signature_item_server, l_statement)
				if not l_predicate.success then
					raise ("Failed to create predicate")
				end
				l_predicate.retrieve (Void,<<a_process_id>>)
				if not l_predicate.success then
					raise ("Failed to execute predicate")
				end
				Result.append_array (l_predicate.array)
			end
		end

	records_touched_by (a_process_ids: ROSE_LINEAR_ARRAY[INTEGER]; a_items: ROSE_LINEAR_ARRAY[INTEGER]): ROSE_LINEAR_ARRAY[G] is
			-- a Void 'a_items' retrieve all the records
		require
			valid_process_ids: a_process_ids /= Void
		local
			l_statement: STRING
			l_predicate: COLLECTING_PREDICATE[G]
			l_count_predicate: INTEGER_PREDICATE
			l_signature: G
			l_select, l_from_clause, l_where_clause: STRING
			l_types: ROSE_LINEAR_ARRAY[STRING]
		do
			create l_signature.default_create
			create Result.make_empty
			l_select := "select distinct " + l_signature.signature_and_data_fields_clause
			create l_count_predicate.make_strict (signature_item_server,"select count (*) from " + data_name_prefix + "data_signatures")
			l_count_predicate.retrieve (Void,Void)
			if a_items /= Void then
				if not a_items.is_empty and not a_process_ids.is_empty then
					create l_types.make_from_array (<<"REAL","INTEGER","DOUBLE","STRING","IMAGE","DATETIME">>)
					from l_types.start until l_types.after loop
						if data_type_available (l_types.current_item) then
							l_from_clause:=" from " + data_name_prefix + "data_signatures INNER JOIN " + data_name_prefix +
								"data_" + l_types.current_item + "_value ON " + data_name_prefix + "data_signatures.signature_id = " +
								data_name_prefix + "data_" + l_types.current_item + "_value.signature_id"
							l_where_clause := " where " + data_name_prefix + "data_" + l_types.current_item + "_value.process_id IN ("
							from a_process_ids.start until a_process_ids.off
							loop
								l_where_clause := l_where_clause + a_process_ids.current_item.out + ","
								a_process_ids.forth
							end
							l_where_clause.prune_all_trailing (',')
							l_where_clause:=l_where_clause + ")"
							l_where_clause := l_where_clause + " and " + data_name_prefix + "data_" + l_types.current_item + "_value.item_id IN ("
							from a_items.start until a_items.after
							loop
								l_where_clause := l_where_clause + a_items.current_item.out + ","
								a_items.forth
							end
							l_where_clause.prune_all_trailing (',')
							l_where_clause:=l_where_clause + ")"
							l_statement:=l_select + l_from_clause + l_where_clause
							create l_predicate.make (signature_item_server, l_statement)
							if not l_predicate.success then
								raise ("Failed to create predicate")
							end
							l_predicate.retrieve (Void, Void)
							if not l_predicate.success then
								raise ("Failed to execute predicate")
							end
							Result.append_array (l_predicate.array)
						end
						l_types.forth
					end
				end
			elseif l_count_predicate.array.first = 0 then
				create l_types.make_from_array (<<"REAL","INTEGER","DOUBLE","STRING","IMAGE","DATETIME">>)
				l_select := "select distinct signature_id "
				from l_types.start until l_types.off loop
					if data_type_available (l_types.current_item) then
						l_from_clause:=" from " + data_name_prefix + "data_" + l_types.current_item + "_value "
						l_where_clause := " where process_id IN ("
						from a_process_ids.start until a_process_ids.off
						loop
							l_where_clause := l_where_clause + a_process_ids.current_item.out + ","
							a_process_ids.forth
						end
						l_where_clause.prune_all_trailing (',')
						l_where_clause:=l_where_clause + ")"
						l_statement:=l_select + l_from_clause + l_where_clause
						create l_predicate.make (signature_item_server, l_statement)
						if not l_predicate.success then
							raise ("Failed to create predicate")
						end
						l_predicate.retrieve (Void, Void)
						if not l_predicate.success then
							raise ("Failed to execute predicate")
						end
						Result.append_array (l_predicate.array)
					end
					l_types.forth
				end
			else
				l_from_clause := " from " + data_name_prefix + "data_signatures"
				l_where_clause:=" where " + data_name_prefix + "data_signatures.last_process_id IN ("
				from a_process_ids.start until a_process_ids.off loop
					l_where_clause := l_where_clause + a_process_ids.current_item.out + ","
					a_process_ids.forth
				end
				l_where_clause.prune_all_trailing (',')
				l_where_clause:=l_where_clause + ")"
				l_statement:=l_select + l_from_clause + l_where_clause
				create l_predicate.make (signature_item_server, l_statement)
				if not l_predicate.success then
					raise ("Failed to create predicate")
				end
				l_predicate.retrieve (Void, Void)
				if not l_predicate.success then
					raise ("Failed to execute predicate")
				end
				Result.append_array (l_predicate.array)
			end
		end

	paths_touched_since (a_process_id: INTEGER): ROSE_LINEAR_ARRAY[INTEGER] is
			-- The collection of path ids updated since `a_process_id'
		do
			create Result.make_empty
		ensure
			valid_result: Result /= Void
		end

	records_count (a_name: STRING; a_value, a_item: INTEGER): INTEGER is
		-- a_name is the name of the signature_field you query on, a_value is the value you pass
		-- a_item=0 means that you count the number of signatures whatever the item is
		local
			l_statement: STRING
			l_predicate: INTEGER_PREDICATE
			l_select, l_from_clause, l_where_clause: STRING
			l_types: ROSE_LINEAR_ARRAY[STRING]
		do
			l_select:="select count (*)"
			if a_item /= 0 then
				create l_types.make_from_array (<<"REAL","INTEGER","DOUBLE","STRING","IMAGE","DATETIME">>)
				from l_types.start until l_types.off
				loop
					if data_type_available (l_types.current_item) then
						l_from_clause:=" from " + data_name_prefix + "data_signatures INNER JOIN " + data_name_prefix +
									   "data_" + l_types.current_item + "_value ON " + data_name_prefix + "data_signatures.signature_id = " +
									   data_name_prefix + "data_" + l_types.current_item + "_value.signature_id"
						l_where_clause:=" where " + data_name_prefix + "data_signatures." + a_name + "= ?"
						l_where_clause := l_where_clause + " and " + data_name_prefix + "data_" + l_types.current_item + "_value.item_id = ? "
						l_statement:=l_select + l_from_clause + l_where_clause
						create l_predicate.make (signature_item_server, l_statement)
						if not l_predicate.success then
							raise ("Failed to create predicate")
						end
						l_predicate.retrieve (Void, <<a_value, a_item>>)
						if not l_predicate.success then
							raise ("Failed to execute predicate")
						end
						if l_predicate.array.count>0 then
							Result := Result + l_predicate.array.item (1)
						end
					end
					l_types.forth
				end
			else
				l_from_clause := " from " + data_name_prefix + "data_signatures"
				l_where_clause:=" where " + data_name_prefix + "data_signatures." + a_name + "= ?"
				l_statement := l_select + l_from_clause + l_where_clause
				create l_predicate.make (signature_item_server, l_statement)
				if not l_predicate.success then
					raise ("Failed to create count predicate")
				end
				l_predicate.retrieve (Void,<<a_value>>)
				if not l_predicate.success then
					raise ("Failed to execute count predicate")
				end
				if l_predicate.array.count>0 then
					Result := l_predicate.array.item (1)
				end
			end
		end

feature {SIGNATURE_TAGGED_ITEM_DATABASE} -- Implementation

	clear_column_comparison is
		do
			column_comparison := Void
		end

	set_restriction_clause (a_restriction_clause: STRING) is
		do
			restriction_clause := a_restriction_clause
		end

	columns_from_restriction_statement (a_statement: STRING): ROSE_LINEAR_ARRAY[STRING] is
		local
			l_comparisons: ROSE_DELIMITED_TEXT
			l_comparison: STRING
			l_current_comparison: STRING
			l_signature: G
		do
			if a_statement /= Void then
				a_statement.to_lower
				restriction_clause := a_statement
				create l_comparisons.make_ignoring_repeats (a_statement,' ')
				create Result.make_empty
				Result.compare_objects
				create column_comparison.make_empty
				from l_comparisons.start until l_comparisons.off loop
					if l_comparisons.current_item.count>0 and then l_comparisons.current_item.item (1).is_equal ('#') then
						l_comparison := l_comparisons.current_item
						l_current_comparison := l_comparison.substring (2,l_comparisons.current_item.count - 3)
						if not Result.has (l_current_comparison) then
							Result.extend (l_current_comparison)
							column_comparison.extend (l_comparisons.item (l_comparisons.index + 1))
						end
					end
					l_comparisons.forth
				end
			else
				create l_signature
				create Result.make_empty
				from l_signature.signature_fields.start until l_signature.signature_fields.off loop
					Result.extend (l_signature.signature_fields.current_item)
					l_signature.signature_fields.forth
				end
			end
		end

	set_record_limit (a_record_limit: INTEGER) is
		do
			record_limit := a_record_limit
			set_ad_hoc_query (record_limit /= 0)
		end

	set_virtual_record_set (a_virtual_record_set: like virtual_record_set) is
		do
			virtual_record_set := a_virtual_record_set
		end

	remake_virtual_record_set is
		do
			create virtual_record_set.make_new_similar_to (virtual_record_set) -- Create new container to avoid changing manager_cache
		end

	set_cache_pool_on (a_value: BOOLEAN) is
		do
			virtual_record_set.set_cache_pool_on (a_value)
		end

	set_current_cache_pool (a_signature: G) is
		require
			signature_not_void: a_signature /= Void
		local
			l_path: INTEGER
		do
			if a_signature /= Void then
				l_path := path (a_signature)
				if l_path /= cache_pool_id or current_cache_pool = Void then
					current_cache_pool := cache_pool.item (l_path)
					cache_pool_id := l_path
					if current_cache_pool = Void then
						create current_cache_pool.make (l_path)
						current_cache_pool.set_virtual_record_template (virtual_record_template)
--						manager_cache.force (virtual_record_set,l_path)
						cache_pool.force (current_cache_pool,l_path)
					end
				end
				if new_query then
					current_cache_pool.reset (l_path)
					new_query := False
				end
			end
		end

	set_virtual_record_template (a_template: ROSE_VIRTUAL_RECORD[G]) is
		do
			virtual_record_template := a_template
			if current_cache_pool = Void then
				create current_cache_pool.make (0)
			end
			current_cache_pool.set_virtual_record_template (virtual_record_template)
			virtual_record_set.set_virtual_record_template (virtual_record_template)
		end

	prepare_to_retrieve_signatures is
		do
			-- Set up system
			actual_last_rose_signature := Void
			create virtual_record_set.make_new_similar_to (virtual_record_set) -- Create new container to avoid changing manager_cache
			create cube_signature_result.make_empty
			create cube_position_result.make (0)
		end

	retrieve_historical_signatures: ROSE_LINEAR_ARRAY[G] is
		do
			synchronize_process_dates
			create Result.make_empty
			if cube_signature_result.count > 0 then
				set_current_cache_pool (cube_signature_result.first)
				virtual_record_set.load_signature_versions (cube_signature_result, current_cache_pool, True)
				from virtual_record_set.cube_result.start until virtual_record_set.cube_result.off loop
					Result.extend (virtual_record_set.cube_result.current_item.signature)
					virtual_record_set.cube_result.forth
				end
				cube_result_table := virtual_record_set.cube_result_table
				actual_last_rose_signature := virtual_record_set.cube_result.last
				--set_cache_from_results (last_rose_signature)	-- Since this is no longer done by set_current_cache_pool.
			else
				create cube_result_table.make (0)
			end
		end

	retrieve_signature_data_history (a_signature: G): ROSE_LINEAR_ARRAY[G] is
		require
			valid_signature: a_signature /= Void
		local
			l_predicate: COLLECTING_PREDICATE[G]
		do
--			create l_predicate.make (signature_item_server,"select a.*,b.*,b.process_id as last_process_id from " +
			create l_predicate.make (signature_item_server,"select a.signature_id," + a_signature.fields_ + ",b.*,b.process_id as last_process_id from " +
									data_name_prefix + "data_signatures a, " +
									data_name_prefix + "data_signatures_data_history b (NOLOCK) " +
									"where a.signature_id = ? and a.signature_id = b.signature_id order by process_id")
			l_predicate.retrieve (Void,<<a_signature.signature_id>>)
			if not l_predicate.success then
				raise ("l_predicate in retrieve_signature_data_history failed.")
			end
			Result := l_predicate.array
		end

	ordered_cube_of_items_from_predicate (a_predicate: ANONYMOUS_PREDICATE; a_argument: ANY; a_arguments: ARRAY[ANY];
										 a_items: ROSE_LINEAR_ARRAY[INTEGER]): ROSE_LINEAR_ARRAY[ROSE_VIRTUAL_RECORD[G]] is
		do
			-- Set up system
			actual_last_rose_signature := Void
			create cube_signature_result.make_empty
			create cube_result_table.make (0)
			create cube_position_result.make (0)
			create virtual_record_set.make_new_similar_to (virtual_record_set) -- Create new container to avoid changing manager_cache
--			new_query := True

			-- Prepare the item map.
			create_signatures_sql_item_map (a_predicate)

			signatures_predicate_statement := ""	-- Should be the actual predicate text
													-- but it is not exported.
			a_predicate.apply (agent handle_signatures, a_argument, a_arguments)
			if not a_predicate.success then
				raise ("a_predicate failed.")
			end

			Result := ordered_cube_of_items_base (a_items)
		end

	ordered_cube_of_items (a_columns: COLUMN_DESCRIPTORS[G];
						   a_items: ROSE_LINEAR_ARRAY[INTEGER]): ROSE_LINEAR_ARRAY[ROSE_VIRTUAL_RECORD[G]] is
		do
			-- Set up system
			actual_last_rose_signature := Void
			create cube_signature_result.make_empty
			create cube_result_table.make (0)
			create cube_position_result.make (0)
			create virtual_record_set.make_new_similar_to (virtual_record_set) -- Create new container to avoid changing manager_cache
			--new_query := True

			-- Get signature ids to be retrieved
			select_signature_ids (a_columns)

			Result := ordered_cube_of_items_base (a_items)
		end

	ordered_cube_of_items_by_signatures (a_signatures: ROSE_LINEAR_ARRAY[G];
										 a_items: ROSE_LINEAR_ARRAY[INTEGER]): ROSE_LINEAR_ARRAY[ROSE_VIRTUAL_RECORD[G]] is
		do
			-- Set up system
			actual_last_rose_signature := Void
			create cube_result_table.make (0)
			create cube_position_result.make (0)
			create virtual_record_set.make_new_similar_to (virtual_record_set) -- Create new container to avoid changing manager_cache

			cube_signature_result := a_signatures
			Result := ordered_cube_of_items_base (a_items)
		end

	create_signatures_sql_item_map (a_predicate: ANONYMOUS_PREDICATE) is
		require
			predicate_success: a_predicate /= Void and then a_predicate.success
		local
			j: INTEGER
			l_signature: G
			l_output_index: INTEGER
		do
			create l_signature.default_create
			create signatures_sql_item_map.make (1, (a_predicate.upper_output_index - 4).max (0))
			from j := 1 until j > internal.field_count (l_signature) loop
				-- Check for columns that have more than 1 entry in the query
				l_output_index := a_predicate.output_index (internal.field_name (j,l_signature))
				if l_output_index /= 0 then
					signatures_sql_item_map.force (j,l_output_index)
				end
				-- Check for columns that have only 1 entry in the query
				l_output_index := a_predicate.output_index (internal.field_name (j,l_signature) + "_")
				if l_output_index /= 0 then
					signatures_sql_item_map.force (j,l_output_index)
				end
				j := j + 1
			end
		end
	signatures_sql_item_map: ROSE_LINEAR_ARRAY[INTEGER]
	signatures_predicate_statement: STRING
	signatures_predicate: ANONYMOUS_PREDICATE

	create_selected_column_ids is
		do
			create selected_column_ids.make_empty
		end

feature {NONE} -- Metrics

	actual_metrics: ROSE_LINEAR_ARRAY[PAIR[STRING, DOUBLE]]
	load_metrics (a_item: like submittable_anchor) is
		require
			valid_item: a_item /= Void
		do
			actual_metrics.append_array (a_item.metrics)
		end

	reset_item_metrics (a_item: like submittable_anchor) is
		require
			valid_item: a_item /= Void
		do
			a_item.reset_metrics
		end

	sig_tagged_trace_requested: BOOLEAN is
		once
			Result := sig_tagged_trace_file /= Void
		end

	sig_tagged_trace_file: RIEM_FILE is
		local
			l_env: ROSE_EXECUTION_ENVIRONMENT
			l_file_name: STRING
		once
			create l_env
			l_file_name := l_env.get ("SIG_TAGGED_TRACE")
			if l_file_name/=Void and then l_file_name.count/=0
			then
				create Result.make_open_write (l_file_name)
			end
		end

feature {SIGNATURE_TAGGED_ITEM_DATABASE} -- Metrics

	close_sig_tagged_trace is
		do
			if sig_tagged_trace_file/=Void
			then
				sig_tagged_trace_file.close
			end
		end

	metrics: ROSE_LINEAR_ARRAY[PAIR[STRING, DOUBLE]] is
		local
			l_pair_created, l_pair_replaced: PAIR[STRING,DOUBLE]
		do
			create actual_metrics.make_empty
			data_items.do_all (agent load_metrics)
			Result := actual_metrics
			if not signature_item_server.registered then
				Result.append_linear (signature_item_server.metrics)
			end
			if signature_item_fast_load_server_saved /= Void and then
				not signature_item_fast_load_server_saved.registered then
				Result.append_linear (signature_item_fast_load_server.metrics)
			end
			create l_pair_created.make (data_name_prefix + "DATA_SIGNATURES_CREATED", signatures_created)
			Result.extend (l_pair_created)
			create l_pair_replaced.make (data_name_prefix + "DATA_SIGNATURES_REPLACED", signatures_replaced)
			Result.extend (l_pair_replaced)
			actual_metrics := Void
		end

	reset_metrics is
		do
			signatures_created := 0
			signatures_replaced := 0
			data_items.do_all (agent reset_item_metrics)
		end

feature {SIGNATURE_TAGGED_ITEM_DATABASE} -- Path

	set_path (a_record: ANY; a_signature: G) is
		do
		end

	path (a_signature: G): INTEGER is
		do
		end

feature {SIGNATURE_TAGGED_ITEM_DATABASE} -- Signature

	submit_signature_data_updates is
			-- Possibly correct comment: Records changes to the data portions of signatures?
		local
			l_from_signature, l_to_signature: G
		do
			if signature_item_server.error_handler = Void then
				-- This print statement is designed to catch the programmers attention.  Don't remove it.
				print ("Rather than crashing, SIGNATURE_TAGGED_SERVER:submit_signature_data_updates%N")
				print ("will now auto-generate an error_handler.%N")
				signature_item_server.set_error_handler (signature_item_server.new_error_handler_print)
			end
			signature_item_server.allow_error (Rose_sql_error_dup_key)
			from signature_data_updates.start until signature_data_updates.off loop
				l_from_signature := signature_data_updates.current_item.first
				l_to_signature := signature_data_updates.current_item.second
				signature_data_history_insert_predicate.execute (l_from_signature, Void)
				if not signature_data_history_insert_predicate.success then
					fail ("submit_sigature_data_updates",Global.interpreted ("($now) could not insert on " + signature_item_server.definition.full_path + "%N"),Void)
				end
				l_to_signature.set_last_process_id (process_id)
				l_to_signature.set_signature_data_process_id (process_id)
				signature_data_update_predicate.execute (l_to_signature, Void)
				if not signature_data_update_predicate.success then
					fail ("submit_sigature_data_updates",Global.interpreted ("($now) could not update on " + signature_item_server.definition.full_path + "%N"),Void)
				end
				signature_data_updates.forth
			end
			signature_item_server.dont_allow_error (Rose_sql_error_dup_key)
			create signature_data_updates.make_empty
		end

	signature_loader_server: SQL_DATA_SERVER
			-- New server connection used by temp_signature_loader

	table_exists(a_server: SQL_DATA_SERVER; a_table_name: STRING): BOOLEAN is
			-- Does 'a_table' already exist?
		local
			l_predicate: ANONYMOUS_PREDICATE
		do
			if a_table_name.item(1)='#'
			then
				if a_table_name.item(2)='#'
				then
					-- Tables of the form ##name are really in tempdb.
					create l_predicate.make_using_server(
						a_server, "select * from tempdb..sysobjects with (nolock) where name = " + "'" + a_table_name + "'")
				else
					-- Tables of the form #name are actually stored as #name______...stuff deleted..._____019212930.
					-- Calling OBJECT_ID() gives the true id (but don't use OBJECT_NAME(OBJECT_ID()) - I already tried it).
					create l_predicate.make_using_server(
						a_server, "select * from tempdb..sysobjects with (nolock) where id = " + "OBJECT_ID('tempdb.." + a_table_name + "')")
				end
			else
				create l_predicate.make_using_server(
					a_server, "select * from sysobjects with (nolock) where name = " + "'" + a_table_name + "'")
			end

			if not l_predicate.success
			then
				raise("Could not make l_predicate")
			end

			l_predicate.retrieve(Void, Void)
			if not l_predicate.success
			then
				raise("l_predicate failed for " + a_server.definition.path + "%N")
			elseif not l_predicate.array.is_empty
			then
				Result := True
			end
		end

	submit_signature_touches_server: SQL_DATA_SERVER
			-- New server connection used to update the signature versions table

--	signature_touches_temp_table_exists: BOOLEAN is
--			-- Does the temp table that is used for the signature touches update already exist?
--		local
--			l_predicate: ANONYMOUS_PREDICATE
--		do
--			if submit_signature_touches_server = Void then
--				submit_signature_touches_server := servers.new_server_from_repository (signature_item_server.logical)
--			end
--			create l_predicate.make_using_server (submit_signature_touches_server, "exec sp_tables '" + signature_touches_table_name + "'")
--			l_predicate.retrieve (Void, Void)
--			if not l_predicate.success then
--				raise ("sp_tables failed for " + submit_signature_touches_server.definition.path + "%N")
--			elseif not l_predicate.array.is_empty then
--				Result := True
--			end
--		end

	submit_signature_touches (a_signature_touches: ROSE_LINEAR_ARRAY[G]) is
			-- Ensure that the versions table is updated
		require
			valid_signature_touches: a_signature_touches /= Void
		local
			l_fast_insert_predicate: DATA_SERVER_FAST_LOAD
			l_signature: G
			l_fast_data_server: SQL_DATA_SERVER
		do
			if not a_signature_touches.is_empty then
				if submit_signature_touches_server = Void then
					submit_signature_touches_server := servers.new_server_from_repository (signature_item_server.logical)
					signature_touches_table_name := "##signature_touch_" + process_id.out + "_" + table_structure.server_spid.out + data_name_prefix

					if table_exists(submit_signature_touches_server, signature_touches_table_name)
					then
						submit_signature_touches_server.sql_execute_immediate("drop table " + signature_touches_table_name)
					end

					submit_signature_touches_server.sql_execute_immediate ("create table " + signature_touches_table_name + "(signature_id int,continuing_process_id int)")
					if not submit_signature_touches_server.success then
						fail ("submit_sigature_touches", Global.interpreted ("($now) could not create the submit_signature_touches temp table on " + signature_item_server.definition.full_path + "%N"),Void)
					end

					submit_signature_touches_server.sql_execute_immediate ("create index signature_index on " + signature_touches_table_name + " (signature_id)")
					if not submit_signature_touches_server.success then
						fail ("submit_signature_touches", "Could not create index", Void)
					end

					submit_signature_touches_server.commit
					if not submit_signature_touches_server.success then
						fail ("submit_signature_touches", "Commit failed after creation of " + signature_touches_table_name, Void)
					end
				end

				create l_signature.default_create
				l_fast_data_server := servers.make_from_repository_fast_load (signature_item_server.logical)
				l_fast_insert_predicate := l_fast_data_server.fast_loader (signature_touches_table_name, l_signature)

				from a_signature_touches.start until a_signature_touches.off loop
					l_fast_insert_predicate.insert (a_signature_touches.current_item)
					a_signature_touches.forth
				end

				l_fast_insert_predicate.commit
				if not l_fast_insert_predicate.success then
					fail ("submit_signature_touches", "failure in fast insert commit", Void)
				end

				submit_signature_touches_update
				a_signature_touches.wipe_out

				signature_item_server.sql_execute_immediate ("truncate table " + signature_touches_table_name)
--				submit_signature_touches_server.commit
--				if not submit_signature_touches_server.success then
--					fail ("submit_signature_touches", "failure in the commit for the drop table", Void)
--				end
			end
		end

	signature_touches_table_name: STRING

	submit_signature_touches_update is
			-- Update versions table but attempt a retry if update fails because of a deadlock with another process
		local
			l_retry: INTEGER
		do
			signature_item_server.sql_execute_immediate ("update " + data_name_prefix + "data_signatures " +
														"set " +
														--"continuing_process_id = " + signature_touches_table_name + ".continuing_process_id " +
														"continuing_process_id = " + process_id.out +
														" from " +
														signature_touches_table_name + " " +
														"where " +
														data_name_prefix + "data_signatures.signature_id = " + signature_touches_table_name + ".signature_id option (loop join)")
			if not signature_item_server.success then
				output (Global.interpreted ("($now) " + data_name + " update in submit_signature_touches failed on " + signature_item_server.definition.full_path + "%N"),Void)
				raise (data_name + " update in submit_signature_touches failed on " + signature_item_server.definition.full_path)
			end
		rescue
			if l_retry < retry_limit then
				l_retry := l_retry + 1
				retry
			end
		end

feature {SIGNATURE_TAGGED_ITEM_DATABASE,SERVED_CLUSTERED_TAGGED_ITEM_DATABASE} -- Implementation

	signature_id (a_signature: G; a_can_read_old_signature, a_can_write_new_signature: BOOLEAN): INTEGER
			-- Returns the signature_id associated with the keys in 'a_signature'.  If no signature can
			-- be found in the data_signatures table, a new signature is created.  If 'a_can_read_old_signature'
			-- is False, no attempt will be made to check for an existing record in the table (presumably,
			-- you already called this feature with 'a_can_read_old_signature' set to True.  If 'a_can_write_new_signature'
			-- is False, no new signature will be created and Result may be zero.
		require
			a_signature_not_void: a_signature /= Void
			must_read_or_write: a_can_read_old_signature or a_can_write_new_signature
		local
			l_integer_ref: INTEGER_REF
		do
			if a_can_read_old_signature
			then
				signature_search.retrieve (a_signature,Void)
				if not signature_search.success then
					raise ("signature_search failed.")
				end
			else
				signature_search.set_container(empty_array)
			end

			if signature_search.array.count = 1 then
				l_integer_ref ?= signature_search.array.item (1).item (1)
				if l_integer_ref.item /= 0 then
					Result := l_integer_ref.item
					last_signature_created := False
				end
			elseif signature_search.array.count = 0 then
				if a_can_write_new_signature
				then
					if client_can_write then
						if table_structure.signature_data_process_id_available then
							signature_insert_with_signature_data_process_id.retrieve (a_signature,<<process_id,process_id,process_id,process_id>>)
							if not signature_insert_with_signature_data_process_id.success then
								raise ("ensure that identity is set to 'YES' on signature_id in " + data_name_prefix + "data_signatures")
							end
							a_signature.set_signature_data_process_id (process_id)
							Result := signature_insert_with_signature_data_process_id.array[1]
						else
							signature_insert.retrieve (a_signature, <<process_id, process_id,process_id>>)
							if not signature_insert.success then
								raise ("ensure that identity is set to 'YES' on signature_id in " + data_name_prefix + "data_signatures")
							end
							Result := signature_insert.array[1]
						end
						last_signature_created := True
						signatures_created := signatures_created + 1
					else
						raise ("This client cannot write to " + data_server_full_name)
					end
				end
			else
				raise ("duplicate signature, check " + data_name_prefix + "data_signatures")
			end
		ensure
			valid_result: Result /= 0 or not a_can_write_new_signature
		end

	signature_id_if_found (a_signature: G): INTEGER is
			-- signature id if found
		local
			l_integer_ref: INTEGER_REF
		do
			signature_search.retrieve (a_signature,Void)
			if not signature_search.success then
				raise ("signature_search failed.")
			end
			if signature_search.array.count = 1 then
				l_integer_ref ?= signature_search.array.item (1).item (1)
				if l_integer_ref.item /= 0 then
					Result := l_integer_ref.item
					-- Removed.	 Signatures are being touched by the trigger signature_version_update
					--a_signature.set_last_process_id (process_id)
					--if not last_signature_created then
					--signature_touch.execute (a_signature,<<process_id>>)
					--if not signature_touch.success then
					--raise ("ensure that identity is set to 'YES' on id in " + data_name_prefix + "data_signatures")
					--end
					--end
					last_signature_created := False
				end
			elseif signature_search.array.count > 1 then
				raise ("duplicate signature, check " + data_name_prefix + "data_signatures")
			end
		end

	retrieve_signature_data (a_signature: G): G is
			-- retrieve signature data
		do
			signature_data_fields_predicate.retrieve (a_signature,Void)
			if not signature_data_fields_predicate.success then
				raise ("signature_data_process_id_search failed.")
			end
			if signature_data_fields_predicate.array.count = 1 then
				Result := signature_data_fields_predicate.array.item (1)
			end
		end

	select_signature_ids (a_columns: COLUMN_DESCRIPTORS[G]) is
		local
			l_column_selected: STRING
			l_insert_predicate: EXECUTION_PREDICATE
			l_predicate_arguments: ROSE_LINEAR_ARRAY[ANY]
			l_column_names: ROSE_LINEAR_ARRAY[STRING]
			l_comparisons: ROSE_DELIMITED_TEXT
			l_comparison: STRING
			l_current_comparison: STRING
			j: INTEGER
			l_shows_up_more_than_once: BOOLEAN
			l_date: ROSE_DATE
			l_predicate: ANONYMOUS_PREDICATE
			l_fast_load_predicate: DATA_SERVER_FAST_LOAD
			l_table_name: STRING
			l_integer_ref: INTEGER_REF
		do
			if a_columns /= Void then
				create l_predicate_arguments.make_empty
				l_predicate_arguments.compare_references
				create l_column_names.make_empty
				from a_columns.start until a_columns.off loop
					-- Use temporary tables to select signatures only when there are more than 1 column values
					l_column_selected := selected_column (a_columns.current_first,a_columns.current_count > 1,False)
					l_shows_up_more_than_once := False
					if restriction_clause /= Void then
						from j := restriction_clause.substring_index ("#" + l_column_selected,j + 1)
						until j > restriction_clause.count or j = 0 loop
							j := restriction_clause.substring_index ("#" + l_column_selected,j + 1)
							if j > 0 then
								l_shows_up_more_than_once := True
							end
						end
					end
					l_column_selected := selected_column (a_columns.current_first,a_columns.current_count > 1 or l_shows_up_more_than_once,True)
					if l_column_selected /= void then
						if a_columns.current_count = 1 and not l_shows_up_more_than_once then
							l_predicate_arguments.extend (a_columns.current_first.value_by_name (l_column_selected))
						else
							if l_column_selected.as_lower.is_equal ("signature_id") then
								create l_date.make_now
								l_table_name := "##" + l_column_selected + l_date.ordered_compact_date.out + l_date.persecs.out
								signature_item_server.sql_execute_immediate ("create table " + l_table_name + " (item int" + ")")
								signature_item_server.commit
								l_fast_load_predicate := signature_item_fast_load_server.fast_loader (l_table_name, create {CELL[INTEGER]}.put (0))
								from a_columns.current_start until a_columns.current_off loop
									l_integer_ref ?= a_columns.current_current_item.value_by_name (l_column_selected)
									l_fast_load_predicate.insert (create {CELL[INTEGER]}.put (l_integer_ref.item))
									a_columns.current_forth
								end
								l_fast_load_predicate.commit
								create l_insert_predicate.make (signature_item_server,"insert into #" + l_column_selected + " (id) select item from " + l_table_name)
								if not l_insert_predicate.success then
									raise ("Failed to create l_insert_predicate")
								end
								l_insert_predicate.execute (Void,Void)
								if not l_insert_predicate.success then
									raise ("l_insert_predicate failed.")
								end
								signature_item_server.sql_execute_immediate ("drop table " + l_table_name)
							else
								create l_insert_predicate.make (signature_item_server, "insert into #" + l_column_selected + " (id) values (?)")
								if not l_insert_predicate.success then
									raise ("Failed to create l_insert_predicate")
								end
								signature_item_server.allow_error(Rose_sql_error_dup_key)
								from
									a_columns.current_start
								until
									a_columns.current_off
								loop
									l_insert_predicate.execute (Void, << a_columns.current_current_item.value_by_name (l_column_selected) >>)
									if not l_insert_predicate.success then
										raise ("l_insert_predicate failed.")
									end
									a_columns.current_forth
								end
								signature_item_server.dont_allow_error(Rose_sql_error_dup_key)
							end
						end
						l_column_names.extend (l_column_selected)
					else
						l_predicate_arguments.extend (0)
					end
					a_columns.forth
				end
			end
			if restriction_clause /= Void then
				restriction_clause.to_lower
				create l_comparisons.make_ignoring_repeats (restriction_clause,' ')
				--create column_comparison.make_empty
				from l_comparisons.start until l_comparisons.off loop
					if l_comparisons.current_item.count > 0 and then l_comparisons.current_item.item (1).is_equal ('#') then
						l_comparison := l_comparisons.current_item
						l_current_comparison := l_comparison.substring (2,l_comparisons.current_item.count - 3)
						from j := 1 until j > internal.field_count (a_columns.first.first) loop
							if field_name (j,a_columns.first.first).is_equal (l_current_comparison) and not selected_column_ids.has (j) then
								selected_column_ids.extend (j)
							end
							j := j + 1
						end
					end
					l_comparisons.forth
				end
			end
			if column_comparison = Void then
				create_default_comparisons
				create column_comparison.make (1,selected_columns.count)
				from column_comparison.start until column_comparison.off loop
					column_comparison.put ("=",column_comparison.index)
					column_comparison.forth
				end
			end
			-- Set signature_ids to be retrieved
			l_predicate := signature_selection
			if l_predicate_arguments.count = 0 then
				l_predicate.apply (agent handle_signatures,Void,Void)
			else
				l_predicate.apply (agent handle_signatures,Void,l_predicate_arguments)
			end

			if not l_predicate.success then
				raise ("signature_selection failed.")
			end

			if sig_tagged_trace_requested
			then
				sig_tagged_trace_file.put_string (database_name + ":" + table_name + ":SIGNATURE_TAGGED_SERVER:select_signature_ids:")
				from
					l_predicate_arguments.start
				until
					l_predicate_arguments.off
				loop
					sig_tagged_trace_file.put_string (l_predicate_arguments.current_item.out)
					l_predicate_arguments.forth
					if not l_predicate_arguments.off
					then
						sig_tagged_trace_file.put_string (", ")
					end
				end

				sig_tagged_trace_file.put_string ("%N")
			end

			drop_column_temp_tables
		end

feature {SIGNATURE_TAGGED_ITEM_DATABASE} -- Data replace

	real_data_replace (a_record: DATA_RECORD[REAL]) is
		do
			real_data.replace (a_record)
		end

	double_data_replace (a_record: DATA_RECORD[DOUBLE]) is
		do
			double_data.replace (a_record)
		end

	integer_data_replace (a_record: DATA_RECORD[INTEGER]) is
		do
			integer_data.replace (a_record)
		end

	string_data_replace (a_record: DATA_RECORD[STRING]) is
		do
			string_data.replace (a_record)
		end

	image_data_replace (a_record: DATA_RECORD[STRING]) is
		do
			image_data.replace (a_record)
		end

	date_data_replace (a_record: DATA_RECORD[ROSE_DATE]) is
		do
			date_data.replace (a_record)
		end

	real_override_data_replace (a_record: DATA_RECORD[REAL]) is
		do
			real_override_data.replace (a_record)
		end

	delete_signature (a_signature: G) is
			-- Delete 'a_signature' from the signature table
			-- Only use for databases with no item data		
		require
			valid_signature: a_signature /= Void
			signature_only: table_structure.signature_only
		local
			l_execution_predicate: EXECUTION_PREDICATE
			l_delete_statement: STRING
		do
			l_delete_statement := "delete from " + data_name_prefix + "data_signatures where signature_id = ?"
			create l_execution_predicate.make (signature_item_server, l_delete_statement)
			if not l_execution_predicate.success then
				fail ("delete signature_record", "Failed to create execution predicate : " + l_delete_statement, Void)
			end
			l_execution_predicate.execute (Void, <<a_signature.signature_id>>)
			if not l_execution_predicate.success then
				fail ("delete signature_record", "Failed to execute predicate : " + l_delete_statement, Void)
			end
		end

	replace_signature (a_signature: G) is
			-- replace signature
		do
			signature_update_predicate.execute (a_signature, Void)
			if not signature_update_predicate.success then
				raise ("signature_update_predicate failed.")
			end
			signatures_replaced := signatures_replaced + 1
		end

	replace_signature_data_array (a_sigs: ROSE_LINEAR_ARRAY[G]) is
			-- Like calling 'replace_signature_data' many times, but much faster.
			-- You MUST perform a commit before calling this routine again.
		do
			if a_sigs.count>0 then
				load_signatures_into_temp_table(a_sigs)		-- Sets 'signature_loader_server'.
				load_signature_data_from_table(signature_loader_table_name)
				signature_item_server.sql_execute_immediate("truncate table " + signature_loader_table_name)
				if not signature_item_server.success
				then
					fail("replace_signature_data_array", "Could not truncate table", Void)
				end
				signature_loader_table_name := Void
			end
		end

	load_signature_data_from_table (a_table_name: STRING) is
			-- This is just like `replace_signature_data_array', but the data is already in a table.
			-- You MUST perform a commit before calling this routine again.
		require
			valid_table_name: a_table_name /= Void and then not a_table_name.is_empty
		local
			l_statement: STRING
			l_signature: G
			l_signature_fields: ROSE_LINEAR_ARRAY[STRING]
		do
			signature_loader_table_name := a_table_name
			create l_signature

			-- Update the signature_id's in the table from the signature_id's in data_signatures.
			-- Set all of the process_id's as well.
			l_statement := "update " + a_table_name +
				" set signature_id=a.signature_id, process_id=" + process_id.out + " " +
				"from " + data_name_prefix + "data_signatures a, " + a_table_name + " b " +
				"where "
			from
				l_signature_fields := l_signature.signature_fields
				l_signature_fields.start
			until
				l_signature_fields.off
			loop
				if not l_signature_fields.at_first
				then
					l_statement.append(" and ")
				end

				l_statement.append("a." + l_signature_fields.current_item + "=b." + l_signature_fields.current_item)
				l_signature_fields.forth
			end

			signature_item_server.sql_execute_immediate(l_statement)
			if not signature_item_server.success
			then
				fail("load_signature_data_from_table", Global.interpreted("($now) the update predicate (1a) failed on " + signature_item_server.definition.full_path + "%N"), Void)
			end

			-- Update the process_id's that weren't already in set above because they are new and we didn't find them in data_signatures.
			l_statement := "update " + a_table_name + " set process_id=" + process_id.out + " where process_id=0"
			signature_item_server.sql_execute_immediate(l_statement)
			if not signature_item_server.success
			then
				fail("load_signature_data_from_table", Global.interpreted("($now) the update predicate (1b) failed on " + signature_item_server.definition.full_path + "%N"), Void)
			end

			-- Copy changed data to the data_history table.
			if table_structure.signature_data_process_id_available
			then
				signature_data_history_insert2_predicate.execute(Void, Void)
				if not signature_item_server.success
				then
					fail("load_signature_data_from_table", Global.interpreted("($now) the history insert predicate failed on " + signature_item_server.definition.full_path + "%N"), Void)
				end
			end

			-- Update the existing records that have changed.
			signature_update2_predicate.execute(Void, Void)
			if not signature_update2_predicate.success
			then
				fail("load_signature_data_from_table", Global.interpreted("($now) the update predicate (2) failed on " + signature_item_server.definition.full_path + "%N"), Void)
			end
			signatures_replaced := signatures_replaced + signature_item_server.rows_affected

			-- Update the continuing_process_id on the existing records that have NOT changed.
			signature_update3_predicate.execute(Void, Void)
			if not signature_update3_predicate.success
			then
				fail("load_signature_data_from_table", Global.interpreted("($now) the update predicate (3) failed on " + signature_item_server.definition.full_path + "%N"), Void)
			end

			-- Insert the new records.
			signature_insert_from_table_predicate.execute(Void, Void)
			if not signature_insert_from_table_predicate.success
			then
				fail("load_signature_data_from_table", Global.interpreted("($now) the insert predicate failed on " + signature_item_server.definition.full_path + "%N"), Void)
			end
		end

	alter_table(a_server: SQL_DATA_SERVER; a_table_name, a_alter_statement: STRING) is
			-- For instance:
			--		alter table ##temp3 drop column signature_id
		do
			a_server.allow_error(Rose_sql_error_no_column)	-- Don't print message or fail on this error.
			a_server.sql_execute_immediate("alter table " + a_table_name + " " + a_alter_statement)
			if not a_server.success then
				fail("alter_table", Global.interpreted("($now) " + a_alter_statement + " failed for " + signature_item_server.definition.full_path + "%N"), Void)
			end
			a_server.dont_allow_error(Rose_sql_error_no_column)
		end

	load_signatures_into_temp_table(a_sigs: ROSE_LINEAR_ARRAY[G]) is
			-- Loads signatures into a temp table.
		local
			l_fast_insert_predicate: DATA_SERVER_FAST_LOAD
			l_signature: G
			l_fast_data_server: SQL_DATA_SERVER
		do
			if a_sigs.count > 0 then
				if signature_loader_server = Void then
					signature_loader_server := servers.new_server_from_repository(signature_item_server.logical)
					signature_loader_server.set_implicit_transaction_off
				end
				if temp_signature_loader_table_name = Void then
					temp_signature_loader_table_name := "##" + data_name_prefix + process_id.out + "signature_loader"
				end
				signature_loader_table_name := temp_signature_loader_table_name
				--			signature_loader_table_name := "tt" + data_name_prefix + process_id.out + "signature_loader"	-- For testing.
				if not table_exists(signature_item_server, signature_loader_table_name) then
					-- The next statement create a table just like data_signatures
					signature_loader_server.sql_execute_immediate("select top 0 * into " + signature_loader_table_name + " from " + data_name_prefix + "data_signatures")
					if not signature_loader_server.success then
						fail("load_signatures_into_temp_table", Global.interpreted("($now) could not create the load_signatures_into_temp_table temp table on " + signature_item_server.definition.full_path + "%N"), Void)
					end
					alter_table(signature_loader_server, signature_loader_table_name, "drop column signature_id")
					alter_table(signature_loader_server, signature_loader_table_name, "add signature_id int default 0 not null")
					alter_table(signature_loader_server, signature_loader_table_name, "add process_id int default 0 not null")
					if has_column (data_name_prefix + "data_signatures", "timestamp") then
						alter_table(signature_loader_server, signature_loader_table_name, "drop column timestamp")
					end
					alter_table(signature_loader_server, signature_loader_table_name, "drop column first_process_id")
					alter_table(signature_loader_server, signature_loader_table_name, "drop column last_process_id")
					alter_table(signature_loader_server, signature_loader_table_name, "drop column continuing_process_id")
					alter_table(signature_loader_server, signature_loader_table_name, "drop column end_current_date")
					--				signature_loader_server.sql_execute_immediate("create index signature_index on " + signature_loader_table_name + " (signature_id)")
					--				if not signature_loader_server.success
					--				then
					--					fail("load_signatures_into_temp_table", "Could not create index", Void)
					--				end
					signature_loader_server.commit
				end
				-- Because we are about to use a fast server, there should not be any open transactions
				-- on the temp table.  If there is, the fast server will be blocked.  What this boils down
				-- to is that the caller should do a commit between calls to this routine.
				create l_signature.default_create
				l_fast_data_server := servers.make_from_repository_fast_load(signature_item_server.logical)
				l_fast_insert_predicate := l_fast_data_server.fast_loader(signature_loader_table_name, l_signature)
				from
					a_sigs.start
				until
					a_sigs.off
				loop
					l_fast_insert_predicate.insert(a_sigs.current_item)
					a_sigs.forth
				end
				l_fast_insert_predicate.commit
				if not l_fast_insert_predicate.success then
					fail("load_signatures_into_temp_table", "failure in fast insert commit", Void)
				end
			end
		end

	replace_signature_data (a_signature: G; a_from_signature: G) is
			-- Replace the data in signature 'a_from_signature' to the data in 'a_signature'
		require
			known_signature: a_signature /= Void and then a_signature.signature_id /= 0
			known_from_signature: a_from_signature /= Void and then a_from_signature.signature_id /= 0
		do
			if not a_signature.data_matches (a_from_signature) then
				if table_structure.signature_data_process_id_available then
					signature_data_updates.extend (create {PAIR[G,G]}.make (a_from_signature.twin, a_signature.twin))
					uncommitted := True
				else
					raise ("Need to add column called SIGNATURE_DATA_PROCESS_ID of type int in data_signatures table")
				end
			end
		end

feature {SIGNATURE_TAGGED_ITEM_DATABASE,SERVED_CLUSTERED_TAGGED_ITEM_DATABASE} -- Process ID management

	set_process_id_manager (a_process_id_manager: SUPPLIER_PROCESS_ID_MANAGER) is
		do
			process_id_manager_saved := a_process_id_manager
			process_id_manager_created := True
		end

	process_id: INTEGER is
			-- This may call 'start_process' if the 'process_id_manager' has not yet been started.
		local
			l_time_now: like time_now
		do
			if process_id_set then
				Result := set_process_id_value
			else
				if not process_id_manager_created then
					l_time_now := time_now
					start_process (Void, l_time_now, l_time_now, l_time_now)
					output (Global.interpreted ("($now) signature_tagged_server: Created process_id " + process_id_manager.process_id.out + " on " + data_server_full_name + "%N"),Void)
				end
				Result := process_id_manager.process_id
			end
		end

	process_id_manager: SUPPLIER_PROCESS_ID_MANAGER is
		do
			if process_id_manager_saved = Void then
				create process_id_manager_saved.make (signature_item_server)
				process_id_manager_created := TRUE
			end
			Result := process_id_manager_saved
		end
	process_id_manager_saved: SUPPLIER_PROCESS_ID_MANAGER

	process_id_set: BOOLEAN
			-- Was the process id externally set?

	set_process_id (a_process_id: INTEGER) is
		do
			process_id_set := True
			if set_process_id_value /= a_process_id then
				set_process_id_value := a_process_id
				if process_id_map.item (a_process_id) = Void then
					if file_date_predicate_saved = Void then
						create file_date_predicate_saved.make (signature_item_server, "select file_date from process (nolock) where process_id = ?")
						if not file_date_predicate_saved.success then raise ("Fail to create file_date selection predicate") end
					end
					file_date_predicate_saved.retrieve (Void, <<a_process_id>>)
					if not file_date_predicate_saved.success then
						raise ("Fail to retrieve file_date with selection predicate")
					elseif file_date_predicate_saved.array.count /= 1 then
						raise ("Error: process_id " + a_process_id.out + " does not correspond to one single file date")
					else
						process_id_map.put (file_date_predicate_saved.array.first, a_process_id)
					end
				end
				--		create_process_table
				--		signature_item_server.sql_execute_immediate ("update #process set process_id = " + a_process_id.out)
				--		if not signature_item_server.success then
				--		   raise ("Could not insert the process_id into the process table.")
				--		end
				--		data_items.do_all (~set_item_process_id (?,a_process_id))
			end
		end
	file_date_predicate_saved: DATE_PREDICATE

feature {SIGNATURE_TAGGED_ITEM_DATABASE} -- Process ID Management

	update_end_time is
			-- update the end time for the process_id manager if it has been created
		do
			if process_id_manager_created then
				process_id_manager.set_end_time (create {ROSE_DATE}.make_now)
				process_id_manager.update_end_time
			end
		end

	start_process (a_process_name: STRING; a_file_date, a_market_date, a_rose_date: ROSE_DATE) is
		do
			if a_process_name /= Void then
				process_name := a_process_name
			end
			process_id_manager.start_process (process_name, a_file_date, a_market_date, a_rose_date)
			if process_id_map.item (process_id_manager.process_id) = Void then
				process_id_map.put (process_id_manager.file_date, process_id_manager.process_id)
			end
			signature_item_server.commit
		end

	terminate_process_id is
		do
			if process_id_manager_created then
				process_id_manager.set_end_time (time_now)
				process_id_manager.update_end_time
				signature_item_server.commit
			end
		end

	process_id_manager_created: BOOLEAN
	process_name: STRING

	set_process_name (a_name: STRING) is
		do
			process_name := a_name
		end

	reset_process_id_lookup_table is
		do
			table_structure.reset_process_id_lookup_table
			-- process_id_map_saved := Void
		end

	process_id_map: HASH_TABLE [ROSE_DATE, INTEGER] is
			-- process_id_map: ROSE_LINEAR_ARRAY [ROSE_DATE] is
		do
			Result := table_structure.process_id_map
		end

	refresh_process_id_map is
		do
			table_structure.refresh_process_id_map
		end

	min_date_time: ROSE_DATE is
		once
			create Result.make_min_date
		end

	synchronize_process_dates is
		do
			from
				cube_signature_result.start
			until
				cube_signature_result.exhausted
			loop
				table_structure.synchronize_process_dates (cube_signature_result.current_item)
				cube_signature_result.forth
			end
		end

feature {SIGNATURE_TAGGED_ITEM_DATABASE} -- Insert

	insert_records_nonlogged (a_array: ROSE_LINEAR_ARRAY[ROSE_VIRTUAL_RECORD[G]]) is
			-- This is a batch operation
			-- All signatures and data sent here will be inserted and commited
			-- Be sure that it is new or the insert will fail
		local
			l_id: INTEGER
		do
			if a_array /= Void and not a_array.is_empty then
				signature_id_predicate.retrieve (void,void)
				if not signature_id_predicate.success
				then
					raise ("Retrieve failed.")
				end

				l_id := signature_id_predicate.array.item (1)
				if l_id < 0 then l_id := 0 end
				from a_array.start until a_array.off loop
					l_id := l_id + 1
					a_array.current_item.signature.set_signature_id (l_id)
					insert_signature (a_array.current_item)
					a_array.forth
				end
				signature_fast_insert.commit
				if not signature_fast_insert.success then
					raise ("Failure in signature_fast_insert")
				end
				commit_items_fast_load
				signatures_created := signatures_created + a_array.count
			end
		end

	insert_signatures_nonlogged (a_array: ROSE_LINEAR_ARRAY[G]) is
			-- This is a batch operation
			-- All signatures sent here will be inserted and commited
			-- Be sure that it is new or the insert will fail
		local
			l_id: INTEGER
		do
			if a_array /= Void and not a_array.is_empty then
				signature_id_predicate.retrieve (void,void)
				if not signature_id_predicate.success then
					raise ("Could not retrieve signature.")
				end

				l_id := signature_id_predicate.array.item (1)
				if l_id < 0 then l_id := 0 end
				from a_array.start until a_array.off loop
					l_id := l_id + 1
					a_array.current_item.set_signature_id (l_id)
					a_array.current_item.set_path (path (a_array.current_item))
					signature_fast_insert.insert (a_array.current_item)
					a_array.forth
				end
				signature_fast_insert.commit
				signatures_created := signatures_created + a_array.count
				--if not signature_fast_insert.success then
				--	raise ("Failure in signature_fast_insert")
				--end
			end
		end

	insert_signature (a_signature: ROSE_VIRTUAL_RECORD[G]) is
		local
			l_real_record: DATA_RECORD[REAL]
			l_double_record: DATA_RECORD[DOUBLE]
			l_integer_record: DATA_RECORD[INTEGER]
			l_string_record: DATA_RECORD[STRING]
			l_date_record: DATA_RECORD[ROSE_DATE]
			l_signature: G
		do
			l_signature := a_signature.signature
			if l_signature.first_process_id /= 0 then
				set_process_id (l_signature.first_process_id)
			elseif l_signature.continuing_process_id /= 0 then
				set_process_id (l_signature.continuing_process_id)
			end
			l_signature.set_first_process_id (process_id)
			l_signature.set_last_process_id (process_id)
			l_signature.set_signature_data_process_id (process_id)
			l_signature.set_end_current_date (max_date.twin)
			set_path (l_signature,l_signature)
			signature_fast_insert.insert (l_signature)
			if not signature_fast_insert.success then
				raise ("Failure in fast inserting a signature")
			end
			if a_signature.has_real_items then
				from a_signature.real_items.start until a_signature.real_items.off loop
					create l_real_record
					l_real_record.set_signature_id (a_signature.signature.signature_id)
					l_real_record.set_value (a_signature.real_items.item_for_iteration)
					l_real_record.set_item_id (a_signature.real_items.key_for_iteration)
					l_real_record.set_process_id (process_id)
					set_path (l_real_record,a_signature.signature)
					real_data.insert (l_real_record.twin)
					a_signature.real_items.forth
				end
			end
			if a_signature.has_double_items then
				from a_signature.double_items.start until a_signature.double_items.off loop
					create l_double_record
					l_double_record.set_signature_id (a_signature.signature.signature_id)
					l_double_record.set_value (a_signature.double_items.item_for_iteration)
					l_double_record.set_item_id (a_signature.double_items.key_for_iteration)
					l_double_record.set_process_id (process_id)
					set_path (l_double_record,a_signature.signature)
					double_data.insert (l_double_record.twin)
					a_signature.double_items.forth
				end
			end
			if a_signature.has_integer_items then
				from a_signature.integer_items.start until a_signature.integer_items.off loop
					create l_integer_record
					l_integer_record.set_signature_id (a_signature.signature.signature_id)
					l_integer_record.set_value (a_signature.integer_items.item_for_iteration)
					l_integer_record.set_item_id (a_signature.integer_items.key_for_iteration)
					l_integer_record.set_process_id (process_id)
					set_path (l_integer_record,a_signature.signature)
					integer_data.insert (l_integer_record.twin)
					a_signature.integer_items.forth
				end
			end
			if a_signature.has_string_items then
				from a_signature.string_items.start until a_signature.string_items.off loop
					create l_string_record
					l_string_record.set_signature_id (a_signature.signature.signature_id)
					l_string_record.set_value (a_signature.string_items.item_for_iteration)
					l_string_record.set_item_id (a_signature.string_items.key_for_iteration)
					l_string_record.set_process_id (process_id)
					set_path (l_string_record,a_signature.signature)
					string_data.insert (l_string_record.twin)
					a_signature.string_items.forth
				end
			end
			if a_signature.has_image_items then
				from a_signature.image_items.start until a_signature.image_items.off loop
					create l_string_record
					l_string_record.set_signature_id (a_signature.signature.signature_id)
					l_string_record.set_value (a_signature.image_items.item_for_iteration)
					l_string_record.set_item_id (a_signature.image_items.key_for_iteration)
					l_string_record.set_process_id (process_id)
					set_path (l_string_record,a_signature.signature)
					image_data.insert (l_string_record.twin)
					a_signature.image_items.forth
				end
			end
			if a_signature.has_date_items then
				from a_signature.date_items.start until a_signature.date_items.off loop
					create l_date_record
					l_date_record.set_signature_id (a_signature.signature.signature_id)
					l_date_record.set_value (a_signature.date_items.item_for_iteration)
					l_date_record.set_item_id (a_signature.date_items.key_for_iteration)
					l_date_record.set_process_id (process_id)
					set_path (l_date_record,a_signature.signature)
					date_data.insert (l_date_record.twin)
					a_signature.date_items.forth
				end
			end
		end

feature {NONE} -- Selection Predicates

	signature_selection: ANONYMOUS_PREDICATE is
		local
			l_statement: STRING
			l_signature: G
			l_inline_parameters: BOOLEAN
			l_restriction_clause: STRING
			l_predicate_template: ROSE_LINEAR_ARRAY[STRING]
			l_group_by_done: BOOLEAN
		do
			create l_signature.default_create
			l_inline_parameters := selected_columns.count /= selected_column_ids.count
			l_statement := "select "
			if record_limit = 0 then
				selected_columns.compare_objects
				from selected_column_ids.start until selected_column_ids.off loop
					if selected_columns.has (field_name (selected_column_ids.current_item,l_signature)) then
						l_statement := l_statement + "#" + field_name (selected_column_ids.current_item,l_signature) + ".id,"
					else
						-- Use actual column name when only 1 entry is being used in query
						l_statement := l_statement + field_name (selected_column_ids.current_item,l_signature) + " as " + field_name (selected_column_ids.current_item,l_signature) + "_,"
					end
					selected_column_ids.forth
				end
			else
				-- Use selected items from subquery
				from selected_columns.start until selected_columns.off loop
					l_statement := l_statement + "closeout.out_" + selected_columns.current_item + ","
					selected_columns.forth
				end
			end
			if l_signature.signatures_fields_clause.count > 0 then
				l_statement := l_statement + l_signature.signature_and_data_fields_clause + ","
			end
			if table_structure.signature_data_process_id_available then
				l_statement := l_statement + data_name_prefix + "data_signatures.signature_data_process_id,"
			end
			l_statement := l_statement +
				data_name_prefix + "data_signatures.signature_id," +
				data_name_prefix + "data_signatures.last_process_id," +
				data_name_prefix + "data_signatures.first_process_id, " +
				data_name_prefix + "data_signatures.continuing_process_id " +
				"from " + data_name_prefix + "data_signatures"
			if record_limit = 0 then
				from selected_column_ids.start until selected_column_ids.off loop
					if selected_columns.has (field_name (selected_column_ids.current_item,l_signature)) then
						l_statement := l_statement + ",#" + field_name (selected_column_ids.current_item,l_signature)
					end
					selected_column_ids.forth
				end
				--if selected_columns.count > 0 then
					l_statement := l_statement + " where "
					if restriction_clause /= Void then
						l_restriction_clause := restriction_clause.twin
						from selected_column_ids.start until selected_column_ids.off loop
							if not selected_columns.has (field_name (selected_column_ids.current_item,l_signature)) then
								-- Use actual column name when only 1 entry is being used in query
								l_restriction_clause.replace_substring_all ("#" + field_name (selected_column_ids.current_item,l_signature) + ".id", "?")
								l_restriction_clause.replace_substring_all ("? = " + field_name (selected_column_ids.current_item,l_signature),
																		   field_name (selected_column_ids.current_item,l_signature) + " = ?")
								if l_predicate_template=Void
								then
									create l_predicate_template.make_empty
								end

								l_predicate_template.extend (field_type_for_template (selected_column_ids.current_item, l_signature))
							end
							selected_column_ids.forth
						end
						l_statement := l_statement + l_restriction_clause
					else
						from selected_column_ids.start until selected_column_ids.off loop
							if not selected_columns.has (field_name (selected_column_ids.current_item,l_signature)) then
								-- Use actual column name when only 1 entry is being used in query
								l_statement := l_statement + " " + field_name (selected_column_ids.current_item,l_signature) + " = ? and "
								if l_predicate_template=Void
								then
									create l_predicate_template.make_empty
								end

								l_predicate_template.extend (field_type_for_template (selected_column_ids.current_item, l_signature))
							else
								l_statement := l_statement + " #" + field_name (selected_column_ids.current_item,l_signature) + ".id = " +
									data_name_prefix + "data_signatures." + field_name (selected_column_ids.current_item,l_signature) +" and "
							end
							selected_column_ids.forth
						end
						l_statement.keep_head (l_statement.count - 4)
					end
				--end
			else
				-- Create subquery to get closest signature in the desired direction
				l_statement := l_statement + ", (select "
				from selected_column_ids.start; column_comparison.start until selected_column_ids.off loop
					if not column_comparison.current_item.is_equal ("=") then
						l_statement := l_statement +  " max (" +
							field_name (selected_column_ids.current_item,l_signature) + ") as match_" +
							field_name (selected_column_ids.current_item,l_signature) + ","
					else
						l_statement := l_statement +
							field_name (selected_column_ids.current_item,l_signature) + " as out_" +
							field_name (selected_column_ids.current_item,l_signature) + ","
					end
					column_comparison.forth
					selected_column_ids.forth
				end

				l_statement.prune_all_trailing (',')
				l_statement := l_statement + " from " + data_name_prefix + "data_signatures"
				from selected_column_ids.start until selected_column_ids.off loop
					if selected_columns.has (field_name (selected_column_ids.current_item,l_signature)) then
						l_statement := l_statement + ",#" + field_name (selected_column_ids.current_item,l_signature)
					end
					selected_column_ids.forth
				end

				l_statement := l_statement + " where "
				if restriction_clause /= Void then
					l_restriction_clause := restriction_clause.twin
					from selected_column_ids.start until selected_column_ids.off loop
						if not selected_columns.has (field_name (selected_column_ids.current_item,l_signature)) then
							-- Use actual column name when only 1 entry is being used in query
							l_restriction_clause.replace_substring_all ("#" + field_name (selected_column_ids.current_item,l_signature) + ".id", "?")
							l_restriction_clause.replace_substring_all ("? = " + field_name (selected_column_ids.current_item,l_signature),
																	   field_name (selected_column_ids.current_item,l_signature) + " = ?")
							if l_predicate_template=Void
							then
								create l_predicate_template.make_empty
							end

							l_predicate_template.extend (field_type_for_template (selected_column_ids.current_item, l_signature))
						end
						selected_column_ids.forth
					end
					l_statement := l_statement + l_restriction_clause
				end

				from selected_column_ids.start; column_comparison.start until selected_column_ids.off loop
					if column_comparison.current_item.is_equal ("=") then
						if not l_group_by_done
						then
							l_statement := l_statement + " group by "
							l_group_by_done := True
						end

						l_statement := l_statement + field_name (selected_column_ids.current_item,l_signature) + ","
					end
					column_comparison.forth
					selected_column_ids.forth
				end

				l_statement.prune_all_trailing (',')
				l_statement := l_statement + ") closeout where	   "
				from selected_column_ids.start; column_comparison.start until selected_column_ids.off loop
					if column_comparison.current_item.is_equal ("=") then
						l_statement := l_statement + " closeout.out_" + field_name (selected_column_ids.current_item,l_signature) + " = "
					else
						l_statement := l_statement + " closeout.match_" + field_name (selected_column_ids.current_item,l_signature) + " = "
					end
					l_statement := l_statement + data_name_prefix + "data_signatures." + field_name (selected_column_ids.current_item,l_signature) +" and "
					selected_column_ids.forth
					column_comparison.forth
				end
				l_statement.keep_head (l_statement.count - 4)
			end

			l_statement.append (" order by signature_id")
			Result := signature_selection_predicate (l_statement, l_predicate_template)
		end

	signature_selection_predicate (a_statement: STRING; a_predicate_template: ARRAY[STRING]): ANONYMOUS_PREDICATE is
		do
			if signatures_predicate_statement = Void or not a_statement.is_equal (signatures_predicate_statement) then
				signatures_predicate_statement := a_statement.twin
				if a_predicate_template = Void then
					create signatures_predicate.make_using_server (signature_item_server,a_statement)
				else
					create signatures_predicate.make_using_server_with_parameter_template (signature_item_server,a_statement, a_predicate_template)
				end
				if not signatures_predicate.success then
					print (a_statement)
					raise ("Failed to create cube_selection predicate")
				end
				--print (a_statement)
				create_signatures_sql_item_map (signatures_predicate)
			end
			Result := signatures_predicate
		end

feature {NONE} -- Implementation

	real_override_data: SIGNATURE_TAGGED_ITEM[REAL] is
		do
			if saved_real_override_data = Void then
				create saved_real_override_data.make (signature_item_server,process_id,"REAL","override_" + data_name)
				data_items.extend (saved_real_override_data)
			end
			Result := saved_real_override_data
		end
	saved_real_override_data: SIGNATURE_TAGGED_ITEM[REAL]

	real_data: SIGNATURE_TAGGED_ITEM[REAL] is
		do
			if saved_real_data = Void then
				create saved_real_data.make (signature_item_server,process_id,"REAL",data_name)
				data_items.extend (saved_real_data)
			end
			Result := saved_real_data
		end
	saved_real_data: SIGNATURE_TAGGED_ITEM[REAL]

	double_data: SIGNATURE_TAGGED_ITEM[DOUBLE] is
		do
			if saved_double_data = Void then
				create saved_double_data.make (signature_item_server,process_id,"DOUBLE",data_name)
				data_items.extend (saved_double_data)
			end
			Result := saved_double_data
		end
	saved_double_data: SIGNATURE_TAGGED_ITEM[DOUBLE]

	integer_data: SIGNATURE_TAGGED_ITEM[INTEGER] is
		do
			if saved_integer_data = Void then
				create saved_integer_data.make (signature_item_server,process_id,"INTEGER",data_name)
				data_items.extend (saved_integer_data)
			end
			Result := saved_integer_data
		end
	saved_integer_data: SIGNATURE_TAGGED_ITEM[INTEGER]

	string_data: SIGNATURE_TAGGED_ITEM[STRING] is
		do
			if saved_string_data = Void then
				create saved_string_data.make (signature_item_server,process_id,"STRING",data_name)
				data_items.extend (saved_string_data)
			end
			Result := saved_string_data
		end
	saved_string_data: SIGNATURE_TAGGED_ITEM[STRING]

	image_data: SIGNATURE_TAGGED_IMAGE_ITEM is
		do
			if saved_image_data = Void then
				create saved_image_data.make (signature_item_server,process_id,data_name)
				data_items.extend (saved_image_data)
			end
			Result := saved_image_data
		end
	saved_image_data: SIGNATURE_TAGGED_IMAGE_ITEM

	date_data: SIGNATURE_TAGGED_ITEM[ROSE_DATE] is
		do
			if saved_date_data = Void then
				create saved_date_data.make (signature_item_server,process_id,"DATETIME",data_name)
				data_items.extend (saved_date_data)
			end
			Result := saved_date_data
		end
	saved_date_data: SIGNATURE_TAGGED_ITEM[ROSE_DATE]

	data_fields: ROSE_LINEAR_ARRAY[STRING]

	field_name (i: INTEGER; a_vessel: ANY): STRING is
		do
			Result := internal.field_name (i,a_vessel)
			if Result.item (1) = '$' then
				Result := Result.substring (3,Result.count) --.net names
			end
		end

	field_type_for_template (i: INTEGER; a_vessel: ANY): STRING is
			-- Use this when making a predicate with templates.
		local
			l_type: INTEGER
		do
			l_type := internal.field_static_type_of_type (i, internal.dynamic_type (a_vessel))
			if l_type = rose_date_time_dynamic_type or else l_type = rose_date_dynamic_type then
				Result := "ROSE_DATE"
			elseif l_type = string_dynamic_type then
				Result := "STRING"
			else
				Result := internal.class_name_of_type (l_type)
			end
			if equal (Result, "INTEGER_32") then
				-- FIXME: this is a temporary solution to the fact that OLEDB classes use integers for type representation
				Result := "INTEGER"
			end
		end

	rose_date_dynamic_type: INTEGER is
		local
			l_object: ROSE_DATE
		once
			create l_object.make_unavailable
			Result := internal.dynamic_type (l_object)
		end

	rose_date_time_dynamic_type: INTEGER is
		local
			l_object: ROSE_DATE
		once
			create l_object.make_max_date
			Result := internal.dynamic_type (l_object)
		end

	string_dynamic_type: INTEGER is
		local
			l_object: STRING
		once
			create l_object.make_empty
			Result := internal.dynamic_type (l_object)
		end

	set_signature_field (a_signature: G; a_field_id: INTEGER; a_value: ANY) is
			-- Set `a_field_id' within `a_signature' to `a_value', respecting the dynamic type.
		do
			inspect internal.field_type (a_field_id, a_signature)
			when {INTERNAL}.INTEGER_32_TYPE then
				if attached {INTEGER_32_REF} a_value as l_ir then
					internal.set_integer_field (a_field_id, a_signature, l_ir.item)
				end
			when {INTERNAL}.INTEGER_16_TYPE then
				if attached {INTEGER_16_REF} a_value as l_i16r then
					internal.set_integer_16_field (a_field_id, a_signature, l_i16r.item)
				end
			when {INTERNAL}.INTEGER_64_TYPE then
				if attached {INTEGER_64_REF} a_value as l_i64r then
					internal.set_integer_64_field (a_field_id, a_signature, l_i64r.item)
				end
			when {INTERNAL}.INTEGER_8_TYPE then
				if attached {INTEGER_8_REF} a_value as l_i8r then
					internal.set_integer_8_field (a_field_id, a_signature, l_i8r.item)
				end
			when {INTERNAL}.CHARACTER_TYPE then
				if attached {STRING} a_value as l_string and then l_string.count = 1 then
					internal.set_character_field (a_field_id, a_signature, l_string.item (1))
				elseif attached {CHARACTER_REF} a_value as l_charr then
						internal.set_character_field (a_field_id, a_signature, l_charr.item)
				end
			when {INTERNAL}.REAL_TYPE then
				if attached {REAL_REF} a_value as l_rr then
					internal.set_real_field (a_field_id, a_signature, l_rr.item)
				end
			when {INTERNAL}.DOUBLE_TYPE then
				if attached {DOUBLE_REF} a_value as l_dr then
					internal.set_double_field (a_field_id, a_signature, l_dr.item)
				end
			when {INTERNAL}.BOOLEAN_TYPE then
				if attached {BOOLEAN_REF} a_value as l_booleanr then
					internal.set_boolean_field (a_field_id, a_signature, l_booleanr.item)
				end
			when {INTERNAL}.REFERENCE_TYPE then
				if attached {STRING} a_value as l_string then
					internal.set_reference_field (a_field_id, a_signature, l_string)
				elseif attached {ROSE_DATE} a_value as l_datetime then
					internal.set_reference_field (a_field_id, a_signature,l_datetime)
				end
			end
		end

	create_default_comparisons is
		do
			create column_comparison.make (1,selected_columns.count)
			from column_comparison.start until column_comparison.off loop
				column_comparison.put ("=",column_comparison.index)
				column_comparison.forth
			end
		end

	ordered_cube_of_items_base (a_items: ROSE_LINEAR_ARRAY[INTEGER]): ROSE_LINEAR_ARRAY[ROSE_VIRTUAL_RECORD[G]] is
			-- Called by 'ordered_cube_of_items' and 'ordered_cube_of_items_from_predicate'.
		do
			synchronize_process_dates
			if not virtual_record_set.cube_result.is_empty then
				set_current_cache_pool (cube_signature_result.first)
				cube_result_table := virtual_record_set.cube_result_table
				actual_last_rose_signature := virtual_record_set.cube_result.last
				Result := virtual_record_set.cube_result
			elseif not cube_signature_result.is_empty then
				set_current_cache_pool (cube_signature_result.first)
				virtual_record_set.load_virtual_records (historical_mode, cube_signature_result, a_items, current_cache_pool, True)
				cube_result_table := virtual_record_set.cube_result_table
				actual_last_rose_signature := virtual_record_set.cube_result.last
				Result := virtual_record_set.cube_result
--				set_cache_from_results (last_rose_signature)
			else
				create Result.make_empty
			end
		end

	handle_signatures (a_row: ARRAY[ANY]) is
			-- A row is {cube_position_item_1,...,cube_position_item_n,
			--			 signature_id,last_process_id,first_process_id,continuing_process_id,date}
			-- Using index i for speed
		local
			i, l_max_i, l_field_id: INTEGER
			l_signature, l_cube_signature: G
			l_signature_id, l_last_process_id, l_first_process_id, l_continuing_process_id: INTEGER_REF
		do
			l_max_i := (a_row.count - 4).min (signatures_sql_item_map.upper)
			l_signature_id ?= a_row.item (l_max_i + 1)
			-- Check for duplicate signature
			if cube_signature_result.count = 0 or else cube_signature_result.last.signature_id /= l_signature_id.item then
				-- Build cube signature result
				create l_signature.default_create
				from i := selected_columns.count + 1 until i > l_max_i loop
					l_field_id := signatures_sql_item_map.item (i)
					if l_field_id > 0 then
						set_signature_field (l_signature, l_field_id, a_row.item (i))
					end
					i := i + 1
				end
				l_signature.set_signature_id (l_signature_id.item)
				l_last_process_id ?= a_row.item (a_row.count - 2)
				l_signature.set_last_process_id (l_last_process_id.item)
				l_first_process_id ?= a_row.item (a_row.count - 1)
				l_signature.set_first_process_id (l_first_process_id.item)
				l_continuing_process_id ?= a_row.item (a_row.count)
				l_signature.set_continuing_process_id (l_continuing_process_id.item)
				set_path (Void, l_signature)
				cube_signature_result.extend (l_signature)

				-- Build cube position result
				if should_build_cube_position_result then
					create l_cube_signature.default_create
					l_max_i := selected_column_ids.count
					from i := 1 until i > l_max_i loop
						l_field_id := selected_column_ids.item (i)
						set_signature_field (l_cube_signature, l_field_id, a_row.item (i))
						i := i + 1
					end
					cube_position_result.force (l_cube_signature, l_signature_id.item)
				end
			end
		end

	selected_column (a_signature: G; a_create_temp_table: BOOLEAN; a_parse: BOOLEAN): STRING is
			-- Name of selected column in `a_signature';
			-- If `a_create_temp_table' then create a temporary table for later query.
		local
			i: INTEGER
		do
			from
				i := 1
			until
				i > internal.field_count (a_signature) or Result /= Void
			loop
				inspect internal.field_type (i,a_signature)
				when {INTERNAL}.CHARACTER_TYPE then
					if attached {CHARACTER_REF} internal.field (i, a_signature) as l_char and then
						l_char.item /= '%/0/' then
						Result := internal.field_name (i, a_signature)
						if a_parse then
							add_column_axis ("VARCHAR (1)", i, a_create_temp_table)
						end
					end
				when {INTERNAL}.INTEGER_8_TYPE then
					if attached {INTEGER_8_REF} internal.field (i, a_signature) as l_int8 and then
						l_int8.item /= 0 then
						Result := internal.field_name (i, a_signature)
						if a_parse then
							add_column_axis ("TINYINT", i, a_create_temp_table)
						end
					end
				when {INTERNAL}.INTEGER_16_TYPE then
					if attached {INTEGER_16_REF} internal.field (i, a_signature) as l_int16 and then
						l_int16.item /= 0 then
						Result := internal.field_name (i, a_signature)
						if a_parse then
							add_column_axis ("SMALLINT", i, a_create_temp_table)
						end
					end
				when {INTERNAL}.INTEGER_64_TYPE then
					if attached {INTEGER_64_REF} internal.field (i, a_signature) as l_int64 and then
						l_int64.item /= 0 then
						Result := internal.field_name (i, a_signature)
						if a_parse then
							add_column_axis ("BIGINT", i, a_create_temp_table)
						end
					end
				when {INTERNAL}.INTEGER_32_TYPE then
					if attached {INTEGER_32_REF} internal.field (i, a_signature) as l_int and then
						l_int.item /= 0 then
						Result := internal.field_name (i, a_signature)
						if a_parse then
							add_column_axis ("INT", i, a_create_temp_table)
						end
					end
				when {INTERNAL}.REAL_TYPE then
					if attached {REAL_REF} internal.field (i, a_signature) as l_real and then
						l_real.item /= 0 then
						Result := internal.field_name (i, a_signature)
						if a_parse then
							add_column_axis ("REAL", i, a_create_temp_table)
						end
					end
				when {INTERNAL}.DOUBLE_TYPE then
					if attached {DOUBLE_REF} internal.field (i, a_signature) as l_double and then
						l_double.item /= 0 then
						Result := internal.field_name (i, a_signature)
						if a_parse then
							add_column_axis ("DOUBLE", i, a_create_temp_table)
						end
					end
				when {INTERNAL}.REFERENCE_TYPE then
					if attached {STRING} internal.field (i, a_signature) then
						Result := internal.field_name (i, a_signature)
						if a_parse then
							add_column_axis ("VARCHAR (7950)", i, a_create_temp_table)
						end
					elseif attached {ROSE_DATE} internal.field (i, a_signature) then
						Result := internal.field_name (i,a_signature)
						if a_parse then
							add_column_axis ("DATETIME", i, a_create_temp_table)
						end
					end
				else
					-- Whatever the type of this column, we don't support it yet.
					-- That doesn't really matter as it is probably zero/Void anyway.
					-- But we need to prevent the "Unmatched inspect value" error
					-- and move past this column so we can get to the real column.
				end
				i := i + 1
			end
		ensure
			some_column_selected: Result /= Void
		end

	add_column_axis (a_type: STRING; a_column: INTEGER; a_create_temp_table: BOOLEAN) is
		require
			valid_column: a_column > 0
		local
			l_column_name : STRING
			l_signature: G
		do
			create l_signature.default_create
			if not selected_column_ids.has (a_column) or
				 (a_create_temp_table and not selected_columns.has (field_name (a_column,l_signature))) then
				selected_column_ids.extend (a_column)
				if a_create_temp_table then --or restriction_uses_column (a_column) then
					l_column_name := field_name (a_column,l_signature)
					selected_columns.extend (l_column_name)
					if table_exists (signature_item_server, "#" + l_column_name) then
						signature_item_server.sql_execute_immediate ("drop table #" + l_column_name)
						if not signature_item_server.success then
							raise ("Could not drop preexisting temp table.")
						end
					end

					signature_item_server.sql_execute_immediate ("create table #" + l_column_name + " (id " + a_type + ")")
					if not signature_item_server.success then
						raise ("Could not create temp table.")
					end

					signature_item_server.sql_execute_immediate ("create unique clustered index " + l_column_name + "_inx on #" + l_column_name + " (id) WITH IGNORE_DUP_KEY")
					if not signature_item_server.success then
						raise ("Could not create temp table index.")
					end

--					-- Some test code in case you don't believe it.
--					if not table_exists(signature_item_server, "#" + l_column_name)
--					then
--						raise("table_exists failed%N")
--					end
				end
			end
		end

	drop_column_temp_tables is
			-- Drop all temporary SQL tables created by `add_column_axis'.
		do
			from selected_columns.start until selected_columns.off loop
				signature_item_server.sql_execute_immediate ("drop table #" + selected_columns.current_item)
				if not signature_item_server.success then
					raise ("Could not drop a temp table.")
				end

				selected_columns.forth
			end
			selected_columns.wipe_out
			selected_column_ids.wipe_out
		end

	commit_items_fast_load is
			-- Commit fast load items
		do
			data_items.do_all (agent commit_item_fast_load)
		end

	commit_item_fast_load (a_item: like submittable_anchor) is
			-- Commit item to DB with fast load
		require
			valid_item: a_item /= Void
		do
			a_item.commit_fast_load
		end

	submittable_anchor: STID_SUBMITTABLE is do  end
			-- Anchor for submittable

	truncate_item_temp_tables (a_item: like submittable_anchor) is
		require
			valid_item: a_item /= Void
		do
			a_item.truncate_temp_tables
		end

	set_item_submit_frequency (a_item: like submittable_anchor; a_freq: INTEGER) is
		require
			valid_item: a_item /= Void
		do
			a_item.set_submit_frequency (a_freq)
		end

	submit_item (a_item: like submittable_anchor) is
			-- Submit item `a_item'.
		require
			valid_item: a_item /= Void
		do
			if a_item.unsubmitted then
				uncommitted := True
				a_item.submit
			end
		end

	cleanup_items_all (a_item: like submittable_anchor) is
		require
			valid_item: a_item /= Void
		do
			a_item.cleanup
		end

	discard_updates (a_item: like submittable_anchor) is
			-- Discard the staged update data
		require
			valid_item: a_item /= Void
		do
			a_item.discard_updates
		end

	empty_array: ROSE_LINEAR_ARRAY[ARRAY[ANY]]
	-- Alway empty array.  We use this to pretend that we retrieved 0 rows.
	once
		create Result.make_empty
	end

feature {SIGNATURE_TAGGED_ITEM_DATABASE} -- Cache Dumps

	dump_all_caches is
		do
			dump_replace_caches
		end

	dump_replace_caches is
		do
			from data_items.start until data_items.off loop
				data_items.current_item.dump_update_cache
				data_items.forth
			end
		end

	dump_replace_cache (a_type: STRING) is
		local
			l_item: STID_SUBMITTABLE
		do
			a_type.to_upper
			if table_structure.data_type_available (a_type, False) then
				from data_items.start until data_items.off loop
					l_item := data_items.current_item
					if l_item.type_name.is_equal (a_type) then
						l_item.dump_update_cache
					end
					data_items.forth
				end
			else
				print ("Data Type " + a_type + " not available%N")
			end
		end

feature {NONE} -- checking to see availability_of columns

	signature_data_process_id_column_available:BOOLEAN is
		local
			l_predicate: ANONYMOUS_PREDICATE
		once
			create l_predicate.make_using_server (signature_item_server,
												 "exec sp_columns @table_name = '" + data_name_prefix +"data_signatures',@column_name = 'signature_data_process_id'")
			l_predicate.retrieve (Void,Void)
			if not l_predicate.success then
				raise ("l_predicate failed.")
			end

			if l_predicate.array.count > 0 then
				Result := True
			end
		end

feature {NONE} -- Predicates

	signature_insert: INTEGER_PREDICATE
		require
			server_not_void: signature_item_server /= Void
		local
			l_statement: STRING
		do
			if signature_insert_saved = Void then
				l_statement := new_signature_insert_statement(False, False)
				create signature_insert_saved.make(signature_item_server, l_statement)
				if not signature_insert_saved.success then
					raise ("Failed to create signature_insert_saved")
				end
			end
			Result := signature_insert_saved
		end
	signature_insert_saved: INTEGER_PREDICATE

	signature_insert_with_signature_data_process_id: INTEGER_PREDICATE
		require
			server_not_void: signature_item_server /= Void
		local
			l_statement: STRING
		do
			if signature_insert_with_signature_data_process_id_saved = Void then
				l_statement := new_signature_insert_statement(True, False)
				create signature_insert_with_signature_data_process_id_saved.make(signature_item_server, l_statement)
				if not signature_insert_with_signature_data_process_id_saved.success then
					raise ("Failed to create signature_insert_with_signature_data_process_id_saved")
				end
			end
			Result := signature_insert_with_signature_data_process_id_saved
		end
	signature_insert_with_signature_data_process_id_saved: INTEGER_PREDICATE

	new_signature_insert_statement(a_has_data_process, a_has_path: BOOLEAN): STRING
			-- Common code for building a statement that inserts a new signature and returns the signature_id.
		require
			server_not_void: signature_item_server /= Void
		local
			l_signature: G
			l_fields: STRING
			l_values: STRING
		do
			create l_signature.default_create

			l_fields := "first_process_id, last_process_id, continuing_process_id, "
			l_values := "?, ?, ?, "

			if a_has_data_process
			then
				l_fields.append("signature_data_process_id, ")
				l_values.append("?, ")
			end

			if a_has_path
			then
				l_fields.append("path, ")
				l_values.append("?, ")
			end

			l_fields.append(l_signature.signature_and_data_fields_clause)
			l_values.append(l_signature.signature_and_data_values_clause)

			Result :=
				"declare @my_table table (signature_id int);%N%
				%insert into " + table_name + "%N%
				%	(" + l_fields + ") %N%
				%	output inserted.signature_id into @my_table%N%
				%	values (" + l_values + ");%N%
				%select * from @my_table"
		end

	signature_data_update_predicate: EXECUTION_PREDICATE is
		local
			l_signature: G
			l_statement: STRING
			l_field_name: STRING
			l_not_first_field: BOOLEAN
		do
			if signature_data_update_predicate_saved = Void then
				l_statement := "update " + table_name + " set "
				create l_signature.default_create
				from
					l_signature.data_fields.start
				until
					l_signature.data_fields.off
				loop
					l_field_name := l_signature.data_fields.current_item
					if l_not_first_field then
						l_statement.append (", ")
					end

					l_not_first_field := True
					l_statement.append ("[" + l_field_name + "]" + " = :" + l_field_name)
					l_signature.data_fields.forth
				end

				l_statement.append (", last_process_id = :last_process_id")
				l_statement.append (", signature_data_process_id = :signature_data_process_id")
				l_statement.append (" where signature_id = :signature_id")

				create signature_data_update_predicate_saved.make (signature_item_server, l_statement)
				if not signature_data_update_predicate_saved.success then
					raise ("Failed to create signature_data_update_predicate.")
				end
			end

			Result := signature_data_update_predicate_saved
		end
	signature_data_update_predicate_saved: EXECUTION_PREDICATE

	check_if_path_clustered is
		local
			l_predicate: ANONYMOUS_PREDICATE
		do
			if signature_item_server /= Void then
				create l_predicate.make_using_server (signature_item_server,
													  "exec sp_columns @table_name = '" + table_name + "',@column_name = 'path'")
				l_predicate.retrieve (Void,Void)
				if l_predicate.array.count > 0 then
					is_path_clustered := True
				end
			end
		end

	signature_data_history_insert_predicate: EXECUTION_PREDICATE is
			-- signature data history insert predicate
		local
			l_signature: G
			l_statement: STRING
			l_field_name: STRING
		do
			if signature_data_history_insert_predicate_saved = Void then
				check_if_path_clustered
				create l_signature.default_create
				l_statement := "insert " + data_name_prefix + "data_signatures_data_history (signature_id,process_id"
				if is_path_clustered then
					l_statement.append (",path")
				end
				from l_signature.data_fields.start	until l_signature.data_fields.off loop
					l_field_name := l_signature.data_fields.current_item
					l_statement.append (", ")
					l_statement.append ("[" + l_field_name + "]")
					l_signature.data_fields.forth
				end
				l_statement.append (") values (:signature_id,:signature_data_process_id")
				if is_path_clustered then
					l_statement.append (",:path")
				end
				from l_signature.data_fields.start	until l_signature.data_fields.off loop
					l_field_name := l_signature.data_fields.current_item
					l_statement.append (", ")
					l_statement.append (":" + l_field_name)
					l_signature.data_fields.forth
				end
				l_statement.append (")")

				create signature_data_history_insert_predicate_saved.make (signature_item_server, l_statement)
				if not signature_data_history_insert_predicate_saved.success then
					raise ("Failed to create signature_data_history_insert_predicate.")
				end
			end
			Result := signature_data_history_insert_predicate_saved
		end
	signature_data_history_insert_predicate_saved: EXECUTION_PREDICATE

	signature_data_history_insert2_predicate: EXECUTION_PREDICATE is
			-- For instance:
			--	insert into data_signatures_data_history
			--	(
			--		signature_id
			--		, process_id
			--		, path
			--		, (other data fields...)
			--	)
			--	select
			--		a.signature_id
			--		, a.process_id
			--		, a.path
			--		, (other data fields...)
			--	from
			--		data_signatures a,
			--		##9signature_loader b
			--	where
			--		a.signature_id=b.signature_id and
			--		(
			--			a.data_field_1<>b.data_field_1 or
			--			(a.data_field_1 is null and b.data_field_1 is not null) or
			--			(b.data_field_1 is null and a.data_field_1 is not null) or
			--			...
			--		)
		local
			l_signature: G
			l_statement: STRING
			l_field_name: STRING
		do
			if signature_data_history_insert2_predicate_saved = Void
			then
				check_if_path_clustered
				create l_signature.default_create
				l_statement := "insert into " + data_name_prefix + "data_signatures_data_history%N"
				l_statement.append("(%N")
				l_statement.append("%Tsignature_id%N")
				l_statement.append("%T, process_id%N")
				if is_path_clustered
				then
					l_statement.append("%T, path%N")
				end

				from
					l_signature.data_fields.start
				until
					l_signature.data_fields.off
				loop
					l_field_name := l_signature.data_fields.current_item
					l_statement.append("%T, [" + l_field_name + "]%N")
					l_signature.data_fields.forth
				end

				l_statement.append(")%N")
				l_statement.append("select%N")
				l_statement.append("%Ta.signature_id%N")
				l_statement.append("%T, a.signature_data_process_id%N")
				if is_path_clustered
				then
					l_statement.append("%T, a.path%N")
				end

				from
					l_signature.data_fields.start
				until
					l_signature.data_fields.off
				loop
					l_field_name := l_signature.data_fields.current_item
					l_statement.append("%T, a.[" + l_field_name + "]%N")
					l_signature.data_fields.forth
				end

				l_statement.append(data_history_from_where_clause(l_signature))

				create signature_data_history_insert2_predicate_saved.make(signature_item_server, l_statement)
				if not signature_data_history_insert2_predicate_saved.success
				then
					raise ("Failed to create signature_data_history_insert2_predicate.")
				end
			end

			Result := signature_data_history_insert2_predicate_saved
		end
	signature_data_history_insert2_predicate_saved: EXECUTION_PREDICATE

	signature_update2_predicate: EXECUTION_PREDICATE is
			-- Update the existing records that have changed.
			-- For instance:
			--	update data_signatures
			--	set
			--		(data fields...),
			--		[signature_data_process_id]=b.[process_id],
			--		[last_process_id]=b.[process_id],
			--		[continuing_process_id]=b.[process_id]
			--	from
			--		data_signatures a,
			--		##9signature_loader b
			--	where
			--		a.signature_id=b.signature_id and
			--		(
			--			a.data_field_1<>b.data_field_1 or
			--			(a.data_field_1 is null and b.data_field_1 is not null) or
			--			(b.data_field_1 is null and a.data_field_1 is not null) or
			--			...
			--		)
		local
			l_statement: STRING
			l_field_name: STRING
			l_signature: G
		do
			if signature_update2_predicate_saved=Void
			then
				create l_signature
				l_statement := "update " + data_name_prefix + "data_signatures%N"
				l_statement.append("set%N")
				from
					l_signature.data_fields.start
				until
					l_signature.data_fields.off
				loop
					l_field_name := l_signature.data_fields.current_item
					l_statement.append("%T[" + l_field_name + "]=b.[" + l_field_name + "],%N")
					l_signature.data_fields.forth
				end

				if table_structure.signature_data_process_id_available then
					l_statement.append("%T[signature_data_process_id]=b.[process_id],%N")
				end
				l_statement.append("%T[last_process_id]=b.[process_id],%N")
				l_statement.append("%T[continuing_process_id]=b.[process_id]%N")
				l_statement.append(data_history_from_where_clause(l_signature))

				create signature_update2_predicate_saved.make(signature_item_server, l_statement)
				if not signature_update2_predicate_saved.success
				then
					raise ("Failed to create signature_update2_predicate.")
				end
			end

			Result := signature_update2_predicate_saved
		end
	signature_update2_predicate_saved: EXECUTION_PREDICATE

	signature_update3_predicate: EXECUTION_PREDICATE is
			-- Update the continuing_process_id on the existing records that have NOT changed.
			-- For instance:
			--	update data_signatures
			--	set
			--		[continuing_process_id]=b.[process_id]
			--	from
			--		data_signatures a,
			--		##9signature_loader b
			--	where
			--		a.signature_id=b.signature_id and
			--		(
			--			a.data_field_1=b.data_field_1 or (a.data_field_1 is null and b.data_field_1 is null) and
			--			...
			--		)
		local
			l_statement: STRING
			l_signature: G
		do
			if signature_update3_predicate_saved=Void
			then
				create l_signature
				l_statement := "update " + data_name_prefix + "data_signatures%N"
				l_statement.append("set%N")
				l_statement.append("%T[continuing_process_id]=b.[process_id]%N")
				l_statement.append(data_history_from_where_equal_clause(l_signature))

				create signature_update3_predicate_saved.make(signature_item_server, l_statement)
				if not signature_update3_predicate_saved.success
				then
					raise ("Failed to create signature_update3_predicate.")
				end
			end

			Result := signature_update3_predicate_saved
		end
	signature_update3_predicate_saved: EXECUTION_PREDICATE

	signature_insert_from_table_predicate: EXECUTION_PREDICATE is
			-- For instance:
			--	insert into data_signatures
			--	(
			--		first_process_id,
			--		last_process_id,
			--		continuing_process_id,
			--		signature_data_process_id,
			--		 (data_fields...)
			--	)
			--	select
			--		[process_id],
			--		[process_id],
			--		[process_id],
			--		[process_id],
			--		 (data_fields...)
			--	from ##9signature_loader
			--	where signature_id=0
		local
			l_statement: STRING
			l_signature: G
		do
			if signature_insert_from_table_predicate_saved=Void
			then
				create l_signature
				l_statement := "insert into " + data_name_prefix + "data_signatures%N"
				l_statement.append("(%N")
				l_statement.append("%Tfirst_process_id,%N")
				l_statement.append("%Tlast_process_id,%N")
				l_statement.append("%Tcontinuing_process_id,%N")
				if table_structure.signature_data_process_id_available then
					l_statement.append("%Tsignature_data_process_id,%N")
				end
				l_statement.append("%T" + l_signature.signature_and_data_fields_clause + "%N")
				l_statement.append(")%N")
				l_statement.append("select%N")
				l_statement.append("%T[process_id],%N")
				l_statement.append("%T[process_id],%N")
				l_statement.append("%T[process_id],%N")
				if table_structure.signature_data_process_id_available then
					l_statement.append("%T[process_id],%N")
				end
				l_statement.append("%T" + l_signature.signature_and_data_fields_clause + "%N")
				l_statement.append("from " + signature_loader_table_name + "%N")
				l_statement.append("where signature_id=0")

				create signature_insert_from_table_predicate_saved.make(signature_item_server, l_statement)
				if not signature_insert_from_table_predicate_saved.success
				then
					raise ("Failed to create signature_insert_from_table_predicate.")
				end
			end
			Result := signature_insert_from_table_predicate_saved
		end
	signature_insert_from_table_predicate_saved: EXECUTION_PREDICATE

	data_history_from_where_clause(a_signature: G): STRING is
			-- Just the 'from' and 'where' part of a larger statement.  For instance:
			--	from
			--		data_signatures a,
			--		##9signature_loader b
			--	where
			--		a.signature_id=b.signature_id and
			--		(
			--			a.data_field<>b.data_field or
			--			(a.data_field is null and b.data_field is not null) or
			--			(b.data_field is null and a.data_field is not null) or
			--			...
			--		)
		local
			l_field_name: STRING
		do
			if data_history_from_where_clause_saved=Void
			then
				data_history_from_where_clause_saved :="from%N"
				data_history_from_where_clause_saved.append("%T" + data_name_prefix + "data_signatures a,%N")
				data_history_from_where_clause_saved.append("%T" + signature_loader_table_name + " b%N")
				data_history_from_where_clause_saved.append("where%N")
				data_history_from_where_clause_saved.append("%Ta.signature_id=b.signature_id ")
				if not a_signature.data_fields.is_empty then
					data_history_from_where_clause_saved.append(" and%N%T(%N")
					from
						a_signature.data_fields.start
					until
						a_signature.data_fields.off
					loop
						l_field_name := a_signature.data_fields.current_item
						-- (a<>b or (a is null and b is not null) or (b is null and a is not null)

						data_history_from_where_clause_saved.append("%T%Ta.[" + l_field_name + "]<>b.[" + l_field_name + "] or%N")
						data_history_from_where_clause_saved.append("%T%T(a.[" + l_field_name + "] is null and b.[" + l_field_name + "] is not null) or%N")
						data_history_from_where_clause_saved.append("%T%T(b.[" + l_field_name + "] is null and a.[" + l_field_name + "] is not null)")
						if not a_signature.data_fields.at_last
						 then
							data_history_from_where_clause_saved.append(" or")
						end

						data_history_from_where_clause_saved.append("%N")
						a_signature.data_fields.forth
					end

					data_history_from_where_clause_saved.append("%T)%N")
				end
			end

			Result := data_history_from_where_clause_saved
		end
	data_history_from_where_clause_saved: STRING

	data_history_from_where_equal_clause(a_signature: G): STRING is
			-- Like 'data_history_from_where_clause' but this one searches for records where ALL fields are equal
			-- instead of ANY field that is NOT equal.
			-- Just the 'from' and 'where' part of a larger statement.  For instance:
			--	from
			--		data_signatures a,
			--		##9signature_loader b
			--	where
			--		a.signature_id=b.signature_id and
			--		(
			--			a.data_field_1=b.data_field_1 or (a.data_field_1 is null and b.data_field_1 is null) and
			--			...
			--		)
		local
			l_field_name: STRING
		do
			if data_history_from_where_equal_clause_saved=Void
			then
				data_history_from_where_equal_clause_saved :="from%N"
				data_history_from_where_equal_clause_saved.append("%T" + data_name_prefix + "data_signatures a,%N")
				data_history_from_where_equal_clause_saved.append("%T" + signature_loader_table_name + " b%N")
				data_history_from_where_equal_clause_saved.append("where%N")
				data_history_from_where_equal_clause_saved.append("%Ta.signature_id=b.signature_id %N")
				if not a_signature.data_fields.is_empty then
					data_history_from_where_equal_clause_saved.append(" and %T(%N")

					from
						a_signature.data_fields.start
					until
						a_signature.data_fields.off
					loop
						l_field_name := a_signature.data_fields.current_item
						-- (a<>b or (a is null and b is not null) or (b is null and a is not null)

						data_history_from_where_equal_clause_saved.append("%T%Ta.[" + l_field_name + "]=b.[" + l_field_name + "] or%N")
						data_history_from_where_equal_clause_saved.append("%T%T(a.[" + l_field_name + "] is null and b.[" + l_field_name + "] is null)")
						if not a_signature.data_fields.at_last
						 then
							data_history_from_where_equal_clause_saved.append(" and")
						end

						data_history_from_where_equal_clause_saved.append("%N")
						a_signature.data_fields.forth
					end

					data_history_from_where_equal_clause_saved.append("%T)%N")
				end
			end

			Result := data_history_from_where_equal_clause_saved
		end
	data_history_from_where_equal_clause_saved: STRING

	data_history_update_predicate: EXECUTION_PREDICATE is
		do
			if data_history_update_predicate_saved = Void then
				create data_history_update_predicate_saved.make (signature_item_server,"update " + data_name_prefix + "data_signatures_data_history " +
																 " set process_id = d.last_process_id from " + data_name_prefix + "data_signatures d," + data_name_prefix + "data_signatures_data_history h" +
																 " where d.signature_id = h.signature_id and h.process_id = 0")
			end
			Result := data_history_update_predicate_saved
		end
	data_history_update_predicate_saved: EXECUTION_PREDICATE

	data_columns: ROSE_LINEAR_ARRAY[STRING] is
		local
			l_predicate: STRING_PREDICATE
		do
			create l_predicate.make (signature_item_server,
									 "select [name] from syscolumns where id = object_id ('" + data_name_prefix + "data_signatures_data_history') order by colid")
			l_predicate.retrieve (Void,Void)
			if not l_predicate.success then
				raise ("Failed the retrieve from syscolumns.")
			end
			Result := l_predicate.array
		end

	has_column (a_table, a_name: STRING): BOOLEAN is
		-- Does 'a_table' have a column named 'a_name'?
		require
			valid_table: a_table /= Void
			valid_name: a_name /= Void
		local
			l_predicate: STRING_PREDICATE
		do
			create l_predicate.make (signature_item_server, "select [name] from syscolumns where id = object_id ('" + a_table + "') and name = '" + a_name + "'")
			l_predicate.retrieve (Void,Void)
			if not l_predicate.success then
				raise ("Failed the retrieve from syscolumns.")
			end
			Result := not l_predicate.array.is_empty
		end

	signature_update_predicate: EXECUTION_PREDICATE is
		local
			l_signature: G
			l_statement: STRING
			l_field_name: STRING
			l_not_first_field: BOOLEAN
		do
			if signature_update_predicate_saved = Void then
				l_statement := "update " + table_name + " set "
				create l_signature.default_create
				from
					l_signature.signature_fields.start
				until
					l_signature.signature_fields.off
				loop
					l_field_name := l_signature.signature_fields.current_item
					if l_not_first_field then
						l_statement.append (", ")
					end
					l_not_first_field := True
					l_statement.append (l_field_name + " = :" + l_field_name)
					l_signature.signature_fields.forth
				end
				from
					l_signature.data_fields.start
				until
					l_signature.data_fields.off
				loop
					l_field_name := l_signature.data_fields.current_item
					if l_not_first_field then
						l_statement.append (", ")
					end
					l_not_first_field := True
					l_statement.append (l_field_name + " = :" + l_field_name)
					l_signature.data_fields.forth
				end
				l_statement.append (" where signature_id = :signature_id")
				create signature_update_predicate_saved.make (signature_item_server, l_statement)
				if not signature_update_predicate_saved.success then
					raise ("Failed to create signature_update_predicate.")
				end
			end
			Result := signature_update_predicate_saved
		end
	signature_update_predicate_saved: EXECUTION_PREDICATE

	signature_touch: EXECUTION_PREDICATE is
		require
			server_not_void: signature_item_server /= Void
		do
			if signature_touch_saved = Void then
				create signature_touch_saved.make (signature_item_server,"update " + table_name + " set continuing_process_id = ? where signature_id = :signature_id")
				if not signature_touch_saved.success then
					raise ("Failed to create signature_touch_saved")
				end
			end
			Result := signature_touch_saved
		end
	signature_touch_saved: EXECUTION_PREDICATE

	signature_search: ANONYMOUS_PREDICATE is
		require
			server_not_void: signature_item_server /= Void
		local
			l_signature: G
		do
			if signature_search_saved = Void then
				create l_signature.default_create
				create signature_search_saved.make_using_server (signature_item_server,"select signature_id from " + table_name + " (nolock) where " + l_signature.signatures_where_clause)
				if not signature_search_saved.success then
					raise ("Failed to make signature_search.")
				end
			end
			Result := signature_search_saved
		end
	signature_search_saved: ANONYMOUS_PREDICATE

	signature_data_fields_predicate:COLLECTING_PREDICATE[G] is
		require
			server_not_void: signature_item_server /= void
		local
			l_signature: G
		do
			if signature_data_fields_predicate_saved = void then
				create l_signature.default_create
				create signature_data_fields_predicate_saved.make (signature_item_server, "select * from " + table_name + " (nolock) where " + l_signature.signatures_where_clause)
				if not signature_data_fields_predicate_saved.success then
					raise ("Failed to make signature_data_process_id_search.")
				end
			end
			Result := signature_data_fields_predicate_saved
		end

	signature_data_fields_predicate_saved:COLLECTING_PREDICATE[G]

	signature_id_predicate: INTEGER_PREDICATE is
		require
			server_not_void: signature_item_server /= Void
		do
			if signature_id_predicate_saved = Void then
				if not signature_item_server.success then
					signature_item_server.clear_error
				end
				--create signature_id_predicate_saved.make (signature_item_server,"select max (signature_id) as signature_id from " + table_name)
				create signature_id_predicate_saved.make (signature_item_server,"select cast (ident_current ('" + table_name + "') as int) as signature_id")
				if not signature_id_predicate_saved.success then
					raise ("failed to create signature_id_predicate")
				end
			end
			Result := signature_id_predicate_saved
		end
	signature_id_predicate_saved: INTEGER_PREDICATE

	signature_by_id_predicate: COLLECTING_PREDICATE[G] is
		require
			server_not_void: signature_item_server /= Void
		do
			if signature_by_id_predicate_saved = Void then
				create signature_by_id_predicate_saved.make (signature_item_server,"select * from " + table_name + " where signature_id = ?")
				if not signature_by_id_predicate_saved.success then
					raise ("failed to create signature_id_predicate")
				end
			end
			Result := signature_by_id_predicate_saved
		end
	signature_by_id_predicate_saved: COLLECTING_PREDICATE[G]

	signature_fast_insert: DATA_SERVER_FAST_LOAD is
		local
			l_signature: G
		do
			if signature_fast_insert_saved = Void then
				create l_signature.default_create
				signature_fast_insert_saved := signature_item_fast_load_server.fast_loader_keep_identity (table_name,l_signature)
			end
			Result := signature_fast_insert_saved
		end
	signature_fast_insert_saved: DATA_SERVER_FAST_LOAD
			-- Cache to the fast load server for inserting signature

feature {NONE} -- Helpers

	internal: INTERNAL is
			-- Utility reference to internal
		once
			create Result
		end

	max_date: ROSE_DATE is
			-- Maximum value for a date
		do
			create Result.make_ccyymmdd (99991231)
		end

	time_now: ROSE_DATE is
		do
			create Result.make_now
			Result.time.minute_back
		end

end
