note

	description:

		"Signature tagged table structure"

	keywords: "database,signature,item,tagged,manager"
	author: "Nelson Chui"
	last_editor: "$Author: rj $"
	date: "$Date: 2014/09/11 $"
	revision: "$Revision: #23 $"
	path: "$File: //rose/source/kernel/persistence/databases/signature_tagged/signature_tagged_table_structure.e $"

class SIGNATURE_TAGGED_TABLE_STRUCTURE

inherit

	SIGNATURE_TAGGED_DATA_TYPES

	ROSE_SHARED_GLOBAL

create

	make

feature {NONE} -- Creation

	make (a_server: SQL_DATA_SERVER; a_data_name: STRING; a_record_templates: ROSE_RECORD_ITEM_TABLE) is
			-- Create the table structure
		require
			server_not_void: a_server /= Void
		do
			data_name := a_data_name
			if a_data_name = Void or else a_data_name.count = 0 then
				data_name_prefix := ""
			else
				data_name_prefix := a_data_name + "_"
			end
			create available_data_types_cached.make (8)
			create enabled_data_types_cached.make (8)
			signature_item_server := a_server
			data_available := get_signature_availability
			set_signature_data_process_id_available
			create enabled_regular_base_types.make_empty
			create enabled_override_base_types.make_empty
			if a_record_templates /= Void then
				create record_templates.make
				record_templates.hash_table_make (a_record_templates.count)
				from
					a_record_templates.start
				until
					a_record_templates.off
				loop
					record_templates.force (a_record_templates.item_for_iteration, a_record_templates.key_for_iteration.as_upper)
					a_record_templates.forth
				end
				create enabled_regular_record_types.make_empty
				create enabled_override_record_types.make_empty
			end
			enable_data_types (False)
			enable_version_pinning
		end

feature -- Access

	signature_item_server: SQL_DATA_SERVER

	data_name, data_name_prefix: STRING

	record_templates: ROSE_RECORD_ITEM_TABLE

	available_data_types: ROSE_LINEAR_ARRAY [STRING] is
			-- Names of data types currently available in the server
		do
			create Result.make_empty
			from
				available_data_types_cached.start
			until
				available_data_types_cached.after
			loop
				if available_data_types_cached.item_for_iteration then
					Result.extend (available_data_types_cached.key_for_iteration)
				end
				available_data_types_cached.forth
			end
		ensure
			available_data_types_not_void: Result /= Void
			no_void_type: not Result.has (Void)
		end

feature -- Overrides

	set_override_family_name (a_value: STRING) is
		do
			override_family_name := a_value
			if a_value /= Void and then not a_value.is_empty then
				enable_data_types (True)
			else
				disable_data_types (True)
			end
		end

	override_family_name: STRING

	override_family_name_prefix: STRING is
		do
			if override_family_name /= Void and then not override_family_name.is_empty then
				Result := override_family_name + "_"
			else
				Result := ""
			end
		end

	override_family_names: ROSE_LINEAR_ARRAY [STRING] is
		local
			l_string_predicate: STRING_PREDICATE
			l_prefix: STRING
		do
			create l_string_predicate.make_strict (signature_item_server, "select * from sysobjects where name like '%%_override_data%%_value'")
			l_string_predicate.retrieve (Void, Void)
			create Result.make_empty
			from
				l_string_predicate.array.start
			until
				l_string_predicate.array.off
			loop
				if l_string_predicate.array.current_item.substring_index ("_override_data", 1) > 0 then
					l_prefix := l_string_predicate.array.current_item.substring (1, l_string_predicate.array.current_item.substring_index ("_override_data", 1) - 1)
				end
				Result.extend (l_prefix)
				l_string_predicate.array.forth
			end
		end

feature -- Status report

	data_available: BOOLEAN

	signature_data_process_id_available: BOOLEAN
			-- True if the signature_data_process_id column exists in the signature

	enabled_regular_signature_data, enabled_override_signature_data: BOOLEAN

	enabled_regular_base_types: ROSE_LINEAR_ARRAY [PAIR [STRING, INTEGER]]

	enabled_override_base_types: ROSE_LINEAR_ARRAY [PAIR [STRING, INTEGER]]

	enabled_regular_record_types: ROSE_LINEAR_ARRAY [ROSE_SIGNATURE_RECORD_ITEM]

	enabled_override_record_types: ROSE_LINEAR_ARRAY [ROSE_SIGNATURE_RECORD_ITEM]

	version_pinning_enabled: BOOLEAN

	signature_only: BOOLEAN is
		do
			Result := (not version_pinning_enabled and (enabled_regular_base_types = Void or enabled_regular_base_types.count = 0) and
				(enabled_regular_record_types = Void or enabled_regular_record_types.count = 0) and
				(enabled_override_base_types = Void or enabled_override_base_types.count = 0) and
				(enabled_override_record_types = Void or enabled_override_record_types.count = 0))
		end

	showing_suppressed_historical_data: BOOLEAN is
		do
			Result := showing_suppressed_historical_data_saved
		end

	server_spid: INTEGER is
		local
			l_predicate: INTEGER_PREDICATE
		do
			if server_spid_saved = 0 then
				create l_predicate.make (signature_item_server, "select @@spid as spid")
				if not l_predicate.success then
					(create {EXCEPTIONS}).raise ("Failed to create spid selection predicate")
				end
				l_predicate.retrieve (Void, Void)
				if not l_predicate.success or else l_predicate.array = Void or else l_predicate.array.count /= 1 then
					(create {EXCEPTIONS}).raise ("Failed to execute spid selection predicate")
				end
				server_spid_saved := l_predicate.array.first
			end
			Result := server_spid_saved
		end

feature -- Data Type Access Control

	show_suppressed_historical_data is
		do
			showing_suppressed_historical_data_saved := True
		end

	hide_suppressed_historical_data is
		do
			showing_suppressed_historical_data_saved := False
		end

	enable_version_pinning is
		do
			if data_type_available (signature_version_type.first, True) then
				version_pinning_enabled := True
			end
		end

	disable_version_pinning is
		do
			if data_type_available (signature_version_type.first, True) then
				version_pinning_enabled := False
			end
		end

	enable_all_data_types is
		local
			i: INTEGER
			l_cursor: CURSOR
		do
			enable_data_type (signature_data_type.first, False)
				-- regular
			enable_data_type (signature_data_type.first, True)
				-- override
			from
				i := base_data_types.lower
			until
				i > base_data_types.upper
			loop
				enable_data_type (base_data_types.item (i).first, False)
					-- regular
				enable_data_type (base_data_types.item (i).first, True)
					-- override
				i := i + 1
			end
			if record_templates /= Void then
				l_cursor := record_templates.cursor
				from
					record_templates.start
				until
					record_templates.off
				loop
					enable_data_type (record_templates.item_for_iteration.name, False)
						-- regular
					enable_data_type (record_templates.item_for_iteration.name, True)
						-- override
					record_templates.forth
				end
				record_templates.go_to (l_cursor)
			end
		end

	disable_all_data_types is
		local
			i: INTEGER
			l_cursor: CURSOR
		do
			disable_data_type (signature_data_type.first, False)
				-- regular
			disable_data_type (signature_data_type.first, True)
				-- override
			from
				i := base_data_types.lower
			until
				i > base_data_types.upper
			loop
				disable_data_type (base_data_types.item (i).first, False)
					-- regular
				disable_data_type (base_data_types.item (i).first, True)
					-- override
				i := i + 1
			end
			if record_templates /= Void then
				l_cursor := record_templates.cursor
				from
					record_templates.start
				until
					record_templates.off
				loop
					disable_data_type (record_templates.item_for_iteration.name, False)
						-- regular
					disable_data_type (record_templates.item_for_iteration.name, True)
						-- override
					record_templates.forth
				end
				record_templates.go_to (l_cursor)
			end
		end

	enable_data_types (a_is_override: BOOLEAN) is
		local
			i: INTEGER
			l_cursor: CURSOR
		do
			enable_data_type (signature_data_type.first, a_is_override)
			from
				i := base_data_types.lower
			until
				i > base_data_types.upper
			loop
				enable_data_type (base_data_types.item (i).first, a_is_override)
				i := i + 1
			end
			if record_templates /= Void then
				l_cursor := record_templates.cursor
				from
					record_templates.start
				until
					record_templates.off
				loop
					enable_data_type (record_templates.item_for_iteration.name, a_is_override)
					record_templates.forth
				end
				record_templates.go_to (l_cursor)
			end
		end

	disable_data_types (a_is_override: BOOLEAN) is
		local
			i: INTEGER
			l_cursor: CURSOR
		do
			disable_data_type (signature_data_type.first, a_is_override)
			from
				i := base_data_types.lower
			until
				i > base_data_types.upper
			loop
				disable_data_type (base_data_types.item (i).first, a_is_override)
				i := i + 1
			end
			if record_templates /= Void then
				l_cursor := record_templates.cursor
				from
					record_templates.start
				until
					record_templates.off
				loop
					disable_data_type (record_templates.item_for_iteration.name, a_is_override)
					record_templates.forth
				end
				record_templates.go_to (l_cursor)
			end
		end

	disable_data_type (a_type: STRING; a_is_override: BOOLEAN) is
		do
			if data_type_available (a_type, a_is_override) and then data_type_enabled (a_type, a_is_override) then
				base_data_types_by_name.search (a_type.as_upper)
				if base_data_types_by_name.found then
					if not a_is_override then
						enabled_regular_base_types.prune_all (base_data_types_by_name.found_item)
					else
						enabled_override_base_types.prune_all (base_data_types_by_name.found_item)
					end
				elseif a_type.as_upper.is_equal (signature_data_type.first) then
					if not a_is_override then
						enabled_regular_signature_data := False
					else
						enabled_override_signature_data := False
					end
				elseif record_templates /= Void then
					record_templates.search (a_type.as_upper)
					if record_templates.found then
						if not a_is_override then
							enabled_regular_record_types.prune_all (record_templates.found_item)
						else
							enabled_override_record_types.prune_all (record_templates.found_item)
						end
					end
				end
				enabled_data_types_cached.force (False, data_type_key (a_type.as_upper, a_is_override))
			end
		end

	enable_data_type (a_type: STRING; a_is_override: BOOLEAN) is
		do
			if data_type_available (a_type, a_is_override) and then not data_type_enabled (a_type, a_is_override) then
				base_data_types_by_name.search (a_type.as_upper)
				if base_data_types_by_name.found then
					if not a_is_override then
						enabled_regular_base_types.extend (base_data_types_by_name.found_item)
					else
						enabled_override_base_types.extend (base_data_types_by_name.found_item)
					end
				elseif a_type.as_upper.is_equal (signature_data_type.first) then
					if not a_is_override then
						enabled_regular_signature_data := signature_data_process_id_available
					else
						enabled_override_signature_data := True
					end
				elseif record_templates /= Void then
					record_templates.search (a_type.as_upper)
					if record_templates.found then
						if not a_is_override then
							enabled_regular_record_types.extend (record_templates.found_item)
						else
							enabled_override_record_types.extend (record_templates.found_item)
						end
					end
				end
				enabled_data_types_cached.force (True, data_type_key (a_type, a_is_override))
			end
		end

feature -- Basic operations

	data_type_enabled (a_type: STRING; a_is_override: BOOLEAN): BOOLEAN is
		local
			l_type: STRING
		do
			l_type := data_type_key (a_type, a_is_override)
			enabled_data_types_cached.search (l_type)
			if enabled_data_types_cached.found then
				Result := enabled_data_types_cached.found_item
			elseif data_type_available (a_type, a_is_override) then
				enabled_data_types_cached.force (False, l_type)
			end
		end

	data_type_available (a_type: STRING; a_is_override: BOOLEAN): BOOLEAN is
		local
			l_type, l_data_name_prefix, l_table_type_name: STRING
		do
			l_type := data_type_key (a_type, a_is_override)
			available_data_types_cached.search (l_type)
			if available_data_types_cached.found then
				Result := available_data_types_cached.found_item
			elseif (a_type.as_upper.is_equal (signature_data_type.first) or else a_type.as_upper.is_equal (signature_version_type.first) or else base_data_types_by_name.has (a_type.as_upper) or else (record_templates /= Void and then record_templates.has (a_type.as_upper))) then
				if a_is_override then
					l_data_name_prefix := override_family_name_prefix + "override_" + data_name_prefix
				else
					l_data_name_prefix := data_name_prefix
				end
				if a_type.as_upper.is_equal (signature_data_type.first) then
					if a_is_override then
						l_table_type_name := "signatures_data"
					else
						l_table_type_name := "signatures_data_history"
					end
				elseif a_type.as_upper.is_equal (signature_version_type.first) then
					l_table_type_name := "signature_versions"
				else
					l_table_type_name := a_type + "_value"
				end
				Result := signature_item_server.has_table (l_data_name_prefix + "data_" + l_table_type_name)
				available_data_types_cached.force (Result, l_type)
			end
		end

feature {TS_TEST_CASE} -- For testing.

	force_signature_data_process_id_available(a_value: like signature_data_process_id_available)
	-- Used to fake the value of 'signature_data_process_id_available'.  For testing only.
	do
		signature_data_process_id_available := a_value
	end

feature {GENERAL_SUPPLIER_UPDATE_TEST} -- Data purging

	clear_all_data_based_on_signature_pattern (a_pattern: STRING) is
			-- Clear all data releated to signtures described by 'a_pattern'
		require
			valid_pattern: a_pattern /= Void and then not a_pattern.is_empty
		local
			l_test_predicate: INTEGER_PREDICATE
		do
			create l_test_predicate.make (signature_item_server, "select count (*) from " + data_name_prefix + "data_signatures where " + a_pattern)
			if not l_test_predicate.success then
				fail ("clear_all_data_based_on_signature_pattern", "$1 is not valid for $2data_signatures", <<a_pattern, data_name_prefix>>)
			end
			from
				available_data_types_cached.start
			until
				available_data_types_cached.off
			loop
				if available_data_types_cached.item_for_iteration then
					clear_data_for_type (a_pattern, available_data_types_cached.key_for_iteration)
				end
				available_data_types_cached.forth
			end
			clear_signature_data_history (a_pattern)
			clear_signatures (a_pattern)
		end

	clear_signatures (a_pattern: STRING) is
			-- Clear signatures described by `a_pattern'
		require
			valid_pattern: a_pattern /= Void and then not a_pattern.is_empty
		local
			l_predicate: EXECUTION_PREDICATE
		do
			create l_predicate.make (signature_item_server, "delete from " + data_name_prefix + "data_signature_versions where " + "signature_id in (select signature_id from " + data_name_prefix + "data_signatures where " + a_pattern + ")")
			if not l_predicate.success then
				fail ("clear_signatures", "could not create signature versions delete predicate", Void)
			end
			l_predicate.execute (Void, Void)
			if not l_predicate.success then
				fail ("clear_signatures", "could not execute signature versions delete predicate", Void)
			end
			create l_predicate.make (signature_item_server, "delete from " + data_name_prefix + "data_signatures where " + "signature_id in (select signature_id from " + data_name_prefix + "data_signatures where " + a_pattern + ")")
			if not l_predicate.success then
				fail ("clear_signatures", "could not create signatures delete predicate", Void)
			end
			l_predicate.execute (Void, Void)
			if not l_predicate.success then
				fail ("clear_signatures", "could not execute signatures delete predicate", Void)
			end
		end

	clear_signature_data_history (a_pattern: STRING) is
			-- Clear signature data history for signatures described by 'a_pattern' if available
		require
			valid_pattern: a_pattern /= Void and then not a_pattern.is_empty
		local
			l_predicate: EXECUTION_PREDICATE
		do
			if signature_data_process_id_available then
				create l_predicate.make (signature_item_server, "delete from " + data_name_prefix + "data_signatures_data_history where " + "signature_id in (select signature_id from " + data_name_prefix + "data_signatures where " + a_pattern + ")")
				if not l_predicate.success then
					fail ("clear_signature_data_history", "could not create signature data history delete predicate", Void)
				end
				l_predicate.execute (Void, Void)
				if not l_predicate.success then
					fail ("clear_signature_data_history", "could not execute signature data history delete predicate", Void)
				end
			end
		end

	clear_data_for_type (a_pattern, a_type: STRING) is
			-- Clear data type 'a_type' for signatures described by 'a_pattern'
		require
			valid_pattern: a_pattern /= Void and then not a_pattern.is_empty
			valid_pattern: a_type /= Void and then not a_type.is_empty
		local
			l_predicate: EXECUTION_PREDICATE
		do
			if a_type /~ signature_data_type.first then
				create l_predicate.make (signature_item_server, "delete from " + data_name_prefix + "data_" + a_type + "_value_history where " + "signature_id in (select signature_id from " + data_name_prefix + "data_signatures where " + a_pattern + ")")
				if not l_predicate.success then
					fail ("clear_data_for_type", "could not create data history delete predicate for $1", <<a_type>>)
				end
				l_predicate.execute (Void, Void)
				if not l_predicate.success then
					fail ("clear_data_for_type", "could not execute data history delete predicate for $1", <<a_type>>)
				end
				create l_predicate.make (signature_item_server, "delete from " + data_name_prefix + "data_" + a_type + "_value where " + "signature_id in (select signature_id from " + data_name_prefix + "data_signatures where " + a_pattern + ")")
				if not l_predicate.success then
					fail ("clear_data_for_type", "could not create data delete predicate for $1", <<a_type>>)
				end
				l_predicate.execute (Void, Void)
				if not l_predicate.success then
					fail ("clear_data_for_type", "could not execute data delete predicate for $1", <<a_type>>)
				end
			end
		end

feature {NONE} -- Local Variables

	server_spid_saved: INTEGER

	showing_suppressed_historical_data_saved: BOOLEAN

	available_data_types_cached: HASH_TABLE [BOOLEAN, STRING]

	enabled_data_types_cached: HASH_TABLE [BOOLEAN, STRING]

feature {NONE} -- Implementation

	data_type_key (a_type: STRING; a_is_override: BOOLEAN): STRING is
		do
			if a_is_override then
				Result := "OVERRIDE," + a_type.as_upper
			else
				Result := a_type.as_upper
			end
		end

	get_signature_availability: BOOLEAN is
		local
			l_predicate: ANONYMOUS_PREDICATE
		do
			create l_predicate.make_using_server (signature_item_server, "exec sp_tables '" + data_name_prefix + "data_signatures'")
			l_predicate.retrieve (Void, Void)
			if not l_predicate.success then
				(create {EXCEPTIONS}).raise ("sp_tables failed.")
			end
			if l_predicate.array.count > 0 then
				Result := True
			end
		end

	set_signature_data_process_id_available is
		local
			l_predicate: ANONYMOUS_PREDICATE
		do
			create l_predicate.make_using_server (signature_item_server, "exec sp_columns @table_name = '" + data_name_prefix + "data_signatures',@column_name = 'signature_data_process_id'")
			l_predicate.retrieve (Void, Void)
			if l_predicate.array.count > 0 then
				signature_data_process_id_available := True
			end
		end

feature -- Process ID Management

	reset_process_id_lookup_table is
		do
			process_id_map_saved := Void
		end

	process_id_map: HASH_TABLE [ROSE_DATE, INTEGER] is
			-- Process-ids indexing the dates when they were created.
		do
			if process_id_map_saved = Void then
				max_cached_process_id := 0
					-- Assuming process IDs are positive, so set the initial value to zero
				refresh_process_id_map
			end
			Result := process_id_map_saved
		end

	refresh_process_id_map is
		local
			l_selection: COLLECTING_PREDICATE [SUPPLIER_PROCESS]
		do
			create l_selection.make (signature_item_server, "select file_date, process_id from process (nolock) " + "where process_id > " + max_cached_process_id.out + " order by process_id")
			if not l_selection.success then
				(create {EXCEPTIONS}).raise ("Failed to create process_id_map selection predicate")
			end
			l_selection.retrieve (Void, Void)
			if not l_selection.success then
				(create {EXCEPTIONS}).raise ("Failed to retrieve process_id_map selection predicate")
			end
			if process_id_map_saved = Void then
				create process_id_map_saved.make (l_selection.array.count * 2)
					-- Make a bigger hash table to minimize clashing of items for efficiency
			end
			from
				l_selection.array.start
			until
				l_selection.array.after
			loop
				process_id_map_saved.put (l_selection.array.current_item.file_date, l_selection.array.current_item.process_id)
				l_selection.array.forth
			end
			if l_selection.array.count > 0 then
				max_cached_process_id := l_selection.array.last.process_id
			end
		end

	synchronize_process_dates (a_signature: ROSE_SIGNATURE_RECORD [ENTITY]) is
		local
			l_start_date: ROSE_DATE
		do
				-- Inspect the continuing_process_id to check whether we have any new updates since we last build our process_id_map
				-- We need this since we want to make sure we have refreshed the process IDs before building the item version
			process_id_map.search (a_signature.continuing_process_id)
			if not process_id_map.found then
					-- Process ID not seen before, refresh process IDs
				refresh_process_id_map
				process_id_map.search (a_signature.continuing_process_id)
				if not process_id_map.found then
						-- Process ID not found even after refresh, enter Void into the process_id_map
					process_id_map.extend (Void, a_signature.continuing_process_id)
				end
			end
			if signature_only then
					-- We are not yet retrieving signature data history so set as_of_date to last_process_id
				process_id_map.search (a_signature.last_process_id)
				if process_id_map.found then
						-- Process ID seen before
					l_start_date := process_id_map.found_item
					if l_start_date = Void then
							-- Previous unknown process ID
						l_start_date := min_date_time
						print_map_signature_date_error_message (a_signature)
					end
					a_signature.set_as_of_date (l_start_date)
				else
						-- Process ID not seen before, refresh process IDs
					refresh_process_id_map
					process_id_map.search (a_signature.last_process_id)
					if process_id_map.found then
						l_start_date := process_id_map.found_item
						if l_start_date = Void then
								-- Incorrect start date from process table
							l_start_date := min_date_time
							print_map_signature_date_error_message (a_signature)
						end
						a_signature.set_as_of_date (l_start_date)
					else
							-- New unknown process ID
						print_map_signature_date_error_message (a_signature)
						process_id_map.extend (Void, a_signature.last_process_id)
						a_signature.set_as_of_date (min_date_time)
					end
				end
			else
					-- Now build the as of date from the first process ID
				process_id_map.search (a_signature.first_process_id)
				if process_id_map.found then
						-- Process ID seen before
					l_start_date := process_id_map.found_item
					if l_start_date = Void then
							-- Previous unknown process ID
						l_start_date := min_date_time
						print_map_signature_date_error_message (a_signature)
					end
					a_signature.set_as_of_date (l_start_date)
				else
						-- Process ID not seen before, refresh process IDs
					refresh_process_id_map
					process_id_map.search (a_signature.first_process_id)
					if process_id_map.found then
						l_start_date := process_id_map.found_item
						if l_start_date = Void then
								-- Incorrect start date from process table
							l_start_date := min_date_time
							print_map_signature_date_error_message (a_signature)
						end
						a_signature.set_as_of_date (l_start_date)
					else
							-- New unknown process ID
						print_map_signature_date_error_message (a_signature)
						process_id_map.extend (Void, a_signature.first_process_id)
						a_signature.set_as_of_date (min_date_time)
					end
				end
			end
		end

feature {NONE} -- Process ID Management Implementation

	process_id_map_saved: HASH_TABLE [ROSE_DATE, INTEGER]
			-- process_id_map_saved: ROSE_LINEAR_ARRAY [ROSE_DATE]

	max_cached_process_id: INTEGER

	print_map_signature_date_error_message (a_signature: ROSE_SIGNATURE_RECORD [ENTITY]) is
		do
			print ("Error: Signature has no corresponding as of date: ")
			print ("Server - " + signature_item_server.node + "/" + signature_item_server.database + "; Data name - " + data_name_prefix + "; Signature ID - " + a_signature.signature_id.out + "; Process ID - " + a_signature.first_process_id.out + "%N")
			print ("Warning: Signature as of date set to minimum date%N")
				--some databases still have unknown process ID, let not raise exception for now-- raise ("Signature date cannot be mapped")
		end

	min_date_time: ROSE_DATE is
		once
			Result := (create {ROSE_CONSTANTS}).constants.minimum_date_time
		end

feature -- Process ID Consistency Check

	process_id_consistent: BOOLEAN is
		do
			Result := True
			Result := Result and data_signatures_process_ids_consistent
			Result := Result and data_signature_versions_process_ids_consistent
			from
				available_data_types_cached.start
			until
				available_data_types_cached.off
			loop
				if available_data_types_cached.item_for_iteration then
					Result := data_type_process_ids_consistent (available_data_types_cached.key_for_iteration)
					Result := data_type_history_process_ids_consistent (available_data_types_cached.key_for_iteration)
				end
				available_data_types_cached.forth
			end
		end

	data_signatures_process_ids_consistent: BOOLEAN is
		local
			l_predicate: INTEGER_PREDICATE
		do
			create l_predicate.make (signature_item_server, "select count(*) as counter from " + data_name_prefix + "data_signatures where " + "first_process_id not in (select process_id from process) or " + "last_process_id not in (select process_id from process) or " + "continuing_process_id not in (select process_id from process)")
			if not l_predicate.success then
				(create {EXCEPTIONS}).raise ("predicate creation failed")
			end
			l_predicate.retrieve (Void, Void)
			Result := count_from_predicate (l_predicate) = 0
		end

	data_signature_versions_process_ids_consistent: BOOLEAN is
		local
			l_predicate: INTEGER_PREDICATE
		do
			create l_predicate.make (signature_item_server, "select count(*) as counter from " + data_name_prefix + "data_signature_versions where process_id not in (select process_id from process)")
			if not l_predicate.success then
				(create {EXCEPTIONS}).raise ("predicate creation failed")
			end
			Result := count_from_predicate (l_predicate) = 0
		end

	data_type_process_ids_consistent (a_type_name: STRING): BOOLEAN is
		local
			l_predicate: INTEGER_PREDICATE
		do
			create l_predicate.make (signature_item_server, "select count(*) as counter from " + data_name_prefix + "data_" + a_type_name + "_value where process_id not in (select process_id from process)")
			if not l_predicate.success then
				(create {EXCEPTIONS}).raise ("predicate creation failed")
			end
			Result := count_from_predicate (l_predicate) = 0
		end

	data_type_history_process_ids_consistent (a_type_name: STRING): BOOLEAN is
		local
			l_predicate: INTEGER_PREDICATE
		do
			create l_predicate.make (signature_item_server, "select count(*) as counter from " + data_name_prefix + "data_" + a_type_name + "_value_history where process_id not in (select process_id from process)")
			if not l_predicate.success then
				(create {EXCEPTIONS}).raise ("predicate creation failed")
			end
			Result := count_from_predicate (l_predicate) = 0
		end

feature -- Counts

	data_signatures_count: INTEGER is
		local
			l_predicate: INTEGER_PREDICATE
		do
			create l_predicate.make (signature_item_server, "select count(*) as counter from " + data_name_prefix + "data_signatures")
			if not l_predicate.success then
				(create {EXCEPTIONS}).raise ("predicate creation failed")
			end
			Result := count_from_predicate (l_predicate)
		end

	data_signature_versions_count: INTEGER is
		local
			l_predicate: INTEGER_PREDICATE
		do
			create l_predicate.make (signature_item_server, "select count(*) as counter from " + data_name_prefix + "data_signature_versions")
			if not l_predicate.success then
				(create {EXCEPTIONS}).raise ("predicate creation failed")
			end
			Result := count_from_predicate (l_predicate)
		end

	data_type_count (a_type_name: STRING): INTEGER is
		local
			l_predicate: INTEGER_PREDICATE
		do
			create l_predicate.make (signature_item_server, "select count(*) as counter from " + data_name_prefix + "data_" + a_type_name + "_value")
			if not l_predicate.success then
				(create {EXCEPTIONS}).raise ("predicate creation failed")
			end
			Result := count_from_predicate (l_predicate)
		end

	data_type_history_count (a_type_name: STRING): INTEGER_64 is
		local
			l_predicate: INTEGER_64_PREDICATE
		do
			create l_predicate.make (signature_item_server, "select count_big(*) as counter from " + data_name_prefix + "data_" + a_type_name + "_value_history")
			if not l_predicate.success then
				(create {EXCEPTIONS}).raise ("predicate creation failed")
			end
			Result := count_64_from_predicate (l_predicate)
		end

	data_type_orphan_count (a_type_name: STRING): INTEGER is
		local
			l_predicate: INTEGER_PREDICATE
		do
			create l_predicate.make (signature_item_server, "select count(*) as counter from " + data_name_prefix + "data_" + a_type_name + "_value where signature_id not in (select signature_id from " + data_name_prefix + "data_signatures)")
			if not l_predicate.success then
				(create {EXCEPTIONS}).raise ("predicate creation failed")
			end
			Result := count_from_predicate (l_predicate)
		end

	data_type_history_orphan_count (a_type_name: STRING): INTEGER is
		local
			l_predicate: INTEGER_PREDICATE
		do
			create l_predicate.make (signature_item_server, "select count(*) as counter from " + data_name_prefix + "data_" + a_type_name + "_value_history where signature_id not in (select signature_id from " + data_name_prefix + "data_signatures)")
			if not l_predicate.success then
				(create {EXCEPTIONS}).raise ("predicate creation failed")
			end
			Result := count_from_predicate (l_predicate)
		end

feature -- Performance

	data_signatures_speed: INTEGER is
		local
			l_predicate: INTEGER_PREDICATE
		do
			create l_predicate.make (signature_item_server, "select signature_id from " + data_name_prefix + "data_signatures where signature_id = 10000")
			if not l_predicate.success then
				(create {EXCEPTIONS}).raise ("predicate creation failed")
			end
			Result := performance_from_predicate (l_predicate)
		end

	data_signature_versions_speed: INTEGER is
		local
			l_predicate: INTEGER_PREDICATE
		do
			create l_predicate.make (signature_item_server, "select signature_id from " + data_name_prefix + "data_signature_versions where signature_id = 10000")
			if not l_predicate.success then
				(create {EXCEPTIONS}).raise ("predicate creation failed")
			end
			Result := performance_from_predicate (l_predicate)
		end

	data_type_speed (a_type_name: STRING): INTEGER is
		local
			l_predicate: INTEGER_PREDICATE
		do
			create l_predicate.make (signature_item_server, "select signature_id from " + data_name_prefix + "data_" + a_type_name + "_value where signature_id = 10000")
			if not l_predicate.success then
				(create {EXCEPTIONS}).raise ("predicate creation failed")
			end
			Result := performance_from_predicate (l_predicate)
		end

	data_type_history_speed (a_type_name: STRING): INTEGER is
		local
			l_predicate: INTEGER_PREDICATE
		do
			create l_predicate.make (signature_item_server, "select signature_id from " + data_name_prefix + "data_" + a_type_name + "_value_history where signature_id = 10000")
			if not l_predicate.success then
				(create {EXCEPTIONS}).raise ("predicate creation failed")
			end
			Result := performance_from_predicate (l_predicate)
		end

feature -- Index Fragmentation

	data_signatures_index_fragmentation: INTEGER is
		local
			l_predicate: DATA_SERVER_SIMPLE_COMMAND
		do
			l_predicate := signature_item_server.simple_command ("dbcc showcontig(" + data_name_prefix + "data_signatures) with all_indexes,tableresults")
			Result := scan_density_from_command (l_predicate)
		end

	data_type_index_fragmentation (a_type_name: STRING): INTEGER is
		local
			l_predicate: DATA_SERVER_SIMPLE_COMMAND
		do
			l_predicate := signature_item_server.simple_command ("dbcc showcontig(" + data_name_prefix + "data_" + a_type_name + "_value) with all_indexes,tableresults")
			Result := scan_density_from_command (l_predicate)
		end

	data_type_history_index_fragmentation (a_type_name: STRING): INTEGER is
		local
			l_predicate: DATA_SERVER_SIMPLE_COMMAND
		do
			l_predicate := signature_item_server.simple_command ("dbcc showcontig(" + data_name_prefix + "data_" + a_type_name + "_value_history) with all_indexes,tableresults")
			Result := scan_density_from_command (l_predicate)
		end

feature {NONE} -- Implementation

	count_from_predicate (a_predicate: INTEGER_PREDICATE): INTEGER is
		do
			a_predicate.retrieve (Void, Void)
			if not a_predicate.success then
				(create {EXCEPTIONS}).raise ("predicate retrieve failed")
			end
			if not a_predicate.array.is_empty and a_predicate.array.first >= 0 then
				Result := a_predicate.array.first
			end
		end

	count_64_from_predicate (a_predicate: INTEGER_64_PREDICATE): INTEGER_64 is
			-- Execute `a_predicate' and return count, violating CQS.
		require
			a_predicate_not_void: a_predicate /= Void
		do
			a_predicate.retrieve (Void, Void)
			if not a_predicate.success then
				(create {EXCEPTIONS}).raise ("predicate retrieve failed")
			end
			if not a_predicate.array.is_empty and a_predicate.array.first >= 0 then
				Result := a_predicate.array.first
			end
		end

	performance_from_predicate (a_predicate: INTEGER_PREDICATE): INTEGER is
		local
			l_process_timer: ROSE_PROCESS_TIMER
		do
			create l_process_timer.make
			a_predicate.retrieve (Void, Void)
			l_process_timer.interrupt
			if not a_predicate.success then
				(create {EXCEPTIONS}).raise ("predicate retrieve failed")
			end
			Result := l_process_timer.cpu.truncated_to_integer
		end

	scan_density_from_command (a_predicate: DATA_SERVER_SIMPLE_COMMAND): INTEGER is
		local
			l_double_ref: DOUBLE_REF
			l_integer: INTEGER
		do
			l_integer := a_predicate.execute (True)
			l_integer := a_predicate.move_first
			l_double_ref ?= a_predicate.get_double_value (16)
			Result := 100
			from
			until
				l_integer < 0
			loop
				l_double_ref ?= a_predicate.get_double_value (16)
				if l_double_ref /= Void then
					Result := Result.min (l_double_ref.item.truncated_to_integer)
				end
				l_integer := a_predicate.move_next
			end
		end

end
