indexing
	description: "Trigger for signature tagged database"
	author: "Mauricio Bustos"
	last_editor: "$Author: rj $"
	date: "$Date: 2009/11/05 $"
	revision: "$Revision: #12 $"
    path: "$File: //rose/source/kernel/persistence/databases/signature_tagged/signature_tagged_trigger.e $"

class
	SIGNATURE_TAGGED_TRIGGER

inherit
	ANY
	SIGNATURE_TAGGED_NAMES
		export
			{NONE} all
		end
	EXCEPTIONS
		rename
			class_name as exceptions_class_name
		export
			{NONE} all
		end
	INTERNAL
		rename
			field_name as internal_field_name
		export
			{NONE} all
		end
	DATABASE_SERVER
		export
			{NONE} all
		end

create
	make

feature {NONE} -- Initialization

	make (a_server: SQL_DATA_SERVER; a_data_name: STRING) is
			-- Create the object
		require
			valid_args: a_server /= Void
		do
			signature_item_server := a_server
			data_name := a_data_name
		end

feature {SIGNATURE_TAGGED_SERVER, PATH_TREE} -- Basic operations

	create_trigger is
			-- Create the trigger if it does not exists or is too old
		local
			l_statement: STRING
			l_version: INTEGER
			l_trigger_creation_date: ROSE_DATE
			l_server: SQL_DATA_SERVER
		do
			if not trigger_created
			then
				l_trigger_creation_date := signature_item_server.trigger_creation_date(table_name, data_name_prefix + "signature_version_update")
				if signature_item_server.has_column_name(data_name_prefix + "data_signatures", "timestamp")
				then
					if l_trigger_creation_date=Void or else l_trigger_creation_date.ccyymmdd < version_2_crdate
					then
						l_version := 2
					end
				else
					if l_trigger_creation_date=Void or else l_trigger_creation_date.ccyymmdd < version_1_crdate
					then
						l_version := 1
					end
				end

				if l_version/=0 and then signature_item_server.has_table(data_name_prefix + "data_signature_versions")
				then
					-- We don't specify a table_name since another table may have our trigger.  This can
					-- happen when someone renames their table to "old_table".  In SQL/Server that takes
					-- the trigger with it and we can't have duplicates.
					if signature_item_server.has_trigger(Void, data_name_prefix + "signature_version_update")
					then
						drop_trigger
					end

					if l_version=1
					then
						l_statement := trigger_version_1
					else
						l_statement := trigger_version_2
					end

					l_server := signature_item_server.auxiliary_server
					l_server.begin_transaction
					l_server.sql_execute_immediate(l_statement)
					if not l_server.success then
						raise ("Could not create trigger.")
					end

					l_server.commit
					if not l_server.success then
						raise ("Commit failed while creating the trigger.")
					end
				end
			end

			trigger_created := True
		ensure
			has_trigger: signature_item_server.has_trigger(table_name, data_name_prefix + "signature_version_update") or
						 not signature_item_server.has_table(data_name_prefix + "data_signature_versions")
		end

	drop_trigger is
			-- Drop the trigger from the database
		do
			if signature_item_server.has_trigger(table_name, data_name_prefix + "signature_version_update") then
				signature_item_server.auxiliary_server.begin_transaction
				signature_item_server.auxiliary_server.sql_execute_immediate ("DROP TRIGGER " + data_name_prefix + "signature_version_update")
				signature_item_server.auxiliary_server.commit
			end

			trigger_created := False
		ensure
			no_trigger: not signature_item_server.has_trigger(table_name, data_name_prefix + "signature_version_update")
		end

feature {NONE} -- Implementation

	version_1_crdate: INTEGER is 20020312

	trigger_version_1: STRING
	-- WARNING!!!  If you change this you must also change 'version_1_crdate'.
	do
		Result :=
			"CREATE TRIGGER " + data_name_prefix + "signature_version_update ON " + table_name + "%N" +
			"FOR UPDATE %N" +
			"AS %N" +
			"BEGIN %N" +
			"%TIF @@ROWCOUNT = 0 %N" +
			"%T%T  RETURN %N" +
			"%N" +
			insert_into_versions +
			"END%N"
	end

	version_2_crdate: INTEGER is 20090704

	trigger_version_2: STRING
	-- WARNING!!!  If you change this you must also change 'version_2_crdate'.
	-- Version 2 is the same as version 1 except that it adds 'update_timestamp' and can only be used
	-- if the column 'timestamp' exists.
	do
		Result :=
			"CREATE TRIGGER " + data_name_prefix + "signature_version_update ON " + table_name + "%N" +
			"FOR UPDATE %N" +
			"AS %N" +
			"BEGIN %N" +
			"%TIF @@ROWCOUNT = 0 %N" +
			"%T%T  RETURN %N" +
			"%N" +
			insert_into_versions +
			"%N" +
			update_timestamp +
			"END%N"
	end

	insert_into_versions: STRING
	--
	-- WARNING! WARNING! WARNING! WARNING! WARNING! WARNING! WARNING! WARNING! WARNING!
	--
	-- If you modify this trigger then you MUST change 'version_1_crdate'
	-- above or the change will NOT take effect.  Use a date that is slightly in
	-- the future.  Also be sure to change the version number in the trigger.
	--
	do
		Result :=
			version_comment("1") +
			"%TINSERT %N" +
			"%T%T " + data_name_prefix + "data_signature_versions (signature_id,process_id) %N" +
			"%TSELECT %N" +
			"%T%Tb.signature_id,b.last_process_id as process_id %N" +
			"%TFROM %N" +
			"%T%Tdeleted d, " + data_name_prefix + "data_signatures b (NOLOCK) %N" +
			"%TWHERE %N" +
			"%T%Td.signature_id = b.signature_id and %N" +
			"%T%Td.last_process_id <> b.last_process_id%N"
	end

	update_timestamp: STRING
	--
	-- WARNING! WARNING! WARNING! WARNING! WARNING! WARNING! WARNING! WARNING! WARNING!
	--
	-- If you modify this trigger then you MUST change 'version_2_crdate'
	-- above or the change will NOT take effect.  Use a date that is slightly in
	-- the future.  Also be sure to change the version number in the trigger.
	--
	do
		Result :=
			version_comment("2") +
			"%TUPDATE %N%
			%%T%T" + data_name_prefix + "data_signatures %N%
			%%TSET %N%
			%%T%Ttimestamp = getutcdate() %N%
			%%TFROM %N%
			%%T%Tdeleted d, " + data_name_prefix + "data_signatures b %N%
			%%TWHERE %N%
			%%T%Td.signature_id = b.signature_id%N"
	end

	version_comment(a_version: STRING): STRING
	-- Let's people know what class created this trigger and when.
	do
		Result := "-- Version " + a_version + ", created by SIGNATURE_TAGGED_TRIGGER on " + (create {ROSE_DATE}.make_now_utc).debug_output + " UT.%N"
	end

	signature_item_server: SQL_DATA_SERVER
			-- The data server upon which the trigger should be created
	trigger_created: BOOLEAN
			-- Has the trigger been created by the current process?

invariant

	valid_server: signature_item_server /= Void

end









