Indexing
	description: "Signature tagged table structure"
	keywords: "database,signature,item,tagged,manager"
	author: "Nelson Chui"
	revision: "$Revision: #8 $"
	path : "$File: //rose/source/kernel/persistence/databases/signature_tagged/signature_tagged_virtual_record_predicate_cache.e $"
	
class
	SIGNATURE_TAGGED_VIRTUAL_RECORD_PREDICATE_CACHE [G -> ROSE_SIGNATURE_RECORD [ENTITY] create default_create end]
	
inherit
	SIGNATURE_TAGGED_DATA_TYPES

create
	make
	
feature -- Initialization
	
	make (a_table_structure: SIGNATURE_TAGGED_TABLE_STRUCTURE) is
		require
			a_table_structure_not_void: a_table_structure /= Void
		do
			create ur_signature.default_create
			table_structure := a_table_structure
			create predicate_cache.make (16)
			create dated_predicate_cache.make (16)
			signature_version_selection_saved := Void
			temp_signature_insert_predicate_saved := Void
			temp_signature_date_insert_predicate_saved := Void
			temp_signature_update_predicate_saved := Void
			override_signature_version_selection_saved := Void
			max_date_string := "'" + (create {ROSE_DATE}.make_max_date).ccyymmdd.out + "'"
		end
	
feature -- Access
	
	ur_signature: G
	
	table_structure: SIGNATURE_TAGGED_TABLE_STRUCTURE
	
feature -- Single field cube selection
	
	base_cube_selection (a_type: STRING; a_historical_mode, a_join_process_locally, a_is_partial_record: BOOLEAN): ANONYMOUS_PREDICATE is
		require
			type_not_void : a_type /= Void
		local
			l_key, l_fields, l_statement : STRING
		do
			l_key := predicate_cache_key (a_type, False, a_historical_mode, a_join_process_locally, a_is_partial_record)
			predicate_cache.search (l_key)
			if predicate_cache.found then
				Result := predicate_cache.found_item
			else
				if not a_join_process_locally then
					l_fields := "signature_id,item_id,file_date,value"
				else
					l_fields := "signature_id,item_id,process_id,value"
				end
				if a_historical_mode then
					l_statement := "select * from (select " + l_fields
				else
					l_statement := "select " + l_fields
				end
				l_statement := l_statement + " from " +
					" #signatures b " + " INNER LOOP JOIN " + 
					table_structure.data_name_prefix + "data_" + a_type + "_value v " +
					" ON " + " v.signature_id = b.id "
				if not a_join_process_locally then
					l_statement := l_statement + " INNER JOIN process d " + " ON " + " v.process_id = d.process_id "
				end
				if a_is_partial_record then
					l_statement := l_statement + " INNER JOIN #variables c " + " ON " + " v.item_id = c.id "
				end
				if a_historical_mode then
					l_statement := l_statement + 
						" union all select " + l_fields + 
						" from " + " #signatures b " + " INNER LOOP JOIN " +
						table_structure.data_name_prefix + "data_" + a_type + "_value_history v " +
						" ON " + " v.signature_id = b.id "
					if not a_join_process_locally then
						l_statement := l_statement + " INNER JOIN process d " + " ON " + " v.process_id = d.process_id "
					end
					if a_is_partial_record then
						l_statement := l_statement + " INNER JOIN #variables c " + " ON " + " v.item_id = c.id "
					end
					l_statement := l_statement + ") a order by signature_id"
				else
					l_statement := l_statement + " order by signature_id"
				end
				if not a_join_process_locally then
					l_statement := l_statement + ", file_date"
				else
					l_statement := l_statement + ", process_id"
				end
				create Result.make_using_server(table_structure.signature_item_server,l_statement)
				if not Result.success then
					print(l_statement)
					(create {EXCEPTIONS}).raise("Failed to create base_cube_selection predicate")
				end
				predicate_cache.force (Result, l_key)
				-- print(l_statement+"%N")
			end
		end
	
	override_base_cube_selection (a_type: STRING; a_historical_mode, a_is_partial_record: BOOLEAN): ANONYMOUS_PREDICATE is
		require
			type_not_void : a_type /= Void
		local
			l_key, l_statement : STRING
		do
			l_key := predicate_cache_key (a_type, True, a_historical_mode, False, a_is_partial_record)
			predicate_cache.search (l_key)
			if predicate_cache.found then
				Result := predicate_cache.found_item
			else
				l_statement := "select signature_id,item_id,start_date,value,end_date,override_type_id,user_id,update_date,comment " +
					" from " + " #signatures a " + " INNER LOOP JOIN " + 
					table_structure.override_family_name_prefix + "override_" + table_structure.data_name_prefix + "data_" + a_type + "_value v " +
					" ON " + " v.signature_id = a.id "
				if a_is_partial_record then
					l_statement := l_statement +
						" INNER JOIN #variables b " + " ON " + " v.item_id = b.id "
				end
				l_statement := l_statement +
					" where v.start_date <= ? "
				if not a_historical_mode then
					l_statement := l_statement +
						" and v.end_date > ? "
				end
				l_statement := l_statement + " order by signature_id, start_date"
				create Result.make_using_server(table_structure.signature_item_server,l_statement)
				if not Result.success then
					print(l_statement)
					(create {EXCEPTIONS}).raise("Failed to create override_base_cube_selection predicate")
				end
				predicate_cache.force (Result, l_key)
				-- print(l_statement+"%N")
			end
		end
	
	dated_base_cube_selection (a_type: STRING): ANONYMOUS_PREDICATE is
		require
			type_not_void : a_type /= Void
		local
			l_statement : STRING
		do
			dated_predicate_cache.search (a_type)
			if dated_predicate_cache.found then
				Result := dated_predicate_cache.found_item
			else
				l_statement := dated_base_cube_selection_statement (a_type, false) + " union all " + 
					dated_base_cube_selection_statement (a_type, True) + " order by signature_id, start_date"
				create Result.make_using_server(table_structure.signature_item_server,l_statement)
				if not Result.success then
					print(l_statement)
					(create {EXCEPTIONS}).raise("Failed to create dated_base_cube_selection predicate")
				end
				predicate_cache.force (Result, a_type)
			end
		end
		
	dated_base_cube_selection_statement (a_type: STRING; a_is_history_table: BOOLEAN): STRING is
		local
			l_history: STRING
		do
			if a_is_history_table then
				l_history := "_history"
			else
				l_history := ""
			end
			Result := "select v.signature_id as signature_id, v.item_id as item_id, max (p.file_date) as start_date, ( " +
				"select value from " + table_structure.data_name_prefix + "data_" + a_type + "_value" + l_history +
				" where signature_id = v.signature_id and item_id = v.item_id and process_id = max (v.process_id)) as value " +
				"from #signatures t inner loop join " + table_structure.data_name_prefix + "data_" + a_type + "_value" + l_history + " v on t.id = v.signature_id " +
				"join process p on v.process_id = p.process_id and p.file_date <= t.version_date " +
				"group by v.signature_id, v.item_id"
		end
	
feature -- Multiple field cube selection
	
	record_cube_selection (a_template: ROSE_SIGNATURE_RECORD_ITEM; a_historical_mode, a_join_process_locally: BOOLEAN): ANONYMOUS_PREDICATE is
		local
			l_key, l_table_name: STRING
		do
			l_key := predicate_cache_key (a_template.name, False, a_historical_mode, a_join_process_locally, False)
			predicate_cache.search (l_key)
			if predicate_cache.found then
				Result := predicate_cache.found_item
			else
				l_table_name := a_template.name + "_value"
				template := a_template
				Result := multiple_field_cube_selection (l_table_name, a_template.all_record_fields, a_historical_mode, a_join_process_locally)
				template := Void
				predicate_cache.force (Result, l_key)
			end
		end

	dated_record_cube_selection (a_template: ROSE_SIGNATURE_RECORD_ITEM): ANONYMOUS_PREDICATE is
		local
			l_table_name: STRING
		do
			dated_predicate_cache.search (a_template.name)
			if dated_predicate_cache.found then
				Result := dated_predicate_cache.found_item
			else
				l_table_name := a_template.name + "_value"
				Result := multiple_field_dated_cube_selection (l_table_name, a_template.record_fields)
				dated_predicate_cache.force (Result, a_template.name)
			end
		end

	signature_data_cube_selection (a_historical_mode, a_join_process_locally: BOOLEAN): ANONYMOUS_PREDICATE is
		local
			l_key, l_table_name: STRING
		do
			l_key := predicate_cache_key ("signature_data", False, a_historical_mode, a_join_process_locally, False)
			predicate_cache.search (l_key)
			if predicate_cache.found then
				Result := predicate_cache.found_item
			else
				l_table_name := "signatures"
				Result := multiple_field_cube_selection (l_table_name, ur_signature.data_fields, a_historical_mode, a_join_process_locally)
				predicate_cache.force (Result, l_key)
			end
		end

	dated_signature_data_cube_selection: ANONYMOUS_PREDICATE is
		local
			l_table_name: STRING
		do
			dated_predicate_cache.search ("signature_data")
			if dated_predicate_cache.found then
				Result := dated_predicate_cache.found_item
			else
				l_table_name := "signatures"
				Result := multiple_field_dated_cube_selection (l_table_name, ur_signature.data_fields)
				dated_predicate_cache.force (Result, "signature_data")
			end
		end

	override_record_cube_selection (a_template: ROSE_SIGNATURE_RECORD_ITEM; a_historical_mode: BOOLEAN): ANONYMOUS_PREDICATE is
		local
			l_key, l_table_name: STRING
		do
			l_key := predicate_cache_key (a_template.name, True, a_historical_mode, False, False)
			predicate_cache.search (l_key)
			if predicate_cache.found then
				Result := predicate_cache.found_item
			else
				l_table_name := a_template.name + "_value"
				Result := multiple_field_override_cube_selection (l_table_name, a_template.record_fields, a_historical_mode)
				predicate_cache.force (Result, l_key)
			end
		end

	override_signature_data_cube_selection (a_historical_mode: BOOLEAN): ANONYMOUS_PREDICATE is
		local
			l_key, l_table_name: STRING
		do
			l_key := predicate_cache_key ("signature_data", True, a_historical_mode, False, False)
			predicate_cache.search (l_key)
			if predicate_cache.found then
				Result := predicate_cache.found_item
			else
				l_table_name := "signatures_data"
				Result := multiple_field_override_cube_selection (l_table_name, ur_signature.data_fields, a_historical_mode)
				predicate_cache.force (Result, l_key)
			end
		end

feature -- Signature ID predicates
	
	temp_signature_insert_predicate: EXECUTION_PREDICATE is
		do
			if temp_signature_insert_predicate_saved = Void then
				create temp_signature_insert_predicate_saved.make(table_structure.signature_item_server, "insert into #signatures(id) values(?)")
				if not temp_signature_insert_predicate_saved.success then
					(create {EXCEPTIONS}).raise("Failed to create signature insert predicate")
				end
			end
			Result := temp_signature_insert_predicate_saved
		end
	
	temp_signature_date_insert_predicate: EXECUTION_PREDICATE is
		do
			if temp_signature_date_insert_predicate_saved = Void then
				create temp_signature_date_insert_predicate_saved.make(table_structure.signature_item_server, "insert into #signatures(id,version_date) values(?,?)")
				if not temp_signature_date_insert_predicate_saved.success then
					(create {EXCEPTIONS}).raise("Failed to create signature insert predicate")
				end
			end
			Result := temp_signature_date_insert_predicate_saved
		end
	
	temp_signature_update_predicate: EXECUTION_PREDICATE is
		do
			if temp_signature_update_predicate_saved = Void then
				create temp_signature_update_predicate_saved.make(table_structure.signature_item_server,
					"update t set version_date = ov.version_date from #signatures t join override_" + table_structure.data_name_prefix + "data_signature_versions ov " +
					"on t.id = ov.signature_id and (ov.version_date is NULL or t.version_date > ov.version_date)")
				if not temp_signature_update_predicate_saved.success then
					(create {EXCEPTIONS}).raise("Failed to create signature update predicate")
				end
			end
			Result := temp_signature_update_predicate_saved
		end
	
	signature_version_selection (a_join_process_locally: BOOLEAN): ANONYMOUS_PREDICATE is
		local
			l_statement: STRING
		do
			if signature_version_selection_saved = Void then
				l_statement := "select dsv.signature_id, dsv.process_id"
				if not a_join_process_locally then
					l_statement := l_statement + ", p.file_date"
				end
				l_statement := l_statement + " from " 
					+ table_structure.data_name_prefix + "data_signature_versions dsv join #signatures t on dsv.signature_id = t.id "
				if not a_join_process_locally then
					l_statement := l_statement + " join process p on dsv.process_id = p.process_id order "
				end
				l_statement := l_statement + " order by signature_id"
				if not a_join_process_locally then
					l_statement := l_statement + ", file_date"
				end
				create signature_version_selection_saved.make_using_server (table_structure.signature_item_server, l_statement)
				if not signature_version_selection_saved.success then
					print (l_statement)
					(create {EXCEPTIONS}).raise ("Failed to create signature_version_selection predicate")
				end
				-- print(l_statement+"%N")
			end
			Result := signature_version_selection_saved
		end

	override_signature_version_selection: ANONYMOUS_PREDICATE is
		do
			if override_signature_version_selection_saved = Void then
				create override_signature_version_selection_saved.make_using_server (table_structure.signature_item_server,
					"select osv.signature_id, osv.version_date from #signatures ts with (index (0)) " 
					+ "join override_" + table_structure.data_name_prefix + "data_signature_versions osv with (index (0)) "
					+ "on ts.id = osv.signature_id order by osv.signature_id")
				if not override_signature_version_selection_saved.success then
					(create {EXCEPTIONS}).raise ("Could not create override signature version selection predicate.")
				end
			end
			Result := override_signature_version_selection_saved
		end
		
feature {NONE} -- Implementation
	max_date_string: STRING

	template: ROSE_SIGNATURE_RECORD_ITEM

	predicate_cache, dated_predicate_cache: HASH_TABLE [ANONYMOUS_PREDICATE, STRING]
	
	signature_version_selection_saved: ANONYMOUS_PREDICATE
	
	override_signature_version_selection_saved: ANONYMOUS_PREDICATE

	temp_signature_insert_predicate_saved, temp_signature_date_insert_predicate_saved, temp_signature_update_predicate_saved: EXECUTION_PREDICATE

	predicate_cache_key (a_type_name: STRING; a_is_override, a_historical_mode, a_join_process_locally, a_is_partial_record: BOOLEAN): STRING is
		local
			l_code: INTEGER
		do
			l_code := 16
			if a_is_override then
				l_code := 1
			end
			if a_historical_mode then
				l_code := l_code + 2
			end
			if a_join_process_locally then
				l_code := l_code + 4
			end
			if a_is_partial_record then
				l_code := l_code + 8
			end
			Result := l_code.out + a_type_name
		end
	
	multiple_field_cube_selection (a_table_name: STRING; a_fields: ROSE_LINEAR_ARRAY [STRING]; 
								   a_historical_mode, a_join_process_locally: BOOLEAN): ANONYMOUS_PREDICATE is
		local
			l_history_table_name, l_current_process_id_name, l_statement: STRING
			l_field_ix: INTEGER
			l_self_archive: BOOLEAN
			l_multi_table_archive: BOOLEAN
		do
			l_self_archive := template/=Void and then template.uses_self_archive
			l_multi_table_archive := a_historical_mode and not l_self_archive
			if a_table_name.as_lower.is_equal ("signatures") then
				l_current_process_id_name := "signature_data_process_id"
				l_history_table_name := "signatures_data"
			else
				l_current_process_id_name := "process_id"
				l_history_table_name := a_table_name
			end

			if l_multi_table_archive
			then
				l_statement := "select * from (%Nselect a.signature_id"
			else
				l_statement := "select%Na.signature_id"
			end

			if not a_join_process_locally then
				l_statement := l_statement + ",%Nc.file_date"
			else
				l_statement := l_statement + ",%Na." + l_current_process_id_name + " as process_id"
			end

			from l_field_ix := a_fields.lower until l_field_ix > a_fields.upper loop
				l_statement := l_statement + ",%Na.[" + a_fields.item (l_field_ix) + "]"
				l_field_ix := l_field_ix + 1
			end

			l_statement := l_statement + "%Nfrom #signatures b INNER LOOP JOIN " + 
				table_structure.data_name_prefix + "data_" + a_table_name +
				" a ON%N" + "a.signature_id = b.id"

			if not a_historical_mode and l_self_archive
			then
				l_statement.append(" and%Na.[" + template.archive_date_field_name + "] = " + max_date_string)
			end

			if not a_join_process_locally then
				l_statement := l_statement + "%NINNER JOIN process c" + " ON%N" + "a." + l_current_process_id_name + " = c.process_id "
			end

			if l_multi_table_archive then
				l_statement := l_statement + "%Nunion all%Nselect d.signature_id"
				if not a_join_process_locally then
					l_statement := l_statement + ",%Nf.file_date"
				else
					l_statement := l_statement + ",%Nd.process_id"
				end

				from l_field_ix := a_fields.lower until l_field_ix > a_fields.upper loop
					l_statement := l_statement + ",%Nd.[" + a_fields.item (l_field_ix) + "]"
					l_field_ix := l_field_ix + 1
				end

				l_statement := l_statement + "%Nfrom " + " #signatures e INNER LOOP JOIN " + 
					table_structure.data_name_prefix + "data_" + l_history_table_name + "_history d ON%N" +
					"d.signature_id = e.id"

				if not a_join_process_locally then
					l_statement := l_statement + " INNER JOIN process f " + " ON " + " d.process_id = f.process_id "
				end

				l_statement := l_statement + "%N) g%Norder by signature_id"
			else
				l_statement := l_statement + "%Norder by signature_id"
			end

			if not a_join_process_locally then
				l_statement := l_statement + ", file_date"
			end

			create Result.make_using_server (table_structure.signature_item_server, l_statement)
			if not Result.success then
				print (l_statement)
				(create {EXCEPTIONS}).raise ("Failed to create record_cube_selection predicate")
			end
			-- print(l_statement+"%N")
		end

	multiple_field_override_cube_selection (a_table_name: STRING; a_fields: ROSE_LINEAR_ARRAY [STRING]; a_historical_mode: BOOLEAN): ANONYMOUS_PREDICATE is
		local
			l_statement: STRING
			l_field_ix: INTEGER
		do
			l_statement := "select signature_id,start_date,end_date,override_type_id,user_id,update_date,comment "
			from l_field_ix := a_fields.lower until l_field_ix > a_fields.upper loop
				l_statement := l_statement + ",[" + a_fields.item (l_field_ix) + "]"
				l_field_ix := l_field_ix + 1
			end
			l_statement := l_statement + 
				" from " + " #signatures b " + " INNER LOOP JOIN " + 
				"override_" + table_structure.data_name_prefix + "data_" + a_table_name + " v " +
				" ON " + " v.signature_id = b.id " +
				" where v.start_date <= ? "
			if not a_historical_mode then
				l_statement := l_statement +
					" and v.end_date > ? "
			end
			-- if a_table_name.as_lower.is_equal ("signatures") then
			-- 	l_statement := l_statement +
			-- 		" and override_type_id <> " + blocking_override_type_id.out
			-- end
			l_statement := l_statement + " order by signature_id, start_date, override_type_id desc"
			create Result.make_using_server (table_structure.signature_item_server, l_statement)
			if not Result.success then
				print (l_statement)
				(create {EXCEPTIONS}).raise ("Failed to create override_record_cube_selection predicate")
			end
			-- print(l_statement+"%N")
		end

	multiple_field_dated_cube_selection (a_table_name: STRING; a_fields: ROSE_LINEAR_ARRAY [STRING]): ANONYMOUS_PREDICATE is
		local
			l_history_table_name, l_current_process_id_name, l_statement: STRING
			l_field_ix: INTEGER
		do
			if a_table_name.as_lower.is_equal ("signatures") then
				l_current_process_id_name := "signature_data_process_id"
				l_history_table_name := "signatures_data"
			else
				l_current_process_id_name := "process_id"
				l_history_table_name := a_table_name
			end
			l_statement := "select v.signature_id as signature_id, p.file_date as start_date"
			from l_field_ix := a_fields.lower until l_field_ix > a_fields.upper loop
				l_statement := l_statement + ", v.[" + a_fields.item (l_field_ix) + "]"
				l_field_ix := l_field_ix + 1
			end
			l_statement := l_statement + " from " + " #signatures t " + " inner loop join " + 
				table_structure.data_name_prefix + "data_" + a_table_name + " v on " + " v.signature_id = t.id"
			l_statement := l_statement + " join process p on v." + l_current_process_id_name + " = p.process_id and p.file_date <= t.version_date"

			l_statement := l_statement + " union all select v.signature_id as signature_id, p.file_date as start_date"
			from l_field_ix := a_fields.lower until l_field_ix > a_fields.upper loop
				l_statement := l_statement + ", v.[" + a_fields.item (l_field_ix) + "]"
				l_field_ix := l_field_ix + 1
			end
			l_statement := l_statement + " from " + " #signatures t " + " inner loop join " + 
				table_structure.data_name_prefix + "data_" + l_history_table_name + "_history v on v.signature_id = t.id"
			l_statement := l_statement + " and v.process_id = (select max (a.process_id) from " +
				table_structure.data_name_prefix + "data_" + l_history_table_name + "_history a " +
				"join process b on a.process_id = b.process_id where a.signature_id = t.id and b.file_date <= t.version_date group by signature_id)"
			l_statement := l_statement + " join process p on v.process_id = p.process_id "
			l_statement := l_statement + " order by signature_id, start_date"

			create Result.make_using_server (table_structure.signature_item_server, l_statement)
			if not Result.success then
				print (l_statement)
				(create {EXCEPTIONS}).raise ("Failed to create record_cube_selection predicate")
			end
			-- print(l_statement+"%N")
		end

end
