note

	description:

		"Signature tagged virtual record set"

	keywords: "database,signature,item,tagged,manager"
	author: "Nelson Chui"
	last_editor: "$Author: $"
	date: "$Date: $"
	revision: "$Revision: $"
	path: "$File: $"

class
	SIGNATURE_TAGGED_VIRTUAL_RECORD_SET [G -> ROSE_SIGNATURE_RECORD [ENTITY] create default_create end]

inherit
	SIGNATURE_TAGGED_DATA_TYPES
	ROSE_SHARED_GLOBAL
	MEMORY

create
	make, make_new_similar_to

feature {NONE} -- Initialization

	make (a_table_structure: SIGNATURE_TAGGED_TABLE_STRUCTURE; a_template: ROSE_VIRTUAL_RECORD [G]) is
			-- make
		require
			a_table_structure_not_void: a_table_structure /= Void
		do
			cube_result_table_on := True -- Set default to on
			cache_pool_on := True -- Set default to on
			table_structure := a_table_structure
			create predicate_cache.make (table_structure)
			set_virtual_record_template (a_template)
			create cube_result.make_empty
			create cube_result_table.make (0)
			temp_table_key := table_structure.signature_item_server.node + "." + table_structure.server_spid.out
		end

	make_new_similar_to (a_other: like Current) is
			-- Make new similar to 'a_other'
		do
			cube_result_table_on := a_other.cube_result_table_on
			cache_pool_on := a_other.cache_pool_on
			table_structure := a_other.table_structure
			predicate_cache := a_other.predicate_cache
			virtual_record_template := a_other.virtual_record_template
			create cube_result.make_empty
			create cube_result_table.make (0)
			temp_table_key := a_other.temp_table_key
		end

feature -- Access

	datetime_now: ROSE_DATE

	historical_mode: BOOLEAN

	items: ROSE_LINEAR_ARRAY [INTEGER]

	process_id_map: HASH_TABLE [ROSE_DATE, INTEGER]

	cube_result : ROSE_LINEAR_ARRAY [ROSE_VIRTUAL_RECORD [G]]

	cube_result_table : HASH_TABLE [ROSE_VIRTUAL_RECORD [G], MULTI_KEY]

	cache_pool: SIGNATURE_TAGGED_CACHE [G]

	cube_result_table_on, cache_pool_on: BOOLEAN

	signatures: ROSE_LINEAR_ARRAY [G]
	
	signatures_with_overrides: ROSE_LINEAR_ARRAY [G] is
			-- Signatures with currently enabled overrides
		local
			l_predicate: COLLECTING_PREDICATE [G]
			l_statement: STRING
		do
			if table_structure.enabled_override_base_types.is_empty then
				create Result.make_empty
			else
				l_statement := "select * from " + table_structure.data_name_prefix + "data_signatures" +
					" where "
				from
					table_structure.enabled_override_base_types.start
				until
					table_structure.enabled_override_base_types.after
				loop
					l_statement.append (" signature_id in (select distinct signature_id from " +
										table_structure.override_family_name_prefix + "override_" + table_structure.data_name_prefix + "data_" +
										table_structure.enabled_override_base_types.current_item.first + "_value) or ")
					table_structure.enabled_override_base_types.forth
				end
				l_statement := l_statement.substring (1, l_statement.count - 4)
				create l_predicate.make (table_structure.signature_item_server, l_statement)
				if not l_predicate.success then
					error.raise ("Could not create predicate")
				end
				l_predicate.retrieve (Void, Void)
				if not l_predicate.success then
					error.raise ("Could not execute predicate")
				end
				Result := l_predicate.array
			end
		ensure
			non_void_result: Result /= Void
		end

feature -- Basic operations

	reset is
			--reset attributes
		do
			datetime_now := Void
			signatures := Void
			items := Void
			create cube_result.make_empty
			create cube_result_table.make (0)
		end

	empty_cube_result is
			-- Empty current result cache
		do
			create cube_result.make_empty
			create cube_result_table.make (0)
		end

	set_cube_result(a_value : like cube_result) is
			-- Set 'cube_result' to 'a_value'
		do
			cube_result := a_value
		end

	set_cube_result_table_on (a_value: BOOLEAN) is
			-- Avoid building cube result table if not necessary (signature.multi_key will not be called)
		do
			cube_result_table_on := a_value
		end

	set_cache_pool_on (a_value: BOOLEAN) is
			-- Avoid using cache pool (recycling containers) if not necessary
		do
			cache_pool_on := a_value
		end

	set_virtual_record_template (a_template: ROSE_VIRTUAL_RECORD [G]) is
			-- Set virtual record template to `a_template' or create a new one if Void
		do
			reset
			if a_template /= Void then
				virtual_record_template := a_template
			else
				create virtual_record_template
			end
		end

	cache_pool_signature (a_current_cache_pool: SIGNATURE_TAGGED_CACHE [G]; a_signature: G): ROSE_VIRTUAL_RECORD [G] is
			-- This procedure is for current data only
		require
			a_signature_not_void: a_signature /= Void
		local
			l_multi_key: MULTI_KEY
		do
			Result := a_current_cache_pool.next_signature (a_signature)
			cube_result.extend (Result)
			if cube_result_table_on then
				l_multi_key := a_signature.multi_key.deep_twin
				cube_result_table.force (Result, l_multi_key)
			end
		end

	load_signature_versions (a_signatures: ROSE_LINEAR_ARRAY [G];
							 a_cache_pool: SIGNATURE_TAGGED_CACHE [G];
							 a_join_process_locally: BOOLEAN
							 ) is
		require
			a_signatures_not_void: a_signatures /= Void
			signatures_in_order: signatures_in_order (a_signatures)
			signatures_dated: signatures_dated (a_signatures)
			a_cache_pool_not_void: a_cache_pool /= Void or else not cache_pool_on
		do
			historical_mode := True
			join_process_locally := a_join_process_locally
			if join_process_locally then
				process_id_map := table_structure.process_id_map
			end
			signatures := a_signatures
			cache_pool := a_cache_pool
			ordered_cube_of_signature_versions
		end

	load_virtual_records (a_historical_mode: BOOLEAN; a_signatures: ROSE_LINEAR_ARRAY [G]; a_items: ROSE_LINEAR_ARRAY [INTEGER];
						  a_cache_pool: SIGNATURE_TAGGED_CACHE [G];
						  a_join_process_locally: BOOLEAN
						  ) is
		require
			a_signatures_not_void: a_signatures /= Void
			signatures_in_order: signatures_in_order (a_signatures)
			signatures_dated: signatures_dated (a_signatures)
			a_cache_pool_not_void: a_cache_pool /= Void or else not cache_pool_on
		do
			create datetime_now.make_now
			historical_mode := a_historical_mode
			items := a_items
			is_partial_record := (items /= Void)
			join_process_locally := a_join_process_locally
			if join_process_locally then
				process_id_map := table_structure.process_id_map
			end
			signatures := a_signatures
			cache_pool := a_cache_pool
			ordered_cube_of_items
		end

	retrieve_signature_versions (a_signatures: ROSE_LINEAR_ARRAY [G]; a_template: like virtual_record_template; a_join_process_locally: BOOLEAN)
		: ROSE_LINEAR_ARRAY [G] is
			-- This routine retrieve signature versions without change the existing settings and caches
		require
			a_signatures_not_void: a_signatures /= Void
			signatures_in_order: signatures_in_order (a_signatures)
		local
			l_cache_pool_on, l_cube_result_table_on: BOOLEAN
			l_cube_result: like cube_result
			l_template: like virtual_record_template
		do
			l_cache_pool_on := cache_pool_on
			cache_pool_on := False
			l_cube_result_table_on := cube_result_table_on
			cube_result_table_on := False
			l_template := virtual_record_template
			if a_template = Void then
				create virtual_record_template
			else
				virtual_record_template := a_template
			end
			l_cube_result := cube_result
			create cube_result.make_empty
			load_signature_versions (dated_signatures (a_signatures), Void, a_join_process_locally)
			create Result.make (cube_result.lower, cube_result.upper)
			from cube_result.start
			until cube_result.after
			loop
				Result.put (cube_result.current_item.signature, cube_result.index)
				cube_result.forth
			end
			cache_pool_on := l_cache_pool_on
			cube_result_table_on := l_cube_result_table_on
			virtual_record_template := l_template
			cube_result := l_cube_result
		end

	retrieve_virtual_records (a_historical_mode: BOOLEAN; a_signatures: ROSE_LINEAR_ARRAY [G]; a_items: ROSE_LINEAR_ARRAY [INTEGER];
							  a_template: like virtual_record_template; a_join_process_locally: BOOLEAN): like cube_result is
			-- This routine retrieve virtual records without change the existing settings and caches
		require
			a_signatures_not_void: a_signatures /= Void
			signatures_in_order: signatures_in_order (a_signatures)
		local
			l_cache_pool_on, l_cube_result_table_on: BOOLEAN
			l_cube_result: like cube_result
			l_template: like virtual_record_template
		do
			l_cache_pool_on := cache_pool_on
			cache_pool_on := False
			l_cube_result_table_on := cube_result_table_on
			cube_result_table_on := False
			l_template := virtual_record_template
			if a_template = Void then
				create virtual_record_template
			else
				virtual_record_template := a_template
			end
			l_cube_result := cube_result
			create cube_result.make_empty
			load_virtual_records (a_historical_mode, dated_signatures (a_signatures), a_items, Void, a_join_process_locally)
			Result := cube_result
			cache_pool_on := l_cache_pool_on
			cube_result_table_on := l_cube_result_table_on
			virtual_record_template := l_template
			cube_result := l_cube_result
		end

	fill_pinned_version_dates (a_signatures: ROSE_LINEAR_ARRAY [G]): BOOLEAN is
			-- Return true if there is an overridden version date
		do
			signatures := a_signatures
			Result := pinned_version_dates_filled
			signatures := Void
		end

	post_rollback_cleanup is
	-- Call this ONLY when debugging and you ALWAYS call 'rollback' instead of 'commit'.
	-- Call this AFTER the call to 'rollback'.  This call is only for debugging and can not
	-- make any guarantees if a 'commit' has ever been called.
	-- Purpose:
	-- A rollback will remove any temp tables that were created.  This call causes use to 'forget'
	-- that we ever made the table and forces the next call to re-created them.
	do
		signature_node_spid_table.remove(temp_table_key)
		variable_node_spid_table.remove(temp_table_key)
	end

feature -- Requirements

	signatures_in_order (a_signatures: ROSE_LINEAR_ARRAY [G]): BOOLEAN is
			-- Is `a_signatures' in signature_id order?
		local
			l_signature_id: INTEGER
		do
			Result := True
			l_signature_id := -1
			from a_signatures.start until a_signatures.after or not Result
			loop
				Result := (a_signatures.current_item.signature_id > l_signature_id)
				l_signature_id := a_signatures.current_item.signature_id
				a_signatures.forth
			end
		end

	signatures_dated (a_signatures: ROSE_LINEAR_ARRAY [G]): BOOLEAN is
		do
			Result := True
			from a_signatures.start until a_signatures.after or not Result
			loop
				Result := (a_signatures.current_item.as_of_date /= Void)
				a_signatures.forth
			end
		end

	ordered_signatures (a_signatures: ROSE_LINEAR_ARRAY [G]): ROSE_LINEAR_ARRAY [G] is
			-- Copy of `a_signatures' in signature_id order
		do
			if not signatures_in_order (a_signatures) then
				create Result.make_from_array (a_signatures)
				Result.sort_by (agent ordering_by_signature_id)
			end
		end

	dated_signatures (a_signatures: ROSE_LINEAR_ARRAY [G]): ROSE_LINEAR_ARRAY [G] is
		local
			l_index: INTEGER
			l_signature: G
		do
			from l_index := a_signatures.lower until l_index > a_signatures.upper
			loop
				if (a_signatures.item (l_index).as_of_date = Void) then
					if Result = Void then
						create Result.make_from_array (a_signatures)
					end
					l_signature := a_signatures.item (l_index).twin
					table_structure.synchronize_process_dates (l_signature)
					Result.put (l_signature, l_index - a_signatures.lower + 1)
				end
				l_index := l_index + 1
			end
			if Result = Void then
				Result := a_signatures
			end
		end

feature {SIGNATURE_TAGGED_VIRTUAL_RECORD_SET} -- Core data members

	table_structure: SIGNATURE_TAGGED_TABLE_STRUCTURE

	virtual_record_template: ROSE_VIRTUAL_RECORD [G]

	predicate_cache: SIGNATURE_TAGGED_VIRTUAL_RECORD_PREDICATE_CACHE [G]

	temp_table_key: STRING

feature {NONE} -- Parameters

	is_partial_record: BOOLEAN

	join_process_locally: BOOLEAN

feature {NONE} -- Implementation

	ordered_cube_of_signature_versions is
			-- ordered cube of signature version
		do
			-- Setup
			fill_signature_temp_table
			create data_stacks.make_empty
			create data_types.make_empty
			-- Retrieve Items
			fill_signature_version_stack
			merge_data_stacks
			-- Cleanup
			data_types := Void
			data_stacks := Void
			drop_signatures_temp_table
		rescue
			drop_signatures_temp_table
			trace ("ordered_cube_of_signature_versions: Exception raised in retrival of records%N",Void)
		end

	ordered_cube_of_items is
		local
			l_collecting: BOOLEAN
		do
			l_collecting := collecting
			collection_off
			create data_stacks.make_empty
			create data_types.make_empty
			if not table_structure.signature_only then
				-- Setup
				fill_signature_temp_table
				fill_variables_temp_table
				-- Retrieve Items
				fill_data_stacks
			end
			merge_data_stacks
			-- Cleanup
			data_types := Void
			data_stacks := Void
			if not table_structure.signature_only then
				drop_signatures_temp_table
				if items /= Void then
					drop_variables_temp_table
				end
			end
			if l_collecting then
				collection_on
			end
		rescue
			if l_collecting then
				collection_on
			end
			if not table_structure.signature_only then
				drop_signatures_temp_table
				if items /= Void then
					drop_variables_temp_table
				end
			end
			trace ("ordered_cube_of_items: Exception raised in retrival of records%N",Void)
		end

feature {NONE} -- Virtual record creation

	default_g: G is
			-- Default value for G
		do end

	merge_data_stacks is
			-- This procedure populates the virtual records into cube_result and cube_result_table, order by signature_id, process_id
		require
			signature_not_void: signatures /= Void
			no_void_in_signatures: not signatures.has_value (default_g)
			data_stacks_not_void: data_stacks /= Void
			no_void_in_data_stacks: not data_stacks.has_value (void)
			cache_pool_not_void: cache_pool_on implies cache_pool /= Void
		local
			l_signature_date_pair: PAIR [INTEGER, ROSE_DATE]
			l_current_signature_id: INTEGER
			l_current_start_date: ROSE_DATE
			l_new_virtual_record: ROSE_VIRTUAL_RECORD [G]
			l_name: STRING
		do
			create cube_result.make_empty
			if cube_result_table_on then
				create cube_result_table.make (signatures.count)
			end
			from
				-- Set signature stack and data stacks to the starting point
				signatures.start
				from data_stacks.start until data_stacks.after
				loop
					data_stacks.current_item.start
					data_stacks.forth
				end
				l_current_signature_id := l_current_signature_id.max_value
				l_signature_date_pair := next_signature_date_pair
			until
				l_signature_date_pair.first = l_current_signature_id.max_value -- No more new signature process pair can be found
			loop
				-- New signature (must be either a signature change or a date change if we reach here).
				-- Signatures have to be ordered by signature id.  If not, application may crash below.  (Signature order is a precondition of load_virtual_records.
				-- If crash, check preconditions are set to Require True in Rose.)
				if l_signature_date_pair.first /= l_current_signature_id then
					if signatures.off then
						if table_structure.data_name = Void then
							l_name := ""
						else
							l_name := "[" + table_structure.data_name + "]"
						end
						if not signatures.is_empty then
							(create {EXCEPTIONS}).raise ("Data items are available that have no corresponding signature in%N" +
														 table_structure.signature_item_server.node + "." + table_structure.signature_item_server.database +
														 " " + l_name + "%N" +
														 "Last signature_id processed was " + signatures.last.signature_id.out)
						else
							(create {EXCEPTIONS}).raise ("Data items are available that have no corresponding signature in%N" +
														 table_structure.signature_item_server.node + "." + table_structure.signature_item_server.database +
														 " " + l_name)
						end
					end
					if cache_pool_on then
						l_new_virtual_record := cache_pool.next_signature (signatures.current_item.twin)
					else
						l_new_virtual_record := virtual_record_template.duplicate
						l_new_virtual_record.make (signatures.current_item.twin)
					end
					populate_output_containers (l_new_virtual_record, l_signature_date_pair.second)
					signatures.forth
				-- New version (if not a signature change, must be a date change)
				elseif historical_mode then
					if cache_pool_on then
						l_new_virtual_record := cache_pool.next_duplicate (l_new_virtual_record)
					else
						l_new_virtual_record := l_new_virtual_record.duplicate
					end
					populate_output_containers (l_new_virtual_record, l_signature_date_pair.second)
				end

				-- Get all data items for the current pair across all data types and populate virtual record
				l_current_signature_id := l_signature_date_pair.first
				if historical_mode then
					l_current_start_date := l_signature_date_pair.second
				end
				from data_stacks.start
				until data_stacks.after
				loop
					if not data_types.item (data_stacks.index).first then -- Regular item stack
						inspect data_types.item (data_stacks.index).second
						when stim_int32_type then
							integer_data_stack ?= data_stacks.current_item
							set_regular_integer_stack (integer_data_stack, l_new_virtual_record, l_current_signature_id, l_current_start_date)
						when stim_real_type then
							real_data_stack ?= data_stacks.current_item
							set_regular_real_stack (real_data_stack, l_new_virtual_record, l_current_signature_id, l_current_start_date)
						when stim_double_type then
							double_data_stack ?= data_stacks.current_item
							set_regular_double_stack (double_data_stack, l_new_virtual_record, l_current_signature_id, l_current_start_date)
						when stim_string_type then
							string_data_stack ?= data_stacks.current_item
							set_regular_string_stack (string_data_stack, l_new_virtual_record, l_current_signature_id, l_current_start_date)
						when stim_image_type then
							image_data_stack ?= data_stacks.current_item
							set_regular_image_stack (image_data_stack, l_new_virtual_record, l_current_signature_id, l_current_start_date)
						when stim_date_type then
							date_data_stack ?= data_stacks.current_item
							set_regular_date_stack (date_data_stack, l_new_virtual_record, l_current_signature_id, l_current_start_date)
						when stim_record_type then
							record_data_stack ?= data_stacks.current_item
							set_regular_record_stack (record_data_stack, l_new_virtual_record, l_current_signature_id, l_current_start_date)
						when stim_signature_data_type then
							signature_data_stack ?= data_stacks.current_item
							set_regular_signature_data (signature_data_stack, l_new_virtual_record, l_current_signature_id, l_current_start_date)
						else
							from until data_stacks.current_item.after
								or else data_stacks.current_item.current_item.signature_id /= l_current_signature_id
								or else (l_current_start_date /= Void and then
										 not data_stacks.current_item.current_item.start_date.is_equal (l_current_start_date))
							loop data_stacks.current_item.forth end
						end
					else -- Override item stack
						set_override_item_stack (data_stacks.current_item, l_new_virtual_record, data_types.item (data_stacks.index).second,
												 l_current_signature_id, l_current_start_date)
					end
					data_stacks.forth
				end
				l_new_virtual_record.set_as_of_date (l_signature_date_pair.second) -- In current mode, we continue updating the largest as-of-date of items
				if l_new_virtual_record.signature /= Void then
					l_new_virtual_record.signature.set_as_of_date (l_signature_date_pair.second) -- In current mode, we continue updating the largest as-of-date of items
				end
				-- Check exit condition
				l_signature_date_pair := next_signature_date_pair
			end
			-- Free memory
			data_stack := Void
			integer_data_stack := Void
			real_data_stack := Void
			double_data_stack := Void
			string_data_stack := Void
			image_data_stack := Void
			date_data_stack := Void
			record_data_stack := Void
			signature_data_stack := Void
		end

	next_signature_date_pair: PAIR [INTEGER, ROSE_DATE] is
			-- next signature_date pair
		do
			if historical_mode then
				Result := next_historical_signature_date_pair
			else
				Result := next_current_signature_date_pair
			end
		end

	next_current_signature_date_pair: PAIR [INTEGER, ROSE_DATE] is
		local
			l_smallest_signature_id, l_current_signature_id: INTEGER
			l_maximum_start_date, l_current_start_date: ROSE_DATE
		do
			-- Get the current pair of signature_id and date from signatures
			if not signatures.after then
				l_smallest_signature_id := signatures.current_item.signature_id
				l_maximum_start_date := signatures.current_item.as_of_date
			else
				l_smallest_signature_id := l_smallest_signature_id.max_value
			end
			-- Get the current pair of signature_id and date from data_stacks
			from data_stacks.start
			until data_stacks.after
			loop
				if not data_stacks.current_item.after then
					l_current_signature_id := data_stacks.current_item.current_item.signature_id
					l_current_start_date := data_stacks.current_item.current_item.start_date
					-- If we find a smaller signature ID, save it in smallest signature ID
					if (l_smallest_signature_id > l_current_signature_id) then
						l_smallest_signature_id := l_current_signature_id
						l_maximum_start_date := l_current_start_date
					-- Elseif we find a smaller process ID for the same signature ID, save it in smallest process ID
					elseif (l_smallest_signature_id = l_current_signature_id) and then (l_maximum_start_date < l_current_start_date) then
						l_maximum_start_date := l_current_start_date
					end
				end
				data_stacks.forth
			end
			-- Trap situations where only the signature data was updated and no data items were updated
			if not signatures.after then
				l_current_start_date := process_id_map.item (signatures.current_item.signature_data_process_id)
				if l_current_start_date /= Void and then l_current_start_date > l_maximum_start_date then
					l_maximum_start_date := l_current_start_date
				end
			end
			create Result.make (l_smallest_signature_id, l_maximum_start_date)
		end

	next_historical_signature_date_pair: PAIR [INTEGER, ROSE_DATE] is
		local
			l_smallest_signature_id, l_current_signature_id: INTEGER
			l_smallest_start_date, l_current_start_date: ROSE_DATE
		do
			-- Get the current pair of signature_id and date from signatures
			if not signatures.after then
				l_smallest_signature_id := signatures.current_item.signature_id
				l_smallest_start_date := signatures.current_item.as_of_date
			else
				l_smallest_signature_id := l_smallest_signature_id.max_value
			end
			-- Get the current pair of signature_id and date from data_stacks
			from data_stacks.start
			until data_stacks.after
			loop
				if not data_stacks.current_item.after then
					l_current_signature_id := data_stacks.current_item.current_item.signature_id
					l_current_start_date := data_stacks.current_item.current_item.start_date
					-- If we find a smaller signature ID, save it in smallest signature ID
					if (l_smallest_signature_id > l_current_signature_id)
					then
						l_smallest_signature_id := l_current_signature_id
						l_smallest_start_date := l_current_start_date
					-- Elseif we find a smaller process ID for the same signature ID, save it in smallest process ID
					elseif (l_smallest_signature_id = l_current_signature_id) and then (l_smallest_start_date > l_current_start_date) then
						l_smallest_start_date := l_current_start_date
					end
				end
				data_stacks.forth
			end
			create Result.make (l_smallest_signature_id, l_smallest_start_date)
		end

	populate_output_containers (a_virtual_record: ROSE_VIRTUAL_RECORD [G]; a_as_of_date: ROSE_DATE) is
		local
			l_multi_key: MULTI_KEY
		do
			if historical_mode then
				if (not table_structure.version_pinning_enabled or else table_structure.showing_suppressed_historical_data or else
					a_virtual_record.signature.pinned_version_date = Void or else a_virtual_record.signature.pinned_version_date >= a_as_of_date)
				then
					if cube_result_table_on then
						l_multi_key := a_virtual_record.multi_key.deep_twin
						l_multi_key.array_force (a_as_of_date, l_multi_key.key_count + 1)
						cube_result_table.force (a_virtual_record, a_virtual_record.multi_key)
						cube_result_table.extend (a_virtual_record, l_multi_key)
					end
					cube_result.extend (a_virtual_record)
				end
			else
				if cube_result_table_on then
					cube_result_table.extend (a_virtual_record, a_virtual_record.multi_key)
				end
				cube_result.extend (a_virtual_record)
			end
		end

feature {NONE} -- Item-specific handling

	set_regular_integer_stack (a_data_stack: ROSE_LINEAR_ARRAY [DATA_RECORD [INTEGER]]; a_virtual_record: ROSE_VIRTUAL_RECORD [G];
							   a_signature_id: INTEGER; a_start_date: ROSE_DATE) is
		local
			l_has_overrides: BOOLEAN
			l_data_item: DATA_RECORD [INTEGER]
			l_overridden_item: OVERRIDE_DATA_RECORD [INTEGER]
		do
			from
				l_has_overrides := (a_virtual_record.overridden_integer_items /= Void) and (a_virtual_record.overridden_integer_items.count > 0)
			until
				a_data_stack.after
					or else a_data_stack.current_item.signature_id /= a_signature_id
					or else (a_start_date /= Void and then not a_data_stack.current_item.start_date.is_equal (a_start_date))
			loop
				l_data_item := a_data_stack.current_item
				if not l_has_overrides then
					a_virtual_record.set_integer_item (l_data_item.value, l_data_item.item_id)
				else
					a_virtual_record.overridden_integer_items.search (l_data_item.item_id)
					if not a_virtual_record.overridden_integer_items.found then
						a_virtual_record.set_integer_item (l_data_item.value, l_data_item.item_id)
					elseif a_virtual_record.overridden_integer_items.found_item.override_type_id = Provisional_override_type_id then
						a_virtual_record.set_integer_item (l_data_item.value, l_data_item.item_id)
						a_virtual_record.overridden_integer_items.remove (l_data_item.item_id)
					else -- Absolute / Confident
						l_overridden_item := a_virtual_record.overridden_integer_items.found_item.twin
						l_overridden_item.set_value (l_data_item.value)
						a_virtual_record.set_overridden_integer_item (l_overridden_item, l_data_item.item_id)
					end
				end
				a_data_stack.forth
			end
		end

	set_regular_real_stack (a_data_stack: ROSE_LINEAR_ARRAY [DATA_RECORD [REAL]]; a_virtual_record: ROSE_VIRTUAL_RECORD [G];
							a_signature_id: INTEGER; a_start_date: ROSE_DATE) is
		local
			l_has_overrides: BOOLEAN
			l_data_item: DATA_RECORD [REAL]
			l_overridden_item: OVERRIDE_DATA_RECORD [REAL]
		do
			from
				l_has_overrides := (a_virtual_record.overridden_real_items /= Void) and (a_virtual_record.overridden_real_items.count > 0)
			until
				a_data_stack.after
					or else a_data_stack.current_item.signature_id /= a_signature_id
					or else (a_start_date /= Void and then not a_data_stack.current_item.start_date.is_equal (a_start_date))
			loop
				l_data_item := a_data_stack.current_item
				if not l_has_overrides then
					a_virtual_record.set_real_item (l_data_item.value, l_data_item.item_id)
				else
					a_virtual_record.overridden_real_items.search (l_data_item.item_id)
					if not a_virtual_record.overridden_real_items.found then
						a_virtual_record.set_real_item (l_data_item.value, l_data_item.item_id)
					elseif a_virtual_record.overridden_real_items.found_item.override_type_id = Provisional_override_type_id then
						a_virtual_record.set_real_item (l_data_item.value, l_data_item.item_id)
						a_virtual_record.overridden_real_items.remove (l_data_item.item_id)
					else -- Absolute / Confident
						l_overridden_item := a_virtual_record.overridden_real_items.found_item.twin
						l_overridden_item.set_value (l_data_item.value)
						a_virtual_record.set_overridden_real_item (l_overridden_item, l_data_item.item_id)
					end
				end
				a_data_stack.forth
			end
		end

	set_regular_double_stack (a_data_stack: ROSE_LINEAR_ARRAY [DATA_RECORD [DOUBLE]]; a_virtual_record: ROSE_VIRTUAL_RECORD [G];
							  a_signature_id: INTEGER; a_start_date: ROSE_DATE) is
		local
			l_has_overrides: BOOLEAN
			l_data_item: DATA_RECORD [DOUBLE]
			l_overridden_item: OVERRIDE_DATA_RECORD [DOUBLE]
		do
			from
				l_has_overrides := (a_virtual_record.overridden_double_items /= Void) and (a_virtual_record.overridden_double_items.count > 0)
			until
				a_data_stack.after
					or else a_data_stack.current_item.signature_id /= a_signature_id
					or else (a_start_date /= Void and then not a_data_stack.current_item.start_date.is_equal (a_start_date))
			loop
				l_data_item := a_data_stack.current_item
				if not l_has_overrides then
					a_virtual_record.set_double_item (l_data_item.value, l_data_item.item_id)
				else
					a_virtual_record.overridden_double_items.search (l_data_item.item_id)
					if not a_virtual_record.overridden_double_items.found then
						a_virtual_record.set_double_item (l_data_item.value, l_data_item.item_id)
					elseif a_virtual_record.overridden_double_items.found_item.override_type_id = Provisional_override_type_id then
						a_virtual_record.set_double_item (l_data_item.value, l_data_item.item_id)
						a_virtual_record.overridden_double_items.remove (l_data_item.item_id)
					else -- Absolute / Confident
						l_overridden_item := a_virtual_record.overridden_double_items.found_item.twin
						l_overridden_item.set_value (l_data_item.value)
						a_virtual_record.set_overridden_double_item (l_overridden_item, l_data_item.item_id)
					end
				end
				a_data_stack.forth
			end
		end

	set_regular_string_stack (a_data_stack: ROSE_LINEAR_ARRAY [DATA_RECORD [STRING]]; a_virtual_record: ROSE_VIRTUAL_RECORD [G];
							  a_signature_id: INTEGER; a_start_date: ROSE_DATE) is
		local
			l_has_overrides: BOOLEAN
			l_data_item: DATA_RECORD [STRING]
			l_overridden_item: OVERRIDE_DATA_RECORD [STRING]
		do
			from
				l_has_overrides := (a_virtual_record.overridden_string_items /= Void) and (a_virtual_record.overridden_string_items.count > 0)
			until
				a_data_stack.after
					or else a_data_stack.current_item.signature_id /= a_signature_id
					or else (a_start_date /= Void and then not a_data_stack.current_item.start_date.is_equal (a_start_date))
			loop
				l_data_item := a_data_stack.current_item
				if not l_has_overrides then
					a_virtual_record.set_string_item (l_data_item.value, l_data_item.item_id)
				else
					a_virtual_record.overridden_string_items.search (l_data_item.item_id)
					if not a_virtual_record.overridden_string_items.found then
						a_virtual_record.set_string_item (l_data_item.value, l_data_item.item_id)
					elseif a_virtual_record.overridden_string_items.found_item.override_type_id = Provisional_override_type_id then
						a_virtual_record.set_string_item (l_data_item.value, l_data_item.item_id)
						a_virtual_record.overridden_string_items.remove (l_data_item.item_id)
					else -- Absolute / Confident
						l_overridden_item := a_virtual_record.overridden_string_items.found_item.twin
						l_overridden_item.set_value (l_data_item.value)
						a_virtual_record.set_overridden_string_item (l_overridden_item, l_data_item.item_id)
					end
				end
				a_data_stack.forth
			end
		end

	set_regular_image_stack (a_data_stack: ROSE_LINEAR_ARRAY [DATA_RECORD [STRING]]; a_virtual_record: ROSE_VIRTUAL_RECORD [G];
							  a_signature_id: INTEGER; a_start_date: ROSE_DATE) is
		local
			l_has_overrides: BOOLEAN
			l_data_item: DATA_RECORD [STRING]
			l_overridden_item: OVERRIDE_DATA_RECORD [STRING]
		do
			from
				l_has_overrides := (a_virtual_record.overridden_image_items /= Void) and (a_virtual_record.overridden_image_items.count > 0)
			until
				a_data_stack.after
					or else a_data_stack.current_item.signature_id /= a_signature_id
					or else (a_start_date /= Void and then not a_data_stack.current_item.start_date.is_equal (a_start_date))
			loop
				l_data_item := a_data_stack.current_item
				if not l_has_overrides then
					a_virtual_record.set_image_item (l_data_item.value, l_data_item.item_id)
				else
					a_virtual_record.overridden_image_items.search (l_data_item.item_id)
					if not a_virtual_record.overridden_image_items.found then
						a_virtual_record.set_image_item (l_data_item.value, l_data_item.item_id)
					elseif a_virtual_record.overridden_image_items.found_item.override_type_id = Provisional_override_type_id then
						a_virtual_record.set_image_item (l_data_item.value, l_data_item.item_id)
						a_virtual_record.overridden_image_items.remove (l_data_item.item_id)
					else -- Absolute / Confident
						l_overridden_item := a_virtual_record.overridden_image_items.found_item.twin
						l_overridden_item.set_value (l_data_item.value)
						a_virtual_record.set_overridden_image_item (l_overridden_item, l_data_item.item_id)
					end
				end
				a_data_stack.forth
			end
		end

	set_regular_date_stack (a_data_stack: ROSE_LINEAR_ARRAY [DATA_RECORD [ROSE_DATE]]; a_virtual_record: ROSE_VIRTUAL_RECORD [G];
							  a_signature_id: INTEGER; a_start_date: ROSE_DATE) is
		local
			l_has_overrides: BOOLEAN
			l_data_item: DATA_RECORD [ROSE_DATE]
			l_overridden_item: OVERRIDE_DATA_RECORD [ROSE_DATE]
		do
			from
				l_has_overrides := (a_virtual_record.overridden_date_items /= Void) and (a_virtual_record.overridden_date_items.count > 0)
			until
				a_data_stack.after
					or else a_data_stack.current_item.signature_id /= a_signature_id
					or else (a_start_date /= Void and then not a_data_stack.current_item.start_date.is_equal (a_start_date))
			loop
				l_data_item := a_data_stack.current_item
				if not l_has_overrides then
					a_virtual_record.set_date_item (l_data_item.value, l_data_item.item_id)
				else
					a_virtual_record.overridden_date_items.search (l_data_item.item_id)
					if not a_virtual_record.overridden_date_items.found then
						a_virtual_record.set_date_item (l_data_item.value, l_data_item.item_id)
					elseif a_virtual_record.overridden_date_items.found_item.override_type_id = Provisional_override_type_id then
						a_virtual_record.set_date_item (l_data_item.value, l_data_item.item_id)
						a_virtual_record.overridden_date_items.remove (l_data_item.item_id)
					else -- Absolute / Confident
						l_overridden_item := a_virtual_record.overridden_date_items.found_item.twin
						l_overridden_item.set_value (l_data_item.value)
						a_virtual_record.set_overridden_date_item (l_overridden_item, l_data_item.item_id)
					end
				end
				a_data_stack.forth
			end
		end

	set_regular_record_stack (a_data_stack: ROSE_LINEAR_ARRAY [ROSE_SIGNATURE_RECORD_ITEM]; a_virtual_record: ROSE_VIRTUAL_RECORD [G];
							  a_signature_id: INTEGER; a_start_date: ROSE_DATE) is
		local
			l_has_overrides: BOOLEAN
			l_data_item: ROSE_SIGNATURE_RECORD_ITEM
			l_overridden_item: OVERRIDE_DATA_RECORD [ROSE_SIGNATURE_RECORD_ITEM]
		do
			from
				l_has_overrides := (a_virtual_record.overridden_records /= Void) and then (a_virtual_record.overridden_records.count > 0)
			until
				a_data_stack.after
					or else a_data_stack.current_item.signature_id /= a_signature_id
					or else (a_start_date /= Void and then not a_data_stack.current_item.start_date.is_equal (a_start_date))
			loop
				l_data_item := a_data_stack.current_item
				if not l_has_overrides then
					a_virtual_record.set_record_item (l_data_item)
				else
					a_virtual_record.overridden_records.search (l_data_item.name)
					if not a_virtual_record.overridden_records.found then
						a_virtual_record.set_record_item (l_data_item)
					elseif a_virtual_record.overridden_records.found_item.override_type_id = Provisional_override_type_id then
						a_virtual_record.set_record_item (l_data_item)
						a_virtual_record.overridden_records.remove (l_data_item.name)
					else -- Absolute / Confident
						l_overridden_item := a_virtual_record.overridden_records.found_item.twin
						l_overridden_item.set_value (l_data_item)
						a_virtual_record.set_overridden_record_item (l_overridden_item, l_data_item.name)
					end
				end
				a_data_stack.forth
			end
		end

	set_regular_signature_data (a_data_stack: ROSE_LINEAR_ARRAY [DATA_RECORD [ARRAY [ANY]]]; a_virtual_record: ROSE_VIRTUAL_RECORD [G];
								a_signature_id: INTEGER; a_start_date: ROSE_DATE) is
		local
			l_has_overrides: BOOLEAN
			l_data_item: DATA_RECORD [ARRAY [ANY]]
			l_overridden_item: OVERRIDE_DATA_RECORD [G]
		do
			from
				l_has_overrides := (a_virtual_record.overridden_signature_data /= Void)
			until
				a_data_stack.after
					or else a_data_stack.current_item.signature_id /= a_signature_id
					or else (a_start_date /= Void and then not a_data_stack.current_item.start_date.is_equal (a_start_date))
			loop
				l_data_item := a_data_stack.current_item
				if not l_has_overrides then
					set_signature_data_fields (l_data_item.value, a_virtual_record.signature)
				else
					if a_virtual_record.overridden_signature_data.override_type_id = Provisional_override_type_id then
						set_signature_data_fields (l_data_item.value, a_virtual_record.signature)
						a_virtual_record.set_overridden_signature_data (Void)
					else -- Absolute / Confident
						l_overridden_item := a_virtual_record.overridden_signature_data
						set_signature_data_fields (l_data_item.value, l_overridden_item.value)
					end
				end
				a_data_stack.forth
			end
		end

feature {NONE} -- Override item-specific handling

	set_override_item_stack (a_data_stack: ROSE_LINEAR_ARRAY [DATA_RECORD_BASE]; a_virtual_record: ROSE_VIRTUAL_RECORD [G]; a_data_type: INTEGER;
							 a_signature_id: INTEGER; a_start_date: ROSE_DATE) is
		local
			l_integer_item: OVERRIDE_DATA_RECORD [INTEGER]
			l_real_item: OVERRIDE_DATA_RECORD [REAL]
			l_double_item: OVERRIDE_DATA_RECORD [DOUBLE]
			l_string_item: OVERRIDE_DATA_RECORD [STRING]
			l_image_item: OVERRIDE_DATA_RECORD [STRING]
			l_date_item: OVERRIDE_DATA_RECORD [ROSE_DATE]
			l_record_item: OVERRIDE_DATA_RECORD [ROSE_SIGNATURE_RECORD_ITEM]
			l_signature_data: OVERRIDE_DATA_RECORD [ARRAY [ANY]]
		do
			from until
				a_data_stack.after
					or else a_data_stack.current_item.signature_id /= a_signature_id
					or else (a_start_date /= Void and then not a_data_stack.current_item.start_date.is_equal (a_start_date))
			loop
				inspect a_data_type
				when stim_int32_type then
					l_integer_item ?= a_data_stack.current_item
					set_override_integer_item (l_integer_item, a_virtual_record)
				when stim_real_type then
					l_real_item ?= a_data_stack.current_item
					set_override_real_item (l_real_item, a_virtual_record)
				when stim_double_type then
					l_double_item ?= a_data_stack.current_item
					set_override_double_item (l_double_item, a_virtual_record)
				when stim_string_type then
					l_string_item ?= a_data_stack.current_item
					set_override_string_item (l_string_item, a_virtual_record)
				when stim_image_type then
					l_image_item ?= a_data_stack.current_item
					set_override_image_item (l_image_item, a_virtual_record)
				when stim_date_type then
					l_date_item ?= a_data_stack.current_item
					set_override_date_item (l_date_item, a_virtual_record)
				when stim_record_type then
					l_record_item ?= a_data_stack.current_item
					set_override_record_item (l_record_item, a_virtual_record)
				when stim_signature_data_type then
					l_signature_data ?= a_data_stack.current_item
					set_override_signature_data (l_signature_data, a_virtual_record)
				end
				a_data_stack.forth
			end
		end

	set_override_integer_item (a_override_item: OVERRIDE_DATA_RECORD [INTEGER]; a_virtual_record: ROSE_VIRTUAL_RECORD [G]) is
		local
			l_override_value: INTEGER
		do
			if a_override_item.override_type_id = Terminating_override_type_id then
				a_virtual_record.overridden_integer_items.search (a_override_item.item_id)
				if a_virtual_record.overridden_integer_items.found
					and then a_virtual_record.overridden_integer_items.found_item.start_date < a_override_item.start_date then
					a_virtual_record.set_integer_item (a_virtual_record.overridden_integer_items.found_item.value, a_override_item.item_id)
					a_virtual_record.overridden_integer_items.remove (a_override_item.item_id)
				end
			else
				l_override_value := a_override_item.value
				if a_virtual_record.overridden_integer_items /= Void then a_virtual_record.overridden_integer_items.search (a_override_item.item_id) end
				if a_virtual_record.overridden_integer_items /= Void and then a_virtual_record.overridden_integer_items.found then
					a_override_item.set_value (a_virtual_record.overridden_integer_items.found_item.value)
				else
					-- There is no overridden regular item, look for item in the regular table
					if a_virtual_record.integer_items /= Void then a_virtual_record.integer_items.search (a_override_item.item_id) end
					if a_virtual_record.integer_items /= Void and then a_virtual_record.integer_items.found then
						a_override_item.set_value (a_virtual_record.integer_items.found_item)
					else
						a_override_item.set_value (ieee_undefined.undefined_integer)
					end
				end
				a_virtual_record.set_integer_item (l_override_value, a_override_item.item_id)
				a_virtual_record.set_overridden_integer_item (a_override_item, a_override_item.item_id)
			end
		end

	set_override_real_item (a_override_item: OVERRIDE_DATA_RECORD [REAL]; a_virtual_record: ROSE_VIRTUAL_RECORD [G]) is
		local
			l_override_value: REAL
		do
			if a_override_item.override_type_id = Terminating_override_type_id then
				a_virtual_record.overridden_real_items.search (a_override_item.item_id)
				if a_virtual_record.overridden_real_items.found
					and then a_virtual_record.overridden_real_items.found_item.start_date < a_override_item.start_date then
					a_virtual_record.set_real_item (a_virtual_record.overridden_real_items.found_item.value, a_override_item.item_id)
					a_virtual_record.overridden_real_items.remove (a_override_item.item_id)
				end
			else
				l_override_value := a_override_item.value
				if a_virtual_record.overridden_real_items /= Void then a_virtual_record.overridden_real_items.search (a_override_item.item_id) end
				if a_virtual_record.overridden_real_items /= Void and then a_virtual_record.overridden_real_items.found then
					a_override_item.set_value (a_virtual_record.overridden_real_items.found_item.value)
				else
					-- There is no overridden regular item, look for item in the regular table
					if a_virtual_record.real_items /= Void then a_virtual_record.real_items.search (a_override_item.item_id) end
					if a_virtual_record.real_items /= Void and then a_virtual_record.real_items.found then
						a_override_item.set_value (a_virtual_record.real_items.found_item)
					else
						a_override_item.set_value (ieee_undefined.undefined_real)
					end
				end
				a_virtual_record.set_real_item (l_override_value, a_override_item.item_id)
				a_virtual_record.set_overridden_real_item (a_override_item, a_override_item.item_id)
			end
		end

	set_override_double_item (a_override_item: OVERRIDE_DATA_RECORD [DOUBLE]; a_virtual_record: ROSE_VIRTUAL_RECORD [G]) is
		local
			l_override_value: DOUBLE
		do
			if a_override_item.override_type_id = Terminating_override_type_id then
				a_virtual_record.overridden_double_items.search (a_override_item.item_id)
				if a_virtual_record.overridden_double_items.found
					and then a_virtual_record.overridden_double_items.found_item.start_date < a_override_item.start_date then
					a_virtual_record.set_double_item (a_virtual_record.overridden_double_items.found_item.value, a_override_item.item_id)
					a_virtual_record.overridden_double_items.remove (a_override_item.item_id)
				end
			else
				l_override_value := a_override_item.value
				if a_virtual_record.overridden_double_items /= Void then a_virtual_record.overridden_double_items.search (a_override_item.item_id) end
				if a_virtual_record.overridden_double_items /= Void and then a_virtual_record.overridden_double_items.found then
					a_override_item.set_value (a_virtual_record.overridden_double_items.found_item.value)
				else
					-- There is no overridden regular item, look for item in the regular table
					if a_virtual_record.double_items /= Void then a_virtual_record.double_items.search (a_override_item.item_id) end
					if a_virtual_record.double_items /= Void and then a_virtual_record.double_items.found then
						a_override_item.set_value (a_virtual_record.double_items.found_item)
					else
						a_override_item.set_value (ieee_undefined.undefined_double)
					end
				end
				a_virtual_record.set_double_item (l_override_value, a_override_item.item_id)
				a_virtual_record.set_overridden_double_item (a_override_item, a_override_item.item_id)
			end
		end

	set_override_string_item (a_override_item: OVERRIDE_DATA_RECORD [STRING]; a_virtual_record: ROSE_VIRTUAL_RECORD [G]) is
		local
			l_override_value: STRING
		do
			if a_override_item.override_type_id = Terminating_override_type_id then
				a_virtual_record.overridden_string_items.search (a_override_item.item_id)
				if a_virtual_record.overridden_string_items.found
					and then a_virtual_record.overridden_string_items.found_item.start_date < a_override_item.start_date then
					a_virtual_record.set_string_item (a_virtual_record.overridden_string_items.found_item.value, a_override_item.item_id)
					a_virtual_record.overridden_string_items.remove (a_override_item.item_id)
				end
			else
				l_override_value := a_override_item.value
				if a_virtual_record.overridden_string_items /= Void then a_virtual_record.overridden_string_items.search (a_override_item.item_id) end
				if a_virtual_record.overridden_string_items /= Void and then a_virtual_record.overridden_string_items.found then
					a_override_item.set_value (a_virtual_record.overridden_string_items.found_item.value)
				else
					-- There is no overridden regular item, look for item in the regular table
					if a_virtual_record.string_items /= Void then a_virtual_record.string_items.search (a_override_item.item_id) end
					if a_virtual_record.string_items /= Void and then a_virtual_record.string_items.found then
						a_override_item.set_value (a_virtual_record.string_items.found_item)
					else
						a_override_item.set_value (Void)
					end
				end
				a_virtual_record.set_string_item (l_override_value, a_override_item.item_id)
				a_virtual_record.set_overridden_string_item (a_override_item, a_override_item.item_id)
			end
		end

	set_override_image_item (a_override_item: OVERRIDE_DATA_RECORD [STRING]; a_virtual_record: ROSE_VIRTUAL_RECORD [G]) is
		local
			l_override_value: STRING
		do
			if a_override_item.override_type_id = Terminating_override_type_id then
				a_virtual_record.overridden_image_items.search (a_override_item.item_id)
				if a_virtual_record.overridden_image_items.found
					and then a_virtual_record.overridden_image_items.found_item.start_date < a_override_item.start_date then
					a_virtual_record.set_image_item (a_virtual_record.overridden_image_items.found_item.value, a_override_item.item_id)
					a_virtual_record.overridden_image_items.remove (a_override_item.item_id)
				end
			else
				l_override_value := a_override_item.value
				if a_virtual_record.overridden_image_items /= Void then a_virtual_record.overridden_image_items.search (a_override_item.item_id) end
				if a_virtual_record.overridden_image_items /= Void and then a_virtual_record.overridden_image_items.found then
					a_override_item.set_value (a_virtual_record.overridden_image_items.found_item.value)
				else
					-- There is no overridden regular item, look for item in the regular table
					if a_virtual_record.image_items /= Void then a_virtual_record.image_items.search (a_override_item.item_id) end
					if a_virtual_record.image_items /= Void and then a_virtual_record.image_items.found then
						a_override_item.set_value (a_virtual_record.image_items.found_item)
					else
						a_override_item.set_value (Void)
					end
				end
				a_virtual_record.set_image_item (l_override_value, a_override_item.item_id)
				a_virtual_record.set_overridden_image_item (a_override_item, a_override_item.item_id)
			end
		end

	set_override_date_item (a_override_item: OVERRIDE_DATA_RECORD [ROSE_DATE]; a_virtual_record: ROSE_VIRTUAL_RECORD [G]) is
		local
			l_override_value: ROSE_DATE
		do
			if a_override_item.override_type_id = Terminating_override_type_id then
				a_virtual_record.overridden_date_items.search (a_override_item.item_id)
				if a_virtual_record.overridden_date_items.found
					and then a_virtual_record.overridden_date_items.found_item.start_date < a_override_item.start_date then
					a_virtual_record.set_date_item (a_virtual_record.overridden_date_items.found_item.value, a_override_item.item_id)
					a_virtual_record.overridden_date_items.remove (a_override_item.item_id)
				end
			else
				l_override_value := a_override_item.value
				if a_virtual_record.overridden_date_items /= Void then a_virtual_record.overridden_date_items.search (a_override_item.item_id) end
				if a_virtual_record.overridden_date_items /= Void and then a_virtual_record.overridden_date_items.found then
					a_override_item.set_value (a_virtual_record.overridden_date_items.found_item.value)
				else
					-- There is no overridden regular item, look for item in the regular table
					if a_virtual_record.date_items /= Void then a_virtual_record.date_items.search (a_override_item.item_id) end
					if a_virtual_record.date_items /= Void and then a_virtual_record.date_items.found then
						a_override_item.set_value (a_virtual_record.date_items.found_item)
					else
						a_override_item.set_value (Void)
					end
				end
				a_virtual_record.set_date_item (l_override_value, a_override_item.item_id)
				a_virtual_record.set_overridden_date_item (a_override_item, a_override_item.item_id)
			end
		end

	set_override_record_item (a_override_item: OVERRIDE_DATA_RECORD [ROSE_SIGNATURE_RECORD_ITEM]; a_virtual_record: ROSE_VIRTUAL_RECORD [G]) is
		local
			l_override_value: ROSE_SIGNATURE_RECORD_ITEM
		do
			l_override_value := a_override_item.value
			if a_override_item.override_type_id = Terminating_override_type_id then
				a_virtual_record.overridden_records.search (l_override_value.name)
				if a_virtual_record.overridden_records.found
					and then a_virtual_record.overridden_records.found_item.start_date < a_override_item.start_date then
					a_virtual_record.set_record_item (a_virtual_record.overridden_records.found_item.value)
					a_virtual_record.overridden_records.remove (l_override_value.name)
				end
			else
				if a_virtual_record.overridden_records /= Void then a_virtual_record.overridden_records.search (l_override_value.name) end
				if a_virtual_record.overridden_records /= Void and then a_virtual_record.overridden_records.found then
					a_override_item.set_value (a_virtual_record.overridden_records.found_item.value)
				else
					-- There is no overridden regular item, look for item in the regular table
					if a_virtual_record.records /= Void then a_virtual_record.records.search (l_override_value.name) end
					if a_virtual_record.records /= Void and then a_virtual_record.records.found then
						a_override_item.set_value (a_virtual_record.records.found_item)
					else
						a_override_item.set_value (Void)
					end
				end
				a_virtual_record.set_record_item (l_override_value)
				a_virtual_record.set_overridden_record_item (a_override_item, l_override_value.name)
			end
		end

	set_override_signature_data (a_override_item: OVERRIDE_DATA_RECORD [ARRAY [ANY]]; a_virtual_record: ROSE_VIRTUAL_RECORD [G]) is
		local
			l_override_item: OVERRIDE_DATA_RECORD [G]
			l_override_value: ARRAY [ANY]
			l_void: G
		do
			l_override_value := a_override_item.value
			if a_override_item.override_type_id = Terminating_override_type_id then
				if a_virtual_record.overridden_signature_data /= Void
					and then a_virtual_record.overridden_signature_data.start_date < a_override_item.start_date then
					a_virtual_record.set_signature (a_virtual_record.overridden_signature_data.value)
					a_virtual_record.set_overridden_signature_data (Void)
				end
			else
				create l_override_item.make (a_override_item.signature_id, a_override_item.start_date, 0, l_void)
				l_override_item.make_override_base (a_override_item.end_date, a_override_item.override_type_id, a_override_item.user_id,
													a_override_item.update_date, a_override_item.comment)
				if a_virtual_record.overridden_signature_data /= Void then
					l_override_item.set_value (a_virtual_record.overridden_signature_data.value)
				else
					-- There is no overridden regular item, look for item in the regular table
					l_override_item.set_value (a_virtual_record.signature.twin)
				end
				set_signature_data_fields (l_override_value, a_virtual_record.signature)
				a_virtual_record.set_overridden_signature_data (l_override_item)
			end
		end

feature {NONE} -- Data stack retrieval by type

	fill_signature_version_stack is
		do
			create data_stack.make_empty
			predicate_cache.signature_version_selection (join_process_locally).apply (agent insert_signature_version, Void, Void)
			if not predicate_cache.signature_version_selection (join_process_locally).success then
				(create {EXCEPTIONS}).raise("signature_version_selection failed.")
			end
			if join_process_locally then sort_by_date (data_stack) end
			if data_stack.count > 0 then
				data_stacks.extend (data_stack)
				data_types.extend (create {PAIR [BOOLEAN, INTEGER]}.make (False, stim_signature_version_type))
			end
			data_stack := Void
		end

	fill_data_stacks is
		local
			l_has_override_data: BOOLEAN
		do
			-- Retrieve regular signature data (We must process regular data before override data)
			if (historical_mode or pinned_version_retrieval) and table_structure.enabled_regular_signature_data then -- In the current mode, signature data is already in the signature
				retrieve_signature_data_stack (False)
			end
			-- Retrieve override signature data (We must process regular data before override data)
			if table_structure.enabled_override_signature_data then
				retrieve_signature_data_stack (True)
			end
			-- Retrieve regular base type item data (We must process regular data before override data)
			from table_structure.enabled_regular_base_types.start
			until table_structure.enabled_regular_base_types.after
			loop
				retrieve_base_type_stack (table_structure.enabled_regular_base_types.current_item, False)
				table_structure.enabled_regular_base_types.forth
			end
			-- Retrieve override base type item data (We must process regular data before override data)
			from table_structure.enabled_override_base_types.start
			until table_structure.enabled_override_base_types.after
			loop
				retrieve_base_type_stack (table_structure.enabled_override_base_types.current_item, True)
				table_structure.enabled_override_base_types.forth
			end
			-- Retrieve all available record items for this selection
			if table_structure.record_templates /= Void then
				-- Retrieve regular record type item data (We must process regular data before override data)
				from table_structure.enabled_regular_record_types.start
				until table_structure.enabled_regular_record_types.after loop
					retrieve_record_type_stack (table_structure.enabled_regular_record_types.current_item, False)
					table_structure.enabled_regular_record_types.forth
				end
				-- Retrieve override record type item data (We must process regular data before override data)
				from table_structure.enabled_override_record_types.start
				until table_structure.enabled_override_record_types.after loop
					retrieve_record_type_stack (table_structure.enabled_override_record_types.current_item, True)
					table_structure.enabled_override_record_types.forth
				end
			end
			-- Check if there is override data
			from data_types.start
			until data_types.after or l_has_override_data
			loop
				l_has_override_data := data_types.current_item.first
				data_types.forth
			end
			-- Sort by date the regular data when 1) process joined locally and 2) history mode or else has override data
			if not pinned_version_retrieval then
				from data_types.start
				until data_types.after
				loop
					if not data_types.current_item.first then -- Check on the regular data only, not the override data
						if join_process_locally and (historical_mode or else l_has_override_data) then
							if not sorted_by_signature_date(data_stacks.item (data_types.index)) then
								sort_by_date (data_stacks.item (data_types.index))
							end
						elseif join_process_locally then
							fill_with_max_start_date (data_stacks.item (data_types.index))
						end
					end
					data_types.forth
				end
			end
		end

	sorted_by_signature_date(a_array: ROSE_LINEAR_ARRAY [DATA_RECORD_BASE]) : BOOLEAN is
		local
			l_item, l_next_item : DATA_RECORD_BASE
		do
			Result := True
			if a_array.count > 1 then
				a_array.start
				l_item := a_array.current_item
				from a_array.forth until a_array.off or not Result loop
					l_next_item := a_array.current_item
					if l_item.signature_id = l_next_item.signature_id then
						if l_item.start_date > l_next_item.start_date then
							Result := False
						end
					end
					l_item := l_next_item
					a_array.forth
				end
			end
		end

	retrieve_base_type_stack (a_type: PAIR [STRING, INTEGER]; a_is_override_data: BOOLEAN) is
			-- retrieve base type stack from database
		local
			l_data_stack: ROSE_LINEAR_ARRAY [DATA_RECORD_BASE]
			l_pred: ANONYMOUS_PREDICATE
			l_join_process_locally: BOOLEAN
		do
			if not a_is_override_data then
				if pinned_version_retrieval then
					l_pred := predicate_cache.dated_base_cube_selection (a_type.first)
					l_join_process_locally := False
				else
					l_pred := predicate_cache.base_cube_selection(a_type.first, historical_mode, join_process_locally, is_partial_record)
					l_join_process_locally := join_process_locally
				end
				inspect a_type.second
				when stim_int32_type then
					create integer_data_stack.make_empty
					l_pred.apply(agent insert_integer_item (?, l_join_process_locally), Void, Void)
					if not l_pred.success then
						(create {EXCEPTIONS}).raise("base_cube_selection failed.")
					end
					l_data_stack := integer_data_stack
					integer_data_stack := Void
				when stim_real_type then
					create real_data_stack.make_empty
					l_pred.apply (agent insert_real_item (?, l_join_process_locally), Void, Void)
					if not l_pred.success then
						(create {EXCEPTIONS}).raise("base_cube_selection failed.")
					end
					l_data_stack := real_data_stack
					real_data_stack := Void
				when stim_double_type then
					create double_data_stack.make_empty
					l_pred.apply (agent insert_double_item (?, l_join_process_locally), Void, Void)
					if not l_pred.success then
						(create {EXCEPTIONS}).raise("base_cube_selection failed.")
					end
					l_data_stack := double_data_stack
					double_data_stack := Void
				when stim_string_type then
					create string_data_stack.make_empty
					l_pred.apply (agent insert_string_item (?, l_join_process_locally), Void, Void)
					if not l_pred.success then
						(create {EXCEPTIONS}).raise("base_cube_selection failed.")
					end
					l_data_stack := string_data_stack
					string_data_stack := Void
				when stim_image_type then
					create image_data_stack.make_empty
					l_pred.apply (agent insert_image_item (?, l_join_process_locally), Void, Void)
					if not l_pred.success then
						(create {EXCEPTIONS}).raise("base_cube_selection failed.")
					end
					l_data_stack := image_data_stack
					image_data_stack := Void
				when stim_date_type then
					create date_data_stack.make_empty
					l_pred.apply (agent insert_date_item (?, l_join_process_locally), Void, Void)
					if not l_pred.success then
						(create {EXCEPTIONS}).raise("base_cube_selection failed.")
					end
					l_data_stack := date_data_stack
					date_data_stack := Void
				end
			else -- Override data type
				data_item_type := a_type.second
				override_needs_to_be_sorted := False
				create data_stack.make_empty
				if not historical_mode then
					l_pred := predicate_cache.override_base_cube_selection (a_type.first, historical_mode, is_partial_record)
					l_pred.apply (agent insert_override_base_item, Void, <<datetime_now, datetime_now>>)
					if not l_pred.success then
						(create {EXCEPTIONS}).raise("base_cube_selection failed.")
					end
				else
					l_pred := predicate_cache.override_base_cube_selection (a_type.first, historical_mode, is_partial_record)
					l_pred.apply (agent insert_override_base_item, Void, <<datetime_now>>)
					if not l_pred.success then
						(create {EXCEPTIONS}).raise("base_cube_selection failed.")
					end
				end
				l_data_stack := data_stack
				if override_needs_to_be_sorted then
					sort_by_date (l_data_stack)
				end
				data_item_type := 0
				data_stack := Void
			end
			if l_data_stack.count > 0 then
				data_stacks.extend (l_data_stack)
				data_types.extend (create {PAIR [BOOLEAN, INTEGER]}.make (a_is_override_data, a_type.second))
			end
		end

	retrieve_record_type_stack (a_template : ROSE_SIGNATURE_RECORD_ITEM; a_is_override_data: BOOLEAN) is
			-- retrieve record type stack from database
		local
			l_pred: ANONYMOUS_PREDICATE
			l_join_process_locally: BOOLEAN
			l_data_stack: ROSE_LINEAR_ARRAY [DATA_RECORD_BASE]
		do
			data_record_template := a_template
			if not a_is_override_data then
				create record_data_stack.make_empty
				if pinned_version_retrieval then
					l_pred := predicate_cache.dated_record_cube_selection (a_template)
					l_join_process_locally := False
				else
					l_pred := predicate_cache.record_cube_selection (a_template, historical_mode, join_process_locally)
					l_join_process_locally := join_process_locally
				end
				l_pred.apply (agent insert_record_item (?, l_join_process_locally), Void, Void)
				if not l_pred.success then
					(create {EXCEPTIONS}).raise("record_cube_selection failed.")
				end
				l_data_stack := record_data_stack
				record_data_stack := Void
			else -- Override data type
				override_needs_to_be_sorted := False
				create data_stack.make_empty
				if not historical_mode then
					predicate_cache.override_record_cube_selection (a_template, historical_mode)
					.apply (agent insert_override_record_item, Void, <<datetime_now, datetime_now>>)
				else
					predicate_cache.override_record_cube_selection (a_template, historical_mode)
					.apply (agent insert_override_record_item, Void, <<datetime_now>>)
				end
				l_data_stack := data_stack
				if override_needs_to_be_sorted then
					sort_by_date (l_data_stack)
				end
				data_stack := Void
			end
			if l_data_stack.count > 0 then
				data_stacks.extend (l_data_stack)
				data_types.extend (create {PAIR [BOOLEAN, INTEGER]}.make (a_is_override_data, stim_record_type))
			end
			data_record_template := Void
		end

	retrieve_signature_data_stack (a_is_override_data: BOOLEAN) is
			-- retrieve signature data stack from database
		local
			l_pred: ANONYMOUS_PREDICATE
			l_join_process_locally: BOOLEAN
			l_data_stack: ROSE_LINEAR_ARRAY [DATA_RECORD_BASE]
--			l_index, l_index_for_last_signature, l_signature_id: INTEGER
--			l_start_date_for_last_signature: ROSE_DATE
--			l_current_record, l_next_record: DATA_RECORD_BASE
		do
			if not a_is_override_data then
				create signature_data_stack.make_empty
				if pinned_version_retrieval then
					l_pred := predicate_cache.dated_signature_data_cube_selection
					l_join_process_locally := False
				else
					l_pred := predicate_cache.signature_data_cube_selection (historical_mode, join_process_locally)
					l_join_process_locally := join_process_locally
				end
				l_pred.apply (agent insert_signature_data (?, l_join_process_locally), Void, Void)
				if not l_pred.success then
					(create {EXCEPTIONS}).raise("record_cube_selection failed.")
				end
				l_data_stack := signature_data_stack
				signature_data_stack := Void
				-- Signature data process id in signature table is really the first process id of the signature data
				-- process id in signature data history table is the process id (and date time) on which the signature data is replaced
				-- Therefore, we need to realign the process id (and date time) to account for this convention
--				if not sorted_by_signature_date (l_data_stack) then
--					sort_by_date (l_data_stack)
--				end
--				from l_index := l_data_stack.upper
--				until l_index < l_data_stack.lower
--				loop
--					l_current_record := l_data_stack.item (l_index)
--					if l_signature_id /= l_current_record.signature_id then
--						l_signature_id := l_current_record.signature_id
--						l_index_for_last_signature := l_index
--						l_start_date_for_last_signature := l_current_record.start_date
--						l_next_record := l_current_record
--					else
--						l_next_record.set_start_date (l_current_record.start_date)
--						l_data_stack.put (l_next_record, l_index)
--						if l_index = l_data_stack.lower or else l_signature_id /= l_data_stack.item (l_index-1).signature_id then
--							l_current_record.set_start_date (l_start_date_for_last_signature)
--							l_data_stack.put (l_current_record, l_index_for_last_signature)
--						else
--							l_next_record := l_current_record
--						end
--					end
--					l_index := l_index - 1
--				end
			else -- Override data type
				override_needs_to_be_sorted := False
				create data_stack.make_empty
				if not historical_mode then
					predicate_cache.override_signature_data_cube_selection (historical_mode)
					.apply (agent insert_override_signature_data, Void, <<datetime_now, datetime_now>>)
				else
					predicate_cache.override_signature_data_cube_selection (historical_mode)
					.apply (agent insert_override_signature_data, Void, <<datetime_now>>)
				end
				l_data_stack := data_stack
				if override_needs_to_be_sorted then
					sort_by_date (l_data_stack)
				end
				data_stack := Void
			end
			if l_data_stack.count > 0 then
				data_stacks.extend (l_data_stack)
				data_types.extend (create {PAIR [BOOLEAN, INTEGER]}.make (a_is_override_data, stim_signature_data_type))
			end
			data_record_template := Void
		end

feature {NONE} -- Data record insertion

	insert_integer_item (a_row: ARRAY [ANY]; a_join_process_locally: BOOLEAN) is
		local
			l_base_item: DATA_RECORD [INTEGER]
			l_signature_id_ref, l_process_id_ref, l_item_id_ref: INTEGER_REF
			l_start_date: ROSE_DATE
			l_ir: INTEGER_REF
		do
			if not a_join_process_locally then
				l_start_date ?= a_row.item (3)
			else
				l_process_id_ref ?= a_row.item (3)
				l_start_date := process_id_map.item (l_process_id_ref.item)
				if l_start_date = Void then
					table_structure.refresh_process_id_map
					l_start_date := process_id_map.item (l_process_id_ref.item)
				end
			end
			l_signature_id_ref ?= a_row.item (1)
			l_item_id_ref ?= a_row.item (2)
			if l_start_date /= Void then
				if a_row.item (4) = Void then -- Make sure we set NULL item as well
					create l_base_item.make (l_signature_id_ref.item, l_start_date, l_item_id_ref.item, ieee_undefined.undefined_integer)
				else
					l_ir ?= a_row.item (4) -- We know it must be an integer type
					create l_base_item.make (l_signature_id_ref.item, l_start_date, l_item_id_ref.item, l_ir.item)
				end
				integer_data_stack.extend (l_base_item)
			else
				raise_no_corresponding_as_of_date ("INTEGER", l_signature_id_ref, l_process_id_ref, l_item_id_ref)
			end
		end

	insert_real_item (a_row: ARRAY [ANY]; a_join_process_locally: BOOLEAN) is
		local
			l_base_item: DATA_RECORD [REAL]
			l_signature_id_ref, l_process_id_ref, l_item_id_ref: INTEGER_REF
			l_start_date: ROSE_DATE
			l_rr: REAL_REF
		do
			if not a_join_process_locally then
				l_start_date ?= a_row.item (3)
			else
				l_process_id_ref ?= a_row.item (3)
				l_start_date := process_id_map.item (l_process_id_ref.item)
				if l_start_date = Void then
					table_structure.refresh_process_id_map
					l_start_date := process_id_map.item (l_process_id_ref.item)
				end
			end
			l_signature_id_ref ?= a_row.item (1)
			l_item_id_ref ?= a_row.item (2)
			if l_start_date /= Void then
				if a_row.item (4) = Void then -- Make sure we set NULL item as well
					create l_base_item.make (l_signature_id_ref.item, l_start_date, l_item_id_ref.item, ieee_undefined.undefined_real)
				else
					l_rr ?= a_row.item (4) -- We know it must be a real type
					create l_base_item.make (l_signature_id_ref.item, l_start_date, l_item_id_ref.item, l_rr.item)
				end
				real_data_stack.extend (l_base_item)
			else
				raise_no_corresponding_as_of_date ("REAL", l_signature_id_ref, l_process_id_ref, l_item_id_ref)
			end
		end

	insert_double_item (a_row: ARRAY [ANY]; a_join_process_locally: BOOLEAN) is
		local
			l_base_item: DATA_RECORD [DOUBLE]
			l_signature_id_ref, l_process_id_ref, l_item_id_ref: INTEGER_REF
			l_start_date: ROSE_DATE
			l_dr: DOUBLE_REF
		do
			if not a_join_process_locally then
				l_start_date ?= a_row.item (3)
			else
				l_process_id_ref ?= a_row.item (3)
				l_start_date := process_id_map.item (l_process_id_ref.item)
				if l_start_date = Void then
					table_structure.refresh_process_id_map
					l_start_date := process_id_map.item (l_process_id_ref.item)
				end
			end
			l_signature_id_ref ?= a_row.item (1)
			l_item_id_ref ?= a_row.item (2)
			if l_start_date /= Void then
				if a_row.item (4) = Void then -- Make sure we set NULL item as well
					create l_base_item.make (l_signature_id_ref.item, l_start_date, l_item_id_ref.item, ieee_undefined.undefined_double)
				else
					l_dr ?= a_row.item (4) -- We know it must be a double type
					create l_base_item.make (l_signature_id_ref.item, l_start_date, l_item_id_ref.item, l_dr.item)
				end
				double_data_stack.extend (l_base_item)
			else
				raise_no_corresponding_as_of_date ("DOUBLE", l_signature_id_ref, l_process_id_ref, l_item_id_ref)
			end
		end

	insert_string_item (a_row: ARRAY [ANY]; a_join_process_locally: BOOLEAN) is
		local
			l_base_item: DATA_RECORD [STRING]
			l_signature_id_ref, l_process_id_ref, l_item_id_ref: INTEGER_REF
			l_start_date: ROSE_DATE
			l_string: STRING
		do
			if not a_join_process_locally then
				l_start_date ?= a_row.item (3)
			else
				l_process_id_ref ?= a_row.item (3)
				l_start_date := process_id_map.item (l_process_id_ref.item)
				if l_start_date = Void then
					table_structure.refresh_process_id_map
					l_start_date := process_id_map.item (l_process_id_ref.item)
				end
			end
			l_signature_id_ref ?= a_row.item (1)
			l_item_id_ref ?= a_row.item (2)
			if l_start_date /= Void then
				l_string ?= a_row.item (4) -- We know it must be a string type
				create l_base_item.make (l_signature_id_ref.item, l_start_date, l_item_id_ref.item, l_string)
				string_data_stack.extend (l_base_item)
			else
				raise_no_corresponding_as_of_date ("STRING", l_signature_id_ref, l_process_id_ref, l_item_id_ref)
			end
		end

	insert_image_item (a_row: ARRAY [ANY]; a_join_process_locally: BOOLEAN) is
		local
			l_base_item: DATA_RECORD [STRING]
			l_signature_id_ref, l_process_id_ref, l_item_id_ref: INTEGER_REF
			l_start_date: ROSE_DATE
			l_image: STRING
		do
			if not a_join_process_locally then
				l_start_date ?= a_row.item (3)
			else
				l_process_id_ref ?= a_row.item (3)
				l_start_date := process_id_map.item (l_process_id_ref.item)
				if l_start_date = Void then
					table_structure.refresh_process_id_map
					l_start_date := process_id_map.item (l_process_id_ref.item)
				end
			end
			l_signature_id_ref ?= a_row.item (1)
			l_item_id_ref ?= a_row.item (2)
			if l_start_date /= Void then
				l_image ?= a_row.item (4) -- We know it must be an image type
				create l_base_item.make (l_signature_id_ref.item, l_start_date, l_item_id_ref.item, l_image)
				image_data_stack.extend (l_base_item)
			else
				raise_no_corresponding_as_of_date ("IMAGE", l_signature_id_ref, l_process_id_ref, l_item_id_ref)
			end
		end

	insert_date_item (a_row: ARRAY [ANY]; a_join_process_locally: BOOLEAN) is
		local
			l_base_item: DATA_RECORD [ROSE_DATE]
			l_signature_id_ref, l_process_id_ref, l_item_id_ref: INTEGER_REF
			l_start_date: ROSE_DATE
			l_date: ROSE_DATE
		do
			if not a_join_process_locally then
				l_start_date ?= a_row.item (3)
			else
				l_process_id_ref ?= a_row.item (3)
				l_start_date := process_id_map.item (l_process_id_ref.item)
				if l_start_date = Void then
					table_structure.refresh_process_id_map
					l_start_date := process_id_map.item (l_process_id_ref.item)
				end
			end
			l_signature_id_ref ?= a_row.item (1)
			l_item_id_ref ?= a_row.item (2)
			if l_start_date /= Void then
				l_date ?= a_row.item (4) -- We know it must be a date type
				create l_base_item.make (l_signature_id_ref.item, l_start_date, l_item_id_ref.item, l_date)
				date_data_stack.extend (l_base_item)
			else
				raise_no_corresponding_as_of_date ("DATE", l_signature_id_ref, l_process_id_ref, l_item_id_ref)
			end
		end

	insert_override_base_item (a_row: ARRAY [ANY]) is
		local
			l_value: ANY
			l_ir: INTEGER_REF
			l_rr: REAL_REF
			l_dr: DOUBLE_REF
			l_string: STRING
			l_image: STRING
			l_datetime: ROSE_DATE
			l_base_item: OVERRIDE_DATA_RECORD_BASE
			l_integer_item: OVERRIDE_DATA_RECORD [INTEGER]
			l_real_item: OVERRIDE_DATA_RECORD [REAL]
			l_double_item: OVERRIDE_DATA_RECORD [DOUBLE]
			l_string_item: OVERRIDE_DATA_RECORD [STRING]
			l_image_item: OVERRIDE_DATA_RECORD [STRING]
			l_date_item: OVERRIDE_DATA_RECORD [ROSE_DATE]
			l_signature_id_ref, l_item_id_ref, l_override_type_id_ref: INTEGER_REF
			l_user_id, l_comment: STRING
			l_start_date, l_end_date, l_update_date: ROSE_DATE
		do
			l_signature_id_ref ?= a_row.item (1)
			l_item_id_ref ?= a_row.item (2)
			l_start_date ?= a_row.item (3)
			l_value := a_row.item (4)
			l_end_date ?= a_row.item (5)
			l_override_type_id_ref ?= a_row.item (6)
			l_user_id ?= a_row.item (7)
			l_update_date ?= a_row.item (8)
			l_comment ?= a_row.item (9)
			inspect data_item_type
			when stim_int32_type then
				if l_value = Void then -- Make sure we set NULL item as well
					create l_integer_item.make (l_signature_id_ref.item, l_start_date, l_item_id_ref.item, ieee_undefined.undefined_integer)
				else
					l_ir ?= l_value -- We know it must be an integer type
					create l_integer_item.make (l_signature_id_ref.item, l_start_date, l_item_id_ref.item, l_ir.item)
				end
				l_integer_item.make_override_base (l_end_date, l_override_type_id_ref.item, l_user_id, l_update_date, l_comment)
				l_base_item := l_integer_item
			when stim_real_type then
				if l_value = Void then -- Make sure we set NULL item as well
					create l_real_item.make (l_signature_id_ref.item, l_start_date, l_item_id_ref.item, ieee_undefined.undefined_real)
				else
					l_rr ?= l_value -- We know it must be a real type
					create l_real_item.make (l_signature_id_ref.item, l_start_date, l_item_id_ref.item, l_rr.item)
				end
				l_real_item.make_override_base (l_end_date, l_override_type_id_ref.item, l_user_id, l_update_date, l_comment)
				l_base_item := l_real_item
			when stim_double_type then
				if l_value = Void then -- Make sure we set NULL item as well
					create l_double_item.make (l_signature_id_ref.item, l_start_date, l_item_id_ref.item, ieee_undefined.undefined_double)
				else
					l_dr ?= l_value -- We know it must be a double type
					create l_double_item.make (l_signature_id_ref.item, l_start_date, l_item_id_ref.item, l_dr.item)
				end
				l_double_item.make_override_base (l_end_date, l_override_type_id_ref.item, l_user_id, l_update_date, l_comment)
				l_base_item := l_double_item
			when stim_string_type then
				l_string ?= l_value -- We know it must be a string type
				create l_string_item.make (l_signature_id_ref.item, l_start_date, l_item_id_ref.item, l_string)
				l_string_item.make_override_base (l_end_date, l_override_type_id_ref.item, l_user_id, l_update_date, l_comment)
				l_base_item := l_string_item
			when stim_image_type then
				l_image ?= l_value -- We know it must be a image type
				create l_image_item.make (l_signature_id_ref.item, l_start_date, l_item_id_ref.item, l_image)
				l_image_item.make_override_base (l_end_date, l_override_type_id_ref.item, l_user_id, l_update_date, l_comment)
				l_base_item := l_image_item
			when stim_date_type then
				l_datetime ?= l_value -- We know it must be a date type
				create l_date_item.make (l_signature_id_ref.item, l_start_date, l_item_id_ref.item, l_datetime)
				l_date_item.make_override_base (l_end_date, l_override_type_id_ref.item, l_user_id, l_update_date, l_comment)
				l_base_item := l_date_item
			end
			data_stack.extend (l_base_item)
			-- Insert terminating item
			if l_end_date <= datetime_now then
				l_base_item := l_base_item.twin
				l_base_item.set_start_date (l_end_date)
				l_base_item.set_override_type_id (Terminating_override_type_id)
				data_stack.extend (l_base_item)
				override_needs_to_be_sorted := True
			end
		end

	insert_record_item (a_row: ARRAY [ANY]; a_join_process_locally: BOOLEAN) is
		local
			l_record : ROSE_SIGNATURE_RECORD_ITEM
			l_signature_id_ref, l_process_id_ref: INTEGER_REF
			l_start_date: ROSE_DATE
		do
			if not a_join_process_locally then
				l_start_date ?= a_row.item (2)
			else
				l_process_id_ref ?= a_row.item (2)
				l_start_date := process_id_map.item (l_process_id_ref.item)
				if l_start_date = Void then
					table_structure.refresh_process_id_map
					l_start_date := process_id_map.item (l_process_id_ref.item)
				end
			end
			l_record := data_record_template.twin
			l_signature_id_ref ?= a_row.item (1)
			if l_start_date /= Void then
				l_record.set_signature_id (l_signature_id_ref.item)
				l_record.set_start_date (l_start_date)
				l_record.set_data_fields (a_row)
				record_data_stack.extend (l_record)
			else
				raise_no_corresponding_as_of_date (l_record.name, l_signature_id_ref, l_process_id_ref, 0)
			end
		end

	insert_override_record_item (a_row: ARRAY [ANY]) is
		local
			l_signature_id_ref: INTEGER_REF
			l_start_date, l_end_date, l_update_date: ROSE_DATE
			l_value: ROSE_SIGNATURE_RECORD_ITEM
			l_record_item : OVERRIDE_DATA_RECORD [ROSE_SIGNATURE_RECORD_ITEM]
			l_override_type_id_ref: INTEGER_REF
			l_user_id, l_comment: STRING
		do
			l_signature_id_ref ?= a_row.item (1)
			l_start_date ?= a_row.item (2)
			l_end_date ?= a_row.item (3)
			l_override_type_id_ref ?= a_row.item (4)
			l_user_id ?= a_row.item (5)
			l_update_date ?= a_row.item (6)
			l_comment ?= a_row.item (7)
			l_value := data_record_template.twin
			l_value.set_signature_id (l_signature_id_ref.item)
			l_value.set_start_date (l_start_date)
			l_value.set_data_fields (a_row)
			create l_record_item.make (l_signature_id_ref.item, l_start_date, 0, l_value)
			l_record_item.make_override_base (l_end_date, l_override_type_id_ref.item, l_user_id, l_update_date, l_comment)
			data_stack.extend (l_record_item)
			-- Insert terminating item
			if l_end_date <= datetime_now then
				l_record_item := l_record_item.twin
				l_record_item.set_start_date (l_end_date)
				l_record_item.set_override_type_id (Terminating_override_type_id)
				data_stack.extend (l_record_item)
				override_needs_to_be_sorted := True
			end
		end

	insert_signature_data (a_row: ARRAY [ANY]; a_join_process_locally: BOOLEAN) is
		local
			l_signature_data: DATA_RECORD [ARRAY [ANY]]
			l_signature_id_ref, l_process_id_ref: INTEGER_REF
			l_start_date: ROSE_DATE
		do
			if not a_join_process_locally then
				l_start_date ?= a_row.item (2)
			else
				l_process_id_ref ?= a_row.item (2)
				l_start_date := process_id_map.item (l_process_id_ref.item)
				if l_start_date = Void then
					table_structure.refresh_process_id_map
					l_start_date := process_id_map.item (l_process_id_ref.item)
				end
			end
			l_signature_id_ref ?= a_row.item (1)
			if l_start_date /= Void then
				create l_signature_data.make (l_signature_id_ref.item, l_start_date, 0, a_row.twin)
				signature_data_stack.extend (l_signature_data)
			else
				raise_no_corresponding_as_of_date ("SIGNATURE_DATA", l_signature_id_ref, l_process_id_ref, 0)
			end
		end

	insert_override_signature_data (a_row: ARRAY [ANY]) is
		require
			valid_row: a_row /= Void
		local
			l_signature_id_ref: INTEGER_REF
			l_start_date, l_end_date, l_update_date: ROSE_DATE
			l_signature_data : OVERRIDE_DATA_RECORD [ARRAY [ANY]]
			l_override_type_id_ref: INTEGER_REF
			l_user_id, l_comment: STRING
		do
			l_signature_id_ref ?= a_row.item (1)
			l_start_date ?= a_row.item (2)
			l_end_date ?= a_row.item (3)
			l_override_type_id_ref ?= a_row.item (4)
			l_user_id ?= a_row.item (5)
			l_update_date ?= a_row.item (6)
			l_comment ?= a_row.item (7)
			create l_signature_data.make (l_signature_id_ref.item, l_start_date, 0, a_row.twin)
			l_signature_data.make_override_base (l_end_date, l_override_type_id_ref.item, l_user_id, l_update_date, l_comment)
			data_stack.extend (l_signature_data)
			-- Insert terminating item
			if l_end_date <= datetime_now then
				l_signature_data := l_signature_data.twin
				l_signature_data.set_start_date (l_end_date)
				l_signature_data.set_override_type_id (Terminating_override_type_id)
				data_stack.extend (l_signature_data)
				override_needs_to_be_sorted := True
			end
		end

	set_signature_data_fields (a_row: ARRAY [ANY]; a_signature: G) is
		require
			a_signature_not_void: a_signature /= Void
		local
			l_type : INTEGER
			l_value: ANY
			l_string: STRING
		do
			from a_signature.data_fields.start
			until a_signature.data_fields.off loop
				l_type := a_signature.field_type (a_signature.data_fields.current_item)
				l_value := a_row.item (a_row.count - a_signature.data_fields.count + a_signature.data_fields.index)
				if l_type = {INTERNAL}.CHARACTER_TYPE then -- Note that all CHAR fields in SQL tables are retrieved as STRING in Eiffel
					l_string ?= l_value
					if l_string /= Void and then l_string.count > 0 then
						a_signature.set_field (a_signature.data_fields.current_item, l_string.item (1))
					else
						a_signature.set_field (a_signature.data_fields.current_item, Void)
					end
				else
					a_signature.set_field (a_signature.data_fields.current_item, l_value)
				end
				a_signature.data_fields.forth
			end
		end

	raise_no_corresponding_as_of_date (a_type: STRING; a_signature_id_ref, a_process_id_ref, a_item_id_ref: INTEGER_REF) is
		local
			l_signature_id, l_process_id, l_item_id: INTEGER_REF
		do
			if a_signature_id_ref /= Void then l_signature_id := a_signature_id_ref.item end
			if a_process_id_ref /= Void then l_process_id := a_process_id_ref.item end
			if a_item_id_ref /= Void then l_item_id := a_item_id_ref.item end
			warn ("raise_no_corresponding_as_of_date", "Error: " + a_type + " has no corresponding as of date: ", Void)
			warn ("raise_no_corresponding_as_of_date", "Server - " + table_structure.signature_item_server.node +
				  "/" + table_structure.signature_item_server.database +
				  "; Data name - " + table_structure.data_name_prefix +
				  "; Signature ID - " + l_signature_id.out +
				  "; Process ID - " + l_process_id.out +
				  "; Item ID - " + l_item_id.out + "%N", Void)
			error.flush_and_acknowledge
			--some databases still have unknown process ID, let not raise exception for now-- (create {EXCEPTIONS}).raise ("Item date cannot be mapped")
		end

feature {NONE} -- Signature Version Construction

	insert_signature_version (a_row: ARRAY [ANY]) is
		local
			l_version: DATA_SIGNATURE_VERSIONS
			l_signature_id_ref, l_process_id_ref: INTEGER_REF
			l_start_date: ROSE_DATE
		do
			l_signature_id_ref ?= a_row.item (1)
			l_process_id_ref ?= a_row.item (2)
			if not join_process_locally then
				l_start_date ?= a_row.item (3)
			else
				l_start_date := process_id_map.item (l_process_id_ref.item)
				if l_start_date = Void then
					table_structure.refresh_process_id_map
					l_start_date := process_id_map.item (l_process_id_ref.item)
				end
			end
			if l_start_date /= Void then
				create l_version.make (l_signature_id_ref.item, l_start_date, l_process_id_ref.item)
				data_stack.extend (l_version)
			else
				warn ("insert_signature_version", "Error: Signature has no corresponding as of date: ", Void)
				warn ("insert_signature_version", "Server - " + table_structure.signature_item_server.node +
					  "/" + table_structure.signature_item_server.database +
					  "; Data name - " + table_structure.data_name_prefix +
					  "; Signature ID - " + l_signature_id_ref.item.out +
					  "; Process ID - " + l_process_id_ref.item.out + "%N", Void)
					error.flush_and_acknowledge
					--some databases still have unknown process ID, let not raise exception for now
					-- (create {EXCEPTIONS}).raise ("Signature date cannot be mapped")
			end
		end

feature {NONE} -- Selection Space Configuration

	aggregate_into_insert_query (a_signature: G; a_loader: DATA_SERVER_FAST_LOAD) is
			-- Build up the single insert statement for signature ids
		require
			valid_server: a_loader /= Void
			valid_signature: a_signature /= Void
		local
			l_template: STID_ID_TEMPLATE
		do
			create l_template
			l_template.set_id (a_signature.signature_id)
			a_loader.insert (l_template)
		end

	has_no_pinned_version_date (a_signature: G): BOOLEAN is
			-- Does 'a_signature' not have a 'pinned_version_date' set
		do
			Result := a_signature.pinned_version_date = Void
		end

	stid_id_insert_threshold: INTEGER is 100
			-- Number of signature ids above which we should use fast loaders

	fill_signature_temp_table is
		local
			l_signature: G
			l_pinned_version_dates_filled: BOOLEAN
			l_template: STID_ID_TEMPLATE
			l_fast_load_server: SQL_DATA_SERVER
			l_fast_loader: DATA_SERVER_FAST_LOAD
			l_signatures_temp_table: STRING
			l_process_timer: ROSE_PROCESS_TIMER
		do
			create_signatures_temp_table
			if signatures.count > stid_id_insert_threshold and then signatures.for_all (agent has_no_pinned_version_date) then
				l_fast_load_server := (create {DATABASE_SERVER}).servers.make_from_repository_fast_load (table_structure.signature_item_server.logical)
				if l_fast_load_server = Void then
					(create {EXCEPTIONS}).raise ("Could not create fast load server for '" + table_structure.signature_item_server.logical + "'")
				end
				create l_process_timer.make
				l_signatures_temp_table := "##signatures" + l_process_timer.start_cpu.out
				l_fast_load_server.sql_execute_immediate ("create table " + l_signatures_temp_table + " (id int)")
				create l_template
				l_fast_loader := l_fast_load_server.fast_loader (l_signatures_temp_table, l_template)
				if l_fast_loader = Void then
					(create {EXCEPTIONS}).raise ("Could not create fast loader for '" + l_signatures_temp_table + "' on '" + table_structure.signature_item_server.logical + "'")
				end
				signatures.do_all (agent aggregate_into_insert_query (?, l_fast_loader))
				l_fast_loader.commit
				table_structure.signature_item_server.sql_execute_immediate ("insert into #signatures (id) (select id from " + l_signatures_temp_table + ")")
				l_fast_load_server.sql_execute_immediate ("drop table " + l_signatures_temp_table)
			else
				from signatures.start until signatures.after loop
					l_signature := signatures.current_item
					if l_signature.pinned_version_date = Void then
						predicate_cache.temp_signature_insert_predicate.execute(Void,<<l_signature.signature_id>>)
						if not predicate_cache.temp_signature_insert_predicate.success then
							(create {EXCEPTIONS}).raise ("signature insert predicate failed.")
						end
					else
						pinned_version_retrieval := not historical_mode -- Need pinned version retrieval for current mode only
						predicate_cache.temp_signature_date_insert_predicate.execute(Void,<<l_signature.signature_id,l_signature.pinned_version_date>>)
						if not predicate_cache.temp_signature_date_insert_predicate.success then
							(create {EXCEPTIONS}).raise ("signature date insert predicate failed.")
						end
					end
					signatures.forth
				end
			end
			if table_structure.version_pinning_enabled then
				l_pinned_version_dates_filled := pinned_version_dates_filled
				if not historical_mode and l_pinned_version_dates_filled then
					pinned_version_retrieval := True
					predicate_cache.temp_signature_update_predicate.execute (Void, Void)
					if not predicate_cache.temp_signature_update_predicate.success then
						(create {EXCEPTIONS}).raise ("signature update predicate failed.")
					end
				end
			end
		end

	pinned_version_dates_filled: BOOLEAN is
		local
			l_predicate: ANONYMOUS_PREDICATE
			l_signature: G
			l_array: EXTENDABLE_BOUNDED_ARRAY [ARRAY [ANY]]
			l_signature_id_ref: INTEGER_REF
			l_start_date: ROSE_DATE
		do
			l_predicate := predicate_cache.override_signature_version_selection
			l_predicate.retrieve (Void, Void)
			if not l_predicate.success then
				(create {EXCEPTIONS}).raise("override_signature_version_selection failed.")
			elseif l_predicate.array.count > 0 then
				-- Insert the version pinned date using a linear merge algorithm
				l_array := l_predicate.array
				from
					signatures.start -- This is sorted by signature_id
					l_array.start -- This is sorted by signature_id
					l_signature_id_ref ?= l_array.current_item.item (1)
				until
					signatures.after or l_array.after
				loop
					l_signature := signatures.current_item
					if not l_array.after and then l_signature.signature_id >= l_signature_id_ref.item then
						if l_signature.signature_id = l_signature_id_ref.item then
							if l_array.current_item.item (2) = Void then
								l_start_date := constants.minimum_date_time
							else
								l_start_date ?= l_array.current_item.item (2)
							end
							if l_signature.pinned_version_date = Void or else l_start_date < l_signature.pinned_version_date then
								Result := True
								l_signature.set_pinned_version_date (l_start_date)
							end
							signatures.forth
						end
						l_array.forth
						if not l_array.after then
							l_signature_id_ref ?= l_array.current_item.item (1)
						end
					else
						signatures.forth
					end
				end
			end
		end

	fill_variables_temp_table is
			-- Fill the variables temp table with the requested ids
		local
			l_predicate: EXECUTION_PREDICATE
		do
			if items /= Void then
				create_variables_temp_table
				create l_predicate.make (table_structure.signature_item_server, "insert into #variables(id) values(?)")
				if not l_predicate.success then
					(create {EXCEPTIONS}).raise ("Failed to create variable insert predicate")
				end
				from items.start until items.off loop
					l_predicate.execute (Void, <<items.current_item>>)
					if not l_predicate.success then
						(create {EXCEPTIONS}).raise ("variable insert predicate failed.")
					end
					items.forth
				end
			end
		end

	signature_node_spid_table, variable_node_spid_table: HASH_TABLE [BOOLEAN, STRING] is
		once
			create Result.make (32)
		end

	create_signatures_temp_table is
		do
			if not signature_node_spid_table.has (temp_table_key) then
				table_structure.signature_item_server.sql_execute_immediate ("create table #signatures(id int not null primary key clustered, version_date datetime null default ('99991231'))")
				if not table_structure.signature_item_server.success then
					(create {EXCEPTIONS}).raise("Could not create table.")
				end
				signature_node_spid_table.put (True, temp_table_key)
			end
		end

	create_variables_temp_table is
		do
			if not variable_node_spid_table.has (temp_table_key) then
				table_structure.signature_item_server.sql_execute_immediate("create table #variables(id INT not null primary key clustered)")
				if not table_structure.signature_item_server.success then
					(create {EXCEPTIONS}).raise("Could not create table.")
				end
				variable_node_spid_table.put (True, temp_table_key)
			end
		end

	drop_signatures_temp_table is
			-- This no longer 'drop's the table, it just 'truncate's it.
		do
			table_structure.signature_item_server.sql_execute_immediate ("truncate table #signatures")
			if not table_structure.signature_item_server.success then
				(create {EXCEPTIONS}).raise("Could not truncate #signatures.")
			end
		end

	drop_variables_temp_table is
		do
			table_structure.signature_item_server.sql_execute_immediate ("truncate table #variables")
			if not table_structure.signature_item_server.success then
				(create {EXCEPTIONS}).raise("Could not truncate #variables.")
			end
		end

feature {NONE} -- Temporary data members

	data_stacks: ROSE_LINEAR_ARRAY [ROSE_LINEAR_ARRAY [DATA_RECORD_BASE]] -- [stack 1, stack 2, ... , stack n]

	data_types: ROSE_LINEAR_ARRAY [PAIR [BOOLEAN, INTEGER]] -- [[is override 1, type index 1], ... , [is override n, type index n]]

	data_item_type: INTEGER

	data_record_template: ROSE_SIGNATURE_RECORD_ITEM

	data_stack: ROSE_LINEAR_ARRAY [DATA_RECORD_BASE]

	integer_data_stack: ROSE_LINEAR_ARRAY [DATA_RECORD [INTEGER]]

	real_data_stack: ROSE_LINEAR_ARRAY [DATA_RECORD [REAL]]

	double_data_stack: ROSE_LINEAR_ARRAY [DATA_RECORD [DOUBLE]]

	string_data_stack: ROSE_LINEAR_ARRAY [DATA_RECORD [STRING]]

	image_data_stack: ROSE_LINEAR_ARRAY [DATA_RECORD [STRING]]

	date_data_stack: ROSE_LINEAR_ARRAY [DATA_RECORD [ROSE_DATE]]

	record_data_stack: ROSE_LINEAR_ARRAY [ROSE_SIGNATURE_RECORD_ITEM]

	signature_data_stack: ROSE_LINEAR_ARRAY [DATA_RECORD [ARRAY [ANY]]]

	pinned_version_retrieval, override_needs_to_be_sorted: BOOLEAN

feature {NONE} -- Partial Sorted Array

	fill_with_max_start_date (a_array: ROSE_LINEAR_ARRAY [DATA_RECORD_BASE]) is
			-- a_array is already sorted by signature_id, needs to be filled with the max start_date by signature ID
		require
			a_array_not_void: a_array /= Void
		local
			l_index, l_start_pos, l_last_pos, l_signature_id: INTEGER
			l_max_start_date: ROSE_DATE
		do
			if a_array.count > 0 then
				from
					l_index := a_array.lower
					l_start_pos := a_array.lower
					l_last_pos := a_array.upper
					l_signature_id := a_array.item (l_index).signature_id
					l_max_start_date := a_array.item (l_index).start_date
				until
					l_index > l_last_pos
				loop
					if l_max_start_date < a_array.item (l_index).start_date then
						l_max_start_date := a_array.item (l_index).start_date
					end
					if l_index = l_last_pos or else a_array.item (l_index + 1).signature_id /= l_signature_id then
						-- Fill with max start date
						from
						until l_start_pos > l_index
						loop
							a_array.item (l_start_pos).set_start_date (l_max_start_date)
							l_start_pos := l_start_pos + 1
						end
						-- Start next
						if l_index < l_last_pos then
							l_signature_id := a_array.item (l_index + 1).signature_id
							l_max_start_date := a_array.item (l_index + 1).start_date
						end
					end
					l_index := l_index + 1
				end
			end
		end

	sort_by_date (a_array: ROSE_LINEAR_ARRAY [DATA_RECORD_BASE]) is
			-- a_array is already sorted by signature_id, needs to be sorted by start_date
		require
			a_array_not_void: a_array /= Void
		local
			l_index, l_start_pos, l_last_pos, l_signature_id: INTEGER
			l_subarray: ROSE_LINEAR_ARRAY [DATA_RECORD_BASE]
		do
			if a_array.count > 0 then
				from
					l_index := a_array.lower
					l_start_pos := a_array.lower
					l_last_pos := a_array.upper
					l_signature_id := a_array.item (l_index).signature_id
				until
					l_index > l_last_pos
				loop
					if l_index = l_last_pos or else a_array.item (l_index + 1).signature_id /= l_signature_id then
						-- Sort the subarray
						if l_index - l_start_pos > 0 then
							l_subarray := a_array.subarray (l_start_pos, l_index)
							l_subarray.sort_by (agent date_ordering)
							a_array.subcopy (l_subarray, l_subarray.lower, l_subarray.upper, l_start_pos)
						end
						-- Start next
						if l_index < l_last_pos then
							l_start_pos := l_index + 1
							l_signature_id := a_array.item (l_index + 1).signature_id
						end
					end
					l_index := l_index + 1
				end
			end
		end

	date_ordering (a_item_1, a_item_2: DATA_RECORD_BASE): BOOLEAN is
		do
			Result := (a_item_1.start_date < a_item_2.start_date)
		end

	ordering_by_signature_id (a_item_1, a_item_2: G): BOOLEAN is
		do
			Result := (a_item_1.signature_id < a_item_2.signature_id)
		end

feature {NONE} -- Constants

	ieee_undefined: ROSE_IEEE_UNDEFINED is
			-- IEEE undefined
		once
			create Result
		end

	constants: ROSE_CONSTANT_VALUES is
			-- Constants
		local
			l_constants: ROSE_CONSTANTS
		once
			create l_constants
			Result := l_constants.constants
		end

end
