indexing

	description: "ROSE_CONTEXT version of PERSISTENT_SIMILARS_GROUP"
	author: "Mark Howard"
	last_editor: "$Author: er $"
	date: "$Date: 2013/06/18 $"
	revision: "$Revision: #37 $"
	path: "$File: //rose/source/kernel/investment_entities/similars_group.e $"
	storable_version: ""

class SIMILARS_GROUP

inherit

	INVESTMENT_ENTITY
		undefine
			name_identifier_key
		end
	PERSISTENT_SIMILARS_GROUP
		undefine
			context,
			make,
			debug_output,
			make_context,
			name_identifier_key
		end

	ATTRIBUTES
		rename
			msci as attributes_msci,
			company_name as attributes_company_name,
			cusip as attributes_cusip,
			sedol as attributes_sedol,
			brim_code as attributes_brim_code,
			old_positional_id as attributes_old_positional_id,
			xrf_code as attributes_xrf_code
		undefine
			is_equal,
			copy
		end

create

	make,
	make_anonymous

feature -- Basic operations

	register is
			-- Register this entity.
		local
			l_id: STRING
		do
			l_id := attribute_by_name ("OLD POSITIONAL ID", today)
			if l_id /= Void and then l_id.is_integer then
				similars_id := l_id.to_integer
			end
			industries := industry_to_similars_group_co_history
		end

feature -- Access

	similars_id: INTEGER
			-- id

	name_identifier_key: STRING is
			-- name identifier key
		once
			Result := "name"
		end

	industries: like industry_to_similars_group_co_history

	industry_from_similars_group (a_date: ROSE_DATE): INDUSTRY is
			-- the industry to which the Current similars group belongs
		local
			l_interval: ENTITY_INTERVAL [ENTITY]
		do
			l_interval := industries.most_recent (a_date)
			if l_interval /= Void then
				Result ?= l_interval.entity
			end
		end

	sectors: like sector_to_similars_group_co_history is
			-- The sectors to which a similar group is related
		do
			if sectors_saved = Void then
				sectors_saved := sector_to_similars_group_co_history
			end
			Result := sectors_saved
		end

	sector_from_similars_group (a_date: ROSE_DATE): SECTOR is
			-- the sector to which the Current similars group belongs
		local
			l_interval: ENTITY_INTERVAL [ENTITY]
		do
			l_interval := sectors.most_recent (a_date)
			if l_interval /= Void then
				Result ?= l_interval.entity
			end
		end

	economic_sector (a_group_name: STRING; a_date: ROSE_DATE): like context.economic_sector_template is
			-- economic sector
		require
			valid_group_name: a_group_name /= Void
		local
			l_econ_sector_group: ECONOMIC_SECTOR_GROUP
			l_econ_sectors: like economic_sector_to_similars_group_co_history
			l_econ_sector: like context.economic_sector_template
			l_found: BOOLEAN
		do
			l_econ_sectors := economic_sector_to_similars_group_co_history
			l_econ_sector_group := context.economic_sector_group_by_name (a_group_name, a_date)
			if l_econ_sectors /= Void and l_econ_sector_group /= Void then
				from
					l_econ_sectors.start
				until
					l_econ_sectors.off or l_found
				loop
					l_econ_sector := l_econ_sectors.current_item.entity
					if l_econ_sector_group.has (l_econ_sector) then
						Result := l_econ_sector
						l_found := True
					end
					l_econ_sectors.forth
				end
			end
		end

	max_similars_id: INTEGER is
			-- max value of old positional id for all similars group entities
		local
			i: INTEGER
			l_similars_groups: ROSE_LINEAR_ARRAY [like Current]
		once
			l_similars_groups := handler.all_entities
			from
				i := l_similars_groups.lower
			until
				i > l_similars_groups.upper
			loop
				Result := Result.max (l_similars_groups.item (i).similars_id)
				i := i + 1
			end
		end

feature {NONE} -- Implementation

	sectors_saved: like sector_to_similars_group_co_history

	economic_sector_group_cache: HASH_TABLE [ECONOMIC_SECTOR_GROUP, KEY_PAIR [STRING, ROSE_DATE]] is
		-- Cache of
		once
			create Result.make (100)
		ensure
			valid_result: Result /= Void
		end

end
