note

	description:

		"Classic singular value decomposition"

	author: "Mark Howard"
	last_editor: "$Author: mk $"
	date: "$Date: 2012/08/20 $"
	revision: "$Revision: #12 $"
	path: "$File: //rose/source/kernel/infrastructure/math/singular_value_decomposition.e $"

class
	SINGULAR_VALUE_DECOMPOSITION

inherit
	DOUBLE_MATH
	EXCEPTIONS

create
	make

feature -- Initialization

	make (a_matrix : like matrix) is
			--create
		local
		--	l_timer : PROCESS_TIMER
			i,j : INTEGER
		do
			matrix := a_matrix
			height := a_matrix.height
			width := a_matrix.width
			create u.make(height,width)
			from i := 1 until i > height loop
				from j := 1 until j > width loop
					--u.put(matrix.item(i,j),i,j)
					u[i, j] := matrix[i, j]
					j := j + 1
				end
				i := i + 1
			end
			create v.make(width,width)
			create w.make(width)
			create diagonal.make(width)
			decompose_svd
			from i := 1 until i > width loop
			--	diagonal.put(w.item(i),i,i)
				diagonal[i, i] := w [i]
				i := i + 1
			end
			tolerance := 2.2204460492503e-14
		end

feature -- Access

	matrix : ABSTRACT_READABLE_MATRIX
	u : MATRIX
	v : MATRIX
	w : VECTOR
	diagonal : DIAGONAL_MATRIX
	width : INTEGER
	height : INTEGER

	pseudo_inverse : MATRIX is
			--see e.g. Golub and Van Loan, Matrix computations
		local
			l_w_inverse : MATRIX
			i : INTEGER
		do
			create l_w_inverse.make(w.count,w.count)
			from i := 1 until i > w.count loop
				if w .item(i).abs < tolerance then
				--	l_w_inverse.put(0,i,i)
					l_w_inverse[i, i] := 0
				else
--					l_w_inverse.put(1/w.item(i),i,i)
					l_w_inverse[i, i] := 1/w[i]
				end
				i := i + 1
			end
			Result := v * l_w_inverse * u.transpose
		end

	preimage (a_vector : VECTOR) : VECTOR is
			-- Preimage
		local
			i, j, jj: INTEGER;
			s,l_thresh: DOUBLE
			l_vector: VECTOR
			l_w : VECTOR
		do
			l_w := clone(w)
			create l_vector.make (width)
			create Result.make(width)
			l_thresh:=tolerance*w.max
			from j := 1 until j > width loop
				if (l_w[j] < l_thresh) then
				--	l_w.put (0.0,j)
					l_w[j] := 0.0
				end
				j :=  j + 1
			end
			from j := 1 until j > width loop
				s:=0.0;
				if (l_w [j] /= 0) then
					from i := 1 until i > height loop
						s := s + u [i,j] * a_vector [i]
						i := i + 1
					end
					s := s / l_w [j]
				end
			--	l_vector.put (s,j)
				l_vector[j] := s
				j := j + 1
			end--}
			from j := 1 until j > width loop
				s:=0.0;
				from jj := 1 until jj > width loop
					s := s + v [j,jj] * l_vector[jj]
					jj := jj + 1
				end
			--	Result.put (s,j)
				Result[j] := s
				j := j + 1
			end
		end

	xtx_preimage (a_vector : VECTOR) : VECTOR is
			--xtx=matrix.transpose * matrix = (u*d*v')'*u*d*v' = v*d*u'*u*d*v' = v*d*d*v', so
			--xtx.inverse = (v*d*d*v').inverse  = v*1/d*1/d*v'
		local
			i, j: INTEGER;
			s,l_thresh,l_max: DOUBLE
			l_vector: VECTOR
			l_d_inverse_square : VECTOR
			l_height : INTEGER
		do
			l_height := w.count
			create l_d_inverse_square.make (l_height)
			create l_vector.make (width)
			create Result.make(width)
			l_max := w.max
			l_thresh:=tolerance*l_max*l_max
			--compute inverse of diagonal square
			from i := 1 until i > l_height loop
				if (w[i]*w[i] < l_thresh) then
				--	l_w.put (0.0,j)
					l_d_inverse_square[i] := 0.0
				else
					l_d_inverse_square[i] := 1 / (w[i]*w[i])
				end
				i :=  i + 1
			end
			--compute (1/d * 1/d) @ (v' @ a_vector)
			--only need to compute v' @ a_vector where l_d_inverse_square is not 0
			from i := 1 until i > l_height loop
				s:=0
				if (l_d_inverse_square [i] /= 0) then
					from j := 1 until j > l_height loop
						--s := s + v'[i,j] * a_vector[j] = s + v[j,i] * a_vector[j]
						s := s + v [j,i] * a_vector [j]
						j := j + 1
					end
					s := s * l_d_inverse_square [i]
				end
				l_vector[i] := s
				i := i + 1
			end
			--compute Result = v @ ((1/d * 1/d) @ (v' @ a_vector))
			from i := 1 until i > l_height loop
				s:=0
				from j := 1 until j > l_height loop
					s := s + v [i,j] * l_vector[j]
					j := j + 1
				end
				Result[i] := s
				i := i + 1
			end
		end
	xtx : MATRIX
		do
			Result := v.product (diagonal * diagonal) * v.transpose
		end
	xtx_image (a_vector : VECTOR) : VECTOR is
			--xtx=matrix.transpose * matrix = (u*d*v')'*u*d*v' = v*d*u'*u*d*v' = v*d*d*v', so
			--xtx.inverse = (v*d*d*v').inverse  = v*1/d*1/d*v'
		local
			i, j: INTEGER;
			s,l_thresh,l_max: DOUBLE
			l_vector: VECTOR
			l_height : INTEGER
		do
			l_height := w.count
			create l_vector.make (width)
			create Result.make(width)
			l_max := w.max
			l_thresh:=tolerance*l_max*l_max
			from i := 1 until i > l_height loop
				s:=0
				if (w[i] /= 0) then
					from j := 1 until j > l_height loop
						--s := s + v'[i,j] * a_vector[j] = s + v[j,i] * a_vector[j]
						s := s + v [j,i] * a_vector [j]
						j := j + 1
					end
					s := s * w [i] * w[i]
				end
				l_vector[i] := s
				i := i + 1
			end
			--compute Result = v @ ((1/d * 1/d) @ (v' @ a_vector))
			from i := 1 until i > l_height loop
				s:=0
				from j := 1 until j > l_height loop
					s := s + v [i,j] * l_vector[j]
					j := j + 1
				end
				Result[i] := s
				i := i + 1
			end
		end

	special (a_count: INTEGER): SPECIAL [DOUBLE] is
			-- Special with 'a_count' elements
		do
			create Result.make_filled (0.0, a_count)
		end

	decompose_svd is
		local
			i,its,j,jj,k,l,nm: INTEGER
			anorm,c,f,g,h,s,scale,x,y,z: DOUBLE
			l_rv: SPECIAL[DOUBLE]
			flag,break1,break2: BOOLEAN
			l_u,l_v : SPECIAL[SPECIAL[DOUBLE]]
			l_w : SPECIAL[DOUBLE]
		do
			l_u := u.data
			l_v := v.data
			l_w := w.data
			l := 0; nm := 0;
			l_rv := special(width + 1)
			g := 0.0
			scale := 0.0
			anorm := 0.0
			from i := 1 until i > width loop
				l:=i+1;
				l_rv.put(scale*g,i)
				g := 0.0
				s := 0.0
				scale := 0.0
				if (i <= height) then
					from k := i until k > height loop
						scale := scale + fabs (l_u.item(k).item(i))
						k := k + 1
					end
					if (scale /= 0.0) then
						--	for (k=i;k<width;k++)
						from k := i until k > height loop
							l_u.item(k).put(l_u.item(k).item(i) / scale,i)
							--						l_u[k][i] /= scale;
							--s := s + l_u[k][i]*l_u[k][i];
							s := s + l_u.item(k).item(i) * l_u.item(k).item(i)
							k := k + 1
						end
						f:=l_u.item(i).item(i);
						g := -SIGN(sqrt(s),f);
						h:=(f*g)-s;
						l_u.item(i).put(f-g,i);
						from j := l until j > width loop
							--					for (j=l;j<width;j++)
							from s := 0.0 k := i until k > height loop
								s := s + l_u.item(k).item(i) * l_u.item(k).item(j)
								k := k + 1
							end
							--						for (s=0.0,k=i;k<width;k++) s += l_u[k][i]*l_u[k][j];
							f:=s/h;
							from k := i until k > height loop
								l_u.item(k).put(l_u.item(k).item(j) + f*l_u.item(k).item(i),j)
								k := k + 1
							end
							--						for (k=i;k<width;k++) l_u[k][j] += f*l_u[k][i];
							j := j + 1
						end
						from k := i until k > height loop
							l_u.item(k).put(l_u.item(k).item(i) * scale,i)
							k := k + 1
						end
						--					for (k=i;k<width;k++) l_u[k][i] *= scale;
					end -- if scale
				end--if
				l_w.put(scale * g,i)
				g := 0.0
				s := 0.0
				scale := 0.0
				--			g=s=scale=0.0;
				if (i <= height and i /= width) then
					from k := l until k > width loop scale := scale + fabs (l_u.item(i).item(k)) k := k + 1 end
					--				for (k=l;k<width;k++) scale += fabs(l_u[i][k]);
					if (scale /= 0) then
						from k := l until k > width loop
							l_u.item(i).put(l_u.item(i).item(k) / scale,k)
							s := s + l_u.item(i).item(k) * l_u.item(i).item(k)
							k := k + 1
						end -- end from
						f := l_u.item(i).item(l)
						g := -SIGN(sqrt(s),f);
						h:=(f*g)-s;
						l_u.item(i).put(f-g,l)
						from k := l until k > width loop l_rv.put(l_u.item(i).item(k)/ h,k) k := k + 1 end
						--					for (k=l;k<width;k++) l_rv[k]=l_u[i][k]/h;
						from j :=l until j > height loop
							--					for (j=l;j<width;j++)
							from s := 0 k := l until k > width loop s := s + l_u.item(j).item(k) * l_u.item(i).item(k) k := k + 1 end
							--						for (s=0.0,k=l;k<width;k++) s += l_u[j][k]*l_u[i][k];
							from k := l until k > width loop
								l_u.item(j).put(l_u.item(j).item(k) + s * l_rv.item(k),k)
								k := k + 1
							end
							--						for (k=l;k<width;k++) l_u[j][k] += s*l_rv[k];
							j := j + 1
						end -- end from
						from k := l until k > width loop
							l_u.item(i).put(l_u.item(i).item(k) * scale,k)
							k := k + 1
						end
						--					for (k=l;k<width;k++) l_u[i][k] *= scale;
					end -- if scale
				end -- if
				--			anorm=FMAX(anorm,(fabs(w[i])+fabs(l_rv[i])));
				anorm:=FMAX(anorm,(fabs(l_w.item(i))+fabs(l_rv.item(i))));
				i := i + 1
			end -- end from
			from i := width until i < 1 loop
				if (i < width ) then
					if (g /= 0) then
						from j := l until j > width loop
							l_v.item(j).put((l_u.item(i).item(j)/ l_u.item(i).item(l))/g,i)
							j := j + 1
						end
						from j := l until j > width loop
							from s := 0.0; k := l until k > width loop
								--								s := s + l_u.item().put(l_u.item(k).item(k,j) + f * l_u.item(k).item(i),j)
								s := s + l_u.item(i).item(k)*l_v.item(k).item(j)
								k := k + 1
							end
							from k := l until k > width loop
								l_v.item(k).put(l_v.item(k).item(j) + s * l_v.item(k).item(i),j)
								k := k + 1
							end
							j := j + 1
						end
					end
					from j := l until j > width loop
						l_v.item(i).put(0.0,j);
						l_v.item(j).put(0.0,i);
						j := j + 1
					end
				end -- end if
				l_v.item(i).put(1.0,i);
				g:=l_rv.item(i);
				l :=i;
				i := i - 1
			end
			from i := height.min (width) until i < 1 loop
				l:=i+1;
				g:=l_w.item(i);
				from j := l until j > width loop
					l_u.item(i).put(0.0,j)
					j := j + 1
				end
				if (g /= 0) then
					g:=1.0/g;
					from j := l until j > width loop
						from s := 0.0 k := l until k > height loop s := s + l_u.item(k).item(i) * l_u.item(k).item(j) k := k + 1 end
						f:=(s/l_u.item(i).item(i))*g;
						from k := i until k > height loop
							l_u.item(k).put(l_u.item(k).item(j) + f*l_u.item(k).item(i),j)
							k := k + 1
						end
						j := j + 1
					end
					from j := i until j > height loop
						l_u.item(j).put(l_u.item(j).item(i) * g,i)
						j := j + 1
					end
				else
					from j := i until j > height loop l_u.item(j).put(0.0,i) j := j + 1 end
				end
				l_u.item(i).put(l_u.item(i).item(i) + 1,i)
				i := i-1;
			end
			from k := width until k < 1 loop --0
				break1 := false
				from its := 1 until its > 30 or break1 loop
					flag:=true;
					break2 := false
					from l := k until l < 1 or break2 loop
						nm:=l-1;
						if ((fabs(l_rv.item(l))+anorm) = anorm) then
							flag:=false;
							break2 := true;
						end
						if (not break2) then
							if ((fabs(l_w.item(nm))+anorm) = anorm) then break2 := true; end
						end
						if (not break2) then
							l := l - 1
						end
					end-- for l
					if flag then
						c:=0.0;
						s:=1.0;
						break2 := false;
						from i:=l until i>k or break2 loop
							f:=s*l_rv.item(i);
							l_rv.put(c*l_rv.item(i),i)
							if ((fabs(f)+anorm) = anorm)then break2 := true;
							else
								g:=l_w.item(i);
								h:=pythag(f,g);
								l_w.put(h,i)
								h:=1.0/h;
								c:=g*h;
								s := -f*h;
								from j:=1 until j > height loop
									y:=l_u.item(j).item(nm);
									z:=l_u.item(j).item(i);
									l_u.item(j).put((y*c)+(z*s),nm)
									l_u.item(j).put((z*c)-(y*s),i)
									j := j + 1
								end
							end-- end if
							i := i + 1
						end-- for i
					end --if flag
					z:=l_w.item(k);
					if (l = k) then
						if (z < 0.0) then
							l_w.put(-z,k)
							from j := 1 until j > width loop l_v.item(j).put(-l_v.item(j).item(k),k) j := j + 1 end
						end
						break1 := true; -- its level
					end
					if (not break1) then
						if (its = 30) then raise("no convergence in 30 l_ucmp iterations"); end
						x:=l_w.item(l);
						nm:=k-1;
						y:=l_w.item(nm);
						g:=l_rv.item(nm);
						h:=l_rv.item(k);
						f:=((y-z)*(y+z)+(g-h)*(g+h))/(2.0*h*y);
						g:=pythag(f,1.0);
						f:=((x-z)*(x+z)+h*((y/(f+SIGN(g,f)))-h))/x;
						c:=1.0;
						s:=1.0;
						from j:=l until j> nm loop
							i:=j+1;
							g:=l_rv.item(i);
							y:=l_w.item(i);
							h:=s*g;
							g:=c*g;
							z:=pythag(f,h);
							l_rv.put(z,j)
							c:=f/z;
							s:=h/z;
							f:=x*c+g*s;
							g := g*c-x*s;
							h:=y*s;
							y := y *c;
							from jj := 1 until jj > width loop
								x:=l_v.item(jj).item(j);
								z:=l_v.item(jj).item(i);
								l_v.item(jj).put((x*c)+(z*s),j);
								l_v.item(jj).put((z*c)-(x*s),i);
								jj := jj + 1
							end--for jj
							z:=pythag(f,h);
							l_w.put(z,j)
							if (z /= 0) then
								z:=1.0/z;
								c:=f*z;
								s:=h*z;
							end-- end if
							f:=c*g+s*y;
							x:=c*y-s*g;
							from jj := 1 until jj > height loop
								y:=l_u.item(jj).item(j);
								z:=l_u.item(jj).item(i);
								l_u.item(jj).put((y*c)+(z*s),j);
								l_u.item(jj).put((z*c)-(y*s),i)
								jj := jj + 1
							end--end for jj
							j := j + 1
						end-- for j
						l_rv.put(0.0,l)
						l_rv.put(f,k)
						l_w.put(x,k)
					end-- if not break
					its := its + 1
				end-- for its
				k := k - 1
			end -- for k
		end

	covariance : SYMMETRIC_MATRIX is
			-- Covariance matrix
		local
			i, j, k: INTEGER
			l_sum: DOUBLE
			l_w: VECTOR
		do
			if covariance_saved = Void then
				create covariance_saved.make(width)
			end
			Result := covariance_saved
			create l_w.make(width)
			from i := 1 until i > width loop
--				l_w.put (0.0,i)
				l_w[i] := 0.0
				if w[i] /= 0.0  then
				--	l_w.put (1.0/(w.item(i)*w.item(i)),i)
					if w[i] > tolerance then
						l_w[i] := 1.0 / (w[i] * w[i])
					else
						l_w[i] := 0
					end
				end
				i := i + 1
			end
			from i := 1 until i > width loop
				from j := 1 until j > i loop
					from l_sum := 0.0 k := 1 until k > width loop
						l_sum := l_sum + v [i,k] * v [j,k] * l_w[k]
						k := k + 1
					end
				--	Result.put(l_sum,i,j)
					Result[i, j] := l_sum
--					Result.put(l_sum,j,i)
					Result[j, i] := l_sum
					j := j + 1
				end
				i := i + 1
			end
		end

	tolerance : DOUBLE
	set_tolerance (a_value : DOUBLE) is
		do
			tolerance := a_value
		end

	is_full_rank: BOOLEAN is
		local
			i: INTEGER
		do
			Result := true
			from i := 1 until i > w.count or not Result loop
				if (w.item (i) < tolerance) then
					Result := false
				end
			i := i + 1
			end
		end

	rank : INTEGER is
		local
			i: INTEGER
		do
			from i := 1 until i > w.count loop
				if (w.item (i) >= tolerance) then
					Result := Result + 1
				end
			i := i + 1
			end
		end

	null_set : ROSE_LINEAR_ARRAY[INTEGER] is
		local
			i: INTEGER
		do
			create Result.make_empty
			from i := 1 until i > w.count loop
				if (w.item (i) < tolerance) then
					Result.extend(i)
				end
			i := i + 1
			end
		end

	covariance_saved : SYMMETRIC_MATRIX

	fabs (d: DOUBLE): DOUBLE is
		do
			if d >=0 then
				Result := d
			else
				Result := -d
			end
		end


	SQR (d: DOUBLE): DOUBLE is
		do
			Result := d * d
		end

	SIGN(a, b: DOUBLE) : DOUBLE is
		do
			if (b >= 0 ) then
				Result := fabs (a)
			else
				Result := - fabs (a)
			end
		end

	FMAX (a, b: DOUBLE): DOUBLE is
		do
			if (a > b) then
				Result := a
			else
				Result := b
			end
		end

	pythag(a, b: DOUBLE): DOUBLE is
		local
			absa, absb: DOUBLE
		do
			absa:=fabs(a);
			absb:=fabs(b);
			if (absa > absb) then Result :=  absa*sqrt(1.0+SQR(absb/absa))
			else
				if (absb = 0 ) then
					Result := 0
				else
					Result :=  absb*sqrt(1.0+SQR(absa/absb));
				end
			end
		end

end -- class ROSE_SVD
