indexing

	description: "Objects that model cursors in oracle rdb"

	author: "Mark Howard"
	last_editor: "$Author: it $"
	date: "$Date: 2008/12/01 $"
	revision: "$Revision: #9 $"

deferred class	SQL_CURSOR

inherit

	TRACEABLE
		redefine
			trace
		end

feature -- Initialization

	make_read (a_data_server: like data_server; a_statement: like statement) is
			-- Initialize with read-only mode.
		require
			a_data_server_not_void: a_data_server /= Void
			a_statement_not_void: a_statement /= Void
		do
			set_mode_read
			make (a_data_server,a_statement)
		end

	make_insert (a_data_server: like data_server; a_statement: like statement) is
			-- Initialize with insertion mode.
		require
			a_data_server_not_void: a_data_server /= Void
			a_statement_not_void: a_statement /= Void
		do
			set_mode_insert
			make (a_data_server,a_statement)
		end

	make_update_only (a_data_server: like data_server; a_statement: like statement) is
			-- Initialize with only-updates mode.
		require
			a_data_server_not_void: a_data_server /= Void
			a_statement_not_void: a_statement /= Void		
		do
			set_mode_update_only
			make (a_data_server,a_statement)
		end

	make_update (a_data_server: like data_server; a_statement: like statement) is
			-- Initialize with update mode.
		require
			a_data_server_not_void: a_data_server /= Void
			a_statement_not_void: a_statement /= Void
		do
			set_mode_update
			make (a_data_server,a_statement)
		end

	make (a_data_server: like data_server; a_statement: like statement) is
			-- Initialize with default mode.
		require
			a_data_server_not_void: a_data_server /= Void
			a_statement_not_void: a_statement /= Void
		deferred
		end

feature -- Access

	data_server: SQL_DATA_SERVER
	statement: SQL_SELECTOR
	index: INTEGER

	is_open: BOOLEAN
	after: BOOLEAN

	before: BOOLEAN is
		do
			Result := index = 0
		end

	off: BOOLEAN is
		do
			Result := before or after
		end

	in_sqlda: SQL_SQLDA is
		do
			Result := statement.in_sqlda
		end

	out_sqlda: SQL_SQLDA is
		do
			Result := statement.out_sqlda
		end

feature -- Basic operations

	set_mode_read is deferred end
	set_mode_update is deferred end
	set_mode_insert is deferred end
	set_mode_update_only is deferred end

	open_with_array (a_arguments: ARRAY [ANY]) is
		do
			set_arguments (Void,a_arguments)
			open
		end

	open_with_object (a_object: ANY) is
		do
			set_arguments (a_object,<<>>)
			open
		end

	set_arguments (a_object: ANY; a_array: ARRAY [ANY]) is
		do
			if in_sqlda.count > 0 then
				in_sqlda.sqlvar.set_parameters (a_object,a_array)
			end
		end

	open is
		require
			is_closed: not is_open
		do
			after := False
			index := 0
			is_open := True
			data_server.open_cursor (Current)
		end

	fetch is
		do
			data_server.fetch (Current)
			after := data_server.eof
			index := index + 1
		end

	prefetch (a_count: INTEGER) is
		do
			data_server.prefetch (Current,a_count)
		end

	close is
		do
			is_open := False
			data_server.close_cursor (Current)
		end

	trace is
		do
			Precursor
			statement.trace
		end

invariant

	data_server_not_void: data_server /= Void
	statement_not_void: statement /= Void

end -- class SQL_CURSOR
