indexing
	description : "Object which facilitates communication between a database engine and eiffel"
	author : "Mark Howard"
	date: "$Date: 2009/08/03 $"
	revision: "$Revision: #11 $"

class
	SQL_FACILITATOR

inherit
	SQL_STATEMENT
		undefine
			clear_error
		redefine
			make, execute
		end
	PREDICATE_FACILITATOR
	SQL_SCRIPT_CAPTURE

feature {NONE} -- Initialization

	make (a_data_server: like data_server; a_statement: STRING) is
			-- creation
		do
			create predicate_statement.make (a_statement)
			Precursor (a_data_server,predicate_statement.parsed_string)
		end

feature -- Queries

	predicate_statement: PREDICATE_STATEMENT

feature -- Commands

	execute (a_object: ANY; a_args: ARRAY [ANY]) is
			-- Execute the statement
		do
			bind_execution_arguments( a_object, a_args )
			Precursor (a_object,a_args)
			if sql_script_capture_required then
				sql_capture_predicate_statement_data (predicate_statement, a_object, a_args, input)
			end
		end


	update_input_map (a_object: ANY) is
			-- setup the input map based on `a_object'
		do
			if input /= Void then
				input.update_input_map (a_object,binding)
			else
				(create {EXCEPTIONS}).raise ("No input in SQL_FACILITATOR, please check you SQL statement")
			end
		end

feature {NONE} -- Implementation

	bind_execution_arguments ( a_object: ANY; a_args: ARRAY [ANY]) is
			-- unified method for both SELECT and EXECUTE predicates to bind SQL execution input arguments to placeholders in the SQL statement
		do
			set_binding (predicate_statement.binding)
			if a_object /= Void then
				if global.internal.dynamic_type (a_object) /= input_type then
					update_input_map (a_object)
					input_type := global.internal.dynamic_type (a_object)
				end
			else --if a_args /= Void then
				if { l_data_structure: SQL_DATA_STRUCTURE } input then
					if l_data_structure.count > 0 and then l_data_structure.input_array_map = Void then
						update_input_map (a_args)
					end
				end
			end

		end

invariant
	predicate_statement_not_void: predicate_statement /= Void

end
