indexing
	description : "Objects for dealing with rows returned from sql queries"
	author : "Mark Howard"
	last_editor: "$Author: rj $"
	date: "$Date: 2009/09/28 $"
	revision: "$Revision: #31 $"
	archive: "$File: //rose/source/kernel/infrastructure/persistence/databases/sql/sql_selector.e $"

deferred class
	SQL_SELECTOR

inherit
	SQL_FACILITATOR
		redefine
			make
		end
	SELECTOR
		undefine
			release
		redefine
			fetch
		end

feature -- Initialization

	make (a_connection: like data_server;a_statement: STRING) is
		-- creation
		do
			Precursor (a_connection,a_statement)
			prefetch := 32767
		end

feature -- Access

	after: BOOLEAN is
			-- are we off the end of the result set?
		do
			Result := cursor.after
		end

	get_integer: INTEGER is
			-- the integer value for this item
		do
			Result := output.item (1).integer_value
		end

	get_double: DOUBLE is
			-- the double value for this item
		do
			Result := output.item (1).double_value
		end

	get_real: REAL is
			-- the real value for this item
		do
			Result := output.item (1).real_value
		end

	get_boolean: BOOLEAN  is
			-- the boolean value for this item
		do
			Result := output.item (1).boolean_value
		end

	get_character: CHARACTER is
			-- the character value for this item
		do
			Result := output.item (1).character_value
		end

	get_string: STRING is
		-- the string value for this item
		do
			Result := output.item (1).string_value
		end

	get_date_time: ROSE_DATE is
		-- the date_time value for this item
		do
			Result := output.item (1).date_time_value
		end

feature -- Basic operations

	open (a_object: ANY; a_args: ARRAY [ANY]) is
			-- open the cursor for the select using the values in `a_object' and `a_args'
		do
			bind_execution_arguments( a_object, a_args )
			cursor.set_arguments (a_object,a_args)	-- NOTE: if Cursor is Void, check your predicate text for an error
			open_selection
		rescue
			if statement /= Void then
				print (statement + "%N")
			end
		end

	close is
			-- close the cursor for the select
		do
			cursor.close
			success := data_server.success
		end

	fetch (a_vessel: ANY) is
			-- fetch the next record in the results set and fill `a_vessel' with the values
		do
			if prefetch > 1 and then cursor.index \\ prefetch = 0 then
				cursor.prefetch (prefetch)
				success := cursor.success
			else
				clear_error
			end
			if success then
				fetch_next
				if a_vessel /= Void and then not after then
					output.get_values (a_vessel)
				end
			end
		end

feature -- Restricted export {ORACLERDB_LIST_SQLVAR}

	fetch_next is
			-- fetch the next record in the results set
		do
			cursor.fetch
			success := cursor.success
		end

	cursor: SQL_CURSOR is
			-- cursor for the result set
		deferred
		end

	open_selection is
		-- open the cursor for the select
		do
			cursor.open
			success := cursor.success
		end

end -- class SQL_SELECTOR
