indexing
	description: "One row of 'select * from master..sysprocesses'."
	author: "RJ"
	last_editor: "$Author: rj $"
	date: "$Date: 2014/05/16 $"
	revision: "$Revision: #10 $"
	path: "$File: //rose/source/applications/system/data_connection_viewer/sql_server_sysprocess.e $"

class
	SQL_SERVER_SYSPROCESS

inherit
	ROSE_FORMATTING_SERVICE

	DEBUG_OUTPUT

feature	-- Columns from master..sysprocesses.  Can't really handle the BINARY types.
	spid: INTEGER										-- SQL Server session ID.

	kpid: INTEGER										-- Microsoft Windows thread ID.

	blocked: INTEGER									-- ID of the session that is blocking the request.

	waittype: INTEGER_16								-- The waittype field is a reserved internal binary column. The value in the waittype field indicates the type of resource that the connection is waiting on.

	waittime: INTEGER_64								-- Current wait time in milliseconds.

	lastwaittype: STRING								-- A string indicating the name of the last or current wait type.

	waitresource: STRING								-- Textual representation of a lock resource.

	dbid: INTEGER										-- ID of the database currently being used by the process.

	uid: INTEGER										-- ID of the user that executed the command.

	cpu: INTEGER										-- Cumulative CPU time for the process.

	physical_io: INTEGER_64								-- Cumulative disk reads and writes for the process.

	memusage: INTEGER									-- Number of pages in the procedure cache that are currently allocated to this process.

	login_time: ROSE_DATE								-- Time at which a client process logged into the server.

	last_batch: ROSE_DATE								-- Last time a client process executed a remote stored procedure call or an EXECUTE statement.

	ecid: INTEGER										-- Execution context ID used to uniquely identify the subthreads operating on behalf of a single process.

	open_tran: INTEGER									-- Number of open transactions for the process.

	status: STRING										-- Process ID status.

--	sid: BINARY											-- Globally unique identifier (GUID) for the user.

	hostname: STRING									-- Name of the workstation.

	program_name: STRING								-- Name of the application program.

	hostprocess: STRING									-- Workstation process ID number.

	cmd: STRING											-- Command currently being executed.

	nt_domain: STRING									-- Microsoft Windows domain for the client, if using Windows Authentication, or a trusted connection.

	nt_username: STRING									-- Windows user name for the process, if using Windows Authentication, or a trusted connection.

	net_address: STRING									-- Assigned unique identifier for the network adapter on the workstation of each user.

	net_library: STRING									-- Column in which the client's network library is stored.

	loginame: STRING									-- Login name.

--	context_info: BINARY								-- Data stored in a batch by using the SET CONTEXT_INFO statement.

--	sql_handle: BINARY									-- Represents the currently executing batch or object.

	stmt_start: INTEGER									-- Starting offset of the current SQL statement for the specified sql_handle.

	stmt_end: INTEGER									-- Ending offset of the current SQL statement for the specified sql_handle.

feature	-- Interface.
	sysprocesses: SQL_SERVER_SYSPROCESSES				-- The container of this object and it's brethren.  The collection of all rows from
														-- 'select * from master..sysprocesses'.

	set_sysprocesses(a_sysprocesses: SQL_SERVER_SYSPROCESSES)
	do
		sysprocesses := a_sysprocesses
	end

	refresh
	-- Do a 'select * from master..sysprocesses where spid=?'.
	require
		set_sysprocesses_called: sysprocesses/=Void
	do
		sysprocesses.refresh_single(Current)
	end

	mark_as_dead is
	-- Sometimes the user will ask to refresh a row that has since disappeared and we must mark it as dead.
	require
		is_ready: lastwaittype/=Void
	do
		if not lastwaittype.has_substring("DEAD-")
		then
			lastwaittype.prepend("DEAD-")
			status.prepend("DEAD-")
			hostname.prepend("DEAD-")
			program_name.prepend("DEAD-")
			hostprocess.prepend("DEAD-")
			cmd.prepend("DEAD-")
			nt_domain.prepend("DEAD-")
			nt_username.prepend("DEAD-")
			net_address.prepend("DEAD-")
			net_library.prepend("DEAD-")
			loginame.prepend("DEAD-")
		end
	end

	elapsed_time_days_mmssfff_from_milliseconds(a_milliseconds: DOUBLE): STRING
	-- Returns the time in days+hh:mm:ss.fff.
	local
		l_end_time: ROSE_DATE
		l_dur: DATE_TIME_DURATION
		l_days: INTEGER
		l_hours: INTEGER
		l_minutes: INTEGER
		l_seconds: DOUBLE
	do
		l_seconds := a_milliseconds / 1000.0
		l_minutes := (l_seconds / 60.0).truncated_to_integer
		l_seconds := l_seconds - l_minutes * 60.0
		l_hours := l_minutes // 60
		l_minutes := l_minutes - l_hours * 60
		l_days := l_hours // 24
		l_hours := l_hours - l_days * 24
		create l_dur.make_fine(0, 0, l_days, l_hours, l_minutes, l_seconds)
		l_end_time := base_time.twin
		l_end_time.add(l_dur)
		l_dur.set_origin_date_time(base_time)

		Result := elapsed_time_mmssfff_from_duration(l_dur, Void)

		-- Result can have leading minutes even if 0.
		-- Strip off leading '0' and ':' until neither is found or the length is 5 (0.000).
		from
			-- Nothing here.
		until
			Result.count <= 5 or else (Result[1] /= '0' and Result[1] /= ':')
		loop
			Result := Result.substring(2, Result.count)
		end
	end

	string_array: ROSE_LINEAR_ARRAY[TUPLE[text: STRING; comparable: COMPARABLE]]
	-- Each element is the string representation of the corresponding column from master..sysprocesses.
	local
		l_cpu: DOUBLE
		l_cpu_text: STRING
		l_parent: like Current
	do
		-- To make a change you must change this statement here and
		-- two in RCMD_DATA_CONNECTION_VIEWER
		-- and in SERVER_REQUEST_TREE_NODE:should_right_justify_child.
		create Result.make_empty
		Result.extend([sysprocesses.name, Void])
		Result.extend([sysprocesses.dbname(dbid), Void])
		Result.extend([hostname, Void])
		Result.extend([loginame, Void])
		Result.extend([program_name, Void])

		if cpu = maximum_integer_32
		then
			-- For some undocumented reason, the cpu time sometimes shows up as 2147483647.  Seems to be when ecid is not zero.
			l_cpu_text := "?"

			if ecid /= 0
			then
				l_parent := sysprocesses.parent_by_spid[spid]
				if l_parent /= Void
				then
					l_cpu := l_parent.cpu - (0.001 * ecid)	-- This will sort just below the parent for ecid = 1 then 2, etc.
				else
					l_cpu := -1.0
				end
			else
				l_cpu := -1.0								-- This will cause Current to sort below 0.  Using 0 causes this one to sort betwixt the real 0's.
			end
		else
			l_cpu := cpu.to_double
			l_cpu_text := elapsed_time_days_mmssfff_from_milliseconds(l_cpu)
		end

		Result.extend([l_cpu_text, l_cpu])
		Result.extend([cmd, Void])
		Result.extend([spid.out, spid])
		Result.extend([blocked.out, blocked])
		Result.extend([elapsed_time_days_mmssfff_from_milliseconds(waittime.to_double), waittime])

		if waittype=0
		then
			Result.extend(["(" + lastwaittype.out + ")", Void])
		else
			Result.extend([lastwaittype.out, Void])
		end

		waitresource.prune_all_trailing(' ')
		Result.extend([waitresource, Void])
		Result.extend([login_time.formatted_out(date_time_format), Void])
		Result.extend([last_batch.formatted_out(date_time_format), Void])

		Result.extend([kpid.out, kpid])
		Result.extend([uid.out, uid])
		Result.extend([physical_io.out, physical_io])
		Result.extend([integer_formatter.formatted(memusage), Void])
		Result.extend([ecid.out, ecid])
		Result.extend([open_tran.out, open_tran])
		Result.extend([status.out, Void])
--		Result.extend([sid.out, Void])

		if hostprocess.is_integer
		then
			Result.extend([hostprocess.out, hostprocess.to_integer])
		else
			Result.extend([hostprocess.out, Void])
		end

		Result.extend([nt_domain.out, Void])
		Result.extend([nt_username.out, Void])
		Result.extend([net_address.out, Void])
		Result.extend([net_library.out, Void])
--		Result.extend([context_info.out, Void])
--		Result.extend([sql_handle.out, Void])
		Result.extend([stmt_start.out, stmt_start])
		Result.extend([stmt_end.out, stmt_end])
	end

	is_interesting_blocker: BOOLEAN is
	-- See comment in SQL_SERVER_SYSPROCESSES.
	do
		Result := sysprocesses.is_interesting_blocker(Current)
	end

	set_is_blocker
	-- Indicate that Current is blocking another connection.
	do
		is_blocker := True
	end

	set_program_name(a_program_name: STRING) is
	do
		program_name := a_program_name
	end

	is_blockee: BOOLEAN is
	do
		Result := blocked/=0 and blocked/=spid and not is_blocker
	end

	is_blocker: BOOLEAN

feature {SQL_SERVER_SYSPROCESSES} -- Interface.
	prune_trailing_spaces is
	-- Prunes trailing spaces from certain fields that always contain trailing spaces.
	do
		lastwaittype.prune_all_trailing(' ')
		status.prune_all_trailing(' ')
		hostname.prune_all_trailing(' ')
		program_name.prune_all_trailing(' ')
		hostprocess.prune_all_trailing(' ')
		cmd.prune_all_trailing(' ')
		nt_domain.prune_all_trailing(' ')
		nt_username.prune_all_trailing(' ')
		net_address.prune_all_trailing(' ')
		net_library.prune_all_trailing(' ')
		loginame.prune_all_trailing(' ')
	end

feature {NONE} -- Implementation.
	debug_output: STRING is
	do
		if sysprocesses/=Void and then sysprocesses.name/=Void
		then
			Result := sysprocesses.name.twin
		else
			Result := ""
		end

		Result.append(spid.out)
	end

	double_formatter: FORMAT_DOUBLE is
	-- Formatter for DOUBLE's.
	once
		create Result.make(4, 3)
		Result.set_separator(',')
	end

	integer_formatter: FORMAT_INTEGER is
	-- Formatter for INTEGER's.
	once
		create Result.make(1)
		Result.set_separator(',')
	end

	date_time_format: STRING = "YYYY/[0]MM/[0]DD [0]HH:[0]MI:[0]SS"

	maximum_integer_32: INTEGER
	-- The max integer.
	once
		Result := (create {INTEGER_32_REF}).max_value
	end

	base_time: ROSE_DATE
	-- A time needed by DATE_TIME_DURATION.  Any time would do.
	once
		create Result.make_today
	end

end
