indexing
	description : "Proxy for the sqlda structure used with prepared statements in dynamic sql"
	author : "Mark Howard"
	date: "$Date: 2008/12/01 $"
	revision: "$Revision: #17 $"

class
	SQL_SQLDA

inherit
	TRACEABLE
		redefine
			trace
		end

feature -- Initialization

	make (a_statement: like statement; a_sqlda: POINTER) is
		do
			statement := a_statement
			id := a_sqlda
			data_server.setup_sqlda (Current)
		end

feature -- Access

	has_list_varbyte: BOOLEAN

	sqld: INTEGER is
		do
		end

	sqlvar: SQL_DATA_STRUCTURE

	data_server: SQL_DATA_SERVER is
		do
			Result := statement.data_server
		end

	statement: SQL_STATEMENT

	count: INTEGER
			-- Number of columns in result.

	map_table: ARRAY [INTEGER]

feature -- Basic operations

	set_has_list_varbyte (a_value: BOOLEAN) is
		do
			has_list_varbyte := True
		end

	set_value (a_index: INTEGER; a_arg: ANY) is
		do
			sqlvar.item (a_index).set_value (a_arg)
		end

	set_parameters (a_object: ANY; a_array: ARRAY [ANY]) is
		do
			sqlvar.set_parameters (a_object,a_array)
		end

	execute (a_object: ANY; a_array: ARRAY [ANY]) is
		do
			sqlvar.execute (a_object,a_array)
		rescue
			if statement /= Void and then statement.statement /= Void then
				print (statement.statement + "%N")
			end
		end

	set_field (a_index: INTEGER; a_object: ANY; a_value: ANY) is
		local
			l_field, l_value: STRING
		do
			l_field ?= internal.field (a_index,a_object)
			l_value ?= a_value
			if l_field /= Void and l_value /= Void then
				l_field.copy (l_value)
			else
				print ("Not copied%N")
			end
		end

	display_values is
		do
			if sqlvar /= Void then
				sqlvar.display_values
			end
		end

	to_arrays (a_containers: ARRAY [ANY]; a_index: INTEGER) is
		local
			l_field: INTEGER
			l_integer_ref: INTEGER_REF
			l_real_ref: REAL_REF
			l_double_ref: DOUBLE_REF
			l_string: STRING
			l_value: ANY
			l_integer_array: ARRAY [INTEGER]
			l_real_array: ARRAY [REAL]
			l_double_array: ARRAY [DOUBLE]
			l_string_array: ARRAY [STRING]
		do
			from
				l_field := 1
			until
				l_field > a_containers.count
			loop
				l_value := sqlvar.item (l_field).value
				l_integer_ref ?= l_value
				l_real_ref ?= l_value
				l_double_ref ?= l_value
				l_string ?= l_value
				if l_integer_ref /= Void then
					l_integer_array ?= a_containers.item (l_field)
					l_integer_array.force (l_integer_ref.item,a_index)
				elseif l_real_ref /= Void then
					l_real_array ?= a_containers.item (l_field)
					l_real_array.force (l_real_ref.item,a_index)
				elseif l_double_ref /= Void then
					l_double_array ?= a_containers.item (l_field)
					l_double_array.force (l_double_ref.item,a_index)
				elseif l_string /= Void then
					l_string_array ?= a_containers.item (l_field)
					l_string_array.force (l_string,a_index)
				end
				l_field := l_field + 1
			end
		end

	to_containers (a_containers: ARRAY [ANY]; a_start: INTEGER) is
		local
			l_temp_array: ARRAY [ANY]
			l_temp_list: LIST [ANY]
			l_field: INTEGER
			l_integer_ref: INTEGER_REF
			l_real_ref: REAL_REF
			l_double_ref: DOUBLE_REF
			l_string: STRING
			l_value: ANY
			l_integer_array: ARRAY [INTEGER]
			l_real_array: ARRAY [REAL]
			l_double_array: ARRAY [DOUBLE]
			l_string_array: ARRAY [STRING]
			l_integer_list: LIST [INTEGER]
			l_real_list: LIST [REAL]
			l_double_list: LIST [DOUBLE]
			l_string_list: LIST [STRING]
			l_counter: INTEGER
		do
			l_counter := a_start
			from
				l_field := 1
			until
				l_field > a_containers.count
			loop
				l_value := sqlvar.item (l_field).value
				if l_value = Void then
					l_temp_array ?= a_containers.item (l_field)
					l_temp_list ?= a_containers.item (l_field)
					if l_temp_array /= Void then
						l_temp_array.force (Void,l_counter)
					elseif l_temp_list /= Void then
						l_temp_list.extend (Void)
					end
				end
				l_integer_ref ?= l_value
				l_real_ref ?= l_value
				l_double_ref ?= l_value
				l_string ?= l_value
				if l_integer_ref /= Void then
					l_integer_array ?= a_containers.item (l_field)
					l_integer_list ?= a_containers.item (l_field)
					if l_integer_array /= Void then
						l_integer_array.force (l_integer_ref.item,l_counter)
					elseif l_integer_list /= Void then
						l_integer_list.extend (l_integer_ref.item)
					end
				elseif l_real_ref /= Void then
					l_real_array ?= a_containers.item (l_field)
					l_real_list ?= a_containers.item (l_field)
					if l_real_array /= Void then
						l_real_array.force (l_real_ref.item,l_counter)
					elseif l_real_list /= Void then
						l_real_list.extend (l_real_ref.item)
					end
				elseif l_double_ref /= Void then
					l_double_array ?= a_containers.item (l_field)
					l_double_list ?= a_containers.item (l_field)
					if l_double_array /= Void then
						l_double_array.force (l_double_ref.item,l_counter)
					elseif l_double_list /= Void then
						l_double_list.extend (l_double_ref.item)
					end
				elseif l_string /= Void then
					l_string_array ?= a_containers.item (l_field)
					l_string_list ?= a_containers.item (l_field)
					if l_string_array /= Void then
						l_string_array.force (l_string,l_counter)
					elseif l_string_list /= Void then
						l_string_list.extend (l_string)
					end
				end
				l_field := l_field + 1
			end
			l_counter := l_counter + 1
		end

	to_array (a_container: ARRAY [ANY]) is
		local
			l_field: INTEGER
		do
			from
				l_field := 1
			until
				l_field > a_container.count or else l_field > sqlvar.count
			loop
				a_container.force (sqlvar.item (l_field).value,l_field)
				l_field := l_field + 1
			end
		end

	get_values (a_object: ANY) is
			-- Store values in the appropriate fields of obj, ignoring null values.
		local
			l_value: ANY
			l_index: INTEGER
			l_string_target: STRING
			l_int_target: INTEGER_REF
			l_real_target: REAL_REF
			l_double_target: DOUBLE_REF
		do
			if map_table = Void then
				update_map_table (a_object)
			end
			from
				l_index := 1
			until
				l_index > count
			loop
				if map_table.item (l_index) > 0 and map_table.item (l_index) <= count then
					l_value := internal.field (map_table.item (l_index),a_object)
					l_int_target ?= l_value
					l_real_target ?= l_value
					l_double_target ?= l_value
					l_string_target ?= l_value
					if l_int_target /= Void then
						l_int_target.set_item (sqlvar.item (l_index).integer_value.item)
					elseif l_real_target /= Void then
						l_real_target.set_item (sqlvar.item (l_index).real_value.item)
					elseif l_double_target /= Void then
						l_double_target.set_item (sqlvar.item (l_index).double_value.item)
					elseif l_string_target /= Void then
						l_string_target.copy (sqlvar.item (l_index).string_value)
					end
				end
				l_index := l_index + 1
			end
		end

	set_values (a_object: ANY) is
		local
			l_index: INTEGER
		do
			if map_table = Void then
				update_map_table (a_object)
			end
			from
				l_index := 1
			until
				l_index > count
			loop
				sqlvar.item (l_index).set_value (internal.field (map_table.item (l_index),a_object))
				l_index := l_index + 1
			end
		end

	update_map_table (a_object: ANY) is
		-- Update map table according to field names.
		require else
			object_exists: a_object /= Void
		local
			i, j, n: INTEGER
			l_searched_name: STRING
		do
			n := internal.field_count (a_object)
			if map_table = Void then
				create map_table.make (1,count)
			elseif map_table.count < count then
				map_table.resize (1,count)
			end
			from
				i := 1
			until
				i > count
			loop
				l_searched_name := clone (sqlvar.item (i).sqlname)
				l_searched_name.to_lower
				from
					j := 1
				until
					j > n or else internal.field_name (j, a_object).is_equal (l_searched_name)
				loop
					j := j + 1
				end
				if j <= n then
					map_table.put (j, i)
				else
					--db_status.set(200)
					i := count
				end
				i := i + 1
			end
		end

	trace is
		do
			Precursor
			display_values
		end

feature {SQL_DATA_SERVER,SQL_SQLVAR}

	set_data_structure (a_data_structure: SQL_DATA_STRUCTURE) is
		do
			sqlvar := a_data_structure
			count := sqlvar.count
		end

	id: POINTER

feature {NONE}

	internal: INTERNAL is
		once
			create Result
		end

end --SQL_SQLDA
