indexing

	description: "[
				  Object representing a prepared sql statement in the context of a 
				  particular connection to a database.
				  ]"
    author: "Mark Howard"
	last_editor: "$Author: rj $"
	date: "$Date: 2012/02/02 $"
	revision: "$Revision: #28 $"
	path: "$File: //rose/source/kernel/infrastructure/persistence/databases/sql/sql_statement.e $"
	last_review: "$Review: $"

class

	SQL_STATEMENT

inherit

	TRACEABLE
		redefine
			trace
		end
	ROSE_SHARED_GLOBAL
		rename
			trace as shared_trace,
			output as shared_output
		end

create

    make

feature {NONE} -- Initialization

	make (a_data_server: like data_server; a_statement: STRING) is
			-- creation
		require
			valid_data_server: a_data_server /= Void and then a_data_server.make_succeeded
			valid_statement: a_statement /= Void
		do
			statement := a_statement
			data_server := a_data_server
			create in_sqlda
			create out_sqlda
			data_server.prepare (Current)
		end

feature -- Access

	statement: STRING
			-- Predicate statement
	data_server: SQL_DATA_SERVER
			-- Data server object that was used at creation
	id: INTEGER
			-- SQL statement id as determined by the provider

feature -- Data descriptors

	in_sqlda: SQL_SQLDA
			-- Input SQL data structure
	out_sqlda: SQL_SQLDA
			-- Output SQL data structure

	input: DATA_DESCRIPTORS [DATA_DESCRIPTOR] is
			-- Data descriptors used as input for the statement
		do
			if in_sqlda /= Void then
				Result := in_sqlda.sqlvar
			end
		end

	output: DATA_DESCRIPTORS [DATA_DESCRIPTOR] is
			-- Output descriptors used as result for the query
		do
			if out_sqlda /= Void then
				Result := out_sqlda.sqlvar
			end
		end

feature -- Basic operations

	set_id (a_id: INTEGER) is
			-- Set the statement id as determined by the provider
		do
			id := a_id
		end

	execute (a_object: ANY; args: ARRAY [ANY]) is
			-- Execute this statement with these arguments
		require
			valid_server: data_server /= Void
		do
			data_server.execute (Current,a_object,args)
		end

	trace is
			-- Print a status message for this statement object
		local
			l_message: STRING
		do
			Precursor
			l_message := "     Timestamp      : " + Global.interpreted ("$now") + "%N"
			if data_server.node /= Void and data_server.database /= Void then
				l_message.append ("     Database       : " + data_server.node + "." + data_server.database + "%N")
			end

			-- In the following, 22 represents the number of characters up to and including the ": ".
			l_message.append ("     Statement      : " + global.formatter.indented(statement, 22, False))

			if l_message.item(l_message.count)/='%N'
			then
				l_message.append_character('%N')
			end

			inform ("trace","%N" + l_message,Void)
			debug("if_no_inform")
				l_message.prepend (generating_type + ".trace%N")
				print (l_message)
			end
			if in_sqlda /= Void and then input /= Void and then in_sqlda.count > 0 then
				print ("     Input SQLDA    : Name                    Type            Value%N")
				in_sqlda.trace
			end
			if out_sqlda /= Void and then output /= Void and then out_sqlda.count > 0 then
				print ("     Output SQLDA   : Name                    Type            Value%N")
				out_sqlda.trace
			end
		end

feature {NONE} -- Implementation

	release is
			-- Action to be executed just before garbage collection
			-- reclaims an object.
		do
			if id /= 0 and then data_server /= Void then
				data_server.release_statement (Current)
				id := 0
			end
		end

end
