note
	description: "OLEDB provider for SQL server"
	last_updater: "$Author: rj $"
	author: "Pierre Nallet"
	last_editor: "$Author: rj $"
	date: "$Date: 2014/09/09 $"
	revision: "$Revision: #81 $"
	path: "$File: //rose/source/kernel/infrastructure/persistence/windows/classic/sqloledb_data_server.e $"

class
	SQLOLEDB_DATA_SERVER

inherit
	OLEDB_ANY_DATA_SERVER
		redefine
			fast_loader,
			fast_loader_with_options,
			fast_loader_keep_identity,
			stream_command,
			simple_command,
			metrics,
			reset_metrics,
			trace,
			version,
			make_with_server_and_database,
			sql_disconnect
		end
	SQLOLEDB_PROPERTY_CONSTANTS
	COM_CLSCTX

create
	make,
	make_no_transaction,
	make_read_only_integrated_security,
	make_integrated_security,
	make_integrated_security_with_timeout,
	make_integrated_security_fast_load,
	make_fast_load,
	make_user_name_password,
	make_with_property_set

feature {NONE} -- Creation

	make_with_property_set (ps: ARRAY [OLEDB_PROPERTY_SET]) is
			-- Make with property set 'ps'.
		require
			ps_not_void: ps /= Void
		do
			if not playback_movie then
				open_with_service_components (class_id, ps, Void)
			end
		end

	make_user_name_password (a_server_name, a_data_base_name, a_user_name, a_password: STRING) is
			-- Creation procedure.
		require
			server_not_void: a_server_name /= Void
			db_name_not_void: a_data_base_name /= Void
		local
			l_ps1, l_ps2: OLEDB_PROPERTY_SET
		do
			create l_ps1.make (constants.DBPROPSET_DBINIT)
			if a_server_name /= Void then
				l_ps1.add_bstr_property (constants.DBPROP_INIT_DATASOURCE, a_server_name)
			end
			if a_data_base_name /= Void then
				l_ps1.add_bstr_property (constants.DBPROP_INIT_CATALOG, a_data_base_name)
			end
			if a_user_name /= Void then
				l_ps1.add_bstr_property (constants.DBPROP_AUTH_USERID, a_user_name)
			end
			if a_password /= Void then
				l_ps1.add_bstr_property (constants.DBPROP_AUTH_PASSWORD, a_password)
			end
			l_ps1.add_integer_property (constants.DBPROP_INIT_OLEDBSERVICES, -1)
			l_ps1.add_integer_property (constants.DBPROP_INIT_TIMEOUT, 100)
				-- Should fix timeout on connect.
			create l_ps2.make (DBPROPSET_SQLSERVERDBINIT)
			l_ps2.add_bstr_property (SSPROP_INIT_APPNAME, global.app.program_path)
			database := a_data_base_name
			node := a_server_name
			open_with_service_components (class_id, <<l_ps1, l_ps2>>, Void)
			is_fast_load := false
		end

	old_make (node_: STRING; name_: STRING; passwd_: STRING; database_: STRING; tag_: STRING) is
		obsolete
			"use SQLOLEDB_DATA_SERVER.make or MSDASQL_DATA_SERVER.make instead"
		do
			make_no_transaction (node_, name_, passwd_)
		end

	make_with_server_and_database (a_server_name, a_database_name: STRING) is
			-- <Precursor>
		do
			make (a_server_name, a_database_name)
		end

	make (a_server_name, a_data_base_name: STRING) is
			-- <Precursor>
		require
			server_not_void: a_server_name /= Void
			db_name_not_void: a_data_base_name /= Void
		local
			l_retry: BOOLEAN
		do
			if not l_retry then
				make_integrated_security (a_server_name, a_data_base_name)
				is_fast_load := false
			else
				error.raise (a_server_name + "." + a_data_base_name)
			end
		rescue
			l_retry := True
			retry
		end

	make_read_only_integrated_security (a_server_name, a_data_base_name: STRING) is
			-- This was an attempt to make a read-only connection.
			-- It was a dismal failure.  It's presence is to serve as a reminder to others.
		require
			names_valid: a_server_name /= Void and a_data_base_name /= Void
		local
			l_ps1, l_ps2: OLEDB_PROPERTY_SET
			l_ps3: OLEDB_PROPERTY_SET
		do
			database := a_data_base_name
			node := a_server_name
			create l_ps1.make (constants.DBPROPSET_DBINIT)
			if a_server_name /= Void then
				l_ps1.add_bstr_property (constants.DBPROP_INIT_DATASOURCE, a_server_name)
			end
			if a_data_base_name /= Void then
				l_ps1.add_bstr_property (constants.DBPROP_INIT_CATALOG, a_data_base_name)
			end
			l_ps1.add_bstr_property (constants.DBPROP_AUTH_INTEGRATED, "SSPI")
			l_ps1.add_integer_property (constants.DBPROP_INIT_OLEDBSERVICES, -1)
			if a_server_name /= Void and a_data_base_name /= Void then
				l_ps1.add_integer_property (constants.DBPROP_INIT_TIMEOUT, 100)
					-- Should fix timeout on connect.
			else
				l_ps1.add_integer_property (constants.DBPROP_INIT_TIMEOUT, 1)
			end
			create l_ps2.make (DBPROPSET_SQLSERVERDBINIT)
			l_ps2.add_bstr_property (SSPROP_INIT_APPNAME, global.app.program_path)
			create l_ps3.make (constants.dbpropset_datasourceinfo)
			l_ps3.add_boolean_property (constants.DBPROP_DATASOURCEREADONLY, True)
			open_with_service_components (class_id, <<l_ps1, l_ps2>>, <<l_ps3>>)
			if status >= 0 then
			--	set_implicit_transaction_on		-- This is now deferred until we have determined that the user wants to write.
				is_fast_load := false
			else
				trace
			end
		end

	make_integrated_security_with_timeout (a_server_name, a_data_base_name: STRING; a_timeout: INTEGER)
			-- Same as 'make_integrated_security' except that it will timeout in 'a_timeout' (in seconds).
		require
			names_valid: a_server_name /= Void and a_data_base_name /= Void
		local
			l_ps1, l_ps2: OLEDB_PROPERTY_SET
		do
			database := a_data_base_name
			node := a_server_name
			create l_ps1.make (constants.DBPROPSET_DBINIT)
			if a_server_name /= Void then
				l_ps1.add_bstr_property (constants.DBPROP_INIT_DATASOURCE, a_server_name)
			end
			if a_data_base_name /= Void then
				l_ps1.add_bstr_property (constants.DBPROP_INIT_CATALOG, a_data_base_name)
			end
			l_ps1.add_bstr_property (constants.DBPROP_AUTH_INTEGRATED, "SSPI")
			l_ps1.add_integer_property (constants.DBPROP_INIT_OLEDBSERVICES, -1)
			l_ps1.add_integer_property (constants.DBPROP_INIT_TIMEOUT, a_timeout)
			l_ps1.add_integer_property (constants.DBPROP_INIT_GENERALTIMEOUT, a_timeout)
			create l_ps2.make (DBPROPSET_SQLSERVERDBINIT)
			l_ps2.add_bstr_property (SSPROP_INIT_APPNAME, global.app.program_path)
			open_with_service_components (class_id, <<l_ps1, l_ps2>>, Void)
			if status >= 0 then
			--	set_implicit_transaction_on		-- This is now deferred until we have determined that the user wants to write.
				is_fast_load := false
			else
				trace
			end
		end

	make_integrated_security_fast_load (a_server_name, a_data_base_name: STRING) is
		require
			names_valid: a_server_name /= Void and a_data_base_name /= Void
		local
			l_ps1, l_ps2: OLEDB_PROPERTY_SET
			l_data_source: OLEDB_DATA_SOURCE
			hr: INTEGER
		do
			success := True
			database := a_data_base_name
			node := a_server_name
			create l_ps1.make (constants.DBPROPSET_DBINIT)
			if a_server_name /= Void then
				l_ps1.add_bstr_property (constants.DBPROP_INIT_DATASOURCE, a_server_name)
			end
			if a_data_base_name /= Void then
				l_ps1.add_bstr_property (constants.DBPROP_INIT_CATALOG, a_data_base_name)
			end
			l_ps1.add_bstr_property (constants.DBPROP_AUTH_INTEGRATED, "SSPI")
			l_ps1.add_integer_property (constants.DBPROP_INIT_TIMEOUT, 100)
				-- Should fix timeout on connect.
			create l_ps2.make (DBPROPSET_SQLSERVERDBINIT)
			l_ps2.add_bstr_property (SSPROP_INIT_APPNAME, global.app.program_path)
			debug ("make_movies")
				create cam.make (generating_type + "/" + node + "/" + database)
			end
			if playback_movie then
				make_succeeded := True
					-- If it didn't succeed then why bother with a movie?
			else
				hr := CoInitialize (default_pointer)
				create l_data_source.open (class_id, <<l_ps1, l_ps2>>)
				create l_ps1.make (DBPROPSET_SQLSERVERDATASOURCE)
				l_ps1.add_boolean_property (SSPROP_ENABLEFASTLOAD, True)
				hr := l_data_source.set_properties (<<l_ps1>>)
				create session.make
				hr := session.open (l_data_source)
				if session.p /= default_pointer then
					sql_error (Current)
					make_succeeded := (hr >= 0)
					if make_succeeded then
						set_implicit_transaction_off
					else
						if error_handler /= Void then
							error_handler.on_error (status, Void, "Unable to connect the server")
						end
						session := Void
					end
				end
			end
			is_fast_load := True
		end

	make_no_transaction (a_providerstring: STRING; a_user_name: STRING; password: STRING) is
		local
			l_ps1, l_ps2: OLEDB_PROPERTY_SET
		do
			create l_ps1.make (constants.DBPROPSET_DBINIT)
			if a_providerstring /= Void then
				l_ps1.add_bstr_property (constants.DBPROP_INIT_PROVIDERSTRING, a_providerstring)
			end
			if a_user_name /= Void then
				l_ps1.add_bstr_property (constants.DBPROP_AUTH_USERID, a_user_name)
			end
			if password /= Void then
				l_ps1.add_bstr_property (constants.DBPROP_AUTH_PASSWORD, password)
			end
			l_ps1.add_integer_property (constants.DBPROP_INIT_OLEDBSERVICES, -1)
			l_ps1.add_integer_property (constants.DBPROP_INIT_TIMEOUT, 100)
				-- Should fix timeout on connect.
			create l_ps2.make (DBPROPSET_SQLSERVERDBINIT)
			l_ps2.add_bstr_property (SSPROP_INIT_APPNAME, global.app.program_path)
			open_with_service_components (class_id, <<l_ps1, l_ps2>>, Void)
			is_fast_load := false
		end

	make_fast_load (a_data_source: STRING; a_catalog: STRING; a_user_name: STRING; a_password: STRING) is
		local
			hr: INTEGER
			l_data_source: OLEDB_DATA_SOURCE
			l_ps1, l_ps2: OLEDB_PROPERTY_SET
		do
			database := a_data_source
			node := a_catalog
			create l_ps1.make (constants.DBPROPSET_DBINIT)
			if a_data_source /= Void then
				l_ps1.add_bstr_property (constants.DBPROP_INIT_DATASOURCE, a_data_source)
			end
			if a_catalog /= Void then
				l_ps1.add_bstr_property (constants.DBPROP_INIT_CATALOG, a_catalog)
			end
			if a_user_name /= Void then
				l_ps1.add_bstr_property (constants.DBPROP_AUTH_USERID, a_user_name)
			end
			if a_password /= Void then
				l_ps1.add_bstr_property (constants.DBPROP_AUTH_PASSWORD, a_password)
			end
			l_ps1.add_integer_property (constants.DBPROP_INIT_TIMEOUT, 100)
				-- Should fix timeout on connect.
			create l_ps2.make (DBPROPSET_SQLSERVERDBINIT)
			l_ps2.add_bstr_property (SSPROP_INIT_APPNAME, global.app.program_path)
			debug ("make_movies")
				create cam.make (generating_type + "/" + node + "/" + database)
			end
			if playback_movie then
				make_succeeded := True
					-- If it didn't succeed then why bother with a movie?
			else
				hr := CoInitialize (default_pointer)
				create l_data_source.open (class_id, <<l_ps1, l_ps2>>)
				create session.make
				hr := session.open (l_data_source)
				if session.p /= default_pointer then
					sql_error (Current)
					make_succeeded := (hr >= 0)
					if not make_succeeded then
						if error_handler /= Void then
							error_handler.on_error (status, Void, "Unable to connect the server")
						end
						session := Void
					end
				end
			end
			is_fast_load := True
		end

feature -- Creation

	make_integrated_security (a_server_name, a_data_base_name: STRING)
			-- Integrated security is the standard Windows security - no password is required.
		require
			server_not_void: a_server_name /= Void
			db_name_not_void: a_data_base_name /= Void
		local
			l_ps1, l_ps2: OLEDB_PROPERTY_SET
		do
			database := a_data_base_name
			node := a_server_name
			create l_ps1.make (constants.DBPROPSET_DBINIT)
			if a_server_name /= Void then
				l_ps1.add_bstr_property (constants.DBPROP_INIT_DATASOURCE, a_server_name)
			end
			if a_data_base_name /= Void then
				l_ps1.add_bstr_property (constants.DBPROP_INIT_CATALOG, a_data_base_name)
			end
			l_ps1.add_bstr_property (constants.DBPROP_AUTH_INTEGRATED, "SSPI")
			l_ps1.add_integer_property (constants.DBPROP_INIT_OLEDBSERVICES, -1)
			if a_server_name /= Void and a_data_base_name /= Void then
				l_ps1.add_integer_property (constants.DBPROP_INIT_TIMEOUT, 100)
					-- Should fix timeout on connect.
			else
				l_ps1.add_integer_property (constants.DBPROP_INIT_TIMEOUT, 1)
			end
			create l_ps2.make (DBPROPSET_SQLSERVERDBINIT)
			l_ps2.add_bstr_property (SSPROP_INIT_APPNAME, global.app.program_path)

			if class_id = E_SQLNCLI_CLSID.item and config.boolean("use_mars", False)
			then
				-- Support for Multiple Access Row Sets (MARS).
				l_ps2.add_boolean_property (SSPROP_INIT_MARSCONNECTION, True)
			end

			open_with_service_components (class_id, <<l_ps1, l_ps2>>, Void)
			if status >= 0 then
			--	set_implicit_transaction_on		-- This is now deferred until we have determined that the user wants to write.
				is_fast_load := false
			else
				trace
			end
end

feature -- Status

	is_fast_load: BOOLEAN
			-- Is this server set up for fast loading?

	is_still_valid: BOOLEAN
			-- Is connection to this data server still valid?
			-- And, is it still connected to the database that it claims to be connected to?
		local
			l_db_name: STRING_PREDICATE
			l_retry: BOOLEAN
		do
			if not l_retry and then database /= Void and then connected then
				begin_transaction

				if success
				then
					create l_db_name.make_strict (Current, "select db_name()")
					l_db_name.retrieve (Void, Void)
					Result := l_db_name.success
						and then not l_db_name.array.is_empty
						and then l_db_name.array.first /= Void
						and then l_db_name.array.first.as_lower ~ database.as_lower
					commit
				end
			end
		rescue
			if not l_retry then
				l_retry := True
				retry
			end
		end

	version: STRING is
			-- Version for the sql data provider
		local
			l_string_predicate: STRING_PREDICATE
		do
			auxiliary_server.begin_transaction
			create l_string_predicate.make_strict (auxiliary_server, "select @@version")
			l_string_predicate.retrieve (Void, Void)
			auxiliary_server.commit
			if not l_string_predicate.array.is_empty then
				Result := l_string_predicate.array.first
			else
				Result := Precursor
			end
		end

	has_auxiliary_server: BOOLEAN
	-- Allows you to test whether an auxiliary server is present without creating one in the process.
	do
		Result := saved_auxiliary_server/=Void
	end

feature -- Access

	property (a_key: STRING): STRING is
			-- value retrieved from the key-value pairs in the _properties table if available
		local
			l_property_predicate: STRING_PREDICATE
			l_has_property_predicate: INTEGER_PREDICATE
		do
			if a_key /= Void then
				auxiliary_server.begin_transaction
				create l_has_property_predicate.make_strict (auxiliary_server, "select id from sysobjects where name = '_properties'")
				l_has_property_predicate.retrieve (Void, Void)
				if not l_has_property_predicate.array.is_empty and then l_has_property_predicate.array.first > 0 then
					create l_property_predicate.make_strict (auxiliary_server, "select value from _properties where [key] = ?")
					l_property_predicate.retrieve (Void, <<a_key>>)
					if not l_property_predicate.array.is_empty then
						Result := l_property_predicate.array.first
					end

					l_property_predicate.facilitator.release	-- Releases the OLEDB resources without waiting for a GC.
					l_property_predicate := Void				-- Make sure no one uses it since it is no longer connected.
				end

				l_has_property_predicate.facilitator.release	-- Releases the OLEDB resources without waiting for a GC.
				l_has_property_predicate := Void				-- Make sure no one uses it since it is no longer connected.
				auxiliary_server.commit
			end
		end

	fast_loader (a_table_name: STRING; a_template: ANY): DATA_SERVER_FAST_LOAD is
			-- A fast loader object that can be used to bulk load data
		local
			l_fast_load: OLEDB_FAST_LOAD [ANY]
		do
			create l_fast_load.make (Current, a_table_name, a_template)
			Result := l_fast_load
		end

	fast_loader_with_options (a_table_name: STRING; a_template: ANY; a_options: STRING): DATA_SERVER_FAST_LOAD is
			-- A fast loader object created with additional options that can be used to bulk load data
		local
			l_fast_load: OLEDB_FAST_LOAD [ANY]
		do
			create l_fast_load.make_with_options (Current, a_table_name, a_template, a_options)
			Result := l_fast_load
		end

	fast_loader_keep_identity (a_table_name: STRING; a_template: ANY): DATA_SERVER_FAST_LOAD is
			-- A fast loader object that can be used to bulk load data.
			-- This loader will maintain any identity column values and ignore the table's sequencing or current status.
		local
			l_fast_load: OLEDB_FAST_LOAD [ANY]
		do
			create l_fast_load.make_keep_identity (Current, a_table_name, a_template)
			Result := l_fast_load
		end

	stream_command (a_predicate_statement: STRING): DATA_SERVER_STREAM_COMMAND is
			-- A stream command object used to retrieve binary data from a column
		local
			l_stream_command: OLEDB_STREAM_COMMAND
		do
			create l_stream_command.make (Current, a_predicate_statement)
			Result := l_stream_command
		end

	simple_command (a_predicate_statement: STRING): DATA_SERVER_SIMPLE_COMMAND is
		local
			l_command: OLEDB_SIMPLE_COMMAND [OLEDB_DYNAMIC_ACCESSOR, OLEDB_ROWSET]
			l_result: INTEGER
		do
			create l_command
			l_result := l_command.open (session, a_predicate_statement, Void)
			l_command.accessor_rowset_make
			if l_result >= 0 then
				Result := l_command
			end
		end

	class_id: POINTER
	-- This class id specifies whether to use either SQLOLEDB or the SQL Native Client.
	once
		if config.boolean("use_sql_native_client", False)
		then
			Result := E_SQLNCLI_CLSID.item
		else
			Result := CLSID_SQLOLEDB
		end
	end

feature -- Repository Mechanism

	user_id: STRING is
			-- User id that is running this process
		local
			l_env: EXECUTION_ENVIRONMENT
			l_userdomain: STRING
			l_username: STRING
		once
			create l_env
			l_userdomain := l_env.get ("USERDOMAIN")
			l_username := l_env.get ("USERNAME")
			if l_userdomain /= Void and l_username /= Void then
				Result := l_userdomain + "\" + l_username
				Result.to_lower
			else
				Result := ""
			end
		ensure
			user_id_exists: Result /= Void
		end

	testenv_enabled: ROSE_DELIMITED_TEXT is
			-- Obsolete
		local
			l_env: EXECUTION_ENVIRONMENT
			l_string: STRING
		once
			create l_env
			l_string := l_env.get ("TESTENV")
			if l_string /= Void then
				l_string.to_lower
				create Result.make (l_string, ',')
			end
		end

feature -- Metrics.

	metrics: ROSE_LINEAR_ARRAY [PAIR [STRING, DOUBLE]] is
			-- Collection of execution metrics for this session
		local
			l_pair: PAIR [STRING, DOUBLE]
			l_exceptions: EXCEPTIONS
		do
			create Result.make_empty
			if not is_fast_load then
				begin_transaction
				cpu_predicate.retrieve (Void, Void)
				if not cpu_predicate.success then
					create l_exceptions
					l_exceptions.raise ("cpu_predicate failed")
				end
				old_cpu := cpu_predicate.array.item (1) / 1000.
				create l_pair.make ("DB/" + full_name (node + "_" + database + "_CPU"), old_cpu - old_cpu_at_reset)
				Result.extend (l_pair)
				physical_io_predicate.retrieve (Void, Void)
				if not physical_io_predicate.success then
					create l_exceptions
					l_exceptions.raise ("physical_io_predicate failed")
				end
				old_io := physical_io_predicate.array.item (1)
				create l_pair.make ("DB/" + full_name (node + "_" + database + "_PHYSICAL_IO"), old_io - old_io_at_reset)
				Result.extend (l_pair)
				commit
			end
		ensure then
			valid_result: Result /= Void
		end

	reset_metrics
			-- <Precursor>
		do
			old_cpu_at_reset := old_cpu
			old_io_at_reset := old_io
		end

	old_cpu: DOUBLE					-- The 'cpu' value at the last call to 'metrics'.

	old_cpu_at_reset: DOUBLE		-- The saved value of 'old_cpu' at the last call to 'reset_metrics'.

	old_io: DOUBLE					-- The 'io' value at the last call to 'metrics'.

	old_io_at_reset: DOUBLE			-- The saved value of 'old_io' at the last call to 'reset_metrics'.

feature -- Utility

	reconnect is
			-- Consider calling 'sql_disconnect' first.
			-- Reconnects this server.
		do
			status := 0
			success := False
			explicit_transaction_depth := 0
			implicit_transactions_set := False
			implicit_transaction := False
			disable_statement_is_feature := False
			make_succeeded := False
			cpu_predicate_saved := Void
			physical_io_predicate_saved := Void
			transaction_depth_predicate_saved := Void
			make (node, database)
			if saved_auxiliary_server/=Void
			then
				saved_auxiliary_server.reconnect
				saved_auxiliary_server.set_error_handler(saved_auxiliary_server.new_error_handler_print)
			end
		end

	sql_disconnect is
			-- Apparently, the connection is not actually released at this point.  The connection continues
			-- to be available to be reused by OLEDB but unavailable to this class (for about 60 seconds).
			-- If you make a new connection within that time limit, this SPID will be reused.  The SQL Profiler
			-- will show nothing until the reuse at which time you will see a "connection reset".
		do
			release_connection	-- This will release the 'auxiliary_server' connection as well.
			Precursor
		end

feature -- Schema

	table_names (a_pattern: STRING): ROSE_LINEAR_ARRAY [STRING] is
			-- Get list of tables that match `a_pattern' (SQL LIKE wildcards)
			-- Result empty if no match or query fails.
			-- (MS SQL server specific implementation)
		local
			l_predicate: STRING_PREDICATE
		do
			create l_predicate.make (Current, "SELECT name FROM sysobjects WHERE name LIKE ?")
			if not l_predicate.success then
				error.abort ("table_names", "Can't build predicate for table names query", Void)
			end
			l_predicate.retrieve (Void, <<a_pattern>>)
			if l_predicate.success then
				Result := l_predicate.array
			else
				create Result.make_empty
			end
		end

feature -- Trace

	trace is
		local
			l_approles: SQL_SERVER_APPROLE
			l_approle_name: STRING
		do
			Precursor
			if connected then
				create l_approles
				l_approle_name := l_approles.set_approle_name (Current)
				if l_approle_name /= Void then
					print ("Approle: " + l_approle_name + "%N")
					print ("         An application role has been set for this db session.%N")
					print ("         Ensure that this session is being used as intended.%N")
				end
			end
		end

feature {NONE} -- Implementation

	cpu_predicate_saved: INTEGER_PREDICATE		-- See 'cpu_predicate'.

	cpu_predicate: INTEGER_PREDICATE is
			-- A predicate to retrieve the CPU time used by this connection.
		local
			l_exceptions: EXCEPTIONS
		do
			if cpu_predicate_saved = Void then
					-- It is my belief that when there are multiple entries in sysprocesses for
					-- a single spid that you should sum up all of the cpu times (except 2147483647 which is bogus when ecid<>0).  RJ.
				create cpu_predicate_saved.make (Current, "select sum(cpu) from master.dbo.sysprocesses where spid = @@SPID and not (cpu = 2147483647 and ecid <> 0)")
				if not cpu_predicate_saved.success then
					create l_exceptions
					l_exceptions.raise ("Failed to make cpu_predicate")
				end
			end
			Result := cpu_predicate_saved
		end

	physical_io_predicate_saved: INTEGER_PREDICATE			-- See 'physical_io_predicate'.

	physical_io_predicate: INTEGER_PREDICATE is
			-- A predicate to retrieve the Physical IO used by this connection.
		local
			l_exceptions: EXCEPTIONS
		do
			if physical_io_predicate_saved = Void then
					-- It is my belief that when there are multiple entries in sysprocesses for
					-- a single spid that they all show the same physical_io.  RJ.
				create physical_io_predicate_saved.make (Current, "select top 1 physical_io from master.dbo.sysprocesses where spid=@@SPID")
				if not physical_io_predicate_saved.success then
					create l_exceptions
					l_exceptions.raise ("Failed to make physical_io_predicate")
				end
			end
			Result := physical_io_predicate_saved
		end

end
