note

	description:

		"Interface class for data that can be submitted by a signature tagged item database"

	author: "Mauricio Bustos"
	last_editor: "$Author: mb $"
	date: "$Date: 2011/04/29 $"
	revision: "$Revision: #4 $"
	path: "$File: //rose/source/kernel/persistence/databases/signature_tagged/stid_submittable.e $"

deferred class STID_SUBMITTABLE

feature -- Basic operations

	submit is
			-- Submit the cached updates to the database
		deferred
		end

	cleanup is
			-- Drop tables that were used for submitting new data
		deferred
		end

	dump_update_cache is
			-- Display the cached updates
		deferred
		end

	reset_metrics is
			-- Clear the currently stored metrics
		deferred
		end

	commit_fast_load is
			-- Commit the fast loaders for insert
		deferred
		end

	truncate_temp_tables is
			-- Truncate temp tables used for submitting new data
		deferred
		end

	discard_updates is
			-- Discard the staged update data
		deferred
		end

feature -- Access	

	unsubmitted: BOOLEAN is
			-- Does this object have any cached unsubmitted data
		deferred
		end

	type_name: STRING is
			-- The type name string for this item
		deferred
		end

	metrics: ROSE_LINEAR_ARRAY [PAIR [STRING, DOUBLE]] is
			-- Collection of metrics that can be submitted for persistence in the process metrics environment
		deferred
		ensure
			valid_result: Result /= Void
		end

	signature_item_server: DATA_SERVER is
		deferred
		end

feature -- Element change

	set_submit_frequency (a_freq: INTEGER) is
			-- Set the auto-submit frequency to a_freq
		deferred
		end

invariant

	valid_server: signature_item_server /= Void
	valid_type_name: type_name /= Void

end
