note

	description:

		"Cache object used by the stid_testenv_database to manage penultimate item value retrieval"

	author: "Mauricio Bustos"
	last_editor: "$Author: er $"
	date: "$Date: 2014/02/26 $"
	revision: "$Revision: #16 $"
	path: "$File: //rose/source/kernel/persistence/databases/signature_tagged/stid_testenv_type_cache.e $"

class
	STID_TESTENV_TYPE_CACHE [reference G -> ROSE_SIGNATURE_RECORD[ENTITY], H -> COMPARABLE]

inherit
	ROSE_SHARED_GLOBAL
		export
			{NONE} all
		end
	ANY

create
    make

feature {NONE} -- Creation

	make (a_data_name: STRING; a_source_server, a_target_server: SQL_DATA_SERVER; a_type_name: STRING) is
			-- Create the cache
		require
			source_server_not_void: a_source_server /= Void
			type_name_not_void: a_type_name /= Void
		local
			l_generic: H
			l_any: ANY
		do
			target_server := a_target_server
			type_name := a_type_name
			create table.make (a_source_server,a_data_name,Void)
			create penultimate_data_item_cache.make (1_000)
			create data_item_cache.make (100)
			create data_items_predicate.make_strict (server,
													 "select item_id,value,signature_id from " + table.data_name_prefix + "data_" + type_name + "_value where process_id = ? " +
													 "union " +
													 "select item_id,value,signature_id from " + table.data_name_prefix + "data_" + type_name + "_value_history where process_id = ? order by signature_id")
			create latest_data_items_cache.make (100)
			create latest_data_items_predicate.make_strict (server,"select * from " + table.data_name_prefix + "data_" + type_name + "_value where signature_id = ?")
			if a_target_server /= Void then
				create penultimate_data_item_predicate.make (a_target_server,
															 "select * from " + table.data_name_prefix + "data_" + type_name + "_value where path = ? and item_id = ? and process_id < ? " +
															 "union " +
															 "select * from " + table.data_name_prefix + "data_" + type_name + "_value_history where path = ? and item_id = ? and process_id < ? " +
															 "order by signature_id, process_id desc")
			end
			create penultimate_selective_data_item_cache.make (10)
			set_z_availability
			if z_data_enabled then
				create z_data_items_cache.make (1_000)
				if a_type_name.is_equal ("real") then
					create z_data_predicate.make_strict (target_server,"select item_id,cast (value as real) as value from z_real_data_view where signature_id = ?")
				elseif a_type_name.is_equal ("integer") then
					create z_data_predicate.make_strict (target_server,"select item_id,cast (value as int) as value,signature_id from z_real_data_view where signature_id = ?")
				end
			end
			l_any := l_generic
			if l_any /= Void then
				is_real_type := (create {INTERNAL}).type_name (l_any).is_equal ("REAL") or else (create {INTERNAL}).type_name (l_any).is_equal ("REAL_32")
				is_integer_type := (create {INTERNAL}).type_name (l_any).is_equal ("INTEGER") or else (create {INTERNAL}).type_name (l_any).is_equal ("INTEGER_32")
			end
		end

feature -- Basic operations

	load_penultimate_data_item_cache (a_item_id: INTEGER; a_process: SUPPLIER_PROCESS) is
			-- Load up the penultimate data cache for this item_id and this process
		require
			valid_process: a_process /= Void
			valid_item_id: a_item_id > 0
		local
			l_insert_process: EXECUTION_PREDICATE
			l_predicate: COLLECTING_PREDICATE[DATA_RECORD[H]]
			l_iterator: PREDICATE_ITERATOR [DATA_RECORD [H]]
			l_timer: ROSE_PROCESS_TIMER
		do
			create l_insert_process.make (target_server,
										  "insert into ##" + pid + "sid_process " +
										  "select signature_id,max (process_id) as process_id from " +
										  " (select signature_id,max (process_id) as process_id from " + table.data_name_prefix + "data_" + type_name + "_value " +
										  "where signature_id in (select signature_id from ##" + pid + "sids) and item_id = " + a_item_id.out + " and process_id <= " + a_process.process_id.out + " group by signature_id " +
										  "union " +
										  "select signature_id,max (process_id) as process_id from " + table.data_name_prefix + "data_" + type_name + "_value_history " +
										  "where signature_id in (select signature_id from ##" + pid + "sids) and item_id = " + a_item_id.out + " and process_id <= " + a_process.process_id.out + " group by signature_id) a group by signature_id")
			target_server.sql_execute_immediate ("truncate table ##" + pid + "sid_process")
			l_insert_process.execute (Void,Void)
			create l_predicate.make_strict (target_server,
											"select b.signature_id,b.value from " + table.data_name_prefix + "data_" + type_name + "_value b, " +
											"##" + pid + "sid_process c " +
											"where b.signature_id = c.signature_id and b.process_id = c.process_id and item_id = " + a_item_id.out +
											" union " +
											"select b.signature_id,b.value from " + table.data_name_prefix + "data_" + type_name + "_value_history b, " +
											"##" + pid + "sid_process c " +
											"where b.signature_id = c.signature_id and b.process_id = c.process_id and item_id = " + a_item_id.out)
			create l_timer
			l_predicate.open (Void,Void)
			l_iterator := l_predicate.iterator
			from l_iterator.start until l_iterator.off loop
				penultimate_selective_data_item_cache.force (l_iterator.item.value,l_iterator.item.signature_id)
				l_iterator.forth
			end
			l_predicate.facilitator.close
			--l_timer.display ("item_id: " + a_item_id.out + "%N")
			--print (Global.interpreted ("($now) item_id: " + a_item_id.out + "%T" + l_timer.formatted_elapsed + "%N"))
		end

feature -- Access

	penultimate_data_item_by_signature (a_signature: G; a_item_id: INTEGER): H is
			-- Query for the penultimate item value for a_signature as of a_signature.last_process_id
		require
			signature_not_void: a_signature /= Void
			valid_item_id: a_item_id > 0
		local
			l_data: HASH_TABLE [H,INTEGER]
			l_count: INTEGER
			l_iterator: PREDICATE_ITERATOR [DATA_RECORD [H]]
		do
			if is_real_type then
				Result ?= Undefined.Undefined_Real
			elseif is_integer_type then
				Result ?= Undefined.Undefined_Integer
			else
				Result ?= Undefined.Undefined_Double
			end
			if z_data_enabled and then z_items.has (a_item_id) then
				Result := z_data_item_by_signature (a_signature,a_item_id)
			else
				if selective_penultimate_selection then
					penultimate_selective_data_item_cache.search (a_signature.signature_id)
					if penultimate_selective_data_item_cache.found then
						Result := penultimate_selective_data_item_cache.found_item
					end
				else
					penultimate_data_item_cache.search (a_signature.path)
					if not penultimate_data_item_cache.found then
						create l_data.make (10_000)
						penultimate_data_item_cache.force (l_data,a_signature.path)
						initialize_penultimate_data_retrieval
						penultimate_data_item_predicate.open (Void,<<a_signature.path,a_item_id,a_signature.last_process_id,a_signature.path,a_item_id,a_signature.last_process_id>>)
						l_iterator := penultimate_data_item_predicate.iterator
						from l_iterator.start until l_iterator.off loop
							handle_penultimate_data_items (l_iterator.item, l_data)
							l_count := l_count + 1
							l_iterator.forth
						end
						penultimate_data_item_predicate.facilitator.close
					else
						l_data := penultimate_data_item_cache.found_item
					end
					l_data.search (a_signature.signature_id)
					if l_data.found then
						Result := l_data.found_item
					end
				end
			end
		end

	latest_data_items_by_signature (a_signature: G): ROSE_LINEAR_ARRAY[DATA_RECORD[H]] is
			-- Query for the latest item values for this signature
		require
			signature_not_void: a_signature /= Void
		local
			l_array: ROSE_LINEAR_ARRAY[DATA_RECORD[H]]
		do
			if latest_data_items_cache /= Void then
				latest_data_items_cache.search (a_signature.signature_id)
				if not latest_data_items_cache.found then
					latest_data_items_predicate.retrieve (Void,<<a_signature.signature_id>>)
					l_array := latest_data_items_predicate.array
					create Result.make_empty
					latest_data_items_cache.force (Result,a_signature.signature_id)
					from l_array.start until l_array.off loop
						Result.extend (l_array.current_item)
						l_array.forth
					end
				else
					Result := latest_data_items_cache.found_item
				end
			end
		end

	data_items_by_signature (a_signature: G): ROSE_LINEAR_ARRAY[DATA_RECORD[H]] is
			-- Query for item values for this signature on a_signature.last_process_id
		require
			signature_not_void: a_signature /= Void
		local
			l_process_cache: HASH_TABLE[ROSE_LINEAR_ARRAY[DATA_RECORD[H]],INTEGER]
			l_iterator: PREDICATE_ITERATOR [DATA_RECORD [H]]
			l_count: INTEGER
			l_timer: ROSE_PROCESS_TIMER
			l_retry_count: INTEGER
		do
			if l_retry_count < 2 then
				if data_item_cache /= Void then
					data_item_cache.search (a_signature.last_process_id)
					if not data_item_cache.found then
						server.commit
						create l_process_cache.make (1_000_000)
						data_item_cache.force (l_process_cache,a_signature.last_process_id)
						initialize_current_data_retrieval
						data_items_predicate.open (Void,<<a_signature.last_process_id,a_signature.last_process_id>>)
						if data_items_predicate.success then
							create l_timer
							l_iterator := data_items_predicate.iterator
							from l_iterator.start until l_iterator.off loop
								handle_data_items (l_iterator.item, l_process_cache)
								l_count := l_count + 1
								l_iterator.forth
							end
							data_items_predicate.facilitator.close
							--l_timer.display ("data_items_by_signature%N")
							--print (Global.interpreted ("($now) " + type_name + " data_items_by_signature%T" + l_timer.formatted_elapsed + "%N"))
						end
						Result := l_process_cache.item (a_signature.signature_id)
					else
						Result := data_item_cache.found_item.item (a_signature.signature_id)
					end
				end
			end
		rescue
			-- Probably deadlocked, try again
			l_retry_count := l_retry_count + 1
			retry
		end

feature -- Element change

	set_selective_penultimate_selection (a_value: BOOLEAN) is
			-- Should we select data signature by signature instead of by path?
		do
			selective_penultimate_selection := a_value
		end

feature -- Caches

	type_name: STRING
			-- The type name for this cache
	selective_penultimate_selection: BOOLEAN
			-- Should we select data signature by signature instead of by path?
	data_item_cache: HASH_TABLE[HASH_TABLE[ROSE_LINEAR_ARRAY[DATA_RECORD[H]],INTEGER],INTEGER]
			-- Cache of item_id values for a process_id keyed by signature_id keyed by item_id
	penultimate_data_item_cache: HASH_TABLE[HASH_TABLE[H,INTEGER],INTEGER]
			-- Penultimate data item values keyed by path and then by signature
	latest_data_items_cache: HASH_TABLE[ROSE_LINEAR_ARRAY[DATA_RECORD[H]],INTEGER]
			-- Cache of latest item_id values keyed by signature_id
	penultimate_selective_data_item_cache: HASH_TABLE[H,INTEGER]
			-- Cache of penultimate item_id values in relation to a process_id keyed by signature_id
	z_data_items_cache: HASH_TABLE[HASH_TABLE[DATA_RECORD[H],INTEGER],INTEGER]
			-- Cache of z_data keyed by signature_id keyed by item_id

feature {NONE} -- Implementation

	initialize_penultimate_data_retrieval is
			-- Initialize the attributes involved in penultimate data retrieval
		do
			current_penultimate_item_signature_id := 0
		end

	current_penultimate_item_signature_id: INTEGER
			-- Current signature id for processing of penultimate data items
	handle_penultimate_data_items (a_record: DATA_RECORD [H]; a_table: HASH_TABLE [H,INTEGER]) is
			-- Process retrieval of penultimate items
		require
			valid_record: a_record /= Void
			valid_table: a_table /= Void
		do
			if current_penultimate_item_signature_id /= a_record.signature_id then
				a_table.force (a_record.value,a_record.signature_id)
				current_penultimate_item_signature_id := a_record.signature_id
			end
		end

	initialize_current_data_retrieval is
			-- Initialize the attributes involved in current data retrieval
		do
			current_item_signature_id := 0
			current_signature_data := Void
		end

	current_item_signature_id: INTEGER
			-- Current signature id for processing of data items
	current_signature_data: ROSE_LINEAR_ARRAY[DATA_RECORD[H]]
			-- Collection of current data for the 'current_item_signature_id'
	handle_data_items (a_record: DATA_RECORD [H]; a_table: HASH_TABLE[ROSE_LINEAR_ARRAY[DATA_RECORD[H]],INTEGER]) is
			-- Process retrieval of penultimate items
		require
			valid_record: a_record /= Void
			valid_table: a_table /= Void
		do
			if current_item_signature_id /= a_record.signature_id then
				current_item_signature_id := a_record.signature_id
				create current_signature_data.make (1,1)
				current_signature_data.put (a_record,1)
				a_table.extend (current_signature_data,current_item_signature_id)
			else
				current_signature_data.extend (a_record)
			end
		end

feature {NONE} -- Predicates

	data_items_predicate: COLLECTING_PREDICATE[DATA_RECORD[H]]
			-- Predicate to find item_ids affected by a particular process_id
	penultimate_data_item_predicate: COLLECTING_PREDICATE[DATA_RECORD[H]]
			-- Predicate to find penultimate item_ids in relation to a process_id
	latest_data_items_predicate: COLLECTING_PREDICATE[DATA_RECORD[H]]
			-- Predicate to find the latest item_id values for a signature
	z_data_predicate: COLLECTING_PREDICATE[DATA_RECORD[H]]
			-- Predicate to populate z_data cache

feature {NONE} -- Storage

	server: SQL_DATA_SERVER is
			-- The server that has the data which we will be comparing
		do
			Result := table.signature_item_server
		end

	table: SIGNATURE_TAGGED_TABLE_STRUCTURE
			-- A table structure object for the database containing the data which we will be comparing
	target_server: SQL_DATA_SERVER
			-- The server against which we are comparing data

feature {NONE} -- Z data access

	z_items: HASH_TABLE [INTEGER,INTEGER]
	z_data_enabled: BOOLEAN

	set_z_availability
			-- Set availability status of z data in the target server
		local
			l_z_items_predicate: INTEGER_PREDICATE
		do
			create z_items.make (100)
			if target_server /= Void then
				z_data_enabled := target_server.has_table ("z_companies") and target_server.has_table ("z_real_data_view")
				if z_data_enabled then
					create l_z_items_predicate.make_strict (target_server,"select distinct item_id from z_companies")
					l_z_items_predicate.retrieve (Void,Void)
					from l_z_items_predicate.array.start until l_z_items_predicate.array.off loop
						z_items.force (l_z_items_predicate.array.current_item,l_z_items_predicate.array.current_item)
						l_z_items_predicate.array.forth
					end
				end
			end
		ensure
			available_items: z_items /= Void
		end

	z_data_item_by_signature (a_signature: G; a_item_id: INTEGER): H is
			-- Z_data value for a_item_id for a_signature
		require
			valid_signature: a_signature /= Void
		local
			l_cache: HASH_TABLE[DATA_RECORD[H],INTEGER]
		do
			if is_real_type then
				Result ?= Undefined.Undefined_Real
			elseif is_integer_type then
				Result ?= Undefined.Undefined_Integer
			else
				Result ?= Undefined.Undefined_Double
			end
			z_data_items_cache.search (a_signature.signature_id)
			if not z_data_items_cache.found then
				create l_cache.make (50)
				z_data_items_cache.force (l_cache,a_signature.signature_id)
				z_data_predicate.retrieve (Void,<<a_signature.signature_id>>)
				from z_data_predicate.array.start until z_data_predicate.array.off loop
					l_cache.force (z_data_predicate.array.current_item,z_data_predicate.array.current_item.item_id)
					z_data_predicate.array.forth
				end
			else
				l_cache := z_data_items_cache.found_item
			end
			l_cache.search (a_item_id)
			if l_cache.found then
				Result := l_cache.found_item.value
			end
		end

feature {NONE} -- Helpers

	pid: STRING is
			-- This application's pid
		once
			Result := Global.app.pid.out
		end

	Undefined: ROSE_LEGACY_UNDEFINED is
			-- Access to undefined
		once
			create Result
		end

	is_real_type: BOOLEAN
			-- Is the generic parameter for this class REAL?
	is_integer_type: BOOLEAN
			-- Is the generic parameter for this class INTEGER?

end







