indexing
	description: "Summary statistics record"
	author: "Michael Zhang"
	date: "$Date: 2008/12/01 $"
	revision: "$Revision: #4 $"
	archive: "$File: //rose/source/kernel/infrastructure/containers/summary_statistics_record.e $"

class
	SUMMARY_STATISTICS_RECORD

inherit
	MISMATCH_CORRECTOR
		redefine
			correct_mismatch
		end
create
	make_from_moments, default_create

feature -- creation
	make_from_moments(a_moments : ROSE_MOMENTS) is
			-- make from moments
		require
			valid_moments : a_moments /= Void
		do
			count := a_moments.count
			min := a_moments.order_statistics[1]
			first_quartile := a_moments.weighted_first_quartile
			median := a_moments.median
			third_quartile := a_moments.weighted_third_quartile
			max := a_moments.order_statistics.last
			mean := a_moments.unweighted_mean
			standard_deviation := a_moments.standard_deviation
			weighted_mean := a_moments.weighted_mean

			square_total := a_moments.square_total
			weighted_total := a_moments.weighted_total

			lower_z_value := a_moments.lower_z_value
			upper_z_value := a_moments.upper_z_value

			skewness := a_moments.skewness
			kurtosis := a_moments.kurtosis
		end

	copy_record(a_record : SUMMARY_STATISTICS_RECORD) is
			-- copy data from another record
		require
			valid_record : a_record /= Void
		do
			count := a_record.count
			min := a_record.min
			first_quartile := a_record.first_quartile
			median := a_record.median
			third_quartile := a_record.third_quartile
			max := a_record.max
			mean := a_record.mean
			standard_deviation := a_record.standard_deviation
			weighted_mean := a_record.weighted_mean

			square_total := a_record.square_total
			weighted_total := a_record.weighted_total

			lower_z_value := a_record.lower_z_value
			upper_z_value := a_record.upper_z_value

			skewness := a_record.skewness
			kurtosis := a_record.kurtosis
		end

feature -- data
	count : INTEGER
	set_count(a_count : like count) is
			--|
		do
			count := a_count
		end

	min : DOUBLE
	set_min(a_min : like min) is
			--|
		do
			min := a_min
		end

	first_quartile : DOUBLE
	set_first_quartile(a_first_quartile : like first_quartile) is
			--|
		do
			first_quartile := a_first_quartile
		end

	median : DOUBLE
	set_median(a_median : like median) is
			--|
		do
			median := a_median
		end

	third_quartile : DOUBLE
	set_third_quartile(a_third_quartile : like third_quartile) is
			--|
		do
			third_quartile := a_third_quartile
		end

	max : DOUBLE
	set_max(a_max : like max) is
			--|
		do
			max := a_max
		end

   weighted_mean : DOUBLE
   set_weighted_mean(a_value : like weighted_mean) is
			--|
      	do
	 		weighted_mean := a_value
      	end

   mean : DOUBLE
   set_mean(a_value : like mean) is
 			--|
     	do
	 		mean := a_value
      	end

   standard_deviation : DOUBLE
   set_standard_deviation(a_value : like standard_deviation) is
			--|
		do
	 	 	standard_deviation := a_value
      	end

	square_total : DOUBLE
	set_square_total(a_square_total : like square_total) is
			--|
		do
			square_total := a_square_total
		end

	weighted_total : DOUBLE
	set_weighted_total(a_weighted_total : like weighted_total) is
			--|
		do
			weighted_total := a_weighted_total
		end

	lower_z_value : DOUBLE
	set_lower_z_value(a_lower_z_value : like lower_z_value) is
			--|
		do
			lower_z_value := a_lower_z_value
		end

	upper_z_value : DOUBLE
	set_upper_z_value(a_upper_z_value : like upper_z_value) is
			--|
		do
			upper_z_value := a_upper_z_value
		end

	skewness : DOUBLE
	set_skewness(a_skewness : like skewness) is
			--|
		do
			skewness := a_skewness
		end

	kurtosis : DOUBLE
	set_kurtosis(a_kurtosis : like kurtosis) is
			--|
		do
			kurtosis := a_kurtosis
		end

feature -- access
	to_vector : VECTOR is
			--|
		do
			create Result.make(15)
			Result[1] := count
			Result[2] := min
			Result[3] := first_quartile
			Result[4] := median
			Result[5] := third_quartile
			Result[6] := max
			Result[7] := mean
			Result[8] := standard_deviation
			Result[9] := weighted_mean
			Result[10] := square_total
			Result[11] := weighted_total
			Result[12] := lower_z_value
			Result[13] := upper_z_value
			Result[14] := skewness
			Result[15] := kurtosis
		end

	item_by_name(a_name : STRING) : DOUBLE is
			-- item by name
		require
			valid_name : a_name /= Void
		local
			l_ix : INTEGER
		do
			l_ix := names.new_index_of (a_name.as_lower)
			if l_ix > 0 then
				inspect l_ix
				when 1 then Result := count
				when 2 then Result := min
				when 3 then Result := first_quartile
				when 4 then Result := median
				when 5 then Result := third_quartile
				when 6 then Result := max
				when 7 then Result := mean
				when 8 then Result := standard_deviation
				when 9 then Result := weighted_mean
				when 10 then Result := square_total
				when 11 then Result := weighted_total
				when 12 then Result := lower_z_value
				when 13 then Result := upper_z_value
				when 14 then Result := skewness
				when 15 then Result := kurtosis
				else
				end
			else
				if a_name.as_lower.is_equal ("hhi") then
					Result := hhi
				elseif a_name.as_lower.is_equal ("total") then
					Result := count * mean
				elseif a_name.as_lower.is_equal ("effective_number") then
					Result := effective_number
				end
			end
		end

	names : ROSE_HASHED_LINEAR_ARRAY[STRING] is
		once
			create Result.make_from_array(<<"count","min","first_quartile","median","third_quartile","max","mean","standard_deviation","weighted_mean", "square_total", "weighted_total", "lower_z_value", "upper_z_value", "skewness", "kurtosis">>)
		end

	hhi : DOUBLE is
			-- Herfindahl-Hirschman Index
		do
			if count > 0 and mean > 0  then
				Result := 10000 * square_total / (mean * count)^2
			end
		end

	effective_number : DOUBLE is
			-- inverse of Herfindahl-Hirschman Index
		do
			if square_total > 0  then
				Result := (mean * count)^2 / square_total
			end
		end

	correct_mismatch is
		local
			l_mismatch: MISMATCH_INFORMATION
		do
			l_mismatch := mismatch_information
		end
end
