indexing
   description: "Supplier attribute type manager."
   keywords: "Attribute type, category, relationship"
   author: "David Stevens"
   date: "$Date: 2008/12/01 $"
class
   SUPPLIER_ATTRIBUTE_TYPE_MANAGER   
inherit
   ERROR_LOG_USE
   DATABASE_SERVER
	-->>CLONER
create 
   make

feature -- Initialization
   
   make (a_data_server : DATA_SERVER) is
      local
	 l_selection: SIMPLE_PREDICATE [SUPPLIER_ATTRIBUTE_TYPE]
	 l_relationship_selection: SIMPLE_PREDICATE [ATTRIBUTE_CATEGORY_RELATIONSHIP]
      do
	 data_server := a_data_server
	 create id_cache.make (0)
	 create name_cache.make (0)
	 create l_selection.make(data_server, select_statement)
	 if not l_selection.success then report_error ("Could not make l_selection predicate", "make") end
	 l_selection.apply (agent add_to_attribute_type_caches (?), Void, Void)
	 if not l_selection.success then report_error ("Could not apply l_selection predicate", "make") end
	 
	 create relationship_by_attribute_type_id_cache.make (0)
	 create relationship_by_category_id_cache.make (0)
	 create l_relationship_selection.make(data_server, "select attribute_type_id, category_id from attribute_category_relationship group by attribute_type_id, category_id")
	 if not l_relationship_selection.success then report_error ("Could not make l_relationship_selection predicate", "make") end
	 l_relationship_selection.apply (agent build_relationship_caches (?), Void, Void)
	 if not l_relationship_selection.success then report_error ("Could not apply l_relationship_selection predicate", "make") end
	 
	 create add_predicate.make(data_server, add_statement)
	 if not add_predicate.success then report_error ("Could not make add_predicate", "make") end
	 create select_by_name_predicate.make(data_server, "select * from attribute_type where name=:name")
	 if not select_by_name_predicate.success then report_error ("Could not make select_by_name_predicate", "make") end
      end
   
feature -- Access   
   
   attribute_type (a_attribute_type_id: INTEGER): SUPPLIER_ATTRIBUTE_TYPE is
      require
	 a_attribute_type_id_not_zero: a_attribute_type_id /= 0
      do
	 Result := id_cache.item (a_attribute_type_id)
	 if Result = Void then report_error ("Attribute_type not found (attribute_type_id) ...: ", "attribute_type") end
      end
   
   attribute_type_id (a_name: STRING): INTEGER is
      require
	 a_name_not_void: a_name /= Void
      do
	 name_cache.search (a_name)
	 if name_cache.found then
	    Result := name_cache.found_item.attribute_type_id
	    if Result = 0 then report_error ("Attribute_type not found (a_name) ...: ", "attribute_type_id") end
	 else
	    report_error ("Attribute_type not found (a_name) ...: ", "attribute_type_id")
	 end
      end
   
	get_or_create_attribute_type_id(a_attribute_type: SUPPLIER_ATTRIBUTE_TYPE): INTEGER is
	do
		name_cache.search(a_attribute_type.name)
		if name_cache.found
		then
			Result := name_cache.found_item.attribute_type_id
		else
			add_attribute_type (a_attribute_type)
			Result := attribute_type_id(a_attribute_type.name)
		end

		if Result=0
		then
			report_error("Attribute_type not found (a_name) ...: ", "get_or_create_attribute_type_id")
		end
	end

	add_attribute_type (a_attribute_type: SUPPLIER_ATTRIBUTE_TYPE) is
	local
		l_attribute_types: ROSE_LINEAR_ARRAY[SUPPLIER_ATTRIBUTE_TYPE]
	do
		add_predicate.execute(a_attribute_type, Void)
		if not add_predicate.success
		then
			report_error("add_predicate failed", "add_attribute_type")
		end

		-- Fetch it back so that we can get the id.
		create l_attribute_types.make_empty
		select_by_name_predicate.set_container(l_attribute_types)
		select_by_name_predicate.retrieve(a_attribute_type, Void)
		if not select_by_name_predicate.success
		then
			report_error("select_by_name_predicate", "add_attribute_type")
		end

		select_by_name_predicate.set_container(Void)
		if l_attribute_types.count/=1
		then
			report_error("select_by_name_predicate did not return 1 row", "add_attribute_type")
		end

		add_to_attribute_type_caches(l_attribute_types.item(1))
	end

   attribute_type_name (a_attribute_type_id: INTEGER): STRING is
      require
	 a_attribute_type_id_not_zero: a_attribute_type_id /= 0
      do
	 id_cache.search (a_attribute_type_id)
	 if id_cache.found then
	    Result := id_cache.found_item.name
	    if Result = Void then report_error ("Attribute_type not found (a_name) ...: ", "attribute_type_name") end
	 else
	    report_error ("Attribute_type not found (attribute_type_id) ...: " + a_attribute_type_id.out, "attribute_type_name")
	 end
      end
   
   attribute_type_is_unique_identifier (a_attribute_type_id: INTEGER): BOOLEAN is
      require
	 a_attribute_type_id_not_zero: a_attribute_type_id /= 0
      do
	 id_cache.search (a_attribute_type_id)
	 if id_cache.found then
	    Result := id_cache.found_item.is_unique_identifier
	 else
	    report_error ("Attribute_type not found (attribute_type_id) ...: " + a_attribute_type_id.out, "attribute_type_is_unique_identifier")
	 end
      end
   
   attribute_type_ids: ARRAYED_LIST [INTEGER] is
      do
	 create Result.make (id_cache.count)
	 from id_cache.start until id_cache.off loop
	    Result.extend (id_cache.item_for_iteration.attribute_type_id)
	    id_cache.forth
	 end
      end
   
   attribute_type_id_set (a_category_id: INTEGER): ROSE_LINEAR_ARRAY [INTEGER] is
      require
	 a_category_id_not_zero: a_category_id /= 0
      do
	 Result := relationship_by_category_id_cache.item (a_category_id)
      end
      
   category_id_set (a_attribute_type_id: INTEGER): ROSE_LINEAR_ARRAY [INTEGER] is
      require
	 a_attribute_type_id_not_zero: a_attribute_type_id /= 0
      do
	 Result := relationship_by_attribute_type_id_cache.item (a_attribute_type_id)
      end
      
feature {WORLDSCOPE_ATTRIBUTE_TYPE_CODE_MANAGER}
   add_to_attribute_type_caches (a_attribute_type: SUPPLIER_ATTRIBUTE_TYPE) is
      do
	 id_cache.put (deep_clone (a_attribute_type), a_attribute_type.attribute_type_id)
	 name_cache.put (deep_clone (a_attribute_type), a_attribute_type.name)
      end
   
feature {NONE} -- Implementation
   
   id_cache: HASH_TABLE [SUPPLIER_ATTRIBUTE_TYPE, INTEGER]
   
   name_cache: HASH_TABLE [SUPPLIER_ATTRIBUTE_TYPE, STRING]
   
   relationship_by_attribute_type_id_cache: HASH_TABLE [ROSE_LINEAR_ARRAY [INTEGER], INTEGER]

   relationship_by_category_id_cache: HASH_TABLE [ROSE_LINEAR_ARRAY [INTEGER], INTEGER]

   select_by_name_predicate: SIMPLE_PREDICATE [SUPPLIER_ATTRIBUTE_TYPE]
   
   add_predicate: EXECUTION_PREDICATE
   
   data_server : DATA_SERVER
   
   
   build_relationship_caches (a_row: ATTRIBUTE_CATEGORY_RELATIONSHIP) is
      local
	 l_array: ROSE_LINEAR_ARRAY [INTEGER]
      do
	 relationship_by_attribute_type_id_cache.search (a_row.attribute_type_id)
	 if not relationship_by_attribute_type_id_cache.found then
	    create l_array.make (1, 1)
	    l_array.put (a_row.category_id, 1)
	    relationship_by_attribute_type_id_cache.extend (l_array, a_row.attribute_type_id)
	 else
	    relationship_by_attribute_type_id_cache.found_item.extend (a_row.category_id)
	 end
	 
	 relationship_by_category_id_cache.search (a_row.category_id)
	 if not relationship_by_category_id_cache.found then
	    create l_array.make (1, 1)
	    l_array.put (a_row.attribute_type_id, 1)
	    relationship_by_category_id_cache.extend (l_array, a_row.category_id)
	 else
	    relationship_by_category_id_cache.found_item.extend (a_row.attribute_type_id)
	 end
      end
   
   report_error (a_message, a_feature_name: STRING) is
      do
	 error_log.set_severity (error_log.Fatal)
	 error_log.set_message (a_message)
	 error_log.set_class_name ("SUPPLIER_ATTRIBUTE_TYPE_MANAGER")
	 error_log.set_feature_name (a_feature_name)
	 error_log.put
      end

	select_statement: STRING is
	do
		Result := "select * from attribute_type"
	end

	add_statement: STRING is
	do
		Result := "insert into attribute_type (name, is_unique_identifier) values (:name, :is_unique_identifier)"
	end

end
