indexing

   description: "Supplier entity attribute."
   status: "Development"
   author: "David Stevens"
   date: "$Date: 2010/07/08 $"
   revision: "$Revision: #38 $"

class

   SUPPLIER_ENTITY_ATTRIBUTE[G->SUPPLIER_ENTITY create default_create end]

inherit

   SUPPLIER_ENTITY_ATTRIBUTE_BASE
		redefine
		--/
		   default_create,
		--/-
		   select_predicate,
		   insert_predicate,
		   update_predicate
		end
	ERROR_LOG_USE
		undefine
			is_equal
	--/
		redefine
			default_create
	--/-
		end

create
	make,
	make_empty,
	make_without_insert,
	default_create

feature {SUPPLIER_ENTITY} -- Initialization
   default_create is
      do
         make_empty
      end

   make (a_supplier_entity: G;
		 a_attribute_type_id: INTEGER; a_value: STRING; a_start_date: ROSE_DATE) is
      require
		 a_supplier_entity_not_void: a_supplier_entity /= Void
		 a_attribute_type_id_not_zero: a_attribute_type_id /= 0
		 a_value_not_void: a_value /= Void
		 a_start_date_not_void: a_start_date /= Void
      do
		 create template.default_create
		 make_without_insert (a_supplier_entity, a_attribute_type_id)
		 start_date := a_start_date.twin
		 create end_date.make_max_date
		 attribute_value := a_value.twin
		 insert
      end

   template : G

   server: ENTITY_SERVER[SUPPLIER_ENTITY] is
   do
      if template=Void
      then
         create template.default_create
      end

      Result ?= template.server
   end

   make_without_insert (a_supplier_entity: G;
						a_attribute_type_id: INTEGER) is
      require
		 a_supplier_entity_not_void: a_supplier_entity /= Void
		 a_attribute_type_id_not_zero: a_attribute_type_id /= 0
      do
		 make_empty
		 category_id := a_supplier_entity.category_id
		 entity_id := a_supplier_entity.entity_id
		 attribute_type_id := a_attribute_type_id
      end

   make_empty is
	   		-- Create default without data
      do
		 create template.default_create
      end

feature -- Access

   attribute_type_name: STRING is
      do
		 Result := attribute_type_manager.attribute_type_name (attribute_type_id)
      end

feature -- Basic operations

   is_equal (other: SUPPLIER_ENTITY_ATTRIBUTE[G]): BOOLEAN is
      do
		 Result := equal( category_id, other.category_id) and
			entity_id = other.entity_id and
			attribute_type_id = other.attribute_type_id and
			equal (start_date, other.start_date) and
			equal (end_date,other.end_date) and
			equal (attribute_value,other.attribute_value)
      end

feature {SUPPLIER_ENTITY, SUPPLIER_ENTITY_CACHE} -- Basic operations

   terminate is
   		-- Terminate attribute.
      do
		 update_predicate.execute (Current, <<end_date, maximum_date>>)
		 if not update_predicate.success then
			error_log.set_severity (error_log.Fatal)
			error_log.set_message ("ERROR: Terminate failed (category_id, entity_id, attribute_type_id) ...: ")
			error_log.append_message (category_id.out)
			error_log.append_message (" ")
			error_log.append_message (entity_id.out)
			error_log.append_message (" ")
			error_log.append_message (attribute_type_id.out)
			error_log.set_class_name ("SUPPLIER_ENTITY_ATTRIBUTE")
			error_log.set_feature_name ("terminate")
			error_log.put
		 end
      end

   insert is
      do
--		 insert_predicate.execute (Current, Void)
          insert_predicate.execute (Void,
                                    <<Current.category_id,
                                      Current.entity_id,
                                      Current.attribute_type_id,
                                      Current.start_date,
                                      Current.end_date,
                                      Current.attribute_value>>)
		 if not insert_predicate.success then
			error_log.set_severity (error_log.Fatal)
			error_log.set_message ("ERROR: Insert failed (category_id, entity_id, attribute_type_id) ...: ")
			error_log.append_message (category_id.out)
			error_log.append_message (" ")
			error_log.append_message (entity_id.out)
			error_log.append_message (" ")
			error_log.append_message (attribute_type_id.out)
			error_log.set_class_name ("SUPPLIER_ENTITY_ATTRIBUTE")
			error_log.set_feature_name ("insert")
			error_log.put
		 end
      end

	update_start_date(a_start_date: ROSE_DATE) is
	do
		update_start_date_predicate.execute(Current, <<a_start_date>>)
		if not update_start_date_predicate.success
		then
			error_log.set_severity(error_log.Fatal)
			error_log.set_message("ERROR: Update start date failed (category_id, entity_id, attribute_type_id) ...: ")
			error_log.append_message(category_id.out)
			error_log.append_message(" ")
			error_log.append_message(entity_id.out)
			error_log.append_message(" ")
			error_log.append_message(attribute_type_id.out)
			error_log.set_class_name("SUPPLIER_ENTITY_ATTRIBUTE")
			error_log.set_feature_name("update_start_date")
			error_log.put
		 end

		start_date := a_start_date.twin
	end

	update_end_date(a_end_date: ROSE_DATE) is
	do
		update_end_date_predicate.execute(Current, <<a_end_date>>)
		if not update_end_date_predicate.success
		then
			error_log.set_severity(error_log.Fatal)
			error_log.set_message("ERROR: Update end date failed (category_id, entity_id, attribute_type_id) ...: ")
			error_log.append_message(category_id.out)
			error_log.append_message(" ")
			error_log.append_message(entity_id.out)
			error_log.append_message(" ")
			error_log.append_message(attribute_type_id.out)
			error_log.set_class_name("SUPPLIER_ENTITY_ATTRIBUTE")
			error_log.set_feature_name("update_end_date")
			error_log.put
		 end

		end_date := a_end_date.twin
	end

feature {SUPPLIER_ENTITY_CACHE} -- Cache

   entity_key: KEY_PAIR [INTEGER, INTEGER] is
      do
		 create Result.make (entity_id, attribute_type_id)
      end

   value_key: MULTI_KEY is
      do
		 create Result.make (<<category_id, attribute_type_id, attribute_value>>)
      end

   attribute_type_manager: SUPPLIER_ATTRIBUTE_TYPE_MANAGER is
         -- Error log.
      do
		Result := template.server.attribute_type_manager
      end

feature {ENTITY_SERVER} -- Implementation

   select_predicate: COLLECTING_PREDICATE [like Current] is
	  do
		 -- For the moment, this is not used.  SUPPLIER_ENTITY has a predicate that might be used.
--		 Result := server.select_attribute_predicate
      end

   insert_predicate: EXECUTION_PREDICATE is
	  do
		 Result := server.insert_entity_attribute_predicate
      end

   update_predicate: EXECUTION_PREDICATE is
      do
		 Result := server.update_entity_attribute_predicate
	  end

   update_start_date_predicate: EXECUTION_PREDICATE is
      do
		 Result := server.update_entity_attribute_start_date_predicate
	  end

	make_update_start_date_predicate: EXECUTION_PREDICATE is
	do
		create Result.make(data_server, update_start_date_statement)
		if not Result.success
		then
		    error_log.set_severity(error_log.Fatal)
		    error_log.set_message("ERROR: Could not make predicate")
		    error_log.set_class_name("SUPPLIER_ENTITY_ATTRIBUTE")
		    error_log.set_feature_name("make_update_start_date_predicate")
		    error_log.put
		end
	end

   update_end_date_predicate: EXECUTION_PREDICATE is
      do
		 Result := server.update_entity_attribute_end_date_predicate
	  end

	make_update_end_date_predicate: EXECUTION_PREDICATE is
	do
		create Result.make(data_server, update_end_date_statement)
		if not Result.success
		then
		    error_log.set_severity(error_log.Fatal)
		    error_log.set_message("ERROR: Could not make predicate")
		    error_log.set_class_name("SUPPLIER_ENTITY_ATTRIBUTE")
		    error_log.set_feature_name("make_update_end_date_predicate")
		    error_log.put
		end
	end


feature {NONE} -- Implementation

   maximum_date: ROSE_DATE is
      once
		 create Result.make_max_date
      end

   data_server: DATA_SERVER is
      do
		 Result := template.data_server
		 -- once
		 -- Result := NAME_OF_SERVER_server
      end

   select_statement: STRING is
	   	local
	   		e: EXCEPTIONS
        do
        	create e
            e.raise("oops")
		 Result := "select * from entity_attribute where category_id = :category_id and entity_id = :entity_id"
      end

   insert_statement: STRING is
      do
		 Result := "insert into entity_attribute (%
				   %category_id, %
				   %entity_id, %
				   %attribute_type_id, %
				   %start_date, %
				   %end_date, %
                   %attribute_value) values (%
                   % ?, ?, ?, ?, ?, ?)"
-- 				   %attribute_value) values (%
-- 				   %:category_id, %
-- 				   %:entity_id, %
-- 				   %:attribute_type_id, %
-- 				   %:start_date, %
-- 				   %:end_date, %
-- 				   %:attribute_value)"
      end

   update_statement: STRING is
      do
	 Result := "update entity_attribute set end_date = ? where %
	 %category_id = :category_id and %
	 %entity_id = :entity_id and %
	 %attribute_type_id = :attribute_type_id and %
	 %end_date = ?"
      end

	update_start_date_statement: STRING is
	do
		Result :=
			"update %
			%entity_attribute %
			%set start_date = ? %
			%where %
	 		%category_id = :category_id and %
	 		%entity_id = :entity_id and %
			%attribute_type_id = :attribute_type_id and %
			%end_date = :end_date"
	end

	update_end_date_statement: STRING is
	do
		Result :=
			"update %
			%entity_attribute %
			%set end_date = ? %
			%where %
	 		%category_id = :category_id and %
	 		%entity_id = :entity_id and %
			%attribute_type_id = :attribute_type_id and %
			%end_date = :end_date"
	end



end -- class SUPPLIER_ENTITY_ATTRIBUTE
