indexing

   description: "Supplier entity cache."
   status: "Development"
   author: "David Stevens"
   last_editor: "$author: $"
   date: "$Date: 2014/03/07 $"
   revision: "$Revision: #52 $"
   path: "$file: $"

class
   SUPPLIER_ENTITY_CACHE [G->SUPPLIER_ENTITY create default_create end]

inherit
   DATABASE_SERVER
   ERROR_LOG_USE

create
	make,
	make_with_attributes,
	make_empty

feature -- Initialization

   make_empty is
   		-- Creation procedure.
	local
		l_entity : G
	do
		create l_entity.default_create
		data_server := l_entity.data_server
	end

   make (a_category_id_set: ARRAY [INTEGER]) is
   		-- Make with 'a_category_id_set'.
      local
		 l_category_attribute_set: ARRAY [TUPLE [INTEGER, ARRAY [INTEGER]]]
		 i: INTEGER
      do
		 create l_category_attribute_set.make (a_category_id_set.lower, a_category_id_set.upper)
		 from
			i := a_category_id_set.lower
		 until
			i > a_category_id_set.upper
		 loop
			l_category_attribute_set.put ([a_category_id_set.item (i), <<>>], i)
			i := i + 1
		 end
		 make_with_attributes (l_category_attribute_set)
      end

   data_server : DATA_SERVER
   server : ENTITY_SERVER [G]

   make_with_attributes (a_category_attribute_set: ARRAY [TUPLE [INTEGER, ARRAY [INTEGER]]]) is
   		-- Creation procedure.
      local
		 l_entity_array: EXTENDABLE_BOUNDED_ARRAY [G]
		 l_attribute_set: DYNAMICALLY_SORTABLE_ARRAY [SUPPLIER_ENTITY_ATTRIBUTE[G]]
		 l_attribute_array: EXTENDABLE_BOUNDED_ARRAY [SUPPLIER_ENTITY_ATTRIBUTE[G]]
		 i, j, k, m: INTEGER
		 l_supplier_entity: G
		 l_temp_cache: like Current
		 l_attribute_type_id_table: HASH_TABLE [INTEGER, INTEGER]
		 l_attribute_type_id_array: ARRAY [INTEGER]
		 l_attribute_list: STRING
		 l_category_id_ref: INTEGER_REF
		 l_category_id: INTEGER
		 l_entity : G
      do
		 create l_entity
		 data_server := l_entity.data_server
		 server ?= l_entity.server
		 -- Set the select statements.
		 select_attribute_statement := "select * from entity_attribute"
		 if a_category_attribute_set = Void then
			select_entity_statement := "select * from entity_id"
		 else
			create id_table.make (100)
			select_entity_statement := "select * from entity_id where "
			select_attribute_statement.append (" where (")
			from
			   i := 1
			until
			   i > a_category_attribute_set.count
			loop
			   l_category_id_ref ?= a_category_attribute_set.item (i).item (1)
			   l_category_id := l_category_id_ref.item
			   l_attribute_type_id_array ?= a_category_attribute_set.item (i).item (2)
			   create l_attribute_type_id_table.make (l_attribute_type_id_array.count)
			   if l_attribute_type_id_array.count = 0 then
				  l_attribute_list := ""
			   else
				  l_attribute_list := " and ("
			   end
			   from
				  j := l_attribute_type_id_array.lower
			   until
				  j > l_attribute_type_id_array.upper
			   loop
				  l_attribute_type_id_table.put
				  (l_attribute_type_id_array.item (j), l_attribute_type_id_array.item (j))
				  if l_attribute_type_id_table.conflict then
					 error_log.set_severity (error_log.Fatal)
					 error_log.set_message ("ERROR: Duplicate attribute_id, category_id ...: ")
					 error_log.append_message (l_attribute_type_id_array.item (j).out)
					 error_log.append_message (", ")
					 error_log.append_message (l_category_id.out)
					 error_log.set_class_name ("SUPPLIER_ENTITY_CACHE")
					 error_log.set_feature_name ("make")
					 error_log.put
				  end
				  l_attribute_list.append ("attribute_type_id = ")
				  l_attribute_list.append (l_attribute_type_id_array.item (j).out)
				  if j = l_attribute_type_id_array.upper then
					 l_attribute_list.append (")")
				  else
					 l_attribute_list.append (" or ")
				  end
				  j := j + 1
			   end
			   id_table.put (l_attribute_type_id_table, l_category_id)
			   if id_table.conflict then
				  error_log.set_severity (error_log.Fatal)
				  error_log.set_message ("ERROR: Duplicate category_id ...: ")
				  error_log.append_message (l_category_id.out)
				  error_log.set_class_name ("SUPPLIER_ENTITY_CACHE")
				  error_log.set_feature_name ("make")
				  error_log.put
			   end
			   select_entity_statement.append ("category_id = ")
			   select_entity_statement.append (l_category_id.out)
			   select_attribute_statement.append ("(category_id = ")
			   select_attribute_statement.append (l_category_id.out)
			   select_attribute_statement.append (l_attribute_list)
			   if i = a_category_attribute_set.count then
				  select_attribute_statement.append ("))")
			   else
				  select_entity_statement.append (" or ")
				  select_attribute_statement.append (") or ")
			   end
			   i := i + 1
			end
		 end
		 select_entity_statement.append (" order by category_id, entity_id, end_date")
		 select_attribute_statement.append (" order by category_id, entity_id, attribute_type_id, end_date")

		 -- Retrieve the entities.
		 create sorted_entity_cache.make_empty
		 select_entity_predicate := make_select_entity_predicate
		 select_attribute_predicate := make_select_attribute_predicate
		 select_entity_predicate.set_container (sorted_entity_cache)
		 select_entity_predicate.retrieve (Void, Void)
		 if not select_entity_predicate.success then
			error_log.set_severity (error_log.Fatal)
			error_log.set_message ("ERROR: Unable to retrieve from entity_id")
			error_log.set_class_name ("SUPPLIER_ENTITY_CACHE")
			error_log.set_feature_name ("make")
			error_log.put
		 end

		 -- Set the entity_cache.
		 create entity_cache.make (sorted_entity_cache.count)
		 from
			i := 1
		 until
			i > sorted_entity_cache.count
		 loop
			from
			   j := i
			until
			   j = sorted_entity_cache.count or else
			   (sorted_entity_cache.item (j).category_id /= sorted_entity_cache.item (j + 1).category_id or
				  sorted_entity_cache.item (j).entity_id /= sorted_entity_cache.item (j + 1).entity_id)
			loop
			   j := j + 1
			end
			create l_entity_array.make (1, (j - i + 1))
			from
			   k := l_entity_array.lower
			   m := i
			until
			   k > l_entity_array.upper
			loop
			   l_entity_array.put (sorted_entity_cache.item (m), k)
			   k := k + 1
			   m := m + 1
			end
			entity_cache.put (l_entity_array, l_entity_array.item (1).key)
			i := m
		 end

		 -- Retrieve the attributes.
		 create l_attribute_set.make_empty
		 select_attribute_predicate.set_container (l_attribute_set)
		 select_attribute_predicate.retrieve (Void, Void)
		 if not select_attribute_predicate.success then
			error_log.set_severity (error_log.Fatal)
			error_log.set_message ("ERROR: Unable to retrieve from entity_attribute")
			error_log.set_class_name ("SUPPLIER_ENTITY_CACHE")
			error_log.set_feature_name ("make")
			error_log.put
		 end

		 -- Set the attribute_by_entity_cache.
		 create attribute_by_entity_cache.make (l_attribute_set.count)
		 from
			i := 1
		 until
			i > l_attribute_set.count
		 loop
			from
			   j := i
			until
			   j = l_attribute_set.count or else
			   (l_attribute_set.item (j).category_id /= l_attribute_set.item (j + 1).category_id or
				  l_attribute_set.item (j).entity_id /= l_attribute_set.item (j + 1).entity_id or
				  l_attribute_set.item (j).attribute_type_id /= l_attribute_set.item (j + 1).attribute_type_id)
			loop
			   j := j + 1
			end
			create l_attribute_array.make (1, (j - i + 1))
			from
			   k := l_attribute_array.lower
			   m := i
			until
			   k > l_attribute_array.upper
			loop
			   l_attribute_array.put (l_attribute_set.item (m), k)
			   k := k + 1
			   m := m + 1
			end
			attribute_by_entity_cache.put (l_attribute_array, l_attribute_array.item (1).entity_key)
			if attribute_by_entity_cache.conflict then
			   error_log.set_severity (error_log.Fatal)
			   error_log.set_message ("ERROR: Non unique supplier_attribute entity key (catid, entid, attid) ...: ")
			   error_log.append_message (l_attribute_array.item (1).category_id.out)
			   error_log.append_message (" ")
			   error_log.append_message (l_attribute_array.item (1).entity_id.out)
			   error_log.append_message (" ")
			   error_log.append_message (l_attribute_array.item (1).attribute_type_id.out)
			   error_log.set_class_name ("SUPPLIER_ENTITY_CACHE")
			   error_log.set_feature_name ("make_with_attributes")
			   error_log.put
			end
			i := m
		 end

		 -- Set the attribute_by_value_cache.
		 create attribute_by_value_cache.make (l_attribute_set.count)
		 l_attribute_set.sort_by (agent compare_by_value_key (?, ?))
		 from
			i := 1
		 until
			i > l_attribute_set.count
		 loop
			from
			   j := i
			until
			   j = l_attribute_set.count or else
			   (l_attribute_set.item (j).category_id /= l_attribute_set.item (j + 1).category_id or
				  l_attribute_set.item (j).attribute_type_id /= l_attribute_set.item (j + 1).attribute_type_id or
				  not l_attribute_set.item (j).attribute_value.is_equal (l_attribute_set.item (j + 1).attribute_value))
			loop
			   j := j + 1
			end
			create l_attribute_array.make (1, (j - i + 1))
			from
			   k := l_attribute_array.lower
			   m := i
			until
			   k > l_attribute_array.upper
			loop
			   l_attribute_array.put (l_attribute_set.item (m), k)
			   k := k + 1
			   m := m + 1
			end
			attribute_by_value_cache.put (l_attribute_array, l_attribute_array.item (1).value_key)
			if attribute_by_value_cache.conflict then
			   error_log.set_severity (error_log.Fatal)
			   error_log.set_message ("ERROR: Non unique supplier_attribute value key (catid, attid, attvl) ...: ")
			   error_log.append_message (l_attribute_array.item (1).category_id.out)
			   error_log.append_message (" ")
			   error_log.append_message (l_attribute_array.item (1).attribute_type_id.out)
			   error_log.append_message (" ")
			   error_log.append_message (l_attribute_array.item (1).attribute_value)
			   error_log.set_class_name ("SUPPLIER_ENTITY_CACHE")
			   error_log.set_feature_name ("make_with_attributes")
			   error_log.put
			end
			i := m
		 end

		 -- Attach the cache to supplier_entity.
		 l_supplier_entity := make_empty_entity
		 l_temp_cache ?= server.set_entity_cache (Current)

		 if l_temp_cache=Void then
			error_log.set_severity (error_log.Fatal)
			error_log.set_message ("ERROR: You must create the cache before the first access to the data.")
			error_log.set_class_name ("SUPPLIER_ENTITY_CACHE")
			error_log.set_feature_name ("make")
			error_log.put
		 end

		 if l_temp_cache/=Current then
			error_log.set_severity (error_log.Fatal)
			error_log.set_message ("ERROR: Only one cache is allowed.")
			error_log.set_class_name ("SUPPLIER_ENTITY_CACHE")
			error_log.set_feature_name ("make")
			error_log.put
		 end
      end

   entity_cache_hits: INTEGER
   entity_cache_misses: INTEGER
   entity_cache_count: INTEGER is
	  do
		 Result := entity_cache.count
	  end
   entity_cache_hit_ratio: DOUBLE is
	  do
		 Result := entity_cache_hits / (entity_cache_hits + entity_cache_misses)
	  end

   attribute_by_entity_cache_hits: INTEGER
   attribute_by_entity_cache_misses: INTEGER
   attribute_by_entity_cache_count: INTEGER is
	  do
		 Result := attribute_by_entity_cache.count
	  end
   attribute_by_entity_cache_hit_ratio: DOUBLE is
	  do
		 Result := attribute_by_entity_cache_hits / (attribute_by_entity_cache_hits + attribute_by_entity_cache_misses)
	  end

   attribute_by_value_cache_hits: INTEGER
   attribute_by_value_cache_misses: INTEGER
   attribute_by_value_cache_count: INTEGER is
	  do
		 Result := attribute_by_value_cache.count
	  end
   attribute_by_value_cache_hit_ratio: DOUBLE is
	  do
		 Result := attribute_by_value_cache_hits / (attribute_by_value_cache_hits + attribute_by_value_cache_misses)
	  end

feature {SUPPLIER_ENTITY} -- Access

   entity (a_category_id: INTEGER; a_entity_id: INTEGER; a_date: ROSE_DATE): G is
      require
		 a_category_id_not_zero: a_category_id /= 0
		 a_entity_id_not_zero: a_entity_id /= 0
		 category_is_cached: has_category (a_category_id)
		 --a_date_not_void: a_date /= Void
      local
		 l_key: KEY_PAIR [INTEGER, INTEGER]
		 l_entity_array: EXTENDABLE_BOUNDED_ARRAY [G]
		 i: INTEGER
		 l_date: ROSE_DATE
      do
		 create l_key.make (a_category_id, a_entity_id)
		 entity_cache.search (l_key)
		 if entity_cache.found then
			l_entity_array := entity_cache.found_item
			if a_date=Void
			 then
			   if l_entity_array.count>0	-- Sanity check only.
				then
				  Result := l_entity_array.item(l_entity_array.upper)
			   end
			else
			   l_date := a_date.not_max
			   from
				  i := l_entity_array.upper
			   until
				  i < l_entity_array.lower or Result/=Void
			   loop
				  if l_entity_array.item (i).start_date <= a_date and then
					 l_entity_array.item (i).end_date > l_date then
					 Result := l_entity_array.item (i)
				  end
				  i := i - 1
			   end
			   if Result = Void and then l_entity_array.count > 0 then
				  Result := l_entity_array.item(l_entity_array.upper)
			   end
			end
		 end

		 if Result/=Void
		  then
			entity_cache_hits := entity_cache_hits + 1
		 else
			entity_cache_misses := entity_cache_misses + 1
		 end
      end

   entity_item (a_category_id: INTEGER): G is
		 -- Entity for 'a_category_id' at the current entity cursor position.
      require
		 valid_cursor: a_category_id /= 0 and then not entity_exhausted (a_category_id)
      do
		 Result := sorted_entity_cache.item (entity_cursor_cache.item (a_category_id))
      end

   attribute_by_value (a_category_id: INTEGER;
					   a_attribute_type_id: INTEGER;
					   a_attribute_value: STRING;
					   a_as_of_date: ROSE_DATE): SUPPLIER_ENTITY_ATTRIBUTE[G] is
      require
		 a_category_id_not_zero: a_category_id /= 0
		 a_attribute_type_id_not_zero: a_attribute_type_id /= 0
		 a_attribute_value_not_void: a_attribute_value /= Void
		 category_is_cached: has_category (a_category_id)
      local
		 l_key: MULTI_KEY
		 l_attribute_array: EXTENDABLE_BOUNDED_ARRAY [SUPPLIER_ENTITY_ATTRIBUTE[G]]
		 i: INTEGER
		 l_as_of_date: ROSE_DATE
      do
		 create l_key.make (<<a_category_id, a_attribute_type_id, a_attribute_value>>)
		 attribute_by_value_cache.search (l_key)
		 if attribute_by_value_cache.found then
			l_attribute_array := attribute_by_value_cache.found_item
			if a_as_of_date = Void then
			   if l_attribute_array.count>0	-- Sanity check only.
				then
				  Result := l_attribute_array.item (l_attribute_array.upper)
			   end
			else
			   l_as_of_date := a_as_of_date.not_max
			   from
				  i := l_attribute_array.upper
			   until
				  i < l_attribute_array.lower or Result/=Void
			   loop
				  if l_attribute_array.item (i).start_date <= a_as_of_date and then
					 l_attribute_array.item (i).end_date > l_as_of_date then
					 Result := l_attribute_array.item (i)
				  end
				  i := i - 1
			   end
			end
		 end

		 if Result/=Void
		  then
			attribute_by_value_cache_hits := attribute_by_value_cache_hits + 1
		 else
			attribute_by_value_cache_misses := attribute_by_value_cache_misses + 1
		 end
      end

   attribute_by_id (a_supplier_entity: SUPPLIER_ENTITY ;
			  a_attribute_type_id: INTEGER; a_date: ROSE_DATE): SUPPLIER_ENTITY_ATTRIBUTE[G] is
      require
		 a_supplier_entity_not_void: a_supplier_entity /= Void
		 a_attribute_type_id_not_zero: a_attribute_type_id /= 0
		 category_is_cached: has_category (a_supplier_entity.category_id)
		 --a_date_not_void: a_date /= Void
      local
		 l_key: KEY_PAIR [INTEGER, INTEGER]
		 l_attribute_array: EXTENDABLE_BOUNDED_ARRAY [SUPPLIER_ENTITY_ATTRIBUTE[G]]
		 i: INTEGER
		 l_date: ROSE_DATE
      do
		 create l_key.make (a_supplier_entity.entity_id, a_attribute_type_id)
		 attribute_by_entity_cache.search (l_key)
		 if attribute_by_entity_cache.found then
			l_attribute_array := attribute_by_entity_cache.found_item
			if a_date = Void then
			   if l_attribute_array.count>0	-- Sanity check only.
			   then
				  Result := l_attribute_array.item (l_attribute_array.upper)
			   end
			else
			   l_date := a_date.not_max
			   from
				  i := l_attribute_array.upper
			   until
				  i < l_attribute_array.lower or Result/=Void
			   loop
				  if l_attribute_array.item (i).start_date <= a_date and then
					 l_attribute_array.item (i).end_date > l_date then
					 Result := l_attribute_array.item (i)
				  end
				  i := i - 1
			   end
			end
		 end

		 if Result/=Void
		  then
			attribute_by_entity_cache_hits := attribute_by_entity_cache_hits + 1
		 else
			attribute_by_entity_cache_misses := attribute_by_entity_cache_misses + 1
		 end
      end

	attributes_between(a_supplier_entity: SUPPLIER_ENTITY; a_attribute_type_id: INTEGER; a_start_date, a_end_date: ROSE_DATE):
		ROSE_LINEAR_ARRAY[SUPPLIER_ENTITY_ATTRIBUTE[G]] is
	-- Get all attributes which were ever alive in the range [a_start_date, a_end_date].
	require
		valid_args: a_supplier_entity /= Void and a_attribute_type_id /= 0 and a_start_date /= Void and a_end_date /= Void and a_start_date < a_end_date
	local
		l_attribute_array: EXTENDABLE_BOUNDED_ARRAY[SUPPLIER_ENTITY_ATTRIBUTE[G]]
		l_key: KEY_PAIR[INTEGER, INTEGER]
		i: INTEGER
	do
		create l_key.make(a_supplier_entity.entity_id, a_attribute_type_id)
		attribute_by_entity_cache.search(l_key)
		if attribute_by_entity_cache.found
		then
			l_attribute_array := attribute_by_entity_cache.found_item
			from
				i := l_attribute_array.lower
			until
				i > l_attribute_array.upper
			loop
				if (l_attribute_array.item(i).start_date<=a_end_date and
				    l_attribute_array.item(i).end_date>a_start_date)
				then
					if Result=Void
					then
						create Result.make_empty
					end

					Result.extend(l_attribute_array.item(i))
				end

				i := i + 1
			end
		end

		if Result/=Void
		then
			attribute_by_entity_cache_hits := attribute_by_entity_cache_hits + 1
		else
			attribute_by_entity_cache_misses := attribute_by_entity_cache_misses + 1
		end
	end

   attribute_history (a_supplier_entity: SUPPLIER_ENTITY ;
					  a_attribute_type_id: INTEGER): EXTENDABLE_BOUNDED_ARRAY [SUPPLIER_ENTITY_ATTRIBUTE[G]] is
			-- Attribute History of an entity
      require
		 a_supplier_entity_not_void: a_supplier_entity /= Void
		 a_attribute_type_id_not_zero: a_attribute_type_id /= 0
		 category_is_cached: has_category (a_supplier_entity.category_id)
      local
		 l_key: KEY_PAIR [INTEGER, INTEGER]
      do
		 create l_key.make (a_supplier_entity.entity_id, a_attribute_type_id)
		 attribute_by_entity_cache.search (l_key)
		 if attribute_by_entity_cache.found then
			Result := attribute_by_entity_cache.found_item
			attribute_by_entity_cache_hits := attribute_by_entity_cache_hits + 1
		 else
			attribute_by_entity_cache_misses := attribute_by_entity_cache_misses + 1
		 end
      end

   next_attribute (a_supplier_entity: SUPPLIER_ENTITY ;
				   a_attribute_type_id: INTEGER; a_date: ROSE_DATE): SUPPLIER_ENTITY_ATTRIBUTE[G] is
      require
		 a_supplier_entity_not_void: a_supplier_entity /= Void
		 a_attribute_type_id_not_zero: a_attribute_type_id /= 0
		 category_is_cached: has_category (a_supplier_entity.category_id)
		 a_date_not_void: a_date /= Void
      local
		 l_key: KEY_PAIR [INTEGER, INTEGER]
		 l_attribute_array: EXTENDABLE_BOUNDED_ARRAY [SUPPLIER_ENTITY_ATTRIBUTE[G]]
		 i: INTEGER
		 l_date: ROSE_DATE
      do
		 create l_key.make (a_supplier_entity.entity_id, a_attribute_type_id)
		 attribute_by_entity_cache.search (l_key)
		 if attribute_by_entity_cache.found then
			l_attribute_array := attribute_by_entity_cache.found_item
			l_date := a_date.not_max
			from
			   i := l_attribute_array.lower
			until
			   i > l_attribute_array.upper or Result/=Void
			loop
			   if l_attribute_array.item (i).end_date > l_date then
				  Result := l_attribute_array.item (i)
			   end
			   i := i + 1
			end
		 end

		 if Result/=Void
		  then
			attribute_by_entity_cache_hits := attribute_by_entity_cache_hits + 1
		 else
			attribute_by_entity_cache_misses := attribute_by_entity_cache_misses + 1
		 end
      end

   entity_start (a_category_id: INTEGER): G is
      require
		 a_category_id_not_zero: a_category_id /= 0
      local
		 i: INTEGER
		 l_found: BOOLEAN
      do
		 entity_exhausted_cache.force (True, a_category_id)
		 from
			i := sorted_entity_cache.lower
			l_found := false
		 until
			i > sorted_entity_cache.upper or l_found
		 loop
			if sorted_entity_cache.item (i).category_id = a_category_id then
			   l_found := true
			   entity_cursor_cache.force (i, a_category_id)
			   entity_exhausted_cache.force (False, a_category_id)
			end
			i := i + 1
		 end
		 Result := sorted_entity_cache.item (entity_cursor_cache.item (a_category_id))
      end

feature {SUPPLIER_ENTITY} -- Cursor movement

   entity_forth (a_category_id: INTEGER) is
		 -- Move to the next entity (according to entity_id).
      require
		 a_category_id_not_zero: a_category_id /= 0
      local
		 l_cursor: INTEGER
      do
		 l_cursor := entity_cursor_cache.item (a_category_id)
		 l_cursor := l_cursor + 1
		 entity_cursor_cache.replace (l_cursor, a_category_id)
		 if l_cursor > sorted_entity_cache.upper or else
			sorted_entity_cache.item (l_cursor).category_id /= a_category_id then
			entity_exhausted_cache.force (True, a_category_id)
		 end
      end

feature {SUPPLIER_ENTITY} -- Status report

   updated: BOOLEAN
		 -- Cache is set to not updated initially and is set to updated as soon as the cache is changed.
		 -- Cursoring is currently supported only when the cache has not been updated.

   entity_exhausted (a_category_id: INTEGER): BOOLEAN is
      require
		 a_category_id_not_zero: a_category_id /= 0
      do
		 Result := entity_exhausted_cache.item (a_category_id)
      end

feature {SUPPLIER_ENTITY,SUPPLIER_ENTITY_CACHE} -- Basic operations

   add_entity (a_entity: G) is
      require
		 a_entity_not_void: a_entity /= Void
		 category_is_cached: has_category (a_entity.category_id)
      local
		 l_entity_array: EXTENDABLE_BOUNDED_ARRAY [G]
		 l_entity: G
      do
		 updated := True
		 entity_cache.search (a_entity.key)
		 if entity_cache.found then
			l_entity_array := entity_cache.found_item
			l_entity := l_entity_array.item (l_entity_array.count)
			if l_entity.end_date < Maximum_date_time then
			   l_entity.set_end_date (a_entity.start_date)
			   l_entity_array.extend (a_entity)
			else
			   error_log.set_severity (error_log.Fatal)
			   error_log.set_message ("ERROR: Non unique supplier_entity entity key (catid, entid) ...: ")
			   error_log.append_message (a_entity.category_id.out)
			   error_log.append_message (" ")
			   error_log.append_message (a_entity.entity_id.out)
			   error_log.set_class_name ("SUPPLIER_ENTITY_CACHE")
			   error_log.set_feature_name ("add_entity")
			   error_log.put
			end
			entity_cache_hits := entity_cache_hits + 1
		 else
			entity_cache_misses := entity_cache_misses + 1
			create l_entity_array.make (1, 1)
			l_entity_array.put (a_entity, 1)
			entity_cache.put (l_entity_array, a_entity.key)
		 end
      end

   terminate_entity (a_entity: G; a_end_date: ROSE_DATE) is
      require
		 a_entity_not_void: a_entity /= Void
		 category_is_cached: has_category (a_entity.category_id)
      local
		 l_entity: G
		 l_attribute: SUPPLIER_ENTITY_ATTRIBUTE[G]
      do
		 updated := True
		 entity_cache.search (a_entity.key)
		 if entity_cache.found then
			l_entity := entity_cache.found_item.item (entity_cache.found_item.count)
			l_entity.set_end_date (a_end_date)
			entity_cache_hits := entity_cache_hits + 1
		 else
			entity_cache_misses := entity_cache_misses + 1
		 end

		 from
			attribute_by_entity_cache.start
		 until
			attribute_by_entity_cache.off
		 loop
			if attribute_by_entity_cache.item_for_iteration.item (1).category_id = a_entity.category_id and
			   attribute_by_entity_cache.item_for_iteration.item (1).entity_id = a_entity.entity_id
			 then
			   l_attribute := attribute_by_entity_cache.item_for_iteration.item
			   (attribute_by_entity_cache.item_for_iteration.count)
			   terminate_attribute (l_attribute, a_end_date)
			end
			attribute_by_entity_cache.forth
		 end

		 --   This is pointless, since we just terminated all potential attributes by going through the
		 --   attribute_by_entity_cache.  Since both caches point to the same set of attributes
		 --   we can stop after the first one.  In any case, the current code has a bug in that it
		 --   terminates the last attribute of the array, but that's probably NOT the one with the
		 --   proper id's.  It is not clear just what the following should be doing even if we needed it.
		 --
		 --	 from
		 --	    attribute_by_value_cache.start
		 --	 until
		 --	    attribute_by_value_cache.off
		 --	 loop
		 --	    if attribute_by_value_cache.item_for_iteration.item (1).category_id = a_entity.category_id and
		 --	       attribute_by_value_cache.item_for_iteration.item (1).entity_id = a_entity.entity_id
		 --	    then
		 --	       l_attribute := attribute_by_value_cache.item_for_iteration.item
		 --	       (attribute_by_value_cache.item_for_iteration.count)
		 --	       terminate_attribute (l_attribute, a_end_date)
		 --	    end
		 --	    attribute_by_value_cache.forth
		 --	 end
      end

   add_attribute (a_attribute: SUPPLIER_ENTITY_ATTRIBUTE[G]) is
      require
		 a_attribute_not_void: a_attribute /= Void
		 category_is_cached: has_category (a_attribute.category_id)
      local
		 l_attribute_array: EXTENDABLE_BOUNDED_ARRAY [SUPPLIER_ENTITY_ATTRIBUTE[G]]
		 l_attribute: SUPPLIER_ENTITY_ATTRIBUTE[G]
      do
		 updated := True
		 attribute_by_entity_cache.search (a_attribute.entity_key)
		 if attribute_by_entity_cache.found then
			l_attribute_array := attribute_by_entity_cache.found_item
			l_attribute := l_attribute_array.item (l_attribute_array.count)
			if l_attribute.end_date < Maximum_date_time then
			   l_attribute.set_end_date (a_attribute.start_date)
			   l_attribute_array.extend (a_attribute)
			else
			   error_log.set_severity (error_log.Fatal)
			   error_log.set_message ("ERROR: Non unique supplier_attribute entity key (catid, entid, attid) ...: ")
			   error_log.append_message (a_attribute.category_id.out)
			   error_log.append_message (" ")
			   error_log.append_message (a_attribute.entity_id.out)
			   error_log.append_message (" ")
			   error_log.append_message (a_attribute.attribute_type_id.out)
			   error_log.set_class_name ("SUPPLIER_ENTITY_CACHE")
			   error_log.set_feature_name ("add_attribute")
			   error_log.put
			end
			attribute_by_entity_cache_hits := attribute_by_entity_cache_hits + 1
		 else
			attribute_by_entity_cache_misses := attribute_by_entity_cache_misses + 1
			create l_attribute_array.make (1, 1)
			l_attribute_array.put (a_attribute, 1)
			attribute_by_entity_cache.put (l_attribute_array, a_attribute.entity_key)
		 end

		 attribute_by_value_cache.search (a_attribute.value_key)
		 if attribute_by_value_cache.found then
			l_attribute_array := attribute_by_value_cache.found_item
			l_attribute := l_attribute_array.item (l_attribute_array.count)
			if l_attribute.attribute_type_manager.attribute_type_is_unique_identifier (a_attribute.attribute_type_id) then
			   l_attribute.set_end_date (a_attribute.start_date)
			end
			l_attribute_array.extend (a_attribute)
			attribute_by_value_cache_hits := attribute_by_value_cache_hits + 1
		 else
			attribute_by_value_cache_misses := attribute_by_value_cache_misses + 1
			create l_attribute_array.make (1, 1)
			l_attribute_array.put (a_attribute, 1)
			attribute_by_value_cache.put (l_attribute_array, a_attribute.value_key)
		 end
      end

   terminate_attribute (a_attribute: SUPPLIER_ENTITY_ATTRIBUTE[G]; a_end_date: ROSE_DATE) is
      require
		 a_attribute_not_void: a_attribute /= Void
		 category_is_cached: has_category (a_attribute.category_id)
      local
		 l_attribute: SUPPLIER_ENTITY_ATTRIBUTE[G]
      do
		 updated := True
		 attribute_by_entity_cache.search (a_attribute.entity_key)
		 if attribute_by_entity_cache.found then
			l_attribute := attribute_by_entity_cache.found_item.item (attribute_by_entity_cache.found_item.count)
			l_attribute.set_end_date (a_end_date)
			attribute_by_entity_cache_hits := attribute_by_entity_cache_hits + 1
		 else
			attribute_by_entity_cache_misses := attribute_by_entity_cache_misses + 1
		 end

		 --   This is pointless, since we just terminated all potential attributes by going through the
		 --   attribute_by_entity_cache.  Since both caches point to the same set of attributes
		 --   we can stop after the first one.  This code is probably right (I think) but we
		 --   don't need it.
		 --	 attribute_by_value_cache.search (a_attribute.value_key)
		 --	 if attribute_by_value_cache.found then
		 --	    l_attribute_array := attribute_by_value_cache.found_item
		 --	    from
		 --	       i := l_attribute_array.upper
		 --	       l_found := false
		 --	    until
		 --	       i < l_attribute_array.lower or l_found
		 --	    loop
		 --	       if l_attribute_array.item (i).entity_id = a_attribute.entity_id and then
		 --		  l_attribute_array.item (i).end_date >= Maximum_date_time
		 --             then
		 --		  l_attribute := l_attribute_array.item (i)
		 --		  l_attribute.set_end_date (a_end_date)
		 --		  l_found := True
		 --	       end
		 --	       i := i - 1
		 --	    end
		 --	 end
		 --
		 --	 if l_found
		 --	 then
		 --	    attribute_by_value_cache_hits := attribute_by_value_cache_hits + 1
		 --	 else
		 --	    attribute_by_value_cache_misses := attribute_by_value_cache_misses + 1
		 --	 end
      end


feature {SUPPLIER_ENTITY, SUPPLIER_ENTITY_CACHE} -- Status report

   has_category (a_category_id: INTEGER): BOOLEAN is
      do
		 Result := id_table = Void or else id_table.has (a_category_id)
      end

   has_category_and_attribute (a_category_id: INTEGER; a_attribute_type_id: INTEGER): BOOLEAN is
      local
		 l_attribute_type_id_table: HASH_TABLE [INTEGER, INTEGER]
      do
		 if id_table = Void then
			Result := True
		 else
			id_table.search (a_category_id)
			if id_table.found then
			   l_attribute_type_id_table := id_table.found_item
			   if l_attribute_type_id_table = Void or else
				  l_attribute_type_id_table.count = 0 then
				  Result := True
			   else
				  l_attribute_type_id_table.search (a_attribute_type_id)
				  Result := l_attribute_type_id_table.found
			   end
			else
			   Result := False
			end
		 end
      end

feature {SUPPLIER_ENTITY_CACHE, SUPPLIER_ENTITY} -- Implementation

	compare_by_value_key (a, b: SUPPLIER_ENTITY_ATTRIBUTE [G]): BOOLEAN is
			-- Order by category_id, attribute_type_id, attribute_value, end_date
		do
			if a.category_id < b.category_id then
				Result := True
			elseif a.category_id = b.category_id then
				if a.attribute_type_id < b.attribute_type_id then
					Result := True
				elseif a.attribute_type_id = b.attribute_type_id then
					if a.attribute_value < b.attribute_value then
						Result := True
					elseif a.attribute_value.is_equal (b.attribute_value) then
						if a.end_date < b.end_date then
							Result := True
						else
							Result := False
						end
					else
						Result := False
					end
				else
					Result := False
				end
			else
				Result := False
			end
		end

feature {SUPPLIER_ENTITY} -- Implementation

   sorted_entity_cache: DYNAMICALLY_SORTABLE_ARRAY [G]

   id_table: HASH_TABLE [HASH_TABLE [INTEGER, INTEGER], INTEGER]

   entity_cache: HASH_TABLE [EXTENDABLE_BOUNDED_ARRAY [G], KEY_PAIR [INTEGER, INTEGER]]

   attribute_by_entity_cache: HASH_TABLE [EXTENDABLE_BOUNDED_ARRAY [SUPPLIER_ENTITY_ATTRIBUTE[G]], KEY_PAIR [INTEGER, INTEGER]]

   attribute_by_value_cache: HASH_TABLE [EXTENDABLE_BOUNDED_ARRAY [SUPPLIER_ENTITY_ATTRIBUTE[G]], MULTI_KEY]

   entity_cursor_cache: HASH_TABLE [INTEGER, INTEGER] is
      once
		 create Result.make (0)
      end

   entity_exhausted_cache: HASH_TABLE [BOOLEAN, INTEGER] is
      once
		 create Result.make (0)
      end


feature {ENTITY_SERVER} -- Implementation
   select_entity_statement: STRING

   select_entity_predicate: COLLECTING_PREDICATE [G]

   make_select_entity_predicate: COLLECTING_PREDICATE [G] is
      do
		 create Result.make (data_server, select_entity_statement)
		 if not Result.success then
			error_log.set_severity (error_log.Fatal)
			error_log.set_message ("ERROR: Could not make predicate")
			error_log.set_class_name ("SUPPLIER_ENTITY_CACHE")
			error_log.set_feature_name ("make_select_entity_predicate")
			error_log.put
		 end
      end

   select_attribute_statement: STRING

   select_attribute_predicate: COLLECTING_PREDICATE [SUPPLIER_ENTITY_ATTRIBUTE[G]]

   make_select_attribute_predicate: COLLECTING_PREDICATE [SUPPLIER_ENTITY_ATTRIBUTE[G]] is
      do
		 create Result.make (data_server, select_attribute_statement)
		 if not Result.success then
			error_log.set_severity (error_log.Fatal)
			error_log.set_message ("ERROR: Could not make predicate")
			error_log.set_class_name ("SUPPLIER_ATTRIBUTE_CACHE")
			error_log.set_feature_name ("make_select_attribute_predicate")
			error_log.put
		 end
      end

   Maximum_date_time: ROSE_DATE is
      once
		 create Result.make_max_date
      end

   make_empty_entity: G is
      do
		 create Result
      end

   make_empty_attribute: SUPPLIER_ENTITY_ATTRIBUTE[G] is
      do
		 create Result
      end

end
