indexing
   
   description: "Supplier Entity Relationship."
   status: "Development"
   author: "David Stevens"
   date: "$Date: 2008/12/01 $"
   revision: "$Revision: #24 $"
   
class
   
   SUPPLIER_ENTITY_RELATIONSHIP
   
inherit
   ANY
   MISSING_PARAMETERS
   ROSE_CONSTANTS
   ERROR_LOG_USE
   
create
   
   make,
   make_empty,
   default_create
   
feature {SUPPLIER_ENTITY, SUPPLIER_ENTITY_RELATIONSHIP_CACHE} -- Initialization
   
   make (a_relationship_type: SUPPLIER_RELATIONSHIP_TYPE;
	 a_entity_1: SUPPLIER_ENTITY;
	 a_entity_2: SUPPLIER_ENTITY;
	 a_start_date: ROSE_DATE;
	 a_weight: DOUBLE) is
      require
	 a_relationship_type_not_void: a_relationship_type /= Void
	 a_entity_1_not_void: a_entity_1 /= Void
	 a_entity_2_not_void: a_entity_2 /= Void
	 a_start_date_not_void: a_start_date /= Void
	 --a_entity_1_end_date_is_max: a_entity_1.end_date.is_equal(Maximum_date_time)
	 --a_entity_2_end_date_is_max: a_entity_2.end_date.is_equal(Maximum_date_time)
      do
	 relationship_type_id := a_relationship_type.relationship_type_id
	 if a_entity_1.category_id = a_relationship_type.category_1_id and then
		a_entity_2.category_id = a_relationship_type.category_2_id then
		-- Like-to-like relationships go here.
	    entity_1_id := a_entity_1.entity_id
	    entity_2_id := a_entity_2.entity_id
	 elseif a_entity_1.category_id = a_relationship_type.category_2_id and then
		    a_entity_2.category_id = a_relationship_type.category_1_id then
	    entity_1_id := a_entity_2.entity_id
	    entity_2_id := a_entity_1.entity_id
	 else
		error_log.set_severity (error_log.fatal)
		error_log.set_message ("Relationship doesn't match the entity categories (rel_type_id, cat1, cat2): ")
		error_log.append_message(a_relationship_type.relationship_type_id.out)
		error_log.append_message(" ")
		error_log.append_message(a_entity_1.category_id.out)
		error_log.append_message(" ")
		error_log.append_message(a_entity_2.category_id.out)
		error_log.set_class_name ("SUPPLIER_ENTITY_RELATIONSHIP")
		error_log.set_feature_name ("make")
		error_log.put
	 end
	 start_date := a_start_date
	 weight := a_weight
	 end_date := constants.Maximum_date_time
      end
   make_empty is
      do
      end
   
feature {SUPPLIER_ENTITY, SUPPLIER_ENTITY_RELATIONSHIP_CACHE} -- Access

   relationship_id: INTEGER

   relationship_type_id: INTEGER
   
   entity_1_id: INTEGER
   
   entity_2_id: INTEGER
   
   start_date: ROSE_DATE
   
   end_date: ROSE_DATE
   
   update_date: ROSE_DATE
  
   weight : DOUBLE
   
feature {SUPPLIER_ENTITY, SUPPLIER_ENTITY_RELATIONSHIP_CACHE}

	set_relationship_id(a_relationship_id: INTEGER) is
	do
		relationship_id := a_relationship_id
	end

   set_end_date (a_value: like end_date) is
      do
	 end_date := a_value
      end
      
feature {SUPPLIER_ENTITY_RELATIONSHIP_CACHE}
   
   entity_1_to_entity_2_key: KEY_PAIR [INTEGER, INTEGER] is
      do
	 create Result.make (relationship_type_id, entity_1_id)
      end
   
   entity_2_to_entity_1_key: KEY_PAIR [INTEGER, INTEGER] is
      do
	 create Result.make (relationship_type_id, entity_2_id)
      end
   
end -- class SUPPLIER_ENTITY_RELATIONSHIP
