indexing

   description: "Supplier entity relationship cache."
   status: "Development"
   author: "David Stevens"
   date: "$Date: 2014/02/26 $"
   revision: "$Revision: #38 $"

class

   SUPPLIER_ENTITY_RELATIONSHIP_CACHE [G -> SUPPLIER_ENTITY  create default_create end]

inherit

   MISSING_PARAMETERS

   DATABASE_SERVER

   ERROR_LOG_USE

   SUPPLIER_ENTITY_CONSTANTS
	ROSE_CONSTANTS
create
	make

feature -- Initialization
   make (a_relationship_type_id_set: ARRAY [INTEGER]) is
      local
		 l_relationship_array: EXTENDABLE_BOUNDED_ARRAY [SUPPLIER_ENTITY_RELATIONSHIP]
		 i, j, k: INTEGER
		 l_relationship_type_id: INTEGER
		 l_entity_1_count_array: EXTENDABLE_BOUNDED_ARRAY [INTEGER]
		 l_entity_2_count_array: EXTENDABLE_BOUNDED_ARRAY [INTEGER]
		 l_relationships: EXTENDABLE_BOUNDED_ARRAY [SUPPLIER_ENTITY_RELATIONSHIP]
		 l_temp_cache: like Current
		 l_entity_1_count, l_entity_2_count, l_total_count: INTEGER
		 l_entity: G
		 l_key: KEY_PAIR [INTEGER, INTEGER]
      do
		 l_entity := make_empty_entity
		 server ?= l_entity.server
		 -- Loop through the relationships.
		 select_relationship_1_statement := "select * from entity_relationship where "
		 if a_relationship_type_id_set /= Void
		 then
			create relationship_type_id_cache.make(a_relationship_type_id_set.count)
			from
			   i := 1
			until
			   i > a_relationship_type_id_set.count
			loop
			   l_relationship_type_id := a_relationship_type_id_set.item (i)
			   relationship_type_id_cache.put(l_relationship_type_id, l_relationship_type_id)
			   if relationship_type_id_cache.conflict
			   then
				  error_log.set_severity (error_log.Fatal)
				  error_log.set_message ("FATAL: Duplicate relationship type id.")
				  error_log.set_class_name ("SUPPLIER_ENTITY_RELATIONSHIP_CACHE")
				  error_log.set_feature_name ("make")
				  error_log.put
			   else
			      if i = 1 then
				     select_relationship_1_statement.append ("(")
			      end
			      select_relationship_1_statement.append ("relationship_type_id = ")
			      select_relationship_1_statement.append (l_relationship_type_id.out)
			      if i = a_relationship_type_id_set.count then
				     select_relationship_1_statement.append (") and ")
			      else
				     select_relationship_1_statement.append (" or ")
			      end
			   end

			   i := i + 1
			end
		 end
		 select_relationship_1_statement.append (" end_date = '")
		 select_relationship_1_statement.append (constants.Maximum_date_time.date.ccyymmdd.out)
		 select_relationship_1_statement.append ("' order by relationship_type_id, entity_1_id")
		 select_relationship_1_predicate := make_select_relationship_1_predicate

		 -- Retrieve the relationships.
		 create l_relationship_array.make_empty
		 select_relationship_1_predicate.set_container (l_relationship_array)
		 select_relationship_1_predicate.retrieve (Void, Void)
		 if not select_relationship_1_predicate.success then
			error_log.set_severity (error_log.Fatal)
			error_log.set_message ("ERROR: Unable to retrieve from entity_relationship")
			error_log.set_class_name ("SUPPLIER_ENTITY_RELATIONSHIP_CACHE")
			error_log.set_feature_name ("make")
			error_log.put
		 end

		 -- Get the counts.
		 l_entity_1_count := 0
		 create l_entity_2_count_array.make_empty
		 from
			i := 1
			l_entity_2_count := 1
		 until
			i > l_relationship_array.count
		 loop
			if (i = l_relationship_array.count) or
			   (l_relationship_array.item (i).relationship_type_id /=
				l_relationship_array.item (i + 1).relationship_type_id) or
			(l_relationship_array.item (i).entity_1_id   /= l_relationship_array.item (i + 1).entity_1_id) then
			l_entity_1_count := l_entity_1_count + 1
			l_entity_2_count_array.force (l_entity_2_count, l_entity_1_count)
			l_entity_2_count := 1
			else
			   l_entity_2_count := l_entity_2_count + 1
			end
			i := i + 1
		 end

		 -- Build the entity_1 cache.
		 create entity_1_to_entity_2_cache.make (l_entity_1_count)
		 from
			i := 1
			j := 1
		 until
			i > l_entity_1_count
		 loop
			create l_relationships.make (1, l_entity_2_count_array.item (i))
			from
			   k := 1
			until
			   k > l_entity_2_count_array.item (i)
			loop
			   l_relationships.put (l_relationship_array.item (j), k)
			   j := j + 1
			   k := k + 1
			end
			entity_1_to_entity_2_cache.put (l_relationships, l_relationships.item (1).entity_1_to_entity_2_key)
			if entity_1_to_entity_2_cache.conflict then
			   error_log.set_severity (error_log.Fatal)
			   error_log.set_message ("ERROR: Non unique entity_1 to entity_2 relationship key (")
			   error_log.append_message ("rel_type, ent_1, ent_2) ...: ")
			   error_log.append_message (l_relationships.item (1).relationship_type_id.out)
			   error_log.append_message (" ")
			   error_log.append_message (l_relationships.item (1).entity_1_id.out)
			   error_log.append_message (" ")
			   error_log.append_message (l_relationships.item (1).entity_2_id.out)
			   error_log.set_class_name ("SUPPLIER_ENTITY_RELATIONSHIP_CACHE")
			   error_log.set_feature_name ("make")
			   error_log.put
			end
			i := i + 1
		 end

		 -- Double check the entity_1 cache.
		 l_total_count := 0
		 from
			entity_1_to_entity_2_cache.start
		 until
			entity_1_to_entity_2_cache.off
		 loop
			l_relationships := entity_1_to_entity_2_cache.item_for_iteration
			l_key := entity_1_to_entity_2_cache.key_for_iteration
			from
			   i := 1
			until
			   i > l_relationships.count
			loop
			   if not l_relationships.item (i).entity_1_to_entity_2_key.is_equal (l_key) then
				  error_log.set_severity (error_log.Fatal)
				  error_log.set_message ("ERROR: Entity_2 key does not match cache key (")
				  error_log.append_message ("rel_type, ent_1, ent_2) ...: ")
				  error_log.append_message (l_relationships.item (1).relationship_type_id.out)
				  error_log.append_message (" ")
				  error_log.append_message (l_relationships.item (1).entity_1_id.out)
				  error_log.append_message (" ")
				  error_log.append_message (l_relationships.item (1).entity_2_id.out)
				  error_log.set_class_name ("SUPPLIER_ENTITY_RELATIONSHIP_CACHE")
				  error_log.set_feature_name ("make")
				  error_log.put
			   end
			   l_total_count := l_total_count + 1
			   i := i + 1
			end
			entity_1_to_entity_2_cache.forth
		 end
		 if l_total_count /= l_relationship_array.count then
			error_log.set_severity (error_log.Fatal)
			error_log.set_message ("ERROR: Incorrect determination of relationships counts.")
			error_log.set_class_name ("SUPPLIER_ENTITY_RELATIONSHIP_CACHE")
			error_log.set_feature_name ("make")
			error_log.put
		 end

		 -- Loop through the relationships.
		 select_relationship_2_statement := "select * from entity_relationship where "
		 if a_relationship_type_id_set /= Void then
			from
			   i := 1
			until
			   i > a_relationship_type_id_set.count
			loop
			   l_relationship_type_id := a_relationship_type_id_set.item (i)
			   if i = 1 then
				  select_relationship_2_statement.append ("(")
			   end
			   select_relationship_2_statement.append ("relationship_type_id = ")
			   select_relationship_2_statement.append (l_relationship_type_id.out)
			   if i = a_relationship_type_id_set.count then
				  select_relationship_2_statement.append (") and ")
			   else
				  select_relationship_2_statement.append (" or ")
			   end
			   i := i + 1
			end
		 end
		 select_relationship_2_statement.append (" end_date = '")
		 select_relationship_2_statement.append (constants.Maximum_date_time.date.ccyymmdd.out)
		 select_relationship_2_statement.append ("' order by relationship_type_id, entity_2_id")

		 -- Retrieve the relationships.
		 select_relationship_2_predicate := make_select_relationship_2_predicate
		 create l_relationship_array.make_empty
		 select_relationship_2_predicate.set_container (l_relationship_array)
		 select_relationship_2_predicate.retrieve (Void, Void)
		 if not select_relationship_2_predicate.success then
			error_log.set_severity (error_log.Fatal)
			error_log.set_message ("ERROR: Unable to retrieve from entity_relationship")
			error_log.set_class_name ("SUPPLIER_ENTITY_RELATIONSHIP_CACHE")
			error_log.set_feature_name ("make")
			error_log.put
		 end

		 -- Get the counts.
		 l_entity_2_count := 0
		 create l_entity_1_count_array.make_empty
		 from
			i := 1
			l_entity_1_count := 1
		 until
			i > l_relationship_array.count
		 loop
			if (i = l_relationship_array.count) or
			   (l_relationship_array.item (i).relationship_type_id /=
				l_relationship_array.item (i + 1).relationship_type_id) or
			(l_relationship_array.item (i).entity_2_id   /= l_relationship_array.item (i + 1).entity_2_id) then
			l_entity_2_count := l_entity_2_count + 1
			l_entity_1_count_array.force (l_entity_1_count, l_entity_2_count)
			l_entity_1_count := 1
			else
			   l_entity_1_count := l_entity_1_count + 1
			end
			i := i + 1
		 end

		 -- Build the entity_2 cache.
		 create entity_2_to_entity_1_cache.make (l_entity_2_count)
		 from
			i := 1
			j := 1
		 until
			i > l_entity_2_count
		 loop
			create l_relationships.make (1, l_entity_1_count_array.item (i))
			from
			   k := 1
			until
			   k > l_entity_1_count_array.item (i)
			loop
			   l_relationships.put (l_relationship_array.item (j), k)
			   j := j + 1
			   k := k + 1
			end
			entity_2_to_entity_1_cache.put (l_relationships, l_relationships.item (1).entity_2_to_entity_1_key)
			if entity_2_to_entity_1_cache.conflict then
			   error_log.set_severity (error_log.Fatal)
			   error_log.set_message ("ERROR: Non unique entity_2 to entity_1 relationship key (")
			   error_log.append_message ("rel_type, ent_2, ent_1) ...: ")
			   error_log.append_message (l_relationships.item (1).relationship_type_id.out)
			   error_log.append_message (" ")
			   error_log.append_message (l_relationships.item (1).entity_2_id.out)
			   error_log.append_message (" ")
			   error_log.append_message (l_relationships.item (1).entity_1_id.out)
			   error_log.set_class_name ("SUPPLIER_ENTITY_RELATIONSHIP_CACHE")
			   error_log.set_feature_name ("make")
			   error_log.put
			end
			i := i + 1
		 end

		 -- Double check the entity_2 cache.
		 l_total_count := 0
		 from
			entity_2_to_entity_1_cache.start
		 until
			entity_2_to_entity_1_cache.off
		 loop
			l_relationships := entity_2_to_entity_1_cache.item_for_iteration
			l_key := entity_2_to_entity_1_cache.key_for_iteration
			from
			   i := 1
			until
			   i > l_relationships.count
			loop
			   if not l_relationships.item (i).entity_2_to_entity_1_key.is_equal (l_key) then
				  error_log.set_severity (error_log.Fatal)
				  error_log.set_message ("ERROR: Entity_1 key does not match cache key (")
				  error_log.append_message ("rel_type, ent_2, cat_1, ent_1) ...: ")
				  error_log.append_message (l_relationships.item (1).relationship_type_id.out)
				  error_log.append_message (" ")
				  error_log.append_message (l_relationships.item (1).entity_2_id.out)
				  error_log.append_message (" ")
				  error_log.append_message (l_relationships.item (1).entity_1_id.out)
				  error_log.set_class_name ("SUPPLIER_ENTITY_RELATIONSHIP_CACHE")
				  error_log.set_feature_name ("make")
				  error_log.put
			   end
			   l_total_count := l_total_count + 1
			   i := i + 1
			end
			entity_2_to_entity_1_cache.forth
		 end
		 if l_total_count /= l_relationship_array.count then
			error_log.set_severity (error_log.Fatal)
			error_log.set_message ("ERROR: Incorrect determination of relationships counts.")
			error_log.set_class_name ("SUPPLIER_ENTITY_RELATIONSHIP_CACHE")
			error_log.set_feature_name ("make")
			error_log.put
		 end

		 -- Set the supplier entity's relationship cache.
		 l_temp_cache ?= l_entity.set_relationship_cache (Current)
		 if l_temp_cache = Void then
			error_log.set_severity (error_log.Fatal)
			error_log.set_message ("ERROR: Reverse assignment failed")
			error_log.set_class_name ("SUPPLIER_ENTITY_RELATIONSHIP_CACHE")
			error_log.set_feature_name ("make")
			error_log.put
		 end
      end

   count: INTEGER is
	  do
		 Result := entity_1_to_entity_2_cache.count
	  end

   hit_ratio: DOUBLE is
	  do
		 Result := hits / (hits + misses)
	  end

   hits: INTEGER

   misses: INTEGER

feature {SUPPLIER_ENTITY} -- Access

   directed_relationships (a_relationship_type: SUPPLIER_RELATIONSHIP_TYPE; a_entity: G; a_relationship_style: BOOLEAN):
      EXTENDABLE_BOUNDED_ARRAY [SUPPLIER_ENTITY_RELATIONSHIP] is
      require
		 a_relationship_type_not_void: a_relationship_type /= Void
		 a_entity_not_void: a_entity /= Void
		 as_of_date_is_max: a_entity.as_of_date.is_equal(constants.Maximum_date_time)
      local
		 l_key: KEY_PAIR [INTEGER, INTEGER]
      do
		 create l_key.make (a_relationship_type.relationship_type_id, a_entity.entity_id)
		 if a_relationship_style=Right_relative and then a_entity.category_id=a_relationship_type.category_1_id then
			Result := entity_1_to_entity_2_cache.item (l_key)
		 elseif a_relationship_style=Left_relative and then a_entity.category_id=a_relationship_type.category_2_id then
			Result := entity_2_to_entity_1_cache.item (l_key)
		 else
			error_log.set_severity (error_log.Fatal)
			error_log.set_message ("ERROR: Invalid relationship_type for this entity (rel_type, cat_id, ent_id) ")
			error_log.append_message (a_relationship_type.relationship_type_id.out)
			error_log.append_message (", ")
			error_log.append_message (a_entity.category_id.out)
			error_log.append_message (", ")
			error_log.append_message (a_entity.entity_id.out)
			error_log.set_class_name ("SUPPLIER_ENTITY_RELATIONSHIP_CACHE")
			error_log.set_feature_name ("directed_relationships")
			error_log.put
		 end

         if Result /= Void
		  then
			hits := hits + 1
		 else
			misses := misses + 1
		 end
      end

   all_relationships (a_entity: G): EXTENDABLE_BOUNDED_ARRAY [SUPPLIER_ENTITY_RELATIONSHIP] is
      require
		 a_entity_not_void: a_entity /= Void
		 as_of_date_is_max: a_entity.as_of_date.is_equal(constants.Maximum_date_time)
      local
		 l_array: EXTENDABLE_BOUNDED_ARRAY [SUPPLIER_ENTITY_RELATIONSHIP]
		 i: INTEGER
      do
		 create Result.make_empty
		 from
			entity_1_to_entity_2_cache.start
		 until
			entity_1_to_entity_2_cache.off
		 loop
			l_array := entity_1_to_entity_2_cache.item_for_iteration
			from
			   i := l_array.lower
			until
			   i > l_array.upper
			loop
			   if l_array.item (i).entity_1_id = a_entity.entity_id or
				  l_array.item (i).entity_2_id = a_entity.entity_id then
				  Result.extend (l_array.item (i))
			   end
			   i := i + 1
			end
			entity_1_to_entity_2_cache.forth
		 end

		 if Result.is_empty
		 then
			Result := Void
		 end
	  ensure
		 not_empty: Result=Void or else not Result.is_empty
      end

	relationship(a_relationship_type_id: INTEGER; a_entity_1, a_entity_2: G): SUPPLIER_ENTITY_RELATIONSHIP is
	require
		 as_of_date_1_is_max: a_entity_1.as_of_date.is_equal(constants.Maximum_date_time)
		 as_of_date_2_is_max: a_entity_2.as_of_date.is_equal(constants.Maximum_date_time)
	local
		i: INTEGER
		l_key: KEY_PAIR [INTEGER, INTEGER]
		l_array,
		l_array_1,
		l_array_2: EXTENDABLE_BOUNDED_ARRAY [SUPPLIER_ENTITY_RELATIONSHIP]
	do
		-- Either the 1-2 cache or the 2-1 cache may be used to find this relationship.
		-- However one of these may result in a very very large list to search.
		-- Think of the industry to company relationship.  Given one company and
		-- one industry we will get one very small list from one cache and one very
		-- large list from the other cache.  We will search the smaller of the two.

		-- Here is another gotcha (that actually happened).  Suppose that entity_1
		-- has a relationship with entity_3 and entity_2 has no relationships at
		-- all.  In the code below, l_array_1 will have data and l_array_2 will be
		-- Void.  If the relationship exists at all then BOTH caches are guaranteed
		-- to have the it.  If one or both caches returns Void then the desired
		-- relationship does not exist.

		-- Find all relationships involving a_entity_1.  If there aren't any then
		-- we are done.  Even if some relationships are returned, the relationship
		-- between 1 and 2 may not actually exist.
		create l_key.make (a_relationship_type_id, a_entity_1.entity_id)
		l_array_1 := entity_1_to_entity_2_cache.item (l_key)

		if l_array_1/=Void
		then
			-- Find all relationships involving a_entity_2.  If there aren't any then
			-- again, we are done.  Why?  Because l_array_1 contains matches against
			-- a_entity_1 but not one of them matched a_entity_2!  If it had then
			-- l_array_2 wouldn't be Void would it?
			l_key.make (a_relationship_type_id, a_entity_2.entity_id)
			l_array_2 := entity_2_to_entity_1_cache.item (l_key)

			if l_array_2/=Void
			then
				-- Since both arrays contain the relationship that we are looking for
				-- let's take the smaller.
				if l_array_1.count<l_array_2.count
				then
					l_array := l_array_1
				else
					l_array := l_array_2
				end

				-- Find the one (if any) that matches.
				from
					i := l_array.lower
				until
					Result/=Void or i>l_array.upper
				loop
					if l_array.item(i).entity_1_id=a_entity_1.entity_id and
					   l_array.item(i).entity_2_id=a_entity_2.entity_id
					then
						Result := l_array.item(i)
					else
						i := i + 1
					end
				end
			end
		end
	end

feature {SUPPLIER_ENTITY} -- Basic operations

   create_relationship (a_relationship_type: SUPPLIER_RELATIONSHIP_TYPE;
						a_entity_1, a_entity_2: G;
						a_start_date: ROSE_DATE;
						a_weight: DOUBLE) is
      require
		 a_relationship_type_not_void: a_relationship_type /= Void
		 a_entity_1_not_void: a_entity_1 /= Void
		 a_entity_2_not_void: a_entity_2 /= Void
		 a_start_date_not_void: a_start_date /= Void
		 -- both start dates must be max (see SUPPLIER_ENTITY_RELATIONSHIP:make).
      local
		 l_relationships: EXTENDABLE_BOUNDED_ARRAY [SUPPLIER_ENTITY_RELATIONSHIP]
		 l_relationship: SUPPLIER_ENTITY_RELATIONSHIP
		 l_direction: BOOLEAN
      do
		 -- Creating a relationship performs all of the validity checks.
		 create l_relationship.make (a_relationship_type, a_entity_1, a_entity_2, a_start_date, a_weight)

		 -- When a relationship is created, the categories may be swapped.
		 -- Find out which way we are doing it.
		 if l_relationship.entity_1_id=a_entity_1.entity_id then
			l_direction := Right_relative
		 else
			l_direction := Left_relative
		 end

		 l_relationships := directed_relationships (a_relationship_type, a_entity_1, l_direction)
		 if l_relationships = Void then
			create l_relationships.make_empty
			l_relationships.extend (l_relationship)
			if l_direction=Right_relative then
			   entity_1_to_entity_2_cache.put (l_relationships, l_relationships.item (1).entity_1_to_entity_2_key)
			else
			   entity_2_to_entity_1_cache.put (l_relationships, l_relationships.item (1).entity_2_to_entity_1_key)
			end
		 else
			l_relationships.extend (l_relationship)
		 end

		 -- Now store the relationship in the other direction.
		 l_direction := not l_direction
		 l_relationships := directed_relationships (a_relationship_type, a_entity_2, l_direction)
		 if l_relationships = Void then
			create l_relationships.make_empty
			l_relationships.extend (l_relationship)
			if l_direction=Right_relative then
			   entity_1_to_entity_2_cache.put (l_relationships, l_relationships.item (1).entity_1_to_entity_2_key)
			else
			   entity_2_to_entity_1_cache.put (l_relationships, l_relationships.item (1).entity_2_to_entity_1_key)
			end
		 else
			l_relationships.extend (l_relationship)
		 end
      end

   delete_relationship (a_relationship: SUPPLIER_ENTITY_RELATIONSHIP) is
      require
		 a_relationship_void: a_relationship /= Void
		 end_date_is_max: a_relationship.end_date.is_equal(constants.Maximum_date_time)
      local
		 l_relationships: EXTENDABLE_BOUNDED_ARRAY [SUPPLIER_ENTITY_RELATIONSHIP]
		 l_new_relationships: EXTENDABLE_BOUNDED_ARRAY [SUPPLIER_ENTITY_RELATIONSHIP]
		 i, l_old_count, l_new_count: INTEGER
      do

		 l_relationships := entity_1_to_entity_2_cache.item (a_relationship.entity_1_to_entity_2_key)
		 if l_relationships = Void then
			error_log.set_severity (error_log.Fatal)
			error_log.set_message ("ERROR: Attempt to delete non-existent relationship (")
			error_log.append_message ("rel_type, ent_1, ent_2) ...: ")
			error_log.append_message (a_relationship.relationship_type_id.out)
			error_log.append_message (" ")
			error_log.append_message (a_relationship.entity_1_id.out)
			error_log.append_message (" ")
			error_log.append_message (a_relationship.entity_2_id.out)
			error_log.set_class_name ("SUPPLIER_ENTITY_RELATIONSHIP_CACHE")
			error_log.set_feature_name ("delete_relationship")
			error_log.put
		 else
			l_old_count := l_relationships.count
			create l_new_relationships.make_empty
			from
			   i := 1
			until
			   i > l_relationships.count
			loop
			   if l_relationships.item (i).entity_2_id /= a_relationship.entity_2_id then
				  l_new_relationships.extend (l_relationships.item (i))
			   end
			   i := i + 1
			end
			l_new_count := l_new_relationships.count
			if l_new_count /= (l_old_count - 1) then
			   error_log.set_severity (error_log.Fatal)
			   error_log.set_message ("ERROR: Did not delete single entity_2 (")
			   error_log.append_message ("rel_type, ent_1, ent_2) ...: ")
			   error_log.append_message (a_relationship.relationship_type_id.out)
			   error_log.append_message (" ")
			   error_log.append_message (a_relationship.entity_1_id.out)
			   error_log.append_message (" ")
			   error_log.append_message (a_relationship.entity_2_id.out)
			   error_log.set_class_name ("SUPPLIER_ENTITY_RELATIONSHIP_CACHE")
			   error_log.set_feature_name ("delete_relationship")
			   error_log.put
			end
			if l_new_relationships.count > 0 then
			   entity_1_to_entity_2_cache.replace (l_new_relationships, a_relationship.entity_1_to_entity_2_key)
			else
			   entity_1_to_entity_2_cache.remove (a_relationship.entity_1_to_entity_2_key)
			end
		 end

		 l_relationships := entity_2_to_entity_1_cache.item (a_relationship.entity_2_to_entity_1_key)
		 if l_relationships = Void then
			error_log.set_severity (error_log.Fatal)
			error_log.set_message ("ERROR: Attempt to delete non-existent relationship (")
			error_log.append_message ("rel_type, ent_2, ent_1) ...: ")
			error_log.append_message (a_relationship.relationship_type_id.out)
			error_log.append_message (" ")
			error_log.append_message (a_relationship.entity_2_id.out)
			error_log.append_message (" ")
			error_log.append_message (a_relationship.entity_1_id.out)
			error_log.set_class_name ("SUPPLIER_ENTITY_RELATIONSHIP_CACHE")
			error_log.set_feature_name ("delete_relationship")
			error_log.put
		 else
			l_old_count := l_relationships.count
			create l_new_relationships.make_empty
			from
			   i := 1
			until
			   i > l_relationships.count
			loop
			   if l_relationships.item (i).entity_1_id /= a_relationship.entity_1_id then
				  l_new_relationships.extend (l_relationships.item (i))
			   end
			   i := i + 1
			end
			l_new_count := l_new_relationships.count
			if l_new_count /= (l_old_count - 1) then
			   error_log.set_severity (error_log.Fatal)
			   error_log.set_message ("ERROR: Did not delete single entity_1 (")
			   error_log.append_message ("rel_type, ent_2, ent_1) ...: ")
			   error_log.append_message (a_relationship.relationship_type_id.out)
			   error_log.append_message (" ")
			   error_log.append_message (a_relationship.entity_2_id.out)
			   error_log.append_message (" ")
			   error_log.append_message (a_relationship.entity_1_id.out)
			   error_log.set_class_name ("SUPPLIER_ENTITY_RELATIONSHIP_CACHE")
			   error_log.set_feature_name ("delete_relationship")
			   error_log.put
			end
			if l_new_relationships.count > 0 then
			   entity_2_to_entity_1_cache.replace (l_new_relationships, a_relationship.entity_2_to_entity_1_key)
			else
			   entity_2_to_entity_1_cache.remove (a_relationship.entity_2_to_entity_1_key)
			end
		 end
      end

feature {SUPPLIER_ENTITY} -- Status report

	all_relationships_are_cached: BOOLEAN is
	do
		Result := relationship_type_id_cache=Void
	end

	relationship_type_id_is_cached(a_relationship_type_id: INTEGER): BOOLEAN is
	do
		if relationship_type_id_cache=Void
		then
			Result := True
		else
			relationship_type_id_cache.search(a_relationship_type_id)
			Result := relationship_type_id_cache.found
		end
	end


feature {NONE} -- Implementation

   entity_1_to_entity_2_cache:
   HASH_TABLE [EXTENDABLE_BOUNDED_ARRAY [SUPPLIER_ENTITY_RELATIONSHIP], KEY_PAIR [INTEGER, INTEGER]]

   entity_2_to_entity_1_cache:
   HASH_TABLE [EXTENDABLE_BOUNDED_ARRAY [SUPPLIER_ENTITY_RELATIONSHIP], KEY_PAIR [INTEGER, INTEGER]]

   relationship_type_id_cache: HASH_TABLE[INTEGER, INTEGER]

   data_server: DATA_SERVER is
      do
		 Result := server.data_server
      end

   server : ENTITY_SERVER[G]

   select_relationship_1_statement: STRING

   select_relationship_1_predicate: COLLECTING_PREDICATE [SUPPLIER_ENTITY_RELATIONSHIP]

   make_select_relationship_1_predicate: COLLECTING_PREDICATE [SUPPLIER_ENTITY_RELATIONSHIP] is
      do
		 create Result.make (data_server, select_relationship_1_statement)
		 if not Result.success then
			error_log.set_severity (error_log.Fatal)
			error_log.set_message ("ERROR: Could not make predicate")
			error_log.set_class_name ("SUPPLIER_ENTITY_RELATIONSHIP_CACHE")
			error_log.set_feature_name ("make_select_relationship_1_predicate")
			error_log.put
		 end
      end

   select_relationship_2_statement: STRING

   select_relationship_2_predicate: COLLECTING_PREDICATE [SUPPLIER_ENTITY_RELATIONSHIP]
   make_select_relationship_2_predicate: COLLECTING_PREDICATE [SUPPLIER_ENTITY_RELATIONSHIP] is
      do
		 create Result.make (data_server, select_relationship_2_statement)
		 if not Result.success then
			error_log.set_severity (error_log.Fatal)
			error_log.set_message ("ERROR: Could not make predicate")
			error_log.set_class_name ("SUPPLIER_ENTITY_RELATIONSHIP_CACHE")
			error_log.set_feature_name ("make_select_relationship_2_predicate")
			error_log.put
		 end
      end

   make_empty_entity: G is
      do
		 create Result
      end

end -- class SUPPLIER_ENTITY_RELATIONSHIP_CACHE
