note

	description:

		"Supplier Listing"

	author: "Michael Zhang"
	last_editor: "$Author: mk $"
	date: "$Date: 2012/06/22 $"
	revision: "$Revision: #15 $"
	path: "$File: //rose/source/kernel/supplier_data/supplier_listing.e $"

deferred class
	SUPPLIER_LISTING

inherit
	PERSISTENT_LISTING
		undefine
			context
		redefine
			id_string
		end

feature -- price access

	price_set_series : DATED_MATRIX is
			-- price set series
		do
			Result := price_report.matrix(Current)
		end

	price_set_interval_series(a_start_date, a_end_date : ROSE_DATE) : DATED_MATRIX is
			-- price set interval series
		do
			Result := price_report.matrix_in_interval(Current, a_start_date, a_end_date)
		end

	price_set_at(a_date : ROSE_DATE) : VECTOR is
			-- price set at
		do
			Result := price_report.double_array_at_date(Current, a_date)
		end

	price_record_at(a_date : ROSE_DATE) : like ur_price_target is
			-- price record at
		do
			Result := price_report.target_at_date(Current, a_date)
		end

	price_records(a_start_date, a_end_date : ROSE_DATE) : ROSE_LINEAR_ARRAY[like ur_price_target] is
			-- price records
		do
			Result := price_report.targets_in_interval (Current, a_start_date, a_end_date)
		end

	price_set_currency_at(a_date : ROSE_DATE) : PERSISTENT_CURRENCY is
			-- price set currency at
		do
			Result := price_report.currency_at_date(Current, a_date)
		end

	price_report : ROSE_SIGNATURE_TAGGED_REPORT[PERSISTENT_LISTING, like ur_price_target] is
			-- price report
		deferred
		end

	ur_price_target: ROSE_SIGNATURE_RECORD [PERSISTENT_LISTING] is
			-- Generic price target
		do end

feature -- identifier information

	company_name_string : STRING is
			-- company name string
		local
			l_security : like context.security_template
			l_company : like context.company_template
		do
			l_security := security
			if l_security /= Void then
				l_company := l_security.company
				if l_company /= Void then
					Result := l_company.attribute_by_name (company_name_attribute_string, Void)
				end
			end
			if Result = Void then
				Result := ""
			end
		end

	company_name_attribute_string : STRING is
			-- company name attribute string
		do
			Result := "COMPANY NAME"
		end

	ticker_exchange_string : STRING is
			-- ticker exchange string
		local
			l_tick : STRING
			l_exchange : like context.exchange_template
		do
			l_tick := ticker_key(Void)
			l_exchange := exchange
			if l_exchange /= Void then
				Result := l_exchange.name_at(Void)
				if l_tick /= Void and then Result /= Void then
					Result := l_tick + " | " + Result
				end
			end
		end

	id_string : STRING is
			-- id string
		local
			l_string : STRING
			l_security : like context.security_template
		do
			Result := safe_key_value (Agent brim_code_key, today)
			if Result /= Void then
				Result := "|BRIMSYM| " + Result
			end
			l_string := safe_key_value (Agent riemsym_code_key, today)
			if l_string /= Void then
				if Result /= Void then
					Result := Result + " |RIEMSYM| " + l_string
				else
					Result := "|RIEMSYM| " + l_string
				end
			end
			l_string := safe_key_value (Agent reuters_code_key, today)
			if l_string /= Void then
				if Result /= Void then
					Result := Result + "%N|RIC| " + l_string
				else
					Result := "|RIC| " + l_string
				end
			end
			l_string := safe_key_value (Agent sedol_key, today)
			if l_string /= Void then
				if Result /= Void then
					Result := Result + " |SEDOL| " + l_string
				else
					Result := "|SEDOL| " + l_string
				end
			end
			l_security := security
			if l_security /= Void then
				l_string := safe_key_value (Agent l_security.cusip_key, today)
				if l_string /= Void then
					if Result /= Void then
						Result := Result + " |CUSIP| " + l_string
					else
						Result := "|CUSIP| " + l_string
					end
				end
			end
			if Result = Void then
				Result := Precursor
			end
		end

feature -- safe attribute access

	safe_key_value(a_key_function : FUNCTION[ANY, TUPLE[ROSE_DATE], STRING]; a_date : ROSE_DATE) : STRING is
			-- safe key value
		do
			Result := a_key_function.item([a_date])
			if Result = Void then
				Result := a_key_function.item([today])
			end
			if Result = Void then
				Result := a_key_function.item([end_date])
			end
			if Result = Void then
				Result := a_key_function.item([start_date])
			end
		end

end
