indexing
   
   description: "Set of permanent id's."
   status: "Development"
   author: "David Stevens"
   date: "$Date: 2008/12/01 $"
   revision: "$Revision: #9 $"
   
class
   
   SUPPLIER_PERMANENT_ID_SET
   
inherit
   
   DATABASE_SERVER
   
   COLLECTING_PREDICATE [SUPPLIER_PERMANENT_ID_NAME]
      rename
         make as collecting_predicate_make,
	   item as collecting_predicate_item
      redefine 
         action
      end
   
create
   
   make,
   make_w_name,
   make_w_current_identifier,
   make_sufficient_size_w_name,
   make_sufficient_size_w_current_identifier
   
feature -- Initialization
   
   make_sufficient_size_w_name (a_server: DATA_SERVER; 
				a_permanent_id_name: STRING; 
				a_table_name: STRING; 
				a_approximate_size: INTEGER) is
         -- Initialize the processor.
      require
         a_server_not_void: a_server /= Void
         a_permanent_id_name_not_void: a_permanent_id_name /= Void
         a_table_name_not_void: a_table_name /= Void
	 a_approximate_size_not_negative: a_approximate_size >= 0
      do
         approximate_size := a_approximate_size
	 make_w_name (a_server, a_permanent_id_name, a_table_name)
      end
   
   make_sufficient_size_w_current_identifier (a_server: DATA_SERVER; 
					      a_category_name: STRING; 
					      a_attribute_type_name: STRING;
					      a_approximate_size: INTEGER) is
         -- Initialize the processor.
      require
         a_server_not_void: a_server /= Void
         a_category_name_not_void: a_category_name /= Void
         a_attribute_type_name_not_void: a_attribute_type_name /= Void
	 a_approximate_size_not_negative: a_approximate_size >= 0
      do
         approximate_size := a_approximate_size
	 make_w_current_identifier (a_server, a_category_name, a_attribute_type_name)
      end
   
   make, make_w_name (a_server: DATA_SERVER; a_permanent_id_name: STRING; a_table_name: STRING) is
         -- Initialize the processor.
      require
         a_server_not_void: a_server /= Void
         a_permanent_id_name_not_void: a_permanent_id_name /= Void
         a_table_name_not_void: a_table_name /= Void
      local
         permanent_id_name: SUPPLIER_PERMANENT_ID_NAME
         sql_statement: STRING
      do
         sql_statement := "select "
         sql_statement.append (a_permanent_id_name)
         sql_statement.append (" as id, name from ")
         sql_statement.append (a_table_name)
         create id_table.make (approximate_size)
         create permanent_id_name.make
         selection_predicate_make (a_server, permanent_id_name, sql_statement)
         retrieve (Void, <<>>)
      end
   
   make_w_current_identifier (a_server: DATA_SERVER; a_category_name: STRING; a_attribute_type_name: STRING) is
         -- Initialize the processor.
      require
         a_server_not_void: a_server /= Void
         a_category_name_not_void: a_category_name /= Void
         a_attribute_type_name_not_void: a_attribute_type_name /= Void
      local
         permanent_id_name: SUPPLIER_PERMANENT_ID_NAME
         sql_statement: STRING
         category: SUPPLIER_CATEGORY
         category_set: ARRAY [SUPPLIER_CATEGORY]
         category_selection: SELECTION_PREDICATE [SUPPLIER_CATEGORY]
         attribute_type_template: SUPPLIER_ATTRIBUTE_TYPE
         attribute_type_set: ARRAY [SUPPLIER_ATTRIBUTE_TYPE]
         attribute_type_selection: SELECTION_PREDICATE [SUPPLIER_ATTRIBUTE_TYPE]
      do
         create id_table.make (approximate_size)
         create category
         create category_set.make (1, 1)
         create category_selection.make (servers.rose_entity_foreground_server, category, "select category_id from category where name = ?")
         category_selection.set_container (category_set)
         category_selection.retrieve (Void, <<a_category_name>>)
         if category_selection.index = 1 then
            create attribute_type_template
            create attribute_type_set.make (1, 1)
            create attribute_type_selection.make (a_server, attribute_type_template, "select attribute_type_id from attribute_type where name = ?")
            attribute_type_selection.set_container (attribute_type_set)
            attribute_type_selection.retrieve (Void, <<a_attribute_type_name>>)
            if attribute_type_selection.index = 1 then
               sql_statement := "select entity_id as id, identifier_value as name from identifier where "
               sql_statement.append ("category_id = ")
               sql_statement.append (category_set.item (1).category_id.out)
               sql_statement.append (" and attribute_type_id = ")
               sql_statement.append (attribute_type_set.item (1).attribute_type_id.out)
               sql_statement.append (" and (end_date = '9999/12/31' or end_date is NULL)")
               create permanent_id_name.make
               selection_predicate_make (a_server, permanent_id_name, sql_statement)
               retrieve (Void, <<>>)
            end               
         end
      end
   
feature -- Access
   
   item (a_name: STRING): INTEGER is
         -- Permanent identifier corresponding to 'a_name'.
      require   
         a_name_not_void: a_name /= Void
      do
         if id_table.has (a_name) then
            Result := id_table.item (a_name)
         end
      end
   
feature -- Element change
   
   extend (a_permanent_id: INTEGER; a_name: STRING) is
         -- Extend the set with 'a_permanent_id' accessed by 'a_name'.
      require   
         a_name_not_void: a_name /= Void
      do
         if not id_table.has (a_name) then
            id_table.force (a_permanent_id, clone (a_name))
         end
      end
   
   replace_key (new_key: STRING; old_key: STRING) is
	 -- Replace 'old_key' with 'new_key'.
      require
	 old_key_exists: item (old_key) /= 0
      do
	 id_table.replace_key (new_key, old_key)
      end
   
   remove (a_key: STRING) is
	 -- Remove the item with key 'a_key'.
      require
	 a_key_not_void: a_key /= Void
      do
	 id_table.remove (a_key);
      end
   
feature {NONE} -- Implementation
   
   action (a_row_selection: SUPPLIER_PERMANENT_ID_NAME) is
         -- Action applied to retrieved record.
      do
         a_row_selection.name.right_adjust
         id_table.force (a_row_selection.id, clone (a_row_selection.name))
      end
   
   id_table: HASH_TABLE [INTEGER, STRING]
   
   approximate_size: INTEGER
   
end -- class SUPPLIER_PERMANENT_ID_SET
