note

	description:

		"Handle process ids."

	author: "David Stevens"
	last_editor: "$Author: er $"
	date: "$Date: 2014/02/26 $"
	revision: "$Revision: #46 $"
	path: "$File: //rose/source/kernel/supplier_data/supplier_process_id_manager.e $"

class	SUPPLIER_PROCESS_ID_MANAGER

inherit

	ANY

	ERROR_LOG_USE
		export {NONE} all	end

	MISSING_PARAMETERS
		export {NONE} all	end

	ROSE_CONSTANTS
		export {NONE} all	end

	PM_RUN_ID
		export {NONE} all	end
	
create
	make

feature {NONE} -- Initialization

	make (a_server: SQL_DATA_SERVER) is
			-- Initialization.
		require
			a_server_not_void: a_server /= Void
		do
			server := a_server
			has_run_id_column := server.has_column_name ("process", "run_id")
			if has_run_id_column then
				create process_inserter.make (server, "insert into process (%
												  %name, %
												  %run_id, %
												  %file_date, %
												  %market_available_date, %
												  %rose_available_date, %
												  %start_time, %
												  %end_time) %
												  %values (?, ?, ?, ?, ?, ?, ?)")
			else
				create process_inserter.make (server, "insert into process (%
												  %name, %
												  %file_date, %
												  %market_available_date, %
												  %rose_available_date, %
												  %start_time, %
												  %end_time) %
												  %values (?, ?, ?, ?, ?, ?)")
			end
			create process
			--			create process_id_selection.make (server, "select top 1 process_id from process (nolock) order by process_id DESC")
			create process_id_selection.make (server, "select cast(@@IDENTITY as int) as process_id")
			create process_updater.make (server, "update process set end_time = ? where process_id = ?")
			create market_available_date_updater.make (server, "update process set market_available_date = ? where process_id = ?")
			set_end_time (constants.maximum_date_time)
			create saved_last_process_id_selection.make_strict (server, "select max (process_id) from process")
		end

feature -- Basic operations

	start_process (a_process_name: STRING; a_file_date, a_market_date, a_rose_date: ROSE_DATE) is
			-- Start process 'a_process_name'
		require
			valid_process_name: a_process_name /= VOid
		local
			l_now: ROSE_DATE
		do
			create l_now.make_now
			l_now.time.minute_back
			set_name (a_process_name)
			set_file_date (a_file_date)
			set_market_available_date (a_market_date)
			set_rose_available_date (a_rose_date)
			set_start_time (l_now)
			set_new_process_id
		end

   	end_process is
			-- Terminate the currently running process
		local
			l_now: ROSE_DATE
   		do
   			create l_now.make_now
   			set_end_time (l_now)
   			update_end_time
   		end

	set_new_process_id is
			-- Get a new process_id and write it to the database.
		require
			name_not_void: name /= Void
			file_date_not_void: file_date /= Void
			market_available_date_not_void: market_available_date /= Void
			rose_available_date_not_void: rose_available_date /= Void
			start_time_not_void: start_time /= Void
		local
			process_set: ARRAY [SUPPLIER_PROCESS]
		do
			if has_run_id_column then
				process_inserter.execute (Void, <<name, run_id, file_date, market_available_date, rose_available_date, start_time, end_time>>)
			else
				process_inserter.execute (Void, <<name, file_date, market_available_date, rose_available_date, start_time, end_time>>)
			end
			if not process_inserter.success then
				output (Global.interpreted ("($now) supplier_process_id_manager: failed to create new process '" + name + "' on " + server.definition.path + "%N"),Void)
				error_log.set_class_name ("SUPPLIER_PROCESS_ID_MANAGER")
				error_log.set_feature_name ("set_new_process_id")
				error_log.set_message ("Error inserting new process_id ....: ")
				error_log.append_message (name)
				error_log.set_severity (error_log.Fatal)
				error_log.put
			end
			create process_set.make (1, 1)
			process_id_selection.set_container (process_set)
			process_id_selection.retrieve (Void, <<>>)
			if process_id_selection.index = 1 then
				process_id := process_set.item (1).process_id
			else
				output (Global.interpreted ("($now) supplier_process_id_manager: failed to retrieve new process '" + name + "' on " + server.definition.path + "%N"),Void)
				error_log.set_class_name ("SUPPLIER_PROCESS_ID_MANAGER")
				error_log.set_feature_name ("set_new_process_id")
				error_log.set_message ("Error in count of process rows retrieved ....: ")
				error_log.append_message (name)
				error_log.set_severity (error_log.Fatal)
				error_log.put
			end
			-- Force process_cache to refresh
			process_cache := Void
		end

	update_end_time is
			-- Update the end time for this process in the database.
		require
			end_time_not_void: end_time /= Void
		do
			process_updater.execute (Void, <<end_time, process_id>>)
			if not process_updater.success then
				error_log.set_class_name ("SUPPLIER_PROCESS_ID_MANAGER")
				error_log.set_feature_name ("update_end_time")
				error_log.set_message ("Error updating end_time ....: ")
				error_log.append_message (process_id.out)
				error_log.set_severity (error_log.Fatal)
				error_log.put
			end
			-- Force process_cache to refresh
			process_cache := Void
		end

	update_name_with_tablock (a_name: STRING) is
			-- Update the end time for this process in the database.
		require
			a_name_not_void: a_name /= Void
        local
            l_updater_with_tablock: EXECUTION_PREDICATE
		do
            set_name (a_name)
			create l_updater_with_tablock.make (server, "update process with (tablock) set name = '" + name + "' where process_id = " + process_id.out)
			if not l_updater_with_tablock.success then
				error_log.set_class_name ("SUPPLIER_PROCESS_ID_MANAGER")
				error_log.set_feature_name ("update_name_with_tablock")
				error_log.set_message ("Error updating name with tablock ....: ")
				error_log.append_message (process_id.out)
				error_log.set_severity (error_log.Fatal)
				error_log.put
			end
			l_updater_with_tablock.execute (Void, Void)
			if not l_updater_with_tablock.success then
				error_log.set_class_name ("SUPPLIER_PROCESS_ID_MANAGER")
				error_log.set_feature_name ("update_name_with_tablock")
				error_log.set_message ("Error updating name with tablock ....: ")
				error_log.append_message (process_id.out)
				error_log.set_severity (error_log.Fatal)
				error_log.put
			end
			-- Force process_cache to refresh
			process_cache := Void
		end

	update_market_available_date is
			-- Update the market_available_date for this process in the database.
		require
			market_available_date_not_void: market_available_date /= Void
		do
			market_available_date_updater.execute (Void, <<market_available_date, process_id>>)
			if not market_available_date_updater.success then
				error_log.set_class_name ("SUPPLIER_PROCESS_ID_MANAGER")
				error_log.set_feature_name ("update_market_available_date")
				error_log.set_message ("Error updating market_available_date ....: ")
				error_log.append_message (process_id.out)
				error_log.set_severity (error_log.Fatal)
				error_log.put
			end
			-- Force process_cache to refresh
			process_cache := Void
		end

	reset_cache is
			-- Detect if there has been a new process created and
			-- either reset the caches to force refresh or get the 
			-- latest process and add it to the caches if it's the 
			-- only one missing
		local
			l_count: INTEGER
			l_last_process: SUPPLIER_PROCESS
		do
			l_count := process_count
			if process_cache /= Void and then l_count > process_cache.count then
				if l_count - process_cache.count = 1 then
					l_last_process := last_process
					if not process_cache.has (l_last_process.process_id) then
						handle_supplier_process_record (l_last_process)
						handle_supplier_process_record_for_file_date_cache (l_last_process)
					else
						file_date_process_cache := Void
						process_cache := Void
					end
				else
					file_date_process_cache := Void
					process_cache := Void
				end
			end
		end

feature -- Access

	process_id: INTEGER

	name: STRING

	file_date: ROSE_DATE

	market_available_date: ROSE_DATE

	rose_available_date: ROSE_DATE

	start_time: ROSE_DATE

	end_time: ROSE_DATE

feature -- Element change

	set_name (a_name: STRING) is
			-- Set 'name' to 'a_name'.
		require
			a_name_not_void: a_name /= Void
		do
			name := a_name
		end

	set_file_date (a_date_time: ROSE_DATE) is
			-- Set 'file_date' to 'a_date_time'.
		require
			a_date_time_not_void: a_date_time /= Void
		do
			file_date := a_date_time
		end

	set_market_available_date (a_date_time: ROSE_DATE) is
			-- Set 'market_available_date' to 'a_date_time'.
		require
			a_date_time_not_void: a_date_time /= Void
		do
			market_available_date := a_date_time
		end

	set_rose_available_date (a_date_time: ROSE_DATE) is
			-- Set 'rose_available_date' to 'a_date_time'.
		require
			a_date_time_not_void: a_date_time /= Void
		do
			rose_available_date := a_date_time
		end

	set_start_time (a_date_time: ROSE_DATE) is
			-- Set 'start_time' to 'a_date_time'.
		require
			a_date_time_not_void: a_date_time /= Void
		do
			start_time := a_date_time
		end

	set_end_time (a_date_time: ROSE_DATE) is
			-- Set 'end_time' to 'a_date_time'.
		require
			a_date_time_not_void: a_date_time /= Void
		do
			end_time := a_date_time
		end

    retrieve_by_process_id (a_process_id: INTEGER) is
        local
            l_process: SUPPLIER_PROCESS
        do
            l_process := process_by_id (a_process_id)
            if l_process = Void then
                process_id := 0
                name := Void
                file_date := Void
                market_available_date := Void
                rose_available_date := Void
                start_time := Void
                end_time := Void
            else
                process_id := l_process.process_id
                name := l_process.name
                file_date := l_process.file_date
                market_available_date := l_process.market_available_date
                rose_available_date := l_process.rose_available_date
                start_time := l_process.start_time
                end_time := l_process.end_time
            end
        end

	set_process (a_name: STRING;
				 a_file_date: ROSE_DATE;
				 a_market_available_date: ROSE_DATE;
				 a_rose_available_date: ROSE_DATE;
				 a_start_time: ROSE_DATE;
				 a_end_time: ROSE_DATE) is
		do
			set_name (a_name)
			set_file_date (a_file_date)
			set_market_available_date (a_market_available_date)
			set_rose_available_date (a_rose_available_date)
			set_start_time (a_start_time)
			set_end_time (a_end_time)
		end

	assign_process_id (a_name: STRING;
					   a_file_date: ROSE_DATE;
					   a_market_available_date: ROSE_DATE;
					   a_rose_available_date: ROSE_DATE) is
		local
			date_time_now: ROSE_DATE
		do
			set_name (a_name)
			set_file_date (a_file_date)
			set_market_available_date (a_market_available_date)
			set_rose_available_date (a_rose_available_date)
			create date_time_now.make_now
			set_start_time (date_time_now)
			set_end_time (constants.maximum_date_time) -- By resetting 'end_time', We can call this procedure without calling 'make' again.
			set_new_process_id
		end

	update_process_id is
			-- Set the end time of the current process to now
		local
			date_time_now: ROSE_DATE
		do
			create date_time_now.make_now
			set_end_time (date_time_now)
			update_end_time
		end

feature -- Query

	all_processes: ROSE_LINEAR_ARRAY[SUPPLIER_PROCESS] is
			-- All process records from `server'
		local
			l_predicate: COLLECTING_PREDICATE[SUPPLIER_PROCESS]
		do
			create l_predicate.make_strict (server, "select * from process (nolock) order by process_id desc")
			l_predicate.retrieve (Void,Void)
			Result := l_predicate.array
		ensure
			not_void: Result /= void
		end

	last_process_id: INTEGER is
		do
			saved_last_process_id_selection.retrieve (Void,Void)
			Result := saved_last_process_id_selection.array.first
		end

	last_process: SUPPLIER_PROCESS is
			-- Most recent process for `server'
		local
			l_process: SUPPLIER_PROCESS
			l_selection: COLLECTING_PREDICATE[SUPPLIER_PROCESS]
		do
			create l_process
			create l_selection.make (server, "select top 1 * from process order by process_id DESC")
			l_selection.retrieve_one (l_process, Void, Void)
			if not l_selection.success then
				error_log.set_class_name ("SUPPLIER_PROCESS_ID_MANAGER")
				error_log.set_feature_name ("last_process")
				error_log.set_message ("Error on retrieving last process")
				error_log.set_severity (error_log.Fatal)
				error_log.put
			elseif l_selection.index = 0 then
				Result := Void
			else
				Result := l_process
			end
		end

    last_unfinished_process: SUPPLIER_PROCESS is
        local
            l_process: SUPPLIER_PROCESS
            l_selection: COLLECTING_PREDICATE [SUPPLIER_PROCESS]
        do
            create l_process
            create l_selection.make (server, "select top 1 * from process where end_time = ? order by process_id DESC")
            if not l_selection.success then
                error_log.set_class_name ("SUPPLIER_PROCESS_ID_MANAGER")
                error_log.set_feature_name ("last_unfinished_process")
                error_log.set_message ("Error on making last unfinished process selector")
                error_log.set_severity (error_log.Fatal)
                error_log.put
            end
            l_selection.retrieve_one (l_process, Void, <<constants.maximum_date_time>>)
            if not l_selection.success then
                error_log.set_class_name ("SUPPLIER_PROCESS_ID_MANAGER")
                error_log.set_feature_name ("last_unfinished_process")
                error_log.set_message ("Error on retrieving last unfinished process")
                error_log.set_severity (error_log.Fatal)
                error_log.put
            elseif l_selection.index = 0 then
                Result := Void
            else
                Result := l_process
            end
        end

    last_finished_process: SUPPLIER_PROCESS is
        local
            l_process: SUPPLIER_PROCESS
            l_selection: COLLECTING_PREDICATE [SUPPLIER_PROCESS]
        do
            create l_process
            create l_selection.make (server, "select top 1 * from process where end_time < ? order by process_id DESC")
            if not l_selection.success then
                error_log.set_class_name ("SUPPLIER_PROCESS_ID_MANAGER")
                error_log.set_feature_name ("last_finished_process")
                error_log.set_message ("Error on making last finished process selector")
                error_log.set_severity (error_log.Fatal)
                error_log.put
            end
            l_selection.retrieve_one (l_process, Void, <<constants.maximum_date_time>>)
            if not l_selection.success then
                error_log.set_class_name ("SUPPLIER_PROCESS_ID_MANAGER")
                error_log.set_feature_name ("last_finished_process")
                error_log.set_message ("Error on retrieving last finished process")
                error_log.set_severity (error_log.Fatal)
                error_log.put
            elseif l_selection.index = 0 then
                Result := Void
            else
                Result := l_process
            end
        end

    process_by_id (a_process_id: INTEGER): SUPPLIER_PROCESS is
			-- Process for `a_process_id', will cache
		require
			valid_id: a_process_id > 0
        local
            l_process: SUPPLIER_PROCESS
            l_selection: COLLECTING_PREDICATE [SUPPLIER_PROCESS]
        do
			if process_cache = Void or else not process_cache.has (a_process_id) then
				if process_cache = Void then
					create process_cache.make (100)
				end
				create l_process
				create l_selection.make (server, "select * from process where process_id = ?")
				if not l_selection.success then
					error.raise ("Could not create predicate")
				end
				l_selection.retrieve_one (l_process, Void, <<a_process_id>>)
				if not l_selection.success then
					error.raise ("Could not execute predicate")
				else
					Result := l_process
					handle_supplier_process_record (l_process)
 				end
			else
				process_cache.search (a_process_id)
				Result := process_cache.found_item
			end
        end

    process_by_file_date (a_file_date: ROSE_DATE): SUPPLIER_PROCESS is
			-- Process for `a_file_date', will cache
		require
			valid_file_date: a_file_date /= Void
        local
            l_process: SUPPLIER_PROCESS
            l_selection: COLLECTING_PREDICATE [SUPPLIER_PROCESS]
        do
			if file_date_process_cache = Void or else not file_date_process_cache.has (a_file_date) then
				if file_date_process_cache = Void then
					create file_date_process_cache.make (100)
				end
				create l_process
				create l_selection.make (server, "select * from process where file_date = ?")
				if not l_selection.success then
					error.raise ("Could not create predicate")
				end
				l_selection.retrieve_one (l_process, Void, <<a_file_date>>)
				if not l_selection.success then
					error.raise ("Could not execute predicate")
				else
					Result := l_process
					handle_supplier_process_record_for_file_date_cache (l_process)
 				end
			else
				file_date_process_cache.search (a_file_date)
				Result := file_date_process_cache.found_item
			end
        end

    cached_process_by_id (a_process_id: INTEGER): SUPPLIER_PROCESS is
			-- Supplier process object keyed to 'a_process_id' retrieved by cached data, will requery if 'a_process_id' is not in the cache
		require
			valid_id: a_process_id > 0
        do
			if process_cache = Void then
				populate_caches
			elseif not process_cache.has (a_process_id) then
				process_cache.force (process_by_id (a_process_id), a_process_id)
			end
			process_cache.search (a_process_id)
			if process_cache.found then
				Result := process_cache.found_item
			end
		end

    cached_process_by_file_date (a_file_date: ROSE_DATE): SUPPLIER_PROCESS is
			-- Supplier process object keyed to 'a_file_date' retrieved by cached data
		require
			valid_file_date: a_file_date /= Void
        do
            if file_date_process_cache = Void then
				populate_caches
			elseif not file_date_process_cache.has (a_file_date) then
				file_date_process_cache.force (process_by_file_date (a_file_date), a_file_date.twin)
			end
			file_date_process_cache.search (a_file_date)
			if file_date_process_cache.found then
				Result := file_date_process_cache.found_item
			end
		end

    last_specific_process (a_process_name: STRING): SUPPLIER_PROCESS is
        require
            a_process_name_not_void: a_process_name /= Void
        local
            l_process: SUPPLIER_PROCESS
            l_selection: COLLECTING_PREDICATE [SUPPLIER_PROCESS]
        do
            create l_process
            create l_selection.make (server, "select top 1 * from process (nolock) where name like ? order by process_id DESC")
            if not l_selection.success then
                error_log.set_class_name ("SUPPLIER_PROCESS_ID_MANAGER")
                error_log.set_feature_name ("last_specific_process")
                error_log.set_message ("Error on making last specific process selector")
                error_log.set_severity (error_log.Fatal)
                error_log.put
            end
            l_selection.retrieve_one (l_process, Void, <<a_process_name>>)
            if not l_selection.success then
                error_log.set_class_name ("SUPPLIER_PROCESS_ID_MANAGER")
                error_log.set_feature_name ("last_specific_process")
                error_log.set_message ("Error on retrieving last specific process")
                error_log.set_severity (error_log.Fatal)
                error_log.put
            elseif l_selection.index = 0 then
                Result := Void
            else
                Result := l_process
            end
        end

    last_finished_like_specific_process (a_process_name: STRING): SUPPLIER_PROCESS is
        require
            a_process_name_not_void: a_process_name /= Void
        local
            l_process: SUPPLIER_PROCESS
            l_selection: COLLECTING_PREDICATE [SUPPLIER_PROCESS]
            l_name: STRING
        do
            create l_process
            create l_selection.make (server, "select top 1 * from process where name like ? and end_time < ? order by process_id DESC")
            if not l_selection.success then
                error_log.set_class_name ("SUPPLIER_PROCESS_ID_MANAGER")
                error_log.set_feature_name ("last_finished_specific_process")
                error_log.set_message ("Error on making last finished like specific process selector")
                error_log.set_severity (error_log.Fatal)
                error_log.put
            end
            l_name:="%%" + a_process_name + "%%"
            l_selection.retrieve_one (l_process, Void, <<l_name,constants.maximum_date_time>>)
            if not l_selection.success then
                error_log.set_class_name ("SUPPLIER_PROCESS_ID_MANAGER")
                error_log.set_feature_name ("last_finished_specific_process")
                error_log.set_message ("Error on retrieving last finished like specific process")
                error_log.set_severity (error_log.Fatal)
                error_log.put
            elseif l_selection.index = 0 then
                Result := Void
            else
                Result := l_process
            end
        end

    last_unfinished_like_specific_process (a_process_name: STRING): SUPPLIER_PROCESS is
        require
            a_process_name_not_void: a_process_name /= Void
        local
            l_process: SUPPLIER_PROCESS
            l_selection: COLLECTING_PREDICATE [SUPPLIER_PROCESS]
            l_name: STRING
        do
            create l_process
            create l_selection.make (server, "select top 1 * from process where name = ? and end_time = ? order by process_id DESC")
            if not l_selection.success then
                error_log.set_class_name ("SUPPLIER_PROCESS_ID_MANAGER")
                error_log.set_feature_name ("last_unfinished_like_specific_process")
                error_log.set_message ("Error on making last unfinished like specific process selector")
                error_log.set_severity (error_log.Fatal)
                error_log.put
            end
            l_name:="%%" + a_process_name + "%%"
            l_selection.retrieve_one (l_process, Void, <<l_name,constants.maximum_date_time>>)
            if not l_selection.success then
                error_log.set_class_name ("SUPPLIER_PROCESS_ID_MANAGER")
                error_log.set_feature_name ("last_unfinished_like_specific_process")
                error_log.set_message ("Error on retrieving last like unfinished specific process")
                error_log.set_severity (error_log.Fatal)
                error_log.put
            elseif l_selection.index = 0 then
                Result := Void
            else
                Result := l_process
            end
        end

    finished_like_specific_processes (a_process_name: STRING): SUPPLIER_PROCESS_HISTORY is
        require
            a_process_name_not_void: a_process_name /= Void
        local
            l_selection: COLLECTING_PREDICATE [SUPPLIER_PROCESS]
            l_name: STRING
        do
            create l_selection.make (server, "select * from process where name like ? and end_time < ? order by process_id")
            if not l_selection.success then
                error_log.set_class_name ("SUPPLIER_PROCESS_ID_MANAGER")
                error_log.set_feature_name ("last_finished_specific_process")
                error_log.set_message ("Error on making last finished like specific process selector")
                error_log.set_severity (error_log.Fatal)
                error_log.put
            end
            l_name:="%%" + a_process_name + "%%"
            l_selection.retrieve (Void, <<l_name, constants.maximum_date_time>>)
            if not l_selection.success then
                error_log.set_class_name ("SUPPLIER_PROCESS_ID_MANAGER")
                error_log.set_feature_name ("last_finished_specific_process")
                error_log.set_message ("Error on retrieving last finished like specific process")
                error_log.set_severity (error_log.Fatal)
                error_log.put
            else
                create Result.make_from_array (l_selection.array)
            end
        end

    finished_like_specific_processes_since (a_process_name: STRING; a_time: ROSE_DATE): SUPPLIER_PROCESS_HISTORY is
        require
            a_process_name_not_void: a_process_name /= Void
        local
            l_selection: COLLECTING_PREDICATE [SUPPLIER_PROCESS]
            l_name: STRING
        do
            create l_selection.make (server, "select * from process where name like ? and start_time > ? and end_time < ? order by process_id")
            if not l_selection.success then
                error_log.set_class_name ("SUPPLIER_PROCESS_ID_MANAGER")
                error_log.set_feature_name ("last_finished_specific_process")
                error_log.set_message ("Error on making last finished like specific process selector")
                error_log.set_severity (error_log.Fatal)
                error_log.put
            end
            l_name:="%%" + a_process_name + "%%"
            l_selection.retrieve (Void, <<l_name, a_time, constants.maximum_date_time>>)
            if not l_selection.success then
                error_log.set_class_name ("SUPPLIER_PROCESS_ID_MANAGER")
                error_log.set_feature_name ("last_finished_specific_process")
                error_log.set_message ("Error on retrieving last finished like specific process")
                error_log.set_severity (error_log.Fatal)
                error_log.put
            else
                create Result.make_from_array (l_selection.array)
            end
        end

    multi_finished_like_specific_processes_since (a_process_names: ARRAY[STRING]; a_time: ROSE_DATE): SUPPLIER_PROCESS_HISTORY is
 		local
 			i: INTEGER
 			l_history: SUPPLIER_PROCESS_HISTORY
        do
        	from i := a_process_names.lower until i > a_process_names.upper loop
        		l_history := finished_like_specific_processes_since (a_process_names.item (i), a_time)
        		if l_history /= Void then
        			if Result = Void then
        				Result := l_history
        			else
        				Result.append_array (l_history)
        			end
        		end
        		i := i + 1
        	end
        end

	process_count: INTEGER is
			-- Current count of processes in the database
        local
            l_process_count: INTEGER_REF
            l_selection: COLLECTING_PREDICATE [INTEGER_REF]
        do
            create l_process_count
            create l_selection.make (server, "select count (*) as item from process")
            if not l_selection.success then
                error_log.set_class_name ("SUPPLIER_PROCESS_ID_MANAGER")
                error_log.set_feature_name ("process_count")
                error_log.set_message ("Error on making process count selector")
                error_log.set_severity (error_log.Fatal)
                error_log.put
            end
            l_selection.retrieve_one (l_process_count, Void, Void)
            if not l_selection.success or l_selection.index = 0 then
                error_log.set_class_name ("SUPPLIER_PROCESS_ID_MANAGER")
                error_log.set_feature_name ("process_count")
                error_log.set_message ("Error on retrieving process count selector")
                error_log.set_severity (error_log.Fatal)
                error_log.put
            end
			Result := l_process_count.item
        end

feature {NONE} -- Implementation

	saved_last_process_id_selection: INTEGER_PREDICATE

	process: SUPPLIER_PROCESS

	process_id_selection: COLLECTING_PREDICATE [SUPPLIER_PROCESS]

	process_inserter: EXECUTION_PREDICATE

	process_updater: EXECUTION_PREDICATE

	market_available_date_updater: EXECUTION_PREDICATE

	server: SQL_DATA_SERVER

	has_run_id_column: BOOLEAN
			-- Does table 'process' in `server' have a column named 'run_id'?

	process_cache: HASH_TABLE[SUPPLIER_PROCESS,INTEGER]
			-- Supplier process objects keyed by process_id
	file_date_process_cache: HASH_TABLE[SUPPLIER_PROCESS,ROSE_DATE]
			-- Supplier process objects keyed by file_date
	
	populate_caches is
			-- Populate the process id and file date caches
		local
			l_processes: like all_processes
		do
			l_processes := all_processes
			create process_cache.make (1_000)
			l_processes.do_all (agent handle_supplier_process_record)
			create file_date_process_cache.make(10)
			l_processes.do_all (agent handle_supplier_process_record_for_file_date_cache)
		ensure
			valid_process_cache: process_cache /= Void
			valid_file_date_cache: file_date_process_cache /= Void
		end
	
	handle_supplier_process_record (a_row: SUPPLIER_PROCESS) is
			-- Process retrieval into the 'process_cache'
		require
			available_cache: process_cache /= Void
			valid_row: a_row /= Void
		do
			process_cache.force (a_row.twin, a_row.process_id)
		end

	handle_supplier_process_record_for_file_date_cache (a_row: SUPPLIER_PROCESS) is
			-- Process retrieval into the 'file_date_process_cache'
		require
			available_cache: file_date_process_cache /= Void
			valid_row: a_row /= Void
		do
			file_date_process_cache.force (a_row.twin, a_row.file_date.twin)
		end

end





