indexing
   
   description: "Supplier relationship type manager."
   status: "Development"
   author: "David Stevens"
   date: "$Date: 2008/12/01 $"
   revision: "$Revision: #9 $"
   
class
   
   SUPPLIER_RELATIONSHIP_TYPE_MANAGER
   
inherit
   
   DATABASE_SERVER
   
   ERROR_LOG_USE
   
create
   
   make
   
feature -- Initialization
   
   make is
      local
	 l_array: EXTENDABLE_BOUNDED_ARRAY [SUPPLIER_RELATIONSHIP_TYPE]
	 i: INTEGER
	 l_selection_predicate: COLLECTING_PREDICATE [SUPPLIER_RELATIONSHIP_TYPE]
      do
	 -- Retrieve the relationship types.
	 create l_selection_predicate.make (servers.rose_entity_foreground_server, "select * from relationship_type")
	 if not l_selection_predicate.success then
	    error_log.set_severity (error_log.Fatal)
	    error_log.set_message ("ERROR: Could not make predicate")
	    error_log.set_class_name ("SUPPLIER_RELATIONSHIP_TYPE_MANAGER")
	    error_log.set_feature_name ("make")
	    error_log.put
	 end
	 create l_array.make_empty
	 l_selection_predicate.set_container (l_array)
	 l_selection_predicate.retrieve (Void, Void)
	 if not l_selection_predicate.success then
	    error_log.set_severity (error_log.Fatal)
	    error_log.set_message ("ERROR: Select relationship_types failed")
	    error_log.set_class_name ("SUPPLIER_RELATIONSHIP_TYPE_MANAGER")
	    error_log.set_feature_name ("make")	    
	    error_log.put
	 end
	 
	 -- Load the caches.
	 create id_cache.make (l_array.count)
	 create name_cache.make (l_array.count)
	 from
	    i := 1
	 until
	    i > l_array.count
	 loop
	    id_cache.put (l_array.item (i), l_array.item (i).relationship_type_id)
	    name_cache.put (l_array.item (i), l_array.item (i).name)
	    i := i + 1
	 end
      end
   
feature -- Access   
   
   relationship_type (a_relationship_type_id: INTEGER): SUPPLIER_RELATIONSHIP_TYPE is
      require
	 a_relationship_type_id_not_zero: a_relationship_type_id /= 0
      do
	 id_cache.search (a_relationship_type_id)
	 if id_cache.found then
	    Result := id_cache.found_item
	 else
	    error_log.set_severity (error_log.Fatal)
	    error_log.set_message ("ERROR: Relationship_type not found (relationship_type_id) ...: ")
	    error_log.append_message (a_relationship_type_id.out)
	    error_log.set_class_name ("SUPPLIER_RELATIONSHIP_TYPE_MANAGER")
	    error_log.set_feature_name ("relationship_type")
	    error_log.put
	 end
      end
   
   relationship_type_id (a_name: STRING): INTEGER is
      require
	 a_name_not_void: a_name /= Void
      do
	 name_cache.search (a_name)
	 if name_cache.found then
	    Result := name_cache.found_item.relationship_type_id
	 else
	    error_log.set_severity (error_log.Fatal)
	    error_log.set_message ("ERROR: Relationship_type not found (name) ...: ")
	    error_log.append_message (a_name)
	    error_log.set_class_name ("SUPPLIER_RELATIONSHIP_TYPE_MANAGER")
	    error_log.set_feature_name ("relationship_type_id")
	    error_log.put
	 end
      end
   
   relationship_type_name (a_relationship_type_id: INTEGER): STRING is
      require
	 a_relationship_type_id_not_zero: a_relationship_type_id /= 0
      do
	 id_cache.search (a_relationship_type_id)
	 if id_cache.found then
	    Result := id_cache.found_item.name
	 else
	    error_log.set_severity (error_log.Fatal)
	    error_log.set_message ("ERROR: Relationship_type not found (relationship_type_id) ...: ")
	    error_log.append_message (a_relationship_type_id.out)
	    error_log.set_class_name ("SUPPLIER_RELATIONSHIP_TYPE_MANAGER")
	    error_log.set_feature_name ("relationship_type_name")
	    error_log.put
	 end
      end
   
feature {NONE} -- Implementation
   
   id_cache: HASH_TABLE [SUPPLIER_RELATIONSHIP_TYPE, INTEGER]
   
   name_cache: HASH_TABLE [SUPPLIER_RELATIONSHIP_TYPE, STRING]
   
end
