indexing
	description: "The Windows structure SYSTEMTIME"
	author: "RJ"
	date: "$Date: 2008/12/01 $"
	revision: "$Revision: #13 $"
	archive: "$File: //rose/source/kernel/infrastructure/windows/systemtime.e $"

class
	SYSTEMTIME

inherit
	EXTERNAL_RECORD
		redefine
--			display,
			write,
			read,
			record_length_
		end
	C_ACCESS_SUPPORT
	EXCEPTIONS
		export
			{NONE}
				all
		end

create
	make,
	make_from_pointer,
	make_from_rose_date_time ,
	make_orinda_standard_timezone_start,
	make_orinda_daylight_timezone_start,
	make_london_standard_timezone_start,
	make_london_daylight_timezone_start,
	make_nodst_timezone_start,
	make_from_filetime,
	default_create

feature
	make(a_systemtime_buffer: MANAGED_POINTER) is
	do
		make_from_pointer(a_systemtime_buffer.item)
	end

	make_from_pointer(a_pointer: POINTER) is
	do
		set(a_pointer)
		read
	end

	make_orinda_standard_timezone_start is
	do
		wYear := 0
		wMonth := 10
		wDayOfWeek := 0
		wDay := 5

		wHour := 2
		wMinute := 0
		wSecond := 0
		wMilliseconds := 0

		create managed_pointer.make(record_length_)
		set(managed_pointer.item)
		write
	end

	make_orinda_daylight_timezone_start is
	do
		wYear := 0
		wMonth := 4
		wDayOfWeek := 0
		wDay := 1

		wHour := 2
		wMinute := 0
		wSecond := 0
		wMilliseconds := 0

		create managed_pointer.make(record_length_)
		set(managed_pointer.item)
		write
	end

	make_london_standard_timezone_start is
	do
		wYear := 0
		wMonth := 10
		wDayOfWeek := 0
		wDay := 5

		wHour := 2
		wMinute := 0
		wSecond := 0
		wMilliseconds := 0

		create managed_pointer.make(record_length_)
		set(managed_pointer.item)
		write
	end

	make_london_daylight_timezone_start is
	do
		wYear := 0
		wMonth := 3
		wDayOfWeek := 0
		wDay := 5

		wHour := 2
		wMinute := 0
		wSecond := 0
		wMilliseconds := 0

		create managed_pointer.make(record_length_)
		set(managed_pointer.item)
		write
	end


	make_nodst_timezone_start is
	do
		wYear := 0
		wMonth := 0
		wDayOfWeek := 0
		wDay := 0

		wHour := 0
		wMinute := 0
		wSecond := 0
		wMilliseconds := 0

		create managed_pointer.make(record_length_)
		set(managed_pointer.item)
		write
	end

	make_from_rose_date_time (a_rose_date: ROSE_DATE) is
	local
		l_time: ROSE_TIME
	do
		wYear := a_rose_date.year
		wMonth := a_rose_date.month
		wDayOfWeek := a_rose_date.day_of_the_week - 1
		wDay := a_rose_date.day

		l_time := a_rose_date.time
		wHour := l_time.hour
		wMinute := l_time.minute
		wSecond := l_time.second
		wMilliseconds := (l_time.fractional_second * 1000).truncated_to_integer

		create managed_pointer.make(record_length_)
		set(managed_pointer.item)
		write
	end

	make_from_filetime(a_filetime: WINDOWS_FILETIME) is
	local
		l_ft_managed_pointer: MANAGED_POINTER
		l_st_managed_pointer: MANAGED_POINTER
		l_status: BOOLEAN
	do
		if a_filetime.source_pointer_=default_pointer
		then
			create l_ft_managed_pointer.make(size_of_filetime)
			a_filetime.set(l_ft_managed_pointer.item)
			a_filetime.write
		else
			-- Hmmm, we are assuming that his pointer is valid!
		end

		create l_st_managed_pointer.make(size_of_systemtime)
		l_status := FileTimeToSystemTime(
						a_filetime.source_pointer_,
						l_st_managed_pointer.item)
		make_from_pointer(l_st_managed_pointer.item)

		if l_ft_managed_pointer/=Void
		then
			a_filetime.set(default_pointer)
		end
	end

	date_time: ROSE_DATE is
	do
		if wyear/=1601
		then
			create Result.make_date_time(wYear, wMonth, wDay, wHour, wMinute, wSecond)
		else
			create Result.make_min_date
		end
	end

	time: ROSE_TIME is
	do
		create Result.make(wHour, wMinute, wSecond)
	end

	local_from_ut_timezone: ROSE_DATE is
	do
		Result := local_time_from_ut(Local_tz_info)
	end

	ut_from_local_timezone: ROSE_DATE is
	do
		Result := ut_from_timezone(local_tz_info)
	end

	ut_from_orinda_timezone: ROSE_DATE is
	do
		Result := ut_from_timezone(orinda_tz_info)
	end

	london_local_from_orinda_timezone: ROSE_DATE is
	local
		l_systemtime: SYSTEMTIME
	do
		Result := ut_from_timezone(orinda_tz_info)
		create l_systemtime.make_from_rose_date_time (Result)
		Result := l_systemtime.local_time_from_ut(london_tz_info)
	end

	tokyo_local_from_orinda_timezone: ROSE_DATE is
	local
		l_systemtime: SYSTEMTIME
	do
		Result := ut_from_timezone(orinda_tz_info)
		create l_systemtime.make_from_rose_date_time (Result)
		Result := l_systemtime.local_time_from_ut(tokyo_tz_info)
	end

	singapore_local_from_orinda_timezone: ROSE_DATE is
	local
		l_systemtime: SYSTEMTIME
	do
		Result := ut_from_timezone(orinda_tz_info)
		create l_systemtime.make_from_rose_date_time (Result)
		Result := l_systemtime.local_time_from_ut(singapore_tz_info)
	end

--Dang.  This just doesn't work unless you have XP or 2003.
--	ut_from_timezone(a_tz_info_pointer: MANAGED_POINTER): ROSE_DATE is
--	local
--		l_universal_time_pointer: MANAGED_POINTER
--		l_status: INTEGER
--		l_systemtime: SYSTEMTIME
--	do
--		create l_universal_time_pointer.make(record_length_)
--		l_status := TzSpecificLocalTimeToSystemTime(
--			a_tz_info_pointer.item, managed_pointer.item, l_universal_time_pointer.item)
--		if l_status=0
--		then
--			l_status := getlasterror
--			raise("TzSpecificLocalTimeToSystemTime failed with " + l_status.out)
--		end
--
--		create l_systemtime.make(l_universal_time_pointer)
--		Result := l_systemtime.date_time
--	end

	ut_from_timezone(a_tz_info: TIME_ZONE_INFORMATION): ROSE_DATE is
	do
		Result := date_time
		if daylight(a_tz_info)
		then
			Result.minute_add(a_tz_info.bias + a_tz_info.daylight_bias)
		else
			Result.minute_add(a_tz_info.bias + a_tz_info.standard_bias)
		end
	end

	local_time_from_ut(a_tz_info: TIME_ZONE_INFORMATION): ROSE_DATE is
	do
		Result := date_time
		if daylight(a_tz_info)
		then
			Result.minute_add(-(a_tz_info.bias + a_tz_info.daylight_bias))
		else
			Result.minute_add(-(a_tz_info.bias + a_tz_info.standard_bias))
		end
	end

	daylight(a_tz_info: TIME_ZONE_INFORMATION): BOOLEAN is
	require
		zero_year: a_tz_info.standard_date.wyear=0 and a_tz_info.daylight_date.wyear=0
	local
		l_date_time: ROSE_DATE
		l_date_time_mins: INTEGER
		l_standard_start_mins: INTEGER
		l_daylight_start_mins: INTEGER
	do
		if a_tz_info.standard_date.is_valid_timezone_start and
		   a_tz_info.daylight_date.is_valid_timezone_start
		then
			l_date_time := date_time
			l_date_time_mins := l_date_time.year_day * 1440 + l_date_time.hour * 60 + l_date_time.minute

			l_standard_start_mins := a_tz_info.standard_date.minutes_from_begining_of_year(wyear)

			l_daylight_start_mins := a_tz_info.daylight_date.minutes_from_begining_of_year(wyear)

			if l_daylight_start_mins<l_standard_start_mins
			then
				Result :=
					l_date_time_mins>=l_daylight_start_mins and
					l_date_time_mins<l_standard_start_mins
			elseif l_daylight_start_mins>l_standard_start_mins
			then
				Result :=
					l_date_time_mins>=l_standard_start_mins and
					l_date_time_mins<l_daylight_start_mins
				Result := not Result
			else
				raise("The timezone has the same starting time for standard and daylight.")
			end
		else
			Result := False
		end
	end

	minutes_from_begining_of_year(a_year: INTEGER): INTEGER is
	local
		l_date_time: ROSE_DATE
		l_year: INTEGER
		l_month: INTEGER
		l_day: INTEGER
		l_day_of_week: INTEGER
		l_diff: INTEGER
	do
		if wyear=0
		then
			l_year := a_year
			l_day := 1
			if wday=5
			then
				l_month := wmonth + 1
				if l_month>12
				then
					l_month := 1
					l_year := l_year + 1
				end
			else
				l_month := wmonth
			end

			create l_date_time.make_date_time(l_year, l_month, l_day, whour, wminute, wsecond)
			l_day_of_week := l_date_time.day_of_the_week - 1
			l_diff := 7 - (l_day_of_week - wdayofweek).abs
			l_diff := l_diff \\ 7
			l_date_time.add_n_days(l_diff)
			if wday=5
			then
				l_date_time.add_n_days(-7)
			else
				l_date_time.add_n_days((wday - 1) * 7)
			end
		else
			l_year := wyear
			l_month := wmonth
			l_day := wday
			create l_date_time.make_date_time(l_year, l_month, l_day, whour, wminute, wsecond)
		end

		Result := l_date_time.year_day * 1440 + l_date_time.hour * 60 + l_date_time.minute
	end

	is_valid_timezone_start: BOOLEAN is
	do
		Result := wmonth/=0
	end

	read is
	do
		wYear := read_wYear
		wMonth := read_wMonth
		wDayOfWeek := read_wDayOfWeek
		wDay := read_wDay
		wHour := read_wHour
		wMinute := read_wMinute
		wSecond := read_wSecond
		wMilliseconds := read_wMilliseconds
	end

	write is
	do
		set_wYear(wYear)
		set_wMonth(wMonth)
		set_wDayOfWeek(wDayOfWeek)
		set_wDay(wDay)
		set_wHour(wHour)
		set_wMinute(wMinute)
		set_wSecond(wSecond)
		set_wMilliseconds(wMilliseconds)
	end

	wYear: INTEGER
	read_wYear: INTEGER is
	do
		Result := short_interpreter_.decode(pointer_ + (0), 2)
	end
	set_wYear(a_value: INTEGER) is
	do
		short_interpreter_.encode(pointer_ + (0), a_value.to_integer_16, 2)
	end

	wMonth: INTEGER
	read_wMonth: INTEGER is
	do
		Result := short_interpreter_.decode(pointer_ + (2), 2)
	end
	set_wMonth(a_value: INTEGER) is
	do
		short_interpreter_.encode(pointer_ + (2), a_value.to_integer_16, 2)
	end

	wDayOfWeek: INTEGER
	read_wDayOfWeek: INTEGER is
	do
		Result := short_interpreter_.decode(pointer_ + (4), 2)
	end
	set_wDayOfWeek(a_value: INTEGER) is
	do
		short_interpreter_.encode(pointer_ + (4), a_value.to_integer_16, 2)
	end

	wDay: INTEGER
	read_wDay: INTEGER is
	do
		Result := short_interpreter_.decode(pointer_ + (6), 2)
	end
	set_wDay(a_value: INTEGER) is
	do
		short_interpreter_.encode(pointer_ + (6), a_value.to_integer_16, 2)
	end

	wHour: INTEGER
	read_wHour: INTEGER is
	do
		Result := short_interpreter_.decode(pointer_ + (8), 2)
	end
	set_wHour(a_value: INTEGER) is
	do
		short_interpreter_.encode(pointer_ + (8), a_value.to_integer_16, 2)
	end

	wMinute: INTEGER
	read_wMinute: INTEGER is
	do
		Result := short_interpreter_.decode(pointer_ + (10), 2)
	end
	set_wMinute(a_value: INTEGER) is
	do
		short_interpreter_.encode(pointer_ + (10), a_value.to_integer_16, 2)
	end

	wSecond: INTEGER
	read_wSecond: INTEGER is
	do
		Result := short_interpreter_.decode(pointer_ + (12), 2)
	end
	set_wSecond(a_value: INTEGER) is
	do
		short_interpreter_.encode(pointer_ + (13), a_value.to_integer_16, 2)
	end

	wMilliseconds: INTEGER
	read_wMilliseconds: INTEGER is
	do
		Result := short_interpreter_.decode(pointer_ + (14), 2)
	end
	set_wMilliseconds(a_value: INTEGER) is
	do
		short_interpreter_.encode(pointer_ + (14), a_value.to_integer_16, 2)
	end

	record_length_: INTEGER is 16


feature {NONE}
	managed_pointer: MANAGED_POINTER


feature {NONE}
	local_tz_info: TIME_ZONE_INFORMATION is
	once
		create Result.make_local
	end

	orinda_tz_info: TIME_ZONE_INFORMATION is
	once
		create Result.make_orinda
	end

	london_tz_info: TIME_ZONE_INFORMATION is
	once
		create Result.make_london
	end

	tokyo_tz_info: TIME_ZONE_INFORMATION is
	once
		create Result.make_tokyo
	end

	singapore_tz_info: TIME_ZONE_INFORMATION is
	once
		create Result.make_singapore
	end

feature {NONE}

	copy2bytes(a_source, a_destination: POINTER) is
	do
		a_destination.memory_copy(a_source, 2)
	end

	copy4bytes(a_source, a_destination: POINTER) is
	do
		a_destination.memory_copy(a_source, 4)
	end


feature {NONE}

	GetTimeZoneInformation (a_pointer: POINTER): INTEGER is
			-- (export status {NONE})
		external
			"C blocking signature (LPTIME_ZONE_INFORMATION): EIF_INTEGER_32 use <windows.h>"
		alias
			"GetTimeZoneInformation"
		end

	FileTimeToSystemTime(a_ft_pointer, a_st_pointer: POINTER): BOOLEAN is
	external
		"C [macro <windows.h>] (FILETIME*, LPSYSTEMTIME): EIF_BOOLEAN"
	alias
		"FileTimeToSystemTime"
	end

--Dang.  This is XP/2003 specific code.
--	TzSpecificLocalTimeToSystemTime(
--		a_tz_info, a_localtime, a_universal_time: POINTER): INTEGER is
--	external
--		"C [macro <windows.h>] (void*, void*, void*): EIF_INTEGER_32"
--	alias
--		"TzSpecificLocalTimeToSystemTime"
--	end

	SIZE_OF_TIME_ZONE_INFORMATION: INTEGER is
	external
		"C [macro <windows.h>] (): EIF_INTEGER_32"
	alias
		"sizeof(TIME_ZONE_INFORMATION)"
	end

	SIZE_OF_SYSTEMTIME: INTEGER is
	external
		"C [macro <windows.h>] (): EIF_INTEGER_32"
	alias
		"sizeof(SYSTEMTIME)"
	end

	SIZE_OF_FILETIME: INTEGER is
	external
		"C [macro <windows.h>] (): EIF_INTEGER_32"
	alias
		"sizeof(FILETIME)"
	end

	getlasterror: INTEGER is
	external
		"C [macro <windows.h>] (): EIF_INTEGER_32"
	alias
		"GetLastError()"
	end

end
