indexing
	description: "Represents a column for ROSE_FORMAT"
	author: "Mark Howard"
	date: "$Date: 2009/09/25 $"
	revision: "$Revision: #5 $"
	archive: "$File: //rose/source/kernel/infrastructure/text/text_column_info.e $"

class
	TEXT_COLUMN_INFO

inherit
	ROSE_CONSTANT_VALUES_AWARE

create
	make

feature {NONE} -- Implementation

	make (a_type: INTEGER; a_name: STRING; a_first, a_second: INTEGER) is
			-- Create with column name 'a_name' of type 'a_type'
			-- 'first' and 'second' is the absolute positions of this columns in the parsed input file
			-- 'type needs to be a basic type, as specified in 'ROSE_CONSTANT_VALUES'
			-- When a record is retrieve in an object, the formatter (ROSE_FORMAT)
			-- will try to map 'name' with the attribute name of the target type
		require
			valid_type: constant_values.is_valid_type (a_type)
			name_not_void_or_empty: (a_name /= Void) and then (not a_name.is_empty)
			first_before_second: a_first >=0 and a_second >= 0 and a_first <= a_second
		do
			type := a_type
			name := a_name
			first := a_first
			second := a_second
		ensure
			type_set: type = a_type
			name_set: name = a_name
			first_set: first = a_first
			second_set: second = a_second
		end

feature -- Access

	type: INTEGER
			-- Type of the column (see ROSE_CONSTANT_VALUES.*_type for possible values)

	name: STRING
			-- Name of the field

	first: INTEGER
		-- First character index for the field

    second: INTEGER
    	-- Last character index for the field

invariant

	valid_type: constant_values.is_valid_type (type)
	name_not_void_or_empty: (name /= Void) and then (not name.is_empty)

end
