note
	description: "Converter of date time from any time zone to any time zone"
	author: "Emmanuel Bouyer"
	last_editor: "$Author: e2 $"
	date: "$Date: 2013/10/03 $"
	revision: "$Revision: #1 $"
	path: "$File: //rose/source/kernel/infrastructure/windows/time_zone_converter.e $"

class
	TIME_ZONE_CONVERTER

inherit
	ELOG_SHARED_LOGGER
		export {NONE} all end
	ROSE_SHARED_GLOBAL

feature -- Access

	time_difference (a_zone1, a_zone2: STRING): DATE_TIME_DURATION
			-- Time difference between `a_zone1' and `a_zone2'
			-- Both strings are keys in the windows registry under `registry_root_path'
			-- see TIME_ZONE_NAMES for examples
			-- You need to ADD this time difference to a time expressed in `a_zone1' to get it in `a_zone2'
		require
			a_zone1_attached: attached a_zone1
			a_zone2_attached: attached a_zone2
		do
			Result := time_difference_with_utc (a_zone2) - time_difference_with_utc (a_zone1)
		ensure
			time_difference_attached: attached Result
			less_than_one_day: Result.date.days_count = 0
			symmetry: Result ~ -time_difference (a_zone2, a_zone1)
		end

	time_difference_with_utc (a_local_time_code: STRING): DATE_TIME_DURATION
			-- Time difference between UTC and `a_local_time_code'
			-- You need to ADD this time difference to a UTC time to get the time in `a_local_time_code'
		require
			a_local_time_code_attached: attached a_local_time_code
		local
			l_zone: ROSE_REGISTRY_KEY_VALUE
			l_registry: ROSE_REGISTRY_ISE_IMPLEMENTATION
			l_rose_utc: ROSE_DATE
			l_utc, l_local_time: C_STRUCT_SYSTEMTIME
			l_timezone_info: C_STRUCT_TIME_ZONE_INFORMATION
		do
			create l_rose_utc.make_today
			create l_registry
			create l_utc.set_from_date (l_rose_utc)
			create l_local_time.make
			l_zone := l_registry.open_key_value (registry_root_path + a_local_time_code, TZI_key)
			if l_zone /= Void then
				create l_timezone_info.make_from_reg_tzi_format (l_zone.data)
				if system_time_to_TZ_local (l_timezone_info.item, l_utc.item, l_local_time.item) then
					Result := l_local_time.date.relative_duration (l_rose_utc)
				else
					logger.error ("Call to SystemTimeToTzSpecificLocalTime ended with error " +global.env.last_system_call_status.out)
				end
			else
				logger.error ("Unknown timezone '"+a_local_time_code+"'")
			end
		ensure
			time_difference_with_utc_attached: attached Result
			less_than_one_day: Result.date.days_count = 0
		end

feature {NONE} -- Implementation

	registry_root_path: STRING = "HKEY_LOCAL_MACHINE\Software\Microsoft\Windows NT\CurrentVersion\Time Zones\"
	TZI_key: STRING = "TZI"

	system_time_to_TZ_local (a_zone: POINTER; a_utc_time: POINTER; a_local_time: POINTER): BOOLEAN
			-- Converts `a_utc_time' into `a_local_time' for `a_zone'
		external
			"C blocking signature (LPTIME_ZONE_INFORMATION, LPSYSTEMTIME, LPSYSTEMTIME): BOOL use <winbase.h>"
		alias "SystemTimeToTzSpecificLocalTime"
		end

end
