indexing
	description: "The Windows structure TIME_ZONE_INFORMATION"
	author: "RJ"
	date: "$Date: 2008/12/01 $"
	revision: "$Revision: #5 $"
	archive: "$File: //rose/source/kernel/infrastructure/windows/time_zone_information.e $"

class
	TIME_ZONE_INFORMATION

inherit
	EXTERNAL_RECORD
		redefine
			default_create,
--			display,
			write,
			read,
			record_length_
		end
	C_ACCESS_SUPPORT
		redefine
			default_create
		end
	EXCEPTIONS
		export
			{NONE}
				all
		redefine
			default_create
		end

create
	default_create,
	make_local,
	make_orinda,
	make_london,
	make_tokyo,
	make_singapore

feature
	default_create is
	do
		create managed_pointer.make(record_length_)
		set(managed_pointer.item)
	end

	make_local is
	local
		l_status: INTEGER
	do
		default_create
		l_status := GetTimeZoneInformation(managed_pointer.item)
		read
	end

	make_orinda is
	do
		bias := 8 * 60
		create standard_date.make_orinda_standard_timezone_start
		standard_bias := 0
		create daylight_date.make_orinda_daylight_timezone_start
		daylight_bias := -60
	end

	make_london is
	do
		bias := 0
		create standard_date.make_london_standard_timezone_start
		standard_bias := 0
		create daylight_date.make_london_daylight_timezone_start
		daylight_bias := -60
	end

	make_tokyo is
	do
		bias := -9 * 60
		create standard_date.make_nodst_timezone_start
		standard_bias := 0
		create daylight_date.make_nodst_timezone_start
		daylight_bias := 0
	end

	make_singapore is
	do
		bias := -8 * 60
		create standard_date.make_nodst_timezone_start
		standard_bias := 0
		create daylight_date.make_nodst_timezone_start
		daylight_bias := 0
	end

	read is
	do
		bias := read_bias
		standard_name := read_standard_name
		standard_date := read_standard_date
		standard_date.read
		standard_bias := read_standard_bias
		daylight_name := read_daylight_name
		daylight_date := read_daylight_date
		daylight_date.read
		daylight_bias := read_daylight_bias
	end

	write is
	do
		set_bias(bias)
		set_standard_name(standard_name)
		set_standard_date(standard_date)
		set_standard_bias(standard_bias)
		set_daylight_name(daylight_name)
		set_daylight_date(daylight_date)
		set_daylight_bias(daylight_bias)
	end

	bias: INTEGER
	read_bias: INTEGER is
	do
		Result := integer_interpreter_.decode(pointer_ + (0), 4)
	end
	set_bias(a_value: INTEGER) is
	do
		integer_interpreter_.encode(pointer_ + (0), a_value, 4)
	end

	standard_name: STRING	-- 32 wide characters (64 bytes) starts at 4.
	read_standard_name: STRING is
	do
	end
	set_standard_name(a_value: STRING) is
	do
	end

	standard_date: SYSTEMTIME
	read_standard_date: SYSTEMTIME is
	do
		create Result.make_from_pointer(pointer_ + (68))
	end
	set_standard_date(a_value: SYSTEMTIME) is
	do
	end

	standard_bias: INTEGER
	read_standard_bias: INTEGER is
	do
		Result := integer_interpreter_.decode(pointer_ + (84), 4)
	end
	set_standard_bias(a_value: INTEGER) is
	do
		integer_interpreter_.encode(pointer_ + (84), a_value, 4)
	end

	daylight_name: STRING	-- 32 wide characters (64 bytes) starts at 88.
	read_daylight_name: STRING is
	do
	end
	set_daylight_name(a_value: STRING) is
	do
	end

	daylight_date: SYSTEMTIME
	read_daylight_date: SYSTEMTIME is
	do
		create Result.make_from_pointer(pointer_ + (152))
	end
	set_daylight_date(a_value: SYSTEMTIME) is
	do
	end

	daylight_bias: INTEGER
	read_daylight_bias: INTEGER is
	do
		Result := integer_interpreter_.decode(pointer_ + (168), 4)
	end
	set_daylight_bias(a_value: INTEGER) is
	do
		integer_interpreter_.encode(pointer_ + (168), a_value, 4)
	end

	record_length_: INTEGER is
	once
		Result := SIZE_OF_TIME_ZONE_INFORMATION
	end

feature {NONE}
	managed_pointer: MANAGED_POINTER

feature {NONE}

	SIZE_OF_TIME_ZONE_INFORMATION: INTEGER is
		external
			"C blocking macro signature (): EIF_INTEGER_32 use <windows.h>"
		alias
			"sizeof(TIME_ZONE_INFORMATION)"
		end

	GetTimeZoneInformation (a_pointer: POINTER): INTEGER is
			-- (export status {NONE})
		external
			"C blocking signature (LPTIME_ZONE_INFORMATION): EIF_INTEGER_32 use <windows.h>"
		alias
			"GetTimeZoneInformation"
		end

end
