indexing
	description: "Tridiagonal matrix ... algorithms are from Numerical Recipes"
	author: "Mark Howard"
	date: "$Date: 2008/12/01 $"
	revision: "$Revision: #7 $"
	archive: "$File: //rose/source/kernel/infrastructure/math/tridiagonal_matrix.e $"

class
	TRIDIAGONAL_MATRIX
inherit
	MATRIX
		rename
			make as matrix_make
		end
create
	make

create {TRIDIAGONAL_MATRIX}

	matrix_make,
	make_identity,
	make_as_inverse

feature --Creation
	make (a : ABSTRACT_READABLE_MATRIX) is
		require
			is_symmetric : a.is_symmetric
		local
			i, j, k,l,n: INTEGER
			l_scale, l_h: DOUBLE
			f, g, hh: DOUBLE
		do
			n := a.width
			matrix_make(n,n)
			create diagonal_elements.make(n)
			create off_diagonal_elements.make(n)
			from i := 1 until i > n loop
				from j := 1 until j > n loop
					Current[i,j] := a[i,j]
					j := j + 1
				end
				i := i + 1
			end
			from i := n until i < 2 loop
				l := i-1
				l_h := 0
				l_scale := 0
				if l > 1 then
					from k := 1 until k > l loop
						l_scale := l_scale + (Current[i,k]).abs
						k := k + 1
					end
					if l_scale = 0.0 then
						off_diagonal_elements[i] := Current[i,l]
					else
						from k := 1 until k > l loop
							Current[i,k] := Current[i,k] / l_scale
							l_h := l_h + Current[i,k]*Current[i,k]
							k := k + 1
						end
						f := Current[i,l]
						if f >= 0 then
							g := -sqrt(l_h)
						else
							g := sqrt(l_h)
						end
						off_diagonal_elements[i] := l_scale * g
						l_h := l_h - f*g
						Current[i,l] := f-g
						f := 0
						from j := 1 until j > l loop
							Current[j,i] := Current[i,j] / l_h
							g := 0
							from k := 1 until k > j loop
								g := g + Current[j,k]*Current[i,k]
								k := k + 1
							end
							from k := j + 1 until k > l loop
								g := g + Current[k,j]*Current[i,k]
								k := k + 1
							end
							off_diagonal_elements[j] := g/l_h
							f := f + off_diagonal_elements[j]*Current[i,j]
							j := j + 1
						end
						hh := f/(l_h+l_h)
						from j := 1 until j > l loop
							f := Current[i,j]
							off_diagonal_elements[j] := off_diagonal_elements[j]-hh*f
							g := off_diagonal_elements[j]
							from k := 1 until k > j loop
								Current[j,k] := Current[j,k] - (f*off_diagonal_elements[k]+g*Current[i,k])
								k := k + 1
							end
							j := j + 1
						end
					end
				else
					off_diagonal_elements[i] := Current[i,l]
				end
				diagonal_elements[i] := l_h
				i := i - 1
			end
			diagonal_elements[1] := 0
			off_diagonal_elements[1] := 0
			from i := 1 until i > n loop
				l := i-1
				if diagonal_elements[i] /= 0 then
					from j := 1 until j > l loop
						g := 0
						from k := 1 until k > l loop
							g := g + Current[i,k]*Current[k,j]
							k := k + 1
						end
						from k := 1 until k > l loop
							Current[k,j] := Current[k,j] - g*Current[k,i]
							k := k + 1
						end
						j := j + 1
					end
				end
				diagonal_elements[i] := Current[i,i]
				Current[i,i] := 1.0
				from j := 1 until j > l loop
					Current[j,i] := 0
					Current[i,j] := 0
					j := j + 1
				end
				i := i + 1
			end
		end
feature --Properties
	diagonal_elements : VECTOR
	off_diagonal_elements : VECTOR
end
