note

	description:

		"ROSE_EXPRESSION with the ability to compute values relative to a universe"

	author: "Mark Howard"
	last_editor: "$Author: mk $"
	date: "$Date: 2011/02/15 $"
	revision: "$Revision: #11 $"
	path: "$File: //rose/source/kernel/math/universe_expression.e $"

deferred class
	UNIVERSE_EXPRESSION[reference G]
inherit
	ROSE_EXPRESSION[G]
feature -- Computed values
	computed_universe_double(a_target : G;a_universe : like universe_target_template; a_agent : like access_agent_template) : DOUBLE is
			--computed value relative to universe
		do
			Result := generated_double(a_target,a_agent)
		end
	computed_rt_universe_double(a_target : G;a_universe : like universe_target_template; a_agent : like access_agent_template) : DOUBLE is
			--computed realtime value relative to universe
		do
			Result := computed_universe_double(a_target, a_universe,a_agent)
		end

	computed_universe_numeric_value (a_target : G;a_universe : like universe_target_template; a_agent : like access_agent_template): DOUBLE is
			--computed universe numeric value
		local
			l_real: REAL
			l_integer: INTEGER
		do
			if is_real then
				l_real := computed_universe_real (a_target,a_universe, a_agent)
				if is_defined_real (l_real) then
					Result := l_real
				else
					Result := undefined_double
				end
			elseif is_integer then
				l_integer := computed_universe_integer (a_target,a_universe, a_agent)
				if is_defined_integer (l_integer) then
					Result := l_integer
				else
					Result := undefined_double
				end
			else
				Result := computed_universe_double (a_target,a_universe, a_agent)
			end
		end
	computed_universe_real(a_target : G;a_universe : like universe_target_template; a_agent :like access_agent_template) : REAL is
		do
			Result := double_value(a_target).truncated_to_real
		end

	computed_universe_integer(a_target : G;a_universe :like universe_target_template; a_agent :like access_agent_template) : INTEGER is
			-- Computed universe integer
		do
			Result := integer_value(a_target)
		end

	computed_universe_string(a_target : G;a_universe : like universe_target_template; a_agent :like access_agent_template) : STRING is
		local
		do
			Result := string_value(a_target)
		end

feature -- Statistics
	universe_moments (a_target : G;a_universe : like universe_target_template; a_weight_expression: ROSE_EXPRESSION[ANY]): like moments_anchor is
			-- Moments for targets in 'a_universe'
		local
			l_index: INTEGER
 			l_universe_target : G
			l_universe : ROSE_LINEAR_ARRAY[G]
			l_data, l_weights : ROSE_LINEAR_ARRAY[DOUBLE]
			l_datum, l_weight: DOUBLE
		do
			l_universe := a_universe.universe_targets
			if l_universe /= Void and then not l_universe.is_empty then
				create l_data.make_empty
				create l_weights.make_empty
				from l_index := l_universe.lower until l_index > l_universe.upper loop
					l_universe_target := l_universe.item (l_index)
					l_datum := Current.get_real_value(l_universe_target)
					l_weight := a_weight_expression.get_real_value(l_universe_target)
					if l_weight > 0.0 and is_defined_double(l_weight) and then is_defined_double(l_datum) then
						l_data.extend(l_datum)
						l_weights.extend(l_weight)
					end
					l_index := l_index + 1
				end
				create Result.make(l_data,l_weights)
				moments_cache.force(Result, a_universe.key_tuple([name, a_weight_expression.name]))
			end
		end


feature -- Computed universe statistics


	computed_universe_moments (a_target : G;a_universe : like universe_target_template; a_weight_expression: ROSE_EXPRESSION [G];
							   a_agent: like access_agent_template): like moments_anchor is
		local
			l_index: INTEGER
 			l_universe_target : G
			l_universe : ROSE_LINEAR_ARRAY[G]
			l_data, l_weights : ROSE_LINEAR_ARRAY[DOUBLE]
			l_datum, l_weight: DOUBLE
		do
			l_universe := a_universe.universe_targets
			if l_universe /= Void and then not l_universe.is_empty then
				create l_data.make_empty
				create l_weights.make_empty
				from l_index := l_universe.lower until l_index > l_universe.upper loop
					l_universe_target := l_universe.item (l_index)
					l_datum := generated_numeric_value(l_universe_target,a_agent)
					l_weight := a_weight_expression.generated_numeric_value(l_universe_target,a_agent)
					if l_weight > 0.0 and is_defined_double(l_weight) and then
						is_defined_double(l_datum) then
						l_data.extend(l_datum)
						l_weights.extend(l_weight)
					end
					l_index := l_index + 1
				end
				create Result.make(l_data,l_weights)
			end
		end

	generated_universe_moments (a_target : G;a_universe : like universe_target_template; a_weight_expression: ROSE_EXPRESSION [G];
							 	a_agent: like access_agent_template): like moments_anchor is
		local
			l_key: TUPLE
		do
			l_key := a_universe.key_tuple ([name, a_weight_expression, a_agent])
			moments_cache.search (l_key)
			if moments_cache.found then
				Result := moments_cache.found_item
			else
				Result := computed_universe_moments (a_target, a_universe,a_weight_expression, a_agent)
				moments_cache.force (Result, l_key)
			end
		end

feature --Templates

	universe_target_template : UNIVERSE_TARGET[G] is
			--template
		do
		end

end
