indexing
	description: "Target represent a universe"
	author: "Mauricio Bustos"
	date: "$Date: 2008/12/01 $"
	revision: "$Revision: #18 $"
	path: "$File: //rose/source/kernel/targets/universe_target.e $"

deferred class
	UNIVERSE_TARGET[reference G]

inherit
	MISMATCH_CORRECTOR
		redefine
			correct_mismatch
		end
feature -- Initialization

	set_universe(a_value : like universe; a_name : STRING; a_date : ROSE_DATE) is
		-- reset the universe
		do
			universe := a_value
			name := a_name
			universe_date := a_date
			universe_targets_saved := Void
		end

feature -- Access
	name : STRING
			-- Name
	universe : ROSE_LINEAR_ARRAY[G]
	universe_date : ROSE_DATE

	target_at(a_ix : INTEGER) : G is
			--item of universe at a_ix
		do
			if universe.valid_index (a_ix) then
				Result := universe.item(a_ix)
			end
		end

feature -- Navigation

	index : INTEGER is do Result := universe.index end
	start is do universe.start end
	forth is do universe.forth end
	off : BOOLEAN is do Result := universe.off end
	current_target : G is do Result := universe.current_item end
	current_universe_target : G is do Result := directed_universe_target(universe.current_item) end
	key : TUPLE[name : STRING;date : ROSE_DATE] is
		do
			Result := [name, universe_date]
			Result.compare_objects
		end
	key_tuple(a_tuple : TUPLE) : TUPLE is
		do
			Result := [a_tuple, name, universe_date]
			Result.compare_objects
		end

feature -- Directed target

	directed_universe_target(a_target : G) : G is
		deferred
		end

	universe_target_at(a_ix : INTEGER) : like directed_universe_target is
		do
			if universe.valid_index (a_ix) then
				Result := directed_universe_target (universe.item(a_ix))
			end
		end

	universe_targets : ROSE_LINEAR_ARRAY [like directed_universe_target] is
			--array of universe targets
		local
			l_index: INTEGER
		do
			if universe_targets_saved = Void then
				create universe_targets_saved.make(universe.lower, universe.upper)
				from l_index := universe.lower until l_index > universe.upper loop
					universe_targets_saved.put(directed_universe_target(universe.item(l_index)), l_index)
					l_index := l_index + 1
				end
			end
			Result := universe_targets_saved
		end

	target : G is deferred end

feature {NONE} -- computed universe features

--	computed_universe_double(a_expression :  like universe_expression_target; a_agent : like access_agent_template) : DOUBLE is
--		do
--			Result := a_expression.computed_universe_double(Current, a_agent)
--		end

--	computed_universe_real(a_expression :  like universe_expression_target; a_agent :  like access_agent_template) : REAL is
--		do
--			Result := a_expression.computed_universe_double(Current, a_agent)
--		end
--feature
--	computed_universe_integer(a_expression :  like universe_expression_target; a_agent :  like access_agent_template) : INTEGER is
--		do
--			Result := a_expression.computed_universe_integer(Current, a_agent)
--		end

--	computed_universe_string_(a_expression :  like universe_expression_target; a_agent :  like access_agent_template) : STRING is
--		do
--			Result := a_expression.computed_universe_string(Current, a_agent)
--		end


feature {NONE} -- Implementation
	universe_targets_saved: ROSE_LINEAR_ARRAY [like directed_universe_target]
	universe_expression_target : UNIVERSE_EXPRESSION[G] is do end
	access_agent_template : GENERIC_ACCESS_CONTROL_AGENT is do  end

feature -- correct mismatch
	correct_mismatch is
			-- Attempt to correct object mismatch using `mismatch_information'.
		local
			l_mismatch: MISMATCH_INFORMATION
		do
			l_mismatch := mismatch_information
		end
end

