note

	description:

		"Underlying implementation of 2 dimensional data as SPECIAL[SPECIAL[DOUBLE]]"

	author: "Mark Howard"
	last_editor: "$Author: mk $"
	date: "$Date: 2012/08/20 $"
	revision: "$Revision: #1 $"
	path: "$File: //rose/source/kernel/infrastructure/math/uses_2d_special_data.e $"

deferred class
	USES_2D_SPECIAL_DATA
inherit
	ROSE_IEEE_UNDEFINED
	LABELED_2D

feature {NONE} --Creation

	make (a_height, a_width: INTEGER) is
			-- Make a matrix with size `a_height, `a_width' and filled with zeroes.
		require
			a_height_non_negative: a_height >= 0
			a_width_non_negative: a_width >= 0
		do
			make_filled (a_height, a_width, 0.0)
		end
	make_filled (a_height, a_width: INTEGER; a_value: DOUBLE)
			-- Make a matrix with size `a_height, `a_width' and filled with `a_value'.
		require
			a_height_non_negative: a_height >= 0
			a_width_non_negative: a_width >= 0
		local
			i: INTEGER
		do
			height := a_height
			width := a_width
			create data.make_filled (Void, a_height + 1)
			from
				i := 1
			until
				i > height
			loop
				data.put (create {SPECIAL [DOUBLE]}.make_filled (a_value, width + 1), i)
				i := i + 1
			end
		end
	make_identity (a_count: INTEGER) is
			-- Make identity matrix of height `a_count'.
		require
			a_count_non_negative: a_count >= 0
		local
			i: INTEGER
		do
			from
				make (a_count, a_count)
				i := 1
			until
				i > a_count
			loop
				put (1, i, i)
				i := i + 1
			end
		end

feature --Access

	data : SPECIAL[SPECIAL[DOUBLE]]
	height : INTEGER
	width : INTEGER
	item alias "[]" (i,j : INTEGER) : DOUBLE assign put
			--Access
		local
			l_row : SPECIAL[DOUBLE]
		do
			if i >= 1 and then i <= height and then j >= 1 and then j <= width then
				l_row := data[i]
				Result := l_row[j]
			else
				Result := undefined_double
			end
		end
	put (a_value : DOUBLE;i,j : INTEGER)
			--Update
		local
			l_row : SPECIAL[DOUBLE]
		do
			if i >= 1 and then i <= height and then j >= 1 and then j <= width then
				l_row := data[i]
				l_row[j] := a_value
			end
		end
	replace_diagonal_with_tolerance (a_tolerance: DOUBLE)
			-- Replace diagonal entries which are undefined doubles or lower than `a_tolerance' with 1.
		local
			i:INTEGER
		do
			from
				i:= 1
			until
				i > height.min (width)
			loop
				if Current [i, i] < a_tolerance or not is_defined_double(Current [i,i]) then
					Current [i, i] := 1.0
				end
				i := i + 1
			end
		end
	replace_diagonal
			-- Replace diagonal entries which are undefined doubles or zero with 1.
		local
			i:INTEGER
		do
			from i:=1 until i>height loop
				if Current[i,i]=0 or not is_defined_double (Current[i,i]) then
					Current[i, i] := 1
				end
				i:=i+1
			end
		end

	replace_null is
			-- Replace undefined doubles and null values with zero.
		local
			i,j:INTEGER
			l_v:DOUBLE
		do
			from i:=1 until i> height loop
				from  j:=1 until j>width loop
					l_v:=Current[i,j]
					if not is_defined_double (l_v) then
						Current[i, j] := 0.0
					end
					j:=j+1
				end
				i:=i+1
			end
		end
	scalar_multiply,scaled (a_value : DOUBLE) : like Current is
			-- returns a matrix like current, but with every element multiplied by `a_value'
		deferred
		end
end
