note

	description:

		"Variable information necessary to create an expression"

	storable_version: ""
	author: "Mark Howard"
	last_editor: "$Author: mb $"
	date: "$Date: 2014/02/19 $"
	revision: "$Revision: #41 $"
	path: "$File: //rose/source/kernel/expressions/variable_description.e $"

class VARIABLE_DESCRIPTION

inherit

	VARIABLE_DESCRIPTION_CONSTANTS
		export
			{NONE} all
		undefine
			is_equal
		end

	DEBUG_OUTPUT
		undefine
			is_equal
		end

	HASHABLE
		redefine
			is_equal
		end

	ROSE_CONSTANTS
		undefine
			is_equal
		end

creation

	default_create,
	make--,
--	make_simple

feature {NONE} -- Initialization

	make (a_name: STRING; a_mnemonic: STRING; a_source: STRING;  a_weighting: ANY; a_period: ANY; a_result_string: ANY; a_parent: STRING;
		 a_j_index: INTEGER; a_gv_index: INTEGER; a_spcs_index: INTEGER; a_risk_index: INTEGER; a_security: INTEGER; a_units: STRING; a_type: STRING;
		 a_lower_winsor: DOUBLE; a_upper_winsor: DOUBLE; a_default_mean: DOUBLE; a_default_standard_deviation: DOUBLE; a_expression: STRING;
		 a_rose_name: ANY; a_user_name: ANY; a_scale: STRING; a_target: STRING; a_stock_or_flow: CHARACTER; a_target_name: ANY;
		 a_group_name: STRING; a_variable_index: INTEGER) is
			-- Creation procedure, don't use
		require
			known_scale: known_scale(a_scale)
			known_target: known_target(a_target)
			known_unit: known_unit(a_units)
			known_type: known_type(a_type)
			known_target_name: known_target_name(a_target_name)
			known_weighting: known_weighting(a_weighting)
		do
			name := a_name
			source := a_source
			mnemonic := a_mnemonic
			name := a_name
			parent := a_parent
			set_j_index (a_j_index)
			set_gv_index (a_gv_index)
			set_spcs_index (a_spcs_index)
			set_risk_index (a_risk_index)
			security := a_security
			units := a_units
			type := a_type
			expression := a_expression
			scale := a_scale
			target := a_target


		--	target_name := a_target_name
			stock_or_flow := a_stock_or_flow
			variable_index := a_variable_index
		end

feature -- Access

	mnemonic: STRING
			-- Mnemonic
	name: STRING
		-- Name
	variable_index: INTEGER
	expression: STRING
		-- Expression
	source: STRING
		-- Source
	parent: STRING
	type: STRING
            -- hold the type
    is_type_string: BOOLEAN is
            -- true if type is equal to string
        do
            Result := type.is_case_insensitive_equal ("STRING")
        end

    is_type_real: BOOLEAN is
            -- true if type is equal to real
        do
            Result := type.is_case_insensitive_equal ("REAL")
        end

    is_type_integer: BOOLEAN is
            -- true if type is equal to integer
        do
            Result := type.is_case_insensitive_equal ("INTEGER")
        end

    is_type_double: BOOLEAN is
            -- true if type is equal to double
        do
            Result := type.is_case_insensitive_equal ("DOUBLE")
        end

    is_type_date: BOOLEAN is
            -- true if type is equal to date
        do
            Result := type.is_case_insensitive_equal ("DATE")
        end

	scale: STRING
	security: INTEGER	-- Not used anymore.  Values of 1, 2 or 3.  Used to restrict access to certain variables.  Didn't work well.
	units: STRING
	target: STRING
	stock_or_flow: CHARACTER
			-- S or F.

	source_table: STRING
			---- This corresponds to a column that indicates the name of the record table this variable is stored in (for record_storage)
			---- or have value "item", if this variable is stored in item_table format
			---- used by signature_tagged_expression
	source_column: STRING
			----- This will be null for variables stored in item format,
			----- and will have the column name for variables that are stored in records
	expression_type: STRING
			----- if there are several types of expressions to be created for a given entity_type,
			----- this is what we will use to determine the type of expression to create,
			----- can be null if there is only one kind of expression associated with that entity_type.

	key_fields: STRING is
			-- Comma delimited string of important attribute values
		do
			Result := "mnemonic,name,variable_index,expression,source,parent,type,scale,security,units,target,stock_or_flow,source_table"
		ensure
			valid_result: Result /= Void
		end

	key_fields_array: ROSE_LINEAR_ARRAY [STRING] is
			-- Array of key attribute names
		local
			l_delimited: ROSE_DELIMITED_TEXT
		do
			create l_delimited.make_removing_spaces (key_fields, ',')
			Result := l_delimited
		ensure
			valid_result: Result /= Void
		end

	formatted_out: STRING is
			-- Displayable string for output
		do
			Result := "%N%TIndex:%T%T" + variable_index.out
			if (type /= Void) then
				Result := Result + "%N%TType:%T%T" + type
			end
		end

	target_name: STRING is
			--defined to be meaningful in descendents ... should probably make it an attribute here but would break storables :(
		do
		end

	debug_output: STRING is
			-- <Precursor>
		do
			Result := name
		end

	hash_code: INTEGER is
			-- any of these should work ... doesn't matter which we use
		do
			if mnemonic /= Void then
				Result := mnemonic.hash_code
			elseif name /= Void then
				Result := name.hash_code
			else
				Result := variable_index.hash_code
			end
		end

	is_equal (a_other: like Current): BOOLEAN is
			-- we consider two descriptions to be the same if they have the same name, mnemonic and variable_index
		do
			Result := a_other /= Void and then variable_index = a_other.variable_index and then mnemonic ~ a_other.mnemonic and then name ~ a_other.name
		end

	string_value_of_attribute (a_name: STRING): STRING is
			-- String representation of the attribute 'a_name'
		require
			valid_name: a_name /= Void
		local
			i: INTEGER
			l_integer_ref: INTEGER_REF
			l_double_ref: DOUBLE_REF
			l_any: ANY
		do
			from
				i := 1
			until
				i > internal.field_count (Current) or
				Result /= Void
			loop
				if internal.field_name (i, Current).is_case_insensitive_equal (a_name) then
					l_any := internal.field (i, Current)
					if l_any /= Void then
						l_integer_ref ?= l_any
						l_double_ref ?= l_any
						if l_integer_ref /= Void then
							if l_integer_ref.item /= constants.missing_integer then
								Result := l_any.out
							else
								Result := Void
							end
						elseif l_double_ref /= Void then
							if is_defined_double (l_double_ref.item) then
								Result := format_double.formatted (l_double_ref.item)
							else
								Result := Void
							end
						else
							Result := l_any.out
						end
						if Result /= Void then
							Result.prune_all_trailing (' ')
						end
					end
				end
				i := i + 1
			end
		end

	difference_description (a_other: VARIABLE_DESCRIPTION): STRING is
			-- A display of what is different to 'a_other'
		require
			valid_other: a_other /= Void
		local
			l_keys: ROSE_LINEAR_ARRAY [STRING]
			l_key: STRING
		do
			l_keys := key_fields_array
			from
				create Result.make_empty
				l_keys.start
			until
				l_keys.off
			loop
				l_key := l_keys.current_item.twin
				from until l_key.count > 15 loop
					l_key.append (" ")
				end
				l_key.append (":")
				add_string_diff (Result, l_key, string_value_of_attribute (l_keys.current_item), a_other.string_value_of_attribute (l_keys.current_item))
				l_keys.forth
			end
			if not Result.is_empty then
				Result := "Changes:%N%N" + Result
			end
		ensure
			valid_result: Result /= Void
		end

feature -- Element modification

	set_j_index (a_value: INTEGER) is
			-- Set the J index.
		do
		end

	set_gv_index (a_value: INTEGER) is
			-- Set the GV index.
		do
		end

	set_spcs_index (a_value: INTEGER) is
			-- Set the spcs index.
		do
		end

	set_risk_index (a_value: INTEGER) is
			-- Set the risk index.
		do
		end

	set_mnemonic (a_value: like mnemonic) is
			-- Set `mnemonic' to `a_value'.
		do
			mnemonic := a_value
		end

	set_name (a_value: like name) is
			-- Set `name' to `a_value'.
		do
			name := a_value
		end

	set_variable_index (a_value: like variable_index) is
			-- Set `variable_index' to `a_value'.
		do
			variable_index := a_value
		end

	set_expression (a_string: STRING) is
			-- Set `expression' to `a_string'
		do
			expression := a_string
		end

	set_source (a_value: like source) is
			--set source
		do
			source := a_value
		end

	set_parent (a_value: like parent) is
			-- Set `parent' to `a_parent'.
		do
			parent := a_value
		end

	set_type (a_value: like type) is
			-- Set `type' to `a_value'.
		do
			type := a_value
		end

	set_units (a_value: like units) is
			-- Set `units' to `a_value'.
		do
			units := a_value
		end

	set_source_table (a_value: like source_table) is
			-- Set `source_table' to `a_value'
		do
			source_table := a_value
		end

	set_source_column (a_value: like source_column) is
			-- Set `source_column' to `a_value'
		do
			source_column := a_value
		end

	set_expression_type (a_value: like expression_type) is
			-- Set the expression type.
		do
			expression_type := a_value
		end

	set_attribute_value_from_string (a_name, a_value: STRING) is
			-- Set `a_name' to `a_value'
		local
			i: INTEGER
			l_type: INTEGER
		do
			from
				i := 1
			until
				i > internal.field_count (Current)
			loop
				if internal.field_name (i, Current).is_case_insensitive_equal (a_name) then
					l_type := internal.field_type (i, Current)
					if l_type = internal.real_64_type and then (a_value = Void or else a_value.is_double or else a_value.is_empty) then
						if a_value = Void or else a_value.is_empty then
							internal.set_real_64_field (i, Current, constants.missing_double)
						else
							internal.set_real_64_field (i, Current, a_value.to_double)
						end
					elseif l_type = internal.integer_32_type and then (a_value = Void or else a_value.is_integer or else a_value.is_empty) then
						if a_value = Void or else a_value.is_empty then
							internal.set_integer_32_field (i, Current, constants.missing_integer)
						else
							internal.set_integer_32_field (i, Current, a_value.to_integer)
						end
					elseif l_type = internal.character_type and then (a_value = Void or else a_value.count = 1 or else a_value.is_empty) then
						if a_value = Void or else a_value.is_empty then
							internal.set_character_8_field (i, Current, '%U')
						else
							internal.set_character_8_field (i, Current, a_value.item (1))
						end
					else
						internal.set_reference_field (i, Current, a_value)
					end
				end
				i := i + 1
			end
		end

feature -- Basic operations

	store is
			-- Persist this description
		local
			l_servers: DATABASE_SERVERS
			l_index: INTEGER
		do
			create l_servers
			l_index := variable_index_from_table
			if l_index > 0
			then
				update_db(l_servers.rose_entity_background_server)
				update_db(l_servers.rose_entity_foreground_server)
			else
				insert_into_db(l_servers.rose_entity_background_server)
				insert_into_db(l_servers.rose_entity_foreground_server)
			end
		end

	add_to_background
	-- Add this variable to the variable_description_table in rose_entity_background.
	local
		l_servers: DATABASE_SERVERS
	do
		create l_servers
		insert_into_db(l_servers.rose_entity_background_server)
	end

	update_background
	-- Update this variable in the variable_description_table in rose_entity_background.
	local
		l_servers: DATABASE_SERVERS
	do
		create l_servers
		update_db(l_servers.rose_entity_background_server)
	end

	clean_and_add_to_background
	-- Removes any rows with the same index or mnemonic (if any) from variable_description_table.
	-- Then adds Current.
	local
		l_servers: DATABASE_SERVERS
		l_server: SQL_DATA_SERVER
	do
		create l_servers
		l_server := l_servers.rose_entity_background_server

		l_server.begin_transaction
		if not l_server.success
		then
			(create {EXCEPTIONS}).raise("Begin transaction failed")
		end

		remove_old_rows(l_server)	-- Removes any rows that would interfere with the following insert.
		insert_into_db(l_server)

		l_server.commit
		if not l_server.success
		then
			(create {EXCEPTIONS}).raise("Commit failed")
		end
	end

feature -- Utility.

	known_scale(a_scale: STRING): BOOLEAN
	-- Test the argument for typo's by comparing to the known values.
	-- Please feel free to add new values, if appropriate.
	do
		Result :=
			a_scale = Void or
			a_scale ~ "" or
			a_scale ~ "1" or
			a_scale ~ k_1000000
	end

	known_target(a_target: STRING): BOOLEAN
	-- Test the argument for typo's by comparing to the known values.
	-- Please feel free to add new values, if appropriate.
	do
		Result :=
			a_target ~ k_company or
			a_target ~ k_security or
			a_target ~ "listing"
	end

	known_unit(a_unit: STRING): BOOLEAN
	-- Test the argument for typo's by comparing to the known values.
	-- Please feel free to add new values, if appropriate.
	do
		Result :=
			a_unit = Void or
			a_unit ~ "" or
			a_unit ~ k_currency or
			a_unit ~ k_share or
			a_unit ~ k_currency_per_share or
			a_unit ~ "currency/(quarter*share)" or
			a_unit ~ "currency/(year*share)" or
			a_unit ~ "1/year"
	end

	known_type(a_type: STRING): BOOLEAN
	-- Test the argument for typo's by comparing to the known values.
	-- Please feel free to add new values, if appropriate.
	do
		Result :=
			a_type ~ k_double or
			a_type ~ k_integer or
			a_type ~ k_string or
			a_type ~ k_real or
			a_type ~ "vector" or
			a_type ~ "array"
	end

	known_target_name(a_target_name: ANY): BOOLEAN
	-- Test the argument for typo's by comparing to the known values.
	-- Please feel free to add new values, if appropriate.
	do
		Result :=
			a_target_name ~ k_model_company_signature or
			a_target_name ~ "model_security_signature" or
			a_target_name ~ "model_listing_signature" or
			a_target_name ~ k_monthly_data or
			a_target_name ~ k_mondex or
			a_target_name ~ k_valdata_signature or
			a_target_name ~ k_rose_company_date or
			a_target_name ~ "rose_entity_date" or
			a_target_name ~ "ibes_summary"
	end

	known_weighting(a_known_weighting: ANY): BOOLEAN
	-- Test the argument for typo's by comparing to the known values.
	-- Please feel free to add new values, if appropriate.
	do
		Result :=
			a_known_weighting = Void or
			a_known_weighting ~ "" or
			a_known_weighting ~ k_per_share or
			a_known_weighting ~ "cap" or
			a_known_weighting ~ "undefined" or
			a_known_weighting ~ "equal" or
			a_known_weighting ~ "basval"
	end

feature {NONE} -- Implementation

	add_string_diff (a_result: STRING; a_label: STRING; a, a_other: STRING) is
			-- Add display of string difference to `a_result'
		require
			a_result_not_void: a_result /= Void
			a_label_not_void: a_label /= Void
		do
			if a /= a_other and then not (a /= Void and a_other /= Void and then a.same_string (a_other)) then
				a_result.append (a_label + displayable_string (a) + "%T->%T" + displayable_string (a_other) + "%N")
			end
		end

	displayable_string (a_value: STRING): STRING is
			-- Display value for 'a_value'
		do
			if a_value = Void then
				Result := "<null>"
			else
				Result := a_value
			end
		ensure
			valid_result: Result /= Void
		end

	update_db (a_server: SQL_DATA_SERVER)
			-- Update this description into 'a_server'.
		require
			valid_server: a_server /= Void
		local
			l_store_predicate: EXECUTION_PREDICATE
		do
			create l_store_predicate.make (a_server,
										   "[
											UPDATE
											variable_description_table
											SET
											name = :name, mnemonic8 = :mnemonic8,
											source = :source, weighting = :weighting, period = :period, result_string = :result_string,
											parent = :parent, j_index = :j_index, gv_index = :gv_index,
											spcs_index = :spcs_index, risk_index = :risk_index, group_name = :group_name,
											security = :security, units = :units, type = :type, lower_winsor = :lower_winsor,
											upper_winsor = :upper_winsor, default_mean = :default_mean, default_standard_deviation = :default_standard_deviation,
											expression = :expression, scale = :scale, target = :target, stock_or_flow = :stock_or_flow,
											target_name = :target_name
											WHERE
											mnemonic = :mnemonic
											]")
		    if not l_store_predicate.success then
				(create {EXCEPTIONS}).raise ("Could not create predicate")
			end
			l_store_predicate.execute (prepared_for_db, <<mnemonic>>)
			if not l_store_predicate.success then
				(create {EXCEPTIONS}).raise ("Could not execute predicate")
			end
		end

	insert_into_db (a_server: SQL_DATA_SERVER)
			-- Insert this description into 'a_server'.
		local
			l_store_predicate: EXECUTION_PREDICATE
			l_servers: DATABASE_SERVERS
		do
			create l_servers
			if a_server = l_servers.rose_entity_background_server then
				create l_store_predicate.make (a_server,
											   "[
												INSERT INTO
												variable_description_table
												(variable_index, mnemonic, name, mnemonic8, source, weighting, period, result_string, parent, j_index, gv_index,
												spcs_index, risk_index, group_name, security, units, type, lower_winsor, upper_winsor, default_mean, default_standard_deviation, expression,
												scale, target, stock_or_flow, target_name)
												VALUES
												(:variable_index, :mnemonic, :name, :mnemonic8, :source, :weighting, :period, :result_string, :parent, :j_index, :gv_index,
												:spcs_index, :risk_index, :group_name, :security, :units, :type, :lower_winsor, :upper_winsor, :default_mean, :default_standard_deviation, :expression,
												:scale, :target, :stock_or_flow, :target_name)
												]")
				if not l_store_predicate.success then
					(create {EXCEPTIONS}).raise ("Could not create predicate")
				end
				l_store_predicate.execute (prepared_for_db, Void)
				if not l_store_predicate.success then
					(create {EXCEPTIONS}).raise ("Could not execute predicate")
				end
			else
				-- Save as above except that 'variable_index' and 'user_name' are added to the end.
				create l_store_predicate.make (a_server,
											   "[
												INSERT INTO
												variable_description_table
												(variable_index, mnemonic, name, mnemonic8, source, weighting, period, result_string, parent, j_index, gv_index,
												spcs_index, risk_index, group_name, security, units, type, lower_winsor, upper_winsor, default_mean, default_standard_deviation, expression,
												scale, target, stock_or_flow, target_name, variable_index, user_name)
												VALUES
												(:variable_index, :mnemonic, :name, :mnemonic8, :source, :weighting, :period, :result_string, :parent, :j_index, :gv_index,
												:spcs_index, :risk_index, :group_name, :security, :units, :type, :lower_winsor, :upper_winsor, :default_mean, :default_standard_deviation, :expression,
												:scale, :target, :stock_or_flow, :target_name, ?, ?)
												]")
				if not l_store_predicate.success then
					(create {EXCEPTIONS}).raise ("Could not create predicate")
				end
				l_store_predicate.execute (prepared_for_db, <<variable_index_from_table, "">>)
				if not l_store_predicate.success then
					(create {EXCEPTIONS}).raise ("Could not execute predicate")
				end
			end
		end

	remove_old_rows(a_server: SQL_DATA_SERVER)
	-- Removes rows with the same index or mnemonic (if any).
	local
		l_predicate: EXECUTION_PREDICATE
	do
		create l_predicate.make(a_server, "delete variable_description_table where variable_index = :variable_index or mnemonic = :mnemonic")
		if not l_predicate.success
		then
			(create {EXCEPTIONS}).raise("Could not create the predicate")
		end

		l_predicate.execute(Current, Void)
		if not l_predicate.success
		then
			(create {EXCEPTIONS}).raise("The predicate failed to execute")
		end
	end

	prepared_for_db: like Current
	-- Returns a copy of Current except that certain columns which require "not null" have Voids replace with "".
	do
		create Result
		Result.copy(Current)

		if expression = Void
		then
			Result.set_expression("")
		end
	end

	variable_index_from_table: INTEGER is
			-- Query the table for 'variable_index'
		local
			l_servers: DATABASE_SERVERS
			l_index: INTEGER_PREDICATE
		do
			create l_servers
			create l_index.make (l_servers.rose_entity_background_server, "select variable_index from variable_description_table where mnemonic = ?")
			if not l_index.success then
				(create {EXCEPTIONS}).raise ("Could not create 'l_index' predicate")
			end
			l_index.retrieve (Void, <<mnemonic>>)
			if not l_index.success then
				(create {EXCEPTIONS}).raise ("Could not retrieve 'l_index' predicate")
			end
			if not l_index.array.is_empty then
				Result := l_index.array.first
			end
		end

	internal: INTERNAL is
			-- Access to INTERNAL
		once
			create Result
		ensure
			valid_result: Result /= Void
		end

	format_double: FORMAT_DOUBLE is
			-- Double data formatter
		once
			create Result.make (20, 10)
		ensure
			valid_result: Result /= Void
		end

end
