note

	description:
	"[
		Data necessary for an expression factory to replicate ROSE_EXPRESSIONs for
		the variables occurring in the legacy system
	]"
	author: "Mark Howard"
	last_editor: "$Author: rj $"
	date: "$Date: 2014/01/22 $"
	revision: "$Revision: #51 $"
	path: "$File: //rose/source/kernel/math/variable_description_record.e $"
	storable_version: ""

class VARIABLE_DESCRIPTION_RECORD

inherit

	RP_SERIALIZABLE

	ROSE_CONSTANTS
		undefine
			is_equal
		end

	VARIABLE_DESCRIPTION
		undefine
			target_name
		redefine
			make,
			set_j_index,
			set_gv_index,
			set_spcs_index,
			set_risk_index,
			formatted_out,
			key_fields
		end

	MISMATCH_CORRECTOR
		undefine
			is_equal
		redefine
			correct_mismatch
		end

creation

	default_create,
	make,
	make_ibes_summary,
	make_computed,
	make_mondat,
	make_valdata,
	make_group,
	make_ibes,
	make_intermediation,
	make_market,
	make_model,
	make_model_company,
	make_model_security,
	make_model_listing,
	make_monthly_marketdata,
	make_mondex,
	make_monthly_rmdata,
	make_rbc,
	make_risk,
	make_segment,
	make_unknown,
	make_valreg,
	make_model_risk,
	make_

feature {NONE} -- Initialization

	make (a_name: STRING; a_mnemonic: STRING; a_source: STRING; a_weighting: STRING; a_period: STRING; a_result_string: STRING; a_parent: STRING;
			a_j_index: INTEGER; a_gv_index: INTEGER; a_spcs_index: INTEGER; a_risk_index: INTEGER; a_security: INTEGER; a_units: STRING;
			 a_type: STRING; a_lower_winsor: DOUBLE; a_upper_winsor: DOUBLE; a_default_mean: DOUBLE; a_default_standard_deviation: DOUBLE;
			  a_expression: STRING; a_rose_name: ANY; a_user_name: ANY; a_scale: STRING; a_target: STRING; a_stock_or_flow: CHARACTER; a_target_name: STRING;
			   a_group_name: STRING; a_variable_index: INTEGER) is
			-- Creation procedure, don't use
		do
			upper_winsor := a_upper_winsor
			lower_winsor := a_lower_winsor
			default_mean := a_default_mean
			default_standard_deviation := a_default_standard_deviation
			mnemonic8 := ""
			group_name := a_group_name
			target_name := a_target_name
			weighting := a_weighting
			period := a_period
			result_string := a_result_string
			if a_variable_index > 0 and then variable_indices.has(a_variable_index) and then variable_indices[a_variable_index] /~ a_mnemonic then
				debug("trace")
					print("duplicate index " + a_variable_index.out + " " + variable_indices[a_variable_index] + "," + a_mnemonic + "(" + a_name + ")%N")
				end
			else
				variable_indices[a_variable_index] := a_mnemonic
			end
			Precursor (a_name, a_mnemonic, a_source, a_weighting, a_period, a_result_string, a_parent, a_j_index, a_gv_index, a_spcs_index, a_risk_index, a_security, a_units, a_type, a_lower_winsor, a_upper_winsor, a_default_mean, a_default_standard_deviation, a_expression, a_rose_name, a_user_name, a_scale, a_target, a_stock_or_flow, a_target_name, a_group_name, a_variable_index)
		end

	make_mondat (a_name: STRING; a_mnemonic: STRING; a_result_string: STRING; a_parent: STRING; a_j_index: INTEGER; a_gv_index: INTEGER; a_spcs_index: INTEGER; a_risk_index: INTEGER; a_security: INTEGER; a_units: STRING; a_type: STRING; a_rose_name: ANY; a_scale: STRING; a_target: STRING; a_stock_or_flow: CHARACTER; a_target_name: STRING; a_weighting: STRING; a_group_name: STRING; a_variable_index: INTEGER) is
			-- Creation procedure, don't use
		do
			make (a_name, a_mnemonic, "mondat", a_weighting, "monthly", a_result_string, a_parent, a_j_index, a_gv_index, a_spcs_index, a_risk_index, a_security, a_units, a_type, QNAN, QNAN, QNAN, QNAN, Void, a_rose_name, Void, a_scale, a_target, a_stock_or_flow, a_target_name, a_group_name, a_variable_index)
			variable_index := a_variable_index
		end

	make_group (a_name: STRING; a_mnemonic: STRING; a_parent: STRING; a_security: INTEGER; a_units: STRING; a_rose_name: ANY; a_scale: STRING; a_target_name: STRING; a_group_name: STRING; a_variable_index:INTEGER) is
			-- Creation procedure, don't use
		do
			make (a_name, a_mnemonic, "group", Void, Void, Void, a_parent, undefined_integer, undefined_integer, undefined_integer, undefined_integer, a_security, a_units, "double", QNAN, QNAN, QNAN, QNAN, Void, a_rose_name, Void, a_scale, "company", ' ', a_target_name, a_group_name, a_variable_index)
		end

	make_ibes (a_name: STRING; a_mnemonic: STRING; a_units: STRING; a_type: STRING; a_target_name: STRING; a_group_name: STRING; a_variable_index: INTEGER) is
			-- Creation procedure, don't use
		do
			make (a_name, a_mnemonic, "ibes", Void, Void, Void, Void, undefined_integer, undefined_integer, undefined_integer, undefined_integer, 1, a_units, a_type, QNAN, QNAN, QNAN, QNAN, Void, Void, "dbo", Void, "company", ' ', a_target_name, a_group_name, a_variable_index)
		end

	make_intermediation (a_name: STRING; a_mnemonic: STRING; a_units: STRING; a_type: STRING; a_target_name: STRING; a_group_name: STRING; a_variable_index: INTEGER) is
			-- Creation procedure, don't use
		do
			make (a_name, a_mnemonic, "intermediation", Void, Void, Void, Void, undefined_integer, undefined_integer, undefined_integer, undefined_integer, 2, a_units, a_type, QNAN, QNAN, QNAN, QNAN, Void, Void, "system_user", Void, "company", ' ', a_target_name, a_group_name, a_variable_index)
		end

	make_market (a_name: STRING; a_mnemonic: STRING; a_parent: STRING; a_units: STRING; a_type: STRING; a_target_name: STRING; a_group_name: STRING; a_variable_index: INTEGER) is
			-- Creation procedure, don't use
		do
			make (a_name, a_mnemonic, "market", Void, Void, Void, a_parent, undefined_integer, undefined_integer, undefined_integer, undefined_integer, 1, a_units, a_type, QNAN, QNAN, QNAN, QNAN, Void, Void, Void, "1", "listing", ' ', a_target_name, a_group_name, a_variable_index)
		end

	make_model (a_name: STRING; a_mnemonic: STRING; a_security: INTEGER; a_type: STRING; a_model_index: STRING; a_variable_index: INTEGER)
			-- Please use 'make_model_company', 'make_model_security' or 'make_model_listing'.
		local
			l_model_index: INTEGER
		do
			make (a_name, a_mnemonic, "model", Void, Void, Void, Void, undefined_integer, undefined_integer, undefined_integer, undefined_integer, a_security, Void, a_type, QNAN, QNAN, QNAN, QNAN, Void, Void, Void, Void, "company", ' ', "model_company_signature", "model", a_variable_index)
			if a_model_index /= Void and then a_model_index.is_integer
			then
				l_model_index := a_model_index.to_integer
				if l_model_index < 10 then
					model_index := "MC_0" + l_model_index.out
				else
					model_index := "MC_" + l_model_index.out
				end
			else
				model_index := a_model_index
			end
		end

	make_model_company (a_name: STRING; a_mnemonic: STRING; a_security: INTEGER; a_units: STRING; a_type: STRING; a_stock_or_flow: CHARACTER; a_model_index: STRING; a_variable_index: INTEGER)
			-- Creation procedure, don't use
		local
			l_model_index: INTEGER
		do
			make (a_name, a_mnemonic, "model", Void, Void, Void, Void, undefined_integer, undefined_integer, undefined_integer, undefined_integer, a_security, a_units, a_type, QNAN, QNAN, QNAN, QNAN, Void, Void, Void, Void, "company", a_stock_or_flow, "model_company_signature", "model", a_variable_index)
			if a_model_index /= Void and then a_model_index.is_integer
			then
				l_model_index := a_model_index.to_integer
				if l_model_index < 10 then
					model_index := "MC_0" + l_model_index.out
				else
					model_index := "MC_" + l_model_index.out
				end
			else
				model_index := a_model_index
			end
		end

	make_model_security (a_name: STRING; a_mnemonic: STRING; a_security: INTEGER; a_units: STRING; a_type: STRING; a_stock_or_flow: CHARACTER; a_model_index: STRING; a_variable_index: INTEGER)
			-- Creation procedure, don't use
		local
			l_model_index: INTEGER
		do
			make (a_name, a_mnemonic, "model", Void, Void, Void, Void, undefined_integer, undefined_integer, undefined_integer, undefined_integer, a_security, a_units, a_type, QNAN, QNAN, QNAN, QNAN, Void, Void, Void, Void, "security", a_stock_or_flow, "model_security_signature", "model", a_variable_index)
			if a_model_index /= Void and then a_model_index.is_integer
			then
				l_model_index := a_model_index.to_integer
				if l_model_index < 10 then
					model_index := "MS_0" + l_model_index.out
				else
					model_index := "MS_" + l_model_index.out
				end
			else
				model_index := a_model_index
			end
		end

	make_model_listing (a_name: STRING; a_mnemonic: STRING; a_security: INTEGER; a_units: STRING; a_type: STRING; a_stock_or_flow: CHARACTER; a_model_index: STRING; a_variable_index: INTEGER)
			-- Creation procedure, don't use
		local
			l_model_index: INTEGER
		do
			make (a_name, a_mnemonic, "model", Void, Void, Void, Void, undefined_integer, undefined_integer, undefined_integer, undefined_integer, a_security, a_units, a_type, QNAN, QNAN, QNAN, QNAN, Void, Void, Void, Void, "listing", a_stock_or_flow, "model_listing_signature", "model", a_variable_index)
			if a_model_index /= Void and then a_model_index.is_integer
			then
				l_model_index := a_model_index.to_integer
				if l_model_index < 10 then
					model_index := "MR_0" + l_model_index.out
				else
					model_index := "MR_" + l_model_index.out
				end
			else
				model_index := a_model_index
			end
		end

	make_model_risk (a_name: STRING; a_mnemonic: STRING; a_security: INTEGER; a_type: STRING; a_model_index: STRING;a_risk_index : INTEGER)
			-- Creation procedure, don't use
		do
			make_model (a_name,a_mnemonic,a_security,a_type,a_model_index, 0)
			variable_index := a_risk_index
		end

	make_mondex (a_name: STRING; a_mnemonic: STRING; a_source: STRING; a_result_string: STRING; a_parent: STRING; a_j_index: INTEGER; a_gv_index: INTEGER; a_spcs_index: INTEGER; a_risk_index: INTEGER; a_security: INTEGER; a_units: STRING; a_type: STRING; a_rose_name: ANY; a_scale: STRING; a_target: STRING; a_stock_or_flow: CHARACTER; a_target_name: STRING; a_group_name: STRING; a_variable_index: INTEGER) is
			-- Creation procedure, don't use
		do
			make (a_name, a_mnemonic, a_source, Void, Void, a_result_string, a_parent, a_j_index, a_gv_index, a_spcs_index, a_risk_index, a_security, a_units, a_type, QNAN, QNAN, QNAN, QNAN, Void, a_rose_name, Void, a_scale, a_target, a_stock_or_flow, a_target_name, a_group_name, a_variable_index)
		end

	make_monthly_marketdata (a_name: STRING; a_mnemonic: STRING; a_security: INTEGER; a_type: STRING; a_target_name: STRING; a_group_name: STRING; a_variable_index: INTEGER) is
			-- Creation procedure, don't use
		do
			make (a_name, a_mnemonic, "monthly_marketdata", Void, Void, Void, Void, undefined_integer, undefined_integer, undefined_integer, undefined_integer, a_security, Void, a_type, QNAN, QNAN, QNAN, QNAN, Void, Void, Void, Void, Void, ' ', a_target_name, a_group_name, a_variable_index)
		end

	make_monthly_rmdata (a_name: STRING; a_mnemonic: STRING; a_type: STRING; a_target_name: STRING; a_group_name: STRING; a_variable_index: INTEGER) is
			-- Creation procedure, don't use
		do
			make (a_name, a_mnemonic, "monthly_rmdata", Void, Void, Void, Void, undefined_integer, undefined_integer, undefined_integer, undefined_integer, 2, Void, a_type, QNAN, QNAN, QNAN, QNAN, Void, Void, Void, Void, "security", ' ', a_target_name, a_group_name, a_variable_index)
		end

	make_rbc (a_name: STRING; a_mnemonic: STRING; a_type: STRING; a_target_name: STRING; a_group_name: STRING; a_variable_index: INTEGER) is
			-- Creation procedure, don't use
		do
			make (a_name, a_mnemonic, "rbc", Void, Void, Void, Void, undefined_integer, undefined_integer, undefined_integer, undefined_integer, 2, Void, a_type, QNAN, QNAN, QNAN, QNAN, Void, Void, Void, Void, "company", ' ', a_target_name, a_group_name, a_variable_index)
		end

	make_ibes_summary (a_name: STRING; a_mnemonic: STRING; a_source: STRING; a_result_string: STRING; a_parent: STRING; a_j_index: INTEGER; a_gv_index: INTEGER; a_spcs_index: INTEGER; a_risk_index: INTEGER; a_security: INTEGER; a_units: STRING; a_type: STRING; a_rose_name: ANY; a_scale: STRING; a_target: STRING; a_stock_or_flow: CHARACTER; a_target_name: STRING; a_group_name: STRING; a_variable_index: INTEGER) is
			-- Creation procedure, don't use
		do
			make (a_name, a_mnemonic, a_source, Void, Void, a_result_string, a_parent, a_j_index, a_gv_index, a_spcs_index, a_risk_index, a_security, a_units, a_type, qnan, qnan, qnan, qnan, Void, a_rose_name, Void, a_scale, a_target, a_stock_or_flow, a_target_name, a_group_name, a_variable_index)
		end

	make_risk (a_name: STRING;
			   a_mnemonic: STRING;
			   a_mnemonic8: STRING;
			   a_parent: STRING;
			   a_j_index: INTEGER;
			   a_risk_index: INTEGER;
			   a_security: INTEGER;
			   a_units: STRING;
			   a_lower_winsor: DOUBLE;
			   a_upper_winsor: DOUBLE;
			   a_default_mean: DOUBLE;
			   a_default_standard_deviation: DOUBLE;
			   a_expression: STRING;
			   a_rose_name: ANY;
			   a_scale: STRING;
			   a_stock_or_flow: CHARACTER;
			   a_target_name: STRING;
			   a_weighting: STRING;
			   a_group_name: STRING;
			   a_variable_index: INTEGER) is
			-- Creation procedure, don't use
		do
			make (a_name, a_mnemonic, "risk", a_weighting, Void, Void, a_parent, a_j_index, undefined_integer, undefined_integer, a_risk_index, a_security, a_units, "double", a_lower_winsor, a_upper_winsor, a_default_mean, a_default_standard_deviation, a_expression, a_rose_name, Void, a_scale, "company", a_stock_or_flow, a_target_name, a_group_name, a_variable_index)
			mnemonic8 := a_mnemonic8
		end

	make_segment (a_name: STRING; a_mnemonic: STRING; a_units: STRING; a_type: STRING; a_target_name: STRING; a_group_name: STRING; a_variable_index: INTEGER) is
			-- Creation procedure, don't use
		do
			make (a_name, a_mnemonic, "segment", Void, Void, Void, Void, undefined_integer, undefined_integer, undefined_integer, undefined_integer, 1, a_units, a_type, QNAN, QNAN, QNAN, QNAN, Void, Void, Void, "1", "segment", ' ', a_target_name, a_group_name, a_variable_index)
		end

	make_unknown (a_name: STRING; a_mnemonic: STRING; a_result_string: STRING; a_parent: STRING; a_j_index: INTEGER; a_gv_index: INTEGER; a_spcs_index: INTEGER; a_risk_index: INTEGER; a_security: INTEGER; a_units: STRING; a_type: STRING; a_rose_name: ANY; a_scale: STRING; a_target: STRING; a_stock_or_flow: CHARACTER; a_target_name: STRING; a_group_name: STRING; a_variable_index: INTEGER) is
			-- Creation procedure, don't use
		do
			make (a_name, a_mnemonic, "unknown", Void, Void, a_result_string, a_parent, a_j_index, a_gv_index, a_spcs_index, a_risk_index, a_security, a_units, a_type, QNAN, QNAN, QNAN, QNAN, Void, a_rose_name, Void, a_scale, a_target, a_stock_or_flow, a_target_name, a_group_name, a_variable_index)
		end

	make_valreg (a_name: STRING; a_mnemonic: STRING; a_security: INTEGER; a_type: STRING; a_target: STRING; a_target_name: STRING; a_group_name: STRING; a_variable_index: INTEGER) is
			-- Creation procedure, don't use
		do
			make (a_name, a_mnemonic, "valreg", Void, Void, Void, Void, undefined_integer, undefined_integer, undefined_integer, undefined_integer, a_security, Void, a_type, QNAN, QNAN, QNAN, QNAN, Void, Void, Void, Void, a_target, ' ', a_target_name, a_group_name, a_variable_index)
		end

	make_ (a_name: STRING; a_mnemonic: STRING; a_source: STRING; a_result_string: STRING; a_parent: STRING; a_j_index: INTEGER; a_gv_index: INTEGER; a_spcs_index: INTEGER; a_risk_index: INTEGER; a_security: INTEGER; a_units: STRING; a_type: STRING; a_rose_name: ANY; a_scale: STRING; a_target: STRING; a_stock_or_flow: CHARACTER; a_target_name: STRING; a_group_name: STRING; a_variable_index: INTEGER) is
			-- Creation procedure, don't use
		do
			make (a_name, a_mnemonic, a_source, Void, Void, a_result_string, a_parent, a_j_index, a_gv_index, a_spcs_index, a_risk_index, a_security, a_units, a_type, QNAN, QNAN, QNAN, QNAN, Void, a_rose_name, Void, a_scale, a_target, a_stock_or_flow, a_target_name, a_group_name, a_variable_index)
		end

	make_valdata (a_name: STRING; a_mnemonic: STRING; a_parent: STRING; a_j_index: INTEGER; a_gv_index: INTEGER; a_spcs_index: INTEGER; a_security: INTEGER; a_units: STRING; a_type: STRING; a_rose_name: ANY; a_target: STRING; a_stock_or_flow: CHARACTER; a_target_name: STRING; a_scale: STRING; a_group_name: STRING; a_variable_index: INTEGER) is
			-- Creation procedure, don't use
		do
			make (a_name, a_mnemonic, "valdata", Void, Void, Void, a_parent, a_j_index, a_gv_index, a_spcs_index, undefined_integer, a_security, a_units, a_type, qnan, qnan, qnan, qnan, Void, a_rose_name, Void, a_scale, a_target, a_stock_or_flow, a_target_name, a_group_name, a_variable_index)
		end

	make_computed (a_name: STRING; a_mnemonic: STRING; a_result_string: STRING; a_parent: STRING; a_security: INTEGER;
				   a_type: STRING; a_expression: STRING; a_scale: like scale; a_target: STRING; a_target_name: STRING; a_weighting: STRING; a_group_name: STRING; a_variable_index: INTEGER) is
			-- Creation procedure, don't use.  There are no 'units' because 'units' are computed.
		do
			make (a_name, a_mnemonic, "computed", a_weighting, Void, a_result_string, a_parent, undefined_integer,
				  undefined_integer, undefined_integer, undefined_integer, a_security, Void, a_type, qnan, qnan,
				  qnan, qnan, a_expression, Void, "dbo", a_scale, a_target, ' ', a_target_name, a_group_name, a_variable_index)
		end

	variable_indices : HASH_TABLE[STRING,INTEGER]
			--All indices to ensure no conflicts
		once
			create Result.make(10000)
		end

feature -- Access

	mnemonic8: STRING
			-- Mnemonic

	weighting: STRING

	period: STRING
			-- At this time, the only valid period is "monthly".

	result_string: STRING

	target_name: STRING
			-- e.g. mondex, rose_company_date, valdata_signature.

	j_index: INTEGER
			-- J index

	gv_index: INTEGER
			-- GV index

	spcs_index: INTEGER
			-- SPCS index

	risk_index: INTEGER
			-- Mondat record risk index

	group_name: STRING
			-- Group name

	lower_winsor: DOUBLE
			-- Lower winsor truncation
	upper_winsor: DOUBLE
			-- Upper winsor truncation
	default_mean: DOUBLE
		-- Default mean

	default_standard_deviation: DOUBLE
		-- Default standard deviation
	model_index: STRING

	model_mnemonic: STRING

	display_name: STRING

	key_fields: STRING is
			-- Comma delimited string of important attribute values
		do
			Result := Precursor + ",mnemonic8,weighting,period,result_string,target_name,j_index,gv_index,spcs_index,risk_index,group_name,lower_winsor,upper_winsor,default_mean,default_standard_deviation,display_name"
		end

	formatted_out: STRING is
			-- <Precursor>
		do
			Result := Precursor
			if lower_winsor /= 0 and is_defined_double (lower_winsor) and upper_winsor /= 0 and is_defined_double (upper_winsor) then
				Result := Result + "%N%TLower Winsor:%T" + lower_winsor.out + "%N%TUpper Winsor:%T" + upper_winsor.out
			end
			if risk_index > 0 then
				Result := Result + "%N%TRisk Index:%T" + risk_index.out
			end
		end

feature -- Element modification

	set_mnemonic8 (a_value: like mnemonic8) is
			-- Set the Mnemonic8.
		do
			mnemonic8 := a_value
		end

	set_weighting (a_value: like weighting) is
			--set weighting
		do
			weighting := a_value
		end

	set_period (a_value: like period) is
			--set period
		do
			period := a_value
		end

	set_result_string (a_value: like result_string)
			--result_string setter
		do
			result_string := a_value
		end

	set_target_name (a_value: like target_name)
			--target_name setter
		do
			target_name := a_value
		end

	set_j_index (a_value: like j_index) is
			-- Set J index a 'a_value'.
		do
			j_index := a_value
		end

	set_gv_index (a_value: like gv_index) is
			-- Set gv_index to 'a_value'.
		do
			gv_index := a_value
		end

	set_spcs_index (a_value: like spcs_index) is
			-- Set 'spcs_index' to 'a_value'.
		do
			spcs_index := a_value
		end

	set_risk_index (a_value: like risk_index) is
			-- Set 'risk_index' to 'a_value'.
		do
			risk_index := a_value
		end

	set_group_name (a_value: like group_name) is
		do
			group_name := a_value
		end

	set_security (a_value: like security) is
			-- Set 'security' to 'a_security'.
		do
			security := a_value
		end

	set_lower_winsor (a_value: like lower_winsor) is
			-- Set 'lower_winsor' to 'a_value'.
		do
			lower_winsor := a_value
		end

	set_upper_winsor (a_value: like upper_winsor) is
			-- Set 'upper_winsor' to 'a_value'.
		do
			upper_winsor := a_value
		end

	set_default_mean (a_value: like default_mean) is
			-- Default average
		do
			default_mean := a_value
		end

	set_default_standard_deviation (a_value: like default_standard_deviation) is
			-- Sets 'default_standard_deviation'.
		do
			default_standard_deviation := a_value
		end

	set_scale (a_value: like scale) is
			-- Set 'scale' to 'a_value'.
		do
			scale := a_value
		end

	set_model_index (a_model_index: like model_index) is
		do
			model_index := a_model_index
		end

	set_model_mnemonic (a_model_mnemonic: like model_mnemonic) is
			-- Set the model mnemonic
		do
			model_mnemonic := a_model_mnemonic
		end

	set_display_name (a_name: STRING) is
			-- Set the display name.
		do
			display_name := a_name
		end

	set_target (a_value: like target) is
			-- Set 'target' to 'a_value'
		do
			target := a_value
		end

	set_stock_or_flow (a_value: like stock_or_flow) is
			-- Set 'stock_or_flow' to 'a_value'.
		do
			stock_or_flow := a_value
		end

	safe (a_string : STRING) : STRING
			-- Turns void into empty string, identity on others
		do
			Result := a_string
			if Result = Void then
				Result := ""
			end
		end

	creation_text : STRING
			--Creation procedure text from this record
		do
			Result := "create Result.make(" +
			"%"" + safe(name) + "%"," +
			"%"" + safe(mnemonic) + "%"," +
			"%"" + safe(source) + "%"," +
			"%"" + safe(weighting) + "%"," +
			"%"" + safe(period) + "%"," +
			"%"" + safe(result_string) + "%"," +
			"%"" + safe(parent) + "%"," +
			j_index.out + "," +
			gv_index.out + "," +
			spcs_index.out + "," +
			risk_index.out + "," +
			security.out + "," +
			"%"" + safe(units) + "%"," +
			 "%"" + safe(type) + "%"," +
			 lower_winsor.out + "," +
			 upper_winsor.out + "," +
			 default_mean.out + "," +
			 default_standard_deviation.out + "," +
			 "%"" + safe(expression) + "%"," +
			 "%"%"," +
			 "%"%"," +
			 "%"" + safe(scale) + "%"," +
			 "%"" + safe(target) + "%"," +
			 "'" + stock_or_flow.out + "'," +
			 "%"" + safe(target_name) + "%"," +
			 "%"" + safe(group_name) + "%"," +
			 variable_index.out + ")"
		end

feature {PERSISTENT_VARIABLE} -- Implementation

	undefined_integer: INTEGER
			-- Undefined integer
		local
			l_ieee: ROSE_IEEE_UNDEFINED
		once
			create l_ieee
			Result := l_ieee.undefined_integer-1
		end

feature {NONE} -- Implementation

	correct_mismatch is
			-- Correct mismatch
		do
		end

end
