indexing
	description: "The FLASHWINFO structure contains the flash status for a window and the number of times the system should flash the window"
	status: "See notice at end of class."
	date: "$Date: 2008/12/01 $"
	revision: "$Revision: #2 $"

class
	WEL_FLASH_WINDOW_INFO

inherit
	WEL_STRUCTURE
		rename
			make as structure_make
		end

create
	make

feature {NONE} -- Initialization

	make is
			-- Create FLASHWINFO struct
		do
			structure_make
			cwel_set_cb_size (item, structure_size)
		end

feature -- Access

	structure_size: INTEGER is
			-- Size to allocate (in bytes)
		once
			Result := c_flash_window_size
		end

	handle: POINTER is
			-- `hwnd' associated with Current
		require
			exists: exists
		do
			Result := cwel_get_handle (item)
		end

	flags: INTEGER is
			-- `dwFlags' associated with Current
		require
			exists: exists
		do
			Result := cwel_get_dwFlags (item)
		end

	count: INTEGER is
			-- `uCount' associated with Current
		require
			exists: exists
		do
			Result := cwel_get_uCount (item)
		end

	timeout: INTEGER is
			-- `dwTimeout' associated with Current
		require
			exists: exists
		do
			Result := cwel_get_dwTimeout (item)
		end

	FLASHW_STOP: INTEGER is
			-- Stop flashing. The system restores the window to its original state.
		external
			"C [macro <windows.h>]"
		alias
			"FLASHW_STOP"
		end

	FLASHW_CAPTION: INTEGER is
			-- Flash the window caption.
		external
			"C [macro <windows.h>]"
		alias
			"FLASHW_CAPTION"
		end

	FLASHW_TRAY: INTEGER is
			-- Flash the taskbar button.
		external
			"C [macro <windows.h>]"
		alias
			"FLASHW_TRAY"
		end

	FLASHW_ALL: INTEGER is
			-- Flash both the window caption and taskbar button. This is equivalent to setting the FLASHW_CAPTION | FLASHW_TRAY flags.
		external
			"C [macro <windows.h>]"
		alias
			"FLASHW_ALL"
		end

	FLASHW_TIMER: INTEGER is
			-- Flash continuously, until the FLASHW_STOP flag is set.
		external
			"C [macro <windows.h>]"
		alias
			"FLASHW_TIMER"
		end

	FLASHW_TIMERNOFG: INTEGER is
			-- Flash continuously until the window comes to the foreground.
		external
			"C [macro <windows.h>]"
		alias
			"FLASHW_TIMERNOFG"
		end

feature -- Setting

	set_handle (p: POINTER) is
			-- Assign `p' to `hwnd' field of Current.
		require
			exists: exists
		do
			cwel_set_handle (item, p)
		end

	set_flags (i: INTEGER) is
			-- Set `dwFlags' associated with Current
		require
			exists: exists
		do
			cwel_set_dwFlags (item, i)
		end

	set_count (i: INTEGER) is
			-- Set `uCount' associated with Current
		require
			exists: exists
		do
			cwel_set_uCount (item, i)
		end

	set_timeout (i: INTEGER) is
			-- Set `dwTimeout' associated with Current
		require
			exists: exists
		do
			cwel_set_dwTimeout (item, i)
		end

	call_FlashWindowEx is
			-- Call 'FlashWindowEx' with Current structure
		local
			b: BOOLEAN
		do
			b := c_FlashWindowEx (item)
		end

feature {NONE} -- Externals

	c_FlashWindowEx (a_handle: POINTER): BOOLEAN is
		external
			"C signature (PFLASHWINFO): EIF_BOOLEAN use <windows.h>"
		alias
			"FlashWindowEx"
		end

	c_flash_window_size: INTEGER is
			-- Size of FLASHWINFO struct
		external
			"C [macro <windows.h>]"
		alias
			"sizeof(FLASHWINFO)"
		end

	cwel_set_cb_size (p: POINTER; v: INTEGER) is
			-- Set `cbSize' field of FLASHWINFO
		external
			"C [struct <windows.h>] (FLASHWINFO, UINT)"
		alias
			"cbSize"
		end

	cwel_set_handle (p: POINTER; v: POINTER) is
			-- Set `hwnd' field of FLASHWINFO
		external
			"C [struct <windows.h>] (FLASHWINFO, HWND)"
		alias
			"hwnd"
		end

	cwel_set_uCount (p: POINTER; v: INTEGER) is
			-- `uCount' field of FLASHWINFO
		external
			"C [struct <windows.h>] (FLASHWINFO, int)"
		alias
			"uCount"
		end

	cwel_set_dwTimeout (p: POINTER; v: INTEGER) is
			-- Set `dwTimeout' field of FLASHWINFO
		external
			"C [struct <windows.h>] (FLASHWINFO, int)"
		alias
			"dwTimeout"
		end

	cwel_set_dwFlags (p: POINTER; v: INTEGER) is
			-- Set `dwFlags' field of FLASHWINFO
		external
			"C [struct <windows.h>] (FLASHWINFO, int)"
		alias
			"dwFlags"
		end

	cwel_get_cb_size (p: POINTER): INTEGER is
			-- Get `cbSize' field of FLASHWINFO
		external
			"C [struct <windows.h>] (FLASHWINFO): EIF_INTEGER"
		alias
			"cbSize"
		end

	cwel_get_handle (p: POINTER): POINTER is
			-- `hwnd' field of FLASHWINFO
		external
			"C [struct <windows.h>] (FLASHWINFO): EIF_INTEGER"
		alias
			"hwnd"
		end

	cwel_get_uCount (p: POINTER): INTEGER is
			-- `uCount' field of FLASHWINFO
		external
			"C [struct <windows.h>] (FLASHWINFO): EIF_INTEGER"
		alias
			"uCount"
		end

	cwel_get_dwTimeout (p: POINTER): INTEGER is
			-- `dwTimeout' field of FLASHWINFO
		external
			"C [struct <windows.h>] (FLASHWINFO): EIF_INTEGER"
		alias
			"dwTimeout"
		end

	cwel_get_dwFlags (p: POINTER): INTEGER is
			-- Set `dwFlags' field of FLASHWINFO
		external
			"C [struct <windows.h>] (FLASHWINFO): EIF_INTEGER"
		alias
			"dwFlags"
		end

end
