indexing
	description: "Contains information about a portion of the %
		%format string that defines a callback field within a %
		%date and time picker (DTP) control. It carries the %
		%substring that defines the callback field and contains a %
		%buffer to receive the string that will be displayed in %
		%the callback field. This structure is used with the %
		%DTN_FORMAT notification message."
	author: "David Jenkins"
	date: "$Date: 2008/12/01 $"
	revision: "$Revision: #2 $"

class
	WEL_NM_DATE_TIME_FORMAT

inherit
	WEL_STRUCTURE

create
	make,
	make_by_nmhdr,
	make_by_pointer

feature {NONE} -- Initialization

	make_by_nmhdr (a_nmhdr: WEL_NMHDR) is
			-- Make the structure with `a_nmhdr'.
		require
			a_nmhdr_not_void: a_nmhdr /= Void
		do
			make_by_pointer (a_nmhdr.item)
		end

feature -- Access

	hdr: WEL_NMHDR is
			-- Information about the Wm_notify message
		do
			create Result.make_by_pointer(c_nm_dtf_get_hdr(item))
		ensure
			result_not_void: Result /= Void
		end

	format:STRING is
			-- Substring that defines a DTP control
			-- callback field.  The substring comprises one or
			-- more "X" characters
		do
			create Result.make_from_c(c_nm_dtf_format(item))
		end

	system_time:MC_SYSTEM_TIME is
			-- The date and time to be formatted
		do
			create Result.make_by_pointer(c_nm_dtf_st(item))
		end

	current_display:STRING is
			-- Current display text of a DTP control
		do
			create Result.make_from_c(c_nm_dtf_pszdisplay(item))
		end

feature -- Element change

	set_display(str:STRING) is
			-- Sets string to be displayed
		require
			string_exists: str /= Void and then
				str.count <= 64
		local
			c_str: ANY
		do
			c_str := str.to_c
			c_nm_dtf_szdisplay(item,$c_str)
		end

feature -- Measurement

	structure_size:INTEGER is
			-- Size to allocate (in bytes)
		once
			Result := c_size_of_nm_datetimeformat
		end

feature {NONE} -- Implementation

	c_size_of_nm_datetimeformat:INTEGER is
		external
			"C[macro %"nmdtfrmt.h%"]"
		alias
			"sizeof(NMDATETIMEFORMAT)"
		end

	c_nm_dtf_get_hdr(handle:POINTER):POINTER is
		external
			"C[macro %"nmdtfrmt.h%"](NMDATETIMEFORMAT*):EIF_POINTER)"
		end

	c_nm_dtf_szdisplay(handle,c_str:POINTER) is
		external
			"C[macro %"nmdtfrmt.h%"](NMDATETIMEFORMAT*,EIF_POINTER)"
		end

	c_nm_dtf_format(handle:POINTER):POINTER is
		external
			"C[struct %"wel.h%"](NMDATETIMEFORMAT):LPCTSTR"
		alias
			"pszFormat"
		end

	c_nm_dtf_st(handle:POINTER):POINTER is
		external
			"C[macro %"nmdtfrmt.h%"](NMDATETIMEFORMAT*):LPSYSTEMTIME"
		end

	c_nm_dtf_pszdisplay(handle:POINTER):POINTER is
		external
			"C[struct %"wel.h%"](NMDATETIMEFORMAT):LPCTSTR"
		alias
			"pszDisplay"
		end

end -- class WEL_NM_DATETIMEFORMAT

--|-------------------------------------------------------------|
--|Created by David Jenkins										|
--|																|
--|It's not my fault if it works								|
--|-------------------------------------------------------------|