indexing
	description: "Contains information about a date and time %
		%picker (DTP) control callback field. It contains a %
		%substring (taken from the control's format string) that %
		%defines a callback field. The structure receives the %
		%maximum allowable size of the text that will be %
		%displayed in the callback field. This structure is used %
		%with the DTN_FORMATQUERY notification message."
	author: "David Jenkins"
	date: "$Date: 2008/12/01 $"
	revision: "$Revision: #2 $"

class
	WEL_NM_DATE_TIME_FORMAT_QUERY

inherit
	WEL_STRUCTURE

create
	make,
	make_by_nmhdr,
	make_by_pointer

feature {NONE} -- Initialization

	make_by_nmhdr (a_nmhdr: WEL_NMHDR) is
			-- Make the structure with `a_nmhdr'.
		require
			a_nmhdr_not_void: a_nmhdr /= Void
		do
			make_by_pointer (a_nmhdr.item)
		end

feature -- Access

	hdr: WEL_NMHDR is
			-- Information about the Wm_notify message
		do
			create Result.make_by_pointer(c_nm_dtfq_get_hdr(item))
		ensure
			result_not_void: Result /= Void
		end

	format:STRING is
			-- A DTP control callback substring, consisting of
			-- one or more "X" characters
		do
			create Result.make_from_c(c_nm_dtfq_format(item))
		end

feature -- Element change

	set_max(sz:WEL_SIZE) is
			-- Sets maximum size of the text that will be displayed
			-- in the DTP callback field.  Setting this value
			-- reduces flicker in the control when it's repainted
		require
			sz_exists: sz /= Void
		do
			c_nm_dtfq_set_max(item,sz.item)
		end

feature -- Measurement

	structure_size:INTEGER is
			-- Size to allocate (in bytes)
		once
			Result := c_size_of_nm_datetimeformatquery
		end

feature {NONE} -- Implementation

	c_size_of_nm_datetimeformatquery:INTEGER is
		external
			"C[macro %"nmdtfqry.h%"]"
		alias
			"sizeof(NMDATETIMEFORMATQUERY)"
		end

	c_nm_dtfq_get_hdr(handle:POINTER):POINTER is
		external
			"C[macro %"nmdtfqry.h%"](NMDATETIMEFORMATQUERY*):EIF_POINTER)"
		end

	c_nm_dtfq_format(handle:POINTER):POINTER is
		external
			"C[struct %"wel.h%"](NMDATETIMEFORMATQUERY):LPCTSTR"
		alias
			"pszFormat"
		end

	c_nm_dtfq_set_max(handle,sz_ptr:POINTER) is
		external
			"C[macro %"nmdtfqry.h%"](NMDATETIMEFORMATQUERY*,PSIZE)"
		end

end -- class WEL_DATE_TIME_FORMAT_QUERY

--|-------------------------------------------------------------|
--|Created by David Jenkins										|
--|																|
--|It's not my fault if it works								|
--|-------------------------------------------------------------|
