indexing
	description: "Contains information specific to an edit %
		%operation that has taken place in a date and time picker %
		%(DTP) control. This message is used with the %
		%DTN_USERSTRING notification message."
	author: "David Jenkins"
	date: "$Date: 2008/12/01 $"
	revision: "$Revision: #3 $"

class
	WEL_NM_DATE_TIME_STRING

inherit
	WEL_STRUCTURE

	GDT_CONSTANTS
		export
			{ANY} valid_gdt_value
			{NONE} all
		undefine
			copy, is_equal
		end

create
	make,
	make_by_nmhdr,
	make_by_pointer

feature {NONE} -- Initialization

	make_by_nmhdr (a_nmhdr: WEL_NMHDR) is
			-- Make the structure with `a_nmhdr'.
		require
			a_nmhdr_not_void: a_nmhdr /= Void
		do
			make_by_pointer (a_nmhdr.item)
		end

feature -- Access

	hdr: WEL_NMHDR is
			-- Information about the Wm_notify message
		do
			create Result.make_by_pointer(c_nm_dts_get_hdr(item))
		ensure
			result_not_void: Result /= Void
		end

	user_string:STRING is
			-- The string the user entered
		do
			create Result.make_from_c(c_nm_dts_user_string(item))
		end

feature -- Measurement

	structure_size:INTEGER is
			-- Size to allocate (in bytes)
		once
			Result := c_size_of_nm_datetimestring
		end

feature -- Element change

	set_system_time(d_t:DATE_TIME) is
			-- Sets the DTP controls time
		require
			d_t_exists: d_t /= Void
		local
			syst: MC_SYSTEM_TIME
		do
			create syst.make_by_date_time(d_t)
			c_nm_dts_set_st(item,syst.item)
		end

	set_flags(f:INTEGER) is
			-- Set to GDT_VALID to indicate the the st
			-- (system_time) member is valid or to
			-- GDT_NONE to set the control to "no date" status
		require
			valid_flag: valid_gdt_value(f)
		do
			c_nm_dts_set_flags(item,f)
		end

feature {NONE} -- Implementation

	c_size_of_nm_datetimestring:INTEGER is
		external
			"C[macro %"nmdtchng.h%"]"
		alias
			"sizeof(NMDATETIMESTRING)"
		end

	c_nm_dts_get_hdr(handle:POINTER):POINTER is
		external
			"C[macro %"nmdtstr.h%"](NMDATETIMESTRING*):EIF_POINTER"
		end

	c_nm_dts_user_string(handle:POINTER):POINTER is
		external
			"C [struct %"wel.h%"] (NMDATETIMESTRING):LPCTSTR"
		alias
			"pszUserString"
		end

	c_nm_dts_set_st(handle,syst_ptr:POINTER) is
		external
			"C[macro %"nmdtstr.h%"](NMDATETIMESTRING*,LPSYSTEMTIME)"
		end

	c_nm_dts_set_flags(handle:POINTER; flag:INTEGER) is
		external
			"C [struct %"wel.h%"] (NMDATETIMESTRING,DWORD)"
		alias
			"dwFlags"
		end


end -- class WEL_NM_DATE_TIME_STRING
