indexing
	description: "Carries information used to describe and handle %
		%a Date and Time Picker DTN_WMKEYDOWN notification message."
	author: "David Jenkins"
	date: "$Date: 2008/12/01 $"
	revision: "$Revision: #2 $"

class
	WEL_NM_DATE_TIME_WMKEYDOWN

inherit
	WEL_STRUCTURE

create
	make,
	make_by_nmhdr,
	make_by_pointer

feature {NONE} -- Initialization

	make_by_nmhdr (a_nmhdr: WEL_NMHDR) is
			-- Make the structure with `a_nmhdr'.
		require
			a_nmhdr_not_void: a_nmhdr /= Void
		do
			make_by_pointer (a_nmhdr.item)
		end

feature -- Access

	hdr: WEL_NMHDR is
			-- Information about the Wm_notify message
		do
			create Result.make_by_pointer(c_nm_dtwmkd_get_hdr(item))
		ensure
			result_not_void: Result /= Void
		end

	virtual_key: INTEGER is
			-- Virtual key code that represents the key that the 
			-- user pressed.
		do
			Result := c_nm_dtwmkd_nvirtkey(item)
		end

	format_substring:STRING is
			-- Defines the callback field.  Contains 1 or more
			-- "X" characters
		do
			create Result.make_from_c(c_nm_dtwmkd_format(item))
		end

	system_time:MC_SYSTEM_TIME is
			-- SYSTEMTIME structure containing the current date 
			-- and time from the DTP control. The owner of the 
			-- control must modify the time information based on 
			-- the user's keystroke.
		do
			create Result.make_by_pointer(c_nm_dtwmkd_st(item))
		end

feature -- Measurement

	structure_size:INTEGER is
			-- Size to allocate (in bytes)
		once
			Result := c_size_of_nm_wmkeydown
		end

feature -- Element change

	set_system_time(st:WEL_SYSTEM_TIME) is
			-- Change DTP control time
		require
			systemtime_not_void: st /= Void
		do
			c_nm_dtwmkd_set_st(item,st.item)
		end

feature {NONE} -- Implementation

	c_size_of_nm_wmkeydown:INTEGER is
		external
			"C[macro %"nmdtwmkd.h%"]"
		alias
			"sizeof(NMDATETIMEWMKEYDOWN)"
		end

	c_nm_dtwmkd_get_hdr(handle:POINTER):POINTER is
		external
			"C[macro %"nmdtwmkd.h%"](NMDATETIMEWMKEYDOWN*):EIF_POINTER"
		end

	c_nm_dtwmkd_nvirtkey(handle:POINTER):INTEGER is
		external
			"C [struct %"wel.h%"] (NMDATETIMEWMKEYDOWN):EIF_INTEGER"
		alias
			"nVirtKey"
		end

	c_nm_dtwmkd_format(handle:POINTER):POINTER is
		external
			"C [struct %"wel.h%"] (NMDATETIMEWMKEYDOWN):EIF_POINTER"
		alias
			"pszFormat"
		end

	c_nm_dtwmkd_st(handle:POINTER):POINTER is
		external
			"C[macro %"nmdtwmkd.h%"](NMDATETIMEWMKEYDOWN*):LPSYSTEMTIME"
		end

	c_nm_dtwmkd_set_st(handle,syst_ptr:POINTER) is
		external
			"C[macro %"nmdtwmkd.h%"](NMDATETIMEWMKEYDOWN*,LPSYSTEMTIME)"
		end

end -- class WEL_NM_DATE_TIME_WMKEYDOWN

--|-------------------------------------------------------------|
--|Created by David Jenkins										|
--|																|
--|It's not my fault if it works								|
--|-------------------------------------------------------------|
