indexing
	description: "The Windows structure WIN32_FIND_DATA"
	author: "RJ"
	date: "$Date: 2009/08/18 $"
	revision: "$Revision: #6 $"

class
	WIN32_FIND_DATA

inherit
	EXTERNAL_RECORD
		redefine
--			display,
			write,
			read,
			record_length_,
			classifier_major_version_,
			classifier_minor_version_
		end
	C_ACCESS_SUPPORT
	EXCEPTIONS
		export
			{NONE}
				all
		end

feature -- Access 
	classifier_major_version_ : INTEGER is 2
		-- Classifier major version

	classifier_minor_version_ : INTEGER is 0

	read is
	do
		file_attributes := read_file_attributes
		creation_time := read_creation_time
		last_access_time := read_last_access_time
		last_write_time := read_last_write_time
		file_size_high := read_file_size_high
		file_size_low := read_file_size_low
		oid := read_oid
--		reserved2 := read_reserved2
		filename := read_filename
--		alternate_filename := read_alternate_filename
	end

	write is
	do
		raise("Not implemented")
	end

	record_length_: INTEGER is
	do
		Result := 4 + 8 + 8 + 8 + 4 + 4 + 4 + 4 + MAX_PATH + 14
	end

	file_attributes: INTEGER
	read_file_attributes: INTEGER is
	do
		Result := integer_interpreter_.decode(pointer_ + (0), 4)
	end

	creation_time: ROSE_DATE
	read_creation_time: ROSE_DATE is
	local
		l_status: BOOLEAN
		l_systemtime: SYSTEMTIME
	do
		l_status := FileTimeToSystemTime(pointer_ + 4, temp_systemtime_buffer.item)
		create l_systemtime.make(temp_systemtime_buffer)
		Result := l_systemtime.date_time
	end

	last_access_time: ROSE_DATE
	read_last_access_time: ROSE_DATE is
	local
		l_status: BOOLEAN
		l_systemtime: SYSTEMTIME
	do
		l_status := FileTimeToSystemTime(pointer_ + 12, temp_systemtime_buffer.item)
		create l_systemtime.make(temp_systemtime_buffer)
		Result := l_systemtime.date_time
	end

	last_write_time: ROSE_DATE
	read_last_write_time: ROSE_DATE is
	local
		l_status: BOOLEAN
		l_systemtime: SYSTEMTIME
	do
		l_status := FileTimeToSystemTime(pointer_ + 20, temp_systemtime_buffer.item)
		create l_systemtime.make(temp_systemtime_buffer)
		Result := l_systemtime.date_time
	end

	file_size_high: NATURAL_32
	read_file_size_high: NATURAL_32 is
	do
		Result := integer_interpreter_.decode(pointer_ + (28), 4).to_natural_32
	end

	file_size_low: NATURAL_32
	read_file_size_low: NATURAL_32 is
	do
		Result := integer_interpreter_.decode(pointer_ + (32), 4).to_natural_32
	end

	oid: INTEGER
	read_oid: INTEGER is
	do
		Result := integer_interpreter_.decode(pointer_ + (36), 4)
	end

	reserved2: INTEGER
	read_reserved2: INTEGER is
	do
		Result := integer_interpreter_.decode(pointer_ + (40), 4)
	end

	filename: STRING
	read_filename: STRING is
	do
			create Result.make_from_c(pointer_ + (44))
	end

	alternate_filename: STRING
	read_alternate_filename: STRING is
	do
			create Result.make_from_c(pointer_ + (44 + MAX_PATH))
	end

	is_dir: BOOLEAN is
	do
		Result := (file_attributes & FILE_ATTRIBUTE_DIRECTORY)/=0
	end

feature {NONE} -- Implementation
	temp_systemtime_buffer: MANAGED_POINTER is
	-- Shared by all WIN32_FIND_DATA's, you must use this and then be willing to throw it away.
	once
		create Result.make((create {SYSTEMTIME}).record_length_)
	end

feature {NONE} -- Implementation

	FILE_ATTRIBUTE_DIRECTORY: INTEGER is
	external
		"C [macro %"winnt.h%"]"
	alias
		"FILE_ATTRIBUTE_DIRECTORY"
	end

	MAX_PATH: INTEGER is
	external
		"C [macro %"stdlib.h%"]"
	alias
		"MAX_PATH"
	end

	FileTimeToSystemTime(a_ft_pointer, a_st_pointer: POINTER): BOOLEAN is
	external
		"C [macro <windows.h>] (FILETIME*, LPSYSTEMTIME): EIF_BOOLEAN"
	alias
		"FileTimeToSystemTime"
	end

end
