note

	description:

	"[
		Interface to the Windows built-in PRNG
	]"

	author: "Ivan Tarnavsky"
	last_editor: "$Author: it $"
	date: "$Date: 2015/02/18 $"
	revision: "$Revision: #1 $"
	path: "$File: //rose/source/kernel/math/random/windows/windows_pseudo_random_number_generator.e $"

class WINDOWS_PSEUDO_RANDOM_NUMBER_GENERATOR

inherit

	DISPOSABLE

create

	make

feature {NONE} -- Initialization

	make
			-- Initalize `Current'
		do
			create random.make ({MANAGED_POINTER}.integer_32_bytes)
			create space.make ({MANAGED_POINTER}.integer_64_bytes)
			if cpp_cryptacquirecontextw (space.item) /= 0 then
				(create {EXCEPTIONS}).raise ("Cannot acquire context")
			end
		end

feature -- Access

	next: INTEGER_32
			-- Random integer
		do
			if cpp_cryptgenrandom (space.item, random.item) /= 0 then
				(create {EXCEPTIONS}).raise ("Cannot generate random number")
			end
			Result := random.read_integer_32 (0)
		end

feature -- Basic Operations

	dispose
			-- <Precursor>
		local
			l_ignored: INTEGER
		do
			l_ignored := cpp_cryptreleasecontext (space.item)
		end

feature {NONE} -- Implementation

	space: MANAGED_POINTER
			-- Memory for HCRYPTPROV

	random: MANAGED_POINTER
			-- Memory for random bytes

	cpp_cryptacquirecontextw (a_pointer: POINTER): INTEGER_32
			-- Call Win32 API CryptAcquireContextW
		external
			"C inline use <Wincrypt.h>"
		alias
			"{
				if (!CryptAcquireContextW((HCRYPTPROV *)$a_pointer, 0, 0, PROV_RSA_FULL, CRYPT_VERIFYCONTEXT | CRYPT_SILENT))
					return 1;
			 	return 0;
			}"
		end

	cpp_cryptreleasecontext (a_pointer: POINTER): INTEGER_32
			-- Call Win32 API CryptReleaseContext
		external
			"C inline use <Wincrypt.h>"
		alias
			"{
				if (!CryptReleaseContext(*(HCRYPTPROV *)$a_pointer, 0))
					return 1;
			 	return 0;
			}"
		end

	cpp_cryptgenrandom (a_pointer, a_random: POINTER): INTEGER_32
			-- Call Win32 API CryptGenRandom
		external
			"C inline use <Wincrypt.h>"
		alias
			"{
				if (!CryptGenRandom(*(HCRYPTPROV *)$a_pointer, 4, (BYTE *)$a_random))
					return 1;
			 	return 0;
			}"
		end

end
