note
	description: "Access to Windows special folders"
	author: "Zoran Simic"
	last_editor: "$Author: zs $"
	date: "$Date: 2011/05/13 $"
	revision: "$Revision: #3 $"
	path: "$File: //rose/source/kernel/infrastructure/support/windows/windows_special_folders.e $"

class
	WINDOWS_SPECIAL_FOLDERS

inherit
	ROSE_SPECIAL_FOLDERS

feature -- Access

	system_directory: attached STRING
			-- Default: C:\WINDOWS
		local
			cs: C_STRING
			i: NATURAL_32
		once
			create Result.make_empty
			create cs.make_empty (MAX_PATH)
			i := GetSystemWindowsDirectory (cs.item, MAX_PATH)
			Result.append_string (cs.string)
		end

	system32_directory: attached STRING
			-- Default: C:\WINDOWS\system32
		once
			Result := get_default_path (CSIDL_SYSTEM)
		end

	program_files: attached STRING
			-- Default: C:\Program Files
		once
			Result := get_default_path (CSIDL_PROGRAM_FILES)
		end

	program_files_common: attached STRING
			-- Default: C:\Program Files\Common Files
		once
			Result := get_default_path (CSIDL_PROGRAM_FILES_COMMON)
		end

	appdata: attached STRING
			-- Default: C:\Documents and Settings\<username>\Application Data
		once
			Result := get_path (CSIDL_APPDATA)
		end

	local_appdata: attached STRING
			-- Default: C:\Documents and Settings\<username>\Local Settings\Application Data
			-- This appdata is not roaming (while 'appdata' is roaming)
		once
			Result := get_path (CSIDL_LOCAL_APPDATA)
		end

	startmenu: attached STRING
			-- Default: C:\Documents and Settings\<username>\Start Menu
		once
			Result := get_path (CSIDL_STARTMENU)
		end

	common_appdata: attached STRING
			-- Default: C:\Documents and Settings\All Users\Application Data
		once
			Result := get_path (CSIDL_COMMON_APPDATA)
		end

	common_startmenu: attached STRING
			-- Default: C:\Documents and Settings\All Users\Start Menu\Programs
		once
			Result := get_path (CSIDL_COMMON_STARTMENU)
		end

	programs: attached STRING
			-- Default: C:\Documents and Settings\<username>\Start Menu\Programs
		once
			Result := get_path (CSIDL_PROGRAMS)
		end

	common_programs: attached STRING
			-- Default: C:\Documents and Settings\All Users\Start Menu\Programs
		once
			Result := get_path (CSIDL_COMMON_PROGRAMS)
		end

	desktop: attached STRING
			-- Default: C:\Documents and Settings\<username>\Desktop
		once
			Result := get_path (CSIDL_DESKTOPDIRECTORY)
		end

	common_desktop: attached STRING
			-- Default: C:\Documents and Settings\All Users\Desktop
		once
			Result := get_path (CSIDL_COMMON_DESKTOPDIRECTORY)
		end

	sendto: attached STRING
			-- Default: C:\Documents and Settings\<username>\SendTo
		once
			Result := get_path (CSIDL_SENDTO)
		end

	admintools: attached STRING
			-- Default: C:\Documents and Settings\<username>\Start Menu\Programs\Administrative Tools
		once
			Result := get_path (CSIDL_ADMINTOOLS)
		end

	common_admintools: attached STRING
			-- Default: C:\Documents and Settings\All Users\Start Menu\Programs\Administrative Tools
		once
			Result := get_path (CSIDL_COMMON_ADMINTOOLS)
		end

	startup: attached STRING
			-- Default: C:\Documents and Settings\<username>\Start Menu\Programs\Startup
		once
			Result := get_path (CSIDL_STARTUP)
		end

	common_startup: attached STRING
			-- Default: C:\Documents and Settings\All Users\Start Menu\Programs\Startup
		once
			Result := get_path (CSIDL_COMMON_STARTUP)
		end

	personal: attached STRING
			-- Default: C:\Documents and Settings\<username>\My Documents
		once
			Result := get_path (CSIDL_PERSONAL)
		end

	common_documents: attached STRING
			-- Default: C:\Documents and Settings\All Users\Documents
		once
			Result := get_path (CSIDL_COMMON_DOCUMENTS)
		end

	cookies: attached STRING
			-- Default: C:\Documents and Settings\<username>\Cookies
		once
			Result := get_path (CSIDL_COOKIES)
		end

	history: attached STRING
			-- Default: C:\Documents and Settings\<username>\Local Settings\History
		once
			Result := get_path (CSIDL_HISTORY)
		end

	internet_cache: attached STRING
			-- Default: C:\Documents and Settings\<username>\Local Settings\Temporary Internet Files
		once
			Result := get_path (CSIDL_INTERNET_CACHE)
		end

	mypictures: attached STRING
			-- Default: C:\Documents and Settings\<username>\My Documents\My Pictures
		once
			Result := get_path (CSIDL_MYPICTURES)
		end

feature {NONE} -- Implementation

	get_path (a_csidl: INTEGER): attached STRING
			-- Path corresponding to 'a_csidl'
		local
			cs: C_STRING
			i: INTEGER
		do
			create Result.make_empty
			create cs.make_empty (MAX_PATH)
			i := SHGetFolderPath (default_pointer, a_csidl, default_pointer, SHGFP_TYPE_CURRENT, cs.item)
			Result.append_string (cs.string)
		end

	get_default_path (a_csidl: INTEGER): attached STRING
			-- Default path corresponding to 'a_csidl'
		local
			cs: C_STRING
			i: INTEGER
		do
			create Result.make_empty
			create cs.make_empty (MAX_PATH)
			i := SHGetFolderPath (default_pointer, a_csidl, default_pointer, SHGFP_TYPE_DEFAULT, cs.item)
			Result.append_string (cs.string)
		end

	GetSystemWindowsDirectory (lpBuffer: POINTER; uSize: INTEGER): NATURAL_32
			-- Path of the shared Windows directory on a multi-user system
		external
			"C blocking signature (LPSTR,UINT): EIF_NATURAL use <windows.h>"
		alias
			"GetSystemWindowsDirectoryA"
		end

	SHGetFolderPath (hwndOwner: POINTER; nFolder: INTEGER; hToken: POINTER; dwFlags: INTEGER; pszPath: POINTER): INTEGER
			-- Takes the CSIDL of a folder and returns the path
		external
			"C blocking signature (HWND,int,HANDLE,DWORD,LPSTR): EIF_INTEGER use <shlobj.h>"
		alias
			"SHGetFolderPathA"
		end

	MAX_PATH: INTEGER
			-- MAx path length
		external
			"C blocking macro use <Windows.h>"
		alias
			"MAX_PATH"
		end

	SHGFP_TYPE_CURRENT: INTEGER
			-- Return the folder's current path
		external
			"C blocking macro use <Windows.h>"
		alias
			"SHGFP_TYPE_CURRENT"
		end

	SHGFP_TYPE_DEFAULT: INTEGER
			-- Return the folder's default path
		external
			"C blocking macro use <Windows.h>"
		alias
			"SHGFP_TYPE_DEFAULT"
		end

	CSIDL_DRIVES: INTEGER
			-- CSIDL_DRIVES
		external
			"C blocking macro use <shlobj.h>"
		alias
			"CSIDL_DRIVES"
		end

	CSIDL_SYSTEM: INTEGER
			-- CSIDL_SYSTEM
		external
			"C blocking macro use <shlobj.h>"
		alias
			"CSIDL_SYSTEM"
		end

	CSIDL_SENDTO: INTEGER
			-- CSIDL_SENDTO
		external
			"C blocking macro use <shlobj.h>"
		alias
			"CSIDL_SENDTO"
		end

	CSIDL_LOCAL_APPDATA: INTEGER
			-- CSIDL_LOCAL_APPDATA
		external
			"C blocking macro use <shlobj.h>"
		alias
			"CSIDL_LOCAL_APPDATA"
		end

	CSIDL_APPDATA: INTEGER
			-- CSIDL_APPDATA
		external
			"C blocking macro use <shlobj.h>"
		alias
			"CSIDL_APPDATA"
		end

	CSIDL_COMMON_APPDATA: INTEGER
			-- CSIDL_COMMON_APPDATA
		external
			"C blocking macro use <shlobj.h>"
		alias
			"CSIDL_COMMON_APPDATA"
		end

	CSIDL_STARTMENU: INTEGER
			-- CSIDL_STARTMENU
		external
			"C blocking macro use <shlobj.h>"
		alias
			"CSIDL_STARTMENU"
		end

	CSIDL_COMMON_STARTMENU: INTEGER
			-- CSIDL_COMMON_STARTMENU
		external
			"C blocking macro use <shlobj.h>"
		alias
			"CSIDL_COMMON_PROGRAMS"
		end

	CSIDL_PROGRAMS: INTEGER
			-- CSIDL_PROGRAMS
		external
			"C blocking macro use <shlobj.h>"
		alias
			"CSIDL_PROGRAMS"
		end

	CSIDL_COMMON_PROGRAMS: INTEGER
			-- CSIDL_COMMON_PROGRAMS
		external
			"C blocking macro use <shlobj.h>"
		alias
			"CSIDL_COMMON_PROGRAMS"
		end

	CSIDL_DESKTOPDIRECTORY: INTEGER
			-- CSIDL_DESKTOPDIRECTORY
		external
			"C blocking macro use <shlobj.h>"
		alias
			"CSIDL_DESKTOP"
		end

	CSIDL_COMMON_DESKTOPDIRECTORY: INTEGER
			-- CSIDL_COMMON_DESKTOPDIRECTORY
		external
			"C blocking macro use <shlobj.h>"
		alias
			"CSIDL_COMMON_DESKTOPDIRECTORY"
		end

	CSIDL_ADMINTOOLS: INTEGER
			-- CSIDL_ADMINTOOLS
		external
			"C blocking macro use <shlobj.h>"
		alias
			"CSIDL_ADMINTOOLS"
		end

	CSIDL_COMMON_ADMINTOOLS: INTEGER
			-- CSIDL_COMMON_ADMINTOOLS
		external
			"C blocking macro use <shlobj.h>"
		alias
			"CSIDL_COMMON_ADMINTOOLS"
		end

	CSIDL_STARTUP: INTEGER
			-- CSIDL_STARTUP
		external
			"C blocking macro use <shlobj.h>"
		alias
			"CSIDL_STARTUP"
		end

	CSIDL_COMMON_STARTUP: INTEGER
			-- CSIDL_COMMON_STARTUP
		external
			"C blocking macro use <shlobj.h>"
		alias
			"CSIDL_COMMON_STARTUP"
		end

	CSIDL_PERSONAL: INTEGER
			-- CSIDL_PERSONAL
		external
			"C blocking macro use <shlobj.h>"
		alias
			"CSIDL_PERSONAL"
		end

	CSIDL_COMMON_DOCUMENTS: INTEGER
			-- CSIDL_COMMON_DOCUMENTS
		external
			"C blocking macro use <shlobj.h>"
		alias
			"CSIDL_COMMON_DOCUMENTS"
		end

	CSIDL_COOKIES: INTEGER
			-- CSIDL_COOKIES
		external
			"C blocking macro use <shlobj.h>"
		alias
			"CSIDL_COOKIES"
		end

	CSIDL_HISTORY: INTEGER
			-- CSIDL_HISTORY
		external
			"C blocking macro use <shlobj.h>"
		alias
			"CSIDL_HISTORY"
		end

	CSIDL_INTERNET_CACHE: INTEGER
			-- CSIDL_INTERNET_CACHE
		external
			"C blocking macro use <shlobj.h>"
		alias
			"CSIDL_INTERNET_CACHE"
		end

	CSIDL_MYPICTURES: INTEGER
			-- CSIDL_MYPICTURES
		external
			"C blocking macro use <shlobj.h>"
		alias
			"CSIDL_MYPICTURES"
		end

	CSIDL_PROGRAM_FILES: INTEGER
			-- CSIDL_PROGRAM_FILES
		external
			"C blocking macro use <shlobj.h>"
		alias
			"CSIDL_PROGRAM_FILES"
		end

	CSIDL_PROGRAM_FILES_COMMON: INTEGER
			-- CSIDL_PROGRAM_FILES_COMMON
		external
			"C blocking macro use <shlobj.h>"
		alias
			"CSIDL_PROGRAM_FILES_COMMON"
		end

	CSIDL_WINDOWS: INTEGER
			-- CSIDL_WINDOWS
		external
			"C blocking macro use <shlobj.h>"
		alias
			"CSIDL_WINDOWS"
		end

end
