indexing
	description: "Windows system codes"
	author: "Zoran Simic"
	date: "$Date: 2010/04/20 $"
	revision: "$Revision: #6 $"
	archive: "$File: //rose/source/kernel/infrastructure/windows/windows_system_codes.e $"

class
	WINDOWS_SYSTEM_CODES

feature -- Access

	GetLastError: INTEGER is
			-- Retrieves the calling thread's last-error code value.
			-- The last-error code is maintained on a per-thread basis.
			-- Multiple threads do not overwrite each other's last-error code
		external
			"C signature (): EIF_INTEGER use <windows.h>"
		alias
			"GetLastError"
		end

	meaning (a_code: INTEGER): STRING is
			-- Error message text associated with `a_code'
		local
			l_buf: POINTER
			l_size: INTEGER
			l_flags: INTEGER
			l_c_string: C_STRING
		do
			l_flags := FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM | FORMAT_MESSAGE_IGNORE_INSERTS
			l_size := c_FormatMessage (l_flags, default_pointer, a_code, 0, $l_buf, 200, default_pointer)
			if l_size > 0 then
				create l_c_string.make_by_pointer (l_buf)
				Result := l_c_string.string
				c_LocalFree (l_buf)
				if Result /= Void then
					Result.prune_all ('%R')
				end
			end
			if Result = Void or else Result.is_empty then
				Result := "Error code " + a_code.out
			end
		end

	ERROR_SUCCESS: INTEGER is 0				-- The operation completed successfully.
	S_OK: INTEGER is 0						-- The operation completed successfully.
	ERROR_INVALID_FUNCTION: INTEGER is 1	-- Incorrect function.
	ERROR_FILE_NOT_FOUND: INTEGER is 2		-- The system cannot find the file specified.
	ERROR_PATH_NOT_FOUND: INTEGER is 3		-- The system cannot find the path specified.
	ERROR_TOO_MANY_OPEN_FILES: INTEGER is 4	-- The system cannot open the file.
	ERROR_ACCESS_DENIED: INTEGER is 5		-- Access is denied.
	ERROR_INVALID_HANDLE: INTEGER is 6		-- The handle is invalid.
	ERROR_ARENA_TRASHED: INTEGER is 7		-- The storage control blocks were destroyed.
	ERROR_NOT_ENOUGH_MEMORY: INTEGER is 8	-- Not enough storage is available to process this command.
	ERROR_INVALID_BLOCK: INTEGER is 9		-- The storage control block address is invalid.
	ERROR_BAD_ENVIRONMENT: INTEGER is 10	-- The environment is incorrect.
	ERROR_BAD_FORMAT: INTEGER is 11			-- An attempt was made to load a program with an incorrect format.
	ERROR_INVALID_ACCESS: INTEGER is 12		-- The access code is invalid.
	ERROR_INVALID_DATA: INTEGER is 13		-- The data is invalid.
	ERROR_OUTOFMEMORY: INTEGER is 14		-- Not enough storage is available to complete this operation.
	ERROR_INVALID_DRIVE: INTEGER is 15		-- The system cannot find the drive specified.
	ERROR_CURRENT_DIRECTORY: INTEGER is 16	-- The directory cannot be removed.
	ERROR_NOT_SAME_DEVICE: INTEGER is 17	-- The system cannot move the file to a different disk drive.
	ERROR_NO_MORE_FILES: INTEGER is 18		-- There are no more files.
	ERROR_WRITE_PROTECT: INTEGER is 19		-- The media is write protected.
	ERROR_BAD_UNIT: INTEGER is 20			-- The system cannot find the device specified.
	ERROR_NOT_READY: INTEGER is 21			-- The device is not ready.
	ERROR_BAD_COMMAND: INTEGER is 22		-- The device does not recognize the command.
	ERROR_CRC: INTEGER is 23				-- Data error (cyclic redundancy check).
	ERROR_BAD_LENGTH: INTEGER is 24			-- The program issued a command but the command length is incorrect.
	ERROR_SEEK: INTEGER is 25				-- The drive cannot locate a specific area or track on the disk.
	ERROR_NOT_DOS_DISK: INTEGER is 26		-- The specified disk or diskette cannot be accessed.
	ERROR_SECTOR_NOT_FOUND: INTEGER is 27	-- The drive cannot find the sector requested.
	ERROR_OUT_OF_PAPER: INTEGER is 28		-- The printer is out of paper.
	ERROR_WRITE_FAULT: INTEGER is 29		-- The system cannot write to the specified device.
	ERROR_READ_FAULT: INTEGER is 30			-- The system cannot read from the specified device.
	ERROR_GEN_FAILURE: INTEGER is 31		-- A device attached to the system is not functioning.
	ERROR_SHARING_VIOLATION: INTEGER is 32	-- The process cannot access the file because it is being used by another process.
	ERROR_LOCK_VIOLATION: INTEGER is 33		-- The process cannot access the file because another process has locked a portion of the file.
	ERROR_WRONG_DISK: INTEGER is 34			-- The wrong diskette is in the drive. Insert %2 (Volume Serial Number: %3) into drive %1.
	ERROR_SHARING_BUFFER_EXCEEDED: INTEGER is 36	-- Too many files opened for sharing.
	ERROR_HANDLE_EOF: INTEGER is 38			-- Reached the end of the file.
	ERROR_HANDLE_DISK_FULL: INTEGER is 39	-- The disk is full.
	ERROR_NOT_SUPPORTED: INTEGER is 50		-- The request is not supported.
	ERROR_REM_NOT_LIST: INTEGER is 51		-- Windows cannot find the network path. Verify that the network path is correct and the destination computer is not busy or turned off. If Windows still cannot find the network path, contact your network administrator.
	ERROR_DUP_NAME: INTEGER is 52			-- You were not connected because a duplicate name exists on the network. Go to System in Control Panel to change the computer name and try again.
	ERROR_BAD_NETPATH: INTEGER is 53		-- The network path was not found.
	ERROR_NETWORK_BUSY: INTEGER is 54		-- The network is busy.
	ERROR_DEV_NOT_EXIST: INTEGER is 55		-- The specified network resource or device is no longer available.
	ERROR_TOO_MANY_CMDS: INTEGER is 56		-- The network BIOS command limit has been reached.
	ERROR_ADAP_HDW_ERR: INTEGER is 57		-- A network adapter hardware error occurred.
	ERROR_BAD_NET_RESP: INTEGER is 58		-- The specified server cannot perform the requested operation.
	ERROR_UNEXP_NET_ERR: INTEGER is 59		-- An unexpected network error occurred.
	ERROR_BAD_REM_ADAP: INTEGER is 60		-- The remote adapter is not compatible.
	ERROR_PRINTQ_FULL: INTEGER is 61		-- The printer queue is full.
	ERROR_NO_SPOOL_SPACE: INTEGER is 62		-- Space to store the file waiting to be printed is not available on the server.
	ERROR_PRINT_CANCELLED: INTEGER is 63	-- Your file waiting to be printed was deleted.
	ERROR_NETNAME_DELETED: INTEGER is 64	-- The specified network name is no longer available.
	ERROR_NETWORK_ACCESS_DENIED: INTEGER is 65	-- Network access is denied.
	ERROR_BAD_DEV_TYPE: INTEGER is 66		-- The network resource type is not correct.
	ERROR_BAD_NET_NAME: INTEGER is 67		-- The network name cannot be found.
	ERROR_TOO_MANY_NAMES: INTEGER is 68		-- The name limit for the local computer network adapter card was exceeded.
	ERROR_TOO_MANY_SESS: INTEGER is 69		-- The network BIOS session limit was exceeded.
	ERROR_SHARING_PAUSED: INTEGER is 70		-- The remote server has been paused or is in the process of being started.
	ERROR_REQ_NOT_ACCEP: INTEGER is 71		-- No more connections can be made to this remote computer at this time because there are already as many connections as the computer can accept.
	ERROR_REDIR_PAUSED: INTEGER is 72		-- The specified printer or disk device has been paused.
	ERROR_FILE_EXISTS: INTEGER is 80		-- The file exists.
	ERROR_CANNOT_MAKE: INTEGER is 82		-- The directory or file cannot be created.
	ERROR_FAIL_I24: INTEGER is 83			-- Fail on INT 24.
	ERROR_OUT_OF_STRUCTURES: INTEGER is 84	-- Storage to process this request is not available.
	ERROR_ALREADY_ASSIGNED: INTEGER is 85	-- The local device name is already in use.
	ERROR_INVALID_PASSWORD: INTEGER is 86	-- The specified network password is not correct.
	ERROR_INVALID_PARAMETER: INTEGER is 87	-- The parameter is incorrect.
	ERROR_NET_WRITE_FAULT: INTEGER is 88	-- A write fault occurred on the network.
	ERROR_NO_PROC_SLOTS: INTEGER is 89		-- The system cannot start another process at this time.
	ERROR_TOO_MANY_SEMAPHORES: INTEGER is 100		-- Cannot create another system semaphore.
	ERROR_EXCL_SEM_ALREADY_OWNED: INTEGER is 101	-- The exclusive semaphore is owned by another process.
	ERROR_SEM_IS_SET: INTEGER is 102				-- The semaphore is set and cannot be closed.
	ERROR_TOO_MANY_SEM_REQUESTS: INTEGER is 103		-- The semaphore cannot be set again.
	ERROR_INVALID_AT_INTERRUPT_TIME: INTEGER is 104	-- Cannot request exclusive semaphores at interrupt time.
	ERROR_SEM_OWNER_DIED: INTEGER is 105	-- The previous ownership of this semaphore has ended.
	ERROR_SEM_USER_LIMIT: INTEGER is 106	-- Insert the diskette for drive %1.
	ERROR_DISK_CHANGE: INTEGER is 107		-- The program stopped because an alternate diskette was not inserted.
	ERROR_DRIVE_LOCKED: INTEGER is 108		-- The disk is in use or locked by another process.
	ERROR_BROKEN_PIPE: INTEGER is 109		-- The pipe has been ended.
	ERROR_OPEN_FAILED: INTEGER is 110		-- The system cannot open the device or file specified.
	ERROR_BUFFER_OVERFLOW: INTEGER is 111	-- The file name is too long.
	ERROR_DISK_FULL: INTEGER is 112			-- There is not enough space on the disk.
	ERROR_NO_MORE_SEARCH_HANDLES: INTEGER is 113	-- No more internal file identifiers available.
	ERROR_INVALID_TARGET_HANDLE: INTEGER is 114		-- The target internal file identifier is incorrect.
	ERROR_INVALID_CATEGORY: INTEGER is 117			-- The IOCTL call made by the application program is not correct.
	ERROR_INVALID_VERIFY_SWITCH: INTEGER is 118		-- The verify-on-write switch parameter value is not correct.
	ERROR_BAD_DRIVER_LEVEL: INTEGER is 119			-- The system does not support the command requested.
	ERROR_CALL_NOT_IMPLEMENTED: INTEGER is 120		-- This function is not supported on this system.
	ERROR_SEM_TIMEOUT: INTEGER is 121				-- The semaphore timeout period has expired.
	ERROR_INSUFFICIENT_BUFFER: INTEGER is 122		-- The data area passed to a system call is too small.
	ERROR_INVALID_NAME: INTEGER is 123		-- The filename, directory name, or volume label syntax is incorrect.
	ERROR_INVALID_LEVEL: INTEGER is 124		-- The system call level is not correct.
	ERROR_NO_VOLUME_LABEL: INTEGER is 125	-- The disk has no volume label.
	ERROR_MOD_NOT_FOUND: INTEGER is 126		-- The specified module could not be found.
	ERROR_PROC_NOT_FOUND: INTEGER is 127	-- The specified procedure could not be found.
	ERROR_WAIT_NO_CHILDREN: INTEGER is 128	-- There are no child processes to wait for.
	ERROR_CHILD_NOT_COMPLETE: INTEGER is 129	-- The %1 application cannot be run in Win32 mode.
	ERROR_DIRECT_ACCESS_HANDLE: INTEGER is 130	-- Attempt to use a file handle to an open disk partition for an operation other than raw disk I/O.
	ERROR_NEGATIVE_SEEK: INTEGER is 131		-- An attempt was made to move the file pointer before the beginning of the file.
	ERROR_SEEK_ON_DEVICE: INTEGER is 132	-- The file pointer cannot be set on the specified device or file.
	ERROR_IS_JOIN_TARGET: INTEGER is 133	-- A JOIN or SUBST command cannot be used for a drive that contains previously joined drives.
	ERROR_IS_JOINED: INTEGER is 134			-- An attempt was made to use a JOIN or SUBST command on a drive that has already been joined.
	ERROR_IS_SUBSTED: INTEGER is 135		-- An attempt was made to use a JOIN or SUBST command on a drive that has already been substituted.
	ERROR_NOT_JOINED: INTEGER is 136		-- The system tried to delete the JOIN of a drive that is not joined.
	ERROR_NOT_SUBSTED: INTEGER is 137		-- The system tried to delete the substitution of a drive that is not substituted.
	ERROR_JOIN_TO_JOIN: INTEGER is 138		-- The system tried to join a drive to a directory on a joined drive.
	ERROR_SUBST_TO_SUBST: INTEGER is 139	-- The system tried to substitute a drive to a directory on a substituted drive.
	ERROR_JOIN_TO_SUBST: INTEGER is 140		-- The system tried to join a drive to a directory on a substituted drive.
	ERROR_SUBST_TO_JOIN: INTEGER is 141		-- The system tried to SUBST a drive to a directory on a joined drive.
	ERROR_BUSY_DRIVE: INTEGER is 142		-- The system cannot perform a JOIN or SUBST at this time.
	ERROR_SAME_DRIVE: INTEGER is 143		-- The system cannot join or substitute a drive to or for a directory on the same drive.
	ERROR_DIR_NOT_ROOT: INTEGER is 144		-- The directory is not a subdirectory of the root directory.
	ERROR_DIR_NOT_EMPTY: INTEGER is 145		-- The directory is not empty.
	ERROR_IS_SUBST_PATH: INTEGER is 146		-- The path specified is being used in a substitute.
	ERROR_IS_JOIN_PATH: INTEGER is 147		-- Not enough resources are available to process this command.
	ERROR_PATH_BUSY: INTEGER is 148			-- The path specified cannot be used at this time.
	ERROR_IS_SUBST_TARGET: INTEGER is 149	-- An attempt was made to join or substitute a drive for which a directory on the drive is the target of a previous substitute.
	ERROR_SYSTEM_TRACE: INTEGER is 150		-- System trace information was not specified in your CONFIG.SYS file, or tracing is disallowed.
	ERROR_INVALID_EVENT_COUNT: INTEGER is 151	-- The number of specified semaphore events for DosMuxSemWait is not correct.
	ERROR_TOO_MANY_MUXWAITERS: INTEGER is 152	-- DosMuxSemWait did not execute; too many semaphores are already set.
	ERROR_INVALID_LIST_FORMAT: INTEGER is 153	-- The DosMuxSemWait list is not correct.
	ERROR_LABEL_TOO_LONG: INTEGER is 154	-- The volume label you entered exceeds the label character limit of the target file system.
	ERROR_TOO_MANY_TCBS: INTEGER is 155		-- Cannot create another thread.
	ERROR_SIGNAL_REFUSED: INTEGER is 156	-- The recipient process has refused the signal.
	ERROR_DISCARDED: INTEGER is 157			-- The segment is already discarded and cannot be locked.
	ERROR_NOT_LOCKED: INTEGER is 158		-- The segment is already unlocked.
	ERROR_BAD_THREADID_ADDR: INTEGER is 159	-- The address for the thread ID is not correct.
	ERROR_BAD_ARGUMENTS: INTEGER is 160		-- One or more arguments are not correct.
	ERROR_BAD_PATHNAME: INTEGER is 161		-- The specified path is invalid.
	ERROR_SIGNAL_PENDING: INTEGER is 162	-- A signal is already pending.
	ERROR_MAX_THRDS_REACHED: INTEGER is 164	-- No more threads can be created in the system.
	ERROR_LOCK_FAILED: INTEGER is 167		-- Unable to lock a region of a file.
	ERROR_BUSY: INTEGER is 170				-- The requested resource is in use.
	ERROR_CANCEL_VIOLATION: INTEGER is 173	-- A lock request was not outstanding for the supplied cancel region.
	ERROR_ATOMIC_LOCKS_NOT_SUPPORTED: INTEGER is 174	-- The file system does not support atomic changes to the lock type.
	ERROR_INVALID_SEGMENT_NUMBER: INTEGER is 180		-- The system detected a segment number that was not correct.
	ERROR_INVALID_ORDINAL: INTEGER is 182	-- The operating system cannot run %1.
	ERROR_ALREADY_EXISTS: INTEGER is 183	-- Cannot create a file when that file already exists.
	ERROR_INVALID_FLAG_NUMBER: INTEGER is 186		-- The flag passed is not correct.
	ERROR_SEM_NOT_FOUND: INTEGER is 187				-- The specified system semaphore name was not found.
	ERROR_INVALID_STARTING_CODESEG: INTEGER is 188	-- The operating system cannot run %1.
	ERROR_INVALID_STACKSEG: INTEGER is 189		-- The operating system cannot run %1.
	ERROR_INVALID_MODULETYPE: INTEGER is 190	-- The operating system cannot run %1.
	ERROR_INVALID_EXE_SIGNATURE: INTEGER is 191	-- Cannot run %1 in Win32 mode.
	ERROR_EXE_MARKED_INVALID: INTEGER is 192	-- The operating system cannot run %1.
	ERROR_BAD_EXE_FORMAT: INTEGER is 193		-- is not a valid Win32 application.
	ERROR_ITERATED_DATA_EXCEEDS_64k: INTEGER is 194	-- The operating system cannot run %1.
	ERROR_INVALID_MINALLOCSIZE: INTEGER is 195		-- The operating system cannot run %1.
	ERROR_DYNLINK_FROM_INVALID_RING: INTEGER is 196	-- The operating system cannot run this application program.
	ERROR_IOPL_NOT_ENABLED: INTEGER is 197			-- The operating system is not presently configured to run this application.
	ERROR_INVALID_SEGDPL: INTEGER is 198			-- The operating system cannot run %1.
	ERROR_AUTODATASEG_EXCEEDS_64k: INTEGER is 199	-- The operating system cannot run this application program.
	ERROR_RING2SEG_MUST_BE_MOVABLE: INTEGER is 200	-- The code segment cannot be greater than or equal to 64K.
	ERROR_RELOC_CHAIN_XEEDS_SEGLIM: INTEGER is 201	-- The operating system cannot run %1.
	ERROR_INFLOOP_IN_RELOC_CHAIN: INTEGER is 202	-- The operating system cannot run %1.
	ERROR_ENVVAR_NOT_FOUND: INTEGER is 203			-- The system could not find the environment option that was entered.
	ERROR_NO_SIGNAL_SENT: INTEGER is 205			-- No process in the command subtree has a signal handler.
	ERROR_FILENAME_EXCED_RANGE: INTEGER is 206		-- The filename or extension is too long.
	ERROR_RING2_STACK_IN_USE: INTEGER is 207		-- The ring 2 stack is in use.
	ERROR_META_EXPANSION_TOO_LONG: INTEGER is 208	-- The global filename characters, * or ?, are entered incorrectly or too many global filename characters are specified.
	ERROR_INVALID_SIGNAL_NUMBER: INTEGER is 209		-- The signal being posted is not correct.
	ERROR_THREAD_1_INACTIVE: INTEGER is 210			-- The signal handler cannot be set.
	ERROR_LOCKED: INTEGER is 212					-- The segment is locked and cannot be reallocated.
	ERROR_TOO_MANY_MODULES: INTEGER is 214			-- Too many dynamic-link modules are attached to this program or dynamic-link module.
	ERROR_NESTING_NOT_ALLOWED: INTEGER is 215		-- Cannot nest calls to LoadModule.
	ERROR_EXE_MACHINE_TYPE_MISMATCH: INTEGER is 216	-- The image file %1 is valid, but is for a machine type other than the current machine.
	ERROR_EXE_CANNOT_MODIFY_SIGNED_BINARY: INTEGER is 217			-- The image file %1 is signed, unable to modify.
	ERROR_EXE_CANNOT_MODIFY_STRONG_SIGNED_BINARY: INTEGER is 218	-- The image file %1 is strong signed, unable to modify.
	ERROR_BAD_PIPE: INTEGER is 230				-- The pipe state is invalid.
	ERROR_PIPE_BUSY: INTEGER is 231				-- All pipe instances are busy.
	ERROR_NO_DATA: INTEGER is 232				-- The pipe is being closed.
	ERROR_PIPE_NOT_CONNECTED: INTEGER is 233	-- No process is on the other end of the pipe.
	ERROR_MORE_DATA: INTEGER is 234				-- More data is available.
	ERROR_VC_DISCONNECTED: INTEGER is 240		-- The session was canceled.
	ERROR_INVALID_EA_NAME: INTEGER is 254		-- The specified extended attribute name was invalid.
	ERROR_EA_LIST_INCONSISTENT: INTEGER is 255	-- The extended attributes are inconsistent.
	WAIT_TIMEOUT: INTEGER is 258				-- The wait operation timed out.
	ERROR_NO_MORE_ITEMS: INTEGER is 259			-- No more data is available.
	ERROR_CANNOT_COPY: INTEGER is 266			-- The copy functions cannot be used.
	ERROR_DIRECTORY: INTEGER is 267				-- The directory name is invalid.
	ERROR_EAS_DIDNT_FIT: INTEGER is 275			-- The extended attributes did not fit in the buffer.
	ERROR_EA_FILE_CORRUPT: INTEGER is 276		-- The extended attribute file on the mounted file system is corrupt.
	ERROR_EA_TABLE_FULL: INTEGER is 277			-- The extended attribute table file is full.
	ERROR_INVALID_EA_HANDLE: INTEGER is 278		-- The specified extended attribute handle is invalid.
	ERROR_EAS_NOT_SUPPORTED: INTEGER is 282		-- The mounted file system does not support extended attributes.
	ERROR_NOT_OWNER: INTEGER is 288				-- Attempt to release mutex not owned by caller.
	ERROR_TOO_MANY_POSTS: INTEGER is 298		-- Too many posts were made to a semaphore.
	ERROR_PARTIAL_COPY: INTEGER is 299			-- Only part of a ReadProcessMemory or WriteProcessMemory request was completed.
	ERROR_OPLOCK_NOT_GRANTED: INTEGER is 300	-- The oplock request is denied.
	ERROR_INVALID_OPLOCK_PROTOCOL: INTEGER is 301	-- An invalid oplock acknowledgment was received by the system.
	ERROR_DISK_TOO_FRAGMENTED: INTEGER is 302	-- The volume is too fragmented to complete this operation.
	ERROR_DELETE_PENDING: INTEGER is 303		-- The file cannot be opened because it is in the process of being deleted.
	ERROR_MR_MID_NOT_FOUND: INTEGER is 317		-- The system cannot find message text for message number 0x%1 in the message file for %2.
	ERROR_SCOPE_NOT_FOUND: INTEGER is 318		-- The scope specified was not found.
	ERROR_INVALID_ADDRESS: INTEGER is 487		-- Attempt to access invalid address.
	ERROR_ARITHMETIC_OVERFLOW: INTEGER is 534	--  Arithmetic result exceeded 32 bits.
	ERROR_PIPE_CONNECTED: INTEGER is 535		-- There is a process on other end of the pipe.
	ERROR_PIPE_LISTENING: INTEGER is 536		-- Waiting for a process to open the other end of the pipe.
	ERROR_EA_ACCESS_DENIED: INTEGER is 994		-- Access to the extended attribute was denied.
	ERROR_OPERATION_ABORTED: INTEGER is 995		-- The I/O operation has been aborted because of either a thread exit or an application request.
	ERROR_IO_INCOMPLETE: INTEGER is 996			-- Overlapped I/O event is not in a signaled state.
	ERROR_IO_PENDING: INTEGER is 997			-- Overlapped I/O operation is in progress.
	ERROR_NOACCESS: INTEGER is 998				-- Invalid access to memory location.
	ERROR_SWAPERROR: INTEGER is 999				-- Error performing inpage operation.

	ERROR_SERVICE_DOES_NOT_EXIST: INTEGER is 1060	-- The specified service does not exist as an installed service.

feature -- Basic operations

	SetLastError (dwErrCode: INTEGER) is
			-- sets the last-error code for the calling thread
		external
			"C blocking signature (DWORD) use <windows.h>"
		alias
			"SetLastError"
		end

feature {NONE} -- Implementation

	c_FormatMessage (dwFlags: INTEGER; lpSource: POINTER; dwMessageId, dwLanguageId: INTEGER;
							lpBuffer: POINTER; nsize: INTEGER; Arguments: POINTER): INTEGER is
			-- Returns number of characters written to output buffer (0 = error)
		external
			"C blocking signature (DWORD, LPCVOID, DWORD, DWORD, LPSTR, DWORD, va_list *): EIF_INTEGER use <windows.h>"
		alias
			"FormatMessageA"
		end

	FORMAT_MESSAGE_ALLOCATE_BUFFER: INTEGER is
			-- FORMAT_MESSAGE_ALLOCATE_BUFFER
		external
			"C blocking macro use <windows.h>"
		alias
			"FORMAT_MESSAGE_ALLOCATE_BUFFER"
		end

	FORMAT_MESSAGE_IGNORE_INSERTS: INTEGER is
			-- FORMAT_MESSAGE_IGNORE_INSERTS
		external
			"C blocking macro use <windows.h>"
		alias
			"FORMAT_MESSAGE_IGNORE_INSERTS"
		end

	FORMAT_MESSAGE_FROM_SYSTEM: INTEGER is
			-- FORMAT_MESSAGE_FROM_SYSTEM
		external
			"C blocking macro use <windows.h>"
		alias
			"FORMAT_MESSAGE_FROM_SYSTEM"
		end

	c_LocalFree (hMem: POINTER) is
			-- LocalFree
		external
			"C blocking signature (HLOCAL) use <windows.h>"
		alias
			"LocalFree"
		end

end
