indexing
	description : "exchange rate record"
	author: "Mauricio Bustos"
	last_editor: "$Author: sm $"
	date: "$Date: 2009/08/14 $";
	revision: "$Revision: #13 $"
	path: "$File: //rose/source/kernel/data/supplier_data/xrate_data_signature.e $"

class
	XRATE_DATA_SIGNATURE

inherit
	ROSE_SIGNATURE_RECORD [PERSISTENT_CURRENCY]
		redefine
			rose_entity_date_make,
			data_manager,
			fields_,
			data_fields_string_,
			set_entity,
			output_string_by_name,
			reconcile_target,
			new_variable_template,
			new_variable
		end

	ROSE_ID_USER
		undefine
			is_equal
		end

create
	make,
	default_create

feature {NONE} -- Initialization

	make(a_context : SUPPLIER_CONTEXT) is
			-- Creation procedure.
		do
			set_context_name(a_context.name)
		end

feature -- Initialization

	rose_entity_date_make(a_entity : PERSISTENT_CURRENCY; a_date : ROSE_DATE) is
		local
			l_context : like context
		do
			if a_entity /= Void then
				l_context := a_entity.context
				if l_context /= Void then
					set_entity(a_entity)
					set_context_name(l_context.name)
				end
			end
			Precursor(a_entity,a_date)
		end

feature -- Access

	base_entity_id : INTEGER
		-- Handle

	target_entity_id : INTEGER
		-- Handle

	data_date : ROSE_DATE
		-- Date

	spot_rate : DOUBLE
		-- Exchange rate

	entity_id : INTEGER

	context_name : STRING
		-- Context name

	new_variable (a_description: VARIABLE_DESCRIPTION): like new_variable_template is
			-- create new variable
		do
			create Result.make (a_description)
		end

feature -- Change

	set_base_entity_id(a_value : like base_entity_id) is
			-- Set 'base_entity_id'	to 'a_value'
		require
			-- there should be a condition of the validity of the length of the handle (<16-bit for now)
		do
			base_entity_id := a_value
		ensure
			base_entity_id_set: base_entity_id = a_value
		end

	set_target_entity_id(a_value : like target_entity_id) is
			-- Set 'target_entity_id' to 'a_value'
		require
			-- there should be a condition of the validity of the length of the handle (<16-bit for now)
		do
			target_entity_id := a_value
		ensure
			target_entity_id_set: target_entity_id = a_value
		end

	set_data_date(a_value : like data_date) is
			-- Set 'data_date' to 'a_value'
		do
			data_date := a_value
		ensure
			data_date_set: data_date = a_value
		end

	set_spot_rate(a_value : like spot_rate) is
			-- Set 'spot_rate' to 'a_value'
		do
			spot_rate := a_value
		ensure
			spot_rate_set: spot_rate = a_value
		end

	set_entity_id(a_value : like entity_id) is
			-- set 'entity_id' to 'a_value'
		do
			entity_id := a_value
		ensure then
			entity_id_set: entity_id = a_value
		end

	set_context_name(a_value : like context_name) is
			-- Set 'context_name' to 'a_value'
		do
			context_name := a_value
		ensure
			context_name_set: context_name = a_value
		end

	set_entity(a_entity : PERSISTENT_CURRENCY) is
		do
			set_target_entity_id(a_entity.handle)
			entity := a_entity
		end

	data_manager : SIGNATURE_TAGGED_ITEM_DATABASE[like Current] is
		local
			l_table_structure : SIGNATURE_TAGGED_TABLE_STRUCTURE
			l_manager : SIGNATURE_TAGGED_ITEM_DATABASE[XRATE_DATA_SIGNATURE]
		do
			xrate_data_managers.search(context.supplier_id)
			if not xrate_data_managers.found then
				create l_table_structure.make(context.server,"xrate",Void)
				if l_table_structure.data_available then
					create l_manager.make_named("xrate","read",context.server)
					xrate_data_managers.force(l_manager,context.supplier_id)
				else
					xrate_data_managers.force(Void,context.supplier_id)
				end
			else
				l_manager := xrate_data_managers.found_item
			end
			Result := l_manager
		end

feature -- Output

	output_string_by_name(a_name : STRING) : STRING is
		local
			l_currency : PERSISTENT_CURRENCY
		do
			a_name.to_lower
			if a_name.is_equal("base_entity_id") then
				l_currency ?= context.entity_at(ids.currency_category_id,base_entity_id)
				if l_currency /= Void and l_currency.is_valid then
					Result := l_currency.attribute_by_name ("ISO CODE",Void)
				end
			elseif a_name.is_equal("target_entity_id") then
				l_currency ?= context.entity_at(ids.currency_category_id,target_entity_id)
				if l_currency /= Void and l_currency.is_valid then
					Result := l_currency.attribute_by_name ("ISO CODE",Void)
				end
			else
				Result := Precursor(a_name)
			end
			if Result = Void then
				Result := "<Void>"
			end
		end

feature -- Constants

	expression_tag : STRING is
		do
			Result := "xrate"
		end

	restriction_statement : STRING is
		do
		ensure then
			result_is_void: Result = Void
		end

	fields_ : STRING is "base_entity_id,target_entity_id,data_date"

	data_fields_string_ : STRING is "spot_rate"

feature  -- Basic Operations

	reconcile_target(a_target : like Current) is
		local
			l_currency : PERSISTENT_CURRENCY
		do
			a_target.set_context_name(context_name)
			l_currency ?= context.entity_at(ids.currency_category_id,a_target.target_entity_id)
			a_target.set_entity(l_currency)
		end

feature {NONE} -- Implementation

	new_variable_template : INDEX_XRATE_EXPRESSION is
			-- Template
		do
		end

	xrate_data_managers : HASH_TABLE[SIGNATURE_TAGGED_ITEM_DATABASE[XRATE_DATA_SIGNATURE],INTEGER] is
			-- Shared structure
		once
			create Result.make(10)
		end

end

