indexing
	description: "xtx matrix which keeps track of row generating max conrributions for each diag"
	author: "Tom Mead"
	date: "$Date: 2008/12/01 $"
	revision: "$Revision: #5 $"
	archive: "$File: //rose/source/kernel/infrastructure/math/xtx_with_contributors.e $"

class
	XTX_WITH_CONTRIBUTORS

inherit
	SYMMETRIC_MATRIX
		rename
			make_xtx as symmetric_make_xtx
		redefine
--			xtx
		end

create
	make_xtx
create {XTX_WITH_CONTRIBUTORS}
	matrix_make,make,make_from_triangle,symmetric_make_xtx

feature -- Access

	make_xtx(a_m: ABSTRACT_READABLE_MATRIX) is
		local
			i,j,k : INTEGER
			l_dot_product : DOUBLE
			l_temp_dot_product: DOUBLE
		do
			number_of_extremes:= 3
			create minimums.make(a_m.width)
			create extreme_contributions.make(1, a_m.width)
			make(a_m.width)
			from i := 1 until i > width loop
				from j := 1 until j > i loop
					l_dot_product := 0
					from k := 1 until k > a_m.height loop
						l_temp_dot_product:= a_m [k, i] * a_m [k, j]
						l_dot_product := l_dot_product + l_temp_dot_product
						if i = j and l_temp_dot_product > minimums.item(i) then
							refresh_extremes(l_temp_dot_product, k, i)
						end
						k := k + 1
					end
				--	put(l_dot_product,i,j)
					Current[i, j] := l_dot_product
					j := j + 1
				end
				i := i + 1
			end
		end

	refresh_extremes(a_value: DOUBLE; a_row_index, a_column_index: INTEGER) is
			--
		local
			i: INTEGER
			l_current_array: ROSE_LINEAR_ARRAY[PAIR[INTEGER, DOUBLE]]
			l_found:  BOOLEAN
			l_found_pos: INTEGER
			l_pair: PAIR[INTEGER, DOUBLE]
		--	l_tail: ROSE_LINEAR_ARRAY[PAIR[INTEGER, DOUBLE]]
			l_new_array, l_temp_array: ROSE_LINEAR_ARRAY[PAIR[INTEGER, DOUBLE]]
		do
			if extreme_contributions.item(a_column_index) = Void then
				create l_temp_array.make(1, number_of_extremes)
				from i:= 1 until i> number_of_extremes loop
					create l_pair.make(0,0)
					l_temp_array.put(l_pair, i)
					i:= i+ 1
				end
				extreme_contributions.put(l_temp_array, a_column_index)
			end
			l_current_array:= extreme_contributions.item(a_column_index)
			from l_current_array.start until l_current_array.off or l_found loop
				if a_value > l_current_array.current_item.second then
					Create l_pair.make(a_row_index, a_value)
					l_found:= true
					l_found_pos:= l_current_array.index
				end
				l_current_array.forth
			end
			if l_pair /= Void then
				create l_new_array.make(1, number_of_extremes)
				from i:= 1 until i > l_found_pos- 1 loop
					l_new_array.put(l_current_array.item(i), i)
					i:= i+ 1
				end
				l_new_array.put(l_pair, l_found_pos)
				from i:= l_found_pos+1 until i> number_of_extremes loop
					l_new_array.put(l_current_array.item(i- 1), i)
					i:= i+1
				end
			end
		extreme_contributions.force (l_new_array, a_column_index)
		minimums.put(extreme_contributions.item(a_column_index).item(number_of_extremes).second, a_column_index)
		end

	number_of_extremes: INTEGER

	extreme_contributions: ROSE_LINEAR_ARRAY[ROSE_LINEAR_ARRAY[PAIR[INTEGER, DOUBLE]]]
		-- largest contributor in position 1, lowest in position

	minimums: VECTOR

feature -- Basic operations

feature {NONE} -- Implementation

end
