note

	description:

		"Abstraction of an investment zone ... built from the ROSE_CONTEXT version of PERSISTENT_INVESTMENT_REGION"

	author: "Mark Howard"
	last_editor: "$Author: e2 $"
	date: "$Date: 2015/02/13 $"
	revision: "$Revision: #178 $"
	path: "$File: //rose/source/kernel/investment_entities/zone.e $"

class ZONE [G -> LOCATION]

inherit

	DEBUG_OUTPUT
		undefine
			is_equal
		end

	NEEDS_HANDLE
		redefine
			is_equal
		end

	HASHABLE
		undefine
			is_equal
		end

	ROSE_INVESTMENT_ENTITIES
		undefine
			is_equal
		end

	ROSE_BUSINESS_DATE
		undefine
			is_equal
		end

	PES_AWARE_CONTEXT
		undefine
			is_equal
		end

	EXCEPTIONS
		export
			{NONE} all
		undefine
			is_equal
		end

	ATTRIBUTES
		rename
			msci as attributes_msci,
			company_name as attributes_company_name,
			cusip as attributes_cusip,
			sedol as attributes_sedol,
			mnemonic as attributes_mnemonic
		undefine
			is_equal
		end

	PM_METRIC_SOURCE
		undefine
			is_equal
		redefine
			metrics
		end

	PM_AWARE
		undefine
			is_equal
		end

create

	make,
	make_index,
	make_future,
	make_world

feature {NONE} -- Creation

	make (a_context: like context; a_investment_region: like context.investment_region_template) is
			--create from investment region for this context
		require
			context_not_void: a_context /= Void
		do
			context := a_context
			if a_investment_region /= Void then
				mnemonic := a_investment_region.mnemonic
				investment_region := a_investment_region
				index := a_investment_region.index
				handle := a_investment_region.handle
			end
			if is_multi_zonal then
				create companies.make (0)
				create securities.make (0)
				create listings.make (0)
				create government_issuers.make (0)
				create government_issues.make (0)
				create government_issue_listings.make (0)
				create bonds.make (0)
				create bond_listings.make (0)
				create futures.make (0)
				create future_listings.make (0)
				create funds.make (0)
				create fund_issues.make (0)
				create fund_listings.make (0)
				create market_indexes.make (0)
				create market_index_listings.make (0)
				create xrf_entity_id_table.make (10)
			else
				create companies.make (max_company_xrf)
				create securities.make (max_security_xrf)
				create listings.make (max_listing_xrf)
				create government_issuers.make (max_company_xrf)
				create government_issues.make (max_security_xrf)
				create government_issue_listings.make (max_listing_xrf)
				create bonds.make (max_security_xrf)
				create bond_listings.make (max_listing_xrf)
				create futures.make (max_security_xrf)
				create future_listings.make (max_listing_xrf)
				create funds.make (max_security_xrf)
				create fund_issues.make (max_security_xrf)
				create fund_listings.make (max_listing_xrf)
				create market_indexes.make (max_security_xrf)
				create market_index_listings.make (max_listing_xrf)
				create xrf_entity_id_table.make (10)
			end
			create all_loaded_table.make (100)
			windows_is_market_data_source := True
		end

	make_future (a_context: like context) is
			-- Make as a zone for futures
		require
			context_not_void: a_context /= Void
		do
			make (a_context, Void)
			mnemonic := 'F'
			zonal_market_data.install (create {MARKET_DATA_F}.make (Current), Current)
			investment_region := context.investment_region_at (attributes_mnemonic ("T"), Void)
		ensure
			investment_region_not_void: investment_region /= Void
		end

	make_index (a_context: like context) is
			-- Make as a zone for indexes
		require
			context_not_void: a_context /= Void
		do
			make (a_context, Void)
			mnemonic := 'I'
			zonal_market_data.install (create {MARKET_DATA_I}.make (Current), Current)
		end

	make_world(a_context: like context)
			-- Make the W zone.
		require
			context_not_void: a_context /= Void
		do
			make(a_context, Void)
			mnemonic := 'W'
			-- Should we set 'investment_region'?  (See 'make_future').
		end

feature -- Access

	context: ROSE_CONTEXT
			--context for this zone

	handle: INTEGER
			-- Same as `entity_id'.

	investment_region: like context.investment_region_template

	windows_is_market_data_source: BOOLEAN

	use_persistent_server_as_market_data_source: BOOLEAN

	mnemonic: CHARACTER
			-- E.g. 'E' for Europe

	index: INTEGER

	hash_code: INTEGER is
			--hash code
		do
			Result := mnemonic.hash_code
		end

	is_equal (a_other: like Current): BOOLEAN is
			--if two zones have the same handle then we consider them to be equal
		do
			Result := same_zone (a_other)
		end

	same_zone (a_other: ZONE [LOCATION]): BOOLEAN is
			-- Same zone?
			-- (wider typed version of is_equal)
		require
			a_other_not_void: a_other /= Void
		do
			Result := handle = a_other.handle
				-- FIXME context should be included?
		end

	components: ARRAY [like context.zone_template]
			-- If `Current' is multi-zonal, then its components;
			-- Otherwise just `Current'
		do
			if is_multi_zonal then
				Result := <<c_zone, e_zone, j_zone, o_zone, s_zone, u_zone>>
			else
				Result := <<Current>>
			end
		ensure
			components_not_void: Result /= Void
			components_not_empty: not Result.is_empty
			no_void_component: not Result.has (Void)
			no_multi_zonal: not Result.there_exists (agent {like context.zone_template}.is_multi_zonal)
		end

	cap_sector_count: INTEGER
			-- How many cap sectors we have in the zone
		do
			if is_c_zone or is_s_zone then
				Result := 4
			else
				Result := 5
			end
		ensure
			definition_c_and_s: (is_c_zone or is_s_zone) = (Result = 4)
			definition_other: not (is_c_zone or is_s_zone) = (Result = 5)
		end

feature -- Status setting

	set_use_persistent_server_as_market_data_source (a_value: BOOLEAN) is
			-- Set 'use_persistent_server_as_market_data_source' to 'a_value'
		do
			use_persistent_server_as_market_data_source := a_value
		end

	set_market_data (a_market_data: like market_data) is
			--set zone specific market data
		do
			market_data_table.search (Current)
			if not market_data_table.found then
				market_data_table.force (a_market_data, Current)
			end
		end

feature -- Access

	market_data_table: HASH_TABLE [MARKET_DATA, ZONE [LOCATION]] is
			-- Market data table
		once
			create Result.make (10)
		ensure
			market_data_table_not_void: Result /= Void
		end

	company (i: INTEGER): detachable like context.company_template is
			-- i-th company
		require
			not_multi_zonal: not is_multi_zonal
		local
			l_listings: ROSE_LINEAR_ARRAY [like context.listing_template]
			l_listing, l_dummy_listing: like context.listing_template
			l_triple: KEY_TRIPLE[INTEGER, CHARACTER, INTEGER]
		do
			companies.search (i)
			if companies.found then
				Result := companies.found_item
			else
				if i < 0 then
					Result := companies.item (i)
					if Result = Void then
						l_listings := context.listing_accessor.entity_list_from_identifier (i.out, Void, fcxrf)
						from
							l_listings.start
						until
							l_listings.off or else Result /= Void
						loop
							l_listing := l_listings.current_item
							if l_listing.zone_mnemonic = mnemonic then
								Result := l_listing.surrogated_company
							end
							l_listings.forth
						end
					end
						-- Check whether security has dummy listing in this zone
					if Result = Void then
						inspect mnemonic.as_upper
						when 'O' then
							l_dummy_listing := context.listing_at (local_fcxrf (i.out), Void)
								-- this is for O zone
						when 'S' then
							l_dummy_listing := context.listing_at (S_FCXRF (i.out), Void)
						when 'E' then
							l_dummy_listing := context.listing_at (E_FCXRF (i.out), Void)
--						when 'J' then
--							l_dummy_listing := context.listing_from_j_fsxrf (i.out, void)
						else
						end
						if l_dummy_listing /= Void and then l_dummy_listing.security /= Void and then l_dummy_listing.security.company /= Void then
							create Result.make_surrogate (l_dummy_listing.security.company, Current)
							Result.set_xrf_code (i)
							companies.put (Result, i)
						end
					end
				else
					Result := ur_company.entity_from_zone_xrf (Current, i)
				end
				if Result = Void then
					companies.put (Void, i)
				else
					add_company (Result)
				end

				create l_triple.make(context.context_index, mnemonic, ids.company_category_id)
				entity_misses[l_triple] := entity_misses[l_triple] + 1
			end
		ensure
			in_zone: Result /= Void implies same_zone (Result.zone)
		end

	security (i: INTEGER): detachable like context.security_template is
			--security by sxrf
		require
			not_multi_zonal: not is_multi_zonal
		local
			l_listings: ROSE_LINEAR_ARRAY [like context.listing_template]
			l_listing, l_dummy_listing: like context.listing_template
			l_triple: KEY_TRIPLE[INTEGER, CHARACTER, INTEGER]
		do
			securities.search (i)
			if securities.found then
				Result := securities.found_item
			else
				if i < 0 then
					l_listings := context.listing_accessor.entity_list_from_identifier (i.out, Void, fsxrf)
					from
						l_listings.start
					until
						l_listings.off or else Result /= Void
					loop
						l_listing := l_listings.current_item
						if l_listing.zone_mnemonic = mnemonic then
							Result := l_listing.surrogated_security
						end
						l_listings.forth
					end
						-- Check whether security has dummy listing in this zone
					if Result = Void then
						inspect mnemonic.as_upper
						when 'O' then
							l_dummy_listing := context.listing_at (local_fsxrf (i.out), Void)
								-- must guarantee fsxrf be unique cross-zonally.
						when 'S' then
							l_dummy_listing := context.listing_at (s_fsxrf (i.out), Void)
						when 'E' then
							l_dummy_listing := context.listing_at (e_fsxrf (i.out), Void)
--						when 'J' then
--							l_dummy_listing := context.listing_from_j_fsxrf (i.out, void)
						else
						end
--						if l_dummy_listing=void then
--							l_dummy_listing := context.listing_from_fsxrf_code(i.out, void)
--						end
						if l_dummy_listing /= Void and then l_dummy_listing.security /= Void then
								-- and then l_dummy_listing.is_dummy_listing_of (mnemonic)then
							create Result.make_surrogate (l_dummy_listing.security, Current)
						end
					end
				else
					Result := ur_security.entity_from_zone_xrf (Current, i)
				end
				securities.put (Result, i)

				create l_triple.make(context.context_index, mnemonic, ids.security_category_id)
				entity_misses[l_triple] := entity_misses[l_triple] + 1
			end
		end

	listing (i: INTEGER): detachable like context.listing_template is
			-- Listing in this zone with xrf code `i'
		require
			not_multi_zonal: not is_multi_zonal
		local
			l_triple: KEY_TRIPLE[INTEGER, CHARACTER, INTEGER]
		do
			listings.search (i)
			if listings.found then
				Result := listings.found_item
			else
				dummy_listings.do_nothing		-- Load all dummy listings into 'listings'.
				listings.search(i)

				if listings.found
				then
					Result := listings.found_item
				else
				Result := ur_listing.entity_from_zone_xrf (Current, i)
				listings.put (Result, i)

				create l_triple.make(context.context_index, mnemonic, ids.listing_category_id)
				entity_misses[l_triple] := entity_misses[l_triple] + 1
			end
		end
		end

	government_issuer (i: INTEGER): detachable like context.government_issuer_template is
			--Government issuer with xrf code i in this zone
		require
			not_multi_zonal: not is_multi_zonal
		do
			government_issuers.search (i)
			if government_issuers.found then
				Result := government_issuers.found_item
			else
				Result := ur_government_issuer.entity_from_zone_xrf (Current, i)
				government_issuers.put (Result, i)
			end
		end

	government_issue (i: INTEGER): detachable like context.government_issue_template is
			--Government issue with xrf code i in this zone
		require
			not_multi_zonal: not is_multi_zonal
		do
			government_issues.search (i)
			if government_issues.found then
				Result := government_issues.found_item
			else
				Result := ur_government_issue.entity_from_zone_xrf (Current, i)
				government_issues.put (Result, i)
			end
		end

	government_issue_listing (i: INTEGER): detachable like context.government_issue_listing_template is
			--Government issue listing with xrf code i in this zone
		require
			not_multi_zonal: not is_multi_zonal
		do
			government_issue_listings.search (i)
			if government_issue_listings.found then
				Result := government_issue_listings.found_item
			else
				Result := ur_government_issue_listing.entity_from_zone_xrf (Current, i)
				government_issue_listings.put (Result, i)
			end
		end

	bond (i: INTEGER): detachable like context.bond_template is
		require
			not_multi_zonal: not is_multi_zonal
		do
			bonds.search (i)
			if bonds.found then
				Result := bonds.found_item
			else
				Result := ur_bond.entity_from_zone_xrf (Current, i)
				bonds.put (Result, i)
			end
		end

	bond_listing (i: INTEGER): detachable like context.bond_listing_template
			--Bond listing by xrf code
		require
			not_multi_zonal: not is_multi_zonal
		do
			bond_listings.search (i)
			if bond_listings.found then
				Result := bond_listings.found_item
			else
				Result := ur_bond_listing.entity_from_zone_xrf (Current, i)
				bond_listings.put (Result, i)
			end
		end

	fund (i: INTEGER): detachable like context.fund_template is
		require
			not_multi_zonal: not is_multi_zonal
		do
			funds.search (i)
			if funds.found then
				Result := funds.found_item
			else
				Result := ur_fund.entity_from_zone_xrf (Current, i)
				funds.put (Result, i)
			end
		end

	fund_issue (i: INTEGER): detachable like context.fund_issue_template is
		require
			not_multi_zonal: not is_multi_zonal
		do
			fund_issues.search (i)
			if fund_issues.found then
				Result := fund_issues.found_item
			else
				Result := ur_fund_issue.entity_from_zone_xrf (Current, i)
				fund_issues.put (Result, i)
			end
		end

	fund_listing (i: INTEGER): detachable like context.fund_listing_template is
		require
			not_multi_zonal: not is_multi_zonal
		do
			fund_listings.search (i)
			if fund_listings.found then
				Result := fund_listings.found_item
			else
				Result := ur_fund_listing.entity_from_zone_xrf (Current, i)
				fund_listings.put (Result, i)
			end
		end

	future (i: INTEGER): detachable like context.future_template is
		require
			not_multi_zonal: not is_multi_zonal
		do
			futures.search (i)
			if futures.found then
				Result := futures.found_item
			else
				Result := ur_future.entity_from_zone_xrf (Current, i)
				futures.put (Result, i)
			end
		end

	future_listing (i: INTEGER): detachable like context.future_listing_template is
		require
			not_multi_zonal: not is_multi_zonal
		do
			future_listings.search (i)
			if future_listings.found then
				Result := future_listings.found_item
			else
				Result := ur_future_listing.entity_from_zone_xrf (Current, i)
				future_listings.put (Result, i)
			end
		end

	market_index (i: INTEGER): detachable like context.market_index_template is
			-- return the market index corresponding to i
		require
			not_multi_zonal: not is_multi_zonal
		do
			market_indexes.search (i)
			if market_indexes.found then
				Result := market_indexes.found_item
			else
				Result := ur_market_index.entity_from_zone_xrf (Current, i)
				market_indexes.put (Result, i)
			end
		end

	market_index_listing (i: INTEGER): detachable like context.market_index_listing_template is
		require
			not_multi_zonal: not is_multi_zonal
		do
			market_index_listings.search (i)
			if market_index_listings.found then
				Result := market_index_listings.found_item
			else
				Result := ur_market_index_listing.entity_from_zone_xrf (Current, i)
				market_index_listings.put (Result, i)
			end
		end

	country (i: INTEGER): detachable like context.country_template is
		do
			if countries.valid_index (i) then
				Result := countries.item (i)
			end
		end

	listing_from_ticker (a_key: STRING; a_date: ROSE_DATE): detachable like context.listing_template is
			-- Returns the listing with ticker equal to 'a_key' in this ZONE.  Since tickers get reused, we check the latest price when
			-- duplicates occur.
		local
			l_listings: ROSE_ENTITY_HISTORY [like context.listing_template]
			l_results: ROSE_LINEAR_ARRAY[like context.listing_template]
			l_zone: STRING
			l_price_record: PRICE_RECORD
			l_last_date: ROSE_DATE
		do
			create l_results.make_empty
			l_listings := context.listing_accessor.entity_history_from_identifier (a_key, ticker)

			-- Find all listings with 'a_key' in 'a_zone'.
			from
				l_listings.start
			until
				l_listings.off or Result /= Void
			loop
				if l_listings.current_item.brackets (a_date) then
					if l_listings.current_item.entity /= Void then
						l_zone := l_listings.current_item.entity.zone_code_key (a_date)
						if l_zone /= Void and then not l_zone.is_empty and then l_zone.item (1) = mnemonic then
							l_results.extend(l_listings.current_item.entity)
						end
					end
				end
				l_listings.forth
			end

			-- If there are multiple results, return the one with the latest price.
			if l_results.count = 1
			then
				Result := l_results[1]
			elseif l_results.count > 1
			then
				across
					l_results as c_results
				loop
					l_price_record := c_results.item.last_price_record(a_date)

					if l_price_record /= Void and then (l_last_date = Void or else l_price_record.date > l_last_date)
					then
						l_last_date := l_price_record.date
						Result := c_results.item
					end
				end
			end
		end

	listing_from_riemsym_code (a_key: STRING; a_date: ROSE_DATE): detachable like context.listing_template is
		local
			l_listings: ROSE_ENTITY_HISTORY [like context.listing_template]
			l_zone: STRING
		do
			l_listings := context.listing_accessor.entity_history_from_identifier (a_key, riemsym)
			from
				l_listings.start
			until
				l_listings.off or Result /= Void
			loop
				if l_listings.current_item.brackets (a_date) then
					l_zone := l_listings.current_item.entity.zone_code_key (a_date)
					if l_zone /= Void and then not l_zone.is_empty and then l_zone.item (1) = mnemonic then
						Result := l_listings.current_item.entity
					end
				end
				l_listings.forth
			end
		end

	instrument_from_riemsym_code (a_key: STRING; a_date: ROSE_DATE): detachable LEGACY_INSTRUMENT is
		local
			l_instruments: ROSE_LINEAR_ARRAY [LEGACY_INSTRUMENT]
			l_instrument: LEGACY_INSTRUMENT
			i: INTEGER
		do
			l_instruments := context.instruments_from_riemsym_code (a_key, a_date)
			from
				i := 1
			until
				i > l_instruments.count or else Result /= Void
			loop
				l_instrument := l_instruments.item (i)
				if l_instrument /= Void and then l_instrument.zone = Current then
					Result := l_instrument
				end
				i := i + 1
			end
		end

	companies_by_xrfs (a_xrfs: ARRAY [INTEGER]): ROSE_LINEAR_ARRAY [like context.company_template] is
		require
			not_multi_zonal: not is_multi_zonal
		local
			i, l_xrf: INTEGER
			l_company: like context.company_template
		do
			create Result.make_empty
			from
				i := a_xrfs.lower
			until
				i > a_xrfs.upper
			loop
				l_xrf := a_xrfs.item (i)
				if valid_company_xrf (l_xrf) then
					l_company := company (l_xrf)
					Result.extend (l_company)
				end
				i := i + 1
			end
		end

feature -- Limits

	valid_company_xrf (a_xrf: INTEGER): BOOLEAN is
			-- Is `a_xrf' a meaningful value?
		require
			not_multi_zonal: not is_multi_zonal
		do
			Result := a_xrf >= min_company_xrf and a_xrf <= max_company_xrf
		end

	min_company_xrf: INTEGER is
		require
			not_multi_zonal: not is_multi_zonal
		do
			if min_cxrf_saved = 0 then
				min_cxrf_saved := context.min_company_xrf (Current)
				if min_cxrf_saved = constants.null_integer then
					min_cxrf_saved := 0
				end
				min_cxrf_saved := min_cxrf_saved - 500
					--to account for dummy listings
			end
			Result := min_cxrf_saved
		end

	max_category_xrf (a_category_id: INTEGER): INTEGER is
		require
			not_multi_zonal: not is_multi_zonal
		do
			Result := context.max_category_xrf (mnemonic, a_category_id)
		end

	max_company_xrf: INTEGER
			-- Highest cxrf in this zone.
		require
			not_multi_zonal: not is_multi_zonal
		do
			Result := max_category_xrf (ids.company_category_id)
		end

	max_security_xrf: INTEGER is
		require
			not_multi_zonal: not is_multi_zonal
		do
			Result := max_category_xrf (ids.security_category_id)
		end

	max_listing_xrf: INTEGER is
		require
			not_multi_zonal: not is_multi_zonal
		do
			Result := max_category_xrf (ids.listing_category_id)
		end

	max_market_index_xrf: INTEGER is
		require
			not_multi_zonal: not is_multi_zonal
		do
			Result := max_category_xrf (ids.market_index_category_id)
		end

	max_future_xrf: INTEGER is
		require
			not_multi_zonal: not is_multi_zonal
		do
			Result := max_category_xrf (ids.future_category_id)
		end

feature -- Extend

	add_company (a_company: like context.company_template)
			-- add `a_company' to `companies' by xrf_code
		require
			not_multi_zonal: not is_multi_zonal
			a_company_not_void: a_company /= Void
		do
			if a_company.xrf_code /= 0 then
			companies.put (a_company, a_company.xrf_code)
		end
		ensure
			company: a_company.xrf_code = 0 or else a_company.is_equal (companies.item (a_company.xrf_code))
		end

	add_security (a_security: like context.security_template)
			-- Add `a_security' to the `securities' by xrf_code
		require
			not_multi_zonal: not is_multi_zonal
			security_not_void: a_security /= Void
		do
			if a_security.xrf_code /= 0 then
			securities.put (a_security, a_security.xrf_code)
			end
		ensure
			security: a_security.xrf_code = 0 or else a_security.is_equal (securities.item (a_security.xrf_code))
		end

	add_listing (a_listing: like context.listing_template)
			-- Add `a_listing' to the `listings' by xrf_code
		require
			not_multi_zonal: not is_multi_zonal
			listing_not_void: a_listing /= Void
		do
			if a_listing.xrf_code /= 0 then
			listings.put (a_listing, a_listing.xrf_code)
			end
		ensure
			listing: a_listing.xrf_code = 0 or else a_listing.is_equal (listings.item (a_listing.xrf_code))
		end

	add_government_issuer (a_issuer: like context.government_issuer_template)
			-- Add `a_issuer' to the `government_issuers' by xrf_code
		require
			not_multi_zonal: not is_multi_zonal
			a_issuer_not_void: a_issuer /= Void
		do
			if a_issuer.xrf_code /= 0 then
			government_issuers.put (a_issuer, a_issuer.xrf_code)
		end
		ensure
			government_issuer: a_issuer.xrf_code = 0 or else a_issuer.is_equal (government_issuers.item (a_issuer.xrf_code))
		end

	add_government_issue (a_issue: like context.government_issue_template)
			-- Add `a_issue' to the `government_issues' by xrf_code
		require
			not_multi_zonal: not is_multi_zonal
			a_issue_not_void: a_issue /= Void
		do
			if a_issue.xrf_code /= 0 then
			government_issues.put (a_issue, a_issue.xrf_code)
		end
		ensure
			government_issue: a_issue.xrf_code = 0 or else a_issue.is_equal (government_issues.item (a_issue.xrf_code))
		end

	add_government_issue_listing (a_listing: like context.government_issue_listing_template)
			-- Add `a_listing' to the `government_issue_listings' by xrf_code
		require
			not_multi_zonal: not is_multi_zonal
			a_listing_not_void: a_listing /= Void
		do
			if a_listing.xrf_code /= 0 then
			government_issue_listings.put (a_listing, a_listing.xrf_code)
		end
		ensure
			government_issue_listing: a_listing.xrf_code = 0 or else a_listing.is_equal (government_issue_listings.item (a_listing.xrf_code))
		end

	add_bond (a_bond: like context.bond_template)
			-- Add `a_bond' to the `bonds' by xrf_code
		require
			not_multi_zonal: not is_multi_zonal
			a_bond_not_void: a_bond /= Void
		do
			if a_bond.xrf_code /= 0 then
				bonds.put (a_bond, a_bond.xrf_code)
		end
		ensure
			bond: a_bond.xrf_code = 0 or else a_bond.is_equal (bonds.item (a_bond.xrf_code))
		end

	add_bond_listing (a_listing: like context.bond_listing_template)
			-- Add `a_listing' to the `bond_listings' by xrf_code
		require
			not_multi_zonal: not is_multi_zonal
			a_listing_not_void: a_listing /= Void
		do
			if a_listing.xrf_code /= 0 then
			bond_listings.put (a_listing, a_listing.xrf_code)
		end
		ensure
			bond_listing: a_listing.xrf_code = 0 or else a_listing.is_equal (bond_listings.item (a_listing.xrf_code))
		end

	add_fund (a_fund: like context.fund_template)
			-- Add `a_fund' to the `funds' by xrf_code
		require
			not_multi_zonal: not is_multi_zonal
			a_fund_not_void: a_fund /= Void
		do
			if a_fund.xrf_code /= 0 then
				funds.put (a_fund, a_fund.xrf_code)
		end
		ensure
			fund: a_fund.xrf_code = 0 or else a_fund.is_equal (funds.item (a_fund.xrf_code))
		end

	add_fund_issue (a_issue: like context.fund_issue_template)
			-- Add `a_issue' to the `fund_issues' by xrf_code
		require
			not_multi_zonal: not is_multi_zonal
			a_issue_not_void: a_issue /= Void
		do
			if a_issue.xrf_code /= 0 then
			fund_issues.put (a_issue, a_issue.xrf_code)
		end
		ensure
			fund_issue: a_issue.xrf_code = 0 or else a_issue.is_equal (fund_issues.item (a_issue.xrf_code))
		end

	add_fund_listing (a_listing: like context.fund_listing_template)
			-- Add `a_listing' to the `fund_listings' by xrf_code
		require
			not_multi_zonal: not is_multi_zonal
			a_listing_not_void: a_listing /= Void
		do
			if a_listing.xrf_code /= 0 then
			fund_listings.put (a_listing, a_listing.xrf_code)
		end
		ensure
			fund_listing: a_listing.xrf_code = 0 or else a_listing.is_equal (fund_listings.item (a_listing.xrf_code))
		end

	add_future (a_issue: like context.future_template)
			-- Add `a_issue' to the `futures' by xrf_code
		require
			not_multi_zonal: not is_multi_zonal
			a_issue_not_void: a_issue /= Void
		do
			if a_issue.xrf_code /= 0 then
			futures.put (a_issue, a_issue.xrf_code)
		end
		ensure
			future: a_issue.xrf_code = 0 or else a_issue.is_equal (futures.item (a_issue.xrf_code))
		end

	add_future_listing (a_listing: like context.future_listing_template)
			-- Add `a_listing' to the `future_listings' by xrf_code
		require
			not_multi_zonal: not is_multi_zonal
			a_listing_not_void: a_listing /= Void
		do
			if a_listing.xrf_code /= 0 then
			future_listings.put (a_listing, a_listing.xrf_code)
		end
		ensure
			future_listing: a_listing.xrf_code = 0 or else a_listing.is_equal (future_listings.item (a_listing.xrf_code))
		end

	add_market_index (a_index: like context.market_index_template)
			-- Add `a_index' to the `market_indexes' by xrf_code
		require
			not_multi_zonal: not is_multi_zonal
			a_index_not_void: a_index /= Void
		do
			if a_index.xrf_code /= 0 then
				market_indexes.put (a_index, a_index.xrf_code)
		end
		ensure
			market_indexe: a_index.xrf_code = 0 or else a_index.is_equal (market_indexes.item (a_index.xrf_code))
		end

	add_market_index_listing (a_listing: like context.market_index_listing_template)
			-- Add `a_listing' to the `market_index_listings' by xrf_code
		require
			not_multi_zonal: not is_multi_zonal
			a_listing_not_void: a_listing /= Void
		do
			if a_listing.xrf_code /= 0 then
			market_index_listings.put (a_listing, a_listing.xrf_code)
		end
		ensure
			market_index_listing: a_listing.xrf_code = 0 or else a_listing.is_equal (market_index_listings.item (a_listing.xrf_code))
		end

	usd_exchange_rate (l_date: ROSE_DATE): REAL is
		do
			inspect mnemonic
			when 'U' then
				Result := 1
			else
					--need to get data
				Result := constants.legacy_missing_real_32
			end
		end

	market_data_error: INTEGER is
		do
			Result := market_data.error
		end

feature --Status

	has_market_date (a_date: ROSE_DATE): BOOLEAN is
		do
			Result := a_date /= Void and then market_dates.has (a_date)
		end

	market_dates: ROSE_DATE_SET is
		local
			l_bitmap: ROSE_BITMAP
			l_countries_with_market_data: like countries_with_market_data
		do
			Result := market_dates_table.item (Current)
			if Result = Void then
				create l_bitmap.make_empty (36500)
				if is_multi_zonal then
					across components as c_components loop
						l_bitmap.or_with (c_components.item.market_dates.membership)
					end
				else
					l_countries_with_market_data := countries_with_market_data
					across countries as c_countries loop
						if l_countries_with_market_data.has (c_countries.item.code) then
							l_bitmap.or_with (c_countries.item.market_date_bitmap)
						end
					end
				end
				create Result.make (l_bitmap, bitmap_start_date)
				market_dates_table.put (Result, Current)
			end
		end

	market_date_before_today: ROSE_DATE is
		do
			create Result.make_today
			Result.add_n_days (-1)
		ensure
			market_date_before_today_not_void: Result /= Void
		end

	partial_month_proportion (a_date: ROSE_DATE): DOUBLE is
			--proportion for smoothing out MTD variable in MONDAT
		require
			a_date_not_void: a_date /= Void
		local
			l_dates: ROSE_LINEAR_ARRAY [ROSE_DATE]
			l_start_date, l_date: ROSE_DATE
			i: INTEGER
		do
			if a_date.is_end_of_month then
				Result := 1.0
			else
				l_start_date := a_date.month_beginning
				create l_dates.make_from_linear (market_day_interval (l_start_date, a_date.rose_month.last))
				l_dates.compare_objects
				if l_dates.has (a_date) then
					i := l_dates.index_of (a_date, 1)
					l_date := a_date
				else
					i := l_dates.index_of (l_date, 1)
				end
				Result := i / (l_dates.count)
			end
		end

feature -- Zone determination

	is_u_zone: BOOLEAN
			-- Is current zone the U zone?
		do
			Result := mnemonic = 'U'
		ensure
			definition: Result = (mnemonic = 'U')
			not_multi_zonal: Result implies not is_multi_zonal
		end

	is_e_zone: BOOLEAN
			-- Is current zone the E zone?
		do
			Result := mnemonic = 'E'
		ensure
			definition: Result = (mnemonic = 'E')
			not_multi_zonal: Result implies not is_multi_zonal
		end

	is_c_zone: BOOLEAN
			-- Is current zone the C zone?
		do
			Result := mnemonic = 'C'
		ensure
			definition: Result = (mnemonic = 'C')
			not_multi_zonal: Result implies not is_multi_zonal
		end

	is_j_zone: BOOLEAN
			-- Is current zone the J zone?
		do
			Result := mnemonic = 'J'
		ensure
			definition: Result = (mnemonic = 'J')
			not_multi_zonal: Result implies not is_multi_zonal
		end

	is_o_zone: BOOLEAN
			-- Is current zone the O zone?
		do
			Result := mnemonic = 'O'
		ensure
			definition: Result = (mnemonic = 'O')
			not_multi_zonal: Result implies not is_multi_zonal
		end

	is_s_zone: BOOLEAN
			-- Is current zone the S zone?
		do
			Result := mnemonic = 'S'
		ensure
			definition: Result = (mnemonic = 'S')
			not_multi_zonal: Result implies not is_multi_zonal
		end

	is_t_zone: BOOLEAN
			-- Is current zone the T zone?
		do
			Result := mnemonic = 'T'
		ensure
			definition: Result = (mnemonic = 'T')
			is_global: Result implies is_global
			is_multi_zonal: Result implies is_multi_zonal
		end

	is_w_zone: BOOLEAN
			-- Is current zone the W zone?
		do
			Result := mnemonic = 'W'
		ensure
			definition: Result = (mnemonic = 'W')
			is_global: Result implies is_global
			is_multi_zonal: Result implies is_multi_zonal
		end

	is_multi_zonal: BOOLEAN
			-- Does current zone cover several zones?
		do
			Result := mnemonic = 'T' or mnemonic = 'W'
		end

	is_global: BOOLEAN
			-- Does current zone cover all zones?
		do
			Result := mnemonic = 'T' or mnemonic = 'W' or mnemonic = 'I' or mnemonic = 'F'
		end

	is_multi_country_zone: BOOLEAN is
			-- is multi country zone
		do
			Result := is_e_zone or is_o_zone or is_s_zone or is_multi_zonal or is_global
		end

feature -- Performance Issues with having HDP handle this request

	countries_with_market_data: ROSE_LINEAR_ARRAY [STRING]
		do
			create Result.make_empty
			Result.compare_objects
			if mnemonic = 'U' then
				Result.extend ("US")
			elseif mnemonic = 'C' then
				Result.extend ("CA")
			elseif mnemonic = 'J' then
				Result.extend ("JP")
			elseif is_multi_zonal then
				across components as c_components loop
					Result.append_array (c_components.item.countries_with_market_data)
				end
			else
				countries_with_market_data_predicate.retrieve (Void, <<mnemonic.out>>)
				across countries_with_market_data_predicate.array as c_countries loop
					if attached c_countries.item as l_country then
						Result.extend (l_country)
					end
				end
			end
		ensure
			countries_with_market_data_not_void: Result /= Void
		end

feature {NONE} -- Implementation

	countries_with_market_data_predicate: STRING_PREDICATE is
		require
			server_not_void: servers.market_data_server /= Void
		once
			create Result.make (servers.market_data_server, "select distinct(iso) from iso_by_rxrf where zone = ?")
		end

	pes_context_id: INTEGER is
			-- Context ID, (for PES)
		do
			Result := context.context_index
		end

feature -- Bank and Market

	bank_dates: ROSE_DATE_SET is
		local
			l_bitmap: ROSE_BITMAP
		do
			Result := bank_dates_table.item (Current)
			if Result = Void then
				create l_bitmap.make_empty (36500)
				across countries as c_countries loop
					if c_countries.item.investable then
						l_bitmap.or_with (c_countries.item.bank_date_bitmap)
					end
				end
				create Result.make (l_bitmap, bitmap_start_date)
				bank_dates_table.put (Result, Current)
			end
		end

	is_using_monthly_data_test_env: BOOLEAN

	set_is_using_monthly_data_test_env (a_value: BOOLEAN) is
		do
			is_using_monthly_data_test_env := a_value
		end

	market_data: MARKET_DATA is
		do
			Result := zonal_market_data.item (Current)
		end

feature -- Cacheing

	clear_cache is
			-- wipe out all containers
		do
			all_companies_loaded := False
			all_securities_loaded := False
			all_listings_loaded := False
			all_government_issuers_loaded := False
			all_government_issues_loaded := False
			all_government_issue_listings_loaded := False
			all_bonds_loaded := False
			all_bond_listings_loaded := False
			all_funds_loaded := False
			all_fund_issues_loaded := False
			all_fund_listings_loaded := False
			all_futures_loaded := False
			all_future_listings_loaded := False
			all_market_indexes_loaded := False
			all_market_index_listings_loaded := False
			companies.wipe_out
			securities.wipe_out
			listings.wipe_out
			government_issuers.wipe_out
			government_issues.wipe_out
			government_issue_listings.wipe_out
			bonds.wipe_out
			bond_listings.wipe_out
			futures.wipe_out
			future_listings.wipe_out
			funds.wipe_out
			fund_issues.wipe_out
			fund_listings.wipe_out
			market_indexes.wipe_out
			market_index_listings.wipe_out
		end

	load_all_entities is
			-- Call all load_all_* features.
		do
			load_all_companies
			load_all_securities
			load_all_listings
			load_all_government_issuers
			load_all_government_issues
			load_all_government_issue_listings
			load_all_bonds
			load_all_bond_listings
			load_all_funds
			load_all_fund_issues
			load_all_fund_listings
			load_all_futures
			load_all_future_listings
			load_all_market_indexes
			load_all_market_index_listings
		end

	add_zonal_xrf (a_record: ZONAL_XRF_RECORD) is
		require
			not_multi_zonal: not is_multi_zonal
			a_record_not_void: a_record /= Void
		do
			if investment_region /= Void then
				investment_region.add_zonal_xrf (a_record.twin)
			end
			add_xrf_entity_id (a_record)
		end

	load_all (a_handler: CATEGORY_HANDLER [like context, ENTITY]) is
			-- Load all xrfs for this zone
		require
			not_multi_zonal: not is_multi_zonal
			a_handler_not_void: a_handler /= Void
		local
			l_array: ROSE_LINEAR_ARRAY [ZONAL_XRF_RECORD]
			l_i: INTEGER
		do
			if not all_loaded (a_handler.category_id) then
				a_handler.load_all_entities
				l_array := context.zonal_xrf_records (a_handler.category_id, mnemonic)
				from
					l_i := l_array.lower
				until
					l_i > l_array.upper
				loop
					add_zonal_xrf (l_array.item (l_i))
					l_i := l_i + 1
				end
				set_all_loaded (a_handler.category_id)
			end
		end

	all_loaded (a_category_id: INTEGER): BOOLEAN is
			--
		do
			Result := all_loaded_table.item (a_category_id)
		end

	set_all_loaded (a_category_id: INTEGER) is
			--
		do
			all_loaded_table.put (True, a_category_id)
		end

	load_all_companies is
			-- Load all companies of this ZONE [LOCATION] into cache
		local
			l_company: like context.company_template
			l_security: like context.security_template
			l_listing: like context.listing_template
			l_query: PES_QUERY_ZONE_LOADER [like context.company_template]
			l_retried: BOOLEAN
			l_fxrf_array: ARRAY [TUPLE [entity_id: INTEGER; category_id: INTEGER; fxrf_code: INTEGER]]
			l_current: TUPLE [entity_id: INTEGER; category_id: INTEGER; fxrf_code: INTEGER]
			l_i: INTEGER
			l_listing_category_id: INTEGER
			l_security_category_id: INTEGER
		do
			if not all_companies_loaded then
				if is_multi_zonal then
					components.do_all (agent {like context.zone_template}.load_all_companies)
					all_companies_loaded := True
				else
					if use_pes and not l_retried then
						create l_query.make (context.context_index)
						l_query.set_mnemonic (mnemonic)
						l_query.set_companies (True)
						if attached {PES_RESULT_ZONE_LOADER [like context.company_template]} pes_api.send_general_request (l_query).get as l_result then
							if l_result.remote_exception then
								raise (l_result.tag_name)
							end
							companies := l_result.table
							all_companies_loaded := True
						end
					else
						load_all_domestic_companies
							-- Now create the surrogate entities
						l_fxrf_array := context.zonal_fxrfs (context.attribute_ids.fcxrf_code_type_id, mnemonic)
						from
							l_i := l_fxrf_array.lower
							l_listing_category_id := context.listing_accessor.category_id
							l_security_category_id := context.security_accessor.category_id
						until
							l_i > l_fxrf_array.upper
						loop
							l_current := l_fxrf_array.item (l_i)
							if l_current.category_id = l_listing_category_id then
								l_listing := context.listing_accessor.entity (l_current.entity_id)
								if l_listing.zone_mnemonic = mnemonic then
									l_company := l_listing.surrogated_company
								end
							elseif l_current.category_id = l_security_category_id then
								l_security := context.security_accessor.entity (l_current.entity_id)
								l_company := l_security.surrogated_company
							end
							if l_company /= Void and then l_company.zone_mnemonic.is_equal (mnemonic) then
								companies.force (l_company, l_current.fxrf_code)
							end
							l_i := l_i + 1
						end
						all_companies_loaded := True
					end
				end
			end
		rescue
			if not l_retried then
				l_retried := True
				retry
			end
		end

	load_all_domestic_companies is
			-- Load all companies of this ZONE[LOCATION] into cache
		require
			not_multi_zonal: not is_multi_zonal
		local
			l_company: like context.company_template
			l_retried: BOOLEAN
			l_company_category: CATEGORY_HANDLER [like context, like context.company_template]
			l_entity_ids: ARRAYED_LIST [INTEGER]
		do
			l_company_category := context.company_accessor
			load_all (l_company_category)
			if xrf_entity_id_table.has (l_company_category.category_id) then
				l_entity_ids := xrf_entity_id_table.item (l_company_category.category_id).linear_representation
				from
					l_entity_ids.start
				until
					l_entity_ids.after
				loop
					l_company := l_company_category.entity (l_entity_ids.item)
					add_company (l_company)
					l_entity_ids.forth
				end
			end
		rescue
			if not l_retried then
				l_retried := True
				retry
			end
		end

	load_all_securities is
			-- load all securities of this zone into securities
		local
			l_listing: like context.listing_template
			l_security: like context.security_template
			l_query: PES_QUERY_ZONE_LOADER [like context.security_template]
			l_retried: BOOLEAN
			l_security_category: CATEGORY_HANDLER [like context, like context.security_template]
			l_entity_ids: ARRAYED_LIST [INTEGER]
			l_fxrf_array: ARRAY [TUPLE [entity_id: INTEGER; category_id: INTEGER; fxrf_code: INTEGER]]
			l_current: TUPLE [entity_id: INTEGER; category_id: INTEGER; fxrf_code: INTEGER]
			l_i: INTEGER
			l_dummy_securities: like dummy_securities
		do
			if not all_securities_loaded then
				if is_multi_zonal then
					components.do_all (agent {like context.zone_template}.load_all_securities)
					all_securities_loaded := True
				else
					if use_pes and not l_retried then
						create l_query.make (context.context_index)
						l_query.set_mnemonic (mnemonic)
						l_query.set_securities (True)
						if attached {PES_RESULT_ZONE_LOADER [like context.security_template]} pes_api.send_general_request (l_query).get as l_result then
							if l_result.remote_exception then
								raise (l_result.tag_name)
							end
							securities := l_result.table
							all_securities_loaded := True
						end
					else
						l_security_category := context.security_accessor
						load_all (l_security_category)
						if xrf_entity_id_table.has (l_security_category.category_id) then
							l_entity_ids := xrf_entity_id_table.item (l_security_category.category_id).linear_representation
							from
								l_entity_ids.start
							until
								l_entity_ids.after
							loop
								l_security := l_security_category.entity (l_entity_ids.item)
								if l_security /= Void then
									add_security (l_security)
								end
								l_entity_ids.forth
							end
						end
						from
							l_dummy_securities := dummy_securities
							l_dummy_securities.start
						until
							l_dummy_securities.off
						loop
							if l_dummy_securities.item_for_iteration /= Void then
								add_security (l_dummy_securities.item_for_iteration)
							end
							l_dummy_securities.forth
						end
							---	 Now create the surrogate entities
						l_fxrf_array := context.zonal_fxrfs (fsxrf_identifier_id, mnemonic) + context.zonal_fxrfs (context.fsxrf_code_type_id, mnemonic)
						from
							l_i := l_fxrf_array.lower
						until
							l_i > l_fxrf_array.upper
						loop
							l_current := l_fxrf_array.item (l_i)
							l_listing := context.listing_accessor.entity (l_current.entity_id)
							if l_listing.zone_mnemonic = mnemonic then
								l_security := l_listing.surrogated_security
							end
							securities.force (l_security, l_current.fxrf_code)
							l_i := l_i + 1
						end
						all_securities_loaded := True
					end
				end
			end
		rescue
			if not l_retried then
				l_retried := True
				retry
			end
		end

	load_all_listings is
			-- Cache all zonal listings
		local
			l_listing: like context.listing_template
			l_query: PES_QUERY_ZONE_LOADER [like context.listing_template]
			l_retried: BOOLEAN
			l_listing_category: CATEGORY_HANDLER [like context, like context.listing_template]
			l_entity_ids: ARRAYED_LIST [INTEGER]
			i: INTEGER
			l_max_listing_xrf: INTEGER
			l_dummy_listing: like context.listing_template
		do
			if not all_listings_loaded then
				if is_multi_zonal then
					components.do_all (agent {like context.zone_template}.load_all_listings)
					all_listings_loaded := True
				else
					if use_pes and not l_retried then
						create l_query.make (context.context_index)
						l_query.set_mnemonic (mnemonic)
						l_query.set_listings (True)
						if attached {PES_RESULT_ZONE_LOADER [like context.listing_template]} pes_api.send_general_request (l_query).get as l_result then
							if l_result.remote_exception then
								raise (l_result.tag_name)
							end
							listings := l_result.table
							all_listings_loaded := True
						end
					else
						l_listing_category := context.listing_accessor
						load_all (l_listing_category)
						if xrf_entity_id_table.has (l_listing_category.category_id) then
							l_entity_ids := xrf_entity_id_table.item (l_listing_category.category_id).linear_representation

							-- Add the regular listings.
							from
								l_entity_ids.start
							until
								l_entity_ids.after
							loop
								l_listing := l_listing_category.entity (l_entity_ids.item)
								add_listing (l_listing)
								l_entity_ids.forth
							end

							-- Add the dummy listings.
							across
								dummy_listings as c_dummy_listings
							loop
								if c_dummy_listings.item > 0
								then
									l_dummy_listing := c_dummy_listings.key.twin
									l_dummy_listing.set_zone(Current)
									l_dummy_listing.set_xrf_code(c_dummy_listings.item)
									add_listing(l_dummy_listing)
								end
							end

							-- Fill in the missing listings (side effect of 'listing').  Note that the entity_id will be zero.
							from
								i := 1
								l_max_listing_xrf := max_listing_xrf
							until
								i > l_max_listing_xrf
							loop
								l_listing := listing (i)
								i := i + 1
							end
						end
						all_listings_loaded := True
					end
				end
			end
		rescue
			if not l_retried then
				l_retried := True
				retry
			end
		end

	load_all_government_issues is
			--cache all government issues
		local
			l_issue: like context.government_issue_template
			l_query: PES_QUERY_ZONE_LOADER [like context.government_issue_template]
			l_retried: BOOLEAN
			l_government_issue_category: CATEGORY_HANDLER [like context, like context.government_issue_template]
			l_entity_ids: ARRAYED_LIST [INTEGER]
		do
			if not all_government_issues_loaded then
				if is_multi_zonal then
					components.do_all (agent {like context.zone_template}.load_all_government_issues)
					all_government_issues_loaded := True
				else
					if use_pes and not l_retried then
						create l_query.make (context.context_index)
						l_query.set_mnemonic (mnemonic)
						l_query.set_government_issues (True)
						if attached {PES_RESULT_ZONE_LOADER [like context.government_issue_template]} pes_api.send_general_request (l_query).get as l_result then
							if l_result.remote_exception then
								raise (l_result.tag_name)
							end
							government_issues := l_result.table
							all_government_issues_loaded := True
						end
					else
						l_government_issue_category := context.government_issue_accessor
						load_all (l_government_issue_category)
						if xrf_entity_id_table.has (l_government_issue_category.category_id) then
							l_entity_ids := xrf_entity_id_table.item (l_government_issue_category.category_id).linear_representation
							from
								l_entity_ids.start
							until
								l_entity_ids.after
							loop
								l_issue := l_government_issue_category.entity (l_entity_ids.item)
								add_government_issue (l_issue)
								l_entity_ids.forth
							end
						end
						all_government_issues_loaded := True
					end
				end
			end
		rescue
			if not l_retried then
				l_retried := True
				retry
			end
		end

	load_all_government_issuers is
		local
			l_issuer: like context.government_issuer_template
			l_query: PES_QUERY_ZONE_LOADER [like context.government_issuer_template]
			l_retried: BOOLEAN
			l_government_issuer_category: CATEGORY_HANDLER [like context, like context.government_issuer_template]
			l_entity_ids: ARRAYED_LIST [INTEGER]
		do
			if not all_government_issuers_loaded then
				if is_multi_zonal then
					components.do_all (agent {like context.zone_template}.load_all_government_issuers)
					all_government_issuers_loaded := True
				else
					if use_pes and not l_retried then
						create l_query.make (context.context_index)
						l_query.set_mnemonic (mnemonic)
						l_query.set_government_issuers (True)
						if attached {PES_RESULT_ZONE_LOADER [like context.government_issuer_template]} pes_api.send_general_request (l_query).get as l_result then
							if l_result.remote_exception then
								raise (l_result.tag_name)
							end
							government_issuers := l_result.table
							all_government_issuers_loaded := True
						end
					else
						l_government_issuer_category := context.government_issuer_accessor
						load_all (l_government_issuer_category)
						if xrf_entity_id_table.has (l_government_issuer_category.category_id) then
							l_entity_ids := xrf_entity_id_table.item (l_government_issuer_category.category_id).linear_representation
							from
								l_entity_ids.start
							until
								l_entity_ids.after
							loop
								l_issuer := l_government_issuer_category.entity (l_entity_ids.item)
								add_government_issuer (l_issuer)
								l_entity_ids.forth
							end
						end
						all_government_issuers_loaded := True
					end
				end
			end
		rescue
			if not l_retried then
				l_retried := True
				retry
			end
		end

	load_all_government_issue_listings is
		local
			l_listing: like context.government_issue_listing_template
			l_query: PES_QUERY_ZONE_LOADER [like context.government_issue_listing_template]
			l_retried: BOOLEAN
			l_government_issue_listing_category: CATEGORY_HANDLER [like context, like context.government_issue_listing_template]
			l_entity_ids: ARRAYED_LIST [INTEGER]
		do
			if not all_government_issue_listings_loaded then
				if is_multi_zonal then
					components.do_all (agent {like context.zone_template}.load_all_government_issue_listings)
					all_government_issue_listings_loaded := True
				else
					if use_pes and not l_retried then
						create l_query.make (context.context_index)
						l_query.set_mnemonic (mnemonic)
						l_query.set_government_issue_listings (True)
						if attached {PES_RESULT_ZONE_LOADER [like context.government_issue_listing_template]} pes_api.send_general_request (l_query).get as l_result then
							if l_result.remote_exception then
								raise (l_result.tag_name)
							end
							government_issue_listings := l_result.table
							all_government_issue_listings_loaded := True
						end
					else
						l_government_issue_listing_category := context.government_issue_listing_accessor
						load_all (l_government_issue_listing_category)
						if xrf_entity_id_table.has (l_government_issue_listing_category.category_id) then
							l_entity_ids := xrf_entity_id_table.item (l_government_issue_listing_category.category_id).linear_representation
							from
								l_entity_ids.start
							until
								l_entity_ids.after
							loop
								l_listing := l_government_issue_listing_category.entity (l_entity_ids.item)
								add_government_issue_listing (l_listing)
								l_entity_ids.forth
							end
						end
						all_government_issue_listings_loaded := True
					end
				end
			end
		rescue
			if not l_retried then
				l_retried := True
				retry
			end
		end

	load_all_bonds is
			-- Load all bonds if not already cached.
		local
			l_issue: like context.bond_template
			l_query: PES_QUERY_ZONE_LOADER [like context.bond_template]
			l_retried: BOOLEAN
			l_bond_category: CATEGORY_HANDLER [like context, like context.bond_template]
			l_entity_ids: ARRAYED_LIST [INTEGER]
		do
			if not all_bonds_loaded then
				if is_multi_zonal then
					components.do_all (agent {like context.zone_template}.load_all_bonds)
					all_bonds_loaded := True
				else
					if use_pes and not l_retried then
						create l_query.make (context.context_index)
						l_query.set_mnemonic (mnemonic)
						l_query.set_bonds (True)
						if attached {PES_RESULT_ZONE_LOADER [like context.bond_template]} pes_api.send_general_request (l_query).get as l_result then
							if l_result.remote_exception then
								raise (l_result.tag_name)
							end
							bonds := l_result.table
							all_bonds_loaded := True
						end
					else
						l_bond_category := context.bond_accessor
						load_all (l_bond_category)
						if xrf_entity_id_table.has (l_bond_category.category_id) then
							l_entity_ids := xrf_entity_id_table.item (l_bond_category.category_id).linear_representation
							from
								l_entity_ids.start
							until
								l_entity_ids.after
							loop
								l_issue := l_bond_category.entity (l_entity_ids.item)
								add_bond (l_issue)
								l_entity_ids.forth
							end
						end
						all_bonds_loaded := True
					end
				end
			end
		rescue
			if not l_retried then
				l_retried := True
				retry
			end
		end

	load_all_bond_listings
			-- Cache all zonal bond listings.
		local
			l_listing: like context.bond_listing_template
			l_query: PES_QUERY_ZONE_LOADER [like context.bond_listing_template]
			l_retried: BOOLEAN
			l_bond_listing_category: CATEGORY_HANDLER [like context, like context.bond_listing_template]
			l_entity_ids: ARRAYED_LIST [INTEGER]
		do
			if not all_bond_listings_loaded then
				if is_multi_zonal then
					components.do_all (agent {like context.zone_template}.load_all_bond_listings)
					all_bond_listings_loaded := True
				else
					if use_pes and not l_retried then
						create l_query.make (context.context_index)
						l_query.set_mnemonic (mnemonic)
						l_query.set_bond_listing (True)
						if attached {PES_RESULT_ZONE_LOADER [like context.bond_listing_template]} pes_api.send_general_request (l_query).get as l_result then
							if l_result.remote_exception then
								raise (l_result.tag_name)
							end
							bond_listings := l_result.table
							all_bond_listings_loaded := True
						end
					else
						l_bond_listing_category := context.bond_listing_accessor
						load_all (l_bond_listing_category)
						if xrf_entity_id_table.has (l_bond_listing_category.category_id) then
							l_entity_ids := xrf_entity_id_table.item (l_bond_listing_category.category_id).linear_representation
							from
								l_entity_ids.start
							until
								l_entity_ids.after
							loop
								l_listing := l_bond_listing_category.entity (l_entity_ids.item)
								add_bond_listing (l_listing)
								l_entity_ids.forth
							end
						end
						all_bond_listings_loaded := True
					end
				end
			end
		rescue
			if not l_retried then
				l_retried := True
				retry
			end
		end

	load_all_funds is
		local
			l_issue: like context.fund_template
			l_query: PES_QUERY_ZONE_LOADER [like context.fund_template]
			l_retried: BOOLEAN
			l_fund_category: CATEGORY_HANDLER [like context, like context.fund_template]
			l_entity_ids: ARRAYED_LIST [INTEGER]
		do
			if not all_funds_loaded then
				if is_multi_zonal then
					components.do_all (agent {like context.zone_template}.load_all_funds)
					all_funds_loaded := True
				else
					if use_pes and not l_retried then
						create l_query.make (context.context_index)
						l_query.set_mnemonic (mnemonic)
						l_query.set_funds (True)
						if attached {PES_RESULT_ZONE_LOADER [like context.fund_template]} pes_api.send_general_request (l_query).get as l_result then
							if l_result.remote_exception then
								raise (l_result.tag_name)
							end
							funds := l_result.table
							all_funds_loaded := True
						end
					else
						l_fund_category := context.fund_accessor
						load_all (l_fund_category)
						if xrf_entity_id_table.has (l_fund_category.category_id) then
							l_entity_ids := xrf_entity_id_table.item (l_fund_category.category_id).linear_representation
							from
								l_entity_ids.start
							until
								l_entity_ids.after
							loop
								l_issue := l_fund_category.entity (l_entity_ids.item)
								add_fund (l_issue)
								l_entity_ids.forth
							end
						end
						all_funds_loaded := True
					end
				end
			end
		rescue
			if not l_retried then
				l_retried := True
				retry
			end
		end

	load_all_fund_issues is
		local
			l_issue: like context.fund_issue_template
			l_query: PES_QUERY_ZONE_LOADER [like context.fund_issue_template]
			l_retried: BOOLEAN
			l_fund_issue_category: CATEGORY_HANDLER [like context, like context.fund_issue_template]
			l_entity_ids: ARRAYED_LIST [INTEGER]
		do
			if not all_fund_issues_loaded then
				if is_multi_zonal then
					components.do_all (agent {like context.zone_template}.load_all_fund_issues)
					all_fund_issues_loaded := True
				else
					if use_pes and not l_retried then
						create l_query.make (context.context_index)
						l_query.set_mnemonic (mnemonic)
						l_query.set_fund_issues (True)
						if attached {PES_RESULT_ZONE_LOADER [like context.fund_issue_template]} pes_api.send_general_request (l_query).get as l_result then
							if l_result.remote_exception then
								raise (l_result.tag_name)
							end
							fund_issues := l_result.table
							all_fund_issues_loaded := True
						end
					else
						l_fund_issue_category := context.fund_issue_accessor
						load_all (l_fund_issue_category)
						if xrf_entity_id_table.has (l_fund_issue_category.category_id) then
							l_entity_ids := xrf_entity_id_table.item (l_fund_issue_category.category_id).linear_representation
							from
								l_entity_ids.start
							until
								l_entity_ids.after
							loop
								l_issue := l_fund_issue_category.entity (l_entity_ids.item)
								add_fund_issue (l_issue)
								l_entity_ids.forth
							end
						end
						all_fund_issues_loaded := True
					end
				end
			end
		rescue
			if not l_retried then
				l_retried := True
				retry
			end
		end

	load_all_fund_listings is
		local
			l_listing: like context.fund_listing_template
			l_query: PES_QUERY_ZONE_LOADER [like context.fund_listing_template]
			l_retried: BOOLEAN
			l_fund_listing_category: CATEGORY_HANDLER [like context, like context.fund_listing_template]
			l_entity_ids: ARRAYED_LIST [INTEGER]
		do
			if not all_fund_listings_loaded then
				if is_multi_zonal then
					components.do_all (agent {like context.zone_template}.load_all_fund_listings)
					all_fund_listings_loaded := True
				else
					if use_pes and not l_retried then
						create l_query.make (context.context_index)
						l_query.set_mnemonic (mnemonic)
						l_query.set_fund_listings (True)
						if attached {PES_RESULT_ZONE_LOADER [like context.fund_listing_template]} pes_api.send_general_request (l_query).get as l_result then
							if l_result.remote_exception then
								raise (l_result.tag_name)
							end
							fund_listings := l_result.table
							all_fund_listings_loaded := True
						end
					else
						l_fund_listing_category := context.fund_listing_accessor
						load_all (l_fund_listing_category)
						if xrf_entity_id_table.has (l_fund_listing_category.category_id) then
							l_entity_ids := xrf_entity_id_table.item (l_fund_listing_category.category_id).linear_representation
							from
								l_entity_ids.start
							until
								l_entity_ids.after
							loop
								l_listing := l_fund_listing_category.entity (l_entity_ids.item)
								add_fund_listing (l_listing)
								l_entity_ids.forth
							end
						end
						all_fund_listings_loaded := True
					end
				end
			end
		rescue
			if not l_retried then
				l_retried := True
				retry
			end
		end

	load_all_futures is
		local
			l_future: like context.future_template
			l_query: PES_QUERY_ZONE_LOADER [like context.future_template]
			l_retried: BOOLEAN
			l_future_category: CATEGORY_HANDLER [like context, like context.future_template]
			l_entity_ids: ARRAYED_LIST [INTEGER]
		do
			if not all_futures_loaded then
				if is_multi_zonal then
					components.do_all (agent {like context.zone_template}.load_all_futures)
					all_futures_loaded := True
				else
					if use_pes and not l_retried then
						create l_query.make (context.context_index)
						l_query.set_mnemonic (mnemonic)
						l_query.set_futures (True)
						if attached {PES_RESULT_ZONE_LOADER [like context.future_template]} pes_api.send_general_request (l_query).get as l_result then
							if l_result.remote_exception then
								raise (l_result.tag_name)
							end
							futures := l_result.table
							all_futures_loaded := True
						end
					else
						l_future_category := context.future_accessor
						load_all (l_future_category)
						if xrf_entity_id_table.has (l_future_category.category_id) then
							l_entity_ids := xrf_entity_id_table.item (l_future_category.category_id).linear_representation
							from
								l_entity_ids.start
							until
								l_entity_ids.after
							loop
								l_future := l_future_category.entity (l_entity_ids.item)
								add_future (l_future)
								l_entity_ids.forth
							end
						end
						all_futures_loaded := True
					end
				end
			end
		rescue
			if not l_retried then
				l_retried := True
				retry
			end
		end

	load_all_future_listings is
		local
			l_listing: like context.future_listing_template
			l_query: PES_QUERY_ZONE_LOADER [like context.future_listing_template]
			l_retried: BOOLEAN
			l_future_listing_category: CATEGORY_HANDLER [like context, like context.future_listing_template]
			l_entity_ids: ARRAYED_LIST [INTEGER]
		do
			if not all_future_listings_loaded then
				if is_multi_zonal then
					components.do_all (agent {like context.zone_template}.load_all_future_listings)
					all_future_listings_loaded := True
				else
					if use_pes and not l_retried then
						create l_query.make (context.context_index)
						l_query.set_mnemonic (mnemonic)
						l_query.set_future_listings (True)
						if attached {PES_RESULT_ZONE_LOADER [like context.future_listing_template]} pes_api.send_general_request (l_query).get as l_result then
							if l_result.remote_exception then
								raise (l_result.tag_name)
							end
							future_listings := l_result.table
							all_future_listings_loaded := True
						end
					else
						l_future_listing_category := context.future_listing_accessor
						load_all (l_future_listing_category)
						if xrf_entity_id_table.has (l_future_listing_category.category_id) then
							l_entity_ids := xrf_entity_id_table.item (l_future_listing_category.category_id).linear_representation
							from
								l_entity_ids.start
							until
								l_entity_ids.after
							loop
								l_listing := l_future_listing_category.entity (l_entity_ids.item)
								add_future_listing (l_listing)
								l_entity_ids.forth
							end
						end
						all_future_listings_loaded := True
					end
				end
			end
		rescue
			if not l_retried then
				l_retried := True
				retry
			end
		end

	load_all_market_indexes is
		local
			l_market_index: like context.market_index_template
			l_query: PES_QUERY_ZONE_LOADER [like context.market_index_template]
			l_retried: BOOLEAN
			l_market_index_category: CATEGORY_HANDLER [like context, like context.market_index_template]
			l_entity_ids: ARRAYED_LIST [INTEGER]
		do
			if not all_market_indexes_loaded then
				if is_multi_zonal then
					components.do_all (agent {like context.zone_template}.load_all_market_indexes)
					all_market_indexes_loaded := True
				else
					if use_pes and not l_retried then
						create l_query.make (context.context_index)
						l_query.set_mnemonic (mnemonic)
						l_query.set_market_indexes (True)
						if attached {PES_RESULT_ZONE_LOADER [like context.market_index_template]} pes_api.send_general_request (l_query).get as l_result then
							if l_result.remote_exception then
								raise (l_result.tag_name)
							end
							market_indexes := l_result.table
							all_market_indexes_loaded := True
						end
					else
						l_market_index_category := context.market_index_accessor
						load_all (l_market_index_category)
						if xrf_entity_id_table.has (l_market_index_category.category_id) then
							l_entity_ids := xrf_entity_id_table.item (l_market_index_category.category_id).linear_representation
							from
								l_entity_ids.start
							until
								l_entity_ids.after
							loop
								l_market_index := l_market_index_category.entity (l_entity_ids.item)
								add_market_index (l_market_index)
								l_entity_ids.forth
							end
						end
						all_market_indexes_loaded := True
					end
				end
			end
		rescue
			if not l_retried then
				l_retried := True
				retry
			end
		end

	load_all_market_index_listings is
		local
			l_listing: like context.market_index_listing_template
			l_query: PES_QUERY_ZONE_LOADER [like context.market_index_listing_template]
			l_retried: BOOLEAN
			l_market_index_listing_category: CATEGORY_HANDLER [like context, like context.market_index_listing_template]
			l_entity_ids: ARRAYED_LIST [INTEGER]
		do
			if not all_market_index_listings_loaded then
				if is_multi_zonal then
					components.do_all (agent {like context.zone_template}.load_all_market_index_listings)
					all_market_index_listings_loaded := True
				else
					if use_pes and not l_retried then
						create l_query.make (context.context_index)
						l_query.set_mnemonic (mnemonic)
						l_query.set_market_index_listings (True)
						if attached {PES_RESULT_ZONE_LOADER [like context.market_index_listing_template]} pes_api.send_general_request (l_query).get as l_result then
							if l_result.remote_exception then
								raise (l_result.tag_name)
							end
							market_index_listings := l_result.table
							all_market_index_listings_loaded := True
						end
					else
						l_market_index_listing_category := context.market_index_listing_accessor
						load_all (l_market_index_listing_category)
						if xrf_entity_id_table.has (l_market_index_listing_category.category_id) then
							l_entity_ids := xrf_entity_id_table.item (l_market_index_listing_category.category_id).linear_representation
							from
								l_entity_ids.start
							until
								l_entity_ids.after
							loop
								l_listing := l_market_index_listing_category.entity (l_entity_ids.item)
								add_market_index_listing (l_listing)
								l_entity_ids.forth
							end
						end
						all_market_index_listings_loaded := True
					end
				end
			end
		rescue
			if not l_retried then
				l_retried := True
				retry
			end
		end

	all_companies: like all_companies_saved is
		do
			if all_companies_saved = Void then
				create all_companies_saved.make_empty
				if is_multi_zonal then
					across components as c_components loop
						c_components.item.load_all_companies	-- This builds 'companies' (important for caching) as well as 'all_domestic_companies'.
						all_companies_saved.append_array (c_components.item.all_domestic_companies)
					end
				else
					load_all_companies
					across companies as c_companies loop
						if c_companies.item /= Void and then c_companies.item.is_valid then
							all_companies_saved.extend (c_companies.item)
						end
					end
				end
			end
			Result := all_companies_saved
		ensure
			all_companies_not_void: Result /= Void
			no_void_company: not Result.has (Void)
		end

	all_domestic_companies: like all_domestic_companies_saved is
		do
			if all_domestic_companies_saved = Void then
				create all_domestic_companies_saved.make_empty
				if is_multi_zonal then
					across components as c_components loop
						all_domestic_companies_saved.append_array (c_components.item.all_domestic_companies)
					end
				else
					load_all_domestic_companies
					across companies as i_companies loop
						if attached i_companies.item as l_company and then l_company.is_valid and then l_company.xrf_code > 0 and then (is_real_company (l_company)) then
							all_domestic_companies_saved.extend (l_company)
						end
					end
				end
			end
			Result := all_domestic_companies_saved
		ensure
			not_void: Result /= Void
			no_void: not Result.has (Void)
			all_in_zone: across Result as c_companies all same_zone (c_companies.item.zone) end
		end

	all_securities: like all_securities_saved is
		do
			if all_securities_saved = Void then
				create all_securities_saved.make_empty
				if is_multi_zonal then
					across components as c_components loop
						all_securities_saved.append_array (c_components.item.all_domestic_securities)
					end
				else
					load_all_securities
					across securities as c_securities loop
						if c_securities.item /= Void and then c_securities.item.is_valid then
							all_securities_saved.extend (c_securities.item)
						end
					end
				end
			end
			Result := all_securities_saved
		ensure
			all_securities_not_void: Result /= Void
			no_void_security: not Result.has (Void)
		end

	all_domestic_securities: like all_domestic_securities_saved is
			-- all securities with xrf > 0
		require
			not_multi_zonal: not is_multi_zonal
		do
			if all_domestic_securities_saved = Void then
				create all_domestic_securities_saved.make_empty
				load_all_securities
				across securities as c_securities loop
					if c_securities.item /= Void and then c_securities.item.is_valid and then c_securities.item.xrf_code > 0 then
						all_domestic_securities_saved.extend (c_securities.item)
					end
				end
			end
			Result := all_domestic_securities_saved
		ensure
			all_domestic_securities: Result /= Void
			no_void_domestics_security: not Result.has (Void)
		end

	all_listings: like all_listings_saved is
		do
			if all_listings_saved = Void then
				create all_listings_saved.make_empty
				if is_multi_zonal then
					across components as c_components loop
						all_listings_saved.append_array (c_components.item.all_listings)
					end
				else
					load_all_listings
					across listings as c_listings loop
						if c_listings.item /= Void then
								--and then c_listings.item.is_valid then
							all_listings_saved.extend (c_listings.item)
						end
					end
				end
			end
			Result := all_listings_saved
		ensure
			all_listings_not_void: Result /= Void
			no_void_listing: not Result.has (Void)
		end

	all_government_issuers: like all_government_issuers_saved is
		do
			if all_government_issuers_saved = Void then
				create all_government_issuers_saved.make_empty
				if is_multi_zonal then
					across components as c_components loop
						all_government_issuers_saved.append_array (c_components.item.all_government_issuers)
					end
				else
					load_all_government_issuers
					across government_issuers as c_government_issuers loop
						if c_government_issuers.item /= Void and then c_government_issuers.item.is_valid then
							all_government_issuers_saved.extend (c_government_issuers.item)
						end
					end
				end
			end
			Result := all_government_issuers_saved
		ensure
			all_government_issuers_not_void: Result /= Void
			no_void_government_issuer: not Result.has (Void)
		end

	all_government_issues: like all_government_issues_saved is
		do
			if all_government_issues_saved = Void then
				create all_government_issues_saved.make_empty
				if is_multi_zonal then
					across components as c_components loop
						all_government_issues_saved.append_array (c_components.item.all_government_issues)
					end
				else
					load_all_government_issues
					across government_issues as c_government_issues loop
						if c_government_issues.item /= Void and then c_government_issues.item.is_valid then
							all_government_issues_saved.extend (c_government_issues.item)
						end
					end
				end
			end
			Result := all_government_issues_saved
		ensure
			all_government_issues_not_void: Result /= Void
			no_void_government_issue: not Result.has (Void)
		end

	all_government_issue_listings: like all_government_issue_listings_saved is
		do
			if all_government_issue_listings_saved = Void then
				create all_government_issue_listings_saved.make_empty
				if is_multi_zonal then
					across components as c_components loop
						all_government_issue_listings_saved.append_array (c_components.item.all_government_issue_listings)
					end
				else
					load_all_government_issue_listings
					across government_issue_listings as c_government_issue_listings loop
						if c_government_issue_listings.item /= Void and then c_government_issue_listings.item.is_valid then
							all_government_issue_listings_saved.extend (c_government_issue_listings.item)
						end
					end
				end
			end
			Result := all_government_issue_listings_saved
		ensure
			all_government_issue_listings_not_void: Result /= Void
			no_void_government_issue_listing: not Result.has (Void)
		end

	all_bonds: like all_bonds_saved is
			--	All bonds for this zone that were able to be loaded by `load_all_bonds'.
		do
			if all_bonds_saved = Void then
				create all_bonds_saved.make_empty
				if is_multi_zonal then
					across components as c_components loop
						all_bonds_saved.append_array (c_components.item.all_bonds)
					end
				else
					load_all_bonds
					across bonds as c_bonds loop
						if c_bonds.item /= Void and then c_bonds.item.is_valid then
							all_bonds_saved.extend (c_bonds.item)
						end
					end
				end
			end
			Result := all_bonds_saved
		ensure
			all_bonds_not_void: Result /= Void
			no_void_bond: not Result.has (Void)
		end

	all_bond_listings: like all_bond_listings_saved is
		do
			if all_bond_listings_saved = Void then
				create all_bond_listings_saved.make_empty
				if is_multi_zonal then
					across components as c_components loop
						all_bond_listings_saved.append_array (c_components.item.all_bond_listings)
					end
				else
					load_all_bond_listings
					across bond_listings as c_bond_listings loop
						if c_bond_listings.item /= Void and then c_bond_listings.item.is_valid then
							all_bond_listings_saved.extend (c_bond_listings.item)
						end
					end
				end
			end
			Result := all_bond_listings_saved
		ensure
			all_bond_listings_not_void: Result /= Void
			no_void_bond_listing: not Result.has (Void)
		end

	all_funds: like all_funds_saved is
		do
			if all_funds_saved = Void then
				create all_funds_saved.make_empty
				if is_multi_zonal then
					across components as c_components loop
						all_funds_saved.append_array (c_components.item.all_funds)
					end
				else
					load_all_funds
					across funds as c_funds loop
						if c_funds.item /= Void and then c_funds.item.is_valid then
							all_funds_saved.extend (c_funds.item)
						end
					end
				end
			end
			Result := all_funds_saved
		ensure
			all_funds_not_void: Result /= Void
			no_void_fund: not Result.has (Void)
		end

	all_fund_issues: like all_fund_issues_saved is
		do
			if all_fund_issues_saved = Void then
				create all_fund_issues_saved.make_empty
				if is_multi_zonal then
					across components as c_components loop
						all_fund_issues_saved.append_array (c_components.item.all_fund_issues)
					end
				else
					load_all_fund_issues
					across fund_issues as c_fund_issues loop
						if c_fund_issues.item /= Void and then c_fund_issues.item.is_valid then
							all_fund_issues_saved.extend (c_fund_issues.item)
						end
					end
				end
			end
			Result := all_fund_issues_saved
		ensure
			all_fund_issues_not_void: Result /= Void
			no_void_fund_issue: not Result.has (Void)
		end

	all_fund_listings: like all_fund_listings_saved is
		do
			if all_fund_listings_saved = Void then
				create all_fund_listings_saved.make_empty
				if is_multi_zonal then
					across components as c_components loop
						all_fund_listings_saved.append_array (c_components.item.all_fund_listings)
					end
				else
					load_all_fund_listings
					across fund_listings as c_fund_listings loop
						if c_fund_listings.item /= Void and then c_fund_listings.item.is_valid then
							all_fund_listings_saved.extend (c_fund_listings.item)
						end
					end
				end
			end
			Result := all_fund_listings_saved
		ensure
			all_fund_listings_not_void: Result /= Void
			no_void_fund_listing: not Result.has (Void)
		end

	all_futures: like all_futures_saved is
		do
			if all_futures_saved = Void then
				create all_futures_saved.make_empty
				if is_multi_zonal then
					across components as c_components loop
						all_futures_saved.append_array (c_components.item.all_futures)
					end
				else
					load_all_futures
					across futures as c_futures loop
						if c_futures.item /= Void and then c_futures.item.is_valid then
							all_futures_saved.extend (c_futures.item)
						end
					end
				end
			end
			Result := all_futures_saved
		ensure
			all_futures_not_void: Result /= Void
			no_void_future: not Result.has (Void)
		end

	all_future_listings: like all_future_listings_saved is
		do
			if all_future_listings_saved = Void then
				create all_future_listings_saved.make_empty
				if is_multi_zonal then
					across components as c_components loop
						all_future_listings_saved.append_array (c_components.item.all_future_listings)
					end
				else
					load_all_future_listings
					across future_listings as c_future_listings loop
						if c_future_listings.item /= Void and then c_future_listings.item.is_valid then
							all_future_listings_saved.extend (c_future_listings.item)
						end
					end
				end
			end
			Result := all_future_listings_saved
		ensure
			all_future_listings_not_void: Result /= Void
			not_void_future_listing: not Result.has (Void)
		end

	all_market_indexes: like all_market_indexes_saved is
		do
			if all_market_indexes_saved = Void then
				create all_market_indexes_saved.make_empty
				if is_multi_zonal then
					across components as c_components loop
						all_market_indexes_saved.append_array (c_components.item.all_market_indexes)
					end
				else
					load_all_market_indexes
					across market_indexes as c_market_indexes loop
						if c_market_indexes.item /= Void and then c_market_indexes.item.is_valid then
							all_market_indexes_saved.extend (c_market_indexes.item)
						end
					end
				end
			end
			Result := all_market_indexes_saved
		ensure
			all_market_indexes_not_void: Result /= Void
			no_void_market_index: not Result.has (Void)
		end

	all_market_index_listings: like all_market_index_listings_saved is
		do
			if all_market_index_listings_saved = Void then
				create all_market_index_listings_saved.make_empty
				if is_multi_zonal then
					across components as c_components loop
						all_market_index_listings_saved.append_array (c_components.item.all_market_index_listings)
					end
				else
					load_all_market_index_listings
					across market_index_listings as c_market_index_listings loop
						if c_market_index_listings.item /= Void and then c_market_index_listings.item.is_valid then
							all_market_index_listings_saved.extend (c_market_index_listings.item)
						end
					end
				end
			end
			Result := all_market_index_listings_saved
		ensure
			all_market_index_listings_not_void: Result /= Void
			no_void_market_index_listing: not Result.has (Void)
		end

	all_countries: like countries is
		do
			Result := countries
		end

feature {SECURITY, LISTING, LISTING_MONTHLY_RMDATA, FVS_METRICS} --Returns

	returns: ZONAL_RETURN is
		do
			Result := zonal_returns.item (Current)
		end

feature {NONE} --Implementation

	currency_saved: like base_currency
			-- Cached version of zonal base currency

	country_saved: like primary_country
			-- Cached version of zone's primary country

	base_country_saved: like base_country
			-- Cached version of country associated with zonal base currency

	market_dates_table: HASH_TABLE [ROSE_DATE_SET, ZONE [LOCATION]] is
			-- Cache of market dates
		once
			create Result.make (10)
		ensure
			market_dates_table_not_void: Result /= Void
		end

	bank_dates_table: HASH_TABLE [ROSE_DATE_SET, ZONE [LOCATION]] is
			-- Cache of bank dates
		once
			create Result.make (10)
		ensure
			bank_dates_table_not_void: Result /= Void
		end

feature -- Zonal Data

	base_currency: like context.currency_template is
			-- Default currency for zone
		do
			if currency_saved = Void then
				inspect mnemonic
				when 'U' then
					currency_saved := context.usd
				when 'J' then
					currency_saved := context.jpy
				when 'C' then
					currency_saved := context.cad
				when 'O' then
					currency_saved := context.usd
				when 'E' then
					currency_saved := context.gbp
				when 'S' then
					currency_saved := context.usd
				when 'W' then
					currency_saved := context.usd
				else
					-- Void
				end
			end
			Result := currency_saved
		end

	base_country: like context.country_template is
			-- Country associated with base currency of zone
		do
			if base_country_saved = Void then
				inspect mnemonic
				when 'U' then
					base_country_saved := context.us
				when 'J' then
					base_country_saved := context.jp
				when 'C' then
					base_country_saved := context.ca
				when 'O' then
					base_country_saved := context.us
				when 'E' then
					base_country_saved := context.gb
				when 'S' then
					base_country_saved := context.us
				else
					-- Void
				end
			end
			Result := base_country_saved
		end

	primary_country: like context.country_template is
			-- Primary country of investment zone
		do
			if country_saved = Void then
				inspect mnemonic
				when 'U' then
					country_saved := context.us
				when 'J' then
					country_saved := context.jp
				when 'C' then
					country_saved := context.ca
				when 'O' then
					country_saved := context.au
				when 'E' then
					country_saved := context.gb
				when 'S' then
					country_saved := context.mx
				else
					-- Void
				end
			end
			Result := country_saved
		end

	zonal_market_data: ONCE_PER_ZONE [MARKET_DATA] is
		once
			create Result.make
		ensure
			zonal_market_data_not_void: Result /= Void
		end

	zonal_returns: ONCE_PER_ZONE [ZONAL_RETURN] is
		once
			create Result.make
		ensure
			zonal_returns_not_void: Result /= Void
		end

	once_returns: ONCE_PER_OBJECT [RM_RETURN]

	rmdat_start_month: ROSE_MONTH is
		do
			inspect mnemonic
			when 'U' then
				create Result.make_from_julian_month (425)
					-- May 1985
			when 'J' then
				create Result.make_from_julian_month (266)
					-- December 1971
			when 'C' then
				create Result.make_from_julian_month (384)
					-- December 1981
			when 'E' then
				create Result.make_from_julian_month (384)
					-- December 1979
			when 'O' then
				create Result.make_from_julian_month (384)
					-- December 1979
			when 'S' then
				create Result.make_from_julian_month (384)
					-- December 1979
			else
			end
		end

feature {INVESTMENT_ENTITY, ZONE} --Investment entity

	countries: ROSE_LINEAR_ARRAY [like context.country_template] is
		do
			if all_countries_saved = Void then
				if not is_multi_zonal then
					all_countries_saved := context.country_to_investment_region.corelated_entities (investment_region)
					from
						all_countries_saved.start
					until
						all_countries_saved.off
					loop
						all_countries_saved.current_item.set_zone (Current)
						all_countries_saved.forth
					end
				elseif not is_global then
					create all_countries_saved.make_empty
					across components as c_components loop
						all_countries_saved.append_array (c_components.item.countries)
					end
				else
					create all_countries_saved.make_from_linear (context.country_accessor.all_entities)
				end
				all_countries_saved.sort_by (agent compare_countries_by_old_positional_id)
			end
			Result := all_countries_saved
		end

	xrf_entity_id (a_category_id: INTEGER; a_xrf: INTEGER): INTEGER is
			-- Entity handle for `a_xrf' of `a_category_id' type
		require
			not_multi_zonal: not is_multi_zonal
		local
			l_table: HASH_TABLE [INTEGER, INTEGER]
		do
			l_table := xrf_entity_id_table.item (a_category_id)
			if l_table /= Void then
				Result := l_table.item (a_xrf)
			else
				Result := -1
			end
		end

feature -- Already loaded flags

	all_companies_loaded: BOOLEAN
			-- Has `load_all_companies' been called more recently than `clear_cache'?

	all_securities_loaded: BOOLEAN
			-- Has `load_all_securities' been called more recently than `clear_cache'?

	all_listings_loaded: BOOLEAN
			-- Has `load_all_listings' been called more recently than `clear_cache'?

	all_government_issuers_loaded: BOOLEAN
			-- Has `load_all_government_issuers' been called more recently than `clear_cache'?

	all_government_issues_loaded: BOOLEAN
			-- Has `load_all_government_issues' been called more recently than `clear_cache'?

	all_government_issue_listings_loaded: BOOLEAN
			-- Has `load_all_government_issue_listings' been called more recently than `clear_cache'?

	all_bonds_loaded: BOOLEAN
			-- Has `load_all_bonds' been called more recently than `clear_cache'?

	all_bond_listings_loaded: BOOLEAN
			-- Has `load_all_bond_listings' been called more recently than `clear_cache'?

	all_funds_loaded: BOOLEAN
			-- Has `load_all_funds' been called more recently than `clear_cache'?

	all_fund_issues_loaded: BOOLEAN
			-- Has `load_all_fund_issues' been called more recently than `clear_cache'?

	all_fund_listings_loaded: BOOLEAN
			-- Has `load_all_fund_listings' been called more recently than `clear_cache'?

	all_futures_loaded: BOOLEAN
			-- Has `load_all_futures' been called more recently than `clear_cache'?

	all_future_listings_loaded: BOOLEAN
			-- Has `load_all_futurex_listings' been called more recently than `clear_cache'?

	all_market_indexes_loaded: BOOLEAN
			-- Has `load_all_market_indexes' been called more recently than `clear_cache'?

	all_market_index_listings_loaded: BOOLEAN
			-- Has `load_all_market_indexe_listings' been called more recently than `clear_cache'?

feature {NONE} --Status implementation

	min_cxrf_saved: INTEGER
			-- actually min_fxrf

	xrf_entity_id_table: HASH_TABLE [HASH_TABLE [INTEGER, INTEGER], INTEGER]
			-- Holds for each category a mapping of xrf codes to entity_ids

	add_xrf_entity_id (a_record: ZONAL_XRF_RECORD) is
			-- Add `a_record.entity_id' to the `xrf_entity_id_table'
		require
			not_multi_zonal: not is_multi_zonal
			a_record_not_void: a_record /= Void
			correct_zone: a_record.zone.item (1) = mnemonic
		local
			l_table: HASH_TABLE [INTEGER, INTEGER]
		do
			l_table := xrf_entity_id_table.item (a_record.category_id)
			if l_table = Void then
				create l_table.make (10000)
				xrf_entity_id_table.put (l_table, a_record.category_id)
			end
			l_table.put (a_record.entity_id, a_record.xrf)
		ensure
			added: xrf_entity_id_table.item (a_record.category_id).has (a_record.xrf) and then xrf_entity_id_table.item (a_record.category_id).item (a_record.xrf).is_equal (a_record.entity_id)
		end

	all_loaded_table: HASH_TABLE [BOOLEAN, INTEGER]

	all_domestic_companies_saved: ROSE_LINEAR_ARRAY [like context.company_template]

	all_companies_saved: ROSE_LINEAR_ARRAY [like context.company_template]

	all_securities_saved: ROSE_LINEAR_ARRAY [like context.security_template]

	all_domestic_securities_saved: ROSE_LINEAR_ARRAY [like context.security_template]

	all_listings_saved: ROSE_LINEAR_ARRAY [like context.listing_template]

	all_government_issuers_saved: ROSE_LINEAR_ARRAY [like context.government_issuer_template]

	all_government_issues_saved: ROSE_LINEAR_ARRAY [like context.government_issue_template]

	all_government_issue_listings_saved: ROSE_LINEAR_ARRAY [like context.government_issue_listing_template]
			--		all_composite_assets_saved : ROSE_LINEAR_ARRAY[like context.composite_asset_template]
			--		all_composite_asset_listings_saved : ROSE_LINEAR_ARRAY[like context.composite_asset_listing_template]

	all_bonds_saved: ROSE_LINEAR_ARRAY [like context.bond_template]

	all_bond_listings_saved: ROSE_LINEAR_ARRAY [like context.bond_listing_template]

	all_funds_saved: ROSE_LINEAR_ARRAY [like context.fund_template]

	all_fund_issues_saved: ROSE_LINEAR_ARRAY [like context.fund_issue_template]

	all_fund_listings_saved: ROSE_LINEAR_ARRAY [like context.fund_listing_template]

	all_futures_saved: ROSE_LINEAR_ARRAY [like context.future_template]

	all_future_listings_saved: ROSE_LINEAR_ARRAY [like context.future_listing_template]

	all_market_indexes_saved: ROSE_LINEAR_ARRAY [like context.market_index_template]

	all_market_index_listings_saved: ROSE_LINEAR_ARRAY [like context.market_index_listing_template]

	all_countries_saved: ROSE_LINEAR_ARRAY [like context.country_template]

feature {COMPANY, PES_QUERY_ZONE_LOADER} -- Caches

	companies: HASH_TABLE [like context.company_template, INTEGER]

	securities: HASH_TABLE [like context.security_template, INTEGER]

	listings: HASH_TABLE [like context.listing_template, INTEGER]

	government_issuers: HASH_TABLE [like context.government_issuer_template, INTEGER]

	government_issues: HASH_TABLE [like context.government_issue_template, INTEGER]

	government_issue_listings: HASH_TABLE [like context.government_issue_listing_template, INTEGER]
			--	composite_assets: HASH_TABLE[like context.composite_asset_template,INTEGER]
			--	composite_asset_listings: HASH_TABLE[like context.composite_asset_listing_template,INTEGER]

	bonds: HASH_TABLE [like context.bond_template, INTEGER]

	bond_listings: HASH_TABLE [like context.bond_listing_template, INTEGER]

	futures: HASH_TABLE [like context.future_template, INTEGER]

	future_listings: HASH_TABLE [like context.future_listing_template, INTEGER]

	funds: HASH_TABLE [like context.fund_template, INTEGER]

	fund_issues: HASH_TABLE [like context.fund_issue_template, INTEGER]

	fund_listings: HASH_TABLE [like context.fund_listing_template, INTEGER]

	market_indexes: HASH_TABLE [like context.market_index_template, INTEGER]

	market_index_listings: HASH_TABLE [like context.market_index_listing_template, INTEGER]

feature --Caches

	all_composite_listings: ROSE_LINEAR_ARRAY [like context.listing_template] is
		local
			i: INTEGER
		do
			Result := all_composite_listings_saved
			if Result = Void then
				create Result.make_empty
				from
					i := 1
				until
					i > all_listings.count
				loop
					if is_composite_entity (all_listings.item (i)) then
						Result.extend (all_listings.item (i))
					end
					i := i + 1
				end
				all_composite_listings_saved := Result
			end
		end

	all_composite_listings_saved: like all_composite_listings

	all_composite_securities: ROSE_LINEAR_ARRAY [like context.security_template] is
		local
			i: INTEGER
		do
			Result := all_composite_securities_saved
			if Result = Void then
				create Result.make_empty
				from
					i := 1
				until
					i > all_securities.count
				loop
					if is_composite_entity (all_securities.item (i)) then
						Result.extend (all_securities.item (i))
					end
					i := i + 1
				end
				all_composite_securities_saved := Result
			end
		end

	all_composite_securities_saved: like all_composite_securities

	all_composite_companies: ROSE_LINEAR_ARRAY [like context.company_template] is
		local
			i: INTEGER
		do
			Result := all_composite_companies_saved
			if Result = Void then
				create Result.make_empty
				from
					i := 1
				until
					i > all_companies.count
				loop
					if is_composite_entity (all_companies.item (i)) then
						Result.extend (all_companies.item (i))
					end
					i := i + 1
				end
				all_composite_companies_saved := Result
			end
		end

	all_composite_companies_saved: like all_composite_companies

	is_composite_entity (a_entity: INVESTMENT_ENTITY): BOOLEAN is
			-- is this entity a composite asset?
		require
			a_entity_not_void: a_entity /= Void
		do
			if attached a_entity.brim_code (Void) as l_brim_code then
				Result := l_brim_code.item (3).is_equal ('C')
			end
		end

	is_real_company (a_entity: INVESTMENT_ENTITY): BOOLEAN
			-- Is `a_entity' real company?
		require
			a_entity_not_void: a_entity /= Void
		do
			if attached a_entity.brim_code (Void) as l_brim_code then
				Result := l_brim_code.item (3).is_equal ('E')
			end
		end

feature --Artifacts

	home_listings : HASH_TABLE[like context.listing_template,INTEGER]
			--Cache of home zone listings keyed by handle of foreign listing in this zone.  This is the inverse map to dummy_listings.
		require
			not_multi_zonal: not is_multi_zonal
		local
			l_cursor : CURSOR
		do
			Result := home_listings_saved
			if Result = Void then
				create home_listings_saved.make (dummy_listings.count)
				l_cursor := dummy_listings.cursor
				from dummy_listings.start until dummy_listings.off loop
					home_listings_saved[listing(dummy_listings.item_for_iteration).handle] := dummy_listings.key_for_iteration
					dummy_listings.forth
				end
				dummy_listings.go_to (l_cursor)
				Result := home_listings_saved
			end
		end

	home_listings_saved : HASH_TABLE[like context.listing_template,INTEGER]


	dummy_listings: HASH_TABLE [INTEGER, like context.listing_template] is
			-- dummy listing xrfs keyed by listing
		require
			not_multi_zonal: not is_multi_zonal
		local
			l_dummy_listing_identifier: ENTITY_IDENTIFIER
			l_dummy_listings: ROSE_LINEAR_ARRAY [ENTITY]
			l_dummy_xrf_code, l_attribute_key: STRING
			l_dummy_listing: like context.listing_template
		do
			Result := dummy_listings_saved
			if Result = Void then
				create Result.make (200)
				l_attribute_key := local_xrfzone_identifier
				l_dummy_listing_identifier := context.identifier_from_name (l_attribute_key)
				l_dummy_listings := l_dummy_listing_identifier.all_entities ("%%", create {ROSE_DATE}.make_ccyymmdd (99991231))
				from
					l_dummy_listings.start
				until
					l_dummy_listings.off
				loop
					if attached {like context.listing_template} l_dummy_listings.current_item as l_listing then
						l_dummy_xrf_code := l_listing.attribute_by_name (l_attribute_key, Void)
						if l_dummy_xrf_code /= Void then
							l_dummy_xrf_code := l_dummy_xrf_code.twin
							l_dummy_xrf_code.prune_all (mnemonic)
							Result.force (l_dummy_xrf_code.to_integer, l_listing)

							-- Create a surrogate with the same handle but a different xrf_zone.
							l_dummy_listing := l_listing.twin
							l_dummy_listing.set_zone(Current)
							l_dummy_listing.set_xrf_code(l_dummy_xrf_code.to_integer)
							add_listing(l_dummy_listing)
						else
							raise ("No `" + l_attribute_key + "' available for " + l_listing.out)
						end
					end
					l_dummy_listings.forth
				end
				dummy_listings_saved := Result
			end
		end

	dummy_securities: ROSE_HASH_TABLE [like context.security_template, like context.listing_template] is
			-- dummy securities keyed by listing
		require
			not_multi_zonal: not is_multi_zonal
		local
			l_listing: like context.LISTING_template
			l_dummy_xrf_code: STRING
			l_security: like context.SECURITY_template
			l_dummy_listings: like dummy_listings
		do
			Result := dummy_securities_saved
			if Result = Void then
				create Result.make (200)
				from
					l_dummy_listings := dummy_listings
					l_dummy_listings.start
				until
					l_dummy_listings.off
				loop
					l_listing := l_dummy_listings.key_for_iteration
					l_security := Void
					l_dummy_xrf_code := l_listing.attribute_by_name (sxrf_identifier, Void)
					if l_dummy_xrf_code = Void then
						l_dummy_xrf_code := l_listing.attribute_by_name ("FSXRF CODE", Void)
					end
					if l_dummy_xrf_code /= Void and l_dummy_xrf_code.is_integer then
						l_security := security (l_dummy_xrf_code.to_integer)
						if l_security /= Void then
							Result.put (l_security, l_listing)
						end
					end
					if l_security = Void then
						l_security := l_listing.security
						Result.put (l_security, l_listing)
					end
					l_dummy_listings.forth
				end
				dummy_securities_saved := Result
			end
		end

	dummy_companies: ROSE_HASH_TABLE [INTEGER --like context.company_template
	, like context.listing_template] is
			-- Dummy listings Foreign cxrfs keyed by listing
		require
			not_multi_zonal: not is_multi_zonal
		local
			l_listing: like context.listing_template
			l_dummy_xrf_code: STRING
			l_company: like context.company_template
			l_dummy_listings: like dummy_listings
		do
			Result := dummy_companies_saved
			if Result = Void then
				create Result.make (200)
				from
					l_dummy_listings := dummy_listings
					l_dummy_listings.start
				until
					l_dummy_listings.off
				loop
					l_listing := l_dummy_listings.key_for_iteration
					l_company := Void
					l_dummy_xrf_code := l_listing.attribute_by_name (lcxrf_identifier, Void)
					if l_dummy_xrf_code = Void then
						l_dummy_xrf_code := l_listing.attribute_by_name ("FCXRF CODE", Void)
					end
					if l_dummy_xrf_code /= Void and l_dummy_xrf_code.is_integer then
						Result.put (l_dummy_xrf_code.to_integer, l_listing)
					end
					l_dummy_listings.forth
				end
				dummy_companies_saved := Result
			end
		end

	dummy_listings_by_security: HASH_TABLE [like context.LISTING_template, like context.SECURITY_template] is
			-- dummy listings keyed by security
		require
			not_multi_zonal: not is_multi_zonal
		local
			l_listing: like context.LISTING_template
			l_security: like context.SECURITY_template
			l_dummy_securities: like dummy_securities
		do
			Result := dummy_listings_by_security_saved
			if Result = Void then
				create Result.make (200)
				from
					l_dummy_securities := dummy_securities
					l_dummy_securities.start
				until
					l_dummy_securities.off
				loop
					l_listing := l_dummy_securities.key_for_iteration
					l_security := l_dummy_securities.item_for_iteration
					Result.put (l_listing, l_security)
					l_dummy_securities.forth
				end
				dummy_listings_by_security_saved := Result
			end
		end

	lcxrf_identifier: STRING is
			-- the attribute identifier name for the local cxrf
		require
			not_multi_zonal: not is_multi_zonal
		do
			if mnemonic.is_equal ('O') or mnemonic.is_equal ('J') then
				Result := "LOCAL FCXRF"
			elseif mnemonic.is_equal ('E') then
				Result := "E FCXRF"
			elseif mnemonic.is_equal ('S') then
				Result := "S FCXRF"
			end
		end

	sxrf_identifier: STRING is
			-- the attribute identifier name for the local sxrf
		require
			not_multi_zonal: not is_multi_zonal
		do
			if mnemonic.is_equal ('O') or mnemonic.is_equal ('J') then
				Result := "LOCAL FSXRF"
			elseif mnemonic.is_equal ('E') then
				Result := "E FSXRF"
			elseif mnemonic.is_equal ('S') then
				Result := "S FSXRF"
			end
		end

	fsxrf_identifier_id: INTEGER is
			-- the attribute identifier id for the local fsxrf
		require
			not_multi_zonal: not is_multi_zonal
		do
			if mnemonic.is_equal ('O') or mnemonic.is_equal ('J') then
				Result := context.local_fsxrf_type_id
			elseif mnemonic.is_equal ('E') then
				Result := context.e_fsxrf_type_id
			elseif mnemonic.is_equal ('S') then
				Result := context.s_fsxrf_type_id
			end
		end

	local_brim_code_identifier: STRING is
			-- the attribute identifier name for the local brim code
		require
			not_multi_zonal: not is_multi_zonal
		do
			if mnemonic.is_equal ('O') or mnemonic.is_equal ('J') then
				Result := "LOCAL BRIM CODE"
			elseif mnemonic.is_equal ('E') then
				Result := "E BRIM CODE"
			elseif mnemonic.is_equal ('S') then
				Result := "S BRIM CODE"
			end
		end

	local_xrfzone_identifier: STRING is
			-- the attribute identifier name for the local xrfzone
		require
			not_multi_zonal: not is_multi_zonal
		do
			if mnemonic.is_equal ('O') then
				Result := "LOCAL XRFZONE"
			else
				Result := mnemonic.out + " XRFZONE"
			end
		end

	local_sedol_identifier: STRING is
			-- the attribute identifier name for the local xrfzone
		require
			not_multi_zonal: not is_multi_zonal
		do
			if mnemonic.is_equal ('O') then
				Result := "LOCAL SEDOL"
			else
				Result := "SEDOL"
			end
		end

	location: LOCATION

feature -- Metrics.

	metrics: ROSE_LINEAR_ARRAY[PAIR[STRING, DOUBLE]]
	-- Returns the metrics stored in 'entity_misses', formatted for process metrics.
	--
	-- You may have come here because you noticed metrics of the form "ZONE entity misses" and
	-- you are wondering what to do.  Let's say that you have a high value for "RESEARCH/U/COMPANY".  If you
	-- cache this early in your program, you can eliminate all of these database calls (in ZONE.company).
	--
	--		research_context.u_zone.load_all_companies
	--
	-- You should supply a method to turn off this caching during debugging.
	local
		l_pair: PAIR [STRING, DOUBLE]
		l_context_index: INTEGER
		l_category_id: INTEGER
		l_context_name: STRING
		l_category_name: STRING
		l_context: SUPPLIER_CONTEXT
	do
		create Result.make_empty

		across
			entity_misses as c_entity_misses
		loop
			l_context_index := c_entity_misses.key.first
			l_category_id := c_entity_misses.key.third

			if attached context.context_from_index(l_context_index) as al_context and then
			   attached al_context.name as al_context_name
			then
				l_context := al_context
				l_context_name := al_context_name + " (" + l_context_index.out + ")"
			else
				l_context := Void
				l_context_name := "context = " + l_context_index.out
			end

			if attached context.category_name(l_category_id) as al_category_name
			then
				l_category_name := al_category_name + " (" + l_category_id.out + ")"
			else
				l_category_name := "category_id = " + l_category_id.out
			end

			create l_pair.make("ZONE entity misses/" + l_context_name + "/" + c_entity_misses.key.second.out + "/" + l_category_name, c_entity_misses.item.to_double)
			Result.extend(l_pair)
		end
	end

	entity_misses: HASH_TABLE[INTEGER_64, KEY_TRIPLE[INTEGER, CHARACTER, INTEGER]]
	-- Number of times we missed the cache and had to go to the database in 'company', 'security' or 'listing', indexed by [context_index, mnemonic, category_id].
	once
		process_metrics.register_source(Current)
		create Result.make(5)
	end

feature {NONE} --Artifact implementation

	dummy_companies_saved: like dummy_companies
			-- Cache for dummy companies

	dummy_securities_saved: like dummy_securities
			-- Cache for dummy securities

	dummy_listings_saved: like dummy_listings
			-- cache for dummy listings

	dummy_listings_by_security_saved: like dummy_listings_by_security
			-- Cache for listings by security

feature {NONE} -- Implementation

	bitmap_start_date: ROSE_DATE is
			-- Start date for bitmap
		once
			create Result.make_ccyymmdd (19500101)
		ensure
			bitmap_start_date_not_void: Result /= Void
		end

	debug_output: STRING is
			-- Debug output
		do
			Result := mnemonic.out
		end

	compare_countries_by_old_positional_id (a, b: like context.country_template): BOOLEAN
			-- Is `a' less than `b'?
		do
			if a /= Void and b /= Void then
				if a.old_positional_id > 0 and b.old_positional_id > 0 then
					Result := a.old_positional_id < b.old_positional_id
				elseif a.old_positional_id <= 0 and b.old_positional_id <= 0 then
					if attached a.iso_code_most_recent_key (Void) as l_iso_a and attached b.iso_code_most_recent_key (Void) as l_iso_b then
						Result := l_iso_a < l_iso_b
					end
				end
			end
		end

end
