note
	description: "Used to handle errors from an SQL_SERVER.  Can provide special processing for certain errors."
	author: "DZ"
	last_editor: "$Author: rj $"
	date: "$Date: 2011/07/12 $"
	revision: "$Revision: #6 $"
	path: "$File: //rose/source/kernel/infrastructure/persistence/databases/sql/sql_error_handler.e $"

deferred class
	SQL_ERROR_HANDLER

feature -- Features to be redefined.

	on_error (error_id: INTEGER; object: ANY; message: STRING) is
		deferred
		end

	allow_error(a_native_error: like error_template) is
	-- Add this error code to the list of errors that can be ignored.
	require
		a_native_error_not_void: a_native_error/=Void
	do
		exceptions.raise("allow_error is not defined.")
	end

	dont_allow_error(a_native_error: like error_template) is
	-- Remove this error code from the list of errors that can be ignored.
	require
		native_error_not_void: a_native_error/=Void
	do
		exceptions.raise("dont_allow_error is not defined.")
	end

	error_was_allowed(a_native_error: like error_template): BOOLEAN
	-- True if the error occurred but was 'allowed'.  Be sure to also call 'clear_last_allowed_error'.
	require
		native_error_not_void: a_native_error/=Void
	do
		exceptions.raise("error_was_allowed is not defined.")
	end

	clear_last_allowed_error
	-- Called in conjunction with 'error_was_allowed'.
	do
		exceptions.raise("clear_last_allowed_error is not defined.")
	end

	error_is_negligible(a_native_error: like error_template): BOOLEAN
	-- True if the passed error is to be ignored.
	require
		a_native_error_not_void: a_native_error/=Void
	do
		exceptions.raise("error_is_negligible is not defined.")
	end

	can_ignore_rose_sql_error(a_rose_sql_error_code: INTEGER): BOOLEAN is
	-- Used to check to see if we can call 'allow_error', etc.
	do
		Result := False
	end

	set_on_error_agent(a_on_error_agent: like on_error_agent)
	-- See 'on_error_agent'.
	do
		on_error_agent := a_on_error_agent
	end

	on_error_agent: PROCEDURE[ANY, TUPLE[error_id: INTEGER; object: ANY; message: STRING]]
	-- Optional agent to be called during 'on_error'.  The TUPLE has the same signature as
	-- 'on_error'.
	note
		option: transient
	attribute
	end


feature {NONE} -- Templates.
	error_template: ANY							-- To be redefined by the descendant.
	do
	end

feature {NONE} -- Implementation.
	exceptions : EXCEPTIONS is
		once
			create Result
		end

end
