indexing
	description: "Representation of 'select * from master..sysproceeses'."
	author: "RJ"
	last_editor: "$Author: rj $"
	date: "$Date: 2014/05/16 $"
	revision: "$Revision: #13 $"
	path: "$File: //rose/source/applications/system/data_connection_viewer/sql_server_sysprocesses.e $"

class
	SQL_SERVER_SYSPROCESSES

inherit
	DEBUG_OUTPUT
	IEEE_MATH
		export
			{NONE} all
		end

create
	make_name_only,
	make

feature -- Interface.
	make_name_only(a_server_name: STRING)
	-- Used for making a fake entry.
	require
		a_server_name_not_void: a_server_name/=Void
	do
		name := a_server_name.twin
	end

	make(a_server_name: STRING) is
	-- The argument 'a_server_name' is the name of a SQL Server machine (e.g. RCSQL5, not RCSQL5\x64Decvel01).
	require
		a_server_name_not_void: a_server_name/=Void
	local
		l_propset: OLEDB_PROPERTY_SET
		l_property_constants: OLEDB_PROPERTY_CONSTANTS
		l_column_statement: STRING
	do
		make_name_only(a_server_name)

		create server.make_integrated_security_with_timeout(a_server_name, "master", default_partner_timeout)
		if server.make_succeeded
		then
			create parent_by_spid.make(100)

			create l_property_constants
			create l_propset.make(l_property_constants.DBPROPSET_ROWSET)
			l_propset.add_integer_property(l_property_constants.dbprop_commandtimeout, default_partner_timeout)
			server.set_default_command_propset(l_propset)
			server.set_implicit_transaction_off
			server.sql_execute_immediate("USE master")

			-- In SQL Server 2005 they changed 'waittime' from 32 to 64 bits.
			-- In SQL Server 2005 we can't directly get 'nt_domain' or 'nt_username'.
			-- We get the error "Protocol error in TDS stream" and sometimes something about the buffer is too large/small.
			l_column_statement := "spid, kpid, blocked, waittype, cast(waittime as bigint) as waittime, lastwaittype, waitresource, dbid, uid, cpu, physical_io, memusage, login_time, last_batch, ecid, open_tran, status, hostname, program_name, hostprocess, cmd, cast(nt_username as varchar(256)) as nt_username, cast(nt_domain as varchar(256)) as nt_domain, net_address, net_library, loginame, stmt_start, stmt_end"
			create get_processes_predicate.make(server, "select " + l_column_statement + " from master..sysprocesses (nolock)")
			connected := get_processes_predicate.success
			if connected
			then
				create get_one_process_predicate.make(
					server, "select " + l_column_statement + " from master..sysprocesses (nolock) where spid=? and ecid=?")
				if not get_one_process_predicate.success
				then
					connected := False
				end
			end

			if connected
			then
				get_free_space_command := server.simple_command("master..xp_fixeddrives")
				if get_free_space_command=Void
				then
					connected := False
				end

				create get_free_space_command2.make(server,
													"[
														SELECT distinct volume_mount_point, available_bytes
														FROM sys.master_files AS f
														CROSS APPLY sys.dm_os_volume_stats(f.database_id, f.file_id)
														order by volume_mount_point
													]"
												   )
				if not get_free_space_command2.success
				then
					connected := False
				end
			end

			if connected
			then
				create get_db_usages_predicate.make(
					server, "select database_id as first, size as second, physical_name as third from master.sys.master_files with (nolock) order by database_id, physical_name")
--					server, "select dbid as first, size as second, filename as third from master..sysaltfiles with (nolock) order by dbid, filename")
--					server, "select database_id as first, size as second, physical_filename as third from master..master_files with (nolock) order by database_id")
				if not get_db_usages_predicate.success
				then
					connected := False
				end
			end
		end
	end

	refresh is
	-- Reads all of the rows in master..sysprocesses.
	do
		if connected
		then
			db_usages_saved := Void
			create processes.make_empty
			server.sql_execute_immediate("USE master")
			get_processes_predicate.set_container(processes)
			get_processes_predicate.retrieve(Void, Void)
			if not get_processes_predicate.success
			then
				success := False
				status_message := "Failed to read the sysprocesses table."
				connected := False
			else
				success := True
				status_message := ""

				-- Mark blockers (and trim trailing spaces).
				-- While we are at it, reformat 'program_name'.
				from
					processes.start
				until
					processes.off
				loop
					prepare_sysprocess(processes.current_item)
					processes.forth
				end
			end
		end
	end

	refresh_single(a_sysprocess: SQL_SERVER_SYSPROCESS) is
	-- Like 'refresh', but for a single SPID.
	require
		a_sysprocess_not_void: a_sysprocess/=Void
	local
		l_sysprocesses: ROSE_LINEAR_ARRAY[SQL_SERVER_SYSPROCESS]
	do
		create l_sysprocesses.make_empty
		server.sql_execute_immediate("USE master")
		get_one_process_predicate.set_container(l_sysprocesses)
		get_one_process_predicate.retrieve(Void, <<a_sysprocess.spid, a_sysprocess.ecid>>)
		if not get_one_process_predicate.success
		then
			success := False
			status_message := "Failed to read the sysprocesses table."
			connected := False
		else
			success := True
			status_message := ""
			if l_sysprocesses.count>0
			then
				a_sysprocess.copy(l_sysprocesses.first)
			else
				a_sysprocess.mark_as_dead
			end

			prepare_sysprocess(a_sysprocess)
		end
	end

	blockers: ROSE_LINEAR_ARRAY[SQL_SERVER_SYSPROCESS] is
	-- List of top-level blockers (those blockers who are not themselves blocked).
	require
		refresh_called: processes/=Void
	do
		create Result.make_empty
		from
			processes.start
		until
			processes.off
		loop

			if processes.current_item.is_blocker and
			   (processes.current_item.blocked=0 or processes.current_item.spid=processes.current_item.blocked)
			then
				Result.extend(processes.current_item)
			end

			processes.forth
		end
	ensure
		result_not_void: Result/=Void
	end

	blockees(a_spid: INTEGER): ROSE_LINEAR_ARRAY[SQL_SERVER_SYSPROCESS]
	-- List of processes that are blocked by another process.
	require
		refresh_called: processes/=Void
	do
		create Result.make_empty
		Result.compare_references
		safe_blockees2(a_spid, Result)
		Result.compare_objects
	end

	is_interesting_blocker(a_process:SQL_SERVER_SYSPROCESS): BOOLEAN is
	-- True if at least one of his blockees has been waiting more than 60 seconds and is not named SQLDMO_% or "Symantec i3".
	require
		refresh_called: processes/=Void
		a_process_not_void: a_process/=Void
	local
		l_saved_index: INTEGER
	do
		l_saved_index := processes.index

		from
			processes.start
		until
			processes.off or Result
		loop
			if processes.current_item.blocked=a_process.spid and then
			   (not equal(processes.current_item.program_name.substring(1, 7), "SQLDMO_") or else
			    not equal(dbname(processes.current_item.dbid), "tempdb")) and then
			   not equal(processes.current_item.program_name.substring(1, 11), "Symantec i3") and then
			   not equal(processes.current_item.program_name.substring(1, 16), "Symantec/Indepth") and then
			   processes.current_item.waittime>60000			-- Greater than 60 seconds.
			then
				Result := True
			end

			processes.forth
		end

		processes.go_i_th(l_saved_index)
	end

	dbname(a_dbid: INTEGER): STRING is
	-- Converts 'a_dbid' into a db name.
	do
		if db_names=Void
		then
			refresh_db_names
		end

		Result := db_names.item(a_dbid)
		if Result=Void
		then
			refresh_db_names	-- Last try.
			Result := db_names.item(a_dbid)
		end

		if Result=Void
		then
			Result := "dbid=" + a_dbid.out
		end
	end

	disk_usages: ROSE_LINEAR_ARRAY[TUPLE[drive_letter: STRING; free_space: DOUBLE; notes: STRING]] is
	-- Returns free space info for each disk on this system.
	local
		l_integer: INTEGER
		l_tuple: TUPLE[drive_letter: STRING; free_space: DOUBLE; notes: STRING]
	do
		create Result.make_empty
		if connected
		then
			l_integer := get_free_space_command.execute(True)
			if l_integer>=0
			then
				from
					l_integer := get_free_space_command.move_first
				until
					l_integer<0
				loop
					create l_tuple
					l_tuple.drive_letter := get_free_space_command.get_uc_string_value(1).string
					l_tuple.free_space := get_free_space_command.get_integer_value(2) / 1024.0
					l_tuple.notes := ""
					Result.extend(l_tuple)

					l_integer := get_free_space_command.move_next
				end
			else
				create l_tuple
				l_tuple.drive_letter := "Error"
				l_tuple.notes := ""
				Result.extend(l_tuple)
			end
		end
	end

	db_usages: ROSE_LINEAR_ARRAY[TUPLE[dbname: STRING; size: DOUBLE; filename: STRING; unused_space: DOUBLE]] is
	-- Returns disk usage info for each database on this system.
	local
		l_raw_db_usage: TUPLE[dbid: INTEGER; pages: INTEGER; filename: STRING]
		l_db_usage: TUPLE[dbname: STRING; size: DOUBLE; filename: STRING; unused_space: DOUBLE]
	do
		if db_usages_saved=Void
		then
			create db_usages_saved.make_empty
			if connected
			then
				server.sql_execute_immediate("USE master")
				get_db_usages_predicate.retrieve(Void, Void)
				if get_db_usages_predicate.success
				then
					from
						get_db_usages_predicate.array.start
					until
						get_db_usages_predicate.array.off
					loop
						l_raw_db_usage := get_db_usages_predicate.array.current_item
						create l_db_usage
						l_db_usage.dbname := dbname(l_raw_db_usage.dbid)
						l_db_usage.size := l_raw_db_usage.pages * pagesize / 1024.0 / 1024.0 / 1024.0	-- Convert bytes to GB.
						l_db_usage.filename := l_raw_db_usage.filename
						l_db_usage.filename.prune_all_trailing(' ')
						l_db_usage.unused_space := qnan
						db_usages_saved.extend(l_db_usage)
						get_db_usages_predicate.array.forth
					end
				end
			end
		end

		Result := db_usages_saved
	end

	unused_spaces(a_database_name: STRING): ROSE_LINEAR_ARRAY[TUPLE[filename: STRING; unused_space: DOUBLE]] is
	-- Returns unused space for this database.
	require
		a_database_name_not_void: a_database_name/=Void
	local
		l_predicate: COLLECTING_PREDICATE[TUPLE[filename: STRING; unused_space: DOUBLE]]
		l_found: BOOLEAN
	do
		server.sql_execute_immediate("USE [" + a_database_name + "]")
		create l_predicate.make(
			server,
			"select RTRIM(filename) as filename, (size - FILEPROPERTY(name, 'SPACEUSED')) * 8192.0 / 1024.0 / 1024.0 / 1024.0 as unused_space %
			%from sysfiles")
		if l_predicate.success
		then
			l_predicate.retrieve(Void, Void)
			if l_predicate.success
			then
				Result := l_predicate.array

				if db_usages_saved/=Void
				then
					from
						Result.start
					until
						Result.off
					loop
						from
							db_usages_saved.start
							l_found := False
						until
							db_usages_saved.off or l_found
						loop
							if equal(db_usages_saved.current_item.filename, Result.current_item.filename)
							then
								l_found := True
								db_usages_saved.current_item.unused_space := Result.current_item.unused_space
							else
								db_usages_saved.forth
							end
						end

						Result.forth
					end
				end
			end
		end
	end

feature {NONE}	-- Implementation.
	safe_blockees(a_spid: INTEGER; a_depth: INTEGER): ROSE_LINEAR_ARRAY[SQL_SERVER_SYSPROCESS]
	-- This version of 'blockees' has the argument 'a_depth' which is used to prevent stack overflows.
	local
		l_secondary_blockees: ROSE_LINEAR_ARRAY[SQL_SERVER_SYSPROCESS]
		l_saved_index: INTEGER
	do
		create Result.make_empty
		l_saved_index := processes.index

		from
			processes.start
		until
			processes.off
		loop
			if processes.current_item.blocked=a_spid and processes.current_item.spid/=a_spid
			then
				Result.extend(processes.current_item)
				if processes.current_item.spid/=0
				then
					-- A deadlock could cause the depth to go to infinity so we stop at 20.
					if a_depth < 20
					then
						l_secondary_blockees := safe_blockees(processes.current_item.spid, a_depth + 1)

						across
							l_secondary_blockees as c_secondary_blockees
						loop
							if not Result.has(c_secondary_blockees.item)
							then
								Result.extend(c_secondary_blockees.item)
							else
								do_nothing
							end
						end

--						if l_secondary_blockees.count>0
--						then
--print(name + "%T" + a_depth.out + "%T" + a_spid.out + "%T" + l_secondary_blockees.count.out + "%N")
--							Result.append_array(l_secondary_blockees)
--						end
					end
				end
			end

			processes.forth
		end

		processes.go_i_th(l_saved_index)
	end

	safe_blockees2(a_spid: INTEGER; a_result: ROSE_LINEAR_ARRAY[SQL_SERVER_SYSPROCESS])
	-- This version of 'blockees' has the argument 'a_depth' which is used to prevent stack overflows.
	local
		l_saved_index: INTEGER
	do
		l_saved_index := processes.index

		from
			processes.start
		until
			processes.off
		loop
			if processes.current_item.blocked=a_spid and processes.current_item.spid/=a_spid
			then
				if not a_result.has(processes.current_item)
				then
					a_result.extend(processes.current_item)

					if processes.current_item.spid/=0
					then
						safe_blockees2(processes.current_item.spid, a_result)
					end
				else
					do_nothing
				end
			end

			processes.forth
		end

		processes.go_i_th(l_saved_index)
	end

	prepare_sysprocess(a_sysprocess: SQL_SERVER_SYSPROCESS) is
	-- Common code used by 'refresh' and 'refresh_single'.
	local
		l_found: BOOLEAN
		l_pos: INTEGER
		l_program_name: STRING
		i: INTEGER
	do
		a_sysprocess.set_sysprocesses(Current)
		a_sysprocess.prune_trailing_spaces

		-- Save 'a_sysprocess' in case it is a parent (a child will have a non-zero ecid).
		if a_sysprocess.ecid = 0
		then
			parent_by_spid[a_sysprocess.spid] := a_sysprocess
		end

		-- Put the program name before the path (c:\folder\abc.exe --> abc.exe - c:\folder).
		l_program_name := a_sysprocess.program_name
		if l_program_name.count/=0 and not l_program_name.has_substring("DEAD-")
		then
			l_pos := l_program_name.last_index_of('\', l_program_name.count)
			if l_pos/=0
			then
				l_program_name :=
					l_program_name.substring(l_pos + 1, l_program_name.count) +
					" - " +
					l_program_name.substring(1, l_pos - 1)
				a_sysprocess.set_program_name(l_program_name)
			end
		end

		-- Find out who is blocking us (top blockers only).
		if a_sysprocess.blocked/=0
		then
			from
				l_found := False
				i := processes.lower
			until
				i>processes.upper or l_found
			loop
				if a_sysprocess.blocked=processes.item(i).spid and
				   a_sysprocess.spid/=processes.item(i).spid and
				   (processes.item(i).blocked=0 or processes.item(i).blocked=processes.item(i).spid)
				then
					processes.item(i).set_is_blocker
					l_found := True
				end

				i := i + 1
			end
		end
	end

	refresh_db_names is
	-- Makes or remakes 'db_names' (a map used to convert 'dbid' into a database name).
	local
		l_pred: COLLECTING_PREDICATE[PAIR[INTEGER, STRING]]
	do
		create l_pred.make(server, "select dbid as first, name as second from master..sysdatabases (nolock)")
		if l_pred.success
		then
			l_pred.retrieve(Void, Void)
			if l_pred.success
			then
				create db_names.make(l_pred.array.count)
				from
					l_pred.array.start
				until
					l_pred.array.off
				loop
					db_names.put(l_pred.array.current_item.second, l_pred.array.current_item.first)
					l_pred.array.forth
				end
			end

		end
	end

	debug_output: STRING
	-- <Precursor>
	do
		Result := name
	end

feature	-- Attributes.
	connected: BOOLEAN							-- True if we think that we are connected.

	db_usages_saved: ROSE_LINEAR_ARRAY[TUPLE[dbname: STRING; size: DOUBLE; filename: STRING; unused_space: DOUBLE]]
												-- Saved value of 'db_usages'.

	name: STRING								-- The name of the SQL Server that we are looking at.

	parent_by_spid: HASH_TABLE[SQL_SERVER_SYSPROCESS, like {SQL_SERVER_SYSPROCESS}.spid]
												-- If the sysprocess has a non-zero ecid you can look up the parent here.

	processes: ROSE_LINEAR_ARRAY[SQL_SERVER_SYSPROCESS]
												-- This is the main storage.  'select * from sysprocesses' goes here.

	server: SQLOLEDB_DATA_SERVER				-- The server we get the data from.

	success: BOOLEAN							-- Indicates the success of certain predicates (not all).

feature {NONE}	-- Attributes.
	db_names: HASH_TABLE[STRING, INTEGER]		-- Maps dbid to name.

	default_partner_timeout: INTEGER is 2		-- How long to wait (in seconds) for this server to respond.

	get_db_usages_predicate: COLLECTING_PREDICATE[TUPLE[dbid: INTEGER; pages: INTEGER; filename: STRING]]
												-- Returns disk usage for a database.

	get_free_space_command: DATA_SERVER_SIMPLE_COMMAND
												-- Returns free space for each drive.

	get_free_space_command2: COLLECTING_PREDICATE[TUPLE[volume_mount_point: STRING; available_bytes: INTEGER_64]]
												-- Returns free space for each mount point.

	get_one_process_predicate: COLLECTING_PREDICATE[SQL_SERVER_SYSPROCESS]
												-- Gets info for a single process.

	get_processes_predicate: COLLECTING_PREDICATE[SQL_SERVER_SYSPROCESS]
												-- Gets info for all processes.

	pagesize: DOUBLE is 8192.0					-- Size of a SQL Server page.  In SQL Server 2005 we will be able
												-- to use 'page_size_bytes' in 'sys.dm_os_memory_clerks'.

	status_message: STRING						-- Status messages go here.  Doesn't seem to be used.

end
