indexing
	description: "Object that is used to output error messages from OLEDB error info objects"
    author: "Pierre Nallet"
	last_editor: "$Author: rj $"
	date: "$Date: 2012/02/02 $"
	revision: "$Revision: #22 $"
	path: "$File: //rose/source/kernel/infrastructure/persistence/windows/classic/oledb_error_handler_print.e $"

class
	OLEDB_ERROR_HANDLER_PRINT

inherit
	OLEDB_ERROR_HANDLER
		undefine
			error_template
		redefine
			allow_error,
			dont_allow_error,
			error_was_allowed,
			clear_last_allowed_error,
			error_is_negligible,
			can_ignore_rose_sql_error,
			on_error_agent
		end
	OLEDB_CONSTANTS
		export
			{NONE} all
		end
	ROSE_SHARED_GLOBAL

feature -- Basic operations

	on_error (a_error_id: INTEGER; a_object: TRACEABLE; a_message: STRING) is
			-- Display details for error codes not declared negligible
		local
			l_sql_error_info: SQL_ERROR_INFO
			l_error_records: ERROR_RECORDS
			l_error_info: ERROR_INFO
			l_child_error_info: ERROR_INFO
			i, j: INTEGER
			l_message: STRING
			l_description, l_source: STRING
			l_broken: BOOLEAN
			l_statement: OLEDB_STATEMENT
			l_silent: BOOLEAN
		do
			last_allowed_error := Void			-- Not too useful since we don't come here on success.
			if error_is_negligible ([a_error_id, 0]) then
				if a_object /= Void then
					a_object.set_status (0)
					a_object.set_success (True)
				end
			else
				l_message := "OLEDB error message details%N"
				l_statement ?= a_object
				if (l_statement /= Void and l_statement.data_server /= Void) then
					l_message.append ("     Database       : " + l_statement.data_server.database + "%N")
				end
				l_message.append ("     Timestamp      : " + Global.interpreted ("$now%N"))
				l_message.append ("     Error ID       : " + a_error_id.out + "%N")
				if a_message /= Void then
					l_message.append ("     Source Message : " + a_message + "%N")
				end
				create l_error_info.make
				if not l_error_info.exists then
					l_message.append ("     Error Code     : Unable to get the error.%N")
				else
					create l_error_records.make (l_error_info)
					if not l_error_records.exists then
						l_message.append ("     Error Code     : Unable to decode the the error records.%N")
					else
						from
							i :=  l_error_records.get_record_count - 1
						until
							i < 0 or l_broken
						loop
							create l_sql_error_info.make_by_record (l_error_records, i)
							if l_sql_error_info.exists and
							   (
							   		(a_error_id = E_FAIL and error_is_negligible([DB_E_INTEGRITYVIOLATION, 0]) and l_sql_error_info.native_error = dup_key_ignored_native_error) or
							   		error_is_negligible([a_error_id, l_sql_error_info.native_error]) or
							   		error_is_negligible([0, l_sql_error_info.native_error])
							   )
							then
								last_allowed_error := [a_error_id, l_sql_error_info.native_error]
								if a_object /= Void then
									if status_agent=Void
									then
										-- By default we indicate success if the error was negligible.
										a_object.set_status (0)
										a_object.set_success (True)
									else
										status_agent.call([agent rose_errors_match(?, last_allowed_error), a_object])
									end
								end

								debug("oledb_error_handler_print")
									print("Allowed error [" + a_error_id.out + ", " + l_sql_error_info.native_error.out + "]%N")
								end

								l_broken := True
								if is_silent_agent=Void
								then
									l_silent := True	-- By default we do NOT print the error if it was negligible.
								else
									l_silent := is_silent_agent.item([agent rose_errors_match(?, last_allowed_error)])
								end
							end

							if not l_silent
							then
								if l_sql_error_info.exists then
									l_message.append ("     Error Code [" + i.out + "] : " + l_sql_error_info.native_error.out + "%N")
								end
								create l_child_error_info.make_by_record (l_error_records, i)
								if not l_child_error_info.exists then
									l_message.append ("     Text       [" + i.out + "] : Unable to decode the the child error info.%N")
								else
									l_description := l_child_error_info.get_description
									if l_description /= Void and then not l_description.is_empty then
										if l_description.count <= 30 then
											l_message.append ("     Text       [" + i.out + "] : " + l_description + "%N")
										else
											j := l_description.index_of (' ', 30)
											if j = 0 then
												l_message.append ("     Text       [" + i.out + "] : " + l_description + "%N")
											else
												l_message.append ("     Text       [" + i.out + "] : " + l_description.substring (1, j) + "%N")
											end
											from until j > l_description.count or j = 0 loop
												l_description.keep_tail (l_description.count - j)
												if l_description.count >= 30 then
													j := l_description.index_of (' ', 30)
												else
													j := 0
												end
												if j = 0 or else l_description.count <= 30 then
													l_message.append ("                    : " + l_description + "%N")
												else
													l_message.append ("                    : " + l_description.substring (1, j) + "%N")
												end
											end
										end
									end
									l_source := l_child_error_info.get_source
									if l_source /= Void then
										l_message.append ("     Source     [" + i.out + "] : " + l_source + "%N")
									end
								end
								i := i - 1
							end
						end
					end
				end

				if not l_silent then
					if a_object /= Void and then not is_signaling.item then
						is_signaling.put (True)
						a_object.trace
						is_signaling.put (False)
					end
					debug("if_no_inform")
						print (l_message)
					end
					inform ("on_error",l_message,Void)
				end

				if on_error_agent /= Void
				then
					on_error_agent.call([a_error_id, a_object, a_message, l_message])
				end
			end
		end

	allow_error (a_oledb_error: like error_template) is
			-- Add this error code to the list of errors that can be ignored.
		do
			if negligible_errors = Void then
				create negligible_errors.make (10)
			end
			negligible_errors.put (0, a_oledb_error)
		end

	dont_allow_error (a_oledb_error: like error_template) is
			-- Remove this error code from the list of errors that can be ignored.
		do
			if negligible_errors /= Void then
				negligible_errors.remove (a_oledb_error)
			end
		end

	error_was_allowed(a_error: like error_template): BOOLEAN
	-- True if the error occurred but was 'allowed'.  Be sure to also call 'clear_last_allowed_error'.
	do
		if last_allowed_error/=Void
		then
			Result := errors_match(a_error, last_allowed_error)
		end
	end

	errors_match(a_test_error, a_actual_error: like error_template): BOOLEAN
	-- True if 'a_test_error' (which can contain wildcards) is a match for 'a_actual_error' (which is an actual OLEDB error).
	-- Either 'oledb_error' or 'native_error' can be zero.  This is the wild card indicator.
	do
		Result :=
			(a_test_error.oledb_error/=0 or a_test_error.native_error/=0) and
			(a_test_error.oledb_error=0  or a_test_error.oledb_error=a_actual_error.oledb_error) and
			(a_test_error.native_error=0 or a_test_error.native_error=a_actual_error.native_error)
	end

	rose_errors_match(a_test_rose_error: INTEGER; a_actual_error: like error_template): BOOLEAN
	-- True if 'a_test_error' (which is in the ROSE format) is a match for 'a_actual_error' (which is an actual OLEDB error).
	do
		Result := errors_match(oledb_error_code_from_rose_sql_error_code(a_test_rose_error), a_actual_error)
	end

	clear_last_allowed_error
	-- Called in conjunction with 'error_was_allowed'.
	do
		last_allowed_error := Void
	end

	always_allow_error (a_oledb_error: like error_template) is
			-- Add this error code to the list of errors that can be ignored
		do
			globally_ignored_errors.put (0, a_oledb_error)
		end

	remove_from_always_allow_error (a_oledb_error: like error_template) is
			-- Add this error code to the list of errors that can be ignored
		do
			globally_ignored_errors.remove (a_oledb_error)
		end

feature -- Access

	error_is_negligible (a_oledb_error: like error_template): BOOLEAN is
			-- Does this error code represent an error that can be ignored
		do
			if negligible_errors /= Void then
				Result := negligible_errors.has (a_oledb_error)
			end
			if not Result then
				Result := globally_ignored_errors.has (a_oledb_error)
			end
		end

	can_ignore_rose_sql_error (a_rose_sql_error_code: INTEGER): BOOLEAN is
			-- Does this server know how to ignore this error?
		do
			Result := can_ignore_error (oledb_error_code_from_rose_sql_error_code (a_rose_sql_error_code))
		end

	can_ignore_error (a_oledb_error_code: like error_template): BOOLEAN is
			-- For the moment, if we get here then we can handle it.  Perhaps there are errors
			-- that we can't handle.  This is a test to see what codes can be passed
			-- to 'allow_error'.
		do
			Result := True
		end

feature {NONE} -- Implementation

	last_allowed_error: like error_template			-- The last error that we allowed to succeed instead of fail.

	negligible_errors: HASH_TABLE[INTEGER, like error_template]
			-- Table of error codes that represent errors that can be ignored

	globally_ignored_errors: HASH_TABLE [INTEGER, TUPLE[oledb_error: INTEGER; native_error: INTEGER]] is
			-- Table of error codes that represent errors that can be ignored, can be set once
		once
			create Result.make (10)
		end

	is_signaling: CELL [BOOLEAN] is
			-- Used to avoid an infinitely recursive trace
		once
			create Result.put (False)
		end

feature -- Agents.
	status_agent: ROUTINE[ANY, TUPLE[FUNCTION[ANY, TUPLE[INTEGER], BOOLEAN], TRACEABLE]]
														-- This agent sets the status on the TRACEABLE during negligible errors.

	set_status_agent(a_agent: like status_agent)
	-- See 'status_agent'.
	do
		status_agent := a_agent
	end

	is_silent_agent: FUNCTION[ANY, TUPLE[FUNCTION[ANY, TUPLE[INTEGER], BOOLEAN]], BOOLEAN]
														-- This agent decides whether a negligible error should print a message.

	set_is_silent_agent(a_agent: like is_silent_agent)
	-- See 'is_silent_agent'.
	do
		is_silent_agent := a_agent
	end

	on_error_agent: PROCEDURE[ANY, TUPLE[error_id: INTEGER; object: ANY; message, error_message: STRING]]
	-- <Precursor>
	-- This redefinition add 'error_message'.
	note
		option: transient
	attribute
	end

end

