note

	description:

		"Color gradients from Color Brewer 2.0 (http://colorbrewer2.org/)"

	author: "Colin Adams"
	last_editor: "$Author: c4 $"
	documentation: "http://colorbrewer2.org/"
	date: "$Date: 2013/06/12 $"
	revision: "$Revision: #1 $"
	path: "$File: //rose/source/kernel/infrastructure/widgets/charts/color_brewer.e $"

class COLOR_BREWER

feature -- Access

	Sequential_data: NATURAL_8 is 0
			-- Data progresses from low to high

	Diverging_data: NATURAL_8 is 1
			-- Both mid-point values and extremes have critical values
	
	Qualitative_data: NATURAL_8 is 2
			-- Each color represents a distinct category. Categories are not ordered.
	
	is_valid_nature (a_nature: NATURAL_8): BOOLEAN is
			-- Is `a_nature' a valid value for `available_sequences'?
		do
			inspect
				a_nature
			when Sequential_data, Diverging_data, Qualitative_data then
				Result := True
			else
				Result := False
			end
		end
	
	available_sequences (a_nature, a_count: NATURAL_8):
		ARRAY [TUPLE [sequence_name: STRING; sequence_array: ARRAY [EV_COLOR]; is_default_choice: BOOLEAN]] is
			-- All sequences of length `a_count' of type `a_nature'
		require
			a_nature_valid: is_valid_nature (a_nature)
		do
			inspect
				a_nature
			when Sequential_data then
				Result := sequential_sequences (a_count)
			when Diverging_data then
				Result := diverging_sequences (a_count)
			when Qualitative_data then
				Result := qualitative_sequences (a_count)
			end				
		ensure
			available_sequences_not_void: Result /= Void
			no_void_sequence: not Result.has (Void)
			correct_count: not Result.is_empty implies Result.count = a_count
		end

	sequential_sequences (a_count: NATURAL_8):
		ARRAY [TUPLE [sequence_name: STRING; sequence_array: ARRAY [EV_COLOR]; is_default_choice: BOOLEAN]] is
			-- All sequences of length `a_count' of sequential data type
		do
			inspect
				a_count
			when 3 then
				Result := <<
					["BuGn", blue_greens_3, False],
					["BuPu", blue_purples_3, False],
					["GnBu", green_blues_3, False],
					["OrRd", orange_reds_3, False],
					["PuBu", purple_blues_3, False],
					["PuBuGn", purple_blue_greens_3, False],
					["PuRd", purple_reds_3, False],
					["RdPu", red_purples_3, False],
					["YlGn", yellow_greens_3, False],
					["YlGnBu", yellow_green_blues_3, False],
					["YlOrBn", yellow_orange_browns_3, False],
					["YlOrRd", yellow_orange_reds_3, False],
					["Blues", blues_3, False],
					["Greens", greens_3, True],
					["Greys", greys_3, False],
					["Oranges", oranges_3, False],
					["Purples", purples_3, False],
					["Reds", reds_3, False]
					>>
			when 4 then
				Result := <<
					["BuGn", blue_greens_4, False],
					["BuPu", blue_purples_4, False],
					["GnBu", green_blues_4, False],
					["OrRd", orange_reds_4, False],
					["PuBu", purple_blues_4, False],
					["PuBuGn", purple_blue_greens_4, False],
					["PuRd", purple_reds_4, False],
					["RdPu", red_purples_4, False],
					["YlGn", yellow_greens_4, False],
					["YlGnBu", yellow_green_blues_4, False],
					["YlOrBn", yellow_orange_browns_4, False],
					["YlOrRd", yellow_orange_reds_4, False],
					["Blues", blues_4, False],
					["Greens", greens_4, True],
					["Greys", greys_4, False],
					["Oranges", oranges_4, False],
					["Purples", purples_4, False],
					["Reds", reds_4, False]
					>>
			when 5 then
				Result := <<
					["BuGn", blue_greens_5, False],
					["BuPu", blue_purples_5, False],
					["GnBu", green_blues_5, False],
					["OrRd", orange_reds_5, False],
					["PuBu", purple_blues_5, False],
					["PuBuGn", purple_blue_greens_5, False],
					["PuRd", purple_reds_5, False],
					["RdPu", red_purples_5, False],
					["YlGn", yellow_greens_5, False],
					["YlGnBu", yellow_green_blues_5, False],
					["YlOrBn", yellow_orange_browns_5, False],
					["YlOrRd", yellow_orange_reds_5, False],
					["Blues", blues_5, False],
					["Greens", greens_5, True],
					["Greys", greys_5, False],
					["Oranges", oranges_5, False],
					["Purples", purples_5, False],
					["Reds", reds_5, False]
					>>
			when 6 then
				Result := <<
					["BuGn", blue_greens_6, False],
					["BuPu", blue_purples_6, False],
					["GnBu", green_blues_6, False],
					["OrRd", orange_reds_6, False],
					["PuBu", purple_blues_6, False],
					["PuBuGn", purple_blue_greens_6, False],
					["PuRd", purple_reds_6, False],
					["RdPu", red_purples_6, False],
					["YlGn", yellow_greens_6, False],
					["YlGnBu", yellow_green_blues_6, False],
					["YlOrBn", yellow_orange_browns_6, False],
					["YlOrRd", yellow_orange_reds_6, False],
					["Blues", blues_6, False],
					["Greens", greens_6, True],
					["Greys", greys_6, False],
					["Oranges", oranges_6, False],
					["Purples", purples_6, False],
					["Reds", reds_6, False]
					>>
			when 7 then
				Result := <<
					["BuGn", blue_greens_7, False],
					["BuPu", blue_purples_7, False],
					["GnBu", green_blues_7, False],
					["OrRd", orange_reds_7, False],
					["PuBu", purple_blues_7, False],
					["PuBuGn", purple_blue_greens_7, False],
					["PuRd", purple_reds_7, False],
					["RdPu", red_purples_7, False],
					["YlGn", yellow_greens_7, False],
					["YlGnBu", yellow_green_blues_7, False],
					["YlOrBn", yellow_orange_browns_7, False],
					["YlOrRd", yellow_orange_reds_7, False],
					["Blues", blues_7, False],
					["Greens", greens_7, True],
					["Greys", greys_7, False],
					["Oranges", oranges_7, False],
					["Purples", purples_7, False],
					["Reds", reds_7, False]
					>>
			when 8 then
				Result := <<
					["BuGn", blue_greens_8, False],
					["BuPu", blue_purples_8, False],
					["GnBu", green_blues_8, False],
					["OrRd", orange_reds_8, False],
					["PuBu", purple_blues_8, False],
					["PuBuGn", purple_blue_greens_8, False],
					["PuRd", purple_reds_8, False],
					["RdPu", red_purples_8, False],
					["YlGn", yellow_greens_8, False],
					["YlGnBu", yellow_green_blues_8, False],
					["YlOrBn", yellow_orange_browns_8, False],
					["YlOrRd", yellow_orange_reds_8, False],
					["Blues", blues_8, False],
					["Greens", greens_8, True],
					["Greys", greys_8, False],
					["Oranges", oranges_8, False],
					["Purples", purples_8, False],
					["Reds", reds_8, False]
					>>
			when 9 then
				Result := <<
					["BuGn", blue_greens_9, False],
					["BuPu", blue_purples_9, False],
					["GnBu", green_blues_9, False],
					["OrRd", orange_reds_9, False],
					["PuBu", purple_blues_9, False],
					["PuBuGn", purple_blue_greens_9, False],
					["PuRd", purple_reds_9, False],
					["RdPu", red_purples_9, False],
					["YlGn", yellow_greens_9, False],
					["YlGnBu", yellow_green_blues_9, False],
					["YlOrBn", yellow_orange_browns_9, False],
					["YlOrRd", yellow_orange_reds_9, False],
					["Blues", blues_9, False],
					["Greens", greens_9, True],
					["Greys", greys_9, False],
					["Oranges", oranges_9, False],
					["Purples", purples_9, False],
					["Reds", reds_9, False]
					>>
			else
				Result := <<>>
			end
		ensure
			sequential_sequences_not_void: Result /= Void
			no_void_sequence: not Result.has (Void)
			correct_count: not Result.is_empty implies Result.count = a_count
		end		
	
	diverging_sequences (a_count: NATURAL_8):
		ARRAY [TUPLE [sequence_name: STRING; sequence_array: ARRAY [EV_COLOR]; is_default_choice: BOOLEAN]] is
			-- All sequences of length `a_count' of diverging data type
		do
			inspect
				a_count
			when 3 then
				Result := <<
					["BrBG", brown_blue_greens_3, False],
					["PiYG", pink_yellow_greens_3, False],
					["PRGn", purple_greens_3, False],
					["PuOr", purple_oranges_3, False],
					["RdBu", red_blues_3, False],
					["RdGy", red_greys_3, False],
					["RdYlBu", red_yellow_blues_3, False],
					["RdYlGn", red_yellow_greens_3, False],
					["Spectral", spectral_3, True]
					>>
			when 4 then
				Result := <<
					["BrBG", brown_blue_greens_4, False],
					["PiYG", pink_yellow_greens_4, False],
					["PRGn", purple_greens_4, False],
					["PuOr", purple_oranges_4, False],
					["RdBu", red_blues_4, False],
					["RdGy", red_greys_4, False],
					["RdYlBu", red_yellow_blues_4, False],
					["RdYlGn", red_yellow_greens_4, False],
					["Spectral", spectral_4, True]
					>>
			when 5 then
				Result := <<
					["BrBG", brown_blue_greens_5, False],
					["PiYG", pink_yellow_greens_5, False],
					["PRGn", purple_greens_5, False],
					["PuOr", purple_oranges_5, False],
					["RdBu", red_blues_5, False],
					["RdGy", red_greys_5, False],
					["RdYlBu", red_yellow_blues_5, False],
					["RdYlGn", red_yellow_greens_5, False],
					["Spectral", spectral_5, True]
					>>
			when 6 then
				Result := <<
					["BrBG", brown_blue_greens_6, False],
					["PiYG", pink_yellow_greens_6, False],
					["PRGn", purple_greens_6, False],
					["PuOr", purple_oranges_6, False],
					["RdBu", red_blues_6, False],
					["RdGy", red_greys_6, False],
					["RdYlBu", red_yellow_blues_6, False],
					["RdYlGn", red_yellow_greens_6, False],
					["Spectral", spectral_6, True]
					>>
			when 7 then
				Result := <<
					["BrBG", brown_blue_greens_7, False],
					["PiYG", pink_yellow_greens_7, False],
					["PRGn", purple_greens_7, False],
					["PuOr", purple_oranges_7, False],
					["RdBu", red_blues_7, False],
					["RdGy", red_greys_7, False],
					["RdYlBu", red_yellow_blues_7, False],
					["RdYlGn", red_yellow_greens_7, False],
					["Spectral", spectral_7, True]
					>>
			when 8 then
				Result := <<
					["BrBG", brown_blue_greens_8, False],
					["PiYG", pink_yellow_greens_8, False],
					["PRGn", purple_greens_8, False],
					["PuOr", purple_oranges_8, False],
					["RdBu", red_blues_8, False],
					["RdGy", red_greys_8, False],
					["RdYlBu", red_yellow_blues_8, False],
					["RdYlGn", red_yellow_greens_8, False],
					["Spectral", spectral_8, True]
					>>
			when 9 then
				Result := <<
					["BrBG", brown_blue_greens_9, False],
					["PiYG", pink_yellow_greens_9, False],
					["PRGn", purple_greens_9, False],
					["PuOr", purple_oranges_9, False],
					["RdBu", red_blues_9, False],
					["RdGy", red_greys_9, False],
					["RdYlBu", red_yellow_blues_9, False],
					["RdYlGn", red_yellow_greens_9, False],
					["Spectral", spectral_9, True]
					>>
			when 10 then
				Result := <<
					["BrBG", brown_blue_greens_10, False],
					["PiYG", pink_yellow_greens_10, False],
					["PRGn", purple_greens_10, False],
					["PuOr", purple_oranges_10, False],
					["RdBu", red_blues_10, False],
					["RdGy", red_greys_10, False],
					["RdYlBu", red_yellow_blues_10, False],
					["RdYlGn", red_yellow_greens_10, False],
					["Spectral", spectral_10, True]
					>>
			when 11 then
				Result := <<
					["BrBG", brown_blue_greens_11, False],
					["PiYG", pink_yellow_greens_11, False],
					["PRGn", purple_greens_11, False],
					["PuOr", purple_oranges_11, False],
					["RdBu", red_blues_11, False],
					["RdGy", red_greys_11, False],
					["RdYlBu", red_yellow_blues_11, False],
					["RdYlGn", red_yellow_greens_11, False],
					["Spectral", spectral_11, True]
					>>
			else
				Result := <<>>
			end
		ensure
			diverging_sequences_not_void: Result /= Void
			no_void_sequence: not Result.has (Void)
			correct_count: not Result.is_empty implies Result.count = a_count
		end

	qualitative_sequences (a_count: NATURAL_8):
		ARRAY [TUPLE [sequence_name: STRING; sequence_array: ARRAY [EV_COLOR]; is_default_choice: BOOLEAN]] is
			-- All sequences of length `a_count' of qualitative data type
		do
			inspect
				a_count
			when 3 then
				Result := <<
					["Accent", accent_3, False],
					["Dark2", dark2_3, False],
					["Paired", paired_3, False],
					["Pastel1", pastel1_3, False],
					["Pastel2", pastel2_3, False],
					["Set1", set1_3, False],
					["Set2", set2_3, False],
					["Set3", set3_3, True]
					>>
			when 4 then
				Result := <<
					["Accent", accent_4, False],
					["Dark2", dark2_4, False],
					["Paired", paired_4, False],
					["Pastel1", pastel1_4, False],
					["Pastel2", pastel2_4, False],
					["Set1", set1_4, False],
					["Set2", set2_4, False],
					["Set3", set3_4, True]
					>>
			when 5 then
				Result := <<
					["Accent", accent_5, False],
					["Dark2", dark2_5, False],
					["Paired", paired_5, False],
					["Pastel1", pastel1_5, False],
					["Pastel2", pastel2_5, False],
					["Set1", set1_5, False],
					["Set2", set2_5, False],
					["Set3", set3_5, True]
					>>
			when 6 then
				Result := <<
					["Accent", accent_6, False],
					["Dark2", dark2_6, False],
					["Paired", paired_6, False],
					["Pastel1", pastel1_6, False],
					["Pastel2", pastel2_6, False],
					["Set1", set1_6, False],
					["Set2", set2_6, False],
					["Set3", set3_6, True]
					>>
			when 7 then
				Result := <<
					["Accent", accent_7, False],
					["Dark2", dark2_7, False],
					["Paired", paired_7, False],
					["Pastel1", pastel1_7, False],
					["Pastel2", pastel2_7, False],
					["Set1", set1_7, False],
					["Set2", set2_7, False],
					["Set3", set3_7, True]
					>>
			when 8 then
				Result := <<
					["Accent", accent_8, False],
					["Dark2", dark2_8, False],
					["Paired", paired_8, False],
					["Pastel1", pastel1_8, False],
					["Pastel2", pastel2_8, False],
					["Set1", set1_8, False],
					["Set2", set2_8, False],
					["Set3", set3_8, True]
					>>
			when 9 then
				Result := <<
					["Paired", paired_9, False],
					["Pastel1", pastel1_9, False],
					["Set1", set1_9, False],
					["Set3", set3_9, True]
					>>
			when 10 then
				Result := <<
					["Paired", paired_10, False],
					["Set3", set3_10, True]
					>>
			when 11 then
				Result := <<
					["Paired", paired_11, False],
					["Set3", set3_11, True]
					>>
			when 12 then
				Result := <<
					["Set3", set3_12, True]
				>>
			else
				Result := <<>>
			end
		ensure
			qualitative_sequences_not_void: Result /= Void
			no_void_sequence: not Result.has (Void)
		end
	
feature -- Sequential single-hue color gradients
	
	blues (a_classes: NATURAL_8): ARRAY [EV_COLOR] is
			-- Blue gradient of `a_classes' shades
		require
			a_classes_large_enough: a_classes >= 3
			a_classes_small_enough: a_classes <= 9
		do
			inspect
				a_classes
			when 3 then
				Result := blues_3
			when 4 then
				Result := blues_4
			when 5 then
				Result := blues_5
			when 6 then
				Result := blues_6
			when 7 then
				Result := blues_7
			when 8 then
				Result := blues_8
			when 9 then
				Result := blues_9
			end
		ensure
			blues_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_a_classes: Result.upper = a_classes
		end
	
	greens (a_classes: NATURAL_8): ARRAY [EV_COLOR] is
			-- Green gradient of `a_classes' shades
		require
			a_classes_large_enough: a_classes >= 3
			a_classes_small_enough: a_classes <= 9
		do
			inspect
				a_classes
			when 3 then
				Result := greens_3
			when 4 then
				Result := greens_4
			when 5 then
				Result := greens_5
			when 6 then
				Result := greens_6
			when 7 then
				Result := greens_7
			when 8 then
				Result := greens_8
			when 9 then
				Result := greens_9
			end
		ensure
			greens_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_a_classes: Result.upper = a_classes
		end
	
	greys (a_classes: NATURAL_8): ARRAY [EV_COLOR] is
			-- Grey gradient of `a_classes' shades
		require
			a_classes_large_enough: a_classes >= 3
			a_classes_small_enough: a_classes <= 9
		do
			inspect
				a_classes
			when 3 then
				Result := greys_3
			when 4 then
				Result := greys_4
			when 5 then
				Result := greys_5
			when 6 then
				Result := greys_6
			when 7 then
				Result := greys_7
			when 8 then
				Result := greys_8
			when 9 then
				Result := greys_9
			end
		ensure
			greys_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_a_classes: Result.upper = a_classes
		end
	
	oranges (a_classes: NATURAL_8): ARRAY [EV_COLOR] is
			-- Orange gradient of `a_classes' shades
		require
			a_classes_large_enough: a_classes >= 3
			a_classes_small_enough: a_classes <= 9
		do
			inspect
				a_classes
			when 3 then
				Result := oranges_3
			when 4 then
				Result := oranges_4
			when 5 then
				Result := oranges_5
			when 6 then
				Result := oranges_6
			when 7 then
				Result := oranges_7
			when 8 then
				Result := oranges_8
			when 9 then
				Result := oranges_9
			end
		ensure
			oranges_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_a_classes: Result.upper = a_classes
		end
	
	purples (a_classes: NATURAL_8): ARRAY [EV_COLOR] is
			-- Purple gradient of `a_classes' shades
		require
			a_classes_large_enough: a_classes >= 3
			a_classes_small_enough: a_classes <= 9
		do
			inspect
				a_classes
			when 3 then
				Result := purples_3
			when 4 then
				Result := purples_4
			when 5 then
				Result := purples_5
			when 6 then
				Result := purples_6
			when 7 then
				Result := purples_7
			when 8 then
				Result := purples_8
			when 9 then
				Result := purples_9
			end
		ensure
			purples_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_a_classes: Result.upper = a_classes
		end
	
	reds (a_classes: NATURAL_8): ARRAY [EV_COLOR] is
			-- Red gradient of `a_classes' shades
		require
			a_classes_large_enough: a_classes >= 3
			a_classes_small_enough: a_classes <= 9
		do
			inspect
				a_classes
			when 3 then
				Result := reds_3
			when 4 then
				Result := reds_4
			when 5 then
				Result := reds_5
			when 6 then
				Result := reds_6
			when 7 then
				Result := reds_7
			when 8 then
				Result := reds_8
			when 9 then
				Result := reds_9
			end
		ensure
			reds_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_a_classes: Result.upper = a_classes
		end

	blues_3: ARRAY [EV_COLOR] is
			-- Blue gradient of 3 shades
		do
			create Result.make (1, 3)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (222, 235, 247))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (158, 202, 225))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (49, 130, 189))
		ensure
			blues_3_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_three: Result.upper = 3
		end

	blues_4: ARRAY [EV_COLOR] is
			-- Blue gradient of 4 shades
		do
			create Result.make (1, 4)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (239, 243, 255))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (189, 215, 231))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (107, 174, 214))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (33, 113, 181))
		ensure
			blues_4_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_four: Result.upper = 4
		end
	
	blues_5: ARRAY [EV_COLOR] is
			-- Blue gradient of 5 shades
		do
			create Result.make (1, 5)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (239, 243, 255))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (189, 215, 231))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (107, 174, 214))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (49, 130, 189))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (8, 81, 156))
		ensure
			blues_5_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_five: Result.upper = 5
		end
	
	blues_6: ARRAY [EV_COLOR] is
			-- Blue gradient of 6 shades
		do
			create Result.make (1, 6)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (239, 243, 255))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (198, 219, 239))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (158, 202, 225))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (107, 174, 214))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (49, 130, 189))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (8, 81, 156))
		ensure
			blues_6_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_six: Result.upper = 6
		end
	
	blues_7: ARRAY [EV_COLOR] is
			-- Blue gradient of 7 shades
		do
			create Result.make (1, 7)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (239, 243, 255))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (198, 219, 239))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (158, 202, 225))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (107, 174, 214))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (66, 146, 198))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (33, 113, 181))
			Result [7] := (create {EV_COLOR}.make_with_8_bit_rgb (8, 69, 148))
		ensure
			blues_7_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_seven: Result.upper = 7
		end
			
	blues_8: ARRAY [EV_COLOR] is
			-- Blue gradient of 8 shades
		do
			create Result.make (1, 8)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (247, 251, 255))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (222, 235, 247))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (198, 219, 239))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (158, 202, 225))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (107, 174, 214))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (66, 146, 198))
			Result [7] := (create {EV_COLOR}.make_with_8_bit_rgb (33, 113, 181))
			Result [8] := (create {EV_COLOR}.make_with_8_bit_rgb (8, 69, 148))
		ensure
			blues_8_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_eight: Result.upper = 8
		end
			
	blues_9: ARRAY [EV_COLOR] is
			-- Blue gradient of 9 shades
		do
			create Result.make (1, 9)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (247, 251, 255))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (222, 235, 247))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (198, 219, 239))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (158, 202, 225))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (107, 174, 214))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (66, 146, 198))
			Result [7] := (create {EV_COLOR}.make_with_8_bit_rgb (33, 113, 181))
			Result [8] := (create {EV_COLOR}.make_with_8_bit_rgb (8, 81, 156))
			Result [9] := (create {EV_COLOR}.make_with_8_bit_rgb (8, 48, 107))
		ensure
			blues_9_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_nine: Result.upper = 9
		end
	
	greens_3: ARRAY [EV_COLOR] is
			-- Green gradient of 3 shades
		do
			create Result.make (1, 3)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (229, 245, 224))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (161, 217, 155))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (49, 163, 84))
		ensure
			greens_3_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_three: Result.upper = 3
		end

	greens_4: ARRAY [EV_COLOR] is
			-- Green gradient of 4 shades
		do
			create Result.make (1, 4)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (237, 248, 233))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (186, 228, 179))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (116, 196, 118))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (35, 139, 69))
		ensure
			greens_4_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_four: Result.upper = 4
		end
	
	greens_5: ARRAY [EV_COLOR] is
			-- Green gradient of 5 shades
		do
			create Result.make (1, 5)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (237, 248, 233))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (186, 228, 179))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (116, 196, 118))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (49, 163, 84))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (0, 109, 44))
		ensure
			greens_5_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_five: Result.upper = 5
		end

	greens_6: ARRAY [EV_COLOR] is
			-- Green gradient of 6 shades
		do
			create Result.make (1, 6)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (237, 248, 233))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (199, 233, 192))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (161, 217, 155))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (116, 196, 118))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (49, 163, 84))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (0, 109, 44))
		ensure
			greens_6_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_six: Result.upper = 6
		end
	
	greens_7: ARRAY [EV_COLOR] is
			-- Green gradient of 7 shades
		do
			create Result.make (1, 7)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (237, 248, 233))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (199, 233, 192))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (161, 217, 155))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (116, 196, 118))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (65, 171, 93))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (35, 139, 69))
			Result [7] := (create {EV_COLOR}.make_with_8_bit_rgb (0, 90, 50))
		ensure
			greens_7_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_seven: Result.upper = 7
		end
	
	greens_8: ARRAY [EV_COLOR] is
			-- Green gradient of 8 shades
		do
			create Result.make (1, 8)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (247, 252, 245))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (229, 245, 224))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (199, 233, 192))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (161, 217, 155))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (116, 196, 118))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (65, 171, 93))
			Result [7] := (create {EV_COLOR}.make_with_8_bit_rgb (35, 139, 69))
			Result [8] := (create {EV_COLOR}.make_with_8_bit_rgb (0, 90, 50))
		ensure
			greens_8_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_eight: Result.upper = 8
		end
					
	greens_9: ARRAY [EV_COLOR] is
			-- Green gradient of 9 shades
		do
			create Result.make (1, 9)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (247, 252, 245))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (229, 245, 224))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (199, 233, 192))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (161, 217, 155))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (116, 196, 118))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (65, 171, 93))
			Result [7] := (create {EV_COLOR}.make_with_8_bit_rgb (35, 139, 69))
			Result [8] := (create {EV_COLOR}.make_with_8_bit_rgb (0, 109, 44))
			Result [9] := (create {EV_COLOR}.make_with_8_bit_rgb (0, 68, 27))
		ensure
			greens_9_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_nine: Result.upper = 9
		end
	
	greys_3: ARRAY [EV_COLOR] is
			-- Grey gradient of 3 shades
		do
			create Result.make (1, 3)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (240, 240, 240))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (189, 189, 189))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (99, 99, 99))
		ensure
			greys_3_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_three: Result.upper = 3
		end

	greys_4: ARRAY [EV_COLOR] is
			-- Grey gradient of 4 shades
		do
			create Result.make (1, 4)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (247, 247, 247))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (204, 204, 204))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (150, 150, 150))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (82, 82, 82))
		ensure
			greys_4_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_four: Result.upper = 4
		end
	
	greys_5: ARRAY [EV_COLOR] is
			-- Grey gradient of 5 shades
		do
			create Result.make (1, 5)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (247, 247, 247))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (204, 204, 204))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (150, 150, 150))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (99, 99, 99))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (37, 37, 37))
		ensure
			greys_5_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_five: Result.upper = 5
		end

	greys_6: ARRAY [EV_COLOR] is
			-- Grey gradient of 6 shades
		do
			create Result.make (1, 6)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (247, 247, 247))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (217, 217, 217))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (189, 189, 189))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (150, 150, 150))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (99, 99, 99))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (37, 37, 37))
		ensure
			greys_6_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_six: Result.upper = 6
		end
	
	greys_7: ARRAY [EV_COLOR] is
			-- Grey gradient of 7 shades
		do
			create Result.make (1, 7)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (247, 247, 247))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (217, 217, 217))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (189, 189, 189))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (150, 150, 150))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (115, 115, 115))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (82, 82, 82))
			Result [7] := (create {EV_COLOR}.make_with_8_bit_rgb (37, 37, 37))
		ensure
			greys_7_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_seven: Result.upper = 7
		end
	
	greys_8: ARRAY [EV_COLOR] is
			-- Grey gradient of 8 shades
		do
			create Result.make (1, 8)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (255, 255, 255))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (240, 240, 240))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (217, 217, 217))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (189, 189, 189))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (150, 150, 150))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (115, 115, 115))
			Result [7] := (create {EV_COLOR}.make_with_8_bit_rgb (82, 82, 82))
			Result [8] := (create {EV_COLOR}.make_with_8_bit_rgb (37, 37, 37))
		ensure
			greys_8_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_eight: Result.upper = 8
		end
					
	greys_9: ARRAY [EV_COLOR] is
			-- Grey gradient of 9 shades
		do
			create Result.make (1, 9)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (255, 255, 255))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (240, 240, 240))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (217, 217, 217))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (189, 189, 189))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (150, 150, 150))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (115, 115, 115))
			Result [7] := (create {EV_COLOR}.make_with_8_bit_rgb (82, 82, 82))
			Result [8] := (create {EV_COLOR}.make_with_8_bit_rgb (37, 37, 37))
			Result [9] := (create {EV_COLOR}.make_with_8_bit_rgb (0, 0, 0))
		ensure
			greys_9_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_nine: Result.upper = 9
		end
	
	oranges_3: ARRAY [EV_COLOR] is
			-- Orange gradient of 3 shades
		do
			create Result.make (1, 3)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (254, 230, 206))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (253, 174, 107))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (230, 85, 13))
		ensure
			oranges_3_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_three: Result.upper = 3
		end

	oranges_4: ARRAY [EV_COLOR] is
			-- Orange gradient of 4 shades
		do
			create Result.make (1, 4)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (254, 237, 222))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (253, 190, 133))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (253, 141, 60))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (217, 71, 1))
		ensure
			oranges_4_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_four: Result.upper = 4
		end
	
	oranges_5: ARRAY [EV_COLOR] is
			-- Orange gradient of 5 shades
		do
			create Result.make (1, 5)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (254, 237, 222))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (253, 190, 133))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (253, 141, 60))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (230, 85, 13))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (166, 54, 3))
		ensure
			oranges_5_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_five: Result.upper = 5
		end

	oranges_6: ARRAY [EV_COLOR] is
			-- Orange gradient of 6 shades
		do
			create Result.make (1, 6)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (254, 237, 222))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (253, 208, 162))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (253, 174, 107))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (253, 141, 60))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (230, 85, 13))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (166, 54, 3))
		ensure
			oranges_6_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_six: Result.upper = 6
		end
	
	oranges_7: ARRAY [EV_COLOR] is
			-- Orange gradient of 7 shades
		do
			create Result.make (1, 7)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (254, 237, 222))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (253, 208, 162))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (253, 174, 107))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (253, 141, 60))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (241, 105, 19))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (217, 72, 1))
			Result [7] := (create {EV_COLOR}.make_with_8_bit_rgb (140, 45, 4))
		ensure
			oranges_7_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_seven: Result.upper = 7
		end
	
	oranges_8: ARRAY [EV_COLOR] is
			-- Orange gradient of 8 shades
		do
			create Result.make (1, 8)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (255, 245, 235))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (254, 230, 206))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (253, 208, 162))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (253, 174, 107))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (253, 141, 60))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (241, 105, 19))
			Result [7] := (create {EV_COLOR}.make_with_8_bit_rgb (217, 72, 1))
			Result [8] := (create {EV_COLOR}.make_with_8_bit_rgb (140, 45, 4))
		ensure
			oranges_8_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_eight: Result.upper = 8
		end
					
	oranges_9: ARRAY [EV_COLOR] is
			-- Orange gradient of 9 shades
		do
			create Result.make (1, 9)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (255, 245, 235))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (254, 230, 206))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (253, 208, 162))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (253, 174, 107))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (253, 141, 60))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (241, 105, 19))
			Result [7] := (create {EV_COLOR}.make_with_8_bit_rgb (217, 72, 1))
			Result [8] := (create {EV_COLOR}.make_with_8_bit_rgb (166, 54, 3))
			Result [9] := (create {EV_COLOR}.make_with_8_bit_rgb (127, 39, 4))
		ensure
			oranges_9_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_nine: Result.upper = 9
		end
	
	purples_3: ARRAY [EV_COLOR] is
			-- Purple gradient of 3 shades
		do
			create Result.make (1, 3)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (239, 237, 245))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (188, 189, 220))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (117, 107, 177))
		ensure
			purples_3_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_three: Result.upper = 3
		end

	purples_4: ARRAY [EV_COLOR] is
			-- Purple gradient of 4 shades
		do
			create Result.make (1, 4)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (242, 240, 247))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (203, 201, 226))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (158, 154, 200))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (106, 81, 163))
		ensure
			purples_4_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_four: Result.upper = 4
		end
	
	purples_5: ARRAY [EV_COLOR] is
			-- Purple gradient of 5 shades
		do
			create Result.make (1, 5)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (242, 240, 247))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (203, 201, 226))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (158, 154, 200))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (117, 107, 177))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (84, 39, 143))
		ensure
			purples_5_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_five: Result.upper = 5
		end
	
	purples_6: ARRAY [EV_COLOR] is
			-- Purple gradient of 6 shades
		do
			create Result.make (1, 6)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (242, 240, 247))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (218, 218, 235))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (188, 189, 220))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (158, 154, 200))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (117, 107, 177))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (84, 39, 143))
		ensure
			purples_6_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_six: Result.upper = 6
		end
	
	purples_7: ARRAY [EV_COLOR] is
			-- Purple gradient of 7 shades
		do
			create Result.make (1, 7)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (242, 240, 247))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (218, 218, 235))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (188, 189, 220))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (158, 154, 200))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (128, 125, 186))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (106, 81, 163))
			Result [7] := (create {EV_COLOR}.make_with_8_bit_rgb (74, 20, 134))
		ensure
			purples_7_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_seven: Result.upper = 7
		end
			
	purples_8: ARRAY [EV_COLOR] is
			-- Purple gradient of 8 shades
		do
			create Result.make (1, 8)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (252, 251, 253))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (239, 237, 245))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (218, 218, 235))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (188, 189, 220))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (158, 154, 200))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (128, 125, 186))
			Result [7] := (create {EV_COLOR}.make_with_8_bit_rgb (106, 81, 163))
			Result [8] := (create {EV_COLOR}.make_with_8_bit_rgb (74, 20, 134))
		ensure
			purples_8_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_eight: Result.upper = 8
		end
			
	purples_9: ARRAY [EV_COLOR] is
			-- Purple gradient of 9 shades
		do
			create Result.make (1, 9)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (252, 251, 253))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (239, 237, 245))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (218, 218, 235))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (188, 189, 220))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (158, 154, 200))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (128, 125, 186))
			Result [7] := (create {EV_COLOR}.make_with_8_bit_rgb (106, 81, 163))
			Result [8] := (create {EV_COLOR}.make_with_8_bit_rgb (84, 39, 143))
			Result [9] := (create {EV_COLOR}.make_with_8_bit_rgb (63, 0, 125))
		ensure
			purples_9_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_nine: Result.upper = 9
		end	
	
	reds_3: ARRAY [EV_COLOR] is
			-- Red gradient of 3 shades
		do
			create Result.make (1, 3)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (254, 224, 210))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (252, 146, 114))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (222, 45, 38))
		ensure
			reds_3_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_three: Result.upper = 3
		end

	reds_4: ARRAY [EV_COLOR] is
			-- Red gradient of 4 shades
		do
			create Result.make (1, 4)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (254, 229, 217))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (252, 174, 145))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (251, 106, 74))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (203, 24, 29))
		ensure
			reds_4_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_four: Result.upper = 4
		end
	
	reds_5: ARRAY [EV_COLOR] is
			-- Red gradient of 5 shades
		do
			create Result.make (1, 5)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (254, 229, 217))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (252, 174, 145))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (251, 106, 74))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (222, 45, 38))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (165, 15, 21))
		ensure
			reds_5_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_five: Result.upper = 5
		end

	reds_6: ARRAY [EV_COLOR] is
			-- Red gradient of 6 shades
		do
			create Result.make (1, 6)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (254, 229, 217))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (252, 187, 161))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (252, 146, 114))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (251, 106, 74))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (222, 45, 38))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (165, 15, 21))
		ensure
			reds_6_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_six: Result.upper = 6
		end
	
	reds_7: ARRAY [EV_COLOR] is
			-- Red gradient of 7 shades
		do
			create Result.make (1, 7)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (254, 229, 217))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (252, 187, 161))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (252, 146, 114))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (251, 106, 74))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (239, 59, 44))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (203, 24, 29))
			Result [7] := (create {EV_COLOR}.make_with_8_bit_rgb (153, 0, 13))
		ensure
			reds_7_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_seven: Result.upper = 7
		end
	
	reds_8: ARRAY [EV_COLOR] is
			-- Red gradient of 8 shades
		do
			create Result.make (1, 8)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (255, 245, 240))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (254, 224, 210))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (252, 224, 210))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (252, 146, 114))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (251, 106, 74))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (239, 59, 44))
			Result [7] := (create {EV_COLOR}.make_with_8_bit_rgb (203, 24, 29))
			Result [8] := (create {EV_COLOR}.make_with_8_bit_rgb (153, 0, 13))
		ensure
			reds_8_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_eight: Result.upper = 8
		end
					
	reds_9: ARRAY [EV_COLOR] is
			-- Red gradient of 9 shades
		do
			create Result.make (1, 9)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (255, 245, 240))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (254, 224, 210))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (252, 187, 161))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (252, 146, 114))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (251, 106, 74))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (239, 59, 44))
			Result [7] := (create {EV_COLOR}.make_with_8_bit_rgb (203, 24, 29))
			Result [8] := (create {EV_COLOR}.make_with_8_bit_rgb (165, 15, 21))
			Result [9] := (create {EV_COLOR}.make_with_8_bit_rgb (103, 0, 13))			
		ensure
			reds_9_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_nine: Result.upper = 9
		end
		
feature -- Sequential multi-hue color gradients
	
	blue_to_greens (a_classes: NATURAL_8): ARRAY [EV_COLOR] is
			-- Blue-green gradient of `a_classes' shades
		require
			a_classes_large_enough: a_classes >= 3
			a_classes_small_enough: a_classes <= 9
		do
			inspect
				a_classes
			when 3 then
				Result := blue_greens_3
			when 4 then
				Result := blue_greens_4
			when 5 then
				Result := blue_greens_5
			when 6 then
				Result := blue_greens_6
			when 7 then
				Result := blue_greens_7
			when 8 then
				Result := blue_greens_8
			when 9 then
				Result := blue_greens_9
			end
		ensure
			blue_to_greens_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_a_classes: Result.upper = a_classes
		end
	
	blue_to_purples (a_classes: NATURAL_8): ARRAY [EV_COLOR] is
			-- Blue-purple gradient of `a_classes' shades
		require
			a_classes_large_enough: a_classes >= 3
			a_classes_small_enough: a_classes <= 9
		do
			inspect
				a_classes
			when 3 then
				Result := blue_purples_3
			when 4 then
				Result := blue_purples_4
			when 5 then
				Result := blue_purples_5
			when 6 then
				Result := blue_purples_6
			when 7 then
				Result := blue_purples_7
			when 8 then
				Result := blue_purples_8
			when 9 then
				Result := blue_purples_9
			end
		ensure
			blue_to_purples_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_a_classes: Result.upper = a_classes
		end
	
	green_to_blues (a_classes: NATURAL_8): ARRAY [EV_COLOR] is
			-- Green-blue gradient of `a_classes' shades
		require
			a_classes_large_enough: a_classes >= 3
			a_classes_small_enough: a_classes <= 9
		do
			inspect
				a_classes
			when 3 then
				Result := green_blues_3
			when 4 then
				Result := green_blues_4
			when 5 then
				Result := green_blues_5
			when 6 then
				Result := green_blues_6
			when 7 then
				Result := green_blues_7
			when 8 then
				Result := green_blues_8
			when 9 then
				Result := green_blues_9
			end
		ensure
			green_to_blues_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_a_classes: Result.upper = a_classes
		end
	
	orange_to_reds (a_classes: NATURAL_8): ARRAY [EV_COLOR] is
			-- Orange-red gradient of `a_classes' shades
		require
			a_classes_large_enough: a_classes >= 3
			a_classes_small_enough: a_classes <= 9
		do
			inspect
				a_classes
			when 3 then
				Result := orange_reds_3
			when 4 then
				Result := orange_reds_4
			when 5 then
				Result := orange_reds_5
			when 6 then
				Result := orange_reds_6
			when 7 then
				Result := orange_reds_7
			when 8 then
				Result := orange_reds_8
			when 9 then
				Result := orange_reds_9
			end
		ensure
			orange_to_reds_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_a_classes: Result.upper = a_classes
		end
	
	purple_to_blues (a_classes: NATURAL_8): ARRAY [EV_COLOR] is
			-- Purple-blue gradient of `a_classes' shades
		require
			a_classes_large_enough: a_classes >= 3
			a_classes_small_enough: a_classes <= 9
		do
			inspect
				a_classes
			when 3 then
				Result := purple_blues_3
			when 4 then
				Result := purple_blues_4
			when 5 then
				Result := purple_blues_5
			when 6 then
				Result := purple_blues_6
			when 7 then
				Result := purple_blues_7
			when 8 then
				Result := purple_blues_8
			when 9 then
				Result := purple_blues_9
			end
		ensure
			purple_to_blues_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_a_classes: Result.upper = a_classes
		end

	purple_blue_greens (a_classes: NATURAL_8): ARRAY [EV_COLOR] is
			-- Purple-blue-green gradient of `a_classes' shades
		require
			a_classes_large_enough: a_classes >= 3
			a_classes_small_enough: a_classes <= 9
		do
			inspect
				a_classes
			when 3 then
				Result := purple_blue_greens_3
			when 4 then
				Result := purple_blue_greens_4
			when 5 then
				Result := purple_blue_greens_5
			when 6 then
				Result := purple_blue_greens_6
			when 7 then
				Result := purple_blue_greens_7
			when 8 then
				Result := purple_blue_greens_8
			when 9 then
				Result := purple_blue_greens_9
			end
		ensure
			purple_blue_greens_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_a_classes: Result.upper = a_classes
		end
			
	purple_to_reds (a_classes: NATURAL_8): ARRAY [EV_COLOR] is
			-- Purple-red gradient of `a_classes' shades
		require
			a_classes_large_enough: a_classes >= 3
			a_classes_small_enough: a_classes <= 9
		do
			inspect
				a_classes
			when 3 then
				Result := purple_reds_3
			when 4 then
				Result := purple_reds_4
			when 5 then
				Result := purple_reds_5
			when 6 then
				Result := purple_reds_6
			when 7 then
				Result := purple_reds_7
			when 8 then
				Result := purple_reds_8
			when 9 then
				Result := purple_reds_9
			end
		ensure
			purple_to_reds_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_a_classes: Result.upper = a_classes
		end

	red_to_purples (a_classes: NATURAL_8): ARRAY [EV_COLOR] is
			-- Red-purple gradient of `a_classes' shades
		require
			a_classes_large_enough: a_classes >= 3
			a_classes_small_enough: a_classes <= 9
		do
			inspect
				a_classes
			when 3 then
				Result := red_purples_3
			when 4 then
				Result := red_purples_4
			when 5 then
				Result := red_purples_5
			when 6 then
				Result := red_purples_6
			when 7 then
				Result := red_purples_7
			when 8 then
				Result := red_purples_8
			when 9 then
				Result := red_purples_9
			end
		ensure
			red_to_purples_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_a_classes: Result.upper = a_classes
		end
	
	yellow_to_greens (a_classes: NATURAL_8): ARRAY [EV_COLOR] is
			-- Yellow-green gradient of `a_classes' shades
		require
			a_classes_large_enough: a_classes >= 3
			a_classes_small_enough: a_classes <= 9
		do
			inspect
				a_classes
			when 3 then
				Result := yellow_greens_3
			when 4 then
				Result := yellow_greens_4
			when 5 then
				Result := yellow_greens_5
			when 6 then
				Result := yellow_greens_6
			when 7 then
				Result := yellow_greens_7
			when 8 then
				Result := yellow_greens_8
			when 9 then
				Result := yellow_greens_9
			end
		ensure
			yellow_to_greens_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_a_classes: Result.upper = a_classes
		end	

	yellow_green_blues (a_classes: NATURAL_8): ARRAY [EV_COLOR] is
			-- Yellow-green-blue gradient of `a_classes' shades
		require
			a_classes_large_enough: a_classes >= 3
			a_classes_small_enough: a_classes <= 9
		do
			inspect
				a_classes
			when 3 then
				Result := yellow_green_blues_3
			when 4 then
				Result := yellow_green_blues_4
			when 5 then
				Result := yellow_green_blues_5
			when 6 then
				Result := yellow_green_blues_6
			when 7 then
				Result := yellow_green_blues_7
			when 8 then
				Result := yellow_green_blues_8
			when 9 then
				Result := yellow_green_blues_9
			end
		ensure
			yellow_green_blues_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_a_classes: Result.upper = a_classes
		end
	
	yellow_orange_browns (a_classes: NATURAL_8): ARRAY [EV_COLOR] is
			-- Yellow-orange-brown gradient of `a_classes' shades
		require
			a_classes_large_enough: a_classes >= 3
			a_classes_small_enough: a_classes <= 9
		do
			inspect
				a_classes
			when 3 then
				Result := yellow_orange_browns_3
			when 4 then
				Result := yellow_orange_browns_4
			when 5 then
				Result := yellow_orange_browns_5
			when 6 then
				Result := yellow_orange_browns_6
			when 7 then
				Result := yellow_orange_browns_7
			when 8 then
				Result := yellow_orange_browns_8
			when 9 then
				Result := yellow_orange_browns_9
			end
		ensure
			yellow_orange_browns_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_a_classes: Result.upper = a_classes
		end
	
	yellow_orange_reds (a_classes: NATURAL_8): ARRAY [EV_COLOR] is
			-- Yellow-orange-red gradient of `a_classes' shades
		require
			a_classes_large_enough: a_classes >= 3
			a_classes_small_enough: a_classes <= 9
		do
			inspect
				a_classes
			when 3 then
				Result := yellow_orange_reds_3
			when 4 then
				Result := yellow_orange_reds_4
			when 5 then
				Result := yellow_orange_reds_5
			when 6 then
				Result := yellow_orange_reds_6
			when 7 then
				Result := yellow_orange_reds_7
			when 8 then
				Result := yellow_orange_reds_8
			when 9 then
				Result := yellow_orange_reds_9
			end
		ensure
			yellow_orange_reds_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_a_classes: Result.upper = a_classes
		end

	blue_greens_3: ARRAY [EV_COLOR] is
			-- Blue-green gradient of 3 shades
		do
			create Result.make (1, 3)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (229, 245, 249))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (153, 216, 201))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (44, 162, 95))
		ensure
			blue_greens_3_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_three: Result.upper = 3
		end

	blue_greens_4: ARRAY [EV_COLOR] is
			-- Blue-green gradient of 4 shades
		do
			create Result.make (1, 4)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (237, 248, 251))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (178, 226, 226))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (102, 194, 164))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (35, 139, 69))
		ensure
			blue_greens_4_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_four: Result.upper = 4
		end
	
	blue_greens_5: ARRAY [EV_COLOR] is
			-- Blue-green gradient of 5 shades
		do
			create Result.make (1, 5)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (237, 248, 251))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (178, 226, 226))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (102, 194, 164))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (44, 162, 95))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (0, 109, 44))
		ensure
			blue_greens_5_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_five: Result.upper = 5
		end
	
	blue_greens_6: ARRAY [EV_COLOR] is
			-- Blue-green gradient of 6 shades
		do
			create Result.make (1, 6)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (237, 248, 251))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (204, 236, 230))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (153, 216, 201))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (102, 194, 164))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (44, 162, 95))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (0, 109, 44))
		ensure
			blue_greens_6_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_six: Result.upper = 6
		end
	
	blue_greens_7: ARRAY [EV_COLOR] is
			-- Blue-green gradient of 7 shades
		do
			create Result.make (1, 7)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (237, 248, 251))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (204, 236, 230))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (153, 216, 201))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (102, 194, 164))		
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (65, 174, 118))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (35, 139, 69))
			Result [7] := (create {EV_COLOR}.make_with_8_bit_rgb (0, 88, 36))
		ensure
			blue_greens_7_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_seven: Result.upper = 7
		end
			
	blue_greens_8: ARRAY [EV_COLOR] is
			-- Blue-green gradient of 8 shades
		do
			create Result.make (1, 8)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (247, 252, 253))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (229, 245, 249))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (204, 236, 230))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (153, 216, 201))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (102, 194, 164))		
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (65, 174, 118))
			Result [7] := (create {EV_COLOR}.make_with_8_bit_rgb (35, 139, 69))
			Result [8] := (create {EV_COLOR}.make_with_8_bit_rgb (0, 88, 36))
		ensure
			blue_greens_8_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_eight: Result.upper = 8
		end
			
	blue_greens_9: ARRAY [EV_COLOR] is
			-- Blue-green gradient of 9 shades
		do
			create Result.make (1, 9)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (247, 252, 253))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (229, 245, 249))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (204, 236, 230))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (153, 216, 201))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (102, 194, 164))		
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (65, 174, 118))
			Result [7] := (create {EV_COLOR}.make_with_8_bit_rgb (35, 139, 69))
			Result [8] := (create {EV_COLOR}.make_with_8_bit_rgb (0, 109, 44))
			Result [9] := (create {EV_COLOR}.make_with_8_bit_rgb (0, 68, 27))
		ensure
			blue_greens_9_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_nine: Result.upper = 9
		end
	
	blue_purples_3: ARRAY [EV_COLOR] is
			-- Blue-purple gradient of 3 shades
		do
			create Result.make (1, 3)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (224, 236, 244))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (158, 188, 218))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (136, 86, 167))
		ensure
			blue_purples_3_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_three: Result.upper = 3
		end

	blue_purples_4: ARRAY [EV_COLOR] is
			-- Blue-purple gradient of 4 shades
		do
			create Result.make (1, 4)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (237, 248, 251))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (179, 205, 227))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (140, 150, 198))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (136, 65, 157))
		ensure
			blue_purples_4_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_four: Result.upper = 4
		end
	
	blue_purples_5: ARRAY [EV_COLOR] is
			-- Blue-purple gradient of 5 shades
		do
			create Result.make (1, 5)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (237, 248, 251))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (179, 205, 227))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (140, 150, 198))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (136, 86, 167))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (129, 15, 124))
		ensure
			blue_purples_5_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_five: Result.upper = 5
		end
	
	blue_purples_6: ARRAY [EV_COLOR] is
			-- Blue-purple gradient of 6 shades
		do
			create Result.make (1, 6)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (237, 248, 251))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (191, 248, 230))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (158, 188, 218))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (140, 150, 198))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (136, 86, 167))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (129, 15, 124))
		ensure
			blue_purples_6_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_six: Result.upper = 6
		end
	
	blue_purples_7: ARRAY [EV_COLOR] is
			-- Blue-purple gradient of 7 shades
		do
			create Result.make (1, 7)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (237, 248, 251))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (191, 211, 230))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (158, 188, 218))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (140, 150, 198))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (140, 107, 177))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (136, 65, 157))
			Result [7] := (create {EV_COLOR}.make_with_8_bit_rgb (110, 1, 107))
		ensure
			blue_purples_7_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_seven: Result.upper = 7
		end
			
	blue_purples_8: ARRAY [EV_COLOR] is
			-- Blue-purple gradient of 8 shades
		do
			create Result.make (1, 8)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (247, 252, 253))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (224, 236, 244))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (191, 211, 230))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (158, 188, 218))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (140, 150, 198))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (140, 107, 177))
			Result [7] := (create {EV_COLOR}.make_with_8_bit_rgb (136, 65, 157))
			Result [8] := (create {EV_COLOR}.make_with_8_bit_rgb (110, 1, 107))
		ensure
			blue_purples_8_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_eight: Result.upper = 8
		end
			
	blue_purples_9: ARRAY [EV_COLOR] is
			-- Blue-purple gradient of 9 shades
		do
			create Result.make (1, 9)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (247, 252, 253))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (224, 236, 244))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (191, 211, 230))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (158, 188, 218))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (140, 150, 198))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (140, 107, 177))
			Result [7] := (create {EV_COLOR}.make_with_8_bit_rgb (136, 65, 157))
			Result [8] := (create {EV_COLOR}.make_with_8_bit_rgb (129, 15, 124))
			Result [9] := (create {EV_COLOR}.make_with_8_bit_rgb (77, 0, 75))
		ensure
			blue_purples_9_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_nine: Result.upper = 9
		end
	
	green_blues_3: ARRAY [EV_COLOR] is
			-- Green-blue gradient of 3 shades
		do
			create Result.make (1, 3)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (224, 243, 219))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (168, 221, 181))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (67, 162, 202))
		ensure
			green_blues_3_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_three: Result.upper = 3
		end

	green_blues_4: ARRAY [EV_COLOR] is
			-- Green-blue gradient of 4 shades
		do
			create Result.make (1, 4)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (240, 249, 232))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (186, 228, 188))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (123, 204, 196))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (43, 140, 190))
		ensure
			green_blues_4_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_four: Result.upper = 4
		end
	
	green_blues_5: ARRAY [EV_COLOR] is
			-- Green-blue gradient of 5 shades
		do
			create Result.make (1, 5)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (240, 249, 232))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (186, 228, 188))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (123, 204, 196))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (67, 162, 202))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (8, 104, 172))
		ensure
			green_blues_5_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_five: Result.upper = 5
		end
	
	green_blues_6: ARRAY [EV_COLOR] is
			-- Green-blue gradient of 6 shades
		do
			create Result.make (1, 6)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (240, 249, 232))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (204, 235, 197))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (168, 221, 181))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (123, 204, 196))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (67, 162, 202))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (8, 104, 172))			
		ensure
			green_blues_6_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_six: Result.upper = 6
		end
	
	green_blues_7: ARRAY [EV_COLOR] is
			-- Green-blue gradient of 7 shades
		do
			create Result.make (1, 7)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (240, 249, 232))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (204, 235, 197))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (168, 221, 181))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (123, 204, 196))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (78, 179, 211))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (43, 140, 190))
			Result [7] := (create {EV_COLOR}.make_with_8_bit_rgb (8, 88, 158))
		ensure
			green_blues_7_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_seven: Result.upper = 7
		end
			
	green_blues_8: ARRAY [EV_COLOR] is
			-- Green-blue gradient of 8 shades
		do
			create Result.make (1, 8)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (247, 252, 253))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (224, 243, 219))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (204, 235, 197))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (168, 221, 181))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (123, 204, 196))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (78, 179, 211))
			Result [7] := (create {EV_COLOR}.make_with_8_bit_rgb (43, 140, 190))
			Result [8] := (create {EV_COLOR}.make_with_8_bit_rgb (8, 88, 158))
		ensure
			green_blues_8_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_eight: Result.upper = 8
		end
			
	green_blues_9: ARRAY [EV_COLOR] is
			-- Green-blue gradient of 9 shades
		do
			create Result.make (1, 9)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (247, 252, 240))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (224, 243, 219))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (204, 235, 197))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (168, 221, 181))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (123, 204, 196))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (78, 179, 211))
			Result [7] := (create {EV_COLOR}.make_with_8_bit_rgb (43, 140, 190))
			Result [8] := (create {EV_COLOR}.make_with_8_bit_rgb (8, 104, 172))
			Result [9] := (create {EV_COLOR}.make_with_8_bit_rgb (8, 64, 129))
		ensure
			green_blues_9_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_nine: Result.upper = 9
		end
	
	orange_reds_3: ARRAY [EV_COLOR] is
			-- Orange-red gradient of 3 shades
		do
			create Result.make (1, 3)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (254, 232, 200))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (253, 187, 132))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (227, 74, 51))
		ensure
			orange_reds_3_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_three: Result.upper = 3
		end

	orange_reds_4: ARRAY [EV_COLOR] is
			-- Orange-red gradient of 4 shades
		do
			create Result.make (1, 4)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (254, 240, 217))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (253, 204, 138))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (252, 141, 89))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (215, 48, 31))
		ensure
			orange_reds_4_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_four: Result.upper = 4
		end
	
	orange_reds_5: ARRAY [EV_COLOR] is
			-- Orange-red gradient of 5 shades
		do
			create Result.make (1, 5)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (254, 240, 217))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (253, 204, 138))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (252, 141, 89))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (227, 74, 51))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (179, 0, 0))
		ensure
			orange_reds_5_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_five: Result.upper = 5
		end
	
	orange_reds_6: ARRAY [EV_COLOR] is
			-- Orange-red gradient of 6 shades
		do
			create Result.make (1, 6)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (254, 240, 217))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (253, 212, 158))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (253, 187, 132))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (252, 141, 89))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (227, 74, 51))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (179, 0, 0))
		ensure
			orange_reds_6_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_six: Result.upper = 6
		end
	
	orange_reds_7: ARRAY [EV_COLOR] is
			-- Orange-red gradient of 7 shades
		do
			create Result.make (1, 7)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (254, 240, 217))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (253, 212, 158))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (253, 187, 132))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (252, 141, 89))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (239, 101, 72))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (215, 48, 31))
			Result [7] := (create {EV_COLOR}.make_with_8_bit_rgb (153, 0, 0))
		ensure
			orange_reds_7_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_seven: Result.upper = 7
		end
			
	orange_reds_8: ARRAY [EV_COLOR] is
			-- Orange-red gradient of 8 shades
		do
			create Result.make (1, 8)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (255, 247, 236))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (254, 232, 200))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (253, 212, 158))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (253, 187, 132))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (252, 141, 89))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (239, 101, 72))
			Result [7] := (create {EV_COLOR}.make_with_8_bit_rgb (215, 48, 31))
			Result [8] := (create {EV_COLOR}.make_with_8_bit_rgb (153, 0, 0))
		ensure
			orange_reds_8_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_eight: Result.upper = 8
		end
			
	orange_reds_9: ARRAY [EV_COLOR] is
			-- Orange-red gradient of 9 shades
		do
			create Result.make (1, 9)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (255, 247, 236))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (254, 232, 200))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (253, 212, 158))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (253, 187, 132))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (252, 141, 89))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (239, 101, 72))
			Result [7] := (create {EV_COLOR}.make_with_8_bit_rgb (215, 48, 31))
			Result [8] := (create {EV_COLOR}.make_with_8_bit_rgb (179, 0, 0))
			Result [9] := (create {EV_COLOR}.make_with_8_bit_rgb (127, 0, 0))
		ensure
			orange_reds_9_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_nine: Result.upper = 9
		end
	
	purple_blues_3: ARRAY [EV_COLOR] is
			-- Purple-blue gradient of 3 shades
		do
			create Result.make (1, 3)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (236, 231, 242))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (166, 189, 219))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (43, 140, 190))
		ensure
			purple_blues_3_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_three: Result.upper = 3
		end

	purple_blues_4: ARRAY [EV_COLOR] is
			-- Purple-blue gradient of 4 shades
		do
			create Result.make (1, 4)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (241, 238, 246))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (189, 201, 225))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (116, 169, 207))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (5, 112, 176))
		ensure
			purple_blues_4_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_four: Result.upper = 4
		end
	
	purple_blues_5: ARRAY [EV_COLOR] is
			-- Purple-blue gradient of 5 shades
		do
			create Result.make (1, 5)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (241, 238, 246))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (189, 201, 225))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (116, 169, 207))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (43, 140, 190))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (4, 90, 141))
		ensure
			purple_blues_5_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_five: Result.upper = 5
		end
	
	purple_blues_6: ARRAY [EV_COLOR] is
			-- Purple-blue gradient of 6 shades
		do
			create Result.make (1, 6)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (241, 238, 246))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (208, 209, 230))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (166, 189, 219))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (116, 169, 207))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (43, 140, 190))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (4, 90, 141))
		ensure
			purple_blues_6_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_six: Result.upper = 6
		end
	
	purple_blues_7: ARRAY [EV_COLOR] is
			-- Purple-blue gradient of 7 shades
		do
			create Result.make (1, 7)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (241, 238, 246))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (208, 209, 230))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (166, 189, 219))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (116, 169, 207))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (54, 144, 192))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (5, 112, 176))
			Result [7] := (create {EV_COLOR}.make_with_8_bit_rgb (3, 78, 123))
		ensure
			purple_blues_7_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_seven: Result.upper = 7
		end
			
	purple_blues_8: ARRAY [EV_COLOR] is
			-- Purple-blue gradient of 8 shades
		do
			create Result.make (1, 8)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (255, 247, 251))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (236, 231, 242))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (208, 209, 230))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (166, 189, 219))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (116, 169, 207))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (54, 144, 192))
			Result [7] := (create {EV_COLOR}.make_with_8_bit_rgb (5, 112, 176))
			Result [8] := (create {EV_COLOR}.make_with_8_bit_rgb (3, 78, 123))
		ensure
			purple_blues_8_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_eight: Result.upper = 8
		end
			
	purple_blues_9: ARRAY [EV_COLOR] is
			-- Purple-blue gradient of 9 shades
		do
			create Result.make (1, 9)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (255, 247, 251))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (236, 231, 242))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (208, 209, 230))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (166, 189, 219))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (116, 169, 207))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (54, 144, 192))
			Result [7] := (create {EV_COLOR}.make_with_8_bit_rgb (5, 112, 176))
			Result [8] := (create {EV_COLOR}.make_with_8_bit_rgb (4, 90, 141))
			Result [9] := (create {EV_COLOR}.make_with_8_bit_rgb (2, 56,88))
		ensure
			purple_blues_9_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_nine: Result.upper = 9
		end
	
	purple_blue_greens_3: ARRAY [EV_COLOR] is
			-- Purple-blue-green gradient of 3 shades
		do
			create Result.make (1, 3)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (236, 226, 240))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (166, 189, 219))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (28, 144, 153))
		ensure
			purple_blue_greens_3_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_three: Result.upper = 3
		end

	purple_blue_greens_4: ARRAY [EV_COLOR] is
			-- Purple-blue-green gradient of 4 shades
		do
			create Result.make (1, 4)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (246, 239, 247))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (189, 201, 225))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (103, 169, 207))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (2, 129, 138))
		ensure
			purple_blue_greens_4_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_four: Result.upper = 4
		end
	
	purple_blue_greens_5: ARRAY [EV_COLOR] is
			-- Purple-blue-green gradient of 5 shades
		do
			create Result.make (1, 5)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (246, 239, 247))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (189, 201, 225))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (103, 169, 207))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (28, 144, 153))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (1, 108, 89))
		ensure
			purple_blue_greens_5_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_five: Result.upper = 5
		end
	
	purple_blue_greens_6: ARRAY [EV_COLOR] is
			-- Purple-blue-green gradient of 6 shades
		do
			create Result.make (1, 6)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (246, 239, 247))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (208, 209, 230))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (166, 189, 219))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (103, 169, 207))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (28, 144, 153))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (1, 108, 89))
		ensure
			purple_blue_greens_6_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_six: Result.upper = 6
		end
	
	purple_blue_greens_7: ARRAY [EV_COLOR] is
			-- Purple-blue-green gradient of 7 shades
		do
			create Result.make (1, 7)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (246, 239, 247))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (208, 209, 230))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (166, 189, 219))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (103, 169, 207))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (54, 144, 192))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (2, 129, 138))
			Result [7] := (create {EV_COLOR}.make_with_8_bit_rgb (1, 100, 80))
		ensure
			purple_blue_greens_7_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_seven: Result.upper = 7
		end
			
	purple_blue_greens_8: ARRAY [EV_COLOR] is
			-- Purple-blue-green gradient of 8 shades
		do
			create Result.make (1, 8)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (255, 247, 251))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (236, 226, 240))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (208, 209, 230))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (166, 189, 219))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (103, 169, 207))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (54, 144, 192))
			Result [7] := (create {EV_COLOR}.make_with_8_bit_rgb (2, 129, 138))
			Result [8] := (create {EV_COLOR}.make_with_8_bit_rgb (1, 100, 80))
		ensure
			purple_blue_greens_8_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_eight: Result.upper = 8
		end
			
	purple_blue_greens_9: ARRAY [EV_COLOR] is
			-- Purple-blue-green gradient of 9 shades
		do
			create Result.make (1, 9)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (255, 247, 251))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (236, 226, 240))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (208, 209, 230))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (166, 189, 219))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (103, 169, 207))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (54, 144, 192))
			Result [7] := (create {EV_COLOR}.make_with_8_bit_rgb (2, 129, 138))
			Result [8] := (create {EV_COLOR}.make_with_8_bit_rgb (1, 108, 89))
			Result [9] := (create {EV_COLOR}.make_with_8_bit_rgb (1, 70, 54))
		ensure
			purple_blue_greens_9_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_nine: Result.upper = 9
		end
	
	purple_reds_3: ARRAY [EV_COLOR] is
			-- Purple-red gradient of 3 shades
		do
			create Result.make (1, 3)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (231, 225, 239))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (201, 148, 199))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (221, 28, 119))
		ensure
			purple_reds_3_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_three: Result.upper = 3
		end

	purple_reds_4: ARRAY [EV_COLOR] is
			-- Purple-red gradient of 4 shades
		do
			create Result.make (1, 4)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (241, 238, 246))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (215, 181, 216))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (223, 101, 176))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (206, 18, 86))
		ensure
			purple_reds_4_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_four: Result.upper = 4
		end
	
	purple_reds_5: ARRAY [EV_COLOR] is
			-- Purple-red gradient of 5 shades
		do
			create Result.make (1, 5)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (241, 238, 246))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (215, 181, 216))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (223, 101, 176))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (221, 28, 119))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (152, 0, 67))
		ensure
			purple_reds_5_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_five: Result.upper = 5
		end
	
	purple_reds_6: ARRAY [EV_COLOR] is
			-- Purple-red gradient of 6 shades
		do
			create Result.make (1, 6)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (241, 238, 246))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (212, 185, 218))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (201, 148, 199))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (223, 101, 176))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (221, 28, 119))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (152, 0, 67))
		ensure
			purple_reds_6_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_six: Result.upper = 6
		end
	
	purple_reds_7: ARRAY [EV_COLOR] is
			-- Purple-red gradient of 7 shades
		do
			create Result.make (1, 7)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (241, 238, 246))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (212, 185, 218))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (201, 148, 199))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (223, 101, 176))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (231, 41, 138))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (206, 18, 86))
			Result [7] := (create {EV_COLOR}.make_with_8_bit_rgb (145, 0, 63))
		ensure
			purple_reds_7_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_seven: Result.upper = 7
		end
			
	purple_reds_8: ARRAY [EV_COLOR] is
			-- Purple-red gradient of 8 shades
		do
			create Result.make (1, 8)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (247, 244, 249))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (231, 225, 239))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (212, 185, 218))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (201, 148, 199))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (223, 101, 176))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (231, 41, 138))
			Result [7] := (create {EV_COLOR}.make_with_8_bit_rgb (206, 18, 86))
			Result [8] := (create {EV_COLOR}.make_with_8_bit_rgb (145, 0, 63))
		ensure
			purple_reds_8_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_eight: Result.upper = 8
		end
			
	purple_reds_9: ARRAY [EV_COLOR] is
			-- Purple-red gradient of 9 shades
		do
			create Result.make (1, 9)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (247, 244, 249))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (231, 225, 239))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (212, 185, 218))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (201, 148, 199))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (223, 101, 176))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (231, 41, 138))
			Result [7] := (create {EV_COLOR}.make_with_8_bit_rgb (206, 18, 86))
			Result [8] := (create {EV_COLOR}.make_with_8_bit_rgb (152, 0, 67))
			Result [9] := (create {EV_COLOR}.make_with_8_bit_rgb (103, 0, 31))
		ensure
			purple_reds_9_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_nine: Result.upper = 9
		end
	
	red_purples_3: ARRAY [EV_COLOR] is
			-- Red-purple gradient of 3 shades
		do
			create Result.make (1, 3)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (253, 224, 221))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (250, 159, 181))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (197, 27, 138))
		ensure
			red_purples_3_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_three: Result.upper = 3
		end

	red_purples_4: ARRAY [EV_COLOR] is
			-- Red-purple gradient of 4 shades
		do
			create Result.make (1, 4)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (254, 235, 226))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (251, 180, 185))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (247, 104, 161))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (174, 1, 126))
		ensure
			red_purples_4_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_four: Result.upper = 4
		end
	
	red_purples_5: ARRAY [EV_COLOR] is
			-- Red-purple gradient of 5 shades
		do
			create Result.make (1, 5)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (254, 235, 226))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (251, 180, 185))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (247, 104, 161))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (197, 27, 138))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (122, 1, 119))
		ensure
			red_purples_5_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_five: Result.upper = 5
		end
	
	red_purples_6: ARRAY [EV_COLOR] is
			-- Red-purple gradient of 6 shades
		do
			create Result.make (1, 6)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (254, 235, 226))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (252, 197, 192))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (250, 159, 181))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (247, 104, 161))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (197, 27, 138))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (122, 1, 119))
		ensure
			red_purples_6_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_six: Result.upper = 6
		end
	
	red_purples_7: ARRAY [EV_COLOR] is
			-- Red-purple gradient of 7 shades
		do
			create Result.make (1, 7)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (254, 235, 226))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (252, 197, 192))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (250, 159, 181))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (247, 104, 161))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (221, 52, 151))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (174, 1, 126))
			Result [7] := (create {EV_COLOR}.make_with_8_bit_rgb (122, 1, 119))
		ensure
			red_purples_7_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_seven: Result.upper = 7
		end
			
	red_purples_8: ARRAY [EV_COLOR] is
			-- Red-purple gradient of 8 shades
		do
			create Result.make (1, 8)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (255, 247, 243))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (253, 224, 221))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (252, 197, 192))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (250, 159, 181))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (247, 104, 161))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (221, 52, 151))
			Result [7] := (create {EV_COLOR}.make_with_8_bit_rgb (174, 1, 126))
			Result [8] := (create {EV_COLOR}.make_with_8_bit_rgb (122, 1, 119))
		ensure
			red_purples_8_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_eight: Result.upper = 8
		end
			
	red_purples_9: ARRAY [EV_COLOR] is
			-- Red-purple gradient of 9 shades
		do
			create Result.make (1, 9)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (255, 247, 243))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (253, 224, 221))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (252, 197, 192))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (250, 159, 181))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (247, 104, 161))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (221, 52, 151))
			Result [7] := (create {EV_COLOR}.make_with_8_bit_rgb (174, 1, 126))
			Result [8] := (create {EV_COLOR}.make_with_8_bit_rgb (122, 1, 119))			
			Result [9] := (create {EV_COLOR}.make_with_8_bit_rgb (73, 0, 106))
		ensure
			red_purples_9_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_nine: Result.upper = 9
		end
	
	yellow_greens_3: ARRAY [EV_COLOR] is
			-- Yellow-green gradient of 3 shades
		do
			create Result.make (1, 3)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (247, 252, 185))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (173, 221, 142))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (49, 163, 84))
		ensure
			yellow_greens_3_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_three: Result.upper = 3
		end

	yellow_greens_4: ARRAY [EV_COLOR] is
			-- Yellow-green gradient of 4 shades
		do
			create Result.make (1, 4)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (255, 255, 204))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (194, 230, 153))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (120, 198, 121))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (35, 132, 67))
		ensure
			yellow_greens_4_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_four: Result.upper = 4
		end
	
	yellow_greens_5: ARRAY [EV_COLOR] is
			-- Yellow-green gradient of 5 shades
		do
			create Result.make (1, 5)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (255, 255, 204))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (194, 230, 153))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (120, 198, 121))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (49, 163, 84))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (0, 104, 55))
		ensure
			yellow_greens_5_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_five: Result.upper = 5
		end
	
	yellow_greens_6: ARRAY [EV_COLOR] is
			-- Yellow-green gradient of 6 shades
		do
			create Result.make (1, 6)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (255, 255, 204))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (217, 240, 163))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (173, 221, 142))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (120, 198, 121))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (49, 163, 84))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (0, 104, 55))
		ensure
			yellow_greens_6_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_six: Result.upper = 6
		end
	
	yellow_greens_7: ARRAY [EV_COLOR] is
			-- Yellow-green gradient of 7 shades
		do
			create Result.make (1, 7)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (255, 255, 204))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (217, 240, 163))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (173, 221, 142))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (120, 198, 121))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (65, 171, 93))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (35, 132, 67))
			Result [7] := (create {EV_COLOR}.make_with_8_bit_rgb (0, 90, 50))
		ensure
			yellow_greens_7_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_seven: Result.upper = 7
		end
			
	yellow_greens_8: ARRAY [EV_COLOR] is
			-- Yellow-green gradient of 8 shades
		do
			create Result.make (1, 8)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (255, 255, 229))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (247, 252, 185))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (217, 240, 163))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (173, 221, 142))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (120, 198, 121))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (65, 171, 93))
			Result [7] := (create {EV_COLOR}.make_with_8_bit_rgb (35, 132, 67))
			Result [8] := (create {EV_COLOR}.make_with_8_bit_rgb (0, 90, 50))
		ensure
			yellow_greens_8_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_eight: Result.upper = 8
		end
			
	yellow_greens_9: ARRAY [EV_COLOR] is
			-- Yellow-green gradient of 9 shades
		do
			create Result.make (1, 9)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (255, 255, 229))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (247, 252, 185))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (217, 240, 163))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (173, 221, 142))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (120, 198, 121))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (65, 171, 93))
			Result [7] := (create {EV_COLOR}.make_with_8_bit_rgb (35, 132, 67))
			Result [8] := (create {EV_COLOR}.make_with_8_bit_rgb (0, 104, 55))
			Result [9] := (create {EV_COLOR}.make_with_8_bit_rgb (0, 69, 41))
		ensure
			yellow_greens_9_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_nine: Result.upper = 9
		end
	
	yellow_green_blues_3: ARRAY [EV_COLOR] is
			-- Yellow-green-blue gradient of 3 shades
		do
			create Result.make (1, 3)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (237, 248, 177))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (127, 205, 187))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (44, 127, 184))
		ensure
			yellow_green_blues_3_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_three: Result.upper = 3
		end

	yellow_green_blues_4: ARRAY [EV_COLOR] is
			-- Yellow-green-blue gradient of 4 shades
		do
			create Result.make (1, 4)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (255, 255, 204))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (161, 218, 180))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (65, 182, 196))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (34, 94, 168))
		ensure
			yellow_green_blues_4_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_four: Result.upper = 4
		end
	
	yellow_green_blues_5: ARRAY [EV_COLOR] is
			-- Yellow-green-blue gradient of 5 shades
		do
			create Result.make (1, 5)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (255, 255, 204))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (161, 218, 180))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (65, 182, 196))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (44, 127, 184))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (37, 52, 148))
		ensure
			yellow_green_blues_5_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_five: Result.upper = 5
		end
	
	yellow_green_blues_6: ARRAY [EV_COLOR] is
			-- Yellow-green-blue gradient of 6 shades
		do
			create Result.make (1, 6)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (255, 255, 204))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (199, 233, 180))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (127, 205, 187))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (65, 182, 196))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (44, 127, 184))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (37, 52, 148))
		ensure
			yellow_green_blues_6_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_six: Result.upper = 6
		end
	
	yellow_green_blues_7: ARRAY [EV_COLOR] is
			-- Yellow-green-blue gradient of 7 shades
		do
			create Result.make (1, 7)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (255, 255, 204))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (199, 233, 180))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (127, 205, 187))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (65, 182, 196))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (29, 145, 192))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (34, 94, 168))
			Result [7] := (create {EV_COLOR}.make_with_8_bit_rgb (12, 44, 132))
		ensure
			yellow_green_blues_7_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_seven: Result.upper = 7
		end
			
	yellow_green_blues_8: ARRAY [EV_COLOR] is
			-- Yellow-green-blue gradient of 8 shades
		do
			create Result.make (1, 8)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (255, 255, 217))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (237, 248, 177))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (199, 233, 180))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (127, 205, 187))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (65, 182, 196))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (29, 145, 192))
			Result [7] := (create {EV_COLOR}.make_with_8_bit_rgb (34, 94, 168))
			Result [8] := (create {EV_COLOR}.make_with_8_bit_rgb (12, 44, 132))
		ensure
			yellow_green_blues_8_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_eight: Result.upper = 8
		end
			
	yellow_green_blues_9: ARRAY [EV_COLOR] is
			-- Yellow-green-blue gradient of 9 shades
		do
			create Result.make (1, 9)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (255, 255, 217))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (237, 248, 177))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (199, 233, 180))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (127, 205, 187))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (65, 182, 196))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (29, 145, 192))
			Result [7] := (create {EV_COLOR}.make_with_8_bit_rgb (34, 94, 168))
			Result [8] := (create {EV_COLOR}.make_with_8_bit_rgb (37, 52, 148))
			Result [9] := (create {EV_COLOR}.make_with_8_bit_rgb (8, 29, 88))
		ensure
			yellow_green_blues_9_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_nine: Result.upper = 9
		end
	
	yellow_orange_browns_3: ARRAY [EV_COLOR] is
			-- Yellow-orange-brown gradient of 3 shades
		do
			create Result.make (1, 3)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (255, 247, 188))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (254, 196, 79))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (217, 95, 14))
		ensure
			yellow_orange_browns_3_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_three: Result.upper = 3
		end

	yellow_orange_browns_4: ARRAY [EV_COLOR] is
			-- Yellow-orange-brown gradient of 4 shades
		do
			create Result.make (1, 4)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (255, 255, 212))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (254, 217, 142))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (254, 153, 41))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (204, 76, 2))
		ensure
			yellow_orange_browns_4_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_four: Result.upper = 4
		end
	
	yellow_orange_browns_5: ARRAY [EV_COLOR] is
			-- Yellow-orange-brown gradient of 5 shades
		do
			create Result.make (1, 5)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (255, 255, 212))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (254, 217, 142))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (254, 153, 41))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (217, 95, 14))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (153, 52, 4))
		ensure
			yellow_orange_browns_5_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_five: Result.upper = 5
		end
	
	yellow_orange_browns_6: ARRAY [EV_COLOR] is
			-- Yellow-orange-brown gradient of 6 shades
		do
			create Result.make (1, 6)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (255, 255, 212))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (254, 227, 145))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (254, 196, 79))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (254, 153, 41))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (217, 95, 14))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (153, 52, 4))
		ensure
			yellow_orange_browns_6_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_six: Result.upper = 6
		end
	
	yellow_orange_browns_7: ARRAY [EV_COLOR] is
			-- Yellow-orange-brown gradient of 7 shades
		do
			create Result.make (1, 7)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (255, 255, 212))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (254, 227, 145))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (254, 196, 79))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (254, 153, 41))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (236, 112, 20))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (204, 76, 2))
			Result [7] := (create {EV_COLOR}.make_with_8_bit_rgb (140, 45, 4))
		ensure
			yellow_orange_browns_7_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_seven: Result.upper = 7
		end
			
	yellow_orange_browns_8: ARRAY [EV_COLOR] is
			-- Yellow-orange-brown gradient of 8 shades
		do
			create Result.make (1, 8)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (255, 255, 229))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (255, 247, 188))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (254, 227, 145))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (254, 196, 79))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (254, 153, 41))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (236, 112, 20))
			Result [7] := (create {EV_COLOR}.make_with_8_bit_rgb (204, 76, 2))
			Result [8] := (create {EV_COLOR}.make_with_8_bit_rgb (140, 45, 4))
		ensure
			yellow_orange_browns_8_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_eight: Result.upper = 8
		end
			
	yellow_orange_browns_9: ARRAY [EV_COLOR] is
			-- Yellow-orange-brown gradient of 9 shades
		do
			create Result.make (1, 9)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (255, 255, 229))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (255, 247, 188))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (254, 227, 145))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (254, 196, 79))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (254, 153, 41))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (236, 112, 20))
			Result [7] := (create {EV_COLOR}.make_with_8_bit_rgb (204, 76, 2))
			Result [8] := (create {EV_COLOR}.make_with_8_bit_rgb (153, 52, 4))
			Result [9] := (create {EV_COLOR}.make_with_8_bit_rgb (102, 37, 6))
		ensure
			yellow_orange_browns_9_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_nine: Result.upper = 9
		end
	
	yellow_orange_reds_3: ARRAY [EV_COLOR] is
			-- Yellow-orange-red gradient of 3 shades
		do
			create Result.make (1, 3)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (255, 237, 160))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (254, 178, 76))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (240, 59, 32))
		ensure
			yellow_orange_reds_3_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_three: Result.upper = 3
		end

	yellow_orange_reds_4: ARRAY [EV_COLOR] is
			-- Yellow-orange-red gradient of 4 shades
		do
			create Result.make (1, 4)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (255, 255, 178))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (254, 204, 92))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (253, 141, 60))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (227, 26, 28))
		ensure
			yellow_orange_reds_4_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_four: Result.upper = 4
		end
	
	yellow_orange_reds_5: ARRAY [EV_COLOR] is
			-- Yellow-orange-red gradient of 5 shades
		do
			create Result.make (1, 5)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (255, 255, 178))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (254, 204, 92))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (253, 141, 60))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (240, 59, 32))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (189, 0, 38))
		ensure
			yellow_orange_reds_5_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_five: Result.upper = 5
		end
	
	yellow_orange_reds_6: ARRAY [EV_COLOR] is
			-- Yellow-orange-red gradient of 6 shades
		do
			create Result.make (1, 6)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (255, 255, 178))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (254, 217, 118))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (254, 178, 76))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (253, 141, 60))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (240, 59, 32))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (189, 0, 38))
		ensure
			yellow_orange_reds_6_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_six: Result.upper = 6
		end
	
	yellow_orange_reds_7: ARRAY [EV_COLOR] is
			-- Yellow-orange-red gradient of 7 shades
		do
			create Result.make (1, 7)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (255, 255, 178))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (254, 217, 118))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (254, 178, 76))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (253, 141, 60))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (252, 78, 42))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (227, 26, 28))
			Result [7] := (create {EV_COLOR}.make_with_8_bit_rgb (177, 0, 38))
		ensure
			yellow_orange_reds_7_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_seven: Result.upper = 7
		end
			
	yellow_orange_reds_8: ARRAY [EV_COLOR] is
			-- Yellow-orange-red gradient of 8 shades
		do
			create Result.make (1, 8)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (255, 255, 204))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (255, 237, 160))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (254, 217, 118))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (254, 178, 76))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (253, 141, 60))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (252, 78, 42))
			Result [7] := (create {EV_COLOR}.make_with_8_bit_rgb (227, 26, 28))
			Result [8] := (create {EV_COLOR}.make_with_8_bit_rgb (177, 0, 38))
		ensure
			yellow_orange_reds_8_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_eight: Result.upper = 8
		end
			
	yellow_orange_reds_9: ARRAY [EV_COLOR] is
			-- Yellow-orange-red gradient of 9 shades
		do
			create Result.make (1, 9)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (255, 255, 204))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (255, 237, 160))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (254, 217, 118))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (254, 178, 76))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (253, 141, 60))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (252, 78, 42))
			Result [7] := (create {EV_COLOR}.make_with_8_bit_rgb (227, 26, 28))
			Result [8] := (create {EV_COLOR}.make_with_8_bit_rgb (189, 0, 38))
			Result [9] := (create {EV_COLOR}.make_with_8_bit_rgb (128, 0, 38))
		ensure
			yellow_orange_reds_9_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_nine: Result.upper = 9
		end
		
feature -- Diverging color gradients
	
	brown_blue_greens (a_classes: NATURAL_8): ARRAY [EV_COLOR] is
			-- Brown-blue-green gradient of `a_classes' shades
		require
			a_classes_large_enough: a_classes >= 3
			a_classes_small_enough: a_classes <= 11
		do
			inspect
				a_classes
			when 3 then
				Result := brown_blue_greens_3
			when 4 then
				Result := brown_blue_greens_4
			when 5 then
				Result := brown_blue_greens_5
			when 6 then
				Result := brown_blue_greens_6
			when 7 then
				Result := brown_blue_greens_7
			when 8 then
				Result := brown_blue_greens_8
			when 9 then
				Result := brown_blue_greens_9
			when 10 then
				Result := brown_blue_greens_10
			when 11 then
				Result := brown_blue_greens_11
			end
		ensure
			brown_blue_greens_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_a_classes: Result.upper = a_classes
		end
	
	pink_yellow_greens (a_classes: NATURAL_8): ARRAY [EV_COLOR] is
			-- Pink-yellow-green gradient of `a_classes' shades
		require
			a_classes_large_enough: a_classes >= 3
			a_classes_small_enough: a_classes <= 11
		do
			inspect
				a_classes
			when 3 then
				Result := pink_yellow_greens_3
			when 4 then
				Result := pink_yellow_greens_4
			when 5 then
				Result := pink_yellow_greens_5
			when 6 then
				Result := pink_yellow_greens_6
			when 7 then
				Result := pink_yellow_greens_7
			when 8 then
				Result := pink_yellow_greens_8
			when 9 then
				Result := pink_yellow_greens_9
			when 10 then
				Result := pink_yellow_greens_10
			when 11 then
				Result := pink_yellow_greens_11
			end
		ensure
			pink_yellow_greens_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_a_classes: Result.upper = a_classes
		end
	
	purple_greens (a_classes: NATURAL_8): ARRAY [EV_COLOR] is
			-- Purple-green gradient of `a_classes' shades
		require
			a_classes_large_enough: a_classes >= 3
			a_classes_small_enough: a_classes <= 11
		do
			inspect
				a_classes
			when 3 then
				Result := purple_greens_3
			when 4 then
				Result := purple_greens_4
			when 5 then
				Result := purple_greens_5
			when 6 then
				Result := purple_greens_6
			when 7 then
				Result := purple_greens_7
			when 8 then
				Result := purple_greens_8
			when 9 then
				Result := purple_greens_9
			when 10 then
				Result := purple_greens_10
			when 11 then
				Result := purple_greens_11
			end
		ensure
			purple_greens_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_a_classes: Result.upper = a_classes
		end
	
	purple_oranges (a_classes: NATURAL_8): ARRAY [EV_COLOR] is
			-- Purple-orange gradient of `a_classes' shades
		require
			a_classes_large_enough: a_classes >= 3
			a_classes_small_enough: a_classes <= 11
		do
			inspect
				a_classes
			when 3 then
				Result := purple_oranges_3
			when 4 then
				Result := purple_oranges_4
			when 5 then
				Result := purple_oranges_5
			when 6 then
				Result := purple_oranges_6
			when 7 then
				Result := purple_oranges_7
			when 8 then
				Result := purple_oranges_8
			when 9 then
				Result := purple_oranges_9
			when 10 then
				Result := purple_oranges_10
			when 11 then
				Result := purple_oranges_11
			end
		ensure
			purple_oranges_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_a_classes: Result.upper = a_classes
		end
	
	red_blues (a_classes: NATURAL_8): ARRAY [EV_COLOR] is
			-- Red-blue gradient of `a_classes' shades
		require
			a_classes_large_enough: a_classes >= 3
			a_classes_small_enough: a_classes <= 11
		do
			inspect
				a_classes
			when 3 then
				Result := red_blues_3
			when 4 then
				Result := red_blues_4
			when 5 then
				Result := red_blues_5
			when 6 then
				Result := red_blues_6
			when 7 then
				Result := red_blues_7
			when 8 then
				Result := red_blues_8
			when 9 then
				Result := red_blues_9
			when 10 then
				Result := red_blues_10
			when 11 then
				Result := red_blues_11
			end
		ensure
			red_blues_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_a_classes: Result.upper = a_classes
		end
	
	red_greys (a_classes: NATURAL_8): ARRAY [EV_COLOR] is
			-- Red-grey gradient of `a_classes' shades
		require
			a_classes_large_enough: a_classes >= 3
			a_classes_small_enough: a_classes <= 11
		do
			inspect
				a_classes
			when 3 then
				Result := red_greys_3
			when 4 then
				Result := red_greys_4
			when 5 then
				Result := red_greys_5
			when 6 then
				Result := red_greys_6
			when 7 then
				Result := red_greys_7
			when 8 then
				Result := red_greys_8
			when 9 then
				Result := red_greys_9
			when 10 then
				Result := red_greys_10
			when 11 then
				Result := red_greys_11
			end
		ensure
			red_greys_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_a_classes: Result.upper = a_classes
		end
	
	red_yellow_blues (a_classes: NATURAL_8): ARRAY [EV_COLOR] is
			-- Red-yellow-blue gradient of `a_classes' shades
		require
			a_classes_large_enough: a_classes >= 3
			a_classes_small_enough: a_classes <= 11
		do
			inspect
				a_classes
			when 3 then
				Result := red_yellow_blues_3
			when 4 then
				Result := red_yellow_blues_4
			when 5 then
				Result := red_yellow_blues_5
			when 6 then
				Result := red_yellow_blues_6
			when 7 then
				Result := red_yellow_blues_7
			when 8 then
				Result := red_yellow_blues_8
			when 9 then
				Result := red_yellow_blues_9
			when 10 then
				Result := red_yellow_blues_10
			when 11 then
				Result := red_yellow_blues_11
			end
		ensure
			red_yellow_blues_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_a_classes: Result.upper = a_classes
		end
	
	red_yellow_greens (a_classes: NATURAL_8): ARRAY [EV_COLOR] is
			-- Red-yellow-green gradient of `a_classes' shades
		require
			a_classes_large_enough: a_classes >= 3
			a_classes_small_enough: a_classes <= 11
		do
			inspect
				a_classes
			when 3 then
				Result := red_yellow_greens_3
			when 4 then
				Result := red_yellow_greens_4
			when 5 then
				Result := red_yellow_greens_5
			when 6 then
				Result := red_yellow_greens_6
			when 7 then
				Result := red_yellow_greens_7
			when 8 then
				Result := red_yellow_greens_8
			when 9 then
				Result := red_yellow_greens_9
			when 10 then
				Result := red_yellow_greens_10
			when 11 then
				Result := red_yellow_greens_11
			end
		ensure
			red_yellow_greens_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_a_classes: Result.upper = a_classes
		end
	
	spectral (a_classes: NATURAL_8): ARRAY [EV_COLOR] is
			-- Spectral gradient of `a_classes' shades
		require
			a_classes_large_enough: a_classes >= 3
			a_classes_small_enough: a_classes <= 11
		do
			inspect
				a_classes
			when 3 then
				Result := spectral_3
			when 4 then
				Result := spectral_4
			when 5 then
				Result := spectral_5
			when 6 then
				Result := spectral_6
			when 7 then
				Result := spectral_7
			when 8 then
				Result := spectral_8
			when 9 then
				Result := spectral_9
			when 10 then
				Result := spectral_10
			when 11 then
				Result := spectral_11
			end
		ensure
			spectral_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_a_classes: Result.upper = a_classes
		end
	
	brown_blue_greens_3: ARRAY [EV_COLOR] is
			-- Brown-blue-green gradient of 3 shades
		do
			create Result.make (1, 3)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (216, 179, 101))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (245, 245, 245))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (90, 180, 172))
		ensure
			brown_blue_greens_3_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_three: Result.upper = 3
		end

	brown_blue_greens_4: ARRAY [EV_COLOR] is
			-- Brown-blue-green gradient of 4 shades
		do
			create Result.make (1, 4)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (166, 97, 26))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (223, 194, 125))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (128, 205, 193))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (1, 133, 113))
		ensure
			brown_blue_greens_4_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_four: Result.upper = 4
		end
	
	brown_blue_greens_5: ARRAY [EV_COLOR] is
			-- Brown-blue-green gradient of 5 shades
		do
			create Result.make (1, 5)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (166, 97, 26))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (223, 194, 125))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (245, 245, 245))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (128, 205, 193))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (1, 133, 113))
		ensure
			brown_blue_greens_5_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_five: Result.upper = 5
		end
	
	brown_blue_greens_6: ARRAY [EV_COLOR] is
			-- Brown-blue-green gradient of 6 shades
		do
			create Result.make (1, 6)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (140, 81, 10))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (216, 179, 101))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (246, 232, 195))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (199, 234, 229))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (90, 180, 172))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (1, 102, 94))
		ensure
			brown_blue_greens_6_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_six: Result.upper = 6
		end
	
	brown_blue_greens_7: ARRAY [EV_COLOR] is
			-- Brown-blue-green gradient of 7 shades
		do
			create Result.make (1, 7)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (140, 81, 10))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (216, 179, 101))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (246, 232, 195))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (245, 245, 245))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (199, 234, 229))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (90, 180, 172))
			Result [7] := (create {EV_COLOR}.make_with_8_bit_rgb (1, 102, 94))
		ensure
			brown_blue_greens_7_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_seven: Result.upper = 7
		end
			
	brown_blue_greens_8: ARRAY [EV_COLOR] is
			-- Brown-blue-green gradient of 8 shades
		do
			create Result.make (1, 8)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (140, 81, 10))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (191, 129, 45))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (223, 129, 125))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (246, 232, 195))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (199, 234, 229))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (128, 205, 193))
			Result [7] := (create {EV_COLOR}.make_with_8_bit_rgb (53, 151, 143))
			Result [8] := (create {EV_COLOR}.make_with_8_bit_rgb (1, 102, 94))
		ensure
			brown_blue_greens_8_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_eight: Result.upper = 8
		end
			
	brown_blue_greens_9: ARRAY [EV_COLOR] is
			-- Brown-blue-green gradient of 9 shades
		do
			create Result.make (1, 9)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (140, 81, 10))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (191, 129, 45))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (223, 129, 125))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (246, 232, 195))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (245, 245, 245))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (199, 234, 229))
			Result [7] := (create {EV_COLOR}.make_with_8_bit_rgb (128, 205, 193))
			Result [8] := (create {EV_COLOR}.make_with_8_bit_rgb (53, 151, 143))
			Result [9] := (create {EV_COLOR}.make_with_8_bit_rgb (1, 102, 94))
		ensure
			brown_blue_greens_9_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_nine: Result.upper = 9
		end
			
	brown_blue_greens_10: ARRAY [EV_COLOR] is
			-- Brown-blue-green gradient of 10 shades
		do
			create Result.make (1, 10)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (84, 48, 5))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (140, 81, 10))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (191, 129, 45))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (223, 129, 125))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (246, 232, 195))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (199, 234, 229))
			Result [7] := (create {EV_COLOR}.make_with_8_bit_rgb (128, 205, 193))
			Result [8] := (create {EV_COLOR}.make_with_8_bit_rgb (53, 151, 143))
			Result [9] := (create {EV_COLOR}.make_with_8_bit_rgb (1, 102, 94))
			Result [10] := (create {EV_COLOR}.make_with_8_bit_rgb (6, 60,  8))
		ensure
			brown_blue_greens_10_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_ten: Result.upper = 10
		end

	brown_blue_greens_11: ARRAY [EV_COLOR] is
			-- Brown-blue-green gradient of 11 shades
		do
			create Result.make (1, 11)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (84, 48, 5))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (140, 81, 10))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (191, 129, 45))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (223, 129, 125))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (246, 232, 195))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (245, 245, 245))
			Result [7] := (create {EV_COLOR}.make_with_8_bit_rgb (199, 234, 229))
			Result [8] := (create {EV_COLOR}.make_with_8_bit_rgb (128, 205, 193))
			Result [9] := (create {EV_COLOR}.make_with_8_bit_rgb (53, 151, 143))
			Result [10] := (create {EV_COLOR}.make_with_8_bit_rgb (1, 102, 94))
			Result [11] := (create {EV_COLOR}.make_with_8_bit_rgb (6, 60,  8))
		ensure
			brown_blue_greens_11_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_eleven: Result.upper = 11
		end
	
	pink_yellow_greens_3: ARRAY [EV_COLOR] is
			-- Pink-yellow-green gradient of 3 shades
		do
			create Result.make (1, 3)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (233, 163, 201))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (247, 247, 247))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (161, 215, 106))
		ensure
			pink_yellow_greens_3_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_three: Result.upper = 3
		end

	pink_yellow_greens_4: ARRAY [EV_COLOR] is
			-- Pink-yellow-green gradient of 4 shades
		do
			create Result.make (1, 4)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (208, 28, 139))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (241, 182, 218))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (184, 225, 134))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (77, 172, 38))
		ensure
			pink_yellow_greens_4_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_four: Result.upper = 4
		end
	
	pink_yellow_greens_5: ARRAY [EV_COLOR] is
			-- Pink-yellow-green gradient of 5 shades
		do
			create Result.make (1, 5)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (208, 28, 139))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (241, 182, 218))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (247, 247, 247))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (184, 225, 134))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (77, 172, 38))
		ensure
			pink_yellow_greens_5_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_five: Result.upper = 5
		end
	
	pink_yellow_greens_6: ARRAY [EV_COLOR] is
			-- Pink-yellow-green gradient of 6 shades
		do
			create Result.make (1, 6)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (197, 27, 125))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (233, 163, 201))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (253, 224, 239))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (230, 245, 208))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (161, 215, 106))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (77, 146, 33))
		ensure
			pink_yellow_greens_6_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_six: Result.upper = 6
		end
	
	pink_yellow_greens_7: ARRAY [EV_COLOR] is
			-- Pink-yellow-green gradient of 7 shades
		do
			create Result.make (1, 7)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (197, 27, 125))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (233, 163, 201))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (253, 224, 239))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (247, 247, 247))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (230, 245, 208))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (161, 215, 106))
			Result [7] := (create {EV_COLOR}.make_with_8_bit_rgb (77, 146, 33))
		ensure
			pink_yellow_greens_7_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_seven: Result.upper = 7
		end
			
	pink_yellow_greens_8: ARRAY [EV_COLOR] is
			-- Pink-yellow-green gradient of 8 shades
		do
			create Result.make (1, 8)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (197, 27, 125))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (222, 119, 174))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (241, 182, 218))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (253, 224, 239))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (230, 245, 208))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (184, 225, 134))
			Result [7] := (create {EV_COLOR}.make_with_8_bit_rgb (127, 188, 65))
			Result [8] := (create {EV_COLOR}.make_with_8_bit_rgb (77, 146, 33))
		ensure
			pink_yellow_greens_8_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_eight: Result.upper = 8
		end
			
	pink_yellow_greens_9: ARRAY [EV_COLOR] is
			-- Pink-yellow-green gradient of 9 shades
		do
			create Result.make (1, 9)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (197, 27, 125))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (222, 119, 174))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (241, 182, 218))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (253, 224, 239))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (247, 247, 247))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (230, 245, 208))
			Result [7] := (create {EV_COLOR}.make_with_8_bit_rgb (184, 225, 134))
			Result [8] := (create {EV_COLOR}.make_with_8_bit_rgb (127, 188, 65))
			Result [9] := (create {EV_COLOR}.make_with_8_bit_rgb (77, 146, 33))
		ensure
			pink_yellow_greens_9_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_nine: Result.upper = 9
		end
			
	pink_yellow_greens_10: ARRAY [EV_COLOR] is
			-- Pink-yellow-green gradient of 10 shades
		do
			create Result.make (1, 10)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (142, 1, 82))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (197, 27, 125))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (222, 119, 174))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (241, 182, 218))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (253, 224, 239))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (230, 245, 208))
			Result [7] := (create {EV_COLOR}.make_with_8_bit_rgb (184, 225, 134))
			Result [8] := (create {EV_COLOR}.make_with_8_bit_rgb (127, 188, 65))
			Result [9] := (create {EV_COLOR}.make_with_8_bit_rgb (77, 146, 33))
			Result [10] := (create {EV_COLOR}.make_with_8_bit_rgb (39, 100, 25))			
		ensure
			pink_yellow_greens_10_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_ten: Result.upper = 10
		end

	pink_yellow_greens_11: ARRAY [EV_COLOR] is
			-- Pink-yellow-green gradient of 11 shades
		do
			create Result.make (1, 11)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (142, 1, 82))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (197, 27, 125))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (222, 119, 174))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (241, 182, 218))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (253, 224, 239))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (247, 247, 247))
			Result [7] := (create {EV_COLOR}.make_with_8_bit_rgb (230, 245, 208))
			Result [8] := (create {EV_COLOR}.make_with_8_bit_rgb (184, 225, 134))
			Result [9] := (create {EV_COLOR}.make_with_8_bit_rgb (127, 188, 65))
			Result [10] := (create {EV_COLOR}.make_with_8_bit_rgb (77, 146, 33))
			Result [11] := (create {EV_COLOR}.make_with_8_bit_rgb (39, 100, 25))	
		ensure
			pink_yellow_greens_11_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_eleven: Result.upper = 11
		end
	
	purple_greens_3: ARRAY [EV_COLOR] is
			-- Purple-green gradient of 3 shades
		do
			create Result.make (1, 3)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (175, 141, 195))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (247, 247, 247))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (127, 191, 123))
		ensure
			purple_greens_3_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_three: Result.upper = 3
		end

	purple_greens_4: ARRAY [EV_COLOR] is
			-- Purple-green gradient of 4 shades
		do
			create Result.make (1, 4)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (123, 50, 148))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (194, 165, 207))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (166, 219, 160))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (0, 136, 55))
		ensure
			purple_greens_4_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_four: Result.upper = 4
		end
	
	purple_greens_5: ARRAY [EV_COLOR] is
			-- Purple-green gradient of 5 shades
		do
			create Result.make (1, 5)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (123, 50, 148))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (194, 165, 207))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (247, 247, 247))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (166, 219, 160))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (0, 136, 55))
		ensure
			purple_greens_5_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_five: Result.upper = 5
		end
	
	purple_greens_6: ARRAY [EV_COLOR] is
			-- Purple-green gradient of 6 shades
		do
			create Result.make (1, 6)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (118, 42, 131))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (175, 141, 195))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (231, 212, 232))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (217, 240, 211))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (127, 191, 123))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (27, 120, 55))
		ensure
			purple_greens_6_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_six: Result.upper = 6
		end
	
	purple_greens_7: ARRAY [EV_COLOR] is
			-- Purple-green gradient of 7 shades
		do
			create Result.make (1, 7)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (118, 42, 131))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (175, 141, 195))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (231, 212, 232))			
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (247, 247, 247))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (217, 240, 211))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (127, 191, 123))
			Result [7] := (create {EV_COLOR}.make_with_8_bit_rgb (27, 120, 55))
		ensure
			purple_greens_7_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_seven: Result.upper = 7
		end
			
	purple_greens_8: ARRAY [EV_COLOR] is
			-- Purple-green gradient of 8 shades
		do
			create Result.make (1, 8)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (118, 42, 131))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (153, 112, 171))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (194, 165, 207))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (231, 212, 232))			
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (217, 240, 211))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (166, 219, 160))
			Result [7] := (create {EV_COLOR}.make_with_8_bit_rgb (90, 174, 97))
			Result [8] := (create {EV_COLOR}.make_with_8_bit_rgb (27, 120, 55))
		ensure
			purple_greens_8_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_eight: Result.upper = 8
		end
			
	purple_greens_9: ARRAY [EV_COLOR] is
			-- Purple-green gradient of 9 shades
		do
			create Result.make (1, 9)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (118, 42, 131))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (153, 112, 171))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (194, 165, 207))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (231, 212, 232))			
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (247, 247, 247))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (217, 240, 211))
			Result [7] := (create {EV_COLOR}.make_with_8_bit_rgb (166, 219, 160))
			Result [8] := (create {EV_COLOR}.make_with_8_bit_rgb (90, 174, 97))
			Result [9] := (create {EV_COLOR}.make_with_8_bit_rgb (27, 120, 55))
		ensure
			purple_greens_9_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_nine: Result.upper = 9
		end
			
	purple_greens_10: ARRAY [EV_COLOR] is
			-- Purple-green gradient of 10 shades
		do
			create Result.make (1, 10)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (64, 0, 75))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (118, 42, 131))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (153, 112, 171))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (194, 165, 207))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (231, 212, 232))			
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (217, 240, 211))
			Result [7] := (create {EV_COLOR}.make_with_8_bit_rgb (166, 219, 160))
			Result [8] := (create {EV_COLOR}.make_with_8_bit_rgb (90, 174, 97))
			Result [9] := (create {EV_COLOR}.make_with_8_bit_rgb (27, 120, 55))
			Result [10] := (create {EV_COLOR}.make_with_8_bit_rgb (0, 68, 27))
		ensure
			purple_greens_10_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_ten: Result.upper = 10
		end

	purple_greens_11: ARRAY [EV_COLOR] is
			-- Purple-green gradient of 11 shades
		do
			create Result.make (1, 11)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (64, 0, 75))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (118, 42, 131))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (153, 112, 171))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (194, 165, 207))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (231, 212, 232))			
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (247, 247, 247))
			Result [7] := (create {EV_COLOR}.make_with_8_bit_rgb (217, 240, 211))
			Result [8] := (create {EV_COLOR}.make_with_8_bit_rgb (166, 219, 160))
			Result [9] := (create {EV_COLOR}.make_with_8_bit_rgb (90, 174, 97))
			Result [10] := (create {EV_COLOR}.make_with_8_bit_rgb (27, 120, 55))
			Result [11] := (create {EV_COLOR}.make_with_8_bit_rgb (0, 68, 27))
		ensure
			purple_greens_11_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_eleven: Result.upper = 11
		end
	
	purple_oranges_3: ARRAY [EV_COLOR] is
			-- Purple-orange gradient of 3 shades
		do
			create Result.make (1, 3)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (241, 163, 64))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (247, 247, 247))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (153, 142, 195))
		ensure
			purple_oranges_3_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_three: Result.upper = 3
		end

	purple_oranges_4: ARRAY [EV_COLOR] is
			-- Purple-orange gradient of 4 shades
		do
			create Result.make (1, 4)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (230, 97, 1))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (253, 184, 99))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (178, 171, 210))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (94, 60, 153))
		ensure
			purple_oranges_4_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_four: Result.upper = 4
		end
	
	purple_oranges_5: ARRAY [EV_COLOR] is
			-- Purple-orange gradient of 5 shades
		do
			create Result.make (1, 5)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (230, 97, 1))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (253, 184, 99))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (247, 247, 247))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (178, 171, 210))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (94, 60, 153))
		ensure
			purple_oranges_5_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_five: Result.upper = 5
		end
	
	purple_oranges_6: ARRAY [EV_COLOR] is
			-- Purple-orange gradient of 6 shades
		do
			create Result.make (1, 6)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (230, 97, 1))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (241, 163, 64))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (254, 224, 182))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (216, 218, 235))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (153, 142, 195))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (84, 39, 136))
		ensure
			purple_oranges_6_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_six: Result.upper = 6
		end
	
	purple_oranges_7: ARRAY [EV_COLOR] is
			-- Purple-orange gradient of 7 shades
		do
			create Result.make (1, 7)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (230, 97, 1))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (241, 163, 64))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (254, 224, 182))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (247, 247, 247))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (216, 218, 235))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (153, 142, 195))
			Result [7] := (create {EV_COLOR}.make_with_8_bit_rgb (84, 39, 136))
		ensure
			purple_oranges_7_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_seven: Result.upper = 7
		end
			
	purple_oranges_8: ARRAY [EV_COLOR] is
			-- Purple-orange gradient of 8 shades
		do
			create Result.make (1, 8)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (179, 88, 6))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (224, 130, 20))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (253, 184, 99))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (254, 224, 182))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (216, 218, 235))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (178, 171, 210))
			Result [7] := (create {EV_COLOR}.make_with_8_bit_rgb (128, 115, 172))
			Result [8] := (create {EV_COLOR}.make_with_8_bit_rgb (84, 39, 136))
		ensure
			purple_oranges_8_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_eight: Result.upper = 8
		end
			
	purple_oranges_9: ARRAY [EV_COLOR] is
			-- Purple-orange gradient of 9 shades
		do
			create Result.make (1, 9)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (179, 88, 6))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (224, 130, 20))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (253, 184, 99))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (254, 224, 182))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (247, 247, 247))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (216, 218, 235))
			Result [7] := (create {EV_COLOR}.make_with_8_bit_rgb (178, 171, 210))
			Result [8] := (create {EV_COLOR}.make_with_8_bit_rgb (128, 115, 172))
			Result [9] := (create {EV_COLOR}.make_with_8_bit_rgb (84, 39, 136))
		ensure
			purple_oranges_9_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_nine: Result.upper = 9
		end
			
	purple_oranges_10: ARRAY [EV_COLOR] is
			-- Purple-orange gradient of 10 shades
		do
			create Result.make (1, 10)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (127, 59, 8))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (179, 88, 6))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (224, 130, 20))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (253, 184, 99))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (254, 224, 182))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (216, 218, 235))
			Result [7] := (create {EV_COLOR}.make_with_8_bit_rgb (178, 171, 210))
			Result [8] := (create {EV_COLOR}.make_with_8_bit_rgb (128, 115, 172))
			Result [9] := (create {EV_COLOR}.make_with_8_bit_rgb (84, 39, 136))
			Result [10] := (create {EV_COLOR}.make_with_8_bit_rgb (45, 0, 75))
		ensure
			purple_oranges_10_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_ten: Result.upper = 10
		end

	purple_oranges_11: ARRAY [EV_COLOR] is
			-- Purple-orange gradient of 11 shades
		do
			create Result.make (1, 11)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (127, 59, 8))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (179, 88, 6))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (224, 130, 20))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (253, 184, 99))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (254, 224, 182))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (247, 247, 247))
			Result [7] := (create {EV_COLOR}.make_with_8_bit_rgb (216, 218, 235))
			Result [8] := (create {EV_COLOR}.make_with_8_bit_rgb (178, 171, 210))
			Result [9] := (create {EV_COLOR}.make_with_8_bit_rgb (128, 115, 172))
			Result [10] := (create {EV_COLOR}.make_with_8_bit_rgb (84, 39, 136))
			Result [11] := (create {EV_COLOR}.make_with_8_bit_rgb (45, 0, 75))
		ensure
			purple_oranges_11_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_eleven: Result.upper = 11
		end
	
	red_blues_3: ARRAY [EV_COLOR] is
			-- Red-blue gradient of 3 shades
		do
			create Result.make (1, 3)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (239, 138, 98))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (247, 247, 247))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (103, 169, 207))
		ensure
			red_blues_3_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_three: Result.upper = 3
		end

	red_blues_4: ARRAY [EV_COLOR] is
			-- Red-blue gradient of 4 shades
		do
			create Result.make (1, 4)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (202, 0, 32))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (244, 165, 130))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (146, 197, 222))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (5, 113, 176))
		ensure
			red_blues_4_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_four: Result.upper = 4
		end
	
	red_blues_5: ARRAY [EV_COLOR] is
			-- Red-blue gradient of 5 shades
		do
			create Result.make (1, 5)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (202, 0, 32))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (244, 165, 130))			
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (247, 247, 247))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (146, 197, 222))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (5, 113, 176))
		ensure
			red_blues_5_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_five: Result.upper = 5
		end
	
	red_blues_6: ARRAY [EV_COLOR] is
			-- Red-blue gradient of 6 shades
		do
			create Result.make (1, 6)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (178, 24, 43))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (239, 138, 98))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (253, 219, 199))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (209, 229, 240))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (103, 169, 207))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (33, 102, 172))
		ensure
			red_blues_6_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_six: Result.upper = 6
		end
	
	red_blues_7: ARRAY [EV_COLOR] is
			-- Red-blue gradient of 7 shades
		do
			create Result.make (1, 7)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (178, 24, 43))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (239, 138, 98))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (253, 219, 199))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (247, 247, 247))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (209, 229, 240))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (103, 169, 207))
			Result [7] := (create {EV_COLOR}.make_with_8_bit_rgb (33, 102, 172))
		ensure
			red_blues_7_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_seven: Result.upper = 7
		end
			
	red_blues_8: ARRAY [EV_COLOR] is
			-- Red-blue gradient of 8 shades
		do
			create Result.make (1, 8)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (178, 24, 43))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (214, 96, 77))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (244, 165, 130))			
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (253, 219, 199))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (209, 229, 240))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (146, 197, 222))
			Result [7] := (create {EV_COLOR}.make_with_8_bit_rgb (67, 147, 195))
			Result [8] := (create {EV_COLOR}.make_with_8_bit_rgb (33, 102, 172))
		ensure
			red_blues_8_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_eight: Result.upper = 8
		end
			
	red_blues_9: ARRAY [EV_COLOR] is
			-- Red-blue gradient of 9 shades
		do
			create Result.make (1, 9)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (178, 24, 43))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (214, 96, 77))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (244, 165, 130))			
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (253, 219, 199))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (247, 247, 247))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (209, 229, 240))
			Result [7] := (create {EV_COLOR}.make_with_8_bit_rgb (146, 197, 222))
			Result [8] := (create {EV_COLOR}.make_with_8_bit_rgb (67, 147, 195))
			Result [9] := (create {EV_COLOR}.make_with_8_bit_rgb (33, 102, 172))
		ensure
			red_blues_9_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_nine: Result.upper = 9
		end
			
	red_blues_10: ARRAY [EV_COLOR] is
			-- Red-blue gradient of 10 shades
		do
			create Result.make (1, 10)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (103, 0, 31))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (178, 24, 43))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (214, 96, 77))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (244, 165, 130))			
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (253, 219, 199))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (209, 229, 240))
			Result [7] := (create {EV_COLOR}.make_with_8_bit_rgb (146, 197, 222))
			Result [8] := (create {EV_COLOR}.make_with_8_bit_rgb (67, 147, 195))
			Result [9] := (create {EV_COLOR}.make_with_8_bit_rgb (33, 102, 172))
			Result [10] := (create {EV_COLOR}.make_with_8_bit_rgb (5, 48, 97))
		ensure
			red_blues_10_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_ten: Result.upper = 10
		end

	red_blues_11: ARRAY [EV_COLOR] is
			-- Red-blue gradient of 11 shades
		do
			create Result.make (1, 11)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (103, 0, 31))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (178, 24, 43))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (214, 96, 77))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (244, 165, 130))			
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (253, 219, 199))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (247, 247, 247))
			Result [7] := (create {EV_COLOR}.make_with_8_bit_rgb (209, 229, 240))
			Result [8] := (create {EV_COLOR}.make_with_8_bit_rgb (146, 197, 222))
			Result [9] := (create {EV_COLOR}.make_with_8_bit_rgb (67, 147, 195))
			Result [10] := (create {EV_COLOR}.make_with_8_bit_rgb (33, 102, 172))
			Result [11] := (create {EV_COLOR}.make_with_8_bit_rgb (5, 48, 97))
		ensure
			red_blues_11_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_eleven: Result.upper = 11
		end
	
	red_greys_3: ARRAY [EV_COLOR] is
			-- Red-grey gradient of 3 shades
		do
			create Result.make (1, 3)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (239, 138, 98))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (255, 255, 255))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (153, 153, 153))
		ensure
			red_greys_3_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_three: Result.upper = 3
		end

	red_greys_4: ARRAY [EV_COLOR] is
			-- Red-grey gradient of 4 shades
		do
			create Result.make (1, 4)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (202, 0, 32))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (244, 165, 130))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (186, 186, 186))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (64, 64, 64))
		ensure
			red_greys_4_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_four: Result.upper = 4
		end
	
	red_greys_5: ARRAY [EV_COLOR] is
			-- Red-grey gradient of 5 shades
		do
			create Result.make (1, 5)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (202, 0, 32))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (244, 165, 130))			
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (255, 255, 255))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (186, 186, 186))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (64, 64, 64))
		ensure
			red_greys_5_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_five: Result.upper = 5
		end
	
	red_greys_6: ARRAY [EV_COLOR] is
			-- Red-grey gradient of 6 shades
		do
			create Result.make (1, 6)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (178, 24, 43))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (239, 138, 98))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (253, 219, 199))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (224, 224, 224))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (153, 153, 153))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (77, 77, 77))
		ensure
			red_greys_6_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_six: Result.upper = 6
		end
	
	red_greys_7: ARRAY [EV_COLOR] is
			-- Red-grey gradient of 7 shades
		do
			create Result.make (1, 7)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (178, 24, 43))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (239, 138, 98))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (253, 219, 199))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (255, 255, 255))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (224, 224, 224))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (153, 153, 153))
			Result [7] := (create {EV_COLOR}.make_with_8_bit_rgb (77, 77, 77))
		ensure
			red_greys_7_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_seven: Result.upper = 7
		end
			
	red_greys_8: ARRAY [EV_COLOR] is
			-- Red-grey gradient of 8 shades
		do
			create Result.make (1, 8)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (178, 24, 43))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (214, 96, 77))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (244, 165, 130))			
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (253, 219, 199))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (224, 224, 224))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (186, 186, 186))
			Result [7] := (create {EV_COLOR}.make_with_8_bit_rgb (153, 153, 153))
			Result [8] := (create {EV_COLOR}.make_with_8_bit_rgb (77, 77, 77))
		ensure
			red_greys_8_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_eight: Result.upper = 8
		end
			
	red_greys_9: ARRAY [EV_COLOR] is
			-- Red-grey gradient of 9 shades
		do
			create Result.make (1, 9)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (178, 24, 43))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (214, 96, 77))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (244, 165, 130))			
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (253, 219, 199))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (255, 255, 255))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (224, 224, 224))
			Result [7] := (create {EV_COLOR}.make_with_8_bit_rgb (186, 186, 186))
			Result [8] := (create {EV_COLOR}.make_with_8_bit_rgb (153, 153, 153))
			Result [9] := (create {EV_COLOR}.make_with_8_bit_rgb (77, 77, 77))
		ensure
			red_greys_9_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_nine: Result.upper = 9
		end
			
	red_greys_10: ARRAY [EV_COLOR] is
			-- Red-grey gradient of 10 shades
		do
			create Result.make (1, 10)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (103, 0, 31))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (178, 24, 43))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (214, 96, 77))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (244, 165, 130))			
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (253, 219, 199))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (224, 224, 224))
			Result [7] := (create {EV_COLOR}.make_with_8_bit_rgb (186, 186, 186))
			Result [8] := (create {EV_COLOR}.make_with_8_bit_rgb (153, 153, 153))
			Result [9] := (create {EV_COLOR}.make_with_8_bit_rgb (77, 77, 77))
			Result [10] := (create {EV_COLOR}.make_with_8_bit_rgb (26, 26, 26))
		ensure
			red_greys_10_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_ten: Result.upper = 10
		end

	red_greys_11: ARRAY [EV_COLOR] is
			-- Red-grey gradient of 11 shades
		do
			create Result.make (1, 11)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (103, 0, 31))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (178, 24, 43))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (214, 96, 77))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (244, 165, 130))			
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (253, 219, 199))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (255, 255, 255))
			Result [7] := (create {EV_COLOR}.make_with_8_bit_rgb (224, 224, 224))
			Result [8] := (create {EV_COLOR}.make_with_8_bit_rgb (186, 186, 186))
			Result [9] := (create {EV_COLOR}.make_with_8_bit_rgb (153, 153, 153))
			Result [10] := (create {EV_COLOR}.make_with_8_bit_rgb (77, 77, 77))
			Result [11] := (create {EV_COLOR}.make_with_8_bit_rgb (26, 26, 26))
		ensure
			red_greys_11_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_eleven: Result.upper = 11
		end
	
	red_yellow_blues_3: ARRAY [EV_COLOR] is
			-- Red-yellow-blue gradient of 3 shades
		do
			create Result.make (1, 3)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (252, 141, 89))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (255, 255, 191))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (145, 191, 219))
		ensure
			red_yellow_blues_3_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_three: Result.upper = 3
		end

	red_yellow_blues_4: ARRAY [EV_COLOR] is
			-- Red-yellow-blue gradient of 4 shades
		do
			create Result.make (1, 4)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (215, 25, 28))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (253, 174, 97))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (171, 217, 233))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (44, 123, 182))
		ensure
			red_yellow_blues_4_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_four: Result.upper = 4
		end
	
	red_yellow_blues_5: ARRAY [EV_COLOR] is
			-- Red-yellow-blue gradient of 5 shades
		do
			create Result.make (1, 5)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (215, 25, 28))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (253, 174, 97))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (255, 255, 191))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (171, 217, 233))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (44, 123, 182))
		ensure
			red_yellow_blues_5_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_five: Result.upper = 5
		end
	
	red_yellow_blues_6: ARRAY [EV_COLOR] is
			-- Red-yellow-blue gradient of 6 shades
		do
			create Result.make (1, 6)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (215, 48, 39))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (252, 141, 89))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (254, 224, 144))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (224, 243, 248))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (145, 191, 219))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (69, 117, 180))
		ensure
			red_yellow_blues_6_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_six: Result.upper = 6
		end
	
	red_yellow_blues_7: ARRAY [EV_COLOR] is
			-- Red-yellow-blue gradient of 7 shades
		do
			create Result.make (1, 7)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (215, 48, 39))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (252, 141, 89))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (254, 224, 144))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (255, 255, 191))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (224, 243, 248))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (145, 191, 219))
			Result [7] := (create {EV_COLOR}.make_with_8_bit_rgb (69, 117, 180))
		ensure
			red_yellow_blues_7_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_seven: Result.upper = 7
		end
			
	red_yellow_blues_8: ARRAY [EV_COLOR] is
			-- Red-yellow-blue gradient of 8 shades
		do
			create Result.make (1, 8)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (215, 48, 39))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (244, 109, 67))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (253, 174, 97))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (254, 224, 144))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (224, 243, 248))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (171, 217, 233))
			Result [7] := (create {EV_COLOR}.make_with_8_bit_rgb (116, 173, 209))
			Result [8] := (create {EV_COLOR}.make_with_8_bit_rgb (69, 117, 180))
		ensure
			red_yellow_blues_8_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_eight: Result.upper = 8
		end
			
	red_yellow_blues_9: ARRAY [EV_COLOR] is
			-- Red-yellow-blue gradient of 9 shades
		do
			create Result.make (1, 9)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (215, 48, 39))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (244, 109, 67))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (253, 174, 97))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (254, 224, 144))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (255, 255, 191))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (224, 243, 248))
			Result [7] := (create {EV_COLOR}.make_with_8_bit_rgb (171, 217, 233))
			Result [8] := (create {EV_COLOR}.make_with_8_bit_rgb (116, 173, 209))
			Result [9] := (create {EV_COLOR}.make_with_8_bit_rgb (69, 117, 180))
		ensure
			red_yellow_blues_9_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_nine: Result.upper = 9
		end
			
	red_yellow_blues_10: ARRAY [EV_COLOR] is
			-- Red-yellow-blue gradient of 10 shades
		do
			create Result.make (1, 10)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (165, 0, 38))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (215, 48, 39))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (244, 109, 67))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (253, 174, 97))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (254, 224, 144))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (224, 243, 248))
			Result [7] := (create {EV_COLOR}.make_with_8_bit_rgb (171, 217, 233))
			Result [8] := (create {EV_COLOR}.make_with_8_bit_rgb (116, 173, 209))
			Result [9] := (create {EV_COLOR}.make_with_8_bit_rgb (69, 117, 180))
			Result [10] := (create {EV_COLOR}.make_with_8_bit_rgb (49, 54, 149))
		ensure
			red_yellow_blues_10_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_ten: Result.upper = 10
		end

	red_yellow_blues_11: ARRAY [EV_COLOR] is
			-- Red-yellow-blue gradient of 11 shades
		do
			create Result.make (1, 11)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (165, 0, 38))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (215, 48, 39))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (244, 109, 67))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (253, 174, 97))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (254, 224, 144))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (255, 255, 191))
			Result [7] := (create {EV_COLOR}.make_with_8_bit_rgb (224, 243, 248))
			Result [8] := (create {EV_COLOR}.make_with_8_bit_rgb (171, 217, 233))
			Result [9] := (create {EV_COLOR}.make_with_8_bit_rgb (116, 173, 209))
			Result [10] := (create {EV_COLOR}.make_with_8_bit_rgb (69, 117, 180))
			Result [11] := (create {EV_COLOR}.make_with_8_bit_rgb (49, 54, 149))
		ensure
			red_yellow_blues_11_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_eleven: Result.upper = 11
		end
	
	red_yellow_greens_3: ARRAY [EV_COLOR] is
			-- Red-yellow-green gradient of 3 shades
		do
			create Result.make (1, 3)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (252, 141, 89))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (255, 255, 191))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (145, 207, 96))
		ensure
			red_yellow_greens_3_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_three: Result.upper = 3
		end

	red_yellow_greens_4: ARRAY [EV_COLOR] is
			-- Red-yellow-green gradient of 4 shades
		do
			create Result.make (1, 4)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (215, 25, 28))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (253, 174, 97))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (166, 217, 106))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (26, 150, 65))
		ensure
			red_yellow_greens_4_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_four: Result.upper = 4
		end
	
	red_yellow_greens_5: ARRAY [EV_COLOR] is
			-- Red-yellow-green gradient of 5 shades
		do
			create Result.make (1, 5)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (215, 25, 28))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (253, 174, 97))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (255, 255, 191))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (166, 217, 106))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (26, 150, 65))
		ensure
			red_yellow_greens_5_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_five: Result.upper = 5
		end
	
	red_yellow_greens_6: ARRAY [EV_COLOR] is
			-- Red-yellow-green gradient of 6 shades
		do
			create Result.make (1, 6)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (215, 48, 39))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (252, 141, 89))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (254, 224, 139))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (217, 239, 139))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (145, 207, 96))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (26, 152, 80))
		ensure
			red_yellow_greens_6_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_six: Result.upper = 6
		end
	
	red_yellow_greens_7: ARRAY [EV_COLOR] is
			-- Red-yellow-green gradient of 7 shades
		do
			create Result.make (1, 7)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (215, 48, 39))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (252, 141, 89))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (254, 224, 139))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (255, 255, 191))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (217, 239, 139))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (145, 207, 96))
			Result [7] := (create {EV_COLOR}.make_with_8_bit_rgb (26, 152, 80))
		ensure
			red_yellow_greens_7_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_seven: Result.upper = 7
		end
			
	red_yellow_greens_8: ARRAY [EV_COLOR] is
			-- Red-yellow-green gradient of 8 shades
		do
			create Result.make (1, 8)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (215, 48, 39))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (244, 109, 67))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (253, 174, 97))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (254, 224, 139))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (217, 239, 139))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (166, 217, 106))
			Result [7] := (create {EV_COLOR}.make_with_8_bit_rgb (102, 189, 99))
			Result [8] := (create {EV_COLOR}.make_with_8_bit_rgb (26, 152, 80))
		ensure
			red_yellow_greens_8_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_eight: Result.upper = 8
		end
			
	red_yellow_greens_9: ARRAY [EV_COLOR] is
			-- Red-yellow-green gradient of 9 shades
		do
			create Result.make (1, 9)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (215, 48, 39))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (244, 109, 67))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (253, 174, 97))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (254, 224, 139))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (255, 255, 191))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (217, 239, 139))
			Result [7] := (create {EV_COLOR}.make_with_8_bit_rgb (166, 217, 106))
			Result [8] := (create {EV_COLOR}.make_with_8_bit_rgb (102, 189, 99))
			Result [9] := (create {EV_COLOR}.make_with_8_bit_rgb (26, 152, 80))
		ensure
			red_yellow_greens_9_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_nine: Result.upper = 9
		end
			
	red_yellow_greens_10: ARRAY [EV_COLOR] is
			-- Red-yellow-green gradient of 10 shades
		do
			create Result.make (1, 10)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (165, 0, 38))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (215, 48, 39))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (244, 109, 67))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (253, 174, 97))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (254, 224, 139))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (217, 239, 139))
			Result [7] := (create {EV_COLOR}.make_with_8_bit_rgb (166, 217, 106))
			Result [8] := (create {EV_COLOR}.make_with_8_bit_rgb (102, 189, 99))
			Result [9] := (create {EV_COLOR}.make_with_8_bit_rgb (26, 152, 80))
			Result [10] := (create {EV_COLOR}.make_with_8_bit_rgb (0, 104, 55))
		ensure
			red_yellow_greens_10_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_ten: Result.upper = 10
		end
	
	red_yellow_greens_11: ARRAY [EV_COLOR] is
			-- Red-yellow-green gradient of 11 shades
		do
			create Result.make (1, 11)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (165, 0, 38))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (215, 48, 39))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (244, 109, 67))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (253, 174, 97))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (254, 224, 139))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (255, 255, 191))
			Result [7] := (create {EV_COLOR}.make_with_8_bit_rgb (217, 239, 139))
			Result [8] := (create {EV_COLOR}.make_with_8_bit_rgb (166, 217, 106))
			Result [9] := (create {EV_COLOR}.make_with_8_bit_rgb (102, 189, 99))
			Result [10] := (create {EV_COLOR}.make_with_8_bit_rgb (26, 152, 80))
			Result [11] := (create {EV_COLOR}.make_with_8_bit_rgb (0, 104, 55))
		ensure
			red_yellow_greens_11_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_eleven: Result.upper = 11
		end
				
   spectral_3: ARRAY [EV_COLOR] is
			-- Spectral gradient of 3 shades
		do
			create Result.make (1, 3)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (252, 141, 89))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (255, 255, 191))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (153, 213, 148))
		ensure
			spectral_3_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_three: Result.upper = 3
		end

	spectral_4: ARRAY [EV_COLOR] is
			-- Spectral gradient of 4 shades
		do
			create Result.make (1, 4)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (215, 25, 28))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (253, 174, 97))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (171, 221, 164))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (43, 131, 186))
		ensure
			spectral_4_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_four: Result.upper = 4
		end
	
	spectral_5: ARRAY [EV_COLOR] is
			-- Spectral gradient of 5 shades
		do
			create Result.make (1, 5)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (215, 25, 28))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (253, 174, 97))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (255, 255, 191))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (171, 221, 164))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (43, 131, 186))
		ensure
			spectral_5_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_five: Result.upper = 5
		end
	
	spectral_6: ARRAY [EV_COLOR] is
			-- Spectral gradient of 6 shades
		do
			create Result.make (1, 6)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (213, 62, 79))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (252, 141, 89))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (254, 224, 139))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (230, 245, 152))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (153, 213, 148))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (50, 136, 189))
		ensure
			spectral_6_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_six: Result.upper = 6
		end
	
	spectral_7: ARRAY [EV_COLOR] is
			-- Spectral gradient of 7 shades
		do
			create Result.make (1, 7)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (213, 62, 79))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (252, 141, 89))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (254, 224, 139))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (255, 255, 191))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (230, 245, 152))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (153, 213, 148))
			Result [7] := (create {EV_COLOR}.make_with_8_bit_rgb (50, 136, 189))
		ensure
			spectral_7_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_seven: Result.upper = 7
		end
			
	spectral_8: ARRAY [EV_COLOR] is
			-- Spectral gradient of 8 shades
		do
			create Result.make (1, 8)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (213, 62, 79))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (244, 109, 67))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (253, 174, 97))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (254, 224, 139))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (230, 245, 152))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (171, 221, 164))
			Result [7] := (create {EV_COLOR}.make_with_8_bit_rgb (102, 194, 165))
			Result [8] := (create {EV_COLOR}.make_with_8_bit_rgb (50, 136, 189))
		ensure
			spectral_8_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_eight: Result.upper = 8
		end
			
	spectral_9: ARRAY [EV_COLOR] is
			-- Spectral gradient of 9 shades
		do
			create Result.make (1, 9)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (213, 62, 79))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (244, 109, 67))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (253, 174, 97))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (254, 224, 139))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (255, 255, 191))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (230, 245, 152))
			Result [7] := (create {EV_COLOR}.make_with_8_bit_rgb (171, 221, 164))
			Result [8] := (create {EV_COLOR}.make_with_8_bit_rgb (102, 194, 165))
			Result [9] := (create {EV_COLOR}.make_with_8_bit_rgb (50, 136, 189))
		ensure
			spectral_9_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_nine: Result.upper = 9
		end
			
	spectral_10: ARRAY [EV_COLOR] is
			-- Spectral gradient of 10 shades
		do
			create Result.make (1, 10)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (158, 1, 66))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (213, 62, 79))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (244, 109, 67))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (253, 174, 97))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (254, 224, 139))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (230, 245, 152))
			Result [7] := (create {EV_COLOR}.make_with_8_bit_rgb (171, 221, 164))
			Result [8] := (create {EV_COLOR}.make_with_8_bit_rgb (102, 194, 165))
			Result [9] := (create {EV_COLOR}.make_with_8_bit_rgb (50, 136, 189))
			Result [10] := (create {EV_COLOR}.make_with_8_bit_rgb (94, 79, 162))
		ensure
			spectral_10_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_ten: Result.upper = 10
		end

	spectral_11: ARRAY [EV_COLOR] is
			-- Spectral gradient of 11 shades
		do
			create Result.make (1, 11)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (158, 1, 66))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (213, 62, 79))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (244, 109, 67))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (253, 174, 97))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (254, 224, 139))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (255, 255, 191))
			Result [7] := (create {EV_COLOR}.make_with_8_bit_rgb (230, 245, 152))
			Result [8] := (create {EV_COLOR}.make_with_8_bit_rgb (171, 221, 164))
			Result [9] := (create {EV_COLOR}.make_with_8_bit_rgb (102, 194, 165))
			Result [10] := (create {EV_COLOR}.make_with_8_bit_rgb (50, 136, 189))
			Result [11] := (create {EV_COLOR}.make_with_8_bit_rgb (94, 79, 162))
		ensure
			spectral_11_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_eleven: Result.upper = 11
		end
	
feature -- Qualitiative color gradients
	
	accent (a_classes: NATURAL_8): ARRAY [EV_COLOR] is
			-- Accent gradient of `a_classes' shades
		require
			a_classes_large_enough: a_classes >= 3
			a_classes_small_enough: a_classes <= 8
		do
			inspect
				a_classes
			when 3 then
				Result := accent_3
			when 4 then
				Result := accent_4
			when 5 then
				Result := accent_5
			when 6 then
				Result := accent_6
			when 7 then
				Result := accent_7
			when 8 then
				Result := accent_8
			end
		ensure
			accent_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_a_classes: Result.upper = a_classes
		end
	
	dark2 (a_classes: NATURAL_8): ARRAY [EV_COLOR] is
			-- Dark2 gradient of `a_classes' shades
		require
			a_classes_large_enough: a_classes >= 3
			a_classes_small_enough: a_classes <= 8
		do
			inspect
				a_classes
			when 3 then
				Result := dark2_3
			when 4 then
				Result := dark2_4
			when 5 then
				Result := dark2_5
			when 6 then
				Result := dark2_6
			when 7 then
				Result := dark2_7
			when 8 then
				Result := dark2_8
			end
		ensure
			dark2_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_a_classes: Result.upper = a_classes
		end	
		
	paired (a_classes: NATURAL_8): ARRAY [EV_COLOR] is
			-- Paired gradient of `a_classes' shades
		require
			a_classes_large_enough: a_classes >= 3
			a_classes_small_enough: a_classes <= 11
		do
			inspect
				a_classes
			when 3 then
				Result := paired_3
			when 4 then
				Result := paired_4
			when 5 then
				Result := paired_5
			when 6 then
				Result := paired_6
			when 7 then
				Result := paired_7
			when 8 then
				Result := paired_8
			when 9 then
				Result := paired_9
			when 10 then
				Result := paired_10
			when 11 then
				Result := paired_11
			end
		ensure
			accent_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_a_classes: Result.upper = a_classes
		end
	
	pastel1 (a_classes: NATURAL_8): ARRAY [EV_COLOR] is
			-- Pastel1 gradient of `a_classes' shades
		require
			a_classes_large_enough: a_classes >= 3
			a_classes_small_enough: a_classes <= 9
		do
			inspect
				a_classes
			when 3 then
				Result := pastel1_3
			when 4 then
				Result := pastel1_4
			when 5 then
				Result := pastel1_5
			when 6 then
				Result := pastel1_6
			when 7 then
				Result := pastel1_7
			when 8 then
				Result := pastel1_8
			when 9 then
				Result := pastel1_9
			end
		ensure
			pastel1_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_a_classes: Result.upper = a_classes
		end
	
	pastel2 (a_classes: NATURAL_8): ARRAY [EV_COLOR] is
			-- Pastel2 gradient of `a_classes' shades
		require
			a_classes_large_enough: a_classes >= 3
			a_classes_small_enough: a_classes <= 8
		do
			inspect
				a_classes
			when 3 then
				Result := pastel2_3
			when 4 then
				Result := pastel2_4
			when 5 then
				Result := pastel2_5
			when 6 then
				Result := pastel2_6
			when 7 then
				Result := pastel2_7
			when 8 then
				Result := pastel2_8
			end
		ensure
			pastel2_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_a_classes: Result.upper = a_classes
		end
	
	set1 (a_classes: NATURAL_8): ARRAY [EV_COLOR] is
			-- Set1 gradient of `a_classes' shades
		require
			a_classes_large_enough: a_classes >= 3
			a_classes_small_enough: a_classes <= 9
		do
			inspect
				a_classes
			when 3 then
				Result := set1_3
			when 4 then
				Result := set1_4
			when 5 then
				Result := set1_5
			when 6 then
				Result := set1_6
			when 7 then
				Result := set1_7
			when 8 then
				Result := set1_8
			when 9 then
				Result := set1_9
			end
		ensure
			set1_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_a_classes: Result.upper = a_classes
		end
	
	set2 (a_classes: NATURAL_8): ARRAY [EV_COLOR] is
			-- Set2 gradient of `a_classes' shades
		require
			a_classes_large_enough: a_classes >= 3
			a_classes_small_enough: a_classes <= 8
		do
			inspect
				a_classes
			when 3 then
				Result := set2_3
			when 4 then
				Result := set2_4
			when 5 then
				Result := set2_5
			when 6 then
				Result := set2_6
			when 7 then
				Result := set2_7
			when 8 then
				Result := set2_8
			end
		ensure
			set2_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_a_classes: Result.upper = a_classes
		end
	
	set3 (a_classes: NATURAL_8): ARRAY [EV_COLOR] is
			-- Set3 gradient of `a_classes' shades
		require
			a_classes_large_enough: a_classes >= 3
			a_classes_small_enough: a_classes <= 12
		do
			inspect
				a_classes
			when 3 then
				Result := set3_3
			when 4 then
				Result := set3_4
			when 5 then
				Result := set3_5
			when 6 then
				Result := set3_6
			when 7 then
				Result := set3_7
			when 8 then
				Result := set3_8
			when 9 then
				Result := set3_9
			when 10 then
				Result := set3_10
			when 11 then
				Result := set3_11
			when 12 then
				Result := set3_12
			end
		ensure
			set3_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_a_classes: Result.upper = a_classes
		end

   accent_3: ARRAY [EV_COLOR] is
			-- Accent gradient of 3 shades
		do
			create Result.make (1, 3)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (127, 201, 127))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (190, 174, 212))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (253, 192, 134))
		ensure
			accent_3_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_three: Result.upper = 3
		end

	accent_4: ARRAY [EV_COLOR] is
			-- Accent gradient of 4 shades
		do
			create Result.make (1, 4)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (127, 201, 127))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (190, 174, 212))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (253, 192, 134))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (255, 255, 153))
		ensure
			accent_4_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_four: Result.upper = 4
		end
	
	accent_5: ARRAY [EV_COLOR] is
			-- Accent gradient of 5 shades
		do
			create Result.make (1, 5)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (127, 201, 127))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (190, 174, 212))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (253, 192, 134))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (255, 255, 153))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (56, 108, 176))
		ensure
			accent_5_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_five: Result.upper = 5
		end
	
	accent_6: ARRAY [EV_COLOR] is
			-- Accent gradient of 6 shades
		do
			create Result.make (1, 6)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (127, 201, 127))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (190, 174, 212))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (253, 192, 134))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (255, 255, 153))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (56, 108, 176))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (240, 2, 127))
		ensure
			accent_6_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_six: Result.upper = 6
		end
	
	accent_7: ARRAY [EV_COLOR] is
			-- Accent gradient of 7 shades
		do
			create Result.make (1, 7)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (127, 201, 127))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (190, 174, 212))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (253, 192, 134))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (255, 255, 153))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (56, 108, 176))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (240, 2, 127))
			Result [7] := (create {EV_COLOR}.make_with_8_bit_rgb (191, 91, 23))
		ensure
			accent_7_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_seven: Result.upper = 7
		end
			
	accent_8: ARRAY [EV_COLOR] is
			-- Accent gradient of 8 shades
		do
			create Result.make (1, 8)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (127, 201, 127))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (190, 174, 212))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (253, 192, 134))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (255, 255, 153))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (56, 108, 176))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (240, 2, 127))
			Result [7] := (create {EV_COLOR}.make_with_8_bit_rgb (191, 91, 23))
			Result [8] := (create {EV_COLOR}.make_with_8_bit_rgb (102, 102, 102))
		ensure
			accent_8_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_eight: Result.upper = 8
		end
	
   dark2_3: ARRAY [EV_COLOR] is
			-- Dark2 gradient of 3 shades
		do
			create Result.make (1, 3)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (27, 158, 119))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (217, 95, 2))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (117, 112, 179))
		ensure
			dark2_3_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_three: Result.upper = 3
		end

	dark2_4: ARRAY [EV_COLOR] is
			-- Dark2 gradient of 4 shades
		do
			create Result.make (1, 4)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (27, 158, 119))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (217, 95, 2))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (117, 112, 179))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (231, 41, 138))
		ensure
			dark2_4_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_four: Result.upper = 4
		end
	
	dark2_5: ARRAY [EV_COLOR] is
			-- Dark2 gradient of 5 shades
		do
			create Result.make (1, 5)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (27, 158, 119))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (217, 95, 2))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (117, 112, 179))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (231, 41, 138))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (102, 166, 30))
		ensure
			dark2_5_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_five: Result.upper = 5
		end
	
	dark2_6: ARRAY [EV_COLOR] is
			-- Dark2 gradient of 6 shades
		do
			create Result.make (1, 6)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (27, 158, 119))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (217, 95, 2))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (117, 112, 179))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (231, 41, 138))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (102, 166, 30))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (230, 171, 2))
		ensure
			dark2_6_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_six: Result.upper = 6
		end
	
	dark2_7: ARRAY [EV_COLOR] is
			-- Dark2 gradient of 7 shades
		do
			create Result.make (1, 7)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (27, 158, 119))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (217, 95, 2))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (117, 112, 179))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (231, 41, 138))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (102, 166, 30))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (230, 171, 2))
			Result [7] := (create {EV_COLOR}.make_with_8_bit_rgb (166, 118, 29))
		ensure
			dark2_7_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_seven: Result.upper = 7
		end
			
	dark2_8: ARRAY [EV_COLOR] is
			-- Dark2 gradient of 8 shades
		do
			create Result.make (1, 8)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (27, 158, 119))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (217, 95, 2))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (117, 112, 179))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (231, 41, 138))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (102, 166, 30))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (230, 171, 2))
			Result [7] := (create {EV_COLOR}.make_with_8_bit_rgb (166, 118, 29))
			Result [8] := (create {EV_COLOR}.make_with_8_bit_rgb (102, 102, 102))
		ensure
			dark2_8_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_eight: Result.upper = 8
		end
				
   paired_3: ARRAY [EV_COLOR] is
			-- Paired gradient of 3 shades
		do
			create Result.make (1, 3)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (166, 206, 227))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (31, 120, 180))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (178, 223, 138))
		ensure
			paired_3_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_three: Result.upper = 3
		end

	paired_4: ARRAY [EV_COLOR] is
			-- Paired gradient of 4 shades
		do
			create Result.make (1, 4)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (166, 206, 227))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (31, 120, 180))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (178, 223, 138))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (51, 160, 44))
		ensure
			paired_4_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_four: Result.upper = 4
		end
	
	paired_5: ARRAY [EV_COLOR] is
			-- Paired gradient of 5 shades
		do
			create Result.make (1, 5)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (166, 206, 227))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (31, 120, 180))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (178, 223, 138))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (51, 160, 44))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (251, 154, 153))
		ensure
			paired_5_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_five: Result.upper = 5
		end
	
	paired_6: ARRAY [EV_COLOR] is
			-- Paired gradient of 6 shades
		do
			create Result.make (1, 6)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (166, 206, 227))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (31, 120, 180))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (178, 223, 138))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (51, 160, 44))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (251, 154, 153))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (227, 26, 28))
		ensure
			paired_6_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_six: Result.upper = 6
		end
	
	paired_7: ARRAY [EV_COLOR] is
			-- Paired gradient of 7 shades
		do
			create Result.make (1, 7)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (166, 206, 227))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (31, 120, 180))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (178, 223, 138))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (51, 160, 44))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (251, 154, 153))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (227, 26, 28))
			Result [7] := (create {EV_COLOR}.make_with_8_bit_rgb (253, 191, 111))
		ensure
			paired_7_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_seven: Result.upper = 7
		end
			
	paired_8: ARRAY [EV_COLOR] is
			-- Paired gradient of 8 shades
		do
			create Result.make (1, 8)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (166, 206, 227))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (31, 120, 180))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (178, 223, 138))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (51, 160, 44))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (251, 154, 153))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (227, 26, 28))
			Result [7] := (create {EV_COLOR}.make_with_8_bit_rgb (253, 191, 111))
			Result [8] := (create {EV_COLOR}.make_with_8_bit_rgb (255, 127, 0))
		ensure
			paired_8_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_eight: Result.upper = 8
		end
			
	paired_9: ARRAY [EV_COLOR] is
			-- Paired gradient of 9 shades
		do
			create Result.make (1, 9)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (166, 206, 227))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (31, 120, 180))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (178, 223, 138))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (51, 160, 44))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (251, 154, 153))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (227, 26, 28))
			Result [7] := (create {EV_COLOR}.make_with_8_bit_rgb (253, 191, 111))
			Result [8] := (create {EV_COLOR}.make_with_8_bit_rgb (255, 127, 0))
			Result [9] := (create {EV_COLOR}.make_with_8_bit_rgb (202, 178, 214))
		ensure
			paired_9_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_nine: Result.upper = 9
		end
			
	paired_10: ARRAY [EV_COLOR] is
			-- Paired gradient of 10 shades
		do
			create Result.make (1, 10)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (166, 206, 227))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (31, 120, 180))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (178, 223, 138))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (51, 160, 44))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (251, 154, 153))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (227, 26, 28))
			Result [7] := (create {EV_COLOR}.make_with_8_bit_rgb (253, 191, 111))
			Result [8] := (create {EV_COLOR}.make_with_8_bit_rgb (255, 127, 0))
			Result [9] := (create {EV_COLOR}.make_with_8_bit_rgb (202, 178, 214))
			Result [10] := (create {EV_COLOR}.make_with_8_bit_rgb (106, 61, 154))
		ensure
			paired_10_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_ten: Result.upper = 10
		end

	paired_11: ARRAY [EV_COLOR] is
			-- Paired gradient of 11 shades
		do
			create Result.make (1, 11)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (166, 206, 227))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (31, 120, 180))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (178, 223, 138))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (51, 160, 44))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (251, 154, 153))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (227, 26, 28))
			Result [7] := (create {EV_COLOR}.make_with_8_bit_rgb (253, 191, 111))
			Result [8] := (create {EV_COLOR}.make_with_8_bit_rgb (255, 127, 0))
			Result [9] := (create {EV_COLOR}.make_with_8_bit_rgb (202, 178, 214))
			Result [10] := (create {EV_COLOR}.make_with_8_bit_rgb (106, 61, 154))
			Result [11] := (create {EV_COLOR}.make_with_8_bit_rgb (255, 255, 153))
		ensure
			paired_11_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_eleven: Result.upper = 11
		end
				
   pastel1_3: ARRAY [EV_COLOR] is
			-- Pastel1 gradient of 3 shades
		do
			create Result.make (1, 3)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (251, 180, 174))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (179, 205, 227))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (204, 235, 197))
		ensure
			pastel1_3_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_three: Result.upper = 3
		end

	pastel1_4: ARRAY [EV_COLOR] is
			-- Pastel1 gradient of 4 shades
		do
			create Result.make (1, 4)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (251, 180, 174))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (179, 205, 227))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (204, 235, 197))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (222, 203, 228))
		ensure
			pastel1_4_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_four: Result.upper = 4
		end
	
	pastel1_5: ARRAY [EV_COLOR] is
			-- Pastel1 gradient of 5 shades
		do
			create Result.make (1, 5)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (251, 180, 174))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (179, 205, 227))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (204, 235, 197))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (222, 203, 228))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (254, 217, 166))
		ensure
			pastel1_5_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_five: Result.upper = 5
		end
	
	pastel1_6: ARRAY [EV_COLOR] is
			-- Pastel1 gradient of 6 shades
		do
			create Result.make (1, 6)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (251, 180, 174))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (179, 205, 227))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (204, 235, 197))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (222, 203, 228))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (254, 217, 166))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (255, 255, 204))
		ensure
			pastel1_6_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_six: Result.upper = 6
		end
	
	pastel1_7: ARRAY [EV_COLOR] is
			-- Pastel1 gradient of 7 shades
		do
			create Result.make (1, 7)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (251, 180, 174))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (179, 205, 227))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (204, 235, 197))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (222, 203, 228))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (254, 217, 166))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (255, 255, 204))
			Result [7] := (create {EV_COLOR}.make_with_8_bit_rgb (229, 216, 189))
		ensure
			pastel1_7_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_seven: Result.upper = 7
		end
			
	pastel1_8: ARRAY [EV_COLOR] is
			-- Pastel1 gradient of 8 shades
		do
			create Result.make (1, 8)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (251, 180, 174))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (179, 205, 227))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (204, 235, 197))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (222, 203, 228))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (254, 217, 166))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (255, 255, 204))
			Result [7] := (create {EV_COLOR}.make_with_8_bit_rgb (229, 216, 189))
			Result [8] := (create {EV_COLOR}.make_with_8_bit_rgb (253, 218, 236))
		ensure
			pastel1_8_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_eight: Result.upper = 8
		end
			
	pastel1_9: ARRAY [EV_COLOR] is
			-- Pastel1 gradient of 9 shades
		do
			create Result.make (1, 9)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (251, 180, 174))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (179, 205, 227))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (204, 235, 197))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (222, 203, 228))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (254, 217, 166))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (255, 255, 204))
			Result [7] := (create {EV_COLOR}.make_with_8_bit_rgb (229, 216, 189))
			Result [8] := (create {EV_COLOR}.make_with_8_bit_rgb (253, 218, 236))
			Result [9] := (create {EV_COLOR}.make_with_8_bit_rgb (242, 242, 242))
		ensure
			pastel1_9_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_nine: Result.upper = 9
		end
	
   pastel2_3: ARRAY [EV_COLOR] is
			-- Pastel2 gradient of 3 shades
		do
			create Result.make (1, 3)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (179, 226, 205))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (253, 205, 172))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (203, 213, 232))
		ensure
			pastel2_3_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_three: Result.upper = 3
		end

	pastel2_4: ARRAY [EV_COLOR] is
			-- Pastel2 gradient of 4 shades
		do
			create Result.make (1, 4)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (179, 226, 205))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (253, 205, 172))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (203, 213, 232))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (244, 202, 228))
		ensure
			pastel2_4_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_four: Result.upper = 4
		end
	
	pastel2_5: ARRAY [EV_COLOR] is
			-- Pastel2 gradient of 5 shades
		do
			create Result.make (1, 5)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (179, 226, 205))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (253, 205, 172))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (203, 213, 232))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (244, 202, 228))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (230, 245, 201))
		ensure
			pastel2_5_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_five: Result.upper = 5
		end
	
	pastel2_6: ARRAY [EV_COLOR] is
			-- Pastel2 gradient of 6 shades
		do
			create Result.make (1, 6)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (179, 226, 205))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (253, 205, 172))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (203, 213, 232))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (244, 202, 228))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (230, 245, 201))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (255, 242, 174))
		ensure
			pastel2_6_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_six: Result.upper = 6
		end
	
	pastel2_7: ARRAY [EV_COLOR] is
			-- Pastel2 gradient of 7 shades
		do
			create Result.make (1, 7)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (179, 226, 205))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (253, 205, 172))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (203, 213, 232))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (244, 202, 228))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (230, 245, 201))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (255, 242, 174))
			Result [7] := (create {EV_COLOR}.make_with_8_bit_rgb (241, 226, 204))
		ensure
			pastel2_7_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_seven: Result.upper = 7
		end
			
	pastel2_8: ARRAY [EV_COLOR] is
			-- Pastel2 gradient of 8 shades
		do
			create Result.make (1, 8)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (179, 226, 205))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (253, 205, 172))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (203, 213, 232))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (244, 202, 228))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (230, 245, 201))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (255, 242, 174))
			Result [7] := (create {EV_COLOR}.make_with_8_bit_rgb (241, 226, 204))
			Result [8] := (create {EV_COLOR}.make_with_8_bit_rgb (204, 204, 204))
		ensure
			pastel2_8_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_eight: Result.upper = 8
		end
	
   set1_3: ARRAY [EV_COLOR] is
			-- Set1 gradient of 3 shades
		do
			create Result.make (1, 3)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (228, 26, 28))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (55, 126, 184))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (77, 175, 74))
		ensure
			set1_3_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_three: Result.upper = 3
		end

	set1_4: ARRAY [EV_COLOR] is
			-- Set1 gradient of 4 shades
		do
			create Result.make (1, 4)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (228, 26, 28))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (55, 126, 184))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (77, 175, 74))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (152, 78, 163))
		ensure
			set1_4_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_four: Result.upper = 4
		end
	
	set1_5: ARRAY [EV_COLOR] is
			-- Set1 gradient of 5 shades
		do
			create Result.make (1, 5)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (228, 26, 28))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (55, 126, 184))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (77, 175, 74))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (152, 78, 163))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (255, 127, 0))
		ensure
			set1_5_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_five: Result.upper = 5
		end
	
	set1_6: ARRAY [EV_COLOR] is
			-- Set1 gradient of 6 shades
		do
			create Result.make (1, 6)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (228, 26, 28))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (55, 126, 184))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (77, 175, 74))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (152, 78, 163))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (255, 127, 0))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (255, 255, 51))
		ensure
			set1_6_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_six: Result.upper = 6
		end
	
	set1_7: ARRAY [EV_COLOR] is
			-- Set1 gradient of 7 shades
		do
			create Result.make (1, 7)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (228, 26, 28))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (55, 126, 184))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (77, 175, 74))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (152, 78, 163))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (255, 127, 0))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (255, 255, 51))
			Result [7] := (create {EV_COLOR}.make_with_8_bit_rgb (166, 86, 40))
		ensure
			set1_7_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_seven: Result.upper = 7
		end
			
	set1_8: ARRAY [EV_COLOR] is
			-- Set1 gradient of 8 shades
		do
			create Result.make (1, 8)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (228, 26, 28))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (55, 126, 184))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (77, 175, 74))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (152, 78, 163))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (255, 127, 0))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (255, 255, 51))
			Result [7] := (create {EV_COLOR}.make_with_8_bit_rgb (166, 86, 40))
			Result [8] := (create {EV_COLOR}.make_with_8_bit_rgb (247, 129, 191))
		ensure
			set1_8_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_eight: Result.upper = 8
		end
			
	set1_9: ARRAY [EV_COLOR] is
			-- Set1 gradient of 9 shades
		do
			create Result.make (1, 9)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (228, 26, 28))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (55, 126, 184))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (77, 175, 74))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (152, 78, 163))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (255, 127, 0))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (255, 255, 51))
			Result [7] := (create {EV_COLOR}.make_with_8_bit_rgb (166, 86, 40))
			Result [8] := (create {EV_COLOR}.make_with_8_bit_rgb (247, 129, 191))
			Result [9] := (create {EV_COLOR}.make_with_8_bit_rgb (153, 153, 153))
		ensure
			set1_9_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_nine: Result.upper = 9
		end
	
   set2_3: ARRAY [EV_COLOR] is
			-- Set2 gradient of 3 shades
		do
			create Result.make (1, 3)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (102, 194, 165))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (252, 141, 98))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (141, 160, 203))
		ensure
			set2_3_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_three: Result.upper = 3
		end

	set2_4: ARRAY [EV_COLOR] is
			-- Set2 gradient of 4 shades
		do
			create Result.make (1, 4)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (102, 194, 165))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (252, 141, 98))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (141, 160, 203))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (231, 138, 195))
		ensure
			set2_4_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_four: Result.upper = 4
		end
	
	set2_5: ARRAY [EV_COLOR] is
			-- Set2 gradient of 5 shades
		do
			create Result.make (1, 5)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (102, 194, 165))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (252, 141, 98))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (141, 160, 203))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (231, 138, 195))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (166, 216, 84))
		ensure
			set2_5_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_five: Result.upper = 5
		end
	
	set2_6: ARRAY [EV_COLOR] is
			-- Set2 gradient of 6 shades
		do
			create Result.make (1, 6)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (102, 194, 165))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (252, 141, 98))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (141, 160, 203))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (231, 138, 195))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (166, 216, 84))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (255, 217, 47))
		ensure
			set2_6_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_six: Result.upper = 6
		end
	
	set2_7: ARRAY [EV_COLOR] is
			-- Set2 gradient of 7 shades
		do
			create Result.make (1, 7)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (102, 194, 165))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (252, 141, 98))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (141, 160, 203))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (231, 138, 195))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (166, 216, 84))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (255, 217, 47))
			Result [7] := (create {EV_COLOR}.make_with_8_bit_rgb (229, 196, 148))
		ensure
			set2_7_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_seven: Result.upper = 7
		end
			
	set2_8: ARRAY [EV_COLOR] is
			-- Set2 gradient of 8 shades
		do
			create Result.make (1, 8)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (102, 194, 165))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (252, 141, 98))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (141, 160, 203))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (231, 138, 195))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (166, 216, 84))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (255, 217, 47))
			Result [7] := (create {EV_COLOR}.make_with_8_bit_rgb (229, 196, 148))
			Result [8] := (create {EV_COLOR}.make_with_8_bit_rgb (179, 179, 179))
		ensure
			set2_8_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_eight: Result.upper = 8
		end
				
   set3_3: ARRAY [EV_COLOR] is
			-- Set3 gradient of 3 shades
		do
			create Result.make (1, 3)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (141, 211, 199))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (255, 255, 179))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (190, 186, 218))
		ensure
			set3_3_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_three: Result.upper = 3
		end

	set3_4: ARRAY [EV_COLOR] is
			-- Set3 gradient of 4 shades
		do
			create Result.make (1, 4)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (141, 211, 199))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (255, 255, 179))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (190, 186, 218))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (251, 128, 114))
		ensure
			set3_4_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_four: Result.upper = 4
		end
	
	set3_5: ARRAY [EV_COLOR] is
			-- Set3 gradient of 5 shades
		do
			create Result.make (1, 5)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (141, 211, 199))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (255, 255, 179))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (190, 186, 218))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (251, 128, 114))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (128, 177, 211))
		ensure
			set3_5_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_five: Result.upper = 5
		end
	
	set3_6: ARRAY [EV_COLOR] is
			-- Set3 gradient of 6 shades
		do
			create Result.make (1, 6)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (141, 211, 199))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (255, 255, 179))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (190, 186, 218))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (251, 128, 114))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (128, 177, 211))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (253, 180, 98))
		ensure
			set3_6_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_six: Result.upper = 6
		end
	
	set3_7: ARRAY [EV_COLOR] is
			-- Set3 gradient of 7 shades
		do
			create Result.make (1, 7)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (141, 211, 199))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (255, 255, 179))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (190, 186, 218))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (251, 128, 114))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (128, 177, 211))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (253, 180, 98))
			Result [7] := (create {EV_COLOR}.make_with_8_bit_rgb (179, 222, 105))
		ensure
			set3_7_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_seven: Result.upper = 7
		end
			
	set3_8: ARRAY [EV_COLOR] is
			-- Set3 gradient of 8 shades
		do
			create Result.make (1, 8)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (141, 211, 199))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (255, 255, 179))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (190, 186, 218))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (251, 128, 114))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (128, 177, 211))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (253, 180, 98))
			Result [7] := (create {EV_COLOR}.make_with_8_bit_rgb (179, 222, 105))
			Result [8] := (create {EV_COLOR}.make_with_8_bit_rgb (252, 205, 229))
		ensure
			set3_8_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_eight: Result.upper = 8
		end
			
	set3_9: ARRAY [EV_COLOR] is
			-- Set3 gradient of 9 shades
		do
			create Result.make (1, 9)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (141, 211, 199))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (255, 255, 179))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (190, 186, 218))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (251, 128, 114))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (128, 177, 211))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (253, 180, 98))
			Result [7] := (create {EV_COLOR}.make_with_8_bit_rgb (179, 222, 105))
			Result [8] := (create {EV_COLOR}.make_with_8_bit_rgb (252, 205, 229))
			Result [9] := (create {EV_COLOR}.make_with_8_bit_rgb (217, 217, 217))
		ensure
			set3_9_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_nine: Result.upper = 9
		end
			
	set3_10: ARRAY [EV_COLOR] is
			-- Set3 gradient of 10 shades
		do
			create Result.make (1, 10)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (141, 211, 199))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (255, 255, 179))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (190, 186, 218))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (251, 128, 114))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (128, 177, 211))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (253, 180, 98))
			Result [7] := (create {EV_COLOR}.make_with_8_bit_rgb (179, 222, 105))
			Result [8] := (create {EV_COLOR}.make_with_8_bit_rgb (252, 205, 229))
			Result [9] := (create {EV_COLOR}.make_with_8_bit_rgb (217, 217, 217))
			Result [10] := (create {EV_COLOR}.make_with_8_bit_rgb (188, 128, 189))
		ensure
			set3_10_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_ten: Result.upper = 10
		end

	set3_11: ARRAY [EV_COLOR] is
			-- Set3 gradient of 11 shades
		do
			create Result.make (1, 11)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (141, 211, 199))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (255, 255, 179))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (190, 186, 218))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (251, 128, 114))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (128, 177, 211))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (253, 180, 98))
			Result [7] := (create {EV_COLOR}.make_with_8_bit_rgb (179, 222, 105))
			Result [8] := (create {EV_COLOR}.make_with_8_bit_rgb (252, 205, 229))
			Result [9] := (create {EV_COLOR}.make_with_8_bit_rgb (217, 217, 217))
			Result [10] := (create {EV_COLOR}.make_with_8_bit_rgb (188, 128, 189))
			Result [11] := (create {EV_COLOR}.make_with_8_bit_rgb (204, 235, 111))
		ensure
			set3_11_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_eleven: Result.upper = 11
		end

	set3_12: ARRAY [EV_COLOR] is
			-- Set3 gradient of 12 shades
		do
			create Result.make (1, 12)
			Result [1] := (create {EV_COLOR}.make_with_8_bit_rgb (141, 211, 199))
			Result [2] := (create {EV_COLOR}.make_with_8_bit_rgb (255, 255, 179))
			Result [3] := (create {EV_COLOR}.make_with_8_bit_rgb (190, 186, 218))
			Result [4] := (create {EV_COLOR}.make_with_8_bit_rgb (251, 128, 114))
			Result [5] := (create {EV_COLOR}.make_with_8_bit_rgb (128, 177, 211))
			Result [6] := (create {EV_COLOR}.make_with_8_bit_rgb (253, 180, 98))
			Result [7] := (create {EV_COLOR}.make_with_8_bit_rgb (179, 222, 105))
			Result [8] := (create {EV_COLOR}.make_with_8_bit_rgb (252, 205, 229))
			Result [9] := (create {EV_COLOR}.make_with_8_bit_rgb (217, 217, 217))
			Result [10] := (create {EV_COLOR}.make_with_8_bit_rgb (188, 128, 189))
			Result [11] := (create {EV_COLOR}.make_with_8_bit_rgb (204, 235, 111))
			Result [12] := (create {EV_COLOR}.make_with_8_bit_rgb (255, 237, 111))
		ensure
			set3_12_not_void: Result /= Void
			no_void_color: not Result.has (Void)
			lower_one: Result.lower = 1
			upper_twelve: Result.upper = 12
		end
		
end
