note

	description:
	"[
		Widget which enables you to display a 2 dimensional 'grid' of items, referenced by a row and column.
		Rows may be parented within other rows, thereby enabling a tree structure.
	]"
	author: "Julian Rogers and others"
	last_editor: "$Author: jq $"
	date: "$Date: 2015/03/11 $"
	revision: "$Revision: #245 $"
	path: "$File: //rose/source/kernel/infrastructure/widgets/grid/rp_data_grid.e $"

class
	RP_DATA_GRID [G]

inherit
	RP_GRID [G]
		rename
			drop_actions as rp_grid_drop_actions,
			set_pebble_function as rp_grid_set_pebble_function,
			has_focus as rp_data_grid_has_focus,
			wipe_out as ev_cell_wipe_out
		redefine
			initialize,
			pointer_motion_actions,
			pointer_button_press_actions,
			pointer_double_press_actions,
			pointer_button_release_actions,
			pointer_enter_actions,
			pointer_leave_actions,
			mouse_wheel_actions,
			key_press_actions,
			key_press_string_actions,
			key_release_actions,
			focus_in_actions,
			pick_actions,
			pick_ended_actions,
			conforming_pick_actions,
			focus_out_actions,
				-- Note that `resize_actions' are not redefined
				-- as the version from the cell applies to the grid.
				-- If it is redefined, then the current resizing implementation
				-- of the grid does not work as the RP_DATA_GRID wraps the grid into
				-- a viewport and a fixed.
			set_focus,
			set_accept_cursor,
			set_deny_cursor,
			accept_cursor,
			deny_cursor,
			foreground_color,
			background_color,
			set_foreground_color,
			set_background_color,
			set_drag_and_drop_mode,
			mode_is_drag_and_drop,
			set_pebble,
			remove_pebble,
			pebble
		end

	EV_KEY_CONSTANTS
		undefine
			copy, is_equal, default_create
		end
	RP_DATA_GRID_ACCESS
		undefine
			copy, is_equal, default_create
		end
	PROFILING_SETTING
		rename
			make as profile_make
		undefine
			copy, is_equal, default_create
		end

create
	default_create, make_from_arrays, make

create {RP_DATA_GRID}
	make_as_title_grid

feature {NONE} -- Initialization

	initialize is
			-- Initialize `Current'.
		do
			implementation.set_state_flag ({EV_ANY_I}.interface_is_initialized_flag, True)
			create grid
			create header_node.make (<<"">>)
			grid.implementation.set_data_grid (Current)
			create grid_and_view_split
			create grid_and_view_container
			create grid_and_header_container
			replace (grid_and_header_container)
			grid.enable_partial_dynamic_content
			grid_and_header_container.extend (grid)
			grid.set_dynamic_content_function (agent display)
			grid.virtual_size_changed_actions.extend (agent set_refresh_needed (?,?))
			grid.virtual_position_changed_actions.extend (agent set_refresh_needed (?,?))
			grid.virtual_position_changed_actions.extend (agent refresh_position (?,?))
			grid.resize_actions.force_extend (agent set_refresh_needed)
			grid.show_vertical_scroll_bar
			grid.show_horizontal_scroll_bar
			grid.implementation.drawer.set_redraw_started_action (agent refresh_widgets)
			grid.mouse_wheel_actions.extend (agent on_mouse_wheel (?))
			grid.item_activate_actions.extend (agent item_activated)
			grid.row_select_actions.extend (agent handle_row_selection)
			grid.row_deselect_actions.extend (agent handle_row_deselection)

			create items_to_filter_on.make_empty
			grid.set_item_pebble_function (agent pebble_agent)
			grid.focus_in_actions.extend (agent clear_filter_gui)
			grid.hide_header
			first_viewable_column := 1

			create locked_rows.make (8)
			grid.post_draw_overlay_actions.extend (agent draw_title_icons)
			grid.post_draw_overlay_actions.extend (agent draw_foreground_for_raised_title_item)
			grid.post_draw_overlay_actions.extend (agent draw_header_border)
			grid.pre_draw_overlay_actions.extend (agent draw_background_for_raised_title_item)
			grid.row_collapse_actions.extend (agent row_collapsed)
			grid.pointer_button_press_item_actions.extend (agent update_title_item_state_on_press)
			grid.pointer_button_press_item_actions.extend (agent handle_press_for_title_items)
			grid.pointer_button_release_item_actions.extend (agent update_title_item_state_on_release)
			grid.pointer_motion_item_actions.extend (agent update_title_item_state_on_motion)
			if not is_title_grid then
				grid.virtual_position_changed_actions.extend (agent update_title_bar_offset)
				grid.header.item_resize_end_actions.extend (agent update_title_width_to_match_grid_column_width)
				grid.header.item_resize_actions.extend (agent update_title_width_to_match_grid_column_width)
				grid.pointer_button_press_item_actions.put_front (agent update_column_size_on_press (?, ?, ?, ?, Current))
				grid.pointer_button_press_item_actions.extend (agent fire_node_button_press_actions)
				grid.pointer_double_press_item_actions.extend (agent update_column_width_on_double_click (?, ?, ?, ?))
				grid.pointer_double_press_item_actions.extend (agent fire_node_double_press_actions)
				grid.pointer_motion_item_actions.extend (agent fire_node_motion_actions)
				grid.pointer_button_release_item_actions.extend (agent update_column_size_on_release (?, ?, ?, ?, Current))
				grid.pointer_motion_item_actions.extend (agent update_column_size_on_motion (?, ?, ?, Current))
				grid.pointer_motion_item_actions.extend (agent update_selection_handling_on_motion (?, ?, ?, Current))
					-- Now initialize the title grid for `Current'
				create title_grid.make_as_title_grid (titles, Current)
				grid_and_header_container.put_front (title_grid)
				grid_and_header_container.disable_item_expand (title_grid)
				set_are_sort_icons_displayed (True)
				set_are_sort_icons_displayed_on_header_node (True)
			else
				enable_vertical_overscroll
				disable_row_height_fixed
			end
			initialize_notify_events
			set_titles (<<"">>)
			create sort_states
			sort_states.add_actions.extend (agent item_added_to_sort_states)
			sort_states.remove_actions.extend (agent item_removed_from_sort_states)
			create widgets_just_refreshed.make (40)
			create widgets_contained_in_fixed.make (40)
			create vertically_synchronized_grids
			create row_mappings.make (1, 0)
			vertically_synchronized_grids.add_actions.extend (agent grid_added_to_vertically_synchronized_grids)
			create horizontally_synchronized_grids
			create column_mappings.make (1, 0)
			horizontally_synchronized_grids.add_actions.extend (agent grid_added_to_horizontally_synchronized_grids)
			set_background_color (grid.background_color)
		end

	make_as_title_grid (a_titles: ARRAY [STRING_GENERAL]; a_parent: RP_DATA_GRID [G]) is
			-- Create `Current' as a title grid of `a_parent', with titles `a_titles'.
		require
			a_parent_not_void: a_parent /= Void
		do
			set_as_title_grid (a_parent)
			make (a_titles, <<>>)
			grid.disable_tabable_to
			grid.fill_background_actions.extend (agent fill_title_grid_background)
			grid.pointer_button_press_item_actions.put_front (agent update_column_size_on_press (?, ?, ?, ?, a_parent))
			grid.pointer_button_release_item_actions.extend (agent update_column_size_on_release (?, ?, ?, ?, a_parent))
			grid.pointer_motion_item_actions.extend (agent update_column_size_on_motion (?, ?, ?, a_parent))
			grid.pointer_double_press_item_actions.extend (agent update_column_width_on_double_click (?, ?, ?, ?))
			grid.focus_in_actions.extend (agent update_selection_color_of_parent)
			grid.focus_out_actions.extend (agent update_selection_color_of_parent)
			grid.pointer_button_press_item_actions.extend (agent fire_node_button_press_actions)
			grid.pointer_double_press_item_actions.extend (agent fire_node_double_press_actions)

				-- We add `additional_height_for_title_row' to account for the extra shading.
			grid.enable_vertical_overscroll
			grid.hide_vertical_scroll_bar
			grid.hide_horizontal_scroll_bar
			grid.disable_row_height_fixed
			grid.row (1).set_height (title_height)
			grid.disable_selection_on_click

				-- These two commands ensure that we are not restricted to the width of
				-- the items for the header grid, ensuring it may always match the position
				-- of the `parent_grid' if the vertical scroll bar is displayed.
			grid.enable_horizontal_overscroll
			grid.implementation.disable_horizontal_offset_set_to_zero_when_items_smaller_than_viewable_width

			grid.enable_column_separators
			grid.enable_row_separators
		ensure
			parent_set: parent_grid = a_parent
		end

feature {NONE}-- Initialization

	make (a_titles: ARRAY [STRING_GENERAL]; a_nodes: ARRAY [RP_GRID_NODE [G]]) is
			-- Create `Current' with titles `a_titles' and fill with `a_nodes'.
		do
			default_create
			populate(a_titles,a_nodes)
		ensure then
			no_rows_when_a_nodes_void: a_nodes = Void and not is_title_grid implies row_count = 0
			row_count_set: a_nodes /= Void and not is_title_grid implies row_count = a_nodes.count
		end

	make_from_arrays (a_titles: ARRAY [STRING_GENERAL]; a_rows: ARRAY [ARRAY [ANY]]) is
			-- Create `Current' with titles `a_titles' and fill with nodes created
			-- from `a_rows'.
		require
			a_rows_not_void: a_rows /= Void
			a_rows_not_has_void: not a_rows.has (Void)
		local
			i: INTEGER
			l_node: RP_GRID_NODE [G]
			l_nodes: ARRAY [RP_GRID_NODE [G]]
		do
			create l_nodes.make (1, a_rows.count)
			from
				i := 1
			until
				i > a_rows.count
			loop
				create l_node.make_from_array (a_rows.item (i))
				l_nodes.put (l_node, i)
				i := i + 1
			end
			make (a_titles, l_nodes)
		ensure
			rows_inserted: row_count = a_rows.count
		end

feature -- Access

	are_raised_titles_enabled: BOOLEAN
		-- Are all title rows exlucding `header_node' displayed with a classic raised style?

	are_raised_titles_enabled_on_header_node: BOOLEAN
		-- Is `header_node' displayed with a classic raised style?

	are_columns_resizable_on_dividers: BOOLEAN
		-- May columns be resized by clicking and dragging from anywhere
		-- on their dividers?
		-- If `False', then the columns may only be resized by users via the
		-- title row of `Current'.

	header_node: RP_GRID_TITLE_NODE [G]
		-- Node displayed as header if `is_header_shown' is `True'.
		-- Note that calling `set_titles', recreates this object, so if you wish
		-- to perform more advanced manipulation of `Current', it is suggested that
		-- you use `set_header_node' directly.
		-- Note that you must not query `grid' directly on this node as it does not refer to `Current'
		-- but instead refers to an implementation level grid.

	are_sort_icons_displayed: BOOLEAN
		-- Are sort icons displayed for title items?

	are_sort_icons_displayed_on_header_node: BOOLEAN
		-- Does `header_node' display sort_icons?

	are_selection_icons_displayed: BOOLEAN
		-- Are selection icons displayed for title items?

	are_selection_icons_displayed_on_header_node: BOOLEAN
		-- Does `header_node' display selection icons?

	are_lock_icons_displayed_on_header_node: BOOLEAN
		-- Does `header_node' display icons enabling the locking of a column?

	is_header_shown: BOOLEAN is
			-- Is a header node shown for `Current'?
		require
			not_destroyed: not is_destroyed
		do
			Result := title_grid.is_show_requested
		end

	row_at_virtual_position (a_virtual_y: INTEGER; a_ignore_locked_rows: BOOLEAN): RP_GRID_NODE [G] is
			-- Row at virtual position `a_virtual_x', `a_virtual_y' or `Void' if none.
		require
			not_destroyed: not is_destroyed
		do
			if attached {EV_GRID_ROW} grid.row_at_virtual_position (a_virtual_y, a_ignore_locked_rows) as l_row then
				if attached {RP_GRID_NODE [G]} l_row.data as l_node then
					Result := l_node
				end
			end
		end

	row_under_pointer: RP_GRID_NODE [G] is
			-- Row under pointer if any, or `Void' if none
		require
			not_destroyed : not is_destroyed
		local
			l_coordinate : EV_COORDINATE
			l_x, l_y : INTEGER
		do
			l_coordinate := virtual_pointer_position
			l_x := l_coordinate.x
			l_y := l_coordinate.y
			if is_valid_virtual_position (l_x, l_y) then
				Result := row_at_virtual_position (l_y, False)
			end
		end

	item_at_virtual_position (a_virtual_x, a_virtual_y: INTEGER): RP_DATA_GRID_ITEM is
			-- Cell at virtual position `a_virtual_x', `a_virtual_y' or
			-- `Void' if none.
		require
			not_destroyed: not is_destroyed
			valid_virtual_position : is_valid_virtual_position (a_virtual_x, a_virtual_y)
		do
			Result ?= grid.item_at_virtual_position (a_virtual_x, a_virtual_y)
		end

	item_under_pointer : RP_DATA_GRID_ITEM is
			-- Cell under pointer if any
			-- `Void' if none
		require
			not_destroyed : not is_destroyed
		local
			l_coordinate : EV_COORDINATE
			l_x, l_y : INTEGER
		do
			l_coordinate := virtual_pointer_position
			l_x := l_coordinate.x
			l_y := l_coordinate.y
			if is_valid_virtual_position (l_x, l_y) then
				Result := item_at_virtual_position (l_x, l_y)
			end
		end

	virtual_pointer_position : EV_COORDINATE is
			-- Position of pointer in virtual coordinates
		require
			not_destroyed : not is_destroyed
		local
			l_coordinate : EV_COORDINATE
			l_hheight : INTEGER
			l_x, l_y : INTEGER
		do
			l_coordinate := pointer_position
			l_x := l_coordinate.x - virtual_x_position
			if title_grid /= Void then
				l_hheight := title_grid.height
			end
			l_y := l_coordinate.y - virtual_y_position - l_hheight
			create Result.make (l_x, l_y)
		end

	is_valid_virtual_position (a_x, a_y : INTEGER) : BOOLEAN is
			-- Do `a_x' and `a_y' describe a valid virtual position?
		require
			not_destroyed : not is_destroyed
		do
			Result := (a_x > 0 and then a_x <= virtual_width) and then (a_y > 0 and then a_y <= virtual_height)
		ensure
			definition: Result = ((a_x > 0 and then a_x <= virtual_width) and then (a_y > 0 and then a_y <= virtual_height))
		end

	first_visible_column : EV_GRID_COLUMN is
			-- First column visible in `Current'.
			-- May be Void if `column_count' = 0 or total width of all columns is zero.
		require
			not_destroyed: not is_destroyed
			is_displayed: is_displayed
		do
			Result := grid.first_visible_column
		end

	is_vertical_scrolling_per_item : BOOLEAN is
			-- Is vertical scrolling performed on a per-item basis?
			-- If `True', each change of the vertical scroll bar increments the vertical
			-- offset by the current row height.
			-- If `False', the scrolling is smooth on a per-pixel basis.
		require
			not_destroyed: not is_destroyed
		do
			Result := grid.is_vertical_scrolling_per_item
		end

	is_single_row_selection_enabled: BOOLEAN is
			-- Does clicking or keyboard navigating via arrow keys select the whole row, unselecting
			-- any previously rows?
		require
			not_destroyed: not is_destroyed
		do
			Result := grid.is_single_row_selection_enabled
		end

	is_multiple_row_selection_enabled: BOOLEAN is
			-- Does clicking or keyboard navigating via arrow keys select the whole row, with multiple
			-- row selection permitted via the use of Ctrl and Shift keys?
		require
			not_destroyed: not is_destroyed
		do
			Result := grid.is_multiple_row_selection_enabled
		end

	is_multiple_item_selection_enabled: BOOLEAN is
			-- Does clicking or keyboard navigating via arrow keys select an item, with multiple
			-- item selection permitted via the use of Ctrl and Shift keys?
		require
			not_destroyed: not is_destroyed
		do
			Result := grid.is_multiple_item_selection_enabled
		end

	is_single_item_selection_enabled: BOOLEAN is
			-- Does clicking or keyboard navigating via arrow keys select an item, unselecting
			-- any previously selected items?
		require
			not_destroyed: not is_destroyed
		do
			Result := grid.is_single_item_selection_enabled
		end

	last_visible_column: EV_GRID_COLUMN is
			-- Index of last column visible in `Current' or 0 if `column_count' = 0.
			-- The last visible column may be only partially visible.
		require
			not_destroyed: not is_destroyed
			is_displayed: is_displayed
		do
			Result := grid.last_visible_column
		end

	first_visible_row: RP_GRID_NODE [G] is
			-- First row visible in `Current' or Void if `row_count' = 0
		require
			not_destroyed: not is_destroyed
			is_displayed: is_displayed
		local
			l_first_visible_row: EV_GRID_ROW
		do
			l_first_visible_row ?= grid.first_visible_row
			if l_first_visible_row /= Void then
				Result ?= l_first_visible_row.data
			end
		end

	last_visible_row: RP_GRID_NODE [G] is
			-- Last row visible in `Current' or Void if `row_count' = 0
			-- The last visible row may be only partially visible.
		require
			not_destroyed: not is_destroyed
			is_displayed: is_displayed
		local
			l_last_visible_row: EV_GRID_ROW
		do
			l_last_visible_row ?= grid.last_visible_row
			if l_last_visible_row /= Void then
				Result ?= l_last_visible_row.data
			end
		end

	visible_row_indexes: ARRAYED_LIST [INTEGER] is
			-- Indexes of all rows that are currently visible in `Current'.
			-- `Result' may not be contiguous if `is_tree_enabled' and one or more of the
			-- visible rows have subrows and are not expanded.
		require
			not_destroyed: not is_destroyed
			is_displayed: is_displayed
		do
			Result := grid.visible_row_indexes
		ensure
			result_not_void: Result /= Void
		end

	viewable_row_count: INTEGER is
			-- Total number of rows visible in `Current'.
		do
			Result := grid.viewable_row_count
		ensure
			result_non_negative: Result >= 0
		end

	viewable_row_indexes: ARRAYED_LIST [INTEGER] is
			-- Indexes of all rows that are currently viewable in the grid in its present state.
			-- For example, if the first node is a non expanded tree that has 10 subrows, the contents
			-- would be 1, 11, 12, 13, 14, ...
		require
			not_destroyed: not is_destroyed
		do
			Result := grid.viewable_row_indexes
		ensure
			result_not_void: Result /= Void
		end

	visible_column_indexes: ARRAYED_LIST [INTEGER] is
			-- All columns that are currently visible in `Current'.
			-- `Result' may not be contiguous if one or more columns are hidden.
		require
			not_destroyed: not is_destroyed
			is_displayed: is_displayed
		do
			Result := grid.visible_column_indexes
		ensure
			result_not_void: Result /= Void
		end

	viewable_column_indexes: ARRAYED_LIST [INTEGER] is
			-- Indexes of all columns that are currently viewable in the grid in its present state.
			-- in essence, the only columns not included are ones that have had `hide' called
		require
			not_destroyed: not is_destroyed
		local
			i: INTEGER
		do
			create Result.make (column_count)
			from i := 1 until i > column_count loop
				if column_displayed (i) then
					Result.extend (i)
				end
				i := i + 1
			end
		ensure
			result_not_void: Result /= Void
		end

	virtual_x_position: INTEGER is
			-- Horizontal offset of viewable area in relation to the left edge of
			-- the virtual area in pixels.
		require
			not_destroyed: not is_destroyed
		do
			Result := grid.virtual_x_position
		ensure
			valid_result: Result >= 0 and Result <= maximum_virtual_x_position
		end

	virtual_y_position: INTEGER is
			-- Vertical offset of viewable area in relation to the top edge of
			-- the virtual area in pixels.
		require
			not_destroyed: not is_destroyed
		do
			Result := grid.virtual_y_position
		ensure
			valid_result: Result >= 0 and Result <= maximum_virtual_y_position
		end

	all_parents_are_expanded (a_row: RP_GRID_NODE [G]): BOOLEAN is
			-- Are all parents expanded for row `a_row'?
			-- Note that the EV_GRID implementation retains row indices for child rows of
			-- parents which have been expanded and collapsed. If we could access this, it would be quicker.
		require
			a_row_not_void: a_row /= Void
		local
			l_parent: RP_GRID_NODE [G]
		do
			Result := True
			l_parent := a_row
			from
			until
				l_parent = Void or not Result
			loop
				l_parent := l_parent.parent_row
				if l_parent /= Void and then not l_parent.is_expanded then
					Result := False
				end
			end
		end

	selected_nodes: ARRAYED_LIST [RP_GRID_NODE [G]] is
			-- All nodes selected in `Current'.
		local
			l_rows: ARRAYED_LIST [EV_GRID_ROW]
			l_grid_node: RP_GRID_NODE [G]
		do
			l_rows := grid.selected_rows
			create Result.make (0)
			from
				l_rows.start
			until
				l_rows.off
			loop
				l_grid_node ?= l_rows.item.data
				Result.extend (l_grid_node)
				l_rows.forth
			end
		ensure
			result_not_void: result /= Void
		end

	is_item_visible (a_item: RP_DATA_GRID_ITEM): BOOLEAN is
			-- Is `a_item' currently visible in `Current'?
		require
			a_item_not_void: a_item /= Void
		local
			l_row_index, l_column_index : INTEGER
			l_node : RP_GRID_NODE [G]
		do
			l_row_index := a_item.row.index
			l_column_index := a_item.index_in_row
			l_node := row (a_item.row.index)
			if l_node /= Void then
				Result := l_node.is_visible and (l_column_index <= grid.last_visible_column.index and l_column_index >= grid.first_visible_column.index and grid.column (l_column_index).is_displayed)
			end
		end

	next_editable_in_row (a_item: RP_DATA_GRID_ITEM; a_direction: INTEGER): RP_DATA_GRID_ITEM is
			-- `Result' is next editable item in row, starting at item `a_item', and iterating
			-- the logical column index by `a_direction'.
		require
			a_item_not_void: a_item /= Void
			a_direction_valid: a_direction = 1 or else a_direction = -1
		local
			l_node: RP_GRID_NODE [G]
			i: INTEGER
		do
			if a_item.is_parented then
				l_node ?= a_item.row.data
				from
					i := a_item.index_in_row + a_direction
				until
					Result /= Void or else i > l_node.column_count or else i < 1
				loop
					Result := l_node.i_th (i)
					if Result /= Void and then not Result.is_editable then
						Result := Void
					end
					i := i + a_direction
				end
			end
		end

	next_editable (a_item: RP_DATA_GRID_ITEM; a_direction: INTEGER): RP_DATA_GRID_ITEM is
			-- `Result' is next editable item in `Current', starting at item `a_item', and iterating
			-- through the logical row and column indexes by `a_direction'.
		require
			a_item_not_void: a_item /= Void
			a_direction_valid: a_direction = 1 or else a_direction = -1
		local
			l_item: RP_DATA_GRID_ITEM
			l_liminal_index, i: INTEGER
			l_node: RP_GRID_NODE [G]
		do
			if a_item /= Void and then a_item.is_parented then
				Result := next_editable_in_row (a_item, a_direction)
				if Result = Void then
					l_item := a_item
					l_node ?= a_item.row.data
					if a_direction = -1 then
						l_liminal_index := first_visible_row.index
					elseif a_direction = 1 then
						l_liminal_index := last_visible_row.index
					end
					from
						i := (l_node.index + a_direction)
					until
						i = (l_liminal_index + a_direction + a_direction) or Result /= Void
					loop
						if is_valid_row_index (i) and then all_parents_are_expanded (row (i)) then
							if Result = Void then
								l_node := row (i)
								if a_direction = 1 then
									l_item := l_node.i_th (1)
								else
									l_item := l_node.i_th (l_node.column_count)
								end
							end
							if l_item.is_editable then
								Result := l_item
							else
								Result := next_editable_in_row (l_item, a_direction)
							end
							if Result /= Void then
								l_node.ensure_visible
							end
						end
						i := i + a_direction
					end
				end
			end
		end

	is_valid_row_index (a_index: INTEGER): BOOLEAN is
			-- Does `a_index' correspond to the index of a row within `Current'?
		do
			Result := a_index > 0 and a_index <= row_count
		end

	is_valid_column_index (a_index : INTEGER) : BOOLEAN is
			-- Does `a_index' correspond to the index of a colunm within `Current'?
		do
			Result := a_index > 0 and a_index <= column_count
		end

	row (i: INTEGER): RP_GRID_NODE [G] is
			-- `Result' is row at index `i'
		require
			is_valid_row_index: is_valid_row_index (i)
		do
			Result ?= grid.row (i).data
		ensure
			result_not_void: Result /= Void
		end

	selected_items: ROSE_LINEAR_ARRAY [RP_DATA_GRID_ITEM] is
			-- All items selected in `Current'.
		local
			l_list: ARRAYED_LIST [EV_GRID_ITEM]
			l_item: RP_DATA_GRID_ITEM
		do
			create Result.make_empty
			l_list := grid.selected_items
			from
				l_list.start
			until
				l_list.off
			loop
				l_item ?= l_list.item
				if l_item /= Void then
					Result.extend (l_item)
				end
				l_list.forth
			end
		ensure
			result_not_void: Result /= Void
		end

	first_row: RP_GRID_NODE [G] is
			-- First row contained in `Current'.
		do
			if row_count > 0 then
				Result := row (1)
			end
		ensure
			result_void_when_empty: row_count = 0 implies Result = Void
			result_not_void_when_not_empty: row_count > 0 implies Result /= Void
		end

	last_row: RP_GRID_NODE [G] is
			-- Last row contained in `Current'.
		do
			if row_count > 0 then
				Result := row (row_count)
			end
		ensure
			result_void_when_empty: row_count = 0 implies Result = Void
			result_not_void_when_not_empty: row_count > 0 implies Result /= Void
		end

	are_sticky_parents_enabled: BOOLEAN is
			-- Are the parent rows of the topmost row in `Current'
			-- always displayed to show the context for that row within
			-- the tree structure?
		do
			Result := update_sticky_rows_agent /= Void
		end

	is_vertical_scrolling_per_item_relative_to_header : BOOLEAN is
			-- Is scrolling per per item relative to bottom of locked header rows?
		do
			Result := vertical_scrolling_per_item_relative_to_header_agent /= Void
		end

	virtual_height: INTEGER is
			-- Height of virtual area in pixels.
		require
			not_destroyed: not is_destroyed
		do
			Result := grid.virtual_height
		ensure
			result_non_negative: Result >= 0
		end

	virtual_width: INTEGER is
			-- Width of virtual area in pixels.
		require
			not_destroyed: not is_destroyed
		do
			Result := grid.virtual_width
		ensure
			result_non_negative: Result >= 0
		end

	maximum_virtual_x_position: INTEGER is
				-- Maximum permitted virtual x position based on current dimensions and properties.
		require
			not_destroyed: not is_destroyed
		do
			Result := grid.maximum_virtual_x_position
		ensure
			result_non_negative: Result >= 0
		end

	maximum_virtual_y_position: INTEGER is
			-- Maximum permitted virtual y position based on current dimensions and properties.
		require
			not_destroyed: not is_destroyed
		do
			Result := grid.maximum_virtual_y_position
		ensure
			result_non_negative: Result >= 0
		end

	column_displayed (a_column: INTEGER): BOOLEAN is
			-- May column `a_column' be displayed when `Current' is?
			-- Will return False if `hide' has been called on column `a_column'.
			-- A value of True does not signify that column `a_column' is visible on screen at that particular time.
		require
			not_destroyed: not is_destroyed
			a_column_within_bounds: a_column > 0 and a_column <= column_count
		do
			Result := grid.column_displayed (a_column)
		end

	is_row_height_fixed: BOOLEAN is
			-- Must all rows in `Current' have the same height?
		require
			not_destroyed: not is_destroyed
		do
			Result := grid.is_row_height_fixed
		end

	is_vertical_overscroll_enabled: BOOLEAN is
			-- Does the virtual height of `Current' include the
			-- position of the final row plus the `viewable_height'.
			-- If `True', this enables vertical scrolling until the last row
			-- is at the very top of the viewable area. If `False', scrolling
			-- may be performed until the last row is at the bottom of the viewable
			-- area.
		require
			not_destroyed: not is_destroyed
		do
			Result := grid.is_vertical_overscroll_enabled
		end

	row_height: INTEGER is
			-- Height of all rows within `Current'. Only has an effect on `Current'
			-- while `is_row_height_fixed', otherwise the individual height of each
			-- row is used directly.
		require
			not_destroyed: not is_destroyed
			is_row_height_fixed: is_row_height_fixed
		do
			Result := grid.row_height
		ensure
			result_non_negative: result >= 0
		end

	are_column_separators_enabled: BOOLEAN is
			-- Is a vertical separator displayed in color `separator_color' between each column?
		require
			not_destroyed: not is_destroyed
		do
			Result := grid.are_column_separators_enabled
		end

	are_row_separators_enabled: BOOLEAN is
			-- Is a horizontal separator displayed in color `separator_color' between each row?
		require
			not_destroyed: not is_destroyed
		do
			Result := grid.are_row_separators_enabled
		end

	has_focus: BOOLEAN is
			-- Does widget have the keyboard focus?
		require
			not_destroyed: not is_destroyed
		do
			Result := grid.has_focus
		end

	row_count: INTEGER is
			-- Number of rows in Current.
		require else
			not_destroyed: not is_destroyed
		do
			Result := grid.row_count
		end

	column (a_column: INTEGER): RP_DATA_GRID_COLUMN [G] is
			-- Column at index `a_column'.
		require
			not_destroyed: not is_destroyed
			valid_column: is_valid_column_index (a_column)
		do
			Result ?= grid.column (a_column).data
			if Result = Void then
				create Result
				grid.column (a_column).set_data (Result)
				Result.set_column (grid.column (a_column))
				Result.set_parent (Current)
			end
		ensure
			column_not_void: Result /= Void
		end

	is_tree_enabled : BOOLEAN is
			-- Is tree functionality enabled?
			-- Must be `True' to perform any tree structure functions on `Current'.
			-- Use `enable_tree' and `disable_tree' to set this state.
		require
			not_destroyed: not is_destroyed
		do
			Result := grid.is_tree_enabled
		end

	column_count : INTEGER is
			-- Number of columns in Current.
		require
			not_destroyed: not is_destroyed
		do
			Result := grid.column_count
		ensure
			result_not_negative: Result >= 0
		end

	total_row_height : INTEGER is
			-- Sum of heights of all rows.
		local
			i : INTEGER
		do
			if is_row_height_fixed then
				Result := row_count * row_height
			else
				from i := 1 until i > row_count loop
					Result := Result + row (i).height
					i := i + 1
				end
			end
		ensure
			result_non_negative: Result >= 0
		end

	sort_states: ACTIVE_LIST [RP_DATA_GRID_SORT_STATE]
		-- Information specifying sort behavior at next call to `sort'.
		-- Each item represents a sort within the previous item (if any).
		-- For example, to sort ascending on column 3 and then descending on
		-- column 4 within the sort of column 3, perform the following (assuming `sort_states' is already empty):
		-- sort_states.extend (create {RP_DATA_GRID_SORT_STATE}.make_with_column_and_direction (3, True))
		-- sort_states.extend (create {RP_DATA_GRID_SORT_STATE}.make_with_column_and_direction (4, False))
		-- Modifying the contents of `Current' causes the associated sort icons for a column (if displayed) to
		-- be updated to reflect the new state.
		-- Note that the contents of `sort_states' may be modified in repsonse to a user selecting the
		-- sort icons if displayed.

	sort_comparator: FUNCTION [ANY, TUPLE [BOOLEAN, RP_GRID_NODE [G], RP_GRID_NODE [G]], BOOLEAN]
		-- Agent used to override the default sorting comparison provided by the grid.
		-- The three arguments are:
		-- 1. The comparison result from the internal grid sort routine.
		-- 2. The first node for comparison
		-- 3. The second node for comparison
		-- Note that if your sort routine returns True when the nodes are the same, then this will cause a crash
		-- in the internals of the sorting.

	set_sort_comparator (a_comparator: like sort_comparator) is
			-- Assign `a_comparator' to `sort_comparator'.
		do
			sort_comparator := a_comparator
		ensure
			sort_comparator_set: sort_comparator = a_comparator
		end

	separator_color: EV_COLOR is
			-- Color used to display column and row separators.
		require
			not_destroyed: not is_destroyed
		do
			Result := grid.separator_color
		ensure
			result_not_void: Result /= Void
		end

	is_selection_on_click_enabled: BOOLEAN is
			-- Will an item be selected if clicked upon?
		require
			not_destroyed: not is_destroyed
		do
			Result := grid.is_selection_on_click_enabled
		end

	is_selection_on_single_button_click_enabled: BOOLEAN is
			-- Will an item be selected if clicked upon via mouse button `1' only?
		require
			not_destroyed: not is_destroyed
		do
			Result := grid.is_selection_on_single_button_click_enabled
		end

	to_array2: ARRAY2 [STRING] is
			-- Copy contents of grid to a two-dimensional array, one row per line.
		do
			Result := columns_to_array2_implementation (Void, True, False, False)
		ensure
			array2_not_void: Result /= Void
			no_void_strings: not Result.has (Void)
		end

	to_array2_filtered: ARRAY2 [STRING] is
			-- Copy contents of grid to a two-dimensional array, one row per
			-- line, don't include filtered rows
		do
			Result := columns_to_array2_implementation (Void, True, False, True)
		ensure
			array2_not_void: Result /= Void
			no_void_strings: not Result.has (Void)
		end

	columns_to_array2 (a_columns: ARRAY [INTEGER]; a_with_titles: BOOLEAN; a_only_selected_rows: BOOLEAN): ARRAY2 [STRING] is
			-- Copy contents of `a_columns' to an array, one row per line.
			-- Include column headers if `a_with_titles' is True.
			-- Take only selected rows into account if `a_only_selected_rows' is
			-- True.
		do
			Result := columns_to_array2_implementation (Void, True, False, False)
		end

	columns_to_string (a_columns: ARRAY [INTEGER]; a_with_titles: BOOLEAN; a_only_selected_rows: BOOLEAN; a_item_seperator: STRING): STRING is
				-- Copy contents of `a_columns' to a string, one row per line, with items seperated by `a_item_seperator'.
				-- Include column headers if `a_with_titles' is True.
				-- Take only selected rows into account if `a_only_selected_rows' is True.
		require
			columns_not_void_implies_valid_column_indexes: a_columns /= Void implies a_columns.for_all (agent is_valid_column_index)
			local
				l_array2: ARRAY2 [STRING]
			do
				create Result.make_empty
				l_array2 := columns_to_array2 (a_columns, a_with_titles, a_only_selected_rows)
				l_array2.do_all (agent columns_to_string_do_all_implementation (?, Result, a_item_seperator, l_array2.width, create {INTEGER_REF}))
			ensure
				Result_not_void: Result /= Void
		end

	focused_selection_color: EV_COLOR is
			-- Color used to show selection within items while focused.
		require
			not_is_destroyed: not is_destroyed
		do
			Result := grid.focused_selection_color
		ensure
			result_not_void: Result /= Void
		end

	non_focused_selection_color: EV_COLOR is
			-- Color used to show selection within items while not focused.
		require
			not_is_destroyed: not is_destroyed
		do
			Result := grid.non_focused_selection_color
		ensure
			result_not_void: Result /= Void
		end

	focused_selection_text_color: EV_COLOR is
			-- Color used to for text of selected items while focused.
		require
			not_is_destroyed: not is_destroyed
		do
			Result := grid.focused_selection_text_color
		ensure
			result_not_void: Result /= Void
		end

	non_focused_selection_text_color: EV_COLOR is
			-- Color used for text of selected items while not focused.
		require
			not_is_destroyed: not is_destroyed
		do
			Result := grid.non_focused_selection_text_color
		ensure
			result_not_void: Result /= Void
		end

	is_locked: BOOLEAN is
			-- Are all graphical updates to `Current' suppressed until
			-- `unlock_update' is called.
		require
			not_is_destroyed: not is_destroyed
		do
			Result := grid.is_locked
		end

	set_drag_and_drop_mode is
			-- Set user interface mode to drag and drop.
		do
			grid.set_drag_and_drop_mode
		end

	mode_is_drag_and_drop: BOOLEAN
			-- Is the user interface mode drag and drop?
		do
			Result := grid.mode_is_drag_and_drop
		end

	accept_cursor: EV_POINTER_STYLE is
			-- `Result' is cursor displayed when the screen pointer is over a
			-- target that accepts `pebble' during pick and drop.
		do
			Result := grid.accept_cursor
		end

	deny_cursor: EV_POINTER_STYLE is
			-- `Result' is cursor displayed when the screen pointer is over a
			-- target that does not accept `pebble' during pick and drop.
		do
			Result := grid.deny_cursor
		end

	foreground_color: EV_COLOR is
			-- Color of foreground features like text.
		do
			Result := grid.foreground_color
		end

	background_color: EV_COLOR is
			-- Color displayed behind foreground features.
		do
			Result := grid.background_color
		end

	is_read_only: BOOLEAN
			-- If `True', users are not able to delete selected rows via the delete key.

	tooltip: STRING is
			-- Tooltip displayed on `Current'.
			-- If `Result' is empty then no tooltip displayed.
		require
			not_destroyed: not is_destroyed
		do
			Result := grid.tooltip
		ensure
			not_void: Result /= Void
		end

	pebble: ANY is
			-- Data to be transported by pick and drop mechanism.
		do
			Result := grid.pebble
		end

	resizing_column: RP_DATA_GRID_COLUMN [G] is
		-- Column which is currently being resized in `Current', `Void' if none.
		require
			not_destroyed: not is_destroyed
		do
			if is_title_grid then
				Result := resizing_column_internal
			else
				if title_grid.resizing_column_internal /= Void then
					Result := column (title_grid.resizing_column_internal.index)
				end
			end
		end

	is_vertical_scroll_bar_show_requested: BOOLEAN is
			-- Will a vertical scroll bar be displayed in `Current' when
			-- `virtual_height' exceeds `viewable_height'?
		do
			Result := grid.is_vertical_scroll_bar_show_requested
		end

	is_horizontal_scroll_bar_show_requested: BOOLEAN is
			-- Will a horizontal scroll bar be displayed in `Current' when
			-- `virtual_width' exceeds `viewable_width'?
		do
			Result := grid.is_horizontal_scroll_bar_show_requested
		end

	activated_item: RP_DATA_GRID_ITEM is
			-- Item that has currently been activated, if any.
		require
			not_destroyed: not is_destroyed
		do
			Result ?= grid.activated_item
		end

feature -- Status setting

	lock_update is
			-- Ensure `is_locked' is `True', thereby preventing graphical
			-- updates until `unlock_update' is called.
		require
			not_is_destroyed: not is_destroyed
		do
			grid.lock_update
		ensure
			is_locked: is_locked
		end

	unlock_update is
			-- Ensure `is_locked' is `False', thereby ensuring graphical
			-- updates occur as normal. The complete client area
			-- is refreshed to synchronize the display with the contents.
		require
			not_is_destroyed: not is_destroyed
		do
			grid.unlock_update
		ensure
			not_is_locked: not is_locked
		end

	set_is_vertical_scrolling_per_item (a_bool : BOOLEAN) is
			-- Set option vertical scrolling is performed on a per-item basis.
		do
			if a_bool then
				grid.enable_vertical_scrolling_per_item
			else
				grid.disable_vertical_scrolling_per_item
			end
		end

	set_column_count_to (a_column_count: INTEGER) is
			-- Resize `Current' to have `a_column_count' columns.
		require
			not_destroyed: not is_destroyed
			a_column_count_positive: a_column_count >= 0
		local
			l_upper: INTEGER
		do
			grid.set_column_count_to (a_column_count)
			if not is_title_grid then
				title_grid.set_column_count_to (a_column_count)
			end
			if a_column_count = 0 then
				set_titles (create {ARRAY [STRING]}.make (1, 0))
			else
				l_upper := a_column_count.min (titles.upper)
				if not titles.is_empty then
					set_titles (titles.subarray (1, l_upper))
				else
					set_titles (<<>>)
				end
			end
		ensure
			column_count_set: column_count = a_column_count
			titles_count_matches_new_column_count: titles.count = a_column_count
			header_node_recreated: header_node /= old header_node
		end

	set_header_node (a_header_node: RP_GRID_TITLE_NODE [G]) is
			-- Assign `a_header_node' to `header_node'.
		require
			a_header_node_not_void: a_header_node /= Void
		local
			i: INTEGER
		do
			title_grid.remove_all_nodes
			title_grid.extend (a_header_node)
			header_node := a_header_node
			if column_count < a_header_node.count then
				grid.set_column_count_to (a_header_node.count)
				from
					i := column_count + 1
				until
					i > a_header_node.count
				loop
					column (i).set_width (title_grid.column (i).width)
					i := i + 1
				end
			end
			if not header_node.secondary_title_nodes.is_empty then
				from header_node.secondary_title_nodes.start until header_node.secondary_title_nodes.off loop
					title_grid.extend (header_node.secondary_title_nodes.item)
					header_node.secondary_title_nodes.forth
				end
			end
			update_title_height
		ensure
			header_node_assigned: header_node = a_header_node
		end

	set_are_raised_titles_enabled (raised: BOOLEAN) is
			-- Set `are_raised_titles_enabled' to `raised'.
		do
			are_raised_titles_enabled := raised
			grid.redraw
			if title_grid /= Void then
				title_grid.grid.redraw
			end
		ensure
			are_raised_titles_enabled_set: are_raised_titles_enabled = raised
		end

	set_are_raised_titles_enabled_on_header_node (raised: BOOLEAN) is
			-- Set `are_raised_titles_enabled_on_header_node' to `raised'.
		do
			are_raised_titles_enabled_on_header_node := raised
			grid.redraw
			if title_grid /= Void then
				title_grid.grid.redraw
			end
		ensure
			are_raised_titles_enabled_set: are_raised_titles_enabled_on_header_node = raised
		end

	set_are_sticky_parents_enabled (enabled: BOOLEAN) is
			-- Set `are_sticky_parents_enabled' to `enabled'.
		do
			if enabled then
				if not are_sticky_parents_enabled then
					update_sticky_rows_agent := agent update_sticky_rows
					virtual_position_changed_actions.extend (update_sticky_rows_agent)
					update_sticky_rows_agent.call ([virtual_x_position, virtual_y_position])
				end
			else
				if are_sticky_parents_enabled then
					virtual_position_changed_actions.prune_all (update_sticky_rows_agent)
					update_sticky_rows_agent := Void
					from
						locked_rows.start
					until
						locked_rows.is_empty
					loop
						locked_rows.item.row.unlock
						locked_rows.remove
					end
				end
			end
		ensure
			are_sticky_parents_enabled_set: are_sticky_parents_enabled = enabled
		end

	set_is_vertical_scrolling_per_item_relative_to_header (a_enabled: BOOLEAN) is
			-- Set `is_vertical_scrolling_per_item_relative_to_header' to `a_enabled'.
--		require
--			not_vertical_scroll_per_item_enabled : a_enabled implies not is_vertical_scrolling_per_item
		do
			if a_enabled then
				enable_vertical_overscroll
				set_is_vertical_scrolling_per_item (False)
				disable_row_height_fixed
				if not is_vertical_scrolling_per_item_relative_to_header then
					vertical_scrolling_per_item_relative_to_header_agent := agent vertical_scrolling_per_item_relative_to_header
						-- As `vertical_scrolling_per_item_relative_to_header_agent' modifies virtual_position
						-- it must be fired first so that the grid can be in its "final state"
						-- when subsequent virtual_position_changed_actions are fired
					virtual_position_changed_actions.put_front (vertical_scrolling_per_item_relative_to_header_agent)
					previous_virtual_position.set (virtual_x_position, virtual_y_position)
					vertical_scrolling_per_item_relative_to_header_agent.call ([virtual_x_position, virtual_y_position])
				end
			else
				if is_vertical_scrolling_per_item_relative_to_header then
					virtual_position_changed_actions.prune_all (vertical_scrolling_per_item_relative_to_header_agent)
					vertical_scrolling_per_item_relative_to_header_agent := Void
				end
			end
		ensure
			are_sticky_parents_enabled_set: is_vertical_scrolling_per_item_relative_to_header = a_enabled
		end

	set_are_columns_resizable_on_dividers_enabled (enabled: BOOLEAN) is
			-- Set `are_columns_resizable_on_dividers' to `enabled'.
		do
			are_columns_resizable_on_dividers := enabled
		ensure
			are_columns_resizable_on_dividers_set: are_columns_resizable_on_dividers = enabled
		end

	set_are_sort_icons_displayed (enabled: BOOLEAN) is
			-- Set `are_sort_icons_displayed' to `enabled'.
		do
			are_sort_icons_displayed := enabled
			grid.redraw
		ensure
			are_sort_icons_displayed_set: are_sort_icons_displayed = enabled
		end

	set_are_sort_icons_displayed_on_header_node (enabled: BOOLEAN) is
			-- Set `are_sort_icons_displayed_on_header_node' to `enabled'.
		do
			are_sort_icons_displayed_on_header_node := enabled
			title_grid.grid.redraw
		ensure
			are_sort_icons_displayed_set: are_sort_icons_displayed_on_header_node = enabled
		end

	set_are_selection_icons_displayed (enabled: BOOLEAN) is
			-- Set `are_selection_icons_displayed' to `enabled'.
		do
			are_selection_icons_displayed := enabled
			grid.redraw
		ensure
			are_selection_icons_displayed_set: are_selection_icons_displayed = enabled
		end

	set_are_selection_icons_displayed_on_header_node (enabled: BOOLEAN) is
			-- Set `are_selection_icons_displayed_on_header_node' to `enabled'.
		do
			are_selection_icons_displayed_on_header_node := enabled
			title_grid.grid.redraw
		ensure
			are_selection_icons_displayed_set: are_selection_icons_displayed_on_header_node = enabled
		end

	set_are_lock_icons_displayed_on_header_node (enabled: BOOLEAN) is
			-- Set `are_lock_icons_displayed_on_header_node' to `enabled'.
		do
			are_lock_icons_displayed_on_header_node := enabled
			title_grid.grid.redraw
		ensure
			are_lock_icons_displayed_set: are_lock_icons_displayed_on_header_node = enabled
		end

	set_focus is
			-- Grab keyboard focus.
		do
			grid.set_focus
		end

	set_focused_selection_color (a_color: EV_COLOR) is
			-- Assign `a_color' to `focused_selection_color'.
		require
			not_is_destroyed: not is_destroyed
			a_color_not_void: a_color /= Void
		do
			grid.set_focused_selection_color (a_color)
		ensure
			focused_selection_color_set: focused_selection_color = a_color
		end

	set_non_focused_selection_color (a_color: EV_COLOR) is
			-- Assign `a_color' to `non_focused_selection_color'.
		require
			not_is_destroyed: not is_destroyed
			a_color_not_void: a_color /= Void
		do
			grid.set_non_focused_selection_color (a_color)
		ensure
			non_focused_selection_color_set: non_focused_selection_color = a_color
		end

	set_focused_selection_text_color (a_color: EV_COLOR) is
			-- Assign `a_color' to `focused_selection_text_color'.
		require
			not_is_destroyed: not is_destroyed
			a_color_not_void: a_color /= Void
		do
			grid.set_focused_selection_text_color (a_color)
		ensure
			focused_selection_text_color_set: focused_selection_text_color = a_color
		end

	set_non_focused_selection_text_color (a_color: EV_COLOR) is
			-- Assign `a_color' to `non_focused_selection_text_color'.
		require
			not_is_destroyed: not is_destroyed
			a_color_not_void: a_color /= Void
		do
			grid.set_non_focused_selection_text_color (a_color)
		ensure
			non_focused_selection_text_color_set: non_focused_selection_text_color = a_color
		end

	enable_vertical_overscroll is
			-- Ensure `is_vertical_overscroll_enabled' is `True'.
		require
			not_destroyed: not is_destroyed
		do
			grid.enable_vertical_overscroll
		ensure
			is_vertical_overscroll_enabled: is_vertical_overscroll_enabled
		end

	disable_vertical_overscroll is
			-- Ensure `is_vertical_overscroll_enabled' is `False'.
		require
			not_destroyed: not is_destroyed
			is_row_height_fixed: is_row_height_fixed
		do
			grid.disable_vertical_overscroll
		ensure
			not_is_vertical_overscroll_enabled: not is_vertical_overscroll_enabled
		end

	hide_column (a_column: INTEGER) is
			-- Ensure column `a_column' is not visible in `Current'.
		require
			not_destroyed: not is_destroyed
			a_column_within_bounds: a_column > 0 and a_column <= column_count
		do
				grid.hide_column (a_column)
				if not is_title_grid then
					title_grid.hide_column (a_column)
				end
		ensure
			column_not_displayed: not column_displayed (a_column)
		end

	show_column (a_column: INTEGER) is
			-- Ensure column `a_column' is visible in `Current'.
		require
			not_destroyed: not is_destroyed
			a_column_within_bounds: a_column > 0 and a_column <= column_count
		do
			grid.show_column (a_column)
			if not is_title_grid then
				title_grid.show_column (a_column)
			end
		ensure
			column_displayed: column_displayed (a_column)
		end

	remove_selection is
			-- Ensure that `selected_items' and `selected_nodes' are empty.
		require
			not_destroyed: not is_destroyed
		do
			grid.remove_selection
		ensure
			selected_items_empty: selected_items.is_empty
			selected_nodes_empty: selected_nodes.is_empty
		end

	select_row (a_row_index: INTEGER) is
			-- Ensure all items in `a_row_index' are selected.
		require
			not_destroyed: not is_destroyed
			valid_row: is_valid_row_index (a_row_index)
		do
			grid.select_row (a_row_index)
		ensure
			row_selected: row (a_row_index).is_selected
		end

	enable_multiple_item_selection is
			-- Allow the user to select more than one item via clicking or navigating using the keyboard arrow keys.
			-- Multiple items may be selected via Ctrl and Shift keys.
		require
			not_destroyed: not is_destroyed
		do
			grid.enable_multiple_item_selection
		ensure
			multiple_item_selection_enabled: is_multiple_item_selection_enabled
		end

	enable_single_item_selection is
			-- Allow the user to select a single item via clicking or navigating using the keyboard arrow keys.
		require
			not_destroyed: not is_destroyed
		do
			grid.enable_single_item_selection
		ensure
			single_item_selection_enabled: is_single_item_selection_enabled
		end

	enable_column_separators is
		-- Ensure `are_column_separators_enabled' is `True'.
		require
			not_destroyed: not is_destroyed
		do
			grid.enable_column_separators
		ensure
			column_separators_enabled: are_column_separators_enabled
		end

	enable_row_separators is
			-- Ensure `are_row_separators_enabled' is `True'.
		require
			not_destroyed: not is_destroyed
		do
			grid.enable_row_separators
		ensure
			row_separators_enabled: are_row_separators_enabled
		end

	disable_row_height_fixed is
				-- Permit rows to have varying heights.
		require
			not_destroyed: not is_destroyed
			is_vertical_overscroll_enabled: is_vertical_overscroll_enabled
		do
			grid.disable_row_height_fixed
		ensure
			row_height_variable: not is_row_height_fixed
		end

	enable_row_height_fixed is
				-- Forbid rows to have varying heights.
		require
			not_destroyed: not is_destroyed
		do
			grid.enable_row_height_fixed
		ensure
			row_height_variable: is_row_height_fixed
		end

	set_is_header_shown (a_show : BOOLEAN) is
			-- Set `is_header_shown' to `a_show'.
		do
			if a_show then
				title_grid.show
			else
				title_grid.hide
			end
		ensure
			is_header_shown: is_header_shown = a_show
		end

	resize_columns_to_content (include_header: BOOLEAN) is
			-- Resize all displayed columns of `Current' to required width of content.
			-- If `include_header' then include required width of header.
		require
			is_displayed: is_displayed
		local
			i: INTEGER
		do
			from
				i := 1
			until
				i > column_count
			loop
				if column (i).is_displayed then
					column (i).resize_to_content (include_header)
				end
				i := i + 1
			end
		end

	resize_columns_to_required_width_of_item_span (a_low, a_high: INTEGER; a_include_header: BOOLEAN) is
			-- Resize all columns to required width of items,
			-- for items contained within (a_start, a_end) range of rows,
			-- (0,0) for header only.
		require
			valid_low: a_low >= 0
			valid_high: a_high >= a_low and a_high <= row_count
		do
			do_all_columns (agent {RP_DATA_GRID_COLUMN [G]}.resize_to_required_width_of_item_span (a_low, a_high, a_include_header))
		end

	set_column_widths (a_widths : ARRAY[INTEGER]) is
			-- Assign contents of `a_widths' to width of columns in `Current'
			-- from `a_widths.lower' to `a_widths.upper'.
		require
			valid_widths: a_widths /= Void
			valid_lower_column: is_valid_column_index (a_widths.lower)
			valid_upper_column: is_valid_column_index (a_widths.upper)
		local
			i : INTEGER
		do
			from i := a_widths.lower until i > a_widths.upper loop
				set_column_width(a_widths.item(i), i)
				i := i + 1
			end
		end

	set_column_width (a_width, a_column: INTEGER) is
			-- Assign `a_width' to `width' of column `a_column'.
		require
			valid_width: a_width >= 0
			valid_column: is_valid_column_index (a_column)
		do
			column (a_column).set_width (a_width)
		ensure
			width_set: column (a_column).width = a_width
		end

	set_pebble_function (a_function: like grid_pebble_agent) is
			-- Assign `a_function' to `grid_pebble_agent'.
		do
			grid_pebble_agent := a_function
		ensure
			grid_pebble_agent_set: grid_pebble_agent = a_function
		end

	enable_multiple_row_selection is
			-- Allow the user to select more than one row via clicking or navigating using the keyboard arrow keys.
			-- Multiple rows may be selected via Ctrl and Shift keys.
		require
			not_is_destroyed: not is_destroyed
		do
			grid.enable_multiple_row_selection
		ensure
			multiple_row_selection_enabled: is_multiple_row_selection_enabled
		end

	enable_single_row_selection is
			-- Allow the user to select a single row via clicking or navigating using the keyboard arrow keys.
		require
			not_destroyed: not is_destroyed
		do
			grid.enable_single_row_selection
		ensure
			single_row_selection_enabled: is_single_row_selection_enabled
		end

	set_item_pebble_function (a_function: FUNCTION [ANY, TUPLE [EV_GRID_ITEM], ANY]) is
			-- Set item pebble function for pick and drop.
			-- (forward to EV_GRID)
		do
			grid.set_item_pebble_function (a_function)
		end

	set_row_height (a_row_height: INTEGER) is
			-- Set height of all rows within `Current' to `a_row_height'
			-- If not `is_row_height_fixed' then use the height individually per row instead.
		require
			not_destroyed: not is_destroyed
			is_row_height_fixed: is_row_height_fixed
			a_row_height_positive: a_row_height >= 1
		do
			grid.set_row_height (a_row_height)
		ensure
			row_height_set: row_height = a_row_height
		end

	set_titles (a_titles: ARRAY [STRING_GENERAL]) is
			-- Set titles to `a_titles', which are displayed in `header_node'.
			-- Note that `header_node' is recreated.
		local
			i: INTEGER
			l_string_32: STRING_32
		do
			if a_titles = Void then
				create titles.make (1, 0)
			else
				create titles.make (a_titles.lower, a_titles.upper)
				from
					i := a_titles.lower
				until
					i > a_titles.upper
				loop
					l_string_32 := a_titles.item (i)
					titles.put (l_string_32, i)
					i := i + 1
				end
			end
			if title_grid /= Void then
				title_grid.set_titles (a_titles)
			end
			if is_title_grid then
				if header_node.grid = Current then
					remove_row (header_node)
				end
				create header_node.make (titles)
				insert_row (header_node, 1)
				header_node.set_height (title_height)
			else
				header_node := title_grid.header_node
			end
			if column_count < titles.count then
				grid.set_column_count_to (titles.count)
				from
					i := column_count + 1
				until
					i > titles.count
				loop
					column (i).set_width (column (i).width)
					i := i + 1
				end
			else
				from
				until
					column_count = titles.count
				loop
					titles.force ("", titles.count + 1)
				end
			end
			update_title_height
		ensure
			a_titles_void_implies_counts_not_changed: a_titles = Void implies titles.count = old column_count and column_count = old column_count
			a_titles_not_void_and_less_than_column_count_implies_counts_not_changed: a_titles /= Void and a_titles.count <= old column_count implies
				titles.count = old column_count and column_count = old column_count
			a_titles_not_void_and_greater_than_column_count_implies_counts_increased: a_titles /= Void and a_titles.count > old column_count implies
				titles.count = a_titles.count and column_count = a_titles.count
			column_count_equals_title_count: column_count = titles.count
			header_node_recreated: header_node /= old header_node
		end

	set_virtual_position (virtual_x, virtual_y: INTEGER) is
			-- Move viewable area of `Current' to virtual position `virtual_x', `virtual_y'.
		require
			not_destroyed: not is_destroyed
			virtual_x_valid: virtual_x >= 0 and virtual_x <= maximum_virtual_x_position
			virtual_y_valid: virtual_y >= 0 and virtual_y <= maximum_virtual_y_position
		do
			grid.set_virtual_position (virtual_x, virtual_y)
		ensure
			virtual_position_set: virtual_x_position = virtual_x and virtual_y_position = virtual_y
		end

	enable_tree (a_enabled: BOOLEAN) is
			--|FIXME rename.
		do
			if a_enabled then
				grid.enable_tree
			else
				grid.disable_tree
			end
		end

	sort is
			-- Sort all rows associated with `Current' based on information in 'sort_states'
		require
			sort_states_exist: sort_states /= Void
		do
			if not sort_states.is_empty and row_count > 0 then
				-- Nothing to do if there are no sort states defined.
				internal_sort (1, grid.row_count, True, Void, sort_states, Void)
				if filtered_nodes /= Void then
					filtered_nodes.wipe_out
				end
				items_to_filter_on.wipe_out
				regex_string := ""
				title_grid.grid.redraw
			end
		end

	sort_rows (a_start_index, a_end_index: INTEGER) is
			-- Sort all top level rows from a_start_index to a_end_index inclusive, based on information in sort_states.
		require
			sort_states_exist: sort_states /= Void
			valid_indexes: a_start_index >= 1 and a_end_index <= row_count and a_start_index <= a_end_index
			start_row_is_top_level: row (a_start_index).parent_row = Void
			end_row_is_top_level: row (a_start_index).parent_row = Void
		do
			if not sort_states.is_empty then
				-- Nothing to do if there are no sort states defined.
				internal_sort (a_start_index, a_end_index, False, Void, sort_states, Void)
			end
		end

	set_separator_color (a_color: EV_COLOR) is
			-- Set `a_color' as `separator_color'.
		require
			not_destroyed: not is_destroyed
			a_color_not_void: a_color /= Void
		do
			grid.set_separator_color (a_color)
			if not is_title_grid then
				title_grid.set_separator_color (a_color)
			end
		ensure
			separator_color_set: separator_color = a_color
		end

	enable_selection_on_single_button_click is
			-- Enable selection handling of items when clicked upon via mouse button `1'.
			-- This is useful when implementing Contextual Menus where the selection may need
			-- to remain unchanged when using mouse button `3' to handle the menu.
		require
			not_destroyed: not is_destroyed
		do
			grid.enable_selection_on_single_button_click
		ensure
			selection_on_single_click_enabled: is_selection_on_single_button_click_enabled and then is_selection_on_click_enabled
		end

	enable_selection_on_click is
			-- Enable selection handling of items when clicked upon via mouse button.
		require
			not_destroyed: not is_destroyed
		do
			grid.enable_selection_on_click
		ensure
			selection_on_click_enabled:is_selection_on_click_enabled
			selection_on_single_click_disabled: not is_selection_on_single_button_click_enabled
		end

	disable_selection_on_click is
			-- Disable selection handling when items are clicked upon.
		require
			not_destroyed: not is_destroyed
		do
			grid.disable_selection_on_click
		ensure
			selection_on_click_disabled: not is_selection_on_click_enabled and then not is_selection_on_single_button_click_enabled
		end

	redraw is
			-- Force `Current' to be re-drawn when next idle.
		require
			not_destroyed: not is_destroyed
		do
			grid.redraw
		end

	set_is_vertical_scroll_bar_show_requested (a_show: BOOLEAN) is
			-- Assign `a_show' to `set_is_vertical_scroll_bar_show_requested'
		do
			if a_show then
				grid.show_vertical_scroll_bar
			else
				grid.hide_vertical_scroll_bar
			end
		ensure
			is_vertical_scroll_bar_show_requested_set: is_vertical_scroll_bar_show_requested = a_show
		end

	set_is_horizontal_scroll_bar_show_requested (a_show: BOOLEAN) is
			-- Assign `a_show' to `set_is_horizontal_scroll_bar_show_requested'
		do
			if a_show then
				grid.show_horizontal_scroll_bar
			else
				grid.hide_horizontal_scroll_bar
			end
		ensure
			is_horizontal_scroll_bar_show_requested_set: is_horizontal_scroll_bar_show_requested = a_show
		end

	set_accept_cursor (a_cursor: like accept_cursor) is
			-- Set `a_cursor' to be displayed when the screen pointer is over a
			-- target that accepts `pebble' during pick and drop.
		do
			check
				not_destroyed: not is_destroyed
			end
			grid.set_accept_cursor (a_cursor)
		end

	set_deny_cursor (a_cursor: like deny_cursor) is
			-- Set `a_cursor' to be displayed when the screen pointer is not
			-- over a valid target.
		do
			check
				not_destroyed: not is_destroyed
			end
			grid.set_deny_cursor (a_cursor)
		end
	set_foreground_color (a_color: like foreground_color) is
			-- Assign `a_color' to `foreground_color'.
		do
			grid.set_foreground_color (a_color)
			Precursor {RP_GRID} (a_color)
		end

	set_background_color (a_color: like background_color) is
			-- Assign `a_color' to `background_color'.
		do
			grid.set_background_color (a_color)
			Precursor {RP_GRID} (a_color)
		end

	set_read_only (a_is_read_only: BOOLEAN) is
			-- Assign `a_is_read_only' to `is_read_only'.
		do
			is_read_only := a_is_read_only
		ensure
			is_read_only_set: is_read_only = a_is_read_only
		end

	set_tooltip (a_tooltip: STRING) is
			-- Assign `a_tooltip' to `tooltip'.
		require
			not_destroyed: not is_destroyed
			tooltip: a_tooltip /= Void
		do
			grid.set_tooltip (a_tooltip)
		ensure
			tooltip_assigned: a_tooltip.is_equal (tooltip)
			cloned: tooltip /= a_tooltip
		end

	remove_tooltip is
			-- Make `tooltip' empty.
		require
			not_destroyed: not is_destroyed
		do
			set_tooltip ("")
		ensure
			tooltip_removed: tooltip.is_empty
		end

	set_pebble (a_pebble: like pebble) is
			-- Assign `a_pebble' to `pebble'.
			-- Overrides `set_pebble_function'.
		do
			grid.set_pebble (a_pebble)
		end

	remove_pebble is
			-- Make `pebble' `Void' and `pebble_function' `Void,
			-- Removing transport.
		do
			grid.remove_pebble
		end

	refresh_all_rows is
			-- Call `refresh' on all rows in `Current'.
		local
			i: INTEGER
		do
			from
				i := 1
			until
				i > row_count
			loop
				row (i).refresh
				i := i + 1
			end
		end

feature -- Element change

	extend (a_node: RP_GRID_NODE [G]) is
			-- Insert `a_node' to `Current' at position `row_count' + 1.
		do
			grid.set_row_count_to (grid.row_count + 1)
			a_node.associate_with_row (grid.row (grid.row_count), Current)
		end

	extend_with_array2 (a_array2: ARRAY2 [ANY]; a_array2_has_titles: BOOLEAN; a_data_array: ARRAY [G]) is
			-- Insert `a_array2.height' rows into `Current' with each new row created with
			-- the contents of the corresponding row in `array2'. If `a_data_array' is not `Void'
			-- then for all items, set the current item `i' as the `data' of the `i_th' newly inserted row.
			-- If a_array2_has_titles is true, the first row of the array2 is used to set the grid titles
		require
			a_array2_not_void: a_array2 /= Void
			has_no_titles_and_data_implies_equal_row_counts: (a_array2_has_titles = false and a_data_array /= Void) implies a_array2.height = a_data_array.count
			has_titles_and_data_implies_data_row_count_smaller_by_one: (a_array2_has_titles = true and a_data_array /= Void) implies a_array2.height = a_data_array.count + 1
		local
			l_array : ARRAY[ANY]
			l_header: RP_GRID_TITLE_NODE[G]
			i, j : INTEGER
			l_data, l_default_data : G
		do
			if a_array2_has_titles then
				create l_array.make(1, a_array2.width)
				from j := 1 until j > a_array2.width loop
					if a_array2.item(1, j) /= Void then
						l_array.put(a_array2.item(1, j), j)
					else
						l_array.put("", j)
					end
					j := j + 1
				end

				-- Insert new title node
				create l_header.make (l_array)
				set_header_node (l_header)
			end

			if a_array2_has_titles then
				i := 2
			else
				i := 1
			end

			from
			until i > a_array2.height loop
				create l_array.make(1, a_array2.width)
				from j := 1 until j > a_array2.width loop
					if a_array2.item(i, j) /= Void then
						l_array.put(a_array2.item(i, j), j)
					else
						l_array.put("", j)
					end
					j := j + 1
				end
				if a_data_array /= Void and then a_data_array.valid_index(i) then
					l_data := a_data_array.item(i)
				else
					l_data := l_default_data
				end
				extend (create {RP_GRID_NODE[G]}.make(l_array, Void, l_data))

				i := i + 1
			end
		ensure
			row_count_increased_correctly:  a_array2_has_titles = false implies row_count = old row_count + a_array2.height
			row_count_increased_correctly: a_array2_has_titles = true implies row_count = old row_count + a_array2.height - 1
		end

	insert (a_node: RP_GRID_NODE [G]; row_index: INTEGER) is
			-- Insert `a_node' immediately before row at index `row_index'.
		require
			a_node_not_void: a_node /= Void
			row_index_within_range: row_index > 0  and row_index <= row_count + 1
			not_inserting_within_existing_subrow_structure: row_index <= row_count implies row (row_index).parent_row = Void
		do
			grid.insert_new_row (row_index)
			a_node.associate_with_row (grid.row (row_index), Current)
		ensure
			node_inserted: a_node.grid = Current and a_node.index = row_index
			row_count_increased: row_count = old row_count + 1
		end

	insert_subrow (a_node, a_parent_node: RP_GRID_NODE [G]; row_index: INTEGER) is
			-- Insert `a_node' immediately before row at index `row_index' with parent `a_parent_node'.
		require
			not_destroyed: not is_destroyed
			tree_enabled: is_tree_enabled
			i_positive: row_index > 0
			i_less_than_row_count: row_index <= row_count + 1
			a_parent_row_not_void: a_parent_node /= Void
			a_parent_row_in_current: a_parent_node.grid = Current
			i_valid_for_parent: row_index > a_parent_node.index and row_index <= a_parent_node.index + a_parent_node.subrow_count_recursive + 1
			not_inserting_within_existing_subrow_structure: row_index < a_parent_node.index + a_parent_node.subrow_count_recursive implies row (row_index).parent_row = a_parent_node
		do
			grid.insert_new_row_parented (row_index, a_parent_node.row)
			a_node.associate_with_row (grid.row (row_index), Current)
		ensure
			node_inserted: a_node.grid = Current and a_node.index = row_index
			row_count_increased: row_count = old row_count + 1
			node_parented: a_node.parent_row = a_parent_node
		end

	remove_all_nodes is
			-- Removes all nodes in `Current'.
		do
			if row_count > 0 then
				remove_rows (1, row_count)
			end
		ensure
			row_count_zero: row_count = 0
		end

	remove_row (a_row: like row) is
			-- Remove `a_row' and all subrows recursively from `Current'.
		require
			row_not_void : a_row /= Void
			row_contained: a_row.grid = Current
		do
			remove_rows (a_row.index, a_row.index + a_row.subrow_count_recursive)
		ensure
			row_removed: a_row.grid = Void
			row_count_decreased: row_count = old row_count - (old a_row.subrow_count_recursive + 1)
		end

	remove_rows (a_start_index, a_end_index: INTEGER) is
			-- Remove rows from `a_start_index' to `a_end_index' inclusive.
		require
			start_index_valid: a_start_index >= 1
			end_index_valid: a_end_index <= row_count
			start_index_less_then_end_index: a_start_index <= a_end_index
			valid_final_row_in_tree_structure: (is_tree_enabled and then highest_parent_row_within_bounds (a_start_index, a_end_index) /= Void implies a_end_index = highest_parent_row_within_bounds (a_start_index, a_end_index).index + highest_parent_row_within_bounds (a_start_index, a_end_index).subrow_count_recursive) or
				(is_tree_enabled and then highest_parent_row_within_bounds (a_start_index, a_end_index) = Void implies row (a_end_index).subrow_count = 0)
		local
			l_nodes: ARRAYED_LIST [RP_GRID_NODE [G]]
			i: INTEGER
			l_node: RP_GRID_NODE [G]
		do
			create l_nodes.make (a_end_index - a_start_index)
			from
				i := a_start_index
			until
				i > a_end_index
			loop
				l_node := row (i)
				l_nodes.extend (l_node)
				pre_remove_event.call ([l_node])
				i := i + 1
			end
			grid.remove_rows (a_start_index, a_end_index)
			from
				l_nodes.start
			until
				l_nodes.off
			loop
				l_node := l_nodes.item
				l_node.dissociate
				l_nodes.forth
			end
		end

	replace_row (original_row, new_row: like row) is
			-- Replace `original_row' with `new_row'.
			-- Child rows of `original_row' are not kept.
		require
			not_destroyed: not is_destroyed
			original_row_not_void: original_row /= Void
			new_row_not_void: new_row /= Void
			original_row_parented: original_row.grid = Current
			new_row_unparented: new_row.grid = Void
		local
			l_parent_node: RP_GRID_NODE [G]
			i: INTEGER
			l_expanded: BOOLEAN
		do
			i := original_row.index
			l_parent_node := original_row.parent_row
			if l_parent_node /= Void then
				l_expanded := l_parent_node.is_expanded
				insert_subrow (new_row, l_parent_node, i)
			else
				insert_row (new_row, i)
			end
			remove_row (original_row)
		ensure
			original_row_removed: original_row.grid = Void
			new_row_inserted: new_row.grid = Current and new_row.index = old original_row.index
		end

	clear is
			-- Clear `Current' by removing all rows.
		local
			i: INTEGER
		do
			from
				i := grid.row_count
			until
				i < 1
			loop
				remove_row (row (i))
				i := i - 1
			end
			grid.clear
		end

	move_row (i, j: INTEGER) is
			-- Move row at index `i' immediately before row at index `j'.
			-- If `j' = `row_count + 1' then row `i' is moved to the last index in the grid.
			-- Row `i' will be unparented if it has a `parent_row'.
		require
			not_destroyed: not is_destroyed
			i_valid: i > 0 and then i <= row_count
			j_valid: j > 0 and then j <= row_count + 1
			row_has_no_subrows: row (i).subrow_count = 0
			not_breaking_existing_subrow_structure:
				j = row_count + 1 or
				(j = i or (j = i + 1 and (i + 1 <= row_count)) and then row (i + 1).parent_row = Void) or
				row (j).parent_row = Void
		do
			grid.move_row (i, j)
		ensure
			rows_moved:
				(j <= i implies row (j) = old row (i)) and
				(j > i implies row (j - 1) = old row (i)) and
				(j < i implies (row (i) = old row ((i - 1).max (1)))) and
				(j > i + 1 implies (row (i) = old row ((i + 1).min (row_count))))
			row_count_unchanged: row_count = old row_count
		end

	move_rows (i, j, n: INTEGER) is
			-- Move `n' rows starting at index `i' immediately before row at index `j'.
			-- If `j' = `row_count + 1' the rows are moved to the very bottom of the grid.
			-- If `is_tree_enabled', all rows moved that share the same tree structure depth
			-- as row `i' are unparented and set as root rows within the grid tree.
			-- All parent rows within the rows moved that have a tree structure depth
			-- greater than that of row `i' are left parented.
		require
			not_destroyed: not is_destroyed
			i_valid: i > 0 and then i <= row_count
			j_valid: j > 0 and then j <= row_count + 1
			n_valid: n > 0 and then i + n <= row_count + 1
			move_not_overlapping: n > 1 implies (j <= i or else j >= i + n)
			rows_may_be_moved: rows_may_be_moved (i, n)
			not_breaking_existing_subrow_structure:
				j = row_count + 1 or
				(j = i or (j = i + n and (i + n <= row_count)) and then row (i + n).parent_row = Void) or
				row (j).parent_row = Void
		do
			grid.move_rows_to_parent (i, j, n, Void)
		ensure
			rows_moved:
				(j <= i implies row (j) = old row (i) and then row (j + n - 1) = old row (i + n - 1)) and
				(j > i + n implies row (j - n) = old row (i) and then row (j - 1) = old row (i + n - 1))
			row_count_unchanged: row_count = old row_count
		end

	populate (a_titles: ARRAY [STRING_GENERAL]; a_nodes: ARRAY [RP_GRID_NODE [G]]) is
			-- Create `Current' with titles `a_titles' and fill with `a_nodes'.
		require
			all_nodes_not_void: a_nodes /= Void implies not a_nodes.has (Void)
		local
			i: INTEGER
		do
			if a_nodes /= Void then
				from
					i := 1
				until
					i > a_nodes.count
				loop
					extend (a_nodes.item (i))
					i := i + 1
				end
			end
			set_titles (a_titles)
		end

	wipe_out is
			-- Remove all contents of `Current' and wipe out it's header.
		local
			i: INTEGER
		do
			from
				i := 1
			until
				i > row_count
			loop
					-- This unlocking code is a temporay work around for a bug in EV_GRID_I `wipe_out'
					-- Remove when fixed. It has been reported as bug #14737
				if row (i).is_locked then
					row (i).unlock
				end
				row (i).dissociate
				i := i + 1
			end
			grid.wipe_out
			if not is_title_grid then
				title_grid.wipe_out
				set_titles (<<>>)
			end
		ensure
			column_count_zero: column_count = 0
			row_count_zero: row_count = 0
		end

feature -- Iteration

	do_all_rows (action: PROCEDURE [ANY, TUPLE [RP_GRID_NODE [G]]]) is
			-- Apply `action' to every row in `Current'.
		require
			action_not_void: action /= Void
		local
			t: TUPLE [RP_GRID_NODE [G]]
			i: INTEGER
		do
			from
				create t
				i := 1
			until
				i > row_count
			loop
				t.put (row (i), 1)
				action.call (t)
				i := i + 1
			end
		end

	do_all_columns (action: PROCEDURE [ANY, TUPLE [RP_DATA_GRID_COLUMN [G]]]) is
			-- Apply `action' to every row in `Current'.
		require
			action_not_void: action /= Void
		local
			t: TUPLE [RP_DATA_GRID_COLUMN [G]]
			i: INTEGER
		do
			from
				create t
				i := 1
			until
				i > column_count
			loop
				t.put (column (i), 1)
				action.call (t)
				i := i + 1
			end
		end

	do_rows_if (action: PROCEDURE [ANY, TUPLE [RP_GRID_NODE [G]]]; a_test: FUNCTION [ANY, TUPLE [RP_GRID_NODE [G]], BOOLEAN]) is
			-- Apply `action' to every row in `Current' that satisfies `a_test'.
		require
			action_not_void: action /= Void
			a_test_not_void: a_test /= Void
		local
			t: TUPLE [RP_GRID_NODE [G]]
			i: INTEGER
		do
			from
				create t
				i := 1
			until
				i > row_count
			loop
				t.put (row (i), 1)
				if a_test.item (t) then
					action.call (t)
				end
				i := i + 1
			end
		end

	do_columns_if (action: PROCEDURE [ANY, TUPLE [RP_DATA_GRID_COLUMN [G]]]; a_test: FUNCTION [ANY, TUPLE [RP_DATA_GRID_COLUMN [G]], BOOLEAN]) is
			-- Apply `action' to every row in `Current' that satisfies `a_test'.
		require
			action_not_void: action /= Void
			a_test_not_void: a_test /= Void
		local
			t: TUPLE [RP_DATA_GRID_COLUMN [G]]
			i: INTEGER
		do
			from
				create t
				i := 1
			until
				i > column_count
			loop
				t.put (column (i), 1)
				if a_test.item (t) then
					action.call (t)
				end
				i := i + 1
			end
		end

	there_exists_a_row (a_test: FUNCTION [ANY, TUPLE [RP_GRID_NODE [G]], BOOLEAN]): BOOLEAN is
			-- Is `a_test' true for at least one row?
		require
			a_test_not_void: a_test /= Void
		local
			t: TUPLE [RP_GRID_NODE [G]]
			i: INTEGER
		do
			from
				create t
				i := 1
			until
				i > row_count or Result
			loop
				t.put (row (i), 1)
				Result := a_test.item (t)
				i := i + 1
			end
		end

	there_exists_a_column (a_test: FUNCTION [ANY, TUPLE [RP_DATA_GRID_COLUMN [G]], BOOLEAN]): BOOLEAN is
			-- Is `a_test' true for at least one column?
		require
			a_test_not_void: a_test /= Void
		local
			t: TUPLE [RP_DATA_GRID_COLUMN [G]]
			i: INTEGER
		do
			from
				create t
				i := 1
			until
				i > column_count or Result
			loop
				t.put (row (i), 1)
				Result := a_test.item (t)
				i := i + 1
			end
		end

	for_all_rows (a_test: FUNCTION [ANY, TUPLE [RP_GRID_NODE [G]], BOOLEAN]): BOOLEAN is
			-- Is `a_test' true for all rows?
		require
			a_test_not_void: a_test /= Void
		local
			t: TUPLE [RP_GRID_NODE [G]]
			i: INTEGER
		do
			from
				create t
				i := 1
				Result := True
			until
				i > row_count or not Result
			loop
				t.put (row (i), 1)
				Result := a_test.item (t)
				i := i + 1
			end
		end

	for_all_columns (a_test: FUNCTION [ANY, TUPLE [RP_DATA_GRID_COLUMN [G]], BOOLEAN]): BOOLEAN is
			-- Is `a_test' true for all columns?
		require
			a_test_not_void: a_test /= Void
		local
			t: TUPLE [RP_DATA_GRID_COLUMN [G]]
			i: INTEGER
		do
			from
				create t
				i := 1
				Result := True
			until
				i > column_count or not Result
			loop
				t.put (row (i), 1)
				Result := a_test.item (t)
				i := i + 1
			end
		end

	do_items_if_with_index (a_is_row_included: FUNCTION [ANY, TUPLE [RP_GRID_NODE [G]], BOOLEAN];
			a_is_column_included: FUNCTION [ANY, TUPLE [RP_DATA_GRID_ITEM], BOOLEAN];
			a_agent: PROCEDURE [ANY, TUPLE [RP_DATA_GRID_ITEM, INTEGER, INTEGER]]) is
			-- Generalized grid output routine: `a_agent' is called on every item
			--   that passes a row (`a_is_row_included') and column (`a_is_column_included') inclusion test, with indexes.
			-- The arguments on the `a_agent' call are the item, the row index within the set of included rows,
			--   and the column index within the set of included columns
		require
			a_agent_not_void: a_agent /= Void
		local
			l_row: CELL [INTEGER]
		do
			create l_row.put (1)
			if a_is_row_included /= Void then
				do_rows_if (agent do_node_items_if_with_index (?, a_is_column_included, a_agent, l_row), a_is_row_included)
			else
				do_all_rows (agent do_node_items_if_with_index (?, a_is_column_included, a_agent, l_row))
			end
		end

feature --Event handling

	pointer_motion_actions: EV_POINTER_MOTION_ACTION_SEQUENCE is
			-- Actions to be performed when screen pointer moves.
		do
			Result := grid.pointer_motion_actions
		end

	pointer_button_press_actions: EV_POINTER_BUTTON_ACTION_SEQUENCE is
			-- Actions to be performed when screen pointer button is pressed.
		do
			Result := grid.pointer_button_press_actions
		end

	pointer_double_press_actions: EV_POINTER_BUTTON_ACTION_SEQUENCE is
			-- Actions to be performed when screen pointer is double clicked.
		do
			Result := grid.pointer_double_press_actions
		end

	pointer_button_release_actions: EV_POINTER_BUTTON_ACTION_SEQUENCE is
			-- Actions to be performed when screen pointer button is released.
		do
			Result := grid.pointer_button_release_actions
		end

	pointer_enter_actions: EV_NOTIFY_ACTION_SEQUENCE is
			-- Actions to be performed when screen pointer enters widget.
		do
			Result := grid.pointer_enter_actions
		end

	pointer_leave_actions: EV_NOTIFY_ACTION_SEQUENCE is
			-- Actions to be performed when screen pointer leaves widget.
		do
			Result := grid.pointer_leave_actions
		end

	pointer_motion_item_actions: ACTION_SEQUENCE [TUPLE [INTEGER, INTEGER, EV_GRID_ITEM]] is
			-- Actions to be performed when a screen pointer moves over a grid.
			-- Arguments (with names for clarity):
			--
			-- x_pos: INTEGER -- The x position of the motion in grid virtual coordinates.
			-- y_pos: INTEGER -- The y position of the motion in grid virtual coordinates.
			-- item: EV_GRID_ITEM -- If the motion occurred above an item, this is the pointed item, otherwise argument is set to `Void'.
		do
			Result := grid.pointer_motion_item_actions
		ensure
			result_not_void: Result /= Void
		end

	pointer_button_press_item_actions: ACTION_SEQUENCE [TUPLE [INTEGER, INTEGER, INTEGER, EV_GRID_ITEM]] is
			-- Actions to be performed when a pointer press event is received by a grid.
			-- Arguments (with names for clarity):
			--
			-- x_pos: INTEGER -- The x position of the press in grid virtual coordinates.
			-- y_pos: INTEGER -- The y position of the press in grid virtual coordinates.
			-- a_button: INTEGER -- The index of the pressed button.
			-- item: EV_GRID_ITEM -- If the press occurred above an item, this is the pointed item, otherwise argument is set to `Void'.
		do
			Result := grid.pointer_button_press_item_actions
		ensure
			result_not_void: Result /= Void
		end

	pointer_double_press_item_actions: ACTION_SEQUENCE [TUPLE [INTEGER, INTEGER, INTEGER, EV_GRID_ITEM]] is
			-- Actions to be performed when a pointer double press event is received by a grid.
			-- Arguments (with names for clarity):
			--
			-- x_pos: INTEGER -- The x position of the double press in grid virtual coordinates.
			-- y_pos: INTEGER -- The y position of the double press in grid virtual coordinates.
			-- a_button: INTEGER -- The index of the pressed button.
			-- item: EV_GRID_ITEM -- If the double press occurred above an item, this is the pointed item, otherwise argument is set to `Void'.
		do
			Result := grid.pointer_double_press_item_actions
		ensure
			result_not_void: Result /= Void
		end

	pointer_button_release_item_actions: ACTION_SEQUENCE [TUPLE [INTEGER, INTEGER, INTEGER, EV_GRID_ITEM]] is
			-- Actions to be performed when a pointer release event is received by a grid.
			-- Arguments (with names for clarity):
			--
			-- x_pos: INTEGER -- The x position of the release in grid virtual coordinates.
			-- y_pos: INTEGER -- The y position of the release in grid virtual coordinates.
			-- a_button: INTEGER -- The index of the released button.
			-- item: EV_GRID_ITEM -- If the release occurred above an item, this is the pointed item, otherwise argument is set to `Void'.
		do
			Result := grid.pointer_button_release_item_actions
		ensure
			result_not_void: Result /= Void
		end

	pointer_enter_item_actions: ACTION_SEQUENCE [TUPLE [BOOLEAN, EV_GRID_ITEM]] is
			-- Actions to be performed when a pointer enter event is received by a grid or grid item
			-- Arguments (with names for clarity):
			--
			-- on_grid: BOOLEAN -- Did the enter event occur for the grid?
			-- item: EV_GRID_ITEM -- If the enter event occurred for an item, this is the item.
			--
			-- Note that `on_grid' may be set to `True' and `item' may be non-Void
			-- in the case where the pointer enters a grid at a location where there
			-- is an item contained.
			-- (from EV_GRID_ACTION_SEQUENCES)
		do
			Result := grid.pointer_enter_item_actions
		end

	pointer_leave_item_actions: ACTION_SEQUENCE [TUPLE [BOOLEAN, EV_GRID_ITEM]] is
			-- Actions to be performed when a pointer leave event is received by a grid or grid item
			-- Arguments (with names for clarity):
			--
			-- on_grid: BOOLEAN -- Did the leave event occur for the grid?
			-- item: EV_GRID_ITEM -- If the leave event occurred for an item, this is the item.
			--
			-- Note that `on_grid' may be set to `True' and `item' may be non-Void
			-- in the case where the pointer leaves a grid from a location where there
			-- was an item contained.
			-- (from EV_GRID_ACTION_SEQUENCES)
		do
			Result := grid.pointer_leave_item_actions
		end

	mouse_wheel_actions: EV_INTEGER_ACTION_SEQUENCE is
			-- Actions to be performed when mouse wheel is rotated.
		do
			Result := grid.mouse_wheel_actions
		end

	key_press_actions: EV_KEY_ACTION_SEQUENCE is
			-- Actions to be performed when a keyboard key is pressed.
		do
			Result := grid.key_press_actions
		end

	key_press_string_actions: EV_KEY_STRING_ACTION_SEQUENCE is
			-- Actions to be performed when a keyboard press generates a displayable character.
		do
			Result := grid.key_press_string_actions
		end

	key_release_actions: EV_KEY_ACTION_SEQUENCE is
			-- Actions to be performed when a keyboard key is released.
		do
			Result := grid.key_release_actions
		end

	focus_in_actions: EV_NOTIFY_ACTION_SEQUENCE is
			-- Actions to be performed when keyboard focus is gained.
		do
			Result := grid.focus_in_actions
		end

	focus_out_actions: EV_NOTIFY_ACTION_SEQUENCE is
			-- Actions to be performed when keyboard focus is lost.
		do
			Result := grid.focus_out_actions
		end

	virtual_position_changed_actions: ACTION_SEQUENCE [TUPLE [INTEGER, INTEGER]] is
			-- Actions to be performed upon next idle after `virtual_x_position' or `virtual_y_position' changed in grid.
			-- Arguments (with names for clarity)
			--
			-- a_virtual_x_position: INTEGER -- New `virtual_x_position' of grid.
			-- a_virtual_y_position: INTEGER -- New `virtual_y_position' of grid.
		do
			Result := grid.virtual_position_changed_actions
		ensure
			result_not_void: Result /= Void
		end

	virtual_size_changed_actions: ACTION_SEQUENCE [TUPLE [INTEGER, INTEGER]] is
			-- Actions to be performed upon next idle after `virtual_width' or `virtual_height' changed in grid.
			-- Arguments (with names for clarity)
			--
			-- a_virtual_width: INTEGER -- New `virtual_width' of grid.
			-- a_virtual_height: INTEGER -- New `virtual_height' of grid.
		do
			Result := grid.virtual_size_changed_actions
		ensure
			result_not_void: Result /= Void
		end

	row_select_actions: EV_GRID_ROW_ACTION_SEQUENCE is
			-- Actions to be performed when a row is selected.
		do
			if row_select_actions_internal = Void then
				create row_select_actions_internal
			end
			Result := row_select_actions_internal
		ensure
			result_not_void: Result /= Void
		end

	row_deselect_actions: EV_GRID_ROW_ACTION_SEQUENCE is
			-- Actions to be performed when a row is deselected.
			-- (from EV_GRID_ACTION_SEQUENCES)
		do
			if row_deselect_actions_internal = Void then
				create row_deselect_actions_internal
			end
			Result := row_deselect_actions_internal
		ensure
			result_not_void: Result /= Void
		end

	pre_draw_overlay_actions: ACTION_SEQUENCE [TUPLE [EV_DRAWABLE, EV_GRID_ITEM, INTEGER, INTEGER]] is
			-- Actions to be performed before the features of an item cell in `Current' have been drawn but after the background of
			-- the cell has been drawn. The four pieces of event data are:
			-- drawable: EV_DRAWABLE The drawable into which you may draw to overlay onto the already drawn background.
			-- grid_item: EV_GRID_ITEM The item which has just been drawn, may be Void in the case that an
			-- item cell is being drawn which does not contain an item.
			-- a_column_index: INTEGER The column index of the grid cell that has just been drawn.
			-- a_row_index: INTEGER The row index of the grid cell that has just been drawn.
			-- This is useful for drawing additional border styles or other such effects. The upper left corner
			-- of the item cell starts at coordinates 0x0 in the passed drawable. All drawing Performed
			-- in the drawable is clipped to `width' of the column at `a_column_index' and `height' of row at `a_row_index'.
			-- Note that the upper left corner of `drawable' corresponds to the upper left corner of the item
			-- cell and not the actual items horizontal position within the cell which may be greater than 0 if
			-- the item is within a tree structure. Use `horizontal_indent' of the item to determine this.
		do
			Result := grid.pre_draw_overlay_actions
		ensure
			not_void: Result /= Void
		end

	post_draw_overlay_actions: ACTION_SEQUENCE [TUPLE [EV_DRAWABLE, EV_GRID_ITEM, INTEGER, INTEGER]] is
			-- Actions to be performed after an item cell in `Current'  has been drawn. The four pieces of event data are:
			-- drawable: EV_DRAWABLE The drawable into which you may draw to overlay onto the already drawn item.
			-- grid_item: EV_GRID_ITEM The item which has just been drawn, may be Void in the case that an
			-- item cell is being drawn which does not contain an item.
			-- a_column_index: INTEGER The column index of the grid cell that has just been drawn.
			-- a_row_index: INTEGER The row index of the grid cell that has just been drawn.

			-- This is useful for drawing additional border styles or other such effects. The upper left corner
			-- of the item cell starts at coordinates 0x0 in the passed drawable. All drawing Performed
			-- in the drawable is clipped to `width' of the column at `a_column_index' and `height' of row at `a_row_index'.
			-- Note that the upper left corner of `drawable' corresponds to the upper left corner of the item
			-- cell and not the actual items horizontal position within the cell which may be greater than 0 if
			-- the item is within a tree structure. Use `horizontal_indent' of the item to determine this.
		do
			Result := grid.post_draw_overlay_actions
		ensure
			not_void: Result /= Void
		end

	remove_event: ROSE_EVENT_TYPE [TUPLE [like row]]

	pre_remove_event: ROSE_EVENT_TYPE [TUPLE [like row]]
			-- Fired before row is removed from grid

	column_resize_actions: ACTION_SEQUENCE [TUPLE [RP_DATA_GRID_COLUMN [G], INTEGER]] is
			-- Actions to be performed when a column in `Current' is resized.
			-- Event data is column resized and new width of that column in pixels.
		do
			if column_resize_actions_internal = Void then
				create column_resize_actions_internal
			end
			Result := column_resize_actions_internal
		end

	column_lock_actions: EV_INTEGER_ACTION_SEQUENCE is
			-- Actions to be performed when a column in `Current' is locked.
		do
			if column_lock_actions_internal = Void then
				create column_lock_actions_internal
			end
			Result := column_lock_actions_internal
		end

	column_unlock_actions: EV_INTEGER_ACTION_SEQUENCE is
			-- Actions to be performed when a column in `Current' is unlocked.
		do
			if column_unlock_actions_internal = Void then
				create column_unlock_actions_internal
			end
			Result := column_unlock_actions_internal
		end

	drop_actions: EV_PND_ACTION_SEQUENCE
		-- Actions to be performed when an object is dropped on `Current'.

	item_drop_actions: ACTION_SEQUENCE [TUPLE [EV_GRID_ITEM, ANY]] is
			-- Actions to be performed when a pebble is dropped on an item.
		do
			Result := grid.item_drop_actions
		ensure
			result_not_void: Result /= Void
		end

	pick_actions: EV_PND_START_ACTION_SEQUENCE is
			-- Actions to be performed when `pebble' is picked up.
		do
			Result := grid.pick_actions
		end

	pick_ended_actions: EV_PND_FINISHED_ACTION_SEQUENCE is
			-- Actions to be performed when a transport from `Current' ends.
			-- If transport completed successfully, then event data
			-- is target. If cancelled, then event data is Void.
		do
			Result := grid.pick_ended_actions
		end

	conforming_pick_actions: EV_NOTIFY_ACTION_SEQUENCE is
			-- Actions to be performed when a pebble that fits here is picked.
		do
			Result := grid.conforming_pick_actions
		end

feature -- Contract Support

	title_grid: RP_DATA_GRID [G]
		-- Grid used internally to support `header_node'.
		-- Returned by querying `header_node.grid'.

	parent_grid: RP_DATA_GRID [G]
		-- Parent grid to which `Current' is associated if
		-- `Current' is a title_grid.
		-- To get `Current' from `header_node', do `header_node.grid.parent_grid'.

	is_title_grid: BOOLEAN is
			-- Is current a title grid within another grid?
			-- `Current' is actually built from two grids, and the `grid' of
			-- `header_node' is a title grid.
		do
			Result := parent_grid /= Void
		ensure
			result_correct: Result = (parent_grid /= Void)
		end

	rows_may_be_moved (a_first_row_index, a_row_count: INTEGER): BOOLEAN is
			-- Do rows from `a_first_row_index' to `a_first_row_index' + `a_row_count' - 1 represent a complete tree structure?
			-- and if row (`a_first_row_index') has a `parent_row', are all rows to be moved nested within that parent
			-- within the tree structure? If `Result' is `True', the rows may be moved without breaking an existing
			-- tree structure.
		require
			row_count_positive: a_row_count >= 1
			first_row_index_valid: a_first_row_index >= 1 and a_first_row_index + a_row_count - 1 <= row_count
		do
			Result := grid.rows_may_be_moved (a_first_row_index, a_row_count)
		end

	highest_parent_row_within_bounds (lower_index, upper_index: INTEGER): EV_GRID_ROW is
			-- Return the highest level `parent_row' recursively of row `upper_index'
			-- that has an index greater or equal to `lower_index'.
		require
			not_destroyed: not is_destroyed
			tree_enabled: is_tree_enabled
			valid_lower_index: lower_index >= 1 and lower_index <= row_count
			valid_upper_index: upper_index >= lower_index and upper_index <= row_count
		do
			Result := grid.highest_parent_row_within_bounds (lower_index, upper_index)
		ensure
			result_not_void: Result /= Void
		end

feature {RP_DATA_GRID_ACCESS} -- Implementation

	previous_virtual_position : PAIR [INTEGER, INTEGER] is
			-- Used by `vertical_scrolling_per_item_relative_to_header' to determine direction of current virtual_position change
		do
			if previous_virtual_position_saved = Void then
				create previous_virtual_position_saved
			end
			Result := previous_virtual_position_saved
		end

	previous_virtual_position_saved : PAIR [INTEGER, INTEGER]

	header: EV_GRID_HEADER is
			-- Grid header
		do
			Result := grid.header
		ensure
			result_not_void: Result /= Void
		end

	set_item (a_column, a_row: INTEGER; a_item: EV_GRID_ITEM) is
			-- Insert `a_item' in cell `a_column', `a_row'.
		do
			grid.set_item (a_column, a_row, a_item)
		end

	is_vscrollbar_click : BOOLEAN
			--has the vscrollbar just been clicked?

	pending_counter : INTEGER

	grid_and_view_container : EV_HORIZONTAL_BOX

	grid_and_view_split : EV_HORIZONTAL_SPLIT_AREA

	fixed : EV_FIXED is
			-- Instance of EV_FIXED into which widgets are placed.
		do
			Result := grid.implementation.fixed
		end

	virtual_vertical_offset_of_fixed: INTEGER is
			-- Top edge of `fixed' in virtual coordinates.
		do
			Result := grid.implementation.internal_client_y - grid.implementation.viewport_y_offset
		end

	virtual_horizontal_offset_of_fixed: INTEGER is
			-- Left edge of `fixed' in virtual coordinates.
		do
			Result := grid.implementation.internal_client_x - grid.implementation.viewport_x_offset
		end

	grid_and_header_container : EV_VERTICAL_BOX
		-- Container to hold the grid and the header grid.

	pointer_button_double_press_received_on_grid (an_x, a_y, button: INTEGER; a_grid_item: EV_GRID_ITEM) is
			-- Respond to double press on grid.
		local
			l_editable_item: EV_GRID_EDITABLE_ITEM
			l_combo_item: EV_GRID_COMBO_ITEM
		do
			if a_grid_item /= Void and not is_showing_as_resizeable then
					-- Do not want to activate an item while
					-- we are in the middle of a column resize.
				l_editable_item ?= a_grid_item
				if l_editable_item /= Void then
					l_editable_item.activate
				end
				l_combo_item ?= a_grid_item
				if l_combo_item /= Void then
					l_combo_item.activate
				end
			end
		end

	add_column_click_action (a_action: PROCEDURE [ANY, TUPLE]; i: INTEGER) is
			-- Add `a_action' to `pointer_button_press_actions' of column i header.
		do
			grid.column (i).header_item.pointer_button_press_actions.extend (agent pointer_button_click_action (a_action, ?, ?, ?, ?, ?, ?, ?, ?))
		end

	pointer_button_click_action (a_action: PROCEDURE [ANY, TUPLE]; i, j, a_button: INTEGER; x, y, z: DOUBLE; m, n: INTEGER) is
			-- Call `a_action' with `a_button'. Other arguments not used.
		do
			a_action.call ([a_button])
		end

	item_added_to_sort_states (an_item: RP_DATA_GRID_SORT_STATE) is
			-- `an_item' has been added to `sort_states' so update icon
			-- of column `an_item.column_index' to reflect state.
		require
			an_item_not_void: an_item /= Void
		do
			update_sort_icon (an_item.column_index)
		end

	item_removed_from_sort_states (an_item: RP_DATA_GRID_SORT_STATE) is
			-- `an_item' has been removed from `sort_states' so update icon
			-- of column `an_item.column_index' to reflect state.
		do
			update_sort_icon (an_item.column_index)
		end

	sort_state_of_column (a_column_index: INTEGER): RP_DATA_GRID_SORT_STATE is
			-- `Result' is sort state within `sort_states' with `column_index'
			-- matching `a_column_index', or Void if None.
		require
			a_column_index_valid: a_column_index >= 1 and a_column_index <= column_count
		local
			cursor: CURSOR
		do
			cursor := sort_states.cursor
			from
				sort_states.start
			until
				sort_states.off or Result /= Void
			loop
				if sort_states.item.column_index = a_column_index then
					Result := sort_states.item
				end
				sort_states.forth
			end
			sort_states.go_to (cursor)
		ensure
			result_not_void_implies_column_index_matches: Result /= Void implies Result.column_index = a_column_index
			sort_states_index_not_changed: sort_states.index = old sort_states.index
		end

	reverse_sorted_columns: ARRAY [BOOLEAN]
		-- This is temporarily used during a sort to permit a quick lookup of
		-- the sort state from a paticular column index. It is only valid during
		-- a sort and is built from `sort_states'.

	update_sort_states_on_click (a_column: INTEGER; a_button: INTEGER) is
			-- Update sort states in response to a click on column `a_column'.
			-- If colunm `a_column' is not sorted, then sort ascending, otherwise
			-- toggle ascending state.
			-- If Ctrl pressed then add to current sort.
			-- If Alt pressed, always sort in reverse.
		require
			valid_column: a_column >= 1 and a_column <= column_count
		local
			sort_state: RP_DATA_GRID_SORT_STATE
		do
			sort_state := sort_state_of_column (a_column)
			if not gui.ev_application.ctrl_pressed then
				sort_states.wipe_out
			end
			if sort_state /= Void then
				if not gui.ev_application.alt_pressed then
					sort_state.set_ascending (not sort_state.ascending)
				else
					sort_state.set_ascending (False)
				end
				if not sort_states.has (sort_state) then
					sort_states.extend (sort_state)
				end
			else
				sort_states.extend (create {RP_DATA_GRID_SORT_STATE}.make_with_column_and_direction (a_column, not gui.ev_application.alt_pressed))
			end
		end

	update_sort_icon (a_column_index: INTEGER) is
			-- Update displayed sort icon for item `a_column_index'
		require
			a_column_index_valid: a_column_index >= 1 and a_column_index <= column_count
		local
			label_item: RP_DATA_GRID_LABEL_ITEM
			l_pixmap: EV_PIXMAP
			sort_state: RP_DATA_GRID_SORT_STATE
		do
			label_item ?= header_node.i_th (a_column_index)
			sort_state := sort_state_of_column (a_column_index)
			if sort_state = Void then
				label_item.set_sort_pixmap (header_node.unsorted_pixmap)
			else
				if sort_state.ascending then
					l_pixmap := header_node.up_pixmap
				else
					l_pixmap := header_node.down_pixmap
				end
				label_item.set_sort_pixmap (l_pixmap)
			end
		end

	item_info_object (a_item: RP_DATA_GRID_ITEM): RP_DATA_GRID_CELL [G] is
			-- Object representing `a_item'.
		require
			a_item_not_void: a_item /= Void
		local
			l_string: STRING
			l_label_item: EV_GRID_LABEL_ITEM
			l_any: ANY
			l_node: RP_GRID_NODE [G]
		do
			l_any := a_item.comparable
			if a_item.is_parented then
				l_node ?= a_item.row.data
			end
			l_label_item ?= a_item
			if l_label_item /= Void then
				l_string := l_label_item.text
			elseif l_any /= Void then
				l_string := l_any.out
			else
				l_string := ""
			end
			create Result.make (l_any, l_node, l_string, a_item)
		ensure
			result_not_void: Result /= Void
		end

	compare_rows (a_1, a_2: ARRAY [RP_DATA_GRID_CELL [G]]): BOOLEAN is
		-- Is row representation `a_1' less than row representation `a_2'?
		require
			a_1_not_void: a_1 /= Void
			a_2_not_void: a_2 /= Void
		local
			l_1, l_2: COMPARABLE
			l_item_1, l_item_2: EV_GRID_ITEM
			l_rp_data_grid_item: RP_DATA_GRID_ITEM
			l_string_1, l_string_2: STRING
			i: INTEGER
			l_differ: INTEGER
			l_regex: LX_DFA_REGULAR_EXPRESSION
		do
			from
				i := 1
			until
				i > a_1.count or else l_differ /= 0
			loop
				l_differ := 0
				l_item_1 := a_1.item (i).item
				l_item_2 := a_2.item (i).item
				if l_item_1 /= Void then
					l_1 ?= l_item_1.data
					l_rp_data_grid_item ?= l_item_1
					if l_rp_data_grid_item /= Void then
						l_1 ?= l_rp_data_grid_item.comparable
					end
				else
					l_1 ?= a_1.item (i).data
				end
				if l_item_2 /= Void then
					l_2 ?= l_item_2.data
					l_rp_data_grid_item ?= l_item_2
					if l_rp_data_grid_item /= Void then
						l_2 ?= l_rp_data_grid_item.comparable
					end
				else
					l_2 ?= a_2.item (i).data
				end
				if regex_string /= Void and then regex_string.count > 0 then
					create l_regex.compile_case_insensitive (regex_string)
				end
			if a_1.item(i).matches_filter /= a_2.item(i).matches_filter then
					if a_1.item(i).matches_filter then
						l_differ := -1
						add_to_filtered_nodes (a_1.item (i).node)
					elseif a_2.item(i).matches_filter then
						l_differ := 1
						add_to_filtered_nodes (a_2.item (i).node)
					end
				elseif l_regex /= Void and then is_regex_match (a_1.item (i), l_regex) /= is_regex_match (a_2.item (i), l_regex) then
					if is_regex_match (a_1.item (i), l_regex) then
						l_differ := -1
					else
						l_differ := 1
					end
				else
						if l_1 /= Void and then l_2 /= Void and then l_1.same_type (l_2) then
							if l_1 < l_2 then
								l_differ := -1
							elseif l_2 < l_1 then
								l_differ := 1
							end
						else
							l_string_1 := a_1.item (i).string
							l_string_2 := a_2.item (i).string
							if l_string_1 < l_string_2 then
								l_differ := -1
							elseif l_string_1 > l_string_2 then
								l_differ := 1
							end
						end
					end
				if reverse_sorted_columns.item (i) then
					l_differ := - l_differ
				end
				i := i + 1
			end
			Result := l_differ = -1
		end

	is_regex_match (a_row: RP_DATA_GRID_CELL [G]; a_regex: LX_DFA_REGULAR_EXPRESSION): BOOLEAN is
			-- Does `a_row.string' match `a_regex'?
			-- Has a side effect of adding `a_row.node' to `filtered_nodes'.
		require
			a_regex_not_void: a_regex /= Void
			a_row_not_void: a_row /= Void
		do
			if a_regex.matches (a_row.string) then
				Result := True
				add_to_filtered_nodes (a_row.node)
			end
		end

	add_to_filtered_nodes (a_node: RP_GRID_NODE [G]) is
			-- Add `a_node' to `filtered_nodes'.
		do
			if filtered_nodes = Void then
				create filtered_nodes.make_empty
				filtered_nodes.compare_references
			end
			if not filtered_nodes.has (a_node) then
				filtered_nodes.extend (a_node)
			end
		ensure
			filtered_nodes_not_void: filtered_nodes /= Void
		end

	filtered_nodes: ROSE_LINEAR_ARRAY [RP_GRID_NODE [G]]

	is_item_to_filter_on (a_item: RP_DATA_GRID_CELL [G]): BOOLEAN is
			-- Is 'a_item' part of 'items_to_filter_on'?
		local
			l_item: RP_DATA_GRID_CELL [G]
		do
			from
				items_to_filter_on.start
			until
				items_to_filter_on.off or Result
			loop
				l_item := items_to_filter_on.current_item
				if l_item /= Void and a_item /= Void then
					Result := l_item.is_equal (a_item)
				end
				items_to_filter_on.forth
			end
		end

	items_to_filter_on: ROSE_LINEAR_ARRAY [RP_DATA_GRID_CELL [G]]

	set_items_to_filter_on (a_items: ARRAY [RP_DATA_GRID_CELL [G]]) is
			-- Asssign `a_items' to `items_to_filter_on'.
		do
			items_to_filter_on := a_items
		ensure
			items_to_filter_on_set: items_to_filter_on = a_items
		end

	regex_string: STRING

	set_regex_string (a_string: STRING) is
			-- Set regex string
		local
			l_subject, l_search: STRING
			i: INTEGER
		do
			l_subject := ""
			l_search := a_string
			if l_search = Void then
				l_search := ""
			end
			if l_search.count > 0 then
				from
					i := 1
				until
					i > l_search.count
				loop
					if regex_characters.has (l_search.item (i)) then
						l_subject.extend ('\')
					end
					l_subject.extend (l_search.item (i))
					i := i + 1
				end
			end
			regex_string := l_search
		end

	sort_type: STRING

	colors: ROSE_STOCK_COLORS is
			-- Once access to stock colors.
		once
			create Result
		ensure
			result_not_void: Result /= Void
		end

	display (a_column, a_row: INTEGER): RP_DATA_GRID_ITEM is
			-- Retrieve item for position `a_column', `a_row' for insertion into the grid.
		local
			l_row: EV_GRID_ROW
			l_node : RP_GRID_NODE [G]
		do
			ensure_sufficient_columns (a_column)
			l_row := grid.row (a_row)
			l_node ?= l_row.data
			if l_node /= Void then
				if a_column >= first_viewable_column and (a_column <= last_viewable_column or last_viewable_column = 0) then
					Result := l_node.i_th (a_column)
					if Result.parent = Void then
							-- Calling `i_th' may or may not have inserted `Result'
							-- in the grid depending on the code path chosen.
							-- We must add it if it is not already added so we can
							-- call `update_item_for_insertion' on the following line.
						grid.set_item (a_column, a_row, Result)
						l_node.update_item_for_insertion (a_column)
					end
				end
			else
				Result := create {RP_DATA_GRID_LABEL_ITEM}.make_with_text ("")
			end
		ensure
			result_not_void: Result /= Void
		end

	insert_subrow_internal (a_row: EV_GRID_ROW; a_node: RP_GRID_NODE [G]; i: INTEGER) is
			-- Add `a_node' as a subrow of `a_row' at index `i'.
		do
			a_row.insert_subrow (i)
			a_node.associate_with_row (a_row.subrow (i), Current)
		end

	insert_row (a_node: RP_GRID_NODE [G]; i: INTEGER) is
				-- Insert row `a_node' at index `i'.
			require
				a_node_not_void: a_node /= Void
			do
				grid.insert_new_row (i)
				a_node.associate_with_row (grid.row (i), Current)
			ensure
				a_node_grid_set: a_node.grid = Current
				a_node_index_set: a_node.index = i
				grid_row_count_increased: grid.row_count = old grid.row_count + 1
		end

	grid_pebble_agent: FUNCTION [ANY, TUPLE [RP_DATA_GRID_ITEM], ANY]

	row_from_ev_grid_item (a_item: EV_GRID_ITEM): RP_GRID_NODE [G] is
			-- Return the grid node associated with `a_item' or `Void' if None.
		do
			if a_item /= Void then
				Result ?= a_item.row.data
			end
		end

	item_from_ev_grid_item (a_item: EV_GRID_ITEM): RP_DATA_GRID_ITEM is
			-- Return the data grid item associated with `a_item'.
		do
			if {l_data_grid_item: RP_DATA_GRID_ITEM} a_item then
				Result := l_data_grid_item
			end
		end

	pebble_agent (a_item: EV_GRID_ITEM): ANY is
			-- Agent used internally to retrieve a pebble
			-- from the item, rows or grid pebble agents.
		local
			l_item: RP_DATA_GRID_ITEM
			l_row: RP_GRID_NODE [G]
		do
			l_item := item_from_ev_grid_item (a_item)
			l_row := row_from_ev_grid_item (a_item)
			if l_item /= Void then
				if l_item.pebble_agent /= Void then
					Result := l_item.pebble_agent.item ([l_item])
				elseif l_row.pebble /= Void then
					Result := l_row.pebble
				elseif l_row.pebble_agent /= Void then
					Result := l_row.pebble_agent.item ([l_row])
				elseif grid_pebble_agent /= Void then
					Result := grid_pebble_agent.item ([l_item])
				end
			elseif grid_pebble_agent /= Void then
				Result := grid_pebble_agent.item ([l_item])
			end
		end

	regex_characters: ROSE_LINEAR_ARRAY [CHARACTER] is
			-- Characters to be escaped in regular expressions.
		once
			create Result.make_from_array (<< '\', '[', ']', '/' >>)
		ensure
			result_not_void: Result /= Void
		end

	dynamic_content_function : FUNCTION [ANY, TUPLE [INTEGER, INTEGER], EV_GRID_ITEM] is
			-- Dynamic content function used internally.
		do
			Result := grid.dynamic_content_function
		end

	set_dynamic_content_function (a_function: FUNCTION [ANY, TUPLE [INTEGER, INTEGER], EV_GRID_ITEM]) is
			-- Assign `a_function' as the dynamic content function of `Current'.
		do
			grid.set_dynamic_content_function (a_function)
		end

feature {RP_GRID_TITLE_NODE, RP_GRID_NODE} -- Title node sorting support

	locked_rows: ARRAYED_LIST [RP_GRID_NODE [G]]
			-- All locked sticky rows ordered from top to bottom.

	height_of_locked_rows : INTEGER is
			-- Sum of heights of rows locked in header
		local
			l_locked_rows: ARRAYED_LIST [RP_GRID_NODE [G]]
			l_cursor: CURSOR
		do
			l_locked_rows := locked_rows
			l_cursor := l_locked_rows.cursor
			from l_locked_rows.start until l_locked_rows.off loop
				if grid.is_row_height_fixed then
					Result := Result + grid.row_height
				else
					Result := Result + l_locked_rows.item.height
				end
				l_locked_rows.forth
			end
			l_locked_rows.go_to (l_cursor)
		end

	internal_sort (a_start_row, a_end_row: INTEGER; a_title_sort: BOOLEAN; a_parent_row: EV_GRID_ROW;
			a_sort_states: ACTIVE_LIST [RP_DATA_GRID_SORT_STATE]; a_reg_ex: LX_DFA_REGULAR_EXPRESSION) is
			-- Sort all subrows associated with `Current', from a_start_row through a_end_row, inclusive.
			--  If a_title_sort is true, then the rows to be sorted are the ones who's parent is a_parent row.
			--  Sorting on a_sort_states, with a_reg_ex applied if not Void.
		require
			valid_indexes: a_start_row <= a_end_row and a_start_row >= 1 and a_end_row <= grid.row_count
			sort_states_exist: a_sort_states /= Void
		local
			l_rows : ROSE_LINEAR_ARRAY[ARRAY[RP_DATA_GRID_CELL[G]]]
			i,j,l_column,l_last_index : INTEGER
			l_row : EV_GRID_ROW
			l_node : RP_GRID_NODE[G]
			l_item : EV_GRID_LABEL_ITEM
			l_toggle_item: RP_DATA_GRID_TOGGLE_ITEM
			l_string : STRING
			l_any : ANY
			l_columns : ROSE_LINEAR_ARRAY[RP_DATA_GRID_CELL[G]]
			l_grid_cell : RP_DATA_GRID_CELL[G]
			l_past_siblings : BOOLEAN
			l_secondary_title_node_count : INTEGER
			l_title_node : RP_GRID_TITLE_NODE [G]
			l_original_virtual_y_position: INTEGER
		do
			Global.start_busy (Void)
			l_original_virtual_y_position := virtual_y_position
			grid.lock_update
			create l_rows.make_empty
			create reverse_sorted_columns.make (1, 1)
			from
				a_sort_states.start
			until
				a_sort_states.off
			loop
				reverse_sorted_columns.force (not a_sort_states.item.ascending, a_sort_states.index)
				a_sort_states.forth
			end
			if a_parent_row /= Void then
				l_title_node ?= row (a_parent_row.index + 1)
				if l_title_node /= void then
					l_secondary_title_node_count := l_title_node.secondary_title_nodes.count
				end
			end
			from i := a_start_row until i > a_end_row or else l_past_siblings loop
				l_node := row (i)
				l_row := l_node.row
				if a_title_sort then
					if a_parent_row /= void then
						l_past_siblings := l_row.parent_row = a_parent_row.parent_row
					else
						l_past_siblings := false
					end
				end
				if not l_past_siblings then
					if not a_title_sort or l_row.parent_row = a_parent_row then
						-- process every row if this is not a_title_sort, otherwise only rows who's parent is a_parent_row
						-- when a_parent_row is Void, this selects the top level rows
						create l_columns.make_empty
						from j:= 1 until j > a_sort_states.count loop
							l_column := a_sort_states.i_th (j).column_index

							l_node ?= l_row.data
							l_item ?= l_node.i_th (l_column)
							l_any := l_node.i_th (l_column).comparable
							if l_item = void then
								l_item ?= l_any
							else
								l_toggle_item ?= l_item
							end
							if l_toggle_item /= Void then
								l_string := l_toggle_item.is_on.out
							elseif l_item /= void then
								l_string := l_item.text
							elseif l_any /= void then
								l_string := l_any.out
							else
								l_string := ""
							end
							create l_grid_cell.make(l_any,l_node,l_string,l_item)
							if a_reg_ex /= void and then a_reg_ex.matches (l_grid_cell.string) then
								l_grid_cell.set_matches_filter(true)
							else
								l_grid_cell.set_matches_filter(is_item_to_filter_on (l_grid_cell))
							end
							l_columns.extend(l_grid_cell)
							j := j + 1
						end
						l_rows.extend(l_columns)
						l_node.set_is_show_requested_saved (l_row.is_show_requested)
						l_last_index := i
					end
				end
				i := i + 1
			end
			if l_last_index > 0 then
				if sort_comparator = Void then
					l_rows.sort_by(agent compare_rows)
				else
					l_rows.sort_by (agent compare_nodes_wrapper)
				end
				grid.implementation.move_rows_for_sort (a_start_row, l_rows)
			end
				-- Now update icons for all sort states as the state may have changed
				-- without the state being removed or added to the list.
			from
				a_sort_states.start
			until
				a_sort_states.off
			loop
				update_sort_icon (a_sort_states.item.column_index)
				a_sort_states.forth
			end
			set_virtual_position (virtual_x_position, l_original_virtual_y_position.min (maximum_virtual_y_position))
			Global.end_busy
			grid.unlock_update
		end

	compare_nodes_wrapper (array1, array2: ARRAY[RP_DATA_GRID_CELL[G]]): BOOLEAN is
			-- A wrapped used by `internal_sort' to map between the node result seen by users
			-- and then interal version.
		require
			array1_not_void: array1 /= Void
			array2_not_void: array2 /= Void
		do
			sort_comparator.call ([compare_rows (array1, array2), array1.item (1).node, array2.item (1).node])
			Result := sort_comparator.last_result
		end


feature {RP_GRID_TITLE_NODE} -- Implementation

	edge_size: INTEGER is 4
		-- Number of pixels from the edge of a column boundry where it may still be dragged by a user.

	is_showing_as_resizeable: BOOLEAN
		-- Is `Current' showing as resizeable?

	select_items_in_column (l_column, start_row_index, end_row_index: INTEGER) is
			-- Select all items in column `l_column' from `start_row_index' to `end_row_index' inclusive.
		require
			valid_column: l_column >= 1 and l_column <= column_count
			valid_rows: start_row_index >= 1 and end_row_index >= start_row_index and end_row_index <= row_count
		local
			i, l_direction : INTEGER
		do
			if gui.ev_application.ctrl_pressed then
				if all_items_in_range_selected_in_column (l_column, start_row_index, end_row_index) then
					update_selection_of_items_in_column (l_column, start_row_index, end_row_index, False)
				else
					update_selection_of_items_in_column (l_column, start_row_index, end_row_index, True)
				end
			elseif gui.ev_application.shift_pressed then
				if last_selected_column /= Void then
					l_direction := (last_selected_column.index - l_column).sign
					from i := 0 until i > (l_column - last_selected_column.index).abs loop
						update_selection_of_items_in_column (l_column + l_direction * i, start_row_index, end_row_index, True)
						i := i + 1
					end
				end
			else
				from i := 1 until i > column_count loop
					update_selection_of_items_in_column (l_column, start_row_index, end_row_index, False)
					i := i + 1
				end
				update_selection_of_items_in_column (l_column, start_row_index, end_row_index, True)
			end
			if all_items_in_range_selected_in_column (l_column, start_row_index, end_row_index) then
				last_selected_column := column (l_column)
			else
				last_selected_column := Void
			end
		end

	all_items_in_range_selected_in_column (a_column, start_row_index, end_row_index: INTEGER): BOOLEAN is
			-- Are all items in column `l_column' from `start_row_index' to `end_row_index' inclusive, selected?
		require
			valid_column: a_column >= 1 and a_column <= column_count
			valid_rows: start_row_index >= 1 and end_row_index >= start_row_index and end_row_index <= row_count
		local
			i: INTEGER
		do
			if start_row_index = 1 and end_row_index = row_count then
				Result := column (a_column).is_selected
			else
				from
					i := start_row_index
					Result := True
				until
					i > end_row_index or not Result
				loop
					Result := Result and column (a_column).i_th (i).is_selected
					i := i + 1
				end
			end
		end


	update_selection_of_items_in_column (a_column, start_row_index, end_row_index: INTEGER; enable: BOOLEAN) is
			-- Set selected state of  all items in column `l_column' from `start_row_index' to `end_row_index' inclusive,
			-- to `enable'.
		require
			valid_column: a_column >= 1 and a_column <= column_count
			valid_rows: start_row_index >= 1 and end_row_index >= start_row_index and end_row_index <= row_count
		local
			i: INTEGER
		do
			if start_row_index = 1 and end_row_index = row_count then
				if enable then
					column (a_column).enable_select
				else
					column (a_column).disable_select
				end
			else
				from
					i := start_row_index
				until
					i > end_row_index
				loop
					if enable then
						column (a_column).i_th (i).enable_select
					else
						column (a_column).i_th (i).disable_select
					end
					i := i + 1
				end
			end
		end

feature {RP_GRID_NODE} -- Implementation

	locked_column_i_for_column (a_column: EV_GRID_COLUMN): EV_GRID_LOCKED_I is
			-- `Result' is internal locked column for column `a_column'.
		require
			a_column_not_void: a_column /= Void
			a_column_locked: a_column.is_locked
		do
			Result := grid.implementation.locked_column_i_for_column (a_column)
		ensure
			result_not_void: Result /= Void
		end

	locked_row_i_for_row (a_row: EV_GRID_ROW): EV_GRID_LOCKED_I is
			-- `Result' is internal locked row for row `a_row'.
		require
			a_row_not_void: a_row /= Void
			a_row_locked: a_row.is_locked
		do
			Result := grid.implementation.locked_row_i_for_row (a_row)
		ensure
			result_not_void: Result /= Void
		end

feature -- Syncronization

	synchronize_vertically (a_other: RP_DATA_GRID [G]) is
			-- Have `a_other' scroll vertically in sync with `Current'.
		require
			a_other_not_void: a_other /= Void
			-- Hm. vertically_synchronized_grids not void - this appears to be a missing (but satisfied) invariant.
			-- Ditto for horizontal.
		do
			vertically_synchronized_grids.extend (a_other)
		ensure
			vertically_synchronized: vertically_synchronized_grids.has (a_other)
		end

	synchronize_horizontally (a_other: RP_DATA_GRID [G]) is
			-- Have `a_other' scroll horizontally in sync with `Current'.
		require
			a_other_not_void: a_other /= Void
		do
			horizontally_synchronized_grids.extend (a_other)
		ensure
			horizontally_synchronized: horizontally_synchronized_grids.has (a_other)
		end

	synchronize_scrolling (a_other: RP_DATA_GRID [G]) is
			-- Have `a_other' scroll horizontally and vertically in sync with `Current'.
		require
			a_other_not_void: a_other /= Void
		do
			vertically_synchronized_grids.extend (a_other)
			horizontally_synchronized_grids.extend (a_other)
		ensure
			vertically_synchronized: vertically_synchronized_grids.has (a_other)
			horizontally_synchronized: horizontally_synchronized_grids.has (a_other)
		end

feature {RP_PIVOT_TABLE, RP_PIVOT_TABLE_VIEW, RP_DATA_GRID_COLUMN, RP_DATA_GRID, RP_GRID_NODE} -- Implementation

	vertically_synchronized_grids: ACTIVE_LIST [RP_DATA_GRID [G]]
			-- Grids that automatically scroll verticaly with `Current'

	horizontally_synchronized_grids: ACTIVE_LIST [RP_DATA_GRID [G]]
			-- Grids that automatically scroll horizontaly with `Current';
			-- Question for Julian: Why does it have to be the same generic type? Can't it be ANY?

	horizontally_synchronized_grid: RP_DATA_GRID [G]

	vertically_synchronized_grid: RP_DATA_GRID [G]

	set_horizontally_synchronized_grid (a_synchronized_grid: RP_DATA_GRID [G]) is
			-- Assign `a_synchronized_grid' to `horizontally_synchronized_grid'.
		do
			horizontally_synchronized_grid := a_synchronized_grid
		ensure
			horizontally_synchronized_grid_set: horizontally_synchronized_grid = a_synchronized_grid
		end

	set_vertically_synchronized_grid (a_synchronized_grid: RP_DATA_GRID [G]) is
			-- Assign `a_synchronized_grid' to `vertically_synchronized_grid'.
		do
			vertically_synchronized_grid := a_synchronized_grid
		ensure
			vertically_synchronized_grid_set: vertically_synchronized_grid = a_synchronized_grid
		end

	grid_added_to_vertically_synchronized_grids (data_grid: RP_DATA_GRID [G]) is
			-- Respond to the addition of `data_grid' to `vertically_synchronized_grids'.
		require
			data_grid_not_void: data_grid /= Void
		do
			data_grid.grid.hide_horizontal_scroll_bar
			data_grid.grid.hide_vertical_scroll_bar
			data_grid.enable_vertical_overscroll
			data_grid.disable_row_height_fixed
			data_grid.set_vertically_synchronized_grid (Current)
			enable_vertical_overscroll
			disable_row_height_fixed
		end

	grid_added_to_horizontally_synchronized_grids (data_grid: RP_DATA_GRID [G]) is
			-- Respond to the addition of `data_grid' to `horizontally_synchronized_grids'.
		require
			data_grid_not_void: data_grid /= Void
		do
			data_grid.grid.hide_horizontal_scroll_bar
			data_grid.grid.hide_vertical_scroll_bar
			data_grid.enable_vertical_overscroll
			data_grid.disable_row_height_fixed
			data_grid.set_horizontally_synchronized_grid (Current)
			data_grid.grid.implementation.disable_horizontal_offset_set_to_zero_when_items_smaller_than_viewable_width
			enable_vertical_overscroll
			disable_row_height_fixed
		end

	refresh_position (a_x, a_y: INTEGER) is
			-- Refresh position of all synchronized grids to match virtual position of `Current' as `a_x', `a_y'.
		do
			from
				horizontally_synchronized_grids.start
			until
				horizontally_synchronized_grids.off
			loop
				horizontally_synchronized_grids.item.set_virtual_position (virtual_x_position.min (horizontally_synchronized_grids.item.maximum_virtual_x_position), horizontally_synchronized_grids.item.virtual_y_position)
				horizontally_synchronized_grids.forth
			end
			from
				vertically_synchronized_grids.start
			until
				vertically_synchronized_grids.off
			loop
				vertically_synchronized_grids.item.set_virtual_position (vertically_synchronized_grids.item.virtual_x_position, virtual_y_position.min (vertically_synchronized_grids.item.maximum_virtual_y_position))
				vertically_synchronized_grids.forth
			end
		end

	synchronize_columns_from_column (a_column: RP_DATA_GRID_COLUMN [G]; a_span: TUPLE [INTEGER, INTEGER]; a_grid: RP_DATA_GRID [G]) is
			-- Synchronize columns contained in `a_grid', associated with `a_column' as defined by `a_span'.
		require
			a_column_not_void: a_column /= Void
			a_span_not_one: a_span.integer_item (1) /= a_span.integer_item (2)
			a_grid_not_void: a_grid /= Void
		local
			l_span: INTEGER
			l_width: INTEGER
			i: INTEGER
			l_extra: INTEGER
		do
			l_span := a_span.integer_item (2) - a_span.integer_item (1) + 1
			l_width := a_column.width // l_span
			l_extra := a_column.width \\ l_span
			from
				i := a_span.integer_item (1)
			until
				i > a_span.integer_item (2)
			loop
				if i < a_span.integer_item (1) + l_extra then
					a_grid.column (i).set_width (l_width + 1)
				else
					a_grid.column (i).set_width (l_width)
				end
				i := i + 1
			end
		end

	synchronize_column_from_columns (a_column: RP_DATA_GRID_COLUMN [G]; a_span: TUPLE [INTEGER, INTEGER]; a_grid, a_grid2: RP_DATA_GRID [G]) is
			-- Synchronize column contained in `a_grid', associated with `a_column' and associated columns in `a_grid2'. `a_span' is
			-- used to determine the span in `a_grid2' which determines how many columns are associated with `a_column'.
		require
			a_column_not_void: a_column /= Void
			a_span_is_one: a_span.integer_item (1) = a_span.integer_item (2)
			a_grid_not_void: a_grid /= Void
			a_grid2_not_void: a_grid2 /= Void
		local
			l_range2: TUPLE [INTEGER, INTEGER]
			i: INTEGER
			l_total_width: INTEGER
		do
			if not a_grid2.column_mappings.is_empty then
				l_range2 := a_grid2.column_mappings.item (a_span.integer_item (1))
				from
					i := l_range2.integer_item (1)
				until
					i > l_range2.integer_item (2)
				loop
					l_total_width := l_total_width + a_grid2.column (i).width
					i := i + 1
				end
				a_grid.column (a_span.integer_item (1)).set_width (l_total_width)
			end
		end

	synchronize_columns (a_column: RP_DATA_GRID_COLUMN [G]) is
			-- Synchronize all columns mapped to `a_column'.
		require
			a_column_not_void: a_column /= Void
		local
			l_range: TUPLE [INTEGER, INTEGER]
		do
			is_synchronizing_columns.put (True)
			if horizontally_synchronized_grid /= Void then
				if horizontally_synchronized_grid.horizontally_synchronized_grids.has (Current) then
					if horizontally_synchronized_grid.column_mappings.valid_index (a_column.index) then
						l_range := horizontally_synchronized_grid.column_mappings.item (a_column.index)
						if l_range.integer_item (1) /= l_range.integer_item (2) then
							synchronize_columns_from_column (a_column, l_range, horizontally_synchronized_grid)
						else
							synchronize_column_from_columns (a_column, l_range, horizontally_synchronized_grid, Current)
						end
					end
				end
			elseif horizontally_synchronized_grids /= Void then
				from
					horizontally_synchronized_grids.start
				until
					horizontally_synchronized_grids.off
				loop
					if horizontally_synchronized_grids.item.column_mappings.valid_index (a_column.index) then
						l_range := horizontally_synchronized_grids.item.column_mappings.item (a_column.index)
						if l_range.integer_item (1) = l_range.integer_item (2) then
							synchronize_column_from_columns (a_column, l_range, horizontally_synchronized_grids.item, Current)
						else
							synchronize_columns_from_column (a_column, l_range, horizontally_synchronized_grids.item)
						end
					end
					horizontally_synchronized_grids.forth
				end
			end
			is_synchronizing_columns.put (False)
		end

	synchronize_rows_from_row (a_row: RP_GRID_NODE [G]; a_span: TUPLE [INTEGER, INTEGER]; a_grid: RP_DATA_GRID [G]) is
			-- Synchronize rows contained in `a_grid', associated with `a_row' as defined by `a_span'.
		require
			a_row_not_void: a_row /= Void
			a_span_not_one: a_span.integer_item (1) /= a_span.integer_item (2)
			a_grid_not_void: a_grid /= Void
		local
			l_span: INTEGER
			l_height: INTEGER
			i: INTEGER
			l_extra: INTEGER
		do
			l_span := a_span.integer_item (2) - a_span.integer_item (1) + 1
			l_height := a_row.height // l_span
			l_extra := a_row.height \\ l_span
			from
				i := a_span.integer_item (1)
			until
				i > a_span.integer_item (2)
			loop
				if i < a_span.integer_item (1) + l_extra then
					a_grid.row (i).set_height (l_height + 1)
				else
					a_grid.row (i).set_height (l_height)
				end
				i := i + 1
			end
		end

	synchronize_row_from_rows (a_row: RP_GRID_NODE [G]; a_span: TUPLE [INTEGER, INTEGER]; a_grid, a_grid2: RP_DATA_GRID [G]) is
			-- Synchronize row contained in `a_grid', associated with `a_row' and associated rows in `a_grid2'. `a_span' is
			-- used to determine the span in `a_grid2' which determines how many rows are associated with `a_row'.
		require
			a_row_not_void: a_row /= Void
			a_span_is_one: a_span.integer_item (1) = a_span.integer_item (2)
			a_grid_not_void: a_grid /= Void
			a_grid2_not_void: a_grid2 /= Void
		local
			l_range2: TUPLE [INTEGER, INTEGER]
			i: INTEGER
			l_total_height: INTEGER
		do
			if not a_grid2.row_mappings.is_empty then
				l_range2 := a_grid2.row_mappings.item (a_span.integer_item (1))
				from
					i := l_range2.integer_item (1)
				until
					i > l_range2.integer_item (2)
				loop
					l_total_height := l_total_height + a_grid2.row (i).height
					i := i + 1
				end
				a_grid.row (a_span.integer_item (1)).set_height (l_total_height)
			end
		end

	synchronize_rows (a_row: RP_GRID_NODE [G]) is
			-- Synchronize all rows mapped to `a_row'.
		require
			a_row_not_void: a_row /= Void
		local
			l_range: TUPLE [INTEGER, INTEGER]
		do
			is_synchronizing_rows.put (True)
			if vertically_synchronized_grid /= Void then
				if vertically_synchronized_grid.vertically_synchronized_grids.has (Current) then
					if vertically_synchronized_grid.row_mappings.valid_index (a_row.index) then
						l_range := vertically_synchronized_grid.row_mappings.item (a_row.index)
						if l_range.integer_item (1) /= l_range.integer_item (2) then
							synchronize_rows_from_row (a_row, l_range, vertically_synchronized_grid)
						else
							synchronize_row_from_rows (a_row, l_range, vertically_synchronized_grid, Current)
						end
					end
				end
			elseif vertically_synchronized_grids /= Void then
				from
					vertically_synchronized_grids.start
				until
					vertically_synchronized_grids.off
				loop
					if vertically_synchronized_grids.item.row_mappings.valid_index (a_row.index) then
						l_range := vertically_synchronized_grids.item.row_mappings.item (a_row.index)
						if l_range.integer_item (1) = l_range.integer_item (2) then
							synchronize_row_from_rows (a_row, l_range, vertically_synchronized_grids.item, Current)
						else
							synchronize_rows_from_row (a_row, l_range, vertically_synchronized_grids.item)
						end
					end
					vertically_synchronized_grids.forth
				end
			end
			is_synchronizing_rows.put (False)
		end

	is_synchronizing_columns: CELL [BOOLEAN] is
			-- Once used to hold whether column synchronization is taking place.
		once
			create Result.put (False)
		end

	is_synchronizing_rows: CELL [BOOLEAN] is
			-- Once used to hold whether row synchronization is taking place.
		once
			create Result.put (False)
		end

	refresh_mappings is
			-- Reset column widths and row heights to what they already are.
			-- Hm. Now why would we want to do that? Make sure Julian is an observer
			-- to the code review.
		local
			i: INTEGER
		do
			from
				i := 1
			until
				i > column_count
			loop
				column (i).set_width (column (i).width)
				i := i + 1
			end
			from
				i := 1
			until
				i > row_count
			loop
				row (i).set_height (row (i).height)
				i := i + 1
			end
		end

	column_mappings: ARRAY [TUPLE [INTEGER, INTEGER]]

	row_mappings: ARRAY [TUPLE [INTEGER, INTEGER]]

feature {RP_DATA_GRID} -- Implementation for resizing

	item_activated (a_item: EV_GRID_ITEM; a_popup_window: EV_POPUP_WINDOW) is
			-- `a_item' has become activated with popup window `a_popup_window'.
		require
			a_item_not_void: a_item /= Void
			a_popup_window_not_void: a_popup_window /= Void
		local
			l_editable_item: RP_DATA_GRID_EDITABLE_ITEM
		do
			l_editable_item ?= a_item
			if l_editable_item /= Void then
				l_editable_item.respond_to_activation (a_popup_window)
			end
		end

	update_column_size_on_press (an_x, a_y, a_button: INTEGER; an_item: EV_GRID_ITEM; a_grid: RP_DATA_GRID [G]) is
			-- A pointer button press event has been received on `Current', so begin
			-- row or column resizing based on the mouse position.
		local
			item_x: INTEGER
			l_column_index: INTEGER
			l_column: RP_DATA_GRID_COLUMN [G]
		do
			if is_title_grid or else are_columns_resizable_on_dividers then
				if a_button = 1 then
					if an_item /= Void and then an_item.is_parented then
					item_x := an_item.virtual_x_position
					if (an_x >= item_x and (an_x - item_x < edge_size)) then
						l_column_index :=  first_visible_column_index_to_left (an_item.column.index)
						if l_column_index > 0 then
							if is_title_grid then
								l_column := parent_grid.column (l_column_index)
							else
								l_column := column (l_column_index)
							end
							if l_column.is_user_resizeable then
								resizing_column_internal := l_column
								relative_column_pos := an_x - item_x + resizing_column_internal.width
								grid.enable_capture_light
							end
						end
					elseif (an_x >= item_x + an_item.column.width - an_item.horizontal_indent - edge_size) then
						l_column_index := an_item.column.index
						if is_title_grid then
							l_column := parent_grid.column (l_column_index)
						else
							l_column := column (l_column_index)
						end
						if l_column.is_user_resizeable then
							resizing_column_internal := l_column
							relative_column_pos :=  an_x - item_x
							grid.enable_capture_light
						end
					end
					else
						if column_count > 0 then
							if is_title_grid then
								l_column := parent_grid.column (parent_grid.column_count)
							else
								l_column := column (column_count)
							end
							if an_x >= l_column.virtual_x_position + l_column.width and an_x < l_column.virtual_x_position + l_column.width + edge_size then
								if l_column.is_user_resizeable then
									resizing_column_internal := l_column
									relative_column_pos :=  an_x - l_column.virtual_x_position + l_column.width
									grid.enable_capture_light
								end
							end
						end
					end
				end
			end
		end

	update_column_width_on_double_click (an_x, a_y, a_button: INTEGER; an_item: EV_GRID_ITEM) is
			-- A pointer button double press event has been received on `Current', so resize
			-- clicked column to visible content if currently resizing.
		local
			l_column: RP_DATA_GRID_COLUMN [G]
			l_column_index: INTEGER
			l_required_width: INTEGER
			item_x: INTEGER
			other_grid: RP_DATA_GRID [G]
			l_first_row, l_last_row: RP_GRID_NODE [G]
			l_data_grid_column: RP_DATA_GRID_COLUMN [G]
		do
			if is_title_grid or else are_columns_resizable_on_dividers then
				if a_button = 1 then
					if an_item /= Void then
					item_x := an_item.virtual_x_position
					if (an_x >= item_x and (an_x - item_x < edge_size)) and first_visible_column_index_to_left (an_item.column.index)> 0 then
						l_column_index := grid.column (first_visible_column_index_to_left (an_item.column.index)).index
					elseif (an_x >= item_x + an_item.column.width - an_item.horizontal_indent - edge_size) then
						l_column_index := an_item.column.index
					end
					elseif column_count > 0 then
						if is_title_grid then
							l_data_grid_column := parent_grid.column (parent_grid.column_count)
						else
							l_data_grid_column := column (column_count)
						end
						if an_x >= l_data_grid_column.virtual_x_position + l_data_grid_column.width and an_x < l_data_grid_column.virtual_x_position + l_data_grid_column.width + edge_size then
							l_column_index := l_data_grid_column.index
						end
					end
					if l_column_index /= 0 then
						if is_title_grid then
							l_data_grid_column := parent_grid.column (l_column_index)
						else
							l_data_grid_column := column (l_column_index)
						end
						if l_data_grid_column.is_user_resizeable then
							l_column := column (l_column_index)
							if is_title_grid then
								other_grid := parent_grid
							else
								other_grid := title_grid
							end
							l_first_row := first_visible_row
							l_last_row := last_visible_row
							if l_first_row /= Void and then l_last_row /= Void then
								l_required_width := l_column.required_width_of_item_span (l_first_row.index, l_last_row.index, False)
								l_required_width := l_required_width.max (minimum_width_of_widgets_in_column (l_column_index))
							end
							if other_grid.is_displayed then
									-- Nothing do do if the `title_grid' was hidden.
							l_first_row := other_grid.first_visible_row
							l_last_row := other_grid.last_visible_row
							if l_first_row /= Void and then l_last_row /= Void then
								l_required_width := l_required_width.max (other_grid.column (l_column_index).required_width_of_item_span (l_first_row.index, l_last_row.index, False))
								l_required_width := l_required_width.max (other_grid.minimum_width_of_widgets_in_column (l_column_index))
							end
							end
							l_column.set_width (l_required_width)
							other_grid.column (l_column_index).set_width (l_required_width)
						end
					end
				end
			end
		end

	first_visible_column_index_to_left (a_column_index: INTEGER): INTEGER is
			-- Return the index of the first non-hidden column to the left of column with index `a_column_index'.
			-- Result is 0 if there is no visible column to the left of column `a_column_index'.
		require
			valid_column_index: a_column_index >= 1 and a_column_index <= column_count
		do
			from
				Result := a_column_index - 1
			until
				Result = 0 or else column_displayed (Result)
			loop
				Result := Result - 1
			end
		ensure
			valid_result: Result >= 0 and Result < a_column_index
		end

	selection_on_click_was_enabled: BOOLEAN
		-- Was `selection_on_click' originally enabled before it
		-- was turned off for our custom processing?

	selection_on_single_button_click_was_enabled: BOOLEAN
		-- Was `selection_on_single_button_click' originally enabled before it
		-- was turned off for our custom processing?

	update_column_size_on_release (an_x, a_y, a_button: INTEGER; an_item: EV_GRID_ITEM; a_grid: RP_DATA_GRID [G]) is
			-- A pointer button release event has been received on `Current', so
			-- complete row or column resizing.
		do
			if resizing_row_internal /= Void or resizing_column_internal /=Void then
				grid.disable_capture_light
				resizing_row_internal := Void
				resizing_column_internal := Void
				relative_row_pos := 0
				relative_column_pos := 0
--				row_height := 0
			end
		end

	update_selection_handling_on_motion (an_x, a_y: INTEGER; an_item: EV_GRID_ITEM; a_grid: RP_DATA_GRID [G]) is
			-- Turn off selection handling on `Current' when above one of the title icons.
			-- This prevents the item becoming selected when the icon is clicked.
		require
			a_grid_not_void: a_grid /= Void
			a_grid_is_not_title_grid: not a_grid.is_title_grid
		local
			l_title_node: RP_GRID_TITLE_NODE [G]
			l_offset: INTEGER
			l_item: RP_DATA_GRID_LABEL_ITEM
			item_x: INTEGER
			selection_must_be_disabled: BOOLEAN
		do
			--|FIXME need to handle all alignments correctly, not just left aligned.
			if an_item /= Void then
				l_title_node ?= an_item.row.data
				if l_title_node /= Void then
					l_item ?= an_item
					l_offset := an_item.virtual_x_position + l_item.text_width + l_item.left_border + icon_spacing
					if a_grid.are_sort_icons_displayed then
						if an_x > l_offset and an_x < l_offset + l_item.sort_pixmap.width then
							if l_title_node.is_column_user_sortable (an_item.column.index) then
							selection_must_be_disabled := True
						end
						end
						l_offset := l_offset + l_item.sort_pixmap.width + icon_spacing
					end
					if a_grid.are_selection_icons_displayed then
						if an_x > l_offset and an_x < l_offset + selection_pixmap.width then
							selection_must_be_disabled := True
						end
						l_offset := l_offset + selection_pixmap.width + icon_spacing
					end
				end
					-- Now prevent the selection from occurring if we are performing a resize event
					-- on a regular grid. The header grid already has selection handling turned off.
				if not a_grid.is_title_grid and a_grid.are_columns_resizable_on_dividers then
					item_x := an_item.virtual_x_position
					if (an_x >= item_x and (an_x - item_x < edge_size)) and an_item.column.index > 1 or
						(an_x >= item_x + an_item.column.width - an_item.horizontal_indent - edge_size) then
						selection_must_be_disabled := True
					end
				end
			end
			if selection_must_be_disabled then
				if is_selection_on_click_enabled then
					original_selection_on_click_state := True
					grid.disable_selection_on_click
				end
			elseif not is_selection_on_click_enabled and original_selection_on_click_state then
				grid.enable_selection_on_click
			end
		end

	original_selection_on_click_state: BOOLEAN
		-- Original state of `is_selection_on_click_enabled' before it was disabled by `update_selection_handling_on_motion'.

	update_column_size_on_motion (an_x, a_y: INTEGER; an_item: EV_GRID_ITEM; a_grid: RP_DATA_GRID [G]) is
			-- A pointer motion event has been received on `Current', so handle resizing
			-- of rows or columns, based on the following two states:
			-- If no row or column is currently being resized, then update the displayed pointer style
			-- to reflect resizing that may take place.
			-- If a row or column is being resized, then perform the resize based on the current mouse
			-- position relative to the original start position.
		local
			item_x: INTEGER
			data_grid_column: RP_DATA_GRID_COLUMN [G]
			l_column_index: INTEGER
			l_minimum_column_width: INTEGER
			retried: BOOLEAN
		do
			if not retried then
				if is_title_grid or else are_columns_resizable_on_dividers then
					if resizing_column_internal = Void then
						if an_item /= Void then
							item_x := an_item.virtual_x_position
							left := False
							right := False
							if (an_x >= item_x and (an_x - item_x < edge_size)) and an_item.column.index > 1 then
								l_column_index := first_visible_column_index_to_left (an_item.column.index)
								if l_column_index > 0 then
									if is_title_grid then
										data_grid_column := parent_grid.column (l_column_index)
									else
										data_grid_column := column (l_column_index)
									end
									if data_grid_column.is_user_resizeable then
										left := True
									end
								end
							elseif (an_x >= item_x + an_item.column.width - an_item.horizontal_indent - edge_size) then
								l_column_index := an_item.column.index
								if is_title_grid then
									data_grid_column := parent_grid.column (l_column_index)
								else
									data_grid_column := column (l_column_index)
								end
								if data_grid_column.is_user_resizeable then
									right := True
								end
							end
						else
							if column_count > 0 then
								if is_title_grid then
									data_grid_column := parent_grid.column (parent_grid.column_count)
								else
									data_grid_column := column (column_count)
								end
								if an_x >= data_grid_column.virtual_x_position + data_grid_column.width and an_x < data_grid_column.virtual_x_position + data_grid_column.width + edge_size then
									right := True
								else
							left := False
							right := False
						end
							end
						end
					else
						if is_title_grid then
							l_minimum_column_width := parent_grid.minimum_width_of_widgets_in_column (resizing_column_internal.index)
						else
							l_minimum_column_width := minimum_width_of_widgets_in_column (resizing_column_internal.index)
						end
						resizing_column_internal.set_width ((an_x - resizing_column_internal.virtual_x_position).max (l_minimum_column_width))
						a_grid.column (resizing_column_internal.index).set_width ((an_x - resizing_column_internal.virtual_x_position).max (l_minimum_column_width))
					end
					if top or bottom or left or right then
						if not is_showing_as_resizeable then
							selection_on_single_button_click_was_enabled := grid.is_selection_on_single_button_click_enabled
							selection_on_click_was_enabled := grid.is_selection_on_click_enabled
							grid.disable_selection_on_click
							if (left or right) and not (top or bottom) then
								grid.set_pointer_style (gui.rep.pixmaps.sizewe_cursor)
							end
						end
						is_showing_as_resizeable := True
					else
						if is_showing_as_resizeable then
							if selection_on_click_was_enabled then
								grid.enable_selection_on_click
							end
							if selection_on_single_button_click_was_enabled then
								grid.enable_selection_on_single_button_click
							end
							grid.set_pointer_style (gui.rep.pixmaps.standard_cursor)
						end
						is_showing_as_resizeable := False
					end
				end
			end
		rescue
			if not retried then
				retried := True
				retry
			end
		end

	resizing_row_internal: EV_GRID_ROW
		-- Row which is currently being resized in `Current', `Void' if none.

	resizing_column_internal: RP_DATA_GRID_COLUMN [G]
		-- Column which is currently being resized in `Current', `Void' if none.

	relative_row_pos: INTEGER

	relative_column_pos: INTEGER

	row_width: INTEGER

	top, left, bottom, right: BOOLEAN
		-- For column and row resizing, these attributes
		-- hold where in relation to the row or column, the drag started.

feature {RP_DATA_GRID_ITEM} -- Implementation

	handle_key_press (a_key: EV_KEY) is
			-- Respond to key `a_key' being pressed.
		require
			a_key_not_void: a_key /= Void
		local
			l_rows: ARRAYED_LIST [EV_GRID_ROW]
			i, j, l_direction: INTEGER
			l_indices: ARRAY [INTEGER]
			l_ordering: ORDERING [INTEGER]
			l_rp_item, l_next_editable: RP_DATA_GRID_ITEM
			l_node: RP_GRID_NODE [G]
			l_title : RP_GRID_TITLE_NODE [G]
			l_string: STRING_32
			l_next_combo: RP_DATA_GRID_COMBO_ITEM
			l_next_field: RP_DATA_GRID_EDITABLE_ITEM
			l_visible_row_indexes: ARRAYED_LIST [INTEGER]
			l_top_row: RP_GRID_NODE [G]
		do
			inspect a_key.code
			when key_delete then
				if not is_read_only then
				l_rows := grid.selected_rows
				create l_indices.make (1, l_rows.count)
				from
					i := 1
				until
					i > l_rows.count
				loop
					l_indices.put (l_rows.i_th (i).index, i)
					i := i + 1
				end
				create l_ordering.make (l_indices, agent integer_less)
				from
					i := l_ordering.count
				until
					i < 1
				loop
					remove_row (row (l_ordering.i_th (i)))
					i := i - 1
				end
				end
			when key_a then
				if gui.ev_application.ctrl_pressed then
					from
						i := 1
					until
						i > grid.row_count
					loop
						if grid.row (i).is_show_requested then
							grid.select_row (i)
						end
						i := i + 1
					end
				end
			when key_c then
				if gui.ev_application.ctrl_pressed then
					l_string := ""
					if gui.ev_application.shift_pressed then
						from
							j := 1
						until
							j > titles.count
						loop
							l_string.append (titles.item (j))
							l_string.append (tab_constant)
							j := j + 1
						end
						l_string.append (new_line_constant)
					end
					Global.start_busy (Void)
					from
						i := 1
					until
						i > row_count
					loop
						l_node := row (i)
						if l_node.is_selected then
							from
								j := 1
							until
								j > grid.column_count
							loop
								if {l_label_item: RP_DATA_GRID_LABEL_ITEM} (l_node.i_th (j)) then
									l_string.append (l_label_item.text)
								end
								if j < grid.column_count then
									l_string.append (tab_constant)
								end
								j := j + 1
							end
							l_string.append (new_line_constant)
						end
						i := i + 1
					end
					l_string.remove (l_string.count)
					Global.end_busy
					gui.ev_application.clipboard.set_text (l_string)
				end
			when key_tab then
				if gui.ev_application.shift_pressed then
					l_direction := -1
				else
					l_direction := 1
				end
				l_rp_item := activated_item
				if l_rp_item /= Void then
					if l_rp_item.is_editable and is_item_visible (l_rp_item) then
						l_rp_item.disable_select
						l_next_editable := next_editable (l_rp_item, l_direction)
						if l_next_editable /= Void and then l_next_editable.is_selectable then
							l_next_combo ?= l_next_editable
							l_next_field ?= l_next_editable
							if l_next_combo /= Void and then l_next_combo.is_parented then
								l_next_combo.ensure_visible
								l_next_combo.activate
								l_next_combo.enable_select
							elseif l_next_field /= Void then
								l_next_field.ensure_visible
								l_next_field.enable_select
								l_next_field.activate
								l_next_field.select_actions.call ([])
							end
						end
					end
				end
			when key_f then
				if gui.ev_application.ctrl_pressed then
					if is_title_grid then
						parent_grid.header_node.show_filter (not parent_grid.header_node.is_filter_shown)
					else
						if not selected_nodes.is_empty then
							l_node := selected_nodes.first
						elseif not selected_items.is_empty then
							l_rp_item := selected_items.first
							if l_rp_item /= Void then
								l_node ?= l_rp_item.row.data
							end
						else
							header_node.show_filter (not header_node.is_filter_shown)
						end
						if l_node /= Void then
							l_title ?= l_node
							if l_title = Void then
								if l_node.parent_row /= Void then
									l_title ?= l_node.parent_row.subrow (1)
								end
							end
							if l_title /= Void then
								l_title.show_filter (not l_title.is_filter_shown)
							elseif l_node.parent_row = Void then
								header_node.show_filter (not header_node.is_filter_shown)
							end
						end
					end
				end
			when key_page_up then
				l_visible_row_indexes := visible_row_indexes
				if not l_visible_row_indexes.is_empty then
					l_top_row := row (l_visible_row_indexes.first)
					if l_top_row.virtual_y_position < virtual_y_position then
						l_top_row := row (l_top_row.index + 1)
					end
					from
						i := l_top_row.index
					until
						i = 0 or j + row (i).height > grid.viewable_height
					loop
						j := j + row (i).height
						i := i - 1
					end
					set_virtual_position (virtual_x_position, grid.row (i + 1).virtual_y_position)
				end
			when key_page_down then
				l_visible_row_indexes := visible_row_indexes
				if not l_visible_row_indexes.is_empty then
					l_top_row := row (l_visible_row_indexes.last)
					if l_top_row.virtual_y_position + l_top_row.height > virtual_y_position + grid.viewable_height then
						l_top_row := row (l_top_row.index - 1)
					end
					set_virtual_position (virtual_x_position, l_top_row.virtual_y_position.min (maximum_virtual_y_position))
				end
			when key_numpad_add then
				if gui.ev_application.ctrl_pressed then
					if is_title_grid then
						parent_grid.resize_columns_to_content (True)
					else
						resize_columns_to_content (True)
					end
				end
			else
			end
		end

feature {NONE} -- Implementation

	columns_to_array2_implementation (a_columns: ARRAY [INTEGER]; a_with_titles: BOOLEAN; a_only_selected_rows: BOOLEAN; a_remove_filtered: BOOLEAN): ARRAY2 [STRING] is
			-- Copy contents of `a_columns' to an array, one row per line.
			-- Include column headers if `a_with_titles' is True.
			-- Take only selected rows into account if `a_only_selected_rows' is True.
		require
			columns_not_void_implies_valid_column_indexes: a_columns /= Void implies a_columns.for_all (agent is_valid_column_index)
		local
			l_node: RP_GRID_NODE [G]
			l_secondary_title_nodes: ACTIVE_LIST [RP_GRID_NODE [G]]
			i, l_result_row_count, l_result_column_count, l_row_index_in_result: INTEGER
		do
			if a_only_selected_rows then
				l_result_row_count := selected_nodes.count
			else
				l_result_row_count := row_count
			end
			if a_with_titles then
				l_result_row_count := l_result_row_count + 1 + header_node.secondary_title_nodes.count
				if header_node.is_filter_shown then
					l_result_row_count := l_result_row_count - 1
				end
			end
			if a_columns /= Void then
				l_result_column_count := a_columns.count
			else
				l_result_column_count := column_count
			end
			create Result.make (l_result_row_count, l_result_column_count)

			l_row_index_in_result := 1
			if a_with_titles then
				add_node_to_array2 (header_node, a_columns, Result, l_row_index_in_result)
				l_row_index_in_result := l_row_index_in_result + 1
				l_secondary_title_nodes := header_node.secondary_title_nodes
				from
					i := 1
				until
					i > l_secondary_title_nodes.count
				loop
					if l_secondary_title_nodes.i_th (i) /= header_node.filter_node then
						add_node_to_array2 (l_secondary_title_nodes.i_th (i), a_columns, Result, l_row_index_in_result)
						l_row_index_in_result := l_row_index_in_result + 1
					end
					i := i + 1
				end
			end

			from
				i := 1
			until
				i > row_count
			loop
				l_node := row (i)
				if (not a_only_selected_rows or else l_node.is_selected) and
					(not a_remove_filtered or else l_node.is_show_requested) then
					add_node_to_array2 (l_node, a_columns, Result, l_row_index_in_result)
					l_row_index_in_result := l_row_index_in_result + 1
				end
				i := i + 1
			end
		ensure
			array2_not_void: Result /= Void
			no_void_strings: not Result.has (Void)
		end

	fire_node_double_press_actions (a_x, a_y, a_button: INTEGER; an_item: EV_GRID_ITEM) is
			-- Fire the `pointer_double_press_actions' for the pointed RP_GRID_NODE (if any).
		local
			l_node: RP_GRID_NODE [G]
			l_item: RP_DATA_GRID_ITEM
			l_row: EV_GRID_ROW
		do
			if an_item /= Void and then an_item.is_parented then
				l_node ?= an_item.row.data
				l_item ?= an_item
			else
				if a_y >= 0 and a_y < virtual_height then
					l_row := grid.row_at_virtual_position (a_y, False)
					if l_row /= Void then
						l_node ?= l_row.data
					end
				end
			end

			if l_node /= Void then
				l_node.pointer_double_press_actions.call ([a_x, a_y - l_node.virtual_y_position, a_button, l_item])
			end
		end

	fire_node_button_press_actions (a_x, a_y, a_button: INTEGER; an_item: EV_GRID_ITEM) is
			-- Fire the `pointer_button_press_actions' for the pointed RP_GRID_NODE (if any).
		local
			l_node: RP_GRID_NODE [G]
			l_item: RP_DATA_GRID_ITEM
			l_row: EV_GRID_ROW
		do
			if an_item /= Void and then an_item.is_parented then
				l_node ?= an_item.row.data
				l_item ?= an_item
			else
				if a_y >= 0 and a_y < virtual_height then
					l_row := grid.row_at_virtual_position (a_y, False)
					if l_row /= Void then
						l_node ?= l_row.data
					end
				end
			end
			if l_node /= Void then
				l_node.pointer_button_press_actions.call ([a_x, a_y - l_node.virtual_y_position, a_button, l_item])
			end
		end

	handle_press_for_title_items (a_x, a_y, a_button: INTEGER; an_item: EV_GRID_ITEM) is
			-- Handle a pointer button press on a grid title item if any.
		local
			l_title_node: RP_GRID_TITLE_NODE [G]
		do
			if an_item /= Void and then an_item.is_parented then
				l_title_node ?= an_item.row.data
				if l_title_node /= Void then
					l_title_node.respond_to_button_press (an_item.column.index, a_x - an_item.virtual_x_position, a_y - an_item.virtual_y_position, a_button)
				end
			end
		end

	fire_node_motion_actions (a_x, a_y: INTEGER; an_item: EV_GRID_ITEM) is
			-- Fire the `pointer_motion_actions' for the pointed RP_GRID_NODE (if any).
		local
			l_node: RP_GRID_NODE [G]
			l_item: RP_DATA_GRID_ITEM
			l_row: EV_GRID_ROW
		do
			if an_item /= Void and then an_item.is_parented then
				l_node ?= an_item.row.data
				l_item ?= an_item
			else
				if a_y >= 0 and a_y < virtual_height then
					l_row := grid.row_at_virtual_position (a_y, False)
					if l_row /= Void then
						l_node ?= l_row.data
					end
				end
			end

			if l_node /= Void then
				l_node.pointer_motion_actions.call ([a_x, a_y - l_node.virtual_y_position, l_item])
			end
		end

	update_selection_color_of_parent is
			-- In response to the focused entering or leaving `Current', update the
			-- `non_focused_selection_color' of `parent_grid' to simulate the appearance
			-- of a single grid.
		require
			is_title_grid: is_title_grid
		do
			if has_focus then
				parent_grid.grid.set_non_focused_selection_color (grid.focused_selection_color)
				parent_grid.grid.set_non_focused_selection_text_color (grid.focused_selection_text_color)
			else
				parent_grid.grid.set_non_focused_selection_color (grid.non_focused_selection_color)
				parent_grid.grid.set_non_focused_selection_text_color (grid.non_focused_selection_text_color)
			end
			parent_grid.grid.redraw
		end

	enable_capture_light is
			-- `enable_capture' but forced to use the real windows capture, not the
			-- WEL_HOOK heavy capture. This is necessary to work around a bug in the heavy capture where
			-- if you call `enable_capture' with the left mouse button down, move the mouse out of your window
			-- and then release the left mouse button, the capture ends.
		do
			grid.enable_capture_light
		end

	disable_capture_light is
			-- `disable_capture' but forced to use the real windows capture, not the
			-- WEL_HOOK heavy capture. This is necessary to work around a bug in the heavy capture where
			-- if you call `enable_capture' with the left mouse button down, move the mouse out of your window
			-- and then release the left mouse button, the capture ends.
		do
			grid.disable_capture_light
		end

	default_row_height: INTEGER is
			-- Default height of a row, based on the height of the default font.
		once
			Result := (create {EV_LABEL}).minimum_height
			if (create {PLATFORM}).is_windows then
				Result := Result + 3
			else
					-- This matches the gtk default row height used for all GtkTreeView variants.
				Result := Result + 6
			end
		ensure
			result_positive: result > 0
		end

	title_height: INTEGER is
			-- Default height of title row.
		once
			Result := default_row_height + additional_height_for_title_row
		ensure
			result_correct: Result = default_row_height + additional_height_for_title_row
		end

	last_selected_column : RP_DATA_GRID_COLUMN [G]

	update_sticky_rows_agent: PROCEDURE [RP_DATA_GRID [G], TUPLE [INTEGER, INTEGER]]
		-- Agent which performs the updating of the sticky rows.
		-- Stored, so it can be accurately removed from an action sequence as required.

	update_sticky_rows (an_x, ay: INTEGER) is
			-- Update all sticky rows in response to the virtual position
			-- changing to `an_x', `a_y'
		local
			l_locked_rows: ARRAYED_LIST [RP_GRID_NODE [G]]
			l_node : RP_GRID_NODE [G]
			l_locked_height: INTEGER
			l_first_visible_row, temp: RP_GRID_NODE [G]
			l_last_locked_row: RP_GRID_NODE [G]
			l_parent: RP_GRID_NODE [G]
			parents, l_title_nodes: ARRAYED_LIST [RP_GRID_NODE[G]]
			one_or_more_rows_locked: BOOLEAN
			adding, removing, depth_greater: BOOLEAN
			title_row: RP_GRID_TITLE_NODE [G]
			height_of_newly_locked_rows: INTEGER
			last_unlocked_index: INTEGER
			l_row: EV_GRID_ROW
			l_height : INTEGER
		do
			virtual_position_changed_actions.block
			one_or_more_rows_locked := True
				-- We must loop until no more rows have been locked in the current loop.
				-- This is necessary as the process of locking/unlocking a row moved the intersection
				-- and this may in turn require one or more rows to be locked.
			from
			until
				not one_or_more_rows_locked
			loop
				l_locked_height := 0
				one_or_more_rows_locked := False
				l_locked_rows := locked_rows
					-- Now determine the vertical offset of the final locked row.
				l_locked_height := height_of_locked_rows
					-- Now determine the final locked row.
				if not l_locked_rows.is_empty then
					l_last_locked_row := locked_rows.last
				else
					l_last_locked_row := Void
				end
				l_row := grid.row_at_virtual_position (grid.virtual_y_position + l_locked_height, True)
				if l_row /= Void then
					l_first_visible_row :=  row (l_row.index)
					depth_greater := False
					if l_last_locked_row /= Void then
						if row_is_title_row (l_last_locked_row) or l_last_locked_row.primary_title_node /= Void then
							depth_greater := row_depth (l_first_visible_row) > row_depth (l_last_locked_row.parent_row)
						else
							depth_greater := row_depth (l_first_visible_row) > row_depth (l_last_locked_row)
						end
					end
					if (l_last_locked_row = Void) or else ((l_first_visible_row.index > l_last_locked_row.index + 1) and (depth_greater)) then
						if (l_last_locked_row = Void) or else (l_first_visible_row.parent_row /= l_last_locked_row.parent_row) then
							if not removing then
								create parents.make (4)
								from
									l_parent := l_first_visible_row.parent_row
								until
									l_parent = Void or else l_parent = l_last_locked_row
								loop
									parents.extend (l_parent)
									l_parent := l_parent.parent_row
								end
								if not parents.is_empty then
									from
										parents.finish
									until
										parents.off
									loop
										height_of_newly_locked_rows := 0
										if not parents.item.row.is_locked then
											l_parent := parents.item
											one_or_more_rows_locked := True
											parents.item.row.lock_at_position (l_locked_height)
											adding := True
											locked_rows.extend (parents.item)
											if grid.is_row_height_fixed then
												l_locked_height := l_locked_height + grid.row_height
												height_of_newly_locked_rows := height_of_newly_locked_rows + grid.row_height
											else
												l_locked_height := l_locked_height + l_parent.row.height
												height_of_newly_locked_rows := height_of_newly_locked_rows + l_parent.row.height
											end
											l_last_locked_row := locked_rows.last
											title_row ?= row (parents.item.index + 1)
											if title_row /= Void  then
												if title_row.secondary_title_nodes /= Void and then not title_row.secondary_title_nodes.is_empty then
													create l_title_nodes.make (0)
													l_title_nodes.append (title_row.secondary_title_nodes)
													l_title_nodes.put_front (title_row)
												else
													create l_title_nodes.make (1)
													l_title_nodes.extend (title_row)
												end
												from l_title_nodes.start until l_title_nodes.off loop
													l_node := l_title_nodes.item
													if l_node /= Void and then l_node.grid = Current then
														l_node.row.lock_at_position (l_locked_height)
														locked_rows.extend (l_node)
														if grid.is_row_height_fixed then
															l_height := grid.row_height
														else
															l_height := l_node.height
														end
														l_locked_height := l_locked_height + l_height
														height_of_newly_locked_rows := height_of_newly_locked_rows + l_height
													end
													l_title_nodes.forth
												end
											end
											temp := l_first_visible_row
											l_first_visible_row :=  row (grid.row_at_virtual_position (grid.virtual_y_position + l_locked_height, True).index)
											if not temp.row.is_locked and temp.index /= locked_rows.last.index + 1 then
												virtual_position_changed_actions.block
												grid.set_virtual_position (grid.virtual_x_position, (grid.virtual_y_position + height_of_newly_locked_rows).min (maximum_virtual_y_position))
												virtual_position_changed_actions.resume
											end
										end
										parents.back
									end
								end
							end
						end
					else
						if not adding then
							if l_last_locked_row.row.is_locked then
								if first_visible_row.parent_row /= l_last_locked_row then
									from
										l_parent := first_visible_row
									until
										l_parent = Void or l_last_locked_row = l_parent
									loop
										l_parent := l_parent.parent_row
									end
								end
								if l_parent = Void or l_last_locked_row = l_parent then
									height_of_newly_locked_rows := 0
									title_row ?= l_last_locked_row
									create l_title_nodes.make (0)
									if title_row /= Void then
										l_title_nodes.extend (title_row)
									elseif l_last_locked_row.primary_title_node /= Void then
										l_title_nodes.append (l_last_locked_row.primary_title_node.secondary_title_nodes)
										l_title_nodes.extend (l_last_locked_row.primary_title_node)
									end
									from l_title_nodes.finish until l_title_nodes.off loop
--										last_unlocked_index := l_last_locked_row.index
										l_title_nodes.item.row.unlock
										locked_rows.finish
										locked_rows.remove
										if grid.is_row_height_fixed then
											l_height := grid.row_height
										else
											l_height := l_last_locked_row.height
										end
										height_of_newly_locked_rows := height_of_newly_locked_rows - l_height
										l_locked_height := l_locked_height - l_height
										l_title_nodes.back
									end
									l_last_locked_row := locked_rows.last
									last_unlocked_index := l_last_locked_row.index
									l_last_locked_row.row.unlock
									locked_rows.finish
									locked_rows.remove
									removing := True
									if grid.is_row_height_fixed then
										l_height := grid.row_height
									else
										l_height := l_last_locked_row.height
									end
									l_locked_height := l_locked_height - l_height
									height_of_newly_locked_rows := height_of_newly_locked_rows - l_height
									one_or_more_rows_locked := True
									l_first_visible_row :=  row (grid.row_at_virtual_position (grid.virtual_y_position + l_locked_height, True).index)
									if l_first_visible_row.index /= l_last_locked_row.index then
										if row_depth (l_last_locked_row) - row_depth (l_first_visible_row) /= 1 then
											virtual_position_changed_actions.block
											grid.set_virtual_position (grid.virtual_x_position, (grid.virtual_y_position - height_of_newly_locked_rows).max (0).min (maximum_virtual_y_position))
											virtual_position_changed_actions.resume
										end
									end
								end
							end
						end
					end
				end
			end
			virtual_position_changed_actions.resume
			if is_vertical_scrolling_per_item_relative_to_header then
				vertical_scrolling_per_item_relative_to_header_agent.call ([0, 0])
			end
			reset_widgets_contained
		end

	vertical_scrolling_per_item_relative_to_header_agent : PROCEDURE [RP_DATA_GRID [G], TUPLE [INTEGER, INTEGER]]

	vertical_scrolling_per_item_relative_to_header (an_x, ay: INTEGER) is
			-- Align top of first visible row below header with the bottom edge of header containing locked header rows
			-- Scoot up or down depending on direction of last virtual_y change
		local
			l_scrolling_up : BOOLEAN
			l_offset : INTEGER
			l_first_row : RP_GRID_NODE [G]
		do
			virtual_position_changed_actions.block
			l_scrolling_up := virtual_y_position < previous_virtual_position.second
			l_first_row := first_visible_row_below_header
			if l_first_row /= Void then
				l_offset := (virtual_y_position + height_of_locked_rows) - virtual_y_position_of_row (l_first_row)
				if l_offset.abs > l_first_row.height then
					do_nothing
				end
				if l_offset /= 0 then
					if l_scrolling_up then
						set_virtual_position (virtual_x_position, (virtual_y_position - l_offset).max (0))
					else
						set_virtual_position (virtual_x_position, (virtual_y_position + (l_first_row.height - l_offset)).min (grid.maximum_virtual_y_position))
					end
				end
			end
			previous_virtual_position.set (virtual_x_position, virtual_y_position)
			virtual_position_changed_actions.resume
		end

	first_visible_row_below_header : RP_GRID_NODE [G] is
			-- May be partially visible
		local
			l_row : EV_GRID_ROW
		do
			l_row := grid.row_at_virtual_position (grid.virtual_y_position + height_of_locked_rows, True)
			if l_row /= Void then
				Result ?= l_row.data
			end
		ensure
			previous_row_not_visible : Result = Void or Result.index = 1 or virtual_y_position_of_row (previous_viewable_row (Result)) < (virtual_y_position + height_of_locked_rows)
		end

	virtual_y_position_of_row (a_row : RP_GRID_NODE [G]) : INTEGER is
			--| FIXME: This is expensive
		require
			a_row_not_void : a_row /= Void
		local
			i : INTEGER
			l_row : RP_GRID_NODE [G]
		do
			if is_row_height_fixed then
				Result := (a_row.index - 1) * row_height
			else
				from i := 1 until i > a_row.index - 1 loop
					l_row := row (i)
					if all_parents_are_expanded (l_row) then
						if is_row_height_fixed then
							Result := Result + grid.row_height
						else
							Result := Result + l_row.height
						end
					end
					i := i + 1
				end
			end
		end

	row_is_title_row (a_row: RP_GRID_NODE [G]): BOOLEAN is
			-- Is `a_row' a title row?
		require
			a_row_not_void: a_row /= Void
		local
			title_row: RP_GRID_TITLE_NODE [G]
		do
			title_row ?= a_row
			Result := title_row /= Void
		end

	row_collapsed (a_row: EV_GRID_ROW) is
			-- Row `a_row' has been collapsed in `grid'.
			-- Remove row and all locked descendents from `locked_rows'.
		require
			a_row_not_void: a_row /= Void
		local
			l_rows: ARRAYED_LIST [RP_GRID_NODE [G]]
		do
			if attached {like row} a_row.data as l_node and then l_node.collapse_actions_saved /= Void then
				l_node.collapse_actions_saved.call (Void)
			end
			reset_widgets_contained
			if not locked_rows.is_empty then
				from
					locked_rows.start
				until
					locked_rows.off or locked_rows.item.row = a_row
				loop
					locked_rows.forth
				end
				if not locked_rows.off and then locked_rows.item.row = a_row then
						-- We perform a two stage pass as calling `unlock' can cause
						-- `locked_rows' to change, so this is to be sure that we do
						-- not crash on an invalid item look up.
					create l_rows.make (4)
					from
						locked_rows.forth
					until
						locked_rows.off
					loop
						l_rows.extend (locked_rows.item)
						locked_rows.remove
					end
					from
						l_rows.start
					until
						l_rows.off
					loop
						l_rows.item.row.unlock
						l_rows.forth
					end
				end
				redraw
			end
		end

	row_depth (a_row: RP_GRID_NODE [G]): INTEGER is
			-- Depth of `row' in parenting structure.
			-- 0 for a top level row.
		require
			a_row_not_void: a_row /= Void
		local
			l_parent: RP_GRID_NODE [G]
		do
			from
				l_parent := a_row.parent_row
			until
				l_parent = Void
			loop
				Result := Result + 1
				l_parent := l_parent.parent_row
			end
		ensure
			result_non_negative: Result >= 0
		end

	draw_header_border (a_drawable: EV_DRAWABLE; a_grid_item: EV_GRID_ITEM; a_column_index, a_row_index: INTEGER) is
			-- Draw an extra wide row separator at the bottom of `a_grid_item' if the item is non `Void' and is
			-- in the final row above the sticky row separator. This shows the divide between the sticky rows and the regular rows.
		require
			a_drawable_not_void: a_drawable /= Void
		do
			if not locked_rows.is_empty then
				-- FIXME need a better way to check this, in case somebody was to lock a row independently.
				if a_row_index = locked_rows.last.index then
					a_drawable.set_foreground_color (black)
					a_drawable.fill_rectangle (0, a_grid_item.height - 2, a_grid_item.width + a_grid_item.horizontal_indent, a_grid_item.height)
				end
			end
		end

	draw_title_icons (a_drawable: EV_DRAWABLE; a_grid_item: EV_GRID_ITEM; a_column_index, a_row_index: INTEGER) is
			-- Draw icons for grid item `a_grid_item' onto `a_drawable'.
		require
			a_drawable_not_void: a_drawable /= Void
		local
			l_height: INTEGER
			l_data_grid_item: RP_DATA_GRID_LABEL_ITEM
			l_grid_title_node: RP_GRID_TITLE_NODE [G]
			l_icon_indent: INTEGER
			l_pixmap: EV_PIXMAP
			l_total_icon_width: INTEGER
			l_column_index: INTEGER
		do
			l_data_grid_item ?= a_grid_item
			if l_data_grid_item /= Void then
				l_grid_title_node ?= l_data_grid_item.row.data
				if l_grid_title_node /= Void then
					if grid.is_row_height_fixed then
						l_height := grid.row_height
					else
						l_height := a_grid_item.row.height
					end
					if (not is_title_grid and are_sort_icons_displayed) or (is_title_grid and parent_grid.are_sort_icons_displayed_on_header_node) then
						if l_grid_title_node.is_column_user_sortable (l_data_grid_item.column.index) then
						l_total_icon_width := l_total_icon_width + l_data_grid_item.sort_pixmap.width + icon_spacing
					end
					end
					if (not is_title_grid and are_selection_icons_displayed) or (is_title_grid and parent_grid.are_selection_icons_displayed_on_header_node) then
						l_total_icon_width := l_total_icon_width + selection_pixmap.width + icon_spacing
					end
					if is_title_grid and parent_grid.are_lock_icons_displayed_on_header_node then
						l_total_icon_width := l_total_icon_width + locked_pixmap.width
					end
					if l_data_grid_item.is_left_aligned then
						l_icon_indent := l_data_grid_item.text_width + l_data_grid_item.left_border + l_data_grid_item.horizontal_indent + icon_spacing
					elseif l_data_grid_item.is_center_aligned then
						l_icon_indent := (l_data_grid_item.width - l_data_grid_item.left_border - l_data_grid_item.right_border - l_data_grid_item.text_width) // 2 + l_data_grid_item.text_width + icon_spacing
					elseif l_data_grid_item.is_right_aligned then
						l_icon_indent := l_data_grid_item.width - l_data_grid_item.right_border - l_data_grid_item.text_width - icon_spacing - l_total_icon_width
					end

					if (not is_title_grid and are_sort_icons_displayed) or (is_title_grid and parent_grid.are_sort_icons_displayed_on_header_node) then
						if l_grid_title_node.is_column_user_sortable (l_data_grid_item.column.index) then
						a_drawable.draw_pixmap (l_icon_indent, (l_height - l_data_grid_item.sort_pixmap.height) // 2, l_data_grid_item.sort_pixmap)
						l_icon_indent := l_icon_indent + l_data_grid_item.sort_pixmap.width + icon_spacing
					end
					end
					if (not is_title_grid and are_selection_icons_displayed) or (is_title_grid and parent_grid.are_selection_icons_displayed_on_header_node) then
						a_drawable.draw_pixmap (l_icon_indent, (l_height - selection_pixmap.height) // 2, selection_pixmap)
						l_icon_indent := l_icon_indent + selection_pixmap.width + icon_spacing
					end
					if is_title_grid and parent_grid.are_lock_icons_displayed_on_header_node then
						if column (a_column_index).is_locked then
							l_pixmap := unlocked_pixmap
						else
							l_pixmap := locked_pixmap
						end
						a_drawable.draw_pixmap (l_icon_indent, (l_height - l_pixmap.height) // 2, l_pixmap)
						l_icon_indent := l_icon_indent + l_pixmap.width + icon_spacing
					end
						-- We now draw an additional bar when the item to the left of this one is 0 pixels wide.
						-- Without this, people had no visual indication that there was a zero width column there and
						-- they were getting lost.
					l_column_index := l_data_grid_item.column.index
					if is_title_grid and then l_column_index > 1 and then column (l_column_index - 1).width = 0 then
						a_drawable.set_foreground_color (gui.rep.colors.black)
						a_drawable.draw_segment (2, 0, 2, l_data_grid_item.height)
					end
				end
			end
		end

	draw_foreground_for_raised_title_item (a_drawable: EV_DRAWABLE; a_grid_item: EV_GRID_ITEM; a_column_index, a_row_index: INTEGER) is
			-- If `a_grid_item' is not `Void' and is within a title row, and raised titles are enabled then draw the foreground for
			-- the raised title style, otherwise do nothing. We do not draw the text or pixmap, as these have already been drawn onto
			-- `a_drawable' by the grid.
		require
			a_drawable_not_void: a_drawable /= Void
		local
			l_x, l_y, l_width, l_height: INTEGER
			l_data_grid_item: RP_DATA_GRID_ITEM
			l_grid_title_node: RP_GRID_TITLE_NODE [G]
			l_is_sunken: BOOLEAN
		do
				l_data_grid_item ?= a_grid_item
				if l_data_grid_item /= Void then
					l_grid_title_node ?= l_data_grid_item.row.data
					if l_grid_title_node /= Void then
							l_is_sunken := l_data_grid_item.row.index = selected_header_item_row_index and
								l_data_grid_item.column.index = selected_header_item_column_index and
								is_header_item_drawn_as_pressed

							if a_grid_item.column.index = 1 and grid.is_tree_enabled then
								l_x := a_grid_item.horizontal_indent - 1
							end
							l_width := a_grid_item.width + a_grid_item.horizontal_indent
							if grid.are_column_separators_enabled and not is_title_grid then
								l_y := - 1
							else
								l_y := 0
							end
							l_height := a_grid_item.height - 1
					if (not is_title_grid and are_raised_titles_enabled) or (is_title_grid and parent_grid.are_raised_titles_enabled_on_header_node) then

							if a_grid_item.column.index = 1 then
								a_drawable.set_foreground_color (black)
								a_drawable.draw_segment (l_x, l_y, l_x, l_height)
							else
								a_drawable.set_foreground_color (very_dark_gray)
								a_drawable.draw_segment (l_x, l_y, l_x, l_height)
								a_drawable.set_foreground_color (black)
							end
							a_drawable.draw_segment (l_x, l_y, l_width, l_y)
							a_drawable.draw_segment (l_x, l_height, l_width, l_height)

							if l_is_sunken then
								a_drawable.set_foreground_color (dark_gray)
							else
								a_drawable.set_foreground_color (very_dark_gray)
							end
							a_drawable.draw_segment (l_x + 1, l_height - 1, l_width - 1, l_height - 1)

							if l_is_sunken then
								a_drawable.set_foreground_color (dark_gray)
							else
								a_drawable.set_foreground_color (white)
							end
							a_drawable.draw_segment (l_x + 1, l_y + 1, l_x + 1, l_height - 2)
							a_drawable.draw_segment (l_x + 2, l_y + 1, l_width - 1, l_y + 1)

							a_drawable.set_foreground_color (dark_gray)
							if not l_is_sunken then
								a_drawable.draw_segment (l_x + 2, l_height - 2, l_width - 2, l_height - 2)
							end
							if a_grid_item.column.index = column_count then
								a_drawable.set_foreground_color (very_dark_gray)
								a_drawable.draw_segment (l_width - 1, l_y + 1, l_width - 1, l_height - 1)
								a_drawable.set_foreground_color (dark_gray)
								a_drawable.draw_segment (l_width - 2, l_y + 2, l_width - 2, l_height - 2)
							else
								a_drawable.draw_segment (l_width - 1, l_y + 2, l_width - 1, l_height - 2)
							end
						end
--|FIXME will be requires shortly when menu handling is implemented.
--						if is_title_grid then
--							a_drawable.draw_pixmap (l_width - menu_png.width - 2, (l_height - menu_png.height) // 2, menu_png)
--						end
				end
			end
		end

-- To be aded when automatic menu handling continues.
--	menu_png: EV_PIXMAP is
--			--
--		once
--			create Result
--			Result.set_with_named_file ("menu_png.png")
--		end

	draw_background_for_raised_title_item (a_drawable: EV_DRAWABLE; a_grid_item: EV_GRID_ITEM; a_column_index, a_row_index: INTEGER) is
			-- If `a_grid_item' is not `Void' and is within a title row, and raised titles are enabled then draw the background for
			-- the raised title style, otherwise do nothing.
		require
			a_drawable_not_void: a_drawable /= Void
		local
			l_x, l_y, l_width, l_height: INTEGER
			l_data_grid_item: RP_DATA_GRID_ITEM
			l_grid_title_node: RP_GRID_TITLE_NODE [G]
		do
				l_data_grid_item ?= a_grid_item
				if l_data_grid_item /= Void then
					l_grid_title_node ?= l_data_grid_item.row.data
					if l_grid_title_node /= Void then
					if l_data_grid_item.background_color = Void and l_grid_title_node.background_color = Void then
						if a_grid_item.column.index = 1 then
							l_x := a_grid_item.horizontal_indent - 1
						end
						l_width := a_grid_item.width + a_grid_item.horizontal_indent
						l_y := 0
						l_height := a_grid_item.height - 1
						if (not is_title_grid and are_raised_titles_enabled) or (is_title_grid and parent_grid.are_raised_titles_enabled_on_header_node) then
						a_drawable.set_foreground_color (gray)
						else
							a_drawable.set_foreground_color (colors.light_gray)
						end
						a_drawable.fill_rectangle (l_x, l_y, l_width, l_height)
					end
				end
			end
		end

	fill_title_grid_background (a_drawable: EV_DRAWABLE; virtual_x, virtual_y, a_width, a_height: INTEGER) is
			-- Fill background of title_grid into `a_drawable' to have a raised appearance
			-- if `are_raised_titles_enabled', a flat appearance otherwise.
		require
			a_drawable_not_void: a_drawable /= Void
		local
			l_height: INTEGER
		do
			l_height := parent_grid.header_node.height
			if parent_grid.are_raised_titles_enabled_on_header_node then
--FIXME				l_height := title_grid.row (1).height
				a_drawable.set_foreground_color (gray)
				a_drawable.fill_rectangle (0, 0, a_width, a_height)
				a_drawable.set_foreground_color (black)
				a_drawable.draw_segment (0, 0, a_width, 0)
				a_drawable.draw_segment (0, l_height - 1, a_width, l_height - 1)
				a_drawable.set_foreground_color (white)
				a_drawable.draw_segment (0, 1, a_width, 1)
				a_drawable.draw_segment (0, 2, 0, l_height - 2)
				a_drawable.set_foreground_color (very_dark_gray)
				a_drawable.draw_segment (0, l_height - 2, a_width, l_height - 2)
				a_drawable.set_foreground_color (dark_gray)
				a_drawable.draw_segment (0, l_height - 3, a_width, l_height - 3)
			else
				a_drawable.set_foreground_color (grid.background_color)
				a_drawable.fill_rectangle (0, 0, a_width, a_height)
				a_drawable.set_foreground_color (grid.separator_color)
				a_drawable.draw_segment (0, l_height - 1, a_width, l_height - 1)
			end
		end

	update_title_width_to_match_grid_column_width (a_header_item: EV_GRID_HEADER_ITEM) is
			-- In response to width of `a_header_item' changing, update width of
			-- title column corresponding to the column associated with `a_header_item'.
		require
			a_header_item_not_void: a_header_item /= Void
		local
			l_index: INTEGER
		do
				-- FIXME The following loop to determine the actual index of `a_header_item'
				-- can be replaced with the following line of code:
				-- l_index := a_header_item.column.index
				-- when `column' is exported from EV_GRID_HEADER_ITEM and available in all versions
				-- we use here. The advantage of this is that it will be faster.
			from
				l_index := 1
			until
				column (l_index).column.header_item = a_header_item
			loop
				l_index := l_index + 1
			end
			if title_grid /= Void then
				title_grid.set_column_width (a_header_item.width, l_index)
			end
		end

	update_title_bar_offset (a_virtual_x, a_virtual_y: INTEGER) is
			-- Update horizontal offset of title bar in response to the
			-- virtual position of `Current' changing.
		do
			if title_grid /= Void then
					-- FIXME we must restrict the title offset to the maximum permitted, for the
					-- case where a horizontal scroll bar is shown in the main grid. This "pulls" the title bar
					-- to the right when we are scrolled to the very right. I do not know a way round this for now as it is
					-- a valid restriction on the title grid.
				title_grid.grid.set_virtual_position (a_virtual_x.min (title_grid.maximum_virtual_x_position), title_grid.grid.virtual_y_position)
			end
		end

	additional_height_for_title_row: INTEGER is 6
		-- Additional height to be added to main title rows.

feature {RP_DATA_GRID, RP_GRID_TITLE_NODE, RP_DATA_GRID_COLUMN, RP_DATA_GRID_ITEM, RP_GRID_NODE, RP_DATA_GRID_FILTER_GUI} -- Implementation

	column_resize_actions_internal: ACTION_SEQUENCE [TUPLE [RP_DATA_GRID_COLUMN [G], INTEGER]]
			-- Actions to be performed when a column in `Current' is resized.
			-- Event data is column resized and new width of that column in pixels.

	column_lock_actions_internal: EV_INTEGER_ACTION_SEQUENCE
			-- Internal actions to be performed when a column in `Current' is locked.

	column_unlock_actions_internal: EV_INTEGER_ACTION_SEQUENCE
			-- Internal actions to be performed when a column in `Current' is unlocked.

	row_select_actions_internal: EV_GRID_ROW_ACTION_SEQUENCE

	row_deselect_actions_internal: EV_GRID_ROW_ACTION_SEQUENCE

	handle_row_selection (a_row: EV_GRID_ROW) is
			-- `a_row' has been selected, so propagate the selection event.
		require
			a_row_not_void: a_row /= Void
			a_row_parented_in_current: a_row.parent = grid
		do
			if row_select_actions_internal /= Void then
				row_select_actions_internal.call ([a_row])
			end
			if attached {like row} a_row.data as l_node and then l_node.select_actions_saved /= Void then
				l_node.select_actions.call (Void)
			end
		end

	handle_row_deselection (a_row: EV_GRID_ROW) is
			-- `a_row' has been deselected, so propagate the selection event.
		require
			a_row_not_void: a_row /= Void
			a_row_parented_in_current: a_row.parent = grid
		do
			if row_deselect_actions_internal /= Void then
				row_deselect_actions_internal.call ([a_row])
			end
			if attached {like row} a_row.data as l_node and then l_node.deselect_actions_saved /= Void then
				l_node.deselect_actions.call (Void)
			end
		end

	clear_filter_gui is
			-- Destroy the current `filter_gui'.
		do
			if filter_gui /= Void then
				filter_gui.destroy
				filter_gui.container.destroy
				filter_gui := Void
			end
		ensure
			filter_gui_void: filter_gui = Void
		end

	set_filter_gui (a_gui: RP_DATA_GRID_FILTER_GUI [G]) is
			-- Assign `a_gui' to `filter_gui'.
		require
			a_gui_not_void: a_gui /= Void
		do
			filter_gui := a_gui
		ensure
			filter_gui_set: filter_gui = a_gui
		end

	selection_pixmap: EV_PIXMAP is
			-- Pixmap used to indicate column selection.
		once
			Result := gui.factory.new_pixmap_from_file ("grid_select_column.ico")
			if Result = Void then
				create Result.make_with_size (16, 16)
				Result.set_foreground_color (grid.focused_selection_color)
				Result.fill_rectangle (0, 0, 16, 16)
			end
		ensure
			Result_not_void: Result /= Void
		end

	locked_pixmap: EV_PIXMAP is
			-- Pixmap used to indicate unlocked column.
		once
			Result := gui.factory.new_pixmap_from_file ("pushpin_gray.ico")
			if Result = Void then
				create Result.make_with_size (16, 16)
				Result.set_foreground_color (colors.green)
				Result.fill_rectangle (0, 0, 16, 16)
			end
		ensure
			Result_not_void: Result /= Void
		end

	unlocked_pixmap: EV_PIXMAP is
			-- Pixmap used to indicate locked column
		once
			Result := gui.factory.new_pixmap_from_file ("pushpin.ico")
			if Result = Void then
				create Result.make_with_size (16, 16)
				Result.set_foreground_color (colors.red)
				Result.fill_rectangle (0, 0, 16, 16)
			end
		ensure
			Result_not_void: Result /= Void
		end

	reset_widgets_contained is
			-- Clear status of widgets contained in `Current'
			-- so they are forced to be refreshed completely upon the
			-- next re-draw.
		do
			widgets_just_refreshed.wipe_out
			clear_stale_widgets
			is_refresh_needed := True
		end

	update_title_height is
			-- Updated height of `title_grid' to that required to display the titles
			-- and any secondary header nodes.
		local
			l_height, l_divider : INTEGER
			l_secondary_nodes : ACTIVE_LIST [RP_GRID_NODE [G]]
		do
			if not is_title_grid then
				title_grid.update_title_height
			else
				if parent_grid.header_node.height /= 16 then
					l_height := parent_grid.header_node.height
				else
					l_height := title_height
				end
				if parent_grid.are_row_separators_enabled then
					l_divider := 1 -- FIXME -- Would be better this value were available as a constant
				end
				l_secondary_nodes := parent_grid.header_node.secondary_title_nodes
				if not l_secondary_nodes.is_empty then
					from l_secondary_nodes.start until l_secondary_nodes.off or l_secondary_nodes.item = Void loop
						l_height := l_height + l_secondary_nodes.item.height + l_divider
						l_secondary_nodes.forth
					end
				end
				set_minimum_height (l_height)
			end
		end

feature {RP_DATA_GRID} -- Implementation

	set_as_title_grid (a_parent_grid: RP_DATA_GRID [G]) is
			-- Set `Current' as a title grid of `a_parent_grid'.
		require
			a_parent_grid_not_void: a_parent_grid /= Void
			a_parent_grid_not_current: a_parent_grid /= Current
			a_parent_grid_not_title_grid: not a_parent_grid.is_title_grid
		do
			parent_grid := a_parent_grid
		ensure
			parent_grid_set: parent_grid = a_parent_grid
			is_title_grid: is_title_grid
		end

	update_title_item_state_on_press (an_x, a_y, a_button: INTEGER; an_item: EV_GRID_ITEM) is
			-- Handle any required internal processing when a button is pressed.
		local
			l_data_grid_item: RP_DATA_GRID_ITEM
			l_grid_title_node: RP_GRID_TITLE_NODE [G]
		do
			if not is_showing_as_resizeable and a_button = 1 then
					-- If a click event has already been received to resize a column
					-- then we must not depress the title item. Also, we only want the
					-- sunken click appearance when the left mouse button is pressed.
				if an_item /= Void and then an_item.is_parented then
					l_data_grid_item ?= an_item
					if l_data_grid_item /= Void then
						l_grid_title_node ?= l_data_grid_item.row.data
						if l_grid_title_node /= Void then
							selected_header_item_column_index := l_data_grid_item.column.index
							selected_header_item_row_index := l_data_grid_item.row.index
							l_data_grid_item.redraw
							grid.enable_capture_light
							is_header_item_pressed := True
							is_header_item_drawn_as_pressed := True
						end
					end
				end
			end
		end

	update_title_item_state_on_release (an_x, a_y, a_button: INTEGER; an_item: EV_GRID_ITEM) is
			-- Handle any required internal processing when a button is released.
		local
			l_item: EV_GRID_ITEM
		do
			if not is_showing_as_resizeable then
				if is_header_item_pressed then
					grid.disable_capture_light
					l_item := grid.item (selected_header_item_column_index, selected_header_item_row_index)
					if l_item /= Void then
							-- This can sometimes be Void as pressing a title row
							-- causes all of the items to be re-inserted during a sort.
						l_item.redraw
					end
					is_header_item_pressed := False
					is_header_item_drawn_as_pressed := False
					selected_header_item_column_index := - 1
					selected_header_item_row_index := - 1
				end
			end
		end

	update_title_item_state_on_motion (an_x, a_y: INTEGER; an_item: EV_GRID_ITEM) is
			-- Handle any required internal processing when a button is pressed.
		local
			l_item: EV_GRID_ITEM
		do
			if not is_showing_as_resizeable then
				if is_header_item_pressed then
					if an_item /= Void then
						if an_item.row.index = selected_header_item_row_index and an_item.column.index = selected_header_item_column_index then
							is_header_item_drawn_as_pressed := True
							an_item.redraw
						else
							is_header_item_drawn_as_pressed := False
							l_item := grid.item (selected_header_item_column_index, selected_header_item_row_index)
							if l_item /= Void then
								-- This can sometimes be Void as pressing a title row
								-- causes all of the items to be re-inserted during a sort.
								l_item.redraw
							end
						end
					else
						is_header_item_drawn_as_pressed := False
						l_item := grid.item (selected_header_item_column_index, selected_header_item_row_index)
						if l_item /= Void then
							-- This can sometimes be Void as pressing a title row
							-- causes all of the items to be re-inserted during a sort.
							l_item.redraw
						end
					end
				end
			end
		end

-- FIXME, with the rebuilding of the header item upon clicking,
-- it is not possible to store a reference to the item, but instead
-- must store the column and row index which is definitely not as good.
--	selected_header_item: RP_DATA_GRID_ITEM
		-- Header item currently sunken in `Current'.

	is_header_item_pressed: BOOLEAN
		-- Is a header item currently acting as pressed within `Current'.
		-- Note that it may not be drawn as pressed if the mouse is not currently
		-- over it.

	is_header_item_drawn_as_pressed: BOOLEAN
		-- Is a header item currently drawn as pressed within `Current'.

	selected_header_item_row_index: INTEGER
		-- Row index of header item that is currently acting as pressed.

	selected_header_item_column_index: INTEGER
		-- Column index of header item that is currently acting as pressed.

	minimum_width_of_widgets_in_column (a_column_index: INTEGER): INTEGER is
			-- Greatest minimum width of all widgets currently displayed in column `a_column_index'.
		require
			valid_a_column_index: a_column_index >= 1 and a_column_index <= column_count
		local
			l_first_visible_row_index, l_last_visible_row_index: INTEGER
			l_first_visible_row, l_last_visible_row: RP_GRID_NODE [G]
			i, j: INTEGER
			l_widget_access_column: INTEGER
			l_node: RP_GRID_NODE [G]
			l_separator_width: INTEGER
			l_widget: EV_WIDGET
			l_total_other_column_width: INTEGER
			l_minimum_width: INTEGER
			l_widget_column_span: INTEGER
			l_final_separator_width: INTEGER
			l_widget_item: RP_DATA_GRID_WIDGET_ITEM
		do
			l_first_visible_row := first_visible_row
			l_last_visible_row := last_visible_row
			if l_first_visible_row /= Void and l_last_visible_row /= Void then
				l_first_visible_row_index := first_visible_row.index
				l_last_visible_row_index := last_visible_row.index
				if are_column_separators_enabled then
					l_separator_width := 1
				end
				from
					i := l_first_visible_row_index
				until
					i > l_last_visible_row_index
				loop
					l_node := row (i)
					l_widget_item := Void
					if {l_widget_item1: RP_DATA_GRID_WIDGET_ITEM} l_node.i_th (a_column_index) then
						l_widget_access_column := a_column_index
						l_widget_item := l_widget_item1
					else
						from
							j := a_column_index - 1
						until
							j < 1 or l_widget_item /= Void
						loop
							if {l_widget_item2: RP_DATA_GRID_WIDGET_ITEM} l_node.i_th (j) then
								l_widget_item := l_widget_item2
							end
							j := j - 1
						end
						l_widget_access_column := j + 1
					end
					if l_widget_item /= Void then
							-- If `widget_access' is not Void, it means the row of the current item contains a widget at a lower column value.

						l_widget := l_widget_item.widget
						if l_widget /= Void then
							l_widget_column_span := l_widget_item.widget_column_span.max (1)
							if l_widget_item.is_widget_width_dominant and l_widget_column_span + l_widget_access_column - 1 >= a_column_index then
									-- Check that the widget item found in the row spans across column `a_column_index' and that the width of the widget is dominant. Nothing to do here if not.

								l_total_other_column_width := column_span (l_widget_access_column, l_widget_access_column + l_widget_column_span - 1) - column (a_column_index).width
									-- Calculate the total widths of all columns that the widget item spans, excluding the column `a_column_index'.

								if l_widget_access_column = 1 then
										-- If the widget spans the first column, then we need to remove the node indent which is the area used to display the
										-- tree node, thereby excluding that area from the total widths of the other columns.
									l_total_other_column_width := l_total_other_column_width - l_node.item_indent
								end

								if a_column_index /= l_widget_access_column + l_widget_column_span - 1 then
										-- Remove the seperator width from the total width of other columns if the
										-- current column is not the final column in the widget span.
									l_total_other_column_width := l_total_other_column_width - l_separator_width
								end
								l_minimum_width := (l_widget.minimum_width - l_total_other_column_width).max (0)
									-- Calculate the minium width for this item in the column by looking at the minimum width of the item
									-- and the space available for it on all other columns. Return at least 0.

								if l_widget_access_column + l_widget_column_span - 1 = a_column_index then
										-- If we are the final column of the widget span, then we should include the width of the
										-- seperator.
									l_final_separator_width := l_separator_width
								end
--								if l_widget_access.is_widget_width_dominant then
										-- If the widget width is dominant, then set the result to the maximum of the previously computed
										-- value and the current maximum required for this row.
									Result := Result.max (l_minimum_width + l_final_separator_width)
--								end
						end
						end
					end
					i := i + 1
				end
			end
		ensure
			result_greater_than_zero: Result >= 0
		end

	on_mouse_wheel (a_int: INTEGER) is
			-- Respond to a mouse wheel change of `a_int' clicks
			-- by moving the grid so that the first visible row index is
			-- changed by `a_int'.
		local
			l_next_first_visible_row: INTEGER
			l_first_visible_row_index: INTEGER
			l_first_visible_row: RP_GRID_NODE [G]
			l_viewable_row_indexes: ARRAYED_LIST [INTEGER]
			l_visible_indexes: ROSE_LINEAR_ARRAY [INTEGER]
		do
			if is_title_grid then
				parent_grid.on_mouse_wheel (a_int)
			elseif row_count > 0 then
				l_first_visible_row := first_visible_row
				if l_first_visible_row /= Void then
					l_first_visible_row_index := first_visible_row.index
					l_viewable_row_indexes := grid.viewable_row_indexes
					if l_viewable_row_indexes.is_empty then
							-- Currently `viewable_row_indexes' is empty if the grid internally does not use them.
							-- We can therefore perform a simple calculation.
							l_next_first_visible_row := (l_first_visible_row_index - a_int).min (row_count).max (1)
					else
						create l_visible_indexes.make_from_linear (l_viewable_row_indexes)
						l_visible_indexes := l_visible_indexes.subarray (1, grid.implementation.visible_row_count)
						l_visible_indexes.set_comparison (agent integers_less_than)
							l_visible_indexes.binary_search (l_first_visible_row_index)
						check
								item_found: l_visible_indexes.current_item = l_first_visible_row_index
						end
						l_visible_indexes.go_i_th (l_visible_indexes.index - a_int)
						if not l_visible_indexes.off then
							l_next_first_visible_row := l_visible_indexes.current_item
						else
								l_next_first_visible_row := l_first_visible_row_index
						end
					end
					grid.set_virtual_position (grid.virtual_x_position, (row (l_next_first_visible_row).row.virtual_y_position_unlocked).min (grid.maximum_virtual_y_position))
				end
			end
		end

	integers_less_than (a_int, b_int: INTEGER): BOOLEAN is
			-- Is `a_int' less than `b_int'?
		do
			Result := a_int < b_int
		end

feature {RP_GRID_TITLE_NODE, RP_DATA_GRID_LABEL_ITEM, RP_DATA_GRID_COLUMN} -- Original implementation

	filter_gui: RP_DATA_GRID_FILTER_GUI [G]
			--	remove_on_focus_list : ROSE_LINEAR_ARRAY [EV_WIDGET]

	black: EV_COLOR is
			-- Once access to EV_COLOR black.
		once
			Result := (create {EV_STOCK_COLORS}).black
		end

	white: EV_COLOR is
			-- Once access to EV_COLOR black.
		once
			Result := (create {EV_STOCK_COLORS}).white
		ensure
			result_not_void: Result /= Void
		end

	gray: EV_COLOR is
			-- Once access to EV_COLOR black.
		once
			Result := (create {EV_LABEL}).background_color
		ensure
			result_not_void: Result /= Void
		end

	dark_gray: EV_COLOR is
			-- Once access to EV_COLOR dark gray.
		once
			Result := create {EV_COLOR}.make_with_8_bit_rgb (150, 150, 150)
		ensure
			result_not_void: Result /= Void
		end

	very_dark_gray: EV_COLOR is
			-- Once access to EV_COLOR very dark gray
		once
			Result := create {EV_COLOR}.make_with_8_bit_rgb (75, 75, 75)
		ensure
			result_not_void: Result /= Void
		end

	icon_spacing: INTEGER is 3
		-- Spacing between icons displayed on header node.

feature -- Original implementation

	grid: ROSE_EV_GRID
		-- Actual grid withi which `Current' is implemented.

feature {RP_GRID_NODE, RP_DATA_GRID} -- Original implementation

	largest_widget_span : INTEGER
		-- Largest column span of any widget contained in `Current'.

	set_largest_widget_span (a_span : INTEGER) is
			-- Assign `a_span' to `largest_widget_span'.
		do
			largest_widget_span := a_span
		ensure
			largest_widget_span_set: largest_widget_span = a_span
		end

	column_span (a_column1, a_column2 : INTEGER) : INTEGER is
			-- Cumulative widths of columns from `a_column1' to `a_column2' inclusive.
		require
			column1_not_greater_than_column2: a_column1 <= a_column2
		local
			i : INTEGER
		do
			if is_valid_column_index (a_column1) and is_valid_column_index (a_column2) then
				from i := a_column1 until i > a_column2 loop
					Result := Result + column (i).width
					i := i + 1
				end
			end
		ensure
			result_non_negative: Result >= 0
		end

	ensure_sufficient_columns (a_column: INTEGER) is
			-- column display limit is set by header columns
		require
			a_column_non_negative: a_column >= 0
		local
			i: INTEGER
			l_label_item: RP_DATA_GRID_ITEM
			l_column: RP_DATA_GRID_COLUMN [G]
			l_title : RP_GRID_TITLE_NODE [G]
		do
			if column_count /= titles.count then
				do_nothing
			end
			if a_column > grid.column_count then
				from
					i := grid.column_count + 1
				until
					i > a_column
				loop
					grid.insert_new_column (i)
					create l_column
					grid.column (i).set_data (l_column)
					l_column.set_column (grid.column (i))
					l_column.set_parent (Current)
					if not is_title_grid and i <= title_grid.column_count then
						grid.column (i).set_width (title_grid.column (i).width)
					end
					grid.column (i).set_title ("")
					i := i + 1
				end
			end
			if titles.count < column_count then
				from
				until
					titles.count = column_count
				loop
					titles.force ("", titles.count + 1)
					if not is_title_grid then
							-- This is a bit of a hack which enables us to add to
							-- an existing title item without having to rebuild it.
						l_label_item := title_grid.header_node.title_item (<<"">>, titles.count)-- + 1)
						title_grid.set_item (titles.count, 1, l_label_item)
					end
				end
			end
			if titles.count > column_count then
					-- This handles the case where you wipe out the grid
					-- and then add a node with less items than the existing
					-- title count.
				from
				until
					titles.count = column_count
				loop
					grid.insert_new_column (column_count + 1)
					grid.column (column_count).set_width (title_grid.column (column_count).width)
				end
			end
			if is_title_grid then
				l_title ?= row (1)
				if l_title /= Void then
					l_title.ensure_sufficient_filter_columns (column_count)
				end
			end
		ensure
			column_count_not_changed_if_sufficient: old column_count >= a_column implies column_count = old column_count
			column_count_changed_if_not_sufficient: old column_count < a_column implies column_count = a_column or column_count = titles.count
			column_count_equals_title_count: column_count = titles.count
		end

	widgets_just_refreshed: HASH_TABLE [RP_DATA_GRID_WIDGET_ITEM, INTEGER]
		-- All widgets that have been refreshed since execution of `refresh_widgets' started.
		-- This table is used in order to decide which widgets are actually still displayed.

	widgets_contained_in_fixed: HASH_TABLE [RP_DATA_GRID_WIDGET_ITEM, INTEGER]
		-- All widgets that are currently contained in `fixed'. By comparing this to
		-- `widgets_just_refreshed' at the end of `refresh_widgets', we can decide which ones
		-- should be removed.

	refresh_widgets is
			-- Refresh all widgets displayed in `Current'.
		local
			l_locked_columns: ARRAYED_LIST [EV_GRID_COLUMN]
			l_locked_rows: ARRAYED_LIST [EV_GRID_ROW]
			l_row : RP_GRID_NODE [G]
			l_first_column, l_last_column : EV_GRID_COLUMN
			l_first_column_index, l_last_column_index : INTEGER
			l_visible_row_indexes: ARRAYED_LIST [INTEGER_32]
		do
			widgets_just_refreshed.wipe_out
			if is_displayed and then is_refresh_needed and not is_redrawing then --and row_count /= 0 then
				if row_count > 0 and column_count > 0 then
					l_visible_row_indexes := visible_row_indexes
					l_first_column := first_visible_column
					l_last_column := last_visible_column
					if l_first_column /= Void and l_last_column /= Void and not visible_row_indexes.is_empty then
						l_first_column_index := l_first_column.index
						l_last_column_index := l_last_column.index

						--|FIXME convert `display_widgets' to take an arrayed list of column indexes,
						--| then if there is a single column locked and you are scrolled through many columns,
						--| you will not need to iterate all of those in the middle that are not displayed.
						l_locked_columns := grid.locked_columns
						from l_locked_columns.start until l_locked_columns.off loop
							l_first_column_index := l_first_column_index.min (l_locked_columns.item.index)
							l_last_column_index := l_last_column_index.max (l_locked_columns.item.index)
							l_locked_columns.forth
						end
						l_locked_rows := grid.locked_rows
						from l_locked_rows.start until l_locked_rows.off loop
							l_row ?= l_locked_rows.item.data
							l_row.display_widgets (l_first_column_index, l_last_column_index)
							l_locked_rows.forth
						end
						from
							l_visible_row_indexes.start
						until
							l_visible_row_indexes.off
						loop
							row (l_visible_row_indexes.item).display_widgets (l_first_column_index, l_last_column_index)
							l_visible_row_indexes.forth
						end
					end
				end
				clear_stale_widgets
				pending_counter := (pending_counter - 1).max (0)
				is_refresh_needed := False
			end
		end

	set_refresh_needed (a_int1, a_int2 : INTEGER) is
			-- Set `is_refresh_needed' to `True'.
		do
			is_refresh_needed := True
		end

feature {NONE} -- Original implementation

	is_refresh_needed : BOOLEAN
		-- Do widgets contained in `Current' (may be none) need to be refreshed?

	clear_stale_widgets is
			-- Void out widgets which are not visible and not persistent
		local
			l_widget_item: RP_DATA_GRID_WIDGET_ITEM
		do
				-- Now remove all of the widgets contained within `fixed' that were not
				-- just placed as visible.
			from
				widgets_contained_in_fixed.start
			until
				widgets_contained_in_fixed.off
			loop
				l_widget_item := widgets_contained_in_fixed.item_for_iteration
				if not widgets_just_refreshed.has (l_widget_item.object_id) then
					widgets_contained_in_fixed.remove (l_widget_item.object_id)
						--| FIXME can we improve this as it appears to be quite slow?
					l_widget_item.widget.parent.prune_all (l_widget_item.widget)
					l_widget_item.widget.resize_actions.wipe_out
				else
					widgets_contained_in_fixed.forth
				end
			end
		end

	is_redrawing : BOOLEAN is
			-- Is `Current' within a redraw?
		do
			Result := grid.implementation.drawer.is_redrawing
		end

	test is
			-- Test `Current' by adding a number of items.
		local
			l_item: RP_DATA_GRID_COMBO_ITEM
			l_nodes: ARRAY [RP_GRID_NODE [G]]
			l_editable: RP_DATA_GRID_EDITABLE_ITEM
			l_default: G
		do
			create l_item.make_with_text ("item")
			l_item.set_item_strings (<< "item" >>)
			l_item.set_background_color (colors.yellow)
			create l_editable.make_with_text ("edit me")
			l_nodes := <<
				create {RP_GRID_NODE [G]}.make (<< "eaaaa", 100, l_item >>, Void, l_default),
				create {RP_GRID_NODE [G]}.make_from_array (<< ["eaaaa", colors.blue, colors.red], 100, l_editable >>),
				create {RP_GRID_NODE [G]}.make (<< "aaaa", 11, "hello" >>, agent tested_rows, l_default)
			>>
			l_nodes.item (l_nodes.upper).i_th (3).drop_actions.extend (agent test_drop_action)
			make (<< "first", "second", "third" >>, l_nodes)
			grid.enable_multiple_row_selection
			set_item_pebble_function (agent test_pebble)
		end

	test_drop_action (a_value: ANY) is
			-- Print `a_value' to console as part of testing via `test'.
		do
			print (a_value)
			print ("tested")
		end

	tested_rows: RP_TITLED_NODES [G] is
			--tested rows list for grid
		local
			l_title: RP_GRID_TITLE_NODE [G]
			i: INTEGER
			l_default: G
		do
			create l_title.make (<< 1, 2, 3, 4, 5 >>)
			create Result.make_empty (l_title)
			from
				i := 1
			until
				i > 10
			loop
				Result.extend (create {RP_GRID_NODE [G]}.make (<< i.out, 11, 10 - i >>, agent tested_rows, l_default))
				i := i + 1
			end
		end

	test_pebble (a_item: EV_GRID_ITEM): STRING is
			-- Return a STRING given `a_item' for use in testing Pick and Drop.
		do
			Result := "Hello"
		end

	integer_less (i, j: INTEGER): BOOLEAN is
			-- Is `i' less than `j'?
		do
			Result := i < j
		end

	first_viewable_column : INTEGER

	set_first_viewable_column (a_column : INTEGER) is
			-- Assign `a_column' to `first_viewable_column'.
		do
			first_viewable_column := a_column
		end

	last_viewable_column : INTEGER

	set_last_viewable_column (a_column : INTEGER) is
			-- Set 'last_viewable_column'
		do
			last_viewable_column := a_column
		end

feature {NONE} -- Implementation

	initialize_notify_events is
			-- Connect notification events to `Current'.
		do
			create remove_event
			create pre_remove_event
			drop_actions := grid.drop_actions
			grid.pointer_double_press_item_actions.extend (agent pointer_button_double_press_received_on_grid)
			key_press_actions.extend (agent handle_key_press)
		end

	columns_to_string_do_all_implementation (a_item, a_result, a_item_seperator: STRING; a_array_width: INTEGER; a_counter: INTEGER_REF) is
			-- Append `a_item' to `a_result', followed by `a_item_seperator' if not at end of line given by `a_array_width' and computed from
			-- modulus of `a_counter', otherwise append a newline character.
		require
			strings_not_void: a_item /= Void and a_result /= Void and a_item_seperator /= Void
		do
			a_result.append (a_item)
			a_counter.set_item (a_counter + 1)
			if (a_counter \\ a_array_width).is_equal (0) then
				a_result.append (new_line_constant)
			else
				a_result.append (a_item_seperator)
			end
		end

	add_node_to_array2 (a_node: RP_GRID_NODE [G]; a_columns: ARRAY [INTEGER]; a_array2: ARRAY2 [STRING]; a_row_index: INTEGER) is
			-- Add representation of `a_node' to `a_array2' at row `a_index'.
			-- `a_columns' specifies the column indexes of `a_node' to be inserted in `a_array2', all are inserted if `a_columns' is Void.
		require
			a_node_not_void: a_node /= Void
			a_array2_not_void: a_array2 /= Void
			a_index_valid: a_row_index >= 1 and a_row_index <= a_array2.height
			a_array2_may_contain_node_when_columns_void: a_columns = Void implies a_node.count <= a_array2.width
			a_array2_may_contain_node_when_columns_not_void: a_columns /= Void implies a_columns.count = a_array2.width
			columns_not_void_implies_valid_column_indexes: a_columns /= Void implies a_columns.for_all (agent is_valid_column_index)
		local
			i: INTEGER
			l_width: INTEGER
			l_column_index: INTEGER
			l_text: STRING
		do
			l_width := a_array2.width
			from
				i := 1
			until
				i > l_width
			loop
				if a_columns = Void then
					l_column_index := i
				else
					l_column_index := a_columns.item (i)
				end
				l_text := ""
				if attached {RP_DATA_GRID_TOGGLE_ITEM} a_node.i_th (l_column_index) as l_toggle_item then
					if l_toggle_item.is_on then
						l_text := "1"
					else
						l_text := "0"
					end
				elseif attached {RP_DATA_GRID_LABEL_ITEM} a_node.i_th (l_column_index) as l_label_item then
					l_text := l_label_item.text
				end
				a_array2.put (l_text, a_row_index, i)
				i := i + 1
			end
		end

	new_line_constant: STRING is "%N"

	tab_constant: STRING is "%T"

feature {NONE} -- Implementation / Iteration support

	do_node_items_if_with_index (a_node: RP_GRID_NODE [G];
			a_is_column_included: FUNCTION [ANY, TUPLE [RP_DATA_GRID_ITEM], BOOLEAN];
			a_agent: PROCEDURE [ANY, TUPLE [RP_DATA_GRID_ITEM, INTEGER, INTEGER]]; a_row_index: CELL [INTEGER]) is
			-- processes all items in `a_node' using do_item_with_index (which will call `a_agent' with item and indexes).
			-- if `a_is_column_included' is not Void, process only items for which `a_is_column_included' is True
			-- increments `a_row_index', which maintains the index within set of processed nodes
		require
			a_node_not_void: a_node /= Void
		local
			l_column: CELL [INTEGER]
		do
			create l_column.put (1)
			if a_is_column_included /= Void then
				a_node.do_if (agent do_item_with_index (?, a_agent, a_row_index, l_column), a_is_column_included)
			else
				a_node.do_all (agent do_item_with_index (?, a_agent, a_row_index, l_column))
			end
			a_row_index.put (a_row_index.item + 1)
		end

	do_item_with_index (a_item: RP_DATA_GRID_ITEM; a_agent: PROCEDURE [ANY, TUPLE [RP_DATA_GRID_ITEM, INTEGER, INTEGER]];
			a_row_index, a_column_index: CELL [INTEGER]) is
			-- calls `a_agent' with `a_item', `a_row_index', and `a_column_index';
			-- increments `a_column_index', which maintains the index within the set of valid items (of a node)
			-- (this is the action part of do_items_if_with_index... it is called with each item that passes the row/column conditions)
		require
			a_agent_not_void: a_agent /= Void
			a_indexes_not_void: a_row_index /= Void and a_column_index /= Void
		do
			a_agent.call ([a_item, a_row_index.item, a_column_index.item])
			a_column_index.put (a_column_index.item + 1)
		end

feature {NONE} -- Assertion-related

	previous_viewable_row (a_row : RP_GRID_NODE [G]) : RP_GRID_NODE [G] is
			-- Viewable: not inside a collapsed parent_row
		require
			a_row_not_void : a_row /= Void
		local
			i : INTEGER
			l_row : RP_GRID_NODE [G]
		do
			from i := a_row.index - 1 until i < 1 or Result /= Void loop
				l_row := row (i)
				if all_parents_are_expanded (l_row) then
					Result := l_row
				end
				i := i - 1
			end
		end

invariant
	header_node_not_void: header_node /= Void

end
