note

	description: "[
		SQL table name parsers.
		Based on parser found in http://docs.openlinksw.com/virtuoso/yacc.html
	]"
	author: "Karine Bezault"
	last_editor: "$Author: an $"
	date: "$Date: 2013/11/11 $"
	revision: "$Revision: #5 $"
	path: "$File: //rose/source/kernel/infrastructure/persistence/sql_table_name_parser.e $"

class SQL_TABLE_NAME_PARSER

inherit

	YY_PARSER_SKELETON
		rename
			make as make_parser_skeleton
		redefine
			report_error
		end

	SQL_TABLE_NAME_SCANNER
		rename
			make as make_sql_table_name_scanner
		redefine
			fatal_error
		end

	KL_SHARED_ARGUMENTS
		export {NONE} all end
	
create

	make, execute


feature {NONE} -- Implementation

	yy_build_parser_tables
			-- Build parser tables.
		do
			yytranslate := yytranslate_template
			yyr1 := yyr1_template
			yytypes1 := yytypes1_template
			yytypes2 := yytypes2_template
			yydefact := yydefact_template
			yydefgoto := yydefgoto_template
			yypact := yypact_template
			yypgoto := yypgoto_template
			yytable := yytable_template
			yycheck := yycheck_template
		end

	yy_create_value_stacks
			-- Create value stacks.
		do
			create yyspecial_routines1
			yyvsc1 := yyInitial_yyvs_size
			yyvs1 := yyspecial_routines1.make (yyvsc1)
			create yyspecial_routines2
			yyvsc2 := yyInitial_yyvs_size
			yyvs2 := yyspecial_routines2.make (yyvsc2)
		end

	yy_init_value_stacks
			-- Initialize value stacks.
		do
			yyvsp1 := -1
			yyvsp2 := -1
		end

	yy_clear_value_stacks
			-- Clear objects in semantic value stacks so that
			-- they can be collected by the garbage collector.
		do
			yyvs1.keep_head (0)
			yyvs2.keep_head (0)
		end

	yy_push_last_value (yychar1: INTEGER)
			-- Push semantic value associated with token `last_token'
			-- (with internal id `yychar1') on top of corresponding
			-- value stack.
		do
			inspect yytypes2.item (yychar1)
			when 1 then
				yyvsp1 := yyvsp1 + 1
				if yyvsp1 >= yyvsc1 then
					debug ("GEYACC")
						std.error.put_line ("Resize yyvs1")
					end
					yyvsc1 := yyvsc1 + yyInitial_yyvs_size
					yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
				end
				yyspecial_routines1.force (yyvs1, last_detachable_any_value, yyvsp1)
			when 2 then
				yyvsp2 := yyvsp2 + 1
				if yyvsp2 >= yyvsc2 then
					debug ("GEYACC")
						std.error.put_line ("Resize yyvs2")
					end
					yyvsc2 := yyvsc2 + yyInitial_yyvs_size
					yyvs2 := yyspecial_routines2.aliased_resized_area (yyvs2, yyvsc2)
				end
				yyspecial_routines2.force (yyvs2, last_string_value, yyvsp2)
			else
				debug ("GEYACC")
					std.error.put_string ("Error in parser: not a token type: ")
					std.error.put_integer (yytypes2.item (yychar1))
					std.error.put_new_line
				end
				abort
			end
		end

	yy_push_error_value
			-- Push semantic value associated with token 'error'
			-- on top of corresponding value stack.
		local
			yyval1: detachable ANY
		do
			yyvsp1 := yyvsp1 + 1
			if yyvsp1 >= yyvsc1 then
				debug ("GEYACC")
					std.error.put_line ("Resize yyvs1")
				end
				yyvsc1 := yyvsc1 + yyInitial_yyvs_size
				yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
			end
			yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
		end

	yy_pop_last_value (yystate: INTEGER)
			-- Pop semantic value from stack when in state `yystate'.
		local
			yy_type_id: INTEGER
		do
			yy_type_id := yytypes1.item (yystate)
			inspect yy_type_id
			when 1 then
				yyvsp1 := yyvsp1 - 1
			when 2 then
				yyvsp2 := yyvsp2 - 1
			else
				debug ("GEYACC")
					std.error.put_string ("Error in parser: unknown type id: ")
					std.error.put_integer (yy_type_id)
					std.error.put_new_line
				end
				abort
			end
		end

	yy_run_geyacc
			-- You must run geyacc to regenerate this class.
		do
		end

feature {NONE} -- Semantic actions

	yy_do_action (yy_act: INTEGER)
			-- Execute semantic action.
		local
			yy_retried: BOOLEAN
			yyval1: detachable ANY
			yyval2: STRING
		do
			if not yy_retried then
				inspect yy_act
when 1 then
--|#line 368 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 368")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 2 then
--|#line 369 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 369")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp1 := yyvsp1 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 3 then
--|#line 370 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 370")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp1 := yyvsp1 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 4 then
--|#line 373 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 373")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp1 := yyvsp1 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 5 then
--|#line 373 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 373")
end

in_select_query := False 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 0
	yyvsp1 := yyvsp1 + 1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 6 then
--|#line 378 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 378")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 7 then
--|#line 379 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 379")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 8 then
--|#line 380 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 380")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 9 then
--|#line 381 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 381")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 10 then
--|#line 382 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 382")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 11 then
--|#line 383 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 383")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 12 then
--|#line 384 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 384")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 13 then
--|#line 385 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 385")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 14 then
--|#line 386 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 386")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 15 then
--|#line 387 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 387")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 16 then
--|#line 388 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 388")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 17 then
--|#line 389 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 389")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -3
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 18 then
--|#line 390 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 390")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 6
	yyvsp1 := yyvsp1 -2
	yyvsp2 := yyvsp2 -3
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 19 then
--|#line 391 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 391")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 20 then
--|#line 392 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 392")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 21 then
--|#line 393 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 393")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 22 then
--|#line 394 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 394")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 23 then
--|#line 395 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 395")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 24 then
--|#line 396 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 396")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 25 then
--|#line 397 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 397")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 26 then
--|#line 398 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 398")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 27 then
--|#line 399 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 399")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -2
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 28 then
--|#line 400 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 400")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 29 then
--|#line 401 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 401")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 30 then
--|#line 402 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 402")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 31 then
--|#line 403 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 403")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 32 then
--|#line 404 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 404")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 33 then
--|#line 405 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 405")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 34 then
--|#line 406 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 406")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 35 then
--|#line 407 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 407")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 36 then
--|#line 408 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 408")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 37 then
--|#line 411 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 411")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 38 then
--|#line 412 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 412")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 39 then
--|#line 413 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 413")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp1 := yyvsp1 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 40 then
--|#line 414 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 414")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp1 := yyvsp1 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 41 then
--|#line 417 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 417")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 42 then
--|#line 418 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 418")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 43 then
--|#line 419 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 419")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 44 then
--|#line 420 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 420")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 45 then
--|#line 421 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 421")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 46 then
--|#line 422 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 422")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 47 then
--|#line 423 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 423")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 48 then
--|#line 424 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 424")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 49 then
--|#line 425 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 425")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 50 then
--|#line 426 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 426")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 51 then
--|#line 427 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 427")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 52 then
--|#line 428 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 428")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 53 then
--|#line 429 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 429")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 54 then
--|#line 430 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 430")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 55 then
--|#line 431 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 431")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 56 then
--|#line 432 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 432")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 57 then
--|#line 435 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 435")
end

yyval2 := yyvs2.item (yyvsp2) 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines2.force (yyvs2, yyval2, yyvsp2)
end
when 58 then
--|#line 437 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 437")
end

yyval2 := yyvs2.item (yyvsp2) 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines2.force (yyvs2, yyval2, yyvsp2)
end
when 59 then
--|#line 439 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 439")
end

yyval2 := yyvs2.item (yyvsp2) 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines2.force (yyvs2, yyval2, yyvsp2)
end
when 60 then
--|#line 441 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 441")
end

yyval2 := yyvs2.item (yyvsp2) 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines2.force (yyvs2, yyval2, yyvsp2)
end
when 61 then
--|#line 443 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 443")
end

yyval2 := yyvs2.item (yyvsp2) 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines2.force (yyvs2, yyval2, yyvsp2)
end
when 62 then
--|#line 445 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 445")
end

yyval2 := yyvs2.item (yyvsp2) 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines2.force (yyvs2, yyval2, yyvsp2)
end
when 63 then
--|#line 447 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 447")
end

yyval2 := yyvs2.item (yyvsp2) 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines2.force (yyvs2, yyval2, yyvsp2)
end
when 64 then
--|#line 449 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 449")
end

yyval2 := yyvs2.item (yyvsp2) 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines2.force (yyvs2, yyval2, yyvsp2)
end
when 65 then
--|#line 451 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 451")
end

yyval2 := yyvs2.item (yyvsp2) 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines2.force (yyvs2, yyval2, yyvsp2)
end
when 66 then
--|#line 453 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 453")
end

yyval2 := yyvs2.item (yyvsp2) 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines2.force (yyvs2, yyval2, yyvsp2)
end
when 67 then
--|#line 455 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 455")
end

yyval2 := yyvs2.item (yyvsp2) 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines2.force (yyvs2, yyval2, yyvsp2)
end
when 68 then
--|#line 457 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 457")
end

yyval2 := yyvs2.item (yyvsp2) 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines2.force (yyvs2, yyval2, yyvsp2)
end
when 69 then
--|#line 459 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 459")
end

yyval2 := yyvs2.item (yyvsp2) 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines2.force (yyvs2, yyval2, yyvsp2)
end
when 70 then
--|#line 461 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 461")
end

yyval2 := yyvs2.item (yyvsp2) 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines2.force (yyvs2, yyval2, yyvsp2)
end
when 71 then
--|#line 463 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 463")
end

yyval2 := yyvs2.item (yyvsp2) 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines2.force (yyvs2, yyval2, yyvsp2)
end
when 72 then
--|#line 465 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 465")
end

yyval2 := yyvs2.item (yyvsp2) 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines2.force (yyvs2, yyval2, yyvsp2)
end
when 73 then
--|#line 467 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 467")
end

yyval2 := yyvs2.item (yyvsp2) 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines2.force (yyvs2, yyval2, yyvsp2)
end
when 74 then
--|#line 469 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 469")
end

yyval2 := yyvs2.item (yyvsp2) 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines2.force (yyvs2, yyval2, yyvsp2)
end
when 75 then
--|#line 471 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 471")
end

yyval2 := yyvs2.item (yyvsp2) 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines2.force (yyvs2, yyval2, yyvsp2)
end
when 76 then
--|#line 473 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 473")
end

yyval2 := yyvs2.item (yyvsp2) 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines2.force (yyvs2, yyval2, yyvsp2)
end
when 77 then
--|#line 475 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 475")
end

yyval2 := yyvs2.item (yyvsp2) 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines2.force (yyvs2, yyval2, yyvsp2)
end
when 78 then
--|#line 477 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 477")
end

yyval2 := yyvs2.item (yyvsp2) 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines2.force (yyvs2, yyval2, yyvsp2)
end
when 79 then
--|#line 479 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 479")
end

yyval2 := yyvs2.item (yyvsp2) 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines2.force (yyvs2, yyval2, yyvsp2)
end
when 80 then
--|#line 481 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 481")
end

yyval2 := yyvs2.item (yyvsp2) 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines2.force (yyvs2, yyval2, yyvsp2)
end
when 81 then
--|#line 483 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 483")
end

yyval2 := yyvs2.item (yyvsp2) 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines2.force (yyvs2, yyval2, yyvsp2)
end
when 82 then
--|#line 485 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 485")
end

yyval2 := yyvs2.item (yyvsp2) 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines2.force (yyvs2, yyval2, yyvsp2)
end
when 83 then
--|#line 487 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 487")
end

yyval2 := yyvs2.item (yyvsp2) 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines2.force (yyvs2, yyval2, yyvsp2)
end
when 84 then
--|#line 489 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 489")
end

yyval2 := yyvs2.item (yyvsp2) 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines2.force (yyvs2, yyval2, yyvsp2)
end
when 85 then
--|#line 491 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 491")
end

yyval2 := yyvs2.item (yyvsp2) 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines2.force (yyvs2, yyval2, yyvsp2)
end
when 86 then
--|#line 493 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 493")
end

yyval2 := yyvs2.item (yyvsp2) 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines2.force (yyvs2, yyval2, yyvsp2)
end
when 87 then
--|#line 495 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 495")
end

yyval2 := yyvs2.item (yyvsp2) 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines2.force (yyvs2, yyval2, yyvsp2)
end
when 88 then
--|#line 497 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 497")
end

yyval2 := yyvs2.item (yyvsp2) 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines2.force (yyvs2, yyval2, yyvsp2)
end
when 89 then
--|#line 499 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 499")
end

yyval2 := yyvs2.item (yyvsp2) 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines2.force (yyvs2, yyval2, yyvsp2)
end
when 90 then
--|#line 501 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 501")
end

yyval2 := yyvs2.item (yyvsp2) 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines2.force (yyvs2, yyval2, yyvsp2)
end
when 91 then
--|#line 503 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 503")
end

yyval2 := yyvs2.item (yyvsp2) 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines2.force (yyvs2, yyval2, yyvsp2)
end
when 92 then
--|#line 505 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 505")
end

yyval2 := yyvs2.item (yyvsp2) 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines2.force (yyvs2, yyval2, yyvsp2)
end
when 93 then
--|#line 507 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 507")
end

yyval2 := yyvs2.item (yyvsp2) 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines2.force (yyvs2, yyval2, yyvsp2)
end
when 94 then
--|#line 509 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 509")
end

yyval2 := yyvs2.item (yyvsp2) 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines2.force (yyvs2, yyval2, yyvsp2)
end
when 95 then
--|#line 511 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 511")
end

yyval2 := yyvs2.item (yyvsp2) 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines2.force (yyvs2, yyval2, yyvsp2)
end
when 96 then
--|#line 513 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 513")
end

yyval2 := yyvs2.item (yyvsp2) 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines2.force (yyvs2, yyval2, yyvsp2)
end
when 97 then
--|#line 515 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 515")
end

yyval2 := yyvs2.item (yyvsp2) 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines2.force (yyvs2, yyval2, yyvsp2)
end
when 98 then
--|#line 517 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 517")
end

yyval2 := yyvs2.item (yyvsp2) 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines2.force (yyvs2, yyval2, yyvsp2)
end
when 99 then
--|#line 519 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 519")
end

yyval2 := yyvs2.item (yyvsp2) 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines2.force (yyvs2, yyval2, yyvsp2)
end
when 100 then
--|#line 521 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 521")
end

yyval2 := yyvs2.item (yyvsp2) 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines2.force (yyvs2, yyval2, yyvsp2)
end
when 101 then
--|#line 523 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 523")
end

yyval2 := yyvs2.item (yyvsp2) 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines2.force (yyvs2, yyval2, yyvsp2)
end
when 102 then
--|#line 525 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 525")
end

yyval2 := yyvs2.item (yyvsp2) 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines2.force (yyvs2, yyval2, yyvsp2)
end
when 103 then
--|#line 527 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 527")
end

yyval2 := yyvs2.item (yyvsp2) 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines2.force (yyvs2, yyval2, yyvsp2)
end
when 104 then
--|#line 529 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 529")
end

yyval2 := yyvs2.item (yyvsp2) 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines2.force (yyvs2, yyval2, yyvsp2)
end
when 105 then
--|#line 531 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 531")
end

yyval2 := yyvs2.item (yyvsp2) 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines2.force (yyvs2, yyval2, yyvsp2)
end
when 106 then
--|#line 533 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 533")
end

yyval2 := yyvs2.item (yyvsp2) 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines2.force (yyvs2, yyval2, yyvsp2)
end
when 107 then
--|#line 535 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 535")
end

yyval2 := yyvs2.item (yyvsp2) 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines2.force (yyvs2, yyval2, yyvsp2)
end
when 108 then
--|#line 537 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 537")
end

yyval2 := yyvs2.item (yyvsp2) 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines2.force (yyvs2, yyval2, yyvsp2)
end
when 109 then
--|#line 539 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 539")
end

yyval2 := yyvs2.item (yyvsp2) 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines2.force (yyvs2, yyval2, yyvsp2)
end
when 110 then
--|#line 541 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 541")
end

yyval2 := yyvs2.item (yyvsp2) 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines2.force (yyvs2, yyval2, yyvsp2)
end
when 111 then
--|#line 543 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 543")
end

yyval2 := yyvs2.item (yyvsp2) 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines2.force (yyvs2, yyval2, yyvsp2)
end
when 112 then
--|#line 545 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 545")
end

yyval2 := yyvs2.item (yyvsp2) 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines2.force (yyvs2, yyval2, yyvsp2)
end
when 113 then
--|#line 547 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 547")
end

yyval2 := yyvs2.item (yyvsp2) 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines2.force (yyvs2, yyval2, yyvsp2)
end
when 114 then
--|#line 549 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 549")
end

yyval2 := yyvs2.item (yyvsp2) 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines2.force (yyvs2, yyval2, yyvsp2)
end
when 115 then
--|#line 553 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 553")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 7
	yyvsp1 := yyvsp1 -4
	yyvsp2 := yyvsp2 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 116 then
--|#line 556 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 556")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 117 then
--|#line 557 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 557")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp1 := yyvsp1 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 118 then
--|#line 560 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 560")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 119 then
--|#line 561 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 561")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 120 then
--|#line 564 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 564")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp1 := yyvsp1 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 121 then
--|#line 565 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 565")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp1 := yyvsp1 -1
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 122 then
--|#line 568 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 568")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 0
	yyvsp1 := yyvsp1 + 1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 123 then
--|#line 569 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 569")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 124 then
--|#line 570 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 570")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp1 := yyvsp1 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 125 then
--|#line 573 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 573")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp2 := yyvsp2 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 126 then
--|#line 574 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 574")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 127 then
--|#line 577 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 577")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp2 := yyvsp2 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 128 then
--|#line 580 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 580")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 0
	yyvsp1 := yyvsp1 + 1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 129 then
--|#line 581 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 581")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 130 then
--|#line 584 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 584")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 131 then
--|#line 585 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 585")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -2
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 132 then
--|#line 586 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 586")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -2
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 133 then
--|#line 589 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 589")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 4
	yyvsp1 := yyvsp1 -2
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 134 then
--|#line 592 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 592")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 0
	yyvsp1 := yyvsp1 + 1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 135 then
--|#line 593 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 593")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp1 := yyvsp1 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 136 then
--|#line 596 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 596")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp2 := yyvsp2 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 137 then
--|#line 597 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 597")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp2 := yyvsp2 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 138 then
--|#line 598 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 598")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 139 then
--|#line 601 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 601")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 140 then
--|#line 602 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 602")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp1 := yyvsp1 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 141 then
--|#line 605 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 605")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 142 then
--|#line 606 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 606")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 143 then
--|#line 607 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 607")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 144 then
--|#line 608 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 608")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 4
	yyvsp1 := yyvsp1 -2
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 145 then
--|#line 609 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 609")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -2
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 146 then
--|#line 610 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 610")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 147 then
--|#line 611 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 611")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 148 then
--|#line 612 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 612")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 149 then
--|#line 613 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 613")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp2 := yyvsp2 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 150 then
--|#line 614 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 614")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 4
	yyvsp1 := yyvsp1 -2
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 151 then
--|#line 615 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 615")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp2 := yyvsp2 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 152 then
--|#line 616 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 616")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 153 then
--|#line 617 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 617")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 154 then
--|#line 618 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 618")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 155 then
--|#line 619 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 619")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -2
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 156 then
--|#line 622 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 622")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 157 then
--|#line 623 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 623")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 4
	yyvsp1 := yyvsp1 -2
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 158 then
--|#line 626 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 626")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 5
	yyvsp1 := yyvsp1 -4
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 159 then
--|#line 627 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 627")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 7
	yyvsp1 := yyvsp1 -6
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 160 then
--|#line 630 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 630")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -2
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 161 then
--|#line 631 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 631")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 7
	yyvsp1 := yyvsp1 -4
	yyvsp2 := yyvsp2 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 162 then
--|#line 632 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 632")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 7
	yyvsp1 := yyvsp1 -4
	yyvsp2 := yyvsp2 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 163 then
--|#line 633 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 633")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 7
	yyvsp1 := yyvsp1 -4
	yyvsp2 := yyvsp2 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 164 then
--|#line 634 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 634")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 5
	yyvsp1 := yyvsp1 -3
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 165 then
--|#line 635 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 635")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 6
	yyvsp1 := yyvsp1 -4
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 166 then
--|#line 636 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 636")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 4
	yyvsp1 := yyvsp1 -2
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 167 then
--|#line 639 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 639")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 0
	yyvsp1 := yyvsp1 + 1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 168 then
--|#line 640 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 640")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -2
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 169 then
--|#line 643 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 643")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 170 then
--|#line 644 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 644")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp1 := yyvsp1 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 171 then
--|#line 647 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 647")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp1 := yyvsp1 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 172 then
--|#line 648 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 648")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 4
	yyvsp1 := yyvsp1 -3
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 173 then
--|#line 651 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 651")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 174 then
--|#line 652 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 652")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 175 then
--|#line 653 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 653")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 176 then
--|#line 654 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 654")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 177 then
--|#line 657 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 657")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 178 then
--|#line 658 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 658")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp1 := yyvsp1 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 179 then
--|#line 661 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 661")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 0
	yyvsp1 := yyvsp1 + 1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 180 then
--|#line 662 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 662")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 181 then
--|#line 665 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 665")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 11
	yyvsp1 := yyvsp1 -7
	yyvsp2 := yyvsp2 -3
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 182 then
--|#line 668 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 668")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 0
	yyvsp1 := yyvsp1 + 1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 183 then
--|#line 669 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 669")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -2
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 184 then
--|#line 670 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 670")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 4
	yyvsp1 := yyvsp1 -1
	yyvsp2 := yyvsp2 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 185 then
--|#line 673 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 673")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 4
	yyvsp2 := yyvsp2 -3
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 186 then
--|#line 676 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 676")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 0
	yyvsp1 := yyvsp1 + 1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 187 then
--|#line 677 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 677")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 188 then
--|#line 680 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 680")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -3
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 189 then
--|#line 681 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 681")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -3
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 190 then
--|#line 684 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 684")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 0
	yyvsp1 := yyvsp1 + 1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 191 then
--|#line 685 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 685")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 192 then
--|#line 688 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 688")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 193 then
--|#line 689 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 689")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp1 := yyvsp1 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 194 then
--|#line 692 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 692")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 195 then
--|#line 693 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 693")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp1 := yyvsp1 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 196 then
--|#line 696 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 696")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 7
	yyvsp1 := yyvsp1 -2
	yyvsp2 := yyvsp2 -4
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 197 then
--|#line 697 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 697")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 7
	yyvsp1 := yyvsp1 -2
	yyvsp2 := yyvsp2 -4
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 198 then
--|#line 698 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 698")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 7
	yyvsp1 := yyvsp1 -2
	yyvsp2 := yyvsp2 -4
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 199 then
--|#line 701 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 701")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 0
	yyvsp1 := yyvsp1 + 1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 200 then
--|#line 702 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 702")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -2
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 201 then
--|#line 705 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 705")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 5
	yyvsp2 := yyvsp2 -4
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 202 then
--|#line 708 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 708")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 0
	yyvsp1 := yyvsp1 + 1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 203 then
--|#line 709 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 709")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 204 then
--|#line 710 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 710")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 205 then
--|#line 713 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 713")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp2 := yyvsp2 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 206 then
--|#line 714 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 714")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 207 then
--|#line 717 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 717")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 208 then
--|#line 718 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 718")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp1 := yyvsp1 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 209 then
--|#line 721 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 721")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 9
	yyvsp1 := yyvsp1 -4
	yyvsp2 := yyvsp2 -4
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 210 then
--|#line 722 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 722")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 9
	yyvsp1 := yyvsp1 -4
	yyvsp2 := yyvsp2 -4
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 211 then
--|#line 723 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 723")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 9
	yyvsp1 := yyvsp1 -4
	yyvsp2 := yyvsp2 -4
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 212 then
--|#line 726 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 726")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 4
	yyvsp2 := yyvsp2 -3
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 213 then
--|#line 729 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 729")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 214 then
--|#line 730 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 730")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 215 then
--|#line 733 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 733")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 7
	yyvsp1 := yyvsp1 -3
	yyvsp2 := yyvsp2 -3
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 216 then
--|#line 734 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 734")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 12
	yyvsp1 := yyvsp1 -6
	yyvsp2 := yyvsp2 -5
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 217 then
--|#line 737 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 737")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 0
	yyvsp1 := yyvsp1 + 1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 218 then
--|#line 738 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 738")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -3
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 219 then
--|#line 741 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 741")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 0
	yyvsp1 := yyvsp1 + 1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 220 then
--|#line 742 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 742")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp1 := yyvsp1 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 221 then
--|#line 745 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 745")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 0
	yyvsp1 := yyvsp1 + 1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 222 then
--|#line 746 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 746")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp1 := yyvsp1 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 223 then
--|#line 749 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 749")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 5
	yyvsp2 := yyvsp2 -4
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 224 then
--|#line 750 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 750")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 7
	yyvsp1 := yyvsp1 -3
	yyvsp2 := yyvsp2 -3
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 225 then
--|#line 751 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 751")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 7
	yyvsp1 := yyvsp1 -2
	yyvsp2 := yyvsp2 -4
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 226 then
--|#line 752 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 752")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 6
	yyvsp1 := yyvsp1 -1
	yyvsp2 := yyvsp2 -4
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 227 then
--|#line 753 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 753")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 7
	yyvsp1 := yyvsp1 -2
	yyvsp2 := yyvsp2 -4
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 228 then
--|#line 754 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 754")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 5
	yyvsp1 := yyvsp1 -2
	yyvsp2 := yyvsp2 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 229 then
--|#line 757 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 757")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 0
	yyvsp1 := yyvsp1 + 1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 230 then
--|#line 758 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 758")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -3
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 231 then
--|#line 761 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 761")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 6
	yyvsp1 := yyvsp1 -2
	yyvsp2 := yyvsp2 -3
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 232 then
--|#line 762 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 762")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 6
	yyvsp1 := yyvsp1 -1
	yyvsp2 := yyvsp2 -4
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 233 then
--|#line 763 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 763")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 6
	yyvsp1 := yyvsp1 -1
	yyvsp2 := yyvsp2 -4
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 234 then
--|#line 764 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 764")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 6
	yyvsp1 := yyvsp1 -1
	yyvsp2 := yyvsp2 -4
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 235 then
--|#line 765 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 765")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 4
	yyvsp1 := yyvsp1 -1
	yyvsp2 := yyvsp2 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 236 then
--|#line 768 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 768")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 0
	yyvsp1 := yyvsp1 + 1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 237 then
--|#line 769 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 769")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -3
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 238 then
--|#line 772 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 772")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -2
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 239 then
--|#line 773 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 773")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 240 then
--|#line 774 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 774")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 241 then
--|#line 777 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 777")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 242 then
--|#line 778 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 778")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp1 := yyvsp1 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 243 then
--|#line 781 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 781")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp1 := yyvsp1 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 244 then
--|#line 782 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 782")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 245 then
--|#line 783 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 783")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 246 then
--|#line 784 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 784")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 247 then
--|#line 785 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 785")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 248 then
--|#line 788 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 788")
end

in_select_query := True 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 249 then
--|#line 792 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 792")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 250 then
--|#line 793 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 793")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp1 := yyvsp1 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 251 then
--|#line 796 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 796")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 252 then
--|#line 797 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 797")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 253 then
--|#line 800 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 800")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 4
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -4
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 254 then
--|#line 803 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 803")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp2 := yyvsp2 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 255 then
--|#line 804 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 804")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp2 := yyvsp2 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 256 then
--|#line 807 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 807")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp2 := yyvsp2 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 257 then
--|#line 808 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 808")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 4
	yyvsp2 := yyvsp2 -3
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 258 then
--|#line 809 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 809")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp2 := yyvsp2 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 259 then
--|#line 810 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 810")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 4
	yyvsp2 := yyvsp2 -3
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 260 then
--|#line 811 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 811")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp2 := yyvsp2 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 261 then
--|#line 814 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 814")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 5
	yyvsp1 := yyvsp1 -1
	yyvsp2 := yyvsp2 -3
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 262 then
--|#line 817 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 817")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 5
	yyvsp1 := yyvsp1 -1
	yyvsp2 := yyvsp2 -3
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 263 then
--|#line 820 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 820")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 5
	yyvsp1 := yyvsp1 -1
	yyvsp2 := yyvsp2 -3
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 264 then
--|#line 823 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 823")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 0
	yyvsp1 := yyvsp1 + 1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 265 then
--|#line 824 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 824")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 5
	yyvsp1 := yyvsp1 -2
	yyvsp2 := yyvsp2 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 266 then
--|#line 827 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 827")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 10
	yyvsp1 := yyvsp1 -6
	yyvsp2 := yyvsp2 -3
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 267 then
--|#line 830 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 830")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 0
	yyvsp1 := yyvsp1 + 1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 268 then
--|#line 831 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 831")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 269 then
--|#line 834 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 834")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 0
	yyvsp1 := yyvsp1 + 1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 270 then
--|#line 835 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 835")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 4
	yyvsp1 := yyvsp1 -1
	yyvsp2 := yyvsp2 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 271 then
--|#line 838 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 838")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 0
	yyvsp1 := yyvsp1 + 1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 272 then
--|#line 839 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 839")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp1 := yyvsp1 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 273 then
--|#line 842 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 842")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 0
	yyvsp1 := yyvsp1 + 1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 274 then
--|#line 843 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 843")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp2 := yyvsp2 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 275 then
--|#line 846 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 846")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 276 then
--|#line 847 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 847")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 277 then
--|#line 848 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 848")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 278 then
--|#line 851 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 851")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 5
	yyvsp2 := yyvsp2 -4
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 279 then
--|#line 852 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 852")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 6
	yyvsp1 := yyvsp1 -1
	yyvsp2 := yyvsp2 -4
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 280 then
--|#line 855 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 855")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 0
	yyvsp1 := yyvsp1 + 1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 281 then
--|#line 856 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 856")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp2 := yyvsp2 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 282 then
--|#line 859 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 859")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 283 then
--|#line 860 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 860")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp1 := yyvsp1 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 284 then
--|#line 863 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 863")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp1 := yyvsp1 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 285 then
--|#line 864 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 864")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp1 := yyvsp1 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 286 then
--|#line 867 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 867")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 0
	yyvsp1 := yyvsp1 + 1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 287 then
--|#line 868 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 868")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 288 then
--|#line 869 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 869")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 289 then
--|#line 872 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 872")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 5
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -5
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 290 then
--|#line 875 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 875")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 5
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -5
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 291 then
--|#line 878 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 878")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 5
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -5
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 292 then
--|#line 881 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 881")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 0
	yyvsp1 := yyvsp1 + 1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 293 then
--|#line 882 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 882")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 294 then
--|#line 885 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 885")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 0
	yyvsp1 := yyvsp1 + 1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 295 then
--|#line 886 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 886")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 296 then
--|#line 889 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 889")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 7
	yyvsp1 := yyvsp1 -1
	yyvsp2 := yyvsp2 -5
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 297 then
--|#line 890 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 890")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 6
	yyvsp2 := yyvsp2 -5
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 298 then
--|#line 893 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 893")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 0
	yyvsp1 := yyvsp1 + 1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 299 then
--|#line 894 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 894")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -2
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 300 then
--|#line 897 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 897")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 4
	yyvsp2 := yyvsp2 -3
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 301 then
--|#line 900 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 900")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 0
	yyvsp1 := yyvsp1 + 1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 302 then
--|#line 901 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 901")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -2
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 303 then
--|#line 904 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 904")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 4
	yyvsp2 := yyvsp2 -3
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 304 then
--|#line 907 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 907")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 15
	yyvsp1 := yyvsp1 -9
	yyvsp2 := yyvsp2 -5
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 305 then
--|#line 910 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 910")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 0
	yyvsp1 := yyvsp1 + 1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 306 then
--|#line 911 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 911")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -2
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 307 then
--|#line 914 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 914")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 0
	yyvsp1 := yyvsp1 + 1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 308 then
--|#line 915 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 915")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp1 := yyvsp1 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 309 then
--|#line 918 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 918")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 6
	yyvsp2 := yyvsp2 -5
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 310 then
--|#line 921 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 921")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 6
	yyvsp2 := yyvsp2 -5
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 311 then
--|#line 924 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 924")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 0
	yyvsp1 := yyvsp1 + 1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 312 then
--|#line 925 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 925")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 313 then
--|#line 928 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 928")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 0
	yyvsp1 := yyvsp1 + 1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 314 then
--|#line 929 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 929")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp2 := yyvsp2 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 315 then
--|#line 932 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 932")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 0
	yyvsp1 := yyvsp1 + 1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 316 then
--|#line 933 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 933")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 5
	yyvsp1 := yyvsp1 -2
	yyvsp2 := yyvsp2 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 317 then
--|#line 936 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 936")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 0
	yyvsp1 := yyvsp1 + 1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 318 then
--|#line 937 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 937")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 319 then
--|#line 940 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 940")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 0
	yyvsp1 := yyvsp1 + 1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 320 then
--|#line 941 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 941")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 321 then
--|#line 944 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 944")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 0
	yyvsp1 := yyvsp1 + 1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 322 then
--|#line 945 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 945")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 323 then
--|#line 948 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 948")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 324 then
--|#line 949 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 949")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 325 then
--|#line 950 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 950")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 326 then
--|#line 951 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 951")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 327 then
--|#line 952 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 952")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 328 then
--|#line 953 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 953")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 329 then
--|#line 954 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 954")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 330 then
--|#line 955 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 955")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 331 then
--|#line 956 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 956")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 332 then
--|#line 957 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 957")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 333 then
--|#line 958 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 958")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp2 := yyvsp2 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 334 then
--|#line 959 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 959")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 335 then
--|#line 960 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 960")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 336 then
--|#line 961 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 961")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 337 then
--|#line 962 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 962")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 338 then
--|#line 963 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 963")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 339 then
--|#line 964 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 964")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 340 then
--|#line 965 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 965")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 341 then
--|#line 966 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 966")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 342 then
--|#line 967 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 967")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 343 then
--|#line 968 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 968")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 344 then
--|#line 969 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 969")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 345 then
--|#line 970 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 970")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 346 then
--|#line 971 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 971")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 347 then
--|#line 972 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 972")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 348 then
--|#line 973 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 973")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 349 then
--|#line 974 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 974")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 350 then
--|#line 975 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 975")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 351 then
--|#line 978 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 978")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -2
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 352 then
--|#line 981 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 981")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 353 then
--|#line 984 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 984")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 354 then
--|#line 987 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 987")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 355 then
--|#line 988 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 988")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -2
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 356 then
--|#line 991 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 991")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 9
	yyvsp1 := yyvsp1 -3
	yyvsp2 := yyvsp2 -5
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 357 then
--|#line 994 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 994")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 7
	yyvsp1 := yyvsp1 -4
	yyvsp2 := yyvsp2 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 358 then
--|#line 995 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 995")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 6
	yyvsp1 := yyvsp1 -4
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 359 then
--|#line 998 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 998")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 0
	yyvsp1 := yyvsp1 + 1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 360 then
--|#line 999 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 999")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 4
	yyvsp1 := yyvsp1 -1
	yyvsp2 := yyvsp2 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 361 then
--|#line 1002 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1002")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 4
	yyvsp1 := yyvsp1 -1
	yyvsp2 := yyvsp2 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 362 then
--|#line 1003 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1003")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 6
	yyvsp1 := yyvsp1 -1
	yyvsp2 := yyvsp2 -4
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 363 then
--|#line 1004 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1004")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 5
	yyvsp1 := yyvsp1 -1
	yyvsp2 := yyvsp2 -3
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 364 then
--|#line 1005 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1005")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 6
	yyvsp1 := yyvsp1 -2
	yyvsp2 := yyvsp2 -3
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 365 then
--|#line 1008 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1008")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp2 := yyvsp2 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 366 then
--|#line 1011 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1011")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -2
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 367 then
--|#line 1014 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1014")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp2 := yyvsp2 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 368 then
--|#line 1017 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1017")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 0
	yyvsp1 := yyvsp1 + 1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 369 then
--|#line 1018 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1018")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 370 then
--|#line 1019 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1019")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 371 then
--|#line 1020 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1020")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 372 then
--|#line 1023 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1023")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 6
	yyvsp1 := yyvsp1 -4
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 373 then
--|#line 1026 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1026")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 4
	yyvsp1 := yyvsp1 -2
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 374 then
--|#line 1027 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1027")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 375 then
--|#line 1030 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1030")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 376 then
--|#line 1031 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1031")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp1 := yyvsp1 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 377 then
--|#line 1034 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1034")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 378 then
--|#line 1037 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1037")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 379 then
--|#line 1038 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1038")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 380 then
--|#line 1039 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1039")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 381 then
--|#line 1040 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1040")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 382 then
--|#line 1041 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1041")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -2
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 383 then
--|#line 1042 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1042")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -3
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 384 then
--|#line 1043 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1043")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -3
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 385 then
--|#line 1044 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1044")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 386 then
--|#line 1047 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1047")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 387 then
--|#line 1048 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1048")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp1 := yyvsp1 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 388 then
--|#line 1049 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1049")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp1 := yyvsp1 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 389 then
--|#line 1052 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1052")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 0
	yyvsp1 := yyvsp1 + 1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 390 then
--|#line 1053 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1053")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 4
	yyvsp1 := yyvsp1 -2
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 391 then
--|#line 1056 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1056")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 0
	yyvsp1 := yyvsp1 + 1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 392 then
--|#line 1057 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1057")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 5
	yyvsp1 := yyvsp1 -2
	yyvsp2 := yyvsp2 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 393 then
--|#line 1060 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1060")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 394 then
--|#line 1061 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1061")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 395 then
--|#line 1064 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1064")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 396 then
--|#line 1065 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1065")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp1 := yyvsp1 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 397 then
--|#line 1068 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1068")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 0
	yyvsp1 := yyvsp1 + 1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 398 then
--|#line 1069 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1069")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp1 := yyvsp1 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 399 then
--|#line 1072 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1072")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp1 := yyvsp1 -1
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 400 then
--|#line 1075 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1075")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 401 then
--|#line 1076 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1076")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -2
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 402 then
--|#line 1079 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1079")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 0
	yyvsp1 := yyvsp1 + 1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 403 then
--|#line 1080 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1080")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 404 then
--|#line 1083 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1083")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 7
	yyvsp1 := yyvsp1 -5
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 405 then
--|#line 1086 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1086")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 0
	yyvsp1 := yyvsp1 + 1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 406 then
--|#line 1087 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1087")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 407 then
--|#line 1088 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1088")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 408 then
--|#line 1091 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1091")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 0
	yyvsp1 := yyvsp1 + 1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 409 then
--|#line 1092 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1092")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -2
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 410 then
--|#line 1095 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1095")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 411 then
--|#line 1096 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1096")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 5
	yyvsp1 := yyvsp1 -3
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 412 then
--|#line 1097 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1097")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 6
	yyvsp1 := yyvsp1 -4
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 413 then
--|#line 1098 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1098")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 6
	yyvsp1 := yyvsp1 -4
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 414 then
--|#line 1099 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1099")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 8
	yyvsp1 := yyvsp1 -6
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 415 then
--|#line 1102 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1102")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 0
	yyvsp1 := yyvsp1 + 1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 416 then
--|#line 1103 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1103")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 417 then
--|#line 1106 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1106")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 9
	yyvsp1 := yyvsp1 -3
	yyvsp2 := yyvsp2 -5
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 418 then
--|#line 1109 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1109")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 0
	yyvsp1 := yyvsp1 + 1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 419 then
--|#line 1110 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1110")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 420 then
--|#line 1111 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1111")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp1 := yyvsp1 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 421 then
--|#line 1114 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1114")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp1 := yyvsp1 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 422 then
--|#line 1117 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1117")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 7
	yyvsp1 := yyvsp1 -4
	yyvsp2 := yyvsp2 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 423 then
--|#line 1120 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1120")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 0
	yyvsp1 := yyvsp1 + 1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 424 then
--|#line 1121 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1121")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp1 := yyvsp1 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 425 then
--|#line 1121 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1121")
end

in_select_query := True 
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 0
	yyvsp1 := yyvsp1 + 1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 426 then
--|#line 1125 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1125")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 427 then
--|#line 1126 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1126")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp1 := yyvsp1 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 428 then
--|#line 1129 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1129")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 429 then
--|#line 1130 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1130")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 430 then
--|#line 1131 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1131")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 431 then
--|#line 1134 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1134")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 0
	yyvsp1 := yyvsp1 + 1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 432 then
--|#line 1135 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1135")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 433 then
--|#line 1138 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1138")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 0
	yyvsp1 := yyvsp1 + 1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 434 then
--|#line 1139 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1139")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 435 then
--|#line 1142 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1142")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 436 then
--|#line 1143 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1143")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 5
	yyvsp1 := yyvsp1 -3
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 437 then
--|#line 1144 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1144")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 6
	yyvsp1 := yyvsp1 -3
	yyvsp2 := yyvsp2 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 438 then
--|#line 1145 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1145")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 4
	yyvsp1 := yyvsp1 -2
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 439 then
--|#line 1146 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1146")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 5
	yyvsp1 := yyvsp1 -2
	yyvsp2 := yyvsp2 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 440 then
--|#line 1147 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1147")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 4
	yyvsp1 := yyvsp1 -2
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 441 then
--|#line 1148 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1148")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 5
	yyvsp1 := yyvsp1 -2
	yyvsp2 := yyvsp2 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 442 then
--|#line 1151 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1151")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 443 then
--|#line 1152 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1152")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 5
	yyvsp1 := yyvsp1 -3
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 444 then
--|#line 1153 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1153")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 6
	yyvsp1 := yyvsp1 -3
	yyvsp2 := yyvsp2 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 445 then
--|#line 1154 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1154")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 4
	yyvsp1 := yyvsp1 -2
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 446 then
--|#line 1155 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1155")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 5
	yyvsp1 := yyvsp1 -2
	yyvsp2 := yyvsp2 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 447 then
--|#line 1156 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1156")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 4
	yyvsp1 := yyvsp1 -2
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 448 then
--|#line 1157 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1157")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 5
	yyvsp1 := yyvsp1 -2
	yyvsp2 := yyvsp2 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 449 then
--|#line 1160 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1160")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 450 then
--|#line 1161 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1161")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 5
	yyvsp1 := yyvsp1 -3
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 451 then
--|#line 1162 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1162")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 6
	yyvsp1 := yyvsp1 -3
	yyvsp2 := yyvsp2 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 452 then
--|#line 1163 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1163")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 4
	yyvsp1 := yyvsp1 -2
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 453 then
--|#line 1164 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1164")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 5
	yyvsp1 := yyvsp1 -2
	yyvsp2 := yyvsp2 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 454 then
--|#line 1165 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1165")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 4
	yyvsp1 := yyvsp1 -2
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 455 then
--|#line 1166 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1166")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 5
	yyvsp1 := yyvsp1 -2
	yyvsp2 := yyvsp2 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 456 then
--|#line 1169 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1169")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 457 then
--|#line 1170 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1170")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 458 then
--|#line 1171 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1171")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 459 then
--|#line 1174 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1174")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 460 then
--|#line 1175 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1175")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 4
	yyvsp1 := yyvsp1 -3
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 461 then
--|#line 1176 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1176")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 462 then
--|#line 1179 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1179")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 463 then
--|#line 1180 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1180")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 464 then
--|#line 1181 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1181")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 4
	yyvsp1 := yyvsp1 -3
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 465 then
--|#line 1182 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1182")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 466 then
--|#line 1185 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1185")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 0
	yyvsp1 := yyvsp1 + 1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 467 then
--|#line 1186 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1186")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 5
	yyvsp1 := yyvsp1 -2
	yyvsp2 := yyvsp2 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 468 then
--|#line 1189 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1189")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 4
	yyvsp1 := yyvsp1 -3
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 469 then
--|#line 1192 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1192")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 4
	yyvsp1 := yyvsp1 -3
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 470 then
--|#line 1195 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1195")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp1 := yyvsp1 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 471 then
--|#line 1198 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1198")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 472 then
--|#line 1201 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1201")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 4
	yyvsp1 := yyvsp1 -3
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 473 then
--|#line 1204 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1204")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 7
	yyvsp1 := yyvsp1 -6
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 474 then
--|#line 1207 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1207")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 0
	yyvsp1 := yyvsp1 + 1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 475 then
--|#line 1208 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1208")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 476 then
--|#line 1211 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1211")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 4
	yyvsp1 := yyvsp1 -2
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 477 then
--|#line 1212 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1212")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -2
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 478 then
--|#line 1215 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1215")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 479 then
--|#line 1216 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1216")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp1 := yyvsp1 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 480 then
--|#line 1219 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1219")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 481 then
--|#line 1220 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1220")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 482 then
--|#line 1221 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1221")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 483 then
--|#line 1222 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1222")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 4
	yyvsp1 := yyvsp1 -1
	yyvsp2 := yyvsp2 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 484 then
--|#line 1225 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1225")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 485 then
--|#line 1228 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1228")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 486 then
--|#line 1229 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1229")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp1 := yyvsp1 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 487 then
--|#line 1232 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1232")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 488 then
--|#line 1233 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1233")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp1 := yyvsp1 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 489 then
--|#line 1236 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1236")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp1 := yyvsp1 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 490 then
--|#line 1239 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1239")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 0
	yyvsp1 := yyvsp1 + 1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 491 then
--|#line 1240 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1240")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 492 then
--|#line 1243 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1243")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 493 then
--|#line 1244 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1244")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 4
	yyvsp1 := yyvsp1 -2
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 494 then
--|#line 1245 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1245")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 5
	yyvsp1 := yyvsp1 -2
	yyvsp2 := yyvsp2 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 495 then
--|#line 1246 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1246")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 496 then
--|#line 1247 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1247")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 6
	yyvsp1 := yyvsp1 -3
	yyvsp2 := yyvsp2 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 497 then
--|#line 1250 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1250")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 498 then
--|#line 1251 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1251")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 499 then
--|#line 1252 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1252")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp2 := yyvsp2 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 500 then
--|#line 1253 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1253")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp1 := yyvsp1 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 501 then
--|#line 1256 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1256")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 0
	yyvsp1 := yyvsp1 + 1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 502 then
--|#line 1257 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1257")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 503 then
--|#line 1258 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1258")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 504 then
--|#line 1259 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1259")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 505 then
--|#line 1260 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1260")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 506 then
--|#line 1261 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1261")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -2
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 507 then
--|#line 1262 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1262")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 508 then
--|#line 1265 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1265")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 0
	yyvsp1 := yyvsp1 + 1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 509 then
--|#line 1266 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1266")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 510 then
--|#line 1269 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1269")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 511 then
--|#line 1270 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1270")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 512 then
--|#line 1273 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1273")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 513 then
--|#line 1274 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1274")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp2 := yyvsp2 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 514 then
--|#line 1275 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1275")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp1 := yyvsp1 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 515 then
--|#line 1278 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1278")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 5
	yyvsp1 := yyvsp1 -3
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 516 then
--|#line 1281 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1281")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 0
	yyvsp1 := yyvsp1 + 1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 517 then
--|#line 1282 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1282")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 518 then
--|#line 1283 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1283")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 4
	yyvsp1 := yyvsp1 -2
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 519 then
--|#line 1286 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1286")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 520 then
--|#line 1289 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1289")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 0
	yyvsp1 := yyvsp1 + 1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 521 then
--|#line 1290 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1290")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp2 := yyvsp2 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 522 then
--|#line 1291 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1291")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 6
	yyvsp1 := yyvsp1 -2
	yyvsp2 := yyvsp2 -3
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 523 then
--|#line 1292 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1292")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 6
	yyvsp1 := yyvsp1 -2
	yyvsp2 := yyvsp2 -3
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 524 then
--|#line 1295 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1295")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 0
	yyvsp1 := yyvsp1 + 1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 525 then
--|#line 1296 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1296")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 526 then
--|#line 1299 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1299")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 0
	yyvsp1 := yyvsp1 + 1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 527 then
--|#line 1300 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1300")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -2
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 528 then
--|#line 1301 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1301")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -3
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 529 then
--|#line 1302 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1302")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 4
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -4
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 530 then
--|#line 1305 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1305")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp1 := yyvsp1 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 531 then
--|#line 1306 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1306")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp1 := yyvsp1 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 532 then
--|#line 1307 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1307")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp1 := yyvsp1 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 533 then
--|#line 1308 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1308")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp1 := yyvsp1 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 534 then
--|#line 1309 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1309")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 535 then
--|#line 1312 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1312")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 536 then
--|#line 1313 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1313")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 537 then
--|#line 1314 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1314")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 538 then
--|#line 1315 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1315")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 539 then
--|#line 1316 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1316")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 540 then
--|#line 1317 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1317")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 541 then
--|#line 1318 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1318")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 542 then
--|#line 1319 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1319")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 543 then
--|#line 1322 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1322")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 544 then
--|#line 1325 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1325")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp1 := yyvsp1 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 545 then
--|#line 1328 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1328")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 546 then
--|#line 1329 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1329")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 547 then
--|#line 1332 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1332")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 6
	yyvsp1 := yyvsp1 -4
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 548 then
--|#line 1333 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1333")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 5
	yyvsp1 := yyvsp1 -3
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 549 then
--|#line 1336 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1336")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 5
	yyvsp1 := yyvsp1 -3
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 550 then
--|#line 1337 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1337")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 4
	yyvsp1 := yyvsp1 -2
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 551 then
--|#line 1340 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1340")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 0
	yyvsp1 := yyvsp1 + 1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 552 then
--|#line 1341 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1341")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 553 then
--|#line 1342 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1342")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 4
	yyvsp2 := yyvsp2 -3
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 554 then
--|#line 1345 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1345")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 4
	yyvsp1 := yyvsp1 -1
	yyvsp2 := yyvsp2 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 555 then
--|#line 1346 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1346")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp2 := yyvsp2 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 556 then
--|#line 1349 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1349")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 4
	yyvsp1 := yyvsp1 -2
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 557 then
--|#line 1350 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1350")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp1 := yyvsp1 -1
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 558 then
--|#line 1351 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1351")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 6
	yyvsp1 := yyvsp1 -4
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 559 then
--|#line 1352 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1352")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 5
	yyvsp1 := yyvsp1 -3
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 560 then
--|#line 1355 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1355")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 4
	yyvsp1 := yyvsp1 -3
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 561 then
--|#line 1358 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1358")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 562 then
--|#line 1359 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1359")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 563 then
--|#line 1360 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1360")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 564 then
--|#line 1363 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1363")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 565 then
--|#line 1366 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1366")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 566 then
--|#line 1369 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1369")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 6
	yyvsp1 := yyvsp1 -5
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 567 then
--|#line 1372 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1372")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp1 := yyvsp1 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 568 then
--|#line 1373 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1373")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp1 := yyvsp1 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 569 then
--|#line 1374 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1374")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp1 := yyvsp1 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 570 then
--|#line 1375 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1375")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp1 := yyvsp1 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 571 then
--|#line 1376 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1376")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp1 := yyvsp1 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 572 then
--|#line 1377 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1377")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp1 := yyvsp1 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 573 then
--|#line 1378 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1378")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp1 := yyvsp1 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 574 then
--|#line 1379 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1379")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp1 := yyvsp1 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 575 then
--|#line 1380 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1380")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp1 := yyvsp1 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 576 then
--|#line 1381 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1381")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp1 := yyvsp1 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 577 then
--|#line 1382 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1382")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp1 := yyvsp1 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 578 then
--|#line 1383 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1383")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 6
	yyvsp1 := yyvsp1 -5
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 579 then
--|#line 1384 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1384")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 580 then
--|#line 1385 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1385")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 581 then
--|#line 1386 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1386")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 582 then
--|#line 1387 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1387")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 583 then
--|#line 1390 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1390")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp1 := yyvsp1 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 584 then
--|#line 1391 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1391")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp1 := yyvsp1 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 585 then
--|#line 1392 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1392")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp1 := yyvsp1 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 586 then
--|#line 1393 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1393")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp1 := yyvsp1 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 587 then
--|#line 1394 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1394")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp1 := yyvsp1 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 588 then
--|#line 1395 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1395")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp1 := yyvsp1 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 589 then
--|#line 1396 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1396")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp1 := yyvsp1 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 590 then
--|#line 1397 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1397")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp1 := yyvsp1 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 591 then
--|#line 1398 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1398")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp1 := yyvsp1 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 592 then
--|#line 1399 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1399")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 593 then
--|#line 1400 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1400")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 594 then
--|#line 1401 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1401")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 595 then
--|#line 1402 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1402")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 596 then
--|#line 1405 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1405")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 597 then
--|#line 1406 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1406")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 598 then
--|#line 1407 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1407")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 599 then
--|#line 1408 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1408")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp1 := yyvsp1 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 600 then
--|#line 1409 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1409")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 5
	yyvsp1 := yyvsp1 -4
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 601 then
--|#line 1410 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1410")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 602 then
--|#line 1411 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1411")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 603 then
--|#line 1412 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1412")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 604 then
--|#line 1413 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1413")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 605 then
--|#line 1414 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1414")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 606 then
--|#line 1415 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1415")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 607 then
--|#line 1416 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1416")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 608 then
--|#line 1417 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1417")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 609 then
--|#line 1418 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1418")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 610 then
--|#line 1419 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1419")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 611 then
--|#line 1420 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1420")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 612 then
--|#line 1421 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1421")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 613 then
--|#line 1424 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1424")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 614 then
--|#line 1425 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1425")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 615 then
--|#line 1426 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1426")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 616 then
--|#line 1427 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1427")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp1 := yyvsp1 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 617 then
--|#line 1428 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1428")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 5
	yyvsp1 := yyvsp1 -4
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 618 then
--|#line 1429 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1429")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 619 then
--|#line 1430 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1430")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 620 then
--|#line 1431 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1431")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 621 then
--|#line 1432 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1432")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 622 then
--|#line 1433 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1433")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 623 then
--|#line 1434 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1434")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 624 then
--|#line 1435 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1435")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 625 then
--|#line 1436 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1436")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 626 then
--|#line 1437 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1437")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 627 then
--|#line 1438 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1438")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 628 then
--|#line 1439 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1439")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 629 then
--|#line 1440 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1440")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 630 then
--|#line 1443 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1443")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 6
	yyvsp1 := yyvsp1 -4
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 631 then
--|#line 1444 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1444")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 8
	yyvsp1 := yyvsp1 -6
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 632 then
--|#line 1447 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1447")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 0
	yyvsp1 := yyvsp1 + 1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 633 then
--|#line 1448 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1448")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -2
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 634 then
--|#line 1451 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1451")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 7
	yyvsp1 := yyvsp1 -4
	yyvsp2 := yyvsp2 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 635 then
--|#line 1454 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1454")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -3
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 636 then
--|#line 1455 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1455")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -3
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 637 then
--|#line 1456 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1456")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 4
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -4
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 638 then
--|#line 1459 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1459")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 4
	yyvsp1 := yyvsp1 -1
	yyvsp2 := yyvsp2 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 639 then
--|#line 1460 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1460")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp2 := yyvsp2 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 640 then
--|#line 1461 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1461")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp2 := yyvsp2 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 641 then
--|#line 1462 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1462")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp2 := yyvsp2 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 642 then
--|#line 1463 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1463")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 643 then
--|#line 1464 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1464")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp1 := yyvsp1 -1
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 644 then
--|#line 1467 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1467")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 4
	yyvsp1 := yyvsp1 -3
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 645 then
--|#line 1468 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1468")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 646 then
--|#line 1471 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1471")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 4
	yyvsp1 := yyvsp1 -3
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 647 then
--|#line 1474 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1474")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 0
	yyvsp1 := yyvsp1 + 1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 648 then
--|#line 1475 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1475")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 649 then
--|#line 1478 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1478")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 0
	yyvsp1 := yyvsp1 + 1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 650 then
--|#line 1479 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1479")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 651 then
--|#line 1482 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1482")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 652 then
--|#line 1483 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1483")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp1 := yyvsp1 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 653 then
--|#line 1486 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1486")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 654 then
--|#line 1487 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1487")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp1 := yyvsp1 -1
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 655 then
--|#line 1488 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1488")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 5
	yyvsp1 := yyvsp1 -2
	yyvsp2 := yyvsp2 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 656 then
--|#line 1489 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1489")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 7
	yyvsp1 := yyvsp1 -3
	yyvsp2 := yyvsp2 -3
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 657 then
--|#line 1490 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1490")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 4
	yyvsp1 := yyvsp1 -2
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 658 then
--|#line 1491 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1491")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 6
	yyvsp1 := yyvsp1 -3
	yyvsp2 := yyvsp2 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 659 then
--|#line 1492 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1492")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 660 then
--|#line 1493 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1493")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 661 then
--|#line 1494 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1494")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 662 then
--|#line 1497 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1497")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp1 := yyvsp1 -1
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 663 then
--|#line 1498 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1498")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 5
	yyvsp1 := yyvsp1 -3
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 664 then
--|#line 1501 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1501")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 665 then
--|#line 1502 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1502")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp1 := yyvsp1 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 666 then
--|#line 1503 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1503")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp1 := yyvsp1 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 667 then
--|#line 1506 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1506")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 0
	yyvsp1 := yyvsp1 + 1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 668 then
--|#line 1507 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1507")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 669 then
--|#line 1508 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1508")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 670 then
--|#line 1509 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1509")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp1 := yyvsp1 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 671 then
--|#line 1510 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1510")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp1 := yyvsp1 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 672 then
--|#line 1511 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1511")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 673 then
--|#line 1514 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1514")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 4
	yyvsp1 := yyvsp1 -3
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 674 then
--|#line 1515 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1515")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 8
	yyvsp1 := yyvsp1 -5
	yyvsp2 := yyvsp2 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 675 then
--|#line 1516 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1516")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 6
	yyvsp1 := yyvsp1 -2
	yyvsp2 := yyvsp2 -3
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 676 then
--|#line 1517 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1517")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 6
	yyvsp1 := yyvsp1 -2
	yyvsp2 := yyvsp2 -3
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 677 then
--|#line 1518 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1518")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 6
	yyvsp1 := yyvsp1 -2
	yyvsp2 := yyvsp2 -3
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 678 then
--|#line 1519 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1519")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 6
	yyvsp1 := yyvsp1 -2
	yyvsp2 := yyvsp2 -3
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 679 then
--|#line 1520 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1520")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 6
	yyvsp1 := yyvsp1 -2
	yyvsp2 := yyvsp2 -3
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 680 then
--|#line 1521 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1521")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 8
	yyvsp1 := yyvsp1 -3
	yyvsp2 := yyvsp2 -4
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 681 then
--|#line 1522 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1522")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 6
	yyvsp1 := yyvsp1 -3
	yyvsp2 := yyvsp2 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 682 then
--|#line 1523 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1523")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp2 := yyvsp2 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 683 then
--|#line 1524 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1524")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 6
	yyvsp1 := yyvsp1 -2
	yyvsp2 := yyvsp2 -3
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 684 then
--|#line 1525 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1525")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 6
	yyvsp1 := yyvsp1 -2
	yyvsp2 := yyvsp2 -3
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 685 then
--|#line 1526 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1526")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 10
	yyvsp1 := yyvsp1 -5
	yyvsp2 := yyvsp2 -4
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 686 then
--|#line 1527 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1527")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 8
	yyvsp1 := yyvsp1 -3
	yyvsp2 := yyvsp2 -4
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 687 then
--|#line 1528 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1528")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 8
	yyvsp1 := yyvsp1 -2
	yyvsp2 := yyvsp2 -5
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 688 then
--|#line 1529 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1529")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 7
	yyvsp1 := yyvsp1 -5
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 689 then
--|#line 1530 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1530")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 690 then
--|#line 1531 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1531")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 691 then
--|#line 1532 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1532")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 4
	yyvsp1 := yyvsp1 -2
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 692 then
--|#line 1533 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1533")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 693 then
--|#line 1534 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1534")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 4
	yyvsp1 := yyvsp1 -2
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 694 then
--|#line 1535 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1535")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 4
	yyvsp1 := yyvsp1 -2
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 695 then
--|#line 1538 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1538")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 4
	yyvsp2 := yyvsp2 -3
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 696 then
--|#line 1539 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1539")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp2 := yyvsp2 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 697 then
--|#line 1542 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1542")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 698 then
--|#line 1543 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1543")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp1 := yyvsp1 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 699 then
--|#line 1546 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1546")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 700 then
--|#line 1547 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1547")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 701 then
--|#line 1548 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1548")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp1 := yyvsp1 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 702 then
--|#line 1549 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1549")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp1 := yyvsp1 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 703 then
--|#line 1552 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1552")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 704 then
--|#line 1553 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1553")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 705 then
--|#line 1554 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1554")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 706 then
--|#line 1557 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1557")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 707 then
--|#line 1558 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1558")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 708 then
--|#line 1561 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1561")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 4
	yyvsp1 := yyvsp1 -1
	yyvsp2 := yyvsp2 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 709 then
--|#line 1564 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1564")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp2 := yyvsp2 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 710 then
--|#line 1567 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1567")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 711 then
--|#line 1568 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1568")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp1 := yyvsp1 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 712 then
--|#line 1571 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1571")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 713 then
--|#line 1572 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1572")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp1 := yyvsp1 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 714 then
--|#line 1575 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1575")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 4
	yyvsp1 := yyvsp1 -1
	yyvsp2 := yyvsp2 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 715 then
--|#line 1576 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1576")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 716 then
--|#line 1579 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1579")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 4
	yyvsp1 := yyvsp1 -1
	yyvsp2 := yyvsp2 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 717 then
--|#line 1580 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1580")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 718 then
--|#line 1583 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1583")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 4
	yyvsp1 := yyvsp1 -2
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 719 then
--|#line 1586 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1586")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 6
	yyvsp1 := yyvsp1 -4
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 720 then
--|#line 1589 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1589")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 721 then
--|#line 1590 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1590")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp1 := yyvsp1 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 722 then
--|#line 1591 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1591")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp1 := yyvsp1 -1
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 723 then
--|#line 1594 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1594")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 5
	yyvsp1 := yyvsp1 -3
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 724 then
--|#line 1595 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1595")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 5
	yyvsp1 := yyvsp1 -2
	yyvsp2 := yyvsp2 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 725 then
--|#line 1596 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1596")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 5
	yyvsp1 := yyvsp1 -2
	yyvsp2 := yyvsp2 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 726 then
--|#line 1597 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1597")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 4
	yyvsp1 := yyvsp1 -2
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 727 then
--|#line 1600 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1600")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 728 then
--|#line 1601 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1601")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 729 then
--|#line 1602 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1602")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 730 then
--|#line 1603 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1603")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 731 then
--|#line 1604 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1604")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 732 then
--|#line 1605 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1605")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 733 then
--|#line 1606 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1606")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 734 then
--|#line 1607 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1607")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 735 then
--|#line 1610 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1610")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 736 then
--|#line 1611 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1611")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 737 then
--|#line 1612 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1612")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 738 then
--|#line 1613 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1613")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 739 then
--|#line 1614 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1614")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp1 := yyvsp1 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 740 then
--|#line 1615 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1615")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp1 := yyvsp1 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 741 then
--|#line 1616 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1616")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 742 then
--|#line 1617 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1617")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp1 := yyvsp1 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 743 then
--|#line 1618 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1618")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp1 := yyvsp1 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 744 then
--|#line 1619 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1619")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 745 then
--|#line 1620 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1620")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 746 then
--|#line 1621 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1621")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 747 then
--|#line 1624 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1624")
end

			yyval2 := yyvs2.item (yyvsp2)
			if in_select_query then
				input_table_names.force (yyval2.as_lower)
			else
				output_table_names.force (yyval2.as_lower)
			end
		
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines2.force (yyvs2, yyval2, yyvsp2)
end
when 748 then
--|#line 1633 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1633")
end

			yyval2 := yyvs2.item (yyvsp2 - 1) + "." + yyvs2.item (yyvsp2)
			if in_select_query then
				input_table_names.force (yyval2.as_lower)
			else
				output_table_names.force (yyval2.as_lower)
			end
		
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp2 := yyvsp2 -1
	yyvsp1 := yyvsp1 -1
	yyspecial_routines2.force (yyvs2, yyval2, yyvsp2)
end
when 749 then
--|#line 1642 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1642")
end

			yyval2 := yyvs2.item (yyvsp2 - 2) + "." + yyvs2.item (yyvsp2 - 1) + "." + yyvs2.item (yyvsp2)
			if in_select_query then
				input_table_names.force (yyval2.as_lower)
			else
				output_table_names.force (yyval2.as_lower)
			end
		
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 5
	yyvsp2 := yyvsp2 -2
	yyvsp1 := yyvsp1 -2
	yyspecial_routines2.force (yyvs2, yyval2, yyvsp2)
end
when 750 then
--|#line 1651 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1651")
end

			yyval2 := yyvs2.item (yyvsp2 - 1) + ".." + yyvs2.item (yyvsp2)
			if in_select_query then
				input_table_names.force (yyval2.as_lower)
			else
				output_table_names.force (yyval2.as_lower)
			end
		
if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 4
	yyvsp2 := yyvsp2 -1
	yyvsp1 := yyvsp1 -2
	yyspecial_routines2.force (yyvs2, yyval2, yyvsp2)
end
when 751 then
--|#line 1662 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1662")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 752 then
--|#line 1663 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1663")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp2 := yyvsp2 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 753 then
--|#line 1664 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1664")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 5
	yyvsp1 := yyvsp1 -1
	yyvsp2 := yyvsp2 -3
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 754 then
--|#line 1665 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1665")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 4
	yyvsp1 := yyvsp1 -1
	yyvsp2 := yyvsp2 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 755 then
--|#line 1668 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1668")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 756 then
--|#line 1669 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1669")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp2 := yyvsp2 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 757 then
--|#line 1670 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1670")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 5
	yyvsp1 := yyvsp1 -1
	yyvsp2 := yyvsp2 -3
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 758 then
--|#line 1671 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1671")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 4
	yyvsp1 := yyvsp1 -1
	yyvsp2 := yyvsp2 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 759 then
--|#line 1674 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1674")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 760 then
--|#line 1675 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1675")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp2 := yyvsp2 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 761 then
--|#line 1676 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1676")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 5
	yyvsp1 := yyvsp1 -1
	yyvsp2 := yyvsp2 -3
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 762 then
--|#line 1677 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1677")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 4
	yyvsp1 := yyvsp1 -1
	yyvsp2 := yyvsp2 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 763 then
--|#line 1680 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1680")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 764 then
--|#line 1681 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1681")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp2 := yyvsp2 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 765 then
--|#line 1682 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1682")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 5
	yyvsp1 := yyvsp1 -1
	yyvsp2 := yyvsp2 -3
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 766 then
--|#line 1683 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1683")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 4
	yyvsp1 := yyvsp1 -1
	yyvsp2 := yyvsp2 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 767 then
--|#line 1686 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1686")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 768 then
--|#line 1687 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1687")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 4
	yyvsp2 := yyvsp2 -3
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 769 then
--|#line 1688 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1688")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp2 := yyvsp2 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 770 then
--|#line 1691 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1691")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp1 := yyvsp1 -1
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 771 then
--|#line 1692 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1692")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 5
	yyvsp1 := yyvsp1 -1
	yyvsp2 := yyvsp2 -3
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 772 then
--|#line 1693 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1693")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 4
	yyvsp1 := yyvsp1 -1
	yyvsp2 := yyvsp2 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 773 then
--|#line 1696 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1696")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 0
	yyvsp1 := yyvsp1 + 1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 774 then
--|#line 1697 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1697")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp1 := yyvsp1 -1
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 775 then
--|#line 1698 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1698")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp1 := yyvsp1 -1
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 776 then
--|#line 1699 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1699")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 777 then
--|#line 1702 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1702")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 778 then
--|#line 1703 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1703")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 779 then
--|#line 1704 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1704")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp2 := yyvsp2 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 780 then
--|#line 1705 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1705")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 5
	yyvsp1 := yyvsp1 -1
	yyvsp2 := yyvsp2 -3
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 781 then
--|#line 1706 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1706")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 7
	yyvsp1 := yyvsp1 -2
	yyvsp2 := yyvsp2 -4
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 782 then
--|#line 1707 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1707")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 6
	yyvsp1 := yyvsp1 -2
	yyvsp2 := yyvsp2 -3
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 783 then
--|#line 1708 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1708")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 784 then
--|#line 1709 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1709")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp1 := yyvsp1 -1
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 785 then
--|#line 1710 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1710")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 5
	yyvsp1 := yyvsp1 -2
	yyvsp2 := yyvsp2 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 786 then
--|#line 1711 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1711")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 7
	yyvsp1 := yyvsp1 -3
	yyvsp2 := yyvsp2 -3
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 787 then
--|#line 1712 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1712")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 6
	yyvsp1 := yyvsp1 -3
	yyvsp2 := yyvsp2 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 788 then
--|#line 1715 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1715")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 789 then
--|#line 1716 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1716")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 4
	yyvsp1 := yyvsp1 -2
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 790 then
--|#line 1717 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1717")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 6
	yyvsp1 := yyvsp1 -4
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 791 then
--|#line 1718 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1718")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 792 then
--|#line 1719 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1719")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 4
	yyvsp1 := yyvsp1 -2
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 793 then
--|#line 1720 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1720")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 6
	yyvsp1 := yyvsp1 -4
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 794 then
--|#line 1721 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1721")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 795 then
--|#line 1722 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1722")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 796 then
--|#line 1723 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1723")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 797 then
--|#line 1724 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1724")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 4
	yyvsp1 := yyvsp1 -2
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 798 then
--|#line 1725 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1725")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 799 then
--|#line 1726 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1726")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -2
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 800 then
--|#line 1727 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1727")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -2
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 801 then
--|#line 1728 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1728")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -2
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 802 then
--|#line 1729 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1729")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 803 then
--|#line 1730 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1730")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 4
	yyvsp1 := yyvsp1 -2
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 804 then
--|#line 1731 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1731")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 4
	yyvsp1 := yyvsp1 -2
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 805 then
--|#line 1732 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1732")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 806 then
--|#line 1733 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1733")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 807 then
--|#line 1734 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1734")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 808 then
--|#line 1735 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1735")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 809 then
--|#line 1736 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1736")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 810 then
--|#line 1737 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1737")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 4
	yyvsp1 := yyvsp1 -2
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 811 then
--|#line 1738 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1738")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 812 then
--|#line 1739 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1739")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 4
	yyvsp1 := yyvsp1 -2
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 813 then
--|#line 1740 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1740")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -2
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 814 then
--|#line 1741 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1741")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 815 then
--|#line 1742 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1742")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 4
	yyvsp1 := yyvsp1 -2
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 816 then
--|#line 1745 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1745")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 817 then
--|#line 1746 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1746")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 818 then
--|#line 1747 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1747")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 819 then
--|#line 1748 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1748")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 4
	yyvsp1 := yyvsp1 -2
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 820 then
--|#line 1749 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1749")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 4
	yyvsp1 := yyvsp1 -2
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 821 then
--|#line 1750 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1750")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 4
	yyvsp1 := yyvsp1 -2
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 822 then
--|#line 1753 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1753")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 823 then
--|#line 1754 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1754")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 4
	yyvsp1 := yyvsp1 -2
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 824 then
--|#line 1757 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1757")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp1 := yyvsp1 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 825 then
--|#line 1758 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1758")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 826 then
--|#line 1759 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1759")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 827 then
--|#line 1762 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1762")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 828 then
--|#line 1763 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1763")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 829 then
--|#line 1764 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1764")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 830 then
--|#line 1765 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1765")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 4
	yyvsp1 := yyvsp1 -2
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 831 then
--|#line 1766 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1766")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 4
	yyvsp1 := yyvsp1 -2
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 832 then
--|#line 1767 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1767")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 833 then
--|#line 1768 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1768")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 4
	yyvsp1 := yyvsp1 -3
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 834 then
--|#line 1769 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1769")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 835 then
--|#line 1770 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1770")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -2
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 836 then
--|#line 1773 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1773")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 837 then
--|#line 1774 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1774")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 838 then
--|#line 1775 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1775")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 839 then
--|#line 1776 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1776")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 7
	yyvsp1 := yyvsp1 -2
	yyvsp2 := yyvsp2 -4
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 840 then
--|#line 1779 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1779")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 841 then
--|#line 1782 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1782")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 842 then
--|#line 1785 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1785")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 843 then
--|#line 1786 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1786")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 844 then
--|#line 1789 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1789")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 845 then
--|#line 1792 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1792")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 0
	yyvsp1 := yyvsp1 + 1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 846 then
--|#line 1793 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1793")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 847 then
--|#line 1796 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1796")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 0
	yyvsp1 := yyvsp1 + 1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 848 then
--|#line 1797 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1797")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp1 := yyvsp1 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 849 then
--|#line 1800 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1800")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 850 then
--|#line 1801 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1801")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 851 then
--|#line 1802 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1802")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp1 := yyvsp1 -1
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 852 then
--|#line 1803 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1803")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 853 then
--|#line 1804 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1804")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 854 then
--|#line 1805 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1805")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 4
	yyvsp1 := yyvsp1 -1
	yyvsp2 := yyvsp2 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 855 then
--|#line 1806 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1806")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp2 := yyvsp2 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 856 then
--|#line 1807 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1807")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -2
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 857 then
--|#line 1808 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1808")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -2
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 858 then
--|#line 1811 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1811")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 12
	yyvsp1 := yyvsp1 -8
	yyvsp2 := yyvsp2 -3
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 859 then
--|#line 1814 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1814")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 0
	yyvsp1 := yyvsp1 + 1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 860 then
--|#line 1815 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1815")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp1 := yyvsp1 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 861 then
--|#line 1818 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1818")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 7
	yyvsp1 := yyvsp1 -5
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 862 then
--|#line 1819 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1819")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 9
	yyvsp1 := yyvsp1 -6
	yyvsp2 := yyvsp2 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 863 then
--|#line 1820 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1820")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 12
	yyvsp1 := yyvsp1 -7
	yyvsp2 := yyvsp2 -4
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 864 then
--|#line 1823 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1823")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 6
	yyvsp1 := yyvsp1 -4
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 865 then
--|#line 1826 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1826")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp1 := yyvsp1 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 866 then
--|#line 1827 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1827")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp1 := yyvsp1 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 867 then
--|#line 1830 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1830")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 6
	yyvsp1 := yyvsp1 -1
	yyvsp2 := yyvsp2 -4
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 868 then
--|#line 1833 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1833")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 869 then
--|#line 1834 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1834")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 870 then
--|#line 1837 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1837")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 0
	yyvsp1 := yyvsp1 + 1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 871 then
--|#line 1838 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1838")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 872 then
--|#line 1841 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1841")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp1 := yyvsp1 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 873 then
--|#line 1842 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1842")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp1 := yyvsp1 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 874 then
--|#line 1845 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1845")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 875 then
--|#line 1846 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1846")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp1 := yyvsp1 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 876 then
--|#line 1849 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1849")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 4
	yyvsp1 := yyvsp1 -3
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 877 then
--|#line 1850 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1850")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 6
	yyvsp1 := yyvsp1 -4
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 878 then
--|#line 1851 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1851")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 6
	yyvsp1 := yyvsp1 -5
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 879 then
--|#line 1854 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1854")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 880 then
--|#line 1855 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1855")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 881 then
--|#line 1856 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1856")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 882 then
--|#line 1859 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1859")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 0
	yyvsp1 := yyvsp1 + 1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 883 then
--|#line 1860 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1860")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 884 then
--|#line 1863 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1863")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 0
	yyvsp1 := yyvsp1 + 1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 885 then
--|#line 1866 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1866")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 886 then
--|#line 1867 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1867")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 887 then
--|#line 1868 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1868")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 888 then
--|#line 1869 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1869")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 889 then
--|#line 1870 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1870")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 890 then
--|#line 1871 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1871")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 891 then
--|#line 1872 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1872")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 892 then
--|#line 1873 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1873")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 893 then
--|#line 1874 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1874")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 894 then
--|#line 1875 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1875")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 895 then
--|#line 1876 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1876")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 896 then
--|#line 1877 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1877")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 897 then
--|#line 1878 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1878")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 898 then
--|#line 1879 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1879")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 0
	yyvsp1 := yyvsp1 + 1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 899 then
--|#line 1882 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1882")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp2 := yyvsp2 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 900 then
--|#line 1885 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1885")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 901 then
--|#line 1886 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1886")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp1 := yyvsp1 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 902 then
--|#line 1889 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1889")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 903 then
--|#line 1890 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1890")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 904 then
--|#line 1891 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1891")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 905 then
--|#line 1892 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1892")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 906 then
--|#line 1893 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1893")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 907 then
--|#line 1894 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1894")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 908 then
--|#line 1895 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1895")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 909 then
--|#line 1898 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1898")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 910 then
--|#line 1899 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1899")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 911 then
--|#line 1900 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1900")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp1 := yyvsp1 -1
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 912 then
--|#line 1901 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1901")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 913 then
--|#line 1902 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1902")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp1 := yyvsp1 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 914 then
--|#line 1903 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1903")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp1 := yyvsp1 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 915 then
--|#line 1906 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1906")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 916 then
--|#line 1907 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1907")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp1 := yyvsp1 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 917 then
--|#line 1908 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1908")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 918 then
--|#line 1909 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1909")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 919 then
--|#line 1910 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1910")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 920 then
--|#line 1911 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1911")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 921 then
--|#line 1912 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1912")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 922 then
--|#line 1915 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1915")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 923 then
--|#line 1916 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1916")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 924 then
--|#line 1917 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1917")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 925 then
--|#line 1920 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1920")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp1 := yyvsp1 -1
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 926 then
--|#line 1923 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1923")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 927 then
--|#line 1924 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1924")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp1 := yyvsp1 -1
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 928 then
--|#line 1927 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1927")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 929 then
--|#line 1928 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1928")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 930 then
--|#line 1929 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1929")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp2 := yyvsp2 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 931 then
--|#line 1930 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1930")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 932 then
--|#line 1931 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1931")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 933 then
--|#line 1934 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1934")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 934 then
--|#line 1935 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1935")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 935 then
--|#line 1936 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1936")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 936 then
--|#line 1937 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1937")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 937 then
--|#line 1938 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1938")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 938 then
--|#line 1939 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1939")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 939 then
--|#line 1940 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1940")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 940 then
--|#line 1941 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1941")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 941 then
--|#line 1942 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1942")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 942 then
--|#line 1943 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1943")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 943 then
--|#line 1944 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1944")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 944 then
--|#line 1945 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1945")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 945 then
--|#line 1946 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1946")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 946 then
--|#line 1947 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1947")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 947 then
--|#line 1950 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1950")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 4
	yyvsp2 := yyvsp2 -3
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 948 then
--|#line 1951 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1951")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 5
	yyvsp2 := yyvsp2 -4
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 949 then
--|#line 1952 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1952")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp2 := yyvsp2 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 950 then
--|#line 1953 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1953")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 6
	yyvsp1 := yyvsp1 -2
	yyvsp2 := yyvsp2 -3
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 951 then
--|#line 1956 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1956")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 952 then
--|#line 1957 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1957")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 953 then
--|#line 1960 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1960")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 954 then
--|#line 1961 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1961")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp1 := yyvsp1 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 955 then
--|#line 1964 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1964")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp1 := yyvsp1 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 956 then
--|#line 1965 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1965")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp1 := yyvsp1 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 957 then
--|#line 1966 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1966")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp1 := yyvsp1 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 958 then
--|#line 1967 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1967")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp1 := yyvsp1 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 959 then
--|#line 1968 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1968")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 960 then
--|#line 1969 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1969")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp1 := yyvsp1 -1
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 961 then
--|#line 1970 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1970")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp1 := yyvsp1 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 962 then
--|#line 1971 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1971")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp1 := yyvsp1 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 963 then
--|#line 1972 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1972")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 964 then
--|#line 1973 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1973")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp1 := yyvsp1 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 965 then
--|#line 1974 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1974")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 966 then
--|#line 1975 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1975")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 967 then
--|#line 1978 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1978")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp1 := yyvsp1 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 968 then
--|#line 1979 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1979")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 6
	yyvsp1 := yyvsp1 -5
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 969 then
--|#line 1982 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1982")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 970 then
--|#line 1983 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1983")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 971 then
--|#line 1986 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1986")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 6
	yyvsp1 := yyvsp1 -4
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 972 then
--|#line 1987 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1987")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 4
	yyvsp1 := yyvsp1 -2
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 973 then
--|#line 1988 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1988")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 5
	yyvsp1 := yyvsp1 -3
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 974 then
--|#line 1989 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1989")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 4
	yyvsp1 := yyvsp1 -2
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 975 then
--|#line 1992 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1992")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 0
	yyvsp1 := yyvsp1 + 1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 976 then
--|#line 1993 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1993")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 977 then
--|#line 1996 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1996")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 5
	yyvsp1 := yyvsp1 -3
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 978 then
--|#line 1997 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 1997")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 979 then
--|#line 2000 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2000")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 4
	yyvsp1 := yyvsp1 -1
	yyvsp2 := yyvsp2 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 980 then
--|#line 2001 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2001")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -3
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 981 then
--|#line 2002 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2002")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -3
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 982 then
--|#line 2005 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2005")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -2
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 983 then
--|#line 2006 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2006")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -3
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 984 then
--|#line 2009 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2009")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 985 then
--|#line 2010 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2010")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 986 then
--|#line 2013 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2013")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 5
	yyvsp1 := yyvsp1 -3
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 987 then
--|#line 2014 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2014")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp1 := yyvsp1 -1
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 988 then
--|#line 2017 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2017")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 989 then
--|#line 2018 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2018")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 990 then
--|#line 2019 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2019")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 991 then
--|#line 2020 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2020")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 992 then
--|#line 2023 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2023")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 0
	yyvsp1 := yyvsp1 + 1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 993 then
--|#line 2024 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2024")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 994 then
--|#line 2025 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2025")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp1 := yyvsp1 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 995 then
--|#line 2028 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2028")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 996 then
--|#line 2029 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2029")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 997 then
--|#line 2030 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2030")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 998 then
--|#line 2033 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2033")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 0
	yyvsp1 := yyvsp1 + 1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 999 then
--|#line 2034 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2034")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1000 then
--|#line 2035 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2035")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp1 := yyvsp1 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1001 then
--|#line 2038 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2038")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 0
	yyvsp1 := yyvsp1 + 1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1002 then
--|#line 2039 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2039")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1003 then
--|#line 2042 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2042")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 4
	yyvsp1 := yyvsp1 -1
	yyvsp2 := yyvsp2 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1004 then
--|#line 2043 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2043")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 9
	yyvsp1 := yyvsp1 -7
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1005 then
--|#line 2044 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2044")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 9
	yyvsp1 := yyvsp1 -4
	yyvsp2 := yyvsp2 -4
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1006 then
--|#line 2047 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2047")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 10
	yyvsp1 := yyvsp1 -4
	yyvsp2 := yyvsp2 -5
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1007 then
--|#line 2050 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2050")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 0
	yyvsp1 := yyvsp1 + 1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1008 then
--|#line 2051 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2051")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1009 then
--|#line 2054 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2054")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1010 then
--|#line 2057 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2057")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1011 then
--|#line 2058 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2058")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1012 then
--|#line 2059 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2059")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -2
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1013 then
--|#line 2062 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2062")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1014 then
--|#line 2063 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2063")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1015 then
--|#line 2064 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2064")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1016 then
--|#line 2067 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2067")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 0
	yyvsp1 := yyvsp1 + 1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1017 then
--|#line 2068 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2068")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1018 then
--|#line 2071 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2071")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1019 then
--|#line 2072 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2072")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp1 := yyvsp1 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1020 then
--|#line 2075 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2075")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -3
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1021 then
--|#line 2076 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2076")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -3
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1022 then
--|#line 2079 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2079")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -3
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1023 then
--|#line 2082 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2082")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -3
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1024 then
--|#line 2083 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2083")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp2 := yyvsp2 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1025 then
--|#line 2084 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2084")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -3
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1026 then
--|#line 2087 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2087")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 0
	yyvsp1 := yyvsp1 + 1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1027 then
--|#line 2088 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2088")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -2
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1028 then
--|#line 2091 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2091")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1029 then
--|#line 2092 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2092")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp2 := yyvsp2 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1030 then
--|#line 2093 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2093")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp2 := yyvsp2 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1031 then
--|#line 2096 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2096")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1032 then
--|#line 2097 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2097")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp1 := yyvsp1 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1033 then
--|#line 2100 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2100")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp1 := yyvsp1 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1034 then
--|#line 2103 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2103")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 0
	yyvsp1 := yyvsp1 + 1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1035 then
--|#line 2104 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2104")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 5
	yyvsp1 := yyvsp1 -2
	yyvsp2 := yyvsp2 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1036 then
--|#line 2107 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2107")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 0
	yyvsp1 := yyvsp1 + 1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1037 then
--|#line 2108 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2108")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 4
	yyvsp1 := yyvsp1 -2
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1038 then
--|#line 2111 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2111")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 0
	yyvsp1 := yyvsp1 + 1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1039 then
--|#line 2112 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2112")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1040 then
--|#line 2115 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2115")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 8
	yyvsp1 := yyvsp1 -5
	yyvsp2 := yyvsp2 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1041 then
--|#line 2118 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2118")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 0
	yyvsp1 := yyvsp1 + 1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1042 then
--|#line 2119 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2119")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp2 := yyvsp2 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1043 then
--|#line 2122 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2122")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1044 then
--|#line 2123 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2123")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp2 := yyvsp2 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1045 then
--|#line 2126 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2126")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1046 then
--|#line 2127 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2127")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp1 := yyvsp1 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1047 then
--|#line 2130 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2130")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 0
	yyvsp1 := yyvsp1 + 1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1048 then
--|#line 2131 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2131")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1049 then
--|#line 2134 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2134")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 0
	yyvsp1 := yyvsp1 + 1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1050 then
--|#line 2135 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2135")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1051 then
--|#line 2138 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2138")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 0
	yyvsp1 := yyvsp1 + 1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1052 then
--|#line 2139 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2139")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -2
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1053 then
--|#line 2140 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2140")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -2
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1054 then
--|#line 2141 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2141")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1055 then
--|#line 2142 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2142")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -2
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1056 then
--|#line 2143 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2143")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1057 then
--|#line 2146 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2146")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 0
	yyvsp1 := yyvsp1 + 1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1058 then
--|#line 2147 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2147")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 7
	yyvsp1 := yyvsp1 -4
	yyvsp2 := yyvsp2 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1059 then
--|#line 2150 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2150")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1060 then
--|#line 2151 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2151")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1061 then
--|#line 2154 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2154")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp1 := yyvsp1 -1
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1062 then
--|#line 2157 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2157")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1063 then
--|#line 2158 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2158")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp1 := yyvsp1 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1064 then
--|#line 2161 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2161")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 0
	yyvsp1 := yyvsp1 + 1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1065 then
--|#line 2162 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2162")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp1 := yyvsp1 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1066 then
--|#line 2165 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2165")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 9
	yyvsp1 := yyvsp1 -4
	yyvsp2 := yyvsp2 -4
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1067 then
--|#line 2168 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2168")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 4
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -4
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1068 then
--|#line 2171 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2171")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp1 := yyvsp1 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1069 then
--|#line 2174 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2174")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1070 then
--|#line 2175 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2175")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp2 := yyvsp2 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1071 then
--|#line 2176 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2176")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 5
	yyvsp1 := yyvsp1 -1
	yyvsp2 := yyvsp2 -3
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1072 then
--|#line 2177 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2177")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 4
	yyvsp1 := yyvsp1 -1
	yyvsp2 := yyvsp2 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1073 then
--|#line 2180 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2180")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1074 then
--|#line 2181 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2181")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp2 := yyvsp2 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1075 then
--|#line 2182 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2182")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 5
	yyvsp1 := yyvsp1 -1
	yyvsp2 := yyvsp2 -3
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1076 then
--|#line 2183 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2183")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 4
	yyvsp1 := yyvsp1 -1
	yyvsp2 := yyvsp2 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1077 then
--|#line 2186 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2186")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1078 then
--|#line 2187 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2187")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp2 := yyvsp2 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1079 then
--|#line 2188 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2188")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 5
	yyvsp1 := yyvsp1 -1
	yyvsp2 := yyvsp2 -3
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1080 then
--|#line 2189 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2189")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 4
	yyvsp1 := yyvsp1 -1
	yyvsp2 := yyvsp2 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1081 then
--|#line 2192 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2192")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 8
	yyvsp1 := yyvsp1 -5
	yyvsp2 := yyvsp2 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1082 then
--|#line 2195 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2195")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 4
	yyvsp1 := yyvsp1 -1
	yyvsp2 := yyvsp2 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1083 then
--|#line 2198 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2198")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 0
	yyvsp1 := yyvsp1 + 1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1084 then
--|#line 2199 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2199")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 5
	yyvsp1 := yyvsp1 -1
	yyvsp2 := yyvsp2 -3
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1085 then
--|#line 2200 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2200")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 5
	yyvsp1 := yyvsp1 -1
	yyvsp2 := yyvsp2 -3
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1086 then
--|#line 2201 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2201")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1087 then
--|#line 2204 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2204")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 0
	yyvsp1 := yyvsp1 + 1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1088 then
--|#line 2205 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2205")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1089 then
--|#line 2208 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2208")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 0
	yyvsp1 := yyvsp1 + 1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1090 then
--|#line 2209 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2209")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1091 then
--|#line 2212 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2212")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp1 := yyvsp1 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1092 then
--|#line 2215 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2215")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1093 then
--|#line 2216 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2216")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp1 := yyvsp1 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1094 then
--|#line 2219 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2219")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 0
	yyvsp1 := yyvsp1 + 1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1095 then
--|#line 2220 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2220")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp2 := yyvsp2 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1096 then
--|#line 2221 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2221")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 6
	yyvsp1 := yyvsp1 -1
	yyvsp2 := yyvsp2 -4
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1097 then
--|#line 2222 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2222")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp2 := yyvsp2 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1098 then
--|#line 2225 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2225")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 0
	yyvsp1 := yyvsp1 + 1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1099 then
--|#line 2226 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2226")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp2 := yyvsp2 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1100 then
--|#line 2229 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2229")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 6
	yyvsp1 := yyvsp1 -4
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1101 then
--|#line 2232 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2232")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 0
	yyvsp1 := yyvsp1 + 1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1102 then
--|#line 2233 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2233")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 4
	yyvsp2 := yyvsp2 -3
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1103 then
--|#line 2234 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2234")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 4
	yyvsp2 := yyvsp2 -3
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1104 then
--|#line 2237 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2237")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 0
	yyvsp1 := yyvsp1 + 1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1105 then
--|#line 2238 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2238")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1106 then
--|#line 2241 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2241")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 0
	yyvsp1 := yyvsp1 + 1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1107 then
--|#line 2242 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2242")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1108 then
--|#line 2245 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2245")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1109 then
--|#line 2246 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2246")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp1 := yyvsp1 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1110 then
--|#line 2249 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2249")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1111 then
--|#line 2250 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2250")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1112 then
--|#line 2251 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2251")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp2 := yyvsp2 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1113 then
--|#line 2252 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2252")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 5
	yyvsp1 := yyvsp1 -2
	yyvsp2 := yyvsp2 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1114 then
--|#line 2253 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2253")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 4
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -4
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1115 then
--|#line 2254 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2254")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 8
	yyvsp1 := yyvsp1 -1
	yyvsp2 := yyvsp2 -6
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1116 then
--|#line 2255 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2255")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 8
	yyvsp1 := yyvsp1 -1
	yyvsp2 := yyvsp2 -6
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1117 then
--|#line 2256 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2256")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -3
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1118 then
--|#line 2257 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2257")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1119 then
--|#line 2258 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2258")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1120 then
--|#line 2261 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2261")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 0
	yyvsp1 := yyvsp1 + 1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1121 then
--|#line 2262 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2262")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1122 then
--|#line 2265 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2265")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1123 then
--|#line 2266 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2266")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp1 := yyvsp1 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1124 then
--|#line 2269 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2269")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 0
	yyvsp1 := yyvsp1 + 1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1125 then
--|#line 2270 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2270")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1126 then
--|#line 2271 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2271")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1127 then
--|#line 2274 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2274")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp1 := yyvsp1 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1128 then
--|#line 2275 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2275")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp1 := yyvsp1 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1129 then
--|#line 2278 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2278")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1130 then
--|#line 2279 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2279")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp1 := yyvsp1 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1131 then
--|#line 2282 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2282")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 4
	yyvsp1 := yyvsp1 -3
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1132 then
--|#line 2285 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2285")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 7
	yyvsp1 := yyvsp1 -4
	yyvsp2 := yyvsp2 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1133 then
--|#line 2286 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2286")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 5
	yyvsp1 := yyvsp1 -2
	yyvsp2 := yyvsp2 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1134 then
--|#line 2289 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2289")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp1 := yyvsp1 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1135 then
--|#line 2290 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2290")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1136 then
--|#line 2293 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2293")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 0
	yyvsp1 := yyvsp1 + 1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1137 then
--|#line 2294 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2294")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -3
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1138 then
--|#line 2295 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2295")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -3
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1139 then
--|#line 2296 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2296")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 6
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -6
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1140 then
--|#line 2299 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2299")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 0
	yyvsp1 := yyvsp1 + 1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1141 then
--|#line 2300 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2300")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1142 then
--|#line 2303 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2303")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 0
	yyvsp1 := yyvsp1 + 1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1143 then
--|#line 2304 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2304")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1144 then
--|#line 2307 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2307")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1145 then
--|#line 2308 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2308")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp1 := yyvsp1 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1146 then
--|#line 2311 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2311")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1147 then
--|#line 2312 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2312")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -3
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1148 then
--|#line 2313 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2313")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -3
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1149 then
--|#line 2314 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2314")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1150 then
--|#line 2315 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2315")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1151 then
--|#line 2316 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2316")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -2
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1152 then
--|#line 2317 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2317")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -2
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1153 then
--|#line 2318 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2318")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -3
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1154 then
--|#line 2319 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2319")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -3
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1155 then
--|#line 2320 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2320")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 5
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -5
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1156 then
--|#line 2321 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2321")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 4
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -4
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1157 then
--|#line 2322 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2322")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp2 := yyvsp2 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1158 then
--|#line 2323 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2323")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 4
	yyvsp2 := yyvsp2 -3
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1159 then
--|#line 2324 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2324")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp2 := yyvsp2 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1160 then
--|#line 2327 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2327")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1161 then
--|#line 2328 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2328")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1162 then
--|#line 2329 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2329")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1163 then
--|#line 2330 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2330")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1164 then
--|#line 2331 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2331")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1165 then
--|#line 2332 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2332")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1166 then
--|#line 2333 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2333")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1167 then
--|#line 2334 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2334")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1168 then
--|#line 2335 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2335")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1169 then
--|#line 2338 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2338")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1170 then
--|#line 2339 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2339")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1171 then
--|#line 2342 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2342")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 0
	yyvsp1 := yyvsp1 + 1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1172 then
--|#line 2343 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2343")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1173 then
--|#line 2346 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2346")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 10
	yyvsp1 := yyvsp1 -6
	yyvsp2 := yyvsp2 -3
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1174 then
--|#line 2347 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2347")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 9
	yyvsp1 := yyvsp1 -3
	yyvsp2 := yyvsp2 -5
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1175 then
--|#line 2350 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2350")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 6
	yyvsp1 := yyvsp1 -5
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1176 then
--|#line 2353 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2353")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 7
	yyvsp1 := yyvsp1 -3
	yyvsp2 := yyvsp2 -3
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1177 then
--|#line 2354 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2354")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 6
	yyvsp1 := yyvsp1 -3
	yyvsp2 := yyvsp2 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1178 then
--|#line 2355 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2355")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp1 := yyvsp1 -1
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1179 then
--|#line 2358 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2358")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 9
	yyvsp1 := yyvsp1 -4
	yyvsp2 := yyvsp2 -4
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1180 then
--|#line 2359 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2359")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 8
	yyvsp1 := yyvsp1 -4
	yyvsp2 := yyvsp2 -3
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1181 then
--|#line 2360 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2360")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp1 := yyvsp1 -1
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1182 then
--|#line 2363 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2363")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 6
	yyvsp1 := yyvsp1 -5
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1183 then
--|#line 2364 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2364")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 4
	yyvsp1 := yyvsp1 -3
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1184 then
--|#line 2365 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2365")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 10
	yyvsp1 := yyvsp1 -8
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1185 then
--|#line 2366 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2366")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 10
	yyvsp1 := yyvsp1 -8
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1186 then
--|#line 2369 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2369")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 8
	yyvsp1 := yyvsp1 -5
	yyvsp2 := yyvsp2 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1187 then
--|#line 2370 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2370")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 6
	yyvsp1 := yyvsp1 -3
	yyvsp2 := yyvsp2 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1188 then
--|#line 2371 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2371")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 12
	yyvsp1 := yyvsp1 -8
	yyvsp2 := yyvsp2 -3
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1189 then
--|#line 2372 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2372")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 12
	yyvsp1 := yyvsp1 -8
	yyvsp2 := yyvsp2 -3
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1190 then
--|#line 2375 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2375")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1191 then
--|#line 2376 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2376")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp1 := yyvsp1 -1
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1192 then
--|#line 2379 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2379")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp1 := yyvsp1 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1193 then
--|#line 2380 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2380")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 7
	yyvsp1 := yyvsp1 -5
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1194 then
--|#line 2381 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2381")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 7
	yyvsp1 := yyvsp1 -5
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1195 then
--|#line 2384 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2384")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1196 then
--|#line 2385 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2385")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 1
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1197 then
--|#line 2388 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2388")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 5
	yyvsp1 := yyvsp1 -3
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1198 then
--|#line 2391 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2391")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 4
	yyvsp1 := yyvsp1 -1
	yyvsp2 := yyvsp2 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1199 then
--|#line 2394 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2394")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp2 := yyvsp2 -2
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1200 then
--|#line 2395 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2395")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 4
	yyvsp2 := yyvsp2 -3
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1201 then
--|#line 2396 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2396")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1202 then
--|#line 2397 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2397")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp1 := yyvsp1 -1
	yyvsp2 := yyvsp2 -1
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1203 then
--|#line 2400 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2400")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 0
	yyvsp1 := yyvsp1 + 1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1204 then
--|#line 2401 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2401")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 4
	yyvsp2 := yyvsp2 -3
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1205 then
--|#line 2402 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2402")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 4
	yyvsp2 := yyvsp2 -3
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1206 then
--|#line 2405 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2405")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 0
	yyvsp1 := yyvsp1 + 1
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1207 then
--|#line 2406 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2406")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 2
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -2
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1208 then
--|#line 2409 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2409")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 7
	yyvsp1 := yyvsp1 -2
	yyvsp2 := yyvsp2 -4
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1209 then
--|#line 2412 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2412")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 7
	yyvsp1 := yyvsp1 -2
	yyvsp2 := yyvsp2 -4
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1210 then
--|#line 2415 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2415")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -3
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
when 1211 then
--|#line 2418 "sql_table_name_parser.y"
debug ("GEYACC")
	std.error.put_line ("Executing parser user-code from file 'sql_table_name_parser.y' at line 2418")
end


if yy_parsing_status >= yyContinue then
	yyssp := yyssp - 3
	yyvsp1 := yyvsp1 + 1
	yyvsp2 := yyvsp2 -3
	if yyvsp1 >= yyvsc1 then
		debug ("GEYACC")
			std.error.put_line ("Resize yyvs1")
		end
		yyvsc1 := yyvsc1 + yyInitial_yyvs_size
		yyvs1 := yyspecial_routines1.aliased_resized_area (yyvs1, yyvsc1)
	end
	yyspecial_routines1.force (yyvs1, yyval1, yyvsp1)
end
				else
					debug ("GEYACC")
						std.error.put_string ("Error in parser: unknown rule id: ")
						std.error.put_integer (yy_act)
						std.error.put_new_line
					end
					abort
				end
			end
		rescue
			if yy_parsing_status = yyAborted then
				yy_retried := True
				retry
			end
		end

	yy_do_error_action (yy_act: INTEGER)
			-- Execute error action.
		do
			inspect yy_act
			when 2404 then
					-- End-of-file expected action.
				report_eof_expected_error
			else
					-- Default action.
				report_error ("parse error")
			end
		end

feature {NONE} -- Table templates

	yytranslate_template: SPECIAL [INTEGER]
			-- Template for `yytranslate'
		once
			Result := yyfixed_array (<<
			    0,    2,    2,    2,    2,    2,    2,    2,    2,    2,
			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
			    2,    2,    2,    2,    2,    2,    2,    2,   22,    2,
			    3,    4,    5,    6,    7,    8,    9,   10,    2,    2,
			    2,    2,    2,    2,    2,    2,    2,    2,    2,   11,
			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
			    2,   12,    2,   13,    2,    2,    2,    2,    2,    2,

			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
			    2,    2,    2,    2,   21,    2,   23,    2,    2,    2,
			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,

			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
			    2,    2,    2,    2,    2,    2,    1,    2,   14,   15,
			   16,   17,   18,   19,   20,   24,   25,   26,   27,   28,
			   29,   30,   31,   32,   33,   34,   35,   36,   37,   38,
			   39,   40,   41,   42,   43,   44,   45,   46,   47,   48,
			   49,   50,   51,   52,   53,   54,   55,   56,   57,   58,

			   59,   60,   61,   62,   63,   64,   65,   66,   67,   68,
			   69,   70,   71,   72,   73,   74,   75,   76,   77,   78,
			   79,   80,   81,   82,   83,   84,   85,   86,   87,   88,
			   89,   90,   91,   92,   93,   94,   95,   96,   97,   98,
			   99,  100,  101,  102,  103,  104,  105,  106,  107,  108,
			  109,  110,  111,  112,  113,  114,  115,  116,  117,  118,
			  119,  120,  121,  122,  123,  124,  125,  126,  127,  128,
			  129,  130,  131,  132,  133,  134,  135,  136,  137,  138,
			  139,  140,  141,  142,  143,  144,  145,  146,  147,  148,
			  149,  150,  151,  152,  153,  154,  155,  156,  157,  158,

			  159,  160,  161,  162,  163,  164,  165,  166,  167,  168,
			  169,  170,  171,  172,  173,  174,  175,  176,  177,  178,
			  179,  180,  181,  182,  183,  184,  185,  186,  187,  188,
			  189,  190,  191,  192,  193,  194,  195,  196,  197,  198,
			  199,  200,  201,  202,  203,  204,  205,  206,  207,  208,
			  209,  210,  211,  212,  213,  214,  215,  216,  217,  218,
			  219,  220,  221,  222,  223,  224,  225,  226,  227,  228,
			  229,  230,  231,  232,  233,  234,  235,  236,  237,  238,
			  239,  240,  241,  242,  243,  244,  245,  246,  247,  248,
			  249,  250,  251,  252,  253,  254,  255,  256,  257,  258,

			  259,  260,  261,  262,  263,  264,  265,  266,  267,  268,
			  269,  270,  271,  272,  273,  274,  275,  276,  277,  278,
			  279,  280,  281,  282,  283,  284,  285,  286,  287,  288,
			  289,  290,  291,  292,  293,  294,  295,  296,  297,  298,
			  299,  300,  301,  302,  303,  304,  305,  306,  307,  308,
			  309,  310,  311,  312,  313, yyDummy>>)
		end

	yyr1_template: SPECIAL [INTEGER]
			-- Template for `yyr1'
		local
			an_array: ARRAY [INTEGER]
		once
			create an_array.make_filled (0, 0, 1211)
			yyr1_template_1 (an_array)
			yyr1_template_2 (an_array)
			Result := yyfixed_array (an_array)
		end

	yyr1_template_1 (an_array: ARRAY [INTEGER])
			-- Fill chunk #1 of template for `yyr1'.
		do
			yyarray_subcopy (an_array, <<
			    0,  316,  316,  316,  317,  319,  318,  318,  318,  318,
			  318,  318,  318,  318,  318,  318,  318,  318,  318,  318,
			  318,  318,  318,  318,  318,  318,  318,  318,  318,  318,
			  318,  318,  318,  318,  318,  318,  318,  320,  320,  320,
			  320,  349,  349,  349,  349,  349,  349,  349,  349,  349,
			  349,  349,  349,  349,  349,  349,  349,  314,  314,  314,
			  314,  314,  314,  314,  314,  314,  314,  314,  314,  314,
			  314,  314,  314,  314,  314,  314,  314,  314,  314,  314,
			  314,  314,  314,  314,  314,  314,  314,  314,  314,  314,
			  314,  314,  314,  314,  314,  314,  314,  314,  314,  314,

			  314,  314,  314,  314,  314,  314,  314,  314,  314,  314,
			  314,  314,  314,  314,  314,  351,  368,  368,  370,  370,
			  371,  371,  377,  377,  377,  378,  378,  380,  381,  381,
			  379,  379,  379,  382,  375,  375,  386,  386,  386,  388,
			  388,  385,  385,  385,  385,  385,  385,  385,  385,  385,
			  385,  385,  385,  385,  385,  385,  389,  389,  391,  391,
			  372,  372,  372,  372,  372,  372,  372,  392,  392,  395,
			  395,  393,  393,  397,  397,  397,  397,  398,  398,  394,
			  394,  352,  400,  400,  400,  354,  402,  402,  353,  353,
			  403,  403,  404,  404,  405,  405,  350,  350,  350,  369,

			  369,  355,  407,  407,  407,  408,  408,  409,  409,  324,
			  324,  324,  323,  410,  410,  321,  321,  413,  413,  384,
			  384,  416,  416,  356,  356,  356,  356,  356,  356,  424,
			  424,  357,  357,  357,  357,  357,  421,  421,  418,  418,
			  418,  425,  425,  426,  426,  426,  426,  426,  427,  420,
			  420,  417,  417,  360,  358,  358,  359,  359,  359,  359,
			  359,  361,  362,  363,  429,  429,  430,  432,  432,  434,
			  434,  435,  435,  436,  436,  437,  437,  437,  342,  342,
			  438,  438,  439,  439,  440,  440,  396,  396,  396,  442,
			  443,  444,  445,  445,  446,  446,  447,  447,  448,  448,

			  449,  450,  450,  451,  452,  457,  457,  460,  460,  461,
			  462,  453,  453,  454,  454,  456,  456,  458,  458,  459,
			  459,  455,  455,  329,  329,  329,  329,  329,  329,  329,
			  329,  329,  329,  329,  329,  329,  329,  329,  329,  329,
			  329,  329,  329,  329,  329,  329,  329,  329,  329,  329,
			  329,  477,  346,  347,  474,  474,  467,  468,  468,  479,
			  479,  344,  344,  344,  344,  466,  348,  339,  486,  486,
			  486,  486,  465,  488,  488,  489,  489,  491,  492,  492,
			  492,  492,  492,  492,  492,  492,  493,  493,  493,  482,
			  482,  494,  494,  495,  495,  496,  496,  497,  497,  343,

			  475,  475,  498,  498,  341,  502,  502,  502,  503,  503,
			  499,  499,  499,  499,  499,  504,  504,  463,  505,  505,
			  505,  506,  464,  480,  480,  508,  484,  484,  509,  509,
			  509,  481,  481,  514,  514,  411,  411,  411,  411,  411,
			  411,  411,  490,  490,  490,  490,  490,  490,  490,  516,
			  516,  516,  516,  516,  516,  516,  519,  519,  519,  515,
			  515,  515,  518,  518,  518,  518,  517,  517,  520,  521,
			  412,  500,  522,  501,  406,  406,  526,  526,  527,  527,
			  528,  528,  528,  528,  507,  529,  529,  415,  415,  531,
			  414,  414,  530,  530,  530,  530,  530,  533,  533,  533,

			  533,  535,  535,  535,  535,  535,  535,  535,  536,  536,
			  537,  537,  532,  532,  532,  538,  539,  539,  539,  513,
			  523,  523,  523,  523,  524,  524,  525,  525,  525,  525,
			  390,  390,  390,  390,  390,  540,  540,  540,  540,  540,
			  540,  540,  540,  548,  541,  401,  401,  542,  542,  543,
			  543,  549,  549,  549,  544,  544,  545,  545,  545,  545,
			  546,  552,  552,  552,  547,  553,  534,  554,  554,  554,
			  554,  554,  554,  554,  554,  554,  554,  554,  554,  554,
			  554,  554,  554,  376,  376,  376,  376,  376,  376,  376,
			  376,  376,  376,  376,  376,  376,  556,  556,  556,  556,

			  556,  556,  556,  556,  556,  556,  556,  556,  556,  556,
			  556,  556,  556,  570,  570,  570,  570,  570,  570,  570,
			  570,  570,  570,  570,  570,  570,  570,  570,  570,  570,
			  562,  562,  573,  573,  563,  441,  441,  441,  574,  574,
			  574,  574,  574,  574,  568,  568,  512,  331,  331,  485,
			  485,  575,  575,  422,  422,  422,  422,  422,  422,  422,
			  422,  422,  577,  577,  578,  578,  578,  579,  579,  579,
			  579,  579,  579,  560,  560,  560,  560,  560,  560,  560,
			  560,  560,  560,  560,  560,  560,  560,  560,  560,  560,
			  560,  560,  560,  560,  560,  557,  557,  551,  551,  483,

			  483,  483,  483,  558,  558,  558,  550,  550,  564,  565,
			  582,  582,  581,  581,  584,  584,  583,  583,  566,  567,
			  580,  580,  580,  559,  559,  559,  559,  433,  433,  433,
			  433,  433,  433,  433,  433,  387,  387,  387,  387,  387,
			  387,  387,  387,  387,  387,  387,  387,  315,  315,  315,
			  315,  383,  383,  383,  383,  431,  431,  431,  431,  586,
			  586,  586,  586,  367,  367,  367,  367,  487,  487,  487,
			  419,  419,  419,  587,  587,  587,  587,  510,  510,  510,
			  510,  510,  510,  510,  510,  510,  510,  510,  588,  588,
			  588,  588,  588,  588,  588,  588,  588,  588,  588,  588,

			  588,  588,  588,  588,  588,  588,  588,  588,  588,  588,
			  588,  588,  588,  588,  588,  588,  572,  572,  572,  572,
			  572,  572,  589,  589,  590,  590,  590,  374,  374,  374,
			  374,  374,  374,  374,  374,  374,  373,  373,  373,  373,
			  399,  478,  585,  585,  428,  592,  592,  593,  593,  476,
			  476,  476,  476,  476,  476,  476,  476,  476,  332,  596,
			  596,  333,  333,  333,  602,  603,  603,  334,  597,  597,
			  595,  595,  594,  594,  604,  604,  605,  605,  605,  606,
			  606,  606,  607,  607,  598,  608,  608,  608,  608,  608,
			  608,  608,  608,  608,  608,  608,  608,  608,  608,  599,

			  609,  609,  610,  610,  610,  610,  610,  610,  610,  612,
			  612,  612,  612,  612,  612,  614,  614,  614,  614,  614,
			  614,  614,  611,  611,  611,  330,  616,  616,  617,  617,
			  617,  617,  617,  618,  618,  618,  618,  618,  618,  618,
			  618,  618,  618,  618,  618,  618,  618,  615,  615,  615,
			  615,  623,  623,  624,  624,  613,  613,  613,  613,  613,
			  613,  613,  613,  613,  613,  613,  613,  620,  620,  625,
			  625,  340,  340,  340,  340,  626,  626,  469,  469,  472,
			  472,  472,  621,  621,  619,  619,  345,  345,  627,  627,
			  627,  627,  628,  628,  628,  629,  629,  629,  630,  630, yyDummy>>,
			1, 1000, 0)
		end

	yyr1_template_2 (an_array: ARRAY [INTEGER])
			-- Fill chunk #2 of template for `yyr1'.
		do
			yyarray_subcopy (an_array, <<
			  630,  631,  631,  622,  622,  622,  336,  634,  634,  636,
			  632,  632,  632,  633,  633,  633,  635,  635,  637,  637,
			  638,  638,  337,  338,  338,  338,  639,  639,  640,  640,
			  640,  641,  641,  642,  643,  643,  644,  644,  645,  645,
			  646,  647,  647,  649,  649,  648,  648,  650,  650,  651,
			  651,  652,  652,  652,  652,  652,  652,  653,  653,  654,
			  654,  655,  656,  656,  657,  657,  322,  473,  555,  591,
			  591,  591,  591,  423,  423,  423,  423,  658,  658,  658,
			  658,  364,  365,  660,  660,  660,  660,  659,  659,  661,
			  661,  664,  665,  665,  667,  667,  667,  667,  668,  668,

			  666,  669,  669,  669,  670,  670,  601,  601,  671,  671,
			  672,  672,  672,  672,  672,  672,  672,  672,  672,  672,
			  662,  662,  673,  673,  675,  675,  675,  676,  676,  677,
			  677,  678,  679,  679,  674,  674,  681,  681,  681,  681,
			  680,  680,  682,  682,  683,  683,  684,  684,  684,  684,
			  684,  684,  684,  684,  684,  684,  684,  684,  684,  684,
			  600,  600,  600,  600,  600,  600,  600,  600,  600,  663,
			  663,  685,  685,  335,  335,  470,  686,  686,  686,  687,
			  687,  687,  569,  569,  569,  569,  471,  471,  471,  471,
			  511,  511,  571,  571,  571,  576,  576,  561,  366,  688,

			  688,  688,  688,  689,  689,  689,  690,  690,  325,  326,
			  327,  328, yyDummy>>,
			1, 212, 1000)
		end

	yytypes1_template: SPECIAL [INTEGER]
			-- Template for `yytypes1'
		local
			an_array: ARRAY [INTEGER]
		once
			create an_array.make_filled (0, 0, 2406)
			yytypes1_template_1 (an_array)
			yytypes1_template_2 (an_array)
			yytypes1_template_3 (an_array)
			Result := yyfixed_array (an_array)
		end

	yytypes1_template_1 (an_array: ARRAY [INTEGER])
			-- Fill chunk #1 of template for `yytypes1'.
		do
			yyarray_subcopy (an_array, <<
			    1,    1,    1,    1,    2,    2,    2,    2,    2,    2,
			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,

			    2,    2,    2,    2,    2,    2,    2,    2,    2,    1,
			    2,    1,    1,    1,    1,    1,    1,    1,    1,    1,
			    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
			    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
			    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
			    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
			    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
			    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
			    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
			    1,    1,    1,    1,    1,    1,    2,    2,    2,    2,

			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
			    2,    2,    1,    1,    1,    1,    1,    1,    1,    1,
			    1,    1,    1,    1,    1,    1,    2,    1,    1,    1,
			    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
			    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
			    1,    1,    1,    1,    1,    1,    2,    2,    2,    2,
			    1,    2,    1,    2,    1,    1,    2,    2,    2,    2,
			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
			    1,    1,    1,    1,    1,    1,    1,    2,    2,    2,
			    1,    2,    2,    2,    2,    2,    2,    1,    2,    1,

			    1,    1,    1,    1,    1,    2,    2,    2,    2,    1,
			    1,    2,    1,    1,    2,    2,    2,    2,    2,    2,
			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
			    2,    1,    2,    2,    2,    1,    2,    1,    1,    1,
			    1,    2,    2,    2,    2,    2,    2,    2,    2,    2,
			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
			    2,    2,    2,    1,    1,    1,    1,    1,    2,    1,
			    1,    1,    1,    2,    1,    2,    2,    2,    2,    2,
			    2,    2,    2,    2,    2,    1,    2,    1,    2,    1,
			    2,    2,    2,    1,    1,    1,    1,    2,    2,    2,

			    2,    2,    1,    1,    1,    2,    2,    1,    1,    2,
			    2,    2,    1,    1,    1,    2,    1,    1,    1,    1,
			    2,    2,    1,    1,    1,    1,    1,    2,    1,    1,
			    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
			    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
			    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
			    1,    2,    1,    1,    1,    2,    1,    1,    2,    1,
			    2,    2,    1,    2,    1,    1,    2,    1,    2,    1,
			    2,    2,    2,    2,    1,    1,    1,    2,    2,    1,
			    2,    2,    2,    2,    1,    1,    1,    2,    1,    1,

			    1,    1,    1,    1,    1,    1,    2,    1,    1,    1,
			    1,    1,    2,    2,    2,    2,    2,    2,    2,    2,
			    2,    2,    2,    2,    2,    1,    1,    1,    1,    1,
			    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
			    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
			    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
			    1,    1,    2,    1,    2,    2,    2,    2,    2,    2,
			    2,    2,    2,    2,    1,    1,    1,    1,    1,    1,
			    1,    1,    1,    1,    1,    1,    2,    2,    1,    2,
			    1,    2,    2,    2,    2,    2,    1,    2,    2,    2,

			    2,    2,    2,    1,    2,    1,    1,    2,    1,    2,
			    2,    2,    2,    1,    2,    2,    2,    2,    2,    2,
			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
			    2,    2,    2,    2,    1,    1,    1,    1,    1,    1,
			    1,    2,    2,    2,    1,    1,    2,    1,    2,    2,
			    2,    1,    1,    2,    2,    1,    2,    2,    1,    2,
			    2,    2,    1,    2,    1,    1,    2,    1,    1,    1,
			    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
			    2,    1,    2,    1,    2,    1,    1,    1,    2,    2,
			    1,    1,    2,    1,    2,    1,    2,    2,    1,    1,

			    1,    1,    1,    1,    1,    1,    1,    1,    2,    2,
			    2,    1,    2,    1,    2,    2,    1,    1,    1,    1,
			    1,    1,    1,    1,    1,    1,    1,    1,    2,    2,
			    1,    1,    2,    1,    2,    1,    1,    1,    2,    2,
			    1,    1,    1,    1,    2,    2,    2,    1,    1,    1,
			    1,    1,    1,    1,    1,    2,    2,    1,    1,    2,
			    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
			    1,    2,    2,    1,    1,    2,    1,    1,    1,    1,
			    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
			    1,    1,    1,    1,    1,    2,    1,    1,    2,    1,

			    2,    2,    2,    2,    1,    2,    1,    2,    2,    2,
			    1,    1,    1,    1,    1,    1,    1,    2,    1,    1,
			    1,    1,    1,    1,    1,    1,    1,    1,    2,    2,
			    2,    1,    1,    1,    1,    1,    1,    2,    2,    1,
			    1,    1,    1,    1,    2,    1,    1,    1,    1,    2,
			    2,    1,    1,    1,    2,    1,    1,    1,    1,    1,
			    1,    1,    2,    1,    1,    1,    1,    1,    1,    1,
			    1,    1,    1,    1,    2,    1,    1,    1,    1,    1,
			    1,    1,    1,    2,    1,    2,    1,    1,    1,    1,
			    1,    1,    1,    1,    1,    1,    1,    1,    1,    2,

			    2,    1,    2,    2,    1,    2,    2,    2,    1,    2,
			    1,    1,    2,    1,    1,    1,    1,    2,    2,    1,
			    1,    1,    1,    1,    1,    2,    2,    2,    1,    2,
			    1,    1,    1,    1,    2,    2,    1,    1,    1,    1,
			    1,    1,    1,    1,    1,    2,    1,    2,    2,    2,
			    1,    2,    2,    1,    2,    2,    1,    2,    2,    2,
			    2,    2,    1,    1,    2,    1,    1,    1,    1,    1,
			    2,    1,    1,    1,    2,    1,    1,    1,    2,    1,
			    1,    1,    1,    1,    1,    1,    2,    1,    1,    2,
			    2,    1,    2,    1,    1,    2,    1,    1,    2,    1, yyDummy>>,
			1, 1000, 0)
		end

	yytypes1_template_2 (an_array: ARRAY [INTEGER])
			-- Fill chunk #2 of template for `yytypes1'.
		do
			yyarray_subcopy (an_array, <<
			    1,    2,    1,    2,    2,    1,    1,    1,    1,    1,
			    2,    2,    1,    1,    1,    1,    1,    2,    1,    1,
			    1,    1,    1,    1,    2,    1,    1,    1,    1,    2,
			    1,    1,    1,    1,    2,    2,    1,    1,    2,    1,
			    2,    1,    1,    1,    1,    1,    1,    1,    1,    1,
			    2,    2,    2,    1,    2,    2,    2,    1,    1,    1,
			    1,    1,    1,    1,    2,    1,    1,    2,    2,    2,
			    1,    1,    2,    2,    1,    1,    1,    1,    2,    1,
			    1,    1,    1,    2,    2,    1,    2,    2,    2,    2,
			    1,    1,    1,    1,    1,    1,    2,    1,    1,    1,

			    1,    1,    1,    1,    1,    1,    2,    2,    2,    2,
			    2,    1,    1,    1,    2,    2,    1,    1,    2,    1,
			    1,    1,    1,    2,    1,    2,    1,    2,    1,    1,
			    1,    1,    1,    1,    1,    2,    2,    1,    1,    1,
			    1,    1,    2,    2,    2,    2,    1,    2,    2,    1,
			    2,    1,    1,    2,    1,    1,    1,    1,    1,    1,
			    1,    1,    1,    1,    1,    1,    2,    2,    2,    2,
			    1,    1,    2,    2,    1,    1,    1,    1,    2,    1,
			    1,    2,    2,    1,    1,    1,    1,    1,    1,    1,
			    1,    1,    1,    1,    1,    1,    1,    1,    2,    1,

			    2,    1,    2,    1,    2,    1,    2,    2,    1,    2,
			    2,    2,    2,    1,    2,    2,    1,    2,    2,    2,
			    2,    1,    1,    1,    1,    1,    1,    2,    2,    2,
			    1,    1,    1,    1,    2,    1,    2,    1,    1,    1,
			    2,    1,    1,    1,    1,    1,    1,    1,    2,    1,
			    1,    1,    2,    1,    1,    1,    2,    2,    2,    1,
			    1,    2,    2,    1,    1,    2,    2,    1,    1,    1,
			    2,    2,    2,    1,    2,    1,    2,    1,    1,    1,
			    2,    2,    2,    2,    1,    1,    1,    1,    2,    1,
			    1,    1,    1,    1,    2,    1,    2,    1,    1,    1,

			    1,    1,    1,    2,    1,    1,    1,    1,    1,    1,
			    1,    1,    1,    1,    1,    1,    1,    2,    1,    1,
			    1,    1,    1,    1,    2,    1,    1,    1,    2,    2,
			    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
			    1,    1,    1,    1,    1,    1,    2,    1,    2,    1,
			    2,    2,    2,    2,    2,    1,    1,    1,    1,    1,
			    2,    1,    1,    1,    1,    1,    2,    1,    1,    2,
			    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
			    1,    1,    1,    2,    2,    1,    1,    1,    1,    1,
			    1,    1,    1,    2,    2,    1,    1,    1,    1,    1,

			    1,    2,    1,    1,    1,    1,    1,    2,    1,    1,
			    1,    1,    1,    2,    1,    1,    1,    2,    1,    1,
			    1,    2,    1,    1,    1,    1,    2,    2,    1,    1,
			    2,    2,    2,    2,    1,    1,    1,    2,    2,    2,
			    2,    2,    1,    1,    1,    1,    1,    1,    1,    1,
			    1,    1,    1,    1,    1,    1,    2,    1,    1,    1,
			    1,    1,    2,    1,    2,    2,    2,    2,    2,    2,
			    2,    2,    2,    2,    1,    1,    2,    2,    1,    1,
			    2,    1,    2,    1,    1,    1,    2,    1,    2,    1,
			    2,    1,    2,    1,    2,    1,    1,    1,    1,    1,

			    1,    2,    1,    1,    1,    1,    2,    2,    1,    2,
			    2,    2,    1,    2,    2,    2,    1,    2,    2,    2,
			    1,    1,    1,    2,    1,    2,    2,    2,    1,    1,
			    2,    1,    2,    1,    2,    1,    1,    1,    1,    1,
			    1,    1,    1,    1,    2,    2,    2,    2,    1,    1,
			    1,    1,    1,    1,    1,    1,    1,    1,    2,    2,
			    2,    2,    2,    2,    1,    1,    2,    1,    1,    2,
			    1,    1,    1,    1,    1,    1,    1,    1,    2,    1,
			    1,    2,    1,    1,    1,    2,    1,    1,    1,    1,
			    1,    1,    1,    1,    1,    1,    2,    2,    1,    1,

			    2,    2,    2,    1,    2,    1,    2,    2,    2,    2,
			    2,    2,    1,    1,    1,    2,    1,    1,    1,    1,
			    1,    1,    1,    1,    2,    2,    1,    2,    1,    1,
			    1,    1,    2,    2,    2,    1,    1,    2,    2,    1,
			    1,    1,    2,    1,    2,    1,    2,    1,    1,    1,
			    1,    2,    1,    1,    1,    1,    1,    1,    2,    2,
			    2,    1,    1,    1,    1,    1,    1,    1,    1,    1,
			    1,    1,    1,    2,    1,    1,    1,    2,    1,    1,
			    2,    1,    1,    1,    2,    2,    1,    2,    2,    1,
			    1,    1,    2,    2,    2,    2,    2,    2,    1,    1,

			    1,    1,    2,    1,    2,    1,    2,    1,    1,    1,
			    1,    2,    2,    1,    1,    2,    1,    1,    1,    2,
			    1,    1,    2,    1,    1,    1,    1,    1,    1,    1,
			    2,    1,    1,    1,    1,    1,    1,    1,    2,    2,
			    2,    2,    2,    2,    2,    2,    2,    2,    2,    1,
			    1,    1,    1,    2,    1,    1,    1,    1,    1,    1,
			    1,    1,    1,    1,    2,    2,    1,    1,    1,    1,
			    1,    1,    1,    2,    2,    2,    1,    2,    1,    2,
			    1,    1,    1,    1,    1,    1,    2,    1,    1,    1,
			    2,    1,    2,    2,    2,    1,    1,    2,    1,    1,

			    1,    1,    2,    1,    1,    1,    1,    1,    1,    1,
			    1,    1,    1,    2,    2,    1,    1,    1,    2,    2,
			    2,    1,    1,    1,    1,    2,    2,    1,    1,    1,
			    1,    1,    1,    1,    1,    2,    1,    2,    1,    2,
			    1,    1,    1,    1,    1,    1,    1,    2,    1,    2,
			    1,    1,    2,    1,    1,    1,    2,    2,    2,    2,
			    2,    2,    2,    2,    2,    2,    2,    2,    1,    1,
			    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
			    1,    1,    1,    1,    1,    1,    1,    1,    1,    2,
			    1,    1,    1,    2,    1,    2,    2,    2,    1,    1,

			    2,    2,    2,    2,    1,    2,    1,    1,    1,    1,
			    2,    1,    1,    2,    1,    1,    1,    1,    2,    1,
			    2,    1,    1,    1,    1,    1,    1,    1,    2,    1,
			    2,    2,    2,    1,    2,    1,    2,    1,    1,    1,
			    2,    1,    2,    2,    2,    2,    2,    2,    2,    1,
			    2,    2,    2,    2,    2,    2,    1,    2,    2,    1,
			    1,    1,    1,    1,    1,    1,    2,    1,    1,    2,
			    2,    1,    1,    1,    1,    1,    2,    2,    1,    1,
			    1,    1,    1,    2,    1,    1,    1,    1,    2,    1,
			    1,    2,    1,    1,    1,    1,    1,    1,    1,    1, yyDummy>>,
			1, 1000, 1000)
		end

	yytypes1_template_3 (an_array: ARRAY [INTEGER])
			-- Fill chunk #3 of template for `yytypes1'.
		do
			yyarray_subcopy (an_array, <<
			    1,    1,    1,    1,    1,    1,    2,    2,    1,    1,
			    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
			    2,    1,    1,    1,    1,    1,    1,    1,    1,    2,
			    2,    1,    1,    1,    1,    2,    2,    1,    2,    1,
			    2,    2,    1,    1,    2,    1,    1,    1,    1,    1,
			    1,    1,    1,    2,    1,    2,    2,    1,    1,    2,
			    1,    2,    1,    1,    1,    1,    2,    1,    2,    1,
			    2,    2,    1,    1,    2,    1,    1,    1,    1,    2,
			    2,    1,    1,    1,    1,    2,    1,    1,    1,    1,
			    2,    1,    1,    1,    1,    1,    1,    1,    1,    1,

			    1,    1,    2,    1,    2,    2,    2,    1,    2,    2,
			    2,    2,    2,    2,    2,    1,    1,    2,    1,    2,
			    1,    1,    1,    1,    1,    1,    2,    1,    1,    1,
			    2,    2,    1,    2,    2,    1,    1,    2,    1,    1,
			    1,    1,    2,    1,    1,    1,    2,    1,    1,    1,
			    1,    1,    1,    1,    2,    1,    1,    1,    1,    1,
			    1,    2,    2,    1,    1,    1,    1,    1,    1,    1,
			    1,    1,    2,    1,    1,    1,    1,    2,    1,    2,
			    2,    1,    2,    2,    1,    1,    1,    1,    2,    1,
			    1,    1,    1,    1,    2,    1,    1,    1,    1,    2,

			    1,    2,    2,    1,    2,    1,    1,    1,    1,    1,
			    1,    2,    1,    1,    1,    2,    2,    1,    2,    1,
			    1,    1,    2,    1,    1,    1,    2,    1,    1,    2,
			    2,    1,    1,    1,    1,    1,    1,    1,    1,    1,
			    1,    1,    2,    2,    1,    1,    1,    2,    2,    1,
			    1,    1,    1,    1,    1,    2,    1,    1,    2,    1,
			    2,    2,    2,    2,    1,    2,    1,    1,    1,    2,
			    1,    1,    1,    1,    1,    2,    1,    1,    1,    2,
			    1,    1,    2,    2,    1,    2,    2,    1,    1,    1,
			    1,    1,    1,    2,    1,    1,    1,    2,    1,    2,

			    2,    1,    1,    1,    1,    1,    1,    1,    1,    2,
			    1,    1,    1,    1,    2,    2,    1,    1,    2,    2,
			    1,    1,    1,    1,    1,    1,    2,    1,    2,    2,
			    1,    1,    2,    1,    2,    2,    2,    1,    1,    2,
			    2,    1,    1,    1,    2,    1,    2,    1,    2,    2,
			    2,    2,    1,    1,    2,    1,    1,    1,    1,    2,
			    2,    2,    2,    1,    1,    2,    1,    2,    1,    2,
			    2,    1,    2,    2,    1,    1,    2,    1,    1,    1,
			    2,    1,    1,    2,    2,    1,    1,    1,    2,    1,
			    2,    1,    1,    1,    2,    1,    2,    2,    1,    1,

			    1,    1,    1,    2,    1,    1,    1, yyDummy>>,
			1, 407, 2000)
		end

	yytypes2_template: SPECIAL [INTEGER]
			-- Template for `yytypes2'
		once
			Result := yyfixed_array (<<
			    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
			    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
			    1,    1,    1,    1,    2,    1,    1,    1,    1,    1,
			    1,    1,    1,    1,    1,    1,    1,    1,    2,    2,
			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,

			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,

			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,

			    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
			    2,    2,    2,    2, yyDummy>>)
		end

	yydefact_template: SPECIAL [INTEGER]
			-- Template for `yydefact'
		local
			an_array: ARRAY [INTEGER]
		once
			create an_array.make_filled (0, 0, 2406)
			yydefact_template_1 (an_array)
			yydefact_template_2 (an_array)
			yydefact_template_3 (an_array)
			Result := yyfixed_array (an_array)
		end

	yydefact_template_1 (an_array: ARRAY [INTEGER])
			-- Fill chunk #1 of template for `yydefact'.
		do
			yyarray_subcopy (an_array, <<
			    5,    1,    0,    5,    0,    0,   97,   95,    0,    0,
			  109,   58,    0,  112,    0,   90,   99,   62,    0,   71,
			  101,   72,   64,  845,    0,   69,  248,  108,  400,  114,
			  660,  103,    0,   65,   93,   77,    0,    0,   88,  104,
			   87,   96,   70,    0,  113,   75,   89,   86,   79,   60,
			  661,   66,  106,  659,   94,   91,   67,  368,   80,   92,
			  110,    0,    0,    0,   63,   73,   85,  111,   59,    0,
			    0,    0,    0,    0,   74,    0,    0,    0,   78,  692,
			  690,  689, 1124,   76,   68,  354,   84,   98,    0,  845,
			   61,  853,   81,    0,   83,    0,    0,    0,    0,    0,

			  105,  102,    0,  107,    0,  100,    0,   82,   57,    0,
			 1069,    4,    6,    7,    8,    9,   10,   11,   12,   13,
			   14,   15,   16,   19,   20,   21,   22,   23,   24,   25,
			   26,   28,   29,   30,   31,   32,   33,   34,   35,   36,
			   37,   38,   41,   42,   43,   44,   45,   46,   47,   48,
			   49,   50,   51,   52,   53,   54,   55,   56,  323,  457,
			    0,  405,  341,  342,  343,  344,  345,  346,  347,  348,
			  349,  350,  325,  326,  327,  328,  329,  330,  331,  332,
			  334,  335,  336,  337,  338,  339,  340,  435,  433,  449,
			  456,  459,  978,    0,    3,  461,  705,  778,    0,  733,

			    0,   60,  661,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,  843,  732,  729,  728,  731,  730,  727,  842,
			    0,  734,    0,    0,  783,    0, 1069,    0,  704,  627,
			  593,  612,  645,  565,    0,  615,  592,  594,  595,  613,
			  614,  618,  619,  620,  621,  622,  623,  624,  625,  626,
			  628,    0, 1190,  703,  720,    0,  351,    0,  747,  773,
			    0,    0,    0,  845,  846,  849,    0,    0,    0,  401,
			  221,    0,  103,  221,  251,  244,    0,  245,  239,  844,
			  249,    0,    0,  240,  241,  221,  252,    0,  366,  841,
			  397,    0,  856,  857,  371,  370,  369,    0,    0,    0,

			    0,    0,  898,  898,    0,    0,  103,    0,  239,    0,
			    0,   57,    0,    0,  649,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,  869,    0,    0,    0,  868,    0,
			    0,    0,  845,    0,    0,  359,  926,    0,  353,    0,
			    0,    0,    0,    0,  175,    0,    0,  311,    0, 1125,
			    0,    0,  869,  176,    0,  174,    0,    0, 1126,    0,
			  173,    0,    0,    0,  177,  180,    0,    0,  355,  352,
			  846,  850,    0,  653,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,   27,  852,    0,    0,
			    0,    0,    0,    0,  457,  405,    0,    0,    0,    0,

			  179,    0,   39,   40,  667,  407,  406,    0,  410,  466,
			  466,  434,    0,    0,    0, 1069,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,  710,    0,    0,    0,    0,
			  591,  590,  589,    0,    0,  543,    0,  405,  593,  534,
			  535,  536,  537,  538,  539,  540,  541,  542,  594,    0,
			    0,    0,    0,  546,  545,    0,    0,    0,    0,    0,
			    0,  898,  987,    0,    0,    0,  721,  667,  301,    0,
			    0,    0,    0,  773,  776,  391,  418,  365,    0,  847,
			    0,    0,  980,  981,    0,    0,  246,    0,    0,  247,
			    0,  238,    0,    0,    0,    0,  243,    0,    0,  399,

			    0,  627,    0,    0,    0,    0,  391,  474,    0,  564,
			  898,    0,    0,    0,    0,  985,    0,    0,    0,  111,
			    0,    0,    0,  179,    0,  918,  894,  963,  885,  895,
			  892,  966,  891,  893,  920,  921,  919,  405,  886,  887,
			  888,  889,  890,    0,  627,    0,  897,  896,  969,  970,
			  978,  628,  915,    0,  917,  975,    0,    0,    0,  965,
			    0,  975,  777,    0,    0,    0,    0,  238,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,  367,  593,
			  612,  651,  592,  594,  595,  650,    0,  189,  258, 1073,
			  202, 1022,    0,  188,    0,  298,  260, 1025, 1210,  186,

			 1211, 1023, 1024,  855,    0,  256,  359,    0,  425,  275,
			  277,  276,    0,    0,  818,  802,  805,  807,    0,  795,
			  798,  811,  788,  809,    0,  794,  796,    0,  791,  806,
			  808,  817,    0,  814,    0,  825,  816,  925,  826,    0,
			    0,    0,    0,  763,  219,  254, 1077, 1087,    0,  312,
			    0,    0,    0,    0,    0,  255,    0,    0,    0,    0,
			    0,    0,    0,    0,  178,    0,    0,  851,    0,    0,
			  667,  647,    0,  647,  647,  647,    0,    0,  647,  647,
			   17,  647,  682,  647,  755,  264,    0,    0,  474,    0,
			  280,    0, 1196,    0, 1195,  654,    0, 1069,  697,  669,

			  664,  668,  672,    0,  699,  471,  470,  700,    0,    0,
			  466,    0,  466,    0,  466, 1195,    0,    0,    0,  667,
			    0,  697,    0,    0,  543,    0,  717,    0,    0,    0,
			    0,  712,  709,  711,  696,    0,  707,  706,    0,    0,
			    0,  667,  532,    0,    0,    0,    0,    0,    0,  616,
			    0,    0,    0,  898,    0,    0,    0,    0,  784, 1195,
			 1191, 1181, 1068,  587,  588,  586,  583,  584,  585,  544,
			    0,    0,    0,  912,    0, 1069,  905,  923,  922,  907,
			  908,  906,    0,  903,  909,    0,  900,  902,  904,  910,
			  924,    0, 1192,  722,    0,    0,  303,    0,  748,    0,

			  477,  773,    0,    0,  391,    0,  770,  838,  837,  836,
			    0,  425,  419,    0,    0,  854,    0,  253,  979,  169,
			    0,    0,    0,    0,    0,  235,  250,  242,    0,    0,
			  393,  395,    0,  593,  594,    0,  667,    0,  391,  767,
			  221,  475,  975,  898,  555,    0,  984,  959,    0,  982,
			    0,    0,  992,    0,    0,    0,  955,  957,  958,    0,
			  956,  916,  898,  974,  961,  962,  964,    0,  972,    0,
			  694,    0,    0,    0,    0,    0,  229,    0,  361,  426,
			  428,  612,  645,    0,    0,    0,  591,  590,  589,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,    0, 1067,

			  259,    0,  204,  203, 1082,    0,    0,    0,  300,  187,
			  185,    0,  257,  425,    0,  431,    0,    0,    0,    0,
			    0,  167,    0,    0,    0,  800,  801,  813,    0,  799,
			    0,    0,    0,    0,  927,  822,  824,  693,  691,    0,
			  212,    0,    0,    0,    0,    0, 1083,    0, 1010, 1011,
			    0,    0,    0,  167,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,  870,  840,    0,  870,    0,    0,    0,
			 1195,    0,    0,  648,    0,    0,    0,    0,    0,    0,
			    0,    0,  697,    0,    0,    0,    0,  267,  870, 1124,
			 1124, 1198,    0,    0,    0,    0,  460,  470, 1195,  657, yyDummy>>,
			1, 1000, 0)
		end

	yydefact_template_2 (an_array: ARRAY [INTEGER])
			-- Fill chunk #2 of template for `yydefact'.
		do
			yyarray_subcopy (an_array, <<
			    0,  474,    0,    0,  642,    0,    0,    0,  673,    0,
			    0,    0,  469,  431,  468,  415,    0,    0,    0,  438,
			  452,    0,  440,  454,  466,    0,  667,    0,    0, 1070,
			    0,    0,    0,  718,    0,    0,    0,  715,  708,  713,
			  695,    0,    0,  726,    0,  533,  551,    0,  557,    0,
			    0,    0,    0,    0,  563,  561,  562,    0,  531,  530,
			  986,    0,    0,    0, 1195,    0,  646,  932,    0,  931,
			    0,    0,  952,  951,    0,    0,  898,    0,  899,  898,
			  644,  667, 1183,  302,  750,    0,  482,  481,    0,  480,
			    0,  478,  391,  775,  774,  772,    0,    0,    0,    0,

			  431,    0,  848,  261,    0,  222,    0,    0,    0,    0,
			  291,  394,    0,  398,    0,    0, 1192,    0,  391,  769,
			    0,  973,  975,  554,  960,  983,    0,    0,    0,  989,
			  990,  991,  988,  993,    0,  898,  474,    0,    0,  976,
			  967,    0,  779,    0,    0,    0,  223,    0,    0,  228,
			    0,    0,  363,    0,    0,  587,  588,  586,  583,  584,
			  585,  576,    0,  577,  652,    0, 1074,  307,  290,  299,
			  263,  431,    0,    0,  389,  432,  424,  278,    0,    0,
			    0,    0,    0,    0,  116,  118,  119,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,    0, 1064,    0,

			  764,    0,    0,    0, 1078, 1088,    0,    0, 1089, 1012,
			  219, 1013, 1015,    0,  307,    0,    0,  289,  292,    0,
			    0,    0,    0,    0, 1203, 1171, 1203,  880,  881,  879,
			  872,    0,  874,    0,    0,    0,    0,  884,  870,  667,
			 1195,    0,  977,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,  756,    0,    0,    0,
			  884,    0,    0,    0,  202, 1124,    0, 1201, 1136,  201,
			  190,  190,  190,  262,    0,    0, 1195,  655,    0,  662,
			    0,  640,  641,  639,  643,  698,  670,  671,    0,  666,
			  665,  701,  702,    0,    0,    0,  773,  492,  484,  501,

			  495,  512,  520,  416,    0,  408,    0,    0,  439,  453,
			  441,  455,    0,  436,  450,    0,    0, 1072,    0, 1197,
			    0,  698,  632,  716,    0,  724,  725,  723,    0,    0,
			  550,    0,    0,  551,    0,  556,    0,  617,  560,    0,
			  431,    0,    0,    0,    0,  785, 1195, 1178,    0,  929,
			  928,    0,    0,  949,    0,  914,  911,  913,  901,    0,
			  749,    0,    0,  476,  771,    0,    0,  421,  420,    0,
			  389,    0,  170,  233,  234,  232,  231,  396,    0,    0,
			  667, 1187,  768,    0,    0,    0,  463,  372,  374,  433,
			  442,  462,  971,    0,  926,  405, 1001,    0, 1003,    0,

			  646,    0,    0,  236,  226,  236,  236,    0,  362,  427,
			  364,  675,  646, 1076,    0,    0,  310,    0,  389,  360,
			  519,    0,  358,  279,  819,  803,  160,  168,  167,  821,
			  829,    0,  828,    0,  134,  827,  832,  179,    0,    0,
			    0,    0,    0,  812,    0,  789,  810,  797,    0,  792,
			  820,  804,  815,    0,    0,    0,  766,    0,  220,  217,
			  213,  457, 1080,    0, 1170, 1166, 1165, 1164, 1167, 1163,
			 1162, 1168, 1161, 1160, 1169, 1086,    0,    0, 1106, 1014,
			    0,  309,    0,  199,  293,  294,    0,  297,    0,  865,
			  867,    0,    0, 1206,    0,    0, 1206,    0,  873,    0,

			  871,    0,    0,    0,  884,    0, 1195,  683,    0,  678,
			  679,  677,    0,    0,  684,  676,    0,   18,  681,  758,
			    0,    0,  268,    0,  267,    0,  202,    0, 1202, 1135,
			    0, 1199,    0, 1142,  191,  167,  167,  167,  286,  281,
			  282,  286,  658,    0,  190,  190,  190,    0,  638,    0,
			  402,  501,  512,    0,    0,  512,  490,    0,  508,  501,
			  508,  505,  508,  507,  511,    0,    0,  524,  408,    0,
			  411,    0,  408,    0,  437,  451, 1175,  719, 1071,    0,
			  630,    0,    0,  714,  552,    0,  559,  548,  549,    0,
			    0,  566,  520,    0,    0,  787, 1195,    0,    0,  930,

			  947,    0,    0, 1182,    0,  479,  378,  380,  381,    0,
			  379,    0,  386,    0,    0,    0,  422,    0,    0,    0,
			    0,  465,    0,    0,  466,  466,    0,    0,    0, 1002,
			    0,  994,  190,  190,  190,    0,    0,  780,    0,  227,
			  225,  224,  230,    0, 1075,    0,    0,  357,    0,  117,
			    0,  835,  834,    0,  121,  120,    0,    0,    0,    0,
			  745,  744,  737,  736,  741,  738,  735,  746,    0,    0,
			    0,  156,    0,    0,    0,    0,  823,    0, 1062,    0,
			    0,  405, 1043, 1038,  765,    0,  215, 1079,    0,    0,
			    0, 1090, 1119, 1118,    0,    0, 1110,    0,    0, 1124,

			 1107, 1108, 1007,    0,    0,  115,    0,  296,  490,  870,
			  866,    0,    0, 1208, 1172,    0, 1209,  875,  884,  759,
			    0,    0,    0,  861,    0,  688,    0,    0,    0,    0,
			  757,    0,  269,    0,    0, 1200,    0, 1101,    0,    0,
			    0,    0,    0,    0,    0,    0, 1149,    0,    0,    0,
			 1134, 1143, 1144,    0,  198,  206,  207,  182,  194,  197,
			  182,  192,  196,  182,  288,  287,  284,    0,  285,  656,
			    0,    0,    0,  635,  636,    0,  663,  397,  404,  513,
			    0,    0,  514,  491,    0,  501,  509,  503,  510,  502,
			  506,  504,    0,    0,    0,  472,  413,  409,    0,  412,

			  467,    0,  633,  634,    0,  558,  547,  524, 1194, 1193,
			    0,    0,  786, 1195,  948,  953,  898,  483,   90,    0,
			  382,  385,    0,  392,  388,    0,    0,  674,    0,    0,
			 1186,  377,    0,  375,  280,  466,    0,  466,    0,  466,
			    0,  470,  998,  167,  167,  167,  968,  782,    0,    0,
			  578,  308,    0,    0,    0,    0,    0,  152,    0,    0,
			  142,  154,  143,    0,    0,    0,  153,    0,    0,  148,
			  135,  146,    0,    0,    0,    0,    0,  742,  739,  743,
			  740,    0,    0,  166,  790,  793,    0, 1065, 1063,    0,
			 1045,    0,    0, 1039, 1057,    0,    0,    0,    0, 1092,

			    0,    0,    0,    0,    0, 1111, 1081, 1121, 1122, 1109,
			    0, 1016,    0,  200,  295,    0,  884,    0, 1207,    0,
			    0,    0,  876,    0,    0,  286,    0,    0,    0,    0,
			  687,  686,  680,  265,    0,  271,    0,  882, 1140,    0,
			    0, 1104, 1137, 1138,    0,    0,    0, 1151,    0, 1146,
			    0,    0,    0, 1152,    0, 1150, 1145,  202,    0,  167,
			  199,    0,  199,    0,  199,  283,  637,  403,  280,    0,
			  493,    0,    0,  497,  516,    0,    0,    0,  521,  525,
			  526,  408,  631,  553,  280,  667,  667, 1180, 1195, 1177,
			    0,  939,    0,  943,  946,  935,  937,  938,  936,  933, yyDummy>>,
			1, 1000, 1000)
		end

	yydefact_template_3 (an_array: ARRAY [INTEGER])
			-- Fill chunk #3 of template for `yydefact'.
		do
			yyarray_subcopy (an_array, <<
			  934,  950,  941,  942,  944,  945,  384,  383,  387,    0,
			  417, 1194, 1193,    0,  373,  464,    0,  445,    0,  447,
			  466,    0,    0,    0,  996,  997,  995,  999,    0,  781,
			  237,  356,  390,  830,  831,    0,  751,  219,  145,    0,
			    0,  155,  147,    0,  141,  833,    0,    0,    0,    0,
			  157,  164, 1060, 1059, 1061, 1026, 1044,    0,    0,    0,
			 1066,  218, 1084, 1085,    0, 1091, 1117, 1112,    0,    0,
			    0,    0, 1124, 1008,    0,    0,  313,    0,    0, 1205,
			 1204, 1174,  884,  884,    0,  760,    0,  171,    0,  182,
			    0,    0,    0,    0,    0,  273,  862, 1127,  883,    0,

			    0, 1129,    0, 1133,    0,    0,    0,  632,    0,    0,
			 1153, 1147, 1148, 1154,    0, 1159, 1157,    0,  205,  183,
			  208,  211,  195,  210,  193,  209,  494,    0,    0,  495,
			    0,    0,  515,    0,  498,    0,    0,    0,  389,  414,
			    0,    0, 1195, 1179,  940,  954,  839,  667,  667,  376,
			  446,  448,    0,  443,  898,    0,  898,    0,  151,    0,
			  122,    0,    0,  138,  139,    0,  149,    0,  165,  179,
			    0,    0,    0,    0, 1046,    0, 1093, 1114,    0,    0,
			    0, 1123,    0,    0, 1017, 1018, 1009, 1006,    0,  321,
			    0,  864,  877,  878,  762,    0,    0,  286,  199,    0,

			 1173,  685,    0,  272,    0,  266,    0,  882, 1128, 1141,
			 1140,  128,    0, 1105, 1094,    0,    0, 1158, 1156,    0,
			  487,    0,  496,  500,    0,  517,  499,    0,    0,    0,
			  527,  473, 1185, 1184,    0,    0,  444, 1005, 1000, 1004,
			    0,    0,  752,    0,  133,  123,  126,    0,    0,    0,
			  144,  150,  161,  162,  163, 1027,    0, 1036,    0, 1113,
			    0,    0,    0,    0,    0,    0,    0,  315,    0,  761,
			  859,  172,  181, 1106,  270,    0, 1098, 1130, 1132,    0,
			  129, 1102, 1103,    0, 1100, 1139, 1155,  184,    0,  489,
			    0,  522,  523,  528, 1189, 1188,    0,  754,    0,    0,

			    0,  124,  136,  137,  140,    0, 1028, 1031,    0,    0,
			 1034, 1047,    0,    0, 1020, 1021, 1019,  314,  322,    0,
			  305,  216,    0,  858,  863,  274,    0, 1131,    0,    0,
			  488,  518,  529,    0,  753,    0,  130,  125,  127,    0,
			    0,    0, 1033,    0,    0, 1038, 1048, 1049,    0,    0,
			    0,    0,  317,  860,    0, 1097, 1095,    0,  158,  131,
			  132, 1030, 1029, 1032,    0,    0, 1041,    0, 1051, 1115,
			 1116,    0,  306,    0,  319, 1099,    0,    0, 1037,    0,
			    0, 1040, 1050,    0,    0, 1058,    0,  318,    0,  304,
			    0,  159,    0,    0, 1055, 1056, 1052, 1053, 1054,  316,

			  320, 1096, 1035, 1042,    0,    0,    0, yyDummy>>,
			1, 407, 2000)
		end

	yydefgoto_template: SPECIAL [INTEGER]
			-- Template for `yydefgoto'
		once
			Result := yyfixed_array (<<
			  226,  259,  194,    1,  111,    2,  112,  113,  114,  115,
			  525,  117,  118,  119,  120,  121,  777,  972,  123,  124,
			  125,  126,  127,  128,  129,  526,  527,  528,  778,  529,
			  530,  531,  532,  138,  533,  140,  141,  534,  535,  536,
			  145,  146,  147,  148,  149,  150,  151,  152,  153,  154,
			  155,  156,  157,  644, 1183, 1705, 1184, 1185, 1755,  819,
			 1434, 1655,  698, 2244, 2245, 2337, 2246, 2281, 1869, 2037,
			  943, 1870, 2164, 1671, 2165, 1442, 1420, 2158, 1188, 1926,
			  363, 1783, 1766,  364,  365,  965, 1960,  782,  910, 1535,
			 1762, 1759,  840,  904, 1756, 1757, 1459,  393,  394, 1686,

			 1784, 2221,  486,  280,  281, 1297,  282, 1639,  160,  590,
			 1149,  283,  284,  395,  286,  987,  162,  685, 1259,  228,
			 1935, 2095, 2205,  613, 1980, 1539, 1540, 1541,  163,  164,
			  165, 1485, 1707,  166,  908,  167,  796,  168,  169,  651,
			 2189, 2267, 2320, 2352, 2374, 2389, 1416,  170,  171,  538,
			  539,  540,  175,  541,  542,  543,  229,  180,  545,  182,
			  546,  547,  185,  186,  290,  608,  915, 1174, 1422,  705,
			  878,  578,  297,  507, 1387, 1832, 1388, 1833, 1612, 1613,
			  806,  831,  832,  499, 1778,  407,  706, 1012,  408, 1570,
			 1305,  811,  812, 1013,  916,  879,  230,  231,  232, 1175,

			  412,  187,  188,  711, 1390,  189,  190,  191, 1014, 1567,
			 1795, 2138,  474, 1090, 1091, 1298, 1551, 2128, 1300, 1974,
			  233, 1564, 1787, 1565, 1301, 2132,  439,  440,  441,  442,
			  443,  444,  445,  446,  447, 1330,  735,  699, 1057,  235,
			  581,  236,  237,  238,  239,  240,  241,  242,  243,  244,
			  245,  246,  247,  248,  249,  250,  251,  252,  635, 1582,
			  700,  585,  695,  701,  702,  703,  253,  730,  424,  425,
			  731,  254, 1720,  475,  636,  936,  637,  193,  265,  815,
			  963, 1235, 2323, 1221, 1503,  552, 1474, 1699, 1222, 1223,
			 1231, 1232, 1233, 2099,  553,  785,  786,  787,  788,  554,

			  555,  790,  337, 1071, 2001,  556,  557,  558,  559, 1074,
			 1816,  560,  863, 1133, 1134, 2027, 2028, 1630,  950, 1213,
			 1911, 2075, 2187, 2184, 2185, 2173, 2307, 2308, 2257, 2345,
			 2310, 1894, 1890, 2381, 1891, 1683, 2347, 2368, 2385, 2060,
			 2054, 1678, 1679, 1455,  647,  946, 1208, 1478, 1906, 1475,
			 1691, 1898, 1531, 2284, 2327, 1941, 2107, 1700, 1701, 1907,
			 1267, 1263, 1938, 2100, 2101, 1268, 2103, 1533, 1750, 1751,
			 1752, 1495, 1347,  255,  991, 1493, 1713, yyDummy>>)
		end

	yypact_template: SPECIAL [INTEGER]
			-- Template for `yypact'
		local
			an_array: ARRAY [INTEGER]
		once
			create an_array.make_filled (0, 0, 2406)
			yypact_template_1 (an_array)
			yypact_template_2 (an_array)
			yypact_template_3 (an_array)
			Result := yyfixed_array (an_array)
		end

	yypact_template_1 (an_array: ARRAY [INTEGER])
			-- Fill chunk #1 of template for `yypact'.
		do
			yyarray_subcopy (an_array, <<
			 -32768, 2698, 3042, 2702, 2681, 16741, -32768, -32768, 31690, 31459,
			 -32768, -32768, 2436, -32768, 2695, -32768, -32768, -32768, 2446, -32768,
			 -32768, -32768, -32768, 2576, 28870, -32768, -32768, -32768, 2389, -32768,
			 -32768, -32768, 25759, -32768, -32768, -32768, 2422, 31690, -32768, -32768,
			 -32768, -32768, -32768, 31690, -32768, -32768, -32768, -32768, -32768, 2613,
			  930, -32768, -32768, -32768, -32768, -32768, -32768,  674, -32768, -32768,
			 -32768, 14704, 2677, 25528, -32768, -32768, -32768, -32768, -32768, 31921,
			 2676, 31690, 1505, 2440, -32768, 28639, 31690, 31690, -32768, 2675,
			 2674, -32768, 1911, -32768, -32768, 2365, -32768, -32768, 31690, 2649,
			 -32768, -32768, -32768, 18422, -32768, 25990, 31690, 27451, 2385, 2645,

			 -32768, -32768,  435, -32768,  695, -32768, 2369, -32768, -32768,  129,
			 1072, -32768,  513, -32768, -32768, -32768, -32768, -32768, -32768, -32768,
			 -32768, -32768, -32768, -32768, -32768, -32768, -32768, -32768, -32768, -32768,
			 -32768, -32768, -32768, -32768, -32768, -32768, -32768, -32768, -32768, -32768,
			 -32768, -32768, -32768, -32768, -32768, -32768, -32768, -32768, -32768, -32768,
			 -32768, -32768, -32768, -32768, -32768, -32768, -32768, -32768, -32768,  816,
			 2662,  562, -32768, -32768, -32768, -32768, -32768, -32768, -32768, -32768,
			 -32768, -32768, -32768, -32768, -32768, -32768, -32768, -32768, -32768, -32768,
			 -32768, -32768, -32768, -32768, -32768, -32768, -32768, -32768,  976, -32768,
			 -32768, -32768, -32768, 2631, -32768,  359, -32768, -32768, 2660, -32768,

			 31690, -32768, -32768, 2658, 2657, 2656, 12376, 2654, 2630, 31690,
			 2650, 27451, -32768, -32768, -32768, -32768, -32768, -32768, -32768, -32768,
			 16159, -32768, 16159, 16159, -32768, 13831, 4153, 2817, -32768, 9986,
			 2300, -32768, -32768, -32768, 1526, 9679, -32768, 2520, -32768, 9372,
			 9065, 8758, 8451, 8144, 7837, 7530, 7223, 6916, 6609, 6302,
			 5995, 2643, 2642, -32768,  563, 2647, -32768, 31690, 2635, 11215,
			 2388, 31690, 2380, 2576, -32768, -32768, 2487, 31690,   99, -32768,
			 2351, 2435, 2430, 2351, -32768, -32768, 2429, -32768, 2414, -32768,
			 -32768, 2428,  558, 2612, -32768, 2351, -32768, 2448, -32768, -32768,
			 1790, 17889, -32768, -32768, -32768, -32768, -32768, 31690, 2232, 2488,

			 13831, 1249, 11789, 11789, 20547, 2417, 2411, 2410, 2387, 2408,
			  134, 2471,  182, 2577, 16450, 2303, 31690, 31690, 31690, 31690,
			 2311, 31690, 31228, 31690, -32768, 31690, 31690, 31690, -32768, 31690,
			 31690, 31690, 2576, 30997, 31690, 2276,  722, 21971, -32768, 16159,
			 16159,  -37, 31690, 31690, -32768, 31690, 31690,  725, 31690, -32768,
			 30766, 31690, 2294, -32768, 2337, -32768, 31690, 31690, -32768, 2412,
			 -32768, 31690, 31690, 2439, -32768,   45, 31690, 2405, -32768, -32768,
			 2569, -32768, 16159, 2575, 2580, 2579, 2574, 2571, 2567, 2566,
			 2565, 2564, 2563, 2561,  527,  192, -32768, -32768, 31690, 31690,
			 31690, 31690, 2416, 2558, -32768,  562, 22691,  315,  859, 2261,

			  782,  694, -32768, -32768, 16159, -32768, -32768, 16159, 2274,  993,
			  946, -32768, 2259, 29353, 16159, 2539, 2543, 2219, 16159, 16159,
			 14122, 16159, 15286,  797,  473, -32768, 2423,  203, 14413, 2531,
			 2353, 1253, 1253, 14122, 13831, 2386, 1367,  562,  207, -32768,
			 -32768, -32768, -32768, -32768, -32768, -32768, -32768, -32768,  190, 20316,
			 16159, 16159, 16159, -32768, -32768, 16159, 16159, 16159, 16159, 16159,
			 16159, 11502, -32768, 16159, 29353,  983, -32768, 16159, 2491, 24539,
			 19346, 31690,  691,  102, -32768, 1921, 27682, -32768, 2532, 2528,
			 31690, 31690, -32768, -32768, 16159, 27682, -32768, 31690, 2516, -32768,
			 27451, -32768, 31690, 30058, 30058,  521, -32768, 2399,  475, -32768,

			 15286, 1300, 2300, 2520, 2522, 2527, 28408, 1893, 1332, -32768,
			 11789, 1275,  301, 31690, 31690, 16159, 15868, 31690, 2250, 2523,
			  158,  447, 30535,  735, 2251, -32768, -32768, -32768, -32768, -32768,
			 -32768, -32768, -32768, -32768, -32768, -32768, -32768,  562, -32768, -32768,
			 -32768, -32768, -32768, 2514, 1387, 2513, -32768, -32768, 2511, 2503,
			 1613, 1326, -32768, 2509, -32768, 2080, 2508, 2506, 2505, -32768,
			 2494, 2080, 2500, 2504, 31690, 2475, 27451, 2221, 31690, 30058,
			 31690, 17602, 15577, 2360, 16159, 16159, 16159, 3382, -32768, 1328,
			 2479, -32768,  910, 1000,  900, 2489, 31690, -32768, 2424, 2485,
			  205, -32768, 2290, -32768, 2352, 2175, -32768, -32768, -32768, 31690,

			 -32768, -32768, -32768, -32768, 31690, 2413, 2276, 16159,  282, -32768,
			 -32768, -32768, 2349, 2381, 2472, 2468, -32768, -32768, 2465, -32768,
			 -32768, 2464, 2462, 2461,  755, -32768, 2460, 2241, 2457, -32768,
			 -32768, 2455, 2454, 2451, 31690, -32768, -32768, 1588, -32768, 2744,
			 2735, 31690, 2426, 2441, 1649, -32768, 2438, 2153,  664, -32768,
			 2249, 2288, 2431, 2302, 2291, -32768, 31690, 31690, 2364, 2376,
			 31690, 2287, 2007, 31690, -32768, 2007, 29353, -32768, 2707, 22460,
			 16159, 16159, 2152, 16159, 16159, 16159, 2396, 16159, 16159, 16159,
			 -32768, 16159, -32768, 16159, 2409, 2190, 2007,  510,  -52, 31690,
			 1449, 16159, -32768, 29353, 1084, -32768, 31690, 18191, 19623, 2402,

			 -32768, 2107, 2088, 2398, 19623, 2393,  623, -32768, 1087, 2332,
			 1567,  129, 1567,  129,  941, -32768, 2395, 3358, 24308, 16159,
			 2390, 3376, 1616, 1928, 1922,  141, 3376, 2335, 16159, 16159,
			   63, -32768, -32768, -32768, -32768, 2268, -32768, -32768, 16159, 16159,
			 2700, 16159, -32768, 1231, 16159, 2392, 16159,  549, 16159, 5688,
			 13540, 14122, 14122, 11789, 16159, 31690, 31690, 29353, -32768, 3844,
			 -32768, -32768, 3376, 2353, 1253, 2353, 1253, 1253, 2353, 3376,
			 3339,  340, 28165, -32768, 16159, 1355, -32768, -32768, -32768, -32768,
			 -32768, -32768, 16159, -32768, -32768,  679, -32768, -32768, -32768, -32768,
			 -32768, 3233, 2384, -32768, 2379, 2183, -32768, 31690, 2373, 26989,

			 -32768,  102, 2377, 2372, 1921, 2167, -32768, -32768, -32768, 2359,
			 2346,  242, -32768, 16159, 2336, -32768, 31690, -32768, 3376, -32768,
			 1614, 2247, 2233, 2225, 2222, 2044, -32768, -32768, 31690, 2333,
			 -32768, -32768, 1604,  189,  187, 29353, 16159, 31690, 1921, -32768,
			 2351, -32768, 2080, 11789, -32768, 2146, 3376, -32768, 3223, -32768,
			 31690, 25013, 10568, 2223, 31690, 16159, -32768, -32768, -32768, 16159,
			 -32768, -32768, 11789, -32768, -32768, -32768, -32768, 16159, -32768, 21032,
			 -32768, 2058, 2049, 2048, 30058, 2046,  124, 2160, 2042, -32768,
			 2300, 1061,  799, 17602,  671, 16159, 1436, 2061, 2043, 16159,
			 16159, 16159, 16159, 16159, 16159, 16159, 16159, 16159, 16450, -32768,

			 -32768, 24077, -32768, -32768, -32768, 31690, 31690, 2215, -32768, -32768,
			 -32768, 31690, -32768,  282,  175, 1776, 1504,  129, 2164, 2299,
			 2298, 26731, 2292, 2289, 2286, -32768, -32768, -32768, 2284, -32768,
			 2283, 2282, 2280, 2278, -32768, 2285, -32768, -32768, -32768, 2246,
			 -32768, 23846, 27682, 2245, 23615, 31690,   71, 2084, -32768, -32768,
			   34, 31690, 2079, 26731, 31690, 31690, 2239, 2235,  544, 16159,
			 2007, 16159,  501, 1622, -32768, 2077, 1622, 2007, 2279, 29353,
			 2272, 2270, 2269, 2273, 2271, 2267, 2266, 2260, 2114, 3154,
			 2248, 2244, 1871, 2243, 2242, 23384, 2070, 1947, 1622,  476,
			  105, -32768, 31690,  848, 31690, 2178, -32768, 1504, 1053, -32768, yyDummy>>,
			1, 1000, 0)
		end

	yypact_template_2 (an_array: ARRAY [INTEGER])
			-- Fill chunk #2 of template for `yypact'.
		do
			yyarray_subcopy (an_array, <<
			 29353,  -52, 16159, 27220, -32768, 16159, 31690, 16159, -32768, 16159,
			 17602, 18884, -32768, 1776, -32768,  179, 16159, 2240,  129, -32768,
			 -32768,  129, -32768, -32768, 1567,  129, 16159, 16159, 31690, 2229,
			 2238, 16159, 16159, -32768, 2219, 16159,  169, 3376, -32768, -32768,
			 -32768, 2664, 2627, -32768, 2237, -32768, 2157, 15286, -32768, 2985,
			 16159, 2234, 16159, 1602, -32768, -32768, -32768, 2232, -32768, 2220,
			 -32768, 1504, 2230, 2226,  897, 20085, -32768, -32768,  108, -32768,
			 2090,  971, -32768, -32768, 2062, 3054, 11502, 3032, -32768, 11502,
			 -32768, 16159, -32768, -32768, -32768, 31690, -32768, -32768, 2207, -32768,
			 1598, -32768, 1921, -32768, -32768, -32768, 2205, 31690, 16159, 27682,

			 1904, 2927, -32768, -32768, 27682, -32768, 30058, 30058, 30058, 30058,
			 -32768, -32768,  475, -32768, 31690, 31690, 2204, 2197, 1921, -32768,
			  185, -32768, 2080, -32768, -32768, -32768, 30304, 31690, 10277, -32768,
			 -32768, 1429, -32768, -32768, 1043, 11789, 1893, 2027, 3011, -32768,
			 3376, 31690, 2187, 30058, 30058, 30058, -32768, 30058, 2155, -32768,
			 17602, 17602, 2042, 17602, 2620, 1384, 1964, 1366, 1873, 1446,
			 1345, 3376, 2947, 3376, -32768, 31690, 2181, 2148, -32768, -32768,
			 -32768, 1879, 31690, 14122, 1309, -32768, -32768, -32768,  129, 2182,
			 2180, 31690, 31690, 1590, -32768, -32768, -32768, 25274,  665, 2179,
			 1578, 2177, 2176, 1576, 2173, 2171, 2170, 2149, 2154, 31690,

			 2151, 1574,  129, 31690, 2147, -32768,  705, 1959, 2103, -32768,
			 1649, -32768, -32768, 1948, 2148, 31690, 1569, -32768, 2124, 31690,
			 2122, 31690, 2136,  413,  145, 1903,  145, -32768, -32768, -32768,
			 -32768, 1565, -32768, 20547, 25013, 1990, 31690, -32768, 1622, 16159,
			 2135, 29353, -32768, 2022, 16159, 2021, 2019, 2016, 16159, 1934,
			 2015, 2012, 16159, 2011, 2009, 31690, 2115, 2120, 31690, 1983,
			 -32768, 1943, 31690, 1935,  205,  445, 31690, -32768, 1859, -32768,
			 1835, 1835, 1835, -32768, 14995, 29353,  732, -32768,  696, 3376,
			 1930, -32768, -32768, 2219, -32768, 19623, 2107, 2088, 2074, 19623,
			 -32768, 19623, -32768,  477, 18884, 2723, 10972, -32768, 2087, 10831,

			 -32768, -32768, 1813, -32768, 2065, 1364, 2296, 27682, -32768, -32768,
			 -32768, -32768,  129, -32768, -32768, 2086, 2601, -32768, 31690, -32768,
			 2099, 3376, 1465, 3376, 16159, -32768, -32768, -32768,  203, 1966,
			 -32768, 1559, 16159, 2157, 15286, -32768, 2867, 5381, -32768, 1653,
			 1776, 2072, 2071, 20801, 31690, -32768, 3439, -32768, 2050, -32768,
			 -32768, 31690, 1793, -32768, 1940, -32768, -32768, -32768, -32768, 2066,
			 -32768, 31690, 26989, -32768, -32768,  625, 2064, 3376, -32768, 13249,
			 1309, 16159, -32768, 2044, 2044, 2044, 2044, -32768, 2060, 2059,
			 16159, 1556, -32768, 2034, 2055,  127,  454, -32768, -32768,  121,
			 -32768, -32768, -32768, 1902, -32768,  562, 14122, 17028, -32768,  580,

			 2040, 2047, 21725,  116, 2044,  116,  116, 1833, 2042, -32768,
			 2042, -32768, 2025, -32768, 31690, 27682, -32768, 12958, 1309, -32768,
			 1137, 2039, -32768, -32768, -32768, -32768, -32768, -32768, 26731, -32768,
			 2038, 24770, 2036, 16159, -32768, -32768, 2033,   45, 1861, 1860,
			 1587, 2028, 1896, -32768, 2006, -32768, -32768, -32768, 2005, -32768,
			 -32768, -32768, -32768, 2004, 2003,   91, -32768, 31690, -32768, 1719,
			 -32768,  122, -32768, 31690, -32768, -32768, -32768, -32768, -32768, -32768,
			 -32768, -32768, -32768, -32768, -32768, 1891, 1992, 2013,  485, -32768,
			 31690, -32768, 2010, 1240, -32768, 1704, 2008, -32768, 2007, -32768,
			 -32768, 1998, 1786, 1693, 31690, 1870, 1693,  832, -32768, 25013,

			 1588, 31690, 1999,   94, -32768, 1997, 1991, -32768, 2835, -32768,
			 -32768, -32768, 2582, 1993, -32768, -32768, 2550, -32768, -32768, -32768,
			 31690, 27682, -32768, 1510, 1947, 29353,  205, 29353, -32768, -32768,
			 25013, -32768, 1946, 1226, -32768, 26473, 26473, 26473, 1752, 1670,
			 -32768,  172, -32768, 29353, 1835, 1835, 1835, 1023, -32768, 16159,
			 1682,  995, 1872, 2723, 1986, 1981, 26221, 18884, 1763,  870,
			 1763, 1796, 1763, -32768, -32768, 1803, 1905, 1631, 1364, 1686,
			 -32768, 16159, 1364, 1554, -32768, -32768, -32768, -32768, -32768, 16159,
			 -32768, 31690, 1961, 3376, -32768,  203, -32768, 3376, -32768, 1513,
			 16159, -32768, 1813, 29353, 29353, -32768, 4767, 1949, 19854, -32768,

			 -32768, 31690,  340, -32768, 1950, -32768, -32768, -32768, -32768, 29827,
			 -32768, 1924, -32768,  998, 31690, 1745, -32768, 2483, 1939, 1938,
			 1942,  359, 16159, 1941,  905,  835, 1638, 16159, 16159, 1137,
			 1914, -32768, 1835, 1835, 1835, 16159, 21494, 1915, 1774, -32768,
			 -32768, -32768, -32768, 16159, -32768, 1913, 1711, -32768,  625, -32768,
			 1889, -32768, -32768, 1888, 3376, 1362, 1887, 1910,  339, 1907,
			 -32768, -32768, -32768, -32768, -32768, -32768, -32768, -32768, 1223, 1213,
			 16159, -32768, 14122, 27682, 1901, 1900, -32768, 1209, -32768,  210,
			 31690,  562, -32768, 1086, -32768, 1828, -32768, -32768, 1708, 1722,
			 31690, -32768, -32768, -32768, 1851,   66, -32768, 1866, 1764, 1386,

			  485, -32768, 1669, 27682, 31690, -32768, 1849, -32768, 27682, 1622,
			 -32768, 1209, 1810, -32768, -32768, 31690, -32768, -32768,  652, 1843,
			 1842, 27682, 1718, -32768, 1723, -32768, 16159, 1730, 1727, 1726,
			 -32768, 1509, 1536, 1694, 1822, -32768, 1822,  285,  428, 1629,
			 1566, 1532, 1547, 1546,  705,  677, -32768, 1540, 1610, 1695,
			 -32768, 1226, -32768, 31690, -32768, -32768, -32768,  114, -32768, 1791,
			  114, -32768, 1788,  114, -32768, -32768, -32768, 14995, -32768, -32768,
			 27682, 27682, 27682, -32768, -32768, 1619, 3376, 1790, -32768, -32768,
			 1787, 29596, -32768, 1781, 1780, 10756, -32768, -32768, -32768, -32768,
			 -32768, -32768, 19115, 12667, 14122,  857, -32768, -32768, 2434, -32768,

			 -32768, 2427, -32768, -32768, 1671, -32768, 3376, 1631, 1775, 1773,
			 29353, 29353, -32768, 4460, -32768, -32768, 12076, -32768, 1607, 1606,
			 -32768, -32768,  625, -32768, -32768, 1768, 31690, -32768, 29353, 29353,
			 1499, 3376, 1476, -32768, 1449, 1567,  127, 1567,  127,  151,
			 2406, 1504, 17315,   59,   59,   59, 3376, -32768, 21263, 1560,
			 3376, -32768, 31690,  869, 1767, 1759, 1506, -32768, 31690, 1593,
			 -32768, -32768, 1751, 1685, 31690, 31690, -32768, 1744, 1543, -32768,
			 -32768, -32768, 1737, 27682, 27682, 1728, 27682, -32768, -32768, -32768,
			 -32768, 2345, 1191, -32768, -32768, -32768, 1110, -32768, -32768, 31690,
			 -32768,  775, 16159, -32768, 1502, 1518, 1702,  705, 1470, -32768,

			 1491, 25013, 1445, 1714,  619, -32768, -32768, 1717, -32768, -32768,
			 1691, 1479, 1710, -32768, -32768, 1707, -32768,  294, -32768, 1526,
			 1621, 1621, -32768, 23153, 31690,  172, 1458, 1580, 31690, 1832,
			 -32768, -32768, -32768, -32768, 16159, 1690, 1510,  496, 1440, 1454,
			 1656, 1584, 1433, -32768, 1489, 1597,  384, -32768, 1594, -32768,
			 1495, 1661, 1660, -32768, 1483, -32768, -32768,  772, 31690,   59,
			 1240, 27682, 1240, 27682, 1240, -32768, -32768, -32768, 27934, 31690,
			 -32768, 1681, 18653, -32768,  497, 29122, 1679, 1676, 1670, 1137,
			 1541, 1364, -32768, -32768, 1449, 16159, 16159, -32768, 1664, -32768,
			 22229, 16159,  340, -32768, -32768, -32768, 1429, -32768, 1451, -32768, yyDummy>>,
			1, 1000, 1000)
		end

	yypact_template_3 (an_array: ARRAY [INTEGER])
			-- Fill chunk #3 of template for `yypact'.
		do
			yyarray_subcopy (an_array, <<
			 -32768, -32768, -32768, -32768, -32768, -32768, -32768, -32768, -32768, 31690,
			 -32768, 1667, 1663, 16159, -32768, -32768,  127, -32768,  127, -32768,
			 1567,  127, 1545, 1653, -32768, 1429, -32768, -32768, 1428, -32768,
			 -32768, -32768, -32768, -32768, -32768, 1652, 1645, 1649, -32768,  156,
			 27682, -32768, -32768, 14122, -32768, -32768, 1400, 1383, 27682, 1350,
			 -32768, -32768, -32768, -32768, -32768, 1596, -32768, 31690, 1504, 1615,
			 -32768, -32768, -32768, -32768, 31690, -32768, -32768, 1588, 1496, 27682,
			 1591, 1589,  550, -32768,  649, 1526, 1327, 1627, 1526, -32768,
			 -32768, -32768, -32768, -32768, 31690, 1623, 1603, -32768, 27682, 1314,
			 1410, 1526, 1450,  269, 1332, 1315, -32768, -32768, -32768, 20547,

			 1346, -32768, 31690, -32768, 25013,  714, 1621, 1465, 1500, 1342,
			 -32768, -32768, -32768, -32768, 1517, -32768, -32768, 1369, -32768, 1209,
			 -32768, -32768, -32768, -32768, -32768, -32768, -32768, 27682, 31690, 1525,
			 1524, 14122, -32768, 31690, -32768, 14995, 14995,  635, 1309, -32768,
			 1508, 1503, 1493, 5074, 3376, -32768, -32768, 16159, 16159, -32768,
			 -32768, -32768,  127, -32768, 11789, 17315, 11789, 1480, -32768, 22922,
			 1115, 1186, 1431, -32768, -32768, 1344, -32768, 1171, -32768,   45,
			 1339, 1254, 31690, 1481, -32768, 31690, -32768, -32768, 1313, 1247,
			 1212, -32768, 1426, 1421, 1447, -32768, -32768, -32768, 1259, 1406,
			 27682, -32768, -32768, -32768, -32768, 31690, 31690,  172, 1240, 1389,

			 -32768, -32768, 16159, -32768, 1373, -32768, 25013,  832, -32768, -32768,
			   72, 1193, 1317, -32768, 1256, 1230, 1219, -32768, -32768, 1349,
			 -32768, 1291, -32768, -32768, 27682, 1137, -32768, 1262, 1261, 1341,
			 -32768, -32768, -32768, -32768, 1359, 1340, -32768, -32768, -32768, -32768,
			 1322, 31690, 1318,  259, -32768, 1115, -32768, 1621, 1294,  156,
			 -32768, -32768, -32768, -32768, -32768, -32768, 16159, 1105, 1278, -32768,
			 1306, 1302, 31690, 31690,  649, 27682, 1134, 1202, 1218, -32768,
			 1271, -32768, -32768,  485, 3376, 16159,  125, -32768, -32768, 1125,
			 -32768, -32768, -32768,  362, -32768, -32768, -32768, -32768, 27682, -32768,
			 1217, -32768, -32768, 1237, 1190, 1089, 1194, -32768, 31690,  143,

			  143, -32768, -32768, -32768, -32768, 1659, 1715, -32768, 1080, 1205,
			  977,  975,  883,  874, -32768, -32768, -32768, -32768, -32768,  866,
			  875, -32768, 31690, -32768, -32768, 3376,  877, -32768, 1140, 1135,
			 -32768, -32768, -32768, 1060, -32768,  434, -32768, -32768, -32768, 31690,
			 31690, 16159, -32768, 14122,  956, 1086, -32768,  927, 31690, 31690,
			 1056,  914,  818, -32768,  948, -32768,  829,  902, -32768, -32768,
			 -32768, -32768, -32768, -32768, 1143,  898,  822,  810,  365, -32768,
			 -32768, 27682, -32768,  804,  700, -32768,  460,  643, -32768, 27682,
			 31690, -32768, -32768,  177,  601, -32768,  962, -32768,  547, -32768,
			  543, -32768,  742,  692, -32768, -32768, -32768, -32768, -32768, -32768,

			 -32768, -32768, -32768, -32768,  559,  267, -32768, yyDummy>>,
			1, 407, 2000)
		end

	yypgoto_template: SPECIAL [INTEGER]
			-- Template for `yypgoto'
		once
			Result := yyfixed_array (<<
			   -2,  534, 2894, -32768, -32768, -32768, -32768, -32768, -32768, -32768,
			   53, -32768, -32768, -32768, -32768, -32768,   17, 1065, -32768, -32768,
			 -32768, -32768, -32768, -32768, -32768, 2891,    6, 2890, 2889, 2888,
			 2884,    3, 2880, -32768, 2879, 2768, 2762,   52,   47,   32,
			 -32768, -32768, -32768, -32768, -32768, -32768, -32768, -32768, -32768, -32768,
			 -32768, -32768, -32768, -303, 1925, -1749, 1448, -1467, -850,  740,
			 -32768, -32768, 1799, -32768,  618,  569,  660, -32768,  697,  999,
			 -1167, -32768,  613, -1820, -32768, 1204, -208, -32768, -32768, -1292,
			 -1408, -482, -1494, 2496, -32768, -32768, -1644, -225, -32768, -1109,
			 -32768, -32768, -608, -1225,  895, -1401, -32768,   40,    7, -32768,

			 1145,  658, -209, -416, 2781,   24,  225,   22,    5, -291,
			 -32768, -32768, 2355,   75,  -71, -32768, -32768, 2458, 1308, -1483,
			 -32768, -32768, -32768, -32768, -669, -1697, 1070, 1826, -32768, -32768,
			 -32768, -32768, -32768, -32768, -32768, -32768, -32768, -32768, -32768, -32768,
			 -32768, -32768, -32768, -32768, -32768, -32768, 1605, -32768, -32768, 2819,
			 2818, 2816, -32768, 2815, 2814,   -1,   70, -32768,    2, -32768,
			 2812, 2811, -32768, -32768,  -51, 2206, -635, -930, -1334, 2202,
			 -549, -32768, -32768, -32768, -32768, -32768, -1102,  794, -1315, 1155,
			 -448, 1688, -32768, 1021, -32768, -372, -666, -1029, -32768, -1464,
			 -32768, -32768, 1698, -889, -32768, 1644,  173,  424, -417, -32768,

			 1407, -626, -1011, -344, -1724, -600, -32768, -744, -32768, 1198,
			  987, -32768, -461, -32768, 1430, -32768, -954, -32768,  817, -32768,
			 -288, 1228, -400, -32768, -1147, -32768, -32768, 2779, -32768, -32768,
			 2722, -32768, -32768,  147, -32768, 1453, -1168,  370, -32768, -32768,
			 1884, -252, -174, -300, -412, -32768,  113, -32768, -32768, -32768,
			 -32768, -32768, -32768, -32768, -32768, -290, 2490, -32768, -389,  666,
			 -383, -32768,  765, 1772, 1770, -289, -32768, -32768, -32768, 2348,
			 2041, -115, -1800, -394, 1582, -32768, -801, -169,   48, -32768,
			 -576, -928, -32768,  676, -1208, -190, 1046,  491, 1523, -32768,
			 -32768, 1265, -1794, -32768, -441, -32768,  552, -32768, -32768, -413,

			 -247, -32768, -32768, -1512, -32768,  951, -836,  943,  937, -32768,
			 -32768, -32768, -480, 1354, -32768,  589, -32768, -32768, -32768, -32768,
			 -32768, -32768, -32768, -32768,  474, -32768,  395, -32768, -32768, -32768,
			 -32768,  389,  680, -32768,  353, -32768, -32768, -32768, -32768, -32768,
			 -32768, 1052, -32768, -32768, 1238, -32768, -32768, -32768, -32768, -1598,
			 -32768, -32768, -1608, -32768, -32768, -32768, -32768, -32768, 1027, -32768,
			 -1602, 2648,  984, -32768,  516, -883,  515, -32768, -32768, -32768,
			  968, -32768, 2275, -280, -32768, 1492, 1220, yyDummy>>)
		end

	yytable_template: SPECIAL [INTEGER]
			-- Template for `yytable'
		local
			an_array: ARRAY [INTEGER]
		once
			create an_array.make_filled (0, 0, 32225)
			yytable_template_1 (an_array)
			yytable_template_2 (an_array)
			yytable_template_3 (an_array)
			yytable_template_4 (an_array)
			yytable_template_5 (an_array)
			yytable_template_6 (an_array)
			yytable_template_7 (an_array)
			yytable_template_8 (an_array)
			yytable_template_9 (an_array)
			yytable_template_10 (an_array)
			yytable_template_11 (an_array)
			yytable_template_12 (an_array)
			yytable_template_13 (an_array)
			yytable_template_14 (an_array)
			yytable_template_15 (an_array)
			yytable_template_16 (an_array)
			yytable_template_17 (an_array)
			yytable_template_18 (an_array)
			yytable_template_19 (an_array)
			yytable_template_20 (an_array)
			yytable_template_21 (an_array)
			yytable_template_22 (an_array)
			yytable_template_23 (an_array)
			yytable_template_24 (an_array)
			yytable_template_25 (an_array)
			yytable_template_26 (an_array)
			yytable_template_27 (an_array)
			yytable_template_28 (an_array)
			yytable_template_29 (an_array)
			yytable_template_30 (an_array)
			yytable_template_31 (an_array)
			yytable_template_32 (an_array)
			yytable_template_33 (an_array)
			Result := yyfixed_array (an_array)
		end

	yytable_template_1 (an_array: ARRAY [INTEGER])
			-- Fill chunk #1 of template for `yytable'.
		do
			yyarray_subcopy (an_array, <<
			  110,  178,  459,  820,  181,  136,  256,  258,  131,  159,
			  509,  505,  551,  551,  584,  737, 1132,  436,  312,  122,
			  784,  996,  268,  691,  707,  997,  338, 1176,  720, 1657,
			  279,  416, 1339,  260,  144,  288, 1616,  369, 1237, 1528,
			 1732,  289,  158, 1479,  462,  652,  841, 1768,  789,  143,
			 1126,  448, 1524,  658,  142,  116,  561, 1299,  839,  662,
			 1260,  279,  582,  665,  489,  754,  713,  289, 1754,  314,
			 1761, 1186,  179,  258,  336,  289,  496,  161,  826,  804,
			  993,  868, 1899, 1302, 1647, 1019,  289, 1022, 1061,  966,
			 1815,  373,  511,  383,  258,  373, 1978, 1908,  374,  335,

			 2082, 2083,  385, 1186, 1796,  472, 1264,  285, 1799, 1389,
			  988, 1020, 2017, 1023, 2019,  192, 1962,  503,  484, 1964,
			  935, 1959, -214,  494, 2086,  360,  448,  736,  503,  503,
			 1385,  494,  109, -214, 1349, 1760, 1763,  371,  285,  466,
			  583,  494, 1212, 2098,  144, 1753, 1949, 1552, 1555, 1680,
			  458,  457,  461,  456,  882,  455,  752,  751, 1266,  143,
			 1584,  852, 1536, 1537,  142,  855,  452,  451,  638,  761,
			 1370,  551, 1340,  935,  458,  457, 1100,  456,  794,  455,
			  450,  729, 1009, 2163, 1038,  411, 1304,  992, 1385, 1137,
			  452,  451, 1261, 1554, 2020,  683,  821, 1211,  415,  463,

			 1207,  460,  463, 2395,  450, 1903,  572,  427,  303,  373,
			  750, 2121,  725, 2123, 2336, 2125,  429,  221,  642,  460,
			  551,  687, 1765, 1887,  707,  742,  743,  841,  219,  218,
			  217,  216,  215,  214, 1677,  213, 1115, 1902, 1114,  756,
			  212, 1418, -423,  709, 1625,  355,  588, 1206,  448, 1099,
			 1875,  353,  596, -423, 2326,  258,  755,  473,  470,  258,
			  448,  209,  605,  842, 1550,  481,  208, 2406,  641,  358,
			 1722,  783,  645,  871,  458,  457,  903,  456, 1171,  455,
			  655, 1764, -423, 1623, 1389,  477, 2213,  503,  310, 1624,
			  452,  451, 2150, -423, 2151,  258, 1724, 2153, 1824, 2063,

			  437, 1735,  562, 1754,  450, 1761, 2394,  483,  707,  482,
			 1504,  479, 2162,  682,  258,  279,  589,  258,  845,  258,
			  258,  279, 1265,  258,  258,  599,  834,  258,  258,  258,
			 1210,  279,  258,  935, 1152,  415,  503, 2102,  939,  344,
			  643,  279, 1874,  646,  648, 1172,  643,   26,  258,  279,
			  793, 1181,  571,  956,  643,  258, 1095, 1070,  606,  258,
			  643,  501, 1121, -423,  643, 1901, 1018, 2300, 1021,  728,
			 1025,  707,  544,  544, 1389,  437, 1391,  537,  537,  349,
			  603,  971, 1529,   26, 1225,   26,  684,  684,  415,  258,
			 1119, 1238, 1308, 1278,  694, 1310, 2270,  503,  438, 1313,

			 1303, 2335,  697, -423, 1340,  199, 1555, 1092, 1348,  816,
			 1592,  715,  470, 2098,   26,  550,  550, 1804, 1309,  360,
			  569, 1311, 1035, -458, 1958, 1314, 1638, 2302, 2236, 2161,
			 1030, 2087, -214, 1500, 1148, 1770, 1771, 1772, 2227, 2228,
			 1383,   26,    4, 1760, 1763, 2198,  510,  759,  902, 2272,
			 1324, 1780, 1044, 2096, -423, 1492, 2176, 1048, 1146,  775,
			 1201, 1293,  715,  551,  502,  697,  882,  798,  800,  801,
			 2181,    4,  707,  438,  809,  548,  548,  563,  279,  817,
			 2145, 2384, -458,  809, 1151,  589, 1384,  579,  373, 2202,
			  258,  279,  279,  781, -423,  823, 2124,  437, 1383,  750,

			 2097,  196, 1698,  844,  838, 1230, 1060, 2008,  780,  437,
			 1922,  258,  268,  779,  776,  849,  824, 2139, -457,  877,
			  258, 1940, 2353, 1843, 1844, 1845, 2112, -458, 1399, 1262,
			  681,  544, 1261,  911, 1490,  267,  537,  260, 1824,  355,
			  841, 2360, 1389, 1058, 1059,  353,  335, 1117, 2080,  990,
			 -423, 1391,  106,  551,  328, 2299, 2329,  401, 1697, 2405,
			  853, 1132,  589, 1261,  373,  494,  258,  279,  289, 2401,
			  285,  873,  551, 2400,  550,  437,  328, -457, 1186,  503,
			  544, 2046, 2047,  437,  258,  537, 1062, 1063,  219, 2079,
			 -423,  421,  875,  882,  732,  438, 1122,  258,  584,  830,

			  212, 1408,  279, 1785, 1410,  406,  400,  438, 1943,  358,
			 1069, 1068, 1067,  266, 1052, 1139, 1011, 1696,  994, 1634,
			 2383,  399, -457,  550, 1290,  989, 1292, 2398,  398,  277,
			  384, 1120,  934,  344,  548,  784, 2359, 1261,  784,  643,
			  358, 1391, 1392,  324, 1364, 1322,  582, 2391,  680, 1611,
			 2111, 1229, -458, 2328,  643,  258, 1229, 1228,  258,   63,
			 2220,  964, 1228,  789,  715,  324,  789,  970,  697,  503,
			 1382, 1921, 1942,  833,  405,  841,  458,  457,  503,  456,
			 1312,  455,  638,  548,  275,  328, 1574,  279,  503, 1269,
			 1079,  998,  452,  451,  258, 1633, 1004,  493, 1718, 2057,

			  935,  829, 1004, 2271, 1051,  949,  450, 2131, 2078,  503,
			 1227, 1682, 1575,  388,  358, 1227, 1029,  697,  825,  349,
			 1695,  948,  465, 2220,  583,  321,  549,  549, 1050, 1737,
			 2397, 2212, 2118,  882,  882, 1546,  882, 1315,  580,  697,
			 1441, 1543, 1694, 1473,  880, 1103, 2402, -457,  331, 1104,
			  349, 2390,  316,  415,  415, 1064, 2170,  273,  366, 1920,
			   32, 2252, 1011, 1335, 1693, -1071, 1632, 1265, 2396, 1338,
			  336,  397, 1440, 1472,  324, 1610, 1205,   26,  389,  420,
			 1692, 1916, 2057, 1609, 1471,  761,  551, 1470,  722,  551,
			 2211,  468, 1359, 1010,  876, 1084, 2130, 1089, 1608, -430,

			 1078, 1439,  458,  457, 2231,  456, -430,  455, 1607, 1391,
			 -430, 1545, 2023, 2403,  279,  360, -324,  270,  452,  451,
			 2388, 2330,  612,  544,  349, 1573,  258, -324,  537,  947,
			 2387,  506,  450,  715,  697, 1118,  503, 2382, 1606,  611,
			 1170, 1153, 1469,  903,  296,  551, 2183, -168, 1125,  415,
			  587, 1130,  258,  591, 2071,  593,  595, -280, 2182,  597,
			  598, -280,  360,  600,  601,  602,  550, 1142, -280, 1129,
			 2160, 1952,  279, 2032, 2192, 2193, 1822, 1468, 1837, -168,
			 2380, -280, 1544, 2070,  654,  549,  783, 1272, 1398,  783,
			  803,  659, 1438, 1611, 1548,  661, 2056,  610, 1467, 1166,

			 -582, 2379,  503,  258,  258,  503, 1343, -582, -168,  279,
			 -579, -582, 1709,  544,  295,  729,  737, -579,  537,  809,
			 -430, -579, 1131, 1273, 1466,  688,  548,  709, 2377, 1465,
			 -1072, 2230,  544, 1502,  549,  355,  294,  537, -430, 1200,
			  809,  353, 1204,  415,  802, 1378, 1379, 2229, 1835,  258,
			 1505,  809,  258,  258,  448, 1522,  550, 1177, 2376, 1437,
			  927,  503, 1841, 1271, 1563,  550, 2399, 1240, 1951, 1104,
			 -280, 1464,  696,  391, 2375,  550,  503,  503, -280,  503,
			 2003, 1950,  355, 1256, 1024,  390,  390, 1229,  353,  712,
			  643, -280,  279, 1228, 2373,  881,  609,  709, 1276, -168, yyDummy>>,
			1, 1000, 0)
		end

	yytable_template_2 (an_array: ARRAY [INTEGER])
			-- Fill chunk #2 of template for `yytable'.
		do
			yyarray_subcopy (an_array, <<
			 -581, 1283, 1823,  697, 1288, 1822, 2026, -581,  219,  258,
			 1562, -581,  463,  348,  650,  902,  548,  327, 1436, 1610,
			  212, -582, 1611, 1628,  697,  548, 1317, 1609,  736,  344,
			 1561, -579,  351,  709, 1270,  548,  710,  649,  709, 1731,
			  411,  973, 1608,  973,  973,  973, 1227, 1560,  973,  973,
			 1397,  973, 1607,  973, 1396, 2372,  880,  926,  925, 2371,
			  348, -429, 1275, 1346, 2358,  638, -168, 2357, -429, -280,
			  995,  579, -429,  345,  775, -653,  344,  775, 1353,  697,
			  343,  396, 1606, 1360, 2342,  709, -1072, 2341, -280, 1563,
			 1016, 1620, 1391, 1000, 1391, 1366, 2367,  809, -1184,  410,

			 2067, -1184,  809,  728,  279,  279,  279,  279,  781,  323,
			 1893,  781,  415,  415, 1015, 1352, 1351, -1070, 1053, 1558,
			  899, -581,  437,  780, 1393, 1394,  780, 1386,  779,  776,
			 2365,  779,  776,  909, 2053, 1562, 2052,  321,  293, 1401,
			  292,  279,  279,  279,  409,  279,  544, 2378, 1610,  544,
			  318,  537,  752,  751,  537, 1561, 1609,  317,  752,  751,
			 1789, 2356, 1791, 1413,  316, 2015, 2355, -280, 2354, 1340,
			  258, 1608, 1560,  737, 2351, 2251, 2350,  549,  716,  258,
			 1427, 1607, -429,  880, 2349,  415,  752,  751, 1629,  550,
			 1559,  957,  550, 2348,  960, 2051, 1419, 1456, 2346, -1185,

			 -429, 1462, -1185, 1395, 2344,  544,  752,  751, 2343, 1461,
			  537, 1606, 1775,  258, 1774, 1773,  810,  258, 1423, 1488,
			 2333, 2331, 2321,  503, 1104, 2288, 2058,  454,  453,  792,
			 1001,  562,  415, 2300,  643, 1045, 1020,  697, 1023, 1506,
			 1880, 1879, 1460, 1749, 1558,  707,  752,  751,  550,  548,
			 1878, 1877,  548, 1519,  458,  457,  643,  456,  458,  455,
			 1526, 2332, 1652,  455, 1530, 2292, 2291,  549, 1767, 1767,
			  452,  451, 1391,  715, 1391,  451,  549, 1391, 2322,  843,
			 1836, 1838, 2319, 1004,  450,  736,  549, 1004,  450, 1004,
			  752,  751,  258,  258,  473, 2289, 1748, 2318, 2288, 2015,

			 -333,  438, 1386, 2210, 2311,  809, 2313,  881,  548, 1892,
			 2312, -333, -610, 1723, 1747, 2309, 1578, 2259,  761, 2026,
			 1104, 2303,  580,  880,  880, 2243,  880, 2298, -580, 2296,
			  638, 1373, 1374, 1375, 1376, -580, 1746,  860, -611, -580,
			  896, 1596, 1597, 2253, 2295, -569, 2088,  895, 2250, 1600,
			 2208, 2249, -569, 2207, 2171, 1745, -569, 1104, -653, 1604,
			 1089,  638, 1110, 2294,  449, 2293, -570, -777, 1403, 1404,
			 1405,  753, 1406, -570, -777, 2000, 2287, -570,  697, 1868,
			  450, 2286,  752,  751, -571, 2283, -1120, 2169, 1136, 1076,
			 2088, -571, 1386, 2282, 2049, -571, 1130, -1120,  857, -610,

			 1637,  450, 1744, 2279, 2168, 2276, 1499, 2088, 1391,  437,
			 2285, 1743, 1644,  809, 1129, 2273, 1309, 1331, 1311,  450,
			  512, 1314, 2275, 2266, 1742, -1120,  809, 1640, 1641,  415,
			 -1120,  967, 2156, 2265,  881, 2155, -575, 1867, -627, 1167,
			 1168, -610, 1866, -575, 1741, 1865, -568, -575, 1864, -580,
			 1704,  458, 1886, -568, 2264, 1684,  455, -568,  999, 1740,
			 -628, 1687, 2089, -611, 1882, 2088, -569, 1131,  451, 1440,
			 2263,  450, 1739, 1261, 2065, 2262, 2261, 2064,  258, -1120,
			 2014,  450, 2260, 2013, 2256, 1214, 1917, -570, 1217, 1218,
			 1858, 2016,  646, 2018, -1120, 2021, 2247,  415, 2248, 1719,

			  549, -1120, 1990,  549, 1975, -571, 2240, 2233, -1182,  707,
			  550, -1182, 2232, 1933, 1863, 1862, 1104, 1805, 1730,  809,
			 1032, 1421,  999,  715,  221,  715, 1998, 2224,  415, 2223,
			 1681, 2218, -1120,  809,  809,  809,  218,  217,  216,  215,
			  214,  715,  213, 2217, 2216, 1296, 1919,  330, 1581, 2215,
			  358,  258, 1575, 2204,  809,  258,  329, -575, 1800,  549,
			 1386, 1104, 1861, 1586, 1860, -1183, 1032, -568, -1183, 1498,
			  548, 2201, 1497, 1483,  881,  881, 1428,  881, 1458,  258,
			 1449, 1104, 1445, 1448,  461, 1444, 1979, 2178,   26, 1859,
			 1670,  715,  715, 1669, 1429, 1668, 1813, 1428, 1858, 1814,

			 1116, 1667, 1363, 1265, 2199, 1362, 1337, 1820, 1113, 1032,
			 2196, 1112, 1825, 1666, 1665, 1664, 1663, 1662, 1105, 1661,
			 1033, 1104, -618, 1032, 1958, -601, 1999, 1669, 1356, 1668,
			 2190, 1358, 2195, -1120, 1847, 1667,  935, 2188, 2180, 2177,
			 2179, 2175,  503, 1011, -1120, 2172,  328, 1666, 1665, 1664,
			 1663, 1662,  942, 1661, 2159, 2157, 1857, 1591, 2154,  709,
			  349, 1187,  995,  327,  458,  457, 2148,  456,  503,  455,
			 2147,  809, 1856, 1810, 1569, 2137, 2152, 1767,  258, 2136,
			  452,  451, 2135,  326, 2127, 2117, 2116, 2115, 1530, 2114,
			 2108, 2106,  325, 1187,  450, 2113, 2140, 2141, 2110, 2109,

			 2104,  809, 1913, 2105, 1589, 2102,  809, 2094, 2340, 2090,
			  761, 2077,  199,  415, 2076, 1426, 2074, 2069, 2073,  809,
			 -593, -593, 2068, -593, 2072, -593, 2066,  460, 2062, 2081,
			 2061, 2048,  638, 2059,  999,  324, -593, -593,  975,  976,
			  977, 2045, 2290,  980,  981, 2044,  983, 2043,  984, 1482,
			 -593, 1957, 2040, 1486, 2039,  323, 1473,  458,  457, 2091,
			  456, 2035,  455, 2034, -593, 1277,  322, 2038,  809,  809,
			  809, 2033, 2030,  452,  451, 2010, 1986, 2009, 1985, 1970,
			 2007, 2006, 1794,  321, 1971,  320, 1472,  450, 1104, 1660,
			  258, 1968, 1983,  498,  319, 1963,  318, 1471, 1961, 2209,

			 1470, 2031, 1765,  317,  227, 1955, 1953, 1946,  715, 1988,
			  316, 1966, 1948, 1947, 2339, 1995, 1973,  315, 1997, 1994,
			 1954,  549, 1993, 1660,  289, 1937,  715,  715, 1296, 1296,
			 1277, 1944, 1945, 1936, 1934, 2167, 2092,  458,  457,  810,
			  456, 2024,  455, 1386, 1372, 1386, 2029, 1932, 1931, 1924,
			  289, 1930, 1923,  452,  451, 1469, 2036, 1928, 2234, 2235,
			  301, 1764, 2041, 2036,  551, 1918,  551,  450, 1234, 1904,
			 -593,  809,  809, -567,  809, 1914,  458,  457,  458,  456,
			 -567,  455, 1910,  455, -567, 2186, 1996, 2055, 2191, 2386,
			 1468,  537,  452,  451, 2219,  451, 1905, 2392, 1897,  415,

			 1900, 2200, 1896, 1895, 1885, 1884,  450, 2237,  450, 2239,
			 1876, 1467, 2025, 1873, 1872, 1855, 1854, 1851, 1852, 1534,
			 1849, 2085, 1719, 2225, 1848, 1842,  415,  458,  457,  550,
			  456, 1839,  455,  458,  457,  638,  456, 1466,  455,  747,
			  454,  453, 1465,  452,  451, 1834, 1830, 1829, 1828,  452,
			  451, 1821, 1826,  362, 1817,  550, 2119,  450, 1811,  809,
			 1566,  809,  361,  450, -572, 1803, 1970, 2126, 1797,  458,
			  258, -572, 1793, 2134,  455, -572, 1792, 1034, 1786, 1790,
			  503,  503,  503,  697,  697, 1782,  451,  746, 2142,  548,
			 1781,  360, 1777, 1779, -567, 1738, 1258, 1728,  359,  450, yyDummy>>,
			1, 1000, 1000)
		end

	yytable_template_3 (an_array: ARRAY [INTEGER])
			-- Fill chunk #3 of template for `yytable'.
		do
			yyarray_subcopy (an_array, <<
			 1543, 1725, 1721, 1712, 1715,  423, 1277, 2146, 1711, 1710,
			  962, 1708, 1706, 1703, 1702,  548, 1690, 1676, 1689,  430,
			 1688,  431,  432, 1386,  435, 1386, 1252, 1677, 1386, 1685,
			 1673, 1672, 1675, 1674, 1659, 1658, 1656,  638,  809, 1653,
			 1542, 1650, 1648, -573, 1643, 1642,  809,  437,  458, 1151,
			 -573,  494,  328,  455, -573,  258, 1636, 1627, 1622, 1635,
			 1621, -574, 1530, 1619, 1618,  451,  458,  809, -574, -179,
			 1603,  455, -574, 1614, 1602,  358, 1599,  745,  450, 1601,
			 1594, 1593, 2194,  451, 1173, -572,  809, 1296, 1585,  357,
			 1576, 1296, 1568,  512, 1557, 1007,  450,  562,  356,  435,

			  643,  744,  415, 1580,  458,  457, 1579,  456, 1542,  455,
			 1549,  355,  354,  577, 1006, 1802, 1532,  353, 1547, 1527,
			  452,  451, 1523, 1521, 1520,  809, 2222, 1525, 1513, 1501,
			 1518, 2226, 1517, 1515,  450, 2364, 1514, 1511,  639,  640,
			 1510,  352, 1509, 1507, 1275,  697,  697, 1489, 1487, 1494,
			 1484, 1415, 1477, 1476, 2024, 1645, 1463, 2242, 1480, 1386,
			 1457,  351,  458,  457, -573,  456, 1454,  455, 1187, 2203,
			 2255,  668,  350, 2258, 1452, 1451, 1453, 1450,  452,  451,
			 1447, 1446, -574, 1443, 1425,  349, 1424, 1417,  809,  348,
			 1414,  347,  450, 2269, 1719, 1407, 1402, 1010,  862,  805,

			  346, 1381,  345,  470,  415,  344,  704, 1380, 1365,  343,
			 1361, 1354, 1369,  717, 1889, 1329,  342,  721,  723,  724,
			  726,  727,  809,  341,  544, 2025,  544,  740, 1350,  537,
			 1342,  537,  724,  435, 1341,  508,  751, 1334, 1318, 2297,
			  549, 1327, 1319, 1307, 1257, 1274, 1254, 1253, 1251,  762,
			  763,  764, 1250, 1248,  765,  766,  767,  768,  769,  770,
			 2314, 2315,  791,  809, 1247,  633,  549,  550,  550,  550,
			 1246, 1245, 2206, 1243, 1242, 1187, 1758, 1187, 1244, 1328,
			 1032, 1241, 1239,  818, 1220,  632,  809, 1236, 1219, 1215,
			 1734, 1209, 1736,  631, 1202, 1198, 2334, 1197, 1178,  727,

			 1572,  458,  457, 1571,  456, 1196,  455, 1195, 1769, 1194,
			 1193, 1192,  460, 1191,  846,  848, 1190,  452,  451, 1189,
			 1719,  630,  629, 1169,  628, 1180, 1179,  548,  548,  548,
			 1150,  450, 1147,  627, 1145, 1144, 1135, 2361, 2362,  749,
			  458,  457,  748,  456, 1143,  455, 2369, 2370, 1123, 2050,
			  458,  457,  626,  456,  485,  455,  452,  451, 1808, 1809,
			 1111, 1109, 1102, 1769, 1108, 1098,  452,  451, 1097,  809,
			  450,  884, 1107,  886,  887,  888, 1094,  809,  258, 1096,
			  450, 1093, 1085, 1082, 1083,  625, 1106, 1081,  450, 1040,
			  749,  458,  457,  748,  456, 1047,  455, 1031, 1026, 1017,

			 1009,  624, 1008,  747,  454,  453,  704,  452,  451, 1005,
			 2022,  458,  457, 1883,  456,  623,  455,  986,  985,  974,
			  978,  450,  958,  622,  621,  959,  961,  452,  451, 2306,
			  955, 1982,  458,  457,  953,  456,  954,  455, 1981,  458,
			  457,  450,  456, 1912,  455,  945,  952,  944,  452,  451,
			  941,  746,  940,  620,  933,  452,  451,  932,  931,  951,
			  930, 1925,  450,  928,  924,  923,  929,  922,  921,  450,
			  721,  920,  721,  721,  721,  919,  979,  721,  982,  619,
			  721,  918,  721,  917,  912,  907,  906, 1827,  458,  457,
			  704,  456,  607,  455,  901,  900,  898,  618,  897,  885,

			  905,  872,  617,  616,  452,  451, 1296,  874,  870,  869,
			 1187, 1758, 1187,  867, 2306, -612,  866,  865,  450,  864,
			  861,  615,  614,  859,  858,  856,  851, 1036, 1037,  854,
			  836,  835,  463,  828,  741,  814,  850, 1041, 1042,  813,
			  795,  745,  822, 1046,  734, 1049,  719,  721,  718,  769,
			  724,  724,  714,  704, 1729,  458,  457,  512,  456,  333,
			  455,  708,  690,  689,  679,  744,  678,  677,  676,  675,
			  674,  452,  451, 1075,  673, 1987, 1989,  672,  549,  549,
			  549, 1077,  671,  670,  669,  450, 1727,  458,  457,  666,
			  456, 1889,  455, 2011, 2012,  667,  660,  663,  657,  656,

			  592,  573,  264,  452,  451, 1577,  458,  457,  586,  456,
			  570,  455, 1101, 1925, 1925,  298,  567,  450,  568,  495,
			  566,  565,  452,  451, 1411,  458,  457,  564,  456,  497,
			  455, 1326,  458,  457,  480,  456,  450,  455,  492,  490,
			  488,  452,  451,  491,  469,  487,  476,  478,  452,  451,
			  467, -629,  464,  428,  704,  450,  426,  372, 1138,  419,
			  418,  417,  450,  414,  413,  404, 1140,  392, 1325,  458,
			  457,  387,  456,  386,  455,  370,  368,  340,  339,  313,
			  304,  332,  291,  287, 1154,  452,  451,  263, 1155, 1156,
			 1157, 1158, 1159, 1160, 1161, 1162, 1163,  577,  262,  450,

			  269, 2122,   -2, 1187, 1043,  458,  457,  195,  456,    3,
			  455,  968,  458,  457,  261,  456, 1716,  455, 1496, 1956,
			 1939,  452,  451, 2277,  760, 2278, 1553, 1909,  452,  451,
			  367, 1888, 1714, 2393, 2366,  450, 2363, 2174, 2316,  938,
			  458,  457,  450,  456, 2238,  455, 1491,  108,  937,  458,
			  457, 1631,  456, 2005,  455, 2143,  452,  451, 1224, 2004,
			 1226,  107, 1717,  105, 2324,  452,  451, 2002, 1927, 1435,
			  450, 1039,  733, 2214,  103, 1287,  101, 1286,  100,  450,
			 2166,  504, 1164,  302,  234, 1294, 1588, 1788, 1925, 2129,
			 1807,   94, 1605,   92, 1984, 1409, 1626, 1368, 1967,   90,

			 1377, 1279,   87, 1853, 1285,   86, 1289, 2149, 1291,  914,
			   84,   83,  913,  184,  183, 1306,  177,  176,  174, 1481,
			  173,  172,  458,  457,   78,  456, 1316,  455, 2197, 1284,
			 1320, 1321, 1733,   74, 1323,  454,  453, 1965,  452,  451,
			  458,  457, 1726,  456,  309,  455,  721,  686, 2268, 1333,
			  827, 1336,  450, 1915, 2120,   68,  452,  451,   67, 1871,
			   66,  664, 2304, 2301, 2042,   65,   64, 1187, 2254, 2338,
			  450, 2280,  458,  457,  403,  456, 1649,  455, 1216,   60,
			  402,  139,  137, 1590,   59,   58,  135,   56,  452,  451,
			  134,  133,  132,  130, 2404,   55,    0, 1367,   54,    0,

			    0,   52,  450,   51,    0,    0,    0,  201,   48,    0,
			    0,    0,    0,    0, 1889,    0,   47,   46,    0,    0,
			    0,   45,    0,    0,    0,    0,    0,  727,    0,   44,
			 1187,    0,  458,  457, 1371,  456,    0,  455,    0,    0,
			   42,   41,   40,    0,    0,   39,    0,   38,  452,  451,
			    0,    0,  458,  457,    0,  456,   35,  455,    0,    0,
			 1412,   34,  450,    0,    0,    0,    0,   33,  452,  451,
			    0,   31,  724,   29,    0,    0,    0,   27,    0,   26,
			   25,    0,  450,    0,    0,    0,    0,   22,    0,   21,
			  458,  457,    0,  456,    0,  455,   20,    0,   19,    0, yyDummy>>,
			1, 1000, 2000)
		end

	yytable_template_4 (an_array: ARRAY [INTEGER])
			-- Fill chunk #4 of template for `yytable'.
		do
			yyarray_subcopy (an_array, <<
			   17, 1332,   16,   15,    0, 2317,  452,  451,    0,    0,
			    0,   13,    0,    0,   11,    0,  458,  457,   10,  456,
			  450,  455,    0,    7, 1400,    0,    0,    6, 1187,    0,
			    0,    0,  452,  451,    0,    0, 1383,  458,  457,    0,
			  456,    0,  455, 1508,    0,  109,  450, 1512,    0,    0,
			    0, 1516,    0,  452,  451,    0,    0,    0,    0,  458,
			  457,    0,  456, 1357,  455,    0,  108,  450,    0,    0,
			    0,    0,    0, 1538,    0,  452,  451,    0,    0,    0,
			  107,  106,  105,    0,    0, 1355,  104,    0,    0,  450,
			    0,    0,    0,  103,  102,  101,    0,  100,   99,    0,

			   98,   97,   96,   95,    0,    0,    0,    0,    0,    0,
			   94,   93,   92,    0,    0,    0,    0,   91,   90,   89,
			   88,   87,    0, 1583,   86,    0,    0,   85,    0,   84,
			   83, 1587,    0,  721,    0,   82,    0,    0,    0,   81,
			   80,   79,    0,   78,    0,    0,   77,    0,   76,    0,
			   75,    0,   74,   73,    0,    0,    0,   72,    0,  458,
			  457, 1249,  456,    0,  455,    0,    0,   71,  724,    0,
			 1617,    0,   70,   69,   68,  452,  451,   67,    0,   66,
			    0,    0,    0,    0,   65,   64,    0,    0,   63,  450,
			   62,    0,    0,    0,    0,  724,   61,    0,   60,    0,

			    0,    0,    0,   59,   58,   57,   56,    0,    0,    0,
			    0,    0,    0,    0,   55,    0,  724,   54,    0,   53,
			   52,    0,   51,   50,    0,    0,   49,   48,  458,  457,
			    0,  456, 1654,  455, 1124,   47,   46,    0,  458,  457,
			   45,  456,    0,  455,  452,  451, 1080,    0,   44,    0,
			    0,    0,    0,   43,  452,  451,    0,    0,  450,   42,
			   41,   40,    0,    0,   39,    0,   38,    0,  450,    0,
			   37,    0,    0,    0,   36,   35,    0,    0,    0,    0,
			   34,    0,    0,    0,    0,    0,   33,    0,    0,   32,
			   31,   30,   29,   28,    0,    0,   27,    0,   26,   25,

			   24,   23,    0,    0,    0,    0,   22,    0,   21,    0,
			    0,    0,    0,    0,    0,   20,    0,   19,   18,   17,
			    0,   16,   15,    0,    0,    0,    0,   14,    0,    0,
			   13,    0,   12,   11,    0,    0,    0,   10,    9,    8,
			    0,    0,    7,    0,  458,  457,    6,  456, 1776,  455,
			    0,    5, 1066,    0,    0,    4,    0,    0,    0,    0,
			  452,  451,    0,  458,  457, 1027,  456,    0,  455,    0,
			 1798,    0,    0,    0,  450,    0,    0,    0, 1801,  452,
			  451,  458,  457,    0,  456,    0,  455,  894,  893, 1806,
			  892,    0,  891,  450,    0,    0,    0,  452,  451,    0,

			    0,    0,    0,  890,  889,    0,    0,    0,    0,    0,
			    0,  450,    0,    0,    0,    0,    0,  450,    0,    0,
			    0, 1831,    0,    0,    0,    0, 1840,  704,    0,    0,
			    0,    0,    0,    0, 1846,    0,    0,    0,    0, -780,
			    0,    0, 1850, -780, -780, -780, -780, -780, 1598, -780,
			 -780, -780, -780,    0, -780, -780, -780, -780, -780,    0,
			 -780, -780,    0, -780,    0,    0,    0,    0,    0, 1881,
			 -780,  724, -1071,    0, -780,    0,    0, -780, -780, -780,
			    0,    0,    0, -780,    0,    0,    0,    0, -780, -780,
			 -780,    0, -780,    0, -780,    0,    0, -780,    0,    0,

			    0,    0,    0, -780, -780,    0,    0, -780,    0, -780,
			    0,    0,    0,    0,    0, -780,    0,    0, -780,    0,
			    0, -780,    0,    0,    0, 1929, -780, -780,    0,    0,
			    0,    0, -780, -780,    0,    0,    0,    0,    0,    0,
			 -780,    0,    0,    0,    0,    0,    0, -780, -780, -780,
			    0,    0, -780,    0, -780,    0,    0, -780,    0,    0,
			 -780, -780, -780,    0,    0,    0, 1538,    0,    0,    0,
			    0, -780,    0, -780, -780,    0, -780,    0, -780, -780,
			    0, -780, -780,    0,    0, -780, -780,    0,    0,    0,
			 -780,    0, 1538,  724, -780, -780,    0,    0,    0, -780,

			 -780, -780,    0, -780,    0,    0,    0, -780,    0, -780,
			 -780, -780, -780,    0, -780,    0, -780, -780, -780, -780,
			    0,    0,    0, -780, -780,    0,    0,    0,    0,    0,
			    0,    0, -780, -780, -780,    0,    0, -780,    0,    0,
			    0,    0,    0,    0,    0, -780,    0,    0,    0,    0,
			    0, -780, -780,    0,    0,    0, -780, -780, -780, -780,
			    0, -780,    0, -780,    0,    0,    0,    0,    0,    0,
			 -780,    0, -780,    0,    0,    0,    0, -780,    0,    0,
			    0,    0,    0, -780,    0,    0, -780, -780, -780, -780,
			    0,  704,    0, -780,    0,    0, -780, -780,    0,    0,

			    0,    0,    0, -780,    0, -780,    0,    0,    0,    0,
			    0,    0, -780,    0, -780,    0, -780,    0, -780, -780,
			 -780,    0,    0,    0,    0,    0,    0, -780,    0,    0,
			 -780,    0, -780, 2093, -780,    0,    0,    0,    0, -780,
			    0,    0,    0, -780,    0, -780,    0, -780,    0, -780,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			 2144,    0,    0,    0,    0,    0,    0,    0,    0,    0,

			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,    0, 1831,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,  724,    0, -779,    0,    0,    0, -779, -779,
			 -779, -779, -779, 1065, -779, -779, -779, -779,    0, -779,
			 -779, -779, -779, -779,    0, -779, -779,    0, -779,    0,
			    0,    0,    0,    0,    0, -779,    0, -1070,    0, -779,
			    0,    0, -779, -779, -779,    0,    0,    0, -779,    0,
			    0,    0,    0, -779, -779, -779,    0, -779,    0, -779,

			    0,    0, -779,    0,    0,    0,    0,    0, -779, -779,
			    0,    0, -779,    0, -779,    0,    0,    0,    0,    0,
			 -779,    0,    0, -779,    0,    0, -779,    0,    0,    0,
			  724, -779, -779,    0, 1538, 1538,    0, -779, -779,    0,
			    0,    0,    0,    0,    0, -779,    0,    0,    0,    0,
			    0,    0, -779, -779, -779,    0,    0, -779,    0, -779,
			    0,    0, -779,    0,    0, -779, -779, -779,    0,    0,
			    0,    0,    0,    0,    0,    0, -779,    0, -779, -779,
			    0, -779,    0, -779, -779,    0, -779, -779,    0,    0,
			 -779, -779,    0,    0,    0, -779,    0,    0,    0, -779, yyDummy>>,
			1, 1000, 3000)
		end

	yytable_template_5 (an_array: ARRAY [INTEGER])
			-- Fill chunk #5 of template for `yytable'.
		do
			yyarray_subcopy (an_array, <<
			 -779, 2274,    0,    0, -779, -779, -779,    0, -779,    0,
			    0,    0, -779,    0, -779, -779, -779, -779,    0, -779,
			    0, -779, -779, -779, -779,    0,    0,    0, -779, -779,
			    0,    0,    0,    0,    0,    0,    0, -779, -779, -779,
			    0,    0, -779,    0,    0,    0,    0,    0,    0,    0,
			 -779,    0,    0,    0,    0, 2305, -779, -779,    0,    0,
			    0, -779, -779, -779, -779,    0, -779,    0, -779,    0,
			    0,    0,    0,    0, 2325, -779,    0, -779,    0,    0,
			    0,    0, -779,    0,    0,    0,    0,    0, -779,    0,
			    0, -779, -779, -779, -779,    0,    0,    0, -779,    0,

			    0, -779, -779,    0,    0,    0,    0,    0, -779,    0,
			 -779,    0,    0,    0,    0,    0,    0, -779,    0, -779,
			    0, -779,    0, -779, -779, -779,    0,    0,    0,    0,
			    0,    0, -779,    0,    0, -779,    0, -779,    0, -779,
			 2305,    0,  724,    0, -779,    0,    0,    0, -779,    0,
			 -779,    0, -779, -777, -779,    0, -653, -777, -777, -777,
			 -777, -777,  449, -777, -777, -777, -777,    0, -777, -777,
			 -777, -777, -777,    0, -777, -777,    0, -777,    0,    0,
			    0,    0,    0,    0, -777,    0,    0,    0, -777,    0,
			    0, -777, -777, -777,    0,    0,    0, -777,    0,    0,

			    0,    0, -777, -777, -777,    0, -777,    0, -777,    0,
			    0, -777,    0,    0,    0,    0,    0, -777, -777,    0,
			    0, -777,    0, -777,    0,    0,    0,    0,    0, -777,
			    0,    0, -777,    0,    0, -777,    0,    0,    0,    0,
			 -777, -777,    0,    0,    0,    0, -777, -777,    0,    0,
			    0,    0,    0,    0, -777,    0,    0,    0,    0,    0,
			    0, -777, -777, -777,    0,    0, -777,    0, -777,    0,
			    0, -777,    0,    0, -777, -777, -777,    0,    0,    0,
			    0,    0,    0,    0,    0, -777,    0, -777, -777,    0,
			 -777,    0, -777, -777,    0, -777, -777,    0,    0, -777,

			 -777,    0,    0,    0, -777,    0,    0,    0, -777, -777,
			    0,    0,    0, -777, -777, -777,    0, -777,    0,    0,
			    0, -777,    0, -777, -777, -777, -777,    0, -777,    0,
			 -777, -777, -777, -777,    0,    0,    0, -777, -777,    0,
			    0,    0,    0,    0,    0,    0, -777, -777, -777,    0,
			    0, -777,    0,    0,    0,    0,    0,    0,    0, -777,
			    0,    0,    0,    0,    0, -777, -777,    0,    0,    0,
			 -777, -777, -777, -777,    0, -777,    0, -777,    0,    0,
			    0,    0,    0,    0, -777,    0, -777,    0,    0,    0,
			    0, -777,    0,    0,    0,    0,    0, -777,    0,    0,

			 -777, -777, -777, -777,    0,    0,    0, -777,    0,    0,
			 -777, -777,    0,    0,    0,    0,    0, -777,    0, -777,
			    0,    0,    0,    0,    0,    0, -777,    0, -777,    0,
			 -777,    0, -777, -777, -777,    0,    0,    0,    0,    0,
			    0, -777,    0,    0, -777,    0, -777,    0, -777,    0,
			    0,    0,    0, -777,    0,    0,    0, -777,    0, -777,
			 -781, -777,    0, -777, -781, -781, -781, -781, -781, 1990,
			 -781, -781, -781, -781,    0, -781, -781, -781, -781, -781,
			    0, -781, -781,    0, -781,    0,    0,    0,    0,    0,
			    0, -781,    0,    0,    0, -781,    0,    0, -781, -781,

			 -781,    0,    0,    0, -781,    0,    0,    0,    0, -781,
			 -781, -781,    0, -781,    0, -781,    0,    0, -781,    0,
			    0,    0,    0,    0, -781, -781,    0,    0, -781,    0,
			 -781,    0,    0,    0,    0,    0, -781,    0,    0, -781,
			    0,    0, -781,    0,    0,    0,    0, -781, -781,    0,
			    0,    0,    0, -781, -781,    0,    0,    0,    0,    0,
			    0, -781,    0,    0,    0,    0,    0,    0, -781, -781,
			 -781,    0,    0, -781,    0, -781,    0,    0, -781,    0,
			    0, -781, -781, -781,    0,    0,    0,    0,    0,    0,
			    0,    0, -781,    0, -781, -781,    0, -781,    0, -781,

			 -781,    0, -781, -781,    0,    0, -781, -781,    0,    0,
			    0, -781,    0,    0,    0, -781, -781,    0,    0,    0,
			 -781, -781, -781,    0, -781,    0,    0,    0, -781,    0,
			 -781, -781, -781, -781,    0, -781,    0, -781, -781, -781,
			 -781,    0,    0,    0, -781, -781,    0,    0,    0,    0,
			    0,    0,    0, -781, -781, -781,    0,    0, -781,    0,
			    0,    0,    0,    0,    0,    0, -781,    0,    0,    0,
			    0,    0, -781, -781,    0,    0,    0, -781, -781, -781,
			 -781,    0, -781,    0, -781,    0,    0,    0,    0,    0,
			    0, -781,    0, -781,    0,    0,    0,    0, -781,    0,

			    0,    0,    0,    0, -781,    0,    0, -781, -781, -781,
			 -781,    0,    0,    0, -781,    0,    0, -781, -781,    0,
			    0,    0,    0,    0, -781,    0, -781,    0,    0,    0,
			    0,    0,    0, -781,    0, -781,    0, -781,    0, -781,
			 -781, -781,    0,    0,    0,    0,    0,    0, -781,    0,
			    0, -781,    0, -781,    0, -781,    0,    0,    0,    0,
			 -781,    0,    0,    0, -781,    0, -781, -782, -781,    0,
			 -781, -782, -782, -782, -782, -782, 1810, -782, -782, -782,
			 -782,    0, -782, -782, -782, -782, -782,    0, -782, -782,
			    0, -782,    0,    0,    0,    0,    0,    0, -782,    0,

			    0,    0, -782,    0,    0, -782, -782, -782,    0,    0,
			    0, -782,    0,    0,    0,    0, -782, -782, -782,    0,
			 -782,    0, -782,    0,    0, -782,    0,    0,    0,    0,
			    0, -782, -782,    0,    0, -782,    0, -782,    0,    0,
			    0,    0,    0, -782,    0,    0, -782,    0,    0, -782,
			    0,    0,    0,    0, -782, -782,    0,    0,    0,    0,
			 -782, -782,    0,    0,    0,    0,    0,    0, -782,    0,
			    0,    0,    0,    0,    0, -782, -782, -782,    0,    0,
			 -782,    0, -782,    0,    0, -782,    0,    0, -782, -782,
			 -782,    0,    0,    0,    0,    0,    0,    0,    0, -782,

			    0, -782, -782,    0, -782,    0, -782, -782,    0, -782,
			 -782,    0,    0, -782, -782,    0,    0,    0, -782,    0,
			    0,    0, -782, -782,    0,    0,    0, -782, -782, -782,
			    0, -782,    0,    0,    0, -782,    0, -782, -782, -782,
			 -782,    0, -782,    0, -782, -782, -782, -782,    0,    0,
			    0, -782, -782,    0,    0,    0,    0,    0,    0,    0,
			 -782, -782, -782,    0,    0, -782,    0,    0,    0,    0,
			    0,    0,    0, -782,    0,    0,    0,    0,    0, -782,
			 -782,    0,    0,    0, -782, -782, -782, -782,    0, -782,
			    0, -782,    0,    0,    0,    0,    0,    0, -782,    0, yyDummy>>,
			1, 1000, 4000)
		end

	yytable_template_6 (an_array: ARRAY [INTEGER])
			-- Fill chunk #6 of template for `yytable'.
		do
			yyarray_subcopy (an_array, <<
			 -782,    0,    0,    0,    0, -782,    0,    0,    0,    0,
			    0, -782,    0,    0, -782, -782, -782, -782,    0,    0,
			    0, -782,    0,    0, -782, -782,    0,    0,    0,    0,
			    0, -782,    0, -782,    0,    0,    0,    0,    0,    0,
			 -782,    0, -782,    0, -782,    0, -782, -782, -782,    0,
			    0,    0,    0,    0,    0, -782,    0,    0, -782,    0,
			 -782,    0, -782,    0,    0,    0,    0, -782,    0,    0,
			    0, -782,    0, -782, -1176, -782,    0, -782, -1176, -1176,
			 -1176, -1176, -1176,    0, -1176, -1176, -1176, -1176,    0, -1176,
			 -1176, -1176, -1176, -1176,    0, -1176, -1176,    0, -1176,    0,

			    0,    0,    0,    0,    0, -1176,    0,    0,    0, -1176,
			    0,    0, -1176, -1176, -1176,    0,    0,    0, -1176,    0,
			    0,    0,    0, -1176, -1176, -1176,    0, -1176,    0, -1176,
			    0,    0, -1176,    0,    0,    0,    0,    0, -1176, -1176,
			    0,    0, -1176,    0, -1176,    0,    0,    0,    0,    0,
			 -1176,    0,    0, -1176,    0,    0, -1176,    0,    0,    0,
			    0, -1176, -1176,    0,    0,    0,    0, -1176, -1176,    0,
			    0,    0,    0,    0,    0, -1176,    0,    0,    0,    0,
			    0,    0, -1176, -1176, -1176,    0,    0, -1176,    0, -1176,
			    0,    0, -1176,    0,    0, -1176, -1176, -1176,    0,    0,

			    0,    0,    0,    0,    0,    0, -1176,    0, -1176, -1176,
			    0, -1176,    0, -1176, -1176,    0, -1176, -1176,    0,    0,
			 -1176, -1176,    0,    0,    0, -1176,    0,    0,    0, -1176,
			 -1176,    0,    0,    0, -1176, -1176, -1176,    0, -1176,    0,
			    0,    0, -1176,    0, -1176, -1176, -1176, -1176,    0, -1176,
			    0, -1176, -1176, -1176, -1176,    0,    0,    0, -1176, -1176,
			    0,    0,    0,    0,    0,    0,    0, -1176, -1176, -1176,
			    0,    0, -1176,    0,    0,    0,    0,    0,    0,    0,
			 -1176,    0,    0,    0,    0,    0, -1176, -1176,    0,    0,
			    0, -1176, -1176, -1176, -1176,    0, -1176,    0, -1176,    0,

			    0,    0,    0,    0,    0, -1176,    0, -1176,    0,    0,
			    0,    0, -1176,    0,    0,    0,    0,    0, -1176,    0,
			    0, -1176, -1176, -1176, -1176,    0,    0,    0, -1176,    0,
			    0, -1176, -1176,    0,    0,    0,    0,    0, -1176,    0,
			 -1176,    0,    0,    0,    0,    0,    0, -1176,    0, -1176,
			    0, -1176,    0, -1176, -1176, -1176,    0,    0,    0,    0,
			    0,    0, -1176,    0,    0, -1176,    0, -1176,    0, -1176,
			    0,    0,    0,    0, -1176,    0,    0,    0, -1176,    0,
			 -1176, -600, -1176,    0, -1176, -600, -600, -600, -600, -600,
			    0, -600, -600, -600, -600,    0, -600, -600, -600, -600,

			 -600,    0, -600, -600,    0, -600,    0,    0,    0,    0,
			    0,    0, -600,    0,    0,    0, -600,    0,    0, -600,
			 -600, -600,    0,    0,    0, -600,    0,    0,    0,    0,
			 -600, -600, -600,    0, -600,    0, -600,    0,    0, -600,
			    0,    0,    0,    0,    0, -600, -600,    0,    0, -600,
			    0, -600,    0,    0,    0,    0,    0, -600,    0,    0,
			 -600,    0,    0, -600,    0,    0,    0,    0, -600, -600,
			    0,    0,    0,    0, -600, -600,    0,    0,    0,    0,
			    0,    0, -600,    0,    0,    0,    0,    0,    0, -600,
			 -600, -600,    0,    0, -600,    0, -600,    0,    0, -600,

			    0,    0, -600, -600, -600,    0,    0,    0,    0,    0,
			    0,    0,    0, -600,    0, -600, -600,    0, -600,    0,
			 -600, -600,    0, -600, -600,    0,    0, -600, -600,    0,
			    0,    0, -600,    0,    0,    0, -600, -600,    0,    0,
			    0, -600, -600, -600,    0, -600,    0,    0,    0, -600,
			    0, -600, -600, -600, -600,    0, -600,    0, -600, -600,
			 -600, -600,    0,    0,    0, -600, -600,    0,    0,    0,
			    0,    0,    0,    0, -600, -600, -600,    0,    0, -600,
			    0,    0,    0,    0,    0,    0,    0, -600,    0,    0,
			    0,    0,    0, -600, -600,    0,    0,    0, -600, -600,

			 -600, -600,    0, -600,    0, -600,    0,    0,    0,    0,
			    0,    0, -600,    0, -600,    0,    0,    0,    0, -600,
			    0,    0,    0,    0,    0, -600,    0,    0, -600, -600,
			 -600, -600,    0,    0,    0, -600,    0,    0, -600, -600,
			    0,    0,    0,    0,    0, -600,    0, -600,    0,    0,
			    0,    0,    0,    0, -600,    0, -600,    0, -600,    0,
			 -600, -600, -600,    0,    0,    0,    0,    0,    0, -600,
			    0,    0, -600,    0, -600,    0, -600,    0,    0,    0,
			    0, -600,    0,    0,    0, -600,    0, -600, -599, -600,
			    0, -600, -599, -599, -599, -599, -599,    0, -599, -599,

			 -599, -599,    0, -599, -599, -599, -599, -599,    0, -599,
			 -599,    0, -599,    0,    0,    0,    0,    0,    0, -599,
			    0,    0,    0, -599,    0,    0, -599, -599, -599,    0,
			    0,    0, -599,    0,    0,    0,    0, -599, -599, -599,
			    0, -599,    0, -599,    0,    0, -599,    0,    0,    0,
			    0,    0, -599, -599,    0,    0, -599,    0, -599,    0,
			    0,    0,    0,    0, -599,    0,    0, -599,    0,    0,
			 -599,    0,    0,    0,    0, -599, -599,    0,    0,    0,
			    0, -599, -599,    0,    0,    0,    0,    0,    0, -599,
			    0,    0,    0,    0,    0,    0, -599, -599, -599,    0,

			    0, -599,    0, -599,    0,    0, -599,    0,    0, -599,
			 -599, -599,    0,    0,    0,    0,    0,    0,    0,    0,
			 -599,    0, -599, -599,    0, -599,    0, -599, -599,    0,
			 -599, -599,    0,    0, -599, -599,    0,    0,    0, -599,
			    0,    0,    0, -599, -599,    0,    0,    0, -599, -599,
			 -599,    0, -599,    0,    0,    0, -599,    0, -599, -599,
			 -599, -599,    0, -599,    0, -599, -599, -599, -599,    0,
			    0,    0, -599, -599,    0,    0,    0,    0,    0,    0,
			    0, -599, -599, -599,    0,    0, -599,    0,    0,    0,
			    0,    0,    0,    0, -599,    0,    0,    0,    0,    0,

			 -599, -599,    0,    0,    0, -599, -599, -599, -599,    0,
			 -599,    0, -599,    0,    0,    0,    0,    0,    0, -599,
			    0, -599,    0,    0,    0,    0, -599,    0,    0,    0,
			    0,    0, -599,    0,    0, -599, -599, -599, -599,    0,
			    0,    0, -599,    0,    0, -599, -599,    0,    0,    0,
			    0,    0, -599,    0, -599,    0,    0,    0,    0,    0,
			    0, -599,    0, -599,    0, -599,    0, -599, -599, -599,
			    0,    0,    0,    0,    0,    0, -599,    0,    0, -599,
			    0, -599,    0, -599,    0,    0,    0,    0, -599,    0,
			    0,    0, -599,    0, -599, -611, -599,    0, -599, -611, yyDummy>>,
			1, 1000, 5000)
		end

	yytable_template_7 (an_array: ARRAY [INTEGER])
			-- Fill chunk #7 of template for `yytable'.
		do
			yyarray_subcopy (an_array, <<
			 -611, -611, -611, -611,    0, -611, -611, -611, -611,    0,
			 -611, -611, -611, -611, -611,    0, -611, -611,    0, -611,
			    0,    0,    0,    0,    0,    0, -611,    0,    0,    0,
			 -611,    0,    0, -611, -611, -611,    0,    0,    0, -611,
			    0,    0,    0,    0, -611, -611, -611,    0, -611,    0,
			 -611,    0,    0, -611,    0,    0,    0,    0,    0, -611,
			 -611,    0,    0, -611,    0, -611,    0,    0,    0,    0,
			    0, -611,    0,    0, -611,    0,    0, -611,    0,    0,
			    0,    0, -611, -611,    0,    0,    0,    0, -611, -611,
			    0,    0,    0,    0,    0,    0, -611,    0,    0,    0,

			    0,    0,    0, -611, -611, -611,    0,    0, -611,    0,
			 -611,    0,    0, -611,    0,    0, -611, -611, -611,    0,
			    0,    0,    0,    0,    0,    0,    0, -611,    0, -611,
			 -611,    0, -611,    0, -611, -611,    0, -611, -611,    0,
			    0, -611, -611,    0,    0,    0, -611,    0,    0,    0,
			 -611, -611,    0,    0,    0, -611, -611, -611,    0, -611,
			    0,    0,    0, -611,    0, -611, -611, -611, -611,    0,
			 -611,    0, -611, -611, -611, -611,    0,    0,    0, -611,
			 -611,    0,    0,    0,    0,    0,    0,    0, -611, -611,
			 -611,    0,    0, -611,    0,    0,    0,    0,    0,    0,

			    0, -611,    0,    0,    0,    0,    0, -611, -611,    0,
			    0,    0, -611, -611, -611, -611,    0, -611,    0, -611,
			    0,    0,    0,    0,    0,    0, -611,    0, -611,    0,
			    0,    0,    0, -611,    0,    0,    0,    0,    0, -611,
			    0,    0, -611, -611, -611, -611,    0,    0,    0, -611,
			    0,    0, -611, -611,    0,    0,    0,    0,    0, -611,
			    0, -611,    0,    0,    0,    0,    0,    0, -611,    0,
			 -611,    0, -611,    0, -611, -611, -611,    0,    0,    0,
			    0,    0,    0, -611,    0,    0, -611,    0, -611,    0,
			 -611,    0,    0,    0,    0, -611,    0,    0,    0, -611,

			    0, -611, -609, -611,    0, -611, -609, -609, -609, -609,
			 -609,    0, -609, -609, -609, -609,    0, -609, -609, -609,
			 -609, -609,    0, -609, -609,    0, -609,    0,    0,    0,
			    0,    0,    0, -609,    0,    0,    0, -609,    0,    0,
			 -609, -609, -609,    0,    0,    0, -609,    0,    0,    0,
			    0, -609, -609, -609,    0, -609,    0, -609,    0,    0,
			 -609,    0,    0,    0,    0,    0, -609, -609,    0,    0,
			 -609,    0, -609,    0,    0,    0,    0,    0, -609,    0,
			    0, -609,    0,    0, -609,    0,    0,    0,    0, -609,
			 -609,    0,    0,    0,    0, -609, -609,    0,    0,    0,

			    0,    0,    0, -609,    0,    0,    0,    0,    0,    0,
			 -609, -609, -609,    0,    0, -609,    0, -609,    0,    0,
			 -609,    0,    0, -609, -609, -609,    0,    0,    0,    0,
			    0,    0,    0,    0, -609,    0, -609, -609,    0, -609,
			    0, -609, -609,    0, -609, -609,    0,    0, -609, -609,
			    0,    0,    0, -609,    0,    0,    0, -609, -609,    0,
			    0,    0, -609, -609, -609,    0, -609,    0,    0,    0,
			 -609,    0, -609, -609, -609, -609,    0, -609,    0, -609,
			 -609, -609, -609,    0,    0,    0, -609, -609,    0,    0,
			    0,    0,    0,    0,    0, -609, -609, -609,    0,    0,

			 -609,    0,    0,    0,    0,    0,    0,    0, -609,    0,
			    0,    0,    0,    0, -609, -609,    0,    0,    0, -609,
			 -609, -609, -609,    0, -609,    0, -609,    0,    0,    0,
			    0,    0,    0, -609,    0, -609,    0,    0,    0,    0,
			 -609,    0,    0,    0,    0,    0, -609,    0,    0, -609,
			 -609, -609, -609,    0,    0,    0, -609,    0,    0, -609,
			 -609,    0,    0,    0,    0,    0, -609,    0, -609,    0,
			    0,    0,    0,    0,    0, -609,    0, -609,    0, -609,
			    0, -609, -609, -609,    0,    0,    0,    0,    0,    0,
			 -609,    0,    0, -609,    0, -609,    0, -609,    0,    0,

			    0,    0, -609,    0,    0,    0, -609,    0, -609, -608,
			 -609,    0, -609, -608, -608, -608, -608, -608,    0, -608,
			 -608, -608, -608,    0, -608, -608, -608, -608, -608,    0,
			 -608, -608,    0, -608,    0,    0,    0,    0,    0,    0,
			 -608,    0,    0,    0, -608,    0,    0, -608, -608, -608,
			    0,    0,    0, -608,    0,    0,    0,    0, -608, -608,
			 -608,    0, -608,    0, -608,    0,    0, -608,    0,    0,
			    0,    0,    0, -608, -608,    0,    0, -608,    0, -608,
			    0,    0,    0,    0,    0, -608,    0,    0, -608,    0,
			    0, -608,    0,    0,    0,    0, -608, -608,    0,    0,

			    0,    0, -608, -608,    0,    0,    0,    0,    0,    0,
			 -608,    0,    0,    0,    0,    0,    0, -608, -608, -608,
			    0,    0, -608,    0, -608,    0,    0, -608,    0,    0,
			 -608, -608, -608,    0,    0,    0,    0,    0,    0,    0,
			    0, -608,    0, -608, -608,    0, -608,    0, -608, -608,
			    0, -608, -608,    0,    0, -608, -608,    0,    0,    0,
			 -608,    0,    0,    0, -608, -608,    0,    0,    0, -608,
			 -608, -608,    0, -608,    0,    0,    0, -608,    0, -608,
			 -608, -608, -608,    0, -608,    0, -608, -608, -608, -608,
			    0,    0,    0, -608, -608,    0,    0,    0,    0,    0,

			    0,    0, -608, -608, -608,    0,    0, -608,    0,    0,
			    0,    0,    0,    0,    0, -608,    0,    0,    0,    0,
			    0, -608, -608,    0,    0,    0, -608, -608, -608, -608,
			    0, -608,    0, -608,    0,    0,    0,    0,    0,    0,
			 -608,    0, -608,    0,    0,    0,    0, -608,    0,    0,
			    0,    0,    0, -608,    0,    0, -608, -608, -608, -608,
			    0,    0,    0, -608,    0,    0, -608, -608,    0,    0,
			    0,    0,    0, -608,    0, -608,    0,    0,    0,    0,
			    0,    0, -608,    0, -608,    0, -608,    0, -608, -608,
			 -608,    0,    0,    0,    0,    0,    0, -608,    0,    0,

			 -608,    0, -608,    0, -608,    0,    0,    0,    0, -608,
			    0,    0,    0, -608,    0, -608, -607, -608,    0, -608,
			 -607, -607, -607, -607, -607,    0, -607, -607, -607, -607,
			    0, -607, -607, -607, -607, -607,    0, -607, -607,    0,
			 -607,    0,    0,    0,    0,    0,    0, -607,    0,    0,
			    0, -607,    0,    0, -607, -607, -607,    0,    0,    0,
			 -607,    0,    0,    0,    0, -607, -607, -607,    0, -607,
			    0, -607,    0,    0, -607,    0,    0,    0,    0,    0,
			 -607, -607,    0,    0, -607,    0, -607,    0,    0,    0,
			    0,    0, -607,    0,    0, -607,    0,    0, -607,    0, yyDummy>>,
			1, 1000, 6000)
		end

	yytable_template_8 (an_array: ARRAY [INTEGER])
			-- Fill chunk #8 of template for `yytable'.
		do
			yyarray_subcopy (an_array, <<
			    0,    0,    0, -607, -607,    0,    0,    0,    0, -607,
			 -607,    0,    0,    0,    0,    0,    0, -607,    0,    0,
			    0,    0,    0,    0, -607, -607, -607,    0,    0, -607,
			    0, -607,    0,    0, -607,    0,    0, -607, -607, -607,
			    0,    0,    0,    0,    0,    0,    0,    0, -607,    0,
			 -607, -607,    0, -607,    0, -607, -607,    0, -607, -607,
			    0,    0, -607, -607,    0,    0,    0, -607,    0,    0,
			    0, -607, -607,    0,    0,    0, -607, -607, -607,    0,
			 -607,    0,    0,    0, -607,    0, -607, -607, -607, -607,
			    0, -607,    0, -607, -607, -607, -607,    0,    0,    0,

			 -607, -607,    0,    0,    0,    0,    0,    0,    0, -607,
			 -607, -607,    0,    0, -607,    0,    0,    0,    0,    0,
			    0,    0, -607,    0,    0,    0,    0,    0, -607, -607,
			    0,    0,    0, -607, -607, -607, -607,    0, -607,    0,
			 -607,    0,    0,    0,    0,    0,    0, -607,    0, -607,
			    0,    0,    0,    0, -607,    0,    0,    0,    0,    0,
			 -607,    0,    0, -607, -607, -607, -607,    0,    0,    0,
			 -607,    0,    0, -607, -607,    0,    0,    0,    0,    0,
			 -607,    0, -607,    0,    0,    0,    0,    0,    0, -607,
			    0, -607,    0, -607,    0, -607, -607, -607,    0,    0,

			    0,    0,    0,    0, -607,    0,    0, -607,    0, -607,
			    0, -607,    0,    0,    0,    0, -607,    0,    0,    0,
			 -607,    0, -607, -606, -607,    0, -607, -606, -606, -606,
			 -606, -606,    0, -606, -606, -606, -606,    0, -606, -606,
			 -606, -606, -606,    0, -606, -606,    0, -606,    0,    0,
			    0,    0,    0,    0, -606,    0,    0,    0, -606,    0,
			    0, -606, -606, -606,    0,    0,    0, -606,    0,    0,
			    0,    0, -606, -606, -606,    0, -606,    0, -606,    0,
			    0, -606,    0,    0,    0,    0,    0, -606, -606,    0,
			    0, -606,    0, -606,    0,    0,    0,    0,    0, -606,

			    0,    0, -606,    0,    0, -606,    0,    0,    0,    0,
			 -606, -606,    0,    0,    0,    0, -606, -606,    0,    0,
			    0,    0,    0,    0, -606,    0,    0,    0,    0,    0,
			    0, -606, -606, -606,    0,    0, -606,    0, -606,    0,
			    0, -606,    0,    0, -606, -606, -606,    0,    0,    0,
			    0,    0,    0,    0,    0, -606,    0, -606, -606,    0,
			 -606,    0, -606, -606,    0, -606, -606,    0,    0, -606,
			 -606,    0,    0,    0, -606,    0,    0,    0, -606, -606,
			    0,    0,    0, -606, -606, -606,    0, -606,    0,    0,
			    0, -606,    0, -606, -606, -606, -606,    0, -606,    0,

			 -606, -606, -606, -606,    0,    0,    0, -606, -606,    0,
			    0,    0,    0,    0,    0,    0, -606, -606, -606,    0,
			    0, -606,    0,    0,    0,    0,    0,    0,    0, -606,
			    0,    0,    0,    0,    0, -606, -606,    0,    0,    0,
			 -606, -606, -606, -606,    0, -606,    0, -606,    0,    0,
			    0,    0,    0,    0, -606,    0, -606,    0,    0,    0,
			    0, -606,    0,    0,    0,    0,    0, -606,    0,    0,
			 -606, -606, -606, -606,    0,    0,    0, -606,    0,    0,
			 -606, -606,    0,    0,    0,    0,    0, -606,    0, -606,
			    0,    0,    0,    0,    0,    0, -606,    0, -606,    0,

			 -606,    0, -606, -606, -606,    0,    0,    0,    0,    0,
			    0, -606,    0,    0, -606,    0, -606,    0, -606,    0,
			    0,    0,    0, -606,    0,    0,    0, -606,    0, -606,
			 -605, -606,    0, -606, -605, -605, -605, -605, -605,    0,
			 -605, -605, -605, -605,    0, -605, -605, -605, -605, -605,
			    0, -605, -605,    0, -605,    0,    0,    0,    0,    0,
			    0, -605,    0,    0,    0, -605,    0,    0, -605, -605,
			 -605,    0,    0,    0, -605,    0,    0,    0,    0, -605,
			 -605, -605,    0, -605,    0, -605,    0,    0, -605,    0,
			    0,    0,    0,    0, -605, -605,    0,    0, -605,    0,

			 -605,    0,    0,    0,    0,    0, -605,    0,    0, -605,
			    0,    0, -605,    0,    0,    0,    0, -605, -605,    0,
			    0,    0,    0, -605, -605,    0,    0,    0,    0,    0,
			    0, -605,    0,    0,    0,    0,    0,    0, -605, -605,
			 -605,    0,    0, -605,    0, -605,    0,    0, -605,    0,
			    0, -605, -605, -605,    0,    0,    0,    0,    0,    0,
			    0,    0, -605,    0, -605, -605,    0, -605,    0, -605,
			 -605,    0, -605, -605,    0,    0, -605, -605,    0,    0,
			    0, -605,    0,    0,    0, -605, -605,    0,    0,    0,
			 -605, -605, -605,    0, -605,    0,    0,    0, -605,    0,

			 -605, -605, -605, -605,    0, -605,    0, -605, -605, -605,
			 -605,    0,    0,    0, -605, -605,    0,    0,    0,    0,
			    0,    0,    0, -605, -605, -605,    0,    0, -605,    0,
			    0,    0,    0,    0,    0,    0, -605,    0,    0,    0,
			    0,    0, -605, -605,    0,    0,    0, -605, -605, -605,
			 -605,    0, -605,    0, -605,    0,    0,    0,    0,    0,
			    0, -605,    0, -605,    0,    0,    0,    0, -605,    0,
			    0,    0,    0,    0, -605,    0,    0, -605, -605, -605,
			 -605,    0,    0,    0, -605,    0,    0, -605, -605,    0,
			    0,    0,    0,    0, -605,    0, -605,    0,    0,    0,

			    0,    0,    0, -605,    0, -605,    0, -605,    0, -605,
			 -605, -605,    0,    0,    0,    0,    0,    0, -605,    0,
			    0, -605,    0, -605,    0, -605,    0,    0,    0,    0,
			 -605,    0,    0,    0, -605,    0, -605, -604, -605,    0,
			 -605, -604, -604, -604, -604, -604,    0, -604, -604, -604,
			 -604,    0, -604, -604, -604, -604, -604,    0, -604, -604,
			    0, -604,    0,    0,    0,    0,    0,    0, -604,    0,
			    0,    0, -604,    0,    0, -604, -604, -604,    0,    0,
			    0, -604,    0,    0,    0,    0, -604, -604, -604,    0,
			 -604,    0, -604,    0,    0, -604,    0,    0,    0,    0,

			    0, -604, -604,    0,    0, -604,    0, -604,    0,    0,
			    0,    0,    0, -604,    0,    0, -604,    0,    0, -604,
			    0,    0,    0,    0, -604, -604,    0,    0,    0,    0,
			 -604, -604,    0,    0,    0,    0,    0,    0, -604,    0,
			    0,    0,    0,    0,    0, -604, -604, -604,    0,    0,
			 -604,    0, -604,    0,    0, -604,    0,    0, -604, -604,
			 -604,    0,    0,    0,    0,    0,    0,    0,    0, -604,
			    0, -604, -604,    0, -604,    0, -604, -604,    0, -604,
			 -604,    0,    0, -604, -604,    0,    0,    0, -604,    0,
			    0,    0, -604, -604,    0,    0,    0, -604, -604, -604, yyDummy>>,
			1, 1000, 7000)
		end

	yytable_template_9 (an_array: ARRAY [INTEGER])
			-- Fill chunk #9 of template for `yytable'.
		do
			yyarray_subcopy (an_array, <<
			    0, -604,    0,    0,    0, -604,    0, -604, -604, -604,
			 -604,    0, -604,    0, -604, -604, -604, -604,    0,    0,
			    0, -604, -604,    0,    0,    0,    0,    0,    0,    0,
			 -604, -604, -604,    0,    0, -604,    0,    0,    0,    0,
			    0,    0,    0, -604,    0,    0,    0,    0,    0, -604,
			 -604,    0,    0,    0, -604, -604, -604, -604,    0, -604,
			    0, -604,    0,    0,    0,    0,    0,    0, -604,    0,
			 -604,    0,    0,    0,    0, -604,    0,    0,    0,    0,
			    0, -604,    0,    0, -604, -604, -604, -604,    0,    0,
			    0, -604,    0,    0, -604, -604,    0,    0,    0,    0,

			    0, -604,    0, -604,    0,    0,    0,    0,    0,    0,
			 -604,    0, -604,    0, -604,    0, -604, -604, -604,    0,
			    0,    0,    0,    0,    0, -604,    0,    0, -604,    0,
			 -604,    0, -604,    0,    0,    0,    0, -604,    0,    0,
			    0, -604,    0, -604, -603, -604,    0, -604, -603, -603,
			 -603, -603, -603,    0, -603, -603, -603, -603,    0, -603,
			 -603, -603, -603, -603,    0, -603, -603,    0, -603,    0,
			    0,    0,    0,    0,    0, -603,    0,    0,    0, -603,
			    0,    0, -603, -603, -603,    0,    0,    0, -603,    0,
			    0,    0,    0, -603, -603, -603,    0, -603,    0, -603,

			    0,    0, -603,    0,    0,    0,    0,    0, -603, -603,
			    0,    0, -603,    0, -603,    0,    0,    0,    0,    0,
			 -603,    0,    0, -603,    0,    0, -603,    0,    0,    0,
			    0, -603, -603,    0,    0,    0,    0, -603, -603,    0,
			    0,    0,    0,    0,    0, -603,    0,    0,    0,    0,
			    0,    0, -603, -603, -603,    0,    0, -603,    0, -603,
			    0,    0, -603,    0,    0, -603, -603, -603,    0,    0,
			    0,    0,    0,    0,    0,    0, -603,    0, -603, -603,
			    0, -603,    0, -603, -603,    0, -603, -603,    0,    0,
			 -603, -603,    0,    0,    0, -603,    0,    0,    0, -603,

			 -603,    0,    0,    0, -603, -603, -603,    0, -603,    0,
			    0,    0, -603,    0, -603, -603, -603, -603,    0, -603,
			    0, -603, -603, -603, -603,    0,    0,    0, -603, -603,
			    0,    0,    0,    0,    0,    0,    0, -603, -603, -603,
			    0,    0, -603,    0,    0,    0,    0,    0,    0,    0,
			 -603,    0,    0,    0,    0,    0, -603, -603,    0,    0,
			    0, -603, -603, -603, -603,    0, -603,    0, -603,    0,
			    0,    0,    0,    0,    0, -603,    0, -603,    0,    0,
			    0,    0, -603,    0,    0,    0,    0,    0, -603,    0,
			    0, -603, -603, -603, -603,    0,    0,    0, -603,    0,

			    0, -603, -603,    0,    0,    0,    0,    0, -603,    0,
			 -603,    0,    0,    0,    0,    0,    0, -603,    0, -603,
			    0, -603,    0, -603, -603, -603,    0,    0,    0,    0,
			    0,    0, -603,    0,    0, -603,    0, -603,    0, -603,
			    0,    0,    0,    0, -603,    0,    0,    0, -603,    0,
			 -603, -602, -603,    0, -603, -602, -602, -602, -602, -602,
			    0, -602, -602, -602, -602,    0, -602, -602, -602, -602,
			 -602,    0, -602, -602,    0, -602,    0,    0,    0,    0,
			    0,    0, -602,    0,    0,    0, -602,    0,    0, -602,
			 -602, -602,    0,    0,    0, -602,    0,    0,    0,    0,

			 -602, -602, -602,    0, -602,    0, -602,    0,    0, -602,
			    0,    0,    0,    0,    0, -602, -602,    0,    0, -602,
			    0, -602,    0,    0,    0,    0,    0, -602,    0,    0,
			 -602,    0,    0, -602,    0,    0,    0,    0, -602, -602,
			    0,    0,    0,    0, -602, -602,    0,    0,    0,    0,
			    0,    0, -602,    0,    0,    0,    0,    0,    0, -602,
			 -602, -602,    0,    0, -602,    0, -602,    0,    0, -602,
			    0,    0, -602, -602, -602,    0,    0,    0,    0,    0,
			    0,    0,    0, -602,    0, -602, -602,    0, -602,    0,
			 -602, -602,    0, -602, -602,    0,    0, -602, -602,    0,

			    0,    0, -602,    0,    0,    0, -602, -602,    0,    0,
			    0, -602, -602, -602,    0, -602,    0,    0,    0, -602,
			    0, -602, -602, -602, -602,    0, -602,    0, -602, -602,
			 -602, -602,    0,    0,    0, -602, -602,    0,    0,    0,
			    0,    0,    0,    0, -602, -602, -602,    0,    0, -602,
			    0,    0,    0,    0,    0,    0,    0, -602,    0,    0,
			    0,    0,    0, -602, -602,    0,    0,    0, -602, -602,
			 -602, -602,    0, -602,    0, -602,    0,    0,    0,    0,
			    0,    0, -602,    0, -602,    0,    0,    0,    0, -602,
			    0,    0,    0,    0,    0, -602,    0,    0, -602, -602,

			 -602, -602,    0,    0,    0, -602,    0,    0, -602, -602,
			    0,    0,    0,    0,    0, -602,    0, -602,    0,    0,
			    0,    0,    0,    0, -602,    0, -602,    0, -602,    0,
			 -602, -602, -602,    0,    0,    0,    0,    0,    0, -602,
			    0,    0, -602,    0, -602,    0, -602,    0,    0,    0,
			    0, -602,    0,    0,    0, -602,    0, -602, -601, -602,
			    0, -602, -601, -601, -601, -601, -601,    0, -601, -601,
			 -601, -601,    0, -601, -601, -601, -601, -601,    0, -601,
			 -601,    0, -601,    0,    0,    0,    0,    0,    0, -601,
			    0,    0,    0, -601,    0,    0, -601, -601, -601,    0,

			    0,    0, -601,    0,    0,    0,    0, -601, -601, -601,
			    0, -601,    0, -601,    0,    0, -601,    0,    0,    0,
			    0,    0, -601, -601,    0,    0, -601,    0, -601,    0,
			    0,    0,    0,    0, -601,    0,    0, -601,    0,    0,
			 -601,    0,    0,    0,    0, -601, -601,    0,    0,    0,
			    0, -601, -601,    0,    0,    0,    0,    0,    0, -601,
			    0,    0,    0,    0,    0,    0, -601, -601, -601,    0,
			    0, -601,    0, -601,    0,    0, -601,    0,    0, -601,
			 -601, -601,    0,    0,    0,    0,    0,    0,    0,    0,
			 -601,    0, -601, -601,    0, -601,    0, -601, -601,    0,

			 -601, -601,    0,    0, -601, -601,    0,    0,    0, -601,
			    0,    0,    0, -601, -601,    0,    0,    0, -601, -601,
			 -601,    0, -601,    0,    0,    0, -601,    0, -601, -601,
			 -601, -601,    0, -601,    0, -601, -601, -601, -601,    0,
			    0,    0, -601, -601,    0,    0,    0,    0,    0,    0,
			    0, -601, -601, -601,    0,    0, -601,    0,    0,    0,
			    0,    0,    0,    0, -601,    0,    0,    0,    0,    0,
			 -601, -601,    0,    0,    0, -601, -601, -601, -601,    0,
			 -601,    0, -601,    0,    0,    0,    0,    0,    0, -601,
			    0, -601,    0,    0,    0,    0, -601,    0,    0,    0, yyDummy>>,
			1, 1000, 8000)
		end

	yytable_template_10 (an_array: ARRAY [INTEGER])
			-- Fill chunk #10 of template for `yytable'.
		do
			yyarray_subcopy (an_array, <<
			    0,    0, -601,    0,    0, -601, -601, -601, -601,    0,
			    0,    0, -601,    0,    0, -601, -601,    0,    0,    0,
			    0,    0, -601,    0, -601,    0,    0,    0,    0,    0,
			    0, -601,    0, -601,    0, -601,    0, -601, -601, -601,
			    0,    0,    0,    0,    0,    0, -601,    0,    0, -601,
			    0, -601,    0, -601,    0,    0,    0,    0, -601,    0,
			    0,    0, -601,    0, -601, -597, -601,    0, -601, -597,
			 -597, -597, -597, -597,    0, -597, -597, -597, -597,    0,
			 -597, -597, -597, -597, -597,    0, -597, -597,    0, -597,
			    0,    0,    0,    0,    0,    0, -597,    0,    0,    0,

			 -597,    0,    0, -597, -597, -597,    0,    0,    0, -597,
			    0,    0,    0,    0, -597, -597, -597,    0, -597,    0,
			 -597,    0,    0, -597,    0,    0,    0,    0,    0, -597,
			 -597,    0,    0, -597,    0, -597,    0,    0,    0,    0,
			    0, -597,    0,    0, -597,    0,    0, -597,    0,    0,
			    0,    0, -597, -597,    0,    0,    0,    0, -597, -597,
			    0,    0,    0,    0,    0,    0, -597,    0,    0,    0,
			    0,    0,    0, -597, -597, -597,    0,    0, -597,    0,
			 -597,    0,    0, -597,    0,    0, -597, -597, -597,    0,
			    0,    0,    0,    0,    0,    0,    0, -597,    0, -597,

			 -597,    0, -597,    0, -597, -597,    0, -597, -597,    0,
			    0, -597, -597,    0,    0,    0, -597,    0,    0,    0,
			 -597, -597,    0,    0,    0, -597, -597, -597,    0, -597,
			    0,    0,    0, -597,    0, -597, -597, -597, -597,    0,
			 -597,    0, -597, -597, -597, -597,    0,    0,    0, -597,
			 -597,    0,    0,    0,    0,    0,    0,    0, -597, -597,
			 -597,    0,    0, -597,    0,    0,    0,    0,    0,    0,
			    0, -597,    0,    0,    0,    0,    0, -597, -597,    0,
			    0,    0, -597, -597, -597, -597,    0, -597,    0, -597,
			    0,    0,    0,    0,    0,    0, -597,    0, -597,    0,

			    0,    0,    0, -597,    0,    0,    0,    0,    0, -597,
			    0,    0, -597, -597, -597, -597,    0,    0,    0, -597,
			    0,    0, -597, -597,    0,    0,    0,    0,    0, -597,
			    0, -597,    0,    0,    0,    0,    0,    0, -597,    0,
			 -597,    0, -597,    0, -597, -597, -597,    0,    0,    0,
			    0,    0,    0, -597,    0,    0, -597,    0, -597,    0,
			 -597,    0,    0,    0,    0, -597,    0,    0,    0, -597,
			    0, -597, -596, -597,    0, -597, -596, -596, -596, -596,
			 -596,    0, -596, -596, -596, -596,    0, -596, -596, -596,
			 -596, -596,    0, -596, -596,    0, -596,    0,    0,    0,

			    0,    0,    0, -596,    0,    0,    0, -596,    0,    0,
			 -596, -596, -596,    0,    0,    0, -596,    0,    0,    0,
			    0, -596, -596, -596,    0, -596,    0, -596,    0,    0,
			 -596,    0,    0,    0,    0,    0, -596, -596,    0,    0,
			 -596,    0, -596,    0,    0,    0,    0,    0, -596,    0,
			    0, -596,    0,    0, -596,    0,    0,    0,    0, -596,
			 -596,    0,    0,    0,    0, -596, -596,    0,    0,    0,
			    0,    0,    0, -596,    0,    0,    0,    0,    0,    0,
			 -596, -596, -596,    0,    0, -596,    0, -596,    0,    0,
			 -596,    0,    0, -596, -596, -596,    0,    0,    0,    0,

			    0,    0,    0,    0, -596,    0, -596, -596,    0, -596,
			    0, -596, -596,    0, -596, -596,    0,    0, -596, -596,
			    0,    0,    0, -596,    0,    0,    0, -596, -596,    0,
			    0,    0, -596, -596, -596,    0, -596,    0,    0,    0,
			 -596,    0, -596, -596, -596, -596,    0, -596,    0, -596,
			 -596, -596, -596,    0,    0,    0, -596, -596,    0,    0,
			    0,    0,    0,    0,    0, -596, -596, -596,    0,    0,
			 -596,    0,    0,    0,    0,    0,    0,    0, -596,    0,
			    0,    0,    0,    0, -596, -596,    0,    0,    0, -596,
			 -596, -596, -596,    0, -596,    0, -596,    0,    0,    0,

			    0,    0,    0, -596,    0, -596,    0,    0,    0,    0,
			 -596,    0,    0,    0,    0,    0, -596,    0,    0, -596,
			 -596, -596, -596,    0,    0,    0, -596,    0,    0, -596,
			 -596,    0,    0,    0,    0,    0, -596,    0, -596,    0,
			    0,    0,    0,    0,    0, -596,    0, -596,    0, -596,
			    0, -596, -596, -596,    0,    0,    0,    0,    0,    0,
			 -596,    0,    0, -596,    0, -596,    0, -596,    0,    0,
			    0,    0, -596,    0,    0,    0, -596,    0, -596, -598,
			 -596,    0, -596, -598, -598, -598, -598, -598,    0, -598,
			 -598, -598, -598,    0, -598, -598, -598, -598, -598,    0,

			 -598, -598,    0, -598,    0,    0,    0,    0,    0,    0,
			 -598,    0,    0,    0, -598,    0,    0, -598, -598, -598,
			    0,    0,    0, -598,    0,    0,    0,    0, -598, -598,
			 -598,    0, -598,    0, -598,    0,    0, -598,    0,    0,
			    0,    0,    0, -598, -598,    0,    0, -598,    0, -598,
			    0,    0,    0,    0,    0, -598,    0,    0, -598,    0,
			    0, -598,    0,    0,    0,    0, -598, -598,    0,    0,
			    0,    0, -598, -598,    0,    0,    0,    0,    0,    0,
			 -598,    0,    0,    0,    0,    0,    0, -598, -598, -598,
			    0,    0, -598,    0, -598,    0,    0, -598,    0,    0,

			 -598, -598, -598,    0,    0,    0,    0,    0,    0,    0,
			    0, -598,    0, -598, -598,    0, -598,    0, -598, -598,
			    0, -598, -598,    0,    0, -598, -598,    0,    0,    0,
			 -598,    0,    0,    0, -598, -598,    0,    0,    0, -598,
			 -598, -598,    0, -598,    0,    0,    0, -598,    0, -598,
			 -598, -598, -598,    0, -598,    0, -598, -598, -598, -598,
			    0,    0,    0, -598, -598,    0,    0,    0,    0,    0,
			    0,    0, -598, -598, -598,    0,    0, -598,    0,    0,
			    0,    0,    0,    0,    0, -598,    0,    0,    0,    0,
			    0, -598, -598,    0,    0,    0, -598, -598, -598, -598,

			    0, -598,    0, -598,    0,    0,    0,    0,    0,    0,
			 -598,    0, -598,    0,    0,    0,    0, -598,    0,    0,
			    0,    0,    0, -598,    0,    0, -598, -598, -598, -598,
			    0,    0,    0, -598,    0,    0, -598, -598,    0,    0,
			    0,    0,    0, -598,    0, -598,    0,    0,    0,    0,
			    0,    0, -598,    0, -598,    0, -598,    0, -598, -598,
			 -598,    0,    0,    0,    0,    0,    0, -598,    0,    0,
			 -598,    0, -598,    0, -598,    0,    0,    0,    0, -598,
			    0,    0,    0, -598,    0, -598, -610, -598,    0, -598,
			 -610, -610, -610, -610, -610,    0, -610, -610, -610, -610, yyDummy>>,
			1, 1000, 9000)
		end

	yytable_template_11 (an_array: ARRAY [INTEGER])
			-- Fill chunk #11 of template for `yytable'.
		do
			yyarray_subcopy (an_array, <<
			    0, -610, -610, -610, -610, -610,    0, -610, -610,    0,
			 -610,    0,    0,    0,    0,    0,    0, -610,    0,    0,
			    0, -610,    0,    0, -610, -610, -610,    0,    0,    0,
			 -610,    0,    0,    0,    0, -610, -610, -610,    0, -610,
			    0, -610,    0,    0, -610,    0,    0,    0,    0,    0,
			 -610, -610,    0,    0, -610,    0, -610,    0,    0,    0,
			    0,    0, -610,    0,    0, -610,    0,    0, -610,    0,
			    0,    0,    0, -610, -610,    0,    0,    0,    0, -610,
			 -610,    0,    0,    0,    0,    0,    0, -610,    0,    0,
			    0,    0,    0,    0, -610, -610, -610,    0,    0, -610,

			    0, -610,    0,    0, -610,    0,    0, -610, -610, -610,
			    0,    0,    0,    0,    0,    0,    0,    0, -610,    0,
			 -610, -610,    0, -610,    0, -610, -610,    0, -610, -610,
			    0,    0, -610, -610,    0,    0,    0, -610,    0,    0,
			    0, -610, -610,    0,    0,    0, -610, -610, -610,    0,
			 -610,    0,    0,    0, -610,    0, -610, -610, -610, -610,
			    0, -610,    0, -610, -610, -610, -610,    0,    0,    0,
			 -610, -610,    0,    0,    0,    0,    0,    0,    0, -610,
			 -610, -610,    0,    0, -610,    0,    0,    0,    0,    0,
			    0,    0, -610,    0,    0,    0,    0,    0, -610, -610,

			    0,    0,    0, -610, -610, -610, -610,    0, -610,    0,
			 -610,    0,    0,    0,    0,    0,    0, -610,    0, -610,
			    0,    0,    0,    0, -610,    0,    0,    0,    0,    0,
			 -610,    0,    0, -610, -610, -610, -610,    0,    0,    0,
			 -610,    0,    0, -610, -610,    0,    0,    0,    0,    0,
			 -610,    0, -610,    0,    0,    0,    0,    0,    0, -610,
			    0, -610,    0, -610,    0, -610, -610, -610,    0,    0,
			    0,    0,    0,    0, -610,    0,    0, -610,    0, -610,
			 1128, -610,  224,  223,    0,  222, -610,    0,    0,    0,
			 -610,  221, -610,    0, -610,    0, -610,    0,    0,    0,

			  220,  108,  219,  218,  217,  216,  215,  214,    0,  213,
			    0,    0,    0,    0,  212,  107,    0,  105,    0,  211,
			    0,    0,  210,    0,    0,    0,    0,    0,  103,    0,
			  101,    0,  100,    0,    0,  209,   97,    0,   95,    0,
			  208,    0,    0,    0,    0,   94,  207,   92,    0,  206,
			  205,    0,    0,   90,    0,    0,   87,    0,  204,   86,
			    0,    0,    0,    0,   84,   83,    0,    0,  203,    0,
			    0,    0,    0,    0,   81,   80,   79,    0,   78,    0,
			    0,    0,    0,    0,    0,    0,    0,   74,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,

			    0,    0,    0,    0,    0,    0,    0,   70,    0,   68,
			    0,    0,   67,    0,   66,    0,    0,    0,    0,   65,
			   64,    0,    0,    0,    0,   62,    0,    0,    0,    0,
			    0,    0,    0,   60,    0,    0,    0,    0,   59,   58,
			    0,   56,    0,    0,    0,    0,    0,    0,    0,   55,
			    0,    0,   54,    0,   53,   52,    0,   51,  202,    0,
			    0,  201,   48,    0,    0,    0,    0,    0,    0,    0,
			   47,   46,    0,    0,  200,   45,    0,    0,    0,  199,
			  198,    0,    0,   44,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,   42,   41,   40,    0,    0,   39,

			    0,   38,    0,    0,    0,    0,    0,    0,    0,    0,
			   35,    0,    0,    0,    0,   34,    0,    0,    0,    0,
			    0,   33,    0,    0,    0,   31,   30,   29,    0,    0,
			    0,   27,    0,   26,   25,    0,    0,    0,    0,    0,
			    0,   22,    0,   21,    0,    0,    0,    0,    0,    0,
			   20,    0,   19,    0,   17,    0,   16,   15,    0,    0,
			    0,  197,   14,    0,    0,   13,    0,    0,   11,    0,
			    0, 1128,   10,  224,    0,  196,    0,    7,    0,    0,
			    0,    6,  221,    0,    0,    0,    0,    0,    0,    0,
			    4,    0,  108,  219,  218,  217,  216,  215,  214,    0,

			  213,    0,    0,    0,    0,  212,  107,    0,  105,    0,
			  211,    0,    0,  210,    0,    0,    0,    0,    0,  103,
			    0,  101,    0,  100,    0,    0,    0,   97,    0,   95,
			    0,    0,    0,    0,    0,    0,   94,   93,   92,    0,
			  206,  205,    0,    0,   90,    0,    0,   87,    0,  204,
			   86,    0,    0,    0,    0,   84,   83,    0,    0,  203,
			    0,    0,    0,    0,    0,   81,   80,   79,    0,   78,
			    0,    0,    0,    0, 1127,    0,    0,    0,   74,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,   70,    0,

			   68,    0,    0,   67,    0,   66,    0,    0,    0,    0,
			   65,   64,    0,    0,    0,    0,   62,    0,    0,    0,
			    0,    0,    0,    0,   60,    0,    0,    0,    0,   59,
			   58,    0,   56,    0,    0,    0,    0,    0,    0,    0,
			   55,    0,    0,   54,    0,   53,   52,    0,   51,  202,
			    0,    0,  201,   48,    0,    0, -486,    0,    0,    0,
			 -486,   47,   46, -486,    0,  200,   45, -486,    0,    0,
			  199,  198,    0,    0,   44,    0,    0,    0,    0,    0,
			 -486,    0,    0,    0,    0,   42,   41,   40,    0,    0,
			   39,    0,   38,    0,    0,    0,    0,    0,    0,    0,

			    0,   35,    0,    0,    0,    0,   34,    0,    0,    0,
			    0,    0,   33,    0,    0,    0,   31,   30,   29,    0,
			 -486,    0,   27,    0,   26,   25,    0,    0,    0,    0,
			    0, -485,   22,    0,   21, -485,    0,    0, -485,    0,
			    0,   20, -485,   19,    0,   17,    0,   16,   15,    0,
			 1563,    0,  197,   14,    0, -485,   13,    0,    0,   11,
			    0,    0,    0,   10,    0,    0,  196,    0,    7, -486,
			    0,    0,    6,    0,    0,    0,    0, -486,    0, -486,
			    0,    4,    0,    0,    0,    0,    0,    0,    0,    0,
			 -486,    0,    0,    0,    0, -485, 1562,    0,    0,    0,

			    0,    0,    0, -486,    0,    0,    0, -486,    0,    0,
			    0,    0,    0,    0,    0,    0, 1561,    0,    0,    0,
			    0,    0,    0,    0, -486, 1563,    0,    0,    0,    0,
			    0,    0,    0, 1560,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,    0, -485,    0,    0,    0,    0,    0,
			    0, 1559, -485,    0, -485,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,    0, -485,    0,    0, -486, -486,
			    0, 1562,    0,    0,    0, 1556,    0,    0, -485,    0,
			    0,    0, -485,    0,    0,    0,    0, -486,    0,    0,
			    0, 1561,    0,    0,    0,    0,  108,    0,    0, -485, yyDummy>>,
			1, 1000, 10000)
		end

	yytable_template_12 (an_array: ARRAY [INTEGER])
			-- Fill chunk #12 of template for `yytable'.
		do
			yyarray_subcopy (an_array, <<
			    0,    0,    0,    0,    0, 1558,    0,    0, 1560,    0,
			  107,    0,  105,    0,    0,    0,    0,    0,    0,    0,
			    0,  471,    0,  103,    0,  101, 1559,  100,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			   94,    0,   92, -485, -485,    0,    0,    0,   90, -486,
			    0,   87,    0,    0,   86,    0,    0,    0,    0,   84,
			   83,    0, -485,    0, -486,    0, -486,    0,    0,    0,
			    0,    0,    0,   78,    0,    0,    0,    0,    0,    0,
			 1558,    0,   74,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,

			    0,    0,    0,    0,   68,    0,    0,   67,    0,   66,
			    0,    0,    0,    0,   65,   64,    0,    0,    0,    0,
			    0,    0,    0,    0, -485,    0,    0,    0,   60,    0,
			    0,    0,    0,   59,   58,    0,   56,    0,    0, -485,
			    0, -485,    0,    0,   55,    0,    0,   54,    0,    0,
			   52,    0,   51,    0,    0,    0,  201,   48,    0,    0,
			    0,    0,    0,    0,    0,   47,   46,    0,    0,    0,
			   45,    0,    0,    0,    0,    0,    0,    0,   44,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,   42,
			   41,   40,    0,    0,   39,    0,   38,    0,    0,    0,

			    0,    0,    0,    0,    0,   35,    0,    0,    0,    0,
			   34,    0,    0,    0,    0,    0,   33,    0,  472,    0,
			   31,    0,   29,    0,    0,    0,   27,    0,    0,   25,
			    0,    0,    0,    0,    0,    0,   22,    0,   21,  108,
			    0,    0,    0,    0,    0,   20,    0,   19,    0,   17,
			    0,   16,   15,  107,    0,  105,    0,    0,    0,    0,
			   13,    0,    0,   11,  471,    0,  103,   10,  101,    0,
			  100,    0,    7,    0,    0,    0,    6,    0,    0,    0,
			    0,    0,  470,   94,    0,   92,    0,    0,    0,    0,
			    0,   90,    0,    0,   87,    0,    0,   86,    0,    0,

			    0,    0,   84,   83,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,   78,    0,    0,    0,
			    0,    0,    0,    0,    0,   74,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,   68,    0,    0,
			   67,    0,   66,    0,    0,    0,    0,   65,   64,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,   60,    0,    0,    0,    0,   59,   58,    0,   56,
			    0,    0,    0,    0,    0,    0,    0,   55,    0,    0,
			   54,    0,    0,   52,    0,   51,    0,    0,    0,  201,

			   48,    0,    0,    0,    0,    0,    0,    0,   47,   46,
			    0,    0,    0,   45,    0,    0,    0,    0,    0,    0,
			    0,   44,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,   42,   41,   40,    0,    0,   39,    0,   38,
			    0,    0,    0,    0,    0,    0,    0,    0,   35,    0,
			    0,    0,    0,   34,    0,    0,    0,    0,    0,   33,
			    0,    0,    0,   31,    0,   29,    0,    0,    0,   27,
			    0,    0,   25,    0,    0,    0,    0,    0,    0,   22,
			    0,   21,    0,    0,    0,    0,    0,    0,   20,    0,
			   19,    0,   17,    0,   16,   15,    0,    0,    0,    0,

			    0,    0,    0,   13,    0,  422,   11,  224,    0,    0,
			   10,    0,  774,    0,    0,    7,  221,    0,    0,    6,
			  454,  453,    0,    0,    0,  470,  108,  219,  218,  217,
			  216,  215,  214,  773,  213,    0,    0,    0,    0,  212,
			  107,    0,  105,    0,  211,    0,  524,  210,    0,    0,
			    0,    0,    0,  103,    0,  101,    0,  100,    0,    0,
			  461,   97,    0,   95,    0,    0,    0,    0,    0,    0,
			   94,   93,   92,    0,  206,  205,    0,    0,   90,    0,
			   88,   87,    0,  204,   86,    0,    0,   85,    0,   84,
			   83,    0,    0,  203,    0,  523,    0,    0,    0,   81,

			   80,   79,    0,   78,    0,    0,    0,    0,  772,    0,
			  522,    0,   74,    0,    0,    0,    0,  521,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,   71,    0,    0,
			    0,    0,   70,   69,   68,    0,  520,  519,    0,   66,
			    0,    0,    0,    0,   65,   64,  518,  517,    0,    0,
			   62,    0,    0,    0,    0,    0,   61,    0,   60,    0,
			    0,    0,    0,   59,   58,   57,   56,    0,    0,    0,
			    0,    0,    0,    0,   55,    0,    0,   54,    0,   53,
			   52,    0,   51,  202,    0,    0,  201,   48,    0,    0,
			    0,    0,    0,    0,    0,   47,   46,    0,    0,  200,

			   45,    0,    0,    0,  199,  198,    0,    0,   44,    0,
			    0,    0,    0,   43,    0,    0,    0,    0,    0,   42,
			   41,   40,    0,    0,   39,    0,   38,    0,    0,    0,
			   37,    0,    0,    0,    0,   35,    0,    0,    0,    0,
			   34,    0,    0,    0,  516,    0,   33,  515,    0,    0,
			   31,   30,   29,   28,    0,    0,   27,    0,   26,   25,
			  514,    0,    0,    0,    0,    0,   22,    0,   21,    0,
			    0,    0,    0,    0,    0,   20,    0,   19,    0,   17,
			    0,   16,   15,    0,    0,    0,  197,   14,    0,    0,
			   13,    0,  422,   11,  224,    0,    0,   10,  513,    0,

			  196,    0,    7,  221,    0,    0,    6,    0,    0,  771,
			    0,    5,    0,  108,  219,  218,  217,  216,  215,  214,
			    0,  213,    0,    0,    0,    0,  212,  107,    0,  105,
			    0,  211,    0,  524,  210,    0,    0,    0,    0,    0,
			  103,    0,  101,    0,  100,    0,    0,  461,   97,    0,
			   95,    0,    0,    0,    0,    0,    0,   94,   93,   92,
			    0,  206,  205,    0,    0,   90,    0,   88,   87,    0,
			  204,   86,    0,    0,   85,    0,   84,   83,    0,    0,
			  203,    0,  523,    0,    0,    0,   81,   80,   79,    0,
			   78,    0,    0,    0,    0,    0,    0,  522,    0,   74,

			    0,    0,    0,    0,  521,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,   71,    0,    0,    0,    0,   70,
			   69,   68,    0,  520,  519,    0,   66,    0,    0,    0,
			    0,   65,   64,  518,  517,    0,    0,   62,    0,    0,
			    0,    0,    0,   61,    0,   60,    0,    0,    0,    0,
			   59,   58,   57,   56,    0,    0,    0,    0,    0,    0,
			    0,   55,    0,    0,   54,    0,   53,   52,    0,   51,
			  202,    0,    0,  201,   48,    0,    0,    0,    0,    0,
			    0,    0,   47,   46,    0,    0,  200,   45,    0,    0,
			    0,  199,  198,    0,    0,   44,    0,    0,    0,    0, yyDummy>>,
			1, 1000, 11000)
		end

	yytable_template_13 (an_array: ARRAY [INTEGER])
			-- Fill chunk #13 of template for `yytable'.
		do
			yyarray_subcopy (an_array, <<
			   43,    0,    0,    0,    0,    0,   42,   41,   40,    0,
			    0,   39,    0,   38,    0,    0,    0,   37,    0,    0,
			    0,    0,   35,    0,    0,    0,    0,   34,    0,    0,
			    0,  516,    0,   33,  515,    0,    0,   31,   30,   29,
			   28,    0,    0,   27,    0,   26,   25,  514,    0,    0,
			    0,    0,    0,   22,    0,   21,    0,    0,    0,    0,
			    0,    0,   20,    0,   19,    0,   17,    0,   16,   15,
			    0,    0,    0,  197,   14,    0,    0,   13,    0,  422,
			   11,  224,    0, 1992,   10,  513,    0,  196,    0,    7,
			  221,    0,    0,    6,    0,    0,    0,    0,    5,    0,

			  108,  219,  218,  217,  216,  215,  214,    0,  213,    0,
			    0,    0,    0,  212,  107,    0,  105,    0,  211,    0,
			    0,  210,    0,    0,    0,    0,    0,  103,    0,  101,
			    0,  100,    0,    0,  461,   97,    0,   95,    0,    0,
			    0,    0,    0,    0,   94,   93,   92,    0,  206,  205,
			    0,    0,   90,    0,   88,   87,    0,  204,   86,    0,
			    0,   85,    0,   84,   83,    0,    0,  203,    0,    0,
			    0,    0,    0,   81,   80,   79,    0,   78,    0,    0,
			    0,    0,    0,    0,  522,    0,   74,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,

			    0,   71,    0,    0,    0,    0,   70,   69,   68,    0,
			  520,  519,    0,   66,    0,    0,    0,    0,   65,   64,
			  518,  517,    0,    0,   62,    0,    0,    0,    0,    0,
			   61,    0,   60,    0,    0,    0,    0,   59,   58,   57,
			   56,    0,    0,    0,    0,    0,    0,    0,   55,    0,
			    0,   54,    0,   53,   52,    0,   51,  202,    0,    0,
			  201,   48,    0,    0,    0,    0,    0,    0,    0,   47,
			   46,    0,    0,  200,   45,    0,    0,    0,  199,  198,
			    0,    0,   44,    0,    0,    0,    0,   43,    0,    0,
			    0,    0,    0,   42,   41,   40,    0,    0,   39,    0,

			   38,    0,    0,    0,   37,    0,    0,    0,    0,   35,
			    0,    0,    0,    0,   34,    0,    0,    0, 1991,    0,
			   33,  515,    0,    0,   31,   30,   29,   28,    0,    0,
			   27,    0,   26,   25,  514,    0,    0,    0,    0,    0,
			   22,    0,   21,    0,    0,    0,    0,    0,    0,   20,
			    0,   19,    0,   17,    0,   16,   15,    0,    0,    0,
			  197,   14,    0,    0,   13,    0,    0,   11,    0,    0,
			    0,   10,  513,    0,  196,    0,    7,    0,    0,  422,
			    6,  224,  223,    0,  222,    5,    0,    0,    0,    0,
			  221,    0,    0,    0,    0,    0,    0,    0,    0,  220,

			  108,  219,  218,  217,  216,  215,  214,    0,  213,    0,
			    0,    0,    0,  212,  107,    0,  105,    0,  211,    0,
			    0,  210,    0,    0,    0,    0,    0,  103,    0,  101,
			    0,  100,    0,    0,  209,   97,    0,   95,    0,  208,
			    0,    0,    0,    0,   94,  207,   92,    0,  206,  205,
			    0,    0,   90,    0,    0,   87,    0,  204,   86,    0,
			    0,    0,    0,   84,   83,    0,    0,  203,    0,    0,
			    0,    0,    0,   81,   80,   79,    0,   78,    0,    0,
			    0,    0,    0,    0,    0,    0,   74,    0,    0,    0,
			    0,    0,    0,    0,  421,    0,    0,    0,    0,    0,

			    0,    0,    0,    0,    0,    0,   70,    0,   68,    0,
			    0,   67,    0,   66,    0,    0,    0,    0,   65,   64,
			    0,    0,    0,    0,   62,    0,    0,    0,    0,    0,
			    0,    0,   60,    0,    0,    0,    0,   59,   58,    0,
			   56,    0,    0,    0,    0,    0,    0,    0,   55,    0,
			    0,   54,    0,   53,   52,    0,   51,  202,    0,    0,
			  201,   48,    0,    0,    0,    0,    0,    0,    0,   47,
			   46,    0,    0,  200,   45,    0,    0,    0,  199,  198,
			    0,    0,   44,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,   42,   41,   40,    0,    0,   39,    0,

			   38,    0,    0,    0,    0,    0,    0,    0,    0,   35,
			    0,    0,    0,    0,   34,    0,    0,    0,    0,    0,
			   33,    0,    0,    0,   31,   30,   29,    0,    0,    0,
			   27,    0,    0,   25,    0,    0,    0,    0,    0,    0,
			   22,    0,   21,    0,    0,    0,    0,    0,    0,   20,
			    0,   19,    0,   17,    0,   16,   15,    0,    0,    0,
			  197,   14,    0,    0,   13,    0,    0,   11,    0,    0,
			  422,   10,  224,  223,  196,  222,    7,    0,    0,    0,
			    6,  221,  420,    0,    0,    0,    0,    0,    0,    0,
			  220,  108,  219,  218,  217,  216,  215,  214,    0,  213,

			    0,    0,    0,    0,  212,  107,    0,  105,    0,  211,
			    0,    0,  210,    0,    0,    0,    0,    0,  103,    0,
			  101,    0,  100,    0,    0,  209,   97,    0,   95,    0,
			  208,    0,    0,    0,    0,   94,  207,   92,    0,  206,
			  205,    0,    0,   90,    0,    0,   87,    0,  204,   86,
			    0,    0,    0,    0,   84,   83,    0,    0,  203,    0,
			    0,    0, 1977,    0,   81,   80,   79,    0,   78,    0,
			    0,    0,    0,    0,    0,    0,    0,   74,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,   70,    0,   68,

			    0,    0,   67,    0,   66,    0,    0,    0,    0,   65,
			   64,    0,    0,    0,    0,   62,    0,    0,    0,    0,
			    0,    0,    0,   60,    0,    0,    0,    0,   59,   58,
			    0,   56,    0,    0,    0,    0,    0,    0,    0,   55,
			    0,    0,   54,    0,   53,   52,    0,   51,  202,    0,
			    0,  201,   48,    0,    0,    0,    0, 1280,    0,    0,
			   47,   46,    0,    0,  200,   45,    0,    0,    0,  199,
			  198,    0,    0,   44,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,   42,   41,   40,    0,    0,   39,
			    0,   38,    0,    0,    0,    0,    0,    0,    0,    0,

			   35,    0,    0,    0,    0,   34,    0,    0,    0,    0,
			    0,   33,    0,    0,    0,   31,   30,   29,    0, 1976,
			    0,   27,    0,    0,   25,    0,    0,    0,    0,    0,
			    0,   22,    0,   21,    0,    0,    0,    0,    0,    0,
			   20,    0,   19,    0,   17,    0,   16,   15,    0,    0,
			    0,  197,   14,    0,    0,   13,    0,    0,   11,    0,
			    0,  434,   10,  224,  223,  196,  222,    7,    0,    0,
			    0,    6,  221,    0,    0,  433,    0,    0,    0,    0,
			    0,  220,  108,  219,  218,  217,  216,  215,  214,    0,
			  213,    0,    0,    0,    0,  212,  107,    0,  105,    0, yyDummy>>,
			1, 1000, 12000)
		end

	yytable_template_14 (an_array: ARRAY [INTEGER])
			-- Fill chunk #14 of template for `yytable'.
		do
			yyarray_subcopy (an_array, <<
			  211,    0,    0,  210,    0,    0,    0,    0,    0,  103,
			    0,  101,    0,  100,    0,    0,  209,   97,    0,   95,
			    0,  208,    0,    0,    0,    0,   94,  207,   92,    0,
			  206,  205,    0,    0,   90,    0,    0,   87,    0,  204,
			   86,    0,    0,    0,    0,   84,   83,    0,    0,  203,
			    0,    0,    0,    0, 1646,   81,   80,   79,    0,   78,
			    0,    0,    0,    0,    0,    0,    0,    0,   74,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,  298,    0,    0,   70,    0,
			   68,    0,    0,   67,    0,   66,    0,    0,    0,    0,

			   65,   64,    0,    0,    0,    0,   62,    0,    0,    0,
			    0,    0,    0,    0,   60,    0,    0,    0,    0,   59,
			   58,    0,   56,    0,    0,    0,    0,    0,    0,    0,
			   55,    0,    0,   54,    0,   53,   52,    0,   51,  202,
			    0,    0,  201,   48,    0,    0,    0,    0,    0,    0,
			    0,   47,   46,    0,    0,  200,   45,    0,    0,    0,
			  199,  198,    0,    0,   44,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,   42,   41,   40,    0,    0,
			   39,    0,   38,    0,    0,    0,    0,    0,    0,    0,
			    0,   35,    0,    0,    0,    0,   34,    0,    0,    0,

			    0,    0,   33,    0,    0,    0,   31,   30,   29,    0,
			    0,    0,   27,    0,    0,   25,    0,    0,    0,    0,
			    0,    0,   22,    0,   21,    0,    0,    0,    0,    0,
			    0,   20,    0,   19,    0,   17,    0,   16,   15,    0,
			    0,    0,  197,   14,    0,    0,   13,    0,    0,   11,
			    0,    0,  434,   10,  224,  223,  196,  222,    7,    0,
			    0,    0,    6,  221,    0,    0,  433,    0,    0,    0,
			    0,    0,  220,  108,  219,  218,  217,  216,  215,  214,
			    0,  213,    0,    0,    0,    0,  212,  107,    0,  105,
			    0,  211,    0,    0,  210,    0,    0,    0,    0,    0,

			  103,    0,  101,    0,  100,    0,    0,  209,   97,    0,
			   95,    0,  208,    0,    0,    0,    0,   94,  207,   92,
			    0,  206,  205,    0,    0,   90,    0,    0,   87,    0,
			  204,   86,    0,    0,    0,    0,   84,   83,    0,    0,
			  203,    0,    0,    0,    0, 1615,   81,   80,   79,    0,
			   78,    0,    0,    0,    0,    0,    0,    0,    0,   74,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,  298,    0,    0,   70,
			    0,   68,    0,    0,   67,    0,   66,    0,    0,    0,
			    0,   65,   64,    0,    0,    0,    0,   62,    0,    0,

			    0,    0,    0,    0,    0,   60,    0,    0,    0,    0,
			   59,   58,    0,   56,    0,    0,    0,    0,    0,    0,
			    0,   55,    0,    0,   54,    0,   53,   52,    0,   51,
			  202,    0,    0,  201,   48,    0,    0,    0,    0,    0,
			    0,    0,   47,   46,    0,    0,  200,   45,    0,    0,
			    0,  199,  198,    0,    0,   44,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,   42,   41,   40,    0,
			    0,   39,    0,   38,    0,    0,    0,    0,    0,    0,
			    0,    0,   35,    0,    0,    0,    0,   34,    0,    0,
			    0,    0,    0,   33,    0,    0,    0,   31,   30,   29,

			    0,    0,    0,   27,    0,    0,   25,    0,    0,    0,
			    0,    0,    0,   22,    0,   21,    0,    0,    0,    0,
			    0,    0,   20,    0,   19,    0,   17,    0,   16,   15,
			    0,    0,    0,  197,   14,    0,    0,   13,    0,    0,
			   11,    0,    0,  422,   10,  224,  223,  196,  222,    7,
			    0,    0,    0,    6,  221,    0,    0,    0,    0,    0,
			    0,    0,    0,  220,  108,  219,  218,  217,  216,  215,
			  214,    0,  213,    0,    0,    0,    0,  212,  107,    0,
			  105,    0,  211, 1056,    0,  210, 1055,    0,    0,    0,
			    0,  103,    0,  101,    0,  100,    0,    0,  209,   97,

			    0,   95,    0,  208,    0,    0,    0,    0,   94,  207,
			   92,    0,  206,  205,    0,    0,   90,    0,    0,   87,
			    0,  204,   86,    0,    0,    0,    0,   84,   83,    0,
			    0,  203,    0,    0,    0,    0,    0,   81,   80,   79,
			    0,   78,    0,    0,    0,    0,    0,    0,    0,    0,
			   74,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			   70,    0,   68,    0,    0,   67,    0,   66,    0,    0,
			    0,    0,   65,   64,    0,    0,    0,    0,   62,    0,
			    0,    0,    0,    0,    0,    0,   60,    0,    0,    0,

			    0,   59,   58,    0,   56,    0,    0,    0,    0,    0,
			    0,    0,   55,    0,    0,   54,    0,   53,   52,    0,
			   51,  202,    0,    0,  201,   48,    0,    0,    0,    0,
			    0,    0,    0,   47,   46,    0,    0,  200,   45,    0,
			    0,    0,  199,  198,    0,    0,   44,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,   42,   41,   40,
			    0,    0,   39,    0,   38,    0,    0,    0,    0,    0,
			    0,    0,    0,   35,    0,    0,    0,    0,   34,    0,
			    0,    0,    0,    0,   33,    0,    0,    0,   31,   30,
			   29,    0,    0,    0,   27,    0,    0,   25,    0,    0,

			    0,    0,    0, 1054,   22,    0,   21,    0,    0,    0,
			    0,    0,    0,   20,    0,   19,    0,   17,    0,   16,
			   15,    0,    0,    0,  197,   14,    0,    0,   13,    0,
			    0,   11,    0,    0,  434,   10,  224,  223,  196,  222,
			    7,    0,    0,    0,    6,  221,    0,    0,  433,    0,
			    0,    0,    0,    0,  220,  108,  219,  218,  217,  216,
			  215,  214,    0,  213,    0,    0,    0,    0,  212,  107,
			    0,  105,    0,  211,    0,    0,  210,    0,    0,    0,
			    0,    0,  103,    0,  101,    0,  100,    0,    0,  209,
			   97,    0,   95,    0,  208,    0,    0,    0,    0,   94,

			  207,   92,    0,  206,  205,    0,    0,   90,    0,    0,
			   87,    0,  204,   86,    0,    0,    0,    0,   84,   83,
			    0,    0,  203,    0,    0,    0,    0,    0,   81,   80,
			   79,    0,   78,    0,    0,    0,    0,    0,    0,    0,
			    0,   74,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,  298,    0,
			    0,   70,    0,   68,    0,    0,   67,    0,   66,    0,
			    0,    0,    0,   65,   64,    0,    0,    0,    0,   62,
			    0,    0,    0,    0,    0,    0,    0,   60,    0,    0,
			    0,    0,   59,   58,    0,   56,    0,    0,    0,    0, yyDummy>>,
			1, 1000, 13000)
		end

	yytable_template_15 (an_array: ARRAY [INTEGER])
			-- Fill chunk #15 of template for `yytable'.
		do
			yyarray_subcopy (an_array, <<
			    0,    0,    0,   55,    0,    0,   54,    0,   53,   52,
			    0,   51,  202,    0,    0,  201,   48,    0,    0,    0,
			    0,    0,    0,    0,   47,   46,    0,    0,  200,   45,
			    0,    0,    0,  199,  198,    0,    0,   44,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,   42,   41,
			   40,    0,    0,   39,    0,   38,    0,    0,    0,    0,
			    0,    0,    0,    0,   35,    0,    0,    0,    0,   34,
			    0,    0,    0,    0,    0,   33,    0,    0,    0,   31,
			   30,   29,    0,    0,    0,   27,    0,   26,   25,    0,
			    0,    0,    0,    0,    0,   22,    0,   21,    0,    0,

			    0,    0,    0,    0,   20,    0,   19,    0,   17,    0,
			   16,   15,    0,    0,    0,  197,   14,    0,    0,   13,
			    0,    0,   11,    0,    0,  434,   10,  224,  223,  196,
			  222,    7,    0,    0,    0,    6,  221,    0,    0,  433,
			    0,    0,    0,    0,    0,  220,  108,  219,  218,  217,
			  216,  215,  214,    0,  213,    0,    0,    0,    0,  212,
			  107,    0,  105,    0,  211,    0,    0,  210,    0,    0,
			    0,    0,    0,  103,    0,  101,    0,  100,    0,    0,
			  209,   97,    0,   95,    0,  208,    0,    0,    0,    0,
			   94,  207,   92,    0,  206,  205,    0,    0,   90,    0,

			    0,   87,    0,  204,   86,    0,    0,    0,    0,   84,
			   83,    0,    0,  203,    0,    0,    0,    0,    0,   81,
			   80,   79,    0,   78,    0,    0,    0,    0,    0,    0,
			    0,    0,   74,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,  298,
			    0,    0,   70,    0,   68,    0,    0,   67,    0,   66,
			    0,    0,    0,    0,   65,   64,    0,    0,    0,    0,
			   62,    0,    0,    0,    0,    0,    0,    0,   60,    0,
			    0,    0,    0,   59,   58,    0,   56,    0,    0,    0,
			    0,    0,    0,    0,   55,    0,    0,   54,    0,   53,

			   52,    0,   51,  202,    0,    0,  201,   48,    0,    0,
			    0,    0,    0,    0,    0,   47,   46,    0,    0,  200,
			   45,    0,    0,    0,  199,  198,    0,    0,   44,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,   42,
			   41,   40,    0,    0,   39,    0,   38,    0,    0,    0,
			    0,    0,    0,    0,    0,   35,    0,    0,    0,    0,
			   34,    0,    0,    0,    0,    0,   33,    0,    0,    0,
			   31,   30,   29,    0,    0,    0,   27,    0,    0,   25,
			    0,    0,    0,    0,    0,    0,   22,    0,   21,    0,
			    0,    0,    0,    0,    0,   20,    0,   19,    0,   17,

			    0,   16,   15,    0,    0,    0,  197,   14,    0,    0,
			   13,    0,    0,   11,    0,    0,  422,   10,  224,  223,
			  196,  222,    7,    0,    0,    0,    6,  221,    0,    0,
			    0,    0,    0,    0,    0,    0,  220,  108,  219,  218,
			  217,  216,  215,  214,    0,  213,    0,    0,    0,    0,
			  212,  107,    0,  105,    0,  211,  739,    0,  210,    0,
			    0,    0,    0,    0,  103,    0,  101,    0,  100,    0,
			    0,  209,   97,    0,   95,    0,  208,    0,    0,    0,
			    0,   94,  207,   92,    0,  206,  205,    0,    0,   90,
			    0,    0,   87,    0,  204,   86,    0,    0,    0,    0,

			   84,   83,    0,    0,  203,    0,    0,    0,    0,    0,
			   81,   80,   79,    0,   78,    0,    0,    0,    0,    0,
			    0,    0,    0,   74,    0,  738,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,   70,    0,   68,    0,    0,   67,    0,
			   66,    0,    0,    0,    0,   65,   64,    0,    0,    0,
			    0,   62,    0,    0,    0,    0,    0,    0,    0,   60,
			    0,    0,    0,    0,   59,   58,    0,   56,    0,    0,
			    0,    0,    0,    0,    0,   55,    0,    0,   54,    0,
			   53,   52,    0,   51,  202,    0,    0,  201,   48,    0,

			    0,    0,    0,    0,    0,    0,   47,   46,    0,    0,
			  200,   45,    0,    0,    0,  199,  198,    0,    0,   44,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			   42,   41,   40,    0,    0,   39,    0,   38,    0,    0,
			    0,    0,    0,    0,    0,    0,   35,    0,    0,    0,
			    0,   34,    0,    0,    0,    0,    0,   33,    0,    0,
			    0,   31,   30,   29,    0,    0,    0,   27,    0,    0,
			   25,    0,    0,    0,    0,    0,    0,   22,    0,   21,
			    0,    0,    0,    0,    0,    0,   20,    0,   19,    0,
			   17,    0,   16,   15,    0,    0,    0,  197,   14,    0,

			    0,   13,    0,    0,   11,    0,    0,  300,   10,  224,
			  223,  196,  222,    7,    0,    0,    0,    6,  221,    0,
			    0,  299,    0,    0,    0,    0,    0,  220,  108,  219,
			  218,  217,  216,  215,  214,    0,  213,    0,    0,    0,
			    0,  212,  107,    0,  105,    0,  211,    0,    0,  210,
			    0,    0,    0,    0,    0,  103,    0,  101,    0,  100,
			    0,    0,  209,   97,    0,   95,    0,  208,    0,    0,
			    0,    0,   94,  207,   92,    0,  206,  205,    0,    0,
			   90,    0,    0,   87,    0,  204,   86,    0,    0,    0,
			    0,   84,   83,    0,    0,  203,    0,    0,    0,    0,

			    0,   81,   80,   79,    0,   78,    0,    0,    0,    0,
			    0,    0,    0,    0,   74,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,  298,    0,    0,   70,    0,   68,    0,    0,   67,
			    0,   66,    0,    0,    0,    0,   65,   64,    0,    0,
			    0,    0,   62,    0,    0,    0,    0,    0,    0,    0,
			   60,    0,    0,    0,    0,   59,   58,    0,   56,    0,
			    0,    0,    0,    0,    0,    0,   55,    0,    0,   54,
			    0,   53,   52,    0,   51,  202,    0,    0,  201,   48,
			    0,    0,    0,    0,    0,    0,    0,   47,   46,    0,

			    0,  200,   45,    0,    0,    0,  199,  198,    0,    0,
			   44,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,   42,   41,   40,    0,    0,   39,    0,   38,    0,
			    0,    0,    0,    0,    0,    0,    0,   35,    0,    0,
			    0,    0,   34,    0,    0,    0,    0,    0,   33,    0,
			    0,    0,   31,   30,   29,    0,    0,    0,   27,    0,
			    0,   25,    0,    0,    0,    0,    0,    0,   22,    0,
			   21,    0,    0,    0,    0,    0,    0,   20,    0,   19,
			    0,   17,    0,   16,   15,    0,    0,    0,  197,   14,
			    0,    0,   13,    0,    0,   11,    0,    0,  422,   10, yyDummy>>,
			1, 1000, 14000)
		end

	yytable_template_16 (an_array: ARRAY [INTEGER])
			-- Fill chunk #16 of template for `yytable'.
		do
			yyarray_subcopy (an_array, <<
			  224,  223,  196,  222,    7,    0,    0,    0,    6,  221,
			    0,    0,    0,    0,    0,    0,    0,    0,  220,  108,
			  219,  218,  217,  216,  215,  214,    0,  213,    0,    0,
			    0,    0,  212,  107,    0,  105,    0,  211,    0,    0,
			  210,    0,    0,    0,    0,    0,  103,    0,  101,    0,
			  100,    0,    0,  209,   97,    0,   95,    0,  208,    0,
			    0,    0,    0,   94,  207,   92,    0,  206,  205,    0,
			    0,   90,    0,    0,   87,    0,  204,   86,    0,    0,
			    0,    0,   84,   83,    0,    0,  203,    0,    0,    0,
			    0,    0,   81,   80,   79,    0,   78,    0,    0,    0,

			    0,    0,    0,    0,    0,   74,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,   70,    0,   68,    0,    0,
			   67,    0,   66,    0,    0,    0,    0,   65,   64,    0,
			    0,    0,    0,   62,    0,    0,    0,    0,    0,    0,
			    0,   60,    0,    0,    0,    0,   59,   58,    0,   56,
			    0,    0,    0,    0,    0,    0,    0,   55,    0,    0,
			   54,    0,   53,   52,    0,   51,  202,    0,    0,  201,
			   48,    0,    0,    0,    0, 1280,    0,    0,   47,   46,
			    0,    0,  200,   45,    0,    0,    0,  199,  198,    0,

			    0,   44,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,   42,   41,   40,    0,    0,   39,    0,   38,
			    0,    0,    0,    0,    0,    0,    0,    0,   35,    0,
			    0,    0,    0,   34,    0,    0,    0,    0,    0,   33,
			    0,    0,    0,   31,   30,   29,    0,    0,    0,   27,
			    0,    0,   25,    0,    0,    0,    0,    0,    0,   22,
			    0,   21,    0,    0,    0,    0,    0,    0,   20,    0,
			   19,    0,   17,    0,   16,   15,    0,    0,    0,  197,
			   14,    0,    0,   13,    0,    0,   11,    0,    0,  422,
			   10,  224,  223,  196,  222,    7,    0,    0,    0,    6,

			  221,    0,    0,    0,    0,    0,    0,    0,    0,  220,
			  108,  219,  218,  217,  216,  215,  214,    0,  213,    0,
			    0,    0,    0,  212,  107,    0,  105,    0,  211,    0,
			    0,  210,    0,    0,    0,    0,    0,  103,    0,  101,
			    0,  100,    0,    0,  209,   97,    0,   95,    0,  208,
			    0,    0,    0,    0,   94,  207,   92,    0,  206,  205,
			    0,    0,   90,    0,    0,   87,    0,  204,   86,    0,
			    0,    0,    0,   84,   83,    0,    0,  203,    0,    0,
			    0,    0,    0,   81,   80,   79,    0,   78,    0,    0,
			    0,    0,    0,    0,    0,    0,   74,    0,    0,    0,

			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,   70,    0,   68,    0,
			    0,   67,    0,   66,    0,    0,    0,    0,   65,   64,
			    0,    0,    0,    0,   62,    0,    0,    0,    0,    0,
			    0,    0,   60,    0,    0,    0,    0,   59,   58,    0,
			   56,    0,    0,    0,    0,    0,    0,    0,   55,    0,
			    0,   54,    0,   53,   52,    0,   51,  202,    0,    0,
			  201,   48,    0,    0,    0,    0,    0,    0,    0,   47,
			   46,    0,    0,  200,   45,    0,    0,    0,  199,  198,
			    0,    0,   44,    0,    0,    0,    0,    0,    0,    0,

			    0,    0,    0,   42,   41,   40,    0,    0,   39,    0,
			   38,    0,    0,    0,    0,    0,    0,    0,    0,   35,
			    0,    0,    0,    0,   34,    0,    0,    0,    0,    0,
			   33,    0,    0,    0,   31,   30,   29,    0,    0,    0,
			   27,    0,   26,   25,    0,    0,    0,    0,    0,    0,
			   22,    0,   21,    0,    0,    0,    0,    0,    0,   20,
			    0,   19,    0,   17,    0,   16,   15,    0,    0,    0,
			  197,   14,    0,    0,   13,    0,    0,   11,    0,    0,
			  422,   10,  224,  223,  196,  222,    7,    0,    0,    0,
			    6,  221,    0,    0,    0,    0,    0,    0,    0,    0,

			  220,  108,  219,  218,  217,  216,  215,  214,    0,  213,
			    0,    0,    0,    0,  212,  107,    0,  105,    0,  211,
			    0,    0,  210,    0,    0,    0,    0,    0,  103,    0,
			  101,    0,  100,    0,    0,  209,   97,    0,   95,    0,
			  208,    0,    0,    0,    0,   94,  207,   92,    0,  206,
			  205,    0,    0,   90,    0,    0,   87,    0,  204,   86,
			    0,    0,    0,    0,   84,   83,    0,    0,  203,    0,
			    0,    0,    0,    0,   81,   80,   79,    0,   78,    0,
			    0,    0,    0,    0,    0,    0,    0,   74,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,

			    0,    0,    0,    0,    0,    0,    0,   70,    0,   68,
			    0,    0,   67,    0,   66,    0,    0,    0,    0,   65,
			   64,    0,    0,    0,    0,   62,    0,    0,    0,    0,
			    0,    0,    0,   60,    0,    0,    0,    0,   59,   58,
			    0,   56,    0,    0,    0,    0,    0,  883,    0,   55,
			    0,    0,   54,    0,   53,   52,    0,   51,  202,    0,
			    0,  201,   48,    0,    0,    0,    0,    0,    0,    0,
			   47,   46,    0,    0,  200,   45,    0,    0,    0,  199,
			  198,    0,    0,   44,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,   42,   41,   40,    0,    0,   39,

			    0,   38,    0,    0,    0,    0,    0,    0,    0,    0,
			   35,    0,    0,    0,    0,   34,    0,    0,    0,    0,
			    0,   33,    0,    0,    0,   31,   30,   29,    0,    0,
			    0,   27,    0,    0,   25,    0,    0,    0,    0,    0,
			    0,   22,    0,   21,    0,    0,    0,    0,    0,    0,
			   20,    0,   19,    0,   17,    0,   16,   15,    0,    0,
			    0,  197,   14,    0,    0,   13,    0,    0,   11,    0,
			    0,  422,   10,  224,  223,  196,  222,    7,    0,  847,
			    0,    6,  221,    0,    0,    0,    0,    0,    0,    0,
			    0,  220,  108,  219,  218,  217,  216,  215,  214,    0,

			  213,    0,    0,    0,    0,  212,  107,    0,  105,    0,
			  211,    0,    0,  210,    0,    0,    0,    0,    0,  103,
			    0,  101,    0,  100,    0,    0,  209,   97,    0,   95,
			    0,  208,    0,    0,    0,    0,   94,  207,   92,    0,
			  206,  205,    0,    0,   90,    0,    0,   87,    0,  204,
			   86,    0,    0,    0,    0,   84,   83,    0,    0,  203,
			    0,    0,    0,    0,    0,   81,   80,   79,    0,   78,
			    0,    0,    0,    0,    0,    0,    0,    0,   74,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,   70,    0, yyDummy>>,
			1, 1000, 15000)
		end

	yytable_template_17 (an_array: ARRAY [INTEGER])
			-- Fill chunk #17 of template for `yytable'.
		do
			yyarray_subcopy (an_array, <<
			   68,    0,    0,   67,    0,   66,    0,    0,    0,    0,
			   65,   64,    0,    0,    0,    0,   62,    0,    0,    0,
			    0,    0,    0,    0,   60,    0,    0,    0,    0,   59,
			   58,    0,   56,    0,    0,    0,    0,    0,    0,    0,
			   55,    0,    0,   54,    0,   53,   52,    0,   51,  202,
			    0,    0,  201,   48,    0,    0,    0,    0,    0,    0,
			    0,   47,   46,    0,    0,  200,   45,    0,    0,    0,
			  199,  198,    0,    0,   44,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,   42,   41,   40,    0,    0,
			   39,    0,   38,    0,    0,    0,    0,    0,    0,    0,

			    0,   35,    0,    0,    0,    0,   34,    0,    0,    0,
			    0,    0,   33,    0,    0,    0,   31,   30,   29,    0,
			    0,    0,   27,    0,    0,   25,    0,    0,    0,    0,
			    0,    0,   22,    0,   21,    0,    0,    0,    0,    0,
			    0,   20,    0,   19,    0,   17,    0,   16,   15,    0,
			    0,    0,  197,   14,    0,    0,   13,    0,    0,   11,
			    0,    0,  422,   10,  224,  223,  196,  222,    7,    0,
			    0,    0,    6,  221,    0,    0,    0,    0,    0,    0,
			    0,    0,  220,  108,  219,  218,  217,  216,  215,  214,
			    0,  213,    0,    0,    0,    0,  212,  107,    0,  105,

			    0,  211,    0,    0,  210,    0,    0,    0,    0,    0,
			  103,    0,  101,    0,  100,    0,    0,  209,   97,    0,
			   95,    0,  208,    0,    0,    0,    0,   94,  207,   92,
			    0,  206,  205,    0,    0,   90,    0,    0,   87,    0,
			  204,   86,    0,    0,    0,    0,   84,   83,    0,    0,
			  203,    0,    0,    0,    0,    0,   81,   80,   79,    0,
			   78,    0,    0,    0,    0,    0,    0,    0,    0,   74,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,   70,
			    0,   68,    0,    0,   67,    0,   66,    0,    0,    0,

			    0,   65,   64,    0,    0,    0,    0,   62,    0,    0,
			    0,    0,    0,    0,    0,   60,    0,    0,    0,    0,
			   59,   58,    0,   56,    0,    0,    0,    0,    0,    0,
			    0,   55,    0,    0,   54,    0,   53,   52,    0,   51,
			  202,    0,    0,  201,   48,    0,    0,    0,    0,    0,
			    0,    0,   47,   46,    0,    0,  200,   45,    0,    0,
			    0,  199,  198,    0,    0,   44,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,   42,   41,   40,    0,
			    0,   39,    0,   38,    0,    0,    0,    0,    0,    0,
			    0,    0,   35,    0,    0,    0,    0,   34,    0,    0,

			    0,    0,    0,   33,    0,    0,    0,   31,   30,   29,
			    0,    0,    0,   27,    0,    0,   25,    0,    0,    0,
			    0,    0,    0,   22,    0,   21,    0,    0,    0,    0,
			    0,    0,   20,    0,   19,    0,   17,    0,   16,   15,
			    0,    0,    0,  197,   14,    0,    0,   13,    0,    0,
			   11,    0,    0,  422,   10,  224,  576,  196,  575,    7,
			    0,    0,    0,    6,  221,    0,    0,    0,    0,    0,
			    0,    0,    0,  574,  108,  219,  218,  217,  216,  215,
			  214,    0,  213,    0,    0,    0,    0,  212,  107,    0,
			  105,    0,  211,    0,    0,  210,    0,    0,    0,    0,

			    0,  103,    0,  101,    0,  100,    0,    0,  209,   97,
			    0,   95,    0,  208,    0,    0,    0,    0,   94,  207,
			   92,    0,  206,  205,    0,    0,   90,    0,    0,   87,
			    0,  204,   86,    0,    0,    0,    0,   84,   83,    0,
			    0,  203,    0,    0,    0,    0,    0,   81,   80,   79,
			    0,   78,    0,    0,    0,    0,    0,    0,    0,    0,
			   74,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			   70,    0,   68,    0,    0,   67,    0,   66,    0,    0,
			    0,    0,   65,   64,    0,    0,    0,    0,   62,    0,

			    0,    0,    0,    0,    0,    0,   60,    0,    0,    0,
			    0,   59,   58,    0,   56,    0,    0,    0,    0,    0,
			    0,    0,   55,    0,    0,   54,    0,   53,   52,    0,
			   51,  202,    0,    0,  201,   48,    0,    0,    0,    0,
			    0,    0,    0,   47,   46,    0,    0,  200,   45,    0,
			    0,    0,  199,  198,    0,    0,   44,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,   42,   41,   40,
			    0,    0,   39,    0,   38,    0,    0,    0,    0,    0,
			    0,    0,    0,   35,    0,    0,    0,    0,   34,    0,
			    0,    0,    0,    0,   33,    0,    0,    0,   31,   30,

			   29,    0,    0,    0,   27,    0,    0,   25,    0,    0,
			    0,    0,    0,    0,   22,    0,   21,    0,    0,    0,
			    0,    0,    0,   20,    0,   19,    0,   17,    0,   16,
			   15,    0,    0,    0,  197,   14,    0,    0,   13,    0,
			    0,   11,    0,    0,  225,   10,  224,  223,  196,  222,
			    7,    0,    0,    0,    6,  221,    0,    0,    0,    0,
			    0,    0,    0,    0,  220,  108,  219,  218,  217,  216,
			  215,  214,    0,  213,    0,    0,    0,    0,  212,  107,
			    0,  105,    0,  211,    0,    0,  210,    0,    0,    0,
			    0,    0,  103,    0,  101,    0,  100,    0,    0,  209,

			   97,    0,   95,    0,  208,    0,    0,    0,    0,   94,
			  207,   92,    0,  206,  205,    0,    0,   90,    0,    0,
			   87,    0,  204,   86,    0,    0,    0,    0,   84,   83,
			    0,    0,  203,    0,    0,    0,    0,    0,   81,   80,
			   79,    0,   78,    0,    0,    0,    0,    0,    0,    0,
			    0,   74,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,   70,    0,   68,    0,    0,   67,    0,   66,    0,
			    0,    0,    0,   65,   64,    0,    0,    0,    0,   62,
			    0,    0,    0,    0,    0,    0,    0,   60,    0,    0,

			    0,    0,   59,   58,    0,   56,    0,    0,    0,    0,
			    0,    0,    0,   55,    0,    0,   54,    0,   53,   52,
			    0,   51,  202,    0,    0,  201,   48,    0,    0,    0,
			    0,    0,    0,    0,   47,   46,    0,    0,  200,   45,
			    0,    0,    0,  199,  198,    0,    0,   44,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,   42,   41,
			   40,    0,    0,   39,    0,   38,    0,    0,    0,    0,
			    0,    0,    0,    0,   35,    0,    0,    0,    0,   34,
			    0,    0,    0,    0,    0,   33,    0,    0,    0,   31,
			   30,   29,    0,    0,    0,   27,    0,    0,   25,    0, yyDummy>>,
			1, 1000, 16000)
		end

	yytable_template_18 (an_array: ARRAY [INTEGER])
			-- Fill chunk #18 of template for `yytable'.
		do
			yyarray_subcopy (an_array, <<
			    0,    0,    0,    0,    0,   22,    0,   21,    0,    0,
			    0,    0,    0,    0,   20,    0,   19,    0,   17,    0,
			   16,   15,    0,    0,    0,  197,   14,    0,    0,   13,
			    0,  422,   11,  224,    0,    0,   10,    0,    0,  196,
			    0,    7,  221,    0,    0,    6,    0,    0,    0,    0,
			    0,    0,  108,  219,  218,  217,  216,  215,  214,    0,
			  213,    0,    0,    0,    0,  212,  107,    0,  105,    0,
			  211,    0,    0,  210,    0,    0,    0,    0,    0,  103,
			    0,  101,    0,  100,    0,    0,    0,   97,    0,   95,
			    0,    0,    0,    0,    0,    0,   94,   93,   92,    0,

			  206,  205,    0,    0,   90,    0,    0,   87,    0,  204,
			   86,    0,    0,    0,    0,   84,   83,    0,    0,  203,
			    0,    0,    0,    0,    0,   81,   80,   79,    0,   78,
			    0,    0,    0,    0, 1127,    0,    0,    0,   74,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,   70,    0,
			   68,    0,    0,   67,    0,   66,    0,    0,    0,    0,
			   65,   64,    0,    0,    0,    0,   62,    0,    0,    0,
			    0,    0,    0,    0,   60,    0,    0,    0,    0,   59,
			   58,    0,   56,    0,    0,    0,    0,    0,    0,    0,

			   55,    0,    0,   54,    0,   53,   52,    0,   51,  202,
			    0,    0,  201,   48,    0,    0,    0,    0,    0,    0,
			    0,   47,   46,    0,    0,  200,   45,    0,    0,    0,
			  199,  198,    0,    0,   44,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,   42,   41,   40,    0,    0,
			   39,    0,   38,    0,    0,    0,    0,    0,    0,    0,
			    0,   35,    0,    0,    0,    0,   34,    0,    0,    0,
			    0,    0,   33,    0,    0,    0,   31,   30,   29,    0,
			    0,    0,   27,    0,    0,   25,    0,    0,    0,    0,
			    0,    0,   22,    0,   21,    0,    0,    0,    0,    0,

			    0,   20,    0,   19,    0,   17,    0,   16,   15,    0,
			    0,    0,  197,   14,    0,    0,   13,    0,  422,   11,
			  224,    0,    0,   10,    0,    0,  196,    0,    7,  221,
			    0,    0,    6,    0,    0,    0,    0,    0,    0,  108,
			  219,  218,  217,  216,  215,  214,    0,  213,    0,    0,
			    0,    0,  212,  107,    0,  105,    0,  211,    0,    0,
			  210,    0,    0,    0,    0,    0,  103,    0,  101,    0,
			  100,    0,    0,    0,   97,    0,   95,    0,    0,    0,
			    0,    0,    0,   94,   93,   92,    0,  206,  205,    0,
			    0,   90,    0,    0,   87,    0,  204,   86,    0,    0,

			    0,    0,   84,   83,    0,    0,  203,    0,    0,    0,
			    0,    0,   81,   80,   79,    0,   78,    0,    0,    0,
			    0,    0,    0,    0,    0,   74,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,   70,    0,   68,    0,    0,
			   67,    0,   66,    0,    0,    0,    0,   65,   64,    0,
			    0,    0,    0,   62,    0,    0,    0,    0,    0,    0,
			    0,   60,    0,    0,    0,    0,   59,   58,    0,   56,
			    0,    0,    0,    0,    0,    0,    0,   55,    0,    0,
			   54,    0,   53,   52,    0,   51,  202,    0,    0,  201,

			   48,    0,    0,    0,    0,    0,    0,    0,   47,   46,
			    0,    0,  200,   45,    0,    0,    0,  199,  198,    0,
			    0,   44,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,   42,   41,   40,    0,    0,   39,    0,   38,
			    0,    0,    0,    0,    0,    0,    0,    0,   35,    0,
			    0,    0,    0,   34,    0,    0,    0,    0,    0,   33,
			    0,    0,    0,   31,   30,   29,    0,    0,    0,   27,
			    0,    0,   25,    0,    0,    0,    0,    0,    0,   22,
			    0,   21,    0,    0,    0,    0,    0,    0,   20,    0,
			   19,    0,   17,    0,   16,   15,    0,    0,    0,  197,

			   14,    0,    0,   13,    0,  422,   11,  224,    0,    0,
			   10,    0,    0,  196,    0,    7,  221,    0,    0,    6,
			    0,    0,    0,    0,    0,    0,  108,  219,  218,  217,
			  216,  215,  214,    0,  213,    0,    0,    0,    0,  212,
			  107,    0,  105,    0,  211,    0,    0,  210,    0,    0,
			    0,    0,    0,  103,    0,  101,    0,  100,    0,    0,
			    0,   97,    0,   95,    0,    0,    0,    0,    0,    0,
			   94,  207,   92,    0,  206,  205,    0,    0,   90,    0,
			    0,   87,    0,  204,   86,    0,    0,    0,    0,   84,
			   83,    0,    0,  203,    0,    0,    0,    0,    0,   81,

			   80,   79,    0,   78,    0,    0,    0,    0,    0,    0,
			    0,    0,   74,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,   70,    0,   68,    0,    0,   67,    0,   66,
			    0,    0,    0,    0,   65,   64,    0,    0,    0,    0,
			   62,    0,    0,    0,    0,    0,    0,    0,   60,    0,
			    0,    0,    0,   59,   58,    0,   56,    0,    0,    0,
			    0,    0,    0,    0,   55,    0,    0,   54,    0,   53,
			   52,    0,   51,  202,    0,    0,  201,   48,    0,    0,
			    0,    0,    0,    0,    0,   47,   46,    0,    0,  200,

			   45,    0,    0,    0,  199,  198,    0,    0,   44,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,   42,
			   41,   40,    0,    0,   39,    0,   38,    0,    0,    0,
			    0,    0,    0,    0,    0,   35,    0,    0,    0,    0,
			   34,    0,    0,    0,    0,    0,   33,    0,    0,    0,
			   31,   30,   29,    0,    0,    0,   27,    0,    0,   25,
			    0,    0,    0,    0,    0,    0,   22,    0,   21,    0,
			    0,    0,    0,    0,    0,   20,    0,   19,    0,   17,
			    0,   16,   15,    0,    0,    0,  197,   14,    0,    0,
			   13,    0,  500,   11,  224,    0,    0,   10,    0,    0,

			  196,    0,    7,  221,    0,    0,    6,    0,    0,    0,
			    0,    0,    0,  108,  219,  218,  217,  216,  215,  214,
			    0,  213,    0,    0,    0,    0,  212,  107,    0,  105,
			    0,  211,    0,    0,  210,    0,    0,    0,    0,    0,
			  103,    0,  101,    0,  100,    0,    0,    0,   97,    0,
			   95,    0,    0,    0,    0,    0,    0,   94,  207,   92,
			    0,  206,  205,    0,    0,   90,    0,    0,   87,    0,
			  204,   86,    0,    0,    0,    0,   84,   83,    0,    0,
			  203,    0,    0,    0,    0,    0,   81,   80,   79,    0,
			   78,    0,    0,    0,    0,    0,    0,    0,    0,   74, yyDummy>>,
			1, 1000, 17000)
		end

	yytable_template_19 (an_array: ARRAY [INTEGER])
			-- Fill chunk #19 of template for `yytable'.
		do
			yyarray_subcopy (an_array, <<
			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,   70,
			    0,   68,    0,    0,   67,    0,   66,    0,    0,    0,
			    0,   65,   64,    0,    0,    0,    0,   62,    0,    0,
			    0,    0,    0,    0,    0,   60,    0,    0,    0,    0,
			   59,   58,    0,   56,    0,    0,    0,    0,    0,    0,
			    0,   55,    0,    0,   54,    0,   53,   52,    0,   51,
			  202,    0,    0,  201,   48,    0,    0,    0,    0,    0,
			    0,    0,   47,   46,    0,    0,  200,   45,    0,    0,
			    0,  199,  198,    0,    0,   44,    0,    0,    0,    0,

			    0,    0,    0,    0,    0,    0,   42,   41,   40,    0,
			    0,   39,    0,   38,    0,    0,    0,    0,    0,    0,
			    0,    0,   35,    0,    0,    0,    0,   34,    0,    0,
			    0,    0,    0,   33,    0,    0,    0,   31,   30,   29,
			    0,    0,    0,   27,    0,    0,   25,    0,    0,    0,
			    0,    0,    0,   22,    0,   21,    0,    0,    0,    0,
			    0,    0,   20,    0,   19,    0,   17,    0,   16,   15,
			    0,    0,    0,  197,   14,    0,    0,   13,    0,    0,
			   11,    0,    0,    0,   10,    0,    0,  196,    0,    7,
			    0,    0,    0,    6, -653, -777, -777, -777, -777, -777,

			  449, -777,    0, -777,    0,    0,    0,    0,    0,    0,
			    0,    0, -777, -777,    0, -777,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,    0, -777, 1002,    0, -777,
			    0, -777,    0,    0,    0,    0,    0,    0,    0,    0,
			 -777,    0, -777,    0, -777,    0, -777,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,    0, -777,
			    0, -777,    0,    0,    0,    0,    0, -777,    0,    0,
			 -777,    0,    0, -777,    0,    0,    0,    0, -777, -777,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,    0, -777,    0,    0,    0,    0,    0,    0,    0,

			    0, -777,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,    0, -777,    0,    0, -777,    0, -777,    0,
			    0,    0,    0, -777, -777,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,    0, -777,    0,    0,
			    0,    0, -777, -777,    0, -777,    0,    0,    0,    0,
			    0,    0,    0, -777,    0,    0, -777,    0,    0, -777,
			    0, -777,    0,    0,    0, -777, -777,    0,    0,    0,
			    0,    0,    0,    0, -777, -777,    0,    0,    0, -777,
			    0,    0,    0,    0,    0,    0,    0, -777,    0,    0,

			    0,    0,    0,    0,    0,    0,    0,    0, -777, -777,
			 -777,    0,    0, -777,    0, -777,    0,    0,    0,    0,
			    0,    0,    0,    0, -777,  372,    0,    0,    0, -777,
			    0,    0,    0,    0,    0, -777,    0,    0,    0, -777,
			    0, -777,    0,    0,    0, -777,  108,    0, -777,    0,
			    0,    0,    0,    0,    0, -777,    0, -777,    0,    0,
			  107,    0,  105,    0, -777,    0, -777,    0, -777,    0,
			 -777, -777,    0,  103,    0,  101,    0,  100,    0, -777,
			    0,    0, -777,    0,    0,    0, -777,    0,    0,    0,
			   94, -777,   92,    0,    0, -777,    0,    0,   90,    0,

			    0,   87,    0,    0,   86,    0,    0,    0,    0,   84,
			   83,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,   78,    0,    0,    0,    0,    0,    0,
			    0,    0,   74,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,   68,    0,    0,   67,    0,   66,
			    0,    0,    0,    0,   65,   64,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,   60,    0,
			    0,    0,    0,   59,   58,    0,   56,    0,    0,    0,
			    0,    0,    0,    0,   55,    0,    0,   54,    0,   53,

			   52,    0,   51,  202,    0,    0,  201,   48,    0,    0,
			    0,    0,    0,    0,    0,   47,   46,    0,    0,    0,
			   45,    0,    0,    0,    0,    0,    0,    0,   44,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,   42,
			   41,   40,    0,    0,   39,    0,   38,    0,    0,    0,
			    0,    0,    0,    0,    0,   35, 1295,    0,    0,    0,
			   34,    0,    0,    0,    0,    0,   33,    0,    0,    0,
			   31,   30,   29,    0,    0,    0,   27,  108,    0,   25,
			    0,    0,    0,    0,    0,    0,   22,    0,   21,    0,
			    0,  107,    0,  105,    0,   20,    0,   19,    0,   17,

			    0,   16,   15,    0,  103,    0,  101,    0,  100,    0,
			   13,    0,    0,   11,    0, 1294,    0,   10,    0,    0,
			    0,   94,    7,   92,    0,    0,    6,    0,    0,   90,
			    0,    0,   87,    0,    0,   86,    0,    0,    0,    0,
			   84,   83,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,   78,    0,    0,    0,    0,    0,
			    0,    0,    0,   74,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,   68,    0,    0,   67,    0,
			   66,    0,    0,    0,    0,   65,   64,    0,    0,    0,

			    0,    0,    0,    0,    0,    0,    0,    0,    0,   60,
			    0,    0,    0,    0,   59,   58,    0,   56,    0,    0,
			    0,    0,    0,    0,    0,   55,    0,    0,   54,    0,
			    0,   52,    0,   51,    0,    0,    0,  201,   48,    0,
			    0,    0,    0,    0,    0,    0,   47,   46,    0,    0,
			    0,   45,    0,    0,    0,    0,    0,    0,    0,   44,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			   42,   41,   40,    0,    0,   39,    0,   38,    0,    0,
			    0,    0,    0,    0,    0,    0,   35, 1295,    0,    0,
			    0,   34,    0,    0,    0,    0,    0,   33,    0,    0,

			    0,   31,    0,   29,    0,    0,    0,   27,  108,   26,
			   25,    0,    0,    0,    0,    0,    0,   22,    0,   21,
			    0,    0,  107,    0,  105,    0,   20,    0,   19,    0,
			   17,    0,   16,   15,    0,  103,    0,  101,    0,  100,
			    0,   13,    0,    0,   11,    0, 1294,    0,   10,    0,
			    0,    0,   94,    7,   92,    0,    0,    6,    0,    0,
			   90,    0,    0,   87,    0,    0,   86,    0,    0,    0,
			    0,   84,   83,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,   78,    0,    0,    0,    0,
			    0,    0,    0,    0,   74,    0,    0,    0,    0,    0, yyDummy>>,
			1, 1000, 18000)
		end

	yytable_template_20 (an_array: ARRAY [INTEGER])
			-- Fill chunk #20 of template for `yytable'.
		do
			yyarray_subcopy (an_array, <<
			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,   68,    0,    0,   67,
			    0,   66,    0,    0,    0,    0,   65,   64,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			   60,    0,    0,    0,    0,   59,   58,    0,   56,    0,
			    0,    0,    0,    0,    0,    0,   55,    0,    0,   54,
			    0,    0,   52,    0,   51,    0,    0,    0,  201,   48,
			    0,    0,    0,    0,    0,    0,    0,   47,   46,    0,
			    0,    0,   45,    0,    0,    0,    0,    0,    0,    0,
			   44,    0,    0,    0,    0,    0,    0,    0,    0,    0,

			    0,   42,   41,   40,    0,    0,   39,    0,   38,    0,
			    0,    0,    0,    0,    0,    0,    0,   35, 1972,    0,
			    0,    0,   34,    0,    0,    0,    0,    0,   33,    0,
			    0,    0,   31,    0,   29,    0,    0,    0,   27,  108,
			    0,   25,    0,    0,    0,    0,    0,    0,   22,    0,
			   21,    0,    0,  107,    0,  105,    0,   20,    0,   19,
			    0,   17,    0,   16,   15,    0,  103,    0,  101,    0,
			  100,    0,   13,    0,    0,   11,    0,    0,    0,   10,
			    0,    0,    0,   94,    7,   92,    0,    0,    6,    0,
			    0,   90,    0,    0,   87,    0,    0,   86,    0,    0,

			    0,    0,   84,   83,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,   78,    0,    0,    0,
			    0,    0,    0,    0,    0,   74,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,   68,    0,    0,
			   67,    0,   66,    0,    0,    0,    0,   65,   64,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,   60,    0,    0,    0,    0,   59,   58,    0,   56,
			    0,    0,    0,    0,    0,    0,    0,   55,    0,    0,
			   54,    0,    0,   52,    0,   51,    0,    0,    0,  201,

			   48,    0,    0,    0,    0,    0,    0,    0,   47,   46,
			    0,    0,    0,   45,    0,    0,    0,    0,    0,    0,
			    0,   44,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,   42,   41,   40,    0,    0,   39,    0,   38,
			    0,    0,    0,    0,    0,    0,    0,    0,   35,  799,
			    0,    0,    0,   34,    0,    0,    0,    0,    0,   33,
			    0,    0,    0,   31,    0,   29,    0,    0,    0,   27,
			  108,    0,   25,    0,    0,    0,    0,    0,    0,   22,
			    0,   21,    0,    0,  107,    0,  105,    0,   20,    0,
			   19,    0,   17,    0,   16,   15,    0,  103,    0,  101,

			    0,  100,    0,   13,    0,    0,   11,    0,    0,    0,
			   10,    0,    0,    0,   94,    7,   92,    0,    0,    6,
			    0,    0,   90,    0,    0,   87,    0,    0,   86,    0,
			    0,    0,    0,   84,   83,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,   78,    0,    0,
			    0,    0,    0,    0,    0,    0,   74,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,   68,    0,
			    0,   67,    0,   66,    0,    0,    0,    0,   65,   64,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,

			    0,    0,   60,    0,    0,    0,    0,   59,   58,    0,
			   56,    0,    0,    0,    0,    0,    0,    0,   55,    0,
			    0,   54,    0,    0,   52,    0,   51,    0,    0,    0,
			  201,   48,    0,    0,    0,    0,    0,    0,    0,   47,
			   46,    0,    0,    0,   45,    0,    0,    0,    0,    0,
			    0,    0,   44,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,   42,   41,   40,    0,    0,   39,    0,
			   38,    0,    0,    0,    0,    0,    0,    0,    0,   35,
			    0,    0,    0,    0,   34,    0,    0,    0,    0,    0,
			   33,    0,    0,    0,   31,    0,   29,    0,    0,    0,

			   27,    0,    0,   25,    0,    0,    0,    0,    0,    0,
			   22,    0,   21,    0,    0,    0,    0,    0,    0,   20,
			    0,   19,    0,   17,    0,   16,   15,    0,  458,  457,
			    0,  456,    0,  455,   13,    0,    0,   11,    0,    0,
			    0,   10,    0,    0,  452,  451,    7,  108,    0,    0,
			    6,    0,    0,    0,    0,    0,    0,    0,  450,    0,
			    0,  107,    0,  105,    0,    0,    0,    0,    0,    0,
			    0,    0, 1003,    0,  103,    0,  101,    0,  100,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,   94,    0,   92,    0,    0,    0,    0,    0,   90,

			    0,    0,   87,    0,    0,   86,    0,    0,    0,    0,
			   84,   83,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,   78,    0,    0,    0,    0,    0,
			    0,    0,    0,   74,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,   68,    0,    0,   67,    0,
			   66,    0,    0,    0,    0,   65,   64,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,   60,
			    0,    0,    0,    0,   59,   58,    0,   56,    0,    0,
			    0,    0,    0,    0,    0,   55,    0,    0,   54,    0,

			    0,   52,    0,   51,    0,    0,    0,  201,   48,    0,
			    0,    0,    0,    0,    0,    0,   47,   46,    0,    0,
			    0,   45,    0,    0,    0,    0,    0,    0,    0,   44,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			   42,   41,   40,    0,    0,   39,    0,   38,    0,    0,
			    0,    0,    0,    0,    0,    0,   35,    0,    0, 1812,
			    0,   34,    0, 1344,    0,    0,    0,   33,    0,    0,
			    0,   31,    0,   29,    0,    0,    0,   27,  108,    0,
			   25,    0,    0,    0,    0,    0,    0,   22,    0,   21,
			    0,    0,  107,    0,  105,    0,   20,    0,   19,    0,

			   17,    0,   16,   15,    0,  103,    0,  101,    0,  100,
			    0,   13,    0,    0,   11,    0,    0,    0,   10,    0,
			    0,    0,   94,    7,   92,    0,    0,    6,    0,    0,
			   90,    0,    0,   87,    0,    0,   86,    0,    0,    0,
			    0,   84,   83,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,   78,    0,    0,    0,    0,
			    0,    0,    0,    0,   74,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,  692,    0,   68,    0,    0,   67,
			    0,   66,    0,    0,    0,    0,   65,   64,    0,    0, yyDummy>>,
			1, 1000, 19000)
		end

	yytable_template_21 (an_array: ARRAY [INTEGER])
			-- Fill chunk #21 of template for `yytable'.
		do
			yyarray_subcopy (an_array, <<
			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			   60,    0,    0,    0,    0,   59,   58,    0,   56,    0,
			    0,    0,    0,    0,    0,    0,   55,    0,    0,   54,
			    0,    0,   52,    0,   51,    0,    0,    0,  201,   48,
			    0,    0,    0,    0,    0,    0,    0,   47,   46,    0,
			    0,    0,   45,    0,    0,    0,    0,    0,    0,    0,
			   44,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,   42,   41,   40,    0,    0,   39,    0,   38,    0,
			    0,    0,    0,    0,    0,    0,    0,   35,    0,    0,
			 1345,    0,   34,    0, 1344,    0,    0,    0,   33,    0,

			    0,    0,   31,    0,   29,    0,    0,    0,   27,  108,
			    0,   25,    0,    0,    0,    0,    0,    0,   22,    0,
			   21,    0,    0,  107,    0,  105,    0,   20,    0,   19,
			    0,   17,    0,   16,   15,    0,  103,    0,  101,    0,
			  100,    0,   13,    0,    0,   11,    0,    0,    0,   10,
			    0,    0,    0,   94,    7,   92,    0,    0,    6,    0,
			    0,   90,    0,    0,   87,    0,    0,   86,    0,    0,
			    0,    0,   84,   83,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,   78,    0,    0,    0,
			    0,    0,    0,    0,    0,   74,    0,    0,    0,    0,

			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,  692,    0,   68,    0,    0,
			   67,    0,   66,    0,    0,    0,    0,   65,   64,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,   60,    0,    0,    0,    0,   59,   58,    0,   56,
			    0,    0,    0,    0,    0,    0,    0,   55,    0,    0,
			   54,    0,    0,   52,    0,   51,    0,    0,    0,  201,
			   48,    0,    0,    0,    0,    0,    0,    0,   47,   46,
			    0,    0,    0,   45,    0,    0,    0,    0,    0,    0,
			    0,   44,    0,    0,    0,    0,    0,    0,    0,    0,

			    0,    0,   42,   41,   40,    0,    0,   39,    0,   38,
			    0,    0,    0,    0,    0,    0,    0,    0,   35,    0,
			    0,  758,    0,   34,    0,  757,    0,    0,    0,   33,
			    0,    0,    0,   31,    0,   29,    0,    0,    0,   27,
			  108,    0,   25,    0,    0,    0,    0,    0,    0,   22,
			    0,   21,    0,    0,  107,    0,  105,    0,   20,    0,
			   19,    0,   17,    0,   16,   15,    0,  103,    0,  101,
			    0,  100,    0,   13,    0,    0,   11,    0,    0,    0,
			   10,    0,    0,    0,   94,    7,   92,    0,    0,    6,
			    0,    0,   90,    0,    0,   87,    0,    0,   86,    0,

			    0,    0,    0,   84,   83,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,   78,    0,    0,
			    0,    0,    0,    0,    0,    0,   74,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,  692,    0,   68,    0,
			    0,   67,    0,   66,    0,    0,    0,    0,   65,   64,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,   60,    0,    0,    0,    0,   59,   58,    0,
			   56,    0,    0,    0,    0,    0,    0,    0,   55,    0,
			    0,   54,    0,    0,   52,    0,   51,    0,    0,    0,

			  201,   48,    0,    0,    0,    0,    0,    0,    0,   47,
			   46,    0,    0,    0,   45,    0,    0,    0,    0,    0,
			    0,    0,   44,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,   42,   41,   40,    0,    0,   39,    0,
			   38,    0,    0,    0,    0,    0,    0,    0,    0,   35,
			    0,    0,  224,    0,   34,    0,    0,    0,    0,    0,
			   33,    0,    0,    0,   31,    0,   29,    0,    0,    0,
			   27,  108,    0,   25,    0,    0,    0,    0,    0,    0,
			   22,    0,   21,    0,    0,  107,    0,  105,    0,   20,
			    0,   19,    0,   17,    0,   16,   15,    0,  103,    0,

			  101,    0,  100,    0,   13,    0,    0,   11,    0,    0,
			    0,   10,    0,    0,    0,   94,    7,   92,    0,    0,
			    6,    0,    0,   90,    0,    0,   87,    0,    0,   86,
			    0,    0,    0,    0,   84,   83,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,   78,    0,
			    0,    0,    0,    0,    0,    0,    0,   74,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,   68,
			    0,    0,   67,    0,   66,    0,    0,    0,    0,   65,
			   64,    0,    0,    0,    0,    0,    0,    0,    0,    0,

			    0,    0,    0,   60,    0,    0,    0,    0,   59,   58,
			    0,   56,    0,    0,    0,    0,    0,    0,    0,   55,
			    0,    0,   54,    0,    0,   52,    0,   51,    0,    0,
			    0,  201,   48,    0,    0,    0,    0,    0,    0,    0,
			   47,   46,    0,    0,    0,   45,    0,    0,    0,    0,
			    0,    0,    0,   44,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,   42,   41,   40,    0,    0,   39,
			    0,   38,    0,    0,    0,    0,    0,    0,    0,    0,
			   35,    0,    0,    0,    0,   34,    0,    0,    0,    0,
			    0,   33,    0,    0,    0,   31,    0,   29,    0,    0,

			    0,   27,    0,    0,   25,    0, 1595,    0,    0,    0,
			    0,   22,    0,   21,    0,    0,    0,    0,    0,    0,
			   20,    0,   19,    0,   17,  108,   16,   15,    0,    0,
			    0,  197,    0,    0,    0,   13,    0,    0,   11,  107,
			    0,  105,   10,    0,    0,    0,    0,    7,    0,    0,
			    0,    6,  103,    0,  101,    0,  100,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,   94,
			    0,   92,    0,    0,    0,    0,    0,   90,    0,    0,
			   87,    0,    0,   86,    0,    0,    0,    0,   84,   83,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,

			    0,    0,   78,    0,    0,    0,    0,    0,    0,    0,
			    0,   74,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,  692,    0,   68,    0,    0,   67,    0,   66,    0,
			    0,    0,    0,   65,   64,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,   60,    0,    0,
			    0,    0,   59,   58,    0,   56,    0,    0,    0,    0,
			    0,    0,    0,   55,    0,    0,   54,    0,    0,   52,
			    0,   51,    0,    0,    0,  201,   48,    0,    0,    0,
			    0,    0,    0,    0,   47,   46,    0,    0,    0,   45, yyDummy>>,
			1, 1000, 20000)
		end

	yytable_template_22 (an_array: ARRAY [INTEGER])
			-- Fill chunk #22 of template for `yytable'.
		do
			yyarray_subcopy (an_array, <<
			    0,    0,    0,    0,    0,    0,    0,   44,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,   42,   41,
			   40,    0,    0,   39,    0,   38,    0,    0,    0,    0,
			    0,    0,    0,    0,   35,    0,    0,  758,    0,   34,
			    0, 1141,    0,    0,    0,   33,    0,    0,    0,   31,
			    0,   29,    0,    0,    0,   27,  108,    0,   25,    0,
			    0,    0,    0,    0,    0,   22,    0,   21,    0,    0,
			  107,    0,  105,    0,   20,    0,   19,    0,   17,    0,
			   16,   15,    0,  103,    0,  101,    0,  100,    0,   13,
			    0,    0,   11,    0,    0,    0,   10,    0,    0,    0,

			   94,    7,   92,    0,    0,    6,    0,    0,   90,    0,
			    0,   87,    0,    0,   86,    0,    0,    0,    0,   84,
			   83,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,   78,    0,    0,    0,    0,    0,    0,
			    0,    0,   74,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,   68,    0,    0,   67,    0,   66,
			    0,    0,    0,    0,   65,   64,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,   60,    0,
			    0,    0,    0,   59,   58,    0,   56,    0,    0,    0,

			    0,    0,    0,    0,   55,    0,    0,   54,    0,    0,
			   52,    0,   51,    0,    0,    0,  201,   48,    0,    0,
			    0,    0,    0,    0,    0,   47,   46,    0,    0,    0,
			   45,    0,    0,    0,    0,    0,    0,    0,   44,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,   42,
			   41,   40,    0,    0,   39,    0,   38,    0,    0,    0,
			    0,    0,    0,    0,    0,   35,    0,    0, 1812,    0,
			   34,    0,    0,    0,    0,    0,   33,    0,    0,    0,
			   31,    0,   29,    0,    0,    0,   27,  108,    0,   25,
			    0,    0,    0,    0,    0,    0,   22,    0,   21,    0,

			    0,  107,    0,  105,    0,   20,    0,   19,    0,   17,
			    0,   16,   15,    0,  103,    0,  101,    0,  100,    0,
			   13,    0,    0,   11,    0,    0,    0,   10,    0,    0,
			    0,   94,    7,   92,    0,    0,    6,    0,    0,   90,
			    0,    0,   87,    0,    0,   86,    0,    0,    0,    0,
			   84,   83,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,   78,    0,    0,    0,    0,    0,
			    0,    0,    0,   74,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,   68,    0,    0,   67,    0,

			   66,    0,    0,    0,    0,   65,   64,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,   60,
			    0,    0,    0,    0,   59,   58,    0,   56,    0,    0,
			    0,    0,    0,    0,    0,   55,    0,    0,   54,    0,
			    0,   52,    0,   51,    0,    0,    0,  201,   48,    0,
			    0,    0,    0,    0,    0,    0,   47,   46,    0,    0,
			    0,   45,    0,    0,    0,    0,    0,    0,    0,   44,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			   42,   41,   40,    0,    0,   39,    0,   38,    0,    0,
			    0,    0,    0,    0,    0,    0,   35,    0,    0, 1595,

			    0,   34,    0,    0,    0,    0,    0,   33,    0,    0,
			    0,   31,    0,   29,    0,    0,    0,   27,  108,    0,
			   25,    0,    0,    0,    0,    0,    0,   22,    0,   21,
			    0,    0,  107,    0,  105,    0,   20,    0,   19,    0,
			   17,    0,   16,   15,    0,  103,    0,  101,    0,  100,
			    0,   13,    0,    0,   11,    0,    0,    0,   10,    0,
			    0,    0,   94,    7,   92,    0,    0,    6,    0,    0,
			   90,    0,    0,   87,    0,    0,   86,    0,    0,    0,
			    0,   84,   83,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,   78,    0,    0,    0,    0,

			    0,    0,    0,    0,   74,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,   68,    0,    0,   67,
			    0,   66,    0,    0,    0,    0,   65,   64,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			   60,    0,    0,    0,    0,   59,   58,    0,   56,    0,
			    0,    0,    0,    0,    0,    0,   55,    0,    0,   54,
			    0,    0,   52,    0,   51,    0,    0,    0,  201,   48,
			    0,    0,    0,    0,    0,    0,    0,   47,   46,    0,
			    0,    0,   45,    0,    0,    0,    0,    0,    0,    0,

			   44,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,   42,   41,   40,    0,    0,   39,    0,   38,    0,
			    0,    0,    0,    0,    0,    0,    0,   35,    0,    0,
			 1345,    0,   34,    0,    0,    0,    0,    0,   33,    0,
			    0,    0,   31,    0,   29,    0,    0,    0,   27,  108,
			    0,   25,    0,    0,    0,    0,    0,    0,   22,    0,
			   21,    0,    0,  107,    0,  105,    0,   20,    0,   19,
			    0,   17,    0,   16,   15,    0,  103,    0,  101,    0,
			  100,    0,   13,    0,    0,   11,    0,    0,    0,   10,
			    0,    0,    0,   94,    7,   92,    0,    0,    6,    0,

			    0,   90,    0,    0,   87,    0,    0,   86,    0,    0,
			    0,    0,   84,   83,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,   78,    0,    0,    0,
			    0,    0,    0,    0,    0,   74,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,   68,    0,    0,
			   67,    0,   66,    0,    0,    0,    0,   65,   64,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,   60,    0,    0,    0,    0,   59,   58,    0,   56,
			    0,    0,    0,    0,    0,    0,    0,   55,    0,    0,

			   54,    0,    0,   52,    0,   51,    0,    0,    0,  201,
			   48,    0,    0,    0,    0,    0,    0,    0,   47,   46,
			    0,    0,    0,   45,    0,    0,    0,    0,    0,    0,
			    0,   44,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,   42,   41,   40,    0,    0,   39,    0,   38,
			    0,    0,    0,    0,    0,    0,    0,    0,   35,    0,
			    0,    0,    0,   34,    0,    0,    0,    0,    0,   33,
			    0,    0,    0,   31,    0,   29,    0,    0,  634,   27,
			    0,    0,   25,    0,    0,    0,    0,    0,    0,   22,
			    0,   21,    0,    0,    0,  108,    0,    0,   20,    0, yyDummy>>,
			1, 1000, 21000)
		end

	yytable_template_23 (an_array: ARRAY [INTEGER])
			-- Fill chunk #23 of template for `yytable'.
		do
			yyarray_subcopy (an_array, <<
			   19,    0,   17,    0,   16,   15,    0,    0,    0,  107,
			    0,  105,    0,   13,    0,    0,   11,  633,    0,    0,
			   10,    0,  103,    0,  101,    7,  100,    0,    0,    6,
			    0,    0,    0,    0,    0,    0,    0,  632,    0,   94,
			    0,   92,    0,    0,    0,  631,    0,   90,    0,    0,
			   87,    0,    0,   86,    0,    0,    0,    0,   84,   83,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,   78,  630,  629,    0,  628,    0,    0,    0,
			    0,   74,    0,    0,    0,  627,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,

			    0,    0,    0,   68,  626,    0,   67,    0,   66,    0,
			    0,    0,    0,   65,   64,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,   60,    0,    0,
			    0,    0,   59,   58,    0,   56,    0,  625,    0,    0,
			    0,    0,    0,   55,    0,    0,   54,    0,    0,   52,
			    0,   51,    0,  624,    0,  201,   48,    0,    0,    0,
			    0,    0,    0,    0,   47,   46,    0,  623,    0,   45,
			    0,    0,    0,    0,    0,  622,  621,   44,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,   42,   41,
			   40,    0,    0,   39,    0,   38,    0,    0,    0,    0,

			    0,    0,    0,    0,   35,  620,    0,    0,    0,   34,
			    0,    0,    0,    0,    0,   33,    0,    0,    0,   31,
			    0,   29,    0,    0,    0,   27,    0,    0,   25,    0,
			    0,  619,    0,    0,    0,   22,    0,   21, 1344,    0,
			    0,    0,    0,    0,   20,    0,   19,    0,   17,  618,
			   16,   15,    0,  108,  617,  616,    0,    0,    0,   13,
			    0,    0,   11,    0,    0,    0,   10,  107,    0,  105,
			    0,    7,    0,  615,  614,    6,    0,    0,    0,    0,
			  103,    0,  101,    0,  100,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,   94,    0,   92,

			    0,    0,    0,    0,    0,   90,    0,    0,   87,    0,
			    0,   86,    0,    0,    0,    0,   84,   83,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			   78,    0,    0,    0,    0,    0,    0,    0,    0,   74,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,  692,
			    0,   68,    0,    0,   67,    0,   66,    0,    0,    0,
			    0,   65,   64,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,   60,    0,    0,    0,    0,
			   59,   58,    0,   56,    0,    0,    0,    0,    0,    0,

			    0,   55,    0,    0,   54,    0,    0,   52,    0,   51,
			    0,    0,    0,  201,   48,    0,    0,    0,    0,    0,
			    0,    0,   47,   46,    0,    0,    0,   45,    0,    0,
			    0,    0,    0,    0,    0,   44,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,   42,   41,   40,    0,
			    0,   39,    0,   38,    0,    0,    0,    0,    0,    0,
			    0,    0,   35,    0,    0,    0,    0,   34,    0,  969,
			    0,    0,    0,   33,    0,    0,    0,   31,    0,   29,
			    0,    0,    0,   27,  108,    0,   25,    0,    0,    0,
			    0,    0,    0,   22,    0,   21,    0,    0,  107,    0,

			  105,    0,   20,    0,   19,    0,   17,    0,   16,   15,
			    0,  103,    0,  101,    0,  100,    0,   13,    0,    0,
			   11,    0,    0,    0,   10,    0,    0,    0,   94,    7,
			   92,    0,    0,    6,    0,    0,   90,    0,    0,   87,
			    0,    0,   86,    0,    0,    0,    0,   84,   83,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,   78,    0,    0,    0,    0,    0,    0,    0,    0,
			   74,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			  692,    0,   68,    0,    0,   67,    0,   66,    0,    0,

			    0,    0,   65,   64,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,   60,    0,    0,    0,
			    0,   59,   58,    0,   56,    0,    0,    0,    0,    0,
			    0,    0,   55,    0,    0,   54,    0,    0,   52,    0,
			   51,    0,    0,    0,  201,   48,    0,    0,    0,    0,
			    0,    0,    0,   47,   46,    0,    0,    0,   45,    0,
			    0,    0,    0,    0,    0,    0,   44,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,   42,   41,   40,
			    0,    0,   39,    0,   38,    0,    0,    0,    0,    0,
			    0,    0,    0,   35,    0,    0,    0,    0,   34,    0,

			  693,    0,    0,    0,   33,    0,    0,    0,   31,    0,
			   29,    0,    0,    0,   27,  108,    0,   25,    0,    0,
			    0,    0,    0,    0,   22,    0,   21,    0,    0,  107,
			    0,  105,    0,   20,    0,   19,    0,   17,    0,   16,
			   15,    0,  103,    0,  101,    0,  100,    0,   13,    0,
			    0,   11,    0,    0,    0,   10,    0,    0,    0,   94,
			    7,   92,    0,    0,    6,    0,    0,   90,    0,    0,
			   87,    0,    0,   86,    0,    0,    0,    0,   84,   83,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,   78,    0,    0,    0,    0,    0,    0,    0,

			    0,   74,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,  692,    0,   68,    0,    0,   67,    0,   66,    0,
			    0,    0,    0,   65,   64,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,   60,    0,    0,
			    0,    0,   59,   58,    0,   56,    0,    0,    0,    0,
			    0,    0,    0,   55,    0,    0,   54,    0,    0,   52,
			    0,   51,    0,    0,    0,  201,   48,    0,    0,    0,
			    0,    0,    0,    0,   47,   46,    0,    0,    0,   45,
			    0,    0,    0,    0,    0,    0,    0,   44,    0,    0,

			    0,    0,    0,    0,    0,    0,    0,    0,   42,   41,
			   40,    0,    0,   39,    0,   38,    0,    0,    0,    0,
			    0,    0,    0,    0,   35,    0,    0,    0,    0,   34,
			    0, 2241,    0,    0,    0,   33,    0,    0,    0,   31,
			    0,   29,    0,    0,    0,   27,  108,    0,   25,    0,
			    0,    0,    0,    0,    0,   22,    0,   21,    0,    0,
			  107,    0,  105,    0,   20,    0,   19,    0,   17,    0,
			   16,   15,    0,  103,    0,  101,    0,  100,    0,   13,
			    0,    0,   11,    0,    0,    0,   10,    0,    0,    0,
			   94,    7,   92,    0,    0,    6,    0,    0,   90,    0, yyDummy>>,
			1, 1000, 22000)
		end

	yytable_template_24 (an_array: ARRAY [INTEGER])
			-- Fill chunk #24 of template for `yytable'.
		do
			yyarray_subcopy (an_array, <<
			    0,   87,    0,    0,   86,    0,    0,    0,    0,   84,
			   83,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,   78,    0,    0,    0,    0,    0,    0,
			    0,    0,   74,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,   68,    0,    0,   67,    0,   66,
			    0,    0,    0,    0,   65,   64,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,   60,    0,
			    0,    0,    0,   59,   58,    0,   56,    0,    0,    0,
			    0,    0,    0,    0,   55,    0,    0,   54,    0,    0,

			   52,    0,   51,    0,    0,    0,  201,   48,    0,    0,
			    0,    0,    0,    0,    0,   47,   46,    0,    0,    0,
			   45,    0,    0,    0,    0,    0,    0,    0,   44,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,   42,
			   41,   40,    0,    0,   39,    0,   38,    0,    0,    0,
			    0,    0,    0,    0,    0,   35,    0,    0,    0,    0,
			   34,    0, 2084,    0,    0,    0,   33,    0,    0,    0,
			   31,    0,   29,    0,    0,    0,   27,  108,    0,   25,
			    0,    0,    0,    0,    0,    0,   22,    0,   21,    0,
			    0,  107,    0,  105,    0,   20,    0,   19,    0,   17,

			    0,   16,   15,    0,  103,    0,  101,    0,  100,    0,
			   13,    0,    0,   11,    0,    0,    0,   10,    0,    0,
			    0,   94,    7,   92,    0,    0,    6,    0,    0,   90,
			    0,    0,   87,    0,    0,   86,    0,    0,    0,    0,
			   84,   83,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,   78,    0,    0,    0,    0,    0,
			    0,    0,    0,   74,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,   68,    0,    0,   67,    0,
			   66,    0,    0,    0,    0,   65,   64,    0,    0,    0,

			    0,    0,    0,    0,    0,    0,    0,    0,    0,   60,
			    0,    0,    0,    0,   59,   58,    0,   56,    0,    0,
			    0,    0,    0,    0,    0,   55,    0,    0,   54,    0,
			    0,   52,    0,   51,    0,    0,    0,  201,   48,    0,
			    0,    0,    0,    0,    0,    0,   47,   46,    0,    0,
			    0,   45,    0,    0,    0,    0,    0,    0,    0,   44,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			   42,   41,   40,    0,    0,   39,    0,   38,    0,    0,
			    0,    0,    0,    0,    0,    0,   35,    0,    0,    0,
			    0,   34,    0, 1255,    0,    0,    0,   33,    0,    0,

			    0,   31,    0,   29,    0,    0,    0,   27,  108,    0,
			   25,    0,    0,    0,    0,    0,    0,   22,    0,   21,
			    0,    0,  107,    0,  105,    0,   20,    0,   19,    0,
			   17,    0,   16,   15,    0,  103,    0,  101,    0,  100,
			    0,   13,    0,    0,   11,    0,    0,    0,   10,    0,
			    0,    0,   94,    7,   92,    0,    0,    6,    0,    0,
			   90,    0,    0,   87,    0,    0,   86,    0,    0,    0,
			    0,   84,   83,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,   78,    0,    0,    0,    0,
			    0,    0,    0,    0,   74,    0,    0,    0,    0,    0,

			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,   68,    0,    0,   67,
			    0,   66,    0,    0,    0,    0,   65,   64,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			   60,    0,    0,    0,    0,   59,   58,    0,   56,    0,
			    0,    0,    0,    0,    0,    0,   55,    0,    0,   54,
			    0,    0,   52,    0,   51,    0,    0,    0,  201,   48,
			    0,    0,    0,    0,    0,    0,    0,   47,   46,    0,
			    0,    0,   45,    0,    0,    0,    0,    0,    0,    0,
			   44,    0,    0,    0,    0,    0,    0,    0,    0,    0,

			    0,   42,   41,   40,    0,    0,   39,    0,   38,    0,
			    0,    0,    0,    0,    0,    0,    0,   35,    0,    0,
			    0,    0,   34,    0, 1203,    0,    0,    0,   33,    0,
			    0,    0,   31,    0,   29,    0,    0,    0,   27,  108,
			    0,   25,    0,    0,    0,    0,    0,    0,   22,    0,
			   21,    0,    0,  107,    0,  105,    0,   20,    0,   19,
			    0,   17,    0,   16,   15,    0,  103,    0,  101,    0,
			  100,    0,   13,    0,    0,   11,    0,    0,    0,   10,
			    0,    0,    0,   94,    7,   92,    0,    0,    6,    0,
			    0,   90,    0,    0,   87,    0,    0,   86,    0,    0,

			    0,    0,   84,   83,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,   78,    0,    0,    0,
			    0,    0,    0,    0,    0,   74,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,   68,    0,    0,
			   67,    0,   66,    0,    0,    0,    0,   65,   64,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,   60,    0,    0,    0,    0,   59,   58,    0,   56,
			    0,    0,    0,    0,    0,    0,    0,   55,    0,    0,
			   54,    0,    0,   52,    0,   51,    0,    0,    0,  201,

			   48,    0,    0,    0,    0,    0,    0,    0,   47,   46,
			    0,    0,    0,   45,    0,    0,    0,    0,    0,    0,
			    0,   44,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,   42,   41,   40,    0,    0,   39,    0,   38,
			    0,    0,    0,    0,    0,    0,    0,    0,   35,    0,
			    0,    0,    0,   34,    0, 1199,    0,    0,    0,   33,
			    0,    0,    0,   31,    0,   29,    0,    0,    0,   27,
			  108,    0,   25,    0,    0,    0,    0,    0,    0,   22,
			    0,   21,    0,    0,  107,    0,  105,    0,   20,    0,
			   19,    0,   17,    0,   16,   15,    0,  103,    0,  101,

			    0,  100,    0,   13,    0,    0,   11,    0,    0,    0,
			   10,    0,    0,    0,   94,    7,   92,    0,    0,    6,
			    0,    0,   90,    0,    0,   87,    0,    0,   86,    0,
			    0,    0,    0,   84,   83,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,   78,    0,    0,
			    0,    0,    0,    0,    0,    0,   74,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,   68,    0,
			    0,   67,    0,   66,    0,    0,    0,    0,   65,   64,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0, yyDummy>>,
			1, 1000, 23000)
		end

	yytable_template_25 (an_array: ARRAY [INTEGER])
			-- Fill chunk #25 of template for `yytable'.
		do
			yyarray_subcopy (an_array, <<
			    0,    0,   60,    0,    0,    0,    0,   59,   58,    0,
			   56,    0,    0,    0,    0,    0,    0,    0,   55,    0,
			    0,   54,    0,    0,   52,    0,   51,    0,    0,    0,
			  201,   48,    0,    0,    0,    0,    0,    0,    0,   47,
			   46,    0,    0,    0,   45,    0,    0,    0,    0,    0,
			    0,    0,   44,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,   42,   41,   40,    0,    0,   39,    0,
			   38,    0,    0,    0,    0,    0,    0,    0,    0,   35,
			    0,    0,    0,    0,   34,    0, 1165,    0,    0,    0,
			   33,    0,    0,    0,   31,    0,   29,    0,    0,    0,

			   27,  108,    0,   25,    0,    0,    0,    0,    0,    0,
			   22,    0,   21,    0,    0,  107,    0,  105,    0,   20,
			    0,   19,    0,   17,    0,   16,   15,    0,  103,    0,
			  101,    0,  100,    0,   13,    0,    0,   11,    0,    0,
			    0,   10,    0,    0,    0,   94,    7,   92,    0,    0,
			    6,    0,    0,   90,    0,    0,   87,    0,    0,   86,
			    0,    0,    0,    0,   84,   83,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,   78,    0,
			    0,    0,    0,    0,    0,    0,    0,   74,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,

			    0,    0,    0,    0,    0,    0,    0,    0,    0,   68,
			    0,    0,   67,    0,   66,    0,    0,    0,    0,   65,
			   64,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,   60,    0,    0,    0,    0,   59,   58,
			    0,   56,    0,    0,    0,    0,    0,    0,    0,   55,
			    0,    0,   54,    0,    0,   52,    0,   51,    0,    0,
			    0,  201,   48,    0,    0,    0,    0,    0,    0,    0,
			   47,   46,    0,    0,    0,   45,    0,    0,    0,    0,
			    0,    0,    0,   44,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,   42,   41,   40,    0,    0,   39,

			    0,   38,    0,    0,    0,    0,    0,    0,    0,    0,
			   35,    0,    0,    0,    0,   34,    0, 1028,    0,    0,
			    0,   33,    0,    0,    0,   31,    0,   29,    0,    0,
			    0,   27,  108,    0,   25,    0,    0,    0,    0,    0,
			    0,   22,    0,   21,    0,    0,  107,    0,  105,    0,
			   20,    0,   19,    0,   17,    0,   16,   15,    0,  103,
			    0,  101,    0,  100,    0,   13,    0,    0,   11,    0,
			    0,    0,   10,    0,    0,    0,   94,    7,   92,    0,
			    0,    6,    0,    0,   90,    0,    0,   87,    0,    0,
			   86,    0,    0,    0,    0,   84,   83,    0,    0,    0,

			    0,    0,    0,    0,    0,    0,    0,    0,    0,   78,
			    0,    0,    0,    0,    0,    0,    0,    0,   74,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			   68,    0,    0,   67,    0,   66,    0,    0,    0,    0,
			   65,   64,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,   60,    0,    0,    0,    0,   59,
			   58,    0,   56,    0,    0,    0,    0,    0,    0,    0,
			   55,    0,    0,   54,    0,    0,   52,    0,   51,    0,
			    0,    0,  201,   48,    0,    0,    0,    0,    0,    0,

			    0,   47,   46,    0,    0,    0,   45,    0,    0,    0,
			    0,    0,    0,    0,   44,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,   42,   41,   40,    0,    0,
			   39,    0,   38,    0,    0,    0,    0,    0,    0,    0,
			    0,   35,    0,    0,    0,    0,   34,    0,  797,    0,
			    0,    0,   33,    0,    0,    0,   31,    0,   29,    0,
			    0,    0,   27,  108,    0,   25,    0,    0,    0,    0,
			    0,    0,   22,    0,   21,    0,    0,  107,    0,  105,
			    0,   20,    0,   19,    0,   17,    0,   16,   15,    0,
			  103,    0,  101,    0,  100,    0,   13,    0,    0,   11,

			    0,    0,    0,   10,    0,    0,    0,   94,    7,   92,
			    0,    0,    6,    0,    0,   90,    0,    0,   87,    0,
			    0,   86,    0,    0,    0,    0,   84,   83,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			   78,    0,    0,    0,    0,    0,    0,    0,    0,   74,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,   68,    0,    0,   67,    0,   66,    0,    0,    0,
			    0,   65,   64,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,   60,    0,    0,    0,    0,

			   59,   58,    0,   56,    0,    0,    0,    0,    0,    0,
			    0,   55,    0,    0,   54,    0,    0,   52,    0,   51,
			    0,    0,    0,  201,   48,    0,    0,    0,    0,    0,
			    0,    0,   47,   46,    0,    0,    0,   45,    0,    0,
			    0,    0,    0,    0,    0,   44,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,   42,   41,   40,    0,
			    0,   39,    0,   38,    0,    0,    0,    0,    0,    0,
			    0,    0,   35,    0,    0,    0,    0,   34,    0,    0,
			    0,    0,    0,   33,    0,    0,    0,   31,    0,   29,
			    0,    0,    0,   27,  108,    0,   25,    0,    0,    0,

			    0,    0,    0,   22,    0,   21,    0,    0,  107,    0,
			  105,    0,   20,    0,   19,    0,   17,    0,   16,   15,
			    0,  103,    0,  101,    0,  100,    0,   13,    0,    0,
			   11,    0,    0,    0,   10,    0,    0,    0,   94,    7,
			   92,    0,    0,    6,    0,    0,   90,    0,    0,   87,
			    0,    0,   86,    0,    0,    0,    0,   84,   83,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,   78,    0,    0,    0,    0,    0,    0,    0,    0,
			   74,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,

			    0,    0,   68,    0,    0,   67,    0,   66,    0,    0,
			    0,    0,   65,   64,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,   60,    0,    0,    0,
			    0,   59,   58,    0,   56,    0,    0,    0,    0,    0,
			    0,    0,   55,    0,    0,   54,    0,    0,   52,    0,
			   51,    0,    0,    0,  201,   48,    0,    0,    0,    0,
			    0,    0,    0,   47,   46,    0,    0,    0,   45,    0,
			    0,    0,    0,    0,    0,  927,   44,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,   42,   41,   40,
			    0,    0,   39,    0,   38,    0,    0,    0,    0,    0, yyDummy>>,
			1, 1000, 24000)
		end

	yytable_template_26 (an_array: ARRAY [INTEGER])
			-- Fill chunk #26 of template for `yytable'.
		do
			yyarray_subcopy (an_array, <<
			    0,    0,    0,   35,    0,    0,    0,    0,   34,    0,
			    0,    0,    0,    0,   33,    0,    0,    0,   31,    0,
			   29,    0,    0,    0,   27,    0,    0,   25,    0,    0,
			    0,    0,    0,    0,   22,    0,   21,  108,    0,    0,
			    0,    0,    0,   20,    0,   19,    0,   17,    0,   16,
			   15,  107,    0,  105,    0,    0,    0,    0,   13,  633,
			    0,   11,    0,    0,  103,   10,  101,    0,  100,    0,
			    7,    0,  926,  925,    6,    0,    0,    0,    0,  632,
			    0,   94, 1651,   92,    0,    0,    0,  631,    0,   90,
			    0,    0,   87,    0,    0,   86,    0,    0,    0,    0,

			   84,   83,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,   78,  630,  629,    0,  628,    0,
			    0,    0,    0,   74,    0,    0,    0,  627,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,   68,  626,    0,   67,    0,
			   66,    0,    0,    0,    0,   65,   64,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,   60,
			    0,    0,    0,    0,   59,   58,    0,   56,    0,  625,
			    0,    0,    0,    0,    0,   55,    0,    0,   54,    0,
			    0,   52,    0,   51,    0,  624,    0,  201,   48,    0,

			    0,    0,    0,    0,    0,    0,   47,   46,    0,  623,
			    0,   45,    0,    0,    0,    0,    0,  622,  621,   44,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			   42,   41,   40,    0,    0,   39,    0,   38,    0,    0,
			    0,    0,    0,    0,    0,    0,   35,  620,    0,    0,
			    0,   34,    0,    0,    0,    0,    0,   33,    0,    0,
			    0,   31,    0,   29,    0,    0,    0,   27,    0,    0,
			   25,    0,    0,  619,    0,    0,    0,   22,    0,   21,
			    0,    0,    0,    0,    0,    0,   20,    0,   19,    0,
			   17,  618,   16,   15,    0,    0,  617,  616,  108,    0,

			    0,   13,    0,    0,   11,    0,    0,    0,   10,    0,
			    0,    0,  107,    7,  105,  615,  614,    6,    0,    0,
			  633,    0,    0, 1433,    0,  103,    0,  101,    0,  100,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			  632,    0,   94,    0,   92,    0,    0,    0, 1432,    0,
			   90,    0,    0,   87,    0,    0,   86,    0,    0,    0,
			    0,   84,   83,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,   78,  630,  629,    0,  628,
			    0,    0,    0,    0,   74,    0,    0,    0,  627,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,

			    0,    0,    0,    0,    0,    0,   68,  626,    0,   67,
			    0,   66,    0,    0,    0,    0,   65,   64,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			   60,    0,    0,    0,    0,   59,   58,    0,   56,    0,
			  625,    0,    0,    0,    0,    0,   55,    0,    0,   54,
			    0,    0,   52,    0,   51,    0, 1431,    0,  201,   48,
			    0,    0,    0,    0,    0,    0,    0,   47,   46,    0,
			  623,    0,   45,    0,    0,    0,    0,    0,  622,  621,
			   44,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,   42,   41,   40,    0,    0,   39,    0,   38,    0,

			    0,    0,    0,    0,    0,    0,    0,   35,  620,    0,
			    0,    0,   34,    0,    0,    0,    0,    0,   33,    0,
			    0,    0,   31,    0,   29,    0,    0,    0,   27,    0,
			    0,   25,    0,    0,  619,    0,    0,    0,   22,    0,
			   21,    0,    0,    0,    0,    0,    0,   20,    0,   19,
			    0,   17,  108,   16,   15,    0,    0,  617,  616,    0,
			    0,    0,   13,    0,    0,   11,  107,    0,  105,   10,
			    0,  308,    0,    0,    7,    0,  615, 1430,    6,  103,
			    0,  101,    0,  100,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,   94,    0,   92,    0,

			    0,    0,    0,    0,   90,    0,    0,   87,    0,    0,
			   86,    0,    0,    0,    0,   84,   83,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,   78,
			    0,    0,    0,    0,    0,    0,  277,    0,   74,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,  307,    0,    0,    0,    0,    0,
			   68,    0,    0,   67,    0,   66,    0,    0,    0,    0,
			   65,   64,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,   60,    0,    0,    0,    0,   59,
			   58,  275,   56,    0,    0,    0,    0,    0,    0,    0,

			   55,    0,    0,   54,    0,    0,   52,    0,   51,    0,
			    0,    0,  201,   48,    0,    0,    0,    0,    0,    0,
			    0,   47,   46,    0,    0,    0,   45,    0,    0,    0,
			    0,    0,    0,    0,   44,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,   42,   41,   40,    0,    0,
			   39,    0,   38,    0,    0,    0,    0,    0,    0,  274,
			    0,   35,    0,    0,  273,    0,   34,    0,    0,    0,
			    0,    0,   33,    0,    0,    0,  306,    0,   29,    0,
			    0,    0,   27,  108,   26,   25,    0,    0,    0,    0,
			    0,    0,   22,    0,   21,    0,    0,  107,    0,  105,

			    0,   20,  278,   19,    0,   17,    0,   16,   15,    0,
			  103,    0,  101,    0,  100,    0,   13,    0,    0,   11,
			  305,    0,    0,   10,  270,    0,    0,   94,    7,   92,
			    0,    0,    6,    0,    0,   90,    0,    0,   87,    0,
			    0,   86,    0,    0,    0,    0,   84,   83,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			   78,    0,    0,    0,    0,    0,    0,  277,    0,   74,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,  276,    0,    0,    0,    0,
			    0,   68,    0,    0,   67,    0,   66,    0,    0,    0,

			    0,   65,   64,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,   60,    0,    0,    0,    0,
			   59,   58,  275,   56,    0,    0,    0,    0,    0,    0,
			    0,   55,    0,    0,   54,    0,    0,   52,    0,   51,
			    0,    0,    0,  201,   48,    0,    0,    0,    0,    0,
			    0,    0,   47,   46,    0,    0,    0,   45,    0,    0,
			    0,    0,    0,    0,    0,   44,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,   42,   41,   40,    0,
			    0,   39,    0,   38,    0,    0,    0,    0,    0,    0,
			  274,    0,   35,    0,    0,  273,    0,   34,    0,    0, yyDummy>>,
			1, 1000, 25000)
		end

	yytable_template_27 (an_array: ARRAY [INTEGER])
			-- Fill chunk #27 of template for `yytable'.
		do
			yyarray_subcopy (an_array, <<
			    0,    0,    0,   33,    0,    0,    0,  272,    0,   29,
			    0,    0,    0,   27,  108,   26,   25,    0,    0,    0,
			    0,    0,    0,   22,    0,   21,    0,    0,  107,    0,
			  105,    0,   20,    0,   19,    0,   17,    0,   16,   15,
			    0,  103,    0,  101,    0,  100,    0,   13,    0,    0,
			   11,  271,    0,    0,   10,  270,    0,    0,   94,    7,
			   92,    0,    0,    6,  382,    0,   90,    0,    0,   87,
			    0,    0,   86,    0,    0,    0,    0,   84,   83,    0,
			    0,  381,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,   78,    0,    0,    0,    0,    0,    0,    0,    0,

			   74,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			  380,    0,   68,    0,    0,   67,    0,   66,    0,    0,
			    0,    0,   65,   64,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,   60,    0,    0,    0,
			    0,   59,   58,    0,   56,    0,    0,    0,    0,    0,
			    0,    0,   55,    0,    0,   54,    0,  379,   52,    0,
			   51,  378,    0,    0,  201,   48,    0,    0,    0,    0,
			    0,    0,    0,   47,   46,    0,    0,    0,   45,    0,
			    0,    0,    0,    0,    0,    0,   44,    0,    0,    0,

			    0,    0,    0,    0,    0,    0,    0,   42,   41,   40,
			    0,    0,   39,    0,   38,    0,    0,    0,    0,    0,
			    0,    0,    0,   35,    0,    0,    0,    0,   34,    0,
			    0,    0,    0,    0,   33,    0,    0,    0,   31,  377,
			   29,    0,    0,    0,   27,  108,    0,   25,    0,    0,
			    0,    0,    0,    0,   22,    0,   21,    0,    0,  107,
			    0,  105,    0,   20,    0,   19,    0,   17,    0,   16,
			   15,    0,  103,    0,  101,  376,  100,    0,   13,    0,
			    0,   11,    0,    0,    0,   10,    0,    0,  375,   94,
			    7,   92,    0,    0,    6,    0,    0,   90,    0,    0,

			   87,    0,    0,   86,    0,    0,    0,    0,   84,   83,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,   78,    0,    0,    0,    0,    0,    0,    0,
			    0,   74,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,   68,    0,    0,   67,    0,   66,    0,
			    0,    0,    0,   65,   64,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,   60,    0,    0,
			    0,    0,   59,   58,    0,   56,    0,    0,    0,    0,
			    0,    0,    0,   55,    0,    0,   54,    0,    0,   52,

			    0,   51,    0,    0,    0,  201,   48,    0,    0,    0,
			    0,    0,    0,    0,   47,   46,    0,    0,    0,   45,
			  803,    0,    0,    0,    0,    0,    0,   44,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,   42,   41,
			   40,    0,    0,   39,    0,   38,    0,    0,    0,    0,
			    0,    0,    0,    0,   35,    0,    0,    0,    0,   34,
			    0,    0,    0,    0,    0,   33,    0,    0,    0,   31,
			    0,   29,    0,    0,  802,   27,    0,    0,   25,    0,
			    0,    0,    0,    0,    0,   22,    0,   21,    0,    0,
			    0,    0,    0,    0,   20,    0,   19,  108,   17,    0,

			   16,   15,    0,    0,  808,  807,    0,    0,    0,   13,
			    0,  107,   11,  105,    0,    0,   10,    0,    0,    0,
			    0,    7,    0,    0,  103,    6,  101,    0,  100,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,   94,    0,   92,    0,    0,    0,    0,    0,   90,
			    0,    0,   87,    0,    0,   86,    0,    0,    0, 1753,
			   84,   83,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,   78,    0,    0,    0,    0,    0,
			    0,    0,    0,   74,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,

			    0,    0,    0,    0,    0,   68,    0,    0,   67,    0,
			   66,    0,    0,    0,    0,   65,   64,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,   60,
			    0,    0,    0,    0,   59,   58,    0,   56,    0,    0,
			    0,    0,    0,    0,    0,   55,    0,    0,   54,    0,
			    0,   52,    0,   51,    0,    0,    0,  201,   48,    0,
			    0,    0,    0,    0,    0,    0,   47,   46,    0,    0,
			    0,   45,    0,    0,    0,    0,    0,    0,    0,   44,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			   42,   41,   40,    0,    0,   39,    0,   38,    0,    0,

			    0,    0,    0,    0,    0,    0,   35,    0,    0,    0,
			    0,   34,    0,    0,    0,    0,    0,   33,    0,    0,
			    0,   31,    0,   29,    0,    0,    0,   27,    0,    0,
			   25,    0,    0,    0,    0,    0,    0,   22,    0,   21,
			    0,    0,    0,    0,    0,    0,   20,    0,   19,    0,
			   17,    0,   16,   15,    0,  108,  808,  807,    0,    0,
			    0,   13,    0,    0,   11, 1181,    0,    0,   10,  107,
			    0,  105,    0,    7,    0,    0,    0,    6,    0,    0,
			    0,    0,  103,    0,  101,    0,  100,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,   94,

			    0,   92,    0,    0,    0,    0,    0,   90,    0,    0,
			   87,    0,    0,   86,    0,    0,    0, 1182,   84,   83,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,   78,    0,    0,    0,    0,    0,    0,    0,
			    0,   74,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,   68,    0,    0,   67,    0,   66,    0,
			    0,    0,    0,   65,   64,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,   60,    0,    0,
			    0,    0,   59,   58,    0,   56,    0,    0,    0,    0,

			    0,    0,    0,   55,    0,    0,   54,    0,    0,   52,
			    0,   51,    0,    0,    0,  201,   48,    0,    0,    0,
			    0,    0,    0,    0,   47,   46,    0,    0,    0,   45,
			    0,    0,    0,    0,    0,    0,    0,   44,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,   42,   41,
			   40,    0,    0,   39,    0,   38,    0,    0,    0,    0,
			    0,    0,    0,    0,   35,    0,    0,    0,    0,   34,
			    0,    0,    0,    0,    0,   33,    0,    0,    0,   31,
			    0,   29,    0,    0,    0,   27,    0,    0,   25,    0,
			    0,    0,    0,    0,    0,   22,    0,   21,    0,    0, yyDummy>>,
			1, 1000, 26000)
		end

	yytable_template_28 (an_array: ARRAY [INTEGER])
			-- Fill chunk #28 of template for `yytable'.
		do
			yyarray_subcopy (an_array, <<
			    0,    0,    0,    0,   20,    0,   19,    0,   17,    0,
			   16,   15,    0,  108,  808,  807,    0,    0,    0,   13,
			    0,    0,   11, 1181,    0,    0,   10,  107,    0,  105,
			    0,    7,    0,    0,    0,    6,    0,    0,    0,    0,
			  103,    0,  101,    0,  100,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,   94,    0,   92,
			    0,    0,    0,    0,    0,   90,    0,    0,   87,    0,
			    0,   86,    0,    0,    0,    0,   84,   83,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			   78,    0,    0,    0,    0,    0,    0,    0,    0,   74,

			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,   68,    0,    0,   67,    0,   66,    0,    0,    0,
			    0,   65,   64,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,   60,    0, 1088,    0,    0,
			   59,   58,    0,   56,    0,    0,    0,    0,    0,    0,
			    0,   55,    0,    0,   54,    0,    0,   52,    0,   51,
			    0,    0,    0,  201,   48,    0,    0,    0,    0,    0,
			    0,    0,   47,   46,    0,    0,    0,   45, 1087,    0,
			    0,    0,    0,    0,    0,   44,    0,    0,    0,    0,

			    0,    0,    0,    0,    0,    0,   42,   41,   40,    0,
			    0,   39,    0,   38,    0,    0,    0,    0,    0,    0,
			    0,    0,   35,    0,    0,    0,    0,   34,    0,    0,
			    0,    0,    0,   33,    0,    0,    0,   31,    0,   29,
			    0,    0, 1086,   27,  108,    0,   25,    0,    0,    0,
			    0,    0,    0,   22,    0,   21,    0,    0,  107,    0,
			  105,    0,   20,    0,   19, 1282,   17,    0,   16,   15,
			    0,  103,    0,  101,    0,  100,    0,   13,    0,    0,
			   11,    0,    0,    0,   10,    0,    0,    0,   94,    7,
			   92,    0,    0,    6,    0,    0,   90,    0,    0,   87,

			    0,    0,   86,    0,    0,    0,    0,   84,   83,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,   78,    0,    0,    0,    0,    0,    0,    0,    0,
			   74,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,   68,    0,    0,   67,    0,   66,    0,    0,
			    0,    0,   65,   64,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,   60,    0,    0,    0,
			    0,   59,   58,    0,   56,    0, 1281,    0,    0,    0,
			    0,    0,   55,    0,    0,   54,    0,    0,   52,    0,

			   51,    0,    0,    0,  201,   48,    0,    0,    0,    0,
			 1280,    0,    0,   47,   46,    0,    0,    0,   45,    0,
			    0,    0,    0,    0,    0,    0,   44,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,   42,   41,   40,
			    0,    0,   39,    0,   38,    0,    0,    0,    0,    0,
			    0,    0,    0,   35,    0,    0,    0,    0,   34,    0,
			    0,    0,    0,    0,   33,    0,    0,    0,   31,    0,
			   29,    0,    0,    0,   27,  108,    0,   25,    0,    0,
			    0,    0,    0,    0,   22,    0,   21,    0,    0,  107,
			    0,  105,    0,   20,    0,   19,    0,   17,    0,   16,

			   15,    0,  103,    0,  101,    0,  100,    0,   13,    0,
			    0,   11,    0,    0,    0,   10,    0,    0,    0,   94,
			    7,   92,    0,    0,    6,    0,    0,   90,    0,    0,
			   87,    0,    0,   86,    0,    0,    0,    0,   84,   83,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,   78,    0,    0,    0,    0,    0,    0,    0,
			    0,   74,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,   68,    0,    0,   67,    0,   66,    0,
			    0,    0,    0,   65,   64,    0,    0,    0,    0,    0,

			    0,    0,    0,    0,    0,    0,    0,   60,    0,    0,
			    0,    0,   59,   58,    0,   56,    0,    0,    0,    0,
			    0,    0,    0,   55,    0,    0,   54,    0,   53,   52,
			    0,   51,  202,    0,    0,  201,   48,    0,    0,    0,
			    0,    0,    0,    0,   47,   46,    0,    0,    0,   45,
			    0,    0,    0,    0,    0,    0,    0,   44,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,   42,   41,
			   40,    0,    0,   39,    0,   38,    0,    0,    0,    0,
			    0,    0,    0,    0,   35,    0,    0,    0,    0,   34,
			    0,    0,    0,    0,    0,   33,    0,    0,    0,   31,

			   30,   29,    0,    0,    0,   27,  108,    0,   25,    0,
			    0,    0,    0,    0,    0,   22,    0,   21,    0,    0,
			  107,    0,  105,    0,   20,    0,   19,    0,   17,    0,
			   16,   15,    0,  103,    0,  101,    0,  100,    0,   13,
			    0,    0,   11,    0,    0,    0,   10,    0,    0,    0,
			   94,    7,   92,    0,    0,    6,    0,    0,   90,    0,
			    0,   87,    0,    0,   86,    0,    0,    0,    0,   84,
			   83,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,   78,    0,    0,    0,    0,    0,    0,
			    0,    0,   74,    0,    0,    0,    0,    0,    0,    0,

			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,   68,    0,    0,   67,    0,   66,
			    0,    0,    0,    0,   65,   64,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,   60,    0,
			    0,    0,    0,   59,   58,    0,   56,    0,    0,    0,
			    0,    0,    0,    0,   55,    0,    0,   54,    0,    0,
			   52,    0,   51,    0,    0,    0,  201,   48,    0,    0,
			    0,    0,    0,    0,    0,   47,   46,    0,    0,    0,
			   45,    0,    0,    0,    0,    0,    0,    0,   44,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,   42,

			   41,   40,    0,    0,   39,    0,   38,    0,    0,    0,
			    0,    0,    0,    0,    0,   35,    0,    0,    0,    0,
			   34,    0,    0,    0,    0,    0,   33,    0,    0,    0,
			   31,    0,   29,    0,    0,    0,   27,    0,    0,   25,
			    0,    0,    0,    0,    0,    0,   22,    0,   21,    0,
			    0,    0,    0,    0,    0,   20,    0,   19,  108,   17,
			    0,   16,   15,    0,    0,  808,  807,    0,    0,    0,
			   13,    0,  107,   11,  105,    0,    0,   10,    0,    0,
			    0,    0,    7, 1969,    0,  103,    6,  101,    0,  100,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0, yyDummy>>,
			1, 1000, 27000)
		end

	yytable_template_29 (an_array: ARRAY [INTEGER])
			-- Fill chunk #29 of template for `yytable'.
		do
			yyarray_subcopy (an_array, <<
			    0,    0,   94,    0,   92,    0,    0,    0,    0,    0,
			   90,    0,    0,   87,    0,    0,   86,    0,    0,    0,
			    0,   84,   83,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,   78,    0,    0,    0,    0,
			    0,    0,    0,    0,   74,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,   68,    0,    0,   67,
			    0,   66,    0,    0,    0,    0,   65,   64,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			   60,    0,    0,    0,    0,   59,   58,    0,   56,    0,

			    0,    0,    0,    0,    0,    0,   55,    0,    0,   54,
			    0,    0,   52,    0,   51,    0,    0,    0,  201,   48,
			    0,    0,    0,    0,    0,    0,    0,   47,   46,    0,
			    0,    0,   45,    0,    0,    0,    0,    0,    0,    0,
			   44,    0,    0,    0,    0,    0,    0,  995,    0,    0,
			    0,   42,   41,   40,    0,    0,   39,    0,   38,    0,
			    0,    0,    0,    0,    0,    0,    0,   35,    0,    0,
			    0,    0,   34,    0,    0,    0,    0,    0,   33,    0,
			    0,    0,   31,    0,   29,    0,    0,    0,   27,  108,
			    0,   25,    0,    0,    0,    0,    0,    0,   22,    0,

			   21,    0,    0,  107,    0,  105,    0,   20,    0,   19,
			    0,   17,    0,   16,   15,    0,  103,    0,  101,    0,
			  100,    0,   13,    0,    0,   11,    0,    0,    0,   10,
			    0,    0,    0,   94,    7,   92,    0,    0,    6,    0,
			    0,   90,    0,    0,   87,    0,    0,   86,    0,    0,
			    0,    0,   84,   83,    0, 1073,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,   78,    0,    0,    0,
			    0,    0,    0,    0,    0,   74,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,    0, 1072,    0,    0,    0,   68,    0,    0,

			   67,    0,   66,    0,    0,    0,    0,   65,   64,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,   60,    0,    0,    0,    0,   59,   58,    0,   56,
			    0,    0,    0,    0,    0,    0,    0,   55,    0,    0,
			   54,    0,    0,   52,    0,   51,    0,    0,    0,  201,
			   48,    0,    0,    0,    0,    0,    0,    0,   47,   46,
			    0,    0,    0,   45,    0,    0,    0,    0,    0,    0,
			    0,   44,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,   42,   41,   40,    0,    0,   39,    0,   38,
			    0,    0,    0,    0,    0,    0,    0,    0,   35,    0,

			    0,    0,    0,   34,    0,    0,    0,    0,    0,   33,
			    0,    0,    0,   31,    0,   29,    0,    0,    0,   27,
			    0,    0,   25,    0,    0,    0,    0,    0,    0,   22,
			    0,   21,  108,    0,    0,    0,    0,    0,   20,    0,
			   19,    0,   17,    0,   16,   15,  107,    0,  105,    0,
			    0,    0,    0,   13,    0,    0,   11,  837,    0,  103,
			   10,  101,    0,  100,    0,    7,    0,    0,    0,    6,
			    0,    0,    0,    0,    0,    0,   94,    0,   92,    0,
			    0,    0,    0,    0,   90,    0,    0,   87,    0,    0,
			   86,    0,    0,    0,    0,   84,   83,    0,    0,    0,

			    0,    0,    0,    0,    0,    0,    0,    0,    0,   78,
			    0,    0,    0,    0,    0,    0,    0,    0,   74,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			   68,    0,    0,   67,    0,   66,    0,    0,    0,    0,
			   65,   64,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,   60,    0,    0,    0,    0,   59,
			   58,    0,   56,    0,    0,    0,    0,    0,    0,    0,
			   55,    0,    0,   54,    0,    0,   52,    0,   51,    0,
			    0,    0,  201,   48,    0,    0,    0,    0,    0,    0,

			    0,   47,   46,    0,    0,    0,   45,    0,    0,    0,
			    0,    0,    0,    0,   44,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,   42,   41,   40,    0,    0,
			   39,    0,   38,    0,    0,    0,    0,    0,    0,    0,
			    0,   35,    0,    0,    0,    0,   34,    0,    0,    0,
			    0,    0,   33,    0,    0,    0,   31,    0,   29,    0,
			    0,    0,   27,  108,    0,   25,    0,    0,    0,    0,
			    0,    0,   22,    0,   21,    0,    0,  107,    0,  105,
			    0,   20,    0,   19,    0,   17,  805,   16,   15,    0,
			  103,    0,  101,    0,  100,    0,   13,    0,    0,   11,

			    0,    0,    0,   10,    0,    0,    0,   94,    7,   92,
			    0,    0,    6,    0,    0,   90,    0,    0,   87,    0,
			    0,   86,    0,    0,    0,    0,   84,   83,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			   78,    0,    0,    0,    0,    0,    0,    0,    0,   74,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,   68,    0,    0,   67,    0,   66,    0,  334,    0,
			    0,   65,   64,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,   60,    0,    0,    0,    0,

			   59,   58,    0,   56,    0,    0,    0,    0,    0,    0,
			    0,   55,    0,    0,   54,    0,    0,   52,    0,   51,
			    0,    0,    0,  201,   48,    0,    0,    0,    0,    0,
			    0,    0,   47,   46,    0,    0,    0,   45,    0,    0,
			    0,    0,    0,    0,    0,   44,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,   42,   41,   40,    0,
			    0,   39,    0,   38,    0,    0,    0,    0,    0,    0,
			    0,    0,   35,    0,    0,    0,    0,   34,    0,    0,
			    0,    0,    0,   33,    0,    0,    0,   31,    0,   29,
			    0,    0,    0,   27,  108,    0,   25,    0,    0,    0,

			    0,    0,    0,   22,    0,   21,    0,    0,  107,    0,
			  105,    0,   20,    0,   19,    0,   17,    0,   16,   15,
			    0,  103,    0,  101,    0,  100,    0,   13,    0,    0,
			   11,    0,    0,    0,   10,    0,    0,  333,   94,    7,
			   92,    0,    0,    6,    0,    0,   90,    0,    0,   87,
			    0,    0,   86,    0,    0,    0,    0,   84,   83,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,   78,    0,    0,    0,    0,    0,    0,    0,    0,
			   74,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0, yyDummy>>,
			1, 1000, 28000)
		end

	yytable_template_30 (an_array: ARRAY [INTEGER])
			-- Fill chunk #30 of template for `yytable'.
		do
			yyarray_subcopy (an_array, <<
			    0,    0,   68,    0,    0,   67,    0,   66,    0,    0,
			    0,    0,   65,   64,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,   60,    0,    0,    0,
			    0,   59,   58,    0,   56,    0,    0,    0,    0,    0,
			    0,    0,   55,    0,    0,   54,    0,    0,   52,    0,
			   51,    0,    0,    0,  201,   48,    0,    0,    0,    0,
			    0,    0,    0,   47,   46,    0,    0,    0,   45,    0,
			    0,    0,    0,    0,    0,    0,   44,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,   42,   41,   40,
			  267,    0,   39,    0,   38,    0,    0,    0,    0,    0,

			    0,    0,    0,   35,    0,    0,    0,    0,   34,    0,
			    0,    0,    0,    0,   33,    0,    0,    0,   31,    0,
			   29,    0,    0,    0,   27,    0,    0,   25,    0,    0,
			    0,    0,    0,    0,   22,    0,   21,    0,    0,    0,
			    0,    0,    0,   20,    0,   19,  108,   17,    0,   16,
			   15,    0,    0,    0,    0,    0,    0,    0,   13,    0,
			  107,   11,  105,    0,    0,   10,    0,    0,  266,    0,
			    7, 2133,    0,  103,    6,  101,    0,  100,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			   94,    0,   92,    0,    0,    0,    0,    0,   90,    0,

			    0,   87,    0,    0,   86,    0,    0,    0,    0,   84,
			   83,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,   78,    0,    0,    0,    0,    0,    0,
			    0,    0,   74,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,   68,    0,    0,   67,    0,   66,
			    0,    0,    0,    0,   65,   64,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,   60,    0,
			    0,    0,    0,   59,   58,    0,   56,    0,    0,    0,
			    0,    0,    0,    0,   55,    0,    0,   54,    0,    0,

			   52,    0,   51,    0,    0,    0,  201,   48,    0,    0,
			    0,    0,    0,    0,    0,   47,   46,    0,    0,    0,
			   45,    0,    0,    0,    0,    0,    0,    0,   44,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,   42,
			   41,   40,    0,    0,   39,    0,   38,    0,    0,    0,
			    0,    0,    0,    0,    0,   35,    0,    0,    0,    0,
			   34,    0,    0,    0,    0,    0,   33,    0,    0,    0,
			   31,    0,   29,    0,    0,    0,   27,  108,    0,   25,
			    0,    0,    0,    0,    0,    0,   22,    0,   21,    0,
			    0,  107,    0,  105,    0,   20,    0,   19,    0,   17,

			    0,   16,   15,    0,  103,    0,  101,    0,  100,    0,
			   13,    0,    0,   11,    0,    0,    0,   10,    0,    0,
			    0,   94,    7,   92,    0,    0,    6,    0,    0,   90,
			    0,    0,   87,    0,    0,   86,    0,    0,    0,    0,
			   84,   83,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,   78,    0,    0,    0,    0,    0,
			    0,    0,    0,   74,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,  692,    0,   68,    0,    0,   67,    0,
			   66,    0,    0,    0,    0,   65,   64,    0,    0,    0,

			    0,    0,    0,    0,    0,    0,    0,    0,    0,   60,
			    0,    0,    0,    0,   59,   58,    0,   56,    0,    0,
			    0,    0,    0,    0,    0,   55,    0,    0,   54,    0,
			    0,   52,    0,   51,    0,    0,    0,  201,   48,    0,
			    0,    0,    0,    0,    0,    0,   47,   46,    0,    0,
			    0,   45,    0,    0,    0,    0,    0,    0,    0,   44,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			   42,   41,   40,    0,    0,   39,    0,   38,    0,    0,
			    0,    0,    0,    0,    0,    0,   35,    0,    0,    0,
			    0,   34,    0,    0,    0,    0,    0,   33,    0,    0,

			    0,   31,    0,   29,    0,    0,    0,   27,    0,    0,
			   25,    0,    0,    0,    0,    0,    0,   22,    0,   21,
			  108,    0,    0,    0,    0,    0,   20,    0,   19,    0,
			   17,    0,   16,   15,  107,    0,  105,    0,    0,    0,
			    0,   13,    0,    0,   11, 1969,    0,  103,   10,  101,
			    0,  100,    0,    7,    0,    0,    0,    6,    0,    0,
			    0,    0,    0,    0,   94,    0,   92,    0,    0,    0,
			    0,    0,   90,    0,    0,   87,    0,    0,   86,    0,
			    0,    0,    0,   84,   83,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,   78,    0,    0,

			    0,    0,    0,    0,    0,    0,   74,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,   68,    0,
			    0,   67,    0,   66,    0,    0,    0,    0,   65,   64,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,   60,    0,    0,    0,    0,   59,   58,    0,
			   56,    0,    0,    0,    0,    0,    0,    0,   55,    0,
			    0,   54,    0,    0,   52,    0,   51,    0,    0,    0,
			  201,   48,    0,    0,    0,    0,    0,    0,    0,   47,
			   46,    0,    0,    0,   45,    0,    0,    0,    0,    0,

			    0,    0,   44,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,   42,   41,   40,    0,    0,   39,    0,
			   38,    0,    0,    0,    0,    0,    0,    0,    0,   35,
			    0,    0,    0,    0,   34,    0,    0,    0,    0,    0,
			   33,    0,    0,    0,   31,    0,   29,    0,    0,    0,
			   27,  108,    0,   25,    0,    0,    0,    0,    0,    0,
			   22,    0,   21,    0,    0,  107,    0,  105,    0,   20,
			    0,   19,    0,   17,    0,   16,   15,    0,  103,    0,
			  101,    0,  100,    0,   13,    0,    0,   11,    0,    0,
			    0,   10,    0,    0,    0,   94,    7,   92,    0,    0,

			    6,    0,    0,   90,    0,    0,   87,    0,    0,   86,
			    0,    0,    0,    0,   84,   83,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,   78,    0,
			    0,    0,    0,    0,    0,    0,    0,   74,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,   68,
			    0,    0,   67,    0,   66,    0,    0,    0,    0,   65,
			   64,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,   60,    0,    0,    0,    0,   59,   58,
			    0,   56,    0,    0,    0,    0,    0,    0,    0,   55, yyDummy>>,
			1, 1000, 29000)
		end

	yytable_template_31 (an_array: ARRAY [INTEGER])
			-- Fill chunk #31 of template for `yytable'.
		do
			yyarray_subcopy (an_array, <<
			    0,    0,   54,    0,    0,   52,    0,   51,    0,    0,
			    0,  201,   48,    0,    0,    0,    0,    0,    0,    0,
			   47,   46,    0,    0,    0,   45,    0,    0,    0,    0,
			    0,    0,    0,   44,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,   42,   41,   40,    0,    0,   39,
			    0,   38,    0,    0, 1819,    0,    0,    0,    0,    0,
			   35,    0,    0,    0,    0,   34,    0,    0,    0,    0,
			    0,   33,    0,    0,    0,   31,    0,   29,    0,    0,
			    0,   27,  108,    0,   25,    0,    0,    0,    0,    0,
			    0,   22,    0,   21,    0,    0,  107,    0,  105,    0,

			   20,    0,   19,    0,   17,    0,   16, 1818,    0,  103,
			    0,  101,    0,  100,    0,   13,    0,    0,   11,    0,
			    0,    0,   10,    0,    0,    0,   94,    7,   92,    0,
			    0,    6,    0,    0,   90,    0,    0,   87,    0,    0,
			   86,    0,    0,    0,    0,   84,   83,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,   78,
			    0,    0,    0,    0,    0,    0,    0,    0,   74,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			   68,    0,    0,   67,    0,   66,    0,    0,    0,    0,

			   65,   64,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,   60,    0,    0,    0,    0,   59,
			   58,    0,   56,    0,    0,    0,    0,    0,    0,    0,
			   55,    0,    0,   54,    0,    0,   52,    0,   51,    0,
			    0,    0,  201,   48,    0,    0,    0,    0,    0,    0,
			    0,   47,   46,    0,    0,    0,   45,    0,    0,    0,
			    0,    0,    0,    0,   44,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,   42,   41,   40,    0,    0,
			   39,    0,   38,    0,    0,    0,    0,    0,    0,  274,
			    0,   35,    0,    0,    0,    0,   34,    0,    0,    0,

			    0,    0,   33,    0,    0,    0,   31,    0,   29,    0,
			    0,    0,   27,    0,    0,   25,    0,    0,    0,    0,
			    0,    0,   22,    0,   21,    0,    0,    0,  108,    0,
			    0,   20,    0,   19,    0,   17,    0,   16,   15,    0,
			    0,    0,  107,    0,  105,    0,   13,    0,    0,   11,
			    0,    0,  935,   10,    0,  103,    0,  101,    7,  100,
			    0,    0,    6,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,   94,    0,   92,    0,    0,    0,    0,    0,
			   90,    0,    0,   87,    0,    0,   86,    0,    0,    0,
			    0,   84,   83,    0,    0,    0,    0,    0,    0,    0,

			    0,    0,    0,    0,    0,   78,    0,    0,    0,    0,
			    0,    0,    0,    0,   74,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,   68,    0,    0,   67,
			    0,   66,    0,    0,    0,    0,   65,   64,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			   60,    0,    0,    0,    0,   59,   58,    0,   56,    0,
			    0,    0,    0,    0,    0,    0,   55,    0,    0,   54,
			    0,    0,   52,    0,   51,    0,    0,    0,  201,   48,
			    0,    0,    0,    0,    0,    0,    0,   47,   46,    0,

			    0,    0,   45,    0,    0,    0,    0,    0,    0,    0,
			   44,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,   42,   41,   40,    0,    0,   39,    0,   38,    0,
			    0,    0,    0,    0,    0,    0,    0,   35,    0,    0,
			    0,    0,   34,    0,    0,    0,    0,    0,   33,    0,
			    0,    0,   31,    0,   29,    0,    0,    0,   27,  108,
			    0,   25,    0,    0,    0,    0,    0,    0,   22,    0,
			   21,    0,    0,  107,    0,  105,    0,   20,    0,   19,
			    0,   17,    0,   16,   15,    0,  103,    0,  101,    0,
			  100,    0,   13,    0,    0,   11,    0,    0,    0,   10,

			    0,    0,    0,   94,    7,   92,    0,    0,    6,    0,
			    0,   90,    0,    0,   87,    0,    0,   86,    0,    0,
			    0,    0,   84,   83,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,   78,    0,    0,    0,
			    0,    0,    0,    0,    0,   74,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,   68,    0,    0,
			   67,    0,   66,    0,  334,    0,    0,   65,   64,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,   60,    0,    0,    0,    0,   59,   58,    0,   56,

			    0,    0,    0,    0,    0,    0,    0,   55,    0,    0,
			   54,    0,    0,   52,    0,   51,    0,    0,    0,  201,
			   48,    0,    0,    0,    0,    0,    0,    0,   47,   46,
			    0,    0,    0,   45,    0,    0,    0,    0,    0,    0,
			    0,   44,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,   42,   41,   40,    0,    0,   39,    0,   38,
			    0,    0,    0,    0,    0,    0,    0,    0,   35,    0,
			    0,    0,    0,   34,    0,    0,    0,    0,    0,   33,
			    0,    0,    0,   31,    0,   29,    0,    0,    0,   27,
			  108,    0,   25,    0,    0,    0,    0,    0,    0,   22,

			    0,   21,    0,    0,  107,    0,  105,    0,   20,    0,
			   19,    0,   17,    0,   16,   15,    0,  103,    0,  101,
			    0,  100,    0,   13,    0,    0,   11,    0,    0,    0,
			   10,    0,    0,    0,   94,    7,   92,    0,    0,    6,
			    0,    0,   90,    0,    0,   87,    0,    0,   86,    0,
			    0,    0,    0,   84,   83,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,   78,    0,    0,
			    0,    0,    0,    0,    0,    0,   74,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,   68,    0,

			    0,   67,    0,   66,    0,    0,    0,    0,   65,   64,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,   60,    0,    0,    0,    0,   59,   58,    0,
			   56,    0,    0,    0,    0,    0,    0,    0,   55,    0,
			    0,   54,    0,    0,   52,    0,   51,  653,    0,    0,
			  201,   48,    0,    0,    0,    0,    0,    0,    0,   47,
			   46,    0,    0,    0,   45,    0,    0,    0,    0,    0,
			    0,    0,   44,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,   42,   41,   40,    0,    0,   39,    0,
			   38,    0,    0,    0,    0,    0,    0,    0,    0,   35, yyDummy>>,
			1, 1000, 30000)
		end

	yytable_template_32 (an_array: ARRAY [INTEGER])
			-- Fill chunk #32 of template for `yytable'.
		do
			yyarray_subcopy (an_array, <<
			    0,    0,    0,    0,   34,    0,    0,    0,    0,    0,
			   33,    0,    0,    0,   31,    0,   29,    0,    0,    0,
			   27,  108,    0,   25,    0,    0,    0,    0,    0,    0,
			   22,    0,   21,    0,    0,  107,    0,  105,    0,   20,
			    0,   19,    0,   17,    0,   16,   15,    0,  103,    0,
			  101,    0,  100,    0,   13,    0,    0,   11,    0,    0,
			    0,   10,    0,    0,    0,   94,    7,   92,    0,    0,
			    6,    0,    0,   90,    0,    0,   87,    0,    0,   86,
			    0,    0,    0,    0,   84,   83,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,   78,    0,

			    0,    0,    0,    0,    0,    0,    0,   74,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,   68,
			    0,    0,   67,    0,   66,    0,    0,    0,    0,   65,
			   64,    0,    0,    0,  604,    0,    0,    0,    0,    0,
			    0,    0,    0,   60,    0,    0,    0,    0,   59,   58,
			    0,   56,    0,    0,    0,    0,    0,    0,    0,   55,
			    0,    0,   54,    0,    0,   52,    0,   51,    0,    0,
			    0,  201,   48,    0,    0,    0,    0,    0,    0,    0,
			   47,   46,    0,    0,    0,   45,    0,    0,    0,    0,

			    0,    0,    0,   44,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,   42,   41,   40,    0,    0,   39,
			    0,   38,    0,    0,    0,    0,    0,    0,    0,    0,
			   35,    0,    0,    0,    0,   34,    0,    0,    0,    0,
			    0,   33,    0,    0,    0,   31,    0,   29,    0,    0,
			    0,   27,  108,    0,   25,    0,    0,    0,    0,    0,
			    0,   22,    0,   21,    0,    0,  107,    0,  105,    0,
			   20,    0,   19,    0,   17,    0,   16,   15,    0,  103,
			    0,  101,    0,  100,    0,   13,    0,    0,   11,    0,
			    0,    0,   10,    0,    0,    0,   94,    7,   92,    0,

			    0,    6,    0,    0,   90,    0,    0,   87,    0,    0,
			   86,    0,    0,    0,    0,   84,   83,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,   78,
			    0,    0,    0,    0,    0,    0,    0,    0,   74,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			   68,    0,    0,   67,    0,   66,    0,    0,    0,    0,
			   65,   64,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,   60,    0,    0,    0,    0,   59,
			   58,    0,   56,    0,    0,    0,    0,    0,    0,    0,

			   55,    0,    0,   54,    0,    0,   52,    0,   51,  594,
			    0,    0,  201,   48,    0,    0,    0,    0,    0,    0,
			    0,   47,   46,    0,    0,    0,   45,    0,    0,    0,
			    0,    0,    0,    0,   44,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,   42,   41,   40,    0,    0,
			   39,    0,   38,    0,    0,    0,    0,    0,    0,    0,
			    0,   35,    0,    0,    0,    0,   34,    0,    0,    0,
			    0,    0,   33,    0,    0,    0,   31,    0,   29,    0,
			    0,    0,   27,  108,    0,   25,    0,    0,    0,    0,
			    0,    0,   22,    0,   21,    0,    0,  107,    0,  105,

			    0,   20,    0,   19,    0,   17,    0,   16,   15,    0,
			  103,    0,  101,    0,  100,    0,   13,    0,    0,   11,
			    0,    0,    0,   10,    0,    0,    0,   94,    7,   92,
			    0,    0,    6,    0,    0,   90,    0,    0,   87,    0,
			    0,   86,    0,    0,    0,    0,   84,   83,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			   78,    0,    0,    0,    0,    0,    0,    0,    0,   74,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,   68,    0,    0,   67,    0,   66,    0,    0,    0,

			    0,   65,   64,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,   60,    0,    0,    0,    0,
			   59,   58,    0,   56,    0,    0,    0,    0,    0,    0,
			    0,   55,    0,    0,   54,    0,    0,   52,    0,   51,
			    0,    0,    0,  201,   48,    0,    0,    0,    0,    0,
			    0,    0,   47,   46,    0,    0,    0,   45,    0,    0,
			    0,    0,    0,    0,    0,   44,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,   42,   41,   40,    0,
			    0,   39,    0,   38,    0,    0,    0,    0,    0,    0,
			    0,    0,   35,    0,    0,    0,    0,   34,    0,    0,

			    0,    0,    0,   33,    0,    0,    0,   31,    0,   29,
			    0,    0,    0,   27,  108,    0,   25,    0,    0,    0,
			  257,    0,    0,   22,    0,   21,    0,    0,  107,    0,
			  105,    0,   20,    0,   19,    0,   17,    0,   16,   15,
			    0,  103,    0,  101,    0,  100,    0,   13,    0,    0,
			   11,    0,    0,    0,   10,    0,    0,    0,   94,    7,
			   92,    0,    0,    6,    0,    0,   90,    0,    0,   87,
			    0,    0,   86,    0,    0,    0,    0,   84,   83,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,   78,    0,    0,    0,    0,    0,    0,    0,    0,

			   74,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,   68,    0,    0,   67,    0,   66,    0,    0,
			    0,    0,   65,   64,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,   60,    0,    0,    0,
			    0,   59,   58,    0,   56,    0,    0,    0,    0,    0,
			    0,    0,   55,    0,    0,   54,    0,    0,   52,    0,
			   51,    0,    0,    0,  201,   48,    0,    0,    0,    0,
			    0,    0,    0,   47,   46,    0,    0,    0,   45,    0,
			    0,    0,    0,    0,    0,    0,   44,    0,    0,    0,

			    0,    0,    0,    0,    0,    0,    0,   42,   41,   40,
			    0,    0,   39,    0,   38,    0,    0,    0,    0,    0,
			    0,    0,    0,   35,    0,    0,    0,    0,   34,    0,
			    0,    0,    0,    0,   33,    0,    0,    0,   31,    0,
			   29,    0,    0,    0,   27,  311,    0,   25,    0,    0,
			    0,    0,    0,    0,   22,    0,   21,    0,    0,  107,
			    0,  105,    0,   20,    0,   19,    0,   17,    0,   16,
			   15,    0,  103,    0,  101,    0,  100,    0,   13,    0,
			    0,   11,    0,    0,    0,   10,    0,    0,    0,   94,
			    7,   92,    0,    0,    6,    0,    0,   90,    0,    0, yyDummy>>,
			1, 1000, 31000)
		end

	yytable_template_33 (an_array: ARRAY [INTEGER])
			-- Fill chunk #33 of template for `yytable'.
		do
			yyarray_subcopy (an_array, <<
			   87,    0,    0,   86,    0,    0,    0,    0,   84,   83,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,   78,    0,    0,    0,    0,    0,    0,    0,
			    0,   74,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
			    0,    0,    0,   68,    0,    0,   67,    0,   66,    0,
			    0,    0,    0,   65,   64,    0,    0,    0,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,   60,    0,    0,
			    0,    0,   59,   58,    0,   56,    0,    0,    0,    0,
			    0,    0,    0,   55,    0,    0,   54,    0,    0,   52,

			    0,   51,    0,    0,    0,  201,   48,    0,    0,    0,
			    0,    0,    0,    0,   47,   46,    0,    0,    0,   45,
			    0,    0,    0,    0,    0,    0,    0,   44,    0,    0,
			    0,    0,    0,    0,    0,    0,    0,    0,   42,   41,
			   40,    0,    0,   39,    0,   38,    0,    0,    0,    0,
			    0,    0,    0,    0,   35,    0,    0,    0,    0,   34,
			    0,    0,    0,    0,    0,   33,    0,    0,    0,   31,
			    0,   29,    0,    0,    0,   27,    0,    0,   25,    0,
			    0,    0,    0,    0,    0,   22,    0,   21,    0,    0,
			    0,    0,    0,    0,   20,    0,   19,    0,   17,    0,

			   16,   15,    0,    0,    0,    0,    0,    0,    0,   13,
			    0,    0,   11,    0,    0,    0,   10,    0,    0,    0,
			    0,    7,    0,    0,    0,    6, yyDummy>>,
			1, 226, 32000)
		end

	yycheck_template: SPECIAL [INTEGER]
			-- Template for `yycheck'
		local
			an_array: ARRAY [INTEGER]
		once
			create an_array.make_filled (0, 0, 32225)
			yycheck_template_1 (an_array)
			yycheck_template_2 (an_array)
			yycheck_template_3 (an_array)
			yycheck_template_4 (an_array)
			yycheck_template_5 (an_array)
			yycheck_template_6 (an_array)
			yycheck_template_7 (an_array)
			yycheck_template_8 (an_array)
			yycheck_template_9 (an_array)
			yycheck_template_10 (an_array)
			yycheck_template_11 (an_array)
			yycheck_template_12 (an_array)
			yycheck_template_13 (an_array)
			yycheck_template_14 (an_array)
			yycheck_template_15 (an_array)
			yycheck_template_16 (an_array)
			yycheck_template_17 (an_array)
			yycheck_template_18 (an_array)
			yycheck_template_19 (an_array)
			yycheck_template_20 (an_array)
			yycheck_template_21 (an_array)
			yycheck_template_22 (an_array)
			yycheck_template_23 (an_array)
			yycheck_template_24 (an_array)
			yycheck_template_25 (an_array)
			yycheck_template_26 (an_array)
			yycheck_template_27 (an_array)
			yycheck_template_28 (an_array)
			yycheck_template_29 (an_array)
			yycheck_template_30 (an_array)
			yycheck_template_31 (an_array)
			yycheck_template_32 (an_array)
			yycheck_template_33 (an_array)
			Result := yyfixed_array (an_array)
		end

	yycheck_template_1 (an_array: ARRAY [INTEGER])
			-- Fill chunk #1 of template for `yycheck'.
		do
			yyarray_subcopy (an_array, <<
			    2,    2,  227,  485,    2,    2,    8,    9,    2,    2,
			  298,  291,  302,  303,  314,  427,  852,  225,   69,    2,
			  461,  690,   24,  395,  407,  691,   77,  916,  417, 1437,
			   32,  200, 1061,    9,    2,   37, 1370,   88,  966, 1264,
			 1523,   43,    2, 1210,  234,  348,  507, 1541,  461,    2,
			  851,  225, 1260,  356,    2,    2,  303, 1011,  506,  362,
			  988,   63,  314,  366,  273,  437,  410,   69, 1535,   71,
			 1537,  921,    2,   75,   76,   77,  285,    2,  494,  473,
			  688,  561, 1690, 1013, 1418,  711,   88,  713,  754,  665,
			 1602,   93,  300,   95,   96,   97, 1793, 1699,   93,   75,

			 1920, 1921,   97,  953, 1568,    3,  989,   32, 1572, 1120,
			  686,  711, 1836,  713, 1838,    2, 1760,  291,   19, 1763,
			   48,    7,    0,    7, 1924,   80,  300,  427,  302,  303,
			    3,    7,    3,   11,   26, 1536, 1537,   89,   63,  254,
			  314,    7,  108, 1937,  112,   86, 1744, 1294, 1295,   58,
			    5,    6,   58,    8,  571,   10,   15,   16,   53,  112,
			 1328,    3, 1271, 1272,  112,  537,   21,   22,  337,  449,
			 1100,  461, 1061,   48,    5,    6,  811,    8,  467,   10,
			   35,  118,    7,   27,  121,   64,    7,  239,    3,  855,
			   21,   22,   87, 1295,   43,    3,  487,  163,  200,   12,

			  129,   12,   12,   26,   35,  139,   24,  209,   61,  211,
			  435, 1960,  420, 1962,   71, 1964,  211,   14,  255,   12,
			  510,  390,   50,   13,  607,  433,  434,  688,   25,   26,
			   27,   28,   29,   30,   24,   32,   49,  171,   49,   49,
			   37, 1171,    0,   92,  123,  200,  317,  176,  422,    7,
			 1658,  206,  323,   11,  129,  257,   49,  259,  310,  261,
			  434,   58,  333,  510, 1293,  267,   63,    0,  305,  164,
			  176,  461,  343,  564,    5,    6,   71,    8,  913,   10,
			  351,  109,    0, 1385, 1295,  261, 2106,  461,   63,  168,
			   21,   22, 2016,   11, 2018,  297, 1504, 2021, 1613, 1897,

			  225, 1526,  304, 1770,   35, 1772,  129,  208,  691,  210,
			 1238,  263,  156,  121,  316,  317,  318,  319,   17,  321,
			  322,  323,  217,  325,  326,  327,  500,  329,  330,  331,
			  296,  333,  334,   48,  883,  337,  510,  265,  641,  294,
			  342,  343,    3,  345,  346,  170,  348,  256,  350,  351,
			  465,  292,  170,  656,  356,  357,  804,   17,  334,  361,
			  362,  291,  842,  121,  366,  299,  710,  108,  712,  306,
			  714,  754,  302,  303, 1385,  300, 1120,  302,  303,  274,
			  332,  670, 1265,  256,  960,  256,  388,  389,  390,  391,
			  838,  967, 1018, 1001,  396, 1021, 2196,  571,  225, 1025,

			  221,  258,  404,  121, 1293,  202, 1553,  801,  300,  480,
			 1340,  413,  310, 2207,  256,  302,  303, 1585, 1018,   80,
			  286, 1021,  281,   64,  310, 1025,  310, 2247, 2152,  273,
			  719, 1925,  310, 1234,  310, 1544, 1545, 1546, 2135, 2136,
			  313,  256,  313, 1844, 1845, 2089,  299,  449,  243, 2198,
			  281, 1553,  741, 1936,  212,  310, 2064,  745,  874,  461,
			  942, 1010,  464,  753,  291,  467,  883,  469,  470,  471,
			 2072,  313,  855,  300,  476,  302,  303,  304,  480,  481,
			 1992,  116,  123,  485,    7,  487,  301,  314,  490,  220,
			  492,  493,  494,  461,  212,  490, 1963,  422,  313,  724,

			    4,  298,   17,  202,  506,    4,  753, 1822,  461,  434,
			 1718,  513,  514,  461,  461,  517,  492, 1981,   64,  570,
			  522,  236, 2322, 1632, 1633, 1634,  142,  168, 1136,   53,
			    3,  461,   87,  604,  121,  220,  461,  513, 1853,  200,
			 1001,  107, 1553,  751,  752,  206,  522,  836,  254,   39,
			  308, 1295,   39,  843,  141,  296,  194,   44,   73,    0,
			  520, 1397,  564,   87,  566,    7,  568,  569,  570,   26,
			  495,  566,  862,   26,  461,  500,  141,  123, 1428,  753,
			  510, 1873, 1874,  508,  586,  510,  755,  756,   25,  295,
			  308,  118,  568, 1010,  121,  422,  843,  599,  898,  124,

			   37, 1150,  604, 1557, 1153,   43,   93,  434,  180,  164,
			  270,  271,  272,  298,   65,  862,  139,  132,  689,   39,
			  255,  108,  168,  510, 1007,  115, 1009,   26,  115,  108,
			   96,  840,  634,  294,  461, 1076,  202,   87, 1079,  641,
			  164, 1385, 1122,  230, 1092, 1034,  898,    4,  121,   24,
			  266,  155,  293,  291,  656,  657,  155,  161,  660,  146,
			 2127,  663,  161, 1076,  666,  230, 1079,  669,  670,  843,
			 1118,   19,  244,  500,  112, 1136,    5,    6,  852,    8,
			 1024,   10,  851,  510,  163,  141, 1312,  689,  862,  992,
			   11,  693,   21,   22,  696,  115,  698,  139, 1499,    7,

			   48,  226,  704, 2197,  155,   41,   35,  210, 1916,  883,
			  214, 1455, 1312,  278,  164,  214,  718,  719,  493,  274,
			  235,   57,  159, 2190,  898,  278,  302,  303,  179, 1530,
			  129,   17, 1957, 1150, 1151,   39, 1153, 1026,  314,  741,
			   75,    9,  257,   38,  571,  816,    4,  293,   72,    7,
			  274,  291,  305,  755,  756,  757, 2048,  236,   82,  107,
			  247, 2169,  139, 1051,  279,   33,  186,  217,  167, 1057,
			  772,  258,  107,   68,  230,  150,  945,  256,  102,  306,
			  295, 1709,    7,  158,   79, 1065, 1076,   82,  418, 1079,
			   76,  257, 1081,  170,  569,  797,  299,  799,  173,    0,

			  121,  136,    5,    6, 2138,    8,    7,   10,  183, 1553,
			   11,  115, 1841,  121,  816,   80,    0,  296,   21,   22,
			  120, 2288,  100,  753,  274, 1307,  828,   11,  753,  165,
			   26,  297,   35,  835,  836,  837, 1010,   27,  213,  117,
			  911,  170,  137,   71,  170, 1135,  197,   75,  850,  851,
			  316,  852,  854,  319,  235,  321,  322,    0,  209,  325,
			  326,    4,   80,  329,  330,  331,  753,  869,   11,  852,
			 2037,  194,  874,    4, 2082, 2083,    7,  172,   43,  107,
			   58,   24,  186,  264,  350,  461, 1076,   39, 1135, 1079,
			  199,  357,  227,   24, 1283,  361,  121,  175,  193,  901,

			    0,    3, 1076,  905,  906, 1079,    9,    7,  136,  911,
			    0,   11, 1488,  843,  240,  118, 1328,    7,  843,  921,
			  121,   11,  852,  994,  219,  391,  753,   92,   26,  224,
			   33,  296,  862, 1236,  510,  200,  262,  862,  139,  941,
			  942,  206,  944,  945,  253, 1114, 1115,  312,   43,  951,
			 1239,  953,  954,  955, 1128, 1258,  843,  917,  129,  294,
			  205, 1135, 1628,  115,   94,  852,    4,  969,  291,    7,
			  113,  266,  278,  278,   26,  862, 1150, 1151,  121, 1153,
			 1816,  304,  200,  985,   43,  291,  291,  155,  206,   43,
			  992,  134,  994,  161,  176,  571,  274,   92, 1000,  227, yyDummy>>,
			1, 1000, 0)
		end

	yycheck_template_2 (an_array: ARRAY [INTEGER])
			-- Fill chunk #2 of template for `yycheck'.
		do
			yyarray_subcopy (an_array, <<
			    0, 1003,    4, 1005, 1006,    7, 1842,    7,   25, 1011,
			  140,   11,   12,  278,  289,  243,  843,  158, 1187,  150,
			   37,  121,   24, 1395, 1026,  852, 1028,  158, 1328,  294,
			  160,  121,  250,   92,  186,  862,   43,  312,   92, 1521,
			   64,  671,  173,  673,  674,  675,  214,  177,  678,  679,
			    7,  681,  183,  683,   11,  141,  883,  302,  303,    3,
			  278,    0,    9, 1065,    4, 1234,  294,    7,    7,  212,
			  213,  898,   11,  291, 1076,    3,  294, 1079,  107, 1081,
			  298,    9,  213, 1085,    4,   92,   33,    7,  231,   94,
			    3, 1380, 1836,    9, 1838, 1097,  169, 1099,    9,  123,

			 1901,   12, 1104,  306, 1106, 1107, 1108, 1109, 1076,  250,
			   24, 1079, 1114, 1115,   27,  144,  145,   33,  748,  249,
			  586,  121, 1047, 1076, 1126, 1127, 1079, 1120, 1076, 1076,
			  174, 1079, 1079,  599,   24,  140,   26,  278,  208, 1141,
			  210, 1143, 1144, 1145,  168, 1147, 1076,    4,  150, 1079,
			  291, 1076,   15,   16, 1079,  160,  158,  298,   15,   16,
			 1560,   26, 1562, 1165,  305, 1834,   26,  310,  291, 2058,
			 1172,  173,  177, 1585,  299,    4,  310,  753,  413, 1181,
			 1182,  183,  121, 1010,  310, 1187,   15,   16, 1396, 1076,
			  195,  657, 1079,  310,  660,    4, 1172, 1199,  223,    9,

			  139, 1203,   12, 1128,  227, 1135,   15,   16,    3, 1202,
			 1135,  213,  189, 1215,  191,  192,  476, 1219, 1178, 1221,
			   26,    4,    4, 1397,    7,    7, 1892,   18,   19,  464,
			  696, 1233, 1234,  108, 1236,    4, 1836, 1239, 1838, 1241,
			   27,   28, 1202,   17,  249, 1628,   15,   16, 1135, 1076,
			   27,   28, 1079, 1255,    5,    6, 1258,    8,    5,   10,
			 1262,   24, 1431,   10, 1266,    4,    4,  843,    7,    7,
			   21,   22, 2016, 1275, 2018,   22,  852, 2021,    7,    4,
			 1624, 1625,   80, 1285,   35, 1585,  862, 1289,   35, 1291,
			   15,   16, 1294, 1295, 1296,    4,   70,  163,    7, 1968,

			    0, 1128, 1295, 2104,   26, 1307,    4,  883, 1135, 1681,
			    4,   11,   12, 1503,   88,  210, 1318,    4, 1598, 2155,
			    7,   27,  898, 1150, 1151,  210, 1153,    9,    0,    7,
			 1499, 1106, 1107, 1108, 1109,    7,  110,   11,   12,   11,
			   12, 1343, 1344,    4,    4,    0,    7,   19,    4, 1351,
			    4,    7,    7,    7,    4,  129,   11,    7,    3, 1361,
			 1362, 1530,  828,    4,    9,   24,    0,   12, 1143, 1144,
			 1145,    4, 1147,    7,   19, 1816,   27,   11, 1380,   17,
			   35,  162,   15,   16,    0,  129,    0,    4,  854,   34,
			    7,    7, 1385,   76, 1876,   11, 1397,   11,   11,   12,

			 1402,   35,  176,  210,    4, 2206, 1233,    7, 2152, 1334,
			  180,  185, 1414, 1415, 1397,   26, 2016, 1047, 2018,   35,
			  171, 2021,   49,   17,  198,   39, 1428, 1405, 1406, 1431,
			   44,  666,    4,  174, 1010,    7,    0,   75,    9,  905,
			  906,   12,   80,    7,  218,   83,    0,   11,   86,  121,
			  210,    5, 1677,    7,    7, 1457,   10,   11,  693,  233,
			    9, 1463,    4,   12, 1672,    7,  121, 1397,   22,  107,
			   49,   35,  246,   87,    4,   49,  264,    7, 1480,   93,
			    4,   35,  235,    7,    3,  951, 1711,  121,  954,  955,
			  236, 1835, 1494, 1837,  108, 1839,  310, 1499,   67, 1501,

			 1076,  115,    9, 1079, 1792,  121,   26,    4,    9, 1892,
			 1397,   12,    4,    4,  152,  153,    7,    4, 1520, 1521,
			    7,  212,  757, 1525,   14, 1527, 1816,    3, 1530,    4,
			 1455,  162,  146, 1535, 1536, 1537,   26,   27,   28,   29,
			   30, 1543,   32,   26,  202, 1011, 1715,   42,   83,   49,
			  164, 1553, 2152,  238, 1556, 1557,   51,  121,    4, 1135,
			 1553,    7,  200,    4,  202,    9,    7,  121,   12,    4,
			 1397,  121,    7,    4, 1150, 1151,    7, 1153,    4, 1581,
			    4,    7,    4,    7,   58,    7, 1794, 2069,  256,  227,
			    3, 1593, 1594,    6,    4,    8, 1598,    7,  236, 1601,

			  835,   14,    4,  217,  194,    7,    4, 1609,    4,    7,
			    7,    7, 1614,   26,   27,   28,   29,   30,    4,   32,
			    4,    7,    9,    7,  310,   12, 1816,    6, 1076,    8,
			    3, 1079,    9,  247, 1636,   14,   48,  310,   49,  143,
			   49,   26, 1816,  139,  258,   49,  141,   26,   27,   28,
			   29,   30,    3,   32,    9,    3,  294,    4,  113,   92,
			  274,  921,  213,  158,    5,    6,    3,    8, 1842,   10,
			    3, 1673,  310,    9,  310,  134, 2020,    7, 1680,    3,
			   21,   22,    3,  178,    3,  202,   26,   26, 1690,  194,
			  257,  107,  187,  953,   35,  101, 1985, 1986,  101,  210,

			  246, 1703, 1704,   47, 1334,  265, 1708,   17,   49,  129,
			 1990,    4,  202, 1715,    4, 1181,  237,    3,   27, 1721,
			    5,    6,  277,    8,    7,   10,  235,   12,   26, 1919,
			  212,    3, 1901,  231,  969,  230,   21,   22,  673,  674,
			  675,    4, 2224,  678,  679,  202,  681,    3,  683, 1215,
			   35, 1753,   67, 1219,    3,  250,   38,    5,    6, 1928,
			    8,  255,   10,    4,   49, 1000,  261,  174, 1770, 1771,
			 1772,    4,  212,   21,   22, 1826,    3,    9,    3, 1781,
			  174,  174,  151,  278,    4,  280,   68,   35,    7,  202,
			 1792,    4,  121,    3,  289,    7,  291,   79,    7, 2102,

			   82, 1852,   50,  298,    5,  110,  266,  275, 1810, 1811,
			  305,  192,  266,  266,  155, 1816, 1792,  312, 1816, 1816,
			  210, 1397, 1816,  202, 1826,    3, 1828, 1829, 1294, 1295,
			 1065,  202,  266,  139,  298, 2043,    4,    5,    6, 1099,
			    8, 1842,   10, 1836, 1104, 1838, 1848,  121,  121,    7,
			 1852,  121,    9,   21,   22,  137, 1858,  134, 2147, 2148,
			   61,  109, 1864, 1865, 2154,   55, 2156,   35,  246,    3,
			  155, 1873, 1874,    0, 1876,   26,    5,    6,    5,    8,
			    7,   10,  213,   10,   11, 2075, 1816, 1889, 2078, 2371,
			  172, 1816,   21,   22, 2119,   22,  132, 2379,  176, 1901,

			   49, 2091,  194,   75,    4,    4,   35, 2154,   35, 2156,
			    3,  193, 1842,    3,   27,   27,   27,    4,  207,   84,
			  146, 1923, 1924, 2131,    9,   11, 1928,    5,    6, 1816,
			    8,  293,   10,    5,    6, 2104,    8,  219,   10,   17,
			   18,   19,  224,   21,   22,    4,    4,    9,    9,   21,
			   22,   27,  207,   42,    4, 1842, 1958,   35,    9, 1961,
			  147, 1963,   51,   35,    0,    4, 1968, 1969,  282,    5,
			 1972,    7,   67, 1975,   10,   11,  173,   49,  215,  183,
			 2154, 2155, 2156, 1985, 1986,    4,   22,   65, 1990, 1816,
			    4,   80,  310,  121,  121,   49,   49,    4,   87,   35, yyDummy>>,
			1, 1000, 1000)
		end

	yycheck_template_3 (an_array: ARRAY [INTEGER])
			-- Fill chunk #3 of template for `yycheck'.
		do
			yyarray_subcopy (an_array, <<
			    9,    4,    3,  310,  134,  206, 1241, 2009,  222,   11,
			    3,    3,  308,    3, 1480, 1842,    3,   13,   26,  220,
			  129,  222,  223, 2016,  225, 2018,  155,   24, 2021,  310,
			  134,    3,   27,   27,  174,  174,    3, 2206, 2040,    3,
			 1275,    3,    3,    0,   19,  212, 2048, 1972,    5,    7,
			    7,    7,  141,   10,   11, 2057,    9,  155,    3,   19,
			   26,    0, 2064,    4,    4,   22,    5, 2069,    7,  158,
			    4,   10,   11,    9,  134,  164,   26,  155,   35,  286,
			    9,    9, 2084,   22,  308,  121, 2088, 1553,  122,  178,
			    4, 1557,   27,  171,    7,    7,   35, 2099,  187,  300,

			 2102,  179, 2104,    4,    5,    6,    7,    8, 1343,   10,
			   36,  200,  201,  314,    7, 1581,  257,  206,  188,  184,
			   21,   22,  139,    3,    9, 2127, 2128,  184,  194,  139,
			  121, 2133,  121,  121,   35, 2343,  121,  121,  339,  340,
			  121,  230,  121,  121,    9, 2147, 2148,   11,   26,  246,
			   26,    3,   49,  194, 2155, 1415,    9, 2159,  210, 2152,
			    9,  250,    5,    6,  121,    8,   12,   10, 1428, 2094,
			 2172,  372,  261, 2175,    4,    4,   27,    4,   21,   22,
			    4,    4,  121,    4,    4,  274,    4,  308, 2190,  278,
			    9,  280,   35, 2195, 2196,   40,    9,  170,  118,  278,

			  289,    4,  291,  310, 2206,  294,  407,    3,    3,  298,
			    3,  149,  308,  414, 1680,   58,  305,  418,  419,  420,
			  421,  422, 2224,  312, 2154, 2155, 2156,  428,  138, 2154,
			    4, 2156,  433,  434,    4,    3,   16,    3,    9, 2241,
			 1816,    4,    4,    3,  174,   67,    4,    4,    4,  450,
			  451,  452,    4,  139,  455,  456,  457,  458,  459,  460,
			 2262, 2263,  463, 2265,    4,   46, 1842, 2154, 2155, 2156,
			    4,    4, 2099,    4,    4, 1535, 1536, 1537,    7,  122,
			    7,    9,    3,  484,   49,   66, 2288,  210,   49,  210,
			 1525,  207, 1527,   74,   49,   49, 2298,   12,  134,  500,

			    4,    5,    6,    7,    8,   27,   10,   27, 1543,   27,
			   27,   27,   12,   27,  515,  516,   27,   21,   22,   27,
			 2322,  102,  103,  108,  105,   27,   27, 2154, 2155, 2156,
			  170,   35,  286,  114,  286,  286,  113, 2339, 2340,    4,
			    5,    6,    7,    8,  286,   10, 2348, 2349,  202,    4,
			    5,    6,  133,    8,    3,   10,   21,   22, 1593, 1594,
			   27,  139,   26, 1598,  139,   19,   21,   22,    9, 2371,
			   35,  572,  139,  574,  575,  576,    4, 2379, 2380,  212,
			   35,    4,    9,    4,  201,  166,  139,    3,   35,  121,
			    4,    5,    6,    7,    8,    3,   10,    7,    3,   67,

			    7,  182,    4,   17,   18,   19,  607,   21,   22,    7,
			    4,    5,    6, 1673,    8,  196,   10,  227,    9,  267,
			   24,   35,   58,  204,  205,   49,  139,   21,   22, 2256,
			  139,    4,    5,    6,    3,    8,  134,   10,    4,    5,
			    6,   35,    8, 1703,   10,  292,  158,    9,   21,   22,
			    9,   65,   26,  234,    3,   21,   22,    3,    3,  210,
			    3, 1721,   35,    3,    3,    3,  225,    3,    3,   35,
			  671,    3,  673,  674,  675,    3,  677,  678,  679,  260,
			  681,  100,  683,  134,   71,  310,  134,    4,    5,    6,
			  691,    8,  216,   10,    9,   71,    7,  278,   19,  139,

			  210,   26,  283,  284,   21,   22, 1972,  286,    4,    9,
			 1770, 1771, 1772,   19, 2341,   12,   11,   11,   35,   11,
			   11,  302,  303,   12,   11,   11,    3,  728,  729,  278,
			    3,    9,   12,  134,    3,    7,  286,  738,  739,    7,
			   49,  155,   26,  744,  121,  746,    3,  748,    9,  750,
			  751,  752,  293,  754,    4,    5,    6,  171,    8,  298,
			   10,  287,    4,  147,    3,  179,    3,    3,    3,    3,
			    3,   21,   22,  774,    3, 1810, 1811,    3, 2154, 2155,
			 2156,  782,    3,    3,    9,   35,    4,    5,    6,  184,
			    8, 2057,   10, 1828, 1829,   26,  184,  158,  261,  305,

			  289,   24,   26,   21,   22,    4,    5,    6,  305,    8,
			  139,   10,  813, 1873, 1874,  127,  229,   35,  210,    7,
			  210,  210,   21,   22,    4,    5,    6,  210,    8,  181,
			   10,    4,    5,    6,  147,    8,   35,   10,  210,  210,
			  210,   21,   22,  229,    9,  210,  258,  267,   21,   22,
			    3,    9,    9,    3,  855,   35,   26,    3,  859,    3,
			    3,    3,   35,    3,   33,    3,  867,  298,    4,    5,
			    6,   26,    8,  288,   10,   26,  311,    3,    3,    3,
			    3,  241,   69,  261,  885,   21,   22,  241,  889,  890,
			  891,  892,  893,  894,  895,  896,  897,  898,    3,   35,

			  311, 1961,    0, 1963,    4,    5,    6,   26,    8,   11,
			   10,    4,    5,    6,  278,    8, 1496,   10, 1226, 1751,
			 1736,   21,   22, 2207,  449, 2210,    3, 1700,   21,   22,
			   82, 1679, 1494, 2380, 2345,   35, 2341, 2057, 2264,    4,
			    5,    6,   35,    8, 2155,   10, 1223,   24,    4,    5,
			    6, 1397,    8, 1816,   10, 1990,   21,   22,  959, 1816,
			  961,   38, 1497,   40, 2273,   21,   22, 1816, 1722, 1187,
			   35,  730,  424, 2107,   51, 1005,   53, 1005,   55,   35,
			 2040,  291,  898,   61,    5,   62, 1333, 1559, 2048, 1972,
			 1592,   68, 1362,   70, 1807, 1151, 1389, 1099, 1777,   76,

			 1112, 1002,   79, 1648, 1005,   82, 1007, 2013, 1009,  607,
			   87,   88,  606,    2,    2, 1016,    2,    2,    2, 1214,
			    2,    2,    5,    6,  101,    8, 1027,   10, 2088, 1003,
			 1031, 1032, 1524,  110, 1035,   18,   19, 1767,   21,   22,
			    5,    6,    7,    8,   63,   10, 1047,  389, 2190, 1050,
			  495, 1052,   35, 1708, 1959,  132,   21,   22,  135, 1655,
			  137,  365, 2249, 2245, 1865,  142,  143, 2127, 2171, 2300,
			   35, 2211,    5,    6,  112,    8, 1428,   10,  953,  156,
			  112,    2,    2,   16,  161,  162,    2,  164,   21,   22,
			    2,    2,    2,    2,    0,  172,   -1, 1098,  175,   -1,

			   -1,  178,   35,  180,   -1,   -1,   -1,  184,  185,   -1,
			   -1,   -1,   -1,   -1, 2380,   -1,  193,  194,   -1,   -1,
			   -1,  198,   -1,   -1,   -1,   -1,   -1, 1128,   -1,  206,
			 2190,   -1,    5,    6,    7,    8,   -1,   10,   -1,   -1,
			  217,  218,  219,   -1,   -1,  222,   -1,  224,   21,   22,
			   -1,   -1,    5,    6,   -1,    8,  233,   10,   -1,   -1,
			   13,  238,   35,   -1,   -1,   -1,   -1,  244,   21,   22,
			   -1,  248, 1173,  250,   -1,   -1,   -1,  254,   -1,  256,
			  257,   -1,   35,   -1,   -1,   -1,   -1,  264,   -1,  266,
			    5,    6,   -1,    8,   -1,   10,  273,   -1,  275,   -1, yyDummy>>,
			1, 1000, 2000)
		end

	yycheck_template_4 (an_array: ARRAY [INTEGER])
			-- Fill chunk #4 of template for `yycheck'.
		do
			yyarray_subcopy (an_array, <<
			  277,   16,  279,  280,   -1, 2265,   21,   22,   -1,   -1,
			   -1,  288,   -1,   -1,  291,   -1,    5,    6,  295,    8,
			   35,   10,   -1,  300,   13,   -1,   -1,  304, 2288,   -1,
			   -1,   -1,   21,   22,   -1,   -1,  313,    5,    6,   -1,
			    8,   -1,   10, 1244,   -1,    3,   35, 1248,   -1,   -1,
			   -1, 1252,   -1,   21,   22,   -1,   -1,   -1,   -1,    5,
			    6,   -1,    8,   31,   10,   -1,   24,   35,   -1,   -1,
			   -1,   -1,   -1, 1274,   -1,   21,   22,   -1,   -1,   -1,
			   38,   39,   40,   -1,   -1,   31,   44,   -1,   -1,   35,
			   -1,   -1,   -1,   51,   52,   53,   -1,   55,   56,   -1,

			   58,   59,   60,   61,   -1,   -1,   -1,   -1,   -1,   -1,
			   68,   69,   70,   -1,   -1,   -1,   -1,   75,   76,   77,
			   78,   79,   -1, 1324,   82,   -1,   -1,   85,   -1,   87,
			   88, 1332,   -1, 1334,   -1,   93,   -1,   -1,   -1,   97,
			   98,   99,   -1,  101,   -1,   -1,  104,   -1,  106,   -1,
			  108,   -1,  110,  111,   -1,   -1,   -1,  115,   -1,    5,
			    6,    7,    8,   -1,   10,   -1,   -1,  125, 1369,   -1,
			 1371,   -1,  130,  131,  132,   21,   22,  135,   -1,  137,
			   -1,   -1,   -1,   -1,  142,  143,   -1,   -1,  146,   35,
			  148,   -1,   -1,   -1,   -1, 1396,  154,   -1,  156,   -1,

			   -1,   -1,   -1,  161,  162,  163,  164,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,  172,   -1, 1417,  175,   -1,  177,
			  178,   -1,  180,  181,   -1,   -1,  184,  185,    5,    6,
			   -1,    8, 1433,   10,   11,  193,  194,   -1,    5,    6,
			  198,    8,   -1,   10,   21,   22,   13,   -1,  206,   -1,
			   -1,   -1,   -1,  211,   21,   22,   -1,   -1,   35,  217,
			  218,  219,   -1,   -1,  222,   -1,  224,   -1,   35,   -1,
			  228,   -1,   -1,   -1,  232,  233,   -1,   -1,   -1,   -1,
			  238,   -1,   -1,   -1,   -1,   -1,  244,   -1,   -1,  247,
			  248,  249,  250,  251,   -1,   -1,  254,   -1,  256,  257,

			  258,  259,   -1,   -1,   -1,   -1,  264,   -1,  266,   -1,
			   -1,   -1,   -1,   -1,   -1,  273,   -1,  275,  276,  277,
			   -1,  279,  280,   -1,   -1,   -1,   -1,  285,   -1,   -1,
			  288,   -1,  290,  291,   -1,   -1,   -1,  295,  296,  297,
			   -1,   -1,  300,   -1,    5,    6,  304,    8, 1549,   10,
			   -1,  309,   13,   -1,   -1,  313,   -1,   -1,   -1,   -1,
			   21,   22,   -1,    5,    6,    7,    8,   -1,   10,   -1,
			 1571,   -1,   -1,   -1,   35,   -1,   -1,   -1, 1579,   21,
			   22,    5,    6,   -1,    8,   -1,   10,    5,    6, 1590,
			    8,   -1,   10,   35,   -1,   -1,   -1,   21,   22,   -1,

			   -1,   -1,   -1,   21,   22,   -1,   -1,   -1,   -1,   -1,
			   -1,   35,   -1,   -1,   -1,   -1,   -1,   35,   -1,   -1,
			   -1, 1622,   -1,   -1,   -1,   -1, 1627, 1628,   -1,   -1,
			   -1,   -1,   -1,   -1, 1635,   -1,   -1,   -1,   -1,    0,
			   -1,   -1, 1643,    4,    5,    6,    7,    8,    9,   10,
			   11,   12,   13,   -1,   15,   16,   17,   18,   19,   -1,
			   21,   22,   -1,   24,   -1,   -1,   -1,   -1,   -1, 1670,
			   31, 1672,   33,   -1,   35,   -1,   -1,   38,   39,   40,
			   -1,   -1,   -1,   44,   -1,   -1,   -1,   -1,   49,   50,
			   51,   -1,   53,   -1,   55,   -1,   -1,   58,   -1,   -1,

			   -1,   -1,   -1,   64,   65,   -1,   -1,   68,   -1,   70,
			   -1,   -1,   -1,   -1,   -1,   76,   -1,   -1,   79,   -1,
			   -1,   82,   -1,   -1,   -1, 1726,   87,   88,   -1,   -1,
			   -1,   -1,   93,   94,   -1,   -1,   -1,   -1,   -1,   -1,
			  101,   -1,   -1,   -1,   -1,   -1,   -1,  108,  109,  110,
			   -1,   -1,  113,   -1,  115,   -1,   -1,  118,   -1,   -1,
			  121,  122,  123,   -1,   -1,   -1, 1767,   -1,   -1,   -1,
			   -1,  132,   -1,  134,  135,   -1,  137,   -1,  139,  140,
			   -1,  142,  143,   -1,   -1,  146,  147,   -1,   -1,   -1,
			  151,   -1, 1793, 1794,  155,  156,   -1,   -1,   -1,  160,

			  161,  162,   -1,  164,   -1,   -1,   -1,  168,   -1,  170,
			  171,  172,  173,   -1,  175,   -1,  177,  178,  179,  180,
			   -1,   -1,   -1,  184,  185,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,  193,  194,  195,   -1,   -1,  198,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,  206,   -1,   -1,   -1,   -1,
			   -1,  212,  213,   -1,   -1,   -1,  217,  218,  219,  220,
			   -1,  222,   -1,  224,   -1,   -1,   -1,   -1,   -1,   -1,
			  231,   -1,  233,   -1,   -1,   -1,   -1,  238,   -1,   -1,
			   -1,   -1,   -1,  244,   -1,   -1,  247,  248,  249,  250,
			   -1, 1892,   -1,  254,   -1,   -1,  257,  258,   -1,   -1,

			   -1,   -1,   -1,  264,   -1,  266,   -1,   -1,   -1,   -1,
			   -1,   -1,  273,   -1,  275,   -1,  277,   -1,  279,  280,
			  281,   -1,   -1,   -1,   -1,   -1,   -1,  288,   -1,   -1,
			  291,   -1,  293, 1934,  295,   -1,   -1,   -1,   -1,  300,
			   -1,   -1,   -1,  304,   -1,  306,   -1,  308,   -1,  310,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			 1991,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,

			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1, 2013,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1, 2043,   -1,    0,   -1,   -1,   -1,    4,    5,
			    6,    7,    8,    9,   10,   11,   12,   13,   -1,   15,
			   16,   17,   18,   19,   -1,   21,   22,   -1,   24,   -1,
			   -1,   -1,   -1,   -1,   -1,   31,   -1,   33,   -1,   35,
			   -1,   -1,   38,   39,   40,   -1,   -1,   -1,   44,   -1,
			   -1,   -1,   -1,   49,   50,   51,   -1,   53,   -1,   55,

			   -1,   -1,   58,   -1,   -1,   -1,   -1,   -1,   64,   65,
			   -1,   -1,   68,   -1,   70,   -1,   -1,   -1,   -1,   -1,
			   76,   -1,   -1,   79,   -1,   -1,   82,   -1,   -1,   -1,
			 2131,   87,   88,   -1, 2135, 2136,   -1,   93,   94,   -1,
			   -1,   -1,   -1,   -1,   -1,  101,   -1,   -1,   -1,   -1,
			   -1,   -1,  108,  109,  110,   -1,   -1,  113,   -1,  115,
			   -1,   -1,  118,   -1,   -1,  121,  122,  123,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,  132,   -1,  134,  135,
			   -1,  137,   -1,  139,  140,   -1,  142,  143,   -1,   -1,
			  146,  147,   -1,   -1,   -1,  151,   -1,   -1,   -1,  155, yyDummy>>,
			1, 1000, 3000)
		end

	yycheck_template_5 (an_array: ARRAY [INTEGER])
			-- Fill chunk #5 of template for `yycheck'.
		do
			yyarray_subcopy (an_array, <<
			  156, 2202,   -1,   -1,  160,  161,  162,   -1,  164,   -1,
			   -1,   -1,  168,   -1,  170,  171,  172,  173,   -1,  175,
			   -1,  177,  178,  179,  180,   -1,   -1,   -1,  184,  185,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,  193,  194,  195,
			   -1,   -1,  198,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			  206,   -1,   -1,   -1,   -1, 2256,  212,  213,   -1,   -1,
			   -1,  217,  218,  219,  220,   -1,  222,   -1,  224,   -1,
			   -1,   -1,   -1,   -1, 2275,  231,   -1,  233,   -1,   -1,
			   -1,   -1,  238,   -1,   -1,   -1,   -1,   -1,  244,   -1,
			   -1,  247,  248,  249,  250,   -1,   -1,   -1,  254,   -1,

			   -1,  257,  258,   -1,   -1,   -1,   -1,   -1,  264,   -1,
			  266,   -1,   -1,   -1,   -1,   -1,   -1,  273,   -1,  275,
			   -1,  277,   -1,  279,  280,  281,   -1,   -1,   -1,   -1,
			   -1,   -1,  288,   -1,   -1,  291,   -1,  293,   -1,  295,
			 2341,   -1, 2343,   -1,  300,   -1,   -1,   -1,  304,   -1,
			  306,   -1,  308,    0,  310,   -1,    3,    4,    5,    6,
			    7,    8,    9,   10,   11,   12,   13,   -1,   15,   16,
			   17,   18,   19,   -1,   21,   22,   -1,   24,   -1,   -1,
			   -1,   -1,   -1,   -1,   31,   -1,   -1,   -1,   35,   -1,
			   -1,   38,   39,   40,   -1,   -1,   -1,   44,   -1,   -1,

			   -1,   -1,   49,   50,   51,   -1,   53,   -1,   55,   -1,
			   -1,   58,   -1,   -1,   -1,   -1,   -1,   64,   65,   -1,
			   -1,   68,   -1,   70,   -1,   -1,   -1,   -1,   -1,   76,
			   -1,   -1,   79,   -1,   -1,   82,   -1,   -1,   -1,   -1,
			   87,   88,   -1,   -1,   -1,   -1,   93,   94,   -1,   -1,
			   -1,   -1,   -1,   -1,  101,   -1,   -1,   -1,   -1,   -1,
			   -1,  108,  109,  110,   -1,   -1,  113,   -1,  115,   -1,
			   -1,  118,   -1,   -1,  121,  122,  123,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,  132,   -1,  134,  135,   -1,
			  137,   -1,  139,  140,   -1,  142,  143,   -1,   -1,  146,

			  147,   -1,   -1,   -1,  151,   -1,   -1,   -1,  155,  156,
			   -1,   -1,   -1,  160,  161,  162,   -1,  164,   -1,   -1,
			   -1,  168,   -1,  170,  171,  172,  173,   -1,  175,   -1,
			  177,  178,  179,  180,   -1,   -1,   -1,  184,  185,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,  193,  194,  195,   -1,
			   -1,  198,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  206,
			   -1,   -1,   -1,   -1,   -1,  212,  213,   -1,   -1,   -1,
			  217,  218,  219,  220,   -1,  222,   -1,  224,   -1,   -1,
			   -1,   -1,   -1,   -1,  231,   -1,  233,   -1,   -1,   -1,
			   -1,  238,   -1,   -1,   -1,   -1,   -1,  244,   -1,   -1,

			  247,  248,  249,  250,   -1,   -1,   -1,  254,   -1,   -1,
			  257,  258,   -1,   -1,   -1,   -1,   -1,  264,   -1,  266,
			   -1,   -1,   -1,   -1,   -1,   -1,  273,   -1,  275,   -1,
			  277,   -1,  279,  280,  281,   -1,   -1,   -1,   -1,   -1,
			   -1,  288,   -1,   -1,  291,   -1,  293,   -1,  295,   -1,
			   -1,   -1,   -1,  300,   -1,   -1,   -1,  304,   -1,  306,
			    0,  308,   -1,  310,    4,    5,    6,    7,    8,    9,
			   10,   11,   12,   13,   -1,   15,   16,   17,   18,   19,
			   -1,   21,   22,   -1,   24,   -1,   -1,   -1,   -1,   -1,
			   -1,   31,   -1,   -1,   -1,   35,   -1,   -1,   38,   39,

			   40,   -1,   -1,   -1,   44,   -1,   -1,   -1,   -1,   49,
			   50,   51,   -1,   53,   -1,   55,   -1,   -1,   58,   -1,
			   -1,   -1,   -1,   -1,   64,   65,   -1,   -1,   68,   -1,
			   70,   -1,   -1,   -1,   -1,   -1,   76,   -1,   -1,   79,
			   -1,   -1,   82,   -1,   -1,   -1,   -1,   87,   88,   -1,
			   -1,   -1,   -1,   93,   94,   -1,   -1,   -1,   -1,   -1,
			   -1,  101,   -1,   -1,   -1,   -1,   -1,   -1,  108,  109,
			  110,   -1,   -1,  113,   -1,  115,   -1,   -1,  118,   -1,
			   -1,  121,  122,  123,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,  132,   -1,  134,  135,   -1,  137,   -1,  139,

			  140,   -1,  142,  143,   -1,   -1,  146,  147,   -1,   -1,
			   -1,  151,   -1,   -1,   -1,  155,  156,   -1,   -1,   -1,
			  160,  161,  162,   -1,  164,   -1,   -1,   -1,  168,   -1,
			  170,  171,  172,  173,   -1,  175,   -1,  177,  178,  179,
			  180,   -1,   -1,   -1,  184,  185,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,  193,  194,  195,   -1,   -1,  198,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,  206,   -1,   -1,   -1,
			   -1,   -1,  212,  213,   -1,   -1,   -1,  217,  218,  219,
			  220,   -1,  222,   -1,  224,   -1,   -1,   -1,   -1,   -1,
			   -1,  231,   -1,  233,   -1,   -1,   -1,   -1,  238,   -1,

			   -1,   -1,   -1,   -1,  244,   -1,   -1,  247,  248,  249,
			  250,   -1,   -1,   -1,  254,   -1,   -1,  257,  258,   -1,
			   -1,   -1,   -1,   -1,  264,   -1,  266,   -1,   -1,   -1,
			   -1,   -1,   -1,  273,   -1,  275,   -1,  277,   -1,  279,
			  280,  281,   -1,   -1,   -1,   -1,   -1,   -1,  288,   -1,
			   -1,  291,   -1,  293,   -1,  295,   -1,   -1,   -1,   -1,
			  300,   -1,   -1,   -1,  304,   -1,  306,    0,  308,   -1,
			  310,    4,    5,    6,    7,    8,    9,   10,   11,   12,
			   13,   -1,   15,   16,   17,   18,   19,   -1,   21,   22,
			   -1,   24,   -1,   -1,   -1,   -1,   -1,   -1,   31,   -1,

			   -1,   -1,   35,   -1,   -1,   38,   39,   40,   -1,   -1,
			   -1,   44,   -1,   -1,   -1,   -1,   49,   50,   51,   -1,
			   53,   -1,   55,   -1,   -1,   58,   -1,   -1,   -1,   -1,
			   -1,   64,   65,   -1,   -1,   68,   -1,   70,   -1,   -1,
			   -1,   -1,   -1,   76,   -1,   -1,   79,   -1,   -1,   82,
			   -1,   -1,   -1,   -1,   87,   88,   -1,   -1,   -1,   -1,
			   93,   94,   -1,   -1,   -1,   -1,   -1,   -1,  101,   -1,
			   -1,   -1,   -1,   -1,   -1,  108,  109,  110,   -1,   -1,
			  113,   -1,  115,   -1,   -1,  118,   -1,   -1,  121,  122,
			  123,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  132,

			   -1,  134,  135,   -1,  137,   -1,  139,  140,   -1,  142,
			  143,   -1,   -1,  146,  147,   -1,   -1,   -1,  151,   -1,
			   -1,   -1,  155,  156,   -1,   -1,   -1,  160,  161,  162,
			   -1,  164,   -1,   -1,   -1,  168,   -1,  170,  171,  172,
			  173,   -1,  175,   -1,  177,  178,  179,  180,   -1,   -1,
			   -1,  184,  185,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			  193,  194,  195,   -1,   -1,  198,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,  206,   -1,   -1,   -1,   -1,   -1,  212,
			  213,   -1,   -1,   -1,  217,  218,  219,  220,   -1,  222,
			   -1,  224,   -1,   -1,   -1,   -1,   -1,   -1,  231,   -1, yyDummy>>,
			1, 1000, 4000)
		end

	yycheck_template_6 (an_array: ARRAY [INTEGER])
			-- Fill chunk #6 of template for `yycheck'.
		do
			yyarray_subcopy (an_array, <<
			  233,   -1,   -1,   -1,   -1,  238,   -1,   -1,   -1,   -1,
			   -1,  244,   -1,   -1,  247,  248,  249,  250,   -1,   -1,
			   -1,  254,   -1,   -1,  257,  258,   -1,   -1,   -1,   -1,
			   -1,  264,   -1,  266,   -1,   -1,   -1,   -1,   -1,   -1,
			  273,   -1,  275,   -1,  277,   -1,  279,  280,  281,   -1,
			   -1,   -1,   -1,   -1,   -1,  288,   -1,   -1,  291,   -1,
			  293,   -1,  295,   -1,   -1,   -1,   -1,  300,   -1,   -1,
			   -1,  304,   -1,  306,    0,  308,   -1,  310,    4,    5,
			    6,    7,    8,   -1,   10,   11,   12,   13,   -1,   15,
			   16,   17,   18,   19,   -1,   21,   22,   -1,   24,   -1,

			   -1,   -1,   -1,   -1,   -1,   31,   -1,   -1,   -1,   35,
			   -1,   -1,   38,   39,   40,   -1,   -1,   -1,   44,   -1,
			   -1,   -1,   -1,   49,   50,   51,   -1,   53,   -1,   55,
			   -1,   -1,   58,   -1,   -1,   -1,   -1,   -1,   64,   65,
			   -1,   -1,   68,   -1,   70,   -1,   -1,   -1,   -1,   -1,
			   76,   -1,   -1,   79,   -1,   -1,   82,   -1,   -1,   -1,
			   -1,   87,   88,   -1,   -1,   -1,   -1,   93,   94,   -1,
			   -1,   -1,   -1,   -1,   -1,  101,   -1,   -1,   -1,   -1,
			   -1,   -1,  108,  109,  110,   -1,   -1,  113,   -1,  115,
			   -1,   -1,  118,   -1,   -1,  121,  122,  123,   -1,   -1,

			   -1,   -1,   -1,   -1,   -1,   -1,  132,   -1,  134,  135,
			   -1,  137,   -1,  139,  140,   -1,  142,  143,   -1,   -1,
			  146,  147,   -1,   -1,   -1,  151,   -1,   -1,   -1,  155,
			  156,   -1,   -1,   -1,  160,  161,  162,   -1,  164,   -1,
			   -1,   -1,  168,   -1,  170,  171,  172,  173,   -1,  175,
			   -1,  177,  178,  179,  180,   -1,   -1,   -1,  184,  185,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,  193,  194,  195,
			   -1,   -1,  198,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			  206,   -1,   -1,   -1,   -1,   -1,  212,  213,   -1,   -1,
			   -1,  217,  218,  219,  220,   -1,  222,   -1,  224,   -1,

			   -1,   -1,   -1,   -1,   -1,  231,   -1,  233,   -1,   -1,
			   -1,   -1,  238,   -1,   -1,   -1,   -1,   -1,  244,   -1,
			   -1,  247,  248,  249,  250,   -1,   -1,   -1,  254,   -1,
			   -1,  257,  258,   -1,   -1,   -1,   -1,   -1,  264,   -1,
			  266,   -1,   -1,   -1,   -1,   -1,   -1,  273,   -1,  275,
			   -1,  277,   -1,  279,  280,  281,   -1,   -1,   -1,   -1,
			   -1,   -1,  288,   -1,   -1,  291,   -1,  293,   -1,  295,
			   -1,   -1,   -1,   -1,  300,   -1,   -1,   -1,  304,   -1,
			  306,    0,  308,   -1,  310,    4,    5,    6,    7,    8,
			   -1,   10,   11,   12,   13,   -1,   15,   16,   17,   18,

			   19,   -1,   21,   22,   -1,   24,   -1,   -1,   -1,   -1,
			   -1,   -1,   31,   -1,   -1,   -1,   35,   -1,   -1,   38,
			   39,   40,   -1,   -1,   -1,   44,   -1,   -1,   -1,   -1,
			   49,   50,   51,   -1,   53,   -1,   55,   -1,   -1,   58,
			   -1,   -1,   -1,   -1,   -1,   64,   65,   -1,   -1,   68,
			   -1,   70,   -1,   -1,   -1,   -1,   -1,   76,   -1,   -1,
			   79,   -1,   -1,   82,   -1,   -1,   -1,   -1,   87,   88,
			   -1,   -1,   -1,   -1,   93,   94,   -1,   -1,   -1,   -1,
			   -1,   -1,  101,   -1,   -1,   -1,   -1,   -1,   -1,  108,
			  109,  110,   -1,   -1,  113,   -1,  115,   -1,   -1,  118,

			   -1,   -1,  121,  122,  123,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,  132,   -1,  134,  135,   -1,  137,   -1,
			  139,  140,   -1,  142,  143,   -1,   -1,  146,  147,   -1,
			   -1,   -1,  151,   -1,   -1,   -1,  155,  156,   -1,   -1,
			   -1,  160,  161,  162,   -1,  164,   -1,   -1,   -1,  168,
			   -1,  170,  171,  172,  173,   -1,  175,   -1,  177,  178,
			  179,  180,   -1,   -1,   -1,  184,  185,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,  193,  194,  195,   -1,   -1,  198,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,  206,   -1,   -1,
			   -1,   -1,   -1,  212,  213,   -1,   -1,   -1,  217,  218,

			  219,  220,   -1,  222,   -1,  224,   -1,   -1,   -1,   -1,
			   -1,   -1,  231,   -1,  233,   -1,   -1,   -1,   -1,  238,
			   -1,   -1,   -1,   -1,   -1,  244,   -1,   -1,  247,  248,
			  249,  250,   -1,   -1,   -1,  254,   -1,   -1,  257,  258,
			   -1,   -1,   -1,   -1,   -1,  264,   -1,  266,   -1,   -1,
			   -1,   -1,   -1,   -1,  273,   -1,  275,   -1,  277,   -1,
			  279,  280,  281,   -1,   -1,   -1,   -1,   -1,   -1,  288,
			   -1,   -1,  291,   -1,  293,   -1,  295,   -1,   -1,   -1,
			   -1,  300,   -1,   -1,   -1,  304,   -1,  306,    0,  308,
			   -1,  310,    4,    5,    6,    7,    8,   -1,   10,   11,

			   12,   13,   -1,   15,   16,   17,   18,   19,   -1,   21,
			   22,   -1,   24,   -1,   -1,   -1,   -1,   -1,   -1,   31,
			   -1,   -1,   -1,   35,   -1,   -1,   38,   39,   40,   -1,
			   -1,   -1,   44,   -1,   -1,   -1,   -1,   49,   50,   51,
			   -1,   53,   -1,   55,   -1,   -1,   58,   -1,   -1,   -1,
			   -1,   -1,   64,   65,   -1,   -1,   68,   -1,   70,   -1,
			   -1,   -1,   -1,   -1,   76,   -1,   -1,   79,   -1,   -1,
			   82,   -1,   -1,   -1,   -1,   87,   88,   -1,   -1,   -1,
			   -1,   93,   94,   -1,   -1,   -1,   -1,   -1,   -1,  101,
			   -1,   -1,   -1,   -1,   -1,   -1,  108,  109,  110,   -1,

			   -1,  113,   -1,  115,   -1,   -1,  118,   -1,   -1,  121,
			  122,  123,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			  132,   -1,  134,  135,   -1,  137,   -1,  139,  140,   -1,
			  142,  143,   -1,   -1,  146,  147,   -1,   -1,   -1,  151,
			   -1,   -1,   -1,  155,  156,   -1,   -1,   -1,  160,  161,
			  162,   -1,  164,   -1,   -1,   -1,  168,   -1,  170,  171,
			  172,  173,   -1,  175,   -1,  177,  178,  179,  180,   -1,
			   -1,   -1,  184,  185,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,  193,  194,  195,   -1,   -1,  198,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,  206,   -1,   -1,   -1,   -1,   -1,

			  212,  213,   -1,   -1,   -1,  217,  218,  219,  220,   -1,
			  222,   -1,  224,   -1,   -1,   -1,   -1,   -1,   -1,  231,
			   -1,  233,   -1,   -1,   -1,   -1,  238,   -1,   -1,   -1,
			   -1,   -1,  244,   -1,   -1,  247,  248,  249,  250,   -1,
			   -1,   -1,  254,   -1,   -1,  257,  258,   -1,   -1,   -1,
			   -1,   -1,  264,   -1,  266,   -1,   -1,   -1,   -1,   -1,
			   -1,  273,   -1,  275,   -1,  277,   -1,  279,  280,  281,
			   -1,   -1,   -1,   -1,   -1,   -1,  288,   -1,   -1,  291,
			   -1,  293,   -1,  295,   -1,   -1,   -1,   -1,  300,   -1,
			   -1,   -1,  304,   -1,  306,    0,  308,   -1,  310,    4, yyDummy>>,
			1, 1000, 5000)
		end

	yycheck_template_7 (an_array: ARRAY [INTEGER])
			-- Fill chunk #7 of template for `yycheck'.
		do
			yyarray_subcopy (an_array, <<
			    5,    6,    7,    8,   -1,   10,   11,   12,   13,   -1,
			   15,   16,   17,   18,   19,   -1,   21,   22,   -1,   24,
			   -1,   -1,   -1,   -1,   -1,   -1,   31,   -1,   -1,   -1,
			   35,   -1,   -1,   38,   39,   40,   -1,   -1,   -1,   44,
			   -1,   -1,   -1,   -1,   49,   50,   51,   -1,   53,   -1,
			   55,   -1,   -1,   58,   -1,   -1,   -1,   -1,   -1,   64,
			   65,   -1,   -1,   68,   -1,   70,   -1,   -1,   -1,   -1,
			   -1,   76,   -1,   -1,   79,   -1,   -1,   82,   -1,   -1,
			   -1,   -1,   87,   88,   -1,   -1,   -1,   -1,   93,   94,
			   -1,   -1,   -1,   -1,   -1,   -1,  101,   -1,   -1,   -1,

			   -1,   -1,   -1,  108,  109,  110,   -1,   -1,  113,   -1,
			  115,   -1,   -1,  118,   -1,   -1,  121,  122,  123,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,  132,   -1,  134,
			  135,   -1,  137,   -1,  139,  140,   -1,  142,  143,   -1,
			   -1,  146,  147,   -1,   -1,   -1,  151,   -1,   -1,   -1,
			  155,  156,   -1,   -1,   -1,  160,  161,  162,   -1,  164,
			   -1,   -1,   -1,  168,   -1,  170,  171,  172,  173,   -1,
			  175,   -1,  177,  178,  179,  180,   -1,   -1,   -1,  184,
			  185,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  193,  194,
			  195,   -1,   -1,  198,   -1,   -1,   -1,   -1,   -1,   -1,

			   -1,  206,   -1,   -1,   -1,   -1,   -1,  212,  213,   -1,
			   -1,   -1,  217,  218,  219,  220,   -1,  222,   -1,  224,
			   -1,   -1,   -1,   -1,   -1,   -1,  231,   -1,  233,   -1,
			   -1,   -1,   -1,  238,   -1,   -1,   -1,   -1,   -1,  244,
			   -1,   -1,  247,  248,  249,  250,   -1,   -1,   -1,  254,
			   -1,   -1,  257,  258,   -1,   -1,   -1,   -1,   -1,  264,
			   -1,  266,   -1,   -1,   -1,   -1,   -1,   -1,  273,   -1,
			  275,   -1,  277,   -1,  279,  280,  281,   -1,   -1,   -1,
			   -1,   -1,   -1,  288,   -1,   -1,  291,   -1,  293,   -1,
			  295,   -1,   -1,   -1,   -1,  300,   -1,   -1,   -1,  304,

			   -1,  306,    0,  308,   -1,  310,    4,    5,    6,    7,
			    8,   -1,   10,   11,   12,   13,   -1,   15,   16,   17,
			   18,   19,   -1,   21,   22,   -1,   24,   -1,   -1,   -1,
			   -1,   -1,   -1,   31,   -1,   -1,   -1,   35,   -1,   -1,
			   38,   39,   40,   -1,   -1,   -1,   44,   -1,   -1,   -1,
			   -1,   49,   50,   51,   -1,   53,   -1,   55,   -1,   -1,
			   58,   -1,   -1,   -1,   -1,   -1,   64,   65,   -1,   -1,
			   68,   -1,   70,   -1,   -1,   -1,   -1,   -1,   76,   -1,
			   -1,   79,   -1,   -1,   82,   -1,   -1,   -1,   -1,   87,
			   88,   -1,   -1,   -1,   -1,   93,   94,   -1,   -1,   -1,

			   -1,   -1,   -1,  101,   -1,   -1,   -1,   -1,   -1,   -1,
			  108,  109,  110,   -1,   -1,  113,   -1,  115,   -1,   -1,
			  118,   -1,   -1,  121,  122,  123,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,  132,   -1,  134,  135,   -1,  137,
			   -1,  139,  140,   -1,  142,  143,   -1,   -1,  146,  147,
			   -1,   -1,   -1,  151,   -1,   -1,   -1,  155,  156,   -1,
			   -1,   -1,  160,  161,  162,   -1,  164,   -1,   -1,   -1,
			  168,   -1,  170,  171,  172,  173,   -1,  175,   -1,  177,
			  178,  179,  180,   -1,   -1,   -1,  184,  185,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,  193,  194,  195,   -1,   -1,

			  198,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  206,   -1,
			   -1,   -1,   -1,   -1,  212,  213,   -1,   -1,   -1,  217,
			  218,  219,  220,   -1,  222,   -1,  224,   -1,   -1,   -1,
			   -1,   -1,   -1,  231,   -1,  233,   -1,   -1,   -1,   -1,
			  238,   -1,   -1,   -1,   -1,   -1,  244,   -1,   -1,  247,
			  248,  249,  250,   -1,   -1,   -1,  254,   -1,   -1,  257,
			  258,   -1,   -1,   -1,   -1,   -1,  264,   -1,  266,   -1,
			   -1,   -1,   -1,   -1,   -1,  273,   -1,  275,   -1,  277,
			   -1,  279,  280,  281,   -1,   -1,   -1,   -1,   -1,   -1,
			  288,   -1,   -1,  291,   -1,  293,   -1,  295,   -1,   -1,

			   -1,   -1,  300,   -1,   -1,   -1,  304,   -1,  306,    0,
			  308,   -1,  310,    4,    5,    6,    7,    8,   -1,   10,
			   11,   12,   13,   -1,   15,   16,   17,   18,   19,   -1,
			   21,   22,   -1,   24,   -1,   -1,   -1,   -1,   -1,   -1,
			   31,   -1,   -1,   -1,   35,   -1,   -1,   38,   39,   40,
			   -1,   -1,   -1,   44,   -1,   -1,   -1,   -1,   49,   50,
			   51,   -1,   53,   -1,   55,   -1,   -1,   58,   -1,   -1,
			   -1,   -1,   -1,   64,   65,   -1,   -1,   68,   -1,   70,
			   -1,   -1,   -1,   -1,   -1,   76,   -1,   -1,   79,   -1,
			   -1,   82,   -1,   -1,   -1,   -1,   87,   88,   -1,   -1,

			   -1,   -1,   93,   94,   -1,   -1,   -1,   -1,   -1,   -1,
			  101,   -1,   -1,   -1,   -1,   -1,   -1,  108,  109,  110,
			   -1,   -1,  113,   -1,  115,   -1,   -1,  118,   -1,   -1,
			  121,  122,  123,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,  132,   -1,  134,  135,   -1,  137,   -1,  139,  140,
			   -1,  142,  143,   -1,   -1,  146,  147,   -1,   -1,   -1,
			  151,   -1,   -1,   -1,  155,  156,   -1,   -1,   -1,  160,
			  161,  162,   -1,  164,   -1,   -1,   -1,  168,   -1,  170,
			  171,  172,  173,   -1,  175,   -1,  177,  178,  179,  180,
			   -1,   -1,   -1,  184,  185,   -1,   -1,   -1,   -1,   -1,

			   -1,   -1,  193,  194,  195,   -1,   -1,  198,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,  206,   -1,   -1,   -1,   -1,
			   -1,  212,  213,   -1,   -1,   -1,  217,  218,  219,  220,
			   -1,  222,   -1,  224,   -1,   -1,   -1,   -1,   -1,   -1,
			  231,   -1,  233,   -1,   -1,   -1,   -1,  238,   -1,   -1,
			   -1,   -1,   -1,  244,   -1,   -1,  247,  248,  249,  250,
			   -1,   -1,   -1,  254,   -1,   -1,  257,  258,   -1,   -1,
			   -1,   -1,   -1,  264,   -1,  266,   -1,   -1,   -1,   -1,
			   -1,   -1,  273,   -1,  275,   -1,  277,   -1,  279,  280,
			  281,   -1,   -1,   -1,   -1,   -1,   -1,  288,   -1,   -1,

			  291,   -1,  293,   -1,  295,   -1,   -1,   -1,   -1,  300,
			   -1,   -1,   -1,  304,   -1,  306,    0,  308,   -1,  310,
			    4,    5,    6,    7,    8,   -1,   10,   11,   12,   13,
			   -1,   15,   16,   17,   18,   19,   -1,   21,   22,   -1,
			   24,   -1,   -1,   -1,   -1,   -1,   -1,   31,   -1,   -1,
			   -1,   35,   -1,   -1,   38,   39,   40,   -1,   -1,   -1,
			   44,   -1,   -1,   -1,   -1,   49,   50,   51,   -1,   53,
			   -1,   55,   -1,   -1,   58,   -1,   -1,   -1,   -1,   -1,
			   64,   65,   -1,   -1,   68,   -1,   70,   -1,   -1,   -1,
			   -1,   -1,   76,   -1,   -1,   79,   -1,   -1,   82,   -1, yyDummy>>,
			1, 1000, 6000)
		end

	yycheck_template_8 (an_array: ARRAY [INTEGER])
			-- Fill chunk #8 of template for `yycheck'.
		do
			yyarray_subcopy (an_array, <<
			   -1,   -1,   -1,   87,   88,   -1,   -1,   -1,   -1,   93,
			   94,   -1,   -1,   -1,   -1,   -1,   -1,  101,   -1,   -1,
			   -1,   -1,   -1,   -1,  108,  109,  110,   -1,   -1,  113,
			   -1,  115,   -1,   -1,  118,   -1,   -1,  121,  122,  123,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  132,   -1,
			  134,  135,   -1,  137,   -1,  139,  140,   -1,  142,  143,
			   -1,   -1,  146,  147,   -1,   -1,   -1,  151,   -1,   -1,
			   -1,  155,  156,   -1,   -1,   -1,  160,  161,  162,   -1,
			  164,   -1,   -1,   -1,  168,   -1,  170,  171,  172,  173,
			   -1,  175,   -1,  177,  178,  179,  180,   -1,   -1,   -1,

			  184,  185,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  193,
			  194,  195,   -1,   -1,  198,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,  206,   -1,   -1,   -1,   -1,   -1,  212,  213,
			   -1,   -1,   -1,  217,  218,  219,  220,   -1,  222,   -1,
			  224,   -1,   -1,   -1,   -1,   -1,   -1,  231,   -1,  233,
			   -1,   -1,   -1,   -1,  238,   -1,   -1,   -1,   -1,   -1,
			  244,   -1,   -1,  247,  248,  249,  250,   -1,   -1,   -1,
			  254,   -1,   -1,  257,  258,   -1,   -1,   -1,   -1,   -1,
			  264,   -1,  266,   -1,   -1,   -1,   -1,   -1,   -1,  273,
			   -1,  275,   -1,  277,   -1,  279,  280,  281,   -1,   -1,

			   -1,   -1,   -1,   -1,  288,   -1,   -1,  291,   -1,  293,
			   -1,  295,   -1,   -1,   -1,   -1,  300,   -1,   -1,   -1,
			  304,   -1,  306,    0,  308,   -1,  310,    4,    5,    6,
			    7,    8,   -1,   10,   11,   12,   13,   -1,   15,   16,
			   17,   18,   19,   -1,   21,   22,   -1,   24,   -1,   -1,
			   -1,   -1,   -1,   -1,   31,   -1,   -1,   -1,   35,   -1,
			   -1,   38,   39,   40,   -1,   -1,   -1,   44,   -1,   -1,
			   -1,   -1,   49,   50,   51,   -1,   53,   -1,   55,   -1,
			   -1,   58,   -1,   -1,   -1,   -1,   -1,   64,   65,   -1,
			   -1,   68,   -1,   70,   -1,   -1,   -1,   -1,   -1,   76,

			   -1,   -1,   79,   -1,   -1,   82,   -1,   -1,   -1,   -1,
			   87,   88,   -1,   -1,   -1,   -1,   93,   94,   -1,   -1,
			   -1,   -1,   -1,   -1,  101,   -1,   -1,   -1,   -1,   -1,
			   -1,  108,  109,  110,   -1,   -1,  113,   -1,  115,   -1,
			   -1,  118,   -1,   -1,  121,  122,  123,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,  132,   -1,  134,  135,   -1,
			  137,   -1,  139,  140,   -1,  142,  143,   -1,   -1,  146,
			  147,   -1,   -1,   -1,  151,   -1,   -1,   -1,  155,  156,
			   -1,   -1,   -1,  160,  161,  162,   -1,  164,   -1,   -1,
			   -1,  168,   -1,  170,  171,  172,  173,   -1,  175,   -1,

			  177,  178,  179,  180,   -1,   -1,   -1,  184,  185,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,  193,  194,  195,   -1,
			   -1,  198,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  206,
			   -1,   -1,   -1,   -1,   -1,  212,  213,   -1,   -1,   -1,
			  217,  218,  219,  220,   -1,  222,   -1,  224,   -1,   -1,
			   -1,   -1,   -1,   -1,  231,   -1,  233,   -1,   -1,   -1,
			   -1,  238,   -1,   -1,   -1,   -1,   -1,  244,   -1,   -1,
			  247,  248,  249,  250,   -1,   -1,   -1,  254,   -1,   -1,
			  257,  258,   -1,   -1,   -1,   -1,   -1,  264,   -1,  266,
			   -1,   -1,   -1,   -1,   -1,   -1,  273,   -1,  275,   -1,

			  277,   -1,  279,  280,  281,   -1,   -1,   -1,   -1,   -1,
			   -1,  288,   -1,   -1,  291,   -1,  293,   -1,  295,   -1,
			   -1,   -1,   -1,  300,   -1,   -1,   -1,  304,   -1,  306,
			    0,  308,   -1,  310,    4,    5,    6,    7,    8,   -1,
			   10,   11,   12,   13,   -1,   15,   16,   17,   18,   19,
			   -1,   21,   22,   -1,   24,   -1,   -1,   -1,   -1,   -1,
			   -1,   31,   -1,   -1,   -1,   35,   -1,   -1,   38,   39,
			   40,   -1,   -1,   -1,   44,   -1,   -1,   -1,   -1,   49,
			   50,   51,   -1,   53,   -1,   55,   -1,   -1,   58,   -1,
			   -1,   -1,   -1,   -1,   64,   65,   -1,   -1,   68,   -1,

			   70,   -1,   -1,   -1,   -1,   -1,   76,   -1,   -1,   79,
			   -1,   -1,   82,   -1,   -1,   -1,   -1,   87,   88,   -1,
			   -1,   -1,   -1,   93,   94,   -1,   -1,   -1,   -1,   -1,
			   -1,  101,   -1,   -1,   -1,   -1,   -1,   -1,  108,  109,
			  110,   -1,   -1,  113,   -1,  115,   -1,   -1,  118,   -1,
			   -1,  121,  122,  123,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,  132,   -1,  134,  135,   -1,  137,   -1,  139,
			  140,   -1,  142,  143,   -1,   -1,  146,  147,   -1,   -1,
			   -1,  151,   -1,   -1,   -1,  155,  156,   -1,   -1,   -1,
			  160,  161,  162,   -1,  164,   -1,   -1,   -1,  168,   -1,

			  170,  171,  172,  173,   -1,  175,   -1,  177,  178,  179,
			  180,   -1,   -1,   -1,  184,  185,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,  193,  194,  195,   -1,   -1,  198,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,  206,   -1,   -1,   -1,
			   -1,   -1,  212,  213,   -1,   -1,   -1,  217,  218,  219,
			  220,   -1,  222,   -1,  224,   -1,   -1,   -1,   -1,   -1,
			   -1,  231,   -1,  233,   -1,   -1,   -1,   -1,  238,   -1,
			   -1,   -1,   -1,   -1,  244,   -1,   -1,  247,  248,  249,
			  250,   -1,   -1,   -1,  254,   -1,   -1,  257,  258,   -1,
			   -1,   -1,   -1,   -1,  264,   -1,  266,   -1,   -1,   -1,

			   -1,   -1,   -1,  273,   -1,  275,   -1,  277,   -1,  279,
			  280,  281,   -1,   -1,   -1,   -1,   -1,   -1,  288,   -1,
			   -1,  291,   -1,  293,   -1,  295,   -1,   -1,   -1,   -1,
			  300,   -1,   -1,   -1,  304,   -1,  306,    0,  308,   -1,
			  310,    4,    5,    6,    7,    8,   -1,   10,   11,   12,
			   13,   -1,   15,   16,   17,   18,   19,   -1,   21,   22,
			   -1,   24,   -1,   -1,   -1,   -1,   -1,   -1,   31,   -1,
			   -1,   -1,   35,   -1,   -1,   38,   39,   40,   -1,   -1,
			   -1,   44,   -1,   -1,   -1,   -1,   49,   50,   51,   -1,
			   53,   -1,   55,   -1,   -1,   58,   -1,   -1,   -1,   -1,

			   -1,   64,   65,   -1,   -1,   68,   -1,   70,   -1,   -1,
			   -1,   -1,   -1,   76,   -1,   -1,   79,   -1,   -1,   82,
			   -1,   -1,   -1,   -1,   87,   88,   -1,   -1,   -1,   -1,
			   93,   94,   -1,   -1,   -1,   -1,   -1,   -1,  101,   -1,
			   -1,   -1,   -1,   -1,   -1,  108,  109,  110,   -1,   -1,
			  113,   -1,  115,   -1,   -1,  118,   -1,   -1,  121,  122,
			  123,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  132,
			   -1,  134,  135,   -1,  137,   -1,  139,  140,   -1,  142,
			  143,   -1,   -1,  146,  147,   -1,   -1,   -1,  151,   -1,
			   -1,   -1,  155,  156,   -1,   -1,   -1,  160,  161,  162, yyDummy>>,
			1, 1000, 7000)
		end

	yycheck_template_9 (an_array: ARRAY [INTEGER])
			-- Fill chunk #9 of template for `yycheck'.
		do
			yyarray_subcopy (an_array, <<
			   -1,  164,   -1,   -1,   -1,  168,   -1,  170,  171,  172,
			  173,   -1,  175,   -1,  177,  178,  179,  180,   -1,   -1,
			   -1,  184,  185,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			  193,  194,  195,   -1,   -1,  198,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,  206,   -1,   -1,   -1,   -1,   -1,  212,
			  213,   -1,   -1,   -1,  217,  218,  219,  220,   -1,  222,
			   -1,  224,   -1,   -1,   -1,   -1,   -1,   -1,  231,   -1,
			  233,   -1,   -1,   -1,   -1,  238,   -1,   -1,   -1,   -1,
			   -1,  244,   -1,   -1,  247,  248,  249,  250,   -1,   -1,
			   -1,  254,   -1,   -1,  257,  258,   -1,   -1,   -1,   -1,

			   -1,  264,   -1,  266,   -1,   -1,   -1,   -1,   -1,   -1,
			  273,   -1,  275,   -1,  277,   -1,  279,  280,  281,   -1,
			   -1,   -1,   -1,   -1,   -1,  288,   -1,   -1,  291,   -1,
			  293,   -1,  295,   -1,   -1,   -1,   -1,  300,   -1,   -1,
			   -1,  304,   -1,  306,    0,  308,   -1,  310,    4,    5,
			    6,    7,    8,   -1,   10,   11,   12,   13,   -1,   15,
			   16,   17,   18,   19,   -1,   21,   22,   -1,   24,   -1,
			   -1,   -1,   -1,   -1,   -1,   31,   -1,   -1,   -1,   35,
			   -1,   -1,   38,   39,   40,   -1,   -1,   -1,   44,   -1,
			   -1,   -1,   -1,   49,   50,   51,   -1,   53,   -1,   55,

			   -1,   -1,   58,   -1,   -1,   -1,   -1,   -1,   64,   65,
			   -1,   -1,   68,   -1,   70,   -1,   -1,   -1,   -1,   -1,
			   76,   -1,   -1,   79,   -1,   -1,   82,   -1,   -1,   -1,
			   -1,   87,   88,   -1,   -1,   -1,   -1,   93,   94,   -1,
			   -1,   -1,   -1,   -1,   -1,  101,   -1,   -1,   -1,   -1,
			   -1,   -1,  108,  109,  110,   -1,   -1,  113,   -1,  115,
			   -1,   -1,  118,   -1,   -1,  121,  122,  123,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,  132,   -1,  134,  135,
			   -1,  137,   -1,  139,  140,   -1,  142,  143,   -1,   -1,
			  146,  147,   -1,   -1,   -1,  151,   -1,   -1,   -1,  155,

			  156,   -1,   -1,   -1,  160,  161,  162,   -1,  164,   -1,
			   -1,   -1,  168,   -1,  170,  171,  172,  173,   -1,  175,
			   -1,  177,  178,  179,  180,   -1,   -1,   -1,  184,  185,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,  193,  194,  195,
			   -1,   -1,  198,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			  206,   -1,   -1,   -1,   -1,   -1,  212,  213,   -1,   -1,
			   -1,  217,  218,  219,  220,   -1,  222,   -1,  224,   -1,
			   -1,   -1,   -1,   -1,   -1,  231,   -1,  233,   -1,   -1,
			   -1,   -1,  238,   -1,   -1,   -1,   -1,   -1,  244,   -1,
			   -1,  247,  248,  249,  250,   -1,   -1,   -1,  254,   -1,

			   -1,  257,  258,   -1,   -1,   -1,   -1,   -1,  264,   -1,
			  266,   -1,   -1,   -1,   -1,   -1,   -1,  273,   -1,  275,
			   -1,  277,   -1,  279,  280,  281,   -1,   -1,   -1,   -1,
			   -1,   -1,  288,   -1,   -1,  291,   -1,  293,   -1,  295,
			   -1,   -1,   -1,   -1,  300,   -1,   -1,   -1,  304,   -1,
			  306,    0,  308,   -1,  310,    4,    5,    6,    7,    8,
			   -1,   10,   11,   12,   13,   -1,   15,   16,   17,   18,
			   19,   -1,   21,   22,   -1,   24,   -1,   -1,   -1,   -1,
			   -1,   -1,   31,   -1,   -1,   -1,   35,   -1,   -1,   38,
			   39,   40,   -1,   -1,   -1,   44,   -1,   -1,   -1,   -1,

			   49,   50,   51,   -1,   53,   -1,   55,   -1,   -1,   58,
			   -1,   -1,   -1,   -1,   -1,   64,   65,   -1,   -1,   68,
			   -1,   70,   -1,   -1,   -1,   -1,   -1,   76,   -1,   -1,
			   79,   -1,   -1,   82,   -1,   -1,   -1,   -1,   87,   88,
			   -1,   -1,   -1,   -1,   93,   94,   -1,   -1,   -1,   -1,
			   -1,   -1,  101,   -1,   -1,   -1,   -1,   -1,   -1,  108,
			  109,  110,   -1,   -1,  113,   -1,  115,   -1,   -1,  118,
			   -1,   -1,  121,  122,  123,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,  132,   -1,  134,  135,   -1,  137,   -1,
			  139,  140,   -1,  142,  143,   -1,   -1,  146,  147,   -1,

			   -1,   -1,  151,   -1,   -1,   -1,  155,  156,   -1,   -1,
			   -1,  160,  161,  162,   -1,  164,   -1,   -1,   -1,  168,
			   -1,  170,  171,  172,  173,   -1,  175,   -1,  177,  178,
			  179,  180,   -1,   -1,   -1,  184,  185,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,  193,  194,  195,   -1,   -1,  198,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,  206,   -1,   -1,
			   -1,   -1,   -1,  212,  213,   -1,   -1,   -1,  217,  218,
			  219,  220,   -1,  222,   -1,  224,   -1,   -1,   -1,   -1,
			   -1,   -1,  231,   -1,  233,   -1,   -1,   -1,   -1,  238,
			   -1,   -1,   -1,   -1,   -1,  244,   -1,   -1,  247,  248,

			  249,  250,   -1,   -1,   -1,  254,   -1,   -1,  257,  258,
			   -1,   -1,   -1,   -1,   -1,  264,   -1,  266,   -1,   -1,
			   -1,   -1,   -1,   -1,  273,   -1,  275,   -1,  277,   -1,
			  279,  280,  281,   -1,   -1,   -1,   -1,   -1,   -1,  288,
			   -1,   -1,  291,   -1,  293,   -1,  295,   -1,   -1,   -1,
			   -1,  300,   -1,   -1,   -1,  304,   -1,  306,    0,  308,
			   -1,  310,    4,    5,    6,    7,    8,   -1,   10,   11,
			   12,   13,   -1,   15,   16,   17,   18,   19,   -1,   21,
			   22,   -1,   24,   -1,   -1,   -1,   -1,   -1,   -1,   31,
			   -1,   -1,   -1,   35,   -1,   -1,   38,   39,   40,   -1,

			   -1,   -1,   44,   -1,   -1,   -1,   -1,   49,   50,   51,
			   -1,   53,   -1,   55,   -1,   -1,   58,   -1,   -1,   -1,
			   -1,   -1,   64,   65,   -1,   -1,   68,   -1,   70,   -1,
			   -1,   -1,   -1,   -1,   76,   -1,   -1,   79,   -1,   -1,
			   82,   -1,   -1,   -1,   -1,   87,   88,   -1,   -1,   -1,
			   -1,   93,   94,   -1,   -1,   -1,   -1,   -1,   -1,  101,
			   -1,   -1,   -1,   -1,   -1,   -1,  108,  109,  110,   -1,
			   -1,  113,   -1,  115,   -1,   -1,  118,   -1,   -1,  121,
			  122,  123,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			  132,   -1,  134,  135,   -1,  137,   -1,  139,  140,   -1,

			  142,  143,   -1,   -1,  146,  147,   -1,   -1,   -1,  151,
			   -1,   -1,   -1,  155,  156,   -1,   -1,   -1,  160,  161,
			  162,   -1,  164,   -1,   -1,   -1,  168,   -1,  170,  171,
			  172,  173,   -1,  175,   -1,  177,  178,  179,  180,   -1,
			   -1,   -1,  184,  185,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,  193,  194,  195,   -1,   -1,  198,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,  206,   -1,   -1,   -1,   -1,   -1,
			  212,  213,   -1,   -1,   -1,  217,  218,  219,  220,   -1,
			  222,   -1,  224,   -1,   -1,   -1,   -1,   -1,   -1,  231,
			   -1,  233,   -1,   -1,   -1,   -1,  238,   -1,   -1,   -1, yyDummy>>,
			1, 1000, 8000)
		end

	yycheck_template_10 (an_array: ARRAY [INTEGER])
			-- Fill chunk #10 of template for `yycheck'.
		do
			yyarray_subcopy (an_array, <<
			   -1,   -1,  244,   -1,   -1,  247,  248,  249,  250,   -1,
			   -1,   -1,  254,   -1,   -1,  257,  258,   -1,   -1,   -1,
			   -1,   -1,  264,   -1,  266,   -1,   -1,   -1,   -1,   -1,
			   -1,  273,   -1,  275,   -1,  277,   -1,  279,  280,  281,
			   -1,   -1,   -1,   -1,   -1,   -1,  288,   -1,   -1,  291,
			   -1,  293,   -1,  295,   -1,   -1,   -1,   -1,  300,   -1,
			   -1,   -1,  304,   -1,  306,    0,  308,   -1,  310,    4,
			    5,    6,    7,    8,   -1,   10,   11,   12,   13,   -1,
			   15,   16,   17,   18,   19,   -1,   21,   22,   -1,   24,
			   -1,   -1,   -1,   -1,   -1,   -1,   31,   -1,   -1,   -1,

			   35,   -1,   -1,   38,   39,   40,   -1,   -1,   -1,   44,
			   -1,   -1,   -1,   -1,   49,   50,   51,   -1,   53,   -1,
			   55,   -1,   -1,   58,   -1,   -1,   -1,   -1,   -1,   64,
			   65,   -1,   -1,   68,   -1,   70,   -1,   -1,   -1,   -1,
			   -1,   76,   -1,   -1,   79,   -1,   -1,   82,   -1,   -1,
			   -1,   -1,   87,   88,   -1,   -1,   -1,   -1,   93,   94,
			   -1,   -1,   -1,   -1,   -1,   -1,  101,   -1,   -1,   -1,
			   -1,   -1,   -1,  108,  109,  110,   -1,   -1,  113,   -1,
			  115,   -1,   -1,  118,   -1,   -1,  121,  122,  123,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,  132,   -1,  134,

			  135,   -1,  137,   -1,  139,  140,   -1,  142,  143,   -1,
			   -1,  146,  147,   -1,   -1,   -1,  151,   -1,   -1,   -1,
			  155,  156,   -1,   -1,   -1,  160,  161,  162,   -1,  164,
			   -1,   -1,   -1,  168,   -1,  170,  171,  172,  173,   -1,
			  175,   -1,  177,  178,  179,  180,   -1,   -1,   -1,  184,
			  185,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  193,  194,
			  195,   -1,   -1,  198,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,  206,   -1,   -1,   -1,   -1,   -1,  212,  213,   -1,
			   -1,   -1,  217,  218,  219,  220,   -1,  222,   -1,  224,
			   -1,   -1,   -1,   -1,   -1,   -1,  231,   -1,  233,   -1,

			   -1,   -1,   -1,  238,   -1,   -1,   -1,   -1,   -1,  244,
			   -1,   -1,  247,  248,  249,  250,   -1,   -1,   -1,  254,
			   -1,   -1,  257,  258,   -1,   -1,   -1,   -1,   -1,  264,
			   -1,  266,   -1,   -1,   -1,   -1,   -1,   -1,  273,   -1,
			  275,   -1,  277,   -1,  279,  280,  281,   -1,   -1,   -1,
			   -1,   -1,   -1,  288,   -1,   -1,  291,   -1,  293,   -1,
			  295,   -1,   -1,   -1,   -1,  300,   -1,   -1,   -1,  304,
			   -1,  306,    0,  308,   -1,  310,    4,    5,    6,    7,
			    8,   -1,   10,   11,   12,   13,   -1,   15,   16,   17,
			   18,   19,   -1,   21,   22,   -1,   24,   -1,   -1,   -1,

			   -1,   -1,   -1,   31,   -1,   -1,   -1,   35,   -1,   -1,
			   38,   39,   40,   -1,   -1,   -1,   44,   -1,   -1,   -1,
			   -1,   49,   50,   51,   -1,   53,   -1,   55,   -1,   -1,
			   58,   -1,   -1,   -1,   -1,   -1,   64,   65,   -1,   -1,
			   68,   -1,   70,   -1,   -1,   -1,   -1,   -1,   76,   -1,
			   -1,   79,   -1,   -1,   82,   -1,   -1,   -1,   -1,   87,
			   88,   -1,   -1,   -1,   -1,   93,   94,   -1,   -1,   -1,
			   -1,   -1,   -1,  101,   -1,   -1,   -1,   -1,   -1,   -1,
			  108,  109,  110,   -1,   -1,  113,   -1,  115,   -1,   -1,
			  118,   -1,   -1,  121,  122,  123,   -1,   -1,   -1,   -1,

			   -1,   -1,   -1,   -1,  132,   -1,  134,  135,   -1,  137,
			   -1,  139,  140,   -1,  142,  143,   -1,   -1,  146,  147,
			   -1,   -1,   -1,  151,   -1,   -1,   -1,  155,  156,   -1,
			   -1,   -1,  160,  161,  162,   -1,  164,   -1,   -1,   -1,
			  168,   -1,  170,  171,  172,  173,   -1,  175,   -1,  177,
			  178,  179,  180,   -1,   -1,   -1,  184,  185,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,  193,  194,  195,   -1,   -1,
			  198,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  206,   -1,
			   -1,   -1,   -1,   -1,  212,  213,   -1,   -1,   -1,  217,
			  218,  219,  220,   -1,  222,   -1,  224,   -1,   -1,   -1,

			   -1,   -1,   -1,  231,   -1,  233,   -1,   -1,   -1,   -1,
			  238,   -1,   -1,   -1,   -1,   -1,  244,   -1,   -1,  247,
			  248,  249,  250,   -1,   -1,   -1,  254,   -1,   -1,  257,
			  258,   -1,   -1,   -1,   -1,   -1,  264,   -1,  266,   -1,
			   -1,   -1,   -1,   -1,   -1,  273,   -1,  275,   -1,  277,
			   -1,  279,  280,  281,   -1,   -1,   -1,   -1,   -1,   -1,
			  288,   -1,   -1,  291,   -1,  293,   -1,  295,   -1,   -1,
			   -1,   -1,  300,   -1,   -1,   -1,  304,   -1,  306,    0,
			  308,   -1,  310,    4,    5,    6,    7,    8,   -1,   10,
			   11,   12,   13,   -1,   15,   16,   17,   18,   19,   -1,

			   21,   22,   -1,   24,   -1,   -1,   -1,   -1,   -1,   -1,
			   31,   -1,   -1,   -1,   35,   -1,   -1,   38,   39,   40,
			   -1,   -1,   -1,   44,   -1,   -1,   -1,   -1,   49,   50,
			   51,   -1,   53,   -1,   55,   -1,   -1,   58,   -1,   -1,
			   -1,   -1,   -1,   64,   65,   -1,   -1,   68,   -1,   70,
			   -1,   -1,   -1,   -1,   -1,   76,   -1,   -1,   79,   -1,
			   -1,   82,   -1,   -1,   -1,   -1,   87,   88,   -1,   -1,
			   -1,   -1,   93,   94,   -1,   -1,   -1,   -1,   -1,   -1,
			  101,   -1,   -1,   -1,   -1,   -1,   -1,  108,  109,  110,
			   -1,   -1,  113,   -1,  115,   -1,   -1,  118,   -1,   -1,

			  121,  122,  123,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,  132,   -1,  134,  135,   -1,  137,   -1,  139,  140,
			   -1,  142,  143,   -1,   -1,  146,  147,   -1,   -1,   -1,
			  151,   -1,   -1,   -1,  155,  156,   -1,   -1,   -1,  160,
			  161,  162,   -1,  164,   -1,   -1,   -1,  168,   -1,  170,
			  171,  172,  173,   -1,  175,   -1,  177,  178,  179,  180,
			   -1,   -1,   -1,  184,  185,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,  193,  194,  195,   -1,   -1,  198,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,  206,   -1,   -1,   -1,   -1,
			   -1,  212,  213,   -1,   -1,   -1,  217,  218,  219,  220,

			   -1,  222,   -1,  224,   -1,   -1,   -1,   -1,   -1,   -1,
			  231,   -1,  233,   -1,   -1,   -1,   -1,  238,   -1,   -1,
			   -1,   -1,   -1,  244,   -1,   -1,  247,  248,  249,  250,
			   -1,   -1,   -1,  254,   -1,   -1,  257,  258,   -1,   -1,
			   -1,   -1,   -1,  264,   -1,  266,   -1,   -1,   -1,   -1,
			   -1,   -1,  273,   -1,  275,   -1,  277,   -1,  279,  280,
			  281,   -1,   -1,   -1,   -1,   -1,   -1,  288,   -1,   -1,
			  291,   -1,  293,   -1,  295,   -1,   -1,   -1,   -1,  300,
			   -1,   -1,   -1,  304,   -1,  306,    0,  308,   -1,  310,
			    4,    5,    6,    7,    8,   -1,   10,   11,   12,   13, yyDummy>>,
			1, 1000, 9000)
		end

	yycheck_template_11 (an_array: ARRAY [INTEGER])
			-- Fill chunk #11 of template for `yycheck'.
		do
			yyarray_subcopy (an_array, <<
			   -1,   15,   16,   17,   18,   19,   -1,   21,   22,   -1,
			   24,   -1,   -1,   -1,   -1,   -1,   -1,   31,   -1,   -1,
			   -1,   35,   -1,   -1,   38,   39,   40,   -1,   -1,   -1,
			   44,   -1,   -1,   -1,   -1,   49,   50,   51,   -1,   53,
			   -1,   55,   -1,   -1,   58,   -1,   -1,   -1,   -1,   -1,
			   64,   65,   -1,   -1,   68,   -1,   70,   -1,   -1,   -1,
			   -1,   -1,   76,   -1,   -1,   79,   -1,   -1,   82,   -1,
			   -1,   -1,   -1,   87,   88,   -1,   -1,   -1,   -1,   93,
			   94,   -1,   -1,   -1,   -1,   -1,   -1,  101,   -1,   -1,
			   -1,   -1,   -1,   -1,  108,  109,  110,   -1,   -1,  113,

			   -1,  115,   -1,   -1,  118,   -1,   -1,  121,  122,  123,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  132,   -1,
			  134,  135,   -1,  137,   -1,  139,  140,   -1,  142,  143,
			   -1,   -1,  146,  147,   -1,   -1,   -1,  151,   -1,   -1,
			   -1,  155,  156,   -1,   -1,   -1,  160,  161,  162,   -1,
			  164,   -1,   -1,   -1,  168,   -1,  170,  171,  172,  173,
			   -1,  175,   -1,  177,  178,  179,  180,   -1,   -1,   -1,
			  184,  185,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  193,
			  194,  195,   -1,   -1,  198,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,  206,   -1,   -1,   -1,   -1,   -1,  212,  213,

			   -1,   -1,   -1,  217,  218,  219,  220,   -1,  222,   -1,
			  224,   -1,   -1,   -1,   -1,   -1,   -1,  231,   -1,  233,
			   -1,   -1,   -1,   -1,  238,   -1,   -1,   -1,   -1,   -1,
			  244,   -1,   -1,  247,  248,  249,  250,   -1,   -1,   -1,
			  254,   -1,   -1,  257,  258,   -1,   -1,   -1,   -1,   -1,
			  264,   -1,  266,   -1,   -1,   -1,   -1,   -1,   -1,  273,
			   -1,  275,   -1,  277,   -1,  279,  280,  281,   -1,   -1,
			   -1,   -1,   -1,   -1,  288,   -1,   -1,  291,   -1,  293,
			    3,  295,    5,    6,   -1,    8,  300,   -1,   -1,   -1,
			  304,   14,  306,   -1,  308,   -1,  310,   -1,   -1,   -1,

			   23,   24,   25,   26,   27,   28,   29,   30,   -1,   32,
			   -1,   -1,   -1,   -1,   37,   38,   -1,   40,   -1,   42,
			   -1,   -1,   45,   -1,   -1,   -1,   -1,   -1,   51,   -1,
			   53,   -1,   55,   -1,   -1,   58,   59,   -1,   61,   -1,
			   63,   -1,   -1,   -1,   -1,   68,   69,   70,   -1,   72,
			   73,   -1,   -1,   76,   -1,   -1,   79,   -1,   81,   82,
			   -1,   -1,   -1,   -1,   87,   88,   -1,   -1,   91,   -1,
			   -1,   -1,   -1,   -1,   97,   98,   99,   -1,  101,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,  110,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,

			   -1,   -1,   -1,   -1,   -1,   -1,   -1,  130,   -1,  132,
			   -1,   -1,  135,   -1,  137,   -1,   -1,   -1,   -1,  142,
			  143,   -1,   -1,   -1,   -1,  148,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,  156,   -1,   -1,   -1,   -1,  161,  162,
			   -1,  164,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  172,
			   -1,   -1,  175,   -1,  177,  178,   -1,  180,  181,   -1,
			   -1,  184,  185,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			  193,  194,   -1,   -1,  197,  198,   -1,   -1,   -1,  202,
			  203,   -1,   -1,  206,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,  217,  218,  219,   -1,   -1,  222,

			   -1,  224,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			  233,   -1,   -1,   -1,   -1,  238,   -1,   -1,   -1,   -1,
			   -1,  244,   -1,   -1,   -1,  248,  249,  250,   -1,   -1,
			   -1,  254,   -1,  256,  257,   -1,   -1,   -1,   -1,   -1,
			   -1,  264,   -1,  266,   -1,   -1,   -1,   -1,   -1,   -1,
			  273,   -1,  275,   -1,  277,   -1,  279,  280,   -1,   -1,
			   -1,  284,  285,   -1,   -1,  288,   -1,   -1,  291,   -1,
			   -1,    3,  295,    5,   -1,  298,   -1,  300,   -1,   -1,
			   -1,  304,   14,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			  313,   -1,   24,   25,   26,   27,   28,   29,   30,   -1,

			   32,   -1,   -1,   -1,   -1,   37,   38,   -1,   40,   -1,
			   42,   -1,   -1,   45,   -1,   -1,   -1,   -1,   -1,   51,
			   -1,   53,   -1,   55,   -1,   -1,   -1,   59,   -1,   61,
			   -1,   -1,   -1,   -1,   -1,   -1,   68,   69,   70,   -1,
			   72,   73,   -1,   -1,   76,   -1,   -1,   79,   -1,   81,
			   82,   -1,   -1,   -1,   -1,   87,   88,   -1,   -1,   91,
			   -1,   -1,   -1,   -1,   -1,   97,   98,   99,   -1,  101,
			   -1,   -1,   -1,   -1,  106,   -1,   -1,   -1,  110,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  130,   -1,

			  132,   -1,   -1,  135,   -1,  137,   -1,   -1,   -1,   -1,
			  142,  143,   -1,   -1,   -1,   -1,  148,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,  156,   -1,   -1,   -1,   -1,  161,
			  162,   -1,  164,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			  172,   -1,   -1,  175,   -1,  177,  178,   -1,  180,  181,
			   -1,   -1,  184,  185,   -1,   -1,    0,   -1,   -1,   -1,
			    4,  193,  194,    7,   -1,  197,  198,   11,   -1,   -1,
			  202,  203,   -1,   -1,  206,   -1,   -1,   -1,   -1,   -1,
			   24,   -1,   -1,   -1,   -1,  217,  218,  219,   -1,   -1,
			  222,   -1,  224,   -1,   -1,   -1,   -1,   -1,   -1,   -1,

			   -1,  233,   -1,   -1,   -1,   -1,  238,   -1,   -1,   -1,
			   -1,   -1,  244,   -1,   -1,   -1,  248,  249,  250,   -1,
			   64,   -1,  254,   -1,  256,  257,   -1,   -1,   -1,   -1,
			   -1,    0,  264,   -1,  266,    4,   -1,   -1,    7,   -1,
			   -1,  273,   11,  275,   -1,  277,   -1,  279,  280,   -1,
			   94,   -1,  284,  285,   -1,   24,  288,   -1,   -1,  291,
			   -1,   -1,   -1,  295,   -1,   -1,  298,   -1,  300,  113,
			   -1,   -1,  304,   -1,   -1,   -1,   -1,  121,   -1,  123,
			   -1,  313,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			  134,   -1,   -1,   -1,   -1,   64,  140,   -1,   -1,   -1,

			   -1,   -1,   -1,  147,   -1,   -1,   -1,  151,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,  160,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,  168,   94,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,  177,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,  113,   -1,   -1,   -1,   -1,   -1,
			   -1,  195,  121,   -1,  123,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,  134,   -1,   -1,  212,  213,
			   -1,  140,   -1,   -1,   -1,    3,   -1,   -1,  147,   -1,
			   -1,   -1,  151,   -1,   -1,   -1,   -1,  231,   -1,   -1,
			   -1,  160,   -1,   -1,   -1,   -1,   24,   -1,   -1,  168, yyDummy>>,
			1, 1000, 10000)
		end

	yycheck_template_12 (an_array: ARRAY [INTEGER])
			-- Fill chunk #12 of template for `yycheck'.
		do
			yyarray_subcopy (an_array, <<
			   -1,   -1,   -1,   -1,   -1,  249,   -1,   -1,  177,   -1,
			   38,   -1,   40,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   49,   -1,   51,   -1,   53,  195,   55,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   68,   -1,   70,  212,  213,   -1,   -1,   -1,   76,  293,
			   -1,   79,   -1,   -1,   82,   -1,   -1,   -1,   -1,   87,
			   88,   -1,  231,   -1,  308,   -1,  310,   -1,   -1,   -1,
			   -1,   -1,   -1,  101,   -1,   -1,   -1,   -1,   -1,   -1,
			  249,   -1,  110,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,

			   -1,   -1,   -1,   -1,  132,   -1,   -1,  135,   -1,  137,
			   -1,   -1,   -1,   -1,  142,  143,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,  293,   -1,   -1,   -1,  156,   -1,
			   -1,   -1,   -1,  161,  162,   -1,  164,   -1,   -1,  308,
			   -1,  310,   -1,   -1,  172,   -1,   -1,  175,   -1,   -1,
			  178,   -1,  180,   -1,   -1,   -1,  184,  185,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,  193,  194,   -1,   -1,   -1,
			  198,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  206,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  217,
			  218,  219,   -1,   -1,  222,   -1,  224,   -1,   -1,   -1,

			   -1,   -1,   -1,   -1,   -1,  233,   -1,   -1,   -1,   -1,
			  238,   -1,   -1,   -1,   -1,   -1,  244,   -1,    3,   -1,
			  248,   -1,  250,   -1,   -1,   -1,  254,   -1,   -1,  257,
			   -1,   -1,   -1,   -1,   -1,   -1,  264,   -1,  266,   24,
			   -1,   -1,   -1,   -1,   -1,  273,   -1,  275,   -1,  277,
			   -1,  279,  280,   38,   -1,   40,   -1,   -1,   -1,   -1,
			  288,   -1,   -1,  291,   49,   -1,   51,  295,   53,   -1,
			   55,   -1,  300,   -1,   -1,   -1,  304,   -1,   -1,   -1,
			   -1,   -1,  310,   68,   -1,   70,   -1,   -1,   -1,   -1,
			   -1,   76,   -1,   -1,   79,   -1,   -1,   82,   -1,   -1,

			   -1,   -1,   87,   88,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,  101,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,  110,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,  132,   -1,   -1,
			  135,   -1,  137,   -1,   -1,   -1,   -1,  142,  143,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,  156,   -1,   -1,   -1,   -1,  161,  162,   -1,  164,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,  172,   -1,   -1,
			  175,   -1,   -1,  178,   -1,  180,   -1,   -1,   -1,  184,

			  185,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  193,  194,
			   -1,   -1,   -1,  198,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,  206,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,  217,  218,  219,   -1,   -1,  222,   -1,  224,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  233,   -1,
			   -1,   -1,   -1,  238,   -1,   -1,   -1,   -1,   -1,  244,
			   -1,   -1,   -1,  248,   -1,  250,   -1,   -1,   -1,  254,
			   -1,   -1,  257,   -1,   -1,   -1,   -1,   -1,   -1,  264,
			   -1,  266,   -1,   -1,   -1,   -1,   -1,   -1,  273,   -1,
			  275,   -1,  277,   -1,  279,  280,   -1,   -1,   -1,   -1,

			   -1,   -1,   -1,  288,   -1,    3,  291,    5,   -1,   -1,
			  295,   -1,   10,   -1,   -1,  300,   14,   -1,   -1,  304,
			   18,   19,   -1,   -1,   -1,  310,   24,   25,   26,   27,
			   28,   29,   30,   31,   32,   -1,   -1,   -1,   -1,   37,
			   38,   -1,   40,   -1,   42,   -1,   44,   45,   -1,   -1,
			   -1,   -1,   -1,   51,   -1,   53,   -1,   55,   -1,   -1,
			   58,   59,   -1,   61,   -1,   -1,   -1,   -1,   -1,   -1,
			   68,   69,   70,   -1,   72,   73,   -1,   -1,   76,   -1,
			   78,   79,   -1,   81,   82,   -1,   -1,   85,   -1,   87,
			   88,   -1,   -1,   91,   -1,   93,   -1,   -1,   -1,   97,

			   98,   99,   -1,  101,   -1,   -1,   -1,   -1,  106,   -1,
			  108,   -1,  110,   -1,   -1,   -1,   -1,  115,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,  125,   -1,   -1,
			   -1,   -1,  130,  131,  132,   -1,  134,  135,   -1,  137,
			   -1,   -1,   -1,   -1,  142,  143,  144,  145,   -1,   -1,
			  148,   -1,   -1,   -1,   -1,   -1,  154,   -1,  156,   -1,
			   -1,   -1,   -1,  161,  162,  163,  164,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,  172,   -1,   -1,  175,   -1,  177,
			  178,   -1,  180,  181,   -1,   -1,  184,  185,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,  193,  194,   -1,   -1,  197,

			  198,   -1,   -1,   -1,  202,  203,   -1,   -1,  206,   -1,
			   -1,   -1,   -1,  211,   -1,   -1,   -1,   -1,   -1,  217,
			  218,  219,   -1,   -1,  222,   -1,  224,   -1,   -1,   -1,
			  228,   -1,   -1,   -1,   -1,  233,   -1,   -1,   -1,   -1,
			  238,   -1,   -1,   -1,  242,   -1,  244,  245,   -1,   -1,
			  248,  249,  250,  251,   -1,   -1,  254,   -1,  256,  257,
			  258,   -1,   -1,   -1,   -1,   -1,  264,   -1,  266,   -1,
			   -1,   -1,   -1,   -1,   -1,  273,   -1,  275,   -1,  277,
			   -1,  279,  280,   -1,   -1,   -1,  284,  285,   -1,   -1,
			  288,   -1,    3,  291,    5,   -1,   -1,  295,  296,   -1,

			  298,   -1,  300,   14,   -1,   -1,  304,   -1,   -1,  307,
			   -1,  309,   -1,   24,   25,   26,   27,   28,   29,   30,
			   -1,   32,   -1,   -1,   -1,   -1,   37,   38,   -1,   40,
			   -1,   42,   -1,   44,   45,   -1,   -1,   -1,   -1,   -1,
			   51,   -1,   53,   -1,   55,   -1,   -1,   58,   59,   -1,
			   61,   -1,   -1,   -1,   -1,   -1,   -1,   68,   69,   70,
			   -1,   72,   73,   -1,   -1,   76,   -1,   78,   79,   -1,
			   81,   82,   -1,   -1,   85,   -1,   87,   88,   -1,   -1,
			   91,   -1,   93,   -1,   -1,   -1,   97,   98,   99,   -1,
			  101,   -1,   -1,   -1,   -1,   -1,   -1,  108,   -1,  110,

			   -1,   -1,   -1,   -1,  115,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,  125,   -1,   -1,   -1,   -1,  130,
			  131,  132,   -1,  134,  135,   -1,  137,   -1,   -1,   -1,
			   -1,  142,  143,  144,  145,   -1,   -1,  148,   -1,   -1,
			   -1,   -1,   -1,  154,   -1,  156,   -1,   -1,   -1,   -1,
			  161,  162,  163,  164,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,  172,   -1,   -1,  175,   -1,  177,  178,   -1,  180,
			  181,   -1,   -1,  184,  185,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,  193,  194,   -1,   -1,  197,  198,   -1,   -1,
			   -1,  202,  203,   -1,   -1,  206,   -1,   -1,   -1,   -1, yyDummy>>,
			1, 1000, 11000)
		end

	yycheck_template_13 (an_array: ARRAY [INTEGER])
			-- Fill chunk #13 of template for `yycheck'.
		do
			yyarray_subcopy (an_array, <<
			  211,   -1,   -1,   -1,   -1,   -1,  217,  218,  219,   -1,
			   -1,  222,   -1,  224,   -1,   -1,   -1,  228,   -1,   -1,
			   -1,   -1,  233,   -1,   -1,   -1,   -1,  238,   -1,   -1,
			   -1,  242,   -1,  244,  245,   -1,   -1,  248,  249,  250,
			  251,   -1,   -1,  254,   -1,  256,  257,  258,   -1,   -1,
			   -1,   -1,   -1,  264,   -1,  266,   -1,   -1,   -1,   -1,
			   -1,   -1,  273,   -1,  275,   -1,  277,   -1,  279,  280,
			   -1,   -1,   -1,  284,  285,   -1,   -1,  288,   -1,    3,
			  291,    5,   -1,    7,  295,  296,   -1,  298,   -1,  300,
			   14,   -1,   -1,  304,   -1,   -1,   -1,   -1,  309,   -1,

			   24,   25,   26,   27,   28,   29,   30,   -1,   32,   -1,
			   -1,   -1,   -1,   37,   38,   -1,   40,   -1,   42,   -1,
			   -1,   45,   -1,   -1,   -1,   -1,   -1,   51,   -1,   53,
			   -1,   55,   -1,   -1,   58,   59,   -1,   61,   -1,   -1,
			   -1,   -1,   -1,   -1,   68,   69,   70,   -1,   72,   73,
			   -1,   -1,   76,   -1,   78,   79,   -1,   81,   82,   -1,
			   -1,   85,   -1,   87,   88,   -1,   -1,   91,   -1,   -1,
			   -1,   -1,   -1,   97,   98,   99,   -1,  101,   -1,   -1,
			   -1,   -1,   -1,   -1,  108,   -1,  110,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,

			   -1,  125,   -1,   -1,   -1,   -1,  130,  131,  132,   -1,
			  134,  135,   -1,  137,   -1,   -1,   -1,   -1,  142,  143,
			  144,  145,   -1,   -1,  148,   -1,   -1,   -1,   -1,   -1,
			  154,   -1,  156,   -1,   -1,   -1,   -1,  161,  162,  163,
			  164,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  172,   -1,
			   -1,  175,   -1,  177,  178,   -1,  180,  181,   -1,   -1,
			  184,  185,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  193,
			  194,   -1,   -1,  197,  198,   -1,   -1,   -1,  202,  203,
			   -1,   -1,  206,   -1,   -1,   -1,   -1,  211,   -1,   -1,
			   -1,   -1,   -1,  217,  218,  219,   -1,   -1,  222,   -1,

			  224,   -1,   -1,   -1,  228,   -1,   -1,   -1,   -1,  233,
			   -1,   -1,   -1,   -1,  238,   -1,   -1,   -1,  242,   -1,
			  244,  245,   -1,   -1,  248,  249,  250,  251,   -1,   -1,
			  254,   -1,  256,  257,  258,   -1,   -1,   -1,   -1,   -1,
			  264,   -1,  266,   -1,   -1,   -1,   -1,   -1,   -1,  273,
			   -1,  275,   -1,  277,   -1,  279,  280,   -1,   -1,   -1,
			  284,  285,   -1,   -1,  288,   -1,   -1,  291,   -1,   -1,
			   -1,  295,  296,   -1,  298,   -1,  300,   -1,   -1,    3,
			  304,    5,    6,   -1,    8,  309,   -1,   -1,   -1,   -1,
			   14,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   23,

			   24,   25,   26,   27,   28,   29,   30,   -1,   32,   -1,
			   -1,   -1,   -1,   37,   38,   -1,   40,   -1,   42,   -1,
			   -1,   45,   -1,   -1,   -1,   -1,   -1,   51,   -1,   53,
			   -1,   55,   -1,   -1,   58,   59,   -1,   61,   -1,   63,
			   -1,   -1,   -1,   -1,   68,   69,   70,   -1,   72,   73,
			   -1,   -1,   76,   -1,   -1,   79,   -1,   81,   82,   -1,
			   -1,   -1,   -1,   87,   88,   -1,   -1,   91,   -1,   -1,
			   -1,   -1,   -1,   97,   98,   99,   -1,  101,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,  110,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,  118,   -1,   -1,   -1,   -1,   -1,

			   -1,   -1,   -1,   -1,   -1,   -1,  130,   -1,  132,   -1,
			   -1,  135,   -1,  137,   -1,   -1,   -1,   -1,  142,  143,
			   -1,   -1,   -1,   -1,  148,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,  156,   -1,   -1,   -1,   -1,  161,  162,   -1,
			  164,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  172,   -1,
			   -1,  175,   -1,  177,  178,   -1,  180,  181,   -1,   -1,
			  184,  185,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  193,
			  194,   -1,   -1,  197,  198,   -1,   -1,   -1,  202,  203,
			   -1,   -1,  206,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,  217,  218,  219,   -1,   -1,  222,   -1,

			  224,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  233,
			   -1,   -1,   -1,   -1,  238,   -1,   -1,   -1,   -1,   -1,
			  244,   -1,   -1,   -1,  248,  249,  250,   -1,   -1,   -1,
			  254,   -1,   -1,  257,   -1,   -1,   -1,   -1,   -1,   -1,
			  264,   -1,  266,   -1,   -1,   -1,   -1,   -1,   -1,  273,
			   -1,  275,   -1,  277,   -1,  279,  280,   -1,   -1,   -1,
			  284,  285,   -1,   -1,  288,   -1,   -1,  291,   -1,   -1,
			    3,  295,    5,    6,  298,    8,  300,   -1,   -1,   -1,
			  304,   14,  306,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   23,   24,   25,   26,   27,   28,   29,   30,   -1,   32,

			   -1,   -1,   -1,   -1,   37,   38,   -1,   40,   -1,   42,
			   -1,   -1,   45,   -1,   -1,   -1,   -1,   -1,   51,   -1,
			   53,   -1,   55,   -1,   -1,   58,   59,   -1,   61,   -1,
			   63,   -1,   -1,   -1,   -1,   68,   69,   70,   -1,   72,
			   73,   -1,   -1,   76,   -1,   -1,   79,   -1,   81,   82,
			   -1,   -1,   -1,   -1,   87,   88,   -1,   -1,   91,   -1,
			   -1,   -1,   95,   -1,   97,   98,   99,   -1,  101,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,  110,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,  130,   -1,  132,

			   -1,   -1,  135,   -1,  137,   -1,   -1,   -1,   -1,  142,
			  143,   -1,   -1,   -1,   -1,  148,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,  156,   -1,   -1,   -1,   -1,  161,  162,
			   -1,  164,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  172,
			   -1,   -1,  175,   -1,  177,  178,   -1,  180,  181,   -1,
			   -1,  184,  185,   -1,   -1,   -1,   -1,  190,   -1,   -1,
			  193,  194,   -1,   -1,  197,  198,   -1,   -1,   -1,  202,
			  203,   -1,   -1,  206,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,  217,  218,  219,   -1,   -1,  222,
			   -1,  224,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,

			  233,   -1,   -1,   -1,   -1,  238,   -1,   -1,   -1,   -1,
			   -1,  244,   -1,   -1,   -1,  248,  249,  250,   -1,  252,
			   -1,  254,   -1,   -1,  257,   -1,   -1,   -1,   -1,   -1,
			   -1,  264,   -1,  266,   -1,   -1,   -1,   -1,   -1,   -1,
			  273,   -1,  275,   -1,  277,   -1,  279,  280,   -1,   -1,
			   -1,  284,  285,   -1,   -1,  288,   -1,   -1,  291,   -1,
			   -1,    3,  295,    5,    6,  298,    8,  300,   -1,   -1,
			   -1,  304,   14,   -1,   -1,   17,   -1,   -1,   -1,   -1,
			   -1,   23,   24,   25,   26,   27,   28,   29,   30,   -1,
			   32,   -1,   -1,   -1,   -1,   37,   38,   -1,   40,   -1, yyDummy>>,
			1, 1000, 12000)
		end

	yycheck_template_14 (an_array: ARRAY [INTEGER])
			-- Fill chunk #14 of template for `yycheck'.
		do
			yyarray_subcopy (an_array, <<
			   42,   -1,   -1,   45,   -1,   -1,   -1,   -1,   -1,   51,
			   -1,   53,   -1,   55,   -1,   -1,   58,   59,   -1,   61,
			   -1,   63,   -1,   -1,   -1,   -1,   68,   69,   70,   -1,
			   72,   73,   -1,   -1,   76,   -1,   -1,   79,   -1,   81,
			   82,   -1,   -1,   -1,   -1,   87,   88,   -1,   -1,   91,
			   -1,   -1,   -1,   -1,   96,   97,   98,   99,   -1,  101,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  110,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,  127,   -1,   -1,  130,   -1,
			  132,   -1,   -1,  135,   -1,  137,   -1,   -1,   -1,   -1,

			  142,  143,   -1,   -1,   -1,   -1,  148,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,  156,   -1,   -1,   -1,   -1,  161,
			  162,   -1,  164,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			  172,   -1,   -1,  175,   -1,  177,  178,   -1,  180,  181,
			   -1,   -1,  184,  185,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,  193,  194,   -1,   -1,  197,  198,   -1,   -1,   -1,
			  202,  203,   -1,   -1,  206,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,  217,  218,  219,   -1,   -1,
			  222,   -1,  224,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,  233,   -1,   -1,   -1,   -1,  238,   -1,   -1,   -1,

			   -1,   -1,  244,   -1,   -1,   -1,  248,  249,  250,   -1,
			   -1,   -1,  254,   -1,   -1,  257,   -1,   -1,   -1,   -1,
			   -1,   -1,  264,   -1,  266,   -1,   -1,   -1,   -1,   -1,
			   -1,  273,   -1,  275,   -1,  277,   -1,  279,  280,   -1,
			   -1,   -1,  284,  285,   -1,   -1,  288,   -1,   -1,  291,
			   -1,   -1,    3,  295,    5,    6,  298,    8,  300,   -1,
			   -1,   -1,  304,   14,   -1,   -1,   17,   -1,   -1,   -1,
			   -1,   -1,   23,   24,   25,   26,   27,   28,   29,   30,
			   -1,   32,   -1,   -1,   -1,   -1,   37,   38,   -1,   40,
			   -1,   42,   -1,   -1,   45,   -1,   -1,   -1,   -1,   -1,

			   51,   -1,   53,   -1,   55,   -1,   -1,   58,   59,   -1,
			   61,   -1,   63,   -1,   -1,   -1,   -1,   68,   69,   70,
			   -1,   72,   73,   -1,   -1,   76,   -1,   -1,   79,   -1,
			   81,   82,   -1,   -1,   -1,   -1,   87,   88,   -1,   -1,
			   91,   -1,   -1,   -1,   -1,   96,   97,   98,   99,   -1,
			  101,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  110,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,  127,   -1,   -1,  130,
			   -1,  132,   -1,   -1,  135,   -1,  137,   -1,   -1,   -1,
			   -1,  142,  143,   -1,   -1,   -1,   -1,  148,   -1,   -1,

			   -1,   -1,   -1,   -1,   -1,  156,   -1,   -1,   -1,   -1,
			  161,  162,   -1,  164,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,  172,   -1,   -1,  175,   -1,  177,  178,   -1,  180,
			  181,   -1,   -1,  184,  185,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,  193,  194,   -1,   -1,  197,  198,   -1,   -1,
			   -1,  202,  203,   -1,   -1,  206,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,  217,  218,  219,   -1,
			   -1,  222,   -1,  224,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,  233,   -1,   -1,   -1,   -1,  238,   -1,   -1,
			   -1,   -1,   -1,  244,   -1,   -1,   -1,  248,  249,  250,

			   -1,   -1,   -1,  254,   -1,   -1,  257,   -1,   -1,   -1,
			   -1,   -1,   -1,  264,   -1,  266,   -1,   -1,   -1,   -1,
			   -1,   -1,  273,   -1,  275,   -1,  277,   -1,  279,  280,
			   -1,   -1,   -1,  284,  285,   -1,   -1,  288,   -1,   -1,
			  291,   -1,   -1,    3,  295,    5,    6,  298,    8,  300,
			   -1,   -1,   -1,  304,   14,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   23,   24,   25,   26,   27,   28,   29,
			   30,   -1,   32,   -1,   -1,   -1,   -1,   37,   38,   -1,
			   40,   -1,   42,   43,   -1,   45,   46,   -1,   -1,   -1,
			   -1,   51,   -1,   53,   -1,   55,   -1,   -1,   58,   59,

			   -1,   61,   -1,   63,   -1,   -1,   -1,   -1,   68,   69,
			   70,   -1,   72,   73,   -1,   -1,   76,   -1,   -1,   79,
			   -1,   81,   82,   -1,   -1,   -1,   -1,   87,   88,   -1,
			   -1,   91,   -1,   -1,   -1,   -1,   -1,   97,   98,   99,
			   -1,  101,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			  110,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			  130,   -1,  132,   -1,   -1,  135,   -1,  137,   -1,   -1,
			   -1,   -1,  142,  143,   -1,   -1,   -1,   -1,  148,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,  156,   -1,   -1,   -1,

			   -1,  161,  162,   -1,  164,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,  172,   -1,   -1,  175,   -1,  177,  178,   -1,
			  180,  181,   -1,   -1,  184,  185,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,  193,  194,   -1,   -1,  197,  198,   -1,
			   -1,   -1,  202,  203,   -1,   -1,  206,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,  217,  218,  219,
			   -1,   -1,  222,   -1,  224,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,  233,   -1,   -1,   -1,   -1,  238,   -1,
			   -1,   -1,   -1,   -1,  244,   -1,   -1,   -1,  248,  249,
			  250,   -1,   -1,   -1,  254,   -1,   -1,  257,   -1,   -1,

			   -1,   -1,   -1,  263,  264,   -1,  266,   -1,   -1,   -1,
			   -1,   -1,   -1,  273,   -1,  275,   -1,  277,   -1,  279,
			  280,   -1,   -1,   -1,  284,  285,   -1,   -1,  288,   -1,
			   -1,  291,   -1,   -1,    3,  295,    5,    6,  298,    8,
			  300,   -1,   -1,   -1,  304,   14,   -1,   -1,   17,   -1,
			   -1,   -1,   -1,   -1,   23,   24,   25,   26,   27,   28,
			   29,   30,   -1,   32,   -1,   -1,   -1,   -1,   37,   38,
			   -1,   40,   -1,   42,   -1,   -1,   45,   -1,   -1,   -1,
			   -1,   -1,   51,   -1,   53,   -1,   55,   -1,   -1,   58,
			   59,   -1,   61,   -1,   63,   -1,   -1,   -1,   -1,   68,

			   69,   70,   -1,   72,   73,   -1,   -1,   76,   -1,   -1,
			   79,   -1,   81,   82,   -1,   -1,   -1,   -1,   87,   88,
			   -1,   -1,   91,   -1,   -1,   -1,   -1,   -1,   97,   98,
			   99,   -1,  101,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,  110,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  127,   -1,
			   -1,  130,   -1,  132,   -1,   -1,  135,   -1,  137,   -1,
			   -1,   -1,   -1,  142,  143,   -1,   -1,   -1,   -1,  148,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,  156,   -1,   -1,
			   -1,   -1,  161,  162,   -1,  164,   -1,   -1,   -1,   -1, yyDummy>>,
			1, 1000, 13000)
		end

	yycheck_template_15 (an_array: ARRAY [INTEGER])
			-- Fill chunk #15 of template for `yycheck'.
		do
			yyarray_subcopy (an_array, <<
			   -1,   -1,   -1,  172,   -1,   -1,  175,   -1,  177,  178,
			   -1,  180,  181,   -1,   -1,  184,  185,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,  193,  194,   -1,   -1,  197,  198,
			   -1,   -1,   -1,  202,  203,   -1,   -1,  206,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  217,  218,
			  219,   -1,   -1,  222,   -1,  224,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,  233,   -1,   -1,   -1,   -1,  238,
			   -1,   -1,   -1,   -1,   -1,  244,   -1,   -1,   -1,  248,
			  249,  250,   -1,   -1,   -1,  254,   -1,  256,  257,   -1,
			   -1,   -1,   -1,   -1,   -1,  264,   -1,  266,   -1,   -1,

			   -1,   -1,   -1,   -1,  273,   -1,  275,   -1,  277,   -1,
			  279,  280,   -1,   -1,   -1,  284,  285,   -1,   -1,  288,
			   -1,   -1,  291,   -1,   -1,    3,  295,    5,    6,  298,
			    8,  300,   -1,   -1,   -1,  304,   14,   -1,   -1,   17,
			   -1,   -1,   -1,   -1,   -1,   23,   24,   25,   26,   27,
			   28,   29,   30,   -1,   32,   -1,   -1,   -1,   -1,   37,
			   38,   -1,   40,   -1,   42,   -1,   -1,   45,   -1,   -1,
			   -1,   -1,   -1,   51,   -1,   53,   -1,   55,   -1,   -1,
			   58,   59,   -1,   61,   -1,   63,   -1,   -1,   -1,   -1,
			   68,   69,   70,   -1,   72,   73,   -1,   -1,   76,   -1,

			   -1,   79,   -1,   81,   82,   -1,   -1,   -1,   -1,   87,
			   88,   -1,   -1,   91,   -1,   -1,   -1,   -1,   -1,   97,
			   98,   99,   -1,  101,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,  110,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  127,
			   -1,   -1,  130,   -1,  132,   -1,   -1,  135,   -1,  137,
			   -1,   -1,   -1,   -1,  142,  143,   -1,   -1,   -1,   -1,
			  148,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  156,   -1,
			   -1,   -1,   -1,  161,  162,   -1,  164,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,  172,   -1,   -1,  175,   -1,  177,

			  178,   -1,  180,  181,   -1,   -1,  184,  185,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,  193,  194,   -1,   -1,  197,
			  198,   -1,   -1,   -1,  202,  203,   -1,   -1,  206,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  217,
			  218,  219,   -1,   -1,  222,   -1,  224,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,  233,   -1,   -1,   -1,   -1,
			  238,   -1,   -1,   -1,   -1,   -1,  244,   -1,   -1,   -1,
			  248,  249,  250,   -1,   -1,   -1,  254,   -1,   -1,  257,
			   -1,   -1,   -1,   -1,   -1,   -1,  264,   -1,  266,   -1,
			   -1,   -1,   -1,   -1,   -1,  273,   -1,  275,   -1,  277,

			   -1,  279,  280,   -1,   -1,   -1,  284,  285,   -1,   -1,
			  288,   -1,   -1,  291,   -1,   -1,    3,  295,    5,    6,
			  298,    8,  300,   -1,   -1,   -1,  304,   14,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   23,   24,   25,   26,
			   27,   28,   29,   30,   -1,   32,   -1,   -1,   -1,   -1,
			   37,   38,   -1,   40,   -1,   42,   43,   -1,   45,   -1,
			   -1,   -1,   -1,   -1,   51,   -1,   53,   -1,   55,   -1,
			   -1,   58,   59,   -1,   61,   -1,   63,   -1,   -1,   -1,
			   -1,   68,   69,   70,   -1,   72,   73,   -1,   -1,   76,
			   -1,   -1,   79,   -1,   81,   82,   -1,   -1,   -1,   -1,

			   87,   88,   -1,   -1,   91,   -1,   -1,   -1,   -1,   -1,
			   97,   98,   99,   -1,  101,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,  110,   -1,  112,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,  130,   -1,  132,   -1,   -1,  135,   -1,
			  137,   -1,   -1,   -1,   -1,  142,  143,   -1,   -1,   -1,
			   -1,  148,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  156,
			   -1,   -1,   -1,   -1,  161,  162,   -1,  164,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,  172,   -1,   -1,  175,   -1,
			  177,  178,   -1,  180,  181,   -1,   -1,  184,  185,   -1,

			   -1,   -1,   -1,   -1,   -1,   -1,  193,  194,   -1,   -1,
			  197,  198,   -1,   -1,   -1,  202,  203,   -1,   -1,  206,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			  217,  218,  219,   -1,   -1,  222,   -1,  224,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,  233,   -1,   -1,   -1,
			   -1,  238,   -1,   -1,   -1,   -1,   -1,  244,   -1,   -1,
			   -1,  248,  249,  250,   -1,   -1,   -1,  254,   -1,   -1,
			  257,   -1,   -1,   -1,   -1,   -1,   -1,  264,   -1,  266,
			   -1,   -1,   -1,   -1,   -1,   -1,  273,   -1,  275,   -1,
			  277,   -1,  279,  280,   -1,   -1,   -1,  284,  285,   -1,

			   -1,  288,   -1,   -1,  291,   -1,   -1,    3,  295,    5,
			    6,  298,    8,  300,   -1,   -1,   -1,  304,   14,   -1,
			   -1,   17,   -1,   -1,   -1,   -1,   -1,   23,   24,   25,
			   26,   27,   28,   29,   30,   -1,   32,   -1,   -1,   -1,
			   -1,   37,   38,   -1,   40,   -1,   42,   -1,   -1,   45,
			   -1,   -1,   -1,   -1,   -1,   51,   -1,   53,   -1,   55,
			   -1,   -1,   58,   59,   -1,   61,   -1,   63,   -1,   -1,
			   -1,   -1,   68,   69,   70,   -1,   72,   73,   -1,   -1,
			   76,   -1,   -1,   79,   -1,   81,   82,   -1,   -1,   -1,
			   -1,   87,   88,   -1,   -1,   91,   -1,   -1,   -1,   -1,

			   -1,   97,   98,   99,   -1,  101,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,  110,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,  127,   -1,   -1,  130,   -1,  132,   -1,   -1,  135,
			   -1,  137,   -1,   -1,   -1,   -1,  142,  143,   -1,   -1,
			   -1,   -1,  148,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			  156,   -1,   -1,   -1,   -1,  161,  162,   -1,  164,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,  172,   -1,   -1,  175,
			   -1,  177,  178,   -1,  180,  181,   -1,   -1,  184,  185,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,  193,  194,   -1,

			   -1,  197,  198,   -1,   -1,   -1,  202,  203,   -1,   -1,
			  206,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,  217,  218,  219,   -1,   -1,  222,   -1,  224,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,  233,   -1,   -1,
			   -1,   -1,  238,   -1,   -1,   -1,   -1,   -1,  244,   -1,
			   -1,   -1,  248,  249,  250,   -1,   -1,   -1,  254,   -1,
			   -1,  257,   -1,   -1,   -1,   -1,   -1,   -1,  264,   -1,
			  266,   -1,   -1,   -1,   -1,   -1,   -1,  273,   -1,  275,
			   -1,  277,   -1,  279,  280,   -1,   -1,   -1,  284,  285,
			   -1,   -1,  288,   -1,   -1,  291,   -1,   -1,    3,  295, yyDummy>>,
			1, 1000, 14000)
		end

	yycheck_template_16 (an_array: ARRAY [INTEGER])
			-- Fill chunk #16 of template for `yycheck'.
		do
			yyarray_subcopy (an_array, <<
			    5,    6,  298,    8,  300,   -1,   -1,   -1,  304,   14,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   23,   24,
			   25,   26,   27,   28,   29,   30,   -1,   32,   -1,   -1,
			   -1,   -1,   37,   38,   -1,   40,   -1,   42,   -1,   -1,
			   45,   -1,   -1,   -1,   -1,   -1,   51,   -1,   53,   -1,
			   55,   -1,   -1,   58,   59,   -1,   61,   -1,   63,   -1,
			   -1,   -1,   -1,   68,   69,   70,   -1,   72,   73,   -1,
			   -1,   76,   -1,   -1,   79,   -1,   81,   82,   -1,   -1,
			   -1,   -1,   87,   88,   -1,   -1,   91,   -1,   -1,   -1,
			   -1,   -1,   97,   98,   99,   -1,  101,   -1,   -1,   -1,

			   -1,   -1,   -1,   -1,   -1,  110,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,  130,   -1,  132,   -1,   -1,
			  135,   -1,  137,   -1,   -1,   -1,   -1,  142,  143,   -1,
			   -1,   -1,   -1,  148,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,  156,   -1,   -1,   -1,   -1,  161,  162,   -1,  164,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,  172,   -1,   -1,
			  175,   -1,  177,  178,   -1,  180,  181,   -1,   -1,  184,
			  185,   -1,   -1,   -1,   -1,  190,   -1,   -1,  193,  194,
			   -1,   -1,  197,  198,   -1,   -1,   -1,  202,  203,   -1,

			   -1,  206,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,  217,  218,  219,   -1,   -1,  222,   -1,  224,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  233,   -1,
			   -1,   -1,   -1,  238,   -1,   -1,   -1,   -1,   -1,  244,
			   -1,   -1,   -1,  248,  249,  250,   -1,   -1,   -1,  254,
			   -1,   -1,  257,   -1,   -1,   -1,   -1,   -1,   -1,  264,
			   -1,  266,   -1,   -1,   -1,   -1,   -1,   -1,  273,   -1,
			  275,   -1,  277,   -1,  279,  280,   -1,   -1,   -1,  284,
			  285,   -1,   -1,  288,   -1,   -1,  291,   -1,   -1,    3,
			  295,    5,    6,  298,    8,  300,   -1,   -1,   -1,  304,

			   14,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   23,
			   24,   25,   26,   27,   28,   29,   30,   -1,   32,   -1,
			   -1,   -1,   -1,   37,   38,   -1,   40,   -1,   42,   -1,
			   -1,   45,   -1,   -1,   -1,   -1,   -1,   51,   -1,   53,
			   -1,   55,   -1,   -1,   58,   59,   -1,   61,   -1,   63,
			   -1,   -1,   -1,   -1,   68,   69,   70,   -1,   72,   73,
			   -1,   -1,   76,   -1,   -1,   79,   -1,   81,   82,   -1,
			   -1,   -1,   -1,   87,   88,   -1,   -1,   91,   -1,   -1,
			   -1,   -1,   -1,   97,   98,   99,   -1,  101,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,  110,   -1,   -1,   -1,

			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,  130,   -1,  132,   -1,
			   -1,  135,   -1,  137,   -1,   -1,   -1,   -1,  142,  143,
			   -1,   -1,   -1,   -1,  148,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,  156,   -1,   -1,   -1,   -1,  161,  162,   -1,
			  164,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  172,   -1,
			   -1,  175,   -1,  177,  178,   -1,  180,  181,   -1,   -1,
			  184,  185,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  193,
			  194,   -1,   -1,  197,  198,   -1,   -1,   -1,  202,  203,
			   -1,   -1,  206,   -1,   -1,   -1,   -1,   -1,   -1,   -1,

			   -1,   -1,   -1,  217,  218,  219,   -1,   -1,  222,   -1,
			  224,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  233,
			   -1,   -1,   -1,   -1,  238,   -1,   -1,   -1,   -1,   -1,
			  244,   -1,   -1,   -1,  248,  249,  250,   -1,   -1,   -1,
			  254,   -1,  256,  257,   -1,   -1,   -1,   -1,   -1,   -1,
			  264,   -1,  266,   -1,   -1,   -1,   -1,   -1,   -1,  273,
			   -1,  275,   -1,  277,   -1,  279,  280,   -1,   -1,   -1,
			  284,  285,   -1,   -1,  288,   -1,   -1,  291,   -1,   -1,
			    3,  295,    5,    6,  298,    8,  300,   -1,   -1,   -1,
			  304,   14,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,

			   23,   24,   25,   26,   27,   28,   29,   30,   -1,   32,
			   -1,   -1,   -1,   -1,   37,   38,   -1,   40,   -1,   42,
			   -1,   -1,   45,   -1,   -1,   -1,   -1,   -1,   51,   -1,
			   53,   -1,   55,   -1,   -1,   58,   59,   -1,   61,   -1,
			   63,   -1,   -1,   -1,   -1,   68,   69,   70,   -1,   72,
			   73,   -1,   -1,   76,   -1,   -1,   79,   -1,   81,   82,
			   -1,   -1,   -1,   -1,   87,   88,   -1,   -1,   91,   -1,
			   -1,   -1,   -1,   -1,   97,   98,   99,   -1,  101,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,  110,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,

			   -1,   -1,   -1,   -1,   -1,   -1,   -1,  130,   -1,  132,
			   -1,   -1,  135,   -1,  137,   -1,   -1,   -1,   -1,  142,
			  143,   -1,   -1,   -1,   -1,  148,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,  156,   -1,   -1,   -1,   -1,  161,  162,
			   -1,  164,   -1,   -1,   -1,   -1,   -1,  170,   -1,  172,
			   -1,   -1,  175,   -1,  177,  178,   -1,  180,  181,   -1,
			   -1,  184,  185,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			  193,  194,   -1,   -1,  197,  198,   -1,   -1,   -1,  202,
			  203,   -1,   -1,  206,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,  217,  218,  219,   -1,   -1,  222,

			   -1,  224,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			  233,   -1,   -1,   -1,   -1,  238,   -1,   -1,   -1,   -1,
			   -1,  244,   -1,   -1,   -1,  248,  249,  250,   -1,   -1,
			   -1,  254,   -1,   -1,  257,   -1,   -1,   -1,   -1,   -1,
			   -1,  264,   -1,  266,   -1,   -1,   -1,   -1,   -1,   -1,
			  273,   -1,  275,   -1,  277,   -1,  279,  280,   -1,   -1,
			   -1,  284,  285,   -1,   -1,  288,   -1,   -1,  291,   -1,
			   -1,    3,  295,    5,    6,  298,    8,  300,   -1,   11,
			   -1,  304,   14,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   23,   24,   25,   26,   27,   28,   29,   30,   -1,

			   32,   -1,   -1,   -1,   -1,   37,   38,   -1,   40,   -1,
			   42,   -1,   -1,   45,   -1,   -1,   -1,   -1,   -1,   51,
			   -1,   53,   -1,   55,   -1,   -1,   58,   59,   -1,   61,
			   -1,   63,   -1,   -1,   -1,   -1,   68,   69,   70,   -1,
			   72,   73,   -1,   -1,   76,   -1,   -1,   79,   -1,   81,
			   82,   -1,   -1,   -1,   -1,   87,   88,   -1,   -1,   91,
			   -1,   -1,   -1,   -1,   -1,   97,   98,   99,   -1,  101,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  110,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  130,   -1, yyDummy>>,
			1, 1000, 15000)
		end

	yycheck_template_17 (an_array: ARRAY [INTEGER])
			-- Fill chunk #17 of template for `yycheck'.
		do
			yyarray_subcopy (an_array, <<
			  132,   -1,   -1,  135,   -1,  137,   -1,   -1,   -1,   -1,
			  142,  143,   -1,   -1,   -1,   -1,  148,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,  156,   -1,   -1,   -1,   -1,  161,
			  162,   -1,  164,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			  172,   -1,   -1,  175,   -1,  177,  178,   -1,  180,  181,
			   -1,   -1,  184,  185,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,  193,  194,   -1,   -1,  197,  198,   -1,   -1,   -1,
			  202,  203,   -1,   -1,  206,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,  217,  218,  219,   -1,   -1,
			  222,   -1,  224,   -1,   -1,   -1,   -1,   -1,   -1,   -1,

			   -1,  233,   -1,   -1,   -1,   -1,  238,   -1,   -1,   -1,
			   -1,   -1,  244,   -1,   -1,   -1,  248,  249,  250,   -1,
			   -1,   -1,  254,   -1,   -1,  257,   -1,   -1,   -1,   -1,
			   -1,   -1,  264,   -1,  266,   -1,   -1,   -1,   -1,   -1,
			   -1,  273,   -1,  275,   -1,  277,   -1,  279,  280,   -1,
			   -1,   -1,  284,  285,   -1,   -1,  288,   -1,   -1,  291,
			   -1,   -1,    3,  295,    5,    6,  298,    8,  300,   -1,
			   -1,   -1,  304,   14,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   23,   24,   25,   26,   27,   28,   29,   30,
			   -1,   32,   -1,   -1,   -1,   -1,   37,   38,   -1,   40,

			   -1,   42,   -1,   -1,   45,   -1,   -1,   -1,   -1,   -1,
			   51,   -1,   53,   -1,   55,   -1,   -1,   58,   59,   -1,
			   61,   -1,   63,   -1,   -1,   -1,   -1,   68,   69,   70,
			   -1,   72,   73,   -1,   -1,   76,   -1,   -1,   79,   -1,
			   81,   82,   -1,   -1,   -1,   -1,   87,   88,   -1,   -1,
			   91,   -1,   -1,   -1,   -1,   -1,   97,   98,   99,   -1,
			  101,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  110,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  130,
			   -1,  132,   -1,   -1,  135,   -1,  137,   -1,   -1,   -1,

			   -1,  142,  143,   -1,   -1,   -1,   -1,  148,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,  156,   -1,   -1,   -1,   -1,
			  161,  162,   -1,  164,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,  172,   -1,   -1,  175,   -1,  177,  178,   -1,  180,
			  181,   -1,   -1,  184,  185,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,  193,  194,   -1,   -1,  197,  198,   -1,   -1,
			   -1,  202,  203,   -1,   -1,  206,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,  217,  218,  219,   -1,
			   -1,  222,   -1,  224,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,  233,   -1,   -1,   -1,   -1,  238,   -1,   -1,

			   -1,   -1,   -1,  244,   -1,   -1,   -1,  248,  249,  250,
			   -1,   -1,   -1,  254,   -1,   -1,  257,   -1,   -1,   -1,
			   -1,   -1,   -1,  264,   -1,  266,   -1,   -1,   -1,   -1,
			   -1,   -1,  273,   -1,  275,   -1,  277,   -1,  279,  280,
			   -1,   -1,   -1,  284,  285,   -1,   -1,  288,   -1,   -1,
			  291,   -1,   -1,    3,  295,    5,    6,  298,    8,  300,
			   -1,   -1,   -1,  304,   14,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   23,   24,   25,   26,   27,   28,   29,
			   30,   -1,   32,   -1,   -1,   -1,   -1,   37,   38,   -1,
			   40,   -1,   42,   -1,   -1,   45,   -1,   -1,   -1,   -1,

			   -1,   51,   -1,   53,   -1,   55,   -1,   -1,   58,   59,
			   -1,   61,   -1,   63,   -1,   -1,   -1,   -1,   68,   69,
			   70,   -1,   72,   73,   -1,   -1,   76,   -1,   -1,   79,
			   -1,   81,   82,   -1,   -1,   -1,   -1,   87,   88,   -1,
			   -1,   91,   -1,   -1,   -1,   -1,   -1,   97,   98,   99,
			   -1,  101,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			  110,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			  130,   -1,  132,   -1,   -1,  135,   -1,  137,   -1,   -1,
			   -1,   -1,  142,  143,   -1,   -1,   -1,   -1,  148,   -1,

			   -1,   -1,   -1,   -1,   -1,   -1,  156,   -1,   -1,   -1,
			   -1,  161,  162,   -1,  164,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,  172,   -1,   -1,  175,   -1,  177,  178,   -1,
			  180,  181,   -1,   -1,  184,  185,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,  193,  194,   -1,   -1,  197,  198,   -1,
			   -1,   -1,  202,  203,   -1,   -1,  206,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,  217,  218,  219,
			   -1,   -1,  222,   -1,  224,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,  233,   -1,   -1,   -1,   -1,  238,   -1,
			   -1,   -1,   -1,   -1,  244,   -1,   -1,   -1,  248,  249,

			  250,   -1,   -1,   -1,  254,   -1,   -1,  257,   -1,   -1,
			   -1,   -1,   -1,   -1,  264,   -1,  266,   -1,   -1,   -1,
			   -1,   -1,   -1,  273,   -1,  275,   -1,  277,   -1,  279,
			  280,   -1,   -1,   -1,  284,  285,   -1,   -1,  288,   -1,
			   -1,  291,   -1,   -1,    3,  295,    5,    6,  298,    8,
			  300,   -1,   -1,   -1,  304,   14,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   23,   24,   25,   26,   27,   28,
			   29,   30,   -1,   32,   -1,   -1,   -1,   -1,   37,   38,
			   -1,   40,   -1,   42,   -1,   -1,   45,   -1,   -1,   -1,
			   -1,   -1,   51,   -1,   53,   -1,   55,   -1,   -1,   58,

			   59,   -1,   61,   -1,   63,   -1,   -1,   -1,   -1,   68,
			   69,   70,   -1,   72,   73,   -1,   -1,   76,   -1,   -1,
			   79,   -1,   81,   82,   -1,   -1,   -1,   -1,   87,   88,
			   -1,   -1,   91,   -1,   -1,   -1,   -1,   -1,   97,   98,
			   99,   -1,  101,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,  110,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,  130,   -1,  132,   -1,   -1,  135,   -1,  137,   -1,
			   -1,   -1,   -1,  142,  143,   -1,   -1,   -1,   -1,  148,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,  156,   -1,   -1,

			   -1,   -1,  161,  162,   -1,  164,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,  172,   -1,   -1,  175,   -1,  177,  178,
			   -1,  180,  181,   -1,   -1,  184,  185,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,  193,  194,   -1,   -1,  197,  198,
			   -1,   -1,   -1,  202,  203,   -1,   -1,  206,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  217,  218,
			  219,   -1,   -1,  222,   -1,  224,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,  233,   -1,   -1,   -1,   -1,  238,
			   -1,   -1,   -1,   -1,   -1,  244,   -1,   -1,   -1,  248,
			  249,  250,   -1,   -1,   -1,  254,   -1,   -1,  257,   -1, yyDummy>>,
			1, 1000, 16000)
		end

	yycheck_template_18 (an_array: ARRAY [INTEGER])
			-- Fill chunk #18 of template for `yycheck'.
		do
			yyarray_subcopy (an_array, <<
			   -1,   -1,   -1,   -1,   -1,  264,   -1,  266,   -1,   -1,
			   -1,   -1,   -1,   -1,  273,   -1,  275,   -1,  277,   -1,
			  279,  280,   -1,   -1,   -1,  284,  285,   -1,   -1,  288,
			   -1,    3,  291,    5,   -1,   -1,  295,   -1,   -1,  298,
			   -1,  300,   14,   -1,   -1,  304,   -1,   -1,   -1,   -1,
			   -1,   -1,   24,   25,   26,   27,   28,   29,   30,   -1,
			   32,   -1,   -1,   -1,   -1,   37,   38,   -1,   40,   -1,
			   42,   -1,   -1,   45,   -1,   -1,   -1,   -1,   -1,   51,
			   -1,   53,   -1,   55,   -1,   -1,   -1,   59,   -1,   61,
			   -1,   -1,   -1,   -1,   -1,   -1,   68,   69,   70,   -1,

			   72,   73,   -1,   -1,   76,   -1,   -1,   79,   -1,   81,
			   82,   -1,   -1,   -1,   -1,   87,   88,   -1,   -1,   91,
			   -1,   -1,   -1,   -1,   -1,   97,   98,   99,   -1,  101,
			   -1,   -1,   -1,   -1,  106,   -1,   -1,   -1,  110,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  130,   -1,
			  132,   -1,   -1,  135,   -1,  137,   -1,   -1,   -1,   -1,
			  142,  143,   -1,   -1,   -1,   -1,  148,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,  156,   -1,   -1,   -1,   -1,  161,
			  162,   -1,  164,   -1,   -1,   -1,   -1,   -1,   -1,   -1,

			  172,   -1,   -1,  175,   -1,  177,  178,   -1,  180,  181,
			   -1,   -1,  184,  185,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,  193,  194,   -1,   -1,  197,  198,   -1,   -1,   -1,
			  202,  203,   -1,   -1,  206,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,  217,  218,  219,   -1,   -1,
			  222,   -1,  224,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,  233,   -1,   -1,   -1,   -1,  238,   -1,   -1,   -1,
			   -1,   -1,  244,   -1,   -1,   -1,  248,  249,  250,   -1,
			   -1,   -1,  254,   -1,   -1,  257,   -1,   -1,   -1,   -1,
			   -1,   -1,  264,   -1,  266,   -1,   -1,   -1,   -1,   -1,

			   -1,  273,   -1,  275,   -1,  277,   -1,  279,  280,   -1,
			   -1,   -1,  284,  285,   -1,   -1,  288,   -1,    3,  291,
			    5,   -1,   -1,  295,   -1,   -1,  298,   -1,  300,   14,
			   -1,   -1,  304,   -1,   -1,   -1,   -1,   -1,   -1,   24,
			   25,   26,   27,   28,   29,   30,   -1,   32,   -1,   -1,
			   -1,   -1,   37,   38,   -1,   40,   -1,   42,   -1,   -1,
			   45,   -1,   -1,   -1,   -1,   -1,   51,   -1,   53,   -1,
			   55,   -1,   -1,   -1,   59,   -1,   61,   -1,   -1,   -1,
			   -1,   -1,   -1,   68,   69,   70,   -1,   72,   73,   -1,
			   -1,   76,   -1,   -1,   79,   -1,   81,   82,   -1,   -1,

			   -1,   -1,   87,   88,   -1,   -1,   91,   -1,   -1,   -1,
			   -1,   -1,   97,   98,   99,   -1,  101,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,  110,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,  130,   -1,  132,   -1,   -1,
			  135,   -1,  137,   -1,   -1,   -1,   -1,  142,  143,   -1,
			   -1,   -1,   -1,  148,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,  156,   -1,   -1,   -1,   -1,  161,  162,   -1,  164,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,  172,   -1,   -1,
			  175,   -1,  177,  178,   -1,  180,  181,   -1,   -1,  184,

			  185,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  193,  194,
			   -1,   -1,  197,  198,   -1,   -1,   -1,  202,  203,   -1,
			   -1,  206,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,  217,  218,  219,   -1,   -1,  222,   -1,  224,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  233,   -1,
			   -1,   -1,   -1,  238,   -1,   -1,   -1,   -1,   -1,  244,
			   -1,   -1,   -1,  248,  249,  250,   -1,   -1,   -1,  254,
			   -1,   -1,  257,   -1,   -1,   -1,   -1,   -1,   -1,  264,
			   -1,  266,   -1,   -1,   -1,   -1,   -1,   -1,  273,   -1,
			  275,   -1,  277,   -1,  279,  280,   -1,   -1,   -1,  284,

			  285,   -1,   -1,  288,   -1,    3,  291,    5,   -1,   -1,
			  295,   -1,   -1,  298,   -1,  300,   14,   -1,   -1,  304,
			   -1,   -1,   -1,   -1,   -1,   -1,   24,   25,   26,   27,
			   28,   29,   30,   -1,   32,   -1,   -1,   -1,   -1,   37,
			   38,   -1,   40,   -1,   42,   -1,   -1,   45,   -1,   -1,
			   -1,   -1,   -1,   51,   -1,   53,   -1,   55,   -1,   -1,
			   -1,   59,   -1,   61,   -1,   -1,   -1,   -1,   -1,   -1,
			   68,   69,   70,   -1,   72,   73,   -1,   -1,   76,   -1,
			   -1,   79,   -1,   81,   82,   -1,   -1,   -1,   -1,   87,
			   88,   -1,   -1,   91,   -1,   -1,   -1,   -1,   -1,   97,

			   98,   99,   -1,  101,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,  110,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,  130,   -1,  132,   -1,   -1,  135,   -1,  137,
			   -1,   -1,   -1,   -1,  142,  143,   -1,   -1,   -1,   -1,
			  148,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  156,   -1,
			   -1,   -1,   -1,  161,  162,   -1,  164,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,  172,   -1,   -1,  175,   -1,  177,
			  178,   -1,  180,  181,   -1,   -1,  184,  185,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,  193,  194,   -1,   -1,  197,

			  198,   -1,   -1,   -1,  202,  203,   -1,   -1,  206,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  217,
			  218,  219,   -1,   -1,  222,   -1,  224,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,  233,   -1,   -1,   -1,   -1,
			  238,   -1,   -1,   -1,   -1,   -1,  244,   -1,   -1,   -1,
			  248,  249,  250,   -1,   -1,   -1,  254,   -1,   -1,  257,
			   -1,   -1,   -1,   -1,   -1,   -1,  264,   -1,  266,   -1,
			   -1,   -1,   -1,   -1,   -1,  273,   -1,  275,   -1,  277,
			   -1,  279,  280,   -1,   -1,   -1,  284,  285,   -1,   -1,
			  288,   -1,    3,  291,    5,   -1,   -1,  295,   -1,   -1,

			  298,   -1,  300,   14,   -1,   -1,  304,   -1,   -1,   -1,
			   -1,   -1,   -1,   24,   25,   26,   27,   28,   29,   30,
			   -1,   32,   -1,   -1,   -1,   -1,   37,   38,   -1,   40,
			   -1,   42,   -1,   -1,   45,   -1,   -1,   -1,   -1,   -1,
			   51,   -1,   53,   -1,   55,   -1,   -1,   -1,   59,   -1,
			   61,   -1,   -1,   -1,   -1,   -1,   -1,   68,   69,   70,
			   -1,   72,   73,   -1,   -1,   76,   -1,   -1,   79,   -1,
			   81,   82,   -1,   -1,   -1,   -1,   87,   88,   -1,   -1,
			   91,   -1,   -1,   -1,   -1,   -1,   97,   98,   99,   -1,
			  101,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  110, yyDummy>>,
			1, 1000, 17000)
		end

	yycheck_template_19 (an_array: ARRAY [INTEGER])
			-- Fill chunk #19 of template for `yycheck'.
		do
			yyarray_subcopy (an_array, <<
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  130,
			   -1,  132,   -1,   -1,  135,   -1,  137,   -1,   -1,   -1,
			   -1,  142,  143,   -1,   -1,   -1,   -1,  148,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,  156,   -1,   -1,   -1,   -1,
			  161,  162,   -1,  164,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,  172,   -1,   -1,  175,   -1,  177,  178,   -1,  180,
			  181,   -1,   -1,  184,  185,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,  193,  194,   -1,   -1,  197,  198,   -1,   -1,
			   -1,  202,  203,   -1,   -1,  206,   -1,   -1,   -1,   -1,

			   -1,   -1,   -1,   -1,   -1,   -1,  217,  218,  219,   -1,
			   -1,  222,   -1,  224,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,  233,   -1,   -1,   -1,   -1,  238,   -1,   -1,
			   -1,   -1,   -1,  244,   -1,   -1,   -1,  248,  249,  250,
			   -1,   -1,   -1,  254,   -1,   -1,  257,   -1,   -1,   -1,
			   -1,   -1,   -1,  264,   -1,  266,   -1,   -1,   -1,   -1,
			   -1,   -1,  273,   -1,  275,   -1,  277,   -1,  279,  280,
			   -1,   -1,   -1,  284,  285,   -1,   -1,  288,   -1,   -1,
			  291,   -1,   -1,   -1,  295,   -1,   -1,  298,   -1,  300,
			   -1,   -1,   -1,  304,    3,    4,    5,    6,    7,    8,

			    9,   10,   -1,   12,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   21,   22,   -1,   24,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   35,   36,   -1,   38,
			   -1,   40,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   49,   -1,   51,   -1,   53,   -1,   55,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   68,
			   -1,   70,   -1,   -1,   -1,   -1,   -1,   76,   -1,   -1,
			   79,   -1,   -1,   82,   -1,   -1,   -1,   -1,   87,   88,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,  101,   -1,   -1,   -1,   -1,   -1,   -1,   -1,

			   -1,  110,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,  132,   -1,   -1,  135,   -1,  137,   -1,
			   -1,   -1,   -1,  142,  143,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,  156,   -1,   -1,
			   -1,   -1,  161,  162,   -1,  164,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,  172,   -1,   -1,  175,   -1,   -1,  178,
			   -1,  180,   -1,   -1,   -1,  184,  185,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,  193,  194,   -1,   -1,   -1,  198,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,  206,   -1,   -1,

			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  217,  218,
			  219,   -1,   -1,  222,   -1,  224,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,  233,    3,   -1,   -1,   -1,  238,
			   -1,   -1,   -1,   -1,   -1,  244,   -1,   -1,   -1,  248,
			   -1,  250,   -1,   -1,   -1,  254,   24,   -1,  257,   -1,
			   -1,   -1,   -1,   -1,   -1,  264,   -1,  266,   -1,   -1,
			   38,   -1,   40,   -1,  273,   -1,  275,   -1,  277,   -1,
			  279,  280,   -1,   51,   -1,   53,   -1,   55,   -1,  288,
			   -1,   -1,  291,   -1,   -1,   -1,  295,   -1,   -1,   -1,
			   68,  300,   70,   -1,   -1,  304,   -1,   -1,   76,   -1,

			   -1,   79,   -1,   -1,   82,   -1,   -1,   -1,   -1,   87,
			   88,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,  101,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,  110,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,  132,   -1,   -1,  135,   -1,  137,
			   -1,   -1,   -1,   -1,  142,  143,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  156,   -1,
			   -1,   -1,   -1,  161,  162,   -1,  164,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,  172,   -1,   -1,  175,   -1,  177,

			  178,   -1,  180,  181,   -1,   -1,  184,  185,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,  193,  194,   -1,   -1,   -1,
			  198,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  206,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  217,
			  218,  219,   -1,   -1,  222,   -1,  224,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,  233,    3,   -1,   -1,   -1,
			  238,   -1,   -1,   -1,   -1,   -1,  244,   -1,   -1,   -1,
			  248,  249,  250,   -1,   -1,   -1,  254,   24,   -1,  257,
			   -1,   -1,   -1,   -1,   -1,   -1,  264,   -1,  266,   -1,
			   -1,   38,   -1,   40,   -1,  273,   -1,  275,   -1,  277,

			   -1,  279,  280,   -1,   51,   -1,   53,   -1,   55,   -1,
			  288,   -1,   -1,  291,   -1,   62,   -1,  295,   -1,   -1,
			   -1,   68,  300,   70,   -1,   -1,  304,   -1,   -1,   76,
			   -1,   -1,   79,   -1,   -1,   82,   -1,   -1,   -1,   -1,
			   87,   88,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,  101,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,  110,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,  132,   -1,   -1,  135,   -1,
			  137,   -1,   -1,   -1,   -1,  142,  143,   -1,   -1,   -1,

			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  156,
			   -1,   -1,   -1,   -1,  161,  162,   -1,  164,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,  172,   -1,   -1,  175,   -1,
			   -1,  178,   -1,  180,   -1,   -1,   -1,  184,  185,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,  193,  194,   -1,   -1,
			   -1,  198,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  206,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			  217,  218,  219,   -1,   -1,  222,   -1,  224,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,  233,    3,   -1,   -1,
			   -1,  238,   -1,   -1,   -1,   -1,   -1,  244,   -1,   -1,

			   -1,  248,   -1,  250,   -1,   -1,   -1,  254,   24,  256,
			  257,   -1,   -1,   -1,   -1,   -1,   -1,  264,   -1,  266,
			   -1,   -1,   38,   -1,   40,   -1,  273,   -1,  275,   -1,
			  277,   -1,  279,  280,   -1,   51,   -1,   53,   -1,   55,
			   -1,  288,   -1,   -1,  291,   -1,   62,   -1,  295,   -1,
			   -1,   -1,   68,  300,   70,   -1,   -1,  304,   -1,   -1,
			   76,   -1,   -1,   79,   -1,   -1,   82,   -1,   -1,   -1,
			   -1,   87,   88,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,  101,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,  110,   -1,   -1,   -1,   -1,   -1, yyDummy>>,
			1, 1000, 18000)
		end

	yycheck_template_20 (an_array: ARRAY [INTEGER])
			-- Fill chunk #20 of template for `yycheck'.
		do
			yyarray_subcopy (an_array, <<
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,  132,   -1,   -1,  135,
			   -1,  137,   -1,   -1,   -1,   -1,  142,  143,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			  156,   -1,   -1,   -1,   -1,  161,  162,   -1,  164,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,  172,   -1,   -1,  175,
			   -1,   -1,  178,   -1,  180,   -1,   -1,   -1,  184,  185,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,  193,  194,   -1,
			   -1,   -1,  198,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			  206,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,

			   -1,  217,  218,  219,   -1,   -1,  222,   -1,  224,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,  233,    3,   -1,
			   -1,   -1,  238,   -1,   -1,   -1,   -1,   -1,  244,   -1,
			   -1,   -1,  248,   -1,  250,   -1,   -1,   -1,  254,   24,
			   -1,  257,   -1,   -1,   -1,   -1,   -1,   -1,  264,   -1,
			  266,   -1,   -1,   38,   -1,   40,   -1,  273,   -1,  275,
			   -1,  277,   -1,  279,  280,   -1,   51,   -1,   53,   -1,
			   55,   -1,  288,   -1,   -1,  291,   -1,   -1,   -1,  295,
			   -1,   -1,   -1,   68,  300,   70,   -1,   -1,  304,   -1,
			   -1,   76,   -1,   -1,   79,   -1,   -1,   82,   -1,   -1,

			   -1,   -1,   87,   88,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,  101,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,  110,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,  132,   -1,   -1,
			  135,   -1,  137,   -1,   -1,   -1,   -1,  142,  143,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,  156,   -1,   -1,   -1,   -1,  161,  162,   -1,  164,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,  172,   -1,   -1,
			  175,   -1,   -1,  178,   -1,  180,   -1,   -1,   -1,  184,

			  185,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  193,  194,
			   -1,   -1,   -1,  198,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,  206,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,  217,  218,  219,   -1,   -1,  222,   -1,  224,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  233,    3,
			   -1,   -1,   -1,  238,   -1,   -1,   -1,   -1,   -1,  244,
			   -1,   -1,   -1,  248,   -1,  250,   -1,   -1,   -1,  254,
			   24,   -1,  257,   -1,   -1,   -1,   -1,   -1,   -1,  264,
			   -1,  266,   -1,   -1,   38,   -1,   40,   -1,  273,   -1,
			  275,   -1,  277,   -1,  279,  280,   -1,   51,   -1,   53,

			   -1,   55,   -1,  288,   -1,   -1,  291,   -1,   -1,   -1,
			  295,   -1,   -1,   -1,   68,  300,   70,   -1,   -1,  304,
			   -1,   -1,   76,   -1,   -1,   79,   -1,   -1,   82,   -1,
			   -1,   -1,   -1,   87,   88,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,  101,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,  110,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  132,   -1,
			   -1,  135,   -1,  137,   -1,   -1,   -1,   -1,  142,  143,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,

			   -1,   -1,  156,   -1,   -1,   -1,   -1,  161,  162,   -1,
			  164,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  172,   -1,
			   -1,  175,   -1,   -1,  178,   -1,  180,   -1,   -1,   -1,
			  184,  185,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  193,
			  194,   -1,   -1,   -1,  198,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,  206,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,  217,  218,  219,   -1,   -1,  222,   -1,
			  224,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  233,
			   -1,   -1,   -1,   -1,  238,   -1,   -1,   -1,   -1,   -1,
			  244,   -1,   -1,   -1,  248,   -1,  250,   -1,   -1,   -1,

			  254,   -1,   -1,  257,   -1,   -1,   -1,   -1,   -1,   -1,
			  264,   -1,  266,   -1,   -1,   -1,   -1,   -1,   -1,  273,
			   -1,  275,   -1,  277,   -1,  279,  280,   -1,    5,    6,
			   -1,    8,   -1,   10,  288,   -1,   -1,  291,   -1,   -1,
			   -1,  295,   -1,   -1,   21,   22,  300,   24,   -1,   -1,
			  304,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   35,   -1,
			   -1,   38,   -1,   40,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   49,   -1,   51,   -1,   53,   -1,   55,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   68,   -1,   70,   -1,   -1,   -1,   -1,   -1,   76,

			   -1,   -1,   79,   -1,   -1,   82,   -1,   -1,   -1,   -1,
			   87,   88,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,  101,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,  110,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,  132,   -1,   -1,  135,   -1,
			  137,   -1,   -1,   -1,   -1,  142,  143,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  156,
			   -1,   -1,   -1,   -1,  161,  162,   -1,  164,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,  172,   -1,   -1,  175,   -1,

			   -1,  178,   -1,  180,   -1,   -1,   -1,  184,  185,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,  193,  194,   -1,   -1,
			   -1,  198,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  206,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			  217,  218,  219,   -1,   -1,  222,   -1,  224,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,  233,   -1,   -1,    5,
			   -1,  238,   -1,    9,   -1,   -1,   -1,  244,   -1,   -1,
			   -1,  248,   -1,  250,   -1,   -1,   -1,  254,   24,   -1,
			  257,   -1,   -1,   -1,   -1,   -1,   -1,  264,   -1,  266,
			   -1,   -1,   38,   -1,   40,   -1,  273,   -1,  275,   -1,

			  277,   -1,  279,  280,   -1,   51,   -1,   53,   -1,   55,
			   -1,  288,   -1,   -1,  291,   -1,   -1,   -1,  295,   -1,
			   -1,   -1,   68,  300,   70,   -1,   -1,  304,   -1,   -1,
			   76,   -1,   -1,   79,   -1,   -1,   82,   -1,   -1,   -1,
			   -1,   87,   88,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,  101,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,  110,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,  130,   -1,  132,   -1,   -1,  135,
			   -1,  137,   -1,   -1,   -1,   -1,  142,  143,   -1,   -1, yyDummy>>,
			1, 1000, 19000)
		end

	yycheck_template_21 (an_array: ARRAY [INTEGER])
			-- Fill chunk #21 of template for `yycheck'.
		do
			yyarray_subcopy (an_array, <<
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			  156,   -1,   -1,   -1,   -1,  161,  162,   -1,  164,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,  172,   -1,   -1,  175,
			   -1,   -1,  178,   -1,  180,   -1,   -1,   -1,  184,  185,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,  193,  194,   -1,
			   -1,   -1,  198,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			  206,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,  217,  218,  219,   -1,   -1,  222,   -1,  224,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,  233,   -1,   -1,
			    5,   -1,  238,   -1,    9,   -1,   -1,   -1,  244,   -1,

			   -1,   -1,  248,   -1,  250,   -1,   -1,   -1,  254,   24,
			   -1,  257,   -1,   -1,   -1,   -1,   -1,   -1,  264,   -1,
			  266,   -1,   -1,   38,   -1,   40,   -1,  273,   -1,  275,
			   -1,  277,   -1,  279,  280,   -1,   51,   -1,   53,   -1,
			   55,   -1,  288,   -1,   -1,  291,   -1,   -1,   -1,  295,
			   -1,   -1,   -1,   68,  300,   70,   -1,   -1,  304,   -1,
			   -1,   76,   -1,   -1,   79,   -1,   -1,   82,   -1,   -1,
			   -1,   -1,   87,   88,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,  101,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,  110,   -1,   -1,   -1,   -1,

			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,  130,   -1,  132,   -1,   -1,
			  135,   -1,  137,   -1,   -1,   -1,   -1,  142,  143,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,  156,   -1,   -1,   -1,   -1,  161,  162,   -1,  164,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,  172,   -1,   -1,
			  175,   -1,   -1,  178,   -1,  180,   -1,   -1,   -1,  184,
			  185,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  193,  194,
			   -1,   -1,   -1,  198,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,  206,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,

			   -1,   -1,  217,  218,  219,   -1,   -1,  222,   -1,  224,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  233,   -1,
			   -1,    5,   -1,  238,   -1,    9,   -1,   -1,   -1,  244,
			   -1,   -1,   -1,  248,   -1,  250,   -1,   -1,   -1,  254,
			   24,   -1,  257,   -1,   -1,   -1,   -1,   -1,   -1,  264,
			   -1,  266,   -1,   -1,   38,   -1,   40,   -1,  273,   -1,
			  275,   -1,  277,   -1,  279,  280,   -1,   51,   -1,   53,
			   -1,   55,   -1,  288,   -1,   -1,  291,   -1,   -1,   -1,
			  295,   -1,   -1,   -1,   68,  300,   70,   -1,   -1,  304,
			   -1,   -1,   76,   -1,   -1,   79,   -1,   -1,   82,   -1,

			   -1,   -1,   -1,   87,   88,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,  101,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,  110,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,  130,   -1,  132,   -1,
			   -1,  135,   -1,  137,   -1,   -1,   -1,   -1,  142,  143,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,  156,   -1,   -1,   -1,   -1,  161,  162,   -1,
			  164,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  172,   -1,
			   -1,  175,   -1,   -1,  178,   -1,  180,   -1,   -1,   -1,

			  184,  185,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  193,
			  194,   -1,   -1,   -1,  198,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,  206,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,  217,  218,  219,   -1,   -1,  222,   -1,
			  224,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  233,
			   -1,   -1,    5,   -1,  238,   -1,   -1,   -1,   -1,   -1,
			  244,   -1,   -1,   -1,  248,   -1,  250,   -1,   -1,   -1,
			  254,   24,   -1,  257,   -1,   -1,   -1,   -1,   -1,   -1,
			  264,   -1,  266,   -1,   -1,   38,   -1,   40,   -1,  273,
			   -1,  275,   -1,  277,   -1,  279,  280,   -1,   51,   -1,

			   53,   -1,   55,   -1,  288,   -1,   -1,  291,   -1,   -1,
			   -1,  295,   -1,   -1,   -1,   68,  300,   70,   -1,   -1,
			  304,   -1,   -1,   76,   -1,   -1,   79,   -1,   -1,   82,
			   -1,   -1,   -1,   -1,   87,   88,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  101,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,  110,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  132,
			   -1,   -1,  135,   -1,  137,   -1,   -1,   -1,   -1,  142,
			  143,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,

			   -1,   -1,   -1,  156,   -1,   -1,   -1,   -1,  161,  162,
			   -1,  164,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  172,
			   -1,   -1,  175,   -1,   -1,  178,   -1,  180,   -1,   -1,
			   -1,  184,  185,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			  193,  194,   -1,   -1,   -1,  198,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,  206,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,  217,  218,  219,   -1,   -1,  222,
			   -1,  224,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			  233,   -1,   -1,   -1,   -1,  238,   -1,   -1,   -1,   -1,
			   -1,  244,   -1,   -1,   -1,  248,   -1,  250,   -1,   -1,

			   -1,  254,   -1,   -1,  257,   -1,    5,   -1,   -1,   -1,
			   -1,  264,   -1,  266,   -1,   -1,   -1,   -1,   -1,   -1,
			  273,   -1,  275,   -1,  277,   24,  279,  280,   -1,   -1,
			   -1,  284,   -1,   -1,   -1,  288,   -1,   -1,  291,   38,
			   -1,   40,  295,   -1,   -1,   -1,   -1,  300,   -1,   -1,
			   -1,  304,   51,   -1,   53,   -1,   55,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   68,
			   -1,   70,   -1,   -1,   -1,   -1,   -1,   76,   -1,   -1,
			   79,   -1,   -1,   82,   -1,   -1,   -1,   -1,   87,   88,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,

			   -1,   -1,  101,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,  110,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,  130,   -1,  132,   -1,   -1,  135,   -1,  137,   -1,
			   -1,   -1,   -1,  142,  143,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,  156,   -1,   -1,
			   -1,   -1,  161,  162,   -1,  164,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,  172,   -1,   -1,  175,   -1,   -1,  178,
			   -1,  180,   -1,   -1,   -1,  184,  185,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,  193,  194,   -1,   -1,   -1,  198, yyDummy>>,
			1, 1000, 20000)
		end

	yycheck_template_22 (an_array: ARRAY [INTEGER])
			-- Fill chunk #22 of template for `yycheck'.
		do
			yyarray_subcopy (an_array, <<
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,  206,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  217,  218,
			  219,   -1,   -1,  222,   -1,  224,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,  233,   -1,   -1,    5,   -1,  238,
			   -1,    9,   -1,   -1,   -1,  244,   -1,   -1,   -1,  248,
			   -1,  250,   -1,   -1,   -1,  254,   24,   -1,  257,   -1,
			   -1,   -1,   -1,   -1,   -1,  264,   -1,  266,   -1,   -1,
			   38,   -1,   40,   -1,  273,   -1,  275,   -1,  277,   -1,
			  279,  280,   -1,   51,   -1,   53,   -1,   55,   -1,  288,
			   -1,   -1,  291,   -1,   -1,   -1,  295,   -1,   -1,   -1,

			   68,  300,   70,   -1,   -1,  304,   -1,   -1,   76,   -1,
			   -1,   79,   -1,   -1,   82,   -1,   -1,   -1,   -1,   87,
			   88,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,  101,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,  110,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,  132,   -1,   -1,  135,   -1,  137,
			   -1,   -1,   -1,   -1,  142,  143,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  156,   -1,
			   -1,   -1,   -1,  161,  162,   -1,  164,   -1,   -1,   -1,

			   -1,   -1,   -1,   -1,  172,   -1,   -1,  175,   -1,   -1,
			  178,   -1,  180,   -1,   -1,   -1,  184,  185,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,  193,  194,   -1,   -1,   -1,
			  198,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  206,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  217,
			  218,  219,   -1,   -1,  222,   -1,  224,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,  233,   -1,   -1,    5,   -1,
			  238,   -1,   -1,   -1,   -1,   -1,  244,   -1,   -1,   -1,
			  248,   -1,  250,   -1,   -1,   -1,  254,   24,   -1,  257,
			   -1,   -1,   -1,   -1,   -1,   -1,  264,   -1,  266,   -1,

			   -1,   38,   -1,   40,   -1,  273,   -1,  275,   -1,  277,
			   -1,  279,  280,   -1,   51,   -1,   53,   -1,   55,   -1,
			  288,   -1,   -1,  291,   -1,   -1,   -1,  295,   -1,   -1,
			   -1,   68,  300,   70,   -1,   -1,  304,   -1,   -1,   76,
			   -1,   -1,   79,   -1,   -1,   82,   -1,   -1,   -1,   -1,
			   87,   88,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,  101,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,  110,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,  132,   -1,   -1,  135,   -1,

			  137,   -1,   -1,   -1,   -1,  142,  143,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  156,
			   -1,   -1,   -1,   -1,  161,  162,   -1,  164,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,  172,   -1,   -1,  175,   -1,
			   -1,  178,   -1,  180,   -1,   -1,   -1,  184,  185,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,  193,  194,   -1,   -1,
			   -1,  198,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  206,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			  217,  218,  219,   -1,   -1,  222,   -1,  224,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,  233,   -1,   -1,    5,

			   -1,  238,   -1,   -1,   -1,   -1,   -1,  244,   -1,   -1,
			   -1,  248,   -1,  250,   -1,   -1,   -1,  254,   24,   -1,
			  257,   -1,   -1,   -1,   -1,   -1,   -1,  264,   -1,  266,
			   -1,   -1,   38,   -1,   40,   -1,  273,   -1,  275,   -1,
			  277,   -1,  279,  280,   -1,   51,   -1,   53,   -1,   55,
			   -1,  288,   -1,   -1,  291,   -1,   -1,   -1,  295,   -1,
			   -1,   -1,   68,  300,   70,   -1,   -1,  304,   -1,   -1,
			   76,   -1,   -1,   79,   -1,   -1,   82,   -1,   -1,   -1,
			   -1,   87,   88,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,  101,   -1,   -1,   -1,   -1,

			   -1,   -1,   -1,   -1,  110,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,  132,   -1,   -1,  135,
			   -1,  137,   -1,   -1,   -1,   -1,  142,  143,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			  156,   -1,   -1,   -1,   -1,  161,  162,   -1,  164,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,  172,   -1,   -1,  175,
			   -1,   -1,  178,   -1,  180,   -1,   -1,   -1,  184,  185,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,  193,  194,   -1,
			   -1,   -1,  198,   -1,   -1,   -1,   -1,   -1,   -1,   -1,

			  206,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,  217,  218,  219,   -1,   -1,  222,   -1,  224,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,  233,   -1,   -1,
			    5,   -1,  238,   -1,   -1,   -1,   -1,   -1,  244,   -1,
			   -1,   -1,  248,   -1,  250,   -1,   -1,   -1,  254,   24,
			   -1,  257,   -1,   -1,   -1,   -1,   -1,   -1,  264,   -1,
			  266,   -1,   -1,   38,   -1,   40,   -1,  273,   -1,  275,
			   -1,  277,   -1,  279,  280,   -1,   51,   -1,   53,   -1,
			   55,   -1,  288,   -1,   -1,  291,   -1,   -1,   -1,  295,
			   -1,   -1,   -1,   68,  300,   70,   -1,   -1,  304,   -1,

			   -1,   76,   -1,   -1,   79,   -1,   -1,   82,   -1,   -1,
			   -1,   -1,   87,   88,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,  101,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,  110,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,  132,   -1,   -1,
			  135,   -1,  137,   -1,   -1,   -1,   -1,  142,  143,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,  156,   -1,   -1,   -1,   -1,  161,  162,   -1,  164,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,  172,   -1,   -1,

			  175,   -1,   -1,  178,   -1,  180,   -1,   -1,   -1,  184,
			  185,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  193,  194,
			   -1,   -1,   -1,  198,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,  206,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,  217,  218,  219,   -1,   -1,  222,   -1,  224,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  233,   -1,
			   -1,   -1,   -1,  238,   -1,   -1,   -1,   -1,   -1,  244,
			   -1,   -1,   -1,  248,   -1,  250,   -1,   -1,    7,  254,
			   -1,   -1,  257,   -1,   -1,   -1,   -1,   -1,   -1,  264,
			   -1,  266,   -1,   -1,   -1,   24,   -1,   -1,  273,   -1, yyDummy>>,
			1, 1000, 21000)
		end

	yycheck_template_23 (an_array: ARRAY [INTEGER])
			-- Fill chunk #23 of template for `yycheck'.
		do
			yyarray_subcopy (an_array, <<
			  275,   -1,  277,   -1,  279,  280,   -1,   -1,   -1,   38,
			   -1,   40,   -1,  288,   -1,   -1,  291,   46,   -1,   -1,
			  295,   -1,   51,   -1,   53,  300,   55,   -1,   -1,  304,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   66,   -1,   68,
			   -1,   70,   -1,   -1,   -1,   74,   -1,   76,   -1,   -1,
			   79,   -1,   -1,   82,   -1,   -1,   -1,   -1,   87,   88,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,  101,  102,  103,   -1,  105,   -1,   -1,   -1,
			   -1,  110,   -1,   -1,   -1,  114,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,

			   -1,   -1,   -1,  132,  133,   -1,  135,   -1,  137,   -1,
			   -1,   -1,   -1,  142,  143,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,  156,   -1,   -1,
			   -1,   -1,  161,  162,   -1,  164,   -1,  166,   -1,   -1,
			   -1,   -1,   -1,  172,   -1,   -1,  175,   -1,   -1,  178,
			   -1,  180,   -1,  182,   -1,  184,  185,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,  193,  194,   -1,  196,   -1,  198,
			   -1,   -1,   -1,   -1,   -1,  204,  205,  206,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  217,  218,
			  219,   -1,   -1,  222,   -1,  224,   -1,   -1,   -1,   -1,

			   -1,   -1,   -1,   -1,  233,  234,   -1,   -1,   -1,  238,
			   -1,   -1,   -1,   -1,   -1,  244,   -1,   -1,   -1,  248,
			   -1,  250,   -1,   -1,   -1,  254,   -1,   -1,  257,   -1,
			   -1,  260,   -1,   -1,   -1,  264,   -1,  266,    9,   -1,
			   -1,   -1,   -1,   -1,  273,   -1,  275,   -1,  277,  278,
			  279,  280,   -1,   24,  283,  284,   -1,   -1,   -1,  288,
			   -1,   -1,  291,   -1,   -1,   -1,  295,   38,   -1,   40,
			   -1,  300,   -1,  302,  303,  304,   -1,   -1,   -1,   -1,
			   51,   -1,   53,   -1,   55,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   68,   -1,   70,

			   -1,   -1,   -1,   -1,   -1,   76,   -1,   -1,   79,   -1,
			   -1,   82,   -1,   -1,   -1,   -1,   87,   88,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			  101,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  110,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  130,
			   -1,  132,   -1,   -1,  135,   -1,  137,   -1,   -1,   -1,
			   -1,  142,  143,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,  156,   -1,   -1,   -1,   -1,
			  161,  162,   -1,  164,   -1,   -1,   -1,   -1,   -1,   -1,

			   -1,  172,   -1,   -1,  175,   -1,   -1,  178,   -1,  180,
			   -1,   -1,   -1,  184,  185,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,  193,  194,   -1,   -1,   -1,  198,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,  206,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,  217,  218,  219,   -1,
			   -1,  222,   -1,  224,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,  233,   -1,   -1,   -1,   -1,  238,   -1,    9,
			   -1,   -1,   -1,  244,   -1,   -1,   -1,  248,   -1,  250,
			   -1,   -1,   -1,  254,   24,   -1,  257,   -1,   -1,   -1,
			   -1,   -1,   -1,  264,   -1,  266,   -1,   -1,   38,   -1,

			   40,   -1,  273,   -1,  275,   -1,  277,   -1,  279,  280,
			   -1,   51,   -1,   53,   -1,   55,   -1,  288,   -1,   -1,
			  291,   -1,   -1,   -1,  295,   -1,   -1,   -1,   68,  300,
			   70,   -1,   -1,  304,   -1,   -1,   76,   -1,   -1,   79,
			   -1,   -1,   82,   -1,   -1,   -1,   -1,   87,   88,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,  101,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			  110,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			  130,   -1,  132,   -1,   -1,  135,   -1,  137,   -1,   -1,

			   -1,   -1,  142,  143,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,  156,   -1,   -1,   -1,
			   -1,  161,  162,   -1,  164,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,  172,   -1,   -1,  175,   -1,   -1,  178,   -1,
			  180,   -1,   -1,   -1,  184,  185,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,  193,  194,   -1,   -1,   -1,  198,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,  206,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,  217,  218,  219,
			   -1,   -1,  222,   -1,  224,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,  233,   -1,   -1,   -1,   -1,  238,   -1,

			    9,   -1,   -1,   -1,  244,   -1,   -1,   -1,  248,   -1,
			  250,   -1,   -1,   -1,  254,   24,   -1,  257,   -1,   -1,
			   -1,   -1,   -1,   -1,  264,   -1,  266,   -1,   -1,   38,
			   -1,   40,   -1,  273,   -1,  275,   -1,  277,   -1,  279,
			  280,   -1,   51,   -1,   53,   -1,   55,   -1,  288,   -1,
			   -1,  291,   -1,   -1,   -1,  295,   -1,   -1,   -1,   68,
			  300,   70,   -1,   -1,  304,   -1,   -1,   76,   -1,   -1,
			   79,   -1,   -1,   82,   -1,   -1,   -1,   -1,   87,   88,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,  101,   -1,   -1,   -1,   -1,   -1,   -1,   -1,

			   -1,  110,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,  130,   -1,  132,   -1,   -1,  135,   -1,  137,   -1,
			   -1,   -1,   -1,  142,  143,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,  156,   -1,   -1,
			   -1,   -1,  161,  162,   -1,  164,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,  172,   -1,   -1,  175,   -1,   -1,  178,
			   -1,  180,   -1,   -1,   -1,  184,  185,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,  193,  194,   -1,   -1,   -1,  198,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,  206,   -1,   -1,

			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  217,  218,
			  219,   -1,   -1,  222,   -1,  224,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,  233,   -1,   -1,   -1,   -1,  238,
			   -1,    9,   -1,   -1,   -1,  244,   -1,   -1,   -1,  248,
			   -1,  250,   -1,   -1,   -1,  254,   24,   -1,  257,   -1,
			   -1,   -1,   -1,   -1,   -1,  264,   -1,  266,   -1,   -1,
			   38,   -1,   40,   -1,  273,   -1,  275,   -1,  277,   -1,
			  279,  280,   -1,   51,   -1,   53,   -1,   55,   -1,  288,
			   -1,   -1,  291,   -1,   -1,   -1,  295,   -1,   -1,   -1,
			   68,  300,   70,   -1,   -1,  304,   -1,   -1,   76,   -1, yyDummy>>,
			1, 1000, 22000)
		end

	yycheck_template_24 (an_array: ARRAY [INTEGER])
			-- Fill chunk #24 of template for `yycheck'.
		do
			yyarray_subcopy (an_array, <<
			   -1,   79,   -1,   -1,   82,   -1,   -1,   -1,   -1,   87,
			   88,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,  101,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,  110,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,  132,   -1,   -1,  135,   -1,  137,
			   -1,   -1,   -1,   -1,  142,  143,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  156,   -1,
			   -1,   -1,   -1,  161,  162,   -1,  164,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,  172,   -1,   -1,  175,   -1,   -1,

			  178,   -1,  180,   -1,   -1,   -1,  184,  185,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,  193,  194,   -1,   -1,   -1,
			  198,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  206,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  217,
			  218,  219,   -1,   -1,  222,   -1,  224,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,  233,   -1,   -1,   -1,   -1,
			  238,   -1,    9,   -1,   -1,   -1,  244,   -1,   -1,   -1,
			  248,   -1,  250,   -1,   -1,   -1,  254,   24,   -1,  257,
			   -1,   -1,   -1,   -1,   -1,   -1,  264,   -1,  266,   -1,
			   -1,   38,   -1,   40,   -1,  273,   -1,  275,   -1,  277,

			   -1,  279,  280,   -1,   51,   -1,   53,   -1,   55,   -1,
			  288,   -1,   -1,  291,   -1,   -1,   -1,  295,   -1,   -1,
			   -1,   68,  300,   70,   -1,   -1,  304,   -1,   -1,   76,
			   -1,   -1,   79,   -1,   -1,   82,   -1,   -1,   -1,   -1,
			   87,   88,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,  101,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,  110,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,  132,   -1,   -1,  135,   -1,
			  137,   -1,   -1,   -1,   -1,  142,  143,   -1,   -1,   -1,

			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  156,
			   -1,   -1,   -1,   -1,  161,  162,   -1,  164,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,  172,   -1,   -1,  175,   -1,
			   -1,  178,   -1,  180,   -1,   -1,   -1,  184,  185,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,  193,  194,   -1,   -1,
			   -1,  198,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  206,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			  217,  218,  219,   -1,   -1,  222,   -1,  224,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,  233,   -1,   -1,   -1,
			   -1,  238,   -1,    9,   -1,   -1,   -1,  244,   -1,   -1,

			   -1,  248,   -1,  250,   -1,   -1,   -1,  254,   24,   -1,
			  257,   -1,   -1,   -1,   -1,   -1,   -1,  264,   -1,  266,
			   -1,   -1,   38,   -1,   40,   -1,  273,   -1,  275,   -1,
			  277,   -1,  279,  280,   -1,   51,   -1,   53,   -1,   55,
			   -1,  288,   -1,   -1,  291,   -1,   -1,   -1,  295,   -1,
			   -1,   -1,   68,  300,   70,   -1,   -1,  304,   -1,   -1,
			   76,   -1,   -1,   79,   -1,   -1,   82,   -1,   -1,   -1,
			   -1,   87,   88,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,  101,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,  110,   -1,   -1,   -1,   -1,   -1,

			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,  132,   -1,   -1,  135,
			   -1,  137,   -1,   -1,   -1,   -1,  142,  143,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			  156,   -1,   -1,   -1,   -1,  161,  162,   -1,  164,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,  172,   -1,   -1,  175,
			   -1,   -1,  178,   -1,  180,   -1,   -1,   -1,  184,  185,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,  193,  194,   -1,
			   -1,   -1,  198,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			  206,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,

			   -1,  217,  218,  219,   -1,   -1,  222,   -1,  224,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,  233,   -1,   -1,
			   -1,   -1,  238,   -1,    9,   -1,   -1,   -1,  244,   -1,
			   -1,   -1,  248,   -1,  250,   -1,   -1,   -1,  254,   24,
			   -1,  257,   -1,   -1,   -1,   -1,   -1,   -1,  264,   -1,
			  266,   -1,   -1,   38,   -1,   40,   -1,  273,   -1,  275,
			   -1,  277,   -1,  279,  280,   -1,   51,   -1,   53,   -1,
			   55,   -1,  288,   -1,   -1,  291,   -1,   -1,   -1,  295,
			   -1,   -1,   -1,   68,  300,   70,   -1,   -1,  304,   -1,
			   -1,   76,   -1,   -1,   79,   -1,   -1,   82,   -1,   -1,

			   -1,   -1,   87,   88,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,  101,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,  110,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,  132,   -1,   -1,
			  135,   -1,  137,   -1,   -1,   -1,   -1,  142,  143,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,  156,   -1,   -1,   -1,   -1,  161,  162,   -1,  164,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,  172,   -1,   -1,
			  175,   -1,   -1,  178,   -1,  180,   -1,   -1,   -1,  184,

			  185,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  193,  194,
			   -1,   -1,   -1,  198,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,  206,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,  217,  218,  219,   -1,   -1,  222,   -1,  224,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  233,   -1,
			   -1,   -1,   -1,  238,   -1,    9,   -1,   -1,   -1,  244,
			   -1,   -1,   -1,  248,   -1,  250,   -1,   -1,   -1,  254,
			   24,   -1,  257,   -1,   -1,   -1,   -1,   -1,   -1,  264,
			   -1,  266,   -1,   -1,   38,   -1,   40,   -1,  273,   -1,
			  275,   -1,  277,   -1,  279,  280,   -1,   51,   -1,   53,

			   -1,   55,   -1,  288,   -1,   -1,  291,   -1,   -1,   -1,
			  295,   -1,   -1,   -1,   68,  300,   70,   -1,   -1,  304,
			   -1,   -1,   76,   -1,   -1,   79,   -1,   -1,   82,   -1,
			   -1,   -1,   -1,   87,   88,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,  101,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,  110,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  132,   -1,
			   -1,  135,   -1,  137,   -1,   -1,   -1,   -1,  142,  143,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, yyDummy>>,
			1, 1000, 23000)
		end

	yycheck_template_25 (an_array: ARRAY [INTEGER])
			-- Fill chunk #25 of template for `yycheck'.
		do
			yyarray_subcopy (an_array, <<
			   -1,   -1,  156,   -1,   -1,   -1,   -1,  161,  162,   -1,
			  164,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  172,   -1,
			   -1,  175,   -1,   -1,  178,   -1,  180,   -1,   -1,   -1,
			  184,  185,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  193,
			  194,   -1,   -1,   -1,  198,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,  206,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,  217,  218,  219,   -1,   -1,  222,   -1,
			  224,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  233,
			   -1,   -1,   -1,   -1,  238,   -1,    9,   -1,   -1,   -1,
			  244,   -1,   -1,   -1,  248,   -1,  250,   -1,   -1,   -1,

			  254,   24,   -1,  257,   -1,   -1,   -1,   -1,   -1,   -1,
			  264,   -1,  266,   -1,   -1,   38,   -1,   40,   -1,  273,
			   -1,  275,   -1,  277,   -1,  279,  280,   -1,   51,   -1,
			   53,   -1,   55,   -1,  288,   -1,   -1,  291,   -1,   -1,
			   -1,  295,   -1,   -1,   -1,   68,  300,   70,   -1,   -1,
			  304,   -1,   -1,   76,   -1,   -1,   79,   -1,   -1,   82,
			   -1,   -1,   -1,   -1,   87,   88,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  101,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,  110,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,

			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  132,
			   -1,   -1,  135,   -1,  137,   -1,   -1,   -1,   -1,  142,
			  143,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,  156,   -1,   -1,   -1,   -1,  161,  162,
			   -1,  164,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  172,
			   -1,   -1,  175,   -1,   -1,  178,   -1,  180,   -1,   -1,
			   -1,  184,  185,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			  193,  194,   -1,   -1,   -1,  198,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,  206,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,  217,  218,  219,   -1,   -1,  222,

			   -1,  224,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			  233,   -1,   -1,   -1,   -1,  238,   -1,    9,   -1,   -1,
			   -1,  244,   -1,   -1,   -1,  248,   -1,  250,   -1,   -1,
			   -1,  254,   24,   -1,  257,   -1,   -1,   -1,   -1,   -1,
			   -1,  264,   -1,  266,   -1,   -1,   38,   -1,   40,   -1,
			  273,   -1,  275,   -1,  277,   -1,  279,  280,   -1,   51,
			   -1,   53,   -1,   55,   -1,  288,   -1,   -1,  291,   -1,
			   -1,   -1,  295,   -1,   -1,   -1,   68,  300,   70,   -1,
			   -1,  304,   -1,   -1,   76,   -1,   -1,   79,   -1,   -1,
			   82,   -1,   -1,   -1,   -1,   87,   88,   -1,   -1,   -1,

			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  101,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  110,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			  132,   -1,   -1,  135,   -1,  137,   -1,   -1,   -1,   -1,
			  142,  143,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,  156,   -1,   -1,   -1,   -1,  161,
			  162,   -1,  164,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			  172,   -1,   -1,  175,   -1,   -1,  178,   -1,  180,   -1,
			   -1,   -1,  184,  185,   -1,   -1,   -1,   -1,   -1,   -1,

			   -1,  193,  194,   -1,   -1,   -1,  198,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,  206,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,  217,  218,  219,   -1,   -1,
			  222,   -1,  224,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,  233,   -1,   -1,   -1,   -1,  238,   -1,    9,   -1,
			   -1,   -1,  244,   -1,   -1,   -1,  248,   -1,  250,   -1,
			   -1,   -1,  254,   24,   -1,  257,   -1,   -1,   -1,   -1,
			   -1,   -1,  264,   -1,  266,   -1,   -1,   38,   -1,   40,
			   -1,  273,   -1,  275,   -1,  277,   -1,  279,  280,   -1,
			   51,   -1,   53,   -1,   55,   -1,  288,   -1,   -1,  291,

			   -1,   -1,   -1,  295,   -1,   -1,   -1,   68,  300,   70,
			   -1,   -1,  304,   -1,   -1,   76,   -1,   -1,   79,   -1,
			   -1,   82,   -1,   -1,   -1,   -1,   87,   88,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			  101,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  110,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,  132,   -1,   -1,  135,   -1,  137,   -1,   -1,   -1,
			   -1,  142,  143,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,  156,   -1,   -1,   -1,   -1,

			  161,  162,   -1,  164,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,  172,   -1,   -1,  175,   -1,   -1,  178,   -1,  180,
			   -1,   -1,   -1,  184,  185,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,  193,  194,   -1,   -1,   -1,  198,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,  206,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,  217,  218,  219,   -1,
			   -1,  222,   -1,  224,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,  233,   -1,   -1,   -1,   -1,  238,   -1,   -1,
			   -1,   -1,   -1,  244,   -1,   -1,   -1,  248,   -1,  250,
			   -1,   -1,   -1,  254,   24,   -1,  257,   -1,   -1,   -1,

			   -1,   -1,   -1,  264,   -1,  266,   -1,   -1,   38,   -1,
			   40,   -1,  273,   -1,  275,   -1,  277,   -1,  279,  280,
			   -1,   51,   -1,   53,   -1,   55,   -1,  288,   -1,   -1,
			  291,   -1,   -1,   -1,  295,   -1,   -1,   -1,   68,  300,
			   70,   -1,   -1,  304,   -1,   -1,   76,   -1,   -1,   79,
			   -1,   -1,   82,   -1,   -1,   -1,   -1,   87,   88,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,  101,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			  110,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,

			   -1,   -1,  132,   -1,   -1,  135,   -1,  137,   -1,   -1,
			   -1,   -1,  142,  143,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,  156,   -1,   -1,   -1,
			   -1,  161,  162,   -1,  164,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,  172,   -1,   -1,  175,   -1,   -1,  178,   -1,
			  180,   -1,   -1,   -1,  184,  185,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,  193,  194,   -1,   -1,   -1,  198,   -1,
			   -1,   -1,   -1,   -1,   -1,  205,  206,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,  217,  218,  219,
			   -1,   -1,  222,   -1,  224,   -1,   -1,   -1,   -1,   -1, yyDummy>>,
			1, 1000, 24000)
		end

	yycheck_template_26 (an_array: ARRAY [INTEGER])
			-- Fill chunk #26 of template for `yycheck'.
		do
			yyarray_subcopy (an_array, <<
			   -1,   -1,   -1,  233,   -1,   -1,   -1,   -1,  238,   -1,
			   -1,   -1,   -1,   -1,  244,   -1,   -1,   -1,  248,   -1,
			  250,   -1,   -1,   -1,  254,   -1,   -1,  257,   -1,   -1,
			   -1,   -1,   -1,   -1,  264,   -1,  266,   24,   -1,   -1,
			   -1,   -1,   -1,  273,   -1,  275,   -1,  277,   -1,  279,
			  280,   38,   -1,   40,   -1,   -1,   -1,   -1,  288,   46,
			   -1,  291,   -1,   -1,   51,  295,   53,   -1,   55,   -1,
			  300,   -1,  302,  303,  304,   -1,   -1,   -1,   -1,   66,
			   -1,   68,  312,   70,   -1,   -1,   -1,   74,   -1,   76,
			   -1,   -1,   79,   -1,   -1,   82,   -1,   -1,   -1,   -1,

			   87,   88,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,  101,  102,  103,   -1,  105,   -1,
			   -1,   -1,   -1,  110,   -1,   -1,   -1,  114,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,  132,  133,   -1,  135,   -1,
			  137,   -1,   -1,   -1,   -1,  142,  143,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  156,
			   -1,   -1,   -1,   -1,  161,  162,   -1,  164,   -1,  166,
			   -1,   -1,   -1,   -1,   -1,  172,   -1,   -1,  175,   -1,
			   -1,  178,   -1,  180,   -1,  182,   -1,  184,  185,   -1,

			   -1,   -1,   -1,   -1,   -1,   -1,  193,  194,   -1,  196,
			   -1,  198,   -1,   -1,   -1,   -1,   -1,  204,  205,  206,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			  217,  218,  219,   -1,   -1,  222,   -1,  224,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,  233,  234,   -1,   -1,
			   -1,  238,   -1,   -1,   -1,   -1,   -1,  244,   -1,   -1,
			   -1,  248,   -1,  250,   -1,   -1,   -1,  254,   -1,   -1,
			  257,   -1,   -1,  260,   -1,   -1,   -1,  264,   -1,  266,
			   -1,   -1,   -1,   -1,   -1,   -1,  273,   -1,  275,   -1,
			  277,  278,  279,  280,   -1,   -1,  283,  284,   24,   -1,

			   -1,  288,   -1,   -1,  291,   -1,   -1,   -1,  295,   -1,
			   -1,   -1,   38,  300,   40,  302,  303,  304,   -1,   -1,
			   46,   -1,   -1,   49,   -1,   51,   -1,   53,   -1,   55,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   66,   -1,   68,   -1,   70,   -1,   -1,   -1,   74,   -1,
			   76,   -1,   -1,   79,   -1,   -1,   82,   -1,   -1,   -1,
			   -1,   87,   88,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,  101,  102,  103,   -1,  105,
			   -1,   -1,   -1,   -1,  110,   -1,   -1,   -1,  114,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,

			   -1,   -1,   -1,   -1,   -1,   -1,  132,  133,   -1,  135,
			   -1,  137,   -1,   -1,   -1,   -1,  142,  143,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			  156,   -1,   -1,   -1,   -1,  161,  162,   -1,  164,   -1,
			  166,   -1,   -1,   -1,   -1,   -1,  172,   -1,   -1,  175,
			   -1,   -1,  178,   -1,  180,   -1,  182,   -1,  184,  185,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,  193,  194,   -1,
			  196,   -1,  198,   -1,   -1,   -1,   -1,   -1,  204,  205,
			  206,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,  217,  218,  219,   -1,   -1,  222,   -1,  224,   -1,

			   -1,   -1,   -1,   -1,   -1,   -1,   -1,  233,  234,   -1,
			   -1,   -1,  238,   -1,   -1,   -1,   -1,   -1,  244,   -1,
			   -1,   -1,  248,   -1,  250,   -1,   -1,   -1,  254,   -1,
			   -1,  257,   -1,   -1,  260,   -1,   -1,   -1,  264,   -1,
			  266,   -1,   -1,   -1,   -1,   -1,   -1,  273,   -1,  275,
			   -1,  277,   24,  279,  280,   -1,   -1,  283,  284,   -1,
			   -1,   -1,  288,   -1,   -1,  291,   38,   -1,   40,  295,
			   -1,   43,   -1,   -1,  300,   -1,  302,  303,  304,   51,
			   -1,   53,   -1,   55,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   68,   -1,   70,   -1,

			   -1,   -1,   -1,   -1,   76,   -1,   -1,   79,   -1,   -1,
			   82,   -1,   -1,   -1,   -1,   87,   88,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  101,
			   -1,   -1,   -1,   -1,   -1,   -1,  108,   -1,  110,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,  126,   -1,   -1,   -1,   -1,   -1,
			  132,   -1,   -1,  135,   -1,  137,   -1,   -1,   -1,   -1,
			  142,  143,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,  156,   -1,   -1,   -1,   -1,  161,
			  162,  163,  164,   -1,   -1,   -1,   -1,   -1,   -1,   -1,

			  172,   -1,   -1,  175,   -1,   -1,  178,   -1,  180,   -1,
			   -1,   -1,  184,  185,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,  193,  194,   -1,   -1,   -1,  198,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,  206,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,  217,  218,  219,   -1,   -1,
			  222,   -1,  224,   -1,   -1,   -1,   -1,   -1,   -1,  231,
			   -1,  233,   -1,   -1,  236,   -1,  238,   -1,   -1,   -1,
			   -1,   -1,  244,   -1,   -1,   -1,  248,   -1,  250,   -1,
			   -1,   -1,  254,   24,  256,  257,   -1,   -1,   -1,   -1,
			   -1,   -1,  264,   -1,  266,   -1,   -1,   38,   -1,   40,

			   -1,  273,   43,  275,   -1,  277,   -1,  279,  280,   -1,
			   51,   -1,   53,   -1,   55,   -1,  288,   -1,   -1,  291,
			  292,   -1,   -1,  295,  296,   -1,   -1,   68,  300,   70,
			   -1,   -1,  304,   -1,   -1,   76,   -1,   -1,   79,   -1,
			   -1,   82,   -1,   -1,   -1,   -1,   87,   88,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			  101,   -1,   -1,   -1,   -1,   -1,   -1,  108,   -1,  110,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,  126,   -1,   -1,   -1,   -1,
			   -1,  132,   -1,   -1,  135,   -1,  137,   -1,   -1,   -1,

			   -1,  142,  143,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,  156,   -1,   -1,   -1,   -1,
			  161,  162,  163,  164,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,  172,   -1,   -1,  175,   -1,   -1,  178,   -1,  180,
			   -1,   -1,   -1,  184,  185,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,  193,  194,   -1,   -1,   -1,  198,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,  206,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,  217,  218,  219,   -1,
			   -1,  222,   -1,  224,   -1,   -1,   -1,   -1,   -1,   -1,
			  231,   -1,  233,   -1,   -1,  236,   -1,  238,   -1,   -1, yyDummy>>,
			1, 1000, 25000)
		end

	yycheck_template_27 (an_array: ARRAY [INTEGER])
			-- Fill chunk #27 of template for `yycheck'.
		do
			yyarray_subcopy (an_array, <<
			   -1,   -1,   -1,  244,   -1,   -1,   -1,  248,   -1,  250,
			   -1,   -1,   -1,  254,   24,  256,  257,   -1,   -1,   -1,
			   -1,   -1,   -1,  264,   -1,  266,   -1,   -1,   38,   -1,
			   40,   -1,  273,   -1,  275,   -1,  277,   -1,  279,  280,
			   -1,   51,   -1,   53,   -1,   55,   -1,  288,   -1,   -1,
			  291,  292,   -1,   -1,  295,  296,   -1,   -1,   68,  300,
			   70,   -1,   -1,  304,   74,   -1,   76,   -1,   -1,   79,
			   -1,   -1,   82,   -1,   -1,   -1,   -1,   87,   88,   -1,
			   -1,   91,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,  101,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,

			  110,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			  130,   -1,  132,   -1,   -1,  135,   -1,  137,   -1,   -1,
			   -1,   -1,  142,  143,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,  156,   -1,   -1,   -1,
			   -1,  161,  162,   -1,  164,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,  172,   -1,   -1,  175,   -1,  177,  178,   -1,
			  180,  181,   -1,   -1,  184,  185,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,  193,  194,   -1,   -1,   -1,  198,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,  206,   -1,   -1,   -1,

			   -1,   -1,   -1,   -1,   -1,   -1,   -1,  217,  218,  219,
			   -1,   -1,  222,   -1,  224,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,  233,   -1,   -1,   -1,   -1,  238,   -1,
			   -1,   -1,   -1,   -1,  244,   -1,   -1,   -1,  248,  249,
			  250,   -1,   -1,   -1,  254,   24,   -1,  257,   -1,   -1,
			   -1,   -1,   -1,   -1,  264,   -1,  266,   -1,   -1,   38,
			   -1,   40,   -1,  273,   -1,  275,   -1,  277,   -1,  279,
			  280,   -1,   51,   -1,   53,  285,   55,   -1,  288,   -1,
			   -1,  291,   -1,   -1,   -1,  295,   -1,   -1,  298,   68,
			  300,   70,   -1,   -1,  304,   -1,   -1,   76,   -1,   -1,

			   79,   -1,   -1,   82,   -1,   -1,   -1,   -1,   87,   88,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,  101,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,  110,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,  132,   -1,   -1,  135,   -1,  137,   -1,
			   -1,   -1,   -1,  142,  143,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,  156,   -1,   -1,
			   -1,   -1,  161,  162,   -1,  164,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,  172,   -1,   -1,  175,   -1,   -1,  178,

			   -1,  180,   -1,   -1,   -1,  184,  185,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,  193,  194,   -1,   -1,   -1,  198,
			  199,   -1,   -1,   -1,   -1,   -1,   -1,  206,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  217,  218,
			  219,   -1,   -1,  222,   -1,  224,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,  233,   -1,   -1,   -1,   -1,  238,
			   -1,   -1,   -1,   -1,   -1,  244,   -1,   -1,   -1,  248,
			   -1,  250,   -1,   -1,  253,  254,   -1,   -1,  257,   -1,
			   -1,   -1,   -1,   -1,   -1,  264,   -1,  266,   -1,   -1,
			   -1,   -1,   -1,   -1,  273,   -1,  275,   24,  277,   -1,

			  279,  280,   -1,   -1,  283,  284,   -1,   -1,   -1,  288,
			   -1,   38,  291,   40,   -1,   -1,  295,   -1,   -1,   -1,
			   -1,  300,   -1,   -1,   51,  304,   53,   -1,   55,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   68,   -1,   70,   -1,   -1,   -1,   -1,   -1,   76,
			   -1,   -1,   79,   -1,   -1,   82,   -1,   -1,   -1,   86,
			   87,   88,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,  101,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,  110,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,

			   -1,   -1,   -1,   -1,   -1,  132,   -1,   -1,  135,   -1,
			  137,   -1,   -1,   -1,   -1,  142,  143,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  156,
			   -1,   -1,   -1,   -1,  161,  162,   -1,  164,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,  172,   -1,   -1,  175,   -1,
			   -1,  178,   -1,  180,   -1,   -1,   -1,  184,  185,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,  193,  194,   -1,   -1,
			   -1,  198,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  206,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			  217,  218,  219,   -1,   -1,  222,   -1,  224,   -1,   -1,

			   -1,   -1,   -1,   -1,   -1,   -1,  233,   -1,   -1,   -1,
			   -1,  238,   -1,   -1,   -1,   -1,   -1,  244,   -1,   -1,
			   -1,  248,   -1,  250,   -1,   -1,   -1,  254,   -1,   -1,
			  257,   -1,   -1,   -1,   -1,   -1,   -1,  264,   -1,  266,
			   -1,   -1,   -1,   -1,   -1,   -1,  273,   -1,  275,   -1,
			  277,   -1,  279,  280,   -1,   24,  283,  284,   -1,   -1,
			   -1,  288,   -1,   -1,  291,  292,   -1,   -1,  295,   38,
			   -1,   40,   -1,  300,   -1,   -1,   -1,  304,   -1,   -1,
			   -1,   -1,   51,   -1,   53,   -1,   55,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   68,

			   -1,   70,   -1,   -1,   -1,   -1,   -1,   76,   -1,   -1,
			   79,   -1,   -1,   82,   -1,   -1,   -1,   86,   87,   88,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,  101,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,  110,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,  132,   -1,   -1,  135,   -1,  137,   -1,
			   -1,   -1,   -1,  142,  143,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,  156,   -1,   -1,
			   -1,   -1,  161,  162,   -1,  164,   -1,   -1,   -1,   -1,

			   -1,   -1,   -1,  172,   -1,   -1,  175,   -1,   -1,  178,
			   -1,  180,   -1,   -1,   -1,  184,  185,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,  193,  194,   -1,   -1,   -1,  198,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,  206,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  217,  218,
			  219,   -1,   -1,  222,   -1,  224,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,  233,   -1,   -1,   -1,   -1,  238,
			   -1,   -1,   -1,   -1,   -1,  244,   -1,   -1,   -1,  248,
			   -1,  250,   -1,   -1,   -1,  254,   -1,   -1,  257,   -1,
			   -1,   -1,   -1,   -1,   -1,  264,   -1,  266,   -1,   -1, yyDummy>>,
			1, 1000, 26000)
		end

	yycheck_template_28 (an_array: ARRAY [INTEGER])
			-- Fill chunk #28 of template for `yycheck'.
		do
			yyarray_subcopy (an_array, <<
			   -1,   -1,   -1,   -1,  273,   -1,  275,   -1,  277,   -1,
			  279,  280,   -1,   24,  283,  284,   -1,   -1,   -1,  288,
			   -1,   -1,  291,  292,   -1,   -1,  295,   38,   -1,   40,
			   -1,  300,   -1,   -1,   -1,  304,   -1,   -1,   -1,   -1,
			   51,   -1,   53,   -1,   55,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   68,   -1,   70,
			   -1,   -1,   -1,   -1,   -1,   76,   -1,   -1,   79,   -1,
			   -1,   82,   -1,   -1,   -1,   -1,   87,   88,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			  101,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  110,

			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,  132,   -1,   -1,  135,   -1,  137,   -1,   -1,   -1,
			   -1,  142,  143,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,  156,   -1,  158,   -1,   -1,
			  161,  162,   -1,  164,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,  172,   -1,   -1,  175,   -1,   -1,  178,   -1,  180,
			   -1,   -1,   -1,  184,  185,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,  193,  194,   -1,   -1,   -1,  198,  199,   -1,
			   -1,   -1,   -1,   -1,   -1,  206,   -1,   -1,   -1,   -1,

			   -1,   -1,   -1,   -1,   -1,   -1,  217,  218,  219,   -1,
			   -1,  222,   -1,  224,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,  233,   -1,   -1,   -1,   -1,  238,   -1,   -1,
			   -1,   -1,   -1,  244,   -1,   -1,   -1,  248,   -1,  250,
			   -1,   -1,  253,  254,   24,   -1,  257,   -1,   -1,   -1,
			   -1,   -1,   -1,  264,   -1,  266,   -1,   -1,   38,   -1,
			   40,   -1,  273,   -1,  275,   45,  277,   -1,  279,  280,
			   -1,   51,   -1,   53,   -1,   55,   -1,  288,   -1,   -1,
			  291,   -1,   -1,   -1,  295,   -1,   -1,   -1,   68,  300,
			   70,   -1,   -1,  304,   -1,   -1,   76,   -1,   -1,   79,

			   -1,   -1,   82,   -1,   -1,   -1,   -1,   87,   88,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,  101,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			  110,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,  132,   -1,   -1,  135,   -1,  137,   -1,   -1,
			   -1,   -1,  142,  143,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,  156,   -1,   -1,   -1,
			   -1,  161,  162,   -1,  164,   -1,  166,   -1,   -1,   -1,
			   -1,   -1,  172,   -1,   -1,  175,   -1,   -1,  178,   -1,

			  180,   -1,   -1,   -1,  184,  185,   -1,   -1,   -1,   -1,
			  190,   -1,   -1,  193,  194,   -1,   -1,   -1,  198,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,  206,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,  217,  218,  219,
			   -1,   -1,  222,   -1,  224,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,  233,   -1,   -1,   -1,   -1,  238,   -1,
			   -1,   -1,   -1,   -1,  244,   -1,   -1,   -1,  248,   -1,
			  250,   -1,   -1,   -1,  254,   24,   -1,  257,   -1,   -1,
			   -1,   -1,   -1,   -1,  264,   -1,  266,   -1,   -1,   38,
			   -1,   40,   -1,  273,   -1,  275,   -1,  277,   -1,  279,

			  280,   -1,   51,   -1,   53,   -1,   55,   -1,  288,   -1,
			   -1,  291,   -1,   -1,   -1,  295,   -1,   -1,   -1,   68,
			  300,   70,   -1,   -1,  304,   -1,   -1,   76,   -1,   -1,
			   79,   -1,   -1,   82,   -1,   -1,   -1,   -1,   87,   88,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,  101,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,  110,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,  132,   -1,   -1,  135,   -1,  137,   -1,
			   -1,   -1,   -1,  142,  143,   -1,   -1,   -1,   -1,   -1,

			   -1,   -1,   -1,   -1,   -1,   -1,   -1,  156,   -1,   -1,
			   -1,   -1,  161,  162,   -1,  164,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,  172,   -1,   -1,  175,   -1,  177,  178,
			   -1,  180,  181,   -1,   -1,  184,  185,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,  193,  194,   -1,   -1,   -1,  198,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,  206,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  217,  218,
			  219,   -1,   -1,  222,   -1,  224,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,  233,   -1,   -1,   -1,   -1,  238,
			   -1,   -1,   -1,   -1,   -1,  244,   -1,   -1,   -1,  248,

			  249,  250,   -1,   -1,   -1,  254,   24,   -1,  257,   -1,
			   -1,   -1,   -1,   -1,   -1,  264,   -1,  266,   -1,   -1,
			   38,   -1,   40,   -1,  273,   -1,  275,   -1,  277,   -1,
			  279,  280,   -1,   51,   -1,   53,   -1,   55,   -1,  288,
			   -1,   -1,  291,   -1,   -1,   -1,  295,   -1,   -1,   -1,
			   68,  300,   70,   -1,   -1,  304,   -1,   -1,   76,   -1,
			   -1,   79,   -1,   -1,   82,   -1,   -1,   -1,   -1,   87,
			   88,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,  101,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,  110,   -1,   -1,   -1,   -1,   -1,   -1,   -1,

			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,  132,   -1,   -1,  135,   -1,  137,
			   -1,   -1,   -1,   -1,  142,  143,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  156,   -1,
			   -1,   -1,   -1,  161,  162,   -1,  164,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,  172,   -1,   -1,  175,   -1,   -1,
			  178,   -1,  180,   -1,   -1,   -1,  184,  185,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,  193,  194,   -1,   -1,   -1,
			  198,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  206,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  217,

			  218,  219,   -1,   -1,  222,   -1,  224,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,  233,   -1,   -1,   -1,   -1,
			  238,   -1,   -1,   -1,   -1,   -1,  244,   -1,   -1,   -1,
			  248,   -1,  250,   -1,   -1,   -1,  254,   -1,   -1,  257,
			   -1,   -1,   -1,   -1,   -1,   -1,  264,   -1,  266,   -1,
			   -1,   -1,   -1,   -1,   -1,  273,   -1,  275,   24,  277,
			   -1,  279,  280,   -1,   -1,  283,  284,   -1,   -1,   -1,
			  288,   -1,   38,  291,   40,   -1,   -1,  295,   -1,   -1,
			   -1,   -1,  300,   49,   -1,   51,  304,   53,   -1,   55,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, yyDummy>>,
			1, 1000, 27000)
		end

	yycheck_template_29 (an_array: ARRAY [INTEGER])
			-- Fill chunk #29 of template for `yycheck'.
		do
			yyarray_subcopy (an_array, <<
			   -1,   -1,   68,   -1,   70,   -1,   -1,   -1,   -1,   -1,
			   76,   -1,   -1,   79,   -1,   -1,   82,   -1,   -1,   -1,
			   -1,   87,   88,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,  101,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,  110,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,  132,   -1,   -1,  135,
			   -1,  137,   -1,   -1,   -1,   -1,  142,  143,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			  156,   -1,   -1,   -1,   -1,  161,  162,   -1,  164,   -1,

			   -1,   -1,   -1,   -1,   -1,   -1,  172,   -1,   -1,  175,
			   -1,   -1,  178,   -1,  180,   -1,   -1,   -1,  184,  185,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,  193,  194,   -1,
			   -1,   -1,  198,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			  206,   -1,   -1,   -1,   -1,   -1,   -1,  213,   -1,   -1,
			   -1,  217,  218,  219,   -1,   -1,  222,   -1,  224,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,  233,   -1,   -1,
			   -1,   -1,  238,   -1,   -1,   -1,   -1,   -1,  244,   -1,
			   -1,   -1,  248,   -1,  250,   -1,   -1,   -1,  254,   24,
			   -1,  257,   -1,   -1,   -1,   -1,   -1,   -1,  264,   -1,

			  266,   -1,   -1,   38,   -1,   40,   -1,  273,   -1,  275,
			   -1,  277,   -1,  279,  280,   -1,   51,   -1,   53,   -1,
			   55,   -1,  288,   -1,   -1,  291,   -1,   -1,   -1,  295,
			   -1,   -1,   -1,   68,  300,   70,   -1,   -1,  304,   -1,
			   -1,   76,   -1,   -1,   79,   -1,   -1,   82,   -1,   -1,
			   -1,   -1,   87,   88,   -1,   90,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,  101,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,  110,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,  128,   -1,   -1,   -1,  132,   -1,   -1,

			  135,   -1,  137,   -1,   -1,   -1,   -1,  142,  143,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,  156,   -1,   -1,   -1,   -1,  161,  162,   -1,  164,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,  172,   -1,   -1,
			  175,   -1,   -1,  178,   -1,  180,   -1,   -1,   -1,  184,
			  185,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  193,  194,
			   -1,   -1,   -1,  198,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,  206,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,  217,  218,  219,   -1,   -1,  222,   -1,  224,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  233,   -1,

			   -1,   -1,   -1,  238,   -1,   -1,   -1,   -1,   -1,  244,
			   -1,   -1,   -1,  248,   -1,  250,   -1,   -1,   -1,  254,
			   -1,   -1,  257,   -1,   -1,   -1,   -1,   -1,   -1,  264,
			   -1,  266,   24,   -1,   -1,   -1,   -1,   -1,  273,   -1,
			  275,   -1,  277,   -1,  279,  280,   38,   -1,   40,   -1,
			   -1,   -1,   -1,  288,   -1,   -1,  291,   49,   -1,   51,
			  295,   53,   -1,   55,   -1,  300,   -1,   -1,   -1,  304,
			   -1,   -1,   -1,   -1,   -1,   -1,   68,   -1,   70,   -1,
			   -1,   -1,   -1,   -1,   76,   -1,   -1,   79,   -1,   -1,
			   82,   -1,   -1,   -1,   -1,   87,   88,   -1,   -1,   -1,

			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  101,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  110,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			  132,   -1,   -1,  135,   -1,  137,   -1,   -1,   -1,   -1,
			  142,  143,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,  156,   -1,   -1,   -1,   -1,  161,
			  162,   -1,  164,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			  172,   -1,   -1,  175,   -1,   -1,  178,   -1,  180,   -1,
			   -1,   -1,  184,  185,   -1,   -1,   -1,   -1,   -1,   -1,

			   -1,  193,  194,   -1,   -1,   -1,  198,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,  206,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,  217,  218,  219,   -1,   -1,
			  222,   -1,  224,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,  233,   -1,   -1,   -1,   -1,  238,   -1,   -1,   -1,
			   -1,   -1,  244,   -1,   -1,   -1,  248,   -1,  250,   -1,
			   -1,   -1,  254,   24,   -1,  257,   -1,   -1,   -1,   -1,
			   -1,   -1,  264,   -1,  266,   -1,   -1,   38,   -1,   40,
			   -1,  273,   -1,  275,   -1,  277,  278,  279,  280,   -1,
			   51,   -1,   53,   -1,   55,   -1,  288,   -1,   -1,  291,

			   -1,   -1,   -1,  295,   -1,   -1,   -1,   68,  300,   70,
			   -1,   -1,  304,   -1,   -1,   76,   -1,   -1,   79,   -1,
			   -1,   82,   -1,   -1,   -1,   -1,   87,   88,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			  101,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  110,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,  132,   -1,   -1,  135,   -1,  137,   -1,  139,   -1,
			   -1,  142,  143,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,  156,   -1,   -1,   -1,   -1,

			  161,  162,   -1,  164,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,  172,   -1,   -1,  175,   -1,   -1,  178,   -1,  180,
			   -1,   -1,   -1,  184,  185,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,  193,  194,   -1,   -1,   -1,  198,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,  206,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,  217,  218,  219,   -1,
			   -1,  222,   -1,  224,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,  233,   -1,   -1,   -1,   -1,  238,   -1,   -1,
			   -1,   -1,   -1,  244,   -1,   -1,   -1,  248,   -1,  250,
			   -1,   -1,   -1,  254,   24,   -1,  257,   -1,   -1,   -1,

			   -1,   -1,   -1,  264,   -1,  266,   -1,   -1,   38,   -1,
			   40,   -1,  273,   -1,  275,   -1,  277,   -1,  279,  280,
			   -1,   51,   -1,   53,   -1,   55,   -1,  288,   -1,   -1,
			  291,   -1,   -1,   -1,  295,   -1,   -1,  298,   68,  300,
			   70,   -1,   -1,  304,   -1,   -1,   76,   -1,   -1,   79,
			   -1,   -1,   82,   -1,   -1,   -1,   -1,   87,   88,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,  101,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			  110,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1, yyDummy>>,
			1, 1000, 28000)
		end

	yycheck_template_30 (an_array: ARRAY [INTEGER])
			-- Fill chunk #30 of template for `yycheck'.
		do
			yyarray_subcopy (an_array, <<
			   -1,   -1,  132,   -1,   -1,  135,   -1,  137,   -1,   -1,
			   -1,   -1,  142,  143,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,  156,   -1,   -1,   -1,
			   -1,  161,  162,   -1,  164,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,  172,   -1,   -1,  175,   -1,   -1,  178,   -1,
			  180,   -1,   -1,   -1,  184,  185,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,  193,  194,   -1,   -1,   -1,  198,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,  206,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,  217,  218,  219,
			  220,   -1,  222,   -1,  224,   -1,   -1,   -1,   -1,   -1,

			   -1,   -1,   -1,  233,   -1,   -1,   -1,   -1,  238,   -1,
			   -1,   -1,   -1,   -1,  244,   -1,   -1,   -1,  248,   -1,
			  250,   -1,   -1,   -1,  254,   -1,   -1,  257,   -1,   -1,
			   -1,   -1,   -1,   -1,  264,   -1,  266,   -1,   -1,   -1,
			   -1,   -1,   -1,  273,   -1,  275,   24,  277,   -1,  279,
			  280,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  288,   -1,
			   38,  291,   40,   -1,   -1,  295,   -1,   -1,  298,   -1,
			  300,   49,   -1,   51,  304,   53,   -1,   55,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   68,   -1,   70,   -1,   -1,   -1,   -1,   -1,   76,   -1,

			   -1,   79,   -1,   -1,   82,   -1,   -1,   -1,   -1,   87,
			   88,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,  101,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,  110,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,  132,   -1,   -1,  135,   -1,  137,
			   -1,   -1,   -1,   -1,  142,  143,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  156,   -1,
			   -1,   -1,   -1,  161,  162,   -1,  164,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,  172,   -1,   -1,  175,   -1,   -1,

			  178,   -1,  180,   -1,   -1,   -1,  184,  185,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,  193,  194,   -1,   -1,   -1,
			  198,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  206,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  217,
			  218,  219,   -1,   -1,  222,   -1,  224,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,  233,   -1,   -1,   -1,   -1,
			  238,   -1,   -1,   -1,   -1,   -1,  244,   -1,   -1,   -1,
			  248,   -1,  250,   -1,   -1,   -1,  254,   24,   -1,  257,
			   -1,   -1,   -1,   -1,   -1,   -1,  264,   -1,  266,   -1,
			   -1,   38,   -1,   40,   -1,  273,   -1,  275,   -1,  277,

			   -1,  279,  280,   -1,   51,   -1,   53,   -1,   55,   -1,
			  288,   -1,   -1,  291,   -1,   -1,   -1,  295,   -1,   -1,
			   -1,   68,  300,   70,   -1,   -1,  304,   -1,   -1,   76,
			   -1,   -1,   79,   -1,   -1,   82,   -1,   -1,   -1,   -1,
			   87,   88,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,  101,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,  110,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,  130,   -1,  132,   -1,   -1,  135,   -1,
			  137,   -1,   -1,   -1,   -1,  142,  143,   -1,   -1,   -1,

			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  156,
			   -1,   -1,   -1,   -1,  161,  162,   -1,  164,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,  172,   -1,   -1,  175,   -1,
			   -1,  178,   -1,  180,   -1,   -1,   -1,  184,  185,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,  193,  194,   -1,   -1,
			   -1,  198,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  206,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			  217,  218,  219,   -1,   -1,  222,   -1,  224,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,  233,   -1,   -1,   -1,
			   -1,  238,   -1,   -1,   -1,   -1,   -1,  244,   -1,   -1,

			   -1,  248,   -1,  250,   -1,   -1,   -1,  254,   -1,   -1,
			  257,   -1,   -1,   -1,   -1,   -1,   -1,  264,   -1,  266,
			   24,   -1,   -1,   -1,   -1,   -1,  273,   -1,  275,   -1,
			  277,   -1,  279,  280,   38,   -1,   40,   -1,   -1,   -1,
			   -1,  288,   -1,   -1,  291,   49,   -1,   51,  295,   53,
			   -1,   55,   -1,  300,   -1,   -1,   -1,  304,   -1,   -1,
			   -1,   -1,   -1,   -1,   68,   -1,   70,   -1,   -1,   -1,
			   -1,   -1,   76,   -1,   -1,   79,   -1,   -1,   82,   -1,
			   -1,   -1,   -1,   87,   88,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,  101,   -1,   -1,

			   -1,   -1,   -1,   -1,   -1,   -1,  110,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  132,   -1,
			   -1,  135,   -1,  137,   -1,   -1,   -1,   -1,  142,  143,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,  156,   -1,   -1,   -1,   -1,  161,  162,   -1,
			  164,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  172,   -1,
			   -1,  175,   -1,   -1,  178,   -1,  180,   -1,   -1,   -1,
			  184,  185,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  193,
			  194,   -1,   -1,   -1,  198,   -1,   -1,   -1,   -1,   -1,

			   -1,   -1,  206,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,  217,  218,  219,   -1,   -1,  222,   -1,
			  224,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  233,
			   -1,   -1,   -1,   -1,  238,   -1,   -1,   -1,   -1,   -1,
			  244,   -1,   -1,   -1,  248,   -1,  250,   -1,   -1,   -1,
			  254,   24,   -1,  257,   -1,   -1,   -1,   -1,   -1,   -1,
			  264,   -1,  266,   -1,   -1,   38,   -1,   40,   -1,  273,
			   -1,  275,   -1,  277,   -1,  279,  280,   -1,   51,   -1,
			   53,   -1,   55,   -1,  288,   -1,   -1,  291,   -1,   -1,
			   -1,  295,   -1,   -1,   -1,   68,  300,   70,   -1,   -1,

			  304,   -1,   -1,   76,   -1,   -1,   79,   -1,   -1,   82,
			   -1,   -1,   -1,   -1,   87,   88,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  101,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,  110,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  132,
			   -1,   -1,  135,   -1,  137,   -1,   -1,   -1,   -1,  142,
			  143,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,  156,   -1,   -1,   -1,   -1,  161,  162,
			   -1,  164,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  172, yyDummy>>,
			1, 1000, 29000)
		end

	yycheck_template_31 (an_array: ARRAY [INTEGER])
			-- Fill chunk #31 of template for `yycheck'.
		do
			yyarray_subcopy (an_array, <<
			   -1,   -1,  175,   -1,   -1,  178,   -1,  180,   -1,   -1,
			   -1,  184,  185,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			  193,  194,   -1,   -1,   -1,  198,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,  206,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,  217,  218,  219,   -1,   -1,  222,
			   -1,  224,   -1,   -1,  227,   -1,   -1,   -1,   -1,   -1,
			  233,   -1,   -1,   -1,   -1,  238,   -1,   -1,   -1,   -1,
			   -1,  244,   -1,   -1,   -1,  248,   -1,  250,   -1,   -1,
			   -1,  254,   24,   -1,  257,   -1,   -1,   -1,   -1,   -1,
			   -1,  264,   -1,  266,   -1,   -1,   38,   -1,   40,   -1,

			  273,   -1,  275,   -1,  277,   -1,  279,  280,   -1,   51,
			   -1,   53,   -1,   55,   -1,  288,   -1,   -1,  291,   -1,
			   -1,   -1,  295,   -1,   -1,   -1,   68,  300,   70,   -1,
			   -1,  304,   -1,   -1,   76,   -1,   -1,   79,   -1,   -1,
			   82,   -1,   -1,   -1,   -1,   87,   88,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  101,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  110,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			  132,   -1,   -1,  135,   -1,  137,   -1,   -1,   -1,   -1,

			  142,  143,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,  156,   -1,   -1,   -1,   -1,  161,
			  162,   -1,  164,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			  172,   -1,   -1,  175,   -1,   -1,  178,   -1,  180,   -1,
			   -1,   -1,  184,  185,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,  193,  194,   -1,   -1,   -1,  198,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,  206,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,  217,  218,  219,   -1,   -1,
			  222,   -1,  224,   -1,   -1,   -1,   -1,   -1,   -1,  231,
			   -1,  233,   -1,   -1,   -1,   -1,  238,   -1,   -1,   -1,

			   -1,   -1,  244,   -1,   -1,   -1,  248,   -1,  250,   -1,
			   -1,   -1,  254,   -1,   -1,  257,   -1,   -1,   -1,   -1,
			   -1,   -1,  264,   -1,  266,   -1,   -1,   -1,   24,   -1,
			   -1,  273,   -1,  275,   -1,  277,   -1,  279,  280,   -1,
			   -1,   -1,   38,   -1,   40,   -1,  288,   -1,   -1,  291,
			   -1,   -1,   48,  295,   -1,   51,   -1,   53,  300,   55,
			   -1,   -1,  304,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   68,   -1,   70,   -1,   -1,   -1,   -1,   -1,
			   76,   -1,   -1,   79,   -1,   -1,   82,   -1,   -1,   -1,
			   -1,   87,   88,   -1,   -1,   -1,   -1,   -1,   -1,   -1,

			   -1,   -1,   -1,   -1,   -1,  101,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,  110,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,  132,   -1,   -1,  135,
			   -1,  137,   -1,   -1,   -1,   -1,  142,  143,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			  156,   -1,   -1,   -1,   -1,  161,  162,   -1,  164,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,  172,   -1,   -1,  175,
			   -1,   -1,  178,   -1,  180,   -1,   -1,   -1,  184,  185,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,  193,  194,   -1,

			   -1,   -1,  198,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			  206,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,  217,  218,  219,   -1,   -1,  222,   -1,  224,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,  233,   -1,   -1,
			   -1,   -1,  238,   -1,   -1,   -1,   -1,   -1,  244,   -1,
			   -1,   -1,  248,   -1,  250,   -1,   -1,   -1,  254,   24,
			   -1,  257,   -1,   -1,   -1,   -1,   -1,   -1,  264,   -1,
			  266,   -1,   -1,   38,   -1,   40,   -1,  273,   -1,  275,
			   -1,  277,   -1,  279,  280,   -1,   51,   -1,   53,   -1,
			   55,   -1,  288,   -1,   -1,  291,   -1,   -1,   -1,  295,

			   -1,   -1,   -1,   68,  300,   70,   -1,   -1,  304,   -1,
			   -1,   76,   -1,   -1,   79,   -1,   -1,   82,   -1,   -1,
			   -1,   -1,   87,   88,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,  101,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,  110,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,  132,   -1,   -1,
			  135,   -1,  137,   -1,  139,   -1,   -1,  142,  143,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,  156,   -1,   -1,   -1,   -1,  161,  162,   -1,  164,

			   -1,   -1,   -1,   -1,   -1,   -1,   -1,  172,   -1,   -1,
			  175,   -1,   -1,  178,   -1,  180,   -1,   -1,   -1,  184,
			  185,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  193,  194,
			   -1,   -1,   -1,  198,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,  206,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,  217,  218,  219,   -1,   -1,  222,   -1,  224,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  233,   -1,
			   -1,   -1,   -1,  238,   -1,   -1,   -1,   -1,   -1,  244,
			   -1,   -1,   -1,  248,   -1,  250,   -1,   -1,   -1,  254,
			   24,   -1,  257,   -1,   -1,   -1,   -1,   -1,   -1,  264,

			   -1,  266,   -1,   -1,   38,   -1,   40,   -1,  273,   -1,
			  275,   -1,  277,   -1,  279,  280,   -1,   51,   -1,   53,
			   -1,   55,   -1,  288,   -1,   -1,  291,   -1,   -1,   -1,
			  295,   -1,   -1,   -1,   68,  300,   70,   -1,   -1,  304,
			   -1,   -1,   76,   -1,   -1,   79,   -1,   -1,   82,   -1,
			   -1,   -1,   -1,   87,   88,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,  101,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,  110,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  132,   -1,

			   -1,  135,   -1,  137,   -1,   -1,   -1,   -1,  142,  143,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,  156,   -1,   -1,   -1,   -1,  161,  162,   -1,
			  164,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  172,   -1,
			   -1,  175,   -1,   -1,  178,   -1,  180,  181,   -1,   -1,
			  184,  185,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  193,
			  194,   -1,   -1,   -1,  198,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,  206,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,  217,  218,  219,   -1,   -1,  222,   -1,
			  224,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  233, yyDummy>>,
			1, 1000, 30000)
		end

	yycheck_template_32 (an_array: ARRAY [INTEGER])
			-- Fill chunk #32 of template for `yycheck'.
		do
			yyarray_subcopy (an_array, <<
			   -1,   -1,   -1,   -1,  238,   -1,   -1,   -1,   -1,   -1,
			  244,   -1,   -1,   -1,  248,   -1,  250,   -1,   -1,   -1,
			  254,   24,   -1,  257,   -1,   -1,   -1,   -1,   -1,   -1,
			  264,   -1,  266,   -1,   -1,   38,   -1,   40,   -1,  273,
			   -1,  275,   -1,  277,   -1,  279,  280,   -1,   51,   -1,
			   53,   -1,   55,   -1,  288,   -1,   -1,  291,   -1,   -1,
			   -1,  295,   -1,   -1,   -1,   68,  300,   70,   -1,   -1,
			  304,   -1,   -1,   76,   -1,   -1,   79,   -1,   -1,   82,
			   -1,   -1,   -1,   -1,   87,   88,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  101,   -1,

			   -1,   -1,   -1,   -1,   -1,   -1,   -1,  110,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  132,
			   -1,   -1,  135,   -1,  137,   -1,   -1,   -1,   -1,  142,
			  143,   -1,   -1,   -1,  147,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,  156,   -1,   -1,   -1,   -1,  161,  162,
			   -1,  164,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  172,
			   -1,   -1,  175,   -1,   -1,  178,   -1,  180,   -1,   -1,
			   -1,  184,  185,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			  193,  194,   -1,   -1,   -1,  198,   -1,   -1,   -1,   -1,

			   -1,   -1,   -1,  206,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,  217,  218,  219,   -1,   -1,  222,
			   -1,  224,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			  233,   -1,   -1,   -1,   -1,  238,   -1,   -1,   -1,   -1,
			   -1,  244,   -1,   -1,   -1,  248,   -1,  250,   -1,   -1,
			   -1,  254,   24,   -1,  257,   -1,   -1,   -1,   -1,   -1,
			   -1,  264,   -1,  266,   -1,   -1,   38,   -1,   40,   -1,
			  273,   -1,  275,   -1,  277,   -1,  279,  280,   -1,   51,
			   -1,   53,   -1,   55,   -1,  288,   -1,   -1,  291,   -1,
			   -1,   -1,  295,   -1,   -1,   -1,   68,  300,   70,   -1,

			   -1,  304,   -1,   -1,   76,   -1,   -1,   79,   -1,   -1,
			   82,   -1,   -1,   -1,   -1,   87,   88,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  101,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  110,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			  132,   -1,   -1,  135,   -1,  137,   -1,   -1,   -1,   -1,
			  142,  143,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,  156,   -1,   -1,   -1,   -1,  161,
			  162,   -1,  164,   -1,   -1,   -1,   -1,   -1,   -1,   -1,

			  172,   -1,   -1,  175,   -1,   -1,  178,   -1,  180,  181,
			   -1,   -1,  184,  185,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,  193,  194,   -1,   -1,   -1,  198,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,  206,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,  217,  218,  219,   -1,   -1,
			  222,   -1,  224,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,  233,   -1,   -1,   -1,   -1,  238,   -1,   -1,   -1,
			   -1,   -1,  244,   -1,   -1,   -1,  248,   -1,  250,   -1,
			   -1,   -1,  254,   24,   -1,  257,   -1,   -1,   -1,   -1,
			   -1,   -1,  264,   -1,  266,   -1,   -1,   38,   -1,   40,

			   -1,  273,   -1,  275,   -1,  277,   -1,  279,  280,   -1,
			   51,   -1,   53,   -1,   55,   -1,  288,   -1,   -1,  291,
			   -1,   -1,   -1,  295,   -1,   -1,   -1,   68,  300,   70,
			   -1,   -1,  304,   -1,   -1,   76,   -1,   -1,   79,   -1,
			   -1,   82,   -1,   -1,   -1,   -1,   87,   88,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			  101,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  110,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,  132,   -1,   -1,  135,   -1,  137,   -1,   -1,   -1,

			   -1,  142,  143,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,  156,   -1,   -1,   -1,   -1,
			  161,  162,   -1,  164,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,  172,   -1,   -1,  175,   -1,   -1,  178,   -1,  180,
			   -1,   -1,   -1,  184,  185,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,  193,  194,   -1,   -1,   -1,  198,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,  206,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,  217,  218,  219,   -1,
			   -1,  222,   -1,  224,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,  233,   -1,   -1,   -1,   -1,  238,   -1,   -1,

			   -1,   -1,   -1,  244,   -1,   -1,   -1,  248,   -1,  250,
			   -1,   -1,   -1,  254,   24,   -1,  257,   -1,   -1,   -1,
			  261,   -1,   -1,  264,   -1,  266,   -1,   -1,   38,   -1,
			   40,   -1,  273,   -1,  275,   -1,  277,   -1,  279,  280,
			   -1,   51,   -1,   53,   -1,   55,   -1,  288,   -1,   -1,
			  291,   -1,   -1,   -1,  295,   -1,   -1,   -1,   68,  300,
			   70,   -1,   -1,  304,   -1,   -1,   76,   -1,   -1,   79,
			   -1,   -1,   82,   -1,   -1,   -1,   -1,   87,   88,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,  101,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,

			  110,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,  132,   -1,   -1,  135,   -1,  137,   -1,   -1,
			   -1,   -1,  142,  143,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,  156,   -1,   -1,   -1,
			   -1,  161,  162,   -1,  164,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,  172,   -1,   -1,  175,   -1,   -1,  178,   -1,
			  180,   -1,   -1,   -1,  184,  185,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,  193,  194,   -1,   -1,   -1,  198,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,  206,   -1,   -1,   -1,

			   -1,   -1,   -1,   -1,   -1,   -1,   -1,  217,  218,  219,
			   -1,   -1,  222,   -1,  224,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,  233,   -1,   -1,   -1,   -1,  238,   -1,
			   -1,   -1,   -1,   -1,  244,   -1,   -1,   -1,  248,   -1,
			  250,   -1,   -1,   -1,  254,   24,   -1,  257,   -1,   -1,
			   -1,   -1,   -1,   -1,  264,   -1,  266,   -1,   -1,   38,
			   -1,   40,   -1,  273,   -1,  275,   -1,  277,   -1,  279,
			  280,   -1,   51,   -1,   53,   -1,   55,   -1,  288,   -1,
			   -1,  291,   -1,   -1,   -1,  295,   -1,   -1,   -1,   68,
			  300,   70,   -1,   -1,  304,   -1,   -1,   76,   -1,   -1, yyDummy>>,
			1, 1000, 31000)
		end

	yycheck_template_33 (an_array: ARRAY [INTEGER])
			-- Fill chunk #33 of template for `yycheck'.
		do
			yyarray_subcopy (an_array, <<
			   79,   -1,   -1,   82,   -1,   -1,   -1,   -1,   87,   88,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,  101,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,  110,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,  132,   -1,   -1,  135,   -1,  137,   -1,
			   -1,   -1,   -1,  142,  143,   -1,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,  156,   -1,   -1,
			   -1,   -1,  161,  162,   -1,  164,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,  172,   -1,   -1,  175,   -1,   -1,  178,

			   -1,  180,   -1,   -1,   -1,  184,  185,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,  193,  194,   -1,   -1,   -1,  198,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,  206,   -1,   -1,
			   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  217,  218,
			  219,   -1,   -1,  222,   -1,  224,   -1,   -1,   -1,   -1,
			   -1,   -1,   -1,   -1,  233,   -1,   -1,   -1,   -1,  238,
			   -1,   -1,   -1,   -1,   -1,  244,   -1,   -1,   -1,  248,
			   -1,  250,   -1,   -1,   -1,  254,   -1,   -1,  257,   -1,
			   -1,   -1,   -1,   -1,   -1,  264,   -1,  266,   -1,   -1,
			   -1,   -1,   -1,   -1,  273,   -1,  275,   -1,  277,   -1,

			  279,  280,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  288,
			   -1,   -1,  291,   -1,   -1,   -1,  295,   -1,   -1,   -1,
			   -1,  300,   -1,   -1,   -1,  304, yyDummy>>,
			1, 226, 32000)
		end

feature {NONE} -- Semantic value stacks

	yyvs1: SPECIAL [detachable ANY]
			-- Stack for semantic values of type detachable ANY

	yyvsc1: INTEGER
			-- Capacity of semantic value stack `yyvs1'

	yyvsp1: INTEGER
			-- Top of semantic value stack `yyvs1'

	yyspecial_routines1: KL_SPECIAL_ROUTINES [detachable ANY]
			-- Routines that ought to be in SPECIAL [detachable ANY]

	yyvs2: SPECIAL [STRING]
			-- Stack for semantic values of type STRING

	yyvsc2: INTEGER
			-- Capacity of semantic value stack `yyvs2'

	yyvsp2: INTEGER
			-- Top of semantic value stack `yyvs2'

	yyspecial_routines2: KL_SPECIAL_ROUTINES [STRING]
			-- Routines that ought to be in SPECIAL [STRING]

feature {NONE} -- Constants

	yyFinal: INTEGER = 2406
			-- Termination state id

	yyFlag: INTEGER = -32768
			-- Most negative INTEGER

	yyNtbase: INTEGER = 314
			-- Number of tokens

	yyLast: INTEGER = 32225
			-- Upper bound of `yytable' and `yycheck'

	yyMax_token: INTEGER = 554
			-- Maximum token id
			-- (upper bound of `yytranslate'.)

	yyNsyms: INTEGER = 691
			-- Number of symbols
			-- (terminal and nonterminal)

feature -- User-defined features



feature {NONE} -- Initialization

	make
			-- Create a new SQL table name parser.
		do
			create input_table_names.make_equal (1)
			create output_table_names.make_equal (1)
			make_sql_table_name_scanner
			make_parser_skeleton
		end

	execute
			-- Parse SQL statement passed as argument.
		do
			make
			set_default_error_action
			parse_sql_statement (Arguments.argument (1))
			std.output.put_line ("Input tables:")
			input_table_names.do_all (agent (std.output).put_line)
			std.output.put_line ("Onput tables:")
			output_table_names.do_all (agent (std.output).put_line)
		end

feature -- Access

	sql_statement: STRING
			-- SQL statement being parsed
			
	input_table_names: DS_HASH_SET [attached STRING]
			-- Names of tables (to be accessed as input) found in last SQL statement parsed

	output_table_names: DS_HASH_SET [attached STRING]
			-- Names of tables (to be accessed as output) found in last SQL statement parsed			
			
feature -- Basic operations

	parse_sql_statement (a_sql_statement: STRING)
			-- Parse SQL query.
		require
			a_sql_statement_not_void: a_sql_statement /= Void
		do
			reset
			input_table_names.wipe_out
			output_table_names.wipe_out
			set_input_buffer (new_string_buffer (a_sql_statement))
			sql_statement := a_sql_statement
			parse
			sql_statement := Void
		end

feature -- Error handling

	error_action: PROCEDURE [ANY, TUPLE [STRING]]
			-- Action to be executed when an error is reported by the parser
			
	set_error_action (a_action: like error_action)
			-- Set `error_action' to `a_action'.
		do
			error_action := a_action
		ensure
			error_action_set: error_action = a_action
		end
		
	set_default_error_action
			-- Set `error_action' to the default error action.
		do
			set_error_action (agent default_report_error)
		end
		
feature {YY_PARSER_ACTION} -- Error handling

	report_error (a_message: STRING)
			-- Report that an error occurred.
		do
			if error_action /= Void then
				error_action.call ([a_message])
			end
		end

	default_report_error (a_message: STRING)
			-- Print error message.
		do
			std.error.put_string (a_message)
			if sql_statement /= Void then
				std.error.put_string (": ")
				std.error.put_string (sql_statement)
			end
			std.error.put_new_line
		end
		
	fatal_error (a_message: STRING)
			-- A fatal error occurred.
		do
			report_error (a_message)
		end
		
invariant

	input_table_names_not_void: input_table_names /= Void
	output_table_names_not_void: output_table_names /= Void
	
end
