note

	description:
	"[
		Variable descriptions (to add a new variable record follow the procedure set out below)
		
				1) Find an unused variable_index.
						Do not try to reuse an apparently unused index.  We have plenty of new ones.
						The highest used variable index should be at the top of this class.
						Add one to the found index and search this class for that number (just in case).
						Coordinate with anyone else who has this class checked out.
				2) Add your 'once' function.
						Usually we add new functions to the TOP of the class text 
						since that will put the highest index first.
						Try to find an existing function that is similar to yours and use it as a template.
						There are several 'make's in VARIABLE_DESCRIPTION to choose from 
						(e.g. 'make_mondat', 'make_risk').
				3) Check in this class now - VERY IMPORTANT.
						Do not wait to check in this class.  You need to reserve the new variable_index now.
						If checking in will cause problems, comment out your code and check in anyway.
				4) Do nothing after check in (check with STIT to see if this applies).
						DO NOT ADD YOUR RECORD TO THE DATABASE!
						After you check in, the variable_definition_replication program will copy your new data to
						both the variable_description_table (which you probably don't use).
		Finally, you should create a variable which references your new descriptor.  See:
			COMPUTED_VARIABLES
			IBES_VARIABLES
			INTERMEDIATE_VARIABLES
			MISCELLANEOUS_COMPANY_VARIABLES
			MODEL_COMPANY_VARIABLES
			MODEL_LISTING_VARIABLES
			MODEL_SECURITY_VARIABLES
			MONDAT_INDUSTRY_VARIABLES
			MONDEX_VARIABLES
			RESEARCH_COMPANY_VARIABLES
			RISK_VARIABLES
			RVAD_VARIABLES
			SECURITY_VARIABLES
			VALDATA_VARIABLES
	]"
	author: "Pierre Nallet"
	last_editor: "$Author: it $"
	date: "$Date: 2015/04/27 $"
	revision: "$Revision: #115 $"
	path: "$File: //rose/source/kernel/expressions/variable_descriptions.e $"

class VARIABLE_DESCRIPTIONS

inherit

	VARIABLE_DESCRIPTION_CONSTANTS
		export {NONE} all end

	ROSE_CONSTANT_VALUES

feature -- Access

	xtp_earn: VARIABLE_DESCRIPTION_RECORD
			-- Extrapolate book value from fiscal date using earnings
		once
			if attached (create {STRING}.make_from_string ("xtp_earn")) as la_mnemonic then
				create Result.make_risk (la_mnemonic, la_mnemonic, la_mnemonic, k_risk, null_integer, null_integer, 2, k_currency, -3.0, 3.0, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", k_risk, 2477)
			end
		ensure
			not_void: Result /= Void
		end

	xtp_div: VARIABLE_DESCRIPTION_RECORD
			-- Extrapolate book value from fiscal date using dividends
		once
			if attached (create {STRING}.make_from_string ("xtp_div")) as la_mnemonic then
				create Result.make_risk (la_mnemonic, la_mnemonic, la_mnemonic, k_risk, null_integer, null_integer, 2, k_currency, -3.0, 3.0, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", k_risk, 2476)
			end
		ensure
			not_void: Result /= Void
		end

	book_value: VARIABLE_DESCRIPTION_RECORD
			-- Total assests less total liabilities
		once
			if attached (create {STRING}.make_from_string ("book_value")) as la_mnemonic then
				create Result.make_risk (la_mnemonic, la_mnemonic, la_mnemonic, k_risk, null_integer, null_integer, 2, k_currency, -3.0, 3.0, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", k_risk, 2475)
			end
		ensure
			not_void: Result /= Void
		end

	fiscal_book: VARIABLE_DESCRIPTION_RECORD
			-- Company book value as of fiscsal report date, before mondat book and dividend extrapolation
		once
			if attached (create {STRING}.make_from_string ("fiscal_book")) as la_mnemonic then
				create Result.make_risk (la_mnemonic, la_mnemonic, la_mnemonic, k_risk, null_integer, null_integer, 2, k_currency, -3.0, 3.0, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", k_risk, 2474)
			end
		ensure
			not_void: Result /= Void
		end

	dems: VARIABLE_DESCRIPTION_RECORD is
			-- Similars Normalized: Delta Earnings conditioned on Profit Margin
		once
			create Result.make_risk ("depm_sim_norm", "dems", "DEPM.SMN", k_risk, null_integer, null_integer, 2, "", -3.0, 3.0, missing_double, missing_double, "SIMNORM:DEPM", Void, Void, ' ', k_monthly_data, "", k_risk, 2473)
		ensure
			not_void: Result /= Void
		end

	dpms: VARIABLE_DESCRIPTION_RECORD is
			-- Similars Normalized: Expected delta profit margin
		once
			create Result.make_risk ("dpmh_sim_norm", "dpms", "DPMH.SMN", k_risk, null_integer, null_integer, 2, "", -3.0, 3.0, missing_double, missing_double, "SIMNORM:DPMH", Void, Void, ' ', k_monthly_data, "", k_risk, 2472)
		ensure
			not_void: Result /= Void
		end

	galpha_all_zero: VARIABLE_DESCRIPTION_RECORD is
			-- Constant set to 0 for consumption by optimization
		once
			create Result.make_model_company ("galpha_all_zero", "galpha_all_zero", 0, "", k_double, ' ', Void, 2471)
		ensure
			not_void: Result /= Void
		end

	grof: VARIABLE_DESCRIPTION_RECORD is
			-- Growth exposure factor
		once
			create Result.make_model_company ("growth_exposure_factor", "grof", 0, "", k_double, ' ', Void, 2470)
		ensure
			grof_attached: Result /= Void
		end

	cfrs: VARIABLE_DESCRIPTION_RECORD is
			-- Shrunk CFSR
		once
			create Result.make_risk ("cfrs", "cfrs", "cfrs", k_risk, null_integer, null_integer, 2, Void, -5.0, 5.0, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", k_risk, 2469)
		ensure
			not_void: Result /= Void
		end

	cfss: VARIABLE_DESCRIPTION_RECORD is
			-- Shrunk CFSN
		once
			create Result.make_risk ("cfss", "cfss", "cfss", k_risk, null_integer, null_integer, 2, Void, -5.0, 5.0, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", k_risk, 2468)
		ensure
			not_void: Result /= Void
		end

	cars: VARIABLE_DESCRIPTION_RECORD is
			-- Shrunk CALR
		once
			create Result.make_risk ("cars", "cars", "cars", k_risk, null_integer, null_integer, 2, Void, -5.0, 5.0, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", k_risk, 2467)
		ensure
			not_void: Result /= Void
		end

	cals: VARIABLE_DESCRIPTION_RECORD is
			-- Shrunk CALF
		once
			create Result.make_risk ("cals", "cals", "cals", k_risk, null_integer, null_integer, 2, Void, -5.0, 5.0, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", k_risk, 2466)
		ensure
			not_void: Result /= Void
		end

	atop_shrinkage: VARIABLE_DESCRIPTION_RECORD is
			-- Assets to Price shrinkage factor
		once
			create Result.make_risk ("atop_shrinkage", "atop_shrinkage", "atop_shrinkage", k_risk, null_integer, null_integer, 2, Void, -5.0, 5.0, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", k_risk, 2465)
		ensure
			not_void: Result /= Void
		end

	a_p: VARIABLE_DESCRIPTION_RECORD is
			-- Assets to Price zone normalized
		once
			create Result.make_risk ("a_p", "a_p", "a_p", k_risk, null_integer, null_integer, 2, Void, -5.0, 5.0, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", k_risk, 2464)
		ensure
			not_void: Result /= Void
		end

	atop: VARIABLE_DESCRIPTION_RECORD is
			-- Assets to Price
		once
			create Result.make_risk ("atop", "atop", "atop", k_risk, null_integer, null_integer, 2, Void, -5.0, 5.0, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", k_risk, 2463)
		ensure
			not_void: Result /= Void
		end

	galpha_emer_small_v1: VARIABLE_DESCRIPTION_RECORD is
			-- Global Emerging Small Cap factor alpha
		once
			create Result.make_model_company ("galpha_emer_small_v1", "galpha_emer_small_v1", 0, "", k_double, ' ', Void, 2462)
		ensure
			attached_result: Result /= Void
		end

	galpha_emer_broad_v1: VARIABLE_DESCRIPTION_RECORD is
			-- Global Emerging Broad factor alpha
		once
			create Result.make_model_company ("galpha_emer_broad_v1", "galpha_emer_broad_v1", 0, "", k_double, ' ', Void, 2461)
		ensure
			attached_result: Result /= Void
		end

	global_zonal_cap_sector: VARIABLE_DESCRIPTION_RECORD is
			-- Zonal capitalization sector computed using the global sector bounds
		once
			create Result.make_model_company ("global_zonal_cap_sector", "gisect_zonal", 0, "", k_integer, ' ', Void, 2460)
		ensure
			global_zonal_cap_sector_attached: Result /= Void
		end

	sdep: VARIABLE_DESCRIPTION_RECORD is
			-- Scaled DEPM
		once
			create Result.make_risk ("sdep", "sdep", "sdep", k_risk, null_integer, null_integer, 2, Void, -5.0, 5.0, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", k_risk, 2459)
		ensure
			not_void: Result /= Void
		end

	ilfp: VARIABLE_DESCRIPTION_RECORD is
			-- Short-term operations
		once
			create Result.make_risk ("ilfp", "ilfp", "ilfp", k_risk, null_integer, null_integer, 2, Void, -5.0, 5.0, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", k_risk, 2458)
		ensure
			not_void: Result /= Void
		end

	global_cap_sector: VARIABLE_DESCRIPTION_RECORD is
			-- Global capitalization sector
		once
			create Result.make_model_company ("global_cap_sector", "gisect", 0, "", k_integer, ' ', Void, 2457)
		end

	model_listing_zone: VARIABLE_DESCRIPTION_RECORD is
			-- Zone for model_listing
		once
			create Result.make_model_listing ("model_listing_zone", "model_izone_r", 0, "", k_string, ' ', Void, 2456)
		end

	model_listing_company_handle: VARIABLE_DESCRIPTION_RECORD is
			-- Company handle for a listing
		once
			create Result.make_model_listing ("model_listing_company_handle", "listing_company_handle", 0, "", k_integer, ' ', Void, 2455)
		end

	model_listing_security_handle: VARIABLE_DESCRIPTION_RECORD is
			-- Security handle for a listing
		once
			create Result.make_model_listing ("model_listing_security_handle", "listing_security_handle", 0, "", k_integer, ' ', Void, 2454)
		end

	model_security_company_handle: VARIABLE_DESCRIPTION_RECORD is
			-- Company handle for a security
		once
			create Result.make_model_security ("model_security_company_handle", "security_company_handle", 0, "", k_integer, ' ', Void, 2453)
		end

	model_security_primary_listing_handle: VARIABLE_DESCRIPTION_RECORD is
			-- Primary listing handle for a security
		once
			create Result.make_model_security ("model_security_primary_listing_handle", "security_primary_listing_handle", 0, "", k_integer, ' ', Void, 2452)
		end

	model_company_primary_security_handle: VARIABLE_DESCRIPTION_RECORD is
			-- Primary security handle for a company
		once
			create Result.make_model_company ("model_primary_security_handle", "primary_security_handle", 0, "", k_integer, ' ', Void, 2451)
		end

	galpha_devel_broad_v1: VARIABLE_DESCRIPTION_RECORD is
			-- Global Mega factor alpha
		once
			create Result.make_model_company ("galpha_devel_broad_v1", "galpha_devel_broad_v1", 0, "", k_double, ' ', Void, 2450)
		ensure
			value_growth_u_jbreakout_not_void: Result /= Void
		end

	valpha_risk_adjusted: VARIABLE_DESCRIPTION_RECORD is
			--phat_to_p_anomaly_ratio (risk-adjusted) - used in MODEL current alpha calculation
		once
			create Result.make_model_company ("phat_to_p_anomaly_ratio_risk_adjusted", "valpha_ra", 0, "", k_double, ' ', Void, 2449)
		ensure
			not_void: Result /= Void
		end

	cfcr: VARIABLE_DESCRIPTION_RECORD is
			-- current alpha risk adjusted with country normalization
		once
			create Result.make_risk ("current_alpha_risk_adjusted_country_normalized", "cfcr", "CFCR", k_risk, null_integer, null_integer, 2, "", -5, 5, missing_double, missing_double, "", Void, Void, '0', k_monthly_data, "", k_risk, 2448)
		ensure
			not_void: Result /= Void
		end

	fgut: VARIABLE_DESCRIPTION_RECORD is
			-- Growth "Unavailable Targets" variable
		once
			create Result.make_mondat ("fgut", "fgut", "", "", null_integer, null_integer, null_integer, null_integer, 1, "", k_integer, Void, Void, k_company, ' ', k_monthly_data, "", k_mondat, 2447)
		ensure
			not_void: Result /= Void
		end

	vver: VARIABLE_DESCRIPTION_RECORD is
			-- 5-year coefficient of variation of VALDATA Earnings per share
		once
			create Result.make_risk ("vver", "vver", "vver", k_risk, null_integer, null_integer, 2, Void, -5.0, 5.0, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", k_risk, 2446)
		ensure
			not_void: Result /= Void
		end

	vsps: VARIABLE_DESCRIPTION_RECORD is
			-- 5-year coefficient of variation of Sales per share
		once
			create Result.make_risk ("vsps", "vsps", "vsps", k_risk, null_integer, null_integer, 2, Void, -5.0, 5.0, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", k_risk, 2445)
		ensure
			not_void: Result /= Void
		end

	vsal: VARIABLE_DESCRIPTION_RECORD is
			-- 5-year coefficient of variation of Sales
		once
			create Result.make_risk ("vsal", "vsal", "vsal", k_risk, null_integer, null_integer, 2, Void, -5.0, 5.0, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", k_risk, 2444)
		ensure
			not_void: Result /= Void
		end

	veps: VARIABLE_DESCRIPTION_RECORD is
			-- 5-year coefficient of variation of Earnings per share
		once
			create Result.make_risk ("veps", "veps", "veps", k_risk, null_integer, null_integer, 2, Void, -5.0, 5.0, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", k_risk, 2443)
		ensure
			not_void: Result /= Void
		end

	trex: VARIABLE_DESCRIPTION_RECORD is
			-- 5-year t-stat of recurring earnings per share
		once
			create Result.make_risk ("trex", "trex", "trex", k_risk, null_integer, null_integer, 2, Void, -5.0, 5.0, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", k_risk, 2442)
		ensure
			not_void: Result /= Void
		end

	tver: VARIABLE_DESCRIPTION_RECORD is
			-- VALDATA Earnings per share 5-year t-stat
		once
			create Result.make_risk ("tver", "tver", "tver", k_risk, null_integer, null_integer, 2, Void, -5.0, 5.0, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", k_risk, 2441)
		ensure
			not_void: Result /= Void
		end

	tsps: VARIABLE_DESCRIPTION_RECORD is
			-- 5-year t-stat of Sales per share
		once
			create Result.make_risk ("tsps", "tsps", "tsps", k_risk, null_integer, null_integer, 2, Void, -5.0, 5.0, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", k_risk, 2440)
		ensure
			not_void: Result /= Void
		end

	tsal: VARIABLE_DESCRIPTION_RECORD is
			-- 5-year t-stat of Sales
		once
			create Result.make_risk ("tsal", "tsal", "tsal", k_risk, null_integer, null_integer, 2, Void, -5.0, 5.0, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", k_risk, 2439)
		ensure
			not_void: Result /= Void
		end

	trog: VARIABLE_DESCRIPTION_RECORD is
			-- Generalized return on equity 5-year t-stat
		once
			create Result.make_risk ("trog", "trog", "trog", k_risk, null_integer, null_integer, 2, Void, -5.0, 5.0, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", k_risk, 2438)
		ensure
			not_void: Result /= Void
		end

	tops: VARIABLE_DESCRIPTION_RECORD is
			-- 5-year t-stat of Operating Income per share
		once
			create Result.make_risk ("tops", "tops", "tops", k_risk, null_integer, null_integer, 2, Void, -5.0, 5.0, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", k_risk, 2437)
		ensure
			not_void: Result /= Void
		end

	toib: VARIABLE_DESCRIPTION_RECORD is
			-- 5-year t-stat of Operating Income Before Depreciation
		once
			create Result.make_risk ("toib", "toib", "toib", k_risk, null_integer, null_integer, 2, Void, -5.0, 5.0, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", k_risk, 2436)
		ensure
			not_void: Result /= Void
		end

	teps: VARIABLE_DESCRIPTION_RECORD is
			-- Earnings per share 5-year t-stat
		once
			create Result.make_risk ("teps", "teps", "teps", k_risk, null_integer, null_integer, 2, Void, -5.0, 5.0, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", k_risk, 2435)
		ensure
			not_void: Result /= Void
		end

	sops: VARIABLE_DESCRIPTION_RECORD is
			-- 5-year sigma of Operating Income per share
		once
			create Result.make_risk ("sops", "sops", "sops", k_risk, null_integer, null_integer, 2, Void, -5.0, 5.0, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", k_risk, 2434)
		ensure
			not_void: Result /= Void
		end

	soib: VARIABLE_DESCRIPTION_RECORD is
			-- 5-year sigma of Operating Income Before Depreciation
		once
			create Result.make_risk ("soib", "soib", "soib", k_risk, null_integer, null_integer, 2, Void, -5.0, 5.0, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", k_risk, 2433)
		ensure
			not_void: Result /= Void
		end

	roe5: VARIABLE_DESCRIPTION_RECORD is
			-- average ROE over the past 5 years
		once
			create Result.make_risk ("roe5", "roe5", "roe5", k_risk, null_integer, null_integer, 2, Void, -5.0, 5.0, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", k_risk, 2432)
		ensure
			not_void: Result /= Void
		end

	grex: VARIABLE_DESCRIPTION_RECORD is
			-- Recurring earnings per share 5-year growth rate
		once
			create Result.make_risk ("grex", "grex", "grex", k_risk, null_integer, null_integer, 2, Void, -5.0, 5.0, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", k_risk, 2431)
		ensure
			not_void: Result /= Void
		end

	gver: VARIABLE_DESCRIPTION_RECORD is
			-- VALDATA Earnings per share 5-year growth rate
		once
			create Result.make_risk ("gver", "gver", "gver", k_risk, null_integer, null_integer, 2, Void, -5.0, 5.0, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", k_risk, 2430)
		ensure
			not_void: Result /= Void
		end

	gsps: VARIABLE_DESCRIPTION_RECORD is
			-- Sales per share 5-year growth rate
		once
			create Result.make_risk ("gsps", "gsps", "gsps", k_risk, null_integer, null_integer, 2, Void, -5.0, 5.0, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", k_risk, 2429)
		ensure
			not_void: Result /= Void
		end

	gsal: VARIABLE_DESCRIPTION_RECORD is
			-- Sales 5-year growth rate
		once
			create Result.make_risk ("gsal", "gsal", "gsal", k_risk, null_integer, null_integer, 2, Void, -5.0, 5.0, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", k_risk, 2428)
		ensure
			not_void: Result /= Void
		end

	grot: VARIABLE_DESCRIPTION_RECORD is
			-- tstat from regression yielding growth rate of recurring earnings per share
		once
			create Result.make_risk ("grot", "grot", "grot", k_risk, null_integer, null_integer, 2, Void, -5.0, 5.0, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", k_risk, 2427)
		ensure
			not_void: Result /= Void
		end

	grog: VARIABLE_DESCRIPTION_RECORD is
			-- Generalized return on equity 5-year growth rate
		once
			create Result.make_risk ("grog", "grog", "grog", k_risk, null_integer, null_integer, 2, Void, -5.0, 5.0, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", k_risk, 2426)
		ensure
			not_void: Result /= Void
		end

	gro5: VARIABLE_DESCRIPTION_RECORD is
			-- growth rate of recurring earnings per share
		once
			create Result.make_risk ("gro5", "gro5", "gro5", k_risk, null_integer, null_integer, 2, Void, -5.0, 5.0, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", k_risk, 2425)
		ensure
			not_void: Result /= Void
		end

	gops: VARIABLE_DESCRIPTION_RECORD is
			-- Operating Income per Share 5-year growth rate
		once
			create Result.make_risk ("gops", "gops", "gops", k_risk, null_integer, null_integer, 2, Void, -5.0, 5.0, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", k_risk, 2424)
		ensure
			not_void: Result /= Void
		end

	goib: VARIABLE_DESCRIPTION_RECORD is
			-- Operating Income Before Depreciation 5-year growth rate
		once
			create Result.make_risk ("goib", "goib", "goib", k_risk, null_integer, null_integer, 2, Void, -5.0, 5.0, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", k_risk, 2423)
		ensure
			not_void: Result /= Void
		end

	geps: VARIABLE_DESCRIPTION_RECORD is
			-- Earnings per share 5-year growth rate
		once
			create Result.make_risk ("geps", "geps", "geps", k_risk, null_integer, null_integer, 2, Void, -5.0, 5.0, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", k_risk, 2422)
		ensure
			not_void: Result /= Void
		end

	etpv: VARIABLE_DESCRIPTION_RECORD is
			-- VALDATA Earnings to Price
		once
			create Result.make_risk ("etpv", "etpv", "etpv", k_risk, null_integer, null_integer, 2, Void, -5.0, 5.0, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", k_risk, 2421)
		ensure
			not_void: Result /= Void
		end

	epsv: VARIABLE_DESCRIPTION_RECORD is
			-- VALDATA Earnings per share
		once
			create Result.make_risk ("epsv", "epsv", "epsv", k_risk, null_integer, null_integer, 2, k_currency_per_share, -5.0, 5.0, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", k_risk, 2420)
		ensure
			not_void: Result /= Void
		end

	eadj: VARIABLE_DESCRIPTION_RECORD is
			-- VALDATA Earnings per share adjusted for changes in shares
		once
			create Result.make_risk ("eadj", "eadj", "eadj", k_risk, null_integer, null_integer, 2, k_currency_per_share, -5.0, 5.0, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", k_risk, 2419)
		ensure
			not_void: Result /= Void
		end

	bkvv: VARIABLE_DESCRIPTION_RECORD is
			-- VALDATA Book Value per share
		once
			create Result.make_risk ("bkvv", "bkvv", "bkvv", k_risk, null_integer, null_integer, 2, k_currency_per_share, -5.0, 5.0, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", k_risk, 2418)
		ensure
			not_void: Result /= Void
		end

	arex: VARIABLE_DESCRIPTION_RECORD is
			-- 5-year average Recurring Earnings per share
		once
			create Result.make_risk ("arex", "arex", "arex", k_risk, null_integer, null_integer, 2, k_currency_per_share, -5.0, 5.0, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", k_risk, 2417)
		ensure
			not_void: Result /= Void
		end

	aver: VARIABLE_DESCRIPTION_RECORD is
			-- 5-year average VALDATA Earnings per share
		once
			create Result.make_risk ("aver", "aver", "aver", k_risk, null_integer, null_integer, 2, k_currency_per_share, -5.0, 5.0, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", k_risk, 2416)
		ensure
			not_void: Result /= Void
		end

	arog: VARIABLE_DESCRIPTION_RECORD is
			-- 5-year average of Generalized return on equity
		once
			create Result.make_risk ("arog", "arog", "arog", k_risk, null_integer, null_integer, 2, Void, -5.0, 5.0, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", k_risk, 2415)
		ensure
			not_void: Result /= Void
		end

	aeps: VARIABLE_DESCRIPTION_RECORD is
			-- 5-year average Earnings per share
		once
			create Result.make_risk ("aeps", "aeps", "aeps", k_risk, null_integer, null_integer, 2, k_currency_per_share, -5.0, 5.0, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", k_risk, 2414)
		ensure
			not_void: Result /= Void
		end

	abps: VARIABLE_DESCRIPTION_RECORD is
			-- 5-year average of Book value per share
		once
			create Result.make_risk ("abps", "abps", "abps", k_risk, null_integer, null_integer, 2, k_currency_per_share, -5.0, 5.0, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", k_risk, 2413)
		ensure
			not_void: Result /= Void
		end

	abkv: VARIABLE_DESCRIPTION_RECORD is
			-- 5-year average of VALDATA book value per share
		once
			create Result.make_risk ("abkv", "abkv", "abkv", k_risk, null_integer, null_integer, 2, k_currency_per_share, -5.0, 5.0, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", k_risk, 2412)
		ensure
			not_void: Result /= Void
		end

	xsps: VARIABLE_DESCRIPTION_RECORD is
			-- Recurring earnings per share
		once
			create Result.make_risk ("xsps", "xsps", "xsps", k_risk, null_integer, null_integer, 2, k_currency_per_share, -5.0, 5.0, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", k_risk, 2411)
		ensure
			not_void: Result /= Void
		end

	fgnd: VARIABLE_DESCRIPTION_RECORD is
			-- Growth "Normal Data" variable
		once
			create Result.make_mondat ("fgnd", "fgnd", "", "", null_integer, null_integer, null_integer, null_integer, 1, "", k_integer, Void, Void, k_company, ' ', k_monthly_data, "", k_mondat, 2410)
		ensure
			not_void: Result /= Void
		end

	fgrd: VARIABLE_DESCRIPTION_RECORD is
			-- Growth "Recent Data" variable
		once
			create Result.make_mondat ("fgrd", "fgrd", "", "", null_integer, null_integer, null_integer, null_integer, 1, "", k_integer, Void, Void, k_company, ' ', k_monthly_data, "", k_mondat, 2409)
		ensure
			not_void: Result /= Void
		end

	ibwt: VARIABLE_DESCRIPTION_RECORD is
			-- I/B/E/S coverage weight
		once
			create Result.make_mondat ("ibwt", "ibwt", Void, Void, null_integer, null_integer, null_integer, null_integer, 2, Void, k_double, Void, Void, k_company, ' ', k_monthly_data, "", k_mondat, 2408)
		ensure
			not_void: Result /= Void
		end

	ibs3: VARIABLE_DESCRIPTION_RECORD is
			-- 3 month moving average of IBES sales growth forecast
		once
			create Result.make_risk ("ibes_sales_growth_3m_average", "ibs3", "ibs3", k_risk, null_integer, null_integer, 2, Void, -5.0, 5.0, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", "", 2407)
		end

	dcto: VARIABLE_DESCRIPTION_RECORD is
			-- Odds of dividend cut
		once
			create Result.make_risk ("dcto", "dcto", "dcto", k_risk, null_integer, null_integer, 2, Void, -4.0, 4.0, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", k_risk, 2406)
		ensure
			not_void: Result /= Void
		end

	fpor: VARIABLE_DESCRIPTION_RECORD is
			-- Future Payout Ratio
		once
			create Result.make_mondat ("fpor", "fpor", "", "", null_integer, null_integer, null_integer, null_integer, 1, "", k_double, Void, Void, k_company, 'F', k_monthly_data, "", k_mondat, 2405)
		ensure
			not_void: Result /= Void
		end

	cpor: VARIABLE_DESCRIPTION_RECORD is
			-- Current Payout Ratio
		once
			create Result.make_mondat ("cpor", "cpor", "", "", null_integer, null_integer, null_integer, null_integer, 1, "", k_double, Void, Void, k_company, 'F', k_monthly_data, "", k_mondat, 2404)
		ensure
			not_void: Result /= Void
		end

	npcn: VARIABLE_DESCRIPTION_RECORD is
			-- Nill Pay Count
		once
			create Result.make_mondat ("npcn", "npcn", "", "", null_integer, null_integer, null_integer, null_integer, 1, "", k_double, Void, Void, k_company, 'F', k_monthly_data, "", k_mondat, 2403)
		ensure
			not_void: Result /= Void
		end

	sest: VARIABLE_DESCRIPTION_RECORD is
			-- Soft Equity/Sales trend
		once
			create Result.make_risk ("sest", "sest", "sest", k_risk, null_integer, null_integer, 2, Void, -3.0, 3.0, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", k_risk, 2402)
		ensure
			not_void: Result /= Void
		end

	ivst: VARIABLE_DESCRIPTION_RECORD is
			-- Inventory/Sales trend
		once
			create Result.make_risk ("ivst", "ivst", "ivst", k_risk, null_integer, null_integer, 2, Void, -3.0, 3.0, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", k_risk, 2401)
		ensure
			not_void: Result /= Void
		end

	exfp: VARIABLE_DESCRIPTION_RECORD is
			-- Extrapolation of fundamentals EFM block
		once
			create Result.make_risk ("exfp", "exfp", "exfp", k_risk, null_integer, null_integer, 2, Void, -3.0, 3.0, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", k_risk, 2400)
		ensure
			not_void: Result /= Void
		end

	dpmh: VARIABLE_DESCRIPTION_RECORD is
			-- Expected delta profit margin
		once
			create Result.make_risk ("dpmh", "dpmh", "dpmh", k_risk, null_integer, null_integer, 2, Void, -3.0, 3.0, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", k_risk, 2399)
		ensure
			not_void: Result /= Void
		end

	depm: VARIABLE_DESCRIPTION_RECORD is
			-- Delta Earnings conditioned on Profit Margin
		once
			create Result.make_risk ("depm", "depm", "depm", k_risk, null_integer, null_integer, 2, Void, -3.0, 3.0, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", k_risk, 2398)
		ensure
			not_void: Result /= Void
		end

	dytr: VARIABLE_DESCRIPTION_RECORD is
			-- Total Dividend to Price (truncated)
		once
			create Result.make_risk ("dytr", "dytr", "dytr", k_risk, null_integer, null_integer, 2, Void, -3.0, 3.0, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", k_risk, 2397)
		ensure
			not_void: Result /= Void
		end

	divt: VARIABLE_DESCRIPTION_RECORD is
			-- Total Dividend
		once
			create Result.make_risk ("divt", "divt", "divt", k_risk, null_integer, null_integer, 2, k_currency_per_share, -3.0, 3.0, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", k_risk, 2396)
		ensure
			not_void: Result /= Void
		end

	tins: VARIABLE_DESCRIPTION_RECORD is
			-- Inventory to Sales 2 Years Trent Factor
		once
			create Result.make_valdata ("inventory_to_sales_2_years_trend_factor", "tins", k_computeds, Null_integer, Null_integer, Null_integer, 2, Void, k_double, Void, k_company, 'X', k_valdata_signature, "1", k_computed, 2395)
		ensure
			not_void: Result /= Void
		end

	tses: VARIABLE_DESCRIPTION_RECORD is
			-- Soft Equity to Sales 2 Years Trend Factor
		once
			create Result.make_valdata ("soft_equity_to_sales_2_years_trend_factor", "tses", k_computeds, Null_integer, Null_integer, Null_integer, 2, Void, k_double, Void, k_company, 'X', k_valdata_signature, "1", k_computed, 2394)
		ensure
			not_void: Result /= Void
		end

	dins: VARIABLE_DESCRIPTION_RECORD is
			-- Delta Inventory to Sales 2 Years
		once
			create Result.make_valdata ("delta_inventory_to_sales_2_years", "dins", k_computeds, Null_integer, Null_integer, Null_integer, 2, Void, k_double, Void, k_company, 'X', k_valdata_signature, "1", k_computed, 2393)
		ensure
			not_void: Result /= Void
		end

	dses: VARIABLE_DESCRIPTION_RECORD is
			-- Delta Soft Equity to Sales 2 Years
		once
			create Result.make_valdata ("delta_soft_equity_to_sales_2_years", "dses", k_computeds, Null_integer, Null_integer, Null_integer, 2, Void, k_double, Void, k_company, 'X', k_valdata_signature, "1", k_computed, 2392)
		ensure
			not_void: Result /= Void
		end

	slbt: VARIABLE_DESCRIPTION_RECORD is
			-- Soft Liabilities
		once
			create Result.make_valdata ("soft_liabilities", "slbt", k_liabilities, Null_integer, Null_integer, Null_integer, 2, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", "liabilities_off_balance_sheet", 2391)
		ensure
			not_void: Result /= Void
		end

	seqy: VARIABLE_DESCRIPTION_RECORD is
			-- Soft Equity
		once
			create Result.make_valdata ("soft_equity", "seqy", k_book, Null_integer, Null_integer, Null_integer, 2, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", "liabilities_off_balance_sheet", 2390)
		ensure
			not_void: Result /= Void
		end

	sast: VARIABLE_DESCRIPTION_RECORD is
			-- Soft Assets
		once
			create Result.make_valdata ("soft_assets", "sast", k_asst, Null_integer, Null_integer, Null_integer, 2, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", "assets_off_balance_sheet", 2389)
		ensure
			not_void: Result /= Void
		end

	hlbt: VARIABLE_DESCRIPTION_RECORD is
			-- Hard Liabilities
		once
			create Result.make_valdata ("hard_liabilities", "hlbt", k_liabilities, Null_integer, Null_integer, Null_integer, 2, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", "liabilities_off_balance_sheet", 2388)
		ensure
			not_void: Result /= Void
		end

	heqy: VARIABLE_DESCRIPTION_RECORD is
			-- Hard Equity
		once
			create Result.make_valdata ("hard_equity", "heqy", k_book, Null_integer, Null_integer, Null_integer, 2, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", "liabilities_off_balance_sheet", 2387)
		ensure
			not_void: Result /= Void
		end

	hast: VARIABLE_DESCRIPTION_RECORD is
			-- Hard Assets
		once
			create Result.make_valdata ("hard_assets", "hast", k_asst, Null_integer, Null_integer, Null_integer, 2, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", "assets_off_balance_sheet", 2386)
		ensure
			not_void: Result /= Void
		end

	model_company_primary_listing_id: VARIABLE_DESCRIPTION_RECORD is
			-- Primary listing handle for a company
		once
			create Result.make_model_company ("model_primary_listing_id", "primary_rid", 0, "", k_integer, ' ', Void, 2384)
		end

	esg_fail: VARIABLE_DESCRIPTION_RECORD is
			-- esg_fail
		once
			create Result.make_risk ("esg_fail", "ESGF", "ESGF", k_risk, null_integer, null_integer, 1, Void, missing_double, missing_double, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", k_risk, 2383)
		end

	esg_wic: VARIABLE_DESCRIPTION_RECORD is
			-- esg_wic
		once
			create Result.make_risk ("esg_wic", "ESGW", "ESGW", k_risk, null_integer, null_integer, 1, Void, missing_double, missing_double, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", k_risk, 2382)
		end

	esg_half: VARIABLE_DESCRIPTION_RECORD is
			-- esg_half (aka partial)
		once
			create Result.make_risk ("esg_half", "ESGH", "ESGH", k_risk, null_integer, null_integer, 1, Void, missing_double, missing_double, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", k_risk, 2381)
		end

	esg_pass: VARIABLE_DESCRIPTION_RECORD is
			-- esg_pass
		once
			create Result.make_risk ("esg_pass", "ESGP", "ESGP", k_risk, null_integer, null_integer, 1, Void, missing_double, missing_double, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", k_risk, 2380)
		end

	esg_score: VARIABLE_DESCRIPTION_RECORD is
			-- esg_score
		once
			create Result.make_risk ("esg_score", "ESGS", "ESGS", k_risk, null_integer, null_integer, 1, Void, missing_double, missing_double, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", k_risk, 2379)
		end

	anes: VARIABLE_DESCRIPTION_RECORD is
			-- Analyst Estimates block of Earnings Forecast Model
		once
			create Result.make_risk ("anes", "anes", "anes", k_risk, null_integer, null_integer, 2, Void, -3.0, 3.0, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", k_risk, 2378)
		ensure
			not_void: Result /= Void
		end

	diep: VARIABLE_DESCRIPTION_RECORD is
			-- I/B/E/S EPS less recurring EPS to Price
		once
			create Result.make_risk ("diep", "diep", "diep", k_risk, null_integer, null_integer, 2, Void, -3.0, 3.0, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", k_risk, 2377)
		ensure
			not_void: Result /= Void
		end

	dfrq: VARIABLE_DESCRIPTION_RECORD is
			-- Dividend payout frequency
		once
			create Result.make_mondat ("dfrq", "dfrq", "", "", null_integer, null_integer, null_integer, null_integer, 1, "", k_integer, Void, Void, k_company, 'F', k_monthly_data, "", k_mondat, 2376)
		ensure
			not_void: Result /= Void
		end

	dvyd: VARIABLE_DESCRIPTION_RECORD is
			-- Total Dividend to Price i.e. dividend yield
		once
			create Result.make_risk ("dvyd", "dvyd", "dvyd", k_risk, null_integer, null_integer, 2, Void, -3.0, 3.0, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", k_risk, 2375)
		ensure
			not_void: Result /= Void
		end

	cfsr: VARIABLE_DESCRIPTION_RECORD is
			--Current alpha, risk adjusted, similars normalized
		once
			create Result.make_risk ("calr_sim_nor", "cfsr", "CALR.SMN", k_risk, null_integer, null_integer, 2, "", -3, 3, missing_double, missing_double, "SIMNORM:CALR", Void, Void, ' ', k_monthly_data, "", k_risk, 2374)
		ensure
			not_void: Result /= Void
		end

	calr: VARIABLE_DESCRIPTION_RECORD is
			--Current alpha, risk adjusted
		once
			create Result.make_risk ("current_alpha_risk_adjusted", "calr", "CuralpRA", k_risk, null_integer, null_integer, 2, "", -5, 5, 0, 0.3, "", Void, Void, ' ', k_monthly_data, "", k_risk, 2373)
		ensure
			not_void: Result /= Void
		end

	valr: VARIABLE_DESCRIPTION_RECORD is
			-- Valuation model fitted value
		once
			create Result.make_mondat ("fitted_value_risk_adjusted", "valr", "", k_per_shares, null_integer, null_integer, null_integer, null_integer, 2, k_currency_per_share, k_double, Void, Void, k_company, ' ', k_monthly_data, k_per_share, k_mondat, 2372)
		ensure
			not_void: Result /= Void
		end

	divr: VARIABLE_DESCRIPTION_RECORD is
			-- valreg internal variable: dividends_ROW
		once
			create Result.make_valdata ("dividends_ROW", "divr", "", 0, 0, 0, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "", "", 2371)
		ensure
			not_void: Result /= Void
		end

	rerr: VARIABLE_DESCRIPTION_RECORD is
			-- valreg internal variable: retained_earnings_12mf_ROW
		once
			create Result.make_valdata ("retained_earnings_12mf_ROW", "rerr", "", 0, 0, 0, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "", "", 2370)
		ensure
			not_void: Result /= Void
		end

	divj: VARIABLE_DESCRIPTION_RECORD is
			-- valreg internal variable: dividends_J
		once
			create Result.make_valdata ("dividends_J", "divj", "", 0, 0, 0, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "", "", 2369)
		ensure
			not_void: Result /= Void
		end

	rerj: VARIABLE_DESCRIPTION_RECORD is
			-- valreg internal variable: retained_earnings_12mf_J
		once
			create Result.make_valdata ("retained_earnings_12mf_J", "rerj", "", 0, 0, 0, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "", "", 2368)
		ensure
			not_void: Result /= Void
		end

	dive: VARIABLE_DESCRIPTION_RECORD is
			-- valreg internal variable: dividends_E
		once
			create Result.make_valdata ("dividends_E", "dive", "", 0, 0, 0, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "", "", 2367)
		ensure
			not_void: Result /= Void
		end

	rere: VARIABLE_DESCRIPTION_RECORD is
			-- valreg internal variable: retained_earnings_12mf_E
		once
			create Result.make_valdata ("retained_earnings_12mf_E", "rere", "", 0, 0, 0, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "", "", 2366)
		ensure
			not_void: Result /= Void
		end

	divu: VARIABLE_DESCRIPTION_RECORD is
			-- valreg internal variable: dividends_UC
		once
			create Result.make_valdata ("dividends_UC", "divu", "", 0, 0, 0, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "", "", 2365)
		ensure
			not_void: Result /= Void
		end

	reru: VARIABLE_DESCRIPTION_RECORD is
			-- valreg internal variable: retained_earnings_12mf_UC
		once
			create Result.make_valdata ("retained_earnings_12mf_UC", "reru", "", 0, 0, 0, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "", "", 2364)
		ensure
			not_void: Result /= Void
		end

	er12: VARIABLE_DESCRIPTION_RECORD is
			-- valreg internal variable: earnings_12_months_forward
		once
			create Result.make_valdata ("earnings_12_months_forward", "er12", "", 0, 0, 0, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "", "", 2363)
		ensure
			not_void: Result /= Void
		end

	lagx: VARIABLE_DESCRIPTION_RECORD is
			-- valreg internal variable: log_average_asset_age_xibes
		once
			create Result.make_valdata ("log_average_asset_age_xibes", "lagx", "", 0, 0, 0, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "", "", 2362)
		ensure
			not_void: Result /= Void
		end

	lagi: VARIABLE_DESCRIPTION_RECORD is
			-- valreg internal variable: log_average_asset_age_wibes
		once
			create Result.make_valdata ("log_average_asset_age_wibes", "lagi", "", 0, 0, 0, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "", "", 2361)
		ensure
			not_void: Result /= Void
		end

	grsx: VARIABLE_DESCRIPTION_RECORD is
			-- valreg internal variable: sales_assets_growth_5_years_xibes
		once
			create Result.make_valdata ("sales_assets_growth_5_years_xibes", "grsx", "", 0, 0, 0, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "", "", 2360)
		ensure
			not_void: Result /= Void
		end

	grsi: VARIABLE_DESCRIPTION_RECORD is
			-- valreg internal variable: sales_assets_growth_5_years_wibes
		once
			create Result.make_valdata ("sales_assets_growth_5_years_wibes", "grsi", "", 0, 0, 0, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "", "", 2359)
		ensure
			not_void: Result /= Void
		end

	libs: VARIABLE_DESCRIPTION_RECORD is
			-- valreg internal variable: log_ibes_sales_growth_xcommodities
		once
			create Result.make_valdata ("log_ibes_sales_growth_cx", "libs", "", 0, 0, 0, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "", "", 2358)
		ensure
			not_void: Result /= Void
		end

	lgro: VARIABLE_DESCRIPTION_RECORD is
			-- valreg internal variable: log_sales_assets_growth_5_years
		once
			create Result.make_valdata ("log_sales_assets_growth_5_years", "lgro", "", 0, 0, 0, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "", "", 2357)
		ensure
			not_void: Result /= Void
		end

	lage: VARIABLE_DESCRIPTION_RECORD is
			-- valreg internal variable: log_average_asset_age
		once
			create Result.make_valdata ("log_average_asset_age", "lage", "", 0, 0, 0, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "", "", 2356)
		ensure
			not_void: Result /= Void
		end

	nwcd: VARIABLE_DESCRIPTION_RECORD is
			-- valreg internal variable: net_working_capital_years
		once
			create Result.make_valdata ("net_working_capital_years", "nwcd", "", 0, 0, 0, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "", "", 2355)
		ensure
			not_void: Result /= Void
		end

	ncsh: VARIABLE_DESCRIPTION_RECORD is
			-- valreg internal variable: net_cash_to_assets
		once
			create Result.make_valdata ("net_cash_to_assets", "ncsh", "", 0, 0, 0, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "", "", 2354)
		ensure
			not_void: Result /= Void
		end

	lomg: VARIABLE_DESCRIPTION_RECORD is
			-- valreg internal variable: log_operating_margin
		once
			create Result.make_valdata ("log_operating_margin", "lomg", "", 0, 0, 0, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "", "", 2353)
		ensure
			not_void: Result /= Void
		end

	grsa: VARIABLE_DESCRIPTION_RECORD is
			-- valreg internal variable: sales_assets_growth_5_years
		once
			create Result.make_valdata ("sales_assets_growth_5_years", "grsa", "", 0, 0, 0, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "", "", 2352)
		ensure
			not_void: Result /= Void
		end

	payt: VARIABLE_DESCRIPTION_RECORD is
			-- valreg internal variable: ("dividend_payout_ratio",
		once
			create Result.make_valdata ("dividend_payout_ratio", "payt", "", 0, 0, 0, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "", "", 2351)
		ensure
			not_void: Result /= Void
		end

	ibsc: VARIABLE_DESCRIPTION_RECORD is
			-- valreg internal variable: ibes_sales_growth_xc
		once
			create Result.make_valdata ("ibes_sales_growth_xc", "ibsc", "", 0, 0, 0, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "", "", 2350)
		ensure
			not_void: Result /= Void
		end

	ibsw: VARIABLE_DESCRIPTION_RECORD is
			-- valreg internal variable: ibes_sales_growth_vr
		once
			create Result.make_valdata ("ibes_sales_growth_vr", "ibsw", "", 0, 0, 0, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "", "", 2349)
		ensure
			not_void: Result /= Void
		end

	dgrw: VARIABLE_DESCRIPTION_RECORD is
			-- valreg internal variable: dividends_growth_5_years_vr
		once
			create Result.make_valdata ("dividends_growth_5_years_vr", "dgrw", "", 0, 0, 0, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "", "", 2348)
		ensure
			not_void: Result /= Void
		end

	roew: VARIABLE_DESCRIPTION_RECORD is
			-- valreg internal variable: roe_5_years_avg_vr
		once
			create Result.make_valdata ("roe_5_years_avg_vr", "roew", "", 0, 0, 0, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "", "", 2347)
		ensure
			not_void: Result /= Void
		end

	agrw: VARIABLE_DESCRIPTION_RECORD is
			-- valreg internal variable: assets_growth_5_years_vr
		once
			create Result.make_valdata ("assets_growth_5_years_vr", "agrw", "", 0, 0, 0, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "", "", 2346)
		ensure
			not_void: Result /= Void
		end

	sgrw: VARIABLE_DESCRIPTION_RECORD is
			-- valreg internal variable: sales_growth_5_years_valurisk
		once
			create Result.make_valdata ("sales_growth_5_years_vr", "sgrw", "", 0, 0, 0, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "", "", 2345)
		ensure
			not_void: Result /= Void
		end

	fr_date: VARIABLE_DESCRIPTION_RECORD is
			-- Fiscal report date
		once
			create Result.make_mondat ("fr_date", "fr_date", "", "", null_integer, null_integer, null_integer, null_integer, 1, "", k_integer, Void, Void, k_company, 'F', k_monthly_data, "", k_mondat, 2344)
		ensure
			not_void: Result /= Void
		end

	fdaa: VARIABLE_DESCRIPTION_RECORD is
			-- Forward 1-year distribution adjustment factor data availability
		once
			create Result.make_mondat ("fdaa", "fdaa", "", "per_shares", null_integer, null_integer, null_integer, null_integer, 1, "", k_integer, Void, Void, k_company, 'F', k_monthly_data, "", k_mondat, 2343)
		ensure
			not_void: Result /= Void
		end

	freb: VARIABLE_DESCRIPTION_RECORD is
			-- Forward recurring earnings to book
		once
			create Result.make_risk ("freb", "freb", "freb", k_risk, null_integer, null_integer, 2, Void, -3.0, 3.0, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", k_risk, 2342)
		ensure
			not_void: Result /= Void
		end

	cmpf: VARIABLE_DESCRIPTION_RECORD is
			-- Competitive Forces Block variable of Earnings Forecast Model
		once
			create Result.make_risk ("cmpf", "cmpf", "cmpf", k_risk, null_integer, null_integer, 2, Void, -3.0, 3.0, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", k_risk, 2341)
		ensure
			not_void: Result /= Void
		end

	edrr: VARIABLE_DESCRIPTION_RECORD is
			-- Estimate of change of robust roe less country average
		once
			create Result.make_risk ("edrr", "edrr", "edrr", k_risk, null_integer, null_integer, 2, Void, -3.0, 3.0, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", k_risk, 2340)
		ensure
			not_void: Result /= Void
		end

	rcbd: VARIABLE_DESCRIPTION_RECORD is
			-- Robust roe less book weighted country average robust roe
		once
			create Result.make_risk ("rcbd", "rcbd", "rcbd", k_risk, null_integer, null_integer, 2, Void, -3.0, 3.0, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", k_risk, 2339)
		ensure
			not_void: Result /= Void
		end

	serv: VARIABLE_DESCRIPTION_RECORD is
			-- Scaled ERV6
		once
			create Result.make_risk ("serv", "serv", "serv", k_risk, null_integer, null_integer, 2, Void, -3.0, 3.0, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", k_risk, 2338)
		ensure
			not_void: Result /= Void
		end

	ibes_sales_growth: VARIABLE_DESCRIPTION_RECORD is
			-- ibsg variable
		once
			create Result.make_mondat ("ibes_sales_growth", "ibsg", Void, Void, null_integer, null_integer, null_integer, null_integer, 2, Void, k_double, Void, Void, k_company, ' ', k_monthly_data, "", k_mondat, 2337)
		ensure
			not_void: Result /= Void
		end

	easn: VARIABLE_DESCRIPTION_RECORD is
			-- eapc_sim_nor
		once
			create Result.make_risk ("eapc_sim_nor", "easn", "EAPC.SMN", k_risk, null_integer, null_integer, 2, "", -3.0, 3.0, missing_double, missing_double, "SIMNORM:EAPC", "EAPC.SimNor", Void, ' ', k_monthly_data, "", k_risk, 2334)
		ensure
			not_void: Result /= Void
		end

	cesn: VARIABLE_DESCRIPTION_RECORD is
			-- cepc_sim_nor
		once
			create Result.make_risk ("cepc_sim_nor", "cesn", "cepc.SMN", k_risk, null_integer, null_integer, 2, "", -3.0, 3.0, missing_double, missing_double, "SIMNORM:CEPC", "CEPC.SimNor", Void, ' ', k_monthly_data, "", k_risk, 2333)
		ensure
			not_void: Result /= Void
		end

	rtsn: VARIABLE_DESCRIPTION_RECORD is
			-- monthly_rate_of_return_sim_nor
		once
			create Result.make_risk ("monthly_rate_of_return_sim_nor", "rtsn", "ROR.SMN", k_risk, null_integer, null_integer, 2, "", -3.0, 3.0, missing_double, missing_double, "SIMNORM:ROR", "ROR.SimNor", Void, ' ', k_monthly_data, "", k_risk, 2332)
		ensure
			not_void: Result /= Void
		end

	cfin: VARIABLE_DESCRIPTION_RECORD is
			-- Industry normalized current alpha
		once
			create Result.make_risk ("current_alpha_industry_normalized", "cfin", "cfin", k_risk, null_integer, null_integer, 2, "", -3.0, 3.0, missing_double, missing_double, "INDNORM:CALF", Void, Void, ' ', k_monthly_data, "", k_risk, 2331)
		ensure
			not_void: Result /= Void
		end

	ceqi: VARIABLE_DESCRIPTION_RECORD is
			-- Hard and soft equity composite
		once
			create Result.make_risk ("ceqi", "ceqi", "ceqi", k_risk, null_integer, null_integer, 2, Void, -3.0, 3.0, missing_double, missing_double, "HEPI * 0.5 + SEPI * 0.5", Void, Void, ' ', k_monthly_data, "", "", 2330)
		ensure
			not_void: Result /= Void
		end

	sepi: VARIABLE_DESCRIPTION_RECORD is
			-- Industry normalized soft equity to price
		once
			create Result.make_risk ("industry_normalized_soft_equity_to_price", "sepi", "sepi", k_risk, null_integer, null_integer, 2, Void, -3.0, 3.0, missing_double, missing_double, "INDNORM:SETP", Void, Void, ' ', k_monthly_data, "", "", 2329)
		ensure
			not_void: Result /= Void
		end

	hepi: VARIABLE_DESCRIPTION_RECORD is
			-- Industry normalized hard equity to price
		once
			create Result.make_risk ("industry_normalized_hard_equity_to_price", "hepi", "hepi", k_risk, null_integer, null_integer, 2, Void, -3.0, 3.0, missing_double, missing_double, "INDNORM:HETP", Void, Void, ' ', k_monthly_data, "", "", 2328)
		ensure
			not_void: Result /= Void
		end

	setp: VARIABLE_DESCRIPTION_RECORD is
			-- Soft equity to price
		once
			create Result.make_risk ("soft_equity_to_price", "setp", "setp", k_risk, null_integer, null_integer, 2, Void, -3.0, 3.0, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", "", 2327)
		ensure
			not_void: Result /= Void
		end

	hetp: VARIABLE_DESCRIPTION_RECORD is
			-- Hard equity to price
		once
			create Result.make_risk ("hard_equity_to_price", "hetp", "hetp", k_risk, null_integer, null_integer, 2, Void, -3.0, 3.0, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", "", 2326)
		ensure
			not_void: Result /= Void
		end

	segc: VARIABLE_DESCRIPTION_RECORD is
			-- Robust characterization of the interaction between SUSE and PNCV
		once
			create Result.make_risk ("segc", "segc", "segc", k_risk, null_integer, null_integer, 2, Void, -3.0, 3.0, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", "", 2325)
		ensure
			not_void: Result /= Void
		end

	fdaj: VARIABLE_DESCRIPTION_RECORD is
			-- Forward 1-year distribution adjustment factor
		once
			create Result.make_mondat ("fdaj", "fdaj", "", "per_shares", null_integer, null_integer, null_integer, null_integer, 1, "", k_double, Void, Void, k_company, 'F', k_monthly_data, "", k_mondat, 2324)
		ensure
			not_void: Result /= Void
		end

	dreb: VARIABLE_DESCRIPTION_RECORD is
			-- Recurring Earnings Change to robust Book
		once
			create Result.make_risk ("dreb", "dreb", "dreb", k_risk, null_integer, null_integer, 2, Void, -3.0, 3.0, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", "", 2322)
		ensure
			not_void: Result /= Void
		end

	rbps: VARIABLE_DESCRIPTION_RECORD is
			-- Robust book per share
		once
			create Result.make_risk ("robust_book_per_share", "rbps", "rbps", k_risk, null_integer, null_integer, 2, k_currency_per_share, -3.0, 3.0, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", "", 2321)
		ensure
			not_void: Result /= Void
		end

	reps: VARIABLE_DESCRIPTION_RECORD is
			-- Recurring earnings per share
		once
			create Result.make_risk ("recurring_earnings_per_share", "reps", "reps", k_risk, null_integer, null_integer, 2, k_currency_per_share, -3.0, 3.0, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", "", 2320)
		ensure
			not_void: Result /= Void
		end

	xips: VARIABLE_DESCRIPTION_RECORD is
			-- Extraordinary and excetional items per share
		once
			create Result.make_risk ("xips", "xips", "xips", k_risk, null_integer, null_integer, 2, k_currency_per_share, -3.0, 3.0, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", "", 2319)
		ensure
			not_void: Result /= Void
		end

	brob: VARIABLE_DESCRIPTION_RECORD is
			-- Book robustness
		once
			create Result.make_risk ("brob", "brob", "brob", k_risk, null_integer, null_integer, 2, Void, -3.0, 3.0, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", "", 2318)
		ensure
			not_void: Result /= Void
		end

	rsn1: VARIABLE_DESCRIPTION_RECORD is
			-- Industry normalized rs11
		once
			create Result.make_risk ("relative_strength_months_2_12_similars_normalized", "rsn1", "RsSt11.SMN", k_risk, null_integer, 151, 2, "", -3, 3, missing_double, missing_double, "SIMNORM:RS11", Void, Void, ' ', k_monthly_data, "", k_risk, 2315)
		end

	hcar: VARIABLE_DESCRIPTION_RECORD is
			-- Home Country Active Return
		once
			create Result.make_risk ("hcar", "hcar", "hcar", k_risk, null_integer, null_integer, 2, Void, -3.0, 3.0, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", "", 2313)
		ensure
			not_void: Result /= Void
		end

	rs11: VARIABLE_DESCRIPTION_RECORD is
			-- Relative Strenght months 2-12
		once
			create Result.make_risk ("relative_strength_months_2_12", "rs11", "rs11", k_risk, null_integer, null_integer, 2, Void, -3.0, 3.0, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", "", 2312)
		ensure
			not_void: Result /= Void
		end

	gqal: VARIABLE_DESCRIPTION_RECORD is
			-- Global Market Quality
		once
			create Result.make_risk ("global_market_quality", "gqal", "gqal", k_risk, null_integer, null_integer, 2, Void, -3.0, 3.0, missing_double, missing_double, "INDFITG:RS12", Void, Void, ' ', k_monthly_data, "", "", 2310)
		ensure
			not_void: Result /= Void
		end

	texx: VARIABLE_DESCRIPTION_RECORD is
			-- Total earnings ex extraordinary and exceptional items
		once
			create Result.make_risk ("texx", "texx", "texx", k_risk, null_integer, null_integer, 2, k_currency, -3.0, 3.0, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", "", 2309)
		ensure
			not_void: Result /= Void
		end

	ttlx: VARIABLE_DESCRIPTION_RECORD is
			-- Total extraordinary and exceptional items
		once
			create Result.make_risk ("ttlx", "ttlx", "ttlx", k_risk, null_integer, null_integer, 2, k_currency, -3.0, 3.0, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", "", 2308)
		ensure
			not_void: Result /= Void
		end

	txra: VARIABLE_DESCRIPTION_RECORD is
			-- Earnings Forecast Model Tax Rate
		once
			create Result.make_risk ("txra", "txra", "txra", k_risk, null_integer, null_integer, 2, Void, -3.0, 3.0, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", "", 2307)
		ensure
			not_void: Result /= Void
		end

	rrxx: VARIABLE_DESCRIPTION_RECORD is
			-- Robust return on equity
		once
			create Result.make_risk ("rrxx", "rrxx", "rrxx", k_risk, null_integer, null_integer, 2, Void, -3.0, 3.0, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", "", 2306)
		ensure
			not_void: Result /= Void
		end

	bkvn: VARIABLE_DESCRIPTION_RECORD is
			-- Breakeven adjustment (valreg internal variable)
		once
			create Result.make_valdata ("breakeven_adjustment", "bkvn", "", 0, 0, 0, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "", "", 2305)
		end

	ibes_reps: VARIABLE_DESCRIPTION_RECORD is
			-- Trailing 12-month eps based on ibes reported.
		once
			create Result.make_mondat ("ibes_earnings_per_share_reported", "ibes_reps", "record.mdc.ibes_reps", k_per_shares, null_integer, null_integer, null_integer, null_integer, 2, k_currency_per_share, k_double, Void, Void, k_company, ' ', k_monthly_data, k_per_share, k_mondat, 2285)
		end

	eps_ibesi: VARIABLE_DESCRIPTION_RECORD is
			-- Forward 12-month eps estimate based on ibes interim estimates.
		once
			create Result.make_mondat ("ibes_earnings_per_share_interim", "eps_ibesi", "record.mdc.eps_ibesi", k_per_shares, null_integer, null_integer, null_integer, null_integer, 2, k_currency_per_share, k_double, Void, Void, k_company, ' ', k_monthly_data, k_per_share, k_mondat, 2284)
		end

	detp: VARIABLE_DESCRIPTION_RECORD is
			-- Change in realized 1 year earnings yield
		once
			create Result.make_risk ("detp", "detp", Void, Void, null_integer, null_integer, 2, Void, missing_double, missing_double, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", "", 2283)
		ensure
			not_void: Result /= Void
		end

	drep: VARIABLE_DESCRIPTION_RECORD is
			-- Change in realized 1 year recurring earnings yield
		once
			create Result.make_risk ("drep", "drep", Void, Void, null_integer, null_integer, 2, Void, missing_double, missing_double, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", "", 2282)
		ensure
			not_void: Result /= Void
		end

	eyld: VARIABLE_DESCRIPTION_RECORD is
			-- Realized 1 year earnings yield
		once
			create Result.make_risk ("eyld", "eyld", Void, Void, null_integer, null_integer, 2, Void, missing_double, missing_double, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", "", 2281)
		ensure
			not_void: Result /= Void
		end

	ryld: VARIABLE_DESCRIPTION_RECORD is
			-- Realized 1 year recurring earnings yield
		once
			create Result.make_risk ("ryld", "ryld", Void, Void, null_integer, null_integer, 2, Void, missing_double, missing_double, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", "", 2280)
		ensure
			not_void: Result /= Void
		end

	seqs: VARIABLE_DESCRIPTION_RECORD is
			-- Soft equity per share
		once
			create Result.make_mondat ("soft_equity_per_share", "seqs", Void, k_per_shares, null_integer, null_integer, null_integer, null_integer, 2, k_currency_per_share, k_double, Void, Void, k_company, ' ', k_monthly_data, "", "", 2279)
		ensure
			not_void: Result /= Void
		end

	heqs: VARIABLE_DESCRIPTION_RECORD is
			-- Hard equity per share
		once
			create Result.make_mondat ("hard_equity_per_share", "heqs", Void, k_per_shares, null_integer, null_integer, null_integer, null_integer, 2, k_currency_per_share, k_double, Void, Void, k_company, ' ', k_monthly_data, "", "", 2278)
		ensure
			not_void: Result /= Void
		end

	ersn: VARIABLE_DESCRIPTION_RECORD is
			-- Similars-normalized version of erhp (projected recurring earnings to price)
		once
			create Result.make_risk ("projected_recurring_e_p_similars_normalized", "ersn", "PrEp.SMN", k_risk, null_integer, null_integer, 3, "", -4, 4, missing_double, missing_double, "SIMNORM:ERHP", Void, "1", '0', k_monthly_data, "", k_risk, 2277)
		ensure
			not_void: Result /= Void
		end

	mtyl: VARIABLE_DESCRIPTION_RECORD is
			-- Mid-term dividend yield
		once
			create Result.make_risk ("mid_term_dividend_yield", "mtyl", "MTYL", k_risk, null_integer, null_integer, 2, Void, -3.0, 3.0, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", "", 2276)
		ensure
			not_void: Result /= Void
		end

	styl: VARIABLE_DESCRIPTION_RECORD is
			-- Short-term dividend yield
		once
			create Result.make_risk ("short_term_dividend_yield", "styl", "STYL", k_risk, null_integer, null_integer, 2, Void, -3.0, 3.0, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", "", 2275)
		ensure
			not_void: Result /= Void
		end

	ryl1: VARIABLE_DESCRIPTION_RECORD is
			-- Tailing 12-month regular dividend yield
		once
			create Result.make_risk ("trailing_12_month_regular_dividend_yield", "ryl1", "RYL1", k_risk, null_integer, null_integer, 2, Void, -5.0, 5.0, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", "", 2274)
		ensure
			not_void: Result /= Void
		end

	rdyl: VARIABLE_DESCRIPTION_RECORD is
			-- Regular dividend yield
		once
			create Result.make_risk ("regular_dividend_yield", "rdyl", "RDYL", k_risk, null_integer, null_integer, 2, Void, -5.0, 5.0, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", "", 2273)
		ensure
			not_void: Result /= Void
		end

	stvy: VARIABLE_DESCRIPTION_RECORD is
			-- Rolling 12-month median short-term volatility
		once
			create Result.make_risk ("short_term_volatility_rolling_12_month_median", "stvy", "STVY", k_risk, null_integer, null_integer, 2, Void, -5.0, 5.0, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", "", 2271)
		ensure
			no_void: Result /= Void
		end

	stvi: VARIABLE_DESCRIPTION_RECORD is
			-- Short-term volatility index variable
		once
			create Result.make_risk ("short_term_volatility_index", "stvi", "STVI", k_risk, null_integer, null_integer, 2, Void, -5.0, 5.0, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", "", 2270)
		ensure
			no_void: Result /= Void
		end

	sens: VARIABLE_DESCRIPTION_RECORD is
			-- Sensitivity of earnings
		once
			create Result.make_risk ("sensitivity_of_earnings", "sens", "SENSEARN", k_risk, null_integer, null_integer, 2, "", -5, 5, 5.0, 6.0, "", Void, Void, ' ', k_monthly_data, "", "", 2269)
		ensure
			not_void: Result /= Void
		end

	mxid: VARIABLE_DESCRIPTION_RECORD is
			-- Maxium illiquidity discount
		once
			create Result.make_risk ("maximum_illiquidity_discount", "mxid", "MXILQDIS", k_risk, null_integer, null_integer, 2, "", -5, 5, 0.005, 0.10, "", Void, Void, ' ', k_monthly_data, "", "", 2268)
		ensure
			not_void: Result /= Void
		end

	paim: VARIABLE_DESCRIPTION_RECORD is
			-- Probability of asset impairment
		once
			create Result.make_risk ("probability_of_asset_impairment", "PAIM", "PASSTIMP", k_risk, null_integer, null_integer, 2, "", -5, 5, 0.05, 0.20, "", Void, Void, ' ', k_monthly_data, "", "", 2267)
		ensure
			not_void: Result /= Void
		end

	dist: VARIABLE_DESCRIPTION_RECORD is
			-- Distress likelihood
		once
			create Result.make_risk ("distress_likelihood", "dist", "DISTPROB", k_risk, null_integer, null_integer, 2, "", -5, 5, 0.05, 0.10, "", Void, Void, ' ', k_monthly_data, "", "", 2266)
		ensure
			not_void: Result /= Void
		end

	rsg1: VARIABLE_DESCRIPTION_RECORD is
			-- relative_strength_global
		once
			create Result.make_risk ("relative_strength_global", "rsg1", "RELSTRG1", k_risk, null_integer, 0, 2, "", -3, 3, 0.07, 0.35, "", Void, Void, ' ', k_monthly_data, "", "", 2265)
		end

	tax_rate: VARIABLE_DESCRIPTION_RECORD is
			--tax_rate
		once
			create Result.make_risk ("tax_rate", "tax_rate", Void, Void, null_integer, null_integer, 2, Void, missing_double, missing_double, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", "", 2263)
		ensure
			not_void: Result /= Void
		end

	roeq_qlty_trbl_z: VARIABLE_DESCRIPTION_RECORD is
			--return_on_equity_5_year_conditioned_upon_qdos_trbl
		once
			create Result.make_risk ("roeq_qlty_trbl_z", "roeq_qlty_trbl_z", Void, Void, null_integer, null_integer, 2, Void, missing_double, missing_double, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", "", 2262)
		ensure
			not_void: Result /= Void
		end

	qlty_qdos_z: VARIABLE_DESCRIPTION_RECORD is
			--quality_growth_conditioned_upon_qdos
		once
			create Result.make_risk ("qlty_qdos_z", "qlty_qdos_z", Void, Void, null_integer, null_integer, 2, Void, missing_double, missing_double, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", "", 2261)
		ensure
			not_void: Result /= Void
		end

	etop_mns: VARIABLE_DESCRIPTION_RECORD is
			--earnings_to_price_negative_or_zero
		once
			create Result.make_risk ("etop_mns", "etop_mns", Void, Void, null_integer, null_integer, 2, Void, missing_double, missing_double, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", "", 2260)
		ensure
			not_void: Result /= Void
		end

	eg_t_qlty_trbl_z: VARIABLE_DESCRIPTION_RECORD is
			--earnings_growth_tstat_conditioned_upon_qdos_trbl
		once
			create Result.make_risk ("eg_t_qlty_trbl_z", "eg_t_qlty_trbl_z", Void, Void, null_integer, null_integer, 2, Void, missing_double, missing_double, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", "", 2259)
		ensure
			not_void: Result /= Void
		end

	ecbeta_zrs12: VARIABLE_DESCRIPTION_RECORD is
			-- Earnings Change Beta Normalized RS12
		once
			create Result.make_mondat ("ecbeta_zrs12", "ecbeta_zrs12", Void, Void, null_integer, null_integer, null_integer, null_integer, 2, "", k_double, Void, "1", k_company, ' ', k_monthly_data, "", "", 2248)
		ensure
			not_void: Result /= Void
		end

	ecbeta_zervr: VARIABLE_DESCRIPTION_RECORD is
			-- Earnings Change Beta Normalized ERVR
		once
			create Result.make_mondat ("ecbeta_zervr", "ecbeta_zervr", Void, Void, null_integer, null_integer, null_integer, null_integer, 2, "", k_double, Void, "1", k_company, ' ', k_monthly_data, "", "", 2247)
		ensure
			not_void: Result /= Void
		end

	ecbeta_zdtoa: VARIABLE_DESCRIPTION_RECORD is
			-- Earnings Change Beta Normalized DTOA
		once
			create Result.make_mondat ("ecbeta_zdtoa", "ecbeta_zdtoa", Void, Void, null_integer, null_integer, null_integer, null_integer, 2, "", k_double, Void, "1", k_company, ' ', k_monthly_data, "", "", 2246)
		ensure
			not_void: Result /= Void
		end

	ecbeta_zdrhp: VARIABLE_DESCRIPTION_RECORD is
			-- Earnings Change Beta Normalized DRHP
		once
			create Result.make_mondat ("ecbeta_zdrhp", "ecbeta_zdrhp", Void, Void, null_integer, null_integer, null_integer, null_integer, 2, "", k_double, Void, "1", k_company, ' ', k_monthly_data, "", "", 2245)
		ensure
			not_void: Result /= Void
		end

	ecbeta_weight: VARIABLE_DESCRIPTION_RECORD is
			-- Earnings Change Beta Weight
		once
			create Result.make_risk ("ecbeta_weight", "ecbeta_weight", "ecbeta_weight", k_risk, null_integer, null_integer, 2, Void, -3.5, 3.5, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", "", 2244)
		ensure
			not_void: Result /= Void
		end

	ecbeta_spcvar: VARIABLE_DESCRIPTION_RECORD is
			-- Earnings Change Beta Specific Variance
		once
			create Result.make_risk ("ecbeta_spcvar", "ecbeta_spcvar", "ecbeta_spcvar", k_risk, null_integer, null_integer, 2, Void, -3.5, 3.5, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", "", 2243)
		ensure
			not_void: Result /= Void
		end

	ecbeta_filter: VARIABLE_DESCRIPTION_RECORD is
			-- Earnings Change Beta Filter
		once
			create Result.make_risk ("ecbeta_filter", "ecbeta_filter", "ecbeta_filter", k_risk, null_integer, null_integer, 2, Void, -3.5, 3.5, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", "", 2242)
		ensure
			not_void: Result /= Void
		end

	ecbeta_covar: VARIABLE_DESCRIPTION_RECORD is
			-- Earnings Change Beta Covariance
		once
			create Result.make_risk ("ecbeta_covar", "ecbeta_covar", "ecbeta_covar", k_risk, null_integer, null_integer, 2, Void, -3.5, 3.5, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", "", 2241)
		ensure
			not_void: Result /= Void
		end

	ebet: VARIABLE_DESCRIPTION_RECORD is
			-- Earnings Change Beta
		once
			create Result.make_risk ("ebet", "ebet", "ebet", k_risk, null_integer, null_integer, 2, Void, -3.5, 3.5, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", "", 2240)
		ensure
			not_void: Result /= Void
		end

	nm5y: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("net_margin_five_year_average", "nm5y", Void, null_integer, null_integer, null_integer, 1, Void, k_double, Void, k_company, 'X', k_valdata_signature, "1", "", 2238)
		end

	i_txr: VARIABLE_DESCRIPTION_RECORD is
			--tax_rate
		once
			create Result.make_risk ("tax_rate", "txrt", Void, Void, null_integer, null_integer, 2, Void, missing_double, missing_double, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", "", 2237)
		ensure
			not_void: Result /= Void
		end

	i_spp: VARIABLE_DESCRIPTION_RECORD is
			--i_spp
		once
			create Result.make_mondat ("i_spp", "ispp", Void, k_mondat, null_integer, null_integer, null_integer, null_integer, null_integer, Void, k_double, Void, Void, k_company, ' ', k_monthly_data, "", "", 2236)
		ensure
			not_void: Result /= Void
		end

	erv6_sm: VARIABLE_DESCRIPTION_RECORD is
			-- Smoothed 6 month earnings revision
		once
			create Result.make_risk ("earnings_revision_6_month_smoothed", "erv6_sm", "erv6_sm", k_risk, null_integer, null_integer, 2, Void, -5.0, 5.0, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", "", 2235)
		end

	erv6: VARIABLE_DESCRIPTION_RECORD is
			-- 6 month earnings revision
		once
			create Result.make_risk ("earnings_revision_6_month", "erv6", "erv6", k_risk, null_integer, null_integer, 2, Void, -5.0, 5.0, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", "", 2234)
		end

	erv3: VARIABLE_DESCRIPTION_RECORD is
			-- 3 month earnings revision
		once
			create Result.make_risk ("earnings_revision_3_month", "erv3", "erv3", k_risk, null_integer, null_integer, 2, Void, -5.0, 5.0, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", "", 2233)
		end

	i_bvps: VARIABLE_DESCRIPTION_RECORD is
			--base_value_per_share
		once
			create Result.make_risk ("base_value_per_share", "bvps", Void, Void, null_integer, null_integer, 2, Void, missing_double, missing_double, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", "", 2231)
		ensure
			not_void: Result /= Void
		end

	i_bps: VARIABLE_DESCRIPTION_RECORD is
			--intermediate_book_per_share
		once
			create Result.make_risk ("intermediate_book_per_share", "ibps", Void, Void, null_integer, null_integer, 2, Void, missing_double, missing_double, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", "", 2230)
		ensure
			not_void: Result /= Void
		end

	i_beps: VARIABLE_DESCRIPTION_RECORD is
			--base_eps
		once
			create Result.make_risk ("base_eps", "beps", Void, Void, null_integer, null_integer, 2, Void, missing_double, missing_double, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", "", 2229)
		ensure
			not_void: Result /= Void
		end

	i_beb: VARIABLE_DESCRIPTION_RECORD is
			--base_earnings_below_average
		once
			create Result.make_risk ("base_earnings_below_average", "beb", Void, Void, null_integer, null_integer, 2, Void, missing_double, missing_double, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", "", 2228)
		ensure
			not_void: Result /= Void
		end

	i_bea: VARIABLE_DESCRIPTION_RECORD is
			--base_earnings_above_average
		once
			create Result.make_risk ("base_earnings_above_average", "bea", Void, Void, null_integer, null_integer, 2, Void, missing_double, missing_double, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", "", 2227)
		ensure
			not_void: Result /= Void
		end

	i_bdip: VARIABLE_DESCRIPTION_RECORD is
			--bayes_fitted_ibes_etop_1yr_delta_to_xstp
		once
			create Result.make_risk ("bayes_fitted_ibes_etop_1yr_delta_to_xstp", "bdip", Void, Void, null_integer, null_integer, 2, Void, missing_double, missing_double, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", "", 2226)
		ensure
			not_void: Result /= Void
		end

	i_bbv: VARIABLE_DESCRIPTION_RECORD is
			--base_earnings_below_average_times_earnings_variance
		once
			create Result.make_risk ("base_earnings_below_average_times_earnings_variance", "bbv", Void, Void, null_integer, null_integer, 2, Void, missing_double, missing_double, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", "", 2225)
		ensure
			not_void: Result /= Void
		end

	i_bav: VARIABLE_DESCRIPTION_RECORD is
			--base_earnings_above_average_times_earnings_variance
		once
			create Result.make_risk ("base_earnings_above_average_times_earnings_variance", "bav", Void, Void, null_integer, null_integer, 2, Void, missing_double, missing_double, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", "", 2224)
		ensure
			not_void: Result /= Void
		end

	i_aroe: VARIABLE_DESCRIPTION_RECORD is
			--average_return_on_equity
		once
			create Result.make_risk ("average_return_on_equity", "aroe", "AverageRtrnEqt", k_risk, null_integer, null_integer, 2, "", -4, 4, 0.15, 0.25, "", Void, Void, ' ', k_monthly_data, "", "", 2223)
		ensure
			not_void: Result /= Void
		end

	i_ib1: VARIABLE_DESCRIPTION_RECORD is
			--etop_ibes_1yr_delta_to_etop
		once
			create Result.make_risk ("etop_ibes_1yr_delta_to_etop", "ib1", Void, Void, null_integer, null_integer, 2, Void, missing_double, missing_double, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", "", 2222)
		ensure
			not_void: Result /= Void
		end

	i_exsp: VARIABLE_DESCRIPTION_RECORD is
			--earnings_excluding_extraordinary_items_to_price
		once
			create Result.make_risk ("earnings_excluding_extraordinary_items_to_price", "exsp", Void, Void, null_integer, null_integer, 2, Void, missing_double, missing_double, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", "", 2221)
		ensure
			not_void: Result /= Void
		end

	i_ditp: VARIABLE_DESCRIPTION_RECORD is
			--delta_inventory_to_price_limited_to_plus_minus_50_percent
		once
			create Result.make_risk ("delta_inventory_to_price_limited_to_plus_minus_50_percent", "diph", Void, Void, null_integer, null_integer, 2, Void, missing_double, missing_double, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", "", 2220)
		ensure
			not_void: Result /= Void
		end

	epsa: VARIABLE_DESCRIPTION_RECORD is
			--earnings per share midcycle
		once
			create Result.make_mondat ("earnings_per_share_midcycle", "epsa", Void, "RISK", null_integer, null_integer, null_integer, null_integer, 2, k_currency_per_share, k_double, Void, Void, k_company, ' ', k_monthly_data, "", "", 2122)
		ensure
			not_void: Result /= Void
		end

	pxsc: VARIABLE_DESCRIPTION_RECORD is
			-- Percent sales foreign classified
		once
			create Result.make_valdata ("percent_sales_foreign_classified", "pxsc", "misc", null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'X', k_valdata_signature, "1", k_miscellaneous, 2120)
		end

	exst: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("exit_stage", "exst", "exst", k_risk, null_integer, null_integer, 2, Void, -3, 3, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", "", 2119)
		end

	enst: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("entry_stage", "enst", "enst", k_risk, null_integer, null_integer, 2, Void, -3, 3, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", "", 2118)
		end

	tbst: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("troubled", "tbst", "tbst", k_risk, null_integer, null_integer, 2, Void, -3, 3, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", "", 2117)
		end

	alfe: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondex ("alfe", "alfe", k_mondex, Void, Void, null_integer, null_integer, null_integer, null_integer, 3, Void, k_double, Void, Void, k_company, ' ', k_mondex, k_risk, 2096)
		end

	alfd: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondex ("alfd", "alfd", k_mondex, Void, Void, null_integer, null_integer, null_integer, null_integer, 3, Void, k_double, Void, Void, k_company, ' ', k_mondex, k_risk, 2095)
		end

	suse: VARIABLE_DESCRIPTION_RECORD is
			--Sustainable earnings
		once
			create Result.make_risk ("sustainable_earnings", "suse", "SUSE", k_risk, null_integer, 150, 2, "", -4, 4, 0.04, 0.04, "", Void, Void, '0', k_monthly_data, "", k_risk, 2091)
		end

	bqty: VARIABLE_DESCRIPTION_RECORD is
			--Book quality growth
		once
			create Result.make_valdata ("book_quality_growth", "bqty", k_asst, 0, 0, 0, 2, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", "balance_sheet_synthesis", 2090)
		end

	rstn: VARIABLE_DESCRIPTION_RECORD is
			-- Theme Normalized Relative Strength
		once
			create Result.make_risk ("relative_strength_theme_nor", "rstn", "RsSt.THN", Void, null_integer, null_integer, 2, "", -3, 3, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", k_risk, 2089)
		ensure
			not_void: Result /= Void
		end

	falphat10: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("falphat10", "falphat10", Void, Void, null_integer, null_integer, 3, Void, missing_double, missing_double, missing_double, missing_double, "", Void, Void, ' ', k_model_company_signature, "", "", 2088)
		end

	falphat09: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("falphat09", "falphat09", Void, Void, null_integer, null_integer, 3, Void, missing_double, missing_double, missing_double, missing_double, "", Void, Void, ' ', k_model_company_signature, "", "", 2087)
		end

	falphat08: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("falphat08", "falphat08", Void, Void, null_integer, null_integer, 3, Void, missing_double, missing_double, missing_double, missing_double, "", Void, Void, ' ', k_model_company_signature, "", "", 2086)
		end

	falphat07: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("falphat07", "falphat07", Void, Void, null_integer, null_integer, 3, Void, missing_double, missing_double, missing_double, missing_double, "", Void, Void, ' ', k_model_company_signature, "", "", 2085)
		end

	falphat06: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("falphat06", "falphat06", Void, Void, null_integer, null_integer, 3, Void, missing_double, missing_double, missing_double, missing_double, "", Void, Void, ' ', k_model_company_signature, "", "", 2084)
		end

	falphat05: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("falphat05", "falphat05", Void, Void, null_integer, null_integer, 3, Void, missing_double, missing_double, missing_double, missing_double, "", Void, Void, ' ', k_model_company_signature, "", "", 2083)
		end

	falphat04: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("falphat04", "falphat04", Void, Void, null_integer, null_integer, 3, Void, missing_double, missing_double, missing_double, missing_double, "", Void, Void, ' ', k_model_company_signature, "", "", 2082)
		end

	falphat03: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("falphat03", "falphat03", Void, Void, null_integer, null_integer, 3, Void, missing_double, missing_double, missing_double, missing_double, "", Void, Void, ' ', k_model_company_signature, "", "", 2081)
		end

	falphat02: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("falphat02", "falphat02", Void, Void, null_integer, null_integer, 3, Void, missing_double, missing_double, missing_double, missing_double, "", Void, Void, ' ', k_model_company_signature, "", "", 2080)
		end

	falphat01: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("falphat01", "falphat01", Void, Void, null_integer, null_integer, 3, Void, missing_double, missing_double, missing_double, missing_double, "", Void, Void, ' ', k_model_company_signature, "", "", 2079)
		end

	rr16: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("raw_risk_state_16", "rr16", Void, k_risk, null_integer, null_integer, 2, Void, -5, 5, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", k_risk, 2078)
		end

	rr15: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("raw_risk_state_15", "rr15", Void, k_risk, null_integer, null_integer, 2, Void, -5, 5, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", k_risk, 2077)
		end

	rr14: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("raw_risk_state_14", "rr14", Void, k_risk, null_integer, null_integer, 2, Void, -5, 5, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", k_risk, 2076)
		end

	rr13: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("raw_risk_state_13", "rr13", Void, k_risk, null_integer, null_integer, 2, Void, -5, 5, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", k_risk, 2075)
		end

	rr12: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("raw_risk_state_12", "rr12", Void, k_risk, null_integer, null_integer, 2, Void, -5, 5, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", k_risk, 2074)
		end

	rr11: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("raw_risk_state_11", "rr11", Void, k_risk, null_integer, null_integer, 2, Void, -5, 5, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", k_risk, 2073)
		end

	rr10: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("raw_risk_state_10", "rr10", Void, k_risk, null_integer, null_integer, 2, Void, -5, 5, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", k_risk, 2072)
		end

	rr09: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("raw_risk_state_9", "rr09", Void, k_risk, null_integer, null_integer, 2, Void, -5, 5, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", k_risk, 2071)
		end

	rr08: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("raw_risk_state_8", "rr08", Void, k_risk, null_integer, null_integer, 2, Void, -5, 5, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", k_risk, 2070)
		end

	rr07: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("raw_risk_state_7", "rr07", Void, k_risk, null_integer, null_integer, 2, Void, -5, 5, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", k_risk, 2069)
		end

	rr06: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("raw_risk_state_6", "rr06", Void, k_risk, null_integer, null_integer, 2, Void, -5, 5, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", k_risk, 2068)
		end

	rr05: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("raw_risk_state_5", "rr05", Void, k_risk, null_integer, null_integer, 2, Void, -5, 5, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", k_risk, 2067)
		end

	rr04: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("raw_risk_state_4", "rr04", Void, k_risk, null_integer, null_integer, 2, Void, -5, 5, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", k_risk, 2066)
		end

	rr03: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("raw_risk_state_3", "rr03", Void, k_risk, null_integer, null_integer, 2, Void, -5, 5, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", k_risk, 2065)
		end

	rr02: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("raw_risk_state_2", "rr02", Void, k_risk, null_integer, null_integer, 2, Void, -5, 5, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", k_risk, 2064)
		end

	rr01: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("raw_risk_state_1", "rr01", Void, k_risk, null_integer, null_integer, 2, Void, -5, 5, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", k_risk, 2063)
		end

	tkpa: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondex ("toyokeizai_12m_trailing_eps_parent", "tkpa", k_mondex, "", k_per_shares, null_integer, null_integer, null_integer, null_integer, 2, k_currency_per_share, k_double, Void, "", k_company, ' ', k_monthly_data, k_mondat, 2061)
		end

	drsn: VARIABLE_DESCRIPTION_RECORD is
			-- Similars Normalized: Projected change of recurring earnings to price
		once
			create Result.make_risk ("drhp_sim_nor", "drsn", "drhp.SMN", k_risk, null_integer, null_integer, 2, "", -3, 3, missing_double, missing_double, "SIMNORM:DRHP", Void, Void, ' ', k_monthly_data, "", k_risk, 2059)
		end

	glre: VARIABLE_DESCRIPTION_RECORD is
			-- Earnings based return on equity
		once
			create Result.make_risk ("robust_return_on_equity", "glre", Void, k_risk, null_integer, null_integer, 2, Void, -3, 3, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", k_risk, 2049)
		end

	tkec: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("toyokeizai_earnings_per_share_consolidated", "tkec", "", k_per_shares, null_integer, null_integer, null_integer, null_integer, 2, k_currency_per_share, k_double, Void, "", k_company, '0', k_monthly_data, k_per_share, k_mondat, 2048)
		end

	cssn: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("cssn", "cssn", "cpst.SMN", k_risk, null_integer, null_integer, 2, "", -3, 3, missing_double, missing_double, "SIMNORM:CPST", Void, Void, ' ', k_monthly_data, "", k_risk, 2047)
		end

	gssn: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("gssn", "gssn", "grst.SMN", k_risk, null_integer, null_integer, 2, "", -3, 3, missing_double, missing_double, "SIMNORM:GRST", Void, Void, ' ', k_monthly_data, "", k_risk, 2046)
		end

	emsn: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("emsn", "emsn", "egst.SMN", k_risk, null_integer, null_integer, 2, "", -3, 3, missing_double, missing_double, "SIMNORM:EGST", Void, Void, ' ', k_monthly_data, "", k_risk, 2045)
		end

	mrsn: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("mrsn", "mrsn", "mtst.SMN", k_risk, null_integer, null_integer, 2, "", -3, 3, missing_double, missing_double, "SIMNORM:MTST", Void, Void, ' ', k_monthly_data, "", k_risk, 2044)
		end

	cpst: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("coper", "cpst", "cpst", k_risk, null_integer, null_integer, 2, Void, -3, 3, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", "", 2043)
		end

	mtst: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("mature", "mtst", "mtst", k_risk, null_integer, null_integer, 2, Void, -3, 3, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", "", 2042)
		end

	grst: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("growth_stage", "grst", "grst", k_risk, null_integer, null_integer, 2, Void, -3, 3, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", "", 2041)
		end

	egst: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("emerging_growth", "egst", "egst", k_risk, null_integer, null_integer, 2, Void, -3, 3, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", "", 2040)
		end

	crsn: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("crsn", "crsn", "cpbd.SMN", k_risk, null_integer, null_integer, 2, "", -3, 3, missing_double, missing_double, "SIMNORM:CPBD", Void, Void, ' ', k_monthly_data, "", k_risk, 2039)
		end

	mtsn: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("mtsn", "mtsn", "mtbd.SMN", k_risk, null_integer, null_integer, 2, "", -3, 3, missing_double, missing_double, "SIMNORM:MTBD", Void, Void, ' ', k_monthly_data, "", k_risk, 2038)
		end

	grsn: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("grsn", "grsn", "grbd.SMN", k_risk, null_integer, null_integer, 2, "", -3, 3, missing_double, missing_double, "SIMNORM:GRBD", Void, Void, ' ', k_monthly_data, "", k_risk, 2037)
		end

	egsn: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("egsn", "egsn", "egbd.SMN", k_risk, null_integer, null_integer, 2, "", -3, 3, missing_double, missing_double, "SIMNORM:EGBD", Void, Void, ' ', k_monthly_data, "", k_risk, 2036)
		end

	cpbd: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("coper_boundary", "cpbd", "cpbd", k_risk, null_integer, null_integer, 2, Void, -3, 3, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", "", 2035)
		end

	mtbd: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("mature_boundary", "mtbd", "mtbd", k_risk, null_integer, null_integer, 2, Void, -3, 3, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", "", 2034)
		end

	grbd: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("growth_boundary", "grbd", "grbd", k_risk, null_integer, null_integer, 2, Void, -3, 3, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", "", 2033)
		end

	egbd: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("emerging_growth_boundary", "egbd", "egbd", k_risk, null_integer, null_integer, 2, Void, -3, 3, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", "", 2032)
		end

	tkea: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("toyokeizai_12m_trailing_eps_consolidated", "tkea", "", k_per_shares, null_integer, null_integer, null_integer, null_integer, 2, k_currency_per_share, k_double, Void, "", k_company, '0', k_monthly_data, k_per_share, k_mondat, 2031)
		end

	hbsn: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("high_road_boundary_metric_sim_nor", "hbsn", "hrbd.SMN", k_risk, null_integer, null_integer, 2, "", -3, 3, missing_double, missing_double, "SIMNORM:HRBD", Void, Void, ' ', k_monthly_data, "", k_risk, 2020)
		end

	mird: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("middle_road_observation", "mird", Void, Void, null_integer, null_integer, null_integer, null_integer, 2, Void, k_double, Void, Void, k_company, ' ', k_monthly_data, "", k_mondat, 2019)
		end

	hrbd: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("high_road_boundary_metric", "hrbd", Void, k_risk, null_integer, null_integer, 2, Void, -3, 3, missing_double, missing_double, "", Void, Void, '0', k_monthly_data, "", "", 2018)
		end

	lrbd: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("low_road_boundary_metric", "lrbd", Void, k_risk, null_integer, null_integer, 2, Void, -3, 3, missing_double, missing_double, "", Void, Void, '0', k_monthly_data, "", "", 2017)
		end

	nwth: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("new_with_history_observation", "nwth", Void, Void, null_integer, null_integer, null_integer, null_integer, 2, Void, k_double, Void, Void, k_company, ' ', k_monthly_data, "", k_mondat, 2016)
		end

	dprt: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("departed_observation", "dprt", Void, Void, null_integer, null_integer, null_integer, null_integer, 2, Void, k_double, Void, Void, k_company, ' ', k_monthly_data, "", k_mondat, 2015)
		end

	stup: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("startup_observation", "stup", Void, Void, null_integer, null_integer, null_integer, null_integer, 2, Void, k_double, Void, Void, k_company, ' ', k_monthly_data, "", k_mondat, 2014)
		end

	hird: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("high_road_observation", "hird", Void, Void, null_integer, null_integer, null_integer, null_integer, 2, Void, k_double, Void, Void, k_company, ' ', k_monthly_data, "", k_mondat, 2013)
		end

	lord: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("low_road_observation", "lord", Void, Void, null_integer, null_integer, null_integer, null_integer, 2, Void, k_double, Void, Void, k_company, ' ', k_monthly_data, "", k_mondat, 2012)
		end

	cosn: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("operating_cash_flow_to_price_sim_norm", "cosn", "CFOP.SMN", k_risk, null_integer, null_integer, 2, "", -3, 3, missing_double, missing_double, "SIMNORM:CFOP", Void, Void, ' ', k_monthly_data, "", k_risk, 2011)
		end

	ffsn: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("cash_flow_to_price_sim_norm", "ffsn", "CTOP.SMN", k_risk, null_integer, null_integer, 2, "", -3, 3, missing_double, missing_double, "SIMNORM:CTOP", Void, Void, ' ', k_monthly_data, "", k_risk, 2010)
		end

	sdadj: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("stock_distribution_adjustment", "sdadj", "", null_integer, null_integer, null_integer, 2, "", k_double, Void, k_company, 'X', k_valdata_signature, "", "", 2009)
		end

	ldadj: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("leading_flow_distribution_adjustment", "ldadj", "", null_integer, null_integer, null_integer, 2, "", k_double, Void, k_company, 'X', k_valdata_signature, "", "", 2008)
		end

	tdadj: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("trailing_flow_distribution_adjustment", "tdadj", "", null_integer, null_integer, null_integer, 2, "", k_double, Void, k_company, 'X', k_valdata_signature, "", "", 2007)
		end

	mqra: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondex ("mqas_adjusted", "mqra", k_mondex, Void, Void, null_integer, null_integer, null_integer, null_integer, 3, Void, k_double, Void, Void, k_company, ' ', k_mondex, k_risk, 2006)
		end

	esra: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondex ("earnings_per_share_sino", "esra", k_mondex, Void, Void, null_integer, null_integer, null_integer, null_integer, 3, Void, k_double, Void, Void, k_company, ' ', k_mondex, k_risk, 2005)
		end

	cara: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondex ("caeq_adj", "cara", k_mondex, Void, Void, null_integer, null_integer, null_integer, null_integer, 3, Void, k_double, Void, Void, k_company, ' ', k_mondex, k_risk, 2004)
		end

	cfop: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("operating_cash_flow_to_price", "cfop", Void, k_risk, null_integer, null_integer, 2, "", -3, 3, 0.07, 0.1, "", Void, Void, ' ', k_monthly_data, "", k_risk, 2002)
		end

	cpsn: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("caeq_sim_nor", "cpsn", "caeq.SMN", k_risk, null_integer, null_integer, 2, "", -3, 3, missing_double, missing_double, "SIMNORM:CAEQ", Void, Void, ' ', k_monthly_data, "", k_risk, 2001)
		end

	caeq: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondex ("current_alpha_adjusted", "caeq", k_mondex, Void, Void, null_integer, null_integer, null_integer, null_integer, 3, Void, k_double, Void, Void, k_company, ' ', k_mondex, k_risk, 2000)
		end

	fxre: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("ff_exchange_rate_effect", "fxre", k_ff, null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "1", k_flow_of_funds, 1996)
		end

	ffao: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("ff_financing_activity_other", "ffao", k_ff, null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "1", k_flow_of_funds, 1995)
		end

	fcdc: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("ff_st_debt_change", "fcdc", k_ff, null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "1", k_flow_of_funds, 1994)
		end

	fiao: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("ff_investing_activity_other", "fiao", k_ff, null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "1", k_flow_of_funds, 1993)
		end

	falc: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("ff_asst_libt_change", "falc", k_ff, null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "1", k_flow_of_funds, 1992)
		end

	ftxc: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("ff_tax_payable_change", "ftxc", k_ff, null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "1", k_flow_of_funds, 1991)
		end

	fapc: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("ff_acc_payable_change", "fapc", k_ff, null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "1", k_flow_of_funds, 1990)
		end

	finc: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("ff_inventory_change", "finc", k_ff, null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "1", k_flow_of_funds, 1989)
		end

	frec: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("ff_receivable_change", "frec", k_ff, null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "1", k_flow_of_funds, 1988)
		end

	fopo: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("ff_funds_from_oper_other", "fopo", k_ff, null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "1", k_flow_of_funds, 1987)
		end

	fopt: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("ff_funds_from_oper_total", "fopt", k_ff, null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "1", k_flow_of_funds, 1986)
		end

	erhp: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("projected_recurring_earnings_to_price", "erhp", "PrjReE/P", "", null_integer, 148, 3, "", -4, 4, missing_double, missing_double, "", Void, "1", '0', k_monthly_data, "", k_risk, 1985)
		end

	dxhp: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("projected_delta_extraordinary_earnings_to_price", "dxhp", "PrjDxE/P", "", null_integer, 147, 3, "", -4, 4, missing_double, missing_double, "", Void, "1", '0', k_monthly_data, "", k_risk, 1984)
		end

	drhp: VARIABLE_DESCRIPTION_RECORD is
			-- Projected change of recurring earnings to price
		once
			create Result.make_risk ("projected_delta_recurring_earnings_to_price", "drhp", "PrjDrE/P", "", null_integer, 146, 3, "", -4, 4, missing_double, missing_double, "", Void, "1", '0', k_monthly_data, "", k_risk, 1983)
		ensure
			not_void: Result /= Void
		end

	rqsn: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("roeg_sim_nor", "rqsn", "roeg.SMN", k_risk, null_integer, null_integer, 2, "", -3, 3, missing_double, missing_double, "SIMNORM:ROEG", Void, Void, ' ', k_monthly_data, "", k_risk, 1981)
		end

	etsn: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("ext_earnings_to_price_sim_norm", "etsn", "ESTP.SMN", k_risk, null_integer, null_integer, 2, "", -3, 3, missing_double, missing_double, "SIMNORM:ESTP", Void, "", '0', k_monthly_data, "", k_risk, 1979)
		end

	estp: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("ext_earnings_to_price_index", "estp", "ESTP", k_risk, null_integer, null_integer, 2, "", -3, 3, missing_double, missing_double, "ETOP_RISK*.3+IBEP*.3+CTOP*.2+XSTP*.2+DSTP*.2", Void, "", '0', k_monthly_data, "", k_risk, 1978)
		end

	dpsn: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("delta_earnings_to_price_sim_norm", "dpsn", "DETP.SMN", k_risk, null_integer, null_integer, 2, "", -3, 3, missing_double, missing_double, "SIMNORM:DETP", Void, "", '0', k_monthly_data, "", k_risk, 1977)
		end

	tdep: VARIABLE_DESCRIPTION_RECORD is
			-- Change in trailing 1 year earnings yield
		once
			create Result.make_risk ("change_in_earnings_to_price", "tdep", "DELTETOP", k_risk, null_integer, null_integer, 2, "", -3, 3, 0.96, 1, "", Void, "", '0', k_monthly_data, "", k_risk, 1976)
		ensure
			not_void: Result /= Void
		end

	spsn: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("sales_to_price_sim_norm", "spsn", "SPSN.SMN", k_risk, null_integer, null_integer, 2, "", -3, 3, missing_double, missing_double, "SIMNORM:STOP", Void, "", '0', k_monthly_data, "", k_risk, 1975)
		end

	dssn: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("delta_sales_to_price_sim_norm", "dssn", "DSTP.SMN", k_risk, null_integer, null_integer, 2, "", -3, 3, missing_double, missing_double, "SIMNORM:DSTP", Void, "", '0', k_monthly_data, "", k_risk, 1974)
		end

	dstp: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("change_in_sales_to_price", "dstp", "DELTSTOP", k_risk, null_integer, null_integer, 2, "", -3, 3, 0.96, 1, "", Void, "", '0', k_monthly_data, "", k_risk, 1973)
		end

	dbrk: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("data_break", "dbrk", "", "", null_integer, null_integer, null_integer, null_integer, 2, "", k_double, Void, "", k_company, '0', k_monthly_data, "", k_mondat, 1972)
		end

	z_primary_currency_m: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("z_primary_market_price_currency", "z_primary_currency_m", "", "", null_integer, null_integer, null_integer, null_integer, 2, "", k_double, Void, "", k_company, 'F', k_monthly_data, "", "", 1971)
		end

	srtg: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("sap_rating", "srtg", "", "", null_integer, null_integer, null_integer, null_integer, 2, "", k_double, Void, "", k_company, '0', k_monthly_data, "", "", 1970)
		end

	mrtg: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("moody_rating", "mrtg", "", "", null_integer, null_integer, null_integer, null_integer, 2, "", k_double, Void, "", k_company, '0', k_monthly_data, "", "", 1969)
		end

	ter2: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("rbc_earnings_revisions_fy2", "ter2", "", k_per_shares, null_integer, null_integer, null_integer, null_integer, 2, k_currency_per_share, k_double, Void, "", k_company, '0', k_monthly_data, k_per_share, k_mondat, 1962)
		end

	ter1: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("rbc_earnings_revisions_fy1", "ter1", "", k_per_shares, null_integer, null_integer, null_integer, null_integer, 2, k_currency_per_share, k_double, Void, "", k_company, '0', k_monthly_data, k_per_share, k_mondat, 1961)
		end

	andt: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("announcement_date_fy0_eps", "andt", "", "", null_integer, null_integer, null_integer, null_integer, 2, "", k_integer, Void, "", k_company, 'F', k_monthly_data, "", "", 1960)
		end

	ify2: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("ibes_fy2_date", "ify2", "", "", null_integer, null_integer, null_integer, null_integer, 2, "", k_integer, Void, "", k_company, 'F', k_monthly_data, "", "", 1959)
		end

	dadj: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("distribution_adjustment", "dadj", "", "", null_integer, null_integer, null_integer, null_integer, 2, "", k_double, Void, "", k_company, '0', k_monthly_data, "", k_mondat, 1958)
		end

	ify1: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("ibes_fy1_date", "ify1", "", "", null_integer, null_integer, null_integer, null_integer, 2, "", k_integer, Void, "", k_company, 'F', k_monthly_data, "", "", 1957)
		end

	rbe2: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("rbc_fy2_eps", "rbe2", "", k_per_shares, null_integer, null_integer, null_integer, null_integer, 2, k_currency_per_share, k_double, Void, "", k_company, '0', k_monthly_data, k_per_share, k_mondat, 1956)
		end

	rbe1: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("rbc_fy1_eps", "rbe1", "", k_per_shares, null_integer, null_integer, null_integer, null_integer, 2, k_currency_per_share, k_double, Void, "", k_company, '0', k_monthly_data, k_per_share, k_mondat, 1955)
		end

	tkep: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("toyokeizai_earnings_per_share", "tkep", "", k_per_shares, null_integer, null_integer, null_integer, null_integer, 2, k_currency_per_share, k_double, Void, "", k_company, '0', k_monthly_data, k_per_share, k_mondat, 1954)
		end

	ibea: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("ibes_act_earnings_per_share", "ibea", "record.mdc.ibes", k_per_shares, null_integer, null_integer, null_integer, null_integer, 2, k_currency_per_share, k_double, Void, "", k_company, '0', k_monthly_data, k_per_share, k_mondat, 1953)
		end

	z_if_fundamental: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("z_if_fundamental", "z_if_fundamental", "", "", null_integer, null_integer, null_integer, null_integer, 2, "", k_integer, Void, "", k_company, 'F', k_monthly_data, "", "", 1952)
		end

	eqsn: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("eqaa_sim_nor", "eqsn", "eqaa.SMN", k_risk, null_integer, null_integer, 2, "", -3, 3, missing_double, missing_double, "SIMNORM:EQAA", Void, Void, ' ', k_monthly_data, "", k_risk, 1951)
		end

	rdsn: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("rder_sim_nor", "rdsn", "rder.SMN", k_risk, null_integer, null_integer, 2, "", -3, 3, missing_double, missing_double, "SIMNORM:RDER", Void, Void, ' ', k_monthly_data, "", k_risk, 1949)
		end

	rder: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondex ("radial_directional", "rder", k_mondex, Void, Void, null_integer, null_integer, null_integer, null_integer, 3, Void, k_double, Void, Void, k_company, ' ', k_mondex, k_risk, 1948)
		end

	cibc: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondex ("trailing_ibes_change_proxcap_nor", "cibc", k_mondex, Void, Void, null_integer, null_integer, null_integer, null_integer, 3, Void, k_double, Void, Void, k_company, ' ', k_mondex, k_risk, 1947)
		end

	cbsn: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("cibc_sim_nor", "cbsn", "cibc.SMN", k_risk, null_integer, null_integer, 2, "", -3, 3, missing_double, missing_double, "SIMNORM:CIBC", Void, Void, ' ', k_monthly_data, "", k_risk, 1946)
		end

	ibsn: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("ibep_sim_nor", "ibsn", "ibep.SMN", k_risk, null_integer, null_integer, 2, "", -3, 3, missing_double, missing_double, "SIMNORM:IBEP", Void, Void, ' ', k_monthly_data, "", k_risk, 1945)
		end

	tvsn: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("tval_sim_nor", "tvsn", "tval.SMN", k_risk, null_integer, null_integer, 2, "", -3, 3, missing_double, missing_double, "SIMNORM:TVAL", Void, Void, ' ', k_monthly_data, "", k_risk, 1943)
		end

	efsn: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("ep12_sim_nor", "efsn", "ep12.SMN", k_risk, null_integer, null_integer, 2, "", -3, 3, missing_double, missing_double, "SIMNORM:EP12", Void, Void, ' ', k_monthly_data, "", k_risk, 1942)
		end

	tval: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondex ("total_value_ep_bp_dp", "tval", k_mondex, Void, Void, null_integer, null_integer, null_integer, null_integer, 3, Void, k_double, Void, Void, k_company, ' ', k_mondex, k_risk, 1941)
		end

	ep12: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondex ("12m_forward_e_to_p_proxcap_nor", "ep12", k_mondex, Void, Void, null_integer, null_integer, null_integer, null_integer, 3, Void, k_double, Void, Void, k_company, ' ', k_mondex, k_risk, 1940)
		end

	ebsn: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("er_h_to_p_sim_norm_bervqx", "ebsn", "EHPB.SMN", k_risk, null_integer, 0, 3, "", -3, 3, missing_double, missing_double, "SIMNORM:EHPB", Void, Void, ' ', k_monthly_data, "", k_risk, 1939)
		end

	desn: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("dehp_sim_nor", "desn", "dehp.SMN", k_risk, null_integer, null_integer, 2, "", -3, 3, missing_double, missing_double, "SIMNORM:DEHP", Void, Void, ' ', k_monthly_data, "", k_risk, 1938)
		end

	dbsn: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("dehp_bervqx_sim_nor", "dbsn", "dehb.SMN", k_risk, null_integer, null_integer, 2, "", -3, 3, missing_double, missing_double, "SIMNORM:DEHB", Void, Void, ' ', k_monthly_data, "", k_risk, 1937)
		end

	ehpb: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("projected_earnings_to_price_bervqx", "ehpb", "EHTP_BER", "", null_integer, 149, 3, "", -4, 4, missing_double, missing_double, "", Void, "1", '0', k_monthly_data, "", k_risk, 1936)
		end

	dehb: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("projected_delta_earnings_to_price_bervqx", "dehb", "DEHP_BER", "", null_integer, 145, 3, "", -4, 4, missing_double, missing_double, "", Void, "1", '0', k_monthly_data, "", k_risk, 1935)
		end

	frtg: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("fundamental_rating", "frtg", "FRTG", k_risk, null_integer, null_integer, 2, "", -3, 3, missing_double, missing_double, "", Void, "", 'F', k_monthly_data, "", "", 1934)
		end

	m1ei: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("usd_libor_1m_yield", "m1ei", "M1LBR_EI", k_risk, null_integer, null_integer, 2, "", -3, 3, missing_double, missing_double, "", Void, "", '0', k_monthly_data, "", "", 1933)
		end

	cdei: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("us_credit_spread", "cdei", "CTSPD_EI", k_risk, null_integer, null_integer, 2, "", -3, 3, missing_double, missing_double, "", Void, "", '0', k_monthly_data, "", "", 1932)
		end

	usei: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("usd_trade_weighted_exchange_rate", "usei", "USD_EI", k_risk, null_integer, null_integer, 2, "", -3, 3, missing_double, missing_double, "", Void, "", '0', k_monthly_data, "", "", 1931)
		end

	tyei: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("us_treasury_10_years_yield", "tyei", "TNYLD_EI", k_risk, null_integer, null_integer, 2, "", -3, 3, missing_double, missing_double, "", Void, "", '0', k_monthly_data, "", "", 1930)
		end

	olei: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("oil_brent_wti_avg", "olei", "OIL_EI", k_risk, null_integer, null_integer, 2, "", -3, 3, missing_double, missing_double, "", Void, "", '0', k_monthly_data, "", "", 1929)
		end

	cmei: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("crb_continuous_commodity_index", "cmei", "CMDTS_EI", k_risk, null_integer, null_integer, 2, "", -3, 3, missing_double, missing_double, "", Void, "", '0', k_monthly_data, "", "", 1928)
		end

	mctp: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_computed ("mature_group_ctop", "mctp", "mature_group * cash_flow_to_price_desc.as_normalized_by_country", k_computeds, 2, k_double, "mctp", Void, k_company, "rose_entity_date", "", "", 1925)
		end

	gctp: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_computed ("grower_group_ctop", "gctp", "grower_group * cash_flow_to_price_desc.as_normalized_by_country", k_computeds, 2, k_double, "gctp", Void, k_company, "rose_entity_date", "", "", 1924)
		end

	mxsp: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_computed ("mature_group_xstp", "mxsp", "mature_group * er_ex_ex_sp_to_p.as_normalized_by_country", k_computeds, 2, k_double, "mxsp", Void, k_company, "rose_entity_date", "", "", 1923)
		end

	gxsp: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_computed ("grower_group_xstp", "gxsp", "grower_group * er_ex_ex_sp_to_p.as_normalized_by_country", k_computeds, 2, k_double, "gxsp", Void, k_company, "rose_entity_date", "", "", 1922)
		end

	fegroc_n: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("fwd_eps_growth_anly_count", "fegroc_n", "", k_risk, null_integer, null_integer, null_integer, null_integer, 2, "", k_double, Void, "", k_company, 'F', k_monthly_data, "", "", 1921)
		end

	fegroc_std: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("fwd_eps_growth_standard_deviation", "fegroc_std", "", k_risk, null_integer, null_integer, 2, "", -3, 3, missing_double, missing_double, "", Void, "", 'F', k_monthly_data, "", "", 1920)
		end

	fegroc: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("fwd_eps_growth_certainty", "fegroc", "", k_risk, null_integer, null_integer, 2, "", -3, 3, missing_double, missing_double, "", Void, "", 'F', k_monthly_data, "", "", 1919)
		end

	sgrk: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("similars_group_rank", "sgrk", Void, Void, null_integer, null_integer, null_integer, null_integer, 2, Void, k_integer, Void, Void, k_company, '0', k_monthly_data, "", k_mondat, 1917)
		end

	dcut: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("probability_of_dividend_cut", "dcut", "DCUTPROB", k_risk, null_integer, null_integer, 2, "", -4, 4, 0.05, 0.1, "", Void, Void, '0', k_monthly_data, "", k_risk, 1915)
		end

	mdta: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_computed ("mature_group_dtoa", "mdta", "mature_group * debt_to_assets.as_normalized_by_country", k_computeds, 2, k_double, "mdta", Void, k_company, "rose_entity_date", "", "", 1914)
		end

	gdta: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_computed ("grower_group_dtoa", "gdta", "grower_group * debt_to_assets.as_normalized_by_country", k_computeds, 2, k_double, "gdta", Void, k_company, "rose_entity_date", "", "", 1913)
		end

	mre1: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_computed ("mature_group_roe", "mre1", "mature_group * return_on_equity.as_normalized_by_country", k_computeds, 2, k_double, "mre1", Void, k_company, "rose_entity_date", "", "", 1912)
		end

	gre1: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_computed ("grower_group_roe", "gre1", "grower_group * return_on_equity.as_normalized_by_country", k_computeds, 2, k_double, "gre1", Void, k_company, "rose_entity_date", "", "", 1911)
		end

	mtgp: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_computed ("mature_group", "mtgp", "mature_stage_observation + coper_stage_observation", k_computeds, 2, k_double, "mtgp", Void, k_company, "rose_entity_date", "", "", 1910)
		end

	grgp: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_computed ("grower_group", "grgp", "growth_stage_observation + emerging_growth_stage_observation", k_computeds, 2, k_double, "grgp", Void, k_company, "rose_entity_date", "", "", 1909)
		end

	fpay: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("forecast_dividend_payout_ratio", "fpay", "F_DIVPAY", k_risk, null_integer, null_integer, 2, "", -4, 4, 0.05, 0.1, "", Void, Void, '0', k_monthly_data, "", k_risk, 1908)
		end

	cpli: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("capital_inv_risk", "cpli", Void, k_risk, null_integer, null_integer, 2, Void, -3, 3, missing_double, missing_double, "", Void, Void, 'F', k_monthly_data, "", "", 1907)
		end

	f_egro: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("fwd_eps_growth", "f_egro", Void, k_risk, null_integer, null_integer, 2, Void, -3, 3, missing_double, missing_double, "", Void, Void, 'F', k_monthly_data, "", "", 1906)
		end

	f_sgro: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("fwd_sales_growth", "f_sgro", Void, k_risk, null_integer, null_integer, 2, Void, -3, 3, missing_double, missing_double, "", Void, Void, 'F', k_monthly_data, "", "", 1905)
		end

	adxr: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("adv_exp_hist_risk", "adxr", Void, k_risk, null_integer, null_integer, 2, Void, -3, 3, missing_double, missing_double, "", Void, Void, 'F', k_monthly_data, "", "", 1903)
		end

	vsls: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("variable_sales", "vsls", Void, k_risk, null_integer, null_integer, 2, Void, -3, 3, missing_double, missing_double, "", Void, Void, 'F', k_monthly_data, "", "", 1902)
		end

	frtc: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("fwd_12m_eps_certainty", "frtc", Void, k_risk, null_integer, null_integer, 2, Void, -3, 3, missing_double, missing_double, "", Void, Void, 'F', k_monthly_data, "", "", 1901)
		end

	fgro: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("fundamental_growth", "fgro", Void, k_risk, null_integer, null_integer, 2, Void, -3, 3, missing_double, missing_double, "", Void, Void, 'F', k_monthly_data, "", "", 1888)
		end

	roex: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("roe_ex_ex", "roex", Void, k_risk, null_integer, null_integer, 2, Void, -3, 3, missing_double, missing_double, "", Void, Void, 'F', k_monthly_data, "", "", 1887)
		end

	sg_t: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("sales_growth_tstat", "sg_t", Void, k_risk, null_integer, null_integer, 2, Void, -3, 3, missing_double, missing_double, "", Void, Void, 'F', k_monthly_data, "", "", 1886)
		end

	mrgn: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("operating_margin", "mrgn", Void, k_risk, null_integer, null_integer, 2, Void, -3, 3, missing_double, missing_double, "", Void, Void, 'F', k_monthly_data, "", "", 1885)
		end

	gvla: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("gross_value_added", "gvla", Void, null_integer, null_integer, null_integer, 1, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "1", "", 1884)
		end

	xstf: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("staff_expense", "xstf", Void, null_integer, null_integer, null_integer, 1, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "1", "", 1883)
		end

	fsgro: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondex ("forward_sales_growth_rate", "fsgro", k_mondex, "", "", null_integer, null_integer, null_integer, null_integer, 2, "", k_double, Void, "", k_company, '0', k_mondex, k_ibes, 1882)
		end

	fegro: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondex ("forward_eps_growth", "fegro", k_mondex, Void, Void, null_integer, null_integer, null_integer, null_integer, 2, Void, k_double, Void, Void, k_company, '0', k_mondex, k_ibes, 1881)
		end

	bass: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("business_assets", "bass", Void, k_risk, null_integer, null_integer, 2, Void, -3, 3, missing_double, missing_double, "", Void, Void, '0', k_monthly_data, "", "", 1876)
		end

	ltcp: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("long_term_capital", "ltcp", Void, k_risk, null_integer, null_integer, 2, Void, -3, 3, missing_double, missing_double, "", Void, Void, '0', k_monthly_data, "", "", 1875)
		end

	oprr: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("oper_ln", "oprr", Void, k_risk, null_integer, null_integer, 2, Void, -3, 3, missing_double, missing_double, "", Void, Void, '0', k_monthly_data, "", "", 1874)
		end

	slsr: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("sales_ln", "slsr", Void, k_risk, null_integer, null_integer, 2, Void, -3, 3, missing_double, missing_double, "", Void, Void, '0', k_monthly_data, "", "", 1872)
		end

	empl: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("employees_ln", "empl", Void, k_risk, null_integer, null_integer, 2, Void, -3, 3, missing_double, missing_double, "", Void, Void, '0', k_monthly_data, "", "", 1871)
		end

	rs12_mtd: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("relative_strength_1_year_mtd_smoothed", "rs12_mtd", "RS12_MTD", k_risk, null_integer, null_integer, 2, "", -3, 3, missing_double, missing_double, "", Void, Void, '0', k_monthly_data, "", k_risk, 1729)
		end

	sizf: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("fundamental_size", "sizf", "test", k_risk, null_integer, null_integer, 2, "", -3, 3, missing_double, missing_double, "", Void, "", '0', k_monthly_data, "", "", 1726)
		end

	z_jm_av_lfis: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("z_last_fiscal_report_availability_jmon", "z_jm_av_lfis", Void, Void, null_integer, null_integer, null_integer, null_integer, 2, Void, k_integer, Void, Void, k_company, ' ', k_monthly_data, "", "", 1725)
		end

	z_source_lfis: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("z_last_fiscal_report_source", "z_source_lfis", Void, Void, null_integer, null_integer, null_integer, null_integer, 2, Void, k_integer, Void, Void, k_company, ' ', k_monthly_data, "", "", 1724)
		end

	mssn: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("ms1y_sim_nor", "mssn", "MS1Y.SMN", k_risk, null_integer, null_integer, 2, "", -3, 3, missing_double, missing_double, "SIMNORM:MS1Y", Void, Void, ' ', k_monthly_data, "", k_risk, 1723)
		end

	bpsn: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("b_p_sim_nor", "bpsn", "b_p.SMN", k_risk, null_integer, null_integer, 2, "", -3, 3, missing_double, missing_double, "SIMNORM:B_P", Void, Void, ' ', k_monthly_data, "", k_risk, 1722)
		end

	rtrn4: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondex ("return_relative_days_16_21", "rtrn4", k_mondex, Void, Void, null_integer, null_integer, null_integer, null_integer, 3, k_currency, k_double, Void, Void, k_company, '0', k_mondex, "", 1720)
		end

	nfiscmonsi: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("number_of_fiscal_months_interim", "nfiscmonsi", Void, null_integer, null_integer, null_integer, 1, Void, k_integer, Void, k_company, '0', k_valdata_signature, "", "", 1719)
		end

	cfcn: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("current_alpha_country_normalized", "cfcn", "CFCN", k_risk, null_integer, null_integer, 2, "", -5, 5, missing_double, missing_double, "", Void, Void, '0', k_monthly_data, "", k_risk, 1718)
		end

	kalf: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("kurrent_alpha", "kalf", "Kuralpha", k_risk, null_integer, null_integer, 2, "", -5, 5, 0, 0.3, "", Void, Void, ' ', k_monthly_data, "", k_risk, 1717)
		end

	peyp: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondex ("projected_earnings_yield", "peyp", k_mondex, Void, Void, null_integer, null_integer, null_integer, null_integer, 3, Void, k_double, Void, "1", k_company, ' ', k_mondex, k_risk, 1716)
		end

	pedp: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondex ("projected_earnings_delta_to_price", "pedp", k_mondex, Void, Void, null_integer, null_integer, null_integer, null_integer, 3, Void, k_double, Void, "1", k_company, ' ', k_mondex, k_risk, 1715)
		end

	dehp_bervqx: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondex ("projected_delta_earnings_to_price_bervqx_mondex", "dehp_bervqx", k_mondex, Void, Void, null_integer, null_integer, null_integer, null_integer, 3, Void, k_double, Void, "1", k_company, ' ', k_mondex, k_risk, 1714)
		end

	obta: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondex ("oil_beta", "obta", k_mondex, Void, Void, null_integer, null_integer, null_integer, null_integer, 3, Void, k_double, Void, "1", k_company, ' ', k_mondex, k_risk, 1713)
		end

	ratg: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondex ("default_risk_index", "ratg", k_mondex, Void, Void, null_integer, null_integer, null_integer, null_integer, 3, Void, k_double, Void, "1", k_company, ' ', k_mondex, k_risk, 1712)
		end

	cfsn: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("calf_sim_nor", "cfsn", "CALF.SMN", k_risk, null_integer, null_integer, 2, "", -3, 3, missing_double, missing_double, "SIMNORM:CALF", Void, Void, ' ', k_monthly_data, "", k_risk, 1711)
		end

	z_jm_fiscal: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("z_latest_valid_fiscal_report_jmon", "z_jm_fiscal", Void, Void, null_integer, null_integer, null_integer, null_integer, 2, Void, k_integer, Void, Void, k_company, 'F', k_monthly_data, "", "", 1702)
		end

	nrco: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("exceptional_items_other", "nrco", "nrec", null_integer, null_integer, null_integer, 1, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "1", k_profit_and_loss, 1701)
		end

	pfdg: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("preferred_government", "pfdg", "pfdc", null_integer, null_integer, null_integer, 1, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", k_liabilities, 1697)
		end

	adxh: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("advertising_expense_history", "adxh", Void, null_integer, null_integer, null_integer, 1, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", "", 1696)
		end

	adex: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("advertising_expense", "adex", Void, null_integer, null_integer, null_integer, 1, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "1", "", 1695)
		end

	cgwi: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("cumulative_goodwill_impairment", "cgwi", Void, null_integer, null_integer, null_integer, 1, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", "", 1694)
		end

	npat: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("non_performing_assets", "npat", Void, null_integer, null_integer, null_integer, 1, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", "", 1693)
		end

	s_id: VARIABLE_DESCRIPTION_RECORD is
			-- Security handle
		once
			create Result.make_model_security ("security_entity_id", "s_id", 2, "", k_integer, ' ', "MS_48", 1692)
		end

	c_id: VARIABLE_DESCRIPTION_RECORD is
			--company_entity_id
		once
			create Result.make_model_company ("company_entity_id", "c_id", 2, "", k_integer, ' ', "306", 1691)
		end

	cepc: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondex ("trailing_eps_change_proxcap_nor", "cepc", k_mondex, Void, Void, null_integer, null_integer, null_integer, null_integer, 3, Void, k_double, Void, Void, k_company, ' ', k_mondex, k_risk, 1690)
		end

	mqae: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("market_quality_by_pcte", "mqae", Void, Void, null_integer, null_integer, null_integer, null_integer, 3, Void, k_double, Void, k_1000000, k_company, ' ', k_monthly_data, "", k_risk, 1689)
		end

	mqac: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("market_quality_by_cqal", "mqac", Void, Void, null_integer, null_integer, null_integer, null_integer, 3, Void, k_double, Void, k_1000000, k_company, ' ', k_monthly_data, "", k_risk, 1688)
		end

	mqar: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("market_quality_simple", "mqar", Void, Void, null_integer, null_integer, null_integer, null_integer, 3, Void, k_double, Void, k_1000000, k_company, ' ', k_mondex, "", k_risk, 1687)
		end

	mqas: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("market_quality_by_similars_group", "mqas", Void, Void, null_integer, null_integer, null_integer, null_integer, 3, Void, k_double, Void, k_1000000, k_company, 'F', k_monthly_data, "", k_risk, 1686)
		end

	sret: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("gain_on_sale_of_property", "sret", "noin", null_integer, null_integer, null_integer, 1, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "1", k_profit_and_loss, 1685)
		end

	dpre: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("depreciation_amortization_of_property", "dpre", "opad", null_integer, null_integer, null_integer, 1, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "1", k_profit_and_loss, 1684)
		end

	mrct: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("rental_commitments", "mrct", Void, null_integer, null_integer, null_integer, 1, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", "", 1683)
		end

	t_qlty_rssm: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondex ("t_qlty_rssm", "t_qlty_rssm", k_mondex, Void, Void, null_integer, null_integer, null_integer, null_integer, 3, Void, k_real, Void, Void, k_company, 'F', k_mondex, k_risk, 1682)
		end

	r_qlty_rssm: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondex ("r_qlty_rssm", "r_qlty_rssm", k_mondex, Void, Void, null_integer, null_integer, null_integer, null_integer, 3, Void, k_real, Void, Void, k_company, 'F', k_mondex, k_risk, 1681)
		end

	t_tval_pncv: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondex ("t_tval_pncv", "t_tval_pncv", k_mondex, Void, Void, null_integer, null_integer, null_integer, null_integer, 3, Void, k_real, Void, Void, k_company, 'F', k_mondex, k_risk, 1680)
		end

	r_tval_pncv: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondex ("r_tval_pncv", "r_tval_pncv", k_mondex, Void, Void, null_integer, null_integer, null_integer, null_integer, 3, Void, k_real, Void, Void, k_company, 'F', k_mondex, k_risk, 1679)
		end

	t_calf_rssm: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondex ("t_calf_rssm", "t_calf_rssm", k_mondex, Void, Void, null_integer, null_integer, null_integer, null_integer, 3, Void, k_real, Void, Void, k_company, 'F', k_mondex, k_risk, 1678)
		end

	r_calf_rssm: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondex ("r_calf_rssm", "r_calf_rssm", k_mondex, Void, Void, null_integer, null_integer, null_integer, null_integer, 3, Void, k_real, Void, Void, k_company, 'F', k_mondex, k_risk, 1677)
		end

	t_tval_mqal: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondex ("t_tval_mqal", "t_tval_mqal", k_mondex, Void, Void, null_integer, null_integer, null_integer, null_integer, 3, Void, k_real, Void, Void, k_company, 'F', k_mondex, k_risk, 1676)
		end

	r_tval_mqal: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondex ("r_tval_mqal", "r_tval_mqal", k_mondex, Void, Void, null_integer, null_integer, null_integer, null_integer, 3, Void, k_real, Void, Void, k_company, 'F', k_mondex, k_risk, 1675)
		end

	t_tval_trad: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondex ("t_tval_trad", "t_tval_trad", k_mondex, Void, Void, null_integer, null_integer, null_integer, null_integer, 3, Void, k_real, Void, Void, k_company, 'F', k_mondex, k_risk, 1674)
		end

	r_tval_trad: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondex ("r_tval_trad", "r_tval_trad", k_mondex, Void, Void, null_integer, null_integer, null_integer, null_integer, 3, Void, k_real, Void, Void, k_company, 'F', k_mondex, k_risk, 1673)
		end

	t_calf_qlty: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondex ("t_calf_qlty", "t_calf_qlty", k_mondex, Void, Void, null_integer, null_integer, null_integer, null_integer, 3, Void, k_real, Void, Void, k_company, 'F', k_mondex, k_risk, 1672)
		end

	r_calf_qlty: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondex ("r_calf_qlty", "r_calf_qlty", k_mondex, Void, Void, null_integer, null_integer, null_integer, null_integer, 3, Void, k_real, Void, Void, k_company, 'F', k_mondex, k_risk, 1671)
		end

	cfli: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("investing_cash_flow", "cfli", Void, null_integer, null_integer, null_integer, 1, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "1", "", 1670)
		end

	cflf: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("financing_cash_flow", "cflf", Void, null_integer, null_integer, null_integer, 1, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "1", "", 1669)
		end

	cflp: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("operating_cash_flow", "cflp", Void, null_integer, null_integer, null_integer, 1, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "1", "", 1666)
		end

	s_frat: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondex ("forward_12m_sales_certainty", "s_frat", k_mondex, Void, Void, null_integer, null_integer, null_integer, null_integer, 2, Void, k_double, Void, Void, k_company, 'S', k_mondex, k_ibes, 1664)
		end

	s_fy0x: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondex ("trailing_12m_sales", "s_fy0x", k_mondex, Void, Void, null_integer, null_integer, null_integer, null_integer, 2, Void, k_double, Void, Void, k_company, 'S', k_mondex, k_ibes, 1663)
		end

	s_fy12m: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondex ("forward_12m_sales", "s_fy12m", k_mondex, Void, Void, null_integer, null_integer, null_integer, null_integer, 2, Void, k_double, Void, Void, k_company, 'S', k_mondex, k_ibes, 1662)
		end

	s_fysd: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondex ("std_of_sales_estimates", "s_fysd", k_mondex, Void, Void, null_integer, null_integer, null_integer, null_integer, 2, Void, k_double, Void, Void, k_company, 'S', k_mondex, k_ibes, 1661)
		end

	s_nwan: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondex ("weighted_num_of_analysts_for_sales", "s_nwan", k_mondex, Void, Void, null_integer, null_integer, null_integer, null_integer, 2, Void, k_double, Void, Void, k_company, 'S', k_mondex, k_ibes, 1660)
		end

	gwaj: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("goodwill_adjustment", "gwaj", Void, null_integer, null_integer, null_integer, 1, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", "", 1657)
		end

	gwim: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("goodwill_impairments", "gwim", Void, null_integer, null_integer, null_integer, 1, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "1", "", 1656)
		end

	nwan: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondex ("weighted_num_of_analysts", "nwan", k_mondex, Void, Void, null_integer, null_integer, null_integer, null_integer, 2, Void, k_double, Void, Void, k_company, ' ', k_mondex, k_ibes, 1643)
		end

	fysd: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondex ("std_of_eps_estimates", "fysd", k_mondex, Void, Void, null_integer, null_integer, null_integer, null_integer, 2, Void, k_double, Void, Void, k_company, ' ', k_mondex, k_ibes, 1642)
		end

	fy12m: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondex ("forward_12m_eps", "fy12m", k_mondex, Void, Void, null_integer, null_integer, null_integer, null_integer, 2, Void, k_double, Void, Void, k_company, ' ', k_mondex, k_ibes, 1641)
		end

	fy0x: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondex ("trailing_12m_eps", "fy0x", k_mondex, Void, Void, null_integer, null_integer, null_integer, null_integer, 2, Void, k_double, Void, Void, k_company, ' ', k_mondex, k_ibes, 1640)
		end

	frat: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondex ("forward_12m_eps_certainty", "frat", k_mondex, Void, Void, null_integer, null_integer, null_integer, null_integer, 2, Void, k_double, Void, Void, k_company, ' ', k_mondex, k_ibes, 1639)
		end

	falpha0_td: VARIABLE_DESCRIPTION_RECORD is
			--FALPHA0_td
		once
			create Result.make_model_company ("falpha0_td", "falpha0_td", 0, "", k_double, ' ', "304", 1626)
		end

	falphaval_td: VARIABLE_DESCRIPTION_RECORD is
			--FALPHAVAL_td
		once
			create Result.make_model_company ("falphaval_td", "falphaval_td", 0, "", k_double, ' ', "303", 1625)
		end

	gralpha2000_td: VARIABLE_DESCRIPTION_RECORD is
			--gralpha2000_td
		once
			create Result.make_model_company ("gralpha2000_td", "gralpha2000_td", 0, "", k_double, ' ', "305", 1624)
		end

	falpha98l_td: VARIABLE_DESCRIPTION_RECORD is
			--FALPHA98L_td
		once
			create Result.make_model_company ("falpha98l_td", "falpha98l_td", 0, "", k_double, ' ', "2", 1623)
		end

	falpha98_td: VARIABLE_DESCRIPTION_RECORD is
			--FALPHA98_td
		once
			create Result.make_model_company ("falpha98_td", "falpha98_td", 0, "", k_double, ' ', "301", 1622)
		end

	expt: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondex ("export_ratio", "expt", k_mondex, Void, Void, null_integer, null_integer, null_integer, null_integer, 3, Void, k_double, Void, Void, k_company, ' ', k_mondex, k_risk, 1621)
		end

	gralpha2000_dsa_inn: VARIABLE_DESCRIPTION_RECORD is
			--gralpha2000_dsa_inn
		once
			create Result.make_model_company ("gralpha2000_dsa_inn", "gralpha2000_dsa_inn", 2, "", k_double, ' ', "219", 1620)
		end

	gralpha2000_dsa_ind: VARIABLE_DESCRIPTION_RECORD is
			--gralpha2000_dsa_ind
		once
			create Result.make_model_company ("gralpha2000_dsa_ind", "gralpha2000_dsa_ind", 2, "", k_double, ' ', "218", 1619)
		end

	gralpha2000_val_inn: VARIABLE_DESCRIPTION_RECORD is
			--gralpha2000_val_inn
		once
			create Result.make_model_company ("gralpha2000_val_inn", "gralpha2000_val_inn", 2, "", k_double, ' ', "217", 1618)
		end

	gralpha2000_val_ind: VARIABLE_DESCRIPTION_RECORD is
			--gralpha2000_val_ind
		once
			create Result.make_model_company ("gralpha2000_val_ind", "gralpha2000_val_ind", 2, "", k_double, ' ', "216", 1617)
		end

	gralpha2000_dsa: VARIABLE_DESCRIPTION_RECORD is
			--gralpha2000_dsa
		once
			create Result.make_model_company ("gralpha2000_dsa", "gralpha2000_dsa", 2, "", k_double, ' ', "211", 1616)
		end

	gralpha2000_val: VARIABLE_DESCRIPTION_RECORD is
			--gralpha2000_val
		once
			create Result.make_model_company ("gralpha2000_val", "gralpha2000_val", 2, "", k_double, ' ', "210", 1615)
		end

	falpha0_dsa_inn: VARIABLE_DESCRIPTION_RECORD is
			--falpha0_dsa_inn
		once
			create Result.make_model_company ("falpha0_dsa_inn", "falpha0_dsa_inn", 2, "", k_double, ' ', "209", 1614)
		end

	falpha0_dsa_ind: VARIABLE_DESCRIPTION_RECORD is
			--falpha0_dsa_ind
		once
			create Result.make_model_company ("falpha0_dsa_ind", "falpha0_dsa_ind", 2, "", k_double, ' ', "208", 1613)
		end

	falpha0_val_inn: VARIABLE_DESCRIPTION_RECORD is
			--falpha0_val_inn
		once
			create Result.make_model_company ("falpha0_val_inn", "falpha0_val_inn", 2, "", k_double, ' ', "207", 1612)
		end

	falpha0_val_ind: VARIABLE_DESCRIPTION_RECORD is
			--falpha0_val_ind
		once
			create Result.make_model_company ("falpha0_val_ind", "falpha0_val_ind", 2, "", k_double, ' ', "206", 1611)
		end

	falpha0_dsa: VARIABLE_DESCRIPTION_RECORD is
			--falpha0_dsa
		once
			create Result.make_model_company ("falpha0_dsa", "falpha0_dsa", 2, "", k_double, ' ', "201", 1610)
		end

	falpha0_val: VARIABLE_DESCRIPTION_RECORD is
			--falpha0_val
		once
			create Result.make_model_company ("falpha0_val", "falpha0_val", 2, "", k_double, ' ', "200", 1609)
		end

	falphaval_dsa_inn: VARIABLE_DESCRIPTION_RECORD is
			--falphaval_dsa_inn
		once
			create Result.make_model_company ("falphaval_dsa_inn", "falphaval_dsa_inn", 2, "", k_double, ' ', "99", 1608)
		end

	falphaval_dsa_ind: VARIABLE_DESCRIPTION_RECORD is
			--falphaval_dsa_ind
		once
			create Result.make_model_company ("falphaval_dsa_ind", "falphaval_dsa_ind", 2, "", k_double, ' ', "98", 1607)
		end

	falphaval_val_inn: VARIABLE_DESCRIPTION_RECORD is
			--falphaval_val_inn
		once
			create Result.make_model_company ("falphaval_val_inn", "falphaval_val_inn", 2, "", k_double, ' ', "97", 1606)
		end

	falphaval_val_ind: VARIABLE_DESCRIPTION_RECORD is
			--falphaval_val_ind
		once
			create Result.make_model_company ("falphaval_val_ind", "falphaval_val_ind", 2, "", k_double, ' ', "6", 1605)
		end

	falphaval_dsa: VARIABLE_DESCRIPTION_RECORD is
			--falphaval_dsa
		once
			create Result.make_model_company ("falphaval_dsa", "falphaval_dsa", 0, "", k_double, ' ', "91", 1604)
		end

	falphaval_val: VARIABLE_DESCRIPTION_RECORD is
			--falphaval_val
		once
			create Result.make_model_company ("falphaval_val", "falphaval_val", 2, "", k_double, ' ', "90", 1603)
		end

	falpha98l_dsa_inn: VARIABLE_DESCRIPTION_RECORD is
			--falpha98l_dsa_inn
		once
			create Result.make_model_company ("falpha98l_dsa_inn", "falpha98l_dsa_inn", 2, "", k_double, ' ', "89", 1602)
		end

	falpha98l_dsa_ind: VARIABLE_DESCRIPTION_RECORD is
			--falpha98l_dsa_ind
		once
			create Result.make_model_company ("falpha98l_dsa_ind", "falpha98l_dsa_ind", 2, "", k_double, ' ', "88", 1601)
		end

	falpha98l_val_inn: VARIABLE_DESCRIPTION_RECORD is
			--falpha98l_val_inn
		once
			create Result.make_model_company ("falpha98l_val_inn", "falpha98l_val_inn", 2, "", k_double, ' ', "87", 1600)
		end

	falpha98l_val_ind: VARIABLE_DESCRIPTION_RECORD is
			--falpha98l_val_ind
		once
			create Result.make_model_company ("falpha98l_val_ind", "falpha98l_val_ind", 2, "", k_double, ' ', "86", 1599)
		end

	falpha98l_dsa: VARIABLE_DESCRIPTION_RECORD is
			--falpha98l_dsa
		once
			create Result.make_model_company ("falpha98l_dsa", "falpha98l_dsa", 2, "", k_double, ' ', "81", 1598)
		end

	falpha98l_val: VARIABLE_DESCRIPTION_RECORD is
			--falpha98l_val
		once
			create Result.make_model_company ("falpha98l_val", "falpha98l_val", 2, "", k_double, ' ', "80", 1597)
		end

	falpha98_dsa_inn: VARIABLE_DESCRIPTION_RECORD is
			--falpha98_dsa_inn
		once
			create Result.make_model_company ("falpha98_dsa_inn", "falpha98_dsa_inn", 2, "", k_double, ' ', "79", 1596)
		end

	falpha98_dsa_ind: VARIABLE_DESCRIPTION_RECORD is
			--falpha98_dsa_ind
		once
			create Result.make_model_company ("falpha98_dsa_ind", "falpha98_dsa_ind", 2, "", k_double, ' ', "78", 1595)
		end

	falpha98_val_inn: VARIABLE_DESCRIPTION_RECORD is
			--falpha98_val_inn
		once
			create Result.make_model_company ("falpha98_val_inn", "falpha98_val_inn", 2, "", k_double, ' ', "77", 1594)
		end

	falpha98_val_ind: VARIABLE_DESCRIPTION_RECORD is
			--falpha98_val_ind
		once
			create Result.make_model_company ("falpha98_val_ind", "falpha98_val_ind", 2, "", k_double, ' ', "76", 1593)
		end

	falpha98_dsa_sino: VARIABLE_DESCRIPTION_RECORD is
			--falpha98_dsa_sino
		once
			create Result.make_model_company ("falpha98_dsa_sino_raw", "falpha98_dsa_sino_raw", 2, "", k_double, ' ', "75", 1592)
		end

	falpha98_dsa_unno: VARIABLE_DESCRIPTION_RECORD is
			--falpha98_dsa_unno
		once
			create Result.make_model_company ("falpha98_dsa_unno_raw", "falpha98_dsa_unno_raw", 2, "", k_double, ' ', "74", 1591)
		end

	falpha98_val_unno: VARIABLE_DESCRIPTION_RECORD is
			--falpha98_val_unno
		once
			create Result.make_model_company ("falpha98_val_unno_raw", "falpha98_val_unno_raw", 2, "", k_double, ' ', "72", 1590)
		end

	falpha98_val_sino: VARIABLE_DESCRIPTION_RECORD is
			--falpha98_val_sino
		once
			create Result.make_model_company ("falpha98_val_sino_raw", "falpha98_val_sino_raw", 2, "", k_double, ' ', "73", 1589)
		end

	falpha98_dsa: VARIABLE_DESCRIPTION_RECORD is
			--falpha98_dsa
		once
			create Result.make_model_company ("falpha98_dsa", "falpha98_dsa", 2, "", k_double, ' ', "71", 1588)
		end

	falpha98_val: VARIABLE_DESCRIPTION_RECORD is
			--falpha98_val
		once
			create Result.make_model_company ("falpha98_val", "falpha98_val", 2, "", k_double, ' ', "70", 1587)
		end

	nvolp12m: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondex ("number_of_prices_with_vol_12m", "nvolp12m", k_mondex, Void, Void, null_integer, null_integer, null_integer, null_integer, 3, k_currency, k_double, Void, Void, k_company, ' ', k_mondex, "", 1586)
		end

	roll_cvs: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondex ("Roll_autocv_spread_measure", "roll_cvs", k_mondex, Void, Void, null_integer, null_integer, null_integer, null_integer, 3, k_currency, k_double, Void, Void, k_company, ' ', k_mondex, "", 1585)
		end

	ami_ill: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondex ("Amihud_illiquidity_measure_mondex", "ami_ill", k_mondex, Void, Void, null_integer, null_integer, null_integer, null_integer, 3, k_currency, k_double, Void, Void, k_company, ' ', k_mondex, "", 1584)
		end

	hhiind: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondex ("industry_breakdown_concentration_measure_mondex", "hhiind", k_mondex, Void, Void, null_integer, null_integer, null_integer, null_integer, 3, k_currency, k_double, Void, Void, k_company, ' ', k_mondex, "", 1583)
		end

	vseg_src: VARIABLE_DESCRIPTION_RECORD is
			--valuation_segment_source
		once
			create Result.make_model_company ("valuation_segment_source", "vseg_src", 2, "", k_string, ' ', Void, 1582)
		end

	vfis_src: VARIABLE_DESCRIPTION_RECORD is
			--valuation_fiscal_source
		once
			create Result.make_model_company ("valuation_fiscal_source", "vfis_src", 2, "", k_string, ' ', Void, 1581)
		end

	status: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("status", "status", Void, null_integer, null_integer, null_integer, 1, Void, k_string, Void, k_company, ' ', k_valdata_signature, "", "", 1580)
		end

	prelim_flag: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("preliminary_flag", "prelim_flag", Void, null_integer, null_integer, null_integer, 1, Void, k_string, Void, k_company, ' ', k_valdata_signature, "", "", 1579)
		end

	is_obs: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("is_obsolete", "is_obs", Void, null_integer, null_integer, null_integer, 1, Void, k_string, Void, k_company, ' ', k_valdata_signature, "", "", 1578)
		end

	country_iso: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("country_iso", "country_iso", Void, null_integer, null_integer, null_integer, 1, Void, k_string, Void, k_company, ' ', k_valdata_signature, "", "", 1577)
		end

	incstdefflag: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("inc_state_deficiency_flag", "incstdefflag", Void, null_integer, null_integer, null_integer, 1, Void, k_string, Void, k_company, ' ', k_valdata_signature, "", "", 1576)
		end

	balshtdefflag: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("bal_sheet_deficiency_flag", "balshtdefflag", Void, null_integer, null_integer, null_integer, 1, Void, k_string, Void, k_company, ' ', k_valdata_signature, "", "", 1575)
		end

	valdata_screen: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("valdata_screen", "valdata_screen", Void, null_integer, null_integer, null_integer, 1, Void, k_string, Void, k_company, ' ', k_valdata_signature, "", "", 1574)
		end

	valflag: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("valflag", "valflag", Void, null_integer, null_integer, null_integer, 1, Void, k_string, Void, k_company, ' ', k_valdata_signature, "", "", 1573)
		end

	nfiscmons: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("number_of_fiscal_months", "nfiscmons", Void, null_integer, null_integer, null_integer, 1, Void, k_string, Void, k_company, ' ', k_valdata_signature, "", "", 1572)
		end

	supplier: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("supplier", "supplier", Void, null_integer, null_integer, null_integer, 1, Void, k_string, Void, k_company, ' ', k_valdata_signature, "", "", 1571)
		end

	source: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("source", "source", Void, null_integer, null_integer, null_integer, 1, Void, k_string, Void, k_company, ' ', k_valdata_signature, "", "", 1569)
		end

	periodicity_id: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("periodicity_id", "periodicity_id", Void, null_integer, null_integer, null_integer, 1, Void, k_string, Void, k_company, ' ', k_valdata_signature, "", "", 1567)
		end

	nvolp3m: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondex ("number_of_prices_with_vol_3m", "nvolp3m", k_mondex, Void, Void, null_integer, null_integer, null_integer, null_integer, 3, k_currency, k_double, Void, Void, k_company, ' ', k_mondex, "", 1564)
		end

	nvolp1m: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondex ("number_of_prices_with_volume", "nvolp1m", k_mondex, Void, Void, null_integer, null_integer, null_integer, null_integer, 3, k_currency, k_double, Void, Void, k_company, ' ', k_mondex, "", 1563)
		end

	z_ixprimary_zonal_ex_break: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("z_zonal_ixprimary_ex_break", "z_ixprimary_zonal_ex_break", Void, Void, null_integer, null_integer, null_integer, null_integer, 2, Void, k_integer, Void, Void, k_company, ' ', k_monthly_data, "", "", 1557)
		end

	ibne: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("ibes_n_estimates", "ibne", Void, Void, null_integer, null_integer, null_integer, null_integer, 2, Void, k_double, Void, Void, k_company, ' ', k_monthly_data, "", k_mondat, 1556)
		end

	erv: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("ibes_earnings_revision", "erv", "ERV", k_risk, null_integer, 0, 2, Void, -3, 3, 1.0, 0.6, "", Void, Void, ' ', k_monthly_data, "", k_risk, 1555)
		end

	brimcode: VARIABLE_DESCRIPTION_RECORD is
			--brimcode
		once
			create Result.make_model_company ("brimcode", "brimcode", 0, "", k_string, ' ', "141", 1554)
		end

	brimsec: VARIABLE_DESCRIPTION_RECORD is
			-- BRIMSEC
		once
			create Result.make_model_security ("brimsec", "BRIMSEC", 1, "", k_string, ' ', "MS_47", 1553)
				--will be changed to 2314
			Result.set_source ("ref")
				-- Changed from "model".
		end

	z_primary_yield: VARIABLE_DESCRIPTION_RECORD is
			-- z_primary_yield
		once
			create Result.make_mondat ("z_primary_yield", "z_primary_yield", Void, Void, null_integer, null_integer, null_integer, null_integer, 2, Void, k_double, Void, Void, k_company, ' ', k_monthly_data, "", "", 1553)
		end

	rs12x1: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondex ("relative_strength_12_month_ex_1", "rs12x1", k_mondex, Void, Void, null_integer, null_integer, null_integer, null_integer, 3, Void, k_double, Void, Void, k_company, ' ', k_mondex, k_risk, 1552)
		end

	rm_pbeta: VARIABLE_DESCRIPTION_RECORD is
			-- Risk model predicted beta
		once
			create Result.make_mondex ("risk_model_predicted_beta", "rm_pbeta", k_mondex, Void, Void, null_integer, null_integer, null_integer, null_integer, 3, Void, k_double, Void, Void, k_company, 'S', k_mondex, "", 1550)
		end

	iprm: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("invest_property_at_market", "iprm", k_asst, null_integer, null_integer, null_integer, 1, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", k_assets, 1549)
		end

	prrv: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("property_revaluation_gain", "prrv", "noin", null_integer, null_integer, null_integer, 1, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "1", k_profit_and_loss, 1548)
		end

	ubet: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("updside_beta", "ubet", "UBETA", k_risk, null_integer, 0, 2, Void, -3, 5, 1, 0.6, "", Void, Void, ' ', k_monthly_data, "", k_risk, 1547)
		end

	dbet: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("downside_beta", "dbet", "DBETA", k_risk, null_integer, 0, 2, Void, -3, 5, 1, 0.6, "", Void, Void, ' ', k_monthly_data, "", k_risk, 1546)
		end

	etax: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("excise_tax", "etax", Void, null_integer, null_integer, null_integer, 1, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "1", "", 1545)
		end

	mcor: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("monetary_correction", "mcor", Void, null_integer, null_integer, null_integer, 1, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "1", "", 1544)
		end

	mcap: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("monetary_capital", "mcap", Void, null_integer, null_integer, null_integer, 1, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", "", 1543)
		end

	rt_ask_siz: VARIABLE_DESCRIPTION_RECORD is
			--rt_ask_size
		once
			create Result.make_model_company ("rt_ask_size", "rt_ask_siz", 0, "currency/share", k_double, 'F', Void, 1541)
		end

	rt_bid_siz: VARIABLE_DESCRIPTION_RECORD is
			--rt_bid_size
		once
			create Result.make_model_company ("rt_bid_size", "rt_bid_siz", 0, "currency/share", k_double, 'F', Void, 1540)
		end

	rt_ask: VARIABLE_DESCRIPTION_RECORD is
			--rt_ask
		once
			create Result.make_model_company ("rt_ask", "rt_ask", 0, "currency/share", k_double, 'F', Void, 1539)
		end

	rt_bid: VARIABLE_DESCRIPTION_RECORD is
			--rt_bid
		once
			create Result.make_model_company ("rt_bid", "rt_bid", 0, "currency/share", k_double, 'F', Void, 1538)
		end

	rt_vol: VARIABLE_DESCRIPTION_RECORD is
			--rt_volume
		once
			create Result.make_model_company ("rt_volume", "rt_vol", 0, "", k_double, 'F', Void, 1537)
		end

	rt_fitted_value: VARIABLE_DESCRIPTION_RECORD is
			--RT_FV
		once
			create Result.make_model_company ("rt_fitted_value", "rt_fv", 0, "currency/share", k_double, 'F', Void, 1535)
		end

	extrap_price: VARIABLE_DESCRIPTION_RECORD is
			--extrapolated_price
		once
			create Result.make_model_company ("extrapolated_price", "extrap_price", 0, "", k_double, ' ', Void, 1534)
		end

	z_ixprimary_zonal: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("z_zonal_ixprimary", "z_ixprimary_zonal", Void, Void, null_integer, null_integer, null_integer, null_integer, 2, Void, k_integer, Void, Void, k_company, 'F', k_monthly_data, "", "", 1533)
		end

	isap_rating: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("sap_rating_index", "isap_rating", Void, Void, null_integer, null_integer, null_integer, null_integer, 2, Void, k_integer, Void, Void, k_company, 'F', k_monthly_data, "", "", 1532)
		end

	iexch: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("exchange_index", "iexch", Void, Void, null_integer, null_integer, null_integer, null_integer, 2, Void, k_integer, Void, Void, k_company, 'F', k_monthly_data, "", "", 1531)
		end

	adiv_dlatest: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("adiv_latest", "adiv_dlatest", Void, Void, null_integer, null_integer, null_integer, null_integer, 2, Void, k_double, Void, Void, k_company, 'F', k_monthly_data, "", "", 1530)
		end

	icur_header_f: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("fund_currency_index", "icur_header_f", Void, Void, null_integer, null_integer, null_integer, null_integer, 2, Void, k_integer, Void, Void, k_company, 'F', k_monthly_data, "", "", 1529)
		end

	z_if_endquart: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("z_if_endquart", "z_if_endquart", Void, Void, null_integer, null_integer, null_integer, null_integer, 2, Void, k_integer, Void, Void, k_company, 'F', k_monthly_data, "", "", 1528)
		end

	z_if_endsem: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("z_if_endsem", "z_if_endsem", Void, Void, null_integer, null_integer, null_integer, null_integer, 2, Void, k_integer, Void, Void, k_company, 'F', k_monthly_data, "", "", 1527)
		end

	z_if_endyear: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("z_if_endyear", "z_if_endyear", Void, Void, null_integer, null_integer, null_integer, null_integer, 2, Void, k_integer, Void, Void, k_company, 'F', k_monthly_data, "", "", 1526)
		end

	z_if_new_fund: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("z_if_new_fundamental", "z_if_new_fund", Void, Void, null_integer, null_integer, null_integer, null_integer, 2, Void, k_integer, Void, Void, k_company, 'F', k_monthly_data, "", "", 1525)
		end

	z_shs_eqv_fccyymm: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("z_most_recent_available_fiscal_report_shares_equiv", "z_shs_eqv_fccyymm", Void, Void, null_integer, null_integer, null_integer, null_integer, 2, Void, k_double, Void, Void, k_company, 'F', k_monthly_data, "", "", 1524)
		end

	z_jm_shs_eqv: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("z_latest_available_fundamental_report_julianmonth", "z_jm_shs_eqv", Void, Void, null_integer, null_integer, null_integer, null_integer, 2, Void, k_integer, Void, Void, k_company, 'F', k_monthly_data, "", "", 1523)
		end

	z_if_j_parent: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("z_if_j_parent", "z_if_j_parent", Void, Void, null_integer, null_integer, null_integer, null_integer, 2, Void, k_integer, Void, Void, k_company, 'F', k_monthly_data, "", "", 1522)
		end

	z_basp: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("z_basis_cost", "z_basp", Void, Void, null_integer, null_integer, null_integer, null_integer, 2, Void, k_double, Void, Void, k_company, 'F', k_monthly_data, "", "", 1521)
		end

	z_volm: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("z_trailing_turnover_value", "z_volm", Void, Void, null_integer, null_integer, null_integer, null_integer, 2, Void, k_double, Void, Void, k_company, 'F', k_monthly_data, "", "", 1520)
		end

	z_stoa: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("z_trailing_turnover", "z_stoa", Void, Void, null_integer, null_integer, null_integer, null_integer, 2, Void, k_double, Void, Void, k_company, 'F', k_monthly_data, "", "", 1519)
		end

	z_ti18: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("z_ti18", "z_ti18", Void, Void, null_integer, null_integer, null_integer, null_integer, 2, Void, k_double, Void, Void, k_company, 'F', k_monthly_data, "", "", 1518)
		end

	z_tnin: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("z_turnover_innovation", "z_tnin", Void, Void, null_integer, null_integer, null_integer, null_integer, 2, Void, k_double, Void, Void, k_company, 'F', k_monthly_data, "", "", 1517)
		end

	z_that: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("z_predicted_turnover", "z_that", Void, Void, null_integer, null_integer, null_integer, null_integer, 2, Void, k_double, Void, Void, k_company, 'F', k_monthly_data, "", "", 1516)
		end

	z_turr: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("z_relative_turnover", "z_turr", Void, Void, null_integer, null_integer, null_integer, null_integer, 2, Void, k_double, Void, Void, k_company, 'F', k_monthly_data, "", "", 1515)
		end

	z_st60: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("z_st60", "z_st60", Void, Void, null_integer, null_integer, null_integer, null_integer, 2, Void, k_double, Void, Void, k_company, 'F', k_monthly_data, "", "", 1514)
		end

	z_st18: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("z_st18", "z_st18", Void, Void, null_integer, null_integer, null_integer, null_integer, 2, Void, k_double, Void, Void, k_company, 'F', k_monthly_data, "", "", 1513)
		end

	z_primary_membership_flags: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("z_primary_membership", "z_primary_membership_flags", Void, Void, null_integer, null_integer, null_integer, null_integer, 2, Void, k_integer, Void, Void, k_company, 'F', k_monthly_data, "", "", 1512)
		end

	z_primary_return: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("z_primary_return", "z_primary_return", Void, Void, null_integer, null_integer, null_integer, null_integer, 2, Void, k_double, Void, Void, k_company, 'F', k_monthly_data, "", "", 1511)
		end

	z_primary_adjusted_price: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("z_adjusted_price", "z_primary_adjusted_price", Void, Void, null_integer, null_integer, null_integer, null_integer, 2, Void, k_double, Void, Void, k_company, 'F', k_monthly_data, "", "", 1508)
		end

	z_primary_iadiv: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("z_indicated_annual_dividend", "z_primary_iadiv", Void, Void, null_integer, null_integer, null_integer, null_integer, 2, Void, k_double, Void, Void, k_company, 'F', k_monthly_data, "", "", 1507)
		end

	z_trdval: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("z_monthly_turnover_value", "z_trdval", Void, Void, null_integer, null_integer, null_integer, null_integer, 2, Void, k_double, Void, Void, k_company, 'F', k_monthly_data, "", "", 1506)
		end

	z_turn: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("z_monthly_turnover", "z_turn", Void, Void, null_integer, null_integer, null_integer, null_integer, 2, Void, k_double, Void, Void, k_company, 'F', k_monthly_data, "", "", 1505)
		end

	z_data_break: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("z_data_break", "z_data_break", Void, Void, null_integer, null_integer, null_integer, null_integer, 2, Void, k_integer, Void, Void, k_company, 'F', k_monthly_data, "", "", 1504)
		end

	z_membership_flags: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("z_index_membership", "z_membership_flags", Void, Void, null_integer, null_integer, null_integer, null_integer, 2, Void, k_integer, Void, Void, k_company, 'F', k_monthly_data, "", "", 1501)
		end

	z_currency_m: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("z_market_price_currency", "z_currency_m", "", "", null_integer, null_integer, null_integer, null_integer, 2, "", k_integer, Void, "", k_security, 'F', k_monthly_data, "", "", 1500)
		end

	z_exch: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("z_exchange", "z_exch", Void, Void, null_integer, null_integer, null_integer, null_integer, 2, Void, k_integer, Void, Void, k_company, 'F', k_monthly_data, "", "", 1499)
		end

	z_mend_lfis: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("z_last_previous_fiscal_julian", "z_mend_lfis", Void, Void, null_integer, null_integer, null_integer, null_integer, 2, Void, k_integer, Void, Void, k_company, 'F', k_monthly_data, "", "", 1498)
		end

	z_nprices: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("z_number_of_prices", "z_nprices", "", "", null_integer, null_integer, null_integer, null_integer, 2, "", k_integer, Void, "", k_security, 'F', k_monthly_data, "", "", 1497)
		end

	z_idate: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("z_final_price_date", "z_idate", "", "", null_integer, null_integer, null_integer, null_integer, 2, "", k_integer, Void, "", k_security, 'F', k_monthly_data, "", "", 1496)
		end

	z_propeq: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("z_proportion_of_equity", "z_propeq", "", "", null_integer, null_integer, null_integer, null_integer, 2, "", k_double, Void, "", k_security, 'F', k_monthly_data, "", "", 1495)
		end

	z_volume: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("z_monthly_volume", "z_volume", "", "", null_integer, null_integer, null_integer, null_integer, 2, "", k_double, Void, "", k_security, 'F', k_monthly_data, "", "", 1494)
		end

	z_yield: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("z_monthly_cash_yield", "z_yield", "", "", null_integer, null_integer, null_integer, null_integer, 2, "", k_double, Void, "", k_security, 'F', k_monthly_data, "", "", 1493)
		end

	z_return: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("z_monthly_return", "z_return", Void, Void, null_integer, null_integer, null_integer, null_integer, 2, Void, k_double, Void, Void, k_company, 'F', k_monthly_data, "", "", 1492)
		end

	z_capt: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("z_capitalization", "z_capt", Void, Void, null_integer, null_integer, null_integer, null_integer, 2, Void, k_double, Void, Void, k_company, 'F', k_monthly_data, "", "", 1491)
		end

	z_shs_listed: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("z_shs_listed", "z_shs_listed", "", "", null_integer, null_integer, null_integer, null_integer, 2, "", k_double, Void, "", k_security, 'F', k_monthly_data, "", "", 1490)
		end

	z_jm_av_shs_lfis: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("z_last_previous_fiscal_availability_julian", "z_jm_av_shs_lfis", Void, Void, null_integer, null_integer, null_integer, null_integer, 2, Void, k_integer, Void, Void, k_company, 'F', k_monthly_data, "", "", 1489)
		end

	z_shsout: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("z_shsout", "z_shsout", "", "", null_integer, null_integer, null_integer, null_integer, 2, "", k_double, Void, "", k_security, 'F', k_monthly_data, "", "", 1488)
		end

	z_price: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("z_monthly_price", "z_price", "", "", null_integer, null_integer, null_integer, null_integer, 2, k_currency_per_share, k_double, Void, "", k_security, 'F', k_monthly_data, "", "", 1487)
		end

	z_foradj: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("z_cumulative_foradj", "z_foradj", "", "", null_integer, null_integer, null_integer, null_integer, 2, "", k_double, Void, "", k_security, 'F', k_monthly_data, "", "", 1486)
		end

	z_primary_value: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("z_primary_valuation", "z_primary_value", Void, Void, null_integer, null_integer, null_integer, null_integer, 2, Void, k_double, Void, Void, k_company, 'F', k_monthly_data, "", "", 1485)
		end

	z_eps_jm_fiscal: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("z_fiscal_report_julian_month", "z_eps_jm_fiscal", Void, Void, null_integer, null_integer, null_integer, null_integer, 2, Void, k_integer, Void, Void, k_company, 'F', k_monthly_data, "", "", 1484)
		end

	z_axps: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("z_primary_security_annual_xx_eps", "z_axps", Void, Void, null_integer, null_integer, null_integer, null_integer, 2, Void, k_double, Void, Void, k_company, 'F', k_monthly_data, "", "", 1483)
		end

	z_abps: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("z_primary_security_annual_bps", "z_abps", Void, Void, null_integer, null_integer, null_integer, null_integer, 2, Void, k_double, Void, Void, k_company, 'F', k_monthly_data, "", "", 1482)
		end

	z_aeps: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("z_primary_security_annual_eps", "z_aeps", Void, Void, null_integer, null_integer, null_integer, null_integer, 2, Void, k_double, Void, Void, k_company, 'F', k_monthly_data, "", "", 1481)
		end

	z_qxps: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("z_primary_security_quarterly_xx_eps", "z_qxps", Void, Void, null_integer, null_integer, null_integer, null_integer, 2, Void, k_double, Void, Void, k_company, 'F', k_monthly_data, "", "", 1480)
		end

	z_qbps: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("z_primary_security_quarterly_bps", "z_qbps", Void, Void, null_integer, null_integer, null_integer, null_integer, 2, Void, k_double, Void, Void, k_company, 'F', k_monthly_data, "", "", 1479)
		end

	z_qeps: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("z_primary_security_quarterly_eps", "z_qeps", Void, Void, null_integer, null_integer, null_integer, null_integer, 2, Void, k_double, Void, Void, k_company, 'F', k_monthly_data, "", "", 1478)
		end

	z_eps: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("z_primary_security_eps", "z_eps", Void, Void, null_integer, null_integer, null_integer, null_integer, 2, Void, k_double, Void, Void, k_company, 'F', k_monthly_data, "", "", 1477)
		end

	z_ibes_totde5_p: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("z_ibes_totdelta_5_parent", "z_ibes_totde5_p", Void, Void, null_integer, null_integer, null_integer, null_integer, 2, Void, k_double, Void, Void, k_company, 'F', k_monthly_data, "", "", 1476)
		end

	z_ibes_consist_totearn_p: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("z_ibes_consistent_totearn_parent", "z_ibes_consist_totearn_p", Void, Void, null_integer, null_integer, null_integer, null_integer, 2, Void, k_double, Void, Void, k_company, 'F', k_monthly_data, "", "", 1475)
		end

	z_ibes_totearn_p: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("z_ibes_totearn_parent", "z_ibes_totearn_p", Void, Void, null_integer, null_integer, null_integer, null_integer, 2, Void, k_double, Void, Void, k_company, 'F', k_monthly_data, "", "", 1474)
		end

	z_ibes_totde5: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("z_ibes_totdelta_5_cons", "z_ibes_totde5", Void, Void, null_integer, null_integer, null_integer, null_integer, 2, Void, k_double, Void, Void, k_company, 'F', k_monthly_data, "", "", 1473)
		end

	z_ibes_consist_totearn: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("z_ibes_consistent_totearn_cons", "z_ibes_consist_totearn", Void, Void, null_integer, null_integer, null_integer, null_integer, 2, Void, k_double, Void, Void, k_company, 'F', k_monthly_data, "", "", 1472)
		end

	z_ibes_totearn: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("z_ibes_totearn_cons", "z_ibes_totearn", Void, Void, null_integer, null_integer, null_integer, null_integer, 2, Void, k_double, Void, Void, k_company, 'F', k_monthly_data, "", "", 1471)
		end

	z_if_npc_company: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("z_new_fundamental_data", "z_if_npc_company", Void, Void, null_integer, null_integer, null_integer, null_integer, 2, Void, k_integer, Void, Void, k_company, 'F', k_monthly_data, "", "", 1470)
		end

	falpha98l_unno: VARIABLE_DESCRIPTION_RECORD is
			--falpha98l_unno
		once
			create Result.make_model_company ("falpha98l_unno_raw", "falpha98l_unno_raw", 0, "", k_double, ' ', "50", 1469)
		end

	falpha98l_sino: VARIABLE_DESCRIPTION_RECORD is
			--falpha98l_sino
		once
			create Result.make_model_company ("falpha98l_sino_raw", "falpha98l_sino_raw", 0, "", k_double, ' ', "50", 1468)
		end

	falphaval_unno: VARIABLE_DESCRIPTION_RECORD is
			--falphaval_unno
		once
			create Result.make_model_company ("falphaval_unno_raw", "falphaval_unno_raw", 0, "", k_double, ' ', "39", 1467)
		end

	falphaval_sino: VARIABLE_DESCRIPTION_RECORD is
			--falphaval_sino
		once
			create Result.make_model_company ("falphaval_sino_raw", "falphaval_sino_raw", 0, "", k_double, ' ', "39", 1466)
		end

	gralpha2000_unno: VARIABLE_DESCRIPTION_RECORD is
			-- gralpha2000_unno
		once
			create Result.make_model_company ("gralpha2000_unno_raw", "gralpha2000_unno_raw", 0, "", k_double, ' ', "49", 1465)
		end

	gralpha2000_sino: VARIABLE_DESCRIPTION_RECORD is
			-- gralpha2000_sino
		once
			create Result.make_model_company ("gralpha2000_sino_raw", "gralpha2000_sino_raw", 0, "", k_double, ' ', "49", 1464)
		end

	falpha98_unno: VARIABLE_DESCRIPTION_RECORD is
			--falpha98_unno
		once
			create Result.make_model_company ("falpha98_unno_raw", "falpha98_unno_raw", 0, "", k_double, ' ', "48", 1463)
		end

	falpha98_sino: VARIABLE_DESCRIPTION_RECORD is
			--falpha98_sino
		once
			create Result.make_model_company ("falpha98_sino_raw", "falpha98_sino_raw", 0, "", k_double, ' ', "48", 1462)
		end

	falpha0_unno: VARIABLE_DESCRIPTION_RECORD is
			--falpha0_unno
		once
			create Result.make_model_company ("falpha0_unno_raw", "falpha0_unno_raw", 0, "", k_double, ' ', "53", 1461)
		end

	falpha0_sino: VARIABLE_DESCRIPTION_RECORD is
			--falpha0_sino
		once
			create Result.make_model_company ("falpha0_sino_raw", "falpha0_sino_raw", 0, "", k_double, ' ', "53", 1460)
		end

	talpha0_unno: VARIABLE_DESCRIPTION_RECORD is
			--talpha0_unno
		once
			create Result.make_model_company ("talpha0_unno_raw", "talpha0_unno_raw", 0, "", k_double, ' ', "0", 1459)
		end

	talpha0_sino: VARIABLE_DESCRIPTION_RECORD is
			--talpha0_sino
		once
			create Result.make_model_company ("talpha0_sino_raw", "talpha0_sino_raw", 1, "", k_double, ' ', "0", 1458)
		end

	gwam: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("goodwill_amortization", "gwam", "depr", null_integer, null_integer, null_integer, 1, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "1", k_profit_and_loss, 1446)
		end

	psof: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("pensions_off_balance_sheet", "psof", "ldbt", null_integer, null_integer, null_integer, 1, k_currency, k_double, Void, k_company, 'X', k_valdata_signature, "1", "liabilities_off_balance_sheet", 1442)
		end

	sbet: VARIABLE_DESCRIPTION_RECORD is
			--security level model sbet
		once
			create Result.make_model_security ("sbet", "sbet", 2, "", k_double, ' ', "MS_38", 1441)
		end

	sigmasq: VARIABLE_DESCRIPTION_RECORD is
			--sigma_square
		once
			create Result.make_model_company ("sigma_square", "sigmasq", 0, "", k_double, ' ', "56", 1440)
		end

	beta: VARIABLE_DESCRIPTION_RECORD is
			--BETA
		once
			create Result.make_model_company ("beta", "beta", 0, "", k_double, ' ', "55", 1439)
		end

	dtas: VARIABLE_DESCRIPTION_RECORD is
			-- dtas
		once
			create Result.make_model_company ("dtas", "dtas", 2, "", k_double, ' ', "25", 1438)
		end

	oppr: VARIABLE_DESCRIPTION_RECORD is
			--OPPR
		once
			create Result.make_model_company ("operating_income_to_price", "oppr", 0, "", k_double, ' ', "24", 1437)
		end

	ibpr: VARIABLE_DESCRIPTION_RECORD is
			--ibes_earnings_to_price_raw
		once
			create Result.make_model_company ("ibes_earnings_to_price_raw", "ibpr", 0, "", k_double, ' ', "23", 1436)
		end

	lepr: VARIABLE_DESCRIPTION_RECORD is
			--LEPR
		once
			create Result.make_model_company ("lepr", "lepr", 0, "", k_double, ' ', "22", 1435)
		end

	ctpr: VARIABLE_DESCRIPTION_RECORD is
			--CTPR
		once
			create Result.make_model_company ("ctpr", "ctpr", 0, "", k_double, ' ', "21", 1434)
		end

	stpr: VARIABLE_DESCRIPTION_RECORD is
			--sales_to_price_raw
		once
			create Result.make_model_company ("sales_to_price_raw", "stpr", 0, "", k_double, ' ', "20", 1433)
		end

	ocal: VARIABLE_DESCRIPTION_RECORD is
			-- ocal
		once
			create Result.make_mondex ("ocuralph", "ocal", k_mondex, "", "", null_integer, null_integer, null_integer, null_integer, 3, "", k_double, Void, "", k_company, ' ', k_mondex, "", 1432)
		end

	dxsp: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondex ("dehp_xs", "dxsp", k_mondex, "", "", null_integer, null_integer, null_integer, 148, 3, "currency", k_double, Void, "", k_company, ' ', k_mondex, "", 1431)
		end

	dehx: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondex ("dehp_exxs", "dehx", k_mondex, "", "", null_integer, null_integer, null_integer, 147, 3, "currency", k_double, Void, "", k_company, ' ', k_mondex, "", 1430)
		end

	ncal: VARIABLE_DESCRIPTION_RECORD is
			-- New current alpha
		once
			create Result.make_mondex ("newcalph", "ncal", k_mondex, "", "", null_integer, null_integer, null_integer, null_integer, 3, "", k_double, Void, "", k_company, ' ', k_mondex, "", 1429)
		end

	sgwx: VARIABLE_DESCRIPTION_RECORD is
			--similars_group_percentage_10
		once
			create Result.make_model_company ("similars_group_percentage_10", "sgwx", 0, "", k_double, ' ', "140", 1428)
		end

	sgw9: VARIABLE_DESCRIPTION_RECORD is
			--similars_group_percentage_9
		once
			create Result.make_model_company ("similars_group_percentage_9", "sgw9", 0, "", k_double, ' ', "139", 1427)
		end

	sgw8: VARIABLE_DESCRIPTION_RECORD is
			--similars_group_percentage_8
		once
			create Result.make_model_company ("similars_group_percentage_8", "sgw8", 0, "", k_double, ' ', "138", 1426)
		end

	sgw7: VARIABLE_DESCRIPTION_RECORD is
			--similars_group_percentage_7
		once
			create Result.make_model_company ("similars_group_percentage_7", "sgw7", 0, "", k_double, ' ', "137", 1425)
		end

	sgw6: VARIABLE_DESCRIPTION_RECORD is
			--similars_group_percentage_6
		once
			create Result.make_model_company ("similars_group_percentage_6", "sgw6", 0, "", k_double, ' ', "136", 1424)
		end

	sgw5: VARIABLE_DESCRIPTION_RECORD is
			--similars_group_percentage_5
		once
			create Result.make_model_company ("similars_group_percentage_5", "sgw5", 0, "", k_double, ' ', "135", 1423)
		end

	sgw4: VARIABLE_DESCRIPTION_RECORD is
			--similars_group_percentage_4
		once
			create Result.make_model_company ("similars_group_percentage_4", "sgw4", 0, "", k_double, ' ', "134", 1422)
		end

	sgw3: VARIABLE_DESCRIPTION_RECORD is
			--similars_group_percentage_3
		once
			create Result.make_model_company ("similars_group_percentage_3", "sgw3", 0, "", k_double, ' ', "133", 1421)
		end

	sgw2: VARIABLE_DESCRIPTION_RECORD is
			--similars_group_percentage_2
		once
			create Result.make_model_company ("similars_group_percentage_2", "sgw2", 0, "", k_double, ' ', "132", 1420)
		end

	sgw1: VARIABLE_DESCRIPTION_RECORD is
			--similars_group_percentage_1
		once
			create Result.make_model_company ("similars_group_percentage_1", "sgw1", 0, "", k_double, ' ', "131", 1419)
		end

	sgix: VARIABLE_DESCRIPTION_RECORD is
			--similars_group_index_10
		once
			create Result.make_model_company ("similars_group_index_10", "sgix", 0, "", k_integer, ' ', "130", 1418)
		end

	sgi9: VARIABLE_DESCRIPTION_RECORD is
			--similars_group_index_9
		once
			create Result.make_model_company ("similars_group_index_9", "sgi9", 0, "", k_integer, ' ', "129", 1417)
		end

	sgi8: VARIABLE_DESCRIPTION_RECORD is
			--similars_group_index_8
		once
			create Result.make_model_company ("similars_group_index_8", "sgi8", 0, "", k_integer, ' ', "128", 1416)
		end

	sgi7: VARIABLE_DESCRIPTION_RECORD is
			--similars_group_index_7
		once
			create Result.make_model_company ("similars_group_index_7", "sgi7", 0, "", k_integer, ' ', "127", 1415)
		end

	sgi6: VARIABLE_DESCRIPTION_RECORD is
			--similars_group_index_6
		once
			create Result.make_model_company ("similars_group_index_6", "sgi6", 0, "", k_integer, ' ', "126", 1414)
		end

	sgi5: VARIABLE_DESCRIPTION_RECORD is
			--similars_group_index_5
		once
			create Result.make_model_company ("similars_group_index_5", "sgi5", 0, "", k_integer, ' ', "125", 1413)
		end

	sgi4: VARIABLE_DESCRIPTION_RECORD is
			--similars_group_index_4
		once
			create Result.make_model_company ("similars_group_index_4", "sgi4", 0, "", k_integer, ' ', "124", 1412)
		end

	sgi3: VARIABLE_DESCRIPTION_RECORD is
			--similars_group_index_3
		once
			create Result.make_model_company ("similars_group_index_3", "sgi3", 0, "", k_integer, ' ', "123", 1411)
		end

	sgi2: VARIABLE_DESCRIPTION_RECORD is
			--similars_group_index_2
		once
			create Result.make_model_company ("similars_group_index_2", "sgi2", 0, "", k_integer, ' ', "122", 1410)
		end

	sgi1: VARIABLE_DESCRIPTION_RECORD is
			--similars_group_index_1
		once
			create Result.make_model_company ("similars_group_index_1", "sgi1", 0, "", k_integer, ' ', "121", 1409)
		end

	igwx: VARIABLE_DESCRIPTION_RECORD is
			--industry_group_percentage_10
		once
			create Result.make_model_company ("industry_group_percentage_10", "igwx", 0, "", k_double, ' ', "120", 1408)
		end

	igw9: VARIABLE_DESCRIPTION_RECORD is
			--industry_group_percentage_9
		once
			create Result.make_model_company ("industry_group_percentage_9", "igw9", 0, "", k_double, ' ', "119", 1407)
		end

	igw8: VARIABLE_DESCRIPTION_RECORD is
			--industry_group_percentage_8
		once
			create Result.make_model_company ("industry_group_percentage_8", "igw8", 0, "", k_double, ' ', "118", 1406)
		end

	igw7: VARIABLE_DESCRIPTION_RECORD is
			--industry_group_percentage_7
		once
			create Result.make_model_company ("industry_group_percentage_7", "igw7", 0, "", k_double, ' ', "117", 1405)
		end

	igw6: VARIABLE_DESCRIPTION_RECORD is
			--industry_group_percentage_6
		once
			create Result.make_model_company ("industry_group_percentage_6", "igw6", 0, "", k_double, ' ', "116", 1404)
		end

	igw5: VARIABLE_DESCRIPTION_RECORD is
			--industry_group_percentage_5
		once
			create Result.make_model_company ("industry_group_percentage_5", "igw5", 0, "", k_double, ' ', "115", 1403)
		end

	igw4: VARIABLE_DESCRIPTION_RECORD is
			--industry_group_percentage_4
		once
			create Result.make_model_company ("industry_group_percentage_4", "igw4", 0, "", k_double, ' ', "114", 1402)
		end

	igw3: VARIABLE_DESCRIPTION_RECORD is
			--industry_group_percentage_3
		once
			create Result.make_model_company ("industry_group_percentage_3", "igw3", 0, "", k_double, ' ', "113", 1401)
		end

	igw2: VARIABLE_DESCRIPTION_RECORD is
			--industry_group_percentage_2
		once
			create Result.make_model_company ("industry_group_percentage_2", "igw2", 0, "", k_double, ' ', "112", 1400)
		end

	igw1: VARIABLE_DESCRIPTION_RECORD is
			--industry_group_percentage_1
		once
			create Result.make_model_company ("industry_group_percentage_1", "igw1", 0, "", k_double, ' ', "111", 1399)
		end

	igix: VARIABLE_DESCRIPTION_RECORD is
			--industry_group_index_10
		once
			create Result.make_model_company ("industry_group_index_10", "igix", 0, "", k_integer, ' ', "110", 1398)
		end

	igi9: VARIABLE_DESCRIPTION_RECORD is
			--industry_group_index_9
		once
			create Result.make_model_company ("industry_group_index_9", "igi9", 0, "", k_integer, ' ', "109", 1397)
		end

	igi8: VARIABLE_DESCRIPTION_RECORD is
			--industry_group_index_8
		once
			create Result.make_model_company ("industry_group_index_8", "igi8", 0, "", k_integer, ' ', "108", 1396)
		end

	igi7: VARIABLE_DESCRIPTION_RECORD is
			--industry_group_index_7
		once
			create Result.make_model_company ("industry_group_index_7", "igi7", 0, "", k_integer, ' ', "107", 1395)
		end

	igi6: VARIABLE_DESCRIPTION_RECORD is
			--industry_group_index_6
		once
			create Result.make_model_company ("industry_group_index_6", "igi6", 0, "", k_integer, ' ', "106", 1394)
		end

	igi5: VARIABLE_DESCRIPTION_RECORD is
			--industry_group_index_5
		once
			create Result.make_model_company ("industry_group_index_5", "igi5", 0, "", k_integer, ' ', "105", 1393)
		end

	igi4: VARIABLE_DESCRIPTION_RECORD is
			--industry_group_index_4
		once
			create Result.make_model_company ("industry_group_index_4", "igi4", 0, "", k_integer, ' ', "104", 1392)
		end

	igi3: VARIABLE_DESCRIPTION_RECORD is
			--industry_group_index_3
		once
			create Result.make_model_company ("industry_group_index_3", "igi3", 0, "", k_integer, ' ', "103", 1391)
		end

	igi2: VARIABLE_DESCRIPTION_RECORD is
			--industry_group_index_2
		once
			create Result.make_model_company ("industry_group_index_2", "igi2", 0, "", k_integer, ' ', "102", 1390)
		end

	igi1: VARIABLE_DESCRIPTION_RECORD is
			--industry_group_index_1
		once
			create Result.make_model_company ("industry_group_index_1", "igi1", 0, "", k_integer, ' ', "101", 1389)
		end

	instit_hold: VARIABLE_DESCRIPTION_RECORD is
			--security level model instit_hold
		once
			create Result.make_model_security ("percentage_of_stock_held_by_institutions", "instit_hold", 0, "", k_integer, ' ', "MS_46", 1388)
		end

	shs20: VARIABLE_DESCRIPTION_RECORD is
			--security level model shs20
		once
			create Result.make_model_security ("shares_20", "shs20", 0, "", k_double, ' ', "MS_45", 1387)
		end

	xrate: VARIABLE_DESCRIPTION_RECORD is
			--security level model xrate
		once
			create Result.make_model_security ("currency_exchange_rate", "xrate", 0, "", k_double, ' ', "MS_44", 1386)
		end

	short_ok: VARIABLE_DESCRIPTION_RECORD is
			--security level model short_ok
		once
			create Result.make_model_security ("can_be_sold_short", "short_ok", 0, "", k_string, ' ', "MS_43", 1385)
		end

	targetday_divyield: VARIABLE_DESCRIPTION_RECORD is
			--security level model targetday_divyield
		once
			create Result.make_model_security ("target_divyield", "targetday_divyield", 0, "", k_double, ' ', "MS_42", 1384)
		end

	exdate: VARIABLE_DESCRIPTION_RECORD is
			--security level model exdate
		once
			create Result.make_model_security ("ex_dividend_date", "exdate", 0, "", k_integer, ' ', "MS_41", 1383)
		end

	adj_to_base: VARIABLE_DESCRIPTION_RECORD is
			--security level model adj_to_base
		once
			create Result.make_model_security ("forwards_cumadj_on_last_monthly_date", "adj_to_base", 0, "", k_double, ' ', "MS_40", 1382)
		end

	adj_to_target: VARIABLE_DESCRIPTION_RECORD is
			--security level model adj_to_target
		once
			create Result.make_model_security ("adjustment_to_target_date", "adj_to_target", 0, "", k_double, ' ', "MS_39", 1381)
		end

	return_wdt: VARIABLE_DESCRIPTION_RECORD is
			--security level model return_wdt
		once
			create Result.make_model_security ("return_week_to_date", "return_wdt", 0, "", k_double, ' ', "MS_37", 1380)
		end

	return_mlag: VARIABLE_DESCRIPTION_RECORD is
			--security level model return_mlag
		once
			create Result.make_model_security ("return_mlag", "return_mlag", 0, "", k_double, ' ', "MS_36", 1379)
		end

	return_mdt: VARIABLE_DESCRIPTION_RECORD is
			--security level model return_mdt
		once
			create Result.make_model_security ("return_mdt", "return_mdt", 0, "", k_double, ' ', "MS_35", 1378)
		end

	return_20: VARIABLE_DESCRIPTION_RECORD is
			--security level model return_20
		once
			create Result.make_model_security ("20_day_lagged_return", "return_20", 0, "", k_double, ' ', "MS_34", 1377)
		end

	return_day: VARIABLE_DESCRIPTION_RECORD is
			--security level model return_day
		once
			create Result.make_model_security ("return_day", "return_day", 0, "", k_double, ' ', "MS_33", 1376)
		end

	price20: VARIABLE_DESCRIPTION_RECORD is
			--security level model price20
		once
			create Result.make_model_security ("price20", "price20", 0, "", k_double, ' ', "MS_32", 1375)
		end

	price0: VARIABLE_DESCRIPTION_RECORD is
			--security level model price0
		once
			create Result.make_model_security ("overnight_base_unit_closing_price", "price0", 0, "", k_double, ' ', "MS_31", 1374)
		end

	old_price: VARIABLE_DESCRIPTION_RECORD is
			--security level model old_price
		once
			create Result.make_model_security ("old_price", "old_price", 0, "", k_integer, ' ', "MS_30", 1373)
		end

	idate_last: VARIABLE_DESCRIPTION_RECORD is
			--security level model idate_last
		once
			create Result.make_model_security ("idate_last", "idate_last", 0, "", k_integer, ' ', "MS_29", 1372)
		end

	volmed: VARIABLE_DESCRIPTION_RECORD is
			--security level model volmed
		once
			create Result.make_model_security ("median_volume", "volmed", 0, "", k_double, ' ', "MS_28", 1371)
		end

	market_value: VARIABLE_DESCRIPTION_RECORD is
			--security level model market_value
		once
			create Result.make_model_security ("market_value", "market_value", 0, "", k_double, ' ', "MS_26", 1370)
		end

	alt_shsout_sup: VARIABLE_DESCRIPTION_RECORD is
			--security level model alt_shsout_sup
		once
			create Result.make_model_security ("alternate_shares_outstanding_supplier", "alt_shsout_sup", 0, "", k_integer, ' ', "MS_25", 1369)
		end

	alt_shsout: VARIABLE_DESCRIPTION_RECORD is
			--security level model alt_shsout
		once
			create Result.make_model_security ("alternate_shares_outstanding", "alt_shsout", 0, "", k_double, ' ', "MS_24", 1368)
		end

	icur_s: VARIABLE_DESCRIPTION_RECORD is
			--security level model icur_s
		once
			create Result.make_model_security ("security_currency_xrf", "icur_s", 0, "", k_integer, ' ', "MS_18", 1367)
		end

	spare_ref: VARIABLE_DESCRIPTION_RECORD is
			--security level model spare_ref
		once
			create Result.make_model_security ("spare_ref", "spare_ref", 0, "", k_string, ' ', "MS_17", 1366)
		end

	lrxrf: VARIABLE_DESCRIPTION_RECORD is
			--security level model lrxrf
		once
			create Result.make_model_security ("local_rxrf", "lrxrf", 0, "", k_integer, ' ', "MS_16", 1365)
		end

	primary_security: VARIABLE_DESCRIPTION_RECORD is
			--security level model primary_security
		once
			create Result.make_model_security ("primary_security", "primary_security", 0, "", k_integer, ' ', "MS_15", 1364)
		end

	baddata_s: VARIABLE_DESCRIPTION_RECORD is
			--security level model baddata_s
		once
			create Result.make_model_security ("baddat_code_s", "baddata_s", 0, "", k_string, ' ', "MS_14", 1363)
		end

	margin_stock: VARIABLE_DESCRIPTION_RECORD is
			--security level model margin_stock
		once
			create Result.make_model_security ("margin_stock", "margin_stock", 0, "", k_string, ' ', "MS_13", 1362)
		end

	ownership: VARIABLE_DESCRIPTION_RECORD is
			--security level model ownership
		once
			create Result.make_model_security ("ownership", "ownership", 0, "", k_string, ' ', "MS_12", 1361)
		end

	iss_type: VARIABLE_DESCRIPTION_RECORD is
			--security level model iss_type
		once
			create Result.make_model_security ("iss_type", "iss_type", 0, "", k_string, ' ', "MS_11", 1360)
		end

	iss_descr: VARIABLE_DESCRIPTION_RECORD is
			--security level model iss_descr
		once
			create Result.make_model_security ("issue_description", "iss_descr", 0, "", k_string, ' ', "MS_10", 1359)
		end

	gvikey: VARIABLE_DESCRIPTION_RECORD is
			--security level model gvikey
		once
			create Result.make_model_security ("gv_security_code", "gvikey", 0, "", k_string, ' ', "MS_09", 1358)
		end

	cusipci: VARIABLE_DESCRIPTION_RECORD is
			--security level model cusipci
		once
			create Result.make_model_security ("cusip_code", "cusipci", 0, "", k_string, ' ', "MS_08", 1357)
		end

	lcxrf: VARIABLE_DESCRIPTION_RECORD is
			--security level model lcxrf
		once
			create Result.make_model_security ("positive_cxrf_if_same_zone_else_negative_fcxrf", "lcxrf", 0, "", k_integer, ' ', "MS_07", 1356)
		end

	izone_c: VARIABLE_DESCRIPTION_RECORD is
			--security level model izone_c
		once
			create Result.make_model_security ("company_zone", "izone_c", 0, "", k_string, ' ', "MS_06", 1355)
		end

	izone_r: VARIABLE_DESCRIPTION_RECORD is
			--security level model izone_r
		once
			create Result.make_model_security ("primary_listing_zone", "izone_r", 0, "", k_string, ' ', "MS_05", 1354)
		end

	izone_s: VARIABLE_DESCRIPTION_RECORD is
			--security level model izone_s
		once
			create Result.make_model_security ("primary_security_zone", "izone_s", 0, "", k_string, ' ', "MS_04", 1353)
		end

	primary_rxrf: VARIABLE_DESCRIPTION_RECORD is
			--security level model primary_rxrf
		once
			create Result.make_model_security ("primary_listing_xrf", "primary_rxrf", 0, "", k_integer, ' ', "MS_03", 1352)
		end

	cxrf: VARIABLE_DESCRIPTION_RECORD is
			--security level model cxrf
		once
			create Result.make_model_security ("company_xrf_code", "cxrf", 0, "", k_integer, ' ', "MS_02", 1351)
		end

	sxrf: VARIABLE_DESCRIPTION_RECORD is
			--security level model sxrf
		once
			create Result.make_model_security ("security_xrf_code", "sxrf", 0, "", k_integer, ' ', "MS_01", 1350)
		end

	liquidity: VARIABLE_DESCRIPTION_RECORD is
			--listing level model liquidity
		once
			create Result.make_model_listing ("liquidity_flag", "liquidity", 0, "", k_integer, ' ', "MR_34", 1349)
		end

	xrate_listing: VARIABLE_DESCRIPTION_RECORD is
			--listing level model xrate
		once
			create Result.make_model_listing ("xrate", "xrate", 0, "", k_double, ' ', "MR_33", 1348)
		end

	atype: VARIABLE_DESCRIPTION_RECORD is
			--listing level model atype
		once
			create Result.make_model_listing ("atype", "atype", 0, "", k_string, ' ', "MR_32", 1347)
		end

	index_compos: VARIABLE_DESCRIPTION_RECORD is
			--listing level model index_compos
		once
			create Result.make_model_listing ("index_of_composite_asset_held", "index_compos", 0, "", k_integer, ' ', "MR_31", 1346)
		end

	index_future_bogie: VARIABLE_DESCRIPTION_RECORD is
			--listing level model index_future_bogie
		once
			create Result.make_model_listing ("index_future_bogie", "index_future_bogie", 0, "", k_integer, ' ', "MR_30", 1345)
		end

	bogshs: VARIABLE_DESCRIPTION_RECORD is
			--listing level model idate_last
		once
			create Result.make_model_listing ("bogshs", "bogshs", 0, "", "array", ' ', "MR_29", 1344)
		end

	index_membership: VARIABLE_DESCRIPTION_RECORD is
			--listing level model idate_last
		once
			create Result.make_model_listing ("bogie_membership_array", "index_membership", 0, "", "array", ' ', "MR_28", 1343)
		end

	idate_last_listing: VARIABLE_DESCRIPTION_RECORD is
			--listing level model idate_last
		once
			create Result.make_model_listing ("latest_available_price", "idate_last", 0, "", k_integer, ' ', "MR_22", 1342)
		end

	icur_r: VARIABLE_DESCRIPTION_RECORD is
			--listing level model icur_r
		once
			create Result.make_model_listing ("listing_currency_id", "icur_r", 0, "", k_integer, ' ', "MR_21", 1341)
		end

	trading_unit: VARIABLE_DESCRIPTION_RECORD is
			--listing level model trading_unit
		once
			create Result.make_model_listing ("trading_unit", "trading_unit", 0, "", k_integer, ' ', "MR_20", 1340)
		end

	local2: VARIABLE_DESCRIPTION_RECORD is
			--listing level model local2
		once
			create Result.make_model_listing ("local2", "local2", 0, "", k_string, ' ', "MR_18", 1339)
		end

	local1: VARIABLE_DESCRIPTION_RECORD is
			--listing level model local1
		once
			create Result.make_model_listing ("local1", "local1", 0, "", k_string, ' ', "MR_17", 1338)
		end

	settle_date: VARIABLE_DESCRIPTION_RECORD is
			--listing level model settle_date
		once
			create Result.make_model_listing ("settle_date", "settle_date", 0, "", k_integer, ' ', "MR_16", 1337)
		end

	baddata_r: VARIABLE_DESCRIPTION_RECORD is
			--listing level model baddata_r
		once
			create Result.make_model_listing ("baddat_code_r", "baddata_r", 0, "", k_string, ' ', "MR_15", 1336)
		end

	rtpricing: VARIABLE_DESCRIPTION_RECORD is
			--listing level model rtpricing
		once
			create Result.make_model_listing ("rtpricing", "rtpricing", 0, "", k_string, ' ', "MR_14", 1335)
		end

	primary_listing: VARIABLE_DESCRIPTION_RECORD is
			--listing level model primary_listing
		once
			create Result.make_model_listing ("primary_listing", "primary_listing", 0, "", k_integer, ' ', "MR_13", 1334)
		end

	valoren: VARIABLE_DESCRIPTION_RECORD is
			--listing level model valoren
		once
			create Result.make_model_listing ("valoren", "valoren", 0, "", k_string, ' ', "MR_12", 1333)
		end

	reuters: VARIABLE_DESCRIPTION_RECORD is
			--listing level model reuters
		once
			create Result.make_model_listing ("reuters", "reuters", 0, "", k_string, ' ', "MR_11", 1332)
		end

	sedol: VARIABLE_DESCRIPTION_RECORD is
			--listing level model sedol
		once
			create Result.make_model_listing ("sedol_key", "sedol", 0, "", k_string, ' ', "MR_10", 1331)
		end

	riemsym: VARIABLE_DESCRIPTION_RECORD is
			--listing level model riemsym
		once
			create Result.make_model_listing ("riemsym", "riemsym", 0, "", k_string, ' ', "MR_09", 1330)
		end

	brimsym: VARIABLE_DESCRIPTION_RECORD is
			--listing level model brimsym
		once
			create Result.make_model_listing ("brimsym", "brimsym", 0, "", k_string, ' ', "MR_08", 1329)
		end

	lcxrf_listing: VARIABLE_DESCRIPTION_RECORD is
			--listing level model lcxrf
		once
			create Result.make_model_listing ("zonal_cxrf", "lcxrf", 0, "", k_integer, ' ', "MR_07", 1328)
		end

	lsxrf: VARIABLE_DESCRIPTION_RECORD is
			--listing level model lsxrf
		once
			create Result.make_model_listing ("zonal_sxrf", "lsxrf", 0, "", k_integer, ' ', "MR_06", 1327)
		end

	izone_c_listing: VARIABLE_DESCRIPTION_RECORD is
			--listing level model izone_c
		once
			create Result.make_model_listing ("company_zone", "izone_c", 0, "", k_string, ' ', "MR_05", 1326)
		end

	izone_s_listing: VARIABLE_DESCRIPTION_RECORD is
			--listing level model izone_s
		once
			create Result.make_model_listing ("security_zone", "izone_s", 0, "", k_string, ' ', "MR_04", 1325)
		end

	cxrf_listing: VARIABLE_DESCRIPTION_RECORD is
			--listing level model cxrf
		once
			create Result.make_model_listing ("company_xrf_code", "cxrf", 0, "", k_integer, ' ', "MR_03", 1324)
		end

	srxf: VARIABLE_DESCRIPTION_RECORD is
			--listing level model srxf
		once
			create Result.make_model_listing ("security_xrf_code", "srxf", 0, "", k_integer, ' ', "MR_02", 1323)
		end

	rxrf: VARIABLE_DESCRIPTION_RECORD is
			--listing level model rxrf
		once
			create Result.make_model_listing ("listing_xrf_code", "rxrf", 0, "", k_integer, ' ', "MR_01", 1322)
		end

	cap_quartile: VARIABLE_DESCRIPTION_RECORD is
			--CAP_QUARTILE
		once
			create Result.make_model_company ("cap_quartile", "cap_quartile", 0, "", k_integer, ' ', "69", 1321)
		end

	val_gro_flag: VARIABLE_DESCRIPTION_RECORD is
			--value_growth_flag
		once
			create Result.make_model_company ("value_growth_flag", "val_gro_flag", 0, "", k_string, ' ', "68", 1320)
		end

	caprelsec: VARIABLE_DESCRIPTION_RECORD is
			--capitalization_relative_to_sector
		once
			create Result.make_model_company ("capitalization_relative_to_sector", "caprelsec", 0, "", k_double, ' ', "67", 1319)
		end

	if_standard: VARIABLE_DESCRIPTION_RECORD is
			--if_standard
		once
			create Result.make_model_company ("if_standard", "if_standard", 0, "", k_string, ' ', "66", 1318)
		end

	izone_c_company: VARIABLE_DESCRIPTION_RECORD is
			--home_country_zone
		once
			create Result.make_model_company ("home_country_zone", "izone_c", 0, "", k_string, ' ', "65", 1317)
		end

	icw: VARIABLE_DESCRIPTION_RECORD is
			--country_weights
		once
			create Result.make_model_company ("country_weights", "icw", 0, "", "vector", ' ', "64", 1316)
		end

	ici: VARIABLE_DESCRIPTION_RECORD is
			--country_group_index
		once
			create Result.make_model_company ("country_group_index", "ici", 0, "", k_integer, ' ', "63", 1315)
		end

	ictry: VARIABLE_DESCRIPTION_RECORD is
			--country_index
		once
			create Result.make_model_company ("country_index", "ictry", 0, "", k_integer, ' ', "62", 1314)
		end

	qntile: VARIABLE_DESCRIPTION_RECORD is
			--quantile_1_15_for_short_term_alpha
		once
			create Result.make_model_company ("quantile_1_15_for_short_term_alpha", "qntile", 0, "", k_integer, ' ', "61", 1313)
		end

	isect: VARIABLE_DESCRIPTION_RECORD is
			--cap_sector
		once
			create Result.make_model_company ("cap_sector", "isect", 0, "", k_integer, ' ', "60", 1312)
		end

	tsell: VARIABLE_DESCRIPTION_RECORD is
			-- tsell
		once
			create Result.make_model_company ("sales_cost", "tsell", 0, "", k_double, ' ', "58", 1311)
		end

	tbuy: VARIABLE_DESCRIPTION_RECORD is
			--tbuy
		once
			create Result.make_model_company ("tbuy", "tbuy", 0, "", k_double, ' ', "57", 1310)
		end

	talpha0: VARIABLE_DESCRIPTION_RECORD is
			--talpha0
		once
			create Result.make_model_company ("talpha0", "talpha0", 0, "", k_double, ' ', "54", 1309)
		end

	falpha0: VARIABLE_DESCRIPTION_RECORD is
			--falpha0
		once
			create Result.make_model_company ("falpha0", "falpha0", 0, "", k_double, ' ', "53", 1308)
		end

	valpha: VARIABLE_DESCRIPTION_RECORD is
			--phat_to_p_anomaly_ratio - used in MODEL current alpha calculation
		once
			create Result.make_model_company ("phat_to_p_anomaly_ratio", "valpha", 0, "", k_double, ' ', "52", 1307)
		end

	grtalpha2000: VARIABLE_DESCRIPTION_RECORD is
			--grtalpha2000
		once
			create Result.make_model_risk ("grtalpha2000", "grtalpha2000", 0, k_double, "51", 1306)
		end

	falpha98l: VARIABLE_DESCRIPTION_RECORD is
			--falpha98l
		once
			create Result.make_model_risk ("falpha98l", "falpha98l", 0, k_double, "50", 1305)
		end

	gralpha2000: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("gralpha2000", "gralpha2000", "GRAL2000", Void, null_integer, null_integer, 3, Void, missing_double, missing_double, missing_double, missing_double, "", Void, "1", ' ', k_monthly_data, "", k_risk, 1304)
		end

	falpha98: VARIABLE_DESCRIPTION_RECORD is
			--falpha98
		once
			create Result.make_model_risk ("falpha98", "falpha98", 0, k_double, "43", 1303)
		end

	largecap_wgt: VARIABLE_DESCRIPTION_RECORD is
			--largecap_wgt
		once
			create Result.make_model_risk ("large_cap_weight", "largecap_wgt", 0, k_double, "47", 1302)
		end

	r8sn: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("relative_strength_18_month_sim_nor", "r8sn", "Rs18.SMN", k_risk, null_integer, null_integer, 2, "", -3, 3, missing_double, missing_double, "SIMNORM:RS18", Void, Void, ' ', k_monthly_data, "", k_risk, 1301)
		end

	rysn: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("relative_strength_1_year_sim_nor", "rysn", "Rs12.SMN", k_risk, null_integer, null_integer, 2, "", -3, 3, missing_double, missing_double, "SIMNORM:RS12", Void, Void, ' ', k_monthly_data, "", k_risk, 1300)
		end

	sgw: VARIABLE_DESCRIPTION_RECORD is
			-- similars_group_weight
		once
			create Result.make_model_company ("similars_group_weight", "sgw", 0, "", "vector", ' ', "44", 1299)
		end

	risk_data: VARIABLE_DESCRIPTION_RECORD is
			--risk_descriptors
		once
			create Result.make_model_company ("risk_descriptors", "risk_data", 0, "", "vector", ' ', "42", 1297)
		end

	fc_period: VARIABLE_DESCRIPTION_RECORD is
			--fc_period
		once
			create Result.make_model_company ("fc_period", "fc_period", 0, "", k_string, ' ', "41", 1296)
		end

	barrabeta: VARIABLE_DESCRIPTION_RECORD is
			--barra_fundamental_beta_prediction
		once
			create Result.make_model_company ("barra_fundamental_beta_prediction", "barrabeta", 0, "", k_double, ' ', "40", 1295)
		end

	falphaval: VARIABLE_DESCRIPTION_RECORD is
			--falphaval
		once
			create Result.make_model_company ("falphaval", "falphaval", 0, "", k_double, ' ', "39", 1294)
		end

	iyymm_fisc: VARIABLE_DESCRIPTION_RECORD is
			--fiscal_month
		once
			create Result.make_model_company ("fiscal_month", "iyymm_fisc", 0, "", k_integer, ' ', "38", 1293)
		end

	fisc_mend_lag: VARIABLE_DESCRIPTION_RECORD is
			--number_of_months_back_to_fiscal_monthend
		once
			create Result.make_model_company ("number_of_months_back_to_fiscal_monthend", "fisc_mend_lag", 0, "", k_integer, ' ', "37", 1292)
		end

	eps_mend_lag: VARIABLE_DESCRIPTION_RECORD is
			--number_of_months_back_to_earnings_monthend
		once
			create Result.make_model_company ("number_of_months_back_to_earnings_monthend", "eps_mend_lag", 0, "", k_integer, ' ', "36", 1291)
		end

	icur_f: VARIABLE_DESCRIPTION_RECORD is
			--currency_index
		once
			create Result.make_model_company ("currency_index", "icur_f", 0, "", k_integer, ' ', "17", 1289)
		end

	icur_m: VARIABLE_DESCRIPTION_RECORD is
			--company_main_currency_index
		once
			create Result.make_model_company ("company_main_currency_index", "icur_m", 0, "", k_integer, ' ', "16", 1288)
		end

	exchange_listing: VARIABLE_DESCRIPTION_RECORD is
			--exchange_listing
		once
			create Result.make_model_company ("exchange_listing", "exchange_listing", 0, "", k_string, ' ', "15", 1287)
		end

	i_valuation_baddata: VARIABLE_DESCRIPTION_RECORD is
			--i_valuation_baddata_code
		once
			create Result.make_model_company ("i_valuation_baddata_code", "i_valuation_baddata", 0, "", k_string, ' ', "14", 1286)
		end

	i_monthlydet_baddata: VARIABLE_DESCRIPTION_RECORD is
			--monthlydetail_bad_data_flag
		once
			create Result.make_model_company ("monthlydetail_bad_data_flag", "i_monthlydet_baddata", 0, "", k_string, ' ', "13", 1285)
		end

	baddata_c: VARIABLE_DESCRIPTION_RECORD is
			--baddat_code_c
		once
			create Result.make_model_company ("baddat_code_c", "baddata_c", 0, "", k_string, ' ', "12", 1284)
		end

	foreign_code: VARIABLE_DESCRIPTION_RECORD is
			--foreign_code
		once
			create Result.make_model_company ("foreign_code", "foreign_code", 0, "", k_string, ' ', "11", 1283)
		end

	factor_alpha: VARIABLE_DESCRIPTION_RECORD is
			--factor_alpha
		once
			create Result.make_model_company ("factor_alpha", "factor_alpha", 0, "", k_double, ' ', "10", 1282)
		end

	industry_code: VARIABLE_DESCRIPTION_RECORD is
			--industry_code
		once
			create Result.make_model_company ("industry_code", "industry_code", 0, "", k_integer, ' ', "9", 1281)
		end

	local_lang_name: VARIABLE_DESCRIPTION_RECORD is
			--local_lang_name
		once
			create Result.make_model_company ("local_lang_name", "local_lang_name", 0, "", k_string, ' ', "8", 1280)
		end

	co_name: VARIABLE_DESCRIPTION_RECORD is
			--company_name
		once
			create Result.make_model_company ("company_name", "co_name", 0, "", k_string, ' ', "7", 1279)
		end

	primary_listing_zone: VARIABLE_DESCRIPTION_RECORD is
			-- Model company primary listing zone
		once
			create Result.make_model ("primary_listing_zone", "izone_r", 0, k_string, "5", 1277)
		ensure
			not_void: Result /= Void
		end

	primary_listing_rxrf: VARIABLE_DESCRIPTION_RECORD is
			-- Model company primary listing xrf code
		once
			create Result.make_model ("primary_listing_rxrf", "primary_rxrf", 0, k_integer, "3", 1275)
		ensure
			not_void: Result /= Void
		end

	primary_sxrf: VARIABLE_DESCRIPTION_RECORD is
			--primary_security_sxrf
		once
			create Result.make_model_company ("primary_security_sxrf", "primary_sxrf", 0, "", k_integer, ' ', "2", 1274)
		end

	ibes_standard_dev_ltg: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_ibes ("ibes_standard_dev_ltg", "ibes_standard_dev_ltg", Void, k_double, k_rose_company_date, k_ibes, 1272)
		end

	ibes_standard_dev_fy3: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_ibes ("ibes_standard_dev_fy3", "ibes_standard_dev_fy3", Void, k_double, k_rose_company_date, k_ibes, 1271)
		end

	ibes_standard_dev_fy2: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_ibes ("ibes_standard_dev_fy2", "ibes_standard_dev_fy2", Void, k_double, k_rose_company_date, k_ibes, 1270)
		end

	ibes_standard_dev_fy1: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_ibes ("ibes_standard_dev_fy1", "ibes_standard_dev_fy1", Void, k_double, k_rose_company_date, k_ibes, 1268)
		end

	ibes_number_down_fy3: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_ibes ("ibes_number_down_fy3", "ibes_number_down_fy3", Void, k_integer, k_rose_company_date, k_ibes, 1267)
		end

	ibes_number_down_fy2: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_ibes ("ibes_number_down_fy2", "ibes_number_down_fy2", Void, k_integer, k_rose_company_date, k_ibes, 1266)
		end

	ibes_number_down_fy1: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_ibes ("ibes_number_down_fy1", "ibes_number_down_fy1", Void, k_integer, k_rose_company_date, k_ibes, 1264)
		end

	ibes_number_up_fy3: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_ibes ("ibes_number_up_fy3", "ibes_number_up_fy3", Void, k_integer, k_rose_company_date, k_ibes, 1263)
		end

	ibes_number_up_fy2: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_ibes ("ibes_number_up_fy2", "ibes_number_up_fy2", Void, k_integer, k_rose_company_date, k_ibes, 1262)
		end

	ibes_number_up_fy1: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_ibes ("ibes_number_up_fy1", "ibes_number_up_fy1", Void, k_integer, k_rose_company_date, k_ibes, 1260)
		end

	scal: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("rsmcalyr", "scal", "RsmCalYR", k_risk, null_integer, 143, 2, Void, -10, 10, missing_double, missing_double, "", Void, Void, 'F', k_monthly_data, "", k_risk, 1257)
		end

	sdir: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("rsmdiryr", "sdir", "RsmDirYR", k_risk, null_integer, 142, 2, Void, -10, 10, missing_double, missing_double, "", Void, Void, 'F', k_monthly_data, "", k_risk, 1256)
		end

	rcal: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("radcalyr", "rcal", "RadCalYR", k_risk, null_integer, 140, 2, Void, -10, 10, missing_double, missing_double, "", Void, Void, 'F', k_monthly_data, "", k_risk, 1254)
		end

	rdir: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("raddiryr", "rdir", "RadDirYR", k_risk, null_integer, 139, 2, Void, -10, 10, missing_double, missing_double, "", Void, Void, 'F', k_monthly_data, "", k_risk, 1253)
		end

	ixprimary: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("ixprimary", "ixprimary", Void, Void, null_integer, null_integer, null_integer, null_integer, 2, Void, k_integer, Void, Void, k_company, 'F', k_monthly_data, "", k_mondat, 1251)
		end

	ntcx: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("net_common_income_after_fasb142_writeoff", "ntcx", "p_and_l", null_integer, null_integer, null_integer, 1, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "1", k_profit_and_loss, 1245)
		end

	xtr0: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("extraordinary_items_incl_fasb142_writeoff", "xtr0", "exdo", null_integer, null_integer, null_integer, 1, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "1", k_profit_and_loss, 1244)
		end

	xtr1: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("fasb142_one_time_writeoff", "xtr1", Void, null_integer, null_integer, null_integer, 1, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "1", k_profit_and_loss, 1243)
		end

	ntix: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("net_income_after_fasb142_writeoff", "ntix", "ntcx", null_integer, null_integer, 223, 1, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "1", k_profit_and_loss, 1237)
		end

	psov: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("pensions_ovr", "psov", "ldbt", null_integer, null_integer, 182, 1, k_currency, k_double, Void, k_company, 'X', k_valdata_signature, "1", "liabilities_off_balance_sheet", 1236)
		end

	psun: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("pensions_undr", "psun", "ldbt", null_integer, null_integer, 181, 1, k_currency, k_double, Void, k_company, 'X', k_valdata_signature, "1", "liabilities_off_balance_sheet", 1235)
		end

	t_calf_mqal: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondex ("t_calf_mqal", "t_calf_mqal", k_mondex, Void, Void, null_integer, null_integer, null_integer, null_integer, 3, Void, k_real, Void, Void, k_company, '0', k_mondex, k_risk, 1171)
		end

	t_btop_ibep: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondex ("t_btop_ibep", "t_btop_ibep", k_mondex, Void, Void, null_integer, null_integer, null_integer, null_integer, 3, Void, k_real, Void, Void, k_company, '0', k_mondex, k_risk, 1170)
		end

	t_btop_ehtp: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondex ("t_btop_ehtp", "t_btop_ehtp", k_mondex, Void, Void, null_integer, null_integer, null_integer, null_integer, 3, Void, k_real, Void, Void, k_company, '0', k_mondex, k_risk, 1169)
		end

	t_etop_ehtp: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondex ("t_etop_ehtp", "t_etop_ehtp", k_mondex, Void, Void, null_integer, null_integer, null_integer, null_integer, 3, Void, k_real, Void, Void, k_company, '0', k_mondex, k_risk, 1168)
		end

	t_roeg_egro: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondex ("t_roeg_egro", "t_roeg_egro", k_mondex, Void, Void, null_integer, null_integer, null_integer, null_integer, 3, Void, k_real, Void, Void, k_company, '0', k_mondex, k_risk, 1167)
		end

	t_roeg_detp: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondex ("t_roeg_detp", "t_roeg_detp", k_mondex, Void, Void, null_integer, null_integer, null_integer, null_integer, 3, Void, k_real, Void, Void, k_company, '0', k_mondex, k_risk, 1166)
		end

	t_roeg_dibp: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondex ("t_roeg_dibp", "t_roeg_dibp", k_mondex, Void, Void, null_integer, null_integer, null_integer, null_integer, 3, Void, k_real, Void, Void, k_company, '0', k_mondex, k_risk, 1165)
		end

	t_dval_rssm: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondex ("t_dval_rssm", "t_dval_rssm", k_mondex, Void, Void, null_integer, null_integer, null_integer, null_integer, 3, Void, k_real, Void, Void, k_company, '0', k_mondex, k_risk, 1164)
		end

	t_tval_rstr: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondex ("t_tval_rstr", "t_tval_rstr", k_mondex, Void, Void, null_integer, null_integer, null_integer, null_integer, 3, Void, k_real, Void, Void, k_company, '0', k_mondex, k_risk, 1163)
		end

	t_etop_btop: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondex ("t_etop_btop", "t_etop_btop", k_mondex, Void, Void, null_integer, null_integer, null_integer, null_integer, 3, Void, k_real, Void, Void, k_company, '0', k_mondex, k_risk, 1162)
		end

	t_tval_qlty: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondex ("t_tval_qlty", "t_tval_qlty", k_mondex, Void, Void, null_integer, null_integer, null_integer, null_integer, 3, Void, k_real, Void, Void, k_company, '0', k_mondex, k_risk, 1161)
		end

	t_escs_rssn: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondex ("t_escs_rssn", "t_escs_rssn", k_mondex, Void, Void, null_integer, null_integer, null_integer, null_integer, 3, Void, k_real, Void, Void, k_company, '0', k_mondex, k_risk, 1160)
		end

	t_ival_inrs: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondex ("t_ival_inrs", "t_ival_inrs", k_mondex, Void, Void, null_integer, null_integer, null_integer, null_integer, 3, Void, k_real, Void, Void, k_company, '0', k_mondex, k_risk, 1159)
		end

	t_tval_inrs: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondex ("t_tval_inrs", "t_tval_inrs", k_mondex, Void, Void, null_integer, null_integer, null_integer, null_integer, 3, Void, k_real, Void, Void, k_company, '0', k_mondex, k_risk, 1158)
		end

	t_cqal_calf: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondex ("t_cqal_calf", "t_cqal_calf", k_mondex, Void, Void, null_integer, null_integer, null_integer, null_integer, 3, Void, k_real, Void, Void, k_company, '0', k_mondex, k_risk, 1157)
		end

	t_egro_vern: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondex ("t_egro_vern", "t_egro_vern", k_mondex, Void, Void, null_integer, null_integer, null_integer, null_integer, 3, Void, k_real, Void, Void, k_company, '0', k_mondex, k_risk, 1156)
		end

	t_rs18_st18: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondex ("t_rs18_st18", "t_rs18_st18", k_mondex, Void, Void, null_integer, null_integer, null_integer, null_integer, 3, Void, k_real, Void, Void, k_company, '0', k_mondex, k_risk, 1155)
		end

	t_roeg_epch: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondex ("t_roeg_epch", "t_roeg_epch", k_mondex, Void, Void, null_integer, null_integer, null_integer, null_integer, 3, Void, k_real, Void, Void, k_company, '0', k_mondex, k_risk, 1154)
		end

	t_roeg_ibch: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondex ("t_roeg_ibch", "t_roeg_ibch", k_mondex, Void, Void, null_integer, null_integer, null_integer, null_integer, 3, Void, k_real, Void, Void, k_company, '0', k_mondex, k_risk, 1153)
		end

	t_tval_rssm: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondex ("t_tval_rssm", "t_tval_rssm", k_mondex, Void, Void, null_integer, null_integer, null_integer, null_integer, 3, Void, k_real, Void, Void, k_company, '0', k_mondex, k_risk, 1152)
		end

	t_calf_sprs: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondex ("t_calf_sprs", "t_calf_sprs", k_mondex, Void, Void, null_integer, null_integer, null_integer, null_integer, 3, Void, k_real, Void, Void, k_company, '0', k_mondex, k_risk, 1151)
		end

	r_calf_mqal: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondex ("r_calf_mqal", "r_calf_mqal", k_mondex, Void, Void, null_integer, null_integer, null_integer, null_integer, 3, Void, k_real, Void, Void, k_company, '0', k_mondex, k_risk, 1150)
		end

	r_btop_ibep: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondex ("r_btop_ibep", "r_btop_ibep", k_mondex, Void, Void, null_integer, null_integer, null_integer, null_integer, 3, Void, k_real, Void, Void, k_company, '0', k_mondex, k_risk, 1149)
		end

	r_btop_ehtp: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondex ("r_btop_ehtp", "r_btop_ehtp", k_mondex, Void, Void, null_integer, null_integer, null_integer, null_integer, 3, Void, k_real, Void, Void, k_company, '0', k_mondex, k_risk, 1148)
		end

	r_etop_ehtp: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondex ("r_etop_ehtp", "r_etop_ehtp", k_mondex, Void, Void, null_integer, null_integer, null_integer, null_integer, 3, Void, k_real, Void, Void, k_company, '0', k_mondex, k_risk, 1147)
		end

	r_roeg_egro: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondex ("r_roeg_egro", "r_roeg_egro", k_mondex, Void, Void, null_integer, null_integer, null_integer, null_integer, 3, Void, k_real, Void, Void, k_company, '0', k_mondex, k_risk, 1146)
		end

	r_roeg_detp: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondex ("r_roeg_detp", "r_roeg_detp", k_mondex, Void, Void, null_integer, null_integer, null_integer, null_integer, 3, Void, k_real, Void, Void, k_company, '0', k_mondex, k_risk, 1145)
		end

	r_roeg_dibp: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondex ("r_roeg_dibp", "r_roeg_dibp", k_mondex, Void, Void, null_integer, null_integer, null_integer, null_integer, 3, Void, k_real, Void, Void, k_company, '0', k_mondex, k_risk, 1144)
		end

	r_dval_rssm: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondex ("r_dval_rssm", "r_dval_rssm", k_mondex, Void, Void, null_integer, null_integer, null_integer, null_integer, 3, Void, k_real, Void, Void, k_company, '0', k_mondex, k_risk, 1143)
		end

	r_tval_rstr: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondex ("r_tval_rstr", "r_tval_rstr", k_mondex, Void, Void, null_integer, null_integer, null_integer, null_integer, 3, Void, k_real, Void, Void, k_company, '0', k_mondex, k_risk, 1142)
		end

	r_etop_btop: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondex ("r_etop_btop", "r_etop_btop", k_mondex, Void, Void, null_integer, null_integer, null_integer, null_integer, 3, Void, k_real, Void, Void, k_company, '0', k_mondex, k_risk, 1141)
		end

	r_tval_qlty: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondex ("r_tval_qlty", "r_tval_qlty", k_mondex, Void, Void, null_integer, null_integer, null_integer, null_integer, 3, Void, k_real, Void, Void, k_company, '0', k_mondex, k_risk, 1140)
		end

	r_escs_rssn: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondex ("r_escs_rssn", "r_escs_rssn", k_mondex, Void, Void, null_integer, null_integer, null_integer, null_integer, 3, Void, k_real, Void, Void, k_company, '0', k_mondex, k_risk, 1139)
		end

	r_ival_inrs: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondex ("r_ival_inrs", "r_ival_inrs", k_mondex, Void, Void, null_integer, null_integer, null_integer, null_integer, 3, Void, k_real, Void, Void, k_company, '0', k_mondex, k_risk, 1138)
		end

	r_tval_inrs: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondex ("r_tval_inrs", "r_tval_inrs", k_mondex, Void, Void, null_integer, null_integer, null_integer, null_integer, 3, Void, k_real, Void, Void, k_company, '0', k_mondex, k_risk, 1137)
		end

	r_cqal_calf: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondex ("r_cqal_calf", "r_cqal_calf", k_mondex, Void, Void, null_integer, null_integer, null_integer, null_integer, 3, Void, k_real, Void, Void, k_company, '0', k_mondex, k_risk, 1136)
		end

	r_egro_vern: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondex ("r_egro_vern", "r_egro_vern", k_mondex, Void, Void, null_integer, null_integer, null_integer, null_integer, 3, Void, k_real, Void, Void, k_company, '0', k_mondex, k_risk, 1135)
		end

	r_rs18_st18: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondex ("r_rs18_st18", "r_rs18_st18", k_mondex, Void, Void, null_integer, null_integer, null_integer, null_integer, 3, Void, k_real, Void, Void, k_company, '0', k_mondex, k_risk, 1134)
		end

	r_roeg_epch: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondex ("r_roeg_epch", "r_roeg_epch", k_mondex, Void, Void, null_integer, null_integer, null_integer, null_integer, 3, Void, k_real, Void, Void, k_company, '0', k_mondex, k_risk, 1133)
		end

	r_roeg_ibch: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondex ("r_roeg_ibch", "r_roeg_ibch", k_mondex, Void, Void, null_integer, null_integer, null_integer, null_integer, 3, Void, k_real, Void, Void, k_company, '0', k_mondex, k_risk, 1132)
		end

	r_tval_rssm: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondex ("r_tval_rssm", "r_tval_rssm", k_mondex, Void, Void, null_integer, null_integer, null_integer, null_integer, 3, Void, k_real, Void, Void, k_company, '0', k_mondex, k_risk, 1131)
		end

	r_calf_sprs: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondex ("r_calf_sprs", "r_calf_sprs", k_mondex, Void, Void, null_integer, null_integer, null_integer, null_integer, 3, Void, k_real, Void, Void, k_company, '0', k_mondex, k_risk, 1130)
		end

	t_egro_flev: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondex ("theta:egro_vs_flev", "t_egro_flev", k_mondex, Void, Void, null_integer, null_integer, null_integer, null_integer, 3, Void, k_real, Void, Void, k_company, '0', k_mondex, k_risk, 1123)
		end

	r_egro_flev: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondex ("radius:egro_vs_flev", "r_egro_flev", k_mondex, Void, Void, null_integer, null_integer, null_integer, null_integer, 3, Void, k_real, Void, Void, k_company, '0', k_mondex, k_risk, 1122)
		end

	mqal: VARIABLE_DESCRIPTION_RECORD is
			-- Market Quality
		once
			create Result.make_risk ("market_quality", "mqal", "mqal", k_risk, null_integer, null_integer, 2, Void, -3.0, 3.0, missing_double, missing_double, "INDFITU:RS12", Void, Void, ' ', k_monthly_data, "", "", 1087)
		ensure
			not_void: Result /= Void
		end

	rm_spcrsk: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondex ("risk_model_srr_predicted_spc_rsk", "rm_spcrsk", k_mondex, Void, Void, null_integer, null_integer, null_integer, null_integer, 3, k_currency, k_double, Void, Void, k_company, '0', k_mondex, "", 1085)
		end

	r5darm3c: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondex ("rolling_5day_abs_return_3_month_count", "r5darm3c", k_mondex, Void, Void, null_integer, null_integer, null_integer, null_integer, 3, k_currency, k_integer, Void, Void, k_company, '0', k_mondex, "", 1084)
		end

	r5darm1c: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondex ("rolling_5day_abs_return_1_month_count", "r5darm1c", k_mondex, Void, Void, null_integer, null_integer, null_integer, null_integer, 3, k_currency, k_integer, Void, Void, k_company, '0', k_mondex, "", 1083)
		end

	logp: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondex ("ln_price", "logp", k_mondex, Void, Void, null_integer, null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, Void, k_company, '0', k_mondex, "", 1082)
		end

	rtrn3: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondex ("return_relative_days_11_15", "rtrn3", k_mondex, Void, Void, null_integer, null_integer, null_integer, null_integer, 3, k_currency, k_double, Void, Void, k_company, '0', k_mondex, "", 1081)
		end

	rtrn2: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondex ("return_relative_days_6_10", "rtrn2", k_mondex, Void, Void, null_integer, null_integer, null_integer, null_integer, 3, k_currency, k_double, Void, Void, k_company, '0', k_mondex, "", 1080)
		end

	rtrn1: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondex ("return_relative_days_1_5", "rtrn1", k_mondex, Void, Void, null_integer, null_integer, null_integer, null_integer, 3, k_currency, k_double, Void, Void, k_company, '0', k_mondex, "", 1079)
		end

	r5darm3: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondex ("rolling_5day_abs_return_3_month", "r5darm3", k_mondex, Void, Void, null_integer, null_integer, null_integer, null_integer, 3, k_currency, k_double, Void, Void, k_company, '0', k_mondex, "", 1078)
		end

	r5darm1: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondex ("rolling_5day_abs_return_1_month", "r5darm1", k_mondex, Void, Void, null_integer, null_integer, null_integer, null_integer, 3, k_currency, k_double, Void, Void, k_company, '0', k_mondex, "", 1077)
		end

	hdvar: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondex ("hd_variable", "hdvar", k_mondex, Void, Void, null_integer, null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, Void, k_company, '0', k_mondex, "", 1075)
		end

	pshs: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("parent_shares", "pshs", Void, null_integer, null_integer, null_integer, 1, k_share, k_double, Void, k_company, 'X', k_rose_company_date, "1", "", 1073)
		end

	cshs: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("common_shares_outstanding", "cshs", Void, null_integer, null_integer, null_integer, 1, k_share, k_double, Void, k_company, 'X', k_rose_company_date, "1", "", 1072)
		end

	osur: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("capital_surplus_other", "osur", "rter", null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "", "", 1061)
		end

	rc1d: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("doubt_receivable", "rc1d", "rcvb", null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "", "", 1060)
		end

	noic: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("non_operating_income_total", "noic", "noin", null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "", "", 1059)
		end

	upnp: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("unfnd_pastsrv", "upnp", "ldbt", null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "", "", 1058)
		end

	upnc: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("unfnd_vestben", "upnc", "ldbt", null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "", "", 1057)
		end

	tsec: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("trade_security", "tsec", "secu", null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "", "", 1056)
		end

	spbr: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("s_and_p_bond_rating", "spbr", Void, null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'X', k_valdata_signature, "", "", 1055)
		end

	shsy: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("four_qtr_sum_shso", "shsy", Void, null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'X', k_valdata_signature, "", "", 1054)
		end

	shsc: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("shares_rsr_cnvrs", "shsc", Void, null_integer, null_integer, null_integer, 2, k_share, k_double, Void, k_company, 'X', k_valdata_signature, "", "", 1053)
		end

	sgen: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("sales_general", "sgen", "sale", null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "", "", 1052)
		end

	scmq: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("fix", "scmq", Void, null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "", "", 1051)
		end

	pnob: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("pension_obligation", "pnob", k_asst, null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "", "", 1050)
		end

	pfdr: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("preferred_rdm_value", "pfdr", Void, null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "", "", 1049)
		end

	pfd: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("preferred_liq_value", "pfd", Void, null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "", "", 1048)
		end

	otut: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("other_ut_sales", "otut", "sale", null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "", "", 1047)
		end

	oink: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("other_income_bank", "oink", "sale", null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "", "", 1046)
		end

	odiv: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("other_dividends", "odiv", Void, null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "", "", 1045)
		end

	nupl: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("plant_nu_util", "nupl", "nplt", null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "", "", 1044)
		end

	lfor: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("loans_foreign", "lfor", "llon", null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "", "", 1043)
		end

	isbr: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("short_borrowing", "isbr", "dbcl", null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "", "", 1042)
		end

	inbk: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("interest_bank", "inbk", "oper", null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "", "", 1041)
		end

	iius: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("interest_treasury", "iius", "sale", null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "", "", 1040)
		end

	iiot: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("interest_other", "iiot", "sale", null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "", "", 1039)
		end

	iiln: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("interest_loans", "iiln", "sale", null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "", "", 1038)
		end

	iilc: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("interest_local", "iilc", "sale", null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "", "", 1037)
		end

	gsut: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("gas_and_water_utility_sales", "gsut", "sale", null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "", "", 1036)
		end

	gsuc: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("gas_and_water_const", "gsuc", "nplt", null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "", "", 1035)
		end

	gspl: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("plant_gas_utilities", "gspl", "nplt", null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "", "", 1034)
		end

	fxdo: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("ff_xitems_and_disc", "fxdo", k_ff, null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "1", k_flow_of_funds, 1033)
		end

	frsv: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("ff_reserves", "frsv", k_ff, null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "1", k_flow_of_funds, 1032)
		end

	fprv: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("ff_provisions", "fprv", k_ff, null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "1", k_flow_of_funds, 1031)
		end

	fibx: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("ff_income_before_xx", "fibx", k_ff, null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "1", k_flow_of_funds, 1030)
		end

	fftx: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("ff_taxation", "fftx", k_ff, null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "1", k_flow_of_funds, 1029)
		end

	feql: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("ff_equity_in_ls", "feql", k_ff, null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "1", k_flow_of_funds, 1028)
		end

	fdtx: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("ff_deferred_tax", "fdtx", k_ff, null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "1", k_flow_of_funds, 1027)
		end

	erup: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("earnings_update", "erup", Void, null_integer, null_integer, null_integer, 2, k_currency_per_share, k_double, Void, k_company, 'F', k_valdata_signature, "", "", 1026)
		end

	epsd: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("earnings_per_share_diluted", "epsd", Void, null_integer, null_integer, null_integer, 2, k_currency_per_share, k_double, Void, k_company, 'F', k_valdata_signature, "", "", 1025)
		end

	eps2: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("eps_diluted_exex", "eps2", Void, null_integer, null_integer, null_integer, 2, k_currency_per_share, k_double, Void, k_company, 'F', k_valdata_signature, "", "", 1024)
		end

	eps1: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("eps_prim_exex", "eps1", Void, null_integer, null_integer, null_integer, 2, k_currency_per_share, k_double, Void, k_company, 'F', k_valdata_signature, "", "", 1023)
		end

	elut: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("electric_utility_sales", "elut", "sale", null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "", "", 1022)
		end

	elpl: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("plant_electric_utility", "elpl", "nplt", null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "", "", 1021)
		end

	dqss: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("delta_share_surplus", "dqss", Void, null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "", "", 1020)
		end

	dile: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("earnings_dilution", "dile", Void, null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "", "", 1019)
		end

	dfot: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("other_def_chg", "dfot", "chdf", null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "", "", 1018)
		end

	dfex: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("def_ex_loss", "dfex", "chdf", null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "", "", 1017)
		end

	crat: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("current_trnsrate", "crat", Void, null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'X', k_valdata_signature, "", "", 1016)
		end

	bkre: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("bank_real_estate", "bkre", "nplt", null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "", "", 1015)
		end

	auop: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("audit_info", "auop", Void, null_integer, null_integer, null_integer, 2, Void, k_double, Void, k_company, 'X', k_valdata_signature, "", "", 1014)
		end

	adte: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("announcement_date", "adte", Void, null_integer, null_integer, null_integer, 2, Void, k_double, Void, k_company, 'X', k_valdata_signature, "", "", 1012)
		end

	adj: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("adjustment_by_ex_date", "adj", Void, null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'X', k_valdata_signature, "", "", 1011)
		end

	acce: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("acct_ch_cm_ef", "acce", k_asst, null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "", "", 1010)
		end

	vssl: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("vessels_and_vehicles", "vssl", "dpas", null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "", "", 1009)
		end

	utoj: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("c_sec_japan", "utoj", Void, null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'X', k_valdata_signature, "", "", 1008)
		end

	urrv: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("unap_rter_rvr", "urrv", "urer", null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "", "", 1007)
		end

	urez: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("unappropriated_retained_earnings_long", "urez", "urer", null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "", "", 1006)
		end

	urer: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("unappropriated_retained_earnings", "urer", Void, null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "", "", 1005)
		end

	ure2: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("unap_rtern", "ure2", "otsr", null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "", "", 1004)
		end

	unin: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("unearned_income", "unin", "clio", null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "", "", 1003)
		end

	txef: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("tax_effect", "txef", Void, null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'X', k_valdata_signature, "", "", 1002)
		end

	trst: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("due_trust", "trst", "sdbt", null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "", "", 1001)
		end

	stbd: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("straight_debt", "stbd", "bond", null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "", "", 1000)
		end

	sptl: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("special_tangible_loss", "sptl", "spex", null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "", "", 999)
		end

	sptg: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("special_tangible_gain", "sptg", "spin", null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "", "", 998)
		end

	spox: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("special_expense_other", "spox", "spex", null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "", "", 997)
		end

	spot: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("special_income_other", "spot", "spin", null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "", "", 996)
		end

	spol: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("special_other_loss", "spol", "spex", null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "", "", 995)
		end

	spog: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("spc_other_gain", "spog", "spin", null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "", "", 994)
		end

	spin: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("special_income", "spin", "ptin", null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "", "", 993)
		end

	spfl: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("special_foreign_exchange_loss", "spfl", "spex", null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "", "", 992)
		end

	spfg: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("special_foreign_exchange_gain", "spfg", "spin", null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "", "", 991)
		end

	spex: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("special_expense", "spex", "ptin", null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "", "", 990)
		end

	spdx: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("special_depreciation", "spdx", "spex", null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "", "", 989)
		end

	slos: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("loss_of_sale_expense", "slos", "loss", null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "", "", 988)
		end

	slex: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("selling_expenses", "slex", "oper", null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "", "", 987)
		end

	shrt: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("short_loans", "shrt", "caco", null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "", "", 986)
		end

	sevr: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("severance_indm", "sevr", "lacr", null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "", "", 985)
		end

	s_tt: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("shares_held", "s_tt", Void, null_integer, null_integer, null_integer, 2, k_share, k_double, Void, k_company, 'X', k_valdata_signature, "", "", 984)
		end

	s_it: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("shares_held_inv_trs", "s_it", Void, null_integer, null_integer, null_integer, 2, k_share, k_double, Void, k_company, 'X', k_valdata_signature, "", "", 983)
		end

	s_in: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("shares_held_by_individuals", "s_in", Void, null_integer, null_integer, null_integer, 2, k_share, k_double, Void, k_company, 'X', k_valdata_signature, "", "", 982)
		end

	s_fo: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("shares_held_by_foreign", "s_fo", Void, null_integer, null_integer, null_integer, 2, k_share, k_double, Void, k_company, 'X', k_valdata_signature, "", "", 981)
		end

	s_fi: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("shares_held_by_financial_insititutions", "s_fi", Void, null_integer, null_integer, null_integer, 2, k_share, k_double, Void, k_company, 'X', k_valdata_signature, "", "", 980)
		end

	s_dr: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("shares_held_by_directors", "s_dr", Void, null_integer, null_integer, null_integer, 2, k_share, k_double, Void, k_company, 'X', k_valdata_signature, "", "", 979)
		end

	s_co: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("shares_held_by_corporations", "s_co", Void, null_integer, null_integer, null_integer, 2, k_share, k_double, Void, k_company, 'X', k_valdata_signature, "", "", 978)
		end

	s_br: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("shares_held_by_brokers", "s_br", Void, null_integer, null_integer, null_integer, 2, k_share, k_double, Void, k_company, 'X', k_valdata_signature, "", "", 977)
		end

	s_bo: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("shares_held_by_others", "s_bo", Void, null_integer, null_integer, null_integer, 2, k_share, k_double, Void, k_company, 'X', k_valdata_signature, "", "", 976)
		end

	rvsr: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("revers_spcrsr", "rvsr", "cibt", null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "", "", 975)
		end

	revx: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("loss_of_reserve_assets", "revx", "loss", null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "", "", 974)
		end

	retu: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("unappr_retained_earnings", "retu", "eqor", null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "", "", 973)
		end

	reta: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("appr_retained_earnings", "reta", "rett", null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "", "", 972)
		end

	redi: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("dividend_interim_received", "redi", Void, null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "", "", 971)
		end

	rddf: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("research_and_development_deferred", "rddf", "char", null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "", "", 970)
		end

	prsr: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("prov_sprsv", "prsr", "cibt", null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "", "", 969)
		end

	prov: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("prov_leg_rsrv", "prov", "otsd", null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "", "", 968)
		end

	prcl: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("principal_in_current_liability", "prcl", "dbcl", null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "", "", 967)
		end

	ppay: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("prepaid_expenses", "ppay", "advx", null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "", "", 966)
		end

	pay3: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("accounts_payable_acquisistion", "pay3", "pay4", null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "", "", 965)
		end

	pay2: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("accounts_payable_other", "pay2", "pay4", null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "", "", 964)
		end

	pay1: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("note_payable_acquisition", "pay1", "pay4", null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "", "", 963)
		end

	parv: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("par_value", "parv", Void, null_integer, null_integer, null_integer, 2, k_currency_per_share, k_double, Void, k_company, 'S', k_valdata_signature, "", "", 962)
		end

	otsz: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("other_surplus_lag", "otsz", "otsr", null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "", "", 961)
		end

	otsr: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("other_surplus", "otsr", Void, null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "", "", 960)
		end

	otsi: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("other_surplus_income", "otsi", "otsr", null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "", "", 959)
		end

	otsd: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("other_surplus_dec", "otsd", "otsr", null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "", "", 958)
		end

	ot_r: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("other_reserves", "ot_r", "rssp", null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "", "", 957)
		end

	os_r: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("overseas_reserves", "os_r", "rssp", null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "", "", 956)
		end

	offb: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("officers_bonus", "offb", "otsd", null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "", "", 955)
		end

	ntrv: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("notes_receivable", "ntrv", "rcvb", null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "", "", 954)
		end

	nsub: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("number_sub", "nsub", Void, null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'X', k_valdata_signature, "", "", 953)
		end

	nren: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("notes_receivable_en", "nren", Void, null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "", "", 952)
		end

	nono: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("nonoperating_income_other", "nono", "noin", null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "", "", 951)
		end

	mssd: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("misc_surplus_dec", "mssd", "otsd", null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "", "", 950)
		end

	lrse: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("legal_reserves_book", "lrse", k_book, null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "", "", 949)
		end

	lrcv: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("receivables_long", "lrcv", "cnio", null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "", "", 948)
		end

	lprx: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("prepay_expense_long", "lprx", "cnio", null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "", "", 947)
		end

	lpay: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("long_payables", "lpay", "ldbo", null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "", "", 946)
		end

	lnot: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("long_notes", "lnot", "ldbo", null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "", "", 945)
		end

	ldbo: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("long_debt_other", "ldbo", "llib", null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "", "", 944)
		end

	ldb1: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("long_debt_other_det", "ldb1", "ldbo", null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "", "", 943)
		end

	lbor: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("long_borrowing", "lbor", "llib", null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "", "", 942)
		end

	issx: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("issuing_expense", "issx", "noex", null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "", "", 941)
		end

	invo: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("investment_other", "invo", "cnio", null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "", "", 940)
		end

	invi: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("investment_income", "invi", "noin", null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "", "", 939)
		end

	intx: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("interest_expense", "intx", "noex", null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "", "", 938)
		end

	inti: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("interest_income", "inti", "invi", null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "", "", 937)
		end

	inst: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("installments", "inst", "clio", null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "", "", 936)
		end

	incl: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("interest_in_current_liability", "incl", "dbcl", null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "", "", 935)
		end

	idrs: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("int_div_reserves", "idrs", "rett", null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "", "", 934)
		end

	fxca: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("forex_conad", "fxca", "curi", null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "", "", 933)
		end

	fx_r: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("foreign_exchange_reserves", "fx_r", "rssp", null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "", "", 932)
		end

	forx: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("foreign_exchange_loss", "forx", "noex", null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "", "", 931)
		end

	forg: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("foreign_exchange_gain", "forg", "noin", null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "", "", 930)
		end

	etxp: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("enterprise_tax_pay", "etxp", "aclb", null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "", "", 929)
		end

	eqip: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("equipment_net", "eqip", "dpas", null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "", "", 928)
		end

	emps: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("employee_savings", "emps", "clio", null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "", "", 927)
		end

	eesb: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("eqrns_ucsb", "eesb", "curi", null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "", "", 926)
		end

	dvir: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("interim_dividends_reserves", "dvir", "urer", null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "", "", 925)
		end

	dvin: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("interim_dividends", "dvin", "urer", null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "", "", 924)
		end

	dpsy: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("dps_year_sen", "dpsy", Void, null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "", "", 923)
		end

	dpsi: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("dps_intim_sen", "dpsi", Void, null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "", "", 922)
		end

	dpse: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("dps_yend_sen", "dpse", Void, null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "", "", 921)
		end

	dp_r: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("dprice_reserves", "dp_r", "rssp", null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "", "", 920)
		end

	divy: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("dividend_at_year_end", "divy", "otsd", null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "", "", 919)
		end

	divi: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("dividend_income", "divi", "invi", null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "", "", 918)
		end

	div6: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("dividend_interim", "div6", Void, null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "", "", 917)
		end

	dg_r: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("deferred_gain_reserves", "dg_r", "rssp", null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "", "", 916)
		end

	db_r: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("bad_debt_reserves", "db_r", "rssp", null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "", "", 915)
		end

	d_and_sd: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("d_and_d_ps_yend", "d&sd", Void, null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "", "", 914)
		end

	curc: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("consolidated_current_income", "curc", Void, null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "", "", 913)
		end

	cur2: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("current_income_2", "cur2", "urer", null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "", "", 912)
		end

	ctxp: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("consolidated_tax_pay", "ctxp", "aclb", null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "", "", 911)
		end

	cprt: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("cap_reserve", "cprt", k_book, null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "", "", 910)
		end

	cost: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("cost_of_sale", "cost", "cott", null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "", "", 909)
		end

	conv: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("convertible_debt", "conv", "bond", null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "", "", 908)
		end

	cnrc: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("consolicated_receivables", "cnrc", "cass", null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "", "", 907)
		end

	cnpy: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("consolidated_payables", "cnpy", "clib", null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "", "", 906)
		end

	cnmi: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("consolidated_minin", "cnmi", "libt", null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "", "", 905)
		end

	cnll: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("consolidated_long_loans", "cnll", "asto", null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "", "", 904)
		end

	cnld: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("consolidated_delta_assets", "cnld", "libt", null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "", "", 903)
		end

	cnlb: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("consolidated_long_borrowing", "cnlb", "llib", null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "", "", 902)
		end

	cnla: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("consolidated_adjustment_liabilities", "cnla", "libt", null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "", "", 901)
		end

	cnio: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("consolidated_investment_other", "cnio", "asto", null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "", "", 900)
		end

	cndl: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("consolidated_delta_liabilities", "cndl", k_asst, null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "", "", 899)
		end

	cnad: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("consolidated_adjustment_assets", "cnad", k_asst, null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "", "", 898)
		end

	capp: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("cap_over_par", "capp", Void, null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "", "", 897)
		end

	borr: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("short_term_borrowing", "borr", "sdbt", null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "", "", 896)
		end

	bons: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("accrued_bonus", "bons", "aclb", null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "", "", 895)
		end

	bokc: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("consolidated_equity", "bokc", Void, null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "", "", 894)
		end

	bo_r: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("bonus_dprc_reserves", "bo_r", "rssp", null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "", "", 893)
		end

	bl8: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("bad_debt_bl8", "bl8", Void, null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "", "", 892)
		end

	amcd: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("amort_cdiff", "amcd", "curi", null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "", "", 891)
		end

	alow: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("allowances", "alow", "cott", null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "", "", 890)
		end

	al7: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("all_debt_al", "al7", Void, null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "", "", 889)
		end

	al1: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("receivable_al1", "al1", Void, null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "", "", 888)
		end

	advx: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("advance_payments_and_prepaid_expenses", "advx", "caso", null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "", "", 887)
		end

	advn: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("customer_advance", "advn", "clio", null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "", "", 886)
		end

	adpy: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("advance_payments", "adpy", "advx", null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "", "", 885)
		end

	acxo: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("accrued_liability_other", "acxo", "aclb", null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "", "", 884)
		end

	acrv: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("accounts_receivable", "acrv", "rcvb", null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "", "", 883)
		end

	acru: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("accrued_income", "acru", "caco", null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "", "", 882)
		end

	acro: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("other_receivables", "acro", "caco", null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "", "", 881)
		end

	acex: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("accrued_expense", "acex", "clio", null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "", "", 880)
		end

	ld1y: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("long_term_debt_due_1_year", "ld1y", "dbcl", null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", "", 879)
		end

	dps: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("dividend_per_share", "dps", Void, null_integer, null_integer, null_integer, 1, k_currency_per_share, k_double, Void, k_company, 'F', k_valdata_signature, "1", "", 814)
		end

	cdiv: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("common_dividends", "cdiv", Void, null_integer, null_integer, null_integer, 1, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "1", "", 813)
		end

	ibes_num_fy3: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_ibes ("ibes_number_of_estimates_fy3", "ibes_num_fy3", Void, k_double, k_rose_company_date, k_ibes, 810)
		end

	ibes_num_fy2: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_ibes ("ibes_number_of_estimates_fy2", "ibes_num_fy2", Void, k_double, k_rose_company_date, k_ibes, 809)
		end

	ibes_num_fy1: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_ibes ("ibes_number_of_estimates_fy1", "ibes_num_fy1", Void, k_double, k_rose_company_date, k_ibes, 807)
		end

	mipcx: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("monthly_industry_pctg_10", "mipcx", Void, k_mondat, null_integer, null_integer, null_integer, null_integer, 2, Void, k_integer, Void, Void, k_company, ' ', k_monthly_data, "", k_mondat, 785)
		end

	mipc9: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("monthly_industry_pctg_9", "mipc9", Void, k_mondat, null_integer, null_integer, null_integer, null_integer, 2, Void, k_integer, Void, Void, k_company, ' ', k_monthly_data, "", k_mondat, 784)
		end

	mipc8: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("monthly_industry_pctg_8", "mipc8", Void, k_mondat, null_integer, null_integer, null_integer, null_integer, 2, Void, k_integer, Void, Void, k_company, ' ', k_monthly_data, "", k_mondat, 783)
		end

	mipc7: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("monthly_industry_pctg_7", "mipc7", Void, k_mondat, null_integer, null_integer, null_integer, null_integer, 2, Void, k_integer, Void, Void, k_company, ' ', k_monthly_data, "", k_mondat, 782)
		end

	mipc6: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("monthly_industry_pctg_6", "mipc6", Void, k_mondat, null_integer, null_integer, null_integer, null_integer, 2, Void, k_integer, Void, Void, k_company, ' ', k_monthly_data, "", k_mondat, 781)
		end

	mipc5: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("monthly_industry_pctg_5", "mipc5", Void, k_mondat, null_integer, null_integer, null_integer, null_integer, 2, Void, k_integer, Void, Void, k_company, ' ', k_monthly_data, "", k_mondat, 780)
		end

	mipc4: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("monthly_industry_pctg_4", "mipc4", Void, k_mondat, null_integer, null_integer, null_integer, null_integer, 2, Void, k_integer, Void, Void, k_company, ' ', k_monthly_data, "", k_mondat, 779)
		end

	mipc3: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("monthly_industry_pctg_3", "mipc3", Void, k_mondat, null_integer, null_integer, null_integer, null_integer, 2, Void, k_integer, Void, Void, k_company, ' ', k_monthly_data, "", k_mondat, 778)
		end

	mipc2: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("monthly_industry_pctg_2", "mipc2", Void, k_mondat, null_integer, null_integer, null_integer, null_integer, 2, Void, k_integer, Void, Void, k_company, ' ', k_monthly_data, "", k_mondat, 777)
		end

	mipc1: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("monthly_industry_pctg_1", "mipc1", Void, k_mondat, null_integer, null_integer, null_integer, null_integer, 2, Void, k_integer, Void, Void, k_company, ' ', k_monthly_data, "", k_mondat, 776)
		end

	mindx: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("monthly_industry_10", "mindx", Void, k_mondat, null_integer, null_integer, null_integer, null_integer, 2, Void, k_integer, Void, Void, k_company, ' ', k_monthly_data, "", k_mondat, 775)
		end

	mind9: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("monthly_industry_9", "mind9", Void, k_mondat, null_integer, null_integer, null_integer, null_integer, 2, Void, k_integer, Void, Void, k_company, ' ', k_monthly_data, "", k_mondat, 774)
		end

	mind8: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("monthly_industry_8", "mind8", Void, k_mondat, null_integer, null_integer, null_integer, null_integer, 2, Void, k_integer, Void, Void, k_company, ' ', k_monthly_data, "", k_mondat, 773)
		end

	mind7: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("monthly_industry_7", "mind7", Void, k_mondat, null_integer, null_integer, null_integer, null_integer, 2, Void, k_integer, Void, Void, k_company, ' ', k_monthly_data, "", k_mondat, 772)
		end

	mind6: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("monthly_industry_6", "mind6", Void, k_mondat, null_integer, null_integer, null_integer, null_integer, 2, Void, k_integer, Void, Void, k_company, ' ', k_monthly_data, "", k_mondat, 771)
		end

	mind5: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("monthly_industry_5", "mind5", Void, k_mondat, null_integer, null_integer, null_integer, null_integer, 2, Void, k_integer, Void, Void, k_company, ' ', k_monthly_data, "", k_mondat, 770)
		end

	mind4: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("monthly_industry_4", "mind4", Void, k_mondat, null_integer, null_integer, null_integer, null_integer, 2, Void, k_integer, Void, Void, k_company, ' ', k_monthly_data, "", k_mondat, 769)
		end

	mind3: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("monthly_industry_3", "mind3", Void, k_mondat, null_integer, null_integer, null_integer, null_integer, 2, Void, k_integer, Void, Void, k_company, ' ', k_monthly_data, "", k_mondat, 768)
		end

	mind2: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("monthly_industry_2", "mind2", Void, k_mondat, null_integer, null_integer, null_integer, null_integer, 2, Void, k_integer, Void, Void, k_company, ' ', k_monthly_data, "", k_mondat, 767)
		end

	mind1: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("monthly_industry_1", "mind1", Void, k_mondat, null_integer, null_integer, null_integer, null_integer, 2, Void, k_integer, Void, Void, k_company, ' ', k_monthly_data, "", k_mondat, 766)
		end

	xlos: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("non_operating_foreign_exchange_loss", "xlos", "noex", 140, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "", k_profit_and_loss, 764)
		end

	strs: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("stat_reserves_other", "strs", "cprt", 113, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "", k_liabilities, 763)
		end

	rvnr: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("revers_neq_rs", "rvnr", "ptin", 145, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "", "", 762)
		end

	lgre: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("legal_reserve", "lgre", "otca", 114, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "", "", 761)
		end

	hb20: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("retained_earnings_lt_20", "hb20", Void, 156, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "", "", 760)
		end

	divs: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("four_quarter_dividends", "divs", Void, 160, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "", "", 759)
		end

	cnso: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("consolidated_subown", "cnso", k_book, 118, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "", k_liabilities, 758)
		end

	cdct: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("cnsl_crtr_dlt", "cdct", k_asst, 122, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "", "", 757)
		end

	cprs: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("capital_reserves", "cprs", "cprt", 112, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", k_liabilities, 754)
		end

	subs: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("subscriptions", "subs", "otca", 111, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", k_liabilities, 753)
		end

	ltld: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("latent_land", "ltld", k_asst, 78, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", k_assets, 752)
		end

	mondat: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_group ("mondat_variable", k_mondat, Void, 1, Void, Void, Void, k_rose_company_date, "group", 750)
		end

	ibes: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_group ("ibes_variable", k_ibes, Void, 1, Void, Void, Void, k_rose_company_date, "group", 749)
		end

	ldbt_ai: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("longterm_debt_ai", "ldbt_ai", "libt", 2105, 2122, 2145, 1, k_currency, k_double, Void, k_company, ' ', k_valdata_signature, "1", k_liabilities, 679)
		end

	sale_ai: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("sales_net_ai", "sale_ai", "gprf", 2126, 2157, 2193, 1, k_currency, k_double, Void, k_company, ' ', k_valdata_signature, "1", k_profit_and_loss, 676)
		end

	oper_ai: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("operating_income_before_depreciation_ai", "oper_ai", "opad", 2129, 2161, 2197, 1, k_currency, k_double, Void, k_company, ' ', k_valdata_signature, "1", k_profit_and_loss, 675)
		end

	asst_ai: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("total_assets_ai", "asst_ai", k_assets, 2074, 2083, 2096, 1, k_currency, k_double, Void, k_company, ' ', k_valdata_signature, "1", k_assets, 674)
		end

	rs06: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondex ("relative_strength_6_month", "rs06", k_mondex, Void, Void, null_integer, null_integer, null_integer, null_integer, 3, Void, k_double, Void, Void, k_company, ' ', k_mondex, k_risk, 671)
		end

	rs03: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondex ("relative_strength_3_month", "rs03", k_mondex, Void, Void, null_integer, null_integer, null_integer, null_integer, 3, Void, k_double, Void, Void, k_company, ' ', k_mondex, k_risk, 670)
		end

	rich: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("cash_rich_ratio", "rich", "CashRich", k_risk, null_integer, 87, 2, Void, -5, 10, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", k_risk, 669)
		end

	indx: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("industry_segment_x", "indx", k_segmnt, 1010, 1010, 1010, 1, Void, k_double, Void, k_company, ' ', k_valdata_signature, "", k_segment, 667)
		end

	ind9: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("industry_segment_9", "ind9", k_segmnt, 1009, 1009, 1009, 1, Void, k_double, Void, k_company, ' ', k_valdata_signature, "", k_segment, 666)
		end

	ind8: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("industry_segment_8", "ind8", k_segmnt, 1008, 1008, 1008, 1, Void, k_double, Void, k_company, ' ', k_valdata_signature, "", k_segment, 665)
		end

	ind7: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("industry_segment_7", "ind7", k_segmnt, 1007, 1007, 1007, 1, Void, k_double, Void, k_company, ' ', k_valdata_signature, "", k_segment, 664)
		end

	ind6: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("industry_segment_6", "ind6", k_segmnt, 1006, 1006, 1006, 1, Void, k_double, Void, k_company, ' ', k_valdata_signature, "", k_segment, 663)
		end

	ind5: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("industry_segment_5", "ind5", k_segmnt, 1005, 1005, 1005, 1, Void, k_double, Void, k_company, ' ', k_valdata_signature, "", k_segment, 662)
		end

	ind4: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("industry_segment_4", "ind4", k_segmnt, 1004, 1004, 1004, 1, Void, k_double, Void, k_company, ' ', k_valdata_signature, "", k_segment, 661)
		end

	ind3: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("industry_segment_3", "ind3", k_segmnt, 1003, 1003, 1003, 1, Void, k_double, Void, k_company, ' ', k_valdata_signature, "", k_segment, 660)
		end

	ind2: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("industry_segment_2", "ind2", k_segmnt, 1002, 1002, 1002, 1, Void, k_double, Void, k_company, ' ', k_valdata_signature, "", k_segment, 659)
		end

	ind1: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("industry_segment_1", "ind1", k_segmnt, 1001, 1001, 1001, 1, Void, k_double, Void, k_company, ' ', k_valdata_signature, "", k_segment, 658)
		end

	rose_vg: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_computed ("value_growth", "rose_vg", Void, Void, 1, k_double, "(etop+btop+yield)/3", Void, k_company, k_rose_company_date, "", k_computed, 622)
		end

	caln: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondex ("current_alpha_new", "caln", k_mondex, Void, Void, null_integer, null_integer, null_integer, null_integer, 3, Void, k_double, Void, Void, k_company, ' ', k_mondex, "", 620)
		end

	valn: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondex ("fitted_value_new", "valn", k_mondex, Void, Void, null_integer, null_integer, null_integer, null_integer, 3, k_currency_per_share, k_double, Void, Void, k_company, ' ', k_mondex, "", 616)
		end

	vwif: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondex ("valreg_weight_imp_factor", "vwif", k_mondex, Void, Void, null_integer, null_integer, null_integer, null_integer, 3, Void, k_double, Void, Void, k_company, ' ', k_mondex, "", 604)
		end

	vwef: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondex ("valreg_weight_eq_factor", "vwef", k_mondex, Void, Void, null_integer, null_integer, null_integer, null_integer, 3, Void, k_double, Void, Void, k_company, ' ', k_mondex, "", 603)
		end

	seps: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("shares_used_to_calc_eps", "seps", Void, null_integer, null_integer, null_integer, 2, k_share, k_double, Void, k_company, 'X', k_valdata_signature, "1", "shares_data", 599)
		end

	ask: VARIABLE_DESCRIPTION_RECORD is
			--listing level model ask
		once
			create Result.make_model_listing ("ask_price", "ask", 1, k_currency_per_share, k_double, ' ', "MR_27", 592)
			Result.set_parent ("market")
			Result.set_group_name ("market")
			Result.set_target_name ("market")
			Result.set_source ("market")
		end

	bid: VARIABLE_DESCRIPTION_RECORD is
			--listing level model bid
		once
			create Result.make_model_listing ("bid_price", "bid", 1, k_currency_per_share, k_double, ' ', "MR_26", 591)
			Result.set_parent ("market")
			Result.set_group_name ("market")
			Result.set_target_name ("market")
			Result.set_source ("market")
		end

	low: VARIABLE_DESCRIPTION_RECORD is
			--listing level model low
		once
			create Result.make_model_listing ("low_price", "low", 1, k_currency_per_share, k_double, ' ', "MR_25", 590)
			Result.set_parent ("market")
			Result.set_group_name ("market")
			Result.set_target_name ("market")
			Result.set_source ("market")
		end

	high: VARIABLE_DESCRIPTION_RECORD is
			--listing level model high
		once
			create Result.make_model_listing ("high_price", "high", 1, k_currency_per_share, k_double, ' ', "MR_24", 589)
			Result.set_parent ("market")
			Result.set_group_name ("market")
			Result.set_target_name ("market")
			Result.set_source ("market")
		end

	close: VARIABLE_DESCRIPTION_RECORD is
			--listing level model close
		once
			create Result.make_model_listing ("closing_price", "close", 1, k_currency_per_share, k_double, ' ', "MR_23", 588)
			Result.set_parent ("market")
			Result.set_group_name ("market")
			Result.set_target_name ("market")
			Result.set_source ("market")
		end

	b_p: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("book_to_price_index", "b_p", "B_P", k_risk, null_integer, 100, 2, "", -4, 4, missing_double, missing_double, "BTOP_RISK", Void, Void, ' ', k_monthly_data, "", k_risk, 586)
		end

	e_p: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("earnings_to_price_index", "e_p", "E_P", k_risk, null_integer, 99, 2, "", -3, 3, missing_double, missing_double, "ETOP_RISK*.3+IBEP*.3+CTOP*.2+XSTP*.2", Void, "", ' ', k_monthly_data, "", k_risk, 585)
		end

	mf2: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("market_flags_2_temporary", "mf2", "record.mdc.market_flags_2", k_per_shares, null_integer, null_integer, null_integer, null_integer, 2, Void, k_integer, Void, Void, k_company, ' ', k_monthly_data, k_per_share, k_mondat, 584)
		end

	mf1: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("market_flags_1_temporary", "mf1", "record.mdc.market_flags", k_per_shares, null_integer, null_integer, null_integer, null_integer, 2, Void, k_integer, Void, Void, k_company, ' ', k_monthly_data, k_per_share, k_mondat, 583)
		end

	ff2: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("fund_flags_2_temporary", "ff2", "record.mdc.fund_flags_2", k_per_shares, null_integer, null_integer, null_integer, null_integer, 2, Void, k_integer, Void, Void, k_company, ' ', k_monthly_data, k_per_share, k_mondat, 582)
		end

	ff1: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("fund_flags_1_temporary", "ff1", "record.mdc.fund_flags", k_per_shares, null_integer, null_integer, null_integer, null_integer, 2, Void, k_integer, Void, Void, k_company, ' ', k_monthly_data, k_per_share, k_mondat, 581)
		end

	shs_listed: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("shares_listed_to_outstanding", "shs_listed", "record.mds.shs_listed", k_per_shares, null_integer, null_integer, null_integer, null_integer, 2, k_share, k_double, Void, k_1000000, k_company, ' ', k_monthly_data, k_per_share, k_mondat, 580)
		end

	shs_listed_security: VARIABLE_DESCRIPTION_RECORD is
			--security level model shs_listed
		once
			create Result.make_model_security ("shares_listed_to_outstanding", "shs_listed", 2, "", k_double, ' ', "MS_20", 580)
		end

	ror: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("monthly_rate_of_return", "ror", "record.mds.ror", k_per_shares, null_integer, null_integer, null_integer, null_integer, 2, "", k_double, Void, Void, k_company, ' ', k_monthly_data, k_per_share, k_mondat, 579)
		end

	noroperps: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("normalized_operating_income_per_share", "noroperps", "record.mdc.noroper", k_per_shares, null_integer, null_integer, null_integer, null_integer, 2, k_currency_per_share, k_double, Void, Void, k_company, ' ', k_monthly_data, k_per_share, k_mondat, 578)
		end

	operps: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("operating_income_per_share", "operps", "record.mdc.oper", k_per_shares, null_integer, null_integer, null_integer, null_integer, 2, k_currency_per_share, k_double, Void, Void, k_company, ' ', k_monthly_data, k_per_share, k_mondat, 577)
		end

	zts: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("zaitech_income", "zts", "record.mdc.zts", k_per_shares, null_integer, null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_1000000, k_company, ' ', k_monthly_data, k_per_share, k_mondat, 576)
		end

	ibescon: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("internally_consistant_ibes", "ibescon", "record.mdc.ibes_consist", k_per_shares, null_integer, null_integer, null_integer, null_integer, 2, k_currency_per_share, k_double, Void, Void, k_company, ' ', k_monthly_data, k_per_share, k_mondat, 575)
		end

	las: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("latent_equity_per_share", "las", "record.mdc.las", k_per_shares, null_integer, null_integer, null_integer, null_integer, 2, k_currency_per_share, k_double, Void, Void, k_company, ' ', k_monthly_data, k_per_share, k_mondat, 574)
		end

	iadiv: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("indicated_annual_dividend", "iadiv", "record.mdc.iadiv", k_per_shares, null_integer, null_integer, null_integer, null_integer, 2, k_currency_per_share, k_double, Void, Void, k_company, ' ', k_monthly_data, k_per_share, k_mondat, 573)
		end

	ibes_med_ltg: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_ibes ("ibes_median_estimate_ltg", "ibes_med_ltg", Void, k_double, k_rose_company_date, k_ibes, 572)
		end

	ibes_num_ltg: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_ibes ("ibes_number_of_estimates_ltg", "ibes_num_ltg", Void, k_double, k_rose_company_date, k_ibes, 571)
		end

	ibes_fy3: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_ibes ("ibes_earnings_per_share_fy3", "ibes_fy3", "currency/(year*share)", k_double, k_rose_company_date, k_ibes, 570)
		end

	ibes_fy2: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_ibes ("ibes_earnings_per_share_fy2", "ibes_fy2", "currency/(year*share)", k_double, k_rose_company_date, k_ibes, 569)
		end

	ibes_fy1: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_ibes ("ibes_earnings_per_share_fy1", "ibes_fy1", "currency/(year*share)", k_double, k_rose_company_date, k_ibes, 568)
		end

	ibes_eps_fy0: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_ibes ("ibes_earnings_per_share_fy0", "ibes_eps_fy0", "currency/(year*share)", k_double, k_rose_company_date, k_ibes, 567)
		end

	ibde5: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("ibes_5_year_annual_delta", "ibde5", "record.mdc.ibes_de5", k_mondat, null_integer, null_integer, null_integer, null_integer, 2, k_currency_per_share, k_double, Void, Void, k_company, ' ', k_monthly_data, "", k_mondat, 566)
		end

	fshseqv: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("fiscal_shares_equivalent", "fshseqv", "record.mdc.shseqv_fccyymm", "shr_data", null_integer, null_integer, null_integer, null_integer, 2, k_share, k_double, Void, k_1000000, k_company, ' ', k_monthly_data, "", k_mondat, 565)
		end

	fcapt: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("fiscal_capitalization", "fcapt", Void, "misc", 0, 0, 0, null_integer, 2, k_currency, k_double, Void, k_1000000, k_company, ' ', k_monthly_data, "", k_mondat, 564)
		end

	dbto_ai: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("debt_other_longterm_ai", "dbto_ai", "ldbt", 2104, 2121, 2143, 1, k_currency, k_double, Void, k_company, ' ', k_valdata_signature, "1", k_liabilities, 563)
		end

	dbtd_ai: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("debt_debentures_ai", "dbtd_ai", "ldbt", 2098, 0, 2142, 1, k_currency, k_double, Void, k_company, ' ', k_valdata_signature, "1", k_liabilities, 562)
		end

	dbtn_ai: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("debt_notes_ai", "dbtn_ai", "ldbt", 2103, 0, 2141, 1, k_currency, k_double, Void, k_company, ' ', k_valdata_signature, "1", k_liabilities, 561)
		end

	irss_ai: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("insurance_reserves_sundry_ai", "irss_ai", "inrs", 0, 2131, 2154, 1, k_currency, k_double, Void, k_company, ' ', k_valdata_signature, "1", k_liabilities, 560)
		end

	irsr_ai: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("insurance_reserves_reinvest_ai", "irsr_ai", "inrs", 0, 2130, 2153, 1, k_currency, k_double, Void, k_company, ' ', k_valdata_signature, "1", k_liabilities, 559)
		end

	irsu_ai: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("insurance_reserves_un_ai", "irsu_ai", "inrs", 0, 2129, 2152, 1, k_currency, k_double, Void, k_company, ' ', k_valdata_signature, "1", k_liabilities, 558)
		end

	irsc_ai: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("insurance_reserves_casualty_ai", "irsc_ai", "inrs", 0, 2128, 2151, 1, k_currency, k_double, Void, k_company, ' ', k_valdata_signature, "1", k_liabilities, 557)
		end

	irsl_ai: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("insurance_reserves_life_ai", "irsl_ai", "inrs", 0, 2127, 2150, 1, k_currency, k_double, Void, k_company, ' ', k_valdata_signature, "1", k_liabilities, 556)
		end

	spal_ai: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("separate_account_liabilites_ai", "spal_ai", "libt", 0, 2126, 2149, 1, k_currency, k_double, Void, k_company, ' ', k_valdata_signature, "1", k_liabilities, 555)
		end

	reil_ai: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("reinsurance_liability_ai", "reil_ai", "libt", 0, 2125, 2148, 1, k_currency, k_double, Void, k_company, ' ', k_valdata_signature, "1", k_liabilities, 554)
		end

	sull_ai: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("sundry_long_term_liabilities_ai", "sull_ai", "liao", 0, 2119, 2138, 1, k_currency, k_double, Void, k_company, ' ', k_valdata_signature, "1", k_liabilities, 553)
		end

	aclo_ai: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("accrued_liability_long_term_ai", "aclo_ai", "lacr", 2102, 2111, 2130, 1, k_currency, k_double, Void, k_company, ' ', k_valdata_signature, "1", k_liabilities, 552)
		end

	clan_ai: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("sundry_current_liability_ai", "clan_ai", "clio", 2096, 2109, 2128, 1, k_currency, k_double, Void, k_company, ' ', k_valdata_signature, "1", k_liabilities, 551)
		end

	pay4_ai: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("payables_other_ai", "pay4_ai", "clio", 2093, 2107, 2126, 1, k_currency, k_double, Void, k_company, ' ', k_valdata_signature, "1", k_liabilities, 550)
		end

	acpy_ai: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("accounts_payable_trd_ai", "acpy_ai", "tpay", 2090, 2104, 2123, 1, k_currency, k_double, Void, k_company, ' ', k_valdata_signature, "1", k_liabilities, 549)
		end

	nopy_ai: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("note_pay_trade_ai", "nopy_ai", "tpay", 2089, 0, 0, 1, k_currency, k_double, Void, k_company, ' ', k_valdata_signature, "1", k_liabilities, 548)
		end

	shbr_ai: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("fl_short_borrow_ai", "shbr_ai", "clib", 2091, 2102, 2121, 1, k_currency, k_double, Void, k_company, ' ', k_valdata_signature, "1", k_liabilities, 547)
		end

	bdep_ai: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("deposits_banks_ai", "bdep_ai", "clib", 0, 2101, 2120, 1, k_currency, k_double, Void, k_company, ' ', k_valdata_signature, "1", k_liabilities, 546)
		end

	idep_ai: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("deposits_savings_ai", "idep_ai", "clib", 2088, 2100, 2119, 1, k_currency, k_double, Void, k_company, ' ', k_valdata_signature, "1", k_liabilities, 545)
		end

	depo_ai: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("deposits_demand_ai", "depo_ai", "clib", 2087, 2099, 2118, 1, k_currency, k_double, Void, k_company, ' ', k_valdata_signature, "1", k_liabilities, 544)
		end

	gsec_ai: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("government_security_ai", "gsec_ai", "secu", 2048, 2051, 2051, 1, k_currency, k_double, Void, k_company, ' ', k_valdata_signature, "1", k_assets, 543)
		end

	shln_ai: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("fl_short_loads_ai", "shln_ai", "cass", 2043, 2050, 2050, 1, k_currency, k_double, Void, k_company, ' ', k_valdata_signature, "1", k_assets, 542)
		end

	rcvb_ai: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("trade_receivables_ai", "rcvb_ai", "recv", 2042, 2042, 2042, 1, k_currency, k_double, Void, k_company, ' ', k_valdata_signature, "1", k_assets, 541)
		end

	rcvo_ai: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("other_receivables_debt_ai", "rcvo_ai", "recv", 2055, 2043, 2043, 1, k_currency, k_double, Void, k_company, ' ', k_valdata_signature, "1", k_assets, 540)
		end

	cash_ai: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("cash_ai", "cash_ai", "cass", 2041, 2041, 2041, 1, k_currency, k_double, Void, k_company, ' ', k_valdata_signature, "1", k_assets, 539)
		end

	lsec_ai: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("local_government_security_ai", "lsec_ai", "secu", 2049, 2052, 2052, 1, k_currency, k_double, Void, k_company, ' ', k_valdata_signature, "1", k_assets, 538)
		end

	bsec_ai: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("corporate_bond_ai", "bsec_ai", "secu", 2050, 2053, 2053, 1, k_currency, k_double, Void, k_company, ' ', k_valdata_signature, "1", k_assets, 537)
		end

	dsec_ai: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("money_market_ai", "dsec_ai", "secu", 0, 2054, 2054, 1, k_currency, k_double, Void, k_company, ' ', k_valdata_signature, "1", k_assets, 536)
		end

	ssec_ai: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("stocks_ai", "ssec_ai", "secu", 2051, 2055, 2055, 1, k_currency, k_double, Void, k_company, ' ', k_valdata_signature, "1", k_assets, 535)
		end

	fsec_ai: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("foreign_security_ai", "fsec_ai", "secu", 2052, 2056, 2056, 1, k_currency, k_double, Void, k_company, ' ', k_valdata_signature, "1", k_assets, 534)
		end

	osec_ai: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("other_security_ai", "osec_ai", "secu", 2053, 2057, 2057, 1, k_currency, k_double, Void, k_company, ' ', k_valdata_signature, "1", k_assets, 533)
		end

	curm_ai: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("sundry_current_assets_ai", "curm_ai", "caco", 2058, 2061, 2061, 1, k_currency, k_double, Void, k_company, ' ', k_valdata_signature, "1", k_assets, 532)
		end

	lcon_ai: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("loans_consumer_ai", "lcon_ai", "llon", 0, 2065, 2072, 1, k_currency, k_double, Void, k_company, ' ', k_valdata_signature, "1", k_assets, 531)
		end

	lmrt_ai: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("loans_mortgage_ai", "lmrt_ai", "llon", 0, 2067, 2074, 1, k_currency, k_double, Void, k_company, ' ', k_valdata_signature, "1", k_assets, 530)
		end

	llea_ai: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("loans_lease_ai", "llea_ai", "llon", 0, 2068, 2075, 1, k_currency, k_double, Void, k_company, ' ', k_valdata_signature, "1", k_assets, 529)
		end

	lopl_ai: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("loans_policy_ai", "lopl_ai", "llon", 0, 2069, 2076, 1, k_currency, k_double, Void, k_company, ' ', k_valdata_signature, "1", k_assets, 528)
		end

	loth_ai: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("loans_other_ai", "loth_ai", "llon", 2070, 2070, 2077, 1, k_currency, k_double, Void, k_company, ' ', k_valdata_signature, "1", k_assets, 527)
		end

	llrs_ai: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("loan_loss_reserves_asset_ai", "llrs_ai", "llon", 0, 2071, 2078, 1, k_currency, k_double, Void, k_company, ' ', k_valdata_signature, "1", k_assets, 526)
		end

	luei_ai: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("loan_unearned_income_ai", "luei_ai", "llon", 0, 2072, 2079, 1, k_currency, k_double, Void, k_company, ' ', k_valdata_signature, "1", k_assets, 525)
		end

	sula_ai: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("assets_sundry_ai", "sula_ai", "aso", 2071, 2087, 2094, 1, k_currency, k_double, Void, k_company, ' ', k_valdata_signature, "1", k_assets, 524)
		end

	spaa_ai: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("separate_account_assets_ai", "spaa_ai", k_asst, 0, 2079, 2086, 1, k_currency, k_double, Void, k_company, ' ', k_valdata_signature, "1", k_assets, 523)
		end

	reia_ai: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("reinsurance_assets_ai", "reia_ai", k_asst, 0, 2078, 2085, 1, k_currency, k_double, Void, k_company, ' ', k_valdata_signature, "1", k_assets, 522)
		end

	sccy_ai: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("security_custody_ai", "sccy_ai", k_asst, 0, 2080, 2087, 1, k_currency, k_double, Void, k_company, ' ', k_valdata_signature, "1", k_assets, 521)
		end

	rsg6: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondex ("residual_strength_growth", "rsg6", k_mondex, Void, Void, null_integer, null_integer, null_integer, null_integer, 3, Void, k_double, Void, Void, k_company, ' ', k_mondex, k_mondex, 520)
		end

	rsv6: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondex ("residual_strength_value", "rsv6", k_mondex, Void, Void, null_integer, null_integer, null_integer, null_integer, 3, Void, k_double, Void, Void, k_company, ' ', k_mondex, k_mondex, 519)
		end

	shr_data: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_group ("shares_data", "shr_data", "fiscal", 1, k_share, Void, k_1000000, k_rose_company_date, "", 518)
		end

	per_shares: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_group ("per_share", "per_shares", Void, 1, "", Void, Void, k_rose_company_date, "", 517)
		end

	oemg: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("operating_per_emp_5", "oemg", "Op/EmGr5", k_risk, null_integer, 60, 2, k_currency, -3, 3, 5, 15, "", Void, k_1000000, ' ', k_monthly_data, "", k_risk, 516)
		end

	olfp: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("olf_4", "olfp", "OLF_O4/P", k_risk, null_integer, 58, 2, k_currency_per_share, -5, 5, 0, 0.08, "", Void, Void, ' ', k_monthly_data, "", k_risk, 515)
		end

	xstp: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("er_ex_ex_sp_to_p", "xstp", "ErXXSp/P", k_risk, null_integer, 57, 2, Void, -3, 3, 0, 0.1, "", Void, k_1000000, ' ', k_monthly_data, "", k_risk, 514)
		end

	elfp: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("earnings_change_from_trailing_4_qtr_to_latest", "elfp", "ELF_E4/P", k_risk, null_integer, 56, 2, Void, -5, 5, 0, 0.04, "", Void, Void, ' ', k_monthly_data, "", k_risk, 513)
		end

	ixfn: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("index_flag_narrow", "ixfn", "INDX FLG", k_risk, null_integer, 49, 2, "", legacy_missing_real_32, 10, 0.5, 0.4, "", Void, Void, ' ', k_monthly_data, "", k_risk, 512)
		end

	ed10: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("earnings_yy_variation_10_year", "ed10", "EPS_yyVR", k_risk, null_integer, 48, 2, "", -3, 3, 0.27, 0.4, "", Void, Void, ' ', k_monthly_data, "", k_risk, 511)
		end

	calf: VARIABLE_DESCRIPTION_RECORD is
			--Current alpha
		once
			create Result.make_risk ("current_alpha", "calf", "Curalpha", k_risk, null_integer, 47, 2, "", -5, 5, 0, 0.3, "", Void, Void, ' ', k_monthly_data, "", k_risk, 510)
		end

	talf: VARIABLE_DESCRIPTION_RECORD is
			--Technical alpha
		once
			create Result.make_risk ("technical_alpha", "talf", "TechAlfa", k_risk, null_integer, 46, 2, "", -3, 3, 0, 0.1, "", Void, Void, ' ', k_monthly_data, "", k_risk, 509)
		end

	fbta: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("fundamental_beta", "fbta", "FundBeta", k_risk, null_integer, 45, 2, "", -3, 3, 1, 0.4, "", Void, Void, ' ', k_monthly_data, "", k_risk, 508)
		end

	eg10: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("earnings_growth_10_year", "eg10", "EGROW_10", k_risk, null_integer, 44, 2, "", -3, 3, 4.5, 20, "", Void, Void, ' ', k_monthly_data, "", k_risk, 507)
		end

	rtrn: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("return", "rtrn", "", k_per_shares, null_integer, null_integer, null_integer, null_integer, 2, "", k_double, Void, Void, k_company, ' ', k_monthly_data, "cap_qlag", k_mondat, 506)
		end

	intr_risk: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("interest_sensitivity", "intr_risk", "Intr.Sns", k_risk, null_integer, 93, 2, "", -3, 3, missing_double, missing_double, "RFRS * .4 + BNDS * .6", Void, Void, 'S', k_monthly_data, "", k_risk, 505)
		end

	yld: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("yield", "yld", "yield", k_per_shares, null_integer, null_integer, null_integer, null_integer, 2, "", k_double, Void, Void, k_company, ' ', k_monthly_data, "cap", "datum", 504)
		end

	ccc3: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("comprehensive_comparison_class_3", "ccc3", "CCClass3", k_risk, 0, 63, 3, "", -7, 5, -0.05, 0.18, "", Void, Void, ' ', k_monthly_data, "", k_risk, 503)
		end

	flev: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("financial_leverage_risk_index", "flev", "Leverage", k_risk, null_integer, 104, 2, "", -3, 3, missing_double, missing_double, "DTOA * .4 + BLEV * .4 + PNCV * .2", Void, Void, ' ', k_monthly_data, "", k_risk, 502)
		end

	xprt: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("export_value", "xprt", "Export", k_risk, null_integer, 103, 2, k_currency, legacy_missing_real_32, 10, missing_double, missing_double, "EXPO", Void, k_1000000, ' ', k_monthly_data, "", k_risk, 501)
		end

	qlty: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("quality_growth", "qlty", "Quality", k_risk, null_integer, 102, 3, "", -3, 10, -0.4, 1, "", Void, Void, ' ', k_monthly_data, "", k_risk, 500)
		end

	ervr: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("earnings_variation", "ervr", "Earn Var", k_risk, null_integer, 101, 2, "", -3, 3, missing_double, missing_double, "VERN * .5 + CUTD * .2 + EC10 * .2 + ED10 * .1", Void, Void, ' ', k_monthly_data, "", k_risk, 499)
		end

	propeq_security: VARIABLE_DESCRIPTION_RECORD is
			--security level model propeq
		once
			create Result.make_model_security ("proportion_of_equity", "propeq", 2, "", k_double, ' ', "MS_23", 498)
		end

	propeq: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("proportion_of_equity", "propeq", "record.mds.propeq", k_per_shares, null_integer, null_integer, null_integer, null_integer, 2, "", k_double, Void, Void, k_company, ' ', k_monthly_data, "undefined", k_mondat, 498)
		end

	grow: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("growth", "grow", "Growth", k_risk, null_integer, 98, 2, "", -3, 3, missing_double, missing_double, "AGRO * .3 + PAY5 * -.3 + EGRO * .1 + OGRO * .1 + SGRO * .2", Void, Void, ' ', k_monthly_data, "", k_risk, 497)
		end

	scss: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("success", "scss", "Success", k_risk, null_integer, 97, 2, "", -3, 3, 0, 0.2, "RV60", Void, Void, ' ', k_monthly_data, "", k_risk, 496)
		end

	size: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("size", "size", "Size", k_risk, null_integer, 96, 2, "", -5, 5, missing_double, missing_double, "CAPT * .9 + ASST_RISK * .1", Void, Void, ' ', k_monthly_data, "", k_risk, 495)
		end

	trad: VARIABLE_DESCRIPTION_RECORD is
			-- Trading activity
		once
			create Result.make_risk ("trading_activity", "trad", "Trad.Act", k_risk, null_integer, 95, 2, "", -3, 3, missing_double, missing_double, "STOA", Void, Void, ' ', k_monthly_data, "", k_risk, 494)
		end

	rstr: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("relative_strength", "rstr", "Rel.Str", k_risk, null_integer, 94, 2, "", -3, 3, missing_double, missing_double, "RS18 * .8 + RS12 * .2", Void, Void, ' ', k_monthly_data, "", k_risk, 493)
		end

	lqty: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("liquidity", "lqty", "Liquidty", k_risk, null_integer, 91, 2, "", -3, 3, missing_double, missing_double, "VOLM * .3 + IXFL * .7", Void, Void, ' ', k_monthly_data, "", k_risk, 492)
		end

	spcv: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("specific_variability", "spcv", "Spc.Var", k_risk, null_integer, 90, 3, "", -3, 4, missing_double, missing_double, "HSIG * .7 + CUMR * .3", Void, Void, ' ', k_monthly_data, "", k_risk, 491)
		end

	sysv: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("systematic_variability", "sysv", "Sys.Var", k_risk, null_integer, 89, 2, "", -3, 3, missing_double, missing_double, "HBET", Void, Void, ' ', k_monthly_data, "", k_risk, 490)
		end

	ti18: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("turnover_inv_exponentially_smoothed", "ti18", "TrnInvXs", k_risk, null_integer, 79, 2, "", -5, 5, 0, 0.5, "", Void, Void, ' ', k_monthly_data, "", k_risk, 489)
		end

	tnin: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("monthly_turnover_inv", "tnin", "MnTrnInv", k_risk, null_integer, 78, 2, "", -6, 10, 0, 0.06, "", Void, Void, ' ', k_monthly_data, "", k_risk, 488)
		end

	that: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("period_relative_turnover", "that", "Prd.RlTv", k_risk, null_integer, 77, 2, "", -6, 10, 1, 1, "", Void, Void, ' ', k_monthly_data, "", k_risk, 487)
		end

	turr: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("monthly_relative_turnover", "turr", "RltvTrnv", k_risk, null_integer, 76, 2, "", legacy_missing_real_32, 15, 1, 1, "", Void, Void, ' ', k_monthly_data, "", k_risk, 486)
		end

	turn: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("monthly_turnover", "turn", "MnthTrnv", k_risk, null_integer, 75, 2, "", legacy_missing_real_32, 15, 0.05, 0.1, "", Void, Void, ' ', k_monthly_data, "", k_risk, 485)
		end

	roeg: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("generalized_return_on_equity", "roeg", "GenrlROE", k_risk, null_integer, 85, 2, "", -5, 5, 0.1, 0.2, "", Void, Void, ' ', k_monthly_data, "", k_risk, 484)
		end

	volm: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("volume_desc", "volm", "", k_risk, null_integer, 8, 2, k_share, -5, 5, 6, 4, "", Void, k_1000000, ' ', k_monthly_data, "", k_risk, 483)
		end

	ccc4: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("comprehensive_comparison_class_4", "ccc4", "CCClass4", k_risk, 0, 64, 3, "", -7, 5, -0.05, 0.2, "", Void, Void, ' ', k_monthly_data, "", k_risk, 482)
		end

	basadj: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("base_adjustment", "basadj", "", "misc", null_integer, null_integer, null_integer, null_integer, 2, "", k_double, Void, Void, k_company, ' ', k_monthly_data, "", k_computed, 481)
		end

	edv2: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("earnings_acceleration", "edv2", Void, k_mondat, null_integer, null_integer, null_integer, null_integer, 2, Void, k_integer, Void, Void, k_company, ' ', k_monthly_data, "", k_risk, 480)
		end

	eg_p: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("ibes_growth_to_price", "eg_p", "IBEGRO/P", k_risk, null_integer, 106, 2, "", -3, 3, missing_double, missing_double, "EGRP", Void, Void, 'F', k_monthly_data, "", k_risk, 479)
		end

	dgro: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("dividend_per_share_growth_5_year", "dgro", "DivGrwth", k_risk, null_integer, 86, 2, "", -3, 3, 0.05, 0.12, "", Void, Void, ' ', k_monthly_data, "", k_risk, 478)
		end

	reeg: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("en_ern_genroe", "reeg", "EnErGROE", k_risk, null_integer, 84, 2, "", -5, 5, 0.1, 0.2, "", Void, Void, ' ', k_monthly_data, "", k_risk, 477)
		end

	ros7: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("return_on_equity_7_year_stdev", "ros7", "ROE7yrSD", k_risk, null_integer, 83, 2, "", -5, 5, 0.1, 0.15, "", Void, Void, ' ', k_monthly_data, "", k_risk, 476)
		end

	res7: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("en_eps_roe7", "res7", "EEROE7sd", k_risk, null_integer, 82, 2, "", -5, 5, 0.1, 0.15, "", Void, Void, ' ', k_monthly_data, "", k_risk, 475)
		end

	rraw: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("raw_return_reversal", "rraw", "RawRtrRv", k_risk, null_integer, 81, 2, "", -8, 8, 0, 0.03, "", Void, Void, ' ', k_monthly_data, "", k_risk, 474)
		end

	ceqs: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("class_earnings_quality", "ceqs", "Cl#ErnQl", k_risk, null_integer, 80, 3, "", legacy_missing_real_32, 999, 6.5, 1.5, "", Void, Void, ' ', k_monthly_data, "", k_risk, 473)
		end

	insh: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("institutional_percent_shares_held", "insh", "InstShHl", k_risk, null_integer, 74, 2, "", legacy_missing_real_32, 10, 0, 0.2, "", Void, Void, ' ', k_monthly_data, "", k_risk, 472)
		end

	rrsn: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("return_reversal_similars_normalized", "rrsn", "RtnRvrSN", k_risk, null_integer, 134, 2, "", -3, 3, missing_double, missing_double, "SIMNORM:RREV", Void, Void, ' ', k_monthly_data, "", k_risk, 471)
		end

	xs_ps: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_computed ("export_sales_per_share", "xs_ps", "net_common_income - net_common_income_ex_ex", k_computeds, 2, k_double, "netc-netx", k_1000000, k_company, k_rose_company_date, "", k_computed, 470)
		end

	adjusted_eps_ibes: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_computed ("adjusted_ibes_projected_earnings", "adjusted_eps_ibes", "ibes_earnings_per_share * (cumulative_adjustment / base_adjustment)", k_computeds, 2, k_double, "eps_ibes*(cumadj/basadj)", Void, k_company, k_rose_company_date, "", k_computed, 469)
		end

	ditp: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_computed ("delta_inventory_to_price", "ditp", "delta_inventory / (price * shares_equivalent)", k_computeds, 2, k_double, "dlin/(price*shseqv)", Void, k_company, k_rose_company_date, "", k_computed, 468)
		end

	drtp: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_computed ("delta_receivable_to_price", "drtp", "delta_receivables / (price * shares_equivalent)", k_computeds, 2, k_double, "dlrc/(price*shseqv)", Void, k_company, k_rose_company_date, "", k_computed, 467)
		end

	cflop: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_computed ("cash_flow_to_price", "cflop", "cash_flow / price", k_computeds, 1, k_double, "cflow/price", Void, k_company, k_rose_company_date, "", k_computed, 466)
		end

	shsout_security: VARIABLE_DESCRIPTION_RECORD is
			--security level model shsout
		once
			create Result.make_model_security ("shares_outstanding", "shsout", 1, "", k_double, ' ', "MS_21", 465)
		end

	shsout: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("shares_outstanding", "shsout", "record.mds.shsout", "shr_data", null_integer, null_integer, null_integer, null_integer, 1, k_share, k_double, Void, k_1000000, k_company, ' ', k_monthly_data, "equal", k_mondat, 465)
		end

	cumadj: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("cumulative_adjustment", "cumadj", "record.mds.cumadj", k_per_shares, null_integer, null_integer, null_integer, null_integer, 1, "", k_double, Void, Void, k_company, ' ', k_monthly_data, k_per_share, k_mondat, 464)
		end

	price: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("price", "price", "record.mds.price", k_per_shares, null_integer, null_integer, null_integer, null_integer, 1, k_currency_per_share, k_double, Void, Void, k_company, ' ', k_monthly_data, k_per_share, k_mondat, 463)
		end

	catr: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("current_alpha_transformed", "catr", "CrAlfaTR", k_risk, null_integer, 108, 2, "", -3, 3, missing_double, missing_double, "CALF * 1.0", Void, Void, ' ', k_monthly_data, "", k_risk, 462)
		end

	sprq: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("quarterly_earnings_surprise_to_price", "sprq", "QrtErnSp", k_risk, null_integer, 129, 2, "", -12, 7, 0, 0.0003, "", Void, Void, ' ', k_monthly_data, "", k_risk, 461)
		end

	zarc: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("zacks_recommendation_change", "zarc", "DL.RecCh", k_risk, null_integer, 128, 2, "", -7, 7, 0, 0.0015, "", Void, Void, ' ', k_monthly_data, "", k_risk, 460)
		end

	eapc: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("earnings_alterations", "eapc", "DL.AL_CF", k_risk, null_integer, 126, 2, "currency/(year*share)", -7, 7, 0, 0.0015, "", Void, k_1000000, ' ', k_monthly_data, "", k_risk, 459)
		end

	psfo: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("percent_shares_held_foreign", "psfo", "ShldFrgn", k_risk, 170, 137, 2, "", -8, 23, 8, 4, "", Void, Void, 'X', k_monthly_data, "", k_risk, 458)
		end

	ms1y: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("market_sentiment_year_alpha", "ms1y", "MktSntYr", k_risk, null_integer, 133, 3, "", -7, 7, 0, 0.007, "", Void, Void, ' ', k_monthly_data, "", k_risk, 457)
		end

	ms1m: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("market_sentiment_monthly_alpha", "ms1m", "MktSntMn", k_risk, null_integer, 132, 3, "", -7, 7, 0, 0.004, "", Void, Void, ' ', k_monthly_data, "", k_risk, 456)
		end

	ms1w: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("market_sentiment_wk_alpha", "ms1w", "MktSntWk", k_risk, null_integer, 131, 3, "", -7, 7, 0, 0.002, "", Void, Void, ' ', k_monthly_data, "", k_risk, 455)
		end

	spry: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("year_end_earnings_surprise_to_price", "spry", "YendErSp", k_risk, null_integer, 130, 2, "", -23, 23, 0, 0.0001, "", Void, Void, ' ', k_monthly_data, "", k_risk, 454)
		end

	edif: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("earnings_esitmate_to_consensus_diff", "edif", "DL.ME_IB", k_risk, null_integer, 125, 2, "currency/(year*share)", -7, 7, 0, 0.0006, "", Void, Void, ' ', k_monthly_data, "", k_risk, 453)
		end

	rrev: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("return_reversal", "rrev", "RtrnRvsl", k_risk, null_integer, 124, 2, "", -7, 7, 0, 0.003, "", Void, Void, ' ', k_monthly_data, "", k_risk, 452)
		end

	cavp: VARIABLE_DESCRIPTION_RECORD is
			-- Current value to price
		once
			create Result.make_risk ("current_value", "cavp", "CurVal/P", k_risk, null_integer, 113, 2, "", -7, 7, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", k_risk, 451)
		end

	yiel: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("yield_risk", "yiel", "Yield", k_risk, null_integer, 92, 2, "", -3, 3, missing_double, missing_double, "YILD * .4 + YLD5 * .6", Void, Void, ' ', k_monthly_data, "", k_risk, 450)
		end

	psin: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("percent_shares_held_individual", "psin", "ShldIndv", k_risk, 171, 138, 2, "", -8, 15, 25, 5, "", Void, Void, 'X', k_monthly_data, "", k_risk, 449)
		end

	psdr: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("percent_shares_held_direct", "psdr", "ShldDrct", k_risk, 169, 136, 2, "", -8, 23, 5, 5, "", Void, Void, 'X', k_monthly_data, "", k_risk, 448)
		end

	psco: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("percent_shares_held_corporate", "psco", "ShldCorp", k_risk, 168, 135, 2, "", -8, 15, 10, 5, "", Void, Void, 'X', k_monthly_data, "", k_risk, 447)
		end

	vtop: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_computed ("value_to_price", "vtop", "fitted_value / price", k_computeds, 2, k_double, "fitted_value / price", Void, k_company, k_monthly_data, "", "datum", 446)
		end

	adjusted_cflow: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_computed ("adjusted_cash_flow", "adjusted_cflow", "cash_flow * (cumulative_adjustment / base_adjustment)", k_computeds, 1, k_double, "cflow*(cumadj/basadj)", Void, k_company, k_rose_company_date, "", k_computed, 445)
		end

	adjusted_sps: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_computed ("adjusted_sales_per_share", "adjusted_sps", "sales_per_share * (cumulative_adjustment / base_adjustment)", k_computeds, 1, k_double, "sps*(cumadj/basadj)", Void, k_company, k_rose_company_date, "", k_computed, 444)
		end

	etop_risk: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("earnings_to_price_risk", "etop_risk", "", k_risk, null_integer, 19, 2, "", -6, 6, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", k_risk, 443)
		end

	itop: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_computed ("ibes_projected_earnings_to_price", "itop", "earnings_per_share_ibes / price", k_computeds, 2, k_double, "eps_ibes/price", Void, k_company, k_rose_company_date, "", k_computed, 442)
		end

	eps_ibes: VARIABLE_DESCRIPTION_RECORD is
			-- Forward 12-month eps estimate based on ibes annual estimates.
		once
			create Result.make_mondat ("ibes_earnings_per_share", "eps_ibes", "record.mdc.ibes", k_per_shares, null_integer, null_integer, null_integer, null_integer, 2, k_currency_per_share, k_double, Void, Void, k_company, ' ', k_monthly_data, k_per_share, k_mondat, 441)
		end

	adjusted_value: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_computed ("adjusted_value", "adjusted_value", "fitted_value * (cumulative_adjustment / base_adjustment)", k_computeds, 2, k_double, "valu*(cumadj/basadj)", Void, k_company, k_rose_company_date, "", k_computed, 439)
		end

	adjusted_bps: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_computed ("adjusted_book_per_share", "adjusted_bps", "book_value_per_share * (cumulative_adjustment / base_adjustment)", k_computeds, 2, k_double, "bps*(cumadj/basadj)", Void, k_company, k_rose_company_date, "", k_computed, 438)
		end

	adjusted_ops: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_computed ("adjusted_operating_income", "adjusted_ops", "operating_income_before_depreciation * (cumulative_adjustment / base_adjustment)", k_computeds, 2, k_double, "oper*(cumadj/basadj)", k_1000000, k_company, k_rose_company_date, "", k_computed, 436)
		end

	adjusted_eps: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_computed ("adjusted_earnings_per_share", "adjusted_eps", "earnings_per_share * (cumulative_adjustment / base_adjustment)", k_computeds, 2, k_double, "eps*(cumadj/basadj)", Void, k_company, k_rose_company_date, "", k_computed, 435)
		end

	groe: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_computed ("generalized_roe", "groe", "generalized_return_on_equity", k_computeds, 2, k_double, "", Void, k_company, k_monthly_data, "basval", k_computed, 434)
		end

	fcap: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("dis_fund_flow", "fcap", "", k_computeds, null_integer, null_integer, null_integer, null_integer, 2, "", k_double, Void, Void, k_company, ' ', k_monthly_data, "", k_computed, 433)
		end

	valw: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_computed ("value_weight", "valw", "", k_computeds, 2, k_double, "bps", Void, k_company, k_rose_company_date, "", k_computed, 431)
		end

	fflw: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("fund_flow", "fflw", k_computeds, null_integer, null_integer, null_integer, 2, k_currency, k_double, Void, k_company, ' ', k_valdata_signature, "1", k_computed, 430)
		end

	slrt: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_group ("retention", "slrt", k_computeds, 2, "", Void, Void, k_rose_company_date, "group", 429)
		end

	ccc2: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("comprehensive_comparison_class_2", "ccc2", "CCClass2", k_risk, 0, 62, 3, "", -6, 5, -0.05, 0.15, "", Void, Void, ' ', k_monthly_data, "", k_risk, 426)
		end

	usd_bval: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_computed ("usd_base_value", "usd_bval", "", k_computeds, 2, k_double, "bps", k_1000000, k_company, k_rose_company_date, "", k_computed, 425)
		end

	usd_cap: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_computed ("usd_capitalization", "usd_cap", "", k_computeds, 2, k_double, "cap", k_1000000, k_company, k_rose_company_date, "", k_computed, 424)
		end

	adjusted_price: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_computed ("adjusted_price", "adjusted_price", "price * (cumulative_adjustment / base_adjustment)", k_computeds, 1, k_double, "price*(cumadj/basadj)", Void, k_company, k_rose_company_date, "", k_computed, 423)
		end

	quarter_distrb: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("quarterly_distribution", "quarter_distrb", "", k_computeds, null_integer, null_integer, null_integer, null_integer, 2, k_currency_per_share, k_double, Void, Void, k_company, ' ', k_monthly_data, "", k_computed, 422)
		end

	quarter_yield: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("quarterly_yield", "quarter_yield", "", k_computeds, null_integer, null_integer, null_integer, null_integer, 2, "", k_double, Void, Void, k_company, ' ', k_monthly_data, "", k_computed, 421)
		end

	quarter_return: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("quarterly_return", "quarter_return", "", k_computeds, null_integer, null_integer, null_integer, null_integer, 2, "", k_double, Void, Void, k_company, ' ', k_monthly_data, "", k_computed, 420)
		end

	shseqv_security: VARIABLE_DESCRIPTION_RECORD is
			--security level model shseqv
		once
			create Result.make_model_security ("shares_equivalent", "shseqv", 2, "", k_double, ' ', "MS_22", 419)
		end

	shseqv: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_computed ("shares_equivalent", "shseqv", "shares_outstanding / proportion_of_equity", k_computeds, 2, k_double, "shsout/propeq", k_1000000, k_company, k_rose_company_date, "", k_computed, 419)
		end

	sps: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("sales_per_share", "sps", "record.mdc.sps", k_per_shares, null_integer, null_integer, null_integer, null_integer, 2, k_currency_per_share, k_double, Void, Void, k_company, ' ', k_monthly_data, k_per_share, k_mondat, 418)
		end

	cflow: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("cash_flow", "cflow", "record.mdc.cflow", k_per_shares, null_integer, null_integer, null_integer, null_integer, 2, k_currency_per_share, k_double, Void, Void, k_company, ' ', k_monthly_data, k_per_share, k_mondat, 417)
		end

	qxps: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("quarterly_earnings_surprise_per_share", "qxps", "record.mdc.qxps", k_per_shares, null_integer, null_integer, null_integer, null_integer, 2, "currency/(quarter*share)", k_double, Void, Void, k_company, ' ', k_monthly_data, "", k_mondat, 416)
		end

	qeps: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("quarterly_earnings_per_share", "qeps", "record.mdc.qeps", k_per_shares, null_integer, null_integer, null_integer, null_integer, 2, "currency/(quarter*share)", k_double, Void, Void, k_company, ' ', k_monthly_data, "", k_mondat, 415)
		end

	bps: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("book_value_per_share", "bps", "record.mdc.bps", k_per_shares, null_integer, null_integer, null_integer, null_integer, 1, k_currency_per_share, k_double, Void, Void, k_company, ' ', k_monthly_data, k_per_share, k_mondat, 414)
		end

	cqsn: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("comp_comparative_quality_similars_normalized", "cqsn", "CCQualSN", k_risk, null_integer, 123, 3, "", -6, 3, missing_double, missing_double, "SIMNORM:CQAL", Void, Void, ' ', k_monthly_data, "", k_risk, 413)
		end

	eqab: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("projected_earnings_quality_below", "eqab", "ErnPrQbl", k_risk, null_integer, 122, 3, "", -4, 4, 0.3, 0.55, "", Void, Void, ' ', k_monthly_data, "", k_risk, 412)
		end

	eqaa: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("projected_earnings_quality_above", "eqaa", "ErnPrQab", k_risk, null_integer, 121, 3, "", -4, 4, 0.03, 0.1, "", Void, Void, ' ', k_monthly_data, "", k_risk, 411)
		end

	eqal: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("projected_earnings_quality", "eqal", "ErnPrJQl", k_risk, null_integer, 120, 3, "", -4, 4, 0.3, 0.5, "", Void, Void, ' ', k_monthly_data, "", k_risk, 410)
		end

	ehsn: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("er_h_to_p_sim_norm", "ehsn", "EHTP.SMN", k_risk, null_integer, 119, 3, "", -3, 3, missing_double, missing_double, "SIMNORM:EHTP", Void, Void, ' ', k_monthly_data, "", k_risk, 409)
		end

	epsn: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("earnings_to_price_sim_norm", "epsn", "EtoP.SMN", k_risk, null_integer, 118, 2, "", -3, 3, missing_double, missing_double, "SIMNORM:E_P", Void, Void, ' ', k_monthly_data, "", k_risk, 408)
		end

	flsn: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("financial_leverage_similars_normalized", "flsn", "FiLv.SMN", k_risk, null_integer, 117, 2, "", -3, 3, missing_double, missing_double, "SIMNORM:FLEV", Void, Void, ' ', k_monthly_data, "", k_risk, 407)
		end

	essn: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("earnings_success_similars_normalized", "essn", "ErSc.SMN", k_risk, null_integer, 116, 2, "", -3, 3, missing_double, missing_double, "SIMNORM:ESCS", Void, Void, ' ', k_monthly_data, "", k_risk, 406)
		end

	trsn: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("trading_activity_similars_normalized", "trsn", "TrAc.SMN", k_risk, null_integer, 115, 2, "", -3, 3, missing_double, missing_double, "SIMNORM:TRAD", Void, Void, ' ', k_monthly_data, "", k_risk, 405)
		end

	rssn: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("relative_strength_sim_nor", "rssn", "RsSt.SMN", k_risk, null_integer, 114, 2, "", -3, 3, missing_double, missing_double, "SIMNORM:RSTR", Void, Void, ' ', k_monthly_data, "", k_risk, 404)
		end

	icbp: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("income_book_pdf_to_price", "icbp", "InBkPd/P", k_risk, null_integer, 112, 2, "", -7, 7, 0, 0.2, "", Void, Void, ' ', k_monthly_data, "", k_risk, 403)
		end

	bvpp: VARIABLE_DESCRIPTION_RECORD is
			-- Valuation model book value to price
		once
			create Result.make_risk ("valuation_book_value_to_price", "bvpp", "BkVlPh/P", k_risk, null_integer, 111, 2, "", -7, 7, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", k_risk, 402)
		end

	ehtp: VARIABLE_DESCRIPTION_RECORD is
			-- Projected earnings to price variable
		once
			create Result.make_risk ("projected_earnings_to_price", "ehtp", "PrjErn/P", k_risk, null_integer, 110, 2, "1/year", -4, 4, 0.05, 0.1, "", Void, Void, ' ', k_monthly_data, "", k_risk, 401)
		ensure
			no_void: Result /= Void
		end

	sgma: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("historical_sigma_5_year", "sgma", "HistSgma", k_risk, null_integer, 107, 2, "", -3, 3, 0, 0.1, "HSIG", Void, Void, ' ', k_monthly_data, "", k_risk, 399)
		end

	eps: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("earnings_per_share", "eps", "record.mdc.eps", k_per_shares, null_integer, null_integer, null_integer, null_integer, 2, k_currency_per_share, k_double, Void, Void, k_company, 'F', k_monthly_data, k_per_share, k_mondat, 398)
		end

	valu: VARIABLE_DESCRIPTION_RECORD is
			-- Valuation model fitted value
		once
			create Result.make_mondat ("fitted_value", "valu", "record.mdc.value", k_per_shares, null_integer, null_integer, null_integer, null_integer, 2, k_currency_per_share, k_double, Void, Void, k_company, ' ', k_monthly_data, k_per_share, k_mondat, 397)
		end

	escs: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("earnings_success", "escs", "EPS Scss", k_risk, null_integer, 105, 2, "", -3, 3, missing_double, missing_double, "EGRO * .6 + EG10 * .3 + DELE * .1", Void, Void, ' ', k_monthly_data, "", k_risk, 396)
		end

	ptri: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("participant_rights", "ptri", k_book, 0, 139, 162, 1, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", k_liabilities, 395)
		end

	ptin: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("pretax_income", "ptin", "cibt", 147, 176, 214, 1, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "1", k_profit_and_loss, 394)
		end

	prvo: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("provisional_other", "prvo", "liao", 108, 116, 135, 1, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", k_liabilities, 393)
		end

	prnr: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("preferred_not_redeemable", "prnr", "pfdc", 0, 135, 158, 1, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", k_liabilities, 392)
		end

	pric: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("spcs_stock_price", "pric", "data_shr", 165, 0, 255, 1, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", "per_share_data", 391)
		end

	prfr: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("preferred_redeemable", "prfr", "pfdc", 0, 134, 157, 1, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", k_liabilities, 390)
		end

	pplb: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("provisional_pension_liability", "pplb", "liao", 101, 114, 133, 1, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", k_liabilities, 389)
		end

	pnun: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("pensions_under", "pnun", "ldbt", 0, 0, 179, 2, k_currency, k_double, Void, k_company, ' ', k_valdata_signature, "1", "liabilities_off_balance_sheet", 388)
		end

	pnov: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("pensions_over", "pnov", "ldbt", 0, 0, 180, 2, k_currency, k_double, Void, k_company, ' ', k_valdata_signature, "1", "liabilities_off_balance_sheet", 387)
		end

	plto: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("plant_other_net", "plto", "dpas", 62, 63, 70, 1, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", k_assets, 386)
		end

	pinv: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("property_development", "pinv", "cass", 0, 44, 44, 1, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", k_assets, 385)
		end

	pfdx: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("preferred_liquid_excess", "pfdx", "pfdc", 0, 0, 186, 2, k_currency, k_double, Void, k_company, ' ', k_valdata_signature, "1", "liabilities_off_balance_sheet", 384)
		end

	pfdc: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("preferred_stock_convertible", "pfdc", "eqts", 0, 136, 159, 1, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", k_liabilities, 383)
		end

	pdva: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("preferred_dividends_arrear", "pdva", "pfdc", 0, 0, 185, 1, k_currency, k_double, Void, k_company, ' ', k_valdata_signature, "1", "liabilities_off_balance_sheet", 382)
		end

	pdiv: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("preferred_dividends", "pdiv", "netc", 0, 186, 224, 1, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "1", k_profit_and_loss, 381)
		end

	pay4: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("payables_other", "pay4", "clio", 93, 107, 126, 1, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", k_liabilities, 380)
		end

	otin: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("other_inventory", "otin", "invn", 47, 48, 48, 1, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", k_assets, 379)
		end

	otca: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("other_shr_cap_book", "otca", k_book, 115, 138, 161, 1, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", k_liabilities, 378)
		end

	osec: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("other_security", "osec", "secu", 53, 57, 57, 1, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", k_assets, 377)
		end

	orca: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("ordinary_capt_book", "orca", k_book, 110, 137, 160, 1, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", k_liabilities, 376)
		end

	oper: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("operating_income_before_depreciation", "oper", "opad", 129, 161, 199, 1, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "1", k_profit_and_loss, 375)
		end

	opef: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("foreign_operating_expenses", "opef", "p_and_l_adj", 0, 0, 236, 2, k_currency, k_double, Void, k_company, ' ', k_valdata_signature, "1", "profit_and_loss_adjustments", 374)
		end

	opad: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("operating_income_after_depreciation", "opad", "orin", 132, 165, 203, 1, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "1", k_profit_and_loss, 373)
		end

	olfl: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("operating_current_anlz", "olfl", "p_and_l_adj", 162, 0, 233, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "1", "profit_and_loss_adjustments", 372)
		end

	nrop: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("normalized_operating_earnings", "nrop", "p_and_l_adj", 157, 189, 230, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "1", "profit_and_loss_adjustments", 371)
		end

	nrec: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("exceptional_items", "nrec", "noin", 143, 170, 208, 1, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "1", k_profit_and_loss, 370)
		end

	nrds: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("notes_receivable_ds", "nrds", "recv", 82, 0, 0, 1, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", "assets_off_balance_sheet", 369)
		end

	nplt: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("plant_net", "nplt", "alfx", 65, 64, 71, 1, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", k_assets, 368)
		end

	noxo: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("non_operating_expense_other", "noxo", "noex", 141, 0, 0, 1, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "1", k_profit_and_loss, 367)
		end

	nopy: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("note_pay_trade", "nopy", "tpay", 89, 0, 0, 1, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", k_liabilities, 366)
		end

	noin: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("non_operating_inc_exp", "noin", "orin", 142, 173, 211, 1, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "1", k_profit_and_loss, 365)
		end

	nixo: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("non_operating_in_x_ot", "nixo", "noin", 136, 172, 210, 1, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "1", k_profit_and_loss, 364)
		end

	nitm: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("net_items", "nitm", "incb", 0, 181, 219, 1, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "1", k_profit_and_loss, 363)
		end

	netx: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("net_income_excluding_ex_ex", "netx", "p_and_l", 154, 188, 225, 1, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "1", k_profit_and_loss, 362)
		end

	neti: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("net_income", "neti", "netc", 152, 185, 227, 1, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "1", k_profit_and_loss, 361)
		end

	netc: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("net_common_income", "netc", "p_and_l", 153, 187, 226, 1, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "1", k_profit_and_loss, 360)
		end

	neqr: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("non_equity_reserves", "neqr", "libt", 0, 132, 155, 1, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", k_liabilities, 359)
		end

	narn: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("natural_resources_net", "narn", "nplt", 0, 0, 67, 1, k_currency, k_double, Void, k_company, ' ', k_valdata_signature, "1", k_assets, 358)
		end

	mrch: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("finished_goods", "mrch", "invn", 44, 47, 47, 1, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", k_assets, 357)
		end

	mksa: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("marketable_security_adj", "mksa", "cass", 83, 0, 102, 1, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", "assets_off_balance_sheet", 356)
		end

	mini: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("minority_interest", "mini", "incb", 0, 180, 218, 1, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "1", k_profit_and_loss, 355)
		end

	minb: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("minority_interest_bs", "minb", "libt", 0, 124, 147, 1, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", k_liabilities, 354)
		end

	luei: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("loan_unearned_income", "luei", "llon", 0, 72, 79, 1, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", k_assets, 353)
		end

	lteq: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("latent_equity", "lteq", k_asst, 77, 0, 0, 2, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", "assets_off_balance_sheet", 352)
		end

	lsec: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("local_gvernment_security", "lsec", "secu", 49, 52, 52, 1, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", k_assets, 351)
		end

	loth: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("loans_other", "loth", "llon", 70, 70, 77, 1, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", k_assets, 350)
		end

	loss: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("loss_on_assets", "loss", "noex", 138, 0, 0, 1, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "1", k_profit_and_loss, 349)
		end

	lopl: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("loans_policy", "lopl", "llon", 0, 69, 76, 1, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", k_assets, 348)
		end

	lmrt: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("loans_mortgage", "lmrt", "llon", 0, 67, 74, 1, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", k_assets, 347)
		end

	llrs: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("loan_loss_reserve_asset", "llrs", "llon", 0, 71, 78, 1, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", k_assets, 346)
		end

	llos: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("loan_losses", "llos", "noex", 139, 0, 0, 1, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "1", k_profit_and_loss, 345)
		end

	dbtd: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("debt_debentures", "dbtd", "ldbt", 98, 0, 142, 1, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", k_liabilities, 344)
		end

	japa: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("japanese_book", "japa", "artfcl", 0, 0, 112, 2, k_currency, k_double, Void, k_company, ' ', k_valdata_signature, "1", "artificial", 343)
		end

	ivsb: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("investment_in_subsidiaries_securities", "ivsb", "ivsc", 68, 74, 81, 1, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", k_assets, 342)
		end

	ivns: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("invest_nonsub", "ivns", "ivsc", 69, 75, 82, 1, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", k_assets, 341)
		end

	ivdo: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("invest_doubt", "ivdo", "asto", 72, 0, 0, 1, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", k_assets, 340)
		end

	itxp: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("income_tax_payable", "itxp", "aclb", 94, 106, 125, 1, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", k_liabilities, 339)
		end

	itax: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("income_taxes", "itax", "incb", 150, 179, 217, 1, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "1", k_profit_and_loss, 338)
		end

	isub: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("investment_in_subsidiaries", "isub", k_asst, 0, 76, 83, 1, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", k_assets, 337)
		end

	irsu: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("insurance_reserves_un", "irsu", "inrs", 0, 129, 152, 1, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", k_liabilities, 336)
		end

	irss: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("insurance_reserves_sundry", "irss", "inrs", 0, 131, 154, 1, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", k_liabilities, 335)
		end

	irsr: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("insurance_reserves_reinvest", "irsr", "inrs", 0, 130, 153, 1, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", k_liabilities, 334)
		end

	irsl: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("insurance_reserves_life", "irsl", "inrs", 0, 127, 150, 1, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", k_liabilities, 333)
		end

	irsc: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("insurance_reserves_casualty", "irsc", "inrs", 0, 128, 151, 1, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", k_liabilities, 332)
		end

	iprp: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("invest_property", "iprp", k_asst, 0, 73, 80, 1, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", k_assets, 331)
		end

	ioth: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("inv_and_adv_other", "ioth", k_asst, 0, 77, 84, 1, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", k_assets, 330)
		end

	invn: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("inventories", "invn", "cass", 0, 49, 49, 1, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", k_assets, 329)
		end

	intr: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("intermediation", "intr", k_computeds, 80, 91, 99, 2, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", k_computed, 328)
		end

	into: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("other_intangibles", "into", "intg", 67, 82, 89, 1, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", k_assets, 327)
		end

	incp: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("interest_capitalized", "incp", "noin", 0, 167, 205, 1, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "1", k_profit_and_loss, 326)
		end

	incb: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("income_before_extraordinary_items", "incb", "neti", 151, 182, 220, 1, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "1", k_profit_and_loss, 325)
		end

	inam: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("intangibles_amortized", "inam", "depr", 0, 163, 201, 1, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "1", k_profit_and_loss, 324)
		end

	iinc: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("non_operating_investment_income", "iinc", "noic", 133, 168, 206, 1, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "1", k_profit_and_loss, 323)
		end

	idep: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("deposits_savings", "idep", "clib", 88, 100, 119, 1, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", k_liabilities, 322)
		end

	ibta: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("income_before_tax_ap", "ibta", "ptin", 144, 174, 212, 1, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "1", k_profit_and_loss, 321)
		end

	gwil: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("goodwill", "gwil", "intg", 66, 81, 88, 1, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", k_assets, 320)
		end

	gwaq: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("goodwill_acquired", "gwaq", "misc", 182, 220, 267, 1, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "1", k_miscellaneous, 319)
		end

	gvts: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("sales_to_government", "gvts", "sale", 0, 0, 194, 1, k_currency, k_double, Void, k_company, ' ', k_valdata_signature, "1", k_profit_and_loss, 318)
		end

	gvgr: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("government_grants", "gvgr", "liao", 0, 112, 131, 1, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", k_liabilities, 317)
		end

	gsec: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("government_security", "gsec", "secu", 48, 51, 51, 1, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", k_assets, 316)
		end

	gain: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("gain_on_sale", "gain", "noin", 134, 171, 209, 1, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "1", k_profit_and_loss, 315)
		end

	ftax: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("noncur_taxes", "ftax", "liao", 0, 113, 132, 1, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", k_liabilities, 314)
		end

	fsts: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("ff_stock_sale", "fsts", k_ff, 0, 196, 243, 1, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "1", k_flow_of_funds, 313)
		end

	fstp: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("ff_stock_purchase", "fstp", k_ff, 0, 203, 250, 1, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "1", k_flow_of_funds, 312)
		end

	fslo: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("ff_sales_loss", "fslo", k_ff, 0, 194, 241, 1, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "1", k_flow_of_funds, 311)
		end

	fsec: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("foreign_security", "fsec", "secu", 52, 56, 56, 1, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", k_assets, 310)
		end

	frtm: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("ff_retirements", "frtm", k_ff, 0, 192, 239, 1, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "1", k_flow_of_funds, 309)
		end

	fpls: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("ff_plant_sale", "fpls", k_ff, 0, 195, 242, 1, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "1", k_flow_of_funds, 308)
		end

	foro: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("for_oth", "foro", "artfcl", 0, 0, 116, 2, k_currency, k_double, Void, k_company, ' ', k_valdata_signature, "1", "artificial", 307)
		end

	fldm: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("ff_long_term_debt", "fldm", k_ff, 0, 202, 249, 1, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "1", k_flow_of_funds, 306)
		end

	fldb: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("ff_ldebt_issue", "fldb", k_ff, 0, 198, 245, 1, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "1", k_flow_of_funds, 305)
		end

	fixd: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("fixed_asset_depreciation", "fixd", "depr", 130, 162, 200, 1, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "1", k_profit_and_loss, 304)
		end

	finv: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("ff_invest", "finv", k_ff, 0, 201, 248, 1, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "1", k_flow_of_funds, 303)
		end

	fins: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("ff_invest_sale", "fins", k_ff, 0, 197, 244, 1, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "1", k_flow_of_funds, 302)
		end

	ffdv: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("ff_dividend", "ffdv", k_ff, 0, 199, 246, 1, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "1", k_flow_of_funds, 301)
		end

	fdpr: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("ff_depreciation", "fdpr", k_ff, 0, 193, 240, 1, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "1", k_flow_of_funds, 300)
		end

	fdiv: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("proposed_dividend", "fdiv", "clio", 0, 108, 127, 1, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", k_liabilities, 299)
		end

	fcpx: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("ff_capital_expense", "fcpx", k_ff, 0, 200, 247, 1, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "1", k_flow_of_funds, 298)
		end

	fchg: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("interest_expense_before_tax", "fchg", "ibta", 137, 166, 204, 1, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "1", k_profit_and_loss, 297)
		end

	dbtn: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("debt_notes", "dbtn", "ldbt", 103, 0, 141, 1, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", k_liabilities, 296)
		end

	facq: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("ff_acquisition", "facq", k_ff, 0, 204, 251, 1, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "1", k_flow_of_funds, 295)
		end

	eqor: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("equity_reserves_other_japan", "eqor", k_book, 0, 0, 0, 1, k_currency, k_double, Void, k_company, ' ', k_valdata_signature, "1", k_liabilities, 294)
		end

	eqrt: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("equity_reserves", "eqrt", k_book, 0, 0, 0, 1, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", k_liabilities, 293)
		end

	llib: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("long_term_liabilities", "llib", "libt", 0, 0, 0, 1, k_currency, k_double, Void, k_company, ' ', k_valdata_signature, "1", k_liabilities, 292)
		end

	curi: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("current_income", "curi", "incb", 0, 0, 0, 1, k_currency, k_double, Void, k_company, ' ', k_valdata_signature, "1", k_profit_and_loss, 291)
		end

	tpay: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("trade_payable", "tpay", "clib", 0, 0, 0, 1, k_currency, k_double, Void, k_company, ' ', k_valdata_signature, "1", k_liabilities, 290)
		end

	sdbt: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("short_term_debt", "sdbt", "clib", 0, 0, 0, 1, k_currency, k_double, Void, k_company, ' ', k_valdata_signature, "1", k_liabilities, 289)
		end

	rvnd: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("reserves_non_distributable", "rvnd", "eqrt", 0, 0, 0, 1, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", k_liabilities, 288)
		end

	rvds: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("reserves_distributable", "rvds", "eqrt", 0, 0, 0, 1, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", k_liabilities, 287)
		end

	rssp: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("returns_special", "rssp", "libt", 0, 0, 0, 1, k_currency, k_double, Void, k_company, ' ', k_valdata_signature, "1", k_liabilities, 286)
		end

	rett: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("appropriated_retained_earnings_total", "rett", "eqor", 0, 0, 0, 1, k_currency, k_double, Void, k_company, ' ', k_valdata_signature, "1", k_liabilities, 285)
		end

	recv: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("receivables", "recv", "cass", 0, 0, 0, 1, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", k_assets, 284)
		end

	orin: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("ordinary_income", "orin", "ptin", 0, 0, 0, 1, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "1", k_profit_and_loss, 283)
		end

	noex: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("non_operating_expense_total", "noex", "orin", 0, 0, 0, 1, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "1", k_profit_and_loss, 282)
		end

	llon: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("loan_longterm", "llon", "asto", 0, 0, 0, 1, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", k_assets, 281)
		end

	liao: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("liabilities_other", "liao", "libt", 0, 0, 0, 1, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", k_liabilities, 280)
		end

	lacr: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("long_accruals", "lacr", "llib", 0, 0, 0, 1, k_currency, k_double, Void, k_company, ' ', k_valdata_signature, "1", k_liabilities, 279)
		end

	ivsc: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("investment_securities_per", "ivsc", "asto", 0, 0, 0, 1, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", k_assets, 278)
		end

	intg: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("intangibles", "intg", "alfx", 0, 0, 0, 1, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", k_assets, 277)
		end

	inrs: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("insurance_reserves_total", "inrs", "libt", 0, 0, 0, 1, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", k_liabilities, 276)
		end

	gprf: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("gross_profit", "gprf", "oper", 0, 0, 0, 1, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "1", k_profit_and_loss, 275)
		end

	exdo: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("extraordinary_items_and_discontinued_ops", "exdo", "neti", 0, 0, 0, 1, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "1", k_profit_and_loss, 274)
		end

	dpas: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("depreciated_assets", "dpas", "nplt", 0, 0, 0, 1, k_currency, k_double, Void, k_company, ' ', k_valdata_signature, "1", k_assets, 273)
		end

	clio: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("current_liability_other", "clio", "clib", 0, 0, 0, 1, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", k_liabilities, 272)
		end

	cibt: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("current_income_before_tax", "cibt", "curi", 0, 0, 0, 1, k_currency, k_double, Void, k_company, ' ', k_valdata_signature, "1", k_profit_and_loss, 271)
		end

	char: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("charge_deferred_total", "char", k_asst, 0, 0, 0, 1, k_currency, k_double, Void, k_company, ' ', k_valdata_signature, "1", k_assets, 270)
		end

	caso: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("current_assets_other", "caso", "cass", 0, 0, 0, 1, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", k_assets, 269)
		end

	caco: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("current_assets_other_consolidated", "caco", "caso", 0, 0, 0, 1, k_currency, k_double, Void, k_company, ' ', k_valdata_signature, "1", k_assets, 268)
		end

	bond: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("bonded_debt", "bond", "llib", 0, 0, 0, 1, k_currency, k_double, Void, k_company, ' ', k_valdata_signature, "1", k_liabilities, 267)
		end

	aso: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("asset_other", "aso", k_asst, 0, 0, 0, 1, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", k_assets, 266)
		end

	alfx: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("all_fixed_assets", "alfx", k_asst, 0, 0, 0, 1, k_currency, k_double, Void, k_company, ' ', k_valdata_signature, "1", k_assets, 265)
		end

	aclb: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("accrued_current_liabilities", "aclb", "clib", 0, 0, 0, 1, k_currency, k_double, Void, k_company, ' ', k_valdata_signature, "1", k_liabilities, 264)
		end

	xxfl: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("earnings_ex_ex_current_anlz", "xxfl", "p_and_l_adj", 163, 0, 234, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "1", "profit_and_loss_adjustments", 263)
		end

	xtra: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("extraordinary_items", "xtra", "exdo", 0, 183, 221, 1, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "1", k_profit_and_loss, 262)
		end

	xsal: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("export_sales", "xsal", "sale", 166, 0, 237, 1, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "1", "profit_and_loss_adjustments", 261)
		end

	xgai: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("exchange_gains_ai", "xgai", "noin", 135, 169, 207, 1, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "1", k_profit_and_loss, 260)
		end

	xdtm: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("extraordinary_items_temporary", "xdtm", "exdo", 0, 0, 229, 2, k_currency, k_double, Void, k_company, ' ', k_valdata_signature, "1", "profit_and_loss_adjustments", 259)
		end

	xajp: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("exchange_adjustment", "xajp", "aso", 0, 86, 93, 1, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", k_assets, 258)
		end

	xadl: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("exchange_adjustment_liability", "xadl", "liao", 107, 117, 136, 1, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", k_liabilities, 257)
		end

	wrbd: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("debt_with_warrant", "wrbd", "bond", 100, 0, 0, 1, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", k_liabilities, 256)
		end

	work: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("work_in_progress", "work", "invn", 45, 46, 46, 1, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", k_assets, 255)
		end

	vest: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("gross_investment", "vest", k_asst, 183, 221, 268, 1, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "1", k_miscellaneous, 254)
		end

	urga: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("unrealized_gain", "urga", "rvnd", 0, 145, 170, 1, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", k_liabilities, 253)
		end

	uk: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("uk", "uk", "artfcl", 0, 0, 114, 2, k_currency, k_double, Void, k_company, ' ', k_valdata_signature, "1", "artificial", 252)
		end

	uapp: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("unappropriated_profit", "uapp", "liao", 0, 118, 137, 1, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", k_liabilities, 251)
		end

	uapl: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("unappropriated_loss", "uapl", "aso", 0, 85, 92, 1, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", k_assets, 250)
		end

	uape: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("unappropriated_pr_eq", "uape", "rvds", 117, 143, 168, 1, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", k_liabilities, 249)
		end

	tyla: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("treasury_stock_l_a", "tyla", "aso", 0, 84, 91, 1, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", k_assets, 248)
		end

	tyca: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("treasury_stock_c_a", "tyca", "caso", 57, 60, 60, 1, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", k_assets, 247)
		end

	txrs: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("tax_reg_reserve_asset", "txrs", k_asst, 0, 98, 110, 1, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", "assets_off_balance_sheet", 246)
		end

	txcf: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("tax_loss_carry_forward", "txcf", "asto", 0, 0, 108, 1, k_currency, k_double, Void, k_company, ' ', k_valdata_signature, "1", "assets_off_balance_sheet", 245)
		end

	trsp: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("treasury_stock_preferred", "trsp", "trsl", 0, 0, 164, 1, k_currency, k_double, Void, k_company, ' ', k_valdata_signature, "1", k_liabilities, 244)
		end

	trsl: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("treasury_stock_lb", "trsl", k_book, 0, 140, 165, 1, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", k_liabilities, 243)
		end

	trsk: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("treasury_stock_asset", "trsk", "caco", 0, 154, 183, 1, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", "liabilities_off_balance_sheet", 242)
		end

	trsc: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("treasury_stock_common", "trsc", "trsl", 0, 0, 163, 1, k_currency, k_double, Void, k_company, ' ', k_valdata_signature, "1", k_liabilities, 241)
		end

	taxr: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("deferred_tax_liability", "taxr", "libt", 0, 123, 146, 1, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", k_liabilities, 240)
		end

	surp: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("capital_surplus_book", "surp", "rett", 0, 141, 166, 1, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", k_liabilities, 239)
		end

	supp: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("raw_materials_inventory", "supp", "invn", 46, 45, 45, 1, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", k_assets, 238)
		end

	sull: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("sundry_long_term_liabilities", "sull", "liao", 0, 119, 138, 1, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", k_liabilities, 237)
		end

	sula: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("assets_sundry", "sula", "aso", 71, 87, 94, 1, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", k_assets, 236)
		end

	ssec: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("stocks", "ssec", "secu", 51, 55, 55, 1, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", k_assets, 235)
		end

	sptm: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("special_items_temporary", "sptm", "noin", 0, 0, 228, 2, k_currency, k_double, Void, k_company, ' ', k_valdata_signature, "1", "profit_and_loss_adjustments", 234)
		end

	sprx: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("sp_rate_adj", "sprx", "ldbt", 0, 0, 184, 2, k_currency, k_double, Void, k_company, ' ', k_valdata_signature, "1", "liabilities_off_balance_sheet", 233)
		end

	spcp: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("special_items_history", "spcp", "nplt", 85, 93, 104, 2, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", "balance_sheet_synthesis", 232)
		end

	spal: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("separate_account_liabilities", "spal", "libt", 0, 126, 149, 1, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", k_liabilities, 231)
		end

	spaa: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("separate_account_assets", "spaa", k_asst, 0, 79, 86, 1, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", k_assets, 230)
		end

	btop: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_computed ("book_to_price", "btop", "book_value_per_share / price", k_computeds, 1, k_double, "bps/price", Void, k_company, k_rose_company_date, "", k_mondat, 229)
		end

	fiscal: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_group ("fiscal_report", "fiscal", Void, 1, "", Void, Void, k_rose_company_date, "", 227)
		end

	segmnt: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_group ("segment", k_segmnt, Void, 2, k_currency, Void, k_1000000, k_rose_company_date, "", 225)
		end

	risk: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_group ("risk", "risk", Void, 1, "", Void, Void, k_rose_company_date, "", 224)
		end

	p_and_l_adj: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_group ("profit_and_loss_adj", "p_and_l_adj", "fiscal", 1, k_currency, Void, k_1000000, k_rose_company_date, "", 223)
		end

	p_and_l: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_group (k_profit_and_loss, "p_and_l", "fiscal", 1, k_currency, Void, k_1000000, k_rose_company_date, "", 222)
		end

	data_shr: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_group ("per_share_delta", "data_shr", "fiscal", 1, "", Void, Void, k_rose_company_date, "", 221)
		end

	delta: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_group ("one_year_delta", "delta", "fiscal", 2, "", Void, Void, k_rose_company_date, "", 220)
		end

	misc: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_group (k_miscellaneous, "misc", "fiscal", 1, "", Void, Void, k_rose_company_date, "", 219)
		end

	liabs: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_group (k_liabilities, "liabs", "fiscal", 1, k_currency, Void, k_1000000, k_rose_company_date, "", 218)
		end

	ff: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_group (k_flow_of_funds, k_ff, "fiscal", 1, k_currency, Void, k_1000000, k_rose_company_date, "", 217)
		end

	artfcl: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_group ("artificial", "artfcl", "fiscal", 1, k_currency, Void, k_1000000, k_rose_company_date, "", 216)
		end

	assets: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_group (k_assets, k_assets, "fiscal", 1, k_currency, Void, k_1000000, k_rose_company_date, "", 215)
		end

	etop: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_computed ("earnings_to_price", "etop", "earnings / price", k_computeds, 1, k_double, "eps/price", Void, k_company, k_rose_company_date, "", k_computed, 214)
		end

	acpy: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("accounts_payable_trd", "acpy", "tpay", 90, 104, 123, 1, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", k_liabilities, 213)
		end

	aclo: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("accrued_liability_long_term_int", "aclo", "lacr", 102, 111, 130, 1, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", k_liabilities, 212)
		end

	acdi: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("accrued_expense_deferred_income", "acdi", "clio", 95, 105, 124, 1, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", k_liabilities, 211)
		end

	ccdr: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("comprehensive_comparison_dynamic_roe", "ccdr", "CC.DyROE", k_risk, null_integer, 59, 3, "", -4, 4, 0, 4, "", Void, Void, ' ', k_monthly_data, "", k_risk, 210)
		end

	delta_ibes_etop: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_computed ("delta_ibes_earnings_to_price", "delta_ibes_etop", "ibes_projected_earnings_to_price - earnings_to_price", "misc", 2, k_double, "itop-etop", Void, k_company, k_rose_company_date, "", k_computed, 209)
		end

	ccda: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("comprehensive_comparison_dynamic_alpha", "ccda", "CC.DynAl", k_risk, null_integer, 26, 3, "", -4, 4, 0, 4, "", Void, Void, ' ', k_monthly_data, "", k_risk, 208)
		end

	shex: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("shares_outstanding_expansion", "shex", "ShsoutCh", k_risk, null_integer, 3, 2, "", legacy_missing_real_32, 999, 0, 0.15, "", Void, Void, ' ', k_monthly_data, "", k_risk, 207)
		end

	rv60: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("success_5_year", "rv60", "RELVAL60", k_risk, null_integer, 7, 2, "", -3, 3, 0.05, 0.2, "", Void, Void, ' ', k_monthly_data, "", k_risk, 206)
		end

	rs12: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("relative_strength_1_year", "rs12", "RELSTR12", k_risk, null_integer, 5, 2, "", -3, 3, 0.07, 0.35, "", Void, Void, ' ', k_monthly_data, "", k_risk, 205)
		end

	ec10: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("earnings_variability_10_year", "ec10", "EPS_CVar", k_risk, null_integer, 42, 2, "", -3, 3, 0.5, 0.5, "", Void, Void, ' ', k_monthly_data, "", k_risk, 204)
		end

	egrp: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("ibes_growth_to_price_desc", "egrp", "EGROTOP", k_risk, null_integer, 40, 2, "", -3, 3, 0.02, 0.008, "", Void, Void, ' ', k_monthly_data, "", k_risk, 203)
		end

	pay5: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("payout_5_year", "pay5", "PAYOUT", k_risk, null_integer, 38, 2, "", -3, 3, 0.5, 0.33, "", Void, Void, ' ', k_monthly_data, "", k_risk, 202)
		end

	computeds: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_group ("computed_variables", k_computeds, Void, 1, "", Void, Void, k_rose_company_date, "", 201)
		end

	e2dv: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("earnings_2nd_derivative", "e2dv", "", k_risk, null_integer, 127, 2, "", -7, 7, 0, 0.001, "", Void, Void, ' ', k_monthly_data, "", k_risk, 200)
		end

	cutd: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("dividend_cuts", "cutd", "DVDNDcut", k_risk, null_integer, 35, 2, "", -3, 3, -0.03, 0.1, "", Void, Void, ' ', k_monthly_data, "", k_risk, 199)
		end

	eg_t: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("earnings_growth_tstat", "eg_t", "EGRO (t)", k_risk, null_integer, 34, 2, "", -3, 3, 2, 4.5, "", Void, Void, ' ', k_monthly_data, "", k_risk, 198)
		end

	roeq: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("return_on_equity_5_year", "roeq", "RTR EQ 5", k_risk, null_integer, 33, 2, "", -3, 3, 0.15, 0.15, "", Void, Void, ' ', k_monthly_data, "", k_risk, 197)
		end

	dele: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("earnings_change", "dele", "ERNGS CH", k_risk, null_integer, 32, 2, "currency/(year*share)", -3, 3, 0.06, 0.5, "", Void, Void, ' ', k_monthly_data, "", k_risk, 196)
		end

	sgro: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("sales_growth_5_year", "sgro", "SALE GRO", k_risk, null_integer, 31, 2, "", -3, 3, 0.05, 0.25, "", Void, Void, ' ', k_monthly_data, "", k_risk, 195)
		end

	ogro: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("operating_growth_5_year", "ogro", "OPER GRO", k_risk, null_integer, 30, 2, "", -3, 3, 0.05, 0.25, "", Void, Void, ' ', k_monthly_data, "", k_risk, 194)
		end

	egro: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("earnings_growth_5_year", "egro", "EARN GRO", k_risk, null_integer, 29, 2, "", -3, 3, 0.05, 0.5, "", Void, Void, ' ', k_monthly_data, "", k_risk, 193)
		end

	agro: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("asset_growth_5_year", "agro", "ASST GRO", k_risk, null_integer, 28, 2, "", -3, 3, 0.1, 0.2, "", Void, Void, ' ', k_monthly_data, "", k_risk, 192)
		end

	asst_risk: VARIABLE_DESCRIPTION_RECORD is
			--log of total assets in usd
		once
			create Result.make_risk ("total_assets_risk", "asst_risk", "TOT ASST", k_risk, null_integer, 27, 2, "", -7, 5, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "equal", k_risk, 191)
		end

	optp: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("operating_earnings_to_price", "optp", "OPER/PRC", k_risk, null_integer, 25, 2, "1/year", -3, 3, 0.15, 0.12, "", Void, Void, ' ', k_monthly_data, "", k_risk, 190)
		end

	ctop: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("cash_flow_to_price_desc", "ctop", "CFLO/PRC", k_risk, null_integer, 24, 2, "", -3, 3, 0.07, 0.1, "", Void, Void, ' ', k_monthly_data, "", k_risk, 189)
		end

	stop: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("sales_to_price", "stop", "SALE/PRC", k_risk, null_integer, 23, 2, "", -3, 3, 0.96, 1, "", Void, Void, ' ', k_monthly_data, "", k_risk, 188)
		end

	ibep: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("ibes_earnings_to_price", "ibep", "IBES/PRC", k_risk, null_integer, 22, 2, "1/year", -3, 3, 0.06, 0.1, "", Void, Void, ' ', k_monthly_data, "", k_risk, 187)
		end

	leqp: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("latent_equity_to_price", "leqp", "LTEQ/PRC", k_risk, null_integer, 21, 2, "", -3, 3, 0.02, 0.2, "", Void, Void, ' ', k_monthly_data, "", k_risk, 186)
		end

	btop_risk: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("book_to_price_risk", "btop_risk", "BOOK/PRC", k_risk, null_integer, 20, 2, "", -6, 4, missing_double, missing_double, "", Void, Void, ' ', k_monthly_data, "", k_risk, 185)
		end

	basp: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("basis_cost_to_price", "basp", "BasCst/P", k_risk, null_integer, 73, 2, "", -6, 6, 0.9, 1, "", Void, Void, ' ', k_monthly_data, "", k_risk, 184)
		end

	xitp: VARIABLE_DESCRIPTION_RECORD is
			-- Extraordinary items to price
		once
			create Result.make_risk ("extraordinary_items_to_price", "xitp", "Ex&SpI/P", k_risk, null_integer, 55, 2, "", -22, 22, 0, 0.04, "", Void, Void, ' ', k_monthly_data, "", k_risk, 183)
		ensure
			not_void: Result /= Void
		end

	xxtp: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("ern_ex_extra_to_p", "xxtp", "ERXX/PRC", k_risk, null_integer, 53, 2, "", -3, 3, 0, 0.04, "", Void, Void, ' ', k_monthly_data, "", k_risk, 182)
		end

	roe1: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("return_on_equity", "roe1", "Rtrn Eqt", k_risk, null_integer, 52, 2, "", -4, 4, 0.15, 0.25, "", Void, Void, ' ', k_monthly_data, "", k_risk, 181)
		end

	trbl: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("troubled_company", "trbl", "Troubled", k_risk, null_integer, 51, 2, "", -10, 3, -0.06, 0.4, "", Void, Void, ' ', k_monthly_data, "", k_risk, 180)
		end

	poli: VARIABLE_DESCRIPTION_RECORD is
			-- Political barriers / risk
		once
			create Result.make_risk ("political_bar", "poli", "Politics", k_risk, null_integer, 50, 2, "", legacy_missing_real_32, 10, 0.25, 0.6, "", Void, Void, ' ', k_monthly_data, "", k_risk, 179)
		end

	ccc5: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("comprehensive_comparison_class_5", "ccc5", "CCClass5", k_risk, 0, 65, 3, "", -7, 5, -0.05, 0.3, "", Void, Void, ' ', k_monthly_data, "", k_risk, 178)
		end

	btov: VARIABLE_DESCRIPTION_RECORD is
			--Book to value
		once
			create Result.make_risk ("book_to_value", "btov", "Book/Val", k_risk, null_integer, 72, 2, "", -6, 6, 0.5, 0.4, "", Void, Void, ' ', k_monthly_data, "", k_risk, 177)
		end

	st60: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("share_turnover_5_year", "st60", "Trnovr5Y", k_risk, null_integer, 71, 2, "", legacy_missing_real_32, 5, 0.5, 0.6, "", Void, Void, ' ', k_monthly_data, "", k_risk, 176)
		end

	st18: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("short_term_trading_exponentially_smoothed", "st18", "ShrTrnXs", k_risk, null_integer, 70, 2, "", legacy_missing_real_32, 5, 0.5, 0.69999999999999996, "", Void, Void, ' ', k_monthly_data, "", k_risk, 175)
		end

	frgr: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("fr_sm_xpx_rgro", "frgr", "FrSmRgro", k_risk, null_integer, 69, 2, k_currency, -5, 5, 0.0004, 0.005, "", Void, Void, ' ', k_monthly_data, "", k_risk, 174)
		end

	fvtb: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("fr_sm_x_px_gv_to_b", "fvtb", "FrSmVs/B", k_risk, null_integer, 68, 2, k_currency, -5, 5, 0.0002, 0.01, "", Void, Void, ' ', k_monthly_data, "", k_risk, 173)
		end

	svtb: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("simil_g_vst_to_book", "svtb", "SimVst/B", k_risk, null_integer, 67, 2, "", -3, 3, 0.25, 0.15, "", Void, Void, ' ', k_monthly_data, "", k_risk, 172)
		end

	cqal: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("comp_comparative_quality", "cqal", "CCQualty", k_risk, null_integer, 66, 3, "", -6, 3, -0.1, 0.15, "", Void, Void, ' ', k_monthly_data, "", k_risk, 171)
		end

	dtoa: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("debt_to_assets", "dtoa", "DBT/ASST", k_risk, null_integer, 16, 2, "", legacy_missing_real_32, 3, 0.28, 0.4, "", Void, Void, ' ', k_monthly_data, "", k_risk, 170)
		end

	ixfl: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("index_flag_wide", "ixfl", "INDX FLG", k_risk, null_integer, 15, 2, "", -3, 10, 0.76, 1, "", Void, Void, ' ', k_monthly_data, "", k_risk, 169)
		end

	yld5: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("yield_5_year", "yld5", "YIELD-5Y", k_risk, null_integer, 14, 2, "", -3, 3, 0.04, 0.04, "", Void, Void, ' ', k_monthly_data, "", k_risk, 168)
		end

	yild: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("current_yield", "yild", "YIELD", k_risk, null_integer, 13, 2, "", -3, 3, 0.04, 0.04, "", Void, Void, ' ', k_monthly_data, "", k_risk, 167)
		end

	bnds: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("bond_beta", "bnds", "BONDSENS", k_risk, null_integer, 11, 2, "", -3, 3, -0.05, 1, "", Void, Void, ' ', k_monthly_data, "", k_risk, 166)
		end

	rfrs: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("risk_free_sensitivity", "rfrs", "RFREESNS", k_risk, null_integer, 10, 2, "", -3, 3, 0, 3, "", Void, Void, ' ', k_monthly_data, "", k_risk, 165)
		end

	stoa: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("share_turnover", "stoa", "TURNOVER", k_risk, null_integer, 9, 2, "", legacy_missing_real_32, 4, 0.5, 0.8, "", Void, Void, ' ', k_monthly_data, "", k_risk, 164)
		end

	volu: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("volume", "volu", "record.mds.volume", "shr_data", null_integer, null_integer, null_integer, null_integer, 2, k_share, k_double, Void, k_1000000, k_company, ' ', k_monthly_data, k_per_share, k_mondat, 163)
		end

	volu_security: VARIABLE_DESCRIPTION_RECORD is
			--security level model volu
		once
			create Result.make_model_security ("volume", "volu", 2, "", k_double, ' ', "MS_27", 163)
		end

	rs18: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("relative_strength_18_month", "rs18", "RELSTRxs", k_risk, null_integer, 6, 2, "", -3, 3, 0.06, 0.3, "", Void, Void, ' ', k_monthly_data, "", k_risk, 162)
		end

	cumr: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("cumulative_range", "cumr", "CUMRANGE", k_risk, null_integer, 4, 2, "", -3, 3, 0.35, 0.25, "", Void, Void, ' ', k_monthly_data, "", k_risk, 161)
		end

	opem: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_mondat ("operating_earnings_per_employee", "opem", "", k_computeds, null_integer, null_integer, null_integer, null_integer, 2, "currency/person", k_double, Void, "1", k_company, ' ', k_monthly_data, "", k_computed, 160)
		end

	hsig: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("historical_sigma", "hsig", "HISTSGMA", k_risk, null_integer, 2, 2, "", -5, 5, 0.2, 0.15, "", Void, Void, ' ', k_monthly_data, "", k_risk, 159)
		end

	hbet: VARIABLE_DESCRIPTION_RECORD is
			-- Historical Beta
		once
			create Result.make_risk ("historical_beta", "hbet", "HISTBETA", k_risk, null_integer, 1, 2, "", -3, 5, 1, 0.6, "", Void, Void, ' ', k_monthly_data, "", k_risk, 158)
		end

	ccc1: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("comprehensive_comparison_class_1", "ccc1", "CCClass1", k_risk, 0, 61, 3, "", -6, 5, -0.04, 0.1, "", Void, Void, ' ', k_monthly_data, "", k_risk, 157)
		end

	aetp: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("earnings_to_price_10_year_ave", "aetp", "ETOP_A10", k_risk, null_integer, 43, 2, "1/year", -3, 3, 0.05, 0.1, "", Void, Void, ' ', k_monthly_data, "", k_risk, 156)
		end

	ibg5: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("ibes_growth_5_year", "ibg5", "IBGROW5", k_risk, null_integer, 41, 2, "", -3, 3, 0.33, 0.2, "", Void, Void, ' ', k_monthly_data, "", k_risk, 155)
		end

	expo: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("export_sales_risk", "expo", "XPRT SAL", k_risk, null_integer, 39, 2, "", legacy_missing_real_32, 10, 35, 20, "", Void, k_1000000, ' ', k_monthly_data, "", k_risk, 154)
		end

	empg: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("employee_growth_5_year", "empg", "EmpyGrow", k_risk, null_integer, 37, 2, "", -3, 3, 3, 11, "", Void, Void, ' ', k_monthly_data, "", k_risk, 153)
		end

	vern: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("variable_earnings", "vern", "VAR ERNG", k_risk, null_integer, 36, 2, "currency/(year*share)", -3, 3, 0.5, 0.75, "", Void, k_1000000, ' ', k_monthly_data, "", k_risk, 152)
		end

	sl10: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("sales_segment_x", "sl10", k_segmnt, 30, 30, 30, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "1", k_segment, 151)
		end

	sl09: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("sales_segment_9", "sl09", k_segmnt, 29, 29, 29, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "1", k_segment, 150)
		end

	sl08: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("sales_segment_8", "sl08", k_segmnt, 28, 28, 28, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "1", k_segment, 149)
		end

	sl07: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("sales_segment_7", "sl07", k_segmnt, 27, 27, 27, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "1", k_segment, 148)
		end

	sl06: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("sales_segment_6", "sl06", k_segmnt, 26, 26, 26, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "1", k_segment, 147)
		end

	sl05: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("sales_segment_5", "sl05", k_segmnt, 25, 25, 25, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "1", k_segment, 146)
		end

	sl04: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("sales_segment_4", "sl04", k_segmnt, 24, 24, 24, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "1", k_segment, 145)
		end

	sl03: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("sales_segment_3", "sl03", k_segmnt, 23, 23, 23, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "1", k_segment, 144)
		end

	sl02: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("sales_segment_2", "sl02", k_segmnt, 22, 22, 22, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "1", k_segment, 143)
		end

	sl01: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("sales_segment_1", "sl01", k_segmnt, 21, 21, 21, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "1", k_segment, 142)
		end

	shsr: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("shares_us", "shsr", "shr_data", 0, 0, 254, 2, k_share, k_double, Void, k_company, ' ', k_valdata_signature, "1", "shares_data", 141)
		end

	shs: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("shares", "shs", "shr_data", 0, 0, 253, 1, k_share, k_double, Void, k_company, 'X', k_valdata_signature, "1", "shares_data", 140)
		end

	shln: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("fl_short_loans", "shln", "cass", 43, 50, 50, 1, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", k_assets, 139)
		end

	shbr: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("fl_short_borrow", "shbr", "clib", 91, 102, 121, 1, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", k_liabilities, 138)
		end

	sgax: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("sga_expense", "sgax", "oper", 128, 160, 198, 1, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "1", k_profit_and_loss, 137)
		end

	secu: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("securities_assets", "secu", "caso", 54, 58, 58, 1, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", k_assets, 136)
		end

	sccy: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("security_custody", "sccy", k_asst, 0, 80, 87, 1, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", k_assets, 135)
		end

	salf: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("foreign_sales", "salf", "p_and_l_adj", 0, 0, 235, 2, k_currency, k_double, Void, k_company, ' ', k_valdata_signature, "1", "profit_and_loss_adjustments", 134)
		end

	sale: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("sales_net", "sale", "gprf", 126, 157, 195, 1, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "1", k_profit_and_loss, 133)
		end

	rvlr: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("revaluation_reserves", "rvlr", "rvnd", 0, 144, 169, 1, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", k_liabilities, 132)
		end

	rter: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("retained_earnings", "rter", "rvds", 116, 142, 167, 1, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", k_liabilities, 131)
		end

	rndp: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("research_and_development_history", "rndp", k_asst, 86, 96, 107, 2, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", "balance_sheet_synthesis", 130)
		end

	reil: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("reinsurace_liability", "reil", "libt", 0, 125, 148, 1, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", k_liabilities, 129)
		end

	reia: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("reinsurance_assets", "reia", k_asst, 0, 78, 85, 1, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", k_assets, 128)
		end

	rcvo: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("other_receivables_debt", "rcvo", "recv", 55, 43, 43, 1, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", k_assets, 127)
		end

	rcvb: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("trade_receivables", "rcvb", "recv", 42, 42, 42, 1, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", k_assets, 126)
		end

	rcdb: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("receivables_doubt", "rcdb", "recv", 81, 0, 100, 1, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", "assets_off_balance_sheet", 125)
		end

	rand: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("research_and_development", "rand", "p_and_l_adj", 159, 190, 231, 1, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "1", "profit_and_loss_adjustments", 124)
		end

	pxsl: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("percent_sales_foreign", "pxsl", "misc", 167, 0, 256, 2, k_currency, k_double, Void, k_company, 'X', k_valdata_signature, "1", k_miscellaneous, 123)
		end

	pxai: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("prepaid_expense_and_accrued_income", "pxai", "caso", 56, 59, 59, 1, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", k_assets, 122)
		end

	llea: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("loans_lease", "llea", "llon", 0, 68, 75, 1, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", k_assets, 121)
		end

	dil2: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("book_dilution_2", "dil2", k_book, 0, 0, 188, 2, k_currency, k_double, Void, k_company, ' ', k_valdata_signature, "1", "liabilities_off_balance_sheet", 120)
		end

	adon: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("acc_adjustment_done", "adon", k_book, 0, 0, 190, 1, k_currency, k_double, Void, k_company, ' ', k_valdata_signature, "1", k_miscellaneous, 119)
		end

	adjusted_shseqv: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_computed ("adjusted_shares_equivalent", "adjusted_shseqv", "shares_equivalent  * (cumulative_adjustment / base_adjustment)", k_computeds, 2, k_double, "shseqv*(cumadj/basadj)", k_1000000, k_company, k_rose_company_date, "", k_computed, 118)
		end

	cast: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("current_alpha_st", "cast", "CrAlfaST", k_risk, null_integer, 54, 2, "", -3, 3, 0, 0.2, "", Void, Void, ' ', k_monthly_data, "", k_risk, 117)
		end

	capt: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("capitalization_desc", "capt", "CPTLIZTN", k_risk, null_integer, 12, 2, "", legacy_missing_real_32, 999, 9, 1, "", Void, Void, ' ', k_monthly_data, "", k_risk, 116)
		end

	sig2: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_computed ("sigma_squared", "sig2", "historical_sigma * historical_sigma", k_computeds, 2, k_double, "hsig*hsig", Void, k_company, k_rose_company_date, "", k_computed, 115)
		end

	blev: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("book_leverage", "blev", "BK LEVRG", k_risk, null_integer, 18, 2, "", -3, 3, 2.1, 1.5, "", Void, Void, ' ', k_monthly_data, "", k_risk, 114)
		end

	pncv: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_risk ("probability_of_non_coverage", "pncv", "NON-COVR", k_risk, null_integer, 17, 2, "", -5, 9, 0.02, 0.1, "", Void, Void, ' ', k_monthly_data, "", k_risk, 113)
		end

	cap: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_computed ("capitalization", "cap", "shares_equivalent * price", k_computeds, 1, k_double, "shseqv*price", k_1000000, k_company, k_rose_company_date, "", k_computed, 112)
		end

	ll_r: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("loan_loss_reserve_liability", "ll_r", "rssp", 106, 0, 134, 1, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", k_liabilities, 111)
		end

	lifr: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("lifo_reserve", "lifr", "invn", 0, 0, 101, 1, k_currency, k_double, Void, k_company, ' ', k_valdata_signature, "1", "assets_off_balance_sheet", 110)
		end

	libt: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("liability_external", "libt", "lbeq", 109, 133, 156, 1, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", k_liabilities, 109)
		end

	lgrs: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("legal_reserves", "lgrs", k_asst, 0, 97, 109, 1, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", "assets_off_balance_sheet", 108)
		end

	lesn: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("leases", "lesn", "nplt", 0, 0, 69, 1, k_currency, k_double, Void, k_company, ' ', k_valdata_signature, "1", k_assets, 107)
		end

	ldbt: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("longterm_debt", "ldbt", "libt", 105, 122, 145, 1, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", k_liabilities, 106)
		end

	lcon: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("loans_consumer", "lcon", "llon", 0, 65, 72, 1, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", k_assets, 105)
		end

	lcom: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("loans_commercial", "lcom", "llon", 0, 66, 73, 1, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", k_assets, 104)
		end

	lbeq: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("total_liability", "lbeq", "liabs", 120, 150, 175, 1, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", k_liabilities, 103)
		end

	land: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("land_net", "land", "nplt", 63, 0, 68, 1, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", k_assets, 102)
		end

	extp: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("extraordinary_items_hist", "extp", "nplt", 0, 94, 105, 2, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", "balance_sheet_synthesis", 101)
		end

	exin: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("expense_interest", "exin", "oper", 0, 158, 196, 1, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "1", k_profit_and_loss, 100)
		end

	euro: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("europe", "euro", "artfcl", 0, 0, 115, 2, k_currency, k_double, Void, k_company, ' ', k_valdata_signature, "1", "artificial", 99)
		end

	erxx: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("xcge_retained_earnings", "erxx", "p_and_l_adj", 0, 0, 0, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "1", "profit_and_loss_adjustments", 98)
		end

	er10: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("operating_earnings_segment_x", "er10", k_segmnt, 10, 10, 10, 2, "currency/(year*share)", k_double, Void, k_company, 'F', k_valdata_signature, "1", k_segment, 97)
		end

	er09: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("operating_earnings_segment_9", "er09", k_segmnt, 9, 9, 9, 2, "currency/(year*share)", k_double, Void, k_company, 'F', k_valdata_signature, "1", k_segment, 96)
		end

	er08: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("operating_earnings_segment_8", "er08", k_segmnt, 8, 8, 8, 2, "currency/(year*share)", k_double, Void, k_company, 'F', k_valdata_signature, "1", k_segment, 95)
		end

	er07: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("operating_earnings_segment_7", "er07", k_segmnt, 7, 7, 7, 2, "currency/(year*share)", k_double, Void, k_company, 'F', k_valdata_signature, "1", k_segment, 94)
		end

	er06: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("operating_earnings_segment_6", "er06", k_segmnt, 6, 6, 6, 2, "currency/(year*share)", k_double, Void, k_company, 'F', k_valdata_signature, "1", k_segment, 93)
		end

	er05: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("operating_earnings_segment_5", "er05", k_segmnt, 5, 5, 5, 2, "currency/(year*share)", k_double, Void, k_company, 'F', k_valdata_signature, "1", k_segment, 92)
		end

	er04: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("operating_earnings_segment_4", "er04", k_segmnt, 4, 4, 4, 2, "currency/(year*share)", k_double, Void, k_company, 'F', k_valdata_signature, "1", k_segment, 91)
		end

	er03: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("operating_earnings_segment_3", "er03", k_segmnt, 3, 3, 3, 2, "currency/(year*share)", k_double, Void, k_company, 'F', k_valdata_signature, "1", k_segment, 90)
		end

	er02: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("operating_earnings_segment_2", "er02", k_segmnt, 2, 2, 2, 2, "currency/(year*share)", k_double, Void, k_company, 'F', k_valdata_signature, "1", k_segment, 89)
		end

	er01: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("operating_earnings_segment_1", "er01", k_segmnt, 1, 1, 1, 2, "currency/(year*share)", k_double, Void, k_company, 'F', k_valdata_signature, "1", k_segment, 88)
		end

	eqts: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("stockholder_equity", "eqts", "lbeq", 0, 149, 174, 1, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", k_liabilities, 87)
		end

	eqro: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("equity_reserves_other", "eqro", "rvnd", 0, 146, 171, 1, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", k_liabilities, 86)
		end

	eqmn: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("equipment", "eqmn", "nplt", 61, 0, 66, 1, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", k_assets, 85)
		end

	empy: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("employees", "empy", "misc", 172, 115, 257, 1, Void, k_double, Void, k_company, 'X', k_valdata_signature, "1", k_miscellaneous, 84)
		end

	eluc: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("electric_utility_construction", "eluc", "nplt", 0, 206, 64, 2, k_currency, k_double, Void, k_company, ' ', k_valdata_signature, "1", k_assets, 83)
		end

	elfl: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("earnings_current_anlz", "elfl", "p_and_l_adj", 161, 0, 232, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "1", "profit_and_loss_adjustments", 82)
		end

	eh10: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("operating_history_segment_x", "eh10", k_segmnt, 40, 40, 40, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "1", k_segment, 81)
		end

	eh09: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("operating_history_segment_9", "eh09", k_segmnt, 39, 39, 39, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "1", k_segment, 80)
		end

	eh08: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("operating_history_segment_8", "eh08", k_segmnt, 38, 38, 38, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "1", k_segment, 79)
		end

	eh07: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("operating_history_segment_7", "eh07", k_segmnt, 37, 37, 37, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "1", k_segment, 78)
		end

	eh06: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("operating_history_segment_6", "eh06", k_segmnt, 36, 36, 36, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "1", k_segment, 77)
		end

	eh05: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("operating_history_segment_5", "eh05", k_segmnt, 35, 35, 35, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "1", k_segment, 76)
		end

	eh04: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("operating_history_segment_4", "eh04", k_segmnt, 34, 34, 34, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "1", k_segment, 75)
		end

	eh03: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("operating_history_segment_3", "eh03", k_segmnt, 33, 33, 33, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "1", k_segment, 74)
		end

	eh02: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("operating_history_segment_2", "eh02", k_segmnt, 32, 32, 32, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "1", k_segment, 73)
		end

	eh01: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("operating_history_segment_1", "eh01", k_segmnt, 31, 31, 31, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "1", k_segment, 72)
		end

	eeaj: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("enhanced_earnings_aj", "eeaj", "p_and_l_adj", 164, 191, 238, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "1", "profit_and_loss_adjustments", 71)
		end

	dtax: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("tax_deferred_income", "dtax", "itax", 149, 177, 215, 1, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "1", k_profit_and_loss, 70)
		end

	dsec: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("money_market", "dsec", "secu", 0, 54, 54, 1, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", k_assets, 69)
		end

	dqux: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("delta_book_unex", "dqux", "delta", 179, 213, 264, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "1", "one_year_delta", 68)
		end

	dqur: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("delta_unrealized_gain_2", "dqur", "delta", 178, 212, 263, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "1", "one_year_delta", 67)
		end

	dqrv: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("delta_rvlu_res", "dqrv", "delta", 177, 211, 262, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "1", "one_year_delta", 66)
		end

	dqre: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("delta_retained_earnings", "dqre", "delta", 176, 210, 261, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "1", "one_year_delta", 65)
		end

	dqis: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("delta_iss_sh_c", "dqis", "delta", 175, 209, 260, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "1", "one_year_delta", 64)
		end

	dqer: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("delta_eq_rs_o", "dqer", "delta", 174, 208, 259, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "1", "one_year_delta", 63)
		end

	dqct: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("delta_cur_trn", "dqct", "delta", 173, 207, 258, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "1", "one_year_delta", 62)
		end

	dopp: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("discontinued_operations_hist", "dopp", "nplt", 0, 95, 106, 2, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", "balance_sheet_synthesis", 61)
		end

	dlug: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("delta_unrealized_gain", "dlug", "delta", 0, 218, 0, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "1", "one_year_delta", 60)
		end

	dlrv: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("delta_revaluation_reserves", "dlrv", "delta", 0, 216, 0, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "1", "one_year_delta", 59)
		end

	dlro: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("delta_eq_rsv_other", "dlro", "delta", 0, 217, 0, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "1", "one_year_delta", 58)
		end

	dlrc: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("delta_receivables", "dlrc", "delta", 180, 214, 265, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "1", "one_year_delta", 57)
		end

	dlin: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("delta_inventory", "dlin", "delta", 181, 215, 266, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "1", "one_year_delta", 56)
		end

	dlct: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("delta_cm_trn_aj", "dlct", "delta", 0, 219, 0, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "1", "one_year_delta", 55)
		end

	diso: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("discontinued_operations", "diso", "exdo", 0, 184, 222, 1, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "1", k_profit_and_loss, 54)
		end

	dilp: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("book_dilution_p", "dilp", k_book, 0, 0, 189, 2, k_currency, k_double, Void, k_company, ' ', k_valdata_signature, "1", "liabilities_off_balance_sheet", 53)
		end

	dil1: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("book_dilution_1", "dil1", k_book, 0, 0, 187, 2, k_currency, k_double, Void, k_company, ' ', k_valdata_signature, "1", "liabilities_off_balance_sheet", 52)
		end

	depr: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("depreciation", "depr", "opad", 131, 164, 202, 1, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "1", k_profit_and_loss, 51)
		end

	depo: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("deposits_demand", "depo", "clib", 87, 99, 118, 1, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", k_liabilities, 50)
		end

	dcic: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("consolidated_current_income_delta", "dcic", "p_and_l_adj", 155, 0, 0, 2, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "1", "profit_and_loss_adjustments", 49)
		end

	dbtz: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("debt_capitalized_leases", "dbtz", "ldbt", 0, 0, 144, 1, k_currency, k_double, Void, k_company, ' ', k_valdata_signature, "1", k_liabilities, 48)
		end

	dbts: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("debt_subordinate", "dbts", "ldbt", 0, 120, 140, 1, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", k_liabilities, 47)
		end

	dbto: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("debt_other_longterm", "dbto", "ldbt", 104, 121, 143, 1, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", k_liabilities, 46)
		end

	dbtc: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("debt_convertible", "dbtc", "ldbt", 99, 0, 139, 1, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", k_liabilities, 45)
		end

	dbkc: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("consolidated_book_delta", "dbkc", k_asst, 121, 0, 0, 2, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", "liabilities_off_balance_sheet", 44)
		end

	dbcl: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("debt_in_current_liability", "dbcl", "sdbt", 92, 103, 122, 1, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", k_liabilities, 43)
		end

	curm: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("sundry_current_assets", "curm", "caco", 58, 61, 61, 1, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", k_assets, 42)
		end

	cumd: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("cumulative_depreciation", "cumd", "nplt", 84, 92, 103, 2, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", "balance_sheet_synthesis", 41)
		end

	ctax: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("taxable_accrued_income", "ctax", "itax", 148, 178, 216, 1, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "1", k_profit_and_loss, 40)
		end

	crtr: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("cumulative_trans_adj_book", "crtr", k_book, 0, 147, 172, 1, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", k_liabilities, 39)
		end

	cott: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("sales_cost", "cott", "gprf", 127, 159, 197, 1, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "1", k_profit_and_loss, 38)
		end

	cnip: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("construction", "cnip", "nplt", 64, 0, 63, 1, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", k_assets, 37)
		end

	clib: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("current_liability", "clib", "libt", 97, 110, 129, 1, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", k_liabilities, 36)
		end

	clan: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("sundry_current_liability", "clan", "clio", 96, 109, 128, 1, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", k_liabilities, 35)
		end

	chdf: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("deferred_charge", "chdf", "char", 73, 83, 90, 1, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", k_assets, 34)
		end

	cflo: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("ff_cash_flow", "cflo", k_ff, 158, 205, 252, 1, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "1", k_flow_of_funds, 33)
		end

	cass: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("current_assets", "cass", k_asst, 59, 62, 62, 1, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", k_assets, 32)
		end

	cash: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("cash", "cash", "cass", 41, 41, 41, 1, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", k_assets, 31)
		end

	caps: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("capital_stock", "caps", k_book, 0, 0, 0, 1, k_currency, k_double, Void, k_company, ' ', k_valdata_signature, "1", k_liabilities, 30)
		end

	cana: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("canadian_book", "cana", "artfcl", 0, 0, 113, 2, k_currency, k_double, Void, k_company, ' ', k_valdata_signature, "1", "artificial", 29)
		end

	bsec: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("corporate_bond", "bsec", "secu", 50, 53, 53, 1, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", k_assets, 28)
		end

	book: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("common_equity", k_book, "eqts", 119, 148, 173, 1, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", k_liabilities, 27)
		end

	bldn: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("buildings_net", "bldn", "dpas", 60, 0, 65, 1, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", k_assets, 26)
		end

	bkxx: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("exchange_rate_book", "bkxx", k_book, 0, 155, 192, 1, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", k_miscellaneous, 25)
		end

	bker: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("book_earnings_xtrp", "bker", k_book, 123, 151, 176, 2, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", "extrapolation", 24)
		end

	bkdv: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("book_dividend_xtrp", "bkdv", k_book, 124, 152, 177, 2, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", "extrapolation", 23)
		end

	bdep: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("deposits_banks", "bdep", "clib", 0, 101, 120, 1, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", k_liabilities, 22)
		end

	bbyo: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("book_held_by_other", "bbyo", k_asst, 79, 0, 0, 2, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", "assets_off_balance_sheet", 21)
		end

	badj: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("book_adjusted", "badj", k_book, 125, 153, 178, 2, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", "extrapolation", 20)
		end

	audx: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("audit_warning", "audx", k_book, 0, 156, 193, 1, Void, k_double, Void, k_company, 'S', k_valdata_signature, "1", k_miscellaneous, 19)
		end

	aszt: VARIABLE_DESCRIPTION_RECORD is
			-- Zaitech assets
		once
			create Result.make_valdata ("zaitech_assets", "aszt", k_asst, 76, 90, 98, 2, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", "intermediation", 18)
		end

	asus: VARIABLE_DESCRIPTION_RECORD is
			-- Assets used
		once
			create Result.make_valdata ("assets_used", "asus", k_asst, 75, 89, 97, 2, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", "intermediation", 17)
		end

	asto: VARIABLE_DESCRIPTION_RECORD is
			-- Other assets
		once
			create Result.make_valdata ("other_assets", "asto", "alfx", 0, 0, 95, 1, k_currency, k_double, Void, k_company, ' ', k_valdata_signature, "1", k_assets, 16)
		end

	asst: VARIABLE_DESCRIPTION_RECORD is
			--Total assets
		once
			create Result.make_valdata ("total_assets", k_asst, k_assets, 74, 88, 96, 1, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", k_assets, 15)
		end

	assf: VARIABLE_DESCRIPTION_RECORD is
		once
			create Result.make_valdata ("foreign_assets", "assf", k_asst, 0, 0, 117, 1, k_currency, k_double, Void, k_company, ' ', k_valdata_signature, "1", "assets_off_balance_sheet", 14)
		end

	as10: VARIABLE_DESCRIPTION_RECORD is
			-- Valdata assets segment 10
		once
			create Result.make_valdata ("assets_segment_x", "as10", k_segmnt, 20, 20, 20, 2, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", k_segment, 13)
		ensure
			not_void: Result /= Void
		end

	as09: VARIABLE_DESCRIPTION_RECORD is
			-- Valdata assets segment 9
		once
			create Result.make_valdata ("assets_segment_9", "as09", k_segmnt, 19, 19, 19, 2, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", k_segment, 12)
		ensure
			not_void: Result /= Void
		end

	as08: VARIABLE_DESCRIPTION_RECORD is
			-- Valdata assets segment 8
		once
			create Result.make_valdata ("assets_segment_8", "as08", k_segmnt, 18, 18, 18, 2, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", k_segment, 11)
		ensure
			not_void: Result /= Void
		end

	as07: VARIABLE_DESCRIPTION_RECORD is
			-- Valdata assets segment 7
		once
			create Result.make_valdata ("assets_segment_7", "as07", k_segmnt, 17, 17, 17, 2, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", k_segment, 10)
		ensure
			not_void: Result /= Void
		end

	as06: VARIABLE_DESCRIPTION_RECORD is
			-- Valdata assets segment 6
		once
			create Result.make_valdata ("assets_segment_6", "as06", k_segmnt, 16, 16, 16, 2, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", k_segment, 9)
		ensure
			not_void: Result /= Void
		end

	as05: VARIABLE_DESCRIPTION_RECORD is
			-- Valdata assets segment 5
		once
			create Result.make_valdata ("assets_segment_5", "as05", k_segmnt, 15, 15, 15, 2, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", k_segment, 8)
		ensure
			not_void: Result /= Void
		end

	as04: VARIABLE_DESCRIPTION_RECORD is
			-- Valdata assets segment 4
		once
			create Result.make_valdata ("assets_segment_4", "as04", k_segmnt, 14, 14, 14, 2, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", k_segment, 7)
		ensure
			not_void: Result /= Void
		end

	as03: VARIABLE_DESCRIPTION_RECORD is
			-- Valdata assets segment 3
		once
			create Result.make_valdata ("assets_segment_3", "as03", k_segmnt, 13, 13, 13, 2, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", k_segment, 6)
		ensure
			not_void: Result /= Void
		end

	as02: VARIABLE_DESCRIPTION_RECORD is
			-- Valdata assets segment 2
		once
			create Result.make_valdata ("assets_segment_2", "as02", k_segmnt, 12, 12, 12, 2, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", k_segment, 5)
		ensure
			not_void: Result /= Void
		end

	as01: VARIABLE_DESCRIPTION_RECORD is
			-- Valdata assets segment 1
		once
			create Result.make_valdata ("assets_segment_1", "as01", k_segmnt, 11, 11, 11, 2, k_currency, k_double, Void, k_company, 'S', k_valdata_signature, "1", k_segment, 4)
		ensure
			not_void: Result /= Void
		end

	apnr: VARIABLE_DESCRIPTION_RECORD is
			-- Appropriated non equity reserves
		once
			create Result.make_valdata ("appropriated_non_equity_reserves", "apnr", "ptin", 146, 175, 213, 1, k_currency, k_double, Void, k_company, 'F', k_valdata_signature, "1", k_profit_and_loss, 3)
		ensure
			not_void: Result /= Void
		end

	afut: VARIABLE_DESCRIPTION_RECORD is
			-- Accumulated adjustment future
		once
			create Result.make_valdata ("acc_adjustment_future", "afut", k_book, 0, 0, 191, 1, k_currency, k_double, Void, k_company, ' ', k_valdata_signature, "1", k_miscellaneous, 2)
		ensure
			not_void: Result /= Void
		end

	adr: VARIABLE_DESCRIPTION_RECORD is
			--adr book value
		once
			create Result.make_valdata ("adr_book", "adr", "artfcl", 0, 0, 111, 2, k_currency, k_double, Void, k_company, ' ', k_valdata_signature, "1", "artificial", 1)
		ensure
			not_void: Result /= Void
		end

end
